!***********************************************************************
!   System:                     O4
!   Functional form:            Permutational Invariant Global Least Square
!   Common name:                O4 singlet
!   Number of derivatives:      0
!   Number of bodies:           4
!   Number of electronic surfaces: 1
!   Interface: Section-2
!
!   References:  doi 10.1063/1.5008973
!	 Tapan K. Mankodi, Upendra V. Bhandarkar,
!        Bhalchandra P. Puranik
!
!   Notes:    
!
!   Data file: coeff_o4.txt
!
!***********************************************************************

!***********************************************************************!
! Global Potential energy surface for 									!
! singlet O4 system using CASSCF+CASPT2  								!	
! Permutational Invariant Global Least Square fit						!
! Input Variable: Interatomic distance R1-R2-R3-R4-R5-R6 in Angstrom	!
! Output Variable: Energy in kcal/mol									!	
! Author of the Code: Tapan K. Mankodi									!
! 					  Indian Institute of Technology Bombay				!
! 					  Mumbai, India										!
! Ref: http://aip.scitation.org/doi/abs/10.1063/1.5008973			!
! Author:  Tapan K. Mankodi, Upendra V. Bhandarkar, 					!
!    	   Bhalchandra P. Puranik										! 
! Corresponding Address: tapan.mankodi@iitb.ac.in						!
!						 bhandarkar@iitb.ac.in							!					
!***********************************************************************!

subroutine pes(x,igrad,p,g,d)

      implicit none
      ! number of electronic state
      integer, parameter :: nstates=1
      integer, parameter :: natoms=4
      integer, intent(in) :: igrad
      double precision, intent(in) :: x(natoms,3)
      double precision, intent(out) :: p(nstates), g(nstates,natoms,3)
      double precision, intent(out) :: d(nstates,nstates,natoms,3)

      double precision :: v, tx(6), sum_d
      integer :: ibond, iatom, idir, j, istate
      !initialize 
      v=0.d0
      g=0.d0
      d=0.d0

      if (igrad==0) then
        tx(1)=sqrt((x(1,1)-x(2,1))**2+(x(1,2)-x(2,2))**2+(x(1,3)-x(2,3))**2)
        tx(2)=sqrt((x(1,1)-x(3,1))**2+(x(1,2)-x(3,2))**2+(x(1,3)-x(3,3))**2)
        tx(3)=sqrt((x(1,1)-x(4,1))**2+(x(1,2)-x(4,2))**2+(x(1,3)-x(4,3))**2)
        tx(4)=sqrt((x(2,1)-x(3,1))**2+(x(2,2)-x(3,2))**2+(x(2,3)-x(3,3))**2)
        tx(5)=sqrt((x(2,1)-x(4,1))**2+(x(2,2)-x(4,2))**2+(x(2,3)-x(4,3))**2)
        tx(6)=sqrt((x(3,1)-x(4,1))**2+(x(3,2)-x(4,2))**2+(x(3,3)-x(4,3))**2)
        call o4(v,tx)
        v=v/23.0609
        do istate=1,nstates
          p(istate)=v
        enddo
      else
        write (*,*) 'Only energy and gradient are available'
      endif

endsubroutine

subroutine o4(v,r)

	implicit none
        double precision, intent(in) :: r(6)
        double precision, intent(out) :: v


        double precision :: xr(6)
	integer							::	mpoints
	double precision				::	coeff(306)
	double precision				::	indices(306,6)
	integer							::	permutation(24,6)
	double precision				::	ae,re
	integer							::	i,j,k
        integer                                 ::              ci,cj,ck,cl
        double precision                ::              mult

	re = 1.208d0
	ae = 0.7d0
	mpoints = 306

        do ci=1,6
          xr(ci) = exp((1.208d0-r(ci))/0.7d0)
        enddo

        call assign(coeff, indices, permutation)

        v=0.0
        do ci=1,306
          do cj = 1,24
            mult=1.0
            do ck=1,6
              mult = mult*(xr(ck)**indices(ci,permutation(cj,ck)+1))
            enddo
            v=v+coeff(ci)*mult
          enddo
        enddo

endsubroutine

subroutine assign(coeff, indices, permutation) 
  double precision,intent(inout) :: coeff(306), indices(306,6)
  integer, intent(inout) :: permutation(24,6)

permutation(1,1)=0
permutation(1,2)=1
permutation(1,3)=2
permutation(1,4)=3
permutation(1,5)=4
permutation(1,6)=5
permutation(2,1)=0
permutation(2,2)=2
permutation(2,3)=1
permutation(2,4)=4
permutation(2,5)=3
permutation(2,6)=5
permutation(3,1)=1
permutation(3,2)=0
permutation(3,3)=2
permutation(3,4)=3
permutation(3,5)=5
permutation(3,6)=4
permutation(4,1)=1
permutation(4,2)=2
permutation(4,3)=0
permutation(4,4)=5
permutation(4,5)=3
permutation(4,6)=4
permutation(5,1)=2
permutation(5,2)=0
permutation(5,3)=1
permutation(5,4)=4
permutation(5,5)=5
permutation(5,6)=3
permutation(6,1)=2
permutation(6,2)=1
permutation(6,3)=0
permutation(6,4)=5
permutation(6,5)=4
permutation(6,6)=3
permutation(7,1)=0
permutation(7,2)=3
permutation(7,3)=4
permutation(7,4)=1
permutation(7,5)=2
permutation(7,6)=5
permutation(8,1)=0
permutation(8,2)=4
permutation(8,3)=3
permutation(8,4)=2
permutation(8,5)=1
permutation(8,6)=5
permutation(9,1)=3
permutation(9,2)=0
permutation(9,3)=4
permutation(9,4)=1
permutation(9,5)=5
permutation(9,6)=2
permutation(10,1)=3
permutation(10,2)=4
permutation(10,3)=0
permutation(10,4)=5
permutation(10,5)=1
permutation(10,6)=2
permutation(11,1)=4
permutation(11,2)=0
permutation(11,3)=3
permutation(11,4)=2
permutation(11,5)=5
permutation(11,6)=1
permutation(12,1)=4
permutation(12,2)=3
permutation(12,3)=0
permutation(12,4)=5
permutation(12,5)=2
permutation(12,6)=1
permutation(13,1)=1
permutation(13,2)=3
permutation(13,3)=5
permutation(13,4)=0
permutation(13,5)=2
permutation(13,6)=4
permutation(14,1)=1
permutation(14,2)=5
permutation(14,3)=3
permutation(14,4)=2
permutation(14,5)=0
permutation(14,6)=4
permutation(15,1)=3
permutation(15,2)=1
permutation(15,3)=5
permutation(15,4)=0
permutation(15,5)=4
permutation(15,6)=2
permutation(16,1)=3
permutation(16,2)=5
permutation(16,3)=1
permutation(16,4)=4
permutation(16,5)=0
permutation(16,6)=2
permutation(17,1)=5
permutation(17,2)=1
permutation(17,3)=3
permutation(17,4)=2
permutation(17,5)=4
permutation(17,6)=0
permutation(18,1)=5
permutation(18,2)=3
permutation(18,3)=1
permutation(18,4)=4
permutation(18,5)=2
permutation(18,6)=0
permutation(19,1)=2
permutation(19,2)=4
permutation(19,3)=5
permutation(19,4)=0
permutation(19,5)=1
permutation(19,6)=3
permutation(20,1)=2
permutation(20,2)=5
permutation(20,3)=4
permutation(20,4)=1
permutation(20,5)=0
permutation(20,6)=3
permutation(21,1)=4
permutation(21,2)=2
permutation(21,3)=5
permutation(21,4)=0
permutation(21,5)=3
permutation(21,6)=1
permutation(22,1)=4
permutation(22,2)=5
permutation(22,3)=2
permutation(22,4)=3
permutation(22,5)=0
permutation(22,6)=1
permutation(23,1)=5
permutation(23,2)=2
permutation(23,3)=4
permutation(23,4)=1
permutation(23,5)=3
permutation(23,6)=0
permutation(24,1)=5
permutation(24,2)=4
permutation(24,3)=2
permutation(24,4)=3
permutation(24,5)=1
permutation(24,6)=0

indices(1,1)=0.0
indices(1,2)=0.0
indices(1,3)=0.0
indices(1,4)=0.0
indices(1,5)=0.0
indices(1,6)=0.0
indices(2,1)=1.0
indices(2,2)=0.0
indices(2,3)=0.0
indices(2,4)=0.0
indices(2,5)=0.0
indices(2,6)=0.0
indices(3,1)=1.0
indices(3,2)=0.0
indices(3,3)=0.0
indices(3,4)=0.0
indices(3,5)=0.0
indices(3,6)=1.0
indices(4,1)=1.0
indices(4,2)=1.0
indices(4,3)=0.0
indices(4,4)=0.0
indices(4,5)=0.0
indices(4,6)=0.0
indices(5,1)=1.0
indices(5,2)=1.0
indices(5,3)=0.0
indices(5,4)=0.0
indices(5,5)=1.0
indices(5,6)=0.0
indices(6,1)=1.0
indices(6,2)=1.0
indices(6,3)=0.0
indices(6,4)=0.0
indices(6,5)=1.0
indices(6,6)=1.0
indices(7,1)=1.0
indices(7,2)=1.0
indices(7,3)=0.0
indices(7,4)=1.0
indices(7,5)=0.0
indices(7,6)=0.0
indices(8,1)=1.0
indices(8,2)=1.0
indices(8,3)=1.0
indices(8,4)=0.0
indices(8,5)=0.0
indices(8,6)=0.0
indices(9,1)=1.0
indices(9,2)=1.0
indices(9,3)=1.0
indices(9,4)=1.0
indices(9,5)=0.0
indices(9,6)=0.0
indices(10,1)=1.0
indices(10,2)=1.0
indices(10,3)=1.0
indices(10,4)=1.0
indices(10,5)=1.0
indices(10,6)=0.0
indices(11,1)=1.0
indices(11,2)=1.0
indices(11,3)=1.0
indices(11,4)=1.0
indices(11,5)=1.0
indices(11,6)=1.0
indices(12,1)=2.0
indices(12,2)=0.0
indices(12,3)=0.0
indices(12,4)=0.0
indices(12,5)=0.0
indices(12,6)=0.0
indices(13,1)=2.0
indices(13,2)=0.0
indices(13,3)=0.0
indices(13,4)=0.0
indices(13,5)=0.0
indices(13,6)=1.0
indices(14,1)=2.0
indices(14,2)=0.0
indices(14,3)=0.0
indices(14,4)=0.0
indices(14,5)=0.0
indices(14,6)=2.0
indices(15,1)=2.0
indices(15,2)=1.0
indices(15,3)=0.0
indices(15,4)=0.0
indices(15,5)=0.0
indices(15,6)=0.0
indices(16,1)=2.0
indices(16,2)=1.0
indices(16,3)=0.0
indices(16,4)=0.0
indices(16,5)=0.0
indices(16,6)=1.0
indices(17,1)=2.0
indices(17,2)=1.0
indices(17,3)=0.0
indices(17,4)=0.0
indices(17,5)=0.0
indices(17,6)=2.0
indices(18,1)=2.0
indices(18,2)=1.0
indices(18,3)=0.0
indices(18,4)=0.0
indices(18,5)=1.0
indices(18,6)=0.0
indices(19,1)=2.0
indices(19,2)=1.0
indices(19,3)=0.0
indices(19,4)=0.0
indices(19,5)=1.0
indices(19,6)=1.0
indices(20,1)=2.0
indices(20,2)=1.0
indices(20,3)=0.0
indices(20,4)=0.0
indices(20,5)=1.0
indices(20,6)=2.0
indices(21,1)=2.0
indices(21,2)=1.0
indices(21,3)=0.0
indices(21,4)=1.0
indices(21,5)=0.0
indices(21,6)=0.0
indices(22,1)=2.0
indices(22,2)=1.0
indices(22,3)=0.0
indices(22,4)=1.0
indices(22,5)=0.0
indices(22,6)=1.0
indices(23,1)=2.0
indices(23,2)=1.0
indices(23,3)=1.0
indices(23,4)=0.0
indices(23,5)=0.0
indices(23,6)=0.0
indices(24,1)=2.0
indices(24,2)=1.0
indices(24,3)=1.0
indices(24,4)=0.0
indices(24,5)=0.0
indices(24,6)=1.0
indices(25,1)=2.0
indices(25,2)=1.0
indices(25,3)=1.0
indices(25,4)=0.0
indices(25,5)=0.0
indices(25,6)=2.0
indices(26,1)=2.0
indices(26,2)=1.0
indices(26,3)=1.0
indices(26,4)=1.0
indices(26,5)=0.0
indices(26,6)=0.0
indices(27,1)=2.0
indices(27,2)=1.0
indices(27,3)=1.0
indices(27,4)=1.0
indices(27,5)=0.0
indices(27,6)=1.0
indices(28,1)=2.0
indices(28,2)=1.0
indices(28,3)=1.0
indices(28,4)=1.0
indices(28,5)=0.0
indices(28,6)=2.0
indices(29,1)=2.0
indices(29,2)=1.0
indices(29,3)=1.0
indices(29,4)=1.0
indices(29,5)=1.0
indices(29,6)=0.0
indices(30,1)=2.0
indices(30,2)=1.0
indices(30,3)=1.0
indices(30,4)=1.0
indices(30,5)=1.0
indices(30,6)=1.0
indices(31,1)=2.0
indices(31,2)=1.0
indices(31,3)=1.0
indices(31,4)=1.0
indices(31,5)=1.0
indices(31,6)=2.0
indices(32,1)=2.0
indices(32,2)=2.0
indices(32,3)=0.0
indices(32,4)=0.0
indices(32,5)=0.0
indices(32,6)=0.0
indices(33,1)=2.0
indices(33,2)=2.0
indices(33,3)=0.0
indices(33,4)=0.0
indices(33,5)=1.0
indices(33,6)=0.0
indices(34,1)=2.0
indices(34,2)=2.0
indices(34,3)=0.0
indices(34,4)=0.0
indices(34,5)=1.0
indices(34,6)=1.0
indices(35,1)=2.0
indices(35,2)=2.0
indices(35,3)=0.0
indices(35,4)=0.0
indices(35,5)=2.0
indices(35,6)=0.0
indices(36,1)=2.0
indices(36,2)=2.0
indices(36,3)=0.0
indices(36,4)=0.0
indices(36,5)=2.0
indices(36,6)=1.0
indices(37,1)=2.0
indices(37,2)=2.0
indices(37,3)=0.0
indices(37,4)=0.0
indices(37,5)=2.0
indices(37,6)=2.0
indices(38,1)=2.0
indices(38,2)=2.0
indices(38,3)=0.0
indices(38,4)=1.0
indices(38,5)=0.0
indices(38,6)=0.0
indices(39,1)=2.0
indices(39,2)=2.0
indices(39,3)=0.0
indices(39,4)=1.0
indices(39,5)=1.0
indices(39,6)=0.0
indices(40,1)=2.0
indices(40,2)=2.0
indices(40,3)=0.0
indices(40,4)=1.0
indices(40,5)=1.0
indices(40,6)=1.0
indices(41,1)=2.0
indices(41,2)=2.0
indices(41,3)=0.0
indices(41,4)=2.0
indices(41,5)=0.0
indices(41,6)=0.0
indices(42,1)=2.0
indices(42,2)=2.0
indices(42,3)=1.0
indices(42,4)=0.0
indices(42,5)=0.0
indices(42,6)=0.0
indices(43,1)=2.0
indices(43,2)=2.0
indices(43,3)=1.0
indices(43,4)=0.0
indices(43,5)=1.0
indices(43,6)=0.0
indices(44,1)=2.0
indices(44,2)=2.0
indices(44,3)=1.0
indices(44,4)=0.0
indices(44,5)=1.0
indices(44,6)=1.0
indices(45,1)=2.0
indices(45,2)=2.0
indices(45,3)=1.0
indices(45,4)=0.0
indices(45,5)=2.0
indices(45,6)=0.0
indices(46,1)=2.0
indices(46,2)=2.0
indices(46,3)=1.0
indices(46,4)=0.0
indices(46,5)=2.0
indices(46,6)=1.0
indices(47,1)=2.0
indices(47,2)=2.0
indices(47,3)=1.0
indices(47,4)=0.0
indices(47,5)=2.0
indices(47,6)=2.0
indices(48,1)=2.0
indices(48,2)=2.0
indices(48,3)=1.0
indices(48,4)=1.0
indices(48,5)=0.0
indices(48,6)=0.0
indices(49,1)=2.0
indices(49,2)=2.0
indices(49,3)=1.0
indices(49,4)=1.0
indices(49,5)=1.0
indices(49,6)=0.0
indices(50,1)=2.0
indices(50,2)=2.0
indices(50,3)=1.0
indices(50,4)=1.0
indices(50,5)=1.0
indices(50,6)=1.0
indices(51,1)=2.0
indices(51,2)=2.0
indices(51,3)=1.0
indices(51,4)=1.0
indices(51,5)=2.0
indices(51,6)=0.0
indices(52,1)=2.0
indices(52,2)=2.0
indices(52,3)=1.0
indices(52,4)=1.0
indices(52,5)=2.0
indices(52,6)=1.0
indices(53,1)=2.0
indices(53,2)=2.0
indices(53,3)=1.0
indices(53,4)=2.0
indices(53,5)=0.0
indices(53,6)=0.0
indices(54,1)=2.0
indices(54,2)=2.0
indices(54,3)=1.0
indices(54,4)=2.0
indices(54,5)=1.0
indices(54,6)=0.0
indices(55,1)=2.0
indices(55,2)=2.0
indices(55,3)=1.0
indices(55,4)=2.0
indices(55,5)=1.0
indices(55,6)=1.0
indices(56,1)=2.0
indices(56,2)=2.0
indices(56,3)=2.0
indices(56,4)=0.0
indices(56,5)=0.0
indices(56,6)=0.0
indices(57,1)=2.0
indices(57,2)=2.0
indices(57,3)=2.0
indices(57,4)=1.0
indices(57,5)=0.0
indices(57,6)=0.0
indices(58,1)=2.0
indices(58,2)=2.0
indices(58,3)=2.0
indices(58,4)=1.0
indices(58,5)=1.0
indices(58,6)=0.0
indices(59,1)=2.0
indices(59,2)=2.0
indices(59,3)=2.0
indices(59,4)=1.0
indices(59,5)=1.0
indices(59,6)=1.0
indices(60,1)=2.0
indices(60,2)=2.0
indices(60,3)=2.0
indices(60,4)=2.0
indices(60,5)=0.0
indices(60,6)=0.0
indices(61,1)=2.0
indices(61,2)=2.0
indices(61,3)=2.0
indices(61,4)=2.0
indices(61,5)=1.0
indices(61,6)=0.0
indices(62,1)=3.0
indices(62,2)=0.0
indices(62,3)=0.0
indices(62,4)=0.0
indices(62,5)=0.0
indices(62,6)=0.0
indices(63,1)=3.0
indices(63,2)=0.0
indices(63,3)=0.0
indices(63,4)=0.0
indices(63,5)=0.0
indices(63,6)=1.0
indices(64,1)=3.0
indices(64,2)=0.0
indices(64,3)=0.0
indices(64,4)=0.0
indices(64,5)=0.0
indices(64,6)=2.0
indices(65,1)=3.0
indices(65,2)=0.0
indices(65,3)=0.0
indices(65,4)=0.0
indices(65,5)=0.0
indices(65,6)=3.0
indices(66,1)=3.0
indices(66,2)=1.0
indices(66,3)=0.0
indices(66,4)=0.0
indices(66,5)=0.0
indices(66,6)=0.0
indices(67,1)=3.0
indices(67,2)=1.0
indices(67,3)=0.0
indices(67,4)=0.0
indices(67,5)=0.0
indices(67,6)=1.0
indices(68,1)=3.0
indices(68,2)=1.0
indices(68,3)=0.0
indices(68,4)=0.0
indices(68,5)=0.0
indices(68,6)=2.0
indices(69,1)=3.0
indices(69,2)=1.0
indices(69,3)=0.0
indices(69,4)=0.0
indices(69,5)=0.0
indices(69,6)=3.0
indices(70,1)=3.0
indices(70,2)=1.0
indices(70,3)=0.0
indices(70,4)=0.0
indices(70,5)=1.0
indices(70,6)=0.0
indices(71,1)=3.0
indices(71,2)=1.0
indices(71,3)=0.0
indices(71,4)=0.0
indices(71,5)=1.0
indices(71,6)=1.0
indices(72,1)=3.0
indices(72,2)=1.0
indices(72,3)=0.0
indices(72,4)=0.0
indices(72,5)=1.0
indices(72,6)=2.0
indices(73,1)=3.0
indices(73,2)=1.0
indices(73,3)=0.0
indices(73,4)=0.0
indices(73,5)=1.0
indices(73,6)=3.0
indices(74,1)=3.0
indices(74,2)=1.0
indices(74,3)=0.0
indices(74,4)=1.0
indices(74,5)=0.0
indices(74,6)=0.0
indices(75,1)=3.0
indices(75,2)=1.0
indices(75,3)=0.0
indices(75,4)=1.0
indices(75,5)=0.0
indices(75,6)=1.0
indices(76,1)=3.0
indices(76,2)=1.0
indices(76,3)=0.0
indices(76,4)=1.0
indices(76,5)=0.0
indices(76,6)=2.0
indices(77,1)=3.0
indices(77,2)=1.0
indices(77,3)=1.0
indices(77,4)=0.0
indices(77,5)=0.0
indices(77,6)=0.0
indices(78,1)=3.0
indices(78,2)=1.0
indices(78,3)=1.0
indices(78,4)=0.0
indices(78,5)=0.0
indices(78,6)=1.0
indices(79,1)=3.0
indices(79,2)=1.0
indices(79,3)=1.0
indices(79,4)=0.0
indices(79,5)=0.0
indices(79,6)=2.0
indices(80,1)=3.0
indices(80,2)=1.0
indices(80,3)=1.0
indices(80,4)=0.0
indices(80,5)=0.0
indices(80,6)=3.0
indices(81,1)=3.0
indices(81,2)=1.0
indices(81,3)=1.0
indices(81,4)=1.0
indices(81,5)=0.0
indices(81,6)=0.0
indices(82,1)=3.0
indices(82,2)=1.0
indices(82,3)=1.0
indices(82,4)=1.0
indices(82,5)=0.0
indices(82,6)=1.0
indices(83,1)=3.0
indices(83,2)=1.0
indices(83,3)=1.0
indices(83,4)=1.0
indices(83,5)=0.0
indices(83,6)=2.0
indices(84,1)=3.0
indices(84,2)=1.0
indices(84,3)=1.0
indices(84,4)=1.0
indices(84,5)=0.0
indices(84,6)=3.0
indices(85,1)=3.0
indices(85,2)=1.0
indices(85,3)=1.0
indices(85,4)=1.0
indices(85,5)=1.0
indices(85,6)=0.0
indices(86,1)=3.0
indices(86,2)=1.0
indices(86,3)=1.0
indices(86,4)=1.0
indices(86,5)=1.0
indices(86,6)=1.0
indices(87,1)=3.0
indices(87,2)=1.0
indices(87,3)=1.0
indices(87,4)=1.0
indices(87,5)=1.0
indices(87,6)=2.0
indices(88,1)=3.0
indices(88,2)=2.0
indices(88,3)=0.0
indices(88,4)=0.0
indices(88,5)=0.0
indices(88,6)=0.0
indices(89,1)=3.0
indices(89,2)=2.0
indices(89,3)=0.0
indices(89,4)=0.0
indices(89,5)=0.0
indices(89,6)=1.0
indices(90,1)=3.0
indices(90,2)=2.0
indices(90,3)=0.0
indices(90,4)=0.0
indices(90,5)=0.0
indices(90,6)=2.0
indices(91,1)=3.0
indices(91,2)=2.0
indices(91,3)=0.0
indices(91,4)=0.0
indices(91,5)=0.0
indices(91,6)=3.0
indices(92,1)=3.0
indices(92,2)=2.0
indices(92,3)=0.0
indices(92,4)=0.0
indices(92,5)=1.0
indices(92,6)=0.0
indices(93,1)=3.0
indices(93,2)=2.0
indices(93,3)=0.0
indices(93,4)=0.0
indices(93,5)=1.0
indices(93,6)=1.0
indices(94,1)=3.0
indices(94,2)=2.0
indices(94,3)=0.0
indices(94,4)=0.0
indices(94,5)=1.0
indices(94,6)=2.0
indices(95,1)=3.0
indices(95,2)=2.0
indices(95,3)=0.0
indices(95,4)=0.0
indices(95,5)=1.0
indices(95,6)=3.0
indices(96,1)=3.0
indices(96,2)=2.0
indices(96,3)=0.0
indices(96,4)=0.0
indices(96,5)=2.0
indices(96,6)=0.0
indices(97,1)=3.0
indices(97,2)=2.0
indices(97,3)=0.0
indices(97,4)=0.0
indices(97,5)=2.0
indices(97,6)=1.0
indices(98,1)=3.0
indices(98,2)=2.0
indices(98,3)=0.0
indices(98,4)=0.0
indices(98,5)=2.0
indices(98,6)=2.0
indices(99,1)=3.0
indices(99,2)=2.0
indices(99,3)=0.0
indices(99,4)=1.0
indices(99,5)=0.0
indices(99,6)=0.0
indices(100,1)=3.0
indices(100,2)=2.0
indices(100,3)=0.0
indices(100,4)=1.0
indices(100,5)=0.0
indices(100,6)=1.0
indices(101,1)=3.0
indices(101,2)=2.0
indices(101,3)=0.0
indices(101,4)=1.0
indices(101,5)=0.0
indices(101,6)=2.0
indices(102,1)=3.0
indices(102,2)=2.0
indices(102,3)=0.0
indices(102,4)=1.0
indices(102,5)=1.0
indices(102,6)=0.0
indices(103,1)=3.0
indices(103,2)=2.0
indices(103,3)=0.0
indices(103,4)=1.0
indices(103,5)=1.0
indices(103,6)=1.0
indices(104,1)=3.0
indices(104,2)=2.0
indices(104,3)=0.0
indices(104,4)=1.0
indices(104,5)=1.0
indices(104,6)=2.0
indices(105,1)=3.0
indices(105,2)=2.0
indices(105,3)=0.0
indices(105,4)=2.0
indices(105,5)=0.0
indices(105,6)=0.0
indices(106,1)=3.0
indices(106,2)=2.0
indices(106,3)=0.0
indices(106,4)=2.0
indices(106,5)=0.0
indices(106,6)=1.0
indices(107,1)=3.0
indices(107,2)=2.0
indices(107,3)=0.0
indices(107,4)=2.0
indices(107,5)=0.0
indices(107,6)=2.0
indices(108,1)=3.0
indices(108,2)=2.0
indices(108,3)=1.0
indices(108,4)=0.0
indices(108,5)=0.0
indices(108,6)=0.0
indices(109,1)=3.0
indices(109,2)=2.0
indices(109,3)=1.0
indices(109,4)=0.0
indices(109,5)=0.0
indices(109,6)=1.0
indices(110,1)=3.0
indices(110,2)=2.0
indices(110,3)=1.0
indices(110,4)=0.0
indices(110,5)=0.0
indices(110,6)=2.0
indices(111,1)=3.0
indices(111,2)=2.0
indices(111,3)=1.0
indices(111,4)=0.0
indices(111,5)=0.0
indices(111,6)=3.0
indices(112,1)=3.0
indices(112,2)=2.0
indices(112,3)=1.0
indices(112,4)=0.0
indices(112,5)=1.0
indices(112,6)=0.0
indices(113,1)=3.0
indices(113,2)=2.0
indices(113,3)=1.0
indices(113,4)=0.0
indices(113,5)=1.0
indices(113,6)=1.0
indices(114,1)=3.0
indices(114,2)=2.0
indices(114,3)=1.0
indices(114,4)=0.0
indices(114,5)=1.0
indices(114,6)=2.0
indices(115,1)=3.0
indices(115,2)=2.0
indices(115,3)=1.0
indices(115,4)=0.0
indices(115,5)=2.0
indices(115,6)=0.0
indices(116,1)=3.0
indices(116,2)=2.0
indices(116,3)=1.0
indices(116,4)=0.0
indices(116,5)=2.0
indices(116,6)=1.0
indices(117,1)=3.0
indices(117,2)=2.0
indices(117,3)=1.0
indices(117,4)=1.0
indices(117,5)=0.0
indices(117,6)=0.0
indices(118,1)=3.0
indices(118,2)=2.0
indices(118,3)=1.0
indices(118,4)=1.0
indices(118,5)=0.0
indices(118,6)=1.0
indices(119,1)=3.0
indices(119,2)=2.0
indices(119,3)=1.0
indices(119,4)=1.0
indices(119,5)=0.0
indices(119,6)=2.0
indices(120,1)=3.0
indices(120,2)=2.0
indices(120,3)=1.0
indices(120,4)=1.0
indices(120,5)=1.0
indices(120,6)=0.0
indices(121,1)=3.0
indices(121,2)=2.0
indices(121,3)=1.0
indices(121,4)=1.0
indices(121,5)=1.0
indices(121,6)=1.0
indices(122,1)=3.0
indices(122,2)=2.0
indices(122,3)=1.0
indices(122,4)=1.0
indices(122,5)=2.0
indices(122,6)=0.0
indices(123,1)=3.0
indices(123,2)=2.0
indices(123,3)=1.0
indices(123,4)=2.0
indices(123,5)=0.0
indices(123,6)=0.0
indices(124,1)=3.0
indices(124,2)=2.0
indices(124,3)=1.0
indices(124,4)=2.0
indices(124,5)=0.0
indices(124,6)=1.0
indices(125,1)=3.0
indices(125,2)=2.0
indices(125,3)=1.0
indices(125,4)=2.0
indices(125,5)=1.0
indices(125,6)=0.0
indices(126,1)=3.0
indices(126,2)=2.0
indices(126,3)=2.0
indices(126,4)=0.0
indices(126,5)=0.0
indices(126,6)=0.0
indices(127,1)=3.0
indices(127,2)=2.0
indices(127,3)=2.0
indices(127,4)=0.0
indices(127,5)=0.0
indices(127,6)=1.0
indices(128,1)=3.0
indices(128,2)=2.0
indices(128,3)=2.0
indices(128,4)=0.0
indices(128,5)=0.0
indices(128,6)=2.0
indices(129,1)=3.0
indices(129,2)=2.0
indices(129,3)=2.0
indices(129,4)=1.0
indices(129,5)=0.0
indices(129,6)=0.0
indices(130,1)=3.0
indices(130,2)=2.0
indices(130,3)=2.0
indices(130,4)=1.0
indices(130,5)=0.0
indices(130,6)=1.0
indices(131,1)=3.0
indices(131,2)=2.0
indices(131,3)=2.0
indices(131,4)=1.0
indices(131,5)=1.0
indices(131,6)=0.0
indices(132,1)=3.0
indices(132,2)=2.0
indices(132,3)=2.0
indices(132,4)=2.0
indices(132,5)=0.0
indices(132,6)=0.0
indices(133,1)=3.0
indices(133,2)=3.0
indices(133,3)=0.0
indices(133,4)=0.0
indices(133,5)=0.0
indices(133,6)=0.0
indices(134,1)=3.0
indices(134,2)=3.0
indices(134,3)=0.0
indices(134,4)=0.0
indices(134,5)=1.0
indices(134,6)=0.0
indices(135,1)=3.0
indices(135,2)=3.0
indices(135,3)=0.0
indices(135,4)=0.0
indices(135,5)=1.0
indices(135,6)=1.0
indices(136,1)=3.0
indices(136,2)=3.0
indices(136,3)=0.0
indices(136,4)=0.0
indices(136,5)=2.0
indices(136,6)=0.0
indices(137,1)=3.0
indices(137,2)=3.0
indices(137,3)=0.0
indices(137,4)=0.0
indices(137,5)=2.0
indices(137,6)=1.0
indices(138,1)=3.0
indices(138,2)=3.0
indices(138,3)=0.0
indices(138,4)=0.0
indices(138,5)=3.0
indices(138,6)=0.0
indices(139,1)=3.0
indices(139,2)=3.0
indices(139,3)=0.0
indices(139,4)=1.0
indices(139,5)=0.0
indices(139,6)=0.0
indices(140,1)=3.0
indices(140,2)=3.0
indices(140,3)=0.0
indices(140,4)=1.0
indices(140,5)=1.0
indices(140,6)=0.0
indices(141,1)=3.0
indices(141,2)=3.0
indices(141,3)=0.0
indices(141,4)=1.0
indices(141,5)=1.0
indices(141,6)=1.0
indices(142,1)=3.0
indices(142,2)=3.0
indices(142,3)=0.0
indices(142,4)=1.0
indices(142,5)=2.0
indices(142,6)=0.0
indices(143,1)=3.0
indices(143,2)=3.0
indices(143,3)=0.0
indices(143,4)=2.0
indices(143,5)=0.0
indices(143,6)=0.0
indices(144,1)=3.0
indices(144,2)=3.0
indices(144,3)=0.0
indices(144,4)=2.0
indices(144,5)=1.0
indices(144,6)=0.0
indices(145,1)=3.0
indices(145,2)=3.0
indices(145,3)=0.0
indices(145,4)=3.0
indices(145,5)=0.0
indices(145,6)=0.0
indices(146,1)=3.0
indices(146,2)=3.0
indices(146,3)=1.0
indices(146,4)=0.0
indices(146,5)=0.0
indices(146,6)=0.0
indices(147,1)=3.0
indices(147,2)=3.0
indices(147,3)=1.0
indices(147,4)=0.0
indices(147,5)=1.0
indices(147,6)=0.0
indices(148,1)=3.0
indices(148,2)=3.0
indices(148,3)=1.0
indices(148,4)=0.0
indices(148,5)=1.0
indices(148,6)=1.0
indices(149,1)=3.0
indices(149,2)=3.0
indices(149,3)=1.0
indices(149,4)=0.0
indices(149,5)=2.0
indices(149,6)=0.0
indices(150,1)=3.0
indices(150,2)=3.0
indices(150,3)=1.0
indices(150,4)=1.0
indices(150,5)=0.0
indices(150,6)=0.0
indices(151,1)=3.0
indices(151,2)=3.0
indices(151,3)=1.0
indices(151,4)=1.0
indices(151,5)=1.0
indices(151,6)=0.0
indices(152,1)=3.0
indices(152,2)=3.0
indices(152,3)=1.0
indices(152,4)=2.0
indices(152,5)=0.0
indices(152,6)=0.0
indices(153,1)=3.0
indices(153,2)=3.0
indices(153,3)=2.0
indices(153,4)=0.0
indices(153,5)=0.0
indices(153,6)=0.0
indices(154,1)=3.0
indices(154,2)=3.0
indices(154,3)=2.0
indices(154,4)=0.0
indices(154,5)=1.0
indices(154,6)=0.0
indices(155,1)=3.0
indices(155,2)=3.0
indices(155,3)=2.0
indices(155,4)=1.0
indices(155,5)=0.0
indices(155,6)=0.0
indices(156,1)=3.0
indices(156,2)=3.0
indices(156,3)=3.0
indices(156,4)=0.0
indices(156,5)=0.0
indices(156,6)=0.0
indices(157,1)=4.0
indices(157,2)=0.0
indices(157,3)=0.0
indices(157,4)=0.0
indices(157,5)=0.0
indices(157,6)=0.0
indices(158,1)=4.0
indices(158,2)=0.0
indices(158,3)=0.0
indices(158,4)=0.0
indices(158,5)=0.0
indices(158,6)=1.0
indices(159,1)=4.0
indices(159,2)=0.0
indices(159,3)=0.0
indices(159,4)=0.0
indices(159,5)=0.0
indices(159,6)=2.0
indices(160,1)=4.0
indices(160,2)=0.0
indices(160,3)=0.0
indices(160,4)=0.0
indices(160,5)=0.0
indices(160,6)=3.0
indices(161,1)=4.0
indices(161,2)=0.0
indices(161,3)=0.0
indices(161,4)=0.0
indices(161,5)=0.0
indices(161,6)=4.0
indices(162,1)=4.0
indices(162,2)=1.0
indices(162,3)=0.0
indices(162,4)=0.0
indices(162,5)=0.0
indices(162,6)=0.0
indices(163,1)=4.0
indices(163,2)=1.0
indices(163,3)=0.0
indices(163,4)=0.0
indices(163,5)=0.0
indices(163,6)=1.0
indices(164,1)=4.0
indices(164,2)=1.0
indices(164,3)=0.0
indices(164,4)=0.0
indices(164,5)=0.0
indices(164,6)=2.0
indices(165,1)=4.0
indices(165,2)=1.0
indices(165,3)=0.0
indices(165,4)=0.0
indices(165,5)=0.0
indices(165,6)=3.0
indices(166,1)=4.0
indices(166,2)=1.0
indices(166,3)=0.0
indices(166,4)=0.0
indices(166,5)=0.0
indices(166,6)=4.0
indices(167,1)=4.0
indices(167,2)=1.0
indices(167,3)=0.0
indices(167,4)=0.0
indices(167,5)=1.0
indices(167,6)=0.0
indices(168,1)=4.0
indices(168,2)=1.0
indices(168,3)=0.0
indices(168,4)=0.0
indices(168,5)=1.0
indices(168,6)=1.0
indices(169,1)=4.0
indices(169,2)=1.0
indices(169,3)=0.0
indices(169,4)=0.0
indices(169,5)=1.0
indices(169,6)=2.0
indices(170,1)=4.0
indices(170,2)=1.0
indices(170,3)=0.0
indices(170,4)=0.0
indices(170,5)=1.0
indices(170,6)=3.0
indices(171,1)=4.0
indices(171,2)=1.0
indices(171,3)=0.0
indices(171,4)=1.0
indices(171,5)=0.0
indices(171,6)=0.0
indices(172,1)=4.0
indices(172,2)=1.0
indices(172,3)=0.0
indices(172,4)=1.0
indices(172,5)=0.0
indices(172,6)=1.0
indices(173,1)=4.0
indices(173,2)=1.0
indices(173,3)=0.0
indices(173,4)=1.0
indices(173,5)=0.0
indices(173,6)=2.0
indices(174,1)=4.0
indices(174,2)=1.0
indices(174,3)=0.0
indices(174,4)=1.0
indices(174,5)=0.0
indices(174,6)=3.0
indices(175,1)=4.0
indices(175,2)=1.0
indices(175,3)=1.0
indices(175,4)=0.0
indices(175,5)=0.0
indices(175,6)=0.0
indices(176,1)=4.0
indices(176,2)=1.0
indices(176,3)=1.0
indices(176,4)=0.0
indices(176,5)=0.0
indices(176,6)=1.0
indices(177,1)=4.0
indices(177,2)=1.0
indices(177,3)=1.0
indices(177,4)=0.0
indices(177,5)=0.0
indices(177,6)=2.0
indices(178,1)=4.0
indices(178,2)=1.0
indices(178,3)=1.0
indices(178,4)=0.0
indices(178,5)=0.0
indices(178,6)=3.0
indices(179,1)=4.0
indices(179,2)=1.0
indices(179,3)=1.0
indices(179,4)=1.0
indices(179,5)=0.0
indices(179,6)=0.0
indices(180,1)=4.0
indices(180,2)=1.0
indices(180,3)=1.0
indices(180,4)=1.0
indices(180,5)=0.0
indices(180,6)=1.0
indices(181,1)=4.0
indices(181,2)=1.0
indices(181,3)=1.0
indices(181,4)=1.0
indices(181,5)=0.0
indices(181,6)=2.0
indices(182,1)=4.0
indices(182,2)=1.0
indices(182,3)=1.0
indices(182,4)=1.0
indices(182,5)=1.0
indices(182,6)=0.0
indices(183,1)=4.0
indices(183,2)=1.0
indices(183,3)=1.0
indices(183,4)=1.0
indices(183,5)=1.0
indices(183,6)=1.0
indices(184,1)=4.0
indices(184,2)=2.0
indices(184,3)=0.0
indices(184,4)=0.0
indices(184,5)=0.0
indices(184,6)=0.0
indices(185,1)=4.0
indices(185,2)=2.0
indices(185,3)=0.0
indices(185,4)=0.0
indices(185,5)=0.0
indices(185,6)=1.0
indices(186,1)=4.0
indices(186,2)=2.0
indices(186,3)=0.0
indices(186,4)=0.0
indices(186,5)=0.0
indices(186,6)=2.0
indices(187,1)=4.0
indices(187,2)=2.0
indices(187,3)=0.0
indices(187,4)=0.0
indices(187,5)=0.0
indices(187,6)=3.0
indices(188,1)=4.0
indices(188,2)=2.0
indices(188,3)=0.0
indices(188,4)=0.0
indices(188,5)=1.0
indices(188,6)=0.0
indices(189,1)=4.0
indices(189,2)=2.0
indices(189,3)=0.0
indices(189,4)=0.0
indices(189,5)=1.0
indices(189,6)=1.0
indices(190,1)=4.0
indices(190,2)=2.0
indices(190,3)=0.0
indices(190,4)=0.0
indices(190,5)=1.0
indices(190,6)=2.0
indices(191,1)=4.0
indices(191,2)=2.0
indices(191,3)=0.0
indices(191,4)=0.0
indices(191,5)=2.0
indices(191,6)=0.0
indices(192,1)=4.0
indices(192,2)=2.0
indices(192,3)=0.0
indices(192,4)=0.0
indices(192,5)=2.0
indices(192,6)=1.0
indices(193,1)=4.0
indices(193,2)=2.0
indices(193,3)=0.0
indices(193,4)=1.0
indices(193,5)=0.0
indices(193,6)=0.0
indices(194,1)=4.0
indices(194,2)=2.0
indices(194,3)=0.0
indices(194,4)=1.0
indices(194,5)=0.0
indices(194,6)=1.0
indices(195,1)=4.0
indices(195,2)=2.0
indices(195,3)=0.0
indices(195,4)=1.0
indices(195,5)=0.0
indices(195,6)=2.0
indices(196,1)=4.0
indices(196,2)=2.0
indices(196,3)=0.0
indices(196,4)=1.0
indices(196,5)=1.0
indices(196,6)=0.0
indices(197,1)=4.0
indices(197,2)=2.0
indices(197,3)=0.0
indices(197,4)=1.0
indices(197,5)=1.0
indices(197,6)=1.0
indices(198,1)=4.0
indices(198,2)=2.0
indices(198,3)=0.0
indices(198,4)=2.0
indices(198,5)=0.0
indices(198,6)=0.0
indices(199,1)=4.0
indices(199,2)=2.0
indices(199,3)=0.0
indices(199,4)=2.0
indices(199,5)=0.0
indices(199,6)=1.0
indices(200,1)=4.0
indices(200,2)=2.0
indices(200,3)=1.0
indices(200,4)=0.0
indices(200,5)=0.0
indices(200,6)=0.0
indices(201,1)=4.0
indices(201,2)=2.0
indices(201,3)=1.0
indices(201,4)=0.0
indices(201,5)=0.0
indices(201,6)=1.0
indices(202,1)=4.0
indices(202,2)=2.0
indices(202,3)=1.0
indices(202,4)=0.0
indices(202,5)=0.0
indices(202,6)=2.0
indices(203,1)=4.0
indices(203,2)=2.0
indices(203,3)=1.0
indices(203,4)=0.0
indices(203,5)=1.0
indices(203,6)=0.0
indices(204,1)=4.0
indices(204,2)=2.0
indices(204,3)=1.0
indices(204,4)=0.0
indices(204,5)=1.0
indices(204,6)=1.0
indices(205,1)=4.0
indices(205,2)=2.0
indices(205,3)=1.0
indices(205,4)=0.0
indices(205,5)=2.0
indices(205,6)=0.0
indices(206,1)=4.0
indices(206,2)=2.0
indices(206,3)=1.0
indices(206,4)=1.0
indices(206,5)=0.0
indices(206,6)=0.0
indices(207,1)=4.0
indices(207,2)=2.0
indices(207,3)=1.0
indices(207,4)=1.0
indices(207,5)=0.0
indices(207,6)=1.0
indices(208,1)=4.0
indices(208,2)=2.0
indices(208,3)=1.0
indices(208,4)=1.0
indices(208,5)=1.0
indices(208,6)=0.0
indices(209,1)=4.0
indices(209,2)=2.0
indices(209,3)=1.0
indices(209,4)=2.0
indices(209,5)=0.0
indices(209,6)=0.0
indices(210,1)=4.0
indices(210,2)=2.0
indices(210,3)=2.0
indices(210,4)=0.0
indices(210,5)=0.0
indices(210,6)=0.0
indices(211,1)=4.0
indices(211,2)=2.0
indices(211,3)=2.0
indices(211,4)=0.0
indices(211,5)=0.0
indices(211,6)=1.0
indices(212,1)=4.0
indices(212,2)=2.0
indices(212,3)=2.0
indices(212,4)=1.0
indices(212,5)=0.0
indices(212,6)=0.0
indices(213,1)=4.0
indices(213,2)=3.0
indices(213,3)=0.0
indices(213,4)=0.0
indices(213,5)=0.0
indices(213,6)=0.0
indices(214,1)=4.0
indices(214,2)=3.0
indices(214,3)=0.0
indices(214,4)=0.0
indices(214,5)=0.0
indices(214,6)=1.0
indices(215,1)=4.0
indices(215,2)=3.0
indices(215,3)=0.0
indices(215,4)=0.0
indices(215,5)=0.0
indices(215,6)=2.0
indices(216,1)=4.0
indices(216,2)=3.0
indices(216,3)=0.0
indices(216,4)=0.0
indices(216,5)=1.0
indices(216,6)=0.0
indices(217,1)=4.0
indices(217,2)=3.0
indices(217,3)=0.0
indices(217,4)=0.0
indices(217,5)=1.0
indices(217,6)=1.0
indices(218,1)=4.0
indices(218,2)=3.0
indices(218,3)=0.0
indices(218,4)=0.0
indices(218,5)=2.0
indices(218,6)=0.0
indices(219,1)=4.0
indices(219,2)=3.0
indices(219,3)=0.0
indices(219,4)=1.0
indices(219,5)=0.0
indices(219,6)=0.0
indices(220,1)=4.0
indices(220,2)=3.0
indices(220,3)=0.0
indices(220,4)=1.0
indices(220,5)=0.0
indices(220,6)=1.0
indices(221,1)=4.0
indices(221,2)=3.0
indices(221,3)=0.0
indices(221,4)=1.0
indices(221,5)=1.0
indices(221,6)=0.0
indices(222,1)=4.0
indices(222,2)=3.0
indices(222,3)=0.0
indices(222,4)=2.0
indices(222,5)=0.0
indices(222,6)=0.0
indices(223,1)=4.0
indices(223,2)=3.0
indices(223,3)=1.0
indices(223,4)=0.0
indices(223,5)=0.0
indices(223,6)=0.0
indices(224,1)=4.0
indices(224,2)=3.0
indices(224,3)=1.0
indices(224,4)=0.0
indices(224,5)=0.0
indices(224,6)=1.0
indices(225,1)=4.0
indices(225,2)=3.0
indices(225,3)=1.0
indices(225,4)=0.0
indices(225,5)=1.0
indices(225,6)=0.0
indices(226,1)=4.0
indices(226,2)=3.0
indices(226,3)=1.0
indices(226,4)=1.0
indices(226,5)=0.0
indices(226,6)=0.0
indices(227,1)=4.0
indices(227,2)=3.0
indices(227,3)=2.0
indices(227,4)=0.0
indices(227,5)=0.0
indices(227,6)=0.0
indices(228,1)=4.0
indices(228,2)=4.0
indices(228,3)=0.0
indices(228,4)=0.0
indices(228,5)=0.0
indices(228,6)=0.0
indices(229,1)=4.0
indices(229,2)=4.0
indices(229,3)=0.0
indices(229,4)=0.0
indices(229,5)=1.0
indices(229,6)=0.0
indices(230,1)=4.0
indices(230,2)=4.0
indices(230,3)=0.0
indices(230,4)=1.0
indices(230,5)=0.0
indices(230,6)=0.0
indices(231,1)=4.0
indices(231,2)=4.0
indices(231,3)=1.0
indices(231,4)=0.0
indices(231,5)=0.0
indices(231,6)=0.0
indices(232,1)=5.0
indices(232,2)=0.0
indices(232,3)=0.0
indices(232,4)=0.0
indices(232,5)=0.0
indices(232,6)=0.0
indices(233,1)=5.0
indices(233,2)=0.0
indices(233,3)=0.0
indices(233,4)=0.0
indices(233,5)=0.0
indices(233,6)=1.0
indices(234,1)=5.0
indices(234,2)=0.0
indices(234,3)=0.0
indices(234,4)=0.0
indices(234,5)=0.0
indices(234,6)=2.0
indices(235,1)=5.0
indices(235,2)=0.0
indices(235,3)=0.0
indices(235,4)=0.0
indices(235,5)=0.0
indices(235,6)=3.0
indices(236,1)=5.0
indices(236,2)=0.0
indices(236,3)=0.0
indices(236,4)=0.0
indices(236,5)=0.0
indices(236,6)=4.0
indices(237,1)=5.0
indices(237,2)=1.0
indices(237,3)=0.0
indices(237,4)=0.0
indices(237,5)=0.0
indices(237,6)=0.0
indices(238,1)=5.0
indices(238,2)=1.0
indices(238,3)=0.0
indices(238,4)=0.0
indices(238,5)=0.0
indices(238,6)=1.0
indices(239,1)=5.0
indices(239,2)=1.0
indices(239,3)=0.0
indices(239,4)=0.0
indices(239,5)=0.0
indices(239,6)=2.0
indices(240,1)=5.0
indices(240,2)=1.0
indices(240,3)=0.0
indices(240,4)=0.0
indices(240,5)=0.0
indices(240,6)=3.0
indices(241,1)=5.0
indices(241,2)=1.0
indices(241,3)=0.0
indices(241,4)=0.0
indices(241,5)=1.0
indices(241,6)=0.0
indices(242,1)=5.0
indices(242,2)=1.0
indices(242,3)=0.0
indices(242,4)=0.0
indices(242,5)=1.0
indices(242,6)=1.0
indices(243,1)=5.0
indices(243,2)=1.0
indices(243,3)=0.0
indices(243,4)=0.0
indices(243,5)=1.0
indices(243,6)=2.0
indices(244,1)=5.0
indices(244,2)=1.0
indices(244,3)=0.0
indices(244,4)=1.0
indices(244,5)=0.0
indices(244,6)=0.0
indices(245,1)=5.0
indices(245,2)=1.0
indices(245,3)=0.0
indices(245,4)=1.0
indices(245,5)=0.0
indices(245,6)=1.0
indices(246,1)=5.0
indices(246,2)=1.0
indices(246,3)=0.0
indices(246,4)=1.0
indices(246,5)=0.0
indices(246,6)=2.0
indices(247,1)=5.0
indices(247,2)=1.0
indices(247,3)=1.0
indices(247,4)=0.0
indices(247,5)=0.0
indices(247,6)=0.0
indices(248,1)=5.0
indices(248,2)=1.0
indices(248,3)=1.0
indices(248,4)=0.0
indices(248,5)=0.0
indices(248,6)=1.0
indices(249,1)=5.0
indices(249,2)=1.0
indices(249,3)=1.0
indices(249,4)=0.0
indices(249,5)=0.0
indices(249,6)=2.0
indices(250,1)=5.0
indices(250,2)=1.0
indices(250,3)=1.0
indices(250,4)=1.0
indices(250,5)=0.0
indices(250,6)=0.0
indices(251,1)=5.0
indices(251,2)=1.0
indices(251,3)=1.0
indices(251,4)=1.0
indices(251,5)=0.0
indices(251,6)=1.0
indices(252,1)=5.0
indices(252,2)=1.0
indices(252,3)=1.0
indices(252,4)=1.0
indices(252,5)=1.0
indices(252,6)=0.0
indices(253,1)=5.0
indices(253,2)=2.0
indices(253,3)=0.0
indices(253,4)=0.0
indices(253,5)=0.0
indices(253,6)=0.0
indices(254,1)=5.0
indices(254,2)=2.0
indices(254,3)=0.0
indices(254,4)=0.0
indices(254,5)=0.0
indices(254,6)=1.0
indices(255,1)=5.0
indices(255,2)=2.0
indices(255,3)=0.0
indices(255,4)=0.0
indices(255,5)=0.0
indices(255,6)=2.0
indices(256,1)=5.0
indices(256,2)=2.0
indices(256,3)=0.0
indices(256,4)=0.0
indices(256,5)=1.0
indices(256,6)=0.0
indices(257,1)=5.0
indices(257,2)=2.0
indices(257,3)=0.0
indices(257,4)=0.0
indices(257,5)=1.0
indices(257,6)=1.0
indices(258,1)=5.0
indices(258,2)=2.0
indices(258,3)=0.0
indices(258,4)=0.0
indices(258,5)=2.0
indices(258,6)=0.0
indices(259,1)=5.0
indices(259,2)=2.0
indices(259,3)=0.0
indices(259,4)=1.0
indices(259,5)=0.0
indices(259,6)=0.0
indices(260,1)=5.0
indices(260,2)=2.0
indices(260,3)=0.0
indices(260,4)=1.0
indices(260,5)=0.0
indices(260,6)=1.0
indices(261,1)=5.0
indices(261,2)=2.0
indices(261,3)=0.0
indices(261,4)=1.0
indices(261,5)=1.0
indices(261,6)=0.0
indices(262,1)=5.0
indices(262,2)=2.0
indices(262,3)=0.0
indices(262,4)=2.0
indices(262,5)=0.0
indices(262,6)=0.0
indices(263,1)=5.0
indices(263,2)=2.0
indices(263,3)=1.0
indices(263,4)=0.0
indices(263,5)=0.0
indices(263,6)=0.0
indices(264,1)=5.0
indices(264,2)=2.0
indices(264,3)=1.0
indices(264,4)=0.0
indices(264,5)=0.0
indices(264,6)=1.0
indices(265,1)=5.0
indices(265,2)=2.0
indices(265,3)=1.0
indices(265,4)=0.0
indices(265,5)=1.0
indices(265,6)=0.0
indices(266,1)=5.0
indices(266,2)=2.0
indices(266,3)=1.0
indices(266,4)=1.0
indices(266,5)=0.0
indices(266,6)=0.0
indices(267,1)=5.0
indices(267,2)=2.0
indices(267,3)=2.0
indices(267,4)=0.0
indices(267,5)=0.0
indices(267,6)=0.0
indices(268,1)=5.0
indices(268,2)=3.0
indices(268,3)=0.0
indices(268,4)=0.0
indices(268,5)=0.0
indices(268,6)=0.0
indices(269,1)=5.0
indices(269,2)=3.0
indices(269,3)=0.0
indices(269,4)=0.0
indices(269,5)=0.0
indices(269,6)=1.0
indices(270,1)=5.0
indices(270,2)=3.0
indices(270,3)=0.0
indices(270,4)=0.0
indices(270,5)=1.0
indices(270,6)=0.0
indices(271,1)=5.0
indices(271,2)=3.0
indices(271,3)=0.0
indices(271,4)=1.0
indices(271,5)=0.0
indices(271,6)=0.0
indices(272,1)=5.0
indices(272,2)=3.0
indices(272,3)=1.0
indices(272,4)=0.0
indices(272,5)=0.0
indices(272,6)=0.0
indices(273,1)=5.0
indices(273,2)=4.0
indices(273,3)=0.0
indices(273,4)=0.0
indices(273,5)=0.0
indices(273,6)=0.0
indices(274,1)=6.0
indices(274,2)=0.0
indices(274,3)=0.0
indices(274,4)=0.0
indices(274,5)=0.0
indices(274,6)=0.0
indices(275,1)=6.0
indices(275,2)=0.0
indices(275,3)=0.0
indices(275,4)=0.0
indices(275,5)=0.0
indices(275,6)=1.0
indices(276,1)=6.0
indices(276,2)=0.0
indices(276,3)=0.0
indices(276,4)=0.0
indices(276,5)=0.0
indices(276,6)=2.0
indices(277,1)=6.0
indices(277,2)=0.0
indices(277,3)=0.0
indices(277,4)=0.0
indices(277,5)=0.0
indices(277,6)=3.0
indices(278,1)=6.0
indices(278,2)=1.0
indices(278,3)=0.0
indices(278,4)=0.0
indices(278,5)=0.0
indices(278,6)=0.0
indices(279,1)=6.0
indices(279,2)=1.0
indices(279,3)=0.0
indices(279,4)=0.0
indices(279,5)=0.0
indices(279,6)=1.0
indices(280,1)=6.0
indices(280,2)=1.0
indices(280,3)=0.0
indices(280,4)=0.0
indices(280,5)=0.0
indices(280,6)=2.0
indices(281,1)=6.0
indices(281,2)=1.0
indices(281,3)=0.0
indices(281,4)=0.0
indices(281,5)=1.0
indices(281,6)=0.0
indices(282,1)=6.0
indices(282,2)=1.0
indices(282,3)=0.0
indices(282,4)=0.0
indices(282,5)=1.0
indices(282,6)=1.0
indices(283,1)=6.0
indices(283,2)=1.0
indices(283,3)=0.0
indices(283,4)=1.0
indices(283,5)=0.0
indices(283,6)=0.0
indices(284,1)=6.0
indices(284,2)=1.0
indices(284,3)=0.0
indices(284,4)=1.0
indices(284,5)=0.0
indices(284,6)=1.0
indices(285,1)=6.0
indices(285,2)=1.0
indices(285,3)=1.0
indices(285,4)=0.0
indices(285,5)=0.0
indices(285,6)=0.0
indices(286,1)=6.0
indices(286,2)=1.0
indices(286,3)=1.0
indices(286,4)=0.0
indices(286,5)=0.0
indices(286,6)=1.0
indices(287,1)=6.0
indices(287,2)=1.0
indices(287,3)=1.0
indices(287,4)=1.0
indices(287,5)=0.0
indices(287,6)=0.0
indices(288,1)=6.0
indices(288,2)=2.0
indices(288,3)=0.0
indices(288,4)=0.0
indices(288,5)=0.0
indices(288,6)=0.0
indices(289,1)=6.0
indices(289,2)=2.0
indices(289,3)=0.0
indices(289,4)=0.0
indices(289,5)=0.0
indices(289,6)=1.0
indices(290,1)=6.0
indices(290,2)=2.0
indices(290,3)=0.0
indices(290,4)=0.0
indices(290,5)=1.0
indices(290,6)=0.0
indices(291,1)=6.0
indices(291,2)=2.0
indices(291,3)=0.0
indices(291,4)=1.0
indices(291,5)=0.0
indices(291,6)=0.0
indices(292,1)=6.0
indices(292,2)=2.0
indices(292,3)=1.0
indices(292,4)=0.0
indices(292,5)=0.0
indices(292,6)=0.0
indices(293,1)=6.0
indices(293,2)=3.0
indices(293,3)=0.0
indices(293,4)=0.0
indices(293,5)=0.0
indices(293,6)=0.0
indices(294,1)=7.0
indices(294,2)=0.0
indices(294,3)=0.0
indices(294,4)=0.0
indices(294,5)=0.0
indices(294,6)=0.0
indices(295,1)=7.0
indices(295,2)=0.0
indices(295,3)=0.0
indices(295,4)=0.0
indices(295,5)=0.0
indices(295,6)=1.0
indices(296,1)=7.0
indices(296,2)=0.0
indices(296,3)=0.0
indices(296,4)=0.0
indices(296,5)=0.0
indices(296,6)=2.0
indices(297,1)=7.0
indices(297,2)=1.0
indices(297,3)=0.0
indices(297,4)=0.0
indices(297,5)=0.0
indices(297,6)=0.0
indices(298,1)=7.0
indices(298,2)=1.0
indices(298,3)=0.0
indices(298,4)=0.0
indices(298,5)=0.0
indices(298,6)=1.0
indices(299,1)=7.0
indices(299,2)=1.0
indices(299,3)=0.0
indices(299,4)=0.0
indices(299,5)=1.0
indices(299,6)=0.0
indices(300,1)=7.0
indices(300,2)=1.0
indices(300,3)=0.0
indices(300,4)=1.0
indices(300,5)=0.0
indices(300,6)=0.0
indices(301,1)=7.0
indices(301,2)=1.0
indices(301,3)=1.0
indices(301,4)=0.0
indices(301,5)=0.0
indices(301,6)=0.0
indices(302,1)=7.0
indices(302,2)=2.0
indices(302,3)=0.0
indices(302,4)=0.0
indices(302,5)=0.0
indices(302,6)=0.0
indices(303,1)=8.0
indices(303,2)=0.0
indices(303,3)=0.0
indices(303,4)=0.0
indices(303,5)=0.0
indices(303,6)=0.0
indices(304,1)=8.0
indices(304,2)=0.0
indices(304,3)=0.0
indices(304,4)=0.0
indices(304,5)=0.0
indices(304,6)=1.0
indices(305,1)=8.0
indices(305,2)=1.0
indices(305,3)=0.0
indices(305,4)=0.0
indices(305,5)=0.0
indices(305,6)=0.0
indices(306,1)=9.0
indices(306,2)=0.0
indices(306,3)=0.0
indices(306,4)=0.0
indices(306,5)=0.0
indices(306,6)=0.0

coeff(1)=20.0090895531
coeff(2)=-11.5887207687
coeff(3)=19.7221707106
coeff(4)=-86.7349555492
coeff(5)=333.942126274
coeff(6)=-448.397304058
coeff(7)=-2.88140773773
coeff(8)=-203.067146063
coeff(9)=326.325057983
coeff(10)=281.495002925
coeff(11)=-1809.10965919
coeff(12)=200.966255665
coeff(13)=-234.479382992
coeff(14)=-55.9843527079
coeff(15)=503.129985809
coeff(16)=-707.194470406
coeff(17)=-363.737837315
coeff(18)=-396.403343737
coeff(19)=1115.15323639
coeff(20)=-1748.20399445
coeff(21)=-77.0317955017
coeff(22)=2888.79562759
coeff(23)=476.395507812
coeff(24)=-450.839231014
coeff(25)=-1970.94978571
coeff(26)=-3461.28881359
coeff(27)=3027.67502499
coeff(28)=-4620.78422451
coeff(29)=285.832065582
coeff(30)=3297.30986023
coeff(31)=-77.477047205
coeff(32)=343.650630951
coeff(33)=455.879302919
coeff(34)=412.268882751
coeff(35)=-636.201309204
coeff(36)=978.688254356
coeff(37)=-53.5518376827
coeff(38)=2943.62019444
coeff(39)=-2367.05615044
coeff(40)=-503.684679031
coeff(41)=2520.60064316
coeff(42)=356.003650665
coeff(43)=1576.39877129
coeff(44)=-595.007182121
coeff(45)=1290.93126726
coeff(46)=-5063.96334267
coeff(47)=-670.436934471
coeff(48)=2551.3109256
coeff(49)=3529.10512257
coeff(50)=-989.953546166
coeff(51)=4063.45350873
coeff(52)=844.417476892
coeff(53)=291.182150364
coeff(54)=1507.01752567
coeff(55)=-1838.02449965
coeff(56)=214.83052063
coeff(57)=-408.909545779
coeff(58)=1449.9099822
coeff(59)=-178.821709514
coeff(60)=1070.1499474
coeff(61)=1816.18722248
coeff(62)=-1044.06338882
coeff(63)=831.840065241
coeff(64)=108.20316422
coeff(65)=-238.437209308
coeff(66)=-822.914962769
coeff(67)=107.77973175
coeff(68)=1887.87818766
coeff(69)=-923.857162237
coeff(70)=1034.83486271
coeff(71)=-350.223211288
coeff(72)=2375.12545949
coeff(73)=-397.753879488
coeff(74)=-3172.84950256
coeff(75)=-1673.45114899
coeff(76)=783.21253109
coeff(77)=-213.199672699
coeff(78)=-125.926317215
coeff(79)=1669.25241613
coeff(80)=-574.365280151
coeff(81)=1255.3966589
coeff(82)=-2441.9034276
coeff(83)=3166.12516212
coeff(84)=-190.102167636
coeff(85)=-2493.67355156
coeff(86)=-717.727438927
coeff(87)=-543.447493911
coeff(88)=-722.413299561
coeff(89)=-646.368333817
coeff(90)=616.371473789
coeff(91)=-44.8517619967
coeff(92)=839.643658161
coeff(93)=-1016.15244389
coeff(94)=382.719281673
coeff(95)=-209.735216975
coeff(96)=-468.410824239
coeff(97)=644.714993
coeff(98)=199.760655046
coeff(99)=-7978.29972458
coeff(100)=2738.93719196
coeff(101)=-892.417957306
coeff(102)=-1371.19241333
coeff(103)=2155.4454546
coeff(104)=640.899897218
coeff(105)=-1766.44680977
coeff(106)=523.868308544
coeff(107)=-455.634624183
coeff(108)=-1872.31838608
coeff(109)=-2169.26821256
coeff(110)=-615.446153164
coeff(111)=255.579271436
coeff(112)=981.065886974
coeff(113)=1689.65754652
coeff(114)=1628.86166786
coeff(115)=-1954.97047943
coeff(116)=-1064.85853148
coeff(117)=-2935.84000158
coeff(118)=-5328.05898476
coeff(119)=-1721.86402029
coeff(120)=-2686.06601715
coeff(121)=2759.20209408
coeff(122)=-460.285434604
coeff(123)=-1207.80733657
coeff(124)=716.659038782
coeff(125)=-632.765380979
coeff(126)=-355.631705284
coeff(127)=-1410.0276103
coeff(128)=-550.133598566
coeff(129)=655.273667336
coeff(130)=-359.271378994
coeff(131)=-867.474842489
coeff(132)=155.854884267
coeff(133)=1918.7529583
coeff(134)=-273.56869483
coeff(135)=385.540159464
coeff(136)=284.800862789
coeff(137)=-1.60925865173
coeff(138)=-103.122198939
coeff(139)=855.737588406
coeff(140)=60.2840312719
coeff(141)=-586.63263905
coeff(142)=538.629257552
coeff(143)=-560.149035811
coeff(144)=111.934583902
coeff(145)=3.09094405174
coeff(146)=1952.92674375
coeff(147)=1678.80417395
coeff(148)=-740.564001679
coeff(149)=660.670416892
coeff(150)=1585.57669389
coeff(151)=1119.16041088
coeff(152)=97.9025948048
coeff(153)=72.6993988156
coeff(154)=608.235356152
coeff(155)=-347.39153564
coeff(156)=-55.8014267087
coeff(157)=1821.89665222
coeff(158)=-1295.59673262
coeff(159)=153.037760705
coeff(160)=204.366000056
coeff(161)=-45.9663589187
coeff(162)=969.368408203
coeff(163)=-21.9984149933
coeff(164)=-1080.64204955
coeff(165)=588.557963312
coeff(166)=-45.0684316382
coeff(167)=-1502.51196718
coeff(168)=-117.528707504
coeff(169)=-1136.01718736
coeff(170)=170.674474597
coeff(171)=5553.68289948
coeff(172)=61.2335009575
coeff(173)=191.709527135
coeff(174)=2.81358644366
coeff(175)=554.647976875
coeff(176)=706.456432402
coeff(177)=-373.016941965
coeff(178)=51.1416481538
coeff(179)=1248.17429733
coeff(180)=1239.90309966
coeff(181)=-145.873588681
coeff(182)=1549.75596702
coeff(183)=-556.724405706
coeff(184)=-831.310619354
coeff(185)=138.431963921
coeff(186)=-237.248182684
coeff(187)=54.0359181166
coeff(188)=169.333351135
coeff(189)=-1298.69425917
coeff(190)=-268.633340597
coeff(191)=302.150424957
coeff(192)=-51.222595036
coeff(193)=4682.08414078
coeff(194)=-939.137205601
coeff(195)=215.376364946
coeff(196)=1138.50523043
coeff(197)=-58.7457249053
coeff(198)=1461.09254742
coeff(199)=-80.5240371823
coeff(200)=-244.148091793
coeff(201)=1395.23323548
coeff(202)=-11.4652409554
coeff(203)=-752.677822441
coeff(204)=264.607505083
coeff(205)=-137.256365702
coeff(206)=82.8996634483
coeff(207)=-48.8548958302
coeff(208)=636.733640313
coeff(209)=-121.441988587
coeff(210)=171.36312139
coeff(211)=88.8648527861
coeff(212)=239.940779582
coeff(213)=-1480.92483091
coeff(214)=-134.271784008
coeff(215)=-44.7077780366
coeff(216)=-15.8731156588
coeff(217)=65.7293751091
coeff(218)=-51.7290290296
coeff(219)=-629.1490767
coeff(220)=69.0455036759
coeff(221)=-246.693205833
coeff(222)=57.1834514141
coeff(223)=-1256.07870877
coeff(224)=-188.590211511
coeff(225)=-471.895686805
coeff(226)=-267.742294133
coeff(227)=49.1050944626
coeff(228)=362.882249832
coeff(229)=16.2537142932
coeff(230)=26.159139812
coeff(231)=138.409025002
coeff(232)=-1598.19285583
coeff(233)=1054.91389465
coeff(234)=-157.756746411
coeff(235)=-19.344933033
coeff(236)=9.40141406469
coeff(237)=-745.146903992
coeff(238)=126.822092056
coeff(239)=260.630813003
coeff(240)=-61.988104362
coeff(241)=701.110708237
coeff(242)=566.48763442
coeff(243)=165.880408585
coeff(244)=-3614.30802155
coeff(245)=-5.4166520834
coeff(246)=-79.5582942963
coeff(247)=-258.157049179
coeff(248)=-669.910598516
coeff(249)=39.4095713496
coeff(250)=-668.660885096
coeff(251)=-22.6420143545
coeff(252)=-275.822512329
coeff(253)=631.650398254
coeff(254)=248.205283523
coeff(255)=19.631254822
coeff(256)=-223.314203501
coeff(257)=266.750038385
coeff(258)=-18.3751270766
coeff(259)=-1500.5615654
coeff(260)=82.746000737
coeff(261)=65.7073403895
coeff(262)=-192.348998842
coeff(263)=370.130644977
coeff(264)=-167.082086161
coeff(265)=189.18458876
coeff(266)=23.9709943533
coeff(267)=-85.9011288658
coeff(268)=96.393843472
coeff(269)=36.2117903419
coeff(270)=28.5201430842
coeff(271)=107.758844096
coeff(272)=51.6429955736
coeff(273)=-79.3071212098
coeff(274)=778.147239685
coeff(275)=-480.50680092
coeff(276)=47.5733121932
coeff(277)=-1.86793363094
coeff(278)=350.167434692
coeff(279)=-89.6645691395
coeff(280)=-22.229791671
coeff(281)=-158.316040277
coeff(282)=-159.940125257
coeff(283)=1006.21036339
coeff(284)=24.3442240432
coeff(285)=38.9910695553
coeff(286)=128.382821523
coeff(287)=31.2633836865
coeff(288)=-57.6192321777
coeff(289)=-76.4117296636
coeff(290)=7.29962405562
coeff(291)=144.0181517
coeff(292)=-12.5960336439
coeff(293)=28.7447488084
coeff(294)=-197.68113327
coeff(295)=117.390559211
coeff(296)=-4.52187299356
coeff(297)=-87.4991579056
coeff(298)=18.567718029
coeff(299)=24.5237945616
coeff(300)=-90.156046629
coeff(301)=-5.12470173836
coeff(302)=-18.2263798118
coeff(303)=20.3886165619
coeff(304)=-12.0787184406
coeff(305)=7.92816877365
coeff(306)=0.0492935180664
endsubroutine



