!***********************************************************************
!   System:                     O3
!   Functional form:            permutation restrained diabatization by 
!                               deep neural network
!   Common name:                O3_14_3Ap_2022 (14 quintet A' states)
!   Number of derivatives:      first derivatives of adiabatic surfaces
!   Number of bodies:           3
!   Number of surfaces:         14
!   Notes:                      -Diatomic potentials contain dispersion terms
!                               -Compile the subroutine requires LAPACK 
!                                subroutine DSYEV
! 
!   Input:                      Cartesian coordinates in bohr
!   Output:                     Adiabatic energies (e) in hartrees,
!                               derivatives of adiabataic energies (g) in 
!                               hartrees/bohr, and NACs (h) in inverse bohrs.
!   Reference:             
!   Z. Varga, Y. Shu, J. Ning, D. G. Truhlar, "Diabatic potential energy surfaces
!   and semiclassical multi-state dynamics for fourteen coupled 3A' states of O3"
!   Electron. Struct. 2022, 4, 047002. DOI 10.1088/2516-1075/ac94ac
!***********************************************************************

      subroutine dpem(x,igrad,u,ug)

      implicit none
      ! number of electronic state
      integer, parameter :: nstates=14
      integer, parameter :: natoms=3
      integer, intent(in) :: igrad
      double precision, intent(in) :: x(natoms,3)
      double precision, intent(out) :: u(nstates,nstates)
      double precision, intent(out) :: ug(nstates,nstates,natoms,3)

      double precision :: v(nstates), tx(natoms,3), tg(nstates,3,3)
      double precision :: td(nstates,nstates,3,3)
      double precision :: t(nstates,nstates)

      integer :: iatom, idir, j, istate, jstate
      !initialize 
      u=0.d0
      ug=0.d0

      ! transform from angstrom to bohr
      do iatom=1,natoms
        do idir=1,3
          tx(iatom,idir)=x(iatom,idir)/0.529177211
        enddo
      enddo
      tg=0.d0
      td=0.d0

      call energy_gradient(tx,v,tg,td,t,u,ug,natoms,nstates)

      u=u*27.211386
      ug=ug*51.422067

      end

  subroutine energy_gradient(x,e,g,h,t,u,ug,n,ns)
    ! n    - number of atoms
    ! ns   - number of states
    ! x    - Cartesian coordinates
    ! e    - adiabatic energies
    ! g    - derivatives of adiabatic energies w.r.t. Cartesians
    ! h    - nonadiabatic coupling vectors w.r.t. Cartesians
    ! t    - transformation matrix
    ! r    - internuclear distances
    ! expr - expr=exp(-a*r)
    ! a    - non-linear parameter in expr=exp(-a*r)
    ! gr   - derivatives of adiabatic energies w.r.t. internuclear
    ! distances
    ! v2b  - two-body energy contributions
    ! g2b  - derivatives of two-body energy contributions w.r.t.
    ! internuclear distances
    ! drdx - dr/dx matrix for converting derivatives

    implicit none
    integer, intent(in) :: n,ns
    real*8, intent(in) :: x(n,3)
    real*8, intent(inout) :: e(ns)
    real*8, intent(inout) :: g(ns,n,3)
    real*8, intent(inout) :: t(ns,ns)
    real*8, intent(inout) :: h(ns,ns,n,3)
    real*8, intent(inout) :: u(ns,ns)
    real*8, intent(inout) :: ug(ns,ns,n,3)

    ! layers and their sizes
    real*8, save :: coeff_l0(3,15), coeff_l1(15,35)
    real*8, save :: coeff_l2(35,55), coeff_l3(55,105)
    real*8, save :: bias_l0(15), bias_l1(35), bias_l2(55), bias_l3(105)
    real*8 :: r(n*(n-1)/2),expr(n*(n-1)/2)
    real*8 :: a
    real*8 :: gr(ns,n*(n-1)/2)
    real*8 :: hr(ns,ns,n*(n-1)/2)
    real*8 :: ur(ns,ns,n*(n-1)/2)
    real*8 :: v2b(ns),g2b(ns,n*(n-1)/2)
    real*8 :: drdx(n*(n-1)/2,3*n)
    integer :: i,j,k,l,m
    logical, save :: first_time_data=.true.

    e=0.d0
    g=0.d0

    ! get ddnn coefficients and bias
    if(first_time_data) then
      call assign_coeff_bias(coeff_l0,coeff_l1,coeff_l2,coeff_l3, &
           bias_l0,bias_l1,bias_l2,bias_l3)
      first_time_data=.false.
    endif
    ! input x is in bohr
    ! convert xyz coordinates to exp of internuclear distances,
    ! expr=exp(-a*r), where a = 1.0 1/bohr
    a = 1.0d0
    call convert_xyz_to_expr(x,expr,r,n,a)
    ! calculate the two-body terms
    call ev2gm2m(r,v2b,g2b,4,1,n)
    ! forward propagate sfp to energy 
    call backprop(expr,n,ns,e,gr,hr,u,ur,t,coeff_l0,coeff_l1,coeff_l2, &
         coeff_l3,bias_l0,bias_l1,bias_l2,bias_l3,v2b,g2b,a)
    ! convert from gradient_r to gradient_xyz
    call grad_r_to_xyz(g,drdx,gr,r,x,n,ns)
    call nac_r_to_xyz(h,drdx,hr,r,x,n,ns)
    call nac_r_to_xyz(ug,drdx,ur,r,x,n,ns)
    ! convert energy from eV to hartree (1/27.2107=0.0367502)
    e(:)=e(:)*0.0367502d0
    u(:,:)=u(:,:)*0.0367502d0
    ! convert grad and nac from eV/bohr to hartree/bohr (1/27.2107=0.0367502)
    g(:,:,:)=g(:,:,:)*0.0367502d0
    ug(:,:,:,:)=ug(:,:,:,:)*0.0367502d0

  endsubroutine energy_gradient

!===========================
! back propagation
  subroutine backprop(expr,n,ns,e,g,h,u,ur,t,coeff_l0,coeff_l1,coeff_l2, &
             coeff_l3,bias_l0,bias_l1,bias_l2,bias_l3,v2b,g2b,a)
    implicit none
    integer, intent(in) :: n, ns
    real*8 :: expr(n*(n-1)/2)
    real*8, intent(inout) :: e(ns), g(ns,n*(n-1)/2),h(ns,ns,n*(n-1)/2)
    real*8, intent(inout) :: t(ns,ns), u(ns,ns), ur(ns,ns,n*(n-1)/2)
    real*8, intent(in) :: coeff_l0(3,15), coeff_l1(15,35)
    real*8, intent(in) :: coeff_l2(35,55),coeff_l3(55,105)
    real*8, intent(in) :: bias_l0(15), bias_l1(35)
    real*8, intent(in) :: bias_l2(55), bias_l3(105)

    real*8 :: a0(15), a1(35), a2(55), a3(105)
    real*8 :: z0(15), z1(35), z2(55), z3(105)
    real*8 :: g0(3,15), g1(3,35) 
    real*8 :: g2(3,55), g3(3,105), g3p(105,3)
    real*8 :: dpem(ns,ns), dvdu(ns,105)

    real*8 :: dudr(ns,ns,n*(n-1)/2),t_trans(ns,ns)
    real*8 :: tmp1(ns,ns,n*(n-1)/2),dgdr(ns,ns,n*(n-1)/2)

    integer :: sum1
    integer :: i,j,k,l

    real*8 :: v2b(ns),g2b(ns,n*(n-1)/2)
    real*8 :: dexpdr(n*(n-1)/2,n*(n-1)/2)
    real*8 :: g3r(105,n*(n-1)/2)
    real*8 :: a

    a0=0.0d0
    a1=0.0d0
    a2=0.0d0
    a3=0.0d0
    z0=0.0d0
    z1=0.0d0
    z2=0.0d0
    z3=0.0d0

    e=0.0d0
    g0=0.0d0
    g1=0.0d0
    g2=0.0d0
    g3=0.0d0

    call mproduct(expr,coeff_l0,z0,1,3,15)
    z0=z0 !+bias_l0
    call activation_gelu(z0,a0,15)

    call mproduct(a0,coeff_l1,z1,1,15,35)
    z1=z1 !+bias_l1
    call activation_gelu(z1,a1,35)

    call mproduct(a1,coeff_l2,z2,1,35,55)
    z2=z2 !+bias_l2
    call activation_gelu(z2,a2,55)

    !diabatic layer, no activation function.
    call mproduct(a2,coeff_l3,z3,1,55,105)
    z3=z3 !+bias_l3
    a3=z3

    !construction many-body diabatic potential energy matrix (mb-dpem)
    do i=1,ns
      dpem(i,i)=a3(i)
    enddo
    do i=1,ns
      do j=i+1,ns
        call sumover(ns,i,sum1)
        k=ns-1+sum1+j-i
        dpem(i,j)=a3(k)
        dpem(j,i)=dpem(i,j)
      enddo
    enddo

    !adding the two-body terms to mb-dpem
    do i=1,ns
      dpem(i,i)=dpem(i,i) + v2b(i)
    enddo

    u=dpem

    ! diagonalize dpem
    call diagonalize(ns,dpem,e,t)

    ! cmpute dvdu 
    do i=1,ns
      do j=1,ns
        dvdu(i,j)=t(j,i)*t(j,i)
      enddo
    enddo 
    do i=1,ns
      do j=1,ns
        do k=j+1,ns
          call sumover(ns,j,sum1)
          l=ns-1+sum1+k-j
          dvdu(i,l)=2.0*t(j,i)*t(k,i)
        enddo
      enddo
    enddo 

   ! compute gradients 
   g0=coeff_l0   
   do i=1,3
     call activation_derivative_gelu(z0, a0, g0(i,:), 15)
   enddo 

   call mproduct(g0,coeff_l1,g1,3,15,35)

   do i=1,3
     call activation_derivative_gelu(z1, a1, g1(i,:), 35)
   enddo   

   call mproduct(g1,coeff_l2,g2,3,35,55)
   do i=1,3
     call activation_derivative_gelu(z2, a2, g2(i,:), 55)
   enddo
 
   call mproduct(g2,coeff_l3,g3,3,55,105)
   do i=1,3
     do j=1,105
       g3p(j,i)=g3(i,j)
     enddo
   enddo 

   ! g3p corresponds to dU_MB/dexpr
   ! convert that to dU_MB/dr with dexp(-a*r)/dr
   ! dexp(-a*r)/dr = -a*exp(-a*r)
   ! the unit of dU_MB/dr is eV/bohr
   dexpdr=0.0d0
   do i=1,3
   dexpdr(i,i)= -a*expr(i)
   enddo

   call mproduct(g3p,dexpdr,g3r,105,3,3)

   ! add dU_2B/dr to dU_MB/dr
   ! the unit of dU_2B/dr is also eV/bohr 
   do i=1,ns
     do j=1,3
     g3r(i,j) = g3r(i,j) + g2b(i,j)
     enddo
   enddo

   call mproduct(dvdu,g3r,g,ns,105,3)

   do i=1,ns
     do j=1,ns
       t_trans(i,j)=t(j,i)
     enddo
   enddo

   ! assign g3r elements to dudr
   do i=1,ns
     dudr(i,i,:)=g3r(i,:)
   enddo 
   do i=1,ns
     do j=i+1,ns 
        call sumover(ns,i,sum1)
        k=ns-1+sum1+j-i
        dudr(i,j,:)=g3r(k,:)
        dudr(j,i,:)=dudr(i,j,:)
     enddo
   enddo

   ur=dudr

   do i=1,n*(n-1)/2
     call mproduct(t_trans,dudr(:,:,i),tmp1(:,:,i),ns,ns,ns)
     call mproduct(tmp1(:,:,i),t,dgdr(:,:,i),ns,ns,ns)
   enddo

   h=0.d0
   do i=1,ns
     do j=i+1,ns
        h(i,j,:)=dgdr(i,j,:)/(e(j)-e(i))
        h(j,i,:)=-h(i,j,:)
     enddo
   enddo

  endsubroutine backprop

!=========================
!activation function, gelu
  subroutine activation_gelu(z, a, n)
    integer i,n
    real*8, intent(in) :: z(n)
    real*8, intent(inout) :: a(n)

    do i=1,n
      a(i)=0.5*z(i)*(1+erf(z(i)/sqrt(2.d0)))
    enddo

  end subroutine activation_gelu

  subroutine activation_derivative_gelu(z, a, b, n)
    integer i,n
    real*8, intent(in) :: z(n), a(n)
    real*8, intent(inout) :: b(n)

    do i=1,n
!     b(i)=b(i)*(0.5*erf(z(i)/sqrt(2.d0))+0.398942*exp(-z(i)**2/2)*z(i)+0.5)
      b(i)=b(i)*(a(i)/z(i) + 0.398942*z(i)*exp(-z(i)**2/2) )
    enddo

  end subroutine activation_derivative_gelu

!==============================
! used to compute index in DPEM
  subroutine sumover(a,b,sum1)
    integer, intent(in) :: a, b
    integer, intent(out) :: sum1
    integer :: num

    sum1=1-a
    do num=a, a-b+1, -1
      sum1=sum1+num
    enddo
  end subroutine sumover

!==============================
! matrix diagonalization
! subroutine diagonalize(n,A_ss,B_ss,U_ss)
  subroutine diagonalize(n,A_ss,EV_s,U_ss)
    implicit none
    ! parameters
    integer, intent(in) :: n
    real*8, intent(in) :: A_ss(n,n)
!   real*8, intent(out) :: B_ss(n,n)
    real*8, intent(out) :: U_ss(n,n)
    ! internal variables
!   real*8 :: EV_s(n)
    real*8, intent(out) :: EV_s(n)
    integer :: io,i
    real*8,allocatable :: lapack_work_d(:)
    integer, save :: lapack_lwork_d 
    logical, save :: first_time_diag=.true.

    U_ss=A_ss
    if (first_time_diag) then
      lapack_lwork_d= -1
      allocate( lapack_work_d(2) )
      call dsyev('V','L',n,U_ss,n,EV_s,lapack_work_d,lapack_lwork_d,io)
      lapack_lwork_d=int(lapack_work_d(1))
      deallocate ( lapack_work_d )
      first_time_diag=.false.
    endif
    allocate( lapack_work_d(lapack_lwork_d) )
    call dsyev('V','L',n,U_ss,n,EV_s,lapack_work_d,lapack_lwork_d,io)
    ! U_ss already holds the transformation matrix
    ! now  building the diagonal matrix A_ss
!   B_ss=0.d0
!   do i=1,n
!     B_ss(i,i)=EV_s(i)
!   enddo

    return

  end subroutine diagonalize

!==========================
! convert de/dr to de/dxyz
  subroutine grad_r_to_xyz(g,drdx,gr,r,x,n,ns)
    implicit none
    integer, intent(in) :: n, ns
    real*8, intent(in) :: x(n,3)
    real*8, intent(in) :: r(n*(n-1)/2)
    real*8, intent(in) :: gr(ns,n*(n-1)/2)
    real*8, intent(inout) :: g(ns,n,3)
    real*8, intent(inout) :: drdx(n*(n-1)/2,3*n)

    real*8 ::  gx(ns,3*n)
    integer :: i,j

    drdx=0.0d0

    ! compute drdx
    drdx(1,1)=(x(1,1)-x(2,1))/r(1)
    drdx(1,2)=(x(1,2)-x(2,2))/r(1)
    drdx(1,3)=(x(1,3)-x(2,3))/r(1)
    drdx(1,4)=-drdx(1,1)
    drdx(1,5)=-drdx(1,2)
    drdx(1,6)=-drdx(1,3)
    drdx(2,1)=(x(1,1)-x(3,1))/r(2)
    drdx(2,2)=(x(1,2)-x(3,2))/r(2)
    drdx(2,3)=(x(1,3)-x(3,3))/r(2)
    drdx(2,7)=-drdx(2,1)
    drdx(2,8)=-drdx(2,2)
    drdx(2,9)=-drdx(2,3)
    drdx(3,4)=(x(2,1)-x(3,1))/r(3)
    drdx(3,5)=(x(2,2)-x(3,2))/r(3)
    drdx(3,6)=(x(2,3)-x(3,3))/r(3)
    drdx(3,7)=-drdx(3,4)
    drdx(3,8)=-drdx(3,5)
    drdx(3,9)=-drdx(3,6)

    call mproduct(gr,drdx,gx,ns,n*(n-1)/2,3*n)

    do i=1,n
      g(:,i,1)=gx(:,3*(i-1)+1)
      g(:,i,2)=gx(:,3*(i-1)+2)
      g(:,i,3)=gx(:,3*(i-1)+3)
    enddo 

  end subroutine grad_r_to_xyz

!==========================
! convert de/dr to de/dxyz
  subroutine nac_r_to_xyz(h,drdx,hr,r,x,n,ns)
    implicit none
    integer, intent(in) :: n, ns
    real*8, intent(in) :: x(n,3)
    real*8, intent(in) :: r(n*(n-1)/2)
    real*8, intent(in) :: hr(ns,ns,n*(n-1)/2)
    real*8, intent(inout) :: h(ns,ns,n,3)
    real*8, intent(inout) :: drdx(n*(n-1)/2,3*n)

    real*8 ::  hx(ns,ns,3*n)
    integer :: i,j

    drdx=0.0d0
    ! compute drdx
    drdx(1,1)=(x(1,1)-x(2,1))/r(1)
    drdx(1,2)=(x(1,2)-x(2,2))/r(1)
    drdx(1,3)=(x(1,3)-x(2,3))/r(1)
    drdx(1,4)=-drdx(1,1)
    drdx(1,5)=-drdx(1,2)
    drdx(1,6)=-drdx(1,3)
    drdx(2,1)=(x(1,1)-x(3,1))/r(2)
    drdx(2,2)=(x(1,2)-x(3,2))/r(2)
    drdx(2,3)=(x(1,3)-x(3,3))/r(2)
    drdx(2,7)=-drdx(2,1)
    drdx(2,8)=-drdx(2,2)
    drdx(2,9)=-drdx(2,3)
    drdx(3,4)=(x(2,1)-x(3,1))/r(3)
    drdx(3,5)=(x(2,2)-x(3,2))/r(3)
    drdx(3,6)=(x(2,3)-x(3,3))/r(3)
    drdx(3,7)=-drdx(3,4)
    drdx(3,8)=-drdx(3,5)
    drdx(3,9)=-drdx(3,6)

    do i=1,ns
      call mproduct(hr(i,:,:),drdx,hx(i,:,:),ns,n*(n-1)/2,3*n)
    enddo 

    do i=1,n
      h(:,:,i,1)=hx(:,:,3*(i-1)+1)
      h(:,:,i,2)=hx(:,:,3*(i-1)+2)
      h(:,:,i,3)=hx(:,:,3*(i-1)+3)
    enddo

  end subroutine nac_r_to_xyz

!==========================
! convert xyz to expr
  subroutine convert_xyz_to_expr(x,expr,r,n,a)
    implicit none
    integer, intent(in) :: n
    real*8, intent(in) :: x(n,3)
    real*8, intent(out) :: expr(n*(n-1)/2)
    real*8, intent(out) :: r(n*(n-1)/2)
    real*8 :: a
    integer :: i, j, l

    l=1
    !pairs of distances
    do i=1,n
      do j=i+1,n
        r(l)=sqrt((x(i,1)-x(j,1))**2+(x(i,2)-x(j,2))**2+(x(i,3)-x(j,3))**2)
        l=l+1
      enddo
    enddo

   do i=1,n*(n-1)/2
     expr(i)=exp(-a*r(i))
   enddo

  endsubroutine convert_xyz_to_expr

!=========================
! matrix vector product 
  subroutine mproduct(a,b,c,l,m,n)
    implicit none
    integer, intent(in) :: l, m, n
    real*8, intent(in) :: a(l,m), b(m,n)
    real*8, intent(inout) :: c(l,n)

    integer :: i, j, k
  
    do i=1,l
      do j=1,n
        c(i,j)=0.d0
        do k=1,m
          c(i,j)=c(i,j)+a(i,k)*b(k,j)
        enddo
      enddo
    enddo
    
    return
  endsubroutine mproduct

!===========================
! assign coefficients
  subroutine assign_coeff_bias(coeff_l0,coeff_l1,coeff_l2,coeff_l3,bias_l0,bias_l1,bias_l2, bias_l3)
    implicit none
    real*8, intent(inout) :: coeff_l0(3,15), coeff_l1(15,35), coeff_l2(35,55), coeff_l3(55,105)
    real*8, intent(inout) :: bias_l0(15), bias_l1(35), bias_l2(55), bias_l3(105)

!coeff layer 0
       coeff_l0(1,1)=  6.3045744896e+00
       coeff_l0(1,2)= -3.2131705284e+00
       coeff_l0(1,3)= -4.7246656418e+00
       coeff_l0(1,4)=  1.9475208521e+00
       coeff_l0(1,5)=  5.5375790596e-01
       coeff_l0(1,6)=  5.5088019371e+00
       coeff_l0(1,7)=  2.6152613163e+00
       coeff_l0(1,8)= -4.8333535194e+00
       coeff_l0(1,9)= -3.0292592049e+00
       coeff_l0(1,10)=  4.7898131609e-01
       coeff_l0(1,11)=  6.2535285950e+00
       coeff_l0(1,12)= -1.8907592297e+00
       coeff_l0(1,13)=  6.7727977037e-01
       coeff_l0(1,14)= -2.1225206852e+00
       coeff_l0(1,15)=  1.3991303444e+00
       coeff_l0(2,1)= -2.4500379562e+00
       coeff_l0(2,2)= -1.9866942167e+00
       coeff_l0(2,3)=  7.2442307472e+00
       coeff_l0(2,4)=  5.0990885496e-01
       coeff_l0(2,5)=  5.0213842392e+00
       coeff_l0(2,6)=  5.4432067871e+00
       coeff_l0(2,7)=  1.8248622417e+00
       coeff_l0(2,8)= -3.0464601517e+00
       coeff_l0(2,9)= -4.2279706001e+00
       coeff_l0(2,10)=  1.7797741890e+00
       coeff_l0(2,11)=  2.0014315844e-01
       coeff_l0(2,12)=  5.6907123327e-01
       coeff_l0(2,13)= -4.9636893272e+00
       coeff_l0(2,14)=  5.3664150238e+00
       coeff_l0(2,15)=  3.2917299867e-01
       coeff_l0(3,1)= -2.9305036068e+00
       coeff_l0(3,2)=  6.4273829460e+00
       coeff_l0(3,3)= -1.8377907276e+00
       coeff_l0(3,4)=  1.8467217684e-01
       coeff_l0(3,5)= -1.6977885962e+00
       coeff_l0(3,6)=  5.3293128014e+00
       coeff_l0(3,7)=  1.6083731651e+00
       coeff_l0(3,8)= -3.1758704185e+00
       coeff_l0(3,9)= -3.9038748741e+00
       coeff_l0(3,10)= -9.1192132235e-01
       coeff_l0(3,11)=  4.4936540723e-01
       coeff_l0(3,12)=  2.8870099783e-01
       coeff_l0(3,13)=  5.0414032936e+00
       coeff_l0(3,14)= -4.3986892700e+00
       coeff_l0(3,15)=  1.7172292471e+00
!coeff layer 1
       coeff_l1(1,1)= -1.6417930126e+00
       coeff_l1(1,2)= -1.8138907850e-01
       coeff_l1(1,3)= -3.7121429443e+00
       coeff_l1(1,4)= -3.0024697781e+00
       coeff_l1(1,5)= -6.2672591209e+00
       coeff_l1(1,6)=  1.2481608391e+00
       coeff_l1(1,7)= -1.2476080656e+00
       coeff_l1(1,8)= -4.4580392838e+00
       coeff_l1(1,9)= -2.3120341301e+00
       coeff_l1(1,10)=  4.7474908829e+00
       coeff_l1(1,11)= -1.9694708586e+00
       coeff_l1(1,12)= -1.4743909836e+00
       coeff_l1(1,13)=  4.6894807816e+00
       coeff_l1(1,14)=  1.4047118425e+00
       coeff_l1(1,15)=  2.8717222214e+00
       coeff_l1(1,16)=  9.3579739332e-01
       coeff_l1(1,17)= -4.2160401344e+00
       coeff_l1(1,18)=  1.7009706497e+00
       coeff_l1(1,19)=  6.1515674591e+00
       coeff_l1(1,20)= -3.8940107822e+00
       coeff_l1(1,21)= -4.9389484525e-01
       coeff_l1(1,22)=  1.3400697708e+00
       coeff_l1(1,23)= -2.0020010471e+00
       coeff_l1(1,24)= -3.0326719284e+00
       coeff_l1(1,25)= -6.8183956146e+00
       coeff_l1(1,26)= -1.1916877031e+00
       coeff_l1(1,27)= -8.3044904470e-01
       coeff_l1(1,28)= -6.8563920259e-01
       coeff_l1(1,29)=  2.5674400330e+00
       coeff_l1(1,30)=  9.0976363420e-01
       coeff_l1(1,31)= -2.3684313297e+00
       coeff_l1(1,32)=  1.6761972904e+00
       coeff_l1(1,33)= -5.0247278214e+00
       coeff_l1(1,34)= -2.8440356255e+00
       coeff_l1(1,35)=  1.4592424631e+00
       coeff_l1(2,1)= -1.2158544064e+00
       coeff_l1(2,2)=  2.0882861316e-01
       coeff_l1(2,3)= -2.5074207783e+00
       coeff_l1(2,4)=  3.0944318771e+00
       coeff_l1(2,5)= -4.4199757576e+00
       coeff_l1(2,6)=  1.3996996880e+00
       coeff_l1(2,7)= -6.5896981955e-01
       coeff_l1(2,8)= -3.0842068195e+00
       coeff_l1(2,9)= -3.6743516922e+00
       coeff_l1(2,10)=  3.8987720013e+00
       coeff_l1(2,11)= -1.3607544899e+00
       coeff_l1(2,12)=  2.2448267937e+00
       coeff_l1(2,13)= -2.3963572979e+00
       coeff_l1(2,14)= -1.8353623152e-01
       coeff_l1(2,15)= -1.3108998537e+00
       coeff_l1(2,16)=  7.5000697374e-01
       coeff_l1(2,17)= -4.6264033318e+00
       coeff_l1(2,18)=  1.0755264759e+00
       coeff_l1(2,19)=  5.5974025726e+00
       coeff_l1(2,20)= -1.6384941339e+00
       coeff_l1(2,21)= -1.3757466078e+00
       coeff_l1(2,22)=  1.8432391882e+00
       coeff_l1(2,23)= -6.8487352133e-01
       coeff_l1(2,24)= -2.2191202641e+00
       coeff_l1(2,25)= -5.2582936287e+00
       coeff_l1(2,26)= -2.3037347794e+00
       coeff_l1(2,27)= -1.1426918507e+00
       coeff_l1(2,28)= -1.5466880053e-02
       coeff_l1(2,29)=  1.7866402864e+00
       coeff_l1(2,30)=  6.1928957701e-01
       coeff_l1(2,31)= -2.5666089058e+00
       coeff_l1(2,32)=  1.5892686844e+00
       coeff_l1(2,33)= -3.9389901161e+00
       coeff_l1(2,34)= -5.4843950272e+00
       coeff_l1(2,35)= -1.2384823561e+00
       coeff_l1(3,1)=  8.3015397191e-02
       coeff_l1(3,2)= -3.9533281326e-01
       coeff_l1(3,3)= -1.4329876900e+00
       coeff_l1(3,4)= -3.6874430180e+00
       coeff_l1(3,5)= -2.1870992184e+00
       coeff_l1(3,6)=  8.5268121958e-01
       coeff_l1(3,7)= -4.2515569925e-01
       coeff_l1(3,8)= -1.4706699848e+00
       coeff_l1(3,9)=  1.4079754353e+00
       coeff_l1(3,10)=  2.4582550526e+00
       coeff_l1(3,11)= -6.8640804291e-01
       coeff_l1(3,12)= -4.9795236588e+00
       coeff_l1(3,13)=  8.7752246857e-01
       coeff_l1(3,14)=  1.2496007681e+00
       coeff_l1(3,15)=  5.4950267076e-01
       coeff_l1(3,16)=  4.7872298956e-01
       coeff_l1(3,17)=  1.7309184074e+00
       coeff_l1(3,18)=  5.7499754429e-01
       coeff_l1(3,19)=  3.2762355804e+00
       coeff_l1(3,20)= -6.9886261225e-01
       coeff_l1(3,21)= -4.2877754569e-01
       coeff_l1(3,22)=  1.2754051685e+00
       coeff_l1(3,23)= -2.8045053482e+00
       coeff_l1(3,24)= -1.0494490862e+00
       coeff_l1(3,25)= -3.0828871727e+00
       coeff_l1(3,26)=  1.0388653278e+00
       coeff_l1(3,27)= -7.0241975784e-01
       coeff_l1(3,28)=  1.6351723671e-01
       coeff_l1(3,29)=  1.1163411140e+00
       coeff_l1(3,30)=  3.1473442912e-01
       coeff_l1(3,31)= -1.6458674669e+00
       coeff_l1(3,32)=  8.1844931841e-01
       coeff_l1(3,33)= -2.0725057125e+00
       coeff_l1(3,34)=  2.0531375408e+00
       coeff_l1(3,35)=  1.5087538958e+00
       coeff_l1(4,1)=  3.9999994636e-01
       coeff_l1(4,2)=  5.6116116047e-01
       coeff_l1(4,3)=  6.9028630853e-02
       coeff_l1(4,4)=  4.5022137463e-02
       coeff_l1(4,5)=  3.2392975688e-01
       coeff_l1(4,6)=  7.9436618090e-01
       coeff_l1(4,7)=  6.6379606724e-01
       coeff_l1(4,8)=  5.4082065821e-01
       coeff_l1(4,9)=  1.5020921826e-01
       coeff_l1(4,10)=  1.0027113557e-01
       coeff_l1(4,11)= -2.8279566765e-01
       coeff_l1(4,12)= -3.2490089536e-01
       coeff_l1(4,13)= -6.9900788367e-02
       coeff_l1(4,14)= -8.4822833538e-02
       coeff_l1(4,15)= -3.3979550004e-01
       coeff_l1(4,16)=  1.0388300419e+00
       coeff_l1(4,17)= -7.7764451504e-01
       coeff_l1(4,18)= -1.1899537407e-02
       coeff_l1(4,19)= -2.5722676516e-01
       coeff_l1(4,20)=  2.8513064981e-01
       coeff_l1(4,21)= -7.6019310951e-01
       coeff_l1(4,22)=  3.0039608479e-01
       coeff_l1(4,23)= -6.1958426237e-01
       coeff_l1(4,24)=  1.6290019453e-01
       coeff_l1(4,25)= -7.0137590170e-01
       coeff_l1(4,26)=  2.2483910620e-01
       coeff_l1(4,27)=  5.5285763741e-01
       coeff_l1(4,28)=  4.5640131831e-01
       coeff_l1(4,29)=  5.7600528002e-02
       coeff_l1(4,30)= -2.1416690946e-01
       coeff_l1(4,31)= -6.2871053815e-02
       coeff_l1(4,32)=  3.2937577367e-01
       coeff_l1(4,33)=  9.6511888504e-01
       coeff_l1(4,34)= -5.0993591547e-01
       coeff_l1(4,35)=  3.6623257399e-01
       coeff_l1(5,1)= -1.9638674259e+00
       coeff_l1(5,2)= -2.3529880047e+00
       coeff_l1(5,3)= -1.4282318354e+00
       coeff_l1(5,4)=  3.3682019711e+00
       coeff_l1(5,5)= -2.4180855751e+00
       coeff_l1(5,6)=  6.2898039818e-01
       coeff_l1(5,7)= -1.1212199926e+00
       coeff_l1(5,8)= -1.7988411188e+00
       coeff_l1(5,9)= -4.0434479713e-02
       coeff_l1(5,10)=  2.1419053078e+00
       coeff_l1(5,11)= -8.9853823185e-01
       coeff_l1(5,12)=  6.3545459509e-01
       coeff_l1(5,13)= -2.5150048733e+00
       coeff_l1(5,14)= -1.8604395390e+00
       coeff_l1(5,15)= -1.4347918034e+00
       coeff_l1(5,16)=  1.4414520562e-01
       coeff_l1(5,17)= -3.2355856895e+00
       coeff_l1(5,18)=  8.0509364605e-01
       coeff_l1(5,19)=  2.4552195072e+00
       coeff_l1(5,20)= -1.2923893929e+00
       coeff_l1(5,21)= -2.1810069680e-01
       coeff_l1(5,22)=  7.4747550488e-01
       coeff_l1(5,23)=  4.1405615807e+00
       coeff_l1(5,24)= -1.1853249073e+00
       coeff_l1(5,25)= -3.5875070095e+00
       coeff_l1(5,26)=  1.3734493256e+00
       coeff_l1(5,27)= -7.6697027683e-01
       coeff_l1(5,28)=  8.3866304159e-01
       coeff_l1(5,29)=  9.8584890366e-01
       coeff_l1(5,30)=  6.8465435505e-01
       coeff_l1(5,31)= -1.5225173235e+00
       coeff_l1(5,32)=  6.1678135395e-01
       coeff_l1(5,33)= -2.4305522442e+00
       coeff_l1(5,34)=  2.8331189156e+00
       coeff_l1(5,35)= -2.5176267624e+00
       coeff_l1(6,1)= -1.7641850710e+00
       coeff_l1(6,2)= -2.4319560826e-01
       coeff_l1(6,3)=  6.9540214539e-01
       coeff_l1(6,4)= -2.2479507923e+00
       coeff_l1(6,5)= -7.8190249205e-01
       coeff_l1(6,6)=  1.0739141703e+00
       coeff_l1(6,7)= -9.3503785133e-01
       coeff_l1(6,8)= -7.7832192183e-01
       coeff_l1(6,9)= -2.9030555859e-02
       coeff_l1(6,10)=  3.8905784488e-02
       coeff_l1(6,11)=  1.5308281183e+00
       coeff_l1(6,12)=  3.6810651422e-01
       coeff_l1(6,13)= -5.1171839237e-01
       coeff_l1(6,14)= -1.3187792897e-01
       coeff_l1(6,15)=  4.0899085999e-01
       coeff_l1(6,16)=  9.1921269894e-02
       coeff_l1(6,17)= -1.1274996996e+00
       coeff_l1(6,18)= -3.1886196136e-01
       coeff_l1(6,19)=  6.3003772497e-01
       coeff_l1(6,20)= -1.2710481882e+00
       coeff_l1(6,21)= -8.8157522678e-01
       coeff_l1(6,22)= -1.9536594152e+00
       coeff_l1(6,23)= -3.6103305221e-01
       coeff_l1(6,24)= -1.3078242540e+00
       coeff_l1(6,25)=  1.0406098366e+01
       coeff_l1(6,26)= -4.0827441216e-01
       coeff_l1(6,27)=  1.8153955936e+00
       coeff_l1(6,28)= -2.9310019016e+00
       coeff_l1(6,29)= -4.1345863342e+00
       coeff_l1(6,30)= -8.6369931698e-01
       coeff_l1(6,31)=  1.0147805214e+00
       coeff_l1(6,32)=  2.8025954962e-01
       coeff_l1(6,33)=  2.6496448517e+00
       coeff_l1(6,34)= -6.4276373386e-01
       coeff_l1(6,35)= -1.0669437647e+00
       coeff_l1(7,1)= -1.7271916568e-01
       coeff_l1(7,2)=  1.1978139877e+00
       coeff_l1(7,3)=  1.4031457901e+00
       coeff_l1(7,4)=  1.6900017262e+00
       coeff_l1(7,5)=  2.7763879299e+00
       coeff_l1(7,6)= -1.3502488136e+00
       coeff_l1(7,7)=  8.8756304979e-01
       coeff_l1(7,8)=  1.0858507156e+00
       coeff_l1(7,9)= -1.2780029774e+00
       coeff_l1(7,10)= -1.4901055098e+00
       coeff_l1(7,11)= -6.4871549606e-01
       coeff_l1(7,12)=  4.7516453266e-01
       coeff_l1(7,13)=  1.4194490910e+00
       coeff_l1(7,14)=  3.7447646260e-01
       coeff_l1(7,15)= -1.2788264751e+00
       coeff_l1(7,16)=  9.8636627197e-01
       coeff_l1(7,17)=  1.3305828571e+00
       coeff_l1(7,18)=  3.7414111197e-02
       coeff_l1(7,19)= -2.3628740311e+00
       coeff_l1(7,20)= -8.2806086540e-01
       coeff_l1(7,21)= -1.5459185839e+00
       coeff_l1(7,22)= -1.2686291337e-01
       coeff_l1(7,23)= -8.0873626471e-01
       coeff_l1(7,24)=  4.6950069070e-01
       coeff_l1(7,25)=  3.0930742621e-01
       coeff_l1(7,26)= -1.0613787174e-01
       coeff_l1(7,27)=  1.2753497362e+00
       coeff_l1(7,28)=  5.7132158428e-02
       coeff_l1(7,29)= -6.6163653135e-01
       coeff_l1(7,30)= -1.1839141846e+00
       coeff_l1(7,31)= -1.2891467810e+00
       coeff_l1(7,32)=  1.8527208269e-01
       coeff_l1(7,33)=  1.8341124058e+00
       coeff_l1(7,34)=  1.4757460356e+00
       coeff_l1(7,35)=  2.6752660275e+00
       coeff_l1(8,1)=  7.5309127569e-01
       coeff_l1(8,2)=  1.7776770592e+00
       coeff_l1(8,3)= -1.9619971514e+00
       coeff_l1(8,4)=  2.3455591202e+00
       coeff_l1(8,5)= -4.1062531471e+00
       coeff_l1(8,6)=  7.5536228716e-02
       coeff_l1(8,7)=  1.3313988447e+00
       coeff_l1(8,8)= -5.0194664001e+00
       coeff_l1(8,9)= -3.2257413864e-01
       coeff_l1(8,10)=  1.8746111393e+00
       coeff_l1(8,11)= -3.3960804939e+00
       coeff_l1(8,12)=  1.1134948730e+00
       coeff_l1(8,13)= -6.6190576553e-01
       coeff_l1(8,14)=  2.1939001083e+00
       coeff_l1(8,15)=  7.0208114386e-01
       coeff_l1(8,16)=  6.5124893188e-01
       coeff_l1(8,17)= -8.1570506096e-02
       coeff_l1(8,18)=  1.7481118441e+00
       coeff_l1(8,19)=  2.3156976700e-01
       coeff_l1(8,20)= -3.5761053562e+00
       coeff_l1(8,21)= -2.0599557459e-01
       coeff_l1(8,22)= -3.8292119503e+00
       coeff_l1(8,23)= -1.9011299610e+00
       coeff_l1(8,24)= -2.4460506439e+00
       coeff_l1(8,25)=  1.6853353977e+00
       coeff_l1(8,26)= -1.4816889763e+00
       coeff_l1(8,27)=  2.9239399433e+00
       coeff_l1(8,28)= -1.7391749620e+00
       coeff_l1(8,29)= -8.3314228058e-01
       coeff_l1(8,30)=  9.4662201405e-01
       coeff_l1(8,31)= -1.3850277662e-01
       coeff_l1(8,32)=  1.5932190418e+00
       coeff_l1(8,33)= -1.2463526726e+00
       coeff_l1(8,34)= -8.8109141588e-01
       coeff_l1(8,35)=  1.8799948692e+00
       coeff_l1(9,1)=  2.9087986946e+00
       coeff_l1(9,2)=  2.0227720737e+00
       coeff_l1(9,3)= -1.0795698166e+00
       coeff_l1(9,4)= -8.2249182463e-01
       coeff_l1(9,5)= -3.5070819855e+00
       coeff_l1(9,6)=  5.1665240526e-01
       coeff_l1(9,7)=  2.9626369476e+00
       coeff_l1(9,8)= -4.8994593620e+00
       coeff_l1(9,9)=  1.0025544167e+00
       coeff_l1(9,10)= -2.4871489406e-01
       coeff_l1(9,11)= -3.3958218098e+00
       coeff_l1(9,12)=  6.6183799505e-01
       coeff_l1(9,13)= -5.0911414623e-01
       coeff_l1(9,14)= -1.4229048491e+00
       coeff_l1(9,15)=  4.6776959300e-01
       coeff_l1(9,16)=  1.7177382708e+00
       coeff_l1(9,17)= -4.1614326835e-01
       coeff_l1(9,18)=  4.5008245111e-01
       coeff_l1(9,19)=  6.5066504478e-01
       coeff_l1(9,20)= -1.5238281488e+00
       coeff_l1(9,21)= -1.5237200260e+00
       coeff_l1(9,22)= -3.6302275658e+00
       coeff_l1(9,23)=  1.7008285224e-01
       coeff_l1(9,24)= -3.3244953156e+00
       coeff_l1(9,25)=  6.5497846603e+00
       coeff_l1(9,26)=  1.8308690786e+00
       coeff_l1(9,27)=  2.8044750690e+00
       coeff_l1(9,28)= -9.7923403978e-01
       coeff_l1(9,29)= -3.3166280389e-01
       coeff_l1(9,30)= -1.4981783926e-01
       coeff_l1(9,31)=  8.9148384333e-01
       coeff_l1(9,32)=  2.5246503353e+00
       coeff_l1(9,33)=  4.8454952240e-01
       coeff_l1(9,34)=  1.9289619923e+00
       coeff_l1(9,35)=  3.4523141384e-01
       coeff_l1(10,1)= -2.3681056499e+00
       coeff_l1(10,2)= -8.8523042202e-01
       coeff_l1(10,3)= -1.8095904589e-01
       coeff_l1(10,4)=  1.3244903088e+00
       coeff_l1(10,5)= -2.1814705431e-01
       coeff_l1(10,6)= -7.8566509485e-01
       coeff_l1(10,7)=  1.7817388475e-01
       coeff_l1(10,8)=  5.8449631929e-01
       coeff_l1(10,9)= -6.0890156031e-01
       coeff_l1(10,10)= -1.2433809787e-01
       coeff_l1(10,11)=  4.7884306312e-01
       coeff_l1(10,12)=  8.7813287973e-01
       coeff_l1(10,13)= -1.6707813367e-02
       coeff_l1(10,14)=  9.8053142428e-02
       coeff_l1(10,15)= -5.4717582464e-01
       coeff_l1(10,16)=  2.3231905699e-01
       coeff_l1(10,17)= -7.7646327019e-01
       coeff_l1(10,18)= -3.3049044013e-01
       coeff_l1(10,19)=  1.1466637999e-01
       coeff_l1(10,20)=  3.2988023758e-01
       coeff_l1(10,21)=  1.2582479417e-01
       coeff_l1(10,22)=  5.5228513479e-01
       coeff_l1(10,23)= -3.0317211524e-02
       coeff_l1(10,24)=  4.7071909904e-01
       coeff_l1(10,25)= -1.1156038046e+00
       coeff_l1(10,26)= -3.4783151746e-01
       coeff_l1(10,27)=  2.3845151067e-01
       coeff_l1(10,28)= -2.4597968161e-01
       coeff_l1(10,29)= -2.7079585195e-01
       coeff_l1(10,30)=  1.8976221979e-01
       coeff_l1(10,31)=  6.8525403738e-02
       coeff_l1(10,32)=  5.9654717334e-03
       coeff_l1(10,33)=  4.8323866725e-01
       coeff_l1(10,34)= -4.4775760174e-01
       coeff_l1(10,35)= -4.8102357984e-01
       coeff_l1(11,1)=  2.9313206673e+00
       coeff_l1(11,2)=  5.2580660582e-01
       coeff_l1(11,3)=  2.8469911218e-01
       coeff_l1(11,4)=  9.7135329247e-01
       coeff_l1(11,5)=  4.5640957355e-01
       coeff_l1(11,6)=  4.0852418542e-01
       coeff_l1(11,7)=  1.7483595014e-01
       coeff_l1(11,8)=  4.6947741508e-01
       coeff_l1(11,9)=  7.1120955050e-02
       coeff_l1(11,10)=  3.6600330472e-01
       coeff_l1(11,11)=  5.0085824728e-01
       coeff_l1(11,12)= -1.7729048729e+00
       coeff_l1(11,13)= -3.6185185909e+00
       coeff_l1(11,14)=  9.3825332820e-02
       coeff_l1(11,15)= -1.0544917583e+00
       coeff_l1(11,16)= -3.0253639817e-01
       coeff_l1(11,17)=  1.2385343313e+00
       coeff_l1(11,18)= -2.6579612494e-01
       coeff_l1(11,19)=  9.3420433998e-01
       coeff_l1(11,20)=  1.7904871702e+00
       coeff_l1(11,21)= -5.4453366995e-01
       coeff_l1(11,22)=  9.1538834572e-01
       coeff_l1(11,23)= -3.5273861885e+00
       coeff_l1(11,24)=  3.0622348189e-01
       coeff_l1(11,25)=  1.1607085168e-01
       coeff_l1(11,26)= -2.9829746485e-01
       coeff_l1(11,27)= -9.4396156073e-01
       coeff_l1(11,28)=  8.4010004997e-01
       coeff_l1(11,29)= -1.9706374407e-01
       coeff_l1(11,30)= -6.8602949381e-02
       coeff_l1(11,31)= -7.4080878496e-01
       coeff_l1(11,32)= -8.0282062292e-02
       coeff_l1(11,33)= -1.5524978936e-01
       coeff_l1(11,34)= -1.0937530994e+00
       coeff_l1(11,35)=  9.7854471207e-01
       coeff_l1(12,1)=  3.1083559990e-01
       coeff_l1(12,2)=  2.9056501389e-01
       coeff_l1(12,3)= -2.8526398540e-01
       coeff_l1(12,4)=  7.7631807327e-01
       coeff_l1(12,5)= -3.5946708918e-01
       coeff_l1(12,6)= -6.0119253397e-01
       coeff_l1(12,7)=  3.7962302566e-01
       coeff_l1(12,8)= -1.8145053089e-01
       coeff_l1(12,9)= -3.7833857536e-01
       coeff_l1(12,10)= -5.0852388144e-01
       coeff_l1(12,11)=  3.9072251320e-01
       coeff_l1(12,12)=  1.7311055958e-01
       coeff_l1(12,13)=  1.0217328556e-02
       coeff_l1(12,14)=  1.4284551144e-01
       coeff_l1(12,15)=  4.3453518301e-02
       coeff_l1(12,16)=  1.5048852563e-01
       coeff_l1(12,17)=  1.6636110842e-01
       coeff_l1(12,18)=  5.3145343065e-01
       coeff_l1(12,19)= -7.7917402983e-01
       coeff_l1(12,20)= -9.6182191372e-01
       coeff_l1(12,21)= -1.0107173584e-02
       coeff_l1(12,22)= -1.4362731576e-01
       coeff_l1(12,23)= -1.3429899514e-01
       coeff_l1(12,24)= -1.3982757926e-01
       coeff_l1(12,25)= -1.8110018969e-01
       coeff_l1(12,26)= -3.6139190197e-01
       coeff_l1(12,27)=  2.6204491034e-02
       coeff_l1(12,28)=  3.3445963264e-01
       coeff_l1(12,29)=  2.1042372286e-01
       coeff_l1(12,30)=  8.8351807790e-04
       coeff_l1(12,31)= -9.6582718194e-02
       coeff_l1(12,32)= -4.6447348595e-01
       coeff_l1(12,33)=  7.3606365919e-01
       coeff_l1(12,34)= -1.1075649261e+00
       coeff_l1(12,35)=  8.8875687122e-01
       coeff_l1(13,1)=  3.2912850380e+00
       coeff_l1(13,2)= -1.2092338800e+00
       coeff_l1(13,3)= -1.4518616199e+00
       coeff_l1(13,4)= -4.4274690747e-01
       coeff_l1(13,5)= -2.2644855976e+00
       coeff_l1(13,6)=  3.3976846933e-01
       coeff_l1(13,7)= -6.5683060884e-01
       coeff_l1(13,8)= -1.5473686457e+00
       coeff_l1(13,9)=  2.2028150558e+00
       coeff_l1(13,10)=  1.6467400789e+00
       coeff_l1(13,11)= -6.7931652069e-01
       coeff_l1(13,12)= -4.2598518729e-01
       coeff_l1(13,13)= -2.2065508366e+00
       coeff_l1(13,14)= -1.9224437475e+00
       coeff_l1(13,15)= -1.4819662571e+00
       coeff_l1(13,16)=  2.3409926891e-01
       coeff_l1(13,17)=  8.4342300892e-01
       coeff_l1(13,18)=  6.8450069427e-01
       coeff_l1(13,19)=  1.9884240627e+00
       coeff_l1(13,20)= -1.4368506670e+00
       coeff_l1(13,21)=  2.9890173674e-01
       coeff_l1(13,22)=  3.7112435699e-01
       coeff_l1(13,23)=  3.1759812832e+00
       coeff_l1(13,24)= -1.1251168251e+00
       coeff_l1(13,25)= -2.7713422775e+00
       coeff_l1(13,26)=  1.8968262672e+00
       coeff_l1(13,27)= -2.7487707138e-01
       coeff_l1(13,28)= -2.1561533213e-01
       coeff_l1(13,29)=  9.0018022060e-01
       coeff_l1(13,30)=  3.1694889069e-01
       coeff_l1(13,31)= -5.5331420898e-01
       coeff_l1(13,32)=  5.1906341314e-01
       coeff_l1(13,33)= -1.7643738985e+00
       coeff_l1(13,34)=  4.3197107315e+00
       coeff_l1(13,35)= -3.0350306034e+00
       coeff_l1(14,1)=  6.7736828327e-01
       coeff_l1(14,2)=  1.6501142979e+00
       coeff_l1(14,3)= -9.9116277695e-01
       coeff_l1(14,4)= -8.4294700623e-01
       coeff_l1(14,5)= -1.9874925613e+00
       coeff_l1(14,6)=  3.9031493664e-01
       coeff_l1(14,7)=  1.7128069699e-01
       coeff_l1(14,8)= -1.5145504475e+00
       coeff_l1(14,9)=  3.6594215780e-02
       coeff_l1(14,10)=  1.0193620920e+00
       coeff_l1(14,11)= -6.5301829576e-01
       coeff_l1(14,12)=  3.2314410210e+00
       coeff_l1(14,13)= -3.6073825359e+00
       coeff_l1(14,14)= -2.1534540653e+00
       coeff_l1(14,15)= -2.6494097710e+00
       coeff_l1(14,16)=  2.8535786271e-01
       coeff_l1(14,17)=  2.3029897213e+00
       coeff_l1(14,18)=  5.5218279362e-01
       coeff_l1(14,19)=  1.7914460897e+00
       coeff_l1(14,20)= -1.1348191500e+00
       coeff_l1(14,21)= -5.7653361559e-01
       coeff_l1(14,22)=  1.2607091665e-01
       coeff_l1(14,23)=  9.8045158386e-01
       coeff_l1(14,24)= -1.1870756149e+00
       coeff_l1(14,25)= -1.3219369650e+00
       coeff_l1(14,26)= -1.4750016928e+00
       coeff_l1(14,27)= -1.2655742466e-01
       coeff_l1(14,28)= -9.1177237034e-01
       coeff_l1(14,29)=  7.1238774061e-01
       coeff_l1(14,30)= -1.6632913053e-01
       coeff_l1(14,31)= -1.1348646879e-01
       coeff_l1(14,32)=  5.5931097269e-01
       coeff_l1(14,33)= -1.5046651363e+00
       coeff_l1(14,34)= -2.7119135857e+00
       coeff_l1(14,35)= -3.6927549839e+00
       coeff_l1(15,1)= -3.7501758337e-01
       coeff_l1(15,2)= -2.5580491871e-02
       coeff_l1(15,3)=  7.8746855259e-01
       coeff_l1(15,4)=  4.9831366539e-01
       coeff_l1(15,5)=  1.0495427847e+00
       coeff_l1(15,6)= -8.1984877586e-01
       coeff_l1(15,7)=  4.0603202581e-01
       coeff_l1(15,8)=  6.7946720123e-01
       coeff_l1(15,9)=  8.9041277766e-02
       coeff_l1(15,10)= -4.8397111893e-01
       coeff_l1(15,11)= -1.9632670283e-01
       coeff_l1(15,12)=  2.1825811267e-01
       coeff_l1(15,13)=  1.0617693663e+00
       coeff_l1(15,14)=  3.7153595686e-01
       coeff_l1(15,15)= -5.7075393200e-01
       coeff_l1(15,16)=  2.2132328153e-01
       coeff_l1(15,17)=  5.3124135733e-01
       coeff_l1(15,18)=  1.3453130424e-01
       coeff_l1(15,19)= -1.2489367723e+00
       coeff_l1(15,20)= -1.5146661550e-02
       coeff_l1(15,21)= -5.5919635296e-01
       coeff_l1(15,22)=  3.2060718536e-01
       coeff_l1(15,23)= -4.8721283674e-01
       coeff_l1(15,24)=  6.1983418465e-01
       coeff_l1(15,25)= -4.7044754028e-01
       coeff_l1(15,26)=  2.5010925531e-01
       coeff_l1(15,27)=  2.1540956199e-01
       coeff_l1(15,28)=  3.6728322506e-01
       coeff_l1(15,29)= -1.3413453102e-01
       coeff_l1(15,30)= -5.9984433651e-01
       coeff_l1(15,31)= -2.4994051456e-01
       coeff_l1(15,32)= -2.0150291920e-01
       coeff_l1(15,33)=  5.5719792843e-01
       coeff_l1(15,34)=  3.5101532936e-01
       coeff_l1(15,35)=  1.0694693327e+00
!coeff layer 2
       coeff_l2(1,1)= -1.3055293560e+00
       coeff_l2(1,2)=  1.7152121663e-01
       coeff_l2(1,3)=  1.9477108121e-01
       coeff_l2(1,4)=  3.2668773085e-02
       coeff_l2(1,5)= -1.8564581871e+00
       coeff_l2(1,6)=  2.7632924914e-01
       coeff_l2(1,7)= -7.1499079466e-01
       coeff_l2(1,8)= -2.3264052868e+00
       coeff_l2(1,9)=  5.9891277552e-01
       coeff_l2(1,10)= -1.9219980240e+00
       coeff_l2(1,11)=  1.1144275665e+00
       coeff_l2(1,12)=  1.8755848706e-01
       coeff_l2(1,13)=  6.0083699226e-01
       coeff_l2(1,14)= -1.8114659935e-02
       coeff_l2(1,15)=  7.7707123756e-01
       coeff_l2(1,16)= -1.0637049675e+00
       coeff_l2(1,17)= -1.1487433910e+00
       coeff_l2(1,18)= -1.4000347257e-01
       coeff_l2(1,19)= -3.1725382805e-01
       coeff_l2(1,20)=  1.1017528772e+00
       coeff_l2(1,21)=  1.2037697434e-01
       coeff_l2(1,22)= -9.7831152380e-02
       coeff_l2(1,23)= -5.3531712294e-01
       coeff_l2(1,24)= -1.3519314528e+00
       coeff_l2(1,25)=  1.5105372667e-01
       coeff_l2(1,26)= -2.2088063881e-02
       coeff_l2(1,27)= -4.3698647618e-01
       coeff_l2(1,28)=  2.7622941136e-01
       coeff_l2(1,29)=  1.0389373638e-02
       coeff_l2(1,30)= -6.1102640629e-01
       coeff_l2(1,31)=  3.1066272259e+00
       coeff_l2(1,32)=  6.2938445807e-01
       coeff_l2(1,33)=  2.1040175855e-01
       coeff_l2(1,34)=  1.1285856366e-01
       coeff_l2(1,35)=  6.4189630747e-01
       coeff_l2(1,36)= -1.6487498283e+00
       coeff_l2(1,37)=  1.7915676534e-01
       coeff_l2(1,38)= -5.4092466831e-01
       coeff_l2(1,39)=  8.0518178642e-02
       coeff_l2(1,40)=  2.7512970567e-01
       coeff_l2(1,41)=  1.6077829599e+00
       coeff_l2(1,42)=  4.1740443558e-02
       coeff_l2(1,43)= -2.9224118590e-01
       coeff_l2(1,44)= -8.2618755102e-01
       coeff_l2(1,45)= -2.6637724042e-01
       coeff_l2(1,46)=  1.2815015018e-01
       coeff_l2(1,47)= -1.3631074689e-02
       coeff_l2(1,48)= -3.0744677782e-01
       coeff_l2(1,49)=  1.3708345592e-01
       coeff_l2(1,50)=  2.8081005812e-01
       coeff_l2(1,51)= -1.0103721172e-01
       coeff_l2(1,52)= -1.0780997276e+00
       coeff_l2(1,53)=  3.3771976829e-01
       coeff_l2(1,54)=  3.4767735004e-01
       coeff_l2(1,55)=  1.0270317644e-01
       coeff_l2(2,1)= -1.0533999205e+00
       coeff_l2(2,2)= -1.3000502586e+00
       coeff_l2(2,3)=  1.2576116323e+00
       coeff_l2(2,4)= -4.2052158713e-01
       coeff_l2(2,5)= -2.4970669746e+00
       coeff_l2(2,6)=  1.9230535254e-02
       coeff_l2(2,7)= -2.1719098091e+00
       coeff_l2(2,8)= -2.2255582809e+00
       coeff_l2(2,9)= -2.0593382418e-01
       coeff_l2(2,10)= -2.0404503345e+00
       coeff_l2(2,11)=  1.8979613781e+00
       coeff_l2(2,12)= -3.0905067921e-01
       coeff_l2(2,13)=  1.8699901178e-02
       coeff_l2(2,14)= -7.3790811002e-02
       coeff_l2(2,15)=  1.4973682165e+00
       coeff_l2(2,16)= -9.8221802711e-01
       coeff_l2(2,17)=  6.2008923292e-01
       coeff_l2(2,18)=  3.1069014221e-02
       coeff_l2(2,19)= -2.0440051556e+00
       coeff_l2(2,20)= -1.6210479736e+00
       coeff_l2(2,21)=  1.2830577791e-01
       coeff_l2(2,22)=  1.6709203720e+00
       coeff_l2(2,23)= -9.8884826899e-01
       coeff_l2(2,24)=  6.5880376101e-01
       coeff_l2(2,25)=  3.5965371132e-01
       coeff_l2(2,26)=  4.4438400865e-01
       coeff_l2(2,27)=  7.6880323887e-01
       coeff_l2(2,28)=  4.9895887375e+00
       coeff_l2(2,29)=  6.6186296940e-01
       coeff_l2(2,30)= -1.3499343395e+00
       coeff_l2(2,31)=  2.8897616863e+00
       coeff_l2(2,32)= -4.9907633662e-01
       coeff_l2(2,33)= -1.5355925560e+00
       coeff_l2(2,34)=  8.9363195002e-02
       coeff_l2(2,35)=  3.4043103456e-01
       coeff_l2(2,36)=  2.4993214607e+00
       coeff_l2(2,37)= -7.0034193993e-01
       coeff_l2(2,38)=  4.8796482384e-02
       coeff_l2(2,39)=  1.6241753101e+00
       coeff_l2(2,40)= -5.6147027761e-02
       coeff_l2(2,41)=  4.4863849878e-01
       coeff_l2(2,42)=  6.0473752022e-01
       coeff_l2(2,43)= -1.9740918875e+00
       coeff_l2(2,44)=  2.0679226518e-01
       coeff_l2(2,45)=  9.6480775392e-04
       coeff_l2(2,46)= -7.7502495050e-01
       coeff_l2(2,47)=  9.1444975138e-01
       coeff_l2(2,48)= -2.9210287333e-01
       coeff_l2(2,49)= -4.0216130018e-01
       coeff_l2(2,50)= -1.1328486204e+00
       coeff_l2(2,51)= -6.9550085068e-01
       coeff_l2(2,52)= -7.7893370390e-01
       coeff_l2(2,53)=  8.6488771439e-01
       coeff_l2(2,54)=  6.2276577950e-01
       coeff_l2(2,55)= -7.9048208892e-02
       coeff_l2(3,1)=  3.5703389645e+00
       coeff_l2(3,2)= -3.0530631542e+00
       coeff_l2(3,3)= -1.8272112608e+00
       coeff_l2(3,4)= -3.8377168775e-01
       coeff_l2(3,5)=  1.8380824327e+00
       coeff_l2(3,6)=  4.6125018597e-01
       coeff_l2(3,7)= -1.4694001675e+00
       coeff_l2(3,8)=  2.7728085518e+00
       coeff_l2(3,9)=  1.0820631981e+00
       coeff_l2(3,10)= -3.5007378459e-01
       coeff_l2(3,11)= -6.7363893986e-01
       coeff_l2(3,12)= -6.3813887537e-02
       coeff_l2(3,13)=  1.2165082693e+00
       coeff_l2(3,14)=  1.0027583838e+00
       coeff_l2(3,15)= -5.9044373035e-01
       coeff_l2(3,16)=  5.8803087473e-01
       coeff_l2(3,17)= -2.7258772850e+00
       coeff_l2(3,18)=  1.3113567829e+00
       coeff_l2(3,19)=  1.0058376789e+00
       coeff_l2(3,20)=  1.7643687725e+00
       coeff_l2(3,21)=  6.2815979123e-02
       coeff_l2(3,22)= -1.0523635149e+00
       coeff_l2(3,23)=  1.9932644069e-01
       coeff_l2(3,24)=  1.3981904984e+00
       coeff_l2(3,25)=  7.7178430557e-01
       coeff_l2(3,26)=  2.6019315720e+00
       coeff_l2(3,27)=  1.1619303226e+00
       coeff_l2(3,28)= -9.8174875975e-01
       coeff_l2(3,29)=  8.5735595226e-01
       coeff_l2(3,30)=  3.2933181524e-01
       coeff_l2(3,31)= -1.6962044239e+00
       coeff_l2(3,32)=  1.7066419125e-02
       coeff_l2(3,33)=  8.7114894390e-01
       coeff_l2(3,34)= -5.3953391314e-01
       coeff_l2(3,35)=  4.1351759434e-01
       coeff_l2(3,36)= -6.0227470398e+00
       coeff_l2(3,37)=  2.5305938721e+00
       coeff_l2(3,38)= -3.1850636005e-01
       coeff_l2(3,39)= -1.6892762482e-01
       coeff_l2(3,40)= -7.7363616228e-01
       coeff_l2(3,41)= -1.6181389093e+00
       coeff_l2(3,42)=  5.1599819213e-02
       coeff_l2(3,43)= -2.2988216877e+00
       coeff_l2(3,44)=  1.6119744778e+00
       coeff_l2(3,45)= -9.2986929417e-01
       coeff_l2(3,46)=  2.3560450971e-01
       coeff_l2(3,47)= -1.0528055429e+00
       coeff_l2(3,48)=  5.0997346640e-01
       coeff_l2(3,49)=  2.7070459723e-01
       coeff_l2(3,50)=  4.3110480905e-01
       coeff_l2(3,51)=  2.8427543640e+00
       coeff_l2(3,52)=  1.5173496306e-01
       coeff_l2(3,53)=  1.4008717537e+00
       coeff_l2(3,54)=  4.8848918080e-01
       coeff_l2(3,55)=  6.0033750534e-01
       coeff_l2(4,1)=  1.3152912855e+00
       coeff_l2(4,2)= -3.5606892109e+00
       coeff_l2(4,3)=  1.4488849640e+00
       coeff_l2(4,4)=  1.0827097297e-01
       coeff_l2(4,5)=  5.7116594315e+00
       coeff_l2(4,6)= -5.1826602221e-01
       coeff_l2(4,7)= -3.5621845722e-01
       coeff_l2(4,8)=  5.0219717026e+00
       coeff_l2(4,9)= -2.2876765728e+00
       coeff_l2(4,10)=  3.7859265804e+00
       coeff_l2(4,11)= -1.1001019478e+00
       coeff_l2(4,12)= -1.9111655951e+00
       coeff_l2(4,13)= -2.1456670761e+00
       coeff_l2(4,14)= -1.1169583797e+00
       coeff_l2(4,15)= -1.5116473436e+00
       coeff_l2(4,16)=  2.0755033493e+00
       coeff_l2(4,17)=  6.3232636452e+00
       coeff_l2(4,18)= -1.2612442672e-01
       coeff_l2(4,19)= -4.5115745068e-01
       coeff_l2(4,20)=  1.3032034636e+00
       coeff_l2(4,21)= -7.0232737064e-01
       coeff_l2(4,22)=  1.6837078333e+00
       coeff_l2(4,23)=  1.2931296825e+00
       coeff_l2(4,24)=  7.6938378811e-01
       coeff_l2(4,25)=  2.4725134373e+00
       coeff_l2(4,26)=  9.6556651592e-01
       coeff_l2(4,27)=  9.2430007458e-01
       coeff_l2(4,28)=  1.2926276922e+00
       coeff_l2(4,29)= -3.6293226480e-01
       coeff_l2(4,30)=  3.8429006934e-01
       coeff_l2(4,31)=  1.8047382832e+00
       coeff_l2(4,32)= -2.3214743137e+00
       coeff_l2(4,33)= -2.3918340206e+00
       coeff_l2(4,34)= -2.4177154899e-01
       coeff_l2(4,35)= -1.5607303381e+00
       coeff_l2(4,36)=  5.3842145205e-01
       coeff_l2(4,37)= -3.0195522308e+00
       coeff_l2(4,38)=  1.8058900833e+00
       coeff_l2(4,39)=  4.6924769878e-01
       coeff_l2(4,40)= -1.3423513174e+00
       coeff_l2(4,41)= -3.2008199692e+00
       coeff_l2(4,42)=  1.1193137169e+00
       coeff_l2(4,43)=  1.5471960306e+00
       coeff_l2(4,44)= -1.6216383874e-01
       coeff_l2(4,45)= -3.3424653113e-02
       coeff_l2(4,46)=  4.8468059301e-01
       coeff_l2(4,47)=  9.0497899055e-01
       coeff_l2(4,48)=  7.4964547157e-01
       coeff_l2(4,49)= -4.5162916183e-01
       coeff_l2(4,50)= -7.4441170692e-01
       coeff_l2(4,51)= -1.0285557508e+00
       coeff_l2(4,52)=  2.1004106998e+00
       coeff_l2(4,53)= -1.4504542351e+00
       coeff_l2(4,54)= -9.6689474583e-01
       coeff_l2(4,55)=  6.7537087202e-01
       coeff_l2(5,1)=  4.6188947558e-01
       coeff_l2(5,2)=  3.6234161854e+00
       coeff_l2(5,3)= -2.1493575573e+00
       coeff_l2(5,4)= -1.3287843466e+00
       coeff_l2(5,5)=  2.2188677788e+00
       coeff_l2(5,6)=  4.8736041784e-01
       coeff_l2(5,7)=  3.7018213272e+00
       coeff_l2(5,8)= -8.6233705282e-01
       coeff_l2(5,9)= -1.2445766926e+00
       coeff_l2(5,10)= -3.8781988621e+00
       coeff_l2(5,11)= -7.9655879736e-01
       coeff_l2(5,12)= -1.5713902712e+00
       coeff_l2(5,13)=  9.5093941689e-01
       coeff_l2(5,14)=  8.1999039650e-01
       coeff_l2(5,15)=  6.1489498615e-01
       coeff_l2(5,16)=  8.4121435881e-01
       coeff_l2(5,17)= -1.3777536154e+00
       coeff_l2(5,18)=  4.5901983976e-01
       coeff_l2(5,19)= -1.0970380306e+00
       coeff_l2(5,20)= -6.4946789742e+00
       coeff_l2(5,21)=  8.5556197166e-01
       coeff_l2(5,22)=  3.6065013409e+00
       coeff_l2(5,23)=  7.9981869459e-01
       coeff_l2(5,24)=  3.1669170856e+00
       coeff_l2(5,25)= -1.1096030474e+00
       coeff_l2(5,26)= -4.0252356529e+00
       coeff_l2(5,27)=  5.8733522892e-01
       coeff_l2(5,28)=  9.9168004990e+00
       coeff_l2(5,29)= -5.9058678150e-01
       coeff_l2(5,30)=  1.0720791668e-01
       coeff_l2(5,31)= -3.6461228132e-01
       coeff_l2(5,32)=  2.2417850494e+00
       coeff_l2(5,33)=  1.7247486115e+00
       coeff_l2(5,34)= -1.8755700588e+00
       coeff_l2(5,35)= -3.7224695086e-01
       coeff_l2(5,36)=  1.2733806610e+01
       coeff_l2(5,37)=  3.9377253056e+00
       coeff_l2(5,38)= -1.1594021320e-01
       coeff_l2(5,39)= -6.2995124608e-03
       coeff_l2(5,40)= -3.5867762566e-01
       coeff_l2(5,41)=  5.7061929256e-02
       coeff_l2(5,42)=  1.0241688490e+00
       coeff_l2(5,43)= -2.9962136745e+00
       coeff_l2(5,44)= -1.6095483303e+00
       coeff_l2(5,45)= -3.6250135303e-01
       coeff_l2(5,46)= -1.7084530592e+00
       coeff_l2(5,47)= -8.6510604620e-01
       coeff_l2(5,48)= -2.5751796784e-04
       coeff_l2(5,49)=  7.6395493746e-01
       coeff_l2(5,50)= -2.3774130344e+00
       coeff_l2(5,51)=  1.2887971401e+00
       coeff_l2(5,52)=  4.1375617981e+00
       coeff_l2(5,53)= -1.3753645420e+00
       coeff_l2(5,54)= -3.2353109121e-01
       coeff_l2(5,55)= -2.9252278805e-01
       coeff_l2(6,1)= -1.8088203669e+00
       coeff_l2(6,2)=  3.5561576486e-01
       coeff_l2(6,3)=  2.2480739653e-01
       coeff_l2(6,4)= -5.2825623751e-01
       coeff_l2(6,5)= -2.0948224068e+00
       coeff_l2(6,6)= -3.3343076706e-01
       coeff_l2(6,7)= -1.1998183727e+00
       coeff_l2(6,8)= -2.1343463659e-01
       coeff_l2(6,9)=  1.5274837017e+00
       coeff_l2(6,10)= -8.0605489016e-01
       coeff_l2(6,11)=  3.0790588856e+00
       coeff_l2(6,12)=  3.3682852983e-01
       coeff_l2(6,13)= -8.0367654562e-01
       coeff_l2(6,14)= -1.9967260957e-01
       coeff_l2(6,15)=  4.0587821603e-01
       coeff_l2(6,16)=  4.2234584689e-01
       coeff_l2(6,17)=  1.1436800957e+00
       coeff_l2(6,18)= -3.3986341953e-01
       coeff_l2(6,19)=  1.2446129322e+00
       coeff_l2(6,20)= -3.3375840187e+00
       coeff_l2(6,21)= -9.2288237810e-01
       coeff_l2(6,22)= -1.5849242210e+00
       coeff_l2(6,23)= -3.2931759953e-01
       coeff_l2(6,24)=  1.5402944386e-01
       coeff_l2(6,25)=  2.5997130200e-02
       coeff_l2(6,26)= -8.6144071817e-01
       coeff_l2(6,27)= -2.5847449899e-01
       coeff_l2(6,28)= -2.1256997585e+00
       coeff_l2(6,29)= -1.6221922636e+00
       coeff_l2(6,30)= -1.0251474380e+00
       coeff_l2(6,31)= -1.5850702524e+00
       coeff_l2(6,32)= -1.6275774240e+00
       coeff_l2(6,33)= -4.3910011649e-01
       coeff_l2(6,34)=  4.3181326985e-01
       coeff_l2(6,35)= -2.3201286793e-02
       coeff_l2(6,36)=  2.9835426807e-01
       coeff_l2(6,37)=  5.8513224125e-01
       coeff_l2(6,38)= -5.3557395935e-01
       coeff_l2(6,39)= -5.2893525362e-01
       coeff_l2(6,40)= -6.2035590410e-01
       coeff_l2(6,41)= -1.5831674337e+00
       coeff_l2(6,42)= -9.0144282579e-01
       coeff_l2(6,43)= -7.7932369709e-01
       coeff_l2(6,44)= -8.5745710135e-01
       coeff_l2(6,45)= -3.7211781740e-01
       coeff_l2(6,46)= -1.0370879173e+00
       coeff_l2(6,47)=  5.5714333057e-01
       coeff_l2(6,48)= -6.8458586931e-01
       coeff_l2(6,49)=  5.4641246796e-01
       coeff_l2(6,50)= -7.5040102005e-02
       coeff_l2(6,51)= -5.9277307987e-01
       coeff_l2(6,52)=  8.8809305429e-01
       coeff_l2(6,53)=  9.8376446962e-01
       coeff_l2(6,54)= -2.1099255979e-01
       coeff_l2(6,55)= -6.7186594009e-01
       coeff_l2(7,1)=  3.1213143468e-01
       coeff_l2(7,2)=  3.0185120106e+00
       coeff_l2(7,3)=  3.9806966782e+00
       coeff_l2(7,4)= -7.3080295324e-01
       coeff_l2(7,5)= -2.1579292417e-01
       coeff_l2(7,6)=  1.5591592789e+00
       coeff_l2(7,7)=  3.3415267467e+00
       coeff_l2(7,8)= -1.8374431133e+00
       coeff_l2(7,9)=  1.4579064846e+00
       coeff_l2(7,10)= -2.5953009129e+00
       coeff_l2(7,11)=  1.3394266716e-04
       coeff_l2(7,12)=  6.8599022925e-02
       coeff_l2(7,13)=  2.4208989143e+00
       coeff_l2(7,14)=  2.4792525768e+00
       coeff_l2(7,15)= -3.6175161600e-01
       coeff_l2(7,16)= -1.0056197643e+00
       coeff_l2(7,17)= -3.9881223440e-01
       coeff_l2(7,18)=  4.9380531907e-01
       coeff_l2(7,19)= -2.0468976498e+00
       coeff_l2(7,20)= -8.7169237137e+00
       coeff_l2(7,21)=  1.8804928288e-02
       coeff_l2(7,22)=  1.1299599409e+00
       coeff_l2(7,23)=  2.8724727035e-01
       coeff_l2(7,24)=  2.2030630112e+00
       coeff_l2(7,25)= -3.6857109070e+00
       coeff_l2(7,26)= -2.8344173431e+00
       coeff_l2(7,27)=  5.4873436689e-01
       coeff_l2(7,28)= -1.8541929722e+00
       coeff_l2(7,29)= -2.6223635674e-01
       coeff_l2(7,30)=  1.6785377264e+00
       coeff_l2(7,31)=  1.1671854258e+00
       coeff_l2(7,32)=  1.9175544977e+00
       coeff_l2(7,33)= -1.0641915798e+00
       coeff_l2(7,34)= -2.4978942871e+00
       coeff_l2(7,35)=  3.5855716467e-01
       coeff_l2(7,36)=  1.0298217773e+01
       coeff_l2(7,37)=  4.2560825348e+00
       coeff_l2(7,38)= -1.0189220905e+00
       coeff_l2(7,39)=  6.7326802015e-01
       coeff_l2(7,40)= -1.0876571387e-01
       coeff_l2(7,41)=  2.2286503315e+00
       coeff_l2(7,42)= -4.2851328850e-02
       coeff_l2(7,43)= -1.1788564920e+00
       coeff_l2(7,44)=  7.2146797180e-01
       coeff_l2(7,45)= -5.9590131044e-01
       coeff_l2(7,46)= -1.6876767874e+00
       coeff_l2(7,47)= -2.4237966537e+00
       coeff_l2(7,48)=  5.0521051884e-01
       coeff_l2(7,49)=  3.7045166492e+00
       coeff_l2(7,50)= -1.4168561697e+00
       coeff_l2(7,51)=  8.3514821529e-01
       coeff_l2(7,52)= -7.8541867435e-02
       coeff_l2(7,53)=  9.7960376740e-01
       coeff_l2(7,54)=  2.2966051102e-01
       coeff_l2(7,55)= -3.7343951408e-03
       coeff_l2(8,1)= -5.7545624673e-02
       coeff_l2(8,2)=  3.0630176067e+00
       coeff_l2(8,3)=  4.2937488556e+00
       coeff_l2(8,4)= -2.4368982017e-01
       coeff_l2(8,5)=  2.7725484967e-01
       coeff_l2(8,6)= -7.2098207474e-01
       coeff_l2(8,7)=  1.6204142570e+00
       coeff_l2(8,8)=  6.1201411486e-01
       coeff_l2(8,9)=  6.9489216805e-01
       coeff_l2(8,10)=  2.7083094120e+00
       coeff_l2(8,11)=  1.7433401942e-01
       coeff_l2(8,12)=  1.6286858916e-01
       coeff_l2(8,13)= -4.4455602765e-01
       coeff_l2(8,14)=  4.8689445853e-01
       coeff_l2(8,15)= -1.2607746124e+00
       coeff_l2(8,16)=  1.1413478851e+00
       coeff_l2(8,17)=  5.6359186172e+00
       coeff_l2(8,18)=  8.4644985199e-01
       coeff_l2(8,19)=  1.1517288536e-01
       coeff_l2(8,20)= -8.1754837036e+00
       coeff_l2(8,21)=  8.9761078358e-01
       coeff_l2(8,22)= -2.4645495415e-01
       coeff_l2(8,23)=  1.6576850414e-01
       coeff_l2(8,24)=  2.9993460178e+00
       coeff_l2(8,25)= -3.8077917099e+00
       coeff_l2(8,26)=  1.1234271526e+00
       coeff_l2(8,27)= -9.3993461132e-01
       coeff_l2(8,28)= -1.2627117157e+01
       coeff_l2(8,29)= -2.3675035685e-02
       coeff_l2(8,30)=  6.2086541206e-02
       coeff_l2(8,31)= -2.2326176167e+00
       coeff_l2(8,32)= -6.0603988171e-01
       coeff_l2(8,33)= -6.3548582792e-01
       coeff_l2(8,34)= -1.1190035343e+00
       coeff_l2(8,35)= -9.6119350195e-01
       coeff_l2(8,36)=  6.8883080482e+00
       coeff_l2(8,37)= -1.4350064993e+00
       coeff_l2(8,38)= -6.1828710139e-02
       coeff_l2(8,39)=  1.7732122540e-01
       coeff_l2(8,40)= -2.6840457693e-02
       coeff_l2(8,41)= -1.2646725178e+00
       coeff_l2(8,42)=  2.5820699334e-01
       coeff_l2(8,43)=  2.3688535690e+00
       coeff_l2(8,44)=  1.4437425137e-01
       coeff_l2(8,45)= -1.0268752575e+00
       coeff_l2(8,46)= -9.3489301205e-01
       coeff_l2(8,47)= -7.9395860434e-01
       coeff_l2(8,48)=  1.5717043877e+00
       coeff_l2(8,49)=  4.0089425445e-01
       coeff_l2(8,50)=  1.1626213789e+00
       coeff_l2(8,51)= -2.8682301044e+00
       coeff_l2(8,52)=  1.0713729858e+00
       coeff_l2(8,53)=  9.6989095211e-01
       coeff_l2(8,54)= -4.7415605187e-01
       coeff_l2(8,55)= -4.8970971256e-02
       coeff_l2(9,1)= -5.6245654821e-02
       coeff_l2(9,2)=  1.8071050942e-01
       coeff_l2(9,3)=  8.2510441542e-01
       coeff_l2(9,4)= -8.2063442469e-01
       coeff_l2(9,5)=  7.5169777870e-01
       coeff_l2(9,6)= -1.0441417694e+00
       coeff_l2(9,7)=  1.3122899532e+00
       coeff_l2(9,8)=  1.9652582407e+00
       coeff_l2(9,9)= -1.6735147238e+00
       coeff_l2(9,10)=  1.8538233042e+00
       coeff_l2(9,11)= -8.9905631542e-01
       coeff_l2(9,12)= -7.6304608583e-01
       coeff_l2(9,13)= -1.4701526165e+00
       coeff_l2(9,14)= -7.8338474035e-01
       coeff_l2(9,15)= -9.6518999338e-01
       coeff_l2(9,16)=  6.0291153193e-01
       coeff_l2(9,17)=  3.1201918125e+00
       coeff_l2(9,18)= -7.9953479767e-01
       coeff_l2(9,19)= -8.4890192747e-01
       coeff_l2(9,20)= -3.2660391331e+00
       coeff_l2(9,21)=  2.4015693367e-01
       coeff_l2(9,22)=  5.7687705755e-01
       coeff_l2(9,23)=  8.2997865975e-02
       coeff_l2(9,24)= -3.9400327951e-02
       coeff_l2(9,25)= -2.9327896237e-01
       coeff_l2(9,26)=  1.1303440332e+00
       coeff_l2(9,27)=  7.0424616337e-01
       coeff_l2(9,28)= -2.0313627720e+00
       coeff_l2(9,29)= -4.6182295680e-01
       coeff_l2(9,30)=  5.8631485701e-01
       coeff_l2(9,31)=  8.9744967222e-01
       coeff_l2(9,32)= -1.3763284683e+00
       coeff_l2(9,33)= -1.7086045742e+00
       coeff_l2(9,34)= -5.0580948591e-01
       coeff_l2(9,35)= -1.4116287231e+00
       coeff_l2(9,36)=  4.7863235474e+00
       coeff_l2(9,37)= -2.6171917915e+00
       coeff_l2(9,38)=  8.5238820314e-01
       coeff_l2(9,39)=  1.0328005254e-01
       coeff_l2(9,40)= -6.4633947611e-01
       coeff_l2(9,41)= -1.8769329786e+00
       coeff_l2(9,42)=  9.1499149799e-01
       coeff_l2(9,43)=  1.0819799900e+00
       coeff_l2(9,44)=  4.8401159048e-01
       coeff_l2(9,45)=  7.8237779438e-02
       coeff_l2(9,46)= -2.1918505430e-01
       coeff_l2(9,47)= -7.8415378928e-02
       coeff_l2(9,48)=  3.9724078774e-01
       coeff_l2(9,49)= -1.6795513034e-01
       coeff_l2(9,50)= -5.8755427599e-01
       coeff_l2(9,51)= -8.2540817559e-02
       coeff_l2(9,52)=  1.2175278664e+00
       coeff_l2(9,53)= -1.3909251690e+00
       coeff_l2(9,54)= -9.2358492315e-02
       coeff_l2(9,55)=  1.6957849264e-01
       coeff_l2(10,1)= -1.1454445124e+00
       coeff_l2(10,2)=  9.7680442035e-02
       coeff_l2(10,3)=  1.4335985184e+00
       coeff_l2(10,4)=  1.9591969252e-01
       coeff_l2(10,5)= -4.9930324554e+00
       coeff_l2(10,6)=  4.0488731861e-01
       coeff_l2(10,7)= -1.2217476368e+00
       coeff_l2(10,8)= -1.3882037401e+00
       coeff_l2(10,9)=  2.1920688450e-01
       coeff_l2(10,10)= -1.0193855762e+00
       coeff_l2(10,11)=  8.3181160688e-01
       coeff_l2(10,12)=  2.7192455530e-01
       coeff_l2(10,13)=  1.2065710872e-01
       coeff_l2(10,14)= -3.8724699616e-01
       coeff_l2(10,15)=  2.9169800878e-01
       coeff_l2(10,16)= -9.7966462374e-01
       coeff_l2(10,17)= -1.9761466980e+00
       coeff_l2(10,18)= -1.0812963247e+00
       coeff_l2(10,19)= -1.1103498936e+00
       coeff_l2(10,20)= -1.4010156393e+00
       coeff_l2(10,21)= -7.0152655244e-02
       coeff_l2(10,22)=  4.9576002359e-01
       coeff_l2(10,23)= -1.3075938225e+00
       coeff_l2(10,24)= -1.7388724089e+00
       coeff_l2(10,25)=  9.1161176562e-02
       coeff_l2(10,26)= -7.7377188206e-01
       coeff_l2(10,27)=  3.9800587296e-01
       coeff_l2(10,28)=  2.7594492435e+00
       coeff_l2(10,29)= -5.2927533397e-04
       coeff_l2(10,30)= -5.1365178823e-01
       coeff_l2(10,31)= -1.5551795959e+00
       coeff_l2(10,32)= -1.0247372389e+00
       coeff_l2(10,33)= -3.2702094316e-01
       coeff_l2(10,34)= -4.1636728565e-04
       coeff_l2(10,35)=  1.2486332655e+00
       coeff_l2(10,36)=  1.0141124725e+00
       coeff_l2(10,37)= -1.8519364595e+00
       coeff_l2(10,38)= -1.9250521064e-01
       coeff_l2(10,39)= -9.2518843710e-02
       coeff_l2(10,40)=  2.4297642708e-01
       coeff_l2(10,41)=  7.1074455976e-01
       coeff_l2(10,42)= -1.2477722168e+00
       coeff_l2(10,43)= -2.2145209312e+00
       coeff_l2(10,44)= -1.4171870947e+00
       coeff_l2(10,45)=  1.8422195315e-01
       coeff_l2(10,46)=  5.2521044016e-01
       coeff_l2(10,47)= -8.1412658095e-02
       coeff_l2(10,48)= -2.9654523730e-01
       coeff_l2(10,49)=  2.2363595665e-01
       coeff_l2(10,50)= -2.8332123160e-01
       coeff_l2(10,51)= -1.9581571221e-01
       coeff_l2(10,52)= -1.2522027493e+00
       coeff_l2(10,53)= -7.5688117743e-01
       coeff_l2(10,54)= -4.1830748320e-01
       coeff_l2(10,55)= -5.5878174305e-01
       coeff_l2(11,1)= -5.0000828505e-01
       coeff_l2(11,2)= -7.4138933420e-01
       coeff_l2(11,3)= -2.8548580408e-01
       coeff_l2(11,4)=  7.7261042595e-01
       coeff_l2(11,5)= -3.5486559868e+00
       coeff_l2(11,6)=  1.3423307240e-01
       coeff_l2(11,7)= -5.1597714424e-01
       coeff_l2(11,8)=  2.2907583714e+00
       coeff_l2(11,9)= -1.1716811657e+00
       coeff_l2(11,10)=  3.5963256359e+00
       coeff_l2(11,11)=  1.5525975227e+00
       coeff_l2(11,12)= -4.8939707875e-01
       coeff_l2(11,13)= -1.4307309389e+00
       coeff_l2(11,14)= -7.0591127872e-01
       coeff_l2(11,15)=  1.2675601244e+00
       coeff_l2(11,16)= -2.5648891460e-03
       coeff_l2(11,17)= -6.9149112701e-01
       coeff_l2(11,18)= -6.1906725168e-01
       coeff_l2(11,19)=  1.7321589589e-01
       coeff_l2(11,20)=  5.7243871689e+00
       coeff_l2(11,21)= -1.2954343557e+00
       coeff_l2(11,22)=  2.8621023893e-01
       coeff_l2(11,23)= -4.3221977353e-01
       coeff_l2(11,24)= -3.7288093567e+00
       coeff_l2(11,25)=  3.0544526577e+00
       coeff_l2(11,26)=  1.3763632774e+00
       coeff_l2(11,27)=  2.7941435575e-01
       coeff_l2(11,28)= -4.0470929146e+00
       coeff_l2(11,29)= -3.7304431200e-01
       coeff_l2(11,30)=  8.1054222584e-01
       coeff_l2(11,31)= -3.3799054623e+00
       coeff_l2(11,32)= -1.6756089926e+00
       coeff_l2(11,33)=  3.4303414822e-01
       coeff_l2(11,34)=  4.1885587573e-01
       coeff_l2(11,35)=  5.7818084955e-01
       coeff_l2(11,36)= -7.9032344818e+00
       coeff_l2(11,37)= -1.5684430301e-01
       coeff_l2(11,38)=  2.7544671297e-01
       coeff_l2(11,39)= -9.9999672174e-01
       coeff_l2(11,40)= -8.1666111946e-02
       coeff_l2(11,41)= -1.5320193768e+00
       coeff_l2(11,42)= -1.4502891898e-01
       coeff_l2(11,43)=  4.4469082355e-01
       coeff_l2(11,44)=  2.0905096531e+00
       coeff_l2(11,45)=  3.2020187378e-01
       coeff_l2(11,46)=  1.7124476433e+00
       coeff_l2(11,47)=  3.5760724545e-01
       coeff_l2(11,48)= -1.6383090615e-01
       coeff_l2(11,49)= -1.0628498793e+00
       coeff_l2(11,50)=  2.1890832484e-01
       coeff_l2(11,51)= -6.6215030849e-02
       coeff_l2(11,52)= -4.5214438438e+00
       coeff_l2(11,53)=  3.0910682678e-01
       coeff_l2(11,54)= -3.0454343557e-01
       coeff_l2(11,55)=  4.8563495278e-02
       coeff_l2(12,1)=  3.2205698490e+00
       coeff_l2(12,2)= -3.2995019108e-02
       coeff_l2(12,3)= -1.1067001820e+00
       coeff_l2(12,4)=  1.4799482822e+00
       coeff_l2(12,5)=  1.8764706850e+00
       coeff_l2(12,6)=  2.4198698997e-01
       coeff_l2(12,7)=  2.1560873985e+00
       coeff_l2(12,8)=  4.6658229828e+00
       coeff_l2(12,9)= -1.4931410551e+00
       coeff_l2(12,10)=  6.5818858147e+00
       coeff_l2(12,11)= -3.4890246391e+00
       coeff_l2(12,12)= -6.2110257149e-01
       coeff_l2(12,13)= -1.7097262144e+00
       coeff_l2(12,14)= -5.3045600653e-02
       coeff_l2(12,15)= -2.0371568203e+00
       coeff_l2(12,16)=  1.9441524744e+00
       coeff_l2(12,17)=  2.9275873303e-01
       coeff_l2(12,18)= -1.4871900082e+00
       coeff_l2(12,19)=  4.6371123195e-01
       coeff_l2(12,20)= -5.4330277443e+00
       coeff_l2(12,21)=  5.5371451378e-01
       coeff_l2(12,22)=  2.0749216080e+00
       coeff_l2(12,23)= -6.6876184940e-01
       coeff_l2(12,24)=  7.6811404228e+00
       coeff_l2(12,25)= -1.2042053044e-01
       coeff_l2(12,26)= -2.2565672398e+00
       coeff_l2(12,27)=  1.1831500530e+00
       coeff_l2(12,28)= -1.0783616453e-01
       coeff_l2(12,29)=  1.1533734947e-01
       coeff_l2(12,30)=  1.7845312357e+00
       coeff_l2(12,31)=  1.1948676109e+00
       coeff_l2(12,32)= -9.9702215195e-01
       coeff_l2(12,33)= -8.5366815329e-01
       coeff_l2(12,34)= -2.7996078134e-01
       coeff_l2(12,35)= -2.2487828732e+00
       coeff_l2(12,36)=  5.9553351402e+00
       coeff_l2(12,37)= -8.0086183548e-01
       coeff_l2(12,38)=  7.7841603756e-01
       coeff_l2(12,39)= -5.3028541803e-01
       coeff_l2(12,40)= -4.3132025003e-01
       coeff_l2(12,41)= -3.7873606682e+00
       coeff_l2(12,42)=  1.5305045843e+00
       coeff_l2(12,43)=  2.0778210163e+00
       coeff_l2(12,44)=  5.1660485268e+00
       coeff_l2(12,45)=  7.8007006645e-01
       coeff_l2(12,46)= -7.1030879021e-01
       coeff_l2(12,47)= -7.9720787704e-02
       coeff_l2(12,48)=  7.0556336641e-01
       coeff_l2(12,49)= -6.0001653433e-01
       coeff_l2(12,50)= -1.3509732485e+00
       coeff_l2(12,51)= -2.3638935387e-01
       coeff_l2(12,52)=  4.5976042747e+00
       coeff_l2(12,53)= -1.0220212936e+00
       coeff_l2(12,54)= -5.5277037621e-01
       coeff_l2(12,55)= -1.8933194876e+00
       coeff_l2(13,1)=  1.0609470606e+00
       coeff_l2(13,2)= -3.9501938820e+00
       coeff_l2(13,3)=  5.8569014072e-01
       coeff_l2(13,4)=  5.5255401134e-01
       coeff_l2(13,5)=  5.2528495789e+00
       coeff_l2(13,6)= -3.1495839357e-01
       coeff_l2(13,7)= -6.2923833728e-02
       coeff_l2(13,8)=  3.4859280586e+00
       coeff_l2(13,9)= -1.4806826115e+00
       coeff_l2(13,10)=  2.9019641876e+00
       coeff_l2(13,11)= -7.5041115284e-01
       coeff_l2(13,12)= -1.4886604548e+00
       coeff_l2(13,13)= -1.4129897356e+00
       coeff_l2(13,14)= -8.3194619417e-01
       coeff_l2(13,15)= -9.5712602139e-01
       coeff_l2(13,16)=  1.3417378664e+00
       coeff_l2(13,17)=  4.5926313400e+00
       coeff_l2(13,18)= -5.3874671459e-02
       coeff_l2(13,19)=  2.9455421492e-02
       coeff_l2(13,20)=  3.5791914463e+00
       coeff_l2(13,21)= -7.1435910463e-01
       coeff_l2(13,22)=  1.4861693382e+00
       coeff_l2(13,23)=  8.3513075113e-01
       coeff_l2(13,24)=  5.2237123251e-01
       coeff_l2(13,25)=  3.0692868233e+00
       coeff_l2(13,26)=  4.6219161153e-01
       coeff_l2(13,27)=  4.7491571307e-01
       coeff_l2(13,28)=  3.1744258404e+00
       coeff_l2(13,29)= -1.1141020805e-01
       coeff_l2(13,30)=  1.1453586817e-01
       coeff_l2(13,31)=  8.1403464079e-01
       coeff_l2(13,32)= -1.8461585045e+00
       coeff_l2(13,33)= -1.5731337070e+00
       coeff_l2(13,34)= -3.2751251012e-02
       coeff_l2(13,35)= -7.2601860762e-01
       coeff_l2(13,36)= -3.1320302486e+00
       coeff_l2(13,37)= -1.4581992626e+00
       coeff_l2(13,38)=  9.4888073206e-01
       coeff_l2(13,39)=  3.4109652042e-01
       coeff_l2(13,40)= -9.0843552351e-01
       coeff_l2(13,41)= -1.8358095884e+00
       coeff_l2(13,42)=  8.1094193459e-01
       coeff_l2(13,43)=  9.9785608053e-01
       coeff_l2(13,44)=  2.8937175870e-01
       coeff_l2(13,45)= -2.2623178363e-01
       coeff_l2(13,46)=  5.6972432137e-01
       coeff_l2(13,47)=  1.0243430138e+00
       coeff_l2(13,48)=  2.4556627870e-01
       coeff_l2(13,49)= -4.9179902673e-01
       coeff_l2(13,50)= -2.8327751160e-01
       coeff_l2(13,51)= -1.0111386776e+00
       coeff_l2(13,52)=  1.3646937609e+00
       coeff_l2(13,53)= -6.5102690458e-01
       coeff_l2(13,54)= -7.6964014769e-01
       coeff_l2(13,55)=  3.6936417222e-01
       coeff_l2(14,1)=  2.4626111984e+00
       coeff_l2(14,2)=  1.6701395512e+00
       coeff_l2(14,3)=  9.6275854111e-01
       coeff_l2(14,4)=  1.7539205551e+00
       coeff_l2(14,5)=  1.3971625566e+00
       coeff_l2(14,6)=  4.8328489065e-01
       coeff_l2(14,7)=  1.6194735765e+00
       coeff_l2(14,8)=  2.7379643917e+00
       coeff_l2(14,9)= -1.4708025753e-01
       coeff_l2(14,10)=  2.7356610298e+00
       coeff_l2(14,11)=  1.8969371915e-01
       coeff_l2(14,12)= -1.0691300780e-01
       coeff_l2(14,13)= -3.5050031543e-01
       coeff_l2(14,14)=  1.1205782890e+00
       coeff_l2(14,15)= -3.7143763900e-01
       coeff_l2(14,16)=  1.3476191759e+00
       coeff_l2(14,17)=  4.9591995776e-02
       coeff_l2(14,18)= -8.1394737959e-01
       coeff_l2(14,19)=  4.0615370870e-01
       coeff_l2(14,20)= -4.7707567215e+00
       coeff_l2(14,21)=  5.2297031879e-01
       coeff_l2(14,22)= -2.0818667114e-01
       coeff_l2(14,23)= -5.8208084106e-01
       coeff_l2(14,24)=  5.5166673660e+00
       coeff_l2(14,25)= -1.2458679676e+00
       coeff_l2(14,26)= -8.5050541162e-01
       coeff_l2(14,27)=  1.9523878396e-01
       coeff_l2(14,28)= -9.8797738552e-01
       coeff_l2(14,29)=  7.8641664982e-01
       coeff_l2(14,30)=  6.9640940428e-01
       coeff_l2(14,31)= -6.9851541519e-01
       coeff_l2(14,32)=  3.3190336823e-01
       coeff_l2(14,33)=  4.0393543243e-01
       coeff_l2(14,34)= -7.7138942480e-01
       coeff_l2(14,35)= -1.1782561541e+00
       coeff_l2(14,36)=  5.7047767639e+00
       coeff_l2(14,37)=  1.4737495184e+00
       coeff_l2(14,38)=  4.6381524205e-01
       coeff_l2(14,39)=  4.0354660153e-01
       coeff_l2(14,40)= -8.7555460632e-02
       coeff_l2(14,41)= -2.4428596497e+00
       coeff_l2(14,42)=  1.6059209108e+00
       coeff_l2(14,43)=  3.1211040020e+00
       coeff_l2(14,44)=  1.6785683632e+00
       coeff_l2(14,45)= -5.8218963444e-02
       coeff_l2(14,46)= -8.9563453197e-01
       coeff_l2(14,47)= -4.0668919683e-01
       coeff_l2(14,48)=  1.3741844893e+00
       coeff_l2(14,49)=  1.6437040269e-01
       coeff_l2(14,50)= -4.4824975729e-01
       coeff_l2(14,51)= -1.0341256857e+00
       coeff_l2(14,52)=  9.5064967871e-01
       coeff_l2(14,53)=  5.3465431929e-01
       coeff_l2(14,54)=  8.6239956319e-02
       coeff_l2(14,55)= -8.6382073164e-01
       coeff_l2(15,1)= -8.1481713057e-01
       coeff_l2(15,2)= -2.3727113008e-01
       coeff_l2(15,3)=  8.0013799667e-01
       coeff_l2(15,4)= -1.7277569771e+00
       coeff_l2(15,5)=  2.5338220596e-01
       coeff_l2(15,6)= -1.2227065563e+00
       coeff_l2(15,7)= -3.7552082539e-01
       coeff_l2(15,8)=  1.4703722000e+00
       coeff_l2(15,9)= -1.7980095148e+00
       coeff_l2(15,10)=  9.1171109676e-01
       coeff_l2(15,11)= -1.0661761761e+00
       coeff_l2(15,12)= -1.0198650360e+00
       coeff_l2(15,13)= -1.2954312563e+00
       coeff_l2(15,14)= -1.1975003481e+00
       coeff_l2(15,15)= -8.9668661356e-01
       coeff_l2(15,16)=  4.4188031554e-01
       coeff_l2(15,17)=  3.4890403748e+00
       coeff_l2(15,18)= -3.2598811388e-01
       coeff_l2(15,19)= -1.0231114626e+00
       coeff_l2(15,20)= -1.9949194193e+00
       coeff_l2(15,21)=  4.7280731797e-01
       coeff_l2(15,22)=  1.6080000401e+00
       coeff_l2(15,23)=  5.6294769049e-01
       coeff_l2(15,24)= -1.7768870592e+00
       coeff_l2(15,25)=  6.5743468702e-02
       coeff_l2(15,26)=  1.3858704567e+00
       coeff_l2(15,27)=  5.1312148571e-01
       coeff_l2(15,28)=  1.4999845624e-01
       coeff_l2(15,29)= -7.0973974466e-01
       coeff_l2(15,30)=  2.2610713542e-01
       coeff_l2(15,31)=  9.5083737373e-01
       coeff_l2(15,32)= -1.2700200081e+00
       coeff_l2(15,33)= -1.8149151802e+00
       coeff_l2(15,34)=  1.7241330817e-02
       coeff_l2(15,35)= -1.3251900673e+00
       coeff_l2(15,36)=  2.8573718071e+00
       coeff_l2(15,37)= -2.9772868156e+00
       coeff_l2(15,38)=  1.3916298151e+00
       coeff_l2(15,39)= -3.0152389407e-01
       coeff_l2(15,40)= -8.5024762154e-01
       coeff_l2(15,41)= -1.9915013313e+00
       coeff_l2(15,42)=  3.7714824080e-01
       coeff_l2(15,43)=  6.6814672947e-01
       coeff_l2(15,44)= -8.3571159840e-01
       coeff_l2(15,45)=  5.3820896149e-01
       coeff_l2(15,46)=  4.1126602888e-01
       coeff_l2(15,47)= -1.2547969818e-01
       coeff_l2(15,48)=  1.1799289286e-01
       coeff_l2(15,49)= -4.4475799799e-01
       coeff_l2(15,50)= -6.1325120926e-01
       coeff_l2(15,51)=  6.7329710722e-01
       coeff_l2(15,52)=  2.1574988961e-01
       coeff_l2(15,53)= -1.4498851299e+00
       coeff_l2(15,54)=  5.6188881397e-02
       coeff_l2(15,55)=  5.8506208658e-01
       coeff_l2(16,1)= -2.5487778187e+00
       coeff_l2(16,2)=  4.3682393432e-01
       coeff_l2(16,3)=  4.7625666857e-01
       coeff_l2(16,4)= -1.1996079683e+00
       coeff_l2(16,5)= -3.2205519080e-01
       coeff_l2(16,6)= -1.7738457918e+00
       coeff_l2(16,7)= -1.4801425934e+00
       coeff_l2(16,8)=  6.6125005484e-02
       coeff_l2(16,9)= -1.5296399593e+00
       coeff_l2(16,10)=  1.6109662056e+00
       coeff_l2(16,11)=  9.0992641449e-01
       coeff_l2(16,12)= -7.8167301416e-01
       coeff_l2(16,13)= -1.8185188770e+00
       coeff_l2(16,14)= -8.5502475500e-02
       coeff_l2(16,15)=  4.7467955947e-01
       coeff_l2(16,16)=  1.5276262164e-01
       coeff_l2(16,17)= -1.6019204855e+00
       coeff_l2(16,18)= -6.4276456833e-01
       coeff_l2(16,19)=  6.2428790331e-01
       coeff_l2(16,20)= -7.4862587452e-01
       coeff_l2(16,21)= -3.2057099044e-02
       coeff_l2(16,22)= -1.0714037418e+00
       coeff_l2(16,23)= -7.8263956308e-01
       coeff_l2(16,24)=  2.1441066265e+00
       coeff_l2(16,25)= -7.5991350412e-01
       coeff_l2(16,26)= -1.2172293663e+00
       coeff_l2(16,27)= -7.1106982231e-01
       coeff_l2(16,28)=  1.5092725754e+00
       coeff_l2(16,29)= -8.4852051735e-01
       coeff_l2(16,30)= -1.1685011387e+00
       coeff_l2(16,31)= -1.0582741499e+00
       coeff_l2(16,32)= -9.9563425779e-01
       coeff_l2(16,33)=  3.4066241980e-01
       coeff_l2(16,34)= -9.9821612239e-02
       coeff_l2(16,35)= -1.4777503908e-01
       coeff_l2(16,36)= -6.1217010021e-01
       coeff_l2(16,37)= -2.0348091424e-01
       coeff_l2(16,38)= -9.4113194942e-01
       coeff_l2(16,39)= -9.6776328981e-02
       coeff_l2(16,40)= -1.6942469776e-01
       coeff_l2(16,41)= -1.0351725817e+00
       coeff_l2(16,42)= -4.7113096714e-01
       coeff_l2(16,43)=  2.4271802902e+00
       coeff_l2(16,44)= -8.1286031008e-01
       coeff_l2(16,45)= -7.7355451882e-02
       coeff_l2(16,46)= -8.9132684469e-01
       coeff_l2(16,47)= -1.5839721262e-01
       coeff_l2(16,48)=  6.6588407755e-01
       coeff_l2(16,49)= -2.3471808434e+00
       coeff_l2(16,50)=  4.7092559934e-01
       coeff_l2(16,51)= -1.0601594448e+00
       coeff_l2(16,52)=  8.9546817541e-01
       coeff_l2(16,53)= -1.0671563148e+00
       coeff_l2(16,54)= -6.9751536846e-01
       coeff_l2(16,55)= -5.4718571901e-01
       coeff_l2(17,1)=  1.4091801643e+00
       coeff_l2(17,2)= -3.8419599533e+00
       coeff_l2(17,3)=  5.4255777597e-01
       coeff_l2(17,4)=  3.9116552472e-01
       coeff_l2(17,5)=  5.8878502846e+00
       coeff_l2(17,6)= -3.1458160281e-01
       coeff_l2(17,7)= -5.6975209713e-01
       coeff_l2(17,8)=  4.4346647263e+00
       coeff_l2(17,9)= -1.8555952311e+00
       coeff_l2(17,10)=  3.8118464947e+00
       coeff_l2(17,11)= -1.4125819206e+00
       coeff_l2(17,12)= -1.7706921101e+00
       coeff_l2(17,13)= -1.7749537230e+00
       coeff_l2(17,14)= -1.0519726276e+00
       coeff_l2(17,15)= -1.4174778461e+00
       coeff_l2(17,16)=  1.8786170483e+00
       coeff_l2(17,17)=  5.4531712532e+00
       coeff_l2(17,18)= -3.0663447455e-02
       coeff_l2(17,19)= -6.5296344459e-02
       coeff_l2(17,20)=  2.4211626053e+00
       coeff_l2(17,21)= -5.5628770590e-01
       coeff_l2(17,22)=  1.9682261944e+00
       coeff_l2(17,23)=  1.0979959965e+00
       coeff_l2(17,24)=  1.5563242435e+00
       coeff_l2(17,25)=  2.8488547802e+00
       coeff_l2(17,26)=  4.3367767334e-01
       coeff_l2(17,27)=  7.3074597120e-01
       coeff_l2(17,28)=  2.6267099380e+00
       coeff_l2(17,29)= -2.2742350399e-01
       coeff_l2(17,30)=  2.6488766074e-01
       coeff_l2(17,31)=  3.8327291012e+00
       coeff_l2(17,32)= -2.1562869549e+00
       coeff_l2(17,33)= -1.8576277494e+00
       coeff_l2(17,34)=  2.8833270073e-02
       coeff_l2(17,35)= -1.2776607275e+00
       coeff_l2(17,36)= -1.7300307751e+00
       coeff_l2(17,37)= -2.0796849728e+00
       coeff_l2(17,38)=  1.5115151405e+00
       coeff_l2(17,39)=  2.0703366399e-01
       coeff_l2(17,40)= -1.1269259453e+00
       coeff_l2(17,41)= -2.7978413105e+00
       coeff_l2(17,42)=  1.0796315670e+00
       coeff_l2(17,43)=  1.2532867193e+00
       coeff_l2(17,44)=  7.5418245792e-01
       coeff_l2(17,45)=  6.0646321625e-02
       coeff_l2(17,46)=  5.6638914347e-01
       coeff_l2(17,47)=  1.0500936508e+00
       coeff_l2(17,48)=  3.7449762225e-01
       coeff_l2(17,49)= -6.1111968756e-01
       coeff_l2(17,50)= -6.2723881006e-01
       coeff_l2(17,51)= -9.3330109119e-01
       coeff_l2(17,52)=  2.0943882465e+00
       coeff_l2(17,53)= -9.6967965364e-01
       coeff_l2(17,54)= -9.4090116024e-01
       coeff_l2(17,55)=  2.8015172482e-01
       coeff_l2(18,1)= -1.7316625118e+00
       coeff_l2(18,2)=  2.5449529290e-01
       coeff_l2(18,3)= -2.0368559659e-01
       coeff_l2(18,4)=  6.3285171986e-02
       coeff_l2(18,5)=  9.8692315817e-01
       coeff_l2(18,6)= -1.1777516603e+00
       coeff_l2(18,7)= -1.4225682616e-01
       coeff_l2(18,8)=  2.6571662426e+00
       coeff_l2(18,9)= -1.1888651848e+00
       coeff_l2(18,10)=  8.2041865587e-01
       coeff_l2(18,11)=  2.5966039300e-01
       coeff_l2(18,12)= -5.6495594978e-01
       coeff_l2(18,13)= -3.5737365484e-01
       coeff_l2(18,14)=  1.0179160833e+00
       coeff_l2(18,15)=  1.0559661388e+00
       coeff_l2(18,16)=  1.2360887527e+00
       coeff_l2(18,17)= -2.4644371867e-01
       coeff_l2(18,18)=  8.9388203621e-01
       coeff_l2(18,19)=  3.3057042956e-01
       coeff_l2(18,20)= -1.5924978256e+00
       coeff_l2(18,21)=  1.6407899559e-01
       coeff_l2(18,22)= -1.0804008693e-01
       coeff_l2(18,23)= -1.2356036156e-01
       coeff_l2(18,24)= -4.2941114306e-01
       coeff_l2(18,25)= -4.1836637259e-01
       coeff_l2(18,26)= -8.2092726231e-01
       coeff_l2(18,27)= -5.9880119562e-01
       coeff_l2(18,28)=  2.4021105766e+00
       coeff_l2(18,29)= -5.9508746862e-01
       coeff_l2(18,30)= -1.1484588385e+00
       coeff_l2(18,31)= -3.2446551323e-01
       coeff_l2(18,32)= -1.7954635620e-01
       coeff_l2(18,33)= -1.6172284260e-02
       coeff_l2(18,34)= -4.2090013623e-01
       coeff_l2(18,35)= -1.7855536938e+00
       coeff_l2(18,36)= -3.0464309454e-01
       coeff_l2(18,37)= -6.9630837440e-01
       coeff_l2(18,38)= -2.0272327960e-01
       coeff_l2(18,39)=  2.5579292327e-02
       coeff_l2(18,40)=  1.8523718417e-01
       coeff_l2(18,41)=  9.9041336775e-01
       coeff_l2(18,42)=  7.6632834971e-02
       coeff_l2(18,43)=  2.6075068116e-01
       coeff_l2(18,44)= -6.9059294462e-01
       coeff_l2(18,45)= -2.1454320848e-01
       coeff_l2(18,46)= -1.7441250384e-01
       coeff_l2(18,47)= -4.9184826016e-01
       coeff_l2(18,48)=  2.2950751781e+00
       coeff_l2(18,49)= -5.0692880154e-01
       coeff_l2(18,50)=  9.9196493626e-01
       coeff_l2(18,51)= -1.5281751752e-01
       coeff_l2(18,52)=  1.4716094732e+00
       coeff_l2(18,53)=  2.0251843929e+00
       coeff_l2(18,54)= -1.3282974064e-01
       coeff_l2(18,55)= -9.2642702162e-02
       coeff_l2(19,1)= -7.7920109034e-01
       coeff_l2(19,2)=  1.7285898924e+00
       coeff_l2(19,3)= -3.0650115013e-01
       coeff_l2(19,4)=  3.7695953250e-01
       coeff_l2(19,5)= -2.2704789639e+00
       coeff_l2(19,6)=  1.0294692516e+00
       coeff_l2(19,7)= -6.3261008263e-01
       coeff_l2(19,8)= -4.4585433006e+00
       coeff_l2(19,9)=  1.5584286451e+00
       coeff_l2(19,10)= -4.1673359871e+00
       coeff_l2(19,11)=  9.3435031176e-01
       coeff_l2(19,12)=  4.4244241714e-01
       coeff_l2(19,13)=  2.0746786594e+00
       coeff_l2(19,14)=  1.2333903313e+00
       coeff_l2(19,15)=  1.9636065960e+00
       coeff_l2(19,16)= -1.3895152807e+00
       coeff_l2(19,17)= -2.7770042419e+00
       coeff_l2(19,18)=  1.0814602375e+00
       coeff_l2(19,19)= -1.1322837323e-02
       coeff_l2(19,20)=  1.4361495972e+00
       coeff_l2(19,21)=  7.0782518014e-03
       coeff_l2(19,22)= -7.8430265188e-01
       coeff_l2(19,23)=  4.7898121178e-02
       coeff_l2(19,24)= -3.9674658775e+00
       coeff_l2(19,25)= -9.7388792038e-01
       coeff_l2(19,26)=  4.5443925261e-01
       coeff_l2(19,27)= -2.8765898943e-01
       coeff_l2(19,28)=  3.6586329937e+00
       coeff_l2(19,29)=  3.2225328684e-01
       coeff_l2(19,30)= -7.2056166828e-02
       coeff_l2(19,31)= -1.6940392256e+00
       coeff_l2(19,32)= -2.2768924236e+00
       coeff_l2(19,33)=  1.2856849432e+00
       coeff_l2(19,34)= -2.5654053688e-01
       coeff_l2(19,35)=  1.4584950209e+00
       coeff_l2(19,36)= -1.0044100136e-01
       coeff_l2(19,37)=  2.3652336597e+00
       coeff_l2(19,38)= -1.6109330654e+00
       coeff_l2(19,39)=  4.2078468204e-01
       coeff_l2(19,40)=  2.8938936070e-02
       coeff_l2(19,41)=  3.0328524113e+00
       coeff_l2(19,42)= -3.2898482680e-01
       coeff_l2(19,43)=  4.6344399452e-01
       coeff_l2(19,44)= -2.4181375504e+00
       coeff_l2(19,45)= -2.9138424993e-01
       coeff_l2(19,46)= -2.5380313396e-01
       coeff_l2(19,47)= -7.5451534986e-01
       coeff_l2(19,48)=  1.3233016431e-01
       coeff_l2(19,49)=  8.1083737314e-02
       coeff_l2(19,50)=  1.0804730654e+00
       coeff_l2(19,51)=  5.5898410082e-01
       coeff_l2(19,52)= -2.4795963764e+00
       coeff_l2(19,53)=  7.3770207167e-01
       coeff_l2(19,54)= -4.1370379925e-01
       coeff_l2(19,55)=  8.2161200047e-01
       coeff_l2(20,1)=  4.2082779109e-02
       coeff_l2(20,2)=  3.1028189659e+00
       coeff_l2(20,3)=  6.5481181145e+00
       coeff_l2(20,4)= -8.4610795975e-01
       coeff_l2(20,5)= -2.2114458084e+00
       coeff_l2(20,6)= -2.0297098160e-01
       coeff_l2(20,7)=  2.7585630417e+00
       coeff_l2(20,8)=  3.2928694040e-02
       coeff_l2(20,9)=  1.7901229858e+00
       coeff_l2(20,10)=  6.1565507203e-02
       coeff_l2(20,11)=  5.3730785847e-01
       coeff_l2(20,12)=  1.0800459385e+00
       coeff_l2(20,13)=  4.7484374046e-01
       coeff_l2(20,14)=  1.8536919355e+00
       coeff_l2(20,15)=  5.4772239923e-01
       coeff_l2(20,16)= -1.4564925432e-01
       coeff_l2(20,17)=  3.8778541088e+00
       coeff_l2(20,18)= -1.5662490129e+00
       coeff_l2(20,19)= -1.4911460876e+00
       coeff_l2(20,20)= -9.5291061401e+00
       coeff_l2(20,21)= -1.1620935053e-01
       coeff_l2(20,22)= -8.1072390079e-01
       coeff_l2(20,23)=  3.8438889384e-01
       coeff_l2(20,24)=  2.0420363545e-01
       coeff_l2(20,25)= -4.3660945892e+00
       coeff_l2(20,26)=  2.4708375335e-02
       coeff_l2(20,27)= -1.0544667244e+00
       coeff_l2(20,28)= -1.0343070030e+01
       coeff_l2(20,29)=  8.7990915775e-01
       coeff_l2(20,30)=  8.8379830122e-01
       coeff_l2(20,31)= -1.8227655888e+00
       coeff_l2(20,32)=  5.8551013470e-02
       coeff_l2(20,33)= -2.8079993725e+00
       coeff_l2(20,34)= -1.1339819431e+00
       coeff_l2(20,35)= -1.3315006495e+00
       coeff_l2(20,36)=  1.3499926567e+01
       coeff_l2(20,37)=  3.6983436346e-01
       coeff_l2(20,38)= -1.5357998610e+00
       coeff_l2(20,39)=  4.4639185071e-01
       coeff_l2(20,40)= -2.8656899929e-01
       coeff_l2(20,41)=  1.3221355677e+00
       coeff_l2(20,42)=  3.1218389049e-02
       coeff_l2(20,43)= -3.5450868607e+00
       coeff_l2(20,44)= -9.3705034256e-01
       coeff_l2(20,45)= -3.7718164921e-01
       coeff_l2(20,46)= -9.7151523829e-01
       coeff_l2(20,47)= -1.6282820702e+00
       coeff_l2(20,48)=  2.0633206367e+00
       coeff_l2(20,49)=  2.4809687138e+00
       coeff_l2(20,50)= -7.2663044930e-01
       coeff_l2(20,51)= -1.4065967798e+00
       coeff_l2(20,52)=  3.0000138283e+00
       coeff_l2(20,53)=  1.0828022957e+00
       coeff_l2(20,54)= -7.4853104353e-01
       coeff_l2(20,55)= -7.9969145358e-02
       coeff_l2(21,1)=  8.5562336445e-01
       coeff_l2(21,2)=  2.0243840218e+00
       coeff_l2(21,3)=  3.9185090065e+00
       coeff_l2(21,4)= -1.3133455813e-01
       coeff_l2(21,5)= -1.6592866182e+00
       coeff_l2(21,6)=  1.7948335409e+00
       coeff_l2(21,7)=  4.2438569069e+00
       coeff_l2(21,8)=  8.3580040932e-01
       coeff_l2(21,9)=  7.0841765404e-01
       coeff_l2(21,10)=  2.8849554062e-01
       coeff_l2(21,11)=  3.0322861671e+00
       coeff_l2(21,12)= -8.3781033754e-02
       coeff_l2(21,13)=  1.3310834169e+00
       coeff_l2(21,14)=  2.3319914341e+00
       coeff_l2(21,15)= -4.1121515632e-01
       coeff_l2(21,16)=  8.3845898509e-02
       coeff_l2(21,17)= -2.8484489918e+00
       coeff_l2(21,18)=  1.4353746176e+00
       coeff_l2(21,19)=  4.9693241715e-01
       coeff_l2(21,20)= -3.2903888226e+00
       coeff_l2(21,21)= -9.4399803877e-01
       coeff_l2(21,22)=  1.7437647283e-01
       coeff_l2(21,23)=  2.1141242981e+00
       coeff_l2(21,24)=  4.5677641034e-01
       coeff_l2(21,25)= -2.3340466022e+00
       coeff_l2(21,26)= -1.4042613506e+00
       coeff_l2(21,27)= -7.9366886616e-01
       coeff_l2(21,28)= -5.3284182549e+00
       coeff_l2(21,29)= -5.0629395247e-01
       coeff_l2(21,30)=  8.2871997356e-01
       coeff_l2(21,31)=  1.9900122881e+00
       coeff_l2(21,32)= -3.1469205022e-01
       coeff_l2(21,33)=  1.7844915390e+00
       coeff_l2(21,34)=  7.8958064318e-01
       coeff_l2(21,35)=  3.5308295488e-01
       coeff_l2(21,36)=  8.4945955276e+00
       coeff_l2(21,37)=  6.4138045311e+00
       coeff_l2(21,38)= -1.8730607629e-01
       coeff_l2(21,39)=  6.2196695805e-01
       coeff_l2(21,40)= -5.8348119259e-01
       coeff_l2(21,41)=  4.0427556634e-01
       coeff_l2(21,42)=  4.8271811008e-01
       coeff_l2(21,43)= -5.1477277279e-01
       coeff_l2(21,44)=  4.8499533534e-01
       coeff_l2(21,45)=  2.9765799642e-01
       coeff_l2(21,46)= -1.4822224379e+00
       coeff_l2(21,47)= -1.8032114953e-02
       coeff_l2(21,48)=  1.3825397491e+00
       coeff_l2(21,49)=  1.0058825016e+00
       coeff_l2(21,50)= -1.6043537855e+00
       coeff_l2(21,51)=  1.6485758126e-01
       coeff_l2(21,52)= -1.6728206873e+00
       coeff_l2(21,53)=  1.7981773615e+00
       coeff_l2(21,54)= -9.5331466198e-01
       coeff_l2(21,55)=  5.0260084867e-01
       coeff_l2(22,1)=  7.1331435442e-01
       coeff_l2(22,2)= -3.2626068592e+00
       coeff_l2(22,3)= -1.8724591732e+00
       coeff_l2(22,4)= -2.0152416229e+00
       coeff_l2(22,5)=  4.9234972000e+00
       coeff_l2(22,6)= -1.2629851103e+00
       coeff_l2(22,7)= -2.7074677944e+00
       coeff_l2(22,8)= -2.2634303570e-01
       coeff_l2(22,9)=  2.2237818688e-02
       coeff_l2(22,10)= -2.5498421192e+00
       coeff_l2(22,11)= -2.1704816818e+00
       coeff_l2(22,12)= -7.3454248905e-01
       coeff_l2(22,13)=  1.2100359201e+00
       coeff_l2(22,14)=  9.8537629843e-01
       coeff_l2(22,15)= -1.5442827940e+00
       coeff_l2(22,16)= -1.2487175465e+00
       coeff_l2(22,17)= -9.1492241621e-01
       coeff_l2(22,18)= -2.1084460616e-01
       coeff_l2(22,19)=  4.1083231568e-01
       coeff_l2(22,20)=  4.8620820045e+00
       coeff_l2(22,21)=  4.2653664947e-01
       coeff_l2(22,22)=  9.1370329261e-02
       coeff_l2(22,23)= -9.4712525606e-01
       coeff_l2(22,24)=  2.0080544949e+00
       coeff_l2(22,25)= -5.7842975855e-01
       coeff_l2(22,26)= -6.5536075830e-01
       coeff_l2(22,27)= -3.8173443079e-01
       coeff_l2(22,28)= -1.8138616085e+00
       coeff_l2(22,29)= -2.8757238388e-01
       coeff_l2(22,30)= -1.2232925892e+00
       coeff_l2(22,31)= -6.4480566978e-01
       coeff_l2(22,32)= -6.1752200127e-01
       coeff_l2(22,33)=  6.1968520284e-02
       coeff_l2(22,34)=  2.0595158637e-01
       coeff_l2(22,35)= -3.3318549395e-01
       coeff_l2(22,36)= -3.7700092793e+00
       coeff_l2(22,37)=  1.4040199518e+00
       coeff_l2(22,38)=  8.3819933236e-02
       coeff_l2(22,39)=  2.1030793190e+00
       coeff_l2(22,40)=  4.7164872289e-02
       coeff_l2(22,41)=  1.2987227440e+00
       coeff_l2(22,42)=  3.2060968876e-01
       coeff_l2(22,43)= -3.0274519920e+00
       coeff_l2(22,44)=  1.6328701973e+00
       coeff_l2(22,45)= -7.4048858881e-01
       coeff_l2(22,46)= -1.1228870153e+00
       coeff_l2(22,47)=  4.6181488037e-01
       coeff_l2(22,48)= -8.5707837343e-01
       coeff_l2(22,49)=  1.3856589794e+00
       coeff_l2(22,50)= -3.0949789286e-01
       coeff_l2(22,51)=  1.6467222571e-01
       coeff_l2(22,52)=  1.7973332405e+00
       coeff_l2(22,53)=  4.5736378431e-01
       coeff_l2(22,54)= -2.0918339491e-01
       coeff_l2(22,55)=  5.1471036673e-01
       coeff_l2(23,1)=  6.3244593143e-01
       coeff_l2(23,2)= -2.2482435703e+00
       coeff_l2(23,3)= -4.6536082029e-01
       coeff_l2(23,4)= -4.6939158440e-01
       coeff_l2(23,5)=  3.9326167107e+00
       coeff_l2(23,6)= -8.2741951942e-01
       coeff_l2(23,7)=  5.1344066858e-01
       coeff_l2(23,8)=  2.0636918545e+00
       coeff_l2(23,9)= -1.1586567163e+00
       coeff_l2(23,10)=  2.1895995140e+00
       coeff_l2(23,11)= -1.4505312443e+00
       coeff_l2(23,12)= -9.4416660070e-01
       coeff_l2(23,13)= -8.0104118586e-01
       coeff_l2(23,14)= -7.0866554976e-01
       coeff_l2(23,15)= -9.6969097853e-01
       coeff_l2(23,16)=  6.1472350359e-01
       coeff_l2(23,17)=  3.2837734222e+00
       coeff_l2(23,18)=  1.5470041335e-01
       coeff_l2(23,19)=  1.3243336976e-01
       coeff_l2(23,20)=  1.7754646540e+00
       coeff_l2(23,21)= -1.9500108063e-01
       coeff_l2(23,22)=  1.0857473612e+00
       coeff_l2(23,23)=  3.9374187589e-01
       coeff_l2(23,24)=  6.4478826523e-01
       coeff_l2(23,25)=  1.5913439989e+00
       coeff_l2(23,26)=  4.8794570565e-01
       coeff_l2(23,27)=  4.3970197439e-01
       coeff_l2(23,28)=  3.8204364777e+00
       coeff_l2(23,29)= -1.7421732843e-01
       coeff_l2(23,30)=  2.7942207456e-01
       coeff_l2(23,31)=  1.6496374607e+00
       coeff_l2(23,32)= -1.4579160213e+00
       coeff_l2(23,33)= -9.1796416044e-01
       coeff_l2(23,34)=  2.0388999954e-02
       coeff_l2(23,35)= -7.3701161146e-01
       coeff_l2(23,36)= -2.4933907986e+00
       coeff_l2(23,37)= -7.0611786842e-01
       coeff_l2(23,38)=  5.1684784889e-01
       coeff_l2(23,39)= -2.4275282025e-01
       coeff_l2(23,40)= -4.7696200013e-01
       coeff_l2(23,41)= -1.4555387497e+00
       coeff_l2(23,42)=  9.3569779396e-01
       coeff_l2(23,43)=  8.0508875847e-01
       coeff_l2(23,44)=  1.3294064999e+00
       coeff_l2(23,45)=  7.0285327733e-02
       coeff_l2(23,46)=  3.5443934798e-01
       coeff_l2(23,47)=  5.5687016249e-01
       coeff_l2(23,48)= -2.0478974283e-01
       coeff_l2(23,49)= -4.7654637694e-01
       coeff_l2(23,50)= -2.5477492809e-01
       coeff_l2(23,51)= -2.3984248936e-01
       coeff_l2(23,52)=  9.5658689737e-01
       coeff_l2(23,53)= -4.2437219620e-01
       coeff_l2(23,54)= -1.8783792853e-01
       coeff_l2(23,55)=  3.4342437983e-02
       coeff_l2(24,1)= -3.6607876420e-01
       coeff_l2(24,2)=  3.1402065754e+00
       coeff_l2(24,3)=  3.8832385540e+00
       coeff_l2(24,4)= -6.7939865589e-01
       coeff_l2(24,5)=  2.8229582310e+00
       coeff_l2(24,6)=  9.1125771403e-02
       coeff_l2(24,7)=  2.2161848545e+00
       coeff_l2(24,8)=  1.4596271515e+00
       coeff_l2(24,9)=  3.7987080216e-01
       coeff_l2(24,10)=  3.4125041962e+00
       coeff_l2(24,11)=  6.9848728180e-01
       coeff_l2(24,12)= -1.2714879513e+00
       coeff_l2(24,13)=  1.6934727430e+00
       coeff_l2(24,14)=  1.1065410376e+00
       coeff_l2(24,15)= -7.3017656803e-01
       coeff_l2(24,16)=  4.2493709922e-01
       coeff_l2(24,17)=  3.7363886833e-02
       coeff_l2(24,18)=  8.7914907932e-01
       coeff_l2(24,19)= -9.5723831654e-01
       coeff_l2(24,20)= -7.0067219734e+00
       coeff_l2(24,21)=  2.9739168286e-01
       coeff_l2(24,22)=  6.8716174364e-01
       coeff_l2(24,23)= -8.7287418544e-02
       coeff_l2(24,24)=  4.4623103142e+00
       coeff_l2(24,25)= -3.4403729439e+00
       coeff_l2(24,26)= -3.8480973244e+00
       coeff_l2(24,27)=  2.5816494226e-01
       coeff_l2(24,28)= -5.9289650917e+00
       coeff_l2(24,29)= -2.9185840487e-01
       coeff_l2(24,30)=  6.8360513449e-01
       coeff_l2(24,31)= -1.2224000692e+00
       coeff_l2(24,32)=  1.5738239288e+00
       coeff_l2(24,33)= -9.1196525097e-01
       coeff_l2(24,34)= -1.2978658676e+00
       coeff_l2(24,35)= -8.3724236488e-01
       coeff_l2(24,36)=  9.2740154266e+00
       coeff_l2(24,37)=  3.0929362774e-01
       coeff_l2(24,38)= -8.1041830778e-01
       coeff_l2(24,39)=  1.5975603461e-01
       coeff_l2(24,40)=  2.4873861670e-01
       coeff_l2(24,41)=  2.9639199376e-01
       coeff_l2(24,42)=  5.5261787027e-02
       coeff_l2(24,43)=  2.0147135258e+00
       coeff_l2(24,44)=  9.1438695788e-02
       coeff_l2(24,45)= -6.8975096941e-01
       coeff_l2(24,46)= -5.1457440853e-01
       coeff_l2(24,47)= -1.2296937704e+00
       coeff_l2(24,48)=  1.3089513779e+00
       coeff_l2(24,49)= -3.2882884145e-01
       coeff_l2(24,50)= -1.8713624775e-01
       coeff_l2(24,51)= -1.0770643502e-01
       coeff_l2(24,52)=  2.4724626541e+00
       coeff_l2(24,53)=  9.0931004286e-01
       coeff_l2(24,54)= -2.5340339541e-01
       coeff_l2(24,55)= -5.5921640247e-02
       coeff_l2(25,1)=  1.0970717669e+00
       coeff_l2(25,2)=  3.3500795364e+00
       coeff_l2(25,3)= -1.9010494649e-01
       coeff_l2(25,4)= -9.6047127247e-01
       coeff_l2(25,5)=  2.3435776234e+00
       coeff_l2(25,6)= -3.0417492986e-01
       coeff_l2(25,7)=  3.2840907574e-01
       coeff_l2(25,8)=  1.0157947540e+00
       coeff_l2(25,9)=  2.0056711510e-02
       coeff_l2(25,10)=  1.9095996022e-01
       coeff_l2(25,11)= -1.5419737101e+00
       coeff_l2(25,12)= -3.6489436030e-01
       coeff_l2(25,13)=  2.0871017873e-01
       coeff_l2(25,14)=  4.5303776860e-01
       coeff_l2(25,15)= -1.1594773531e+00
       coeff_l2(25,16)=  1.0359811783e+00
       coeff_l2(25,17)=  1.2648019791e+00
       coeff_l2(25,18)=  6.4035832882e-02
       coeff_l2(25,19)= -1.2892794609e-01
       coeff_l2(25,20)= -5.8606514931e+00
       coeff_l2(25,21)=  7.0694178343e-02
       coeff_l2(25,22)= -1.7216165066e+00
       coeff_l2(25,23)=  9.5321792364e-01
       coeff_l2(25,24)=  1.1991999149e+00
       coeff_l2(25,25)= -3.6559870243e+00
       coeff_l2(25,26)=  5.1868325472e-01
       coeff_l2(25,27)= -1.4793536663e+00
       coeff_l2(25,28)= -1.7501697540e+01
       coeff_l2(25,29)=  3.8601574302e-01
       coeff_l2(25,30)=  2.1082773805e-01
       coeff_l2(25,31)= -9.7959146500e+00
       coeff_l2(25,32)= -2.4499842525e-01
       coeff_l2(25,33)=  4.9553799629e-01
       coeff_l2(25,34)= -7.7580887079e-01
       coeff_l2(25,35)= -1.3064576387e+00
       coeff_l2(25,36)=  7.9030170441e+00
       coeff_l2(25,37)= -7.0312702656e-01
       coeff_l2(25,38)=  1.7266739905e-01
       coeff_l2(25,39)=  3.7716197968e-01
       coeff_l2(25,40)=  3.1178179383e-01
       coeff_l2(25,41)= -6.3833229244e-02
       coeff_l2(25,42)=  3.1292888522e-01
       coeff_l2(25,43)= -4.3188503385e-01
       coeff_l2(25,44)=  2.2555565834e-01
       coeff_l2(25,45)=  7.4176043272e-02
       coeff_l2(25,46)= -9.1994714737e-01
       coeff_l2(25,47)= -7.1347314119e-01
       coeff_l2(25,48)=  7.6121473312e-01
       coeff_l2(25,49)=  7.5127065182e-01
       coeff_l2(25,50)= -1.0862048715e-02
       coeff_l2(25,51)= -4.3891537189e-01
       coeff_l2(25,52)=  7.8601628542e-01
       coeff_l2(25,53)= -2.7062535286e-01
       coeff_l2(25,54)= -5.3715284914e-02
       coeff_l2(25,55)=  1.8441924453e-01
       coeff_l2(26,1)=  1.7396585941e+00
       coeff_l2(26,2)=  3.9389651269e-02
       coeff_l2(26,3)=  8.7259644270e-01
       coeff_l2(26,4)= -8.4405404329e-01
       coeff_l2(26,5)=  3.5929861069e+00
       coeff_l2(26,6)= -1.3544089794e+00
       coeff_l2(26,7)=  7.7094323933e-02
       coeff_l2(26,8)=  2.2323315144e+00
       coeff_l2(26,9)= -3.4816798568e-01
       coeff_l2(26,10)=  1.8834260702e+00
       coeff_l2(26,11)= -3.2206263859e-03
       coeff_l2(26,12)= -4.8820689321e-01
       coeff_l2(26,13)=  2.9885217547e-01
       coeff_l2(26,14)=  1.2977042198e+00
       coeff_l2(26,15)= -4.9940821528e-01
       coeff_l2(26,16)=  5.3312933445e-01
       coeff_l2(26,17)=  9.6410530806e-01
       coeff_l2(26,18)=  8.9220184088e-01
       coeff_l2(26,19)=  6.7976838350e-01
       coeff_l2(26,20)= -1.7568063736e+00
       coeff_l2(26,21)=  5.9158645570e-02
       coeff_l2(26,22)= -7.4663722515e-01
       coeff_l2(26,23)=  9.0907499194e-02
       coeff_l2(26,24)=  3.4491887093e+00
       coeff_l2(26,25)= -1.2324192524e+00
       coeff_l2(26,26)= -1.0665761232e+00
       coeff_l2(26,27)=  5.5740922689e-01
       coeff_l2(26,28)=  1.3770104647e+00
       coeff_l2(26,29)=  1.6642953455e-01
       coeff_l2(26,30)=  3.9115229249e-01
       coeff_l2(26,31)=  4.5200234652e-01
       coeff_l2(26,32)=  1.7671978474e-01
       coeff_l2(26,33)=  2.5659620762e-01
       coeff_l2(26,34)= -3.3929759264e-01
       coeff_l2(26,35)= -7.9699069262e-01
       coeff_l2(26,36)=  2.2452175617e+00
       coeff_l2(26,37)=  2.0601089001e+00
       coeff_l2(26,38)=  4.3045794964e-01
       coeff_l2(26,39)= -1.2873843312e-01
       coeff_l2(26,40)= -1.0877420753e-01
       coeff_l2(26,41)= -1.8408222198e+00
       coeff_l2(26,42)=  6.5617814660e-02
       coeff_l2(26,43)=  1.9298163652e+00
       coeff_l2(26,44)= -3.1053963304e-01
       coeff_l2(26,45)=  1.4429938793e-01
       coeff_l2(26,46)= -3.4659987688e-01
       coeff_l2(26,47)= -4.3327003717e-01
       coeff_l2(26,48)=  1.1941392422e+00
       coeff_l2(26,49)= -9.5251694322e-02
       coeff_l2(26,50)= -5.0780326128e-01
       coeff_l2(26,51)= -6.0547417402e-01
       coeff_l2(26,52)=  9.6610568464e-02
       coeff_l2(26,53)=  5.6264799833e-01
       coeff_l2(26,54)= -4.5878264308e-01
       coeff_l2(26,55)= -4.4043514132e-01
       coeff_l2(27,1)=  1.2276651710e-01
       coeff_l2(27,2)=  3.7380341440e-02
       coeff_l2(27,3)= -5.9946447611e-01
       coeff_l2(27,4)=  3.6050859094e-01
       coeff_l2(27,5)=  4.0360264778e+00
       coeff_l2(27,6)= -7.5207346678e-01
       coeff_l2(27,7)=  1.5991758108e+00
       coeff_l2(27,8)= -1.2772424221e+00
       coeff_l2(27,9)=  7.8628724813e-01
       coeff_l2(27,10)= -1.1874661446e+00
       coeff_l2(27,11)=  6.5116578341e-01
       coeff_l2(27,12)=  1.3999599218e+00
       coeff_l2(27,13)=  7.5361967087e-01
       coeff_l2(27,14)=  5.9330856800e-01
       coeff_l2(27,15)=  9.3214130402e-01
       coeff_l2(27,16)= -6.8832308054e-01
       coeff_l2(27,17)=  2.9065833092e+00
       coeff_l2(27,18)=  7.6103508472e-01
       coeff_l2(27,19)= -2.7575820684e-01
       coeff_l2(27,20)=  2.1248459816e+00
       coeff_l2(27,21)= -9.8578321934e-01
       coeff_l2(27,22)= -2.3217050731e-01
       coeff_l2(27,23)= -2.5350294113e+00
       coeff_l2(27,24)=  5.8045806885e+00
       coeff_l2(27,25)=  2.4916060269e-01
       coeff_l2(27,26)= -4.2044305801e+00
       coeff_l2(27,27)=  5.1598153077e-03
       coeff_l2(27,28)=  4.6874038875e-02
       coeff_l2(27,29)=  6.6644899547e-02
       coeff_l2(27,30)= -8.8845837116e-01
       coeff_l2(27,31)= -1.0761140585e+00
       coeff_l2(27,32)= -5.0328105688e-01
       coeff_l2(27,33)=  2.0778615475e+00
       coeff_l2(27,34)= -1.8581830263e+00
       coeff_l2(27,35)=  6.6843777895e-01
       coeff_l2(27,36)= -3.3399894238e+00
       coeff_l2(27,37)=  1.9628866911e+00
       coeff_l2(27,38)= -1.1750272512e+00
       coeff_l2(27,39)=  3.2485401630e-01
       coeff_l2(27,40)= -1.4577099085e+00
       coeff_l2(27,41)= -2.6641449332e-01
       coeff_l2(27,42)=  6.8721157312e-01
       coeff_l2(27,43)=  9.6730184555e-01
       coeff_l2(27,44)=  6.6377389431e-01
       coeff_l2(27,45)= -1.7480462790e+00
       coeff_l2(27,46)= -2.1197055280e-01
       coeff_l2(27,47)= -5.6023979187e-01
       coeff_l2(27,48)= -9.6935361624e-02
       coeff_l2(27,49)=  1.3856558800e+00
       coeff_l2(27,50)= -7.0589952171e-02
       coeff_l2(27,51)= -1.0856598616e+00
       coeff_l2(27,52)=  3.2959141731e+00
       coeff_l2(27,53)=  1.1697987318e+00
       coeff_l2(27,54)= -5.4791694880e-01
       coeff_l2(27,55)= -5.3305029869e-01
       coeff_l2(28,1)=  1.6972197294e+00
       coeff_l2(28,2)=  3.1026592255e+00
       coeff_l2(28,3)=  5.2452087402e+00
       coeff_l2(28,4)= -1.5111199617e+00
       coeff_l2(28,5)= -3.4247951508e+00
       coeff_l2(28,6)=  4.7726544738e-01
       coeff_l2(28,7)=  2.6658270359e+00
       coeff_l2(28,8)=  1.4748325348e+00
       coeff_l2(28,9)=  4.1413360834e-01
       coeff_l2(28,10)= -1.2451385260e+00
       coeff_l2(28,11)= -3.5519906878e-01
       coeff_l2(28,12)=  5.4416638613e-01
       coeff_l2(28,13)=  1.7263844013e+00
       coeff_l2(28,14)=  2.9346013069e+00
       coeff_l2(28,15)= -7.6051968336e-01
       coeff_l2(28,16)= -1.5689026564e-02
       coeff_l2(28,17)=  1.6301671267e+00
       coeff_l2(28,18)=  9.2665511370e-01
       coeff_l2(28,19)= -4.1860276461e-01
       coeff_l2(28,20)= -7.8136963844e+00
       coeff_l2(28,21)= -5.0710058212e-01
       coeff_l2(28,22)= -7.9684890807e-02
       coeff_l2(28,23)=  4.3306773901e-01
       coeff_l2(28,24)=  2.6153251529e-01
       coeff_l2(28,25)= -3.7472290993e+00
       coeff_l2(28,26)= -1.4949799776e+00
       coeff_l2(28,27)= -1.6908552498e-02
       coeff_l2(28,28)= -5.9802999496e+00
       coeff_l2(28,29)= -8.6092913151e-01
       coeff_l2(28,30)=  1.5034770966e-01
       coeff_l2(28,31)=  1.0740593076e-01
       coeff_l2(28,32)=  4.8320931196e-01
       coeff_l2(28,33)=  4.7864800692e-01
       coeff_l2(28,34)= -2.1238572598e+00
       coeff_l2(28,35)= -4.3237876892e-01
       coeff_l2(28,36)=  1.2010883331e+01
       coeff_l2(28,37)=  4.1173348427e+00
       coeff_l2(28,38)= -9.9031519890e-01
       coeff_l2(28,39)=  1.8106521666e-01
       coeff_l2(28,40)= -4.1225102544e-01
       coeff_l2(28,41)= -5.8767002821e-01
       coeff_l2(28,42)=  1.0834060907e+00
       coeff_l2(28,43)= -1.7626966238e+00
       coeff_l2(28,44)= -1.2345004827e-01
       coeff_l2(28,45)= -7.9420751333e-01
       coeff_l2(28,46)= -1.0869823694e+00
       coeff_l2(28,47)= -1.6254957914e+00
       coeff_l2(28,48)=  1.5696283579e+00
       coeff_l2(28,49)=  2.1358585358e+00
       coeff_l2(28,50)= -7.6064252853e-01
       coeff_l2(28,51)= -1.6332591772e+00
       coeff_l2(28,52)= -1.8445834517e-01
       coeff_l2(28,53)=  1.7793668509e+00
       coeff_l2(28,54)= -7.2526031733e-01
       coeff_l2(28,55)=  6.3596799970e-02
       coeff_l2(29,1)=  3.0191698670e-01
       coeff_l2(29,2)=  2.3283188343e+00
       coeff_l2(29,3)=  3.6646120548e+00
       coeff_l2(29,4)= -1.7840746641e+00
       coeff_l2(29,5)= -1.1041065454e+00
       coeff_l2(29,6)=  1.0432323068e-01
       coeff_l2(29,7)=  2.2152628899e+00
       coeff_l2(29,8)=  3.0638462305e-01
       coeff_l2(29,9)=  4.7546145320e-01
       coeff_l2(29,10)= -3.0180308968e-02
       coeff_l2(29,11)=  2.2882449627e+00
       coeff_l2(29,12)= -2.1254475415e-01
       coeff_l2(29,13)=  3.4849343300e+00
       coeff_l2(29,14)=  1.8724648952e+00
       coeff_l2(29,15)=  2.0977577567e-01
       coeff_l2(29,16)= -2.3618173599e-01
       coeff_l2(29,17)=  1.0994646549e+00
       coeff_l2(29,18)=  1.8550826311e+00
       coeff_l2(29,19)= -8.2576489449e-01
       coeff_l2(29,20)= -7.7452178001e+00
       coeff_l2(29,21)=  1.1715413630e-01
       coeff_l2(29,22)=  1.0364117622e+00
       coeff_l2(29,23)= -7.0916825533e-01
       coeff_l2(29,24)=  6.5494321287e-02
       coeff_l2(29,25)= -2.3060226440e+00
       coeff_l2(29,26)= -4.2983636856e+00
       coeff_l2(29,27)= -4.3547260761e-01
       coeff_l2(29,28)= -4.2170462608e+00
       coeff_l2(29,29)= -6.6722559929e-01
       coeff_l2(29,30)=  7.2418600321e-01
       coeff_l2(29,31)=  2.2914412022e+00
       coeff_l2(29,32)=  1.6925728321e-01
       coeff_l2(29,33)= -1.5181597471e+00
       coeff_l2(29,34)= -4.4959265739e-02
       coeff_l2(29,35)=  5.8812880516e-01
       coeff_l2(29,36)=  9.2252979279e+00
       coeff_l2(29,37)=  3.8829836845e+00
       coeff_l2(29,38)= -1.3130619526e+00
       coeff_l2(29,39)= -4.6487072110e-01
       coeff_l2(29,40)= -3.6502179503e-01
       coeff_l2(29,41)=  7.5315392017e-01
       coeff_l2(29,42)= -1.3047050238e+00
       coeff_l2(29,43)= -4.2293578386e-01
       coeff_l2(29,44)=  3.1259812415e-02
       coeff_l2(29,45)= -1.3275773823e-01
       coeff_l2(29,46)= -2.5380783081e+00
       coeff_l2(29,47)= -9.1552191973e-01
       coeff_l2(29,48)=  3.2538683414e+00
       coeff_l2(29,49)=  1.8619089127e+00
       coeff_l2(29,50)= -9.8241400719e-01
       coeff_l2(29,51)= -4.2979064584e-01
       coeff_l2(29,52)=  1.4651495218e+00
       coeff_l2(29,53)=  1.2308709621e+00
       coeff_l2(29,54)=  2.8091135621e-01
       coeff_l2(29,55)= -1.3649414480e-01
       coeff_l2(30,1)=  6.5177243948e-01
       coeff_l2(30,2)=  1.2534081936e+00
       coeff_l2(30,3)=  6.5861207247e-01
       coeff_l2(30,4)= -1.5577324629e+00
       coeff_l2(30,5)=  4.1159844398e+00
       coeff_l2(30,6)= -1.7124572992e+00
       coeff_l2(30,7)=  6.3473296165e-01
       coeff_l2(30,8)= -1.1036634445e+00
       coeff_l2(30,9)=  2.6618492603e-01
       coeff_l2(30,10)= -8.2646143436e-01
       coeff_l2(30,11)=  1.4478551149e+00
       coeff_l2(30,12)=  1.9512282312e-01
       coeff_l2(30,13)= -4.7729668021e-01
       coeff_l2(30,14)=  3.5665004980e-03
       coeff_l2(30,15)= -7.1142077446e-02
       coeff_l2(30,16)=  6.3718283176e-01
       coeff_l2(30,17)=  1.4166578650e-01
       coeff_l2(30,18)=  1.1902266741e-01
       coeff_l2(30,19)=  3.5383710265e-01
       coeff_l2(30,20)= -1.8737430573e+00
       coeff_l2(30,21)= -1.8479177356e-01
       coeff_l2(30,22)=  5.6925421814e-04
       coeff_l2(30,23)= -3.8122862577e-01
       coeff_l2(30,24)=  1.7521933317e+00
       coeff_l2(30,25)= -1.4944435358e+00
       coeff_l2(30,26)= -2.7059395313e+00
       coeff_l2(30,27)= -1.0930551291e+00
       coeff_l2(30,28)=  1.8979252577e+00
       coeff_l2(30,29)= -4.8952615261e-01
       coeff_l2(30,30)= -2.4001547694e-01
       coeff_l2(30,31)=  5.5254369974e-01
       coeff_l2(30,32)=  7.4480527639e-01
       coeff_l2(30,33)=  1.0790405273e+00
       coeff_l2(30,34)=  3.3342954516e-01
       coeff_l2(30,35)= -3.1102663279e-01
       coeff_l2(30,36)= -8.2863754034e-01
       coeff_l2(30,37)=  1.9507875443e+00
       coeff_l2(30,38)= -7.1012943983e-01
       coeff_l2(30,39)=  2.4036759138e-01
       coeff_l2(30,40)= -2.4381986260e-01
       coeff_l2(30,41)= -8.0363802612e-02
       coeff_l2(30,42)= -1.3254526258e-01
       coeff_l2(30,43)=  1.4599020779e-01
       coeff_l2(30,44)=  1.7233930826e+00
       coeff_l2(30,45)= -3.8117885590e-01
       coeff_l2(30,46)= -5.6328135729e-01
       coeff_l2(30,47)= -3.7854394317e-01
       coeff_l2(30,48)=  4.9235311151e-01
       coeff_l2(30,49)= -3.6306473613e-01
       coeff_l2(30,50)= -5.4245460033e-01
       coeff_l2(30,51)= -1.1026255786e-01
       coeff_l2(30,52)=  8.0218952894e-01
       coeff_l2(30,53)=  1.5030888319e+00
       coeff_l2(30,54)= -8.2296021283e-02
       coeff_l2(30,55)= -5.0643688440e-01
       coeff_l2(31,1)=  3.0006557703e-01
       coeff_l2(31,2)=  1.7587991953e+00
       coeff_l2(31,3)=  2.0905494690e+00
       coeff_l2(31,4)= -6.1390525103e-01
       coeff_l2(31,5)=  1.2819949389e+00
       coeff_l2(31,6)= -2.0144930482e-01
       coeff_l2(31,7)=  2.1130895615e+00
       coeff_l2(31,8)=  7.3004937172e-01
       coeff_l2(31,9)=  5.5876106024e-01
       coeff_l2(31,10)= -4.5725904405e-02
       coeff_l2(31,11)=  2.8451153636e-01
       coeff_l2(31,12)= -4.1617561132e-02
       coeff_l2(31,13)=  1.1485297680e+00
       coeff_l2(31,14)=  1.3646548986e+00
       coeff_l2(31,15)=  4.2396166921e-01
       coeff_l2(31,16)=  4.0963774920e-01
       coeff_l2(31,17)=  3.1762143970e-01
       coeff_l2(31,18)=  2.1659077704e-01
       coeff_l2(31,19)= -2.9498896003e-01
       coeff_l2(31,20)= -4.3394026756e+00
       coeff_l2(31,21)=  1.2291149795e-01
       coeff_l2(31,22)=  3.5912299156e-01
       coeff_l2(31,23)=  3.4017229080e-01
       coeff_l2(31,24)=  2.9005506039e+00
       coeff_l2(31,25)= -2.1596484184e+00
       coeff_l2(31,26)= -1.9915980101e+00
       coeff_l2(31,27)=  7.8346455097e-01
       coeff_l2(31,28)= -2.7435691357e+00
       coeff_l2(31,29)=  1.5446455777e-01
       coeff_l2(31,30)=  8.6375206709e-01
       coeff_l2(31,31)=  5.0385540724e-01
       coeff_l2(31,32)= -7.6102179289e-01
       coeff_l2(31,33)=  4.6680353582e-02
       coeff_l2(31,34)= -9.5769470930e-01
       coeff_l2(31,35)= -1.4564931393e+00
       coeff_l2(31,36)=  3.9092416763e+00
       coeff_l2(31,37)=  1.1942315102e+00
       coeff_l2(31,38)= -4.4512668252e-01
       coeff_l2(31,39)=  1.4112346172e+00
       coeff_l2(31,40)= -1.7061382532e-01
       coeff_l2(31,41)= -9.0073309839e-02
       coeff_l2(31,42)=  1.9732190669e-01
       coeff_l2(31,43)=  4.8895069957e-01
       coeff_l2(31,44)= -1.9891615212e-01
       coeff_l2(31,45)= -2.3841026425e-01
       coeff_l2(31,46)=  3.1383526325e-01
       coeff_l2(31,47)= -1.1810446978e+00
       coeff_l2(31,48)=  1.0922396183e+00
       coeff_l2(31,49)=  4.6845811605e-01
       coeff_l2(31,50)= -6.7101973295e-01
       coeff_l2(31,51)= -9.1610044241e-01
       coeff_l2(31,52)=  7.6740604639e-01
       coeff_l2(31,53)=  1.8940908909e+00
       coeff_l2(31,54)= -6.3950079679e-01
       coeff_l2(31,55)=  2.6318773162e-03
       coeff_l2(32,1)= -7.0544266701e-01
       coeff_l2(32,2)=  8.5637867451e-01
       coeff_l2(32,3)=  1.2219033241e+00
       coeff_l2(32,4)= -7.3667871952e-01
       coeff_l2(32,5)=  3.1148221493e+00
       coeff_l2(32,6)= -1.4785780907e+00
       coeff_l2(32,7)= -1.5199564695e+00
       coeff_l2(32,8)=  2.9377856255e+00
       coeff_l2(32,9)=  3.1971637160e-02
       coeff_l2(32,10)=  7.9211384058e-01
       coeff_l2(32,11)=  2.6146163940e+00
       coeff_l2(32,12)= -1.4940327406e+00
       coeff_l2(32,13)= -5.2020591497e-01
       coeff_l2(32,14)=  1.2628251314e+00
       coeff_l2(32,15)=  1.6307063401e-02
       coeff_l2(32,16)=  9.3006253242e-01
       coeff_l2(32,17)=  5.9471112490e-01
       coeff_l2(32,18)=  6.6345000267e-01
       coeff_l2(32,19)=  4.4982650876e-01
       coeff_l2(32,20)= -5.9534754753e+00
       coeff_l2(32,21)= -1.7116021365e-02
       coeff_l2(32,22)= -1.5448150635e+00
       coeff_l2(32,23)= -7.6222032309e-01
       coeff_l2(32,24)= -2.1092355251e-01
       coeff_l2(32,25)= -1.0567780733e+00
       coeff_l2(32,26)=  2.3167781830e+00
       coeff_l2(32,27)= -2.0890805870e-02
       coeff_l2(32,28)=  4.1675215960e-01
       coeff_l2(32,29)= -9.5759677887e-01
       coeff_l2(32,30)= -1.1898169518e+00
       coeff_l2(32,31)= -2.1945656836e-01
       coeff_l2(32,32)= -6.6490453482e-01
       coeff_l2(32,33)=  6.7825341225e-01
       coeff_l2(32,34)=  1.3867995143e-01
       coeff_l2(32,35)=  3.3405646682e-01
       coeff_l2(32,36)=  1.7359253168e+00
       coeff_l2(32,37)=  4.5469483733e-01
       coeff_l2(32,38)= -2.1697270870e-01
       coeff_l2(32,39)=  5.9770607948e-01
       coeff_l2(32,40)=  2.2453267872e-01
       coeff_l2(32,41)= -8.9553397894e-01
       coeff_l2(32,42)= -1.3899895549e-01
       coeff_l2(32,43)=  1.7167206109e-01
       coeff_l2(32,44)= -7.6648600399e-02
       coeff_l2(32,45)= -5.6353098154e-01
       coeff_l2(32,46)= -1.0600148439e+00
       coeff_l2(32,47)=  1.2029525042e+00
       coeff_l2(32,48)=  2.4828255177e+00
       coeff_l2(32,49)= -2.5573122501e-01
       coeff_l2(32,50)= -2.6422977448e-01
       coeff_l2(32,51)= -1.1371616125e+00
       coeff_l2(32,52)= -1.3040512800e+00
       coeff_l2(32,53)=  1.3765034676e+00
       coeff_l2(32,54)= -9.4829328358e-02
       coeff_l2(32,55)=  5.5162531137e-01
       coeff_l2(33,1)=  2.7764034271e-01
       coeff_l2(33,2)= -9.3073558807e+00
       coeff_l2(33,3)=  4.4000291824e-01
       coeff_l2(33,4)=  2.6044356823e+00
       coeff_l2(33,5)=  3.7476050854e-01
       coeff_l2(33,6)=  1.2313805521e-02
       coeff_l2(33,7)=  3.0898490548e-01
       coeff_l2(33,8)=  1.9357594252e+00
       coeff_l2(33,9)= -6.8046230078e-01
       coeff_l2(33,10)=  3.2866540551e-01
       coeff_l2(33,11)= -7.0901978016e-01
       coeff_l2(33,12)=  8.5689038038e-01
       coeff_l2(33,13)= -1.2703956366e+00
       coeff_l2(33,14)= -2.0807352066e+00
       coeff_l2(33,15)= -9.8080821335e-02
       coeff_l2(33,16)=  7.5346177816e-01
       coeff_l2(33,17)=  6.7099910975e-01
       coeff_l2(33,18)= -6.0088783503e-01
       coeff_l2(33,19)=  8.7805831432e-01
       coeff_l2(33,20)=  1.2981944084e+01
       coeff_l2(33,21)= -8.0811494589e-01
       coeff_l2(33,22)= -7.9824209213e-01
       coeff_l2(33,23)= -1.5961310267e-01
       coeff_l2(33,24)= -2.2491034865e-01
       coeff_l2(33,25)=  7.4238109589e+00
       coeff_l2(33,26)= -1.4029964209e+00
       coeff_l2(33,27)=  3.5853517056e+00
       coeff_l2(33,28)=  2.3664367199e+00
       coeff_l2(33,29)= -2.5305017829e-01
       coeff_l2(33,30)=  3.2863143831e-02
       coeff_l2(33,31)= -1.9172836542e+00
       coeff_l2(33,32)=  6.7213153839e-01
       coeff_l2(33,33)= -2.1684134007e+00
       coeff_l2(33,34)=  1.7384020090e+00
       coeff_l2(33,35)=  4.1278429329e-02
       coeff_l2(33,36)= -1.4541345596e+01
       coeff_l2(33,37)= -5.7203155756e-01
       coeff_l2(33,38)=  3.1696334481e-01
       coeff_l2(33,39)= -6.2010622025e-01
       coeff_l2(33,40)= -1.6941412687e+00
       coeff_l2(33,41)= -2.5206825733e+00
       coeff_l2(33,42)=  1.0831025988e-01
       coeff_l2(33,43)=  2.0012433529e+00
       coeff_l2(33,44)= -8.8631170988e-01
       coeff_l2(33,45)= -4.1994407773e-02
       coeff_l2(33,46)=  1.8700267076e+00
       coeff_l2(33,47)=  1.5443943739e+00
       coeff_l2(33,48)= -1.7584837675e+00
       coeff_l2(33,49)= -1.6256468296e+00
       coeff_l2(33,50)= -1.8256125972e-02
       coeff_l2(33,51)=  6.5954744816e-01
       coeff_l2(33,52)=  1.8762767315e+00
       coeff_l2(33,53)= -1.1038706303e+00
       coeff_l2(33,54)=  3.2426255941e-01
       coeff_l2(33,55)= -1.2529631853e+00
       coeff_l2(34,1)=  3.2128443718e+00
       coeff_l2(34,2)= -1.1225847006e+00
       coeff_l2(34,3)= -1.5316122770e+00
       coeff_l2(34,4)=  1.8951033354e+00
       coeff_l2(34,5)=  2.3849854469e+00
       coeff_l2(34,6)=  7.9599827528e-01
       coeff_l2(34,7)=  2.0603303909e+00
       coeff_l2(34,8)=  4.4903893471e+00
       coeff_l2(34,9)= -1.1797215939e+00
       coeff_l2(34,10)=  6.1038994789e+00
       coeff_l2(34,11)= -3.4441151619e+00
       coeff_l2(34,12)= -4.6490734816e-01
       coeff_l2(34,13)= -1.5381754637e+00
       coeff_l2(34,14)= -3.1951910257e-01
       coeff_l2(34,15)= -1.9272562265e+00
       coeff_l2(34,16)=  1.9703311920e+00
       coeff_l2(34,17)=  2.4786797166e-01
       coeff_l2(34,18)= -1.2115823030e+00
       coeff_l2(34,19)=  6.7997139692e-01
       coeff_l2(34,20)= -2.8035819530e+00
       coeff_l2(34,21)=  1.2999202311e-01
       coeff_l2(34,22)=  2.2258849144e+00
       coeff_l2(34,23)= -3.4166005254e-01
       coeff_l2(34,24)=  6.7037525177e+00
       coeff_l2(34,25)=  1.0129213333e+00
       coeff_l2(34,26)= -1.8953380585e+00
       coeff_l2(34,27)=  9.6879774332e-01
       coeff_l2(34,28)=  1.8273546696e+00
       coeff_l2(34,29)=  1.7704118788e-01
       coeff_l2(34,30)=  1.5963852406e+00
       coeff_l2(34,31)=  1.7267948389e+00
       coeff_l2(34,32)= -5.3963649273e-01
       coeff_l2(34,33)= -6.1826306581e-01
       coeff_l2(34,34)= -8.5070589557e-03
       coeff_l2(34,35)= -1.6801850796e+00
       coeff_l2(34,36)=  2.5089156628e+00
       coeff_l2(34,37)= -6.8587476015e-01
       coeff_l2(34,38)=  6.0484457016e-01
       coeff_l2(34,39)= -5.2693027258e-01
       coeff_l2(34,40)= -3.3373507857e-01
       coeff_l2(34,41)= -3.1707987785e+00
       coeff_l2(34,42)=  1.4691461325e+00
       coeff_l2(34,43)=  1.4377404451e+00
       coeff_l2(34,44)=  5.2182106972e+00
       coeff_l2(34,45)=  6.4533644915e-01
       coeff_l2(34,46)= -3.5084024072e-01
       coeff_l2(34,47)=  3.3355715871e-01
       coeff_l2(34,48)=  3.0270937085e-01
       coeff_l2(34,49)= -5.5137115717e-01
       coeff_l2(34,50)= -1.0780392885e+00
       coeff_l2(34,51)= -9.7488917410e-02
       coeff_l2(34,52)=  4.3339719772e+00
       coeff_l2(34,53)= -8.2670044899e-01
       coeff_l2(34,54)= -3.7196934223e-01
       coeff_l2(34,55)= -1.5334383249e+00
       coeff_l2(35,1)=  3.0864882469e+00
       coeff_l2(35,2)= -1.5374637842e+00
       coeff_l2(35,3)= -1.8653993607e+00
       coeff_l2(35,4)=  1.2020969391e+00
       coeff_l2(35,5)=  3.5021896362e+00
       coeff_l2(35,6)=  2.6062798500e-01
       coeff_l2(35,7)=  1.9833618402e+00
       coeff_l2(35,8)=  4.5803241730e+00
       coeff_l2(35,9)= -1.3432976007e+00
       coeff_l2(35,10)=  6.3030591011e+00
       coeff_l2(35,11)= -3.8100204468e+00
       coeff_l2(35,12)= -6.3942056894e-01
       coeff_l2(35,13)= -1.5758017302e+00
       coeff_l2(35,14)= -3.8406533003e-01
       coeff_l2(35,15)= -2.1133930683e+00
       coeff_l2(35,16)=  1.8842613697e+00
       coeff_l2(35,17)=  9.3429923058e-01
       coeff_l2(35,18)= -6.7778927088e-01
       coeff_l2(35,19)=  6.9561678171e-01
       coeff_l2(35,20)= -2.3732342720e+00
       coeff_l2(35,21)= -1.7729702592e-01
       coeff_l2(35,22)=  1.6722086668e+00
       coeff_l2(35,23)= -9.6074722707e-02
       coeff_l2(35,24)=  6.4091081619e+00
       coeff_l2(35,25)=  8.5645914078e-01
       coeff_l2(35,26)= -1.8552945852e+00
       coeff_l2(35,27)=  1.2990207672e+00
       coeff_l2(35,28)=  8.6481708288e-01
       coeff_l2(35,29)= -8.6360126734e-02
       coeff_l2(35,30)=  1.6433742046e+00
       coeff_l2(35,31)=  1.3092516661e+00
       coeff_l2(35,32)= -1.3312551975e+00
       coeff_l2(35,33)= -8.3454895020e-01
       coeff_l2(35,34)= -1.1860217899e-02
       coeff_l2(35,35)= -1.7611653805e+00
       coeff_l2(35,36)=  2.2945070267e+00
       coeff_l2(35,37)= -9.8074918985e-01
       coeff_l2(35,38)=  5.8971816301e-01
       coeff_l2(35,39)= -6.2351453304e-01
       coeff_l2(35,40)= -3.2656469941e-01
       coeff_l2(35,41)= -2.8455243111e+00
       coeff_l2(35,42)=  1.2563351393e+00
       coeff_l2(35,43)=  8.8493293524e-01
       coeff_l2(35,44)=  5.2526311874e+00
       coeff_l2(35,45)=  5.8722132444e-01
       coeff_l2(35,46)= -3.3712244034e-01
       coeff_l2(35,47)=  4.6619826555e-01
       coeff_l2(35,48)=  2.5762617588e-01
       coeff_l2(35,49)= -5.6247198582e-01
       coeff_l2(35,50)= -1.1477961540e+00
       coeff_l2(35,51)= -2.1406637132e-01
       coeff_l2(35,52)=  4.9286699295e+00
       coeff_l2(35,53)= -9.6665358543e-01
       coeff_l2(35,54)= -7.7654987574e-01
       coeff_l2(35,55)= -1.4733831882e+00
!coeff layer 3
       coeff_l3(1,1)=  1.2871248722e+00
       coeff_l3(1,2)=  2.4038743973e+00
       coeff_l3(1,3)=  1.8620682955e+00
       coeff_l3(1,4)=  1.8394842148e+00
       coeff_l3(1,5)=  2.3490698338e+00
       coeff_l3(1,6)=  1.8633376360e+00
       coeff_l3(1,7)=  6.3958805799e-01
       coeff_l3(1,8)=  8.6147153378e-01
       coeff_l3(1,9)=  9.1989195347e-01
       coeff_l3(1,10)=  1.5815505981e+00
       coeff_l3(1,11)=  7.0819580555e-01
       coeff_l3(1,12)=  6.3815557957e-01
       coeff_l3(1,13)=  1.8905639648e+00
       coeff_l3(1,14)=  1.3490658998e+00
       coeff_l3(1,15)= -7.7209740877e-01
       coeff_l3(1,16)=  1.0513395071e-01
       coeff_l3(1,17)=  2.2968697548e-01
       coeff_l3(1,18)= -1.0258458853e+00
       coeff_l3(1,19)=  5.8467179537e-01
       coeff_l3(1,20)=  5.2078551054e-01
       coeff_l3(1,21)= -2.1807666868e-02
       coeff_l3(1,22)=  1.2077908516e+00
       coeff_l3(1,23)= -2.5070378184e-01
       coeff_l3(1,24)=  1.6167039871e+00
       coeff_l3(1,25)= -9.5451831818e-01
       coeff_l3(1,26)=  1.5883848667e+00
       coeff_l3(1,27)=  4.3578886986e-01
       coeff_l3(1,28)= -2.0105315745e-01
       coeff_l3(1,29)=  3.2700526714e-01
       coeff_l3(1,30)=  1.9848659635e-01
       coeff_l3(1,31)= -4.8080745339e-01
       coeff_l3(1,32)=  4.1223105788e-01
       coeff_l3(1,33)=  4.7644859552e-01
       coeff_l3(1,34)=  4.0848451853e-01
       coeff_l3(1,35)= -5.4143756628e-01
       coeff_l3(1,36)= -1.4286060333e+00
       coeff_l3(1,37)=  1.0483348370e+00
       coeff_l3(1,38)=  6.9544476271e-01
       coeff_l3(1,39)= -6.0968381166e-01
       coeff_l3(1,40)=  7.9432129860e-01
       coeff_l3(1,41)=  9.4564282894e-01
       coeff_l3(1,42)= -2.2933391854e-02
       coeff_l3(1,43)=  3.2019663602e-02
       coeff_l3(1,44)=  3.0959707499e-01
       coeff_l3(1,45)=  1.4378266335e+00
       coeff_l3(1,46)= -1.6162208319e+00
       coeff_l3(1,47)=  1.6383649111e+00
       coeff_l3(1,48)=  1.4956998825e+00
       coeff_l3(1,49)=  1.2246410847e+00
       coeff_l3(1,50)=  6.3865029812e-01
       coeff_l3(1,51)= -3.2573428750e-01
       coeff_l3(1,52)=  1.3568648100e+00
       coeff_l3(1,53)= -4.0857532620e-01
       coeff_l3(1,54)= -1.1407035589e+00
       coeff_l3(1,55)= -3.8462468982e-01
       coeff_l3(1,56)= -4.3826472759e-01
       coeff_l3(1,57)=  1.0230062008e+00
       coeff_l3(1,58)= -9.1141089797e-02
       coeff_l3(1,59)=  3.2453730702e-01
       coeff_l3(1,60)= -1.4372227192e+00
       coeff_l3(1,61)=  7.8545039892e-01
       coeff_l3(1,62)= -5.8433008194e-01
       coeff_l3(1,63)= -1.6947053671e+00
       coeff_l3(1,64)= -1.3129930496e+00
       coeff_l3(1,65)=  4.0395295620e-01
       coeff_l3(1,66)=  8.6900281906e-01
       coeff_l3(1,67)=  4.5637592673e-01
       coeff_l3(1,68)=  9.5253962278e-01
       coeff_l3(1,69)= -5.1549875736e-01
       coeff_l3(1,70)= -1.3280041516e-01
       coeff_l3(1,71)=  6.3929361105e-01
       coeff_l3(1,72)=  5.1591075957e-02
       coeff_l3(1,73)= -5.6120026112e-01
       coeff_l3(1,74)=  6.0269653797e-01
       coeff_l3(1,75)=  1.3581475019e+00
       coeff_l3(1,76)=  1.7072468996e-01
       coeff_l3(1,77)=  7.8030967712e-01
       coeff_l3(1,78)= -1.7683383822e-01
       coeff_l3(1,79)=  1.0669136047e+00
       coeff_l3(1,80)=  2.3121911287e-01
       coeff_l3(1,81)=  2.1415370703e-01
       coeff_l3(1,82)=  1.5407482386e+00
       coeff_l3(1,83)= -3.4850889444e-01
       coeff_l3(1,84)=  8.9835667610e-01
       coeff_l3(1,85)= -2.2235565186e+00
       coeff_l3(1,86)=  2.7138039470e-01
       coeff_l3(1,87)=  4.7789174318e-01
       coeff_l3(1,88)=  1.3634282351e+00
       coeff_l3(1,89)=  6.8493026495e-01
       coeff_l3(1,90)= -1.1890636384e-01
       coeff_l3(1,91)=  3.1857952476e-01
       coeff_l3(1,92)= -3.4807899594e-01
       coeff_l3(1,93)= -2.3998263478e-01
       coeff_l3(1,94)=  1.2446948290e+00
       coeff_l3(1,95)= -1.4737710953e+00
       coeff_l3(1,96)= -3.6929547787e-01
       coeff_l3(1,97)= -1.0155408382e+00
       coeff_l3(1,98)=  1.2674015760e+00
       coeff_l3(1,99)= -4.5636415482e-01
       coeff_l3(1,100)=  1.8844915926e-01
       coeff_l3(1,101)=  2.6823884249e-01
       coeff_l3(1,102)=  9.9628292024e-02
       coeff_l3(1,103)= -6.0213613510e-01
       coeff_l3(1,104)=  4.8630449176e-01
       coeff_l3(1,105)=  5.4859197140e-01
       coeff_l3(2,1)= -4.2835223675e-01
       coeff_l3(2,2)= -2.9001194239e-01
       coeff_l3(2,3)= -1.6177096963e-01
       coeff_l3(2,4)= -1.4178949594e-01
       coeff_l3(2,5)= -3.0000102520e-01
       coeff_l3(2,6)=  4.9062069505e-02
       coeff_l3(2,7)= -8.9248187840e-02
       coeff_l3(2,8)=  2.3803746700e-01
       coeff_l3(2,9)=  3.5109045506e+00
       coeff_l3(2,10)=  2.7871353626e+00
       coeff_l3(2,11)=  4.6070804596e+00
       coeff_l3(2,12)=  3.2385807037e+00
       coeff_l3(2,13)=  3.1713631153e+00
       coeff_l3(2,14)=  3.6138017178e+00
       coeff_l3(2,15)= -6.0926456004e-02
       coeff_l3(2,16)=  3.2562538981e-01
       coeff_l3(2,17)= -1.7243322730e-01
       coeff_l3(2,18)= -3.1041705608e-01
       coeff_l3(2,19)=  2.2437104583e-01
       coeff_l3(2,20)=  2.9607543349e-01
       coeff_l3(2,21)=  9.5009587705e-02
       coeff_l3(2,22)=  1.0810117237e-02
       coeff_l3(2,23)=  1.3797490299e-01
       coeff_l3(2,24)=  3.9827331901e-01
       coeff_l3(2,25)= -6.3395458460e-01
       coeff_l3(2,26)= -7.4809074402e-02
       coeff_l3(2,27)= -2.0247663558e-01
       coeff_l3(2,28)=  5.2297574282e-01
       coeff_l3(2,29)= -1.3117513061e-01
       coeff_l3(2,30)= -4.1322213411e-01
       coeff_l3(2,31)= -3.3636572957e-01
       coeff_l3(2,32)= -1.3219354441e-03
       coeff_l3(2,33)=  2.3596470058e-01
       coeff_l3(2,34)=  1.5825058520e-01
       coeff_l3(2,35)= -3.0580574647e-02
       coeff_l3(2,36)=  8.1718933582e-01
       coeff_l3(2,37)= -1.0257216692e+00
       coeff_l3(2,38)=  5.0776088238e-01
       coeff_l3(2,39)=  3.9860787988e-01
       coeff_l3(2,40)=  2.4111528695e-01
       coeff_l3(2,41)= -2.7415437624e-02
       coeff_l3(2,42)= -5.0142115355e-01
       coeff_l3(2,43)= -1.3256394863e-01
       coeff_l3(2,44)=  2.5417940691e-02
       coeff_l3(2,45)= -2.3791496456e-01
       coeff_l3(2,46)=  8.8281023502e-01
       coeff_l3(2,47)= -5.7094699144e-01
       coeff_l3(2,48)= -9.7462177277e-01
       coeff_l3(2,49)=  3.6021077633e-01
       coeff_l3(2,50)=  3.7532857060e-01
       coeff_l3(2,51)= -1.5302717686e-01
       coeff_l3(2,52)= -1.0836268961e-01
       coeff_l3(2,53)= -1.0928735882e-01
       coeff_l3(2,54)= -1.8259987235e-01
       coeff_l3(2,55)=  6.3481815159e-02
       coeff_l3(2,56)= -6.9138747454e-01
       coeff_l3(2,57)= -1.8861025572e-01
       coeff_l3(2,58)=  4.6403267980e-01
       coeff_l3(2,59)=  2.6195582747e-01
       coeff_l3(2,60)=  1.7439162731e-01
       coeff_l3(2,61)=  1.1367969513e+00
       coeff_l3(2,62)=  2.0960587263e-01
       coeff_l3(2,63)= -3.2426169515e-01
       coeff_l3(2,64)= -4.6599739790e-01
       coeff_l3(2,65)= -8.6887612939e-02
       coeff_l3(2,66)=  9.9512988329e-01
       coeff_l3(2,67)=  2.9656165838e-01
       coeff_l3(2,68)= -1.4164354801e+00
       coeff_l3(2,69)= -7.4022680521e-01
       coeff_l3(2,70)=  3.8191936910e-02
       coeff_l3(2,71)=  6.1043524742e-01
       coeff_l3(2,72)= -4.8531405628e-02
       coeff_l3(2,73)= -1.1232552528e+00
       coeff_l3(2,74)= -8.1837695837e-01
       coeff_l3(2,75)=  7.6582711935e-01
       coeff_l3(2,76)= -3.6346203089e-01
       coeff_l3(2,77)= -1.6521912813e-01
       coeff_l3(2,78)=  1.7738094926e-01
       coeff_l3(2,79)= -4.4885325432e-01
       coeff_l3(2,80)= -1.7411102355e-01
       coeff_l3(2,81)= -4.7072792053e-01
       coeff_l3(2,82)= -5.8838397264e-01
       coeff_l3(2,83)= -1.0638028383e+00
       coeff_l3(2,84)= -1.5935918689e-01
       coeff_l3(2,85)= -3.8840031624e-01
       coeff_l3(2,86)=  3.4202438593e-01
       coeff_l3(2,87)= -1.1419413239e-01
       coeff_l3(2,88)= -9.4827377796e-01
       coeff_l3(2,89)=  4.9106761813e-02
       coeff_l3(2,90)= -4.2184928060e-01
       coeff_l3(2,91)= -1.3056649268e-01
       coeff_l3(2,92)= -2.2261116505e+00
       coeff_l3(2,93)= -1.3977626562e+00
       coeff_l3(2,94)= -4.7195756435e-01
       coeff_l3(2,95)= -5.7840400934e-01
       coeff_l3(2,96)=  7.8257393837e-01
       coeff_l3(2,97)= -1.9965964556e+00
       coeff_l3(2,98)=  1.0912793875e+00
       coeff_l3(2,99)=  4.0349823236e-01
       coeff_l3(2,100)=  6.2454454601e-02
       coeff_l3(2,101)= -6.1420589685e-01
       coeff_l3(2,102)= -3.3214818686e-02
       coeff_l3(2,103)= -4.9023902416e-01
       coeff_l3(2,104)=  1.3768587112e+00
       coeff_l3(2,105)=  2.2939815521e+00
       coeff_l3(3,1)= -1.3889603615e+00
       coeff_l3(3,2)= -3.4525728226e-01
       coeff_l3(3,3)= -1.0637696832e-01
       coeff_l3(3,4)= -2.5133880973e-01
       coeff_l3(3,5)= -1.0803971291e+00
       coeff_l3(3,6)= -1.2121078968e+00
       coeff_l3(3,7)= -4.8087194562e-01
       coeff_l3(3,8)= -3.7393242121e-01
       coeff_l3(3,9)= -2.1894974709e+00
       coeff_l3(3,10)= -2.9366114140e+00
       coeff_l3(3,11)= -1.4190716743e+00
       coeff_l3(3,12)= -1.1702477932e+00
       coeff_l3(3,13)= -2.3493506908e+00
       coeff_l3(3,14)= -2.7304763794e+00
       coeff_l3(3,15)=  2.8229605407e-02
       coeff_l3(3,16)=  6.6373789310e-01
       coeff_l3(3,17)=  7.9925340414e-01
       coeff_l3(3,18)= -5.5139958858e-02
       coeff_l3(3,19)=  6.7412132025e-01
       coeff_l3(3,20)=  4.1205292940e-01
       coeff_l3(3,21)= -7.3157185316e-01
       coeff_l3(3,22)= -1.4421343803e-01
       coeff_l3(3,23)=  6.9327664375e-01
       coeff_l3(3,24)= -6.0105526447e-01
       coeff_l3(3,25)=  6.4494818449e-01
       coeff_l3(3,26)= -1.1523368359e+00
       coeff_l3(3,27)=  4.7254869342e-01
       coeff_l3(3,28)=  2.2635729611e-01
       coeff_l3(3,29)= -5.3347494453e-02
       coeff_l3(3,30)=  7.1922385693e-01
       coeff_l3(3,31)= -1.7773565650e-01
       coeff_l3(3,32)= -1.1039813608e-01
       coeff_l3(3,33)=  2.1014977247e-02
       coeff_l3(3,34)= -1.8644621372e+00
       coeff_l3(3,35)= -1.4788945913e+00
       coeff_l3(3,36)= -1.6606918573e+00
       coeff_l3(3,37)= -1.3543924093e+00
       coeff_l3(3,38)=  5.9772226959e-02
       coeff_l3(3,39)= -7.5851565599e-01
       coeff_l3(3,40)=  5.6159663200e-01
       coeff_l3(3,41)=  8.0663099885e-02
       coeff_l3(3,42)= -5.3294366598e-01
       coeff_l3(3,43)= -2.3239922523e-01
       coeff_l3(3,44)= -1.2988486886e-01
       coeff_l3(3,45)=  7.5328445435e-01
       coeff_l3(3,46)=  7.7299857140e-01
       coeff_l3(3,47)= -1.9638794661e-01
       coeff_l3(3,48)=  9.3717724085e-01
       coeff_l3(3,49)= -2.3381598294e-01
       coeff_l3(3,50)= -2.7052035928e-01
       coeff_l3(3,51)=  5.5081003904e-01
       coeff_l3(3,52)= -7.5049161911e-01
       coeff_l3(3,53)=  6.9456607103e-01
       coeff_l3(3,54)= -8.9368784428e-01
       coeff_l3(3,55)= -1.2134459019e+00
       coeff_l3(3,56)= -3.1903421879e-01
       coeff_l3(3,57)=  6.4915102720e-01
       coeff_l3(3,58)= -6.6648915410e-02
       coeff_l3(3,59)=  3.5347285867e-01
       coeff_l3(3,60)= -3.7971824408e-01
       coeff_l3(3,61)=  3.0717790127e-01
       coeff_l3(3,62)=  1.1261119843e+00
       coeff_l3(3,63)=  6.3098326325e-02
       coeff_l3(3,64)=  1.5291448832e+00
       coeff_l3(3,65)= -2.0774599910e-01
       coeff_l3(3,66)=  9.2395007610e-01
       coeff_l3(3,67)=  4.8235967755e-01
       coeff_l3(3,68)= -2.5756713748e-01
       coeff_l3(3,69)=  8.3349657059e-01
       coeff_l3(3,70)= -1.1451700926e+00
       coeff_l3(3,71)= -3.0206221342e-01
       coeff_l3(3,72)=  3.2481136918e-01
       coeff_l3(3,73)=  3.0840003490e-01
       coeff_l3(3,74)=  4.3652144074e-01
       coeff_l3(3,75)=  6.3185024261e-01
       coeff_l3(3,76)=  1.8584368229e+00
       coeff_l3(3,77)= -1.3210173845e+00
       coeff_l3(3,78)= -9.9023967981e-02
       coeff_l3(3,79)= -1.0665361881e+00
       coeff_l3(3,80)=  1.2794603407e-01
       coeff_l3(3,81)= -1.0193966627e+00
       coeff_l3(3,82)=  1.3509722948e+00
       coeff_l3(3,83)= -7.4023056030e-01
       coeff_l3(3,84)= -1.1770476103e+00
       coeff_l3(3,85)=  3.9856290817e-01
       coeff_l3(3,86)=  1.0339981318e+00
       coeff_l3(3,87)=  4.4857174158e-01
       coeff_l3(3,88)=  1.0326538086e+00
       coeff_l3(3,89)= -1.2695610523e-01
       coeff_l3(3,90)= -1.0548181534e+00
       coeff_l3(3,91)=  2.6192250848e-01
       coeff_l3(3,92)= -1.9525996447e+00
       coeff_l3(3,93)= -6.3866890967e-02
       coeff_l3(3,94)= -7.0126897097e-01
       coeff_l3(3,95)= -3.8623371720e-01
       coeff_l3(3,96)=  2.7117937803e-01
       coeff_l3(3,97)= -1.9797128439e+00
       coeff_l3(3,98)=  9.8809683323e-01
       coeff_l3(3,99)=  1.0367009640e+00
       coeff_l3(3,100)= -1.0007928610e+00
       coeff_l3(3,101)= -2.6499789953e-01
       coeff_l3(3,102)=  1.2015261650e+00
       coeff_l3(3,103)=  1.0728178173e-01
       coeff_l3(3,104)= -8.6812891066e-02
       coeff_l3(3,105)=  1.4432412386e+00
       coeff_l3(4,1)= -2.7761399746e+00
       coeff_l3(4,2)= -1.5841727257e+00
       coeff_l3(4,3)= -2.0044178963e+00
       coeff_l3(4,4)= -1.1487512589e+00
       coeff_l3(4,5)= -2.5145728588e+00
       coeff_l3(4,6)= -2.6151020527e+00
       coeff_l3(4,7)= -1.9505902529e+00
       coeff_l3(4,8)= -8.3106559515e-01
       coeff_l3(4,9)= -1.4944303036e+00
       coeff_l3(4,10)= -1.7530981302e+00
       coeff_l3(4,11)= -1.7793756723e+00
       coeff_l3(4,12)= -2.2453651428e+00
       coeff_l3(4,13)= -2.4726657867e+00
       coeff_l3(4,14)= -2.6203291416e+00
       coeff_l3(4,15)= -7.4830526114e-01
       coeff_l3(4,16)= -6.6240274906e-01
       coeff_l3(4,17)=  2.6540309191e-01
       coeff_l3(4,18)= -5.3115248680e-01
       coeff_l3(4,19)= -6.4843195677e-01
       coeff_l3(4,20)= -1.4018570185e+00
       coeff_l3(4,21)= -7.5966483355e-01
       coeff_l3(4,22)=  1.5310233831e+00
       coeff_l3(4,23)=  1.9418656826e-01
       coeff_l3(4,24)= -1.2461206317e-01
       coeff_l3(4,25)=  1.6809723377e+00
       coeff_l3(4,26)= -9.2054289579e-01
       coeff_l3(4,27)=  1.8754450083e+00
       coeff_l3(4,28)= -8.4671193361e-01
       coeff_l3(4,29)= -5.5446982384e-01
       coeff_l3(4,30)=  3.6604887247e-01
       coeff_l3(4,31)=  4.4605526328e-01
       coeff_l3(4,32)= -9.0631133318e-01
       coeff_l3(4,33)= -2.4273201823e-01
       coeff_l3(4,34)=  4.0246644616e-01
       coeff_l3(4,35)=  4.2975121737e-01
       coeff_l3(4,36)= -1.3647873402e+00
       coeff_l3(4,37)=  9.4404417276e-01
       coeff_l3(4,38)=  1.2156920135e-01
       coeff_l3(4,39)= -6.0184621811e-01
       coeff_l3(4,40)=  4.3538796902e-01
       coeff_l3(4,41)=  9.6281093359e-01
       coeff_l3(4,42)= -3.6146202683e-01
       coeff_l3(4,43)=  2.2460608184e-01
       coeff_l3(4,44)= -2.4303239584e-01
       coeff_l3(4,45)= -5.6666404009e-01
       coeff_l3(4,46)=  6.8487864733e-01
       coeff_l3(4,47)=  7.7810364962e-01
       coeff_l3(4,48)=  8.9065748453e-01
       coeff_l3(4,49)=  8.9215242863e-01
       coeff_l3(4,50)=  1.3286738098e-01
       coeff_l3(4,51)= -7.3819178343e-01
       coeff_l3(4,52)= -6.0888774693e-02
       coeff_l3(4,53)= -2.0763533190e-02
       coeff_l3(4,54)=  9.2055365443e-02
       coeff_l3(4,55)= -6.6799089313e-02
       coeff_l3(4,56)= -3.9331421256e-01
       coeff_l3(4,57)=  2.9855391383e-01
       coeff_l3(4,58)= -6.1206269264e-01
       coeff_l3(4,59)=  3.0632695556e-01
       coeff_l3(4,60)=  1.7282429338e-01
       coeff_l3(4,61)=  3.6981052160e-01
       coeff_l3(4,62)=  1.2791544199e-01
       coeff_l3(4,63)=  1.8400959969e+00
       coeff_l3(4,64)=  2.3241186142e-01
       coeff_l3(4,65)= -5.0507295132e-01
       coeff_l3(4,66)= -7.3509776592e-01
       coeff_l3(4,67)=  2.8464025259e-01
       coeff_l3(4,68)=  1.3424296379e+00
       coeff_l3(4,69)=  6.8303585052e-01
       coeff_l3(4,70)= -2.9560601711e-01
       coeff_l3(4,71)=  5.2520221472e-01
       coeff_l3(4,72)= -3.6086732149e-01
       coeff_l3(4,73)=  4.8626372218e-01
       coeff_l3(4,74)=  8.7421584129e-01
       coeff_l3(4,75)=  3.4844693542e-01
       coeff_l3(4,76)=  9.3499040604e-01
       coeff_l3(4,77)=  2.6050349697e-03
       coeff_l3(4,78)=  4.3778640032e-01
       coeff_l3(4,79)= -1.1529440433e-01
       coeff_l3(4,80)=  1.2163532972e+00
       coeff_l3(4,81)= -4.5704078674e-01
       coeff_l3(4,82)=  3.4901824594e-01
       coeff_l3(4,83)=  1.7363806963e+00
       coeff_l3(4,84)=  1.5475255251e+00
       coeff_l3(4,85)=  3.1316766143e-01
       coeff_l3(4,86)=  2.5043126941e-01
       coeff_l3(4,87)= -8.2463109493e-01
       coeff_l3(4,88)=  8.6534440517e-01
       coeff_l3(4,89)= -1.0822247267e+00
       coeff_l3(4,90)=  5.0551581383e-01
       coeff_l3(4,91)=  1.3047393560e+00
       coeff_l3(4,92)= -5.0545696169e-02
       coeff_l3(4,93)=  9.3322294950e-01
       coeff_l3(4,94)=  9.7114101052e-02
       coeff_l3(4,95)=  7.4619901180e-01
       coeff_l3(4,96)=  6.3446617126e-01
       coeff_l3(4,97)= -4.0798789263e-01
       coeff_l3(4,98)=  4.6067535877e-01
       coeff_l3(4,99)= -7.5543731451e-01
       coeff_l3(4,100)=  2.8052389622e-01
       coeff_l3(4,101)= -2.5875568390e-01
       coeff_l3(4,102)= -1.0550147295e+00
       coeff_l3(4,103)=  3.0265116692e-01
       coeff_l3(4,104)= -2.7899730206e-01
       coeff_l3(4,105)= -5.5184584856e-01
       coeff_l3(5,1)=  1.0977379084e+00
       coeff_l3(5,2)=  2.1091809273e+00
       coeff_l3(5,3)=  1.8330186605e+00
       coeff_l3(5,4)=  1.5589325428e+00
       coeff_l3(5,5)=  1.8084192276e+00
       coeff_l3(5,6)=  1.9971405268e+00
       coeff_l3(5,7)=  1.2406322956e+00
       coeff_l3(5,8)=  1.1395276785e+00
       coeff_l3(5,9)=  2.3833312988e+00
       coeff_l3(5,10)=  2.8714787960e+00
       coeff_l3(5,11)=  1.8771297932e+00
       coeff_l3(5,12)=  1.9197783470e+00
       coeff_l3(5,13)=  1.8727843761e+00
       coeff_l3(5,14)=  2.3133239746e+00
       coeff_l3(5,15)=  2.2468827665e-01
       coeff_l3(5,16)= -2.5975799188e-02
       coeff_l3(5,17)=  1.4792260528e-01
       coeff_l3(5,18)= -3.8753533363e-01
       coeff_l3(5,19)= -3.3726447821e-01
       coeff_l3(5,20)=  7.4710988998e-01
       coeff_l3(5,21)= -2.7555650473e-01
       coeff_l3(5,22)= -1.1532282829e+00
       coeff_l3(5,23)= -1.5308047831e-01
       coeff_l3(5,24)=  4.9115221947e-02
       coeff_l3(5,25)= -1.3286044598e+00
       coeff_l3(5,26)=  9.1599220037e-01
       coeff_l3(5,27)= -3.0197322369e-01
       coeff_l3(5,28)= -4.0640391409e-02
       coeff_l3(5,29)=  1.2997413874e+00
       coeff_l3(5,30)= -3.0102550983e-01
       coeff_l3(5,31)=  6.0378599912e-02
       coeff_l3(5,32)= -6.8540108204e-01
       coeff_l3(5,33)=  5.1910734177e-01
       coeff_l3(5,34)=  9.4937795401e-01
       coeff_l3(5,35)=  1.0869983435e+00
       coeff_l3(5,36)=  1.4233345985e+00
       coeff_l3(5,37)= -6.9928920269e-01
       coeff_l3(5,38)=  7.4260719121e-02
       coeff_l3(5,39)=  1.1803929694e-02
       coeff_l3(5,40)= -1.9177013636e-01
       coeff_l3(5,41)=  2.7874088287e-01
       coeff_l3(5,42)=  6.5461099148e-01
       coeff_l3(5,43)=  2.0010843873e-01
       coeff_l3(5,44)=  6.9639778137e-01
       coeff_l3(5,45)= -1.4845498838e-02
       coeff_l3(5,46)=  2.8278374672e-01
       coeff_l3(5,47)=  9.0769948438e-03
       coeff_l3(5,48)= -1.4419786930e+00
       coeff_l3(5,49)= -8.2910460234e-01
       coeff_l3(5,50)=  8.1178939342e-01
       coeff_l3(5,51)= -9.0087372065e-01
       coeff_l3(5,52)=  7.2125035524e-01
       coeff_l3(5,53)=  3.4210723639e-01
       coeff_l3(5,54)= -5.8413320780e-01
       coeff_l3(5,55)=  3.2442510128e-01
       coeff_l3(5,56)= -3.4465837479e-01
       coeff_l3(5,57)= -4.2366108298e-01
       coeff_l3(5,58)= -3.0847427249e-01
       coeff_l3(5,59)= -2.1516303718e-01
       coeff_l3(5,60)=  6.0687738657e-01
       coeff_l3(5,61)= -2.6608826593e-02
       coeff_l3(5,62)=  7.9158020020e-01
       coeff_l3(5,63)= -2.0365181565e-01
       coeff_l3(5,64)= -1.6937444210e+00
       coeff_l3(5,65)= -5.9675455093e-01
       coeff_l3(5,66)=  8.1912392378e-01
       coeff_l3(5,67)= -9.3611605465e-02
       coeff_l3(5,68)=  1.7758192122e-01
       coeff_l3(5,69)= -1.1393903494e+00
       coeff_l3(5,70)= -6.3862353563e-01
       coeff_l3(5,71)=  5.2546858788e-01
       coeff_l3(5,72)=  5.6124013662e-01
       coeff_l3(5,73)= -8.2232064009e-01
       coeff_l3(5,74)= -1.3964790106e+00
       coeff_l3(5,75)= -1.4695667028e+00
       coeff_l3(5,76)= -8.9142739773e-01
       coeff_l3(5,77)=  2.6434519887e-01
       coeff_l3(5,78)= -3.9928069711e-01
       coeff_l3(5,79)=  4.8304104805e-01
       coeff_l3(5,80)= -9.3710017204e-01
       coeff_l3(5,81)=  5.1016315818e-02
       coeff_l3(5,82)=  3.7440919876e-01
       coeff_l3(5,83)= -7.1664100885e-01
       coeff_l3(5,84)= -8.0024373531e-01
       coeff_l3(5,85)= -1.1737962961e+00
       coeff_l3(5,86)= -2.4130707979e-01
       coeff_l3(5,87)= -4.1549065709e-01
       coeff_l3(5,88)= -9.1645562649e-01
       coeff_l3(5,89)=  1.8896828592e-01
       coeff_l3(5,90)= -1.0366142988e+00
       coeff_l3(5,91)=  4.5343190432e-01
       coeff_l3(5,92)=  1.5241957903e+00
       coeff_l3(5,93)= -3.0379429460e-01
       coeff_l3(5,94)=  2.6574471593e-01
       coeff_l3(5,95)=  6.3095015287e-01
       coeff_l3(5,96)= -1.3550288677e+00
       coeff_l3(5,97)=  6.8419134617e-01
       coeff_l3(5,98)= -1.2155927420e+00
       coeff_l3(5,99)=  1.0353451967e+00
       coeff_l3(5,100)=  8.9914458990e-01
       coeff_l3(5,101)= -2.5550004840e-01
       coeff_l3(5,102)=  4.8877686262e-01
       coeff_l3(5,103)=  1.1864234209e+00
       coeff_l3(5,104)=  3.9568331838e-01
       coeff_l3(5,105)= -5.1956850290e-01
       coeff_l3(6,1)= -1.6706314087e+00
       coeff_l3(6,2)= -3.8909618855e+00
       coeff_l3(6,3)= -2.2974050045e+00
       coeff_l3(6,4)= -1.2481514215e+00
       coeff_l3(6,5)= -3.5837876797e+00
       coeff_l3(6,6)= -3.5942111015e+00
       coeff_l3(6,7)= -1.8804655075e+00
       coeff_l3(6,8)= -2.9515719414e+00
       coeff_l3(6,9)= -2.8392398357e+00
       coeff_l3(6,10)=  8.3122104406e-02
       coeff_l3(6,11)= -3.9859479666e-01
       coeff_l3(6,12)= -6.7495983839e-01
       coeff_l3(6,13)= -1.0224356651e+00
       coeff_l3(6,14)= -7.0289665461e-01
       coeff_l3(6,15)=  4.8861864209e-01
       coeff_l3(6,16)=  2.7249026299e-01
       coeff_l3(6,17)= -8.7243482471e-02
       coeff_l3(6,18)=  1.5932737291e-01
       coeff_l3(6,19)= -4.0050527453e-01
       coeff_l3(6,20)= -5.0149548054e-01
       coeff_l3(6,21)= -5.5472147465e-01
       coeff_l3(6,22)=  1.1819474399e-02
       coeff_l3(6,23)=  2.6813498139e-01
       coeff_l3(6,24)= -7.8008192778e-01
       coeff_l3(6,25)=  2.1485683918e+00
       coeff_l3(6,26)= -1.0848423243e+00
       coeff_l3(6,27)= -6.0558360815e-01
       coeff_l3(6,28)= -2.2706676275e-02
       coeff_l3(6,29)= -9.4067198038e-01
       coeff_l3(6,30)=  7.5881618261e-01
       coeff_l3(6,31)= -8.5751138628e-02
       coeff_l3(6,32)=  1.7609104514e-01
       coeff_l3(6,33)=  5.4562044144e-01
       coeff_l3(6,34)= -7.6090432703e-02
       coeff_l3(6,35)=  4.5298308134e-01
       coeff_l3(6,36)= -1.6490851343e-01
       coeff_l3(6,37)=  4.9417391419e-01
       coeff_l3(6,38)= -5.2792733908e-01
       coeff_l3(6,39)=  1.3601349294e-01
       coeff_l3(6,40)= -7.8178673983e-01
       coeff_l3(6,41)= -5.4373764992e-01
       coeff_l3(6,42)= -5.2013581991e-01
       coeff_l3(6,43)= -4.1928980500e-02
       coeff_l3(6,44)= -3.0222019553e-01
       coeff_l3(6,45)= -1.8470842838e+00
       coeff_l3(6,46)=  4.1988483071e-01
       coeff_l3(6,47)=  3.6334287375e-02
       coeff_l3(6,48)= -1.7265476286e-01
       coeff_l3(6,49)= -2.8620642424e-01
       coeff_l3(6,50)= -7.2641450167e-01
       coeff_l3(6,51)= -6.5928941965e-01
       coeff_l3(6,52)= -1.3299711943e+00
       coeff_l3(6,53)=  2.2270855308e-01
       coeff_l3(6,54)=  1.2370932102e+00
       coeff_l3(6,55)=  3.8324514031e-01
       coeff_l3(6,56)=  5.3237527609e-01
       coeff_l3(6,57)=  2.7300477028e-01
       coeff_l3(6,58)= -4.3289208412e-01
       coeff_l3(6,59)=  2.0956356823e-01
       coeff_l3(6,60)=  8.7033683062e-01
       coeff_l3(6,61)=  1.5111899376e+00
       coeff_l3(6,62)= -1.3099558353e+00
       coeff_l3(6,63)=  4.7580093145e-01
       coeff_l3(6,64)= -8.7526953220e-01
       coeff_l3(6,65)=  1.6694356501e-01
       coeff_l3(6,66)= -1.4835485220e+00
       coeff_l3(6,67)= -4.5939946175e-01
       coeff_l3(6,68)=  1.0569825768e-01
       coeff_l3(6,69)= -1.8272790313e-01
       coeff_l3(6,70)=  8.8000905514e-01
       coeff_l3(6,71)=  1.1744262278e-01
       coeff_l3(6,72)= -1.0749976635e+00
       coeff_l3(6,73)=  1.0039045811e+00
       coeff_l3(6,74)=  5.6703644991e-01
       coeff_l3(6,75)=  2.4490508437e-01
       coeff_l3(6,76)= -6.2767100334e-01
       coeff_l3(6,77)=  4.7057729959e-01
       coeff_l3(6,78)=  1.3381091356e+00
       coeff_l3(6,79)=  8.3944588900e-01
       coeff_l3(6,80)= -2.1890302002e-01
       coeff_l3(6,81)=  4.8894456029e-01
       coeff_l3(6,82)= -1.2943351269e+00
       coeff_l3(6,83)=  9.0808105469e-01
       coeff_l3(6,84)=  1.0390233994e+00
       coeff_l3(6,85)=  4.8852932453e-01
       coeff_l3(6,86)= -1.5363491774e+00
       coeff_l3(6,87)=  1.2451753765e-01
       coeff_l3(6,88)=  3.6804333329e-01
       coeff_l3(6,89)= -5.9923750162e-01
       coeff_l3(6,90)=  9.5999169350e-01
       coeff_l3(6,91)= -3.6401504278e-01
       coeff_l3(6,92)= -5.7492926717e-02
       coeff_l3(6,93)=  6.0694660060e-03
       coeff_l3(6,94)= -9.9436871707e-02
       coeff_l3(6,95)= -1.8614944816e-01
       coeff_l3(6,96)=  6.0924178362e-01
       coeff_l3(6,97)= -4.3827140331e-01
       coeff_l3(6,98)=  4.3143290281e-01
       coeff_l3(6,99)= -3.8039457798e-01
       coeff_l3(6,100)=  9.0576797724e-02
       coeff_l3(6,101)=  8.2533292472e-02
       coeff_l3(6,102)= -4.6828937531e-01
       coeff_l3(6,103)= -1.8365910649e-01
       coeff_l3(6,104)=  3.6141657829e-01
       coeff_l3(6,105)= -1.0672386885e+00
       coeff_l3(7,1)=  1.4128800631e+00
       coeff_l3(7,2)=  1.7582472563e+00
       coeff_l3(7,3)=  3.1467577815e-01
       coeff_l3(7,4)=  1.3495115936e-01
       coeff_l3(7,5)=  1.0181357861e+00
       coeff_l3(7,6)= -3.7091502547e-01
       coeff_l3(7,7)= -5.5119329691e-01
       coeff_l3(7,8)=  1.3584581017e-01
       coeff_l3(7,9)=  3.1988048553e+00
       coeff_l3(7,10)=  1.7772716284e+00
       coeff_l3(7,11)=  2.4002995491e+00
       coeff_l3(7,12)=  1.3176263571e+00
       coeff_l3(7,13)=  1.2576572895e+00
       coeff_l3(7,14)=  1.3824710846e+00
       coeff_l3(7,15)=  7.0070087910e-02
       coeff_l3(7,16)=  5.1655548811e-01
       coeff_l3(7,17)= -2.1416623890e-01
       coeff_l3(7,18)=  1.0933909565e-02
       coeff_l3(7,19)= -2.5191378593e-01
       coeff_l3(7,20)= -2.2790582478e-01
       coeff_l3(7,21)= -3.3045282960e-01
       coeff_l3(7,22)= -1.6488997638e-01
       coeff_l3(7,23)= -7.3369890451e-01
       coeff_l3(7,24)= -6.0316246748e-01
       coeff_l3(7,25)= -9.9116265774e-03
       coeff_l3(7,26)=  9.1399598122e-01
       coeff_l3(7,27)=  1.1823686361e+00
       coeff_l3(7,28)=  3.0756436288e-02
       coeff_l3(7,29)=  1.0030155182e+00
       coeff_l3(7,30)= -1.4708182216e-01
       coeff_l3(7,31)=  6.2318283319e-01
       coeff_l3(7,32)= -1.0142289400e+00
       coeff_l3(7,33)=  1.2207995653e+00
       coeff_l3(7,34)= -2.7116093040e-01
       coeff_l3(7,35)=  9.3587893248e-01
       coeff_l3(7,36)=  6.8410247564e-01
       coeff_l3(7,37)=  1.7598743737e-01
       coeff_l3(7,38)=  4.2389541864e-01
       coeff_l3(7,39)= -7.6478213072e-01
       coeff_l3(7,40)=  7.2151625156e-01
       coeff_l3(7,41)=  1.0047928095e+00
       coeff_l3(7,42)=  5.1066559553e-01
       coeff_l3(7,43)= -7.9554453492e-02
       coeff_l3(7,44)=  1.8733963370e-01
       coeff_l3(7,45)= -5.0338745117e-01
       coeff_l3(7,46)=  8.8935947418e-01
       coeff_l3(7,47)=  1.5261280537e-01
       coeff_l3(7,48)= -1.7231170833e-01
       coeff_l3(7,49)= -2.3727874756e+00
       coeff_l3(7,50)=  4.7974738479e-01
       coeff_l3(7,51)= -9.2785435915e-01
       coeff_l3(7,52)=  9.2654265463e-03
       coeff_l3(7,53)=  8.6520916224e-01
       coeff_l3(7,54)= -1.0361732244e+00
       coeff_l3(7,55)= -5.6397217512e-01
       coeff_l3(7,56)=  2.2217750549e-01
       coeff_l3(7,57)= -1.0078618526e+00
       coeff_l3(7,58)=  3.8819977641e-01
       coeff_l3(7,59)=  1.1780397892e+00
       coeff_l3(7,60)=  1.2077438831e+00
       coeff_l3(7,61)= -1.0536491871e+00
       coeff_l3(7,62)=  1.1718020439e+00
       coeff_l3(7,63)=  1.8527019024e-01
       coeff_l3(7,64)= -1.0987368822e+00
       coeff_l3(7,65)= -1.6087820530e+00
       coeff_l3(7,66)=  3.2924264669e-01
       coeff_l3(7,67)= -2.0213451982e-01
       coeff_l3(7,68)=  3.9960506558e-01
       coeff_l3(7,69)=  1.5108111501e-01
       coeff_l3(7,70)= -1.1258255243e+00
       coeff_l3(7,71)= -2.1196040511e-01
       coeff_l3(7,72)=  5.7336688042e-01
       coeff_l3(7,73)= -2.2292691469e-01
       coeff_l3(7,74)= -1.1429377794e+00
       coeff_l3(7,75)= -8.1534588337e-01
       coeff_l3(7,76)= -2.7743712068e-01
       coeff_l3(7,77)=  1.1020821333e+00
       coeff_l3(7,78)= -4.5211985707e-01
       coeff_l3(7,79)=  6.5879350901e-01
       coeff_l3(7,80)=  7.1600154042e-02
       coeff_l3(7,81)= -7.3469620943e-01
       coeff_l3(7,82)= -8.0698929727e-02
       coeff_l3(7,83)= -1.3451900482e+00
       coeff_l3(7,84)= -1.0091981888e+00
       coeff_l3(7,85)= -3.9140504599e-01
       coeff_l3(7,86)= -9.5561552048e-01
       coeff_l3(7,87)= -1.1712836027e+00
       coeff_l3(7,88)=  1.9428730011e-01
       coeff_l3(7,89)=  1.1680574417e+00
       coeff_l3(7,90)= -5.0462466478e-01
       coeff_l3(7,91)=  5.8133441210e-01
       coeff_l3(7,92)=  1.1011935472e+00
       coeff_l3(7,93)=  6.1819779873e-01
       coeff_l3(7,94)=  6.6153550148e-01
       coeff_l3(7,95)=  1.5611410141e+00
       coeff_l3(7,96)= -1.8751236200e+00
       coeff_l3(7,97)=  1.2413074970e+00
       coeff_l3(7,98)= -7.5267511606e-01
       coeff_l3(7,99)= -6.8748041987e-02
       coeff_l3(7,100)=  1.1084192991e+00
       coeff_l3(7,101)=  7.8894788027e-01
       coeff_l3(7,102)= -6.7153662443e-01
       coeff_l3(7,103)=  1.1619933844e+00
       coeff_l3(7,104)=  5.3952958435e-02
       coeff_l3(7,105)= -6.8899017572e-01
       coeff_l3(8,1)=  4.3448349833e-01
       coeff_l3(8,2)= -1.5521810055e+00
       coeff_l3(8,3)= -5.8948588371e-01
       coeff_l3(8,4)= -6.9509941339e-01
       coeff_l3(8,5)= -1.2758471966e+00
       coeff_l3(8,6)=  1.2125042677e+00
       coeff_l3(8,7)= -4.1261333972e-02
       coeff_l3(8,8)=  7.8200656176e-01
       coeff_l3(8,9)=  1.0240930319e+00
       coeff_l3(8,10)= -2.1891562939e+00
       coeff_l3(8,11)=  8.2094979286e-01
       coeff_l3(8,12)=  6.3478976488e-01
       coeff_l3(8,13)= -1.0584949255e+00
       coeff_l3(8,14)=  2.2857458889e-01
       coeff_l3(8,15)= -1.0381663218e-02
       coeff_l3(8,16)=  5.4424166679e-01
       coeff_l3(8,17)= -6.0532617569e-01
       coeff_l3(8,18)=  2.1325601265e-02
       coeff_l3(8,19)=  2.6599977165e-02
       coeff_l3(8,20)= -6.5823894739e-01
       coeff_l3(8,21)=  6.2838423252e-01
       coeff_l3(8,22)=  2.7111238241e-01
       coeff_l3(8,23)=  8.4981638193e-01
       coeff_l3(8,24)= -2.2493457794e+00
       coeff_l3(8,25)=  2.0593197346e+00
       coeff_l3(8,26)= -1.9393523932e+00
       coeff_l3(8,27)=  3.7726715207e-01
       coeff_l3(8,28)= -9.2873394489e-02
       coeff_l3(8,29)= -1.4879983664e+00
       coeff_l3(8,30)= -1.3936349154e+00
       coeff_l3(8,31)=  5.5342262983e-01
       coeff_l3(8,32)= -1.4348289967e+00
       coeff_l3(8,33)= -1.0983982086e+00
       coeff_l3(8,34)= -1.5615901947e+00
       coeff_l3(8,35)=  4.9416229129e-01
       coeff_l3(8,36)=  9.4099640846e-01
       coeff_l3(8,37)= -8.7446969748e-01
       coeff_l3(8,38)= -6.9989317656e-01
       coeff_l3(8,39)=  4.2371660471e-01
       coeff_l3(8,40)= -6.1583632231e-01
       coeff_l3(8,41)= -1.3698967695e+00
       coeff_l3(8,42)= -1.1288033724e+00
       coeff_l3(8,43)= -6.3358861208e-01
       coeff_l3(8,44)= -2.8265741467e-01
       coeff_l3(8,45)= -8.5331594944e-01
       coeff_l3(8,46)=  2.1966316700e+00
       coeff_l3(8,47)= -1.8233611584e+00
       coeff_l3(8,48)= -7.2289764881e-01
       coeff_l3(8,49)= -5.8166986704e-01
       coeff_l3(8,50)= -2.4877750874e+00
       coeff_l3(8,51)=  2.1621920168e-01
       coeff_l3(8,52)= -1.1118682623e+00
       coeff_l3(8,53)= -1.1270466447e-01
       coeff_l3(8,54)=  1.2489764690e+00
       coeff_l3(8,55)=  5.8011126518e-01
       coeff_l3(8,56)=  6.5444552898e-01
       coeff_l3(8,57)= -2.0676329136e+00
       coeff_l3(8,58)=  1.1727248430e+00
       coeff_l3(8,59)=  2.8176981211e-01
       coeff_l3(8,60)=  1.3507585526e+00
       coeff_l3(8,61)= -1.0343873501e-01
       coeff_l3(8,62)=  3.2953202724e-01
       coeff_l3(8,63)=  1.8007056713e+00
       coeff_l3(8,64)=  1.1643006802e+00
       coeff_l3(8,65)=  3.3659455180e-01
       coeff_l3(8,66)= -1.3788179159e+00
       coeff_l3(8,67)=  8.7696605921e-01
       coeff_l3(8,68)= -2.2057445049e+00
       coeff_l3(8,69)=  7.9624241590e-01
       coeff_l3(8,70)=  1.9932570457e+00
       coeff_l3(8,71)=  6.7372217774e-02
       coeff_l3(8,72)= -7.0870494843e-01
       coeff_l3(8,73)= -7.2200852633e-01
       coeff_l3(8,74)= -7.1759390831e-01
       coeff_l3(8,75)=  8.7568056583e-01
       coeff_l3(8,76)=  5.7897275686e-01
       coeff_l3(8,77)= -2.4039268494e+00
       coeff_l3(8,78)= -2.4226242304e-01
       coeff_l3(8,79)= -2.6875622272e+00
       coeff_l3(8,80)= -1.8090787530e-01
       coeff_l3(8,81)= -5.0164788961e-01
       coeff_l3(8,82)= -1.1864094734e+00
       coeff_l3(8,83)=  5.1859092712e-01
       coeff_l3(8,84)= -4.1826501489e-02
       coeff_l3(8,85)=  2.3951580524e+00
       coeff_l3(8,86)=  8.8627713919e-01
       coeff_l3(8,87)= -9.3159925938e-01
       coeff_l3(8,88)= -1.9040130377e+00
       coeff_l3(8,89)= -2.2571797669e-01
       coeff_l3(8,90)=  1.4834121466e+00
       coeff_l3(8,91)= -8.2491153479e-01
       coeff_l3(8,92)= -3.7787753344e-01
       coeff_l3(8,93)= -4.3597492576e-01
       coeff_l3(8,94)= -2.5673761368e+00
       coeff_l3(8,95)=  1.0529813766e+00
       coeff_l3(8,96)=  2.4828453064e+00
       coeff_l3(8,97)= -1.4977896214e+00
       coeff_l3(8,98)=  2.2373162210e-01
       coeff_l3(8,99)= -4.6663060784e-01
       coeff_l3(8,100)= -1.7296837568e+00
       coeff_l3(8,101)= -5.1144379377e-01
       coeff_l3(8,102)= -4.3243041635e-01
       coeff_l3(8,103)= -8.2561475039e-01
       coeff_l3(8,104)=  5.8631861210e-01
       coeff_l3(8,105)=  1.7315765619e+00
       coeff_l3(9,1)= -4.8635099083e-02
       coeff_l3(9,2)= -1.4722515345e+00
       coeff_l3(9,3)= -4.6659222245e-01
       coeff_l3(9,4)= -5.7906758785e-01
       coeff_l3(9,5)= -1.0420215130e+00
       coeff_l3(9,6)= -2.4378967285e-01
       coeff_l3(9,7)=  3.2000559568e-01
       coeff_l3(9,8)=  4.1987630725e-01
       coeff_l3(9,9)= -4.6764418483e-01
       coeff_l3(9,10)= -2.7232587337e+00
       coeff_l3(9,11)= -1.6036753654e+00
       coeff_l3(9,12)= -1.4913510084e+00
       coeff_l3(9,13)= -1.3732358217e+00
       coeff_l3(9,14)= -1.4260821342e+00
       coeff_l3(9,15)=  3.2339894772e-01
       coeff_l3(9,16)=  7.8127689660e-02
       coeff_l3(9,17)= -1.0357763618e-01
       coeff_l3(9,18)= -4.5598205179e-02
       coeff_l3(9,19)=  5.3942072392e-01
       coeff_l3(9,20)= -1.7316760123e-01
       coeff_l3(9,21)=  5.2490115166e-01
       coeff_l3(9,22)=  1.1816264391e+00
       coeff_l3(9,23)=  3.1638863683e-01
       coeff_l3(9,24)= -1.5172898769e+00
       coeff_l3(9,25)=  4.0486392379e-01
       coeff_l3(9,26)= -7.5941777229e-01
       coeff_l3(9,27)=  5.6446981430e-01
       coeff_l3(9,28)=  3.6598464847e-01
       coeff_l3(9,29)= -9.6540844440e-01
       coeff_l3(9,30)= -5.5888658762e-01
       coeff_l3(9,31)=  3.5434588790e-01
       coeff_l3(9,32)= -6.1354893446e-01
       coeff_l3(9,33)= -3.6123961210e-01
       coeff_l3(9,34)= -1.1689177752e+00
       coeff_l3(9,35)=  1.1304268241e-01
       coeff_l3(9,36)=  1.6903570294e-01
       coeff_l3(9,37)= -5.0522339344e-01
       coeff_l3(9,38)= -1.1875565350e-01
       coeff_l3(9,39)= -3.0913960934e-01
       coeff_l3(9,40)= -1.0125558376e+00
       coeff_l3(9,41)= -7.1679109335e-01
       coeff_l3(9,42)= -4.8357573152e-01
       coeff_l3(9,43)=  1.8469759822e-01
       coeff_l3(9,44)= -6.1664290726e-02
       coeff_l3(9,45)= -6.1825573444e-01
       coeff_l3(9,46)=  5.3917467594e-01
       coeff_l3(9,47)= -2.3522669077e-01
       coeff_l3(9,48)=  6.7198318243e-01
       coeff_l3(9,49)= -1.2622565031e+00
       coeff_l3(9,50)= -1.4866985083e+00
       coeff_l3(9,51)=  6.1692867428e-02
       coeff_l3(9,52)=  3.4672117233e-01
       coeff_l3(9,53)= -7.3585963249e-01
       coeff_l3(9,54)=  1.2687009573e-01
       coeff_l3(9,55)=  1.8434117734e-01
       coeff_l3(9,56)=  2.9672271013e-01
       coeff_l3(9,57)= -9.5685040951e-01
       coeff_l3(9,58)=  6.7150318623e-01
       coeff_l3(9,59)=  7.8917026520e-01
       coeff_l3(9,60)=  1.0884268582e-01
       coeff_l3(9,61)= -6.0810643435e-01
       coeff_l3(9,62)=  6.1873328686e-01
       coeff_l3(9,63)= -3.2339688390e-02
       coeff_l3(9,64)=  1.2169445753e+00
       coeff_l3(9,65)= -8.0693370104e-01
       coeff_l3(9,66)= -3.8546350598e-01
       coeff_l3(9,67)=  4.6417391300e-01
       coeff_l3(9,68)= -5.6100821495e-01
       coeff_l3(9,69)=  8.7137156725e-01
       coeff_l3(9,70)=  6.3098949194e-01
       coeff_l3(9,71)=  8.6290419102e-02
       coeff_l3(9,72)=  5.8301579952e-01
       coeff_l3(9,73)=  8.8260844350e-02
       coeff_l3(9,74)= -1.4004629850e-01
       coeff_l3(9,75)=  6.0414266586e-01
       coeff_l3(9,76)=  8.1206172705e-01
       coeff_l3(9,77)= -1.1401901245e+00
       coeff_l3(9,78)= -1.1730705500e+00
       coeff_l3(9,79)= -2.0105605125e+00
       coeff_l3(9,80)=  4.0914991498e-01
       coeff_l3(9,81)= -5.8825343847e-01
       coeff_l3(9,82)=  7.2216933966e-01
       coeff_l3(9,83)=  6.2518012524e-01
       coeff_l3(9,84)= -1.7188549042e-01
       coeff_l3(9,85)=  1.1725083590e+00
       coeff_l3(9,86)=  5.8841264248e-01
       coeff_l3(9,87)= -1.8773597479e-01
       coeff_l3(9,88)= -1.6484096646e-01
       coeff_l3(9,89)=  7.9542535543e-01
       coeff_l3(9,90)=  5.1141226292e-01
       coeff_l3(9,91)=  2.3029039800e-01
       coeff_l3(9,92)= -9.1037100554e-01
       coeff_l3(9,93)= -4.6425551176e-01
       coeff_l3(9,94)= -1.2789613008e+00
       coeff_l3(9,95)=  6.6166377068e-01
       coeff_l3(9,96)=  1.3027879000e+00
       coeff_l3(9,97)= -1.7259196043e+00
       coeff_l3(9,98)=  9.2571318150e-01
       coeff_l3(9,99)= -1.0227023363e+00
       coeff_l3(9,100)= -3.5769787431e-01
       coeff_l3(9,101)=  1.0605239123e-01
       coeff_l3(9,102)= -2.6456353068e-01
       coeff_l3(9,103)= -9.6514117718e-01
       coeff_l3(9,104)=  1.3145657778e+00
       coeff_l3(9,105)=  1.9700114727e+00
       coeff_l3(10,1)= -6.2024986744e-01
       coeff_l3(10,2)= -1.9376844168e+00
       coeff_l3(10,3)= -5.3601968288e-01
       coeff_l3(10,4)= -3.6200928688e-01
       coeff_l3(10,5)= -2.1032028198e+00
       coeff_l3(10,6)= -4.5990083367e-02
       coeff_l3(10,7)= -1.9107436761e-02
       coeff_l3(10,8)= -1.3175203800e+00
       coeff_l3(10,9)= -1.9302129745e+00
       coeff_l3(10,10)= -4.5991808176e-01
       coeff_l3(10,11)=  7.5580644608e-01
       coeff_l3(10,12)= -7.6181602478e-01
       coeff_l3(10,13)=  5.5860131979e-01
       coeff_l3(10,14)=  3.5222053528e-01
       coeff_l3(10,15)= -2.0545585155e+00
       coeff_l3(10,16)=  5.8607387543e-01
       coeff_l3(10,17)= -3.2905337214e-01
       coeff_l3(10,18)=  2.1171803772e-01
       coeff_l3(10,19)=  4.4434541464e-01
       coeff_l3(10,20)= -7.2419053316e-01
       coeff_l3(10,21)=  4.4115942717e-01
       coeff_l3(10,22)=  4.8964187503e-01
       coeff_l3(10,23)=  1.4802063704e+00
       coeff_l3(10,24)= -1.2691538334e+00
       coeff_l3(10,25)=  1.2584381104e+00
       coeff_l3(10,26)= -3.3251628876e+00
       coeff_l3(10,27)= -9.3563950062e-01
       coeff_l3(10,28)=  7.5119072199e-01
       coeff_l3(10,29)= -1.2531998158e+00
       coeff_l3(10,30)= -9.3774646521e-01
       coeff_l3(10,31)= -1.3571350276e-01
       coeff_l3(10,32)=  1.3015619516e+00
       coeff_l3(10,33)= -2.5685778260e-01
       coeff_l3(10,34)=  2.6166851521e+00
       coeff_l3(10,35)=  7.6014667749e-01
       coeff_l3(10,36)=  7.8008925915e-01
       coeff_l3(10,37)=  9.8986625671e-01
       coeff_l3(10,38)=  4.7467970848e-01
       coeff_l3(10,39)=  1.2101081610e+00
       coeff_l3(10,40)= -1.0045769215e+00
       coeff_l3(10,41)= -4.9496781826e-01
       coeff_l3(10,42)= -1.0490225554e+00
       coeff_l3(10,43)=  1.3392835855e-01
       coeff_l3(10,44)= -3.9909917861e-02
       coeff_l3(10,45)= -8.5057646036e-01
       coeff_l3(10,46)= -9.0321940184e-01
       coeff_l3(10,47)=  9.5332309604e-02
       coeff_l3(10,48)=  2.3032346368e-01
       coeff_l3(10,49)=  7.9092495143e-03
       coeff_l3(10,50)=  6.2489658594e-01
       coeff_l3(10,51)=  1.3691401482e+00
       coeff_l3(10,52)= -1.0375273228e+00
       coeff_l3(10,53)= -2.7160850167e-01
       coeff_l3(10,54)=  2.5814361572e+00
       coeff_l3(10,55)= -1.4374006987e+00
       coeff_l3(10,56)= -2.5361232758e+00
       coeff_l3(10,57)=  7.2734475136e-01
       coeff_l3(10,58)= -2.7964303493e+00
       coeff_l3(10,59)= -8.6981177330e-01
       coeff_l3(10,60)=  2.3707796633e-01
       coeff_l3(10,61)=  2.0439155102e+00
       coeff_l3(10,62)= -1.1749213934e+00
       coeff_l3(10,63)= -2.5758340955e-01
       coeff_l3(10,64)=  6.2343442440e-01
       coeff_l3(10,65)=  1.7408201694e+00
       coeff_l3(10,66)= -2.7491486073e+00
       coeff_l3(10,67)= -1.0833536088e-01
       coeff_l3(10,68)=  1.9598956108e+00
       coeff_l3(10,69)=  8.3794391155e-01
       coeff_l3(10,70)=  1.3277769089e+00
       coeff_l3(10,71)= -1.2709648609e+00
       coeff_l3(10,72)=  1.4717195928e-01
       coeff_l3(10,73)= -2.0748372078e+00
       coeff_l3(10,74)=  1.6404391527e+00
       coeff_l3(10,75)=  7.6126068830e-01
       coeff_l3(10,76)= -7.4102485180e-01
       coeff_l3(10,77)= -9.8565256596e-01
       coeff_l3(10,78)= -2.1865457296e-02
       coeff_l3(10,79)= -8.0703991652e-01
       coeff_l3(10,80)= -9.9531263113e-01
       coeff_l3(10,81)=  2.8880655766e+00
       coeff_l3(10,82)= -1.9662326574e+00
       coeff_l3(10,83)=  1.4345320463e+00
       coeff_l3(10,84)= -1.0220386982e+00
       coeff_l3(10,85)=  2.0316433907e+00
       coeff_l3(10,86)= -1.3556272984e+00
       coeff_l3(10,87)= -1.0114401579e+00
       coeff_l3(10,88)= -1.3965350389e+00
       coeff_l3(10,89)=  7.8613865376e-01
       coeff_l3(10,90)=  1.1158300638e+00
       coeff_l3(10,91)= -3.4819245338e+00
       coeff_l3(10,92)= -1.9594184160e+00
       coeff_l3(10,93)=  2.9071068764e-01
       coeff_l3(10,94)= -3.5246345401e-01
       coeff_l3(10,95)= -1.5547422171e+00
       coeff_l3(10,96)=  2.2109324932e+00
       coeff_l3(10,97)= -6.0868489742e-01
       coeff_l3(10,98)=  3.6200755835e-01
       coeff_l3(10,99)=  3.4880745411e-01
       coeff_l3(10,100)= -2.0651578903e+00
       coeff_l3(10,101)=  1.0458388329e+00
       coeff_l3(10,102)=  1.1447225809e+00
       coeff_l3(10,103)= -2.0975577831e+00
       coeff_l3(10,104)= -2.1354588866e-01
       coeff_l3(10,105)=  5.9980183840e-01
       coeff_l3(11,1)=  9.0734589100e-01
       coeff_l3(11,2)=  9.0678179264e-01
       coeff_l3(11,3)=  2.6705271006e-01
       coeff_l3(11,4)=  7.2198134661e-01
       coeff_l3(11,5)=  3.8989365101e-01
       coeff_l3(11,6)=  4.1238570213e-01
       coeff_l3(11,7)= -3.0080315471e-01
       coeff_l3(11,8)=  2.4821209908e-01
       coeff_l3(11,9)=  2.6129012108e+00
       coeff_l3(11,10)=  2.9703176022e+00
       coeff_l3(11,11)=  1.8519357443e+00
       coeff_l3(11,12)=  1.8330541849e+00
       coeff_l3(11,13)=  1.5427228212e+00
       coeff_l3(11,14)=  2.6498727798e+00
       coeff_l3(11,15)=  2.7338567376e-01
       coeff_l3(11,16)= -4.6388372779e-01
       coeff_l3(11,17)= -2.7813437581e-01
       coeff_l3(11,18)= -1.5665332973e-01
       coeff_l3(11,19)= -3.6823257804e-01
       coeff_l3(11,20)=  6.2982678413e-02
       coeff_l3(11,21)=  1.5318618715e-01
       coeff_l3(11,22)= -4.5617577434e-01
       coeff_l3(11,23)= -1.2520115376e+00
       coeff_l3(11,24)=  3.4983763099e-01
       coeff_l3(11,25)= -1.1035746336e+00
       coeff_l3(11,26)=  9.8523020744e-01
       coeff_l3(11,27)= -3.2731947303e-01
       coeff_l3(11,28)=  4.2014595866e-01
       coeff_l3(11,29)=  1.4671556950e+00
       coeff_l3(11,30)=  1.3401570730e-02
       coeff_l3(11,31)= -6.6343641281e-01
       coeff_l3(11,32)= -1.1517602205e-01
       coeff_l3(11,33)=  7.3124700785e-01
       coeff_l3(11,34)=  1.2336331606e+00
       coeff_l3(11,35)=  6.0314261913e-01
       coeff_l3(11,36)=  8.5422635078e-01
       coeff_l3(11,37)= -1.3876125216e-01
       coeff_l3(11,38)= -4.1845548153e-01
       coeff_l3(11,39)=  3.1465327740e-01
       coeff_l3(11,40)= -6.2460739166e-02
       coeff_l3(11,41)= -1.0872261226e-01
       coeff_l3(11,42)=  9.4221156836e-01
       coeff_l3(11,43)=  5.7135611773e-01
       coeff_l3(11,44)=  6.4099961519e-01
       coeff_l3(11,45)=  2.1435578167e-01
       coeff_l3(11,46)= -1.7050275207e-01
       coeff_l3(11,47)=  1.4330641031e+00
       coeff_l3(11,48)= -1.0586410761e+00
       coeff_l3(11,49)= -2.5163134933e-01
       coeff_l3(11,50)=  1.3490759134e+00
       coeff_l3(11,51)= -1.8890731335e+00
       coeff_l3(11,52)=  1.0956093073e+00
       coeff_l3(11,53)=  3.2141361386e-02
       coeff_l3(11,54)=  6.8618938327e-02
       coeff_l3(11,55)=  8.2969194651e-01
       coeff_l3(11,56)=  7.2616028786e-01
       coeff_l3(11,57)=  8.8141933084e-02
       coeff_l3(11,58)= -6.9453388453e-01
       coeff_l3(11,59)= -7.3224976659e-02
       coeff_l3(11,60)=  9.1960892081e-02
       coeff_l3(11,61)=  7.3027230799e-02
       coeff_l3(11,62)= -3.7917101383e-01
       coeff_l3(11,63)=  2.9157690704e-02
       coeff_l3(11,64)= -1.9030333757e+00
       coeff_l3(11,65)= -3.5396623611e-01
       coeff_l3(11,66)= -6.8823969364e-01
       coeff_l3(11,67)= -1.4933569729e-01
       coeff_l3(11,68)=  8.8221853971e-01
       coeff_l3(11,69)= -8.5999518633e-01
       coeff_l3(11,70)= -3.8778144121e-01
       coeff_l3(11,71)=  1.4794026315e-01
       coeff_l3(11,72)=  3.8518282771e-01
       coeff_l3(11,73)=  9.3820448965e-03
       coeff_l3(11,74)= -2.0743227005e+00
       coeff_l3(11,75)= -1.5796152353e+00
       coeff_l3(11,76)= -1.2936606407e+00
       coeff_l3(11,77)=  1.5915306807e+00
       coeff_l3(11,78)= -1.7648719251e-01
       coeff_l3(11,79)=  9.3279510736e-01
       coeff_l3(11,80)= -1.1053401232e+00
       coeff_l3(11,81)= -1.8684832752e-01
       coeff_l3(11,82)=  5.3705487400e-02
       coeff_l3(11,83)= -1.1604499817e-01
       coeff_l3(11,84)=  2.0370844007e-01
       coeff_l3(11,85)=  5.5580291897e-02
       coeff_l3(11,86)= -5.4955011606e-01
       coeff_l3(11,87)=  1.6392979026e-01
       coeff_l3(11,88)= -2.2739236057e-01
       coeff_l3(11,89)=  5.9628015757e-01
       coeff_l3(11,90)= -2.2475948930e-01
       coeff_l3(11,91)=  5.0223886967e-02
       coeff_l3(11,92)=  1.8420356512e+00
       coeff_l3(11,93)= -5.8239179850e-01
       coeff_l3(11,94)=  1.0142629147e+00
       coeff_l3(11,95)=  4.0284976363e-01
       coeff_l3(11,96)= -3.5038167238e-01
       coeff_l3(11,97)=  2.4424581528e+00
       coeff_l3(11,98)= -3.4472897649e-01
       coeff_l3(11,99)= -4.4534951448e-01
       coeff_l3(11,100)=  1.4493434429e+00
       coeff_l3(11,101)=  3.3227109909e-01
       coeff_l3(11,102)= -1.2254913151e-01
       coeff_l3(11,103)=  1.3018965721e+00
       coeff_l3(11,104)= -7.7463406324e-01
       coeff_l3(11,105)= -2.0849111080e+00
       coeff_l3(12,1)= -8.7157231569e-01
       coeff_l3(12,2)= -2.8871339560e-01
       coeff_l3(12,3)= -6.4941710234e-01
       coeff_l3(12,4)= -5.1185077429e-01
       coeff_l3(12,5)= -2.7474632859e-01
       coeff_l3(12,6)= -5.7618111372e-01
       coeff_l3(12,7)= -4.7453969717e-01
       coeff_l3(12,8)=  4.7903361917e-01
       coeff_l3(12,9)= -1.4782040119e+00
       coeff_l3(12,10)= -5.3100651503e-01
       coeff_l3(12,11)= -1.6128575802e+00
       coeff_l3(12,12)= -1.6640552282e+00
       coeff_l3(12,13)= -2.1720018387e+00
       coeff_l3(12,14)= -1.5551182032e+00
       coeff_l3(12,15)=  3.5203214735e-02
       coeff_l3(12,16)= -5.6969314814e-01
       coeff_l3(12,17)=  6.1404746771e-01
       coeff_l3(12,18)=  1.1361199617e+00
       coeff_l3(12,19)= -6.6288910806e-02
       coeff_l3(12,20)= -2.6579782367e-01
       coeff_l3(12,21)=  5.7614725828e-01
       coeff_l3(12,22)= -1.5926714242e-01
       coeff_l3(12,23)=  1.6974998638e-02
       coeff_l3(12,24)= -6.2496072054e-01
       coeff_l3(12,25)=  4.4415941834e-01
       coeff_l3(12,26)=  2.1196907759e-01
       coeff_l3(12,27)=  1.0959030688e-01
       coeff_l3(12,28)=  4.7934326530e-01
       coeff_l3(12,29)= -3.0234139413e-03
       coeff_l3(12,30)=  5.7594078779e-01
       coeff_l3(12,31)=  8.3167403936e-02
       coeff_l3(12,32)=  5.5272656679e-01
       coeff_l3(12,33)=  1.4227132499e-01
       coeff_l3(12,34)=  1.0257221758e-01
       coeff_l3(12,35)=  2.1353366971e-01
       coeff_l3(12,36)=  4.0461581200e-02
       coeff_l3(12,37)=  9.7932535410e-01
       coeff_l3(12,38)=  5.9520924091e-01
       coeff_l3(12,39)=  5.7775944471e-01
       coeff_l3(12,40)=  2.2104422748e-01
       coeff_l3(12,41)= -6.9409489632e-02
       coeff_l3(12,42)=  1.4201417565e-01
       coeff_l3(12,43)= -9.3456929922e-01
       coeff_l3(12,44)= -1.5399744511e+00
       coeff_l3(12,45)= -2.1802173555e-01
       coeff_l3(12,46)=  1.9060096145e-01
       coeff_l3(12,47)=  3.4368611872e-02
       coeff_l3(12,48)= -2.4387310445e-01
       coeff_l3(12,49)=  4.4232466817e-01
       coeff_l3(12,50)=  4.1234430671e-01
       coeff_l3(12,51)=  6.9722391665e-02
       coeff_l3(12,52)= -5.4505962133e-01
       coeff_l3(12,53)=  5.5722075701e-01
       coeff_l3(12,54)=  7.2749495506e-01
       coeff_l3(12,55)=  2.9596996307e-01
       coeff_l3(12,56)= -7.7036239207e-02
       coeff_l3(12,57)=  2.0469781756e-01
       coeff_l3(12,58)= -4.5283779502e-01
       coeff_l3(12,59)= -1.5378841162e+00
       coeff_l3(12,60)= -3.3265671134e-01
       coeff_l3(12,61)=  1.2272192165e-02
       coeff_l3(12,62)= -1.2011288404e+00
       coeff_l3(12,63)=  7.9887717962e-01
       coeff_l3(12,64)= -3.0453017354e-01
       coeff_l3(12,65)=  7.8060567379e-01
       coeff_l3(12,66)= -6.4390653372e-01
       coeff_l3(12,67)= -6.1763536930e-01
       coeff_l3(12,68)=  1.0819905996e+00
       coeff_l3(12,69)= -1.9447146356e-01
       coeff_l3(12,70)= -8.0829727650e-01
       coeff_l3(12,71)= -4.3363085389e-01
       coeff_l3(12,72)=  4.2511698604e-01
       coeff_l3(12,73)=  3.5857704282e-01
       coeff_l3(12,74)=  1.0658376217e+00
       coeff_l3(12,75)=  8.1324112415e-01
       coeff_l3(12,76)= -4.7321695089e-01
       coeff_l3(12,77)=  4.2126154900e-01
       coeff_l3(12,78)=  1.9112496078e-01
       coeff_l3(12,79)=  1.0060942173e+00
       coeff_l3(12,80)=  1.0244680643e+00
       coeff_l3(12,81)=  1.1712543964e+00
       coeff_l3(12,82)= -1.0860356092e+00
       coeff_l3(12,83)= -1.6347476840e-01
       coeff_l3(12,84)= -3.2191255689e-01
       coeff_l3(12,85)= -2.6132985950e-02
       coeff_l3(12,86)= -1.0115412474e+00
       coeff_l3(12,87)= -4.6878206730e-01
       coeff_l3(12,88)=  5.6162047386e-01
       coeff_l3(12,89)=  1.6012968123e-01
       coeff_l3(12,90)= -1.3105553389e-01
       coeff_l3(12,91)= -2.2203820944e-01
       coeff_l3(12,92)= -1.1294445992e+00
       coeff_l3(12,93)=  2.6421451569e-01
       coeff_l3(12,94)=  3.6960911751e-01
       coeff_l3(12,95)=  1.0653080046e-01
       coeff_l3(12,96)=  7.9023706913e-01
       coeff_l3(12,97)=  1.1440426111e+00
       coeff_l3(12,98)= -4.8235118389e-01
       coeff_l3(12,99)=  5.8431231976e-01
       coeff_l3(12,100)= -1.5322337858e-02
       coeff_l3(12,101)=  7.1388292313e-01
       coeff_l3(12,102)= -3.2234838605e-01
       coeff_l3(12,103)= -3.9903265238e-01
       coeff_l3(12,104)= -7.0552843809e-01
       coeff_l3(12,105)= -8.0590939522e-01
       coeff_l3(13,1)=  2.4261403084e+00
       coeff_l3(13,2)=  4.0582293272e-01
       coeff_l3(13,3)=  2.0114936829e+00
       coeff_l3(13,4)=  7.4472051859e-01
       coeff_l3(13,5)=  1.3688491583e+00
       coeff_l3(13,6)=  2.8636407852e+00
       coeff_l3(13,7)=  1.4056141376e+00
       coeff_l3(13,8)=  2.7038326263e+00
       coeff_l3(13,9)= -3.9182886481e-01
       coeff_l3(13,10)= -8.6436104774e-01
       coeff_l3(13,11)= -4.0301835537e-01
       coeff_l3(13,12)= -3.9186492562e-01
       coeff_l3(13,13)= -8.2200801373e-01
       coeff_l3(13,14)= -2.1436508745e-02
       coeff_l3(13,15)=  5.1740407944e-01
       coeff_l3(13,16)=  5.8731812239e-01
       coeff_l3(13,17)= -3.8832166791e-01
       coeff_l3(13,18)= -1.2665157318e+00
       coeff_l3(13,19)= -8.1779551506e-01
       coeff_l3(13,20)= -1.4997106791e+00
       coeff_l3(13,21)=  8.3481931686e-01
       coeff_l3(13,22)=  1.0708197355e+00
       coeff_l3(13,23)=  2.1417140961e+00
       coeff_l3(13,24)= -9.9551606178e-01
       coeff_l3(13,25)=  2.2854542732e+00
       coeff_l3(13,26)= -7.7521163225e-01
       coeff_l3(13,27)=  1.3413233757e+00
       coeff_l3(13,28)= -6.9872885942e-01
       coeff_l3(13,29)= -8.3327180147e-01
       coeff_l3(13,30)= -5.0353463739e-02
       coeff_l3(13,31)=  2.8676012158e-01
       coeff_l3(13,32)= -4.8804670572e-01
       coeff_l3(13,33)=  3.0997928977e-01
       coeff_l3(13,34)= -7.5639210641e-02
       coeff_l3(13,35)=  9.1600573063e-01
       coeff_l3(13,36)= -1.3139307499e-01
       coeff_l3(13,37)=  1.2217644453e+00
       coeff_l3(13,38)= -1.1642730981e-01
       coeff_l3(13,39)= -9.9277503788e-02
       coeff_l3(13,40)= -1.6328272223e-01
       coeff_l3(13,41)= -1.3422575593e-01
       coeff_l3(13,42)= -8.0354458094e-01
       coeff_l3(13,43)= -8.7638676167e-01
       coeff_l3(13,44)=  3.0003651045e-03
       coeff_l3(13,45)= -4.0667980909e-01
       coeff_l3(13,46)= -5.7179194689e-01
       coeff_l3(13,47)= -4.7131010890e-01
       coeff_l3(13,48)= -1.1911915988e-01
       coeff_l3(13,49)=  1.5433726311e+00
       coeff_l3(13,50)= -2.2341992855e+00
       coeff_l3(13,51)=  1.1568547785e-01
       coeff_l3(13,52)= -2.8517532628e-03
       coeff_l3(13,53)= -1.3318516314e-01
       coeff_l3(13,54)=  1.4777109623e+00
       coeff_l3(13,55)=  3.1659075618e-01
       coeff_l3(13,56)=  6.3149678707e-01
       coeff_l3(13,57)= -9.9659240246e-01
       coeff_l3(13,58)= -5.6471740827e-03
       coeff_l3(13,59)=  7.8208357096e-01
       coeff_l3(13,60)= -5.3741699457e-01
       coeff_l3(13,61)=  4.0574258566e-01
       coeff_l3(13,62)= -8.9307403564e-01
       coeff_l3(13,63)=  5.8772742748e-01
       coeff_l3(13,64)=  1.9611436129e+00
       coeff_l3(13,65)=  1.3507174253e+00
       coeff_l3(13,66)= -4.5526555181e-01
       coeff_l3(13,67)= -3.2225874066e-01
       coeff_l3(13,68)=  2.8583100438e-01
       coeff_l3(13,69)=  1.9545161724e+00
       coeff_l3(13,70)=  8.2740008831e-01
       coeff_l3(13,71)= -3.9211577177e-01
       coeff_l3(13,72)=  5.5569481850e-01
       coeff_l3(13,73)= -3.5580220819e-01
       coeff_l3(13,74)=  6.2153697014e-01
       coeff_l3(13,75)=  2.1739428043e+00
       coeff_l3(13,76)=  5.3241771460e-01
       coeff_l3(13,77)= -1.6498919725e+00
       coeff_l3(13,78)=  1.0472928733e-01
       coeff_l3(13,79)= -4.2335519195e-01
       coeff_l3(13,80)=  1.8101594448e+00
       coeff_l3(13,81)= -7.2061061859e-01
       coeff_l3(13,82)= -1.1354430914e+00
       coeff_l3(13,83)=  1.7682546377e+00
       coeff_l3(13,84)=  1.5056201220e+00
       coeff_l3(13,85)=  1.0066021681e+00
       coeff_l3(13,86)= -6.4422291517e-01
       coeff_l3(13,87)= -1.7238457203e+00
       coeff_l3(13,88)=  1.3691223860e+00
       coeff_l3(13,89)= -6.7655980587e-01
       coeff_l3(13,90)=  1.5951426029e+00
       coeff_l3(13,91)= -7.8825816512e-02
       coeff_l3(13,92)= -1.5072355270e+00
       coeff_l3(13,93)= -1.0281832218e+00
       coeff_l3(13,94)= -9.8159068823e-01
       coeff_l3(13,95)= -5.5426847935e-01
       coeff_l3(13,96)=  2.9616258144e+00
       coeff_l3(13,97)= -2.9276814461e+00
       coeff_l3(13,98)=  4.6813556552e-01
       coeff_l3(13,99)= -6.9675028324e-01
       coeff_l3(13,100)= -1.3076564074e+00
       coeff_l3(13,101)= -1.2093898058e+00
       coeff_l3(13,102)= -2.3376953602e+00
       coeff_l3(13,103)= -7.5767004490e-01
       coeff_l3(13,104)=  2.8906574845e-01
       coeff_l3(13,105)=  1.6869512796e+00
       coeff_l3(14,1)= -5.4726505280e-01
       coeff_l3(14,2)=  5.7414823771e-01
       coeff_l3(14,3)= -6.2897151709e-01
       coeff_l3(14,4)= -2.2034771740e-01
       coeff_l3(14,5)= -6.7810255289e-01
       coeff_l3(14,6)=  1.3595501184e+00
       coeff_l3(14,7)=  2.7815863490e-01
       coeff_l3(14,8)=  1.1392344236e+00
       coeff_l3(14,9)=  2.6571643353e+00
       coeff_l3(14,10)=  2.8635153770e+00
       coeff_l3(14,11)=  4.0652842522e+00
       coeff_l3(14,12)=  3.7286825180e+00
       coeff_l3(14,13)=  3.7013692856e+00
       coeff_l3(14,14)=  2.4522371292e+00
       coeff_l3(14,15)= -3.4396079183e-01
       coeff_l3(14,16)= -2.8473630548e-01
       coeff_l3(14,17)= -1.9729234278e-01
       coeff_l3(14,18)= -1.9388377666e-01
       coeff_l3(14,19)=  2.3280239105e-01
       coeff_l3(14,20)=  6.6230762005e-01
       coeff_l3(14,21)= -1.3130284846e-01
       coeff_l3(14,22)= -4.7668606043e-01
       coeff_l3(14,23)= -1.0277756453e+00
       coeff_l3(14,24)= -5.8874058723e-01
       coeff_l3(14,25)= -1.5731570721e+00
       coeff_l3(14,26)=  9.4408631325e-02
       coeff_l3(14,27)=  1.0559340566e-01
       coeff_l3(14,28)=  1.3528998196e-01
       coeff_l3(14,29)=  1.0054361820e+00
       coeff_l3(14,30)= -6.5625041723e-01
       coeff_l3(14,31)=  3.7852445245e-01
       coeff_l3(14,32)= -3.0448102951e-01
       coeff_l3(14,33)=  2.8420323506e-03
       coeff_l3(14,34)=  2.7688556910e-01
       coeff_l3(14,35)= -3.3493199944e-01
       coeff_l3(14,36)=  5.8840209246e-01
       coeff_l3(14,37)= -5.7400852442e-02
       coeff_l3(14,38)=  4.6433874965e-01
       coeff_l3(14,39)=  4.5677848160e-02
       coeff_l3(14,40)= -3.5742524266e-01
       coeff_l3(14,41)=  1.8064469099e-01
       coeff_l3(14,42)=  1.5773960948e-01
       coeff_l3(14,43)=  3.5180613399e-02
       coeff_l3(14,44)= -5.7544046640e-01
       coeff_l3(14,45)= -8.4611403942e-01
       coeff_l3(14,46)=  1.4340840578e+00
       coeff_l3(14,47)=  2.6262706518e-01
       coeff_l3(14,48)= -6.2656082213e-02
       coeff_l3(14,49)= -6.3930553198e-01
       coeff_l3(14,50)=  3.2016825676e-01
       coeff_l3(14,51)= -8.3946609497e-01
       coeff_l3(14,52)=  4.4634291530e-01
       coeff_l3(14,53)=  1.1157266796e-01
       coeff_l3(14,54)= -1.1712237597e+00
       coeff_l3(14,55)=  9.0592116117e-01
       coeff_l3(14,56)= -8.6601084471e-01
       coeff_l3(14,57)= -2.0962733030e-01
       coeff_l3(14,58)=  3.2335829735e-01
       coeff_l3(14,59)=  6.0799610615e-01
       coeff_l3(14,60)=  3.2738509774e-01
       coeff_l3(14,61)= -2.7012896538e-01
       coeff_l3(14,62)=  4.6346047521e-01
       coeff_l3(14,63)=  1.2686403096e-01
       coeff_l3(14,64)= -1.3468055725e+00
       coeff_l3(14,65)= -7.7232712507e-01
       coeff_l3(14,66)=  8.9231818914e-01
       coeff_l3(14,67)=  1.1499203444e+00
       coeff_l3(14,68)=  2.9124716297e-02
       coeff_l3(14,69)= -2.2131268978e+00
       coeff_l3(14,70)= -1.3624756038e-01
       coeff_l3(14,71)=  7.9649049044e-01
       coeff_l3(14,72)= -7.2453874350e-01
       coeff_l3(14,73)= -5.7676315308e-01
       coeff_l3(14,74)= -5.4471617937e-01
       coeff_l3(14,75)= -5.2094364166e-01
       coeff_l3(14,76)= -2.4352483451e-01
       coeff_l3(14,77)= -4.8118676990e-02
       coeff_l3(14,78)= -5.9390789270e-01
       coeff_l3(14,79)= -5.0262433290e-01
       coeff_l3(14,80)= -1.9888989627e-01
       coeff_l3(14,81)=  7.0428174734e-01
       coeff_l3(14,82)=  7.1423053741e-01
       coeff_l3(14,83)= -5.8681237698e-01
       coeff_l3(14,84)= -3.2890623808e-01
       coeff_l3(14,85)= -3.9010369778e-01
       coeff_l3(14,86)=  3.2350745797e-01
       coeff_l3(14,87)=  2.1912270784e-01
       coeff_l3(14,88)= -1.2761565447e+00
       coeff_l3(14,89)=  9.2551209033e-02
       coeff_l3(14,90)= -4.1447836161e-01
       coeff_l3(14,91)= -2.2952643037e-01
       coeff_l3(14,92)=  8.5357296467e-01
       coeff_l3(14,93)= -1.1696265936e+00
       coeff_l3(14,94)= -1.2068824768e+00
       coeff_l3(14,95)= -2.2068265826e-02
       coeff_l3(14,96)=  6.7954575643e-03
       coeff_l3(14,97)= -1.0831625462e+00
       coeff_l3(14,98)=  2.5008231401e-01
       coeff_l3(14,99)=  4.7399798036e-01
       coeff_l3(14,100)= -6.7129440606e-02
       coeff_l3(14,101)= -8.8594949245e-01
       coeff_l3(14,102)= -3.7733200192e-01
       coeff_l3(14,103)=  1.4432980120e-01
       coeff_l3(14,104)=  1.5670744181e+00
       coeff_l3(14,105)=  1.1452115774e+00
       coeff_l3(15,1)=  2.2026016712e+00
       coeff_l3(15,2)=  1.9584363699e+00
       coeff_l3(15,3)=  1.4314390421e+00
       coeff_l3(15,4)=  1.6127026081e+00
       coeff_l3(15,5)=  1.9605298042e+00
       coeff_l3(15,6)=  1.7128521204e+00
       coeff_l3(15,7)=  1.7820044756e+00
       coeff_l3(15,8)=  2.2910916805e+00
       coeff_l3(15,9)=  1.6526167393e+00
       coeff_l3(15,10)= -3.3103424311e-01
       coeff_l3(15,11)=  1.4536721706e+00
       coeff_l3(15,12)= -1.3601796329e-01
       coeff_l3(15,13)=  9.5877045393e-01
       coeff_l3(15,14)=  1.2341732979e+00
       coeff_l3(15,15)=  1.0803240538e+00
       coeff_l3(15,16)=  1.2814676762e-01
       coeff_l3(15,17)= -3.3286491036e-01
       coeff_l3(15,18)=  1.1324027777e+00
       coeff_l3(15,19)= -4.8966798186e-01
       coeff_l3(15,20)=  3.5006579757e-01
       coeff_l3(15,21)=  4.3472006917e-02
       coeff_l3(15,22)=  2.2158757783e-03
       coeff_l3(15,23)=  4.1719070077e-01
       coeff_l3(15,24)=  1.0376631021e+00
       coeff_l3(15,25)=  5.5934405327e-01
       coeff_l3(15,26)= -1.8455352783e+00
       coeff_l3(15,27)= -9.0874814987e-01
       coeff_l3(15,28)= -4.6622887254e-01
       coeff_l3(15,29)= -4.9463054538e-01
       coeff_l3(15,30)=  4.3248522282e-01
       coeff_l3(15,31)= -1.6959120333e-01
       coeff_l3(15,32)= -9.6849745512e-01
       coeff_l3(15,33)= -8.5220688581e-01
       coeff_l3(15,34)=  7.8793722391e-01
       coeff_l3(15,35)=  2.5891953707e-01
       coeff_l3(15,36)=  1.4492919445e+00
       coeff_l3(15,37)= -5.3392434120e-01
       coeff_l3(15,38)=  6.2137335539e-01
       coeff_l3(15,39)=  8.2916164398e-01
       coeff_l3(15,40)=  9.3676127493e-02
       coeff_l3(15,41)= -6.4624398947e-01
       coeff_l3(15,42)= -7.2917991877e-01
       coeff_l3(15,43)= -4.5134949684e-01
       coeff_l3(15,44)= -7.7564358711e-02
       coeff_l3(15,45)= -2.1703878045e-01
       coeff_l3(15,46)=  9.5610100031e-01
       coeff_l3(15,47)= -7.3420143127e-01
       coeff_l3(15,48)= -1.8743352890e+00
       coeff_l3(15,49)= -5.4141896963e-01
       coeff_l3(15,50)= -1.2337702513e+00
       coeff_l3(15,51)=  3.6987188458e-01
       coeff_l3(15,52)= -1.3990733624e+00
       coeff_l3(15,53)=  2.9182634316e-03
       coeff_l3(15,54)=  1.0172990561e+00
       coeff_l3(15,55)=  6.6238480806e-01
       coeff_l3(15,56)= -6.7158561945e-01
       coeff_l3(15,57)= -8.6503195763e-01
       coeff_l3(15,58)=  7.8532469273e-01
       coeff_l3(15,59)= -6.1675518751e-01
       coeff_l3(15,60)=  1.9160331488e+00
       coeff_l3(15,61)= -1.0836782455e+00
       coeff_l3(15,62)=  6.9714921713e-01
       coeff_l3(15,63)=  8.0360531807e-01
       coeff_l3(15,64)= -2.2255386412e-01
       coeff_l3(15,65)= -4.6655407548e-01
       coeff_l3(15,66)= -3.8581719995e-01
       coeff_l3(15,67)=  4.8885732889e-01
       coeff_l3(15,68)= -1.4789966345e+00
       coeff_l3(15,69)=  5.6708288193e-01
       coeff_l3(15,70)=  8.0640196800e-01
       coeff_l3(15,71)=  1.1097719520e-01
       coeff_l3(15,72)= -8.7433099747e-02
       coeff_l3(15,73)= -1.0943067819e-01
       coeff_l3(15,74)= -4.4797143340e-01
       coeff_l3(15,75)= -2.7232724428e-01
       coeff_l3(15,76)=  9.0340548754e-01
       coeff_l3(15,77)= -9.6905755997e-01
       coeff_l3(15,78)=  2.2746205330e-01
       coeff_l3(15,79)= -3.1541001797e-01
       coeff_l3(15,80)= -2.2921234369e-01
       coeff_l3(15,81)=  7.3895102739e-01
       coeff_l3(15,82)= -7.1624919772e-02
       coeff_l3(15,83)= -5.3470280021e-02
       coeff_l3(15,84)= -1.0365315676e+00
       coeff_l3(15,85)=  3.9069214463e-01
       coeff_l3(15,86)=  5.3543132544e-01
       coeff_l3(15,87)=  2.6872304082e-01
       coeff_l3(15,88)= -1.8083467484e+00
       coeff_l3(15,89)=  3.6502155662e-01
       coeff_l3(15,90)= -8.6654329300e-01
       coeff_l3(15,91)= -1.6863438487e-01
       coeff_l3(15,92)= -4.7055922449e-02
       coeff_l3(15,93)= -1.6928638518e-01
       coeff_l3(15,94)= -5.2107369900e-01
       coeff_l3(15,95)=  2.1111680567e-01
       coeff_l3(15,96)=  2.3488813639e-01
       coeff_l3(15,97)=  1.0762526095e-01
       coeff_l3(15,98)= -8.2472115755e-02
       coeff_l3(15,99)= -4.4504195452e-01
       coeff_l3(15,100)= -1.4200612903e-01
       coeff_l3(15,101)=  2.4630258977e-01
       coeff_l3(15,102)=  2.7691295370e-02
       coeff_l3(15,103)=  1.0322145373e-01
       coeff_l3(15,104)=  8.1267267466e-01
       coeff_l3(15,105)=  5.4503262043e-01
       coeff_l3(16,1)= -1.5776841640e+00
       coeff_l3(16,2)=  4.6063238382e-01
       coeff_l3(16,3)= -9.3846613169e-01
       coeff_l3(16,4)= -1.1600912809e+00
       coeff_l3(16,5)= -3.3019742370e-01
       coeff_l3(16,6)= -2.1291606426e+00
       coeff_l3(16,7)= -2.8982448578e-01
       coeff_l3(16,8)= -5.0977122784e-01
       coeff_l3(16,9)= -1.4885764122e+00
       coeff_l3(16,10)=  2.5285758972e+00
       coeff_l3(16,11)= -1.6506956518e-01
       coeff_l3(16,12)= -9.2702984810e-01
       coeff_l3(16,13)=  1.0426605940e+00
       coeff_l3(16,14)= -7.4898463488e-01
       coeff_l3(16,15)=  6.2022209167e-01
       coeff_l3(16,16)= -6.5522605181e-01
       coeff_l3(16,17)=  5.6942963600e-01
       coeff_l3(16,18)=  2.7231141925e-01
       coeff_l3(16,19)= -7.9895234108e-01
       coeff_l3(16,20)=  1.9105021656e-01
       coeff_l3(16,21)= -7.7470129728e-01
       coeff_l3(16,22)= -9.1198720038e-03
       coeff_l3(16,23)= -2.1460223198e-01
       coeff_l3(16,24)=  1.5342051983e+00
       coeff_l3(16,25)= -5.0970804691e-01
       coeff_l3(16,26)=  4.1534340382e-01
       coeff_l3(16,27)= -1.7964297533e-01
       coeff_l3(16,28)=  3.6621075869e-01
       coeff_l3(16,29)=  7.2793525457e-01
       coeff_l3(16,30)=  1.7856911421e+00
       coeff_l3(16,31)= -3.9258308709e-02
       coeff_l3(16,32)=  1.0950665474e+00
       coeff_l3(16,33)=  1.2583552599e+00
       coeff_l3(16,34)=  2.0161938667e+00
       coeff_l3(16,35)=  1.3910919428e-01
       coeff_l3(16,36)=  2.6002312079e-02
       coeff_l3(16,37)=  1.0615954548e-01
       coeff_l3(16,38)=  5.9844195843e-01
       coeff_l3(16,39)=  3.6391457915e-01
       coeff_l3(16,40)=  5.0439071655e-01
       coeff_l3(16,41)=  8.3268773556e-01
       coeff_l3(16,42)=  1.0283730030e+00
       coeff_l3(16,43)= -6.3863113523e-02
       coeff_l3(16,44)=  7.3565214872e-02
       coeff_l3(16,45)= -7.1147298813e-01
       coeff_l3(16,46)= -9.4226676226e-01
       coeff_l3(16,47)=  5.7718688250e-01
       coeff_l3(16,48)= -5.4717493057e-01
       coeff_l3(16,49)=  5.9093993902e-01
       coeff_l3(16,50)=  1.7659782171e+00
       coeff_l3(16,51)= -5.0392723083e-01
       coeff_l3(16,52)= -2.4385924637e-01
       coeff_l3(16,53)=  7.7791434526e-01
       coeff_l3(16,54)= -2.3761786520e-02
       coeff_l3(16,55)=  9.5167770982e-02
       coeff_l3(16,56)= -6.4825177193e-01
       coeff_l3(16,57)=  6.5497416258e-01
       coeff_l3(16,58)= -1.2252043486e+00
       coeff_l3(16,59)=  3.1556885689e-02
       coeff_l3(16,60)= -6.5840089321e-01
       coeff_l3(16,61)= -2.8906506300e-01
       coeff_l3(16,62)= -1.6333945096e-01
       coeff_l3(16,63)= -5.8469569683e-01
       coeff_l3(16,64)= -4.4846224785e-01
       coeff_l3(16,65)= -4.6984338760e-01
       coeff_l3(16,66)=  1.7632887363e+00
       coeff_l3(16,67)= -1.2003678083e+00
       coeff_l3(16,68)=  1.7201429605e+00
       coeff_l3(16,69)= -5.9745162725e-01
       coeff_l3(16,70)= -1.7846097946e+00
       coeff_l3(16,71)= -4.2657145858e-01
       coeff_l3(16,72)=  1.0063000917e+00
       coeff_l3(16,73)=  4.4288921356e-01
       coeff_l3(16,74)=  7.5524073839e-01
       coeff_l3(16,75)= -7.1565234661e-01
       coeff_l3(16,76)= -1.2068073750e+00
       coeff_l3(16,77)=  1.3380458355e+00
       coeff_l3(16,78)=  2.7586272359e-01
       coeff_l3(16,79)=  2.7405998707e+00
       coeff_l3(16,80)=  5.9860670567e-01
       coeff_l3(16,81)=  8.9260822535e-01
       coeff_l3(16,82)= -6.9065272808e-01
       coeff_l3(16,83)= -1.3446430862e-01
       coeff_l3(16,84)= -5.3264981508e-01
       coeff_l3(16,85)= -1.0819327831e+00
       coeff_l3(16,86)= -1.4775273800e+00
       coeff_l3(16,87)=  2.1988853812e-01
       coeff_l3(16,88)=  1.5290384293e+00
       coeff_l3(16,89)= -2.4308200181e-01
       coeff_l3(16,90)= -1.2057530880e+00
       coeff_l3(16,91)=  1.1768471450e-01
       coeff_l3(16,92)= -1.5235233307e+00
       coeff_l3(16,93)= -5.4847663641e-01
       coeff_l3(16,94)=  1.4844232798e+00
       coeff_l3(16,95)= -7.8480648994e-01
       coeff_l3(16,96)= -8.8247030973e-01
       coeff_l3(16,97)=  6.0545903444e-01
       coeff_l3(16,98)= -1.7896440625e-01
       coeff_l3(16,99)=  6.8147164583e-01
       coeff_l3(16,100)=  1.4535008669e+00
       coeff_l3(16,101)= -4.5163463801e-02
       coeff_l3(16,102)= -7.7376556396e-01
       coeff_l3(16,103)=  5.1568233967e-01
       coeff_l3(16,104)=  7.9657152295e-02
       coeff_l3(16,105)= -4.4981506467e-01
       coeff_l3(17,1)= -1.7902580500e+00
       coeff_l3(17,2)= -3.4106931686e+00
       coeff_l3(17,3)= -2.5951876640e+00
       coeff_l3(17,4)= -2.2288923264e+00
       coeff_l3(17,5)= -2.9538414478e+00
       coeff_l3(17,6)= -3.5518686771e+00
       coeff_l3(17,7)= -1.8747681379e+00
       coeff_l3(17,8)= -1.7599782944e+00
       coeff_l3(17,9)= -3.7155404091e+00
       coeff_l3(17,10)= -4.5130691528e+00
       coeff_l3(17,11)= -3.5094802380e+00
       coeff_l3(17,12)= -2.8295519352e+00
       coeff_l3(17,13)= -3.0547087193e+00
       coeff_l3(17,14)= -3.0676121712e+00
       coeff_l3(17,15)=  7.3153764009e-02
       coeff_l3(17,16)= -1.4016033709e-01
       coeff_l3(17,17)= -1.4807248116e-01
       coeff_l3(17,18)=  8.3170926571e-01
       coeff_l3(17,19)=  3.9506274462e-01
       coeff_l3(17,20)= -1.3667577505e+00
       coeff_l3(17,21)=  3.6504980922e-01
       coeff_l3(17,22)=  1.6099706888e+00
       coeff_l3(17,23)=  3.2946655154e-01
       coeff_l3(17,24)= -1.6609178483e-01
       coeff_l3(17,25)=  1.4943572283e+00
       coeff_l3(17,26)= -1.2937008142e+00
       coeff_l3(17,27)=  4.8535984755e-01
       coeff_l3(17,28)=  4.1911970824e-02
       coeff_l3(17,29)= -1.7376569510e+00
       coeff_l3(17,30)=  4.3054684997e-01
       coeff_l3(17,31)= -3.8970923424e-01
       coeff_l3(17,32)=  1.1902177334e+00
       coeff_l3(17,33)= -8.7510561943e-01
       coeff_l3(17,34)= -1.4316838980e+00
       coeff_l3(17,35)= -1.5270537138e+00
       coeff_l3(17,36)= -2.1131238937e+00
       coeff_l3(17,37)=  1.2213820219e+00
       coeff_l3(17,38)= -1.7926894128e-01
       coeff_l3(17,39)= -1.2413335592e-01
       coeff_l3(17,40)= -1.8131659925e-01
       coeff_l3(17,41)= -3.6267536879e-01
       coeff_l3(17,42)= -9.4239211082e-01
       coeff_l3(17,43)= -4.7539792955e-02
       coeff_l3(17,44)= -1.2014291286e+00
       coeff_l3(17,45)=  1.0367809981e-01
       coeff_l3(17,46)= -6.0158735514e-01
       coeff_l3(17,47)=  3.1060713530e-01
       coeff_l3(17,48)=  1.9489672184e+00
       coeff_l3(17,49)=  9.4747471809e-01
       coeff_l3(17,50)= -1.2598930597e+00
       coeff_l3(17,51)=  1.4738913774e+00
       coeff_l3(17,52)= -6.4608329535e-01
       coeff_l3(17,53)= -5.9685111046e-01
       coeff_l3(17,54)=  7.4243986607e-01
       coeff_l3(17,55)= -4.1340985894e-01
       coeff_l3(17,56)=  1.7838142812e-01
       coeff_l3(17,57)=  1.0195602179e+00
       coeff_l3(17,58)=  3.6214596033e-01
       coeff_l3(17,59)=  2.0804266632e-01
       coeff_l3(17,60)= -6.9498246908e-01
       coeff_l3(17,61)=  7.5895234942e-02
       coeff_l3(17,62)= -1.1537135839e+00
       coeff_l3(17,63)=  1.5622022748e-01
       coeff_l3(17,64)=  2.6208987236e+00
       coeff_l3(17,65)=  6.2948012352e-01
       coeff_l3(17,66)= -1.6967780590e+00
       coeff_l3(17,67)= -6.5422490239e-02
       coeff_l3(17,68)= -1.7335194862e-03
       coeff_l3(17,69)=  1.8877564669e+00
       coeff_l3(17,70)=  7.2522544861e-01
       coeff_l3(17,71)= -6.3999778032e-01
       coeff_l3(17,72)= -2.5519984961e-01
       coeff_l3(17,73)=  1.5612072945e+00
       coeff_l3(17,74)=  1.7636880875e+00
       coeff_l3(17,75)=  1.9401904345e+00
       coeff_l3(17,76)=  1.4681906700e+00
       coeff_l3(17,77)= -2.1896475554e-01
       coeff_l3(17,78)=  4.4000846148e-01
       coeff_l3(17,79)= -1.0213240385e+00
       coeff_l3(17,80)=  1.2683241367e+00
       coeff_l3(17,81)= -5.6451016665e-01
       coeff_l3(17,82)= -6.9972830825e-03
       coeff_l3(17,83)=  1.0612028837e+00
       coeff_l3(17,84)=  1.0741660595e+00
       coeff_l3(17,85)=  1.6456092596e+00
       coeff_l3(17,86)=  3.2491073012e-01
       coeff_l3(17,87)=  6.9689708948e-01
       coeff_l3(17,88)=  1.3429322243e+00
       coeff_l3(17,89)= -1.1867685616e-01
       coeff_l3(17,90)=  1.3484892845e+00
       coeff_l3(17,91)= -1.8061800301e-01
       coeff_l3(17,92)= -2.2095403671e+00
       coeff_l3(17,93)=  6.6304421425e-01
       coeff_l3(17,94)= -2.3032839596e-01
       coeff_l3(17,95)= -7.1319723129e-01
       coeff_l3(17,96)=  1.6789587736e+00
       coeff_l3(17,97)= -9.5878148079e-01
       coeff_l3(17,98)=  1.5914367437e+00
       coeff_l3(17,99)= -1.4842163324e+00
       coeff_l3(17,100)= -9.4458472729e-01
       coeff_l3(17,101)=  2.9602617025e-01
       coeff_l3(17,102)= -6.7422181368e-01
       coeff_l3(17,103)= -1.5909860134e+00
       coeff_l3(17,104)= -3.7788620591e-01
       coeff_l3(17,105)=  4.9869543314e-01
       coeff_l3(18,1)=  1.5668653250e+00
       coeff_l3(18,2)=  3.6151618958e+00
       coeff_l3(18,3)=  2.3175730705e+00
       coeff_l3(18,4)=  2.2673354149e+00
       coeff_l3(18,5)=  7.8857606649e-01
       coeff_l3(18,6)=  1.8756691217e+00
       coeff_l3(18,7)=  1.5542570353e+00
       coeff_l3(18,8)=  4.5111539960e-01
       coeff_l3(18,9)=  2.5784127712e+00
       coeff_l3(18,10)=  2.5362517834e+00
       coeff_l3(18,11)=  1.4641036987e+00
       coeff_l3(18,12)=  7.8381919861e-01
       coeff_l3(18,13)=  6.3135111332e-01
       coeff_l3(18,14)=  1.3089772463e+00
       coeff_l3(18,15)= -5.4837161303e-01
       coeff_l3(18,16)=  1.9684118032e-01
       coeff_l3(18,17)= -2.1263684332e-01
       coeff_l3(18,18)= -8.0708754063e-01
       coeff_l3(18,19)=  5.1577311754e-01
       coeff_l3(18,20)=  9.0268832445e-01
       coeff_l3(18,21)= -5.7914555073e-01
       coeff_l3(18,22)= -4.7485339642e-01
       coeff_l3(18,23)= -5.4228806496e-01
       coeff_l3(18,24)= -3.9038237929e-01
       coeff_l3(18,25)=  3.7197026610e-01
       coeff_l3(18,26)= -5.9153991938e-01
       coeff_l3(18,27)=  1.5417867899e-01
       coeff_l3(18,28)=  1.2758798897e-01
       coeff_l3(18,29)= -3.8261130452e-01
       coeff_l3(18,30)= -1.0541126728e+00
       coeff_l3(18,31)=  4.3611225486e-01
       coeff_l3(18,32)= -5.9869062901e-01
       coeff_l3(18,33)= -1.6234475374e-01
       coeff_l3(18,34)=  2.4184937775e-01
       coeff_l3(18,35)=  5.2838164568e-01
       coeff_l3(18,36)=  6.6055113077e-01
       coeff_l3(18,37)= -1.3298529387e+00
       coeff_l3(18,38)=  7.1136969328e-01
       coeff_l3(18,39)=  1.0239118338e-01
       coeff_l3(18,40)=  1.1477376521e-01
       coeff_l3(18,41)=  5.0934475660e-01
       coeff_l3(18,42)=  9.7032845020e-02
       coeff_l3(18,43)= -1.1741530895e-01
       coeff_l3(18,44)=  5.5627387762e-01
       coeff_l3(18,45)=  4.8452008516e-02
       coeff_l3(18,46)=  3.0043992400e-01
       coeff_l3(18,47)= -1.2852361202e+00
       coeff_l3(18,48)= -1.0768648386e+00
       coeff_l3(18,49)= -1.0626291037e+00
       coeff_l3(18,50)= -8.3170756698e-02
       coeff_l3(18,51)= -9.0741741657e-01
       coeff_l3(18,52)= -1.5384516120e-01
       coeff_l3(18,53)= -8.0608785152e-01
       coeff_l3(18,54)= -3.7894728780e-01
       coeff_l3(18,55)= -1.3137176633e-01
       coeff_l3(18,56)= -3.2816523314e-01
       coeff_l3(18,57)= -4.6407496929e-01
       coeff_l3(18,58)=  2.6993334293e-01
       coeff_l3(18,59)=  1.0853620768e+00
       coeff_l3(18,60)=  5.9941131622e-02
       coeff_l3(18,61)= -7.8062951565e-02
       coeff_l3(18,62)=  8.4259641171e-01
       coeff_l3(18,63)=  1.3745351136e-01
       coeff_l3(18,64)= -1.3944259882e+00
       coeff_l3(18,65)= -1.6532777250e-01
       coeff_l3(18,66)=  9.1438406706e-01
       coeff_l3(18,67)=  3.9609059691e-01
       coeff_l3(18,68)= -4.5372182131e-01
       coeff_l3(18,69)= -1.2264617682e+00
       coeff_l3(18,70)=  4.0615704656e-01
       coeff_l3(18,71)=  8.3629024029e-01
       coeff_l3(18,72)= -5.4792141914e-01
       coeff_l3(18,73)= -7.5907099247e-01
       coeff_l3(18,74)= -1.1631003618e+00
       coeff_l3(18,75)= -1.5371657908e-01
       coeff_l3(18,76)= -2.4227513000e-02
       coeff_l3(18,77)= -5.5084162951e-01
       coeff_l3(18,78)= -4.6702158451e-01
       coeff_l3(18,79)= -7.1500927210e-01
       coeff_l3(18,80)= -7.0328897238e-01
       coeff_l3(18,81)= -3.8706183434e-01
       coeff_l3(18,82)=  2.5869837403e-01
       coeff_l3(18,83)= -4.9823451042e-01
       coeff_l3(18,84)= -1.4585297108e+00
       coeff_l3(18,85)=  2.0839184523e-01
       coeff_l3(18,86)=  5.9701949358e-01
       coeff_l3(18,87)= -9.8161888123e-01
       coeff_l3(18,88)= -1.0430873632e+00
       coeff_l3(18,89)=  1.4159861207e-01
       coeff_l3(18,90)= -1.5648829937e+00
       coeff_l3(18,91)=  8.9084064960e-01
       coeff_l3(18,92)=  1.8406066895e+00
       coeff_l3(18,93)= -1.1244864017e-01
       coeff_l3(18,94)=  3.6197713017e-01
       coeff_l3(18,95)=  8.6540329456e-01
       coeff_l3(18,96)= -1.0393408537e+00
       coeff_l3(18,97)= -7.0805686712e-01
       coeff_l3(18,98)= -4.4128593802e-01
       coeff_l3(18,99)=  1.8046951294e-01
       coeff_l3(18,100)=  5.2648890018e-01
       coeff_l3(18,101)= -7.7906715870e-01
       coeff_l3(18,102)=  1.0119040012e+00
       coeff_l3(18,103)=  1.5145232677e+00
       coeff_l3(18,104)=  1.1203652620e+00
       coeff_l3(18,105)= -6.5402932465e-02
       coeff_l3(19,1)=  2.5857360363e+00
       coeff_l3(19,2)=  2.5205097198e+00
       coeff_l3(19,3)=  9.1386067867e-01
       coeff_l3(19,4)=  9.5849233866e-01
       coeff_l3(19,5)=  1.9249285460e+00
       coeff_l3(19,6)= -1.2449624389e-01
       coeff_l3(19,7)=  1.9120587111e+00
       coeff_l3(19,8)= -1.9667240977e-01
       coeff_l3(19,9)= -1.4369341135e+00
       coeff_l3(19,10)= -4.5464372635e-01
       coeff_l3(19,11)= -1.2031674385e+00
       coeff_l3(19,12)= -3.7154090405e-01
       coeff_l3(19,13)= -1.2113114595e+00
       coeff_l3(19,14)= -1.4313795567e+00
       coeff_l3(19,15)=  5.1657426357e-01
       coeff_l3(19,16)= -2.6382604241e-01
       coeff_l3(19,17)= -2.9424098134e-01
       coeff_l3(19,18)=  3.1669077277e-01
       coeff_l3(19,19)=  2.2139960527e-01
       coeff_l3(19,20)=  7.0219360292e-02
       coeff_l3(19,21)= -5.1960408688e-02
       coeff_l3(19,22)= -1.7509057522e+00
       coeff_l3(19,23)= -6.8831694126e-01
       coeff_l3(19,24)= -9.1754025221e-01
       coeff_l3(19,25)= -9.0817636251e-01
       coeff_l3(19,26)= -2.9360222816e-01
       coeff_l3(19,27)= -4.7391211987e-01
       coeff_l3(19,28)=  5.8110040426e-01
       coeff_l3(19,29)=  1.0187377930e+00
       coeff_l3(19,30)= -2.7119684219e-01
       coeff_l3(19,31)= -4.0543663502e-01
       coeff_l3(19,32)=  6.4990061522e-01
       coeff_l3(19,33)= -7.8716896474e-02
       coeff_l3(19,34)=  8.5449606180e-01
       coeff_l3(19,35)= -1.0516222715e+00
       coeff_l3(19,36)=  8.0512189865e-01
       coeff_l3(19,37)= -2.1884518862e-01
       coeff_l3(19,38)= -1.5222200751e-01
       coeff_l3(19,39)=  1.5497261286e-01
       coeff_l3(19,40)=  1.6682487726e-01
       coeff_l3(19,41)=  4.2167195678e-01
       coeff_l3(19,42)=  1.7393460870e-01
       coeff_l3(19,43)=  8.1541603804e-01
       coeff_l3(19,44)=  4.5832210779e-01
       coeff_l3(19,45)= -1.8726104498e-01
       coeff_l3(19,46)= -8.0438029766e-01
       coeff_l3(19,47)= -7.1372610331e-01
       coeff_l3(19,48)= -2.8165405989e-01
       coeff_l3(19,49)= -1.9281373024e+00
       coeff_l3(19,50)=  1.7477312088e+00
       coeff_l3(19,51)=  7.3206210136e-01
       coeff_l3(19,52)=  2.4390995502e-01
       coeff_l3(19,53)= -6.3710367680e-01
       coeff_l3(19,54)= -7.2036141157e-01
       coeff_l3(19,55)= -1.5522906780e+00
       coeff_l3(19,56)=  5.9662556648e-01
       coeff_l3(19,57)=  1.0823895931e+00
       coeff_l3(19,58)=  5.3662959486e-02
       coeff_l3(19,59)=  4.0206629038e-01
       coeff_l3(19,60)=  9.0615361929e-01
       coeff_l3(19,61)=  5.1435999572e-02
       coeff_l3(19,62)=  1.1052894406e-02
       coeff_l3(19,63)= -5.5565220118e-01
       coeff_l3(19,64)=  6.2582617998e-01
       coeff_l3(19,65)= -7.5694406033e-01
       coeff_l3(19,66)=  1.2295040488e-01
       coeff_l3(19,67)= -9.1511803865e-01
       coeff_l3(19,68)=  5.9786474705e-01
       coeff_l3(19,69)=  1.6904734075e-02
       coeff_l3(19,70)= -1.8296966553e+00
       coeff_l3(19,71)=  2.1676760912e-01
       coeff_l3(19,72)= -4.4761845469e-01
       coeff_l3(19,73)=  1.1631106138e+00
       coeff_l3(19,74)= -2.4309922755e-01
       coeff_l3(19,75)= -2.6086440086e+00
       coeff_l3(19,76)= -9.0164415538e-02
       coeff_l3(19,77)=  9.8206055164e-01
       coeff_l3(19,78)=  1.5269212425e-01
       coeff_l3(19,79)= -6.7075736821e-02
       coeff_l3(19,80)=  7.8286790848e-01
       coeff_l3(19,81)=  3.5557073355e-01
       coeff_l3(19,82)=  1.7316755652e-01
       coeff_l3(19,83)= -5.9247732162e-01
       coeff_l3(19,84)= -1.5694160759e-01
       coeff_l3(19,85)= -2.7398679405e-02
       coeff_l3(19,86)= -4.4665113091e-01
       coeff_l3(19,87)= -3.2697010040e-01
       coeff_l3(19,88)=  4.6713914722e-02
       coeff_l3(19,89)= -1.2040216923e+00
       coeff_l3(19,90)= -2.1692512035e+00
       coeff_l3(19,91)=  6.8005752563e-01
       coeff_l3(19,92)=  1.2518169880e+00
       coeff_l3(19,93)=  1.3449671865e-01
       coeff_l3(19,94)=  3.9777988195e-01
       coeff_l3(19,95)=  6.9914168119e-01
       coeff_l3(19,96)= -2.5248682499e+00
       coeff_l3(19,97)=  2.2554113865e+00
       coeff_l3(19,98)= -1.2994543314e+00
       coeff_l3(19,99)=  1.5583656728e-01
       coeff_l3(19,100)=  1.5803503990e+00
       coeff_l3(19,101)=  2.4109998718e-02
       coeff_l3(19,102)=  8.5074949265e-01
       coeff_l3(19,103)=  1.8483687639e+00
       coeff_l3(19,104)= -7.5780820847e-01
       coeff_l3(19,105)= -1.5153423548e+00
       coeff_l3(20,1)= -2.2902972996e-01
       coeff_l3(20,2)= -1.2128765136e-01
       coeff_l3(20,3)= -1.6409794986e-01
       coeff_l3(20,4)= -3.2604524493e-01
       coeff_l3(20,5)= -3.3545562625e-01
       coeff_l3(20,6)=  7.7452436090e-02
       coeff_l3(20,7)= -1.0801993310e-01
       coeff_l3(20,8)=  4.0959060192e-01
       coeff_l3(20,9)=  2.3354423046e+00
       coeff_l3(20,10)=  1.9647989273e+00
       coeff_l3(20,11)=  3.4098823071e+00
       coeff_l3(20,12)=  2.4566690922e+00
       coeff_l3(20,13)=  2.6027655602e+00
       coeff_l3(20,14)=  2.4063439369e+00
       coeff_l3(20,15)= -1.5125785023e-02
       coeff_l3(20,16)=  2.1757040918e-01
       coeff_l3(20,17)= -9.1370850801e-02
       coeff_l3(20,18)= -2.4908396602e-01
       coeff_l3(20,19)=  2.0713415742e-01
       coeff_l3(20,20)=  2.3528054357e-01
       coeff_l3(20,21)= -5.0991658121e-02
       coeff_l3(20,22)=  2.1900841966e-02
       coeff_l3(20,23)=  7.3830045760e-02
       coeff_l3(20,24)=  2.6183274388e-01
       coeff_l3(20,25)= -4.2058730125e-01
       coeff_l3(20,26)=  1.3201976195e-02
       coeff_l3(20,27)= -3.7058189511e-02
       coeff_l3(20,28)=  3.2843226194e-01
       coeff_l3(20,29)= -1.5694266558e-01
       coeff_l3(20,30)= -2.7493956685e-01
       coeff_l3(20,31)= -7.2242408991e-02
       coeff_l3(20,32)= -3.1829103827e-02
       coeff_l3(20,33)=  1.4186462760e-01
       coeff_l3(20,34)=  1.3965488970e-01
       coeff_l3(20,35)= -7.0283107460e-02
       coeff_l3(20,36)=  4.4404819608e-01
       coeff_l3(20,37)= -4.8520573974e-01
       coeff_l3(20,38)=  3.8317492604e-01
       coeff_l3(20,39)=  2.6410207152e-01
       coeff_l3(20,40)=  1.5159134567e-01
       coeff_l3(20,41)= -1.2733338773e-01
       coeff_l3(20,42)= -3.9084914327e-01
       coeff_l3(20,43)= -1.5183460712e-01
       coeff_l3(20,44)= -4.4160418212e-02
       coeff_l3(20,45)= -1.6983875632e-01
       coeff_l3(20,46)=  5.9395879507e-01
       coeff_l3(20,47)= -3.4950926900e-01
       coeff_l3(20,48)= -5.2329677343e-01
       coeff_l3(20,49)=  3.3609095216e-01
       coeff_l3(20,50)=  2.7921262383e-01
       coeff_l3(20,51)= -1.7553861439e-01
       coeff_l3(20,52)= -1.5585781634e-01
       coeff_l3(20,53)=  5.4781222716e-03
       coeff_l3(20,54)= -1.4424817264e-01
       coeff_l3(20,55)=  2.8825712204e-01
       coeff_l3(20,56)= -3.1897756457e-01
       coeff_l3(20,57)= -2.2261999547e-01
       coeff_l3(20,58)=  5.6615990400e-01
       coeff_l3(20,59)=  1.9156962633e-01
       coeff_l3(20,60)= -1.6074199975e-01
       coeff_l3(20,61)=  5.1070559025e-01
       coeff_l3(20,62)=  1.9007502496e-01
       coeff_l3(20,63)= -2.5959423184e-01
       coeff_l3(20,64)= -1.9710023701e-01
       coeff_l3(20,65)= -5.2236560732e-02
       coeff_l3(20,66)=  8.0404937267e-01
       coeff_l3(20,67)=  4.0586996078e-01
       coeff_l3(20,68)= -1.0627720356e+00
       coeff_l3(20,69)= -5.5262434483e-01
       coeff_l3(20,70)=  1.9027937204e-02
       coeff_l3(20,71)=  4.1247922182e-01
       coeff_l3(20,72)= -2.6866787672e-01
       coeff_l3(20,73)= -5.6636303663e-01
       coeff_l3(20,74)= -2.4824190140e-01
       coeff_l3(20,75)=  4.8213815689e-01
       coeff_l3(20,76)= -3.4694346786e-01
       coeff_l3(20,77)= -2.4074772955e-04
       coeff_l3(20,78)=  2.6571175456e-01
       coeff_l3(20,79)= -2.0399630070e-01
       coeff_l3(20,80)=  1.4278036356e-01
       coeff_l3(20,81)= -1.8526078761e-01
       coeff_l3(20,82)= -3.3859550953e-01
       coeff_l3(20,83)= -6.7657834291e-01
       coeff_l3(20,84)=  1.7082500458e-01
       coeff_l3(20,85)= -3.3438622952e-01
       coeff_l3(20,86)=  1.8395769596e-01
       coeff_l3(20,87)=  2.8112345934e-01
       coeff_l3(20,88)= -3.9313995838e-01
       coeff_l3(20,89)=  1.6628101468e-01
       coeff_l3(20,90)= -2.9317311943e-02
       coeff_l3(20,91)= -3.5682752728e-01
       coeff_l3(20,92)= -1.4703032970e+00
       coeff_l3(20,93)= -1.0875558853e+00
       coeff_l3(20,94)= -5.1440155506e-01
       coeff_l3(20,95)= -5.5220228434e-01
       coeff_l3(20,96)=  8.0334603786e-01
       coeff_l3(20,97)= -1.5477555990e+00
       coeff_l3(20,98)=  9.4536721706e-01
       coeff_l3(20,99)=  1.7537331581e-01
       coeff_l3(20,100)= -7.3080182076e-02
       coeff_l3(20,101)= -3.7500849366e-01
       coeff_l3(20,102)= -3.3168470860e-01
       coeff_l3(20,103)= -5.0492656231e-01
       coeff_l3(20,104)=  1.0496920347e+00
       coeff_l3(20,105)=  1.7484357357e+00
       coeff_l3(21,1)=  1.7309703827e+00
       coeff_l3(21,2)=  7.8120356798e-01
       coeff_l3(21,3)=  1.5491375923e+00
       coeff_l3(21,4)=  1.1268821955e+00
       coeff_l3(21,5)=  2.0566413403e+00
       coeff_l3(21,6)=  4.4859564304e-01
       coeff_l3(21,7)=  2.4490778446e+00
       coeff_l3(21,8)=  3.0143091679e+00
       coeff_l3(21,9)=  2.9644048214e-01
       coeff_l3(21,10)=  3.0667150021e-01
       coeff_l3(21,11)=  1.1707439423e+00
       coeff_l3(21,12)=  1.0297571868e-01
       coeff_l3(21,13)=  3.8068056107e-01
       coeff_l3(21,14)= -4.8383720219e-02
       coeff_l3(21,15)= -2.8337916732e-01
       coeff_l3(21,16)= -1.9784943759e-01
       coeff_l3(21,17)= -2.8437608480e-01
       coeff_l3(21,18)= -6.5814271569e-02
       coeff_l3(21,19)= -6.0693699121e-01
       coeff_l3(21,20)=  3.2687491179e-01
       coeff_l3(21,21)=  4.0597912669e-01
       coeff_l3(21,22)= -6.6591286659e-01
       coeff_l3(21,23)=  1.4234584570e-01
       coeff_l3(21,24)= -6.4711785316e-01
       coeff_l3(21,25)=  2.5353550911e-01
       coeff_l3(21,26)=  1.0184397548e-01
       coeff_l3(21,27)=  1.1713989824e-01
       coeff_l3(21,28)= -3.4466780722e-02
       coeff_l3(21,29)=  2.3543366790e-01
       coeff_l3(21,30)= -4.9232929945e-02
       coeff_l3(21,31)=  2.2204141319e-01
       coeff_l3(21,32)=  3.8547676802e-01
       coeff_l3(21,33)=  4.9236562848e-01
       coeff_l3(21,34)= -7.6636737585e-01
       coeff_l3(21,35)= -4.4513512403e-02
       coeff_l3(21,36)= -8.8184259832e-02
       coeff_l3(21,37)= -7.2455644608e-01
       coeff_l3(21,38)=  6.8857514858e-01
       coeff_l3(21,39)= -6.9732952118e-01
       coeff_l3(21,40)= -7.0395655930e-02
       coeff_l3(21,41)=  4.2872905731e-01
       coeff_l3(21,42)=  2.0918850601e-01
       coeff_l3(21,43)=  8.7661063671e-01
       coeff_l3(21,44)= -1.8408645689e-01
       coeff_l3(21,45)= -1.6788178682e-01
       coeff_l3(21,46)=  1.2075209618e+00
       coeff_l3(21,47)=  3.4409266710e-01
       coeff_l3(21,48)= -5.7030022144e-01
       coeff_l3(21,49)=  5.1286265254e-02
       coeff_l3(21,50)=  2.1895167232e-01
       coeff_l3(21,51)=  5.1390193403e-02
       coeff_l3(21,52)=  5.1897847652e-01
       coeff_l3(21,53)=  2.2362396121e-01
       coeff_l3(21,54)= -6.6274046898e-01
       coeff_l3(21,55)=  9.3503445387e-02
       coeff_l3(21,56)= -2.3561410606e-01
       coeff_l3(21,57)= -7.4228376150e-01
       coeff_l3(21,58)=  4.2057663202e-02
       coeff_l3(21,59)=  5.6388091296e-02
       coeff_l3(21,60)= -3.1993165612e-01
       coeff_l3(21,61)= -4.6653980017e-01
       coeff_l3(21,62)=  3.0790290236e-01
       coeff_l3(21,63)= -4.8164734244e-01
       coeff_l3(21,64)=  1.1575287580e+00
       coeff_l3(21,65)= -3.3302539587e-01
       coeff_l3(21,66)= -4.7093778849e-01
       coeff_l3(21,67)=  6.2782377005e-02
       coeff_l3(21,68)=  6.2308970839e-02
       coeff_l3(21,69)= -2.3344201036e-03
       coeff_l3(21,70)=  4.4135265052e-03
       coeff_l3(21,71)=  5.8227026463e-01
       coeff_l3(21,72)= -1.5051832795e-01
       coeff_l3(21,73)= -3.0238640308e-01
       coeff_l3(21,74)= -7.7647238970e-01
       coeff_l3(21,75)= -8.7428092957e-01
       coeff_l3(21,76)=  7.7345776558e-01
       coeff_l3(21,77)=  6.9942724705e-01
       coeff_l3(21,78)= -2.0246352255e-01
       coeff_l3(21,79)=  3.3753526211e-01
       coeff_l3(21,80)= -7.6343262196e-01
       coeff_l3(21,81)=  3.1513339281e-01
       coeff_l3(21,82)=  1.7548778653e-01
       coeff_l3(21,83)= -5.6149452925e-01
       coeff_l3(21,84)= -7.4278092384e-01
       coeff_l3(21,85)=  6.8249642849e-01
       coeff_l3(21,86)= -1.6588358581e-01
       coeff_l3(21,87)=  3.0308461189e-01
       coeff_l3(21,88)=  6.2326622009e-01
       coeff_l3(21,89)= -8.1059932709e-02
       coeff_l3(21,90)= -9.4702616334e-02
       coeff_l3(21,91)= -8.3102181554e-02
       coeff_l3(21,92)=  4.0426012874e-01
       coeff_l3(21,93)= -2.9877454042e-01
       coeff_l3(21,94)=  2.9646651819e-02
       coeff_l3(21,95)= -4.2586588860e-01
       coeff_l3(21,96)= -5.6554991007e-01
       coeff_l3(21,97)=  7.8097417951e-02
       coeff_l3(21,98)=  2.7868512273e-01
       coeff_l3(21,99)=  9.5459866524e-01
       coeff_l3(21,100)= -2.0991912484e-01
       coeff_l3(21,101)=  1.9444961846e-01
       coeff_l3(21,102)=  6.6503041983e-01
       coeff_l3(21,103)=  4.5123136044e-01
       coeff_l3(21,104)= -8.9029091597e-01
       coeff_l3(21,105)= -2.7062028646e-01
       coeff_l3(22,1)=  9.0427500010e-01
       coeff_l3(22,2)=  6.6925173998e-01
       coeff_l3(22,3)=  1.6701768339e-01
       coeff_l3(22,4)=  1.5644870698e-01
       coeff_l3(22,5)=  9.4562047720e-01
       coeff_l3(22,6)=  5.0407934189e-01
       coeff_l3(22,7)=  2.4563083053e-01
       coeff_l3(22,8)=  1.1009597778e+00
       coeff_l3(22,9)=  1.9333437681e+00
       coeff_l3(22,10)=  1.5901461840e+00
       coeff_l3(22,11)=  2.2147579193e+00
       coeff_l3(22,12)=  2.2927370071e+00
       coeff_l3(22,13)=  2.3136277199e+00
       coeff_l3(22,14)=  2.8378422260e+00
       coeff_l3(22,15)= -3.2797572017e-01
       coeff_l3(22,16)= -6.0545998812e-01
       coeff_l3(22,17)= -4.1626760364e-01
       coeff_l3(22,18)=  6.3214969635e-01
       coeff_l3(22,19)= -6.7145156860e-01
       coeff_l3(22,20)=  1.0762587786e+00
       coeff_l3(22,21)= -1.5753500164e-01
       coeff_l3(22,22)=  1.2572012842e-01
       coeff_l3(22,23)= -5.0791954994e-01
       coeff_l3(22,24)=  6.7882376909e-01
       coeff_l3(22,25)= -8.1065320969e-01
       coeff_l3(22,26)=  1.2668544054e+00
       coeff_l3(22,27)= -5.3035503626e-01
       coeff_l3(22,28)=  1.6361007094e-01
       coeff_l3(22,29)= -2.8833436966e-01
       coeff_l3(22,30)=  2.9530003667e-01
       coeff_l3(22,31)=  2.1409422159e-01
       coeff_l3(22,32)= -3.0211544037e-01
       coeff_l3(22,33)= -3.8304093480e-01
       coeff_l3(22,34)=  1.4911547303e-01
       coeff_l3(22,35)=  2.3071087897e-01
       coeff_l3(22,36)=  3.5030892491e-01
       coeff_l3(22,37)=  8.7708926201e-01
       coeff_l3(22,38)= -1.0253150016e-02
       coeff_l3(22,39)= -1.1703483015e-01
       coeff_l3(22,40)= -5.7085877657e-01
       coeff_l3(22,41)= -9.0830445290e-02
       coeff_l3(22,42)=  4.2029264569e-01
       coeff_l3(22,43)= -9.8429702222e-02
       coeff_l3(22,44)= -8.8884815574e-02
       coeff_l3(22,45)= -6.3709199429e-02
       coeff_l3(22,46)= -2.3908633739e-03
       coeff_l3(22,47)=  2.9194974899e-01
       coeff_l3(22,48)= -4.1989624500e-01
       coeff_l3(22,49)=  1.2357731350e-02
       coeff_l3(22,50)= -4.7749355435e-01
       coeff_l3(22,51)= -3.8919723034e-01
       coeff_l3(22,52)=  8.3505344391e-01
       coeff_l3(22,53)= -5.4896634817e-01
       coeff_l3(22,54)= -9.9420857430e-01
       coeff_l3(22,55)=  8.4059667587e-01
       coeff_l3(22,56)=  7.6594680548e-01
       coeff_l3(22,57)= -5.5435724556e-02
       coeff_l3(22,58)=  1.0104521513e+00
       coeff_l3(22,59)= -1.4195235446e-02
       coeff_l3(22,60)=  5.3403210640e-01
       coeff_l3(22,61)= -2.4791848660e-01
       coeff_l3(22,62)= -3.7771281600e-01
       coeff_l3(22,63)= -4.6620449424e-01
       coeff_l3(22,64)= -8.5163123906e-02
       coeff_l3(22,65)=  2.2339093685e-01
       coeff_l3(22,66)= -1.2048161775e-01
       coeff_l3(22,67)=  6.6345244646e-01
       coeff_l3(22,68)=  2.2751776874e-01
       coeff_l3(22,69)= -4.4115865231e-01
       coeff_l3(22,70)= -5.5575680733e-01
       coeff_l3(22,71)=  8.4479087591e-01
       coeff_l3(22,72)=  7.4063487351e-02
       coeff_l3(22,73)=  3.7681788206e-02
       coeff_l3(22,74)= -9.0520912409e-01
       coeff_l3(22,75)= -1.9810563326e-01
       coeff_l3(22,76)= -4.1389980912e-01
       coeff_l3(22,77)=  7.0663762093e-01
       coeff_l3(22,78)= -3.0100712180e-01
       coeff_l3(22,79)=  4.6991637349e-01
       coeff_l3(22,80)=  6.2952721119e-01
       coeff_l3(22,81)=  1.7830736935e-01
       coeff_l3(22,82)=  3.3012247086e-01
       coeff_l3(22,83)=  1.4700041711e-01
       coeff_l3(22,84)=  2.2023354471e-01
       coeff_l3(22,85)= -3.6924958229e-01
       coeff_l3(22,86)=  8.0471292138e-02
       coeff_l3(22,87)=  7.1963351965e-01
       coeff_l3(22,88)= -6.4387184381e-01
       coeff_l3(22,89)=  6.2970027328e-02
       coeff_l3(22,90)= -6.9576120377e-01
       coeff_l3(22,91)=  4.7661516070e-01
       coeff_l3(22,92)= -1.0669341087e+00
       coeff_l3(22,93)= -1.0276447535e+00
       coeff_l3(22,94)= -3.6161825061e-01
       coeff_l3(22,95)= -8.4227102995e-01
       coeff_l3(22,96)=  1.4481961727e+00
       coeff_l3(22,97)= -1.1348913908e+00
       coeff_l3(22,98)=  5.0426179171e-01
       coeff_l3(22,99)= -6.6254711151e-01
       coeff_l3(22,100)=  1.9980898499e-01
       coeff_l3(22,101)= -1.8894016743e-01
       coeff_l3(22,102)= -6.2258672714e-01
       coeff_l3(22,103)= -8.7306183577e-01
       coeff_l3(22,104)=  6.4762502909e-01
       coeff_l3(22,105)=  1.6081173420e+00
       coeff_l3(23,1)=  1.4264044166e-01
       coeff_l3(23,2)=  2.9893022776e-01
       coeff_l3(23,3)=  1.3061386347e+00
       coeff_l3(23,4)=  1.7202129364e+00
       coeff_l3(23,5)=  1.9132317305e+00
       coeff_l3(23,6)= -1.2118073553e-01
       coeff_l3(23,7)=  2.9152414203e-01
       coeff_l3(23,8)=  3.8126194477e-01
       coeff_l3(23,9)=  1.8493376970e+00
       coeff_l3(23,10)=  9.3859702349e-01
       coeff_l3(23,11)=  1.6737382412e+00
       coeff_l3(23,12)=  1.8451659679e+00
       coeff_l3(23,13)=  9.5194160938e-01
       coeff_l3(23,14)=  1.1012859344e+00
       coeff_l3(23,15)= -2.5181156397e-01
       coeff_l3(23,16)=  1.4555692673e-01
       coeff_l3(23,17)=  4.1559308767e-01
       coeff_l3(23,18)=  4.8980902880e-02
       coeff_l3(23,19)=  4.2267921567e-01
       coeff_l3(23,20)=  1.1006072760e+00
       coeff_l3(23,21)= -4.9761077762e-01
       coeff_l3(23,22)= -9.1237795353e-01
       coeff_l3(23,23)= -1.1151421070e+00
       coeff_l3(23,24)=  8.7649446726e-01
       coeff_l3(23,25)= -2.3106865883e+00
       coeff_l3(23,26)=  1.3373051882e+00
       coeff_l3(23,27)= -1.1982442141e+00
       coeff_l3(23,28)= -1.1839117110e-01
       coeff_l3(23,29)=  1.1301527023e+00
       coeff_l3(23,30)=  4.2580235004e-01
       coeff_l3(23,31)= -2.3289264739e-01
       coeff_l3(23,32)=  2.4279107153e-01
       coeff_l3(23,33)=  7.5516492128e-01
       coeff_l3(23,34)=  9.4978086650e-02
       coeff_l3(23,35)=  3.2010391355e-01
       coeff_l3(23,36)=  6.1226032674e-02
       coeff_l3(23,37)= -6.7120313644e-01
       coeff_l3(23,38)= -2.5886437297e-01
       coeff_l3(23,39)= -3.3368676901e-01
       coeff_l3(23,40)=  6.6832232475e-01
       coeff_l3(23,41)=  5.2717721462e-01
       coeff_l3(23,42)=  6.6238707304e-01
       coeff_l3(23,43)=  7.2796857357e-01
       coeff_l3(23,44)=  1.5591114759e-01
       coeff_l3(23,45)=  9.0163689852e-01
       coeff_l3(23,46)=  2.3799602687e-01
       coeff_l3(23,47)=  1.3622864485e+00
       coeff_l3(23,48)=  2.8032952547e-01
       coeff_l3(23,49)= -3.1206929684e-01
       coeff_l3(23,50)=  1.9976619482e+00
       coeff_l3(23,51)= -1.6821363568e-01
       coeff_l3(23,52)=  9.5043712854e-01
       coeff_l3(23,53)=  3.8637417555e-01
       coeff_l3(23,54)= -8.7007164955e-01
       coeff_l3(23,55)= -5.3693914413e-01
       coeff_l3(23,56)= -6.4467333257e-02
       coeff_l3(23,57)=  6.2401777506e-01
       coeff_l3(23,58)= -4.3432858586e-01
       coeff_l3(23,59)= -7.8008604050e-01
       coeff_l3(23,60)= -3.4340396523e-01
       coeff_l3(23,61)=  4.7766533494e-01
       coeff_l3(23,62)=  4.2273086309e-01
       coeff_l3(23,63)= -7.7453231812e-01
       coeff_l3(23,64)= -6.5791606903e-01
       coeff_l3(23,65)= -5.5924713612e-01
       coeff_l3(23,66)= -3.0532544851e-01
       coeff_l3(23,67)=  3.9587780833e-01
       coeff_l3(23,68)= -2.2023892403e-01
       coeff_l3(23,69)= -5.4739546776e-01
       coeff_l3(23,70)= -1.0478165150e+00
       coeff_l3(23,71)=  8.0661118031e-02
       coeff_l3(23,72)=  2.3630629480e-01
       coeff_l3(23,73)= -5.6171689183e-02
       coeff_l3(23,74)= -8.1154268980e-01
       coeff_l3(23,75)= -5.8853018284e-01
       coeff_l3(23,76)= -2.8186270595e-01
       coeff_l3(23,77)=  2.1962101460e+00
       coeff_l3(23,78)=  1.6284251213e-01
       coeff_l3(23,79)=  5.5179208517e-01
       coeff_l3(23,80)= -6.7384433746e-01
       coeff_l3(23,81)= -8.1934577227e-01
       coeff_l3(23,82)=  1.1859831810e+00
       coeff_l3(23,83)= -6.7483007908e-01
       coeff_l3(23,84)=  3.7492790818e-01
       coeff_l3(23,85)= -6.1640095711e-01
       coeff_l3(23,86)= -3.6354564130e-02
       coeff_l3(23,87)=  1.2547508478e+00
       coeff_l3(23,88)= -6.1020332575e-01
       coeff_l3(23,89)=  3.1948533654e-01
       coeff_l3(23,90)= -5.1546907425e-01
       coeff_l3(23,91)=  2.0068714023e-01
       coeff_l3(23,92)=  1.3431038857e+00
       coeff_l3(23,93)=  1.1724461317e+00
       coeff_l3(23,94)=  6.9098567963e-01
       coeff_l3(23,95)=  3.9317388088e-02
       coeff_l3(23,96)= -1.3983567953e+00
       coeff_l3(23,97)=  1.3244608641e+00
       coeff_l3(23,98)= -8.9029538631e-01
       coeff_l3(23,99)=  1.2655203342e+00
       coeff_l3(23,100)=  1.7113843560e-01
       coeff_l3(23,101)=  4.7818076611e-01
       coeff_l3(23,102)=  1.4638314247e+00
       coeff_l3(23,103)=  2.2817061841e-01
       coeff_l3(23,104)= -1.4172335863e+00
       coeff_l3(23,105)= -1.4526705742e+00
       coeff_l3(24,1)= -3.2093387097e-02
       coeff_l3(24,2)=  1.8889374733e+00
       coeff_l3(24,3)=  1.5147482157e+00
       coeff_l3(24,4)=  1.8238831758e+00
       coeff_l3(24,5)=  1.1833887100e+00
       coeff_l3(24,6)=  1.1096365452e+00
       coeff_l3(24,7)=  2.3912920952e+00
       coeff_l3(24,8)=  1.3955109119e+00
       coeff_l3(24,9)=  1.7163418531e+00
       coeff_l3(24,10)=  1.8033076525e+00
       coeff_l3(24,11)=  4.3633079529e-01
       coeff_l3(24,12)=  1.1264238358e+00
       coeff_l3(24,13)=  1.7352617979e+00
       coeff_l3(24,14)=  1.9637526274e+00
       coeff_l3(24,15)= -3.5501495004e-01
       coeff_l3(24,16)=  8.0559152365e-01
       coeff_l3(24,17)= -1.5920689702e-01
       coeff_l3(24,18)= -8.5166615248e-01
       coeff_l3(24,19)=  4.4153192639e-01
       coeff_l3(24,20)=  4.2704394460e-01
       coeff_l3(24,21)=  6.0280272737e-03
       coeff_l3(24,22)= -4.6989572048e-01
       coeff_l3(24,23)= -6.6113936901e-01
       coeff_l3(24,24)= -1.2612921000e-01
       coeff_l3(24,25)= -1.2504972219e+00
       coeff_l3(24,26)=  1.3684829473e+00
       coeff_l3(24,27)= -2.3577071726e-01
       coeff_l3(24,28)=  1.1538519859e+00
       coeff_l3(24,29)=  4.1285118461e-01
       coeff_l3(24,30)= -3.0470737815e-01
       coeff_l3(24,31)= -9.0159094334e-01
       coeff_l3(24,32)= -4.1635036469e-01
       coeff_l3(24,33)=  2.7364400029e-01
       coeff_l3(24,34)= -4.7161906958e-01
       coeff_l3(24,35)=  9.1420376301e-01
       coeff_l3(24,36)=  3.2222077250e-01
       coeff_l3(24,37)= -1.0313007832e+00
       coeff_l3(24,38)=  6.2221519649e-02
       coeff_l3(24,39)=  6.2588101625e-01
       coeff_l3(24,40)=  8.0280438066e-02
       coeff_l3(24,41)=  8.3052799106e-02
       coeff_l3(24,42)=  5.5349719524e-01
       coeff_l3(24,43)=  8.0569124222e-01
       coeff_l3(24,44)=  6.0479432344e-01
       coeff_l3(24,45)= -5.5303171277e-02
       coeff_l3(24,46)= -7.3358500004e-01
       coeff_l3(24,47)= -3.6056733131e-01
       coeff_l3(24,48)= -1.6898865700e+00
       coeff_l3(24,49)= -1.3003758192e+00
       coeff_l3(24,50)=  1.3481801376e-02
       coeff_l3(24,51)= -9.6434080601e-01
       coeff_l3(24,52)=  2.9718445614e-02
       coeff_l3(24,53)=  3.8864308596e-01
       coeff_l3(24,54)= -2.1128651500e-01
       coeff_l3(24,55)= -4.5696821809e-01
       coeff_l3(24,56)=  1.3722484931e-02
       coeff_l3(24,57)=  8.4767363966e-02
       coeff_l3(24,58)=  3.7407517433e-01
       coeff_l3(24,59)= -2.3313436657e-02
       coeff_l3(24,60)= -1.1644791812e-01
       coeff_l3(24,61)=  2.8737187386e-01
       coeff_l3(24,62)= -6.8177804351e-02
       coeff_l3(24,63)= -7.3054015636e-01
       coeff_l3(24,64)= -1.9054129124e+00
       coeff_l3(24,65)= -1.6835789382e-01
       coeff_l3(24,66)= -1.4346821606e-01
       coeff_l3(24,67)=  1.9239191711e-01
       coeff_l3(24,68)=  8.5210353136e-01
       coeff_l3(24,69)= -2.3431293666e-01
       coeff_l3(24,70)= -2.6994115114e-01
       coeff_l3(24,71)=  7.5732660294e-01
       coeff_l3(24,72)=  6.6876751184e-01
       coeff_l3(24,73)= -4.5894154906e-01
       coeff_l3(24,74)= -1.3619657755e+00
       coeff_l3(24,75)= -1.2918713093e+00
       coeff_l3(24,76)= -6.8550592661e-01
       coeff_l3(24,77)=  4.3878886104e-01
       coeff_l3(24,78)=  1.9098114967e-01
       coeff_l3(24,79)= -5.3029346466e-01
       coeff_l3(24,80)= -1.3587763309e+00
       coeff_l3(24,81)= -9.9408036470e-01
       coeff_l3(24,82)=  6.1974734068e-01
       coeff_l3(24,83)= -7.4718129635e-01
       coeff_l3(24,84)= -9.5064830780e-01
       coeff_l3(24,85)= -5.7235538960e-01
       coeff_l3(24,86)= -4.1269907355e-01
       coeff_l3(24,87)= -6.4691150188e-01
       coeff_l3(24,88)= -7.1713596582e-01
       coeff_l3(24,89)=  2.4089519680e-01
       coeff_l3(24,90)= -1.0697759390e+00
       coeff_l3(24,91)= -2.3674950004e-01
       coeff_l3(24,92)=  6.7311501503e-01
       coeff_l3(24,93)= -1.0751286149e-01
       coeff_l3(24,94)=  1.1508169770e-01
       coeff_l3(24,95)=  6.3786536455e-01
       coeff_l3(24,96)= -1.3203972578e+00
       coeff_l3(24,97)= -1.0350904614e-01
       coeff_l3(24,98)= -5.2105540037e-01
       coeff_l3(24,99)=  4.9117437005e-01
       coeff_l3(24,100)=  9.4945621490e-01
       coeff_l3(24,101)= -4.0916168690e-01
       coeff_l3(24,102)=  8.4535515308e-01
       coeff_l3(24,103)=  1.0231636763e+00
       coeff_l3(24,104)= -4.4034081697e-01
       coeff_l3(24,105)= -1.2168331146e+00
       coeff_l3(25,1)=  7.0069694519e-01
       coeff_l3(25,2)=  5.2749258280e-01
       coeff_l3(25,3)=  2.1501617134e-01
       coeff_l3(25,4)=  1.5681810677e-01
       coeff_l3(25,5)=  4.5082572848e-02
       coeff_l3(25,6)= -6.2111232430e-02
       coeff_l3(25,7)=  1.8446303904e-01
       coeff_l3(25,8)= -6.6168552637e-01
       coeff_l3(25,9)= -3.6782815456e+00
       coeff_l3(25,10)= -3.3638708591e+00
       coeff_l3(25,11)= -5.6865148544e+00
       coeff_l3(25,12)= -4.0170817375e+00
       coeff_l3(25,13)= -4.2071881294e+00
       coeff_l3(25,14)= -4.0870823860e+00
       coeff_l3(25,15)=  1.3959512115e-01
       coeff_l3(25,16)= -3.4037584066e-01
       coeff_l3(25,17)=  3.1141650677e-01
       coeff_l3(25,18)=  3.5667830706e-01
       coeff_l3(25,19)= -8.5132680833e-02
       coeff_l3(25,20)= -4.9903357029e-01
       coeff_l3(25,21)= -2.2708751261e-01
       coeff_l3(25,22)= -1.2630911171e-01
       coeff_l3(25,23)= -3.0220040679e-01
       coeff_l3(25,24)= -3.6040976644e-01
       coeff_l3(25,25)=  3.7102013826e-01
       coeff_l3(25,26)=  4.7234991193e-01
       coeff_l3(25,27)=  3.0795323849e-01
       coeff_l3(25,28)= -7.2931814194e-01
       coeff_l3(25,29)=  4.0054821968e-01
       coeff_l3(25,30)=  1.2767487764e-01
       coeff_l3(25,31)=  3.6224481463e-01
       coeff_l3(25,32)=  9.1895513237e-02
       coeff_l3(25,33)= -5.6390261650e-01
       coeff_l3(25,34)= -5.0035558641e-02
       coeff_l3(25,35)= -6.1252795160e-02
       coeff_l3(25,36)= -8.5379254818e-01
       coeff_l3(25,37)=  1.1262151003e+00
       coeff_l3(25,38)= -7.9140371084e-01
       coeff_l3(25,39)= -2.6930946112e-01
       coeff_l3(25,40)= -2.6078221202e-01
       coeff_l3(25,41)= -1.8942099065e-02
       coeff_l3(25,42)=  6.3882011175e-01
       coeff_l3(25,43)=  1.2994471192e-01
       coeff_l3(25,44)= -3.9743818343e-02
       coeff_l3(25,45)=  2.3549105227e-01
       coeff_l3(25,46)= -1.0328346491e+00
       coeff_l3(25,47)=  4.7280132771e-01
       coeff_l3(25,48)=  6.8381053209e-01
       coeff_l3(25,49)= -2.2189293802e-01
       coeff_l3(25,50)= -4.2212361097e-01
       coeff_l3(25,51)=  4.0339738131e-01
       coeff_l3(25,52)=  2.1140876412e-01
       coeff_l3(25,53)=  3.8375712931e-02
       coeff_l3(25,54)=  7.1634955704e-02
       coeff_l3(25,55)= -2.3485861719e-01
       coeff_l3(25,56)=  1.1806757450e+00
       coeff_l3(25,57)=  1.0444477201e-02
       coeff_l3(25,58)= -3.2290160656e-02
       coeff_l3(25,59)=  1.0835310072e-01
       coeff_l3(25,60)= -3.0915999413e-01
       coeff_l3(25,61)= -1.3213717937e+00
       coeff_l3(25,62)= -4.8084720969e-01
       coeff_l3(25,63)=  4.8328661919e-01
       coeff_l3(25,64)=  5.9090918303e-01
       coeff_l3(25,65)=  5.1390546560e-01
       coeff_l3(25,66)= -8.4135514498e-01
       coeff_l3(25,67)= -1.2928210199e-01
       coeff_l3(25,68)=  1.0647786856e+00
       coeff_l3(25,69)=  1.2885328531e+00
       coeff_l3(25,70)= -7.2980910540e-02
       coeff_l3(25,71)= -7.1146219969e-01
       coeff_l3(25,72)= -1.2048543245e-01
       coeff_l3(25,73)=  1.2217371464e+00
       coeff_l3(25,74)=  6.0576218367e-01
       coeff_l3(25,75)= -1.2261986732e+00
       coeff_l3(25,76)=  4.5219266415e-01
       coeff_l3(25,77)= -9.3507021666e-02
       coeff_l3(25,78)= -1.3615171611e-01
       coeff_l3(25,79)=  1.7537470162e-01
       coeff_l3(25,80)=  3.2588887215e-01
       coeff_l3(25,81)=  4.8749116063e-01
       coeff_l3(25,82)=  1.0238749981e+00
       coeff_l3(25,83)=  1.1883890629e+00
       coeff_l3(25,84)=  4.8621252179e-01
       coeff_l3(25,85)=  8.3282887936e-01
       coeff_l3(25,86)= -3.8238936663e-01
       coeff_l3(25,87)=  4.2731636763e-01
       coeff_l3(25,88)=  1.1063784361e+00
       coeff_l3(25,89)= -2.5026634336e-01
       coeff_l3(25,90)=  7.0684611797e-01
       coeff_l3(25,91)=  2.4824544787e-01
       coeff_l3(25,92)=  3.2132563591e+00
       coeff_l3(25,93)=  1.7598438263e+00
       coeff_l3(25,94)=  3.2444858551e-01
       coeff_l3(25,95)=  5.5726814270e-01
       coeff_l3(25,96)= -9.8647999763e-01
       coeff_l3(25,97)=  2.6875326633e+00
       coeff_l3(25,98)= -1.7525408268e+00
       coeff_l3(25,99)= -3.9343518019e-01
       coeff_l3(25,100)=  8.2636311650e-02
       coeff_l3(25,101)=  6.3819527626e-01
       coeff_l3(25,102)=  2.3474696279e-01
       coeff_l3(25,103)=  8.6030364037e-01
       coeff_l3(25,104)= -1.5485597849e+00
       coeff_l3(25,105)= -3.2531871796e+00
       coeff_l3(26,1)= -4.9871078134e-01
       coeff_l3(26,2)= -5.9073352814e-01
       coeff_l3(26,3)= -7.0549029112e-01
       coeff_l3(26,4)= -7.2392898798e-01
       coeff_l3(26,5)= -3.6514174938e-01
       coeff_l3(26,6)= -6.6880607605e-01
       coeff_l3(26,7)= -5.8430945873e-01
       coeff_l3(26,8)= -5.5718785524e-01
       coeff_l3(26,9)= -1.7155097723e+00
       coeff_l3(26,10)= -1.2617397308e+00
       coeff_l3(26,11)= -2.4540235996e+00
       coeff_l3(26,12)= -2.1960909367e+00
       coeff_l3(26,13)= -9.9211454391e-01
       coeff_l3(26,14)= -1.7867989540e+00
       coeff_l3(26,15)=  1.3128685951e-01
       coeff_l3(26,16)= -4.6186241508e-01
       coeff_l3(26,17)= -1.4734607935e-01
       coeff_l3(26,18)= -1.8517881632e-02
       coeff_l3(26,19)= -1.0662974417e-01
       coeff_l3(26,20)= -8.0255359411e-02
       coeff_l3(26,21)= -1.1584884487e-02
       coeff_l3(26,22)=  8.9318752289e-01
       coeff_l3(26,23)=  7.3227030039e-01
       coeff_l3(26,24)= -6.2496703118e-02
       coeff_l3(26,25)=  8.1571459770e-01
       coeff_l3(26,26)= -1.5207581222e-01
       coeff_l3(26,27)=  5.3526312113e-01
       coeff_l3(26,28)= -4.4743391871e-01
       coeff_l3(26,29)= -3.8726603985e-01
       coeff_l3(26,30)=  4.3949636817e-01
       coeff_l3(26,31)=  2.9158926010e-01
       coeff_l3(26,32)= -2.7312612534e-01
       coeff_l3(26,33)=  8.9890494943e-02
       coeff_l3(26,34)= -6.0843136162e-02
       coeff_l3(26,35)=  1.9158828259e-01
       coeff_l3(26,36)= -2.7247393131e-01
       coeff_l3(26,37)=  8.7178844213e-01
       coeff_l3(26,38)=  1.5442334116e-01
       coeff_l3(26,39)=  9.3958638608e-02
       coeff_l3(26,40)= -1.7460847273e-02
       coeff_l3(26,41)= -1.3146521151e-01
       coeff_l3(26,42)= -2.8677199036e-02
       coeff_l3(26,43)= -4.0708050132e-01
       coeff_l3(26,44)= -7.9607613385e-02
       coeff_l3(26,45)= -2.5050330162e-01
       coeff_l3(26,46)= -7.9845631123e-01
       coeff_l3(26,47)=  4.7778975964e-01
       coeff_l3(26,48)=  6.7394191027e-01
       coeff_l3(26,49)=  4.0287297964e-01
       coeff_l3(26,50)= -7.4775552750e-01
       coeff_l3(26,51)= -1.6100895405e-01
       coeff_l3(26,52)= -7.9489916563e-02
       coeff_l3(26,53)= -1.3552759588e-01
       coeff_l3(26,54)=  4.0242472291e-01
       coeff_l3(26,55)=  3.2942283154e-01
       coeff_l3(26,56)=  3.6202493310e-01
       coeff_l3(26,57)= -9.3159005046e-02
       coeff_l3(26,58)=  2.3180011660e-02
       coeff_l3(26,59)=  9.1301664710e-02
       coeff_l3(26,60)= -1.3603588939e-01
       coeff_l3(26,61)= -6.0516756773e-01
       coeff_l3(26,62)= -3.0923694372e-01
       coeff_l3(26,63)= -4.7619432211e-02
       coeff_l3(26,64)=  1.2632519007e-01
       coeff_l3(26,65)=  9.8738439381e-02
       coeff_l3(26,66)=  2.3407181725e-02
       coeff_l3(26,67)= -4.9365963787e-02
       coeff_l3(26,68)=  6.9967627525e-01
       coeff_l3(26,69)=  6.3241457939e-01
       coeff_l3(26,70)=  1.4335818589e-01
       coeff_l3(26,71)= -2.6507592201e-01
       coeff_l3(26,72)=  7.0052541792e-02
       coeff_l3(26,73)= -2.7495793998e-02
       coeff_l3(26,74)=  9.7379994392e-01
       coeff_l3(26,75)=  4.5788207650e-01
       coeff_l3(26,76)=  1.5775309503e-01
       coeff_l3(26,77)=  2.4729274213e-02
       coeff_l3(26,78)= -2.2550080717e-01
       coeff_l3(26,79)=  4.8550102115e-01
       coeff_l3(26,80)=  3.5562247038e-01
       coeff_l3(26,81)=  5.5305856466e-01
       coeff_l3(26,82)= -4.3580472469e-02
       coeff_l3(26,83)=  1.0627168417e+00
       coeff_l3(26,84)=  5.1956146955e-01
       coeff_l3(26,85)=  4.7316823155e-02
       coeff_l3(26,86)= -5.8533376455e-01
       coeff_l3(26,87)=  2.7243128419e-01
       coeff_l3(26,88)=  1.0294972658e+00
       coeff_l3(26,89)=  3.0146211386e-01
       coeff_l3(26,90)=  4.8665082455e-01
       coeff_l3(26,91)= -2.2664816678e-01
       coeff_l3(26,92)=  2.9582878947e-01
       coeff_l3(26,93)=  1.3102224469e-01
       coeff_l3(26,94)=  2.6268982887e-01
       coeff_l3(26,95)= -2.4028784037e-01
       coeff_l3(26,96)=  6.7885214090e-01
       coeff_l3(26,97)=  3.3720600605e-01
       coeff_l3(26,98)=  1.4643925428e-01
       coeff_l3(26,99)= -8.1870788336e-01
       coeff_l3(26,100)=  2.9548370838e-01
       coeff_l3(26,101)=  4.7006142139e-01
       coeff_l3(26,102)= -1.0221209526e+00
       coeff_l3(26,103)= -1.7957332730e-01
       coeff_l3(26,104)=  1.5836168826e-01
       coeff_l3(26,105)= -1.0513968021e-01
       coeff_l3(27,1)= -8.1833970547e-01
       coeff_l3(27,2)= -1.4463664293e+00
       coeff_l3(27,3)= -3.1038665771e-01
       coeff_l3(27,4)= -1.3628653288e+00
       coeff_l3(27,5)= -1.1227393150e+00
       coeff_l3(27,6)= -1.4422695637e+00
       coeff_l3(27,7)=  2.1564027667e-01
       coeff_l3(27,8)= -4.7118043900e-01
       coeff_l3(27,9)=  8.8540039957e-02
       coeff_l3(27,10)= -6.0492104292e-01
       coeff_l3(27,11)= -4.8027658463e-01
       coeff_l3(27,12)=  5.4993655533e-02
       coeff_l3(27,13)= -1.7889553308e-01
       coeff_l3(27,14)=  1.4769001305e-01
       coeff_l3(27,15)=  8.3090955019e-01
       coeff_l3(27,16)= -2.1511761844e-01
       coeff_l3(27,17)= -8.3767545223e-01
       coeff_l3(27,18)=  2.6718300581e-01
       coeff_l3(27,19)= -4.0304952860e-01
       coeff_l3(27,20)= -2.2197119892e-01
       coeff_l3(27,21)=  3.8049009442e-01
       coeff_l3(27,22)= -1.7140294611e-01
       coeff_l3(27,23)=  1.9298051298e-01
       coeff_l3(27,24)= -2.8963282704e-01
       coeff_l3(27,25)= -3.2896932960e-01
       coeff_l3(27,26)=  1.1268200874e+00
       coeff_l3(27,27)= -9.8732642829e-02
       coeff_l3(27,28)=  1.0470281541e-01
       coeff_l3(27,29)= -2.0763838291e-01
       coeff_l3(27,30)=  5.3406160325e-02
       coeff_l3(27,31)= -4.5922300220e-01
       coeff_l3(27,32)=  8.3479374647e-01
       coeff_l3(27,33)= -2.8276044130e-01
       coeff_l3(27,34)= -9.3411082029e-01
       coeff_l3(27,35)=  9.4227895141e-02
       coeff_l3(27,36)=  1.0297596455e+00
       coeff_l3(27,37)= -3.1854218245e-01
       coeff_l3(27,38)= -4.3158760667e-01
       coeff_l3(27,39)=  6.2358760834e-01
       coeff_l3(27,40)= -8.7063121796e-01
       coeff_l3(27,41)= -8.3094745874e-02
       coeff_l3(27,42)=  3.1325328350e-01
       coeff_l3(27,43)=  4.5069038868e-01
       coeff_l3(27,44)=  3.9688923955e-01
       coeff_l3(27,45)=  8.5903769732e-01
       coeff_l3(27,46)=  5.1061773300e-01
       coeff_l3(27,47)= -1.2402554750e+00
       coeff_l3(27,48)= -7.9587393999e-01
       coeff_l3(27,49)=  2.3661319911e-01
       coeff_l3(27,50)= -5.9518980980e-01
       coeff_l3(27,51)= -4.0922265500e-02
       coeff_l3(27,52)=  4.3252885342e-01
       coeff_l3(27,53)= -2.2158764303e-01
       coeff_l3(27,54)= -9.1315436363e-01
       coeff_l3(27,55)= -1.3439677656e-01
       coeff_l3(27,56)= -5.9142988920e-01
       coeff_l3(27,57)= -2.2894696891e-01
       coeff_l3(27,58)=  5.9196621180e-01
       coeff_l3(27,59)= -7.3401350528e-03
       coeff_l3(27,60)= -6.5077859163e-01
       coeff_l3(27,61)= -2.8031235933e-01
       coeff_l3(27,62)=  9.1236221790e-01
       coeff_l3(27,63)= -7.2797662020e-01
       coeff_l3(27,64)=  7.4024921656e-01
       coeff_l3(27,65)= -4.0695619583e-01
       coeff_l3(27,66)=  8.5854744911e-01
       coeff_l3(27,67)=  3.3489477634e-01
       coeff_l3(27,68)= -1.3033598661e+00
       coeff_l3(27,69)= -3.2728489488e-02
       coeff_l3(27,70)=  4.9586659670e-01
       coeff_l3(27,71)=  3.0740043521e-01
       coeff_l3(27,72)=  1.7694832087e+00
       coeff_l3(27,73)= -5.5552196503e-01
       coeff_l3(27,74)= -1.9762243032e+00
       coeff_l3(27,75)=  8.7238675356e-01
       coeff_l3(27,76)=  1.6876526177e-01
       coeff_l3(27,77)= -2.7550560236e-01
       coeff_l3(27,78)= -1.8577070534e-01
       coeff_l3(27,79)=  1.0502892733e-01
       coeff_l3(27,80)= -7.6282858849e-01
       coeff_l3(27,81)= -1.1057579517e+00
       coeff_l3(27,82)= -2.2888244689e-01
       coeff_l3(27,83)= -1.6290293932e+00
       coeff_l3(27,84)= -1.2781176567e+00
       coeff_l3(27,85)=  1.3582801819e+00
       coeff_l3(27,86)=  7.2380208969e-01
       coeff_l3(27,87)=  4.9907618761e-01
       coeff_l3(27,88)=  2.5888720155e-01
       coeff_l3(27,89)=  7.4457031488e-01
       coeff_l3(27,90)= -4.9687826633e-01
       coeff_l3(27,91)=  4.0521621704e-01
       coeff_l3(27,92)= -7.2178453207e-01
       coeff_l3(27,93)= -8.2466500998e-01
       coeff_l3(27,94)=  4.2139518261e-01
       coeff_l3(27,95)= -1.0132511854e+00
       coeff_l3(27,96)=  9.4859492779e-01
       coeff_l3(27,97)= -1.1362564564e+00
       coeff_l3(27,98)=  9.9410289526e-01
       coeff_l3(27,99)=  5.6390136480e-01
       coeff_l3(27,100)= -1.9962690771e-01
       coeff_l3(27,101)= -2.9564321041e-01
       coeff_l3(27,102)= -8.8770184666e-03
       coeff_l3(27,103)= -4.3358817697e-01
       coeff_l3(27,104)=  5.0693064928e-01
       coeff_l3(27,105)=  2.6850824356e+00
       coeff_l3(28,1)=  2.8298977017e-01
       coeff_l3(28,2)=  6.6343337297e-01
       coeff_l3(28,3)=  1.1349087954e+00
       coeff_l3(28,4)=  3.2147544622e-01
       coeff_l3(28,5)=  4.1400501132e-01
       coeff_l3(28,6)=  1.7004741430e+00
       coeff_l3(28,7)=  8.6270225048e-01
       coeff_l3(28,8)=  5.2595263720e-01
       coeff_l3(28,9)=  2.0545141697e+00
       coeff_l3(28,10)=  1.2329233885e+00
       coeff_l3(28,11)=  2.0352852345e+00
       coeff_l3(28,12)=  1.9888703823e+00
       coeff_l3(28,13)=  2.2668485641e+00
       coeff_l3(28,14)=  1.5489896536e+00
       coeff_l3(28,15)= -1.7165717483e-01
       coeff_l3(28,16)= -2.0463545620e-01
       coeff_l3(28,17)=  6.2003910542e-02
       coeff_l3(28,18)=  2.6275175810e-01
       coeff_l3(28,19)= -9.7408139706e-01
       coeff_l3(28,20)= -2.0979708433e-01
       coeff_l3(28,21)=  1.9078473747e-01
       coeff_l3(28,22)= -5.1503950357e-01
       coeff_l3(28,23)=  5.9009808302e-01
       coeff_l3(28,24)=  9.7444915771e-01
       coeff_l3(28,25)=  8.5009589791e-02
       coeff_l3(28,26)= -3.9393657446e-01
       coeff_l3(28,27)=  2.3310998082e-01
       coeff_l3(28,28)=  2.9082590342e-01
       coeff_l3(28,29)=  1.2197054178e-01
       coeff_l3(28,30)=  5.0508743525e-01
       coeff_l3(28,31)= -2.8789123893e-01
       coeff_l3(28,32)=  4.8248130083e-01
       coeff_l3(28,33)=  4.8255136609e-01
       coeff_l3(28,34)=  3.9282079786e-02
       coeff_l3(28,35)=  2.8499515727e-02
       coeff_l3(28,36)=  6.0851269960e-01
       coeff_l3(28,37)= -2.7866852283e-01
       coeff_l3(28,38)= -1.1105039120e+00
       coeff_l3(28,39)= -1.0414205790e+00
       coeff_l3(28,40)= -2.0357939601e-01
       coeff_l3(28,41)= -3.9067247510e-01
       coeff_l3(28,42)=  6.6510893404e-02
       coeff_l3(28,43)=  1.8658766523e-02
       coeff_l3(28,44)=  4.5735752583e-01
       coeff_l3(28,45)=  1.4032539129e+00
       coeff_l3(28,46)=  6.4664769173e-01
       coeff_l3(28,47)= -3.8462972641e-01
       coeff_l3(28,48)= -1.3802821636e+00
       coeff_l3(28,49)=  4.9224701524e-01
       coeff_l3(28,50)= -4.9160307646e-01
       coeff_l3(28,51)= -2.0212249458e-01
       coeff_l3(28,52)=  1.5494577587e-01
       coeff_l3(28,53)= -2.2807148099e-01
       coeff_l3(28,54)= -1.2366320007e-02
       coeff_l3(28,55)= -1.2981122732e-01
       coeff_l3(28,56)= -9.3606084585e-02
       coeff_l3(28,57)= -4.0231442451e-01
       coeff_l3(28,58)=  2.7735975385e-01
       coeff_l3(28,59)= -5.8371365070e-01
       coeff_l3(28,60)= -5.8365762234e-02
       coeff_l3(28,61)= -5.8178079128e-01
       coeff_l3(28,62)=  4.4695886970e-01
       coeff_l3(28,63)= -9.3830488622e-03
       coeff_l3(28,64)= -1.8611112237e-01
       coeff_l3(28,65)=  1.2726859748e-01
       coeff_l3(28,66)=  8.4706342220e-01
       coeff_l3(28,67)=  2.8649371117e-02
       coeff_l3(28,68)= -1.0039781332e+00
       coeff_l3(28,69)= -4.5858031511e-01
       coeff_l3(28,70)=  5.0307780504e-01
       coeff_l3(28,71)= -3.4018087387e-01
       coeff_l3(28,72)=  7.7017116547e-01
       coeff_l3(28,73)= -1.0745786428e+00
       coeff_l3(28,74)= -1.7145555019e+00
       coeff_l3(28,75)=  2.9891985655e-01
       coeff_l3(28,76)=  1.2114652991e-01
       coeff_l3(28,77)= -1.0922749341e-01
       coeff_l3(28,78)=  2.2431802750e-01
       coeff_l3(28,79)=  4.6461051702e-01
       coeff_l3(28,80)= -4.2402452230e-01
       coeff_l3(28,81)= -2.8128877282e-01
       coeff_l3(28,82)= -1.2351221405e-02
       coeff_l3(28,83)= -1.2086324692e+00
       coeff_l3(28,84)= -1.4310581684e+00
       coeff_l3(28,85)=  6.6437709332e-01
       coeff_l3(28,86)=  8.6449390650e-01
       coeff_l3(28,87)= -2.2267645597e-01
       coeff_l3(28,88)= -9.3052864075e-01
       coeff_l3(28,89)=  5.0892984867e-01
       coeff_l3(28,90)= -4.5567882061e-01
       coeff_l3(28,91)=  2.9570636153e-01
       coeff_l3(28,92)= -1.0286304951e+00
       coeff_l3(28,93)= -8.7665545940e-01
       coeff_l3(28,94)=  4.6296320856e-02
       coeff_l3(28,95)= -7.6982980967e-01
       coeff_l3(28,96)=  6.5203034878e-01
       coeff_l3(28,97)= -1.2495623827e+00
       coeff_l3(28,98)=  1.0357536077e+00
       coeff_l3(28,99)=  5.2890878916e-01
       coeff_l3(28,100)= -4.6288070083e-01
       coeff_l3(28,101)= -7.8646814823e-01
       coeff_l3(28,102)=  8.1304118037e-02
       coeff_l3(28,103)= -4.4089892507e-01
       coeff_l3(28,104)=  5.0334155560e-01
       coeff_l3(28,105)=  1.5463581085e+00
       coeff_l3(29,1)= -1.2703798711e-01
       coeff_l3(29,2)= -5.1355743408e-01
       coeff_l3(29,3)= -3.8005009294e-01
       coeff_l3(29,4)= -9.0331417322e-01
       coeff_l3(29,5)= -1.6359143257e+00
       coeff_l3(29,6)= -1.7157285213e+00
       coeff_l3(29,7)= -5.4026514292e-01
       coeff_l3(29,8)= -1.8289840221e+00
       coeff_l3(29,9)= -4.4636949897e-01
       coeff_l3(29,10)=  7.6013165712e-01
       coeff_l3(29,11)= -5.0885331631e-01
       coeff_l3(29,12)=  1.0175999403e+00
       coeff_l3(29,13)= -9.3647450209e-02
       coeff_l3(29,14)=  1.1888721585e-01
       coeff_l3(29,15)=  1.7303808033e-01
       coeff_l3(29,16)= -2.4523997307e-01
       coeff_l3(29,17)= -7.1114927530e-01
       coeff_l3(29,18)=  4.0100893378e-01
       coeff_l3(29,19)= -1.2571631372e-01
       coeff_l3(29,20)= -9.2513263226e-01
       coeff_l3(29,21)=  6.5975660086e-01
       coeff_l3(29,22)= -8.8631439209e-01
       coeff_l3(29,23)=  5.4564309120e-01
       coeff_l3(29,24)= -4.9576349556e-02
       coeff_l3(29,25)=  7.4945557117e-01
       coeff_l3(29,26)=  1.0108210891e-01
       coeff_l3(29,27)=  1.9617882371e-01
       coeff_l3(29,28)= -5.3938385099e-04
       coeff_l3(29,29)=  7.1392536163e-02
       coeff_l3(29,30)=  6.2299984694e-01
       coeff_l3(29,31)=  8.1974327564e-01
       coeff_l3(29,32)=  6.1042100191e-01
       coeff_l3(29,33)= -1.1754257679e+00
       coeff_l3(29,34)= -1.2758116722e+00
       coeff_l3(29,35)= -7.9185187817e-01
       coeff_l3(29,36)= -7.6570463181e-01
       coeff_l3(29,37)=  4.7214359045e-01
       coeff_l3(29,38)= -1.0368342698e-01
       coeff_l3(29,39)= -2.1900112927e-01
       coeff_l3(29,40)= -1.3819763660e+00
       coeff_l3(29,41)= -1.4007070065e+00
       coeff_l3(29,42)= -1.8644529581e-01
       coeff_l3(29,43)=  4.6970120072e-01
       coeff_l3(29,44)=  2.1681231447e-03
       coeff_l3(29,45)= -1.5873050690e+00
       coeff_l3(29,46)= -1.2478782982e-01
       coeff_l3(29,47)= -8.2421171665e-01
       coeff_l3(29,48)= -8.1207448244e-01
       coeff_l3(29,49)=  1.2018341571e-01
       coeff_l3(29,50)= -8.0842477083e-01
       coeff_l3(29,51)=  4.2575061321e-01
       coeff_l3(29,52)= -5.8715730906e-01
       coeff_l3(29,53)= -6.5628033876e-01
       coeff_l3(29,54)=  8.1186598539e-01
       coeff_l3(29,55)= -2.5288105011e-01
       coeff_l3(29,56)=  1.1866952181e+00
       coeff_l3(29,57)=  9.1210472584e-01
       coeff_l3(29,58)=  7.0711404085e-01
       coeff_l3(29,59)= -6.0505896807e-01
       coeff_l3(29,60)= -8.1723198295e-02
       coeff_l3(29,61)= -9.4543164968e-01
       coeff_l3(29,62)= -8.6113131046e-01
       coeff_l3(29,63)=  7.4166007340e-02
       coeff_l3(29,64)=  1.9323842525e+00
       coeff_l3(29,65)=  1.3165077567e-01
       coeff_l3(29,66)= -7.7076807618e-02
       coeff_l3(29,67)= -7.6397895813e-01
       coeff_l3(29,68)=  1.8025906384e-01
       coeff_l3(29,69)=  9.5528525114e-01
       coeff_l3(29,70)=  2.5664269924e-01
       coeff_l3(29,71)= -9.1286909580e-01
       coeff_l3(29,72)= -4.7735208273e-01
       coeff_l3(29,73)=  1.4912180901e+00
       coeff_l3(29,74)=  1.0995166302e+00
       coeff_l3(29,75)= -1.0369920731e+00
       coeff_l3(29,76)=  2.8567001224e-01
       coeff_l3(29,77)= -9.0722191334e-01
       coeff_l3(29,78)=  1.7555886507e-01
       coeff_l3(29,79)=  5.4527223110e-02
       coeff_l3(29,80)=  6.0243022442e-01
       coeff_l3(29,81)=  4.8719185591e-01
       coeff_l3(29,82)=  3.4098458290e-01
       coeff_l3(29,83)=  2.5588586926e-01
       coeff_l3(29,84)=  9.7158448771e-03
       coeff_l3(29,85)=  9.5209491253e-01
       coeff_l3(29,86)=  7.2465926409e-01
       coeff_l3(29,87)=  1.8245664239e-01
       coeff_l3(29,88)=  1.1755328178e+00
       coeff_l3(29,89)= -4.8356002569e-01
       coeff_l3(29,90)=  1.5935790539e-01
       coeff_l3(29,91)=  2.3277458549e-01
       coeff_l3(29,92)=  8.5214787722e-01
       coeff_l3(29,93)= -2.1791703999e-01
       coeff_l3(29,94)=  9.0534895658e-01
       coeff_l3(29,95)=  1.1941727251e-02
       coeff_l3(29,96)= -4.2636033893e-01
       coeff_l3(29,97)=  1.1600695848e+00
       coeff_l3(29,98)=  8.4997758269e-02
       coeff_l3(29,99)= -1.2535060644e+00
       coeff_l3(29,100)=  1.2351647019e-01
       coeff_l3(29,101)=  8.9449185133e-01
       coeff_l3(29,102)= -1.8233235180e-01
       coeff_l3(29,103)=  1.0150886774e+00
       coeff_l3(29,104)= -4.1273033619e-01
       coeff_l3(29,105)= -9.1562736034e-01
       coeff_l3(30,1)= -1.6490764618e+00
       coeff_l3(30,2)= -3.0792419910e+00
       coeff_l3(30,3)= -1.3973294497e+00
       coeff_l3(30,4)= -1.4812222719e+00
       coeff_l3(30,5)= -2.8533852100e+00
       coeff_l3(30,6)= -1.0542839766e+00
       coeff_l3(30,7)=  4.8410374671e-02
       coeff_l3(30,8)= -1.3902403116e+00
       coeff_l3(30,9)=  1.1559072882e-01
       coeff_l3(30,10)= -1.0564044863e-01
       coeff_l3(30,11)=  1.2344635725e+00
       coeff_l3(30,12)=  6.8820703030e-01
       coeff_l3(30,13)=  1.3986892700e+00
       coeff_l3(30,14)=  1.2809257507e+00
       coeff_l3(30,15)=  1.0027420521e+00
       coeff_l3(30,16)=  5.5621021986e-01
       coeff_l3(30,17)=  5.7751494646e-01
       coeff_l3(30,18)=  1.1400456429e+00
       coeff_l3(30,19)=  9.3439811468e-01
       coeff_l3(30,20)=  1.0444699228e-01
       coeff_l3(30,21)=  4.6564266086e-01
       coeff_l3(30,22)= -1.0925493240e+00
       coeff_l3(30,23)=  6.3090503216e-01
       coeff_l3(30,24)=  2.3083844781e-01
       coeff_l3(30,25)= -2.1888203919e-01
       coeff_l3(30,26)=  2.8390979767e-01
       coeff_l3(30,27)= -1.0106378794e+00
       coeff_l3(30,28)=  3.4988200665e-01
       coeff_l3(30,29)=  1.5469039679e+00
       coeff_l3(30,30)= -3.7714126706e-01
       coeff_l3(30,31)= -9.5943653584e-01
       coeff_l3(30,32)=  1.2494479865e-01
       coeff_l3(30,33)= -6.1179715395e-01
       coeff_l3(30,34)= -5.2599674463e-01
       coeff_l3(30,35)= -3.5506588221e-01
       coeff_l3(30,36)=  3.6682346463e-01
       coeff_l3(30,37)= -6.7224782705e-01
       coeff_l3(30,38)= -4.4479674101e-01
       coeff_l3(30,39)= -2.9668331146e-01
       coeff_l3(30,40)= -2.1594827175e+00
       coeff_l3(30,41)= -1.8901665211e+00
       coeff_l3(30,42)=  1.4193867147e-01
       coeff_l3(30,43)=  9.9975770712e-01
       coeff_l3(30,44)= -1.3224072456e+00
       coeff_l3(30,45)= -2.7755051851e-01
       coeff_l3(30,46)= -2.0395159721e-01
       coeff_l3(30,47)= -8.3846545219e-01
       coeff_l3(30,48)= -1.1322834492e+00
       coeff_l3(30,49)= -6.5683263540e-01
       coeff_l3(30,50)=  2.0454816520e-01
       coeff_l3(30,51)=  1.2785574198e+00
       coeff_l3(30,52)= -2.0785611868e-01
       coeff_l3(30,53)= -8.6670957506e-02
       coeff_l3(30,54)= -7.8344482183e-01
       coeff_l3(30,55)=  1.2233786583e+00
       coeff_l3(30,56)= -5.1919597387e-01
       coeff_l3(30,57)=  6.1860305071e-01
       coeff_l3(30,58)= -3.0161494017e-01
       coeff_l3(30,59)= -1.2189217806e+00
       coeff_l3(30,60)=  4.3209204078e-01
       coeff_l3(30,61)=  3.1992462277e-01
       coeff_l3(30,62)=  3.9707678556e-01
       coeff_l3(30,63)=  6.3816273212e-01
       coeff_l3(30,64)=  9.9076676369e-01
       coeff_l3(30,65)= -2.6672241092e-01
       coeff_l3(30,66)=  1.7231293917e+00
       coeff_l3(30,67)= -8.0237466097e-01
       coeff_l3(30,68)= -5.4408669472e-01
       coeff_l3(30,69)= -5.2423417568e-01
       coeff_l3(30,70)=  3.1682822853e-02
       coeff_l3(30,71)=  6.2104332447e-01
       coeff_l3(30,72)= -5.3666532040e-01
       coeff_l3(30,73)=  1.7145072222e+00
       coeff_l3(30,74)=  1.4327757061e-01
       coeff_l3(30,75)= -1.9158676863e+00
       coeff_l3(30,76)=  7.1168768406e-01
       coeff_l3(30,77)= -9.6400952339e-01
       coeff_l3(30,78)=  6.3594180346e-01
       coeff_l3(30,79)= -9.8335731030e-01
       coeff_l3(30,80)= -3.3890753984e-01
       coeff_l3(30,81)= -4.8218682408e-01
       coeff_l3(30,82)= -6.2237703800e-01
       coeff_l3(30,83)= -1.7954704165e-01
       coeff_l3(30,84)= -1.0541919470e+00
       coeff_l3(30,85)= -1.0866987705e+00
       coeff_l3(30,86)=  4.3955221772e-01
       coeff_l3(30,87)= -3.9793574810e-01
       coeff_l3(30,88)=  3.2179478556e-02
       coeff_l3(30,89)= -4.0772062540e-01
       coeff_l3(30,90)= -9.9832236767e-02
       coeff_l3(30,91)=  7.0318692923e-01
       coeff_l3(30,92)= -5.2532726526e-01
       coeff_l3(30,93)= -1.0248639584e+00
       coeff_l3(30,94)= -2.4747524410e-02
       coeff_l3(30,95)=  1.0718214512e+00
       coeff_l3(30,96)= -2.6867645979e-01
       coeff_l3(30,97)= -1.1328817606e+00
       coeff_l3(30,98)= -2.8464269638e-01
       coeff_l3(30,99)=  1.0730270147e+00
       coeff_l3(30,100)=  2.1036040783e-01
       coeff_l3(30,101)= -8.1273460388e-01
       coeff_l3(30,102)= -1.8333820999e-01
       coeff_l3(30,103)=  2.1337424219e-01
       coeff_l3(30,104)=  4.8603090644e-01
       coeff_l3(30,105)=  1.1191740036e+00
       coeff_l3(31,1)= -2.3787865043e-01
       coeff_l3(31,2)= -6.7789161205e-01
       coeff_l3(31,3)= -1.0954675674e+00
       coeff_l3(31,4)= -4.5688307285e-01
       coeff_l3(31,5)= -3.4265637398e-01
       coeff_l3(31,6)= -1.6870422363e+00
       coeff_l3(31,7)= -1.0177253485e+00
       coeff_l3(31,8)= -6.8272602558e-01
       coeff_l3(31,9)= -2.3048892021e+00
       coeff_l3(31,10)= -1.2057559490e+00
       coeff_l3(31,11)= -2.3758981228e+00
       coeff_l3(31,12)= -2.1387577057e+00
       coeff_l3(31,13)= -2.5971381664e+00
       coeff_l3(31,14)= -2.0655076504e+00
       coeff_l3(31,15)= -2.7084812522e-02
       coeff_l3(31,16)=  1.8247038126e-01
       coeff_l3(31,17)= -4.3812502176e-02
       coeff_l3(31,18)= -2.3249231279e-01
       coeff_l3(31,19)=  1.1385061741e+00
       coeff_l3(31,20)=  1.9448271394e-01
       coeff_l3(31,21)=  1.5452095866e-01
       coeff_l3(31,22)=  6.9006043673e-01
       coeff_l3(31,23)= -6.1208021641e-01
       coeff_l3(31,24)= -9.8351657391e-01
       coeff_l3(31,25)= -2.2799508646e-02
       coeff_l3(31,26)=  5.1371192932e-01
       coeff_l3(31,27)= -3.0439725518e-01
       coeff_l3(31,28)= -5.1240998507e-01
       coeff_l3(31,29)= -3.0083891749e-01
       coeff_l3(31,30)= -5.0640386343e-01
       coeff_l3(31,31)=  4.9166268110e-01
       coeff_l3(31,32)= -4.5597037673e-01
       coeff_l3(31,33)= -2.2807855904e-01
       coeff_l3(31,34)= -1.0945082456e-01
       coeff_l3(31,35)= -2.0811650157e-01
       coeff_l3(31,36)= -6.6916292906e-01
       coeff_l3(31,37)=  5.5055117607e-01
       coeff_l3(31,38)=  1.1448796988e+00
       coeff_l3(31,39)=  1.0116583109e+00
       coeff_l3(31,40)=  2.4934612215e-01
       coeff_l3(31,41)=  4.0938448906e-01
       coeff_l3(31,42)=  1.6348895431e-01
       coeff_l3(31,43)= -2.9151099920e-01
       coeff_l3(31,44)= -5.6190204620e-01
       coeff_l3(31,45)= -1.4460523129e+00
       coeff_l3(31,46)= -6.1433833838e-01
       coeff_l3(31,47)=  5.1772022247e-01
       coeff_l3(31,48)=  1.7920750380e+00
       coeff_l3(31,49)= -5.1142472029e-01
       coeff_l3(31,50)=  5.8473318815e-01
       coeff_l3(31,51)=  4.0814399719e-01
       coeff_l3(31,52)= -1.1078406125e-02
       coeff_l3(31,53)= -3.4607246518e-02
       coeff_l3(31,54)=  6.9592595100e-02
       coeff_l3(31,55)=  1.2035010755e-01
       coeff_l3(31,56)=  2.0533922315e-01
       coeff_l3(31,57)=  2.6606455445e-01
       coeff_l3(31,58)= -4.1247957945e-01
       coeff_l3(31,59)=  6.5175265074e-01
       coeff_l3(31,60)=  4.7404233366e-02
       coeff_l3(31,61)=  3.5149633884e-01
       coeff_l3(31,62)= -3.1551307440e-01
       coeff_l3(31,63)=  1.2223611772e-01
       coeff_l3(31,64)=  4.8668357730e-01
       coeff_l3(31,65)= -3.1813660264e-01
       coeff_l3(31,66)= -1.1019614935e+00
       coeff_l3(31,67)= -1.1156529188e-01
       coeff_l3(31,68)=  1.1053670645e+00
       coeff_l3(31,69)=  4.4598418474e-01
       coeff_l3(31,70)= -5.8399665356e-01
       coeff_l3(31,71)=  2.3139052093e-01
       coeff_l3(31,72)= -7.9214060307e-01
       coeff_l3(31,73)=  1.1542052031e+00
       coeff_l3(31,74)=  1.9166914225e+00
       coeff_l3(31,75)= -3.1801694632e-01
       coeff_l3(31,76)=  1.5195515007e-02
       coeff_l3(31,77)=  1.4903752506e-01
       coeff_l3(31,78)= -3.1233760715e-01
       coeff_l3(31,79)= -3.0287593603e-01
       coeff_l3(31,80)=  6.6500550508e-01
       coeff_l3(31,81)=  3.1089699268e-01
       coeff_l3(31,82)=  3.4089248627e-02
       coeff_l3(31,83)=  1.5530854464e+00
       coeff_l3(31,84)=  1.6118743420e+00
       coeff_l3(31,85)= -6.4809018373e-01
       coeff_l3(31,86)= -8.6176615953e-01
       coeff_l3(31,87)=  2.5669750571e-01
       coeff_l3(31,88)=  8.2879161835e-01
       coeff_l3(31,89)= -5.6396824121e-01
       coeff_l3(31,90)=  6.2301564217e-01
       coeff_l3(31,91)= -7.3418527842e-02
       coeff_l3(31,92)=  1.2298251390e+00
       coeff_l3(31,93)=  9.4755578041e-01
       coeff_l3(31,94)=  6.3007213175e-02
       coeff_l3(31,95)=  8.8516324759e-01
       coeff_l3(31,96)= -8.1585168839e-01
       coeff_l3(31,97)=  1.4985930920e+00
       coeff_l3(31,98)= -1.0374996662e+00
       coeff_l3(31,99)= -6.2427371740e-01
       coeff_l3(31,100)=  3.1316137314e-01
       coeff_l3(31,101)=  7.2021764517e-01
       coeff_l3(31,102)=  1.4704899490e-01
       coeff_l3(31,103)=  5.4547834396e-01
       coeff_l3(31,104)= -6.8845862150e-01
       coeff_l3(31,105)= -1.6281049252e+00
       coeff_l3(32,1)=  1.3057470322e-01
       coeff_l3(32,2)=  3.9484006166e-01
       coeff_l3(32,3)=  1.0720978975e+00
       coeff_l3(32,4)=  1.0092496872e+00
       coeff_l3(32,5)=  1.2420783043e+00
       coeff_l3(32,6)=  1.6528890133e+00
       coeff_l3(32,7)=  2.3962745667e+00
       coeff_l3(32,8)=  2.6253881454e+00
       coeff_l3(32,9)=  1.4045990705e+00
       coeff_l3(32,10)=  5.0102257729e-01
       coeff_l3(32,11)=  2.5863176584e-01
       coeff_l3(32,12)=  7.9911571741e-01
       coeff_l3(32,13)=  1.4371948242e+00
       coeff_l3(32,14)=  1.0335834026e+00
       coeff_l3(32,15)=  1.0161374509e-01
       coeff_l3(32,16)=  2.7606654167e-01
       coeff_l3(32,17)= -4.9326175451e-01
       coeff_l3(32,18)=  5.3123408556e-01
       coeff_l3(32,19)=  8.9207649231e-01
       coeff_l3(32,20)=  7.6771730185e-01
       coeff_l3(32,21)=  4.5502299070e-01
       coeff_l3(32,22)= -9.1388918459e-02
       coeff_l3(32,23)= -2.9066967964e-01
       coeff_l3(32,24)= -2.4807875976e-02
       coeff_l3(32,25)= -9.7264736891e-01
       coeff_l3(32,26)=  6.4088545740e-02
       coeff_l3(32,27)= -3.8241413236e-01
       coeff_l3(32,28)=  7.0250965655e-02
       coeff_l3(32,29)=  7.4787700176e-01
       coeff_l3(32,30)= -3.0426514149e-01
       coeff_l3(32,31)= -5.4776924849e-01
       coeff_l3(32,32)= -5.8577889204e-01
       coeff_l3(32,33)= -6.5224510431e-01
       coeff_l3(32,34)= -7.6406663656e-01
       coeff_l3(32,35)=  4.6784493327e-01
       coeff_l3(32,36)=  1.1780607514e-02
       coeff_l3(32,37)= -1.5069083869e-01
       coeff_l3(32,38)= -3.7577462196e-01
       coeff_l3(32,39)= -1.0000041872e-01
       coeff_l3(32,40)= -7.7088469267e-01
       coeff_l3(32,41)= -5.0401532650e-01
       coeff_l3(32,42)=  2.2541327775e-01
       coeff_l3(32,43)= -1.4749786258e-01
       coeff_l3(32,44)=  5.0302588940e-01
       coeff_l3(32,45)=  1.0726898164e-01
       coeff_l3(32,46)= -7.3479121923e-01
       coeff_l3(32,47)= -8.4595900774e-01
       coeff_l3(32,48)= -1.1135770082e+00
       coeff_l3(32,49)= -1.4451627433e-01
       coeff_l3(32,50)= -8.8942182064e-01
       coeff_l3(32,51)=  2.1015641093e-01
       coeff_l3(32,52)=  2.5459340215e-01
       coeff_l3(32,53)= -6.5013414621e-01
       coeff_l3(32,54)=  3.6947962642e-01
       coeff_l3(32,55)=  8.1579577923e-01
       coeff_l3(32,56)=  2.9908758402e-01
       coeff_l3(32,57)= -7.7797240019e-01
       coeff_l3(32,58)=  6.2274795771e-01
       coeff_l3(32,59)= -3.4935623407e-01
       coeff_l3(32,60)=  1.2907472253e-01
       coeff_l3(32,61)= -1.3095363975e-01
       coeff_l3(32,62)=  7.0009484887e-02
       coeff_l3(32,63)=  4.4440126419e-01
       coeff_l3(32,64)= -6.3588745892e-02
       coeff_l3(32,65)=  4.8904106021e-01
       coeff_l3(32,66)= -4.7717279196e-01
       coeff_l3(32,67)=  3.2753416896e-01
       coeff_l3(32,68)= -4.8063439131e-01
       coeff_l3(32,69)= -2.7352967858e-01
       coeff_l3(32,70)= -2.0627079904e-01
       coeff_l3(32,71)= -3.2936342061e-02
       coeff_l3(32,72)=  1.9654518366e-01
       coeff_l3(32,73)=  1.7528006434e-01
       coeff_l3(32,74)= -1.3206187487e+00
       coeff_l3(32,75)= -1.3398326635e+00
       coeff_l3(32,76)= -4.1527625918e-01
       coeff_l3(32,77)= -1.0070096701e-01
       coeff_l3(32,78)=  1.0724041611e-01
       coeff_l3(32,79)= -2.8369981050e-01
       coeff_l3(32,80)= -1.0701125860e+00
       coeff_l3(32,81)= -4.0956586599e-01
       coeff_l3(32,82)=  1.0010589361e+00
       coeff_l3(32,83)=  1.5438146889e-01
       coeff_l3(32,84)=  4.4701671600e-01
       coeff_l3(32,85)= -5.1348078251e-01
       coeff_l3(32,86)=  1.1348257065e+00
       coeff_l3(32,87)=  6.3929390907e-01
       coeff_l3(32,88)= -8.3776122332e-01
       coeff_l3(32,89)=  5.2525416017e-02
       coeff_l3(32,90)= -5.2503924817e-02
       coeff_l3(32,91)=  3.6325886846e-01
       coeff_l3(32,92)=  2.7512142062e-01
       coeff_l3(32,93)=  3.0319858342e-02
       coeff_l3(32,94)=  3.6479923129e-01
       coeff_l3(32,95)= -1.1202264577e-01
       coeff_l3(32,96)= -1.5905313194e-01
       coeff_l3(32,97)=  5.3828948736e-01
       coeff_l3(32,98)= -1.0512282848e+00
       coeff_l3(32,99)= -5.4647143930e-02
       coeff_l3(32,100)= -4.7526657581e-02
       coeff_l3(32,101)=  3.6619238555e-02
       coeff_l3(32,102)=  2.0382722840e-02
       coeff_l3(32,103)=  3.9680442214e-01
       coeff_l3(32,104)= -3.7276414037e-01
       coeff_l3(32,105)= -7.0190989971e-01
       coeff_l3(33,1)=  3.5824365616e+00
       coeff_l3(33,2)=  3.4548945427e+00
       coeff_l3(33,3)=  3.4397442341e+00
       coeff_l3(33,4)=  3.3222670555e+00
       coeff_l3(33,5)=  2.6081683636e+00
       coeff_l3(33,6)=  3.6475870609e+00
       coeff_l3(33,7)=  1.5102019310e+00
       coeff_l3(33,8)=  2.0469999313e+00
       coeff_l3(33,9)=  1.1136718988e+00
       coeff_l3(33,10)=  7.7425137162e-02
       coeff_l3(33,11)=  8.0389767885e-02
       coeff_l3(33,12)=  7.5507313013e-01
       coeff_l3(33,13)=  6.4561265707e-01
       coeff_l3(33,14)=  7.7178633213e-01
       coeff_l3(33,15)= -4.3312919140e-01
       coeff_l3(33,16)= -6.6735786200e-01
       coeff_l3(33,17)= -6.3412450254e-02
       coeff_l3(33,18)= -1.0116869211e-01
       coeff_l3(33,19)= -2.6117008924e-01
       coeff_l3(33,20)= -9.9089048803e-02
       coeff_l3(33,21)=  2.7323189378e-01
       coeff_l3(33,22)= -1.0351002216e+00
       coeff_l3(33,23)= -6.1255478859e-01
       coeff_l3(33,24)= -3.0576792359e-01
       coeff_l3(33,25)= -3.8635820150e-01
       coeff_l3(33,26)=  6.0685163736e-01
       coeff_l3(33,27)= -4.1289651394e-01
       coeff_l3(33,28)= -6.7191076279e-01
       coeff_l3(33,29)=  1.2746416032e-01
       coeff_l3(33,30)= -3.2605981827e-01
       coeff_l3(33,31)=  3.8223835826e-01
       coeff_l3(33,32)= -1.6240689158e-01
       coeff_l3(33,33)= -2.9766359925e-01
       coeff_l3(33,34)= -1.5670675039e-01
       coeff_l3(33,35)= -7.8527368605e-02
       coeff_l3(33,36)= -4.0795508027e-01
       coeff_l3(33,37)=  4.1315117478e-01
       coeff_l3(33,38)= -7.9496222734e-01
       coeff_l3(33,39)= -1.1006005108e-01
       coeff_l3(33,40)= -6.6310507059e-01
       coeff_l3(33,41)= -1.2250881642e-01
       coeff_l3(33,42)=  4.9513965845e-01
       coeff_l3(33,43)= -1.9249966741e-01
       coeff_l3(33,44)=  5.3084588051e-01
       coeff_l3(33,45)=  9.1126573086e-01
       coeff_l3(33,46)= -1.7577692866e-02
       coeff_l3(33,47)=  8.7018050253e-02
       coeff_l3(33,48)=  5.5725592375e-01
       coeff_l3(33,49)=  8.9790213108e-01
       coeff_l3(33,50)= -5.7086610794e-01
       coeff_l3(33,51)=  8.4929279983e-02
       coeff_l3(33,52)=  1.4495185018e-01
       coeff_l3(33,53)=  5.6611608714e-02
       coeff_l3(33,54)=  3.6910894513e-01
       coeff_l3(33,55)=  3.8277575374e-01
       coeff_l3(33,56)=  8.8250356913e-01
       coeff_l3(33,57)=  2.5999695063e-01
       coeff_l3(33,58)=  2.6935800910e-01
       coeff_l3(33,59)= -6.5886354446e-01
       coeff_l3(33,60)= -4.0397489071e-01
       coeff_l3(33,61)= -7.0183783770e-01
       coeff_l3(33,62)=  2.2683691233e-02
       coeff_l3(33,63)=  5.1298874617e-01
       coeff_l3(33,64)= -2.1443942189e-01
       coeff_l3(33,65)=  9.5489126444e-01
       coeff_l3(33,66)= -5.9764248133e-01
       coeff_l3(33,67)= -1.0454726405e-02
       coeff_l3(33,68)= -2.9211461544e-01
       coeff_l3(33,69)=  2.9711994529e-01
       coeff_l3(33,70)=  1.0732597113e+00
       coeff_l3(33,71)= -9.1181272268e-01
       coeff_l3(33,72)= -6.9232439995e-01
       coeff_l3(33,73)= -2.2705736756e-01
       coeff_l3(33,74)=  2.8619581833e-02
       coeff_l3(33,75)=  2.5482445490e-03
       coeff_l3(33,76)=  2.4797803164e-01
       coeff_l3(33,77)= -4.9898535013e-01
       coeff_l3(33,78)=  1.8399642408e-01
       coeff_l3(33,79)=  2.6403185725e-01
       coeff_l3(33,80)= -9.9697850645e-02
       coeff_l3(33,81)=  3.8900020719e-01
       coeff_l3(33,82)=  3.4754272550e-02
       coeff_l3(33,83)=  1.0461293161e-01
       coeff_l3(33,84)=  6.4907141030e-02
       coeff_l3(33,85)=  5.8202105761e-01
       coeff_l3(33,86)=  6.7894995213e-01
       coeff_l3(33,87)=  3.3497422934e-01
       coeff_l3(33,88)= -8.8426738977e-01
       coeff_l3(33,89)=  4.3940252066e-01
       coeff_l3(33,90)=  9.2891669273e-01
       coeff_l3(33,91)= -6.1750149727e-01
       coeff_l3(33,92)=  2.1444718838e+00
       coeff_l3(33,93)=  8.6228901148e-01
       coeff_l3(33,94)= -5.2954858541e-01
       coeff_l3(33,95)= -2.5762721896e-01
       coeff_l3(33,96)=  1.4070752263e-01
       coeff_l3(33,97)=  9.2227065563e-01
       coeff_l3(33,98)= -3.1114798784e-01
       coeff_l3(33,99)= -1.9353365898e-01
       coeff_l3(33,100)= -6.8325978518e-01
       coeff_l3(33,101)= -1.6533103585e-01
       coeff_l3(33,102)=  2.1712365746e-01
       coeff_l3(33,103)= -1.3772235811e-01
       coeff_l3(33,104)= -1.2464596033e+00
       coeff_l3(33,105)= -1.7582737207e+00
       coeff_l3(34,1)=  3.1039886475e+00
       coeff_l3(34,2)=  2.9572939873e+00
       coeff_l3(34,3)=  2.1016035080e+00
       coeff_l3(34,4)=  3.0815622807e+00
       coeff_l3(34,5)=  2.0677077770e+00
       coeff_l3(34,6)=  3.3912358284e+00
       coeff_l3(34,7)=  2.8726069927e+00
       coeff_l3(34,8)=  2.3050003052e+00
       coeff_l3(34,9)=  1.2362947464e+00
       coeff_l3(34,10)=  1.2717186213e+00
       coeff_l3(34,11)=  4.5212063193e-01
       coeff_l3(34,12)=  1.4353014231e+00
       coeff_l3(34,13)=  5.1379048824e-01
       coeff_l3(34,14)=  1.6421506405e+00
       coeff_l3(34,15)= -4.3680197001e-01
       coeff_l3(34,16)= -1.3206316531e-01
       coeff_l3(34,17)= -8.5208386183e-02
       coeff_l3(34,18)=  2.7185949683e-01
       coeff_l3(34,19)= -3.0373498797e-02
       coeff_l3(34,20)=  1.7931239307e-01
       coeff_l3(34,21)=  9.8220729828e-01
       coeff_l3(34,22)= -1.2748825550e-01
       coeff_l3(34,23)= -7.2290903330e-01
       coeff_l3(34,24)= -8.2938295603e-01
       coeff_l3(34,25)= -3.0615848303e-01
       coeff_l3(34,26)=  2.8242266178e-01
       coeff_l3(34,27)= -2.1187195182e-01
       coeff_l3(34,28)= -5.1180040836e-01
       coeff_l3(34,29)=  3.5818594694e-01
       coeff_l3(34,30)= -6.0167503357e-01
       coeff_l3(34,31)=  5.9519076347e-01
       coeff_l3(34,32)= -1.8465393782e-01
       coeff_l3(34,33)=  6.1893269420e-02
       coeff_l3(34,34)=  4.5280465484e-01
       coeff_l3(34,35)= -4.7985130548e-01
       coeff_l3(34,36)=  5.9276449680e-01
       coeff_l3(34,37)=  1.0084050149e-01
       coeff_l3(34,38)=  2.6006451249e-01
       coeff_l3(34,39)=  3.6310088634e-01
       coeff_l3(34,40)= -9.5839315653e-01
       coeff_l3(34,41)=  5.3311467171e-01
       coeff_l3(34,42)=  1.5249428153e-01
       coeff_l3(34,43)=  2.2307366133e-01
       coeff_l3(34,44)=  9.4522583485e-01
       coeff_l3(34,45)= -6.6360914707e-01
       coeff_l3(34,46)= -8.9835095406e-01
       coeff_l3(34,47)=  1.0414590687e-01
       coeff_l3(34,48)=  8.5280072689e-01
       coeff_l3(34,49)= -5.7933872938e-01
       coeff_l3(34,50)=  8.3066159487e-01
       coeff_l3(34,51)= -4.3464562297e-01
       coeff_l3(34,52)=  2.0081141591e-01
       coeff_l3(34,53)= -8.6266323924e-02
       coeff_l3(34,54)= -2.2804915905e-01
       coeff_l3(34,55)= -4.2689558864e-01
       coeff_l3(34,56)=  5.6410568953e-01
       coeff_l3(34,57)=  1.1074510813e+00
       coeff_l3(34,58)=  1.4664806426e-01
       coeff_l3(34,59)=  5.6530731916e-01
       coeff_l3(34,60)=  2.5925874710e-02
       coeff_l3(34,61)= -6.2578982115e-01
       coeff_l3(34,62)= -4.2191526294e-01
       coeff_l3(34,63)= -3.1841713190e-01
       coeff_l3(34,64)=  1.3545492291e-01
       coeff_l3(34,65)=  1.3076627254e-01
       coeff_l3(34,66)= -1.2686903477e+00
       coeff_l3(34,67)= -4.0517199039e-01
       coeff_l3(34,68)=  1.0795919895e+00
       coeff_l3(34,69)=  4.9841837026e-03
       coeff_l3(34,70)=  1.8371863663e-01
       coeff_l3(34,71)=  2.1754187346e-01
       coeff_l3(34,72)=  2.1731126308e-01
       coeff_l3(34,73)= -8.2015521824e-02
       coeff_l3(34,74)=  5.4337557405e-02
       coeff_l3(34,75)= -2.2437578440e-01
       coeff_l3(34,76)=  1.2173012830e-02
       coeff_l3(34,77)=  1.3227375746e+00
       coeff_l3(34,78)= -8.0143493414e-01
       coeff_l3(34,79)=  1.9651150703e-01
       coeff_l3(34,80)= -9.1383881867e-02
       coeff_l3(34,81)=  1.0102889687e-01
       coeff_l3(34,82)=  6.0132080317e-01
       coeff_l3(34,83)= -4.6637481451e-01
       coeff_l3(34,84)=  1.2636615038e+00
       coeff_l3(34,85)= -2.9791671038e-01
       coeff_l3(34,86)= -9.9280482531e-01
       coeff_l3(34,87)= -7.9746887088e-02
       coeff_l3(34,88)= -6.0314434767e-01
       coeff_l3(34,89)=  8.7361794710e-01
       coeff_l3(34,90)= -1.6834321618e-01
       coeff_l3(34,91)= -6.9251292944e-01
       coeff_l3(34,92)=  6.5431666374e-01
       coeff_l3(34,93)=  8.3339494467e-01
       coeff_l3(34,94)=  1.0881283283e+00
       coeff_l3(34,95)=  6.7065727711e-01
       coeff_l3(34,96)= -7.9303252697e-01
       coeff_l3(34,97)=  1.1811763048e+00
       coeff_l3(34,98)=  1.2117476761e-01
       coeff_l3(34,99)=  7.1675342321e-01
       coeff_l3(34,100)=  6.8344068527e-01
       coeff_l3(34,101)=  6.1246210337e-01
       coeff_l3(34,102)=  6.1493766308e-01
       coeff_l3(34,103)= -5.4757958651e-01
       coeff_l3(34,104)= -1.0115215778e+00
       coeff_l3(34,105)= -9.8770755529e-01
       coeff_l3(35,1)=  1.0317234993e+00
       coeff_l3(35,2)=  9.5154374838e-01
       coeff_l3(35,3)=  1.5001465082e+00
       coeff_l3(35,4)=  1.5424218178e+00
       coeff_l3(35,5)=  9.3256741762e-01
       coeff_l3(35,6)=  1.0197595358e+00
       coeff_l3(35,7)= -5.6990589947e-02
       coeff_l3(35,8)=  7.3471105099e-01
       coeff_l3(35,9)=  3.3201959133e+00
       coeff_l3(35,10)=  3.5530676842e+00
       coeff_l3(35,11)=  2.9090106487e+00
       coeff_l3(35,12)=  2.0570445061e+00
       coeff_l3(35,13)=  3.1165416241e+00
       coeff_l3(35,14)=  2.3347117901e+00
       coeff_l3(35,15)= -6.9873714447e-01
       coeff_l3(35,16)= -5.9427421540e-02
       coeff_l3(35,17)= -2.9059115052e-01
       coeff_l3(35,18)=  2.5465133786e-01
       coeff_l3(35,19)=  3.4886771441e-01
       coeff_l3(35,20)=  4.2014947534e-01
       coeff_l3(35,21)=  1.1869899929e-01
       coeff_l3(35,22)= -1.7438422143e-01
       coeff_l3(35,23)=  1.0727906227e+00
       coeff_l3(35,24)= -6.7573153973e-01
       coeff_l3(35,25)= -1.2981322408e-01
       coeff_l3(35,26)=  6.4440637827e-01
       coeff_l3(35,27)= -1.6681157053e-01
       coeff_l3(35,28)=  7.9286277294e-02
       coeff_l3(35,29)= -9.7110018134e-02
       coeff_l3(35,30)=  2.6800447702e-01
       coeff_l3(35,31)= -2.0455712080e-01
       coeff_l3(35,32)=  2.3439303041e-01
       coeff_l3(35,33)=  5.9238914400e-02
       coeff_l3(35,34)=  1.4687514305e+00
       coeff_l3(35,35)=  8.8215768337e-01
       coeff_l3(35,36)=  7.6823920012e-01
       coeff_l3(35,37)= -2.5639474392e-01
       coeff_l3(35,38)= -2.4479785562e-01
       coeff_l3(35,39)= -9.6988308430e-01
       coeff_l3(35,40)= -8.0550864339e-02
       coeff_l3(35,41)= -3.0338966846e-01
       coeff_l3(35,42)= -1.9149725139e-01
       coeff_l3(35,43)= -1.2465224415e-01
       coeff_l3(35,44)=  8.5321277380e-01
       coeff_l3(35,45)= -2.8819224238e-01
       coeff_l3(35,46)= -2.7192112803e-01
       coeff_l3(35,47)=  3.3094751835e-01
       coeff_l3(35,48)= -6.4802378416e-01
       coeff_l3(35,49)= -4.3531006575e-01
       coeff_l3(35,50)= -2.6927113533e-01
       coeff_l3(35,51)= -3.3675143123e-01
       coeff_l3(35,52)=  1.1716620922e+00
       coeff_l3(35,53)=  1.3336131349e-02
       coeff_l3(35,54)= -1.3199821115e-01
       coeff_l3(35,55)=  3.6260503531e-01
       coeff_l3(35,56)= -3.9727061987e-01
       coeff_l3(35,57)= -6.4795029163e-01
       coeff_l3(35,58)= -6.4314758778e-01
       coeff_l3(35,59)= -3.8066565990e-01
       coeff_l3(35,60)=  2.6472702622e-01
       coeff_l3(35,61)=  6.4878058434e-01
       coeff_l3(35,62)=  1.8721239269e-01
       coeff_l3(35,63)= -6.2611870468e-02
       coeff_l3(35,64)= -4.0146505833e-01
       coeff_l3(35,65)=  8.3030372858e-01
       coeff_l3(35,66)= -2.5850284100e-01
       coeff_l3(35,67)= -4.2085751891e-01
       coeff_l3(35,68)= -4.2529571801e-02
       coeff_l3(35,69)= -6.7308235168e-01
       coeff_l3(35,70)= -6.2474972010e-01
       coeff_l3(35,71)= -9.9431949854e-01
       coeff_l3(35,72)=  4.3027546257e-02
       coeff_l3(35,73)= -1.6498279572e+00
       coeff_l3(35,74)= -1.4585033655e+00
       coeff_l3(35,75)=  9.9123731256e-02
       coeff_l3(35,76)= -1.4821702242e+00
       coeff_l3(35,77)=  1.2353360653e+00
       coeff_l3(35,78)= -6.8831205368e-01
       coeff_l3(35,79)=  1.3446841240e+00
       coeff_l3(35,80)= -1.6859544516e+00
       coeff_l3(35,81)=  7.9557436705e-01
       coeff_l3(35,82)= -4.3714994192e-01
       coeff_l3(35,83)=  4.9418441951e-02
       coeff_l3(35,84)= -8.2152110338e-01
       coeff_l3(35,85)= -6.1568051577e-01
       coeff_l3(35,86)=  2.0199215412e-01
       coeff_l3(35,87)= -5.7383265346e-02
       coeff_l3(35,88)= -8.6196100712e-01
       coeff_l3(35,89)=  2.5089821219e-01
       coeff_l3(35,90)=  2.9467356205e-01
       coeff_l3(35,91)= -5.8090156317e-01
       coeff_l3(35,92)= -9.3864953518e-01
       coeff_l3(35,93)= -1.0203474760e+00
       coeff_l3(35,94)=  2.6520609856e-01
       coeff_l3(35,95)= -8.8456958532e-01
       coeff_l3(35,96)=  1.0614972115e+00
       coeff_l3(35,97)= -1.9237841666e-01
       coeff_l3(35,98)= -6.3124907017e-01
       coeff_l3(35,99)=  3.5634715110e-02
       coeff_l3(35,100)= -4.5604214072e-01
       coeff_l3(35,101)=  4.0464848280e-01
       coeff_l3(35,102)= -1.2808240652e+00
       coeff_l3(35,103)= -2.4995303154e-01
       coeff_l3(35,104)=  1.2928006053e-01
       coeff_l3(35,105)= -6.1101502180e-01
       coeff_l3(36,1)=  4.2058703303e-01
       coeff_l3(36,2)=  2.4876941741e-01
       coeff_l3(36,3)=  3.0750662088e-01
       coeff_l3(36,4)=  3.1747996807e-01
       coeff_l3(36,5)=  5.3300124407e-01
       coeff_l3(36,6)=  2.0429451019e-02
       coeff_l3(36,7)=  2.0196636021e-01
       coeff_l3(36,8)= -1.1058924720e-02
       coeff_l3(36,9)= -2.1085183620e+00
       coeff_l3(36,10)= -1.8797703981e+00
       coeff_l3(36,11)= -2.9252557755e+00
       coeff_l3(36,12)= -1.8221757412e+00
       coeff_l3(36,13)= -2.3898773193e+00
       coeff_l3(36,14)= -2.1118695736e+00
       coeff_l3(36,15)=  1.1985448748e-01
       coeff_l3(36,16)= -4.2995281518e-02
       coeff_l3(36,17)=  1.3108828664e-01
       coeff_l3(36,18)=  3.3757320046e-01
       coeff_l3(36,19)= -5.4736807942e-02
       coeff_l3(36,20)= -1.5823651850e-01
       coeff_l3(36,21)= -2.8372587636e-02
       coeff_l3(36,22)= -1.2309682369e-01
       coeff_l3(36,23)= -7.1168906987e-02
       coeff_l3(36,24)= -4.1388776898e-01
       coeff_l3(36,25)=  2.8695079684e-01
       coeff_l3(36,26)=  5.6276433170e-02
       coeff_l3(36,27)= -9.1120824218e-03
       coeff_l3(36,28)= -2.3376750946e-01
       coeff_l3(36,29)=  2.0224794745e-01
       coeff_l3(36,30)=  2.1703188121e-01
       coeff_l3(36,31)=  1.0126376897e-01
       coeff_l3(36,32)= -8.2943312824e-02
       coeff_l3(36,33)= -2.0501354337e-01
       coeff_l3(36,34)= -1.5483240783e-01
       coeff_l3(36,35)=  6.4785003662e-02
       coeff_l3(36,36)= -3.9820832014e-01
       coeff_l3(36,37)=  4.0417161584e-01
       coeff_l3(36,38)= -2.9792788625e-01
       coeff_l3(36,39)= -2.8231900930e-01
       coeff_l3(36,40)= -1.2386170775e-01
       coeff_l3(36,41)= -4.4942561537e-02
       coeff_l3(36,42)=  2.6384916902e-01
       coeff_l3(36,43)=  1.8279153109e-01
       coeff_l3(36,44)=  1.1128645390e-02
       coeff_l3(36,45)=  2.6588627696e-01
       coeff_l3(36,46)= -3.7466314435e-01
       coeff_l3(36,47)=  2.8098592162e-01
       coeff_l3(36,48)=  3.2688760757e-01
       coeff_l3(36,49)= -5.3952509165e-01
       coeff_l3(36,50)= -2.0286588371e-01
       coeff_l3(36,51)=  1.1368839443e-01
       coeff_l3(36,52)=  1.5500149131e-01
       coeff_l3(36,53)=  3.3020537347e-02
       coeff_l3(36,54)=  2.5514015928e-02
       coeff_l3(36,55)= -2.2283507884e-01
       coeff_l3(36,56)=  3.8103798032e-01
       coeff_l3(36,57)=  3.6821816117e-02
       coeff_l3(36,58)= -2.3749493062e-01
       coeff_l3(36,59)= -2.5682863593e-01
       coeff_l3(36,60)=  1.4258717000e-01
       coeff_l3(36,61)= -5.9467905760e-01
       coeff_l3(36,62)= -9.7201533616e-02
       coeff_l3(36,63)=  2.0916348696e-01
       coeff_l3(36,64)=  3.0299559236e-01
       coeff_l3(36,65)= -6.2645986676e-02
       coeff_l3(36,66)= -9.1821175814e-01
       coeff_l3(36,67)= -1.9271609187e-01
       coeff_l3(36,68)=  7.6813226938e-01
       coeff_l3(36,69)=  4.5089825988e-01
       coeff_l3(36,70)= -2.0408326387e-01
       coeff_l3(36,71)= -2.4409085512e-01
       coeff_l3(36,72)=  2.4297395349e-01
       coeff_l3(36,73)=  6.8216061592e-01
       coeff_l3(36,74)=  1.4395138621e-01
       coeff_l3(36,75)= -6.2464320660e-01
       coeff_l3(36,76)=  3.5657024384e-01
       coeff_l3(36,77)=  1.4125575125e-01
       coeff_l3(36,78)= -2.2549422085e-01
       coeff_l3(36,79)=  3.4580718726e-02
       coeff_l3(36,80)= -3.6666169763e-02
       coeff_l3(36,81)=  1.6174562275e-01
       coeff_l3(36,82)=  6.0392379761e-01
       coeff_l3(36,83)=  4.8983877897e-01
       coeff_l3(36,84)= -5.5452868342e-02
       coeff_l3(36,85)=  1.7070077360e-01
       coeff_l3(36,86)= -6.5197080374e-02
       coeff_l3(36,87)= -7.9401865602e-02
       coeff_l3(36,88)=  4.7368195653e-01
       coeff_l3(36,89)= -3.3462677151e-02
       coeff_l3(36,90)=  2.2793319076e-02
       coeff_l3(36,91)=  3.1870669127e-01
       coeff_l3(36,92)=  1.3303933144e+00
       coeff_l3(36,93)=  9.0780043602e-01
       coeff_l3(36,94)=  3.8451349735e-01
       coeff_l3(36,95)=  5.4566979408e-01
       coeff_l3(36,96)= -5.7395744324e-01
       coeff_l3(36,97)=  1.3123828173e+00
       coeff_l3(36,98)= -8.6012738943e-01
       coeff_l3(36,99)= -1.0022014380e-01
       coeff_l3(36,100)= -4.4358346611e-02
       coeff_l3(36,101)=  5.0864243507e-01
       coeff_l3(36,102)=  3.7277868390e-01
       coeff_l3(36,103)=  3.5387706757e-01
       coeff_l3(36,104)= -9.2301970720e-01
       coeff_l3(36,105)= -1.4207723141e+00
       coeff_l3(37,1)=  1.9874778986e+00
       coeff_l3(37,2)=  3.1237642765e+00
       coeff_l3(37,3)=  1.1129896641e+00
       coeff_l3(37,4)=  2.3778514862e+00
       coeff_l3(37,5)=  2.4842510223e+00
       coeff_l3(37,6)=  2.3427538872e+00
       coeff_l3(37,7)= -6.6278034449e-01
       coeff_l3(37,8)= -1.1830805987e-01
       coeff_l3(37,9)=  7.6594227552e-01
       coeff_l3(37,10)=  1.3874304295e+00
       coeff_l3(37,11)= -1.7663898468e+00
       coeff_l3(37,12)= -1.6924953088e-02
       coeff_l3(37,13)= -3.1048548222e-01
       coeff_l3(37,14)=  4.2880836874e-02
       coeff_l3(37,15)= -7.1661597490e-01
       coeff_l3(37,16)= -1.2327320576e+00
       coeff_l3(37,17)=  5.5573064089e-01
       coeff_l3(37,18)= -1.9665632248e+00
       coeff_l3(37,19)= -1.1130748987e+00
       coeff_l3(37,20)=  3.2417982817e-01
       coeff_l3(37,21)=  4.7112509608e-01
       coeff_l3(37,22)=  6.8157315254e-02
       coeff_l3(37,23)= -7.4719613791e-01
       coeff_l3(37,24)=  1.1050422192e+00
       coeff_l3(37,25)= -1.7057096958e+00
       coeff_l3(37,26)=  1.6616593301e-01
       coeff_l3(37,27)= -1.5138695240e+00
       coeff_l3(37,28)=  1.1626559496e+00
       coeff_l3(37,29)= -2.5797960758e+00
       coeff_l3(37,30)=  8.6195307970e-01
       coeff_l3(37,31)=  8.1877195835e-01
       coeff_l3(37,32)=  4.9277848005e-01
       coeff_l3(37,33)=  6.0250955820e-01
       coeff_l3(37,34)=  3.2013896108e-01
       coeff_l3(37,35)= -2.7093205601e-02
       coeff_l3(37,36)= -3.5153374076e-02
       coeff_l3(37,37)=  1.6765472293e-01
       coeff_l3(37,38)=  8.0553358793e-01
       coeff_l3(37,39)=  2.5246560574e-01
       coeff_l3(37,40)=  3.1421494484e+00
       coeff_l3(37,41)=  2.2319886684e+00
       coeff_l3(37,42)= -2.7562195063e-01
       coeff_l3(37,43)= -7.3803126812e-01
       coeff_l3(37,44)=  8.2918071747e-01
       coeff_l3(37,45)= -7.5061768293e-01
       coeff_l3(37,46)= -4.2467838526e-01
       coeff_l3(37,47)=  1.4692119360e+00
       coeff_l3(37,48)=  1.1163752079e+00
       coeff_l3(37,49)=  1.0056713820e+00
       coeff_l3(37,50)= -1.8058738708e+00
       coeff_l3(37,51)= -2.3374974728e+00
       coeff_l3(37,52)=  3.5613986850e-01
       coeff_l3(37,53)= -1.6296072006e+00
       coeff_l3(37,54)=  1.1470782757e+00
       coeff_l3(37,55)=  4.6667999029e-01
       coeff_l3(37,56)=  5.2930110693e-01
       coeff_l3(37,57)= -1.0061932802e+00
       coeff_l3(37,58)=  8.8475787640e-01
       coeff_l3(37,59)= -1.4780507982e-01
       coeff_l3(37,60)= -2.1076691151e+00
       coeff_l3(37,61)= -5.9102028608e-01
       coeff_l3(37,62)= -9.5395892859e-01
       coeff_l3(37,63)= -1.9941997528e+00
       coeff_l3(37,64)= -1.2395253181e+00
       coeff_l3(37,65)=  1.0509732179e-02
       coeff_l3(37,66)= -6.8506109715e-01
       coeff_l3(37,67)=  1.5340719223e+00
       coeff_l3(37,68)= -5.1270675659e-01
       coeff_l3(37,69)= -2.9699918628e-01
       coeff_l3(37,70)=  5.1929235458e-01
       coeff_l3(37,71)= -3.9052420855e-01
       coeff_l3(37,72)=  2.7038052678e-02
       coeff_l3(37,73)= -2.0107641220e+00
       coeff_l3(37,74)=  1.9969370365e+00
       coeff_l3(37,75)=  1.9359019399e-01
       coeff_l3(37,76)= -5.6418991089e-01
       coeff_l3(37,77)=  1.2161127329e+00
       coeff_l3(37,78)= -1.0290650129e+00
       coeff_l3(37,79)=  6.5859889984e-01
       coeff_l3(37,80)=  6.5901428461e-01
       coeff_l3(37,81)=  1.9166673422e+00
       coeff_l3(37,82)=  1.4885393381e+00
       coeff_l3(37,83)=  1.4631549120e+00
       coeff_l3(37,84)=  1.4867756367e+00
       coeff_l3(37,85)= -8.7443596125e-01
       coeff_l3(37,86)=  6.6975009441e-01
       coeff_l3(37,87)=  2.4083659649e+00
       coeff_l3(37,88)= -1.0719488859e+00
       coeff_l3(37,89)= -3.4441092610e-01
       coeff_l3(37,90)= -4.7057482600e-01
       coeff_l3(37,91)= -6.1542391777e-01
       coeff_l3(37,92)=  1.0457291603e+00
       coeff_l3(37,93)=  9.8861122131e-01
       coeff_l3(37,94)= -2.0725982189e+00
       coeff_l3(37,95)=  2.2470581532e-01
       coeff_l3(37,96)= -7.3772883415e-01
       coeff_l3(37,97)=  5.0269001722e-01
       coeff_l3(37,98)=  3.4366562963e-01
       coeff_l3(37,99)= -7.9307460785e-01
       coeff_l3(37,100)=  3.8571327925e-01
       coeff_l3(37,101)=  9.3733584881e-01
       coeff_l3(37,102)=  3.4704321623e-01
       coeff_l3(37,103)=  2.9757276177e-01
       coeff_l3(37,104)=  1.8457745016e-01
       coeff_l3(37,105)= -9.8586875200e-01
       coeff_l3(38,1)=  1.4551844597e+00
       coeff_l3(38,2)=  9.9006611109e-01
       coeff_l3(38,3)=  1.4979078770e+00
       coeff_l3(38,4)=  1.2240340710e+00
       coeff_l3(38,5)=  1.8073018789e+00
       coeff_l3(38,6)=  2.2427213192e+00
       coeff_l3(38,7)=  3.3074779510e+00
       coeff_l3(38,8)=  2.5018808842e+00
       coeff_l3(38,9)= -1.6170462966e-01
       coeff_l3(38,10)= -7.8774148226e-01
       coeff_l3(38,11)= -2.9212027788e-01
       coeff_l3(38,12)= -6.4331644773e-01
       coeff_l3(38,13)=  1.4871953428e-01
       coeff_l3(38,14)=  4.2535491288e-02
       coeff_l3(38,15)=  5.9413361549e-01
       coeff_l3(38,16)=  8.3690658212e-02
       coeff_l3(38,17)= -5.5787587166e-01
       coeff_l3(38,18)=  1.0885866731e-01
       coeff_l3(38,19)=  3.6991095543e-01
       coeff_l3(38,20)=  4.2348134518e-01
       coeff_l3(38,21)=  4.0415544063e-02
       coeff_l3(38,22)= -2.6548340917e-01
       coeff_l3(38,23)=  2.2515712678e-01
       coeff_l3(38,24)= -5.4012265056e-02
       coeff_l3(38,25)= -4.4337549806e-01
       coeff_l3(38,26)=  1.7667935789e-01
       coeff_l3(38,27)= -1.4315077662e-01
       coeff_l3(38,28)= -1.4476262033e-01
       coeff_l3(38,29)= -1.2953354418e-01
       coeff_l3(38,30)= -3.7590795755e-01
       coeff_l3(38,31)= -2.6727053523e-01
       coeff_l3(38,32)=  2.7757707238e-01
       coeff_l3(38,33)=  3.6183026433e-01
       coeff_l3(38,34)=  8.1420494244e-03
       coeff_l3(38,35)= -2.7483647689e-02
       coeff_l3(38,36)= -9.7756326199e-02
       coeff_l3(38,37)=  4.6049442887e-01
       coeff_l3(38,38)= -4.7812786698e-01
       coeff_l3(38,39)=  2.8980806470e-01
       coeff_l3(38,40)=  3.3256536722e-01
       coeff_l3(38,41)= -5.4537129402e-01
       coeff_l3(38,42)=  2.1772779524e-01
       coeff_l3(38,43)=  3.3357495070e-01
       coeff_l3(38,44)=  7.6283103228e-01
       coeff_l3(38,45)=  8.5544759035e-01
       coeff_l3(38,46)= -5.4238623381e-01
       coeff_l3(38,47)=  2.3711900413e-01
       coeff_l3(38,48)= -2.3618090898e-02
       coeff_l3(38,49)=  1.7011296004e-02
       coeff_l3(38,50)= -9.3083448708e-02
       coeff_l3(38,51)=  2.7634322643e-02
       coeff_l3(38,52)= -3.3622771502e-01
       coeff_l3(38,53)=  1.4272570610e-01
       coeff_l3(38,54)= -4.1540482640e-01
       coeff_l3(38,55)=  5.2795797586e-01
       coeff_l3(38,56)=  4.7877267003e-01
       coeff_l3(38,57)= -3.4409162402e-01
       coeff_l3(38,58)=  1.4843194187e-01
       coeff_l3(38,59)= -3.2403460145e-01
       coeff_l3(38,60)= -7.4201978743e-02
       coeff_l3(38,61)=  2.2043262422e-01
       coeff_l3(38,62)=  2.8967064619e-01
       coeff_l3(38,63)= -5.5111682415e-01
       coeff_l3(38,64)=  3.0391363427e-02
       coeff_l3(38,65)=  2.5157585740e-01
       coeff_l3(38,66)=  4.1168760508e-03
       coeff_l3(38,67)= -1.8789019436e-02
       coeff_l3(38,68)=  1.9432301819e-01
       coeff_l3(38,69)=  8.4018272161e-01
       coeff_l3(38,70)=  3.6630854011e-01
       coeff_l3(38,71)= -2.1055658162e-01
       coeff_l3(38,72)=  1.8799996376e-01
       coeff_l3(38,73)= -7.4559122324e-01
       coeff_l3(38,74)=  8.3879303932e-01
       coeff_l3(38,75)=  4.6465972066e-01
       coeff_l3(38,76)=  1.6303536296e-01
       coeff_l3(38,77)=  2.9357939959e-01
       coeff_l3(38,78)= -3.9417189360e-01
       coeff_l3(38,79)=  6.1638134718e-01
       coeff_l3(38,80)= -6.9116298109e-03
       coeff_l3(38,81)=  2.9378324747e-01
       coeff_l3(38,82)=  1.1187644303e-01
       coeff_l3(38,83)= -4.0353015065e-02
       coeff_l3(38,84)= -2.5413078070e-01
       coeff_l3(38,85)=  1.3294436038e-01
       coeff_l3(38,86)=  6.8537458777e-02
       coeff_l3(38,87)= -1.6767935455e-01
       coeff_l3(38,88)= -4.2991635203e-01
       coeff_l3(38,89)=  9.1577447951e-02
       coeff_l3(38,90)= -9.3611273915e-03
       coeff_l3(38,91)= -2.4098953605e-01
       coeff_l3(38,92)= -2.5902971625e-01
       coeff_l3(38,93)= -5.8545827866e-01
       coeff_l3(38,94)= -3.1944376230e-01
       coeff_l3(38,95)= -7.8535157442e-01
       coeff_l3(38,96)=  9.0382790565e-01
       coeff_l3(38,97)= -1.8953496218e-01
       coeff_l3(38,98)=  2.5406187773e-01
       coeff_l3(38,99)=  4.8919793963e-01
       coeff_l3(38,100)= -6.3162881136e-01
       coeff_l3(38,101)= -4.5831960440e-01
       coeff_l3(38,102)=  7.6880127192e-01
       coeff_l3(38,103)= -1.4526675940e+00
       coeff_l3(38,104)= -3.8159802556e-02
       coeff_l3(38,105)=  1.0131763220e+00
       coeff_l3(39,1)=  1.6420165300e+00
       coeff_l3(39,2)=  1.5704925060e+00
       coeff_l3(39,3)=  1.4889110327e+00
       coeff_l3(39,4)=  1.0787864923e+00
       coeff_l3(39,5)=  2.7732691765e+00
       coeff_l3(39,6)=  2.5895416737e+00
       coeff_l3(39,7)=  1.6339062452e+00
       coeff_l3(39,8)=  2.2443304062e+00
       coeff_l3(39,9)=  1.1160870790e+00
       coeff_l3(39,10)=  2.9638883471e-01
       coeff_l3(39,11)=  1.9501664639e+00
       coeff_l3(39,12)=  1.7821041346e+00
       coeff_l3(39,13)=  1.5702598095e+00
       coeff_l3(39,14)=  1.8534733057e+00
       coeff_l3(39,15)= -2.3019741476e-01
       coeff_l3(39,16)=  7.0062351227e-01
       coeff_l3(39,17)= -6.1292171478e-01
       coeff_l3(39,18)= -4.3412280083e-01
       coeff_l3(39,19)=  5.5178260803e-01
       coeff_l3(39,20)= -1.1973582208e-01
       coeff_l3(39,21)=  1.5162467957e-01
       coeff_l3(39,22)=  3.4402713180e-01
       coeff_l3(39,23)=  1.3105836511e-01
       coeff_l3(39,24)=  1.3073712587e-01
       coeff_l3(39,25)= -9.6743941307e-01
       coeff_l3(39,26)=  1.4742985964e+00
       coeff_l3(39,27)= -6.5018296242e-01
       coeff_l3(39,28)=  5.9465628862e-01
       coeff_l3(39,29)=  1.0071755946e-01
       coeff_l3(39,30)= -3.9863333106e-01
       coeff_l3(39,31)= -2.7930855751e-01
       coeff_l3(39,32)=  7.4699878693e-02
       coeff_l3(39,33)=  3.6854723096e-01
       coeff_l3(39,34)=  9.6739572473e-04
       coeff_l3(39,35)= -1.6177251935e-01
       coeff_l3(39,36)=  4.7010973096e-01
       coeff_l3(39,37)= -8.1212788820e-01
       coeff_l3(39,38)= -3.6980513483e-02
       coeff_l3(39,39)=  3.5142537951e-01
       coeff_l3(39,40)=  1.8205277622e-01
       coeff_l3(39,41)= -3.3445882797e-01
       coeff_l3(39,42)= -2.5378957391e-01
       coeff_l3(39,43)= -7.2456985712e-02
       coeff_l3(39,44)=  6.2801301479e-01
       coeff_l3(39,45)=  9.6988213062e-01
       coeff_l3(39,46)=  3.9635339379e-01
       coeff_l3(39,47)= -9.2796105146e-01
       coeff_l3(39,48)= -8.5086929798e-01
       coeff_l3(39,49)=  9.9358540773e-01
       coeff_l3(39,50)=  3.4320101142e-01
       coeff_l3(39,51)=  1.8503104150e-01
       coeff_l3(39,52)= -6.3429653645e-02
       coeff_l3(39,53)=  2.7885916829e-01
       coeff_l3(39,54)= -1.0584395379e-01
       coeff_l3(39,55)= -5.9150552750e-01
       coeff_l3(39,56)=  3.0007874966e-01
       coeff_l3(39,57)= -6.2452995777e-01
       coeff_l3(39,58)=  1.0381221771e+00
       coeff_l3(39,59)= -1.8859826028e-01
       coeff_l3(39,60)= -3.8992822170e-01
       coeff_l3(39,61)=  1.1088373661e+00
       coeff_l3(39,62)= -5.4754412174e-01
       coeff_l3(39,63)= -4.0030494332e-01
       coeff_l3(39,64)= -4.9603930116e-01
       coeff_l3(39,65)=  7.7267843485e-01
       coeff_l3(39,66)=  7.6434856653e-01
       coeff_l3(39,67)=  1.4466251433e-01
       coeff_l3(39,68)= -1.3799108267e+00
       coeff_l3(39,69)= -6.5217107534e-01
       coeff_l3(39,70)=  3.6927085370e-02
       coeff_l3(39,71)=  4.2707914114e-01
       coeff_l3(39,72)= -1.8040668219e-02
       coeff_l3(39,73)= -1.2123923302e+00
       coeff_l3(39,74)= -9.7108471394e-01
       coeff_l3(39,75)=  9.4661587477e-01
       coeff_l3(39,76)= -2.8087416291e-01
       coeff_l3(39,77)=  2.8012709692e-02
       coeff_l3(39,78)= -3.3730933070e-01
       coeff_l3(39,79)=  1.8411752582e-01
       coeff_l3(39,80)= -2.2990728915e-01
       coeff_l3(39,81)=  2.7692854404e-01
       coeff_l3(39,82)= -8.1037586927e-01
       coeff_l3(39,83)= -1.7045825720e+00
       coeff_l3(39,84)=  2.5687491894e-01
       coeff_l3(39,85)= -7.9799652100e-01
       coeff_l3(39,86)=  7.0247197151e-01
       coeff_l3(39,87)= -4.1377821565e-01
       coeff_l3(39,88)= -2.8598758578e-01
       coeff_l3(39,89)= -1.2185820937e-01
       coeff_l3(39,90)=  4.9138218164e-01
       coeff_l3(39,91)= -9.3045634031e-01
       coeff_l3(39,92)= -2.4936821461e+00
       coeff_l3(39,93)= -8.6040610075e-01
       coeff_l3(39,94)= -3.5121169686e-01
       coeff_l3(39,95)= -1.9004896879e+00
       coeff_l3(39,96)=  1.5098973513e+00
       coeff_l3(39,97)= -1.0395022631e+00
       coeff_l3(39,98)=  9.9315476418e-01
       coeff_l3(39,99)=  6.6486708820e-02
       coeff_l3(39,100)= -5.3896391392e-01
       coeff_l3(39,101)= -1.4119508862e-01
       coeff_l3(39,102)=  2.4805946276e-02
       coeff_l3(39,103)= -1.6760675907e+00
       coeff_l3(39,104)=  1.5504155308e-03
       coeff_l3(39,105)=  2.3256707191e+00
       coeff_l3(40,1)=  1.7954543829e+00
       coeff_l3(40,2)=  6.4383405447e-01
       coeff_l3(40,3)= -1.2555243075e-01
       coeff_l3(40,4)=  1.6515972614e+00
       coeff_l3(40,5)=  1.1342998743e+00
       coeff_l3(40,6)= -3.9291452616e-02
       coeff_l3(40,7)= -8.5263407230e-01
       coeff_l3(40,8)= -6.0181017034e-03
       coeff_l3(40,9)=  5.1716488600e-01
       coeff_l3(40,10)=  8.0824631453e-01
       coeff_l3(40,11)=  1.4005011320e+00
       coeff_l3(40,12)=  1.1805599928e+00
       coeff_l3(40,13)=  1.2011663616e-01
       coeff_l3(40,14)=  1.0371642113e+00
       coeff_l3(40,15)= -6.5339595079e-01
       coeff_l3(40,16)=  3.9473984390e-02
       coeff_l3(40,17)=  3.6960789561e-01
       coeff_l3(40,18)= -9.6406286955e-01
       coeff_l3(40,19)= -5.7683217525e-01
       coeff_l3(40,20)= -4.8925796151e-01
       coeff_l3(40,21)= -7.5816728175e-02
       coeff_l3(40,22)=  1.6610672474e+00
       coeff_l3(40,23)=  1.5967613459e-01
       coeff_l3(40,24)=  1.5481944084e+00
       coeff_l3(40,25)= -1.1961696297e-01
       coeff_l3(40,26)= -5.0501364470e-01
       coeff_l3(40,27)= -5.2726376057e-01
       coeff_l3(40,28)=  1.6946166754e-01
       coeff_l3(40,29)= -1.7241303623e-01
       coeff_l3(40,30)=  1.0548790544e-01
       coeff_l3(40,31)=  9.8250144720e-01
       coeff_l3(40,32)= -1.2988984585e-01
       coeff_l3(40,33)=  1.0891704559e+00
       coeff_l3(40,34)=  6.7430031300e-01
       coeff_l3(40,35)=  2.9386761785e-01
       coeff_l3(40,36)= -8.2575696707e-01
       coeff_l3(40,37)=  6.9966769218e-01
       coeff_l3(40,38)= -6.0544526204e-03
       coeff_l3(40,39)= -2.6217371225e-01
       coeff_l3(40,40)=  7.7665621042e-01
       coeff_l3(40,41)=  9.4675928354e-02
       coeff_l3(40,42)=  1.7354330420e-01
       coeff_l3(40,43)= -9.2815166712e-01
       coeff_l3(40,44)= -2.3182485998e-01
       coeff_l3(40,45)= -1.0136480331e+00
       coeff_l3(40,46)= -1.2000107765e+00
       coeff_l3(40,47)=  1.0479664803e+00
       coeff_l3(40,48)=  1.3756091595e+00
       coeff_l3(40,49)=  1.1010086536e+00
       coeff_l3(40,50)= -8.8655185699e-01
       coeff_l3(40,51)=  1.1722456664e-01
       coeff_l3(40,52)= -5.5824267864e-01
       coeff_l3(40,53)=  2.3845225573e-01
       coeff_l3(40,54)=  5.0514066219e-01
       coeff_l3(40,55)=  1.2220199406e-01
       coeff_l3(40,56)= -2.5820074603e-02
       coeff_l3(40,57)= -5.0257349014e-01
       coeff_l3(40,58)= -1.0928058624e+00
       coeff_l3(40,59)=  8.3025437593e-01
       coeff_l3(40,60)=  1.6723272204e-01
       coeff_l3(40,61)=  4.3637300842e-03
       coeff_l3(40,62)= -5.7502818108e-01
       coeff_l3(40,63)= -2.5315796956e-03
       coeff_l3(40,64)=  1.2815364599e+00
       coeff_l3(40,65)=  6.1802333593e-01
       coeff_l3(40,66)=  6.4834505320e-01
       coeff_l3(40,67)=  5.0998669863e-01
       coeff_l3(40,68)=  9.5814275742e-01
       coeff_l3(40,69)=  1.2593753338e+00
       coeff_l3(40,70)= -4.2328456044e-01
       coeff_l3(40,71)= -8.0211061239e-01
       coeff_l3(40,72)=  8.2635933161e-01
       coeff_l3(40,73)= -1.3977397680e+00
       coeff_l3(40,74)=  6.4075767994e-01
       coeff_l3(40,75)=  5.8012747765e-01
       coeff_l3(40,76)=  1.5182504058e-01
       coeff_l3(40,77)=  3.9748305082e-01
       coeff_l3(40,78)= -3.2318326831e-01
       coeff_l3(40,79)=  3.6725324392e-01
       coeff_l3(40,80)=  3.0200004578e-01
       coeff_l3(40,81)=  2.0497396588e-02
       coeff_l3(40,82)= -4.0743193030e-01
       coeff_l3(40,83)=  1.9003802538e+00
       coeff_l3(40,84)=  1.1676422358e+00
       coeff_l3(40,85)= -2.9722958803e-01
       coeff_l3(40,86)= -7.3530203104e-01
       coeff_l3(40,87)=  3.1766006351e-01
       coeff_l3(40,88)=  1.1114935875e+00
       coeff_l3(40,89)=  3.4764578938e-01
       coeff_l3(40,90)=  4.9881268293e-02
       coeff_l3(40,91)= -6.6264361143e-01
       coeff_l3(40,92)= -1.1362857819e+00
       coeff_l3(40,93)=  1.4845719934e-01
       coeff_l3(40,94)= -1.2453333288e-01
       coeff_l3(40,95)=  6.9019198418e-02
       coeff_l3(40,96)=  1.4585431814e+00
       coeff_l3(40,97)=  3.1635984778e-01
       coeff_l3(40,98)=  5.1281744242e-01
       coeff_l3(40,99)=  8.1216461957e-02
       coeff_l3(40,100)= -6.1331999302e-01
       coeff_l3(40,101)=  1.0977187008e-01
       coeff_l3(40,102)= -3.7768032402e-02
       coeff_l3(40,103)= -6.0470616817e-01
       coeff_l3(40,104)= -6.5856671333e-01
       coeff_l3(40,105)=  1.7785818875e-01
       coeff_l3(41,1)= -8.0204880238e-01
       coeff_l3(41,2)= -2.2730159760e+00
       coeff_l3(41,3)= -1.7807641029e+00
       coeff_l3(41,4)= -1.7013101578e+00
       coeff_l3(41,5)= -2.0829706192e+00
       coeff_l3(41,6)= -2.3241097927e+00
       coeff_l3(41,7)= -2.5613987446e+00
       coeff_l3(41,8)= -1.8884750605e+00
       coeff_l3(41,9)= -2.2797365189e+00
       coeff_l3(41,10)= -2.5630722046e+00
       coeff_l3(41,11)= -8.4039670229e-01
       coeff_l3(41,12)= -1.3666591644e+00
       coeff_l3(41,13)= -1.5072134733e+00
       coeff_l3(41,14)= -2.5156753063e+00
       coeff_l3(41,15)= -7.2223252058e-01
       coeff_l3(41,16)= -2.7780261636e-01
       coeff_l3(41,17)= -5.2623754740e-01
       coeff_l3(41,18)=  1.7660173774e-01
       coeff_l3(41,19)=  4.3767482042e-01
       coeff_l3(41,20)= -5.8133071661e-01
       coeff_l3(41,21)= -4.7737252712e-01
       coeff_l3(41,22)=  9.5619565248e-01
       coeff_l3(41,23)= -1.7584568262e-01
       coeff_l3(41,24)= -1.3799986243e-01
       coeff_l3(41,25)=  8.5198760033e-01
       coeff_l3(41,26)= -1.1083027124e+00
       coeff_l3(41,27)= -5.9643882513e-01
       coeff_l3(41,28)=  6.3666087389e-01
       coeff_l3(41,29)= -1.3030524254e+00
       coeff_l3(41,30)=  8.8546395302e-01
       coeff_l3(41,31)=  8.7108957767e-01
       coeff_l3(41,32)= -2.0193071663e-01
       coeff_l3(41,33)=  1.8075583875e-01
       coeff_l3(41,34)= -2.4195358753e+00
       coeff_l3(41,35)= -5.4420802742e-02
       coeff_l3(41,36)= -1.3965824842e+00
       coeff_l3(41,37)= -1.0062045604e-01
       coeff_l3(41,38)= -2.6430270076e-01
       coeff_l3(41,39)= -6.1027228832e-01
       coeff_l3(41,40)=  3.3599126339e-01
       coeff_l3(41,41)=  1.3623607159e-01
       coeff_l3(41,42)= -4.9060636759e-01
       coeff_l3(41,43)= -8.1645846367e-02
       coeff_l3(41,44)= -6.8406707048e-01
       coeff_l3(41,45)= -4.7237059474e-01
       coeff_l3(41,46)=  7.8744959831e-01
       coeff_l3(41,47)= -5.6240004301e-01
       coeff_l3(41,48)=  2.0305609703e+00
       coeff_l3(41,49)=  7.6149290800e-01
       coeff_l3(41,50)= -1.3128929138e+00
       coeff_l3(41,51)=  1.4407839775e+00
       coeff_l3(41,52)= -1.3506314754e+00
       coeff_l3(41,53)= -4.7592571378e-01
       coeff_l3(41,54)=  2.1786323190e-01
       coeff_l3(41,55)= -1.5627853572e-01
       coeff_l3(41,56)=  1.6647042334e-01
       coeff_l3(41,57)= -6.2758940458e-01
       coeff_l3(41,58)=  4.9767953157e-01
       coeff_l3(41,59)= -7.7808409929e-01
       coeff_l3(41,60)= -1.3001834154e+00
       coeff_l3(41,61)= -2.9060593247e-01
       coeff_l3(41,62)=  5.4740801454e-02
       coeff_l3(41,63)= -5.0136532634e-02
       coeff_l3(41,64)=  2.3505370617e+00
       coeff_l3(41,65)=  3.5086485744e-01
       coeff_l3(41,66)= -6.1234867573e-01
       coeff_l3(41,67)=  3.8449710608e-01
       coeff_l3(41,68)= -7.3206138611e-01
       coeff_l3(41,69)=  3.2207226753e-01
       coeff_l3(41,70)=  1.0180507600e-01
       coeff_l3(41,71)= -1.2692247629e+00
       coeff_l3(41,72)= -5.9849798679e-01
       coeff_l3(41,73)=  3.8607189059e-01
       coeff_l3(41,74)=  1.9389618635e+00
       coeff_l3(41,75)=  9.5566201210e-01
       coeff_l3(41,76)=  6.0517907143e-01
       coeff_l3(41,77)= -5.8874845505e-01
       coeff_l3(41,78)=  1.0163645744e+00
       coeff_l3(41,79)= -7.8963065147e-01
       coeff_l3(41,80)=  1.8551069498e+00
       coeff_l3(41,81)= -3.6540545523e-02
       coeff_l3(41,82)= -6.3434380293e-01
       coeff_l3(41,83)=  4.4411447644e-01
       coeff_l3(41,84)= -3.0762124062e-01
       coeff_l3(41,85)=  1.3048229218e+00
       coeff_l3(41,86)=  6.6694146395e-01
       coeff_l3(41,87)= -1.1515409499e-01
       coeff_l3(41,88)=  4.2447254062e-01
       coeff_l3(41,89)= -1.3033261895e-01
       coeff_l3(41,90)=  1.4419567585e+00
       coeff_l3(41,91)=  8.5502201691e-03
       coeff_l3(41,92)= -8.0368959904e-01
       coeff_l3(41,93)=  7.2146517038e-01
       coeff_l3(41,94)= -1.8551985919e-01
       coeff_l3(41,95)= -2.3948843777e-01
       coeff_l3(41,96)=  1.2140804529e+00
       coeff_l3(41,97)= -6.3872051239e-01
       coeff_l3(41,98)=  1.0719790459e+00
       coeff_l3(41,99)= -7.9053509235e-01
       coeff_l3(41,100)= -1.7856969833e+00
       coeff_l3(41,101)=  5.2778124809e-01
       coeff_l3(41,102)=  8.0604869127e-01
       coeff_l3(41,103)= -1.2429107428e+00
       coeff_l3(41,104)= -6.3968515396e-01
       coeff_l3(41,105)=  1.1644238234e+00
       coeff_l3(42,1)= -1.0102035999e+00
       coeff_l3(42,2)= -1.8041952848e+00
       coeff_l3(42,3)= -5.2356922626e-01
       coeff_l3(42,4)= -1.2419846058e+00
       coeff_l3(42,5)= -6.0676914454e-01
       coeff_l3(42,6)=  4.1665613651e-02
       coeff_l3(42,7)=  2.0970535278e-01
       coeff_l3(42,8)= -1.8592983484e-01
       coeff_l3(42,9)= -1.2277172804e+00
       coeff_l3(42,10)= -1.4760187864e+00
       coeff_l3(42,11)= -9.7388523817e-01
       coeff_l3(42,12)= -1.8965742588e+00
       coeff_l3(42,13)= -1.1538270712e+00
       coeff_l3(42,14)= -1.5614954233e+00
       coeff_l3(42,15)=  4.1710385680e-01
       coeff_l3(42,16)= -9.0679302812e-02
       coeff_l3(42,17)= -4.5488438010e-01
       coeff_l3(42,18)=  7.0513027906e-01
       coeff_l3(42,19)= -3.0937778950e-01
       coeff_l3(42,20)= -8.6736857891e-01
       coeff_l3(42,21)= -7.4328434467e-01
       coeff_l3(42,22)=  2.7681803703e-01
       coeff_l3(42,23)= -2.3099198937e-01
       coeff_l3(42,24)=  5.1082789898e-01
       coeff_l3(42,25)= -4.4481050223e-02
       coeff_l3(42,26)=  2.2727534175e-01
       coeff_l3(42,27)= -5.0106763840e-01
       coeff_l3(42,28)= -2.6057036594e-03
       coeff_l3(42,29)= -7.0926517248e-01
       coeff_l3(42,30)=  5.4830837250e-01
       coeff_l3(42,31)=  1.8738147616e-01
       coeff_l3(42,32)=  9.4484341145e-01
       coeff_l3(42,33)=  3.3222410083e-01
       coeff_l3(42,34)=  2.9719737172e-01
       coeff_l3(42,35)=  1.3238453865e-01
       coeff_l3(42,36)= -1.7691606283e-01
       coeff_l3(42,37)=  8.7323886156e-01
       coeff_l3(42,38)=  9.2064514756e-02
       coeff_l3(42,39)=  2.5364083052e-01
       coeff_l3(42,40)=  3.5818508267e-01
       coeff_l3(42,41)= -1.8977405131e-01
       coeff_l3(42,42)=  3.4666791558e-01
       coeff_l3(42,43)= -1.0522578657e-01
       coeff_l3(42,44)=  8.9738093317e-02
       coeff_l3(42,45)=  3.8818836212e-01
       coeff_l3(42,46)= -8.3389496803e-01
       coeff_l3(42,47)=  1.0504759103e-01
       coeff_l3(42,48)=  5.7650589943e-01
       coeff_l3(42,49)=  1.1365087032e+00
       coeff_l3(42,50)= -1.3351881504e+00
       coeff_l3(42,51)=  1.0467195511e+00
       coeff_l3(42,52)= -8.9062249660e-01
       coeff_l3(42,53)= -2.9772058129e-01
       coeff_l3(42,54)=  7.2463136911e-01
       coeff_l3(42,55)=  3.4558704495e-01
       coeff_l3(42,56)= -6.2614783645e-02
       coeff_l3(42,57)= -1.8708090484e-01
       coeff_l3(42,58)= -6.8267440796e-01
       coeff_l3(42,59)= -5.2913993597e-01
       coeff_l3(42,60)= -1.0253883600e+00
       coeff_l3(42,61)= -1.6372342110e+00
       coeff_l3(42,62)= -5.3781682253e-01
       coeff_l3(42,63)= -7.9478156567e-01
       coeff_l3(42,64)=  5.4965782166e-01
       coeff_l3(42,65)= -1.8987385929e-01
       coeff_l3(42,66)= -1.6146054864e-01
       coeff_l3(42,67)= -4.8816433549e-01
       coeff_l3(42,68)=  2.4706770480e-01
       coeff_l3(42,69)=  1.7140015960e-01
       coeff_l3(42,70)= -1.0641208291e-01
       coeff_l3(42,71)= -1.4621889591e+00
       coeff_l3(42,72)= -1.1385605484e-01
       coeff_l3(42,73)=  5.9615808725e-01
       coeff_l3(42,74)= -3.9032921195e-02
       coeff_l3(42,75)=  7.2653239965e-01
       coeff_l3(42,76)=  2.5961551070e-01
       coeff_l3(42,77)=  3.3209195733e-01
       coeff_l3(42,78)=  5.3440481424e-01
       coeff_l3(42,79)=  7.8324913979e-01
       coeff_l3(42,80)=  8.0787621439e-02
       coeff_l3(42,81)=  4.2206283659e-02
       coeff_l3(42,82)= -1.4155765772e+00
       coeff_l3(42,83)=  7.9373198748e-01
       coeff_l3(42,84)= -5.1364696026e-01
       coeff_l3(42,85)=  7.9060661793e-01
       coeff_l3(42,86)= -1.4103240967e+00
       coeff_l3(42,87)=  1.2593813241e-01
       coeff_l3(42,88)=  9.9599444866e-01
       coeff_l3(42,89)= -6.7152959108e-01
       coeff_l3(42,90)=  8.3611434698e-01
       coeff_l3(42,91)=  1.8981708586e-01
       coeff_l3(42,92)= -1.0169041157e+00
       coeff_l3(42,93)=  1.0031212568e+00
       coeff_l3(42,94)=  3.7275931239e-01
       coeff_l3(42,95)= -3.0413210392e-01
       coeff_l3(42,96)=  6.2416332960e-01
       coeff_l3(42,97)=  4.7795984149e-01
       coeff_l3(42,98)=  2.9959994555e-01
       coeff_l3(42,99)= -1.1667306423e+00
       coeff_l3(42,100)= -8.2462704182e-01
       coeff_l3(42,101)=  6.9852453470e-01
       coeff_l3(42,102)= -4.0212598443e-01
       coeff_l3(42,103)= -1.2132312059e+00
       coeff_l3(42,104)= -1.4852225780e-01
       coeff_l3(42,105)=  2.3391443491e-01
       coeff_l3(43,1)=  2.4751427174e+00
       coeff_l3(43,2)=  2.1271946430e+00
       coeff_l3(43,3)=  1.9233263731e+00
       coeff_l3(43,4)=  1.4559562206e+00
       coeff_l3(43,5)=  1.5586555004e+00
       coeff_l3(43,6)=  2.6886186600e+00
       coeff_l3(43,7)=  8.9272785187e-01
       coeff_l3(43,8)=  1.7817056179e+00
       coeff_l3(43,9)=  3.3450524807e+00
       coeff_l3(43,10)=  1.9139968157e+00
       coeff_l3(43,11)=  3.8820126057e+00
       coeff_l3(43,12)=  2.9948720932e+00
       coeff_l3(43,13)=  2.3941652775e+00
       coeff_l3(43,14)=  2.4545834064e+00
       coeff_l3(43,15)=  6.2458115816e-01
       coeff_l3(43,16)=  6.2743471935e-03
       coeff_l3(43,17)= -9.4975598156e-02
       coeff_l3(43,18)= -4.3064862490e-01
       coeff_l3(43,19)= -7.7264375985e-02
       coeff_l3(43,20)= -2.3596949875e-01
       coeff_l3(43,21)= -1.2763777375e-01
       coeff_l3(43,22)= -8.5343068838e-01
       coeff_l3(43,23)=  6.1523014307e-01
       coeff_l3(43,24)= -1.3110750914e-01
       coeff_l3(43,25)= -4.2826071382e-01
       coeff_l3(43,26)=  8.2041434944e-02
       coeff_l3(43,27)= -2.5407103822e-02
       coeff_l3(43,28)= -7.3135346174e-01
       coeff_l3(43,29)=  6.9983547926e-01
       coeff_l3(43,30)= -5.0975853205e-01
       coeff_l3(43,31)=  5.0255340338e-01
       coeff_l3(43,32)=  4.6067652293e-03
       coeff_l3(43,33)= -1.1301533878e-01
       coeff_l3(43,34)=  2.6182377338e-01
       coeff_l3(43,35)=  4.1055500507e-01
       coeff_l3(43,36)=  9.9609696865e-01
       coeff_l3(43,37)= -5.1789647341e-01
       coeff_l3(43,38)= -2.9929082841e-02
       coeff_l3(43,39)= -6.1534970999e-01
       coeff_l3(43,40)= -3.3313763142e-01
       coeff_l3(43,41)=  8.7805995718e-03
       coeff_l3(43,42)= -3.3575236797e-01
       coeff_l3(43,43)= -4.2621293664e-01
       coeff_l3(43,44)= -2.4034747481e-01
       coeff_l3(43,45)=  3.2156217843e-02
       coeff_l3(43,46)=  1.1200243235e+00
       coeff_l3(43,47)= -6.8982672691e-01
       coeff_l3(43,48)= -1.0879966021e+00
       coeff_l3(43,49)=  8.8324677944e-01
       coeff_l3(43,50)= -1.9864992797e-01
       coeff_l3(43,51)=  3.3314126730e-01
       coeff_l3(43,52)=  2.1780298557e-03
       coeff_l3(43,53)=  1.1899960041e-01
       coeff_l3(43,54)=  2.0136941969e-01
       coeff_l3(43,55)=  1.0449111462e-01
       coeff_l3(43,56)= -4.5915412903e-01
       coeff_l3(43,57)= -1.1672865152e+00
       coeff_l3(43,58)=  2.8965282440e-01
       coeff_l3(43,59)= -3.3140867949e-01
       coeff_l3(43,60)= -4.3398734927e-01
       coeff_l3(43,61)= -9.4697870314e-02
       coeff_l3(43,62)=  2.7429595590e-01
       coeff_l3(43,63)=  3.1325605512e-01
       coeff_l3(43,64)=  3.5110071301e-01
       coeff_l3(43,65)= -2.0056070387e-01
       coeff_l3(43,66)=  1.3053077459e+00
       coeff_l3(43,67)=  2.4120160937e-01
       coeff_l3(43,68)= -1.9047745466e+00
       coeff_l3(43,69)= -8.8014906645e-01
       coeff_l3(43,70)=  4.6747702360e-01
       coeff_l3(43,71)=  2.4796250463e-01
       coeff_l3(43,72)= -4.2748544365e-02
       coeff_l3(43,73)= -3.7869116664e-01
       coeff_l3(43,74)= -3.7301906943e-01
       coeff_l3(43,75)=  8.9580559731e-01
       coeff_l3(43,76)=  4.9750249833e-02
       coeff_l3(43,77)= -2.8045699000e-01
       coeff_l3(43,78)= -5.6329798698e-01
       coeff_l3(43,79)=  2.0750632882e-01
       coeff_l3(43,80)= -1.3602611423e-01
       coeff_l3(43,81)=  1.4487618208e-01
       coeff_l3(43,82)= -2.4032659829e-01
       coeff_l3(43,83)= -8.4218525887e-01
       coeff_l3(43,84)= -3.0384838581e-02
       coeff_l3(43,85)= -1.0894106328e-01
       coeff_l3(43,86)=  7.0530349016e-01
       coeff_l3(43,87)=  4.0672950447e-02
       coeff_l3(43,88)= -4.4805120677e-02
       coeff_l3(43,89)= -3.8236200809e-02
       coeff_l3(43,90)=  2.4403682351e-01
       coeff_l3(43,91)= -1.6510595381e-01
       coeff_l3(43,92)=  2.8986459970e-01
       coeff_l3(43,93)=  2.2136431932e-01
       coeff_l3(43,94)= -1.2084058672e-01
       coeff_l3(43,95)= -6.4890438318e-01
       coeff_l3(43,96)=  7.1965229511e-01
       coeff_l3(43,97)= -6.0756200552e-01
       coeff_l3(43,98)= -3.4343042970e-01
       coeff_l3(43,99)=  7.2193521261e-01
       coeff_l3(43,100)= -6.6719585657e-01
       coeff_l3(43,101)= -3.7638720870e-01
       coeff_l3(43,102)= -1.8975991011e-01
       coeff_l3(43,103)= -4.5586770773e-01
       coeff_l3(43,104)=  6.0564690828e-01
       coeff_l3(43,105)=  1.3129703999e+00
       coeff_l3(44,1)=  1.6142717600e+00
       coeff_l3(44,2)=  7.7039080858e-01
       coeff_l3(44,3)=  1.2992796898e+00
       coeff_l3(44,4)=  1.6595215797e+00
       coeff_l3(44,5)=  1.5427973270e+00
       coeff_l3(44,6)=  1.2427792549e+00
       coeff_l3(44,7)=  2.6264834404e+00
       coeff_l3(44,8)=  2.4764151573e+00
       coeff_l3(44,9)=  9.0685433149e-01
       coeff_l3(44,10)=  1.2518477440e+00
       coeff_l3(44,11)=  1.5679301023e+00
       coeff_l3(44,12)=  1.2521159649e+00
       coeff_l3(44,13)=  1.6128221750e+00
       coeff_l3(44,14)=  2.4858417511e+00
       coeff_l3(44,15)=  3.0377200246e-01
       coeff_l3(44,16)=  6.9785080850e-02
       coeff_l3(44,17)= -9.1184034944e-02
       coeff_l3(44,18)=  1.3998891413e-01
       coeff_l3(44,19)=  1.9426979125e-01
       coeff_l3(44,20)= -2.7638936043e-01
       coeff_l3(44,21)=  7.2949743271e-01
       coeff_l3(44,22)=  3.1327286363e-01
       coeff_l3(44,23)=  6.2055331469e-01
       coeff_l3(44,24)= -7.3217338324e-01
       coeff_l3(44,25)=  7.0704765618e-02
       coeff_l3(44,26)= -4.5862782001e-01
       coeff_l3(44,27)=  1.0897984728e-02
       coeff_l3(44,28)=  2.0841488242e-01
       coeff_l3(44,29)= -9.8418988287e-02
       coeff_l3(44,30)= -5.9868580103e-01
       coeff_l3(44,31)= -7.6138836145e-01
       coeff_l3(44,32)= -1.7292392254e-01
       coeff_l3(44,33)= -6.9617921114e-01
       coeff_l3(44,34)=  3.8597121835e-01
       coeff_l3(44,35)= -3.8414591551e-01
       coeff_l3(44,36)= -4.6306360513e-02
       coeff_l3(44,37)= -3.5714599490e-01
       coeff_l3(44,38)= -3.7852796912e-01
       coeff_l3(44,39)=  2.5521239638e-01
       coeff_l3(44,40)= -6.5599077940e-01
       coeff_l3(44,41)= -5.8604305983e-01
       coeff_l3(44,42)= -4.3821263313e-01
       coeff_l3(44,43)= -9.7484039143e-03
       coeff_l3(44,44)= -1.2966281176e-01
       coeff_l3(44,45)=  1.6384863853e-01
       coeff_l3(44,46)=  2.8836098313e-01
       coeff_l3(44,47)= -6.4793944359e-01
       coeff_l3(44,48)= -4.9954551458e-01
       coeff_l3(44,49)=  3.0996337533e-01
       coeff_l3(44,50)= -5.3281515837e-01
       coeff_l3(44,51)=  9.0249103308e-01
       coeff_l3(44,52)= -3.0228659511e-01
       coeff_l3(44,53)=  2.0813897252e-01
       coeff_l3(44,54)=  2.7121515945e-02
       coeff_l3(44,55)=  3.1853136420e-01
       coeff_l3(44,56)= -6.0625773668e-01
       coeff_l3(44,57)=  3.1215235591e-01
       coeff_l3(44,58)=  5.9680974483e-01
       coeff_l3(44,59)=  4.2940995097e-01
       coeff_l3(44,60)=  2.3571868241e-01
       coeff_l3(44,61)=  1.1646775007e+00
       coeff_l3(44,62)= -2.6064488292e-01
       coeff_l3(44,63)=  7.6223236322e-01
       coeff_l3(44,64)= -4.1549876332e-01
       coeff_l3(44,65)=  9.9299287796e-01
       coeff_l3(44,66)=  6.3864898682e-01
       coeff_l3(44,67)= -2.5590041280e-01
       coeff_l3(44,68)= -1.0670708418e+00
       coeff_l3(44,69)=  1.7607927322e-01
       coeff_l3(44,70)=  1.1506193876e+00
       coeff_l3(44,71)=  6.1834591627e-01
       coeff_l3(44,72)= -1.6625648737e-01
       coeff_l3(44,73)= -2.8520274162e-01
       coeff_l3(44,74)=  4.3540385365e-01
       coeff_l3(44,75)=  1.9743254781e-01
       coeff_l3(44,76)= -3.5151398182e-01
       coeff_l3(44,77)= -4.1066020727e-01
       coeff_l3(44,78)=  1.6104739904e-01
       coeff_l3(44,79)= -1.0487012863e+00
       coeff_l3(44,80)=  1.5306505561e-01
       coeff_l3(44,81)= -2.0995816588e-01
       coeff_l3(44,82)= -2.2287710011e-01
       coeff_l3(44,83)= -3.5213580728e-01
       coeff_l3(44,84)=  9.5112629235e-02
       coeff_l3(44,85)= -4.4240501523e-01
       coeff_l3(44,86)=  4.0308940411e-01
       coeff_l3(44,87)= -7.2488987446e-01
       coeff_l3(44,88)=  2.0707121491e-01
       coeff_l3(44,89)=  2.3823454976e-02
       coeff_l3(44,90)=  9.7421519458e-02
       coeff_l3(44,91)=  9.3187084422e-03
       coeff_l3(44,92)= -1.1320794821e+00
       coeff_l3(44,93)= -1.0965623856e+00
       coeff_l3(44,94)= -1.0102810711e-01
       coeff_l3(44,95)= -2.9094514251e-01
       coeff_l3(44,96)=  9.5565539598e-01
       coeff_l3(44,97)= -1.3300629854e+00
       coeff_l3(44,98)=  5.9036374092e-01
       coeff_l3(44,99)=  3.5357823968e-01
       coeff_l3(44,100)= -2.7059698105e-01
       coeff_l3(44,101)= -1.0022395849e+00
       coeff_l3(44,102)=  1.2221693993e-01
       coeff_l3(44,103)= -4.8163408041e-01
       coeff_l3(44,104)=  1.0776003599e+00
       coeff_l3(44,105)=  5.8447033167e-01
       coeff_l3(45,1)= -3.5522073507e-01
       coeff_l3(45,2)= -2.8002527356e-01
       coeff_l3(45,3)= -1.0608316660e+00
       coeff_l3(45,4)= -1.3194400072e-01
       coeff_l3(45,5)=  7.2029215097e-01
       coeff_l3(45,6)=  1.3502274454e-01
       coeff_l3(45,7)= -6.3610137440e-03
       coeff_l3(45,8)=  7.5749123096e-01
       coeff_l3(45,9)= -1.6243035793e+00
       coeff_l3(45,10)= -4.1478443891e-02
       coeff_l3(45,11)= -9.2337155342e-01
       coeff_l3(45,12)=  2.6304426789e-01
       coeff_l3(45,13)=  1.5079100430e-01
       coeff_l3(45,14)= -1.0895125866e+00
       coeff_l3(45,15)=  6.2969237566e-02
       coeff_l3(45,16)= -1.8256509304e-01
       coeff_l3(45,17)=  3.5964226723e-01
       coeff_l3(45,18)= -8.3651739359e-01
       coeff_l3(45,19)= -4.0310904384e-01
       coeff_l3(45,20)= -3.6102595925e-01
       coeff_l3(45,21)= -5.1983422041e-01
       coeff_l3(45,22)=  1.1806036234e+00
       coeff_l3(45,23)= -3.7488129735e-01
       coeff_l3(45,24)=  4.9185806513e-01
       coeff_l3(45,25)= -1.2202172279e+00
       coeff_l3(45,26)= -2.9216134548e-01
       coeff_l3(45,27)= -8.2236464368e-04
       coeff_l3(45,28)=  3.3213460445e-01
       coeff_l3(45,29)=  4.8463860154e-01
       coeff_l3(45,30)=  4.6899333596e-01
       coeff_l3(45,31)=  4.8519313335e-01
       coeff_l3(45,32)=  4.6449053288e-01
       coeff_l3(45,33)= -2.3779538274e-01
       coeff_l3(45,34)= -2.8988066316e-01
       coeff_l3(45,35)=  2.3419171572e-01
       coeff_l3(45,36)= -7.4124908447e-01
       coeff_l3(45,37)=  4.5899799466e-01
       coeff_l3(45,38)=  1.3567720652e+00
       coeff_l3(45,39)=  2.9038624838e-02
       coeff_l3(45,40)=  9.3265998363e-01
       coeff_l3(45,41)=  1.0063329935e+00
       coeff_l3(45,42)= -2.0109352469e-01
       coeff_l3(45,43)=  1.0783341527e-01
       coeff_l3(45,44)=  4.3170467019e-01
       coeff_l3(45,45)=  4.0522214770e-01
       coeff_l3(45,46)= -3.0243250728e-01
       coeff_l3(45,47)=  7.0649373531e-01
       coeff_l3(45,48)=  1.5469741821e+00
       coeff_l3(45,49)=  2.5849136710e-01
       coeff_l3(45,50)=  8.0940979719e-01
       coeff_l3(45,51)=  2.0187328756e-01
       coeff_l3(45,52)=  1.8163803816e+00
       coeff_l3(45,53)= -7.1934223175e-01
       coeff_l3(45,54)= -3.1866642833e-01
       coeff_l3(45,55)= -3.5791289806e-01
       coeff_l3(45,56)= -1.8931174278e-01
       coeff_l3(45,57)=  5.1456481218e-01
       coeff_l3(45,58)= -1.6473799944e-01
       coeff_l3(45,59)= -3.3191761374e-01
       coeff_l3(45,60)= -1.7749003172e+00
       coeff_l3(45,61)=  2.4014991522e-01
       coeff_l3(45,62)=  4.4800269604e-01
       coeff_l3(45,63)= -1.4848672152e+00
       coeff_l3(45,64)=  5.8493047953e-01
       coeff_l3(45,65)= -5.8903217316e-01
       coeff_l3(45,66)=  2.7276772261e-01
       coeff_l3(45,67)= -1.5086044371e-01
       coeff_l3(45,68)=  8.2900410891e-01
       coeff_l3(45,69)=  9.0324467421e-01
       coeff_l3(45,70)= -1.6432591677e+00
       coeff_l3(45,71)=  3.8907038979e-03
       coeff_l3(45,72)= -6.9771450758e-01
       coeff_l3(45,73)= -3.0798664689e-01
       coeff_l3(45,74)=  1.4137543440e+00
       coeff_l3(45,75)= -1.3622307777e-01
       coeff_l3(45,76)=  2.2827595472e-01
       coeff_l3(45,77)=  8.3197611570e-01
       coeff_l3(45,78)=  6.7023336887e-01
       coeff_l3(45,79)=  9.9932366610e-01
       coeff_l3(45,80)=  1.2244323492e+00
       coeff_l3(45,81)=  1.1588306427e+00
       coeff_l3(45,82)=  6.9937378168e-01
       coeff_l3(45,83)=  1.6961792111e-01
       coeff_l3(45,84)=  1.6840670109e+00
       coeff_l3(45,85)= -5.6905990839e-01
       coeff_l3(45,86)= -5.4725673050e-02
       coeff_l3(45,87)=  6.9056344032e-01
       coeff_l3(45,88)=  7.0267105103e-01
       coeff_l3(45,89)= -1.6837699413e+00
       coeff_l3(45,90)=  4.1150093079e-01
       coeff_l3(45,91)=  4.3287423253e-01
       coeff_l3(45,92)= -2.6234966516e-01
       coeff_l3(45,93)= -1.9606845081e-01
       coeff_l3(45,94)= -2.1074122190e-01
       coeff_l3(45,95)= -1.1088391542e+00
       coeff_l3(45,96)= -8.0851721764e-01
       coeff_l3(45,97)= -4.3586730957e-01
       coeff_l3(45,98)= -2.0554983616e-01
       coeff_l3(45,99)= -4.7915077209e-01
       coeff_l3(45,100)=  1.1400114745e-01
       coeff_l3(45,101)= -6.4420467615e-01
       coeff_l3(45,102)= -1.1751970053e+00
       coeff_l3(45,103)=  3.7070505321e-02
       coeff_l3(45,104)=  5.6023484468e-01
       coeff_l3(45,105)= -2.0617917180e-01
       coeff_l3(46,1)= -1.4900219440e+00
       coeff_l3(46,2)= -1.3524144888e+00
       coeff_l3(46,3)= -7.0558905602e-01
       coeff_l3(46,4)=  1.3130456209e-01
       coeff_l3(46,5)= -4.9252003431e-01
       coeff_l3(46,6)= -4.3091624975e-01
       coeff_l3(46,7)= -5.7365097106e-02
       coeff_l3(46,8)= -8.3558976650e-01
       coeff_l3(46,9)= -2.9716351032e+00
       coeff_l3(46,10)= -2.4741003513e+00
       coeff_l3(46,11)= -3.2221548557e+00
       coeff_l3(46,12)= -2.8752889633e+00
       coeff_l3(46,13)= -2.7166194916e+00
       coeff_l3(46,14)= -1.4961436987e+00
       coeff_l3(46,15)= -2.8063926101e-01
       coeff_l3(46,16)= -3.9823513478e-02
       coeff_l3(46,17)= -1.0934913158e-01
       coeff_l3(46,18)= -1.9972538948e-01
       coeff_l3(46,19)= -5.4548597336e-01
       coeff_l3(46,20)= -2.0515224338e-01
       coeff_l3(46,21)=  4.4679671526e-02
       coeff_l3(46,22)=  3.8909876347e-01
       coeff_l3(46,23)= -1.8164102733e-01
       coeff_l3(46,24)=  3.2303372025e-01
       coeff_l3(46,25)=  7.0297819376e-01
       coeff_l3(46,26)= -6.6645485163e-01
       coeff_l3(46,27)= -7.4062854052e-01
       coeff_l3(46,28)= -1.1663319916e-01
       coeff_l3(46,29)= -2.7289885283e-01
       coeff_l3(46,30)=  1.6014895439e+00
       coeff_l3(46,31)= -6.5243273973e-01
       coeff_l3(46,32)=  1.2983584404e-01
       coeff_l3(46,33)=  6.2459337711e-01
       coeff_l3(46,34)= -9.1884636879e-01
       coeff_l3(46,35)=  4.9234890938e-01
       coeff_l3(46,36)= -1.0934172869e+00
       coeff_l3(46,37)=  4.2938604951e-01
       coeff_l3(46,38)= -6.7283183336e-01
       coeff_l3(46,39)= -6.6095048189e-01
       coeff_l3(46,40)=  3.3155611157e-01
       coeff_l3(46,41)=  7.4998021126e-01
       coeff_l3(46,42)=  4.5148161054e-01
       coeff_l3(46,43)=  4.3314164877e-01
       coeff_l3(46,44)= -1.2629432976e-01
       coeff_l3(46,45)=  7.3561990261e-01
       coeff_l3(46,46)= -7.7430844307e-01
       coeff_l3(46,47)=  1.1875435114e+00
       coeff_l3(46,48)=  1.1638104916e+00
       coeff_l3(46,49)=  4.2501795292e-01
       coeff_l3(46,50)= -7.2258919477e-01
       coeff_l3(46,51)=  3.6096104980e-01
       coeff_l3(46,52)=  5.4559445381e-01
       coeff_l3(46,53)= -1.2282509357e-02
       coeff_l3(46,54)=  7.3553925753e-01
       coeff_l3(46,55)= -1.3068590164e+00
       coeff_l3(46,56)= -2.3112791777e-01
       coeff_l3(46,57)=  1.0543404818e+00
       coeff_l3(46,58)= -1.9474902153e+00
       coeff_l3(46,59)= -4.5921897888e-01
       coeff_l3(46,60)= -3.6124858260e-01
       coeff_l3(46,61)=  5.9777182341e-01
       coeff_l3(46,62)= -8.7923842669e-01
       coeff_l3(46,63)= -1.1144340038e-01
       coeff_l3(46,64)= -1.3406515121e-01
       coeff_l3(46,65)=  1.8370409310e-01
       coeff_l3(46,66)= -4.4224226475e-01
       coeff_l3(46,67)= -6.1634343863e-01
       coeff_l3(46,68)=  1.4592022896e+00
       coeff_l3(46,69)=  1.5905761719e+00
       coeff_l3(46,70)=  2.5017198920e-01
       coeff_l3(46,71)= -3.0095866323e-01
       coeff_l3(46,72)=  1.0361107588e+00
       coeff_l3(46,73)=  3.1750574708e-01
       coeff_l3(46,74)=  1.0339971781e+00
       coeff_l3(46,75)=  1.4264115095e+00
       coeff_l3(46,76)=  4.4720637798e-01
       coeff_l3(46,77)=  1.8390811980e-01
       coeff_l3(46,78)= -5.6025183201e-01
       coeff_l3(46,79)=  1.3584773540e+00
       coeff_l3(46,80)=  1.9815777242e-01
       coeff_l3(46,81)=  7.8719168901e-01
       coeff_l3(46,82)= -2.5455197692e-01
       coeff_l3(46,83)=  7.1882808208e-01
       coeff_l3(46,84)= -1.7034394741e+00
       coeff_l3(46,85)=  1.0454888344e+00
       coeff_l3(46,86)= -3.8028952479e-01
       coeff_l3(46,87)= -1.6522428989e+00
       coeff_l3(46,88)=  4.0869009495e-01
       coeff_l3(46,89)= -5.4903405905e-01
       coeff_l3(46,90)= -1.9804489613e-01
       coeff_l3(46,91)=  5.6943947077e-01
       coeff_l3(46,92)=  2.8732407372e-03
       coeff_l3(46,93)=  1.5646489859e+00
       coeff_l3(46,94)=  1.1538565159e+00
       coeff_l3(46,95)=  7.8017830849e-02
       coeff_l3(46,96)= -3.5897520185e-01
       coeff_l3(46,97)=  6.2314438820e-01
       coeff_l3(46,98)= -6.0994875431e-01
       coeff_l3(46,99)= -4.6444290876e-01
       coeff_l3(46,100)= -3.8304603100e-01
       coeff_l3(46,101)=  3.1987464428e-01
       coeff_l3(46,102)=  6.1912751198e-01
       coeff_l3(46,103)= -2.8154507279e-01
       coeff_l3(46,104)= -1.3090057373e+00
       coeff_l3(46,105)= -5.9677022696e-01
       coeff_l3(47,1)=  2.0753599703e-01
       coeff_l3(47,2)=  3.7922042608e-01
       coeff_l3(47,3)=  1.4307543039e+00
       coeff_l3(47,4)=  7.2335678339e-01
       coeff_l3(47,5)=  2.0814120770e+00
       coeff_l3(47,6)=  4.9550539255e-01
       coeff_l3(47,7)= -1.8770873547e-02
       coeff_l3(47,8)= -4.4627240300e-01
       coeff_l3(47,9)= -4.5009604096e-01
       coeff_l3(47,10)=  4.1258969903e-01
       coeff_l3(47,11)=  6.9312673807e-01
       coeff_l3(47,12)=  1.0999673605e+00
       coeff_l3(47,13)=  1.9358740747e-01
       coeff_l3(47,14)=  9.1602480412e-01
       coeff_l3(47,15)=  4.4843637943e-01
       coeff_l3(47,16)= -8.2796476781e-02
       coeff_l3(47,17)= -1.3782510161e-01
       coeff_l3(47,18)=  6.7751467228e-02
       coeff_l3(47,19)= -7.9698258638e-01
       coeff_l3(47,20)=  1.0398329496e+00
       coeff_l3(47,21)=  1.0035395622e+00
       coeff_l3(47,22)= -2.3713226318e+00
       coeff_l3(47,23)= -4.9834874272e-01
       coeff_l3(47,24)=  4.3492937088e-01
       coeff_l3(47,25)= -1.3904451132e+00
       coeff_l3(47,26)=  5.7630014420e-01
       coeff_l3(47,27)= -9.0677475929e-01
       coeff_l3(47,28)=  5.2478641272e-01
       coeff_l3(47,29)=  4.0291777253e-01
       coeff_l3(47,30)= -2.7147653699e-01
       coeff_l3(47,31)= -4.1710284352e-01
       coeff_l3(47,32)=  3.6065533757e-01
       coeff_l3(47,33)=  8.3624869585e-01
       coeff_l3(47,34)= -1.5736964941e+00
       coeff_l3(47,35)= -3.3363038301e-01
       coeff_l3(47,36)=  2.9600611329e-01
       coeff_l3(47,37)= -8.5558587313e-01
       coeff_l3(47,38)=  1.4364761114e-01
       coeff_l3(47,39)=  1.6792188585e-01
       coeff_l3(47,40)= -9.4373518229e-01
       coeff_l3(47,41)= -2.8127232194e-01
       coeff_l3(47,42)=  7.9511827230e-01
       coeff_l3(47,43)=  5.6996381283e-01
       coeff_l3(47,44)=  5.1050037146e-02
       coeff_l3(47,45)=  5.6646335125e-01
       coeff_l3(47,46)=  6.1646783352e-01
       coeff_l3(47,47)= -2.3015449941e-01
       coeff_l3(47,48)= -9.3906211853e-01
       coeff_l3(47,49)= -1.8952518702e-01
       coeff_l3(47,50)=  1.7122139037e-01
       coeff_l3(47,51)= -1.4482708275e-01
       coeff_l3(47,52)=  3.6764064431e-01
       coeff_l3(47,53)=  2.4779034778e-02
       coeff_l3(47,54)=  3.5672688484e-01
       coeff_l3(47,55)=  2.7235174179e-01
       coeff_l3(47,56)= -6.9389879704e-01
       coeff_l3(47,57)=  6.9810318947e-01
       coeff_l3(47,58)= -1.2686008215e+00
       coeff_l3(47,59)= -9.7714453936e-01
       coeff_l3(47,60)=  1.3571449518e+00
       coeff_l3(47,61)=  1.8031068146e-01
       coeff_l3(47,62)=  8.9436048269e-01
       coeff_l3(47,63)= -1.1703706346e-02
       coeff_l3(47,64)= -1.2700219154e+00
       coeff_l3(47,65)= -1.2042481899e+00
       coeff_l3(47,66)=  1.8899939954e-01
       coeff_l3(47,67)= -1.1164596081e+00
       coeff_l3(47,68)=  2.8118965030e-01
       coeff_l3(47,69)= -1.7453938723e+00
       coeff_l3(47,70)=  5.8750230074e-01
       coeff_l3(47,71)=  3.4339699149e-01
       coeff_l3(47,72)=  1.2700630426e+00
       coeff_l3(47,73)=  5.4457581043e-01
       coeff_l3(47,74)= -1.6439880133e+00
       coeff_l3(47,75)=  6.6474199295e-01
       coeff_l3(47,76)=  7.5245523453e-01
       coeff_l3(47,77)= -1.5767115355e+00
       coeff_l3(47,78)= -5.8462059498e-01
       coeff_l3(47,79)= -7.8204327822e-01
       coeff_l3(47,80)= -1.7004638910e+00
       coeff_l3(47,81)= -1.2168244123e+00
       coeff_l3(47,82)= -6.4100170135e-01
       coeff_l3(47,83)= -3.8432905078e-01
       coeff_l3(47,84)= -2.2712841034e+00
       coeff_l3(47,85)= -3.0868655443e-01
       coeff_l3(47,86)=  9.3707036972e-01
       coeff_l3(47,87)= -1.0590907335e+00
       coeff_l3(47,88)= -1.2667487860e+00
       coeff_l3(47,89)=  6.8076473475e-01
       coeff_l3(47,90)= -2.2084321976e+00
       coeff_l3(47,91)=  1.3905360699e+00
       coeff_l3(47,92)=  2.2653657943e-02
       coeff_l3(47,93)= -2.3229424655e-01
       coeff_l3(47,94)= -2.7217051387e-01
       coeff_l3(47,95)=  6.4886718988e-01
       coeff_l3(47,96)= -1.5578912497e+00
       coeff_l3(47,97)= -2.5480803847e-01
       coeff_l3(47,98)=  3.1987354159e-01
       coeff_l3(47,99)=  9.2178499699e-01
       coeff_l3(47,100)=  3.8769158721e-01
       coeff_l3(47,101)= -1.1358492374e+00
       coeff_l3(47,102)=  2.0305233002e+00
       coeff_l3(47,103)=  1.1486903429e+00
       coeff_l3(47,104)=  5.7837896049e-02
       coeff_l3(47,105)=  5.6982874870e-01
       coeff_l3(48,1)= -1.7186738253e+00
       coeff_l3(48,2)= -2.2445905209e+00
       coeff_l3(48,3)= -1.5708861351e+00
       coeff_l3(48,4)= -1.8130705357e+00
       coeff_l3(48,5)= -2.7906990051e+00
       coeff_l3(48,6)= -2.8140616417e+00
       coeff_l3(48,7)= -2.3266324997e+00
       coeff_l3(48,8)= -3.1995999813e+00
       coeff_l3(48,9)=  4.5082831383e-01
       coeff_l3(48,10)=  8.9790147543e-01
       coeff_l3(48,11)=  1.2428609133e+00
       coeff_l3(48,12)= -1.3054877520e+00
       coeff_l3(48,13)=  1.3464777470e+00
       coeff_l3(48,14)=  1.8409556150e-01
       coeff_l3(48,15)= -3.5401993990e-01
       coeff_l3(48,16)= -5.0813502073e-01
       coeff_l3(48,17)= -1.3932702132e-02
       coeff_l3(48,18)= -8.3403813839e-01
       coeff_l3(48,19)= -5.0285971165e-01
       coeff_l3(48,20)= -3.2344117761e-01
       coeff_l3(48,21)=  8.0196723342e-02
       coeff_l3(48,22)=  6.2813860178e-01
       coeff_l3(48,23)=  6.0424512625e-01
       coeff_l3(48,24)=  1.9782412052e+00
       coeff_l3(48,25)=  1.0126106739e+00
       coeff_l3(48,26)= -4.2001736164e-01
       coeff_l3(48,27)=  3.8412925601e-01
       coeff_l3(48,28)= -9.6231043339e-02
       coeff_l3(48,29)= -1.5606049299e+00
       coeff_l3(48,30)= -3.1458880752e-02
       coeff_l3(48,31)= -2.3983453214e-01
       coeff_l3(48,32)=  8.4301429987e-01
       coeff_l3(48,33)=  4.2083194852e-01
       coeff_l3(48,34)=  4.2580303550e-01
       coeff_l3(48,35)=  2.4188974500e-01
       coeff_l3(48,36)= -6.8472132087e-02
       coeff_l3(48,37)=  4.2616868019e-01
       coeff_l3(48,38)=  9.4019256532e-02
       coeff_l3(48,39)=  5.7327795029e-01
       coeff_l3(48,40)=  5.2048105001e-01
       coeff_l3(48,41)=  3.5968816280e-01
       coeff_l3(48,42)= -4.3100780249e-01
       coeff_l3(48,43)= -4.5257759094e-01
       coeff_l3(48,44)=  1.7875379324e-01
       coeff_l3(48,45)= -5.4923737049e-01
       coeff_l3(48,46)= -1.2557975054e+00
       coeff_l3(48,47)= -4.5450031757e-02
       coeff_l3(48,48)=  5.0701308250e-01
       coeff_l3(48,49)=  2.3270270824e+00
       coeff_l3(48,50)=  3.3321782947e-01
       coeff_l3(48,51)=  4.4537690282e-01
       coeff_l3(48,52)= -5.1956564188e-01
       coeff_l3(48,53)= -4.4377845526e-01
       coeff_l3(48,54)=  1.1750204563e+00
       coeff_l3(48,55)=  4.0789544582e-01
       coeff_l3(48,56)= -8.9391291142e-02
       coeff_l3(48,57)=  8.3425027132e-01
       coeff_l3(48,58)= -3.8269168139e-01
       coeff_l3(48,59)=  1.6384895146e-01
       coeff_l3(48,60)= -1.1598114967e+00
       coeff_l3(48,61)=  1.1329112053e+00
       coeff_l3(48,62)= -4.5574942231e-01
       coeff_l3(48,63)= -7.4942117929e-01
       coeff_l3(48,64)=  2.0450548828e-01
       coeff_l3(48,65)=  8.3261567354e-01
       coeff_l3(48,66)=  1.3912565708e+00
       coeff_l3(48,67)= -7.4097615480e-01
       coeff_l3(48,68)= -7.9918551445e-01
       coeff_l3(48,69)=  8.2713687420e-01
       coeff_l3(48,70)=  1.1588823795e+00
       coeff_l3(48,71)= -4.5199385285e-01
       coeff_l3(48,72)= -7.7074784040e-01
       coeff_l3(48,73)= -6.5689617395e-01
       coeff_l3(48,74)=  1.4367045164e+00
       coeff_l3(48,75)=  1.7450275421e+00
       coeff_l3(48,76)= -1.1676031351e+00
       coeff_l3(48,77)=  2.0280723274e-01
       coeff_l3(48,78)=  1.5960869789e+00
       coeff_l3(48,79)=  1.0878914595e+00
       coeff_l3(48,80)=  6.5714114904e-01
       coeff_l3(48,81)= -6.9467359781e-01
       coeff_l3(48,82)= -2.4376440048e+00
       coeff_l3(48,83)=  6.2173402309e-01
       coeff_l3(48,84)=  1.2594490051e+00
       coeff_l3(48,85)=  2.2371220589e-01
       coeff_l3(48,86)= -9.3284642696e-01
       coeff_l3(48,87)=  5.6415337324e-01
       coeff_l3(48,88)=  1.3751508296e-01
       coeff_l3(48,89)= -5.6940905750e-02
       coeff_l3(48,90)=  9.5124953985e-01
       coeff_l3(48,91)= -6.4249217510e-01
       coeff_l3(48,92)= -1.9129461050e+00
       coeff_l3(48,93)= -5.7536149025e-01
       coeff_l3(48,94)=  4.3066084385e-01
       coeff_l3(48,95)= -7.0337843895e-01
       coeff_l3(48,96)=  5.1712679863e-01
       coeff_l3(48,97)= -1.2040197849e+00
       coeff_l3(48,98)=  1.4779680967e+00
       coeff_l3(48,99)= -6.0864484310e-01
       coeff_l3(48,100)=  6.3851439953e-01
       coeff_l3(48,101)= -4.9426794052e-01
       coeff_l3(48,102)= -1.2998116016e+00
       coeff_l3(48,103)= -3.5140156746e-01
       coeff_l3(48,104)=  7.8107315302e-01
       coeff_l3(48,105)=  6.5265709162e-01
       coeff_l3(49,1)= -2.4371044636e+00
       coeff_l3(49,2)= -1.7382818460e+00
       coeff_l3(49,3)= -3.0477595329e+00
       coeff_l3(49,4)= -2.2651371956e+00
       coeff_l3(49,5)= -1.1643335819e+00
       coeff_l3(49,6)= -1.8260346651e+00
       coeff_l3(49,7)= -8.0476540327e-01
       coeff_l3(49,8)= -1.5857126713e+00
       coeff_l3(49,9)= -2.9511537552e+00
       coeff_l3(49,10)= -1.9601874352e+00
       coeff_l3(49,11)= -1.9336240292e+00
       coeff_l3(49,12)= -2.0781443119e+00
       coeff_l3(49,13)= -1.8226563931e-01
       coeff_l3(49,14)= -2.8514554501e+00
       coeff_l3(49,15)= -4.5031952858e-01
       coeff_l3(49,16)=  4.7873184085e-01
       coeff_l3(49,17)= -5.9327763319e-01
       coeff_l3(49,18)= -1.7796406150e-01
       coeff_l3(49,19)=  7.9972416162e-01
       coeff_l3(49,20)=  5.5438894033e-01
       coeff_l3(49,21)= -8.5568135977e-01
       coeff_l3(49,22)=  1.6188020706e+00
       coeff_l3(49,23)=  7.6638340950e-01
       coeff_l3(49,24)=  7.7214556932e-01
       coeff_l3(49,25)=  1.0093991756e+00
       coeff_l3(49,26)= -1.2291883677e-01
       coeff_l3(49,27)=  8.3998978138e-01
       coeff_l3(49,28)= -3.1599202752e-01
       coeff_l3(49,29)= -7.1249380708e-02
       coeff_l3(49,30)=  6.1887925863e-01
       coeff_l3(49,31)=  9.8198525608e-02
       coeff_l3(49,32)= -3.8766109943e-01
       coeff_l3(49,33)= -2.3117531091e-02
       coeff_l3(49,34)= -7.9666286707e-01
       coeff_l3(49,35)= -7.4977713823e-01
       coeff_l3(49,36)= -8.3467715979e-01
       coeff_l3(49,37)=  1.3307132721e+00
       coeff_l3(49,38)=  1.0204403400e+00
       coeff_l3(49,39)= -2.1194876730e-01
       coeff_l3(49,40)=  1.2589888573e+00
       coeff_l3(49,41)=  1.6132099554e-02
       coeff_l3(49,42)= -5.1159566641e-01
       coeff_l3(49,43)= -4.1849178076e-01
       coeff_l3(49,44)= -9.5542985201e-01
       coeff_l3(49,45)=  2.8495262377e-03
       coeff_l3(49,46)= -1.1402733326e+00
       coeff_l3(49,47)=  4.6768748760e-01
       coeff_l3(49,48)=  1.3846487999e+00
       coeff_l3(49,49)= -1.3377518952e-01
       coeff_l3(49,50)=  2.2610117495e-01
       coeff_l3(49,51)= -8.4917470813e-02
       coeff_l3(49,52)= -7.3659795523e-01
       coeff_l3(49,53)=  2.7883204818e-01
       coeff_l3(49,54)= -9.8626470566e-01
       coeff_l3(49,55)= -3.9505347610e-01
       coeff_l3(49,56)=  1.3791936636e-01
       coeff_l3(49,57)=  4.1633123159e-01
       coeff_l3(49,58)=  4.3525332212e-01
       coeff_l3(49,59)=  4.5352607965e-01
       coeff_l3(49,60)= -9.3794576824e-02
       coeff_l3(49,61)= -1.6658931971e-01
       coeff_l3(49,62)= -3.3505067229e-01
       coeff_l3(49,63)= -2.0806294680e-01
       coeff_l3(49,64)= -2.4170722067e-01
       coeff_l3(49,65)=  1.6357815266e-01
       coeff_l3(49,66)=  1.2922041416e+00
       coeff_l3(49,67)=  1.2797174454e+00
       coeff_l3(49,68)=  9.7628980875e-01
       coeff_l3(49,69)= -1.1330031604e-01
       coeff_l3(49,70)= -6.0035103559e-01
       coeff_l3(49,71)=  4.9257808924e-01
       coeff_l3(49,72)= -1.3350479603e+00
       coeff_l3(49,73)=  1.0602114201e+00
       coeff_l3(49,74)=  2.1145830154e+00
       coeff_l3(49,75)= -4.0693688393e-01
       coeff_l3(49,76)= -6.8259246647e-02
       coeff_l3(49,77)=  7.2120112181e-01
       coeff_l3(49,78)=  1.1867399514e-01
       coeff_l3(49,79)=  5.4832130671e-01
       coeff_l3(49,80)= -2.2502644360e-01
       coeff_l3(49,81)=  9.7492820024e-01
       coeff_l3(49,82)=  4.1605925560e-01
       coeff_l3(49,83)=  2.9644623399e-01
       coeff_l3(49,84)=  5.3753954172e-01
       coeff_l3(49,85)= -9.3852198124e-01
       coeff_l3(49,86)= -8.4604454041e-01
       coeff_l3(49,87)=  1.2725239992e+00
       coeff_l3(49,88)=  2.3226754665e+00
       coeff_l3(49,89)= -3.3513408899e-01
       coeff_l3(49,90)=  3.1388148665e-01
       coeff_l3(49,91)= -9.1631311178e-01
       coeff_l3(49,92)=  8.5940784216e-01
       coeff_l3(49,93)=  6.4854067564e-01
       coeff_l3(49,94)=  8.0145305395e-01
       coeff_l3(49,95)= -1.1512268782e+00
       coeff_l3(49,96)= -7.2976756096e-01
       coeff_l3(49,97)=  1.3526781797e+00
       coeff_l3(49,98)=  1.8092621863e-01
       coeff_l3(49,99)= -1.0652357340e+00
       coeff_l3(49,100)=  1.5831166506e-01
       coeff_l3(49,101)=  8.1877386570e-01
       coeff_l3(49,102)= -1.0862174034e+00
       coeff_l3(49,103)=  4.8891133070e-01
       coeff_l3(49,104)= -2.1043317020e-01
       coeff_l3(49,105)=  2.4321436882e-01
       coeff_l3(50,1)= -1.0779294968e+00
       coeff_l3(50,2)= -1.4055222273e+00
       coeff_l3(50,3)= -6.7955249548e-01
       coeff_l3(50,4)= -1.3456477225e-01
       coeff_l3(50,5)= -5.3086227179e-01
       coeff_l3(50,6)= -4.5437592268e-01
       coeff_l3(50,7)= -5.3710103035e-01
       coeff_l3(50,8)= -7.0480209589e-01
       coeff_l3(50,9)= -2.4450726509e+00
       coeff_l3(50,10)= -1.5611667633e+00
       coeff_l3(50,11)= -2.5163326263e+00
       coeff_l3(50,12)= -1.7754619122e+00
       coeff_l3(50,13)= -3.6710600853e+00
       coeff_l3(50,14)= -2.2318685055e+00
       coeff_l3(50,15)= -1.7088803649e-01
       coeff_l3(50,16)=  9.8793482780e-01
       coeff_l3(50,17)=  6.3770294189e-01
       coeff_l3(50,18)=  9.1109263897e-01
       coeff_l3(50,19)= -4.5560306311e-01
       coeff_l3(50,20)= -3.3039116859e-01
       coeff_l3(50,21)= -6.7138105631e-02
       coeff_l3(50,22)= -5.2251029015e-01
       coeff_l3(50,23)= -4.8594480753e-01
       coeff_l3(50,24)=  1.7311282456e-01
       coeff_l3(50,25)=  5.6334912777e-01
       coeff_l3(50,26)= -5.4330330342e-02
       coeff_l3(50,27)= -1.4396204054e-01
       coeff_l3(50,28)= -6.9927535951e-02
       coeff_l3(50,29)=  4.3757706881e-01
       coeff_l3(50,30)= -2.4528135359e-01
       coeff_l3(50,31)= -2.4424493313e-01
       coeff_l3(50,32)=  2.4372868240e-01
       coeff_l3(50,33)= -6.9778943062e-01
       coeff_l3(50,34)= -4.3867146969e-01
       coeff_l3(50,35)=  4.5143035054e-01
       coeff_l3(50,36)= -3.6725801229e-01
       coeff_l3(50,37)= -1.0611521453e-01
       coeff_l3(50,38)= -5.0668436289e-01
       coeff_l3(50,39)= -5.3177261353e-01
       coeff_l3(50,40)=  1.3995407522e-01
       coeff_l3(50,41)=  2.9353606701e-01
       coeff_l3(50,42)=  2.7493837476e-01
       coeff_l3(50,43)=  7.5542801619e-01
       coeff_l3(50,44)= -2.4607624114e-01
       coeff_l3(50,45)=  4.9417206645e-01
       coeff_l3(50,46)=  3.8682255149e-01
       coeff_l3(50,47)= -2.4724376202e-01
       coeff_l3(50,48)= -4.6799418330e-01
       coeff_l3(50,49)= -8.9294844866e-01
       coeff_l3(50,50)=  1.7599602044e-01
       coeff_l3(50,51)=  3.1505894661e-01
       coeff_l3(50,52)= -3.0524161458e-01
       coeff_l3(50,53)=  7.9575568438e-01
       coeff_l3(50,54)= -1.7904399335e-01
       coeff_l3(50,55)= -1.5982286632e-01
       coeff_l3(50,56)= -7.4040907621e-01
       coeff_l3(50,57)= -3.5058337450e-01
       coeff_l3(50,58)= -1.3233532906e+00
       coeff_l3(50,59)= -6.7349940538e-01
       coeff_l3(50,60)=  4.8604455590e-01
       coeff_l3(50,61)=  1.6928989440e-02
       coeff_l3(50,62)=  1.6122597456e-01
       coeff_l3(50,63)=  8.7955492735e-01
       coeff_l3(50,64)= -3.1527885795e-01
       coeff_l3(50,65)= -1.0662890226e-01
       coeff_l3(50,66)= -2.0395891666e+00
       coeff_l3(50,67)= -2.8400769830e-01
       coeff_l3(50,68)=  6.2211132050e-01
       coeff_l3(50,69)= -6.1131519079e-01
       coeff_l3(50,70)=  6.3300520182e-02
       coeff_l3(50,71)=  1.5788444877e-01
       coeff_l3(50,72)=  5.3067511320e-01
       coeff_l3(50,73)=  8.3513671160e-01
       coeff_l3(50,74)= -1.0284988880e+00
       coeff_l3(50,75)= -9.5773172379e-01
       coeff_l3(50,76)=  2.0223575830e-01
       coeff_l3(50,77)=  1.7245925963e-01
       coeff_l3(50,78)=  4.1475257277e-01
       coeff_l3(50,79)= -3.4850499034e-01
       coeff_l3(50,80)= -1.7989360094e+00
       coeff_l3(50,81)= -8.0601912737e-01
       coeff_l3(50,82)=  2.7713575959e-01
       coeff_l3(50,83)= -9.8702482879e-02
       coeff_l3(50,84)= -6.1214572191e-01
       coeff_l3(50,85)= -6.6575556993e-02
       coeff_l3(50,86)=  1.7425127327e-01
       coeff_l3(50,87)= -7.0004016161e-01
       coeff_l3(50,88)= -4.1523197293e-01
       coeff_l3(50,89)= -8.3344084024e-01
       coeff_l3(50,90)=  9.6330672503e-02
       coeff_l3(50,91)=  8.3828550577e-01
       coeff_l3(50,92)=  1.5594272614e+00
       coeff_l3(50,93)=  1.5258674622e+00
       coeff_l3(50,94)=  7.2415673733e-01
       coeff_l3(50,95)=  1.5069346428e+00
       coeff_l3(50,96)= -1.7331242561e+00
       coeff_l3(50,97)=  1.7992441654e+00
       coeff_l3(50,98)= -1.7780649662e+00
       coeff_l3(50,99)=  1.3421580791e+00
       coeff_l3(50,100)= -7.6707613468e-01
       coeff_l3(50,101)=  3.3637437224e-01
       coeff_l3(50,102)=  1.9942797422e+00
       coeff_l3(50,103)=  7.6896649599e-01
       coeff_l3(50,104)= -1.6106613874e+00
       coeff_l3(50,105)= -2.3002107143e+00
       coeff_l3(51,1)= -2.6328438520e-01
       coeff_l3(51,2)= -8.0384862423e-01
       coeff_l3(51,3)= -2.1026535034e+00
       coeff_l3(51,4)= -1.5625422001e+00
       coeff_l3(51,5)= -2.1541571617e+00
       coeff_l3(51,6)= -3.4741491079e-01
       coeff_l3(51,7)= -2.6583633423e+00
       coeff_l3(51,8)= -3.5714764595e+00
       coeff_l3(51,9)= -1.1851155758e+00
       coeff_l3(51,10)= -6.7106747627e-01
       coeff_l3(51,11)= -1.8409662247e+00
       coeff_l3(51,12)= -1.9200277328e+00
       coeff_l3(51,13)= -1.8904545307e+00
       coeff_l3(51,14)= -8.9325410128e-01
       coeff_l3(51,15)= -3.1989020109e-01
       coeff_l3(51,16)= -1.0157456398e+00
       coeff_l3(51,17)= -1.3488301039e+00
       coeff_l3(51,18)= -1.7246396840e-01
       coeff_l3(51,19)= -8.1440532207e-01
       coeff_l3(51,20)= -6.3104850054e-01
       coeff_l3(51,21)= -1.2952999212e-02
       coeff_l3(51,22)=  1.0442558527e+00
       coeff_l3(51,23)=  5.9119153768e-02
       coeff_l3(51,24)= -1.1258858442e+00
       coeff_l3(51,25)=  1.6527594328e+00
       coeff_l3(51,26)= -2.4475617707e-01
       coeff_l3(51,27)=  8.6634659767e-01
       coeff_l3(51,28)=  1.8483689055e-02
       coeff_l3(51,29)= -1.4134269953e+00
       coeff_l3(51,30)= -3.7334218621e-01
       coeff_l3(51,31)=  2.4347886443e-01
       coeff_l3(51,32)=  2.2069998085e-01
       coeff_l3(51,33)= -9.8924197257e-02
       coeff_l3(51,34)=  1.3123930991e-01
       coeff_l3(51,35)=  1.9079035521e-01
       coeff_l3(51,36)=  3.5828679800e-01
       coeff_l3(51,37)=  1.4040464163e+00
       coeff_l3(51,38)=  8.0036091805e-01
       coeff_l3(51,39)=  1.3639352322e+00
       coeff_l3(51,40)=  1.4195960760e-01
       coeff_l3(51,41)=  8.0164265633e-01
       coeff_l3(51,42)= -2.9940349981e-02
       coeff_l3(51,43)= -4.2600566149e-01
       coeff_l3(51,44)=  1.5273778141e-01
       coeff_l3(51,45)= -8.3991283178e-01
       coeff_l3(51,46)= -4.8998606205e-01
       coeff_l3(51,47)= -9.6454121172e-02
       coeff_l3(51,48)=  1.1963418871e-01
       coeff_l3(51,49)=  3.0095937848e-01
       coeff_l3(51,50)= -8.3220928907e-01
       coeff_l3(51,51)= -2.3787026107e-01
       coeff_l3(51,52)= -7.0015567541e-01
       coeff_l3(51,53)= -2.5096949935e-01
       coeff_l3(51,54)=  7.6236659288e-01
       coeff_l3(51,55)=  5.0647366047e-01
       coeff_l3(51,56)= -5.6355452538e-01
       coeff_l3(51,57)=  1.5004070103e-01
       coeff_l3(51,58)= -1.1836894602e-01
       coeff_l3(51,59)= -5.5213785172e-01
       coeff_l3(51,60)=  8.3671349287e-01
       coeff_l3(51,61)= -2.8166618943e-01
       coeff_l3(51,62)=  2.6442557573e-01
       coeff_l3(51,63)= -6.2054514885e-02
       coeff_l3(51,64)= -3.1935018301e-01
       coeff_l3(51,65)=  1.1221828461e+00
       coeff_l3(51,66)= -9.5909172297e-01
       coeff_l3(51,67)= -3.3224815130e-01
       coeff_l3(51,68)=  2.5929074287e+00
       coeff_l3(51,69)=  1.0245441198e+00
       coeff_l3(51,70)=  6.0045725107e-01
       coeff_l3(51,71)= -3.8139939308e-01
       coeff_l3(51,72)=  6.0019064695e-02
       coeff_l3(51,73)=  3.4995806217e-01
       coeff_l3(51,74)=  1.9411669672e-01
       coeff_l3(51,75)=  3.8209369779e-01
       coeff_l3(51,76)=  1.2106329948e-01
       coeff_l3(51,77)= -1.1325808764e+00
       coeff_l3(51,78)= -4.8271620274e-01
       coeff_l3(51,79)= -1.0432220995e-01
       coeff_l3(51,80)=  3.6614626646e-01
       coeff_l3(51,81)=  1.1618653536e+00
       coeff_l3(51,82)= -1.6579180956e+00
       coeff_l3(51,83)=  1.0757580996e+00
       coeff_l3(51,84)= -3.6901646852e-01
       coeff_l3(51,85)=  1.4701931477e+00
       coeff_l3(51,86)= -1.8365513086e+00
       coeff_l3(51,87)= -1.0889183283e+00
       coeff_l3(51,88)=  4.4388575479e-03
       coeff_l3(51,89)= -1.0766521096e-02
       coeff_l3(51,90)= -5.2842700481e-01
       coeff_l3(51,91)= -1.0535315424e-01
       coeff_l3(51,92)= -7.6324295998e-01
       coeff_l3(51,93)=  6.4723783731e-01
       coeff_l3(51,94)=  2.4546438456e-01
       coeff_l3(51,95)= -1.3551678509e-02
       coeff_l3(51,96)=  9.3506240845e-01
       coeff_l3(51,97)=  1.4556918144e+00
       coeff_l3(51,98)=  8.5663557053e-01
       coeff_l3(51,99)= -1.0069510937e+00
       coeff_l3(51,100)=  2.9342621565e-01
       coeff_l3(51,101)= -2.3197355866e-01
       coeff_l3(51,102)= -5.1131957769e-01
       coeff_l3(51,103)= -2.1004621685e-01
       coeff_l3(51,104)=  8.2599353790e-01
       coeff_l3(51,105)= -8.9570534229e-01
       coeff_l3(52,1)= -4.3762823939e-01
       coeff_l3(52,2)= -3.3186814785e+00
       coeff_l3(52,3)= -2.6113049984e+00
       coeff_l3(52,4)= -2.5541846752e+00
       coeff_l3(52,5)= -1.9930022955e+00
       coeff_l3(52,6)= -2.1694886684e+00
       coeff_l3(52,7)= -3.3917548656e+00
       coeff_l3(52,8)= -2.0069785118e+00
       coeff_l3(52,9)= -2.4760043621e+00
       coeff_l3(52,10)= -2.5773439407e+00
       coeff_l3(52,11)= -3.4304890037e-01
       coeff_l3(52,12)= -1.6074049473e+00
       coeff_l3(52,13)= -2.4984121323e+00
       coeff_l3(52,14)= -2.8246998787e+00
       coeff_l3(52,15)=  9.5284871757e-02
       coeff_l3(52,16)= -8.6550664902e-01
       coeff_l3(52,17)=  4.2821556330e-01
       coeff_l3(52,18)=  1.3724653721e+00
       coeff_l3(52,19)= -3.3049264550e-01
       coeff_l3(52,20)= -3.5841715336e-01
       coeff_l3(52,21)=  3.8157129288e-01
       coeff_l3(52,22)=  6.0450768471e-01
       coeff_l3(52,23)=  5.8254426718e-01
       coeff_l3(52,24)=  3.9383044839e-01
       coeff_l3(52,25)=  1.7137682438e+00
       coeff_l3(52,26)= -1.8038569689e+00
       coeff_l3(52,27)= -3.0260052532e-02
       coeff_l3(52,28)= -1.2587063313e+00
       coeff_l3(52,29)= -5.8604651690e-01
       coeff_l3(52,30)=  6.5326946974e-01
       coeff_l3(52,31)=  1.0506827831e+00
       coeff_l3(52,32)=  7.9962080717e-01
       coeff_l3(52,33)= -2.8974658251e-01
       coeff_l3(52,34)=  3.9760193229e-01
       coeff_l3(52,35)= -1.7263994217e+00
       coeff_l3(52,36)= -8.5852038860e-01
       coeff_l3(52,37)=  1.1439956427e+00
       coeff_l3(52,38)= -9.9807143211e-02
       coeff_l3(52,39)= -8.1135624647e-01
       coeff_l3(52,40)=  2.4832502007e-02
       coeff_l3(52,41)= -9.4050966203e-02
       coeff_l3(52,42)= -6.3976478577e-01
       coeff_l3(52,43)= -8.2783043385e-01
       coeff_l3(52,44)= -9.8234891891e-01
       coeff_l3(52,45)= -3.3409774303e-02
       coeff_l3(52,46)=  1.1060266495e+00
       coeff_l3(52,47)=  6.6953599453e-01
       coeff_l3(52,48)=  2.7090713978e+00
       coeff_l3(52,49)=  1.7836745977e+00
       coeff_l3(52,50)=  4.1797143221e-01
       coeff_l3(52,51)=  1.3516401052e+00
       coeff_l3(52,52)= -2.5440710783e-01
       coeff_l3(52,53)= -5.1913207769e-01
       coeff_l3(52,54)=  3.0148681998e-01
       coeff_l3(52,55)=  4.8347130418e-01
       coeff_l3(52,56)= -3.2971721143e-02
       coeff_l3(52,57)=  2.6082977653e-01
       coeff_l3(52,58)= -6.5684992075e-01
       coeff_l3(52,59)= -3.0866047367e-02
       coeff_l3(52,60)=  1.1626622640e-02
       coeff_l3(52,61)= -1.6995260119e-01
       coeff_l3(52,62)=  1.2742680311e-01
       coeff_l3(52,63)=  1.1221728325e+00
       coeff_l3(52,64)=  2.9029953480e+00
       coeff_l3(52,65)=  3.2194781303e-01
       coeff_l3(52,66)= -8.8924795389e-02
       coeff_l3(52,67)= -3.3970299363e-01
       coeff_l3(52,68)= -1.0987451077e+00
       coeff_l3(52,69)=  4.6773296595e-01
       coeff_l3(52,70)=  2.8134658933e-01
       coeff_l3(52,71)= -1.2543830872e+00
       coeff_l3(52,72)= -1.0652117729e+00
       coeff_l3(52,73)=  6.7796653509e-01
       coeff_l3(52,74)=  2.1648459435e+00
       coeff_l3(52,75)=  1.7549282312e+00
       coeff_l3(52,76)=  1.3223779202e+00
       coeff_l3(52,77)= -3.1011787057e-01
       coeff_l3(52,78)=  5.2521899343e-02
       coeff_l3(52,79)=  6.3927859068e-01
       coeff_l3(52,80)=  1.9593201876e+00
       coeff_l3(52,81)=  1.3975783587e+00
       coeff_l3(52,82)= -9.7382235527e-01
       coeff_l3(52,83)=  1.2101420164e+00
       coeff_l3(52,84)=  1.4576128721e+00
       coeff_l3(52,85)=  9.3544238806e-01
       coeff_l3(52,86)=  7.9944562912e-01
       coeff_l3(52,87)=  9.8541438580e-01
       coeff_l3(52,88)=  8.0784952641e-01
       coeff_l3(52,89)= -2.6664322615e-01
       coeff_l3(52,90)=  1.4611333609e+00
       coeff_l3(52,91)=  2.1107080579e-01
       coeff_l3(52,92)= -1.3560892344e+00
       coeff_l3(52,93)=  1.0638753884e-02
       coeff_l3(52,94)= -1.9551649690e-01
       coeff_l3(52,95)= -7.0181649923e-01
       coeff_l3(52,96)=  2.0057308674e+00
       coeff_l3(52,97)=  1.1792789213e-02
       coeff_l3(52,98)=  1.0726590157e+00
       coeff_l3(52,99)= -4.1481348872e-01
       coeff_l3(52,100)= -1.3566826582e+00
       coeff_l3(52,101)=  6.6273438931e-01
       coeff_l3(52,102)= -8.5903441906e-01
       coeff_l3(52,103)= -1.5533059835e+00
       coeff_l3(52,104)=  7.6813422143e-02
       coeff_l3(52,105)=  1.4423040152e+00
       coeff_l3(53,1)= -1.1580672264e+00
       coeff_l3(53,2)= -2.3793370724e+00
       coeff_l3(53,3)= -2.5723783970e+00
       coeff_l3(53,4)= -1.6308306456e+00
       coeff_l3(53,5)= -1.2224522829e+00
       coeff_l3(53,6)= -2.1429636478e+00
       coeff_l3(53,7)=  3.0790701509e-01
       coeff_l3(53,8)= -1.0077724457e+00
       coeff_l3(53,9)=  9.8622930050e-01
       coeff_l3(53,10)=  3.3711183071e-01
       coeff_l3(53,11)=  2.2385330200e+00
       coeff_l3(53,12)=  1.5381081104e+00
       coeff_l3(53,13)=  9.8172432184e-01
       coeff_l3(53,14)=  1.9209337234e+00
       coeff_l3(53,15)= -5.3805142641e-02
       coeff_l3(53,16)=  9.5015680790e-01
       coeff_l3(53,17)= -1.1021067947e-01
       coeff_l3(53,18)=  1.2462363243e+00
       coeff_l3(53,19)=  1.5873529911e+00
       coeff_l3(53,20)=  7.7552700043e-01
       coeff_l3(53,21)= -6.5989196301e-01
       coeff_l3(53,22)= -1.4473451376e+00
       coeff_l3(53,23)= -1.2745968103e+00
       coeff_l3(53,24)=  1.9354255497e-01
       coeff_l3(53,25)=  1.6966501474e+00
       coeff_l3(53,26)= -4.0753322840e-01
       coeff_l3(53,27)= -9.3889403343e-01
       coeff_l3(53,28)= -1.1258772612e+00
       coeff_l3(53,29)=  9.5244652033e-01
       coeff_l3(53,30)=  5.2973949909e-01
       coeff_l3(53,31)= -7.1345645189e-01
       coeff_l3(53,32)= -1.6474872828e-01
       coeff_l3(53,33)= -9.3863034248e-01
       coeff_l3(53,34)=  4.6280318499e-01
       coeff_l3(53,35)=  2.6290729642e-01
       coeff_l3(53,36)=  3.4349143505e-01
       coeff_l3(53,37)= -7.8306770325e-01
       coeff_l3(53,38)= -1.4802957773e+00
       coeff_l3(53,39)=  1.0538543463e+00
       coeff_l3(53,40)= -3.6817744374e-02
       coeff_l3(53,41)= -1.3878279924e+00
       coeff_l3(53,42)= -7.4080184102e-02
       coeff_l3(53,43)=  1.3359187841e+00
       coeff_l3(53,44)= -3.1012475491e-01
       coeff_l3(53,45)=  1.9481061026e-02
       coeff_l3(53,46)=  6.7232137918e-01
       coeff_l3(53,47)= -2.2843524814e-01
       coeff_l3(53,48)= -8.3066529036e-01
       coeff_l3(53,49)= -8.5671800375e-01
       coeff_l3(53,50)=  1.0609222651e+00
       coeff_l3(53,51)=  1.5111316442e+00
       coeff_l3(53,52)= -1.0243701935e+00
       coeff_l3(53,53)=  7.5529074669e-01
       coeff_l3(53,54)=  6.1458534002e-01
       coeff_l3(53,55)= -7.5402480364e-01
       coeff_l3(53,56)=  6.5912604332e-01
       coeff_l3(53,57)= -7.3547708988e-01
       coeff_l3(53,58)= -5.0199873745e-02
       coeff_l3(53,59)= -1.9380373955e+00
       coeff_l3(53,60)=  2.7251718044e+00
       coeff_l3(53,61)= -2.2315055132e-01
       coeff_l3(53,62)= -1.2407605648e+00
       coeff_l3(53,63)=  2.1326501667e-01
       coeff_l3(53,64)= -1.0737364292e+00
       coeff_l3(53,65)= -9.0467923880e-01
       coeff_l3(53,66)= -1.2456156015e+00
       coeff_l3(53,67)= -4.8120579123e-01
       coeff_l3(53,68)= -1.4498311281e+00
       coeff_l3(53,69)=  6.5093106031e-01
       coeff_l3(53,70)=  1.2421584129e+00
       coeff_l3(53,71)=  5.8357769251e-01
       coeff_l3(53,72)= -1.5038838387e+00
       coeff_l3(53,73)=  1.0767019987e+00
       coeff_l3(53,74)= -9.9193744361e-02
       coeff_l3(53,75)= -1.4987068176e+00
       coeff_l3(53,76)=  3.8948392868e-01
       coeff_l3(53,77)= -1.6910711303e-02
       coeff_l3(53,78)=  1.1442598104e+00
       coeff_l3(53,79)=  1.1424332857e-01
       coeff_l3(53,80)= -1.1701264232e-01
       coeff_l3(53,81)=  1.0570135117e+00
       coeff_l3(53,82)= -9.6488338709e-01
       coeff_l3(53,83)= -4.8439460993e-01
       coeff_l3(53,84)=  4.6069806814e-01
       coeff_l3(53,85)= -6.6138029099e-01
       coeff_l3(53,86)= -5.6713825464e-01
       coeff_l3(53,87)= -7.5024169683e-01
       coeff_l3(53,88)= -1.3635100126e+00
       coeff_l3(53,89)= -7.1606093645e-01
       coeff_l3(53,90)=  2.3546057940e-01
       coeff_l3(53,91)= -8.3701658249e-01
       coeff_l3(53,92)=  9.0951830149e-01
       coeff_l3(53,93)=  6.7120856047e-01
       coeff_l3(53,94)=  5.0023543835e-01
       coeff_l3(53,95)=  2.8914599419e+00
       coeff_l3(53,96)= -1.1560163498e+00
       coeff_l3(53,97)=  3.6505393982e+00
       coeff_l3(53,98)= -1.5652767420e+00
       coeff_l3(53,99)=  3.2136198878e-01
       coeff_l3(53,100)=  1.1678224802e-01
       coeff_l3(53,101)=  1.8195177317e+00
       coeff_l3(53,102)=  1.2624397278e+00
       coeff_l3(53,103)= -4.4081926346e-01
       coeff_l3(53,104)= -1.6391404867e+00
       coeff_l3(53,105)= -2.4839084148e+00
       coeff_l3(54,1)= -2.2619362921e-02
       coeff_l3(54,2)= -1.4981475472e-01
       coeff_l3(54,3)= -2.3429049551e-01
       coeff_l3(54,4)= -1.5302573331e-02
       coeff_l3(54,5)=  6.6353368759e-01
       coeff_l3(54,6)= -3.4239056706e-01
       coeff_l3(54,7)= -2.0684754476e-02
       coeff_l3(54,8)= -6.3618689775e-01
       coeff_l3(54,9)= -4.5184966922e-01
       coeff_l3(54,10)=  1.9992010295e-01
       coeff_l3(54,11)=  5.0286489725e-01
       coeff_l3(54,12)=  7.7102261782e-01
       coeff_l3(54,13)=  2.5418689847e-01
       coeff_l3(54,14)= -4.6181075275e-02
       coeff_l3(54,15)= -3.1320497394e-01
       coeff_l3(54,16)= -3.5706830025e-01
       coeff_l3(54,17)= -2.7835208178e-01
       coeff_l3(54,18)= -4.3548876047e-01
       coeff_l3(54,19)= -5.2340632677e-01
       coeff_l3(54,20)= -1.9137258828e-01
       coeff_l3(54,21)=  3.8891106844e-01
       coeff_l3(54,22)=  8.4091436863e-01
       coeff_l3(54,23)=  1.8472281098e-01
       coeff_l3(54,24)= -2.1733601391e-01
       coeff_l3(54,25)=  8.0011111498e-01
       coeff_l3(54,26)= -1.6157782078e-01
       coeff_l3(54,27)=  2.5286936760e-01
       coeff_l3(54,28)= -8.6353890598e-02
       coeff_l3(54,29)= -7.3853337765e-01
       coeff_l3(54,30)=  2.0663802326e-01
       coeff_l3(54,31)=  8.5460829735e-01
       coeff_l3(54,32)=  2.7068096399e-01
       coeff_l3(54,33)=  1.1929073334e+00
       coeff_l3(54,34)=  3.6464321613e-01
       coeff_l3(54,35)= -8.1314754486e-01
       coeff_l3(54,36)= -7.6380455494e-01
       coeff_l3(54,37)=  1.4719655514e+00
       coeff_l3(54,38)=  2.4370357394e-01
       coeff_l3(54,39)=  6.9084233046e-01
       coeff_l3(54,40)=  2.1307690144e+00
       coeff_l3(54,41)=  2.4952332675e-01
       coeff_l3(54,42)=  4.9952250719e-01
       coeff_l3(54,43)=  2.9431933165e-01
       coeff_l3(54,44)= -2.4277405441e-01
       coeff_l3(54,45)= -1.6821201146e-01
       coeff_l3(54,46)= -1.4478379488e+00
       coeff_l3(54,47)=  3.2642731071e-01
       coeff_l3(54,48)=  3.5527798533e-01
       coeff_l3(54,49)=  2.8895375133e-01
       coeff_l3(54,50)=  8.5339736938e-01
       coeff_l3(54,51)= -6.7180293798e-01
       coeff_l3(54,52)= -4.5473921299e-01
       coeff_l3(54,53)=  1.3503137231e-01
       coeff_l3(54,54)= -7.3411263525e-02
       coeff_l3(54,55)=  5.5010265112e-01
       coeff_l3(54,56)=  4.3827596307e-01
       coeff_l3(54,57)= -3.1783989072e-01
       coeff_l3(54,58)=  7.8431779146e-01
       coeff_l3(54,59)=  7.6511710882e-01
       coeff_l3(54,60)= -1.7086403444e-02
       coeff_l3(54,61)=  2.4214731157e-01
       coeff_l3(54,62)= -4.4997319579e-01
       coeff_l3(54,63)= -2.7189028263e-01
       coeff_l3(54,64)=  6.3394114375e-02
       coeff_l3(54,65)=  3.4288010001e-01
       coeff_l3(54,66)=  1.8364691734e-01
       coeff_l3(54,67)=  5.3390467167e-01
       coeff_l3(54,68)= -2.0550911129e-01
       coeff_l3(54,69)=  1.3981133699e-02
       coeff_l3(54,70)= -6.0121196508e-01
       coeff_l3(54,71)=  4.1407451034e-01
       coeff_l3(54,72)= -5.3698247671e-01
       coeff_l3(54,73)= -5.7488370687e-02
       coeff_l3(54,74)=  9.5099067688e-01
       coeff_l3(54,75)= -8.3371520042e-02
       coeff_l3(54,76)= -8.9060026407e-01
       coeff_l3(54,77)=  5.4050105810e-01
       coeff_l3(54,78)=  1.0923454165e-01
       coeff_l3(54,79)=  4.3210682273e-01
       coeff_l3(54,80)=  5.4921168089e-01
       coeff_l3(54,81)=  2.4506844580e-01
       coeff_l3(54,82)= -7.5998245738e-03
       coeff_l3(54,83)=  3.8993549347e-01
       coeff_l3(54,84)=  1.1972051859e+00
       coeff_l3(54,85)= -4.5061442256e-01
       coeff_l3(54,86)= -1.0551851988e+00
       coeff_l3(54,87)=  7.6738446951e-01
       coeff_l3(54,88)=  2.7514544129e-01
       coeff_l3(54,89)= -5.0586271286e-01
       coeff_l3(54,90)=  4.8259414732e-02
       coeff_l3(54,91)= -4.1134852171e-01
       coeff_l3(54,92)= -1.2735697031e+00
       coeff_l3(54,93)= -7.0460522175e-01
       coeff_l3(54,94)= -5.1553940773e-01
       coeff_l3(54,95)= -4.7394120693e-01
       coeff_l3(54,96)=  8.9249336720e-01
       coeff_l3(54,97)= -4.5538106561e-01
       coeff_l3(54,98)=  1.2247511148e+00
       coeff_l3(54,99)= -6.8590068817e-01
       coeff_l3(54,100)=  3.7093058228e-01
       coeff_l3(54,101)=  9.1606140137e-01
       coeff_l3(54,102)= -2.4141626060e-01
       coeff_l3(54,103)= -1.2713705301e+00
       coeff_l3(54,104)=  5.1433753967e-01
       coeff_l3(54,105)=  1.5066851377e+00
       coeff_l3(55,1)=  3.6295530200e-01
       coeff_l3(55,2)= -7.2284972668e-01
       coeff_l3(55,3)= -3.8373604417e-01
       coeff_l3(55,4)= -5.7222608477e-02
       coeff_l3(55,5)=  2.1437031031e-01
       coeff_l3(55,6)= -5.8314824104e-01
       coeff_l3(55,7)=  8.0987799168e-01
       coeff_l3(55,8)=  1.2835559249e-01
       coeff_l3(55,9)= -9.3317127228e-01
       coeff_l3(55,10)= -1.7118999958e+00
       coeff_l3(55,11)= -6.0160696507e-01
       coeff_l3(55,12)= -5.4942607880e-01
       coeff_l3(55,13)= -8.3650976419e-01
       coeff_l3(55,14)= -1.0044634342e+00
       coeff_l3(55,15)= -1.3666003011e-02
       coeff_l3(55,16)= -2.5712957978e-01
       coeff_l3(55,17)=  1.4874504507e-01
       coeff_l3(55,18)= -3.9076709747e-01
       coeff_l3(55,19)=  7.1122753620e-01
       coeff_l3(55,20)= -1.1415684223e+00
       coeff_l3(55,21)= -5.0843352079e-01
       coeff_l3(55,22)=  7.6870816946e-01
       coeff_l3(55,23)= -1.9550654292e-01
       coeff_l3(55,24)= -4.9696948379e-02
       coeff_l3(55,25)=  9.7542715073e-01
       coeff_l3(55,26)= -1.2918089628e+00
       coeff_l3(55,27)= -8.6231553555e-01
       coeff_l3(55,28)=  5.0273233652e-01
       coeff_l3(55,29)=  4.0591430664e-01
       coeff_l3(55,30)=  5.7822841406e-01
       coeff_l3(55,31)= -8.6271643639e-02
       coeff_l3(55,32)=  2.0309926569e-01
       coeff_l3(55,33)= -2.6852095127e-01
       coeff_l3(55,34)=  3.5314612091e-02
       coeff_l3(55,35)= -7.9088449478e-01
       coeff_l3(55,36)= -1.1833609343e+00
       coeff_l3(55,37)=  9.1842420399e-02
       coeff_l3(55,38)= -7.2706498206e-02
       coeff_l3(55,39)= -6.8599797785e-02
       coeff_l3(55,40)= -7.2269020602e-03
       coeff_l3(55,41)= -2.2672036290e-01
       coeff_l3(55,42)=  5.4501181841e-01
       coeff_l3(55,43)= -5.9872204065e-01
       coeff_l3(55,44)= -1.0890791416e+00
       coeff_l3(55,45)=  4.1400596499e-02
       coeff_l3(55,46)=  8.1787854433e-01
       coeff_l3(55,47)=  5.5380004644e-01
       coeff_l3(55,48)=  1.0746399164e+00
       coeff_l3(55,49)= -3.7356057763e-01
       coeff_l3(55,50)= -8.0724799633e-01
       coeff_l3(55,51)=  1.1175642014e+00
       coeff_l3(55,52)=  1.1501559056e-02
       coeff_l3(55,53)= -8.3916813135e-01
       coeff_l3(55,54)= -3.9660209417e-01
       coeff_l3(55,55)= -4.9082726240e-01
       coeff_l3(55,56)= -9.1548725963e-02
       coeff_l3(55,57)=  8.5961066186e-02
       coeff_l3(55,58)=  3.2247029245e-02
       coeff_l3(55,59)=  2.8364794329e-02
       coeff_l3(55,60)= -1.4224752784e-01
       coeff_l3(55,61)=  5.1227146387e-01
       coeff_l3(55,62)= -7.8623771667e-01
       coeff_l3(55,63)=  4.1173171252e-02
       coeff_l3(55,64)=  2.9449316859e-01
       coeff_l3(55,65)= -6.7684274912e-01
       coeff_l3(55,66)=  2.6196849346e-01
       coeff_l3(55,67)=  3.0263689160e-01
       coeff_l3(55,68)=  3.3893683553e-01
       coeff_l3(55,69)=  4.7630468011e-01
       coeff_l3(55,70)=  7.6419347525e-01
       coeff_l3(55,71)= -5.8855098486e-01
       coeff_l3(55,72)= -7.3138362169e-01
       coeff_l3(55,73)=  8.1946784258e-01
       coeff_l3(55,74)=  1.3049871922e+00
       coeff_l3(55,75)= -4.6713921428e-01
       coeff_l3(55,76)=  1.3280160427e+00
       coeff_l3(55,77)= -1.6675896645e+00
       coeff_l3(55,78)= -7.4587142467e-01
       coeff_l3(55,79)= -1.4239588976e+00
       coeff_l3(55,80)=  1.4113346338e+00
       coeff_l3(55,81)= -6.9682113826e-02
       coeff_l3(55,82)=  1.1398279667e-01
       coeff_l3(55,83)=  1.4448156953e-01
       coeff_l3(55,84)= -3.3450357616e-02
       coeff_l3(55,85)=  2.4461552501e-01
       coeff_l3(55,86)=  1.1098014116e+00
       coeff_l3(55,87)=  5.6643117219e-02
       coeff_l3(55,88)=  7.7060654759e-02
       coeff_l3(55,89)= -2.6858711243e-01
       coeff_l3(55,90)=  4.3889376521e-01
       coeff_l3(55,91)=  3.1239089370e-01
       coeff_l3(55,92)= -5.0470316410e-01
       coeff_l3(55,93)=  4.8921728134e-01
       coeff_l3(55,94)= -6.8822729588e-01
       coeff_l3(55,95)=  1.5813376009e-01
       coeff_l3(55,96)= -5.5253660679e-01
       coeff_l3(55,97)= -1.1522982121e+00
       coeff_l3(55,98)=  6.2523198128e-01
       coeff_l3(55,99)= -8.1090295315e-01
       coeff_l3(55,100)= -9.3792635202e-01
       coeff_l3(55,101)= -8.9724510908e-01
       coeff_l3(55,102)= -5.7306176424e-01
       coeff_l3(55,103)= -5.8091551065e-01
       coeff_l3(55,104)=  3.8921073079e-01
       coeff_l3(55,105)=  2.4093320966e-01
!bias layer 0
       bias_l0=0.0d0
!bias layer 1
       bias_l1=0.0d0
!bias layer 2
       bias_l2=0.0d0
!bias layer 3
       bias_l3=0.0d0

  endsubroutine assign_coeff_bias

      subroutine ev2gm2m(dist,v2b,g2b,imol,igrad,n)

      implicit none

      integer,intent(in) :: imol, igrad
      integer,intent(in) :: n
      double precision,intent(in)  :: dist(n*(n-1)/2)
      double precision,intent(out) :: v2b(14), g2b(14,3)
      double precision :: v(14), grad(14)
      double precision :: r
      integer :: i,j

      ! dist should be in bohr
      ! it is converted to be r in Ang
      ! v2b is in eV
      ! grad2b is in eV/bohr

      ! V0 is 120.17443 kcal/mol
      v2b = 120.17443d0/23.061d0
      g2b = 0.0d0
      do i=1,n*(n-1)/2
        r=dist(i)*0.529177d0
        call ev2gm2(r,v,grad,imol,igrad)
        v2b = v2b + v/23.061d0
        if (igrad.eq.1) then
          do j=1,14
            g2b(j,i) = grad(j)/(23.061d0/0.529177d0)
          enddo
        endif
      enddo

      end subroutine ev2gm2m

      subroutine ev2gm2(r,v,grad,imol,igrad) 
!**********************************************************************
!
! Compute the diatomic potential of groupd-state triplet O2
!
! References: J. Chem. Phys. 132, 074307 (2010)
!
! Input:  r      interatomic distance in Angstrom
! Output: V      potential in kcal/mol
!         grad   gradient (kcal/mol)/Angstrom
!
!**********************************************************************
      implicit none

      integer,intent(in) :: imol, igrad
      double precision,intent(in)  :: r
      double precision,intent(out) :: v(14), grad(14)
! Parameters of analytical even-tempered Gaussian expansions for the
! ground state potential energy curve of O2 CBS+SR+SO+CV.
! Units: alpha in Angstrom^-2, beta=dimensionless, as_k in milihartree.
      double precision :: alpha,beta,as(0:7)
      double precision :: gam, del
      integer :: i,j,k 
! Dispersion variables
      double precision :: dist(1),disp,dispdr(1)

      v=0.0d0
      grad=0.0d0

! list of considered states in 3A' symmetry
! V( 1) - O2(3Sig_g^-) + O(3P)
! V( 2) - O2(3Sig_g^-) + O(3P)
! V( 3) - O2(1Del_g)   + O(3P)
! V( 4) - O2(1Del_g)   + O(3P)
! V( 5) - O2(1Del_g)   + O(3P)
! V( 6) - O2(1Sig_g^+) + O(3P)
! V( 7) - mixed curve including O2(3Sig_g^-) + O(1D)
! V( 8) - mixed curve including O2(3Sig_g^-) + O(1D)
! V( 9) - O2(1Sig_u^-) + O(3P)
! V(10) - O2(1Sig_u^-) + O(3P)
! V(11) - O2(3Del_u)   + O(3P)
! V(12) - O2(3Del_u)   + O(3P)
! V(13) - O2(3Del_u)   + O(3P)
! V(14) - O2(3Sig_u^+) + O(3P)


! Calculate dispersion correction for all states
        dist(1)=r
        call d3disp(dist,disp,dispdr,1,imol)

      do i=1,14
      if ((i.eq.1).or.(i.eq.2)) then
      ! Original parameters
      !      alpha = 0.785d0
      !      beta = 1.307d0
      !      as(0) = -2388.5641690d0
      !      as(1) = 18086.977116d0
      !      as(2) = -71760.197585d0
      !      as(3) = 154738.09175d0
      !      as(4) = -215074.85646d0
      !      as(5) = 214799.54567d0
      !      as(6) = -148395.42850d0
      !      as(7) = 73310.781453d0

      ! O2 X 3Sig_g^-
      ! Modified parameters for D3(BJ)
       alpha = 9.439784362354936d-1
       beta =  1.262242998506810d0
       as(0) = -1.488979427684798d3
       as(1) =  1.881435846488955d4
       as(2) = -1.053475425838226d5
       as(3) =  2.755135591229064d5
       as(4) = -4.277588997761775d5
       as(5) =  4.404104009614092d5
       as(6) = -2.946204062950765d5
       as(7) =  1.176861219078620d5
      else if ((i.eq.3).or.(i.eq.4).or.(i.eq.5)) then
      ! O2 a 1Del_g
       alpha =  0.926743129655067d0
       beta =   1.49310094503326d0
       as(0) = -288.154824977704d0
       as(1) = -1534.79419793776d0
       as(2) =  4738.54413352897d0
       as(3) = -6160.20291806525d0
       as(4) =  16509.1318136742d0
       as(5) = -15472.8382171874d0
       as(6) =  47185.6933654363d0
       as(7) = -56237.4017550829d0
      else if (i.eq.6) then
      ! O2 b 1Sig_g^+
       alpha =  0.859772042751452d0
       beta =   1.52376335851761d0
       as(0) = -177.229021473969d0
       as(1) = -1114.51639743388d0
       as(2) =  2893.23140655769d0
       as(3) = -3085.76375068364d0
       as(4) =  13366.3326753626d0
       as(5) = -15292.8470132772d0
       as(6) =  65232.9925774647d0
       as(7) = -124289.923005226d0
      else if ((i.eq.7).or.(i.eq.8)) then
       ! mixed energy curve
       alpha =  2.19185728150371d0
       beta =   1.05793185102898d0
       as(0) = -2663791.40882274d0
       as(1) =  4890643.13480523d0
       as(2) =  18211797.6008347d0
       as(3) = -69478406.8831105d0
       as(4) =  88925293.4564620d0
       as(5) = -48996476.3149754d0
       as(6) =  7263885.80760647d0
       as(7) =  1859624.51906546d0
       gam = 15.5242851149696d0
       del = 39.7454754114259d0
      else if ((i.eq.9).or.(i.eq.10)) then
      ! O2 c 1Sig_u^-
       alpha =  0.950046799307d0
       beta =   1.642688451339d0
       as(0) = -408.242294143843d0
       as(1) =  506.602117753963d0
       as(2) =  3224.66123488842d0
       as(3) = -2481.06848002454d0
       as(4) =  39820.2268713929d0
       as(5) = -274901.008455779d0
       as(6) =  4805910.07331395d0
       as(7) = -208131033.883674d0
      else if ((i.eq.11).or.(i.eq.12).or.(i.eq.13)) then
      ! O2 C 3Del_u
       alpha =  1.11399797348271d0
       beta =   1.63184883563828d0
       as(0) = -511.287998639776d0
       as(1) =  1050.53472811448d0
       as(2) =  4538.27073425035d0
       as(3) = -6586.90846267541d0
       as(4) =  36344.1183383940d0
       as(5) =  9867.76816893001d0
       as(6) = -2502205.95698573d0
       as(7) =  186852078.118477d0
      else if (i.eq.14) then
      ! O2 A 3Sig_u^+
       alpha =  1.24888948675275d0
       beta =   1.61630730207906d0
       as(0) = -753.149761854632d0
       as(1) =  2601.16339452451d0
       as(2) =  1889.44303142137d0
       as(3) = -692.325517266325d0
       as(4) =  46293.6020025018d0
       as(5) = -50650.1633376840d0
       as(6) = -16795533.6522413d0
       as(7) =  2063993155.90011d0
      endif

      do k=0,7
       v(i)= v(i) + as(k)*dexp(-alpha*beta**k*r**2)
      enddo
      if ((i.eq.7).or.(i.eq.8)) then
       v(i)= v(i) + gam*dexp(-del*(r-1.7d0)**2)
      endif
      if ((i.eq.1).or.(i.eq.2)) then
! From milihartree to kcal/mol
       v(i)=v(i)*627.509523475149d-3
      endif

! Add D3 dispersion correction
        v(i)=v(i)+disp

! Compute the gradient if needed
      if (igrad.eq.1) then
       do k=0,7
        grad(i)=grad(i)-2.d0*as(k)*alpha*beta**k*r*dexp(-alpha*beta**k*r**2)
       enddo
       if ((i.eq.7).or.(i.eq.8)) then
        grad(i)=grad(i)-2.0d0*del*gam*(r-1.7d0)*dexp(-del*(r-1.7d0)**2)
       endif
       if ((i.eq.1).or.(i.eq.2)) then
! Convert from milihartree/A to (kcal/mol)/A
        grad(i)=grad(i)*627.509523475149d-3
       endif
        grad(i)= grad(i) + dispdr(1)
      endif

      enddo

      return
      end subroutine ev2gm2

      subroutine d3disp(dist,disp,dispdr,igrad,imol)
!**********************************************************************
! Dispersion correction based on Grimme's D3(BJ) calculation for
! diatomic pairs
!
! Several subroutines of DFTD3 V3.1 Rev 1 by Grimme were merged into 
! subroutine edisp and they have been heavily modified to calculate only
! that dispersion energy correction that is needed.
!
! S. Grimme, J. Antony, S. Ehrlich and H. Krieg
! J. Chem. Phys, 132 (2010), 154104
! and 
! S. Grimme, S. Ehrlich and L. Goerigk, J. Comput. Chem, 32 (2011),
! 1456-1465
!
! The C6 values are fixed.
!
!**********************************************************************

      double precision cn(2),s6,s8,rs6,rs8
      double precision dist(1), e6(1), e8(1), disp, dispdr(1), c6(2)
      double precision e6dr(1),e8dr(1)
      integer iz(2), mxc(94), i, j, igrad
      double precision c6ab(94,94,5,5,3)
      double precision r2r4(94)
      double precision autoang,autokcal
      integer imol

      autoang =0.52917726d0
      autokcal=627.509541d0

! Generalized parameters for BJ damping from P. Verma, B. Wang, 
! L. E. Fernandez, and D. G. Truhlar, J. Phys. Chem. A 121, 2855 (2017)
      s6= 1.0d0
      s8= 2.0d0
      rs6= 0.5299d0
      rs8= 2.20d0

      do i=1,1
      dist(i)=dist(i)/autoang
      enddo

      if (imol.eq.4) then
! iz for O2 system
      iz(1)=8
      iz(2)=8

! C6 for O2 system
      c6(1)=12.8d0
      c6(2)=12.8d0
      else
! currently imol = 4 is used
      stop
      endif

! Calculate dispersion correction
      call edisp(94,5,2,dist,iz,mxc, &
           rs6,rs8,e6,e8,e6dr,e8dr,c6,0)

      disp = 0.0d0

      do i=1,1
      disp =disp + (-s6*e6(i)-s8*e8(i))*autokcal
      enddo

      if (igrad .eq. 1) then
      call edisp(94,5,2,dist,iz,mxc, &
           rs6,rs8,e6,e8,e6dr,e8dr,c6,1)

      dispdr(:) = 0.0d0

      do i=1,1
      dispdr(i) =dispdr(i) + (-s6*e6dr(i)-s8*e8dr(i))*autokcal/autoang
      enddo
      endif

      do i=1,1
      dist(i)=dist(i)*autoang
      enddo

      end subroutine d3disp

!**********************************************************************
! compute energy
!**********************************************************************
      subroutine edisp(max_elem,maxc,n,dist,iz,mxc, &
                 rs6,rs8,e6,e8,e6dr,e8dr,c6a,igrad)

      integer n,iz(2),max_elem,maxc,mxc(max_elem) 
      double precision dist(1),r2r4(max_elem),r0ab(max_elem,max_elem)
      double precision rs6,rs8,rcov(max_elem)
      double precision c6ab(max_elem,max_elem,maxc,maxc,3)
      double precision e6(1), e8(1), c6a(2), e6dr(1), e8dr(1)
       
      integer iat,jat,igrad
      double precision r,tmp,c6,c8,a1,a2
      double precision damp6,damp8
      double precision cn(n)
      double precision r2ab(n*n),cc6ab(n*n),dmp(n*n)
      integer step

      e6(:) =0.0d0
      e8(:) =0.0d0

      e6dr(:) =0.0d0
      e8dr(:) =0.0d0

      a1=rs6
      a2=rs8 

!  r2r4 =sqrt(0.5*r2r4(i)*dfloat(i)**0.5 ) with i=elementnumber
!  the large number of digits is just to keep the results consistent
!  with older versions. They should not imply any higher accuracy than
!  the old values
      r2r4(1:94)=(/ &
      2.00734898d0,  1.56637132d0,  5.01986934d0,  3.85379032d0, &
      3.64446594d0,  3.10492822d0,  2.71175247d0,  2.59361680d0, &
      2.38825250d0,  2.21522516d0,  6.58585536d0,  5.46295967d0, &
      5.65216669d0,  4.88284902d0,  4.29727576d0,  4.04108902d0, &
      3.72932356d0,  3.44677275d0,  7.97762753d0,  7.07623947d0, &
      6.60844053d0,  6.28791364d0,  6.07728703d0,  5.54643096d0, &
      5.80491167d0,  5.58415602d0,  5.41374528d0,  5.28497229d0, &
      5.22592821d0,  5.09817141d0,  6.12149689d0,  5.54083734d0, &
      5.06696878d0,  4.87005108d0,  4.59089647d0,  4.31176304d0, &
      9.55461698d0,  8.67396077d0,  7.97210197d0,  7.43439917d0, &
      6.58711862d0,  6.19536215d0,  6.01517290d0,  5.81623410d0, &
      5.65710424d0,  5.52640661d0,  5.44263305d0,  5.58285373d0, &
      7.02081898d0,  6.46815523d0,  5.98089120d0,  5.81686657d0, &
      5.53321815d0,  5.25477007d0, 11.02204549d0,  0.15679528d0, &
      9.35167836d0,  9.06926079d0,  8.97241155d0,  8.90092807d0, &
      8.85984840d0,  8.81736827d0,  8.79317710d0,  7.89969626d0, &
      8.80588454d0,  8.42439218d0,  8.54289262d0,  8.47583370d0, &
      8.45090888d0,  8.47339339d0,  7.83525634d0,  8.20702843d0, &
      7.70559063d0,  7.32755997d0,  7.03887381d0,  6.68978720d0, &
      6.05450052d0,  5.88752022d0,  5.70661499d0,  5.78450695d0, &
      7.79780729d0,  7.26443867d0,  6.78151984d0,  6.67883169d0, &
      6.39024318d0,  6.09527958d0, 11.79156076d0, 11.10997644d0, &
      9.51377795d0,  8.67197068d0,  8.77140725d0,  8.65402716d0, &
      8.53923501d0,  8.85024712d0 /)

! these new data are scaled with k2=4./3.  and converted a_0 via
! autoang=0.52917726d0
      rcov(1:94)=(/ &
       0.80628308d0, 1.15903197d0, 3.02356173d0, 2.36845659d0, &
       1.94011865d0, 1.88972601d0, 1.78894056d0, 1.58736983d0, &
       1.61256616d0, 1.68815527d0, 3.52748848d0, 3.14954334d0, &
       2.84718717d0, 2.62041997d0, 2.77159820d0, 2.57002732d0, &
       2.49443835d0, 2.41884923d0, 4.43455700d0, 3.88023730d0, &
       3.35111422d0, 3.07395437d0, 3.04875805d0, 2.77159820d0, &
       2.69600923d0, 2.62041997d0, 2.51963467d0, 2.49443835d0, &
       2.54483100d0, 2.74640188d0, 2.82199085d0, 2.74640188d0, &
       2.89757982d0, 2.77159820d0, 2.87238349d0, 2.94797246d0, &
       4.76210950d0, 4.20778980d0, 3.70386304d0, 3.50229216d0, &
       3.32591790d0, 3.12434702d0, 2.89757982d0, 2.84718717d0, &
       2.84718717d0, 2.72120556d0, 2.89757982d0, 3.09915070d0, &
       3.22513231d0, 3.17473967d0, 3.17473967d0, 3.09915070d0, &
       3.32591790d0, 3.30072128d0, 5.26603625d0, 4.43455700d0, &
       4.08180818d0, 3.70386304d0, 3.98102289d0, 3.95582657d0, &
       3.93062995d0, 3.90543362d0, 3.80464833d0, 3.82984466d0, &
       3.80464833d0, 3.77945201d0, 3.75425569d0, 3.75425569d0, &
       3.72905937d0, 3.85504098d0, 3.67866672d0, 3.45189952d0, &
       3.30072128d0, 3.09915070d0, 2.97316878d0, 2.92277614d0, &
       2.79679452d0, 2.82199085d0, 2.84718717d0, 3.32591790d0, &
       3.27552496d0, 3.27552496d0, 3.42670319d0, 3.30072128d0, &
       3.47709584d0, 3.57788113d0, 5.06446567d0, 4.56053862d0, &
       4.20778980d0, 3.98102289d0, 3.82984466d0, 3.85504098d0, &
       3.88023730d0, 3.90543362d0 /)

! DFT-D3
      step=0
      do iat=1,n-1
         do jat=iat+1,n
         step=step+1
         r=dist(step)
         c6=c6a(step)
! r2r4 stored in main as sqrt
         c8 =3.0d0*c6*r2r4(iz(iat))*r2r4(iz(jat))

! energy for BJ damping
          tmp=sqrt(c8/c6)
          e6(step)= c6/(r**6+(a1*tmp+a2)**6)
          e8(step)= c8/(r**8+(a1*tmp+a2)**8)
! calculate gradients
         if (igrad .eq. 1) then
! grad for BJ damping
          e6dr(step)=c6*(-6*r**5)/(r**6+(a1*tmp+a2)**6)**2
          e8dr(step)=c8*(-8*r**7)/(r**8+(a1*tmp+a2)**8)**2
         endif
         enddo
      enddo

      end subroutine edisp

