!***********************************************************************
!   System:                     O3
!   Functional form:            permutation restrained diabatization by
!                               deep neural network
!   Common name:                O3_14_3Ap_2023 (14 quintet A' states)
!   Number of derivatives:      first derivatives of adiabatic surfaces
!   Number of bodies:           3
!   Number of surfaces:         14
!   Notes:                      -Diatomic potentials contain dispersion terms
!                               -Compile the subroutine requires LAPACK
!                                subroutine DSYEV
! 
!   Input:                      Cartesian coordinates in bohr
!   Output:                     Adiabatic energies (e) in hartrees,
!                               derivatives of adiabataic energies (g) in
!                               hartrees/bohr, and NACs (h) in inverse bohrs.
!   Reference:
!   F. Badichi Akher, Y. Shu, Z. Varga, S. Bhaumik, D. G. Truhlar, 
!   "Activation function for constraining the behavior of a neural network
!   potential with a lower-level potential", in preparation
!***********************************************************************

      subroutine pes(x,igrad,p,g,d)

      implicit none
      ! number of electronic state
      integer, parameter :: nstates=14
      integer, parameter :: natoms=3
      integer, intent(in) :: igrad
      double precision, intent(in) :: x(natoms,3)
      double precision, intent(out) :: p(nstates), g(nstates,natoms,3)
      double precision, intent(out) :: d(nstates,nstates,natoms,3)

      double precision :: v(nstates), tx(natoms,3), tg(nstates,3,3)
      double precision :: td(nstates,nstates,3,3)
      double precision :: t(nstates,nstates)

      integer :: iatom, idir, j, istate, jstate
      !initialize 
      v=0.d0
      g=0.d0
      d=0.d0

      ! transform from angstrom to bohr
      do iatom=1,natoms
        do idir=1,3
          tx(iatom,idir)=x(iatom,idir)/0.529177211
        enddo
      enddo
      tg=0.d0
      td=0.d0

      call energy_gradient(tx,v,tg,td,t,natoms,nstates)

      do istate=1,nstates
        p(istate)=v(istate)*27.211386
      enddo

      do istate=1,nstates
        do iatom=1,natoms
          do idir=1,3
            g(istate,iatom,idir)=tg(istate,iatom,idir)*51.422067
          enddo
        enddo
      enddo

      do istate=1,nstates
        do jstate=1, nstates
          do iatom=1,natoms
            do idir=1,3
              d(istate,jstate,iatom,idir)=td(istate,jstate,iatom,idir)/0.529177211
            enddo
          enddo
        enddo
      enddo

      end

  subroutine energy_gradient(x,e,g,h,t,n,ns)
    ! n    - number of atoms: 3
    ! ns   - number of states: 14
    ! x    - Cartesian coordinates
    ! e    - adiabatic energies
    ! g    - derivatives of adiabatic energies w.r.t. Cartesians
    ! h    - nonadiabatic coupling vectors w.r.t. Cartesians
    ! t    - transformation matrix
    ! r    - internuclear distances
    ! gr   - derivatives of adiabatic energies w.r.t. internuclear
    ! distances
    ! v2b  - two-body energy contributions
    ! g2b  - derivatives of two-body energy contributions w.r.t.
    ! internuclear distances
    ! drdx - dr/dx matrix for converting derivatives

    implicit none
    integer, intent(in) :: n,ns
    real*8, intent(in) :: x(n,3)
    real*8, intent(inout) :: e(ns)
    real*8, intent(inout) :: g(ns,n,3)
    real*8, intent(inout) :: t(ns,ns)
    real*8, intent(inout) :: h(ns,ns,n,3)

    ! layers and their sizes
    real*8, save :: coeff_l0(3,30), coeff_l1(30,65), coeff_l2(65,95), coeff_l3(95,125), coeff_l4(125,105)
    real*8, save :: bias_l0(30), bias_l1(65), bias_l2(95), bias_l3(125), bias_l4(105)
    real*8 :: r(n*(n-1)/2)
    real*8 :: a
    real*8 :: gr(ns,n*(n-1)/2)
    real*8 :: hr(ns,ns,n*(n-1)/2)
    real*8 :: v2b(ns),g2b(ns,n*(n-1)/2)
    real*8 :: drdx(n*(n-1)/2,3*n)
    integer :: i,j,k,l,m
    logical, save :: first_time_data=.true.

    ! In this version we used the new activation functional 
    ! So DPEM layer becomes:
    ! A(r)*B(r)*dpem_layer
    ! the f,a,b parameters are 2, -2, and 7.5
    ! A(r)=0.5+0.5*tanh(2*(r+2))
    ! B(r)=0.5+0.5*yanh(2*(-x+7.5))
    e=0.d0
    g=0.d0
    h=0.d0

    ! get ddnn coefficients and bias
    if(first_time_data) then
      call assign_coeff_bias(coeff_l0,coeff_l1,coeff_l2,coeff_l3,coeff_l4,bias_l0,bias_l1,bias_l2,bias_l3,bias_l4)
      first_time_data=.false.
    endif
    ! input x is in bohr
    call convert_xyz_to_r(x,r,n)
    ! calculate the two-body terms
    call ev2gm2m(r,v2b,g2b,4,1,n)
    ! forward propagate sfp to energy 
    call backprop(r,n,ns,e,gr,hr,t,coeff_l0,coeff_l1,coeff_l2,coeff_l3,coeff_l4,bias_l0,bias_l1,bias_l2,bias_l3,bias_l4,v2b,g2b)
    ! convert from gradient_r to gradient_xyz
    call grad_r_to_xyz(g,drdx,gr,r,x,n,ns)
    call nac_r_to_xyz(h,drdx,hr,r,x,n,ns)
    ! convert energy from eV to hartree (1/27.2107=0.0367502)
    e(:)=e(:)*0.0367502d0
    ! convert grad and nac from eV/bohr to hartree/bohr (1/27.2107=0.0367502)
    g(:,:,:)=g(:,:,:)*0.0367502d0

  endsubroutine energy_gradient

!===========================
! back propagation
  subroutine backprop(r,n,ns,e,g,h,t,coeff_l0,coeff_l1,coeff_l2,coeff_l3,coeff_l4,bias_l0,bias_l1,bias_l2,bias_l3,bias_l4,v2b,g2b)
    implicit none
    integer, intent(in) :: n, ns
    real*8 :: r(n*(n-1)/2)
    real*8, intent(inout) :: e(ns), g(ns,n*(n-1)/2), h(ns,ns,n*(n-1)/2)
    real*8, intent(inout) :: t(ns,ns)
    real*8, intent(in) :: coeff_l0(3,30), coeff_l1(30,65), coeff_l2(65,95), coeff_l3(95,125), coeff_l4(125,105)
    real*8, intent(in) :: bias_l0(30), bias_l1(65), bias_l2(95), bias_l3(125), bias_l4(105)

    real*8 :: a0(30), a1(65), a2(95), a3(125), a4(105)
    real*8 :: z0(30), z1(65), z2(95), z3(125), z4(105)
    real*8 :: g0(3,30), g1(3,65), g2(3,95), g3(3,125), g4(3,105), g4p(105,3)
    real*8 :: dpem(ns,ns), dvdu(ns,105)
    real*8 :: dudr(ns,ns,3), tmpmat(ns,ns)

    integer :: sum1
    integer :: i,j,k,l

    real*8 :: v2b(ns),g2b(ns,n*(n-1)/2)

    a0=0.0d0
    a1=0.0d0
    a2=0.0d0
    a3=0.0d0
    a4=0.0d0
    z0=0.0d0
    z1=0.0d0
    z2=0.0d0
    z3=0.0d0
    z4=0.0d0

    e=0.0d0
    g0=0.0d0
    g1=0.0d0
    g2=0.0d0
    g3=0.0d0
    g4=0.0d0

    call mproduct(r,coeff_l0,z0,1,3,30)
    z0=z0 !+bias_l0
    call activation_gelu(z0,a0,30)

    call mproduct(a0,coeff_l1,z1,1,30,65)
    z1=z1 !+bias_l1
    call activation_gelu(z1,a1,65)

    call mproduct(a1,coeff_l2,z2,1,65,95)
    z2=z2 !+bias_l2
    call activation_gelu(z2,a2,95)
    
    call mproduct(a2,coeff_l3,z3,1,95,125)
    z3=z3 !+bias_l3
    call activation_gelu(z3,a3,125)

    !diabatic layer, use activation functional 
    call mproduct(a3,coeff_l4,z4,1,125,105)
    z4=z4 !+bias_l4
    call activation_functional_heviside(z4,a4,105,r,n*(n-1)/2)

    !construction many-body diabatic potential energy matrix (mb-dpem)
    do i=1,ns
      dpem(i,i)=a4(i)
    enddo
    do i=1,ns
      do j=i+1,ns
        call sumover(ns,i,sum1)
        k=ns-1+sum1+j-i
        dpem(i,j)=a4(k)
        dpem(j,i)=dpem(i,j)
      enddo
    enddo

    !adding the two-body terms to mb-dpem
    do i=1,ns
      dpem(i,i)=dpem(i,i) + v2b(i)
    enddo

    ! diagonalize dpem
    call diagonalize(ns,dpem,e,t)

    ! cmpute dvdu 
    do i=1,ns
      do j=1,ns
        dvdu(i,j)=t(j,i)*t(j,i)
      enddo
    enddo 
    do i=1,ns
      do j=1,ns
        do k=j+1,ns
          call sumover(ns,j,sum1)
          l=ns-1+sum1+k-j
          dvdu(i,l)=2.0*t(j,i)*t(k,i)
        enddo
      enddo
    enddo 

   ! compute gradients 
   g0=coeff_l0   
   do i=1,3
     call activation_derivative_gelu(z0, a0, g0(i,:), 30)
   enddo 

   call mproduct(g0,coeff_l1,g1,3,30,65)

   do i=1,3
     call activation_derivative_gelu(z1, a1, g1(i,:), 65)
   enddo   

   call mproduct(g1,coeff_l2,g2,3,65,95)
   do i=1,3
     call activation_derivative_gelu(z2, a2, g2(i,:), 95)
   enddo

   call mproduct(g2,coeff_l3,g3,3,95,125)
   do i=1,3
     call activation_derivative_gelu(z3, a3, g3(i,:), 125)
   enddo 

   call mproduct(g3,coeff_l4,g4,3,125,105)
   ! here to add actional functional derivative
   call activation_functional_derivative_terms(g4,z4,105,r,n*(n-1)/2)

   do i=1,3
     do j=1,105
       g4p(j,i)=g4(i,j)
     enddo
   enddo 
 
   do i=1,ns
     do j=1,3
     g4p(i,j) = g4p(i,j) + g2b(i,j)
     enddo
   enddo

   call mproduct(dvdu,g4p,g,ns,105,3)

   ! compute NACs
   ! g4p is the DPEM gradients 
   dudr(:,:,:)=0.d0
   do i=1,ns
     dudr(i,i,:)=g4p(i,:)
   enddo
   do i=1,ns
     do j=i+1,ns
        call sumover(ns,i,sum1)
        k=ns-1+sum1+j-i
        dudr(i,j,:)=g4p(k,:)
        dudr(j,i,:)=dudr(i,j,:)
     enddo
   enddo

   h(:,:,:)=0.d0
   do i=1,3
     tmpmat(:,:)=dudr(:,:,i)
     ! C^T*dU*C
     tmpmat=matmul(transpose(t), matmul(tmpmat,t))
     ! F_ij = (C^T*dU*C)_ij / (V_j-V_i)
     do j=1,ns
       do k=j+1,ns
         if ((e(k)-e(j)) > 1.d-8) then
           h(j,k,i)=tmpmat(j,k)/(e(k)-e(j))
         else
           h(j,k,i)=tmpmat(j,k)/1.d-8
         endif
         h(k,j,i)=-h(j,k,i)
       enddo
     enddo
   enddo

  endsubroutine backprop

!=========================
!activation function, gelu
  subroutine activation_gelu(z, a, n)
    integer i,n
    real*8, intent(in) :: z(n)
    real*8, intent(inout) :: a(n)

    do i=1,n
      a(i)=0.5*z(i)*(1+erf(z(i)/sqrt(2.d0)))
    enddo

  end subroutine activation_gelu

  subroutine activation_derivative_gelu(z, a, b, n)
    integer i,n
    real*8, intent(in) :: z(n), a(n)
    real*8, intent(inout) :: b(n)

    do i=1,n
!     b(i)=b(i)*(0.5*erf(z(i)/sqrt(2.d0))+0.398942*exp(-z(i)**2/2)*z(i)+0.5)
      b(i)=b(i)*(a(i)/z(i) + 0.398942*z(i)*exp(-z(i)**2/2) )
    enddo

  end subroutine activation_derivative_gelu

!=============================
!activation functional heviside
! A(r)*B(r)*dpem_layer
! the f,a,b parameters are 2, -2, and 7.5
! A(r)=0.5+0.5*tanh(2*(r+2))
! B(r)=0.5+0.5*tanh(2*(-x+7.5))
  subroutine activation_functional_heviside(z, a, n, r, nbonds)
    integer, intent(in) :: n, nbonds
    real*8, intent(in) :: z(n)
    real*8, intent(in) :: r(nbonds)
    real*8, intent(inout) :: a(n)
    real*8 :: r_max, Ar, Br
    real*8, parameter :: af_f=2, af_a=-2, af_b=7.5

    integer :: i

    r_max=maxval(r,nbonds)
    Ar=0.5+0.5*tanh(af_f*(r_max-af_a))
    Br=0.5+0.5*tanh(af_f*(-r_max+af_b))
    do i=1,n
      a(i)=Ar*Br*z(i)
    enddo

  end subroutine activation_functional_heviside

!=============================
!add activation functional derivative terms
!call activation_functional_derivative_terms(g4,21,n*(n-1)/2,z4)
  subroutine activation_functional_derivative_terms(g, z, n, r, nbonds)
    integer, intent(in) :: n, nbonds
    real*8, intent(in) :: z(n)
    real*8, intent(in) :: r(nbonds)
    real*8, intent(inout) :: g(nbonds,n)

    real*8 :: r_max
    real*8 :: Ar, Br
    real*8, parameter :: af_f=2, af_a=-2, af_b=7.5
    real*8 :: dfdr(nbonds)

    integer :: i

    r_max=maxval(r,nbonds)
    Ar=0.5+0.5*tanh(af_f*(r_max-af_a))
    Br=0.5+0.5*tanh(af_f*(-r_max+af_b))

    if (r(1).ge.r(2) .and. r(1).ge.r(3)) then 
      dfdr(1)=(0.5+(0.5*sinh(15.0-2*r(1)))/cosh(15.0-2*r(1)))/(cosh(2.0*r(1)+4.0)**2)& 
        &+ (-0.5-(0.5*sinh(2.0*r(1)+4.0))/(cosh(2.0*r(1)+4.0)))/((cosh(15.0-2*r(1)))**2)
    else 
      dfdr(1)=0.d0
    endif

    if (r(2).ge.r(1) .and. r(2).ge.r(3)) then
      dfdr(2)=(0.5+(0.5*sinh(15.0-2*r(2)))/cosh(15.0-2*r(2)))/(cosh(2.0*r(2)+4.0)**2)&
        &+ (-0.5-(0.5*sinh(2.0*r(2)+4.0))/(cosh(2.0*r(2)+4.0)))/((cosh(15.0-2*r(2)))**2)
    else
      dfdr(2)=0.d0
    endif

    if (r(3).ge.r(2) .and. r(3).ge.r(1)) then
      dfdr(3)=(0.5+(0.5*sinh(15.0-2*r(3)))/cosh(15.0-2*r(3)))/(cosh(2.0*r(3)+4.0)**2)&
        &+ (-0.5-(0.5*sinh(2.0*r(3)+4.0))/(cosh(2.0*r(3)+4.0)))/((cosh(15.0-2*r(3)))**2)
    else
      dfdr(3)=0.d0
    endif

    do i=1,nbonds
      g(i,:)=dfdr(i)*z(:)+Ar*Br*g(i,:)
    enddo

  end subroutine activation_functional_derivative_terms

!==============================
! used to compute index in DPEM
  subroutine sumover(a,b,sum1)
    integer, intent(in) :: a, b
    integer, intent(out) :: sum1
    integer :: num

    sum1=1-a
    do num=a, a-b+1, -1
      sum1=sum1+num
    enddo
  end subroutine sumover

!==============================
! matrix diagonalization
! subroutine diagonalize(n,A_ss,B_ss,U_ss)
  subroutine diagonalize(n,A_ss,EV_s,U_ss)
    implicit none
    ! parameters
    integer, intent(in) :: n
    real*8, intent(in) :: A_ss(n,n)
!   real*8, intent(out) :: B_ss(n,n)
    real*8, intent(out) :: U_ss(n,n)
    ! internal variables
!   real*8 :: EV_s(n)
    real*8, intent(out) :: EV_s(n)
    integer :: io,i
    real*8,allocatable :: lapack_work_d(:)
    integer, save :: lapack_lwork_d 
    logical, save :: first_time_diag=.true.

    U_ss=A_ss
    if (first_time_diag) then
      lapack_lwork_d= -1
      allocate( lapack_work_d(2) )
      call dsyev('V','L',n,U_ss,n,EV_s,lapack_work_d,lapack_lwork_d,io)
      lapack_lwork_d=int(lapack_work_d(1))
      deallocate ( lapack_work_d )
      first_time_diag=.false.
    endif
    allocate( lapack_work_d(lapack_lwork_d) )
    call dsyev('V','L',n,U_ss,n,EV_s,lapack_work_d,lapack_lwork_d,io)
    ! U_ss already holds the transformation matrix
    ! now  building the diagonal matrix A_ss
!   B_ss=0.d0
!   do i=1,n
!     B_ss(i,i)=EV_s(i)
!   enddo

    return

  end subroutine diagonalize

!==========================
! convert de/dr to de/dxyz
  subroutine grad_r_to_xyz(g,drdx,gr,r,x,n,ns)
    implicit none
    integer, intent(in) :: n, ns
    real*8, intent(in) :: x(n,3)
    real*8, intent(in) :: r(n*(n-1)/2)
    real*8, intent(in) :: gr(ns,n*(n-1)/2)
    real*8, intent(inout) :: g(ns,n,3)
    real*8, intent(inout) :: drdx(n*(n-1)/2,3*n)

    real*8 ::  gx(ns,3*n)
    integer :: i,j

    drdx=0.0d0

    ! compute drdx
    drdx(1,1)=(x(1,1)-x(2,1))/r(1)
    drdx(1,2)=(x(1,2)-x(2,2))/r(1)
    drdx(1,3)=(x(1,3)-x(2,3))/r(1)
    drdx(1,4)=-drdx(1,1)
    drdx(1,5)=-drdx(1,2)
    drdx(1,6)=-drdx(1,3)
    drdx(2,1)=(x(1,1)-x(3,1))/r(2)
    drdx(2,2)=(x(1,2)-x(3,2))/r(2)
    drdx(2,3)=(x(1,3)-x(3,3))/r(2)
    drdx(2,7)=-drdx(2,1)
    drdx(2,8)=-drdx(2,2)
    drdx(2,9)=-drdx(2,3)
    drdx(3,4)=(x(2,1)-x(3,1))/r(3)
    drdx(3,5)=(x(2,2)-x(3,2))/r(3)
    drdx(3,6)=(x(2,3)-x(3,3))/r(3)
    drdx(3,7)=-drdx(3,4)
    drdx(3,8)=-drdx(3,5)
    drdx(3,9)=-drdx(3,6)

    call mproduct(gr,drdx,gx,ns,n*(n-1)/2,3*n)

    do i=1,n
      g(:,i,1)=gx(:,3*(i-1)+1)
      g(:,i,2)=gx(:,3*(i-1)+2)
      g(:,i,3)=gx(:,3*(i-1)+3)
    enddo 

  end subroutine grad_r_to_xyz

!==========================
! convert d/dr to d/dxyz for NAC
  subroutine nac_r_to_xyz(h,drdx,hr,r,x,n,ns)
    implicit none
    integer, intent(in) :: n, ns
    real*8, intent(in) :: x(n,3)
    real*8, intent(in) :: r(n*(n-1)/2)
    real*8, intent(in) :: hr(ns,ns,n*(n-1)/2)
    real*8, intent(inout) :: h(ns,ns,n,3)
    real*8, intent(inout) :: drdx(n*(n-1)/2,3*n)

    real*8 ::  hr_vec(ns*ns,n*(n-1)/2)
    real*8 ::  hx(ns*ns,3*n)

    integer :: i,j,k,l

    drdx=0.0d0

    ! compute drdx
    drdx(1,1)=(x(1,1)-x(2,1))/r(1)
    drdx(1,2)=(x(1,2)-x(2,2))/r(1)
    drdx(1,3)=(x(1,3)-x(2,3))/r(1)
    drdx(1,4)=-drdx(1,1)
    drdx(1,5)=-drdx(1,2)
    drdx(1,6)=-drdx(1,3)
    drdx(2,1)=(x(1,1)-x(3,1))/r(2)
    drdx(2,2)=(x(1,2)-x(3,2))/r(2)
    drdx(2,3)=(x(1,3)-x(3,3))/r(2)
    drdx(2,7)=-drdx(2,1)
    drdx(2,8)=-drdx(2,2)
    drdx(2,9)=-drdx(2,3)
    drdx(3,4)=(x(2,1)-x(3,1))/r(3)
    drdx(3,5)=(x(2,2)-x(3,2))/r(3)
    drdx(3,6)=(x(2,3)-x(3,3))/r(3)
    drdx(3,7)=-drdx(3,4)
    drdx(3,8)=-drdx(3,5)
    drdx(3,9)=-drdx(3,6)

    do i=1,ns
      do j=1,ns
        k=(i-1)*ns+j
        hr_vec(k,:)=hr(i,j,:)
      enddo
    enddo
    call mproduct(hr_vec,drdx,hx,ns*ns,n*(n-1)/2,3*n)

    do i=1,n
      do j=1,ns
        do k=1,ns
          l=(j-1)*ns+k
          h(j,k,i,1)=hx(l,3*(i-1)+1)
          h(j,k,i,2)=hx(l,3*(i-1)+2)
          h(j,k,i,3)=hx(l,3*(i-1)+3)
        enddo
      enddo
    enddo

  end subroutine nac_r_to_xyz

!==========================
! convert xyz to r
  subroutine convert_xyz_to_r(x,r,n)
    implicit none
    integer, intent(in) :: n
    real*8, intent(in) :: x(n,3)
    real*8, intent(out) :: r(n*(n-1)/2)
    integer :: i, j, l

    l=1
    !pairs of distances
    do i=1,n
      do j=i+1,n
        r(l)=sqrt((x(i,1)-x(j,1))**2+(x(i,2)-x(j,2))**2+(x(i,3)-x(j,3))**2)
        l=l+1
      enddo
    enddo

  endsubroutine convert_xyz_to_r

!=========================
! matrix vector product 
  subroutine mproduct(a,b,c,l,m,n)
    implicit none
    integer, intent(in) :: l, m, n
    real*8, intent(in) :: a(l,m), b(m,n)
    real*8, intent(inout) :: c(l,n)

    integer :: i, j, k
  
    do i=1,l
      do j=1,n
        c(i,j)=0.d0
        do k=1,m
          c(i,j)=c(i,j)+a(i,k)*b(k,j)
        enddo
      enddo
    enddo
    
    return
  endsubroutine mproduct

!===========================
! assign coefficients
  subroutine assign_coeff_bias(coeff_l0,coeff_l1,coeff_l2,coeff_l3,coeff_l4,bias_l0,bias_l1,bias_l2,bias_l3,bias_l4)
    implicit none
    real*8, intent(inout) :: coeff_l0(3,30), coeff_l1(30,65), coeff_l2(65,95), coeff_l3(95,125), coeff_l4(125,105)
    real*8, intent(inout) :: bias_l0(30), bias_l1(65), bias_l2(95), bias_l3(125), bias_l4(105)

!coeff layer 0
       coeff_l0(1,1)= -1.0398012400e+00
       coeff_l0(1,2)= -7.4754759669e-02
       coeff_l0(1,3)=  1.6810336113e+00
       coeff_l0(1,4)= -4.7095471621e-01
       coeff_l0(1,5)= -5.2835743874e-02
       coeff_l0(1,6)= -4.0939420462e-02
       coeff_l0(1,7)= -7.1858391166e-02
       coeff_l0(1,8)=  1.3363657892e-01
       coeff_l0(1,9)= -1.8201451302e+00
       coeff_l0(1,10)= -3.2007411122e-01
       coeff_l0(1,11)=  2.2620074451e-01
       coeff_l0(1,12)= -7.5529259443e-01
       coeff_l0(1,13)=  4.1610217094e-01
       coeff_l0(1,14)= -2.1343550086e-01
       coeff_l0(1,15)= -8.5928463936e-01
       coeff_l0(1,16)=  1.9790884852e-01
       coeff_l0(1,17)=  1.1987788230e-01
       coeff_l0(1,18)= -1.4117668569e-01
       coeff_l0(1,19)= -6.3915991783e-01
       coeff_l0(1,20)= -8.5114574432e-01
       coeff_l0(1,21)= -6.9679930806e-02
       coeff_l0(1,22)= -9.3517266214e-02
       coeff_l0(1,23)=  1.7511093616e-01
       coeff_l0(1,24)=  9.8075702786e-02
       coeff_l0(1,25)= -9.7578272223e-02
       coeff_l0(1,26)= -1.1023914814e+00
       coeff_l0(1,27)= -1.2000156194e-01
       coeff_l0(1,28)=  4.1472175717e-01
       coeff_l0(1,29)= -7.5395053625e-01
       coeff_l0(1,30)= -1.7234898806e+00
       coeff_l0(2,1)= -7.3872101307e-01
       coeff_l0(2,2)= -6.7387476563e-02
       coeff_l0(2,3)= -1.6860231161e+00
       coeff_l0(2,4)= -4.7489559650e-01
       coeff_l0(2,5)= -4.2566764355e-01
       coeff_l0(2,6)= -3.3196923323e-03
       coeff_l0(2,7)= -5.7601504028e-02
       coeff_l0(2,8)=  6.8680897355e-02
       coeff_l0(2,9)=  1.5618579090e-01
       coeff_l0(2,10)=  4.3466791511e-01
       coeff_l0(2,11)= -3.5826969147e-01
       coeff_l0(2,12)= -8.4528851509e-01
       coeff_l0(2,13)=  1.6708483920e-02
       coeff_l0(2,14)= -2.1440102160e-01
       coeff_l0(2,15)= -8.1968516111e-01
       coeff_l0(2,16)=  2.0691916347e-01
       coeff_l0(2,17)=  2.6819625497e-01
       coeff_l0(2,18)=  2.7936082333e-02
       coeff_l0(2,19)=  4.0126022696e-01
       coeff_l0(2,20)= -9.7129726410e-01
       coeff_l0(2,21)= -9.4937212765e-02
       coeff_l0(2,22)= -9.1533929110e-02
       coeff_l0(2,23)=  8.2600144669e-03
       coeff_l0(2,24)= -2.6303184032e-01
       coeff_l0(2,25)= -9.2521332204e-02
       coeff_l0(2,26)= -1.0621364117e+00
       coeff_l0(2,27)= -1.5793590248e-01
       coeff_l0(2,28)= -6.6167694330e-01
       coeff_l0(2,29)= -1.3277215958e+00
       coeff_l0(2,30)=  2.6564115286e-01
       coeff_l0(3,1)= -1.1862210035e+00
       coeff_l0(3,2)= -6.8964146078e-02
       coeff_l0(3,3)= -4.8646386131e-04
       coeff_l0(3,4)=  6.8231952190e-01
       coeff_l0(3,5)= -5.4854172468e-01
       coeff_l0(3,6)=  7.7894467115e-01
       coeff_l0(3,7)= -8.3507649601e-02
       coeff_l0(3,8)=  8.6010120809e-02
       coeff_l0(3,9)=  1.8010027707e-01
       coeff_l0(3,10)= -2.9972192645e-01
       coeff_l0(3,11)= -2.4520573020e-01
       coeff_l0(3,12)= -6.3631904125e-01
       coeff_l0(3,13)=  7.2705172002e-02
       coeff_l0(3,14)=  3.9078468084e-01
       coeff_l0(3,15)= -7.7053147554e-01
       coeff_l0(3,16)= -1.2851681709e+00
       coeff_l0(3,17)=  1.2130790949e-01
       coeff_l0(3,18)= -1.6101269424e-01
       coeff_l0(3,19)=  4.0241917968e-01
       coeff_l0(3,20)= -8.0947285891e-01
       coeff_l0(3,21)= -6.8634867668e-02
       coeff_l0(3,22)= -4.3066132814e-02
       coeff_l0(3,23)=  2.2076177597e-01
       coeff_l0(3,24)=  7.9646438360e-02
       coeff_l0(3,25)= -6.4481168985e-02
       coeff_l0(3,26)= -8.5213178396e-01
       coeff_l0(3,27)=  7.8755868599e-03
       coeff_l0(3,28)=  3.9578583837e-01
       coeff_l0(3,29)= -1.2905076742e+00
       coeff_l0(3,30)=  1.8517194688e-01
!coeff layer 1
       coeff_l1(1,1)=  2.3952613771e-01
       coeff_l1(1,2)=  1.2856754661e-01
       coeff_l1(1,3)= -1.3651050627e-01
       coeff_l1(1,4)= -2.4003267288e-01
       coeff_l1(1,5)=  1.9277086854e-01
       coeff_l1(1,6)= -9.2979729176e-02
       coeff_l1(1,7)=  2.1232001483e-01
       coeff_l1(1,8)=  6.1252671294e-03
       coeff_l1(1,9)= -1.1639665067e-01
       coeff_l1(1,10)=  7.9663008451e-02
       coeff_l1(1,11)= -1.5654927492e-01
       coeff_l1(1,12)= -7.4221313000e-02
       coeff_l1(1,13)=  2.3126332462e-01
       coeff_l1(1,14)=  9.8180741072e-02
       coeff_l1(1,15)=  3.0653131008e-01
       coeff_l1(1,16)= -1.7565259477e-03
       coeff_l1(1,17)= -1.0895318538e-01
       coeff_l1(1,18)=  1.4523738995e-02
       coeff_l1(1,19)= -1.5828356147e-01
       coeff_l1(1,20)=  6.7650534213e-02
       coeff_l1(1,21)=  2.8932685964e-03
       coeff_l1(1,22)=  1.5605084598e-01
       coeff_l1(1,23)=  2.3443247378e-01
       coeff_l1(1,24)= -4.6150032431e-02
       coeff_l1(1,25)= -7.7989928424e-02
       coeff_l1(1,26)= -1.4833883941e-01
       coeff_l1(1,27)= -8.4869936109e-02
       coeff_l1(1,28)= -4.3517641723e-02
       coeff_l1(1,29)= -1.8852050602e-01
       coeff_l1(1,30)= -1.3652305305e-01
       coeff_l1(1,31)= -1.9474940002e-01
       coeff_l1(1,32)= -2.1691404283e-01
       coeff_l1(1,33)= -2.4294140935e-01
       coeff_l1(1,34)= -1.6272498295e-02
       coeff_l1(1,35)= -3.9917565882e-02
       coeff_l1(1,36)=  7.6663382351e-02
       coeff_l1(1,37)=  4.2142156512e-02
       coeff_l1(1,38)= -5.3262270987e-02
       coeff_l1(1,39)= -1.4715985954e-01
       coeff_l1(1,40)= -3.3180758357e-01
       coeff_l1(1,41)=  7.0780642331e-02
       coeff_l1(1,42)=  2.0248486102e-01
       coeff_l1(1,43)= -3.9872944355e-02
       coeff_l1(1,44)=  1.3596194983e-01
       coeff_l1(1,45)=  1.3899752498e-01
       coeff_l1(1,46)= -7.9103991389e-02
       coeff_l1(1,47)= -2.0141701400e-01
       coeff_l1(1,48)=  1.2451200187e-01
       coeff_l1(1,49)=  2.3469097912e-01
       coeff_l1(1,50)=  1.0370508581e-01
       coeff_l1(1,51)=  6.9361582398e-02
       coeff_l1(1,52)= -1.4881150424e-01
       coeff_l1(1,53)=  1.8809355795e-01
       coeff_l1(1,54)= -1.9703957438e-01
       coeff_l1(1,55)=  2.3097580671e-01
       coeff_l1(1,56)= -1.2821860611e-01
       coeff_l1(1,57)=  2.5908514857e-01
       coeff_l1(1,58)=  8.8786222041e-02
       coeff_l1(1,59)=  2.5118109584e-01
       coeff_l1(1,60)= -1.4072778821e-01
       coeff_l1(1,61)=  1.1914338171e-01
       coeff_l1(1,62)=  8.2517810166e-02
       coeff_l1(1,63)= -1.0216119885e-01
       coeff_l1(1,64)=  4.8539163545e-03
       coeff_l1(1,65)=  4.9122843891e-02
       coeff_l1(2,1)=  4.0323483944e-01
       coeff_l1(2,2)= -4.8836622387e-02
       coeff_l1(2,3)=  2.5489723682e-01
       coeff_l1(2,4)=  8.4559124708e-01
       coeff_l1(2,5)=  1.6263987124e-01
       coeff_l1(2,6)= -6.3531599939e-02
       coeff_l1(2,7)= -3.4373512864e-01
       coeff_l1(2,8)=  3.0934356153e-02
       coeff_l1(2,9)= -1.2692922354e-01
       coeff_l1(2,10)=  1.3787752390e-01
       coeff_l1(2,11)= -3.6234106869e-02
       coeff_l1(2,12)=  1.6980542243e-01
       coeff_l1(2,13)=  5.7179264724e-02
       coeff_l1(2,14)= -3.9909678698e-01
       coeff_l1(2,15)=  5.8634108305e-01
       coeff_l1(2,16)=  5.0251871347e-01
       coeff_l1(2,17)=  3.1816735268e+00
       coeff_l1(2,18)= -3.2921850681e-01
       coeff_l1(2,19)= -1.8888045847e-01
       coeff_l1(2,20)= -1.0705747455e-01
       coeff_l1(2,21)= -1.5128682554e-01
       coeff_l1(2,22)= -8.2242137194e-01
       coeff_l1(2,23)= -4.4118037820e-01
       coeff_l1(2,24)=  3.2061893940e+00
       coeff_l1(2,25)= -3.4605376422e-03
       coeff_l1(2,26)=  7.2149664629e-04
       coeff_l1(2,27)=  1.1037002802e+00
       coeff_l1(2,28)=  7.1447992325e-01
       coeff_l1(2,29)= -4.5361742377e-03
       coeff_l1(2,30)=  2.4939803779e-01
       coeff_l1(2,31)=  5.3595155478e-01
       coeff_l1(2,32)=  7.2577551007e-02
       coeff_l1(2,33)=  3.2225021720e-01
       coeff_l1(2,34)=  1.4457713813e-02
       coeff_l1(2,35)=  1.5712170303e-01
       coeff_l1(2,36)= -2.5645142794e-01
       coeff_l1(2,37)=  1.6455357894e-02
       coeff_l1(2,38)= -3.8007932901e-01
       coeff_l1(2,39)=  6.7197158933e-02
       coeff_l1(2,40)= -1.0177334547e+00
       coeff_l1(2,41)=  1.1670695245e-01
       coeff_l1(2,42)= -8.8119715452e-01
       coeff_l1(2,43)= -1.1794526577e+00
       coeff_l1(2,44)=  4.3435895443e-01
       coeff_l1(2,45)=  2.3910617828e+00
       coeff_l1(2,46)= -1.1001442373e-01
       coeff_l1(2,47)= -1.0581561327e+00
       coeff_l1(2,48)=  1.2654052675e-01
       coeff_l1(2,49)=  2.6212197542e-01
       coeff_l1(2,50)= -3.1180566549e-01
       coeff_l1(2,51)=  6.5356396139e-02
       coeff_l1(2,52)=  7.7328157425e-01
       coeff_l1(2,53)=  1.6390421987e-01
       coeff_l1(2,54)=  8.3520874381e-02
       coeff_l1(2,55)= -3.5756400228e-01
       coeff_l1(2,56)= -2.4839852750e-01
       coeff_l1(2,57)=  3.5207949579e-02
       coeff_l1(2,58)= -1.0105706751e-01
       coeff_l1(2,59)= -9.8226690292e-01
       coeff_l1(2,60)= -1.7821199894e+00
       coeff_l1(2,61)= -2.1262362599e-01
       coeff_l1(2,62)=  6.9249972701e-02
       coeff_l1(2,63)= -1.0782990456e+00
       coeff_l1(2,64)=  6.2554335594e-01
       coeff_l1(2,65)= -3.0215367675e-01
       coeff_l1(3,1)= -2.4024282396e-01
       coeff_l1(3,2)= -1.8033447862e-01
       coeff_l1(3,3)= -1.3946731389e-01
       coeff_l1(3,4)= -7.7945344150e-02
       coeff_l1(3,5)= -2.7049636841e-01
       coeff_l1(3,6)=  3.6233872175e-01
       coeff_l1(3,7)=  1.4052340388e-01
       coeff_l1(3,8)= -2.4375627935e-01
       coeff_l1(3,9)= -3.6434909701e-01
       coeff_l1(3,10)= -3.5163393617e-01
       coeff_l1(3,11)= -1.9602252170e-02
       coeff_l1(3,12)=  1.9163858891e-01
       coeff_l1(3,13)=  8.8543899357e-02
       coeff_l1(3,14)=  3.2786108553e-02
       coeff_l1(3,15)= -1.8732391298e-01
       coeff_l1(3,16)=  9.3389284611e-01
       coeff_l1(3,17)=  1.5777906775e-01
       coeff_l1(3,18)= -1.1607467383e-01
       coeff_l1(3,19)= -1.5372426808e-01
       coeff_l1(3,20)=  6.6311681271e-01
       coeff_l1(3,21)=  3.8954910636e-01
       coeff_l1(3,22)= -3.1514912844e-01
       coeff_l1(3,23)= -1.7507925630e-01
       coeff_l1(3,24)=  3.5567171872e-02
       coeff_l1(3,25)=  2.5961801410e-01
       coeff_l1(3,26)=  5.0044643879e-01
       coeff_l1(3,27)=  3.4141755104e-01
       coeff_l1(3,28)=  7.2596639395e-01
       coeff_l1(3,29)=  4.2666745186e-01
       coeff_l1(3,30)= -9.1105930507e-02
       coeff_l1(3,31)=  2.7364692092e-01
       coeff_l1(3,32)= -1.0307436436e-01
       coeff_l1(3,33)=  3.8014736772e-01
       coeff_l1(3,34)=  2.0608273149e-01
       coeff_l1(3,35)=  3.1127003953e-02
       coeff_l1(3,36)=  1.1868169159e-01
       coeff_l1(3,37)=  2.1356566250e-01
       coeff_l1(3,38)= -2.6100763679e-01
       coeff_l1(3,39)= -2.7960270643e-01
       coeff_l1(3,40)= -1.1644729227e-01
       coeff_l1(3,41)= -1.8284280598e-01
       coeff_l1(3,42)=  6.3118889928e-02
       coeff_l1(3,43)= -6.0817599297e-01
       coeff_l1(3,44)=  1.6648147106e+00
       coeff_l1(3,45)= -7.4131868780e-02
       coeff_l1(3,46)= -3.8262388110e-01
       coeff_l1(3,47)= -7.8284674883e-01
       coeff_l1(3,48)= -1.9741661847e-01
       coeff_l1(3,49)= -5.6587970257e-01
       coeff_l1(3,50)= -3.6059337854e-01
       coeff_l1(3,51)=  1.0403163731e-01
       coeff_l1(3,52)=  3.1583333015e-01
       coeff_l1(3,53)= -2.9409408569e-01
       coeff_l1(3,54)= -6.8507599831e-01
       coeff_l1(3,55)=  9.1791808605e-01
       coeff_l1(3,56)= -1.0685511827e+00
       coeff_l1(3,57)= -2.9348227382e-01
       coeff_l1(3,58)= -1.7466372252e-01
       coeff_l1(3,59)=  8.2755953074e-02
       coeff_l1(3,60)= -3.2000029087e-01
       coeff_l1(3,61)= -3.9860278368e-02
       coeff_l1(3,62)=  5.0411935896e-02
       coeff_l1(3,63)=  1.9841784239e-01
       coeff_l1(3,64)= -1.9579710066e-01
       coeff_l1(3,65)= -2.2562500834e-01
       coeff_l1(4,1)=  4.1170991957e-02
       coeff_l1(4,2)= -6.3418626785e-02
       coeff_l1(4,3)=  1.5150304139e-01
       coeff_l1(4,4)=  6.9023036957e-01
       coeff_l1(4,5)= -7.8186297417e-01
       coeff_l1(4,6)= -4.8461604118e-01
       coeff_l1(4,7)=  6.1257451773e-01
       coeff_l1(4,8)=  4.1349118948e-01
       coeff_l1(4,9)= -8.2688635588e-01
       coeff_l1(4,10)= -3.9018487930e-01
       coeff_l1(4,11)=  2.4240435660e-01
       coeff_l1(4,12)=  5.8233487606e-01
       coeff_l1(4,13)= -6.8217194080e-01
       coeff_l1(4,14)= -4.8703733087e-01
       coeff_l1(4,15)=  6.9549804926e-01
       coeff_l1(4,16)= -8.7702393532e-01
       coeff_l1(4,17)=  2.8265577555e-01
       coeff_l1(4,18)=  3.9129039645e-01
       coeff_l1(4,19)= -1.2093024254e+00
       coeff_l1(4,20)= -1.7636279762e-01
       coeff_l1(4,21)= -5.2556675673e-01
       coeff_l1(4,22)=  6.8781934679e-02
       coeff_l1(4,23)= -5.4897689819e-01
       coeff_l1(4,24)=  6.6738754511e-01
       coeff_l1(4,25)= -4.5483464003e-01
       coeff_l1(4,26)= -1.5412831306e+00
       coeff_l1(4,27)= -1.0876704454e+00
       coeff_l1(4,28)=  2.7695930004e+00
       coeff_l1(4,29)= -7.1094405651e-01
       coeff_l1(4,30)=  1.6037605703e-01
       coeff_l1(4,31)=  3.2533118129e-01
       coeff_l1(4,32)= -9.2849510908e-01
       coeff_l1(4,33)=  5.3454148769e-01
       coeff_l1(4,34)=  1.9951486588e-01
       coeff_l1(4,35)= -3.4516558051e-01
       coeff_l1(4,36)=  9.3135017157e-01
       coeff_l1(4,37)= -1.0449413061e+00
       coeff_l1(4,38)= -8.4440827370e-01
       coeff_l1(4,39)=  5.5487221479e-01
       coeff_l1(4,40)= -6.3198357821e-02
       coeff_l1(4,41)= -3.4961518645e-01
       coeff_l1(4,42)=  8.3922266960e-02
       coeff_l1(4,43)= -7.7751469612e-01
       coeff_l1(4,44)= -2.5889480114e-01
       coeff_l1(4,45)= -3.8678282499e-01
       coeff_l1(4,46)=  6.9512087107e-01
       coeff_l1(4,47)= -1.0631397963e+00
       coeff_l1(4,48)= -5.8062016964e-01
       coeff_l1(4,49)= -4.2318639159e-01
       coeff_l1(4,50)=  2.5359299779e-01
       coeff_l1(4,51)=  1.9841246307e-01
       coeff_l1(4,52)= -7.4039375782e-01
       coeff_l1(4,53)= -2.4780163169e-01
       coeff_l1(4,54)= -4.2145964503e-01
       coeff_l1(4,55)= -8.3503782749e-02
       coeff_l1(4,56)= -6.3768172264e-01
       coeff_l1(4,57)=  2.5714555383e-01
       coeff_l1(4,58)=  1.9058290869e-02
       coeff_l1(4,59)= -9.8868846893e-01
       coeff_l1(4,60)=  4.1517713666e-01
       coeff_l1(4,61)= -1.8656525016e-01
       coeff_l1(4,62)= -1.8483881652e-01
       coeff_l1(4,63)= -3.2156103849e-01
       coeff_l1(4,64)=  1.0231697559e-01
       coeff_l1(4,65)= -9.7750043869e-01
       coeff_l1(5,1)= -2.7013596892e-01
       coeff_l1(5,2)=  2.2503834218e-02
       coeff_l1(5,3)= -6.2676632404e-01
       coeff_l1(5,4)=  1.6688162088e-01
       coeff_l1(5,5)= -1.6359783709e-01
       coeff_l1(5,6)= -8.7523713708e-02
       coeff_l1(5,7)= -2.3328174651e-01
       coeff_l1(5,8)= -1.6383945942e-01
       coeff_l1(5,9)= -5.0334598869e-02
       coeff_l1(5,10)=  2.2057873011e-01
       coeff_l1(5,11)= -8.6261741817e-02
       coeff_l1(5,12)= -8.6891047657e-02
       coeff_l1(5,13)=  3.3142751455e-01
       coeff_l1(5,14)=  6.9669499993e-02
       coeff_l1(5,15)=  5.1436299086e-01
       coeff_l1(5,16)= -2.4680754542e-01
       coeff_l1(5,17)=  2.6422703266e-01
       coeff_l1(5,18)= -1.5417882800e-01
       coeff_l1(5,19)=  1.6166083515e-01
       coeff_l1(5,20)= -1.1859215051e-01
       coeff_l1(5,21)= -6.8188108504e-02
       coeff_l1(5,22)= -8.9577353001e-01
       coeff_l1(5,23)=  3.0981352925e-01
       coeff_l1(5,24)=  2.4724353850e-01
       coeff_l1(5,25)= -1.2512789667e-01
       coeff_l1(5,26)= -1.0642649233e-01
       coeff_l1(5,27)= -7.7441796660e-02
       coeff_l1(5,28)=  3.1831124425e-01
       coeff_l1(5,29)=  5.1012050360e-02
       coeff_l1(5,30)=  6.3131831586e-02
       coeff_l1(5,31)= -1.9628170878e-02
       coeff_l1(5,32)=  2.1677719057e-01
       coeff_l1(5,33)= -2.5030247867e-02
       coeff_l1(5,34)=  7.5175285339e-02
       coeff_l1(5,35)=  2.8150575235e-02
       coeff_l1(5,36)=  1.0171023756e-01
       coeff_l1(5,37)=  2.3926199973e-01
       coeff_l1(5,38)= -3.8743677735e-01
       coeff_l1(5,39)= -2.2772923112e-02
       coeff_l1(5,40)= -9.7985984758e-03
       coeff_l1(5,41)= -8.2791440189e-02
       coeff_l1(5,42)= -3.8860601187e-01
       coeff_l1(5,43)= -1.0116823018e-01
       coeff_l1(5,44)= -1.4842526615e-01
       coeff_l1(5,45)=  4.2588061094e-01
       coeff_l1(5,46)= -1.4176480472e-03
       coeff_l1(5,47)=  1.3550193049e-02
       coeff_l1(5,48)=  3.3666782081e-02
       coeff_l1(5,49)=  1.2199215591e-01
       coeff_l1(5,50)= -5.0807094574e-01
       coeff_l1(5,51)= -2.1129518747e-01
       coeff_l1(5,52)=  2.9050981998e-01
       coeff_l1(5,53)=  1.7940317094e-01
       coeff_l1(5,54)= -2.5918096304e-01
       coeff_l1(5,55)=  6.1010640860e-01
       coeff_l1(5,56)=  6.4450047910e-02
       coeff_l1(5,57)=  1.9662836194e-01
       coeff_l1(5,58)=  1.8922555447e-01
       coeff_l1(5,59)=  9.5096215606e-02
       coeff_l1(5,60)= -1.9432044029e-01
       coeff_l1(5,61)= -1.7334140837e-02
       coeff_l1(5,62)=  5.8231059462e-02
       coeff_l1(5,63)= -5.3853571415e-01
       coeff_l1(5,64)=  6.9858551025e-02
       coeff_l1(5,65)=  2.2905617952e-02
       coeff_l1(6,1)=  7.8840595484e-01
       coeff_l1(6,2)= -2.2134397924e-01
       coeff_l1(6,3)= -3.3741739392e-01
       coeff_l1(6,4)= -1.1286331415e+00
       coeff_l1(6,5)= -8.0418598652e-01
       coeff_l1(6,6)= -1.1600826979e+00
       coeff_l1(6,7)= -1.2647350319e-02
       coeff_l1(6,8)=  6.2949156761e-01
       coeff_l1(6,9)= -5.8244878054e-01
       coeff_l1(6,10)= -7.8495991230e-01
       coeff_l1(6,11)= -9.9081255496e-02
       coeff_l1(6,12)= -1.1219401658e-01
       coeff_l1(6,13)=  1.9715264440e-01
       coeff_l1(6,14)= -7.5125938654e-01
       coeff_l1(6,15)=  9.4576857984e-02
       coeff_l1(6,16)= -7.3199266195e-01
       coeff_l1(6,17)= -1.1259294748e+00
       coeff_l1(6,18)=  5.8352112770e-02
       coeff_l1(6,19)= -7.4253439903e-01
       coeff_l1(6,20)= -1.2298607081e-01
       coeff_l1(6,21)= -5.7983046770e-01
       coeff_l1(6,22)=  8.3947157860e-01
       coeff_l1(6,23)=  6.1431229115e-01
       coeff_l1(6,24)= -1.1312830448e+00
       coeff_l1(6,25)=  8.0869264901e-02
       coeff_l1(6,26)=  5.1438611746e-01
       coeff_l1(6,27)= -3.0886521935e-01
       coeff_l1(6,28)= -7.0488560200e-01
       coeff_l1(6,29)=  5.8755987883e-01
       coeff_l1(6,30)= -2.8458669782e-01
       coeff_l1(6,31)= -3.9540925622e-01
       coeff_l1(6,32)=  7.6586699486e-01
       coeff_l1(6,33)= -7.0988017321e-01
       coeff_l1(6,34)=  3.2822170854e-01
       coeff_l1(6,35)=  4.0247324109e-01
       coeff_l1(6,36)= -1.0802950710e-01
       coeff_l1(6,37)= -3.1162005663e-01
       coeff_l1(6,38)= -6.6712850332e-01
       coeff_l1(6,39)= -3.2040376216e-02
       coeff_l1(6,40)=  1.6327987611e-01
       coeff_l1(6,41)=  1.4427986741e-01
       coeff_l1(6,42)= -7.3854696751e-01
       coeff_l1(6,43)=  1.8427511454e+00
       coeff_l1(6,44)= -1.8671368361e+00
       coeff_l1(6,45)=  7.4312388897e-01
       coeff_l1(6,46)= -1.5185749158e-02
       coeff_l1(6,47)=  1.4091328382e+00
       coeff_l1(6,48)=  1.1923128366e-01
       coeff_l1(6,49)= -4.9702984095e-01
       coeff_l1(6,50)=  1.4775498211e-01
       coeff_l1(6,51)= -2.7868443727e-01
       coeff_l1(6,52)= -7.8789693117e-01
       coeff_l1(6,53)=  4.3122136593e-01
       coeff_l1(6,54)=  1.5331262350e+00
       coeff_l1(6,55)=  1.0879464149e+00
       coeff_l1(6,56)= -1.3013881445e-01
       coeff_l1(6,57)= -1.5445050001e+00
       coeff_l1(6,58)=  4.0677896142e-01
       coeff_l1(6,59)=  1.3688310385e+00
       coeff_l1(6,60)= -4.5320749283e-01
       coeff_l1(6,61)= -5.5668455362e-01
       coeff_l1(6,62)=  1.2196631730e-01
       coeff_l1(6,63)= -9.9594908953e-01
       coeff_l1(6,64)=  2.1056738496e-01
       coeff_l1(6,65)= -9.5471322536e-01
       coeff_l1(7,1)=  4.1973814368e-01
       coeff_l1(7,2)= -4.6322807670e-02
       coeff_l1(7,3)=  8.5610109568e-01
       coeff_l1(7,4)=  7.2545695305e-01
       coeff_l1(7,5)=  2.6378956437e-01
       coeff_l1(7,6)=  1.9365869462e-01
       coeff_l1(7,7)= -1.7751261592e-01
       coeff_l1(7,8)= -8.1381872296e-03
       coeff_l1(7,9)= -1.3235296309e-01
       coeff_l1(7,10)= -2.1328359842e-02
       coeff_l1(7,11)=  4.7297213227e-02
       coeff_l1(7,12)=  4.3986925483e-01
       coeff_l1(7,13)=  7.6963447034e-02
       coeff_l1(7,14)= -4.0775182843e-01
       coeff_l1(7,15)=  3.8770124316e-01
       coeff_l1(7,16)=  1.6665133834e-01
       coeff_l1(7,17)=  2.9031424522e+00
       coeff_l1(7,18)= -2.1251630783e-01
       coeff_l1(7,19)=  8.8394910097e-02
       coeff_l1(7,20)=  2.6235896349e-01
       coeff_l1(7,21)= -3.0443421006e-01
       coeff_l1(7,22)= -3.9421755075e-01
       coeff_l1(7,23)= -2.0301562548e-01
       coeff_l1(7,24)=  2.8057534695e+00
       coeff_l1(7,25)= -2.7871900797e-01
       coeff_l1(7,26)= -2.1172093228e-02
       coeff_l1(7,27)=  5.3653293848e-01
       coeff_l1(7,28)=  4.0416556597e-01
       coeff_l1(7,29)= -2.3424026370e-01
       coeff_l1(7,30)= -1.5034469962e-01
       coeff_l1(7,31)=  2.7996644378e-01
       coeff_l1(7,32)= -7.0358559489e-02
       coeff_l1(7,33)=  4.6371543407e-01
       coeff_l1(7,34)=  7.9669222236e-02
       coeff_l1(7,35)=  1.1395023763e-01
       coeff_l1(7,36)=  2.1949003637e-01
       coeff_l1(7,37)= -2.0961296558e-01
       coeff_l1(7,38)=  1.7064106464e-01
       coeff_l1(7,39)= -3.0737224221e-01
       coeff_l1(7,40)= -6.7424756289e-01
       coeff_l1(7,41)=  1.4637137949e-01
       coeff_l1(7,42)= -1.1377898455e+00
       coeff_l1(7,43)= -6.0041165352e-01
       coeff_l1(7,44)=  3.8144007325e-01
       coeff_l1(7,45)=  2.1516878605e+00
       coeff_l1(7,46)=  1.7217254639e-01
       coeff_l1(7,47)= -3.7749439478e-01
       coeff_l1(7,48)=  1.1686776578e-01
       coeff_l1(7,49)=  2.2311861813e-01
       coeff_l1(7,50)= -3.1883157790e-02
       coeff_l1(7,51)= -1.4495077729e-01
       coeff_l1(7,52)=  4.9743565917e-01
       coeff_l1(7,53)=  2.2230826318e-02
       coeff_l1(7,54)=  7.1185670793e-02
       coeff_l1(7,55)= -4.8061820865e-01
       coeff_l1(7,56)=  3.5249922425e-02
       coeff_l1(7,57)= -3.2192364335e-02
       coeff_l1(7,58)=  2.1519599855e-01
       coeff_l1(7,59)= -8.9355117083e-01
       coeff_l1(7,60)= -1.3000423908e+00
       coeff_l1(7,61)= -1.4840668440e-01
       coeff_l1(7,62)= -1.3077113032e-01
       coeff_l1(7,63)= -7.7583229542e-01
       coeff_l1(7,64)=  9.6298265457e-01
       coeff_l1(7,65)=  8.8665083051e-02
       coeff_l1(8,1)= -5.7005679607e-01
       coeff_l1(8,2)= -4.6262472868e-01
       coeff_l1(8,3)=  4.4153118134e-01
       coeff_l1(8,4)=  2.0129182339e+00
       coeff_l1(8,5)= -6.7538946867e-01
       coeff_l1(8,6)= -2.6406767964e-01
       coeff_l1(8,7)= -4.4313779473e-01
       coeff_l1(8,8)= -9.5764964819e-01
       coeff_l1(8,9)=  7.1733079851e-02
       coeff_l1(8,10)= -2.5458383560e-01
       coeff_l1(8,11)= -3.7356179953e-01
       coeff_l1(8,12)=  2.6907196641e-01
       coeff_l1(8,13)= -4.5580980182e-01
       coeff_l1(8,14)= -2.6832827926e-01
       coeff_l1(8,15)= -9.6121817827e-01
       coeff_l1(8,16)= -4.9978703260e-01
       coeff_l1(8,17)=  1.3858450651e+00
       coeff_l1(8,18)= -1.0518896580e+00
       coeff_l1(8,19)= -1.5806318820e-01
       coeff_l1(8,20)= -1.1293381453e+00
       coeff_l1(8,21)= -8.0709767342e-01
       coeff_l1(8,22)=  6.7230892181e-01
       coeff_l1(8,23)=  3.1797502190e-02
       coeff_l1(8,24)=  1.5137845278e+00
       coeff_l1(8,25)= -3.9458805323e-01
       coeff_l1(8,26)= -1.2730152607e+00
       coeff_l1(8,27)=  6.4245933294e-01
       coeff_l1(8,28)=  3.4112116694e-01
       coeff_l1(8,29)= -2.9330706596e-01
       coeff_l1(8,30)=  1.8284621835e-01
       coeff_l1(8,31)= -3.5739535093e-01
       coeff_l1(8,32)= -8.8722074032e-01
       coeff_l1(8,33)=  9.2396259308e-01
       coeff_l1(8,34)= -9.9305761978e-03
       coeff_l1(8,35)=  5.4342365265e-01
       coeff_l1(8,36)= -5.1740849018e-01
       coeff_l1(8,37)= -6.6067612171e-01
       coeff_l1(8,38)= -1.2767024338e-01
       coeff_l1(8,39)= -4.2836371064e-01
       coeff_l1(8,40)=  4.2067837715e-01
       coeff_l1(8,41)= -2.5452813506e-01
       coeff_l1(8,42)= -1.7469123006e-01
       coeff_l1(8,43)=  7.8614246845e-01
       coeff_l1(8,44)= -8.1417687237e-02
       coeff_l1(8,45)=  1.3961833715e+00
       coeff_l1(8,46)= -7.2639524937e-01
       coeff_l1(8,47)=  7.2881740332e-01
       coeff_l1(8,48)= -2.3202677071e-01
       coeff_l1(8,49)= -7.8447693586e-01
       coeff_l1(8,50)= -6.2234026194e-01
       coeff_l1(8,51)= -5.3228437901e-01
       coeff_l1(8,52)=  9.6528106928e-01
       coeff_l1(8,53)= -8.0278879404e-01
       coeff_l1(8,54)= -1.2331653833e+00
       coeff_l1(8,55)= -1.3615390062e+00
       coeff_l1(8,56)= -8.0572271347e-01
       coeff_l1(8,57)= -1.7749652267e-01
       coeff_l1(8,58)= -8.1187438965e-01
       coeff_l1(8,59)= -5.4536592960e-01
       coeff_l1(8,60)=  3.6890155077e-01
       coeff_l1(8,61)= -6.0346150398e-01
       coeff_l1(8,62)= -4.4717857242e-01
       coeff_l1(8,63)=  1.6433127224e-01
       coeff_l1(8,64)=  7.6900768280e-01
       coeff_l1(8,65)= -5.7117456198e-01
       coeff_l1(9,1)=  4.0544766933e-02
       coeff_l1(9,2)= -6.6868767142e-02
       coeff_l1(9,3)=  5.1957195997e-01
       coeff_l1(9,4)= -2.2734366357e-01
       coeff_l1(9,5)=  2.5769063830e-01
       coeff_l1(9,6)= -6.3760802150e-02
       coeff_l1(9,7)= -1.4721080661e-01
       coeff_l1(9,8)=  2.2572764754e-01
       coeff_l1(9,9)=  5.2856974304e-02
       coeff_l1(9,10)=  3.3560150862e-01
       coeff_l1(9,11)=  2.2418583930e-01
       coeff_l1(9,12)= -8.7531730533e-02
       coeff_l1(9,13)= -4.4360339642e-02
       coeff_l1(9,14)= -2.8550463915e-01
       coeff_l1(9,15)=  5.8667695522e-01
       coeff_l1(9,16)=  4.1848972440e-01
       coeff_l1(9,17)=  1.2855714560e+00
       coeff_l1(9,18)= -4.1985362768e-01
       coeff_l1(9,19)= -1.4888057113e-01
       coeff_l1(9,20)= -1.1787122488e-01
       coeff_l1(9,21)=  1.0378801078e-01
       coeff_l1(9,22)=  1.8244343996e-01
       coeff_l1(9,23)=  1.0616274551e-02
       coeff_l1(9,24)=  1.3347126245e+00
       coeff_l1(9,25)= -9.0442337096e-02
       coeff_l1(9,26)= -4.0333319455e-02
       coeff_l1(9,27)=  3.7039470673e-01
       coeff_l1(9,28)=  2.7834460139e-01
       coeff_l1(9,29)= -1.2687721848e-01
       coeff_l1(9,30)= -6.1345241964e-02
       coeff_l1(9,31)=  2.0570410416e-03
       coeff_l1(9,32)=  2.9394772649e-01
       coeff_l1(9,33)= -2.3070287704e-01
       coeff_l1(9,34)=  1.7451781034e-01
       coeff_l1(9,35)= -1.2281069905e-01
       coeff_l1(9,36)= -9.7827821970e-02
       coeff_l1(9,37)= -2.3815727234e-01
       coeff_l1(9,38)=  9.1732852161e-03
       coeff_l1(9,39)= -9.3969777226e-02
       coeff_l1(9,40)= -4.1815314442e-02
       coeff_l1(9,41)=  1.9386660308e-02
       coeff_l1(9,42)= -1.4955136776e+00
       coeff_l1(9,43)= -4.1209197044e-01
       coeff_l1(9,44)= -2.2380864620e-01
       coeff_l1(9,45)=  1.6052772999e+00
       coeff_l1(9,46)=  2.9885032773e-01
       coeff_l1(9,47)= -4.8558622599e-02
       coeff_l1(9,48)=  3.0428779125e-01
       coeff_l1(9,49)=  2.7658861876e-01
       coeff_l1(9,50)=  8.2829929888e-02
       coeff_l1(9,51)= -1.0738574713e-01
       coeff_l1(9,52)=  1.6026444733e-01
       coeff_l1(9,53)= -1.0596199334e-01
       coeff_l1(9,54)=  7.8554028273e-01
       coeff_l1(9,55)= -1.7186076641e+00
       coeff_l1(9,56)=  4.0945723653e-01
       coeff_l1(9,57)=  4.5416030288e-01
       coeff_l1(9,58)= -2.2513824701e-01
       coeff_l1(9,59)=  5.5631671101e-02
       coeff_l1(9,60)= -6.0233235359e-01
       coeff_l1(9,61)= -8.3337265253e-01
       coeff_l1(9,62)= -2.0957189798e-01
       coeff_l1(9,63)= -2.7587142587e-01
       coeff_l1(9,64)=  5.0832748413e-01
       coeff_l1(9,65)=  3.8298702240e-01
       coeff_l1(10,1)= -9.4907157123e-02
       coeff_l1(10,2)=  1.9177292287e-01
       coeff_l1(10,3)= -1.1581302434e-01
       coeff_l1(10,4)=  1.1698990315e-01
       coeff_l1(10,5)=  1.7363263667e-01
       coeff_l1(10,6)=  4.0482693911e-01
       coeff_l1(10,7)=  1.3081772625e-01
       coeff_l1(10,8)= -4.2507594824e-01
       coeff_l1(10,9)=  9.3899205327e-02
       coeff_l1(10,10)= -3.3501467109e-01
       coeff_l1(10,11)= -3.3063501120e-02
       coeff_l1(10,12)=  4.9741986394e-01
       coeff_l1(10,13)=  3.3791914582e-01
       coeff_l1(10,14)= -7.1363970637e-02
       coeff_l1(10,15)= -6.5124315023e-01
       coeff_l1(10,16)=  1.2666256428e+00
       coeff_l1(10,17)=  1.0121111870e+00
       coeff_l1(10,18)= -2.3958045244e-01
       coeff_l1(10,19)= -3.4646549821e-01
       coeff_l1(10,20)= -7.0590496063e-02
       coeff_l1(10,21)= -1.1748676747e-01
       coeff_l1(10,22)= -1.0281323195e+00
       coeff_l1(10,23)= -4.3940237164e-01
       coeff_l1(10,24)=  7.7783381939e-01
       coeff_l1(10,25)= -3.0056661367e-01
       coeff_l1(10,26)=  1.4897963405e-01
       coeff_l1(10,27)=  7.7843028307e-01
       coeff_l1(10,28)= -3.0428874493e-01
       coeff_l1(10,29)=  3.0933383107e-01
       coeff_l1(10,30)= -2.9791408777e-01
       coeff_l1(10,31)=  8.2294508815e-02
       coeff_l1(10,32)= -3.0546498299e-01
       coeff_l1(10,33)=  1.0207920074e+00
       coeff_l1(10,34)= -1.8117956817e-01
       coeff_l1(10,35)=  3.5407084227e-01
       coeff_l1(10,36)=  6.0280159116e-02
       coeff_l1(10,37)=  2.7405101806e-02
       coeff_l1(10,38)= -6.3991492987e-01
       coeff_l1(10,39)= -7.8279070556e-02
       coeff_l1(10,40)= -3.9291477203e-01
       coeff_l1(10,41)= -3.0289745331e-01
       coeff_l1(10,42)= -9.6142061055e-02
       coeff_l1(10,43)= -1.5729209185e+00
       coeff_l1(10,44)=  1.3281764984e+00
       coeff_l1(10,45)=  4.7383090854e-01
       coeff_l1(10,46)= -2.9619389772e-01
       coeff_l1(10,47)= -1.9249120951e+00
       coeff_l1(10,48)=  1.0043984652e-01
       coeff_l1(10,49)=  1.4668975770e-01
       coeff_l1(10,50)= -1.4113333821e-01
       coeff_l1(10,51)=  1.9324029982e-01
       coeff_l1(10,52)=  9.7626870871e-01
       coeff_l1(10,53)=  3.3838257194e-01
       coeff_l1(10,54)= -6.5727287531e-01
       coeff_l1(10,55)=  1.0288484097e+00
       coeff_l1(10,56)= -6.1942428350e-01
       coeff_l1(10,57)=  2.3168314993e-01
       coeff_l1(10,58)=  1.1531746387e-01
       coeff_l1(10,59)= -9.5955479145e-01
       coeff_l1(10,60)= -8.8550001383e-01
       coeff_l1(10,61)=  5.9471935034e-01
       coeff_l1(10,62)= -5.9087701142e-02
       coeff_l1(10,63)= -1.7746691406e-01
       coeff_l1(10,64)= -1.5649619699e-01
       coeff_l1(10,65)= -2.6350006461e-01
       coeff_l1(11,1)=  3.3733266592e-01
       coeff_l1(11,2)= -1.1534968019e-01
       coeff_l1(11,3)=  6.8195903301e-01
       coeff_l1(11,4)=  6.1559039354e-01
       coeff_l1(11,5)=  3.3400595188e-01
       coeff_l1(11,6)= -1.5668913722e-02
       coeff_l1(11,7)=  7.2195768356e-02
       coeff_l1(11,8)= -2.9359143972e-01
       coeff_l1(11,9)=  1.8840356171e-01
       coeff_l1(11,10)= -1.6329881549e-01
       coeff_l1(11,11)=  4.5814029872e-02
       coeff_l1(11,12)=  3.4044167399e-01
       coeff_l1(11,13)=  2.0913107693e-01
       coeff_l1(11,14)= -5.1300853491e-01
       coeff_l1(11,15)=  2.8948503733e-01
       coeff_l1(11,16)= -3.9730291814e-02
       coeff_l1(11,17)=  1.6467188597e+00
       coeff_l1(11,18)= -2.7163088322e-01
       coeff_l1(11,19)=  4.0037676692e-02
       coeff_l1(11,20)=  1.3814426027e-02
       coeff_l1(11,21)= -1.4200598001e-01
       coeff_l1(11,22)= -6.1389803886e-01
       coeff_l1(11,23)= -1.2421385944e-01
       coeff_l1(11,24)=  1.5964107513e+00
       coeff_l1(11,25)= -9.2386245728e-02
       coeff_l1(11,26)= -1.9634951651e-01
       coeff_l1(11,27)=  3.5466754436e-01
       coeff_l1(11,28)=  5.4758512974e-01
       coeff_l1(11,29)=  8.6077684537e-03
       coeff_l1(11,30)= -1.0575425625e-01
       coeff_l1(11,31)=  1.4473336935e-01
       coeff_l1(11,32)=  4.0344756097e-02
       coeff_l1(11,33)= -9.8477549851e-02
       coeff_l1(11,34)= -1.3785088062e-01
       coeff_l1(11,35)=  1.0593102872e-01
       coeff_l1(11,36)= -2.5265306234e-01
       coeff_l1(11,37)=  2.4328109622e-01
       coeff_l1(11,38)= -1.2645724416e-01
       coeff_l1(11,39)= -1.8514193594e-01
       coeff_l1(11,40)= -2.6806157827e-01
       coeff_l1(11,41)= -4.3076202273e-02
       coeff_l1(11,42)= -7.6982885599e-01
       coeff_l1(11,43)= -3.6143064499e-01
       coeff_l1(11,44)=  2.6191097498e-01
       coeff_l1(11,45)=  1.1354396343e+00
       coeff_l1(11,46)=  7.0100732148e-02
       coeff_l1(11,47)= -4.0658104420e-01
       coeff_l1(11,48)=  4.7202657908e-02
       coeff_l1(11,49)=  2.6032668538e-03
       coeff_l1(11,50)= -5.4433983564e-01
       coeff_l1(11,51)=  7.7681867406e-03
       coeff_l1(11,52)=  4.9189671874e-01
       coeff_l1(11,53)=  5.2890229225e-01
       coeff_l1(11,54)= -1.1635141820e-01
       coeff_l1(11,55)= -5.9845745564e-01
       coeff_l1(11,56)= -2.1414262056e-01
       coeff_l1(11,57)=  5.9996551275e-01
       coeff_l1(11,58)= -1.6172039509e-01
       coeff_l1(11,59)= -4.9846655130e-01
       coeff_l1(11,60)= -7.8219902515e-01
       coeff_l1(11,61)=  1.1983248591e-01
       coeff_l1(11,62)= -1.3494634628e-01
       coeff_l1(11,63)= -6.5803104639e-01
       coeff_l1(11,64)=  7.2395521402e-01
       coeff_l1(11,65)= -3.4267086536e-02
       coeff_l1(12,1)= -1.2774233520e-01
       coeff_l1(12,2)=  1.5904942155e-01
       coeff_l1(12,3)=  3.9858588576e-01
       coeff_l1(12,4)=  5.3896043450e-02
       coeff_l1(12,5)=  9.1727627441e-03
       coeff_l1(12,6)= -2.7690485120e-02
       coeff_l1(12,7)= -9.4265230000e-02
       coeff_l1(12,8)= -1.2942291796e-01
       coeff_l1(12,9)= -1.8092307448e-01
       coeff_l1(12,10)=  2.2505035996e-01
       coeff_l1(12,11)= -7.8441113234e-02
       coeff_l1(12,12)= -1.2145397067e-01
       coeff_l1(12,13)= -1.0440314561e-01
       coeff_l1(12,14)=  4.7679305077e-02
       coeff_l1(12,15)=  7.7092394233e-02
       coeff_l1(12,16)= -4.3970489502e-01
       coeff_l1(12,17)=  3.0697363615e-01
       coeff_l1(12,18)= -4.8820030689e-01
       coeff_l1(12,19)= -1.2926189601e-01
       coeff_l1(12,20)=  4.2827244848e-02
       coeff_l1(12,21)=  1.8463127315e-01
       coeff_l1(12,22)= -6.6222228110e-02
       coeff_l1(12,23)=  2.7993724216e-03
       coeff_l1(12,24)=  1.0458301008e-01
       coeff_l1(12,25)= -1.2212879211e-01
       coeff_l1(12,26)=  3.4908302128e-02
       coeff_l1(12,27)=  1.6246847808e-01
       coeff_l1(12,28)=  1.2161830068e-01
       coeff_l1(12,29)=  2.4520224333e-01
       coeff_l1(12,30)= -3.2600776758e-03
       coeff_l1(12,31)=  3.4446856380e-01
       coeff_l1(12,32)=  5.2761655301e-02
       coeff_l1(12,33)= -3.1244486570e-02
       coeff_l1(12,34)= -2.3119676113e-01
       coeff_l1(12,35)= -1.3743475080e-01
       coeff_l1(12,36)=  3.1993597746e-02
       coeff_l1(12,37)= -1.2080132216e-01
       coeff_l1(12,38)= -2.6589256525e-01
       coeff_l1(12,39)= -1.1904919147e-01
       coeff_l1(12,40)= -1.3302405179e-01
       coeff_l1(12,41)= -1.7294883728e-01
       coeff_l1(12,42)= -8.1272971630e-01
       coeff_l1(12,43)= -7.8029029071e-02
       coeff_l1(12,44)= -3.2845377922e-01
       coeff_l1(12,45)=  2.9619324207e-01
       coeff_l1(12,46)=  1.6408352554e-01
       coeff_l1(12,47)=  7.0900291204e-01
       coeff_l1(12,48)= -2.1795913577e-02
       coeff_l1(12,49)= -1.5506742895e-01
       coeff_l1(12,50)= -2.7350805700e-02
       coeff_l1(12,51)= -2.8007388115e-01
       coeff_l1(12,52)= -1.7260696739e-02
       coeff_l1(12,53)= -2.6739180088e-01
       coeff_l1(12,54)= -1.4143331349e-01
       coeff_l1(12,55)= -4.9465939403e-01
       coeff_l1(12,56)= -2.2903420031e-01
       coeff_l1(12,57)= -3.2807114720e-01
       coeff_l1(12,58)= -2.9258176684e-02
       coeff_l1(12,59)=  1.4380727708e-01
       coeff_l1(12,60)= -2.5503072143e-01
       coeff_l1(12,61)= -1.9837805629e-01
       coeff_l1(12,62)= -8.2780838013e-02
       coeff_l1(12,63)= -3.9823585749e-01
       coeff_l1(12,64)=  1.3299860060e-01
       coeff_l1(12,65)= -3.8703900645e-04
       coeff_l1(13,1)= -3.4938403964e-01
       coeff_l1(13,2)=  9.2713264748e-03
       coeff_l1(13,3)=  5.7146281004e-01
       coeff_l1(13,4)=  5.9738993645e-01
       coeff_l1(13,5)= -9.0652644634e-01
       coeff_l1(13,6)=  5.1609897614e-01
       coeff_l1(13,7)= -4.9365559220e-01
       coeff_l1(13,8)=  5.2867162228e-01
       coeff_l1(13,9)= -5.2235513926e-01
       coeff_l1(13,10)=  1.2340801954e-01
       coeff_l1(13,11)= -9.6642576158e-02
       coeff_l1(13,12)= -1.5383102000e-01
       coeff_l1(13,13)= -4.9547997117e-01
       coeff_l1(13,14)= -5.6318651885e-02
       coeff_l1(13,15)=  1.5488921106e-01
       coeff_l1(13,16)= -2.4734957218e+00
       coeff_l1(13,17)=  5.2409261465e-01
       coeff_l1(13,18)= -5.1494097710e-01
       coeff_l1(13,19)= -4.4589084387e-01
       coeff_l1(13,20)= -1.7401438951e+00
       coeff_l1(13,21)= -6.4947921038e-01
       coeff_l1(13,22)= -7.3637956381e-01
       coeff_l1(13,23)=  2.2880485654e-01
       coeff_l1(13,24)=  3.7294512987e-01
       coeff_l1(13,25)= -7.5901269913e-01
       coeff_l1(13,26)= -1.0793484449e+00
       coeff_l1(13,27)= -2.4786160886e-01
       coeff_l1(13,28)= -6.9742566347e-01
       coeff_l1(13,29)= -8.0050379038e-01
       coeff_l1(13,30)= -2.2382643819e-01
       coeff_l1(13,31)= -4.2480391264e-01
       coeff_l1(13,32)=  1.0265778750e-01
       coeff_l1(13,33)= -3.6512374878e-01
       coeff_l1(13,34)= -7.7429294586e-01
       coeff_l1(13,35)= -2.2484771907e-01
       coeff_l1(13,36)= -5.8192569762e-02
       coeff_l1(13,37)= -1.2094578743e+00
       coeff_l1(13,38)= -2.8483018279e-01
       coeff_l1(13,39)=  1.3904927671e-01
       coeff_l1(13,40)= -7.6125644147e-02
       coeff_l1(13,41)= -1.3293463737e-02
       coeff_l1(13,42)= -5.3543228656e-02
       coeff_l1(13,43)= -8.1295073032e-02
       coeff_l1(13,44)= -1.4232413769e+00
       coeff_l1(13,45)= -1.5616968274e-01
       coeff_l1(13,46)=  3.9814844728e-01
       coeff_l1(13,47)=  3.0056360364e-01
       coeff_l1(13,48)=  1.1298101395e-01
       coeff_l1(13,49)= -1.1851375103e+00
       coeff_l1(13,50)= -1.6288627684e-01
       coeff_l1(13,51)= -7.2246477008e-02
       coeff_l1(13,52)= -5.1360058784e-01
       coeff_l1(13,53)= -1.8317426443e+00
       coeff_l1(13,54)=  3.3677554131e-01
       coeff_l1(13,55)= -2.2147064209e+00
       coeff_l1(13,56)=  5.6069904566e-01
       coeff_l1(13,57)=  2.1063358784e+00
       coeff_l1(13,58)=  5.1317220926e-01
       coeff_l1(13,59)= -1.3430267572e+00
       coeff_l1(13,60)=  9.3529206514e-01
       coeff_l1(13,61)=  7.0476388931e-01
       coeff_l1(13,62)= -1.8484009802e-01
       coeff_l1(13,63)=  2.6878681779e-01
       coeff_l1(13,64)=  2.2590965033e-02
       coeff_l1(13,65)=  7.7603882551e-01
       coeff_l1(14,1)=  4.1158863902e-01
       coeff_l1(14,2)=  2.4115747213e-01
       coeff_l1(14,3)=  1.5524345636e-01
       coeff_l1(14,4)=  1.0670999289e+00
       coeff_l1(14,5)= -1.4964605868e-01
       coeff_l1(14,6)=  1.5393107897e-03
       coeff_l1(14,7)= -1.6991354525e-01
       coeff_l1(14,8)= -3.2856068015e-01
       coeff_l1(14,9)= -1.7413899302e-01
       coeff_l1(14,10)=  1.8374255300e-01
       coeff_l1(14,11)=  1.5566691756e-02
       coeff_l1(14,12)=  3.7242242694e-01
       coeff_l1(14,13)=  3.2301944494e-01
       coeff_l1(14,14)=  3.6876641214e-02
       coeff_l1(14,15)=  6.2922155485e-03
       coeff_l1(14,16)=  1.6694905758e+00
       coeff_l1(14,17)=  1.6916573048e+00
       coeff_l1(14,18)= -8.2573063672e-02
       coeff_l1(14,19)= -4.2415180802e-01
       coeff_l1(14,20)=  1.0789080858e+00
       coeff_l1(14,21)= -4.1428574920e-01
       coeff_l1(14,22)= -4.8017579317e-01
       coeff_l1(14,23)= -8.9420080185e-01
       coeff_l1(14,24)=  1.2216742039e+00
       coeff_l1(14,25)= -1.6735230386e-01
       coeff_l1(14,26)= -7.3277786374e-02
       coeff_l1(14,27)=  5.6880486012e-01
       coeff_l1(14,28)=  1.1016467214e-01
       coeff_l1(14,29)= -5.4005149752e-02
       coeff_l1(14,30)=  1.0074285418e-01
       coeff_l1(14,31)=  7.1600526571e-01
       coeff_l1(14,32)= -1.8545268476e-01
       coeff_l1(14,33)= -2.1723049879e-01
       coeff_l1(14,34)= -4.3606707454e-01
       coeff_l1(14,35)= -8.1657069921e-01
       coeff_l1(14,36)=  3.9745900035e-01
       coeff_l1(14,37)= -4.1600248218e-01
       coeff_l1(14,38)= -4.3700751662e-01
       coeff_l1(14,39)= -3.1505275518e-02
       coeff_l1(14,40)= -4.6999085695e-02
       coeff_l1(14,41)= -1.3599021733e-01
       coeff_l1(14,42)=  1.2073919773e+00
       coeff_l1(14,43)= -1.8283039331e+00
       coeff_l1(14,44)=  2.7909842134e-01
       coeff_l1(14,45)= -1.1249455214e+00
       coeff_l1(14,46)= -2.5641795993e-01
       coeff_l1(14,47)= -2.4731888771e+00
       coeff_l1(14,48)= -2.1937757730e-01
       coeff_l1(14,49)= -2.2021178901e-01
       coeff_l1(14,50)= -2.2258648276e-01
       coeff_l1(14,51)=  8.1771589816e-02
       coeff_l1(14,52)=  5.1310890913e-01
       coeff_l1(14,53)= -3.8136091828e-01
       coeff_l1(14,54)=  1.2889489532e-01
       coeff_l1(14,55)= -6.8860048056e-01
       coeff_l1(14,56)= -8.6588788033e-01
       coeff_l1(14,57)=  7.2043165565e-02
       coeff_l1(14,58)= -9.3637877703e-01
       coeff_l1(14,59)= -1.2904887199e+00
       coeff_l1(14,60)= -5.5510699749e-01
       coeff_l1(14,61)= -2.4451462924e-01
       coeff_l1(14,62)= -1.2417072058e-01
       coeff_l1(14,63)= -1.3525468297e-02
       coeff_l1(14,64)= -1.0736489296e+00
       coeff_l1(14,65)= -2.0379756391e-01
       coeff_l1(15,1)=  7.0040993392e-02
       coeff_l1(15,2)= -1.3238237798e-01
       coeff_l1(15,3)=  6.1830598116e-01
       coeff_l1(15,4)= -6.6143244505e-02
       coeff_l1(15,5)= -1.2313177437e-01
       coeff_l1(15,6)= -3.8367468119e-01
       coeff_l1(15,7)=  1.6318900883e-01
       coeff_l1(15,8)=  2.3781622946e-01
       coeff_l1(15,9)= -1.0025464743e-01
       coeff_l1(15,10)= -1.7898860574e-01
       coeff_l1(15,11)=  1.4278835617e-02
       coeff_l1(15,12)= -2.3456247151e-01
       coeff_l1(15,13)= -3.1181097031e-01
       coeff_l1(15,14)=  9.4882875681e-02
       coeff_l1(15,15)= -8.2241423428e-02
       coeff_l1(15,16)=  4.0894761682e-02
       coeff_l1(15,17)=  3.1316500902e-01
       coeff_l1(15,18)= -2.6431581378e-01
       coeff_l1(15,19)= -4.4516792893e-01
       coeff_l1(15,20)= -9.4357684255e-02
       coeff_l1(15,21)=  3.4346458316e-01
       coeff_l1(15,22)= -1.5133567154e-01
       coeff_l1(15,23)=  2.0935793221e-01
       coeff_l1(15,24)=  3.0251735449e-01
       coeff_l1(15,25)=  1.4280499518e-01
       coeff_l1(15,26)= -4.8300021887e-01
       coeff_l1(15,27)= -3.2513746619e-01
       coeff_l1(15,28)=  3.9397627115e-01
       coeff_l1(15,29)= -1.6130553558e-02
       coeff_l1(15,30)=  3.3134263754e-01
       coeff_l1(15,31)=  3.0896291137e-01
       coeff_l1(15,32)= -1.3707949221e-01
       coeff_l1(15,33)= -3.7244656682e-01
       coeff_l1(15,34)= -5.3639596701e-01
       coeff_l1(15,35)= -2.5434616208e-01
       coeff_l1(15,36)= -1.1854190379e-01
       coeff_l1(15,37)= -1.1174973845e-01
       coeff_l1(15,38)= -1.0582990944e-01
       coeff_l1(15,39)=  1.1527519673e-01
       coeff_l1(15,40)= -3.0691405758e-02
       coeff_l1(15,41)= -2.5994924363e-03
       coeff_l1(15,42)= -2.3021881282e-01
       coeff_l1(15,43)=  9.2888124287e-02
       coeff_l1(15,44)= -7.7681206167e-02
       coeff_l1(15,45)=  2.3228089511e-01
       coeff_l1(15,46)=  7.5835339725e-02
       coeff_l1(15,47)=  4.6080619097e-01
       coeff_l1(15,48)= -1.4534315094e-02
       coeff_l1(15,49)= -1.4802595973e-01
       coeff_l1(15,50)= -1.2605561316e-01
       coeff_l1(15,51)= -1.8781277537e-01
       coeff_l1(15,52)= -1.0753163695e-01
       coeff_l1(15,53)= -3.2575020194e-01
       coeff_l1(15,54)= -6.0915119946e-02
       coeff_l1(15,55)= -2.3897631466e-01
       coeff_l1(15,56)=  9.1317789629e-03
       coeff_l1(15,57)= -2.0769809186e-01
       coeff_l1(15,58)= -3.5211332142e-02
       coeff_l1(15,59)= -6.8817056715e-02
       coeff_l1(15,60)=  6.5142947435e-01
       coeff_l1(15,61)= -3.7404149771e-01
       coeff_l1(15,62)= -1.6854645312e-01
       coeff_l1(15,63)=  2.7343666553e-01
       coeff_l1(15,64)=  7.2749041021e-02
       coeff_l1(15,65)= -2.1288831532e-01
       coeff_l1(16,1)=  3.0907776952e-01
       coeff_l1(16,2)=  1.4397370815e-01
       coeff_l1(16,3)= -1.2817896605e+00
       coeff_l1(16,4)=  3.3961150050e-01
       coeff_l1(16,5)=  5.3623970598e-02
       coeff_l1(16,6)=  9.1125917435e-01
       coeff_l1(16,7)= -3.1857022643e-01
       coeff_l1(16,8)= -2.3954533041e-01
       coeff_l1(16,9)= -2.5230354071e-01
       coeff_l1(16,10)=  4.1137018800e-01
       coeff_l1(16,11)=  2.9821869731e-01
       coeff_l1(16,12)=  1.9275178015e-01
       coeff_l1(16,13)=  9.5592193305e-02
       coeff_l1(16,14)= -7.3914863169e-02
       coeff_l1(16,15)=  7.2112977505e-02
       coeff_l1(16,16)= -3.2100606710e-02
       coeff_l1(16,17)=  4.7450393438e-01
       coeff_l1(16,18)= -5.1948142052e-01
       coeff_l1(16,19)=  2.1899168193e-01
       coeff_l1(16,20)= -2.1597279608e-01
       coeff_l1(16,21)= -1.5513795614e-01
       coeff_l1(16,22)= -1.5116331577e+00
       coeff_l1(16,23)= -5.2076017857e-01
       coeff_l1(16,24)=  7.5826728344e-01
       coeff_l1(16,25)= -1.5608763695e-01
       coeff_l1(16,26)= -3.4944203496e-01
       coeff_l1(16,27)=  1.5655932426e+00
       coeff_l1(16,28)= -8.0776335672e-03
       coeff_l1(16,29)=  4.1967475414e-01
       coeff_l1(16,30)= -3.7791880965e-01
       coeff_l1(16,31)=  6.0390673578e-02
       coeff_l1(16,32)=  5.9651136398e-01
       coeff_l1(16,33)=  1.9607558250e+00
       coeff_l1(16,34)= -6.6846239567e-01
       coeff_l1(16,35)= -3.3094203472e-01
       coeff_l1(16,36)= -3.7100095302e-02
       coeff_l1(16,37)= -1.0470650196e+00
       coeff_l1(16,38)=  3.1494376063e-01
       coeff_l1(16,39)= -5.4215032607e-02
       coeff_l1(16,40)= -7.0055431128e-01
       coeff_l1(16,41)=  1.5596984327e-01
       coeff_l1(16,42)= -5.4189104587e-02
       coeff_l1(16,43)= -1.0283074379e+00
       coeff_l1(16,44)=  1.8104153872e+00
       coeff_l1(16,45)= -1.9377921820e+00
       coeff_l1(16,46)= -6.1608028412e-01
       coeff_l1(16,47)= -2.1775131226e+00
       coeff_l1(16,48)=  3.2734769583e-01
       coeff_l1(16,49)=  2.4584361911e-01
       coeff_l1(16,50)= -6.6754817963e-01
       coeff_l1(16,51)=  2.1834915504e-02
       coeff_l1(16,52)= -1.2602264062e-02
       coeff_l1(16,53)= -8.4562683105e-01
       coeff_l1(16,54)=  2.5270879269e-01
       coeff_l1(16,55)= -5.9022642672e-03
       coeff_l1(16,56)= -8.5790693760e-02
       coeff_l1(16,57)=  1.4955561161e+00
       coeff_l1(16,58)= -3.6363941431e-01
       coeff_l1(16,59)= -1.4771556854e+00
       coeff_l1(16,60)= -1.7291663215e-02
       coeff_l1(16,61)= -1.1954309940e+00
       coeff_l1(16,62)=  3.6394484341e-02
       coeff_l1(16,63)=  7.6837342978e-01
       coeff_l1(16,64)= -1.1323962212e+00
       coeff_l1(16,65)=  3.9832368493e-01
       coeff_l1(17,1)= -2.9012754560e-01
       coeff_l1(17,2)=  2.3848486599e-03
       coeff_l1(17,3)=  1.1176000834e+00
       coeff_l1(17,4)=  1.2970601320e+00
       coeff_l1(17,5)= -2.8799790144e-01
       coeff_l1(17,6)=  1.2753075361e+00
       coeff_l1(17,7)=  3.4588611126e-01
       coeff_l1(17,8)= -5.7362443209e-01
       coeff_l1(17,9)=  1.3851647079e-01
       coeff_l1(17,10)=  8.7281718850e-02
       coeff_l1(17,11)= -1.0812323540e-01
       coeff_l1(17,12)=  3.8640818000e-01
       coeff_l1(17,13)= -7.9336892813e-03
       coeff_l1(17,14)= -1.0513227433e-01
       coeff_l1(17,15)=  1.9702050090e-01
       coeff_l1(17,16)= -1.0178968906e+00
       coeff_l1(17,17)= -4.1928678751e-01
       coeff_l1(17,18)= -5.0906175375e-01
       coeff_l1(17,19)=  3.3390302211e-02
       coeff_l1(17,20)= -1.5429106951e+00
       coeff_l1(17,21)= -6.4023560286e-01
       coeff_l1(17,22)= -8.2405465841e-01
       coeff_l1(17,23)=  7.6114118099e-01
       coeff_l1(17,24)= -1.1688285321e-01
       coeff_l1(17,25)= -2.6916247606e-01
       coeff_l1(17,26)= -5.0215458870e-01
       coeff_l1(17,27)= -1.8785400689e-01
       coeff_l1(17,28)=  5.2051651478e-01
       coeff_l1(17,29)= -1.2316531688e-01
       coeff_l1(17,30)= -4.2952454090e-01
       coeff_l1(17,31)= -4.1156172752e-02
       coeff_l1(17,32)= -4.8273047805e-01
       coeff_l1(17,33)=  1.3468256593e-01
       coeff_l1(17,34)=  6.6721461713e-02
       coeff_l1(17,35)=  1.3739107549e-01
       coeff_l1(17,36)= -8.2070671022e-02
       coeff_l1(17,37)= -9.0723022819e-02
       coeff_l1(17,38)= -1.1399035454e+00
       coeff_l1(17,39)= -4.1116485000e-01
       coeff_l1(17,40)=  7.1946203709e-01
       coeff_l1(17,41)= -2.5626835227e-01
       coeff_l1(17,42)=  1.4009769261e-01
       coeff_l1(17,43)= -3.5918137431e-01
       coeff_l1(17,44)=  5.8983689547e-01
       coeff_l1(17,45)= -4.6528825164e-01
       coeff_l1(17,46)= -1.7579694092e-01
       coeff_l1(17,47)=  1.0129857063e+00
       coeff_l1(17,48)= -2.2720509768e-01
       coeff_l1(17,49)= -1.9312259555e-01
       coeff_l1(17,50)= -7.0097237825e-01
       coeff_l1(17,51)= -2.3692405224e-01
       coeff_l1(17,52)= -8.8502511382e-02
       coeff_l1(17,53)=  8.0490365624e-02
       coeff_l1(17,54)= -2.6271698475e+00
       coeff_l1(17,55)= -2.8664892912e-01
       coeff_l1(17,56)= -1.0341902971e+00
       coeff_l1(17,57)=  1.1668138504e+00
       coeff_l1(17,58)= -5.6108647585e-01
       coeff_l1(17,59)=  5.4306104779e-02
       coeff_l1(17,60)=  1.0738652945e+00
       coeff_l1(17,61)= -1.6887135804e-01
       coeff_l1(17,62)= -1.5260602534e-01
       coeff_l1(17,63)= -7.5610303879e-01
       coeff_l1(17,64)=  9.4427508116e-01
       coeff_l1(17,65)=  9.6983462572e-02
       coeff_l1(18,1)=  1.2795141339e-01
       coeff_l1(18,2)= -1.7950911820e-01
       coeff_l1(18,3)=  1.2777919769e+00
       coeff_l1(18,4)=  8.8200223446e-01
       coeff_l1(18,5)= -3.0741694570e-01
       coeff_l1(18,6)= -2.4144387245e-01
       coeff_l1(18,7)=  1.6657382250e-01
       coeff_l1(18,8)=  1.0216727108e-01
       coeff_l1(18,9)=  3.3712542057e-01
       coeff_l1(18,10)= -6.9612964988e-02
       coeff_l1(18,11)= -1.6167117655e-01
       coeff_l1(18,12)=  1.9226005673e-01
       coeff_l1(18,13)= -1.5042771399e-01
       coeff_l1(18,14)=  7.0528931916e-02
       coeff_l1(18,15)= -5.3145829588e-03
       coeff_l1(18,16)=  4.9914103746e-01
       coeff_l1(18,17)=  2.1916592121e+00
       coeff_l1(18,18)= -5.3508692980e-01
       coeff_l1(18,19)= -2.6440052316e-02
       coeff_l1(18,20)=  1.8293105066e-02
       coeff_l1(18,21)=  1.2962858379e-01
       coeff_l1(18,22)= -5.0261634588e-01
       coeff_l1(18,23)=  3.0303162336e-01
       coeff_l1(18,24)=  1.4171049595e+00
       coeff_l1(18,25)= -2.7757105231e-01
       coeff_l1(18,26)=  1.1129565537e-01
       coeff_l1(18,27)=  1.9602799416e-01
       coeff_l1(18,28)=  4.5671552420e-01
       coeff_l1(18,29)= -1.4980070293e-01
       coeff_l1(18,30)=  3.0179375410e-01
       coeff_l1(18,31)=  7.6826162636e-02
       coeff_l1(18,32)= -4.3333122134e-01
       coeff_l1(18,33)= -3.5463517904e-01
       coeff_l1(18,34)= -5.0974470377e-01
       coeff_l1(18,35)= -2.2403001785e-01
       coeff_l1(18,36)= -1.7081787810e-02
       coeff_l1(18,37)= -5.0135141611e-01
       coeff_l1(18,38)= -3.9292126894e-01
       coeff_l1(18,39)= -2.6896199584e-01
       coeff_l1(18,40)= -2.8329500929e-02
       coeff_l1(18,41)= -2.0565055311e-01
       coeff_l1(18,42)= -1.2556859255e+00
       coeff_l1(18,43)=  1.0205209255e-01
       coeff_l1(18,44)=  3.4812241793e-01
       coeff_l1(18,45)=  7.3791694641e-01
       coeff_l1(18,46)= -1.5313524008e-01
       coeff_l1(18,47)=  1.4127363265e-01
       coeff_l1(18,48)= -2.9126134515e-01
       coeff_l1(18,49)= -2.4958478287e-02
       coeff_l1(18,50)= -3.8448554277e-01
       coeff_l1(18,51)= -1.0136280954e-01
       coeff_l1(18,52)= -2.9742341489e-02
       coeff_l1(18,53)=  1.8168075383e-01
       coeff_l1(18,54)=  2.4029308558e-01
       coeff_l1(18,55)= -4.3042775989e-01
       coeff_l1(18,56)=  4.3365536258e-03
       coeff_l1(18,57)= -7.6052933931e-01
       coeff_l1(18,58)= -3.0861326959e-03
       coeff_l1(18,59)= -1.1340440512e+00
       coeff_l1(18,60)=  4.2281997204e-01
       coeff_l1(18,61)= -2.9570856690e-01
       coeff_l1(18,62)= -4.4150665402e-01
       coeff_l1(18,63)= -3.0806010962e-01
       coeff_l1(18,64)=  4.3801474571e-01
       coeff_l1(18,65)= -5.6371420622e-01
       coeff_l1(19,1)= -4.5390400290e-01
       coeff_l1(19,2)=  1.8537810072e-02
       coeff_l1(19,3)=  3.2448083162e-01
       coeff_l1(19,4)=  9.6545797586e-01
       coeff_l1(19,5)= -3.9134398103e-01
       coeff_l1(19,6)=  1.6268357635e-01
       coeff_l1(19,7)=  9.3520423397e-03
       coeff_l1(19,8)= -2.0243270695e-01
       coeff_l1(19,9)=  1.4283140004e-01
       coeff_l1(19,10)= -2.3534126580e-01
       coeff_l1(19,11)=  2.6599633694e-01
       coeff_l1(19,12)= -3.1506085396e-01
       coeff_l1(19,13)= -8.7141431868e-03
       coeff_l1(19,14)= -4.1906946898e-01
       coeff_l1(19,15)= -1.8451121449e-01
       coeff_l1(19,16)=  2.0041658878e+00
       coeff_l1(19,17)=  1.4099185467e+00
       coeff_l1(19,18)= -8.1552013755e-02
       coeff_l1(19,19)= -1.7873670161e-01
       coeff_l1(19,20)=  2.8101286292e-01
       coeff_l1(19,21)= -3.6183753610e-01
       coeff_l1(19,22)= -8.2965672016e-01
       coeff_l1(19,23)= -4.6153765917e-01
       coeff_l1(19,24)=  6.9830930233e-01
       coeff_l1(19,25)= -4.6448301524e-02
       coeff_l1(19,26)=  3.2239064574e-01
       coeff_l1(19,27)=  3.6069187522e-01
       coeff_l1(19,28)=  5.5533576012e-01
       coeff_l1(19,29)= -1.9354441762e-01
       coeff_l1(19,30)=  8.6570441723e-02
       coeff_l1(19,31)=  2.7300226688e-01
       coeff_l1(19,32)= -7.8208321333e-01
       coeff_l1(19,33)= -6.5006852150e-01
       coeff_l1(19,34)= -3.4072729945e-01
       coeff_l1(19,35)= -2.3084178567e-01
       coeff_l1(19,36)= -1.1482357047e-02
       coeff_l1(19,37)=  4.3057155609e-01
       coeff_l1(19,38)= -9.3905878067e-01
       coeff_l1(19,39)= -5.8539289236e-01
       coeff_l1(19,40)=  2.0188194513e-01
       coeff_l1(19,41)= -4.5006874204e-01
       coeff_l1(19,42)=  8.2979094982e-01
       coeff_l1(19,43)= -1.0477119684e+00
       coeff_l1(19,44)=  1.4410173893e-01
       coeff_l1(19,45)= -6.6715025902e-01
       coeff_l1(19,46)= -4.9723342061e-01
       coeff_l1(19,47)= -1.4304578304e+00
       coeff_l1(19,48)= -1.7770008743e-01
       coeff_l1(19,49)=  1.0675244033e-01
       coeff_l1(19,50)= -5.7777076960e-01
       coeff_l1(19,51)=  1.2070412189e-01
       coeff_l1(19,52)=  4.1733920574e-01
       coeff_l1(19,53)= -6.0324198008e-01
       coeff_l1(19,54)= -1.6230575740e-01
       coeff_l1(19,55)=  3.8903602958e-01
       coeff_l1(19,56)= -2.1091029644e+00
       coeff_l1(19,57)= -8.7696027756e-01
       coeff_l1(19,58)= -8.0115634203e-01
       coeff_l1(19,59)= -1.4449291229e+00
       coeff_l1(19,60)=  1.8858635426e-01
       coeff_l1(19,61)= -1.4462368190e-01
       coeff_l1(19,62)= -3.0797153711e-01
       coeff_l1(19,63)=  4.9298369884e-01
       coeff_l1(19,64)= -6.5917384624e-01
       coeff_l1(19,65)= -7.9131501913e-01
       coeff_l1(20,1)= -2.5186526775e-01
       coeff_l1(20,2)= -2.3511599004e-01
       coeff_l1(20,3)= -3.4613680840e-01
       coeff_l1(20,4)= -1.1536163092e-01
       coeff_l1(20,5)= -3.8303248584e-02
       coeff_l1(20,6)=  1.1114536971e-01
       coeff_l1(20,7)= -1.4269252121e-01
       coeff_l1(20,8)= -1.3467884064e-01
       coeff_l1(20,9)=  2.0813245326e-02
       coeff_l1(20,10)=  1.3171783090e-01
       coeff_l1(20,11)= -1.7642059922e-01
       coeff_l1(20,12)=  1.7193350196e-01
       coeff_l1(20,13)=  2.0637477934e-01
       coeff_l1(20,14)= -2.6567262411e-01
       coeff_l1(20,15)=  1.6535897553e-01
       coeff_l1(20,16)= -2.5301617384e-01
       coeff_l1(20,17)= -1.6099813581e-01
       coeff_l1(20,18)= -3.0099657178e-01
       coeff_l1(20,19)=  1.4767092466e-01
       coeff_l1(20,20)=  1.9164359570e-01
       coeff_l1(20,21)= -2.1563793719e-01
       coeff_l1(20,22)= -2.5111016631e-01
       coeff_l1(20,23)=  1.7796633765e-02
       coeff_l1(20,24)= -2.5578308105e-01
       coeff_l1(20,25)=  1.6627717018e-01
       coeff_l1(20,26)=  2.0283451676e-01
       coeff_l1(20,27)=  2.4428157508e-01
       coeff_l1(20,28)=  1.0278176516e-01
       coeff_l1(20,29)= -1.1865965277e-01
       coeff_l1(20,30)= -3.3016398549e-01
       coeff_l1(20,31)=  1.0053106397e-01
       coeff_l1(20,32)= -1.3021907210e-01
       coeff_l1(20,33)=  1.6202685237e-01
       coeff_l1(20,34)= -2.1603757516e-02
       coeff_l1(20,35)= -2.1251261234e-01
       coeff_l1(20,36)=  1.2717235088e-01
       coeff_l1(20,37)= -8.4619410336e-02
       coeff_l1(20,38)= -4.4038739800e-01
       coeff_l1(20,39)= -1.9046217203e-01
       coeff_l1(20,40)= -6.6991001368e-02
       coeff_l1(20,41)= -1.1394506693e-01
       coeff_l1(20,42)= -6.4519725740e-02
       coeff_l1(20,43)= -4.0775406361e-01
       coeff_l1(20,44)=  1.4450824261e-01
       coeff_l1(20,45)=  1.4702817798e-01
       coeff_l1(20,46)=  2.0704936236e-02
       coeff_l1(20,47)= -2.6946619153e-01
       coeff_l1(20,48)= -1.8051028252e-02
       coeff_l1(20,49)= -1.2209106982e-01
       coeff_l1(20,50)= -1.1186476797e-01
       coeff_l1(20,51)=  7.3675073683e-02
       coeff_l1(20,52)= -3.4012666345e-01
       coeff_l1(20,53)=  2.2701688111e-01
       coeff_l1(20,54)= -1.6302527487e-01
       coeff_l1(20,55)=  2.9452401400e-01
       coeff_l1(20,56)= -3.6394873261e-01
       coeff_l1(20,57)=  1.4842756093e-01
       coeff_l1(20,58)=  8.7046138942e-02
       coeff_l1(20,59)=  1.8318861723e-01
       coeff_l1(20,60)= -1.8117536604e-01
       coeff_l1(20,61)= -2.1864005923e-01
       coeff_l1(20,62)= -2.5380203128e-01
       coeff_l1(20,63)= -4.8802426457e-01
       coeff_l1(20,64)= -6.3975408673e-02
       coeff_l1(20,65)=  1.2098913640e-01
       coeff_l1(21,1)=  2.6765382290e-01
       coeff_l1(21,2)= -6.6272713244e-02
       coeff_l1(21,3)=  9.3791079521e-01
       coeff_l1(21,4)=  5.0306355953e-01
       coeff_l1(21,5)=  1.8906097859e-02
       coeff_l1(21,6)= -1.5544229187e-02
       coeff_l1(21,7)= -2.5679525733e-01
       coeff_l1(21,8)=  2.5050764903e-02
       coeff_l1(21,9)=  3.4774345160e-01
       coeff_l1(21,10)=  1.0061258078e-01
       coeff_l1(21,11)= -2.4403296411e-01
       coeff_l1(21,12)=  3.7871631980e-01
       coeff_l1(21,13)= -8.0570742488e-02
       coeff_l1(21,14)=  1.5927638113e-01
       coeff_l1(21,15)=  2.7703893185e-01
       coeff_l1(21,16)=  7.6244884729e-01
       coeff_l1(21,17)=  2.9687855244e+00
       coeff_l1(21,18)= -2.1636974812e-01
       coeff_l1(21,19)= -1.5712314844e-01
       coeff_l1(21,20)=  2.3650234938e-01
       coeff_l1(21,21)=  1.2002114952e-01
       coeff_l1(21,22)= -8.3355194330e-01
       coeff_l1(21,23)= -2.5832095742e-01
       coeff_l1(21,24)=  2.3825163841e+00
       coeff_l1(21,25)=  7.7102310024e-03
       coeff_l1(21,26)= -2.2621744871e-01
       coeff_l1(21,27)=  7.9465824366e-01
       coeff_l1(21,28)=  5.9476023912e-01
       coeff_l1(21,29)= -2.9771497846e-01
       coeff_l1(21,30)=  2.2359673679e-01
       coeff_l1(21,31)=  1.7838837206e-01
       coeff_l1(21,32)= -7.3990769684e-02
       coeff_l1(21,33)=  3.0867063999e-01
       coeff_l1(21,34)=  5.7410944253e-02
       coeff_l1(21,35)=  4.1391170025e-01
       coeff_l1(21,36)= -1.1972887814e-01
       coeff_l1(21,37)= -3.8533862680e-02
       coeff_l1(21,38)=  1.1116827279e-01
       coeff_l1(21,39)= -1.1307378113e-01
       coeff_l1(21,40)= -4.9255678058e-01
       coeff_l1(21,41)= -2.4229192734e-01
       coeff_l1(21,42)= -4.9565136433e-01
       coeff_l1(21,43)= -8.0159038305e-01
       coeff_l1(21,44)=  2.2760947049e-01
       coeff_l1(21,45)=  1.8564870358e+00
       coeff_l1(21,46)=  1.3448928297e-01
       coeff_l1(21,47)= -7.7362287045e-01
       coeff_l1(21,48)=  4.4964369386e-02
       coeff_l1(21,49)=  1.3969794847e-02
       coeff_l1(21,50)= -3.6162880063e-01
       coeff_l1(21,51)= -2.7425411344e-01
       coeff_l1(21,52)=  4.7246950865e-01
       coeff_l1(21,53)= -4.6960943937e-01
       coeff_l1(21,54)=  4.9663594365e-01
       coeff_l1(21,55)= -2.3818725348e-01
       coeff_l1(21,56)=  1.2055385113e-01
       coeff_l1(21,57)=  5.0674248487e-02
       coeff_l1(21,58)= -1.7678704858e-01
       coeff_l1(21,59)= -9.1133809090e-01
       coeff_l1(21,60)= -1.0133801699e+00
       coeff_l1(21,61)= -2.0520624518e-01
       coeff_l1(21,62)=  1.4658086002e-01
       coeff_l1(21,63)= -3.8724002242e-01
       coeff_l1(21,64)=  4.5108765364e-01
       coeff_l1(21,65)=  2.3811650276e-01
       coeff_l1(22,1)= -3.7394806743e-02
       coeff_l1(22,2)= -2.5728136301e-01
       coeff_l1(22,3)=  8.4039288759e-01
       coeff_l1(22,4)=  8.1471413374e-01
       coeff_l1(22,5)= -4.2018018663e-02
       coeff_l1(22,6)= -1.1873916537e-01
       coeff_l1(22,7)= -8.3347320557e-02
       coeff_l1(22,8)= -4.2124658823e-01
       coeff_l1(22,9)=  1.8606789410e-01
       coeff_l1(22,10)= -4.7918338329e-02
       coeff_l1(22,11)=  5.8533523232e-02
       coeff_l1(22,12)=  1.4998883009e-01
       coeff_l1(22,13)= -2.9398745298e-01
       coeff_l1(22,14)= -2.5966399908e-01
       coeff_l1(22,15)=  2.0119626820e-01
       coeff_l1(22,16)=  6.8872809410e-01
       coeff_l1(22,17)=  2.5974507332e+00
       coeff_l1(22,18)= -5.0695669651e-01
       coeff_l1(22,19)= -1.9800129533e-01
       coeff_l1(22,20)= -1.7796401680e-01
       coeff_l1(22,21)=  6.3134849072e-02
       coeff_l1(22,22)= -4.5578277111e-01
       coeff_l1(22,23)= -1.6840758920e-01
       coeff_l1(22,24)=  2.5061209202e+00
       coeff_l1(22,25)= -2.7017894387e-01
       coeff_l1(22,26)= -2.3101928830e-01
       coeff_l1(22,27)=  6.9936227798e-01
       coeff_l1(22,28)=  2.9725056887e-01
       coeff_l1(22,29)=  6.4286574721e-02
       coeff_l1(22,30)= -1.2886093557e-01
       coeff_l1(22,31)= -9.1193385422e-02
       coeff_l1(22,32)= -2.8199446201e-01
       coeff_l1(22,33)=  2.8146857023e-01
       coeff_l1(22,34)= -3.0737665296e-01
       coeff_l1(22,35)=  1.8028779328e-01
       coeff_l1(22,36)=  1.6674049199e-01
       coeff_l1(22,37)= -2.5448697805e-01
       coeff_l1(22,38)=  1.4223136008e-01
       coeff_l1(22,39)= -2.6214447618e-01
       coeff_l1(22,40)= -6.6609823704e-01
       coeff_l1(22,41)= -1.3757824898e-01
       coeff_l1(22,42)= -1.0620034933e+00
       coeff_l1(22,43)= -7.6413744688e-01
       coeff_l1(22,44)=  4.2879590392e-01
       coeff_l1(22,45)=  1.0771394968e+00
       coeff_l1(22,46)= -2.7788782120e-01
       coeff_l1(22,47)= -8.1884807348e-01
       coeff_l1(22,48)=  4.6173918992e-02
       coeff_l1(22,49)=  2.6382723451e-01
       coeff_l1(22,50)= -3.0841287971e-01
       coeff_l1(22,51)= -1.1887615919e-01
       coeff_l1(22,52)=  2.2302573919e-01
       coeff_l1(22,53)=  4.2899813503e-02
       coeff_l1(22,54)=  2.9721674323e-01
       coeff_l1(22,55)= -1.3455551118e-02
       coeff_l1(22,56)=  1.6348619247e-03
       coeff_l1(22,57)= -3.4153270721e-01
       coeff_l1(22,58)=  9.3856751919e-02
       coeff_l1(22,59)= -6.0226339102e-01
       coeff_l1(22,60)= -6.0586756468e-01
       coeff_l1(22,61)= -7.6476559043e-02
       coeff_l1(22,62)= -1.3218040764e-01
       coeff_l1(22,63)= -6.8510764837e-01
       coeff_l1(22,64)=  4.4643577933e-01
       coeff_l1(22,65)=  2.3651908338e-01
       coeff_l1(23,1)= -1.3370629400e-02
       coeff_l1(23,2)=  9.8674476147e-02
       coeff_l1(23,3)=  3.1394794583e-02
       coeff_l1(23,4)= -2.0531608164e-01
       coeff_l1(23,5)= -1.1005978584e+00
       coeff_l1(23,6)= -2.0182723999e+00
       coeff_l1(23,7)= -2.8904050589e-01
       coeff_l1(23,8)= -1.2275027484e-01
       coeff_l1(23,9)= -1.3809987903e-01
       coeff_l1(23,10)= -6.4758175611e-01
       coeff_l1(23,11)= -2.8122574091e-01
       coeff_l1(23,12)= -3.1133726239e-01
       coeff_l1(23,13)= -4.7504320741e-02
       coeff_l1(23,14)= -3.6024708301e-02
       coeff_l1(23,15)=  2.4165727198e-01
       coeff_l1(23,16)= -1.9392189384e-01
       coeff_l1(23,17)=  4.8153120279e-01
       coeff_l1(23,18)= -2.2532941401e-01
       coeff_l1(23,19)= -1.4710873365e+00
       coeff_l1(23,20)= -2.6512783766e-01
       coeff_l1(23,21)= -8.0666583776e-01
       coeff_l1(23,22)=  4.2554977536e-01
       coeff_l1(23,23)=  3.1741979718e-01
       coeff_l1(23,24)=  8.4083068371e-01
       coeff_l1(23,25)= -2.4874326587e-01
       coeff_l1(23,26)= -9.8125672340e-01
       coeff_l1(23,27)= -3.4130293876e-02
       coeff_l1(23,28)= -5.5217862129e-01
       coeff_l1(23,29)= -1.0729448497e-01
       coeff_l1(23,30)= -4.8556271195e-01
       coeff_l1(23,31)=  2.0719642937e-01
       coeff_l1(23,32)=  4.8731032759e-02
       coeff_l1(23,33)= -4.5608511567e-01
       coeff_l1(23,34)= -3.0811280012e-01
       coeff_l1(23,35)=  6.3059985638e-01
       coeff_l1(23,36)= -3.1105092168e-01
       coeff_l1(23,37)= -7.7974247932e-01
       coeff_l1(23,38)= -6.9923174381e-01
       coeff_l1(23,39)= -3.1431296468e-01
       coeff_l1(23,40)= -2.6673523709e-02
       coeff_l1(23,41)=  4.4330380857e-02
       coeff_l1(23,42)=  1.8482527137e-01
       coeff_l1(23,43)=  8.6605280638e-01
       coeff_l1(23,44)= -1.5261169672e+00
       coeff_l1(23,45)=  1.6975588799e+00
       coeff_l1(23,46)=  1.0257466882e-01
       coeff_l1(23,47)=  6.4404660463e-01
       coeff_l1(23,48)= -1.5848031640e-01
       coeff_l1(23,49)= -8.7508702278e-01
       coeff_l1(23,50)= -2.1560576558e-01
       coeff_l1(23,51)= -3.5100886226e-01
       coeff_l1(23,52)=  1.1470547915e+00
       coeff_l1(23,53)=  1.0260918140e+00
       coeff_l1(23,54)=  8.8017451763e-01
       coeff_l1(23,55)=  1.7805318534e-01
       coeff_l1(23,56)= -2.5857535005e-01
       coeff_l1(23,57)= -1.3432285786e+00
       coeff_l1(23,58)= -2.3412178457e-01
       coeff_l1(23,59)=  2.0463019609e-02
       coeff_l1(23,60)= -1.3497122526e+00
       coeff_l1(23,61)= -1.1302873492e-01
       coeff_l1(23,62)= -1.8867251277e-01
       coeff_l1(23,63)= -3.3853328228e-01
       coeff_l1(23,64)=  1.2451684475e+00
       coeff_l1(23,65)= -7.5694924593e-01
       coeff_l1(24,1)=  3.8173758984e-01
       coeff_l1(24,2)= -2.1361703053e-02
       coeff_l1(24,3)= -1.4885687828e+00
       coeff_l1(24,4)=  4.6782273054e-01
       coeff_l1(24,5)= -3.2865989208e-01
       coeff_l1(24,6)= -1.3300314546e-01
       coeff_l1(24,7)= -4.7413909435e-01
       coeff_l1(24,8)= -1.7719329894e-01
       coeff_l1(24,9)= -3.2838279009e-01
       coeff_l1(24,10)= -2.6763364673e-01
       coeff_l1(24,11)= -3.0361397192e-02
       coeff_l1(24,12)=  1.4200614393e-01
       coeff_l1(24,13)= -3.7587031722e-01
       coeff_l1(24,14)= -4.3285399675e-02
       coeff_l1(24,15)= -7.2872680426e-01
       coeff_l1(24,16)=  1.0301594734e+00
       coeff_l1(24,17)=  1.8597496748e+00
       coeff_l1(24,18)=  1.8879495561e-02
       coeff_l1(24,19)= -5.9801715612e-01
       coeff_l1(24,20)=  1.4713872671e+00
       coeff_l1(24,21)= -4.4358029962e-01
       coeff_l1(24,22)=  2.7266878635e-02
       coeff_l1(24,23)= -7.5860053301e-01
       coeff_l1(24,24)=  1.7916779518e+00
       coeff_l1(24,25)= -4.5709988475e-01
       coeff_l1(24,26)= -6.9802010059e-01
       coeff_l1(24,27)=  5.2549993992e-01
       coeff_l1(24,28)=  1.1912630796e+00
       coeff_l1(24,29)= -3.0224001408e-01
       coeff_l1(24,30)=  3.0760881305e-01
       coeff_l1(24,31)=  1.0234495401e+00
       coeff_l1(24,32)= -3.0540159345e-01
       coeff_l1(24,33)=  4.6733629704e-01
       coeff_l1(24,34)= -3.2272452116e-01
       coeff_l1(24,35)= -5.5172008276e-01
       coeff_l1(24,36)= -3.2076783478e-02
       coeff_l1(24,37)= -1.0235527754e+00
       coeff_l1(24,38)= -1.5846374631e-01
       coeff_l1(24,39)= -3.7136971951e-01
       coeff_l1(24,40)= -9.2857295275e-01
       coeff_l1(24,41)= -2.5156849623e-01
       coeff_l1(24,42)=  2.0667509735e-01
       coeff_l1(24,43)= -1.2416455746e+00
       coeff_l1(24,44)= -2.4979254603e-01
       coeff_l1(24,45)= -1.0030470043e-01
       coeff_l1(24,46)= -6.2372982502e-01
       coeff_l1(24,47)= -1.5165429115e+00
       coeff_l1(24,48)=  4.1978418827e-02
       coeff_l1(24,49)= -4.5876795053e-01
       coeff_l1(24,50)= -1.2542143464e-01
       coeff_l1(24,51)= -2.0962563157e-01
       coeff_l1(24,52)=  5.7856786251e-01
       coeff_l1(24,53)= -2.3141014576e-01
       coeff_l1(24,54)=  6.1235976219e-01
       coeff_l1(24,55)=  8.8604480028e-01
       coeff_l1(24,56)= -4.6424853802e-01
       coeff_l1(24,57)= -1.4024035931e+00
       coeff_l1(24,58)= -5.1330071688e-01
       coeff_l1(24,59)= -2.5839978456e-01
       coeff_l1(24,60)= -1.3015387058e+00
       coeff_l1(24,61)= -9.0148633718e-01
       coeff_l1(24,62)= -4.2768847942e-01
       coeff_l1(24,63)=  4.5016819239e-01
       coeff_l1(24,64)= -1.0607297421e+00
       coeff_l1(24,65)= -6.5236204863e-01
       coeff_l1(25,1)=  5.9638634324e-02
       coeff_l1(25,2)= -2.3956412077e-01
       coeff_l1(25,3)=  1.4034270048e+00
       coeff_l1(25,4)=  3.0785101652e-01
       coeff_l1(25,5)=  4.5658610761e-02
       coeff_l1(25,6)= -1.3950490393e-02
       coeff_l1(25,7)=  1.2390308827e-01
       coeff_l1(25,8)= -3.2437178493e-01
       coeff_l1(25,9)= -1.2652242184e-01
       coeff_l1(25,10)= -9.0517252684e-03
       coeff_l1(25,11)= -2.0287485421e-01
       coeff_l1(25,12)=  3.3868396282e-01
       coeff_l1(25,13)= -3.0207078904e-02
       coeff_l1(25,14)= -9.2305410653e-03
       coeff_l1(25,15)=  2.9369147494e-02
       coeff_l1(25,16)=  3.8027337193e-01
       coeff_l1(25,17)=  2.1516332626e+00
       coeff_l1(25,18)= -2.6398712397e-01
       coeff_l1(25,19)= -3.0052825809e-01
       coeff_l1(25,20)= -4.8295515776e-01
       coeff_l1(25,21)=  1.5182617307e-01
       coeff_l1(25,22)=  2.7782611549e-02
       coeff_l1(25,23)=  2.7983456850e-01
       coeff_l1(25,24)=  2.1394965649e+00
       coeff_l1(25,25)= -3.2417437434e-01
       coeff_l1(25,26)=  1.3030716777e-01
       coeff_l1(25,27)=  5.7344067097e-01
       coeff_l1(25,28)=  2.4030590057e-01
       coeff_l1(25,29)=  2.0828156173e-01
       coeff_l1(25,30)=  5.5587481707e-02
       coeff_l1(25,31)= -2.5004148483e-01
       coeff_l1(25,32)= -1.2020986527e-01
       coeff_l1(25,33)=  9.5746792853e-02
       coeff_l1(25,34)= -1.9893176854e-01
       coeff_l1(25,35)=  4.6628374606e-02
       coeff_l1(25,36)= -6.3642054796e-02
       coeff_l1(25,37)= -3.3442497253e-01
       coeff_l1(25,38)=  2.4022324383e-01
       coeff_l1(25,39)= -7.3376201093e-02
       coeff_l1(25,40)= -2.6164129376e-01
       coeff_l1(25,41)= -9.5393851399e-02
       coeff_l1(25,42)= -8.8555502892e-01
       coeff_l1(25,43)=  3.8576057553e-01
       coeff_l1(25,44)=  5.8774459362e-01
       coeff_l1(25,45)=  2.0467436314e+00
       coeff_l1(25,46)= -2.0491784811e-01
       coeff_l1(25,47)=  9.5305934548e-02
       coeff_l1(25,48)=  2.3691567779e-01
       coeff_l1(25,49)= -6.1101190746e-02
       coeff_l1(25,50)= -2.9685330391e-01
       coeff_l1(25,51)= -2.5188192725e-01
       coeff_l1(25,52)=  4.0616399050e-01
       coeff_l1(25,53)=  2.6058846712e-01
       coeff_l1(25,54)=  8.1262183189e-01
       coeff_l1(25,55)= -1.0456950665e+00
       coeff_l1(25,56)=  8.1935532391e-02
       coeff_l1(25,57)= -3.4229823947e-01
       coeff_l1(25,58)=  3.0637320876e-01
       coeff_l1(25,59)= -7.0372176170e-01
       coeff_l1(25,60)= -1.0092333555e+00
       coeff_l1(25,61)= -2.5443881750e-01
       coeff_l1(25,62)= -2.1137895063e-02
       coeff_l1(25,63)= -2.0572185516e-02
       coeff_l1(25,64)=  1.3128693104e+00
       coeff_l1(25,65)=  1.5955969691e-01
       coeff_l1(26,1)= -5.1208627224e-01
       coeff_l1(26,2)=  5.3455601446e-03
       coeff_l1(26,3)=  5.5675983429e-01
       coeff_l1(26,4)= -1.4824648201e-01
       coeff_l1(26,5)= -2.8473737836e-01
       coeff_l1(26,6)= -7.7562816441e-02
       coeff_l1(26,7)= -1.2567187659e-02
       coeff_l1(26,8)=  1.3137124479e-01
       coeff_l1(26,9)=  4.7942854464e-02
       coeff_l1(26,10)=  1.7875117064e-01
       coeff_l1(26,11)=  2.9121452570e-01
       coeff_l1(26,12)= -1.7526023090e-01
       coeff_l1(26,13)= -1.9329555333e-01
       coeff_l1(26,14)= -7.5453490019e-02
       coeff_l1(26,15)=  1.4595168829e-01
       coeff_l1(26,16)= -4.4915586710e-02
       coeff_l1(26,17)=  1.6389778256e-01
       coeff_l1(26,18)= -1.4061149955e-01
       coeff_l1(26,19)= -4.9183037132e-02
       coeff_l1(26,20)= -1.6177351773e-01
       coeff_l1(26,21)=  1.4112967253e-01
       coeff_l1(26,22)= -1.6978172958e-01
       coeff_l1(26,23)=  6.6989041865e-02
       coeff_l1(26,24)= -9.7398698330e-02
       coeff_l1(26,25)=  1.2944927812e-01
       coeff_l1(26,26)= -3.6699220538e-01
       coeff_l1(26,27)=  9.6155807376e-02
       coeff_l1(26,28)= -9.2700816691e-02
       coeff_l1(26,29)=  4.7645896673e-02
       coeff_l1(26,30)=  2.5599421933e-02
       coeff_l1(26,31)=  2.7232609689e-02
       coeff_l1(26,32)= -1.0018420219e-01
       coeff_l1(26,33)= -3.6399949342e-02
       coeff_l1(26,34)=  1.3857345283e-01
       coeff_l1(26,35)= -1.1806542054e-02
       coeff_l1(26,36)= -2.1422141790e-01
       coeff_l1(26,37)= -2.9231327772e-01
       coeff_l1(26,38)= -1.9970527291e-01
       coeff_l1(26,39)=  5.6505486369e-02
       coeff_l1(26,40)=  2.5659292936e-01
       coeff_l1(26,41)= -2.2319696844e-01
       coeff_l1(26,42)=  5.0397992134e-01
       coeff_l1(26,43)= -2.4739213288e-01
       coeff_l1(26,44)=  9.8363339901e-02
       coeff_l1(26,45)=  1.9334429502e-01
       coeff_l1(26,46)= -1.2897436321e-01
       coeff_l1(26,47)=  3.4365873784e-02
       coeff_l1(26,48)=  1.3341850042e-01
       coeff_l1(26,49)= -8.3029337227e-02
       coeff_l1(26,50)= -6.7675285041e-02
       coeff_l1(26,51)=  1.4436802268e-01
       coeff_l1(26,52)=  1.4918851852e-01
       coeff_l1(26,53)= -2.0809084177e-01
       coeff_l1(26,54)= -3.4395191073e-01
       coeff_l1(26,55)=  2.2228289396e-02
       coeff_l1(26,56)=  4.3511494994e-02
       coeff_l1(26,57)= -2.9632264748e-02
       coeff_l1(26,58)= -9.3994416296e-02
       coeff_l1(26,59)= -1.9595609978e-02
       coeff_l1(26,60)= -6.8495427258e-03
       coeff_l1(26,61)= -1.5497685969e-01
       coeff_l1(26,62)=  1.7920240760e-02
       coeff_l1(26,63)=  5.8352228254e-02
       coeff_l1(26,64)=  2.4779498577e-01
       coeff_l1(26,65)=  2.7528232336e-01
       coeff_l1(27,1)=  1.4502181113e-01
       coeff_l1(27,2)=  6.8178169429e-02
       coeff_l1(27,3)=  1.4639604092e+00
       coeff_l1(27,4)=  7.4538105726e-01
       coeff_l1(27,5)=  1.4020236209e-02
       coeff_l1(27,6)=  2.1314473450e-01
       coeff_l1(27,7)= -2.3567891121e-01
       coeff_l1(27,8)=  2.3743735254e-01
       coeff_l1(27,9)=  2.6847007871e-01
       coeff_l1(27,10)=  2.6390111446e-01
       coeff_l1(27,11)=  1.2730014324e-01
       coeff_l1(27,12)= -8.6837038398e-02
       coeff_l1(27,13)=  5.9738796204e-02
       coeff_l1(27,14)= -2.9035544395e-01
       coeff_l1(27,15)=  1.5007907152e-01
       coeff_l1(27,16)=  1.4454655349e-01
       coeff_l1(27,17)=  1.3457800150e+00
       coeff_l1(27,18)= -8.1782466173e-01
       coeff_l1(27,19)= -3.1902253628e-02
       coeff_l1(27,20)= -5.3127038479e-01
       coeff_l1(27,21)=  1.9457019866e-01
       coeff_l1(27,22)= -6.0207056999e-01
       coeff_l1(27,23)= -1.4891500771e-01
       coeff_l1(27,24)=  1.2928237915e+00
       coeff_l1(27,25)=  4.0882688016e-02
       coeff_l1(27,26)= -8.1366822124e-02
       coeff_l1(27,27)=  6.0319381952e-01
       coeff_l1(27,28)= -1.0681557655e-01
       coeff_l1(27,29)=  2.1914401650e-01
       coeff_l1(27,30)=  1.1605453491e-01
       coeff_l1(27,31)=  9.8822973669e-02
       coeff_l1(27,32)=  3.7074583769e-01
       coeff_l1(27,33)=  1.4897577465e-01
       coeff_l1(27,34)= -2.0040328801e-01
       coeff_l1(27,35)=  1.3824506663e-02
       coeff_l1(27,36)=  1.9851912558e-01
       coeff_l1(27,37)= -2.7451640368e-01
       coeff_l1(27,38)= -1.9618399441e-01
       coeff_l1(27,39)= -3.2156714797e-01
       coeff_l1(27,40)= -1.1417443305e-01
       coeff_l1(27,41)= -3.2308447361e-01
       coeff_l1(27,42)= -1.6689282656e+00
       coeff_l1(27,43)= -1.9196788967e-01
       coeff_l1(27,44)=  3.5525748134e-01
       coeff_l1(27,45)=  9.3943250179e-01
       coeff_l1(27,46)= -1.2588796020e-01
       coeff_l1(27,47)= -2.7937775850e-01
       coeff_l1(27,48)= -1.6592693329e-01
       coeff_l1(27,49)=  1.5138901770e-01
       coeff_l1(27,50)= -3.5707896948e-01
       coeff_l1(27,51)=  7.9183682799e-02
       coeff_l1(27,52)= -2.4321135879e-01
       coeff_l1(27,53)= -4.1490423679e-01
       coeff_l1(27,54)=  5.1376527548e-01
       coeff_l1(27,55)= -8.5373371840e-01
       coeff_l1(27,56)=  1.1549216509e-01
       coeff_l1(27,57)=  3.1779557467e-01
       coeff_l1(27,58)=  1.5441256762e-01
       coeff_l1(27,59)= -7.6554250717e-01
       coeff_l1(27,60)=  1.2577663362e-01
       coeff_l1(27,61)= -2.7330943942e-01
       coeff_l1(27,62)=  3.5903416574e-02
       coeff_l1(27,63)= -4.6284500510e-02
       coeff_l1(27,64)=  5.9305363894e-01
       coeff_l1(27,65)=  3.4303313494e-01
       coeff_l1(28,1)=  6.7269355059e-02
       coeff_l1(28,2)=  1.4592462778e-01
       coeff_l1(28,3)=  7.2523951530e-01
       coeff_l1(28,4)=  8.8207566738e-01
       coeff_l1(28,5)= -5.1435595751e-01
       coeff_l1(28,6)= -2.8206598759e-01
       coeff_l1(28,7)=  1.3845439255e-01
       coeff_l1(28,8)= -8.8560320437e-02
       coeff_l1(28,9)= -4.2094850540e-01
       coeff_l1(28,10)= -2.7032724023e-01
       coeff_l1(28,11)=  2.2284117341e-01
       coeff_l1(28,12)= -4.9222871661e-01
       coeff_l1(28,13)=  2.5867930055e-01
       coeff_l1(28,14)= -5.2782975137e-02
       coeff_l1(28,15)=  2.5199100375e-01
       coeff_l1(28,16)=  1.4696675539e+00
       coeff_l1(28,17)=  2.0311323926e-02
       coeff_l1(28,18)=  3.3837190270e-01
       coeff_l1(28,19)= -3.2192361355e-01
       coeff_l1(28,20)=  1.4934084415e+00
       coeff_l1(28,21)= -2.7816203237e-01
       coeff_l1(28,22)= -2.9431587458e-01
       coeff_l1(28,23)= -1.3071469963e-01
       coeff_l1(28,24)=  2.0353065431e-01
       coeff_l1(28,25)= -1.6323997080e-01
       coeff_l1(28,26)=  2.2769805789e-01
       coeff_l1(28,27)= -5.5590426922e-01
       coeff_l1(28,28)= -4.9544891715e-01
       coeff_l1(28,29)= -5.3779113293e-01
       coeff_l1(28,30)= -6.1429345608e-01
       coeff_l1(28,31)=  1.9400840998e-01
       coeff_l1(28,32)= -6.6775506735e-01
       coeff_l1(28,33)= -8.4401798248e-01
       coeff_l1(28,34)= -4.6475149691e-02
       coeff_l1(28,35)= -2.2170212865e-01
       coeff_l1(28,36)= -1.2514025904e-02
       coeff_l1(28,37)= -1.3569746912e-01
       coeff_l1(28,38)=  1.2064377218e-01
       coeff_l1(28,39)= -1.2302817404e-01
       coeff_l1(28,40)=  7.0156067610e-01
       coeff_l1(28,41)= -9.5865890384e-02
       coeff_l1(28,42)=  5.9551441669e-01
       coeff_l1(28,43)= -3.6689225584e-02
       coeff_l1(28,44)= -5.5437356234e-01
       coeff_l1(28,45)= -9.1573297977e-01
       coeff_l1(28,46)= -1.1944048852e-01
       coeff_l1(28,47)=  3.7884403020e-02
       coeff_l1(28,48)=  2.9454258084e-01
       coeff_l1(28,49)= -2.7977302670e-01
       coeff_l1(28,50)=  8.6535625160e-02
       coeff_l1(28,51)=  1.6843259335e-01
       coeff_l1(28,52)= -4.0734976530e-01
       coeff_l1(28,53)=  4.3309870362e-01
       coeff_l1(28,54)=  1.8987625837e-01
       coeff_l1(28,55)= -3.2535338402e-01
       coeff_l1(28,56)= -1.0016659498e+00
       coeff_l1(28,57)= -6.9299304485e-01
       coeff_l1(28,58)= -7.2269135714e-01
       coeff_l1(28,59)= -8.1960844994e-01
       coeff_l1(28,60)=  7.1241563559e-01
       coeff_l1(28,61)= -3.4223380685e-01
       coeff_l1(28,62)= -8.3940938115e-02
       coeff_l1(28,63)= -5.3235423565e-01
       coeff_l1(28,64)= -2.3811790347e-01
       coeff_l1(28,65)= -3.2332006097e-01
       coeff_l1(29,1)=  8.0503582954e-02
       coeff_l1(29,2)= -6.1123803258e-02
       coeff_l1(29,3)= -3.6489269137e-01
       coeff_l1(29,4)= -7.1044832468e-02
       coeff_l1(29,5)= -1.8957975507e-01
       coeff_l1(29,6)= -6.2026497908e-03
       coeff_l1(29,7)= -2.6340645552e-01
       coeff_l1(29,8)=  5.1470924169e-02
       coeff_l1(29,9)= -1.9218889996e-02
       coeff_l1(29,10)=  1.3925102353e-01
       coeff_l1(29,11)=  1.8082606792e-01
       coeff_l1(29,12)=  7.2306513786e-02
       coeff_l1(29,13)= -1.4108286798e-01
       coeff_l1(29,14)= -1.4199087024e-01
       coeff_l1(29,15)=  2.8607788682e-01
       coeff_l1(29,16)= -2.4299040437e-01
       coeff_l1(29,17)= -5.5048588663e-02
       coeff_l1(29,18)= -1.2517178059e-01
       coeff_l1(29,19)= -7.3722556233e-02
       coeff_l1(29,20)= -1.4278717339e-01
       coeff_l1(29,21)= -5.1387403160e-02
       coeff_l1(29,22)= -1.7654798925e-01
       coeff_l1(29,23)= -9.8810650408e-02
       coeff_l1(29,24)= -2.4977292120e-01
       coeff_l1(29,25)= -8.7336696684e-02
       coeff_l1(29,26)=  8.2431502640e-02
       coeff_l1(29,27)=  2.1630677581e-01
       coeff_l1(29,28)=  3.0251124501e-01
       coeff_l1(29,29)=  2.4834360182e-01
       coeff_l1(29,30)= -2.2179166973e-01
       coeff_l1(29,31)=  1.5621154010e-01
       coeff_l1(29,32)= -1.0560543835e-01
       coeff_l1(29,33)=  1.0902716895e-04
       coeff_l1(29,34)= -3.9316922426e-02
       coeff_l1(29,35)= -2.6185205206e-02
       coeff_l1(29,36)= -2.6426129043e-02
       coeff_l1(29,37)=  2.5933375955e-01
       coeff_l1(29,38)= -3.3076691628e-01
       coeff_l1(29,39)= -2.0831994712e-01
       coeff_l1(29,40)= -2.3086911440e-01
       coeff_l1(29,41)=  5.0133578479e-02
       coeff_l1(29,42)= -5.2034258842e-02
       coeff_l1(29,43)= -4.4798123837e-01
       coeff_l1(29,44)=  1.3257557154e-01
       coeff_l1(29,45)= -1.8885137141e-01
       coeff_l1(29,46)= -1.1058407836e-02
       coeff_l1(29,47)= -2.5025194883e-01
       coeff_l1(29,48)=  1.0080197453e-01
       coeff_l1(29,49)= -6.5210804343e-02
       coeff_l1(29,50)= -1.1014013737e-01
       coeff_l1(29,51)=  3.2484654337e-02
       coeff_l1(29,52)= -1.9122743607e-01
       coeff_l1(29,53)=  9.7074881196e-02
       coeff_l1(29,54)=  6.8492464721e-02
       coeff_l1(29,55)= -1.3095244765e-01
       coeff_l1(29,56)=  1.2136671692e-02
       coeff_l1(29,57)=  2.1252271533e-01
       coeff_l1(29,58)= -1.4789137244e-01
       coeff_l1(29,59)=  8.7475501001e-02
       coeff_l1(29,60)= -2.1366798878e-01
       coeff_l1(29,61)= -1.2436699122e-01
       coeff_l1(29,62)= -1.6000342369e-01
       coeff_l1(29,63)= -4.4359028339e-01
       coeff_l1(29,64)=  1.9124118984e-01
       coeff_l1(29,65)=  1.7641590536e-01
       coeff_l1(30,1)= -1.4102968574e-01
       coeff_l1(30,2)=  3.6705136299e-01
       coeff_l1(30,3)= -1.0728913546e+00
       coeff_l1(30,4)=  1.8813546002e-01
       coeff_l1(30,5)=  1.8724222481e-01
       coeff_l1(30,6)= -4.8455858231e-01
       coeff_l1(30,7)= -2.5948715210e-01
       coeff_l1(30,8)=  1.1723179370e-01
       coeff_l1(30,9)=  8.1895035505e-01
       coeff_l1(30,10)=  5.4836194962e-02
       coeff_l1(30,11)=  2.8586134315e-01
       coeff_l1(30,12)=  3.0282053351e-01
       coeff_l1(30,13)= -1.3366976380e-01
       coeff_l1(30,14)= -5.1768428087e-01
       coeff_l1(30,15)=  2.7718690038e-01
       coeff_l1(30,16)=  4.1438474655e+00
       coeff_l1(30,17)=  1.2976645231e+00
       coeff_l1(30,18)= -3.6881315708e-01
       coeff_l1(30,19)=  1.8854010105e-01
       coeff_l1(30,20)=  1.1210598797e-01
       coeff_l1(30,21)=  2.1755772829e-01
       coeff_l1(30,22)= -2.7913808823e-01
       coeff_l1(30,23)= -4.2833322287e-01
       coeff_l1(30,24)=  2.2638656199e-01
       coeff_l1(30,25)= -1.3144917786e-01
       coeff_l1(30,26)=  1.0331577808e-01
       coeff_l1(30,27)=  1.5381627083e+00
       coeff_l1(30,28)=  2.2893340588e+00
       coeff_l1(30,29)= -3.4110629559e-01
       coeff_l1(30,30)= -3.8996949792e-01
       coeff_l1(30,31)= -6.1593693495e-01
       coeff_l1(30,32)=  1.6350027919e-01
       coeff_l1(30,33)=  7.3166924715e-01
       coeff_l1(30,34)=  2.3445868865e-02
       coeff_l1(30,35)=  3.8642835617e-01
       coeff_l1(30,36)= -2.1463729441e-01
       coeff_l1(30,37)=  1.0500190407e-01
       coeff_l1(30,38)= -4.7924262285e-01
       coeff_l1(30,39)=  1.1912842281e-02
       coeff_l1(30,40)= -8.2112652063e-01
       coeff_l1(30,41)=  4.6691279858e-02
       coeff_l1(30,42)=  1.1414055824e+00
       coeff_l1(30,43)= -5.8413749933e-01
       coeff_l1(30,44)=  2.5037437677e-01
       coeff_l1(30,45)= -1.2710347176e+00
       coeff_l1(30,46)= -2.0991547499e-03
       coeff_l1(30,47)= -2.1701142788e+00
       coeff_l1(30,48)=  2.1255113184e-01
       coeff_l1(30,49)= -1.8075077236e-01
       coeff_l1(30,50)= -4.3837475777e-01
       coeff_l1(30,51)=  4.4134718180e-01
       coeff_l1(30,52)=  4.0296405554e-01
       coeff_l1(30,53)= -9.8174464703e-01
       coeff_l1(30,54)=  1.9830583036e-01
       coeff_l1(30,55)=  1.4686485529e+00
       coeff_l1(30,56)=  4.0698552132e-01
       coeff_l1(30,57)=  6.5193414688e-01
       coeff_l1(30,58)= -1.3508610427e-01
       coeff_l1(30,59)= -6.5142768621e-01
       coeff_l1(30,60)=  5.0241130590e-01
       coeff_l1(30,61)=  7.4865585566e-01
       coeff_l1(30,62)=  3.2500964403e-01
       coeff_l1(30,63)= -9.6428498626e-02
       coeff_l1(30,64)= -1.5728789568e+00
       coeff_l1(30,65)=  2.0028287172e-01
!coeff layer 2
       coeff_l2(1,1)= -1.2410203218e+00
       coeff_l2(1,2)=  2.2132311761e-01
       coeff_l2(1,3)= -5.6005018950e-01
       coeff_l2(1,4)=  6.4211386442e-01
       coeff_l2(1,5)= -4.7503963113e-01
       coeff_l2(1,6)= -7.8307962418e-01
       coeff_l2(1,7)= -1.7955573276e-02
       coeff_l2(1,8)=  2.0789112151e-01
       coeff_l2(1,9)= -2.5749933720e-01
       coeff_l2(1,10)=  5.6069713831e-01
       coeff_l2(1,11)= -3.7145057321e-01
       coeff_l2(1,12)= -3.2061040401e-02
       coeff_l2(1,13)= -2.4029760063e-01
       coeff_l2(1,14)= -2.0723558962e-01
       coeff_l2(1,15)= -3.1059342623e-01
       coeff_l2(1,16)= -4.8795351386e-01
       coeff_l2(1,17)= -1.4195011556e-01
       coeff_l2(1,18)=  6.9616258144e-02
       coeff_l2(1,19)=  2.2025551647e-02
       coeff_l2(1,20)=  3.8339161873e-01
       coeff_l2(1,21)= -2.5204071403e-01
       coeff_l2(1,22)= -2.4370138347e-01
       coeff_l2(1,23)=  1.7075280845e-01
       coeff_l2(1,24)=  9.9043793976e-02
       coeff_l2(1,25)= -8.2103803754e-02
       coeff_l2(1,26)= -1.3697201014e-01
       coeff_l2(1,27)=  2.2899852693e-01
       coeff_l2(1,28)=  5.0031248480e-02
       coeff_l2(1,29)=  2.0525096357e-01
       coeff_l2(1,30)= -1.8200965226e-01
       coeff_l2(1,31)=  1.3443720341e-01
       coeff_l2(1,32)=  1.9447623193e-01
       coeff_l2(1,33)=  2.7342087030e-01
       coeff_l2(1,34)=  1.6882313415e-02
       coeff_l2(1,35)=  8.8613021374e-01
       coeff_l2(1,36)= -1.9782549143e-01
       coeff_l2(1,37)= -8.7498348951e-01
       coeff_l2(1,38)=  1.1272707582e-01
       coeff_l2(1,39)=  3.2858866453e-01
       coeff_l2(1,40)=  6.1475825310e-01
       coeff_l2(1,41)=  4.6049430966e-02
       coeff_l2(1,42)= -6.9038368762e-02
       coeff_l2(1,43)=  1.1793004349e-02
       coeff_l2(1,44)= -1.1717519164e-01
       coeff_l2(1,45)=  5.7141132653e-02
       coeff_l2(1,46)=  1.8741677701e-01
       coeff_l2(1,47)=  6.6416567564e-01
       coeff_l2(1,48)=  2.7162402868e-01
       coeff_l2(1,49)=  8.6166858673e-02
       coeff_l2(1,50)=  6.6016203165e-01
       coeff_l2(1,51)= -2.4543359876e-01
       coeff_l2(1,52)=  6.8049006164e-02
       coeff_l2(1,53)= -2.4546448886e-01
       coeff_l2(1,54)= -1.6304244101e-01
       coeff_l2(1,55)=  5.1186281443e-01
       coeff_l2(1,56)= -1.4956958592e-01
       coeff_l2(1,57)= -3.9605385065e-01
       coeff_l2(1,58)= -5.9192430973e-01
       coeff_l2(1,59)=  3.1249895692e-01
       coeff_l2(1,60)=  9.7447320819e-02
       coeff_l2(1,61)= -7.1930892766e-02
       coeff_l2(1,62)= -1.3099950552e-01
       coeff_l2(1,63)=  6.0779428482e-01
       coeff_l2(1,64)= -1.0889061540e-01
       coeff_l2(1,65)= -4.1255027056e-01
       coeff_l2(1,66)= -6.2208026648e-01
       coeff_l2(1,67)= -4.0946085006e-02
       coeff_l2(1,68)=  1.6922903061e-01
       coeff_l2(1,69)= -3.8861446083e-02
       coeff_l2(1,70)=  1.2799137831e-01
       coeff_l2(1,71)=  3.3142659068e-01
       coeff_l2(1,72)= -5.0786118954e-02
       coeff_l2(1,73)= -1.1383208632e-01
       coeff_l2(1,74)=  1.4007358253e-01
       coeff_l2(1,75)=  1.8374556303e-01
       coeff_l2(1,76)= -1.9745630026e-01
       coeff_l2(1,77)=  3.4502074122e-02
       coeff_l2(1,78)=  7.0126771927e-02
       coeff_l2(1,79)= -8.4556829929e-01
       coeff_l2(1,80)= -1.1937032938e+00
       coeff_l2(1,81)= -1.0111943632e-01
       coeff_l2(1,82)= -5.6903797388e-01
       coeff_l2(1,83)= -2.9741248488e-01
       coeff_l2(1,84)=  9.7584143281e-02
       coeff_l2(1,85)= -4.7191753983e-01
       coeff_l2(1,86)=  1.1855342984e-01
       coeff_l2(1,87)=  1.5036350489e-01
       coeff_l2(1,88)= -2.7878490090e-01
       coeff_l2(1,89)= -1.7747576535e-01
       coeff_l2(1,90)=  1.3988026381e+00
       coeff_l2(1,91)= -9.0973354876e-02
       coeff_l2(1,92)=  2.7652867138e-02
       coeff_l2(1,93)= -6.1143796891e-02
       coeff_l2(1,94)=  6.3563004136e-02
       coeff_l2(1,95)= -1.8573340774e-01
       coeff_l2(2,1)= -6.1162300408e-02
       coeff_l2(2,2)= -9.3019135296e-02
       coeff_l2(2,3)= -2.4521583319e-01
       coeff_l2(2,4)= -1.3154496439e-02
       coeff_l2(2,5)= -9.5969063044e-01
       coeff_l2(2,6)= -5.2845120430e-01
       coeff_l2(2,7)= -1.1104265451e+00
       coeff_l2(2,8)=  1.4442925155e-01
       coeff_l2(2,9)= -9.3518439680e-03
       coeff_l2(2,10)=  9.8251420259e-01
       coeff_l2(2,11)= -2.9044252634e-01
       coeff_l2(2,12)= -1.4019270241e-01
       coeff_l2(2,13)= -1.3944816589e-01
       coeff_l2(2,14)= -4.5289787650e-01
       coeff_l2(2,15)= -3.5302150249e-01
       coeff_l2(2,16)=  1.9400784373e-01
       coeff_l2(2,17)=  7.2887748480e-02
       coeff_l2(2,18)=  1.6132821143e-01
       coeff_l2(2,19)= -1.0018026084e-01
       coeff_l2(2,20)= -1.4105996490e-01
       coeff_l2(2,21)= -3.8590663671e-01
       coeff_l2(2,22)= -2.4915182590e-01
       coeff_l2(2,23)=  1.6935245693e-01
       coeff_l2(2,24)= -1.2874841690e-01
       coeff_l2(2,25)=  1.1015984416e-01
       coeff_l2(2,26)= -1.0895083845e-01
       coeff_l2(2,27)=  2.2138716280e-01
       coeff_l2(2,28)= -3.2480806112e-01
       coeff_l2(2,29)=  3.9410069585e-02
       coeff_l2(2,30)=  6.9738328457e-02
       coeff_l2(2,31)=  1.3370661438e-01
       coeff_l2(2,32)=  3.0013836920e-02
       coeff_l2(2,33)=  1.6208539903e-01
       coeff_l2(2,34)=  2.1177713573e-01
       coeff_l2(2,35)=  8.5499686003e-01
       coeff_l2(2,36)= -4.4201123714e-01
       coeff_l2(2,37)= -7.1229952574e-01
       coeff_l2(2,38)= -1.2776447833e-01
       coeff_l2(2,39)=  2.9603722692e-01
       coeff_l2(2,40)= -8.9105628431e-02
       coeff_l2(2,41)=  8.6282439530e-02
       coeff_l2(2,42)=  2.7679476142e-01
       coeff_l2(2,43)= -3.8471949100e-01
       coeff_l2(2,44)= -2.0326651633e-01
       coeff_l2(2,45)= -3.5579422116e-01
       coeff_l2(2,46)=  2.5760108232e-01
       coeff_l2(2,47)=  4.7127056122e-01
       coeff_l2(2,48)= -1.4892412350e-02
       coeff_l2(2,49)= -8.6506985128e-02
       coeff_l2(2,50)=  1.0484275818e+00
       coeff_l2(2,51)= -2.2679622471e-01
       coeff_l2(2,52)= -1.1624832451e-01
       coeff_l2(2,53)= -5.5873308331e-02
       coeff_l2(2,54)= -2.0101258159e-01
       coeff_l2(2,55)=  5.4661113769e-02
       coeff_l2(2,56)= -3.5833290219e-01
       coeff_l2(2,57)= -1.2348137051e-01
       coeff_l2(2,58)= -4.2257168889e-01
       coeff_l2(2,59)= -2.2297476232e-01
       coeff_l2(2,60)=  3.7889722735e-02
       coeff_l2(2,61)= -6.4803608693e-03
       coeff_l2(2,62)=  2.7731069922e-01
       coeff_l2(2,63)= -3.6829371005e-02
       coeff_l2(2,64)= -4.1064164042e-01
       coeff_l2(2,65)= -2.3111641407e-01
       coeff_l2(2,66)= -3.1423825026e-01
       coeff_l2(2,67)=  4.3649554253e-01
       coeff_l2(2,68)=  2.4693992734e-01
       coeff_l2(2,69)=  2.6754831895e-02
       coeff_l2(2,70)=  2.7360143140e-02
       coeff_l2(2,71)=  5.9667587280e-01
       coeff_l2(2,72)=  3.2774351537e-02
       coeff_l2(2,73)= -2.3725137115e-01
       coeff_l2(2,74)= -2.5386777520e-01
       coeff_l2(2,75)=  4.7276031226e-02
       coeff_l2(2,76)=  4.3583344668e-02
       coeff_l2(2,77)= -2.3449505866e-01
       coeff_l2(2,78)=  5.2317503840e-02
       coeff_l2(2,79)= -7.1030294895e-01
       coeff_l2(2,80)= -4.7769281268e-01
       coeff_l2(2,81)= -2.0030844212e-01
       coeff_l2(2,82)= -2.8909948468e-01
       coeff_l2(2,83)= -1.0813160241e-01
       coeff_l2(2,84)= -1.9299712032e-02
       coeff_l2(2,85)= -2.8997562826e-02
       coeff_l2(2,86)=  2.4851150811e-01
       coeff_l2(2,87)= -2.1640652418e-01
       coeff_l2(2,88)=  8.6047850549e-02
       coeff_l2(2,89)= -2.9295533895e-01
       coeff_l2(2,90)=  5.2803421021e-01
       coeff_l2(2,91)= -1.6516700387e-01
       coeff_l2(2,92)= -1.5075215697e-01
       coeff_l2(2,93)= -2.2884576023e-01
       coeff_l2(2,94)= -8.8094450533e-02
       coeff_l2(2,95)= -3.5460594296e-01
       coeff_l2(3,1)=  3.1898880005e-01
       coeff_l2(3,2)=  7.2367572784e-01
       coeff_l2(3,3)= -2.9890590906e-01
       coeff_l2(3,4)=  3.0575728044e-02
       coeff_l2(3,5)=  4.5425659418e-01
       coeff_l2(3,6)= -1.4473737776e-01
       coeff_l2(3,7)= -5.9547406435e-01
       coeff_l2(3,8)= -7.7300995588e-01
       coeff_l2(3,9)=  8.1547796726e-02
       coeff_l2(3,10)=  1.5384557247e+00
       coeff_l2(3,11)= -2.6303788647e-02
       coeff_l2(3,12)=  1.8397862092e-02
       coeff_l2(3,13)= -1.7648348212e-01
       coeff_l2(3,14)= -2.1668280661e-01
       coeff_l2(3,15)= -3.1622940302e-01
       coeff_l2(3,16)=  2.5840920210e-01
       coeff_l2(3,17)= -1.7604994774e-01
       coeff_l2(3,18)= -5.6006515026e-01
       coeff_l2(3,19)= -2.0728203654e-01
       coeff_l2(3,20)=  6.5391577780e-02
       coeff_l2(3,21)= -3.3741532825e-03
       coeff_l2(3,22)=  6.0461151600e-01
       coeff_l2(3,23)= -5.2995866537e-01
       coeff_l2(3,24)= -1.7620757222e-01
       coeff_l2(3,25)=  1.1002805233e+00
       coeff_l2(3,26)=  1.3789071143e-01
       coeff_l2(3,27)= -5.8178478479e-01
       coeff_l2(3,28)= -7.9761110246e-02
       coeff_l2(3,29)=  1.6251582652e-02
       coeff_l2(3,30)= -2.8215926141e-02
       coeff_l2(3,31)= -3.2903981209e-01
       coeff_l2(3,32)= -1.3494026661e-01
       coeff_l2(3,33)=  3.1618732214e-01
       coeff_l2(3,34)= -4.9406731129e-01
       coeff_l2(3,35)= -2.6574522257e-02
       coeff_l2(3,36)=  9.3403704464e-02
       coeff_l2(3,37)= -2.0605225861e-01
       coeff_l2(3,38)=  7.0975117385e-02
       coeff_l2(3,39)=  1.8956945837e-01
       coeff_l2(3,40)= -1.2436080724e-01
       coeff_l2(3,41)= -1.1141259670e+00
       coeff_l2(3,42)= -6.7986583710e-01
       coeff_l2(3,43)=  1.3397792578e+00
       coeff_l2(3,44)=  1.0763647407e-01
       coeff_l2(3,45)=  2.4864375591e-03
       coeff_l2(3,46)=  6.7844915390e-01
       coeff_l2(3,47)=  1.5787785053e+00
       coeff_l2(3,48)= -1.3601517677e-01
       coeff_l2(3,49)=  3.1378036737e-01
       coeff_l2(3,50)=  8.3529496193e-01
       coeff_l2(3,51)=  1.2902431190e-01
       coeff_l2(3,52)= -8.0447804928e-01
       coeff_l2(3,53)=  1.8050594628e-01
       coeff_l2(3,54)= -3.9826610684e-01
       coeff_l2(3,55)= -1.1046811938e-02
       coeff_l2(3,56)= -3.5869023204e-01
       coeff_l2(3,57)=  4.3200675398e-02
       coeff_l2(3,58)=  1.0963662863e+00
       coeff_l2(3,59)= -3.5692033172e-01
       coeff_l2(3,60)= -8.2962282002e-02
       coeff_l2(3,61)=  1.7470349371e-01
       coeff_l2(3,62)=  5.7409214973e-01
       coeff_l2(3,63)= -9.6983492374e-02
       coeff_l2(3,64)=  7.5919276476e-01
       coeff_l2(3,65)= -2.1962799132e-01
       coeff_l2(3,66)=  2.8292729855e+00
       coeff_l2(3,67)=  1.2752805948e+00
       coeff_l2(3,68)=  5.7547062635e-01
       coeff_l2(3,69)= -4.4093993306e-01
       coeff_l2(3,70)= -3.6569440365e-01
       coeff_l2(3,71)=  9.1067767143e-01
       coeff_l2(3,72)= -4.1811195016e-01
       coeff_l2(3,73)= -1.0945550203e+00
       coeff_l2(3,74)=  3.9351873100e-02
       coeff_l2(3,75)=  2.1819500625e-01
       coeff_l2(3,76)= -6.3826316036e-03
       coeff_l2(3,77)=  9.6845287085e-01
       coeff_l2(3,78)= -3.4076333046e-01
       coeff_l2(3,79)=  5.5572893471e-02
       coeff_l2(3,80)=  5.2985399961e-01
       coeff_l2(3,81)= -2.9672434926e-01
       coeff_l2(3,82)= -2.9757049680e-01
       coeff_l2(3,83)= -2.7363210917e-01
       coeff_l2(3,84)= -9.1176354885e-01
       coeff_l2(3,85)= -3.5159325600e-01
       coeff_l2(3,86)= -5.4522854090e-01
       coeff_l2(3,87)=  4.3883915991e-02
       coeff_l2(3,88)=  4.4169181585e-01
       coeff_l2(3,89)=  1.5888790786e-01
       coeff_l2(3,90)=  6.9760859013e-01
       coeff_l2(3,91)=  3.2743954659e-01
       coeff_l2(3,92)= -6.1341226101e-02
       coeff_l2(3,93)= -4.6136227250e-01
       coeff_l2(3,94)=  5.6252982467e-02
       coeff_l2(3,95)= -4.0909534693e-01
       coeff_l2(4,1)=  2.4054554105e-01
       coeff_l2(4,2)= -1.8389263749e-01
       coeff_l2(4,3)= -1.2232901156e-01
       coeff_l2(4,4)=  1.0065637827e+00
       coeff_l2(4,5)=  1.0114035010e-01
       coeff_l2(4,6)=  9.7736477852e-01
       coeff_l2(4,7)=  2.2427201271e-01
       coeff_l2(4,8)= -2.1184046566e-01
       coeff_l2(4,9)= -9.2850923538e-02
       coeff_l2(4,10)=  7.5266009569e-01
       coeff_l2(4,11)= -1.7739719152e-01
       coeff_l2(4,12)= -2.9156122357e-02
       coeff_l2(4,13)=  4.1222381592e-01
       coeff_l2(4,14)=  2.5499576703e-02
       coeff_l2(4,15)= -2.4822914600e-01
       coeff_l2(4,16)=  4.4801086187e-01
       coeff_l2(4,17)= -8.1438943744e-02
       coeff_l2(4,18)= -7.0933021605e-02
       coeff_l2(4,19)= -7.3150180280e-02
       coeff_l2(4,20)=  2.6677811146e-01
       coeff_l2(4,21)=  1.0718363523e-01
       coeff_l2(4,22)=  9.5415538549e-01
       coeff_l2(4,23)= -1.7120248079e-01
       coeff_l2(4,24)=  1.3157013059e-01
       coeff_l2(4,25)=  1.1757522821e+00
       coeff_l2(4,26)= -1.8770261109e-01
       coeff_l2(4,27)=  1.0651954263e-01
       coeff_l2(4,28)=  8.4976130724e-01
       coeff_l2(4,29)= -2.8688472509e-01
       coeff_l2(4,30)= -9.0923979878e-02
       coeff_l2(4,31)=  7.5077950954e-01
       coeff_l2(4,32)= -1.8361809850e-01
       coeff_l2(4,33)=  1.6200695932e-01
       coeff_l2(4,34)=  2.8396853805e-01
       coeff_l2(4,35)=  1.9019955397e+00
       coeff_l2(4,36)=  1.5807725489e-01
       coeff_l2(4,37)= -3.0920332670e-01
       coeff_l2(4,38)=  2.4949846789e-02
       coeff_l2(4,39)=  9.6495002508e-01
       coeff_l2(4,40)=  2.1639677882e-01
       coeff_l2(4,41)= -4.3126598001e-01
       coeff_l2(4,42)= -1.7102614045e-01
       coeff_l2(4,43)=  9.2821151018e-01
       coeff_l2(4,44)=  2.9202646017e-01
       coeff_l2(4,45)=  8.3432495594e-01
       coeff_l2(4,46)=  1.4006721973e-01
       coeff_l2(4,47)=  1.1589417458e+00
       coeff_l2(4,48)= -2.5113764405e-01
       coeff_l2(4,49)=  4.1757985950e-02
       coeff_l2(4,50)=  2.4961583614e+00
       coeff_l2(4,51)= -7.7829444408e-01
       coeff_l2(4,52)=  2.2027538717e-01
       coeff_l2(4,53)=  1.7561356723e-01
       coeff_l2(4,54)= -6.4633858204e-01
       coeff_l2(4,55)=  5.1545935869e-01
       coeff_l2(4,56)= -4.2943224311e-02
       coeff_l2(4,57)=  2.4583971500e-01
       coeff_l2(4,58)=  2.2548586130e-01
       coeff_l2(4,59)=  1.6240946949e-01
       coeff_l2(4,60)= -2.4801260233e-01
       coeff_l2(4,61)=  5.0387341529e-02
       coeff_l2(4,62)=  8.6148582399e-02
       coeff_l2(4,63)=  1.8654906750e+00
       coeff_l2(4,64)=  4.4811025262e-01
       coeff_l2(4,65)= -6.7955762148e-01
       coeff_l2(4,66)=  1.3428761959e+00
       coeff_l2(4,67)=  4.4153109193e-01
       coeff_l2(4,68)=  4.4874584675e-01
       coeff_l2(4,69)=  4.6599995345e-02
       coeff_l2(4,70)= -1.2832267582e-01
       coeff_l2(4,71)=  1.8517224491e-01
       coeff_l2(4,72)= -6.9421756268e-01
       coeff_l2(4,73)= -3.5323441029e-01
       coeff_l2(4,74)= -2.1209689975e-01
       coeff_l2(4,75)=  1.9456726313e-01
       coeff_l2(4,76)=  1.1779699475e-01
       coeff_l2(4,77)=  5.8228278160e-01
       coeff_l2(4,78)= -3.6805233359e-01
       coeff_l2(4,79)= -6.8355900049e-01
       coeff_l2(4,80)=  1.9374439120e-01
       coeff_l2(4,81)=  1.5285043418e-01
       coeff_l2(4,82)=  3.2106542587e-01
       coeff_l2(4,83)= -3.7002470344e-02
       coeff_l2(4,84)= -1.2646122277e-01
       coeff_l2(4,85)= -9.8491713405e-02
       coeff_l2(4,86)= -2.6684203744e-01
       coeff_l2(4,87)=  4.6223482490e-01
       coeff_l2(4,88)= -6.1594033241e-01
       coeff_l2(4,89)= -5.0158160925e-01
       coeff_l2(4,90)= -2.6094693691e-02
       coeff_l2(4,91)=  3.5569253564e-01
       coeff_l2(4,92)=  3.8878142834e-01
       coeff_l2(4,93)=  4.8038581014e-01
       coeff_l2(4,94)= -2.1521785855e-01
       coeff_l2(4,95)=  2.9883334041e-01
       coeff_l2(5,1)= -4.1230618954e-01
       coeff_l2(5,2)=  1.8983022869e-01
       coeff_l2(5,3)= -2.2500331700e-01
       coeff_l2(5,4)= -9.4079956412e-02
       coeff_l2(5,5)= -4.0274995565e-01
       coeff_l2(5,6)= -2.4266882241e-01
       coeff_l2(5,7)= -9.0246325731e-01
       coeff_l2(5,8)= -1.0598501563e-01
       coeff_l2(5,9)= -2.1187399328e-01
       coeff_l2(5,10)=  3.0759134889e-01
       coeff_l2(5,11)= -4.1743239760e-01
       coeff_l2(5,12)= -2.9901185632e-01
       coeff_l2(5,13)= -1.6549593210e-01
       coeff_l2(5,14)= -3.1591203809e-01
       coeff_l2(5,15)= -1.6487392783e-01
       coeff_l2(5,16)= -2.0546160638e-02
       coeff_l2(5,17)= -1.7456842959e-01
       coeff_l2(5,18)= -1.7544181645e-01
       coeff_l2(5,19)= -4.4081442058e-02
       coeff_l2(5,20)=  1.4027942717e-01
       coeff_l2(5,21)= -3.3679679036e-01
       coeff_l2(5,22)= -8.0293744802e-02
       coeff_l2(5,23)=  5.5479694158e-02
       coeff_l2(5,24)= -1.5893357992e-01
       coeff_l2(5,25)=  2.6060527191e-02
       coeff_l2(5,26)=  1.1807224900e-01
       coeff_l2(5,27)=  4.8692482524e-03
       coeff_l2(5,28)= -1.9432557747e-02
       coeff_l2(5,29)= -1.9236820936e-01
       coeff_l2(5,30)= -2.0147211850e-01
       coeff_l2(5,31)=  5.0629597157e-02
       coeff_l2(5,32)= -2.9624715447e-02
       coeff_l2(5,33)=  4.3256729841e-02
       coeff_l2(5,34)=  1.1934757233e-01
       coeff_l2(5,35)=  3.5532265902e-01
       coeff_l2(5,36)= -5.3192657232e-01
       coeff_l2(5,37)= -3.0697819591e-01
       coeff_l2(5,38)= -3.3826783299e-02
       coeff_l2(5,39)=  9.7017645836e-02
       coeff_l2(5,40)=  1.7288149893e-01
       coeff_l2(5,41)= -8.6030967534e-02
       coeff_l2(5,42)= -5.7316970080e-02
       coeff_l2(5,43)= -2.8181591630e-01
       coeff_l2(5,44)= -4.4618170708e-02
       coeff_l2(5,45)= -3.2094275951e-01
       coeff_l2(5,46)= -5.8702137321e-02
       coeff_l2(5,47)=  2.1989616752e-01
       coeff_l2(5,48)=  5.1914351061e-03
       coeff_l2(5,49)=  1.6978573799e-01
       coeff_l2(5,50)=  6.8235051632e-01
       coeff_l2(5,51)= -1.3991467655e-01
       coeff_l2(5,52)=  1.3154281676e-01
       coeff_l2(5,53)= -1.9291448593e-01
       coeff_l2(5,54)= -2.1789330244e-01
       coeff_l2(5,55)= -1.3009972870e-02
       coeff_l2(5,56)=  7.1868017316e-02
       coeff_l2(5,57)=  4.5528557152e-02
       coeff_l2(5,58)= -3.9936548471e-01
       coeff_l2(5,59)=  8.9255310595e-03
       coeff_l2(5,60)= -1.3610324264e-01
       coeff_l2(5,61)= -3.9642745256e-01
       coeff_l2(5,62)= -3.0381355435e-02
       coeff_l2(5,63)=  8.9164637029e-02
       coeff_l2(5,64)= -1.4955012500e-01
       coeff_l2(5,65)=  4.3388519436e-02
       coeff_l2(5,66)= -5.1806259155e-01
       coeff_l2(5,67)= -1.0552508384e-01
       coeff_l2(5,68)= -1.3476791978e-01
       coeff_l2(5,69)=  5.4479982704e-02
       coeff_l2(5,70)=  7.6333597302e-02
       coeff_l2(5,71)=  2.9180964828e-01
       coeff_l2(5,72)=  1.4452951029e-02
       coeff_l2(5,73)= -1.6099995375e-01
       coeff_l2(5,74)= -2.5484618545e-01
       coeff_l2(5,75)=  6.7980855703e-02
       coeff_l2(5,76)= -2.3754030466e-01
       coeff_l2(5,77)= -1.6767659783e-01
       coeff_l2(5,78)=  1.9334867597e-01
       coeff_l2(5,79)= -3.3171984553e-01
       coeff_l2(5,80)= -5.0037574768e-01
       coeff_l2(5,81)=  2.6200076565e-02
       coeff_l2(5,82)= -1.8461494148e-01
       coeff_l2(5,83)= -2.8288963437e-01
       coeff_l2(5,84)=  1.5274092555e-01
       coeff_l2(5,85)= -2.2155432031e-02
       coeff_l2(5,86)=  1.0227169842e-01
       coeff_l2(5,87)=  1.0247675329e-01
       coeff_l2(5,88)=  8.7384924293e-02
       coeff_l2(5,89)= -2.4125958979e-01
       coeff_l2(5,90)= -2.6559893042e-02
       coeff_l2(5,91)=  2.6035815477e-01
       coeff_l2(5,92)= -6.6838294268e-02
       coeff_l2(5,93)= -1.4476603270e-01
       coeff_l2(5,94)=  2.1227378398e-02
       coeff_l2(5,95)=  7.5302340090e-02
       coeff_l2(6,1)= -5.8734197170e-02
       coeff_l2(6,2)= -7.7978205681e-01
       coeff_l2(6,3)= -1.1496253312e-01
       coeff_l2(6,4)= -7.8556738794e-02
       coeff_l2(6,5)=  8.2849341631e-01
       coeff_l2(6,6)= -1.1840776354e-01
       coeff_l2(6,7)=  7.0933616161e-01
       coeff_l2(6,8)= -1.0315876454e-01
       coeff_l2(6,9)=  1.5700152516e-01
       coeff_l2(6,10)= -8.1557190418e-01
       coeff_l2(6,11)=  1.4768952131e-01
       coeff_l2(6,12)=  1.3755942881e-01
       coeff_l2(6,13)= -1.1726459116e-01
       coeff_l2(6,14)=  1.7796023190e-01
       coeff_l2(6,15)= -1.3745155931e-01
       coeff_l2(6,16)=  1.0018548369e-01
       coeff_l2(6,17)=  1.6075611115e-01
       coeff_l2(6,18)= -5.1377043128e-02
       coeff_l2(6,19)=  2.1468068659e-01
       coeff_l2(6,20)= -2.8209492564e-01
       coeff_l2(6,21)=  8.3428913355e-01
       coeff_l2(6,22)=  1.4582383633e-01
       coeff_l2(6,23)= -2.9765114188e-01
       coeff_l2(6,24)=  3.8014435768e-01
       coeff_l2(6,25)=  1.6791442037e-01
       coeff_l2(6,26)=  1.6417032480e-01
       coeff_l2(6,27)= -2.5561660528e-01
       coeff_l2(6,28)= -9.2250712216e-02
       coeff_l2(6,29)=  6.8991556764e-02
       coeff_l2(6,30)=  1.9559918437e-03
       coeff_l2(6,31)=  2.6308497787e-01
       coeff_l2(6,32)=  4.0039499290e-03
       coeff_l2(6,33)= -1.2504833937e-01
       coeff_l2(6,34)=  1.2895759940e-01
       coeff_l2(6,35)=  1.4537162781e+00
       coeff_l2(6,36)=  1.1692250520e-01
       coeff_l2(6,37)=  8.5605829954e-01
       coeff_l2(6,38)= -3.0313724279e-01
       coeff_l2(6,39)=  5.3627365828e-01
       coeff_l2(6,40)=  9.3740239739e-02
       coeff_l2(6,41)=  1.6325963661e-02
       coeff_l2(6,42)= -2.5708961487e-01
       coeff_l2(6,43)= -7.0962309837e-02
       coeff_l2(6,44)=  6.6753782332e-02
       coeff_l2(6,45)= -2.6097673178e-01
       coeff_l2(6,46)= -2.1063639224e-01
       coeff_l2(6,47)= -1.6522264481e-01
       coeff_l2(6,48)= -2.4789513648e-01
       coeff_l2(6,49)= -3.2427459955e-01
       coeff_l2(6,50)=  8.4467792511e-01
       coeff_l2(6,51)= -3.9282396436e-01
       coeff_l2(6,52)= -4.5942178369e-01
       coeff_l2(6,53)= -7.3910713196e-02
       coeff_l2(6,54)=  4.8325601220e-01
       coeff_l2(6,55)=  3.8906347752e-01
       coeff_l2(6,56)= -1.2703660727e+00
       coeff_l2(6,57)=  1.1465854198e-01
       coeff_l2(6,58)= -1.1266153306e-01
       coeff_l2(6,59)= -3.8563853502e-01
       coeff_l2(6,60)= -2.4424789846e-01
       coeff_l2(6,61)= -8.3634741604e-02
       coeff_l2(6,62)=  4.8937422037e-01
       coeff_l2(6,63)=  2.7176415920e-01
       coeff_l2(6,64)=  4.5951807499e-01
       coeff_l2(6,65)=  3.1174685806e-02
       coeff_l2(6,66)= -6.9742649794e-01
       coeff_l2(6,67)= -1.0051429272e+00
       coeff_l2(6,68)= -6.3484472036e-01
       coeff_l2(6,69)= -1.3234244287e-01
       coeff_l2(6,70)=  1.0277085751e-01
       coeff_l2(6,71)= -9.6181660891e-01
       coeff_l2(6,72)= -1.4234216511e-01
       coeff_l2(6,73)= -2.7187284827e-01
       coeff_l2(6,74)=  1.8408012390e-01
       coeff_l2(6,75)= -6.9855183363e-02
       coeff_l2(6,76)=  4.1886955500e-02
       coeff_l2(6,77)= -2.2214272618e-01
       coeff_l2(6,78)= -4.1115173697e-01
       coeff_l2(6,79)=  8.0318957567e-01
       coeff_l2(6,80)=  7.7656781673e-01
       coeff_l2(6,81)= -4.0905842185e-01
       coeff_l2(6,82)=  1.0781743377e-01
       coeff_l2(6,83)=  1.7155905068e-01
       coeff_l2(6,84)= -6.3482843339e-02
       coeff_l2(6,85)= -6.1318409443e-01
       coeff_l2(6,86)= -8.3356518298e-03
       coeff_l2(6,87)= -1.3898019493e-01
       coeff_l2(6,88)= -1.8485988677e-01
       coeff_l2(6,89)=  2.9112616181e-01
       coeff_l2(6,90)= -6.2893927097e-01
       coeff_l2(6,91)= -1.9632405043e-01
       coeff_l2(6,92)= -4.7621431947e-01
       coeff_l2(6,93)= -4.9347046018e-01
       coeff_l2(6,94)=  1.5760511160e-02
       coeff_l2(6,95)= -2.1140961349e-01
       coeff_l2(7,1)=  1.4249835908e-01
       coeff_l2(7,2)= -1.4899878204e-01
       coeff_l2(7,3)= -1.4626280963e-01
       coeff_l2(7,4)=  3.6703339219e-01
       coeff_l2(7,5)= -1.0012208670e-01
       coeff_l2(7,6)= -2.3927752674e-01
       coeff_l2(7,7)=  1.3883221149e-01
       coeff_l2(7,8)= -1.8656104803e-01
       coeff_l2(7,9)=  9.6201397479e-02
       coeff_l2(7,10)=  3.7682077289e-01
       coeff_l2(7,11)=  1.4003676176e-01
       coeff_l2(7,12)=  1.8723969162e-01
       coeff_l2(7,13)= -3.0349910259e-01
       coeff_l2(7,14)=  9.2786990106e-02
       coeff_l2(7,15)= -7.2581872344e-02
       coeff_l2(7,16)= -7.9600095749e-02
       coeff_l2(7,17)= -1.1733263731e-01
       coeff_l2(7,18)= -4.2090136558e-03
       coeff_l2(7,19)=  1.0578875989e-01
       coeff_l2(7,20)= -9.2391952872e-02
       coeff_l2(7,21)=  2.7771985531e-01
       coeff_l2(7,22)= -5.2260760218e-02
       coeff_l2(7,23)=  3.7590485066e-02
       coeff_l2(7,24)= -1.0446442664e-01
       coeff_l2(7,25)= -1.6331140697e-01
       coeff_l2(7,26)= -1.5995272994e-01
       coeff_l2(7,27)=  9.5231113955e-03
       coeff_l2(7,28)= -1.7280611396e-01
       coeff_l2(7,29)=  2.4724524468e-02
       coeff_l2(7,30)=  5.0624232739e-02
       coeff_l2(7,31)=  1.3201616704e-01
       coeff_l2(7,32)= -1.5245570242e-01
       coeff_l2(7,33)=  1.7383731902e-01
       coeff_l2(7,34)=  6.3029229641e-02
       coeff_l2(7,35)=  5.5843567848e-01
       coeff_l2(7,36)= -3.8810394704e-02
       coeff_l2(7,37)= -3.9188799262e-01
       coeff_l2(7,38)=  3.5899991635e-03
       coeff_l2(7,39)= -3.0231416225e-01
       coeff_l2(7,40)= -3.9423778653e-02
       coeff_l2(7,41)= -1.0801455379e-01
       coeff_l2(7,42)= -5.7043880224e-02
       coeff_l2(7,43)=  8.3415314555e-02
       coeff_l2(7,44)= -2.4480469525e-01
       coeff_l2(7,45)= -1.4988867939e-01
       coeff_l2(7,46)=  5.0210887194e-01
       coeff_l2(7,47)=  6.2182832509e-02
       coeff_l2(7,48)= -2.5032814592e-03
       coeff_l2(7,49)=  7.3140263557e-02
       coeff_l2(7,50)=  4.5318499207e-01
       coeff_l2(7,51)=  1.2790568173e-01
       coeff_l2(7,52)=  3.6080386490e-02
       coeff_l2(7,53)=  7.4581287801e-02
       coeff_l2(7,54)=  1.5523144603e-01
       coeff_l2(7,55)=  1.8054135144e-01
       coeff_l2(7,56)=  1.3186065480e-02
       coeff_l2(7,57)= -3.2688358426e-01
       coeff_l2(7,58)=  2.4243687093e-01
       coeff_l2(7,59)=  8.7534956634e-02
       coeff_l2(7,60)= -3.0079094693e-02
       coeff_l2(7,61)=  2.7183353901e-01
       coeff_l2(7,62)= -2.9996812344e-01
       coeff_l2(7,63)=  2.6654037833e-01
       coeff_l2(7,64)=  1.2449397892e-01
       coeff_l2(7,65)= -9.6360564232e-02
       coeff_l2(7,66)=  4.1359007359e-01
       coeff_l2(7,67)=  6.6028165817e-01
       coeff_l2(7,68)=  2.4003033340e-01
       coeff_l2(7,69)=  5.8214887977e-02
       coeff_l2(7,70)=  1.5085363388e-01
       coeff_l2(7,71)=  4.2141529918e-01
       coeff_l2(7,72)= -1.4363232069e-02
       coeff_l2(7,73)=  2.0665279031e-01
       coeff_l2(7,74)=  2.4758104235e-02
       coeff_l2(7,75)=  3.5120267421e-02
       coeff_l2(7,76)=  4.3470460922e-02
       coeff_l2(7,77)=  2.6432678103e-02
       coeff_l2(7,78)= -5.1857098937e-02
       coeff_l2(7,79)= -4.2358458042e-01
       coeff_l2(7,80)=  2.6184287667e-01
       coeff_l2(7,81)= -1.9648776948e-01
       coeff_l2(7,82)= -4.0860253572e-01
       coeff_l2(7,83)=  2.1774202585e-01
       coeff_l2(7,84)=  7.6632343233e-02
       coeff_l2(7,85)= -6.3248351216e-02
       coeff_l2(7,86)=  1.2325540930e-01
       coeff_l2(7,87)= -3.8922633976e-02
       coeff_l2(7,88)=  5.3788255900e-02
       coeff_l2(7,89)= -4.9447249621e-03
       coeff_l2(7,90)=  3.4007179737e-01
       coeff_l2(7,91)= -3.4763124585e-01
       coeff_l2(7,92)=  1.9620975852e-01
       coeff_l2(7,93)= -1.1935577542e-01
       coeff_l2(7,94)=  1.0025592893e-01
       coeff_l2(7,95)= -1.3827490807e-01
       coeff_l2(8,1)= -7.6970517635e-01
       coeff_l2(8,2)=  2.1229642630e-01
       coeff_l2(8,3)= -1.7797909677e-01
       coeff_l2(8,4)=  2.3142148554e-01
       coeff_l2(8,5)= -1.0321092606e+00
       coeff_l2(8,6)= -2.9129323363e-01
       coeff_l2(8,7)= -8.2011729479e-01
       coeff_l2(8,8)=  9.7922407091e-02
       coeff_l2(8,9)= -1.6952183843e-01
       coeff_l2(8,10)=  7.2293776274e-01
       coeff_l2(8,11)= -2.4012678768e-03
       coeff_l2(8,12)= -1.8469361961e-01
       coeff_l2(8,13)= -2.0428664982e-01
       coeff_l2(8,14)= -2.8806936741e-01
       coeff_l2(8,15)= -1.8042659760e-01
       coeff_l2(8,16)= -3.8500735164e-01
       coeff_l2(8,17)= -6.6249124706e-02
       coeff_l2(8,18)= -1.2528164685e-01
       coeff_l2(8,19)= -1.3289302588e-01
       coeff_l2(8,20)=  1.0305074602e-01
       coeff_l2(8,21)= -2.5742539763e-01
       coeff_l2(8,22)= -1.8645174801e-01
       coeff_l2(8,23)=  1.0230947286e-01
       coeff_l2(8,24)=  9.9191121757e-02
       coeff_l2(8,25)= -7.9230763018e-02
       coeff_l2(8,26)= -1.7211319506e-01
       coeff_l2(8,27)=  6.3159950078e-02
       coeff_l2(8,28)=  3.0997139215e-01
       coeff_l2(8,29)=  1.7603421211e-01
       coeff_l2(8,30)=  5.2285704762e-02
       coeff_l2(8,31)=  1.8545362353e-01
       coeff_l2(8,32)=  2.0937567949e-01
       coeff_l2(8,33)= -6.9160960615e-02
       coeff_l2(8,34)=  2.2340138257e-01
       coeff_l2(8,35)=  1.1008365154e+00
       coeff_l2(8,36)= -2.7239429951e-01
       coeff_l2(8,37)= -7.6595526934e-01
       coeff_l2(8,38)= -1.0709786788e-02
       coeff_l2(8,39)=  3.7768695503e-02
       coeff_l2(8,40)=  2.3528407514e-01
       coeff_l2(8,41)=  1.3383753598e-01
       coeff_l2(8,42)=  4.3623369187e-02
       coeff_l2(8,43)= -7.4087285995e-01
       coeff_l2(8,44)=  1.2942730682e-03
       coeff_l2(8,45)= -1.5871092677e-01
       coeff_l2(8,46)=  2.5427636504e-01
       coeff_l2(8,47)=  9.1655805707e-02
       coeff_l2(8,48)=  1.1542874575e-01
       coeff_l2(8,49)=  1.4791028202e-01
       coeff_l2(8,50)=  7.0493155718e-01
       coeff_l2(8,51)= -1.7158648372e-01
       coeff_l2(8,52)=  7.8870281577e-02
       coeff_l2(8,53)= -1.7957270145e-01
       coeff_l2(8,54)=  1.9873596728e-01
       coeff_l2(8,55)=  3.2645148039e-01
       coeff_l2(8,56)=  4.0139434859e-03
       coeff_l2(8,57)=  7.7247194946e-02
       coeff_l2(8,58)= -6.6572701931e-01
       coeff_l2(8,59)=  8.3094380796e-02
       coeff_l2(8,60)=  2.5742456317e-01
       coeff_l2(8,61)= -1.6633458436e-01
       coeff_l2(8,62)=  2.8445968032e-01
       coeff_l2(8,63)=  5.3133630753e-01
       coeff_l2(8,64)= -3.9536528289e-02
       coeff_l2(8,65)= -2.0620892942e-01
       coeff_l2(8,66)= -8.6170101166e-01
       coeff_l2(8,67)=  6.4401417971e-01
       coeff_l2(8,68)=  2.7705938555e-03
       coeff_l2(8,69)=  1.0760640353e-01
       coeff_l2(8,70)= -9.3610517681e-02
       coeff_l2(8,71)=  7.5787580013e-01
       coeff_l2(8,72)= -4.0188621730e-02
       coeff_l2(8,73)= -5.4338756949e-02
       coeff_l2(8,74)= -3.3335406333e-02
       coeff_l2(8,75)=  1.3847465813e-01
       coeff_l2(8,76)= -1.2196560204e-01
       coeff_l2(8,77)= -2.6941448450e-01
       coeff_l2(8,78)= -1.1644026963e-03
       coeff_l2(8,79)= -1.2762836218e+00
       coeff_l2(8,80)= -5.8846962452e-01
       coeff_l2(8,81)= -1.5103547275e-01
       coeff_l2(8,82)= -3.7558743358e-01
       coeff_l2(8,83)= -1.7667068541e-01
       coeff_l2(8,84)= -3.9472196251e-02
       coeff_l2(8,85)= -8.0298990011e-02
       coeff_l2(8,86)=  3.2488229871e-01
       coeff_l2(8,87)= -2.3292937875e-01
       coeff_l2(8,88)= -2.1439993754e-02
       coeff_l2(8,89)= -1.0514692962e-01
       coeff_l2(8,90)=  8.6935371161e-01
       coeff_l2(8,91)= -3.2681033015e-01
       coeff_l2(8,92)=  1.1501905322e-01
       coeff_l2(8,93)=  5.7641737163e-02
       coeff_l2(8,94)= -1.9659100473e-01
       coeff_l2(8,95)=  4.8147723079e-02
       coeff_l2(9,1)= -3.6991673708e-01
       coeff_l2(9,2)=  1.0707997531e-01
       coeff_l2(9,3)= -1.1686851084e-01
       coeff_l2(9,4)=  1.0768876225e-01
       coeff_l2(9,5)= -7.2038871050e-01
       coeff_l2(9,6)= -4.5890781283e-01
       coeff_l2(9,7)= -9.0428406000e-01
       coeff_l2(9,8)=  1.3541850448e-01
       coeff_l2(9,9)= -3.5388013721e-01
       coeff_l2(9,10)=  1.1007452011e+00
       coeff_l2(9,11)= -3.5126659274e-01
       coeff_l2(9,12)= -8.7773285806e-02
       coeff_l2(9,13)= -2.5156971067e-02
       coeff_l2(9,14)= -3.2677125931e-01
       coeff_l2(9,15)= -1.6473092139e-01
       coeff_l2(9,16)=  1.8980662571e-03
       coeff_l2(9,17)=  7.4980869889e-02
       coeff_l2(9,18)=  8.2918047905e-02
       coeff_l2(9,19)= -2.0642498136e-01
       coeff_l2(9,20)=  1.7884980142e-01
       coeff_l2(9,21)= -2.8030711412e-01
       coeff_l2(9,22)=  8.3975128829e-02
       coeff_l2(9,23)=  1.0755397379e-01
       coeff_l2(9,24)=  2.0592312515e-01
       coeff_l2(9,25)=  2.7267464995e-01
       coeff_l2(9,26)= -1.5328355134e-01
       coeff_l2(9,27)=  2.5012260675e-01
       coeff_l2(9,28)=  3.6438274384e-01
       coeff_l2(9,29)= -1.7784808576e-01
       coeff_l2(9,30)=  1.4848980308e-01
       coeff_l2(9,31)=  7.3627084494e-02
       coeff_l2(9,32)= -8.5926763713e-02
       coeff_l2(9,33)=  1.4403724670e-01
       coeff_l2(9,34)= -9.8807319999e-02
       coeff_l2(9,35)=  8.1820142269e-01
       coeff_l2(9,36)= -5.7849597931e-01
       coeff_l2(9,37)= -6.8243044615e-01
       coeff_l2(9,38)= -2.2644221783e-02
       coeff_l2(9,39)=  2.2564928234e-01
       coeff_l2(9,40)=  1.9301854074e-01
       coeff_l2(9,41)=  1.6811163723e-01
       coeff_l2(9,42)=  1.9118121266e-01
       coeff_l2(9,43)= -1.5326653421e-01
       coeff_l2(9,44)= -3.8149237633e-02
       coeff_l2(9,45)= -5.1524645090e-01
       coeff_l2(9,46)=  5.3411805630e-01
       coeff_l2(9,47)=  2.5521689653e-01
       coeff_l2(9,48)=  1.1002928764e-01
       coeff_l2(9,49)=  1.2930303812e-02
       coeff_l2(9,50)=  3.4758085012e-01
       coeff_l2(9,51)=  5.9449054301e-02
       coeff_l2(9,52)=  1.3874746859e-01
       coeff_l2(9,53)= -1.5780357644e-02
       coeff_l2(9,54)=  3.4308820963e-02
       coeff_l2(9,55)=  3.3127788454e-02
       coeff_l2(9,56)=  4.1670918465e-01
       coeff_l2(9,57)= -9.5103941858e-02
       coeff_l2(9,58)= -3.6258369684e-01
       coeff_l2(9,59)= -7.1390934289e-02
       coeff_l2(9,60)=  1.6054411232e-01
       coeff_l2(9,61)= -4.8749843240e-01
       coeff_l2(9,62)=  5.0187733024e-02
       coeff_l2(9,63)= -1.4166080952e-01
       coeff_l2(9,64)= -3.6018794775e-01
       coeff_l2(9,65)= -1.9454814494e-01
       coeff_l2(9,66)= -1.1867738515e-01
       coeff_l2(9,67)=  3.2367849350e-01
       coeff_l2(9,68)= -5.1945250481e-02
       coeff_l2(9,69)=  2.6355341077e-01
       coeff_l2(9,70)=  1.4749017358e-01
       coeff_l2(9,71)=  7.1656876802e-01
       coeff_l2(9,72)=  9.7851105034e-02
       coeff_l2(9,73)= -2.8167131543e-01
       coeff_l2(9,74)=  4.8103876412e-02
       coeff_l2(9,75)= -1.0984271765e-01
       coeff_l2(9,76)= -1.1304239929e-01
       coeff_l2(9,77)= -1.3322602212e-01
       coeff_l2(9,78)= -9.8598077893e-03
       coeff_l2(9,79)= -7.5970691442e-01
       coeff_l2(9,80)= -3.6503744125e-01
       coeff_l2(9,81)= -8.7618574500e-02
       coeff_l2(9,82)= -2.1340940893e-01
       coeff_l2(9,83)= -3.8300830126e-01
       coeff_l2(9,84)=  2.4399751425e-01
       coeff_l2(9,85)=  1.8301694095e-01
       coeff_l2(9,86)=  1.6469837725e-01
       coeff_l2(9,87)=  2.3102043569e-01
       coeff_l2(9,88)=  1.1110683531e-01
       coeff_l2(9,89)= -3.5681545734e-01
       coeff_l2(9,90)=  5.3231173754e-01
       coeff_l2(9,91)= -1.5347997844e-01
       coeff_l2(9,92)=  2.6960265636e-01
       coeff_l2(9,93)= -2.4822540581e-01
       coeff_l2(9,94)=  1.0230804794e-02
       coeff_l2(9,95)= -1.5117958188e-01
       coeff_l2(10,1)= -2.8226357698e-01
       coeff_l2(10,2)=  8.6733423173e-02
       coeff_l2(10,3)= -1.4843742549e-01
       coeff_l2(10,4)= -1.4381816983e-01
       coeff_l2(10,5)= -9.7401332855e-01
       coeff_l2(10,6)= -5.5980813503e-01
       coeff_l2(10,7)= -8.3069640398e-01
       coeff_l2(10,8)=  1.8642963469e-01
       coeff_l2(10,9)= -3.5723102093e-01
       coeff_l2(10,10)=  8.1923502684e-01
       coeff_l2(10,11)= -3.0718356371e-01
       coeff_l2(10,12)= -3.2789030671e-01
       coeff_l2(10,13)= -9.3255136162e-03
       coeff_l2(10,14)= -3.4676697850e-01
       coeff_l2(10,15)=  1.1528077303e-03
       coeff_l2(10,16)= -3.0074052513e-02
       coeff_l2(10,17)= -1.3201995194e-01
       coeff_l2(10,18)= -1.4126402326e-02
       coeff_l2(10,19)=  4.1608069092e-02
       coeff_l2(10,20)=  1.6337764263e-01
       coeff_l2(10,21)= -3.3383712173e-01
       coeff_l2(10,22)=  1.1082845740e-02
       coeff_l2(10,23)=  7.1479782462e-02
       coeff_l2(10,24)=  7.2703577578e-02
       coeff_l2(10,25)=  9.9026262760e-02
       coeff_l2(10,26)= -1.1888008565e-02
       coeff_l2(10,27)= -1.0841108114e-01
       coeff_l2(10,28)=  2.9431516305e-02
       coeff_l2(10,29)= -1.6475807130e-01
       coeff_l2(10,30)= -1.2993994355e-01
       coeff_l2(10,31)= -1.5133789182e-01
       coeff_l2(10,32)= -7.7909827232e-02
       coeff_l2(10,33)=  1.2134612352e-01
       coeff_l2(10,34)=  1.1458622664e-01
       coeff_l2(10,35)=  7.1613883972e-01
       coeff_l2(10,36)= -5.7123172283e-01
       coeff_l2(10,37)= -6.7264777422e-01
       coeff_l2(10,38)= -7.2784386575e-03
       coeff_l2(10,39)=  1.3044434786e-01
       coeff_l2(10,40)= -5.9663388878e-02
       coeff_l2(10,41)= -1.0266787559e-01
       coeff_l2(10,42)=  1.3269880414e-01
       coeff_l2(10,43)= -1.3009557128e-01
       coeff_l2(10,44)= -1.3911926746e-01
       coeff_l2(10,45)= -3.2075998187e-01
       coeff_l2(10,46)=  2.7708956599e-01
       coeff_l2(10,47)=  4.0152743459e-01
       coeff_l2(10,48)=  1.7535619438e-01
       coeff_l2(10,49)=  5.3870290518e-02
       coeff_l2(10,50)=  7.9050135612e-01
       coeff_l2(10,51)=  1.0881178826e-01
       coeff_l2(10,52)= -1.3860009611e-01
       coeff_l2(10,53)= -2.3577971756e-01
       coeff_l2(10,54)= -1.9558694959e-01
       coeff_l2(10,55)= -5.2781347185e-02
       coeff_l2(10,56)= -3.5941836238e-01
       coeff_l2(10,57)= -2.8357928991e-01
       coeff_l2(10,58)= -7.6749277115e-01
       coeff_l2(10,59)= -3.4443512559e-03
       coeff_l2(10,60)=  3.7173658609e-02
       coeff_l2(10,61)= -4.3796434999e-02
       coeff_l2(10,62)= -3.8710705936e-02
       coeff_l2(10,63)= -1.3306921721e-01
       coeff_l2(10,64)= -4.3071082234e-01
       coeff_l2(10,65)= -4.4861793518e-01
       coeff_l2(10,66)= -1.1756847054e-01
       coeff_l2(10,67)=  1.0602173209e-01
       coeff_l2(10,68)=  7.8484989703e-02
       coeff_l2(10,69)= -8.3200428635e-03
       coeff_l2(10,70)=  1.3890570030e-02
       coeff_l2(10,71)=  5.6373095512e-01
       coeff_l2(10,72)= -1.5304252505e-01
       coeff_l2(10,73)= -2.9139694571e-01
       coeff_l2(10,74)=  5.1221735775e-03
       coeff_l2(10,75)= -1.1408069730e-01
       coeff_l2(10,76)=  1.0844437778e-01
       coeff_l2(10,77)= -1.6981142759e-01
       coeff_l2(10,78)= -1.0027345270e-01
       coeff_l2(10,79)= -2.0737342536e-01
       coeff_l2(10,80)= -2.9819646478e-01
       coeff_l2(10,81)= -1.5857456625e-01
       coeff_l2(10,82)= -2.4909645319e-01
       coeff_l2(10,83)= -7.2477072477e-02
       coeff_l2(10,84)= -2.2323636338e-02
       coeff_l2(10,85)=  1.2290874869e-01
       coeff_l2(10,86)=  1.7887456715e-01
       coeff_l2(10,87)=  2.4710100889e-01
       coeff_l2(10,88)= -1.5930922329e-01
       coeff_l2(10,89)= -1.3237197697e-01
       coeff_l2(10,90)=  6.2040738761e-02
       coeff_l2(10,91)= -1.0742198676e-01
       coeff_l2(10,92)=  5.2721500397e-02
       coeff_l2(10,93)=  1.7491881549e-01
       coeff_l2(10,94)= -1.1333973706e-01
       coeff_l2(10,95)= -7.4559129775e-02
       coeff_l2(11,1)= -4.3882957101e-01
       coeff_l2(11,2)=  1.1543338746e-01
       coeff_l2(11,3)= -8.7154105306e-02
       coeff_l2(11,4)=  2.0624004304e-01
       coeff_l2(11,5)= -6.4450794458e-01
       coeff_l2(11,6)= -2.6566544175e-01
       coeff_l2(11,7)= -3.2168737054e-01
       coeff_l2(11,8)=  8.8536739349e-02
       coeff_l2(11,9)=  1.9634039700e-01
       coeff_l2(11,10)=  9.0436381102e-01
       coeff_l2(11,11)=  8.6061209440e-02
       coeff_l2(11,12)=  9.4559744000e-02
       coeff_l2(11,13)=  4.4570043683e-02
       coeff_l2(11,14)= -1.1965519935e-01
       coeff_l2(11,15)=  2.0165041089e-01
       coeff_l2(11,16)= -9.6780151129e-02
       coeff_l2(11,17)= -1.2705454230e-01
       coeff_l2(11,18)=  1.7562185228e-01
       coeff_l2(11,19)= -1.4679184556e-01
       coeff_l2(11,20)=  1.2051260471e-01
       coeff_l2(11,21)=  1.1557444930e-01
       coeff_l2(11,22)= -3.2036352158e-01
       coeff_l2(11,23)=  9.3481659889e-02
       coeff_l2(11,24)= -3.9677556604e-02
       coeff_l2(11,25)= -5.4183978587e-02
       coeff_l2(11,26)= -4.3674405664e-02
       coeff_l2(11,27)=  2.0871074498e-01
       coeff_l2(11,28)= -1.2311063707e-02
       coeff_l2(11,29)=  6.7546148784e-03
       coeff_l2(11,30)=  8.9782491326e-02
       coeff_l2(11,31)= -1.3105851412e-01
       coeff_l2(11,32)= -4.2752519250e-02
       coeff_l2(11,33)=  1.0317766666e-01
       coeff_l2(11,34)= -1.9660708308e-01
       coeff_l2(11,35)=  4.3444603682e-01
       coeff_l2(11,36)=  6.4233779907e-02
       coeff_l2(11,37)= -8.3891063929e-01
       coeff_l2(11,38)= -1.0201356560e-01
       coeff_l2(11,39)= -1.2679558992e-01
       coeff_l2(11,40)=  5.4280233599e-05
       coeff_l2(11,41)= -1.6174381971e-01
       coeff_l2(11,42)=  1.4365172386e-01
       coeff_l2(11,43)=  2.0512539148e-01
       coeff_l2(11,44)=  4.0263470262e-02
       coeff_l2(11,45)= -1.7919540405e-01
       coeff_l2(11,46)=  2.7297842503e-01
       coeff_l2(11,47)=  2.6314175129e-01
       coeff_l2(11,48)=  2.7796585113e-02
       coeff_l2(11,49)= -2.0734447241e-01
       coeff_l2(11,50)=  7.0846402645e-01
       coeff_l2(11,51)= -1.0612709820e-01
       coeff_l2(11,52)= -1.9701088965e-01
       coeff_l2(11,53)=  7.7888935804e-02
       coeff_l2(11,54)= -1.8706478179e-01
       coeff_l2(11,55)=  3.6677546799e-02
       coeff_l2(11,56)= -2.9937180877e-01
       coeff_l2(11,57)= -1.0379793495e-01
       coeff_l2(11,58)= -2.4755063653e-01
       coeff_l2(11,59)= -3.5469546914e-02
       coeff_l2(11,60)= -1.1516875587e-02
       coeff_l2(11,61)=  1.5434599482e-02
       coeff_l2(11,62)=  3.0473783612e-02
       coeff_l2(11,63)=  3.1565228105e-01
       coeff_l2(11,64)= -9.3366414309e-02
       coeff_l2(11,65)= -3.5451269150e-01
       coeff_l2(11,66)=  4.5500181615e-02
       coeff_l2(11,67)=  7.2207945585e-01
       coeff_l2(11,68)=  2.8779494762e-01
       coeff_l2(11,69)=  3.8628183305e-02
       coeff_l2(11,70)= -1.5506544150e-02
       coeff_l2(11,71)=  5.7359236479e-01
       coeff_l2(11,72)= -1.5868330002e-01
       coeff_l2(11,73)= -1.8971094489e-01
       coeff_l2(11,74)= -2.4609635770e-01
       coeff_l2(11,75)= -2.0348447561e-01
       coeff_l2(11,76)= -1.8458366394e-01
       coeff_l2(11,77)=  1.4932055771e-01
       coeff_l2(11,78)=  1.4674718678e-01
       coeff_l2(11,79)= -3.0730330944e-01
       coeff_l2(11,80)= -4.2825523019e-01
       coeff_l2(11,81)= -5.4387453943e-02
       coeff_l2(11,82)= -1.4609913528e-01
       coeff_l2(11,83)=  1.4361782372e-01
       coeff_l2(11,84)=  1.1152524501e-01
       coeff_l2(11,85)= -3.4209168516e-03
       coeff_l2(11,86)=  5.9226650745e-02
       coeff_l2(11,87)=  5.1328662783e-02
       coeff_l2(11,88)=  2.5498026609e-01
       coeff_l2(11,89)= -8.9830040932e-02
       coeff_l2(11,90)=  5.0947904587e-01
       coeff_l2(11,91)= -3.0077603459e-01
       coeff_l2(11,92)=  6.6626608372e-02
       coeff_l2(11,93)= -2.4316519499e-01
       coeff_l2(11,94)= -1.1632717401e-01
       coeff_l2(11,95)= -4.1140955873e-03
       coeff_l2(12,1)= -6.5397465229e-01
       coeff_l2(12,2)=  1.6464938223e-01
       coeff_l2(12,3)= -1.2969769537e-01
       coeff_l2(12,4)=  2.2638162971e-01
       coeff_l2(12,5)= -6.4741551876e-01
       coeff_l2(12,6)=  1.0718867183e-01
       coeff_l2(12,7)= -5.1751202345e-01
       coeff_l2(12,8)= -5.8804242872e-03
       coeff_l2(12,9)=  7.9330010340e-03
       coeff_l2(12,10)=  7.4240289629e-02
       coeff_l2(12,11)= -2.5273060799e-01
       coeff_l2(12,12)= -7.6135262847e-02
       coeff_l2(12,13)= -8.5212692618e-02
       coeff_l2(12,14)= -3.7724342942e-01
       coeff_l2(12,15)= -3.7419050932e-02
       coeff_l2(12,16)=  6.4588442445e-02
       coeff_l2(12,17)= -7.6909959316e-02
       coeff_l2(12,18)=  3.6068886518e-02
       coeff_l2(12,19)=  3.9123058319e-02
       coeff_l2(12,20)=  1.5100206435e-01
       coeff_l2(12,21)= -1.2763912976e-01
       coeff_l2(12,22)=  4.1927836835e-02
       coeff_l2(12,23)= -2.3155087605e-02
       coeff_l2(12,24)= -2.4233108759e-01
       coeff_l2(12,25)=  3.2596552372e-01
       coeff_l2(12,26)=  1.0157737881e-01
       coeff_l2(12,27)=  1.8014456332e-01
       coeff_l2(12,28)= -7.8500777483e-02
       coeff_l2(12,29)= -1.6275364161e-01
       coeff_l2(12,30)= -7.6951280236e-02
       coeff_l2(12,31)=  1.7150445282e-01
       coeff_l2(12,32)= -9.0303190053e-02
       coeff_l2(12,33)=  3.3587506413e-01
       coeff_l2(12,34)=  1.4593298733e-01
       coeff_l2(12,35)=  7.3222571611e-01
       coeff_l2(12,36)= -3.3583042026e-01
       coeff_l2(12,37)= -4.8934358358e-01
       coeff_l2(12,38)=  1.1982146651e-01
       coeff_l2(12,39)=  1.3992729783e-01
       coeff_l2(12,40)=  1.6617996991e-01
       coeff_l2(12,41)= -9.3809708953e-02
       coeff_l2(12,42)= -3.2073076814e-02
       coeff_l2(12,43)= -1.5084886551e-01
       coeff_l2(12,44)= -2.1556118131e-01
       coeff_l2(12,45)= -2.5289648771e-01
       coeff_l2(12,46)=  2.8729566932e-01
       coeff_l2(12,47)=  3.1197169423e-01
       coeff_l2(12,48)=  9.7297675908e-02
       coeff_l2(12,49)=  1.2848901749e-01
       coeff_l2(12,50)=  9.8392671347e-01
       coeff_l2(12,51)= -6.6949062049e-02
       coeff_l2(12,52)= -4.6280919923e-04
       coeff_l2(12,53)= -3.3171996474e-01
       coeff_l2(12,54)= -1.3858160377e-01
       coeff_l2(12,55)=  2.2212207317e-01
       coeff_l2(12,56)= -9.9471487105e-02
       coeff_l2(12,57)= -2.6273843646e-01
       coeff_l2(12,58)= -5.7598614693e-01
       coeff_l2(12,59)=  7.1186363697e-02
       coeff_l2(12,60)=  1.5609706938e-01
       coeff_l2(12,61)= -1.8014834821e-01
       coeff_l2(12,62)=  2.3895294964e-01
       coeff_l2(12,63)=  2.4286779761e-01
       coeff_l2(12,64)= -3.8060727715e-01
       coeff_l2(12,65)= -1.4902813733e-01
       coeff_l2(12,66)= -4.1278833151e-01
       coeff_l2(12,67)= -4.7231946141e-02
       coeff_l2(12,68)= -1.9839107990e-01
       coeff_l2(12,69)= -1.6244795918e-01
       coeff_l2(12,70)=  1.5139904618e-01
       coeff_l2(12,71)=  4.4315898418e-01
       coeff_l2(12,72)= -7.4766807258e-02
       coeff_l2(12,73)= -1.8356388807e-01
       coeff_l2(12,74)=  3.0996929854e-02
       coeff_l2(12,75)=  8.8919699192e-02
       coeff_l2(12,76)=  3.5395585001e-02
       coeff_l2(12,77)= -2.3034839332e-01
       coeff_l2(12,78)= -1.0424242914e-01
       coeff_l2(12,79)= -6.1827427149e-01
       coeff_l2(12,80)= -3.0878046155e-01
       coeff_l2(12,81)=  5.6023266166e-02
       coeff_l2(12,82)= -1.2525884807e-01
       coeff_l2(12,83)= -2.3053675890e-01
       coeff_l2(12,84)= -9.3191288412e-02
       coeff_l2(12,85)= -2.5902071595e-01
       coeff_l2(12,86)=  2.7026006579e-01
       coeff_l2(12,87)= -7.3637396097e-02
       coeff_l2(12,88)= -7.6305888593e-02
       coeff_l2(12,89)= -3.3949059248e-01
       coeff_l2(12,90)=  2.7451291680e-01
       coeff_l2(12,91)=  2.4668358266e-02
       coeff_l2(12,92)= -1.1964745820e-01
       coeff_l2(12,93)= -4.0955647826e-01
       coeff_l2(12,94)= -2.0594868809e-02
       coeff_l2(12,95)= -5.7193610817e-02
       coeff_l2(13,1)= -4.2126244307e-01
       coeff_l2(13,2)= -1.2614072859e-01
       coeff_l2(13,3)=  3.5729847848e-02
       coeff_l2(13,4)=  7.3947295547e-02
       coeff_l2(13,5)= -7.6517707109e-01
       coeff_l2(13,6)= -5.2433490753e-01
       coeff_l2(13,7)= -6.2513786554e-01
       coeff_l2(13,8)=  3.0491571873e-02
       coeff_l2(13,9)= -1.3772100210e-01
       coeff_l2(13,10)=  7.1098548174e-01
       coeff_l2(13,11)= -8.4642030299e-02
       coeff_l2(13,12)= -7.8146755695e-02
       coeff_l2(13,13)= -1.8274065852e-01
       coeff_l2(13,14)= -1.8836365640e-01
       coeff_l2(13,15)= -2.8479433060e-01
       coeff_l2(13,16)= -7.7010460198e-02
       coeff_l2(13,17)= -9.5569677651e-02
       coeff_l2(13,18)= -1.4075449109e-01
       coeff_l2(13,19)= -2.4782903492e-01
       coeff_l2(13,20)= -1.8312084675e-01
       coeff_l2(13,21)= -4.8941078782e-01
       coeff_l2(13,22)= -3.1034505367e-01
       coeff_l2(13,23)= -1.8319502473e-02
       coeff_l2(13,24)= -1.6043478251e-01
       coeff_l2(13,25)=  2.2461931407e-01
       coeff_l2(13,26)= -2.7405408025e-01
       coeff_l2(13,27)=  2.0743523538e-01
       coeff_l2(13,28)=  9.4814233482e-02
       coeff_l2(13,29)= -5.3802886978e-03
       coeff_l2(13,30)= -8.4331825376e-02
       coeff_l2(13,31)=  1.5180559456e-01
       coeff_l2(13,32)= -6.2374252826e-02
       coeff_l2(13,33)=  2.5429630280e-01
       coeff_l2(13,34)=  1.7834565043e-01
       coeff_l2(13,35)=  9.4882065058e-01
       coeff_l2(13,36)= -6.0661268234e-01
       coeff_l2(13,37)= -6.9016849995e-01
       coeff_l2(13,38)=  1.4010421932e-01
       coeff_l2(13,39)=  3.8109049201e-01
       coeff_l2(13,40)= -5.6559398770e-02
       coeff_l2(13,41)=  9.5059890300e-03
       coeff_l2(13,42)=  6.1236552894e-02
       coeff_l2(13,43)= -3.1498026848e-01
       coeff_l2(13,44)= -3.2153706998e-02
       coeff_l2(13,45)=  9.4545863569e-02
       coeff_l2(13,46)=  4.7176048160e-01
       coeff_l2(13,47)=  3.6958634853e-01
       coeff_l2(13,48)= -6.4083211124e-02
       coeff_l2(13,49)= -3.0249893665e-01
       coeff_l2(13,50)=  1.0701390505e+00
       coeff_l2(13,51)=  6.8200297654e-02
       coeff_l2(13,52)= -1.7203089595e-01
       coeff_l2(13,53)= -3.5583582520e-01
       coeff_l2(13,54)= -2.6144811418e-03
       coeff_l2(13,55)=  4.3360337615e-02
       coeff_l2(13,56)= -2.3130671680e-01
       coeff_l2(13,57)= -4.1040855646e-01
       coeff_l2(13,58)= -2.5437670946e-01
       coeff_l2(13,59)= -5.4561562836e-02
       coeff_l2(13,60)=  1.1524639279e-01
       coeff_l2(13,61)= -3.6456984282e-01
       coeff_l2(13,62)=  4.4221159816e-01
       coeff_l2(13,63)=  3.3025535941e-01
       coeff_l2(13,64)= -3.9162209630e-01
       coeff_l2(13,65)= -4.7042530775e-01
       coeff_l2(13,66)= -5.5285471678e-01
       coeff_l2(13,67)=  2.0973199606e-01
       coeff_l2(13,68)=  1.6616731882e-01
       coeff_l2(13,69)= -1.9263939559e-01
       coeff_l2(13,70)=  1.8513004482e-01
       coeff_l2(13,71)=  5.0194728374e-01
       coeff_l2(13,72)= -1.2609376013e-01
       coeff_l2(13,73)= -2.6734309271e-02
       coeff_l2(13,74)= -2.7623111010e-01
       coeff_l2(13,75)= -3.5467767715e-01
       coeff_l2(13,76)=  1.1195127852e-02
       coeff_l2(13,77)= -2.6338672638e-01
       coeff_l2(13,78)=  1.8887962401e-01
       coeff_l2(13,79)= -9.1885560751e-01
       coeff_l2(13,80)= -7.1113950014e-01
       coeff_l2(13,81)= -1.0410983115e-01
       coeff_l2(13,82)=  2.6573132724e-02
       coeff_l2(13,83)= -2.9031747580e-01
       coeff_l2(13,84)=  7.5476996601e-02
       coeff_l2(13,85)= -2.7504169941e-01
       coeff_l2(13,86)=  8.5897855461e-02
       coeff_l2(13,87)= -5.3332466632e-02
       coeff_l2(13,88)=  1.0701560229e-01
       coeff_l2(13,89)= -5.4407727718e-01
       coeff_l2(13,90)=  5.0731414557e-01
       coeff_l2(13,91)=  3.1208617147e-03
       coeff_l2(13,92)=  3.2589465380e-01
       coeff_l2(13,93)= -3.3268037438e-01
       coeff_l2(13,94)=  8.6727470160e-02
       coeff_l2(13,95)= -2.0896967500e-02
       coeff_l2(14,1)= -4.6965651214e-02
       coeff_l2(14,2)= -6.8593166769e-02
       coeff_l2(14,3)= -6.0729160905e-02
       coeff_l2(14,4)=  2.3760260642e-01
       coeff_l2(14,5)=  9.8978810012e-02
       coeff_l2(14,6)=  6.3028596342e-02
       coeff_l2(14,7)= -1.5066726506e-01
       coeff_l2(14,8)= -2.0572496578e-02
       coeff_l2(14,9)= -5.2779890597e-02
       coeff_l2(14,10)= -7.8491806984e-02
       coeff_l2(14,11)= -4.5703988522e-02
       coeff_l2(14,12)= -5.3994930349e-03
       coeff_l2(14,13)= -2.5232839584e-01
       coeff_l2(14,14)= -3.2273247838e-02
       coeff_l2(14,15)= -1.6461083293e-01
       coeff_l2(14,16)= -1.5568254888e-01
       coeff_l2(14,17)= -1.0681254417e-01
       coeff_l2(14,18)= -7.6283611357e-02
       coeff_l2(14,19)= -1.6821214929e-02
       coeff_l2(14,20)= -1.5432623029e-01
       coeff_l2(14,21)= -3.0849575996e-01
       coeff_l2(14,22)=  5.5352020264e-01
       coeff_l2(14,23)= -2.1469368041e-01
       coeff_l2(14,24)=  7.4509218335e-02
       coeff_l2(14,25)=  3.0408397317e-01
       coeff_l2(14,26)=  6.3703425229e-02
       coeff_l2(14,27)=  3.5559546202e-02
       coeff_l2(14,28)=  1.2596850097e-01
       coeff_l2(14,29)= -1.4036001265e-01
       coeff_l2(14,30)=  1.5914877877e-02
       coeff_l2(14,31)=  6.9561645389e-02
       coeff_l2(14,32)= -6.5465457737e-02
       coeff_l2(14,33)=  9.3348249793e-02
       coeff_l2(14,34)=  2.5590706617e-02
       coeff_l2(14,35)=  5.3722918034e-01
       coeff_l2(14,36)= -7.0954006910e-01
       coeff_l2(14,37)= -1.2817107141e-01
       coeff_l2(14,38)= -7.4757680297e-02
       coeff_l2(14,39)=  4.3756383657e-01
       coeff_l2(14,40)=  2.8317934275e-01
       coeff_l2(14,41)= -2.3719411343e-02
       coeff_l2(14,42)=  9.2278920114e-02
       coeff_l2(14,43)= -1.4616438746e-01
       coeff_l2(14,44)=  2.0206336677e-01
       coeff_l2(14,45)=  2.1262164414e-01
       coeff_l2(14,46)=  1.7414079607e-01
       coeff_l2(14,47)=  3.1002584100e-01
       coeff_l2(14,48)=  1.0197113268e-02
       coeff_l2(14,49)=  2.0638951659e-01
       coeff_l2(14,50)= -1.0092581064e-01
       coeff_l2(14,51)= -1.6848747432e-01
       coeff_l2(14,52)= -5.2251674235e-02
       coeff_l2(14,53)= -8.5533499718e-02
       coeff_l2(14,54)=  1.6627643257e-02
       coeff_l2(14,55)=  2.7148491144e-01
       coeff_l2(14,56)= -4.1345607489e-02
       coeff_l2(14,57)=  1.1141820997e-01
       coeff_l2(14,58)= -2.6324100792e-02
       coeff_l2(14,59)= -2.9139694571e-01
       coeff_l2(14,60)=  1.3293591142e-01
       coeff_l2(14,61)=  3.7993717939e-02
       coeff_l2(14,62)=  2.8043842316e-01
       coeff_l2(14,63)=  1.7749033868e-01
       coeff_l2(14,64)= -9.8750546575e-02
       coeff_l2(14,65)= -6.3147477806e-02
       coeff_l2(14,66)= -6.4714848995e-02
       coeff_l2(14,67)= -2.0419947803e-01
       coeff_l2(14,68)=  1.4879010618e-01
       coeff_l2(14,69)= -2.4203313887e-01
       coeff_l2(14,70)=  1.8488460779e-01
       coeff_l2(14,71)=  2.0140369236e-01
       coeff_l2(14,72)= -4.7911665752e-04
       coeff_l2(14,73)=  2.1453523636e-01
       coeff_l2(14,74)=  2.2607752681e-01
       coeff_l2(14,75)=  1.7121300101e-01
       coeff_l2(14,76)=  2.6631936431e-02
       coeff_l2(14,77)=  2.0910598338e-01
       coeff_l2(14,78)=  1.6513569653e-01
       coeff_l2(14,79)= -2.9619963607e-04
       coeff_l2(14,80)= -4.3822717667e-01
       coeff_l2(14,81)=  1.3215398788e-01
       coeff_l2(14,82)=  2.3259979486e-01
       coeff_l2(14,83)=  2.7426969260e-02
       coeff_l2(14,84)=  5.9067144990e-02
       coeff_l2(14,85)= -9.7548201680e-02
       coeff_l2(14,86)=  4.4150125235e-02
       coeff_l2(14,87)= -2.1135893464e-01
       coeff_l2(14,88)=  1.8353113532e-01
       coeff_l2(14,89)=  1.3628408313e-01
       coeff_l2(14,90)=  4.5661273599e-01
       coeff_l2(14,91)= -1.8963269889e-01
       coeff_l2(14,92)=  3.7371951342e-01
       coeff_l2(14,93)= -2.4555151165e-01
       coeff_l2(14,94)= -1.7533768713e-01
       coeff_l2(14,95)= -1.1733983457e-01
       coeff_l2(15,1)= -9.5281475782e-01
       coeff_l2(15,2)= -1.2025672942e-01
       coeff_l2(15,3)= -3.9644619823e-01
       coeff_l2(15,4)=  2.0315162838e-01
       coeff_l2(15,5)= -8.4187757969e-01
       coeff_l2(15,6)= -5.0573444366e-01
       coeff_l2(15,7)= -7.8962129354e-01
       coeff_l2(15,8)= -3.8712298870e-01
       coeff_l2(15,9)= -1.9168601930e-01
       coeff_l2(15,10)=  1.6875689030e+00
       coeff_l2(15,11)= -3.2229843736e-01
       coeff_l2(15,12)=  5.9054415673e-02
       coeff_l2(15,13)= -2.1314331889e-01
       coeff_l2(15,14)= -3.0407553911e-01
       coeff_l2(15,15)= -3.5689690709e-01
       coeff_l2(15,16)= -1.3854998350e-01
       coeff_l2(15,17)= -2.0100943744e-01
       coeff_l2(15,18)= -1.3458625972e-01
       coeff_l2(15,19)=  8.7227351964e-02
       coeff_l2(15,20)= -6.1712965369e-02
       coeff_l2(15,21)= -2.2494003177e-01
       coeff_l2(15,22)= -5.9030598402e-01
       coeff_l2(15,23)=  7.2793692350e-02
       coeff_l2(15,24)= -1.0142122954e-01
       coeff_l2(15,25)=  4.5324674249e-01
       coeff_l2(15,26)= -4.7300702333e-01
       coeff_l2(15,27)= -1.0837416351e-01
       coeff_l2(15,28)= -2.9986467957e-01
       coeff_l2(15,29)= -1.6127018631e-01
       coeff_l2(15,30)=  2.8214398772e-02
       coeff_l2(15,31)=  3.7006744742e-01
       coeff_l2(15,32)=  8.8549531996e-02
       coeff_l2(15,33)=  3.0310434103e-01
       coeff_l2(15,34)= -3.2590001822e-01
       coeff_l2(15,35)=  1.1827303171e+00
       coeff_l2(15,36)= -3.8866487145e-01
       coeff_l2(15,37)= -1.0765399933e+00
       coeff_l2(15,38)= -1.7306540161e-02
       coeff_l2(15,39)=  7.9155170918e-01
       coeff_l2(15,40)=  2.7852207422e-01
       coeff_l2(15,41)= -3.1754910946e-01
       coeff_l2(15,42)= -2.6082357392e-02
       coeff_l2(15,43)= -3.1130549312e-01
       coeff_l2(15,44)=  1.5700578690e-01
       coeff_l2(15,45)= -3.0529314280e-01
       coeff_l2(15,46)=  5.4060161114e-01
       coeff_l2(15,47)=  4.5000383258e-01
       coeff_l2(15,48)= -1.8592759967e-01
       coeff_l2(15,49)= -3.3989757299e-02
       coeff_l2(15,50)=  1.1696486473e+00
       coeff_l2(15,51)= -2.2721163929e-02
       coeff_l2(15,52)= -9.2057853937e-02
       coeff_l2(15,53)= -1.4759381115e-01
       coeff_l2(15,54)= -2.1383088082e-02
       coeff_l2(15,55)=  2.6954925060e-01
       coeff_l2(15,56)= -2.8592604399e-01
       coeff_l2(15,57)= -4.7277435660e-01
       coeff_l2(15,58)= -9.8983687162e-01
       coeff_l2(15,59)= -1.9456924498e-01
       coeff_l2(15,60)= -1.1427102610e-02
       coeff_l2(15,61)= -2.1293114126e-01
       coeff_l2(15,62)=  2.0232313871e-01
       coeff_l2(15,63)=  5.9334385395e-01
       coeff_l2(15,64)= -2.2403188050e-01
       coeff_l2(15,65)= -2.6608353853e-01
       coeff_l2(15,66)= -1.3058272004e-01
       coeff_l2(15,67)=  5.1800012589e-01
       coeff_l2(15,68)=  1.6043409705e-01
       coeff_l2(15,69)= -2.3367974162e-01
       coeff_l2(15,70)= -1.2303020060e-01
       coeff_l2(15,71)=  1.1282154322e+00
       coeff_l2(15,72)= -4.3634831905e-02
       coeff_l2(15,73)= -7.0497977734e-01
       coeff_l2(15,74)= -1.5707345307e-01
       coeff_l2(15,75)= -1.0084882379e-02
       coeff_l2(15,76)= -2.3641254008e-01
       coeff_l2(15,77)= -1.0001792759e-01
       coeff_l2(15,78)= -2.7091277763e-02
       coeff_l2(15,79)= -1.2677258253e+00
       coeff_l2(15,80)= -6.5367025137e-01
       coeff_l2(15,81)= -3.8771873713e-01
       coeff_l2(15,82)= -6.5024173260e-01
       coeff_l2(15,83)= -2.3629550636e-01
       coeff_l2(15,84)= -4.3744927645e-01
       coeff_l2(15,85)= -2.8136268258e-01
       coeff_l2(15,86)=  3.6470514536e-01
       coeff_l2(15,87)= -1.0505932570e-01
       coeff_l2(15,88)=  3.7485186011e-02
       coeff_l2(15,89)=  3.1308519840e-01
       coeff_l2(15,90)=  6.9036942720e-01
       coeff_l2(15,91)= -3.8485798985e-02
       coeff_l2(15,92)=  1.2818491086e-03
       coeff_l2(15,93)= -3.8252806664e-01
       coeff_l2(15,94)= -1.2507389486e-01
       coeff_l2(15,95)= -3.4019258618e-01
       coeff_l2(16,1)=  2.0080441236e-01
       coeff_l2(16,2)=  6.7051172256e-01
       coeff_l2(16,3)= -2.4778391421e-01
       coeff_l2(16,4)=  2.5933387280e+00
       coeff_l2(16,5)=  9.9782878160e-01
       coeff_l2(16,6)=  1.1012499332e+00
       coeff_l2(16,7)=  1.4372050762e-01
       coeff_l2(16,8)= -9.9437668920e-02
       coeff_l2(16,9)= -5.5245388299e-02
       coeff_l2(16,10)= -1.2442885637e+00
       coeff_l2(16,11)= -1.8450596929e-01
       coeff_l2(16,12)= -5.0133228302e-02
       coeff_l2(16,13)= -1.2182291597e-01
       coeff_l2(16,14)=  1.4624693990e-01
       coeff_l2(16,15)= -6.0021061450e-02
       coeff_l2(16,16)= -6.2724786997e-01
       coeff_l2(16,17)=  4.3604258448e-02
       coeff_l2(16,18)= -1.0672059655e-01
       coeff_l2(16,19)= -1.1059013009e-01
       coeff_l2(16,20)= -1.1504165083e-01
       coeff_l2(16,21)= -1.1429739743e-01
       coeff_l2(16,22)= -2.7377057076e-01
       coeff_l2(16,23)=  5.3807966411e-02
       coeff_l2(16,24)=  5.7419657707e-02
       coeff_l2(16,25)= -4.4218224287e-01
       coeff_l2(16,26)=  8.4747201204e-01
       coeff_l2(16,27)=  5.3586743772e-02
       coeff_l2(16,28)=  8.3960127831e-01
       coeff_l2(16,29)= -6.6246166825e-02
       coeff_l2(16,30)= -2.8382448480e-02
       coeff_l2(16,31)=  1.1349158287e+00
       coeff_l2(16,32)=  1.2622475624e-01
       coeff_l2(16,33)= -1.3858982921e-01
       coeff_l2(16,34)= -1.1808983982e-01
       coeff_l2(16,35)=  1.2845940590e+00
       coeff_l2(16,36)= -1.9917882979e-01
       coeff_l2(16,37)=  1.0525517464e+00
       coeff_l2(16,38)= -1.7912776768e-01
       coeff_l2(16,39)=  3.8371029496e-01
       coeff_l2(16,40)=  9.2672652006e-01
       coeff_l2(16,41)=  2.1140547469e-02
       coeff_l2(16,42)= -5.0312589854e-02
       coeff_l2(16,43)= -8.6526858807e-01
       coeff_l2(16,44)= -3.9715611935e-01
       coeff_l2(16,45)=  2.3339329660e-01
       coeff_l2(16,46)= -5.2909773588e-01
       coeff_l2(16,47)= -6.3825976849e-01
       coeff_l2(16,48)= -4.5480057597e-02
       coeff_l2(16,49)= -5.1344288513e-03
       coeff_l2(16,50)=  2.8260430694e-01
       coeff_l2(16,51)= -3.1112823635e-02
       coeff_l2(16,52)=  6.2590323389e-02
       coeff_l2(16,53)= -9.1921137646e-03
       coeff_l2(16,54)= -2.0451268554e-01
       coeff_l2(16,55)=  1.5383696556e+00
       coeff_l2(16,56)= -2.3102645576e-01
       coeff_l2(16,57)=  1.9234217703e-01
       coeff_l2(16,58)= -2.1265719086e-02
       coeff_l2(16,59)= -7.9032689333e-02
       coeff_l2(16,60)= -2.0131525397e-01
       coeff_l2(16,61)=  2.0127162337e-01
       coeff_l2(16,62)= -4.7258172184e-02
       coeff_l2(16,63)=  3.0702927113e+00
       coeff_l2(16,64)=  1.4750878811e+00
       coeff_l2(16,65)=  9.2191964388e-01
       coeff_l2(16,66)= -1.1486247778e+00
       coeff_l2(16,67)= -6.3772386312e-01
       coeff_l2(16,68)= -5.5650717020e-01
       coeff_l2(16,69)= -1.2463872135e-01
       coeff_l2(16,70)=  2.0838009194e-02
       coeff_l2(16,71)= -1.0674341917e+00
       coeff_l2(16,72)= -7.8467570245e-02
       coeff_l2(16,73)=  1.6764113307e-01
       coeff_l2(16,74)= -5.9097480029e-02
       coeff_l2(16,75)= -5.5842351168e-02
       coeff_l2(16,76)= -1.2879215181e-01
       coeff_l2(16,77)= -3.5619229078e-01
       coeff_l2(16,78)= -8.6525596678e-02
       coeff_l2(16,79)= -1.1357048750e+00
       coeff_l2(16,80)= -5.7965803891e-02
       coeff_l2(16,81)= -1.0319671780e-01
       coeff_l2(16,82)=  2.8592148423e-01
       coeff_l2(16,83)=  1.8280138075e-01
       coeff_l2(16,84)= -1.6401906312e-01
       coeff_l2(16,85)=  1.0588854551e-01
       coeff_l2(16,86)=  1.1977300793e-01
       coeff_l2(16,87)= -2.1795388311e-02
       coeff_l2(16,88)=  4.0038142353e-02
       coeff_l2(16,89)= -3.4905180335e-01
       coeff_l2(16,90)= -1.9160194695e-01
       coeff_l2(16,91)= -2.8502225876e-01
       coeff_l2(16,92)=  2.0328554511e-01
       coeff_l2(16,93)= -5.4663449526e-02
       coeff_l2(16,94)=  1.2982279062e-01
       coeff_l2(16,95)= -2.6185154915e-01
       coeff_l2(17,1)= -1.2855837345e+00
       coeff_l2(17,2)= -1.2619949877e-01
       coeff_l2(17,3)= -1.2481707335e+00
       coeff_l2(17,4)=  2.6514039040e+00
       coeff_l2(17,5)= -2.8036639094e-01
       coeff_l2(17,6)=  1.0579886436e+00
       coeff_l2(17,7)= -1.4374690056e+00
       coeff_l2(17,8)= -7.2020769119e-02
       coeff_l2(17,9)= -6.8081550300e-02
       coeff_l2(17,10)=  9.9021816254e-01
       coeff_l2(17,11)= -8.5207051039e-01
       coeff_l2(17,12)= -2.6749983430e-01
       coeff_l2(17,13)= -8.4285700321e-01
       coeff_l2(17,14)= -5.0235712528e-01
       coeff_l2(17,15)= -5.4141694307e-01
       coeff_l2(17,16)= -5.4986065626e-01
       coeff_l2(17,17)= -3.4690716863e-01
       coeff_l2(17,18)=  2.8092296794e-02
       coeff_l2(17,19)= -1.0291702300e-01
       coeff_l2(17,20)=  4.1867172718e-01
       coeff_l2(17,21)= -6.4276593924e-01
       coeff_l2(17,22)= -5.5935633183e-01
       coeff_l2(17,23)= -4.6986693144e-01
       coeff_l2(17,24)= -2.5895851851e-01
       coeff_l2(17,25)= -6.0173618793e-01
       coeff_l2(17,26)= -1.6689287126e-01
       coeff_l2(17,27)= -1.1598407477e-01
       coeff_l2(17,28)=  1.7504751682e+00
       coeff_l2(17,29)= -1.6808342934e-01
       coeff_l2(17,30)= -1.7365844548e-01
       coeff_l2(17,31)=  1.2398666143e+00
       coeff_l2(17,32)= -1.8555007875e-01
       coeff_l2(17,33)=  1.3618689030e-02
       coeff_l2(17,34)= -1.0422629863e-01
       coeff_l2(17,35)=  2.8432800770e+00
       coeff_l2(17,36)=  2.5673627853e-01
       coeff_l2(17,37)= -1.1676454544e+00
       coeff_l2(17,38)= -1.9454368949e-01
       coeff_l2(17,39)=  7.9085677862e-01
       coeff_l2(17,40)=  7.8946244717e-01
       coeff_l2(17,41)= -1.5814116597e-01
       coeff_l2(17,42)= -1.2774516642e-01
       coeff_l2(17,43)= -2.3815132678e-01
       coeff_l2(17,44)= -4.6435683966e-01
       coeff_l2(17,45)=  6.5255439281e-01
       coeff_l2(17,46)=  2.3488280177e-01
       coeff_l2(17,47)=  5.9079825878e-01
       coeff_l2(17,48)= -2.0054681599e-01
       coeff_l2(17,49)= -3.3100447059e-01
       coeff_l2(17,50)=  2.5926864147e+00
       coeff_l2(17,51)= -6.9824582338e-01
       coeff_l2(17,52)=  6.8024318898e-04
       coeff_l2(17,53)=  2.0344065130e-01
       coeff_l2(17,54)= -1.8125909567e-01
       coeff_l2(17,55)=  9.2785650492e-01
       coeff_l2(17,56)= -2.3413573205e-01
       coeff_l2(17,57)= -9.0951263905e-01
       coeff_l2(17,58)= -7.8291648626e-01
       coeff_l2(17,59)= -1.3131538033e-01
       coeff_l2(17,60)=  1.5142292250e-04
       coeff_l2(17,61)= -8.6686819792e-02
       coeff_l2(17,62)= -8.6456304789e-01
       coeff_l2(17,63)=  4.7581763268e+00
       coeff_l2(17,64)=  8.6648470163e-01
       coeff_l2(17,65)= -6.0791844130e-01
       coeff_l2(17,66)= -4.7002771497e-01
       coeff_l2(17,67)=  1.7691563368e+00
       coeff_l2(17,68)=  3.3432906866e-01
       coeff_l2(17,69)= -4.4966313243e-01
       coeff_l2(17,70)=  2.7405112982e-01
       coeff_l2(17,71)=  1.9497224092e+00
       coeff_l2(17,72)= -4.7763055563e-01
       coeff_l2(17,73)= -4.6665659547e-01
       coeff_l2(17,74)= -2.5896117091e-01
       coeff_l2(17,75)= -2.1856917441e-01
       coeff_l2(17,76)= -3.4351763129e-01
       coeff_l2(17,77)= -2.3408403993e-01
       coeff_l2(17,78)= -3.4209847450e-01
       coeff_l2(17,79)= -3.0128178596e+00
       coeff_l2(17,80)= -1.2467775345e+00
       coeff_l2(17,81)= -1.3405658305e-01
       coeff_l2(17,82)=  1.1405752972e-02
       coeff_l2(17,83)= -1.6695155203e-01
       coeff_l2(17,84)= -8.8841393590e-02
       coeff_l2(17,85)= -1.3271327317e-01
       coeff_l2(17,86)= -1.6889691353e-01
       coeff_l2(17,87)= -5.1321405172e-01
       coeff_l2(17,88)=  4.0675002337e-01
       coeff_l2(17,89)= -7.7762466669e-01
       coeff_l2(17,90)=  3.0779516697e+00
       coeff_l2(17,91)= -4.5629061759e-02
       coeff_l2(17,92)=  1.1904144287e+00
       coeff_l2(17,93)= -2.8167578578e-01
       coeff_l2(17,94)= -1.5092162788e-01
       coeff_l2(17,95)= -3.5733762383e-01
       coeff_l2(18,1)=  4.3693518639e-01
       coeff_l2(18,2)= -3.2029804587e-01
       coeff_l2(18,3)= -2.4913156033e-01
       coeff_l2(18,4)=  1.1718282849e-01
       coeff_l2(18,5)=  7.0875024796e-01
       coeff_l2(18,6)= -4.1214872152e-02
       coeff_l2(18,7)=  1.1881420016e-01
       coeff_l2(18,8)= -5.0481718779e-01
       coeff_l2(18,9)= -1.0075693019e-02
       coeff_l2(18,10)=  7.8680342436e-01
       coeff_l2(18,11)= -5.6924704462e-02
       coeff_l2(18,12)=  1.8923898041e-01
       coeff_l2(18,13)= -2.9420548677e-01
       coeff_l2(18,14)= -3.2868146896e-02
       coeff_l2(18,15)= -1.6597010195e-01
       coeff_l2(18,16)= -9.8068807274e-03
       coeff_l2(18,17)=  7.9691484571e-02
       coeff_l2(18,18)=  8.6661502719e-02
       coeff_l2(18,19)= -1.3507506251e-01
       coeff_l2(18,20)= -6.1201835051e-03
       coeff_l2(18,21)=  2.8757345676e-01
       coeff_l2(18,22)= -5.3044050932e-02
       coeff_l2(18,23)=  9.4139084220e-02
       coeff_l2(18,24)=  6.3664957881e-02
       coeff_l2(18,25)= -3.0031384900e-02
       coeff_l2(18,26)= -2.4744930863e-01
       coeff_l2(18,27)=  7.0960216224e-02
       coeff_l2(18,28)= -1.4602780342e-01
       coeff_l2(18,29)= -2.7617180347e-01
       coeff_l2(18,30)= -2.4774497375e-02
       coeff_l2(18,31)=  3.1666796654e-02
       coeff_l2(18,32)=  2.2485041618e-01
       coeff_l2(18,33)=  1.5251326561e-01
       coeff_l2(18,34)= -5.7485401630e-01
       coeff_l2(18,35)=  5.4622405767e-01
       coeff_l2(18,36)=  5.2754646540e-01
       coeff_l2(18,37)=  4.2541630566e-02
       coeff_l2(18,38)= -3.0470192432e-01
       coeff_l2(18,39)=  1.8721972406e-01
       coeff_l2(18,40)= -1.1069543660e-01
       coeff_l2(18,41)=  1.3788680732e-01
       coeff_l2(18,42)= -3.5741221160e-02
       coeff_l2(18,43)=  3.4138774872e-01
       coeff_l2(18,44)= -7.8978441656e-02
       coeff_l2(18,45)= -1.4648546278e-01
       coeff_l2(18,46)=  2.6062038541e-01
       coeff_l2(18,47)=  1.6834853590e-01
       coeff_l2(18,48)= -1.8134315312e-01
       coeff_l2(18,49)= -3.3484447002e-01
       coeff_l2(18,50)=  4.5638528466e-01
       coeff_l2(18,51)= -3.7901270390e-01
       coeff_l2(18,52)= -2.8575393558e-01
       coeff_l2(18,53)=  1.3399396837e-01
       coeff_l2(18,54)=  8.7885893881e-02
       coeff_l2(18,55)=  5.1947373897e-02
       coeff_l2(18,56)= -1.9320707023e-01
       coeff_l2(18,57)= -3.0680570006e-01
       coeff_l2(18,58)=  1.0463406891e-01
       coeff_l2(18,59)= -1.8035943806e-01
       coeff_l2(18,60)= -1.0363183916e-01
       coeff_l2(18,61)= -7.5376577675e-02
       coeff_l2(18,62)=  1.5609456599e-01
       coeff_l2(18,63)=  6.3586093485e-02
       coeff_l2(18,64)=  9.4667309523e-01
       coeff_l2(18,65)= -3.4065985680e-01
       coeff_l2(18,66)=  1.0682889223e+00
       coeff_l2(18,67)=  8.2492113113e-01
       coeff_l2(18,68)=  4.2230424285e-01
       coeff_l2(18,69)= -2.1610578895e-01
       coeff_l2(18,70)= -1.8946882337e-02
       coeff_l2(18,71)=  1.4041881263e-01
       coeff_l2(18,72)= -1.3596986234e-01
       coeff_l2(18,73)= -3.5355064273e-01
       coeff_l2(18,74)=  1.9567394257e-01
       coeff_l2(18,75)= -1.0586503893e-01
       coeff_l2(18,76)= -3.8162365556e-02
       coeff_l2(18,77)=  4.0215498209e-01
       coeff_l2(18,78)= -4.1786739230e-01
       coeff_l2(18,79)=  1.5590961277e-01
       coeff_l2(18,80)=  5.7684254646e-01
       coeff_l2(18,81)= -9.5016136765e-02
       coeff_l2(18,82)= -3.3448606730e-01
       coeff_l2(18,83)=  1.5293513238e-01
       coeff_l2(18,84)= -2.6570436358e-01
       coeff_l2(18,85)= -3.6328282952e-01
       coeff_l2(18,86)=  6.9853350520e-02
       coeff_l2(18,87)= -2.6651343703e-01
       coeff_l2(18,88)=  2.8719512746e-02
       coeff_l2(18,89)= -2.7672788501e-01
       coeff_l2(18,90)=  1.5780667961e-01
       coeff_l2(18,91)= -4.1749766469e-01
       coeff_l2(18,92)= -2.0564804971e-01
       coeff_l2(18,93)= -2.6233428717e-01
       coeff_l2(18,94)= -2.4568066001e-01
       coeff_l2(18,95)= -2.2057262063e-01
       coeff_l2(19,1)= -3.0080655217e-01
       coeff_l2(19,2)= -7.2381079197e-02
       coeff_l2(19,3)= -1.0308733582e-01
       coeff_l2(19,4)= -1.7821763456e-01
       coeff_l2(19,5)= -8.9418637753e-01
       coeff_l2(19,6)= -3.5663044453e-01
       coeff_l2(19,7)= -5.7086092234e-01
       coeff_l2(19,8)= -2.2155961394e-01
       coeff_l2(19,9)= -3.0149099231e-01
       coeff_l2(19,10)=  4.9675932527e-01
       coeff_l2(19,11)= -4.4724144042e-02
       coeff_l2(19,12)= -3.1521879137e-02
       coeff_l2(19,13)=  9.5436342061e-02
       coeff_l2(19,14)= -3.5173246264e-01
       coeff_l2(19,15)= -3.3521720767e-01
       coeff_l2(19,16)=  1.3839983940e-01
       coeff_l2(19,17)= -6.0111846775e-02
       coeff_l2(19,18)= -9.8340315744e-03
       coeff_l2(19,19)= -1.8399911001e-02
       coeff_l2(19,20)=  1.0757284611e-01
       coeff_l2(19,21)= -2.3610745370e-01
       coeff_l2(19,22)= -2.7513590455e-01
       coeff_l2(19,23)= -1.6087594628e-01
       coeff_l2(19,24)= -5.0326578319e-02
       coeff_l2(19,25)=  3.7473389506e-01
       coeff_l2(19,26)=  4.3498422951e-02
       coeff_l2(19,27)=  2.2577701136e-02
       coeff_l2(19,28)= -1.3674305379e-01
       coeff_l2(19,29)= -1.4220650494e-01
       coeff_l2(19,30)=  9.0674266219e-02
       coeff_l2(19,31)= -7.7719643712e-02
       coeff_l2(19,32)= -1.6331969202e-01
       coeff_l2(19,33)= -8.4603510797e-02
       coeff_l2(19,34)= -6.7036472261e-02
       coeff_l2(19,35)=  4.6741998196e-01
       coeff_l2(19,36)= -3.8212764263e-01
       coeff_l2(19,37)= -7.6196509600e-01
       coeff_l2(19,38)=  2.8728883713e-02
       coeff_l2(19,39)=  2.2502185404e-01
       coeff_l2(19,40)= -6.9537378848e-02
       coeff_l2(19,41)=  7.7810347080e-02
       coeff_l2(19,42)=  1.6494782642e-02
       coeff_l2(19,43)= -4.0404193103e-02
       coeff_l2(19,44)= -7.4017703533e-02
       coeff_l2(19,45)= -2.4091912434e-02
       coeff_l2(19,46)=  2.2062008083e-01
       coeff_l2(19,47)=  5.1657861471e-01
       coeff_l2(19,48)= -1.5946947038e-01
       coeff_l2(19,49)= -1.2908174098e-01
       coeff_l2(19,50)=  4.7892743349e-01
       coeff_l2(19,51)= -2.3143659532e-01
       coeff_l2(19,52)= -1.4180422761e-02
       coeff_l2(19,53)= -3.4215524793e-01
       coeff_l2(19,54)= -2.2914730012e-01
       coeff_l2(19,55)= -1.2317820638e-01
       coeff_l2(19,56)= -2.0950277150e-01
       coeff_l2(19,57)= -1.4004467428e-01
       coeff_l2(19,58)= -1.0354442596e+00
       coeff_l2(19,59)= -2.0227458328e-02
       coeff_l2(19,60)= -2.2011564672e-01
       coeff_l2(19,61)= -3.8277712464e-01
       coeff_l2(19,62)=  2.4599395692e-02
       coeff_l2(19,63)= -1.8002372980e-01
       coeff_l2(19,64)= -3.5415369272e-01
       coeff_l2(19,65)= -4.3688797951e-01
       coeff_l2(19,66)= -1.5759149194e-01
       coeff_l2(19,67)=  1.7226354778e-01
       coeff_l2(19,68)= -7.3787219822e-02
       coeff_l2(19,69)=  7.7222706750e-03
       coeff_l2(19,70)= -5.1719918847e-02
       coeff_l2(19,71)=  4.1122174263e-01
       coeff_l2(19,72)=  1.0741766542e-02
       coeff_l2(19,73)= -1.5519438684e-01
       coeff_l2(19,74)= -2.5139290094e-01
       coeff_l2(19,75)= -1.0085823387e-01
       coeff_l2(19,76)= -1.9527234137e-01
       coeff_l2(19,77)= -3.4868300427e-03
       coeff_l2(19,78)= -6.5888985991e-02
       coeff_l2(19,79)= -6.0309386253e-01
       coeff_l2(19,80)= -3.7067511678e-01
       coeff_l2(19,81)= -1.9185881317e-01
       coeff_l2(19,82)= -3.4153959155e-01
       coeff_l2(19,83)= -8.8939592242e-02
       coeff_l2(19,84)= -1.2448285520e-01
       coeff_l2(19,85)= -2.4059315026e-01
       coeff_l2(19,86)=  3.5825484991e-01
       coeff_l2(19,87)=  1.0902464390e-01
       coeff_l2(19,88)= -3.8160506636e-02
       coeff_l2(19,89)= -2.0182693005e-01
       coeff_l2(19,90)=  1.9302353263e-01
       coeff_l2(19,91)= -1.2901271880e-01
       coeff_l2(19,92)=  1.7742817104e-01
       coeff_l2(19,93)=  1.6447234154e-01
       coeff_l2(19,94)= -8.7641989812e-03
       coeff_l2(19,95)= -2.2374017537e-01
       coeff_l2(20,1)=  3.1883147359e-01
       coeff_l2(20,2)= -1.3481789827e-01
       coeff_l2(20,3)= -1.5091060102e-01
       coeff_l2(20,4)=  1.9158121347e+00
       coeff_l2(20,5)=  1.3223017454e+00
       coeff_l2(20,6)=  1.6573376656e+00
       coeff_l2(20,7)= -3.0410674214e-01
       coeff_l2(20,8)=  2.5793486834e-01
       coeff_l2(20,9)=  1.2452518940e-01
       coeff_l2(20,10)= -1.9493867159e+00
       coeff_l2(20,11)= -6.9749191403e-02
       coeff_l2(20,12)= -1.2713088095e-01
       coeff_l2(20,13)= -6.5717384219e-02
       coeff_l2(20,14)= -2.2663439810e-01
       coeff_l2(20,15)=  4.4155148789e-03
       coeff_l2(20,16)= -4.8781752586e-01
       coeff_l2(20,17)= -2.3537366092e-01
       coeff_l2(20,18)=  1.4052921534e-01
       coeff_l2(20,19)= -7.4790962040e-02
       coeff_l2(20,20)= -2.6213738322e-01
       coeff_l2(20,21)= -7.4640795588e-02
       coeff_l2(20,22)=  3.4213459492e-01
       coeff_l2(20,23)=  7.7510140836e-02
       coeff_l2(20,24)= -1.3131958246e-01
       coeff_l2(20,25)= -5.4452323914e-01
       coeff_l2(20,26)=  1.0700230300e-01
       coeff_l2(20,27)=  6.6648058593e-02
       coeff_l2(20,28)=  1.7556285858e-01
       coeff_l2(20,29)=  2.0239815116e-01
       coeff_l2(20,30)=  5.4194301367e-02
       coeff_l2(20,31)=  2.4851264060e-01
       coeff_l2(20,32)= -3.2287694514e-02
       coeff_l2(20,33)= -2.8100499511e-01
       coeff_l2(20,34)=  2.4918182194e-01
       coeff_l2(20,35)=  7.2609150410e-01
       coeff_l2(20,36)= -3.7974275649e-02
       coeff_l2(20,37)=  1.0563431978e+00
       coeff_l2(20,38)=  4.4134099036e-02
       coeff_l2(20,39)=  4.4722837210e-01
       coeff_l2(20,40)= -2.8643158078e-01
       coeff_l2(20,41)= -5.1139514893e-02
       coeff_l2(20,42)=  1.7599473894e-01
       coeff_l2(20,43)= -1.0543395281e+00
       coeff_l2(20,44)= -1.0055398941e-01
       coeff_l2(20,45)=  2.1072599292e-01
       coeff_l2(20,46)= -7.4975836277e-01
       coeff_l2(20,47)= -9.4741678238e-01
       coeff_l2(20,48)= -1.7669187486e-01
       coeff_l2(20,49)= -8.4570553154e-03
       coeff_l2(20,50)=  2.9377576709e-01
       coeff_l2(20,51)= -1.2978625298e-01
       coeff_l2(20,52)=  5.0415411592e-02
       coeff_l2(20,53)=  4.3663293123e-01
       coeff_l2(20,54)= -9.1205224395e-02
       coeff_l2(20,55)=  5.1611989737e-01
       coeff_l2(20,56)= -1.9876319915e-02
       coeff_l2(20,57)=  3.1339558959e-01
       coeff_l2(20,58)=  3.3615058661e-01
       coeff_l2(20,59)=  9.2461600900e-02
       coeff_l2(20,60)=  6.5498873591e-02
       coeff_l2(20,61)= -1.2255465984e-01
       coeff_l2(20,62)= -1.4693352580e-01
       coeff_l2(20,63)=  1.9930939674e+00
       coeff_l2(20,64)=  6.9005429745e-01
       coeff_l2(20,65)=  5.4820668697e-01
       coeff_l2(20,66)= -8.4102851152e-01
       coeff_l2(20,67)= -1.5652312040e+00
       coeff_l2(20,68)= -4.3340015411e-01
       coeff_l2(20,69)=  1.1851230264e-01
       coeff_l2(20,70)= -5.3850203753e-02
       coeff_l2(20,71)= -1.2677456141e+00
       coeff_l2(20,72)= -1.0931397229e-01
       coeff_l2(20,73)= -1.5876296163e-01
       coeff_l2(20,74)= -2.6099389791e-01
       coeff_l2(20,75)= -3.9887882769e-02
       coeff_l2(20,76)= -2.1338330582e-02
       coeff_l2(20,77)= -4.2849805951e-01
       coeff_l2(20,78)=  1.5846079215e-02
       coeff_l2(20,79)=  2.9281169176e-01
       coeff_l2(20,80)=  5.7021188736e-01
       coeff_l2(20,81)=  9.1594673693e-02
       coeff_l2(20,82)=  5.7393276691e-01
       coeff_l2(20,83)= -2.4861809611e-01
       coeff_l2(20,84)= -1.2248165160e-01
       coeff_l2(20,85)= -2.3264338076e-01
       coeff_l2(20,86)=  2.3234377801e-01
       coeff_l2(20,87)= -6.2507309020e-02
       coeff_l2(20,88)= -2.6354575157e-01
       coeff_l2(20,89)= -1.1299829930e-02
       coeff_l2(20,90)= -7.9287081957e-01
       coeff_l2(20,91)=  5.3044207394e-02
       coeff_l2(20,92)=  3.0678864568e-02
       coeff_l2(20,93)=  4.4063717127e-01
       coeff_l2(20,94)=  1.3296478987e-01
       coeff_l2(20,95)= -1.8170149624e-01
       coeff_l2(21,1)= -1.6306887567e-01
       coeff_l2(21,2)= -1.5143108368e-01
       coeff_l2(21,3)= -8.0747261643e-02
       coeff_l2(21,4)= -3.6511328071e-02
       coeff_l2(21,5)= -1.1402797699e+00
       coeff_l2(21,6)= -3.2055157423e-01
       coeff_l2(21,7)= -8.5970175266e-01
       coeff_l2(21,8)= -6.9289252162e-02
       coeff_l2(21,9)= -1.0570392013e-01
       coeff_l2(21,10)=  6.3979130983e-01
       coeff_l2(21,11)= -2.0403423905e-01
       coeff_l2(21,12)=  4.4016528875e-02
       coeff_l2(21,13)=  9.2363022268e-02
       coeff_l2(21,14)= -2.5181353092e-01
       coeff_l2(21,15)= -1.1973511428e-01
       coeff_l2(21,16)=  1.8772517145e-01
       coeff_l2(21,17)= -1.5970097482e-01
       coeff_l2(21,18)= -1.2022123486e-01
       coeff_l2(21,19)=  6.3210204244e-02
       coeff_l2(21,20)=  3.3628389239e-02
       coeff_l2(21,21)= -5.2649134398e-01
       coeff_l2(21,22)= -1.4199379086e-01
       coeff_l2(21,23)=  1.7927581072e-01
       coeff_l2(21,24)=  1.7350137234e-01
       coeff_l2(21,25)=  1.7666414380e-01
       coeff_l2(21,26)= -3.1419399381e-01
       coeff_l2(21,27)=  1.5070486069e-01
       coeff_l2(21,28)= -8.4562033415e-02
       coeff_l2(21,29)=  5.9730913490e-02
       coeff_l2(21,30)= -1.9640719891e-01
       coeff_l2(21,31)=  1.4554674737e-02
       coeff_l2(21,32)=  1.1937336624e-01
       coeff_l2(21,33)= -2.0337718725e-01
       coeff_l2(21,34)= -9.8114386201e-02
       coeff_l2(21,35)=  5.2137750387e-01
       coeff_l2(21,36)= -7.4250131845e-01
       coeff_l2(21,37)= -6.6981518269e-01
       coeff_l2(21,38)=  9.6544556320e-02
       coeff_l2(21,39)=  1.5929253399e-01
       coeff_l2(21,40)= -8.2464419305e-02
       coeff_l2(21,41)=  9.5224566758e-02
       coeff_l2(21,42)=  3.0637133121e-01
       coeff_l2(21,43)= -4.0150794387e-01
       coeff_l2(21,44)=  5.8537837118e-02
       coeff_l2(21,45)= -2.3505370319e-01
       coeff_l2(21,46)=  1.6739474237e-01
       coeff_l2(21,47)=  4.3694329262e-01
       coeff_l2(21,48)=  7.1564078331e-02
       coeff_l2(21,49)= -1.1551476270e-01
       coeff_l2(21,50)=  5.2377754450e-01
       coeff_l2(21,51)= -5.9836335480e-02
       coeff_l2(21,52)=  1.7919662595e-01
       coeff_l2(21,53)= -2.3673450574e-02
       coeff_l2(21,54)= -1.9922901690e-01
       coeff_l2(21,55)= -1.4558529854e-01
       coeff_l2(21,56)= -1.0412104428e-01
       coeff_l2(21,57)=  2.2605910897e-02
       coeff_l2(21,58)= -8.7074446678e-01
       coeff_l2(21,59)= -2.2369259968e-02
       coeff_l2(21,60)=  1.0483378917e-01
       coeff_l2(21,61)= -6.7133060656e-03
       coeff_l2(21,62)=  2.0240826905e-01
       coeff_l2(21,63)= -1.2639032304e-01
       coeff_l2(21,64)= -4.7764539719e-01
       coeff_l2(21,65)= -3.5974422097e-01
       coeff_l2(21,66)= -5.9851592779e-01
       coeff_l2(21,67)=  2.2793093324e-01
       coeff_l2(21,68)= -5.1833212376e-02
       coeff_l2(21,69)=  1.4528101310e-02
       coeff_l2(21,70)= -5.9134192765e-02
       coeff_l2(21,71)=  1.7194244266e-01
       coeff_l2(21,72)= -6.5838545561e-02
       coeff_l2(21,73)= -2.9728844762e-01
       coeff_l2(21,74)= -2.7622619271e-01
       coeff_l2(21,75)= -1.5495559573e-01
       coeff_l2(21,76)=  5.7354509830e-02
       coeff_l2(21,77)= -3.6452579498e-01
       coeff_l2(21,78)= -3.7989616394e-02
       coeff_l2(21,79)= -1.5167699754e-01
       coeff_l2(21,80)= -7.1733081341e-01
       coeff_l2(21,81)= -2.9009145498e-01
       coeff_l2(21,82)= -9.4855487347e-02
       coeff_l2(21,83)= -3.6320203543e-01
       coeff_l2(21,84)= -6.8872079253e-02
       coeff_l2(21,85)= -1.4382974803e-01
       coeff_l2(21,86)=  4.8730018735e-01
       coeff_l2(21,87)= -1.1868768185e-01
       coeff_l2(21,88)= -1.5389949083e-01
       coeff_l2(21,89)= -4.1123056412e-01
       coeff_l2(21,90)=  2.5569647551e-01
       coeff_l2(21,91)=  1.9809238613e-01
       coeff_l2(21,92)=  7.8600816429e-02
       coeff_l2(21,93)=  9.7904220223e-02
       coeff_l2(21,94)= -1.9382144511e-01
       coeff_l2(21,95)= -1.6950382292e-01
       coeff_l2(22,1)= -1.4881087840e-01
       coeff_l2(22,2)= -1.3272403181e-01
       coeff_l2(22,3)= -4.4388577342e-01
       coeff_l2(22,4)=  5.1246815920e-01
       coeff_l2(22,5)= -2.9310888052e-01
       coeff_l2(22,6)=  2.2996737063e-01
       coeff_l2(22,7)=  6.1631923914e-01
       coeff_l2(22,8)=  1.8407223746e-02
       coeff_l2(22,9)=  4.5128077269e-01
       coeff_l2(22,10)=  8.2546919584e-01
       coeff_l2(22,11)=  3.0426183343e-01
       coeff_l2(22,12)=  1.3450171053e-01
       coeff_l2(22,13)= -3.3385631442e-01
       coeff_l2(22,14)= -7.4802197516e-02
       coeff_l2(22,15)= -1.8969975412e-02
       coeff_l2(22,16)= -5.5036717653e-01
       coeff_l2(22,17)= -6.0694616288e-02
       coeff_l2(22,18)= -3.6408677697e-02
       coeff_l2(22,19)=  1.2967155874e-01
       coeff_l2(22,20)=  1.0505828261e-01
       coeff_l2(22,21)=  1.1597508192e+00
       coeff_l2(22,22)= -7.8264880180e-01
       coeff_l2(22,23)= -2.6534169912e-01
       coeff_l2(22,24)= -3.2051882148e-01
       coeff_l2(22,25)= -6.1462080479e-01
       coeff_l2(22,26)= -4.8534464836e-01
       coeff_l2(22,27)= -1.1037034541e-01
       coeff_l2(22,28)=  1.1490888149e-01
       coeff_l2(22,29)= -4.2482100427e-02
       coeff_l2(22,30)= -3.2605993748e-01
       coeff_l2(22,31)=  6.4004532993e-02
       coeff_l2(22,32)=  2.9161542654e-01
       coeff_l2(22,33)= -2.1008312702e-01
       coeff_l2(22,34)= -2.3878024518e-01
       coeff_l2(22,35)= -1.1437594146e-01
       coeff_l2(22,36)=  6.7469722033e-01
       coeff_l2(22,37)= -2.9022875428e-01
       coeff_l2(22,38)= -8.9094139636e-02
       coeff_l2(22,39)= -3.2650643587e-01
       coeff_l2(22,40)=  9.5811933279e-02
       coeff_l2(22,41)=  7.0727503300e-01
       coeff_l2(22,42)= -3.6237496138e-01
       coeff_l2(22,43)=  3.0782639980e-01
       coeff_l2(22,44)= -3.3855745196e-01
       coeff_l2(22,45)= -6.6523486748e-03
       coeff_l2(22,46)=  5.5313295126e-01
       coeff_l2(22,47)= -3.4241122007e-01
       coeff_l2(22,48)= -1.8854860216e-02
       coeff_l2(22,49)= -2.1547193825e-01
       coeff_l2(22,50)=  3.7376713753e-01
       coeff_l2(22,51)= -1.5706901252e-01
       coeff_l2(22,52)= -4.3050196767e-01
       coeff_l2(22,53)=  6.7879480124e-01
       coeff_l2(22,54)=  4.0543919802e-01
       coeff_l2(22,55)= -1.0403585434e+00
       coeff_l2(22,56)= -9.1121155024e-01
       coeff_l2(22,57)= -4.3709653616e-01
       coeff_l2(22,58)=  7.1761965752e-01
       coeff_l2(22,59)=  5.9009138495e-02
       coeff_l2(22,60)= -3.6654155701e-02
       coeff_l2(22,61)=  1.6048002243e-01
       coeff_l2(22,62)= -2.2684797645e-01
       coeff_l2(22,63)=  3.1614691019e-01
       coeff_l2(22,64)=  1.1846482009e-01
       coeff_l2(22,65)= -4.3794062734e-01
       coeff_l2(22,66)=  2.6962295175e-01
       coeff_l2(22,67)=  9.4686830044e-01
       coeff_l2(22,68)=  4.2173948884e-01
       coeff_l2(22,69)= -2.3884715140e-01
       coeff_l2(22,70)= -1.8682789803e-01
       coeff_l2(22,71)=  5.7980626822e-01
       coeff_l2(22,72)=  3.2339036465e-01
       coeff_l2(22,73)= -3.1893035769e-01
       coeff_l2(22,74)= -3.4438133240e-02
       coeff_l2(22,75)=  5.4005987942e-02
       coeff_l2(22,76)=  1.6564469039e-01
       coeff_l2(22,77)= -4.7464564443e-01
       coeff_l2(22,78)= -2.7414792776e-01
       coeff_l2(22,79)= -9.2414176464e-01
       coeff_l2(22,80)=  2.5492173433e-01
       coeff_l2(22,81)= -1.0720821619e+00
       coeff_l2(22,82)= -9.1668754816e-01
       coeff_l2(22,83)=  3.3887431026e-01
       coeff_l2(22,84)= -2.3459941149e-01
       coeff_l2(22,85)= -1.3503423929e+00
       coeff_l2(22,86)= -5.4935449362e-01
       coeff_l2(22,87)= -4.8152926564e-01
       coeff_l2(22,88)= -1.7927311361e-01
       coeff_l2(22,89)= -1.5016025305e-01
       coeff_l2(22,90)=  5.2060544491e-01
       coeff_l2(22,91)= -1.5753740072e-01
       coeff_l2(22,92)=  9.2285767198e-02
       coeff_l2(22,93)=  4.6809550375e-02
       coeff_l2(22,94)= -2.3824207485e-01
       coeff_l2(22,95)= -5.7935476303e-01
       coeff_l2(23,1)= -9.5960684121e-02
       coeff_l2(23,2)=  9.8476514220e-02
       coeff_l2(23,3)=  1.6628241539e-01
       coeff_l2(23,4)= -1.1508125067e+00
       coeff_l2(23,5)= -6.0857647657e-01
       coeff_l2(23,6)=  2.0672044158e-01
       coeff_l2(23,7)= -2.4514648318e-01
       coeff_l2(23,8)= -1.6651549935e-01
       coeff_l2(23,9)=  1.0829756968e-02
       coeff_l2(23,10)= -1.8245536089e-01
       coeff_l2(23,11)=  4.8397719860e-02
       coeff_l2(23,12)= -2.2984899580e-01
       coeff_l2(23,13)= -6.9959655404e-02
       coeff_l2(23,14)= -8.1468932331e-02
       coeff_l2(23,15)= -4.9709403515e-01
       coeff_l2(23,16)=  3.9258378744e-01
       coeff_l2(23,17)=  2.1987538040e-01
       coeff_l2(23,18)= -1.0678934306e-01
       coeff_l2(23,19)= -2.6372771710e-02
       coeff_l2(23,20)=  4.0699288249e-02
       coeff_l2(23,21)=  3.5420808941e-02
       coeff_l2(23,22)=  4.8348081112e-01
       coeff_l2(23,23)= -9.6047565341e-02
       coeff_l2(23,24)=  2.9687047005e-01
       coeff_l2(23,25)=  3.9803680778e-01
       coeff_l2(23,26)=  2.5284644961e-01
       coeff_l2(23,27)= -1.3888937235e-01
       coeff_l2(23,28)= -1.3551336527e-01
       coeff_l2(23,29)= -7.6877713203e-02
       coeff_l2(23,30)= -1.4948050678e-01
       coeff_l2(23,31)= -8.8214039803e-02
       coeff_l2(23,32)=  1.8128679693e-01
       coeff_l2(23,33)= -2.7835011482e-01
       coeff_l2(23,34)= -7.2838954628e-02
       coeff_l2(23,35)= -9.4867187738e-01
       coeff_l2(23,36)= -1.5091417730e-01
       coeff_l2(23,37)= -3.2097241282e-01
       coeff_l2(23,38)= -5.9642892331e-02
       coeff_l2(23,39)=  4.8019817472e-01
       coeff_l2(23,40)=  1.6935694963e-02
       coeff_l2(23,41)= -8.8709443808e-03
       coeff_l2(23,42)=  1.2627586722e-01
       coeff_l2(23,43)=  2.0649552345e-02
       coeff_l2(23,44)=  4.0248334408e-01
       coeff_l2(23,45)= -3.8479480147e-01
       coeff_l2(23,46)= -5.0743860006e-01
       coeff_l2(23,47)=  2.1507392824e-01
       coeff_l2(23,48)=  3.4953892231e-02
       coeff_l2(23,49)=  1.3966342807e-01
       coeff_l2(23,50)=  7.1789756417e-02
       coeff_l2(23,51)= -3.7336269021e-01
       coeff_l2(23,52)=  1.7428903282e-01
       coeff_l2(23,53)= -2.9659536481e-01
       coeff_l2(23,54)= -1.8210899830e-01
       coeff_l2(23,55)=  3.0113538727e-02
       coeff_l2(23,56)= -1.5564247966e-01
       coeff_l2(23,57)=  5.0188183784e-01
       coeff_l2(23,58)= -7.1755731106e-01
       coeff_l2(23,59)=  2.9453998804e-01
       coeff_l2(23,60)= -1.2784495950e-01
       coeff_l2(23,61)= -7.2984375060e-02
       coeff_l2(23,62)=  4.2261800170e-01
       coeff_l2(23,63)= -1.6749920845e+00
       coeff_l2(23,64)=  3.0804783106e-01
       coeff_l2(23,65)=  2.1423038840e-01
       coeff_l2(23,66)= -2.5934150815e-01
       coeff_l2(23,67)= -7.3342400789e-01
       coeff_l2(23,68)= -3.9195239544e-01
       coeff_l2(23,69)= -2.3207427561e-01
       coeff_l2(23,70)= -1.9475874305e-01
       coeff_l2(23,71)= -4.1322800517e-01
       coeff_l2(23,72)= -1.5634040534e-01
       coeff_l2(23,73)= -1.8734569848e-01
       coeff_l2(23,74)= -2.2944998741e-01
       coeff_l2(23,75)= -4.2089331895e-02
       coeff_l2(23,76)= -1.6112254560e-01
       coeff_l2(23,77)= -3.7479835749e-01
       coeff_l2(23,78)= -3.0545338988e-01
       coeff_l2(23,79)=  3.6158806086e-01
       coeff_l2(23,80)= -1.8739968538e-01
       coeff_l2(23,81)=  3.6354678869e-01
       coeff_l2(23,82)=  1.9384507835e-01
       coeff_l2(23,83)= -3.2171401381e-01
       coeff_l2(23,84)= -2.4915312231e-01
       coeff_l2(23,85)= -5.0876047462e-02
       coeff_l2(23,86)=  1.6260038316e-01
       coeff_l2(23,87)=  2.7283173800e-01
       coeff_l2(23,88)= -4.8236972094e-01
       coeff_l2(23,89)= -1.6417290270e-01
       coeff_l2(23,90)= -2.2308497131e-01
       coeff_l2(23,91)=  1.1400309950e-01
       coeff_l2(23,92)= -7.4543081224e-02
       coeff_l2(23,93)=  3.9850458503e-01
       coeff_l2(23,94)= -1.5462426841e-01
       coeff_l2(23,95)= -2.1216690540e-01
       coeff_l2(24,1)= -1.0789299011e+00
       coeff_l2(24,2)= -3.7047079206e-01
       coeff_l2(24,3)= -6.5748375654e-01
       coeff_l2(24,4)=  2.1411309242e+00
       coeff_l2(24,5)= -6.8609571457e-01
       coeff_l2(24,6)=  9.5058643818e-01
       coeff_l2(24,7)= -7.1388953924e-01
       coeff_l2(24,8)= -2.5588890910e-01
       coeff_l2(24,9)= -1.7061129212e-01
       coeff_l2(24,10)=  1.0860186815e+00
       coeff_l2(24,11)= -4.1812118888e-01
       coeff_l2(24,12)= -1.7750334740e-01
       coeff_l2(24,13)= -5.4747915268e-01
       coeff_l2(24,14)= -2.2012528777e-01
       coeff_l2(24,15)= -3.0697783828e-01
       coeff_l2(24,16)= -4.4387480617e-01
       coeff_l2(24,17)= -1.0695963353e-01
       coeff_l2(24,18)=  3.3374655992e-02
       coeff_l2(24,19)= -2.8660538793e-01
       coeff_l2(24,20)=  1.3123096526e-01
       coeff_l2(24,21)= -4.7460517287e-01
       coeff_l2(24,22)= -6.0191297531e-01
       coeff_l2(24,23)= -7.6365470886e-02
       coeff_l2(24,24)=  5.2130386233e-02
       coeff_l2(24,25)= -2.7795889974e-01
       coeff_l2(24,26)= -4.1366556287e-01
       coeff_l2(24,27)= -1.9168202579e-01
       coeff_l2(24,28)=  1.3523912430e+00
       coeff_l2(24,29)= -1.5771195292e-01
       coeff_l2(24,30)= -9.8657503724e-02
       coeff_l2(24,31)=  1.1291288137e+00
       coeff_l2(24,32)= -1.4462685585e-01
       coeff_l2(24,33)=  5.4085814953e-01
       coeff_l2(24,34)=  2.3933564126e-01
       coeff_l2(24,35)=  3.3834722042e+00
       coeff_l2(24,36)=  3.2572066784e-01
       coeff_l2(24,37)= -1.2125494480e+00
       coeff_l2(24,38)=  2.7828752995e-02
       coeff_l2(24,39)=  8.6123299599e-01
       coeff_l2(24,40)=  5.1269000769e-01
       coeff_l2(24,41)=  1.1314065754e-01
       coeff_l2(24,42)= -1.8357971683e-02
       coeff_l2(24,43)= -8.0584371462e-03
       coeff_l2(24,44)= -4.8956993222e-01
       coeff_l2(24,45)=  3.2442665100e-01
       coeff_l2(24,46)=  2.9824084044e-01
       coeff_l2(24,47)=  4.8930004239e-01
       coeff_l2(24,48)= -1.4403021336e-01
       coeff_l2(24,49)= -2.2100779414e-01
       coeff_l2(24,50)=  2.1189327240e+00
       coeff_l2(24,51)= -5.9843832254e-01
       coeff_l2(24,52)= -7.5827807188e-02
       coeff_l2(24,53)=  4.5852744579e-01
       coeff_l2(24,54)=  6.1874389648e-02
       coeff_l2(24,55)=  8.5625761747e-01
       coeff_l2(24,56)= -6.0240209103e-01
       coeff_l2(24,57)= -5.2981793880e-01
       coeff_l2(24,58)= -4.7769322991e-01
       coeff_l2(24,59)= -2.5474381447e-01
       coeff_l2(24,60)=  3.9899118245e-02
       coeff_l2(24,61)= -3.3522561193e-01
       coeff_l2(24,62)= -3.1998312473e-01
       coeff_l2(24,63)=  4.3648381233e+00
       coeff_l2(24,64)=  3.2493200898e-01
       coeff_l2(24,65)= -3.7259924412e-01
       coeff_l2(24,66)= -6.5845960379e-01
       coeff_l2(24,67)=  1.2030528784e+00
       coeff_l2(24,68)=  9.2688508332e-02
       coeff_l2(24,69)= -2.3606920242e-01
       coeff_l2(24,70)=  2.3089140654e-01
       coeff_l2(24,71)=  1.3753297329e+00
       coeff_l2(24,72)= -2.8771200776e-01
       coeff_l2(24,73)= -2.2684958577e-01
       coeff_l2(24,74)=  3.0527341366e-01
       coeff_l2(24,75)= -1.9601105154e-01
       coeff_l2(24,76)= -7.1542724967e-02
       coeff_l2(24,77)= -5.2523910999e-01
       coeff_l2(24,78)= -8.6587183177e-02
       coeff_l2(24,79)= -3.0435075760e+00
       coeff_l2(24,80)= -1.1218650341e+00
       coeff_l2(24,81)= -1.7071306705e-02
       coeff_l2(24,82)= -8.9780509472e-02
       coeff_l2(24,83)= -3.6500313878e-01
       coeff_l2(24,84)= -1.7629174516e-02
       coeff_l2(24,85)= -1.2369044870e-01
       coeff_l2(24,86)= -1.8025552854e-02
       coeff_l2(24,87)= -5.6283962727e-01
       coeff_l2(24,88)=  2.8012275696e-01
       coeff_l2(24,89)= -4.2241862416e-01
       coeff_l2(24,90)=  2.5776422024e+00
       coeff_l2(24,91)= -2.7951049805e-01
       coeff_l2(24,92)=  6.8834972382e-01
       coeff_l2(24,93)= -3.5484907031e-01
       coeff_l2(24,94)= -6.6001281142e-02
       coeff_l2(24,95)= -1.3376474380e-01
       coeff_l2(25,1)= -2.8779718280e-01
       coeff_l2(25,2)=  3.3897776157e-02
       coeff_l2(25,3)= -2.3353853822e-01
       coeff_l2(25,4)=  8.8132917881e-02
       coeff_l2(25,5)= -2.0955516025e-02
       coeff_l2(25,6)= -3.6600497365e-01
       coeff_l2(25,7)=  5.4201092571e-02
       coeff_l2(25,8)= -1.2815323472e-01
       coeff_l2(25,9)=  1.7082603276e-01
       coeff_l2(25,10)=  3.9211097360e-01
       coeff_l2(25,11)= -1.0993750393e-01
       coeff_l2(25,12)=  1.3315482438e-01
       coeff_l2(25,13)= -4.0560957044e-02
       coeff_l2(25,14)= -3.5422753543e-02
       coeff_l2(25,15)= -5.7013016194e-02
       coeff_l2(25,16)= -5.3215056658e-02
       coeff_l2(25,17)= -1.6504971310e-02
       coeff_l2(25,18)= -3.0584702268e-02
       coeff_l2(25,19)=  2.6281461120e-01
       coeff_l2(25,20)=  4.2241144925e-02
       coeff_l2(25,21)=  9.0935304761e-02
       coeff_l2(25,22)= -2.5875765085e-01
       coeff_l2(25,23)= -9.1231934726e-02
       coeff_l2(25,24)= -1.5141955018e-01
       coeff_l2(25,25)= -1.9046090543e-01
       coeff_l2(25,26)= -1.7730404437e-01
       coeff_l2(25,27)=  1.2250760198e-01
       coeff_l2(25,28)=  2.2160568833e-01
       coeff_l2(25,29)= -1.8275870010e-02
       coeff_l2(25,30)=  1.1398865283e-01
       coeff_l2(25,31)= -1.2849660218e-01
       coeff_l2(25,32)=  2.1530023217e-01
       coeff_l2(25,33)=  1.1352942884e-01
       coeff_l2(25,34)=  2.6770779863e-02
       coeff_l2(25,35)=  1.8666696548e-01
       coeff_l2(25,36)=  6.0643303394e-01
       coeff_l2(25,37)= -3.1932154298e-01
       coeff_l2(25,38)= -1.7507399619e-01
       coeff_l2(25,39)=  1.4758202434e-01
       coeff_l2(25,40)= -6.6661693156e-02
       coeff_l2(25,41)=  9.4298109412e-02
       coeff_l2(25,42)=  1.6912901402e-01
       coeff_l2(25,43)=  1.9792219996e-01
       coeff_l2(25,44)= -2.3908358812e-01
       coeff_l2(25,45)=  1.1573700607e-01
       coeff_l2(25,46)=  1.4450280368e-01
       coeff_l2(25,47)=  3.3622568846e-01
       coeff_l2(25,48)=  2.4256885052e-02
       coeff_l2(25,49)= -6.5643243492e-02
       coeff_l2(25,50)=  3.6303073168e-01
       coeff_l2(25,51)= -6.4784735441e-02
       coeff_l2(25,52)=  6.7932337523e-02
       coeff_l2(25,53)=  9.2422077432e-03
       coeff_l2(25,54)=  7.4389815331e-02
       coeff_l2(25,55)= -9.5028549433e-02
       coeff_l2(25,56)= -7.5042791665e-02
       coeff_l2(25,57)= -3.9955884218e-01
       coeff_l2(25,58)=  1.0688673705e-01
       coeff_l2(25,59)= -6.6778793931e-02
       coeff_l2(25,60)= -1.0791336000e-01
       coeff_l2(25,61)= -3.0417319387e-02
       coeff_l2(25,62)= -3.3047172427e-01
       coeff_l2(25,63)= -1.1347883940e-01
       coeff_l2(25,64)=  1.7687886953e-01
       coeff_l2(25,65)= -4.8916876316e-01
       coeff_l2(25,66)=  6.4439535141e-01
       coeff_l2(25,67)=  8.0437070131e-01
       coeff_l2(25,68)= -1.9974647090e-02
       coeff_l2(25,69)= -1.3921985030e-01
       coeff_l2(25,70)= -3.6223866045e-02
       coeff_l2(25,71)=  3.0415362120e-01
       coeff_l2(25,72)=  2.0453321934e-01
       coeff_l2(25,73)= -5.7549405843e-02
       coeff_l2(25,74)= -2.8448259458e-02
       coeff_l2(25,75)= -2.3779512942e-01
       coeff_l2(25,76)=  2.4347294122e-02
       coeff_l2(25,77)=  4.2646107078e-01
       coeff_l2(25,78)=  4.3386634439e-02
       coeff_l2(25,79)= -1.2979792431e-02
       coeff_l2(25,80)=  1.9229044020e-01
       coeff_l2(25,81)= -1.8996235728e-01
       coeff_l2(25,82)= -9.4460353255e-02
       coeff_l2(25,83)= -8.9680731297e-02
       coeff_l2(25,84)=  5.4521836340e-02
       coeff_l2(25,85)= -1.4824147522e-01
       coeff_l2(25,86)= -2.8224605322e-01
       coeff_l2(25,87)= -1.5285275877e-01
       coeff_l2(25,88)=  9.5809683204e-02
       coeff_l2(25,89)=  2.9931202531e-01
       coeff_l2(25,90)=  1.9546456635e-01
       coeff_l2(25,91)= -1.7260536551e-01
       coeff_l2(25,92)=  3.0377435684e-01
       coeff_l2(25,93)= -1.8247820437e-01
       coeff_l2(25,94)= -8.6010079831e-03
       coeff_l2(25,95)= -2.3321297765e-01
       coeff_l2(26,1)=  6.0874307528e-03
       coeff_l2(26,2)= -5.5254745530e-04
       coeff_l2(26,3)= -2.0126377046e-01
       coeff_l2(26,4)= -1.8998987973e-01
       coeff_l2(26,5)= -1.1817643642e+00
       coeff_l2(26,6)= -3.6598086357e-01
       coeff_l2(26,7)= -8.2367336750e-01
       coeff_l2(26,8)=  4.7836188227e-02
       coeff_l2(26,9)= -3.0226677656e-01
       coeff_l2(26,10)=  5.5313253403e-01
       coeff_l2(26,11)= -5.4065879434e-02
       coeff_l2(26,12)= -9.3728028238e-02
       coeff_l2(26,13)=  9.2480666935e-02
       coeff_l2(26,14)= -3.3664286137e-01
       coeff_l2(26,15)= -2.3773971945e-02
       coeff_l2(26,16)=  1.3453048468e-01
       coeff_l2(26,17)=  6.2505476177e-02
       coeff_l2(26,18)=  4.2885266244e-02
       coeff_l2(26,19)=  2.5955377147e-02
       coeff_l2(26,20)= -4.4225309044e-02
       coeff_l2(26,21)= -4.4184750319e-01
       coeff_l2(26,22)= -1.0140578263e-02
       coeff_l2(26,23)=  1.4683461189e-01
       coeff_l2(26,24)=  2.2474569082e-01
       coeff_l2(26,25)=  4.7151505947e-01
       coeff_l2(26,26)= -4.6660147607e-02
       coeff_l2(26,27)= -5.1844358444e-02
       coeff_l2(26,28)= -4.8464277387e-01
       coeff_l2(26,29)= -1.9870872796e-01
       coeff_l2(26,30)=  1.4414140582e-01
       coeff_l2(26,31)=  4.2863301933e-02
       coeff_l2(26,32)= -7.4807658792e-02
       coeff_l2(26,33)= -1.1533922702e-01
       coeff_l2(26,34)= -4.6641141176e-02
       coeff_l2(26,35)=  5.4404330254e-01
       coeff_l2(26,36)= -6.5579801798e-01
       coeff_l2(26,37)= -3.1921085715e-01
       coeff_l2(26,38)=  3.5773674026e-03
       coeff_l2(26,39)=  2.7873679996e-01
       coeff_l2(26,40)= -9.6821725368e-02
       coeff_l2(26,41)= -8.3987988532e-02
       coeff_l2(26,42)=  2.3496311903e-01
       coeff_l2(26,43)= -6.3570266962e-01
       coeff_l2(26,44)= -1.8139632419e-02
       coeff_l2(26,45)= -2.5010094047e-01
       coeff_l2(26,46)=  2.6891943812e-01
       coeff_l2(26,47)=  4.8792174459e-01
       coeff_l2(26,48)=  2.0472139120e-01
       coeff_l2(26,49)=  6.1967480928e-02
       coeff_l2(26,50)=  4.9739238620e-01
       coeff_l2(26,51)= -4.7300145030e-02
       coeff_l2(26,52)=  1.2686997652e-01
       coeff_l2(26,53)= -1.1465842277e-01
       coeff_l2(26,54)= -3.6868814379e-02
       coeff_l2(26,55)= -1.5547819436e-01
       coeff_l2(26,56)= -2.4001315236e-01
       coeff_l2(26,57)=  1.0603526235e-01
       coeff_l2(26,58)= -7.1667248011e-01
       coeff_l2(26,59)= -2.9900231957e-01
       coeff_l2(26,60)= -5.8071617037e-02
       coeff_l2(26,61)= -6.1310045421e-02
       coeff_l2(26,62)=  3.5615244508e-01
       coeff_l2(26,63)= -1.4525625110e-01
       coeff_l2(26,64)= -6.4021742344e-01
       coeff_l2(26,65)=  3.4517835826e-02
       coeff_l2(26,66)= -4.2013326287e-01
       coeff_l2(26,67)=  1.8283875287e-01
       coeff_l2(26,68)= -2.2466631606e-02
       coeff_l2(26,69)=  7.1692258120e-02
       coeff_l2(26,70)=  2.6753816754e-02
       coeff_l2(26,71)=  3.4261924028e-01
       coeff_l2(26,72)=  5.1282551140e-02
       coeff_l2(26,73)=  2.0439088345e-02
       coeff_l2(26,74)= -9.6022441983e-02
       coeff_l2(26,75)=  2.4419197813e-02
       coeff_l2(26,76)=  2.4552959949e-02
       coeff_l2(26,77)= -2.0698475838e-01
       coeff_l2(26,78)= -1.3357095420e-01
       coeff_l2(26,79)= -6.4107298851e-01
       coeff_l2(26,80)= -5.3027564287e-01
       coeff_l2(26,81)= -1.2499804795e-01
       coeff_l2(26,82)=  1.0206732899e-01
       coeff_l2(26,83)= -1.2978616357e-01
       coeff_l2(26,84)=  1.1920562387e-01
       coeff_l2(26,85)=  1.9945582747e-01
       coeff_l2(26,86)=  1.6575549543e-01
       coeff_l2(26,87)=  8.4626071155e-02
       coeff_l2(26,88)=  8.2025468349e-02
       coeff_l2(26,89)=  9.1246858239e-02
       coeff_l2(26,90)= -3.7471514195e-02
       coeff_l2(26,91)=  9.9181987345e-02
       coeff_l2(26,92)= -2.3158463836e-01
       coeff_l2(26,93)=  9.3159958720e-02
       coeff_l2(26,94)= -2.0326577127e-01
       coeff_l2(26,95)= -2.2893580794e-01
       coeff_l2(27,1)= -7.2605550289e-01
       coeff_l2(27,2)= -2.5796079636e-01
       coeff_l2(27,3)=  1.8725065887e-01
       coeff_l2(27,4)=  6.7721819878e-01
       coeff_l2(27,5)=  1.1054247618e-02
       coeff_l2(27,6)=  6.4521437883e-01
       coeff_l2(27,7)= -3.9821198583e-01
       coeff_l2(27,8)= -1.5189935267e-01
       coeff_l2(27,9)= -3.9309374988e-02
       coeff_l2(27,10)= -4.7101598978e-01
       coeff_l2(27,11)= -2.3649284244e-01
       coeff_l2(27,12)=  3.0654752254e-01
       coeff_l2(27,13)= -3.4921368957e-01
       coeff_l2(27,14)=  2.2532884777e-01
       coeff_l2(27,15)= -5.4475475103e-02
       coeff_l2(27,16)= -1.8814162910e-01
       coeff_l2(27,17)=  1.0406743735e-01
       coeff_l2(27,18)= -1.7687742412e-01
       coeff_l2(27,19)= -1.7919412255e-01
       coeff_l2(27,20)=  1.3390564919e-01
       coeff_l2(27,21)= -7.9278007150e-02
       coeff_l2(27,22)=  2.0655426383e-01
       coeff_l2(27,23)= -2.4610251188e-01
       coeff_l2(27,24)= -1.4484742284e-01
       coeff_l2(27,25)=  2.1760746837e-01
       coeff_l2(27,26)=  4.6954420209e-01
       coeff_l2(27,27)=  1.4703392982e-01
       coeff_l2(27,28)=  6.5657436848e-01
       coeff_l2(27,29)=  7.6668858528e-02
       coeff_l2(27,30)= -9.8685279489e-02
       coeff_l2(27,31)=  2.3537039757e-01
       coeff_l2(27,32)= -6.9836646318e-02
       coeff_l2(27,33)=  4.7858643532e-01
       coeff_l2(27,34)=  3.8714420795e-01
       coeff_l2(27,35)=  1.3613454103e+00
       coeff_l2(27,36)= -4.0107023716e-01
       coeff_l2(27,37)=  5.1879032981e-05
       coeff_l2(27,38)= -7.1890562773e-02
       coeff_l2(27,39)=  3.2258751988e-01
       coeff_l2(27,40)=  8.6972612143e-01
       coeff_l2(27,41)= -1.7984007299e-01
       coeff_l2(27,42)=  4.5037642121e-02
       coeff_l2(27,43)= -5.3354358673e-01
       coeff_l2(27,44)=  1.2961371243e-01
       coeff_l2(27,45)=  3.7140002847e-01
       coeff_l2(27,46)= -2.1279616654e-01
       coeff_l2(27,47)=  8.6776532233e-02
       coeff_l2(27,48)= -5.1410242915e-02
       coeff_l2(27,49)= -2.5966519117e-01
       coeff_l2(27,50)=  1.1525943279e+00
       coeff_l2(27,51)= -2.6771423221e-01
       coeff_l2(27,52)= -1.1590018868e-01
       coeff_l2(27,53)= -1.2387850881e-01
       coeff_l2(27,54)=  1.5243338048e-01
       coeff_l2(27,55)=  1.1860562563e+00
       coeff_l2(27,56)= -6.6395038366e-01
       coeff_l2(27,57)= -5.0815593451e-02
       coeff_l2(27,58)= -5.1488757133e-01
       coeff_l2(27,59)= -9.9048078060e-02
       coeff_l2(27,60)=  9.2323817313e-02
       coeff_l2(27,61)=  2.1903632581e-01
       coeff_l2(27,62)=  2.8428882360e-01
       coeff_l2(27,63)=  1.3051246405e+00
       coeff_l2(27,64)=  4.9637293816e-01
       coeff_l2(27,65)=  3.7319394946e-01
       coeff_l2(27,66)= -1.0292761326e+00
       coeff_l2(27,67)= -6.9456994534e-01
       coeff_l2(27,68)= -5.0529941916e-02
       coeff_l2(27,69)= -2.2617502511e-01
       coeff_l2(27,70)=  1.4000320435e-01
       coeff_l2(27,71)=  1.2926182151e-01
       coeff_l2(27,72)= -4.6312364936e-01
       coeff_l2(27,73)= -1.3597297668e-01
       coeff_l2(27,74)=  2.1469598636e-02
       coeff_l2(27,75)= -6.6316622496e-01
       coeff_l2(27,76)= -3.6372655630e-01
       coeff_l2(27,77)= -3.1902685761e-01
       coeff_l2(27,78)=  3.7741910666e-02
       coeff_l2(27,79)= -4.7410836816e-01
       coeff_l2(27,80)= -3.1615149975e-01
       coeff_l2(27,81)=  1.6545890272e-01
       coeff_l2(27,82)=  3.2071739435e-01
       coeff_l2(27,83)= -1.6848488012e-03
       coeff_l2(27,84)=  1.0519617051e-01
       coeff_l2(27,85)=  1.1203765869e-01
       coeff_l2(27,86)= -2.0005689561e-01
       coeff_l2(27,87)= -2.5122317672e-01
       coeff_l2(27,88)= -1.6809651256e-01
       coeff_l2(27,89)= -2.2366586328e-01
       coeff_l2(27,90)=  1.1776695251e+00
       coeff_l2(27,91)= -3.0137917399e-01
       coeff_l2(27,92)=  2.2811180353e-01
       coeff_l2(27,93)=  2.8062224388e-01
       coeff_l2(27,94)=  1.3485360146e-01
       coeff_l2(27,95)= -3.3095248044e-02
       coeff_l2(28,1)= -1.0206466913e+00
       coeff_l2(28,2)= -4.4781770557e-02
       coeff_l2(28,3)= -2.8026533127e-01
       coeff_l2(28,4)=  1.5934623480e+00
       coeff_l2(28,5)=  3.7721699476e-01
       coeff_l2(28,6)=  8.9717882872e-01
       coeff_l2(28,7)= -3.9112567902e-01
       coeff_l2(28,8)= -1.6039164737e-02
       coeff_l2(28,9)= -2.4550311267e-01
       coeff_l2(28,10)= -3.6476007104e-01
       coeff_l2(28,11)= -8.6351290345e-02
       coeff_l2(28,12)= -8.4996454418e-02
       coeff_l2(28,13)= -7.2649919987e-01
       coeff_l2(28,14)=  8.0794151872e-03
       coeff_l2(28,15)=  5.6318100542e-02
       coeff_l2(28,16)= -6.9659060240e-01
       coeff_l2(28,17)=  5.9378061444e-02
       coeff_l2(28,18)= -1.5402039886e-01
       coeff_l2(28,19)=  4.7976348549e-02
       coeff_l2(28,20)= -2.3289784789e-01
       coeff_l2(28,21)= -4.5731744170e-01
       coeff_l2(28,22)= -3.1256136298e-01
       coeff_l2(28,23)=  5.7324815542e-02
       coeff_l2(28,24)= -1.7160226405e-01
       coeff_l2(28,25)= -2.7801364660e-01
       coeff_l2(28,26)=  2.5000444055e-01
       coeff_l2(28,27)=  1.4570643008e-01
       coeff_l2(28,28)=  7.1577441692e-01
       coeff_l2(28,29)=  1.5573605895e-01
       coeff_l2(28,30)=  7.3742307723e-02
       coeff_l2(28,31)=  5.5258095264e-01
       coeff_l2(28,32)= -9.1544613242e-02
       coeff_l2(28,33)= -1.6617375612e-01
       coeff_l2(28,34)= -3.5859085619e-03
       coeff_l2(28,35)=  1.1310718060e+00
       coeff_l2(28,36)=  1.1704198271e-01
       coeff_l2(28,37)=  4.6069866419e-01
       coeff_l2(28,38)=  2.2165167332e-01
       coeff_l2(28,39)=  9.5761650801e-01
       coeff_l2(28,40)=  5.0576013327e-01
       coeff_l2(28,41)= -6.5476931632e-02
       coeff_l2(28,42)=  1.1686876416e-01
       coeff_l2(28,43)= -9.3847984076e-01
       coeff_l2(28,44)= -5.4729902744e-01
       coeff_l2(28,45)=  2.7796873450e-01
       coeff_l2(28,46)= -3.0987086892e-01
       coeff_l2(28,47)= -3.8283243775e-01
       coeff_l2(28,48)=  5.2507881075e-02
       coeff_l2(28,49)= -5.8138418198e-01
       coeff_l2(28,50)=  1.5370855331e+00
       coeff_l2(28,51)= -8.2489028573e-02
       coeff_l2(28,52)= -1.4495432377e-01
       coeff_l2(28,53)=  1.8989017606e-01
       coeff_l2(28,54)= -1.0328945518e-01
       coeff_l2(28,55)=  1.7225749493e+00
       coeff_l2(28,56)= -8.6981028318e-02
       coeff_l2(28,57)=  2.1646317840e-01
       coeff_l2(28,58)= -8.5001587868e-02
       coeff_l2(28,59)=  1.5248736739e-01
       coeff_l2(28,60)=  8.8183715940e-02
       coeff_l2(28,61)= -6.0505073518e-02
       coeff_l2(28,62)=  4.1960504651e-01
       coeff_l2(28,63)=  2.4061517715e+00
       coeff_l2(28,64)=  1.0140573978e+00
       coeff_l2(28,65)=  2.3363590240e-01
       coeff_l2(28,66)= -1.4524183273e+00
       coeff_l2(28,67)= -5.0533980131e-01
       coeff_l2(28,68)= -3.0404549837e-01
       coeff_l2(28,69)=  6.3703924417e-02
       coeff_l2(28,70)=  1.2018100917e-01
       coeff_l2(28,71)= -4.1256442666e-01
       coeff_l2(28,72)= -1.2642841041e-01
       coeff_l2(28,73)= -6.0098931193e-02
       coeff_l2(28,74)= -4.6394807100e-01
       coeff_l2(28,75)= -3.5722190142e-01
       coeff_l2(28,76)= -3.1339358538e-03
       coeff_l2(28,77)= -6.0917472839e-01
       coeff_l2(28,78)=  2.3740120232e-01
       coeff_l2(28,79)= -1.1466867924e+00
       coeff_l2(28,80)= -5.3024303913e-01
       coeff_l2(28,81)= -1.2721100450e-01
       coeff_l2(28,82)= -4.9158569425e-02
       coeff_l2(28,83)= -5.2108359337e-01
       coeff_l2(28,84)= -5.4675202817e-02
       coeff_l2(28,85)=  2.4796208367e-02
       coeff_l2(28,86)=  2.1592348814e-01
       coeff_l2(28,87)= -1.7337749898e-01
       coeff_l2(28,88)=  1.8729221821e-01
       coeff_l2(28,89)=  3.3909669146e-03
       coeff_l2(28,90)=  2.1795016527e-01
       coeff_l2(28,91)= -7.6879858971e-02
       coeff_l2(28,92)=  3.4584240057e-03
       coeff_l2(28,93)= -1.9467911124e-01
       coeff_l2(28,94)= -2.1116401255e-01
       coeff_l2(28,95)= -2.5467592478e-01
       coeff_l2(29,1)= -5.5582404137e-01
       coeff_l2(29,2)= -1.3090598583e-01
       coeff_l2(29,3)=  6.4625993371e-02
       coeff_l2(29,4)= -2.5653012097e-02
       coeff_l2(29,5)= -1.1110870838e+00
       coeff_l2(29,6)= -4.5722499490e-01
       coeff_l2(29,7)= -1.0052598715e+00
       coeff_l2(29,8)=  1.1266252398e-01
       coeff_l2(29,9)= -2.0417217910e-01
       coeff_l2(29,10)=  5.6906044483e-01
       coeff_l2(29,11)= -5.4486638308e-01
       coeff_l2(29,12)= -1.8225799501e-01
       coeff_l2(29,13)= -4.5424422622e-01
       coeff_l2(29,14)= -3.1476435065e-01
       coeff_l2(29,15)= -2.0937256515e-01
       coeff_l2(29,16)= -1.4893643558e-01
       coeff_l2(29,17)=  1.5785435215e-02
       coeff_l2(29,18)= -1.4285254478e-01
       coeff_l2(29,19)= -4.1047683917e-03
       coeff_l2(29,20)= -1.9704085588e-01
       coeff_l2(29,21)= -4.0903127193e-01
       coeff_l2(29,22)= -5.4180437326e-01
       coeff_l2(29,23)= -2.5899758935e-01
       coeff_l2(29,24)= -7.7807269990e-02
       coeff_l2(29,25)=  1.2058698386e-01
       coeff_l2(29,26)=  3.6588340299e-04
       coeff_l2(29,27)=  2.0706453919e-01
       coeff_l2(29,28)=  8.2911014557e-02
       coeff_l2(29,29)=  7.4028201401e-02
       coeff_l2(29,30)= -1.9601342082e-01
       coeff_l2(29,31)=  1.7065389454e-01
       coeff_l2(29,32)=  2.6486292481e-03
       coeff_l2(29,33)=  1.2025352567e-01
       coeff_l2(29,34)= -2.5997946039e-02
       coeff_l2(29,35)=  9.0263044834e-01
       coeff_l2(29,36)= -7.8288102150e-01
       coeff_l2(29,37)= -3.8916456699e-01
       coeff_l2(29,38)=  1.2791624665e-01
       coeff_l2(29,39)=  5.9525642544e-02
       coeff_l2(29,40)=  1.8065454066e-01
       coeff_l2(29,41)= -2.2848220542e-03
       coeff_l2(29,42)= -5.7362660766e-02
       coeff_l2(29,43)= -2.8132748604e-01
       coeff_l2(29,44)= -1.3534940779e-01
       coeff_l2(29,45)= -9.3021154404e-02
       coeff_l2(29,46)=  4.8194584250e-01
       coeff_l2(29,47)=  3.5941290855e-01
       coeff_l2(29,48)= -4.9907606095e-02
       coeff_l2(29,49)=  2.9608505964e-01
       coeff_l2(29,50)=  1.1834992170e+00
       coeff_l2(29,51)=  3.2582074404e-02
       coeff_l2(29,52)=  6.0470737517e-02
       coeff_l2(29,53)= -3.1012386084e-01
       coeff_l2(29,54)=  3.0137437582e-01
       coeff_l2(29,55)=  6.8998731673e-02
       coeff_l2(29,56)= -2.7708691359e-01
       coeff_l2(29,57)=  1.5944836661e-02
       coeff_l2(29,58)= -3.9505928755e-01
       coeff_l2(29,59)=  1.5139569342e-01
       coeff_l2(29,60)=  8.6396351457e-02
       coeff_l2(29,61)= -4.0897327662e-01
       coeff_l2(29,62)=  1.9363135099e-01
       coeff_l2(29,63)=  1.5545675159e-01
       coeff_l2(29,64)= -1.1940243840e-01
       coeff_l2(29,65)= -8.2245481014e-01
       coeff_l2(29,66)= -8.0906087160e-01
       coeff_l2(29,67)=  2.3976160586e-01
       coeff_l2(29,68)= -7.4177116156e-02
       coeff_l2(29,69)= -2.2219328582e-01
       coeff_l2(29,70)= -1.0084660351e-01
       coeff_l2(29,71)=  4.0001538396e-01
       coeff_l2(29,72)= -2.1714615822e-01
       coeff_l2(29,73)= -8.5550189018e-02
       coeff_l2(29,74)= -2.3240640759e-01
       coeff_l2(29,75)= -2.0148314536e-01
       coeff_l2(29,76)= -1.0823044181e-01
       coeff_l2(29,77)= -2.4410839379e-01
       coeff_l2(29,78)=  1.1697982997e-01
       coeff_l2(29,79)= -1.4164803028e+00
       coeff_l2(29,80)= -5.3233879805e-01
       coeff_l2(29,81)=  1.4391265810e-01
       coeff_l2(29,82)= -1.5823933482e-01
       coeff_l2(29,83)= -1.3980247080e-01
       coeff_l2(29,84)=  1.8957144022e-01
       coeff_l2(29,85)= -5.6967359781e-01
       coeff_l2(29,86)=  1.3079008460e-01
       coeff_l2(29,87)= -2.0201914012e-01
       coeff_l2(29,88)= -7.1476861835e-02
       coeff_l2(29,89)= -3.5102021694e-01
       coeff_l2(29,90)=  7.2007441521e-01
       coeff_l2(29,91)= -3.5142108798e-01
       coeff_l2(29,92)= -1.3434566557e-01
       coeff_l2(29,93)= -3.8437879086e-01
       coeff_l2(29,94)=  4.9474611878e-02
       coeff_l2(29,95)= -2.0117731392e-01
       coeff_l2(30,1)= -4.5302078128e-02
       coeff_l2(30,2)= -2.7294930816e-01
       coeff_l2(30,3)= -1.4281429350e-01
       coeff_l2(30,4)=  5.2798252553e-02
       coeff_l2(30,5)= -2.1393157542e-01
       coeff_l2(30,6)= -1.1205217242e-01
       coeff_l2(30,7)= -7.0229947567e-01
       coeff_l2(30,8)= -5.2514243126e-01
       coeff_l2(30,9)= -5.4387584329e-02
       coeff_l2(30,10)=  2.4211168289e-01
       coeff_l2(30,11)= -1.8718786538e-02
       coeff_l2(30,12)=  2.7502369881e-01
       coeff_l2(30,13)= -2.0425072312e-01
       coeff_l2(30,14)=  1.1870329827e-01
       coeff_l2(30,15)=  9.6403166652e-02
       coeff_l2(30,16)= -1.1826375872e-01
       coeff_l2(30,17)= -1.8221676350e-01
       coeff_l2(30,18)= -1.0695912689e-01
       coeff_l2(30,19)=  1.1184594780e-01
       coeff_l2(30,20)=  9.3259818852e-02
       coeff_l2(30,21)= -4.3705113232e-02
       coeff_l2(30,22)= -1.0442455858e-01
       coeff_l2(30,23)=  2.3398626596e-02
       coeff_l2(30,24)= -8.6032018065e-02
       coeff_l2(30,25)=  1.3020336628e-01
       coeff_l2(30,26)= -3.5093903542e-01
       coeff_l2(30,27)= -1.2708717585e-01
       coeff_l2(30,28)= -8.8482640684e-02
       coeff_l2(30,29)= -1.8131691217e-01
       coeff_l2(30,30)= -1.0622221977e-01
       coeff_l2(30,31)=  1.1144318432e-01
       coeff_l2(30,32)= -1.2056586891e-01
       coeff_l2(30,33)=  3.6697557569e-01
       coeff_l2(30,34)= -5.7941341400e-01
       coeff_l2(30,35)= -8.3280675113e-02
       coeff_l2(30,36)=  4.5434528589e-01
       coeff_l2(30,37)= -2.0995759964e-01
       coeff_l2(30,38)= -2.2443728149e-01
       coeff_l2(30,39)= -1.4116893522e-02
       coeff_l2(30,40)= -3.0914884806e-01
       coeff_l2(30,41)=  5.5993669666e-03
       coeff_l2(30,42)= -3.9569705725e-01
       coeff_l2(30,43)=  3.6231848598e-01
       coeff_l2(30,44)= -2.0152407885e-01
       coeff_l2(30,45)= -3.2032597065e-01
       coeff_l2(30,46)=  1.1364101619e-01
       coeff_l2(30,47)=  2.4068616331e-01
       coeff_l2(30,48)= -1.9439318776e-01
       coeff_l2(30,49)= -8.4738776088e-02
       coeff_l2(30,50)= -1.1760088056e-01
       coeff_l2(30,51)=  1.6990913451e-01
       coeff_l2(30,52)= -2.4634844065e-01
       coeff_l2(30,53)=  1.6216678917e-01
       coeff_l2(30,54)= -2.8247466683e-01
       coeff_l2(30,55)=  8.7139338255e-02
       coeff_l2(30,56)=  1.5768969432e-02
       coeff_l2(30,57)= -3.6686787009e-01
       coeff_l2(30,58)= -2.6336389780e-01
       coeff_l2(30,59)= -1.5622410178e-01
       coeff_l2(30,60)=  1.7944286764e-01
       coeff_l2(30,61)= -2.3544521630e-01
       coeff_l2(30,62)=  2.8662824631e-01
       coeff_l2(30,63)=  3.1974684447e-02
       coeff_l2(30,64)=  4.0676444769e-01
       coeff_l2(30,65)= -4.1811674833e-01
       coeff_l2(30,66)=  3.7044581771e-01
       coeff_l2(30,67)=  3.0610316992e-01
       coeff_l2(30,68)=  1.3583616912e-01
       coeff_l2(30,69)= -3.4263290465e-02
       coeff_l2(30,70)= -2.0244142413e-01
       coeff_l2(30,71)=  2.1680349112e-01
       coeff_l2(30,72)=  1.8152380362e-03
       coeff_l2(30,73)= -4.5442396402e-01
       coeff_l2(30,74)=  9.5858864486e-02
       coeff_l2(30,75)= -2.0868779719e-01
       coeff_l2(30,76)=  7.2103552520e-02
       coeff_l2(30,77)=  9.6928648651e-02
       coeff_l2(30,78)= -1.4424636960e-01
       coeff_l2(30,79)=  4.7565951943e-01
       coeff_l2(30,80)=  2.2744953632e-01
       coeff_l2(30,81)= -3.4673288465e-01
       coeff_l2(30,82)= -1.3638798892e-01
       coeff_l2(30,83)= -2.8641444445e-01
       coeff_l2(30,84)= -3.8866204023e-01
       coeff_l2(30,85)= -1.6716182232e-01
       coeff_l2(30,86)= -2.3878383636e-01
       coeff_l2(30,87)=  2.0636558533e-02
       coeff_l2(30,88)=  1.5686991811e-01
       coeff_l2(30,89)=  1.2222911417e-01
       coeff_l2(30,90)=  1.0216282308e-01
       coeff_l2(30,91)= -1.6093816608e-02
       coeff_l2(30,92)=  1.5611203015e-01
       coeff_l2(30,93)= -3.5083273053e-01
       coeff_l2(30,94)=  1.1145456694e-02
       coeff_l2(30,95)= -4.4777181745e-01
       coeff_l2(31,1)=  7.3756611347e-01
       coeff_l2(31,2)=  3.0613955855e-01
       coeff_l2(31,3)= -2.1161365509e-01
       coeff_l2(31,4)=  1.3865557909e+00
       coeff_l2(31,5)=  8.4438633919e-01
       coeff_l2(31,6)=  4.7309234738e-01
       coeff_l2(31,7)= -2.9924979806e-01
       coeff_l2(31,8)=  1.9657898694e-02
       coeff_l2(31,9)= -1.0860726237e-01
       coeff_l2(31,10)= -7.7314472198e-01
       coeff_l2(31,11)=  4.2374752462e-02
       coeff_l2(31,12)= -1.4636018872e-01
       coeff_l2(31,13)= -3.7181758881e-01
       coeff_l2(31,14)=  9.3069627881e-02
       coeff_l2(31,15)= -7.2498962283e-02
       coeff_l2(31,16)= -9.8824396729e-02
       coeff_l2(31,17)= -2.1795372665e-01
       coeff_l2(31,18)=  5.7005863637e-02
       coeff_l2(31,19)=  1.9048932940e-02
       coeff_l2(31,20)= -3.4374698997e-01
       coeff_l2(31,21)= -6.4509123564e-02
       coeff_l2(31,22)=  6.5061673522e-02
       coeff_l2(31,23)= -1.3357394934e-01
       coeff_l2(31,24)= -7.0353657007e-02
       coeff_l2(31,25)=  1.1138964444e-01
       coeff_l2(31,26)=  5.1133972406e-01
       coeff_l2(31,27)= -1.6379129142e-02
       coeff_l2(31,28)=  4.1904639453e-02
       coeff_l2(31,29)=  7.0119276643e-02
       coeff_l2(31,30)=  4.0375730395e-01
       coeff_l2(31,31)=  2.8933110833e-01
       coeff_l2(31,32)=  1.1881135404e-01
       coeff_l2(31,33)=  8.3638109267e-02
       coeff_l2(31,34)= -5.8314707130e-02
       coeff_l2(31,35)=  9.7451418638e-01
       coeff_l2(31,36)= -2.4662689865e-01
       coeff_l2(31,37)=  8.1782370806e-01
       coeff_l2(31,38)=  2.5609709322e-02
       coeff_l2(31,39)=  6.4815300703e-01
       coeff_l2(31,40)= -7.6849788427e-02
       coeff_l2(31,41)= -6.6863857210e-02
       coeff_l2(31,42)= -1.9409552217e-02
       coeff_l2(31,43)= -4.4952228665e-01
       coeff_l2(31,44)= -2.4133072793e-01
       coeff_l2(31,45)= -3.0059704185e-01
       coeff_l2(31,46)= -1.3188609481e-01
       coeff_l2(31,47)= -3.6606156826e-01
       coeff_l2(31,48)= -2.8396931291e-01
       coeff_l2(31,49)= -1.8701526523e-01
       coeff_l2(31,50)=  2.3506435752e-01
       coeff_l2(31,51)= -5.8348670602e-02
       coeff_l2(31,52)= -3.0188161135e-01
       coeff_l2(31,53)= -6.1151552945e-02
       coeff_l2(31,54)=  1.3302415609e-01
       coeff_l2(31,55)=  2.7753347158e-01
       coeff_l2(31,56)= -1.5490106307e-02
       coeff_l2(31,57)=  1.5209595859e-01
       coeff_l2(31,58)=  1.5081803501e-01
       coeff_l2(31,59)=  5.9704497457e-02
       coeff_l2(31,60)= -1.3671988249e-01
       coeff_l2(31,61)=  1.5522283502e-02
       coeff_l2(31,62)=  4.3342947960e-02
       coeff_l2(31,63)=  1.4845482111e+00
       coeff_l2(31,64)=  6.6027230024e-01
       coeff_l2(31,65)=  3.3052837849e-01
       coeff_l2(31,66)= -7.2535210848e-01
       coeff_l2(31,67)= -4.2303147912e-01
       coeff_l2(31,68)= -4.1058745235e-02
       coeff_l2(31,69)= -1.0740636848e-02
       coeff_l2(31,70)=  2.2888782620e-01
       coeff_l2(31,71)= -1.6771849990e-01
       coeff_l2(31,72)=  2.3908194900e-01
       coeff_l2(31,73)= -2.2457960248e-01
       coeff_l2(31,74)= -1.3837397099e-01
       coeff_l2(31,75)= -2.3151220754e-02
       coeff_l2(31,76)=  3.8187414408e-02
       coeff_l2(31,77)=  7.2991855443e-02
       coeff_l2(31,78)=  5.3084702231e-03
       coeff_l2(31,79)= -5.6416045874e-02
       coeff_l2(31,80)=  4.1158825159e-01
       coeff_l2(31,81)= -5.1433749497e-02
       coeff_l2(31,82)=  2.0650102198e-01
       coeff_l2(31,83)= -1.7435872555e-01
       coeff_l2(31,84)=  1.1555267125e-01
       coeff_l2(31,85)= -1.3422846096e-03
       coeff_l2(31,86)=  1.5783596784e-02
       coeff_l2(31,87)=  7.8293815255e-02
       coeff_l2(31,88)= -2.7284222841e-01
       coeff_l2(31,89)= -3.7531468272e-01
       coeff_l2(31,90)= -5.8888810873e-01
       coeff_l2(31,91)=  6.8308107555e-02
       coeff_l2(31,92)= -9.5646552742e-02
       coeff_l2(31,93)= -1.0647730529e-01
       coeff_l2(31,94)=  4.0049634874e-02
       coeff_l2(31,95)= -2.3514355719e-01
       coeff_l2(32,1)= -1.2300304323e-01
       coeff_l2(32,2)= -5.9564912319e-01
       coeff_l2(32,3)= -1.9293665886e-02
       coeff_l2(32,4)= -7.0902816951e-02
       coeff_l2(32,5)= -9.4178432226e-01
       coeff_l2(32,6)= -4.4389122725e-01
       coeff_l2(32,7)= -4.3211925030e-01
       coeff_l2(32,8)= -4.4540639967e-02
       coeff_l2(32,9)= -1.2866634130e-01
       coeff_l2(32,10)=  1.0384327173e+00
       coeff_l2(32,11)= -2.9922306538e-01
       coeff_l2(32,12)= -2.1758820117e-01
       coeff_l2(32,13)= -1.2707291543e-01
       coeff_l2(32,14)= -8.0472946167e-02
       coeff_l2(32,15)=  2.6213705540e-02
       coeff_l2(32,16)=  3.9131313562e-02
       coeff_l2(32,17)= -9.4812765718e-02
       coeff_l2(32,18)=  1.6580554843e-01
       coeff_l2(32,19)= -4.9495115876e-02
       coeff_l2(32,20)=  8.9796157554e-03
       coeff_l2(32,21)=  1.3935041428e-01
       coeff_l2(32,22)= -2.9720392823e-01
       coeff_l2(32,23)= -1.6069993377e-01
       coeff_l2(32,24)= -3.5542778671e-02
       coeff_l2(32,25)= -3.8166037202e-01
       coeff_l2(32,26)= -4.5126470923e-01
       coeff_l2(32,27)= -8.4246858954e-02
       coeff_l2(32,28)=  2.0464029908e-01
       coeff_l2(32,29)= -1.0261995345e-01
       coeff_l2(32,30)= -4.0110796690e-02
       coeff_l2(32,31)=  1.9208630919e-01
       coeff_l2(32,32)= -6.5224125981e-02
       coeff_l2(32,33)=  2.1972076595e-01
       coeff_l2(32,34)= -7.0222184062e-02
       coeff_l2(32,35)=  6.2591326237e-01
       coeff_l2(32,36)= -1.7652982473e-01
       coeff_l2(32,37)= -7.1108955145e-01
       coeff_l2(32,38)=  5.8844525367e-02
       coeff_l2(32,39)= -2.4892368913e-01
       coeff_l2(32,40)= -7.8278899193e-02
       coeff_l2(32,41)=  1.9537155330e-01
       coeff_l2(32,42)= -7.1754917502e-02
       coeff_l2(32,43)=  3.6413034797e-01
       coeff_l2(32,44)= -1.8233433366e-01
       coeff_l2(32,45)=  4.1679224523e-05
       coeff_l2(32,46)=  5.3655999899e-01
       coeff_l2(32,47)= -1.4434100688e-01
       coeff_l2(32,48)= -6.4317770302e-03
       coeff_l2(32,49)= -4.7744470835e-01
       coeff_l2(32,50)=  1.2696442604e+00
       coeff_l2(32,51)= -1.6684135795e-01
       coeff_l2(32,52)= -1.3060120866e-02
       coeff_l2(32,53)= -4.1897844523e-02
       coeff_l2(32,54)=  2.1525779366e-01
       coeff_l2(32,55)= -2.9236671329e-01
       coeff_l2(32,56)=  1.8349885941e-01
       coeff_l2(32,57)= -3.1393894553e-01
       coeff_l2(32,58)= -4.1730785370e-01
       coeff_l2(32,59)= -1.4378687739e-01
       coeff_l2(32,60)= -1.6260181367e-01
       coeff_l2(32,61)=  3.5681724548e-02
       coeff_l2(32,62)= -8.2813344896e-02
       coeff_l2(32,63)= -1.8234184384e-01
       coeff_l2(32,64)=  6.8319924176e-02
       coeff_l2(32,65)= -8.6401253939e-01
       coeff_l2(32,66)= -3.3051109314e-01
       coeff_l2(32,67)=  4.0835285187e-01
       coeff_l2(32,68)=  3.2420790195e-01
       coeff_l2(32,69)=  2.3122951388e-03
       coeff_l2(32,70)=  1.8524201214e-01
       coeff_l2(32,71)=  4.0696784854e-01
       coeff_l2(32,72)=  5.4497681558e-03
       coeff_l2(32,73)= -5.5409178138e-02
       coeff_l2(32,74)= -1.1958061717e-02
       coeff_l2(32,75)= -3.5006648302e-01
       coeff_l2(32,76)= -1.2074620277e-01
       coeff_l2(32,77)= -5.8558923006e-01
       coeff_l2(32,78)=  1.4627471566e-01
       coeff_l2(32,79)= -5.3064608574e-01
       coeff_l2(32,80)=  6.0523748398e-01
       coeff_l2(32,81)= -2.0644041896e-01
       coeff_l2(32,82)= -7.0902490616e-01
       coeff_l2(32,83)= -9.9469825625e-02
       coeff_l2(32,84)=  6.8482942879e-02
       coeff_l2(32,85)= -1.3923627138e-01
       coeff_l2(32,86)=  1.6857929528e-01
       coeff_l2(32,87)= -3.2356080413e-01
       coeff_l2(32,88)= -2.3654367775e-02
       coeff_l2(32,89)= -3.4057646990e-01
       coeff_l2(32,90)=  5.8753395081e-01
       coeff_l2(32,91)= -2.7159217000e-01
       coeff_l2(32,92)= -1.9624739885e-01
       coeff_l2(32,93)= -2.4052728713e-01
       coeff_l2(32,94)= -1.2077308446e-01
       coeff_l2(32,95)= -1.6347879171e-01
       coeff_l2(33,1)= -2.3872752488e-01
       coeff_l2(33,2)= -4.6525552869e-02
       coeff_l2(33,3)=  3.0211892724e-01
       coeff_l2(33,4)= -1.2591485679e-01
       coeff_l2(33,5)=  5.2179533243e-01
       coeff_l2(33,6)=  2.8971225023e-01
       coeff_l2(33,7)= -9.0952940285e-02
       coeff_l2(33,8)=  3.0878457427e-01
       coeff_l2(33,9)= -1.1435875297e-01
       coeff_l2(33,10)= -9.3915611506e-01
       coeff_l2(33,11)= -4.7131955624e-01
       coeff_l2(33,12)= -1.6858619452e-01
       coeff_l2(33,13)=  1.0146118701e-01
       coeff_l2(33,14)= -4.3436938524e-01
       coeff_l2(33,15)= -1.8991246819e-01
       coeff_l2(33,16)= -2.7180323005e-01
       coeff_l2(33,17)= -1.6457197070e-01
       coeff_l2(33,18)= -5.1179885864e-02
       coeff_l2(33,19)=  1.3289600611e-01
       coeff_l2(33,20)= -4.5300608873e-01
       coeff_l2(33,21)= -1.8859298527e-01
       coeff_l2(33,22)=  2.1335418522e-01
       coeff_l2(33,23)= -1.6752603650e-01
       coeff_l2(33,24)= -9.6405006945e-02
       coeff_l2(33,25)= -1.9650565088e-01
       coeff_l2(33,26)=  3.3979439735e-01
       coeff_l2(33,27)= -9.7244873643e-02
       coeff_l2(33,28)=  4.4448593259e-01
       coeff_l2(33,29)=  5.8842591941e-02
       coeff_l2(33,30)= -8.4798932076e-02
       coeff_l2(33,31)=  1.8619459867e-01
       coeff_l2(33,32)= -1.2553855777e-01
       coeff_l2(33,33)=  3.3112917095e-02
       coeff_l2(33,34)=  5.3225278854e-01
       coeff_l2(33,35)=  1.7063504457e+00
       coeff_l2(33,36)= -1.0624305010e+00
       coeff_l2(33,37)=  4.4639274478e-01
       coeff_l2(33,38)=  8.0006830394e-02
       coeff_l2(33,39)=  2.0450757444e-01
       coeff_l2(33,40)=  5.2897143364e-01
       coeff_l2(33,41)= -1.9718058407e-01
       coeff_l2(33,42)= -6.9070063531e-02
       coeff_l2(33,43)= -6.0632610321e-01
       coeff_l2(33,44)= -1.0188651085e-01
       coeff_l2(33,45)=  4.6045094728e-01
       coeff_l2(33,46)= -2.6942560077e-01
       coeff_l2(33,47)= -1.2245047092e-01
       coeff_l2(33,48)= -1.0045013577e-01
       coeff_l2(33,49)= -9.7632832825e-02
       coeff_l2(33,50)=  1.5192277730e-01
       coeff_l2(33,51)= -2.0236422122e-01
       coeff_l2(33,52)= -2.2942890227e-01
       coeff_l2(33,53)= -6.8244062364e-02
       coeff_l2(33,54)=  3.7135356665e-01
       coeff_l2(33,55)=  1.1069477797e+00
       coeff_l2(33,56)= -5.6567114592e-01
       coeff_l2(33,57)=  5.5439823866e-01
       coeff_l2(33,58)= -5.9405010939e-01
       coeff_l2(33,59)= -1.3443440199e-01
       coeff_l2(33,60)=  1.4613969624e-01
       coeff_l2(33,61)= -2.4755769968e-01
       coeff_l2(33,62)= -2.2838346660e-02
       coeff_l2(33,63)=  8.0311363935e-01
       coeff_l2(33,64)=  4.5415124297e-01
       coeff_l2(33,65)=  5.3684079647e-01
       coeff_l2(33,66)= -1.1172379255e+00
       coeff_l2(33,67)= -1.2929284573e+00
       coeff_l2(33,68)= -2.2215268016e-01
       coeff_l2(33,69)=  1.1724805087e-01
       coeff_l2(33,70)=  2.5148680806e-01
       coeff_l2(33,71)= -7.7248233557e-01
       coeff_l2(33,72)= -3.8860607147e-01
       coeff_l2(33,73)= -2.9168042541e-01
       coeff_l2(33,74)=  4.7315269709e-01
       coeff_l2(33,75)= -1.6322258115e-01
       coeff_l2(33,76)= -5.8600794524e-02
       coeff_l2(33,77)= -1.5166752040e-01
       coeff_l2(33,78)=  1.6369436681e-01
       coeff_l2(33,79)=  4.4864794612e-01
       coeff_l2(33,80)=  4.1113492846e-01
       coeff_l2(33,81)= -3.0551560223e-02
       coeff_l2(33,82)=  8.9848348871e-03
       coeff_l2(33,83)= -1.8076899648e-01
       coeff_l2(33,84)=  4.6419929713e-02
       coeff_l2(33,85)= -6.9483458064e-03
       coeff_l2(33,86)=  4.5154878497e-01
       coeff_l2(33,87)= -8.8891789317e-02
       coeff_l2(33,88)= -3.3003845811e-01
       coeff_l2(33,89)=  4.3424460292e-01
       coeff_l2(33,90)= -4.3380877376e-01
       coeff_l2(33,91)=  1.3912917674e-01
       coeff_l2(33,92)= -1.5012083948e-01
       coeff_l2(33,93)=  9.9490985274e-02
       coeff_l2(33,94)=  1.1719376594e-01
       coeff_l2(33,95)=  3.8490390778e-01
       coeff_l2(34,1)=  2.5693103671e-02
       coeff_l2(34,2)= -4.1609227657e-01
       coeff_l2(34,3)= -3.1040251255e-01
       coeff_l2(34,4)=  2.9325631261e-01
       coeff_l2(34,5)= -1.2244116515e-01
       coeff_l2(34,6)= -1.2182120699e-03
       coeff_l2(34,7)= -7.9388491809e-02
       coeff_l2(34,8)= -6.5676309168e-02
       coeff_l2(34,9)=  2.3373171687e-01
       coeff_l2(34,10)=  6.2141090631e-01
       coeff_l2(34,11)= -1.1189939082e-01
       coeff_l2(34,12)=  2.5251764059e-01
       coeff_l2(34,13)= -4.3930092454e-01
       coeff_l2(34,14)= -2.6839298010e-01
       coeff_l2(34,15)= -1.4757323265e-01
       coeff_l2(34,16)= -3.0526164919e-02
       coeff_l2(34,17)=  2.4277720600e-02
       coeff_l2(34,18)=  2.2696076334e-01
       coeff_l2(34,19)= -1.4132523537e-01
       coeff_l2(34,20)=  8.7503492832e-02
       coeff_l2(34,21)=  2.1704243124e-01
       coeff_l2(34,22)= -2.2725044191e-01
       coeff_l2(34,23)= -1.8589445949e-01
       coeff_l2(34,24)=  1.6446624696e-01
       coeff_l2(34,25)= -1.0558905452e-01
       coeff_l2(34,26)= -3.2102245092e-01
       coeff_l2(34,27)=  1.1263708770e-01
       coeff_l2(34,28)= -9.7170703113e-02
       coeff_l2(34,29)= -1.4722418785e-01
       coeff_l2(34,30)=  1.7095376551e-01
       coeff_l2(34,31)=  3.1736508012e-01
       coeff_l2(34,32)= -3.7317071110e-02
       coeff_l2(34,33)=  2.6755377650e-01
       coeff_l2(34,34)= -4.6292051673e-01
       coeff_l2(34,35)=  7.5686830282e-01
       coeff_l2(34,36)=  2.6674512029e-01
       coeff_l2(34,37)= -6.6188067198e-01
       coeff_l2(34,38)= -2.7884364128e-02
       coeff_l2(34,39)= -3.9752002805e-02
       coeff_l2(34,40)= -4.9407657236e-02
       coeff_l2(34,41)=  9.4993904233e-02
       coeff_l2(34,42)= -1.5379349887e-01
       coeff_l2(34,43)=  6.0408967733e-01
       coeff_l2(34,44)= -2.6792258024e-01
       coeff_l2(34,45)= -3.9231014252e-01
       coeff_l2(34,46)=  4.5711138844e-01
       coeff_l2(34,47)=  2.5259953737e-01
       coeff_l2(34,48)= -2.9313001037e-01
       coeff_l2(34,49)= -2.6709148288e-01
       coeff_l2(34,50)=  6.8318921328e-01
       coeff_l2(34,51)= -4.9381867051e-02
       coeff_l2(34,52)= -9.6754327416e-02
       coeff_l2(34,53)=  8.4269657731e-02
       coeff_l2(34,54)=  5.8075774461e-02
       coeff_l2(34,55)= -8.4846876562e-02
       coeff_l2(34,56)= -1.9053986669e-01
       coeff_l2(34,57)= -6.0856914520e-01
       coeff_l2(34,58)= -2.2483742237e-01
       coeff_l2(34,59)= -2.6890784502e-01
       coeff_l2(34,60)= -1.2190563232e-01
       coeff_l2(34,61)=  1.3092036545e-01
       coeff_l2(34,62)=  1.1012708768e-02
       coeff_l2(34,63)=  3.6022165418e-01
       coeff_l2(34,64)=  6.3573546708e-02
       coeff_l2(34,65)= -5.9705269337e-01
       coeff_l2(34,66)=  3.3429795504e-01
       coeff_l2(34,67)=  9.3336158991e-01
       coeff_l2(34,68)=  3.2110676169e-01
       coeff_l2(34,69)= -2.9117891192e-01
       coeff_l2(34,70)= -1.9531258941e-01
       coeff_l2(34,71)=  8.3534324169e-01
       coeff_l2(34,72)=  6.1293687671e-02
       coeff_l2(34,73)= -2.4342407286e-01
       coeff_l2(34,74)=  8.1026151776e-02
       coeff_l2(34,75)= -2.2903275490e-01
       coeff_l2(34,76)= -1.4703065157e-01
       coeff_l2(34,77)=  1.3685534894e-01
       coeff_l2(34,78)= -2.4064382911e-01
       coeff_l2(34,79)= -2.1552585065e-01
       coeff_l2(34,80)=  7.4043415487e-02
       coeff_l2(34,81)= -1.5513363481e-01
       coeff_l2(34,82)= -4.9566322565e-01
       coeff_l2(34,83)= -1.1681602150e-01
       coeff_l2(34,84)= -2.1171462536e-01
       coeff_l2(34,85)= -7.3683641851e-02
       coeff_l2(34,86)= -1.4198882878e-01
       coeff_l2(34,87)= -1.3167291880e-01
       coeff_l2(34,88)=  2.1484781802e-01
       coeff_l2(34,89)= -5.5304031819e-02
       coeff_l2(34,90)=  5.1324254274e-01
       coeff_l2(34,91)= -2.5491672754e-01
       coeff_l2(34,92)=  1.9120867550e-01
       coeff_l2(34,93)= -3.7330079079e-01
       coeff_l2(34,94)=  1.0468322784e-01
       coeff_l2(34,95)= -2.3192740977e-01
       coeff_l2(35,1)= -7.2486913204e-01
       coeff_l2(35,2)=  6.7056097090e-02
       coeff_l2(35,3)=  6.8545639515e-01
       coeff_l2(35,4)= -3.1011816859e-01
       coeff_l2(35,5)=  2.1957507730e-01
       coeff_l2(35,6)=  1.7458635569e-01
       coeff_l2(35,7)= -6.5013366938e-01
       coeff_l2(35,8)=  1.5182809532e-01
       coeff_l2(35,9)= -2.4602200091e-01
       coeff_l2(35,10)=  2.8765782714e-01
       coeff_l2(35,11)= -8.8253326714e-02
       coeff_l2(35,12)= -2.3698616028e-01
       coeff_l2(35,13)= -1.0941012949e-01
       coeff_l2(35,14)= -2.6933357120e-01
       coeff_l2(35,15)=  1.0299804620e-02
       coeff_l2(35,16)=  1.5884219110e-01
       coeff_l2(35,17)= -2.1132452786e-01
       coeff_l2(35,18)= -2.2521038353e-01
       coeff_l2(35,19)=  5.0312217325e-02
       coeff_l2(35,20)=  1.9621910527e-02
       coeff_l2(35,21)= -2.8295633197e-01
       coeff_l2(35,22)=  7.8293867409e-02
       coeff_l2(35,23)= -1.7692004144e-01
       coeff_l2(35,24)= -2.7694737911e-01
       coeff_l2(35,25)=  2.3596072197e-01
       coeff_l2(35,26)= -9.2477127910e-03
       coeff_l2(35,27)= -2.5115311146e-01
       coeff_l2(35,28)=  4.5421960950e-01
       coeff_l2(35,29)=  2.4564513937e-02
       coeff_l2(35,30)= -3.3886343241e-02
       coeff_l2(35,31)= -1.4792862535e-01
       coeff_l2(35,32)= -3.0447903275e-01
       coeff_l2(35,33)=  2.4895708263e-01
       coeff_l2(35,34)=  1.4597390592e-01
       coeff_l2(35,35)= -5.4630476236e-01
       coeff_l2(35,36)= -9.5051281154e-02
       coeff_l2(35,37)= -3.2568022609e-01
       coeff_l2(35,38)= -5.6252382696e-02
       coeff_l2(35,39)=  4.3736344576e-01
       coeff_l2(35,40)=  3.0299079418e-01
       coeff_l2(35,41)= -1.2024046481e-01
       coeff_l2(35,42)= -1.5741281211e-02
       coeff_l2(35,43)= -1.9497446716e-01
       coeff_l2(35,44)=  3.6071610451e-01
       coeff_l2(35,45)= -1.8084369600e-01
       coeff_l2(35,46)=  4.6700004488e-02
       coeff_l2(35,47)=  2.2886808962e-02
       coeff_l2(35,48)= -8.3900228143e-02
       coeff_l2(35,49)= -2.8803086281e-01
       coeff_l2(35,50)=  1.0106607676e+00
       coeff_l2(35,51)= -3.1721028686e-01
       coeff_l2(35,52)=  1.0429278016e-01
       coeff_l2(35,53)=  8.1672862172e-02
       coeff_l2(35,54)=  2.0715788007e-02
       coeff_l2(35,55)= -1.3834531419e-02
       coeff_l2(35,56)= -2.1306899190e-01
       coeff_l2(35,57)= -2.8747396544e-02
       coeff_l2(35,58)= -6.6044348478e-01
       coeff_l2(35,59)= -1.6883783042e-01
       coeff_l2(35,60)=  1.1075478047e-01
       coeff_l2(35,61)= -8.6328513920e-02
       coeff_l2(35,62)=  2.7093115449e-01
       coeff_l2(35,63)= -1.0172406584e-01
       coeff_l2(35,64)= -2.6933491230e-01
       coeff_l2(35,65)=  2.9111519456e-01
       coeff_l2(35,66)=  3.1375203282e-02
       coeff_l2(35,67)= -2.2600144148e-01
       coeff_l2(35,68)= -1.1315088719e-02
       coeff_l2(35,69)= -1.2336874753e-01
       coeff_l2(35,70)=  2.8762897477e-02
       coeff_l2(35,71)=  4.1997230053e-01
       coeff_l2(35,72)=  1.7155933380e-01
       coeff_l2(35,73)= -2.0768666267e-01
       coeff_l2(35,74)= -1.1355914921e-01
       coeff_l2(35,75)= -3.4351482987e-02
       coeff_l2(35,76)= -2.2979338467e-01
       coeff_l2(35,77)=  1.9837367535e-01
       coeff_l2(35,78)= -1.9227880985e-02
       coeff_l2(35,79)= -3.2242143154e-01
       coeff_l2(35,80)= -2.7325880527e-01
       coeff_l2(35,81)= -3.8035072386e-02
       coeff_l2(35,82)=  5.3046739101e-01
       coeff_l2(35,83)= -4.8761535436e-02
       coeff_l2(35,84)=  1.3156196475e-01
       coeff_l2(35,85)= -1.9173856080e-01
       coeff_l2(35,86)= -1.9980125129e-01
       coeff_l2(35,87)=  2.8619736433e-01
       coeff_l2(35,88)= -1.8767237663e-01
       coeff_l2(35,89)=  1.0521798581e-01
       coeff_l2(35,90)=  2.7416130900e-01
       coeff_l2(35,91)=  3.1017264724e-01
       coeff_l2(35,92)=  7.5946971774e-02
       coeff_l2(35,93)=  1.4475540817e-01
       coeff_l2(35,94)= -1.0700739920e-01
       coeff_l2(35,95)=  3.3084341884e-01
       coeff_l2(36,1)= -4.5351505280e-01
       coeff_l2(36,2)= -2.5546485186e-01
       coeff_l2(36,3)= -3.2768401504e-01
       coeff_l2(36,4)=  3.8344648480e-01
       coeff_l2(36,5)=  2.9226036742e-02
       coeff_l2(36,6)= -3.3359733224e-01
       coeff_l2(36,7)=  3.7609245628e-02
       coeff_l2(36,8)=  2.3109407723e-01
       coeff_l2(36,9)=  2.8440186381e-01
       coeff_l2(36,10)=  6.5638577938e-01
       coeff_l2(36,11)=  8.1877388060e-02
       coeff_l2(36,12)=  4.0340445936e-02
       coeff_l2(36,13)= -1.2228023261e-01
       coeff_l2(36,14)= -1.5461274981e-01
       coeff_l2(36,15)=  3.3366778493e-01
       coeff_l2(36,16)= -1.7690771818e-01
       coeff_l2(36,17)= -1.6699758172e-01
       coeff_l2(36,18)= -7.1801804006e-02
       coeff_l2(36,19)= -7.2699241340e-02
       coeff_l2(36,20)=  1.3463015854e-01
       coeff_l2(36,21)=  1.8664893508e-01
       coeff_l2(36,22)= -2.6602923870e-01
       coeff_l2(36,23)= -7.7525079250e-02
       coeff_l2(36,24)=  9.1058053076e-02
       coeff_l2(36,25)=  5.9232741594e-02
       coeff_l2(36,26)= -2.7709624171e-01
       coeff_l2(36,27)= -2.9885167256e-02
       coeff_l2(36,28)= -1.9033826888e-01
       coeff_l2(36,29)=  2.9528904706e-02
       coeff_l2(36,30)=  1.6998708248e-01
       coeff_l2(36,31)=  4.6538367867e-02
       coeff_l2(36,32)= -1.2342908233e-01
       coeff_l2(36,33)=  4.0516942739e-02
       coeff_l2(36,34)=  8.3835408092e-02
       coeff_l2(36,35)= -1.8192945048e-02
       coeff_l2(36,36)=  1.5338917077e-01
       coeff_l2(36,37)= -2.5543883443e-01
       coeff_l2(36,38)=  4.6198159456e-02
       coeff_l2(36,39)= -1.6809654236e-01
       coeff_l2(36,40)= -1.4062796533e-01
       coeff_l2(36,41)=  2.8414193541e-02
       coeff_l2(36,42)=  2.2684687376e-01
       coeff_l2(36,43)=  3.9186719060e-01
       coeff_l2(36,44)= -2.6124808192e-01
       coeff_l2(36,45)= -2.1759554744e-01
       coeff_l2(36,46)=  1.1519294977e-01
       coeff_l2(36,47)= -1.7943321168e-01
       coeff_l2(36,48)=  2.8616082668e-01
       coeff_l2(36,49)= -7.2259497829e-03
       coeff_l2(36,50)=  5.5085575581e-01
       coeff_l2(36,51)=  9.9590204656e-02
       coeff_l2(36,52)=  4.8109378666e-02
       coeff_l2(36,53)=  9.3317046762e-02
       coeff_l2(36,54)= -2.2551707923e-01
       coeff_l2(36,55)=  4.1634026915e-02
       coeff_l2(36,56)= -2.9977405071e-01
       coeff_l2(36,57)= -3.4155753255e-01
       coeff_l2(36,58)= -1.4992518350e-02
       coeff_l2(36,59)= -1.2041973323e-01
       coeff_l2(36,60)=  1.2358349562e-01
       coeff_l2(36,61)= -2.2200716659e-02
       coeff_l2(36,62)=  4.4876617193e-01
       coeff_l2(36,63)= -9.1722486541e-03
       coeff_l2(36,64)= -4.0783572942e-02
       coeff_l2(36,65)= -4.4820713997e-01
       coeff_l2(36,66)=  4.6810263395e-01
       coeff_l2(36,67)=  8.2031166553e-01
       coeff_l2(36,68)=  1.9917128980e-01
       coeff_l2(36,69)= -2.3344843090e-01
       coeff_l2(36,70)=  3.0019793659e-02
       coeff_l2(36,71)=  3.8954681158e-01
       coeff_l2(36,72)= -1.3076027855e-02
       coeff_l2(36,73)=  8.1195741892e-02
       coeff_l2(36,74)= -1.6010938585e-01
       coeff_l2(36,75)= -4.0718767792e-02
       coeff_l2(36,76)= -4.8607867211e-02
       coeff_l2(36,77)=  5.1002241671e-02
       coeff_l2(36,78)=  2.3446136713e-01
       coeff_l2(36,79)=  3.6526057869e-02
       coeff_l2(36,80)= -3.9422929287e-02
       coeff_l2(36,81)= -2.2030185163e-01
       coeff_l2(36,82)= -3.5182875395e-01
       coeff_l2(36,83)=  2.9526010156e-02
       coeff_l2(36,84)=  6.1985813081e-02
       coeff_l2(36,85)=  4.0275458246e-02
       coeff_l2(36,86)= -2.1567723155e-01
       coeff_l2(36,87)=  7.9987727106e-02
       coeff_l2(36,88)=  6.4590618014e-02
       coeff_l2(36,89)=  2.3232875764e-01
       coeff_l2(36,90)=  3.3170816302e-01
       coeff_l2(36,91)= -3.0366554856e-01
       coeff_l2(36,92)= -1.3280901313e-01
       coeff_l2(36,93)= -5.7734362781e-02
       coeff_l2(36,94)=  9.1630183160e-02
       coeff_l2(36,95)= -3.1893554330e-01
       coeff_l2(37,1)= -5.3114306927e-01
       coeff_l2(37,2)=  2.5522656739e-02
       coeff_l2(37,3)= -3.8631838560e-01
       coeff_l2(37,4)= -1.8444150686e-01
       coeff_l2(37,5)= -6.4341217279e-01
       coeff_l2(37,6)= -3.0256378651e-01
       coeff_l2(37,7)= -4.3051987886e-01
       coeff_l2(37,8)=  9.5974780619e-02
       coeff_l2(37,9)= -3.2147708535e-01
       coeff_l2(37,10)=  2.4953137338e-01
       coeff_l2(37,11)=  3.4917220473e-02
       coeff_l2(37,12)= -9.0908765793e-02
       coeff_l2(37,13)= -1.2186755193e-03
       coeff_l2(37,14)= -4.7000959516e-01
       coeff_l2(37,15)= -3.3697411418e-01
       coeff_l2(37,16)=  8.9526727796e-02
       coeff_l2(37,17)=  1.5648947656e-01
       coeff_l2(37,18)=  8.8726310059e-03
       coeff_l2(37,19)= -3.1868880987e-01
       coeff_l2(37,20)= -3.1058585271e-02
       coeff_l2(37,21)= -3.7076777220e-01
       coeff_l2(37,22)=  5.6487705559e-02
       coeff_l2(37,23)=  1.8460232019e-01
       coeff_l2(37,24)=  1.1990995705e-01
       coeff_l2(37,25)=  1.2214709073e-01
       coeff_l2(37,26)=  1.3852025568e-01
       coeff_l2(37,27)=  1.5836012363e-01
       coeff_l2(37,28)=  2.7804997563e-01
       coeff_l2(37,29)= -2.0928107202e-01
       coeff_l2(37,30)= -1.1585008353e-01
       coeff_l2(37,31)=  3.5322475433e-01
       coeff_l2(37,32)=  1.2612840533e-01
       coeff_l2(37,33)= -8.6887635291e-02
       coeff_l2(37,34)= -2.4266104400e-01
       coeff_l2(37,35)=  4.2403388023e-01
       coeff_l2(37,36)= -6.7843848467e-01
       coeff_l2(37,37)= -5.2973091602e-01
       coeff_l2(37,38)= -3.0379850417e-02
       coeff_l2(37,39)=  3.0556872487e-01
       coeff_l2(37,40)= -1.2241560966e-01
       coeff_l2(37,41)=  1.4710344374e-01
       coeff_l2(37,42)=  1.3051365316e-01
       coeff_l2(37,43)= -6.0451525450e-01
       coeff_l2(37,44)= -2.4522078037e-01
       coeff_l2(37,45)= -3.2055824995e-01
       coeff_l2(37,46)=  6.9139420986e-01
       coeff_l2(37,47)=  1.2397669256e-01
       coeff_l2(37,48)=  8.8898114860e-02
       coeff_l2(37,49)= -6.8025849760e-02
       coeff_l2(37,50)=  8.6533212662e-01
       coeff_l2(37,51)= -1.1327655613e-01
       coeff_l2(37,52)=  6.0293287970e-03
       coeff_l2(37,53)= -2.1895317733e-01
       coeff_l2(37,54)= -3.2286092639e-02
       coeff_l2(37,55)= -2.3400647938e-01
       coeff_l2(37,56)= -3.6092621088e-01
       coeff_l2(37,57)= -4.2281750590e-02
       coeff_l2(37,58)= -4.4258084893e-01
       coeff_l2(37,59)= -9.1676957905e-02
       coeff_l2(37,60)=  8.1207595766e-02
       coeff_l2(37,61)= -3.3321252465e-01
       coeff_l2(37,62)=  3.3483186364e-01
       coeff_l2(37,63)= -1.0204213858e-01
       coeff_l2(37,64)= -3.2572585344e-01
       coeff_l2(37,65)= -2.6842581108e-02
       coeff_l2(37,66)= -1.5631830692e-01
       coeff_l2(37,67)= -2.1040830016e-01
       coeff_l2(37,68)= -5.5533293635e-03
       coeff_l2(37,69)=  1.8655593693e-01
       coeff_l2(37,70)=  3.8723920006e-03
       coeff_l2(37,71)=  3.6724126339e-01
       coeff_l2(37,72)= -2.7544263005e-01
       coeff_l2(37,73)= -2.3594641685e-01
       coeff_l2(37,74)= -1.0610895604e-01
       coeff_l2(37,75)= -6.0694456100e-02
       coeff_l2(37,76)= -1.0520331562e-02
       coeff_l2(37,77)= -2.3501488566e-01
       coeff_l2(37,78)= -6.0059186071e-02
       coeff_l2(37,79)= -6.7322379351e-01
       coeff_l2(37,80)= -3.5872402787e-01
       coeff_l2(37,81)= -4.6505278349e-01
       coeff_l2(37,82)= -2.9054915905e-01
       coeff_l2(37,83)= -2.6357921958e-01
       coeff_l2(37,84)= -7.4183136225e-02
       coeff_l2(37,85)= -2.0772734284e-01
       coeff_l2(37,86)=  3.0069786310e-01
       coeff_l2(37,87)= -8.9904613793e-02
       coeff_l2(37,88)= -1.0435538739e-01
       coeff_l2(37,89)= -3.5017549992e-01
       coeff_l2(37,90)=  2.2591857612e-01
       coeff_l2(37,91)=  2.6571038365e-01
       coeff_l2(37,92)=  8.2129314542e-02
       coeff_l2(37,93)= -7.6247282326e-02
       coeff_l2(37,94)=  5.5412195623e-02
       coeff_l2(37,95)= -4.1108834743e-01
       coeff_l2(38,1)=  2.6758340001e-01
       coeff_l2(38,2)=  3.6285567284e-01
       coeff_l2(38,3)=  7.8448124230e-02
       coeff_l2(38,4)= -7.9661622643e-02
       coeff_l2(38,5)= -2.6018795371e-01
       coeff_l2(38,6)=  5.0448700786e-02
       coeff_l2(38,7)=  2.8652021289e-01
       coeff_l2(38,8)= -1.0966298729e-01
       coeff_l2(38,9)=  3.1292998791e-01
       coeff_l2(38,10)=  4.8288732767e-01
       coeff_l2(38,11)=  5.7057060301e-02
       coeff_l2(38,12)=  2.2612267733e-01
       coeff_l2(38,13)=  4.7785913944e-01
       coeff_l2(38,14)= -2.9890537262e-02
       coeff_l2(38,15)=  3.2885849476e-01
       coeff_l2(38,16)= -9.8849078640e-03
       coeff_l2(38,17)= -9.4639033079e-02
       coeff_l2(38,18)= -2.7388104796e-01
       coeff_l2(38,19)= -6.0986630619e-02
       coeff_l2(38,20)=  2.8459569812e-01
       coeff_l2(38,21)=  1.9389504194e-01
       coeff_l2(38,22)=  1.8560145691e-05
       coeff_l2(38,23)= -6.5680004656e-02
       coeff_l2(38,24)= -4.6468663961e-02
       coeff_l2(38,25)=  1.0327223688e-01
       coeff_l2(38,26)= -1.0177430697e-02
       coeff_l2(38,27)= -1.4438284934e-01
       coeff_l2(38,28)= -1.3135810196e-01
       coeff_l2(38,29)=  4.9585681409e-02
       coeff_l2(38,30)=  4.9050085247e-02
       coeff_l2(38,31)= -8.2212246954e-02
       coeff_l2(38,32)= -2.0109055936e-01
       coeff_l2(38,33)=  3.4093059599e-02
       coeff_l2(38,34)= -1.7868481576e-01
       coeff_l2(38,35)= -6.5102897584e-02
       coeff_l2(38,36)= -2.1166564897e-02
       coeff_l2(38,37)= -6.4048355818e-01
       coeff_l2(38,38)= -5.1241267473e-02
       coeff_l2(38,39)= -4.1062036157e-01
       coeff_l2(38,40)= -2.9173167422e-04
       coeff_l2(38,41)= -7.7332914807e-03
       coeff_l2(38,42)= -2.4245826900e-01
       coeff_l2(38,43)=  3.7246090174e-01
       coeff_l2(38,44)=  3.0342536047e-02
       coeff_l2(38,45)= -1.1699161679e-01
       coeff_l2(38,46)=  4.5870717615e-02
       coeff_l2(38,47)=  8.7190449238e-02
       coeff_l2(38,48)=  2.3582780734e-02
       coeff_l2(38,49)=  1.3736128807e-01
       coeff_l2(38,50)= -2.9629442096e-01
       coeff_l2(38,51)=  3.1218624115e-01
       coeff_l2(38,52)= -1.4314173162e-01
       coeff_l2(38,53)=  3.0921434984e-02
       coeff_l2(38,54)= -2.9502657056e-01
       coeff_l2(38,55)= -2.6523973793e-03
       coeff_l2(38,56)=  4.2054768652e-02
       coeff_l2(38,57)= -2.8715567663e-02
       coeff_l2(38,58)= -7.9293571413e-02
       coeff_l2(38,59)= -1.4123438299e-01
       coeff_l2(38,60)=  1.0940794647e-01
       coeff_l2(38,61)=  1.9346466288e-03
       coeff_l2(38,62)= -6.0612410307e-01
       coeff_l2(38,63)=  8.3723425865e-02
       coeff_l2(38,64)=  5.6380107999e-02
       coeff_l2(38,65)= -2.5854942203e-01
       coeff_l2(38,66)=  8.7057101727e-01
       coeff_l2(38,67)=  7.5938045979e-01
       coeff_l2(38,68)=  1.7950810492e-01
       coeff_l2(38,69)= -8.4934599698e-02
       coeff_l2(38,70)= -4.2687472701e-01
       coeff_l2(38,71)=  1.5272808075e-01
       coeff_l2(38,72)=  1.7466923222e-02
       coeff_l2(38,73)= -1.1052719317e-02
       coeff_l2(38,74)= -6.1250317842e-02
       coeff_l2(38,75)= -1.6949729621e-01
       coeff_l2(38,76)=  1.3936622441e-01
       coeff_l2(38,77)=  6.2625162303e-02
       coeff_l2(38,78)= -1.8262942135e-01
       coeff_l2(38,79)=  2.3882399499e-01
       coeff_l2(38,80)=  1.8884700537e-01
       coeff_l2(38,81)=  7.3205277324e-02
       coeff_l2(38,82)= -3.3950439095e-01
       coeff_l2(38,83)=  9.5317952335e-02
       coeff_l2(38,84)= -1.1989872903e-01
       coeff_l2(38,85)=  1.0547709465e-01
       coeff_l2(38,86)= -1.4881570637e-01
       coeff_l2(38,87)=  3.8582009077e-01
       coeff_l2(38,88)=  1.5129184723e-01
       coeff_l2(38,89)= -2.7540165186e-01
       coeff_l2(38,90)=  1.1581658572e-01
       coeff_l2(38,91)= -7.7807321213e-03
       coeff_l2(38,92)= -7.9535007477e-02
       coeff_l2(38,93)=  4.0201578289e-02
       coeff_l2(38,94)=  2.8252154589e-01
       coeff_l2(38,95)= -4.0446391702e-01
       coeff_l2(39,1)=  1.5264530480e-01
       coeff_l2(39,2)= -5.4917775095e-02
       coeff_l2(39,3)= -2.7604889870e-01
       coeff_l2(39,4)=  2.1017250419e-01
       coeff_l2(39,5)= -1.1064906418e-01
       coeff_l2(39,6)= -3.2799112797e-01
       coeff_l2(39,7)= -6.2482725829e-02
       coeff_l2(39,8)= -1.9917117059e-01
       coeff_l2(39,9)=  7.4728675187e-02
       coeff_l2(39,10)=  5.4429334402e-01
       coeff_l2(39,11)= -1.6005042195e-01
       coeff_l2(39,12)=  8.4767483175e-02
       coeff_l2(39,13)=  5.1558729261e-02
       coeff_l2(39,14)= -7.5809970498e-02
       coeff_l2(39,15)=  2.2832278907e-01
       coeff_l2(39,16)=  6.9658793509e-02
       coeff_l2(39,17)= -1.7087557912e-01
       coeff_l2(39,18)=  2.5718701072e-03
       coeff_l2(39,19)= -2.1037945151e-01
       coeff_l2(39,20)= -8.3321422338e-02
       coeff_l2(39,21)=  5.8799892664e-02
       coeff_l2(39,22)=  1.1845906824e-01
       coeff_l2(39,23)=  3.2201748341e-02
       coeff_l2(39,24)=  9.2863470316e-02
       coeff_l2(39,25)= -1.2230522186e-01
       coeff_l2(39,26)= -4.0604490787e-03
       coeff_l2(39,27)= -2.0060120150e-02
       coeff_l2(39,28)=  1.0019465536e-01
       coeff_l2(39,29)= -1.9451077282e-01
       coeff_l2(39,30)=  1.3893373311e-01
       coeff_l2(39,31)=  1.8091656268e-01
       coeff_l2(39,32)= -2.2197037935e-02
       coeff_l2(39,33)=  1.5461413562e-01
       coeff_l2(39,34)=  3.5913072526e-02
       coeff_l2(39,35)=  7.0558160543e-01
       coeff_l2(39,36)= -2.2600083053e-01
       coeff_l2(39,37)= -6.0405611992e-01
       coeff_l2(39,38)= -1.0281918198e-01
       coeff_l2(39,39)= -2.5622582436e-01
       coeff_l2(39,40)=  4.3292433023e-02
       coeff_l2(39,41)= -5.3108528256e-02
       coeff_l2(39,42)= -1.6635791957e-01
       coeff_l2(39,43)=  7.2511836886e-02
       coeff_l2(39,44)= -4.7104537487e-02
       coeff_l2(39,45)= -1.3458316028e-01
       coeff_l2(39,46)=  3.8485172391e-01
       coeff_l2(39,47)=  1.0502468050e-01
       coeff_l2(39,48)=  9.5269195735e-02
       coeff_l2(39,49)= -1.8185302615e-01
       coeff_l2(39,50)=  1.5839205682e-01
       coeff_l2(39,51)=  1.4720014296e-02
       coeff_l2(39,52)= -2.6753988862e-01
       coeff_l2(39,53)=  1.3232868910e-01
       coeff_l2(39,54)= -3.1198415905e-02
       coeff_l2(39,55)=  1.8979795277e-01
       coeff_l2(39,56)= -5.1872473955e-01
       coeff_l2(39,57)= -2.7937164903e-01
       coeff_l2(39,58)= -1.2474146485e-01
       coeff_l2(39,59)= -2.2357343137e-01
       coeff_l2(39,60)=  1.8844291568e-01
       coeff_l2(39,61)=  3.3986698836e-02
       coeff_l2(39,62)=  3.5409256816e-01
       coeff_l2(39,63)=  2.0904007554e-01
       coeff_l2(39,64)=  2.0359505713e-01
       coeff_l2(39,65)= -1.6655489802e-02
       coeff_l2(39,66)=  5.7839924097e-01
       coeff_l2(39,67)=  9.1367429495e-01
       coeff_l2(39,68)=  3.4578412771e-01
       coeff_l2(39,69)= -2.9434400797e-01
       coeff_l2(39,70)= -1.9203646481e-01
       coeff_l2(39,71)=  1.6430145502e-01
       coeff_l2(39,72)= -4.3409389257e-01
       coeff_l2(39,73)= -7.4131332338e-02
       coeff_l2(39,74)= -8.7186098099e-02
       coeff_l2(39,75)= -1.2350244075e-01
       coeff_l2(39,76)=  5.4661709815e-02
       coeff_l2(39,77)=  1.2251380831e-01
       coeff_l2(39,78)= -1.3636839390e-01
       coeff_l2(39,79)= -3.5893931985e-01
       coeff_l2(39,80)= -5.4462641478e-02
       coeff_l2(39,81)= -1.4188005030e-01
       coeff_l2(39,82)= -5.4170864820e-01
       coeff_l2(39,83)=  1.9766052067e-01
       coeff_l2(39,84)= -2.7033781633e-02
       coeff_l2(39,85)= -1.5816622972e-01
       coeff_l2(39,86)= -1.9181509316e-01
       coeff_l2(39,87)= -1.8848094344e-01
       coeff_l2(39,88)=  1.6292594373e-01
       coeff_l2(39,89)=  2.9944169521e-01
       coeff_l2(39,90)=  3.3420690894e-01
       coeff_l2(39,91)= -2.2378081083e-01
       coeff_l2(39,92)= -1.3970442116e-01
       coeff_l2(39,93)= -1.8292656541e-01
       coeff_l2(39,94)=  2.0619489253e-01
       coeff_l2(39,95)= -2.5009810925e-01
       coeff_l2(40,1)=  8.1423807144e-01
       coeff_l2(40,2)=  5.0003761053e-01
       coeff_l2(40,3)= -1.2063923478e-01
       coeff_l2(40,4)= -7.3469436169e-01
       coeff_l2(40,5)=  4.6050551534e-01
       coeff_l2(40,6)=  1.8402309716e-01
       coeff_l2(40,7)=  4.0758043528e-01
       coeff_l2(40,8)= -3.6387839913e-01
       coeff_l2(40,9)=  9.9987231195e-02
       coeff_l2(40,10)= -8.0763208866e-01
       coeff_l2(40,11)=  2.0544971526e-01
       coeff_l2(40,12)=  2.2190932930e-01
       coeff_l2(40,13)=  1.3279695809e-01
       coeff_l2(40,14)= -2.1622030437e-01
       coeff_l2(40,15)= -5.5443239212e-01
       coeff_l2(40,16)=  3.5680106282e-01
       coeff_l2(40,17)= -1.5044023097e-01
       coeff_l2(40,18)= -2.3330381513e-01
       coeff_l2(40,19)= -1.2626239657e-01
       coeff_l2(40,20)=  1.3283784688e-01
       coeff_l2(40,21)= -4.8772299290e-01
       coeff_l2(40,22)=  7.5351327658e-01
       coeff_l2(40,23)= -2.3647511005e-01
       coeff_l2(40,24)= -1.4926326275e-01
       coeff_l2(40,25)=  4.8570847511e-01
       coeff_l2(40,26)=  4.7964030504e-01
       coeff_l2(40,27)= -2.2186760604e-01
       coeff_l2(40,28)= -3.1319305301e-02
       coeff_l2(40,29)= -9.9360011518e-02
       coeff_l2(40,30)= -1.3535656035e-01
       coeff_l2(40,31)= -2.8595209122e-02
       coeff_l2(40,32)= -2.0817913115e-01
       coeff_l2(40,33)= -7.8377708793e-02
       coeff_l2(40,34)= -2.6910617948e-01
       coeff_l2(40,35)= -1.2240982056e+00
       coeff_l2(40,36)= -1.1553261429e-01
       coeff_l2(40,37)=  3.3661833405e-01
       coeff_l2(40,38)= -2.2601301968e-01
       coeff_l2(40,39)= -3.4798610955e-02
       coeff_l2(40,40)= -3.9143458009e-01
       coeff_l2(40,41)= -7.8324162960e-01
       coeff_l2(40,42)= -7.2807550430e-01
       coeff_l2(40,43)=  6.6644823551e-01
       coeff_l2(40,44)=  1.8561150134e-01
       coeff_l2(40,45)=  1.8039456010e-01
       coeff_l2(40,46)=  2.6095890999e-01
       coeff_l2(40,47)=  1.9318696856e-01
       coeff_l2(40,48)= -3.9302593470e-01
       coeff_l2(40,49)=  3.0790165067e-01
       coeff_l2(40,50)= -1.2733973563e-02
       coeff_l2(40,51)=  1.0441192240e-01
       coeff_l2(40,52)= -3.8012048602e-01
       coeff_l2(40,53)=  1.4604011178e-01
       coeff_l2(40,54)= -4.5528045297e-01
       coeff_l2(40,55)= -6.2034040689e-01
       coeff_l2(40,56)= -2.3081028461e-01
       coeff_l2(40,57)=  6.5383201838e-01
       coeff_l2(40,58)=  9.4102960825e-01
       coeff_l2(40,59)= -4.5837002993e-01
       coeff_l2(40,60)= -8.6653679609e-02
       coeff_l2(40,61)= -4.2269977927e-01
       coeff_l2(40,62)=  1.4669845998e-01
       coeff_l2(40,63)= -8.8658970594e-01
       coeff_l2(40,64)=  3.5166749358e-01
       coeff_l2(40,65)=  5.3161513805e-01
       coeff_l2(40,66)=  6.5901947021e-01
       coeff_l2(40,67)=  1.8538978696e-01
       coeff_l2(40,68)=  1.9126051664e-01
       coeff_l2(40,69)= -3.1738966703e-01
       coeff_l2(40,70)= -1.5611059964e-01
       coeff_l2(40,71)=  3.1732073426e-01
       coeff_l2(40,72)= -3.0961123109e-01
       coeff_l2(40,73)= -1.0174845457e+00
       coeff_l2(40,74)=  2.4758048356e-01
       coeff_l2(40,75)= -1.3311426342e-01
       coeff_l2(40,76)=  7.9534292221e-02
       coeff_l2(40,77)=  4.0036550164e-01
       coeff_l2(40,78)= -3.7806966901e-01
       coeff_l2(40,79)=  1.0485001802e+00
       coeff_l2(40,80)=  3.6084365845e-01
       coeff_l2(40,81)= -3.6404922605e-02
       coeff_l2(40,82)=  2.0901364088e-01
       coeff_l2(40,83)= -2.6829943061e-01
       coeff_l2(40,84)= -6.2839043140e-01
       coeff_l2(40,85)= -4.9343362451e-01
       coeff_l2(40,86)= -9.1576971114e-02
       coeff_l2(40,87)= -1.5605664253e-01
       coeff_l2(40,88)=  2.8015288711e-01
       coeff_l2(40,89)= -1.6662298143e-01
       coeff_l2(40,90)= -2.7670180798e-01
       coeff_l2(40,91)=  2.7400213480e-01
       coeff_l2(40,92)= -3.0572624877e-02
       coeff_l2(40,93)=  2.1718443930e-01
       coeff_l2(40,94)= -1.6914676130e-01
       coeff_l2(40,95)=  3.4489285946e-01
       coeff_l2(41,1)= -3.6917799711e-01
       coeff_l2(41,2)= -5.8057926595e-02
       coeff_l2(41,3)=  7.9357912764e-03
       coeff_l2(41,4)=  1.3449287415e-01
       coeff_l2(41,5)= -1.6841199994e-01
       coeff_l2(41,6)= -1.8703697622e-01
       coeff_l2(41,7)= -5.7160985470e-01
       coeff_l2(41,8)= -2.3651441559e-02
       coeff_l2(41,9)= -1.1335480958e-02
       coeff_l2(41,10)=  2.8194662929e-01
       coeff_l2(41,11)= -1.3964281976e-01
       coeff_l2(41,12)= -1.3617020845e-01
       coeff_l2(41,13)= -1.9530786574e-01
       coeff_l2(41,14)= -2.5830727816e-01
       coeff_l2(41,15)=  1.6346175969e-01
       coeff_l2(41,16)= -1.3477264345e-01
       coeff_l2(41,17)= -1.7283516005e-02
       coeff_l2(41,18)=  4.9985904247e-02
       coeff_l2(41,19)= -5.9041187167e-02
       coeff_l2(41,20)= -5.0166472793e-02
       coeff_l2(41,21)=  1.6981221735e-01
       coeff_l2(41,22)= -2.0805174112e-01
       coeff_l2(41,23)= -1.7151290178e-01
       coeff_l2(41,24)= -1.2872022390e-01
       coeff_l2(41,25)= -2.2799198329e-01
       coeff_l2(41,26)= -1.9507004321e-01
       coeff_l2(41,27)=  1.8159957230e-01
       coeff_l2(41,28)= -6.5454974771e-02
       coeff_l2(41,29)=  1.1189761572e-02
       coeff_l2(41,30)=  1.7473493516e-01
       coeff_l2(41,31)=  2.9798907042e-01
       coeff_l2(41,32)= -1.2748771906e-01
       coeff_l2(41,33)= -6.6359497607e-02
       coeff_l2(41,34)=  1.9326403737e-01
       coeff_l2(41,35)=  8.6231690645e-01
       coeff_l2(41,36)=  2.0635052025e-01
       coeff_l2(41,37)= -1.1149816215e-01
       coeff_l2(41,38)= -2.2000906989e-02
       coeff_l2(41,39)=  4.5079809427e-01
       coeff_l2(41,40)=  1.4836937189e-01
       coeff_l2(41,41)=  2.6890102029e-01
       coeff_l2(41,42)=  1.1735963821e-01
       coeff_l2(41,43)= -1.3784557581e-01
       coeff_l2(41,44)= -4.0957588702e-02
       coeff_l2(41,45)= -2.9716706276e-01
       coeff_l2(41,46)=  9.9262677133e-02
       coeff_l2(41,47)= -7.3081911542e-03
       coeff_l2(41,48)= -1.0275363177e-01
       coeff_l2(41,49)= -2.9243635014e-02
       coeff_l2(41,50)=  7.2059750557e-01
       coeff_l2(41,51)= -1.5646329522e-01
       coeff_l2(41,52)= -1.0866442323e-01
       coeff_l2(41,53)=  2.8309246525e-02
       coeff_l2(41,54)=  1.7506735027e-01
       coeff_l2(41,55)=  3.6500617862e-01
       coeff_l2(41,56)= -9.2615857720e-02
       coeff_l2(41,57)= -1.0112229735e-01
       coeff_l2(41,58)= -4.2664220929e-01
       coeff_l2(41,59)=  2.1628595889e-02
       coeff_l2(41,60)= -1.3906225562e-01
       coeff_l2(41,61)= -1.0980071872e-01
       coeff_l2(41,62)=  1.9512793422e-01
       coeff_l2(41,63)=  4.8506271839e-01
       coeff_l2(41,64)=  2.8642880917e-01
       coeff_l2(41,65)=  3.4962151200e-02
       coeff_l2(41,66)= -3.3511292934e-01
       coeff_l2(41,67)= -3.9572891593e-01
       coeff_l2(41,68)= -1.8284414709e-01
       coeff_l2(41,69)= -1.6920050979e-01
       coeff_l2(41,70)= -3.8340426981e-02
       coeff_l2(41,71)= -2.7978757024e-01
       coeff_l2(41,72)= -1.5275867283e-01
       coeff_l2(41,73)=  1.0257966071e-01
       coeff_l2(41,74)= -7.3501653969e-02
       coeff_l2(41,75)=  8.8327474892e-02
       coeff_l2(41,76)=  1.2400169671e-01
       coeff_l2(41,77)= -3.4832257032e-01
       coeff_l2(41,78)=  1.6284085810e-01
       coeff_l2(41,79)= -2.8484222293e-01
       coeff_l2(41,80)= -3.2086956501e-01
       coeff_l2(41,81)= -8.7753556669e-02
       coeff_l2(41,82)=  1.2713478506e-01
       coeff_l2(41,83)= -1.3497655094e-01
       coeff_l2(41,84)=  5.3753308021e-03
       coeff_l2(41,85)= -2.7712121606e-01
       coeff_l2(41,86)=  1.4885449409e-01
       coeff_l2(41,87)= -3.5332258791e-02
       coeff_l2(41,88)= -2.8008666635e-01
       coeff_l2(41,89)= -3.8660481572e-02
       coeff_l2(41,90)=  2.9426753521e-01
       coeff_l2(41,91)= -1.5261645615e-01
       coeff_l2(41,92)= -2.6013452560e-02
       coeff_l2(41,93)=  2.4756988510e-02
       coeff_l2(41,94)= -1.2232793868e-01
       coeff_l2(41,95)= -1.8489637971e-01
       coeff_l2(42,1)=  4.2532995343e-01
       coeff_l2(42,2)=  4.8137170076e-01
       coeff_l2(42,3)= -8.5410511494e-01
       coeff_l2(42,4)=  2.9635119438e-01
       coeff_l2(42,5)=  1.5393171310e+00
       coeff_l2(42,6)=  9.8910409212e-01
       coeff_l2(42,7)=  3.6504462361e-01
       coeff_l2(42,8)= -3.2102605700e-01
       coeff_l2(42,9)= -6.1947286129e-01
       coeff_l2(42,10)= -2.1259655952e+00
       coeff_l2(42,11)= -2.6294243336e-01
       coeff_l2(42,12)= -5.2214646339e-01
       coeff_l2(42,13)= -4.6797385812e-01
       coeff_l2(42,14)= -1.4169133902e+00
       coeff_l2(42,15)= -3.6912593246e-01
       coeff_l2(42,16)=  3.2325056195e-01
       coeff_l2(42,17)= -1.7647179961e-01
       coeff_l2(42,18)= -2.2835397720e-01
       coeff_l2(42,19)= -2.5952264667e-01
       coeff_l2(42,20)= -2.4168014526e-01
       coeff_l2(42,21)= -8.0677610636e-01
       coeff_l2(42,22)=  5.2698415518e-01
       coeff_l2(42,23)= -2.1627651155e-01
       coeff_l2(42,24)= -1.9084961712e-01
       coeff_l2(42,25)= -1.8948353827e-01
       coeff_l2(42,26)=  7.7351254225e-01
       coeff_l2(42,27)= -3.2567095757e-01
       coeff_l2(42,28)=  3.5733383894e-01
       coeff_l2(42,29)=  5.5392660201e-02
       coeff_l2(42,30)= -4.1126957536e-01
       coeff_l2(42,31)=  2.7747198939e-01
       coeff_l2(42,32)= -1.6596736014e-01
       coeff_l2(42,33)= -7.7402985096e-01
       coeff_l2(42,34)= -8.4194475412e-01
       coeff_l2(42,35)=  2.6471367478e-01
       coeff_l2(42,36)= -6.5062588453e-01
       coeff_l2(42,37)=  1.1376039982e+00
       coeff_l2(42,38)= -2.2811690345e-02
       coeff_l2(42,39)=  6.5780621767e-01
       coeff_l2(42,40)=  1.2706360221e-01
       coeff_l2(42,41)= -6.9404131174e-01
       coeff_l2(42,42)= -3.5289549828e-01
       coeff_l2(42,43)=  2.7420282364e-01
       coeff_l2(42,44)=  5.0334924459e-01
       coeff_l2(42,45)=  6.5509909391e-01
       coeff_l2(42,46)= -6.1872833967e-01
       coeff_l2(42,47)= -5.4688382149e-01
       coeff_l2(42,48)=  8.9761652052e-02
       coeff_l2(42,49)=  2.5433707237e-01
       coeff_l2(42,50)=  7.8309822083e-01
       coeff_l2(42,51)= -2.9544714093e-01
       coeff_l2(42,52)= -1.7094433308e-01
       coeff_l2(42,53)=  2.8732424974e-01
       coeff_l2(42,54)= -1.4494872093e-01
       coeff_l2(42,55)=  3.2549777627e-01
       coeff_l2(42,56)= -1.2412859201e+00
       coeff_l2(42,57)=  8.5202497244e-01
       coeff_l2(42,58)=  5.2518117428e-01
       coeff_l2(42,59)= -2.3084419966e-01
       coeff_l2(42,60)= -2.7906903625e-01
       coeff_l2(42,61)= -5.4024678469e-01
       coeff_l2(42,62)=  3.6332270503e-01
       coeff_l2(42,63)= -8.2516461611e-02
       coeff_l2(42,64)=  7.2536087036e-01
       coeff_l2(42,65)=  9.9025231600e-01
       coeff_l2(42,66)= -2.0514743030e-01
       coeff_l2(42,67)= -1.1515132189e+00
       coeff_l2(42,68)= -4.3121585250e-01
       coeff_l2(42,69)= -2.5308755040e-01
       coeff_l2(42,70)= -2.6687243581e-01
       coeff_l2(42,71)= -7.8791922331e-01
       coeff_l2(42,72)= -4.4158354402e-01
       coeff_l2(42,73)= -8.1392514706e-01
       coeff_l2(42,74)=  4.4627928734e-01
       coeff_l2(42,75)=  1.6191303730e-01
       coeff_l2(42,76)= -1.9349746406e-01
       coeff_l2(42,77)= -9.9539086223e-02
       coeff_l2(42,78)= -4.1436064243e-01
       coeff_l2(42,79)=  1.7023289204e+00
       coeff_l2(42,80)=  6.0961765051e-01
       coeff_l2(42,81)= -3.4344458580e-01
       coeff_l2(42,82)=  1.2148209810e+00
       coeff_l2(42,83)= -4.9237906933e-01
       coeff_l2(42,84)= -5.4146504402e-01
       coeff_l2(42,85)= -2.9176121950e-01
       coeff_l2(42,86)=  1.6886310279e-01
       coeff_l2(42,87)=  4.7755353153e-03
       coeff_l2(42,88)= -9.0762287378e-02
       coeff_l2(42,89)=  6.7267619073e-02
       coeff_l2(42,90)= -1.3505725861e+00
       coeff_l2(42,91)=  7.0681136847e-01
       coeff_l2(42,92)=  4.2463924736e-02
       coeff_l2(42,93)=  2.3058508337e-01
       coeff_l2(42,94)= -2.5511735678e-01
       coeff_l2(42,95)= -7.3261511326e-01
       coeff_l2(43,1)=  4.4390740991e-01
       coeff_l2(43,2)= -1.4266668260e-01
       coeff_l2(43,3)=  1.1794447899e-01
       coeff_l2(43,4)= -2.2145450115e+00
       coeff_l2(43,5)=  4.6288520098e-01
       coeff_l2(43,6)= -5.6421458721e-01
       coeff_l2(43,7)= -1.0629707575e+00
       coeff_l2(43,8)= -2.4182967842e-01
       coeff_l2(43,9)= -2.5664207339e-01
       coeff_l2(43,10)= -8.4948576987e-02
       coeff_l2(43,11)=  1.3039356470e-01
       coeff_l2(43,12)= -3.6617019773e-01
       coeff_l2(43,13)=  1.6626438498e-01
       coeff_l2(43,14)= -6.4091789722e-01
       coeff_l2(43,15)=  2.8353534639e-02
       coeff_l2(43,16)=  7.6155233383e-01
       coeff_l2(43,17)= -6.2679827213e-02
       coeff_l2(43,18)= -7.5766697526e-02
       coeff_l2(43,19)=  5.9128653258e-02
       coeff_l2(43,20)= -5.0442051888e-01
       coeff_l2(43,21)=  2.1553412080e-01
       coeff_l2(43,22)=  4.0342193842e-01
       coeff_l2(43,23)= -7.1349635720e-02
       coeff_l2(43,24)= -1.5329468250e-01
       coeff_l2(43,25)=  4.3048062921e-01
       coeff_l2(43,26)=  5.3866559267e-01
       coeff_l2(43,27)= -2.1852277219e-01
       coeff_l2(43,28)= -1.9647696614e-01
       coeff_l2(43,29)= -2.7745172381e-01
       coeff_l2(43,30)=  1.2445383519e-01
       coeff_l2(43,31)= -8.8775390387e-01
       coeff_l2(43,32)= -3.5317125916e-01
       coeff_l2(43,33)= -3.0725792050e-01
       coeff_l2(43,34)= -5.0846058130e-01
       coeff_l2(43,35)= -2.5004136562e+00
       coeff_l2(43,36)=  9.9490188062e-02
       coeff_l2(43,37)=  9.3032449484e-01
       coeff_l2(43,38)= -3.0026065186e-02
       coeff_l2(43,39)= -1.7455153167e-02
       coeff_l2(43,40)= -2.9056718946e-01
       coeff_l2(43,41)= -1.1276460439e-01
       coeff_l2(43,42)= -5.3052264452e-01
       coeff_l2(43,43)=  1.9864970446e-01
       coeff_l2(43,44)=  2.7352258563e-01
       coeff_l2(43,45)= -6.9507431984e-01
       coeff_l2(43,46)= -2.0626375079e-01
       coeff_l2(43,47)= -7.5454212725e-02
       coeff_l2(43,48)= -2.0679281652e-01
       coeff_l2(43,49)=  1.9976759329e-02
       coeff_l2(43,50)= -1.1141064018e-01
       coeff_l2(43,51)= -4.6502584219e-01
       coeff_l2(43,52)=  3.9924975485e-02
       coeff_l2(43,53)= -4.5015177131e-01
       coeff_l2(43,54)=  2.0441721380e-01
       coeff_l2(43,55)= -5.8844262362e-01
       coeff_l2(43,56)= -3.0712994933e-01
       coeff_l2(43,57)=  3.1914868951e-01
       coeff_l2(43,58)= -5.2064001560e-01
       coeff_l2(43,59)= -3.4271740913e-01
       coeff_l2(43,60)=  2.0492618904e-02
       coeff_l2(43,61)= -1.1335575581e-01
       coeff_l2(43,62)=  7.7411484718e-01
       coeff_l2(43,63)= -2.8459191322e+00
       coeff_l2(43,64)=  4.6473443508e-02
       coeff_l2(43,65)=  3.0618596077e-01
       coeff_l2(43,66)=  1.9396100193e-02
       coeff_l2(43,67)= -5.6591695547e-01
       coeff_l2(43,68)= -3.6487510800e-01
       coeff_l2(43,69)=  4.8517037183e-02
       coeff_l2(43,70)= -3.3296874166e-01
       coeff_l2(43,71)= -8.4240943193e-01
       coeff_l2(43,72)=  1.8422400951e-01
       coeff_l2(43,73)= -9.3502444029e-01
       coeff_l2(43,74)= -2.1111400425e-01
       coeff_l2(43,75)= -3.0831837654e-01
       coeff_l2(43,76)= -1.1935418844e-01
       coeff_l2(43,77)=  3.5512618721e-02
       coeff_l2(43,78)= -6.3653677702e-01
       coeff_l2(43,79)=  7.4085998535e-01
       coeff_l2(43,80)=  6.9405895472e-01
       coeff_l2(43,81)= -2.6009824872e-01
       coeff_l2(43,82)=  8.7538921833e-01
       coeff_l2(43,83)=  5.2895814180e-02
       coeff_l2(43,84)= -5.0819742680e-01
       coeff_l2(43,85)= -1.3852959871e-01
       coeff_l2(43,86)= -3.8388201594e-01
       coeff_l2(43,87)=  8.8788434863e-02
       coeff_l2(43,88)= -5.9362137318e-01
       coeff_l2(43,89)=  3.2294178009e-01
       coeff_l2(43,90)= -1.3188365698e+00
       coeff_l2(43,91)=  1.3644298911e-01
       coeff_l2(43,92)= -2.0767097175e-01
       coeff_l2(43,93)= -6.1274993420e-01
       coeff_l2(43,94)= -5.8515793085e-01
       coeff_l2(43,95)= -6.6098397970e-01
       coeff_l2(44,1)= -3.5096129775e-01
       coeff_l2(44,2)= -3.6507385969e-01
       coeff_l2(44,3)=  1.0396917909e-01
       coeff_l2(44,4)=  3.5023361444e-02
       coeff_l2(44,5)=  3.5372945666e-01
       coeff_l2(44,6)=  5.0380420685e-01
       coeff_l2(44,7)=  2.2878573835e-01
       coeff_l2(44,8)=  1.4411954582e-01
       coeff_l2(44,9)= -4.6216413379e-02
       coeff_l2(44,10)= -7.9064065218e-01
       coeff_l2(44,11)= -2.9655085877e-02
       coeff_l2(44,12)= -1.6686403751e-01
       coeff_l2(44,13)= -1.2118014693e-01
       coeff_l2(44,14)= -2.5012981892e-01
       coeff_l2(44,15)= -1.6469724476e-01
       coeff_l2(44,16)= -3.0790576339e-01
       coeff_l2(44,17)=  2.2994531691e-01
       coeff_l2(44,18)= -1.1592777073e-01
       coeff_l2(44,19)=  1.5869064629e-01
       coeff_l2(44,20)= -2.0721861720e-01
       coeff_l2(44,21)=  1.5804458410e-02
       coeff_l2(44,22)= -5.1598154008e-02
       coeff_l2(44,23)= -2.3982319236e-01
       coeff_l2(44,24)= -4.4690351933e-02
       coeff_l2(44,25)= -1.1503817886e-01
       coeff_l2(44,26)= -7.6038320549e-03
       coeff_l2(44,27)= -2.5559566915e-02
       coeff_l2(44,28)=  4.0465083718e-01
       coeff_l2(44,29)=  2.3301234469e-02
       coeff_l2(44,30)=  7.6649151742e-02
       coeff_l2(44,31)= -9.1451354325e-02
       coeff_l2(44,32)=  1.1038186401e-01
       coeff_l2(44,33)=  1.0217953473e-02
       coeff_l2(44,34)=  1.7612011731e-01
       coeff_l2(44,35)=  3.0991647243e+00
       coeff_l2(44,36)= -1.5490001440e-01
       coeff_l2(44,37)=  5.6854796410e-01
       coeff_l2(44,38)=  6.7573770881e-02
       coeff_l2(44,39)=  2.0580761135e-01
       coeff_l2(44,40)=  1.2463837862e-01
       coeff_l2(44,41)= -2.0199269056e-01
       coeff_l2(44,42)= -2.0537145436e-01
       coeff_l2(44,43)= -6.4000779390e-01
       coeff_l2(44,44)=  2.0719937980e-01
       coeff_l2(44,45)=  2.2025570273e-01
       coeff_l2(44,46)= -5.9776103497e-01
       coeff_l2(44,47)= -3.4603923559e-01
       coeff_l2(44,48)=  1.1516220123e-01
       coeff_l2(44,49)=  1.0814005136e-01
       coeff_l2(44,50)=  3.6952573061e-01
       coeff_l2(44,51)=  2.2824572027e-01
       coeff_l2(44,52)=  3.7173464894e-02
       coeff_l2(44,53)= -1.5402059257e-01
       coeff_l2(44,54)= -1.2867318094e-01
       coeff_l2(44,55)=  5.8941411972e-01
       coeff_l2(44,56)= -3.4789538383e-01
       coeff_l2(44,57)=  3.8025832176e-01
       coeff_l2(44,58)= -1.3341125846e-01
       coeff_l2(44,59)=  7.0114418864e-02
       coeff_l2(44,60)= -1.2736761570e-01
       coeff_l2(44,61)= -1.6194799542e-01
       coeff_l2(44,62)=  2.2566045821e-01
       coeff_l2(44,63)=  3.0779936910e-01
       coeff_l2(44,64)= -8.8396053761e-03
       coeff_l2(44,65)=  2.3343460262e-01
       coeff_l2(44,66)= -5.9581911564e-01
       coeff_l2(44,67)= -2.1467010677e-01
       coeff_l2(44,68)= -1.9134178758e-02
       coeff_l2(44,69)= -2.1845455468e-01
       coeff_l2(44,70)=  8.5065796971e-02
       coeff_l2(44,71)= -1.4758425951e-01
       coeff_l2(44,72)= -3.4214109182e-01
       coeff_l2(44,73)=  1.5598171949e-01
       coeff_l2(44,74)=  4.1381320357e-01
       coeff_l2(44,75)= -6.6800527275e-02
       coeff_l2(44,76)= -1.7241568863e-01
       coeff_l2(44,77)= -1.3131400943e-01
       coeff_l2(44,78)= -4.7279568389e-03
       coeff_l2(44,79)=  3.7567040324e-01
       coeff_l2(44,80)=  1.9077597558e-01
       coeff_l2(44,81)= -7.2773560882e-02
       coeff_l2(44,82)=  3.0761379004e-01
       coeff_l2(44,83)= -1.0731298476e-01
       coeff_l2(44,84)= -5.6221384555e-02
       coeff_l2(44,85)= -2.4694949389e-01
       coeff_l2(44,86)=  1.2510541081e-01
       coeff_l2(44,87)= -2.2040973604e-01
       coeff_l2(44,88)= -1.5658542514e-01
       coeff_l2(44,89)=  2.7567435056e-02
       coeff_l2(44,90)=  2.7852538228e-01
       coeff_l2(44,91)= -3.9447449148e-02
       coeff_l2(44,92)= -5.3781550378e-02
       coeff_l2(44,93)= -4.9846637994e-02
       coeff_l2(44,94)=  1.4173074067e-01
       coeff_l2(44,95)=  7.6922118664e-02
       coeff_l2(45,1)= -1.9273334742e+00
       coeff_l2(45,2)= -1.2507782876e-01
       coeff_l2(45,3)=  7.6897002757e-02
       coeff_l2(45,4)=  2.5189262629e-01
       coeff_l2(45,5)= -1.0101788044e+00
       coeff_l2(45,6)= -1.6634242535e+00
       coeff_l2(45,7)= -1.6917377710e+00
       coeff_l2(45,8)= -1.0158614814e-01
       coeff_l2(45,9)= -3.4682713449e-02
       coeff_l2(45,10)=  2.8302369118e+00
       coeff_l2(45,11)= -7.4120485783e-01
       coeff_l2(45,12)= -4.1108358651e-02
       coeff_l2(45,13)= -9.2023581266e-01
       coeff_l2(45,14)= -3.0522733927e-01
       coeff_l2(45,15)=  3.0776074529e-01
       coeff_l2(45,16)= -2.8888663650e-01
       coeff_l2(45,17)= -1.6500771046e-01
       coeff_l2(45,18)= -3.1852301955e-01
       coeff_l2(45,19)=  3.2814013958e-01
       coeff_l2(45,20)= -3.0715888739e-01
       coeff_l2(45,21)=  2.2217005491e-01
       coeff_l2(45,22)= -1.0638470650e+00
       coeff_l2(45,23)= -1.2714995444e-01
       coeff_l2(45,24)= -4.5603695512e-01
       coeff_l2(45,25)=  5.8971440792e-01
       coeff_l2(45,26)= -3.4631824493e-01
       coeff_l2(45,27)= -2.5636470318e-01
       coeff_l2(45,28)=  1.2463960797e-01
       coeff_l2(45,29)= -4.7470297664e-02
       coeff_l2(45,30)= -2.3656240664e-03
       coeff_l2(45,31)= -3.7200170755e-01
       coeff_l2(45,32)= -5.4286543280e-02
       coeff_l2(45,33)=  6.6484332085e-01
       coeff_l2(45,34)=  3.9739364386e-01
       coeff_l2(45,35)=  1.2049778700e+00
       coeff_l2(45,36)= -5.2055400610e-01
       coeff_l2(45,37)= -1.6447092295e+00
       coeff_l2(45,38)= -3.4919444472e-02
       coeff_l2(45,39)=  4.0414562821e-01
       coeff_l2(45,40)=  6.6892826557e-01
       coeff_l2(45,41)= -9.4641983509e-02
       coeff_l2(45,42)= -1.5694470704e-01
       coeff_l2(45,43)=  7.5520002842e-01
       coeff_l2(45,44)= -3.8808619976e-01
       coeff_l2(45,45)= -1.3283038139e+00
       coeff_l2(45,46)=  1.4890679121e+00
       coeff_l2(45,47)=  1.6390523911e+00
       coeff_l2(45,48)=  1.5432460234e-02
       coeff_l2(45,49)= -3.3643224835e-01
       coeff_l2(45,50)=  2.3225405216e+00
       coeff_l2(45,51)= -9.0876020491e-02
       coeff_l2(45,52)= -1.8898716569e-01
       coeff_l2(45,53)= -3.6886534095e-01
       coeff_l2(45,54)=  1.4866712689e-01
       coeff_l2(45,55)=  8.5815793276e-01
       coeff_l2(45,56)= -1.8507638574e-01
       coeff_l2(45,57)= -1.0123488903e+00
       coeff_l2(45,58)= -9.1691577435e-01
       coeff_l2(45,59)= -6.4131337404e-01
       coeff_l2(45,60)=  1.0760712624e-01
       coeff_l2(45,61)= -1.5639796853e-01
       coeff_l2(45,62)=  1.0047382116e+00
       coeff_l2(45,63)=  1.2403103709e-01
       coeff_l2(45,64)=  8.3393171430e-02
       coeff_l2(45,65)= -7.0691674948e-01
       coeff_l2(45,66)=  6.8846064806e-01
       coeff_l2(45,67)=  2.0932996273e+00
       coeff_l2(45,68)=  3.3239752054e-01
       coeff_l2(45,69)= -2.3309139535e-02
       coeff_l2(45,70)= -2.5102245808e-01
       coeff_l2(45,71)=  1.6594226360e+00
       coeff_l2(45,72)=  9.1526284814e-02
       coeff_l2(45,73)= -4.9752074480e-01
       coeff_l2(45,74)= -4.7072291374e-02
       coeff_l2(45,75)= -2.6919010282e-01
       coeff_l2(45,76)= -2.3386548460e-01
       coeff_l2(45,77)=  3.7852126360e-01
       coeff_l2(45,78)= -4.0116256475e-01
       coeff_l2(45,79)= -1.3952491283e+00
       coeff_l2(45,80)= -1.3134573698e+00
       coeff_l2(45,81)= -4.2820006609e-01
       coeff_l2(45,82)= -9.3631744385e-01
       coeff_l2(45,83)= -2.3845314980e-01
       coeff_l2(45,84)= -2.2431266308e-01
       coeff_l2(45,85)= -7.1718716621e-01
       coeff_l2(45,86)=  9.5622474328e-03
       coeff_l2(45,87)= -1.5056125820e-01
       coeff_l2(45,88)= -4.2252460122e-01
       coeff_l2(45,89)=  6.2466714531e-02
       coeff_l2(45,90)=  1.1551014185e+00
       coeff_l2(45,91)= -6.9358837605e-01
       coeff_l2(45,92)=  4.2232540250e-01
       coeff_l2(45,93)= -1.6626188755e+00
       coeff_l2(45,94)= -7.9524628818e-02
       coeff_l2(45,95)= -3.8090807199e-01
       coeff_l2(46,1)= -3.2024046779e-01
       coeff_l2(46,2)= -9.1875739396e-02
       coeff_l2(46,3)= -2.2844463587e-01
       coeff_l2(46,4)=  1.7440184951e-01
       coeff_l2(46,5)= -4.8762211204e-01
       coeff_l2(46,6)= -2.8449276090e-01
       coeff_l2(46,7)= -3.3704343438e-01
       coeff_l2(46,8)= -1.0055484623e-01
       coeff_l2(46,9)= -1.7162440345e-02
       coeff_l2(46,10)= -2.3557281494e-01
       coeff_l2(46,11)=  6.7203857005e-02
       coeff_l2(46,12)= -4.4471666217e-02
       coeff_l2(46,13)= -3.1883299351e-01
       coeff_l2(46,14)= -9.9347927608e-04
       coeff_l2(46,15)= -1.3353005052e-01
       coeff_l2(46,16)= -3.5829129815e-01
       coeff_l2(46,17)= -7.1306623518e-02
       coeff_l2(46,18)=  1.3278207183e-01
       coeff_l2(46,19)=  5.9235477820e-03
       coeff_l2(46,20)= -6.0879565775e-02
       coeff_l2(46,21)= -1.5387124382e-02
       coeff_l2(46,22)= -1.3786888123e-01
       coeff_l2(46,23)= -1.7334792018e-01
       coeff_l2(46,24)=  1.4923566580e-01
       coeff_l2(46,25)= -1.4932978153e-01
       coeff_l2(46,26)=  6.0811769217e-02
       coeff_l2(46,27)=  1.5612877905e-01
       coeff_l2(46,28)=  3.9276685566e-02
       coeff_l2(46,29)= -7.2589486837e-02
       coeff_l2(46,30)= -6.0713640414e-04
       coeff_l2(46,31)=  2.1907792985e-01
       coeff_l2(46,32)=  2.2935409844e-01
       coeff_l2(46,33)= -7.1651265025e-02
       coeff_l2(46,34)=  7.5809180737e-02
       coeff_l2(46,35)=  4.0511739254e-01
       coeff_l2(46,36)= -4.1966089606e-01
       coeff_l2(46,37)= -6.8755820394e-02
       coeff_l2(46,38)= -9.9474191666e-02
       coeff_l2(46,39)=  4.4976112247e-01
       coeff_l2(46,40)= -2.1537883207e-02
       coeff_l2(46,41)=  1.7885746434e-02
       coeff_l2(46,42)=  3.9166118950e-02
       coeff_l2(46,43)=  1.0044602677e-02
       coeff_l2(46,44)= -3.0209216475e-01
       coeff_l2(46,45)= -2.4575254321e-01
       coeff_l2(46,46)=  1.2652225792e-01
       coeff_l2(46,47)= -6.4807951450e-01
       coeff_l2(46,48)=  1.7649289966e-01
       coeff_l2(46,49)= -1.9148695469e-01
       coeff_l2(46,50)=  1.0185040236e+00
       coeff_l2(46,51)= -1.5704925358e-01
       coeff_l2(46,52)= -1.7737033963e-01
       coeff_l2(46,53)= -1.5169622004e-01
       coeff_l2(46,54)=  7.9092830420e-02
       coeff_l2(46,55)=  1.3193467259e-01
       coeff_l2(46,56)= -1.9853931665e-01
       coeff_l2(46,57)= -4.2887848616e-01
       coeff_l2(46,58)= -4.7909596562e-01
       coeff_l2(46,59)= -4.7004502267e-02
       coeff_l2(46,60)= -9.5278024673e-02
       coeff_l2(46,61)= -9.2576354742e-02
       coeff_l2(46,62)=  4.1311579943e-01
       coeff_l2(46,63)=  5.8596426249e-01
       coeff_l2(46,64)=  6.7334763706e-02
       coeff_l2(46,65)=  7.5472109020e-02
       coeff_l2(46,66)= -6.0589307547e-01
       coeff_l2(46,67)=  1.9148091972e-01
       coeff_l2(46,68)=  1.9393195212e-01
       coeff_l2(46,69)= -1.4560183883e-01
       coeff_l2(46,70)= -1.3470935822e-01
       coeff_l2(46,71)=  2.4946422875e-01
       coeff_l2(46,72)=  7.1170203388e-02
       coeff_l2(46,73)=  7.6086133718e-02
       coeff_l2(46,74)=  1.8385660648e-01
       coeff_l2(46,75)= -7.3531828821e-02
       coeff_l2(46,76)= -1.3783778250e-01
       coeff_l2(46,77)= -4.2534923553e-01
       coeff_l2(46,78)=  1.0077323020e-01
       coeff_l2(46,79)= -6.0632520914e-01
       coeff_l2(46,80)=  8.5877358913e-02
       coeff_l2(46,81)= -2.5121697783e-01
       coeff_l2(46,82)= -2.5636509061e-01
       coeff_l2(46,83)= -4.9103043973e-02
       coeff_l2(46,84)= -1.2363481522e-01
       coeff_l2(46,85)=  2.0099715889e-01
       coeff_l2(46,86)=  2.3802520335e-01
       coeff_l2(46,87)= -1.1471968889e-01
       coeff_l2(46,88)= -6.8223260343e-02
       coeff_l2(46,89)= -1.9239154458e-01
       coeff_l2(46,90)=  5.7650750875e-01
       coeff_l2(46,91)= -2.1393230557e-01
       coeff_l2(46,92)=  2.6446646452e-01
       coeff_l2(46,93)= -8.1114672124e-02
       coeff_l2(46,94)=  3.0573310796e-03
       coeff_l2(46,95)= -4.6914435923e-02
       coeff_l2(47,1)=  1.3184513152e-01
       coeff_l2(47,2)= -3.7345027924e-01
       coeff_l2(47,3)= -3.0061560869e-01
       coeff_l2(47,4)= -2.8822462559e+00
       coeff_l2(47,5)= -4.0759170055e-01
       coeff_l2(47,6)= -1.2310155630e+00
       coeff_l2(47,7)=  2.7765586972e-01
       coeff_l2(47,8)= -2.3887362331e-03
       coeff_l2(47,9)= -7.4678070843e-02
       coeff_l2(47,10)=  4.6663317084e-01
       coeff_l2(47,11)= -1.1702314764e-01
       coeff_l2(47,12)= -2.6135665178e-01
       coeff_l2(47,13)= -1.0668020695e-01
       coeff_l2(47,14)=  1.9905653596e-01
       coeff_l2(47,15)= -6.5827852488e-01
       coeff_l2(47,16)= -6.5248265862e-02
       coeff_l2(47,17)= -1.2981046736e-01
       coeff_l2(47,18)=  6.4690527506e-03
       coeff_l2(47,19)=  6.2595475465e-03
       coeff_l2(47,20)= -1.9284109771e-01
       coeff_l2(47,21)=  9.5081515610e-03
       coeff_l2(47,22)= -6.6290533543e-01
       coeff_l2(47,23)=  4.4734537601e-01
       coeff_l2(47,24)=  4.0138956159e-02
       coeff_l2(47,25)= -5.6172007322e-01
       coeff_l2(47,26)= -6.1324167252e-01
       coeff_l2(47,27)= -2.9408430681e-02
       coeff_l2(47,28)= -4.0685287118e-01
       coeff_l2(47,29)= -2.1541406214e-01
       coeff_l2(47,30)=  1.6913244128e-01
       coeff_l2(47,31)= -1.2548167706e+00
       coeff_l2(47,32)=  1.9914600253e-01
       coeff_l2(47,33)= -4.4176340103e-02
       coeff_l2(47,34)= -2.8340123594e-02
       coeff_l2(47,35)= -2.8233337402e+00
       coeff_l2(47,36)= -3.7569922209e-01
       coeff_l2(47,37)= -4.8614713550e-01
       coeff_l2(47,38)= -6.0138426721e-02
       coeff_l2(47,39)= -5.3587895632e-01
       coeff_l2(47,40)= -7.0721007884e-02
       coeff_l2(47,41)=  2.7078711987e-01
       coeff_l2(47,42)= -1.1665753275e-01
       coeff_l2(47,43)=  3.0568623543e-01
       coeff_l2(47,44)=  1.4841911197e-01
       coeff_l2(47,45)= -1.0045024753e-01
       coeff_l2(47,46)= -1.6956085339e-02
       coeff_l2(47,47)= -3.4965050220e-01
       coeff_l2(47,48)=  9.2741124332e-02
       coeff_l2(47,49)= -3.0504524708e-01
       coeff_l2(47,50)= -5.5929762125e-01
       coeff_l2(47,51)= -5.2319067717e-01
       coeff_l2(47,52)= -8.2022941206e-04
       coeff_l2(47,53)=  1.3224668801e-02
       coeff_l2(47,54)=  2.9808682203e-01
       coeff_l2(47,55)= -9.9629050493e-01
       coeff_l2(47,56)=  9.7373597324e-02
       coeff_l2(47,57)= -4.8260718584e-01
       coeff_l2(47,58)= -5.6175184250e-01
       coeff_l2(47,59)=  9.2757552862e-02
       coeff_l2(47,60)= -3.6290842295e-01
       coeff_l2(47,61)=  3.5397239029e-02
       coeff_l2(47,62)= -3.1969055533e-01
       coeff_l2(47,63)= -3.5650167465e+00
       coeff_l2(47,64)= -4.9971824884e-01
       coeff_l2(47,65)= -7.3984122276e-01
       coeff_l2(47,66)= -2.0206543803e-01
       coeff_l2(47,67)=  3.3593064547e-01
       coeff_l2(47,68)= -1.4102419838e-02
       coeff_l2(47,69)= -1.0674636811e-01
       coeff_l2(47,70)=  1.7495715618e-01
       coeff_l2(47,71)=  9.9010318518e-02
       coeff_l2(47,72)=  1.8635137379e-01
       coeff_l2(47,73)= -2.5658091903e-01
       coeff_l2(47,74)= -1.2865555286e-01
       coeff_l2(47,75)= -7.1674208157e-03
       coeff_l2(47,76)= -1.3828603923e-01
       coeff_l2(47,77)= -1.3070195913e-01
       coeff_l2(47,78)= -4.8669150472e-01
       coeff_l2(47,79)=  2.6338452101e-01
       coeff_l2(47,80)= -8.7567515671e-02
       coeff_l2(47,81)= -2.1988698840e-01
       coeff_l2(47,82)= -5.1673412323e-01
       coeff_l2(47,83)=  1.3225707412e-01
       coeff_l2(47,84)= -2.6432040334e-01
       coeff_l2(47,85)=  9.3310222030e-02
       coeff_l2(47,86)= -2.0129434764e-01
       coeff_l2(47,87)=  1.8711873889e-01
       coeff_l2(47,88)= -1.9176438451e-01
       coeff_l2(47,89)=  2.4909172952e-01
       coeff_l2(47,90)=  1.2101613730e-01
       coeff_l2(47,91)= -1.7356455326e-02
       coeff_l2(47,92)= -1.6703616083e-01
       coeff_l2(47,93)= -1.9648720045e-03
       coeff_l2(47,94)= -7.6528556645e-02
       coeff_l2(47,95)= -2.4967493117e-01
       coeff_l2(48,1)= -2.2016355395e-01
       coeff_l2(48,2)= -5.7423865795e-01
       coeff_l2(48,3)= -6.8640947342e-02
       coeff_l2(48,4)=  1.9684064388e-01
       coeff_l2(48,5)= -5.5663704872e-01
       coeff_l2(48,6)= -5.7260207832e-03
       coeff_l2(48,7)= -6.0684221983e-01
       coeff_l2(48,8)=  1.8575803936e-01
       coeff_l2(48,9)= -1.3250768185e-01
       coeff_l2(48,10)=  5.3288489580e-01
       coeff_l2(48,11)= -3.3660307527e-01
       coeff_l2(48,12)= -6.6038160585e-03
       coeff_l2(48,13)=  5.3490784019e-02
       coeff_l2(48,14)= -3.1298691034e-01
       coeff_l2(48,15)= -1.9964654744e-01
       coeff_l2(48,16)=  9.7423732281e-02
       coeff_l2(48,17)=  1.3073487580e-01
       coeff_l2(48,18)=  1.4296619594e-01
       coeff_l2(48,19)= -1.3211891055e-01
       coeff_l2(48,20)=  2.1607266366e-01
       coeff_l2(48,21)= -5.1250076294e-01
       coeff_l2(48,22)= -1.5819561481e-01
       coeff_l2(48,23)= -4.1003532708e-02
       coeff_l2(48,24)= -1.2770152092e-01
       coeff_l2(48,25)= -4.8225419596e-03
       coeff_l2(48,26)= -3.8071912527e-01
       coeff_l2(48,27)= -5.7186480612e-02
       coeff_l2(48,28)= -7.2427332401e-02
       coeff_l2(48,29)=  1.1915181577e-01
       coeff_l2(48,30)=  1.5616902709e-01
       coeff_l2(48,31)=  1.3844819367e-01
       coeff_l2(48,32)= -1.2322697788e-01
       coeff_l2(48,33)=  1.0858360678e-01
       coeff_l2(48,34)=  4.5605301857e-03
       coeff_l2(48,35)=  9.0381824970e-01
       coeff_l2(48,36)= -5.1579898596e-01
       coeff_l2(48,37)= -6.8207710981e-01
       coeff_l2(48,38)= -1.0719446838e-01
       coeff_l2(48,39)=  1.5299302526e-02
       coeff_l2(48,40)= -6.9381006062e-02
       coeff_l2(48,41)=  1.0708780587e-01
       coeff_l2(48,42)=  1.0021255910e-01
       coeff_l2(48,43)= -1.8225176632e-01
       coeff_l2(48,44)= -1.0268012993e-02
       coeff_l2(48,45)= -3.5527405143e-01
       coeff_l2(48,46)=  3.5341683030e-01
       coeff_l2(48,47)= -1.4723236859e-01
       coeff_l2(48,48)= -1.4136603475e-01
       coeff_l2(48,49)= -9.2594586313e-02
       coeff_l2(48,50)=  1.1255662441e+00
       coeff_l2(48,51)=  1.8095470965e-02
       coeff_l2(48,52)= -1.6339632869e-01
       coeff_l2(48,53)= -2.3753629625e-01
       coeff_l2(48,54)= -2.3447404802e-01
       coeff_l2(48,55)=  4.7273963690e-02
       coeff_l2(48,56)= -1.0059975833e-01
       coeff_l2(48,57)= -1.1582784355e-01
       coeff_l2(48,58)= -3.7627100945e-01
       coeff_l2(48,59)= -2.3867480457e-01
       coeff_l2(48,60)=  1.4742130041e-01
       coeff_l2(48,61)= -3.6545443535e-01
       coeff_l2(48,62)=  1.8272702396e-01
       coeff_l2(48,63)=  3.2504093647e-01
       coeff_l2(48,64)= -3.2536065578e-01
       coeff_l2(48,65)= -6.4145755768e-01
       coeff_l2(48,66)= -2.6599934697e-01
       coeff_l2(48,67)=  2.7620065212e-01
       coeff_l2(48,68)=  2.4301585555e-01
       coeff_l2(48,69)= -3.8548480719e-02
       coeff_l2(48,70)= -3.7905283272e-02
       coeff_l2(48,71)=  3.5763487220e-01
       coeff_l2(48,72)= -1.4131699502e-01
       coeff_l2(48,73)= -2.6645514369e-01
       coeff_l2(48,74)= -3.1520566344e-01
       coeff_l2(48,75)= -1.5323193371e-01
       coeff_l2(48,76)= -2.4423177540e-01
       coeff_l2(48,77)= -5.2396804094e-02
       coeff_l2(48,78)= -1.0222263634e-01
       coeff_l2(48,79)= -4.4875076413e-01
       coeff_l2(48,80)=  5.6232351810e-02
       coeff_l2(48,81)= -3.0452892184e-01
       coeff_l2(48,82)= -1.3379214704e-01
       coeff_l2(48,83)= -6.0850113630e-02
       coeff_l2(48,84)= -3.8131020963e-02
       coeff_l2(48,85)=  6.3001148403e-02
       coeff_l2(48,86)=  2.3655940592e-01
       coeff_l2(48,87)= -1.9887093455e-02
       coeff_l2(48,88)=  9.1109737754e-02
       coeff_l2(48,89)= -5.8057177067e-01
       coeff_l2(48,90)=  3.1268307567e-01
       coeff_l2(48,91)= -3.0601376295e-01
       coeff_l2(48,92)=  1.0421710461e-01
       coeff_l2(48,93)=  5.9376228601e-02
       coeff_l2(48,94)= -1.5334193408e-01
       coeff_l2(48,95)= -1.8648323417e-01
       coeff_l2(49,1)= -2.9368966818e-01
       coeff_l2(49,2)= -6.6821582615e-02
       coeff_l2(49,3)= -2.8708219528e-01
       coeff_l2(49,4)= -1.1179145426e-01
       coeff_l2(49,5)= -5.9203189611e-01
       coeff_l2(49,6)= -6.5047174692e-02
       coeff_l2(49,7)= -1.0841379166e+00
       coeff_l2(49,8)=  7.4719920754e-02
       coeff_l2(49,9)= -1.2864729762e-01
       coeff_l2(49,10)=  1.0320758075e-01
       coeff_l2(49,11)= -2.1196299791e-01
       coeff_l2(49,12)= -3.0402541161e-01
       coeff_l2(49,13)= -4.2256373912e-02
       coeff_l2(49,14)= -2.6593834162e-01
       coeff_l2(49,15)= -1.3083857298e-01
       coeff_l2(49,16)= -7.9743668437e-02
       coeff_l2(49,17)=  1.1428578198e-01
       coeff_l2(49,18)= -7.8231319785e-02
       coeff_l2(49,19)=  4.5720324852e-03
       coeff_l2(49,20)=  8.0463126302e-02
       coeff_l2(49,21)= -3.6227792501e-01
       coeff_l2(49,22)= -3.2796123624e-01
       coeff_l2(49,23)=  2.4733481929e-02
       coeff_l2(49,24)= -9.5788061619e-02
       coeff_l2(49,25)=  2.4779608846e-01
       coeff_l2(49,26)= -1.1506892741e-01
       coeff_l2(49,27)= -1.4969804883e-01
       coeff_l2(49,28)= -1.0042361915e-01
       coeff_l2(49,29)= -2.1228124201e-01
       coeff_l2(49,30)= -1.7932562530e-01
       coeff_l2(49,31)= -1.4446456730e-01
       coeff_l2(49,32)=  7.8289143741e-02
       coeff_l2(49,33)=  3.2862875611e-02
       coeff_l2(49,34)=  1.1224227399e-01
       coeff_l2(49,35)=  1.1568661779e-01
       coeff_l2(49,36)= -3.1149238348e-01
       coeff_l2(49,37)= -6.4594805241e-01
       coeff_l2(49,38)=  1.3906586170e-01
       coeff_l2(49,39)=  2.4852064252e-01
       coeff_l2(49,40)=  1.4149388671e-01
       coeff_l2(49,41)=  1.1899983883e-01
       coeff_l2(49,42)= -5.7762779295e-02
       coeff_l2(49,43)= -3.6469784379e-01
       coeff_l2(49,44)=  1.9963815808e-02
       coeff_l2(49,45)= -9.0777255595e-02
       coeff_l2(49,46)=  6.8841665983e-02
       coeff_l2(49,47)=  3.6895334721e-01
       coeff_l2(49,48)=  7.6486818492e-02
       coeff_l2(49,49)=  1.2749801576e-01
       coeff_l2(49,50)=  7.1101248264e-01
       coeff_l2(49,51)= -1.8161320686e-01
       coeff_l2(49,52)= -8.4854297340e-02
       coeff_l2(49,53)= -6.3863322139e-02
       coeff_l2(49,54)=  9.9247664213e-02
       coeff_l2(49,55)=  1.6330279410e-01
       coeff_l2(49,56)=  1.4376674592e-01
       coeff_l2(49,57)= -2.0113402605e-01
       coeff_l2(49,58)= -7.2363495827e-01
       coeff_l2(49,59)=  6.4564459026e-02
       coeff_l2(49,60)=  1.0785293579e-01
       coeff_l2(49,61)= -1.6266772524e-02
       coeff_l2(49,62)=  5.5045101792e-02
       coeff_l2(49,63)= -1.5951704979e-01
       coeff_l2(49,64)= -5.7716298103e-01
       coeff_l2(49,65)=  8.6034700274e-02
       coeff_l2(49,66)= -6.2546306849e-01
       coeff_l2(49,67)=  4.5439962298e-02
       coeff_l2(49,68)= -1.4889141917e-01
       coeff_l2(49,69)=  1.9742955267e-01
       coeff_l2(49,70)=  2.0950922370e-01
       coeff_l2(49,71)=  4.2442217469e-01
       coeff_l2(49,72)= -2.2637915611e-01
       coeff_l2(49,73)= -8.7205767632e-02
       coeff_l2(49,74)= -6.3326142728e-02
       coeff_l2(49,75)= -1.5570676327e-01
       coeff_l2(49,76)=  5.8837689459e-02
       coeff_l2(49,77)= -3.5850012302e-01
       coeff_l2(49,78)= -1.1263283342e-01
       coeff_l2(49,79)= -3.8952043653e-01
       coeff_l2(49,80)= -3.2793930173e-01
       coeff_l2(49,81)=  6.2607802451e-02
       coeff_l2(49,82)= -8.9099772274e-02
       coeff_l2(49,83)= -2.5022929907e-01
       coeff_l2(49,84)=  2.4338593706e-02
       coeff_l2(49,85)= -1.5014477074e-01
       coeff_l2(49,86)=  3.8010367751e-01
       coeff_l2(49,87)= -2.2277416289e-01
       coeff_l2(49,88)=  2.8891444206e-02
       coeff_l2(49,89)= -2.6247182488e-01
       coeff_l2(49,90)=  3.3549983054e-02
       coeff_l2(49,91)= -1.2654954195e-01
       coeff_l2(49,92)= -1.8579371274e-01
       coeff_l2(49,93)= -6.7587919533e-02
       coeff_l2(49,94)= -1.6119465232e-01
       coeff_l2(49,95)=  3.5819113255e-02
       coeff_l2(50,1)=  1.7812685668e-01
       coeff_l2(50,2)= -3.9728078246e-01
       coeff_l2(50,3)= -3.0377140641e-01
       coeff_l2(50,4)= -1.5133929253e-01
       coeff_l2(50,5)=  3.7792697549e-01
       coeff_l2(50,6)= -1.1916118860e-01
       coeff_l2(50,7)=  1.6405352950e-01
       coeff_l2(50,8)= -1.2978965044e-01
       coeff_l2(50,9)=  7.0900633931e-02
       coeff_l2(50,10)=  6.4090293646e-01
       coeff_l2(50,11)=  1.7012569308e-01
       coeff_l2(50,12)= -7.4007377028e-02
       coeff_l2(50,13)= -1.4522494376e-01
       coeff_l2(50,14)=  1.2942549586e-01
       coeff_l2(50,15)= -1.6898334026e-01
       coeff_l2(50,16)=  9.4742387533e-02
       coeff_l2(50,17)= -5.4244466126e-02
       coeff_l2(50,18)=  6.5130576491e-02
       coeff_l2(50,19)=  6.0713898391e-02
       coeff_l2(50,20)=  2.8156286478e-01
       coeff_l2(50,21)= -2.6602655649e-02
       coeff_l2(50,22)= -2.5363659859e-01
       coeff_l2(50,23)=  8.3240166306e-02
       coeff_l2(50,24)= -2.8800686821e-02
       coeff_l2(50,25)= -1.2490972877e-02
       coeff_l2(50,26)= -1.4832170308e-01
       coeff_l2(50,27)= -1.8028485775e-01
       coeff_l2(50,28)= -3.6163136363e-01
       coeff_l2(50,29)=  2.5274783839e-03
       coeff_l2(50,30)=  1.6240574419e-02
       coeff_l2(50,31)=  1.2364785373e-01
       coeff_l2(50,32)=  1.8631142378e-01
       coeff_l2(50,33)= -1.7610340146e-03
       coeff_l2(50,34)= -5.8339571953e-01
       coeff_l2(50,35)=  5.6205463409e-01
       coeff_l2(50,36)=  7.0103883743e-01
       coeff_l2(50,37)= -1.0404607281e-02
       coeff_l2(50,38)= -2.5112512708e-01
       coeff_l2(50,39)=  4.1389826685e-02
       coeff_l2(50,40)= -2.1340094507e-01
       coeff_l2(50,41)=  2.3146702349e-01
       coeff_l2(50,42)= -1.6661459208e-01
       coeff_l2(50,43)=  3.7806397676e-01
       coeff_l2(50,44)=  2.4999244511e-01
       coeff_l2(50,45)= -1.3980281353e-01
       coeff_l2(50,46)= -5.7987779379e-02
       coeff_l2(50,47)=  1.2942390144e-01
       coeff_l2(50,48)= -2.7719995379e-01
       coeff_l2(50,49)= -4.7979700565e-01
       coeff_l2(50,50)=  4.5858857036e-01
       coeff_l2(50,51)= -8.0443136394e-02
       coeff_l2(50,52)= -3.2458364964e-01
       coeff_l2(50,53)= -1.1112304777e-01
       coeff_l2(50,54)=  1.8368478119e-01
       coeff_l2(50,55)=  5.3658246994e-02
       coeff_l2(50,56)= -3.9978089929e-01
       coeff_l2(50,57)= -4.2672163248e-01
       coeff_l2(50,58)=  1.3386408798e-02
       coeff_l2(50,59)= -2.4665729702e-01
       coeff_l2(50,60)= -2.2017925978e-01
       coeff_l2(50,61)=  4.5347589999e-02
       coeff_l2(50,62)=  6.0419745743e-02
       coeff_l2(50,63)= -2.6908421516e-01
       coeff_l2(50,64)=  7.9713857174e-01
       coeff_l2(50,65)= -1.0985111445e-01
       coeff_l2(50,66)=  8.2930994034e-01
       coeff_l2(50,67)=  6.9928407669e-01
       coeff_l2(50,68)=  3.1947588921e-01
       coeff_l2(50,69)= -8.6865261197e-02
       coeff_l2(50,70)=  8.6384661496e-02
       coeff_l2(50,71)=  1.1258844286e-01
       coeff_l2(50,72)= -1.8877175450e-01
       coeff_l2(50,73)= -2.6519876719e-01
       coeff_l2(50,74)= -9.7804576159e-02
       coeff_l2(50,75)= -3.3633741736e-01
       coeff_l2(50,76)= -5.0406903028e-02
       coeff_l2(50,77)=  7.1261119843e-01
       coeff_l2(50,78)= -1.1389668286e-01
       coeff_l2(50,79)=  5.5853694677e-02
       coeff_l2(50,80)=  5.6255692244e-01
       coeff_l2(50,81)= -1.0980605334e-01
       coeff_l2(50,82)= -3.0583354831e-01
       coeff_l2(50,83)= -8.3897836506e-02
       coeff_l2(50,84)= -8.4426134825e-02
       coeff_l2(50,85)= -6.7061819136e-02
       coeff_l2(50,86)=  2.9399245977e-01
       coeff_l2(50,87)= -3.3353036642e-01
       coeff_l2(50,88)=  5.2731867880e-02
       coeff_l2(50,89)=  6.9698631763e-02
       coeff_l2(50,90)=  2.8188106418e-01
       coeff_l2(50,91)= -3.2125300169e-01
       coeff_l2(50,92)= -7.9650461674e-02
       coeff_l2(50,93)= -2.5077560544e-01
       coeff_l2(50,94)= -6.8314142525e-02
       coeff_l2(50,95)= -4.0753088892e-02
       coeff_l2(51,1)= -3.3193516731e-01
       coeff_l2(51,2)= -3.2941125333e-02
       coeff_l2(51,3)= -3.5005679727e-01
       coeff_l2(51,4)=  6.5803118050e-02
       coeff_l2(51,5)= -6.6496258974e-01
       coeff_l2(51,6)= -5.5731070042e-01
       coeff_l2(51,7)= -1.8938252330e-01
       coeff_l2(51,8)= -2.1437668800e-01
       coeff_l2(51,9)= -4.5607253909e-02
       coeff_l2(51,10)=  1.0348404646e+00
       coeff_l2(51,11)= -7.5212694705e-02
       coeff_l2(51,12)=  1.3988217711e-01
       coeff_l2(51,13)= -2.6492300630e-01
       coeff_l2(51,14)= -5.6786820292e-02
       coeff_l2(51,15)= -3.8923371583e-02
       coeff_l2(51,16)= -2.9154239222e-02
       coeff_l2(51,17)=  7.7616222203e-02
       coeff_l2(51,18)=  1.0977663100e-01
       coeff_l2(51,19)= -6.0515340418e-02
       coeff_l2(51,20)= -1.1229716241e-01
       coeff_l2(51,21)=  9.7791671753e-02
       coeff_l2(51,22)=  1.2892396189e-02
       coeff_l2(51,23)= -4.1005719453e-02
       coeff_l2(51,24)=  1.0503427684e-01
       coeff_l2(51,25)=  1.8625542521e-01
       coeff_l2(51,26)= -2.1075436473e-01
       coeff_l2(51,27)= -1.4138020575e-01
       coeff_l2(51,28)= -2.2093531489e-01
       coeff_l2(51,29)= -2.0874674618e-01
       coeff_l2(51,30)=  1.9201658666e-01
       coeff_l2(51,31)=  1.4212726057e-01
       coeff_l2(51,32)=  1.6528658569e-01
       coeff_l2(51,33)=  3.5137516260e-01
       coeff_l2(51,34)= -2.6021089405e-02
       coeff_l2(51,35)=  6.7473232746e-01
       coeff_l2(51,36)= -6.2824621797e-02
       coeff_l2(51,37)= -5.8093261719e-01
       coeff_l2(51,38)= -3.6344215274e-02
       coeff_l2(51,39)= -4.8584181815e-02
       coeff_l2(51,40)= -1.1952399462e-01
       coeff_l2(51,41)=  1.3505525887e-01
       coeff_l2(51,42)= -4.6682719141e-02
       coeff_l2(51,43)=  1.1058031023e-01
       coeff_l2(51,44)= -1.9992417097e-01
       coeff_l2(51,45)= -3.0634853244e-01
       coeff_l2(51,46)=  3.9461427927e-01
       coeff_l2(51,47)=  5.4174160957e-01
       coeff_l2(51,48)= -1.8735121191e-01
       coeff_l2(51,49)= -1.3120532036e-01
       coeff_l2(51,50)=  8.6855489016e-01
       coeff_l2(51,51)= -1.7833806574e-01
       coeff_l2(51,52)= -2.4586181343e-01
       coeff_l2(51,53)= -6.5310418606e-02
       coeff_l2(51,54)= -2.7483883500e-01
       coeff_l2(51,55)=  1.0376371443e-01
       coeff_l2(51,56)= -2.4699723721e-01
       coeff_l2(51,57)= -2.4146041274e-01
       coeff_l2(51,58)= -3.6808189750e-01
       coeff_l2(51,59)= -1.8765822053e-01
       coeff_l2(51,60)= -7.4664577842e-02
       coeff_l2(51,61)= -1.3932444155e-01
       coeff_l2(51,62)=  2.3034444451e-01
       coeff_l2(51,63)=  1.5420919657e-01
       coeff_l2(51,64)= -3.4471020103e-01
       coeff_l2(51,65)= -1.9641543925e-01
       coeff_l2(51,66)=  1.6999571025e-01
       coeff_l2(51,67)=  9.0585011244e-01
       coeff_l2(51,68)=  4.0972128510e-01
       coeff_l2(51,69)= -1.6289420426e-02
       coeff_l2(51,70)= -1.1857186258e-01
       coeff_l2(51,71)=  6.1440020800e-01
       coeff_l2(51,72)= -2.3661103845e-01
       coeff_l2(51,73)= -1.4845554531e-01
       coeff_l2(51,74)= -2.7763888240e-02
       coeff_l2(51,75)= -5.4949764162e-02
       coeff_l2(51,76)= -9.9624894559e-02
       coeff_l2(51,77)= -4.9490861595e-02
       coeff_l2(51,78)= -1.2100458890e-01
       coeff_l2(51,79)= -5.5832147598e-01
       coeff_l2(51,80)= -3.2233795524e-01
       coeff_l2(51,81)= -1.0983382165e-01
       coeff_l2(51,82)= -3.8447549939e-01
       coeff_l2(51,83)=  1.3258276880e-01
       coeff_l2(51,84)=  7.1992546320e-02
       coeff_l2(51,85)= -3.1818576157e-02
       coeff_l2(51,86)= -3.9667475969e-02
       coeff_l2(51,87)= -2.1433795989e-01
       coeff_l2(51,88)=  4.5269880444e-02
       coeff_l2(51,89)= -7.6665095985e-02
       coeff_l2(51,90)=  4.9244800210e-01
       coeff_l2(51,91)= -2.7171385288e-01
       coeff_l2(51,92)= -2.9189115390e-02
       coeff_l2(51,93)= -3.2689848542e-01
       coeff_l2(51,94)= -6.9143928587e-02
       coeff_l2(51,95)= -3.2148327678e-02
       coeff_l2(52,1)= -1.7379601002e+00
       coeff_l2(52,2)=  2.0623192191e-01
       coeff_l2(52,3)=  6.4566904306e-01
       coeff_l2(52,4)= -5.7088798285e-01
       coeff_l2(52,5)= -4.9034315348e-01
       coeff_l2(52,6)=  2.5570709258e-02
       coeff_l2(52,7)= -5.2413070202e-01
       coeff_l2(52,8)=  2.5680184364e-02
       coeff_l2(52,9)= -1.1476145685e-01
       coeff_l2(52,10)=  1.0206520557e+00
       coeff_l2(52,11)= -4.6319493651e-01
       coeff_l2(52,12)= -6.7358776927e-02
       coeff_l2(52,13)= -4.6737945080e-01
       coeff_l2(52,14)= -7.4005122297e-03
       coeff_l2(52,15)= -3.9016219974e-01
       coeff_l2(52,16)= -2.7338752151e-01
       coeff_l2(52,17)= -1.9229610264e-01
       coeff_l2(52,18)= -2.0309047401e-01
       coeff_l2(52,19)= -3.5445508361e-01
       coeff_l2(52,20)=  9.2709757388e-02
       coeff_l2(52,21)=  2.7676117420e-01
       coeff_l2(52,22)= -7.4144476093e-04
       coeff_l2(52,23)=  2.3102584481e-01
       coeff_l2(52,24)= -4.4636070728e-02
       coeff_l2(52,25)=  1.0843703151e-01
       coeff_l2(52,26)=  3.8897523284e-01
       coeff_l2(52,27)= -3.4969924018e-03
       coeff_l2(52,28)=  2.3713700473e-01
       coeff_l2(52,29)=  3.6409407854e-02
       coeff_l2(52,30)=  2.1820977330e-02
       coeff_l2(52,31)=  2.0657861605e-02
       coeff_l2(52,32)= -4.4123268127e-01
       coeff_l2(52,33)=  4.1430887580e-01
       coeff_l2(52,34)=  1.2277401984e-01
       coeff_l2(52,35)=  1.5040391684e-01
       coeff_l2(52,36)= -9.4731020927e-01
       coeff_l2(52,37)= -1.8795445561e-01
       coeff_l2(52,38)=  1.6487437487e-01
       coeff_l2(52,39)=  7.0463204384e-01
       coeff_l2(52,40)=  1.0089292526e+00
       coeff_l2(52,41)= -4.0718328953e-01
       coeff_l2(52,42)= -8.9509978890e-02
       coeff_l2(52,43)= -5.8566868305e-01
       coeff_l2(52,44)= -1.5019915998e-01
       coeff_l2(52,45)= -2.1476380527e-01
       coeff_l2(52,46)= -3.1148582697e-02
       coeff_l2(52,47)=  1.1627122015e-01
       coeff_l2(52,48)=  6.7398488522e-02
       coeff_l2(52,49)=  8.6226962507e-02
       coeff_l2(52,50)=  1.0038098097e+00
       coeff_l2(52,51)= -1.8482452631e-01
       coeff_l2(52,52)=  3.3838376403e-01
       coeff_l2(52,53)= -2.1645185351e-01
       coeff_l2(52,54)= -1.4582005143e-01
       coeff_l2(52,55)=  1.0431358814e+00
       coeff_l2(52,56)=  1.5443554521e-01
       coeff_l2(52,57)=  3.2807371020e-01
       coeff_l2(52,58)= -2.7278909087e-01
       coeff_l2(52,59)=  2.6936775446e-01
       coeff_l2(52,60)=  3.3527144790e-01
       coeff_l2(52,61)= -1.2403526157e-01
       coeff_l2(52,62)=  5.0440579653e-01
       coeff_l2(52,63)=  1.6879403591e-01
       coeff_l2(52,64)=  5.3634041548e-01
       coeff_l2(52,65)= -6.0417778790e-02
       coeff_l2(52,66)= -7.7652472258e-01
       coeff_l2(52,67)=  1.3739224523e-02
       coeff_l2(52,68)= -3.5072755814e-01
       coeff_l2(52,69)=  1.9668956101e-01
       coeff_l2(52,70)= -1.1864573509e-01
       coeff_l2(52,71)=  3.1058755517e-01
       coeff_l2(52,72)= -4.8804390430e-01
       coeff_l2(52,73)= -2.7288055420e-01
       coeff_l2(52,74)= -8.9348167181e-02
       coeff_l2(52,75)=  3.0099907517e-01
       coeff_l2(52,76)= -1.2928502262e-01
       coeff_l2(52,77)= -1.1603382975e-01
       coeff_l2(52,78)= -7.6399475336e-02
       coeff_l2(52,79)= -1.0280334949e+00
       coeff_l2(52,80)= -1.4561507702e+00
       coeff_l2(52,81)=  4.2951947451e-01
       coeff_l2(52,82)= -2.2582273185e-01
       coeff_l2(52,83)= -4.0054199100e-01
       coeff_l2(52,84)= -4.1574039496e-03
       coeff_l2(52,85)=  4.3760454282e-03
       coeff_l2(52,86)=  1.8836224079e-01
       coeff_l2(52,87)= -3.4611183405e-01
       coeff_l2(52,88)= -3.7511947751e-01
       coeff_l2(52,89)=  1.2437636405e-01
       coeff_l2(52,90)=  9.1739064455e-01
       coeff_l2(52,91)= -2.0076973736e-01
       coeff_l2(52,92)=  1.1626677960e-01
       coeff_l2(52,93)=  5.0544422865e-01
       coeff_l2(52,94)= -1.2042419985e-02
       coeff_l2(52,95)=  1.7726927996e-01
       coeff_l2(53,1)= -4.0871021152e-01
       coeff_l2(53,2)=  4.8840814829e-01
       coeff_l2(53,3)=  3.3313721418e-01
       coeff_l2(53,4)=  3.9518231153e-01
       coeff_l2(53,5)= -1.4794437587e-01
       coeff_l2(53,6)= -3.7380534410e-01
       coeff_l2(53,7)= -4.2238137126e-01
       coeff_l2(53,8)=  2.1777302027e-01
       coeff_l2(53,9)= -2.7617132664e-01
       coeff_l2(53,10)=  2.1535269916e-02
       coeff_l2(53,11)= -4.4177079201e-01
       coeff_l2(53,12)= -2.4757276475e-01
       coeff_l2(53,13)= -3.5135212541e-01
       coeff_l2(53,14)= -4.9317201972e-01
       coeff_l2(53,15)= -5.4910707474e-01
       coeff_l2(53,16)=  1.4147050679e-01
       coeff_l2(53,17)=  6.3976660371e-02
       coeff_l2(53,18)= -2.4766758084e-02
       coeff_l2(53,19)=  2.4376016110e-02
       coeff_l2(53,20)= -6.9944061339e-02
       coeff_l2(53,21)= -9.7731643915e-01
       coeff_l2(53,22)= -9.1591939330e-02
       coeff_l2(53,23)=  1.1103381962e-01
       coeff_l2(53,24)= -9.8236285150e-02
       coeff_l2(53,25)=  2.9731172323e-01
       coeff_l2(53,26)=  4.7721913457e-01
       coeff_l2(53,27)=  2.3196774721e-01
       coeff_l2(53,28)=  9.1864401475e-03
       coeff_l2(53,29)= -1.3956803083e-01
       coeff_l2(53,30)= -3.9554838091e-02
       coeff_l2(53,31)=  2.8795108199e-01
       coeff_l2(53,32)=  1.6469594836e-01
       coeff_l2(53,33)= -1.1640026234e-02
       coeff_l2(53,34)=  7.7029936947e-03
       coeff_l2(53,35)=  1.7434836924e-01
       coeff_l2(53,36)= -5.2030456066e-01
       coeff_l2(53,37)=  4.2466989160e-01
       coeff_l2(53,38)= -1.1174028367e-01
       coeff_l2(53,39)=  2.5590807199e-01
       coeff_l2(53,40)= -3.0943991616e-02
       coeff_l2(53,41)=  2.3198248446e-01
       coeff_l2(53,42)=  2.9332092404e-01
       coeff_l2(53,43)= -7.7047884464e-01
       coeff_l2(53,44)= -1.4667958952e-02
       coeff_l2(53,45)= -9.9277853966e-02
       coeff_l2(53,46)=  2.1560783684e-01
       coeff_l2(53,47)=  2.9975098372e-01
       coeff_l2(53,48)=  2.2377997637e-01
       coeff_l2(53,49)=  3.1058481336e-01
       coeff_l2(53,50)=  7.1528875828e-01
       coeff_l2(53,51)= -2.2624170408e-02
       coeff_l2(53,52)= -8.6234159768e-02
       coeff_l2(53,53)= -4.9930524081e-02
       coeff_l2(53,54)= -1.4894431829e-01
       coeff_l2(53,55)=  2.9475137591e-01
       coeff_l2(53,56)= -9.8460525274e-02
       coeff_l2(53,57)=  2.7313995361e-01
       coeff_l2(53,58)=  8.9761994779e-02
       coeff_l2(53,59)=  5.9357501566e-02
       coeff_l2(53,60)=  3.6523401737e-02
       coeff_l2(53,61)= -5.3917008638e-01
       coeff_l2(53,62)=  1.9083596766e-01
       coeff_l2(53,63)=  3.2702594995e-01
       coeff_l2(53,64)=  2.2856102884e-01
       coeff_l2(53,65)= -5.9031866491e-02
       coeff_l2(53,66)= -8.6735016108e-01
       coeff_l2(53,67)= -3.0751112103e-01
       coeff_l2(53,68)= -1.0240647942e-01
       coeff_l2(53,69)= -4.5042498969e-03
       coeff_l2(53,70)=  6.4125113189e-02
       coeff_l2(53,71)=  8.2218658924e-01
       coeff_l2(53,72)= -7.5506872963e-04
       coeff_l2(53,73)=  1.3320426457e-02
       coeff_l2(53,74)=  2.5933708996e-02
       coeff_l2(53,75)= -2.4428589642e-01
       coeff_l2(53,76)=  7.2824835777e-02
       coeff_l2(53,77)=  2.8164453804e-02
       coeff_l2(53,78)=  7.1270287037e-02
       coeff_l2(53,79)= -8.2626682520e-01
       coeff_l2(53,80)= -4.9736362696e-01
       coeff_l2(53,81)= -9.2567801476e-02
       coeff_l2(53,82)= -8.0392792821e-02
       coeff_l2(53,83)= -2.1716634929e-01
       coeff_l2(53,84)=  1.4067944139e-02
       coeff_l2(53,85)= -5.4199077189e-02
       coeff_l2(53,86)=  4.2320176959e-01
       coeff_l2(53,87)= -9.3172229826e-02
       coeff_l2(53,88)= -1.4960403740e-01
       coeff_l2(53,89)= -1.9583907723e-01
       coeff_l2(53,90)=  4.5879527926e-01
       coeff_l2(53,91)=  4.0184628963e-01
       coeff_l2(53,92)=  8.0656416714e-02
       coeff_l2(53,93)=  9.0298298746e-03
       coeff_l2(53,94)= -2.0421727002e-01
       coeff_l2(53,95)=  1.1583052576e-01
       coeff_l2(54,1)=  3.8445684314e-01
       coeff_l2(54,2)=  6.4981490374e-02
       coeff_l2(54,3)= -1.6253035069e+00
       coeff_l2(54,4)=  3.9617210627e-01
       coeff_l2(54,5)=  2.5843292475e-01
       coeff_l2(54,6)=  2.9204842448e-01
       coeff_l2(54,7)=  8.4331196547e-01
       coeff_l2(54,8)= -2.6407915354e-01
       coeff_l2(54,9)= -1.5480574965e-01
       coeff_l2(54,10)=  2.8271377087e-01
       coeff_l2(54,11)= -2.0270501077e-01
       coeff_l2(54,12)=  1.4590083063e-01
       coeff_l2(54,13)=  6.5228289366e-01
       coeff_l2(54,14)= -5.5191749334e-01
       coeff_l2(54,15)= -1.7722557485e-01
       coeff_l2(54,16)= -1.0944118723e-02
       coeff_l2(54,17)=  1.3513903320e-01
       coeff_l2(54,18)= -4.1173651814e-02
       coeff_l2(54,19)= -6.9049142301e-02
       coeff_l2(54,20)=  7.0021134615e-01
       coeff_l2(54,21)= -6.8006426096e-02
       coeff_l2(54,22)= -9.2385122553e-03
       coeff_l2(54,23)=  3.5474386811e-01
       coeff_l2(54,24)=  1.5584257245e-01
       coeff_l2(54,25)=  7.3195606470e-02
       coeff_l2(54,26)= -5.3982305527e-01
       coeff_l2(54,27)= -1.8646575511e-01
       coeff_l2(54,28)=  3.1320679188e-01
       coeff_l2(54,29)=  3.8549429178e-01
       coeff_l2(54,30)= -3.2349646091e-01
       coeff_l2(54,31)=  6.0887372494e-01
       coeff_l2(54,32)=  8.3529293537e-01
       coeff_l2(54,33)=  3.5184162855e-01
       coeff_l2(54,34)= -2.7025550604e-01
       coeff_l2(54,35)= -3.7322074175e-01
       coeff_l2(54,36)=  7.1968668699e-01
       coeff_l2(54,37)= -5.1928299665e-01
       coeff_l2(54,38)=  1.0034348816e-01
       coeff_l2(54,39)= -1.5598551929e-01
       coeff_l2(54,40)= -1.6497506201e-01
       coeff_l2(54,41)= -3.6187097430e-02
       coeff_l2(54,42)= -1.1963717639e-01
       coeff_l2(54,43)= -1.4376720414e-02
       coeff_l2(54,44)= -1.6288037598e-01
       coeff_l2(54,45)=  5.1312118769e-01
       coeff_l2(54,46)= -1.3408190012e-01
       coeff_l2(54,47)=  4.9109482765e-01
       coeff_l2(54,48)=  1.3575142622e-01
       coeff_l2(54,49)= -3.4534391761e-01
       coeff_l2(54,50)= -1.3568241596e+00
       coeff_l2(54,51)= -3.4051388502e-02
       coeff_l2(54,52)= -1.0365369171e-01
       coeff_l2(54,53)=  2.6428619027e-01
       coeff_l2(54,54)= -5.1436662674e-01
       coeff_l2(54,55)= -1.2062708139e+00
       coeff_l2(54,56)= -6.5393382311e-01
       coeff_l2(54,57)= -5.2908682823e-01
       coeff_l2(54,58)=  1.0129165649e+00
       coeff_l2(54,59)=  5.5425703526e-01
       coeff_l2(54,60)= -3.2515916228e-01
       coeff_l2(54,61)= -1.2781681120e-01
       coeff_l2(54,62)= -1.4630998373e+00
       coeff_l2(54,63)=  2.3885008693e-01
       coeff_l2(54,64)= -4.6116834879e-01
       coeff_l2(54,65)= -3.4898191690e-01
       coeff_l2(54,66)=  1.6110913754e+00
       coeff_l2(54,67)=  2.9521176219e-01
       coeff_l2(54,68)=  3.1855228543e-01
       coeff_l2(54,69)= -2.1208282560e-03
       coeff_l2(54,70)=  3.6554168910e-03
       coeff_l2(54,71)=  7.0864933729e-01
       coeff_l2(54,72)= -4.0708091110e-03
       coeff_l2(54,73)= -1.6799651086e-01
       coeff_l2(54,74)=  1.8931420147e-01
       coeff_l2(54,75)=  5.6147402525e-01
       coeff_l2(54,76)= -1.3300292194e-01
       coeff_l2(54,77)=  9.5710271597e-01
       coeff_l2(54,78)= -3.2125669718e-01
       coeff_l2(54,79)= -9.0600706637e-02
       coeff_l2(54,80)= -6.4967310429e-01
       coeff_l2(54,81)= -2.1651215851e-01
       coeff_l2(54,82)=  1.2385430187e-01
       coeff_l2(54,83)= -1.9222229719e-01
       coeff_l2(54,84)= -6.3889399171e-03
       coeff_l2(54,85)= -6.8629336357e-01
       coeff_l2(54,86)= -5.1340150833e-01
       coeff_l2(54,87)=  5.5517274141e-01
       coeff_l2(54,88)=  3.8979759812e-01
       coeff_l2(54,89)= -3.4936708212e-01
       coeff_l2(54,90)=  8.4129178524e-01
       coeff_l2(54,91)=  2.3074617982e-01
       coeff_l2(54,92)= -5.5720090866e-02
       coeff_l2(54,93)=  1.0251179934e+00
       coeff_l2(54,94)=  2.9107356071e-01
       coeff_l2(54,95)= -4.9986889958e-01
       coeff_l2(55,1)=  1.3893304765e-01
       coeff_l2(55,2)=  2.3549418151e-01
       coeff_l2(55,3)=  3.7221089005e-03
       coeff_l2(55,4)=  2.1739752293e+00
       coeff_l2(55,5)=  6.5708261728e-01
       coeff_l2(55,6)=  4.8311835527e-01
       coeff_l2(55,7)=  8.7746930122e-01
       coeff_l2(55,8)= -1.1757397652e-01
       coeff_l2(55,9)= -9.2980593443e-02
       coeff_l2(55,10)= -3.8431153297e+00
       coeff_l2(55,11)= -5.9081427753e-02
       coeff_l2(55,12)=  4.0312297642e-02
       coeff_l2(55,13)=  2.3586142808e-02
       coeff_l2(55,14)= -3.1765028834e-01
       coeff_l2(55,15)=  5.4241758585e-01
       coeff_l2(55,16)= -8.7616316974e-02
       coeff_l2(55,17)=  3.3323162794e-01
       coeff_l2(55,18)= -1.0147570074e-01
       coeff_l2(55,19)=  1.4756445587e-01
       coeff_l2(55,20)= -6.6046243906e-01
       coeff_l2(55,21)= -3.1464975327e-02
       coeff_l2(55,22)=  1.2885206938e+00
       coeff_l2(55,23)=  2.1197763085e-01
       coeff_l2(55,24)=  2.2706435621e-01
       coeff_l2(55,25)=  9.4254752621e-03
       coeff_l2(55,26)=  1.0535552502e+00
       coeff_l2(55,27)= -1.7083351314e-01
       coeff_l2(55,28)=  3.0199089646e-01
       coeff_l2(55,29)=  2.7538222075e-01
       coeff_l2(55,30)= -2.3993802071e-01
       coeff_l2(55,31)=  1.4143961668e-01
       coeff_l2(55,32)= -4.7786170244e-01
       coeff_l2(55,33)=  1.3436211646e-01
       coeff_l2(55,34)= -9.0791356564e-01
       coeff_l2(55,35)=  9.1681289673e-01
       coeff_l2(55,36)= -4.0974521637e-01
       coeff_l2(55,37)=  1.8530322313e+00
       coeff_l2(55,38)= -1.1705600657e-02
       coeff_l2(55,39)=  1.8474067748e-01
       coeff_l2(55,40)= -2.6265340857e-03
       coeff_l2(55,41)= -9.8464226723e-01
       coeff_l2(55,42)=  5.1542174071e-02
       coeff_l2(55,43)= -4.1823732853e-01
       coeff_l2(55,44)= -1.5168549120e-01
       coeff_l2(55,45)=  1.2672133744e-01
       coeff_l2(55,46)= -8.3463257551e-01
       coeff_l2(55,47)= -1.5707854033e+00
       coeff_l2(55,48)= -1.3959097862e-01
       coeff_l2(55,49)=  7.3768883944e-02
       coeff_l2(55,50)=  9.5547431707e-01
       coeff_l2(55,51)=  1.7220339179e-01
       coeff_l2(55,52)= -2.0733034611e-01
       coeff_l2(55,53)=  3.0236592889e-01
       coeff_l2(55,54)= -2.0483602583e-01
       coeff_l2(55,55)=  9.5016688108e-01
       coeff_l2(55,56)= -1.3737776279e+00
       coeff_l2(55,57)=  6.3557898998e-01
       coeff_l2(55,58)=  3.6256688833e-01
       coeff_l2(55,59)= -6.5768229961e-01
       coeff_l2(55,60)=  5.2343640476e-02
       coeff_l2(55,61)=  1.9270475209e-01
       coeff_l2(55,62)=  4.9903187156e-01
       coeff_l2(55,63)=  1.1127810478e+00
       coeff_l2(55,64)=  8.6083352566e-01
       coeff_l2(55,65)=  1.2759580612e+00
       coeff_l2(55,66)= -1.3994529247e+00
       coeff_l2(55,67)= -1.5991728306e+00
       coeff_l2(55,68)= -3.3265629411e-01
       coeff_l2(55,69)=  5.5387318134e-02
       coeff_l2(55,70)= -3.6941328645e-01
       coeff_l2(55,71)= -1.8309372663e+00
       coeff_l2(55,72)= -4.3088892102e-01
       coeff_l2(55,73)= -2.1445338428e-01
       coeff_l2(55,74)= -2.7977326512e-01
       coeff_l2(55,75)= -4.7179067135e-01
       coeff_l2(55,76)= -4.2079919577e-01
       coeff_l2(55,77)= -7.7147215605e-01
       coeff_l2(55,78)= -2.9555898905e-01
       coeff_l2(55,79)=  1.0896002054e+00
       coeff_l2(55,80)=  1.3390480280e+00
       coeff_l2(55,81)= -1.0879292488e+00
       coeff_l2(55,82)=  1.0242167860e-01
       coeff_l2(55,83)=  1.5818792582e-01
       coeff_l2(55,84)= -1.4133495092e-01
       coeff_l2(55,85)=  9.8601870239e-02
       coeff_l2(55,86)=  2.7493870258e-01
       coeff_l2(55,87)= -2.2787816823e-01
       coeff_l2(55,88)= -2.1775667369e-01
       coeff_l2(55,89)= -3.3833509684e-01
       coeff_l2(55,90)= -5.9688377380e-01
       coeff_l2(55,91)=  3.5385027528e-01
       coeff_l2(55,92)=  5.4534548521e-01
       coeff_l2(55,93)= -2.5934243202e-01
       coeff_l2(55,94)= -4.2971797287e-02
       coeff_l2(55,95)= -1.2107481956e+00
       coeff_l2(56,1)= -1.4212535322e-01
       coeff_l2(56,2)= -6.1853446066e-02
       coeff_l2(56,3)= -9.7064115107e-02
       coeff_l2(56,4)=  4.8633884639e-02
       coeff_l2(56,5)= -2.7895319462e-01
       coeff_l2(56,6)= -5.9090540744e-03
       coeff_l2(56,7)=  1.1657997966e-01
       coeff_l2(56,8)= -4.7278833389e-01
       coeff_l2(56,9)=  3.2222756743e-01
       coeff_l2(56,10)=  5.4629147053e-01
       coeff_l2(56,11)= -6.8701580167e-02
       coeff_l2(56,12)=  6.5436936915e-02
       coeff_l2(56,13)= -5.1141798496e-01
       coeff_l2(56,14)=  4.7610457987e-03
       coeff_l2(56,15)= -1.6256628931e-01
       coeff_l2(56,16)=  2.5925058126e-01
       coeff_l2(56,17)=  3.1587090343e-02
       coeff_l2(56,18)=  1.4657823741e-01
       coeff_l2(56,19)=  7.2095200419e-02
       coeff_l2(56,20)=  1.4821486175e-01
       coeff_l2(56,21)=  4.5903676748e-01
       coeff_l2(56,22)= -8.3959960938e-01
       coeff_l2(56,23)= -1.8018491566e-01
       coeff_l2(56,24)= -1.2688590586e-01
       coeff_l2(56,25)=  3.1812673807e-01
       coeff_l2(56,26)=  1.2313532829e-01
       coeff_l2(56,27)=  7.1699116379e-03
       coeff_l2(56,28)=  3.9713181555e-02
       coeff_l2(56,29)= -1.2706166506e-01
       coeff_l2(56,30)= -1.0079798102e-01
       coeff_l2(56,31)=  2.5381246209e-01
       coeff_l2(56,32)= -2.6196393371e-01
       coeff_l2(56,33)=  3.6063048244e-01
       coeff_l2(56,34)= -6.9049239159e-01
       coeff_l2(56,35)=  3.1648740172e-01
       coeff_l2(56,36)=  1.8370459974e-01
       coeff_l2(56,37)= -4.7481566668e-01
       coeff_l2(56,38)= -1.1030728370e-01
       coeff_l2(56,39)=  1.7429624498e-01
       coeff_l2(56,40)= -3.0511470512e-02
       coeff_l2(56,41)= -3.0059355497e-01
       coeff_l2(56,42)= -1.1162315309e-01
       coeff_l2(56,43)=  4.9604392052e-01
       coeff_l2(56,44)= -4.3269714713e-01
       coeff_l2(56,45)= -3.4570068121e-01
       coeff_l2(56,46)=  2.0767042041e-01
       coeff_l2(56,47)=  6.8504446745e-01
       coeff_l2(56,48)= -5.7962462306e-02
       coeff_l2(56,49)=  2.3249359801e-02
       coeff_l2(56,50)=  9.7738260031e-01
       coeff_l2(56,51)=  7.9192243516e-02
       coeff_l2(56,52)= -2.5518587232e-01
       coeff_l2(56,53)=  3.0888302717e-03
       coeff_l2(56,54)= -1.2384045869e-01
       coeff_l2(56,55)=  1.6126063466e-01
       coeff_l2(56,56)= -4.9197444320e-01
       coeff_l2(56,57)= -1.2129598856e-01
       coeff_l2(56,58)= -4.0352335572e-01
       coeff_l2(56,59)= -2.5901898742e-01
       coeff_l2(56,60)= -5.7642187923e-02
       coeff_l2(56,61)=  4.3025825173e-02
       coeff_l2(56,62)=  2.6487040520e-01
       coeff_l2(56,63)= -2.3883514106e-01
       coeff_l2(56,64)=  7.5019873679e-02
       coeff_l2(56,65)= -2.3387466371e-01
       coeff_l2(56,66)=  1.9719268382e-01
       coeff_l2(56,67)=  1.1913492680e+00
       coeff_l2(56,68)=  5.0333225727e-01
       coeff_l2(56,69)= -1.3734194636e-01
       coeff_l2(56,70)= -1.0101505369e-01
       coeff_l2(56,71)=  3.7235748768e-01
       coeff_l2(56,72)=  4.2421287298e-01
       coeff_l2(56,73)= -6.8418580294e-01
       coeff_l2(56,74)= -2.7499574423e-01
       coeff_l2(56,75)= -4.5789501071e-01
       coeff_l2(56,76)= -1.9055737555e-01
       coeff_l2(56,77)=  4.8100164533e-01
       coeff_l2(56,78)= -4.3523013592e-01
       coeff_l2(56,79)= -2.7332335711e-01
       coeff_l2(56,80)= -2.1787668765e-01
       coeff_l2(56,81)=  1.9745542109e-01
       coeff_l2(56,82)= -7.3131114244e-01
       coeff_l2(56,83)=  1.4204566181e-01
       coeff_l2(56,84)= -2.3219262064e-01
       coeff_l2(56,85)= -4.4068264961e-01
       coeff_l2(56,86)= -2.4201387167e-01
       coeff_l2(56,87)=  5.9668082744e-02
       coeff_l2(56,88)= -5.0668805838e-02
       coeff_l2(56,89)=  7.6734706759e-02
       coeff_l2(56,90)=  3.1591147184e-01
       coeff_l2(56,91)= -1.6098377109e-01
       coeff_l2(56,92)= -2.0604318380e-01
       coeff_l2(56,93)= -3.0204424262e-01
       coeff_l2(56,94)= -8.8806319982e-03
       coeff_l2(56,95)= -3.2272547483e-01
       coeff_l2(57,1)=  1.6847331822e-01
       coeff_l2(57,2)=  2.3585024476e-01
       coeff_l2(57,3)=  7.2114951909e-02
       coeff_l2(57,4)=  3.8368138671e-01
       coeff_l2(57,5)=  8.1317102909e-01
       coeff_l2(57,6)=  3.3212757111e-01
       coeff_l2(57,7)= -3.3623073250e-02
       coeff_l2(57,8)= -2.4961975217e-01
       coeff_l2(57,9)=  6.0538437217e-02
       coeff_l2(57,10)= -2.8225719929e+00
       coeff_l2(57,11)=  6.0055800714e-03
       coeff_l2(57,12)=  4.0836490691e-02
       coeff_l2(57,13)= -3.6683335900e-01
       coeff_l2(57,14)=  1.4005543292e-01
       coeff_l2(57,15)=  8.2832105458e-02
       coeff_l2(57,16)= -9.4407849014e-02
       coeff_l2(57,17)=  1.6374735534e-01
       coeff_l2(57,18)=  6.8521149457e-02
       coeff_l2(57,19)=  1.9037880003e-01
       coeff_l2(57,20)= -7.6105546951e-01
       coeff_l2(57,21)=  6.5328992903e-02
       coeff_l2(57,22)=  6.8241721392e-01
       coeff_l2(57,23)= -2.0427282155e-01
       coeff_l2(57,24)=  1.0575378686e-01
       coeff_l2(57,25)= -1.4848564565e-01
       coeff_l2(57,26)=  9.5242476463e-01
       coeff_l2(57,27)= -1.0676626861e-01
       coeff_l2(57,28)=  1.0045315325e-01
       coeff_l2(57,29)=  2.2282314301e-01
       coeff_l2(57,30)= -5.0994958729e-02
       coeff_l2(57,31)=  3.9436677098e-01
       coeff_l2(57,32)= -3.8225767016e-01
       coeff_l2(57,33)= -1.6807129979e-01
       coeff_l2(57,34)=  2.1601605415e-01
       coeff_l2(57,35)=  7.6545923948e-01
       coeff_l2(57,36)= -6.1589974165e-01
       coeff_l2(57,37)=  1.3974198103e+00
       coeff_l2(57,38)= -8.8348734379e-01
       coeff_l2(57,39)=  1.7726401985e-01
       coeff_l2(57,40)=  7.1483463049e-02
       coeff_l2(57,41)= -1.0962325335e+00
       coeff_l2(57,42)= -3.1428119540e-01
       coeff_l2(57,43)= -4.4406861067e-01
       coeff_l2(57,44)= -1.7515975237e-01
       coeff_l2(57,45)= -1.3751581311e-02
       coeff_l2(57,46)= -5.8319890499e-01
       coeff_l2(57,47)= -1.0399882793e+00
       coeff_l2(57,48)= -7.3810115457e-02
       coeff_l2(57,49)=  6.9605506957e-02
       coeff_l2(57,50)=  5.0125253201e-01
       coeff_l2(57,51)=  2.5010609627e-01
       coeff_l2(57,52)= -8.5695065558e-02
       coeff_l2(57,53)= -8.2274526358e-02
       coeff_l2(57,54)=  1.2474560738e-01
       coeff_l2(57,55)=  3.3565366268e-01
       coeff_l2(57,56)= -1.0868963003e+00
       coeff_l2(57,57)=  1.0335191488e+00
       coeff_l2(57,58)= -1.2150809914e-01
       coeff_l2(57,59)= -7.8805851936e-01
       coeff_l2(57,60)= -7.4252359569e-02
       coeff_l2(57,61)=  2.0013745874e-02
       coeff_l2(57,62)=  9.7986334562e-01
       coeff_l2(57,63)=  1.1075177193e+00
       coeff_l2(57,64)=  7.8679937124e-01
       coeff_l2(57,65)=  1.3017437458e+00
       coeff_l2(57,66)= -1.5395371914e+00
       coeff_l2(57,67)= -1.3352528811e+00
       coeff_l2(57,68)= -5.9427154064e-01
       coeff_l2(57,69)= -2.3194946349e-01
       coeff_l2(57,70)= -1.8927957863e-02
       coeff_l2(57,71)= -1.6592042446e+00
       coeff_l2(57,72)= -2.2038827837e-01
       coeff_l2(57,73)= -7.9027301073e-01
       coeff_l2(57,74)=  4.9295160174e-01
       coeff_l2(57,75)= -5.2716714144e-01
       coeff_l2(57,76)= -2.0671740174e-01
       coeff_l2(57,77)= -7.1458148956e-01
       coeff_l2(57,78)= -1.9742162526e-01
       coeff_l2(57,79)=  8.9179420471e-01
       coeff_l2(57,80)=  9.4281530380e-01
       coeff_l2(57,81)= -7.9492121935e-01
       coeff_l2(57,82)=  8.6859673262e-01
       coeff_l2(57,83)=  1.1553441733e-01
       coeff_l2(57,84)= -2.7871695161e-01
       coeff_l2(57,85)= -9.4612252712e-01
       coeff_l2(57,86)= -2.5869277120e-01
       coeff_l2(57,87)= -3.9647954702e-01
       coeff_l2(57,88)= -3.0971169472e-01
       coeff_l2(57,89)=  1.0473933071e-01
       coeff_l2(57,90)= -1.1499238014e+00
       coeff_l2(57,91)= -4.5550853014e-02
       coeff_l2(57,92)=  2.7123600245e-01
       coeff_l2(57,93)= -5.4543155432e-01
       coeff_l2(57,94)=  1.5326200426e-01
       coeff_l2(57,95)=  1.2168727815e-01
       coeff_l2(58,1)= -5.1293563843e-01
       coeff_l2(58,2)=  1.5247650445e-01
       coeff_l2(58,3)= -3.7238195539e-01
       coeff_l2(58,4)= -2.8532555699e-01
       coeff_l2(58,5)= -5.8484721184e-01
       coeff_l2(58,6)= -4.1685923934e-01
       coeff_l2(58,7)= -7.9925400019e-01
       coeff_l2(58,8)=  1.2155289203e-01
       coeff_l2(58,9)= -3.1598142232e-04
       coeff_l2(58,10)= -8.4873639047e-02
       coeff_l2(58,11)= -7.6653376222e-02
       coeff_l2(58,12)= -2.7116259933e-01
       coeff_l2(58,13)=  1.1844869703e-02
       coeff_l2(58,14)= -5.3842312098e-01
       coeff_l2(58,15)= -2.0430025458e-01
       coeff_l2(58,16)= -2.3379459977e-01
       coeff_l2(58,17)=  1.2040576339e-01
       coeff_l2(58,18)=  1.4729514718e-01
       coeff_l2(58,19)= -8.4077700973e-02
       coeff_l2(58,20)= -2.0340587944e-02
       coeff_l2(58,21)=  3.4766677022e-01
       coeff_l2(58,22)= -2.2811116278e-01
       coeff_l2(58,23)= -1.6184951365e-01
       coeff_l2(58,24)= -7.2816848755e-02
       coeff_l2(58,25)=  1.9193606451e-02
       coeff_l2(58,26)= -8.7600052357e-02
       coeff_l2(58,27)=  2.3815798759e-01
       coeff_l2(58,28)=  1.1040301621e-01
       coeff_l2(58,29)= -1.8036197871e-02
       coeff_l2(58,30)= -3.8148850203e-02
       coeff_l2(58,31)= -1.6773770750e-01
       coeff_l2(58,32)= -9.6835382283e-02
       coeff_l2(58,33)=  4.8134677112e-02
       coeff_l2(58,34)=  2.6454335451e-01
       coeff_l2(58,35)=  8.6662828922e-01
       coeff_l2(58,36)= -9.8325467110e-01
       coeff_l2(58,37)= -3.1091004610e-01
       coeff_l2(58,38)= -2.1511031687e-01
       coeff_l2(58,39)=  9.0323857963e-02
       coeff_l2(58,40)= -9.8621346056e-02
       coeff_l2(58,41)= -3.0050331727e-02
       coeff_l2(58,42)=  1.1260224134e-01
       coeff_l2(58,43)= -8.3796834946e-01
       coeff_l2(58,44)=  6.9533325732e-02
       coeff_l2(58,45)= -1.9852404296e-01
       coeff_l2(58,46)=  4.2346313596e-02
       coeff_l2(58,47)= -3.1130704284e-01
       coeff_l2(58,48)=  2.3411354423e-01
       coeff_l2(58,49)=  2.1347396076e-01
       coeff_l2(58,50)=  1.0933744907e+00
       coeff_l2(58,51)= -1.3793741167e-01
       coeff_l2(58,52)=  3.0920198560e-01
       coeff_l2(58,53)= -2.1352767944e-02
       coeff_l2(58,54)=  1.7228129506e-01
       coeff_l2(58,55)=  1.3869260252e-01
       coeff_l2(58,56)=  1.2816222012e-01
       coeff_l2(58,57)= -1.9779053330e-01
       coeff_l2(58,58)= -5.8644741774e-01
       coeff_l2(58,59)= -4.0522629023e-01
       coeff_l2(58,60)=  1.3794953004e-02
       coeff_l2(58,61)= -1.6528038681e-01
       coeff_l2(58,62)=  7.5649446249e-01
       coeff_l2(58,63)= -1.9809032977e-01
       coeff_l2(58,64)= -2.3189523816e-01
       coeff_l2(58,65)=  3.3514398336e-01
       coeff_l2(58,66)= -1.4405122995e+00
       coeff_l2(58,67)=  2.6276642084e-01
       coeff_l2(58,68)=  2.1174006164e-01
       coeff_l2(58,69)=  8.9273102582e-02
       coeff_l2(58,70)=  9.7859039903e-02
       coeff_l2(58,71)=  3.0747303739e-02
       coeff_l2(58,72)=  1.6138836741e-02
       coeff_l2(58,73)=  2.2154059261e-02
       coeff_l2(58,74)= -3.1730439514e-02
       coeff_l2(58,75)= -2.3559588194e-01
       coeff_l2(58,76)= -1.9718715549e-01
       coeff_l2(58,77)= -9.0200853348e-01
       coeff_l2(58,78)=  7.3246143758e-02
       coeff_l2(58,79)= -6.9106888771e-01
       coeff_l2(58,80)= -3.6980591714e-02
       coeff_l2(58,81)= -2.0550429821e-01
       coeff_l2(58,82)= -4.6532773972e-01
       coeff_l2(58,83)= -1.4080218971e-01
       coeff_l2(58,84)= -7.6014980674e-02
       coeff_l2(58,85)=  6.7726814747e-01
       coeff_l2(58,86)=  5.2646356821e-01
       coeff_l2(58,87)=  1.0931842029e-01
       coeff_l2(58,88)= -2.6046267152e-01
       coeff_l2(58,89)=  6.1562702060e-02
       coeff_l2(58,90)=  1.5228472650e-01
       coeff_l2(58,91)= -3.0698505044e-01
       coeff_l2(58,92)= -1.9852802157e-02
       coeff_l2(58,93)= -3.4072372317e-01
       coeff_l2(58,94)= -8.7272390723e-02
       coeff_l2(58,95)= -2.0637325943e-01
       coeff_l2(59,1)= -1.0615711212e+00
       coeff_l2(59,2)= -5.9273952246e-01
       coeff_l2(59,3)= -1.5810236335e-02
       coeff_l2(59,4)=  6.4881116152e-01
       coeff_l2(59,5)= -4.7932818532e-01
       coeff_l2(59,6)=  4.2093411088e-01
       coeff_l2(59,7)= -2.9400816560e-01
       coeff_l2(59,8)= -1.4340218902e-01
       coeff_l2(59,9)=  2.2382147610e-01
       coeff_l2(59,10)= -2.0115746558e-01
       coeff_l2(59,11)= -1.2966234982e-01
       coeff_l2(59,12)= -2.6671543717e-01
       coeff_l2(59,13)= -1.0206078291e+00
       coeff_l2(59,14)=  3.6933553219e-01
       coeff_l2(59,15)= -2.5293883681e-01
       coeff_l2(59,16)= -4.5313733816e-01
       coeff_l2(59,17)= -2.7429574728e-01
       coeff_l2(59,18)= -9.6435910091e-03
       coeff_l2(59,19)= -1.1162751168e-01
       coeff_l2(59,20)= -2.1643981338e-01
       coeff_l2(59,21)= -2.4246408045e-01
       coeff_l2(59,22)= -1.6390042305e+00
       coeff_l2(59,23)= -4.2617571354e-01
       coeff_l2(59,24)= -5.0427515060e-02
       coeff_l2(59,25)= -8.0899578333e-01
       coeff_l2(59,26)= -3.3961942792e-01
       coeff_l2(59,27)=  1.2463388592e-01
       coeff_l2(59,28)=  3.5442495346e-01
       coeff_l2(59,29)= -1.5240180492e-01
       coeff_l2(59,30)=  2.7117240429e-01
       coeff_l2(59,31)=  6.6236454248e-01
       coeff_l2(59,32)=  2.3074990511e-01
       coeff_l2(59,33)= -8.4223754704e-02
       coeff_l2(59,34)= -9.5689490438e-02
       coeff_l2(59,35)=  1.0867912769e+00
       coeff_l2(59,36)=  4.2656370997e-01
       coeff_l2(59,37)= -5.5089521408e-01
       coeff_l2(59,38)=  7.6762527227e-02
       coeff_l2(59,39)=  5.3640985489e-01
       coeff_l2(59,40)= -2.0066343248e-01
       coeff_l2(59,41)=  6.8158000708e-01
       coeff_l2(59,42)= -1.4974349737e-01
       coeff_l2(59,43)= -8.2995787263e-02
       coeff_l2(59,44)= -6.0698539019e-01
       coeff_l2(59,45)= -3.1906810403e-01
       coeff_l2(59,46)=  4.0292349458e-01
       coeff_l2(59,47)= -1.0925382376e+00
       coeff_l2(59,48)=  1.0576233268e-01
       coeff_l2(59,49)= -1.5154175460e-01
       coeff_l2(59,50)=  2.2113294601e+00
       coeff_l2(59,51)= -3.4921967983e-01
       coeff_l2(59,52)=  2.8770962730e-02
       coeff_l2(59,53)= -3.9294976741e-02
       coeff_l2(59,54)=  6.2684789300e-02
       coeff_l2(59,55)=  2.6123759151e-01
       coeff_l2(59,56)=  1.6332331300e-01
       coeff_l2(59,57)=  5.8076575398e-02
       coeff_l2(59,58)=  3.3733379096e-02
       coeff_l2(59,59)=  3.8634291291e-01
       coeff_l2(59,60)= -2.9325783253e-01
       coeff_l2(59,61)=  1.0025459528e-01
       coeff_l2(59,62)=  2.1042618155e-01
       coeff_l2(59,63)=  1.2327712774e+00
       coeff_l2(59,64)=  6.2161380053e-01
       coeff_l2(59,65)= -6.4896327257e-01
       coeff_l2(59,66)= -1.1713166237e+00
       coeff_l2(59,67)=  8.6688625813e-01
       coeff_l2(59,68)= -4.2415687442e-01
       coeff_l2(59,69)= -2.4294836819e-01
       coeff_l2(59,70)=  1.2395832688e-01
       coeff_l2(59,71)=  3.0159488320e-01
       coeff_l2(59,72)=  5.5754804611e-01
       coeff_l2(59,73)= -2.6629635692e-01
       coeff_l2(59,74)= -8.8566696644e-01
       coeff_l2(59,75)= -1.8116733432e-01
       coeff_l2(59,76)=  1.9902585074e-02
       coeff_l2(59,77)= -1.0559638739e+00
       coeff_l2(59,78)= -6.1553526670e-02
       coeff_l2(59,79)= -2.1598255634e+00
       coeff_l2(59,80)= -7.7670246363e-01
       coeff_l2(59,81)= -3.0881049111e-02
       coeff_l2(59,82)= -3.6688095331e-01
       coeff_l2(59,83)=  7.5682648458e-03
       coeff_l2(59,84)= -1.6286435723e-01
       coeff_l2(59,85)= -1.1341142654e-01
       coeff_l2(59,86)=  8.9006461203e-02
       coeff_l2(59,87)= -3.8746723533e-01
       coeff_l2(59,88)=  3.0518487096e-02
       coeff_l2(59,89)= -3.9164885879e-01
       coeff_l2(59,90)=  7.8121364117e-01
       coeff_l2(59,91)= -7.2417551279e-01
       coeff_l2(59,92)=  5.2265518904e-01
       coeff_l2(59,93)= -7.6002717018e-02
       coeff_l2(59,94)= -2.2185687721e-01
       coeff_l2(59,95)= -5.6728469208e-03
       coeff_l2(60,1)=  1.3508914709e+00
       coeff_l2(60,2)=  1.6635990143e-01
       coeff_l2(60,3)=  3.7149313092e-01
       coeff_l2(60,4)= -5.7735216618e-01
       coeff_l2(60,5)=  8.3590042591e-01
       coeff_l2(60,6)=  1.3781127930e+00
       coeff_l2(60,7)=  5.7178145647e-01
       coeff_l2(60,8)= -3.4021613002e-01
       coeff_l2(60,9)=  1.2065266818e-01
       coeff_l2(60,10)= -4.0624466538e-01
       coeff_l2(60,11)=  2.5358271599e-01
       coeff_l2(60,12)=  7.0223614573e-02
       coeff_l2(60,13)=  6.7950880527e-01
       coeff_l2(60,14)=  3.5653132200e-01
       coeff_l2(60,15)= -3.9357873797e-01
       coeff_l2(60,16)=  2.9298162460e-01
       coeff_l2(60,17)= -1.9645503163e-01
       coeff_l2(60,18)= -5.2947300673e-01
       coeff_l2(60,19)= -3.0258074403e-01
       coeff_l2(60,20)=  4.1992431879e-01
       coeff_l2(60,21)= -1.7526534200e-01
       coeff_l2(60,22)=  6.3617151976e-01
       coeff_l2(60,23)= -1.5103243291e-01
       coeff_l2(60,24)= -1.2515054643e-01
       coeff_l2(60,25)= -7.0326209068e-02
       coeff_l2(60,26)=  2.7885955572e-01
       coeff_l2(60,27)= -1.3740262389e-01
       coeff_l2(60,28)=  3.4215453267e-01
       coeff_l2(60,29)=  2.5133442134e-02
       coeff_l2(60,30)= -2.1558789536e-02
       coeff_l2(60,31)= -4.1370704770e-01
       coeff_l2(60,32)= -7.6690897346e-02
       coeff_l2(60,33)=  1.0889321566e-02
       coeff_l2(60,34)= -2.7499273419e-01
       coeff_l2(60,35)= -1.2703996897e+00
       coeff_l2(60,36)=  3.9863401651e-01
       coeff_l2(60,37)=  7.3948061466e-01
       coeff_l2(60,38)=  1.5485979617e-01
       coeff_l2(60,39)= -1.3453352451e-01
       coeff_l2(60,40)= -7.1946561337e-01
       coeff_l2(60,41)= -4.4841411710e-01
       coeff_l2(60,42)= -5.5841326714e-01
       coeff_l2(60,43)= -3.8599956036e-01
       coeff_l2(60,44)=  2.6086437702e-01
       coeff_l2(60,45)=  8.0273813009e-01
       coeff_l2(60,46)= -3.0419564247e-01
       coeff_l2(60,47)= -1.8806037307e-01
       coeff_l2(60,48)= -3.7432533503e-01
       coeff_l2(60,49)=  4.0713497996e-01
       coeff_l2(60,50)= -8.7958365679e-01
       coeff_l2(60,51)=  2.9495239258e-01
       coeff_l2(60,52)= -3.0507147312e-01
       coeff_l2(60,53)=  5.3405225277e-01
       coeff_l2(60,54)= -4.8509338498e-01
       coeff_l2(60,55)=  1.0832883418e-01
       coeff_l2(60,56)=  9.8891399801e-02
       coeff_l2(60,57)=  9.3231640756e-02
       coeff_l2(60,58)=  1.3425768614e+00
       coeff_l2(60,59)=  2.1769197285e-01
       coeff_l2(60,60)= -2.2896723822e-02
       coeff_l2(60,61)=  2.9250609875e-01
       coeff_l2(60,62)= -7.9441052675e-01
       coeff_l2(60,63)= -1.0501722246e-01
       coeff_l2(60,64)= -1.1212017387e-01
       coeff_l2(60,65)=  3.5714513063e-01
       coeff_l2(60,66)=  6.0564237833e-01
       coeff_l2(60,67)= -3.7679854035e-01
       coeff_l2(60,68)=  4.6683272719e-01
       coeff_l2(60,69)= -1.3442863524e-01
       coeff_l2(60,70)= -1.0457118601e-01
       coeff_l2(60,71)= -9.3115188181e-02
       coeff_l2(60,72)= -1.2253189087e-01
       coeff_l2(60,73)= -8.9017891884e-01
       coeff_l2(60,74)= -6.2189787626e-01
       coeff_l2(60,75)=  2.5238755345e-01
       coeff_l2(60,76)=  8.6648866534e-02
       coeff_l2(60,77)=  1.9743834436e-01
       coeff_l2(60,78)= -5.9319633245e-01
       coeff_l2(60,79)=  3.6806237698e-01
       coeff_l2(60,80)=  6.7722398043e-01
       coeff_l2(60,81)=  9.9419288337e-02
       coeff_l2(60,82)=  6.2948089838e-01
       coeff_l2(60,83)=  2.5168401003e-01
       coeff_l2(60,84)= -7.2519820929e-01
       coeff_l2(60,85)=  8.5168761015e-01
       coeff_l2(60,86)= -2.2307558358e-01
       coeff_l2(60,87)= -1.1587700248e-01
       coeff_l2(60,88)= -1.0990554467e-02
       coeff_l2(60,89)= -8.3832763135e-02
       coeff_l2(60,90)= -8.8412684202e-01
       coeff_l2(60,91)=  6.1171996593e-01
       coeff_l2(60,92)= -8.7482467294e-02
       coeff_l2(60,93)=  9.1605967283e-01
       coeff_l2(60,94)=  1.3314551115e-01
       coeff_l2(60,95)=  4.1903722286e-01
       coeff_l2(61,1)= -1.2615242600e-01
       coeff_l2(61,2)=  2.7715423703e-01
       coeff_l2(61,3)=  3.3369949460e-01
       coeff_l2(61,4)=  4.8884770274e-01
       coeff_l2(61,5)= -2.1059766412e-01
       coeff_l2(61,6)=  2.8997883201e-01
       coeff_l2(61,7)=  1.9128569961e-01
       coeff_l2(61,8)= -5.0808176398e-02
       coeff_l2(61,9)= -2.5352883339e-01
       coeff_l2(61,10)= -1.2778503895e+00
       coeff_l2(61,11)= -5.0056021661e-02
       coeff_l2(61,12)= -3.2153669000e-01
       coeff_l2(61,13)= -4.7975209355e-01
       coeff_l2(61,14)= -9.3488663435e-02
       coeff_l2(61,15)= -5.4127961397e-01
       coeff_l2(61,16)= -1.3910844922e-01
       coeff_l2(61,17)= -2.1960911155e-01
       coeff_l2(61,18)=  1.9393327832e-01
       coeff_l2(61,19)= -2.9040557146e-01
       coeff_l2(61,20)= -3.0599039793e-01
       coeff_l2(61,21)= -3.3029273152e-01
       coeff_l2(61,22)= -2.4763043225e-01
       coeff_l2(61,23)= -1.9504053891e-01
       coeff_l2(61,24)= -3.7884646654e-01
       coeff_l2(61,25)=  9.4587085769e-03
       coeff_l2(61,26)=  4.5915356278e-01
       coeff_l2(61,27)=  1.9331574440e-01
       coeff_l2(61,28)=  2.2477383912e-01
       coeff_l2(61,29)=  1.9431720674e-01
       coeff_l2(61,30)= -1.4680628479e-01
       coeff_l2(61,31)= -1.0583452880e-02
       coeff_l2(61,32)= -9.4058737159e-02
       coeff_l2(61,33)= -2.8630763292e-01
       coeff_l2(61,34)=  1.5144707263e-01
       coeff_l2(61,35)=  4.3117699027e-01
       coeff_l2(61,36)= -8.2851266861e-01
       coeff_l2(61,37)=  6.3285624981e-01
       coeff_l2(61,38)=  3.9461459965e-02
       coeff_l2(61,39)=  5.7384848595e-01
       coeff_l2(61,40)=  4.2603138089e-01
       coeff_l2(61,41)= -2.8763848543e-01
       coeff_l2(61,42)= -1.0240025818e-01
       coeff_l2(61,43)= -5.3971964121e-01
       coeff_l2(61,44)= -2.7025380731e-01
       coeff_l2(61,45)= -7.9209797084e-02
       coeff_l2(61,46)= -1.1013115942e-01
       coeff_l2(61,47)= -8.5555630922e-01
       coeff_l2(61,48)= -1.5529148513e-03
       coeff_l2(61,49)=  7.8806236386e-02
       coeff_l2(61,50)=  1.2917146683e+00
       coeff_l2(61,51)=  1.9791292027e-02
       coeff_l2(61,52)=  1.9128218293e-01
       coeff_l2(61,53)= -1.2977740169e-01
       coeff_l2(61,54)=  6.5352069214e-03
       coeff_l2(61,55)=  3.6449921131e-01
       coeff_l2(61,56)= -2.5781646371e-01
       coeff_l2(61,57)=  3.8590717316e-01
       coeff_l2(61,58)= -2.7236464620e-01
       coeff_l2(61,59)= -1.5236179344e-02
       coeff_l2(61,60)= -4.8213880509e-02
       coeff_l2(61,61)= -8.1613019109e-02
       coeff_l2(61,62)=  4.4641152024e-01
       coeff_l2(61,63)=  4.3254706264e-01
       coeff_l2(61,64)=  4.1222065687e-01
       coeff_l2(61,65)=  5.0657218695e-01
       coeff_l2(61,66)= -1.2594534159e+00
       coeff_l2(61,67)= -5.2560013533e-01
       coeff_l2(61,68)= -2.8250652552e-01
       coeff_l2(61,69)= -2.2746618837e-02
       coeff_l2(61,70)= -2.3816687986e-02
       coeff_l2(61,71)= -7.7344191074e-01
       coeff_l2(61,72)= -1.7519128323e-01
       coeff_l2(61,73)= -7.9370498657e-02
       coeff_l2(61,74)= -4.3614000082e-01
       coeff_l2(61,75)= -2.8667226434e-01
       coeff_l2(61,76)= -1.4597287774e-01
       coeff_l2(61,77)= -5.7285106182e-01
       coeff_l2(61,78)=  3.7290122360e-02
       coeff_l2(61,79)= -1.3861542940e-01
       coeff_l2(61,80)= -1.1593107134e-01
       coeff_l2(61,81)=  7.7249817550e-02
       coeff_l2(61,82)= -5.3584024310e-02
       coeff_l2(61,83)=  1.5146382153e-01
       coeff_l2(61,84)= -5.8644980192e-02
       coeff_l2(61,85)=  4.4160860777e-01
       coeff_l2(61,86)=  5.7557684183e-01
       coeff_l2(61,87)= -2.3439495265e-01
       coeff_l2(61,88)= -4.2103615403e-01
       coeff_l2(61,89)= -1.3170689344e-01
       coeff_l2(61,90)= -3.9869347215e-01
       coeff_l2(61,91)= -1.2665127218e-01
       coeff_l2(61,92)=  9.4459645450e-02
       coeff_l2(61,93)= -3.8326805830e-01
       coeff_l2(61,94)= -1.8440237641e-01
       coeff_l2(61,95)=  1.1241769418e-02
       coeff_l2(62,1)=  7.5558409095e-02
       coeff_l2(62,2)= -4.0310925245e-01
       coeff_l2(62,3)= -2.3194660246e-01
       coeff_l2(62,4)=  2.4005332589e-01
       coeff_l2(62,5)= -2.4569016695e-01
       coeff_l2(62,6)= -2.9964518547e-01
       coeff_l2(62,7)= -1.5744373202e-02
       coeff_l2(62,8)= -7.3730774224e-02
       coeff_l2(62,9)=  7.2662487626e-02
       coeff_l2(62,10)=  9.6487873793e-01
       coeff_l2(62,11)= -3.2274831086e-02
       coeff_l2(62,12)=  1.0266629606e-01
       coeff_l2(62,13)= -1.5998952091e-01
       coeff_l2(62,14)=  1.9899499416e-01
       coeff_l2(62,15)= -1.2034043670e-02
       coeff_l2(62,16)=  6.7681208253e-02
       coeff_l2(62,17)= -1.6045354307e-02
       coeff_l2(62,18)=  2.1233215928e-01
       coeff_l2(62,19)= -7.8236326575e-02
       coeff_l2(62,20)=  3.2216265798e-01
       coeff_l2(62,21)=  1.6877432168e-01
       coeff_l2(62,22)= -4.1122072935e-01
       coeff_l2(62,23)= -1.8276655674e-01
       coeff_l2(62,24)=  2.8559282422e-01
       coeff_l2(62,25)= -7.4217796326e-02
       coeff_l2(62,26)= -2.7287712693e-01
       coeff_l2(62,27)= -7.1429600939e-03
       coeff_l2(62,28)= -1.1656893790e-01
       coeff_l2(62,29)= -6.4077034593e-02
       coeff_l2(62,30)=  1.7861239612e-01
       coeff_l2(62,31)=  2.0991930366e-01
       coeff_l2(62,32)= -4.0369935334e-02
       coeff_l2(62,33)=  3.2010513544e-01
       coeff_l2(62,34)= -2.8232839704e-01
       coeff_l2(62,35)=  7.3093307018e-01
       coeff_l2(62,36)=  2.0714391768e-01
       coeff_l2(62,37)= -4.8805752397e-01
       coeff_l2(62,38)= -2.6478093863e-01
       coeff_l2(62,39)=  5.0314251333e-02
       coeff_l2(62,40)= -2.2648322582e-01
       coeff_l2(62,41)=  3.3240878582e-01
       coeff_l2(62,42)= -6.3812665641e-02
       coeff_l2(62,43)=  4.6297404170e-01
       coeff_l2(62,44)=  2.0214976370e-01
       coeff_l2(62,45)= -6.9581590593e-02
       coeff_l2(62,46)=  5.0560861826e-01
       coeff_l2(62,47)=  2.1001833677e-01
       coeff_l2(62,48)= -9.5077708364e-02
       coeff_l2(62,49)= -2.7085077763e-01
       coeff_l2(62,50)=  4.1155067086e-01
       coeff_l2(62,51)= -2.2583408654e-01
       coeff_l2(62,52)= -3.8131266832e-01
       coeff_l2(62,53)=  2.1079514176e-02
       coeff_l2(62,54)=  7.8286910430e-03
       coeff_l2(62,55)= -1.4650520682e-01
       coeff_l2(62,56)= -4.2840802670e-01
       coeff_l2(62,57)= -2.7006006241e-01
       coeff_l2(62,58)= -2.9708969593e-01
       coeff_l2(62,59)=  9.6310123801e-02
       coeff_l2(62,60)=  5.3783334792e-02
       coeff_l2(62,61)= -6.6493071616e-02
       coeff_l2(62,62)=  3.0075290799e-01
       coeff_l2(62,63)= -1.1321718246e-01
       coeff_l2(62,64)=  1.1490842700e-01
       coeff_l2(62,65)= -6.3519167900e-01
       coeff_l2(62,66)=  5.9088033438e-01
       coeff_l2(62,67)=  9.0658313036e-01
       coeff_l2(62,68)=  2.3729301989e-01
       coeff_l2(62,69)= -2.4899587035e-01
       coeff_l2(62,70)=  2.4201020598e-01
       coeff_l2(62,71)=  5.0630062819e-01
       coeff_l2(62,72)= -3.9249491692e-01
       coeff_l2(62,73)= -1.1146239936e-01
       coeff_l2(62,74)= -1.2186104804e-01
       coeff_l2(62,75)= -4.0926951170e-01
       coeff_l2(62,76)= -2.0786464214e-01
       coeff_l2(62,77)=  1.9680662453e-01
       coeff_l2(62,78)= -1.7157407105e-01
       coeff_l2(62,79)= -2.9247391224e-01
       coeff_l2(62,80)= -1.1468052864e-01
       coeff_l2(62,81)= -3.6294737458e-01
       coeff_l2(62,82)= -1.6006208956e-01
       coeff_l2(62,83)=  9.5078796148e-02
       coeff_l2(62,84)= -1.7022117972e-01
       coeff_l2(62,85)= -3.9761215448e-01
       coeff_l2(62,86)= -1.7427206039e-02
       coeff_l2(62,87)= -1.9253031909e-01
       coeff_l2(62,88)=  1.2765456736e-01
       coeff_l2(62,89)= -4.9824920297e-01
       coeff_l2(62,90)=  3.3202341199e-01
       coeff_l2(62,91)= -3.1063902378e-01
       coeff_l2(62,92)= -7.9127363861e-03
       coeff_l2(62,93)= -9.3321993947e-02
       coeff_l2(62,94)= -3.4718370438e-01
       coeff_l2(62,95)=  7.0409383625e-03
       coeff_l2(63,1)=  6.5960097313e-01
       coeff_l2(63,2)=  3.4593245387e-01
       coeff_l2(63,3)= -3.0323010683e-01
       coeff_l2(63,4)= -1.6670428216e-01
       coeff_l2(63,5)=  1.6067168117e-01
       coeff_l2(63,6)=  1.7464053631e-01
       coeff_l2(63,7)=  7.2168087959e-01
       coeff_l2(63,8)= -7.8977930546e-01
       coeff_l2(63,9)=  4.9336805940e-01
       coeff_l2(63,10)=  3.3733353019e-01
       coeff_l2(63,11)=  2.9690191150e-01
       coeff_l2(63,12)=  7.7010208368e-01
       coeff_l2(63,13)=  5.3392523527e-01
       coeff_l2(63,14)=  5.3003036976e-01
       coeff_l2(63,15)=  2.6215296984e-01
       coeff_l2(63,16)= -1.0829625279e-01
       coeff_l2(63,17)= -1.2276498228e-01
       coeff_l2(63,18)= -1.2552167475e-01
       coeff_l2(63,19)=  6.4209260046e-02
       coeff_l2(63,20)=  1.4882300794e-01
       coeff_l2(63,21)=  3.7338766456e-01
       coeff_l2(63,22)=  2.9807361960e-01
       coeff_l2(63,23)= -2.3823222518e-01
       coeff_l2(63,24)=  1.0142657906e-01
       coeff_l2(63,25)=  4.3926590681e-01
       coeff_l2(63,26)= -2.7489456534e-01
       coeff_l2(63,27)= -2.5243905187e-01
       coeff_l2(63,28)= -8.8141091168e-02
       coeff_l2(63,29)=  9.1288350523e-02
       coeff_l2(63,30)=  2.1143792197e-02
       coeff_l2(63,31)=  2.6709678769e-01
       coeff_l2(63,32)= -1.8520176411e-02
       coeff_l2(63,33)=  1.3960285485e-01
       coeff_l2(63,34)= -8.9616692066e-01
       coeff_l2(63,35)= -5.8560734987e-01
       coeff_l2(63,36)=  6.2077850103e-01
       coeff_l2(63,37)= -4.3814271688e-01
       coeff_l2(63,38)= -3.8989618421e-01
       coeff_l2(63,39)=  2.2856804729e-01
       coeff_l2(63,40)= -2.3539252579e-01
       coeff_l2(63,41)= -3.1761935353e-01
       coeff_l2(63,42)= -4.3503376842e-01
       coeff_l2(63,43)=  6.3659942150e-01
       coeff_l2(63,44)=  1.8780348822e-02
       coeff_l2(63,45)= -5.8734929562e-01
       coeff_l2(63,46)=  4.5005127788e-02
       coeff_l2(63,47)=  6.0209661722e-01
       coeff_l2(63,48)= -1.1628706008e-01
       coeff_l2(63,49)=  2.0840497315e-01
       coeff_l2(63,50)= -1.6206568480e-01
       coeff_l2(63,51)=  3.3401972055e-01
       coeff_l2(63,52)= -8.8355356455e-01
       coeff_l2(63,53)=  2.3028209805e-01
       coeff_l2(63,54)= -3.7191057205e-01
       coeff_l2(63,55)=  1.7157666385e-02
       coeff_l2(63,56)= -4.6817865968e-01
       coeff_l2(63,57)= -4.6884188056e-01
       coeff_l2(63,58)=  5.1759678125e-01
       coeff_l2(63,59)= -6.5143078566e-01
       coeff_l2(63,60)=  5.7662814856e-02
       coeff_l2(63,61)=  2.6643084362e-02
       coeff_l2(63,62)=  3.1324136257e-01
       coeff_l2(63,63)=  7.8677006066e-02
       coeff_l2(63,64)=  7.6854020357e-02
       coeff_l2(63,65)=  5.4879192263e-02
       coeff_l2(63,66)=  1.5743136406e+00
       coeff_l2(63,67)=  1.4094693661e+00
       coeff_l2(63,68)=  4.7164618969e-01
       coeff_l2(63,69)= -2.7751323581e-01
       coeff_l2(63,70)= -3.2285428047e-01
       coeff_l2(63,71)= -1.8916460872e-01
       coeff_l2(63,72)=  5.8523710817e-02
       coeff_l2(63,73)= -9.6420538425e-01
       coeff_l2(63,74)= -3.3807477355e-01
       coeff_l2(63,75)= -2.2188176215e-01
       coeff_l2(63,76)=  1.8277521431e-01
       coeff_l2(63,77)=  4.2649617791e-01
       coeff_l2(63,78)= -3.1937536597e-01
       coeff_l2(63,79)=  1.4360173047e-01
       coeff_l2(63,80)=  4.4780415297e-01
       coeff_l2(63,81)= -4.8571324348e-01
       coeff_l2(63,82)= -3.0690884590e-01
       coeff_l2(63,83)=  3.2612916827e-01
       coeff_l2(63,84)= -8.1940639019e-01
       coeff_l2(63,85)= -4.5524623990e-01
       coeff_l2(63,86)= -3.6680456996e-01
       coeff_l2(63,87)= -9.7547791898e-02
       coeff_l2(63,88)=  3.4905543923e-01
       coeff_l2(63,89)=  1.8800012767e-01
       coeff_l2(63,90)=  2.1591130644e-02
       coeff_l2(63,91)=  7.2388596833e-02
       coeff_l2(63,92)= -1.1052464694e-01
       coeff_l2(63,93)=  6.7903883755e-02
       coeff_l2(63,94)= -4.7641586512e-02
       coeff_l2(63,95)= -5.8780521154e-01
       coeff_l2(64,1)= -1.2476297617e+00
       coeff_l2(64,2)= -1.2921813130e-01
       coeff_l2(64,3)= -3.2149806619e-01
       coeff_l2(64,4)= -1.2121763229e+00
       coeff_l2(64,5)= -6.6096615791e-01
       coeff_l2(64,6)= -1.4625136852e+00
       coeff_l2(64,7)= -5.0024092197e-01
       coeff_l2(64,8)=  2.3148918152e-01
       coeff_l2(64,9)= -1.4888478816e-01
       coeff_l2(64,10)=  1.3732442856e+00
       coeff_l2(64,11)= -1.1403712630e-01
       coeff_l2(64,12)= -3.6616671085e-01
       coeff_l2(64,13)=  1.0086116195e-01
       coeff_l2(64,14)= -1.8780240417e-01
       coeff_l2(64,15)= -3.0781322718e-01
       coeff_l2(64,16)=  4.7323112376e-03
       coeff_l2(64,17)= -1.1926528811e-01
       coeff_l2(64,18)= -4.1355136782e-03
       coeff_l2(64,19)=  2.0705348253e-01
       coeff_l2(64,20)=  3.0571255088e-01
       coeff_l2(64,21)= -2.7857136726e-01
       coeff_l2(64,22)= -2.3873478174e-01
       coeff_l2(64,23)= -1.3470864855e-02
       coeff_l2(64,24)= -1.9452139735e-02
       coeff_l2(64,25)=  4.3461403251e-01
       coeff_l2(64,26)= -5.0382107496e-01
       coeff_l2(64,27)= -2.5877752900e-01
       coeff_l2(64,28)= -1.1141150445e-01
       coeff_l2(64,29)= -2.2654162347e-01
       coeff_l2(64,30)= -3.9889666438e-01
       coeff_l2(64,31)= -3.3232471347e-01
       coeff_l2(64,32)= -1.6459083185e-02
       coeff_l2(64,33)= -2.0907858014e-01
       coeff_l2(64,34)=  3.5871860385e-01
       coeff_l2(64,35)= -9.9726551771e-01
       coeff_l2(64,36)=  5.2739715576e-01
       coeff_l2(64,37)= -9.0132105350e-01
       coeff_l2(64,38)=  1.2790279090e-01
       coeff_l2(64,39)=  2.6744940877e-01
       coeff_l2(64,40)=  2.6071196795e-01
       coeff_l2(64,41)= -2.7447581291e-01
       coeff_l2(64,42)=  1.0437948257e-01
       coeff_l2(64,43)=  2.8768610954e-01
       coeff_l2(64,44)= -5.1620610058e-02
       coeff_l2(64,45)= -6.5346598625e-01
       coeff_l2(64,46)=  7.9989683628e-01
       coeff_l2(64,47)=  8.8402956724e-01
       coeff_l2(64,48)= -1.8791226670e-02
       coeff_l2(64,49)= -1.7545667291e-01
       coeff_l2(64,50)=  3.0105322599e-01
       coeff_l2(64,51)= -1.7716117203e-01
       coeff_l2(64,52)=  3.3736017346e-01
       coeff_l2(64,53)= -4.5777913928e-01
       coeff_l2(64,54)=  6.8979367614e-02
       coeff_l2(64,55)= -1.5348196030e-01
       coeff_l2(64,56)=  8.2002663612e-01
       coeff_l2(64,57)= -4.9411860108e-01
       coeff_l2(64,58)= -6.4566534758e-01
       coeff_l2(64,59)=  1.6114129126e-01
       coeff_l2(64,60)= -3.2241156697e-01
       coeff_l2(64,61)= -2.5608966127e-02
       coeff_l2(64,62)= -1.3809978962e-01
       coeff_l2(64,63)= -1.5653849840e+00
       coeff_l2(64,64)= -4.0566399693e-01
       coeff_l2(64,65)= -5.3954869509e-01
       coeff_l2(64,66)=  3.9588591456e-01
       coeff_l2(64,67)=  6.7717480659e-01
       coeff_l2(64,68)= -5.2617602050e-02
       coeff_l2(64,69)=  2.0628431439e-01
       coeff_l2(64,70)= -1.9335694611e-01
       coeff_l2(64,71)=  1.2932741642e+00
       coeff_l2(64,72)=  1.4157971740e-01
       coeff_l2(64,73)= -8.6999706924e-02
       coeff_l2(64,74)= -1.2102783471e-01
       coeff_l2(64,75)=  1.7544296384e-01
       coeff_l2(64,76)=  3.1043831259e-02
       coeff_l2(64,77)=  4.7510030866e-01
       coeff_l2(64,78)= -1.5131336637e-02
       coeff_l2(64,79)= -5.5418527126e-01
       coeff_l2(64,80)= -1.2323231697e+00
       coeff_l2(64,81)=  9.5101445913e-02
       coeff_l2(64,82)= -8.4212404490e-01
       coeff_l2(64,83)= -1.9687923789e-01
       coeff_l2(64,84)=  2.9643429443e-02
       coeff_l2(64,85)=  4.5772606134e-01
       coeff_l2(64,86)= -1.9434785843e-01
       coeff_l2(64,87)=  1.1576618254e-01
       coeff_l2(64,88)= -1.0419161059e-02
       coeff_l2(64,89)= -2.6573207974e-01
       coeff_l2(64,90)=  8.7851744890e-01
       coeff_l2(64,91)= -1.7668139935e-01
       coeff_l2(64,92)= -8.4295749664e-02
       coeff_l2(64,93)= -5.8954291046e-02
       coeff_l2(64,94)= -2.5081196427e-01
       coeff_l2(64,95)=  9.1853871942e-02
       coeff_l2(65,1)= -1.4983515441e-01
       coeff_l2(65,2)=  1.1637584120e-01
       coeff_l2(65,3)=  3.7528702524e-03
       coeff_l2(65,4)= -5.2310705185e-01
       coeff_l2(65,5)= -5.4874241352e-01
       coeff_l2(65,6)= -3.3731383085e-01
       coeff_l2(65,7)= -6.1727094650e-01
       coeff_l2(65,8)= -2.7720645070e-01
       coeff_l2(65,9)=  9.1717317700e-02
       coeff_l2(65,10)=  1.0015748739e+00
       coeff_l2(65,11)= -7.4190281332e-02
       coeff_l2(65,12)=  1.6213510931e-01
       coeff_l2(65,13)= -8.5227889940e-03
       coeff_l2(65,14)= -3.1833124161e-01
       coeff_l2(65,15)= -3.3192199469e-01
       coeff_l2(65,16)=  1.8022096157e-01
       coeff_l2(65,17)=  7.2829611599e-02
       coeff_l2(65,18)= -7.6737582684e-02
       coeff_l2(65,19)= -2.1206778288e-01
       coeff_l2(65,20)=  3.8089227019e-05
       coeff_l2(65,21)= -4.5719581842e-01
       coeff_l2(65,22)= -2.0845459402e-01
       coeff_l2(65,23)= -1.3434228487e-02
       coeff_l2(65,24)= -1.9436042011e-01
       coeff_l2(65,25)=  5.7547897100e-01
       coeff_l2(65,26)= -1.6594338417e-01
       coeff_l2(65,27)= -1.9126811624e-01
       coeff_l2(65,28)= -4.5243822038e-02
       coeff_l2(65,29)= -4.8701476306e-02
       coeff_l2(65,30)=  1.7702160776e-01
       coeff_l2(65,31)=  1.4176282287e-01
       coeff_l2(65,32)= -1.1336156726e-01
       coeff_l2(65,33)= -2.8656458482e-02
       coeff_l2(65,34)= -5.8239364624e-01
       coeff_l2(65,35)=  1.2091102600e+00
       coeff_l2(65,36)= -3.8280650973e-01
       coeff_l2(65,37)= -4.0522751212e-01
       coeff_l2(65,38)= -3.0991196632e-01
       coeff_l2(65,39)=  7.7785855532e-01
       coeff_l2(65,40)= -5.2107214928e-02
       coeff_l2(65,41)= -2.1189993620e-01
       coeff_l2(65,42)= -4.7884285450e-01
       coeff_l2(65,43)= -2.1820425987e-01
       coeff_l2(65,44)=  3.2361321151e-02
       coeff_l2(65,45)= -2.8364399076e-01
       coeff_l2(65,46)=  5.3235745430e-01
       coeff_l2(65,47)=  8.0008792877e-01
       coeff_l2(65,48)=  8.2960590720e-02
       coeff_l2(65,49)=  1.8542756140e-01
       coeff_l2(65,50)=  8.3659982681e-01
       coeff_l2(65,51)= -1.6989727318e-01
       coeff_l2(65,52)= -4.0694591403e-01
       coeff_l2(65,53)= -2.9336285591e-01
       coeff_l2(65,54)=  2.0217301324e-02
       coeff_l2(65,55)=  1.5761232376e-01
       coeff_l2(65,56)= -6.9976496696e-01
       coeff_l2(65,57)= -3.0769592524e-01
       coeff_l2(65,58)= -3.6449092627e-01
       coeff_l2(65,59)= -4.5621725917e-01
       coeff_l2(65,60)=  9.5302648842e-02
       coeff_l2(65,61)= -4.5359039307e-01
       coeff_l2(65,62)=  3.8084784150e-01
       coeff_l2(65,63)=  4.8194132745e-02
       coeff_l2(65,64)= -1.0524081439e-01
       coeff_l2(65,65)= -1.3286221027e-01
       coeff_l2(65,66)=  3.7871319056e-01
       coeff_l2(65,67)=  5.4833745956e-01
       coeff_l2(65,68)=  8.7854556739e-02
       coeff_l2(65,69)= -3.4251824021e-01
       coeff_l2(65,70)=  3.7634277251e-03
       coeff_l2(65,71)=  6.7200177908e-01
       coeff_l2(65,72)= -4.9304560525e-04
       coeff_l2(65,73)= -1.1964583397e+00
       coeff_l2(65,74)= -1.1460630596e-01
       coeff_l2(65,75)= -1.9385094941e-01
       coeff_l2(65,76)= -1.0182851553e-01
       coeff_l2(65,77)=  4.5047205687e-01
       coeff_l2(65,78)= -3.9092087746e-01
       coeff_l2(65,79)= -5.4821234941e-01
       coeff_l2(65,80)= -7.6191313565e-02
       coeff_l2(65,81)= -2.6944172382e-01
       coeff_l2(65,82)= -3.4221073985e-01
       coeff_l2(65,83)= -3.4553635120e-01
       coeff_l2(65,84)= -3.8008278608e-01
       coeff_l2(65,85)= -2.6679062843e-01
       coeff_l2(65,86)=  5.1476335526e-01
       coeff_l2(65,87)= -1.2553900480e-01
       coeff_l2(65,88)= -2.7882659342e-03
       coeff_l2(65,89)= -1.3490058482e-01
       coeff_l2(65,90)=  1.6854089499e-01
       coeff_l2(65,91)=  2.2038289905e-01
       coeff_l2(65,92)= -8.1600546837e-02
       coeff_l2(65,93)= -1.1969005316e-01
       coeff_l2(65,94)= -2.4399436079e-03
       coeff_l2(65,95)= -4.7111696005e-01
!coeff layer 3
       coeff_l3(1,1)= -5.2820706367e-01
       coeff_l3(1,2)=  8.5061721504e-02
       coeff_l3(1,3)=  3.4052170813e-03
       coeff_l3(1,4)=  4.3498703837e-01
       coeff_l3(1,5)=  3.7279848475e-03
       coeff_l3(1,6)= -4.1831500828e-02
       coeff_l3(1,7)= -1.4347725548e-02
       coeff_l3(1,8)= -1.3840126991e-01
       coeff_l3(1,9)=  1.9436937571e-01
       coeff_l3(1,10)=  1.8208600581e-01
       coeff_l3(1,11)=  2.8602192178e-02
       coeff_l3(1,12)=  8.7992519140e-02
       coeff_l3(1,13)= -5.6229096651e-01
       coeff_l3(1,14)= -3.8818338513e-01
       coeff_l3(1,15)=  1.7989400029e-01
       coeff_l3(1,16)=  7.5907236896e-03
       coeff_l3(1,17)= -5.1174767315e-02
       coeff_l3(1,18)= -3.8310736418e-01
       coeff_l3(1,19)= -2.2009751201e-01
       coeff_l3(1,20)=  1.7247065902e-01
       coeff_l3(1,21)=  4.6721655130e-01
       coeff_l3(1,22)=  6.5612569451e-02
       coeff_l3(1,23)=  2.2560078651e-02
       coeff_l3(1,24)=  1.4581200480e-01
       coeff_l3(1,25)=  8.6242032051e-01
       coeff_l3(1,26)= -1.8623900414e-01
       coeff_l3(1,27)=  2.6244327426e-01
       coeff_l3(1,28)=  1.6168184578e-01
       coeff_l3(1,29)=  3.0373463035e-01
       coeff_l3(1,30)=  8.2240277529e-01
       coeff_l3(1,31)=  1.5226756036e-01
       coeff_l3(1,32)= -3.4417137504e-02
       coeff_l3(1,33)=  2.2388871014e-01
       coeff_l3(1,34)= -1.4023882151e-01
       coeff_l3(1,35)= -1.3211986423e-01
       coeff_l3(1,36)= -1.0345010459e-01
       coeff_l3(1,37)=  8.3703882992e-02
       coeff_l3(1,38)= -2.6830479503e-01
       coeff_l3(1,39)=  7.0423610508e-02
       coeff_l3(1,40)=  3.5965409875e-01
       coeff_l3(1,41)=  1.6172292829e-01
       coeff_l3(1,42)=  1.5876837075e-01
       coeff_l3(1,43)=  1.2083132565e-01
       coeff_l3(1,44)=  3.3010467887e-01
       coeff_l3(1,45)= -1.1661381274e-01
       coeff_l3(1,46)=  5.6979346275e-01
       coeff_l3(1,47)=  3.5517486930e-01
       coeff_l3(1,48)= -1.0654667765e-01
       coeff_l3(1,49)=  2.4441442490e+00
       coeff_l3(1,50)= -3.9158504456e-02
       coeff_l3(1,51)= -1.6183458269e-01
       coeff_l3(1,52)=  3.2963782549e-02
       coeff_l3(1,53)=  2.9517072439e-01
       coeff_l3(1,54)=  8.4213042259e-01
       coeff_l3(1,55)=  7.2298204899e-01
       coeff_l3(1,56)=  1.3196630776e-01
       coeff_l3(1,57)= -1.0226829350e-01
       coeff_l3(1,58)=  1.1665085554e+00
       coeff_l3(1,59)=  2.1248993278e-01
       coeff_l3(1,60)= -1.2683192492e+00
       coeff_l3(1,61)=  3.4189876169e-02
       coeff_l3(1,62)=  1.4407958090e-01
       coeff_l3(1,63)=  6.2413334846e-01
       coeff_l3(1,64)=  2.1372228861e-01
       coeff_l3(1,65)=  2.0621256530e-01
       coeff_l3(1,66)= -1.9195072353e-02
       coeff_l3(1,67)= -2.4244569242e-02
       coeff_l3(1,68)= -1.1389059573e-01
       coeff_l3(1,69)=  1.6093012691e-01
       coeff_l3(1,70)=  2.6361893862e-02
       coeff_l3(1,71)= -4.5341840386e-01
       coeff_l3(1,72)= -2.2325657308e-02
       coeff_l3(1,73)= -1.3761243224e-01
       coeff_l3(1,74)=  3.5974475741e-01
       coeff_l3(1,75)=  2.8453791142e-01
       coeff_l3(1,76)=  1.4909775257e+00
       coeff_l3(1,77)= -1.4621055126e-01
       coeff_l3(1,78)= -1.0510081053e-01
       coeff_l3(1,79)=  1.0344055295e-01
       coeff_l3(1,80)=  1.4715272188e+00
       coeff_l3(1,81)=  1.0924682021e-01
       coeff_l3(1,82)= -1.9794134796e-01
       coeff_l3(1,83)=  3.2603281736e-01
       coeff_l3(1,84)=  1.2141742557e-01
       coeff_l3(1,85)=  5.1492117345e-03
       coeff_l3(1,86)= -1.6717743874e-01
       coeff_l3(1,87)= -2.0723235607e-01
       coeff_l3(1,88)=  2.1481679380e-01
       coeff_l3(1,89)=  8.9931976795e-01
       coeff_l3(1,90)=  1.4962896705e-01
       coeff_l3(1,91)=  1.7784959078e-01
       coeff_l3(1,92)= -1.6866289079e-01
       coeff_l3(1,93)= -3.5336220264e-01
       coeff_l3(1,94)=  3.8568142802e-02
       coeff_l3(1,95)= -1.5240407549e-02
       coeff_l3(1,96)=  4.0062263608e-01
       coeff_l3(1,97)= -1.9196966290e-01
       coeff_l3(1,98)= -3.1178429723e-01
       coeff_l3(1,99)= -6.9240897894e-01
       coeff_l3(1,100)= -1.4064215124e-01
       coeff_l3(1,101)= -6.2392264605e-02
       coeff_l3(1,102)=  5.3101494908e-02
       coeff_l3(1,103)=  8.7368950248e-02
       coeff_l3(1,104)=  1.7687596381e-01
       coeff_l3(1,105)=  5.8785982430e-02
       coeff_l3(1,106)=  3.1090566888e-02
       coeff_l3(1,107)=  2.2538822889e-01
       coeff_l3(1,108)= -3.4164097905e-01
       coeff_l3(1,109)=  1.1996263266e-01
       coeff_l3(1,110)=  8.3268024027e-02
       coeff_l3(1,111)= -1.4662115276e-01
       coeff_l3(1,112)=  5.7502649724e-02
       coeff_l3(1,113)= -1.3847814500e-01
       coeff_l3(1,114)=  4.8241215944e-01
       coeff_l3(1,115)= -1.2267944217e-01
       coeff_l3(1,116)=  2.1193970740e-01
       coeff_l3(1,117)=  5.3859156370e-01
       coeff_l3(1,118)=  3.4768286347e-01
       coeff_l3(1,119)=  3.4424457699e-02
       coeff_l3(1,120)= -1.3377167284e-01
       coeff_l3(1,121)=  9.3928635120e-02
       coeff_l3(1,122)= -7.7102988958e-02
       coeff_l3(1,123)= -2.4462573230e-02
       coeff_l3(1,124)=  2.6961782575e-01
       coeff_l3(1,125)=  6.0216255486e-02
       coeff_l3(2,1)= -7.5245350599e-01
       coeff_l3(2,2)=  2.0239265636e-02
       coeff_l3(2,3)=  2.8851810098e-01
       coeff_l3(2,4)= -1.9290392101e-01
       coeff_l3(2,5)=  4.1149578989e-02
       coeff_l3(2,6)=  1.8974497914e-01
       coeff_l3(2,7)= -7.6307795942e-02
       coeff_l3(2,8)= -2.1764780581e-01
       coeff_l3(2,9)= -1.6423349082e-01
       coeff_l3(2,10)= -3.1497639418e-01
       coeff_l3(2,11)=  2.9534250498e-03
       coeff_l3(2,12)= -5.3999239206e-01
       coeff_l3(2,13)= -2.4947044253e-01
       coeff_l3(2,14)= -3.4457921982e-01
       coeff_l3(2,15)= -9.1888830066e-02
       coeff_l3(2,16)= -1.1320703663e-02
       coeff_l3(2,17)= -4.6793189645e-01
       coeff_l3(2,18)= -4.8905527592e-01
       coeff_l3(2,19)=  9.4244129956e-02
       coeff_l3(2,20)= -1.9972558320e-01
       coeff_l3(2,21)=  1.4674809575e-01
       coeff_l3(2,22)= -5.0016518682e-02
       coeff_l3(2,23)=  8.1859281636e-05
       coeff_l3(2,24)= -1.6850768030e-01
       coeff_l3(2,25)=  3.8425216079e-01
       coeff_l3(2,26)=  3.9533115923e-02
       coeff_l3(2,27)=  2.7810338140e-01
       coeff_l3(2,28)=  2.5286778808e-01
       coeff_l3(2,29)= -7.0185291767e-01
       coeff_l3(2,30)=  5.3793382645e-01
       coeff_l3(2,31)= -1.3288262486e-01
       coeff_l3(2,32)=  1.3477531075e-01
       coeff_l3(2,33)= -4.9459401518e-02
       coeff_l3(2,34)=  1.2057245523e-01
       coeff_l3(2,35)= -7.0521765947e-01
       coeff_l3(2,36)=  5.5107690394e-02
       coeff_l3(2,37)=  9.9903881550e-02
       coeff_l3(2,38)= -9.2374277115e-01
       coeff_l3(2,39)=  5.7177111506e-02
       coeff_l3(2,40)= -2.8936895728e-01
       coeff_l3(2,41)= -2.9464003444e-01
       coeff_l3(2,42)=  1.7084965110e-01
       coeff_l3(2,43)= -3.2137143612e-01
       coeff_l3(2,44)= -1.4782758057e-01
       coeff_l3(2,45)=  3.5779196024e-01
       coeff_l3(2,46)=  2.8719341755e-01
       coeff_l3(2,47)=  1.1683594435e-01
       coeff_l3(2,48)= -2.7986481786e-01
       coeff_l3(2,49)=  8.6952757835e-01
       coeff_l3(2,50)= -2.1284900606e-02
       coeff_l3(2,51)= -9.4991743565e-02
       coeff_l3(2,52)= -6.4204491675e-02
       coeff_l3(2,53)= -1.4322302304e-02
       coeff_l3(2,54)= -7.9847753048e-02
       coeff_l3(2,55)= -3.6868542433e-02
       coeff_l3(2,56)= -2.2079291940e-01
       coeff_l3(2,57)= -3.0238583684e-02
       coeff_l3(2,58)=  6.2564688921e-01
       coeff_l3(2,59)= -1.2954576313e-01
       coeff_l3(2,60)=  4.4182160497e-01
       coeff_l3(2,61)=  1.5407346189e-01
       coeff_l3(2,62)=  1.1667037755e-01
       coeff_l3(2,63)=  7.9609014094e-02
       coeff_l3(2,64)= -1.5855431557e-01
       coeff_l3(2,65)= -5.0006888807e-02
       coeff_l3(2,66)=  1.6135960817e-01
       coeff_l3(2,67)=  6.7108136415e-01
       coeff_l3(2,68)= -2.4249291420e-01
       coeff_l3(2,69)=  4.3679863214e-02
       coeff_l3(2,70)=  2.1474756300e-01
       coeff_l3(2,71)= -5.7775032520e-01
       coeff_l3(2,72)=  1.3678315282e-01
       coeff_l3(2,73)=  9.6985943615e-02
       coeff_l3(2,74)=  2.6473233104e-01
       coeff_l3(2,75)= -8.6507342756e-02
       coeff_l3(2,76)=  3.6887663603e-01
       coeff_l3(2,77)=  3.5382647067e-02
       coeff_l3(2,78)=  7.5794667006e-02
       coeff_l3(2,79)= -1.4574234188e-01
       coeff_l3(2,80)=  3.8500198722e-01
       coeff_l3(2,81)= -1.3623744249e-01
       coeff_l3(2,82)=  6.7881420255e-02
       coeff_l3(2,83)= -2.9465541244e-01
       coeff_l3(2,84)=  4.6706569195e-01
       coeff_l3(2,85)= -1.3560236990e-01
       coeff_l3(2,86)=  1.4372907579e-01
       coeff_l3(2,87)= -5.3263440728e-02
       coeff_l3(2,88)= -1.1837227643e-01
       coeff_l3(2,89)=  1.0997200012e-01
       coeff_l3(2,90)=  1.8135629594e-01
       coeff_l3(2,91)= -3.2495492697e-01
       coeff_l3(2,92)=  6.4139127731e-02
       coeff_l3(2,93)= -4.8241015524e-02
       coeff_l3(2,94)= -1.2014080584e-01
       coeff_l3(2,95)= -1.5748691559e-01
       coeff_l3(2,96)= -2.0043161511e-01
       coeff_l3(2,97)= -1.4591380954e-01
       coeff_l3(2,98)=  3.0136322975e-01
       coeff_l3(2,99)= -2.9270032048e-01
       coeff_l3(2,100)= -1.3981899619e-01
       coeff_l3(2,101)= -3.6412143707e-01
       coeff_l3(2,102)= -9.5649845898e-02
       coeff_l3(2,103)= -1.1845230311e-01
       coeff_l3(2,104)= -1.0673642159e-01
       coeff_l3(2,105)= -2.8883203864e-01
       coeff_l3(2,106)= -4.0901520848e-01
       coeff_l3(2,107)= -1.7175531015e-02
       coeff_l3(2,108)= -2.5413623452e-01
       coeff_l3(2,109)= -9.4261482358e-02
       coeff_l3(2,110)= -7.7288627625e-02
       coeff_l3(2,111)= -1.2962032855e-01
       coeff_l3(2,112)= -6.4730262756e-01
       coeff_l3(2,113)= -4.7608396411e-01
       coeff_l3(2,114)= -1.7710867524e-01
       coeff_l3(2,115)=  1.1888056062e-02
       coeff_l3(2,116)= -6.1626541615e-01
       coeff_l3(2,117)=  6.0932300985e-02
       coeff_l3(2,118)=  2.7321755886e-01
       coeff_l3(2,119)= -1.4351645112e-01
       coeff_l3(2,120)= -1.3796030544e-02
       coeff_l3(2,121)= -2.4689024687e-01
       coeff_l3(2,122)= -1.4120097458e-01
       coeff_l3(2,123)=  1.3087156415e-01
       coeff_l3(2,124)= -5.5731635541e-02
       coeff_l3(2,125)= -1.0852022469e-01
       coeff_l3(3,1)= -1.2653564215e+00
       coeff_l3(3,2)=  1.4211589098e-01
       coeff_l3(3,3)=  3.7108632922e-01
       coeff_l3(3,4)= -1.3711554930e-02
       coeff_l3(3,5)=  4.1835099459e-02
       coeff_l3(3,6)=  1.8830955029e-01
       coeff_l3(3,7)=  1.4029228687e-01
       coeff_l3(3,8)=  4.9515329301e-02
       coeff_l3(3,9)=  2.5037242100e-02
       coeff_l3(3,10)= -1.0981948674e-01
       coeff_l3(3,11)= -1.4208185673e-01
       coeff_l3(3,12)= -5.6056082249e-01
       coeff_l3(3,13)= -4.2888185382e-01
       coeff_l3(3,14)= -2.9579359293e-01
       coeff_l3(3,15)=  8.2577675581e-02
       coeff_l3(3,16)=  9.7300842404e-02
       coeff_l3(3,17)= -1.2483666092e-01
       coeff_l3(3,18)= -1.0925998688e+00
       coeff_l3(3,19)=  1.6568499804e-01
       coeff_l3(3,20)= -9.2817656696e-02
       coeff_l3(3,21)= -1.0726948828e-01
       coeff_l3(3,22)=  3.4454202652e-01
       coeff_l3(3,23)=  1.0048275441e-01
       coeff_l3(3,24)=  1.5195424855e-01
       coeff_l3(3,25)=  7.2448176146e-01
       coeff_l3(3,26)=  6.8569302559e-02
       coeff_l3(3,27)=  2.8998515010e-01
       coeff_l3(3,28)=  2.5914618373e-01
       coeff_l3(3,29)= -4.8643499613e-01
       coeff_l3(3,30)= -1.6851159930e-01
       coeff_l3(3,31)=  1.9835779071e-01
       coeff_l3(3,32)=  6.8699598312e-02
       coeff_l3(3,33)= -1.0636155494e-02
       coeff_l3(3,34)= -7.6093681157e-02
       coeff_l3(3,35)=  2.7218976617e-01
       coeff_l3(3,36)=  2.9659387469e-01
       coeff_l3(3,37)=  2.0165788010e-02
       coeff_l3(3,38)= -7.4425679445e-01
       coeff_l3(3,39)=  1.4757208526e-01
       coeff_l3(3,40)= -6.3765652478e-02
       coeff_l3(3,41)= -6.1010861397e-01
       coeff_l3(3,42)=  2.2918352485e-01
       coeff_l3(3,43)= -3.2668173313e-01
       coeff_l3(3,44)=  2.2104054689e-01
       coeff_l3(3,45)= -3.7075555325e-01
       coeff_l3(3,46)=  1.8974308670e-01
       coeff_l3(3,47)= -9.2775523663e-03
       coeff_l3(3,48)= -4.4625166059e-01
       coeff_l3(3,49)=  3.6649304628e-01
       coeff_l3(3,50)= -1.7301079631e-01
       coeff_l3(3,51)= -1.1355711520e-01
       coeff_l3(3,52)=  1.1366279423e-01
       coeff_l3(3,53)= -8.0067172647e-02
       coeff_l3(3,54)=  4.2440029979e-01
       coeff_l3(3,55)=  2.3055431247e-01
       coeff_l3(3,56)=  1.4028562605e-01
       coeff_l3(3,57)=  1.1782240123e-02
       coeff_l3(3,58)=  1.3884255290e-01
       coeff_l3(3,59)=  1.5775632858e-01
       coeff_l3(3,60)=  1.5405137092e-02
       coeff_l3(3,61)=  1.5747636557e-02
       coeff_l3(3,62)=  2.6613789797e-01
       coeff_l3(3,63)=  2.4060124159e-01
       coeff_l3(3,64)=  1.0072854906e-01
       coeff_l3(3,65)=  1.0334480554e-01
       coeff_l3(3,66)= -7.6926499605e-02
       coeff_l3(3,67)= -2.2284176946e-01
       coeff_l3(3,68)= -9.3598626554e-02
       coeff_l3(3,69)= -2.8052702546e-01
       coeff_l3(3,70)= -1.9284727052e-02
       coeff_l3(3,71)= -1.4619816542e+00
       coeff_l3(3,72)=  2.6593407989e-01
       coeff_l3(3,73)= -7.5521267951e-02
       coeff_l3(3,74)=  1.3817143440e-01
       coeff_l3(3,75)=  9.8589591682e-02
       coeff_l3(3,76)=  3.2240065932e-01
       coeff_l3(3,77)= -2.1575458348e-02
       coeff_l3(3,78)=  3.4337695688e-02
       coeff_l3(3,79)=  1.2376457453e-01
       coeff_l3(3,80)=  5.2197414637e-01
       coeff_l3(3,81)=  1.2505168095e-02
       coeff_l3(3,82)= -2.7177816629e-01
       coeff_l3(3,83)=  2.3765547574e-01
       coeff_l3(3,84)=  2.3425704241e-01
       coeff_l3(3,85)=  4.6170106530e-01
       coeff_l3(3,86)= -1.3633839786e-01
       coeff_l3(3,87)= -2.2240525484e-01
       coeff_l3(3,88)=  1.8411714584e-02
       coeff_l3(3,89)= -6.8494506180e-02
       coeff_l3(3,90)=  2.0322835445e-01
       coeff_l3(3,91)=  7.7133871615e-02
       coeff_l3(3,92)=  5.5527698249e-02
       coeff_l3(3,93)= -1.7089240253e-01
       coeff_l3(3,94)= -2.2334825993e-01
       coeff_l3(3,95)= -6.4394690096e-02
       coeff_l3(3,96)=  1.4244297147e-01
       coeff_l3(3,97)= -1.8945732713e-01
       coeff_l3(3,98)=  2.5347590446e-01
       coeff_l3(3,99)=  3.8322433829e-01
       coeff_l3(3,100)= -1.2136667967e-01
       coeff_l3(3,101)= -1.4938546345e-02
       coeff_l3(3,102)=  9.3164660037e-02
       coeff_l3(3,103)=  1.9874925911e-01
       coeff_l3(3,104)=  2.1928152442e-01
       coeff_l3(3,105)= -5.7512265444e-01
       coeff_l3(3,106)= -1.9086033106e-01
       coeff_l3(3,107)= -3.4288233519e-01
       coeff_l3(3,108)= -2.8151929379e-01
       coeff_l3(3,109)=  9.7458586097e-02
       coeff_l3(3,110)=  1.1025825888e-01
       coeff_l3(3,111)= -2.8521201015e-01
       coeff_l3(3,112)= -5.3259509802e-01
       coeff_l3(3,113)= -7.7546691895e-01
       coeff_l3(3,114)= -1.4114086516e-02
       coeff_l3(3,115)= -1.7648498714e-01
       coeff_l3(3,116)= -2.1834242344e-01
       coeff_l3(3,117)=  1.0992932320e-01
       coeff_l3(3,118)= -3.8667675853e-01
       coeff_l3(3,119)= -1.3762073219e-01
       coeff_l3(3,120)= -3.8229681551e-02
       coeff_l3(3,121)=  8.0928891897e-02
       coeff_l3(3,122)=  8.5797809064e-02
       coeff_l3(3,123)= -4.4942747802e-02
       coeff_l3(3,124)=  1.6703408211e-02
       coeff_l3(3,125)=  2.6759658009e-02
       coeff_l3(4,1)=  7.8717023134e-02
       coeff_l3(4,2)= -1.2961858511e-01
       coeff_l3(4,3)=  2.9764431715e-01
       coeff_l3(4,4)=  2.5485992432e-01
       coeff_l3(4,5)= -1.0288672894e-01
       coeff_l3(4,6)=  3.3130121883e-03
       coeff_l3(4,7)=  1.5925087929e+00
       coeff_l3(4,8)=  2.3923787475e-01
       coeff_l3(4,9)= -9.0319551528e-02
       coeff_l3(4,10)= -2.9947545379e-02
       coeff_l3(4,11)=  4.6031095088e-02
       coeff_l3(4,12)=  1.1236508191e-01
       coeff_l3(4,13)=  2.6339349151e-01
       coeff_l3(4,14)=  2.7634884464e-04
       coeff_l3(4,15)= -1.1186509579e-01
       coeff_l3(4,16)= -1.1141694151e-02
       coeff_l3(4,17)=  7.5808294117e-02
       coeff_l3(4,18)=  7.1572944522e-02
       coeff_l3(4,19)=  2.3503872752e-01
       coeff_l3(4,20)= -1.4231643081e-01
       coeff_l3(4,21)=  4.1843265295e-01
       coeff_l3(4,22)=  7.0075400174e-02
       coeff_l3(4,23)= -3.1212717295e-01
       coeff_l3(4,24)= -1.0755060613e-01
       coeff_l3(4,25)=  2.6944298297e-02
       coeff_l3(4,26)= -9.1252490878e-02
       coeff_l3(4,27)=  2.4736399651e+00
       coeff_l3(4,28)=  1.8921104670e+00
       coeff_l3(4,29)=  3.9534148574e-01
       coeff_l3(4,30)= -6.0036121868e-03
       coeff_l3(4,31)= -1.1541797966e-01
       coeff_l3(4,32)= -1.9189602137e-01
       coeff_l3(4,33)=  1.3127861917e-01
       coeff_l3(4,34)= -5.1984928548e-02
       coeff_l3(4,35)=  3.8001617789e-01
       coeff_l3(4,36)= -1.6185091808e-02
       coeff_l3(4,37)= -1.6965962946e-01
       coeff_l3(4,38)=  6.2014043331e-01
       coeff_l3(4,39)=  1.1727257073e-01
       coeff_l3(4,40)=  2.8277695179e-01
       coeff_l3(4,41)=  5.5198806524e-01
       coeff_l3(4,42)= -3.5474435426e-03
       coeff_l3(4,43)=  1.7375306785e-01
       coeff_l3(4,44)= -1.7497517169e-01
       coeff_l3(4,45)= -1.5295854211e-01
       coeff_l3(4,46)= -9.9439129233e-02
       coeff_l3(4,47)=  1.3894864917e-01
       coeff_l3(4,48)= -9.1410808265e-02
       coeff_l3(4,49)=  3.7466576099e+00
       coeff_l3(4,50)= -1.4215363562e-01
       coeff_l3(4,51)= -4.4293526560e-02
       coeff_l3(4,52)=  1.0054278374e-01
       coeff_l3(4,53)= -1.3058355451e-01
       coeff_l3(4,54)=  9.7711199522e-01
       coeff_l3(4,55)=  2.0546175539e-01
       coeff_l3(4,56)=  3.7618130445e-02
       coeff_l3(4,57)=  3.9423767477e-02
       coeff_l3(4,58)=  4.7603040934e-02
       coeff_l3(4,59)= -3.2358804345e-01
       coeff_l3(4,60)= -2.4545092583e+00
       coeff_l3(4,61)= -7.7809147537e-02
       coeff_l3(4,62)= -2.3124623299e-01
       coeff_l3(4,63)=  8.5983669758e-01
       coeff_l3(4,64)= -2.5546854734e-01
       coeff_l3(4,65)= -2.0557808876e-01
       coeff_l3(4,66)= -1.2102041394e-01
       coeff_l3(4,67)= -2.7875885367e-01
       coeff_l3(4,68)=  5.5655792356e-02
       coeff_l3(4,69)= -8.8616669178e-02
       coeff_l3(4,70)=  9.0341679752e-02
       coeff_l3(4,71)=  7.9504840076e-02
       coeff_l3(4,72)=  6.7236120813e-03
       coeff_l3(4,73)=  1.0086911172e-01
       coeff_l3(4,74)=  6.2566649914e-01
       coeff_l3(4,75)= -1.7728641629e-01
       coeff_l3(4,76)=  1.5595495701e+00
       coeff_l3(4,77)=  6.7697489262e-01
       coeff_l3(4,78)= -3.8134444505e-02
       coeff_l3(4,79)=  3.9713027945e-05
       coeff_l3(4,80)=  1.5745736361e+00
       coeff_l3(4,81)= -1.7548240721e-01
       coeff_l3(4,82)=  2.8890559450e-02
       coeff_l3(4,83)=  9.1273665428e-02
       coeff_l3(4,84)= -1.5161387622e-01
       coeff_l3(4,85)= -1.8097320199e-01
       coeff_l3(4,86)=  3.5530049354e-02
       coeff_l3(4,87)=  1.7356211320e-02
       coeff_l3(4,88)=  9.6127346158e-02
       coeff_l3(4,89)=  2.1122384071e-01
       coeff_l3(4,90)= -9.7144022584e-02
       coeff_l3(4,91)=  7.4484354258e-01
       coeff_l3(4,92)= -1.1648746207e-02
       coeff_l3(4,93)= -1.1269979179e-01
       coeff_l3(4,94)=  2.9365810752e-01
       coeff_l3(4,95)= -1.0362486541e-01
       coeff_l3(4,96)= -2.9853973538e-02
       coeff_l3(4,97)=  2.3280456662e-01
       coeff_l3(4,98)=  1.0267872363e-02
       coeff_l3(4,99)= -6.5652656555e-01
       coeff_l3(4,100)= -2.0754264668e-02
       coeff_l3(4,101)=  3.4328024834e-02
       coeff_l3(4,102)=  1.0062950104e-01
       coeff_l3(4,103)= -2.5541329384e-01
       coeff_l3(4,104)= -1.4242565632e-01
       coeff_l3(4,105)=  2.7234500647e-01
       coeff_l3(4,106)= -9.2776365578e-02
       coeff_l3(4,107)= -1.9202154875e-01
       coeff_l3(4,108)=  1.2000433356e-01
       coeff_l3(4,109)= -1.2708668411e-01
       coeff_l3(4,110)=  9.0089879930e-02
       coeff_l3(4,111)=  2.2128807008e-01
       coeff_l3(4,112)= -3.3842399716e-01
       coeff_l3(4,113)= -2.3196948692e-02
       coeff_l3(4,114)=  8.9845520258e-01
       coeff_l3(4,115)=  3.0966669321e-01
       coeff_l3(4,116)=  1.0210645944e-01
       coeff_l3(4,117)=  1.6964372993e-01
       coeff_l3(4,118)=  3.3224946260e-01
       coeff_l3(4,119)= -8.7565563619e-02
       coeff_l3(4,120)=  6.8722240627e-02
       coeff_l3(4,121)= -3.4386128187e-01
       coeff_l3(4,122)=  9.1611109674e-02
       coeff_l3(4,123)=  8.9228242636e-02
       coeff_l3(4,124)=  2.4304684252e-02
       coeff_l3(4,125)=  5.3236983716e-02
       coeff_l3(5,1)= -8.8669729233e-01
       coeff_l3(5,2)= -8.8624989986e-01
       coeff_l3(5,3)=  3.2358470559e-01
       coeff_l3(5,4)=  1.1151924133e+00
       coeff_l3(5,5)= -1.5931868553e-01
       coeff_l3(5,6)= -1.6994355619e-01
       coeff_l3(5,7)= -2.3670692742e-01
       coeff_l3(5,8)=  3.5090401769e-01
       coeff_l3(5,9)= -2.9025328159e-01
       coeff_l3(5,10)= -3.1302090734e-02
       coeff_l3(5,11)=  2.9344698787e-01
       coeff_l3(5,12)= -8.7092834711e-01
       coeff_l3(5,13)= -9.4646495581e-01
       coeff_l3(5,14)= -8.0687558651e-01
       coeff_l3(5,15)= -4.4841903448e-01
       coeff_l3(5,16)= -1.7014694214e-01
       coeff_l3(5,17)= -2.9339316487e-01
       coeff_l3(5,18)= -4.3218687177e-01
       coeff_l3(5,19)=  2.2020423412e-01
       coeff_l3(5,20)= -3.6616611481e-01
       coeff_l3(5,21)= -1.4952707291e-01
       coeff_l3(5,22)= -4.2018007487e-02
       coeff_l3(5,23)=  3.6397155374e-02
       coeff_l3(5,24)= -3.9065644145e-01
       coeff_l3(5,25)= -2.5037291646e-01
       coeff_l3(5,26)= -1.6197539866e-01
       coeff_l3(5,27)= -1.3183290958e+00
       coeff_l3(5,28)= -9.9039578438e-01
       coeff_l3(5,29)=  2.1114645898e-01
       coeff_l3(5,30)=  7.4689579010e-01
       coeff_l3(5,31)= -3.4152910113e-01
       coeff_l3(5,32)= -4.2960265279e-01
       coeff_l3(5,33)=  2.4806648493e-02
       coeff_l3(5,34)=  1.6664215922e-01
       coeff_l3(5,35)=  1.0313221216e+00
       coeff_l3(5,36)= -1.3222761452e-01
       coeff_l3(5,37)= -1.2044326216e-01
       coeff_l3(5,38)=  1.7092596292e+00
       coeff_l3(5,39)= -5.5441099405e-01
       coeff_l3(5,40)=  4.7536630183e-02
       coeff_l3(5,41)=  3.2325667143e-01
       coeff_l3(5,42)= -4.2864036560e-01
       coeff_l3(5,43)= -6.8036621809e-01
       coeff_l3(5,44)= -1.5246310830e-01
       coeff_l3(5,45)= -6.2873490155e-02
       coeff_l3(5,46)=  4.0904027224e-01
       coeff_l3(5,47)=  3.6281999946e-01
       coeff_l3(5,48)= -1.6940519214e-01
       coeff_l3(5,49)=  2.3281812668e+00
       coeff_l3(5,50)= -6.4367555082e-02
       coeff_l3(5,51)= -2.0435734093e-01
       coeff_l3(5,52)= -4.1468867660e-01
       coeff_l3(5,53)= -1.3843728229e-02
       coeff_l3(5,54)= -8.6345815659e-01
       coeff_l3(5,55)=  3.4679612517e-01
       coeff_l3(5,56)= -4.4112406671e-02
       coeff_l3(5,57)= -6.7789629102e-02
       coeff_l3(5,58)= -1.3245339692e-01
       coeff_l3(5,59)=  1.3475744426e-01
       coeff_l3(5,60)=  4.1901335716e+00
       coeff_l3(5,61)= -3.1259799004e-01
       coeff_l3(5,62)= -2.6922625303e-01
       coeff_l3(5,63)= -1.0748449713e-01
       coeff_l3(5,64)=  9.7536854446e-02
       coeff_l3(5,65)= -2.5132647157e-01
       coeff_l3(5,66)= -2.7291092277e-01
       coeff_l3(5,67)=  1.0816920996e+00
       coeff_l3(5,68)=  2.4784325063e-01
       coeff_l3(5,69)=  5.0332397223e-02
       coeff_l3(5,70)= -4.6309211850e-01
       coeff_l3(5,71)= -6.3622987270e-01
       coeff_l3(5,72)= -3.3470571041e-01
       coeff_l3(5,73)=  2.3018242419e-01
       coeff_l3(5,74)=  9.3360286951e-01
       coeff_l3(5,75)= -3.7599644065e-01
       coeff_l3(5,76)=  1.8449641466e+00
       coeff_l3(5,77)=  1.8817125261e-01
       coeff_l3(5,78)= -1.3606850058e-02
       coeff_l3(5,79)= -1.5975707769e-01
       coeff_l3(5,80)=  3.9620247483e-01
       coeff_l3(5,81)=  1.5507642925e-01
       coeff_l3(5,82)=  2.8406879306e-01
       coeff_l3(5,83)=  3.3324214816e-01
       coeff_l3(5,84)=  3.2069402933e-01
       coeff_l3(5,85)= -4.9143433571e-01
       coeff_l3(5,86)=  5.4050870240e-02
       coeff_l3(5,87)= -1.6175067425e-01
       coeff_l3(5,88)=  3.8004112244e-01
       coeff_l3(5,89)=  5.4906326532e-01
       coeff_l3(5,90)=  1.3458143175e-01
       coeff_l3(5,91)=  6.6721397638e-01
       coeff_l3(5,92)= -2.8002217412e-01
       coeff_l3(5,93)=  1.2890177965e-01
       coeff_l3(5,94)= -1.0759974271e-01
       coeff_l3(5,95)= -3.9077776670e-01
       coeff_l3(5,96)= -1.6423305497e-02
       coeff_l3(5,97)= -2.3926679790e-01
       coeff_l3(5,98)=  1.8920764327e-01
       coeff_l3(5,99)= -1.3739451766e-01
       coeff_l3(5,100)= -1.9038842618e-01
       coeff_l3(5,101)=  5.2229851484e-02
       coeff_l3(5,102)=  1.8908172846e-02
       coeff_l3(5,103)=  2.5055184960e-01
       coeff_l3(5,104)= -2.8350347281e-01
       coeff_l3(5,105)=  6.5737664700e-01
       coeff_l3(5,106)= -8.2861647010e-02
       coeff_l3(5,107)= -2.9786357656e-02
       coeff_l3(5,108)= -1.2268019468e-01
       coeff_l3(5,109)= -5.7588589191e-01
       coeff_l3(5,110)=  1.6419920325e-01
       coeff_l3(5,111)= -3.2546040416e-01
       coeff_l3(5,112)=  4.8103883862e-01
       coeff_l3(5,113)=  3.7676611543e-01
       coeff_l3(5,114)= -7.1031093597e-01
       coeff_l3(5,115)=  3.1385535002e-01
       coeff_l3(5,116)=  8.8201802969e-01
       coeff_l3(5,117)=  5.8048391342e-01
       coeff_l3(5,118)=  2.6144817472e-01
       coeff_l3(5,119)= -1.8946351111e-01
       coeff_l3(5,120)= -3.8716033101e-02
       coeff_l3(5,121)=  9.7402133048e-02
       coeff_l3(5,122)= -7.8035168350e-02
       coeff_l3(5,123)= -4.3591383100e-01
       coeff_l3(5,124)= -5.1372015476e-01
       coeff_l3(5,125)=  2.4775739014e-01
       coeff_l3(6,1)= -7.2483472526e-02
       coeff_l3(6,2)= -1.0623667389e-01
       coeff_l3(6,3)=  1.1844617128e-01
       coeff_l3(6,4)=  1.7001684010e-01
       coeff_l3(6,5)=  3.9656382054e-02
       coeff_l3(6,6)= -1.2869323790e-01
       coeff_l3(6,7)=  6.5774941444e-01
       coeff_l3(6,8)=  9.7072340548e-02
       coeff_l3(6,9)= -5.2870433778e-02
       coeff_l3(6,10)=  1.0119182616e-01
       coeff_l3(6,11)=  1.4844552614e-02
       coeff_l3(6,12)= -2.4614264071e-01
       coeff_l3(6,13)= -4.5501217246e-01
       coeff_l3(6,14)=  6.0332184657e-03
       coeff_l3(6,15)= -1.7372651026e-02
       coeff_l3(6,16)= -1.1586239189e-01
       coeff_l3(6,17)=  3.6854248494e-02
       coeff_l3(6,18)=  1.8367852271e-01
       coeff_l3(6,19)= -3.4966340661e-01
       coeff_l3(6,20)=  2.4021004140e-01
       coeff_l3(6,21)=  4.5166361332e-01
       coeff_l3(6,22)= -5.4728873074e-02
       coeff_l3(6,23)=  8.8331196457e-03
       coeff_l3(6,24)= -1.1094094068e-01
       coeff_l3(6,25)= -2.7074968815e-01
       coeff_l3(6,26)= -2.9863737524e-02
       coeff_l3(6,27)=  4.0256470442e-01
       coeff_l3(6,28)=  7.2446979582e-02
       coeff_l3(6,29)=  6.2715911865e-01
       coeff_l3(6,30)=  1.4378483593e-01
       coeff_l3(6,31)= -1.2178499997e-01
       coeff_l3(6,32)= -7.4687086046e-02
       coeff_l3(6,33)=  1.6438937187e-01
       coeff_l3(6,34)= -8.9857175946e-02
       coeff_l3(6,35)=  2.3767058551e-01
       coeff_l3(6,36)=  1.8074214458e-02
       coeff_l3(6,37)= -1.9273705781e-01
       coeff_l3(6,38)=  1.3153406382e+00
       coeff_l3(6,39)=  9.9130704999e-02
       coeff_l3(6,40)=  1.5406005085e-01
       coeff_l3(6,41)=  1.2196126580e-01
       coeff_l3(6,42)=  7.2656020522e-02
       coeff_l3(6,43)=  1.2863768637e-01
       coeff_l3(6,44)= -9.5184989274e-02
       coeff_l3(6,45)=  1.8375594914e-01
       coeff_l3(6,46)=  4.7608971596e-02
       coeff_l3(6,47)=  2.5329772383e-02
       coeff_l3(6,48)=  2.6513412595e-01
       coeff_l3(6,49)=  2.7038171291e+00
       coeff_l3(6,50)= -7.5392538309e-01
       coeff_l3(6,51)=  2.1704637911e-03
       coeff_l3(6,52)= -5.8552403003e-02
       coeff_l3(6,53)=  3.5835284740e-02
       coeff_l3(6,54)= -6.6822493076e-01
       coeff_l3(6,55)=  2.9416406155e-01
       coeff_l3(6,56)= -8.5185216740e-03
       coeff_l3(6,57)= -7.1198730730e-03
       coeff_l3(6,58)= -6.6255748272e-01
       coeff_l3(6,59)= -6.4602875710e-01
       coeff_l3(6,60)=  3.3371577263e+00
       coeff_l3(6,61)= -3.7542834878e-02
       coeff_l3(6,62)= -4.6055953950e-02
       coeff_l3(6,63)=  1.6381278634e-01
       coeff_l3(6,64)= -9.3440942466e-02
       coeff_l3(6,65)= -9.7944289446e-02
       coeff_l3(6,66)= -1.1864776909e-01
       coeff_l3(6,67)=  5.7434362173e-01
       coeff_l3(6,68)= -1.5200151503e-01
       coeff_l3(6,69)= -3.3872258663e-01
       coeff_l3(6,70)= -5.8136966079e-02
       coeff_l3(6,71)= -8.5364043713e-02
       coeff_l3(6,72)=  5.0487730652e-02
       coeff_l3(6,73)= -2.8897708282e-02
       coeff_l3(6,74)=  6.7639535666e-01
       coeff_l3(6,75)=  2.3397646844e-02
       coeff_l3(6,76)=  1.3547067642e+00
       coeff_l3(6,77)= -1.1125207692e-01
       coeff_l3(6,78)= -1.7870262265e-01
       coeff_l3(6,79)=  3.9175809361e-03
       coeff_l3(6,80)=  8.6169999838e-01
       coeff_l3(6,81)= -3.7523421645e-01
       coeff_l3(6,82)=  6.5422773361e-02
       coeff_l3(6,83)= -1.1510327458e-01
       coeff_l3(6,84)= -2.6252079010e-01
       coeff_l3(6,85)= -5.6681823730e-01
       coeff_l3(6,86)=  8.8842943311e-02
       coeff_l3(6,87)= -5.4005645216e-02
       coeff_l3(6,88)=  4.1934472322e-01
       coeff_l3(6,89)= -5.0434816629e-02
       coeff_l3(6,90)=  3.8223949075e-01
       coeff_l3(6,91)= -1.2437270582e-01
       coeff_l3(6,92)=  1.1375372857e-01
       coeff_l3(6,93)= -2.7463623881e-01
       coeff_l3(6,94)=  7.9291827977e-02
       coeff_l3(6,95)=  8.4389597178e-02
       coeff_l3(6,96)=  1.5507337451e-01
       coeff_l3(6,97)=  1.3556322455e-01
       coeff_l3(6,98)= -8.1120744348e-02
       coeff_l3(6,99)=  1.6051778197e-01
       coeff_l3(6,100)=  2.3359764367e-02
       coeff_l3(6,101)=  3.5475182533e-01
       coeff_l3(6,102)= -1.0941084474e-01
       coeff_l3(6,103)= -1.7439691722e-01
       coeff_l3(6,104)=  2.4545010179e-02
       coeff_l3(6,105)= -3.0523091555e-01
       coeff_l3(6,106)= -7.0258036256e-02
       coeff_l3(6,107)= -1.1450450867e-01
       coeff_l3(6,108)=  1.4548368752e-01
       coeff_l3(6,109)= -8.2415826619e-02
       coeff_l3(6,110)= -1.0814575106e-01
       coeff_l3(6,111)=  1.6774839163e-01
       coeff_l3(6,112)=  1.3905501366e-01
       coeff_l3(6,113)= -1.0312731564e-01
       coeff_l3(6,114)= -5.4696041346e-01
       coeff_l3(6,115)=  4.0079411119e-02
       coeff_l3(6,116)=  5.6069320440e-01
       coeff_l3(6,117)=  5.6411303580e-02
       coeff_l3(6,118)= -1.1391234398e-01
       coeff_l3(6,119)=  9.5682218671e-02
       coeff_l3(6,120)= -1.4440864325e-01
       coeff_l3(6,121)= -6.4850562811e-01
       coeff_l3(6,122)= -7.3044106364e-02
       coeff_l3(6,123)=  6.3389822841e-02
       coeff_l3(6,124)= -1.1982502788e-01
       coeff_l3(6,125)=  8.8857784867e-02
       coeff_l3(7,1)= -1.8427751958e-01
       coeff_l3(7,2)= -5.2226525545e-01
       coeff_l3(7,3)=  5.1195514202e-01
       coeff_l3(7,4)=  2.3728978634e-01
       coeff_l3(7,5)=  3.0171719193e-01
       coeff_l3(7,6)=  6.6263809800e-02
       coeff_l3(7,7)= -1.0263846815e-01
       coeff_l3(7,8)=  4.5304962993e-01
       coeff_l3(7,9)= -8.6521148682e-02
       coeff_l3(7,10)= -1.8882668018e-01
       coeff_l3(7,11)=  4.5636719465e-01
       coeff_l3(7,12)=  2.7816012502e-01
       coeff_l3(7,13)=  6.7049883306e-02
       coeff_l3(7,14)= -4.2017373443e-01
       coeff_l3(7,15)= -1.1803895980e-01
       coeff_l3(7,16)= -6.0304030776e-03
       coeff_l3(7,17)= -3.6694398522e-01
       coeff_l3(7,18)=  7.3575282097e-01
       coeff_l3(7,19)= -7.4342526495e-02
       coeff_l3(7,20)= -2.0310202241e-01
       coeff_l3(7,21)=  1.5565793216e-01
       coeff_l3(7,22)=  2.5537261739e-02
       coeff_l3(7,23)= -1.3003939763e-02
       coeff_l3(7,24)= -7.8538542986e-01
       coeff_l3(7,25)= -4.6783512831e-01
       coeff_l3(7,26)= -1.1486129463e-01
       coeff_l3(7,27)=  9.9050648510e-02
       coeff_l3(7,28)= -3.5358175635e-01
       coeff_l3(7,29)=  5.4243439436e-01
       coeff_l3(7,30)= -3.3736693859e-01
       coeff_l3(7,31)= -4.4669041038e-01
       coeff_l3(7,32)= -5.2108705044e-01
       coeff_l3(7,33)=  5.4147332907e-02
       coeff_l3(7,34)= -1.3739372790e-01
       coeff_l3(7,35)= -3.0348283052e-01
       coeff_l3(7,36)= -2.1909388900e-01
       coeff_l3(7,37)= -4.8835626245e-01
       coeff_l3(7,38)=  1.5834225416e+00
       coeff_l3(7,39)= -2.4236606061e-01
       coeff_l3(7,40)=  2.1901930869e-01
       coeff_l3(7,41)=  1.0260111094e+00
       coeff_l3(7,42)= -3.8649401069e-01
       coeff_l3(7,43)=  5.0966136158e-02
       coeff_l3(7,44)= -4.1318351030e-01
       coeff_l3(7,45)= -1.6006705165e-01
       coeff_l3(7,46)= -6.9249862432e-01
       coeff_l3(7,47)=  2.1211189032e-01
       coeff_l3(7,48)=  2.8358998895e-01
       coeff_l3(7,49)=  2.2496935725e-01
       coeff_l3(7,50)=  1.5155954659e-01
       coeff_l3(7,51)= -3.1094774604e-01
       coeff_l3(7,52)= -3.1954663992e-01
       coeff_l3(7,53)= -3.3806633949e-01
       coeff_l3(7,54)=  3.1233910471e-02
       coeff_l3(7,55)= -3.7153914571e-02
       coeff_l3(7,56)=  3.6431219429e-02
       coeff_l3(7,57)=  2.0584693179e-02
       coeff_l3(7,58)= -3.4394290298e-02
       coeff_l3(7,59)= -7.2970725596e-02
       coeff_l3(7,60)=  6.9237715006e-01
       coeff_l3(7,61)= -6.9976918399e-02
       coeff_l3(7,62)= -2.3469386995e-01
       coeff_l3(7,63)= -2.8471803665e-01
       coeff_l3(7,64)= -1.8343740702e-01
       coeff_l3(7,65)= -1.4666803181e-01
       coeff_l3(7,66)= -4.0742057562e-01
       coeff_l3(7,67)=  9.9625712633e-01
       coeff_l3(7,68)= -1.7003051937e-01
       coeff_l3(7,69)= -4.0275841951e-01
       coeff_l3(7,70)= -5.2564680576e-01
       coeff_l3(7,71)=  6.0563945770e-01
       coeff_l3(7,72)= -5.0651974976e-02
       coeff_l3(7,73)= -1.8334309757e-01
       coeff_l3(7,74)=  4.4829404354e-01
       coeff_l3(7,75)=  4.4209133834e-02
       coeff_l3(7,76)=  8.6185835302e-02
       coeff_l3(7,77)= -1.1308641732e-01
       coeff_l3(7,78)= -2.9116678238e-01
       coeff_l3(7,79)= -1.0392872244e-01
       coeff_l3(7,80)=  9.7352251410e-02
       coeff_l3(7,81)=  4.2648696899e-01
       coeff_l3(7,82)=  1.2987802923e-01
       coeff_l3(7,83)=  3.5094776750e-01
       coeff_l3(7,84)= -2.2956382483e-02
       coeff_l3(7,85)=  1.0389640927e-01
       coeff_l3(7,86)= -6.4965587854e-01
       coeff_l3(7,87)=  1.3654674590e-01
       coeff_l3(7,88)= -9.6042886376e-02
       coeff_l3(7,89)= -7.4267435074e-01
       coeff_l3(7,90)= -9.6380777657e-02
       coeff_l3(7,91)=  2.6206213236e-01
       coeff_l3(7,92)= -8.3352670074e-02
       coeff_l3(7,93)= -3.2870993018e-01
       coeff_l3(7,94)=  3.3762624860e-01
       coeff_l3(7,95)= -3.9546352625e-01
       coeff_l3(7,96)=  3.4862987697e-02
       coeff_l3(7,97)= -1.7356240749e-01
       coeff_l3(7,98)=  3.0191415548e-01
       coeff_l3(7,99)= -3.8342231512e-01
       coeff_l3(7,100)= -1.8953150511e-01
       coeff_l3(7,101)=  3.3053526282e-01
       coeff_l3(7,102)= -1.4682579041e-01
       coeff_l3(7,103)=  1.2674081326e-01
       coeff_l3(7,104)= -1.1051877588e-01
       coeff_l3(7,105)=  7.8118330240e-01
       coeff_l3(7,106)=  1.0824848711e-01
       coeff_l3(7,107)= -6.3576084375e-01
       coeff_l3(7,108)=  1.1114235222e-01
       coeff_l3(7,109)= -4.2840889096e-01
       coeff_l3(7,110)=  2.4830661714e-01
       coeff_l3(7,111)=  3.9292957634e-02
       coeff_l3(7,112)= -4.2486414313e-01
       coeff_l3(7,113)= -4.9019992352e-01
       coeff_l3(7,114)=  1.2353318743e-02
       coeff_l3(7,115)= -1.8537998199e-02
       coeff_l3(7,116)=  4.7910946608e-01
       coeff_l3(7,117)=  2.8432050347e-01
       coeff_l3(7,118)= -9.8183989525e-02
       coeff_l3(7,119)= -1.7639082670e-01
       coeff_l3(7,120)=  7.4042752385e-02
       coeff_l3(7,121)= -1.8398383260e-01
       coeff_l3(7,122)= -6.8081371486e-02
       coeff_l3(7,123)= -3.0248108506e-01
       coeff_l3(7,124)= -4.6257179976e-01
       coeff_l3(7,125)= -1.6762316227e-01
       coeff_l3(8,1)=  6.0573625378e-03
       coeff_l3(8,2)=  2.0476672053e-01
       coeff_l3(8,3)=  1.0140129179e-01
       coeff_l3(8,4)= -3.1814672053e-02
       coeff_l3(8,5)=  1.0434960574e-01
       coeff_l3(8,6)=  1.5121760964e-01
       coeff_l3(8,7)= -7.4530206621e-02
       coeff_l3(8,8)= -1.4235733449e-01
       coeff_l3(8,9)= -4.6694751829e-02
       coeff_l3(8,10)= -2.0108765364e-01
       coeff_l3(8,11)= -6.1293523759e-02
       coeff_l3(8,12)=  1.6078156233e-01
       coeff_l3(8,13)=  3.8225916214e-03
       coeff_l3(8,14)=  1.4654548466e-01
       coeff_l3(8,15)=  2.5567617267e-02
       coeff_l3(8,16)=  2.4262173101e-02
       coeff_l3(8,17)=  1.0797011107e-01
       coeff_l3(8,18)=  3.0181961134e-02
       coeff_l3(8,19)= -2.4091536179e-02
       coeff_l3(8,20)=  1.5276981890e-01
       coeff_l3(8,21)= -1.0070330650e-01
       coeff_l3(8,22)=  1.4859728515e-01
       coeff_l3(8,23)= -1.0146004707e-01
       coeff_l3(8,24)= -2.3923966289e-01
       coeff_l3(8,25)= -1.6942040622e-01
       coeff_l3(8,26)=  1.9577790052e-02
       coeff_l3(8,27)= -1.1969686486e-02
       coeff_l3(8,28)=  1.2539611757e-01
       coeff_l3(8,29)= -1.9529566169e-01
       coeff_l3(8,30)= -7.5084954500e-02
       coeff_l3(8,31)= -1.0813779384e-01
       coeff_l3(8,32)=  8.9877516031e-02
       coeff_l3(8,33)=  1.5501715243e-02
       coeff_l3(8,34)= -2.3594839871e-01
       coeff_l3(8,35)=  8.7896503508e-02
       coeff_l3(8,36)=  1.3750523329e-01
       coeff_l3(8,37)= -1.9394839182e-02
       coeff_l3(8,38)= -9.5072105527e-02
       coeff_l3(8,39)=  9.9122993648e-02
       coeff_l3(8,40)= -3.1544134021e-02
       coeff_l3(8,41)= -1.5841510892e-01
       coeff_l3(8,42)=  4.1464522481e-02
       coeff_l3(8,43)=  2.0268771332e-03
       coeff_l3(8,44)= -3.5897031426e-01
       coeff_l3(8,45)= -3.2850891352e-02
       coeff_l3(8,46)= -2.5686463341e-02
       coeff_l3(8,47)=  9.9812023342e-02
       coeff_l3(8,48)= -6.4980804920e-02
       coeff_l3(8,49)= -3.0367445946e-01
       coeff_l3(8,50)= -7.4268721044e-02
       coeff_l3(8,51)= -5.9628903866e-02
       coeff_l3(8,52)= -9.4471335411e-02
       coeff_l3(8,53)= -1.8003877997e-01
       coeff_l3(8,54)=  1.0119770467e-01
       coeff_l3(8,55)= -5.3704779595e-02
       coeff_l3(8,56)= -5.1981553435e-02
       coeff_l3(8,57)=  7.4965521693e-02
       coeff_l3(8,58)=  5.5179141462e-02
       coeff_l3(8,59)=  1.1982730031e-01
       coeff_l3(8,60)= -4.1589266062e-01
       coeff_l3(8,61)= -1.5559685230e-01
       coeff_l3(8,62)= -1.0765819997e-01
       coeff_l3(8,63)= -1.7013485730e-01
       coeff_l3(8,64)= -1.4840580523e-02
       coeff_l3(8,65)= -1.1558102071e-01
       coeff_l3(8,66)=  2.7284495533e-02
       coeff_l3(8,67)= -3.2163289189e-01
       coeff_l3(8,68)= -1.1201395839e-01
       coeff_l3(8,69)= -7.1030571125e-03
       coeff_l3(8,70)=  2.4590276182e-02
       coeff_l3(8,71)= -1.9215497375e-01
       coeff_l3(8,72)= -2.5533005595e-01
       coeff_l3(8,73)= -8.8645704091e-02
       coeff_l3(8,74)=  1.0228244960e-01
       coeff_l3(8,75)= -2.0523831248e-02
       coeff_l3(8,76)= -2.4372860789e-01
       coeff_l3(8,77)= -6.8665772676e-02
       coeff_l3(8,78)=  5.5603720248e-02
       coeff_l3(8,79)= -7.6766237617e-02
       coeff_l3(8,80)= -6.5472088754e-02
       coeff_l3(8,81)=  6.1065919697e-02
       coeff_l3(8,82)=  1.0459899902e-01
       coeff_l3(8,83)= -1.8132913113e-01
       coeff_l3(8,84)= -1.9904850423e-01
       coeff_l3(8,85)=  8.6800441146e-02
       coeff_l3(8,86)= -5.9950280935e-02
       coeff_l3(8,87)=  1.3008040190e-01
       coeff_l3(8,88)= -2.6850718260e-01
       coeff_l3(8,89)= -8.8817812502e-02
       coeff_l3(8,90)= -1.7657294869e-01
       coeff_l3(8,91)=  1.0894736648e-01
       coeff_l3(8,92)= -3.1687431037e-02
       coeff_l3(8,93)= -1.1666668952e-01
       coeff_l3(8,94)=  2.1584132314e-01
       coeff_l3(8,95)=  5.5205319077e-02
       coeff_l3(8,96)=  4.1414231062e-02
       coeff_l3(8,97)=  2.4854833260e-02
       coeff_l3(8,98)=  1.9635812938e-01
       coeff_l3(8,99)=  1.4228920639e-01
       coeff_l3(8,100)=  1.2051177770e-01
       coeff_l3(8,101)=  3.8699325174e-02
       coeff_l3(8,102)=  6.1813522130e-02
       coeff_l3(8,103)=  7.0940844715e-02
       coeff_l3(8,104)= -2.2532438859e-02
       coeff_l3(8,105)= -4.3720069528e-01
       coeff_l3(8,106)=  6.9955028594e-02
       coeff_l3(8,107)= -1.4596438408e-01
       coeff_l3(8,108)=  1.9260963798e-01
       coeff_l3(8,109)= -1.1543337256e-01
       coeff_l3(8,110)=  7.6885201037e-02
       coeff_l3(8,111)= -3.6012127995e-02
       coeff_l3(8,112)= -1.9539020956e-01
       coeff_l3(8,113)= -6.1912201345e-02
       coeff_l3(8,114)= -4.6024113894e-02
       coeff_l3(8,115)= -4.3538533151e-02
       coeff_l3(8,116)= -3.5540130734e-01
       coeff_l3(8,117)= -2.0297664404e-01
       coeff_l3(8,118)=  2.2423307598e-01
       coeff_l3(8,119)= -1.0441384465e-01
       coeff_l3(8,120)= -4.3743206188e-03
       coeff_l3(8,121)= -1.7804001272e-01
       coeff_l3(8,122)= -6.2826149166e-02
       coeff_l3(8,123)=  1.5261578560e-01
       coeff_l3(8,124)= -2.1059289575e-02
       coeff_l3(8,125)= -9.5592901111e-02
       coeff_l3(9,1)=  2.5331941247e-01
       coeff_l3(9,2)= -2.6882458478e-02
       coeff_l3(9,3)= -1.8251173198e-01
       coeff_l3(9,4)=  1.1892205477e-01
       coeff_l3(9,5)= -1.4145354927e-01
       coeff_l3(9,6)= -2.2520139813e-01
       coeff_l3(9,7)= -1.4591002464e-01
       coeff_l3(9,8)=  1.4938108623e-01
       coeff_l3(9,9)= -1.9289149344e-01
       coeff_l3(9,10)= -2.4835918099e-02
       coeff_l3(9,11)= -1.1959634721e-01
       coeff_l3(9,12)=  6.5242695808e-01
       coeff_l3(9,13)=  5.9600657225e-01
       coeff_l3(9,14)=  1.5480865538e-01
       coeff_l3(9,15)= -3.2057437301e-01
       coeff_l3(9,16)= -3.7155601382e-01
       coeff_l3(9,17)= -1.5241916990e-03
       coeff_l3(9,18)=  6.6849911213e-01
       coeff_l3(9,19)= -1.8175171316e-01
       coeff_l3(9,20)=  6.5368287265e-02
       coeff_l3(9,21)= -2.2040529549e-01
       coeff_l3(9,22)=  1.7875470221e-01
       coeff_l3(9,23)= -5.0936692953e-01
       coeff_l3(9,24)= -3.4433799982e-01
       coeff_l3(9,25)= -1.6269011796e-01
       coeff_l3(9,26)=  5.6958943605e-03
       coeff_l3(9,27)= -4.0089905262e-02
       coeff_l3(9,28)=  1.8183571100e-01
       coeff_l3(9,29)=  6.0255730152e-01
       coeff_l3(9,30)= -1.5883752704e-01
       coeff_l3(9,31)= -4.1796001792e-01
       coeff_l3(9,32)= -4.4394277036e-02
       coeff_l3(9,33)= -4.6873070300e-02
       coeff_l3(9,34)= -9.7281269729e-02
       coeff_l3(9,35)= -1.1518441886e-01
       coeff_l3(9,36)=  4.1297271848e-02
       coeff_l3(9,37)= -4.1852653027e-02
       coeff_l3(9,38)=  6.0804426670e-01
       coeff_l3(9,39)= -1.8937570974e-02
       coeff_l3(9,40)=  1.1479484290e-01
       coeff_l3(9,41)=  3.6021757126e-01
       coeff_l3(9,42)= -4.1286703944e-01
       coeff_l3(9,43)=  3.6742079258e-01
       coeff_l3(9,44)= -4.9416702986e-01
       coeff_l3(9,45)= -9.9932461977e-02
       coeff_l3(9,46)=  6.9686971605e-02
       coeff_l3(9,47)=  1.1597336084e-01
       coeff_l3(9,48)=  2.6204875112e-01
       coeff_l3(9,49)= -5.6432551146e-01
       coeff_l3(9,50)= -7.0981546305e-03
       coeff_l3(9,51)=  1.9010221586e-02
       coeff_l3(9,52)=  1.5513802879e-02
       coeff_l3(9,53)= -1.0011455417e-01
       coeff_l3(9,54)= -1.1157404631e-01
       coeff_l3(9,55)= -2.4498657882e-01
       coeff_l3(9,56)=  1.7691139877e-01
       coeff_l3(9,57)= -1.8305498362e-01
       coeff_l3(9,58)= -1.7130693793e-01
       coeff_l3(9,59)= -2.5235710666e-02
       coeff_l3(9,60)= -1.5626382828e-01
       coeff_l3(9,61)= -1.6772210598e-01
       coeff_l3(9,62)= -1.5358664095e-01
       coeff_l3(9,63)= -2.3376888037e-01
       coeff_l3(9,64)= -1.8180333078e-01
       coeff_l3(9,65)= -2.3327529430e-01
       coeff_l3(9,66)= -2.0199425519e-02
       coeff_l3(9,67)=  5.4726015776e-02
       coeff_l3(9,68)=  1.2176226079e-01
       coeff_l3(9,69)= -1.0105531663e-02
       coeff_l3(9,70)= -8.1033088267e-02
       coeff_l3(9,71)=  6.0738646984e-01
       coeff_l3(9,72)= -4.1459670663e-01
       coeff_l3(9,73)= -1.8166959286e-01
       coeff_l3(9,74)=  4.5361358672e-02
       coeff_l3(9,75)= -2.4743700027e-01
       coeff_l3(9,76)= -4.9192970991e-01
       coeff_l3(9,77)= -3.7515014410e-01
       coeff_l3(9,78)= -1.4407117665e-01
       coeff_l3(9,79)= -8.6862258613e-02
       coeff_l3(9,80)= -1.4057117701e-01
       coeff_l3(9,81)=  1.8521863222e-01
       coeff_l3(9,82)=  1.1270204186e-01
       coeff_l3(9,83)=  7.8952133656e-02
       coeff_l3(9,84)= -2.9620230198e-01
       coeff_l3(9,85)=  1.1066348106e-01
       coeff_l3(9,86)=  2.2293107584e-02
       coeff_l3(9,87)=  1.3987083733e-01
       coeff_l3(9,88)= -3.2506637275e-02
       coeff_l3(9,89)= -6.5958879888e-02
       coeff_l3(9,90)= -5.7806247473e-01
       coeff_l3(9,91)= -7.4411630630e-02
       coeff_l3(9,92)= -1.7029261589e-01
       coeff_l3(9,93)= -1.1940439790e-01
       coeff_l3(9,94)=  2.9405292869e-01
       coeff_l3(9,95)=  9.8213642836e-02
       coeff_l3(9,96)=  1.2983663380e-01
       coeff_l3(9,97)= -2.7771906927e-02
       coeff_l3(9,98)=  2.2982375696e-02
       coeff_l3(9,99)= -8.4621287882e-02
       coeff_l3(9,100)=  2.6899585128e-01
       coeff_l3(9,101)= -8.1141024828e-02
       coeff_l3(9,102)= -6.8117986666e-04
       coeff_l3(9,103)= -1.2099010497e-01
       coeff_l3(9,104)= -1.2601777911e-01
       coeff_l3(9,105)= -1.6984511167e-02
       coeff_l3(9,106)=  1.2032773346e-02
       coeff_l3(9,107)=  9.1046862304e-02
       coeff_l3(9,108)=  2.0520381629e-01
       coeff_l3(9,109)= -1.0880582780e-01
       coeff_l3(9,110)=  6.5184794366e-03
       coeff_l3(9,111)=  3.2558068633e-01
       coeff_l3(9,112)=  1.0909610242e-01
       coeff_l3(9,113)=  1.0864427686e-01
       coeff_l3(9,114)= -8.4558837116e-02
       coeff_l3(9,115)=  4.1251283884e-01
       coeff_l3(9,116)=  2.4416354299e-01
       coeff_l3(9,117)= -4.4979322702e-03
       coeff_l3(9,118)=  3.1326749921e-01
       coeff_l3(9,119)=  1.7318145931e-01
       coeff_l3(9,120)=  3.2524424791e-01
       coeff_l3(9,121)=  1.4458276331e-01
       coeff_l3(9,122)=  5.5968839675e-02
       coeff_l3(9,123)= -2.3804815114e-01
       coeff_l3(9,124)=  6.1051107943e-02
       coeff_l3(9,125)= -9.7423292696e-02
       coeff_l3(10,1)= -3.4707108140e-01
       coeff_l3(10,2)= -7.5516268611e-02
       coeff_l3(10,3)= -4.6276688576e-02
       coeff_l3(10,4)= -1.0123456717e+00
       coeff_l3(10,5)= -3.5936722159e-01
       coeff_l3(10,6)= -1.7933990061e-01
       coeff_l3(10,7)= -8.3486008644e-01
       coeff_l3(10,8)= -1.6316250563e+00
       coeff_l3(10,9)= -2.2207276523e-01
       coeff_l3(10,10)=  3.7429061532e-01
       coeff_l3(10,11)= -6.6596075892e-02
       coeff_l3(10,12)=  3.3009627461e-01
       coeff_l3(10,13)= -8.6298055947e-02
       coeff_l3(10,14)= -2.5601553917e-01
       coeff_l3(10,15)= -1.9924052060e-01
       coeff_l3(10,16)= -1.1982905120e-01
       coeff_l3(10,17)= -1.7894113064e-01
       coeff_l3(10,18)=  1.1312096380e-02
       coeff_l3(10,19)=  2.8063325211e-02
       coeff_l3(10,20)=  1.6042891145e-01
       coeff_l3(10,21)= -5.5868484080e-02
       coeff_l3(10,22)=  2.2528065369e-02
       coeff_l3(10,23)= -2.1769614518e-01
       coeff_l3(10,24)= -1.2750767171e-01
       coeff_l3(10,25)= -6.5753501654e-01
       coeff_l3(10,26)= -3.4088205546e-02
       coeff_l3(10,27)= -4.4067630172e-01
       coeff_l3(10,28)= -1.6401301622e+00
       coeff_l3(10,29)=  6.2008589506e-01
       coeff_l3(10,30)= -9.2017287016e-01
       coeff_l3(10,31)=  1.9928140566e-02
       coeff_l3(10,32)=  1.3539706171e-01
       coeff_l3(10,33)= -3.4059712291e-01
       coeff_l3(10,34)= -7.5804204680e-03
       coeff_l3(10,35)= -5.8432334661e-01
       coeff_l3(10,36)=  7.8404797241e-03
       coeff_l3(10,37)= -1.6818839312e-01
       coeff_l3(10,38)=  1.7115171254e-01
       coeff_l3(10,39)= -2.1452704072e-01
       coeff_l3(10,40)= -3.6832308769e-01
       coeff_l3(10,41)= -5.9320080280e-01
       coeff_l3(10,42)= -1.8478976190e-01
       coeff_l3(10,43)= -1.9732013345e-01
       coeff_l3(10,44)= -4.9168348312e-01
       coeff_l3(10,45)=  2.1813106537e-01
       coeff_l3(10,46)= -4.8615318537e-01
       coeff_l3(10,47)= -1.2229530513e-01
       coeff_l3(10,48)= -2.2220167518e-01
       coeff_l3(10,49)= -4.3053059578e+00
       coeff_l3(10,50)=  4.2391553521e-02
       coeff_l3(10,51)= -2.4907301366e-01
       coeff_l3(10,52)= -1.8619046314e-03
       coeff_l3(10,53)= -8.5075038671e-01
       coeff_l3(10,54)= -2.5868158340e+00
       coeff_l3(10,55)= -7.0303320885e-01
       coeff_l3(10,56)= -4.3569624424e-01
       coeff_l3(10,57)= -2.2063647211e-01
       coeff_l3(10,58)=  1.4377346039e+00
       coeff_l3(10,59)= -6.3973858953e-02
       coeff_l3(10,60)= -4.1386117935e+00
       coeff_l3(10,61)= -9.5123223960e-02
       coeff_l3(10,62)= -3.2103207707e-01
       coeff_l3(10,63)= -1.5364412963e-01
       coeff_l3(10,64)= -5.0603091717e-01
       coeff_l3(10,65)= -1.9968326390e-01
       coeff_l3(10,66)= -2.1919915080e-01
       coeff_l3(10,67)=  5.5417793989e-01
       coeff_l3(10,68)= -4.1941964626e-01
       coeff_l3(10,69)=  1.3887491822e-01
       coeff_l3(10,70)= -1.2206599861e-01
       coeff_l3(10,71)=  1.4363220893e-02
       coeff_l3(10,72)= -1.1982928216e-01
       coeff_l3(10,73)=  1.8439324573e-02
       coeff_l3(10,74)=  2.7714982629e-01
       coeff_l3(10,75)=  2.9928520322e-01
       coeff_l3(10,76)= -3.2050230503e+00
       coeff_l3(10,77)= -1.6344082355e-01
       coeff_l3(10,78)= -5.5385578424e-02
       coeff_l3(10,79)= -4.1588049382e-02
       coeff_l3(10,80)= -1.7854732275e+00
       coeff_l3(10,81)= -1.2868408859e-01
       coeff_l3(10,82)= -4.0950110555e-01
       coeff_l3(10,83)=  1.5757197514e-02
       coeff_l3(10,84)= -4.9179822206e-01
       coeff_l3(10,85)= -1.0547886044e-01
       coeff_l3(10,86)=  1.3142900169e-01
       coeff_l3(10,87)=  3.9798490703e-02
       coeff_l3(10,88)=  8.8295295835e-02
       coeff_l3(10,89)=  1.2188390493e+00
       coeff_l3(10,90)= -3.1857064366e-01
       coeff_l3(10,91)= -3.9668396115e-01
       coeff_l3(10,92)= -2.9972779751e-01
       coeff_l3(10,93)=  1.2303033471e-01
       coeff_l3(10,94)=  5.3075021505e-01
       coeff_l3(10,95)= -1.7515012622e-01
       coeff_l3(10,96)=  1.4985945821e-01
       coeff_l3(10,97)= -3.5433098674e-01
       coeff_l3(10,98)=  1.6056749225e-01
       coeff_l3(10,99)= -1.4316898584e-01
       coeff_l3(10,100)= -9.8989069462e-02
       coeff_l3(10,101)=  6.8165397644e-01
       coeff_l3(10,102)=  2.4807621539e-01
       coeff_l3(10,103)= -2.1384215355e-01
       coeff_l3(10,104)=  1.6513894498e-01
       coeff_l3(10,105)= -2.5729952380e-02
       coeff_l3(10,106)= -2.1266171336e-01
       coeff_l3(10,107)=  8.5042335093e-02
       coeff_l3(10,108)= -1.7159061134e-01
       coeff_l3(10,109)=  6.0723844916e-02
       coeff_l3(10,110)= -1.7331489921e-01
       coeff_l3(10,111)=  2.4140663445e-01
       coeff_l3(10,112)= -5.6466645002e-01
       coeff_l3(10,113)=  6.5115898848e-02
       coeff_l3(10,114)= -1.2244201899e+00
       coeff_l3(10,115)=  3.4657875076e-03
       coeff_l3(10,116)= -1.6188738346e+00
       coeff_l3(10,117)= -2.9815578461e-01
       coeff_l3(10,118)=  7.1702994406e-02
       coeff_l3(10,119)= -1.3036471605e-01
       coeff_l3(10,120)= -1.1675707996e-01
       coeff_l3(10,121)= -6.4618259668e-02
       coeff_l3(10,122)= -1.9469845295e-01
       coeff_l3(10,123)=  5.2392728627e-02
       coeff_l3(10,124)= -5.9236131608e-02
       coeff_l3(10,125)=  2.5129449368e-01
       coeff_l3(11,1)= -3.0555346608e-01
       coeff_l3(11,2)=  6.9700635970e-02
       coeff_l3(11,3)= -2.5568562746e-01
       coeff_l3(11,4)=  8.1610426307e-02
       coeff_l3(11,5)=  5.0711113960e-02
       coeff_l3(11,6)= -3.9769444615e-02
       coeff_l3(11,7)= -2.3243519664e-01
       coeff_l3(11,8)=  1.3903882354e-02
       coeff_l3(11,9)=  7.3958456516e-02
       coeff_l3(11,10)=  4.2129512876e-02
       coeff_l3(11,11)=  4.8772621900e-02
       coeff_l3(11,12)=  2.8189229965e-01
       coeff_l3(11,13)=  4.6172644943e-02
       coeff_l3(11,14)=  1.0058687627e-01
       coeff_l3(11,15)= -1.2517683208e-01
       coeff_l3(11,16)=  9.8765872419e-02
       coeff_l3(11,17)= -1.5896986425e-01
       coeff_l3(11,18)=  3.6042499542e-01
       coeff_l3(11,19)= -1.3547730446e-01
       coeff_l3(11,20)= -8.9390985668e-02
       coeff_l3(11,21)= -3.0321683735e-02
       coeff_l3(11,22)=  1.9875423983e-02
       coeff_l3(11,23)=  5.5978726596e-03
       coeff_l3(11,24)=  1.9355881959e-02
       coeff_l3(11,25)=  5.0988931209e-02
       coeff_l3(11,26)=  3.6613952368e-02
       coeff_l3(11,27)= -2.0043468475e-01
       coeff_l3(11,28)=  1.5974491835e-01
       coeff_l3(11,29)=  3.7967747450e-01
       coeff_l3(11,30)=  7.2127960622e-02
       coeff_l3(11,31)= -7.8693434596e-02
       coeff_l3(11,32)= -1.4267876744e-01
       coeff_l3(11,33)=  7.9158283770e-02
       coeff_l3(11,34)= -7.7565602958e-02
       coeff_l3(11,35)= -9.9606625736e-02
       coeff_l3(11,36)=  5.5988216773e-03
       coeff_l3(11,37)=  4.7338385135e-02
       coeff_l3(11,38)=  7.5132369995e-02
       coeff_l3(11,39)= -6.2090910971e-02
       coeff_l3(11,40)=  9.2607261613e-03
       coeff_l3(11,41)=  9.8235481977e-01
       coeff_l3(11,42)= -1.2295373529e-01
       coeff_l3(11,43)=  3.1065491959e-02
       coeff_l3(11,44)=  1.5246255789e-03
       coeff_l3(11,45)= -3.2628294826e-01
       coeff_l3(11,46)=  1.5304918587e-01
       coeff_l3(11,47)=  2.3892235756e-01
       coeff_l3(11,48)=  7.8530527651e-02
       coeff_l3(11,49)= -5.2895635366e-01
       coeff_l3(11,50)=  3.5017323494e-01
       coeff_l3(11,51)=  4.1957534850e-02
       coeff_l3(11,52)=  8.9069426060e-02
       coeff_l3(11,53)=  4.2293950915e-02
       coeff_l3(11,54)=  8.4961913526e-02
       coeff_l3(11,55)=  6.0578271747e-02
       coeff_l3(11,56)= -2.2896820679e-02
       coeff_l3(11,57)= -7.7997997403e-02
       coeff_l3(11,58)=  7.7606379986e-02
       coeff_l3(11,59)=  4.2657977343e-01
       coeff_l3(11,60)=  2.1046285331e-01
       coeff_l3(11,61)=  1.9464567304e-01
       coeff_l3(11,62)=  5.1835902035e-02
       coeff_l3(11,63)= -4.4752189517e-01
       coeff_l3(11,64)= -1.0951181501e-01
       coeff_l3(11,65)= -1.9279922545e-01
       coeff_l3(11,66)=  1.5589465201e-01
       coeff_l3(11,67)=  5.9570991993e-01
       coeff_l3(11,68)= -1.7644170672e-02
       coeff_l3(11,69)=  8.5885427892e-02
       coeff_l3(11,70)= -3.3127989620e-02
       coeff_l3(11,71)=  2.4645063281e-01
       coeff_l3(11,72)= -1.0979886353e-01
       coeff_l3(11,73)= -9.9143959582e-02
       coeff_l3(11,74)= -6.3094235957e-02
       coeff_l3(11,75)=  3.5989150405e-02
       coeff_l3(11,76)= -5.4729849100e-01
       coeff_l3(11,77)=  2.3726880550e-02
       coeff_l3(11,78)= -2.0404806733e-01
       coeff_l3(11,79)=  1.0064346343e-01
       coeff_l3(11,80)= -5.3067833185e-01
       coeff_l3(11,81)=  2.3787504435e-01
       coeff_l3(11,82)= -8.5860222578e-02
       coeff_l3(11,83)= -8.0865463242e-03
       coeff_l3(11,84)=  1.5195886791e-01
       coeff_l3(11,85)=  6.1959981918e-02
       coeff_l3(11,86)=  1.1769214272e-01
       coeff_l3(11,87)= -9.9552692845e-03
       coeff_l3(11,88)=  2.1814672649e-01
       coeff_l3(11,89)= -7.7995918691e-03
       coeff_l3(11,90)= -4.4935524464e-02
       coeff_l3(11,91)= -9.7408942878e-02
       coeff_l3(11,92)= -1.2571837008e-01
       coeff_l3(11,93)= -2.4369451404e-01
       coeff_l3(11,94)=  1.5040315688e-02
       coeff_l3(11,95)= -7.0325478911e-02
       coeff_l3(11,96)=  3.8510400802e-02
       coeff_l3(11,97)=  6.2171570957e-02
       coeff_l3(11,98)=  7.6027624309e-02
       coeff_l3(11,99)=  2.6870974898e-01
       coeff_l3(11,100)=  1.4610236883e-01
       coeff_l3(11,101)= -1.0865830630e-01
       coeff_l3(11,102)= -1.7872078717e-01
       coeff_l3(11,103)= -1.7259593308e-01
       coeff_l3(11,104)=  5.1995221525e-02
       coeff_l3(11,105)=  2.1893826127e-01
       coeff_l3(11,106)=  1.9946855307e-01
       coeff_l3(11,107)=  5.4511222988e-02
       coeff_l3(11,108)= -8.7751790881e-02
       coeff_l3(11,109)= -4.9968179315e-02
       coeff_l3(11,110)= -3.9768166840e-02
       coeff_l3(11,111)=  7.3192626238e-02
       coeff_l3(11,112)=  4.6850603819e-01
       coeff_l3(11,113)= -4.3487753719e-03
       coeff_l3(11,114)=  1.4246217906e-01
       coeff_l3(11,115)=  1.6243970394e-01
       coeff_l3(11,116)=  5.4413020611e-01
       coeff_l3(11,117)=  2.3074540496e-01
       coeff_l3(11,118)= -8.2961872220e-02
       coeff_l3(11,119)= -4.1003406048e-02
       coeff_l3(11,120)= -4.1472099721e-02
       coeff_l3(11,121)=  4.8851776123e-01
       coeff_l3(11,122)= -7.3209449649e-02
       coeff_l3(11,123)=  8.5541203618e-02
       coeff_l3(11,124)= -4.1291922331e-02
       coeff_l3(11,125)=  7.4017360806e-02
       coeff_l3(12,1)=  2.2529780865e-02
       coeff_l3(12,2)=  1.8266876042e-01
       coeff_l3(12,3)= -1.0887818038e-01
       coeff_l3(12,4)=  6.8307787180e-02
       coeff_l3(12,5)= -6.9928877056e-02
       coeff_l3(12,6)= -1.2537425756e-01
       coeff_l3(12,7)= -1.6647806764e-01
       coeff_l3(12,8)=  1.1766749620e-01
       coeff_l3(12,9)=  1.5618839860e-01
       coeff_l3(12,10)= -5.0111971796e-02
       coeff_l3(12,11)= -6.5449122339e-03
       coeff_l3(12,12)=  1.1637637019e-01
       coeff_l3(12,13)=  8.7279453874e-02
       coeff_l3(12,14)= -1.1022669822e-01
       coeff_l3(12,15)=  3.1214938499e-03
       coeff_l3(12,16)=  5.1719978452e-02
       coeff_l3(12,17)= -1.4661839604e-01
       coeff_l3(12,18)=  5.1057118177e-01
       coeff_l3(12,19)= -4.6306826174e-02
       coeff_l3(12,20)=  2.2601850331e-02
       coeff_l3(12,21)=  1.2771241367e-01
       coeff_l3(12,22)= -2.2559743375e-02
       coeff_l3(12,23)=  2.6340353489e-01
       coeff_l3(12,24)=  1.7891278863e-01
       coeff_l3(12,25)= -1.4479635656e-01
       coeff_l3(12,26)= -1.0498634726e-01
       coeff_l3(12,27)=  1.9989955425e-01
       coeff_l3(12,28)=  3.7541050464e-02
       coeff_l3(12,29)=  2.2905594110e-01
       coeff_l3(12,30)=  1.6269303858e-02
       coeff_l3(12,31)= -6.3557289541e-02
       coeff_l3(12,32)=  9.0501479805e-02
       coeff_l3(12,33)=  1.5906238928e-02
       coeff_l3(12,34)=  2.1566298604e-01
       coeff_l3(12,35)= -1.2140928954e-01
       coeff_l3(12,36)= -1.2255016714e-01
       coeff_l3(12,37)=  1.0206278414e-01
       coeff_l3(12,38)=  4.5910668373e-01
       coeff_l3(12,39)= -1.5274545550e-01
       coeff_l3(12,40)=  8.4873419255e-03
       coeff_l3(12,41)=  3.5472950339e-01
       coeff_l3(12,42)= -1.0491679609e-01
       coeff_l3(12,43)=  2.5604739785e-02
       coeff_l3(12,44)= -1.6424782574e-02
       coeff_l3(12,45)=  8.9089846006e-04
       coeff_l3(12,46)= -3.3815819770e-02
       coeff_l3(12,47)=  1.0282828659e-01
       coeff_l3(12,48)=  1.0692973621e-02
       coeff_l3(12,49)= -3.4690517187e-01
       coeff_l3(12,50)= -2.1961623803e-02
       coeff_l3(12,51)= -8.5909135640e-02
       coeff_l3(12,52)=  1.1724628508e-01
       coeff_l3(12,53)=  5.2375558764e-02
       coeff_l3(12,54)= -6.0081098229e-02
       coeff_l3(12,55)=  1.3951475918e-01
       coeff_l3(12,56)= -1.6338415444e-01
       coeff_l3(12,57)=  1.2125325948e-01
       coeff_l3(12,58)= -1.9184956327e-02
       coeff_l3(12,59)= -6.1160486192e-02
       coeff_l3(12,60)=  8.8571876287e-02
       coeff_l3(12,61)= -9.5698060468e-03
       coeff_l3(12,62)=  1.7484609783e-01
       coeff_l3(12,63)= -2.1342250705e-01
       coeff_l3(12,64)=  1.3327053189e-01
       coeff_l3(12,65)=  1.8563713133e-01
       coeff_l3(12,66)=  8.0898568034e-02
       coeff_l3(12,67)=  2.0250695944e-01
       coeff_l3(12,68)=  3.3625143114e-03
       coeff_l3(12,69)= -9.5178864896e-02
       coeff_l3(12,70)=  4.2106434703e-02
       coeff_l3(12,71)=  3.9549940825e-01
       coeff_l3(12,72)=  2.2009626031e-02
       coeff_l3(12,73)=  1.0532811284e-01
       coeff_l3(12,74)= -3.4339766949e-02
       coeff_l3(12,75)=  1.5253357589e-01
       coeff_l3(12,76)= -2.7598193288e-01
       coeff_l3(12,77)=  7.7899560332e-02
       coeff_l3(12,78)= -6.4544887282e-03
       coeff_l3(12,79)= -2.9562192038e-02
       coeff_l3(12,80)= -3.5184884071e-01
       coeff_l3(12,81)= -4.1763599962e-02
       coeff_l3(12,82)= -7.5121834874e-02
       coeff_l3(12,83)=  1.2765878439e-01
       coeff_l3(12,84)= -8.1238575280e-02
       coeff_l3(12,85)= -1.0256691277e-01
       coeff_l3(12,86)= -9.4250254333e-02
       coeff_l3(12,87)= -1.0374265909e-01
       coeff_l3(12,88)= -1.2670743465e-01
       coeff_l3(12,89)=  7.9926326871e-02
       coeff_l3(12,90)=  9.3406759202e-02
       coeff_l3(12,91)= -1.9065028429e-01
       coeff_l3(12,92)=  3.2130405307e-02
       coeff_l3(12,93)= -1.5905034542e-01
       coeff_l3(12,94)=  2.6167547330e-03
       coeff_l3(12,95)=  2.0954091847e-01
       coeff_l3(12,96)=  1.7048473656e-01
       coeff_l3(12,97)= -1.6856443882e-01
       coeff_l3(12,98)= -1.9267106056e-01
       coeff_l3(12,99)=  1.1267722398e-01
       coeff_l3(12,100)=  2.0565076172e-01
       coeff_l3(12,101)= -4.0919966996e-02
       coeff_l3(12,102)= -9.5162950456e-02
       coeff_l3(12,103)=  1.4304617047e-01
       coeff_l3(12,104)=  7.2523146868e-02
       coeff_l3(12,105)=  2.4820722640e-01
       coeff_l3(12,106)=  1.1961077154e-01
       coeff_l3(12,107)=  6.0910489410e-02
       coeff_l3(12,108)= -2.1849870682e-02
       coeff_l3(12,109)= -1.1564373970e-01
       coeff_l3(12,110)=  1.6940876842e-01
       coeff_l3(12,111)=  1.4312517643e-01
       coeff_l3(12,112)=  1.8047808111e-01
       coeff_l3(12,113)=  1.5037810802e-01
       coeff_l3(12,114)= -1.1829779297e-01
       coeff_l3(12,115)=  5.5809314363e-03
       coeff_l3(12,116)=  2.2747509181e-01
       coeff_l3(12,117)=  1.8709647655e-01
       coeff_l3(12,118)=  3.9415001869e-02
       coeff_l3(12,119)= -3.7644464523e-02
       coeff_l3(12,120)= -6.3934758306e-02
       coeff_l3(12,121)= -2.8519406915e-01
       coeff_l3(12,122)= -3.7535935640e-02
       coeff_l3(12,123)=  1.0795747489e-01
       coeff_l3(12,124)=  1.2052947283e-01
       coeff_l3(12,125)=  4.7147624195e-02
       coeff_l3(13,1)= -5.7633894682e-01
       coeff_l3(13,2)= -9.2349633574e-02
       coeff_l3(13,3)= -2.4078057706e-01
       coeff_l3(13,4)= -1.3091968000e-01
       coeff_l3(13,5)=  1.5556722879e-01
       coeff_l3(13,6)=  3.6087680608e-02
       coeff_l3(13,7)=  1.8038025498e-01
       coeff_l3(13,8)=  8.6014896631e-02
       coeff_l3(13,9)=  2.7299749851e-01
       coeff_l3(13,10)=  9.4824291766e-02
       coeff_l3(13,11)=  3.8086503744e-02
       coeff_l3(13,12)= -5.2139264345e-01
       coeff_l3(13,13)= -3.5957494378e-01
       coeff_l3(13,14)= -1.6470989585e-01
       coeff_l3(13,15)=  2.1724811196e-01
       coeff_l3(13,16)=  1.2051725388e-01
       coeff_l3(13,17)= -3.1393358111e-01
       coeff_l3(13,18)= -1.1198073626e+00
       coeff_l3(13,19)=  6.7637622356e-02
       coeff_l3(13,20)= -2.2949852049e-01
       coeff_l3(13,21)= -2.8719879687e-02
       coeff_l3(13,22)= -2.2862782702e-02
       coeff_l3(13,23)=  2.6885092258e-01
       coeff_l3(13,24)=  3.1103301048e-01
       coeff_l3(13,25)=  7.1655742824e-02
       coeff_l3(13,26)=  6.0100056231e-02
       coeff_l3(13,27)= -2.2644695640e-01
       coeff_l3(13,28)=  6.1979956925e-02
       coeff_l3(13,29)= -7.3626023531e-01
       coeff_l3(13,30)=  5.8029824495e-01
       coeff_l3(13,31)=  1.3628056645e-01
       coeff_l3(13,32)=  2.2189723328e-02
       coeff_l3(13,33)= -6.6108889878e-03
       coeff_l3(13,34)= -9.7755186260e-02
       coeff_l3(13,35)= -4.3656277657e-01
       coeff_l3(13,36)= -8.4921918809e-02
       coeff_l3(13,37)= -1.7814492807e-02
       coeff_l3(13,38)= -6.9799727201e-01
       coeff_l3(13,39)=  5.9108518064e-02
       coeff_l3(13,40)= -6.9872230291e-02
       coeff_l3(13,41)= -1.1532849818e-01
       coeff_l3(13,42)=  1.9503301382e-01
       coeff_l3(13,43)=  2.2374037653e-02
       coeff_l3(13,44)=  3.1348037720e-01
       coeff_l3(13,45)=  3.1946450472e-01
       coeff_l3(13,46)= -1.8275476992e-01
       coeff_l3(13,47)=  5.2036393434e-02
       coeff_l3(13,48)= -2.8968575597e-01
       coeff_l3(13,49)=  2.8064700961e-01
       coeff_l3(13,50)= -3.3797436953e-01
       coeff_l3(13,51)= -3.7439909577e-01
       coeff_l3(13,52)= -3.3017082606e-03
       coeff_l3(13,53)= -1.3256752491e-01
       coeff_l3(13,54)=  4.7981813550e-01
       coeff_l3(13,55)=  6.7175272852e-03
       coeff_l3(13,56)= -4.1491720825e-02
       coeff_l3(13,57)=  1.2578281760e-01
       coeff_l3(13,58)=  1.1762953997e+00
       coeff_l3(13,59)= -1.3190704584e-01
       coeff_l3(13,60)= -7.9350310564e-01
       coeff_l3(13,61)=  1.7834785581e-01
       coeff_l3(13,62)=  9.8419897258e-02
       coeff_l3(13,63)=  4.3990221620e-01
       coeff_l3(13,64)= -2.6013035327e-02
       coeff_l3(13,65)=  3.3571851254e-01
       coeff_l3(13,66)= -2.5063104928e-02
       coeff_l3(13,67)= -4.6963065863e-01
       coeff_l3(13,68)= -1.1066220701e-01
       coeff_l3(13,69)= -1.4962719753e-02
       coeff_l3(13,70)=  1.0297564417e-01
       coeff_l3(13,71)= -1.0945246220e+00
       coeff_l3(13,72)=  1.6266578436e-01
       coeff_l3(13,73)=  2.0117585361e-01
       coeff_l3(13,74)= -3.7486886978e-01
       coeff_l3(13,75)=  2.6952129602e-01
       coeff_l3(13,76)=  1.6327749193e-01
       coeff_l3(13,77)=  2.1389666200e-01
       coeff_l3(13,78)= -4.6249959618e-02
       coeff_l3(13,79)=  1.0530747473e-01
       coeff_l3(13,80)=  5.5021804571e-01
       coeff_l3(13,81)= -3.1078979373e-01
       coeff_l3(13,82)= -1.0910486430e-01
       coeff_l3(13,83)= -1.8014955521e-01
       coeff_l3(13,84)= -2.3420625925e-01
       coeff_l3(13,85)=  7.7778466046e-02
       coeff_l3(13,86)=  1.6325870156e-01
       coeff_l3(13,87)= -2.7204796672e-01
       coeff_l3(13,88)= -4.3455672264e-01
       coeff_l3(13,89)=  6.0991221666e-01
       coeff_l3(13,90)=  1.4669694006e-01
       coeff_l3(13,91)= -1.6351555288e-01
       coeff_l3(13,92)= -1.3411366940e-01
       coeff_l3(13,93)=  1.4092515409e-01
       coeff_l3(13,94)= -2.9472693801e-01
       coeff_l3(13,95)= -3.8084807992e-01
       coeff_l3(13,96)= -1.9034947455e-01
       coeff_l3(13,97)= -2.1836027503e-01
       coeff_l3(13,98)=  1.6119214892e-01
       coeff_l3(13,99)= -4.2621579766e-01
       coeff_l3(13,100)= -3.9574339986e-01
       coeff_l3(13,101)= -1.2635774910e-01
       coeff_l3(13,102)= -1.2374970131e-02
       coeff_l3(13,103)= -7.4612624943e-02
       coeff_l3(13,104)=  2.2181741893e-01
       coeff_l3(13,105)= -2.3439241946e-01
       coeff_l3(13,106)=  1.2207878381e-01
       coeff_l3(13,107)= -2.7927166224e-01
       coeff_l3(13,108)= -4.4836521149e-01
       coeff_l3(13,109)=  2.3473069072e-01
       coeff_l3(13,110)=  1.8254209310e-02
       coeff_l3(13,111)= -3.7674245238e-01
       coeff_l3(13,112)= -7.0692402124e-01
       coeff_l3(13,113)= -6.7720389366e-01
       coeff_l3(13,114)=  1.6786770523e-01
       coeff_l3(13,115)= -3.9868551493e-01
       coeff_l3(13,116)= -1.0886365175e+00
       coeff_l3(13,117)= -1.6278105974e-01
       coeff_l3(13,118)=  1.2706451118e-01
       coeff_l3(13,119)=  1.4321394265e-01
       coeff_l3(13,120)= -1.5732957423e-01
       coeff_l3(13,121)= -3.9770373702e-01
       coeff_l3(13,122)=  9.4316238537e-03
       coeff_l3(13,123)= -4.5805234462e-02
       coeff_l3(13,124)=  5.1376555115e-02
       coeff_l3(13,125)=  1.6484111547e-02
       coeff_l3(14,1)= -5.2211421728e-01
       coeff_l3(14,2)= -4.7172489576e-03
       coeff_l3(14,3)= -3.0068606138e-01
       coeff_l3(14,4)=  1.4626215398e-01
       coeff_l3(14,5)= -1.5869289637e-02
       coeff_l3(14,6)= -5.7965658605e-02
       coeff_l3(14,7)= -1.3316075504e-01
       coeff_l3(14,8)= -6.2770582736e-04
       coeff_l3(14,9)= -1.5497882664e-01
       coeff_l3(14,10)=  1.3404536247e-01
       coeff_l3(14,11)=  1.8436260521e-01
       coeff_l3(14,12)=  2.1962127090e-01
       coeff_l3(14,13)=  2.7331796288e-01
       coeff_l3(14,14)= -3.2481083274e-01
       coeff_l3(14,15)= -1.8830533326e-01
       coeff_l3(14,16)= -2.2634077072e-01
       coeff_l3(14,17)= -7.6756738126e-02
       coeff_l3(14,18)=  2.3630525172e-01
       coeff_l3(14,19)= -1.2997822464e-01
       coeff_l3(14,20)= -1.1309353262e-01
       coeff_l3(14,21)= -4.1217762232e-01
       coeff_l3(14,22)=  8.9242659509e-02
       coeff_l3(14,23)= -6.2180221081e-02
       coeff_l3(14,24)= -1.2525662780e-01
       coeff_l3(14,25)=  9.6939682961e-02
       coeff_l3(14,26)= -2.3112924397e-01
       coeff_l3(14,27)= -2.7611395717e-01
       coeff_l3(14,28)=  8.9316383004e-02
       coeff_l3(14,29)=  3.6348387599e-01
       coeff_l3(14,30)= -2.4369770288e-01
       coeff_l3(14,31)= -1.5319445729e-01
       coeff_l3(14,32)=  6.7793200724e-03
       coeff_l3(14,33)=  8.4583073854e-02
       coeff_l3(14,34)= -2.2940188646e-01
       coeff_l3(14,35)= -1.3387760520e-01
       coeff_l3(14,36)=  2.7050152421e-01
       coeff_l3(14,37)=  8.0928085372e-03
       coeff_l3(14,38)=  6.4936870337e-01
       coeff_l3(14,39)=  1.1139721610e-03
       coeff_l3(14,40)=  3.6053322256e-02
       coeff_l3(14,41)=  6.5850967169e-01
       coeff_l3(14,42)= -2.5244629383e-01
       coeff_l3(14,43)=  1.2934158742e-01
       coeff_l3(14,44)= -1.4913557470e-01
       coeff_l3(14,45)= -1.5763209760e-01
       coeff_l3(14,46)= -5.8534782380e-02
       coeff_l3(14,47)= -6.5659567714e-02
       coeff_l3(14,48)= -4.0831919760e-02
       coeff_l3(14,49)= -7.3998779058e-01
       coeff_l3(14,50)= -1.3017220795e-01
       coeff_l3(14,51)=  2.5314025581e-02
       coeff_l3(14,52)=  8.5950642824e-02
       coeff_l3(14,53)=  6.6851161420e-02
       coeff_l3(14,54)= -2.6936626434e-01
       coeff_l3(14,55)=  2.2341124713e-02
       coeff_l3(14,56)=  1.2058251351e-01
       coeff_l3(14,57)= -2.5408353657e-02
       coeff_l3(14,58)= -1.4770291746e-01
       coeff_l3(14,59)=  1.6406102479e-01
       coeff_l3(14,60)= -1.7036868632e-01
       coeff_l3(14,61)= -1.5531139076e-01
       coeff_l3(14,62)= -3.5247094929e-02
       coeff_l3(14,63)= -4.4240561128e-01
       coeff_l3(14,64)=  1.4160741866e-01
       coeff_l3(14,65)= -2.5080727413e-02
       coeff_l3(14,66)= -1.2007026374e-01
       coeff_l3(14,67)=  2.9994058609e-01
       coeff_l3(14,68)= -1.6421158612e-01
       coeff_l3(14,69)= -2.4817237258e-01
       coeff_l3(14,70)= -1.2829685211e-01
       coeff_l3(14,71)=  3.8123661280e-01
       coeff_l3(14,72)= -1.2861122191e-01
       coeff_l3(14,73)=  7.1106679738e-02
       coeff_l3(14,74)= -3.6048434675e-02
       coeff_l3(14,75)=  1.0430177301e-01
       coeff_l3(14,76)= -8.4152901173e-01
       coeff_l3(14,77)= -3.8135204464e-02
       coeff_l3(14,78)= -5.8866608888e-02
       coeff_l3(14,79)= -1.8030659854e-01
       coeff_l3(14,80)= -4.5949563384e-01
       coeff_l3(14,81)=  1.3173629344e-01
       coeff_l3(14,82)=  3.2647017390e-02
       coeff_l3(14,83)=  1.6819616780e-02
       coeff_l3(14,84)= -7.6144941151e-02
       coeff_l3(14,85)=  3.4848493338e-01
       coeff_l3(14,86)=  1.2918178737e-01
       coeff_l3(14,87)=  6.7675575614e-02
       coeff_l3(14,88)= -1.2328978628e-01
       coeff_l3(14,89)=  1.8165020272e-02
       coeff_l3(14,90)= -9.1834235936e-03
       coeff_l3(14,91)=  1.1697866023e-01
       coeff_l3(14,92)=  5.7794701308e-02
       coeff_l3(14,93)= -2.1237312257e-01
       coeff_l3(14,94)=  2.8271113988e-03
       coeff_l3(14,95)=  1.7560192943e-01
       coeff_l3(14,96)=  2.8993904591e-01
       coeff_l3(14,97)=  1.9753368199e-01
       coeff_l3(14,98)= -6.6611744463e-02
       coeff_l3(14,99)= -5.2252437919e-02
       coeff_l3(14,100)=  1.3893032074e-01
       coeff_l3(14,101)= -3.2231232524e-01
       coeff_l3(14,102)= -1.5946699679e-01
       coeff_l3(14,103)=  6.5390847623e-02
       coeff_l3(14,104)= -1.0070717335e-01
       coeff_l3(14,105)=  5.3607773781e-01
       coeff_l3(14,106)=  8.2801423967e-02
       coeff_l3(14,107)= -2.0698948205e-01
       coeff_l3(14,108)= -2.1376943216e-02
       coeff_l3(14,109)= -1.6442666948e-01
       coeff_l3(14,110)= -1.7148863524e-02
       coeff_l3(14,111)=  1.2551465631e-01
       coeff_l3(14,112)= -2.5227680802e-01
       coeff_l3(14,113)= -6.5181061625e-02
       coeff_l3(14,114)=  1.0561051220e-01
       coeff_l3(14,115)=  1.6894267872e-02
       coeff_l3(14,116)=  1.0100974888e-01
       coeff_l3(14,117)=  6.4520232379e-02
       coeff_l3(14,118)=  1.4406932890e-01
       coeff_l3(14,119)=  1.5799211338e-02
       coeff_l3(14,120)=  1.8047260121e-02
       coeff_l3(14,121)=  1.2695783377e-01
       coeff_l3(14,122)= -1.4929631352e-01
       coeff_l3(14,123)= -1.3915915973e-02
       coeff_l3(14,124)= -7.3404818773e-02
       coeff_l3(14,125)=  4.3051969260e-02
       coeff_l3(15,1)=  2.9413917661e-01
       coeff_l3(15,2)= -1.4960264787e-02
       coeff_l3(15,3)=  4.2489238083e-02
       coeff_l3(15,4)=  3.8009953499e-01
       coeff_l3(15,5)=  5.2693929523e-02
       coeff_l3(15,6)= -1.3090786338e-01
       coeff_l3(15,7)= -3.3828732371e-01
       coeff_l3(15,8)=  2.8515380621e-01
       coeff_l3(15,9)= -1.8071810901e-01
       coeff_l3(15,10)= -1.2823261321e-01
       coeff_l3(15,11)=  1.7390827835e-01
       coeff_l3(15,12)=  7.9754096270e-01
       coeff_l3(15,13)=  6.8058627844e-01
       coeff_l3(15,14)=  5.0771734677e-03
       coeff_l3(15,15)= -2.3288448155e-01
       coeff_l3(15,16)= -3.0111381412e-01
       coeff_l3(15,17)=  2.3680338264e-01
       coeff_l3(15,18)=  6.1080747843e-01
       coeff_l3(15,19)= -1.9963656366e-01
       coeff_l3(15,20)=  3.4903982282e-01
       coeff_l3(15,21)= -5.0524264574e-01
       coeff_l3(15,22)= -2.6029901579e-02
       coeff_l3(15,23)= -5.5226778984e-01
       coeff_l3(15,24)= -4.6382072568e-01
       coeff_l3(15,25)=  3.3316094428e-02
       coeff_l3(15,26)= -2.0316796005e-01
       coeff_l3(15,27)= -1.4402376115e-01
       coeff_l3(15,28)=  2.4249134585e-03
       coeff_l3(15,29)=  9.3598651886e-01
       coeff_l3(15,30)= -2.0850127563e-02
       coeff_l3(15,31)= -2.3435400426e-01
       coeff_l3(15,32)= -2.6445159316e-01
       coeff_l3(15,33)=  2.1199591458e-02
       coeff_l3(15,34)= -2.1839337051e-01
       coeff_l3(15,35)= -6.1519369483e-02
       coeff_l3(15,36)= -1.3914197683e-01
       coeff_l3(15,37)= -3.8490936160e-02
       coeff_l3(15,38)=  9.1518235207e-01
       coeff_l3(15,39)= -4.3986033648e-02
       coeff_l3(15,40)=  3.7317231297e-02
       coeff_l3(15,41)=  6.2071138620e-01
       coeff_l3(15,42)= -3.8821488619e-01
       coeff_l3(15,43)=  1.3876730204e-01
       coeff_l3(15,44)= -5.5467891693e-01
       coeff_l3(15,45)=  1.5840403736e-01
       coeff_l3(15,46)=  1.5114264190e-01
       coeff_l3(15,47)= -1.7400442064e-01
       coeff_l3(15,48)=  2.3674342036e-01
       coeff_l3(15,49)= -6.4209043980e-01
       coeff_l3(15,50)=  2.3874215782e-02
       coeff_l3(15,51)=  4.4476997107e-02
       coeff_l3(15,52)= -5.4286628962e-02
       coeff_l3(15,53)= -1.1898998171e-01
       coeff_l3(15,54)= -7.6001256704e-02
       coeff_l3(15,55)= -8.1917241216e-02
       coeff_l3(15,56)=  2.7045598626e-01
       coeff_l3(15,57)=  1.6086732969e-02
       coeff_l3(15,58)= -3.4836888313e-01
       coeff_l3(15,59)= -1.0401789844e-01
       coeff_l3(15,60)= -7.4555270374e-02
       coeff_l3(15,61)= -4.3768066168e-01
       coeff_l3(15,62)= -3.6172065139e-01
       coeff_l3(15,63)= -2.2393700480e-01
       coeff_l3(15,64)= -8.9000344276e-02
       coeff_l3(15,65)= -2.3556567729e-01
       coeff_l3(15,66)= -6.5461168997e-03
       coeff_l3(15,67)=  8.3932332695e-02
       coeff_l3(15,68)=  9.4557483681e-04
       coeff_l3(15,69)=  4.7003347427e-02
       coeff_l3(15,70)= -2.9899987578e-01
       coeff_l3(15,71)=  5.1561820507e-01
       coeff_l3(15,72)= -3.1029722095e-01
       coeff_l3(15,73)= -1.0868194699e-01
       coeff_l3(15,74)=  2.1427890658e-01
       coeff_l3(15,75)= -4.0060019493e-01
       coeff_l3(15,76)= -4.3570408225e-01
       coeff_l3(15,77)= -6.3133877516e-01
       coeff_l3(15,78)=  8.2428634167e-02
       coeff_l3(15,79)= -2.7029973269e-01
       coeff_l3(15,80)= -5.5014681816e-01
       coeff_l3(15,81)=  2.0016624033e-01
       coeff_l3(15,82)=  1.2147349119e-01
       coeff_l3(15,83)= -1.9997432828e-03
       coeff_l3(15,84)= -1.0386613011e-01
       coeff_l3(15,85)= -2.8644114733e-01
       coeff_l3(15,86)= -4.7363243997e-02
       coeff_l3(15,87)=  1.4912442863e-01
       coeff_l3(15,88)= -6.0026305914e-01
       coeff_l3(15,89)= -2.5832056999e-02
       coeff_l3(15,90)= -7.5433826447e-01
       coeff_l3(15,91)= -3.0044081807e-01
       coeff_l3(15,92)=  6.4954072237e-02
       coeff_l3(15,93)=  8.8310994208e-02
       coeff_l3(15,94)=  3.8840451837e-01
       coeff_l3(15,95)=  3.7056231499e-01
       coeff_l3(15,96)=  1.4766480029e-01
       coeff_l3(15,97)=  2.0194961131e-01
       coeff_l3(15,98)= -1.6353191435e-01
       coeff_l3(15,99)= -2.0799224079e-01
       coeff_l3(15,100)=  2.9248860478e-01
       coeff_l3(15,101)= -2.1954268217e-01
       coeff_l3(15,102)=  1.6227751970e-02
       coeff_l3(15,103)= -2.1139362454e-01
       coeff_l3(15,104)= -1.1093632877e-01
       coeff_l3(15,105)=  3.0765888095e-01
       coeff_l3(15,106)=  2.0927710831e-01
       coeff_l3(15,107)=  1.4897394180e-01
       coeff_l3(15,108)=  3.9566797018e-01
       coeff_l3(15,109)= -2.6190850139e-01
       coeff_l3(15,110)=  1.2311099470e-01
       coeff_l3(15,111)=  4.1082945466e-01
       coeff_l3(15,112)=  2.5895413756e-01
       coeff_l3(15,113)=  2.8353318572e-01
       coeff_l3(15,114)=  1.6230417788e-01
       coeff_l3(15,115)=  4.4160535932e-01
       coeff_l3(15,116)= -7.7988252044e-02
       coeff_l3(15,117)= -1.4594216831e-02
       coeff_l3(15,118)=  6.3072884083e-01
       coeff_l3(15,119)= -6.2820918858e-02
       coeff_l3(15,120)=  1.5438383818e-01
       coeff_l3(15,121)= -3.5681929439e-02
       coeff_l3(15,122)=  2.3377741873e-01
       coeff_l3(15,123)= -2.2872872651e-01
       coeff_l3(15,124)= -2.7601030469e-01
       coeff_l3(15,125)= -5.8954734355e-02
       coeff_l3(16,1)= -3.6256775260e-01
       coeff_l3(16,2)=  1.7824299634e-01
       coeff_l3(16,3)= -8.1494808197e-02
       coeff_l3(16,4)= -1.9708551466e-01
       coeff_l3(16,5)=  1.9018778205e-01
       coeff_l3(16,6)= -7.4497975409e-02
       coeff_l3(16,7)= -8.4655219316e-01
       coeff_l3(16,8)=  1.0327620804e-01
       coeff_l3(16,9)= -1.1593127623e-02
       coeff_l3(16,10)= -2.6985335350e-01
       coeff_l3(16,11)= -4.0018986911e-02
       coeff_l3(16,12)= -1.9079631567e-01
       coeff_l3(16,13)=  1.1239285767e-01
       coeff_l3(16,14)=  4.4844742864e-02
       coeff_l3(16,15)= -1.1792008579e-01
       coeff_l3(16,16)= -9.5589570701e-03
       coeff_l3(16,17)= -1.8173556775e-02
       coeff_l3(16,18)= -8.3764582872e-02
       coeff_l3(16,19)= -1.9878920913e-01
       coeff_l3(16,20)= -9.0561687946e-02
       coeff_l3(16,21)= -2.4853090942e-01
       coeff_l3(16,22)= -1.6739776731e-01
       coeff_l3(16,23)= -1.0052122176e-01
       coeff_l3(16,24)= -1.0628692806e-01
       coeff_l3(16,25)= -1.5989924967e-01
       coeff_l3(16,26)= -8.3512797952e-02
       coeff_l3(16,27)= -1.3544740677e+00
       coeff_l3(16,28)= -8.1261694431e-01
       coeff_l3(16,29)= -4.1790044308e-01
       coeff_l3(16,30)= -2.0305781066e-01
       coeff_l3(16,31)= -1.3793757558e-01
       coeff_l3(16,32)= -5.0087690353e-02
       coeff_l3(16,33)= -1.5043166280e-01
       coeff_l3(16,34)=  2.5003230572e-01
       coeff_l3(16,35)= -1.8008473516e-01
       coeff_l3(16,36)=  4.4290602207e-02
       coeff_l3(16,37)= -5.0184264779e-02
       coeff_l3(16,38)=  6.0383662581e-02
       coeff_l3(16,39)=  8.2129366696e-02
       coeff_l3(16,40)= -3.0454773456e-02
       coeff_l3(16,41)= -1.4687153697e-01
       coeff_l3(16,42)= -1.5169265866e-01
       coeff_l3(16,43)= -7.5793907046e-02
       coeff_l3(16,44)=  2.3801192641e-02
       coeff_l3(16,45)= -5.9490241110e-02
       coeff_l3(16,46)= -1.0970366001e-01
       coeff_l3(16,47)= -1.3407747447e-01
       coeff_l3(16,48)= -1.1579374969e-01
       coeff_l3(16,49)= -5.6366765499e-01
       coeff_l3(16,50)= -1.8659902737e-02
       coeff_l3(16,51)=  3.3280450851e-02
       coeff_l3(16,52)=  7.4912674725e-02
       coeff_l3(16,53)=  1.0243093967e-01
       coeff_l3(16,54)= -1.1304524541e-01
       coeff_l3(16,55)=  8.5208542645e-02
       coeff_l3(16,56)=  1.0442698747e-01
       coeff_l3(16,57)= -1.0428562760e-01
       coeff_l3(16,58)= -3.3981695771e-01
       coeff_l3(16,59)= -4.9621049315e-02
       coeff_l3(16,60)=  5.3248208761e-01
       coeff_l3(16,61)=  9.7942501307e-02
       coeff_l3(16,62)= -2.1489854157e-01
       coeff_l3(16,63)= -1.2551154196e-01
       coeff_l3(16,64)=  8.1197142601e-02
       coeff_l3(16,65)= -1.3454274833e-01
       coeff_l3(16,66)= -1.4710937440e-01
       coeff_l3(16,67)= -1.2764431536e-01
       coeff_l3(16,68)= -9.7338050604e-02
       coeff_l3(16,69)= -1.8292762339e-01
       coeff_l3(16,70)= -2.3866329342e-02
       coeff_l3(16,71)= -6.6271759570e-02
       coeff_l3(16,72)= -1.7339397967e-01
       coeff_l3(16,73)= -1.3872779906e-02
       coeff_l3(16,74)= -3.2025587559e-01
       coeff_l3(16,75)=  2.2750096023e-01
       coeff_l3(16,76)= -1.3454802334e-01
       coeff_l3(16,77)=  1.1285617016e-02
       coeff_l3(16,78)= -5.8570165187e-02
       coeff_l3(16,79)= -6.7125365138e-02
       coeff_l3(16,80)= -3.2188680768e-01
       coeff_l3(16,81)=  1.9226829708e-01
       coeff_l3(16,82)=  8.2243010402e-02
       coeff_l3(16,83)= -2.7998089790e-01
       coeff_l3(16,84)= -2.4082566798e-01
       coeff_l3(16,85)= -9.6439868212e-02
       coeff_l3(16,86)= -3.4836905543e-03
       coeff_l3(16,87)= -2.8813200071e-02
       coeff_l3(16,88)= -3.5950890183e-01
       coeff_l3(16,89)= -1.2145201117e-01
       coeff_l3(16,90)= -2.7533665299e-01
       coeff_l3(16,91)= -4.6114271879e-01
       coeff_l3(16,92)= -2.7651159093e-02
       coeff_l3(16,93)= -6.1349667609e-02
       coeff_l3(16,94)= -1.7285430431e-01
       coeff_l3(16,95)=  7.8426741064e-02
       coeff_l3(16,96)= -3.2100759447e-02
       coeff_l3(16,97)=  7.6833404601e-02
       coeff_l3(16,98)= -3.3059909940e-02
       coeff_l3(16,99)=  1.0504159145e-03
       coeff_l3(16,100)= -3.9067175239e-02
       coeff_l3(16,101)= -1.1627859622e-01
       coeff_l3(16,102)=  1.0221674293e-01
       coeff_l3(16,103)=  3.9603978395e-02
       coeff_l3(16,104)=  8.2499049604e-02
       coeff_l3(16,105)= -1.6942001879e-01
       coeff_l3(16,106)=  3.0192036182e-02
       coeff_l3(16,107)= -1.2167543173e-01
       coeff_l3(16,108)=  1.3080629520e-02
       coeff_l3(16,109)=  1.5982531011e-01
       coeff_l3(16,110)=  1.5052147210e-02
       coeff_l3(16,111)= -6.7904628813e-02
       coeff_l3(16,112)= -2.9121637344e-01
       coeff_l3(16,113)= -2.2821766138e-01
       coeff_l3(16,114)=  1.5394185483e-01
       coeff_l3(16,115)= -1.2750487030e-01
       coeff_l3(16,116)=  1.5451525152e-01
       coeff_l3(16,117)= -1.6769923270e-01
       coeff_l3(16,118)= -2.0229932666e-01
       coeff_l3(16,119)= -3.9465643466e-02
       coeff_l3(16,120)= -8.7878264487e-02
       coeff_l3(16,121)= -4.0850050747e-02
       coeff_l3(16,122)=  1.7847505212e-01
       coeff_l3(16,123)= -4.2814962566e-02
       coeff_l3(16,124)=  3.2215707004e-02
       coeff_l3(16,125)=  7.1064040065e-02
       coeff_l3(17,1)= -2.0094236359e-02
       coeff_l3(17,2)= -1.1935718358e-01
       coeff_l3(17,3)=  2.4027081206e-02
       coeff_l3(17,4)= -1.1545824260e-01
       coeff_l3(17,5)= -3.5792998970e-02
       coeff_l3(17,6)= -1.6412131488e-02
       coeff_l3(17,7)=  8.4715835750e-02
       coeff_l3(17,8)= -7.6680064201e-02
       coeff_l3(17,9)= -5.0725843757e-02
       coeff_l3(17,10)=  2.5144802406e-02
       coeff_l3(17,11)=  1.2188243121e-01
       coeff_l3(17,12)= -3.4283131361e-02
       coeff_l3(17,13)=  4.4599443674e-02
       coeff_l3(17,14)=  1.4883340895e-01
       coeff_l3(17,15)= -6.4330743626e-03
       coeff_l3(17,16)=  9.7502246499e-02
       coeff_l3(17,17)= -8.4041900933e-02
       coeff_l3(17,18)=  1.2868052721e-01
       coeff_l3(17,19)= -7.2339713573e-02
       coeff_l3(17,20)=  3.3328600228e-02
       coeff_l3(17,21)=  2.1285280585e-02
       coeff_l3(17,22)=  1.3989819586e-01
       coeff_l3(17,23)=  1.6523140669e-01
       coeff_l3(17,24)= -1.1289184541e-01
       coeff_l3(17,25)= -6.5099284053e-02
       coeff_l3(17,26)= -1.3756982982e-01
       coeff_l3(17,27)=  1.8945068121e-01
       coeff_l3(17,28)=  2.4502830207e-01
       coeff_l3(17,29)= -1.0803005099e-01
       coeff_l3(17,30)= -4.7086436301e-02
       coeff_l3(17,31)= -1.3649238646e-01
       coeff_l3(17,32)= -1.4919078350e-01
       coeff_l3(17,33)=  1.0255552828e-01
       coeff_l3(17,34)=  7.0131994784e-02
       coeff_l3(17,35)= -6.1896285042e-03
       coeff_l3(17,36)= -1.0946750641e-01
       coeff_l3(17,37)= -2.1304035559e-02
       coeff_l3(17,38)=  2.5701540709e-01
       coeff_l3(17,39)=  3.3386234194e-02
       coeff_l3(17,40)=  1.7418859527e-02
       coeff_l3(17,41)=  8.7949909270e-02
       coeff_l3(17,42)=  8.8484985754e-03
       coeff_l3(17,43)=  1.7694631219e-01
       coeff_l3(17,44)= -8.3916783333e-02
       coeff_l3(17,45)=  1.5057595074e-01
       coeff_l3(17,46)= -2.5151653215e-02
       coeff_l3(17,47)=  9.7304217517e-02
       coeff_l3(17,48)= -1.9324349239e-02
       coeff_l3(17,49)= -4.5827761292e-01
       coeff_l3(17,50)= -2.0118483901e-01
       coeff_l3(17,51)= -1.4046205580e-01
       coeff_l3(17,52)= -1.6352966428e-01
       coeff_l3(17,53)= -1.3315962255e-01
       coeff_l3(17,54)=  1.2723773718e-01
       coeff_l3(17,55)= -6.0111440718e-02
       coeff_l3(17,56)=  1.0829335451e-01
       coeff_l3(17,57)= -6.1662822962e-02
       coeff_l3(17,58)= -2.2040730715e-01
       coeff_l3(17,59)= -1.4700870216e-01
       coeff_l3(17,60)= -4.3394288421e-01
       coeff_l3(17,61)= -1.2479423732e-01
       coeff_l3(17,62)= -7.0395305753e-02
       coeff_l3(17,63)= -2.3689040542e-01
       coeff_l3(17,64)= -5.8476470411e-02
       coeff_l3(17,65)= -4.8985064030e-02
       coeff_l3(17,66)=  1.2608648837e-01
       coeff_l3(17,67)= -5.1155969501e-02
       coeff_l3(17,68)= -1.3295091689e-01
       coeff_l3(17,69)= -2.4615004659e-01
       coeff_l3(17,70)= -7.2354018688e-02
       coeff_l3(17,71)= -9.7563140094e-02
       coeff_l3(17,72)=  6.3951358199e-02
       coeff_l3(17,73)=  6.2955550849e-02
       coeff_l3(17,74)= -1.6967708245e-02
       coeff_l3(17,75)= -4.9619484693e-02
       coeff_l3(17,76)= -3.4206575155e-01
       coeff_l3(17,77)=  4.7544039786e-02
       coeff_l3(17,78)= -1.5928941965e-01
       coeff_l3(17,79)=  5.2033357322e-02
       coeff_l3(17,80)= -4.1187581420e-01
       coeff_l3(17,81)= -2.2792531550e-01
       coeff_l3(17,82)= -1.2772604823e-01
       coeff_l3(17,83)=  3.4147400409e-02
       coeff_l3(17,84)= -1.9973288476e-01
       coeff_l3(17,85)= -1.9451518357e-01
       coeff_l3(17,86)=  8.3140701056e-02
       coeff_l3(17,87)= -2.0694354177e-01
       coeff_l3(17,88)= -1.7202937603e-01
       coeff_l3(17,89)= -9.0888269246e-02
       coeff_l3(17,90)=  7.6036840677e-02
       coeff_l3(17,91)=  2.1886620671e-02
       coeff_l3(17,92)= -8.0249071121e-02
       coeff_l3(17,93)= -6.6123805940e-02
       coeff_l3(17,94)=  2.9900047928e-02
       coeff_l3(17,95)=  2.0023630559e-01
       coeff_l3(17,96)=  1.2757967412e-01
       coeff_l3(17,97)= -4.7441501170e-02
       coeff_l3(17,98)=  1.0697885603e-01
       coeff_l3(17,99)= -1.0470885038e-01
       coeff_l3(17,100)=  9.9764307961e-03
       coeff_l3(17,101)= -1.2667308748e-01
       coeff_l3(17,102)=  1.3373099267e-01
       coeff_l3(17,103)= -1.3701221347e-01
       coeff_l3(17,104)=  6.0145538300e-02
       coeff_l3(17,105)= -5.9035420418e-02
       coeff_l3(17,106)= -1.2070449442e-01
       coeff_l3(17,107)= -4.7092244029e-02
       coeff_l3(17,108)= -3.7459410727e-02
       coeff_l3(17,109)=  8.4510415792e-02
       coeff_l3(17,110)= -9.0183623135e-02
       coeff_l3(17,111)=  6.3749211840e-03
       coeff_l3(17,112)= -5.2451923490e-02
       coeff_l3(17,113)=  1.7638009042e-02
       coeff_l3(17,114)=  1.0242987424e-01
       coeff_l3(17,115)= -1.5801337361e-01
       coeff_l3(17,116)= -5.0549972802e-02
       coeff_l3(17,117)= -5.5460207164e-02
       coeff_l3(17,118)= -1.2674620375e-02
       coeff_l3(17,119)=  5.0302468240e-02
       coeff_l3(17,120)=  9.9365688860e-02
       coeff_l3(17,121)=  2.8114354238e-02
       coeff_l3(17,122)= -1.1085779220e-01
       coeff_l3(17,123)=  1.1288583279e-02
       coeff_l3(17,124)=  4.7636455856e-03
       coeff_l3(17,125)=  5.9848584235e-02
       coeff_l3(18,1)= -1.0804475099e-01
       coeff_l3(18,2)= -3.1403252482e-01
       coeff_l3(18,3)=  1.3617409766e-01
       coeff_l3(18,4)=  1.7909958959e-01
       coeff_l3(18,5)= -1.8663862720e-02
       coeff_l3(18,6)=  2.2749489546e-01
       coeff_l3(18,7)= -1.6925294697e-01
       coeff_l3(18,8)=  9.9335378036e-03
       coeff_l3(18,9)=  5.0788331777e-02
       coeff_l3(18,10)= -4.1885510087e-02
       coeff_l3(18,11)=  9.7150519490e-02
       coeff_l3(18,12)= -2.6316592097e-01
       coeff_l3(18,13)= -3.0730923754e-04
       coeff_l3(18,14)= -5.7830695063e-02
       coeff_l3(18,15)= -1.6883704066e-01
       coeff_l3(18,16)=  3.8266524673e-02
       coeff_l3(18,17)= -1.8840886652e-01
       coeff_l3(18,18)= -1.5244600177e-01
       coeff_l3(18,19)= -6.5913386643e-02
       coeff_l3(18,20)= -1.5335491300e-01
       coeff_l3(18,21)= -1.5259876847e-01
       coeff_l3(18,22)=  1.4985443652e-01
       coeff_l3(18,23)= -8.7771207094e-02
       coeff_l3(18,24)= -2.4254068732e-01
       coeff_l3(18,25)= -4.3833968043e-01
       coeff_l3(18,26)= -1.3052758295e-03
       coeff_l3(18,27)= -1.8909244239e-01
       coeff_l3(18,28)= -1.7373068631e-01
       coeff_l3(18,29)=  7.6232720166e-03
       coeff_l3(18,30)= -1.3733784854e-01
       coeff_l3(18,31)= -1.2361918390e-01
       coeff_l3(18,32)= -2.7416078374e-02
       coeff_l3(18,33)= -1.3658093521e-03
       coeff_l3(18,34)= -3.1158458441e-03
       coeff_l3(18,35)= -2.1028748155e-01
       coeff_l3(18,36)= -2.4895658717e-02
       coeff_l3(18,37)= -1.4622764289e-01
       coeff_l3(18,38)=  4.5343172550e-01
       coeff_l3(18,39)= -2.9519408941e-01
       coeff_l3(18,40)=  1.1718820781e-01
       coeff_l3(18,41)= -1.0861712694e-01
       coeff_l3(18,42)= -7.6445318758e-02
       coeff_l3(18,43)= -1.1361453682e-01
       coeff_l3(18,44)= -3.5285297036e-01
       coeff_l3(18,45)= -1.6565062106e-01
       coeff_l3(18,46)= -2.8335601091e-01
       coeff_l3(18,47)=  2.2548204288e-02
       coeff_l3(18,48)= -1.4507346787e-02
       coeff_l3(18,49)= -4.8152518272e-01
       coeff_l3(18,50)= -5.1874961704e-02
       coeff_l3(18,51)= -1.7749032378e-01
       coeff_l3(18,52)= -1.0280054063e-01
       coeff_l3(18,53)= -2.1790987253e-01
       coeff_l3(18,54)=  1.2270382792e-01
       coeff_l3(18,55)= -2.8845716268e-02
       coeff_l3(18,56)=  1.2260536104e-01
       coeff_l3(18,57)= -1.3182879984e-01
       coeff_l3(18,58)= -3.3210583497e-03
       coeff_l3(18,59)=  4.9749065191e-02
       coeff_l3(18,60)= -1.2391376495e-01
       coeff_l3(18,61)=  7.8376002610e-02
       coeff_l3(18,62)= -1.4624541998e-01
       coeff_l3(18,63)= -2.7156257629e-01
       coeff_l3(18,64)= -2.3956111073e-01
       coeff_l3(18,65)= -5.3756318986e-02
       coeff_l3(18,66)= -1.3142964244e-01
       coeff_l3(18,67)=  1.2192965299e-01
       coeff_l3(18,68)= -4.9296900630e-02
       coeff_l3(18,69)= -1.8948474526e-01
       coeff_l3(18,70)= -3.1357154250e-01
       coeff_l3(18,71)= -8.0756004900e-03
       coeff_l3(18,72)= -2.1109201480e-03
       coeff_l3(18,73)=  1.0198502243e-01
       coeff_l3(18,74)=  2.6504185796e-01
       coeff_l3(18,75)= -1.2371068448e-01
       coeff_l3(18,76)= -3.1729781628e-01
       coeff_l3(18,77)= -2.3995110393e-01
       coeff_l3(18,78)= -2.2109158337e-01
       coeff_l3(18,79)= -3.0714541674e-02
       coeff_l3(18,80)= -4.0168961883e-01
       coeff_l3(18,81)=  1.2959663570e-01
       coeff_l3(18,82)=  1.3461560011e-01
       coeff_l3(18,83)=  2.4468097091e-01
       coeff_l3(18,84)= -1.6634242237e-01
       coeff_l3(18,85)=  2.0816087723e-01
       coeff_l3(18,86)= -3.2861372828e-01
       coeff_l3(18,87)=  1.8146266043e-01
       coeff_l3(18,88)= -3.2691347599e-01
       coeff_l3(18,89)= -4.9037463963e-02
       coeff_l3(18,90)= -6.8749614060e-02
       coeff_l3(18,91)=  4.8567403108e-02
       coeff_l3(18,92)=  2.8462249041e-01
       coeff_l3(18,93)= -2.9125198722e-01
       coeff_l3(18,94)=  1.4921672642e-01
       coeff_l3(18,95)= -1.6541582346e-01
       coeff_l3(18,96)=  5.7865932584e-02
       coeff_l3(18,97)=  3.8273338228e-02
       coeff_l3(18,98)= -2.8505472466e-02
       coeff_l3(18,99)= -5.0438549370e-02
       coeff_l3(18,100)= -3.1550920010e-01
       coeff_l3(18,101)=  5.1911685616e-02
       coeff_l3(18,102)= -1.9004228711e-01
       coeff_l3(18,103)= -1.1490949243e-01
       coeff_l3(18,104)=  1.6294205561e-02
       coeff_l3(18,105)=  2.8432460502e-02
       coeff_l3(18,106)=  1.6261737049e-01
       coeff_l3(18,107)= -2.6400053501e-01
       coeff_l3(18,108)= -1.9242266193e-02
       coeff_l3(18,109)= -7.5763493776e-02
       coeff_l3(18,110)=  1.2879514694e-01
       coeff_l3(18,111)=  9.6432700753e-02
       coeff_l3(18,112)= -4.7269809246e-01
       coeff_l3(18,113)= -1.1848626286e-01
       coeff_l3(18,114)=  1.2139571458e-01
       coeff_l3(18,115)= -3.1607249379e-01
       coeff_l3(18,116)= -1.7054697499e-02
       coeff_l3(18,117)=  1.3555316627e-01
       coeff_l3(18,118)=  1.3828052580e-01
       coeff_l3(18,119)= -9.5407433808e-02
       coeff_l3(18,120)= -1.2455432862e-01
       coeff_l3(18,121)= -1.1515527964e-01
       coeff_l3(18,122)= -8.0341830850e-02
       coeff_l3(18,123)= -4.8305541277e-02
       coeff_l3(18,124)= -2.9805970192e-01
       coeff_l3(18,125)= -1.5527413785e-01
       coeff_l3(19,1)=  1.5311922133e-01
       coeff_l3(19,2)=  2.7964431792e-02
       coeff_l3(19,3)=  1.3875977695e-01
       coeff_l3(19,4)= -6.5281733871e-02
       coeff_l3(19,5)= -5.1910072565e-02
       coeff_l3(19,6)= -1.1873178929e-01
       coeff_l3(19,7)= -1.9795951247e-01
       coeff_l3(19,8)=  1.4897742867e-01
       coeff_l3(19,9)= -8.3722077310e-02
       coeff_l3(19,10)= -1.1424958706e-01
       coeff_l3(19,11)=  7.9636335373e-02
       coeff_l3(19,12)=  2.7841967344e-01
       coeff_l3(19,13)=  1.2745492160e-01
       coeff_l3(19,14)= -3.5396158695e-02
       coeff_l3(19,15)=  2.4612750858e-03
       coeff_l3(19,16)=  8.1979939714e-03
       coeff_l3(19,17)=  5.9848960489e-02
       coeff_l3(19,18)=  3.5467231274e-01
       coeff_l3(19,19)= -1.0322009027e-01
       coeff_l3(19,20)= -2.2258825600e-01
       coeff_l3(19,21)=  8.6471840739e-02
       coeff_l3(19,22)=  1.3005572557e-01
       coeff_l3(19,23)= -2.3086646199e-01
       coeff_l3(19,24)=  5.5879715830e-02
       coeff_l3(19,25)=  2.5570389628e-01
       coeff_l3(19,26)= -7.0559494197e-02
       coeff_l3(19,27)=  2.1274834871e-02
       coeff_l3(19,28)= -3.7289448082e-02
       coeff_l3(19,29)= -3.6032631993e-02
       coeff_l3(19,30)=  2.2749805450e-01
       coeff_l3(19,31)=  1.1011055112e-01
       coeff_l3(19,32)=  7.5419373810e-02
       coeff_l3(19,33)=  6.2038801610e-02
       coeff_l3(19,34)=  3.3730141819e-02
       coeff_l3(19,35)= -3.0383062363e-01
       coeff_l3(19,36)=  1.0263802856e-01
       coeff_l3(19,37)= -9.5045305789e-02
       coeff_l3(19,38)=  2.3278330266e-01
       coeff_l3(19,39)= -5.7086683810e-02
       coeff_l3(19,40)= -4.7590747476e-02
       coeff_l3(19,41)=  2.6239714026e-01
       coeff_l3(19,42)= -7.4308671057e-02
       coeff_l3(19,43)=  6.0644887388e-02
       coeff_l3(19,44)=  3.1450685114e-02
       coeff_l3(19,45)= -1.0167286545e-01
       coeff_l3(19,46)= -4.4088639319e-02
       coeff_l3(19,47)= -3.9587013423e-02
       coeff_l3(19,48)=  6.3793092966e-02
       coeff_l3(19,49)= -9.6761971712e-02
       coeff_l3(19,50)=  2.0886773244e-02
       coeff_l3(19,51)= -1.3843591511e-01
       coeff_l3(19,52)=  1.8918487430e-01
       coeff_l3(19,53)= -3.8991220295e-02
       coeff_l3(19,54)= -2.9054895043e-01
       coeff_l3(19,55)=  1.5749217570e-01
       coeff_l3(19,56)=  1.7469762266e-01
       coeff_l3(19,57)=  7.0518173277e-02
       coeff_l3(19,58)= -2.9327082634e-01
       coeff_l3(19,59)= -1.6420048475e-01
       coeff_l3(19,60)=  1.0718493909e-01
       coeff_l3(19,61)= -1.7487601936e-01
       coeff_l3(19,62)= -8.1850357354e-02
       coeff_l3(19,63)=  1.4907714725e-01
       coeff_l3(19,64)= -1.7870335281e-01
       coeff_l3(19,65)= -8.3070598543e-02
       coeff_l3(19,66)= -7.4682250619e-02
       coeff_l3(19,67)=  1.4857941866e-01
       coeff_l3(19,68)=  9.2279098928e-02
       coeff_l3(19,69)=  2.0918097347e-02
       coeff_l3(19,70)=  2.5380942971e-02
       coeff_l3(19,71)=  1.0104910284e-01
       coeff_l3(19,72)= -1.0525210947e-01
       coeff_l3(19,73)=  2.1205903962e-02
       coeff_l3(19,74)= -5.8537151664e-02
       coeff_l3(19,75)=  1.4630830847e-02
       coeff_l3(19,76)= -5.9570636600e-02
       coeff_l3(19,77)= -3.0923292041e-01
       coeff_l3(19,78)= -9.3812562525e-02
       coeff_l3(19,79)=  7.6322324574e-02
       coeff_l3(19,80)= -6.6949784756e-02
       coeff_l3(19,81)=  2.9342669994e-02
       coeff_l3(19,82)=  3.5714756697e-02
       coeff_l3(19,83)=  9.5469318330e-02
       coeff_l3(19,84)=  6.5141327679e-02
       coeff_l3(19,85)=  2.3950648308e-01
       coeff_l3(19,86)=  5.6739740074e-02
       coeff_l3(19,87)=  1.6661140323e-01
       coeff_l3(19,88)= -2.1596984565e-01
       coeff_l3(19,89)= -1.3324413449e-02
       coeff_l3(19,90)= -1.1684378237e-01
       coeff_l3(19,91)=  3.4988056868e-02
       coeff_l3(19,92)= -1.8200814724e-02
       coeff_l3(19,93)= -3.0678093433e-01
       coeff_l3(19,94)=  6.0298680328e-03
       coeff_l3(19,95)=  1.6349229217e-01
       coeff_l3(19,96)= -1.0620137304e-01
       coeff_l3(19,97)= -8.7133944035e-02
       coeff_l3(19,98)=  1.0122806579e-01
       coeff_l3(19,99)=  1.4427646995e-01
       coeff_l3(19,100)=  2.4890442193e-01
       coeff_l3(19,101)=  5.3938113153e-02
       coeff_l3(19,102)= -4.9554105848e-02
       coeff_l3(19,103)= -9.0517185628e-02
       coeff_l3(19,104)= -1.5195542574e-01
       coeff_l3(19,105)=  2.4758592248e-02
       coeff_l3(19,106)= -8.5082665086e-02
       coeff_l3(19,107)= -2.6495862007e-01
       coeff_l3(19,108)= -4.1943654418e-02
       coeff_l3(19,109)=  1.1574187130e-01
       coeff_l3(19,110)= -9.4789609313e-02
       coeff_l3(19,111)=  2.0211508870e-01
       coeff_l3(19,112)= -8.1454291940e-02
       coeff_l3(19,113)=  4.6243466437e-02
       coeff_l3(19,114)=  6.3132539392e-02
       coeff_l3(19,115)=  1.2532469817e-02
       coeff_l3(19,116)= -1.6672244295e-02
       coeff_l3(19,117)=  4.0156215429e-02
       coeff_l3(19,118)=  4.3683153391e-01
       coeff_l3(19,119)= -9.2687353492e-02
       coeff_l3(19,120)=  1.5642441809e-01
       coeff_l3(19,121)= -1.3251379132e-01
       coeff_l3(19,122)= -8.8493525982e-02
       coeff_l3(19,123)= -1.2543350458e-01
       coeff_l3(19,124)=  1.0574293137e-01
       coeff_l3(19,125)=  7.3951795697e-02
       coeff_l3(20,1)= -4.0704774857e-01
       coeff_l3(20,2)=  3.1908851117e-02
       coeff_l3(20,3)=  3.0105885863e-01
       coeff_l3(20,4)= -5.3410425782e-02
       coeff_l3(20,5)=  9.8287783563e-02
       coeff_l3(20,6)=  5.0011570565e-03
       coeff_l3(20,7)= -4.0726407315e-04
       coeff_l3(20,8)=  1.2064288370e-02
       coeff_l3(20,9)= -4.5641060919e-02
       coeff_l3(20,10)=  3.2343905419e-02
       coeff_l3(20,11)= -9.4944827259e-02
       coeff_l3(20,12)=  1.6786314547e-01
       coeff_l3(20,13)=  2.4145053327e-01
       coeff_l3(20,14)= -1.3267502189e-01
       coeff_l3(20,15)= -1.8812781572e-01
       coeff_l3(20,16)= -1.6939052939e-01
       coeff_l3(20,17)= -4.0256315470e-01
       coeff_l3(20,18)=  4.1293863207e-02
       coeff_l3(20,19)= -8.4955200553e-02
       coeff_l3(20,20)= -1.9110077620e-01
       coeff_l3(20,21)=  2.2304973006e-01
       coeff_l3(20,22)= -4.1308596730e-02
       coeff_l3(20,23)= -8.5041364655e-03
       coeff_l3(20,24)=  4.5511551201e-02
       coeff_l3(20,25)= -1.1033464223e-01
       coeff_l3(20,26)= -1.5078921616e-01
       coeff_l3(20,27)= -2.1974970587e-03
       coeff_l3(20,28)=  2.3356392980e-01
       coeff_l3(20,29)= -1.7900879681e-01
       coeff_l3(20,30)= -3.4176295996e-01
       coeff_l3(20,31)=  1.2474021874e-02
       coeff_l3(20,32)=  1.0667805374e-01
       coeff_l3(20,33)= -2.3697361350e-01
       coeff_l3(20,34)= -8.6339510977e-02
       coeff_l3(20,35)= -9.3623824418e-02
       coeff_l3(20,36)=  2.0999547839e-01
       coeff_l3(20,37)= -8.5203960538e-02
       coeff_l3(20,38)=  3.6626350880e-01
       coeff_l3(20,39)=  7.5011722744e-02
       coeff_l3(20,40)= -4.5998662710e-02
       coeff_l3(20,41)=  9.5353275537e-02
       coeff_l3(20,42)= -5.6461384520e-03
       coeff_l3(20,43)= -2.3426420987e-02
       coeff_l3(20,44)= -1.4655154943e-01
       coeff_l3(20,45)= -3.7739016116e-02
       coeff_l3(20,46)= -1.1765935272e-01
       coeff_l3(20,47)=  3.7161551416e-02
       coeff_l3(20,48)= -3.5074902698e-03
       coeff_l3(20,49)= -8.8096928596e-01
       coeff_l3(20,50)= -4.9391321838e-02
       coeff_l3(20,51)=  1.4565755427e-01
       coeff_l3(20,52)=  5.4426986724e-02
       coeff_l3(20,53)=  4.8717439175e-02
       coeff_l3(20,54)= -3.4563219547e-01
       coeff_l3(20,55)= -3.1716102362e-01
       coeff_l3(20,56)= -1.1917930096e-01
       coeff_l3(20,57)=  1.3393069804e-01
       coeff_l3(20,58)=  7.7186077833e-01
       coeff_l3(20,59)= -1.7002613842e-01
       coeff_l3(20,60)= -1.2610882521e+00
       coeff_l3(20,61)= -8.4222696722e-02
       coeff_l3(20,62)=  4.1567623615e-02
       coeff_l3(20,63)= -6.4792013168e-01
       coeff_l3(20,64)=  1.2600908987e-02
       coeff_l3(20,65)=  1.9938457757e-02
       coeff_l3(20,66)= -2.6069796085e-01
       coeff_l3(20,67)=  3.7472888827e-01
       coeff_l3(20,68)= -1.4795400202e-01
       coeff_l3(20,69)=  1.4307564497e-01
       coeff_l3(20,70)= -2.3271524906e-01
       coeff_l3(20,71)=  1.5948845446e-01
       coeff_l3(20,72)= -1.2325850874e-01
       coeff_l3(20,73)=  1.5957665443e-01
       coeff_l3(20,74)= -4.4700443745e-01
       coeff_l3(20,75)= -1.5971069038e-01
       coeff_l3(20,76)= -4.9802771211e-01
       coeff_l3(20,77)= -1.6779789329e-01
       coeff_l3(20,78)= -4.0387022309e-03
       coeff_l3(20,79)= -1.5129783750e-01
       coeff_l3(20,80)= -8.3872115612e-01
       coeff_l3(20,81)=  4.4459888339e-01
       coeff_l3(20,82)= -1.1282841861e-01
       coeff_l3(20,83)= -3.5205315799e-02
       coeff_l3(20,84)=  1.8207231164e-01
       coeff_l3(20,85)=  9.8967343569e-02
       coeff_l3(20,86)=  3.3337984234e-02
       coeff_l3(20,87)= -2.1127726138e-01
       coeff_l3(20,88)=  3.0102291703e-01
       coeff_l3(20,89)= -3.2667797804e-01
       coeff_l3(20,90)= -1.4553745091e-01
       coeff_l3(20,91)= -3.4760481119e-01
       coeff_l3(20,92)= -3.6326888949e-02
       coeff_l3(20,93)= -1.4701279998e-01
       coeff_l3(20,94)=  3.2053358853e-02
       coeff_l3(20,95)= -7.3043897748e-02
       coeff_l3(20,96)= -1.0917188227e-01
       coeff_l3(20,97)= -1.6908484697e-01
       coeff_l3(20,98)=  1.0328777134e-01
       coeff_l3(20,99)= -8.9834302664e-02
       coeff_l3(20,100)= -1.3349039853e-01
       coeff_l3(20,101)=  2.2191117704e-01
       coeff_l3(20,102)=  1.1925270408e-01
       coeff_l3(20,103)= -1.8682561815e-01
       coeff_l3(20,104)= -9.9500894547e-02
       coeff_l3(20,105)=  2.0305036008e-01
       coeff_l3(20,106)= -2.3917993903e-01
       coeff_l3(20,107)= -1.3063965738e-01
       coeff_l3(20,108)=  1.6965529323e-01
       coeff_l3(20,109)= -6.6489011049e-02
       coeff_l3(20,110)= -3.2915195823e-01
       coeff_l3(20,111)= -8.6483903229e-02
       coeff_l3(20,112)= -4.5572942495e-01
       coeff_l3(20,113)= -7.9212442040e-02
       coeff_l3(20,114)= -4.3880984187e-02
       coeff_l3(20,115)=  4.0952652693e-01
       coeff_l3(20,116)= -5.5314254016e-02
       coeff_l3(20,117)= -2.8038969636e-01
       coeff_l3(20,118)= -3.3501470089e-01
       coeff_l3(20,119)= -6.6649720073e-02
       coeff_l3(20,120)= -6.1403468251e-02
       coeff_l3(20,121)=  4.4796667062e-03
       coeff_l3(20,122)= -4.0479935706e-02
       coeff_l3(20,123)=  1.0943729430e-01
       coeff_l3(20,124)=  8.0903328955e-02
       coeff_l3(20,125)=  2.0321987569e-02
       coeff_l3(21,1)=  7.6915788651e-01
       coeff_l3(21,2)= -5.5636178702e-02
       coeff_l3(21,3)= -6.6816478968e-02
       coeff_l3(21,4)=  4.7028592229e-01
       coeff_l3(21,5)=  2.4355466664e-01
       coeff_l3(21,6)= -9.4108358026e-02
       coeff_l3(21,7)= -1.5874540806e-01
       coeff_l3(21,8)=  1.6802483797e-01
       coeff_l3(21,9)= -1.0920299590e-01
       coeff_l3(21,10)=  2.3770666122e-01
       coeff_l3(21,11)=  2.5174289942e-01
       coeff_l3(21,12)=  1.0108851194e+00
       coeff_l3(21,13)=  3.4241682291e-01
       coeff_l3(21,14)=  3.2522103190e-01
       coeff_l3(21,15)= -6.4909346402e-02
       coeff_l3(21,16)= -4.1944697499e-01
       coeff_l3(21,17)=  3.3320999146e-01
       coeff_l3(21,18)=  9.6001684666e-01
       coeff_l3(21,19)= -7.5847089291e-02
       coeff_l3(21,20)= -1.8313832581e-01
       coeff_l3(21,21)= -3.6941164732e-01
       coeff_l3(21,22)=  4.1142269969e-02
       coeff_l3(21,23)= -4.5643925667e-01
       coeff_l3(21,24)= -1.8514995277e-01
       coeff_l3(21,25)=  1.1540619284e-01
       coeff_l3(21,26)= -1.9873064756e-01
       coeff_l3(21,27)= -3.1558999419e-01
       coeff_l3(21,28)= -3.9149064571e-02
       coeff_l3(21,29)=  8.5579746962e-01
       coeff_l3(21,30)= -2.3177337646e-01
       coeff_l3(21,31)= -2.8392383456e-01
       coeff_l3(21,32)= -1.0239692777e-01
       coeff_l3(21,33)=  2.0476934314e-01
       coeff_l3(21,34)=  9.8011337221e-02
       coeff_l3(21,35)= -2.9240947962e-01
       coeff_l3(21,36)= -1.0331948847e-01
       coeff_l3(21,37)= -1.4326517284e-01
       coeff_l3(21,38)=  4.6702948213e-01
       coeff_l3(21,39)= -2.6920618489e-02
       coeff_l3(21,40)=  2.3718227446e-01
       coeff_l3(21,41)=  7.5316888094e-01
       coeff_l3(21,42)= -3.8529703021e-01
       coeff_l3(21,43)=  2.7375867963e-01
       coeff_l3(21,44)= -3.3251988888e-01
       coeff_l3(21,45)=  3.0140361190e-01
       coeff_l3(21,46)=  1.2150064856e-01
       coeff_l3(21,47)=  5.7666439563e-02
       coeff_l3(21,48)=  2.0979298651e-01
       coeff_l3(21,49)= -6.0486447811e-01
       coeff_l3(21,50)=  3.3798497915e-01
       coeff_l3(21,51)= -6.2545157969e-02
       coeff_l3(21,52)=  1.4370889962e-01
       coeff_l3(21,53)= -6.0467857867e-02
       coeff_l3(21,54)= -5.6710973382e-02
       coeff_l3(21,55)=  2.6143772528e-02
       coeff_l3(21,56)=  2.6110801101e-01
       coeff_l3(21,57)= -1.9135206938e-01
       coeff_l3(21,58)= -3.5894680023e-01
       coeff_l3(21,59)= -7.6548375189e-02
       coeff_l3(21,60)=  7.2822409868e-01
       coeff_l3(21,61)= -2.7209755778e-01
       coeff_l3(21,62)= -4.6144095063e-01
       coeff_l3(21,63)= -7.8465151787e-01
       coeff_l3(21,64)= -1.5367109329e-02
       coeff_l3(21,65)= -2.1024432778e-01
       coeff_l3(21,66)= -2.6099488139e-02
       coeff_l3(21,67)= -5.7219732553e-02
       coeff_l3(21,68)=  1.3227075338e-01
       coeff_l3(21,69)=  2.0844133198e-01
       coeff_l3(21,70)= -2.9623603821e-01
       coeff_l3(21,71)=  9.2641949654e-01
       coeff_l3(21,72)= -4.3910393119e-01
       coeff_l3(21,73)= -1.1498771608e-02
       coeff_l3(21,74)= -2.3089432716e-01
       coeff_l3(21,75)= -2.4422475696e-01
       coeff_l3(21,76)= -9.0495389700e-01
       coeff_l3(21,77)= -3.3385297656e-01
       coeff_l3(21,78)= -5.6713711470e-02
       coeff_l3(21,79)=  3.9215482771e-02
       coeff_l3(21,80)=  1.0651697405e-02
       coeff_l3(21,81)= -2.1014864743e-01
       coeff_l3(21,82)=  2.9147276655e-02
       coeff_l3(21,83)= -5.7641297579e-02
       coeff_l3(21,84)= -3.2128787041e-01
       coeff_l3(21,85)= -1.7827002704e-01
       coeff_l3(21,86)=  2.5581958890e-01
       coeff_l3(21,87)=  1.8191714585e-01
       coeff_l3(21,88)= -6.3371628523e-02
       coeff_l3(21,89)= -1.9159121811e-01
       coeff_l3(21,90)= -5.4006826878e-01
       coeff_l3(21,91)=  1.1404373497e-01
       coeff_l3(21,92)= -7.8530656174e-03
       coeff_l3(21,93)=  5.4498261213e-01
       coeff_l3(21,94)=  8.2212728262e-01
       coeff_l3(21,95)=  2.5571411848e-01
       coeff_l3(21,96)=  2.0708949864e-01
       coeff_l3(21,97)=  1.0838196427e-01
       coeff_l3(21,98)= -4.5970642567e-01
       coeff_l3(21,99)= -5.2511714399e-02
       coeff_l3(21,100)=  7.2059184313e-02
       coeff_l3(21,101)= -4.4625705481e-01
       coeff_l3(21,102)=  3.0655297637e-01
       coeff_l3(21,103)= -5.4716843367e-01
       coeff_l3(21,104)= -3.7292110920e-01
       coeff_l3(21,105)=  6.6623979807e-01
       coeff_l3(21,106)=  3.2523852587e-01
       coeff_l3(21,107)=  3.3826404810e-01
       coeff_l3(21,108)=  2.1576577425e-01
       coeff_l3(21,109)= -2.6905718446e-01
       coeff_l3(21,110)= -7.4503056705e-02
       coeff_l3(21,111)=  3.6386379600e-01
       coeff_l3(21,112)=  8.1874914467e-02
       coeff_l3(21,113)=  5.1694130898e-01
       coeff_l3(21,114)=  4.0612190962e-02
       coeff_l3(21,115)=  4.3045333028e-01
       coeff_l3(21,116)=  7.9169529676e-01
       coeff_l3(21,117)=  5.5444817990e-02
       coeff_l3(21,118)=  4.4337645173e-01
       coeff_l3(21,119)= -2.0739106461e-02
       coeff_l3(21,120)=  3.9558520913e-01
       coeff_l3(21,121)=  8.4419712424e-02
       coeff_l3(21,122)=  7.7493898571e-02
       coeff_l3(21,123)=  5.6525211781e-02
       coeff_l3(21,124)= -1.7669151723e-01
       coeff_l3(21,125)=  3.2455185056e-01
       coeff_l3(22,1)= -1.1199789047e+00
       coeff_l3(22,2)= -1.8194924295e-01
       coeff_l3(22,3)=  3.7268707156e-01
       coeff_l3(22,4)=  8.9844994247e-02
       coeff_l3(22,5)=  4.0973845869e-02
       coeff_l3(22,6)=  2.1543224156e-01
       coeff_l3(22,7)= -6.2811559439e-01
       coeff_l3(22,8)= -2.7203992009e-01
       coeff_l3(22,9)=  7.0114403963e-02
       coeff_l3(22,10)= -2.7757433057e-01
       coeff_l3(22,11)= -6.0155980289e-02
       coeff_l3(22,12)= -1.0858423710e+00
       coeff_l3(22,13)= -5.3021800518e-01
       coeff_l3(22,14)= -3.8605576754e-01
       coeff_l3(22,15)= -1.2037938833e-01
       coeff_l3(22,16)=  9.7373187542e-02
       coeff_l3(22,17)= -4.4351479411e-01
       coeff_l3(22,18)= -1.4067772627e+00
       coeff_l3(22,19)= -2.8115245700e-01
       coeff_l3(22,20)= -3.0191380531e-02
       coeff_l3(22,21)= -1.5376272611e-02
       coeff_l3(22,22)= -6.0376107693e-02
       coeff_l3(22,23)=  8.9476570487e-02
       coeff_l3(22,24)=  1.0645216703e-01
       coeff_l3(22,25)= -7.7017188072e-01
       coeff_l3(22,26)= -8.3778731525e-02
       coeff_l3(22,27)= -1.6667943001e+00
       coeff_l3(22,28)= -8.1288164854e-01
       coeff_l3(22,29)= -1.2504262924e+00
       coeff_l3(22,30)=  8.3386087418e-01
       coeff_l3(22,31)=  1.9845223427e-01
       coeff_l3(22,32)=  7.6595470309e-02
       coeff_l3(22,33)= -1.0164342076e-01
       coeff_l3(22,34)=  6.8953827024e-02
       coeff_l3(22,35)=  6.5413069725e-01
       coeff_l3(22,36)=  1.2741714716e-01
       coeff_l3(22,37)=  1.4712253213e-01
       coeff_l3(22,38)=  3.0357271433e-02
       coeff_l3(22,39)= -6.7565128207e-02
       coeff_l3(22,40)= -2.9006814957e-01
       coeff_l3(22,41)= -1.4190781116e+00
       coeff_l3(22,42)= -2.4212396238e-03
       coeff_l3(22,43)= -3.5144466162e-01
       coeff_l3(22,44)=  1.0173425078e-01
       coeff_l3(22,45)= -8.0080583692e-02
       coeff_l3(22,46)=  1.3029515743e-01
       coeff_l3(22,47)=  1.1712296307e-01
       coeff_l3(22,48)= -5.7070058584e-01
       coeff_l3(22,49)=  9.6852421761e-01
       coeff_l3(22,50)= -2.1645690501e-01
       coeff_l3(22,51)= -5.7749140263e-01
       coeff_l3(22,52)=  2.2700539231e-01
       coeff_l3(22,53)= -8.1504927948e-03
       coeff_l3(22,54)= -6.8082594872e-01
       coeff_l3(22,55)= -2.2758208215e-02
       coeff_l3(22,56)= -1.8882749602e-02
       coeff_l3(22,57)= -1.3731037080e-01
       coeff_l3(22,58)=  6.6258883476e-01
       coeff_l3(22,59)= -1.2961086631e-01
       coeff_l3(22,60)=  6.3342881203e-01
       coeff_l3(22,61)=  1.3762570918e-01
       coeff_l3(22,62)= -9.4418771565e-02
       coeff_l3(22,63)=  1.3735410571e-01
       coeff_l3(22,64)= -2.0191568881e-02
       coeff_l3(22,65)= -1.6895128414e-02
       coeff_l3(22,66)=  2.8550107032e-02
       coeff_l3(22,67)=  1.0950242728e-01
       coeff_l3(22,68)= -5.2910220623e-01
       coeff_l3(22,69)=  7.3036715388e-02
       coeff_l3(22,70)= -2.5494489074e-01
       coeff_l3(22,71)= -1.6139848232e+00
       coeff_l3(22,72)= -4.5990593731e-02
       coeff_l3(22,73)= -3.9900232106e-02
       coeff_l3(22,74)= -2.8863310814e-01
       coeff_l3(22,75)=  5.3570307791e-02
       coeff_l3(22,76)=  7.7559953928e-01
       coeff_l3(22,77)= -2.1635793149e-01
       coeff_l3(22,78)=  4.3265260756e-02
       coeff_l3(22,79)=  1.8550256267e-02
       coeff_l3(22,80)=  5.0055277348e-01
       coeff_l3(22,81)= -1.6714963317e-01
       coeff_l3(22,82)= -1.6386806965e-01
       coeff_l3(22,83)= -6.2283765525e-02
       coeff_l3(22,84)= -1.6683901846e-01
       coeff_l3(22,85)= -2.1045041084e-01
       coeff_l3(22,86)= -2.0580761135e-02
       coeff_l3(22,87)= -2.7400472760e-01
       coeff_l3(22,88)= -3.6000585556e-01
       coeff_l3(22,89)=  3.0101031065e-01
       coeff_l3(22,90)=  2.5995656848e-01
       coeff_l3(22,91)=  8.1372179091e-02
       coeff_l3(22,92)= -1.0774516314e-01
       coeff_l3(22,93)=  8.1032745540e-02
       coeff_l3(22,94)= -1.9211530685e-01
       coeff_l3(22,95)= -3.8396072388e-01
       coeff_l3(22,96)= -6.7073398829e-01
       coeff_l3(22,97)= -2.0833637565e-02
       coeff_l3(22,98)=  1.5634171665e-01
       coeff_l3(22,99)=  1.1904831976e-01
       coeff_l3(22,100)= -8.9377798140e-02
       coeff_l3(22,101)=  2.9006361961e-01
       coeff_l3(22,102)=  1.3577874005e-01
       coeff_l3(22,103)=  2.0089611411e-02
       coeff_l3(22,104)=  1.1553938501e-02
       coeff_l3(22,105)= -3.4361243248e-01
       coeff_l3(22,106)= -4.4155019522e-01
       coeff_l3(22,107)= -1.5684695542e-01
       coeff_l3(22,108)= -1.6465291381e-01
       coeff_l3(22,109)= -2.2154981270e-02
       coeff_l3(22,110)= -7.3273390532e-02
       coeff_l3(22,111)= -4.1953179240e-01
       coeff_l3(22,112)= -3.2260727882e-01
       coeff_l3(22,113)= -7.0323443413e-01
       coeff_l3(22,114)= -8.0438804626e-01
       coeff_l3(22,115)=  2.0407432690e-02
       coeff_l3(22,116)= -6.7948418856e-01
       coeff_l3(22,117)= -2.6536807418e-01
       coeff_l3(22,118)= -1.0787224025e-01
       coeff_l3(22,119)=  1.6792227328e-01
       coeff_l3(22,120)= -2.8832727671e-01
       coeff_l3(22,121)=  2.5872426108e-02
       coeff_l3(22,122)= -1.7577630281e-01
       coeff_l3(22,123)= -1.2053333223e-01
       coeff_l3(22,124)= -3.5040566325e-01
       coeff_l3(22,125)=  9.4064317644e-02
       coeff_l3(23,1)= -2.7552509308e-01
       coeff_l3(23,2)= -3.1597420573e-02
       coeff_l3(23,3)= -1.1455789208e-01
       coeff_l3(23,4)= -2.3525221646e-01
       coeff_l3(23,5)= -4.9769643694e-02
       coeff_l3(23,6)=  1.1648407578e-01
       coeff_l3(23,7)=  2.3361022770e-01
       coeff_l3(23,8)= -1.4148177207e-01
       coeff_l3(23,9)=  7.8479237854e-02
       coeff_l3(23,10)= -6.5167009830e-02
       coeff_l3(23,11)= -1.4994338155e-01
       coeff_l3(23,12)= -3.6757469177e-02
       coeff_l3(23,13)= -1.8091475964e-01
       coeff_l3(23,14)=  5.4090883583e-02
       coeff_l3(23,15)= -1.5159861743e-01
       coeff_l3(23,16)=  7.6308473945e-02
       coeff_l3(23,17)= -1.3111409545e-01
       coeff_l3(23,18)= -3.9151754975e-01
       coeff_l3(23,19)= -5.1190424711e-02
       coeff_l3(23,20)= -3.6267772317e-02
       coeff_l3(23,21)= -3.7642383575e-01
       coeff_l3(23,22)=  7.9844355583e-02
       coeff_l3(23,23)=  1.0102367401e-01
       coeff_l3(23,24)=  1.2733148038e-01
       coeff_l3(23,25)= -1.0291454196e-01
       coeff_l3(23,26)=  9.2248126864e-02
       coeff_l3(23,27)=  1.2306449562e-01
       coeff_l3(23,28)=  8.9758321643e-02
       coeff_l3(23,29)= -2.8765383363e-01
       coeff_l3(23,30)= -1.0297700763e-01
       coeff_l3(23,31)= -1.0071281344e-01
       coeff_l3(23,32)=  8.0505251884e-02
       coeff_l3(23,33)= -4.6264067292e-02
       coeff_l3(23,34)=  1.5885221958e-01
       coeff_l3(23,35)= -3.5893410444e-01
       coeff_l3(23,36)=  9.3262173235e-02
       coeff_l3(23,37)=  1.6213817894e-01
       coeff_l3(23,38)= -2.5638917089e-01
       coeff_l3(23,39)= -1.3357295096e-01
       coeff_l3(23,40)= -1.0260163248e-01
       coeff_l3(23,41)= -2.8882429004e-01
       coeff_l3(23,42)=  1.2242744118e-01
       coeff_l3(23,43)= -8.1816948950e-02
       coeff_l3(23,44)= -7.9820238054e-02
       coeff_l3(23,45)= -7.0015572011e-02
       coeff_l3(23,46)=  5.0026625395e-02
       coeff_l3(23,47)= -5.0602775067e-02
       coeff_l3(23,48)= -7.9778760672e-02
       coeff_l3(23,49)= -2.7844804525e-01
       coeff_l3(23,50)=  1.9557707012e-02
       coeff_l3(23,51)= -9.1535605490e-02
       coeff_l3(23,52)=  8.5795328021e-02
       coeff_l3(23,53)=  1.0557855666e-01
       coeff_l3(23,54)=  5.4018191993e-02
       coeff_l3(23,55)= -1.0500709713e-01
       coeff_l3(23,56)= -1.3360385597e-01
       coeff_l3(23,57)=  7.4371404946e-02
       coeff_l3(23,58)=  5.0628568977e-02
       coeff_l3(23,59)= -2.9431143403e-01
       coeff_l3(23,60)= -7.8179365396e-01
       coeff_l3(23,61)= -9.4591438770e-02
       coeff_l3(23,62)=  1.6655965149e-01
       coeff_l3(23,63)= -1.6082513332e-01
       coeff_l3(23,64)=  6.6312544048e-02
       coeff_l3(23,65)= -6.5704934299e-02
       coeff_l3(23,66)=  1.7761148512e-01
       coeff_l3(23,67)= -1.8953520060e-01
       coeff_l3(23,68)=  2.0579937845e-02
       coeff_l3(23,69)= -8.1838749349e-02
       coeff_l3(23,70)=  4.5753281564e-02
       coeff_l3(23,71)= -3.7047588825e-01
       coeff_l3(23,72)= -2.6289859787e-02
       coeff_l3(23,73)= -4.8547722399e-02
       coeff_l3(23,74)= -1.6974391043e-01
       coeff_l3(23,75)=  8.3525158465e-02
       coeff_l3(23,76)= -5.7625222951e-02
       coeff_l3(23,77)=  7.5533159077e-02
       coeff_l3(23,78)= -4.9209360033e-02
       coeff_l3(23,79)= -8.8378272951e-02
       coeff_l3(23,80)= -1.5819954872e-01
       coeff_l3(23,81)= -1.8731823564e-01
       coeff_l3(23,82)=  3.2984395511e-03
       coeff_l3(23,83)= -2.7144485712e-01
       coeff_l3(23,84)=  4.6334650367e-02
       coeff_l3(23,85)= -2.1874772012e-01
       coeff_l3(23,86)=  5.8248933405e-02
       coeff_l3(23,87)= -1.8277990818e-01
       coeff_l3(23,88)= -2.9632568359e-01
       coeff_l3(23,89)=  1.8257038295e-01
       coeff_l3(23,90)= -2.2413238883e-01
       coeff_l3(23,91)= -2.5199401379e-01
       coeff_l3(23,92)= -8.8581321761e-03
       coeff_l3(23,93)=  6.6430874169e-02
       coeff_l3(23,94)= -2.1012307703e-01
       coeff_l3(23,95)=  1.0618001223e-01
       coeff_l3(23,96)=  3.0447673053e-02
       coeff_l3(23,97)=  1.4762546122e-01
       coeff_l3(23,98)= -5.5955792777e-03
       coeff_l3(23,99)= -1.4137077332e-01
       coeff_l3(23,100)=  4.4428732246e-02
       coeff_l3(23,101)= -7.7288307250e-02
       coeff_l3(23,102)= -1.1777953058e-01
       coeff_l3(23,103)= -1.9265182316e-02
       coeff_l3(23,104)= -1.1717509478e-01
       coeff_l3(23,105)= -7.9858802259e-02
       coeff_l3(23,106)= -9.6912190318e-02
       coeff_l3(23,107)= -2.4900786579e-01
       coeff_l3(23,108)=  1.0087637603e-01
       coeff_l3(23,109)=  1.3138599694e-01
       coeff_l3(23,110)= -1.5418112278e-02
       coeff_l3(23,111)= -1.2335862964e-02
       coeff_l3(23,112)= -4.0801087022e-01
       coeff_l3(23,113)= -2.5156581402e-01
       coeff_l3(23,114)= -1.3936072588e-01
       coeff_l3(23,115)= -3.3487606049e-01
       coeff_l3(23,116)= -2.5896361470e-01
       coeff_l3(23,117)= -4.4308599085e-02
       coeff_l3(23,118)=  3.8901254535e-02
       coeff_l3(23,119)= -6.2008146197e-02
       coeff_l3(23,120)= -6.6388785839e-02
       coeff_l3(23,121)= -2.1144203842e-01
       coeff_l3(23,122)=  5.2586119622e-02
       coeff_l3(23,123)=  1.0900825262e-01
       coeff_l3(23,124)=  3.7575431168e-02
       coeff_l3(23,125)=  1.8738757074e-01
       coeff_l3(24,1)= -1.2073878944e-01
       coeff_l3(24,2)=  1.6791884601e-01
       coeff_l3(24,3)= -9.2399477959e-02
       coeff_l3(24,4)= -1.0143047571e-01
       coeff_l3(24,5)=  1.5986998379e-01
       coeff_l3(24,6)=  7.8792005777e-02
       coeff_l3(24,7)=  1.2436934561e-01
       coeff_l3(24,8)= -3.2620403171e-01
       coeff_l3(24,9)=  1.0482093692e-01
       coeff_l3(24,10)= -7.4059151113e-02
       coeff_l3(24,11)=  5.5403638631e-02
       coeff_l3(24,12)= -1.7595516518e-02
       coeff_l3(24,13)= -2.1693147719e-01
       coeff_l3(24,14)=  1.2232120149e-02
       coeff_l3(24,15)= -1.4028300345e-01
       coeff_l3(24,16)= -4.8753809184e-02
       coeff_l3(24,17)=  8.0325789750e-02
       coeff_l3(24,18)= -2.0984916389e-01
       coeff_l3(24,19)=  1.0573331267e-01
       coeff_l3(24,20)=  1.8738266826e-01
       coeff_l3(24,21)= -1.6999118030e-01
       coeff_l3(24,22)= -1.3024674356e-01
       coeff_l3(24,23)= -1.7493011430e-02
       coeff_l3(24,24)= -1.6964150965e-01
       coeff_l3(24,25)=  5.1518082619e-02
       coeff_l3(24,26)=  8.3173394203e-02
       coeff_l3(24,27)=  3.3673810959e-01
       coeff_l3(24,28)=  3.6529767513e-01
       coeff_l3(24,29)= -2.1934850514e-01
       coeff_l3(24,30)= -1.5811040998e-01
       coeff_l3(24,31)=  7.0442542434e-02
       coeff_l3(24,32)=  6.6866740584e-02
       coeff_l3(24,33)= -9.0201988816e-02
       coeff_l3(24,34)=  9.7416331992e-03
       coeff_l3(24,35)= -1.6811870039e-01
       coeff_l3(24,36)= -7.1133293211e-02
       coeff_l3(24,37)=  7.7436104417e-02
       coeff_l3(24,38)= -5.5337679386e-01
       coeff_l3(24,39)=  1.5262222290e-01
       coeff_l3(24,40)= -6.7971400917e-02
       coeff_l3(24,41)= -6.2294006348e-01
       coeff_l3(24,42)= -1.3517808914e-01
       coeff_l3(24,43)=  1.0565620661e-01
       coeff_l3(24,44)=  1.3684852421e-01
       coeff_l3(24,45)= -1.1001432687e-01
       coeff_l3(24,46)= -1.4195538126e-02
       coeff_l3(24,47)= -1.8404144049e-01
       coeff_l3(24,48)= -3.8974761963e-02
       coeff_l3(24,49)= -1.6487328708e-01
       coeff_l3(24,50)= -1.6027322412e-01
       coeff_l3(24,51)=  7.9942017794e-02
       coeff_l3(24,52)=  1.0957351327e-01
       coeff_l3(24,53)=  1.5627397597e-01
       coeff_l3(24,54)= -8.2224965096e-02
       coeff_l3(24,55)= -9.1877996922e-02
       coeff_l3(24,56)= -5.7462181896e-02
       coeff_l3(24,57)= -1.1909435689e-01
       coeff_l3(24,58)=  1.6675646603e-01
       coeff_l3(24,59)=  1.1350765079e-01
       coeff_l3(24,60)= -3.8437232375e-01
       coeff_l3(24,61)= -2.2482611239e-02
       coeff_l3(24,62)= -2.9969394673e-03
       coeff_l3(24,63)= -3.8107710425e-03
       coeff_l3(24,64)= -1.4427579939e-01
       coeff_l3(24,65)= -5.8664601296e-02
       coeff_l3(24,66)=  7.8456215560e-02
       coeff_l3(24,67)= -3.1383645535e-01
       coeff_l3(24,68)= -2.6081889868e-01
       coeff_l3(24,69)= -1.1452224106e-01
       coeff_l3(24,70)= -9.8004482687e-02
       coeff_l3(24,71)= -3.0992883444e-01
       coeff_l3(24,72)=  3.6866877228e-02
       coeff_l3(24,73)=  6.9106720388e-02
       coeff_l3(24,74)= -1.2020155042e-01
       coeff_l3(24,75)=  1.0807169229e-01
       coeff_l3(24,76)= -2.3749070242e-02
       coeff_l3(24,77)= -8.5246590897e-03
       coeff_l3(24,78)= -1.3117970899e-02
       coeff_l3(24,79)= -1.0276235640e-01
       coeff_l3(24,80)=  4.3053042889e-01
       coeff_l3(24,81)= -3.6544397473e-01
       coeff_l3(24,82)=  1.9552746788e-02
       coeff_l3(24,83)= -5.8282967657e-02
       coeff_l3(24,84)= -1.6998451948e-01
       coeff_l3(24,85)= -1.9249390066e-01
       coeff_l3(24,86)= -2.4771269411e-02
       coeff_l3(24,87)= -1.5477620065e-01
       coeff_l3(24,88)= -3.0365297198e-01
       coeff_l3(24,89)=  1.5443670750e-01
       coeff_l3(24,90)= -1.6058874130e-01
       coeff_l3(24,91)=  8.6018905044e-02
       coeff_l3(24,92)= -1.1368263513e-01
       coeff_l3(24,93)= -1.8337249756e-01
       coeff_l3(24,94)=  1.2552908063e-01
       coeff_l3(24,95)= -2.3499056697e-02
       coeff_l3(24,96)=  1.1251752824e-01
       coeff_l3(24,97)= -9.0174034238e-02
       coeff_l3(24,98)= -7.1252562106e-02
       coeff_l3(24,99)= -1.4093062282e-01
       coeff_l3(24,100)= -1.5475364029e-01
       coeff_l3(24,101)= -1.9621890038e-02
       coeff_l3(24,102)= -9.9347904325e-02
       coeff_l3(24,103)=  1.9969403744e-01
       coeff_l3(24,104)=  1.4881150424e-01
       coeff_l3(24,105)= -5.2730128169e-02
       coeff_l3(24,106)= -6.4781075343e-03
       coeff_l3(24,107)= -1.6526107490e-01
       coeff_l3(24,108)=  9.5831714571e-02
       coeff_l3(24,109)=  1.0774330050e-01
       coeff_l3(24,110)= -7.5962536037e-02
       coeff_l3(24,111)=  5.6983668357e-02
       coeff_l3(24,112)= -3.6576163769e-01
       coeff_l3(24,113)= -7.4263080955e-02
       coeff_l3(24,114)= -1.6587689519e-01
       coeff_l3(24,115)=  1.3273313642e-01
       coeff_l3(24,116)= -8.5097625852e-02
       coeff_l3(24,117)= -2.4521650374e-01
       coeff_l3(24,118)= -2.6249168441e-02
       coeff_l3(24,119)=  8.6985886097e-02
       coeff_l3(24,120)=  6.0424804688e-03
       coeff_l3(24,121)= -4.7803163528e-02
       coeff_l3(24,122)=  1.4259463549e-01
       coeff_l3(24,123)= -1.4932996593e-02
       coeff_l3(24,124)= -3.7337280810e-03
       coeff_l3(24,125)=  7.3972113431e-02
       coeff_l3(25,1)= -4.3899363279e-01
       coeff_l3(25,2)= -1.6197712719e-01
       coeff_l3(25,3)=  1.4222061634e-01
       coeff_l3(25,4)= -7.5648909807e-01
       coeff_l3(25,5)= -1.5855461359e-01
       coeff_l3(25,6)= -3.4522265196e-01
       coeff_l3(25,7)= -8.8307434320e-01
       coeff_l3(25,8)= -4.7915452719e-01
       coeff_l3(25,9)=  1.1596050858e-01
       coeff_l3(25,10)=  2.4037714303e-01
       coeff_l3(25,11)= -3.1200030446e-01
       coeff_l3(25,12)= -6.8278002739e-01
       coeff_l3(25,13)= -8.7003335357e-02
       coeff_l3(25,14)=  1.5870656073e-01
       coeff_l3(25,15)= -1.1722610146e-01
       coeff_l3(25,16)= -4.6538788825e-02
       coeff_l3(25,17)= -1.0245929472e-02
       coeff_l3(25,18)= -8.0312246084e-01
       coeff_l3(25,19)=  4.5043427497e-02
       coeff_l3(25,20)=  1.5280598402e-01
       coeff_l3(25,21)= -5.1117545366e-01
       coeff_l3(25,22)= -1.7271826044e-02
       coeff_l3(25,23)= -1.3364896178e-01
       coeff_l3(25,24)= -5.6085452437e-02
       coeff_l3(25,25)= -1.0407330990e+00
       coeff_l3(25,26)= -1.1804077774e-02
       coeff_l3(25,27)= -1.0877163410e+00
       coeff_l3(25,28)= -7.4213933945e-01
       coeff_l3(25,29)= -1.0277394056e+00
       coeff_l3(25,30)= -4.6179479361e-01
       coeff_l3(25,31)=  7.1403302252e-02
       coeff_l3(25,32)= -1.7555460334e-02
       coeff_l3(25,33)= -5.1046305895e-01
       coeff_l3(25,34)= -4.4336371124e-02
       coeff_l3(25,35)= -3.5329133272e-01
       coeff_l3(25,36)= -8.1318996847e-02
       coeff_l3(25,37)= -7.0154175162e-02
       coeff_l3(25,38)= -8.9209043980e-01
       coeff_l3(25,39)=  5.4306078702e-02
       coeff_l3(25,40)= -4.3554890156e-01
       coeff_l3(25,41)= -5.7976490259e-01
       coeff_l3(25,42)= -8.1924505532e-02
       coeff_l3(25,43)=  2.3437091708e-01
       coeff_l3(25,44)= -1.0838474333e-01
       coeff_l3(25,45)=  2.7062681317e-01
       coeff_l3(25,46)= -6.8754839897e-01
       coeff_l3(25,47)= -1.8512046337e-01
       coeff_l3(25,48)= -7.5058311224e-01
       coeff_l3(25,49)= -2.1863147616e-01
       coeff_l3(25,50)=  1.4485174417e-01
       coeff_l3(25,51)= -1.8564580381e-01
       coeff_l3(25,52)= -4.0557410568e-02
       coeff_l3(25,53)= -4.2636862397e-01
       coeff_l3(25,54)= -7.9354315996e-01
       coeff_l3(25,55)= -7.6693600416e-01
       coeff_l3(25,56)= -3.9046591520e-01
       coeff_l3(25,57)= -3.3691510558e-01
       coeff_l3(25,58)= -1.6948762536e-01
       coeff_l3(25,59)= -1.3651153445e-01
       coeff_l3(25,60)= -4.4605743885e-01
       coeff_l3(25,61)= -2.6994559169e-01
       coeff_l3(25,62)= -2.9551449418e-01
       coeff_l3(25,63)= -3.0710376799e-02
       coeff_l3(25,64)= -2.9546940327e-01
       coeff_l3(25,65)= -2.8741851449e-01
       coeff_l3(25,66)= -5.1584880799e-02
       coeff_l3(25,67)= -5.9889906645e-01
       coeff_l3(25,68)=  1.2034393847e-01
       coeff_l3(25,69)= -3.8097676635e-01
       coeff_l3(25,70)= -1.5433238447e-01
       coeff_l3(25,71)= -7.8001666069e-01
       coeff_l3(25,72)= -2.1413180232e-01
       coeff_l3(25,73)= -5.4219104350e-02
       coeff_l3(25,74)= -3.8951718807e-01
       coeff_l3(25,75)= -3.0801028013e-01
       coeff_l3(25,76)= -6.9100737572e-01
       coeff_l3(25,77)= -1.0901284963e-01
       coeff_l3(25,78)= -4.8429004848e-02
       coeff_l3(25,79)=  1.5295881033e-01
       coeff_l3(25,80)= -1.8056595325e-01
       coeff_l3(25,81)=  1.1378732324e-01
       coeff_l3(25,82)= -3.1699064374e-01
       coeff_l3(25,83)= -2.8310686350e-01
       coeff_l3(25,84)= -3.5108190030e-02
       coeff_l3(25,85)=  2.3434416950e-01
       coeff_l3(25,86)=  1.7447860539e-01
       coeff_l3(25,87)= -1.3265608251e-01
       coeff_l3(25,88)= -7.2972558439e-02
       coeff_l3(25,89)=  1.6285622120e-01
       coeff_l3(25,90)= -3.0372467637e-01
       coeff_l3(25,91)=  1.3079141080e-01
       coeff_l3(25,92)= -2.6368454099e-01
       coeff_l3(25,93)=  3.4586567432e-02
       coeff_l3(25,94)= -4.0310516953e-01
       coeff_l3(25,95)= -2.2956359386e-01
       coeff_l3(25,96)= -6.2962383032e-01
       coeff_l3(25,97)= -1.5943488479e-01
       coeff_l3(25,98)=  8.3361893892e-02
       coeff_l3(25,99)= -1.7118670046e-01
       coeff_l3(25,100)= -2.8326961398e-01
       coeff_l3(25,101)= -1.0010489821e-01
       coeff_l3(25,102)= -4.1429232806e-02
       coeff_l3(25,103)= -1.8528078496e-01
       coeff_l3(25,104)=  1.5971928835e-01
       coeff_l3(25,105)= -4.3368287385e-02
       coeff_l3(25,106)= -2.1396775544e-01
       coeff_l3(25,107)= -5.3760845214e-02
       coeff_l3(25,108)= -2.8088703752e-02
       coeff_l3(25,109)= -6.5796181560e-02
       coeff_l3(25,110)= -1.5901427716e-02
       coeff_l3(25,111)= -1.2934100628e-01
       coeff_l3(25,112)=  2.8150492907e-01
       coeff_l3(25,113)= -7.8281082213e-02
       coeff_l3(25,114)= -4.1419568658e-01
       coeff_l3(25,115)= -1.0967570543e-01
       coeff_l3(25,116)= -4.9516853690e-01
       coeff_l3(25,117)= -1.1648705006e+00
       coeff_l3(25,118)=  3.2172426581e-02
       coeff_l3(25,119)= -1.2558668852e-01
       coeff_l3(25,120)= -2.6389715075e-01
       coeff_l3(25,121)= -7.5493313372e-02
       coeff_l3(25,122)= -1.8361386657e-01
       coeff_l3(25,123)= -1.4467814565e-01
       coeff_l3(25,124)= -7.2371698916e-02
       coeff_l3(25,125)=  1.2628816068e-01
       coeff_l3(26,1)= -8.0798774958e-01
       coeff_l3(26,2)= -1.4943918586e-01
       coeff_l3(26,3)= -4.4009864330e-01
       coeff_l3(26,4)=  1.7144630849e-01
       coeff_l3(26,5)= -4.2413376272e-02
       coeff_l3(26,6)= -1.1498031765e-01
       coeff_l3(26,7)=  2.1708974242e-01
       coeff_l3(26,8)= -1.6862709820e-01
       coeff_l3(26,9)= -3.0228963122e-02
       coeff_l3(26,10)= -1.8874283135e-01
       coeff_l3(26,11)=  2.1540783346e-02
       coeff_l3(26,12)= -9.5373317599e-02
       coeff_l3(26,13)= -1.4353066683e-01
       coeff_l3(26,14)= -4.5515429974e-01
       coeff_l3(26,15)=  7.7267259359e-02
       coeff_l3(26,16)=  5.8236073703e-02
       coeff_l3(26,17)= -7.0684902370e-02
       coeff_l3(26,18)=  4.6905510128e-02
       coeff_l3(26,19)=  2.4293184280e-01
       coeff_l3(26,20)= -2.1708852053e-01
       coeff_l3(26,21)= -3.9987748861e-01
       coeff_l3(26,22)=  1.1441678554e-01
       coeff_l3(26,23)= -1.0247406550e-02
       coeff_l3(26,24)= -6.6876977682e-02
       coeff_l3(26,25)=  4.9089792371e-01
       coeff_l3(26,26)=  5.8734223247e-02
       coeff_l3(26,27)=  2.3358899355e-01
       coeff_l3(26,28)=  9.6047118306e-02
       coeff_l3(26,29)=  4.3218463659e-02
       coeff_l3(26,30)=  7.6526385546e-01
       coeff_l3(26,31)=  4.5845363289e-02
       coeff_l3(26,32)=  6.8539649248e-02
       coeff_l3(26,33)=  1.5481866896e-01
       coeff_l3(26,34)=  1.0155875236e-01
       coeff_l3(26,35)= -3.3913689852e-01
       coeff_l3(26,36)=  4.4501323253e-02
       coeff_l3(26,37)= -9.1587036848e-02
       coeff_l3(26,38)=  2.3362600803e-01
       coeff_l3(26,39)=  4.2484659702e-02
       coeff_l3(26,40)=  4.4033039361e-02
       coeff_l3(26,41)=  2.1824496984e-01
       coeff_l3(26,42)=  1.4037269354e-01
       coeff_l3(26,43)= -2.3638626933e-01
       coeff_l3(26,44)=  1.4471130073e-01
       coeff_l3(26,45)=  2.7203842998e-01
       coeff_l3(26,46)=  3.3763816953e-01
       coeff_l3(26,47)=  1.6107769310e-01
       coeff_l3(26,48)= -1.0770951211e-01
       coeff_l3(26,49)=  1.7258498669e+00
       coeff_l3(26,50)= -3.0521416664e-01
       coeff_l3(26,51)= -2.9188508168e-02
       coeff_l3(26,52)=  5.3889069706e-02
       coeff_l3(26,53)= -4.4328171760e-02
       coeff_l3(26,54)= -6.1008310318e-01
       coeff_l3(26,55)=  3.1956726313e-01
       coeff_l3(26,56)= -1.8893191218e-01
       coeff_l3(26,57)= -2.8355705738e-01
       coeff_l3(26,58)=  3.7283033133e-01
       coeff_l3(26,59)= -8.8769562542e-02
       coeff_l3(26,60)=  2.0831363201e+00
       coeff_l3(26,61)=  1.0816518962e-01
       coeff_l3(26,62)= -9.5605790615e-02
       coeff_l3(26,63)= -1.1957251281e-01
       coeff_l3(26,64)= -9.8698206246e-02
       coeff_l3(26,65)= -5.8967169374e-02
       coeff_l3(26,66)= -1.4726300538e-01
       coeff_l3(26,67)= -1.4631079137e-01
       coeff_l3(26,68)= -3.9169162512e-01
       coeff_l3(26,69)=  6.8564474583e-02
       coeff_l3(26,70)= -2.4918690324e-02
       coeff_l3(26,71)= -1.3546642661e-01
       coeff_l3(26,72)=  3.7612479180e-02
       coeff_l3(26,73)=  7.9501546919e-02
       coeff_l3(26,74)=  1.3349130750e-01
       coeff_l3(26,75)= -7.9400561750e-02
       coeff_l3(26,76)=  1.0812337399e+00
       coeff_l3(26,77)= -2.0947767794e-01
       coeff_l3(26,78)=  2.5279531255e-02
       coeff_l3(26,79)= -2.8067135811e-01
       coeff_l3(26,80)=  7.2521346807e-01
       coeff_l3(26,81)= -1.6778865457e-01
       coeff_l3(26,82)=  2.6744224131e-02
       coeff_l3(26,83)= -2.1830850840e-01
       coeff_l3(26,84)= -8.5482710600e-01
       coeff_l3(26,85)=  9.7524657845e-02
       coeff_l3(26,86)= -3.4687440842e-02
       coeff_l3(26,87)= -8.6585015059e-02
       coeff_l3(26,88)= -5.7937704027e-02
       coeff_l3(26,89)=  6.5035003424e-01
       coeff_l3(26,90)=  3.8582488894e-01
       coeff_l3(26,91)= -1.5174713731e-01
       coeff_l3(26,92)= -2.5028547272e-02
       coeff_l3(26,93)= -1.7124719918e-01
       coeff_l3(26,94)= -2.2017267346e-01
       coeff_l3(26,95)=  1.5248290263e-02
       coeff_l3(26,96)= -1.7969053984e-01
       coeff_l3(26,97)= -2.1129225194e-01
       coeff_l3(26,98)=  1.4310884476e-01
       coeff_l3(26,99)=  5.3552601486e-02
       coeff_l3(26,100)=  3.7867240608e-02
       coeff_l3(26,101)=  6.0672551394e-01
       coeff_l3(26,102)= -7.0820689201e-02
       coeff_l3(26,103)= -7.0772864856e-03
       coeff_l3(26,104)= -3.6541053653e-01
       coeff_l3(26,105)=  3.8223394752e-01
       coeff_l3(26,106)= -3.6976601928e-02
       coeff_l3(26,107)= -1.6359564662e-01
       coeff_l3(26,108)=  8.5864691064e-03
       coeff_l3(26,109)= -1.6075216234e-01
       coeff_l3(26,110)= -2.8113251925e-01
       coeff_l3(26,111)= -1.0212093592e-01
       coeff_l3(26,112)= -3.6073556542e-01
       coeff_l3(26,113)= -1.9998688251e-02
       coeff_l3(26,114)=  2.3685175180e-01
       coeff_l3(26,115)= -6.6994330846e-03
       coeff_l3(26,116)= -2.7037543058e-01
       coeff_l3(26,117)=  1.5535844862e-01
       coeff_l3(26,118)=  1.7276225984e-01
       coeff_l3(26,119)= -1.2006393820e-01
       coeff_l3(26,120)= -1.7892134190e-01
       coeff_l3(26,121)= -1.3563071191e-01
       coeff_l3(26,122)= -4.7178059816e-02
       coeff_l3(26,123)= -1.7201016843e-01
       coeff_l3(26,124)= -2.6132804155e-01
       coeff_l3(26,125)=  3.2655048370e-01
       coeff_l3(27,1)= -1.8075056374e-01
       coeff_l3(27,2)= -8.7527595460e-03
       coeff_l3(27,3)=  3.7151940167e-02
       coeff_l3(27,4)=  7.9102851450e-02
       coeff_l3(27,5)=  7.4424751103e-02
       coeff_l3(27,6)= -3.0703110620e-02
       coeff_l3(27,7)= -4.6472664922e-02
       coeff_l3(27,8)=  1.8814550713e-03
       coeff_l3(27,9)= -4.6869136393e-02
       coeff_l3(27,10)= -1.2724371254e-01
       coeff_l3(27,11)=  9.4078078866e-02
       coeff_l3(27,12)= -1.6952098906e-01
       coeff_l3(27,13)= -1.7566141486e-01
       coeff_l3(27,14)=  6.5190084279e-02
       coeff_l3(27,15)=  6.6653460264e-02
       coeff_l3(27,16)= -1.8855119124e-03
       coeff_l3(27,17)= -5.1445342600e-02
       coeff_l3(27,18)= -1.6084863245e-01
       coeff_l3(27,19)=  8.5989125073e-02
       coeff_l3(27,20)=  1.3820379972e-02
       coeff_l3(27,21)= -6.1963614076e-02
       coeff_l3(27,22)=  4.5995555818e-02
       coeff_l3(27,23)= -2.3133303225e-01
       coeff_l3(27,24)= -7.8341037035e-02
       coeff_l3(27,25)= -1.2651135027e-01
       coeff_l3(27,26)=  1.1920533329e-01
       coeff_l3(27,27)=  1.0642810911e-01
       coeff_l3(27,28)= -1.1774768680e-01
       coeff_l3(27,29)= -3.1415411830e-01
       coeff_l3(27,30)=  7.0601463318e-02
       coeff_l3(27,31)= -6.5812036395e-02
       coeff_l3(27,32)= -5.1028314978e-02
       coeff_l3(27,33)=  1.4384548180e-02
       coeff_l3(27,34)=  1.3329073787e-01
       coeff_l3(27,35)= -1.2528656423e-01
       coeff_l3(27,36)=  3.3285588026e-02
       coeff_l3(27,37)=  7.5613957597e-04
       coeff_l3(27,38)= -2.5172032416e-02
       coeff_l3(27,39)= -1.2500026263e-02
       coeff_l3(27,40)= -1.3914136589e-01
       coeff_l3(27,41)= -1.3777539134e-01
       coeff_l3(27,42)= -9.6003502607e-02
       coeff_l3(27,43)= -6.7068874836e-02
       coeff_l3(27,44)= -2.4560408294e-01
       coeff_l3(27,45)= -7.1330867708e-02
       coeff_l3(27,46)= -1.9643504173e-02
       coeff_l3(27,47)= -5.2396465093e-02
       coeff_l3(27,48)= -2.0194803178e-01
       coeff_l3(27,49)= -4.4153884053e-01
       coeff_l3(27,50)= -1.1269525625e-02
       coeff_l3(27,51)=  4.2574021965e-02
       coeff_l3(27,52)=  8.9660301805e-02
       coeff_l3(27,53)=  5.7966951281e-02
       coeff_l3(27,54)= -2.1774005890e-01
       coeff_l3(27,55)= -1.4497214556e-01
       coeff_l3(27,56)= -8.5184343159e-02
       coeff_l3(27,57)= -6.0827150941e-02
       coeff_l3(27,58)= -3.6981351674e-02
       coeff_l3(27,59)=  2.1082612872e-01
       coeff_l3(27,60)= -2.0861481130e-01
       coeff_l3(27,61)= -1.0868576914e-01
       coeff_l3(27,62)= -9.9280811846e-02
       coeff_l3(27,63)= -1.5195892751e-01
       coeff_l3(27,64)=  1.1845633388e-01
       coeff_l3(27,65)=  9.5136575401e-02
       coeff_l3(27,66)=  7.2562076151e-02
       coeff_l3(27,67)= -1.4916174114e-01
       coeff_l3(27,68)= -1.8883626163e-01
       coeff_l3(27,69)= -1.9835743308e-01
       coeff_l3(27,70)=  5.0452984869e-02
       coeff_l3(27,71)= -1.6424737871e-01
       coeff_l3(27,72)=  3.9885334671e-02
       coeff_l3(27,73)= -4.3458476663e-02
       coeff_l3(27,74)= -8.5909236223e-03
       coeff_l3(27,75)=  1.9755514339e-02
       coeff_l3(27,76)= -4.9908009171e-01
       coeff_l3(27,77)= -2.2180838883e-01
       coeff_l3(27,78)= -1.3117113709e-01
       coeff_l3(27,79)= -9.2129692435e-02
       coeff_l3(27,80)= -1.7512767017e-01
       coeff_l3(27,81)= -2.9074307531e-02
       coeff_l3(27,82)=  8.3275176585e-03
       coeff_l3(27,83)= -1.2077094615e-01
       coeff_l3(27,84)= -3.2232906669e-02
       coeff_l3(27,85)=  4.2651630938e-02
       coeff_l3(27,86)=  3.3067598939e-02
       coeff_l3(27,87)= -1.2714211643e-01
       coeff_l3(27,88)= -3.4727838635e-01
       coeff_l3(27,89)=  7.9330869019e-02
       coeff_l3(27,90)= -1.9845043123e-01
       coeff_l3(27,91)= -1.0650964081e-01
       coeff_l3(27,92)= -1.4355391264e-01
       coeff_l3(27,93)= -2.4899457395e-01
       coeff_l3(27,94)=  1.0298908502e-01
       coeff_l3(27,95)=  1.4171586372e-03
       coeff_l3(27,96)= -6.2115695328e-02
       coeff_l3(27,97)= -6.3247694634e-03
       coeff_l3(27,98)=  1.1451399326e-01
       coeff_l3(27,99)=  9.2191115022e-02
       coeff_l3(27,100)=  4.2320780456e-02
       coeff_l3(27,101)= -1.3609748334e-02
       coeff_l3(27,102)=  8.5685849190e-02
       coeff_l3(27,103)= -7.9126589000e-02
       coeff_l3(27,104)= -1.4065232873e-01
       coeff_l3(27,105)= -2.0348097384e-01
       coeff_l3(27,106)= -1.5220450237e-02
       coeff_l3(27,107)= -9.2043377459e-02
       coeff_l3(27,108)= -4.7253590077e-02
       coeff_l3(27,109)= -8.4454633296e-02
       coeff_l3(27,110)= -8.0778315663e-02
       coeff_l3(27,111)= -5.7015053928e-02
       coeff_l3(27,112)= -1.9805245101e-01
       coeff_l3(27,113)= -7.3386631906e-02
       coeff_l3(27,114)= -1.6496139765e-01
       coeff_l3(27,115)= -2.9988998175e-01
       coeff_l3(27,116)= -7.2260409594e-02
       coeff_l3(27,117)= -7.2049577720e-03
       coeff_l3(27,118)=  1.4618146420e-01
       coeff_l3(27,119)=  1.2354030460e-01
       coeff_l3(27,120)= -6.4810782671e-02
       coeff_l3(27,121)= -1.9073350728e-01
       coeff_l3(27,122)=  3.0235901941e-03
       coeff_l3(27,123)=  1.0723326355e-01
       coeff_l3(27,124)= -1.1856026947e-01
       coeff_l3(27,125)=  1.4494460821e-01
       coeff_l3(28,1)=  5.3225398064e-01
       coeff_l3(28,2)= -1.4904370764e-03
       coeff_l3(28,3)= -3.5916423798e-01
       coeff_l3(28,4)= -7.7697411180e-02
       coeff_l3(28,5)= -5.8494489640e-02
       coeff_l3(28,6)=  8.6284391582e-02
       coeff_l3(28,7)=  1.7156658173e+00
       coeff_l3(28,8)= -1.2858013809e-01
       coeff_l3(28,9)= -1.8572282791e-01
       coeff_l3(28,10)= -4.9098521471e-01
       coeff_l3(28,11)=  1.5308916569e-01
       coeff_l3(28,12)= -3.7915354967e-01
       coeff_l3(28,13)= -4.3625861406e-01
       coeff_l3(28,14)= -1.0193446279e-01
       coeff_l3(28,15)= -2.8005864471e-02
       coeff_l3(28,16)=  1.3764360920e-02
       coeff_l3(28,17)= -3.1452772021e-01
       coeff_l3(28,18)= -3.6587923765e-01
       coeff_l3(28,19)=  2.1186478436e-01
       coeff_l3(28,20)= -7.2516947985e-02
       coeff_l3(28,21)=  2.7796918154e-01
       coeff_l3(28,22)= -2.3215864599e-01
       coeff_l3(28,23)= -1.9494272768e-01
       coeff_l3(28,24)= -1.3255001605e-01
       coeff_l3(28,25)= -1.4623288810e-01
       coeff_l3(28,26)= -1.5985485911e-01
       coeff_l3(28,27)=  2.0456182957e+00
       coeff_l3(28,28)=  1.4719551802e+00
       coeff_l3(28,29)= -1.7809292674e-01
       coeff_l3(28,30)= -9.0508259833e-02
       coeff_l3(28,31)= -5.1423746347e-01
       coeff_l3(28,32)= -1.8372048438e-01
       coeff_l3(28,33)=  1.0541430861e-01
       coeff_l3(28,34)= -3.6831969023e-01
       coeff_l3(28,35)= -1.6274556518e-01
       coeff_l3(28,36)= -2.2097322345e-01
       coeff_l3(28,37)=  1.0365446657e-01
       coeff_l3(28,38)=  6.2304288149e-01
       coeff_l3(28,39)= -2.0806182921e-01
       coeff_l3(28,40)= -2.7413526550e-02
       coeff_l3(28,41)= -9.3101257086e-01
       coeff_l3(28,42)= -1.7262272537e-01
       coeff_l3(28,43)= -2.7819925547e-01
       coeff_l3(28,44)= -1.9861140847e-01
       coeff_l3(28,45)= -1.5774372220e-01
       coeff_l3(28,46)=  8.2809560001e-02
       coeff_l3(28,47)= -7.8856661916e-02
       coeff_l3(28,48)= -3.0493998528e-01
       coeff_l3(28,49)=  2.2928838730e+00
       coeff_l3(28,50)=  6.3694864511e-01
       coeff_l3(28,51)=  8.5903726518e-02
       coeff_l3(28,52)= -9.9231399596e-02
       coeff_l3(28,53)= -1.8225727975e-01
       coeff_l3(28,54)=  7.5231826305e-01
       coeff_l3(28,55)=  2.5017535686e-01
       coeff_l3(28,56)= -1.6230297089e-01
       coeff_l3(28,57)= -1.8671597540e-01
       coeff_l3(28,58)= -1.8621641397e+00
       coeff_l3(28,59)=  5.2042299509e-01
       coeff_l3(28,60)=  5.9130901098e-01
       coeff_l3(28,61)=  2.1191680804e-02
       coeff_l3(28,62)= -1.0970130563e-01
       coeff_l3(28,63)= -5.4572839290e-03
       coeff_l3(28,64)= -2.9579043388e-01
       coeff_l3(28,65)=  7.7914656140e-03
       coeff_l3(28,66)= -1.9271148741e-01
       coeff_l3(28,67)=  3.2336646318e-01
       coeff_l3(28,68)= -2.9420229793e-01
       coeff_l3(28,69)=  5.1295483112e-01
       coeff_l3(28,70)= -5.4639145732e-02
       coeff_l3(28,71)= -4.8631122708e-01
       coeff_l3(28,72)=  6.7453160882e-02
       coeff_l3(28,73)=  1.3181211054e-01
       coeff_l3(28,74)=  5.7371687889e-01
       coeff_l3(28,75)= -1.3687656820e-01
       coeff_l3(28,76)=  1.0469782352e+00
       coeff_l3(28,77)= -4.3912664056e-01
       coeff_l3(28,78)= -1.5723505616e-01
       coeff_l3(28,79)= -1.7519828677e-01
       coeff_l3(28,80)=  2.2641596198e-01
       coeff_l3(28,81)=  2.8602721170e-02
       coeff_l3(28,82)=  1.3045208156e-01
       coeff_l3(28,83)=  2.6118403673e-01
       coeff_l3(28,84)=  1.3698872179e-02
       coeff_l3(28,85)= -5.9989058971e-01
       coeff_l3(28,86)= -1.8663811684e-01
       coeff_l3(28,87)=  9.7450435162e-02
       coeff_l3(28,88)=  3.1368076801e-01
       coeff_l3(28,89)= -3.1135725975e-01
       coeff_l3(28,90)=  2.8676640987e-01
       coeff_l3(28,91)=  5.6485801935e-01
       coeff_l3(28,92)=  6.0563880950e-02
       coeff_l3(28,93)= -2.7540156245e-01
       coeff_l3(28,94)= -3.7465582136e-03
       coeff_l3(28,95)= -7.6274457388e-03
       coeff_l3(28,96)= -1.8376539648e-01
       coeff_l3(28,97)=  2.3779323325e-02
       coeff_l3(28,98)= -2.4908227846e-02
       coeff_l3(28,99)= -1.4331825078e-01
       coeff_l3(28,100)=  1.2934833765e-01
       coeff_l3(28,101)=  5.4511225224e-01
       coeff_l3(28,102)=  1.2224842794e-02
       coeff_l3(28,103)= -3.4517365694e-01
       coeff_l3(28,104)= -1.6968917847e-01
       coeff_l3(28,105)= -4.3959784508e-01
       coeff_l3(28,106)= -1.0835708678e-01
       coeff_l3(28,107)=  2.3686446249e-01
       coeff_l3(28,108)= -6.4830087125e-02
       coeff_l3(28,109)= -7.1909502149e-03
       coeff_l3(28,110)= -5.2928239107e-02
       coeff_l3(28,111)=  1.3658562303e-01
       coeff_l3(28,112)= -5.6384645402e-02
       coeff_l3(28,113)= -1.3526879251e-01
       coeff_l3(28,114)=  1.9413594902e-01
       coeff_l3(28,115)=  6.3311910629e-01
       coeff_l3(28,116)=  3.0266091228e-01
       coeff_l3(28,117)= -1.5794250369e-01
       coeff_l3(28,118)=  3.9628571272e-01
       coeff_l3(28,119)=  1.6659334302e-01
       coeff_l3(28,120)= -1.4169548452e-01
       coeff_l3(28,121)=  3.1044250727e-01
       coeff_l3(28,122)=  9.6071876585e-02
       coeff_l3(28,123)=  7.1520850062e-02
       coeff_l3(28,124)= -1.2452444434e-01
       coeff_l3(28,125)= -6.1113249511e-02
       coeff_l3(29,1)= -1.7246462405e-01
       coeff_l3(29,2)= -1.0069729388e-01
       coeff_l3(29,3)=  5.1488850266e-02
       coeff_l3(29,4)= -1.3626036048e-01
       coeff_l3(29,5)=  1.2797270715e-01
       coeff_l3(29,6)=  2.0229089260e-01
       coeff_l3(29,7)=  4.1574714705e-03
       coeff_l3(29,8)=  1.3536658883e-01
       coeff_l3(29,9)=  1.9179758430e-01
       coeff_l3(29,10)=  1.3700953126e-01
       coeff_l3(29,11)= -3.0845781788e-02
       coeff_l3(29,12)=  1.3137595356e-01
       coeff_l3(29,13)= -4.5729782432e-02
       coeff_l3(29,14)= -1.0073792189e-02
       coeff_l3(29,15)=  1.5197135508e-01
       coeff_l3(29,16)=  1.1035794020e-01
       coeff_l3(29,17)= -4.2051129043e-02
       coeff_l3(29,18)=  1.8538306653e-01
       coeff_l3(29,19)=  1.2508358061e-01
       coeff_l3(29,20)=  4.8660587519e-02
       coeff_l3(29,21)=  4.7487191856e-02
       coeff_l3(29,22)= -1.4376643300e-01
       coeff_l3(29,23)=  2.4619202316e-01
       coeff_l3(29,24)= -1.1911065131e-01
       coeff_l3(29,25)= -9.0882785618e-02
       coeff_l3(29,26)= -4.3721672148e-02
       coeff_l3(29,27)=  1.3211591542e-01
       coeff_l3(29,28)=  2.9231914878e-01
       coeff_l3(29,29)= -1.0077681392e-01
       coeff_l3(29,30)=  1.2381271459e-02
       coeff_l3(29,31)= -1.0343028605e-01
       coeff_l3(29,32)= -1.2938890606e-02
       coeff_l3(29,33)=  2.4045486934e-03
       coeff_l3(29,34)=  4.6945583075e-02
       coeff_l3(29,35)= -2.5563699007e-01
       coeff_l3(29,36)=  3.1202938408e-02
       coeff_l3(29,37)= -5.4549731314e-02
       coeff_l3(29,38)=  2.5501915812e-01
       coeff_l3(29,39)= -9.6041783690e-02
       coeff_l3(29,40)=  1.6769396141e-02
       coeff_l3(29,41)=  2.3032826185e-01
       coeff_l3(29,42)=  1.4303751290e-01
       coeff_l3(29,43)=  7.0607177913e-02
       coeff_l3(29,44)=  1.1359769851e-01
       coeff_l3(29,45)=  9.0336970985e-02
       coeff_l3(29,46)= -2.8503373265e-01
       coeff_l3(29,47)=  3.2922007143e-02
       coeff_l3(29,48)= -4.4022962451e-02
       coeff_l3(29,49)=  1.7130499706e-02
       coeff_l3(29,50)= -1.0644570738e-01
       coeff_l3(29,51)= -3.2504647970e-02
       coeff_l3(29,52)= -1.3390308619e-01
       coeff_l3(29,53)=  4.4810485095e-02
       coeff_l3(29,54)=  2.9629325867e-01
       coeff_l3(29,55)= -5.0385270268e-02
       coeff_l3(29,56)=  8.6463421583e-02
       coeff_l3(29,57)=  1.8005834520e-01
       coeff_l3(29,58)=  1.5636530519e-01
       coeff_l3(29,59)= -1.7343702912e-01
       coeff_l3(29,60)= -3.5468596220e-01
       coeff_l3(29,61)=  2.4326678365e-02
       coeff_l3(29,62)=  2.1616323292e-01
       coeff_l3(29,63)= -6.6260740161e-02
       coeff_l3(29,64)=  1.0252648965e-02
       coeff_l3(29,65)= -1.4356388152e-01
       coeff_l3(29,66)=  1.4600324631e-01
       coeff_l3(29,67)=  2.7326959372e-01
       coeff_l3(29,68)= -2.0569722354e-01
       coeff_l3(29,69)= -9.4331070781e-02
       coeff_l3(29,70)=  5.9460442513e-02
       coeff_l3(29,71)=  2.6830926538e-01
       coeff_l3(29,72)=  1.8282723427e-01
       coeff_l3(29,73)=  1.2511223555e-01
       coeff_l3(29,74)= -1.4898318052e-01
       coeff_l3(29,75)=  3.1517475843e-02
       coeff_l3(29,76)= -1.1645960808e-01
       coeff_l3(29,77)=  1.1417070031e-01
       coeff_l3(29,78)= -1.2296430767e-01
       coeff_l3(29,79)=  1.1254111677e-01
       coeff_l3(29,80)= -1.7822641879e-02
       coeff_l3(29,81)=  3.4707054496e-01
       coeff_l3(29,82)=  3.4665878862e-02
       coeff_l3(29,83)=  7.8830249608e-02
       coeff_l3(29,84)=  2.8608130291e-02
       coeff_l3(29,85)=  2.5843268633e-01
       coeff_l3(29,86)= -1.7184817791e-01
       coeff_l3(29,87)=  1.2356929481e-01
       coeff_l3(29,88)=  1.7147216201e-01
       coeff_l3(29,89)= -1.0283602774e-01
       coeff_l3(29,90)=  5.9646815062e-02
       coeff_l3(29,91)=  5.8297194541e-02
       coeff_l3(29,92)= -3.1634392217e-03
       coeff_l3(29,93)= -8.5243530571e-02
       coeff_l3(29,94)= -9.0615212917e-02
       coeff_l3(29,95)=  1.3395865262e-01
       coeff_l3(29,96)= -4.6815048903e-02
       coeff_l3(29,97)= -9.9253118038e-02
       coeff_l3(29,98)=  3.4515562654e-01
       coeff_l3(29,99)=  5.2936924621e-03
       coeff_l3(29,100)=  4.7464370728e-02
       coeff_l3(29,101)=  1.4744721353e-01
       coeff_l3(29,102)= -1.2404428422e-01
       coeff_l3(29,103)=  7.9540804029e-02
       coeff_l3(29,104)=  1.6124451160e-01
       coeff_l3(29,105)=  1.0535127670e-01
       coeff_l3(29,106)=  2.9068218544e-02
       coeff_l3(29,107)=  2.5042202324e-02
       coeff_l3(29,108)= -1.5744632483e-01
       coeff_l3(29,109)= -1.3004122674e-01
       coeff_l3(29,110)=  7.5549937785e-02
       coeff_l3(29,111)=  4.1409485857e-04
       coeff_l3(29,112)= -1.6421729326e-01
       coeff_l3(29,113)= -1.6138900816e-01
       coeff_l3(29,114)=  1.9190704823e-01
       coeff_l3(29,115)= -2.5899171829e-01
       coeff_l3(29,116)=  2.4824588001e-01
       coeff_l3(29,117)= -1.6468745470e-01
       coeff_l3(29,118)=  2.5987941772e-02
       coeff_l3(29,119)=  5.6739810854e-02
       coeff_l3(29,120)= -5.5204398930e-02
       coeff_l3(29,121)=  2.7575010434e-02
       coeff_l3(29,122)=  6.7311845720e-02
       coeff_l3(29,123)=  8.7061859667e-02
       coeff_l3(29,124)= -1.3132224977e-01
       coeff_l3(29,125)=  7.6680235565e-02
       coeff_l3(30,1)= -2.5305703282e-01
       coeff_l3(30,2)= -3.7694430351e-01
       coeff_l3(30,3)=  1.2065098435e-01
       coeff_l3(30,4)=  1.6284167767e-01
       coeff_l3(30,5)=  7.2548329830e-02
       coeff_l3(30,6)= -1.1695190333e-02
       coeff_l3(30,7)= -2.1640846133e-01
       coeff_l3(30,8)=  2.4096632842e-03
       coeff_l3(30,9)=  1.1581705511e-01
       coeff_l3(30,10)= -2.5835311413e-01
       coeff_l3(30,11)=  6.9208577275e-02
       coeff_l3(30,12)= -5.3009903058e-03
       coeff_l3(30,13)= -1.8185067177e-01
       coeff_l3(30,14)= -7.8381113708e-02
       coeff_l3(30,15)=  8.9819364250e-02
       coeff_l3(30,16)=  1.9115157425e-01
       coeff_l3(30,17)= -1.0612574406e-02
       coeff_l3(30,18)= -2.4727647007e-01
       coeff_l3(30,19)= -1.9397202134e-01
       coeff_l3(30,20)= -1.1994291842e-01
       coeff_l3(30,21)= -1.0149025917e-01
       coeff_l3(30,22)= -1.4914418757e-01
       coeff_l3(30,23)= -5.4103046656e-02
       coeff_l3(30,24)= -3.7201857567e-01
       coeff_l3(30,25)= -1.6657839715e-01
       coeff_l3(30,26)= -4.0435664356e-02
       coeff_l3(30,27)=  2.6046195626e-01
       coeff_l3(30,28)=  4.0591776371e-02
       coeff_l3(30,29)= -2.4742706120e-01
       coeff_l3(30,30)= -4.5017469674e-02
       coeff_l3(30,31)= -1.4316961169e-02
       coeff_l3(30,32)= -2.4356181920e-01
       coeff_l3(30,33)=  8.8532343507e-02
       coeff_l3(30,34)= -1.5745027363e-01
       coeff_l3(30,35)= -1.5965756774e-01
       coeff_l3(30,36)= -1.3083279133e-01
       coeff_l3(30,37)= -2.3211149871e-01
       coeff_l3(30,38)= -1.5283347666e-01
       coeff_l3(30,39)=  2.9392709257e-04
       coeff_l3(30,40)= -1.0221839696e-01
       coeff_l3(30,41)=  2.2957059741e-01
       coeff_l3(30,42)= -1.1012858152e-01
       coeff_l3(30,43)=  5.9911351651e-02
       coeff_l3(30,44)= -1.8457502127e-01
       coeff_l3(30,45)=  1.7594091594e-02
       coeff_l3(30,46)= -1.3044954836e-01
       coeff_l3(30,47)=  6.8357758224e-02
       coeff_l3(30,48)= -1.4274044335e-01
       coeff_l3(30,49)= -2.2194011137e-02
       coeff_l3(30,50)= -8.1715777516e-02
       coeff_l3(30,51)= -8.8774308562e-02
       coeff_l3(30,52)=  8.3962798119e-02
       coeff_l3(30,53)= -3.8855634630e-02
       coeff_l3(30,54)=  2.3863527179e-01
       coeff_l3(30,55)=  3.5586092621e-02
       coeff_l3(30,56)= -1.0380617529e-01
       coeff_l3(30,57)=  1.7908641696e-01
       coeff_l3(30,58)=  1.8073402345e-01
       coeff_l3(30,59)=  1.5632510185e-01
       coeff_l3(30,60)= -2.8590521216e-01
       coeff_l3(30,61)= -7.9311899841e-02
       coeff_l3(30,62)= -1.3999694586e-01
       coeff_l3(30,63)= -5.0147236325e-03
       coeff_l3(30,64)=  7.9410001636e-02
       coeff_l3(30,65)= -6.4923964441e-02
       coeff_l3(30,66)= -1.1312225461e-01
       coeff_l3(30,67)=  3.2376474142e-01
       coeff_l3(30,68)= -1.2280370295e-01
       coeff_l3(30,69)= -3.1855237484e-01
       coeff_l3(30,70)= -2.2164787352e-01
       coeff_l3(30,71)= -1.8254871666e-01
       coeff_l3(30,72)= -1.7020063475e-02
       coeff_l3(30,73)=  3.5101328045e-02
       coeff_l3(30,74)=  2.7671045065e-01
       coeff_l3(30,75)= -1.6491429508e-01
       coeff_l3(30,76)= -1.5757966042e-01
       coeff_l3(30,77)=  9.8158903420e-02
       coeff_l3(30,78)= -1.8156200647e-01
       coeff_l3(30,79)=  1.6956590116e-01
       coeff_l3(30,80)= -1.8953180313e-01
       coeff_l3(30,81)=  1.4195851982e-01
       coeff_l3(30,82)=  9.3939471990e-03
       coeff_l3(30,83)= -1.2051070482e-01
       coeff_l3(30,84)= -1.9243070483e-01
       coeff_l3(30,85)= -1.8044069409e-01
       coeff_l3(30,86)= -1.4830410480e-01
       coeff_l3(30,87)= -7.6622240245e-02
       coeff_l3(30,88)= -1.2412367761e-01
       coeff_l3(30,89)= -3.6707948893e-02
       coeff_l3(30,90)=  3.5259339958e-02
       coeff_l3(30,91)= -8.9565180242e-03
       coeff_l3(30,92)=  6.6447168589e-02
       coeff_l3(30,93)=  7.6817139983e-02
       coeff_l3(30,94)=  5.7741597295e-02
       coeff_l3(30,95)= -2.6162207127e-01
       coeff_l3(30,96)=  9.9335268140e-02
       coeff_l3(30,97)= -6.3510529697e-02
       coeff_l3(30,98)=  9.9586293101e-02
       coeff_l3(30,99)= -1.7374230921e-01
       coeff_l3(30,100)= -2.0838701725e-01
       coeff_l3(30,101)=  4.5415606350e-02
       coeff_l3(30,102)= -1.7952638865e-01
       coeff_l3(30,103)=  1.9155001268e-02
       coeff_l3(30,104)= -1.6409900784e-01
       coeff_l3(30,105)=  1.6430908442e-01
       coeff_l3(30,106)= -4.4310808182e-02
       coeff_l3(30,107)= -1.4584887028e-01
       coeff_l3(30,108)=  3.2370027155e-02
       coeff_l3(30,109)= -1.6259165108e-01
       coeff_l3(30,110)=  1.3584089279e-01
       coeff_l3(30,111)= -1.3985896111e-01
       coeff_l3(30,112)= -1.2726871669e-01
       coeff_l3(30,113)= -2.9337105155e-01
       coeff_l3(30,114)=  1.5486569703e-01
       coeff_l3(30,115)= -2.0469108224e-01
       coeff_l3(30,116)=  2.2311178967e-02
       coeff_l3(30,117)= -1.3499696739e-02
       coeff_l3(30,118)= -2.3832049221e-02
       coeff_l3(30,119)= -5.0130249001e-03
       coeff_l3(30,120)= -1.3535010815e-01
       coeff_l3(30,121)=  2.9280392453e-02
       coeff_l3(30,122)=  1.3931916654e-01
       coeff_l3(30,123)= -2.0315660536e-01
       coeff_l3(30,124)= -2.1302755177e-01
       coeff_l3(30,125)=  4.6447787434e-02
       coeff_l3(31,1)= -1.8473705649e-01
       coeff_l3(31,2)=  1.3020673394e-01
       coeff_l3(31,3)=  2.4588121474e-01
       coeff_l3(31,4)=  7.7791921794e-02
       coeff_l3(31,5)= -1.6372515261e-01
       coeff_l3(31,6)= -1.7772465944e-01
       coeff_l3(31,7)=  1.0478745699e+00
       coeff_l3(31,8)= -1.7957526445e-01
       coeff_l3(31,9)=  2.2854857147e-02
       coeff_l3(31,10)= -7.9775057733e-02
       coeff_l3(31,11)= -2.1450485289e-01
       coeff_l3(31,12)= -2.2501932085e-01
       coeff_l3(31,13)= -1.7668578029e-01
       coeff_l3(31,14)= -8.2571797073e-02
       coeff_l3(31,15)=  9.9441021681e-02
       coeff_l3(31,16)= -7.9552177340e-03
       coeff_l3(31,17)= -6.5920218825e-02
       coeff_l3(31,18)= -6.3583123684e-01
       coeff_l3(31,19)=  8.9044682682e-02
       coeff_l3(31,20)= -1.6150979698e-01
       coeff_l3(31,21)=  2.3865078390e-01
       coeff_l3(31,22)=  2.3770166934e-01
       coeff_l3(31,23)= -1.5710833669e-01
       coeff_l3(31,24)=  3.7893861532e-02
       coeff_l3(31,25)= -2.0706157386e-01
       coeff_l3(31,26)=  2.5079369545e-02
       coeff_l3(31,27)=  1.5911630392e+00
       coeff_l3(31,28)=  1.2135367393e+00
       coeff_l3(31,29)= -2.5749686360e-01
       coeff_l3(31,30)= -1.3815101981e-01
       coeff_l3(31,31)=  2.9561430216e-02
       coeff_l3(31,32)= -6.0321320780e-03
       coeff_l3(31,33)= -8.6407944560e-02
       coeff_l3(31,34)=  1.1053717695e-02
       coeff_l3(31,35)=  3.1072816253e-01
       coeff_l3(31,36)=  1.3340817392e-01
       coeff_l3(31,37)=  1.3317885995e-01
       coeff_l3(31,38)= -5.1338100433e-01
       coeff_l3(31,39)=  9.5991112292e-02
       coeff_l3(31,40)=  1.0580683127e-02
       coeff_l3(31,41)=  5.9466284513e-01
       coeff_l3(31,42)=  8.2257010043e-02
       coeff_l3(31,43)= -9.9423818290e-02
       coeff_l3(31,44)=  1.8684018403e-02
       coeff_l3(31,45)= -9.3168757856e-02
       coeff_l3(31,46)= -1.5002089739e-01
       coeff_l3(31,47)=  5.5889006704e-02
       coeff_l3(31,48)= -4.2914906144e-01
       coeff_l3(31,49)=  1.0333095789e+00
       coeff_l3(31,50)= -9.1414175928e-02
       coeff_l3(31,51)= -1.5911537409e-01
       coeff_l3(31,52)=  2.6872027665e-02
       coeff_l3(31,53)= -1.8650229275e-01
       coeff_l3(31,54)=  6.7202997208e-01
       coeff_l3(31,55)=  1.1946146190e-01
       coeff_l3(31,56)=  4.0396158583e-03
       coeff_l3(31,57)= -2.7041566372e-01
       coeff_l3(31,58)= -5.2310138941e-02
       coeff_l3(31,59)= -6.8386667967e-01
       coeff_l3(31,60)= -6.2256228924e-01
       coeff_l3(31,61)=  1.5954235569e-02
       coeff_l3(31,62)=  1.0895833373e-01
       coeff_l3(31,63)=  4.5728716999e-02
       coeff_l3(31,64)= -6.2564067543e-02
       coeff_l3(31,65)=  6.7248180509e-02
       coeff_l3(31,66)= -1.3761949539e-01
       coeff_l3(31,67)=  5.0173246861e-01
       coeff_l3(31,68)= -1.6268207133e-01
       coeff_l3(31,69)= -2.6261491701e-02
       coeff_l3(31,70)= -9.2452064157e-02
       coeff_l3(31,71)= -2.5259318948e-01
       coeff_l3(31,72)= -1.0159134865e-01
       coeff_l3(31,73)=  5.5115401745e-02
       coeff_l3(31,74)=  3.3685350418e-01
       coeff_l3(31,75)= -1.6939234734e-01
       coeff_l3(31,76)=  2.6170122623e-01
       coeff_l3(31,77)=  7.2249203920e-02
       coeff_l3(31,78)=  2.9715567362e-03
       coeff_l3(31,79)=  3.3089201897e-02
       coeff_l3(31,80)=  5.3946256638e-01
       coeff_l3(31,81)= -4.6414190531e-01
       coeff_l3(31,82)= -8.2665294409e-02
       coeff_l3(31,83)= -2.0496161282e-01
       coeff_l3(31,84)=  1.5872816741e-01
       coeff_l3(31,85)= -5.0041830540e-01
       coeff_l3(31,86)= -1.3959005475e-01
       coeff_l3(31,87)=  1.2095348909e-02
       coeff_l3(31,88)=  1.2742783874e-03
       coeff_l3(31,89)=  8.6237035692e-02
       coeff_l3(31,90)= -1.1528337002e-01
       coeff_l3(31,91)=  1.5663553029e-02
       coeff_l3(31,92)= -5.1154505461e-02
       coeff_l3(31,93)= -6.5043032169e-02
       coeff_l3(31,94)=  4.5798014849e-02
       coeff_l3(31,95)=  6.1117086560e-02
       coeff_l3(31,96)= -2.4974210188e-02
       coeff_l3(31,97)=  2.0626110490e-03
       coeff_l3(31,98)= -2.0841546357e-01
       coeff_l3(31,99)= -1.1926901340e-01
       coeff_l3(31,100)= -2.4191897362e-02
       coeff_l3(31,101)= -7.5775422156e-02
       coeff_l3(31,102)= -1.3036707044e-01
       coeff_l3(31,103)= -3.6529269069e-02
       coeff_l3(31,104)= -1.1916647106e-01
       coeff_l3(31,105)=  3.2593980432e-01
       coeff_l3(31,106)=  5.8072786778e-02
       coeff_l3(31,107)= -1.7403826118e-02
       coeff_l3(31,108)=  8.4284637123e-03
       coeff_l3(31,109)=  1.3229801320e-02
       coeff_l3(31,110)= -3.8137912750e-02
       coeff_l3(31,111)=  8.5929460824e-02
       coeff_l3(31,112)= -4.0643531084e-01
       coeff_l3(31,113)= -4.2727321386e-02
       coeff_l3(31,114)=  4.5229122043e-01
       coeff_l3(31,115)=  6.4752072096e-02
       coeff_l3(31,116)= -2.6254452765e-02
       coeff_l3(31,117)= -6.7089013755e-02
       coeff_l3(31,118)=  5.9092569351e-01
       coeff_l3(31,119)= -6.5350256860e-02
       coeff_l3(31,120)= -1.7248749733e-01
       coeff_l3(31,121)= -7.8290820122e-01
       coeff_l3(31,122)= -8.9092776761e-05
       coeff_l3(31,123)=  3.2322816551e-02
       coeff_l3(31,124)= -1.2584273517e-01
       coeff_l3(31,125)=  1.3450472057e-01
       coeff_l3(32,1)= -3.3853855729e-01
       coeff_l3(32,2)=  2.3761088029e-02
       coeff_l3(32,3)= -4.0911410004e-02
       coeff_l3(32,4)=  3.5261787474e-02
       coeff_l3(32,5)=  1.4999562502e-01
       coeff_l3(32,6)=  2.2737330198e-01
       coeff_l3(32,7)=  1.0982222110e-01
       coeff_l3(32,8)= -1.7263910174e-01
       coeff_l3(32,9)=  3.3857140690e-02
       coeff_l3(32,10)= -2.0832348615e-02
       coeff_l3(32,11)=  1.7135231197e-01
       coeff_l3(32,12)= -6.7987702787e-02
       coeff_l3(32,13)= -8.4649592638e-02
       coeff_l3(32,14)= -9.5107123256e-02
       coeff_l3(32,15)= -1.3101992197e-02
       coeff_l3(32,16)= -1.9551740587e-01
       coeff_l3(32,17)=  4.6166669577e-02
       coeff_l3(32,18)= -5.1026839018e-01
       coeff_l3(32,19)= -9.3269823119e-03
       coeff_l3(32,20)= -7.5668476522e-02
       coeff_l3(32,21)= -2.2460865974e-01
       coeff_l3(32,22)= -1.9256953383e-03
       coeff_l3(32,23)= -1.2041374296e-01
       coeff_l3(32,24)= -1.8907365203e-01
       coeff_l3(32,25)= -4.0342520922e-02
       coeff_l3(32,26)= -1.0947179049e-01
       coeff_l3(32,27)=  1.6818854213e-01
       coeff_l3(32,28)= -2.7834884822e-02
       coeff_l3(32,29)= -4.9004915357e-01
       coeff_l3(32,30)= -3.1993719935e-01
       coeff_l3(32,31)= -5.1421646029e-02
       coeff_l3(32,32)= -6.2514387071e-02
       coeff_l3(32,33)= -5.4119909182e-03
       coeff_l3(32,34)= -6.6790975630e-02
       coeff_l3(32,35)=  1.3573446870e-01
       coeff_l3(32,36)=  1.9059500098e-01
       coeff_l3(32,37)=  7.9133555293e-02
       coeff_l3(32,38)= -1.2194156647e-01
       coeff_l3(32,39)=  8.1192702055e-02
       coeff_l3(32,40)= -2.1348339319e-01
       coeff_l3(32,41)= -6.9454735518e-01
       coeff_l3(32,42)= -6.3042119145e-02
       coeff_l3(32,43)=  5.6770626456e-02
       coeff_l3(32,44)= -1.2749604881e-01
       coeff_l3(32,45)=  6.3663698733e-02
       coeff_l3(32,46)= -9.8097987473e-02
       coeff_l3(32,47)=  7.4106849730e-02
       coeff_l3(32,48)=  6.1633344740e-02
       coeff_l3(32,49)= -7.1770501137e-01
       coeff_l3(32,50)= -7.2713255882e-02
       coeff_l3(32,51)= -1.5794870257e-01
       coeff_l3(32,52)= -9.1549992561e-02
       coeff_l3(32,53)= -2.1065981686e-01
       coeff_l3(32,54)= -1.9872833788e-01
       coeff_l3(32,55)= -2.1465986967e-02
       coeff_l3(32,56)= -2.4807296693e-02
       coeff_l3(32,57)= -1.8920015544e-02
       coeff_l3(32,58)= -1.8150112033e-01
       coeff_l3(32,59)= -1.5538378060e-01
       coeff_l3(32,60)= -1.4437878132e-01
       coeff_l3(32,61)=  1.2118789554e-01
       coeff_l3(32,62)=  4.5765366405e-02
       coeff_l3(32,63)= -5.4220795631e-02
       coeff_l3(32,64)= -3.5614527762e-02
       coeff_l3(32,65)= -5.9844467789e-02
       coeff_l3(32,66)= -5.7151671499e-02
       coeff_l3(32,67)= -2.0830151439e-01
       coeff_l3(32,68)= -8.3372071385e-02
       coeff_l3(32,69)= -2.5059831142e-01
       coeff_l3(32,70)= -5.6359868497e-02
       coeff_l3(32,71)= -5.8751606941e-01
       coeff_l3(32,72)=  5.5189564824e-02
       coeff_l3(32,73)=  9.9724300206e-02
       coeff_l3(32,74)=  1.9728614017e-02
       coeff_l3(32,75)=  8.8911317289e-03
       coeff_l3(32,76)= -1.4543141425e-01
       coeff_l3(32,77)= -1.6982659698e-01
       coeff_l3(32,78)=  6.8154975772e-02
       coeff_l3(32,79)= -1.3372184336e-01
       coeff_l3(32,80)= -2.6985958219e-01
       coeff_l3(32,81)= -2.1705903113e-01
       coeff_l3(32,82)=  3.5052116960e-02
       coeff_l3(32,83)=  4.1271217167e-02
       coeff_l3(32,84)= -8.0898195505e-02
       coeff_l3(32,85)= -3.7602627277e-01
       coeff_l3(32,86)= -5.9389457107e-02
       coeff_l3(32,87)= -2.0556390285e-02
       coeff_l3(32,88)= -4.4284570217e-01
       coeff_l3(32,89)= -1.8084888160e-01
       coeff_l3(32,90)= -2.6531520486e-01
       coeff_l3(32,91)= -3.4274491668e-01
       coeff_l3(32,92)= -3.3355006599e-04
       coeff_l3(32,93)=  7.0900082588e-02
       coeff_l3(32,94)= -7.4260026217e-02
       coeff_l3(32,95)= -2.5404226035e-02
       coeff_l3(32,96)= -3.1402382255e-01
       coeff_l3(32,97)=  2.1630467474e-01
       coeff_l3(32,98)= -1.4379711449e-01
       coeff_l3(32,99)= -2.0479857922e-01
       coeff_l3(32,100)= -7.5947716832e-02
       coeff_l3(32,101)=  1.0846636444e-01
       coeff_l3(32,102)= -1.6122447327e-02
       coeff_l3(32,103)=  2.8821315616e-02
       coeff_l3(32,104)= -1.5279343724e-01
       coeff_l3(32,105)= -2.2178338468e-01
       coeff_l3(32,106)=  8.9503817260e-02
       coeff_l3(32,107)= -3.8856763858e-03
       coeff_l3(32,108)=  1.1451971531e-01
       coeff_l3(32,109)= -1.5742518008e-01
       coeff_l3(32,110)= -1.3095675409e-01
       coeff_l3(32,111)= -1.6230586171e-01
       coeff_l3(32,112)= -5.2548265457e-01
       coeff_l3(32,113)= -8.2761600614e-02
       coeff_l3(32,114)=  1.4516644180e-01
       coeff_l3(32,115)= -1.1228947900e-02
       coeff_l3(32,116)= -9.2827081680e-02
       coeff_l3(32,117)= -2.8235384822e-01
       coeff_l3(32,118)=  1.2816224992e-01
       coeff_l3(32,119)= -7.5920663774e-02
       coeff_l3(32,120)=  1.4622176997e-02
       coeff_l3(32,121)= -6.2238854170e-01
       coeff_l3(32,122)= -3.5451937001e-03
       coeff_l3(32,123)= -1.9098481163e-02
       coeff_l3(32,124)=  1.0557162762e-01
       coeff_l3(32,125)= -1.3376213610e-01
       coeff_l3(33,1)= -6.7410342395e-02
       coeff_l3(33,2)= -1.4208018780e-01
       coeff_l3(33,3)= -2.8745049238e-01
       coeff_l3(33,4)= -1.3072369993e-01
       coeff_l3(33,5)=  1.1866218597e-01
       coeff_l3(33,6)= -1.9803945720e-01
       coeff_l3(33,7)=  1.4333041385e-02
       coeff_l3(33,8)= -7.5348839164e-02
       coeff_l3(33,9)= -6.9716591388e-03
       coeff_l3(33,10)=  5.2299026400e-02
       coeff_l3(33,11)=  5.4787644185e-03
       coeff_l3(33,12)= -5.6430649012e-02
       coeff_l3(33,13)=  1.5224187076e-01
       coeff_l3(33,14)=  1.7306014895e-01
       coeff_l3(33,15)= -4.9619600177e-02
       coeff_l3(33,16)=  1.1812216043e-01
       coeff_l3(33,17)=  1.0454637557e-01
       coeff_l3(33,18)=  9.3576319516e-02
       coeff_l3(33,19)= -1.6742874682e-01
       coeff_l3(33,20)=  2.4094102904e-02
       coeff_l3(33,21)=  1.9290124997e-02
       coeff_l3(33,22)= -1.4460040629e-01
       coeff_l3(33,23)= -9.6433915198e-02
       coeff_l3(33,24)=  7.1991033852e-02
       coeff_l3(33,25)= -1.6237342358e-01
       coeff_l3(33,26)= -4.5054420829e-02
       coeff_l3(33,27)= -2.1703170240e-01
       coeff_l3(33,28)=  1.4386545122e-01
       coeff_l3(33,29)= -3.1895777583e-01
       coeff_l3(33,30)= -1.4936579764e-01
       coeff_l3(33,31)=  1.1022127420e-01
       coeff_l3(33,32)=  2.0030017477e-03
       coeff_l3(33,33)= -5.2491907030e-02
       coeff_l3(33,34)= -4.7181814909e-02
       coeff_l3(33,35)= -2.1738424897e-01
       coeff_l3(33,36)=  8.8847547770e-02
       coeff_l3(33,37)= -2.1196400747e-02
       coeff_l3(33,38)= -3.0743414164e-01
       coeff_l3(33,39)= -4.2652614415e-02
       coeff_l3(33,40)= -5.6869607419e-02
       coeff_l3(33,41)=  4.1663028300e-02
       coeff_l3(33,42)=  6.5524764359e-02
       coeff_l3(33,43)=  1.4745649695e-01
       coeff_l3(33,44)=  1.9095818698e-01
       coeff_l3(33,45)= -5.7056106627e-02
       coeff_l3(33,46)= -5.8310475200e-02
       coeff_l3(33,47)= -1.7200461030e-01
       coeff_l3(33,48)= -1.3394284248e-01
       coeff_l3(33,49)= -2.5587040186e-01
       coeff_l3(33,50)=  2.1875426173e-01
       coeff_l3(33,51)=  8.0981977284e-02
       coeff_l3(33,52)= -9.0468361974e-02
       coeff_l3(33,53)=  8.7277263403e-02
       coeff_l3(33,54)= -1.0823545046e-02
       coeff_l3(33,55)= -9.6735708416e-02
       coeff_l3(33,56)= -5.7583477348e-02
       coeff_l3(33,57)= -9.4340831041e-02
       coeff_l3(33,58)= -3.3026155829e-01
       coeff_l3(33,59)=  4.1239678860e-02
       coeff_l3(33,60)= -1.4258636236e+00
       coeff_l3(33,61)=  9.8460186273e-03
       coeff_l3(33,62)=  7.2863891721e-02
       coeff_l3(33,63)=  1.4219996333e-01
       coeff_l3(33,64)=  6.9048814476e-02
       coeff_l3(33,65)= -5.3232658654e-02
       coeff_l3(33,66)= -1.2552762032e-01
       coeff_l3(33,67)= -2.1798481047e-01
       coeff_l3(33,68)=  3.2190027833e-01
       coeff_l3(33,69)= -4.5761264861e-02
       coeff_l3(33,70)= -1.1486446112e-01
       coeff_l3(33,71)=  1.7385295033e-01
       coeff_l3(33,72)=  5.7985730469e-02
       coeff_l3(33,73)= -4.0118426085e-02
       coeff_l3(33,74)= -1.0323158652e-01
       coeff_l3(33,75)=  1.7557872832e-01
       coeff_l3(33,76)= -1.8543222547e-01
       coeff_l3(33,77)= -3.7618018687e-02
       coeff_l3(33,78)=  2.0736502111e-01
       coeff_l3(33,79)=  4.1511405259e-02
       coeff_l3(33,80)=  2.4935828522e-02
       coeff_l3(33,81)= -2.9346886277e-01
       coeff_l3(33,82)=  2.4066571146e-02
       coeff_l3(33,83)= -1.2611263432e-02
       coeff_l3(33,84)=  1.6822898760e-03
       coeff_l3(33,85)=  7.3925621808e-02
       coeff_l3(33,86)=  2.5997176766e-01
       coeff_l3(33,87)=  1.6972687840e-01
       coeff_l3(33,88)= -1.8018364906e-01
       coeff_l3(33,89)= -2.4482558668e-01
       coeff_l3(33,90)= -3.4532693028e-01
       coeff_l3(33,91)= -4.2104292661e-02
       coeff_l3(33,92)= -5.2990958095e-02
       coeff_l3(33,93)=  7.7320151031e-02
       coeff_l3(33,94)= -8.6730808020e-02
       coeff_l3(33,95)= -5.2527159452e-02
       coeff_l3(33,96)= -8.0510176718e-02
       coeff_l3(33,97)= -8.1762358546e-02
       coeff_l3(33,98)=  2.7788801119e-02
       coeff_l3(33,99)= -1.7497116700e-02
       coeff_l3(33,100)= -2.1465928853e-01
       coeff_l3(33,101)= -2.9064235091e-01
       coeff_l3(33,102)=  4.9496650696e-02
       coeff_l3(33,103)=  3.3960819244e-02
       coeff_l3(33,104)= -1.1722476780e-01
       coeff_l3(33,105)=  1.4864626527e-01
       coeff_l3(33,106)= -2.5823788717e-02
       coeff_l3(33,107)=  4.3935689330e-01
       coeff_l3(33,108)=  4.0906984359e-02
       coeff_l3(33,109)= -1.4672926068e-01
       coeff_l3(33,110)=  9.0746521950e-02
       coeff_l3(33,111)=  1.2612210214e-01
       coeff_l3(33,112)=  2.2487479448e-01
       coeff_l3(33,113)=  2.9825839400e-01
       coeff_l3(33,114)=  1.7672787607e-01
       coeff_l3(33,115)=  2.1581043303e-01
       coeff_l3(33,116)=  1.2249276042e-01
       coeff_l3(33,117)= -2.1543087065e-01
       coeff_l3(33,118)= -4.0594294667e-01
       coeff_l3(33,119)= -1.2448056042e-01
       coeff_l3(33,120)=  3.8703806698e-02
       coeff_l3(33,121)= -3.2639664412e-01
       coeff_l3(33,122)= -1.0141001642e-01
       coeff_l3(33,123)= -7.7984027565e-02
       coeff_l3(33,124)= -1.8684660317e-03
       coeff_l3(33,125)=  3.3001916017e-03
       coeff_l3(34,1)= -2.6914077997e-01
       coeff_l3(34,2)= -1.1912132800e-01
       coeff_l3(34,3)= -3.4462571144e-01
       coeff_l3(34,4)= -2.7591708302e-01
       coeff_l3(34,5)=  7.1459025145e-02
       coeff_l3(34,6)=  1.8552270532e-01
       coeff_l3(34,7)=  5.5164031684e-02
       coeff_l3(34,8)= -5.0098702312e-02
       coeff_l3(34,9)=  4.0949515998e-02
       coeff_l3(34,10)= -2.1250769496e-01
       coeff_l3(34,11)= -1.8778437749e-02
       coeff_l3(34,12)= -1.4206665754e-01
       coeff_l3(34,13)=  5.2426964045e-02
       coeff_l3(34,14)= -2.0634335279e-01
       coeff_l3(34,15)=  1.0607182235e-01
       coeff_l3(34,16)= -1.4389079809e-01
       coeff_l3(34,17)= -1.1821447313e-01
       coeff_l3(34,18)= -4.4079408050e-01
       coeff_l3(34,19)= -5.6779481471e-02
       coeff_l3(34,20)= -1.8015620112e-01
       coeff_l3(34,21)= -1.4551356435e-01
       coeff_l3(34,22)=  9.7639024258e-02
       coeff_l3(34,23)= -1.5051619709e-01
       coeff_l3(34,24)=  7.8444778919e-02
       coeff_l3(34,25)= -1.0761877894e-01
       coeff_l3(34,26)= -7.1414351463e-02
       coeff_l3(34,27)=  1.7910036445e-01
       coeff_l3(34,28)=  1.1537028104e-01
       coeff_l3(34,29)= -2.2576738894e-01
       coeff_l3(34,30)= -7.0916354656e-02
       coeff_l3(34,31)= -1.6979478300e-01
       coeff_l3(34,32)= -1.0915721953e-01
       coeff_l3(34,33)= -2.5259524584e-01
       coeff_l3(34,34)=  1.4149199426e-01
       coeff_l3(34,35)=  2.3118574917e-01
       coeff_l3(34,36)=  1.2784726918e-01
       coeff_l3(34,37)= -5.1008909941e-02
       coeff_l3(34,38)= -5.1507085562e-01
       coeff_l3(34,39)=  1.6077247262e-01
       coeff_l3(34,40)= -1.3175079599e-02
       coeff_l3(34,41)= -2.0615071058e-01
       coeff_l3(34,42)=  4.8306141049e-02
       coeff_l3(34,43)= -2.7689847350e-01
       coeff_l3(34,44)= -7.7444680035e-02
       coeff_l3(34,45)= -1.9176532328e-01
       coeff_l3(34,46)= -1.6149871051e-01
       coeff_l3(34,47)= -2.9295182228e-01
       coeff_l3(34,48)=  3.5139175598e-03
       coeff_l3(34,49)=  7.9060715437e-01
       coeff_l3(34,50)= -3.1922960281e-01
       coeff_l3(34,51)= -1.6783626378e-01
       coeff_l3(34,52)=  6.6948018968e-02
       coeff_l3(34,53)=  1.3204032183e-01
       coeff_l3(34,54)= -4.8388976604e-02
       coeff_l3(34,55)= -4.9090031534e-03
       coeff_l3(34,56)=  1.1101586372e-01
       coeff_l3(34,57)= -2.5479568169e-02
       coeff_l3(34,58)= -1.3080605865e-01
       coeff_l3(34,59)= -3.0514964461e-01
       coeff_l3(34,60)= -3.3879283816e-02
       coeff_l3(34,61)= -1.1617580801e-01
       coeff_l3(34,62)=  1.3619578443e-02
       coeff_l3(34,63)=  1.7105698586e-01
       coeff_l3(34,64)=  3.4026250243e-02
       coeff_l3(34,65)= -2.1467870101e-02
       coeff_l3(34,66)= -1.2660254538e-01
       coeff_l3(34,67)= -4.6684873104e-01
       coeff_l3(34,68)= -1.2279400975e-01
       coeff_l3(34,69)= -3.3951538801e-01
       coeff_l3(34,70)=  7.0691466331e-02
       coeff_l3(34,71)= -3.2020363212e-01
       coeff_l3(34,72)= -5.5730279535e-02
       coeff_l3(34,73)=  1.2534621172e-02
       coeff_l3(34,74)= -7.8694418073e-02
       coeff_l3(34,75)=  3.5767514259e-02
       coeff_l3(34,76)=  2.7534668334e-03
       coeff_l3(34,77)=  2.2326213121e-01
       coeff_l3(34,78)=  3.5000581294e-02
       coeff_l3(34,79)= -2.3356366903e-02
       coeff_l3(34,80)=  1.9246222079e-01
       coeff_l3(34,81)= -4.0524387360e-01
       coeff_l3(34,82)= -1.7509137094e-01
       coeff_l3(34,83)= -4.9457415938e-02
       coeff_l3(34,84)= -1.4562605321e-01
       coeff_l3(34,85)= -2.6146864891e-01
       coeff_l3(34,86)=  8.1735879183e-02
       coeff_l3(34,87)= -1.3532899320e-01
       coeff_l3(34,88)= -3.0611035228e-01
       coeff_l3(34,89)=  1.4934519306e-02
       coeff_l3(34,90)= -1.0787366331e-01
       coeff_l3(34,91)=  9.4136513770e-02
       coeff_l3(34,92)= -1.1963204294e-01
       coeff_l3(34,93)= -1.9814519584e-01
       coeff_l3(34,94)=  8.2044526935e-02
       coeff_l3(34,95)= -1.2812502682e-01
       coeff_l3(34,96)= -1.8430274725e-01
       coeff_l3(34,97)=  6.6713127308e-03
       coeff_l3(34,98)=  2.2016791999e-01
       coeff_l3(34,99)= -5.8414787054e-02
       coeff_l3(34,100)=  1.8660316244e-02
       coeff_l3(34,101)=  1.5888752416e-02
       coeff_l3(34,102)= -1.3703548908e-01
       coeff_l3(34,103)= -3.9065718651e-02
       coeff_l3(34,104)= -2.3413752019e-01
       coeff_l3(34,105)= -6.3741528988e-01
       coeff_l3(34,106)= -8.1157013774e-02
       coeff_l3(34,107)= -7.3919260502e-01
       coeff_l3(34,108)=  9.1875128448e-02
       coeff_l3(34,109)= -1.0422820598e-01
       coeff_l3(34,110)= -8.8946811855e-02
       coeff_l3(34,111)=  1.2623234093e-01
       coeff_l3(34,112)= -4.8900443316e-01
       coeff_l3(34,113)= -4.1988052428e-02
       coeff_l3(34,114)=  5.7349499315e-02
       coeff_l3(34,115)= -5.0431972742e-01
       coeff_l3(34,116)= -3.5192869604e-02
       coeff_l3(34,117)= -2.9341807961e-01
       coeff_l3(34,118)= -1.6314695776e-01
       coeff_l3(34,119)=  1.7014445364e-01
       coeff_l3(34,120)= -1.2480738014e-01
       coeff_l3(34,121)= -3.6722427607e-01
       coeff_l3(34,122)= -1.3124322891e-01
       coeff_l3(34,123)=  4.6258896589e-02
       coeff_l3(34,124)= -1.7684711516e-01
       coeff_l3(34,125)=  1.8124464154e-01
       coeff_l3(35,1)= -5.2722054720e-01
       coeff_l3(35,2)= -2.2036854923e-01
       coeff_l3(35,3)=  1.3058742881e-01
       coeff_l3(35,4)= -1.9493347406e-01
       coeff_l3(35,5)=  5.0538133830e-02
       coeff_l3(35,6)=  7.4980400503e-02
       coeff_l3(35,7)=  1.4688212872e+00
       coeff_l3(35,8)= -2.4049673975e-01
       coeff_l3(35,9)=  1.6830524802e-01
       coeff_l3(35,10)= -1.3458354771e-01
       coeff_l3(35,11)= -3.0871155858e-01
       coeff_l3(35,12)= -6.6961568594e-01
       coeff_l3(35,13)= -6.2765961885e-01
       coeff_l3(35,14)= -2.1227754653e-01
       coeff_l3(35,15)= -6.8426299840e-03
       coeff_l3(35,16)=  1.4784134924e-01
       coeff_l3(35,17)= -4.2454662919e-01
       coeff_l3(35,18)= -1.4279961586e+00
       coeff_l3(35,19)=  4.6924105287e-01
       coeff_l3(35,20)=  3.7380740047e-02
       coeff_l3(35,21)=  9.7781139612e-01
       coeff_l3(35,22)= -2.4504928291e-01
       coeff_l3(35,23)=  9.7460716963e-02
       coeff_l3(35,24)= -1.5195265412e-01
       coeff_l3(35,25)= -1.2494506687e-01
       coeff_l3(35,26)=  4.4253575616e-03
       coeff_l3(35,27)=  2.3766441345e+00
       coeff_l3(35,28)=  1.7340313196e+00
       coeff_l3(35,29)= -1.3140348196e+00
       coeff_l3(35,30)= -1.1900353432e-01
       coeff_l3(35,31)= -1.0742058605e-01
       coeff_l3(35,32)= -1.2160082348e-02
       coeff_l3(35,33)=  1.5658245981e-01
       coeff_l3(35,34)= -1.2536914647e-01
       coeff_l3(35,35)= -1.6703726351e-01
       coeff_l3(35,36)= -3.8991626352e-03
       coeff_l3(35,37)= -7.7043659985e-02
       coeff_l3(35,38)= -1.6712369919e+00
       coeff_l3(35,39)= -2.2673933208e-01
       coeff_l3(35,40)= -1.7154766619e-01
       coeff_l3(35,41)= -8.6813288927e-01
       coeff_l3(35,42)=  2.6719728112e-01
       coeff_l3(35,43)= -2.4300591648e-01
       coeff_l3(35,44)=  4.1277911514e-02
       coeff_l3(35,45)= -3.8358949125e-02
       coeff_l3(35,46)= -5.4926687479e-01
       coeff_l3(35,47)= -1.1816948652e-01
       coeff_l3(35,48)= -4.7149702907e-01
       coeff_l3(35,49)=  5.3996891975e+00
       coeff_l3(35,50)= -1.9976384938e-01
       coeff_l3(35,51)= -1.9860106707e-01
       coeff_l3(35,52)= -4.2564414442e-02
       coeff_l3(35,53)= -2.4398941547e-02
       coeff_l3(35,54)=  1.2181771994e+00
       coeff_l3(35,55)=  2.0408208668e-01
       coeff_l3(35,56)= -1.5839976072e-01
       coeff_l3(35,57)= -2.7675991878e-02
       coeff_l3(35,58)=  3.2608762383e-01
       coeff_l3(35,59)= -4.5257985592e-01
       coeff_l3(35,60)= -2.3892016411e+00
       coeff_l3(35,61)=  2.3959605396e-01
       coeff_l3(35,62)=  4.4569127262e-02
       coeff_l3(35,63)=  1.4799720049e+00
       coeff_l3(35,64)= -2.4859324098e-02
       coeff_l3(35,65)= -2.0981489122e-01
       coeff_l3(35,66)= -1.9251164794e-01
       coeff_l3(35,67)= -5.2520208061e-02
       coeff_l3(35,68)= -1.3748575747e-01
       coeff_l3(35,69)= -1.3240197301e-01
       coeff_l3(35,70)= -2.1929357946e-01
       coeff_l3(35,71)= -1.1689329147e+00
       coeff_l3(35,72)=  2.9192626476e-02
       coeff_l3(35,73)= -6.4940407872e-02
       coeff_l3(35,74)=  2.8637298942e-01
       coeff_l3(35,75)=  3.1897049397e-02
       coeff_l3(35,76)=  2.3024287224e+00
       coeff_l3(35,77)=  1.2970333099e+00
       coeff_l3(35,78)=  6.8086721003e-02
       coeff_l3(35,79)= -5.1274240017e-02
       coeff_l3(35,80)=  1.8912674189e+00
       coeff_l3(35,81)= -2.8795510530e-01
       coeff_l3(35,82)= -1.7828921974e-01
       coeff_l3(35,83)=  1.5186464787e-01
       coeff_l3(35,84)= -1.2230005115e-01
       coeff_l3(35,85)= -4.1670966148e-01
       coeff_l3(35,86)= -7.5190536678e-02
       coeff_l3(35,87)=  1.3663117588e-01
       coeff_l3(35,88)= -3.6147400737e-01
       coeff_l3(35,89)=  1.0350636393e-01
       coeff_l3(35,90)=  2.4404816329e-01
       coeff_l3(35,91)=  1.7400151491e+00
       coeff_l3(35,92)= -2.3082832992e-01
       coeff_l3(35,93)= -4.4165011495e-02
       coeff_l3(35,94)= -1.3993383944e-01
       coeff_l3(35,95)= -2.9396647215e-01
       coeff_l3(35,96)= -2.0467807353e-01
       coeff_l3(35,97)= -7.0915535092e-02
       coeff_l3(35,98)= -1.2530390918e-01
       coeff_l3(35,99)= -5.8351278305e-01
       coeff_l3(35,100)= -5.4136133194e-01
       coeff_l3(35,101)=  2.7014235035e-02
       coeff_l3(35,102)= -1.6911569983e-02
       coeff_l3(35,103)=  7.7544823289e-02
       coeff_l3(35,104)=  1.5417070687e-01
       coeff_l3(35,105)= -9.7238373756e-01
       coeff_l3(35,106)= -1.4116729796e-01
       coeff_l3(35,107)= -3.2708638906e-01
       coeff_l3(35,108)= -3.0054154992e-01
       coeff_l3(35,109)= -1.7412820458e-01
       coeff_l3(35,110)=  1.5692700446e-01
       coeff_l3(35,111)= -4.6458005905e-01
       coeff_l3(35,112)= -9.2096978426e-01
       coeff_l3(35,113)= -5.0319510698e-01
       coeff_l3(35,114)=  7.0542091131e-01
       coeff_l3(35,115)= -1.3459592126e-02
       coeff_l3(35,116)= -6.9128632545e-01
       coeff_l3(35,117)= -4.7272670269e-01
       coeff_l3(35,118)= -4.5102035999e-01
       coeff_l3(35,119)= -6.2728978693e-02
       coeff_l3(35,120)= -6.4653746784e-02
       coeff_l3(35,121)= -5.1700824499e-01
       coeff_l3(35,122)= -9.8546393216e-02
       coeff_l3(35,123)=  6.5741308033e-02
       coeff_l3(35,124)=  2.0997749269e-01
       coeff_l3(35,125)=  5.8055069298e-02
       coeff_l3(36,1)= -8.7286180258e-01
       coeff_l3(36,2)= -4.7343316674e-01
       coeff_l3(36,3)=  1.8819670379e-01
       coeff_l3(36,4)=  1.4808581769e-01
       coeff_l3(36,5)= -3.8391190767e-01
       coeff_l3(36,6)= -4.2659604549e-01
       coeff_l3(36,7)= -3.1378743052e-01
       coeff_l3(36,8)= -2.8142866492e-01
       coeff_l3(36,9)= -4.1259589791e-01
       coeff_l3(36,10)= -1.2379755080e-01
       coeff_l3(36,11)= -3.6983555555e-01
       coeff_l3(36,12)=  5.6793749332e-01
       coeff_l3(36,13)=  5.9792327881e-01
       coeff_l3(36,14)= -5.2138864994e-01
       coeff_l3(36,15)= -3.1831857562e-01
       coeff_l3(36,16)= -1.4261135459e-01
       coeff_l3(36,17)= -4.1466787457e-01
       coeff_l3(36,18)= -3.2049912214e-01
       coeff_l3(36,19)= -1.1125851423e-01
       coeff_l3(36,20)= -5.1716715097e-01
       coeff_l3(36,21)= -1.8248002231e-01
       coeff_l3(36,22)= -8.7990343571e-01
       coeff_l3(36,23)= -2.5694739819e-01
       coeff_l3(36,24)= -2.2816604376e-01
       coeff_l3(36,25)=  7.2629666328e-01
       coeff_l3(36,26)= -1.6823576391e-01
       coeff_l3(36,27)= -2.2754278779e-01
       coeff_l3(36,28)= -6.1172634363e-01
       coeff_l3(36,29)= -3.2757809758e-01
       coeff_l3(36,30)= -8.7362653017e-01
       coeff_l3(36,31)= -1.6150148213e-01
       coeff_l3(36,32)= -1.5595087409e-01
       coeff_l3(36,33)= -4.1758507490e-01
       coeff_l3(36,34)= -2.1388942003e-01
       coeff_l3(36,35)= -8.3453744650e-01
       coeff_l3(36,36)= -2.0079241693e-01
       coeff_l3(36,37)= -2.4425591528e-01
       coeff_l3(36,38)= -1.8686076999e-01
       coeff_l3(36,39)= -2.9447761178e-01
       coeff_l3(36,40)= -3.7497654557e-02
       coeff_l3(36,41)=  1.3279651403e+00
       coeff_l3(36,42)= -2.4280408025e-01
       coeff_l3(36,43)=  4.8298791051e-02
       coeff_l3(36,44)= -2.2559306026e-01
       coeff_l3(36,45)= -1.2076689005e+00
       coeff_l3(36,46)= -4.4265103340e-01
       coeff_l3(36,47)=  1.0125954449e-01
       coeff_l3(36,48)= -2.6478552818e-01
       coeff_l3(36,49)= -1.6951481104e+00
       coeff_l3(36,50)=  4.1583444923e-02
       coeff_l3(36,51)= -4.6314701438e-01
       coeff_l3(36,52)= -2.4103298783e-01
       coeff_l3(36,53)=  3.3022154123e-02
       coeff_l3(36,54)= -2.0082414150e-02
       coeff_l3(36,55)= -3.0445298553e-01
       coeff_l3(36,56)= -1.8839311600e-01
       coeff_l3(36,57)= -6.3564204611e-03
       coeff_l3(36,58)=  2.4837206304e-01
       coeff_l3(36,59)=  2.1405373514e-01
       coeff_l3(36,60)=  6.8591140211e-02
       coeff_l3(36,61)= -2.7419865131e-01
       coeff_l3(36,62)= -4.8196136951e-01
       coeff_l3(36,63)= -1.3805688620e+00
       coeff_l3(36,64)=  1.4955378138e-02
       coeff_l3(36,65)= -3.4985074401e-01
       coeff_l3(36,66)= -2.5744488835e-01
       coeff_l3(36,67)=  9.1365896165e-02
       coeff_l3(36,68)=  8.3653710783e-02
       coeff_l3(36,69)=  4.7314640880e-01
       coeff_l3(36,70)= -3.8847076893e-01
       coeff_l3(36,71)= -2.5052076578e-01
       coeff_l3(36,72)= -3.4543940425e-01
       coeff_l3(36,73)=  5.6614989042e-01
       coeff_l3(36,74)= -2.5666925311e-01
       coeff_l3(36,75)= -3.3797577024e-01
       coeff_l3(36,76)= -5.3670698404e-01
       coeff_l3(36,77)= -9.5407670736e-01
       coeff_l3(36,78)= -4.1434654593e-01
       coeff_l3(36,79)= -2.5794675946e-01
       coeff_l3(36,80)=  2.8370296955e-01
       coeff_l3(36,81)=  3.9052841067e-01
       coeff_l3(36,82)=  1.8040704727e-01
       coeff_l3(36,83)=  2.6492658257e-01
       coeff_l3(36,84)=  1.1380932294e-02
       coeff_l3(36,85)=  4.8966020346e-01
       coeff_l3(36,86)= -1.2796752155e-01
       coeff_l3(36,87)= -1.0604672134e-01
       coeff_l3(36,88)=  6.6891235113e-01
       coeff_l3(36,89)= -4.2437878251e-01
       coeff_l3(36,90)= -4.0833690763e-01
       coeff_l3(36,91)= -8.2089567184e-01
       coeff_l3(36,92)= -2.1963587403e-01
       coeff_l3(36,93)= -5.0996994972e-01
       coeff_l3(36,94)=  1.7535804212e-01
       coeff_l3(36,95)= -2.5994822383e-01
       coeff_l3(36,96)=  3.3351483941e-01
       coeff_l3(36,97)= -1.6121837497e-01
       coeff_l3(36,98)= -2.6160103083e-01
       coeff_l3(36,99)=  2.9916854575e-02
       coeff_l3(36,100)= -2.1481266618e-01
       coeff_l3(36,101)= -6.5822231770e-01
       coeff_l3(36,102)= -3.9184309542e-02
       coeff_l3(36,103)= -5.5032607168e-02
       coeff_l3(36,104)= -2.8691881895e-01
       coeff_l3(36,105)=  1.0510225296e+00
       coeff_l3(36,106)= -1.5018978715e-01
       coeff_l3(36,107)=  3.4356650710e-01
       coeff_l3(36,108)= -2.6118075848e-01
       coeff_l3(36,109)= -8.2987934351e-02
       coeff_l3(36,110)= -8.0108582973e-02
       coeff_l3(36,111)=  4.7894384712e-02
       coeff_l3(36,112)= -4.4198018312e-01
       coeff_l3(36,113)= -2.7887678146e-01
       coeff_l3(36,114)=  1.5779879689e-01
       coeff_l3(36,115)=  3.8626305759e-02
       coeff_l3(36,116)=  1.1533095837e+00
       coeff_l3(36,117)=  2.0437562838e-02
       coeff_l3(36,118)=  1.4812133312e+00
       coeff_l3(36,119)= -1.4320749044e-01
       coeff_l3(36,120)= -1.0005988181e-02
       coeff_l3(36,121)= -2.4184970558e-01
       coeff_l3(36,122)= -1.3218125701e-01
       coeff_l3(36,123)= -3.1480625272e-01
       coeff_l3(36,124)=  1.3331128657e-01
       coeff_l3(36,125)=  6.7749214172e-01
       coeff_l3(37,1)= -2.6020565629e-01
       coeff_l3(37,2)= -1.6539825499e-01
       coeff_l3(37,3)=  1.5851449966e-01
       coeff_l3(37,4)=  7.2314697504e-01
       coeff_l3(37,5)= -6.0575295240e-02
       coeff_l3(37,6)= -1.2650436163e-01
       coeff_l3(37,7)= -7.5370746851e-01
       coeff_l3(37,8)=  2.8056916595e-01
       coeff_l3(37,9)= -8.2883611321e-02
       coeff_l3(37,10)=  2.6326772571e-01
       coeff_l3(37,11)= -1.2005190551e-01
       coeff_l3(37,12)= -4.0241336823e-01
       coeff_l3(37,13)= -4.0796980262e-01
       coeff_l3(37,14)= -4.7512060404e-01
       coeff_l3(37,15)=  7.7669732273e-02
       coeff_l3(37,16)=  5.5273890495e-02
       coeff_l3(37,17)=  8.5344210267e-02
       coeff_l3(37,18)=  2.2076599300e-02
       coeff_l3(37,19)= -3.1644257903e-01
       coeff_l3(37,20)=  2.9062193632e-01
       coeff_l3(37,21)= -9.9925726652e-02
       coeff_l3(37,22)=  2.0993974805e-01
       coeff_l3(37,23)= -2.1963711828e-02
       coeff_l3(37,24)= -3.2324340194e-02
       coeff_l3(37,25)=  7.8459584713e-01
       coeff_l3(37,26)= -1.0793328285e-01
       coeff_l3(37,27)= -1.4040639400e+00
       coeff_l3(37,28)= -1.0911124945e+00
       coeff_l3(37,29)=  7.0953619480e-01
       coeff_l3(37,30)=  1.8716911077e+00
       coeff_l3(37,31)=  1.7184405029e-01
       coeff_l3(37,32)=  5.2444424480e-02
       coeff_l3(37,33)= -6.0649920255e-02
       coeff_l3(37,34)= -6.3291087747e-02
       coeff_l3(37,35)=  9.1299897432e-01
       coeff_l3(37,36)=  1.1226385087e-01
       coeff_l3(37,37)=  1.9976833835e-02
       coeff_l3(37,38)=  3.7143293023e-01
       coeff_l3(37,39)=  3.8480209187e-03
       coeff_l3(37,40)=  4.2853495479e-01
       coeff_l3(37,41)=  2.7811780572e-01
       coeff_l3(37,42)= -2.0975219086e-02
       coeff_l3(37,43)=  2.8587943316e-01
       coeff_l3(37,44)=  2.2968003154e-01
       coeff_l3(37,45)=  1.0374919325e-01
       coeff_l3(37,46)=  1.3799949884e+00
       coeff_l3(37,47)=  1.4438608289e-01
       coeff_l3(37,48)=  1.0194966942e-01
       coeff_l3(37,49)=  3.2049386501e+00
       coeff_l3(37,50)=  3.0105820298e-01
       coeff_l3(37,51)=  5.1202422380e-01
       coeff_l3(37,52)= -3.8179524243e-02
       coeff_l3(37,53)=  2.1853226423e-01
       coeff_l3(37,54)= -1.4062939882e+00
       coeff_l3(37,55)=  7.1815747023e-01
       coeff_l3(37,56)=  3.7055626512e-02
       coeff_l3(37,57)= -1.9487861544e-02
       coeff_l3(37,58)=  1.1316614598e-01
       coeff_l3(37,59)=  2.9465477914e-02
       coeff_l3(37,60)=  4.4348006248e+00
       coeff_l3(37,61)= -1.9255675375e-01
       coeff_l3(37,62)= -3.7920601666e-02
       coeff_l3(37,63)= -2.8686258197e-01
       coeff_l3(37,64)=  3.4997656941e-01
       coeff_l3(37,65)= -1.7773908377e-01
       coeff_l3(37,66)= -1.7843133211e-01
       coeff_l3(37,67)=  1.4945293963e-01
       coeff_l3(37,68)=  7.2608150542e-02
       coeff_l3(37,69)= -3.1964686513e-01
       coeff_l3(37,70)=  3.6452442408e-02
       coeff_l3(37,71)= -8.5408650339e-02
       coeff_l3(37,72)=  2.5934195146e-02
       coeff_l3(37,73)= -3.1794577837e-01
       coeff_l3(37,74)=  5.6223869324e-01
       coeff_l3(37,75)= -2.2108849883e-01
       coeff_l3(37,76)=  2.1845369339e+00
       coeff_l3(37,77)= -9.0243935585e-02
       coeff_l3(37,78)= -3.1453721225e-02
       coeff_l3(37,79)= -2.1239804104e-02
       coeff_l3(37,80)=  1.1099981070e+00
       coeff_l3(37,81)=  1.9421605766e-01
       coeff_l3(37,82)=  1.4228044450e-01
       coeff_l3(37,83)=  6.8076036870e-02
       coeff_l3(37,84)= -4.2203369737e-01
       coeff_l3(37,85)=  2.9429327697e-02
       coeff_l3(37,86)= -1.7005576193e-01
       coeff_l3(37,87)=  2.1276107058e-02
       coeff_l3(37,88)=  1.8494482338e-01
       coeff_l3(37,89)=  1.3234103918e+00
       coeff_l3(37,90)=  1.4037859440e-01
       coeff_l3(37,91)=  2.1492989361e-01
       coeff_l3(37,92)= -3.4583345987e-03
       coeff_l3(37,93)=  1.5523290634e-01
       coeff_l3(37,94)= -6.6849184036e-01
       coeff_l3(37,95)=  4.9430720508e-02
       coeff_l3(37,96)=  1.4832058549e-01
       coeff_l3(37,97)= -1.1767398566e-01
       coeff_l3(37,98)=  1.8189577758e-01
       coeff_l3(37,99)=  1.0878720284e+00
       coeff_l3(37,100)= -6.2991224229e-02
       coeff_l3(37,101)=  2.6568517089e-01
       coeff_l3(37,102)=  3.9827346802e-02
       coeff_l3(37,103)= -8.2283899188e-02
       coeff_l3(37,104)=  7.4523538351e-02
       coeff_l3(37,105)=  2.1263226867e-01
       coeff_l3(37,106)=  3.3098068833e-01
       coeff_l3(37,107)= -3.2777032256e-01
       coeff_l3(37,108)=  1.1062663794e-01
       coeff_l3(37,109)= -4.6664819121e-02
       coeff_l3(37,110)= -5.5533088744e-02
       coeff_l3(37,111)= -3.7543427944e-01
       coeff_l3(37,112)=  4.3025601655e-02
       coeff_l3(37,113)=  4.2920115590e-01
       coeff_l3(37,114)= -5.9516036510e-01
       coeff_l3(37,115)=  5.0839833915e-02
       coeff_l3(37,116)= -3.1002148986e-01
       coeff_l3(37,117)=  8.5372292995e-01
       coeff_l3(37,118)= -2.2922031581e-01
       coeff_l3(37,119)= -1.3415788114e-01
       coeff_l3(37,120)= -1.7687270045e-01
       coeff_l3(37,121)=  1.4709728956e-01
       coeff_l3(37,122)=  3.5029955208e-02
       coeff_l3(37,123)= -5.1185069606e-03
       coeff_l3(37,124)= -5.0435997546e-02
       coeff_l3(37,125)= -5.3203499317e-01
       coeff_l3(38,1)= -5.2685493231e-01
       coeff_l3(38,2)= -4.2110126466e-02
       coeff_l3(38,3)= -3.7932074070e-01
       coeff_l3(38,4)= -1.3051186502e-01
       coeff_l3(38,5)=  1.0100891441e-01
       coeff_l3(38,6)=  1.0406014323e-01
       coeff_l3(38,7)= -1.2500120699e-01
       coeff_l3(38,8)= -5.1247328520e-03
       coeff_l3(38,9)= -1.5834440291e-01
       coeff_l3(38,10)= -1.3637019694e-01
       coeff_l3(38,11)= -1.5279388428e-01
       coeff_l3(38,12)=  1.0711665452e-01
       coeff_l3(38,13)= -1.5939643979e-01
       coeff_l3(38,14)= -2.3320133984e-01
       coeff_l3(38,15)= -4.6065900475e-02
       coeff_l3(38,16)=  8.7708882987e-02
       coeff_l3(38,17)= -6.0292005539e-02
       coeff_l3(38,18)= -2.7372270823e-01
       coeff_l3(38,19)=  1.2318053842e-01
       coeff_l3(38,20)=  2.4476999044e-01
       coeff_l3(38,21)= -7.4358589947e-02
       coeff_l3(38,22)= -1.2262993306e-01
       coeff_l3(38,23)=  1.1223463714e-01
       coeff_l3(38,24)=  1.9093078375e-01
       coeff_l3(38,25)=  8.6863756180e-02
       coeff_l3(38,26)= -2.5716406107e-01
       coeff_l3(38,27)=  1.3994875550e-01
       coeff_l3(38,28)=  4.1739255190e-02
       coeff_l3(38,29)=  4.7880064696e-02
       coeff_l3(38,30)= -9.0558417141e-03
       coeff_l3(38,31)= -1.4660668373e-01
       coeff_l3(38,32)=  3.8609623909e-02
       coeff_l3(38,33)=  5.0698645413e-02
       coeff_l3(38,34)=  6.5620638430e-02
       coeff_l3(38,35)=  1.1050201952e-01
       coeff_l3(38,36)= -3.8612563163e-02
       coeff_l3(38,37)=  1.0744301230e-01
       coeff_l3(38,38)=  6.0790404677e-03
       coeff_l3(38,39)= -1.0641341656e-01
       coeff_l3(38,40)= -1.7947471142e-01
       coeff_l3(38,41)=  6.2286290340e-03
       coeff_l3(38,42)=  1.0638613254e-02
       coeff_l3(38,43)= -3.1925361603e-02
       coeff_l3(38,44)=  2.1932184696e-02
       coeff_l3(38,45)= -5.5047863722e-01
       coeff_l3(38,46)=  3.2208364457e-02
       coeff_l3(38,47)= -7.1030020714e-02
       coeff_l3(38,48)=  4.7736350447e-02
       coeff_l3(38,49)= -6.2132847309e-01
       coeff_l3(38,50)=  8.0600447953e-02
       coeff_l3(38,51)= -7.4443250895e-02
       coeff_l3(38,52)= -2.6929864287e-01
       coeff_l3(38,53)=  2.6210194826e-01
       coeff_l3(38,54)= -2.2021961212e-01
       coeff_l3(38,55)=  9.0028658509e-02
       coeff_l3(38,56)= -1.5779717267e-01
       coeff_l3(38,57)= -1.3553416729e-01
       coeff_l3(38,58)= -1.8356837332e-01
       coeff_l3(38,59)=  2.6441049576e-01
       coeff_l3(38,60)= -2.8991124034e-01
       coeff_l3(38,61)=  1.7964375019e-01
       coeff_l3(38,62)=  1.0568347760e-02
       coeff_l3(38,63)= -2.4951355159e-01
       coeff_l3(38,64)=  2.2674451768e-01
       coeff_l3(38,65)=  7.3119983077e-02
       coeff_l3(38,66)= -4.4634338468e-02
       coeff_l3(38,67)= -1.3801752031e-01
       coeff_l3(38,68)= -2.1826167405e-01
       coeff_l3(38,69)=  3.6003446579e-01
       coeff_l3(38,70)= -1.8354693055e-01
       coeff_l3(38,71)= -5.1322363317e-02
       coeff_l3(38,72)=  1.0953992605e-01
       coeff_l3(38,73)= -3.3042196184e-02
       coeff_l3(38,74)=  7.3034465313e-02
       coeff_l3(38,75)=  2.2814138234e-01
       coeff_l3(38,76)= -1.4594149590e-01
       coeff_l3(38,77)= -8.3735100925e-02
       coeff_l3(38,78)= -6.3913859427e-02
       coeff_l3(38,79)= -2.0551916678e-03
       coeff_l3(38,80)= -7.3121450841e-02
       coeff_l3(38,81)= -1.5498879552e-01
       coeff_l3(38,82)= -5.4989729077e-02
       coeff_l3(38,83)= -2.0392721891e-01
       coeff_l3(38,84)= -2.1512764692e-01
       coeff_l3(38,85)= -6.9045978598e-03
       coeff_l3(38,86)=  4.8435930163e-02
       coeff_l3(38,87)= -2.4656442925e-02
       coeff_l3(38,88)= -1.2084717304e-01
       coeff_l3(38,89)=  1.9634106755e-01
       coeff_l3(38,90)= -1.9156731665e-01
       coeff_l3(38,91)= -6.7470049858e-01
       coeff_l3(38,92)= -1.1351430416e-01
       coeff_l3(38,93)= -3.8770893216e-01
       coeff_l3(38,94)=  4.0465544909e-02
       coeff_l3(38,95)=  1.3654848933e-01
       coeff_l3(38,96)=  5.4816126823e-02
       coeff_l3(38,97)= -8.5676699877e-02
       coeff_l3(38,98)=  4.2237702757e-02
       coeff_l3(38,99)=  6.0701660812e-02
       coeff_l3(38,100)= -8.1364050508e-02
       coeff_l3(38,101)=  9.5210470259e-02
       coeff_l3(38,102)=  4.0058113635e-02
       coeff_l3(38,103)=  1.5031231940e-01
       coeff_l3(38,104)= -1.3767221570e-01
       coeff_l3(38,105)=  3.2083126903e-01
       coeff_l3(38,106)= -2.3593481630e-02
       coeff_l3(38,107)=  2.5716075301e-01
       coeff_l3(38,108)= -2.8893961571e-03
       coeff_l3(38,109)= -3.6854922771e-02
       coeff_l3(38,110)=  6.7167855799e-02
       coeff_l3(38,111)=  3.3882427961e-02
       coeff_l3(38,112)= -3.6132943630e-01
       coeff_l3(38,113)=  7.5122334063e-02
       coeff_l3(38,114)=  3.9206972718e-01
       coeff_l3(38,115)=  1.4535125811e-03
       coeff_l3(38,116)= -8.5969366133e-02
       coeff_l3(38,117)= -1.8450234830e-01
       coeff_l3(38,118)= -1.4373636246e-01
       coeff_l3(38,119)= -7.6250150800e-02
       coeff_l3(38,120)= -1.0641917586e-01
       coeff_l3(38,121)= -2.5119766593e-01
       coeff_l3(38,122)=  3.6696106195e-02
       coeff_l3(38,123)= -4.9467340112e-02
       coeff_l3(38,124)= -1.5858757496e-01
       coeff_l3(38,125)= -1.8414250016e-01
       coeff_l3(39,1)=  7.6894730330e-01
       coeff_l3(39,2)= -1.1576165259e-01
       coeff_l3(39,3)= -2.3988322914e-01
       coeff_l3(39,4)= -6.7059046030e-01
       coeff_l3(39,5)= -3.5064110160e-01
       coeff_l3(39,6)= -4.6531683207e-01
       coeff_l3(39,7)=  3.0931040645e-01
       coeff_l3(39,8)= -8.2947164774e-01
       coeff_l3(39,9)= -1.3501940668e-01
       coeff_l3(39,10)=  9.5636889338e-02
       coeff_l3(39,11)= -6.3511210680e-01
       coeff_l3(39,12)= -1.0928441286e+00
       coeff_l3(39,13)= -1.5648082495e+00
       coeff_l3(39,14)=  3.6589908600e-01
       coeff_l3(39,15)= -2.9768154025e-01
       coeff_l3(39,16)= -9.1230876744e-02
       coeff_l3(39,17)=  2.3424172401e-01
       coeff_l3(39,18)= -1.7225900888e+00
       coeff_l3(39,19)= -3.6981671583e-03
       coeff_l3(39,20)= -5.2771013975e-01
       coeff_l3(39,21)= -3.4865444899e-01
       coeff_l3(39,22)=  2.7552893758e-01
       coeff_l3(39,23)= -4.3882972002e-01
       coeff_l3(39,24)= -2.3545104265e-01
       coeff_l3(39,25)= -1.4130715132e+00
       coeff_l3(39,26)=  8.5518792272e-02
       coeff_l3(39,27)=  4.0314900875e-01
       coeff_l3(39,28)=  5.6021577120e-01
       coeff_l3(39,29)= -1.5768198967e+00
       coeff_l3(39,30)= -3.7694755942e-02
       coeff_l3(39,31)= -2.1927276626e-02
       coeff_l3(39,32)=  2.1906387992e-03
       coeff_l3(39,33)= -6.4963299036e-01
       coeff_l3(39,34)= -1.2545132637e-01
       coeff_l3(39,35)=  6.1690372229e-01
       coeff_l3(39,36)= -5.3735159338e-02
       coeff_l3(39,37)= -3.2087010145e-01
       coeff_l3(39,38)= -1.8379722834e+00
       coeff_l3(39,39)=  1.3254998624e-01
       coeff_l3(39,40)= -5.2862423658e-01
       coeff_l3(39,41)= -2.2385454178e-01
       coeff_l3(39,42)= -1.0103442706e-02
       coeff_l3(39,43)= -1.1417419463e-01
       coeff_l3(39,44)= -1.6040509939e-01
       coeff_l3(39,45)=  3.9641645551e-01
       coeff_l3(39,46)= -4.5300161839e-01
       coeff_l3(39,47)= -5.8263623714e-01
       coeff_l3(39,48)= -7.9284459352e-01
       coeff_l3(39,49)=  1.6423420906e+00
       coeff_l3(39,50)=  4.3444818258e-01
       coeff_l3(39,51)= -1.8102973700e-01
       coeff_l3(39,52)= -2.0351593196e-01
       coeff_l3(39,53)= -1.6830758750e-01
       coeff_l3(39,54)=  2.7814939618e-01
       coeff_l3(39,55)= -4.9093857408e-01
       coeff_l3(39,56)= -7.1568650007e-01
       coeff_l3(39,57)= -4.6305289865e-01
       coeff_l3(39,58)= -1.1506619453e+00
       coeff_l3(39,59)=  2.9714021087e-01
       coeff_l3(39,60)=  4.0414831042e-01
       coeff_l3(39,61)= -4.4779139757e-01
       coeff_l3(39,62)= -3.3917421103e-01
       coeff_l3(39,63)=  1.8730881810e-01
       coeff_l3(39,64)= -4.6147972345e-01
       coeff_l3(39,65)= -6.8959540129e-01
       coeff_l3(39,66)= -2.9310801625e-01
       coeff_l3(39,67)= -8.7655997276e-01
       coeff_l3(39,68)=  2.0816077292e-01
       coeff_l3(39,69)=  6.4686894417e-02
       coeff_l3(39,70)= -4.4836267829e-01
       coeff_l3(39,71)= -1.4510204792e+00
       coeff_l3(39,72)= -1.7233067751e-01
       coeff_l3(39,73)= -1.0838593245e+00
       coeff_l3(39,74)= -5.5077743530e-01
       coeff_l3(39,75)= -4.8967728019e-01
       coeff_l3(39,76)=  8.8536131382e-01
       coeff_l3(39,77)=  1.7938801646e-01
       coeff_l3(39,78)=  9.5486640930e-02
       coeff_l3(39,79)= -4.1625842452e-01
       coeff_l3(39,80)=  5.9967148304e-01
       coeff_l3(39,81)= -1.6069269180e-01
       coeff_l3(39,82)= -5.0536638498e-01
       coeff_l3(39,83)=  1.6813234985e-01
       coeff_l3(39,84)= -2.8339228034e-01
       coeff_l3(39,85)=  6.8623971939e-01
       coeff_l3(39,86)=  2.4688741565e-01
       coeff_l3(39,87)= -2.8673034906e-01
       coeff_l3(39,88)= -4.6682199836e-01
       coeff_l3(39,89)= -1.3588537276e-01
       coeff_l3(39,90)= -5.6422746181e-01
       coeff_l3(39,91)=  3.5137233138e-01
       coeff_l3(39,92)= -4.2758521438e-01
       coeff_l3(39,93)= -2.1885295212e-01
       coeff_l3(39,94)= -7.0578593016e-01
       coeff_l3(39,95)= -5.0080114603e-01
       coeff_l3(39,96)= -8.7926030159e-01
       coeff_l3(39,97)= -2.2467140853e-01
       coeff_l3(39,98)=  5.0213824958e-02
       coeff_l3(39,99)=  5.4092567414e-02
       coeff_l3(39,100)= -5.6557792425e-01
       coeff_l3(39,101)= -2.6663589478e-01
       coeff_l3(39,102)= -4.2500421405e-01
       coeff_l3(39,103)= -2.9468461871e-01
       coeff_l3(39,104)= -1.4422395825e-01
       coeff_l3(39,105)=  5.8289278299e-02
       coeff_l3(39,106)= -5.5131167173e-01
       coeff_l3(39,107)= -9.0625949204e-02
       coeff_l3(39,108)=  1.9065714441e-03
       coeff_l3(39,109)= -2.2925588489e-01
       coeff_l3(39,110)= -2.9533344507e-01
       coeff_l3(39,111)= -3.6113247275e-01
       coeff_l3(39,112)=  3.9326423407e-01
       coeff_l3(39,113)=  4.2868210003e-03
       coeff_l3(39,114)= -2.4452513456e-01
       coeff_l3(39,115)= -7.8237213194e-02
       coeff_l3(39,116)= -7.2579056025e-01
       coeff_l3(39,117)= -8.6447429657e-01
       coeff_l3(39,118)=  4.2294792831e-02
       coeff_l3(39,119)= -3.0777344108e-01
       coeff_l3(39,120)= -4.5288786292e-01
       coeff_l3(39,121)=  1.1230266094e-01
       coeff_l3(39,122)= -2.4072676897e-01
       coeff_l3(39,123)= -1.8826881051e-01
       coeff_l3(39,124)= -1.4843064547e-01
       coeff_l3(39,125)= -3.4107831120e-01
       coeff_l3(40,1)= -2.6058945805e-02
       coeff_l3(40,2)=  8.1273838878e-02
       coeff_l3(40,3)= -4.0969944000e-01
       coeff_l3(40,4)=  2.8742838651e-02
       coeff_l3(40,5)= -4.9396310002e-02
       coeff_l3(40,6)= -2.5142238010e-03
       coeff_l3(40,7)=  6.7377209663e-01
       coeff_l3(40,8)=  2.3537743092e-01
       coeff_l3(40,9)= -2.2413596511e-02
       coeff_l3(40,10)=  6.6170617938e-02
       coeff_l3(40,11)=  5.1434013993e-02
       coeff_l3(40,12)= -4.0155049413e-02
       coeff_l3(40,13)= -6.7958392203e-02
       coeff_l3(40,14)=  8.3930967376e-03
       coeff_l3(40,15)= -2.5704013184e-02
       coeff_l3(40,16)= -2.7650627494e-01
       coeff_l3(40,17)=  7.6337955892e-02
       coeff_l3(40,18)= -2.1065631509e-01
       coeff_l3(40,19)= -2.4452984333e-02
       coeff_l3(40,20)=  1.6651391983e-01
       coeff_l3(40,21)=  1.2733305991e-01
       coeff_l3(40,22)= -3.1161841750e-01
       coeff_l3(40,23)= -1.2624678016e-01
       coeff_l3(40,24)= -1.9549392164e-01
       coeff_l3(40,25)= -1.2351678312e-01
       coeff_l3(40,26)= -1.2213212252e-01
       coeff_l3(40,27)=  8.9266675711e-01
       coeff_l3(40,28)=  9.4049227238e-01
       coeff_l3(40,29)= -1.3978737593e-01
       coeff_l3(40,30)= -8.4166176617e-02
       coeff_l3(40,31)= -6.6202558577e-02
       coeff_l3(40,32)= -6.5522804856e-02
       coeff_l3(40,33)=  2.7271702886e-01
       coeff_l3(40,34)= -5.6310907006e-02
       coeff_l3(40,35)=  3.4788358212e-01
       coeff_l3(40,36)=  4.7759007663e-02
       coeff_l3(40,37)= -3.9296546020e-03
       coeff_l3(40,38)=  3.7287625670e-01
       coeff_l3(40,39)= -5.8359809220e-02
       coeff_l3(40,40)= -1.3639189303e-02
       coeff_l3(40,41)=  8.7622612715e-02
       coeff_l3(40,42)= -1.0185994953e-01
       coeff_l3(40,43)=  5.9092335403e-02
       coeff_l3(40,44)= -2.3806343973e-01
       coeff_l3(40,45)= -1.4887666702e-01
       coeff_l3(40,46)= -1.3881465793e-01
       coeff_l3(40,47)= -2.6231551543e-02
       coeff_l3(40,48)= -7.8905440867e-02
       coeff_l3(40,49)=  9.8856687546e-01
       coeff_l3(40,50)= -7.7293470502e-02
       coeff_l3(40,51)= -7.3642887175e-02
       coeff_l3(40,52)= -1.8380907178e-01
       coeff_l3(40,53)= -2.3299092054e-01
       coeff_l3(40,54)=  5.4416376352e-01
       coeff_l3(40,55)=  6.8888068199e-02
       coeff_l3(40,56)=  1.4383801818e-01
       coeff_l3(40,57)= -8.7048329413e-02
       coeff_l3(40,58)= -1.2266231328e-01
       coeff_l3(40,59)= -1.6699714959e-01
       coeff_l3(40,60)= -7.8229421377e-01
       coeff_l3(40,61)= -1.2596639991e-01
       coeff_l3(40,62)= -1.4821574092e-01
       coeff_l3(40,63)=  2.4649672210e-01
       coeff_l3(40,64)= -2.0981813967e-01
       coeff_l3(40,65)= -5.3535923362e-02
       coeff_l3(40,66)= -4.0683712810e-02
       coeff_l3(40,67)= -4.3466344476e-01
       coeff_l3(40,68)= -1.5177725255e-01
       coeff_l3(40,69)= -3.1950578094e-02
       coeff_l3(40,70)=  1.7610944808e-01
       coeff_l3(40,71)= -1.9800703228e-01
       coeff_l3(40,72)= -1.6507381573e-02
       coeff_l3(40,73)= -1.3002067804e-01
       coeff_l3(40,74)= -9.4644390047e-02
       coeff_l3(40,75)=  2.5485387072e-02
       coeff_l3(40,76)=  6.5300250053e-01
       coeff_l3(40,77)= -1.1275497824e-01
       coeff_l3(40,78)=  7.2536678053e-03
       coeff_l3(40,79)= -2.4037902057e-01
       coeff_l3(40,80)=  8.2100558281e-01
       coeff_l3(40,81)=  1.0438331962e-01
       coeff_l3(40,82)=  9.3049421906e-02
       coeff_l3(40,83)= -1.4140065014e-01
       coeff_l3(40,84)= -1.7327672243e-01
       coeff_l3(40,85)=  5.6888186373e-03
       coeff_l3(40,86)= -8.0189898610e-02
       coeff_l3(40,87)= -9.6658356488e-03
       coeff_l3(40,88)=  5.7865440845e-02
       coeff_l3(40,89)=  1.2237293273e-01
       coeff_l3(40,90)=  1.2673199177e-01
       coeff_l3(40,91)=  1.9818533957e-01
       coeff_l3(40,92)= -2.0405246317e-01
       coeff_l3(40,93)= -1.1352564394e-01
       coeff_l3(40,94)=  2.9785677791e-01
       coeff_l3(40,95)= -5.1774889231e-02
       coeff_l3(40,96)=  2.6301149279e-02
       coeff_l3(40,97)=  3.3424101770e-02
       coeff_l3(40,98)= -6.7707404494e-02
       coeff_l3(40,99)= -3.8928601146e-01
       coeff_l3(40,100)=  4.2082317173e-02
       coeff_l3(40,101)= -1.6703483462e-01
       coeff_l3(40,102)= -1.2263371050e-01
       coeff_l3(40,103)= -9.9381506443e-02
       coeff_l3(40,104)= -1.2834948301e-01
       coeff_l3(40,105)=  9.5321953297e-02
       coeff_l3(40,106)=  1.8591985106e-01
       coeff_l3(40,107)=  2.7609580755e-01
       coeff_l3(40,108)=  3.1452845782e-02
       coeff_l3(40,109)= -1.4658863842e-01
       coeff_l3(40,110)=  4.2375694960e-02
       coeff_l3(40,111)=  2.4566684663e-01
       coeff_l3(40,112)= -8.5683710873e-02
       coeff_l3(40,113)= -2.4223078787e-01
       coeff_l3(40,114)=  5.0849497318e-01
       coeff_l3(40,115)= -1.9260664284e-01
       coeff_l3(40,116)= -2.3869511485e-01
       coeff_l3(40,117)= -1.1387943476e-01
       coeff_l3(40,118)=  2.7206632495e-01
       coeff_l3(40,119)=  5.5911593139e-02
       coeff_l3(40,120)=  2.1580375731e-02
       coeff_l3(40,121)=  7.9275174066e-03
       coeff_l3(40,122)= -3.0627662316e-02
       coeff_l3(40,123)= -9.3585453928e-02
       coeff_l3(40,124)=  2.4091901258e-02
       coeff_l3(40,125)= -2.1355394274e-03
       coeff_l3(41,1)= -3.1451493502e-01
       coeff_l3(41,2)= -4.1247174144e-02
       coeff_l3(41,3)= -4.7485683113e-02
       coeff_l3(41,4)= -5.6427586824e-02
       coeff_l3(41,5)=  5.4099339992e-02
       coeff_l3(41,6)=  1.0318554938e-01
       coeff_l3(41,7)= -1.0410574079e-01
       coeff_l3(41,8)= -2.4063210189e-01
       coeff_l3(41,9)= -6.3411682844e-02
       coeff_l3(41,10)= -1.6620156169e-01
       coeff_l3(41,11)= -1.3838208281e-02
       coeff_l3(41,12)= -1.0167506337e-01
       coeff_l3(41,13)= -1.2402129173e-01
       coeff_l3(41,14)= -1.6639120877e-02
       coeff_l3(41,15)=  1.3454520702e-01
       coeff_l3(41,16)=  1.8028251827e-01
       coeff_l3(41,17)= -2.3201742768e-01
       coeff_l3(41,18)= -5.0983434916e-01
       coeff_l3(41,19)=  4.8280116171e-02
       coeff_l3(41,20)=  1.4575128257e-01
       coeff_l3(41,21)= -2.8233878314e-02
       coeff_l3(41,22)= -1.2788087130e-01
       coeff_l3(41,23)=  2.2423066199e-01
       coeff_l3(41,24)= -5.6880705059e-02
       coeff_l3(41,25)= -3.3983731270e-01
       coeff_l3(41,26)=  8.8775284588e-02
       coeff_l3(41,27)=  1.3763540983e-01
       coeff_l3(41,28)=  1.6467078030e-01
       coeff_l3(41,29)= -7.5397294760e-01
       coeff_l3(41,30)=  7.2799086571e-02
       coeff_l3(41,31)= -2.8428776190e-02
       coeff_l3(41,32)= -7.3377445340e-02
       coeff_l3(41,33)= -4.0792822838e-02
       coeff_l3(41,34)=  7.4493594468e-02
       coeff_l3(41,35)= -1.1637218297e-02
       coeff_l3(41,36)= -5.4777566344e-02
       coeff_l3(41,37)=  1.5660724044e-01
       coeff_l3(41,38)= -5.8221298456e-01
       coeff_l3(41,39)=  4.9572389573e-02
       coeff_l3(41,40)= -1.6254024208e-01
       coeff_l3(41,41)= -7.3600673676e-01
       coeff_l3(41,42)=  2.1023910493e-02
       coeff_l3(41,43)=  2.9372215271e-02
       coeff_l3(41,44)= -5.2124667913e-02
       coeff_l3(41,45)= -7.3447301984e-02
       coeff_l3(41,46)= -4.1175656021e-02
       coeff_l3(41,47)= -1.3210833073e-01
       coeff_l3(41,48)= -1.5290157497e-01
       coeff_l3(41,49)= -1.1495231092e-01
       coeff_l3(41,50)= -2.5726151466e-01
       coeff_l3(41,51)=  3.5007118713e-03
       coeff_l3(41,52)= -2.3367149755e-02
       coeff_l3(41,53)=  2.4841742590e-02
       coeff_l3(41,54)= -2.3396459222e-01
       coeff_l3(41,55)= -2.5479452685e-02
       coeff_l3(41,56)=  7.7502839267e-02
       coeff_l3(41,57)= -1.3405624032e-01
       coeff_l3(41,58)=  3.4220242500e-01
       coeff_l3(41,59)= -4.4793531299e-01
       coeff_l3(41,60)= -8.2079869509e-01
       coeff_l3(41,61)=  7.6703503728e-02
       coeff_l3(41,62)= -1.2582462281e-02
       coeff_l3(41,63)= -1.0240598023e-01
       coeff_l3(41,64)= -6.9582559168e-02
       coeff_l3(41,65)=  1.0065934621e-02
       coeff_l3(41,66)= -1.3724936545e-01
       coeff_l3(41,67)= -2.0190197229e-01
       coeff_l3(41,68)= -2.3884490132e-01
       coeff_l3(41,69)= -3.0960664153e-01
       coeff_l3(41,70)= -5.2929106168e-03
       coeff_l3(41,71)= -5.4005914927e-01
       coeff_l3(41,72)=  2.4000887573e-01
       coeff_l3(41,73)=  7.5260153972e-04
       coeff_l3(41,74)= -3.2833218575e-02
       coeff_l3(41,75)=  2.6859232783e-01
       coeff_l3(41,76)= -1.1110936850e-01
       coeff_l3(41,77)= -1.1700489372e-01
       coeff_l3(41,78)=  1.7159689218e-02
       coeff_l3(41,79)=  1.0166244209e-01
       coeff_l3(41,80)= -1.6838269308e-02
       coeff_l3(41,81)= -1.7361731827e-01
       coeff_l3(41,82)=  8.0186448991e-02
       coeff_l3(41,83)= -1.3356538117e-01
       coeff_l3(41,84)= -4.2462144047e-03
       coeff_l3(41,85)= -2.7650365233e-01
       coeff_l3(41,86)= -1.8859259784e-01
       coeff_l3(41,87)= -2.6211142540e-01
       coeff_l3(41,88)= -2.7087801695e-01
       coeff_l3(41,89)=  3.6321949214e-02
       coeff_l3(41,90)=  3.0152708292e-02
       coeff_l3(41,91)=  1.1012163013e-01
       coeff_l3(41,92)=  6.7984429188e-03
       coeff_l3(41,93)= -4.9359377474e-02
       coeff_l3(41,94)= -1.1541218311e-01
       coeff_l3(41,95)= -1.2364985049e-01
       coeff_l3(41,96)= -2.7604109049e-01
       coeff_l3(41,97)=  6.7161768675e-02
       coeff_l3(41,98)= -1.3249342144e-01
       coeff_l3(41,99)=  7.8253582120e-02
       coeff_l3(41,100)= -1.4668294787e-01
       coeff_l3(41,101)=  8.7841339409e-02
       coeff_l3(41,102)= -1.9546183944e-01
       coeff_l3(41,103)=  6.8816351704e-03
       coeff_l3(41,104)=  1.0997767001e-01
       coeff_l3(41,105)= -2.0910115540e-01
       coeff_l3(41,106)= -1.0283102840e-01
       coeff_l3(41,107)= -5.4787391424e-01
       coeff_l3(41,108)=  4.2757693678e-02
       coeff_l3(41,109)= -1.7793087289e-02
       coeff_l3(41,110)=  6.1912253499e-02
       coeff_l3(41,111)= -9.5860652626e-02
       coeff_l3(41,112)= -2.0562799275e-01
       coeff_l3(41,113)= -3.0883648992e-01
       coeff_l3(41,114)= -5.9649068862e-02
       coeff_l3(41,115)= -1.7780841887e-01
       coeff_l3(41,116)= -3.9928051829e-01
       coeff_l3(41,117)= -2.5626179576e-01
       coeff_l3(41,118)= -1.1664593220e-01
       coeff_l3(41,119)= -5.8254621923e-02
       coeff_l3(41,120)= -4.7951519489e-02
       coeff_l3(41,121)= -3.1509083509e-01
       coeff_l3(41,122)= -1.5404449403e-01
       coeff_l3(41,123)=  1.1629740894e-01
       coeff_l3(41,124)=  4.3403029442e-02
       coeff_l3(41,125)=  2.5088796020e-01
       coeff_l3(42,1)= -3.8775838912e-02
       coeff_l3(42,2)=  2.9510950670e-02
       coeff_l3(42,3)=  6.7687900737e-03
       coeff_l3(42,4)=  2.3319127038e-02
       coeff_l3(42,5)=  1.4273902774e-01
       coeff_l3(42,6)= -2.2630309686e-02
       coeff_l3(42,7)= -4.9589727074e-02
       coeff_l3(42,8)=  9.5039673150e-02
       coeff_l3(42,9)= -8.0825917423e-02
       coeff_l3(42,10)= -1.8035367131e-02
       coeff_l3(42,11)= -1.1722223461e-01
       coeff_l3(42,12)= -1.5197179280e-02
       coeff_l3(42,13)=  3.7337257527e-03
       coeff_l3(42,14)=  7.8990980983e-02
       coeff_l3(42,15)= -1.7612767220e-01
       coeff_l3(42,16)=  9.6806034446e-02
       coeff_l3(42,17)= -5.4508227855e-02
       coeff_l3(42,18)= -3.9829324931e-02
       coeff_l3(42,19)= -1.8622793257e-01
       coeff_l3(42,20)=  6.1751842499e-02
       coeff_l3(42,21)= -9.7078420222e-03
       coeff_l3(42,22)=  8.3383053541e-02
       coeff_l3(42,23)= -9.9905515090e-03
       coeff_l3(42,24)= -1.4176566899e-01
       coeff_l3(42,25)= -1.2400435656e-01
       coeff_l3(42,26)= -1.4841030538e-01
       coeff_l3(42,27)= -1.2386208028e-01
       coeff_l3(42,28)= -8.9296303689e-02
       coeff_l3(42,29)= -2.3967335001e-02
       coeff_l3(42,30)= -1.0381675325e-02
       coeff_l3(42,31)=  6.7384235561e-02
       coeff_l3(42,32)=  5.4580733180e-02
       coeff_l3(42,33)=  9.8067566752e-02
       coeff_l3(42,34)=  3.6050196737e-02
       coeff_l3(42,35)=  7.3799766600e-02
       coeff_l3(42,36)=  8.8258340955e-02
       coeff_l3(42,37)=  1.1678360403e-01
       coeff_l3(42,38)= -1.5459848801e-03
       coeff_l3(42,39)=  7.8157573938e-02
       coeff_l3(42,40)=  7.3745891452e-02
       coeff_l3(42,41)=  8.9541725814e-02
       coeff_l3(42,42)=  4.7666866332e-02
       coeff_l3(42,43)=  8.9559219778e-02
       coeff_l3(42,44)= -4.4541969895e-02
       coeff_l3(42,45)=  9.7248971462e-02
       coeff_l3(42,46)=  1.3465340436e-01
       coeff_l3(42,47)= -1.5049311519e-01
       coeff_l3(42,48)=  8.7494915351e-03
       coeff_l3(42,49)= -4.3664783239e-01
       coeff_l3(42,50)=  1.2681193650e-01
       coeff_l3(42,51)=  1.0646340251e-01
       coeff_l3(42,52)=  7.6650403440e-02
       coeff_l3(42,53)= -6.8823620677e-02
       coeff_l3(42,54)= -1.1635357887e-01
       coeff_l3(42,55)=  2.3670690134e-02
       coeff_l3(42,56)= -1.0129881650e-01
       coeff_l3(42,57)= -6.3273355365e-02
       coeff_l3(42,58)= -1.0834570974e-01
       coeff_l3(42,59)=  2.0154831931e-02
       coeff_l3(42,60)= -1.3367240131e-01
       coeff_l3(42,61)= -8.4225371480e-02
       coeff_l3(42,62)= -4.5388903469e-02
       coeff_l3(42,63)= -2.4436524510e-01
       coeff_l3(42,64)=  1.0455044359e-01
       coeff_l3(42,65)=  1.1225871742e-01
       coeff_l3(42,66)=  6.0806374997e-02
       coeff_l3(42,67)= -1.4669684693e-02
       coeff_l3(42,68)=  5.2273198962e-02
       coeff_l3(42,69)=  2.2188604344e-03
       coeff_l3(42,70)= -1.2994497083e-02
       coeff_l3(42,71)=  1.9399882853e-01
       coeff_l3(42,72)= -2.3171095178e-02
       coeff_l3(42,73)=  4.4257972389e-02
       coeff_l3(42,74)=  2.2921515629e-02
       coeff_l3(42,75)= -1.9314062595e-01
       coeff_l3(42,76)= -2.1196442842e-01
       coeff_l3(42,77)= -2.0784871280e-01
       coeff_l3(42,78)= -1.6932916641e-01
       coeff_l3(42,79)=  4.6368416399e-02
       coeff_l3(42,80)= -2.7049061656e-01
       coeff_l3(42,81)= -4.7565281391e-02
       coeff_l3(42,82)=  4.6961072832e-02
       coeff_l3(42,83)= -1.4987157285e-01
       coeff_l3(42,84)= -1.9265134633e-01
       coeff_l3(42,85)= -7.8184686601e-02
       coeff_l3(42,86)= -9.6192061901e-02
       coeff_l3(42,87)=  1.6815790907e-02
       coeff_l3(42,88)= -3.0501639843e-01
       coeff_l3(42,89)=  2.7110910742e-04
       coeff_l3(42,90)= -2.0952603221e-01
       coeff_l3(42,91)= -9.6698090434e-02
       coeff_l3(42,92)= -2.3592762649e-02
       coeff_l3(42,93)= -3.8814783096e-02
       coeff_l3(42,94)=  1.4769643545e-01
       coeff_l3(42,95)=  3.6949831992e-02
       coeff_l3(42,96)=  4.9831313081e-03
       coeff_l3(42,97)=  1.0543860495e-01
       coeff_l3(42,98)= -4.4864185154e-02
       coeff_l3(42,99)=  1.2869472802e-01
       coeff_l3(42,100)= -1.1100171134e-02
       coeff_l3(42,101)=  4.5249707997e-02
       coeff_l3(42,102)= -3.6351859570e-02
       coeff_l3(42,103)= -7.9223170877e-02
       coeff_l3(42,104)= -2.9879737645e-02
       coeff_l3(42,105)=  1.7552383244e-01
       coeff_l3(42,106)=  1.3682826422e-02
       coeff_l3(42,107)= -3.4456126392e-02
       coeff_l3(42,108)=  5.2508149296e-02
       coeff_l3(42,109)= -2.6722084731e-02
       coeff_l3(42,110)=  1.9398610294e-01
       coeff_l3(42,111)= -2.8819879517e-02
       coeff_l3(42,112)= -2.3800523579e-01
       coeff_l3(42,113)= -8.1406660378e-02
       coeff_l3(42,114)=  1.0261807591e-02
       coeff_l3(42,115)= -2.2945338488e-01
       coeff_l3(42,116)= -1.7371813953e-01
       coeff_l3(42,117)= -2.7290688828e-02
       coeff_l3(42,118)=  2.2453927994e-01
       coeff_l3(42,119)= -2.0633254200e-02
       coeff_l3(42,120)= -8.9276293293e-03
       coeff_l3(42,121)=  2.2816760466e-02
       coeff_l3(42,122)=  2.2366160527e-02
       coeff_l3(42,123)=  7.6185390353e-02
       coeff_l3(42,124)=  5.4445348680e-02
       coeff_l3(42,125)= -6.9369651377e-02
       coeff_l3(43,1)=  1.7586469650e-01
       coeff_l3(43,2)= -1.8222257495e-01
       coeff_l3(43,3)= -2.2071042657e-01
       coeff_l3(43,4)= -4.8891621828e-01
       coeff_l3(43,5)= -5.5900909938e-03
       coeff_l3(43,6)= -5.4704177380e-01
       coeff_l3(43,7)= -7.3648345470e-01
       coeff_l3(43,8)=  2.1711367369e-01
       coeff_l3(43,9)= -1.3637386262e-01
       coeff_l3(43,10)=  2.4031659588e-02
       coeff_l3(43,11)=  3.3503079414e-01
       coeff_l3(43,12)=  7.1653312445e-01
       coeff_l3(43,13)=  3.6131531000e-01
       coeff_l3(43,14)=  4.8527839780e-01
       coeff_l3(43,15)= -3.7230396271e-01
       coeff_l3(43,16)= -2.6095899940e-01
       coeff_l3(43,17)=  1.9323571026e-01
       coeff_l3(43,18)=  4.0622040629e-01
       coeff_l3(43,19)= -2.3770697415e-01
       coeff_l3(43,20)= -3.6334094405e-01
       coeff_l3(43,21)=  1.6617882252e-01
       coeff_l3(43,22)= -2.3519765586e-02
       coeff_l3(43,23)= -5.0909274817e-01
       coeff_l3(43,24)= -1.3724629581e-01
       coeff_l3(43,25)= -2.3169133067e-01
       coeff_l3(43,26)= -2.3614071310e-01
       coeff_l3(43,27)= -1.8500850201e+00
       coeff_l3(43,28)= -1.3907639980e+00
       coeff_l3(43,29)=  4.0232378989e-02
       coeff_l3(43,30)= -9.1623926163e-01
       coeff_l3(43,31)= -3.1650331616e-01
       coeff_l3(43,32)=  2.0616596565e-02
       coeff_l3(43,33)= -2.9611399770e-01
       coeff_l3(43,34)= -1.2082234770e-01
       coeff_l3(43,35)=  7.2174243629e-02
       coeff_l3(43,36)=  2.9552546144e-01
       coeff_l3(43,37)= -2.0024143159e-01
       coeff_l3(43,38)=  6.6145735979e-01
       coeff_l3(43,39)= -4.7346451320e-03
       coeff_l3(43,40)=  6.2881730497e-02
       coeff_l3(43,41)=  4.3890932202e-01
       coeff_l3(43,42)= -5.5283528566e-01
       coeff_l3(43,43)=  3.2859438658e-01
       coeff_l3(43,44)= -4.2846000195e-01
       coeff_l3(43,45)= -1.9221219420e-01
       coeff_l3(43,46)=  1.1716063321e-01
       coeff_l3(43,47)= -5.8287656307e-01
       coeff_l3(43,48)= -1.7321644723e-01
       coeff_l3(43,49)= -1.3586091995e+00
       coeff_l3(43,50)=  4.2828831077e-01
       coeff_l3(43,51)=  3.5394933075e-02
       coeff_l3(43,52)=  6.9349773228e-02
       coeff_l3(43,53)= -2.3097923025e-02
       coeff_l3(43,54)= -6.5620593727e-02
       coeff_l3(43,55)= -3.6325696111e-01
       coeff_l3(43,56)= -5.8449547738e-02
       coeff_l3(43,57)=  7.1888580918e-02
       coeff_l3(43,58)= -1.0878343582e+00
       coeff_l3(43,59)=  7.2379338741e-01
       coeff_l3(43,60)= -1.1776758432e+00
       coeff_l3(43,61)= -6.2918829918e-01
       coeff_l3(43,62)= -5.8035647869e-01
       coeff_l3(43,63)= -5.6928116083e-01
       coeff_l3(43,64)= -2.3510581255e-01
       coeff_l3(43,65)= -2.6329398155e-01
       coeff_l3(43,66)= -2.9173311591e-01
       coeff_l3(43,67)= -6.3226515055e-01
       coeff_l3(43,68)=  4.4450563192e-01
       coeff_l3(43,69)=  5.1892185211e-01
       coeff_l3(43,70)= -3.4990096092e-01
       coeff_l3(43,71)=  5.8671110868e-01
       coeff_l3(43,72)= -4.4237309694e-01
       coeff_l3(43,73)=  4.3303796649e-01
       coeff_l3(43,74)= -2.4237373471e-01
       coeff_l3(43,75)= -2.5208935142e-01
       coeff_l3(43,76)= -7.8066104650e-01
       coeff_l3(43,77)= -4.0869721770e-01
       coeff_l3(43,78)= -7.1531541646e-02
       coeff_l3(43,79)= -8.0263540149e-02
       coeff_l3(43,80)= -5.9512156248e-01
       coeff_l3(43,81)=  1.3557427563e-02
       coeff_l3(43,82)=  1.3691010885e-02
       coeff_l3(43,83)=  9.6697762609e-02
       coeff_l3(43,84)=  6.0084003210e-01
       coeff_l3(43,85)=  2.8537115455e-01
       coeff_l3(43,86)=  2.4645691738e-02
       coeff_l3(43,87)= -1.6144479811e-01
       coeff_l3(43,88)=  2.7402141690e-01
       coeff_l3(43,89)= -7.8708714247e-01
       coeff_l3(43,90)= -9.9202811718e-01
       coeff_l3(43,91)= -5.9740114957e-02
       coeff_l3(43,92)= -2.1020103991e-01
       coeff_l3(43,93)= -7.6833977364e-03
       coeff_l3(43,94)=  1.5959537029e-01
       coeff_l3(43,95)= -1.3354736567e-01
       coeff_l3(43,96)=  2.5149166584e-01
       coeff_l3(43,97)= -1.8343760073e-01
       coeff_l3(43,98)= -8.0740764737e-02
       coeff_l3(43,99)=  2.6959913969e-01
       coeff_l3(43,100)= -1.0798543692e-01
       coeff_l3(43,101)= -4.1964888573e-02
       coeff_l3(43,102)= -2.2193514742e-03
       coeff_l3(43,103)= -1.8518248573e-02
       coeff_l3(43,104)= -1.0698529333e-01
       coeff_l3(43,105)=  5.6174749136e-01
       coeff_l3(43,106)=  3.9125287533e-01
       coeff_l3(43,107)=  4.7944158316e-01
       coeff_l3(43,108)=  4.4312810898e-01
       coeff_l3(43,109)= -3.8375426084e-02
       coeff_l3(43,110)=  4.8347808421e-02
       coeff_l3(43,111)=  4.1879868507e-01
       coeff_l3(43,112)=  4.8705264926e-01
       coeff_l3(43,113)=  1.9846020639e-01
       coeff_l3(43,114)= -2.2457195818e-01
       coeff_l3(43,115)=  3.5817509890e-01
       coeff_l3(43,116)=  1.1516641378e+00
       coeff_l3(43,117)= -5.4379466921e-02
       coeff_l3(43,118)=  8.1996805966e-02
       coeff_l3(43,119)= -1.9202242792e-01
       coeff_l3(43,120)= -1.3127703965e-01
       coeff_l3(43,121)=  4.4795849919e-01
       coeff_l3(43,122)= -2.0044291019e-01
       coeff_l3(43,123)= -2.9867208004e-01
       coeff_l3(43,124)=  1.9053119421e-01
       coeff_l3(43,125)=  1.4476557076e-01
       coeff_l3(44,1)=  4.5118016005e-01
       coeff_l3(44,2)= -2.0496343076e-01
       coeff_l3(44,3)= -2.8698515892e-01
       coeff_l3(44,4)= -1.3113355637e-01
       coeff_l3(44,5)=  2.9653742909e-01
       coeff_l3(44,6)= -1.5507101081e-02
       coeff_l3(44,7)= -5.0785714388e-01
       coeff_l3(44,8)=  9.5831640065e-02
       coeff_l3(44,9)= -1.4316939749e-02
       coeff_l3(44,10)=  1.3729731739e-01
       coeff_l3(44,11)= -6.2043294311e-03
       coeff_l3(44,12)= -4.7685724497e-01
       coeff_l3(44,13)= -4.6420529485e-01
       coeff_l3(44,14)=  2.7826181054e-01
       coeff_l3(44,15)= -6.1510741711e-02
       coeff_l3(44,16)= -1.3127785921e-01
       coeff_l3(44,17)= -7.1642331779e-02
       coeff_l3(44,18)= -5.2418810129e-01
       coeff_l3(44,19)= -1.2443336844e-01
       coeff_l3(44,20)=  1.6002163291e-01
       coeff_l3(44,21)= -2.6655149460e-01
       coeff_l3(44,22)=  1.2104881555e-01
       coeff_l3(44,23)= -5.9035025537e-02
       coeff_l3(44,24)= -6.0276690871e-02
       coeff_l3(44,25)= -3.1009411812e-01
       coeff_l3(44,26)=  1.5798422694e-01
       coeff_l3(44,27)= -1.1355261803e+00
       coeff_l3(44,28)= -8.8774281740e-01
       coeff_l3(44,29)= -4.9842208624e-01
       coeff_l3(44,30)= -3.6441057920e-02
       coeff_l3(44,31)=  3.9005182683e-02
       coeff_l3(44,32)= -1.0747102648e-01
       coeff_l3(44,33)= -2.1035289764e-01
       coeff_l3(44,34)=  1.2105901539e-01
       coeff_l3(44,35)=  3.3228909969e-01
       coeff_l3(44,36)= -1.2335899286e-02
       coeff_l3(44,37)=  8.5270227864e-03
       coeff_l3(44,38)= -1.4569351077e-01
       coeff_l3(44,39)= -1.5306836925e-02
       coeff_l3(44,40)= -1.3927382231e-01
       coeff_l3(44,41)= -2.0431040227e-01
       coeff_l3(44,42)= -1.1195595562e-01
       coeff_l3(44,43)= -4.0736947209e-02
       coeff_l3(44,44)= -1.7310771346e-01
       coeff_l3(44,45)=  4.3397074938e-01
       coeff_l3(44,46)= -2.5584781170e-01
       coeff_l3(44,47)= -3.5609441996e-01
       coeff_l3(44,48)= -2.4219837785e-01
       coeff_l3(44,49)= -1.1560379714e-01
       coeff_l3(44,50)= -1.1475119740e-02
       coeff_l3(44,51)= -9.9017359316e-02
       coeff_l3(44,52)= -1.1786769331e-01
       coeff_l3(44,53)=  2.1973535419e-01
       coeff_l3(44,54)=  2.2414645553e-01
       coeff_l3(44,55)= -3.3718742430e-02
       coeff_l3(44,56)= -2.7865624055e-02
       coeff_l3(44,57)= -1.4809390903e-01
       coeff_l3(44,58)= -9.8109193146e-02
       coeff_l3(44,59)= -1.2114521116e-01
       coeff_l3(44,60)=  1.1299705505e+00
       coeff_l3(44,61)= -1.5411083400e-01
       coeff_l3(44,62)= -1.4413234591e-01
       coeff_l3(44,63)= -2.3805338889e-02
       coeff_l3(44,64)=  8.7145663798e-02
       coeff_l3(44,65)=  1.4005182311e-02
       coeff_l3(44,66)= -2.4054995179e-01
       coeff_l3(44,67)= -8.7434291840e-01
       coeff_l3(44,68)=  2.5352874398e-01
       coeff_l3(44,69)= -1.6727964580e-01
       coeff_l3(44,70)= -4.0681916475e-01
       coeff_l3(44,71)= -2.7775028348e-01
       coeff_l3(44,72)= -1.0022634268e-01
       coeff_l3(44,73)= -2.7231770754e-01
       coeff_l3(44,74)= -2.6197639108e-01
       coeff_l3(44,75)=  7.2097033262e-02
       coeff_l3(44,76)=  2.0953959227e-01
       coeff_l3(44,77)=  1.0224561393e-01
       coeff_l3(44,78)= -4.9372050911e-02
       coeff_l3(44,79)=  2.6047194377e-02
       coeff_l3(44,80)=  8.8020578027e-02
       coeff_l3(44,81)= -2.1249291301e-01
       coeff_l3(44,82)= -1.3853959739e-01
       coeff_l3(44,83)= -1.1954712123e-01
       coeff_l3(44,84)= -3.2210674882e-01
       coeff_l3(44,85)=  2.6297541335e-02
       coeff_l3(44,86)=  5.5328313261e-02
       coeff_l3(44,87)= -7.6963968575e-02
       coeff_l3(44,88)= -8.5248179734e-02
       coeff_l3(44,89)= -3.6641120911e-01
       coeff_l3(44,90)= -2.4852482975e-01
       coeff_l3(44,91)=  1.0615282506e-01
       coeff_l3(44,92)= -2.2641769052e-01
       coeff_l3(44,93)=  3.9727020264e-01
       coeff_l3(44,94)= -2.9103705287e-01
       coeff_l3(44,95)= -4.0568938851e-01
       coeff_l3(44,96)= -1.5522539616e-01
       coeff_l3(44,97)=  5.5775805376e-03
       coeff_l3(44,98)= -1.3663469255e-01
       coeff_l3(44,99)=  7.2153974324e-03
       coeff_l3(44,100)= -3.5223335028e-01
       coeff_l3(44,101)= -4.3979054689e-01
       coeff_l3(44,102)= -2.0414356142e-02
       coeff_l3(44,103)=  1.8675267696e-01
       coeff_l3(44,104)=  8.6838066578e-02
       coeff_l3(44,105)= -4.0077473968e-02
       coeff_l3(44,106)= -2.4056863785e-01
       coeff_l3(44,107)=  2.0630970597e-02
       coeff_l3(44,108)= -7.7868387103e-02
       coeff_l3(44,109)= -4.5330442488e-02
       coeff_l3(44,110)=  1.7944329977e-01
       coeff_l3(44,111)= -2.0810434222e-01
       coeff_l3(44,112)=  1.9435109571e-02
       coeff_l3(44,113)= -9.2351131141e-02
       coeff_l3(44,114)=  1.5311446786e-01
       coeff_l3(44,115)=  6.6903762519e-02
       coeff_l3(44,116)= -1.7498613894e-01
       coeff_l3(44,117)= -1.9549163058e-02
       coeff_l3(44,118)= -2.9569324851e-01
       coeff_l3(44,119)= -5.1177754998e-02
       coeff_l3(44,120)= -2.3086740077e-01
       coeff_l3(44,121)= -6.6354960203e-02
       coeff_l3(44,122)= -8.3529122174e-02
       coeff_l3(44,123)= -1.2075325102e-01
       coeff_l3(44,124)= -1.8822312355e-01
       coeff_l3(44,125)= -1.9852855802e-01
       coeff_l3(45,1)= -1.9924390316e-01
       coeff_l3(45,2)= -7.4380546808e-02
       coeff_l3(45,3)= -1.0176531225e-01
       coeff_l3(45,4)=  1.2890791893e-01
       coeff_l3(45,5)= -4.6419348568e-02
       coeff_l3(45,6)= -1.8880151212e-02
       coeff_l3(45,7)=  4.4928038120e-01
       coeff_l3(45,8)=  3.4196007997e-02
       coeff_l3(45,9)= -1.7565625906e-01
       coeff_l3(45,10)= -8.2602344453e-02
       coeff_l3(45,11)=  1.5374171734e-01
       coeff_l3(45,12)= -5.3805583715e-01
       coeff_l3(45,13)= -3.5696139932e-01
       coeff_l3(45,14)= -1.7157407105e-01
       coeff_l3(45,15)=  1.0004388541e-01
       coeff_l3(45,16)= -4.4302277267e-02
       coeff_l3(45,17)= -9.2339418828e-02
       coeff_l3(45,18)= -5.0174826384e-01
       coeff_l3(45,19)= -7.6684527099e-02
       coeff_l3(45,20)= -3.4228050709e-01
       coeff_l3(45,21)= -4.9969755113e-02
       coeff_l3(45,22)=  6.6621817648e-02
       coeff_l3(45,23)= -1.9821929932e-01
       coeff_l3(45,24)= -5.4899346083e-02
       coeff_l3(45,25)= -1.4738237858e-01
       coeff_l3(45,26)= -2.0349175611e-04
       coeff_l3(45,27)=  4.6596285701e-01
       coeff_l3(45,28)=  2.4166834354e-01
       coeff_l3(45,29)= -3.1872382760e-01
       coeff_l3(45,30)= -5.0667995214e-01
       coeff_l3(45,31)= -3.6166003346e-01
       coeff_l3(45,32)= -1.0618030280e-01
       coeff_l3(45,33)=  3.1231302768e-02
       coeff_l3(45,34)=  3.1655630469e-01
       coeff_l3(45,35)=  2.7940350771e-01
       coeff_l3(45,36)= -3.4618321061e-01
       coeff_l3(45,37)= -1.4617082477e-01
       coeff_l3(45,38)=  6.6796410084e-01
       coeff_l3(45,39)=  1.0623801500e-01
       coeff_l3(45,40)=  1.3340550661e-01
       coeff_l3(45,41)= -3.5464420915e-01
       coeff_l3(45,42)= -2.2469529882e-02
       coeff_l3(45,43)= -3.1814720482e-02
       coeff_l3(45,44)= -1.0901521146e-01
       coeff_l3(45,45)= -2.3984856904e-01
       coeff_l3(45,46)= -2.2153660655e-01
       coeff_l3(45,47)=  2.2678729147e-03
       coeff_l3(45,48)= -8.8191628456e-02
       coeff_l3(45,49)=  1.0286666155e+00
       coeff_l3(45,50)= -5.5107462406e-01
       coeff_l3(45,51)= -1.4023037255e-01
       coeff_l3(45,52)= -1.4534348249e-01
       coeff_l3(45,53)= -1.0968860239e-01
       coeff_l3(45,54)= -1.1941802502e-01
       coeff_l3(45,55)=  2.6636379957e-01
       coeff_l3(45,56)=  1.4578551054e-01
       coeff_l3(45,57)=  1.4829717577e-01
       coeff_l3(45,58)= -1.1972949505e+00
       coeff_l3(45,59)= -3.5798129439e-01
       coeff_l3(45,60)=  1.3377915621e+00
       coeff_l3(45,61)= -5.9117600322e-02
       coeff_l3(45,62)= -2.6168274879e-01
       coeff_l3(45,63)=  2.3696509004e-01
       coeff_l3(45,64)= -1.4343158901e-01
       coeff_l3(45,65)= -6.7193480209e-03
       coeff_l3(45,66)=  8.2529494539e-03
       coeff_l3(45,67)=  5.9240770340e-01
       coeff_l3(45,68)= -3.0712363124e-01
       coeff_l3(45,69)= -1.4639626443e-01
       coeff_l3(45,70)= -7.3201112449e-02
       coeff_l3(45,71)= -4.0660804510e-01
       coeff_l3(45,72)=  4.5962065458e-02
       coeff_l3(45,73)= -3.8869059086e-01
       coeff_l3(45,74)=  1.8205618858e-01
       coeff_l3(45,75)= -3.9800864458e-01
       coeff_l3(45,76)=  7.4218040705e-01
       coeff_l3(45,77)=  1.6290566325e-01
       coeff_l3(45,78)=  1.1346502230e-02
       coeff_l3(45,79)= -2.0242445171e-01
       coeff_l3(45,80)=  3.8626351953e-01
       coeff_l3(45,81)= -6.4388471842e-01
       coeff_l3(45,82)=  8.5039928555e-02
       coeff_l3(45,83)= -3.4824941307e-02
       coeff_l3(45,84)= -5.5939847231e-01
       coeff_l3(45,85)= -2.9666256905e-01
       coeff_l3(45,86)= -2.0023660734e-02
       coeff_l3(45,87)= -8.0248050392e-02
       coeff_l3(45,88)= -5.8115895838e-02
       coeff_l3(45,89)= -2.1401728690e-01
       coeff_l3(45,90)=  1.1884244159e-02
       coeff_l3(45,91)=  9.2282339931e-02
       coeff_l3(45,92)= -1.0023206472e-01
       coeff_l3(45,93)= -3.2082194090e-01
       coeff_l3(45,94)=  3.1970599294e-01
       coeff_l3(45,95)= -3.5835620761e-01
       coeff_l3(45,96)= -1.9216898084e-01
       coeff_l3(45,97)= -5.2168860566e-04
       coeff_l3(45,98)=  1.3989171945e-02
       coeff_l3(45,99)= -3.0311835930e-02
       coeff_l3(45,100)= -1.8923485279e-01
       coeff_l3(45,101)= -1.2904559076e-01
       coeff_l3(45,102)= -5.6015614420e-02
       coeff_l3(45,103)= -3.1668156385e-01
       coeff_l3(45,104)= -7.5960099697e-02
       coeff_l3(45,105)= -4.7646820545e-01
       coeff_l3(45,106)= -1.3446268439e-01
       coeff_l3(45,107)= -3.4155043960e-01
       coeff_l3(45,108)= -1.2432835996e-01
       coeff_l3(45,109)= -1.8623268604e-01
       coeff_l3(45,110)= -1.7473886255e-03
       coeff_l3(45,111)=  1.0326023400e-01
       coeff_l3(45,112)= -5.2193880081e-01
       coeff_l3(45,113)= -5.4054760933e-01
       coeff_l3(45,114)= -7.4287787080e-02
       coeff_l3(45,115)= -2.9090267420e-01
       coeff_l3(45,116)= -2.2305607796e-01
       coeff_l3(45,117)=  1.3701586053e-02
       coeff_l3(45,118)=  5.6687974930e-01
       coeff_l3(45,119)=  3.9126083255e-02
       coeff_l3(45,120)= -1.3138781488e-01
       coeff_l3(45,121)= -2.0152102411e-01
       coeff_l3(45,122)=  4.0295202285e-02
       coeff_l3(45,123)= -1.7521443963e-01
       coeff_l3(45,124)= -2.8848532587e-02
       coeff_l3(45,125)= -3.6354057491e-02
       coeff_l3(46,1)=  1.4405316114e-01
       coeff_l3(46,2)= -3.7713783979e-01
       coeff_l3(46,3)= -6.9053041935e-01
       coeff_l3(46,4)= -3.6546492577e-01
       coeff_l3(46,5)= -6.2257599086e-02
       coeff_l3(46,6)= -1.0494343191e-01
       coeff_l3(46,7)= -5.1761072874e-01
       coeff_l3(46,8)= -3.8518616557e-01
       coeff_l3(46,9)= -2.5464591384e-01
       coeff_l3(46,10)= -3.2939639688e-01
       coeff_l3(46,11)= -1.6404156387e-01
       coeff_l3(46,12)= -2.0200130343e-01
       coeff_l3(46,13)= -4.9777355790e-01
       coeff_l3(46,14)= -1.7425386608e-01
       coeff_l3(46,15)= -2.4218860269e-01
       coeff_l3(46,16)= -1.4915429056e-01
       coeff_l3(46,17)= -2.3128105700e-01
       coeff_l3(46,18)= -8.7341618538e-01
       coeff_l3(46,19)=  3.4085206687e-02
       coeff_l3(46,20)= -5.2714133263e-01
       coeff_l3(46,21)= -2.6864245534e-01
       coeff_l3(46,22)= -2.7604189515e-01
       coeff_l3(46,23)= -8.4769539535e-02
       coeff_l3(46,24)= -1.9039681554e-01
       coeff_l3(46,25)=  1.5297058225e-01
       coeff_l3(46,26)=  5.9847515076e-02
       coeff_l3(46,27)= -8.2020297647e-02
       coeff_l3(46,28)= -5.6162905693e-01
       coeff_l3(46,29)= -8.6037701368e-01
       coeff_l3(46,30)= -5.4424619675e-01
       coeff_l3(46,31)= -1.9116206467e-01
       coeff_l3(46,32)= -1.8506263196e-01
       coeff_l3(46,33)= -5.0149118900e-01
       coeff_l3(46,34)= -9.0320976451e-03
       coeff_l3(46,35)= -8.2517242432e-01
       coeff_l3(46,36)= -3.1581389904e-01
       coeff_l3(46,37)= -6.2725998461e-02
       coeff_l3(46,38)= -6.0204595327e-01
       coeff_l3(46,39)= -7.5819849968e-02
       coeff_l3(46,40)= -4.6368309855e-01
       coeff_l3(46,41)= -1.0666353703e+00
       coeff_l3(46,42)= -4.8243656754e-02
       coeff_l3(46,43)= -1.0329066217e-01
       coeff_l3(46,44)=  6.1359219253e-02
       coeff_l3(46,45)= -1.9533312321e-01
       coeff_l3(46,46)= -2.0409983397e-01
       coeff_l3(46,47)= -1.9003899395e-01
       coeff_l3(46,48)= -6.4466732740e-01
       coeff_l3(46,49)= -2.2069568634e+00
       coeff_l3(46,50)=  4.1998735070e-01
       coeff_l3(46,51)=  3.0039191246e-01
       coeff_l3(46,52)= -8.3588846028e-02
       coeff_l3(46,53)= -1.7399545014e-01
       coeff_l3(46,54)= -3.5416778922e-01
       coeff_l3(46,55)= -4.3896627426e-01
       coeff_l3(46,56)= -4.3004322052e-01
       coeff_l3(46,57)= -6.5874516964e-02
       coeff_l3(46,58)= -6.4762210846e-01
       coeff_l3(46,59)=  8.8687635958e-02
       coeff_l3(46,60)= -2.2362871170e+00
       coeff_l3(46,61)=  6.4481481910e-02
       coeff_l3(46,62)= -3.3322833478e-02
       coeff_l3(46,63)= -5.8517432213e-01
       coeff_l3(46,64)= -2.6944151521e-01
       coeff_l3(46,65)= -1.8195147812e-01
       coeff_l3(46,66)= -2.3016773164e-01
       coeff_l3(46,67)= -3.6390647292e-02
       coeff_l3(46,68)= -2.3920187354e-01
       coeff_l3(46,69)= -1.8913248554e-02
       coeff_l3(46,70)= -2.6218974590e-01
       coeff_l3(46,71)= -8.2523375750e-01
       coeff_l3(46,72)= -1.9755849242e-01
       coeff_l3(46,73)=  1.2748891115e-01
       coeff_l3(46,74)= -2.4446137249e-01
       coeff_l3(46,75)= -2.2361375391e-01
       coeff_l3(46,76)= -2.0202636719e+00
       coeff_l3(46,77)=  3.0703881383e-01
       coeff_l3(46,78)= -2.3820409551e-02
       coeff_l3(46,79)= -2.0275771618e-01
       coeff_l3(46,80)= -1.0077747107e+00
       coeff_l3(46,81)= -3.0644200742e-03
       coeff_l3(46,82)= -9.6623629332e-02
       coeff_l3(46,83)=  4.8099972308e-02
       coeff_l3(46,84)=  5.2122926712e-01
       coeff_l3(46,85)= -6.8193875253e-02
       coeff_l3(46,86)= -3.9998576045e-02
       coeff_l3(46,87)=  5.3173783235e-03
       coeff_l3(46,88)=  1.4685486257e-01
       coeff_l3(46,89)=  9.4058461487e-02
       coeff_l3(46,90)= -1.6755476594e-01
       coeff_l3(46,91)= -6.0399925709e-01
       coeff_l3(46,92)= -1.4728251100e-01
       coeff_l3(46,93)=  2.7897170186e-01
       coeff_l3(46,94)= -2.7284982800e-01
       coeff_l3(46,95)= -2.3630106449e-01
       coeff_l3(46,96)= -2.3831799626e-01
       coeff_l3(46,97)= -2.6531293988e-01
       coeff_l3(46,98)=  2.6360137388e-02
       coeff_l3(46,99)=  2.1300867200e-02
       coeff_l3(46,100)= -1.8433856964e-01
       coeff_l3(46,101)= -8.5445135832e-02
       coeff_l3(46,102)=  9.8933868110e-02
       coeff_l3(46,103)=  1.5742221475e-01
       coeff_l3(46,104)= -2.2439979017e-01
       coeff_l3(46,105)= -6.1182748526e-02
       coeff_l3(46,106)= -9.8846375942e-02
       coeff_l3(46,107)= -4.9014285207e-01
       coeff_l3(46,108)= -1.0828076303e-01
       coeff_l3(46,109)= -1.3523480296e-01
       coeff_l3(46,110)=  4.1738813743e-03
       coeff_l3(46,111)= -2.3447597027e-01
       coeff_l3(46,112)= -1.5165713429e-01
       coeff_l3(46,113)= -4.9662643671e-01
       coeff_l3(46,114)= -2.4668878317e-01
       coeff_l3(46,115)= -1.6784039140e-01
       coeff_l3(46,116)= -7.8403013945e-01
       coeff_l3(46,117)=  6.6918477416e-02
       coeff_l3(46,118)=  3.4529057145e-01
       coeff_l3(46,119)= -1.6310791671e-01
       coeff_l3(46,120)= -1.4234295487e-01
       coeff_l3(46,121)=  3.0823609233e-01
       coeff_l3(46,122)= -2.9220578074e-01
       coeff_l3(46,123)= -1.0246132314e-01
       coeff_l3(46,124)=  6.5463341773e-02
       coeff_l3(46,125)= -2.2115652263e-01
       coeff_l3(47,1)=  1.6640840471e-01
       coeff_l3(47,2)= -1.6402767599e-01
       coeff_l3(47,3)= -5.5462479591e-01
       coeff_l3(47,4)= -1.0234934092e+00
       coeff_l3(47,5)=  1.2697800994e-02
       coeff_l3(47,6)= -3.8174217939e-01
       coeff_l3(47,7)= -5.6237286329e-01
       coeff_l3(47,8)= -8.4569150209e-01
       coeff_l3(47,9)=  2.0342789590e-02
       coeff_l3(47,10)=  2.8275931254e-02
       coeff_l3(47,11)= -2.5323545933e-01
       coeff_l3(47,12)= -3.8039177656e-01
       coeff_l3(47,13)= -1.8448042870e-01
       coeff_l3(47,14)=  2.4061219394e-01
       coeff_l3(47,15)= -4.2195402086e-02
       coeff_l3(47,16)= -1.3468338549e-01
       coeff_l3(47,17)= -1.3265870512e-02
       coeff_l3(47,18)= -1.6915664673e+00
       coeff_l3(47,19)=  3.2395713031e-02
       coeff_l3(47,20)=  5.7045078278e-01
       coeff_l3(47,21)= -1.3309946656e-01
       coeff_l3(47,22)=  1.2045416981e-01
       coeff_l3(47,23)=  3.6986168474e-02
       coeff_l3(47,24)=  2.4317207932e-01
       coeff_l3(47,25)= -2.4082713127e+00
       coeff_l3(47,26)=  4.6000607312e-02
       coeff_l3(47,27)= -8.8338357210e-01
       coeff_l3(47,28)= -4.9686932564e-01
       coeff_l3(47,29)= -1.1806653738e+00
       coeff_l3(47,30)= -5.7259297371e-01
       coeff_l3(47,31)=  2.1970050037e-01
       coeff_l3(47,32)=  1.1770714819e-01
       coeff_l3(47,33)= -6.6345554590e-01
       coeff_l3(47,34)=  1.0636036098e-01
       coeff_l3(47,35)= -4.6089228988e-01
       coeff_l3(47,36)= -3.9011634886e-02
       coeff_l3(47,37)= -1.0083293170e-01
       coeff_l3(47,38)= -5.5781012774e-01
       coeff_l3(47,39)= -6.4723104239e-02
       coeff_l3(47,40)= -3.4738010168e-01
       coeff_l3(47,41)=  6.8516099453e-01
       coeff_l3(47,42)= -5.2002158016e-02
       coeff_l3(47,43)= -1.4560952783e-01
       coeff_l3(47,44)= -1.3936623931e-01
       coeff_l3(47,45)=  1.6839528084e-01
       coeff_l3(47,46)= -1.3830919266e+00
       coeff_l3(47,47)= -3.6079850793e-01
       coeff_l3(47,48)= -7.9529184103e-01
       coeff_l3(47,49)= -1.0626370907e+00
       coeff_l3(47,50)= -4.1178487241e-02
       coeff_l3(47,51)=  6.0119915754e-02
       coeff_l3(47,52)=  3.6395256757e-04
       coeff_l3(47,53)= -2.8807911277e-01
       coeff_l3(47,54)= -6.7699038982e-01
       coeff_l3(47,55)= -6.8238234520e-01
       coeff_l3(47,56)= -4.8721599579e-01
       coeff_l3(47,57)= -2.6950010657e-01
       coeff_l3(47,58)= -8.3457845449e-01
       coeff_l3(47,59)=  2.6934033632e-01
       coeff_l3(47,60)= -1.1490902901e+00
       coeff_l3(47,61)= -1.7251896858e-01
       coeff_l3(47,62)= -9.1042695567e-03
       coeff_l3(47,63)= -1.7355424166e-01
       coeff_l3(47,64)= -5.1616036892e-01
       coeff_l3(47,65)= -2.5774285197e-01
       coeff_l3(47,66)= -2.4774914980e-01
       coeff_l3(47,67)= -1.0609989166e+00
       coeff_l3(47,68)=  2.4929119647e-01
       coeff_l3(47,69)=  3.0124088749e-02
       coeff_l3(47,70)= -2.9075571895e-01
       coeff_l3(47,71)= -1.6602178812e+00
       coeff_l3(47,72)= -2.2192393243e-01
       coeff_l3(47,73)=  8.7954819202e-02
       coeff_l3(47,74)= -7.2347819805e-01
       coeff_l3(47,75)= -1.8507964909e-01
       coeff_l3(47,76)= -5.9491938353e-01
       coeff_l3(47,77)=  1.5510147810e-01
       coeff_l3(47,78)=  2.0964010060e-01
       coeff_l3(47,79)= -2.0126845629e-04
       coeff_l3(47,80)= -6.2799561024e-01
       coeff_l3(47,81)= -1.2489463389e-01
       coeff_l3(47,82)= -5.7107818127e-01
       coeff_l3(47,83)= -2.4327483773e-01
       coeff_l3(47,84)=  4.1311245412e-02
       coeff_l3(47,85)=  3.4208551049e-02
       coeff_l3(47,86)=  1.3705214858e-01
       coeff_l3(47,87)= -2.0814390481e-01
       coeff_l3(47,88)= -9.0292900801e-02
       coeff_l3(47,89)= -9.7548991442e-01
       coeff_l3(47,90)= -6.0477530956e-01
       coeff_l3(47,91)=  1.4976200461e-01
       coeff_l3(47,92)= -5.2675074339e-01
       coeff_l3(47,93)= -5.6010577828e-02
       coeff_l3(47,94)= -7.7611917257e-01
       coeff_l3(47,95)= -3.7476739287e-01
       coeff_l3(47,96)= -7.9588949680e-01
       coeff_l3(47,97)= -2.6977664232e-01
       coeff_l3(47,98)= -1.8662393093e-01
       coeff_l3(47,99)= -9.1918356717e-02
       coeff_l3(47,100)= -4.1150057316e-01
       coeff_l3(47,101)= -2.3216515779e-01
       coeff_l3(47,102)= -7.1173705161e-02
       coeff_l3(47,103)=  4.8107292503e-02
       coeff_l3(47,104)=  1.6751272976e-01
       coeff_l3(47,105)= -3.3332683146e-02
       coeff_l3(47,106)= -1.2747550011e-01
       coeff_l3(47,107)=  2.0427082479e-01
       coeff_l3(47,108)= -1.0764201730e-01
       coeff_l3(47,109)=  1.8652398884e-01
       coeff_l3(47,110)=  2.1699009836e-01
       coeff_l3(47,111)= -5.2161169052e-01
       coeff_l3(47,112)=  1.8861357868e-01
       coeff_l3(47,113)= -5.2182388306e-01
       coeff_l3(47,114)=  7.2752773762e-01
       coeff_l3(47,115)=  3.4150991589e-02
       coeff_l3(47,116)= -1.2672039680e-02
       coeff_l3(47,117)= -1.9838212729e+00
       coeff_l3(47,118)= -4.4008988142e-01
       coeff_l3(47,119)= -2.3489321768e-01
       coeff_l3(47,120)= -4.8047471046e-01
       coeff_l3(47,121)=  3.6127042025e-02
       coeff_l3(47,122)= -1.0935354978e-01
       coeff_l3(47,123)= -2.4087359011e-01
       coeff_l3(47,124)= -3.7447607517e-01
       coeff_l3(47,125)=  7.6656371355e-02
       coeff_l3(48,1)= -1.5540385246e-01
       coeff_l3(48,2)= -1.3831944205e-02
       coeff_l3(48,3)=  6.6538296640e-02
       coeff_l3(48,4)= -7.7273733914e-02
       coeff_l3(48,5)=  1.2916466594e-01
       coeff_l3(48,6)=  1.2679065764e-01
       coeff_l3(48,7)= -1.9316435792e-03
       coeff_l3(48,8)= -2.7568636462e-02
       coeff_l3(48,9)=  9.2352457345e-02
       coeff_l3(48,10)= -1.0727719963e-01
       coeff_l3(48,11)= -9.0166859329e-02
       coeff_l3(48,12)=  1.7375975847e-01
       coeff_l3(48,13)= -1.9391773269e-02
       coeff_l3(48,14)=  7.1049652994e-02
       coeff_l3(48,15)=  3.4884434193e-02
       coeff_l3(48,16)=  1.0230166465e-01
       coeff_l3(48,17)=  6.8046256900e-02
       coeff_l3(48,18)=  1.0642142594e-01
       coeff_l3(48,19)= -1.9998500124e-02
       coeff_l3(48,20)= -9.2945016921e-02
       coeff_l3(48,21)= -9.4357468188e-02
       coeff_l3(48,22)=  5.1517244428e-03
       coeff_l3(48,23)=  3.6215893924e-02
       coeff_l3(48,24)= -4.8067984171e-03
       coeff_l3(48,25)= -1.3871313632e-01
       coeff_l3(48,26)=  8.1252209842e-02
       coeff_l3(48,27)=  2.4325005710e-01
       coeff_l3(48,28)=  2.0760472119e-01
       coeff_l3(48,29)= -5.4192811251e-02
       coeff_l3(48,30)= -3.7908118218e-02
       coeff_l3(48,31)= -1.6796113551e-01
       coeff_l3(48,32)= -7.5085293502e-03
       coeff_l3(48,33)= -1.3551229239e-01
       coeff_l3(48,34)=  3.2567791641e-03
       coeff_l3(48,35)= -1.2523417175e-01
       coeff_l3(48,36)= -1.0606632382e-01
       coeff_l3(48,37)= -6.8996042013e-02
       coeff_l3(48,38)=  1.7727343738e-01
       coeff_l3(48,39)=  1.5583871573e-04
       coeff_l3(48,40)=  1.1835331470e-01
       coeff_l3(48,41)=  8.2998976111e-02
       coeff_l3(48,42)=  1.6486080363e-02
       coeff_l3(48,43)=  2.3102703691e-01
       coeff_l3(48,44)= -2.3335647583e-01
       coeff_l3(48,45)=  4.8743512481e-02
       coeff_l3(48,46)=  1.2060777098e-01
       coeff_l3(48,47)=  5.2648972720e-02
       coeff_l3(48,48)= -2.1424165461e-03
       coeff_l3(48,49)= -3.2282543182e-01
       coeff_l3(48,50)= -3.8551457226e-02
       coeff_l3(48,51)=  8.5656065494e-03
       coeff_l3(48,52)= -1.8033288419e-01
       coeff_l3(48,53)=  4.7719832510e-02
       coeff_l3(48,54)= -8.6999855936e-02
       coeff_l3(48,55)= -1.3198602013e-02
       coeff_l3(48,56)=  1.4672043920e-01
       coeff_l3(48,57)=  1.2805238366e-01
       coeff_l3(48,58)= -2.1093818545e-01
       coeff_l3(48,59)= -4.4116382487e-03
       coeff_l3(48,60)= -2.6080447435e-01
       coeff_l3(48,61)=  1.1927565187e-01
       coeff_l3(48,62)= -7.9889900982e-02
       coeff_l3(48,63)= -2.0754058659e-01
       coeff_l3(48,64)=  4.1128460318e-02
       coeff_l3(48,65)= -1.5218544006e-01
       coeff_l3(48,66)=  1.6471788287e-01
       coeff_l3(48,67)=  3.3985733986e-01
       coeff_l3(48,68)= -8.5386119783e-02
       coeff_l3(48,69)= -3.1757906079e-01
       coeff_l3(48,70)= -1.0036315024e-01
       coeff_l3(48,71)=  2.4309109151e-01
       coeff_l3(48,72)= -1.2634147704e-01
       coeff_l3(48,73)= -4.3744980358e-03
       coeff_l3(48,74)= -6.6880263388e-02
       coeff_l3(48,75)= -2.2328546643e-01
       coeff_l3(48,76)= -2.5739270449e-01
       coeff_l3(48,77)= -1.7966786027e-01
       coeff_l3(48,78)=  1.5915831551e-02
       coeff_l3(48,79)= -3.5286363214e-02
       coeff_l3(48,80)= -1.3314539194e-01
       coeff_l3(48,81)=  7.9628629610e-03
       coeff_l3(48,82)=  1.4203256369e-01
       coeff_l3(48,83)= -6.8568184972e-02
       coeff_l3(48,84)= -5.9090334922e-02
       coeff_l3(48,85)=  1.0656988621e-01
       coeff_l3(48,86)= -1.2043005228e-01
       coeff_l3(48,87)=  1.1166583747e-01
       coeff_l3(48,88)= -7.9544492066e-02
       coeff_l3(48,89)= -1.3244313002e-01
       coeff_l3(48,90)= -1.9241368771e-01
       coeff_l3(48,91)= -1.1387962848e-02
       coeff_l3(48,92)=  1.3398407400e-01
       coeff_l3(48,93)= -8.0525003374e-02
       coeff_l3(48,94)= -1.8790541217e-02
       coeff_l3(48,95)=  1.0203509592e-02
       coeff_l3(48,96)=  1.1560852826e-01
       coeff_l3(48,97)=  1.3180746697e-02
       coeff_l3(48,98)= -1.7826959491e-01
       coeff_l3(48,99)= -8.1937491894e-02
       coeff_l3(48,100)=  2.1260732412e-01
       coeff_l3(48,101)=  1.4214881696e-02
       coeff_l3(48,102)= -1.7341774702e-01
       coeff_l3(48,103)= -1.2594692409e-01
       coeff_l3(48,104)=  6.4265154302e-02
       coeff_l3(48,105)=  3.2707488537e-01
       coeff_l3(48,106)= -3.6493837833e-02
       coeff_l3(48,107)= -7.6947726309e-02
       coeff_l3(48,108)=  2.1653199196e-01
       coeff_l3(48,109)=  7.1858447045e-03
       coeff_l3(48,110)=  1.4691135287e-01
       coeff_l3(48,111)=  2.9577253386e-02
       coeff_l3(48,112)= -6.2099568546e-02
       coeff_l3(48,113)=  1.8089310825e-01
       coeff_l3(48,114)=  1.8675720319e-02
       coeff_l3(48,115)=  8.6842559278e-02
       coeff_l3(48,116)= -1.6645856202e-01
       coeff_l3(48,117)= -8.8585846126e-02
       coeff_l3(48,118)=  2.0919236541e-01
       coeff_l3(48,119)=  1.7022725940e-01
       coeff_l3(48,120)=  5.2130781114e-02
       coeff_l3(48,121)= -9.1834001243e-02
       coeff_l3(48,122)= -1.0967945680e-02
       coeff_l3(48,123)= -1.5896645188e-01
       coeff_l3(48,124)= -5.4209128022e-02
       coeff_l3(48,125)=  1.2729890645e-01
       coeff_l3(49,1)= -4.8462653160e-01
       coeff_l3(49,2)=  8.6052216589e-02
       coeff_l3(49,3)= -2.6068955660e-01
       coeff_l3(49,4)= -8.0012351274e-02
       coeff_l3(49,5)= -7.6140873134e-02
       coeff_l3(49,6)=  2.3613362014e-01
       coeff_l3(49,7)=  8.9411646128e-02
       coeff_l3(49,8)= -2.3378914595e-01
       coeff_l3(49,9)=  1.2026160210e-01
       coeff_l3(49,10)=  8.2776814699e-02
       coeff_l3(49,11)= -3.1376089901e-02
       coeff_l3(49,12)= -3.8703498244e-01
       coeff_l3(49,13)= -3.3455708623e-01
       coeff_l3(49,14)= -1.5389584005e-01
       coeff_l3(49,15)=  1.7739719152e-01
       coeff_l3(49,16)=  1.5113577247e-01
       coeff_l3(49,17)= -3.0298647285e-01
       coeff_l3(49,18)= -5.5696678162e-01
       coeff_l3(49,19)=  6.2634415925e-02
       coeff_l3(49,20)= -7.5631372631e-02
       coeff_l3(49,21)=  1.1830259115e-01
       coeff_l3(49,22)=  1.1947301030e-01
       coeff_l3(49,23)=  1.2230773270e-01
       coeff_l3(49,24)= -9.6042059362e-02
       coeff_l3(49,25)=  2.1791490912e-01
       coeff_l3(49,26)=  4.8340272158e-02
       coeff_l3(49,27)= -1.1394312978e-01
       coeff_l3(49,28)= -9.8139256239e-02
       coeff_l3(49,29)= -6.1871576309e-01
       coeff_l3(49,30)=  4.5291799307e-01
       coeff_l3(49,31)=  1.8827332184e-02
       coeff_l3(49,32)=  1.6694620252e-01
       coeff_l3(49,33)=  8.9080132544e-02
       coeff_l3(49,34)=  9.6447959542e-02
       coeff_l3(49,35)=  2.7020129561e-01
       coeff_l3(49,36)=  4.6248774976e-02
       coeff_l3(49,37)=  1.4750882983e-01
       coeff_l3(49,38)= -1.1032052040e+00
       coeff_l3(49,39)=  2.0295639336e-01
       coeff_l3(49,40)= -2.2917440534e-01
       coeff_l3(49,41)= -7.3507821560e-01
       coeff_l3(49,42)= -4.1232034564e-02
       coeff_l3(49,43)= -1.7792187631e-01
       coeff_l3(49,44)=  2.9151281714e-01
       coeff_l3(49,45)=  7.5046233833e-02
       coeff_l3(49,46)=  8.3642108366e-03
       coeff_l3(49,47)=  2.9419315979e-02
       coeff_l3(49,48)= -3.3052846789e-01
       coeff_l3(49,49)= -3.0609592795e-01
       coeff_l3(49,50)= -1.3502819836e-01
       coeff_l3(49,51)= -7.6373569667e-02
       coeff_l3(49,52)= -1.0321012884e-01
       coeff_l3(49,53)= -1.7024789704e-03
       coeff_l3(49,54)= -9.7116470337e-02
       coeff_l3(49,55)=  2.6981180534e-02
       coeff_l3(49,56)=  5.1711965352e-02
       coeff_l3(49,57)=  1.1434000731e-01
       coeff_l3(49,58)=  6.2659299374e-01
       coeff_l3(49,59)= -4.4720354676e-01
       coeff_l3(49,60)= -3.4534931183e-01
       coeff_l3(49,61)=  1.9941404462e-01
       coeff_l3(49,62)=  7.6876893640e-02
       coeff_l3(49,63)=  1.1938087642e-01
       coeff_l3(49,64)=  9.9434545264e-03
       coeff_l3(49,65)=  2.5791656971e-01
       coeff_l3(49,66)=  3.3843342215e-02
       coeff_l3(49,67)=  2.1744337678e-01
       coeff_l3(49,68)= -2.6956877112e-01
       coeff_l3(49,69)= -1.6640818119e-01
       coeff_l3(49,70)= -6.8428456783e-02
       coeff_l3(49,71)= -5.7504034042e-01
       coeff_l3(49,72)=  5.0319459289e-02
       coeff_l3(49,73)= -3.0508765578e-01
       coeff_l3(49,74)=  8.3723835647e-02
       coeff_l3(49,75)=  1.3596223295e-01
       coeff_l3(49,76)=  1.9048569724e-02
       coeff_l3(49,77)= -6.5982088447e-02
       coeff_l3(49,78)=  4.4105178677e-03
       coeff_l3(49,79)=  4.8945568502e-02
       coeff_l3(49,80)= -2.2447794676e-01
       coeff_l3(49,81)= -3.8217699528e-01
       coeff_l3(49,82)= -2.7215373516e-01
       coeff_l3(49,83)=  5.7482019067e-02
       coeff_l3(49,84)=  1.3850515708e-02
       coeff_l3(49,85)= -1.1616725475e-01
       coeff_l3(49,86)= -7.7639169991e-02
       coeff_l3(49,87)= -4.7208774835e-02
       coeff_l3(49,88)= -1.3764080405e-01
       coeff_l3(49,89)=  3.1948890537e-02
       coeff_l3(49,90)=  1.6206577420e-01
       coeff_l3(49,91)=  1.5861002728e-02
       coeff_l3(49,92)= -1.0073387995e-02
       coeff_l3(49,93)= -1.3118439913e-01
       coeff_l3(49,94)= -3.5210275650e-01
       coeff_l3(49,95)= -4.3671771884e-02
       coeff_l3(49,96)= -2.1171043813e-01
       coeff_l3(49,97)= -1.9819934666e-01
       coeff_l3(49,98)=  1.5217715502e-01
       coeff_l3(49,99)=  7.4115984142e-02
       coeff_l3(49,100)= -3.0144178867e-01
       coeff_l3(49,101)= -3.4592062235e-02
       coeff_l3(49,102)=  4.1332025081e-02
       coeff_l3(49,103)=  5.5616345257e-02
       coeff_l3(49,104)= -1.2567263842e-01
       coeff_l3(49,105)= -3.5234636068e-01
       coeff_l3(49,106)= -8.7515525520e-02
       coeff_l3(49,107)= -2.1745172143e-01
       coeff_l3(49,108)= -1.5683697164e-01
       coeff_l3(49,109)=  5.7457339019e-02
       coeff_l3(49,110)= -1.7928043008e-01
       coeff_l3(49,111)= -1.7672754824e-01
       coeff_l3(49,112)= -5.0209486485e-01
       coeff_l3(49,113)= -2.3159289360e-01
       coeff_l3(49,114)=  1.1910407245e-01
       coeff_l3(49,115)= -3.7224957347e-01
       coeff_l3(49,116)= -6.7324197292e-01
       coeff_l3(49,117)= -1.0528118908e-01
       coeff_l3(49,118)= -1.5067887306e-01
       coeff_l3(49,119)=  1.0697443038e-01
       coeff_l3(49,120)= -1.0420368053e-03
       coeff_l3(49,121)= -3.8934496045e-01
       coeff_l3(49,122)=  1.0895536840e-01
       coeff_l3(49,123)=  2.3969924450e-01
       coeff_l3(49,124)= -5.6696191430e-02
       coeff_l3(49,125)=  1.3876046240e-01
       coeff_l3(50,1)= -1.6425083578e-01
       coeff_l3(50,2)= -1.7419479787e-01
       coeff_l3(50,3)= -3.5644856095e-01
       coeff_l3(50,4)= -4.6598741412e-01
       coeff_l3(50,5)=  5.0122171640e-02
       coeff_l3(50,6)=  2.1304963157e-02
       coeff_l3(50,7)=  2.4821898937e+00
       coeff_l3(50,8)= -1.8970509768e+00
       coeff_l3(50,9)= -2.3009099066e-01
       coeff_l3(50,10)= -3.3249333501e-01
       coeff_l3(50,11)=  3.7209838629e-01
       coeff_l3(50,12)= -5.3282719105e-02
       coeff_l3(50,13)= -4.7673279047e-01
       coeff_l3(50,14)= -2.0324315131e-01
       coeff_l3(50,15)= -1.5825128555e-01
       coeff_l3(50,16)=  9.1987378895e-02
       coeff_l3(50,17)= -4.1403025389e-01
       coeff_l3(50,18)= -1.2963105440e+00
       coeff_l3(50,19)=  1.1344911903e-01
       coeff_l3(50,20)= -1.5110203028e+00
       coeff_l3(50,21)=  1.1866428703e-01
       coeff_l3(50,22)= -3.7228416651e-02
       coeff_l3(50,23)= -6.1278954148e-02
       coeff_l3(50,24)= -6.8855506182e-01
       coeff_l3(50,25)= -8.3715409040e-01
       coeff_l3(50,26)=  5.6186538190e-02
       coeff_l3(50,27)=  2.1908154488e+00
       coeff_l3(50,28)=  2.2969801426e+00
       coeff_l3(50,29)= -1.6926702261e+00
       coeff_l3(50,30)= -2.0875587463e+00
       coeff_l3(50,31)= -3.6734823138e-02
       coeff_l3(50,32)= -1.3830059767e-01
       coeff_l3(50,33)= -5.3473156691e-01
       coeff_l3(50,34)=  4.5748781413e-02
       coeff_l3(50,35)=  8.4124505520e-02
       coeff_l3(50,36)= -1.1547498405e-01
       coeff_l3(50,37)= -2.7942249179e-01
       coeff_l3(50,38)= -1.7253506184e+00
       coeff_l3(50,39)= -1.7263491452e-01
       coeff_l3(50,40)= -3.1083554029e-01
       coeff_l3(50,41)= -2.7235519141e-02
       coeff_l3(50,42)= -1.8841516972e-01
       coeff_l3(50,43)= -2.6198059320e-01
       coeff_l3(50,44)= -3.2851755619e-01
       coeff_l3(50,45)= -1.8789097667e-01
       coeff_l3(50,46)= -1.5553981066e+00
       coeff_l3(50,47)= -7.9573428631e-01
       coeff_l3(50,48)= -4.9774605036e-01
       coeff_l3(50,49)= -4.6045377851e-01
       coeff_l3(50,50)= -3.5365349054e-01
       coeff_l3(50,51)=  2.5060394406e-01
       coeff_l3(50,52)= -3.6603286862e-01
       coeff_l3(50,53)= -3.8961070776e-01
       coeff_l3(50,54)=  1.4787300825e+00
       coeff_l3(50,55)= -3.6936292052e-01
       coeff_l3(50,56)= -1.6631284356e-01
       coeff_l3(50,57)= -5.3893428296e-02
       coeff_l3(50,58)= -3.6151127815e+00
       coeff_l3(50,59)= -3.3138045669e-01
       coeff_l3(50,60)= -9.1670060158e-01
       coeff_l3(50,61)= -7.8695803881e-02
       coeff_l3(50,62)= -1.8770514056e-02
       coeff_l3(50,63)=  4.2893186212e-01
       coeff_l3(50,64)= -5.2352923155e-01
       coeff_l3(50,65)= -1.8450509757e-02
       coeff_l3(50,66)= -2.8083094954e-01
       coeff_l3(50,67)= -8.8142879307e-02
       coeff_l3(50,68)= -2.9963798821e-02
       coeff_l3(50,69)= -9.7929559648e-02
       coeff_l3(50,70)= -5.1566261053e-01
       coeff_l3(50,71)= -1.7287962437e+00
       coeff_l3(50,72)= -9.9523685873e-02
       coeff_l3(50,73)= -9.1656583548e-01
       coeff_l3(50,74)= -7.0663108490e-03
       coeff_l3(50,75)= -2.6571720839e-01
       coeff_l3(50,76)=  1.4343290031e-01
       coeff_l3(50,77)=  8.3884999156e-02
       coeff_l3(50,78)=  1.2626013160e-01
       coeff_l3(50,79)= -2.4821066856e-01
       coeff_l3(50,80)= -3.7984378636e-02
       coeff_l3(50,81)= -2.3331275582e-01
       coeff_l3(50,82)= -5.7666397095e-01
       coeff_l3(50,83)= -7.0307143033e-02
       coeff_l3(50,84)= -7.1282213926e-01
       coeff_l3(50,85)= -3.0201241374e-01
       coeff_l3(50,86)= -2.1622981131e-01
       coeff_l3(50,87)= -7.0193879306e-02
       coeff_l3(50,88)= -1.1627668887e-01
       coeff_l3(50,89)= -2.5407874584e+00
       coeff_l3(50,90)= -3.7108829618e-01
       coeff_l3(50,91)= -3.6849901080e-02
       coeff_l3(50,92)= -1.9651052356e-01
       coeff_l3(50,93)=  2.6647928357e-01
       coeff_l3(50,94)= -9.7346001863e-01
       coeff_l3(50,95)= -3.6965364218e-01
       coeff_l3(50,96)= -9.5140546560e-01
       coeff_l3(50,97)= -3.6365401000e-02
       coeff_l3(50,98)= -2.5662067533e-01
       coeff_l3(50,99)= -2.0496068895e-01
       coeff_l3(50,100)= -2.1506279707e-01
       coeff_l3(50,101)= -2.9231652617e-01
       coeff_l3(50,102)=  6.8462342024e-02
       coeff_l3(50,103)=  2.1004997194e-01
       coeff_l3(50,104)= -4.2242330313e-01
       coeff_l3(50,105)= -5.5516004562e-01
       coeff_l3(50,106)= -6.8560105562e-01
       coeff_l3(50,107)=  1.2111785263e-01
       coeff_l3(50,108)= -1.0026806034e-02
       coeff_l3(50,109)= -3.0626443028e-01
       coeff_l3(50,110)=  7.0646382868e-02
       coeff_l3(50,111)= -6.0884177685e-01
       coeff_l3(50,112)= -1.4075859785e+00
       coeff_l3(50,113)= -1.7717385292e-01
       coeff_l3(50,114)=  9.4144940376e-01
       coeff_l3(50,115)= -2.2393703461e-01
       coeff_l3(50,116)=  5.4656185210e-02
       coeff_l3(50,117)= -6.6276210546e-01
       coeff_l3(50,118)= -7.9592412710e-01
       coeff_l3(50,119)= -1.6934685409e-01
       coeff_l3(50,120)= -1.4273574948e-01
       coeff_l3(50,121)= -3.1712037325e-01
       coeff_l3(50,122)=  7.3483206332e-02
       coeff_l3(50,123)= -2.3843739927e-01
       coeff_l3(50,124)= -8.9690424502e-03
       coeff_l3(50,125)= -2.4722927809e-01
       coeff_l3(51,1)= -2.4471418560e-01
       coeff_l3(51,2)= -9.0707182884e-02
       coeff_l3(51,3)= -6.5992460586e-03
       coeff_l3(51,4)=  2.8936907649e-01
       coeff_l3(51,5)= -3.9550546557e-02
       coeff_l3(51,6)= -1.1204428226e-01
       coeff_l3(51,7)=  6.6451495513e-03
       coeff_l3(51,8)=  1.5347110108e-02
       coeff_l3(51,9)= -1.2317065150e-01
       coeff_l3(51,10)= -1.8555080891e-01
       coeff_l3(51,11)=  1.5562814474e-01
       coeff_l3(51,12)=  1.7138141394e-01
       coeff_l3(51,13)= -4.7669205815e-03
       coeff_l3(51,14)= -6.9381326437e-02
       coeff_l3(51,15)= -6.1399016529e-02
       coeff_l3(51,16)= -1.6901971400e-01
       coeff_l3(51,17)=  1.0569183528e-01
       coeff_l3(51,18)=  2.5651288033e-01
       coeff_l3(51,19)= -2.9930168390e-01
       coeff_l3(51,20)=  2.1027739346e-01
       coeff_l3(51,21)= -6.0255199671e-02
       coeff_l3(51,22)=  1.1101923883e-01
       coeff_l3(51,23)= -2.1888250113e-01
       coeff_l3(51,24)= -5.3895674646e-02
       coeff_l3(51,25)= -2.8644543141e-02
       coeff_l3(51,26)= -5.4681714624e-02
       coeff_l3(51,27)= -2.4590827525e-01
       coeff_l3(51,28)=  6.9355532527e-02
       coeff_l3(51,29)=  4.9936416745e-01
       coeff_l3(51,30)=  4.3537627906e-02
       coeff_l3(51,31)= -3.6105895042e-01
       coeff_l3(51,32)= -1.6219137609e-01
       coeff_l3(51,33)=  2.4272698164e-01
       coeff_l3(51,34)= -3.2756438851e-01
       coeff_l3(51,35)=  9.8248533905e-02
       coeff_l3(51,36)=  1.3035959005e-01
       coeff_l3(51,37)=  4.9925837666e-02
       coeff_l3(51,38)=  2.2184087336e-01
       coeff_l3(51,39)= -1.4480829239e-01
       coeff_l3(51,40)= -7.2305589914e-02
       coeff_l3(51,41)=  7.5729161501e-01
       coeff_l3(51,42)= -1.1587210000e-01
       coeff_l3(51,43)=  2.8942161798e-01
       coeff_l3(51,44)= -3.5446682572e-01
       coeff_l3(51,45)= -6.9940418005e-02
       coeff_l3(51,46)=  1.2156988680e-01
       coeff_l3(51,47)=  4.4502552599e-02
       coeff_l3(51,48)=  1.5366199426e-02
       coeff_l3(51,49)= -7.0001207292e-02
       coeff_l3(51,50)= -1.4441758394e-01
       coeff_l3(51,51)=  7.0363722742e-02
       coeff_l3(51,52)= -5.0935685635e-02
       coeff_l3(51,53)=  1.2429571897e-01
       coeff_l3(51,54)= -5.7136591524e-02
       coeff_l3(51,55)= -1.4561893046e-01
       coeff_l3(51,56)=  2.3922882974e-01
       coeff_l3(51,57)=  1.0104528815e-01
       coeff_l3(51,58)= -2.0880366862e-01
       coeff_l3(51,59)=  1.7441456020e-01
       coeff_l3(51,60)=  1.6398534179e-02
       coeff_l3(51,61)= -1.5058629215e-01
       coeff_l3(51,62)= -2.2751443088e-01
       coeff_l3(51,63)= -9.5160111785e-02
       coeff_l3(51,64)= -1.0591270402e-02
       coeff_l3(51,65)= -1.5260456502e-01
       coeff_l3(51,66)=  3.9690624923e-02
       coeff_l3(51,67)=  1.1666893214e-01
       coeff_l3(51,68)= -1.7413271964e-01
       coeff_l3(51,69)=  1.5680471063e-01
       coeff_l3(51,70)= -1.6682304442e-01
       coeff_l3(51,71)=  2.5264525414e-01
       coeff_l3(51,72)= -7.7988855541e-02
       coeff_l3(51,73)= -8.4390185773e-02
       coeff_l3(51,74)=  6.4392634667e-03
       coeff_l3(51,75)= -1.0619654506e-01
       coeff_l3(51,76)= -4.3565514684e-01
       coeff_l3(51,77)= -1.7025971413e-01
       coeff_l3(51,78)= -1.5892153978e-01
       coeff_l3(51,79)= -1.5830149874e-02
       coeff_l3(51,80)= -2.7920594811e-01
       coeff_l3(51,81)=  2.2184329107e-02
       coeff_l3(51,82)=  1.4952096343e-01
       coeff_l3(51,83)=  5.6909158826e-02
       coeff_l3(51,84)= -3.5783354193e-02
       coeff_l3(51,85)=  2.4771171808e-01
       coeff_l3(51,86)=  7.1930453181e-02
       coeff_l3(51,87)=  5.0716836005e-02
       coeff_l3(51,88)= -1.1568236351e-01
       coeff_l3(51,89)=  1.2337952107e-01
       coeff_l3(51,90)= -4.0945121646e-01
       coeff_l3(51,91)= -1.7030802369e-01
       coeff_l3(51,92)= -1.4081357419e-01
       coeff_l3(51,93)= -2.6464262605e-01
       coeff_l3(51,94)=  2.5798097253e-01
       coeff_l3(51,95)=  6.8816795945e-02
       coeff_l3(51,96)= -2.0461165905e-01
       coeff_l3(51,97)=  1.4506296813e-01
       coeff_l3(51,98)=  8.1492222846e-02
       coeff_l3(51,99)= -1.1613458395e-01
       coeff_l3(51,100)=  1.3880470395e-01
       coeff_l3(51,101)= -2.2674342617e-02
       coeff_l3(51,102)= -9.7883999348e-02
       coeff_l3(51,103)= -8.8663369417e-02
       coeff_l3(51,104)= -9.7319789231e-02
       coeff_l3(51,105)=  4.0632358193e-01
       coeff_l3(51,106)= -1.0225588083e-01
       coeff_l3(51,107)= -2.5287345052e-02
       coeff_l3(51,108)=  1.6226807237e-01
       coeff_l3(51,109)= -7.5006000698e-02
       coeff_l3(51,110)=  1.6603820026e-01
       coeff_l3(51,111)=  3.1347724795e-01
       coeff_l3(51,112)= -5.6137971580e-02
       coeff_l3(51,113)= -4.6018864959e-02
       coeff_l3(51,114)= -6.7720137537e-02
       coeff_l3(51,115)=  1.1524558999e-02
       coeff_l3(51,116)=  1.6233785450e-01
       coeff_l3(51,117)=  1.6222630441e-01
       coeff_l3(51,118)=  2.8082185984e-01
       coeff_l3(51,119)=  2.5404023007e-02
       coeff_l3(51,120)=  2.7102106810e-01
       coeff_l3(51,121)=  6.4759813249e-02
       coeff_l3(51,122)= -2.1958418190e-02
       coeff_l3(51,123)= -1.4674763381e-01
       coeff_l3(51,124)= -1.7679530382e-01
       coeff_l3(51,125)= -2.0179511979e-02
       coeff_l3(52,1)= -7.8262156248e-01
       coeff_l3(52,2)= -6.0812529176e-02
       coeff_l3(52,3)=  9.6832662821e-02
       coeff_l3(52,4)= -9.9904850125e-02
       coeff_l3(52,5)=  9.6079126000e-02
       coeff_l3(52,6)= -6.3534401357e-02
       coeff_l3(52,7)= -2.8264367953e-02
       coeff_l3(52,8)= -1.5351857245e-01
       coeff_l3(52,9)=  4.4076111168e-02
       coeff_l3(52,10)=  1.1965278536e-01
       coeff_l3(52,11)=  4.9506861717e-02
       coeff_l3(52,12)= -5.1281166077e-01
       coeff_l3(52,13)= -1.6800700128e-01
       coeff_l3(52,14)= -2.3052412271e-01
       coeff_l3(52,15)=  7.2290539742e-02
       coeff_l3(52,16)=  1.2583632022e-02
       coeff_l3(52,17)= -1.1406321824e-01
       coeff_l3(52,18)= -6.9510579109e-01
       coeff_l3(52,19)=  2.2413878143e-01
       coeff_l3(52,20)= -4.0835123509e-02
       coeff_l3(52,21)= -1.1881494522e-01
       coeff_l3(52,22)= -6.4270414412e-02
       coeff_l3(52,23)=  2.6317173243e-01
       coeff_l3(52,24)=  3.0038166046e-01
       coeff_l3(52,25)=  1.7443947494e-01
       coeff_l3(52,26)=  1.9774818420e-01
       coeff_l3(52,27)=  7.1412093937e-02
       coeff_l3(52,28)=  1.9048425555e-01
       coeff_l3(52,29)= -5.3923112154e-01
       coeff_l3(52,30)= -2.8411632776e-01
       coeff_l3(52,31)=  1.3793374598e-01
       coeff_l3(52,32)= -2.5082590058e-02
       coeff_l3(52,33)= -9.7320884466e-02
       coeff_l3(52,34)=  1.1258862913e-01
       coeff_l3(52,35)= -4.6136254072e-01
       coeff_l3(52,36)= -1.6352100670e-01
       coeff_l3(52,37)=  1.8107429147e-01
       coeff_l3(52,38)= -1.2625934184e-01
       coeff_l3(52,39)= -1.0142979771e-01
       coeff_l3(52,40)= -1.0878051072e-01
       coeff_l3(52,41)= -1.6345502436e-01
       coeff_l3(52,42)=  1.6334393620e-01
       coeff_l3(52,43)= -3.7482410669e-01
       coeff_l3(52,44)=  6.6880047321e-02
       coeff_l3(52,45)= -4.6408677101e-01
       coeff_l3(52,46)= -2.2199107707e-01
       coeff_l3(52,47)=  1.1417558789e-01
       coeff_l3(52,48)= -1.6537541151e-01
       coeff_l3(52,49)= -3.1409040093e-01
       coeff_l3(52,50)=  1.9264319539e-01
       coeff_l3(52,51)=  1.5283426270e-02
       coeff_l3(52,52)=  1.3533510268e-01
       coeff_l3(52,53)= -8.3500199020e-02
       coeff_l3(52,54)= -9.9079743028e-02
       coeff_l3(52,55)=  1.6758065671e-02
       coeff_l3(52,56)= -6.7865289748e-02
       coeff_l3(52,57)= -1.0198655725e-01
       coeff_l3(52,58)=  4.1210547090e-01
       coeff_l3(52,59)= -3.7179955840e-01
       coeff_l3(52,60)= -4.6596437693e-01
       coeff_l3(52,61)=  2.8155136108e-01
       coeff_l3(52,62)=  4.1972979903e-02
       coeff_l3(52,63)= -1.7643766105e-01
       coeff_l3(52,64)= -4.2754154652e-02
       coeff_l3(52,65)= -6.2404330820e-02
       coeff_l3(52,66)= -8.1625603139e-02
       coeff_l3(52,67)= -3.5122880340e-01
       coeff_l3(52,68)= -3.1838816404e-01
       coeff_l3(52,69)=  8.0749399960e-02
       coeff_l3(52,70)=  2.3555620015e-01
       coeff_l3(52,71)= -7.1848392487e-01
       coeff_l3(52,72)= -1.1734168977e-02
       coeff_l3(52,73)=  1.2731915340e-02
       coeff_l3(52,74)= -3.7730994821e-01
       coeff_l3(52,75)=  5.9265013784e-02
       coeff_l3(52,76)= -4.3061742187e-01
       coeff_l3(52,77)= -1.0758996010e-01
       coeff_l3(52,78)=  1.5262386203e-01
       coeff_l3(52,79)= -5.3724579513e-02
       coeff_l3(52,80)= -3.2241827250e-01
       coeff_l3(52,81)=  1.7413139343e-01
       coeff_l3(52,82)= -1.2214790285e-01
       coeff_l3(52,83)=  2.6485199109e-02
       coeff_l3(52,84)=  1.1947090179e-01
       coeff_l3(52,85)=  1.6602094471e-01
       coeff_l3(52,86)= -1.7292521894e-01
       coeff_l3(52,87)= -1.8395576626e-02
       coeff_l3(52,88)=  1.2037111446e-02
       coeff_l3(52,89)=  1.0963977873e-01
       coeff_l3(52,90)=  2.7188733220e-01
       coeff_l3(52,91)=  6.1097532511e-02
       coeff_l3(52,92)= -1.2171998620e-01
       coeff_l3(52,93)= -1.0884073377e-01
       coeff_l3(52,94)= -2.0536284149e-01
       coeff_l3(52,95)= -1.0138647258e-01
       coeff_l3(52,96)= -5.0146028399e-02
       coeff_l3(52,97)=  7.2798624635e-02
       coeff_l3(52,98)= -2.5765590370e-02
       coeff_l3(52,99)=  2.3842282593e-02
       coeff_l3(52,100)= -9.0244643390e-02
       coeff_l3(52,101)=  1.2409270555e-01
       coeff_l3(52,102)=  1.1291565746e-01
       coeff_l3(52,103)=  3.1356658787e-02
       coeff_l3(52,104)=  9.0771645308e-02
       coeff_l3(52,105)= -3.0014431104e-02
       coeff_l3(52,106)= -9.8401166499e-02
       coeff_l3(52,107)= -1.4711598866e-02
       coeff_l3(52,108)= -2.3011593521e-01
       coeff_l3(52,109)=  5.1334155723e-03
       coeff_l3(52,110)=  7.4649587274e-02
       coeff_l3(52,111)= -1.6540928185e-01
       coeff_l3(52,112)= -3.9107289910e-01
       coeff_l3(52,113)= -5.0548148155e-01
       coeff_l3(52,114)=  1.4373662416e-03
       coeff_l3(52,115)= -1.6358645260e-01
       coeff_l3(52,116)= -6.0050785542e-01
       coeff_l3(52,117)= -3.6987262964e-01
       coeff_l3(52,118)= -3.4305328131e-01
       coeff_l3(52,119)=  3.5265043378e-02
       coeff_l3(52,120)= -1.1967936903e-01
       coeff_l3(52,121)= -2.6505964994e-01
       coeff_l3(52,122)=  7.2254322469e-02
       coeff_l3(52,123)=  3.0146086589e-02
       coeff_l3(52,124)= -2.3340299726e-02
       coeff_l3(52,125)=  5.9298966080e-02
       coeff_l3(53,1)= -1.7370130122e-01
       coeff_l3(53,2)= -1.1677371711e-01
       coeff_l3(53,3)= -1.8060445786e-02
       coeff_l3(53,4)= -6.6902235150e-02
       coeff_l3(53,5)= -2.8994470835e-02
       coeff_l3(53,6)=  1.7299044877e-02
       coeff_l3(53,7)=  9.8858624697e-02
       coeff_l3(53,8)=  3.4630626440e-02
       coeff_l3(53,9)=  1.5941171348e-01
       coeff_l3(53,10)= -8.4128111601e-02
       coeff_l3(53,11)=  1.8799600005e-01
       coeff_l3(53,12)=  4.2027644813e-02
       coeff_l3(53,13)= -6.4114227891e-02
       coeff_l3(53,14)= -2.4020136893e-01
       coeff_l3(53,15)=  6.6853545606e-02
       coeff_l3(53,16)=  7.4427992105e-02
       coeff_l3(53,17)= -1.1407723650e-02
       coeff_l3(53,18)=  3.5997163504e-03
       coeff_l3(53,19)= -1.6836836934e-01
       coeff_l3(53,20)= -1.2394209951e-01
       coeff_l3(53,21)=  2.7339118719e-01
       coeff_l3(53,22)=  8.4999233484e-02
       coeff_l3(53,23)=  7.9680867493e-02
       coeff_l3(53,24)= -1.2016553432e-01
       coeff_l3(53,25)= -1.2370665371e-01
       coeff_l3(53,26)=  1.3997302949e-01
       coeff_l3(53,27)=  8.5516370833e-02
       coeff_l3(53,28)=  9.1526776552e-02
       coeff_l3(53,29)= -7.6202623546e-02
       coeff_l3(53,30)= -6.3161291182e-02
       coeff_l3(53,31)= -2.2298194468e-01
       coeff_l3(53,32)= -1.0794383474e-02
       coeff_l3(53,33)=  1.8818387389e-01
       coeff_l3(53,34)=  1.3189259171e-01
       coeff_l3(53,35)= -2.3240391165e-02
       coeff_l3(53,36)= -1.1251287907e-01
       coeff_l3(53,37)= -3.5471960902e-02
       coeff_l3(53,38)=  6.3517582417e-01
       coeff_l3(53,39)= -1.7297717929e-01
       coeff_l3(53,40)=  2.2837286815e-02
       coeff_l3(53,41)=  3.2818788290e-01
       coeff_l3(53,42)= -3.2929811627e-02
       coeff_l3(53,43)=  8.9695163071e-02
       coeff_l3(53,44)= -1.1552698910e-01
       coeff_l3(53,45)=  1.9971044734e-02
       coeff_l3(53,46)=  4.1072349995e-02
       coeff_l3(53,47)=  5.7034846395e-02
       coeff_l3(53,48)=  1.4296536148e-01
       coeff_l3(53,49)= -2.4976885319e-01
       coeff_l3(53,50)= -5.9013918042e-02
       coeff_l3(53,51)= -2.0642085373e-01
       coeff_l3(53,52)= -1.3528743386e-01
       coeff_l3(53,53)=  5.4695934057e-02
       coeff_l3(53,54)= -3.1069064140e-01
       coeff_l3(53,55)=  1.6097357497e-02
       coeff_l3(53,56)=  8.0061040819e-02
       coeff_l3(53,57)=  1.2514621019e-01
       coeff_l3(53,58)= -2.5668954849e-01
       coeff_l3(53,59)= -2.3745401204e-01
       coeff_l3(53,60)=  7.0326894522e-01
       coeff_l3(53,61)=  1.6168862581e-01
       coeff_l3(53,62)= -5.4193539545e-03
       coeff_l3(53,63)=  1.5800952911e-02
       coeff_l3(53,64)=  2.3018838838e-02
       coeff_l3(53,65)=  6.6807717085e-02
       coeff_l3(53,66)= -1.9948912784e-02
       coeff_l3(53,67)=  2.3797775805e-01
       coeff_l3(53,68)= -8.6656868458e-02
       coeff_l3(53,69)= -2.1089522541e-01
       coeff_l3(53,70)=  1.8408976495e-02
       coeff_l3(53,71)=  3.2477512956e-01
       coeff_l3(53,72)=  1.7167498171e-01
       coeff_l3(53,73)= -1.5158146620e-01
       coeff_l3(53,74)=  3.8412109017e-01
       coeff_l3(53,75)=  8.7003134191e-02
       coeff_l3(53,76)= -1.3214420527e-02
       coeff_l3(53,77)=  4.2328536510e-02
       coeff_l3(53,78)= -1.7828375101e-01
       coeff_l3(53,79)=  1.3477198780e-01
       coeff_l3(53,80)= -4.2162436247e-01
       coeff_l3(53,81)=  1.9821086526e-01
       coeff_l3(53,82)=  1.8183901906e-02
       coeff_l3(53,83)= -1.0676013678e-01
       coeff_l3(53,84)= -1.3942779601e-01
       coeff_l3(53,85)= -2.5673711300e-01
       coeff_l3(53,86)= -8.3027578890e-02
       coeff_l3(53,87)= -8.3479158580e-02
       coeff_l3(53,88)= -2.5290939957e-03
       coeff_l3(53,89)= -1.4597983658e-01
       coeff_l3(53,90)=  1.6035562754e-01
       coeff_l3(53,91)= -7.4604554102e-03
       coeff_l3(53,92)= -4.7520976514e-02
       coeff_l3(53,93)= -1.2847852707e-01
       coeff_l3(53,94)= -1.4413759112e-01
       coeff_l3(53,95)=  9.7585082054e-02
       coeff_l3(53,96)=  7.1831330657e-02
       coeff_l3(53,97)=  3.3062871546e-02
       coeff_l3(53,98)= -7.7852167189e-02
       coeff_l3(53,99)=  1.5838505328e-01
       coeff_l3(53,100)= -1.6471137106e-01
       coeff_l3(53,101)=  3.1401544809e-01
       coeff_l3(53,102)= -1.8069501221e-01
       coeff_l3(53,103)= -1.0903000832e-01
       coeff_l3(53,104)= -3.3646166325e-02
       coeff_l3(53,105)=  4.4855430722e-01
       coeff_l3(53,106)= -1.6770300269e-01
       coeff_l3(53,107)= -2.0144045353e-01
       coeff_l3(53,108)=  1.1602362990e-01
       coeff_l3(53,109)= -9.6333846450e-02
       coeff_l3(53,110)= -3.5304486752e-02
       coeff_l3(53,111)=  1.0157229006e-01
       coeff_l3(53,112)= -1.0844137520e-01
       coeff_l3(53,113)= -1.5045566857e-01
       coeff_l3(53,114)=  3.2154899091e-02
       coeff_l3(53,115)=  1.5257520974e-01
       coeff_l3(53,116)=  1.7830418050e-01
       coeff_l3(53,117)=  9.5962040126e-02
       coeff_l3(53,118)=  2.9298487306e-01
       coeff_l3(53,119)=  1.4327692986e-01
       coeff_l3(53,120)=  6.6359780729e-02
       coeff_l3(53,121)= -1.0194637626e-01
       coeff_l3(53,122)= -8.9580118656e-02
       coeff_l3(53,123)= -1.1429142952e-01
       coeff_l3(53,124)= -3.3614121377e-02
       coeff_l3(53,125)= -1.1994680762e-01
       coeff_l3(54,1)=  7.8588992357e-01
       coeff_l3(54,2)=  1.4528033137e-01
       coeff_l3(54,3)= -3.7450733781e-01
       coeff_l3(54,4)=  2.2417703271e-01
       coeff_l3(54,5)=  1.5085522830e-01
       coeff_l3(54,6)=  1.0873205960e-02
       coeff_l3(54,7)=  5.8498803526e-02
       coeff_l3(54,8)= -2.1898369491e-01
       coeff_l3(54,9)= -1.6885380447e-01
       coeff_l3(54,10)= -2.7903431654e-01
       coeff_l3(54,11)= -5.9182833880e-02
       coeff_l3(54,12)=  2.9385691881e-01
       coeff_l3(54,13)=  6.1319489032e-02
       coeff_l3(54,14)=  5.1236383617e-02
       coeff_l3(54,15)= -2.7787117288e-02
       coeff_l3(54,16)= -1.9665819407e-01
       coeff_l3(54,17)=  2.1415911615e-01
       coeff_l3(54,18)=  3.4283739328e-01
       coeff_l3(54,19)= -3.6471597850e-02
       coeff_l3(54,20)= -5.5138832331e-01
       coeff_l3(54,21)= -6.8678393960e-02
       coeff_l3(54,22)=  4.3434768915e-02
       coeff_l3(54,23)= -2.7536353469e-01
       coeff_l3(54,24)= -2.8953236341e-01
       coeff_l3(54,25)=  1.5872848034e-01
       coeff_l3(54,26)= -2.9297259450e-01
       coeff_l3(54,27)= -1.0415256768e-01
       coeff_l3(54,28)=  2.7442730963e-02
       coeff_l3(54,29)=  1.7175699770e-01
       coeff_l3(54,30)=  2.6067802310e-01
       coeff_l3(54,31)= -1.6857407987e-01
       coeff_l3(54,32)=  8.8732605800e-03
       coeff_l3(54,33)=  8.8484913111e-02
       coeff_l3(54,34)=  9.3033790588e-02
       coeff_l3(54,35)=  4.2646065354e-01
       coeff_l3(54,36)= -2.2109302878e-01
       coeff_l3(54,37)=  5.5290326476e-02
       coeff_l3(54,38)= -2.0517866313e-01
       coeff_l3(54,39)=  2.1444988251e-01
       coeff_l3(54,40)=  2.6650249958e-01
       coeff_l3(54,41)=  4.5578452945e-01
       coeff_l3(54,42)= -2.6293843985e-01
       coeff_l3(54,43)= -2.0294703543e-02
       coeff_l3(54,44)= -2.0072188973e-01
       coeff_l3(54,45)= -2.2742438316e-01
       coeff_l3(54,46)=  2.6964461803e-01
       coeff_l3(54,47)= -5.1750965416e-02
       coeff_l3(54,48)=  1.0499145836e-01
       coeff_l3(54,49)=  7.1500241756e-01
       coeff_l3(54,50)=  7.0663861930e-02
       coeff_l3(54,51)= -1.7542296648e-01
       coeff_l3(54,52)= -2.4969053268e-01
       coeff_l3(54,53)=  5.4774284363e-02
       coeff_l3(54,54)= -3.7239736319e-01
       coeff_l3(54,55)=  3.4324136376e-01
       coeff_l3(54,56)= -8.5809305310e-02
       coeff_l3(54,57)= -3.2005678862e-02
       coeff_l3(54,58)= -1.0385704786e-01
       coeff_l3(54,59)=  1.8276500702e-01
       coeff_l3(54,60)=  1.4498682320e-01
       coeff_l3(54,61)= -1.8859086931e-01
       coeff_l3(54,62)= -2.0412899554e-02
       coeff_l3(54,63)=  1.7316414416e-01
       coeff_l3(54,64)= -9.4354180619e-03
       coeff_l3(54,65)= -8.3913402632e-03
       coeff_l3(54,66)=  9.9364787340e-02
       coeff_l3(54,67)= -4.8276504874e-01
       coeff_l3(54,68)= -6.3027016819e-02
       coeff_l3(54,69)=  7.1090406179e-01
       coeff_l3(54,70)=  4.4163309038e-02
       coeff_l3(54,71)=  3.6353537440e-01
       coeff_l3(54,72)= -1.8304459751e-01
       coeff_l3(54,73)= -4.3380159140e-01
       coeff_l3(54,74)= -1.0906738229e-02
       coeff_l3(54,75)= -3.6096221209e-01
       coeff_l3(54,76)=  3.4698900580e-01
       coeff_l3(54,77)= -5.0972066820e-02
       coeff_l3(54,78)=  7.3411107063e-02
       coeff_l3(54,79)=  1.6991201788e-02
       coeff_l3(54,80)=  5.6114500761e-01
       coeff_l3(54,81)=  4.0831867605e-02
       coeff_l3(54,82)=  8.1762611866e-02
       coeff_l3(54,83)=  3.7628597021e-01
       coeff_l3(54,84)= -4.3128344417e-01
       coeff_l3(54,85)= -1.8205085397e-01
       coeff_l3(54,86)=  3.0204129219e-01
       coeff_l3(54,87)=  1.7105498910e-01
       coeff_l3(54,88)=  2.6206374168e-01
       coeff_l3(54,89)= -1.8943274394e-02
       coeff_l3(54,90)= -1.2504144013e-01
       coeff_l3(54,91)= -2.5056177378e-01
       coeff_l3(54,92)= -2.2767258808e-02
       coeff_l3(54,93)=  1.3461031020e-01
       coeff_l3(54,94)=  2.3770126700e-01
       coeff_l3(54,95)=  7.0856742561e-02
       coeff_l3(54,96)=  1.2103019655e-01
       coeff_l3(54,97)= -5.1114663482e-02
       coeff_l3(54,98)= -2.2765819728e-01
       coeff_l3(54,99)=  3.1759271026e-01
       coeff_l3(54,100)=  4.4316973537e-02
       coeff_l3(54,101)= -2.8501793742e-01
       coeff_l3(54,102)=  7.1153961122e-02
       coeff_l3(54,103)= -1.0679014027e-01
       coeff_l3(54,104)= -2.5647884607e-01
       coeff_l3(54,105)=  5.6528306007e-01
       coeff_l3(54,106)=  9.9426567554e-02
       coeff_l3(54,107)=  1.4129841328e-01
       coeff_l3(54,108)=  8.1554479897e-02
       coeff_l3(54,109)= -5.4078758694e-03
       coeff_l3(54,110)= -1.1284283549e-01
       coeff_l3(54,111)=  6.6597118974e-02
       coeff_l3(54,112)=  5.5680316687e-01
       coeff_l3(54,113)=  2.5988581777e-01
       coeff_l3(54,114)=  1.3346381485e-01
       coeff_l3(54,115)=  3.4517711401e-01
       coeff_l3(54,116)=  4.7098535299e-01
       coeff_l3(54,117)=  3.4157308936e-01
       coeff_l3(54,118)= -3.4070092440e-01
       coeff_l3(54,119)=  7.7062942088e-02
       coeff_l3(54,120)=  1.1504245549e-01
       coeff_l3(54,121)=  2.7013817430e-01
       coeff_l3(54,122)= -3.5714920610e-02
       coeff_l3(54,123)= -4.5052688569e-02
       coeff_l3(54,124)= -1.3182769716e-01
       coeff_l3(54,125)= -6.1375509948e-02
       coeff_l3(55,1)= -1.6185246408e-01
       coeff_l3(55,2)= -5.1845867187e-02
       coeff_l3(55,3)=  1.5399236232e-02
       coeff_l3(55,4)=  1.9564111531e-01
       coeff_l3(55,5)= -1.6943514347e-01
       coeff_l3(55,6)= -3.0915212631e-01
       coeff_l3(55,7)=  1.3387672901e+00
       coeff_l3(55,8)= -9.2880278826e-02
       coeff_l3(55,9)= -3.2996773720e-02
       coeff_l3(55,10)= -1.2374544144e-01
       coeff_l3(55,11)= -3.3935275674e-01
       coeff_l3(55,12)= -2.6806670427e-01
       coeff_l3(55,13)= -3.6883103848e-01
       coeff_l3(55,14)= -1.0218456388e-01
       coeff_l3(55,15)= -2.7670031786e-01
       coeff_l3(55,16)= -1.1092088372e-01
       coeff_l3(55,17)= -2.0187395811e-01
       coeff_l3(55,18)= -1.9447033107e-01
       coeff_l3(55,19)=  1.4545620978e-01
       coeff_l3(55,20)= -3.9832901210e-02
       coeff_l3(55,21)= -1.1248946190e-01
       coeff_l3(55,22)= -6.7899093032e-02
       coeff_l3(55,23)= -3.6730566621e-01
       coeff_l3(55,24)= -1.2897610664e-02
       coeff_l3(55,25)= -4.9940157682e-02
       coeff_l3(55,26)= -3.8284659386e-01
       coeff_l3(55,27)=  1.8312145472e+00
       coeff_l3(55,28)=  1.3602113724e+00
       coeff_l3(55,29)= -1.2822006643e-01
       coeff_l3(55,30)= -1.5653774142e-01
       coeff_l3(55,31)= -3.1600058079e-01
       coeff_l3(55,32)= -9.4992183149e-02
       coeff_l3(55,33)= -8.3635300398e-02
       coeff_l3(55,34)=  2.7282210067e-02
       coeff_l3(55,35)= -7.1222521365e-02
       coeff_l3(55,36)=  1.8358357484e-03
       coeff_l3(55,37)= -1.5770484507e-01
       coeff_l3(55,38)= -1.6450503469e-01
       coeff_l3(55,39)=  4.1606523097e-02
       coeff_l3(55,40)= -3.7451097369e-01
       coeff_l3(55,41)= -2.9406437278e-01
       coeff_l3(55,42)= -3.1738750637e-02
       coeff_l3(55,43)= -8.7617650628e-02
       coeff_l3(55,44)= -2.4979902804e-01
       coeff_l3(55,45)= -6.9001980126e-02
       coeff_l3(55,46)= -4.3181963265e-02
       coeff_l3(55,47)= -2.5739145279e-01
       coeff_l3(55,48)= -2.6009973884e-01
       coeff_l3(55,49)=  2.6629703045e+00
       coeff_l3(55,50)= -2.0809084177e-01
       coeff_l3(55,51)= -6.3629284501e-02
       coeff_l3(55,52)= -2.7586683631e-01
       coeff_l3(55,53)= -1.1385349184e-01
       coeff_l3(55,54)=  5.2117663622e-01
       coeff_l3(55,55)=  4.0655024350e-03
       coeff_l3(55,56)= -2.1390981972e-01
       coeff_l3(55,57)= -1.5126563609e-01
       coeff_l3(55,58)= -6.9450479746e-01
       coeff_l3(55,59)=  1.9248524308e-01
       coeff_l3(55,60)=  1.4346046448e+00
       coeff_l3(55,61)= -3.9886274934e-01
       coeff_l3(55,62)= -1.0474874079e-01
       coeff_l3(55,63)=  2.9658054933e-02
       coeff_l3(55,64)= -4.0019002557e-01
       coeff_l3(55,65)= -1.5053187311e-01
       coeff_l3(55,66)= -2.0358714461e-01
       coeff_l3(55,67)= -5.0255262852e-01
       coeff_l3(55,68)= -3.9884203672e-01
       coeff_l3(55,69)= -7.9552777112e-02
       coeff_l3(55,70)= -1.1685730517e-01
       coeff_l3(55,71)= -3.5525721312e-01
       coeff_l3(55,72)= -1.3916985691e-01
       coeff_l3(55,73)= -5.2039074898e-01
       coeff_l3(55,74)= -1.6134195030e-01
       coeff_l3(55,75)= -1.4111338556e-01
       coeff_l3(55,76)=  1.4992108345e+00
       coeff_l3(55,77)=  2.3056410253e-01
       coeff_l3(55,78)= -9.0814687312e-02
       coeff_l3(55,79)= -2.0247352123e-01
       coeff_l3(55,80)=  1.4493236542e+00
       coeff_l3(55,81)= -3.1652588397e-02
       coeff_l3(55,82)= -1.6219614446e-01
       coeff_l3(55,83)= -3.4262225032e-02
       coeff_l3(55,84)= -2.2890588641e-01
       coeff_l3(55,85)= -1.4581061900e-01
       coeff_l3(55,86)= -2.3313531280e-01
       coeff_l3(55,87)= -5.0715960562e-02
       coeff_l3(55,88)= -2.6588413119e-01
       coeff_l3(55,89)= -2.2154876590e-01
       coeff_l3(55,90)= -5.1487158984e-02
       coeff_l3(55,91)=  1.5627535060e-02
       coeff_l3(55,92)= -2.0089241862e-01
       coeff_l3(55,93)=  1.1902777851e-01
       coeff_l3(55,94)=  1.4283730090e-01
       coeff_l3(55,95)= -1.2305707484e-01
       coeff_l3(55,96)= -2.4120202661e-01
       coeff_l3(55,97)= -1.3198043406e-01
       coeff_l3(55,98)= -2.3150177300e-01
       coeff_l3(55,99)= -3.9069157839e-01
       coeff_l3(55,100)=  5.5256418884e-02
       coeff_l3(55,101)=  4.3394703418e-02
       coeff_l3(55,102)= -2.7477061749e-01
       coeff_l3(55,103)= -1.4855402708e-01
       coeff_l3(55,104)= -2.2934770584e-01
       coeff_l3(55,105)=  2.6267649606e-02
       coeff_l3(55,106)= -3.2644319534e-01
       coeff_l3(55,107)= -4.6466168761e-01
       coeff_l3(55,108)=  4.1910447180e-02
       coeff_l3(55,109)= -3.3461675048e-02
       coeff_l3(55,110)= -7.8860364854e-02
       coeff_l3(55,111)=  6.2778718770e-02
       coeff_l3(55,112)= -1.4329157770e-01
       coeff_l3(55,113)= -2.3506979644e-01
       coeff_l3(55,114)=  7.2971987724e-01
       coeff_l3(55,115)=  1.1898864992e-02
       coeff_l3(55,116)=  1.1115533859e-01
       coeff_l3(55,117)=  3.7135705352e-02
       coeff_l3(55,118)=  2.2802773118e-01
       coeff_l3(55,119)= -1.5025621653e-01
       coeff_l3(55,120)=  1.2868490815e-01
       coeff_l3(55,121)=  2.1807058156e-01
       coeff_l3(55,122)= -1.6701193154e-01
       coeff_l3(55,123)=  4.3360799551e-02
       coeff_l3(55,124)=  2.4981016293e-02
       coeff_l3(55,125)=  2.7326388285e-02
       coeff_l3(56,1)= -1.7005808353e+00
       coeff_l3(56,2)= -2.1409386769e-02
       coeff_l3(56,3)= -5.0544809550e-02
       coeff_l3(56,4)= -9.1305203736e-02
       coeff_l3(56,5)= -5.3982131183e-02
       coeff_l3(56,6)=  1.9190591574e-01
       coeff_l3(56,7)=  6.2914881855e-03
       coeff_l3(56,8)= -4.3957166374e-02
       coeff_l3(56,9)=  4.5730669051e-02
       coeff_l3(56,10)=  2.6345548034e-01
       coeff_l3(56,11)=  1.4307951927e-01
       coeff_l3(56,12)= -5.3712582588e-01
       coeff_l3(56,13)= -5.0287187099e-01
       coeff_l3(56,14)= -4.0513679385e-01
       coeff_l3(56,15)= -9.5109350979e-02
       coeff_l3(56,16)=  4.9022164196e-02
       coeff_l3(56,17)= -3.7803325057e-01
       coeff_l3(56,18)= -1.7789604664e+00
       coeff_l3(56,19)= -1.4079624414e-01
       coeff_l3(56,20)=  9.6479795873e-02
       coeff_l3(56,21)=  1.8207788467e-01
       coeff_l3(56,22)= -9.2006802559e-02
       coeff_l3(56,23)=  2.5585407019e-01
       coeff_l3(56,24)=  1.9611318409e-01
       coeff_l3(56,25)=  2.7546337247e-01
       coeff_l3(56,26)= -6.3348442316e-02
       coeff_l3(56,27)=  2.7746828273e-02
       coeff_l3(56,28)= -8.4119118750e-02
       coeff_l3(56,29)= -1.9703318179e-01
       coeff_l3(56,30)= -1.4697943628e-01
       coeff_l3(56,31)=  1.4517410100e-01
       coeff_l3(56,32)=  1.5622837842e-01
       coeff_l3(56,33)= -8.6184598505e-02
       coeff_l3(56,34)= -1.1265182495e-01
       coeff_l3(56,35)=  2.1973662078e-02
       coeff_l3(56,36)= -8.9792184532e-02
       coeff_l3(56,37)=  3.0990995467e-02
       coeff_l3(56,38)= -4.4556942582e-01
       coeff_l3(56,39)= -7.4487239122e-02
       coeff_l3(56,40)= -1.8230885267e-01
       coeff_l3(56,41)= -2.1174639463e-01
       coeff_l3(56,42)=  1.5626107156e-01
       coeff_l3(56,43)= -4.5832151175e-01
       coeff_l3(56,44)=  2.5655639172e-01
       coeff_l3(56,45)= -8.9882522821e-01
       coeff_l3(56,46)= -3.5497912765e-01
       coeff_l3(56,47)=  5.1887605339e-02
       coeff_l3(56,48)= -5.0006324053e-01
       coeff_l3(56,49)= -7.9282861948e-01
       coeff_l3(56,50)=  9.7228683531e-02
       coeff_l3(56,51)= -3.1655013561e-01
       coeff_l3(56,52)=  1.0320441425e-01
       coeff_l3(56,53)=  1.6293275356e-01
       coeff_l3(56,54)=  8.2821786404e-01
       coeff_l3(56,55)=  1.3216466643e-02
       coeff_l3(56,56)=  1.7392362654e-01
       coeff_l3(56,57)= -4.4071350247e-02
       coeff_l3(56,58)=  5.8034026623e-01
       coeff_l3(56,59)=  1.1393632740e-01
       coeff_l3(56,60)= -8.0129450560e-01
       coeff_l3(56,61)=  2.1526446566e-02
       coeff_l3(56,62)=  2.2363781929e-01
       coeff_l3(56,63)=  1.0802179575e-01
       coeff_l3(56,64)=  2.7894189954e-01
       coeff_l3(56,65)=  1.1637354642e-01
       coeff_l3(56,66)=  3.8580421824e-03
       coeff_l3(56,67)= -6.9174367189e-01
       coeff_l3(56,68)= -8.0448761582e-03
       coeff_l3(56,69)= -7.7139198780e-02
       coeff_l3(56,70)=  8.0716244876e-02
       coeff_l3(56,71)= -1.6820365191e+00
       coeff_l3(56,72)=  1.8348465860e-01
       coeff_l3(56,73)=  3.4143105149e-01
       coeff_l3(56,74)=  1.8425045907e-01
       coeff_l3(56,75)=  2.0144656301e-01
       coeff_l3(56,76)= -2.5862646103e-01
       coeff_l3(56,77)= -3.4126937389e-02
       coeff_l3(56,78)= -2.0958410203e-01
       coeff_l3(56,79)=  7.5362928212e-02
       coeff_l3(56,80)=  1.1312524974e-01
       coeff_l3(56,81)= -1.0401642323e-01
       coeff_l3(56,82)= -5.8982543647e-02
       coeff_l3(56,83)=  6.2329076231e-02
       coeff_l3(56,84)= -7.4520546198e-01
       coeff_l3(56,85)= -1.5282303095e-01
       coeff_l3(56,86)= -1.2472233921e-01
       coeff_l3(56,87)= -3.0678403378e-01
       coeff_l3(56,88)= -1.2312955409e-01
       coeff_l3(56,89)=  1.7665286362e-01
       coeff_l3(56,90)=  1.3314574957e-01
       coeff_l3(56,91)= -5.4159069061e-01
       coeff_l3(56,92)= -1.5862420201e-01
       coeff_l3(56,93)= -4.9021590501e-02
       coeff_l3(56,94)=  1.1094596237e-02
       coeff_l3(56,95)= -2.4821840227e-01
       coeff_l3(56,96)=  1.2879367173e-01
       coeff_l3(56,97)=  1.6208434105e-01
       coeff_l3(56,98)= -1.8930666149e-02
       coeff_l3(56,99)= -1.0551245883e-02
       coeff_l3(56,100)= -6.4651533961e-02
       coeff_l3(56,101)=  5.2640803158e-02
       coeff_l3(56,102)=  1.8508149683e-01
       coeff_l3(56,103)=  2.0737262070e-01
       coeff_l3(56,104)=  2.6141083241e-01
       coeff_l3(56,105)= -3.8843601942e-02
       coeff_l3(56,106)= -1.5608558059e-01
       coeff_l3(56,107)= -9.3127347529e-02
       coeff_l3(56,108)= -3.1149536371e-01
       coeff_l3(56,109)= -6.1091206968e-02
       coeff_l3(56,110)=  2.0889879763e-01
       coeff_l3(56,111)= -1.9028782845e-01
       coeff_l3(56,112)= -1.2689332962e+00
       coeff_l3(56,113)= -9.4609504938e-01
       coeff_l3(56,114)=  1.0942202061e-01
       coeff_l3(56,115)= -3.0434164405e-01
       coeff_l3(56,116)= -1.0157819092e-01
       coeff_l3(56,117)= -1.4765366912e-01
       coeff_l3(56,118)=  4.5262345672e-01
       coeff_l3(56,119)= -1.2136274576e-01
       coeff_l3(56,120)= -1.1121740192e-01
       coeff_l3(56,121)= -9.3684680760e-02
       coeff_l3(56,122)= -1.6576690972e-01
       coeff_l3(56,123)=  1.7016886175e-01
       coeff_l3(56,124)=  2.9373165965e-01
       coeff_l3(56,125)= -1.4482377470e-01
       coeff_l3(57,1)= -1.2879815102e+00
       coeff_l3(57,2)= -1.2739266269e-02
       coeff_l3(57,3)=  1.6539004445e-01
       coeff_l3(57,4)=  1.1734376848e-01
       coeff_l3(57,5)= -4.4156230986e-02
       coeff_l3(57,6)=  1.1395989358e-01
       coeff_l3(57,7)= -6.6393399239e-01
       coeff_l3(57,8)= -3.6327123642e-02
       coeff_l3(57,9)= -7.6416365802e-02
       coeff_l3(57,10)= -7.0763476193e-02
       coeff_l3(57,11)=  1.6903862357e-02
       coeff_l3(57,12)= -5.9718179703e-01
       coeff_l3(57,13)= -2.1776068211e-01
       coeff_l3(57,14)= -4.5268860459e-01
       coeff_l3(57,15)= -6.8862564862e-02
       coeff_l3(57,16)= -5.4900096729e-03
       coeff_l3(57,17)= -3.0034372211e-01
       coeff_l3(57,18)= -5.8123004436e-01
       coeff_l3(57,19)= -1.0473658890e-01
       coeff_l3(57,20)= -1.1832936108e-01
       coeff_l3(57,21)= -1.0372871533e-02
       coeff_l3(57,22)= -2.9587262543e-04
       coeff_l3(57,23)=  6.3267059624e-02
       coeff_l3(57,24)=  9.2383623123e-02
       coeff_l3(57,25)= -1.8388850987e-01
       coeff_l3(57,26)= -2.5055617094e-02
       coeff_l3(57,27)= -1.0699663162e+00
       coeff_l3(57,28)= -9.0884566307e-01
       coeff_l3(57,29)= -5.0959348679e-01
       coeff_l3(57,30)=  7.5912499428e-01
       coeff_l3(57,31)=  2.0734729245e-02
       coeff_l3(57,32)= -1.4082454145e-02
       coeff_l3(57,33)=  8.0359049141e-02
       coeff_l3(57,34)=  1.4835652895e-02
       coeff_l3(57,35)= -3.5769630224e-02
       coeff_l3(57,36)=  3.1536373496e-01
       coeff_l3(57,37)= -1.6168247163e-01
       coeff_l3(57,38)= -6.7808282375e-01
       coeff_l3(57,39)=  1.5156902373e-01
       coeff_l3(57,40)= -2.1132342517e-01
       coeff_l3(57,41)= -6.5602350235e-01
       coeff_l3(57,42)= -1.1846954376e-01
       coeff_l3(57,43)= -3.1273359060e-01
       coeff_l3(57,44)=  8.4936246276e-02
       coeff_l3(57,45)=  2.7614880353e-02
       coeff_l3(57,46)=  1.0450330377e-01
       coeff_l3(57,47)= -1.2735424936e-01
       coeff_l3(57,48)= -3.9986023307e-01
       coeff_l3(57,49)=  1.3544939756e+00
       coeff_l3(57,50)=  2.7742790058e-02
       coeff_l3(57,51)=  8.8358901441e-02
       coeff_l3(57,52)= -1.6654461622e-01
       coeff_l3(57,53)= -1.6960490495e-02
       coeff_l3(57,54)= -5.0842696428e-01
       coeff_l3(57,55)=  1.4468379319e-01
       coeff_l3(57,56)= -1.8212461844e-02
       coeff_l3(57,57)= -1.7712371051e-01
       coeff_l3(57,58)=  2.8780117631e-01
       coeff_l3(57,59)= -1.1772251874e-01
       coeff_l3(57,60)=  1.6253534555e+00
       coeff_l3(57,61)= -1.0226187110e-01
       coeff_l3(57,62)= -4.6761717647e-02
       coeff_l3(57,63)= -6.2546811998e-02
       coeff_l3(57,64)=  4.8657268286e-02
       coeff_l3(57,65)=  8.5025459528e-02
       coeff_l3(57,66)=  3.5473752767e-02
       coeff_l3(57,67)= -5.7795658708e-02
       coeff_l3(57,68)= -5.4503440857e-01
       coeff_l3(57,69)= -1.7094719410e-01
       coeff_l3(57,70)= -5.8946289122e-02
       coeff_l3(57,71)= -5.5070179701e-01
       coeff_l3(57,72)=  1.1365024000e-01
       coeff_l3(57,73)= -1.6156752408e-01
       coeff_l3(57,74)= -1.8809336424e-01
       coeff_l3(57,75)= -5.9537101537e-02
       coeff_l3(57,76)=  1.0676757097e+00
       coeff_l3(57,77)=  2.5359308347e-03
       coeff_l3(57,78)=  2.6129284874e-02
       coeff_l3(57,79)= -2.1836078167e-01
       coeff_l3(57,80)=  1.1064966917e+00
       coeff_l3(57,81)= -1.9717292488e-01
       coeff_l3(57,82)=  3.6374367774e-02
       coeff_l3(57,83)= -9.1517210007e-02
       coeff_l3(57,84)= -3.6329802126e-02
       coeff_l3(57,85)= -3.3865697682e-02
       coeff_l3(57,86)= -1.3600496948e-01
       coeff_l3(57,87)= -5.1040530205e-02
       coeff_l3(57,88)= -2.7230075002e-01
       coeff_l3(57,89)=  2.4405327439e-01
       coeff_l3(57,90)=  2.1389441192e-01
       coeff_l3(57,91)= -3.5746642947e-01
       coeff_l3(57,92)=  6.4495927654e-03
       coeff_l3(57,93)= -5.0434052944e-02
       coeff_l3(57,94)= -8.8223688304e-02
       coeff_l3(57,95)= -4.7774329782e-02
       coeff_l3(57,96)= -3.1081527472e-01
       coeff_l3(57,97)= -1.2951014936e-01
       coeff_l3(57,98)=  5.3408708423e-02
       coeff_l3(57,99)= -9.2565625906e-02
       coeff_l3(57,100)= -4.5642931014e-02
       coeff_l3(57,101)= -1.5191230178e-01
       coeff_l3(57,102)=  1.2119679153e-01
       coeff_l3(57,103)=  1.1717292666e-01
       coeff_l3(57,104)=  8.6958624423e-02
       coeff_l3(57,105)= -3.4258335829e-01
       coeff_l3(57,106)= -1.5134456754e-01
       coeff_l3(57,107)= -4.7482860088e-01
       coeff_l3(57,108)= -4.2185407132e-02
       coeff_l3(57,109)= -9.2889033258e-02
       coeff_l3(57,110)= -7.1838088334e-02
       coeff_l3(57,111)= -4.2477244139e-01
       coeff_l3(57,112)= -8.0284941196e-01
       coeff_l3(57,113)= -3.4274417162e-01
       coeff_l3(57,114)= -4.4313070178e-01
       coeff_l3(57,115)= -3.7541069090e-02
       coeff_l3(57,116)= -9.1343814135e-01
       coeff_l3(57,117)= -1.1979098618e-01
       coeff_l3(57,118)= -7.4213618040e-01
       coeff_l3(57,119)=  6.5199397504e-02
       coeff_l3(57,120)= -2.4594998360e-01
       coeff_l3(57,121)= -2.5883278251e-01
       coeff_l3(57,122)= -1.6891522706e-01
       coeff_l3(57,123)= -4.2568538338e-02
       coeff_l3(57,124)= -2.1598312259e-01
       coeff_l3(57,125)= -5.3716826439e-01
       coeff_l3(58,1)= -8.1928479671e-01
       coeff_l3(58,2)= -6.3623672724e-01
       coeff_l3(58,3)=  4.4022598863e-01
       coeff_l3(58,4)=  7.1697570384e-02
       coeff_l3(58,5)= -1.2154097110e-01
       coeff_l3(58,6)= -2.7522498369e-01
       coeff_l3(58,7)= -8.0307203531e-01
       coeff_l3(58,8)= -6.3558638096e-02
       coeff_l3(58,9)= -4.1636219621e-01
       coeff_l3(58,10)= -3.4750539064e-01
       coeff_l3(58,11)= -1.9334616140e-02
       coeff_l3(58,12)= -2.2849276662e-01
       coeff_l3(58,13)= -5.4690080881e-01
       coeff_l3(58,14)= -6.2282860279e-01
       coeff_l3(58,15)=  2.0694330335e-02
       coeff_l3(58,16)= -4.5182894915e-02
       coeff_l3(58,17)= -5.2654230595e-01
       coeff_l3(58,18)= -1.1169928312e-01
       coeff_l3(58,19)= -3.6336842179e-01
       coeff_l3(58,20)= -7.7724963427e-01
       coeff_l3(58,21)=  7.9633378983e-01
       coeff_l3(58,22)=  1.0074660927e-01
       coeff_l3(58,23)=  1.5563516319e-01
       coeff_l3(58,24)= -5.6048256159e-01
       coeff_l3(58,25)=  8.6448532343e-01
       coeff_l3(58,26)= -1.7177654803e-01
       coeff_l3(58,27)= -7.8744536638e-01
       coeff_l3(58,28)= -8.2866811752e-01
       coeff_l3(58,29)=  2.3737607896e-01
       coeff_l3(58,30)= -4.4950747490e-01
       coeff_l3(58,31)= -8.9628523588e-01
       coeff_l3(58,32)= -1.8894761801e-01
       coeff_l3(58,33)=  1.0406262428e-01
       coeff_l3(58,34)= -3.1818619370e-01
       coeff_l3(58,35)=  3.4751188755e-01
       coeff_l3(58,36)= -5.5805063248e-01
       coeff_l3(58,37)= -4.9602288753e-02
       coeff_l3(58,38)=  8.6257189512e-01
       coeff_l3(58,39)= -2.0204798877e-01
       coeff_l3(58,40)= -2.0126222074e-01
       coeff_l3(58,41)=  3.0016952753e-01
       coeff_l3(58,42)= -2.0784284174e-01
       coeff_l3(58,43)= -3.0353158712e-01
       coeff_l3(58,44)= -1.5388257802e-01
       coeff_l3(58,45)= -4.7240385413e-01
       coeff_l3(58,46)= -2.3746512830e-01
       coeff_l3(58,47)=  5.6217366457e-01
       coeff_l3(58,48)= -1.9408790767e-01
       coeff_l3(58,49)=  1.7845559865e-02
       coeff_l3(58,50)= -3.5825520754e-03
       coeff_l3(58,51)= -3.0400913954e-01
       coeff_l3(58,52)=  7.0972934365e-02
       coeff_l3(58,53)= -4.0783047676e-01
       coeff_l3(58,54)= -1.2179303169e+00
       coeff_l3(58,55)= -7.1086406708e-02
       coeff_l3(58,56)=  3.3326130360e-02
       coeff_l3(58,57)= -2.0274586976e-01
       coeff_l3(58,58)=  4.7029611468e-01
       coeff_l3(58,59)= -1.9699819386e-01
       coeff_l3(58,60)= -1.2654646998e-03
       coeff_l3(58,61)= -2.2497211397e-01
       coeff_l3(58,62)= -2.1576248109e-01
       coeff_l3(58,63)=  1.7949461937e-01
       coeff_l3(58,64)= -7.5566209853e-02
       coeff_l3(58,65)= -1.4662778378e-01
       coeff_l3(58,66)= -9.8834827542e-02
       coeff_l3(58,67)=  2.7087087631e+00
       coeff_l3(58,68)= -1.1537675858e+00
       coeff_l3(58,69)=  2.6593834162e-01
       coeff_l3(58,70)= -4.0162530541e-01
       coeff_l3(58,71)=  1.7148108780e-01
       coeff_l3(58,72)= -2.3782308400e-01
       coeff_l3(58,73)= -2.3789037764e-01
       coeff_l3(58,74)=  1.4500616789e+00
       coeff_l3(58,75)= -2.1122424304e-01
       coeff_l3(58,76)= -5.9154899791e-03
       coeff_l3(58,77)= -7.0469504595e-01
       coeff_l3(58,78)= -7.9597033560e-02
       coeff_l3(58,79)= -2.4040387943e-02
       coeff_l3(58,80)= -5.0884388387e-02
       coeff_l3(58,81)= -4.1992497444e-01
       coeff_l3(58,82)=  2.1538187563e-01
       coeff_l3(58,83)= -1.8117803335e-01
       coeff_l3(58,84)=  4.2340108752e-01
       coeff_l3(58,85)= -7.4183154106e-01
       coeff_l3(58,86)= -1.4928881824e-01
       coeff_l3(58,87)=  1.5578892827e-01
       coeff_l3(58,88)=  5.5233466625e-01
       coeff_l3(58,89)= -2.6209452748e-01
       coeff_l3(58,90)=  1.2640249729e-01
       coeff_l3(58,91)=  3.4637302160e-01
       coeff_l3(58,92)= -2.8359353542e-01
       coeff_l3(58,93)= -6.8971663713e-01
       coeff_l3(58,94)=  2.1163069177e-03
       coeff_l3(58,95)= -3.0603566766e-01
       coeff_l3(58,96)= -8.2623790950e-03
       coeff_l3(58,97)= -1.7203716934e-01
       coeff_l3(58,98)= -3.7043544650e-01
       coeff_l3(58,99)= -1.6924200952e-01
       coeff_l3(58,100)= -1.8749785423e-01
       coeff_l3(58,101)=  4.5764911175e-01
       coeff_l3(58,102)= -2.4517782032e-01
       coeff_l3(58,103)= -4.2931094766e-01
       coeff_l3(58,104)= -6.2760061026e-01
       coeff_l3(58,105)=  7.7682334185e-01
       coeff_l3(58,106)= -2.6550891995e-01
       coeff_l3(58,107)= -6.9160515070e-01
       coeff_l3(58,108)= -1.7131961882e-01
       coeff_l3(58,109)= -1.3296709955e-01
       coeff_l3(58,110)= -2.2983540595e-01
       coeff_l3(58,111)= -2.8322178125e-01
       coeff_l3(58,112)=  2.0626549423e-01
       coeff_l3(58,113)= -5.6179249287e-01
       coeff_l3(58,114)= -4.5903238654e-01
       coeff_l3(58,115)=  8.7678992748e-01
       coeff_l3(58,116)= -3.4153410792e-01
       coeff_l3(58,117)=  2.8254809976e-01
       coeff_l3(58,118)=  1.0669637918e+00
       coeff_l3(58,119)=  8.9189916849e-02
       coeff_l3(58,120)= -1.0163744539e-01
       coeff_l3(58,121)= -2.9614162445e-01
       coeff_l3(58,122)=  8.4350425750e-03
       coeff_l3(58,123)= -1.2897053361e-01
       coeff_l3(58,124)= -3.4016948938e-01
       coeff_l3(58,125)= -4.4445064664e-01
       coeff_l3(59,1)= -9.1489541531e-01
       coeff_l3(59,2)=  1.5424399078e-01
       coeff_l3(59,3)= -3.1250888109e-01
       coeff_l3(59,4)= -2.3320996761e-01
       coeff_l3(59,5)=  1.6047506034e-01
       coeff_l3(59,6)=  1.7336089909e-01
       coeff_l3(59,7)=  3.2469320297e-01
       coeff_l3(59,8)= -7.6330035925e-02
       coeff_l3(59,9)= -1.1838559806e-01
       coeff_l3(59,10)=  1.9618680701e-02
       coeff_l3(59,11)=  5.7201575488e-02
       coeff_l3(59,12)= -5.1866245270e-01
       coeff_l3(59,13)= -5.7353985310e-01
       coeff_l3(59,14)= -6.0520816594e-02
       coeff_l3(59,15)=  1.6657698154e-01
       coeff_l3(59,16)=  1.4403051138e-01
       coeff_l3(59,17)= -4.0809521079e-01
       coeff_l3(59,18)= -1.0354915857e+00
       coeff_l3(59,19)=  3.7787783891e-02
       coeff_l3(59,20)= -2.0954649150e-01
       coeff_l3(59,21)=  3.1256610155e-01
       coeff_l3(59,22)= -7.0189706981e-02
       coeff_l3(59,23)=  3.2860383391e-02
       coeff_l3(59,24)=  1.7879977822e-01
       coeff_l3(59,25)= -2.3375479877e-01
       coeff_l3(59,26)=  1.3456541300e-01
       coeff_l3(59,27)=  8.7987259030e-02
       coeff_l3(59,28)=  6.0809303075e-02
       coeff_l3(59,29)= -9.0317243338e-01
       coeff_l3(59,30)=  1.6665792465e-01
       coeff_l3(59,31)= -1.2690749019e-02
       coeff_l3(59,32)= -2.4073157459e-02
       coeff_l3(59,33)= -5.5665999651e-02
       coeff_l3(59,34)= -6.2492273748e-02
       coeff_l3(59,35)=  6.6484469175e-01
       coeff_l3(59,36)=  5.4065126926e-02
       coeff_l3(59,37)=  1.7752785981e-01
       coeff_l3(59,38)= -3.1033578515e-01
       coeff_l3(59,39)=  1.3565823436e-01
       coeff_l3(59,40)= -1.8261767924e-01
       coeff_l3(59,41)=  1.0885811597e-01
       coeff_l3(59,42)=  1.6964805126e-01
       coeff_l3(59,43)= -3.1091180444e-01
       coeff_l3(59,44)=  4.1704945266e-02
       coeff_l3(59,45)=  4.0511995554e-02
       coeff_l3(59,46)= -4.0246790648e-01
       coeff_l3(59,47)= -9.6156813204e-02
       coeff_l3(59,48)= -3.0114060640e-01
       coeff_l3(59,49)= -3.5537955165e-01
       coeff_l3(59,50)= -4.7016970813e-02
       coeff_l3(59,51)= -1.4509305358e-01
       coeff_l3(59,52)= -2.4313006550e-02
       coeff_l3(59,53)=  2.0539560914e-01
       coeff_l3(59,54)=  4.3246936053e-02
       coeff_l3(59,55)=  3.4782925248e-01
       coeff_l3(59,56)=  1.6217021644e-01
       coeff_l3(59,57)=  1.6883161664e-01
       coeff_l3(59,58)= -4.0222057700e-01
       coeff_l3(59,59)= -4.4929528236e-01
       coeff_l3(59,60)=  3.7276151776e-01
       coeff_l3(59,61)=  1.8378017470e-02
       coeff_l3(59,62)= -1.0657434352e-02
       coeff_l3(59,63)=  5.7282859087e-01
       coeff_l3(59,64)=  2.8544155881e-02
       coeff_l3(59,65)=  1.0737486929e-01
       coeff_l3(59,66)=  1.5997258946e-02
       coeff_l3(59,67)= -1.7905570567e-01
       coeff_l3(59,68)= -2.4443936348e-01
       coeff_l3(59,69)= -4.4705919921e-02
       coeff_l3(59,70)=  8.7168011814e-03
       coeff_l3(59,71)= -7.3668324947e-01
       coeff_l3(59,72)=  3.5353749990e-02
       coeff_l3(59,73)=  2.8938394040e-02
       coeff_l3(59,74)=  8.1531638280e-03
       coeff_l3(59,75)= -8.5918791592e-02
       coeff_l3(59,76)=  2.7964732051e-01
       coeff_l3(59,77)=  1.4263209701e-01
       coeff_l3(59,78)= -3.2266501337e-02
       coeff_l3(59,79)= -1.5321424603e-01
       coeff_l3(59,80)= -9.0355902910e-02
       coeff_l3(59,81)= -7.0750534534e-02
       coeff_l3(59,82)= -2.2536350787e-01
       coeff_l3(59,83)= -1.4920380712e-01
       coeff_l3(59,84)=  9.0102087706e-03
       coeff_l3(59,85)= -1.0136593133e-01
       coeff_l3(59,86)=  1.6152867675e-01
       coeff_l3(59,87)= -7.0958524942e-02
       coeff_l3(59,88)= -3.4057784081e-01
       coeff_l3(59,89)=  8.1476897001e-02
       coeff_l3(59,90)=  1.4848874509e-01
       coeff_l3(59,91)= -4.6668127179e-02
       coeff_l3(59,92)=  7.4419528246e-02
       coeff_l3(59,93)=  2.3590630293e-01
       coeff_l3(59,94)= -4.4199135154e-02
       coeff_l3(59,95)= -7.6416499913e-02
       coeff_l3(59,96)= -2.6756069064e-01
       coeff_l3(59,97)= -2.2504264489e-02
       coeff_l3(59,98)=  2.2663320601e-01
       coeff_l3(59,99)= -3.6522802711e-01
       coeff_l3(59,100)= -2.2226005793e-01
       coeff_l3(59,101)= -4.9447037280e-02
       coeff_l3(59,102)=  1.1203300208e-01
       coeff_l3(59,103)= -4.5123651624e-02
       coeff_l3(59,104)= -4.6285599470e-02
       coeff_l3(59,105)= -2.1591167152e-01
       coeff_l3(59,106)= -1.4679075778e-01
       coeff_l3(59,107)= -3.5196170211e-01
       coeff_l3(59,108)= -2.9525265098e-01
       coeff_l3(59,109)=  6.3896767795e-02
       coeff_l3(59,110)=  5.0984859467e-02
       coeff_l3(59,111)= -3.5922470689e-01
       coeff_l3(59,112)= -6.9072711468e-01
       coeff_l3(59,113)= -4.2553302646e-01
       coeff_l3(59,114)=  3.6075699329e-01
       coeff_l3(59,115)= -2.3798295856e-01
       coeff_l3(59,116)= -2.4775442481e-01
       coeff_l3(59,117)= -2.3060725629e-01
       coeff_l3(59,118)=  2.6126182079e-01
       coeff_l3(59,119)=  1.8443277478e-01
       coeff_l3(59,120)= -1.9366581738e-01
       coeff_l3(59,121)= -5.8042198420e-01
       coeff_l3(59,122)= -1.3786718249e-01
       coeff_l3(59,123)=  2.0236852765e-01
       coeff_l3(59,124)=  2.0286515355e-01
       coeff_l3(59,125)= -1.0713085532e-01
       coeff_l3(60,1)= -6.4315311611e-02
       coeff_l3(60,2)= -8.1399828196e-02
       coeff_l3(60,3)= -3.3572068810e-01
       coeff_l3(60,4)= -7.6470725238e-02
       coeff_l3(60,5)= -2.0566116273e-01
       coeff_l3(60,6)=  8.6125925183e-02
       coeff_l3(60,7)= -3.2809671015e-02
       coeff_l3(60,8)= -2.1814224124e-01
       coeff_l3(60,9)= -1.6433791816e-01
       coeff_l3(60,10)=  5.1198331639e-03
       coeff_l3(60,11)= -1.2603527308e-01
       coeff_l3(60,12)= -1.7301875353e-01
       coeff_l3(60,13)=  1.4989238977e-01
       coeff_l3(60,14)= -7.0286720991e-02
       coeff_l3(60,15)= -1.4769460075e-02
       coeff_l3(60,16)= -9.3655437231e-03
       coeff_l3(60,17)= -1.1042881012e-01
       coeff_l3(60,18)= -3.0752348900e-01
       coeff_l3(60,19)=  1.1301658303e-01
       coeff_l3(60,20)= -1.3138798997e-02
       coeff_l3(60,21)=  3.2835599035e-02
       coeff_l3(60,22)=  2.8356376290e-01
       coeff_l3(60,23)= -8.7075807154e-02
       coeff_l3(60,24)= -6.6317223012e-02
       coeff_l3(60,25)= -2.9619491100e-01
       coeff_l3(60,26)=  1.3559769094e-01
       coeff_l3(60,27)=  1.1654756218e-01
       coeff_l3(60,28)=  3.2323457301e-02
       coeff_l3(60,29)= -3.8532331586e-01
       coeff_l3(60,30)= -6.6136777401e-02
       coeff_l3(60,31)=  2.9873449355e-02
       coeff_l3(60,32)= -1.8187053502e-01
       coeff_l3(60,33)= -2.7808699012e-01
       coeff_l3(60,34)= -1.2731906772e-01
       coeff_l3(60,35)= -1.1572121829e-01
       coeff_l3(60,36)=  1.8961687386e-01
       coeff_l3(60,37)= -1.0945213586e-01
       coeff_l3(60,38)= -4.3600928783e-01
       coeff_l3(60,39)=  4.3990410864e-02
       coeff_l3(60,40)= -2.5710594654e-01
       coeff_l3(60,41)= -2.7864000201e-01
       coeff_l3(60,42)= -1.3094902039e-01
       coeff_l3(60,43)= -4.7040976584e-02
       coeff_l3(60,44)= -2.9855912551e-02
       coeff_l3(60,45)=  1.9249530137e-01
       coeff_l3(60,46)=  8.7029926479e-02
       coeff_l3(60,47)= -2.2983989120e-01
       coeff_l3(60,48)= -1.8312680721e-01
       coeff_l3(60,49)= -2.6025751233e-01
       coeff_l3(60,50)= -3.5031512380e-02
       coeff_l3(60,51)= -1.5796551108e-01
       coeff_l3(60,52)= -8.5786394775e-02
       coeff_l3(60,53)=  1.6963753104e-01
       coeff_l3(60,54)= -2.6615828276e-01
       coeff_l3(60,55)= -1.5892679989e-01
       coeff_l3(60,56)= -1.4092724025e-01
       coeff_l3(60,57)= -3.6325383931e-02
       coeff_l3(60,58)= -2.0698027313e-01
       coeff_l3(60,59)=  2.6052176952e-01
       coeff_l3(60,60)= -5.5653822422e-01
       coeff_l3(60,61)=  4.7148637474e-02
       coeff_l3(60,62)=  8.3028739318e-03
       coeff_l3(60,63)=  2.2306950390e-01
       coeff_l3(60,64)=  1.1375923641e-02
       coeff_l3(60,65)=  3.1782041769e-03
       coeff_l3(60,66)= -8.6454913020e-02
       coeff_l3(60,67)= -1.1062890291e-01
       coeff_l3(60,68)= -4.0429692715e-02
       coeff_l3(60,69)= -2.1246588230e-01
       coeff_l3(60,70)= -2.1873775125e-01
       coeff_l3(60,71)= -1.7483364046e-01
       coeff_l3(60,72)= -1.4017392695e-01
       coeff_l3(60,73)= -1.0737808049e-01
       coeff_l3(60,74)= -3.0170869827e-01
       coeff_l3(60,75)= -7.4549853802e-02
       coeff_l3(60,76)= -4.9042183906e-02
       coeff_l3(60,77)=  5.8795690536e-02
       coeff_l3(60,78)=  4.4758260250e-02
       coeff_l3(60,79)= -1.0830477625e-01
       coeff_l3(60,80)= -1.3227750361e-01
       coeff_l3(60,81)= -1.2595637143e-01
       coeff_l3(60,82)=  3.0348405242e-02
       coeff_l3(60,83)= -4.5452207327e-02
       coeff_l3(60,84)= -2.5560045615e-02
       coeff_l3(60,85)= -2.7007805184e-02
       coeff_l3(60,86)=  4.9834068865e-02
       coeff_l3(60,87)= -1.5207290649e-01
       coeff_l3(60,88)= -1.9911068678e-01
       coeff_l3(60,89)= -1.2458641082e-01
       coeff_l3(60,90)= -1.4915062487e-01
       coeff_l3(60,91)= -7.2740048170e-02
       coeff_l3(60,92)= -2.0326265693e-01
       coeff_l3(60,93)=  2.0675320923e-01
       coeff_l3(60,94)=  4.6308968216e-02
       coeff_l3(60,95)=  2.2058669478e-02
       coeff_l3(60,96)= -4.3409813195e-02
       coeff_l3(60,97)= -9.4900093973e-02
       coeff_l3(60,98)= -1.4123726636e-02
       coeff_l3(60,99)=  1.7032487690e-01
       coeff_l3(60,100)= -1.8877999485e-01
       coeff_l3(60,101)=  1.3122924604e-02
       coeff_l3(60,102)= -1.0834045708e-02
       coeff_l3(60,103)=  5.3549852222e-02
       coeff_l3(60,104)=  3.5294793546e-02
       coeff_l3(60,105)= -2.0947001874e-01
       coeff_l3(60,106)= -5.9437837452e-02
       coeff_l3(60,107)=  7.8643359244e-02
       coeff_l3(60,108)=  2.5190453976e-02
       coeff_l3(60,109)=  4.4838838279e-02
       coeff_l3(60,110)= -1.4832744002e-01
       coeff_l3(60,111)= -1.5658885241e-02
       coeff_l3(60,112)= -3.8257277012e-01
       coeff_l3(60,113)=  1.3358984888e-01
       coeff_l3(60,114)= -1.1781291105e-02
       coeff_l3(60,115)= -1.6374875605e-01
       coeff_l3(60,116)= -7.3208212852e-02
       coeff_l3(60,117)= -3.1117096543e-01
       coeff_l3(60,118)= -2.1419894695e-01
       coeff_l3(60,119)= -1.8380562961e-01
       coeff_l3(60,120)=  3.6585871130e-02
       coeff_l3(60,121)=  7.4732422829e-02
       coeff_l3(60,122)=  8.1960828975e-03
       coeff_l3(60,123)=  1.4766817912e-02
       coeff_l3(60,124)= -1.5295907855e-01
       coeff_l3(60,125)= -8.0482847989e-02
       coeff_l3(61,1)= -1.7901359126e-02
       coeff_l3(61,2)= -8.6307577789e-02
       coeff_l3(61,3)= -2.9188619927e-02
       coeff_l3(61,4)=  2.6022580266e-01
       coeff_l3(61,5)=  1.6262011230e-01
       coeff_l3(61,6)=  4.5245170593e-02
       coeff_l3(61,7)= -9.0649649501e-02
       coeff_l3(61,8)=  1.4663885534e-01
       coeff_l3(61,9)=  1.1059924960e-01
       coeff_l3(61,10)= -1.1614071578e-01
       coeff_l3(61,11)=  1.3233855367e-01
       coeff_l3(61,12)=  3.7534025311e-01
       coeff_l3(61,13)=  2.0394761860e-01
       coeff_l3(61,14)= -5.7598445565e-02
       coeff_l3(61,15)= -1.3683074713e-01
       coeff_l3(61,16)=  2.8326099738e-02
       coeff_l3(61,17)= -1.2500580400e-02
       coeff_l3(61,18)=  3.1066441536e-01
       coeff_l3(61,19)= -2.1538388729e-01
       coeff_l3(61,20)= -1.1006620526e-01
       coeff_l3(61,21)= -1.6402541101e-01
       coeff_l3(61,22)=  4.8152606934e-02
       coeff_l3(61,23)=  1.2163130939e-01
       coeff_l3(61,24)= -1.1596548557e-01
       coeff_l3(61,25)= -1.0820574313e-01
       coeff_l3(61,26)= -7.6092280447e-02
       coeff_l3(61,27)= -6.2978073955e-02
       coeff_l3(61,28)= -1.2983265519e-01
       coeff_l3(61,29)=  3.9987471700e-01
       coeff_l3(61,30)=  1.2362290174e-01
       coeff_l3(61,31)= -1.9049561024e-01
       coeff_l3(61,32)= -1.7975190282e-01
       coeff_l3(61,33)=  5.4243646562e-02
       coeff_l3(61,34)= -2.9000138864e-02
       coeff_l3(61,35)=  1.2070846558e-01
       coeff_l3(61,36)=  2.0933004096e-02
       coeff_l3(61,37)=  1.7691826448e-02
       coeff_l3(61,38)=  5.0582861900e-01
       coeff_l3(61,39)= -1.9602791965e-01
       coeff_l3(61,40)=  1.5071044862e-01
       coeff_l3(61,41)=  5.2756869793e-01
       coeff_l3(61,42)= -2.3960860074e-01
       coeff_l3(61,43)= -2.8954440728e-02
       coeff_l3(61,44)= -7.5254343450e-02
       coeff_l3(61,45)= -1.9423465431e-01
       coeff_l3(61,46)= -2.5340881944e-01
       coeff_l3(61,47)=  2.5427715853e-02
       coeff_l3(61,48)=  2.0996929705e-01
       coeff_l3(61,49)= -8.9861668646e-02
       coeff_l3(61,50)=  1.5788613260e-01
       coeff_l3(61,51)= -1.1739594489e-01
       coeff_l3(61,52)=  1.3949014246e-01
       coeff_l3(61,53)= -9.3208856881e-02
       coeff_l3(61,54)=  1.5549075603e-01
       coeff_l3(61,55)= -7.9997949302e-02
       coeff_l3(61,56)=  1.8654083833e-02
       coeff_l3(61,57)= -9.1941066086e-02
       coeff_l3(61,58)= -4.6929880977e-02
       coeff_l3(61,59)=  2.7786082029e-01
       coeff_l3(61,60)= -2.3331317306e-01
       coeff_l3(61,61)= -1.4655807754e-03
       coeff_l3(61,62)= -1.4005421102e-01
       coeff_l3(61,63)= -1.0534117371e-01
       coeff_l3(61,64)=  5.1945713349e-03
       coeff_l3(61,65)= -5.7312294841e-02
       coeff_l3(61,66)= -1.4630068839e-01
       coeff_l3(61,67)=  1.9568829238e-01
       coeff_l3(61,68)=  9.2788174748e-02
       coeff_l3(61,69)= -7.1043483913e-02
       coeff_l3(61,70)= -2.5513976812e-01
       coeff_l3(61,71)=  3.4309360385e-01
       coeff_l3(61,72)=  5.7234237902e-03
       coeff_l3(61,73)= -1.4649659395e-01
       coeff_l3(61,74)=  1.1472984403e-01
       coeff_l3(61,75)= -7.1538753808e-02
       coeff_l3(61,76)= -4.7619253397e-01
       coeff_l3(61,77)= -1.0347685777e-02
       coeff_l3(61,78)=  7.8338980675e-02
       coeff_l3(61,79)=  1.3392499089e-01
       coeff_l3(61,80)= -5.4160189629e-01
       coeff_l3(61,81)= -1.7372553051e-01
       coeff_l3(61,82)=  5.8688975871e-02
       coeff_l3(61,83)=  1.7924256623e-01
       coeff_l3(61,84)=  1.0128059983e-01
       coeff_l3(61,85)=  9.5291934907e-02
       coeff_l3(61,86)= -9.8179228604e-02
       coeff_l3(61,87)=  1.1957897991e-01
       coeff_l3(61,88)=  4.0698621422e-02
       coeff_l3(61,89)= -2.5377674028e-02
       coeff_l3(61,90)= -1.2438473105e-01
       coeff_l3(61,91)=  8.0813586712e-02
       coeff_l3(61,92)=  2.3765531182e-01
       coeff_l3(61,93)= -2.0391336083e-01
       coeff_l3(61,94)= -7.5318127871e-02
       coeff_l3(61,95)= -8.1381626427e-02
       coeff_l3(61,96)=  1.4755167067e-01
       coeff_l3(61,97)=  2.7184855193e-02
       coeff_l3(61,98)= -8.5209874669e-04
       coeff_l3(61,99)=  1.4279924333e-01
       coeff_l3(61,100)= -9.7555071115e-02
       coeff_l3(61,101)= -1.2596087158e-01
       coeff_l3(61,102)= -2.4787288159e-02
       coeff_l3(61,103)= -6.6078923643e-02
       coeff_l3(61,104)= -5.4741248488e-02
       coeff_l3(61,105)=  1.8020235002e-01
       coeff_l3(61,106)= -8.6266197264e-02
       coeff_l3(61,107)= -9.8712975159e-03
       coeff_l3(61,108)=  6.6422104836e-02
       coeff_l3(61,109)= -2.7438046411e-02
       coeff_l3(61,110)=  3.5565827042e-02
       coeff_l3(61,111)= -4.4356588274e-02
       coeff_l3(61,112)=  2.8407683969e-01
       coeff_l3(61,113)= -2.3868834972e-01
       coeff_l3(61,114)= -4.0554832667e-02
       coeff_l3(61,115)= -4.4181700796e-02
       coeff_l3(61,116)=  2.8499609232e-01
       coeff_l3(61,117)=  1.7220881581e-01
       coeff_l3(61,118)=  2.2765487432e-01
       coeff_l3(61,119)= -5.7207874954e-02
       coeff_l3(61,120)=  1.3961167634e-01
       coeff_l3(61,121)=  9.0190790594e-02
       coeff_l3(61,122)= -2.5649989024e-02
       coeff_l3(61,123)= -2.5742834806e-01
       coeff_l3(61,124)= -2.4194326997e-01
       coeff_l3(61,125)=  1.0032438673e-02
       coeff_l3(62,1)=  1.8528439999e+00
       coeff_l3(62,2)= -4.6005573124e-02
       coeff_l3(62,3)=  7.5457900763e-02
       coeff_l3(62,4)= -4.7706350684e-01
       coeff_l3(62,5)= -2.7967000008e-01
       coeff_l3(62,6)= -3.7096732855e-01
       coeff_l3(62,7)= -4.4646447897e-01
       coeff_l3(62,8)= -5.0521349907e-01
       coeff_l3(62,9)=  2.3259537295e-02
       coeff_l3(62,10)= -1.0091666132e-01
       coeff_l3(62,11)= -2.2834710777e-01
       coeff_l3(62,12)= -5.1702159643e-01
       coeff_l3(62,13)= -5.7866132259e-01
       coeff_l3(62,14)=  2.7649173141e-01
       coeff_l3(62,15)= -1.0869266838e-01
       coeff_l3(62,16)=  1.0309753567e-01
       coeff_l3(62,17)=  4.5753720403e-01
       coeff_l3(62,18)=  3.9526081085e-01
       coeff_l3(62,19)=  3.5891305655e-02
       coeff_l3(62,20)= -4.1260966659e-01
       coeff_l3(62,21)= -5.5504173040e-01
       coeff_l3(62,22)=  1.9695740938e-01
       coeff_l3(62,23)= -5.3971177340e-01
       coeff_l3(62,24)= -5.1654994488e-01
       coeff_l3(62,25)= -1.2258658409e+00
       coeff_l3(62,26)= -4.6844877303e-02
       coeff_l3(62,27)= -5.9147673845e-01
       coeff_l3(62,28)= -2.0265740156e-01
       coeff_l3(62,29)= -1.3557335138e+00
       coeff_l3(62,30)=  2.8667461872e-01
       coeff_l3(62,31)= -2.1507605910e-02
       coeff_l3(62,32)=  2.8936749697e-01
       coeff_l3(62,33)= -4.8754298687e-01
       coeff_l3(62,34)=  9.1482557356e-02
       coeff_l3(62,35)= -6.6244482994e-01
       coeff_l3(62,36)= -1.1744736135e-01
       coeff_l3(62,37)= -3.9186784625e-01
       coeff_l3(62,38)= -1.2787966728e+00
       coeff_l3(62,39)=  1.5696084499e-01
       coeff_l3(62,40)= -2.7365913987e-01
       coeff_l3(62,41)= -1.5886578560e+00
       coeff_l3(62,42)= -1.6776032746e-02
       coeff_l3(62,43)= -8.7180212140e-02
       coeff_l3(62,44)= -1.1784827709e-01
       coeff_l3(62,45)=  1.0113028288e+00
       coeff_l3(62,46)= -1.0027141869e-01
       coeff_l3(62,47)= -1.1380850077e+00
       coeff_l3(62,48)= -3.9963322878e-01
       coeff_l3(62,49)=  1.4509747028e+00
       coeff_l3(62,50)= -5.7216686010e-01
       coeff_l3(62,51)=  8.6790579557e-01
       coeff_l3(62,52)= -2.5341248512e-01
       coeff_l3(62,53)= -8.2288458943e-02
       coeff_l3(62,54)= -8.1889115274e-02
       coeff_l3(62,55)= -3.7857666612e-01
       coeff_l3(62,56)= -4.4054877758e-01
       coeff_l3(62,57)= -3.9504125714e-01
       coeff_l3(62,58)= -1.1696397066e+00
       coeff_l3(62,59)= -3.7513667345e-01
       coeff_l3(62,60)=  8.2012563944e-01
       coeff_l3(62,61)= -3.4158757329e-01
       coeff_l3(62,62)= -3.2854419947e-01
       coeff_l3(62,63)=  2.5074782968e-01
       coeff_l3(62,64)= -4.4009277225e-01
       coeff_l3(62,65)= -2.9600858688e-01
       coeff_l3(62,66)= -1.2939064205e-01
       coeff_l3(62,67)= -2.1906483173e-01
       coeff_l3(62,68)= -3.6187672615e-01
       coeff_l3(62,69)= -2.6875767112e-01
       coeff_l3(62,70)= -4.0690949559e-01
       coeff_l3(62,71)=  3.8441663980e-01
       coeff_l3(62,72)=  3.4577980638e-02
       coeff_l3(62,73)= -9.3903559446e-01
       coeff_l3(62,74)= -3.2946628332e-01
       coeff_l3(62,75)= -2.4165135622e-01
       coeff_l3(62,76)=  4.4587653875e-01
       coeff_l3(62,77)=  1.5181513131e-01
       coeff_l3(62,78)=  1.9796422124e-01
       coeff_l3(62,79)= -3.7135273218e-02
       coeff_l3(62,80)=  3.8571780920e-01
       coeff_l3(62,81)= -7.5792300701e-01
       coeff_l3(62,82)= -4.6157869697e-01
       coeff_l3(62,83)= -9.1171495616e-02
       coeff_l3(62,84)=  9.9424272776e-02
       coeff_l3(62,85)= -8.3964699507e-01
       coeff_l3(62,86)=  2.2378010675e-02
       coeff_l3(62,87)=  1.7253383994e-01
       coeff_l3(62,88)= -4.4240385294e-01
       coeff_l3(62,89)= -6.5932452679e-01
       coeff_l3(62,90)= -3.7014332414e-01
       coeff_l3(62,91)=  7.6149851084e-02
       coeff_l3(62,92)= -4.6916148067e-01
       coeff_l3(62,93)=  7.0341438055e-02
       coeff_l3(62,94)= -4.5756238699e-01
       coeff_l3(62,95)= -4.1176483035e-01
       coeff_l3(62,96)= -1.1098010540e+00
       coeff_l3(62,97)= -3.0933889747e-01
       coeff_l3(62,98)=  2.8356242180e-01
       coeff_l3(62,99)= -2.6397344470e-01
       coeff_l3(62,100)= -2.5294059515e-01
       coeff_l3(62,101)= -9.8415561020e-02
       coeff_l3(62,102)= -2.2282236814e-01
       coeff_l3(62,103)= -1.3536529243e-01
       coeff_l3(62,104)= -1.4706538618e-01
       coeff_l3(62,105)= -9.7480750084e-01
       coeff_l3(62,106)= -6.9717545994e-03
       coeff_l3(62,107)= -1.7144304514e-01
       coeff_l3(62,108)=  3.4986025095e-01
       coeff_l3(62,109)=  2.8699092567e-02
       coeff_l3(62,110)= -1.8924498931e-02
       coeff_l3(62,111)= -3.0727481842e-01
       coeff_l3(62,112)=  5.7383167744e-01
       coeff_l3(62,113)=  8.6873614788e-01
       coeff_l3(62,114)=  2.4201920256e-02
       coeff_l3(62,115)= -6.8766564131e-01
       coeff_l3(62,116)= -1.1212337017e+00
       coeff_l3(62,117)= -4.7818791866e-01
       coeff_l3(62,118)= -6.5579128265e-01
       coeff_l3(62,119)= -2.8029260039e-01
       coeff_l3(62,120)= -5.1257330924e-02
       coeff_l3(62,121)= -5.3136545420e-01
       coeff_l3(62,122)= -2.0607511699e-01
       coeff_l3(62,123)= -7.8145019710e-02
       coeff_l3(62,124)= -6.0858167708e-02
       coeff_l3(62,125)= -1.2530431151e-01
       coeff_l3(63,1)= -3.1766116619e-01
       coeff_l3(63,2)= -9.4319730997e-02
       coeff_l3(63,3)=  7.1405494213e-01
       coeff_l3(63,4)= -3.9839920402e-01
       coeff_l3(63,5)=  5.6790921837e-02
       coeff_l3(63,6)=  9.9801629782e-02
       coeff_l3(63,7)=  2.5822973251e+00
       coeff_l3(63,8)= -1.3838145137e-01
       coeff_l3(63,9)= -2.5120675564e-02
       coeff_l3(63,10)= -1.9306269288e-01
       coeff_l3(63,11)=  6.0454424471e-02
       coeff_l3(63,12)= -4.5206108689e-01
       coeff_l3(63,13)= -5.5443972349e-01
       coeff_l3(63,14)= -2.1070368588e-01
       coeff_l3(63,15)=  1.1367829889e-01
       coeff_l3(63,16)=  1.0772643238e-01
       coeff_l3(63,17)= -4.8312562704e-01
       coeff_l3(63,18)= -9.9353456497e-01
       coeff_l3(63,19)=  4.2971634865e-01
       coeff_l3(63,20)= -4.7705508769e-03
       coeff_l3(63,21)=  6.3074302673e-01
       coeff_l3(63,22)= -2.3813031614e-02
       coeff_l3(63,23)=  1.5672864020e-01
       coeff_l3(63,24)=  1.4714215696e-01
       coeff_l3(63,25)= -1.7409214377e-01
       coeff_l3(63,26)=  6.7123621702e-02
       coeff_l3(63,27)=  3.7794630527e+00
       coeff_l3(63,28)=  2.5929527283e+00
       coeff_l3(63,29)= -8.1219393015e-01
       coeff_l3(63,30)= -4.8861186951e-02
       coeff_l3(63,31)=  3.1948143244e-01
       coeff_l3(63,32)=  2.1123783290e-01
       coeff_l3(63,33)=  2.8280111030e-02
       coeff_l3(63,34)=  1.0047852248e-01
       coeff_l3(63,35)=  5.7878452539e-01
       coeff_l3(63,36)=  6.6417404450e-03
       coeff_l3(63,37)=  3.6611616611e-02
       coeff_l3(63,38)= -8.0365097523e-01
       coeff_l3(63,39)=  1.1137440056e-01
       coeff_l3(63,40)= -2.0008155704e-01
       coeff_l3(63,41)= -5.7079415768e-02
       coeff_l3(63,42)=  1.5106309950e-01
       coeff_l3(63,43)= -2.0792701840e-01
       coeff_l3(63,44)=  2.4241843820e-01
       coeff_l3(63,45)= -2.2641193867e-01
       coeff_l3(63,46)= -1.9320116937e-01
       coeff_l3(63,47)=  6.6032089293e-02
       coeff_l3(63,48)= -1.7536546290e-01
       coeff_l3(63,49)=  5.4022417068e+00
       coeff_l3(63,50)= -3.4713086486e-01
       coeff_l3(63,51)=  8.2197129726e-02
       coeff_l3(63,52)=  5.8789469302e-02
       coeff_l3(63,53)= -3.6634571850e-02
       coeff_l3(63,54)=  1.5575361252e+00
       coeff_l3(63,55)=  2.1340729296e-01
       coeff_l3(63,56)=  5.8909751475e-02
       coeff_l3(63,57)= -4.6993806958e-02
       coeff_l3(63,58)= -3.6483723670e-02
       coeff_l3(63,59)= -7.1145939827e-01
       coeff_l3(63,60)= -2.7496545315e+00
       coeff_l3(63,61)=  1.3677029312e-01
       coeff_l3(63,62)=  2.6592528820e-01
       coeff_l3(63,63)=  8.2265096903e-01
       coeff_l3(63,64)=  2.1923177410e-03
       coeff_l3(63,65)=  9.1140545905e-02
       coeff_l3(63,66)= -2.4576444179e-02
       coeff_l3(63,67)= -5.2359677851e-02
       coeff_l3(63,68)= -3.4011149406e-01
       coeff_l3(63,69)= -1.5301157534e-01
       coeff_l3(63,70)=  2.9842382669e-01
       coeff_l3(63,71)= -8.4759455919e-01
       coeff_l3(63,72)=  2.7399596572e-01
       coeff_l3(63,73)=  2.1503654122e-01
       coeff_l3(63,74)=  5.1371991634e-01
       coeff_l3(63,75)=  1.8460744619e-01
       coeff_l3(63,76)=  2.0676679611e+00
       coeff_l3(63,77)=  1.3300035000e+00
       coeff_l3(63,78)= -6.9731464610e-03
       coeff_l3(63,79)=  2.2705888748e-01
       coeff_l3(63,80)=  1.9885094166e+00
       coeff_l3(63,81)= -3.8075682521e-01
       coeff_l3(63,82)= -2.7513232827e-01
       coeff_l3(63,83)=  5.3705754690e-03
       coeff_l3(63,84)=  1.2066559494e-01
       coeff_l3(63,85)= -3.3953318000e-01
       coeff_l3(63,86)=  1.4345140755e-01
       coeff_l3(63,87)= -1.7217203975e-01
       coeff_l3(63,88)=  7.4395529926e-02
       coeff_l3(63,89)=  1.6683059931e-01
       coeff_l3(63,90)=  2.1503184736e-01
       coeff_l3(63,91)=  1.2254505157e+00
       coeff_l3(63,92)=  4.4134989381e-02
       coeff_l3(63,93)= -2.3680919409e-01
       coeff_l3(63,94)= -2.5466141105e-01
       coeff_l3(63,95)= -2.7430838346e-01
       coeff_l3(63,96)= -2.3737409711e-01
       coeff_l3(63,97)=  5.1632918417e-02
       coeff_l3(63,98)= -8.2035288215e-02
       coeff_l3(63,99)= -3.8990181684e-01
       coeff_l3(63,100)= -3.0216410756e-01
       coeff_l3(63,101)=  2.4170875549e-01
       coeff_l3(63,102)=  7.2133228183e-02
       coeff_l3(63,103)=  1.2131182104e-01
       coeff_l3(63,104)= -2.2702813148e-02
       coeff_l3(63,105)= -3.9546725154e-01
       coeff_l3(63,106)= -1.7219169438e-01
       coeff_l3(63,107)= -8.2847887278e-01
       coeff_l3(63,108)= -1.6827374697e-01
       coeff_l3(63,109)=  2.1068821847e-01
       coeff_l3(63,110)= -7.1816794574e-02
       coeff_l3(63,111)= -3.0441844463e-01
       coeff_l3(63,112)= -5.9348750114e-01
       coeff_l3(63,113)= -3.3284908533e-01
       coeff_l3(63,114)=  1.2726721764e+00
       coeff_l3(63,115)= -5.3827720881e-01
       coeff_l3(63,116)= -3.0908387899e-01
       coeff_l3(63,117)= -2.4871414900e-01
       coeff_l3(63,118)= -1.9836999476e-01
       coeff_l3(63,119)= -1.4676347375e-01
       coeff_l3(63,120)= -1.6977451742e-01
       coeff_l3(63,121)= -4.1460976005e-01
       coeff_l3(63,122)=  8.5347265005e-02
       coeff_l3(63,123)=  1.4398901165e-01
       coeff_l3(63,124)=  1.0671193153e-01
       coeff_l3(63,125)=  3.2846507430e-01
       coeff_l3(64,1)= -4.8843285441e-01
       coeff_l3(64,2)= -2.5359335542e-01
       coeff_l3(64,3)=  5.9303450584e-01
       coeff_l3(64,4)=  2.8294396400e-01
       coeff_l3(64,5)= -4.0720425546e-02
       coeff_l3(64,6)= -1.0003423691e-01
       coeff_l3(64,7)=  5.2815562487e-01
       coeff_l3(64,8)= -2.9491370916e-01
       coeff_l3(64,9)= -3.2173267007e-01
       coeff_l3(64,10)= -7.2565525770e-01
       coeff_l3(64,11)=  5.7216413319e-02
       coeff_l3(64,12)= -3.7865546346e-01
       coeff_l3(64,13)= -4.3099579215e-01
       coeff_l3(64,14)= -3.0837216973e-01
       coeff_l3(64,15)=  1.1244159937e-01
       coeff_l3(64,16)= -5.3403623402e-02
       coeff_l3(64,17)= -3.7148049474e-01
       coeff_l3(64,18)=  8.1255733967e-02
       coeff_l3(64,19)= -9.7478076816e-02
       coeff_l3(64,20)= -4.2676299810e-01
       coeff_l3(64,21)=  5.0346606970e-01
       coeff_l3(64,22)= -9.7884722054e-02
       coeff_l3(64,23)=  5.4649914091e-05
       coeff_l3(64,24)= -3.7005564570e-01
       coeff_l3(64,25)= -4.6019160748e-01
       coeff_l3(64,26)= -3.1487730145e-01
       coeff_l3(64,27)=  6.9991606474e-01
       coeff_l3(64,28)=  3.4562894702e-01
       coeff_l3(64,29)=  9.6150912344e-02
       coeff_l3(64,30)=  3.3166825771e-02
       coeff_l3(64,31)= -2.8960433602e-01
       coeff_l3(64,32)= -1.8815438449e-01
       coeff_l3(64,33)=  1.0055515170e-01
       coeff_l3(64,34)= -4.0260527283e-02
       coeff_l3(64,35)= -7.0043587685e-01
       coeff_l3(64,36)=  1.3226929307e-01
       coeff_l3(64,37)= -2.8453099728e-01
       coeff_l3(64,38)= -7.1574527025e-01
       coeff_l3(64,39)= -3.8083028048e-02
       coeff_l3(64,40)=  7.4765500613e-03
       coeff_l3(64,41)=  1.0411289930e+00
       coeff_l3(64,42)=  7.2673738003e-02
       coeff_l3(64,43)= -5.8101829141e-02
       coeff_l3(64,44)= -2.1859307587e-01
       coeff_l3(64,45)= -8.6569868028e-02
       coeff_l3(64,46)= -2.3982304335e-01
       coeff_l3(64,47)=  3.6005774140e-01
       coeff_l3(64,48)= -1.7486536503e-01
       coeff_l3(64,49)=  2.9221212864e+00
       coeff_l3(64,50)=  1.6377320886e-01
       coeff_l3(64,51)= -2.9829263687e-01
       coeff_l3(64,52)= -5.3566887975e-02
       coeff_l3(64,53)= -3.3500099182e-01
       coeff_l3(64,54)= -1.0663869232e-01
       coeff_l3(64,55)=  3.2914370298e-01
       coeff_l3(64,56)= -1.9882185757e-01
       coeff_l3(64,57)= -1.6512669623e-01
       coeff_l3(64,58)= -7.6995706558e-01
       coeff_l3(64,59)=  4.8156194389e-03
       coeff_l3(64,60)=  1.0823860168e+00
       coeff_l3(64,61)= -1.8334001303e-02
       coeff_l3(64,62)= -1.5153408051e-01
       coeff_l3(64,63)=  4.1254460812e-01
       coeff_l3(64,64)= -3.2033231854e-01
       coeff_l3(64,65)= -8.9726522565e-02
       coeff_l3(64,66)=  5.0663709641e-02
       coeff_l3(64,67)=  1.9309341908e-01
       coeff_l3(64,68)= -3.0714195967e-01
       coeff_l3(64,69)=  2.5667259097e-01
       coeff_l3(64,70)= -7.6650403440e-02
       coeff_l3(64,71)=  8.5863068700e-02
       coeff_l3(64,72)= -1.3065466285e-01
       coeff_l3(64,73)=  6.1825066805e-01
       coeff_l3(64,74)=  4.8019975424e-01
       coeff_l3(64,75)= -6.5979152918e-02
       coeff_l3(64,76)=  1.7978881598e+00
       coeff_l3(64,77)= -6.8535402417e-02
       coeff_l3(64,78)= -2.3655968904e-01
       coeff_l3(64,79)= -2.1258708835e-01
       coeff_l3(64,80)=  2.2206714153e+00
       coeff_l3(64,81)=  1.1297687143e-01
       coeff_l3(64,82)= -1.2957228720e-01
       coeff_l3(64,83)=  6.6232666373e-02
       coeff_l3(64,84)= -8.0387109518e-01
       coeff_l3(64,85)=  1.8856734037e-01
       coeff_l3(64,86)= -3.2126851380e-02
       coeff_l3(64,87)= -4.3999504298e-02
       coeff_l3(64,88)=  3.2160297036e-01
       coeff_l3(64,89)= -1.4500007033e-01
       coeff_l3(64,90)=  2.6642459631e-01
       coeff_l3(64,91)= -5.0498616695e-01
       coeff_l3(64,92)=  4.9086198211e-02
       coeff_l3(64,93)= -3.8886752725e-01
       coeff_l3(64,94)= -1.9923828542e-01
       coeff_l3(64,95)= -2.9137286544e-01
       coeff_l3(64,96)=  1.1106009036e-01
       coeff_l3(64,97)= -1.8963631988e-01
       coeff_l3(64,98)= -1.8593266606e-01
       coeff_l3(64,99)= -1.6081176698e-01
       coeff_l3(64,100)= -2.4023152888e-01
       coeff_l3(64,101)= -5.3856030107e-02
       coeff_l3(64,102)= -2.7498757839e-01
       coeff_l3(64,103)= -1.3674189150e-01
       coeff_l3(64,104)= -2.6441824436e-01
       coeff_l3(64,105)=  3.3906549215e-01
       coeff_l3(64,106)= -3.3387780190e-01
       coeff_l3(64,107)=  3.0363028869e-02
       coeff_l3(64,108)= -2.0810553432e-01
       coeff_l3(64,109)= -7.2969965637e-02
       coeff_l3(64,110)= -6.0556806624e-02
       coeff_l3(64,111)= -8.3608567715e-02
       coeff_l3(64,112)= -5.4756510258e-01
       coeff_l3(64,113)= -1.2812504172e-01
       coeff_l3(64,114)= -8.2474201918e-02
       coeff_l3(64,115)=  9.5355615020e-02
       coeff_l3(64,116)=  9.9693197012e-01
       coeff_l3(64,117)=  5.7225324214e-02
       coeff_l3(64,118)=  1.0146882534e+00
       coeff_l3(64,119)= -9.2217080295e-02
       coeff_l3(64,120)=  3.1872395426e-02
       coeff_l3(64,121)=  1.2764827907e-01
       coeff_l3(64,122)= -9.2124044895e-02
       coeff_l3(64,123)= -1.7017261684e-01
       coeff_l3(64,124)= -1.4747439325e-01
       coeff_l3(64,125)=  7.3187482357e-01
       coeff_l3(65,1)= -6.5886390209e-01
       coeff_l3(65,2)=  1.7975591123e-01
       coeff_l3(65,3)=  1.1676179618e-01
       coeff_l3(65,4)=  4.2114675045e-02
       coeff_l3(65,5)=  1.1993239634e-02
       coeff_l3(65,6)= -1.5103018284e-01
       coeff_l3(65,7)=  1.4183813334e-01
       coeff_l3(65,8)= -1.5078927577e-01
       coeff_l3(65,9)= -1.1544446647e-01
       coeff_l3(65,10)= -2.4891772866e-01
       coeff_l3(65,11)=  2.9525188729e-02
       coeff_l3(65,12)= -4.0291982889e-01
       coeff_l3(65,13)= -5.1481425762e-01
       coeff_l3(65,14)= -3.9087784290e-01
       coeff_l3(65,15)= -9.8642230034e-02
       coeff_l3(65,16)=  2.4763599038e-02
       coeff_l3(65,17)=  9.5711819828e-02
       coeff_l3(65,18)= -4.2971137166e-01
       coeff_l3(65,19)=  7.8317604959e-02
       coeff_l3(65,20)=  1.7242500186e-01
       coeff_l3(65,21)= -4.5292866230e-01
       coeff_l3(65,22)= -1.2809886597e-02
       coeff_l3(65,23)= -1.1589326710e-01
       coeff_l3(65,24)= -1.4737580717e-01
       coeff_l3(65,25)=  3.2664301991e-01
       coeff_l3(65,26)= -2.3720878363e-01
       coeff_l3(65,27)= -2.3855723441e-02
       coeff_l3(65,28)=  1.4121206477e-02
       coeff_l3(65,29)=  2.7517265081e-01
       coeff_l3(65,30)=  8.9518082142e-01
       coeff_l3(65,31)=  1.3741786778e-01
       coeff_l3(65,32)= -1.3538430631e-01
       coeff_l3(65,33)=  1.9539766014e-01
       coeff_l3(65,34)=  1.8126985431e-01
       coeff_l3(65,35)=  1.0120528936e-01
       coeff_l3(65,36)=  2.9597258568e-01
       coeff_l3(65,37)= -8.7307997048e-02
       coeff_l3(65,38)=  1.7595215142e-01
       coeff_l3(65,39)= -3.4532710910e-02
       coeff_l3(65,40)= -1.1506135762e-01
       coeff_l3(65,41)=  3.1928628683e-01
       coeff_l3(65,42)= -1.1483798921e-01
       coeff_l3(65,43)= -3.5542547703e-01
       coeff_l3(65,44)= -5.4640587419e-02
       coeff_l3(65,45)=  3.1594684720e-01
       coeff_l3(65,46)=  2.5133821368e-01
       coeff_l3(65,47)=  2.0702865720e-01
       coeff_l3(65,48)= -1.8255253136e-01
       coeff_l3(65,49)=  1.6078960896e+00
       coeff_l3(65,50)= -3.7666156888e-01
       coeff_l3(65,51)= -1.2951166928e-01
       coeff_l3(65,52)=  2.0197177306e-02
       coeff_l3(65,53)= -1.6530482471e-01
       coeff_l3(65,54)= -7.3643988371e-01
       coeff_l3(65,55)=  3.2458627224e-01
       coeff_l3(65,56)= -4.7654267401e-02
       coeff_l3(65,57)= -1.6084663570e-01
       coeff_l3(65,58)=  1.0811251402e+00
       coeff_l3(65,59)= -2.0032015443e-01
       coeff_l3(65,60)=  2.2509057522e+00
       coeff_l3(65,61)=  8.1875570118e-02
       coeff_l3(65,62)= -1.9808039069e-02
       coeff_l3(65,63)= -1.6767290235e-01
       coeff_l3(65,64)=  9.6728913486e-02
       coeff_l3(65,65)= -2.4589352310e-01
       coeff_l3(65,66)= -1.2120647728e-01
       coeff_l3(65,67)= -6.9604910910e-02
       coeff_l3(65,68)= -2.7378511429e-01
       coeff_l3(65,69)=  1.5687775612e-01
       coeff_l3(65,70)= -6.7648090422e-02
       coeff_l3(65,71)= -5.8551168442e-01
       coeff_l3(65,72)= -3.0952828005e-02
       coeff_l3(65,73)= -7.1726525202e-03
       coeff_l3(65,74)= -9.9447168410e-02
       coeff_l3(65,75)= -4.3301644921e-01
       coeff_l3(65,76)=  1.3372377157e+00
       coeff_l3(65,77)= -2.9685634375e-01
       coeff_l3(65,78)= -1.5529951453e-01
       coeff_l3(65,79)= -1.1769820005e-02
       coeff_l3(65,80)=  8.0046981573e-01
       coeff_l3(65,81)= -2.4053581001e-04
       coeff_l3(65,82)= -4.7588966787e-02
       coeff_l3(65,83)= -2.8328773379e-01
       coeff_l3(65,84)= -9.7539013624e-01
       coeff_l3(65,85)=  8.4064498544e-02
       coeff_l3(65,86)= -8.2157664001e-02
       coeff_l3(65,87)= -6.0449488461e-02
       coeff_l3(65,88)= -3.6138662696e-01
       coeff_l3(65,89)=  1.0004804134e+00
       coeff_l3(65,90)=  2.7048680186e-01
       coeff_l3(65,91)=  3.2209351659e-01
       coeff_l3(65,92)= -7.6625406742e-02
       coeff_l3(65,93)= -1.6961944057e-03
       coeff_l3(65,94)=  4.2236957699e-02
       coeff_l3(65,95)=  1.4197318815e-02
       coeff_l3(65,96)= -2.1194155514e-01
       coeff_l3(65,97)= -1.1596813798e-02
       coeff_l3(65,98)= -8.9402891695e-02
       coeff_l3(65,99)=  1.0429560393e-01
       coeff_l3(65,100)=  7.7547781169e-02
       coeff_l3(65,101)=  7.4820315838e-01
       coeff_l3(65,102)= -1.0207022727e-01
       coeff_l3(65,103)= -4.4105190784e-02
       coeff_l3(65,104)= -2.3016594350e-02
       coeff_l3(65,105)= -4.6881625056e-01
       coeff_l3(65,106)= -2.3957952857e-01
       coeff_l3(65,107)= -8.1824994087e-01
       coeff_l3(65,108)=  1.5082496405e-01
       coeff_l3(65,109)= -1.1740081757e-01
       coeff_l3(65,110)= -4.0179049969e-01
       coeff_l3(65,111)= -2.0245058835e-01
       coeff_l3(65,112)= -1.9157071412e-01
       coeff_l3(65,113)=  2.8313558549e-02
       coeff_l3(65,114)= -4.7647675872e-01
       coeff_l3(65,115)= -4.7353029251e-01
       coeff_l3(65,116)= -1.2054156065e+00
       coeff_l3(65,117)=  7.6243773103e-02
       coeff_l3(65,118)=  6.7663758993e-01
       coeff_l3(65,119)= -1.6930520535e-01
       coeff_l3(65,120)= -1.2149509788e-01
       coeff_l3(65,121)= -3.3131363988e-01
       coeff_l3(65,122)=  2.6090556756e-02
       coeff_l3(65,123)= -2.0801221952e-02
       coeff_l3(65,124)= -4.5381516218e-02
       coeff_l3(65,125)=  2.1554549038e-01
       coeff_l3(66,1)= -9.2454642057e-01
       coeff_l3(66,2)= -6.4509707689e-01
       coeff_l3(66,3)=  1.9317601621e-01
       coeff_l3(66,4)= -3.0089402199e-01
       coeff_l3(66,5)= -2.7577355504e-01
       coeff_l3(66,6)= -3.8338038325e-01
       coeff_l3(66,7)= -2.1661574841e+00
       coeff_l3(66,8)=  1.4999690652e-01
       coeff_l3(66,9)= -1.3627018034e-01
       coeff_l3(66,10)= -5.6459271908e-01
       coeff_l3(66,11)= -9.1051533818e-02
       coeff_l3(66,12)= -2.8003489971e-01
       coeff_l3(66,13)=  7.5619781017e-01
       coeff_l3(66,14)= -3.8625475764e-01
       coeff_l3(66,15)= -5.2771490812e-01
       coeff_l3(66,16)= -3.5619190335e-01
       coeff_l3(66,17)= -3.9167219400e-01
       coeff_l3(66,18)= -1.1027203798e+00
       coeff_l3(66,19)=  2.5880023837e-01
       coeff_l3(66,20)= -1.7177101970e-01
       coeff_l3(66,21)=  8.2047587633e-01
       coeff_l3(66,22)= -3.2227611542e-01
       coeff_l3(66,23)= -2.4255694449e-01
       coeff_l3(66,24)= -1.4491420984e-01
       coeff_l3(66,25)= -5.5068310350e-02
       coeff_l3(66,26)= -2.1657621861e-01
       coeff_l3(66,27)= -2.4612057209e+00
       coeff_l3(66,28)= -2.1108422279e+00
       coeff_l3(66,29)=  1.1488405615e-01
       coeff_l3(66,30)= -2.0304780006e+00
       coeff_l3(66,31)= -4.4068998098e-01
       coeff_l3(66,32)= -2.4518068135e-01
       coeff_l3(66,33)= -6.2896043062e-01
       coeff_l3(66,34)= -6.7092895508e-02
       coeff_l3(66,35)= -3.8937557489e-02
       coeff_l3(66,36)=  2.9134135693e-02
       coeff_l3(66,37)= -3.9216521382e-01
       coeff_l3(66,38)=  4.4469857216e-01
       coeff_l3(66,39)= -4.6213626862e-01
       coeff_l3(66,40)= -7.8609377146e-01
       coeff_l3(66,41)=  1.0252152681e+00
       coeff_l3(66,42)= -4.4249364734e-01
       coeff_l3(66,43)= -1.9125211239e-01
       coeff_l3(66,44)= -5.3459417820e-01
       coeff_l3(66,45)= -1.1078667641e+00
       coeff_l3(66,46)= -9.7634917498e-01
       coeff_l3(66,47)=  5.7077032328e-01
       coeff_l3(66,48)= -4.9603164196e-01
       coeff_l3(66,49)= -2.9684491158e+00
       coeff_l3(66,50)= -8.1498146057e-02
       coeff_l3(66,51)= -9.5188927650e-01
       coeff_l3(66,52)= -3.0289086699e-01
       coeff_l3(66,53)= -6.8878650665e-01
       coeff_l3(66,54)= -7.7203169465e-02
       coeff_l3(66,55)= -9.4502788782e-01
       coeff_l3(66,56)= -2.6299369335e-01
       coeff_l3(66,57)= -3.7057006359e-01
       coeff_l3(66,58)= -8.3486133814e-01
       coeff_l3(66,59)= -2.8280308470e-02
       coeff_l3(66,60)= -1.2432963848e+00
       coeff_l3(66,61)= -3.5190179944e-01
       coeff_l3(66,62)= -3.1763520837e-01
       coeff_l3(66,63)= -3.3968773484e-01
       coeff_l3(66,64)= -6.9502443075e-01
       coeff_l3(66,65)= -2.9342794418e-01
       coeff_l3(66,66)= -2.1522158384e-01
       coeff_l3(66,67)=  1.4974451065e+00
       coeff_l3(66,68)=  5.1343537867e-02
       coeff_l3(66,69)=  1.4423206449e-02
       coeff_l3(66,70)= -7.8934061527e-01
       coeff_l3(66,71)= -1.3160533905e+00
       coeff_l3(66,72)= -5.0736099482e-01
       coeff_l3(66,73)=  3.9508414268e-01
       coeff_l3(66,74)=  1.1192400455e+00
       coeff_l3(66,75)= -2.7651003003e-01
       coeff_l3(66,76)= -2.3184390068e+00
       coeff_l3(66,77)= -6.0676068068e-02
       coeff_l3(66,78)= -3.8060954213e-01
       coeff_l3(66,79)= -2.0587893203e-02
       coeff_l3(66,80)= -1.3258743286e+00
       coeff_l3(66,81)=  2.5637415051e-01
       coeff_l3(66,82)=  3.3561915159e-01
       coeff_l3(66,83)=  2.0457842946e-01
       coeff_l3(66,84)=  6.2130814791e-01
       coeff_l3(66,85)= -1.9857977331e-01
       coeff_l3(66,86)= -1.8721023202e-01
       coeff_l3(66,87)= -3.2050639391e-01
       coeff_l3(66,88)=  3.2349309325e-01
       coeff_l3(66,89)= -1.0442939997e+00
       coeff_l3(66,90)= -5.6083059311e-01
       coeff_l3(66,91)= -2.1041415632e-01
       coeff_l3(66,92)= -5.1059699059e-01
       coeff_l3(66,93)= -2.4012923241e-01
       coeff_l3(66,94)= -4.0785339475e-01
       coeff_l3(66,95)= -3.1439414620e-01
       coeff_l3(66,96)=  1.7905154824e-01
       coeff_l3(66,97)= -3.6349552870e-01
       coeff_l3(66,98)= -2.7911406755e-01
       coeff_l3(66,99)=  6.4094895124e-01
       coeff_l3(66,100)= -4.8582053185e-01
       coeff_l3(66,101)= -3.0566492677e-01
       coeff_l3(66,102)= -4.7446224093e-01
       coeff_l3(66,103)= -6.9598436356e-02
       coeff_l3(66,104)= -2.0688717067e-01
       coeff_l3(66,105)=  5.9080648422e-01
       coeff_l3(66,106)= -1.8785326183e-01
       coeff_l3(66,107)=  9.2698134482e-02
       coeff_l3(66,108)= -5.0406265259e-01
       coeff_l3(66,109)= -1.8831665814e-01
       coeff_l3(66,110)= -6.0381844640e-02
       coeff_l3(66,111)= -2.7397495508e-01
       coeff_l3(66,112)=  3.4637165070e-01
       coeff_l3(66,113)= -8.9077550173e-01
       coeff_l3(66,114)= -4.9841398001e-01
       coeff_l3(66,115)=  2.0164130628e-01
       coeff_l3(66,116)=  9.1496807337e-01
       coeff_l3(66,117)= -6.5672540665e-01
       coeff_l3(66,118)=  7.1867120266e-01
       coeff_l3(66,119)= -2.1714241803e-01
       coeff_l3(66,120)= -1.5978115797e-01
       coeff_l3(66,121)= -3.4724500030e-02
       coeff_l3(66,122)= -3.9585840702e-01
       coeff_l3(66,123)= -2.1696896851e-01
       coeff_l3(66,124)= -3.0231934786e-01
       coeff_l3(66,125)=  7.4286080897e-02
       coeff_l3(67,1)=  2.4030476809e-01
       coeff_l3(67,2)= -2.6177039742e-01
       coeff_l3(67,3)= -4.4504037499e-01
       coeff_l3(67,4)= -3.1037151814e-01
       coeff_l3(67,5)= -1.7425608635e-01
       coeff_l3(67,6)= -1.0407003760e-01
       coeff_l3(67,7)=  1.5734612942e-01
       coeff_l3(67,8)= -2.8961312771e-01
       coeff_l3(67,9)= -1.9305510819e-01
       coeff_l3(67,10)= -4.3407285213e-01
       coeff_l3(67,11)=  2.3083940148e-01
       coeff_l3(67,12)=  1.0254890919e+00
       coeff_l3(67,13)=  5.7721990347e-01
       coeff_l3(67,14)= -1.3747401536e-01
       coeff_l3(67,15)= -1.8784986436e-01
       coeff_l3(67,16)= -2.3978403211e-01
       coeff_l3(67,17)=  4.4760710001e-01
       coeff_l3(67,18)=  3.4928175807e-01
       coeff_l3(67,19)= -5.2135597914e-02
       coeff_l3(67,20)= -4.8702391982e-01
       coeff_l3(67,21)= -1.4239607751e-01
       coeff_l3(67,22)=  5.3610201925e-02
       coeff_l3(67,23)= -1.1213430762e-01
       coeff_l3(67,24)= -3.6965051293e-01
       coeff_l3(67,25)=  1.9707535207e-01
       coeff_l3(67,26)=  2.0395734906e-01
       coeff_l3(67,27)=  8.5371184349e-01
       coeff_l3(67,28)= -2.6534205675e-01
       coeff_l3(67,29)=  3.8234019279e-01
       coeff_l3(67,30)= -4.5479410887e-01
       coeff_l3(67,31)= -3.0448859930e-01
       coeff_l3(67,32)= -8.6171008646e-02
       coeff_l3(67,33)= -6.7516422272e-01
       coeff_l3(67,34)= -5.8061089367e-02
       coeff_l3(67,35)= -3.3891844749e-01
       coeff_l3(67,36)= -5.4348643869e-02
       coeff_l3(67,37)=  3.7682347465e-03
       coeff_l3(67,38)=  9.8690092564e-01
       coeff_l3(67,39)= -3.9746603370e-01
       coeff_l3(67,40)= -1.1603350192e-01
       coeff_l3(67,41)=  3.0774840713e-01
       coeff_l3(67,42)= -2.9143351316e-01
       coeff_l3(67,43)=  2.2016055882e-01
       coeff_l3(67,44)= -3.0358448625e-01
       coeff_l3(67,45)= -1.9139201939e-01
       coeff_l3(67,46)=  1.7383921146e-01
       coeff_l3(67,47)= -2.2170920670e-01
       coeff_l3(67,48)=  1.9546326250e-02
       coeff_l3(67,49)= -4.6915187836e+00
       coeff_l3(67,50)= -8.2346454263e-02
       coeff_l3(67,51)=  6.9815254211e-01
       coeff_l3(67,52)= -1.8183072098e-03
       coeff_l3(67,53)= -3.2960513234e-01
       coeff_l3(67,54)= -2.0882384479e-01
       coeff_l3(67,55)= -6.6823792458e-01
       coeff_l3(67,56)= -3.3522534370e-01
       coeff_l3(67,57)= -9.2750564218e-02
       coeff_l3(67,58)=  3.2089078426e-01
       coeff_l3(67,59)= -2.1612013876e-01
       coeff_l3(67,60)= -4.4608225822e+00
       coeff_l3(67,61)= -1.9041949511e-01
       coeff_l3(67,62)= -9.0870194137e-02
       coeff_l3(67,63)= -5.2353298664e-01
       coeff_l3(67,64)= -2.4099487066e-01
       coeff_l3(67,65)= -1.7898246646e-02
       coeff_l3(67,66)= -4.1697776318e-01
       coeff_l3(67,67)=  3.5285443068e-01
       coeff_l3(67,68)= -7.1489341557e-02
       coeff_l3(67,69)= -2.0473471284e-01
       coeff_l3(67,70)= -3.4599441290e-01
       coeff_l3(67,71)=  1.6376942396e-01
       coeff_l3(67,72)= -1.0009783506e-01
       coeff_l3(67,73)=  3.7533101439e-01
       coeff_l3(67,74)=  9.8961286247e-02
       coeff_l3(67,75)= -4.5949932933e-01
       coeff_l3(67,76)= -3.3178405762e+00
       coeff_l3(67,77)=  2.5216642022e-01
       coeff_l3(67,78)=  4.0175370872e-02
       coeff_l3(67,79)= -4.3970513344e-01
       coeff_l3(67,80)= -1.9566980600e+00
       coeff_l3(67,81)=  3.2012042403e-01
       coeff_l3(67,82)=  6.0678124428e-02
       coeff_l3(67,83)=  2.7385950089e-01
       coeff_l3(67,84)= -6.8641334772e-02
       coeff_l3(67,85)=  3.8202834129e-01
       coeff_l3(67,86)= -1.7267860472e-02
       coeff_l3(67,87)= -2.9223456979e-01
       coeff_l3(67,88)= -1.7205971479e-01
       coeff_l3(67,89)=  8.0815672874e-02
       coeff_l3(67,90)= -6.8727743626e-01
       coeff_l3(67,91)= -5.9414094687e-01
       coeff_l3(67,92)= -3.3310163766e-02
       coeff_l3(67,93)= -3.1441602111e-01
       coeff_l3(67,94)=  1.0806301236e-01
       coeff_l3(67,95)= -1.1729265004e-01
       coeff_l3(67,96)=  4.4485440850e-01
       coeff_l3(67,97)=  4.5614998788e-02
       coeff_l3(67,98)= -3.7763707340e-02
       coeff_l3(67,99)= -3.7109038234e-01
       coeff_l3(67,100)= -7.9421103001e-02
       coeff_l3(67,101)=  9.1385081410e-02
       coeff_l3(67,102)=  3.5948753357e-01
       coeff_l3(67,103)=  2.5823665783e-02
       coeff_l3(67,104)= -1.7252692580e-01
       coeff_l3(67,105)=  2.1634164453e-01
       coeff_l3(67,106)=  2.7512541413e-01
       coeff_l3(67,107)= -5.3997701406e-01
       coeff_l3(67,108)=  1.3261927664e-01
       coeff_l3(67,109)= -8.0117307603e-02
       coeff_l3(67,110)= -2.0731465518e-01
       coeff_l3(67,111)= -8.1354435533e-03
       coeff_l3(67,112)=  7.2745651007e-01
       coeff_l3(67,113)=  3.1211474538e-01
       coeff_l3(67,114)=  2.0770427585e-01
       coeff_l3(67,115)=  1.2121953815e-01
       coeff_l3(67,116)=  1.2768660784e+00
       coeff_l3(67,117)=  2.0932167768e-01
       coeff_l3(67,118)=  7.9476958513e-01
       coeff_l3(67,119)= -7.1548931301e-02
       coeff_l3(67,120)=  1.9806900620e-01
       coeff_l3(67,121)= -1.8077050149e-01
       coeff_l3(67,122)= -1.9657291472e-01
       coeff_l3(67,123)=  8.1078983843e-02
       coeff_l3(67,124)=  7.8011346050e-03
       coeff_l3(67,125)= -4.0703476407e-03
       coeff_l3(68,1)=  2.8331640363e-01
       coeff_l3(68,2)= -4.2199537158e-02
       coeff_l3(68,3)=  5.6941613555e-02
       coeff_l3(68,4)= -1.3977478258e-02
       coeff_l3(68,5)= -1.2839545310e-01
       coeff_l3(68,6)= -1.1195412278e-01
       coeff_l3(68,7)= -7.0515628904e-03
       coeff_l3(68,8)= -1.8814320862e-01
       coeff_l3(68,9)= -1.0791237466e-02
       coeff_l3(68,10)= -2.6069980860e-01
       coeff_l3(68,11)= -2.6731005311e-01
       coeff_l3(68,12)=  1.2536643445e-01
       coeff_l3(68,13)=  1.0754605383e-01
       coeff_l3(68,14)=  1.7872133851e-01
       coeff_l3(68,15)= -1.5049201250e-01
       coeff_l3(68,16)= -7.4713110924e-02
       coeff_l3(68,17)=  1.5409834683e-01
       coeff_l3(68,18)= -7.9129617661e-03
       coeff_l3(68,19)=  4.5700166374e-02
       coeff_l3(68,20)= -9.8293438554e-02
       coeff_l3(68,21)=  1.0983716697e-01
       coeff_l3(68,22)= -2.3094652221e-02
       coeff_l3(68,23)= -2.3922710121e-01
       coeff_l3(68,24)= -2.0222193003e-01
       coeff_l3(68,25)= -3.5016825795e-01
       coeff_l3(68,26)= -1.2721009552e-01
       coeff_l3(68,27)=  4.1511837393e-02
       coeff_l3(68,28)=  2.5186674669e-02
       coeff_l3(68,29)= -5.4898613691e-01
       coeff_l3(68,30)= -1.9903656840e-01
       coeff_l3(68,31)= -1.6266374290e-01
       coeff_l3(68,32)= -1.1287859827e-01
       coeff_l3(68,33)= -2.8533756733e-01
       coeff_l3(68,34)=  1.8227756023e-01
       coeff_l3(68,35)= -8.4179455042e-01
       coeff_l3(68,36)=  1.7231921852e-01
       coeff_l3(68,37)= -3.9388872683e-02
       coeff_l3(68,38)=  6.2305396795e-01
       coeff_l3(68,39)=  1.1839146167e-01
       coeff_l3(68,40)= -8.9665621519e-02
       coeff_l3(68,41)=  2.4906602502e-01
       coeff_l3(68,42)= -1.7890492082e-01
       coeff_l3(68,43)= -1.1929375678e-01
       coeff_l3(68,44)= -3.4169748425e-01
       coeff_l3(68,45)= -1.9682943821e-01
       coeff_l3(68,46)=  7.3527358472e-02
       coeff_l3(68,47)= -3.4215712547e-01
       coeff_l3(68,48)= -2.3777772486e-01
       coeff_l3(68,49)= -1.4277702570e+00
       coeff_l3(68,50)=  9.5949769020e-02
       coeff_l3(68,51)=  1.4689886570e-01
       coeff_l3(68,52)= -8.6002700031e-02
       coeff_l3(68,53)=  8.0745734274e-02
       coeff_l3(68,54)=  5.9490308166e-02
       coeff_l3(68,55)= -4.2948895693e-01
       coeff_l3(68,56)= -4.1653096676e-02
       coeff_l3(68,57)= -4.7502472997e-02
       coeff_l3(68,58)= -2.1556280553e-01
       coeff_l3(68,59)=  7.7150273137e-03
       coeff_l3(68,60)= -9.4332861900e-01
       coeff_l3(68,61)= -7.3025710881e-02
       coeff_l3(68,62)= -2.4134498835e-01
       coeff_l3(68,63)= -2.0440446213e-02
       coeff_l3(68,64)= -2.1476772428e-01
       coeff_l3(68,65)= -1.0688460618e-01
       coeff_l3(68,66)= -1.2472092360e-01
       coeff_l3(68,67)= -2.2658088803e-01
       coeff_l3(68,68)=  2.5899568200e-01
       coeff_l3(68,69)= -1.3039501011e-01
       coeff_l3(68,70)= -3.0150833726e-01
       coeff_l3(68,71)= -2.0010069013e-01
       coeff_l3(68,72)= -8.7044395506e-02
       coeff_l3(68,73)=  2.1762159467e-01
       coeff_l3(68,74)= -1.0140740871e-01
       coeff_l3(68,75)= -3.2172521949e-01
       coeff_l3(68,76)= -1.1420595646e+00
       coeff_l3(68,77)=  2.6298296452e-01
       coeff_l3(68,78)= -4.2467091233e-02
       coeff_l3(68,79)= -2.4478393793e-01
       coeff_l3(68,80)= -6.4981514215e-01
       coeff_l3(68,81)= -1.4695340395e-01
       coeff_l3(68,82)= -1.0650539398e-01
       coeff_l3(68,83)=  2.9856088758e-01
       coeff_l3(68,84)= -1.3564677536e-01
       coeff_l3(68,85)= -1.0901046544e-01
       coeff_l3(68,86)=  1.2984865904e-01
       coeff_l3(68,87)= -1.4886935055e-01
       coeff_l3(68,88)=  1.2569580972e-01
       coeff_l3(68,89)= -4.5353230834e-01
       coeff_l3(68,90)= -5.2450990677e-01
       coeff_l3(68,91)= -5.5258929729e-01
       coeff_l3(68,92)= -1.5183205903e-01
       coeff_l3(68,93)=  5.9620428085e-01
       coeff_l3(68,94)=  1.3461604714e-01
       coeff_l3(68,95)= -1.0439942777e-01
       coeff_l3(68,96)= -1.3381730020e-01
       coeff_l3(68,97)= -4.8837456852e-02
       coeff_l3(68,98)=  3.3528646827e-01
       coeff_l3(68,99)= -1.9023230672e-01
       coeff_l3(68,100)= -1.1473800987e-01
       coeff_l3(68,101)= -1.8513912335e-02
       coeff_l3(68,102)= -2.5730326772e-01
       coeff_l3(68,103)=  1.9651234150e-02
       coeff_l3(68,104)= -2.3350718617e-01
       coeff_l3(68,105)= -2.5502943993e-01
       coeff_l3(68,106)= -4.9011114985e-02
       coeff_l3(68,107)= -3.8006916642e-02
       coeff_l3(68,108)=  1.3657633960e-01
       coeff_l3(68,109)= -8.7385684252e-02
       coeff_l3(68,110)= -1.3370585442e-01
       coeff_l3(68,111)=  2.6802551001e-02
       coeff_l3(68,112)= -2.0292605460e-01
       coeff_l3(68,113)= -1.6794361174e-01
       coeff_l3(68,114)= -8.5963480175e-02
       coeff_l3(68,115)=  9.0037984774e-03
       coeff_l3(68,116)=  2.4661949277e-01
       coeff_l3(68,117)= -2.6122099161e-01
       coeff_l3(68,118)= -1.0077495575e+00
       coeff_l3(68,119)=  1.8082327442e-03
       coeff_l3(68,120)=  1.3522686437e-02
       coeff_l3(68,121)=  2.1789094433e-02
       coeff_l3(68,122)=  6.4797983505e-03
       coeff_l3(68,123)= -1.0160659999e-01
       coeff_l3(68,124)=  4.1039668024e-02
       coeff_l3(68,125)=  1.1110488325e-02
       coeff_l3(69,1)= -2.1389436722e-01
       coeff_l3(69,2)= -6.0782648623e-02
       coeff_l3(69,3)=  2.9049587250e-01
       coeff_l3(69,4)= -3.8101814687e-02
       coeff_l3(69,5)= -9.0838104486e-02
       coeff_l3(69,6)=  1.0795761645e-01
       coeff_l3(69,7)= -1.8737347424e-01
       coeff_l3(69,8)= -1.4799360931e-01
       coeff_l3(69,9)=  1.8094371259e-01
       coeff_l3(69,10)= -1.0226858407e-01
       coeff_l3(69,11)= -3.3286556602e-02
       coeff_l3(69,12)= -2.3762369156e-01
       coeff_l3(69,13)= -2.5887954235e-01
       coeff_l3(69,14)= -8.2950912416e-02
       coeff_l3(69,15)=  7.5849704444e-02
       coeff_l3(69,16)= -1.0133786500e-01
       coeff_l3(69,17)= -6.1116483063e-02
       coeff_l3(69,18)= -1.6952386498e-01
       coeff_l3(69,19)= -8.7221413851e-02
       coeff_l3(69,20)= -2.9680119827e-02
       coeff_l3(69,21)= -9.3025147915e-02
       coeff_l3(69,22)=  1.4483149350e-01
       coeff_l3(69,23)= -1.0374979675e-01
       coeff_l3(69,24)=  1.0912582278e-02
       coeff_l3(69,25)=  3.5033039749e-02
       coeff_l3(69,26)= -6.0827564448e-02
       coeff_l3(69,27)= -3.0739828944e-01
       coeff_l3(69,28)= -2.8367987275e-01
       coeff_l3(69,29)= -1.9087189436e-01
       coeff_l3(69,30)=  1.0909561813e-01
       coeff_l3(69,31)= -3.3472936600e-02
       coeff_l3(69,32)=  1.1213642359e-01
       coeff_l3(69,33)= -1.7856138945e-01
       coeff_l3(69,34)= -1.3024108112e-01
       coeff_l3(69,35)= -3.1222838163e-01
       coeff_l3(69,36)= -7.0257648826e-02
       coeff_l3(69,37)= -5.2917502820e-02
       coeff_l3(69,38)= -7.1790151298e-02
       coeff_l3(69,39)=  3.7426248193e-02
       coeff_l3(69,40)= -1.2926527858e-01
       coeff_l3(69,41)= -8.6070336401e-02
       coeff_l3(69,42)= -1.4395435154e-01
       coeff_l3(69,43)=  1.2465840578e-01
       coeff_l3(69,44)= -1.3100090623e-01
       coeff_l3(69,45)=  1.6496080160e-01
       coeff_l3(69,46)= -4.8527572304e-02
       coeff_l3(69,47)=  1.9644379616e-01
       coeff_l3(69,48)=  2.9619222041e-03
       coeff_l3(69,49)= -8.4315919876e-01
       coeff_l3(69,50)= -3.8771906495e-01
       coeff_l3(69,51)=  6.0310482979e-02
       coeff_l3(69,52)=  1.2885235250e-01
       coeff_l3(69,53)=  8.9997472242e-03
       coeff_l3(69,54)=  1.3595014811e-01
       coeff_l3(69,55)=  2.1611407399e-02
       coeff_l3(69,56)= -1.4445987344e-01
       coeff_l3(69,57)= -1.2541046739e-01
       coeff_l3(69,58)=  1.5132980049e-01
       coeff_l3(69,59)= -6.5243758261e-02
       coeff_l3(69,60)=  2.6256931946e-02
       coeff_l3(69,61)=  1.1891873181e-01
       coeff_l3(69,62)=  3.5917632282e-02
       coeff_l3(69,63)= -1.2209428102e-01
       coeff_l3(69,64)=  1.4235320687e-01
       coeff_l3(69,65)= -5.6025288999e-02
       coeff_l3(69,66)= -1.2058246881e-01
       coeff_l3(69,67)= -1.2304271758e-01
       coeff_l3(69,68)= -9.2831879854e-02
       coeff_l3(69,69)= -3.2669833302e-01
       coeff_l3(69,70)= -1.0412123054e-01
       coeff_l3(69,71)= -1.9307874143e-01
       coeff_l3(69,72)=  3.1376986299e-03
       coeff_l3(69,73)=  5.4399748333e-03
       coeff_l3(69,74)= -2.1845389903e-01
       coeff_l3(69,75)=  3.8350854069e-02
       coeff_l3(69,76)= -5.7325816154e-01
       coeff_l3(69,77)= -2.5956833363e-01
       coeff_l3(69,78)=  4.5393262058e-02
       coeff_l3(69,79)= -5.9442467988e-02
       coeff_l3(69,80)= -1.7480182648e-01
       coeff_l3(69,81)= -3.2131493092e-02
       coeff_l3(69,82)=  1.9450498745e-02
       coeff_l3(69,83)=  6.6109165549e-02
       coeff_l3(69,84)= -9.2633850873e-03
       coeff_l3(69,85)= -1.1993799359e-01
       coeff_l3(69,86)= -9.4817794859e-02
       coeff_l3(69,87)= -3.8749821484e-02
       coeff_l3(69,88)= -1.5804874897e-01
       coeff_l3(69,89)= -5.4554164410e-02
       coeff_l3(69,90)= -1.9109201431e-01
       coeff_l3(69,91)= -4.8759934306e-01
       coeff_l3(69,92)=  1.4033721760e-02
       coeff_l3(69,93)= -1.3166425750e-02
       coeff_l3(69,94)=  1.5010337532e-01
       coeff_l3(69,95)=  9.1217942536e-02
       coeff_l3(69,96)= -7.3311045766e-02
       coeff_l3(69,97)=  9.3733765185e-02
       coeff_l3(69,98)= -4.6969866753e-01
       coeff_l3(69,99)= -6.0693144798e-01
       coeff_l3(69,100)= -7.1519829333e-02
       coeff_l3(69,101)=  6.1353929341e-02
       coeff_l3(69,102)= -8.4285803139e-02
       coeff_l3(69,103)= -5.7400785387e-02
       coeff_l3(69,104)=  1.5691348910e-01
       coeff_l3(69,105)= -2.5580894202e-02
       coeff_l3(69,106)= -9.0523809195e-02
       coeff_l3(69,107)= -2.2829094529e-01
       coeff_l3(69,108)=  7.7376291156e-02
       coeff_l3(69,109)=  5.3257811815e-02
       coeff_l3(69,110)= -1.4647050202e-01
       coeff_l3(69,111)=  6.8331725895e-02
       coeff_l3(69,112)= -4.7847098112e-01
       coeff_l3(69,113)= -1.9086992741e-01
       coeff_l3(69,114)= -1.0782813653e-02
       coeff_l3(69,115)= -1.2259533256e-01
       coeff_l3(69,116)= -2.4846206605e-01
       coeff_l3(69,117)= -1.2225334346e-01
       coeff_l3(69,118)= -4.0346596390e-02
       coeff_l3(69,119)= -1.8453396857e-02
       coeff_l3(69,120)= -1.7077520490e-01
       coeff_l3(69,121)= -2.2174330056e-01
       coeff_l3(69,122)=  7.8220024705e-02
       coeff_l3(69,123)=  1.1300247163e-01
       coeff_l3(69,124)=  1.5063370764e-01
       coeff_l3(69,125)= -1.7495378852e-01
       coeff_l3(70,1)= -1.3768187165e-01
       coeff_l3(70,2)= -3.1076642871e-01
       coeff_l3(70,3)=  9.8087631166e-02
       coeff_l3(70,4)=  1.0754387826e-01
       coeff_l3(70,5)=  1.6847544909e-01
       coeff_l3(70,6)= -1.7458133399e-01
       coeff_l3(70,7)=  1.4968264103e-01
       coeff_l3(70,8)= -1.0752869397e-01
       coeff_l3(70,9)= -7.6675690711e-02
       coeff_l3(70,10)=  4.4979225844e-02
       coeff_l3(70,11)= -6.5872363746e-02
       coeff_l3(70,12)= -2.7308031917e-01
       coeff_l3(70,13)=  5.4814726114e-02
       coeff_l3(70,14)= -2.4368356168e-01
       coeff_l3(70,15)= -1.1717583239e-01
       coeff_l3(70,16)=  2.1805125475e-01
       coeff_l3(70,17)= -3.1446512789e-02
       coeff_l3(70,18)= -1.3740383089e-01
       coeff_l3(70,19)= -1.4686533809e-01
       coeff_l3(70,20)= -2.8577542305e-01
       coeff_l3(70,21)=  2.1549502015e-01
       coeff_l3(70,22)= -4.2886693031e-02
       coeff_l3(70,23)=  8.8888145983e-02
       coeff_l3(70,24)= -3.4346869588e-01
       coeff_l3(70,25)= -2.7206772566e-01
       coeff_l3(70,26)= -1.3186134398e-01
       coeff_l3(70,27)=  2.4569448829e-01
       coeff_l3(70,28)= -2.0583719015e-01
       coeff_l3(70,29)= -4.1663467884e-01
       coeff_l3(70,30)= -1.6322842240e-01
       coeff_l3(70,31)= -1.6508188844e-01
       coeff_l3(70,32)= -1.5364995599e-01
       coeff_l3(70,33)= -6.8210527301e-02
       coeff_l3(70,34)= -5.0526343286e-02
       coeff_l3(70,35)= -2.0931613445e-01
       coeff_l3(70,36)=  6.0931565240e-03
       coeff_l3(70,37)=  3.2417476177e-02
       coeff_l3(70,38)= -3.9503496885e-01
       coeff_l3(70,39)= -2.7887454629e-01
       coeff_l3(70,40)=  1.6862285137e-01
       coeff_l3(70,41)=  9.5634693280e-03
       coeff_l3(70,42)=  7.3422607966e-03
       coeff_l3(70,43)= -9.0647824109e-02
       coeff_l3(70,44)=  5.3871057928e-02
       coeff_l3(70,45)=  4.9809336662e-02
       coeff_l3(70,46)= -2.6781097054e-01
       coeff_l3(70,47)= -1.3528372347e-01
       coeff_l3(70,48)= -1.3474938273e-01
       coeff_l3(70,49)=  1.3249203563e-01
       coeff_l3(70,50)= -1.7159056664e-01
       coeff_l3(70,51)= -3.7784297019e-02
       coeff_l3(70,52)=  2.1325152367e-02
       coeff_l3(70,53)= -1.3166601770e-02
       coeff_l3(70,54)= -8.9471556246e-02
       coeff_l3(70,55)= -2.3461602628e-02
       coeff_l3(70,56)= -7.1465313435e-02
       coeff_l3(70,57)=  1.6865417361e-01
       coeff_l3(70,58)= -1.7893878743e-02
       coeff_l3(70,59)= -9.4221986830e-02
       coeff_l3(70,60)= -9.5331341028e-02
       coeff_l3(70,61)=  1.3183219731e-01
       coeff_l3(70,62)= -1.7415329814e-01
       coeff_l3(70,63)= -7.5746729970e-02
       coeff_l3(70,64)=  3.9737120271e-02
       coeff_l3(70,65)= -2.1762675047e-01
       coeff_l3(70,66)= -7.8645095229e-02
       coeff_l3(70,67)=  1.8611086905e-01
       coeff_l3(70,68)= -1.8839465082e-01
       coeff_l3(70,69)= -2.5152537227e-01
       coeff_l3(70,70)= -1.9168876112e-01
       coeff_l3(70,71)= -3.8889786601e-01
       coeff_l3(70,72)=  2.1371439099e-02
       coeff_l3(70,73)=  5.3021930158e-02
       coeff_l3(70,74)=  3.8574013859e-02
       coeff_l3(70,75)= -1.2697286904e-01
       coeff_l3(70,76)=  1.0078976303e-01
       coeff_l3(70,77)= -1.0067109764e-01
       coeff_l3(70,78)=  1.6798919067e-02
       coeff_l3(70,79)=  6.9209869253e-04
       coeff_l3(70,80)= -5.9108693153e-02
       coeff_l3(70,81)=  1.4269721508e-01
       coeff_l3(70,82)=  2.3345001042e-01
       coeff_l3(70,83)= -1.6679182649e-02
       coeff_l3(70,84)=  2.0665859804e-02
       coeff_l3(70,85)=  1.4938680828e-01
       coeff_l3(70,86)= -1.7134661973e-01
       coeff_l3(70,87)=  2.5222299155e-03
       coeff_l3(70,88)=  6.5843567252e-02
       coeff_l3(70,89)=  2.7266182005e-02
       coeff_l3(70,90)= -1.5790522099e-01
       coeff_l3(70,91)=  8.1043854356e-02
       coeff_l3(70,92)=  4.4036280364e-02
       coeff_l3(70,93)= -1.3892291114e-02
       coeff_l3(70,94)= -1.2780559063e-01
       coeff_l3(70,95)= -7.0282898843e-02
       coeff_l3(70,96)=  1.1529590935e-02
       coeff_l3(70,97)= -5.2546624094e-02
       coeff_l3(70,98)=  1.4814823866e-01
       coeff_l3(70,99)= -3.1516757607e-01
       coeff_l3(70,100)= -1.7448340356e-01
       coeff_l3(70,101)=  4.4877503067e-02
       coeff_l3(70,102)= -2.0675566047e-03
       coeff_l3(70,103)=  2.2046290338e-01
       coeff_l3(70,104)= -6.2379883602e-03
       coeff_l3(70,105)= -6.2103424221e-02
       coeff_l3(70,106)=  1.0964079201e-01
       coeff_l3(70,107)= -3.0142354965e-01
       coeff_l3(70,108)= -1.4689269662e-01
       coeff_l3(70,109)= -2.3416658863e-02
       coeff_l3(70,110)= -7.6159485616e-03
       coeff_l3(70,111)=  8.8935624808e-03
       coeff_l3(70,112)= -1.4674375951e-01
       coeff_l3(70,113)= -1.6336198151e-01
       coeff_l3(70,114)=  6.4015887678e-02
       coeff_l3(70,115)= -2.8569710255e-01
       coeff_l3(70,116)=  1.2637895346e-01
       coeff_l3(70,117)= -1.2269455194e-01
       coeff_l3(70,118)=  5.9325069189e-02
       coeff_l3(70,119)=  5.0084225833e-02
       coeff_l3(70,120)= -5.6825142354e-02
       coeff_l3(70,121)= -1.7882531881e-01
       coeff_l3(70,122)=  1.0893080384e-01
       coeff_l3(70,123)= -1.4330540597e-01
       coeff_l3(70,124)= -2.3536820710e-01
       coeff_l3(70,125)=  2.9925787821e-02
       coeff_l3(71,1)= -2.8437533975e-01
       coeff_l3(71,2)= -4.1908419132e-01
       coeff_l3(71,3)= -6.5932176076e-03
       coeff_l3(71,4)= -3.6945706606e-01
       coeff_l3(71,5)=  1.0778532922e-01
       coeff_l3(71,6)= -5.2295349538e-02
       coeff_l3(71,7)= -4.4569817185e-01
       coeff_l3(71,8)= -7.3755604029e-01
       coeff_l3(71,9)= -2.3696367443e-01
       coeff_l3(71,10)= -1.8201075494e-01
       coeff_l3(71,11)=  1.0262108594e-01
       coeff_l3(71,12)= -2.3042850196e-01
       coeff_l3(71,13)=  2.1026510000e-01
       coeff_l3(71,14)= -1.4894025028e-01
       coeff_l3(71,15)=  3.2832203433e-03
       coeff_l3(71,16)= -2.4790838361e-01
       coeff_l3(71,17)= -1.3455887139e-01
       coeff_l3(71,18)= -6.7826527357e-01
       coeff_l3(71,19)= -2.9014545679e-01
       coeff_l3(71,20)= -6.6362893581e-01
       coeff_l3(71,21)= -5.8977651596e-01
       coeff_l3(71,22)=  1.1378167570e-01
       coeff_l3(71,23)= -1.9461409748e-01
       coeff_l3(71,24)= -2.9401531816e-01
       coeff_l3(71,25)= -1.8325623870e-01
       coeff_l3(71,26)= -5.3528752178e-02
       coeff_l3(71,27)= -1.3349682093e-02
       coeff_l3(71,28)= -4.1130548716e-01
       coeff_l3(71,29)= -4.3143278360e-01
       coeff_l3(71,30)= -3.1405675411e-01
       coeff_l3(71,31)= -1.0598262399e-01
       coeff_l3(71,32)= -8.8587179780e-02
       coeff_l3(71,33)= -3.5165622830e-01
       coeff_l3(71,34)= -7.8659243882e-02
       coeff_l3(71,35)= -6.9542706013e-01
       coeff_l3(71,36)= -2.2695560008e-02
       coeff_l3(71,37)= -4.1588675231e-02
       coeff_l3(71,38)= -1.2396715581e-02
       coeff_l3(71,39)= -3.9864879847e-01
       coeff_l3(71,40)= -1.6553634405e-01
       coeff_l3(71,41)= -3.1777945161e-01
       coeff_l3(71,42)= -2.0194074512e-01
       coeff_l3(71,43)=  9.9258378148e-02
       coeff_l3(71,44)= -3.8457605243e-01
       coeff_l3(71,45)=  4.7885123640e-02
       coeff_l3(71,46)= -3.0973300338e-01
       coeff_l3(71,47)= -3.6892846227e-01
       coeff_l3(71,48)= -4.9060323834e-01
       coeff_l3(71,49)= -3.8143467903e+00
       coeff_l3(71,50)= -1.0807508230e-01
       coeff_l3(71,51)=  1.9609199464e-01
       coeff_l3(71,52)= -1.8503014743e-01
       coeff_l3(71,53)= -2.5991880894e-01
       coeff_l3(71,54)= -2.9703113437e-01
       coeff_l3(71,55)= -6.4830654860e-01
       coeff_l3(71,56)= -3.0599898100e-01
       coeff_l3(71,57)=  1.8115824088e-02
       coeff_l3(71,58)=  1.1745792627e+00
       coeff_l3(71,59)=  1.6476349533e-01
       coeff_l3(71,60)= -3.9962301254e+00
       coeff_l3(71,61)= -1.2994910777e-01
       coeff_l3(71,62)= -3.5477599502e-01
       coeff_l3(71,63)= -2.8087180853e-01
       coeff_l3(71,64)= -2.9596921802e-01
       coeff_l3(71,65)= -7.7242635190e-02
       coeff_l3(71,66)= -7.2840631008e-02
       coeff_l3(71,67)= -6.5596026182e-01
       coeff_l3(71,68)=  1.5006974339e-01
       coeff_l3(71,69)=  3.2297737896e-02
       coeff_l3(71,70)= -5.1208293438e-01
       coeff_l3(71,71)= -1.0713683367e+00
       coeff_l3(71,72)= -2.5842958689e-01
       coeff_l3(71,73)=  1.9549633563e-01
       coeff_l3(71,74)= -1.1713655293e-01
       coeff_l3(71,75)= -3.7156236172e-01
       coeff_l3(71,76)= -2.9634580612e+00
       coeff_l3(71,77)=  4.4357344508e-02
       coeff_l3(71,78)=  1.1077627540e-02
       coeff_l3(71,79)= -3.2436737418e-01
       coeff_l3(71,80)= -1.8308138847e+00
       coeff_l3(71,81)=  1.9950373098e-02
       coeff_l3(71,82)= -5.2042435855e-02
       coeff_l3(71,83)= -2.6872518659e-01
       coeff_l3(71,84)= -4.8693162203e-01
       coeff_l3(71,85)=  2.1197516471e-03
       coeff_l3(71,86)= -4.4505860656e-02
       coeff_l3(71,87)= -4.4412515126e-04
       coeff_l3(71,88)= -4.2487201095e-01
       coeff_l3(71,89)= -5.5646724999e-02
       coeff_l3(71,90)= -9.0463507175e-01
       coeff_l3(71,91)= -9.6017673612e-02
       coeff_l3(71,92)= -8.1079840660e-02
       coeff_l3(71,93)= -1.7450776696e-01
       coeff_l3(71,94)= -2.2025313228e-02
       coeff_l3(71,95)= -2.1426355839e-01
       coeff_l3(71,96)=  1.0407304019e-01
       coeff_l3(71,97)= -7.7784750611e-03
       coeff_l3(71,98)=  4.2281699181e-01
       coeff_l3(71,99)= -2.2058999538e-01
       coeff_l3(71,100)= -2.7872386575e-01
       coeff_l3(71,101)= -1.9983312115e-02
       coeff_l3(71,102)=  3.3342835307e-01
       coeff_l3(71,103)=  3.6661096383e-03
       coeff_l3(71,104)= -1.9296804070e-01
       coeff_l3(71,105)= -4.1866329312e-01
       coeff_l3(71,106)=  1.9082780182e-01
       coeff_l3(71,107)=  6.6617717966e-03
       coeff_l3(71,108)= -9.3380287290e-02
       coeff_l3(71,109)= -2.1760731936e-01
       coeff_l3(71,110)=  6.3640870154e-02
       coeff_l3(71,111)=  3.2461166382e-02
       coeff_l3(71,112)=  3.0715084076e-01
       coeff_l3(71,113)=  1.5332552604e-02
       coeff_l3(71,114)= -1.9064721465e-01
       coeff_l3(71,115)=  8.9598439634e-02
       coeff_l3(71,116)= -2.5674510002e-01
       coeff_l3(71,117)= -1.6237033904e-01
       coeff_l3(71,118)= -3.0037325621e-01
       coeff_l3(71,119)= -1.7639590800e-01
       coeff_l3(71,120)= -9.6884392202e-02
       coeff_l3(71,121)=  2.6348089799e-02
       coeff_l3(71,122)=  1.4592467807e-02
       coeff_l3(71,123)= -1.0140910745e-03
       coeff_l3(71,124)= -3.4257099032e-02
       coeff_l3(71,125)= -2.5718840957e-01
       coeff_l3(72,1)=  6.5287984908e-02
       coeff_l3(72,2)= -3.4103024518e-04
       coeff_l3(72,3)= -1.5740028024e-01
       coeff_l3(72,4)=  7.0669487119e-02
       coeff_l3(72,5)=  4.9895405769e-02
       coeff_l3(72,6)=  2.0076245070e-01
       coeff_l3(72,7)= -2.1865461767e-01
       coeff_l3(72,8)= -1.8530048430e-01
       coeff_l3(72,9)= -1.2187330425e-01
       coeff_l3(72,10)= -2.2719952464e-01
       coeff_l3(72,11)=  3.8471752778e-03
       coeff_l3(72,12)=  3.5101246834e-01
       coeff_l3(72,13)=  3.6950033903e-01
       coeff_l3(72,14)= -1.7945100367e-01
       coeff_l3(72,15)= -1.1736656725e-01
       coeff_l3(72,16)= -1.5405070782e-01
       coeff_l3(72,17)=  1.5931728482e-01
       coeff_l3(72,18)=  6.8109583855e-01
       coeff_l3(72,19)=  9.1493017972e-02
       coeff_l3(72,20)= -1.3082297146e-01
       coeff_l3(72,21)=  2.6114434004e-01
       coeff_l3(72,22)= -3.8047969341e-02
       coeff_l3(72,23)= -3.6485433578e-01
       coeff_l3(72,24)= -2.9562205076e-01
       coeff_l3(72,25)=  5.9462577105e-01
       coeff_l3(72,26)= -4.3367698789e-01
       coeff_l3(72,27)= -1.2408974767e-01
       coeff_l3(72,28)= -2.3798816255e-04
       coeff_l3(72,29)=  7.8642773628e-01
       coeff_l3(72,30)= -1.3567933440e-01
       coeff_l3(72,31)= -5.0844508410e-01
       coeff_l3(72,32)= -1.8540482223e-01
       coeff_l3(72,33)=  1.8625554442e-01
       coeff_l3(72,34)=  4.8094041646e-02
       coeff_l3(72,35)= -1.2596218288e-01
       coeff_l3(72,36)=  4.8145648092e-02
       coeff_l3(72,37)= -1.0356304795e-01
       coeff_l3(72,38)= -1.3794972897e+00
       coeff_l3(72,39)=  1.1846586317e-01
       coeff_l3(72,40)=  6.7445836961e-02
       coeff_l3(72,41)=  3.2633334398e-01
       coeff_l3(72,42)= -3.4862816334e-01
       coeff_l3(72,43)=  1.1937672645e-01
       coeff_l3(72,44)= -2.9263934493e-01
       coeff_l3(72,45)=  1.2665830553e-01
       coeff_l3(72,46)=  3.0019736290e-01
       coeff_l3(72,47)=  2.3470584303e-02
       coeff_l3(72,48)=  2.1804712713e-01
       coeff_l3(72,49)= -1.5567218885e-02
       coeff_l3(72,50)=  1.4926508069e-01
       coeff_l3(72,51)=  3.0865282752e-03
       coeff_l3(72,52)= -3.6737895012e-01
       coeff_l3(72,53)=  2.2345890105e-01
       coeff_l3(72,54)=  1.6190813854e-02
       coeff_l3(72,55)=  2.5964644551e-01
       coeff_l3(72,56)=  6.7799702287e-02
       coeff_l3(72,57)=  9.1183938086e-02
       coeff_l3(72,58)=  1.0158408433e-02
       coeff_l3(72,59)= -4.0077865124e-03
       coeff_l3(72,60)=  9.2863148451e-01
       coeff_l3(72,61)= -1.7120775580e-01
       coeff_l3(72,62)= -2.1618968248e-01
       coeff_l3(72,63)= -1.2595981359e-01
       coeff_l3(72,64)=  2.2690784186e-02
       coeff_l3(72,65)=  3.1514767557e-02
       coeff_l3(72,66)=  1.9912099838e-01
       coeff_l3(72,67)=  2.5326496363e-01
       coeff_l3(72,68)=  6.1650700867e-02
       coeff_l3(72,69)=  1.1582662910e-01
       coeff_l3(72,70)=  2.0683294162e-02
       coeff_l3(72,71)=  6.0280591249e-01
       coeff_l3(72,72)= -2.6058664918e-01
       coeff_l3(72,73)=  2.2507874668e-01
       coeff_l3(72,74)=  5.8898702264e-02
       coeff_l3(72,75)= -4.4119089842e-01
       coeff_l3(72,76)= -4.2107468843e-01
       coeff_l3(72,77)=  1.7617844045e-01
       coeff_l3(72,78)=  9.8900958896e-02
       coeff_l3(72,79)= -2.3859364912e-02
       coeff_l3(72,80)=  2.0654277503e-01
       coeff_l3(72,81)=  1.2370066345e-01
       coeff_l3(72,82)= -5.3199749440e-02
       coeff_l3(72,83)= -3.9425748587e-01
       coeff_l3(72,84)= -3.2185941935e-02
       coeff_l3(72,85)=  7.1830131114e-02
       coeff_l3(72,86)=  1.3517645001e-01
       coeff_l3(72,87)=  1.6743968427e-01
       coeff_l3(72,88)= -1.8054009974e-01
       coeff_l3(72,89)= -1.4571790397e-01
       coeff_l3(72,90)= -3.4762939811e-01
       coeff_l3(72,91)= -3.4477636218e-01
       coeff_l3(72,92)= -5.9187640436e-03
       coeff_l3(72,93)=  3.7791463733e-01
       coeff_l3(72,94)=  5.4905998707e-01
       coeff_l3(72,95)=  4.7599080950e-02
       coeff_l3(72,96)=  8.7803244591e-02
       coeff_l3(72,97)=  1.8659880757e-01
       coeff_l3(72,98)=  5.5279359221e-02
       coeff_l3(72,99)=  1.9529755414e-01
       coeff_l3(72,100)=  2.6529553533e-01
       coeff_l3(72,101)= -1.5427885950e-01
       coeff_l3(72,102)=  2.2973687947e-01
       coeff_l3(72,103)= -2.3205703497e-01
       coeff_l3(72,104)= -5.8348196745e-01
       coeff_l3(72,105)= -3.8278140128e-02
       coeff_l3(72,106)= -8.5422165692e-02
       coeff_l3(72,107)=  1.2743064761e-01
       coeff_l3(72,108)=  6.4952112734e-02
       coeff_l3(72,109)= -7.0543564856e-02
       coeff_l3(72,110)= -1.0138007998e-01
       coeff_l3(72,111)=  2.6835438609e-01
       coeff_l3(72,112)= -9.8717242479e-02
       coeff_l3(72,113)=  2.4107864499e-01
       coeff_l3(72,114)=  3.6585316062e-01
       coeff_l3(72,115)=  2.5874695182e-01
       coeff_l3(72,116)=  2.6507985592e-01
       coeff_l3(72,117)=  5.0393834710e-02
       coeff_l3(72,118)=  6.7671906948e-01
       coeff_l3(72,119)= -9.2057585716e-02
       coeff_l3(72,120)=  2.8050667048e-01
       coeff_l3(72,121)=  1.0461367667e-01
       coeff_l3(72,122)=  3.2567072660e-02
       coeff_l3(72,123)=  7.2056770325e-02
       coeff_l3(72,124)=  2.0828640461e-01
       coeff_l3(72,125)= -8.1321410835e-02
       coeff_l3(73,1)=  3.0545750260e-01
       coeff_l3(73,2)= -1.0944943130e-01
       coeff_l3(73,3)= -6.1649244279e-02
       coeff_l3(73,4)=  1.3734357059e-01
       coeff_l3(73,5)=  1.2649357319e-01
       coeff_l3(73,6)= -8.9231714606e-02
       coeff_l3(73,7)=  1.8796272576e-02
       coeff_l3(73,8)=  1.4976368845e-01
       coeff_l3(73,9)= -1.9042161107e-01
       coeff_l3(73,10)= -2.2568961605e-02
       coeff_l3(73,11)= -8.0604763934e-04
       coeff_l3(73,12)=  2.3501382768e-01
       coeff_l3(73,13)=  2.2631934285e-01
       coeff_l3(73,14)=  6.8959921598e-02
       coeff_l3(73,15)= -3.8436236978e-01
       coeff_l3(73,16)= -1.7677657306e-01
       coeff_l3(73,17)=  1.5707109869e-01
       coeff_l3(73,18)=  3.2588085532e-01
       coeff_l3(73,19)= -2.9933547974e-01
       coeff_l3(73,20)= -1.2316503376e-01
       coeff_l3(73,21)= -2.6188844442e-01
       coeff_l3(73,22)=  7.8764289618e-02
       coeff_l3(73,23)= -3.5179468989e-01
       coeff_l3(73,24)= -2.1778565645e-01
       coeff_l3(73,25)= -1.5712438524e-01
       coeff_l3(73,26)= -1.5900902450e-01
       coeff_l3(73,27)= -1.4322949946e-01
       coeff_l3(73,28)=  2.2450944781e-01
       coeff_l3(73,29)=  3.5197079182e-01
       coeff_l3(73,30)= -2.6470670104e-01
       coeff_l3(73,31)= -4.8022043705e-01
       coeff_l3(73,32)= -2.7619099617e-01
       coeff_l3(73,33)=  7.2180137038e-02
       coeff_l3(73,34)= -3.4217512608e-01
       coeff_l3(73,35)= -3.6950118374e-04
       coeff_l3(73,36)=  9.0377062559e-02
       coeff_l3(73,37)= -1.3757316768e-01
       coeff_l3(73,38)=  6.3476330042e-01
       coeff_l3(73,39)= -1.4949616790e-01
       coeff_l3(73,40)=  2.3134370148e-01
       coeff_l3(73,41)=  2.8070652485e-01
       coeff_l3(73,42)= -3.2620316744e-01
       coeff_l3(73,43)=  6.5398640931e-02
       coeff_l3(73,44)= -7.6291459799e-01
       coeff_l3(73,45)=  7.3594480753e-02
       coeff_l3(73,46)=  2.5430259109e-01
       coeff_l3(73,47)= -9.8877409473e-03
       coeff_l3(73,48)= -7.0574559271e-02
       coeff_l3(73,49)= -2.7023079991e-01
       coeff_l3(73,50)= -5.7665444911e-03
       coeff_l3(73,51)=  7.2040036321e-02
       coeff_l3(73,52)= -1.8851973116e-02
       coeff_l3(73,53)= -1.4039449394e-02
       coeff_l3(73,54)=  1.1745631695e-02
       coeff_l3(73,55)= -1.3022334874e-01
       coeff_l3(73,56)=  2.5965606794e-02
       coeff_l3(73,57)=  2.2779658437e-02
       coeff_l3(73,58)= -4.2460790277e-01
       coeff_l3(73,59)=  1.3300004601e-01
       coeff_l3(73,60)=  2.0706525445e-01
       coeff_l3(73,61)= -4.3046358228e-01
       coeff_l3(73,62)= -3.3731162548e-01
       coeff_l3(73,63)= -1.5189361759e-02
       coeff_l3(73,64)= -3.4742744174e-03
       coeff_l3(73,65)= -1.2103766948e-01
       coeff_l3(73,66)= -1.0191397369e-01
       coeff_l3(73,67)=  3.9496932179e-02
       coeff_l3(73,68)=  3.2290804386e-01
       coeff_l3(73,69)= -1.1140472442e-01
       coeff_l3(73,70)= -3.0257225037e-01
       coeff_l3(73,71)=  2.2119520605e-01
       coeff_l3(73,72)= -2.8076708317e-01
       coeff_l3(73,73)=  8.4220886230e-02
       coeff_l3(73,74)=  1.2349831313e-01
       coeff_l3(73,75)= -2.3938761652e-01
       coeff_l3(73,76)= -3.8277816772e-01
       coeff_l3(73,77)= -3.3610197902e-01
       coeff_l3(73,78)= -1.3638201356e-01
       coeff_l3(73,79)= -2.6839083433e-01
       coeff_l3(73,80)= -7.0421651006e-02
       coeff_l3(73,81)=  7.4658729136e-03
       coeff_l3(73,82)=  2.7891829610e-01
       coeff_l3(73,83)=  1.4772525430e-01
       coeff_l3(73,84)= -1.9880296290e-01
       coeff_l3(73,85)= -1.9171753526e-01
       coeff_l3(73,86)= -9.9475987256e-02
       coeff_l3(73,87)=  2.5510144234e-01
       coeff_l3(73,88)= -4.8875507712e-01
       coeff_l3(73,89)=  9.7000315785e-02
       coeff_l3(73,90)= -7.2453248501e-01
       coeff_l3(73,91)= -2.6631778479e-01
       coeff_l3(73,92)= -7.9158835113e-02
       coeff_l3(73,93)=  1.8503828347e-01
       coeff_l3(73,94)=  4.8976889253e-01
       coeff_l3(73,95)=  1.3302396238e-01
       coeff_l3(73,96)=  2.3485706747e-01
       coeff_l3(73,97)=  1.6721314192e-01
       coeff_l3(73,98)= -2.7984343469e-02
       coeff_l3(73,99)= -2.6584070921e-01
       coeff_l3(73,100)=  2.0773348212e-01
       coeff_l3(73,101)= -1.8916928768e-01
       coeff_l3(73,102)=  5.7241134346e-02
       coeff_l3(73,103)= -1.9803090394e-01
       coeff_l3(73,104)= -1.1771506816e-01
       coeff_l3(73,105)= -1.7975214124e-01
       coeff_l3(73,106)=  2.8964745998e-01
       coeff_l3(73,107)= -4.8655401915e-02
       coeff_l3(73,108)=  3.0440133810e-01
       coeff_l3(73,109)= -2.1912956238e-01
       coeff_l3(73,110)=  7.5566194952e-02
       coeff_l3(73,111)=  2.8305983543e-01
       coeff_l3(73,112)=  1.2974466383e-01
       coeff_l3(73,113)= -8.6354194209e-03
       coeff_l3(73,114)=  2.7477217838e-02
       coeff_l3(73,115)=  2.8241026402e-01
       coeff_l3(73,116)= -2.2996084392e-01
       coeff_l3(73,117)= -1.6931842268e-01
       coeff_l3(73,118)=  3.4835052490e-01
       coeff_l3(73,119)= -1.3012012467e-02
       coeff_l3(73,120)=  1.4062063396e-01
       coeff_l3(73,121)=  1.2310191989e-01
       coeff_l3(73,122)=  1.7203839123e-01
       coeff_l3(73,123)= -1.0077271610e-01
       coeff_l3(73,124)= -1.4339213073e-01
       coeff_l3(73,125)= -1.6659425199e-01
       coeff_l3(74,1)=  1.3523839414e-01
       coeff_l3(74,2)= -8.0377971753e-03
       coeff_l3(74,3)=  4.4989943504e-02
       coeff_l3(74,4)= -1.2097281870e-03
       coeff_l3(74,5)=  7.8553736210e-02
       coeff_l3(74,6)=  1.0592625290e-01
       coeff_l3(74,7)= -6.8356310949e-03
       coeff_l3(74,8)= -1.0414332151e-01
       coeff_l3(74,9)=  4.0854293853e-02
       coeff_l3(74,10)=  1.1336323619e-01
       coeff_l3(74,11)= -1.0746984044e-03
       coeff_l3(74,12)= -9.5353193581e-02
       coeff_l3(74,13)= -1.3119108975e-01
       coeff_l3(74,14)= -1.7366187274e-01
       coeff_l3(74,15)= -5.8697097003e-02
       coeff_l3(74,16)= -5.5277168751e-02
       coeff_l3(74,17)=  1.1750429124e-01
       coeff_l3(74,18)= -1.6762048006e-01
       coeff_l3(74,19)=  2.5856555440e-03
       coeff_l3(74,20)=  5.2263423800e-02
       coeff_l3(74,21)= -7.6820708811e-02
       coeff_l3(74,22)= -1.0028666817e-02
       coeff_l3(74,23)=  1.5222084522e-01
       coeff_l3(74,24)=  9.4119384885e-02
       coeff_l3(74,25)= -1.0920869559e-01
       coeff_l3(74,26)=  3.8606036454e-02
       coeff_l3(74,27)=  6.5459854901e-02
       coeff_l3(74,28)=  1.2156644464e-01
       coeff_l3(74,29)= -2.2310136259e-01
       coeff_l3(74,30)=  4.5426376164e-02
       coeff_l3(74,31)= -4.2905494571e-02
       coeff_l3(74,32)=  2.0354464650e-02
       coeff_l3(74,33)= -1.4502735436e-01
       coeff_l3(74,34)= -6.5850026906e-02
       coeff_l3(74,35)= -1.7885537446e-01
       coeff_l3(74,36)=  4.7790434211e-02
       coeff_l3(74,37)=  1.2476053834e-01
       coeff_l3(74,38)= -3.0676838756e-01
       coeff_l3(74,39)= -9.1699957848e-02
       coeff_l3(74,40)= -2.4352058768e-02
       coeff_l3(74,41)= -4.7575032711e-01
       coeff_l3(74,42)= -1.0927109420e-01
       coeff_l3(74,43)= -1.2972847559e-02
       coeff_l3(74,44)=  6.1266578734e-02
       coeff_l3(74,45)= -5.8621643111e-03
       coeff_l3(74,46)= -5.8919701725e-02
       coeff_l3(74,47)= -1.6337204725e-02
       coeff_l3(74,48)= -5.1451418549e-02
       coeff_l3(74,49)=  1.1728903055e+00
       coeff_l3(74,50)= -5.3649678826e-02
       coeff_l3(74,51)= -2.4839110672e-01
       coeff_l3(74,52)=  5.0768997520e-02
       coeff_l3(74,53)=  5.8983251452e-02
       coeff_l3(74,54)= -1.1094621383e-02
       coeff_l3(74,55)=  2.0144058764e-01
       coeff_l3(74,56)=  1.6624329984e-01
       coeff_l3(74,57)=  5.0808601081e-02
       coeff_l3(74,58)=  3.1482115388e-01
       coeff_l3(74,59)= -4.1718266904e-02
       coeff_l3(74,60)= -1.3433820009e-01
       coeff_l3(74,61)= -1.3898764551e-01
       coeff_l3(74,62)= -1.0532138497e-01
       coeff_l3(74,63)=  2.1525378525e-01
       coeff_l3(74,64)=  2.4028372020e-02
       coeff_l3(74,65)= -1.0578843951e-01
       coeff_l3(74,66)= -1.9639292359e-01
       coeff_l3(74,67)=  1.0889369994e-01
       coeff_l3(74,68)= -7.6687820256e-02
       coeff_l3(74,69)= -1.3451451063e-01
       coeff_l3(74,70)= -1.3808077201e-02
       coeff_l3(74,71)= -3.4296888858e-02
       coeff_l3(74,72)=  1.3778170943e-01
       coeff_l3(74,73)= -2.5945320725e-02
       coeff_l3(74,74)= -6.9705367088e-02
       coeff_l3(74,75)= -8.9374437928e-02
       coeff_l3(74,76)=  8.4290134907e-01
       coeff_l3(74,77)=  6.4856274985e-03
       coeff_l3(74,78)=  9.9820673466e-02
       coeff_l3(74,79)=  1.4445331693e-01
       coeff_l3(74,80)=  1.3943783939e-01
       coeff_l3(74,81)=  1.4626197517e-01
       coeff_l3(74,82)=  1.0680457950e-01
       coeff_l3(74,83)=  1.0895298421e-01
       coeff_l3(74,84)=  2.1970134974e-01
       coeff_l3(74,85)=  2.3687554896e-01
       coeff_l3(74,86)=  8.5942009464e-03
       coeff_l3(74,87)= -4.6661753207e-02
       coeff_l3(74,88)= -8.5585989058e-02
       coeff_l3(74,89)=  4.0636643767e-02
       coeff_l3(74,90)= -3.5167098045e-02
       coeff_l3(74,91)=  3.9224421978e-01
       coeff_l3(74,92)=  4.3548163027e-02
       coeff_l3(74,93)=  2.9921224341e-02
       coeff_l3(74,94)=  6.3770405948e-02
       coeff_l3(74,95)= -1.4838908613e-01
       coeff_l3(74,96)= -1.0640592873e-01
       coeff_l3(74,97)= -2.1570354700e-01
       coeff_l3(74,98)=  3.6319482327e-01
       coeff_l3(74,99)= -2.6884856820e-01
       coeff_l3(74,100)= -2.1699815989e-03
       coeff_l3(74,101)=  2.0665313303e-01
       coeff_l3(74,102)= -9.5188831910e-03
       coeff_l3(74,103)= -4.4195603579e-02
       coeff_l3(74,104)=  1.2206915021e-01
       coeff_l3(74,105)= -3.1417131424e-01
       coeff_l3(74,106)= -9.9185459316e-02
       coeff_l3(74,107)= -2.3340654373e-01
       coeff_l3(74,108)=  8.5609197617e-02
       coeff_l3(74,109)= -1.3630856574e-01
       coeff_l3(74,110)=  3.8108661771e-02
       coeff_l3(74,111)= -5.8356039226e-02
       coeff_l3(74,112)= -3.3241458237e-02
       coeff_l3(74,113)= -6.9393858314e-02
       coeff_l3(74,114)=  2.7211317793e-02
       coeff_l3(74,115)= -4.2017817497e-02
       coeff_l3(74,116)=  7.8763447702e-02
       coeff_l3(74,117)= -2.7071246505e-01
       coeff_l3(74,118)= -2.3525536060e-01
       coeff_l3(74,119)= -8.2116760314e-02
       coeff_l3(74,120)=  8.9797645807e-02
       coeff_l3(74,121)= -8.1845410168e-02
       coeff_l3(74,122)= -5.0231326371e-02
       coeff_l3(74,123)=  4.8855595291e-02
       coeff_l3(74,124)= -1.6079884768e-01
       coeff_l3(74,125)=  1.7737290263e-01
       coeff_l3(75,1)= -6.5125733614e-01
       coeff_l3(75,2)= -7.4150800705e-02
       coeff_l3(75,3)=  1.4155985415e-01
       coeff_l3(75,4)= -1.4512898028e-01
       coeff_l3(75,5)= -6.1725404114e-02
       coeff_l3(75,6)= -4.4417187572e-02
       coeff_l3(75,7)=  8.3314599469e-03
       coeff_l3(75,8)= -4.6768584847e-01
       coeff_l3(75,9)=  3.9697244763e-02
       coeff_l3(75,10)=  1.4634533226e-01
       coeff_l3(75,11)= -1.3656820357e-01
       coeff_l3(75,12)= -6.5699833632e-01
       coeff_l3(75,13)= -5.3637808561e-01
       coeff_l3(75,14)= -1.4334963262e-01
       coeff_l3(75,15)= -1.1958491802e-01
       coeff_l3(75,16)=  7.0429995656e-02
       coeff_l3(75,17)= -4.8311012983e-01
       coeff_l3(75,18)= -5.4329484701e-01
       coeff_l3(75,19)=  2.7804997563e-01
       coeff_l3(75,20)= -8.8045097888e-02
       coeff_l3(75,21)=  2.2427183390e-01
       coeff_l3(75,22)= -2.1560953557e-01
       coeff_l3(75,23)=  1.0347665846e-01
       coeff_l3(75,24)=  1.6017900780e-02
       coeff_l3(75,25)=  3.8127872348e-01
       coeff_l3(75,26)= -2.2319911048e-02
       coeff_l3(75,27)= -1.5746277571e-01
       coeff_l3(75,28)= -3.7174290419e-01
       coeff_l3(75,29)= -1.9263079762e-01
       coeff_l3(75,30)=  1.6383868456e-01
       coeff_l3(75,31)=  5.1416043192e-02
       coeff_l3(75,32)= -3.0876580626e-02
       coeff_l3(75,33)= -2.0079594851e-01
       coeff_l3(75,34)=  1.0979973525e-01
       coeff_l3(75,35)= -1.1852691174e+00
       coeff_l3(75,36)=  8.2770548761e-03
       coeff_l3(75,37)= -4.8715040088e-02
       coeff_l3(75,38)=  2.8217992187e-01
       coeff_l3(75,39)= -1.7335757613e-01
       coeff_l3(75,40)= -3.5720598698e-01
       coeff_l3(75,41)= -6.7501747608e-01
       coeff_l3(75,42)=  1.0697802901e-01
       coeff_l3(75,43)= -3.1339162588e-01
       coeff_l3(75,44)=  6.1815444380e-02
       coeff_l3(75,45)= -7.1006113291e-01
       coeff_l3(75,46)= -2.3454113305e-01
       coeff_l3(75,47)=  4.3518537283e-01
       coeff_l3(75,48)= -1.8141233921e-01
       coeff_l3(75,49)= -1.0497216880e-01
       coeff_l3(75,50)= -6.0611747205e-02
       coeff_l3(75,51)= -2.2935590148e-01
       coeff_l3(75,52)=  7.3217011988e-02
       coeff_l3(75,53)= -8.3696126938e-02
       coeff_l3(75,54)= -1.9833259284e-01
       coeff_l3(75,55)= -1.1276357621e-01
       coeff_l3(75,56)=  9.6159234643e-02
       coeff_l3(75,57)= -1.2416719645e-01
       coeff_l3(75,58)=  7.1242308617e-01
       coeff_l3(75,59)= -2.4857456982e-01
       coeff_l3(75,60)= -3.2626599073e-01
       coeff_l3(75,61)= -1.2061227113e-01
       coeff_l3(75,62)= -3.0972892419e-02
       coeff_l3(75,63)=  2.6383814216e-01
       coeff_l3(75,64)= -9.6913449466e-02
       coeff_l3(75,65)= -5.1437657326e-02
       coeff_l3(75,66)= -3.5475909710e-02
       coeff_l3(75,67)=  5.8528733253e-01
       coeff_l3(75,68)=  7.1491524577e-02
       coeff_l3(75,69)=  1.6299334168e-01
       coeff_l3(75,70)=  4.4074025005e-02
       coeff_l3(75,71)= -7.8142040968e-01
       coeff_l3(75,72)=  1.5903049707e-01
       coeff_l3(75,73)= -1.4684642851e-01
       coeff_l3(75,74)=  1.2319024652e-01
       coeff_l3(75,75)=  3.4564085305e-02
       coeff_l3(75,76)= -3.1661656499e-01
       coeff_l3(75,77)=  1.3518576324e-01
       coeff_l3(75,78)=  3.8697324693e-02
       coeff_l3(75,79)= -9.2004179955e-02
       coeff_l3(75,80)=  6.6640682518e-02
       coeff_l3(75,81)= -4.1404116154e-01
       coeff_l3(75,82)=  2.4189550430e-02
       coeff_l3(75,83)= -1.2782341242e-01
       coeff_l3(75,84)= -2.8252893686e-01
       coeff_l3(75,85)= -3.5868164897e-01
       coeff_l3(75,86)= -1.3280133903e-01
       coeff_l3(75,87)= -1.2863837183e-01
       coeff_l3(75,88)=  2.9140906408e-02
       coeff_l3(75,89)=  1.0300716013e-01
       coeff_l3(75,90)=  4.3622389436e-01
       coeff_l3(75,91)= -7.5081157684e-01
       coeff_l3(75,92)= -9.3024007976e-02
       coeff_l3(75,93)= -8.9908830822e-02
       coeff_l3(75,94)= -2.4943013489e-01
       coeff_l3(75,95)= -3.0875930190e-01
       coeff_l3(75,96)= -2.3213440180e-01
       coeff_l3(75,97)= -6.1403114349e-02
       coeff_l3(75,98)= -1.6849266365e-02
       coeff_l3(75,99)= -9.6797689795e-02
       coeff_l3(75,100)= -1.9814034924e-02
       coeff_l3(75,101)= -2.1272039413e-01
       coeff_l3(75,102)=  6.8347424269e-02
       coeff_l3(75,103)=  8.9324012399e-02
       coeff_l3(75,104)= -1.4948999509e-02
       coeff_l3(75,105)=  1.4318884909e-01
       coeff_l3(75,106)= -1.9739553332e-01
       coeff_l3(75,107)=  2.3281548172e-02
       coeff_l3(75,108)= -7.1811325848e-02
       coeff_l3(75,109)= -1.0605055094e-01
       coeff_l3(75,110)= -1.9522082061e-02
       coeff_l3(75,111)= -3.9559248090e-01
       coeff_l3(75,112)= -4.4376313686e-01
       coeff_l3(75,113)= -3.6585301161e-01
       coeff_l3(75,114)= -1.3768462837e-01
       coeff_l3(75,115)=  2.9727384448e-01
       coeff_l3(75,116)= -7.1536891162e-02
       coeff_l3(75,117)= -1.1034212261e-01
       coeff_l3(75,118)= -4.0214034915e-01
       coeff_l3(75,119)=  7.1312867105e-02
       coeff_l3(75,120)= -8.1862747669e-02
       coeff_l3(75,121)= -1.9158784300e-02
       coeff_l3(75,122)= -1.9973637164e-01
       coeff_l3(75,123)= -2.1951762959e-02
       coeff_l3(75,124)=  9.2934645712e-02
       coeff_l3(75,125)= -4.0485709906e-02
       coeff_l3(76,1)= -2.2543011606e-01
       coeff_l3(76,2)= -1.3008370996e-01
       coeff_l3(76,3)=  1.7733084038e-02
       coeff_l3(76,4)= -1.1377132498e-02
       coeff_l3(76,5)=  3.4248463809e-02
       coeff_l3(76,6)=  8.1805430353e-02
       coeff_l3(76,7)=  4.8626601696e-02
       coeff_l3(76,8)= -1.2101062573e-02
       coeff_l3(76,9)= -9.5513258129e-03
       coeff_l3(76,10)= -1.5486897528e-01
       coeff_l3(76,11)=  1.7261421680e-01
       coeff_l3(76,12)= -1.9987383485e-01
       coeff_l3(76,13)= -1.9558688998e-01
       coeff_l3(76,14)= -2.5362530351e-01
       coeff_l3(76,15)=  2.0419257879e-01
       coeff_l3(76,16)=  7.0535629988e-02
       coeff_l3(76,17)= -1.2476956844e-01
       coeff_l3(76,18)= -2.1628050506e-01
       coeff_l3(76,19)= -6.6222846508e-02
       coeff_l3(76,20)= -1.7446772754e-01
       coeff_l3(76,21)=  1.6948011518e-01
       coeff_l3(76,22)= -8.5273742676e-02
       coeff_l3(76,23)=  2.9836270213e-01
       coeff_l3(76,24)=  1.6241291538e-02
       coeff_l3(76,25)= -1.8765041232e-01
       coeff_l3(76,26)= -2.5499742478e-02
       coeff_l3(76,27)=  2.9772642255e-01
       coeff_l3(76,28)=  1.1890754104e-02
       coeff_l3(76,29)= -1.8483373523e-01
       coeff_l3(76,30)=  6.4837642014e-02
       coeff_l3(76,31)=  2.6289565489e-02
       coeff_l3(76,32)= -5.0655502826e-02
       coeff_l3(76,33)= -1.5400825441e-01
       coeff_l3(76,34)=  1.2632323802e-01
       coeff_l3(76,35)=  6.6466972232e-02
       coeff_l3(76,36)=  1.8350969255e-01
       coeff_l3(76,37)= -9.2107169330e-02
       coeff_l3(76,38)= -3.3590483665e-01
       coeff_l3(76,39)=  5.9619685635e-03
       coeff_l3(76,40)= -2.2782262415e-02
       coeff_l3(76,41)=  3.4070065618e-01
       coeff_l3(76,42)= -9.5875516534e-02
       coeff_l3(76,43)=  4.5297984034e-02
       coeff_l3(76,44)= -6.6743078642e-03
       coeff_l3(76,45)= -5.8394715190e-02
       coeff_l3(76,46)= -3.5685518384e-01
       coeff_l3(76,47)= -7.3592565954e-02
       coeff_l3(76,48)= -3.1875833869e-02
       coeff_l3(76,49)= -1.7837567255e-02
       coeff_l3(76,50)= -1.2039133906e-01
       coeff_l3(76,51)= -1.4288756251e-01
       coeff_l3(76,52)=  8.5432091728e-03
       coeff_l3(76,53)= -4.1684582829e-02
       coeff_l3(76,54)= -5.2330689505e-03
       coeff_l3(76,55)= -1.1396547407e-01
       coeff_l3(76,56)= -2.9615009204e-02
       coeff_l3(76,57)= -1.0660254955e-01
       coeff_l3(76,58)=  2.2252827883e-02
       coeff_l3(76,59)= -2.4244274199e-01
       coeff_l3(76,60)= -2.7707317844e-02
       coeff_l3(76,61)=  2.2360395640e-02
       coeff_l3(76,62)= -7.3921419680e-02
       coeff_l3(76,63)=  4.0966860950e-02
       coeff_l3(76,64)=  1.0391896963e-01
       coeff_l3(76,65)= -9.1210573912e-02
       coeff_l3(76,66)= -1.7715834081e-02
       coeff_l3(76,67)=  3.5966047645e-01
       coeff_l3(76,68)= -1.3888734579e-01
       coeff_l3(76,69)=  8.0667901784e-03
       coeff_l3(76,70)= -8.6824096739e-02
       coeff_l3(76,71)= -1.5556228161e-01
       coeff_l3(76,72)= -1.2208247185e-01
       coeff_l3(76,73)= -9.8027803004e-02
       coeff_l3(76,74)= -1.6934303939e-01
       coeff_l3(76,75)=  1.7221535742e-01
       coeff_l3(76,76)= -1.1913783103e-01
       coeff_l3(76,77)= -2.1432915702e-02
       coeff_l3(76,78)=  8.1597834826e-02
       coeff_l3(76,79)=  9.5236539841e-02
       coeff_l3(76,80)= -4.2115101218e-01
       coeff_l3(76,81)=  4.4465169311e-02
       coeff_l3(76,82)=  9.3250460923e-02
       coeff_l3(76,83)=  3.0631275848e-02
       coeff_l3(76,84)=  1.3838619925e-03
       coeff_l3(76,85)= -4.8215072602e-02
       coeff_l3(76,86)= -5.6573662907e-02
       coeff_l3(76,87)= -4.9622494727e-02
       coeff_l3(76,88)=  5.6959107518e-02
       coeff_l3(76,89)= -9.4407595694e-02
       coeff_l3(76,90)=  1.6233959794e-01
       coeff_l3(76,91)=  9.2631138861e-02
       coeff_l3(76,92)= -1.2510055304e-01
       coeff_l3(76,93)= -3.0087062716e-01
       coeff_l3(76,94)= -5.7497952133e-02
       coeff_l3(76,95)= -1.3436400890e-01
       coeff_l3(76,96)= -2.6816976070e-01
       coeff_l3(76,97)= -3.0593881384e-03
       coeff_l3(76,98)=  1.2265532464e-01
       coeff_l3(76,99)=  5.0636835396e-02
       coeff_l3(76,100)=  5.7566326112e-02
       coeff_l3(76,101)= -6.0847338289e-02
       coeff_l3(76,102)= -6.4147725701e-02
       coeff_l3(76,103)=  1.0587070137e-01
       coeff_l3(76,104)= -1.8948836625e-01
       coeff_l3(76,105)=  3.2667815685e-01
       coeff_l3(76,106)= -3.2799284905e-02
       coeff_l3(76,107)=  5.0225250423e-02
       coeff_l3(76,108)=  6.5360153094e-03
       coeff_l3(76,109)= -1.7217750847e-01
       coeff_l3(76,110)=  1.0853973031e-01
       coeff_l3(76,111)= -9.0791441500e-02
       coeff_l3(76,112)=  5.4890681058e-02
       coeff_l3(76,113)= -1.8129421771e-01
       coeff_l3(76,114)=  1.7019352317e-01
       coeff_l3(76,115)= -1.4777734876e-01
       coeff_l3(76,116)=  1.0253787786e-01
       coeff_l3(76,117)= -1.4289410412e-01
       coeff_l3(76,118)= -7.3673881590e-02
       coeff_l3(76,119)=  3.6652393639e-02
       coeff_l3(76,120)= -1.9152216613e-01
       coeff_l3(76,121)= -4.7510173917e-01
       coeff_l3(76,122)= -1.3862402178e-02
       coeff_l3(76,123)= -1.8836705387e-01
       coeff_l3(76,124)=  6.3217997551e-02
       coeff_l3(76,125)= -1.0499985516e-01
       coeff_l3(77,1)=  4.2452445626e-01
       coeff_l3(77,2)= -3.2111087441e-01
       coeff_l3(77,3)=  1.7670650780e-01
       coeff_l3(77,4)= -2.5589486957e-01
       coeff_l3(77,5)=  1.8358993530e-01
       coeff_l3(77,6)= -9.3312062323e-02
       coeff_l3(77,7)= -9.7042065859e-01
       coeff_l3(77,8)= -2.3380498588e-01
       coeff_l3(77,9)= -1.6641838849e-01
       coeff_l3(77,10)= -1.6636608541e-01
       coeff_l3(77,11)= -2.0175378025e-01
       coeff_l3(77,12)=  1.6240262985e-01
       coeff_l3(77,13)=  6.1941150576e-02
       coeff_l3(77,14)= -4.8820471764e-01
       coeff_l3(77,15)= -2.8913164139e-01
       coeff_l3(77,16)= -2.0413610339e-01
       coeff_l3(77,17)= -8.9192435145e-02
       coeff_l3(77,18)= -4.4891837239e-01
       coeff_l3(77,19)= -2.0708357915e-02
       coeff_l3(77,20)= -7.8072622418e-02
       coeff_l3(77,21)=  3.2242947817e-01
       coeff_l3(77,22)=  2.9234705493e-02
       coeff_l3(77,23)= -2.3193448782e-02
       coeff_l3(77,24)= -2.8123900294e-01
       coeff_l3(77,25)= -1.6954553127e+00
       coeff_l3(77,26)= -8.2478858531e-02
       coeff_l3(77,27)= -1.1709272861e+00
       coeff_l3(77,28)= -9.0049761534e-01
       coeff_l3(77,29)= -1.5459792316e-01
       coeff_l3(77,30)= -8.0717958510e-02
       coeff_l3(77,31)= -1.2117920071e-01
       coeff_l3(77,32)= -3.5783299804e-01
       coeff_l3(77,33)= -3.8875076920e-02
       coeff_l3(77,34)= -8.9627183974e-02
       coeff_l3(77,35)=  2.8528389335e-01
       coeff_l3(77,36)=  3.9213441312e-02
       coeff_l3(77,37)= -3.1828337908e-01
       coeff_l3(77,38)=  5.9761291742e-01
       coeff_l3(77,39)= -4.3465000391e-01
       coeff_l3(77,40)= -4.7554489970e-01
       coeff_l3(77,41)=  7.8523159027e-01
       coeff_l3(77,42)= -2.1609926224e-01
       coeff_l3(77,43)= -1.4320917428e-02
       coeff_l3(77,44)= -5.0381177664e-01
       coeff_l3(77,45)= -2.2348839045e-01
       coeff_l3(77,46)= -1.3207484484e+00
       coeff_l3(77,47)=  2.1378034353e-01
       coeff_l3(77,48)= -2.8365626931e-01
       coeff_l3(77,49)= -1.0793811083e+00
       coeff_l3(77,50)= -1.1156263202e-01
       coeff_l3(77,51)= -6.6019093990e-01
       coeff_l3(77,52)= -2.0085689425e-01
       coeff_l3(77,53)= -2.1506845951e-01
       coeff_l3(77,54)= -5.2855134010e-01
       coeff_l3(77,55)= -2.7066373825e-01
       coeff_l3(77,56)= -4.1143927723e-02
       coeff_l3(77,57)= -1.8415232003e-01
       coeff_l3(77,58)= -7.3188954592e-01
       coeff_l3(77,59)= -5.0527930260e-01
       coeff_l3(77,60)= -4.9661809206e-01
       coeff_l3(77,61)= -2.2690448165e-01
       coeff_l3(77,62)= -3.6594551802e-01
       coeff_l3(77,63)=  4.4282257557e-02
       coeff_l3(77,64)= -3.5381463170e-01
       coeff_l3(77,65)= -2.7058032155e-01
       coeff_l3(77,66)= -4.0247926116e-01
       coeff_l3(77,67)=  9.6406680346e-01
       coeff_l3(77,68)= -6.4913444221e-02
       coeff_l3(77,69)=  8.8121555746e-02
       coeff_l3(77,70)= -5.8359998465e-01
       coeff_l3(77,71)= -9.8329603672e-01
       coeff_l3(77,72)= -1.5146747231e-01
       coeff_l3(77,73)=  2.9787652194e-02
       coeff_l3(77,74)=  4.0202534199e-01
       coeff_l3(77,75)= -3.2762803137e-02
       coeff_l3(77,76)= -8.1628508866e-02
       coeff_l3(77,77)= -2.6427924633e-01
       coeff_l3(77,78)= -6.7348539829e-02
       coeff_l3(77,79)= -1.6808586195e-02
       coeff_l3(77,80)= -4.6870043874e-01
       coeff_l3(77,81)=  2.4302919209e-01
       coeff_l3(77,82)=  7.8128501773e-02
       coeff_l3(77,83)=  4.2555424571e-01
       coeff_l3(77,84)=  2.3962074518e-01
       coeff_l3(77,85)= -1.3974508643e-01
       coeff_l3(77,86)= -3.4525942802e-01
       coeff_l3(77,87)= -3.8081027567e-02
       coeff_l3(77,88)=  4.7129037976e-01
       coeff_l3(77,89)= -4.9383905530e-01
       coeff_l3(77,90)= -2.1414741874e-01
       coeff_l3(77,91)= -8.5814960301e-02
       coeff_l3(77,92)= -1.6363428533e-01
       coeff_l3(77,93)= -1.4686948061e-01
       coeff_l3(77,94)= -4.2503902316e-01
       coeff_l3(77,95)= -2.1791572869e-01
       coeff_l3(77,96)= -4.1654998064e-01
       coeff_l3(77,97)= -2.3359011114e-01
       coeff_l3(77,98)=  1.0810893029e-01
       coeff_l3(77,99)= -7.1396611631e-02
       coeff_l3(77,100)= -2.5617983937e-01
       coeff_l3(77,101)=  1.7308558524e-01
       coeff_l3(77,102)= -3.8969960809e-01
       coeff_l3(77,103)=  2.6544594765e-01
       coeff_l3(77,104)= -1.4619499445e-01
       coeff_l3(77,105)=  6.7160435021e-02
       coeff_l3(77,106)= -2.9371950775e-02
       coeff_l3(77,107)= -5.2623517811e-02
       coeff_l3(77,108)= -2.0426170528e-01
       coeff_l3(77,109)= -2.0749489963e-01
       coeff_l3(77,110)=  7.6650805771e-02
       coeff_l3(77,111)= -1.2071008235e-01
       coeff_l3(77,112)=  7.0854411460e-03
       coeff_l3(77,113)= -5.4775553942e-01
       coeff_l3(77,114)=  1.2483747303e-01
       coeff_l3(77,115)=  2.4300435558e-02
       coeff_l3(77,116)=  8.5625737906e-01
       coeff_l3(77,117)= -1.0141738653e+00
       coeff_l3(77,118)=  3.9574491978e-01
       coeff_l3(77,119)= -7.0441335440e-02
       coeff_l3(77,120)= -1.1371693760e-01
       coeff_l3(77,121)=  1.0621903464e-02
       coeff_l3(77,122)= -2.1613804996e-01
       coeff_l3(77,123)= -2.4802194536e-01
       coeff_l3(77,124)= -5.3134614229e-01
       coeff_l3(77,125)= -2.6381585002e-01
       coeff_l3(78,1)= -2.5082382560e-01
       coeff_l3(78,2)= -8.0845035613e-02
       coeff_l3(78,3)=  6.2797077000e-02
       coeff_l3(78,4)= -5.7730622590e-02
       coeff_l3(78,5)= -6.0545474291e-02
       coeff_l3(78,6)=  6.9104813039e-02
       coeff_l3(78,7)=  1.5022856183e-02
       coeff_l3(78,8)= -2.4749974906e-01
       coeff_l3(78,9)= -1.0272379220e-01
       coeff_l3(78,10)= -1.0012255609e-01
       coeff_l3(78,11)=  6.6790260375e-02
       coeff_l3(78,12)= -3.4687015414e-01
       coeff_l3(78,13)= -3.2135745883e-01
       coeff_l3(78,14)= -2.6119353250e-02
       coeff_l3(78,15)=  2.2783361375e-02
       coeff_l3(78,16)=  4.7273077071e-02
       coeff_l3(78,17)= -9.7014464438e-02
       coeff_l3(78,18)= -4.7247949243e-01
       coeff_l3(78,19)= -3.0078148469e-02
       coeff_l3(78,20)=  1.1753479019e-02
       coeff_l3(78,21)= -2.1631744504e-01
       coeff_l3(78,22)=  1.3978937268e-01
       coeff_l3(78,23)=  8.2501515746e-02
       coeff_l3(78,24)= -1.0244152695e-01
       coeff_l3(78,25)=  8.5351876915e-02
       coeff_l3(78,26)= -8.7987743318e-02
       coeff_l3(78,27)=  8.6470767856e-02
       coeff_l3(78,28)=  5.8434218168e-02
       coeff_l3(78,29)= -7.0387870073e-01
       coeff_l3(78,30)=  2.8981531505e-03
       coeff_l3(78,31)= -4.0987439454e-02
       coeff_l3(78,32)=  7.0932218805e-03
       coeff_l3(78,33)= -1.7542649806e-01
       coeff_l3(78,34)=  9.5901049674e-02
       coeff_l3(78,35)= -1.0322999209e-01
       coeff_l3(78,36)=  7.2322107852e-02
       coeff_l3(78,37)=  1.4055487514e-01
       coeff_l3(78,38)= -8.3686244488e-01
       coeff_l3(78,39)= -6.4678207040e-02
       coeff_l3(78,40)= -1.5350238979e-01
       coeff_l3(78,41)= -6.0644274950e-01
       coeff_l3(78,42)=  2.2088536620e-01
       coeff_l3(78,43)= -6.7389510572e-02
       coeff_l3(78,44)=  1.0276038200e-01
       coeff_l3(78,45)= -1.0330533609e-02
       coeff_l3(78,46)= -2.1871410310e-01
       coeff_l3(78,47)= -1.7643718049e-02
       coeff_l3(78,48)= -1.0329874605e-01
       coeff_l3(78,49)= -3.7435963750e-01
       coeff_l3(78,50)= -1.5561898053e-01
       coeff_l3(78,51)= -7.7624909580e-02
       coeff_l3(78,52)=  1.1823614687e-01
       coeff_l3(78,53)=  4.2450550944e-02
       coeff_l3(78,54)= -1.5892554820e-01
       coeff_l3(78,55)= -8.7750665843e-02
       coeff_l3(78,56)= -1.6266585886e-01
       coeff_l3(78,57)=  4.7628935426e-02
       coeff_l3(78,58)=  1.1101792008e-01
       coeff_l3(78,59)= -3.9805471897e-02
       coeff_l3(78,60)= -2.1144109964e-01
       coeff_l3(78,61)= -4.5339889824e-02
       coeff_l3(78,62)=  7.0189028978e-02
       coeff_l3(78,63)= -1.6564670950e-02
       coeff_l3(78,64)=  1.1999554932e-02
       coeff_l3(78,65)=  1.5090471506e-01
       coeff_l3(78,66)=  7.2739832103e-02
       coeff_l3(78,67)= -5.4449969530e-01
       coeff_l3(78,68)= -2.4396459758e-01
       coeff_l3(78,69)= -2.5422686711e-02
       coeff_l3(78,70)=  1.1266700923e-01
       coeff_l3(78,71)= -5.5305445194e-01
       coeff_l3(78,72)=  2.1240571141e-01
       coeff_l3(78,73)= -2.5745274499e-02
       coeff_l3(78,74)= -4.1164409369e-02
       coeff_l3(78,75)= -5.9523753822e-02
       coeff_l3(78,76)= -2.0986478031e-01
       coeff_l3(78,77)=  2.6683202013e-02
       coeff_l3(78,78)= -1.0958311707e-01
       coeff_l3(78,79)=  1.6877788305e-01
       coeff_l3(78,80)= -4.9692731351e-02
       coeff_l3(78,81)= -2.7575358748e-02
       coeff_l3(78,82)= -7.4682824314e-02
       coeff_l3(78,83)=  1.1577777565e-01
       coeff_l3(78,84)=  1.3842536509e-01
       coeff_l3(78,85)=  8.5100978613e-02
       coeff_l3(78,86)=  5.6919846684e-02
       coeff_l3(78,87)= -5.6774079800e-02
       coeff_l3(78,88)= -9.3583934009e-02
       coeff_l3(78,89)= -1.4580017328e-01
       coeff_l3(78,90)= -6.3024811447e-02
       coeff_l3(78,91)= -6.9312013686e-02
       coeff_l3(78,92)=  1.2797181308e-01
       coeff_l3(78,93)= -2.7337469161e-02
       coeff_l3(78,94)= -1.2184800953e-01
       coeff_l3(78,95)= -8.6420923471e-02
       coeff_l3(78,96)= -1.2332123518e-01
       coeff_l3(78,97)= -1.0372312367e-01
       coeff_l3(78,98)=  1.5916554630e-01
       coeff_l3(78,99)=  1.4102102257e-02
       coeff_l3(78,100)= -1.9604729116e-01
       coeff_l3(78,101)= -1.3860914111e-01
       coeff_l3(78,102)=  8.5468493402e-02
       coeff_l3(78,103)=  2.2431451827e-02
       coeff_l3(78,104)= -9.6380732954e-02
       coeff_l3(78,105)= -5.0682169199e-01
       coeff_l3(78,106)= -1.4391274750e-01
       coeff_l3(78,107)= -2.7781382203e-01
       coeff_l3(78,108)= -1.4595092833e-01
       coeff_l3(78,109)=  1.1335614324e-01
       coeff_l3(78,110)=  1.3950249553e-01
       coeff_l3(78,111)= -1.4528702199e-01
       coeff_l3(78,112)= -3.8555899262e-01
       coeff_l3(78,113)= -4.0965893865e-01
       coeff_l3(78,114)=  1.3757033646e-01
       coeff_l3(78,115)= -5.3682023287e-01
       coeff_l3(78,116)= -2.8929162025e-01
       coeff_l3(78,117)= -2.5935596228e-01
       coeff_l3(78,118)= -3.3395880461e-01
       coeff_l3(78,119)= -4.0810378268e-03
       coeff_l3(78,120)= -2.0363634825e-01
       coeff_l3(78,121)= -1.4460286498e-01
       coeff_l3(78,122)=  8.9056238532e-02
       coeff_l3(78,123)=  1.4648653567e-01
       coeff_l3(78,124)=  1.3936122879e-02
       coeff_l3(78,125)=  1.4421282709e-01
       coeff_l3(79,1)=  2.6595425606e-01
       coeff_l3(79,2)= -3.9716005325e-01
       coeff_l3(79,3)=  4.1955441236e-02
       coeff_l3(79,4)=  3.0681535602e-01
       coeff_l3(79,5)= -1.7297573388e-01
       coeff_l3(79,6)= -1.2439040840e-01
       coeff_l3(79,7)= -2.6387834549e-01
       coeff_l3(79,8)=  3.6093536019e-01
       coeff_l3(79,9)=  6.8461723626e-02
       coeff_l3(79,10)=  2.0886108279e-01
       coeff_l3(79,11)=  1.5197971463e-01
       coeff_l3(79,12)= -1.2977726758e-01
       coeff_l3(79,13)=  3.7610810250e-02
       coeff_l3(79,14)= -2.2035878897e-01
       coeff_l3(79,15)=  1.1431995034e-01
       coeff_l3(79,16)=  1.4314037561e-01
       coeff_l3(79,17)=  2.3166264594e-01
       coeff_l3(79,18)=  2.4925205112e-01
       coeff_l3(79,19)=  3.3554455638e-01
       coeff_l3(79,20)=  3.5844916105e-01
       coeff_l3(79,21)=  3.5407111049e-01
       coeff_l3(79,22)=  6.9321114570e-03
       coeff_l3(79,23)=  8.0910697579e-02
       coeff_l3(79,24)=  2.2843737900e-01
       coeff_l3(79,25)=  7.5938481092e-01
       coeff_l3(79,26)=  2.9198655486e-01
       coeff_l3(79,27)= -7.8153973818e-01
       coeff_l3(79,28)= -7.4750810862e-01
       coeff_l3(79,29)=  3.7859287858e-01
       coeff_l3(79,30)=  4.1669782996e-01
       coeff_l3(79,31)=  4.0744742751e-01
       coeff_l3(79,32)=  6.3162259758e-02
       coeff_l3(79,33)=  2.6430694386e-02
       coeff_l3(79,34)= -1.7921067774e-01
       coeff_l3(79,35)=  1.2633603811e-01
       coeff_l3(79,36)= -2.0295260847e-01
       coeff_l3(79,37)= -1.7167657614e-02
       coeff_l3(79,38)=  8.6084060371e-02
       coeff_l3(79,39)=  8.4995225072e-02
       coeff_l3(79,40)=  3.5378858447e-02
       coeff_l3(79,41)=  2.6420679688e-01
       coeff_l3(79,42)= -2.4576778524e-03
       coeff_l3(79,43)=  7.6044268906e-02
       coeff_l3(79,44)=  3.2839018106e-01
       coeff_l3(79,45)=  2.0494754612e-01
       coeff_l3(79,46)=  4.0317103267e-01
       coeff_l3(79,47)= -2.2604132071e-02
       coeff_l3(79,48)= -2.3041386157e-02
       coeff_l3(79,49)=  4.8716135025e+00
       coeff_l3(79,50)=  2.9529210925e-01
       coeff_l3(79,51)=  2.4609901011e-01
       coeff_l3(79,52)=  2.0390982926e-01
       coeff_l3(79,53)=  1.9557395577e-01
       coeff_l3(79,54)=  7.0319873095e-01
       coeff_l3(79,55)=  4.4816780090e-01
       coeff_l3(79,56)= -5.7595193386e-02
       coeff_l3(79,57)= -1.0388954729e-01
       coeff_l3(79,58)=  7.7280461788e-01
       coeff_l3(79,59)=  2.3781934381e-01
       coeff_l3(79,60)=  1.0431005955e+00
       coeff_l3(79,61)=  3.2618243247e-02
       coeff_l3(79,62)= -1.2615603209e-01
       coeff_l3(79,63)=  9.6233373880e-01
       coeff_l3(79,64)=  1.9358526170e-01
       coeff_l3(79,65)= -1.9656093419e-01
       coeff_l3(79,66)=  1.7079472542e-02
       coeff_l3(79,67)=  1.6575852036e-01
       coeff_l3(79,68)=  8.5202492774e-02
       coeff_l3(79,69)=  1.8522624671e-01
       coeff_l3(79,70)= -5.9206608683e-02
       coeff_l3(79,71)=  9.1804824769e-02
       coeff_l3(79,72)= -1.5387465060e-01
       coeff_l3(79,73)= -1.6846281290e-01
       coeff_l3(79,74)=  1.1437872052e-01
       coeff_l3(79,75)=  3.3151254058e-01
       coeff_l3(79,76)=  2.9862453938e+00
       coeff_l3(79,77)=  3.7553060055e-01
       coeff_l3(79,78)=  1.7992883921e-01
       coeff_l3(79,79)=  1.0655896366e-01
       coeff_l3(79,80)=  2.0419943333e+00
       coeff_l3(79,81)=  3.0504032969e-01
       coeff_l3(79,82)= -2.0969504118e-01
       coeff_l3(79,83)=  1.8357259035e-01
       coeff_l3(79,84)= -2.7467471361e-01
       coeff_l3(79,85)=  2.6480266452e-01
       coeff_l3(79,86)= -1.5682581067e-01
       coeff_l3(79,87)= -1.4230051637e-01
       coeff_l3(79,88)=  8.2026608288e-03
       coeff_l3(79,89)=  4.2479622364e-01
       coeff_l3(79,90)=  2.0023103058e-01
       coeff_l3(79,91)=  5.8370357752e-01
       coeff_l3(79,92)= -8.2006968558e-02
       coeff_l3(79,93)= -8.1182554364e-02
       coeff_l3(79,94)= -2.0567893982e-02
       coeff_l3(79,95)= -2.8811895847e-01
       coeff_l3(79,96)= -1.2931936979e-01
       coeff_l3(79,97)= -2.5340339541e-01
       coeff_l3(79,98)=  1.9187340140e-01
       coeff_l3(79,99)= -5.3988528252e-01
       coeff_l3(79,100)= -3.1359088421e-01
       coeff_l3(79,101)=  4.5249322057e-01
       coeff_l3(79,102)= -3.9925292134e-01
       coeff_l3(79,103)=  1.8153410405e-02
       coeff_l3(79,104)=  5.5283628404e-02
       coeff_l3(79,105)= -1.5099473298e-01
       coeff_l3(79,106)=  2.8895288706e-01
       coeff_l3(79,107)= -2.0294207335e-01
       coeff_l3(79,108)= -1.0105276108e-01
       coeff_l3(79,109)=  7.1027837694e-03
       coeff_l3(79,110)= -1.9387546182e-01
       coeff_l3(79,111)= -1.2737217546e-01
       coeff_l3(79,112)= -1.7719331384e-01
       coeff_l3(79,113)=  1.7050057650e-01
       coeff_l3(79,114)=  1.0265630484e+00
       coeff_l3(79,115)=  1.2852616608e-01
       coeff_l3(79,116)= -1.3334499300e-01
       coeff_l3(79,117)=  5.2238380909e-01
       coeff_l3(79,118)=  9.1396041214e-02
       coeff_l3(79,119)= -1.5506269038e-01
       coeff_l3(79,120)= -2.6157227159e-01
       coeff_l3(79,121)=  1.2445646524e-01
       coeff_l3(79,122)= -1.3809560239e-01
       coeff_l3(79,123)= -2.1722047031e-01
       coeff_l3(79,124)= -2.4778330699e-02
       coeff_l3(79,125)= -1.8089194596e-01
       coeff_l3(80,1)=  2.3365636170e-01
       coeff_l3(80,2)= -5.1254826784e-01
       coeff_l3(80,3)=  3.0919715762e-01
       coeff_l3(80,4)=  6.5799587965e-01
       coeff_l3(80,5)=  1.3239064813e-01
       coeff_l3(80,6)= -3.2194408774e-01
       coeff_l3(80,7)= -1.7048196793e+00
       coeff_l3(80,8)= -1.0094327480e-01
       coeff_l3(80,9)= -2.8087845445e-01
       coeff_l3(80,10)=  9.0187668800e-02
       coeff_l3(80,11)=  3.9857789874e-02
       coeff_l3(80,12)=  6.8618547916e-01
       coeff_l3(80,13)=  2.4524451792e-01
       coeff_l3(80,14)= -3.1972616911e-01
       coeff_l3(80,15)=  3.5419408232e-02
       coeff_l3(80,16)= -9.6401482821e-02
       coeff_l3(80,17)=  1.0164203495e-01
       coeff_l3(80,18)=  8.9505189657e-01
       coeff_l3(80,19)= -2.1196311712e-01
       coeff_l3(80,20)= -2.7208083868e-01
       coeff_l3(80,21)= -1.9056916237e-02
       coeff_l3(80,22)= -3.3417702653e-03
       coeff_l3(80,23)=  1.7987471074e-02
       coeff_l3(80,24)= -4.1375353932e-01
       coeff_l3(80,25)= -2.7198645473e-01
       coeff_l3(80,26)= -2.6336482167e-01
       coeff_l3(80,27)= -2.3191492558e+00
       coeff_l3(80,28)= -2.1148266792e+00
       coeff_l3(80,29)=  5.7174640894e-01
       coeff_l3(80,30)=  7.8991621733e-01
       coeff_l3(80,31)= -1.0917045921e-01
       coeff_l3(80,32)= -1.7635418475e-01
       coeff_l3(80,33)=  1.3051697612e-01
       coeff_l3(80,34)= -1.6457177699e-01
       coeff_l3(80,35)=  1.8939262629e-01
       coeff_l3(80,36)= -1.8110775948e-01
       coeff_l3(80,37)= -2.5731673837e-01
       coeff_l3(80,38)=  1.2815976143e+00
       coeff_l3(80,39)= -1.4786753058e-01
       coeff_l3(80,40)=  1.1693239212e-01
       coeff_l3(80,41)=  8.6055433750e-01
       coeff_l3(80,42)= -3.2793116570e-01
       coeff_l3(80,43)=  1.0063265264e-01
       coeff_l3(80,44)= -5.2477478981e-02
       coeff_l3(80,45)= -5.9370166063e-01
       coeff_l3(80,46)= -2.5253534317e-01
       coeff_l3(80,47)=  2.1878427267e-01
       coeff_l3(80,48)=  3.0980095267e-01
       coeff_l3(80,49)= -4.8959463835e-01
       coeff_l3(80,50)=  4.8040699959e-01
       coeff_l3(80,51)= -3.6749735475e-01
       coeff_l3(80,52)= -3.7446987699e-04
       coeff_l3(80,53)= -1.9405505061e-01
       coeff_l3(80,54)= -1.0136438608e+00
       coeff_l3(80,55)=  1.3397035189e-02
       coeff_l3(80,56)= -4.8274286091e-02
       coeff_l3(80,57)= -4.4186469167e-02
       coeff_l3(80,58)=  7.2966820002e-01
       coeff_l3(80,59)=  8.2266949117e-02
       coeff_l3(80,60)=  2.8875746727e+00
       coeff_l3(80,61)= -2.5691479445e-01
       coeff_l3(80,62)= -1.7981180549e-01
       coeff_l3(80,63)= -5.6404763460e-01
       coeff_l3(80,64)= -1.0186898708e-01
       coeff_l3(80,65)= -3.7112152576e-01
       coeff_l3(80,66)= -2.7298945189e-01
       coeff_l3(80,67)=  6.7379313707e-01
       coeff_l3(80,68)= -8.3374530077e-02
       coeff_l3(80,69)=  1.4437384903e-01
       coeff_l3(80,70)= -4.3044602871e-01
       coeff_l3(80,71)=  9.2852091789e-01
       coeff_l3(80,72)= -6.7289523780e-02
       coeff_l3(80,73)= -1.1347813904e-01
       coeff_l3(80,74)=  1.5332435071e-01
       coeff_l3(80,75)= -8.7928883731e-02
       coeff_l3(80,76)= -4.6789079905e-02
       coeff_l3(80,77)=  7.6326234266e-03
       coeff_l3(80,78)= -2.6434889436e-01
       coeff_l3(80,79)=  1.1080523580e-01
       coeff_l3(80,80)= -5.6300169230e-01
       coeff_l3(80,81)=  3.3955439925e-01
       coeff_l3(80,82)=  1.8040205538e-01
       coeff_l3(80,83)=  5.4829411209e-02
       coeff_l3(80,84)=  9.6452414989e-02
       coeff_l3(80,85)=  3.6720100790e-02
       coeff_l3(80,86)= -2.3130562156e-02
       coeff_l3(80,87)= -5.7961389422e-02
       coeff_l3(80,88)=  7.1078294516e-01
       coeff_l3(80,89)=  6.7792671919e-01
       coeff_l3(80,90)= -7.9825118184e-02
       coeff_l3(80,91)= -7.4191987514e-02
       coeff_l3(80,92)= -9.1308742762e-02
       coeff_l3(80,93)=  8.1007607281e-02
       coeff_l3(80,94)= -2.6294425130e-01
       coeff_l3(80,95)= -1.2998124957e-01
       coeff_l3(80,96)=  1.1532603949e-01
       coeff_l3(80,97)= -2.9620915651e-01
       coeff_l3(80,98)= -9.7224369645e-02
       coeff_l3(80,99)=  5.2707684040e-01
       coeff_l3(80,100)= -3.8602616638e-02
       coeff_l3(80,101)=  3.1192126870e-01
       coeff_l3(80,102)= -4.6735832095e-01
       coeff_l3(80,103)= -1.9336615503e-01
       coeff_l3(80,104)= -2.5116866827e-01
       coeff_l3(80,105)=  8.4265601635e-01
       coeff_l3(80,106)=  1.6381397843e-01
       coeff_l3(80,107)=  2.1229666471e-01
       coeff_l3(80,108)= -3.0929185450e-02
       coeff_l3(80,109)= -2.4152854085e-01
       coeff_l3(80,110)= -8.1323817372e-02
       coeff_l3(80,111)=  1.0757677257e-01
       coeff_l3(80,112)=  1.0432127714e+00
       coeff_l3(80,113)=  1.5815949440e-01
       coeff_l3(80,114)= -8.2380557060e-01
       coeff_l3(80,115)=  7.2117310762e-01
       coeff_l3(80,116)=  4.0794202685e-01
       coeff_l3(80,117)=  6.4723157883e-01
       coeff_l3(80,118)=  3.0972602963e-01
       coeff_l3(80,119)= -4.2378969491e-02
       coeff_l3(80,120)=  1.9278782606e-01
       coeff_l3(80,121)=  2.2093746066e-01
       coeff_l3(80,122)=  1.9720155746e-02
       coeff_l3(80,123)= -4.1597163677e-01
       coeff_l3(80,124)= -3.7949764729e-01
       coeff_l3(80,125)= -3.7003976107e-01
       coeff_l3(81,1)= -7.8934538364e-01
       coeff_l3(81,2)= -3.4159682691e-02
       coeff_l3(81,3)=  1.5883027017e-01
       coeff_l3(81,4)= -1.5662154555e-01
       coeff_l3(81,5)=  4.9531981349e-02
       coeff_l3(81,6)=  1.2355946749e-01
       coeff_l3(81,7)=  4.2267575860e-02
       coeff_l3(81,8)= -3.0477043986e-01
       coeff_l3(81,9)= -1.4969591051e-02
       coeff_l3(81,10)= -2.7359863743e-02
       coeff_l3(81,11)=  2.2392988205e-02
       coeff_l3(81,12)= -6.3061696291e-01
       coeff_l3(81,13)= -3.7461960316e-01
       coeff_l3(81,14)= -4.1912209243e-02
       coeff_l3(81,15)=  1.0764334351e-01
       coeff_l3(81,16)=  1.3880433142e-01
       coeff_l3(81,17)= -2.5394791365e-01
       coeff_l3(81,18)= -7.9480540752e-01
       coeff_l3(81,19)= -8.9660277590e-03
       coeff_l3(81,20)= -1.8333369493e-01
       coeff_l3(81,21)= -2.8260535002e-01
       coeff_l3(81,22)=  1.5181018412e-01
       coeff_l3(81,23)=  9.6819870174e-02
       coeff_l3(81,24)=  2.0734737813e-01
       coeff_l3(81,25)=  4.8400899395e-03
       coeff_l3(81,26)=  2.9530987144e-02
       coeff_l3(81,27)= -9.9748589098e-02
       coeff_l3(81,28)= -3.7475544959e-02
       coeff_l3(81,29)= -7.3027461767e-01
       coeff_l3(81,30)= -1.9109137356e-01
       coeff_l3(81,31)=  1.2856133282e-01
       coeff_l3(81,32)=  1.9465163350e-01
       coeff_l3(81,33)= -2.4865776300e-01
       coeff_l3(81,34)= -7.5341172516e-02
       coeff_l3(81,35)=  5.6253266521e-03
       coeff_l3(81,36)= -2.2045519948e-01
       coeff_l3(81,37)= -1.5636330470e-02
       coeff_l3(81,38)= -2.2718305886e-01
       coeff_l3(81,39)= -1.4052365720e-01
       coeff_l3(81,40)=  1.8105415627e-02
       coeff_l3(81,41)= -8.4355181456e-01
       coeff_l3(81,42)=  2.6938799024e-01
       coeff_l3(81,43)= -3.0285403132e-01
       coeff_l3(81,44)=  3.0214461684e-01
       coeff_l3(81,45)= -9.4125837088e-02
       coeff_l3(81,46)= -6.3016407192e-02
       coeff_l3(81,47)=  3.5513928160e-03
       coeff_l3(81,48)= -3.7846800685e-01
       coeff_l3(81,49)= -3.0761697888e-01
       coeff_l3(81,50)=  3.1454008073e-02
       coeff_l3(81,51)=  2.3693684489e-02
       coeff_l3(81,52)=  2.1701176465e-01
       coeff_l3(81,53)= -2.2763726115e-01
       coeff_l3(81,54)= -9.2273145914e-02
       coeff_l3(81,55)= -3.9641421288e-02
       coeff_l3(81,56)= -1.9684492052e-01
       coeff_l3(81,57)=  1.4522145689e-01
       coeff_l3(81,58)=  9.4375026226e-01
       coeff_l3(81,59)= -2.5296914577e-01
       coeff_l3(81,60)= -5.8948397636e-01
       coeff_l3(81,61)=  6.0320500284e-02
       coeff_l3(81,62)=  2.0223906636e-01
       coeff_l3(81,63)= -4.0319144726e-01
       coeff_l3(81,64)=  3.8008533418e-02
       coeff_l3(81,65)=  1.0740807652e-01
       coeff_l3(81,66)= -1.8752275035e-02
       coeff_l3(81,67)= -3.1937891245e-01
       coeff_l3(81,68)= -3.3182221651e-01
       coeff_l3(81,69)=  1.2245398946e-02
       coeff_l3(81,70)= -8.7601646781e-02
       coeff_l3(81,71)= -6.7538756132e-01
       coeff_l3(81,72)=  2.3107845336e-02
       coeff_l3(81,73)= -3.7979491055e-02
       coeff_l3(81,74)= -2.1928529441e-01
       coeff_l3(81,75)=  2.1954253316e-01
       coeff_l3(81,76)= -1.7890027165e-01
       coeff_l3(81,77)=  5.0203807652e-02
       coeff_l3(81,78)= -5.7460263371e-02
       coeff_l3(81,79)=  9.6656285226e-02
       coeff_l3(81,80)= -4.8258695006e-01
       coeff_l3(81,81)= -1.7520938814e-01
       coeff_l3(81,82)= -2.2725027800e-01
       coeff_l3(81,83)= -1.7463332415e-01
       coeff_l3(81,84)=  5.7300072163e-02
       coeff_l3(81,85)= -5.2123993635e-01
       coeff_l3(81,86)=  5.0452001393e-02
       coeff_l3(81,87)=  9.3934452161e-03
       coeff_l3(81,88)= -5.0777697563e-01
       coeff_l3(81,89)= -3.1391927600e-01
       coeff_l3(81,90)=  2.7073213458e-01
       coeff_l3(81,91)=  2.1230201423e-01
       coeff_l3(81,92)=  1.1280921847e-01
       coeff_l3(81,93)= -1.4518725872e-01
       coeff_l3(81,94)= -2.1492876112e-01
       coeff_l3(81,95)= -3.6281436682e-02
       coeff_l3(81,96)= -2.1125569940e-01
       coeff_l3(81,97)= -2.6419827715e-02
       coeff_l3(81,98)= -2.8353987262e-02
       coeff_l3(81,99)= -2.5412344933e-01
       coeff_l3(81,100)= -1.3044707477e-01
       coeff_l3(81,101)= -9.0988174081e-02
       coeff_l3(81,102)=  8.9571689023e-04
       coeff_l3(81,103)=  9.4711937010e-02
       coeff_l3(81,104)=  1.1539611965e-01
       coeff_l3(81,105)= -2.7153873444e-01
       coeff_l3(81,106)= -3.2683005929e-01
       coeff_l3(81,107)=  1.9599907100e-02
       coeff_l3(81,108)= -1.1528475583e-01
       coeff_l3(81,109)=  1.9250083715e-02
       coeff_l3(81,110)= -1.1216581613e-01
       coeff_l3(81,111)= -1.3108430803e-01
       coeff_l3(81,112)= -2.8874480724e-01
       coeff_l3(81,113)= -4.1730579734e-01
       coeff_l3(81,114)= -8.5357204080e-02
       coeff_l3(81,115)= -1.0700325668e-01
       coeff_l3(81,116)= -7.5176876783e-01
       coeff_l3(81,117)= -2.3065902293e-01
       coeff_l3(81,118)= -1.9266669452e-01
       coeff_l3(81,119)= -9.1139174998e-02
       coeff_l3(81,120)= -5.1990624517e-02
       coeff_l3(81,121)= -2.4778877199e-01
       coeff_l3(81,122)= -1.0552543215e-02
       coeff_l3(81,123)=  1.4357197285e-01
       coeff_l3(81,124)= -9.1878309846e-02
       coeff_l3(81,125)=  1.7502706498e-02
       coeff_l3(82,1)=  1.7012344301e-01
       coeff_l3(82,2)=  1.0754061863e-02
       coeff_l3(82,3)= -1.8280309439e-01
       coeff_l3(82,4)= -2.8045025468e-01
       coeff_l3(82,5)=  1.0185968131e-01
       coeff_l3(82,6)=  1.5289893746e-01
       coeff_l3(82,7)= -6.1603873968e-01
       coeff_l3(82,8)= -1.8473930657e-02
       coeff_l3(82,9)=  1.7424832284e-01
       coeff_l3(82,10)=  1.0793571919e-01
       coeff_l3(82,11)= -3.0344906449e-01
       coeff_l3(82,12)= -1.0430011749e+00
       coeff_l3(82,13)= -9.1371220350e-01
       coeff_l3(82,14)=  1.1013170332e-01
       coeff_l3(82,15)=  1.5749258921e-02
       coeff_l3(82,16)= -6.2715530396e-02
       coeff_l3(82,17)=  4.2052339762e-02
       coeff_l3(82,18)= -1.3530699015e+00
       coeff_l3(82,19)= -2.9005456716e-02
       coeff_l3(82,20)=  2.6830384135e-01
       coeff_l3(82,21)= -5.7214188576e-01
       coeff_l3(82,22)=  1.8864199519e-01
       coeff_l3(82,23)= -1.1434327811e-01
       coeff_l3(82,24)=  5.0218862295e-01
       coeff_l3(82,25)=  1.3155137189e-02
       coeff_l3(82,26)=  1.9967757165e-01
       coeff_l3(82,27)= -1.1963311434e+00
       coeff_l3(82,28)= -6.9527244568e-01
       coeff_l3(82,29)= -2.0167519152e-01
       coeff_l3(82,30)=  4.3228343129e-01
       coeff_l3(82,31)=  3.6488381028e-01
       coeff_l3(82,32)=  1.1590261012e-01
       coeff_l3(82,33)= -2.8925514221e-01
       coeff_l3(82,34)=  2.0731551573e-02
       coeff_l3(82,35)=  6.5626603365e-01
       coeff_l3(82,36)=  8.8583603501e-03
       coeff_l3(82,37)=  7.8731523827e-03
       coeff_l3(82,38)= -5.7782685757e-01
       coeff_l3(82,39)=  6.6280744970e-02
       coeff_l3(82,40)= -3.6397954822e-01
       coeff_l3(82,41)= -7.7583825588e-01
       coeff_l3(82,42)= -4.0936022997e-02
       coeff_l3(82,43)= -2.5997421145e-01
       coeff_l3(82,44)=  2.1797116101e-01
       coeff_l3(82,45)= -1.9615605474e-01
       coeff_l3(82,46)= -6.0032181442e-02
       coeff_l3(82,47)= -4.7003012151e-02
       coeff_l3(82,48)= -6.5044081211e-01
       coeff_l3(82,49)=  1.4110593796e+00
       coeff_l3(82,50)= -1.6545556486e-01
       coeff_l3(82,51)= -5.4595317692e-02
       coeff_l3(82,52)=  1.7048512399e-01
       coeff_l3(82,53)= -1.3237092644e-02
       coeff_l3(82,54)=  4.5761048794e-02
       coeff_l3(82,55)= -5.9607803822e-02
       coeff_l3(82,56)= -1.5460816026e-01
       coeff_l3(82,57)= -2.3587481678e-01
       coeff_l3(82,58)=  1.0031695664e-01
       coeff_l3(82,59)= -2.3663955927e-01
       coeff_l3(82,60)=  2.0889918804e+00
       coeff_l3(82,61)= -3.4728962928e-02
       coeff_l3(82,62)= -1.5372177958e-01
       coeff_l3(82,63)=  1.4453284442e-02
       coeff_l3(82,64)=  2.3702055216e-01
       coeff_l3(82,65)=  9.7076602280e-02
       coeff_l3(82,66)= -5.0484351814e-02
       coeff_l3(82,67)= -6.0109996796e-01
       coeff_l3(82,68)=  6.7712940276e-02
       coeff_l3(82,69)= -4.0661910176e-01
       coeff_l3(82,70)= -2.3780408502e-01
       coeff_l3(82,71)= -1.2472629547e+00
       coeff_l3(82,72)= -1.7618331313e-01
       coeff_l3(82,73)= -7.0193028450e-01
       coeff_l3(82,74)= -5.8336627483e-01
       coeff_l3(82,75)= -4.9409437925e-02
       coeff_l3(82,76)=  9.6905750036e-01
       coeff_l3(82,77)=  1.5984340012e-01
       coeff_l3(82,78)=  1.6472749412e-01
       coeff_l3(82,79)= -4.2957592756e-02
       coeff_l3(82,80)=  5.0766551495e-01
       coeff_l3(82,81)= -2.6819196343e-01
       coeff_l3(82,82)= -1.9844084978e-01
       coeff_l3(82,83)= -2.1710640192e-01
       coeff_l3(82,84)= -4.8105126619e-01
       coeff_l3(82,85)= -9.9404655397e-02
       coeff_l3(82,86)=  1.0302397609e-01
       coeff_l3(82,87)= -4.1628763080e-02
       coeff_l3(82,88)= -2.8697738051e-01
       coeff_l3(82,89)=  2.7707302570e-01
       coeff_l3(82,90)=  6.8253353238e-02
       coeff_l3(82,91)=  2.2526313365e-01
       coeff_l3(82,92)= -4.1558057070e-02
       coeff_l3(82,93)=  5.6445783377e-01
       coeff_l3(82,94)= -7.0231509209e-01
       coeff_l3(82,95)= -4.3466404080e-01
       coeff_l3(82,96)= -2.8735202551e-01
       coeff_l3(82,97)= -3.4681007266e-01
       coeff_l3(82,98)= -1.5789357945e-02
       coeff_l3(82,99)=  6.7208337784e-01
       coeff_l3(82,100)= -4.3433228135e-01
       coeff_l3(82,101)= -2.1219506860e-01
       coeff_l3(82,102)= -2.2995902598e-01
       coeff_l3(82,103)= -8.3248227835e-02
       coeff_l3(82,104)=  1.1189455539e-01
       coeff_l3(82,105)= -5.8512389660e-01
       coeff_l3(82,106)= -3.1098383665e-01
       coeff_l3(82,107)= -1.7507073283e-01
       coeff_l3(82,108)= -1.5616457164e-01
       coeff_l3(82,109)=  4.8170525581e-02
       coeff_l3(82,110)=  5.1984810270e-03
       coeff_l3(82,111)= -3.0768638849e-01
       coeff_l3(82,112)= -4.2233261466e-01
       coeff_l3(82,113)= -1.4064531028e-01
       coeff_l3(82,114)= -4.4203135371e-01
       coeff_l3(82,115)= -4.0076646209e-01
       coeff_l3(82,116)= -1.0876652002e+00
       coeff_l3(82,117)= -1.2214017659e-01
       coeff_l3(82,118)= -9.0951830149e-01
       coeff_l3(82,119)= -1.3981716335e-01
       coeff_l3(82,120)= -2.1749518812e-01
       coeff_l3(82,121)= -1.3864888251e-01
       coeff_l3(82,122)= -2.3772484064e-01
       coeff_l3(82,123)= -1.7921240628e-01
       coeff_l3(82,124)= -1.4912688732e-01
       coeff_l3(82,125)= -1.7155699432e-01
       coeff_l3(83,1)=  1.8133236468e-01
       coeff_l3(83,2)= -1.4870889485e-01
       coeff_l3(83,3)=  4.8802200705e-02
       coeff_l3(83,4)=  2.0791731775e-01
       coeff_l3(83,5)=  1.2326903641e-01
       coeff_l3(83,6)=  1.5162999928e-01
       coeff_l3(83,7)= -2.1484385431e-01
       coeff_l3(83,8)=  4.8959903419e-02
       coeff_l3(83,9)= -2.5396993756e-01
       coeff_l3(83,10)= -9.6564352512e-02
       coeff_l3(83,11)=  2.6083461940e-02
       coeff_l3(83,12)=  2.4770532548e-01
       coeff_l3(83,13)=  3.1727198511e-02
       coeff_l3(83,14)= -2.4902547896e-01
       coeff_l3(83,15)= -8.2887984812e-02
       coeff_l3(83,16)=  1.1547192931e-01
       coeff_l3(83,17)= -8.0407768488e-02
       coeff_l3(83,18)=  5.1353770494e-01
       coeff_l3(83,19)=  3.4249309450e-02
       coeff_l3(83,20)= -2.2310547531e-01
       coeff_l3(83,21)=  1.8875820562e-02
       coeff_l3(83,22)= -6.9222822785e-02
       coeff_l3(83,23)= -1.5664230287e-01
       coeff_l3(83,24)= -2.4166814983e-01
       coeff_l3(83,25)= -1.8515205383e-01
       coeff_l3(83,26)= -4.9486055970e-02
       coeff_l3(83,27)= -1.7534509301e-01
       coeff_l3(83,28)= -1.5424212813e-01
       coeff_l3(83,29)=  3.6696159840e-01
       coeff_l3(83,30)=  6.0502327979e-02
       coeff_l3(83,31)= -1.4457856119e-01
       coeff_l3(83,32)= -9.9332980812e-02
       coeff_l3(83,33)=  1.4814826846e-01
       coeff_l3(83,34)= -9.5936022699e-03
       coeff_l3(83,35)=  4.5180997252e-01
       coeff_l3(83,36)= -9.0229332447e-02
       coeff_l3(83,37)= -1.8573217094e-01
       coeff_l3(83,38)=  7.4944692850e-01
       coeff_l3(83,39)= -1.5996819735e-01
       coeff_l3(83,40)=  1.7014031112e-01
       coeff_l3(83,41)=  6.0579854250e-01
       coeff_l3(83,42)= -3.4399658442e-01
       coeff_l3(83,43)= -1.0659885406e-01
       coeff_l3(83,44)= -3.6021000147e-01
       coeff_l3(83,45)= -3.0056816339e-01
       coeff_l3(83,46)= -1.3432119787e-01
       coeff_l3(83,47)= -1.9824676216e-02
       coeff_l3(83,48)=  1.7639005184e-01
       coeff_l3(83,49)=  4.1829615831e-02
       coeff_l3(83,50)=  3.8069716096e-01
       coeff_l3(83,51)=  2.4906819686e-02
       coeff_l3(83,52)= -2.1454481408e-02
       coeff_l3(83,53)= -7.0217996836e-02
       coeff_l3(83,54)=  7.1444600821e-02
       coeff_l3(83,55)=  6.5699182451e-02
       coeff_l3(83,56)= -9.5104366541e-02
       coeff_l3(83,57)=  8.2565143704e-02
       coeff_l3(83,58)= -1.6354802251e-01
       coeff_l3(83,59)=  5.6416279078e-01
       coeff_l3(83,60)= -1.4980809391e-01
       coeff_l3(83,61)= -8.4048043936e-03
       coeff_l3(83,62)= -1.9234825671e-01
       coeff_l3(83,63)= -6.7248582840e-02
       coeff_l3(83,64)=  3.1879808754e-02
       coeff_l3(83,65)= -3.2313432544e-02
       coeff_l3(83,66)= -4.4869659469e-03
       coeff_l3(83,67)=  3.7248194218e-01
       coeff_l3(83,68)=  9.6183627844e-02
       coeff_l3(83,69)=  2.2816851735e-01
       coeff_l3(83,70)= -3.4992134571e-01
       coeff_l3(83,71)=  2.1625526249e-01
       coeff_l3(83,72)= -1.1026362330e-01
       coeff_l3(83,73)= -1.8293710053e-01
       coeff_l3(83,74)=  2.7293106914e-01
       coeff_l3(83,75)= -1.8586626649e-01
       coeff_l3(83,76)= -3.0406734347e-01
       coeff_l3(83,77)= -1.8380196765e-02
       coeff_l3(83,78)= -3.0877092481e-01
       coeff_l3(83,79)=  1.1911635846e-01
       coeff_l3(83,80)= -3.0121180415e-01
       coeff_l3(83,81)=  7.8710407019e-02
       coeff_l3(83,82)=  1.8473388255e-01
       coeff_l3(83,83)=  4.0993598104e-01
       coeff_l3(83,84)= -1.4032824337e-01
       coeff_l3(83,85)= -1.2957905233e-01
       coeff_l3(83,86)= -2.4042129517e-01
       coeff_l3(83,87)=  2.0689465106e-01
       coeff_l3(83,88)=  4.2138360441e-03
       coeff_l3(83,89)= -3.1268593669e-01
       coeff_l3(83,90)= -2.6218256354e-01
       coeff_l3(83,91)= -1.3672415912e-01
       coeff_l3(83,92)=  1.4076061547e-01
       coeff_l3(83,93)= -3.5664062947e-02
       coeff_l3(83,94)=  1.1727307923e-02
       coeff_l3(83,95)= -1.8741773069e-01
       coeff_l3(83,96)=  4.5086074620e-02
       coeff_l3(83,97)=  1.6084282100e-01
       coeff_l3(83,98)=  9.4958506525e-02
       coeff_l3(83,99)= -1.4064459503e-01
       coeff_l3(83,100)=  9.8708577454e-02
       coeff_l3(83,101)= -1.9606623054e-01
       coeff_l3(83,102)= -3.6542143673e-02
       coeff_l3(83,103)= -5.1527872682e-02
       coeff_l3(83,104)=  1.3950040564e-02
       coeff_l3(83,105)=  4.9694493413e-01
       coeff_l3(83,106)=  1.8535740674e-01
       coeff_l3(83,107)=  2.0439791679e-01
       coeff_l3(83,108)=  1.2382635474e-01
       coeff_l3(83,109)= -3.3456137776e-01
       coeff_l3(83,110)= -3.1342070550e-02
       coeff_l3(83,111)=  6.9252058864e-02
       coeff_l3(83,112)=  4.0940725803e-01
       coeff_l3(83,113)=  1.6458360478e-02
       coeff_l3(83,114)=  4.3611686677e-02
       coeff_l3(83,115)=  1.0002801567e-01
       coeff_l3(83,116)=  3.7012538314e-01
       coeff_l3(83,117)=  4.3111628294e-01
       coeff_l3(83,118)=  3.9205458760e-01
       coeff_l3(83,119)= -1.4830906689e-01
       coeff_l3(83,120)=  6.2498752028e-02
       coeff_l3(83,121)=  2.2436310351e-01
       coeff_l3(83,122)= -1.0394466668e-01
       coeff_l3(83,123)= -2.5001683831e-01
       coeff_l3(83,124)= -9.6088327467e-02
       coeff_l3(83,125)=  2.6036219671e-02
       coeff_l3(84,1)= -2.5170448422e-01
       coeff_l3(84,2)=  1.6274826229e-01
       coeff_l3(84,3)= -8.2467161119e-02
       coeff_l3(84,4)= -3.8842100650e-02
       coeff_l3(84,5)= -2.4806461297e-03
       coeff_l3(84,6)=  1.2755906582e-01
       coeff_l3(84,7)= -2.0367319882e-01
       coeff_l3(84,8)= -6.6453628242e-02
       coeff_l3(84,9)= -1.0377560556e-01
       coeff_l3(84,10)= -8.0732226372e-02
       coeff_l3(84,11)=  2.5929071009e-02
       coeff_l3(84,12)= -1.6840938479e-02
       coeff_l3(84,13)= -8.9193381369e-02
       coeff_l3(84,14)=  9.4054704532e-03
       coeff_l3(84,15)=  1.6224924475e-02
       coeff_l3(84,16)=  6.4989782870e-02
       coeff_l3(84,17)=  7.4214488268e-02
       coeff_l3(84,18)= -3.0897957087e-01
       coeff_l3(84,19)= -1.1484908313e-01
       coeff_l3(84,20)=  9.3958213925e-02
       coeff_l3(84,21)= -1.8111959100e-01
       coeff_l3(84,22)=  1.4309762046e-02
       coeff_l3(84,23)=  1.4484271407e-01
       coeff_l3(84,24)=  1.3507613540e-01
       coeff_l3(84,25)= -8.4203630686e-02
       coeff_l3(84,26)=  1.3971386850e-01
       coeff_l3(84,27)=  6.7339986563e-02
       coeff_l3(84,28)=  4.1832692921e-02
       coeff_l3(84,29)= -3.7225297093e-01
       coeff_l3(84,30)=  4.0328830481e-02
       coeff_l3(84,31)= -8.4413439035e-02
       coeff_l3(84,32)= -3.8543120027e-02
       coeff_l3(84,33)= -2.9694798868e-03
       coeff_l3(84,34)=  1.2300773524e-02
       coeff_l3(84,35)=  2.6402063668e-02
       coeff_l3(84,36)=  9.0389229357e-02
       coeff_l3(84,37)= -1.2115433812e-01
       coeff_l3(84,38)= -1.2058248371e-01
       coeff_l3(84,39)=  5.6333303452e-02
       coeff_l3(84,40)= -2.2761641070e-02
       coeff_l3(84,41)= -3.4211871028e-01
       coeff_l3(84,42)= -4.8316992819e-02
       coeff_l3(84,43)= -1.6042138636e-01
       coeff_l3(84,44)= -3.9678895846e-03
       coeff_l3(84,45)= -5.7457152754e-02
       coeff_l3(84,46)= -1.9612073898e-02
       coeff_l3(84,47)=  3.8062676787e-02
       coeff_l3(84,48)= -8.3322815597e-02
       coeff_l3(84,49)= -4.6744534373e-01
       coeff_l3(84,50)= -9.4966605306e-02
       coeff_l3(84,51)=  1.0939567536e-01
       coeff_l3(84,52)=  1.3191968203e-01
       coeff_l3(84,53)=  3.6315280944e-02
       coeff_l3(84,54)= -1.4587457478e-01
       coeff_l3(84,55)= -1.1903978139e-01
       coeff_l3(84,56)= -1.9464111328e-01
       coeff_l3(84,57)= -7.5129084289e-02
       coeff_l3(84,58)= -5.1683522761e-02
       coeff_l3(84,59)= -9.0960249305e-02
       coeff_l3(84,60)= -1.9219709560e-02
       coeff_l3(84,61)=  5.9305105358e-02
       coeff_l3(84,62)= -5.5761303753e-02
       coeff_l3(84,63)= -6.7772001028e-02
       coeff_l3(84,64)= -5.5296845734e-02
       coeff_l3(84,65)= -4.4987604022e-02
       coeff_l3(84,66)= -1.4256794751e-01
       coeff_l3(84,67)=  3.4723643214e-02
       coeff_l3(84,68)=  5.2774280310e-02
       coeff_l3(84,69)=  6.5738797188e-02
       coeff_l3(84,70)=  1.4835239947e-01
       coeff_l3(84,71)= -1.7219422758e-01
       coeff_l3(84,72)= -6.0364678502e-02
       coeff_l3(84,73)= -1.4559251070e-01
       coeff_l3(84,74)= -1.8148284405e-02
       coeff_l3(84,75)=  1.2844158709e-01
       coeff_l3(84,76)= -4.0015286207e-01
       coeff_l3(84,77)= -1.1977550387e-01
       coeff_l3(84,78)=  1.2737248838e-01
       coeff_l3(84,79)=  1.0106667876e-02
       coeff_l3(84,80)= -2.0403903723e-01
       coeff_l3(84,81)= -2.4614313245e-01
       coeff_l3(84,82)= -5.6485917419e-02
       coeff_l3(84,83)= -8.3418473601e-02
       coeff_l3(84,84)=  1.3504490256e-02
       coeff_l3(84,85)=  1.3582982123e-01
       coeff_l3(84,86)=  3.5168092698e-02
       coeff_l3(84,87)= -1.3657127321e-01
       coeff_l3(84,88)= -1.8923752010e-01
       coeff_l3(84,89)=  1.0606919974e-01
       coeff_l3(84,90)= -5.6605760008e-02
       coeff_l3(84,91)= -4.8809658736e-02
       coeff_l3(84,92)= -1.4030577242e-01
       coeff_l3(84,93)=  3.6685813218e-02
       coeff_l3(84,94)=  9.7839690745e-02
       coeff_l3(84,95)=  2.4913156405e-02
       coeff_l3(84,96)=  2.4000568315e-02
       coeff_l3(84,97)=  6.0262825340e-02
       coeff_l3(84,98)=  1.3282106817e-01
       coeff_l3(84,99)=  5.3754586726e-02
       coeff_l3(84,100)=  5.5794361979e-02
       coeff_l3(84,101)= -1.8415562809e-01
       coeff_l3(84,102)= -9.3451388180e-02
       coeff_l3(84,103)= -7.4493125081e-02
       coeff_l3(84,104)=  1.2179730833e-01
       coeff_l3(84,105)= -2.4918179214e-01
       coeff_l3(84,106)=  2.4065056816e-02
       coeff_l3(84,107)=  1.9754560664e-02
       coeff_l3(84,108)= -1.3759618998e-01
       coeff_l3(84,109)=  5.7265583426e-02
       coeff_l3(84,110)=  4.2960092425e-02
       coeff_l3(84,111)= -9.9000371993e-02
       coeff_l3(84,112)=  6.8656378426e-03
       coeff_l3(84,113)= -2.0875094831e-01
       coeff_l3(84,114)= -8.1308789551e-02
       coeff_l3(84,115)= -1.0127132386e-01
       coeff_l3(84,116)= -3.3210238814e-01
       coeff_l3(84,117)= -1.1231576651e-01
       coeff_l3(84,118)=  2.7322186157e-02
       coeff_l3(84,119)= -1.3889236376e-02
       coeff_l3(84,120)= -6.3045464456e-02
       coeff_l3(84,121)= -2.2612577304e-02
       coeff_l3(84,122)= -1.1353889108e-01
       coeff_l3(84,123)= -8.6660586298e-02
       coeff_l3(84,124)=  1.3454599679e-01
       coeff_l3(84,125)= -7.0936933160e-02
       coeff_l3(85,1)= -8.2265818119e-01
       coeff_l3(85,2)=  4.3699610978e-02
       coeff_l3(85,3)=  1.5502078831e-01
       coeff_l3(85,4)= -1.4083380997e-01
       coeff_l3(85,5)= -7.9867310822e-02
       coeff_l3(85,6)=  2.6886764169e-01
       coeff_l3(85,7)=  8.5913978517e-02
       coeff_l3(85,8)= -3.0163340271e-02
       coeff_l3(85,9)= -2.5594183244e-03
       coeff_l3(85,10)= -2.1984292567e-01
       coeff_l3(85,11)= -6.8094402552e-02
       coeff_l3(85,12)= -5.7478380203e-01
       coeff_l3(85,13)= -3.6858171225e-01
       coeff_l3(85,14)= -3.2156872749e-01
       coeff_l3(85,15)=  6.1511918902e-02
       coeff_l3(85,16)= -7.9647853971e-02
       coeff_l3(85,17)= -4.0776094794e-01
       coeff_l3(85,18)= -1.0858110189e+00
       coeff_l3(85,19)=  4.0799301118e-02
       coeff_l3(85,20)=  1.8228204921e-02
       coeff_l3(85,21)= -6.7626798153e-01
       coeff_l3(85,22)= -2.9382905364e-01
       coeff_l3(85,23)= -2.9072608799e-02
       coeff_l3(85,24)=  8.1548161805e-02
       coeff_l3(85,25)=  4.5510523021e-02
       coeff_l3(85,26)= -2.1795129776e-01
       coeff_l3(85,27)= -7.5312279165e-02
       coeff_l3(85,28)= -4.1282907128e-02
       coeff_l3(85,29)= -3.1277993321e-01
       coeff_l3(85,30)= -1.3076581061e-01
       coeff_l3(85,31)=  2.1443843842e-01
       coeff_l3(85,32)= -4.2261332273e-02
       coeff_l3(85,33)= -1.4120802283e-01
       coeff_l3(85,34)=  2.1389436722e-01
       coeff_l3(85,35)= -6.5615785122e-01
       coeff_l3(85,36)=  7.4389912188e-02
       coeff_l3(85,37)= -1.0444807261e-01
       coeff_l3(85,38)= -2.6304838061e-01
       coeff_l3(85,39)=  6.2579207122e-02
       coeff_l3(85,40)= -1.3952990063e-02
       coeff_l3(85,41)= -7.4236649275e-01
       coeff_l3(85,42)=  3.2312531024e-02
       coeff_l3(85,43)= -1.3619996607e-01
       coeff_l3(85,44)=  4.3965749443e-02
       coeff_l3(85,45)= -5.5757355690e-01
       coeff_l3(85,46)= -7.1689128876e-02
       coeff_l3(85,47)=  4.1000541300e-02
       coeff_l3(85,48)= -1.5081234276e-01
       coeff_l3(85,49)= -6.2560200691e-01
       coeff_l3(85,50)= -2.6678749919e-01
       coeff_l3(85,51)= -2.4608729873e-03
       coeff_l3(85,52)=  2.7423170209e-01
       coeff_l3(85,53)= -9.3962542713e-02
       coeff_l3(85,54)=  6.0047405958e-01
       coeff_l3(85,55)=  1.2173005939e-01
       coeff_l3(85,56)= -6.9283202291e-02
       coeff_l3(85,57)= -1.1896621436e-01
       coeff_l3(85,58)=  6.3397860527e-01
       coeff_l3(85,59)= -2.6118850708e-01
       coeff_l3(85,60)= -1.0176849365e+00
       coeff_l3(85,61)=  6.6297821701e-02
       coeff_l3(85,62)=  1.3559566438e-01
       coeff_l3(85,63)= -1.1220251769e-01
       coeff_l3(85,64)=  9.9560610950e-02
       coeff_l3(85,65)=  1.7277488112e-01
       coeff_l3(85,66)=  2.2125532851e-02
       coeff_l3(85,67)= -7.0473068953e-01
       coeff_l3(85,68)= -1.8487791717e-01
       coeff_l3(85,69)= -2.2593535483e-01
       coeff_l3(85,70)=  2.4069962092e-03
       coeff_l3(85,71)= -1.1901756525e+00
       coeff_l3(85,72)= -2.3862544447e-02
       coeff_l3(85,73)= -2.6692253351e-01
       coeff_l3(85,74)= -4.8936504126e-01
       coeff_l3(85,75)=  2.3265221715e-01
       coeff_l3(85,76)= -5.3709107637e-01
       coeff_l3(85,77)=  3.5450810194e-01
       coeff_l3(85,78)=  8.9387977496e-03
       coeff_l3(85,79)=  1.2058886886e-01
       coeff_l3(85,80)= -4.3495446444e-01
       coeff_l3(85,81)= -8.8393896818e-02
       coeff_l3(85,82)= -1.4548079669e-01
       coeff_l3(85,83)= -1.7590457201e-01
       coeff_l3(85,84)=  7.7928379178e-02
       coeff_l3(85,85)= -1.1801012792e-02
       coeff_l3(85,86)= -8.2597129047e-02
       coeff_l3(85,87)= -1.9303272665e-01
       coeff_l3(85,88)= -3.9810869098e-01
       coeff_l3(85,89)=  2.5382357836e-01
       coeff_l3(85,90)=  2.2629752755e-01
       coeff_l3(85,91)= -2.3745227605e-02
       coeff_l3(85,92)= -1.5333165228e-01
       coeff_l3(85,93)=  1.8872871995e-01
       coeff_l3(85,94)= -1.2806566060e-01
       coeff_l3(85,95)= -5.2614435554e-02
       coeff_l3(85,96)=  5.4843354970e-02
       coeff_l3(85,97)= -1.6126997769e-01
       coeff_l3(85,98)= -8.5428595543e-02
       coeff_l3(85,99)= -1.2204775214e-01
       coeff_l3(85,100)= -7.4409611523e-02
       coeff_l3(85,101)= -4.0150919557e-01
       coeff_l3(85,102)=  4.3643712997e-02
       coeff_l3(85,103)= -1.1701483279e-01
       coeff_l3(85,104)=  9.1927722096e-02
       coeff_l3(85,105)= -4.7346839309e-01
       coeff_l3(85,106)= -3.3764287829e-01
       coeff_l3(85,107)=  9.2967236415e-03
       coeff_l3(85,108)= -1.7361303791e-02
       coeff_l3(85,109)=  1.2696623802e-01
       coeff_l3(85,110)=  6.9740414619e-02
       coeff_l3(85,111)= -9.9871113896e-02
       coeff_l3(85,112)= -5.5246680975e-01
       coeff_l3(85,113)= -3.4919574857e-01
       coeff_l3(85,114)= -1.4662778378e-01
       coeff_l3(85,115)= -2.8687235713e-01
       coeff_l3(85,116)= -5.8300656080e-01
       coeff_l3(85,117)= -8.3099119365e-02
       coeff_l3(85,118)=  2.3422518745e-02
       coeff_l3(85,119)=  1.3847717643e-01
       coeff_l3(85,120)= -1.5194323659e-01
       coeff_l3(85,121)= -1.4825919271e-01
       coeff_l3(85,122)=  1.2925116718e-01
       coeff_l3(85,123)=  9.5416657627e-02
       coeff_l3(85,124)= -1.3623720407e-01
       coeff_l3(85,125)= -1.6807343811e-02
       coeff_l3(86,1)= -5.8944344521e-01
       coeff_l3(86,2)= -5.3467944264e-02
       coeff_l3(86,3)=  4.5171812177e-02
       coeff_l3(86,4)= -1.8880812824e-01
       coeff_l3(86,5)=  9.0407319367e-02
       coeff_l3(86,6)=  2.9689458013e-01
       coeff_l3(86,7)=  1.2569974363e-01
       coeff_l3(86,8)= -1.1609356850e-01
       coeff_l3(86,9)= -8.4900535643e-02
       coeff_l3(86,10)= -2.7180761099e-02
       coeff_l3(86,11)= -1.2422252446e-01
       coeff_l3(86,12)= -3.8392692804e-01
       coeff_l3(86,13)= -3.1194821000e-01
       coeff_l3(86,14)= -3.8780438900e-01
       coeff_l3(86,15)= -2.7463475242e-02
       coeff_l3(86,16)=  1.0599062592e-01
       coeff_l3(86,17)= -3.0395638943e-01
       coeff_l3(86,18)= -5.1509606838e-01
       coeff_l3(86,19)=  7.0410996675e-02
       coeff_l3(86,20)= -1.4224774204e-02
       coeff_l3(86,21)= -9.4795688987e-02
       coeff_l3(86,22)=  1.1948837340e-01
       coeff_l3(86,23)=  4.4092211872e-02
       coeff_l3(86,24)= -1.6515803337e-01
       coeff_l3(86,25)= -8.9688628912e-02
       coeff_l3(86,26)= -1.4293177985e-02
       coeff_l3(86,27)=  2.6037544012e-01
       coeff_l3(86,28)=  1.8791174889e-01
       coeff_l3(86,29)= -7.7875471115e-01
       coeff_l3(86,30)= -9.3044914305e-02
       coeff_l3(86,31)= -1.2346230447e-01
       coeff_l3(86,32)= -1.0892226547e-01
       coeff_l3(86,33)= -3.5957019776e-02
       coeff_l3(86,34)= -3.7766546011e-03
       coeff_l3(86,35)= -2.3164902627e-01
       coeff_l3(86,36)= -2.0341079682e-02
       coeff_l3(86,37)=  3.4011397511e-02
       coeff_l3(86,38)= -4.1869384050e-01
       coeff_l3(86,39)= -5.6678432971e-02
       coeff_l3(86,40)= -1.1143293232e-01
       coeff_l3(86,41)= -6.2194037437e-01
       coeff_l3(86,42)=  7.8104725108e-03
       coeff_l3(86,43)= -3.4284377098e-01
       coeff_l3(86,44)=  3.5213651136e-03
       coeff_l3(86,45)= -1.6493732110e-02
       coeff_l3(86,46)= -5.5336493254e-01
       coeff_l3(86,47)=  9.4228982925e-02
       coeff_l3(86,48)= -3.0581688881e-01
       coeff_l3(86,49)=  3.8957819343e-01
       coeff_l3(86,50)= -1.3415724039e-01
       coeff_l3(86,51)= -3.0354306102e-02
       coeff_l3(86,52)= -1.1498533189e-01
       coeff_l3(86,53)= -2.1429327130e-01
       coeff_l3(86,54)=  2.2604884207e-01
       coeff_l3(86,55)=  2.1642884612e-01
       coeff_l3(86,56)= -1.8062674999e-01
       coeff_l3(86,57)= -2.1993795037e-01
       coeff_l3(86,58)=  5.6297354400e-02
       coeff_l3(86,59)= -3.2456275821e-01
       coeff_l3(86,60)= -1.7367059365e-02
       coeff_l3(86,61)=  1.7208823562e-01
       coeff_l3(86,62)= -7.1108087897e-02
       coeff_l3(86,63)=  4.8680198193e-01
       coeff_l3(86,64)=  3.2698009163e-02
       coeff_l3(86,65)= -3.4058808815e-03
       coeff_l3(86,66)=  2.8917565942e-02
       coeff_l3(86,67)=  6.4725667238e-02
       coeff_l3(86,68)= -2.1788799763e-01
       coeff_l3(86,69)= -2.8499767184e-01
       coeff_l3(86,70)=  1.5097722411e-01
       coeff_l3(86,71)= -4.0048524737e-01
       coeff_l3(86,72)=  2.6671671867e-01
       coeff_l3(86,73)=  1.4983884990e-02
       coeff_l3(86,74)= -7.6124485349e-04
       coeff_l3(86,75)=  9.2502444983e-02
       coeff_l3(86,76)=  1.9522447884e-01
       coeff_l3(86,77)=  2.4585865438e-02
       coeff_l3(86,78)=  5.3008329123e-02
       coeff_l3(86,79)=  1.0544808954e-01
       coeff_l3(86,80)=  2.1912166476e-01
       coeff_l3(86,81)=  9.1388255358e-02
       coeff_l3(86,82)= -2.3479302227e-01
       coeff_l3(86,83)=  1.4263898134e-01
       coeff_l3(86,84)=  8.1935659051e-02
       coeff_l3(86,85)= -2.5179895759e-01
       coeff_l3(86,86)= -1.1560936272e-01
       coeff_l3(86,87)= -2.3714064155e-03
       coeff_l3(86,88)= -2.1574421227e-01
       coeff_l3(86,89)= -2.4354499578e-01
       coeff_l3(86,90)=  3.0628606677e-01
       coeff_l3(86,91)=  3.9882773161e-01
       coeff_l3(86,92)=  8.7337773293e-03
       coeff_l3(86,93)= -2.0033870637e-01
       coeff_l3(86,94)= -3.2635882497e-01
       coeff_l3(86,95)= -2.6449102163e-01
       coeff_l3(86,96)= -8.6860112846e-02
       coeff_l3(86,97)= -1.0338289291e-01
       coeff_l3(86,98)= -7.8189698979e-03
       coeff_l3(86,99)= -1.5387074649e-01
       coeff_l3(86,100)= -1.4353224635e-01
       coeff_l3(86,101)= -4.0462445468e-02
       coeff_l3(86,102)= -4.2780704796e-02
       coeff_l3(86,103)=  3.9726514369e-02
       coeff_l3(86,104)=  1.0919492692e-01
       coeff_l3(86,105)= -3.6046427488e-01
       coeff_l3(86,106)=  4.5588087291e-02
       coeff_l3(86,107)= -3.2716062665e-01
       coeff_l3(86,108)= -5.2164994180e-02
       coeff_l3(86,109)= -1.9087453187e-01
       coeff_l3(86,110)= -3.8829442114e-02
       coeff_l3(86,111)= -1.3322693110e-01
       coeff_l3(86,112)= -4.4106334448e-01
       coeff_l3(86,113)= -2.9590755701e-01
       coeff_l3(86,114)= -5.9682484716e-02
       coeff_l3(86,115)= -3.8497853279e-01
       coeff_l3(86,116)= -3.0811037868e-02
       coeff_l3(86,117)= -1.1690759659e-01
       coeff_l3(86,118)= -3.4686428308e-01
       coeff_l3(86,119)=  1.7322180793e-02
       coeff_l3(86,120)=  4.6176902950e-02
       coeff_l3(86,121)= -1.3103368878e-01
       coeff_l3(86,122)= -8.7644256651e-02
       coeff_l3(86,123)= -5.1303960383e-02
       coeff_l3(86,124)= -1.5171140432e-01
       coeff_l3(86,125)= -1.7398068681e-02
       coeff_l3(87,1)= -4.2916801572e-01
       coeff_l3(87,2)= -5.3280018270e-02
       coeff_l3(87,3)=  1.3293147087e-01
       coeff_l3(87,4)= -3.3299842477e-01
       coeff_l3(87,5)=  2.1924003959e-03
       coeff_l3(87,6)=  1.6938839853e-01
       coeff_l3(87,7)= -1.7441257834e-01
       coeff_l3(87,8)= -4.7380857170e-02
       coeff_l3(87,9)=  1.0718653351e-01
       coeff_l3(87,10)=  2.0518870652e-01
       coeff_l3(87,11)= -1.8787194788e-01
       coeff_l3(87,12)= -4.7412902117e-01
       coeff_l3(87,13)= -6.1783504486e-01
       coeff_l3(87,14)= -7.4138388038e-02
       coeff_l3(87,15)=  1.6338470578e-01
       coeff_l3(87,16)= -8.5561521351e-02
       coeff_l3(87,17)= -3.5520514846e-01
       coeff_l3(87,18)= -8.2359814644e-01
       coeff_l3(87,19)=  9.2655494809e-02
       coeff_l3(87,20)=  3.0796998739e-01
       coeff_l3(87,21)= -5.7563501596e-01
       coeff_l3(87,22)=  1.2900279462e-01
       coeff_l3(87,23)=  1.3551901281e-01
       coeff_l3(87,24)=  1.6050674021e-01
       coeff_l3(87,25)=  2.0008659363e-01
       coeff_l3(87,26)=  1.1848236620e-01
       coeff_l3(87,27)= -1.6421139240e-01
       coeff_l3(87,28)= -4.8325419426e-02
       coeff_l3(87,29)= -6.4221006632e-01
       coeff_l3(87,30)= -6.8828666210e-01
       coeff_l3(87,31)= -2.4409938604e-02
       coeff_l3(87,32)= -9.0693153441e-02
       coeff_l3(87,33)= -1.9729901850e-01
       coeff_l3(87,34)=  1.5180535614e-01
       coeff_l3(87,35)=  1.9781017303e-01
       coeff_l3(87,36)= -8.5305668414e-02
       coeff_l3(87,37)=  2.2668303922e-02
       coeff_l3(87,38)= -6.5071374178e-01
       coeff_l3(87,39)= -1.5630391240e-01
       coeff_l3(87,40)= -1.1761822551e-01
       coeff_l3(87,41)= -6.7883044481e-01
       coeff_l3(87,42)= -8.3098731935e-02
       coeff_l3(87,43)= -1.7976751924e-01
       coeff_l3(87,44)=  1.9527883828e-01
       coeff_l3(87,45)= -1.9893923402e-01
       coeff_l3(87,46)= -1.9196653366e-01
       coeff_l3(87,47)= -6.5552555025e-02
       coeff_l3(87,48)= -1.7982046306e-01
       coeff_l3(87,49)= -3.9534193277e-01
       coeff_l3(87,50)= -2.6593628526e-01
       coeff_l3(87,51)=  5.4819207639e-02
       coeff_l3(87,52)=  2.8620393947e-02
       coeff_l3(87,53)=  2.0591748878e-02
       coeff_l3(87,54)=  3.9175045490e-01
       coeff_l3(87,55)= -1.1636676267e-02
       coeff_l3(87,56)= -1.4663156867e-01
       coeff_l3(87,57)=  1.2219619006e-01
       coeff_l3(87,58)=  2.8703409433e-01
       coeff_l3(87,59)= -1.3454459608e-01
       coeff_l3(87,60)= -3.0500540137e-01
       coeff_l3(87,61)=  8.0773726106e-02
       coeff_l3(87,62)= -6.5600633621e-02
       coeff_l3(87,63)= -3.2002097368e-01
       coeff_l3(87,64)=  1.8567262590e-01
       coeff_l3(87,65)=  2.5549158454e-01
       coeff_l3(87,66)=  7.0260703564e-02
       coeff_l3(87,67)= -3.2130008936e-01
       coeff_l3(87,68)= -5.3521406651e-01
       coeff_l3(87,69)= -2.2735772654e-02
       coeff_l3(87,70)= -2.2050390020e-02
       coeff_l3(87,71)= -7.3166143894e-01
       coeff_l3(87,72)= -4.4469960034e-02
       coeff_l3(87,73)= -1.2878541648e-01
       coeff_l3(87,74)= -3.2072864473e-02
       coeff_l3(87,75)=  1.7976915836e-01
       coeff_l3(87,76)= -1.1838772148e-01
       coeff_l3(87,77)=  2.2126458585e-01
       coeff_l3(87,78)=  2.4593423307e-01
       coeff_l3(87,79)=  1.2957803905e-01
       coeff_l3(87,80)=  3.2484129071e-02
       coeff_l3(87,81)= -5.4606330395e-01
       coeff_l3(87,82)= -1.4119149745e-01
       coeff_l3(87,83)= -8.6907237768e-02
       coeff_l3(87,84)= -1.1783043295e-01
       coeff_l3(87,85)= -1.5925993025e-01
       coeff_l3(87,86)=  4.5711569488e-02
       coeff_l3(87,87)= -1.4211061597e-01
       coeff_l3(87,88)= -2.0685470104e-01
       coeff_l3(87,89)= -1.6745299101e-01
       coeff_l3(87,90)=  1.7422012985e-02
       coeff_l3(87,91)= -8.4593303502e-02
       coeff_l3(87,92)=  6.2955103815e-02
       coeff_l3(87,93)=  1.0208265483e-01
       coeff_l3(87,94)=  3.3125352114e-02
       coeff_l3(87,95)= -1.3681857288e-01
       coeff_l3(87,96)=  9.9922657013e-02
       coeff_l3(87,97)=  3.0948484316e-02
       coeff_l3(87,98)= -3.0280479789e-01
       coeff_l3(87,99)= -4.9042955041e-02
       coeff_l3(87,100)= -1.0850848258e-01
       coeff_l3(87,101)= -2.1556566656e-01
       coeff_l3(87,102)= -4.5735206455e-02
       coeff_l3(87,103)=  1.1102729477e-02
       coeff_l3(87,104)=  1.5094402432e-01
       coeff_l3(87,105)= -4.1770195961e-01
       coeff_l3(87,106)= -2.6326760650e-01
       coeff_l3(87,107)= -2.8035658598e-01
       coeff_l3(87,108)= -9.3159846961e-02
       coeff_l3(87,109)= -1.3232621364e-02
       coeff_l3(87,110)=  6.5222620964e-02
       coeff_l3(87,111)= -5.2754059434e-02
       coeff_l3(87,112)= -8.8153934479e-01
       coeff_l3(87,113)= -3.9052501321e-01
       coeff_l3(87,114)=  3.9447221160e-01
       coeff_l3(87,115)= -4.2877739668e-01
       coeff_l3(87,116)= -2.0171558857e-01
       coeff_l3(87,117)= -2.6010942459e-01
       coeff_l3(87,118)= -2.6609879732e-01
       coeff_l3(87,119)=  7.9020358622e-02
       coeff_l3(87,120)= -8.7212137878e-02
       coeff_l3(87,121)= -1.4482334256e-01
       coeff_l3(87,122)= -1.2306079268e-01
       coeff_l3(87,123)= -4.7012392431e-02
       coeff_l3(87,124)= -3.1072374433e-02
       coeff_l3(87,125)=  7.3605112731e-02
       coeff_l3(88,1)= -3.8426589966e-01
       coeff_l3(88,2)=  9.2593304813e-02
       coeff_l3(88,3)=  1.3712471724e-01
       coeff_l3(88,4)= -2.2612364590e-01
       coeff_l3(88,5)= -2.8453907371e-01
       coeff_l3(88,6)= -8.5420534015e-02
       coeff_l3(88,7)= -1.7306755483e-01
       coeff_l3(88,8)= -1.0518690944e-01
       coeff_l3(88,9)=  4.7748181969e-02
       coeff_l3(88,10)= -6.2804758549e-02
       coeff_l3(88,11)= -1.3166669011e-01
       coeff_l3(88,12)=  3.2375548035e-03
       coeff_l3(88,13)=  1.8813478947e-01
       coeff_l3(88,14)= -2.6406371593e-01
       coeff_l3(88,15)=  5.5020075291e-02
       coeff_l3(88,16)= -1.9948656857e-01
       coeff_l3(88,17)= -7.8298255801e-02
       coeff_l3(88,18)= -1.0818063468e-01
       coeff_l3(88,19)= -6.1091531068e-02
       coeff_l3(88,20)= -9.3634940684e-02
       coeff_l3(88,21)=  6.4299665391e-02
       coeff_l3(88,22)= -1.7222009599e-02
       coeff_l3(88,23)= -1.5361873806e-01
       coeff_l3(88,24)= -3.2766588032e-02
       coeff_l3(88,25)= -1.4837205410e-01
       coeff_l3(88,26)= -2.7733204886e-02
       coeff_l3(88,27)=  8.8675878942e-02
       coeff_l3(88,28)= -1.1698370799e-02
       coeff_l3(88,29)= -1.2301142514e-01
       coeff_l3(88,30)= -1.3678023219e-01
       coeff_l3(88,31)=  5.4788265377e-02
       coeff_l3(88,32)=  8.2263007760e-02
       coeff_l3(88,33)= -1.2125056237e-01
       coeff_l3(88,34)=  7.5147733092e-02
       coeff_l3(88,35)= -1.6246022284e-01
       coeff_l3(88,36)=  3.1604576856e-02
       coeff_l3(88,37)= -2.1005333960e-01
       coeff_l3(88,38)= -3.3175081015e-01
       coeff_l3(88,39)=  4.5456912369e-02
       coeff_l3(88,40)= -6.9653645158e-02
       coeff_l3(88,41)=  4.7276809812e-01
       coeff_l3(88,42)= -1.6405580938e-01
       coeff_l3(88,43)=  8.2368915901e-03
       coeff_l3(88,44)= -9.5201730728e-02
       coeff_l3(88,45)= -9.5891945064e-02
       coeff_l3(88,46)= -1.2062439928e-03
       coeff_l3(88,47)=  3.6201070994e-02
       coeff_l3(88,48)=  3.8289733231e-02
       coeff_l3(88,49)=  1.3675989211e-01
       coeff_l3(88,50)= -1.6135887802e-01
       coeff_l3(88,51)= -2.9440110922e-01
       coeff_l3(88,52)= -3.7904012948e-02
       coeff_l3(88,53)=  3.0033975840e-02
       coeff_l3(88,54)= -1.7711597681e-01
       coeff_l3(88,55)= -1.5027983487e-01
       coeff_l3(88,56)= -8.4676206112e-02
       coeff_l3(88,57)=  6.6024966538e-02
       coeff_l3(88,58)=  4.3233469129e-01
       coeff_l3(88,59)= -3.4654149413e-01
       coeff_l3(88,60)= -1.2048380375e+00
       coeff_l3(88,61)=  4.1489977390e-02
       coeff_l3(88,62)= -2.3765860498e-01
       coeff_l3(88,63)= -1.2869279087e-01
       coeff_l3(88,64)=  2.6298429817e-02
       coeff_l3(88,65)= -1.0403077304e-01
       coeff_l3(88,66)= -1.6965861619e-01
       coeff_l3(88,67)=  4.8879468441e-01
       coeff_l3(88,68)= -1.7815473676e-01
       coeff_l3(88,69)= -9.2620097101e-02
       coeff_l3(88,70)=  9.5019452274e-03
       coeff_l3(88,71)= -4.1189607233e-02
       coeff_l3(88,72)=  5.1021773368e-02
       coeff_l3(88,73)=  9.1422453523e-02
       coeff_l3(88,74)=  3.1232491136e-01
       coeff_l3(88,75)= -1.0589738935e-01
       coeff_l3(88,76)=  2.2923916578e-01
       coeff_l3(88,77)= -1.1538843066e-01
       coeff_l3(88,78)=  9.6418820322e-02
       coeff_l3(88,79)=  5.4831568152e-02
       coeff_l3(88,80)= -2.3911099136e-01
       coeff_l3(88,81)= -7.2419777513e-02
       coeff_l3(88,82)= -4.1529070586e-02
       coeff_l3(88,83)=  6.3617318869e-02
       coeff_l3(88,84)=  3.6819255352e-01
       coeff_l3(88,85)=  1.6465328634e-01
       coeff_l3(88,86)= -2.0195677876e-01
       coeff_l3(88,87)= -4.7670051455e-02
       coeff_l3(88,88)= -3.3346414566e-01
       coeff_l3(88,89)=  2.3530702293e-01
       coeff_l3(88,90)= -6.1008583754e-02
       coeff_l3(88,91)= -2.4214087427e-01
       coeff_l3(88,92)= -8.6129948497e-02
       coeff_l3(88,93)= -3.2068982720e-01
       coeff_l3(88,94)=  7.4831009842e-03
       coeff_l3(88,95)= -6.8109363317e-02
       coeff_l3(88,96)= -8.4266580641e-02
       coeff_l3(88,97)= -2.0146174356e-03
       coeff_l3(88,98)=  1.1292444170e-01
       coeff_l3(88,99)= -2.0433026552e-01
       coeff_l3(88,100)= -5.8539181948e-02
       coeff_l3(88,101)=  2.1467845142e-01
       coeff_l3(88,102)=  5.3379204124e-02
       coeff_l3(88,103)= -3.6988627166e-02
       coeff_l3(88,104)= -1.4310999215e-01
       coeff_l3(88,105)=  3.4857075661e-02
       coeff_l3(88,106)= -2.6885343716e-02
       coeff_l3(88,107)= -1.8052387238e-01
       coeff_l3(88,108)= -7.8894667327e-02
       coeff_l3(88,109)= -1.7298416793e-01
       coeff_l3(88,110)=  9.9539542571e-03
       coeff_l3(88,111)= -8.3702355623e-02
       coeff_l3(88,112)= -3.2161775231e-01
       coeff_l3(88,113)=  2.8200108558e-02
       coeff_l3(88,114)=  1.7408670485e-01
       coeff_l3(88,115)=  3.1699025631e-01
       coeff_l3(88,116)=  9.2017605901e-02
       coeff_l3(88,117)= -1.5602794290e-01
       coeff_l3(88,118)=  3.6338847876e-01
       coeff_l3(88,119)= -6.1759348959e-02
       coeff_l3(88,120)= -2.0563572645e-01
       coeff_l3(88,121)= -3.0494561791e-01
       coeff_l3(88,122)= -4.1914947331e-02
       coeff_l3(88,123)=  3.1438893639e-03
       coeff_l3(88,124)= -3.5842854530e-02
       coeff_l3(88,125)=  5.1491249353e-02
       coeff_l3(89,1)=  6.0272479057e-01
       coeff_l3(89,2)= -8.4154367447e-02
       coeff_l3(89,3)= -9.3925702572e-01
       coeff_l3(89,4)=  1.4015719295e-01
       coeff_l3(89,5)=  5.6225985289e-02
       coeff_l3(89,6)= -1.4442224801e-01
       coeff_l3(89,7)= -1.3735632598e-01
       coeff_l3(89,8)=  3.0021351576e-01
       coeff_l3(89,9)= -4.5228131115e-02
       coeff_l3(89,10)=  7.0919273421e-03
       coeff_l3(89,11)=  5.9973452240e-02
       coeff_l3(89,12)=  8.8819187880e-01
       coeff_l3(89,13)=  2.9695615172e-01
       coeff_l3(89,14)=  2.6451254264e-02
       coeff_l3(89,15)=  6.6014297307e-02
       coeff_l3(89,16)= -9.8117627203e-02
       coeff_l3(89,17)=  2.2508926690e-01
       coeff_l3(89,18)=  1.5014576912e+00
       coeff_l3(89,19)= -4.8401360400e-03
       coeff_l3(89,20)=  1.9775962830e-01
       coeff_l3(89,21)= -8.9915983379e-02
       coeff_l3(89,22)=  6.7023023963e-02
       coeff_l3(89,23)= -9.8380908370e-02
       coeff_l3(89,24)=  2.7179930359e-02
       coeff_l3(89,25)=  4.9221000075e-01
       coeff_l3(89,26)=  5.9714563191e-02
       coeff_l3(89,27)= -3.8828358054e-02
       coeff_l3(89,28)= -1.8434557319e-01
       coeff_l3(89,29)=  7.0526939631e-01
       coeff_l3(89,30)=  1.0796222836e-01
       coeff_l3(89,31)= -3.4498527646e-01
       coeff_l3(89,32)=  1.0545763373e-01
       coeff_l3(89,33)=  1.3035450876e-01
       coeff_l3(89,34)=  4.3820749968e-02
       coeff_l3(89,35)=  4.1855868697e-01
       coeff_l3(89,36)= -1.7736124992e-01
       coeff_l3(89,37)= -1.2697409093e-01
       coeff_l3(89,38)=  6.8772095442e-01
       coeff_l3(89,39)=  2.3905257881e-01
       coeff_l3(89,40)= -4.5283615589e-02
       coeff_l3(89,41)=  5.7028454542e-01
       coeff_l3(89,42)= -1.1812097579e-01
       coeff_l3(89,43)=  2.8170308471e-01
       coeff_l3(89,44)= -1.7010763288e-02
       coeff_l3(89,45)=  5.2048444748e-01
       coeff_l3(89,46)=  3.3781293035e-01
       coeff_l3(89,47)= -2.6485285163e-01
       coeff_l3(89,48)=  2.2773456573e-01
       coeff_l3(89,49)= -6.3632750511e-01
       coeff_l3(89,50)= -2.4807190895e-01
       coeff_l3(89,51)=  6.8486815691e-01
       coeff_l3(89,52)= -2.3126487434e-01
       coeff_l3(89,53)=  2.0459358394e-01
       coeff_l3(89,54)=  1.1553755403e-01
       coeff_l3(89,55)= -3.1949177384e-02
       coeff_l3(89,56)= -4.6869341284e-02
       coeff_l3(89,57)=  1.2344994582e-02
       coeff_l3(89,58)= -4.2764011025e-01
       coeff_l3(89,59)= -6.7024809122e-01
       coeff_l3(89,60)=  6.7827516794e-01
       coeff_l3(89,61)= -1.8601274490e-01
       coeff_l3(89,62)= -1.2084426731e-01
       coeff_l3(89,63)=  1.5851812065e-01
       coeff_l3(89,64)= -3.3351682127e-02
       coeff_l3(89,65)= -3.7455346435e-02
       coeff_l3(89,66)=  2.8071988374e-02
       coeff_l3(89,67)=  1.6421225667e-01
       coeff_l3(89,68)=  1.3373567164e-01
       coeff_l3(89,69)=  6.8719774485e-02
       coeff_l3(89,70)= -1.0178625584e-01
       coeff_l3(89,71)=  1.3590739965e+00
       coeff_l3(89,72)= -1.1676513590e-02
       coeff_l3(89,73)= -6.6166169941e-02
       coeff_l3(89,74)= -1.9088606536e-01
       coeff_l3(89,75)=  1.6027858853e-01
       coeff_l3(89,76)=  4.0484905243e-02
       coeff_l3(89,77)= -3.1265076995e-01
       coeff_l3(89,78)= -1.2325143069e-01
       coeff_l3(89,79)= -1.3359422982e-01
       coeff_l3(89,80)= -3.6489823461e-01
       coeff_l3(89,81)= -4.4897906482e-02
       coeff_l3(89,82)= -2.7570357919e-01
       coeff_l3(89,83)= -3.5054823756e-01
       coeff_l3(89,84)= -1.1847390980e-01
       coeff_l3(89,85)= -3.3936211467e-01
       coeff_l3(89,86)= -2.2006244957e-01
       coeff_l3(89,87)=  2.1164163947e-01
       coeff_l3(89,88)=  1.3694672287e-01
       coeff_l3(89,89)= -5.9100203216e-02
       coeff_l3(89,90)= -3.3022630215e-01
       coeff_l3(89,91)= -2.4448357522e-01
       coeff_l3(89,92)=  1.0505677015e-01
       coeff_l3(89,93)=  1.6718848050e-01
       coeff_l3(89,94)=  5.5125057697e-01
       coeff_l3(89,95)=  8.4971271455e-02
       coeff_l3(89,96)=  1.9028621912e-01
       coeff_l3(89,97)= -1.1188343167e-01
       coeff_l3(89,98)= -2.0483373106e-01
       coeff_l3(89,99)=  1.6983304918e-01
       coeff_l3(89,100)=  3.3216968179e-01
       coeff_l3(89,101)=  3.2036814839e-02
       coeff_l3(89,102)= -1.0538108647e-01
       coeff_l3(89,103)= -8.6454905570e-02
       coeff_l3(89,104)= -1.4149564505e-01
       coeff_l3(89,105)=  1.6721186042e-01
       coeff_l3(89,106)=  4.7343844175e-01
       coeff_l3(89,107)= -3.6109823734e-02
       coeff_l3(89,108)=  4.3225938082e-01
       coeff_l3(89,109)=  1.6348992288e-01
       coeff_l3(89,110)= -1.0772360116e-01
       coeff_l3(89,111)=  3.5240522027e-01
       coeff_l3(89,112)=  3.9820039272e-01
       coeff_l3(89,113)=  4.8818910122e-01
       coeff_l3(89,114)=  4.3001428246e-01
       coeff_l3(89,115)=  1.7140015960e-01
       coeff_l3(89,116)=  4.5660114288e-01
       coeff_l3(89,117)=  3.3301764727e-01
       coeff_l3(89,118)= -5.3633219004e-01
       coeff_l3(89,119)= -1.2577198446e-02
       coeff_l3(89,120)=  5.0268176943e-02
       coeff_l3(89,121)= -2.2674506903e-01
       coeff_l3(89,122)= -5.3072672337e-02
       coeff_l3(89,123)=  9.8931126297e-02
       coeff_l3(89,124)=  1.2354370952e-01
       coeff_l3(89,125)= -1.9560530782e-01
       coeff_l3(90,1)=  3.4193924069e-01
       coeff_l3(90,2)= -5.7340722531e-02
       coeff_l3(90,3)= -8.1368647516e-02
       coeff_l3(90,4)= -1.4671730995e-01
       coeff_l3(90,5)= -1.9592399895e-01
       coeff_l3(90,6)=  1.1347802356e-02
       coeff_l3(90,7)=  1.3988571167e+00
       coeff_l3(90,8)=  3.6913238466e-02
       coeff_l3(90,9)= -1.6423577908e-03
       coeff_l3(90,10)= -1.5535455942e-01
       coeff_l3(90,11)=  4.1140034795e-02
       coeff_l3(90,12)=  5.2567828447e-02
       coeff_l3(90,13)=  5.6456550956e-02
       coeff_l3(90,14)= -3.5385437310e-02
       coeff_l3(90,15)=  4.7132421285e-02
       coeff_l3(90,16)= -3.0595418066e-02
       coeff_l3(90,17)= -7.3883965611e-02
       coeff_l3(90,18)= -1.2865410745e-01
       coeff_l3(90,19)=  1.3280761242e-01
       coeff_l3(90,20)= -2.3395635188e-01
       coeff_l3(90,21)= -1.1116405576e-01
       coeff_l3(90,22)=  3.0705759302e-02
       coeff_l3(90,23)= -2.4103680253e-01
       coeff_l3(90,24)= -9.0655222535e-02
       coeff_l3(90,25)= -9.3276143074e-02
       coeff_l3(90,26)=  1.7685174942e-02
       coeff_l3(90,27)=  2.1166269779e+00
       coeff_l3(90,28)=  1.5136392117e+00
       coeff_l3(90,29)= -3.0836224556e-01
       coeff_l3(90,30)= -2.7327057719e-01
       coeff_l3(90,31)= -2.2725109756e-01
       coeff_l3(90,32)= -5.7630240917e-02
       coeff_l3(90,33)= -2.0169457793e-01
       coeff_l3(90,34)=  1.1212950945e-01
       coeff_l3(90,35)=  3.7970466656e-04
       coeff_l3(90,36)= -2.0114101470e-01
       coeff_l3(90,37)=  8.2667153329e-03
       coeff_l3(90,38)=  2.0728570223e-01
       coeff_l3(90,39)= -1.6557604074e-01
       coeff_l3(90,40)=  3.2465238124e-02
       coeff_l3(90,41)= -4.1333067417e-01
       coeff_l3(90,42)= -2.2020807862e-01
       coeff_l3(90,43)= -2.0234477520e-01
       coeff_l3(90,44)= -1.0808432847e-01
       coeff_l3(90,45)= -2.1406640112e-01
       coeff_l3(90,46)= -3.6798694730e-01
       coeff_l3(90,47)= -3.6353066564e-01
       coeff_l3(90,48)= -3.0256235600e-01
       coeff_l3(90,49)=  3.0012753010e+00
       coeff_l3(90,50)=  6.8526744843e-01
       coeff_l3(90,51)= -1.4118994772e-01
       coeff_l3(90,52)= -5.4779373109e-02
       coeff_l3(90,53)=  3.4350063652e-03
       coeff_l3(90,54)=  7.0214366913e-01
       coeff_l3(90,55)= -3.8246726990e-01
       coeff_l3(90,56)=  2.0743319765e-03
       coeff_l3(90,57)= -5.8205132373e-03
       coeff_l3(90,58)= -3.6861714721e-01
       coeff_l3(90,59)= -1.9105263054e-01
       coeff_l3(90,60)= -4.1543455124e+00
       coeff_l3(90,61)= -2.9616597295e-01
       coeff_l3(90,62)= -2.2099184990e-01
       coeff_l3(90,63)=  4.4131129980e-01
       coeff_l3(90,64)= -2.5971528888e-01
       coeff_l3(90,65)= -4.9285199493e-02
       coeff_l3(90,66)= -8.1233531237e-02
       coeff_l3(90,67)=  8.4246499464e-03
       coeff_l3(90,68)= -1.5910172462e-01
       coeff_l3(90,69)= -9.1155484319e-02
       coeff_l3(90,70)= -1.4856973290e-01
       coeff_l3(90,71)= -3.9846310019e-01
       coeff_l3(90,72)= -6.0577388853e-02
       coeff_l3(90,73)= -3.2558631152e-02
       coeff_l3(90,74)=  1.4813111722e-01
       coeff_l3(90,75)= -2.1119257808e-01
       coeff_l3(90,76)=  8.6629098654e-01
       coeff_l3(90,77)=  2.3802727461e-01
       coeff_l3(90,78)= -1.4763171971e-01
       coeff_l3(90,79)= -2.1703004837e-01
       coeff_l3(90,80)=  4.7457191348e-01
       coeff_l3(90,81)= -3.4339648485e-01
       coeff_l3(90,82)= -4.1521783918e-02
       coeff_l3(90,83)=  6.5433196723e-02
       coeff_l3(90,84)= -1.8086469173e-01
       coeff_l3(90,85)= -3.0320164561e-01
       coeff_l3(90,86)= -4.4363871217e-01
       coeff_l3(90,87)= -2.2370128334e-01
       coeff_l3(90,88)= -9.9749259651e-02
       coeff_l3(90,89)= -2.1317738295e-01
       coeff_l3(90,90)= -4.2377221584e-01
       coeff_l3(90,91)=  6.9624310732e-01
       coeff_l3(90,92)= -7.0972062647e-02
       coeff_l3(90,93)=  2.1897293627e-02
       coeff_l3(90,94)= -1.2322831154e-01
       coeff_l3(90,95)= -2.3581276834e-01
       coeff_l3(90,96)= -1.8026819825e-01
       coeff_l3(90,97)= -6.7714743316e-02
       coeff_l3(90,98)=  7.2098165751e-02
       coeff_l3(90,99)= -7.3565852642e-01
       coeff_l3(90,100)= -3.1620594859e-01
       coeff_l3(90,101)= -7.3464199901e-02
       coeff_l3(90,102)= -1.5625129640e-01
       coeff_l3(90,103)= -1.4471839368e-01
       coeff_l3(90,104)= -2.0395085216e-01
       coeff_l3(90,105)= -3.1000405550e-01
       coeff_l3(90,106)=  3.9676591754e-02
       coeff_l3(90,107)=  9.4918000698e-01
       coeff_l3(90,108)= -1.3114460744e-02
       coeff_l3(90,109)= -2.6543247700e-01
       coeff_l3(90,110)=  3.6739077419e-02
       coeff_l3(90,111)= -9.1205425560e-03
       coeff_l3(90,112)= -1.6573566198e-01
       coeff_l3(90,113)=  4.5372236520e-02
       coeff_l3(90,114)=  9.4271165133e-01
       coeff_l3(90,115)=  6.5502822399e-01
       coeff_l3(90,116)= -1.2774659693e-01
       coeff_l3(90,117)= -1.1311434209e-01
       coeff_l3(90,118)=  6.2576740980e-02
       coeff_l3(90,119)=  6.5163686872e-02
       coeff_l3(90,120)= -2.1912373602e-01
       coeff_l3(90,121)=  9.7927346826e-02
       coeff_l3(90,122)=  8.9611940086e-02
       coeff_l3(90,123)=  4.9131602282e-04
       coeff_l3(90,124)= -2.7225807309e-01
       coeff_l3(90,125)=  1.3589239120e-01
       coeff_l3(91,1)= -1.2351097912e-01
       coeff_l3(91,2)= -4.9386661500e-02
       coeff_l3(91,3)= -2.5176728144e-02
       coeff_l3(91,4)= -3.9194774628e-01
       coeff_l3(91,5)=  8.8982775807e-02
       coeff_l3(91,6)= -2.3883521557e-01
       coeff_l3(91,7)= -6.7725527287e-01
       coeff_l3(91,8)= -6.4664080739e-02
       coeff_l3(91,9)= -4.4105529785e-02
       coeff_l3(91,10)=  1.4406031370e-01
       coeff_l3(91,11)= -4.5416179299e-01
       coeff_l3(91,12)= -9.8600322008e-01
       coeff_l3(91,13)= -8.3176726103e-01
       coeff_l3(91,14)=  1.2965714931e-01
       coeff_l3(91,15)= -8.0539450049e-02
       coeff_l3(91,16)=  7.4372805655e-02
       coeff_l3(91,17)= -1.4242847264e-01
       coeff_l3(91,18)= -6.8800234795e-01
       coeff_l3(91,19)= -2.1734856069e-01
       coeff_l3(91,20)=  6.3602574170e-02
       coeff_l3(91,21)= -4.2978066206e-01
       coeff_l3(91,22)=  2.0896350965e-02
       coeff_l3(91,23)= -9.1326005757e-02
       coeff_l3(91,24)=  3.0956219882e-02
       coeff_l3(91,25)=  2.3586198222e-03
       coeff_l3(91,26)=  4.0150314569e-02
       coeff_l3(91,27)= -9.5342278481e-01
       coeff_l3(91,28)= -5.0072616339e-01
       coeff_l3(91,29)= -2.6388317347e-01
       coeff_l3(91,30)=  1.1463888735e-01
       coeff_l3(91,31)=  1.9812138751e-02
       coeff_l3(91,32)= -8.6291499436e-02
       coeff_l3(91,33)= -3.6911839247e-01
       coeff_l3(91,34)= -9.6578158438e-02
       coeff_l3(91,35)=  1.7118331790e-01
       coeff_l3(91,36)= -1.0387619585e-01
       coeff_l3(91,37)=  1.0302909464e-01
       coeff_l3(91,38)= -1.7025965452e-01
       coeff_l3(91,39)=  3.8463082165e-02
       coeff_l3(91,40)= -6.8307429552e-02
       coeff_l3(91,41)= -2.6755809784e-01
       coeff_l3(91,42)= -8.3832532167e-02
       coeff_l3(91,43)= -2.2104309499e-01
       coeff_l3(91,44)= -1.4934112132e-01
       coeff_l3(91,45)=  2.1893428266e-01
       coeff_l3(91,46)= -4.0886096656e-02
       coeff_l3(91,47)= -2.1797615290e-01
       coeff_l3(91,48)= -3.2690528035e-01
       coeff_l3(91,49)=  3.9171156287e-01
       coeff_l3(91,50)=  6.8549565971e-02
       coeff_l3(91,51)= -2.6372084022e-01
       coeff_l3(91,52)=  1.7797076702e-01
       coeff_l3(91,53)=  2.4651284330e-03
       coeff_l3(91,54)=  7.4332371354e-02
       coeff_l3(91,55)= -2.9909402132e-01
       coeff_l3(91,56)= -2.4053464830e-01
       coeff_l3(91,57)= -2.4767552316e-01
       coeff_l3(91,58)= -3.6826616526e-01
       coeff_l3(91,59)= -2.8547230177e-03
       coeff_l3(91,60)=  1.3738288879e+00
       coeff_l3(91,61)= -2.7319368720e-01
       coeff_l3(91,62)= -4.5399855822e-02
       coeff_l3(91,63)= -9.8004333675e-02
       coeff_l3(91,64)= -7.5766235590e-02
       coeff_l3(91,65)= -4.5547194779e-02
       coeff_l3(91,66)= -2.3794956505e-01
       coeff_l3(91,67)= -3.5750624537e-01
       coeff_l3(91,68)= -4.3352257460e-02
       coeff_l3(91,69)= -1.5206572413e-01
       coeff_l3(91,70)= -4.5850941539e-01
       coeff_l3(91,71)= -4.2921364307e-01
       coeff_l3(91,72)= -1.9139392674e-01
       coeff_l3(91,73)=  9.8769627512e-02
       coeff_l3(91,74)= -3.4304690361e-01
       coeff_l3(91,75)= -1.4479480684e-01
       coeff_l3(91,76)=  5.7211345434e-01
       coeff_l3(91,77)=  1.1233378202e-01
       coeff_l3(91,78)= -3.9643809199e-02
       coeff_l3(91,79)= -7.4534870684e-02
       coeff_l3(91,80)=  1.0090343654e-01
       coeff_l3(91,81)=  1.3304173946e-01
       coeff_l3(91,82)= -9.2389106750e-02
       coeff_l3(91,83)=  6.3120499253e-02
       coeff_l3(91,84)= -5.4053768516e-02
       coeff_l3(91,85)=  3.0144962668e-01
       coeff_l3(91,86)=  1.8090155721e-01
       coeff_l3(91,87)= -1.1153119802e-01
       coeff_l3(91,88)=  5.2770745009e-02
       coeff_l3(91,89)=  3.1017282605e-01
       coeff_l3(91,90)= -2.4598079920e-01
       coeff_l3(91,91)=  1.9642356038e-01
       coeff_l3(91,92)= -1.6023196280e-01
       coeff_l3(91,93)= -4.0706887841e-01
       coeff_l3(91,94)= -4.6947464347e-01
       coeff_l3(91,95)= -3.6292535067e-01
       coeff_l3(91,96)= -1.7384058237e-01
       coeff_l3(91,97)= -3.4783366323e-01
       coeff_l3(91,98)=  2.2683472931e-01
       coeff_l3(91,99)= -6.9110780954e-02
       coeff_l3(91,100)= -7.0268020034e-02
       coeff_l3(91,101)=  6.2954276800e-03
       coeff_l3(91,102)= -3.5710778832e-01
       coeff_l3(91,103)= -3.6054641008e-02
       coeff_l3(91,104)= -3.3284261823e-02
       coeff_l3(91,105)= -2.0051091909e-01
       coeff_l3(91,106)= -1.6209590435e-01
       coeff_l3(91,107)= -6.4385883510e-02
       coeff_l3(91,108)= -9.2560827732e-02
       coeff_l3(91,109)= -1.7271732911e-02
       coeff_l3(91,110)=  3.6475017667e-02
       coeff_l3(91,111)= -2.3330859840e-01
       coeff_l3(91,112)= -1.7452526093e-01
       coeff_l3(91,113)= -5.0887361169e-02
       coeff_l3(91,114)= -5.0262701511e-01
       coeff_l3(91,115)= -2.0813590288e-01
       coeff_l3(91,116)= -5.5554801226e-01
       coeff_l3(91,117)= -2.4735820293e-01
       coeff_l3(91,118)= -1.9144476950e-01
       coeff_l3(91,119)= -1.6914454103e-01
       coeff_l3(91,120)= -3.3623293042e-01
       coeff_l3(91,121)=  2.7148464322e-01
       coeff_l3(91,122)= -1.2298909947e-02
       coeff_l3(91,123)= -1.0642065853e-01
       coeff_l3(91,124)= -8.4454201162e-02
       coeff_l3(91,125)=  1.2054289132e-01
       coeff_l3(92,1)=  1.1657279730e-01
       coeff_l3(92,2)= -1.8689399958e-01
       coeff_l3(92,3)= -7.9694437981e-01
       coeff_l3(92,4)= -1.3840156794e-01
       coeff_l3(92,5)= -8.2482501864e-02
       coeff_l3(92,6)=  1.8352811038e-01
       coeff_l3(92,7)=  1.0016211271e+00
       coeff_l3(92,8)= -3.5902482271e-01
       coeff_l3(92,9)=  7.1669846773e-02
       coeff_l3(92,10)= -2.0520865917e-01
       coeff_l3(92,11)= -1.4803557098e-01
       coeff_l3(92,12)= -3.4383022785e-01
       coeff_l3(92,13)= -2.0791622996e-01
       coeff_l3(92,14)= -2.7794135734e-02
       coeff_l3(92,15)= -8.4199652076e-02
       coeff_l3(92,16)=  1.0051082075e-01
       coeff_l3(92,17)= -1.8643483520e-01
       coeff_l3(92,18)= -7.6237291098e-01
       coeff_l3(92,19)=  5.7724188082e-03
       coeff_l3(92,20)= -7.6784491539e-03
       coeff_l3(92,21)=  3.4770828206e-03
       coeff_l3(92,22)= -9.0661704540e-02
       coeff_l3(92,23)= -3.9147682488e-02
       coeff_l3(92,24)=  1.8433403969e-01
       coeff_l3(92,25)= -2.8364631534e-01
       coeff_l3(92,26)= -2.3873606697e-02
       coeff_l3(92,27)=  9.8402589560e-01
       coeff_l3(92,28)=  5.8462941647e-01
       coeff_l3(92,29)= -8.4169071913e-01
       coeff_l3(92,30)= -2.3200838268e-01
       coeff_l3(92,31)=  2.0665961504e-01
       coeff_l3(92,32)= -1.8546447158e-02
       coeff_l3(92,33)= -3.1418177485e-01
       coeff_l3(92,34)=  8.5225604475e-02
       coeff_l3(92,35)=  7.4365788698e-01
       coeff_l3(92,36)=  3.0701579526e-02
       coeff_l3(92,37)=  1.1679029465e-01
       coeff_l3(92,38)= -6.5552222729e-01
       coeff_l3(92,39)= -1.4940530062e-01
       coeff_l3(92,40)= -2.9583454132e-01
       coeff_l3(92,41)= -1.2921285629e+00
       coeff_l3(92,42)=  1.3013277948e-01
       coeff_l3(92,43)= -2.6359954476e-01
       coeff_l3(92,44)=  2.3713584244e-01
       coeff_l3(92,45)= -1.8550777435e-01
       coeff_l3(92,46)= -6.6507466137e-02
       coeff_l3(92,47)= -1.3885954022e-01
       coeff_l3(92,48)= -3.9581117034e-01
       coeff_l3(92,49)=  6.1852073669e-01
       coeff_l3(92,50)=  4.9353815615e-02
       coeff_l3(92,51)=  3.9134312421e-02
       coeff_l3(92,52)=  2.3825397715e-02
       coeff_l3(92,53)= -1.2155807018e-01
       coeff_l3(92,54)=  7.3869061470e-01
       coeff_l3(92,55)= -1.6673079133e-01
       coeff_l3(92,56)= -9.9770277739e-02
       coeff_l3(92,57)= -6.7849010229e-02
       coeff_l3(92,58)= -5.3179693222e-01
       coeff_l3(92,59)=  2.9704876244e-02
       coeff_l3(92,60)= -7.9013401270e-01
       coeff_l3(92,61)= -5.4737593979e-02
       coeff_l3(92,62)=  9.4604842365e-02
       coeff_l3(92,63)=  2.9750084877e-01
       coeff_l3(92,64)= -9.7502619028e-02
       coeff_l3(92,65)=  6.3960000873e-02
       coeff_l3(92,66)= -4.7516833991e-02
       coeff_l3(92,67)= -4.8819902539e-01
       coeff_l3(92,68)= -1.8135756254e-01
       coeff_l3(92,69)=  3.8689900190e-02
       coeff_l3(92,70)= -1.0441085696e-01
       coeff_l3(92,71)= -6.9047397375e-01
       coeff_l3(92,72)=  4.0279913694e-02
       coeff_l3(92,73)= -9.6831195056e-02
       coeff_l3(92,74)= -5.4071658850e-01
       coeff_l3(92,75)=  8.9860983193e-02
       coeff_l3(92,76)=  4.1779109836e-01
       coeff_l3(92,77)=  3.4791070223e-01
       coeff_l3(92,78)= -7.0538386703e-02
       coeff_l3(92,79)= -9.1452784836e-02
       coeff_l3(92,80)=  3.3717468381e-01
       coeff_l3(92,81)= -3.8140404224e-01
       coeff_l3(92,82)= -2.3717787862e-01
       coeff_l3(92,83)=  2.4667111039e-01
       coeff_l3(92,84)= -1.4803853631e-01
       coeff_l3(92,85)= -3.4334421158e-01
       coeff_l3(92,86)=  5.8929171413e-02
       coeff_l3(92,87)= -6.3371315598e-02
       coeff_l3(92,88)= -5.7798154652e-02
       coeff_l3(92,89)= -1.9668069482e-01
       coeff_l3(92,90)=  1.4660930634e-01
       coeff_l3(92,91)=  7.6025277376e-01
       coeff_l3(92,92)=  3.1297050416e-02
       coeff_l3(92,93)= -3.9113804698e-01
       coeff_l3(92,94)= -4.1661076248e-02
       coeff_l3(92,95)= -3.1814128160e-01
       coeff_l3(92,96)= -3.5577196628e-02
       coeff_l3(92,97)=  3.5722900182e-02
       coeff_l3(92,98)= -1.3156202435e-01
       coeff_l3(92,99)= -4.1992503405e-01
       coeff_l3(92,100)= -1.8771353364e-01
       coeff_l3(92,101)=  7.0283152163e-02
       coeff_l3(92,102)= -3.6775562912e-02
       coeff_l3(92,103)= -9.4220302999e-02
       coeff_l3(92,104)= -9.3788720667e-02
       coeff_l3(92,105)= -8.8453644514e-01
       coeff_l3(92,106)= -6.8519793451e-02
       coeff_l3(92,107)= -2.4532729387e-01
       coeff_l3(92,108)=  2.0685993135e-02
       coeff_l3(92,109)= -8.2810044289e-02
       coeff_l3(92,110)= -8.5822522640e-02
       coeff_l3(92,111)= -2.2893008590e-01
       coeff_l3(92,112)= -5.1959800720e-01
       coeff_l3(92,113)= -4.9368367763e-04
       coeff_l3(92,114)=  1.4689429104e-01
       coeff_l3(92,115)= -4.3964660168e-01
       coeff_l3(92,116)= -1.7659357190e-01
       coeff_l3(92,117)= -5.0158295780e-02
       coeff_l3(92,118)= -3.0767014623e-01
       coeff_l3(92,119)= -1.3606367633e-02
       coeff_l3(92,120)= -3.2492575049e-01
       coeff_l3(92,121)=  1.9411610067e-01
       coeff_l3(92,122)= -3.5558518022e-02
       coeff_l3(92,123)=  2.4574403651e-03
       coeff_l3(92,124)=  1.9806772470e-01
       coeff_l3(92,125)=  1.7568376660e-01
       coeff_l3(93,1)= -6.5658879280e-01
       coeff_l3(93,2)= -8.5565485060e-02
       coeff_l3(93,3)= -2.6062691212e-01
       coeff_l3(93,4)= -6.8233929574e-02
       coeff_l3(93,5)=  9.3839399517e-02
       coeff_l3(93,6)=  2.4685990810e-01
       coeff_l3(93,7)=  2.8546890616e-01
       coeff_l3(93,8)= -1.1686797440e-01
       coeff_l3(93,9)=  9.0483702719e-02
       coeff_l3(93,10)= -2.2700072825e-01
       coeff_l3(93,11)= -2.4222955108e-02
       coeff_l3(93,12)= -5.5508887768e-01
       coeff_l3(93,13)= -3.7183633447e-01
       coeff_l3(93,14)= -1.9505081698e-02
       coeff_l3(93,15)= -3.7154607475e-02
       coeff_l3(93,16)=  9.2200867832e-02
       coeff_l3(93,17)= -5.0407177210e-01
       coeff_l3(93,18)= -1.0998350382e+00
       coeff_l3(93,19)=  2.9165926576e-01
       coeff_l3(93,20)= -2.8880399466e-01
       coeff_l3(93,21)= -1.4746995270e-01
       coeff_l3(93,22)= -4.4741752744e-01
       coeff_l3(93,23)=  2.5977846980e-01
       coeff_l3(93,24)=  7.2726696730e-02
       coeff_l3(93,25)=  1.1508251919e-04
       coeff_l3(93,26)= -1.8332296610e-01
       coeff_l3(93,27)=  1.0780270398e-01
       coeff_l3(93,28)=  3.3482241631e-01
       coeff_l3(93,29)= -7.5659328699e-01
       coeff_l3(93,30)=  1.8230560422e-01
       coeff_l3(93,31)=  2.4979448318e-01
       coeff_l3(93,32)= -1.3571920991e-01
       coeff_l3(93,33)= -2.4311116338e-01
       coeff_l3(93,34)=  2.1433304250e-01
       coeff_l3(93,35)= -1.8517124653e+00
       coeff_l3(93,36)= -9.4433821738e-02
       coeff_l3(93,37)= -5.8925643563e-02
       coeff_l3(93,38)= -6.2506932020e-01
       coeff_l3(93,39)=  6.0795549303e-02
       coeff_l3(93,40)= -1.4980594814e-01
       coeff_l3(93,41)= -3.3029344678e-01
       coeff_l3(93,42)= -5.1084016450e-03
       coeff_l3(93,43)= -2.8125417233e-01
       coeff_l3(93,44)=  2.9455161095e-01
       coeff_l3(93,45)= -5.2248793840e-01
       coeff_l3(93,46)= -1.9925814867e-01
       coeff_l3(93,47)=  3.3521226048e-01
       coeff_l3(93,48)= -3.6368665099e-01
       coeff_l3(93,49)= -2.0913341641e-01
       coeff_l3(93,50)=  1.3033887744e-01
       coeff_l3(93,51)= -2.3849844933e-01
       coeff_l3(93,52)= -1.0781765729e-01
       coeff_l3(93,53)=  1.3167399168e-01
       coeff_l3(93,54)=  2.5090268254e-01
       coeff_l3(93,55)=  2.1220052242e-01
       coeff_l3(93,56)= -4.1630499065e-02
       coeff_l3(93,57)=  1.5512099862e-01
       coeff_l3(93,58)=  1.1210736036e+00
       coeff_l3(93,59)=  5.0726376474e-02
       coeff_l3(93,60)= -5.9340614080e-01
       coeff_l3(93,61)=  2.5397699326e-02
       coeff_l3(93,62)=  2.1632549167e-01
       coeff_l3(93,63)= -4.3755450845e-01
       coeff_l3(93,64)=  4.6122800559e-02
       coeff_l3(93,65)=  2.7996712923e-01
       coeff_l3(93,66)=  1.0115467012e-01
       coeff_l3(93,67)= -9.6693283319e-01
       coeff_l3(93,68)= -3.7246757746e-01
       coeff_l3(93,69)= -1.3532190025e-01
       coeff_l3(93,70)=  9.6503995359e-02
       coeff_l3(93,71)= -1.0618140697e+00
       coeff_l3(93,72)=  1.2413080782e-01
       coeff_l3(93,73)=  6.7137300968e-02
       coeff_l3(93,74)= -5.3639417887e-01
       coeff_l3(93,75)=  2.8054511547e-01
       coeff_l3(93,76)= -6.3115626574e-01
       coeff_l3(93,77)=  3.0594842508e-02
       coeff_l3(93,78)= -3.0582124367e-02
       coeff_l3(93,79)=  1.2126988173e-01
       coeff_l3(93,80)= -1.1861076951e-01
       coeff_l3(93,81)= -3.0830428004e-02
       coeff_l3(93,82)= -1.5153498948e-01
       coeff_l3(93,83)= -1.6241014004e-01
       coeff_l3(93,84)= -1.0600504279e-01
       coeff_l3(93,85)=  9.6267335117e-02
       coeff_l3(93,86)= -2.5293607265e-02
       coeff_l3(93,87)= -3.5582419485e-02
       coeff_l3(93,88)=  7.5157113373e-02
       coeff_l3(93,89)=  2.8404432535e-01
       coeff_l3(93,90)=  4.1665602475e-03
       coeff_l3(93,91)= -1.0492622852e+00
       coeff_l3(93,92)=  1.0448394716e-01
       coeff_l3(93,93)= -1.0063284077e-02
       coeff_l3(93,94)= -1.0412887484e-01
       coeff_l3(93,95)= -2.5905232877e-02
       coeff_l3(93,96)= -7.1440428495e-02
       coeff_l3(93,97)=  5.8348696679e-02
       coeff_l3(93,98)= -1.0379017889e-01
       coeff_l3(93,99)= -5.0598036498e-02
       coeff_l3(93,100)= -3.5786536336e-01
       coeff_l3(93,101)= -2.0228628814e-01
       coeff_l3(93,102)=  6.1301264912e-02
       coeff_l3(93,103)=  1.5721444786e-01
       coeff_l3(93,104)=  1.0038721561e-01
       coeff_l3(93,105)= -4.7818380594e-01
       coeff_l3(93,106)= -2.5291758776e-01
       coeff_l3(93,107)=  2.4834442884e-02
       coeff_l3(93,108)= -2.2623896599e-01
       coeff_l3(93,109)=  1.4111268520e-01
       coeff_l3(93,110)=  6.4540073276e-02
       coeff_l3(93,111)= -1.6678127646e-01
       coeff_l3(93,112)= -6.8160700798e-01
       coeff_l3(93,113)= -3.4694322944e-01
       coeff_l3(93,114)=  2.3294883966e-01
       coeff_l3(93,115)= -3.1314677000e-01
       coeff_l3(93,116)= -2.9428413510e-01
       coeff_l3(93,117)= -1.4076265693e-01
       coeff_l3(93,118)= -1.3506492972e-01
       coeff_l3(93,119)= -9.3625955284e-02
       coeff_l3(93,120)= -1.8666781485e-01
       coeff_l3(93,121)= -6.7602530122e-02
       coeff_l3(93,122)= -1.3771060109e-01
       coeff_l3(93,123)=  2.8395326808e-02
       coeff_l3(93,124)= -8.2051623613e-03
       coeff_l3(93,125)=  8.8194377720e-02
       coeff_l3(94,1)=  5.6432276964e-02
       coeff_l3(94,2)=  1.4202472568e-01
       coeff_l3(94,3)=  1.2069322914e-01
       coeff_l3(94,4)=  2.0451395214e-01
       coeff_l3(94,5)= -9.2721179128e-02
       coeff_l3(94,6)=  3.0534630641e-02
       coeff_l3(94,7)=  1.9629530609e-02
       coeff_l3(94,8)=  1.4202784002e-01
       coeff_l3(94,9)= -4.3725069612e-02
       coeff_l3(94,10)= -1.6333116591e-01
       coeff_l3(94,11)=  5.2871543914e-02
       coeff_l3(94,12)=  2.4925915897e-01
       coeff_l3(94,13)=  4.7137531638e-01
       coeff_l3(94,14)=  7.4478536844e-02
       coeff_l3(94,15)= -1.8074062467e-01
       coeff_l3(94,16)= -2.1781058609e-01
       coeff_l3(94,17)= -2.7292232960e-02
       coeff_l3(94,18)=  3.2578665018e-01
       coeff_l3(94,19)= -1.1389528960e-01
       coeff_l3(94,20)=  5.8358211070e-02
       coeff_l3(94,21)= -2.8905250132e-02
       coeff_l3(94,22)=  1.7409512401e-01
       coeff_l3(94,23)= -3.0852997303e-01
       coeff_l3(94,24)= -1.7125691473e-01
       coeff_l3(94,25)= -1.0534535348e-01
       coeff_l3(94,26)= -2.4537973106e-01
       coeff_l3(94,27)=  9.8504558206e-02
       coeff_l3(94,28)=  2.2363804746e-03
       coeff_l3(94,29)=  4.4923859835e-01
       coeff_l3(94,30)= -2.7438244224e-01
       coeff_l3(94,31)= -1.7860263586e-01
       coeff_l3(94,32)= -2.1050500870e-01
       coeff_l3(94,33)=  1.7946738005e-01
       coeff_l3(94,34)= -2.6304959320e-03
       coeff_l3(94,35)=  9.8798751831e-02
       coeff_l3(94,36)=  2.0896644890e-01
       coeff_l3(94,37)= -2.7340653539e-01
       coeff_l3(94,38)=  4.5841044188e-01
       coeff_l3(94,39)= -1.0328638367e-02
       coeff_l3(94,40)=  3.7139080465e-02
       coeff_l3(94,41)=  5.1890558004e-01
       coeff_l3(94,42)= -3.8435798883e-01
       coeff_l3(94,43)=  1.2640500069e-01
       coeff_l3(94,44)= -4.0613153577e-01
       coeff_l3(94,45)= -5.0283493474e-03
       coeff_l3(94,46)= -1.1445685290e-02
       coeff_l3(94,47)=  9.3004465103e-02
       coeff_l3(94,48)= -3.3938561683e-04
       coeff_l3(94,49)= -1.5511418879e-01
       coeff_l3(94,50)= -2.0100097358e-01
       coeff_l3(94,51)= -2.0180651918e-03
       coeff_l3(94,52)=  1.9292598590e-02
       coeff_l3(94,53)= -3.7367157638e-02
       coeff_l3(94,54)= -1.5722954273e-01
       coeff_l3(94,55)=  8.1078313291e-02
       coeff_l3(94,56)=  2.7320234105e-02
       coeff_l3(94,57)= -1.6571454704e-01
       coeff_l3(94,58)= -2.8720489144e-01
       coeff_l3(94,59)= -1.1550275981e-01
       coeff_l3(94,60)=  4.4727955014e-02
       coeff_l3(94,61)= -2.9214841127e-01
       coeff_l3(94,62)= -2.4836219847e-01
       coeff_l3(94,63)=  4.6165596694e-02
       coeff_l3(94,64)= -6.7873261869e-02
       coeff_l3(94,65)= -2.5018808246e-01
       coeff_l3(94,66)=  1.5086606145e-01
       coeff_l3(94,67)=  4.8564415425e-02
       coeff_l3(94,68)= -3.7774279714e-02
       coeff_l3(94,69)= -1.4361862838e-01
       coeff_l3(94,70)= -3.2345324755e-02
       coeff_l3(94,71)=  1.6671381891e-01
       coeff_l3(94,72)= -3.2209241390e-01
       coeff_l3(94,73)= -9.6594199538e-02
       coeff_l3(94,74)=  1.1006005853e-01
       coeff_l3(94,75)= -3.3070373535e-01
       coeff_l3(94,76)= -3.3863449097e-01
       coeff_l3(94,77)= -4.0526831150e-01
       coeff_l3(94,78)=  3.5507243127e-02
       coeff_l3(94,79)= -1.5231935680e-01
       coeff_l3(94,80)= -1.5461204946e-01
       coeff_l3(94,81)=  1.0882116109e-01
       coeff_l3(94,82)= -3.9803031832e-02
       coeff_l3(94,83)= -1.4500018209e-02
       coeff_l3(94,84)= -1.2015911192e-01
       coeff_l3(94,85)=  2.3454128206e-01
       coeff_l3(94,86)= -7.1464300156e-02
       coeff_l3(94,87)=  9.1117814183e-02
       coeff_l3(94,88)= -2.7677676082e-01
       coeff_l3(94,89)=  1.1715650558e-01
       coeff_l3(94,90)= -3.6757904291e-01
       coeff_l3(94,91)= -2.8478375077e-01
       coeff_l3(94,92)= -1.1237762868e-01
       coeff_l3(94,93)= -1.6429468989e-01
       coeff_l3(94,94)=  1.8677037954e-01
       coeff_l3(94,95)=  2.4058607221e-01
       coeff_l3(94,96)= -6.4197047614e-03
       coeff_l3(94,97)=  5.2029799670e-02
       coeff_l3(94,98)= -3.4338668920e-03
       coeff_l3(94,99)= -1.0115183145e-01
       coeff_l3(94,100)=  1.5881396830e-01
       coeff_l3(94,101)= -9.0311467648e-02
       coeff_l3(94,102)=  4.9958437681e-02
       coeff_l3(94,103)= -1.2927508354e-01
       coeff_l3(94,104)= -1.7427755520e-02
       coeff_l3(94,105)= -8.0629728734e-02
       coeff_l3(94,106)=  6.5015673637e-02
       coeff_l3(94,107)= -2.1847362816e-01
       coeff_l3(94,108)=  1.5206423402e-01
       coeff_l3(94,109)= -1.3951534033e-01
       coeff_l3(94,110)= -8.7302230299e-02
       coeff_l3(94,111)=  3.0069586635e-01
       coeff_l3(94,112)=  1.4628341794e-01
       coeff_l3(94,113)=  1.5189798176e-01
       coeff_l3(94,114)=  1.1535406113e-01
       coeff_l3(94,115)=  1.5246322751e-01
       coeff_l3(94,116)= -5.1627606153e-03
       coeff_l3(94,117)=  2.1629545838e-02
       coeff_l3(94,118)=  5.4738497734e-01
       coeff_l3(94,119)=  1.1696683615e-01
       coeff_l3(94,120)=  2.4735422432e-01
       coeff_l3(94,121)= -3.8888726383e-02
       coeff_l3(94,122)=  5.1723822951e-02
       coeff_l3(94,123)= -2.1065880358e-01
       coeff_l3(94,124)=  6.1834059656e-02
       coeff_l3(94,125)= -1.6514785588e-01
       coeff_l3(95,1)= -1.3313465118e+00
       coeff_l3(95,2)=  2.0345518366e-02
       coeff_l3(95,3)= -1.1208192259e-01
       coeff_l3(95,4)= -3.6624208093e-01
       coeff_l3(95,5)=  4.6481952071e-02
       coeff_l3(95,6)=  2.8669610620e-02
       coeff_l3(95,7)=  1.7091229558e-01
       coeff_l3(95,8)= -1.5705725551e-01
       coeff_l3(95,9)=  3.7025645375e-02
       coeff_l3(95,10)= -1.3136911392e-01
       coeff_l3(95,11)= -1.1834500730e-01
       coeff_l3(95,12)= -3.8790559769e-01
       coeff_l3(95,13)= -5.1124221087e-01
       coeff_l3(95,14)= -4.1762140393e-01
       coeff_l3(95,15)=  8.0365404487e-02
       coeff_l3(95,16)=  4.3459370732e-02
       coeff_l3(95,17)= -1.7073261738e-01
       coeff_l3(95,18)= -8.5200643539e-01
       coeff_l3(95,19)=  4.7117929906e-02
       coeff_l3(95,20)= -3.4313824028e-02
       coeff_l3(95,21)=  1.2669871747e-01
       coeff_l3(95,22)=  2.0102633536e-01
       coeff_l3(95,23)=  1.1212283373e-01
       coeff_l3(95,24)= -7.9633198678e-02
       coeff_l3(95,25)=  4.5622065663e-02
       coeff_l3(95,26)=  4.8744425178e-02
       coeff_l3(95,27)=  6.1742790043e-02
       coeff_l3(95,28)=  1.4009292237e-02
       coeff_l3(95,29)= -5.2436000109e-01
       coeff_l3(95,30)= -2.5268268585e-01
       coeff_l3(95,31)=  2.3988583684e-01
       coeff_l3(95,32)=  2.0022571087e-01
       coeff_l3(95,33)= -1.2078044936e-02
       coeff_l3(95,34)= -1.1071296036e-01
       coeff_l3(95,35)= -3.1471896172e-01
       coeff_l3(95,36)=  8.6260579526e-02
       coeff_l3(95,37)= -7.3227025568e-02
       coeff_l3(95,38)=  1.5617989004e-02
       coeff_l3(95,39)= -4.5940954238e-02
       coeff_l3(95,40)= -3.6628346890e-02
       coeff_l3(95,41)= -4.8204028606e-01
       coeff_l3(95,42)=  2.0419443026e-03
       coeff_l3(95,43)= -4.2387244105e-01
       coeff_l3(95,44)=  2.5175100565e-01
       coeff_l3(95,45)= -3.8519418240e-01
       coeff_l3(95,46)= -2.6576465368e-01
       coeff_l3(95,47)=  1.2427787483e-01
       coeff_l3(95,48)= -1.3957433403e-01
       coeff_l3(95,49)= -3.3834442496e-01
       coeff_l3(95,50)= -3.8581174612e-01
       coeff_l3(95,51)= -1.9509729743e-01
       coeff_l3(95,52)=  2.1299226210e-02
       coeff_l3(95,53)= -2.4677764624e-02
       coeff_l3(95,54)=  6.6927641630e-02
       coeff_l3(95,55)=  3.5866800696e-02
       coeff_l3(95,56)= -6.3632942736e-02
       coeff_l3(95,57)= -8.9677438140e-02
       coeff_l3(95,58)=  6.6353368759e-01
       coeff_l3(95,59)= -4.2704787850e-01
       coeff_l3(95,60)= -8.8422968984e-03
       coeff_l3(95,61)=  1.7647820711e-01
       coeff_l3(95,62)=  3.3208897803e-03
       coeff_l3(95,63)= -2.0684501529e-01
       coeff_l3(95,64)=  3.6741103977e-02
       coeff_l3(95,65)=  2.6592352986e-01
       coeff_l3(95,66)= -1.2656190991e-01
       coeff_l3(95,67)=  4.1883921623e-01
       coeff_l3(95,68)= -3.4906411171e-01
       coeff_l3(95,69)= -1.4545546472e-01
       coeff_l3(95,70)= -1.8668598495e-03
       coeff_l3(95,71)= -6.1173582077e-01
       coeff_l3(95,72)=  2.0014888048e-01
       coeff_l3(95,73)= -2.5676490739e-02
       coeff_l3(95,74)= -2.6474634185e-02
       coeff_l3(95,75)= -3.4832522273e-02
       coeff_l3(95,76)= -2.3484596610e-01
       coeff_l3(95,77)=  5.7101268321e-02
       coeff_l3(95,78)= -3.9705831558e-02
       coeff_l3(95,79)=  7.2300344706e-02
       coeff_l3(95,80)=  1.2760035694e-01
       coeff_l3(95,81)= -1.5998436511e-01
       coeff_l3(95,82)= -1.0898849368e-01
       coeff_l3(95,83)=  1.1544081569e-01
       coeff_l3(95,84)=  3.0499154702e-02
       coeff_l3(95,85)= -5.3687119484e-01
       coeff_l3(95,86)=  9.5848500729e-02
       coeff_l3(95,87)= -1.9970567524e-01
       coeff_l3(95,88)= -1.3818976283e-01
       coeff_l3(95,89)= -4.9169942737e-01
       coeff_l3(95,90)=  8.6388848722e-02
       coeff_l3(95,91)= -5.0953414291e-02
       coeff_l3(95,92)=  9.3143925071e-02
       coeff_l3(95,93)= -2.6756012440e-01
       coeff_l3(95,94)= -1.7433395982e-01
       coeff_l3(95,95)= -2.4098779261e-01
       coeff_l3(95,96)=  3.5485555418e-04
       coeff_l3(95,97)=  1.2062200159e-01
       coeff_l3(95,98)=  8.2326799631e-02
       coeff_l3(95,99)=  1.0863132775e-01
       coeff_l3(95,100)= -2.1171352267e-01
       coeff_l3(95,101)= -9.3656405807e-03
       coeff_l3(95,102)=  6.0047198087e-02
       coeff_l3(95,103)=  2.9665404558e-01
       coeff_l3(95,104)=  3.4956101328e-02
       coeff_l3(95,105)=  2.1690903232e-02
       coeff_l3(95,106)= -3.9248532057e-01
       coeff_l3(95,107)= -4.6455654502e-01
       coeff_l3(95,108)= -6.7838974297e-02
       coeff_l3(95,109)= -2.5174509734e-02
       coeff_l3(95,110)=  1.6638699919e-02
       coeff_l3(95,111)= -1.5012280643e-01
       coeff_l3(95,112)= -6.6190344095e-01
       coeff_l3(95,113)= -4.0060693026e-01
       coeff_l3(95,114)=  6.5486513078e-02
       coeff_l3(95,115)= -3.0727550387e-02
       coeff_l3(95,116)= -2.6945868134e-01
       coeff_l3(95,117)= -3.2562121749e-01
       coeff_l3(95,118)=  1.2245766073e-01
       coeff_l3(95,119)=  7.2968542576e-02
       coeff_l3(95,120)=  3.0099567026e-02
       coeff_l3(95,121)= -2.3882569373e-01
       coeff_l3(95,122)=  1.1530227214e-01
       coeff_l3(95,123)= -2.4918418378e-02
       coeff_l3(95,124)= -2.3597249761e-02
       coeff_l3(95,125)=  3.2227593660e-01
!coeff layer 4
       coeff_l4(1,1)= -2.0917847753e-01
       coeff_l4(1,2)= -3.0639316887e-02
       coeff_l4(1,3)= -2.3904466629e-01
       coeff_l4(1,4)= -1.0988600254e+00
       coeff_l4(1,5)= -2.5567648932e-02
       coeff_l4(1,6)= -9.7379016876e-01
       coeff_l4(1,7)= -1.7072305679e+00
       coeff_l4(1,8)= -6.6380292177e-01
       coeff_l4(1,9)=  8.3785164356e-01
       coeff_l4(1,10)=  5.8873492479e-01
       coeff_l4(1,11)=  1.5639865398e+00
       coeff_l4(1,12)=  1.4232579470e+00
       coeff_l4(1,13)=  3.5757520795e-01
       coeff_l4(1,14)=  1.7374505997e+00
       coeff_l4(1,15)= -1.4537584782e-01
       coeff_l4(1,16)= -2.5936210155e-01
       coeff_l4(1,17)= -6.3811548054e-02
       coeff_l4(1,18)= -5.9130132198e-02
       coeff_l4(1,19)=  1.0673830658e-01
       coeff_l4(1,20)=  8.9564114809e-01
       coeff_l4(1,21)= -1.9896408916e-01
       coeff_l4(1,22)=  1.3600315154e-01
       coeff_l4(1,23)= -5.2244764566e-01
       coeff_l4(1,24)=  4.6331919730e-02
       coeff_l4(1,25)=  5.2092701197e-01
       coeff_l4(1,26)=  2.5742569566e-01
       coeff_l4(1,27)= -7.9620498419e-01
       coeff_l4(1,28)=  3.3283585310e-01
       coeff_l4(1,29)= -3.5547807813e-01
       coeff_l4(1,30)=  5.1572227478e-01
       coeff_l4(1,31)= -5.1867056638e-02
       coeff_l4(1,32)= -1.1365216970e-01
       coeff_l4(1,33)= -1.6816094518e-01
       coeff_l4(1,34)= -1.7404508591e-01
       coeff_l4(1,35)= -9.9357195199e-02
       coeff_l4(1,36)=  6.9866347313e-01
       coeff_l4(1,37)=  2.1891653538e-02
       coeff_l4(1,38)= -4.5195940137e-01
       coeff_l4(1,39)=  1.2043386698e-02
       coeff_l4(1,40)=  1.2573938072e-01
       coeff_l4(1,41)=  2.9367688298e-01
       coeff_l4(1,42)= -3.4436523914e-01
       coeff_l4(1,43)=  4.4551730156e-01
       coeff_l4(1,44)= -3.9982688427e-01
       coeff_l4(1,45)=  8.5882313550e-02
       coeff_l4(1,46)= -8.1200057268e-01
       coeff_l4(1,47)= -1.4829921722e-01
       coeff_l4(1,48)= -1.0463276505e-01
       coeff_l4(1,49)=  5.7954750955e-02
       coeff_l4(1,50)= -2.7987455949e-02
       coeff_l4(1,51)=  2.7660465240e-01
       coeff_l4(1,52)=  3.7394565344e-01
       coeff_l4(1,53)=  1.0820357800e+00
       coeff_l4(1,54)= -4.5284697413e-01
       coeff_l4(1,55)=  2.3231700063e-01
       coeff_l4(1,56)= -9.0536452830e-02
       coeff_l4(1,57)= -2.6346564293e-01
       coeff_l4(1,58)=  2.2294923663e-02
       coeff_l4(1,59)=  4.4769182801e-01
       coeff_l4(1,60)= -4.0701788664e-01
       coeff_l4(1,61)=  6.8503564596e-01
       coeff_l4(1,62)= -1.1020114273e-01
       coeff_l4(1,63)=  1.1886361986e-01
       coeff_l4(1,64)=  6.0697323084e-01
       coeff_l4(1,65)=  6.4979940653e-01
       coeff_l4(1,66)=  4.3922778964e-01
       coeff_l4(1,67)= -6.7738044262e-01
       coeff_l4(1,68)=  3.5351383686e-01
       coeff_l4(1,69)=  1.9928576052e-01
       coeff_l4(1,70)=  1.6925062239e-01
       coeff_l4(1,71)=  8.4985309839e-01
       coeff_l4(1,72)= -4.8962354660e-01
       coeff_l4(1,73)= -1.0209581852e+00
       coeff_l4(1,74)=  1.6910222173e-01
       coeff_l4(1,75)=  8.2818377018e-01
       coeff_l4(1,76)= -1.5397197008e-01
       coeff_l4(1,77)= -9.9531489611e-01
       coeff_l4(1,78)=  7.5297099352e-01
       coeff_l4(1,79)= -6.6896736622e-01
       coeff_l4(1,80)= -7.6089851558e-02
       coeff_l4(1,81)=  4.1915720701e-01
       coeff_l4(1,82)=  9.5220786333e-01
       coeff_l4(1,83)= -1.8464480340e-01
       coeff_l4(1,84)= -4.0473598987e-02
       coeff_l4(1,85)=  9.4324225187e-01
       coeff_l4(1,86)= -2.0178193226e-02
       coeff_l4(1,87)=  1.2718199193e-01
       coeff_l4(1,88)= -2.7602478862e-01
       coeff_l4(1,89)=  4.2761281133e-02
       coeff_l4(1,90)= -7.9907607287e-03
       coeff_l4(1,91)=  3.3543407917e-01
       coeff_l4(1,92)= -1.6090580821e-01
       coeff_l4(1,93)= -2.1661187708e-01
       coeff_l4(1,94)= -1.2293947488e-01
       coeff_l4(1,95)= -6.6894876957e-01
       coeff_l4(1,96)=  2.2383648902e-03
       coeff_l4(1,97)=  3.3281353116e-01
       coeff_l4(1,98)=  1.9244045019e-01
       coeff_l4(1,99)=  2.1618124843e-01
       coeff_l4(1,100)= -4.2786908150e-01
       coeff_l4(1,101)=  9.6810603142e-01
       coeff_l4(1,102)=  3.7482696772e-01
       coeff_l4(1,103)=  6.9040223956e-02
       coeff_l4(1,104)=  1.7984750867e-01
       coeff_l4(1,105)= -1.4319079928e-02
       coeff_l4(2,1)=  2.6421533898e-02
       coeff_l4(2,2)=  2.4243444204e-02
       coeff_l4(2,3)=  2.5907287002e-01
       coeff_l4(2,4)=  7.1048907936e-02
       coeff_l4(2,5)=  6.3015580177e-02
       coeff_l4(2,6)=  1.6473741829e-01
       coeff_l4(2,7)=  1.1965285242e-01
       coeff_l4(2,8)= -4.8941183835e-02
       coeff_l4(2,9)=  4.8947922885e-02
       coeff_l4(2,10)= -8.1390673295e-03
       coeff_l4(2,11)=  1.9858497381e-01
       coeff_l4(2,12)=  3.3676049113e-01
       coeff_l4(2,13)=  5.5213768035e-02
       coeff_l4(2,14)=  9.3449205160e-02
       coeff_l4(2,15)=  4.2955331504e-02
       coeff_l4(2,16)= -1.4135041833e-01
       coeff_l4(2,17)=  1.2372119725e-01
       coeff_l4(2,18)=  3.9259493351e-02
       coeff_l4(2,19)=  8.7769073434e-04
       coeff_l4(2,20)= -2.3129185662e-02
       coeff_l4(2,21)= -1.1166385561e-01
       coeff_l4(2,22)=  5.1114275120e-03
       coeff_l4(2,23)=  1.7624151707e-01
       coeff_l4(2,24)= -2.4594249204e-02
       coeff_l4(2,25)=  1.2979520857e-01
       coeff_l4(2,26)= -6.1693556607e-02
       coeff_l4(2,27)= -1.8932078034e-02
       coeff_l4(2,28)=  7.8634977341e-02
       coeff_l4(2,29)=  2.7729926631e-02
       coeff_l4(2,30)=  2.1374024451e-02
       coeff_l4(2,31)= -2.8992574662e-02
       coeff_l4(2,32)=  5.9406694025e-02
       coeff_l4(2,33)=  6.2096182257e-02
       coeff_l4(2,34)= -4.1157085449e-02
       coeff_l4(2,35)= -5.7996258140e-02
       coeff_l4(2,36)=  6.3569724560e-02
       coeff_l4(2,37)=  1.6648536921e-01
       coeff_l4(2,38)= -7.6902456582e-02
       coeff_l4(2,39)= -1.3687436283e-01
       coeff_l4(2,40)= -1.3142707944e-01
       coeff_l4(2,41)=  1.5910306573e-01
       coeff_l4(2,42)=  3.6933213472e-02
       coeff_l4(2,43)=  7.5919236988e-03
       coeff_l4(2,44)=  1.3469134271e-01
       coeff_l4(2,45)=  1.3412387669e-01
       coeff_l4(2,46)= -3.0689692125e-02
       coeff_l4(2,47)= -7.0359833539e-02
       coeff_l4(2,48)= -7.9115189612e-02
       coeff_l4(2,49)=  1.3918118551e-02
       coeff_l4(2,50)=  8.5950188339e-02
       coeff_l4(2,51)=  3.8411475718e-02
       coeff_l4(2,52)=  1.1586122960e-01
       coeff_l4(2,53)= -1.0207906365e-01
       coeff_l4(2,54)= -7.9524181783e-02
       coeff_l4(2,55)= -6.3514895737e-02
       coeff_l4(2,56)=  1.5779052675e-01
       coeff_l4(2,57)= -1.4095318317e-01
       coeff_l4(2,58)= -2.0434357226e-02
       coeff_l4(2,59)= -6.7112203687e-03
       coeff_l4(2,60)= -2.1390415728e-02
       coeff_l4(2,61)= -2.3234620690e-02
       coeff_l4(2,62)=  1.3592277467e-01
       coeff_l4(2,63)=  2.1509341896e-02
       coeff_l4(2,64)=  8.8002197444e-02
       coeff_l4(2,65)=  5.8045212179e-02
       coeff_l4(2,66)= -4.8544887453e-02
       coeff_l4(2,67)= -1.3548347168e-02
       coeff_l4(2,68)=  4.8305734992e-02
       coeff_l4(2,69)= -1.9330123067e-01
       coeff_l4(2,70)=  1.2224823982e-01
       coeff_l4(2,71)=  2.0619638264e-02
       coeff_l4(2,72)= -3.9051003754e-02
       coeff_l4(2,73)=  9.7362482920e-03
       coeff_l4(2,74)= -5.2282519639e-02
       coeff_l4(2,75)= -1.0477133840e-01
       coeff_l4(2,76)=  2.3093493283e-01
       coeff_l4(2,77)= -9.3533627689e-02
       coeff_l4(2,78)=  5.8474514633e-02
       coeff_l4(2,79)= -6.2339685857e-02
       coeff_l4(2,80)= -1.2050738186e-01
       coeff_l4(2,81)= -1.2132433057e-01
       coeff_l4(2,82)=  1.9908867776e-01
       coeff_l4(2,83)= -1.6141892970e-01
       coeff_l4(2,84)= -6.4228079282e-03
       coeff_l4(2,85)= -1.5202009678e-01
       coeff_l4(2,86)=  1.6691210866e-01
       coeff_l4(2,87)= -9.6799343824e-02
       coeff_l4(2,88)=  5.1377579570e-02
       coeff_l4(2,89)=  8.2378331572e-03
       coeff_l4(2,90)=  1.1517845839e-01
       coeff_l4(2,91)=  1.1319238693e-01
       coeff_l4(2,92)=  3.2039839774e-02
       coeff_l4(2,93)= -1.4479041100e-02
       coeff_l4(2,94)=  1.0692308098e-01
       coeff_l4(2,95)= -1.2298928946e-01
       coeff_l4(2,96)=  1.4279809594e-01
       coeff_l4(2,97)= -2.2597150877e-02
       coeff_l4(2,98)= -1.1932353489e-02
       coeff_l4(2,99)=  4.4941123575e-02
       coeff_l4(2,100)=  1.0363866389e-01
       coeff_l4(2,101)=  4.9995373935e-02
       coeff_l4(2,102)= -4.7108653933e-02
       coeff_l4(2,103)= -8.1544965506e-02
       coeff_l4(2,104)=  5.6423708797e-02
       coeff_l4(2,105)=  1.1716516316e-01
       coeff_l4(3,1)=  2.6345765591e-01
       coeff_l4(3,2)=  2.1859979630e-01
       coeff_l4(3,3)= -4.0479081869e-01
       coeff_l4(3,4)=  3.7359228730e-01
       coeff_l4(3,5)=  1.4559458196e-01
       coeff_l4(3,6)=  8.7906464934e-02
       coeff_l4(3,7)=  2.4872885644e-01
       coeff_l4(3,8)= -1.6309128702e-01
       coeff_l4(3,9)= -1.1089503765e+00
       coeff_l4(3,10)= -3.2212764025e-01
       coeff_l4(3,11)= -1.7647872865e-01
       coeff_l4(3,12)= -8.1434950233e-02
       coeff_l4(3,13)= -1.8425446749e-01
       coeff_l4(3,14)= -8.9829385281e-01
       coeff_l4(3,15)=  6.7413501441e-02
       coeff_l4(3,16)=  3.4454040229e-02
       coeff_l4(3,17)=  1.4249642193e-01
       coeff_l4(3,18)=  4.3432867527e-01
       coeff_l4(3,19)= -3.3683739603e-02
       coeff_l4(3,20)=  1.3328501582e-01
       coeff_l4(3,21)= -2.9947847128e-01
       coeff_l4(3,22)=  1.1053287983e-01
       coeff_l4(3,23)= -3.4436860681e-01
       coeff_l4(3,24)= -1.3771110773e-01
       coeff_l4(3,25)=  3.8790377975e-01
       coeff_l4(3,26)= -1.7538455129e-01
       coeff_l4(3,27)=  5.8374259621e-02
       coeff_l4(3,28)= -1.3915814459e-01
       coeff_l4(3,29)= -1.5556161106e-01
       coeff_l4(3,30)=  2.9440850019e-01
       coeff_l4(3,31)= -1.3084411621e-01
       coeff_l4(3,32)=  9.7262468189e-03
       coeff_l4(3,33)=  1.4431887865e-01
       coeff_l4(3,34)= -3.6092871428e-01
       coeff_l4(3,35)=  2.4061498046e-01
       coeff_l4(3,36)=  3.2415792346e-02
       coeff_l4(3,37)= -1.8872343004e-01
       coeff_l4(3,38)=  2.5435879827e-01
       coeff_l4(3,39)=  5.6684815884e-01
       coeff_l4(3,40)= -3.5893976688e-01
       coeff_l4(3,41)= -9.5777302980e-02
       coeff_l4(3,42)=  2.4144031107e-01
       coeff_l4(3,43)=  4.0777358413e-01
       coeff_l4(3,44)= -1.6101795435e-01
       coeff_l4(3,45)=  2.6590240002e-01
       coeff_l4(3,46)= -7.7606505156e-01
       coeff_l4(3,47)= -2.7347940207e-01
       coeff_l4(3,48)=  2.3101039231e-01
       coeff_l4(3,49)= -2.3059329391e-01
       coeff_l4(3,50)= -5.8088386059e-01
       coeff_l4(3,51)= -2.7027377486e-01
       coeff_l4(3,52)= -2.9139301181e-01
       coeff_l4(3,53)= -2.2966764867e-01
       coeff_l4(3,54)=  2.2729168832e-01
       coeff_l4(3,55)=  3.6794835329e-01
       coeff_l4(3,56)= -7.1876995265e-02
       coeff_l4(3,57)=  4.0248423815e-02
       coeff_l4(3,58)= -3.3580896258e-01
       coeff_l4(3,59)= -4.5449995995e-01
       coeff_l4(3,60)=  1.2275919318e-02
       coeff_l4(3,61)= -1.8483661115e-01
       coeff_l4(3,62)=  9.9977895617e-02
       coeff_l4(3,63)=  1.5208770335e-01
       coeff_l4(3,64)=  1.4171519876e-01
       coeff_l4(3,65)=  4.2171314359e-01
       coeff_l4(3,66)= -1.6063834727e-01
       coeff_l4(3,67)=  1.1612245440e-01
       coeff_l4(3,68)= -3.3979949355e-01
       coeff_l4(3,69)=  6.3713140786e-02
       coeff_l4(3,70)=  8.2408115268e-02
       coeff_l4(3,71)=  2.2848619521e-01
       coeff_l4(3,72)= -5.0846654177e-01
       coeff_l4(3,73)= -1.1643634737e-01
       coeff_l4(3,74)= -1.0032653809e-01
       coeff_l4(3,75)= -4.2767855339e-03
       coeff_l4(3,76)=  1.3367372751e-01
       coeff_l4(3,77)=  4.0591487288e-01
       coeff_l4(3,78)=  1.6062065959e-01
       coeff_l4(3,79)= -1.2653358281e-01
       coeff_l4(3,80)= -9.4070009887e-02
       coeff_l4(3,81)= -2.0961092412e-01
       coeff_l4(3,82)=  2.6580166817e-01
       coeff_l4(3,83)=  1.3703924417e-01
       coeff_l4(3,84)=  4.8038902879e-01
       coeff_l4(3,85)= -2.1594317257e-01
       coeff_l4(3,86)=  3.8547399640e-01
       coeff_l4(3,87)= -4.6946752071e-01
       coeff_l4(3,88)= -1.2726657093e-01
       coeff_l4(3,89)=  9.5731213689e-02
       coeff_l4(3,90)= -1.9862712920e-01
       coeff_l4(3,91)=  1.8317820132e-01
       coeff_l4(3,92)= -7.6168733835e-01
       coeff_l4(3,93)= -1.8294969201e-01
       coeff_l4(3,94)=  1.2354690582e-01
       coeff_l4(3,95)= -2.6301452518e-01
       coeff_l4(3,96)=  2.1560962498e-01
       coeff_l4(3,97)=  7.1285361052e-01
       coeff_l4(3,98)= -5.3377318382e-01
       coeff_l4(3,99)=  2.4688866735e-01
       coeff_l4(3,100)= -9.9956110120e-02
       coeff_l4(3,101)=  9.3379639089e-02
       coeff_l4(3,102)= -1.3434499502e-02
       coeff_l4(3,103)=  1.8353685737e-01
       coeff_l4(3,104)=  2.1834339201e-01
       coeff_l4(3,105)= -2.5560942292e-01
       coeff_l4(4,1)=  6.2011027336e-01
       coeff_l4(4,2)=  7.3346441984e-01
       coeff_l4(4,3)=  7.5072294474e-01
       coeff_l4(4,4)=  8.3360332251e-01
       coeff_l4(4,5)=  5.9234327078e-01
       coeff_l4(4,6)=  8.4346568584e-01
       coeff_l4(4,7)=  2.6683336496e-01
       coeff_l4(4,8)=  3.9069819450e-01
       coeff_l4(4,9)=  1.9496951997e-01
       coeff_l4(4,10)=  2.7776056528e-01
       coeff_l4(4,11)= -2.0295293629e-01
       coeff_l4(4,12)=  1.8230816722e-01
       coeff_l4(4,13)= -1.3663727045e-01
       coeff_l4(4,14)=  1.7433376610e-01
       coeff_l4(4,15)=  1.5105505288e-01
       coeff_l4(4,16)=  2.4498400092e-01
       coeff_l4(4,17)= -3.3318296075e-01
       coeff_l4(4,18)=  1.5709160268e-01
       coeff_l4(4,19)= -8.2228228450e-02
       coeff_l4(4,20)=  1.0430528224e-01
       coeff_l4(4,21)= -5.7249769568e-02
       coeff_l4(4,22)= -4.6697277576e-02
       coeff_l4(4,23)=  8.2584775984e-02
       coeff_l4(4,24)=  1.4313136041e-01
       coeff_l4(4,25)= -4.4673405588e-02
       coeff_l4(4,26)=  6.8390719593e-02
       coeff_l4(4,27)= -4.2852494866e-02
       coeff_l4(4,28)=  1.7625771463e-01
       coeff_l4(4,29)= -1.4300976694e-01
       coeff_l4(4,30)=  1.0975941271e-01
       coeff_l4(4,31)=  1.2214570493e-01
       coeff_l4(4,32)= -4.2574115098e-02
       coeff_l4(4,33)=  8.0191627145e-02
       coeff_l4(4,34)=  2.7091380954e-01
       coeff_l4(4,35)=  1.8176017702e-01
       coeff_l4(4,36)=  8.0393843353e-02
       coeff_l4(4,37)=  2.3667255044e-01
       coeff_l4(4,38)=  3.0039390922e-01
       coeff_l4(4,39)=  3.7843510509e-02
       coeff_l4(4,40)=  1.1675193906e-01
       coeff_l4(4,41)= -9.1932281852e-02
       coeff_l4(4,42)= -2.1780456603e-01
       coeff_l4(4,43)=  1.1629692465e-01
       coeff_l4(4,44)=  1.1011517048e-01
       coeff_l4(4,45)= -1.4534907043e-01
       coeff_l4(4,46)=  1.3247147202e-01
       coeff_l4(4,47)=  1.7392399907e-01
       coeff_l4(4,48)=  9.4715259969e-02
       coeff_l4(4,49)=  1.4791470766e-01
       coeff_l4(4,50)= -9.4296559691e-03
       coeff_l4(4,51)= -2.0225843415e-02
       coeff_l4(4,52)=  9.2546520755e-03
       coeff_l4(4,53)=  3.5750981420e-02
       coeff_l4(4,54)= -1.3917197287e-01
       coeff_l4(4,55)= -9.2731922865e-02
       coeff_l4(4,56)= -5.7700328529e-02
       coeff_l4(4,57)= -2.3999357596e-02
       coeff_l4(4,58)=  8.3016060293e-02
       coeff_l4(4,59)=  1.3516551256e-01
       coeff_l4(4,60)=  7.7804520726e-02
       coeff_l4(4,61)=  1.0834541172e-01
       coeff_l4(4,62)= -8.7321981788e-02
       coeff_l4(4,63)= -4.0752839297e-02
       coeff_l4(4,64)=  9.9240625277e-03
       coeff_l4(4,65)= -3.0965566635e-02
       coeff_l4(4,66)= -2.1576608717e-01
       coeff_l4(4,67)= -9.5025800169e-02
       coeff_l4(4,68)=  1.3021788001e-01
       coeff_l4(4,69)= -8.6347647011e-02
       coeff_l4(4,70)=  8.0645464361e-02
       coeff_l4(4,71)=  5.7116061449e-02
       coeff_l4(4,72)= -3.3763968386e-03
       coeff_l4(4,73)= -1.8430288881e-02
       coeff_l4(4,74)=  6.9381413050e-03
       coeff_l4(4,75)= -3.2226893306e-01
       coeff_l4(4,76)= -6.6435463727e-02
       coeff_l4(4,77)= -1.5110071003e-01
       coeff_l4(4,78)= -7.4341282248e-02
       coeff_l4(4,79)=  2.3109093308e-01
       coeff_l4(4,80)= -4.9931216240e-01
       coeff_l4(4,81)= -1.6432877630e-02
       coeff_l4(4,82)=  1.5891243517e-01
       coeff_l4(4,83)= -5.3547095507e-02
       coeff_l4(4,84)= -2.5953346491e-01
       coeff_l4(4,85)=  1.7385828495e-01
       coeff_l4(4,86)= -6.3485294580e-02
       coeff_l4(4,87)=  3.1794422865e-01
       coeff_l4(4,88)=  1.0194455087e-01
       coeff_l4(4,89)=  3.3596155047e-01
       coeff_l4(4,90)= -1.2825410068e-01
       coeff_l4(4,91)= -2.3961164057e-02
       coeff_l4(4,92)=  2.0386964083e-01
       coeff_l4(4,93)= -3.1789336354e-02
       coeff_l4(4,94)=  2.7092024684e-01
       coeff_l4(4,95)= -4.3943682313e-01
       coeff_l4(4,96)= -9.0159118176e-02
       coeff_l4(4,97)=  1.9040979445e-01
       coeff_l4(4,98)=  3.3525586128e-02
       coeff_l4(4,99)=  6.6806107759e-02
       coeff_l4(4,100)=  2.8387394547e-01
       coeff_l4(4,101)=  3.2485410571e-02
       coeff_l4(4,102)= -9.5570720732e-02
       coeff_l4(4,103)=  2.5962796807e-01
       coeff_l4(4,104)=  2.1639201790e-02
       coeff_l4(4,105)= -1.6082264483e-01
       coeff_l4(5,1)= -6.4261764288e-01
       coeff_l4(5,2)= -4.2091029882e-01
       coeff_l4(5,3)= -5.8903598785e-01
       coeff_l4(5,4)= -5.2646654844e-01
       coeff_l4(5,5)= -4.6950852871e-01
       coeff_l4(5,6)= -5.9167164564e-01
       coeff_l4(5,7)= -4.3251952529e-01
       coeff_l4(5,8)= -6.0701674223e-01
       coeff_l4(5,9)=  4.0981639177e-02
       coeff_l4(5,10)= -5.9902825160e-04
       coeff_l4(5,11)= -2.8949868679e-01
       coeff_l4(5,12)= -1.9103185833e-01
       coeff_l4(5,13)= -8.2330554724e-02
       coeff_l4(5,14)= -2.4912908673e-01
       coeff_l4(5,15)=  2.5887179375e-01
       coeff_l4(5,16)= -9.5057167113e-02
       coeff_l4(5,17)=  1.5511803329e-01
       coeff_l4(5,18)=  2.3977284133e-01
       coeff_l4(5,19)= -1.6261842847e-01
       coeff_l4(5,20)=  6.4029999077e-02
       coeff_l4(5,21)=  1.5241721272e-01
       coeff_l4(5,22)=  1.2075528502e-01
       coeff_l4(5,23)= -6.0563921928e-02
       coeff_l4(5,24)=  8.1061013043e-02
       coeff_l4(5,25)=  1.1908063293e-01
       coeff_l4(5,26)=  2.4679969996e-02
       coeff_l4(5,27)=  1.0835991800e-01
       coeff_l4(5,28)= -2.6079615578e-02
       coeff_l4(5,29)=  4.1412144899e-02
       coeff_l4(5,30)=  5.1973029971e-02
       coeff_l4(5,31)= -1.5631084144e-01
       coeff_l4(5,32)= -5.3623795509e-02
       coeff_l4(5,33)= -1.0868693888e-01
       coeff_l4(5,34)= -2.5661259890e-02
       coeff_l4(5,35)= -1.7553994060e-01
       coeff_l4(5,36)= -7.6614670455e-02
       coeff_l4(5,37)= -1.0614587367e-01
       coeff_l4(5,38)= -2.0203357935e-01
       coeff_l4(5,39)= -2.2162896395e-01
       coeff_l4(5,40)= -1.0983043909e-01
       coeff_l4(5,41)= -1.4046519995e-02
       coeff_l4(5,42)= -2.3692954332e-02
       coeff_l4(5,43)=  3.2598014921e-02
       coeff_l4(5,44)= -1.0180901736e-01
       coeff_l4(5,45)= -4.9552634358e-02
       coeff_l4(5,46)= -4.1033517569e-02
       coeff_l4(5,47)= -1.2194250524e-01
       coeff_l4(5,48)= -2.2253584117e-02
       coeff_l4(5,49)= -1.1225587875e-01
       coeff_l4(5,50)= -1.5024934709e-01
       coeff_l4(5,51)=  1.3686579466e-01
       coeff_l4(5,52)=  1.6275559366e-01
       coeff_l4(5,53)=  5.9618704021e-02
       coeff_l4(5,54)=  4.3699041009e-02
       coeff_l4(5,55)=  3.6948610097e-02
       coeff_l4(5,56)=  6.1847403646e-02
       coeff_l4(5,57)= -1.7353929579e-01
       coeff_l4(5,58)= -7.9104349017e-02
       coeff_l4(5,59)=  6.2457133085e-02
       coeff_l4(5,60)= -2.9884219170e-01
       coeff_l4(5,61)= -3.5638924688e-02
       coeff_l4(5,62)=  1.2761719525e-01
       coeff_l4(5,63)= -1.4172720909e-01
       coeff_l4(5,64)=  6.8033419549e-02
       coeff_l4(5,65)= -2.2941719741e-02
       coeff_l4(5,66)= -1.0260895640e-01
       coeff_l4(5,67)=  1.1373487860e-01
       coeff_l4(5,68)= -2.9039351270e-02
       coeff_l4(5,69)= -9.6987180412e-02
       coeff_l4(5,70)=  4.5009341091e-02
       coeff_l4(5,71)= -3.4800533205e-02
       coeff_l4(5,72)= -7.9683503136e-03
       coeff_l4(5,73)= -1.2919241562e-02
       coeff_l4(5,74)= -2.5468766689e-02
       coeff_l4(5,75)=  6.6206194460e-02
       coeff_l4(5,76)=  1.6640089452e-01
       coeff_l4(5,77)= -9.7698710859e-02
       coeff_l4(5,78)= -1.1230008304e-01
       coeff_l4(5,79)= -7.6281507500e-03
       coeff_l4(5,80)= -9.2118419707e-02
       coeff_l4(5,81)= -1.4943913557e-03
       coeff_l4(5,82)=  9.7536645830e-02
       coeff_l4(5,83)=  1.2184133381e-01
       coeff_l4(5,84)= -5.1659092307e-02
       coeff_l4(5,85)= -2.1273382008e-01
       coeff_l4(5,86)= -3.2485987991e-02
       coeff_l4(5,87)= -3.0809601769e-02
       coeff_l4(5,88)= -1.0395719856e-01
       coeff_l4(5,89)=  5.9608478099e-02
       coeff_l4(5,90)= -7.2177641094e-02
       coeff_l4(5,91)= -1.0913821310e-01
       coeff_l4(5,92)= -1.8701235950e-01
       coeff_l4(5,93)= -6.3276708126e-02
       coeff_l4(5,94)=  2.7403089404e-01
       coeff_l4(5,95)=  9.6456639469e-02
       coeff_l4(5,96)= -1.3391222060e-01
       coeff_l4(5,97)=  1.7421083152e-01
       coeff_l4(5,98)= -4.1633408517e-02
       coeff_l4(5,99)=  2.3046372831e-01
       coeff_l4(5,100)= -1.3276810944e-01
       coeff_l4(5,101)=  1.7415931821e-01
       coeff_l4(5,102)=  5.6247111410e-02
       coeff_l4(5,103)=  2.2025096416e-01
       coeff_l4(5,104)= -3.5980880260e-02
       coeff_l4(5,105)= -1.0779312253e-01
       coeff_l4(6,1)= -5.5317801237e-01
       coeff_l4(6,2)= -4.9933326244e-01
       coeff_l4(6,3)= -5.9420013428e-01
       coeff_l4(6,4)= -8.4015059471e-01
       coeff_l4(6,5)= -7.2096818686e-01
       coeff_l4(6,6)= -7.8303378820e-01
       coeff_l4(6,7)= -6.2627971172e-01
       coeff_l4(6,8)= -7.9364776611e-01
       coeff_l4(6,9)= -1.9879665971e-01
       coeff_l4(6,10)= -3.8343033195e-01
       coeff_l4(6,11)= -2.8534477949e-01
       coeff_l4(6,12)= -4.2492040992e-01
       coeff_l4(6,13)= -1.6462697089e-01
       coeff_l4(6,14)= -2.9995799065e-01
       coeff_l4(6,15)= -1.3719078898e-01
       coeff_l4(6,16)= -1.7875714600e-01
       coeff_l4(6,17)= -3.5880491138e-02
       coeff_l4(6,18)=  7.2373688221e-02
       coeff_l4(6,19)= -1.6179179773e-02
       coeff_l4(6,20)=  4.4427331537e-02
       coeff_l4(6,21)= -3.1170692295e-02
       coeff_l4(6,22)=  8.0136626959e-02
       coeff_l4(6,23)= -9.6937432885e-02
       coeff_l4(6,24)=  1.5397875011e-01
       coeff_l4(6,25)=  1.9202666357e-02
       coeff_l4(6,26)=  5.5798862129e-02
       coeff_l4(6,27)= -1.4216051996e-01
       coeff_l4(6,28)=  7.8407563269e-03
       coeff_l4(6,29)= -1.9475452602e-02
       coeff_l4(6,30)=  5.8467105031e-02
       coeff_l4(6,31)= -1.1292354017e-01
       coeff_l4(6,32)=  9.5672287047e-02
       coeff_l4(6,33)=  6.3339443877e-03
       coeff_l4(6,34)=  1.6865614802e-02
       coeff_l4(6,35)=  5.9255543165e-03
       coeff_l4(6,36)= -6.5922245383e-02
       coeff_l4(6,37)=  1.2946384959e-02
       coeff_l4(6,38)= -1.0018242896e-01
       coeff_l4(6,39)=  1.1497107893e-01
       coeff_l4(6,40)= -8.1917904317e-02
       coeff_l4(6,41)= -4.0239598602e-02
       coeff_l4(6,42)=  8.0433465540e-02
       coeff_l4(6,43)=  8.9097261429e-02
       coeff_l4(6,44)=  1.2099224329e-01
       coeff_l4(6,45)= -1.6076797619e-02
       coeff_l4(6,46)=  8.4148652852e-02
       coeff_l4(6,47)= -6.5976753831e-04
       coeff_l4(6,48)= -1.7346747220e-02
       coeff_l4(6,49)=  7.3225110769e-02
       coeff_l4(6,50)= -6.1080623418e-02
       coeff_l4(6,51)=  2.1348153055e-01
       coeff_l4(6,52)=  6.3996851444e-02
       coeff_l4(6,53)=  7.8962683678e-02
       coeff_l4(6,54)=  1.3396151364e-01
       coeff_l4(6,55)=  9.1766163707e-02
       coeff_l4(6,56)=  1.2380722910e-01
       coeff_l4(6,57)= -8.5894428194e-02
       coeff_l4(6,58)=  3.8439803757e-03
       coeff_l4(6,59)=  4.2659904808e-02
       coeff_l4(6,60)= -1.3124865294e-01
       coeff_l4(6,61)= -1.5423306823e-01
       coeff_l4(6,62)= -4.5496813953e-02
       coeff_l4(6,63)=  2.3906396329e-01
       coeff_l4(6,64)=  1.6304311156e-01
       coeff_l4(6,65)=  2.4601733685e-01
       coeff_l4(6,66)=  9.0201459825e-02
       coeff_l4(6,67)=  9.6278786659e-02
       coeff_l4(6,68)=  8.1387676299e-02
       coeff_l4(6,69)=  3.0231406912e-02
       coeff_l4(6,70)= -1.0686661303e-01
       coeff_l4(6,71)= -2.7898052707e-02
       coeff_l4(6,72)=  1.2580761313e-01
       coeff_l4(6,73)= -6.4216300845e-02
       coeff_l4(6,74)= -2.8173658997e-02
       coeff_l4(6,75)= -6.2776677310e-02
       coeff_l4(6,76)=  8.0760754645e-02
       coeff_l4(6,77)= -1.7439438403e-01
       coeff_l4(6,78)=  7.1381062269e-02
       coeff_l4(6,79)=  4.0330043994e-03
       coeff_l4(6,80)= -9.5338724554e-02
       coeff_l4(6,81)= -7.6705557294e-03
       coeff_l4(6,82)=  4.5485667884e-02
       coeff_l4(6,83)= -8.7710291147e-02
       coeff_l4(6,84)= -4.9719922245e-03
       coeff_l4(6,85)= -1.3523126021e-02
       coeff_l4(6,86)= -2.9385017231e-02
       coeff_l4(6,87)= -5.8986034244e-02
       coeff_l4(6,88)=  1.0858562589e-01
       coeff_l4(6,89)=  2.7982831001e-02
       coeff_l4(6,90)=  1.2369464152e-02
       coeff_l4(6,91)= -4.5341420919e-02
       coeff_l4(6,92)=  1.8715517595e-02
       coeff_l4(6,93)= -2.4728573859e-02
       coeff_l4(6,94)=  2.8064042330e-01
       coeff_l4(6,95)=  1.3346463442e-01
       coeff_l4(6,96)= -2.6944258809e-01
       coeff_l4(6,97)=  2.8227066994e-01
       coeff_l4(6,98)= -6.3136406243e-02
       coeff_l4(6,99)=  2.6833382249e-01
       coeff_l4(6,100)= -1.2044036388e-01
       coeff_l4(6,101)= -9.6145682037e-02
       coeff_l4(6,102)=  1.4997333288e-01
       coeff_l4(6,103)=  3.6294203252e-02
       coeff_l4(6,104)=  7.7497467399e-02
       coeff_l4(6,105)=  7.5707145035e-02
       coeff_l4(7,1)=  5.5057093501e-02
       coeff_l4(7,2)=  1.8443794549e-01
       coeff_l4(7,3)=  1.8838378787e-01
       coeff_l4(7,4)=  2.3990243673e-01
       coeff_l4(7,5)=  1.6095449030e-01
       coeff_l4(7,6)=  3.8631609082e-01
       coeff_l4(7,7)=  4.2248943448e-01
       coeff_l4(7,8)=  1.5042467415e-01
       coeff_l4(7,9)= -7.9822546244e-01
       coeff_l4(7,10)= -5.1565504074e-01
       coeff_l4(7,11)= -3.1016354561e+00
       coeff_l4(7,12)= -3.2265751362e+00
       coeff_l4(7,13)= -2.9042899609e+00
       coeff_l4(7,14)=  5.9566657990e-02
       coeff_l4(7,15)= -3.9977338165e-02
       coeff_l4(7,16)= -2.5079021230e-02
       coeff_l4(7,17)=  1.7707289662e-03
       coeff_l4(7,18)= -1.7637571692e-01
       coeff_l4(7,19)=  1.7097165808e-02
       coeff_l4(7,20)=  1.2724792957e-01
       coeff_l4(7,21)= -1.2682861090e-01
       coeff_l4(7,22)= -1.9531738758e-01
       coeff_l4(7,23)=  3.8700278848e-02
       coeff_l4(7,24)=  3.0731309205e-02
       coeff_l4(7,25)= -1.1771164834e-01
       coeff_l4(7,26)= -1.0857196897e-01
       coeff_l4(7,27)=  1.4843177795e-01
       coeff_l4(7,28)= -6.2986709177e-02
       coeff_l4(7,29)= -3.3511269838e-02
       coeff_l4(7,30)=  1.6194152832e-01
       coeff_l4(7,31)=  8.8089786470e-02
       coeff_l4(7,32)=  1.0037428886e-01
       coeff_l4(7,33)=  1.0219533741e-01
       coeff_l4(7,34)=  4.6247560531e-02
       coeff_l4(7,35)= -4.7120075673e-02
       coeff_l4(7,36)=  2.1941187978e-01
       coeff_l4(7,37)=  1.0180310346e-02
       coeff_l4(7,38)=  1.4981342852e-01
       coeff_l4(7,39)=  3.6735314876e-02
       coeff_l4(7,40)=  1.5106299892e-02
       coeff_l4(7,41)= -6.9404907525e-02
       coeff_l4(7,42)=  4.4859007001e-02
       coeff_l4(7,43)= -7.7091626823e-02
       coeff_l4(7,44)=  4.3499566615e-02
       coeff_l4(7,45)=  3.2289654016e-02
       coeff_l4(7,46)= -3.1355265528e-02
       coeff_l4(7,47)=  2.1020685136e-01
       coeff_l4(7,48)=  3.7653040141e-02
       coeff_l4(7,49)= -2.3696210980e-01
       coeff_l4(7,50)=  1.7958506942e-01
       coeff_l4(7,51)=  1.0142227262e-01
       coeff_l4(7,52)= -1.3447917998e-02
       coeff_l4(7,53)= -1.3154670596e-01
       coeff_l4(7,54)=  1.8293676898e-02
       coeff_l4(7,55)=  2.5945952535e-01
       coeff_l4(7,56)= -6.0800299048e-02
       coeff_l4(7,57)=  1.5739470720e-01
       coeff_l4(7,58)= -2.5178983808e-01
       coeff_l4(7,59)=  5.8354396373e-02
       coeff_l4(7,60)=  9.0294852853e-02
       coeff_l4(7,61)= -9.8255261779e-02
       coeff_l4(7,62)=  8.2760199904e-02
       coeff_l4(7,63)=  9.3493582681e-03
       coeff_l4(7,64)= -2.4805776775e-02
       coeff_l4(7,65)= -7.5282283127e-02
       coeff_l4(7,66)=  1.4121295512e-01
       coeff_l4(7,67)=  3.0520206690e-01
       coeff_l4(7,68)=  1.0193470120e-01
       coeff_l4(7,69)= -1.4350502752e-03
       coeff_l4(7,70)=  2.9532995075e-02
       coeff_l4(7,71)=  4.7783520073e-02
       coeff_l4(7,72)=  2.7759566903e-02
       coeff_l4(7,73)=  1.7351147532e-01
       coeff_l4(7,74)= -3.2228747010e-01
       coeff_l4(7,75)=  3.2199048996e-01
       coeff_l4(7,76)= -1.9902345538e-01
       coeff_l4(7,77)= -2.8285844252e-02
       coeff_l4(7,78)= -6.5599001944e-02
       coeff_l4(7,79)= -4.2371995747e-02
       coeff_l4(7,80)=  1.4492703974e-01
       coeff_l4(7,81)= -6.4975239336e-02
       coeff_l4(7,82)= -4.5194160193e-02
       coeff_l4(7,83)=  1.4988619089e-01
       coeff_l4(7,84)=  2.5503817014e-03
       coeff_l4(7,85)=  9.5973350108e-02
       coeff_l4(7,86)=  1.2389144301e-01
       coeff_l4(7,87)= -7.7252961695e-02
       coeff_l4(7,88)= -1.6292399168e-01
       coeff_l4(7,89)= -2.2915057838e-01
       coeff_l4(7,90)=  2.7394971251e-01
       coeff_l4(7,91)=  2.6137556881e-02
       coeff_l4(7,92)=  4.6895635128e-01
       coeff_l4(7,93)= -5.5200201273e-01
       coeff_l4(7,94)= -1.6706323624e+00
       coeff_l4(7,95)=  1.2551140040e-02
       coeff_l4(7,96)= -1.2301479578e+00
       coeff_l4(7,97)= -1.1126950979e+00
       coeff_l4(7,98)=  1.8872308731e-01
       coeff_l4(7,99)=  7.8739382327e-02
       coeff_l4(7,100)=  4.0445104241e-01
       coeff_l4(7,101)= -2.6569548249e-01
       coeff_l4(7,102)= -2.3151932657e-01
       coeff_l4(7,103)=  1.0062740743e-01
       coeff_l4(7,104)=  3.1264033914e-01
       coeff_l4(7,105)= -7.6140940189e-02
       coeff_l4(8,1)=  7.5292640924e-01
       coeff_l4(8,2)=  6.8862527609e-01
       coeff_l4(8,3)= -1.0481493548e-02
       coeff_l4(8,4)=  2.6232764125e-01
       coeff_l4(8,5)=  3.4395140409e-01
       coeff_l4(8,6)=  4.5677164197e-01
       coeff_l4(8,7)=  3.5586214066e-01
       coeff_l4(8,8)=  2.4082921445e-01
       coeff_l4(8,9)=  5.7698756456e-01
       coeff_l4(8,10)=  1.0866000652e+00
       coeff_l4(8,11)=  5.9457325935e-01
       coeff_l4(8,12)=  4.4583261013e-01
       coeff_l4(8,13)=  3.7010639906e-01
       coeff_l4(8,14)=  7.5925076008e-01
       coeff_l4(8,15)= -1.1591076106e-01
       coeff_l4(8,16)=  1.1042752862e-01
       coeff_l4(8,17)= -1.7086665332e-01
       coeff_l4(8,18)=  4.2609184980e-01
       coeff_l4(8,19)=  1.9375561178e-01
       coeff_l4(8,20)=  5.7409632951e-02
       coeff_l4(8,21)=  3.5605987906e-01
       coeff_l4(8,22)= -8.6074106395e-02
       coeff_l4(8,23)=  4.2862510681e-01
       coeff_l4(8,24)= -1.6790558398e-01
       coeff_l4(8,25)=  1.0779551417e-01
       coeff_l4(8,26)= -1.8478222191e-01
       coeff_l4(8,27)= -2.3509806395e-01
       coeff_l4(8,28)=  5.8440860361e-02
       coeff_l4(8,29)=  2.5779974461e-01
       coeff_l4(8,30)= -1.0302031785e-01
       coeff_l4(8,31)= -3.9559450746e-01
       coeff_l4(8,32)=  2.6438098401e-02
       coeff_l4(8,33)=  2.5385640562e-02
       coeff_l4(8,34)= -1.7621402442e-01
       coeff_l4(8,35)= -1.7982436717e-01
       coeff_l4(8,36)= -1.8512234092e-01
       coeff_l4(8,37)=  4.6657267958e-02
       coeff_l4(8,38)= -1.0842266679e-01
       coeff_l4(8,39)= -4.2829412967e-02
       coeff_l4(8,40)= -1.1704628170e-01
       coeff_l4(8,41)=  6.3280840404e-03
       coeff_l4(8,42)= -1.9995699823e-01
       coeff_l4(8,43)=  1.2886849046e-01
       coeff_l4(8,44)= -4.0233496577e-02
       coeff_l4(8,45)=  5.2040454000e-02
       coeff_l4(8,46)=  6.0568302870e-02
       coeff_l4(8,47)=  1.9028612971e-01
       coeff_l4(8,48)=  6.7019388080e-02
       coeff_l4(8,49)=  2.4828167260e-01
       coeff_l4(8,50)=  7.6743662357e-02
       coeff_l4(8,51)= -9.3147441745e-02
       coeff_l4(8,52)= -1.0191769153e-01
       coeff_l4(8,53)= -3.0436953902e-01
       coeff_l4(8,54)= -2.9704681039e-01
       coeff_l4(8,55)=  1.9277273118e-01
       coeff_l4(8,56)= -5.2765536308e-01
       coeff_l4(8,57)= -2.3595078290e-01
       coeff_l4(8,58)=  4.0442097932e-02
       coeff_l4(8,59)= -1.5785585344e-01
       coeff_l4(8,60)=  2.7902690694e-02
       coeff_l4(8,61)=  2.3785486817e-01
       coeff_l4(8,62)=  3.4530892968e-01
       coeff_l4(8,63)=  6.2934957445e-02
       coeff_l4(8,64)=  8.1731699407e-02
       coeff_l4(8,65)= -2.3264115676e-02
       coeff_l4(8,66)=  7.7665798366e-02
       coeff_l4(8,67)= -1.3798110187e-01
       coeff_l4(8,68)= -5.0711729564e-03
       coeff_l4(8,69)=  5.1326639950e-02
       coeff_l4(8,70)= -1.0535745881e-02
       coeff_l4(8,71)= -1.0859108716e-01
       coeff_l4(8,72)=  1.0373160243e-01
       coeff_l4(8,73)=  2.0096172392e-01
       coeff_l4(8,74)=  5.7548288256e-02
       coeff_l4(8,75)=  3.2339211553e-02
       coeff_l4(8,76)= -1.2197381258e-01
       coeff_l4(8,77)=  1.1067312956e-01
       coeff_l4(8,78)=  4.0394383669e-01
       coeff_l4(8,79)= -2.2382427752e-01
       coeff_l4(8,80)=  3.3307906985e-01
       coeff_l4(8,81)=  9.8524838686e-02
       coeff_l4(8,82)= -8.4380589426e-02
       coeff_l4(8,83)= -2.5286799669e-01
       coeff_l4(8,84)= -1.8261185288e-01
       coeff_l4(8,85)=  3.8489781320e-02
       coeff_l4(8,86)= -4.9546372145e-02
       coeff_l4(8,87)= -4.3232154101e-02
       coeff_l4(8,88)=  2.8642830253e-01
       coeff_l4(8,89)=  7.2554834187e-02
       coeff_l4(8,90)=  1.4862759411e-01
       coeff_l4(8,91)= -7.7406436205e-02
       coeff_l4(8,92)=  1.1059145629e-01
       coeff_l4(8,93)= -7.5696200132e-02
       coeff_l4(8,94)= -1.6492393613e-01
       coeff_l4(8,95)=  3.5348090529e-01
       coeff_l4(8,96)=  4.1262331605e-01
       coeff_l4(8,97)=  1.4531053603e-01
       coeff_l4(8,98)= -5.6106925011e-02
       coeff_l4(8,99)= -3.3463507891e-01
       coeff_l4(8,100)=  1.2032429874e-01
       coeff_l4(8,101)=  1.2839818001e-01
       coeff_l4(8,102)=  4.0482991934e-01
       coeff_l4(8,103)= -1.0375697166e-01
       coeff_l4(8,104)=  8.8310234249e-02
       coeff_l4(8,105)=  7.5035852194e-01
       coeff_l4(9,1)= -5.1693820953e-01
       coeff_l4(9,2)=  9.5524027944e-02
       coeff_l4(9,3)= -2.4014349282e-01
       coeff_l4(9,4)= -3.1495863199e-01
       coeff_l4(9,5)= -2.4443331361e-01
       coeff_l4(9,6)= -3.5632148385e-01
       coeff_l4(9,7)= -3.7269312143e-01
       coeff_l4(9,8)= -1.4603196084e-01
       coeff_l4(9,9)= -2.0984316245e-02
       coeff_l4(9,10)= -1.2173807621e-01
       coeff_l4(9,11)=  2.2628515959e-01
       coeff_l4(9,12)=  1.9640772045e-01
       coeff_l4(9,13)=  5.3735226393e-02
       coeff_l4(9,14)=  1.4333386719e-01
       coeff_l4(9,15)=  7.0656955242e-02
       coeff_l4(9,16)= -1.2705317140e-01
       coeff_l4(9,17)= -2.1911808848e-01
       coeff_l4(9,18)= -4.4154088944e-02
       coeff_l4(9,19)= -1.1033286154e-01
       coeff_l4(9,20)= -6.4851194620e-02
       coeff_l4(9,21)=  2.0774149895e-01
       coeff_l4(9,22)=  1.1719647050e-01
       coeff_l4(9,23)= -2.9155666009e-02
       coeff_l4(9,24)=  3.9558969438e-02
       coeff_l4(9,25)=  4.5074131340e-02
       coeff_l4(9,26)= -2.7981076390e-02
       coeff_l4(9,27)=  1.1335467547e-01
       coeff_l4(9,28)=  1.2735210359e-01
       coeff_l4(9,29)= -3.0458489060e-01
       coeff_l4(9,30)=  9.8267339170e-02
       coeff_l4(9,31)= -1.2431868166e-01
       coeff_l4(9,32)=  3.6925755441e-02
       coeff_l4(9,33)=  2.2903537750e-01
       coeff_l4(9,34)=  1.9916130602e-01
       coeff_l4(9,35)=  1.1527305841e-01
       coeff_l4(9,36)= -8.4382191300e-02
       coeff_l4(9,37)= -1.8613995984e-02
       coeff_l4(9,38)=  2.7755996585e-01
       coeff_l4(9,39)= -1.3785204291e-01
       coeff_l4(9,40)= -1.5567165613e-01
       coeff_l4(9,41)= -7.6314814389e-02
       coeff_l4(9,42)= -5.6870941073e-02
       coeff_l4(9,43)=  3.7362050265e-02
       coeff_l4(9,44)= -1.1733529717e-01
       coeff_l4(9,45)= -1.6848179698e-01
       coeff_l4(9,46)= -8.8719822466e-02
       coeff_l4(9,47)=  5.2594590932e-02
       coeff_l4(9,48)=  1.6461126506e-01
       coeff_l4(9,49)=  2.0122489333e-01
       coeff_l4(9,50)=  1.1430106312e-01
       coeff_l4(9,51)=  1.8816766143e-01
       coeff_l4(9,52)= -1.0658068955e-01
       coeff_l4(9,53)=  4.4761542231e-02
       coeff_l4(9,54)=  1.5070548654e-01
       coeff_l4(9,55)= -8.4420830011e-02
       coeff_l4(9,56)= -9.2661697417e-03
       coeff_l4(9,57)= -1.8943502009e-01
       coeff_l4(9,58)= -1.2441594154e-02
       coeff_l4(9,59)=  2.1341274679e-01
       coeff_l4(9,60)=  1.0363861918e-01
       coeff_l4(9,61)= -1.2362511456e-01
       coeff_l4(9,62)=  1.0412593931e-01
       coeff_l4(9,63)= -2.7085936069e-01
       coeff_l4(9,64)=  6.1305098236e-02
       coeff_l4(9,65)= -1.0275031626e-01
       coeff_l4(9,66)= -4.4098474085e-02
       coeff_l4(9,67)=  1.0330336541e-01
       coeff_l4(9,68)=  1.7391119897e-01
       coeff_l4(9,69)=  1.7717802525e-01
       coeff_l4(9,70)=  7.7032998204e-02
       coeff_l4(9,71)=  9.0468041599e-02
       coeff_l4(9,72)= -1.5394197404e-01
       coeff_l4(9,73)=  1.2874554843e-02
       coeff_l4(9,74)= -4.2201690376e-02
       coeff_l4(9,75)= -4.2882375419e-02
       coeff_l4(9,76)= -8.5763961077e-02
       coeff_l4(9,77)= -1.1086961627e-01
       coeff_l4(9,78)= -4.2798671871e-02
       coeff_l4(9,79)= -3.1439282000e-02
       coeff_l4(9,80)= -4.2147707194e-02
       coeff_l4(9,81)= -1.6174422204e-01
       coeff_l4(9,82)=  2.0240755752e-02
       coeff_l4(9,83)= -6.0146197677e-02
       coeff_l4(9,84)= -1.4592984319e-01
       coeff_l4(9,85)=  9.1657310724e-02
       coeff_l4(9,86)= -3.1430619955e-01
       coeff_l4(9,87)= -9.2278175056e-02
       coeff_l4(9,88)=  8.3805575967e-02
       coeff_l4(9,89)= -2.6442411542e-01
       coeff_l4(9,90)=  2.2552600503e-01
       coeff_l4(9,91)= -4.4816467911e-02
       coeff_l4(9,92)=  4.7626975924e-02
       coeff_l4(9,93)=  2.8608307242e-01
       coeff_l4(9,94)=  4.5593317598e-02
       coeff_l4(9,95)=  1.5471853316e-01
       coeff_l4(9,96)=  3.7887789309e-02
       coeff_l4(9,97)=  3.5189487040e-02
       coeff_l4(9,98)=  3.7641227245e-02
       coeff_l4(9,99)= -7.5522042811e-02
       coeff_l4(9,100)= -2.1409720182e-02
       coeff_l4(9,101)= -5.1055010408e-02
       coeff_l4(9,102)= -4.4694717973e-02
       coeff_l4(9,103)=  1.9537431002e-01
       coeff_l4(9,104)=  1.0304968804e-01
       coeff_l4(9,105)= -7.9012356699e-02
       coeff_l4(10,1)=  6.3856667280e-01
       coeff_l4(10,2)=  3.9863362908e-01
       coeff_l4(10,3)=  4.1824477911e-01
       coeff_l4(10,4)=  4.0897232294e-01
       coeff_l4(10,5)= -1.5191279352e-02
       coeff_l4(10,6)=  1.2548714876e-01
       coeff_l4(10,7)=  7.3003745079e-01
       coeff_l4(10,8)=  3.6443498731e-01
       coeff_l4(10,9)= -9.8422074318e-01
       coeff_l4(10,10)= -8.2796901464e-01
       coeff_l4(10,11)=  5.7372476906e-02
       coeff_l4(10,12)=  6.0389053077e-02
       coeff_l4(10,13)= -4.9529305100e-01
       coeff_l4(10,14)= -6.7485642433e-01
       coeff_l4(10,15)=  3.2459139824e-02
       coeff_l4(10,16)=  2.4621820450e-01
       coeff_l4(10,17)= -1.0788469017e-01
       coeff_l4(10,18)=  1.3008141518e-01
       coeff_l4(10,19)=  9.5180027187e-02
       coeff_l4(10,20)=  1.6646741331e-01
       coeff_l4(10,21)=  1.3229709864e-01
       coeff_l4(10,22)= -1.4683780074e-01
       coeff_l4(10,23)= -1.4144575596e-01
       coeff_l4(10,24)=  6.4214661717e-02
       coeff_l4(10,25)=  4.0285900235e-01
       coeff_l4(10,26)=  1.0695927590e-01
       coeff_l4(10,27)= -2.8299465775e-01
       coeff_l4(10,28)= -1.3684062287e-02
       coeff_l4(10,29)= -3.1352993846e-01
       coeff_l4(10,30)=  1.0172735900e-01
       coeff_l4(10,31)=  2.7195861936e-01
       coeff_l4(10,32)=  4.2400839925e-01
       coeff_l4(10,33)= -5.2133956924e-03
       coeff_l4(10,34)=  2.1686097607e-02
       coeff_l4(10,35)= -1.7319969833e-01
       coeff_l4(10,36)= -3.2661423087e-01
       coeff_l4(10,37)=  4.8546540737e-01
       coeff_l4(10,38)=  2.4923732877e-01
       coeff_l4(10,39)= -4.4848322868e-02
       coeff_l4(10,40)= -1.6713824868e-01
       coeff_l4(10,41)=  2.1179278195e-01
       coeff_l4(10,42)= -6.9656134583e-03
       coeff_l4(10,43)=  1.4004293084e-01
       coeff_l4(10,44)= -1.0617225617e-01
       coeff_l4(10,45)=  1.3705015182e-01
       coeff_l4(10,46)=  1.1331252754e-01
       coeff_l4(10,47)=  1.1562146246e-01
       coeff_l4(10,48)=  1.3738342561e-02
       coeff_l4(10,49)= -1.7023950815e-01
       coeff_l4(10,50)= -4.0550458431e-01
       coeff_l4(10,51)= -4.3952941895e-02
       coeff_l4(10,52)=  1.9408211112e-01
       coeff_l4(10,53)=  1.0895990580e-01
       coeff_l4(10,54)= -5.9305503964e-02
       coeff_l4(10,55)= -5.6442320347e-01
       coeff_l4(10,56)= -1.2963159010e-02
       coeff_l4(10,57)= -3.5309544206e-01
       coeff_l4(10,58)=  1.6287678853e-02
       coeff_l4(10,59)=  1.2336536497e-01
       coeff_l4(10,60)= -2.9835641384e-01
       coeff_l4(10,61)= -1.2216661125e-01
       coeff_l4(10,62)= -2.3350895941e-01
       coeff_l4(10,63)=  1.3307861984e-01
       coeff_l4(10,64)=  9.8864855245e-03
       coeff_l4(10,65)= -2.1784095094e-02
       coeff_l4(10,66)=  2.2804199159e-01
       coeff_l4(10,67)=  1.4021314681e-01
       coeff_l4(10,68)= -2.9046395421e-01
       coeff_l4(10,69)=  2.8247952461e-02
       coeff_l4(10,70)= -5.8746087551e-01
       coeff_l4(10,71)= -2.1891112626e-01
       coeff_l4(10,72)= -3.1379655004e-01
       coeff_l4(10,73)= -1.9288752973e-01
       coeff_l4(10,74)=  9.7066199780e-01
       coeff_l4(10,75)=  7.5895622373e-02
       coeff_l4(10,76)=  2.9024507850e-02
       coeff_l4(10,77)=  2.4038827419e-01
       coeff_l4(10,78)= -2.0278322697e-01
       coeff_l4(10,79)=  3.9255964756e-01
       coeff_l4(10,80)= -7.6485984027e-02
       coeff_l4(10,81)=  3.4754339606e-02
       coeff_l4(10,82)= -5.5658478290e-02
       coeff_l4(10,83)= -2.4198406935e-01
       coeff_l4(10,84)= -2.7270141244e-01
       coeff_l4(10,85)=  1.9694377482e-01
       coeff_l4(10,86)= -3.7958967686e-01
       coeff_l4(10,87)=  9.9194027483e-02
       coeff_l4(10,88)= -1.7995488644e-01
       coeff_l4(10,89)= -3.8207039237e-01
       coeff_l4(10,90)=  1.5637673438e-02
       coeff_l4(10,91)=  9.8406761885e-02
       coeff_l4(10,92)=  2.6696661115e-01
       coeff_l4(10,93)=  3.7223964930e-01
       coeff_l4(10,94)=  7.1371711791e-02
       coeff_l4(10,95)= -1.6829611361e-01
       coeff_l4(10,96)=  8.3933748305e-02
       coeff_l4(10,97)= -1.0220355541e-01
       coeff_l4(10,98)=  3.9862185717e-01
       coeff_l4(10,99)=  3.9092126489e-01
       coeff_l4(10,100)= -3.1109948177e-03
       coeff_l4(10,101)= -1.3919301331e-01
       coeff_l4(10,102)=  2.9493495822e-01
       coeff_l4(10,103)= -3.8540009409e-02
       coeff_l4(10,104)= -1.1422558129e-01
       coeff_l4(10,105)= -1.9275933504e-01
       coeff_l4(11,1)= -1.1710676551e-01
       coeff_l4(11,2)=  2.3589262366e-01
       coeff_l4(11,3)= -2.6057105511e-02
       coeff_l4(11,4)= -2.0517550409e-02
       coeff_l4(11,5)=  2.2928167135e-02
       coeff_l4(11,6)=  1.3567465544e-01
       coeff_l4(11,7)=  2.9724004865e-01
       coeff_l4(11,8)=  9.0247489512e-02
       coeff_l4(11,9)= -1.5334682167e-01
       coeff_l4(11,10)= -3.7695786357e-01
       coeff_l4(11,11)= -2.0399682224e-01
       coeff_l4(11,12)= -1.1773803830e-01
       coeff_l4(11,13)= -2.0364977419e-01
       coeff_l4(11,14)= -1.0678567737e-01
       coeff_l4(11,15)= -5.2780028433e-02
       coeff_l4(11,16)= -8.2447148860e-02
       coeff_l4(11,17)= -1.2198705226e-01
       coeff_l4(11,18)= -5.4102066904e-02
       coeff_l4(11,19)=  5.8952447027e-02
       coeff_l4(11,20)= -1.4960280061e-01
       coeff_l4(11,21)= -1.2244814634e-01
       coeff_l4(11,22)= -1.7602866888e-01
       coeff_l4(11,23)=  2.0271742344e-01
       coeff_l4(11,24)= -3.2827326655e-01
       coeff_l4(11,25)=  2.7105156332e-03
       coeff_l4(11,26)= -1.3947065175e-01
       coeff_l4(11,27)=  1.2027499825e-01
       coeff_l4(11,28)=  2.0139442757e-02
       coeff_l4(11,29)=  2.8657238930e-02
       coeff_l4(11,30)=  2.3175020516e-01
       coeff_l4(11,31)= -3.8746792078e-01
       coeff_l4(11,32)=  2.2387821227e-02
       coeff_l4(11,33)=  2.0824988186e-01
       coeff_l4(11,34)=  7.2431564331e-02
       coeff_l4(11,35)=  1.6933876276e-01
       coeff_l4(11,36)=  6.5631218255e-02
       coeff_l4(11,37)=  3.8218755275e-02
       coeff_l4(11,38)= -1.1915085452e-05
       coeff_l4(11,39)= -1.3825349510e-01
       coeff_l4(11,40)=  1.4331860840e-01
       coeff_l4(11,41)=  1.6848841682e-02
       coeff_l4(11,42)= -2.2197808325e-01
       coeff_l4(11,43)= -3.6967244744e-01
       coeff_l4(11,44)= -1.2421379238e-01
       coeff_l4(11,45)=  1.4713831246e-01
       coeff_l4(11,46)= -2.3741172627e-02
       coeff_l4(11,47)= -1.7976826429e-01
       coeff_l4(11,48)= -5.1229845732e-02
       coeff_l4(11,49)=  2.8822848201e-01
       coeff_l4(11,50)= -7.8283019364e-02
       coeff_l4(11,51)= -4.0552178025e-01
       coeff_l4(11,52)= -5.7084716856e-02
       coeff_l4(11,53)=  4.8798460513e-02
       coeff_l4(11,54)=  1.6048662364e-01
       coeff_l4(11,55)= -9.7878247499e-02
       coeff_l4(11,56)=  1.1227846146e-01
       coeff_l4(11,57)=  3.3346074820e-01
       coeff_l4(11,58)= -1.0649098456e-01
       coeff_l4(11,59)= -3.1066411734e-01
       coeff_l4(11,60)=  1.9138145447e-01
       coeff_l4(11,61)=  2.1791785955e-01
       coeff_l4(11,62)= -2.9540097341e-02
       coeff_l4(11,63)= -3.4864199162e-01
       coeff_l4(11,64)=  7.9016305972e-04
       coeff_l4(11,65)= -5.9204380959e-03
       coeff_l4(11,66)= -7.6436907053e-02
       coeff_l4(11,67)=  2.8371863067e-02
       coeff_l4(11,68)= -2.5462210178e-02
       coeff_l4(11,69)=  9.9722012877e-02
       coeff_l4(11,70)= -1.6616632044e-01
       coeff_l4(11,71)= -3.4222323447e-02
       coeff_l4(11,72)= -1.0482756793e-01
       coeff_l4(11,73)=  5.8317165822e-02
       coeff_l4(11,74)= -2.5094741583e-01
       coeff_l4(11,75)=  6.2199248932e-03
       coeff_l4(11,76)= -1.5514522791e-01
       coeff_l4(11,77)= -2.1465477347e-01
       coeff_l4(11,78)=  2.6706509292e-02
       coeff_l4(11,79)=  3.5099588335e-02
       coeff_l4(11,80)= -6.9056443870e-02
       coeff_l4(11,81)= -1.6536000371e-01
       coeff_l4(11,82)=  2.1270738542e-01
       coeff_l4(11,83)=  2.9282695055e-01
       coeff_l4(11,84)=  1.8202823400e-01
       coeff_l4(11,85)= -1.4050032198e-01
       coeff_l4(11,86)= -5.2634179592e-02
       coeff_l4(11,87)= -8.6239889264e-02
       coeff_l4(11,88)=  1.9843988121e-01
       coeff_l4(11,89)= -6.2458690256e-02
       coeff_l4(11,90)=  2.4253030121e-01
       coeff_l4(11,91)=  1.5919107199e-01
       coeff_l4(11,92)= -8.7912641466e-03
       coeff_l4(11,93)=  9.2020295560e-02
       coeff_l4(11,94)=  5.8024164289e-02
       coeff_l4(11,95)= -1.0395944864e-01
       coeff_l4(11,96)=  8.8477402925e-02
       coeff_l4(11,97)= -7.4852004647e-02
       coeff_l4(11,98)= -1.5238183737e-01
       coeff_l4(11,99)=  1.2077944726e-01
       coeff_l4(11,100)=  2.9668682814e-01
       coeff_l4(11,101)=  8.2494109869e-02
       coeff_l4(11,102)= -1.7733098567e-01
       coeff_l4(11,103)= -6.5934495069e-04
       coeff_l4(11,104)= -1.8982483447e-01
       coeff_l4(11,105)=  9.7446450964e-03
       coeff_l4(12,1)=  6.2522214651e-01
       coeff_l4(12,2)=  6.1849361658e-01
       coeff_l4(12,3)=  7.6960670948e-01
       coeff_l4(12,4)=  5.7269763947e-01
       coeff_l4(12,5)=  5.8987605572e-01
       coeff_l4(12,6)=  4.9588814378e-01
       coeff_l4(12,7)=  1.3434697390e+00
       coeff_l4(12,8)=  1.1753334999e+00
       coeff_l4(12,9)=  7.3534607887e-01
       coeff_l4(12,10)=  6.1601608992e-01
       coeff_l4(12,11)= -1.0787173510e+00
       coeff_l4(12,12)= -3.1941026449e-01
       coeff_l4(12,13)= -8.3414152265e-02
       coeff_l4(12,14)=  4.8529812694e-01
       coeff_l4(12,15)= -4.9258701503e-02
       coeff_l4(12,16)=  7.2727188468e-02
       coeff_l4(12,17)=  6.4188152552e-02
       coeff_l4(12,18)=  1.3089486957e-01
       coeff_l4(12,19)= -6.4052879810e-02
       coeff_l4(12,20)=  2.0706880093e-01
       coeff_l4(12,21)= -1.5643529594e-01
       coeff_l4(12,22)=  1.0747116804e-01
       coeff_l4(12,23)= -2.7114921808e-01
       coeff_l4(12,24)=  1.0241777636e-02
       coeff_l4(12,25)= -3.6377735436e-02
       coeff_l4(12,26)=  1.9405421615e-01
       coeff_l4(12,27)=  1.3049086928e-01
       coeff_l4(12,28)=  2.8745111823e-01
       coeff_l4(12,29)=  3.0611759052e-02
       coeff_l4(12,30)= -2.9420557618e-01
       coeff_l4(12,31)=  8.9293032885e-02
       coeff_l4(12,32)= -9.7625322640e-02
       coeff_l4(12,33)= -8.9513026178e-02
       coeff_l4(12,34)= -1.9185635448e-01
       coeff_l4(12,35)= -2.7848365903e-01
       coeff_l4(12,36)= -3.8452666998e-01
       coeff_l4(12,37)=  1.3929230045e-04
       coeff_l4(12,38)=  4.1626032442e-02
       coeff_l4(12,39)= -3.3890210092e-02
       coeff_l4(12,40)=  2.9002937675e-01
       coeff_l4(12,41)= -7.4499092996e-02
       coeff_l4(12,42)= -5.5340390652e-02
       coeff_l4(12,43)= -5.4527509212e-01
       coeff_l4(12,44)=  1.9655707479e-01
       coeff_l4(12,45)= -4.2117783427e-01
       coeff_l4(12,46)=  9.9035046995e-02
       coeff_l4(12,47)=  1.0413601249e-01
       coeff_l4(12,48)= -2.4172158912e-02
       coeff_l4(12,49)=  7.2984732687e-02
       coeff_l4(12,50)= -1.2112342566e-01
       coeff_l4(12,51)= -4.3934691697e-02
       coeff_l4(12,52)= -1.4144903421e-01
       coeff_l4(12,53)= -3.7758854032e-01
       coeff_l4(12,54)=  5.8608907275e-03
       coeff_l4(12,55)= -1.6805727780e-01
       coeff_l4(12,56)=  1.5216757357e-01
       coeff_l4(12,57)=  4.1821515560e-01
       coeff_l4(12,58)=  1.9820351899e-01
       coeff_l4(12,59)= -6.9561325014e-02
       coeff_l4(12,60)= -1.8668906391e-01
       coeff_l4(12,61)= -4.3700572103e-02
       coeff_l4(12,62)= -1.2880256772e-01
       coeff_l4(12,63)=  3.2369443774e-01
       coeff_l4(12,64)=  1.0171989352e-01
       coeff_l4(12,65)= -1.7546656728e-01
       coeff_l4(12,66)= -3.7949216366e-01
       coeff_l4(12,67)=  7.7448362112e-01
       coeff_l4(12,68)= -2.0877081156e-01
       coeff_l4(12,69)= -2.5094354153e-01
       coeff_l4(12,70)= -1.0195042938e-01
       coeff_l4(12,71)= -3.6263251305e-01
       coeff_l4(12,72)=  1.4938069880e-01
       coeff_l4(12,73)=  2.5457614660e-01
       coeff_l4(12,74)=  3.2567515969e-01
       coeff_l4(12,75)= -1.5014992654e-01
       coeff_l4(12,76)=  3.1665214896e-01
       coeff_l4(12,77)=  3.8858079910e-01
       coeff_l4(12,78)= -4.7959524393e-01
       coeff_l4(12,79)= -1.7897179723e-01
       coeff_l4(12,80)= -2.7920636535e-01
       coeff_l4(12,81)=  2.1263990551e-02
       coeff_l4(12,82)= -2.0822940767e-01
       coeff_l4(12,83)=  1.1923950166e-01
       coeff_l4(12,84)=  8.7177082896e-03
       coeff_l4(12,85)= -4.6628632117e-04
       coeff_l4(12,86)= -1.2662526965e-01
       coeff_l4(12,87)=  5.0662147999e-01
       coeff_l4(12,88)= -3.1892812252e-01
       coeff_l4(12,89)=  1.1260637641e-01
       coeff_l4(12,90)= -7.1424186230e-02
       coeff_l4(12,91)=  1.9370836020e-01
       coeff_l4(12,92)= -3.3499041200e-01
       coeff_l4(12,93)= -2.6232603192e-01
       coeff_l4(12,94)=  5.1463697106e-02
       coeff_l4(12,95)= -4.2936426401e-01
       coeff_l4(12,96)= -4.4939571619e-01
       coeff_l4(12,97)= -3.0423104763e-01
       coeff_l4(12,98)=  2.7352400124e-02
       coeff_l4(12,99)= -1.4803434908e-01
       coeff_l4(12,100)=  9.6934390068e-01
       coeff_l4(12,101)= -8.1918996572e-01
       coeff_l4(12,102)= -1.0167604685e+00
       coeff_l4(12,103)=  7.1715795994e-01
       coeff_l4(12,104)=  6.1478352547e-01
       coeff_l4(12,105)= -3.5377883911e-01
       coeff_l4(13,1)=  7.3021697998e-01
       coeff_l4(13,2)=  7.9307562113e-01
       coeff_l4(13,3)=  4.1986957192e-01
       coeff_l4(13,4)=  2.3851788044e-01
       coeff_l4(13,5)=  6.9245302677e-01
       coeff_l4(13,6)= -4.1224323213e-02
       coeff_l4(13,7)=  6.0038846731e-01
       coeff_l4(13,8)= -5.7454818487e-01
       coeff_l4(13,9)= -1.9450370455e-03
       coeff_l4(13,10)= -3.8135882467e-02
       coeff_l4(13,11)= -1.6862393618e+00
       coeff_l4(13,12)= -1.1606316566e+00
       coeff_l4(13,13)= -8.7675559521e-01
       coeff_l4(13,14)= -3.1917217374e-01
       coeff_l4(13,15)= -7.0557728410e-02
       coeff_l4(13,16)=  1.8875414506e-02
       coeff_l4(13,17)=  7.0667684078e-02
       coeff_l4(13,18)=  4.0969884396e-01
       coeff_l4(13,19)= -2.8257808089e-01
       coeff_l4(13,20)= -1.5706282854e-01
       coeff_l4(13,21)= -3.5052895546e-01
       coeff_l4(13,22)=  1.0362547636e-01
       coeff_l4(13,23)= -2.3460975289e-01
       coeff_l4(13,24)=  4.1852056980e-01
       coeff_l4(13,25)= -4.5441351831e-03
       coeff_l4(13,26)=  2.5014877319e-01
       coeff_l4(13,27)=  1.4192910492e-01
       coeff_l4(13,28)=  1.8782992661e-01
       coeff_l4(13,29)=  1.7573784292e-01
       coeff_l4(13,30)= -2.4065072834e-01
       coeff_l4(13,31)= -3.3427119255e-02
       coeff_l4(13,32)= -3.0492573977e-01
       coeff_l4(13,33)=  9.2419246212e-03
       coeff_l4(13,34)= -2.8228378296e-01
       coeff_l4(13,35)= -2.0441116393e-01
       coeff_l4(13,36)= -6.7452900112e-02
       coeff_l4(13,37)=  2.8351154923e-01
       coeff_l4(13,38)= -1.3941259682e-01
       coeff_l4(13,39)=  1.4274759591e-01
       coeff_l4(13,40)= -1.3161517680e-01
       coeff_l4(13,41)=  4.2517650872e-02
       coeff_l4(13,42)=  3.1193307042e-01
       coeff_l4(13,43)= -1.5858781338e-01
       coeff_l4(13,44)= -4.0005333722e-03
       coeff_l4(13,45)= -2.5939232111e-01
       coeff_l4(13,46)=  6.4716726542e-02
       coeff_l4(13,47)=  1.2557269633e-01
       coeff_l4(13,48)= -5.1471972466e-01
       coeff_l4(13,49)=  1.9140413404e-01
       coeff_l4(13,50)= -2.8772577643e-01
       coeff_l4(13,51)=  8.2534318790e-03
       coeff_l4(13,52)= -1.1944950372e-01
       coeff_l4(13,53)= -3.1064900756e-01
       coeff_l4(13,54)=  3.7821170688e-01
       coeff_l4(13,55)= -4.9666556716e-01
       coeff_l4(13,56)=  4.4689595699e-01
       coeff_l4(13,57)=  1.8350684643e-01
       coeff_l4(13,58)=  1.3900007308e-01
       coeff_l4(13,59)= -3.0847859383e-01
       coeff_l4(13,60)= -2.4555392563e-01
       coeff_l4(13,61)= -3.7919658422e-01
       coeff_l4(13,62)=  8.0134786665e-02
       coeff_l4(13,63)= -9.9002562463e-02
       coeff_l4(13,64)=  3.6824488640e-01
       coeff_l4(13,65)= -6.7123591900e-02
       coeff_l4(13,66)= -2.8312173486e-01
       coeff_l4(13,67)=  1.1875879019e-01
       coeff_l4(13,68)= -1.5185363591e-01
       coeff_l4(13,69)= -1.7304211855e-01
       coeff_l4(13,70)=  2.3237919435e-02
       coeff_l4(13,71)=  1.8479439616e-01
       coeff_l4(13,72)=  1.2241315097e-01
       coeff_l4(13,73)=  3.1047359109e-01
       coeff_l4(13,74)=  5.5520880222e-01
       coeff_l4(13,75)=  5.8978326619e-02
       coeff_l4(13,76)= -4.8793409020e-02
       coeff_l4(13,77)=  9.9838405848e-02
       coeff_l4(13,78)= -6.6856211424e-01
       coeff_l4(13,79)=  4.4055619836e-01
       coeff_l4(13,80)= -7.1858459711e-01
       coeff_l4(13,81)= -1.7392034829e-01
       coeff_l4(13,82)= -1.7877310514e-01
       coeff_l4(13,83)=  3.1880620867e-02
       coeff_l4(13,84)=  1.2615451217e-01
       coeff_l4(13,85)=  4.4576647878e-01
       coeff_l4(13,86)= -7.1299493313e-01
       coeff_l4(13,87)=  8.6210256815e-01
       coeff_l4(13,88)= -5.1914495230e-01
       coeff_l4(13,89)=  6.7146319151e-01
       coeff_l4(13,90)= -1.9672894478e-01
       coeff_l4(13,91)=  7.5577861071e-01
       coeff_l4(13,92)= -4.5574718714e-01
       coeff_l4(13,93)= -6.7495030165e-01
       coeff_l4(13,94)=  3.0245473981e-01
       coeff_l4(13,95)= -6.5888023376e-01
       coeff_l4(13,96)= -4.1583663225e-01
       coeff_l4(13,97)=  2.2174938023e-01
       coeff_l4(13,98)=  3.9031222463e-01
       coeff_l4(13,99)= -1.8794305623e-02
       coeff_l4(13,100)=  3.2604101300e-01
       coeff_l4(13,101)= -5.3145962954e-01
       coeff_l4(13,102)= -9.7674101591e-01
       coeff_l4(13,103)=  1.3264544308e-01
       coeff_l4(13,104)=  1.2052338570e-01
       coeff_l4(13,105)=  5.2524968982e-02
       coeff_l4(14,1)= -5.0775617361e-01
       coeff_l4(14,2)= -4.5083469152e-01
       coeff_l4(14,3)= -5.4797852039e-01
       coeff_l4(14,4)= -2.3386751115e-01
       coeff_l4(14,5)=  5.9308249503e-02
       coeff_l4(14,6)= -1.8296754360e-01
       coeff_l4(14,7)= -2.2028747480e-04
       coeff_l4(14,8)= -1.6706016660e-01
       coeff_l4(14,9)=  1.2559568882e-01
       coeff_l4(14,10)= -4.7461611032e-01
       coeff_l4(14,11)=  6.6476023197e-01
       coeff_l4(14,12)=  3.9729708433e-01
       coeff_l4(14,13)=  5.5414277315e-01
       coeff_l4(14,14)=  5.0400584936e-01
       coeff_l4(14,15)= -2.1693935990e-01
       coeff_l4(14,16)= -1.5628549457e-01
       coeff_l4(14,17)= -2.4023123085e-01
       coeff_l4(14,18)= -3.8960136473e-02
       coeff_l4(14,19)=  2.0617660880e-01
       coeff_l4(14,20)=  1.7055492103e-01
       coeff_l4(14,21)=  2.1953193843e-01
       coeff_l4(14,22)= -6.6940598190e-02
       coeff_l4(14,23)= -1.0500587523e-01
       coeff_l4(14,24)=  1.4076693356e-01
       coeff_l4(14,25)=  1.7405116558e-01
       coeff_l4(14,26)=  5.2570782602e-02
       coeff_l4(14,27)= -2.6207232475e-01
       coeff_l4(14,28)=  1.9679889083e-01
       coeff_l4(14,29)=  3.0923461542e-02
       coeff_l4(14,30)= -2.0454591513e-01
       coeff_l4(14,31)=  7.3539121076e-03
       coeff_l4(14,32)=  3.1817936897e-01
       coeff_l4(14,33)=  1.5659193695e-01
       coeff_l4(14,34)= -3.5149645805e-01
       coeff_l4(14,35)= -2.0565097034e-01
       coeff_l4(14,36)=  3.4200605750e-01
       coeff_l4(14,37)=  1.0349328816e-01
       coeff_l4(14,38)=  2.0518453419e-01
       coeff_l4(14,39)= -2.0820036530e-01
       coeff_l4(14,40)=  1.6039107740e-01
       coeff_l4(14,41)= -3.1491285563e-01
       coeff_l4(14,42)=  1.2896104157e-01
       coeff_l4(14,43)=  2.9857876897e-01
       coeff_l4(14,44)=  3.6695659161e-01
       coeff_l4(14,45)= -3.2567507029e-01
       coeff_l4(14,46)= -2.0068982244e-01
       coeff_l4(14,47)= -2.3963500559e-01
       coeff_l4(14,48)=  1.9076389074e-01
       coeff_l4(14,49)= -1.7595744133e-01
       coeff_l4(14,50)=  1.3465268910e-01
       coeff_l4(14,51)= -1.4143948257e-01
       coeff_l4(14,52)=  3.4506380558e-01
       coeff_l4(14,53)=  1.5068159997e-01
       coeff_l4(14,54)= -6.8523921072e-02
       coeff_l4(14,55)=  3.4811714292e-01
       coeff_l4(14,56)= -3.1778663397e-01
       coeff_l4(14,57)=  1.6377915442e-01
       coeff_l4(14,58)=  7.6320946217e-02
       coeff_l4(14,59)=  1.1261084676e-01
       coeff_l4(14,60)= -1.9412627816e-01
       coeff_l4(14,61)=  9.6303679049e-02
       coeff_l4(14,62)=  3.0708110332e-01
       coeff_l4(14,63)=  1.1935662478e-01
       coeff_l4(14,64)=  1.0105491430e-01
       coeff_l4(14,65)=  1.6244995594e-01
       coeff_l4(14,66)= -1.1413494498e-01
       coeff_l4(14,67)= -3.8619855046e-01
       coeff_l4(14,68)= -2.6132484898e-02
       coeff_l4(14,69)=  1.5839113295e-01
       coeff_l4(14,70)=  2.0069189370e-01
       coeff_l4(14,71)=  2.0254671574e-01
       coeff_l4(14,72)= -2.2697379813e-02
       coeff_l4(14,73)= -5.9525626898e-01
       coeff_l4(14,74)= -1.8999375403e-01
       coeff_l4(14,75)= -1.0707432777e-01
       coeff_l4(14,76)= -1.2683652341e-02
       coeff_l4(14,77)= -6.5723657608e-02
       coeff_l4(14,78)= -1.0117673874e-01
       coeff_l4(14,79)= -4.1450080276e-01
       coeff_l4(14,80)= -4.7014933079e-03
       coeff_l4(14,81)=  1.5216429718e-02
       coeff_l4(14,82)= -4.6797549725e-01
       coeff_l4(14,83)= -4.0608841181e-01
       coeff_l4(14,84)=  1.0912917554e-01
       coeff_l4(14,85)=  7.2739559412e-01
       coeff_l4(14,86)= -4.1194349527e-01
       coeff_l4(14,87)=  1.6258910298e-01
       coeff_l4(14,88)= -5.2767772228e-02
       coeff_l4(14,89)=  4.6910041571e-01
       coeff_l4(14,90)= -5.8463435620e-02
       coeff_l4(14,91)=  6.4433626831e-02
       coeff_l4(14,92)=  5.7307258248e-02
       coeff_l4(14,93)=  1.7008280754e-01
       coeff_l4(14,94)=  3.4406185150e-01
       coeff_l4(14,95)= -1.0788240284e-01
       coeff_l4(14,96)=  5.8202099055e-02
       coeff_l4(14,97)=  5.7383722067e-01
       coeff_l4(14,98)= -5.3859822452e-02
       coeff_l4(14,99)= -6.0184794664e-01
       coeff_l4(14,100)= -1.4026264846e-01
       coeff_l4(14,101)=  6.0897654295e-01
       coeff_l4(14,102)=  2.0083300769e-02
       coeff_l4(14,103)= -2.3937562108e-01
       coeff_l4(14,104)=  1.7147405446e-01
       coeff_l4(14,105)= -3.3710598946e-01
       coeff_l4(15,1)= -7.5366300344e-01
       coeff_l4(15,2)= -5.2682554722e-01
       coeff_l4(15,3)= -3.4517326951e-01
       coeff_l4(15,4)= -3.7283489108e-01
       coeff_l4(15,5)= -5.0515878201e-01
       coeff_l4(15,6)= -6.4174091816e-01
       coeff_l4(15,7)= -6.0783481598e-01
       coeff_l4(15,8)= -4.2498713732e-01
       coeff_l4(15,9)=  1.3709682226e-01
       coeff_l4(15,10)= -2.5698868558e-03
       coeff_l4(15,11)=  2.3913906515e-01
       coeff_l4(15,12)=  4.1626811028e-01
       coeff_l4(15,13)=  3.5298132896e-01
       coeff_l4(15,14)=  9.2721767724e-02
       coeff_l4(15,15)=  1.0919821262e-01
       coeff_l4(15,16)=  3.4289583564e-02
       coeff_l4(15,17)= -2.2367665172e-01
       coeff_l4(15,18)= -8.3656243980e-02
       coeff_l4(15,19)=  2.8932297602e-02
       coeff_l4(15,20)=  1.4769772068e-02
       coeff_l4(15,21)= -9.9319033325e-02
       coeff_l4(15,22)=  2.0378261805e-01
       coeff_l4(15,23)= -2.1239854395e-02
       coeff_l4(15,24)=  1.1890510470e-01
       coeff_l4(15,25)=  1.7288485542e-02
       coeff_l4(15,26)=  2.7692189440e-02
       coeff_l4(15,27)=  1.0037180781e-01
       coeff_l4(15,28)=  4.4354788959e-02
       coeff_l4(15,29)= -4.4001746923e-02
       coeff_l4(15,30)=  1.7552655935e-01
       coeff_l4(15,31)= -1.1330851167e-01
       coeff_l4(15,32)=  5.1619023085e-02
       coeff_l4(15,33)=  1.3991314173e-01
       coeff_l4(15,34)= -8.6503528059e-02
       coeff_l4(15,35)=  6.6707357764e-02
       coeff_l4(15,36)=  1.8409900367e-01
       coeff_l4(15,37)=  9.2524416745e-02
       coeff_l4(15,38)=  8.0877617002e-02
       coeff_l4(15,39)= -2.9634556174e-01
       coeff_l4(15,40)=  1.2358632684e-01
       coeff_l4(15,41)= -8.9331127703e-02
       coeff_l4(15,42)=  1.0787413269e-01
       coeff_l4(15,43)=  1.2356714346e-02
       coeff_l4(15,44)= -1.0798612982e-01
       coeff_l4(15,45)= -1.1870498955e-01
       coeff_l4(15,46)= -6.8307809532e-02
       coeff_l4(15,47)= -1.7922632396e-01
       coeff_l4(15,48)= -4.1300203651e-02
       coeff_l4(15,49)=  5.7108823210e-02
       coeff_l4(15,50)=  8.5357751232e-05
       coeff_l4(15,51)=  1.8969303370e-01
       coeff_l4(15,52)= -9.3151941895e-02
       coeff_l4(15,53)=  6.2369462103e-03
       coeff_l4(15,54)=  8.9090883732e-02
       coeff_l4(15,55)= -1.2040765584e-01
       coeff_l4(15,56)= -7.6381832361e-02
       coeff_l4(15,57)= -1.2211481482e-01
       coeff_l4(15,58)= -1.0167859495e-01
       coeff_l4(15,59)=  9.2058025301e-02
       coeff_l4(15,60)= -7.4549920857e-02
       coeff_l4(15,61)= -5.0328452140e-02
       coeff_l4(15,62)=  1.3022118807e-01
       coeff_l4(15,63)= -1.7391262949e-01
       coeff_l4(15,64)=  9.0633034706e-02
       coeff_l4(15,65)=  1.4691576362e-01
       coeff_l4(15,66)= -6.3580870628e-02
       coeff_l4(15,67)= -8.8073164225e-03
       coeff_l4(15,68)= -1.1428222060e-01
       coeff_l4(15,69)=  1.1282209307e-01
       coeff_l4(15,70)=  1.5070798993e-01
       coeff_l4(15,71)= -1.6386929154e-01
       coeff_l4(15,72)=  3.4171320498e-02
       coeff_l4(15,73)=  2.9067371041e-02
       coeff_l4(15,74)=  1.8102325499e-02
       coeff_l4(15,75)= -6.7798383534e-02
       coeff_l4(15,76)=  2.0189942792e-02
       coeff_l4(15,77)= -1.1770816147e-01
       coeff_l4(15,78)=  5.1750350744e-02
       coeff_l4(15,79)=  5.4447267205e-02
       coeff_l4(15,80)= -1.1356626637e-02
       coeff_l4(15,81)=  9.0008661151e-02
       coeff_l4(15,82)= -6.4503230155e-02
       coeff_l4(15,83)=  9.4832390547e-02
       coeff_l4(15,84)= -4.9194592983e-02
       coeff_l4(15,85)=  9.3752883375e-02
       coeff_l4(15,86)= -1.6930571198e-01
       coeff_l4(15,87)= -1.8416201696e-02
       coeff_l4(15,88)=  9.0563848615e-02
       coeff_l4(15,89)=  9.0457603335e-02
       coeff_l4(15,90)= -1.4472934417e-02
       coeff_l4(15,91)=  1.0238441080e-01
       coeff_l4(15,92)=  5.9319160879e-02
       coeff_l4(15,93)=  1.8538296223e-01
       coeff_l4(15,94)=  3.9715152979e-01
       coeff_l4(15,95)=  7.6449230313e-02
       coeff_l4(15,96)=  3.8198558614e-03
       coeff_l4(15,97)=  2.0942211151e-01
       coeff_l4(15,98)=  8.6280025542e-02
       coeff_l4(15,99)=  7.4723765254e-02
       coeff_l4(15,100)=  1.7382059991e-01
       coeff_l4(15,101)=  1.5926592052e-01
       coeff_l4(15,102)= -1.1802567542e-01
       coeff_l4(15,103)=  7.8557945788e-02
       coeff_l4(15,104)= -2.2241426632e-02
       coeff_l4(15,105)= -1.0003007948e-02
       coeff_l4(16,1)= -7.9119198024e-02
       coeff_l4(16,2)=  1.9669228792e-01
       coeff_l4(16,3)= -5.2588562539e-05
       coeff_l4(16,4)=  6.2198340893e-02
       coeff_l4(16,5)= -1.0614072904e-02
       coeff_l4(16,6)= -4.4335429557e-03
       coeff_l4(16,7)=  9.0472213924e-02
       coeff_l4(16,8)=  1.0461331904e-01
       coeff_l4(16,9)=  2.5827336311e-01
       coeff_l4(16,10)=  5.7421997190e-02
       coeff_l4(16,11)=  2.4614816904e-01
       coeff_l4(16,12)=  1.6960205138e-01
       coeff_l4(16,13)=  1.8961678445e-01
       coeff_l4(16,14)=  4.2908272147e-01
       coeff_l4(16,15)=  1.7088885605e-01
       coeff_l4(16,16)= -1.2321212329e-02
       coeff_l4(16,17)=  4.3237034231e-02
       coeff_l4(16,18)= -1.4207507670e-01
       coeff_l4(16,19)= -2.8484087437e-02
       coeff_l4(16,20)= -2.4326907471e-02
       coeff_l4(16,21)= -1.9025844336e-01
       coeff_l4(16,22)=  8.9541941881e-02
       coeff_l4(16,23)=  1.4345601201e-01
       coeff_l4(16,24)= -1.4980756678e-03
       coeff_l4(16,25)=  1.4288413338e-02
       coeff_l4(16,26)=  1.3000148535e-01
       coeff_l4(16,27)=  1.5526649356e-01
       coeff_l4(16,28)=  1.3824085705e-02
       coeff_l4(16,29)= -5.3990721703e-02
       coeff_l4(16,30)=  6.0579907149e-02
       coeff_l4(16,31)= -2.8003994375e-02
       coeff_l4(16,32)= -7.7740147710e-02
       coeff_l4(16,33)=  1.5545784496e-02
       coeff_l4(16,34)=  5.3216669708e-02
       coeff_l4(16,35)=  1.5824481845e-01
       coeff_l4(16,36)= -1.6713815928e-01
       coeff_l4(16,37)= -1.0287558287e-01
       coeff_l4(16,38)=  2.2352138162e-01
       coeff_l4(16,39)= -4.9375854433e-02
       coeff_l4(16,40)=  6.8168573081e-02
       coeff_l4(16,41)=  5.5258832872e-02
       coeff_l4(16,42)=  1.2808155268e-02
       coeff_l4(16,43)=  1.3960713148e-01
       coeff_l4(16,44)=  7.1959674358e-02
       coeff_l4(16,45)= -1.4008156955e-01
       coeff_l4(16,46)=  1.0721150041e-01
       coeff_l4(16,47)= -1.3103576005e-01
       coeff_l4(16,48)=  6.3486337662e-02
       coeff_l4(16,49)= -8.2791700959e-02
       coeff_l4(16,50)= -7.0314541459e-02
       coeff_l4(16,51)=  1.4598645270e-01
       coeff_l4(16,52)=  6.3523158431e-02
       coeff_l4(16,53)=  1.6502267122e-01
       coeff_l4(16,54)=  1.0051093996e-01
       coeff_l4(16,55)= -1.1009853333e-01
       coeff_l4(16,56)=  1.1933325976e-01
       coeff_l4(16,57)= -1.3138963282e-01
       coeff_l4(16,58)=  4.7548733652e-02
       coeff_l4(16,59)=  5.4326400161e-02
       coeff_l4(16,60)=  5.2680537105e-02
       coeff_l4(16,61)= -2.1518532932e-01
       coeff_l4(16,62)= -7.6960086823e-02
       coeff_l4(16,63)=  5.4247882217e-02
       coeff_l4(16,64)=  1.5812461078e-01
       coeff_l4(16,65)=  1.4145977795e-01
       coeff_l4(16,66)=  4.4161081314e-02
       coeff_l4(16,67)= -7.6023280621e-02
       coeff_l4(16,68)=  4.1063517332e-02
       coeff_l4(16,69)=  1.2261933833e-01
       coeff_l4(16,70)=  1.3376198709e-01
       coeff_l4(16,71)= -1.0868722200e-01
       coeff_l4(16,72)=  6.0982655734e-02
       coeff_l4(16,73)= -1.3001385331e-01
       coeff_l4(16,74)=  6.5020762384e-02
       coeff_l4(16,75)= -1.2658524513e-01
       coeff_l4(16,76)=  1.1708410084e-01
       coeff_l4(16,77)= -8.9117757976e-02
       coeff_l4(16,78)= -4.3665926903e-02
       coeff_l4(16,79)= -1.2408470362e-01
       coeff_l4(16,80)= -4.4235251844e-02
       coeff_l4(16,81)= -3.9748605341e-02
       coeff_l4(16,82)= -1.1187317222e-01
       coeff_l4(16,83)=  8.9797109365e-02
       coeff_l4(16,84)= -1.3350209594e-01
       coeff_l4(16,85)=  7.3329545557e-02
       coeff_l4(16,86)=  1.1335316300e-01
       coeff_l4(16,87)= -1.5622545779e-01
       coeff_l4(16,88)=  1.3108782470e-01
       coeff_l4(16,89)=  9.4111919403e-02
       coeff_l4(16,90)=  1.0001607239e-01
       coeff_l4(16,91)=  4.7845058143e-02
       coeff_l4(16,92)= -1.4651629329e-01
       coeff_l4(16,93)=  3.6229055375e-02
       coeff_l4(16,94)= -2.3495967034e-03
       coeff_l4(16,95)= -9.1080948710e-02
       coeff_l4(16,96)=  3.8188796490e-02
       coeff_l4(16,97)=  4.8214867711e-02
       coeff_l4(16,98)=  2.0653892308e-02
       coeff_l4(16,99)=  1.3451640308e-01
       coeff_l4(16,100)=  1.5413059294e-01
       coeff_l4(16,101)=  3.1110480428e-02
       coeff_l4(16,102)= -2.4175262079e-02
       coeff_l4(16,103)=  1.6811478138e-01
       coeff_l4(16,104)= -1.1551669985e-01
       coeff_l4(16,105)= -1.1490237713e-01
       coeff_l4(17,1)=  3.7023660541e-01
       coeff_l4(17,2)= -3.6849913001e-01
       coeff_l4(17,3)= -1.6422300041e-01
       coeff_l4(17,4)= -5.6494012475e-02
       coeff_l4(17,5)= -8.7504111230e-02
       coeff_l4(17,6)=  6.8016231060e-02
       coeff_l4(17,7)=  4.7761368752e-01
       coeff_l4(17,8)=  2.0706999302e-01
       coeff_l4(17,9)=  2.4375848472e-01
       coeff_l4(17,10)= -1.6081956029e-01
       coeff_l4(17,11)=  9.3430882692e-01
       coeff_l4(17,12)=  3.1421169639e-01
       coeff_l4(17,13)=  1.9118721783e-01
       coeff_l4(17,14)=  7.8197622299e-01
       coeff_l4(17,15)= -3.4705856442e-01
       coeff_l4(17,16)= -2.7682328224e-01
       coeff_l4(17,17)= -8.0323457718e-02
       coeff_l4(17,18)= -2.9788288474e-01
       coeff_l4(17,19)= -1.4114698768e-01
       coeff_l4(17,20)= -2.6457285881e-01
       coeff_l4(17,21)=  3.4424790740e-01
       coeff_l4(17,22)=  1.3567817211e-01
       coeff_l4(17,23)= -8.3922415972e-02
       coeff_l4(17,24)=  6.0795313120e-01
       coeff_l4(17,25)= -2.8209632635e-01
       coeff_l4(17,26)=  2.6495036483e-01
       coeff_l4(17,27)=  2.0957438275e-02
       coeff_l4(17,28)=  1.3335740566e-01
       coeff_l4(17,29)= -1.6028329730e-01
       coeff_l4(17,30)= -2.0006868243e-01
       coeff_l4(17,31)= -4.6377975494e-02
       coeff_l4(17,32)=  4.0945041180e-01
       coeff_l4(17,33)= -5.6069888175e-02
       coeff_l4(17,34)= -3.4550823271e-02
       coeff_l4(17,35)=  2.7885887027e-01
       coeff_l4(17,36)=  6.6417992115e-02
       coeff_l4(17,37)=  3.9642345905e-01
       coeff_l4(17,38)=  1.7179162800e-01
       coeff_l4(17,39)= -2.1855480969e-01
       coeff_l4(17,40)=  3.1729307771e-01
       coeff_l4(17,41)= -7.9562269151e-02
       coeff_l4(17,42)=  7.7085480094e-02
       coeff_l4(17,43)=  4.9717423320e-01
       coeff_l4(17,44)= -2.8785759211e-01
       coeff_l4(17,45)= -1.4626730978e-01
       coeff_l4(17,46)= -1.5824137628e-01
       coeff_l4(17,47)= -1.1240892112e-02
       coeff_l4(17,48)= -1.9199413061e-01
       coeff_l4(17,49)=  1.2351468904e-03
       coeff_l4(17,50)=  2.3816412687e-01
       coeff_l4(17,51)=  1.2355689704e-01
       coeff_l4(17,52)=  7.5530633330e-02
       coeff_l4(17,53)= -3.4547641873e-01
       coeff_l4(17,54)= -5.9561636299e-03
       coeff_l4(17,55)=  5.7606393099e-01
       coeff_l4(17,56)= -5.2223813534e-01
       coeff_l4(17,57)=  3.3476510644e-01
       coeff_l4(17,58)= -9.6917264163e-02
       coeff_l4(17,59)= -3.3564579487e-01
       coeff_l4(17,60)=  2.6869890094e-01
       coeff_l4(17,61)=  6.8494987488e-01
       coeff_l4(17,62)= -4.5154690742e-03
       coeff_l4(17,63)= -2.2912509739e-01
       coeff_l4(17,64)=  2.6190117002e-01
       coeff_l4(17,65)= -2.9855856672e-02
       coeff_l4(17,66)= -1.5344847739e-01
       coeff_l4(17,67)= -2.8505140543e-01
       coeff_l4(17,68)=  9.3484334648e-02
       coeff_l4(17,69)=  4.7715109587e-01
       coeff_l4(17,70)= -1.4720487595e-01
       coeff_l4(17,71)=  1.6101384163e-01
       coeff_l4(17,72)=  1.3316820376e-02
       coeff_l4(17,73)= -1.4879286289e-01
       coeff_l4(17,74)= -5.4029834270e-01
       coeff_l4(17,75)= -3.8467344642e-01
       coeff_l4(17,76)= -7.8950062394e-02
       coeff_l4(17,77)= -3.5677903891e-01
       coeff_l4(17,78)=  1.8706634641e-01
       coeff_l4(17,79)= -1.4563512802e-01
       coeff_l4(17,80)= -3.8971635699e-01
       coeff_l4(17,81)= -7.9678021371e-02
       coeff_l4(17,82)= -1.0828661174e-01
       coeff_l4(17,83)=  6.6735491157e-02
       coeff_l4(17,84)=  4.6150963753e-02
       coeff_l4(17,85)=  3.3089172840e-01
       coeff_l4(17,86)=  4.2338715866e-03
       coeff_l4(17,87)=  1.1354000866e-01
       coeff_l4(17,88)=  8.6587378755e-03
       coeff_l4(17,89)=  1.6562916338e-01
       coeff_l4(17,90)= -2.2696220875e-01
       coeff_l4(17,91)=  1.8398320675e-01
       coeff_l4(17,92)= -2.7373018861e-01
       coeff_l4(17,93)= -1.5204531373e-03
       coeff_l4(17,94)= -4.5310858637e-02
       coeff_l4(17,95)= -3.7944462895e-01
       coeff_l4(17,96)=  4.0172806382e-01
       coeff_l4(17,97)=  3.1747213006e-01
       coeff_l4(17,98)= -1.1164814234e-01
       coeff_l4(17,99)= -2.2588127851e-01
       coeff_l4(17,100)= -1.7811614275e-01
       coeff_l4(17,101)= -3.9089098573e-01
       coeff_l4(17,102)= -4.5149266720e-02
       coeff_l4(17,103)= -3.8269644976e-01
       coeff_l4(17,104)=  2.0874187350e-01
       coeff_l4(17,105)= -4.0004420280e-01
       coeff_l4(18,1)=  1.8817920685e+00
       coeff_l4(18,2)=  1.9235852957e+00
       coeff_l4(18,3)=  1.8983775377e+00
       coeff_l4(18,4)=  1.6847877502e+00
       coeff_l4(18,5)=  1.7505921125e+00
       coeff_l4(18,6)=  1.5690970421e+00
       coeff_l4(18,7)=  1.8707915545e+00
       coeff_l4(18,8)=  2.0942907333e+00
       coeff_l4(18,9)=  1.2150250673e+00
       coeff_l4(18,10)=  1.2644402981e+00
       coeff_l4(18,11)= -1.7037831247e-01
       coeff_l4(18,12)=  4.2262729257e-02
       coeff_l4(18,13)=  3.4173402190e-01
       coeff_l4(18,14)=  1.2828441858e+00
       coeff_l4(18,15)= -2.6014238596e-01
       coeff_l4(18,16)=  3.5832333565e-01
       coeff_l4(18,17)= -5.3632818162e-02
       coeff_l4(18,18)=  1.4463748038e-01
       coeff_l4(18,19)=  1.6452160478e-01
       coeff_l4(18,20)=  7.9820267856e-02
       coeff_l4(18,21)=  1.1967343092e-01
       coeff_l4(18,22)= -5.7348493487e-02
       coeff_l4(18,23)= -2.2333176434e-01
       coeff_l4(18,24)=  2.5376129150e-01
       coeff_l4(18,25)= -1.1310175806e-01
       coeff_l4(18,26)= -1.1275689304e-01
       coeff_l4(18,27)= -6.8441450596e-02
       coeff_l4(18,28)=  1.5459297597e-01
       coeff_l4(18,29)= -2.5830802042e-03
       coeff_l4(18,30)= -3.7045678496e-01
       coeff_l4(18,31)= -1.7218364403e-02
       coeff_l4(18,32)=  8.8603600860e-02
       coeff_l4(18,33)=  2.1751061082e-01
       coeff_l4(18,34)= -2.1048089862e-01
       coeff_l4(18,35)= -2.5725388527e-01
       coeff_l4(18,36)=  7.1020382456e-03
       coeff_l4(18,37)= -1.0456325859e-01
       coeff_l4(18,38)=  3.2223987579e-01
       coeff_l4(18,39)=  6.3399538398e-02
       coeff_l4(18,40)=  4.4717088342e-02
       coeff_l4(18,41)= -5.0519579649e-01
       coeff_l4(18,42)= -1.7549508810e-01
       coeff_l4(18,43)= -3.2715335488e-01
       coeff_l4(18,44)=  3.0569982529e-01
       coeff_l4(18,45)= -9.6535846591e-02
       coeff_l4(18,46)= -3.8229767233e-02
       coeff_l4(18,47)=  2.3781475425e-01
       coeff_l4(18,48)=  1.6936914623e-01
       coeff_l4(18,49)= -9.9373176694e-02
       coeff_l4(18,50)=  5.4479882121e-02
       coeff_l4(18,51)= -8.6693204939e-02
       coeff_l4(18,52)= -1.2590517104e-01
       coeff_l4(18,53)= -4.0671238303e-01
       coeff_l4(18,54)= -2.4884603918e-01
       coeff_l4(18,55)= -4.5697886497e-02
       coeff_l4(18,56)= -3.4593382478e-01
       coeff_l4(18,57)=  2.4932810664e-01
       coeff_l4(18,58)=  7.2369582951e-02
       coeff_l4(18,59)=  1.0030503571e-01
       coeff_l4(18,60)=  1.1742368340e-01
       coeff_l4(18,61)= -7.9612821341e-02
       coeff_l4(18,62)=  9.4095669687e-02
       coeff_l4(18,63)=  4.4244427234e-02
       coeff_l4(18,64)=  2.9205637053e-02
       coeff_l4(18,65)=  6.8799816072e-02
       coeff_l4(18,66)=  2.8899265453e-02
       coeff_l4(18,67)=  6.8706989288e-02
       coeff_l4(18,68)= -1.0353176296e-01
       coeff_l4(18,69)= -1.8373680115e-01
       coeff_l4(18,70)=  8.6022347212e-02
       coeff_l4(18,71)=  7.7675998211e-02
       coeff_l4(18,72)= -2.3898748681e-02
       coeff_l4(18,73)= -4.2071599513e-02
       coeff_l4(18,74)=  2.7930870652e-01
       coeff_l4(18,75)= -1.8213151395e-01
       coeff_l4(18,76)=  2.0951667428e-01
       coeff_l4(18,77)= -1.8125630915e-01
       coeff_l4(18,78)= -1.9915136695e-01
       coeff_l4(18,79)= -3.2822739333e-02
       coeff_l4(18,80)= -1.3123412430e-01
       coeff_l4(18,81)= -1.3118633628e-01
       coeff_l4(18,82)= -9.7469247878e-02
       coeff_l4(18,83)= -7.4707865715e-02
       coeff_l4(18,84)= -1.2562477589e-01
       coeff_l4(18,85)=  7.1195438504e-03
       coeff_l4(18,86)=  2.3233409226e-01
       coeff_l4(18,87)=  1.8811942637e-01
       coeff_l4(18,88)=  2.0254065096e-01
       coeff_l4(18,89)=  1.6294418275e-01
       coeff_l4(18,90)= -1.4915725589e-01
       coeff_l4(18,91)=  2.8733617067e-01
       coeff_l4(18,92)=  1.8483620137e-02
       coeff_l4(18,93)= -2.5662097335e-01
       coeff_l4(18,94)= -2.1728843451e-01
       coeff_l4(18,95)=  1.5788832679e-03
       coeff_l4(18,96)= -5.4247593880e-01
       coeff_l4(18,97)= -4.0387541056e-01
       coeff_l4(18,98)=  2.9663768411e-01
       coeff_l4(18,99)= -1.2757670879e-01
       coeff_l4(18,100)=  3.6535853148e-01
       coeff_l4(18,101)= -5.7512098551e-01
       coeff_l4(18,102)= -5.4703179747e-02
       coeff_l4(18,103)=  4.5803183317e-01
       coeff_l4(18,104)=  1.2220816314e-01
       coeff_l4(18,105)=  8.0019548535e-02
       coeff_l4(19,1)= -1.0519422293e+00
       coeff_l4(19,2)= -7.2615742683e-01
       coeff_l4(19,3)= -4.7720247507e-01
       coeff_l4(19,4)= -3.0470624566e-01
       coeff_l4(19,5)= -5.6672710180e-01
       coeff_l4(19,6)= -3.5604748130e-01
       coeff_l4(19,7)= -6.7968827486e-01
       coeff_l4(19,8)= -4.8056101799e-01
       coeff_l4(19,9)= -3.1772744656e-01
       coeff_l4(19,10)= -1.9555059075e-01
       coeff_l4(19,11)= -5.4841279984e-01
       coeff_l4(19,12)= -4.7405433655e-01
       coeff_l4(19,13)= -4.6500751376e-01
       coeff_l4(19,14)=  2.6881185174e-01
       coeff_l4(19,15)=  2.1488770843e-01
       coeff_l4(19,16)=  1.6887490451e-01
       coeff_l4(19,17)=  1.2650045753e-01
       coeff_l4(19,18)=  5.1110289991e-02
       coeff_l4(19,19)= -3.0330151320e-01
       coeff_l4(19,20)= -2.8247715905e-02
       coeff_l4(19,21)= -2.7072820067e-01
       coeff_l4(19,22)= -1.2420760840e-01
       coeff_l4(19,23)=  3.6799848080e-02
       coeff_l4(19,24)= -1.0515377671e-01
       coeff_l4(19,25)=  2.0401401818e-01
       coeff_l4(19,26)= -1.5839542449e-01
       coeff_l4(19,27)=  4.5952439308e-02
       coeff_l4(19,28)= -2.3325291276e-01
       coeff_l4(19,29)=  1.6339153051e-01
       coeff_l4(19,30)=  6.3364163041e-02
       coeff_l4(19,31)=  2.7155554295e-01
       coeff_l4(19,32)= -5.0524819642e-02
       coeff_l4(19,33)=  8.2609392703e-02
       coeff_l4(19,34)= -1.5202142298e-01
       coeff_l4(19,35)=  9.4886481762e-02
       coeff_l4(19,36)= -1.6381514072e-01
       coeff_l4(19,37)= -5.9918113053e-02
       coeff_l4(19,38)= -2.1930366755e-02
       coeff_l4(19,39)= -1.0860209912e-01
       coeff_l4(19,40)=  2.6650187373e-01
       coeff_l4(19,41)= -4.6347077936e-02
       coeff_l4(19,42)=  1.4553911984e-01
       coeff_l4(19,43)= -5.2178103477e-02
       coeff_l4(19,44)= -2.1163802594e-02
       coeff_l4(19,45)=  2.2622292861e-02
       coeff_l4(19,46)=  4.6967025846e-02
       coeff_l4(19,47)=  8.8486745954e-02
       coeff_l4(19,48)= -9.5716200769e-02
       coeff_l4(19,49)=  5.3421866149e-02
       coeff_l4(19,50)= -5.3677488118e-02
       coeff_l4(19,51)=  1.1718815565e-01
       coeff_l4(19,52)=  1.3067180291e-02
       coeff_l4(19,53)= -1.9938170910e-01
       coeff_l4(19,54)= -3.7847332656e-02
       coeff_l4(19,55)= -2.6448830962e-02
       coeff_l4(19,56)= -1.1622313410e-01
       coeff_l4(19,57)=  2.4754817784e-01
       coeff_l4(19,58)=  2.0500950515e-02
       coeff_l4(19,59)= -1.0097157955e-01
       coeff_l4(19,60)=  1.9319498539e-01
       coeff_l4(19,61)= -7.7574394643e-02
       coeff_l4(19,62)=  1.1246962100e-01
       coeff_l4(19,63)=  1.6709873080e-01
       coeff_l4(19,64)=  2.1524813026e-02
       coeff_l4(19,65)=  1.4807635546e-01
       coeff_l4(19,66)=  2.3654223979e-01
       coeff_l4(19,67)= -6.0068987310e-02
       coeff_l4(19,68)= -1.0224669427e-01
       coeff_l4(19,69)=  8.9955076575e-02
       coeff_l4(19,70)=  4.1518129408e-02
       coeff_l4(19,71)=  2.8759378195e-01
       coeff_l4(19,72)=  2.0881053805e-01
       coeff_l4(19,73)=  3.9043586701e-02
       coeff_l4(19,74)= -4.5086126775e-02
       coeff_l4(19,75)=  3.9347371785e-04
       coeff_l4(19,76)=  2.3867677897e-02
       coeff_l4(19,77)=  7.4839696288e-02
       coeff_l4(19,78)=  1.6342414310e-03
       coeff_l4(19,79)=  3.2927889377e-02
       coeff_l4(19,80)= -1.6291207075e-01
       coeff_l4(19,81)= -2.2752726078e-01
       coeff_l4(19,82)=  7.1856498718e-02
       coeff_l4(19,83)= -8.0468475819e-02
       coeff_l4(19,84)= -2.2025762126e-02
       coeff_l4(19,85)=  2.2240137681e-02
       coeff_l4(19,86)=  2.9007482529e-01
       coeff_l4(19,87)=  5.5942587554e-02
       coeff_l4(19,88)=  1.8607628345e-01
       coeff_l4(19,89)=  6.1242101947e-04
       coeff_l4(19,90)=  1.8676068634e-02
       coeff_l4(19,91)=  1.9588621333e-02
       coeff_l4(19,92)=  2.1998788416e-01
       coeff_l4(19,93)= -2.6268243790e-01
       coeff_l4(19,94)=  1.9132509828e-01
       coeff_l4(19,95)=  3.1782829762e-01
       coeff_l4(19,96)= -1.2252860703e-02
       coeff_l4(19,97)= -4.3153065443e-01
       coeff_l4(19,98)=  1.1336100101e-01
       coeff_l4(19,99)= -4.4565584511e-02
       coeff_l4(19,100)=  1.4886395633e-01
       coeff_l4(19,101)= -1.7198042944e-02
       coeff_l4(19,102)=  9.8488137126e-02
       coeff_l4(19,103)= -6.1979264021e-02
       coeff_l4(19,104)= -2.4821745232e-02
       coeff_l4(19,105)=  9.3998521566e-02
       coeff_l4(20,1)= -3.9632033557e-02
       coeff_l4(20,2)=  3.3507245779e-01
       coeff_l4(20,3)=  1.7911639810e-01
       coeff_l4(20,4)= -1.5229966491e-02
       coeff_l4(20,5)= -1.1187165976e-01
       coeff_l4(20,6)= -3.9376434684e-01
       coeff_l4(20,7)= -6.0653799772e-01
       coeff_l4(20,8)= -4.6859860420e-01
       coeff_l4(20,9)= -4.2104795575e-01
       coeff_l4(20,10)= -4.5634084940e-01
       coeff_l4(20,11)= -4.2834792286e-02
       coeff_l4(20,12)=  4.0133193135e-01
       coeff_l4(20,13)=  2.3442091048e-01
       coeff_l4(20,14)= -3.8255685568e-01
       coeff_l4(20,15)= -3.5399023443e-02
       coeff_l4(20,16)= -2.2521547973e-01
       coeff_l4(20,17)= -3.9105485193e-03
       coeff_l4(20,18)= -2.2200353444e-01
       coeff_l4(20,19)= -2.4902088940e-01
       coeff_l4(20,20)=  2.9185757041e-01
       coeff_l4(20,21)= -1.5910974145e-01
       coeff_l4(20,22)= -2.2588600218e-01
       coeff_l4(20,23)= -1.9468899071e-01
       coeff_l4(20,24)=  2.6775112748e-01
       coeff_l4(20,25)= -9.5833189785e-02
       coeff_l4(20,26)=  2.0180638134e-01
       coeff_l4(20,27)= -5.2801899612e-02
       coeff_l4(20,28)= -8.8247433305e-02
       coeff_l4(20,29)= -6.5958462656e-02
       coeff_l4(20,30)=  2.9815152287e-01
       coeff_l4(20,31)=  4.7314378619e-01
       coeff_l4(20,32)=  4.5145735145e-01
       coeff_l4(20,33)= -9.2564009130e-02
       coeff_l4(20,34)=  2.4500438571e-01
       coeff_l4(20,35)=  1.3881848752e-01
       coeff_l4(20,36)= -2.5168099999e-01
       coeff_l4(20,37)= -1.5096729994e-01
       coeff_l4(20,38)=  7.8059367836e-02
       coeff_l4(20,39)= -4.8947324976e-03
       coeff_l4(20,40)= -4.2256131768e-01
       coeff_l4(20,41)=  4.0816938877e-01
       coeff_l4(20,42)=  3.0484381318e-01
       coeff_l4(20,43)=  5.3327664733e-02
       coeff_l4(20,44)=  3.0922001600e-01
       coeff_l4(20,45)= -4.2215931416e-01
       coeff_l4(20,46)= -4.6950194240e-01
       coeff_l4(20,47)= -6.8210297823e-01
       coeff_l4(20,48)=  3.7777081132e-02
       coeff_l4(20,49)= -4.5763538219e-03
       coeff_l4(20,50)= -3.5195961595e-01
       coeff_l4(20,51)=  4.6507340670e-01
       coeff_l4(20,52)=  1.8372805417e-01
       coeff_l4(20,53)=  4.5271691680e-01
       coeff_l4(20,54)=  4.4649189711e-01
       coeff_l4(20,55)= -3.7709465623e-01
       coeff_l4(20,56)=  1.4624299109e-01
       coeff_l4(20,57)=  3.9425253868e-02
       coeff_l4(20,58)= -5.7516872883e-01
       coeff_l4(20,59)= -2.6135703921e-01
       coeff_l4(20,60)=  2.8145294636e-02
       coeff_l4(20,61)= -1.4867198467e-01
       coeff_l4(20,62)=  1.1303981394e-01
       coeff_l4(20,63)=  1.7894183099e-01
       coeff_l4(20,64)=  3.5587826371e-01
       coeff_l4(20,65)= -3.7305634469e-02
       coeff_l4(20,66)= -2.8867352009e-01
       coeff_l4(20,67)= -3.8116794080e-02
       coeff_l4(20,68)=  3.6372244358e-01
       coeff_l4(20,69)= -4.9207650125e-02
       coeff_l4(20,70)=  3.5523065925e-01
       coeff_l4(20,71)=  2.6219528913e-01
       coeff_l4(20,72)= -2.3295749724e-01
       coeff_l4(20,73)= -4.0635955334e-01
       coeff_l4(20,74)= -4.7233855724e-01
       coeff_l4(20,75)= -2.0272164047e-01
       coeff_l4(20,76)=  1.2882584333e-01
       coeff_l4(20,77)=  2.9322922230e-02
       coeff_l4(20,78)= -4.9137275666e-02
       coeff_l4(20,79)= -1.0327742994e-01
       coeff_l4(20,80)= -3.8747468591e-01
       coeff_l4(20,81)= -2.5593653321e-01
       coeff_l4(20,82)= -1.2055156380e-01
       coeff_l4(20,83)=  7.9951556399e-03
       coeff_l4(20,84)=  1.3207702897e-02
       coeff_l4(20,85)=  7.8989692032e-02
       coeff_l4(20,86)= -1.4148886502e-01
       coeff_l4(20,87)= -6.2358919531e-02
       coeff_l4(20,88)= -1.1176544242e-02
       coeff_l4(20,89)= -6.5853215754e-02
       coeff_l4(20,90)= -4.9039283395e-01
       coeff_l4(20,91)= -2.5346264243e-02
       coeff_l4(20,92)= -5.4042804241e-01
       coeff_l4(20,93)=  7.2109311819e-02
       coeff_l4(20,94)=  6.1601567268e-01
       coeff_l4(20,95)= -2.7748355269e-01
       coeff_l4(20,96)=  1.8522004783e-01
       coeff_l4(20,97)=  1.3316233456e-01
       coeff_l4(20,98)= -2.6522019878e-02
       coeff_l4(20,99)= -1.5369468927e-01
       coeff_l4(20,100)=  4.7946614027e-01
       coeff_l4(20,101)= -2.3389630020e-01
       coeff_l4(20,102)= -4.8411995173e-01
       coeff_l4(20,103)=  2.8955373913e-02
       coeff_l4(20,104)=  2.4452777207e-01
       coeff_l4(20,105)= -3.3640265465e-01
       coeff_l4(21,1)= -3.2257759571e-01
       coeff_l4(21,2)= -4.1956764460e-01
       coeff_l4(21,3)= -5.2405011654e-01
       coeff_l4(21,4)= -6.8209642172e-01
       coeff_l4(21,5)= -6.9933503866e-01
       coeff_l4(21,6)= -2.7591380477e-01
       coeff_l4(21,7)= -5.7042884827e-01
       coeff_l4(21,8)= -5.2772665024e-01
       coeff_l4(21,9)= -8.4329748154e-01
       coeff_l4(21,10)= -4.8215404153e-01
       coeff_l4(21,11)= -5.0392460823e-01
       coeff_l4(21,12)= -4.6421143413e-01
       coeff_l4(21,13)= -7.5271093845e-01
       coeff_l4(21,14)= -9.2202126980e-01
       coeff_l4(21,15)= -2.0690399408e-01
       coeff_l4(21,16)= -3.2461777329e-01
       coeff_l4(21,17)= -2.5528496504e-01
       coeff_l4(21,18)=  2.9962742329e-01
       coeff_l4(21,19)=  2.8140753508e-01
       coeff_l4(21,20)=  3.4629619122e-01
       coeff_l4(21,21)=  1.8999111652e-01
       coeff_l4(21,22)= -1.7197778821e-01
       coeff_l4(21,23)= -1.3761317730e-01
       coeff_l4(21,24)=  2.0637480915e-01
       coeff_l4(21,25)= -7.6881861687e-01
       coeff_l4(21,26)=  6.9798633456e-02
       coeff_l4(21,27)= -6.0633981228e-01
       coeff_l4(21,28)=  2.5294250250e-01
       coeff_l4(21,29)=  2.6125615835e-01
       coeff_l4(21,30)= -2.3720897734e-01
       coeff_l4(21,31)=  1.5835435688e-01
       coeff_l4(21,32)=  2.9152859934e-03
       coeff_l4(21,33)=  5.0480537117e-02
       coeff_l4(21,34)= -3.7766590714e-01
       coeff_l4(21,35)=  1.2854327261e-01
       coeff_l4(21,36)=  3.2980075479e-01
       coeff_l4(21,37)= -2.1912278607e-02
       coeff_l4(21,38)=  1.2714678049e-01
       coeff_l4(21,39)= -1.1258001905e-03
       coeff_l4(21,40)=  1.3278706372e-01
       coeff_l4(21,41)=  3.7396386266e-02
       coeff_l4(21,42)= -2.1825474501e-01
       coeff_l4(21,43)=  9.5888487995e-02
       coeff_l4(21,44)=  3.6784194526e-04
       coeff_l4(21,45)=  1.8196603283e-02
       coeff_l4(21,46)= -4.0510642529e-01
       coeff_l4(21,47)=  2.9700362682e-01
       coeff_l4(21,48)=  1.0874941200e-01
       coeff_l4(21,49)= -3.7253201008e-01
       coeff_l4(21,50)= -5.6895925663e-03
       coeff_l4(21,51)=  5.3385023028e-02
       coeff_l4(21,52)=  1.4301341772e-01
       coeff_l4(21,53)=  3.3750366420e-02
       coeff_l4(21,54)= -1.9519893825e-01
       coeff_l4(21,55)=  4.0868678689e-01
       coeff_l4(21,56)= -8.8323108852e-02
       coeff_l4(21,57)= -1.0524174571e-01
       coeff_l4(21,58)= -1.9299417734e-01
       coeff_l4(21,59)=  3.6679437757e-01
       coeff_l4(21,60)= -3.6840450764e-01
       coeff_l4(21,61)= -6.5990306437e-02
       coeff_l4(21,62)=  1.2869870663e-01
       coeff_l4(21,63)=  1.4547024854e-02
       coeff_l4(21,64)=  2.3919910192e-01
       coeff_l4(21,65)= -3.4320327640e-01
       coeff_l4(21,66)= -2.2982180119e-01
       coeff_l4(21,67)= -2.2324912250e-01
       coeff_l4(21,68)= -6.1174195260e-02
       coeff_l4(21,69)=  1.1744915694e-01
       coeff_l4(21,70)= -1.1682593077e-01
       coeff_l4(21,71)=  6.4114108682e-02
       coeff_l4(21,72)=  1.7526808381e-01
       coeff_l4(21,73)= -1.3535867631e-01
       coeff_l4(21,74)= -3.2477152348e-01
       coeff_l4(21,75)=  2.5479337201e-02
       coeff_l4(21,76)=  1.4606116712e-01
       coeff_l4(21,77)= -3.9945683966e-05
       coeff_l4(21,78)= -1.0500989109e-01
       coeff_l4(21,79)= -1.6391424835e-01
       coeff_l4(21,80)= -2.7185292915e-02
       coeff_l4(21,81)= -1.7006021738e-01
       coeff_l4(21,82)=  1.8479706347e-01
       coeff_l4(21,83)= -1.1941580474e-01
       coeff_l4(21,84)= -2.8294837475e-01
       coeff_l4(21,85)=  2.9741778970e-01
       coeff_l4(21,86)= -3.3250454068e-01
       coeff_l4(21,87)= -1.7237260938e-01
       coeff_l4(21,88)= -1.4214270748e-02
       coeff_l4(21,89)=  1.1985336989e-01
       coeff_l4(21,90)=  2.8566950932e-02
       coeff_l4(21,91)=  1.2066800147e-02
       coeff_l4(21,92)= -5.5007584393e-02
       coeff_l4(21,93)=  3.2000800967e-01
       coeff_l4(21,94)= -3.5027042031e-01
       coeff_l4(21,95)=  9.1465222836e-01
       coeff_l4(21,96)=  4.3076655269e-01
       coeff_l4(21,97)= -1.5136247873e-01
       coeff_l4(21,98)= -3.0349034071e-01
       coeff_l4(21,99)= -3.3323073387e-01
       coeff_l4(21,100)= -1.0037831962e-01
       coeff_l4(21,101)=  4.2809990048e-01
       coeff_l4(21,102)= -2.6528289914e-01
       coeff_l4(21,103)= -9.5859445632e-02
       coeff_l4(21,104)=  2.7259987593e-01
       coeff_l4(21,105)= -7.0822641253e-02
       coeff_l4(22,1)= -1.0583087206e+00
       coeff_l4(22,2)= -8.8139826059e-01
       coeff_l4(22,3)= -8.4743750095e-01
       coeff_l4(22,4)= -7.5002986193e-01
       coeff_l4(22,5)= -8.5768485069e-01
       coeff_l4(22,6)= -6.7910319567e-01
       coeff_l4(22,7)= -6.5632230043e-01
       coeff_l4(22,8)= -9.1272282600e-01
       coeff_l4(22,9)= -2.8964230418e-01
       coeff_l4(22,10)= -4.7174212337e-01
       coeff_l4(22,11)=  9.1898217797e-02
       coeff_l4(22,12)= -2.5220969319e-01
       coeff_l4(22,13)=  1.6407489777e-01
       coeff_l4(22,14)= -2.2321490943e-01
       coeff_l4(22,15)= -1.4376299083e-01
       coeff_l4(22,16)= -1.4304292202e-01
       coeff_l4(22,17)= -9.5989190042e-02
       coeff_l4(22,18)=  2.6237692684e-02
       coeff_l4(22,19)= -1.9621509314e-01
       coeff_l4(22,20)= -8.5149727762e-02
       coeff_l4(22,21)=  1.4901748300e-01
       coeff_l4(22,22)= -2.8144024312e-02
       coeff_l4(22,23)=  4.6035894752e-01
       coeff_l4(22,24)= -9.3347012997e-02
       coeff_l4(22,25)=  1.4845369756e-01
       coeff_l4(22,26)= -2.5789567828e-01
       coeff_l4(22,27)= -3.1286498904e-01
       coeff_l4(22,28)=  5.0731506199e-02
       coeff_l4(22,29)=  6.4982645214e-02
       coeff_l4(22,30)= -4.9847643822e-02
       coeff_l4(22,31)=  2.2478135303e-02
       coeff_l4(22,32)= -4.2926943302e-01
       coeff_l4(22,33)=  2.0093680918e-01
       coeff_l4(22,34)= -1.1035624146e-01
       coeff_l4(22,35)=  9.0456679463e-02
       coeff_l4(22,36)= -9.9909231067e-02
       coeff_l4(22,37)=  1.4595875144e-01
       coeff_l4(22,38)= -1.8831412494e-01
       coeff_l4(22,39)=  1.8516196311e-01
       coeff_l4(22,40)= -1.4181958139e-01
       coeff_l4(22,41)=  1.7643699050e-01
       coeff_l4(22,42)= -3.2339864969e-01
       coeff_l4(22,43)=  1.8256658316e-01
       coeff_l4(22,44)= -5.4964836687e-02
       coeff_l4(22,45)=  2.8874373436e-01
       coeff_l4(22,46)=  6.0720611364e-02
       coeff_l4(22,47)= -3.2860916853e-01
       coeff_l4(22,48)= -3.5286110640e-01
       coeff_l4(22,49)= -1.6982501373e-02
       coeff_l4(22,50)=  2.2886995971e-01
       coeff_l4(22,51)= -2.5412312150e-01
       coeff_l4(22,52)=  2.9901114106e-01
       coeff_l4(22,53)=  1.2404058129e-02
       coeff_l4(22,54)=  7.7996075153e-02
       coeff_l4(22,55)= -2.5313687325e-01
       coeff_l4(22,56)=  1.2473168224e-01
       coeff_l4(22,57)=  1.2540282309e-01
       coeff_l4(22,58)= -2.4236178398e-01
       coeff_l4(22,59)=  9.7000189126e-02
       coeff_l4(22,60)=  2.8808116913e-01
       coeff_l4(22,61)=  1.9727701321e-02
       coeff_l4(22,62)= -9.1340579093e-02
       coeff_l4(22,63)= -2.5032121688e-02
       coeff_l4(22,64)=  8.4319785237e-02
       coeff_l4(22,65)=  9.8063059151e-02
       coeff_l4(22,66)=  1.2670758367e-01
       coeff_l4(22,67)= -1.6457365453e-01
       coeff_l4(22,68)= -4.1929498315e-01
       coeff_l4(22,69)=  2.3672553897e-01
       coeff_l4(22,70)=  9.7625009716e-02
       coeff_l4(22,71)=  1.8628160655e-01
       coeff_l4(22,72)= -3.1865373254e-02
       coeff_l4(22,73)= -7.4640534818e-02
       coeff_l4(22,74)= -1.2723195553e-01
       coeff_l4(22,75)= -5.4727021605e-02
       coeff_l4(22,76)= -5.3121280670e-01
       coeff_l4(22,77)=  2.1824014187e-01
       coeff_l4(22,78)= -1.0968976468e-01
       coeff_l4(22,79)=  2.0025333762e-01
       coeff_l4(22,80)= -4.0981460363e-02
       coeff_l4(22,81)=  8.2752136514e-03
       coeff_l4(22,82)=  2.0550532639e-01
       coeff_l4(22,83)=  2.0850357413e-01
       coeff_l4(22,84)=  1.5276330709e-01
       coeff_l4(22,85)=  1.0375453532e-01
       coeff_l4(22,86)=  4.9687460065e-02
       coeff_l4(22,87)=  1.3632328808e-01
       coeff_l4(22,88)= -3.5276955366e-01
       coeff_l4(22,89)= -2.0758642256e-01
       coeff_l4(22,90)=  1.2317988276e-01
       coeff_l4(22,91)= -1.0197762400e-01
       coeff_l4(22,92)= -2.1613940597e-02
       coeff_l4(22,93)=  8.0232098699e-02
       coeff_l4(22,94)= -2.3059394956e-01
       coeff_l4(22,95)= -5.4145079851e-01
       coeff_l4(22,96)=  1.7786304653e-01
       coeff_l4(22,97)= -2.6705935597e-01
       coeff_l4(22,98)= -1.6227769852e-01
       coeff_l4(22,99)=  2.1825805306e-01
       coeff_l4(22,100)= -6.0749940574e-02
       coeff_l4(22,101)=  1.3105207682e-01
       coeff_l4(22,102)= -7.4298597872e-02
       coeff_l4(22,103)= -2.2095976397e-02
       coeff_l4(22,104)=  1.1075111479e-01
       coeff_l4(22,105)=  1.9924037158e-02
       coeff_l4(23,1)= -5.1601773500e-01
       coeff_l4(23,2)= -5.6610411406e-01
       coeff_l4(23,3)= -2.9353177547e-01
       coeff_l4(23,4)= -5.0551354885e-01
       coeff_l4(23,5)= -5.4812282324e-01
       coeff_l4(23,6)= -5.2039021254e-01
       coeff_l4(23,7)= -4.0146321058e-01
       coeff_l4(23,8)= -3.3014157414e-01
       coeff_l4(23,9)= -1.4507147670e-01
       coeff_l4(23,10)= -1.6418260336e-01
       coeff_l4(23,11)=  2.4481609464e-02
       coeff_l4(23,12)=  1.2400978804e-01
       coeff_l4(23,13)= -9.3752130866e-02
       coeff_l4(23,14)= -1.6312350333e-01
       coeff_l4(23,15)= -6.3191369176e-02
       coeff_l4(23,16)= -1.6173887253e-01
       coeff_l4(23,17)= -4.3915465474e-02
       coeff_l4(23,18)= -3.0284471810e-02
       coeff_l4(23,19)=  5.9046123177e-02
       coeff_l4(23,20)=  8.8172756135e-02
       coeff_l4(23,21)=  1.1781051010e-01
       coeff_l4(23,22)= -1.9569942728e-02
       coeff_l4(23,23)=  6.2772817910e-02
       coeff_l4(23,24)=  4.9180474132e-02
       coeff_l4(23,25)= -1.4136451483e-01
       coeff_l4(23,26)= -3.9946295321e-02
       coeff_l4(23,27)=  1.2098573148e-01
       coeff_l4(23,28)=  1.2491944432e-01
       coeff_l4(23,29)=  4.0726818144e-02
       coeff_l4(23,30)= -6.8524628878e-02
       coeff_l4(23,31)=  1.3540934026e-01
       coeff_l4(23,32)=  5.0561714917e-02
       coeff_l4(23,33)= -2.1592585370e-02
       coeff_l4(23,34)=  1.3954274356e-01
       coeff_l4(23,35)=  1.0948478431e-01
       coeff_l4(23,36)= -1.3274469972e-01
       coeff_l4(23,37)= -7.7536925673e-02
       coeff_l4(23,38)= -1.3317047060e-01
       coeff_l4(23,39)= -1.6423647106e-01
       coeff_l4(23,40)=  1.1224664003e-01
       coeff_l4(23,41)=  2.6164239272e-02
       coeff_l4(23,42)=  1.2510603666e-01
       coeff_l4(23,43)= -4.6885255724e-02
       coeff_l4(23,44)= -9.1396935284e-02
       coeff_l4(23,45)= -2.9301567003e-02
       coeff_l4(23,46)=  4.6182855964e-02
       coeff_l4(23,47)=  1.1223349720e-01
       coeff_l4(23,48)= -1.2688288093e-01
       coeff_l4(23,49)=  4.5860540122e-02
       coeff_l4(23,50)= -6.5337061882e-02
       coeff_l4(23,51)= -9.2023201287e-02
       coeff_l4(23,52)=  1.4871840179e-01
       coeff_l4(23,53)=  1.4110271633e-01
       coeff_l4(23,54)= -8.3164900541e-02
       coeff_l4(23,55)= -3.6367002875e-02
       coeff_l4(23,56)=  9.3080766499e-02
       coeff_l4(23,57)= -3.4837707877e-02
       coeff_l4(23,58)=  8.6289554834e-02
       coeff_l4(23,59)=  1.4921969734e-02
       coeff_l4(23,60)= -7.2582159191e-03
       coeff_l4(23,61)= -8.0548755825e-02
       coeff_l4(23,62)=  2.2598519921e-01
       coeff_l4(23,63)=  4.2912662029e-02
       coeff_l4(23,64)= -8.7724259356e-04
       coeff_l4(23,65)=  2.3380950093e-02
       coeff_l4(23,66)= -1.4398203231e-02
       coeff_l4(23,67)=  1.6049531102e-01
       coeff_l4(23,68)=  4.3941695243e-02
       coeff_l4(23,69)=  1.3598974049e-01
       coeff_l4(23,70)=  1.9619785249e-01
       coeff_l4(23,71)= -1.0599910468e-01
       coeff_l4(23,72)=  6.9625899196e-02
       coeff_l4(23,73)=  1.4967705309e-01
       coeff_l4(23,74)=  1.5073063970e-01
       coeff_l4(23,75)=  1.2827962637e-01
       coeff_l4(23,76)=  1.3225342333e-01
       coeff_l4(23,77)=  1.2267153710e-01
       coeff_l4(23,78)= -1.4346607029e-01
       coeff_l4(23,79)= -6.8338848650e-02
       coeff_l4(23,80)=  1.1921752244e-01
       coeff_l4(23,81)= -1.0548562557e-01
       coeff_l4(23,82)= -1.0303518921e-01
       coeff_l4(23,83)=  3.6497294903e-02
       coeff_l4(23,84)=  1.3059714437e-01
       coeff_l4(23,85)= -1.4631485939e-01
       coeff_l4(23,86)=  9.4519749284e-02
       coeff_l4(23,87)=  8.8485211134e-02
       coeff_l4(23,88)=  1.1529732496e-01
       coeff_l4(23,89)=  7.2355903685e-02
       coeff_l4(23,90)= -8.4750205278e-02
       coeff_l4(23,91)= -1.4758710563e-01
       coeff_l4(23,92)= -3.3341854811e-02
       coeff_l4(23,93)=  1.0316805542e-01
       coeff_l4(23,94)= -6.6321469843e-02
       coeff_l4(23,95)= -6.0337852687e-02
       coeff_l4(23,96)= -1.1541348696e-01
       coeff_l4(23,97)= -1.0486894101e-01
       coeff_l4(23,98)= -2.5656800717e-02
       coeff_l4(23,99)= -1.0820093006e-01
       coeff_l4(23,100)=  6.1333887279e-03
       coeff_l4(23,101)=  2.7118703350e-02
       coeff_l4(23,102)=  1.2827499211e-01
       coeff_l4(23,103)=  1.8161879480e-01
       coeff_l4(23,104)= -9.0832955902e-04
       coeff_l4(23,105)=  4.2261924595e-02
       coeff_l4(24,1)= -4.7348555923e-01
       coeff_l4(24,2)= -3.0139273405e-01
       coeff_l4(24,3)= -2.3562584817e-01
       coeff_l4(24,4)= -3.2354676723e-01
       coeff_l4(24,5)= -5.6810945272e-01
       coeff_l4(24,6)= -4.3903747201e-01
       coeff_l4(24,7)= -2.2261281312e-01
       coeff_l4(24,8)= -3.4113350511e-01
       coeff_l4(24,9)=  2.3343354464e-01
       coeff_l4(24,10)= -2.0569072664e-01
       coeff_l4(24,11)=  8.9298225939e-02
       coeff_l4(24,12)=  3.7275400758e-01
       coeff_l4(24,13)= -1.1416199617e-02
       coeff_l4(24,14)=  2.0077963173e-01
       coeff_l4(24,15)=  3.3860245347e-01
       coeff_l4(24,16)=  1.5304344893e-01
       coeff_l4(24,17)= -1.4428605139e-01
       coeff_l4(24,18)= -1.8665863201e-02
       coeff_l4(24,19)=  7.3588890955e-03
       coeff_l4(24,20)=  4.4567896985e-03
       coeff_l4(24,21)=  1.9834727049e-01
       coeff_l4(24,22)= -8.0693550408e-02
       coeff_l4(24,23)= -1.0059646517e-01
       coeff_l4(24,24)=  6.9440871477e-02
       coeff_l4(24,25)=  6.5519221127e-02
       coeff_l4(24,26)=  3.8570038974e-02
       coeff_l4(24,27)= -3.9155732840e-02
       coeff_l4(24,28)= -5.9751890600e-02
       coeff_l4(24,29)= -2.1456232667e-01
       coeff_l4(24,30)=  1.5243031085e-01
       coeff_l4(24,31)= -2.7470620349e-02
       coeff_l4(24,32)=  4.5691454411e-01
       coeff_l4(24,33)= -6.7154221237e-02
       coeff_l4(24,34)=  1.2587406673e-02
       coeff_l4(24,35)= -1.5135106444e-01
       coeff_l4(24,36)= -1.4895056188e-01
       coeff_l4(24,37)= -1.4931644313e-02
       coeff_l4(24,38)=  2.8903337196e-02
       coeff_l4(24,39)= -1.0183988512e-01
       coeff_l4(24,40)= -1.2793424726e-01
       coeff_l4(24,41)= -1.1783645302e-01
       coeff_l4(24,42)=  1.9420816004e-01
       coeff_l4(24,43)= -2.7661671862e-02
       coeff_l4(24,44)=  1.3392703235e-01
       coeff_l4(24,45)=  1.2530137599e-01
       coeff_l4(24,46)= -1.2748499215e-01
       coeff_l4(24,47)=  5.3248345852e-02
       coeff_l4(24,48)=  2.5175020099e-01
       coeff_l4(24,49)=  8.3046974614e-03
       coeff_l4(24,50)= -1.8512831628e-01
       coeff_l4(24,51)= -3.9495807141e-02
       coeff_l4(24,52)=  8.9218253270e-03
       coeff_l4(24,53)= -2.3907396942e-02
       coeff_l4(24,54)= -1.4848896861e-01
       coeff_l4(24,55)= -3.2280318439e-02
       coeff_l4(24,56)= -1.5250682831e-01
       coeff_l4(24,57)=  7.8045524657e-02
       coeff_l4(24,58)= -3.1436849385e-02
       coeff_l4(24,59)=  2.5033375621e-01
       coeff_l4(24,60)= -5.6227724999e-02
       coeff_l4(24,61)= -7.9046681523e-02
       coeff_l4(24,62)=  1.7566458881e-01
       coeff_l4(24,63)= -1.8604533374e-01
       coeff_l4(24,64)= -1.2673759460e-01
       coeff_l4(24,65)= -5.8306206018e-02
       coeff_l4(24,66)=  6.8955548108e-02
       coeff_l4(24,67)=  7.8422419727e-02
       coeff_l4(24,68)=  2.5689184666e-01
       coeff_l4(24,69)=  2.4563871324e-01
       coeff_l4(24,70)=  3.6589545012e-01
       coeff_l4(24,71)= -7.3071524501e-02
       coeff_l4(24,72)= -1.3928900659e-01
       coeff_l4(24,73)= -2.9263975099e-02
       coeff_l4(24,74)=  1.2853403389e-01
       coeff_l4(24,75)=  2.4508231878e-01
       coeff_l4(24,76)=  2.1087504923e-02
       coeff_l4(24,77)= -1.4880517125e-01
       coeff_l4(24,78)=  7.1541488171e-02
       coeff_l4(24,79)=  1.9526742399e-01
       coeff_l4(24,80)=  1.7583666742e-01
       coeff_l4(24,81)= -2.7482904494e-02
       coeff_l4(24,82)=  6.5330907702e-02
       coeff_l4(24,83)= -8.6981683969e-02
       coeff_l4(24,84)= -1.3222509623e-01
       coeff_l4(24,85)=  5.6609869003e-02
       coeff_l4(24,86)= -8.7034888566e-02
       coeff_l4(24,87)= -3.2552456856e-01
       coeff_l4(24,88)=  7.3619171977e-02
       coeff_l4(24,89)= -2.5860458612e-01
       coeff_l4(24,90)= -7.5608819723e-02
       coeff_l4(24,91)= -2.6740035415e-01
       coeff_l4(24,92)=  8.4835834801e-02
       coeff_l4(24,93)=  2.4746957421e-01
       coeff_l4(24,94)=  3.2236295938e-01
       coeff_l4(24,95)=  1.1750326306e-01
       coeff_l4(24,96)=  1.6179026663e-01
       coeff_l4(24,97)=  1.2616066635e-01
       coeff_l4(24,98)=  9.3223817647e-02
       coeff_l4(24,99)= -3.4407489002e-02
       coeff_l4(24,100)= -3.2587385178e-01
       coeff_l4(24,101)=  1.2625491247e-02
       coeff_l4(24,102)= -3.3983208239e-02
       coeff_l4(24,103)=  4.3368585408e-02
       coeff_l4(24,104)= -1.2503026426e-01
       coeff_l4(24,105)= -1.9069437683e-01
       coeff_l4(25,1)= -9.4720053673e-01
       coeff_l4(25,2)= -5.0044745207e-01
       coeff_l4(25,3)= -5.0817048550e-01
       coeff_l4(25,4)= -1.0540108681e+00
       coeff_l4(25,5)= -2.7386447787e-01
       coeff_l4(25,6)= -3.8294935226e-01
       coeff_l4(25,7)= -9.1228884459e-01
       coeff_l4(25,8)= -4.0308910608e-01
       coeff_l4(25,9)= -4.4825196266e-01
       coeff_l4(25,10)= -6.8562316895e-01
       coeff_l4(25,11)= -2.4914388359e-01
       coeff_l4(25,12)= -2.9598578811e-01
       coeff_l4(25,13)= -2.0576070249e-01
       coeff_l4(25,14)= -8.1247264147e-01
       coeff_l4(25,15)=  3.2324212790e-01
       coeff_l4(25,16)=  1.5234789252e-01
       coeff_l4(25,17)=  6.6711330414e-01
       coeff_l4(25,18)= -3.3553144336e-01
       coeff_l4(25,19)=  8.1283047795e-02
       coeff_l4(25,20)=  4.9898573756e-01
       coeff_l4(25,21)= -2.7038049698e-01
       coeff_l4(25,22)=  2.0370356739e-01
       coeff_l4(25,23)= -3.5403600335e-01
       coeff_l4(25,24)=  2.5246006250e-01
       coeff_l4(25,25)= -4.1624899954e-02
       coeff_l4(25,26)=  1.3707599044e-01
       coeff_l4(25,27)=  3.3494357020e-02
       coeff_l4(25,28)= -3.2355976105e-01
       coeff_l4(25,29)= -3.3351388574e-01
       coeff_l4(25,30)=  1.7194744945e-01
       coeff_l4(25,31)= -3.8110479712e-02
       coeff_l4(25,32)=  8.3770029247e-02
       coeff_l4(25,33)=  9.5852902159e-03
       coeff_l4(25,34)= -1.1190244555e-01
       coeff_l4(25,35)=  7.7547781169e-02
       coeff_l4(25,36)= -2.1893098950e-01
       coeff_l4(25,37)= -3.7841200829e-01
       coeff_l4(25,38)= -2.0906829834e-01
       coeff_l4(25,39)= -6.9002732635e-02
       coeff_l4(25,40)= -3.9946156740e-01
       coeff_l4(25,41)=  6.4359962940e-01
       coeff_l4(25,42)=  5.3362351656e-01
       coeff_l4(25,43)= -4.9520637840e-02
       coeff_l4(25,44)=  1.4574915171e-01
       coeff_l4(25,45)= -2.3326122761e-01
       coeff_l4(25,46)= -4.3898946047e-01
       coeff_l4(25,47)= -3.8645076752e-01
       coeff_l4(25,48)= -4.4263795018e-01
       coeff_l4(25,49)= -4.9488268793e-02
       coeff_l4(25,50)= -9.9923938513e-02
       coeff_l4(25,51)=  5.2892273664e-01
       coeff_l4(25,52)= -3.2839885354e-01
       coeff_l4(25,53)=  2.8117510676e-01
       coeff_l4(25,54)=  1.3593778014e-01
       coeff_l4(25,55)= -2.8570896387e-01
       coeff_l4(25,56)=  1.2876805663e-01
       coeff_l4(25,57)=  1.0716420412e-01
       coeff_l4(25,58)= -1.5948042274e-01
       coeff_l4(25,59)=  1.1909218878e-01
       coeff_l4(25,60)= -2.5074338913e-01
       coeff_l4(25,61)= -2.0218722522e-01
       coeff_l4(25,62)= -1.7401418090e-01
       coeff_l4(25,63)=  4.1131865978e-01
       coeff_l4(25,64)=  1.2104241550e-01
       coeff_l4(25,65)= -5.3866751492e-02
       coeff_l4(25,66)= -1.5088315308e-01
       coeff_l4(25,67)=  4.6885744669e-03
       coeff_l4(25,68)=  2.3772586882e-01
       coeff_l4(25,69)=  1.7494659871e-02
       coeff_l4(25,70)=  1.6056525707e-01
       coeff_l4(25,71)=  4.1892647743e-02
       coeff_l4(25,72)= -1.1590226740e-01
       coeff_l4(25,73)= -4.8780891299e-01
       coeff_l4(25,74)=  1.6921047866e-01
       coeff_l4(25,75)= -8.2684308290e-02
       coeff_l4(25,76)=  3.5447677970e-01
       coeff_l4(25,77)= -2.5609059259e-02
       coeff_l4(25,78)=  2.1784314513e-01
       coeff_l4(25,79)= -1.5176011622e-01
       coeff_l4(25,80)=  3.2709670067e-01
       coeff_l4(25,81)=  1.6250324249e-01
       coeff_l4(25,82)=  1.3155730069e-01
       coeff_l4(25,83)= -4.0002610534e-02
       coeff_l4(25,84)=  4.0716677904e-01
       coeff_l4(25,85)=  1.0678119957e-01
       coeff_l4(25,86)= -2.5038436055e-01
       coeff_l4(25,87)=  2.3036138713e-01
       coeff_l4(25,88)= -7.0206135511e-02
       coeff_l4(25,89)= -1.0145966895e-02
       coeff_l4(25,90)= -4.3631423265e-02
       coeff_l4(25,91)= -4.2441081256e-02
       coeff_l4(25,92)= -4.1353806853e-01
       coeff_l4(25,93)=  3.5508733988e-01
       coeff_l4(25,94)=  1.0780000687e+00
       coeff_l4(25,95)= -2.7922242880e-01
       coeff_l4(25,96)= -2.3121564090e-01
       coeff_l4(25,97)=  5.5982702971e-01
       coeff_l4(25,98)=  3.5726448894e-01
       coeff_l4(25,99)= -2.6720795035e-01
       coeff_l4(25,100)= -4.9712675810e-01
       coeff_l4(25,101)= -3.1172686815e-01
       coeff_l4(25,102)= -8.4197513759e-02
       coeff_l4(25,103)= -8.3241663873e-02
       coeff_l4(25,104)=  1.5860456228e-01
       coeff_l4(25,105)= -4.0147176385e-01
       coeff_l4(26,1)= -5.5548262596e-01
       coeff_l4(26,2)= -4.4456401467e-01
       coeff_l4(26,3)= -7.0025855303e-01
       coeff_l4(26,4)= -5.1076084375e-01
       coeff_l4(26,5)= -8.4584879875e-01
       coeff_l4(26,6)= -4.3813472986e-01
       coeff_l4(26,7)= -6.7668873072e-01
       coeff_l4(26,8)= -7.2610068321e-01
       coeff_l4(26,9)= -1.4407801628e-01
       coeff_l4(26,10)=  2.7314191684e-02
       coeff_l4(26,11)= -1.5104344487e-01
       coeff_l4(26,12)= -3.4281930327e-01
       coeff_l4(26,13)= -3.0874210596e-01
       coeff_l4(26,14)=  1.3334456086e-01
       coeff_l4(26,15)=  1.1001286656e-01
       coeff_l4(26,16)= -4.1827712208e-02
       coeff_l4(26,17)= -9.1060705483e-02
       coeff_l4(26,18)=  3.5412527621e-02
       coeff_l4(26,19)=  1.1625868268e-02
       coeff_l4(26,20)= -1.7633371055e-01
       coeff_l4(26,21)=  1.1214102805e-01
       coeff_l4(26,22)= -3.7370003760e-02
       coeff_l4(26,23)=  6.4794912934e-02
       coeff_l4(26,24)=  9.6688464284e-02
       coeff_l4(26,25)= -1.1132002808e-02
       coeff_l4(26,26)= -6.0223322362e-02
       coeff_l4(26,27)= -9.1373190284e-02
       coeff_l4(26,28)= -3.1904304028e-01
       coeff_l4(26,29)=  3.5841148347e-02
       coeff_l4(26,30)=  1.5796607733e-01
       coeff_l4(26,31)=  4.8537421972e-02
       coeff_l4(26,32)=  2.3424924910e-01
       coeff_l4(26,33)=  1.0160652548e-01
       coeff_l4(26,34)= -7.6417930424e-02
       coeff_l4(26,35)=  1.4508107305e-01
       coeff_l4(26,36)=  3.9288535714e-02
       coeff_l4(26,37)=  2.6345057413e-02
       coeff_l4(26,38)=  2.7633973956e-01
       coeff_l4(26,39)= -1.3573656976e-01
       coeff_l4(26,40)= -2.1076865494e-02
       coeff_l4(26,41)= -1.2143645436e-01
       coeff_l4(26,42)= -7.9458281398e-02
       coeff_l4(26,43)= -9.5990352333e-02
       coeff_l4(26,44)= -8.9237578213e-02
       coeff_l4(26,45)=  1.8489003181e-01
       coeff_l4(26,46)=  1.8447528780e-01
       coeff_l4(26,47)= -9.9973425269e-02
       coeff_l4(26,48)=  2.6196327806e-01
       coeff_l4(26,49)=  3.9498809725e-02
       coeff_l4(26,50)=  1.5072041750e-01
       coeff_l4(26,51)=  5.2518453449e-02
       coeff_l4(26,52)=  1.7864102125e-01
       coeff_l4(26,53)= -5.2256979048e-02
       coeff_l4(26,54)= -6.3663810492e-02
       coeff_l4(26,55)= -6.6836856306e-02
       coeff_l4(26,56)= -9.7755752504e-02
       coeff_l4(26,57)=  1.0081666708e-01
       coeff_l4(26,58)= -3.9613917470e-02
       coeff_l4(26,59)=  5.2006173879e-02
       coeff_l4(26,60)=  6.2918052077e-02
       coeff_l4(26,61)= -2.3136921227e-02
       coeff_l4(26,62)=  4.6292547137e-02
       coeff_l4(26,63)= -1.4019788802e-01
       coeff_l4(26,64)=  1.7443647981e-01
       coeff_l4(26,65)=  1.6394007206e-01
       coeff_l4(26,66)=  3.2193135470e-02
       coeff_l4(26,67)= -2.3318771273e-03
       coeff_l4(26,68)=  9.0008415282e-02
       coeff_l4(26,69)=  2.7179226279e-01
       coeff_l4(26,70)=  1.4242747426e-01
       coeff_l4(26,71)=  2.9357776046e-01
       coeff_l4(26,72)=  1.3414870203e-01
       coeff_l4(26,73)=  6.5809041262e-02
       coeff_l4(26,74)=  1.3992141187e-01
       coeff_l4(26,75)= -9.4819717109e-02
       coeff_l4(26,76)=  5.8135967702e-02
       coeff_l4(26,77)= -1.7861375213e-01
       coeff_l4(26,78)= -1.8382222950e-01
       coeff_l4(26,79)= -1.3906574249e-01
       coeff_l4(26,80)= -2.2448085248e-02
       coeff_l4(26,81)= -1.9058524072e-01
       coeff_l4(26,82)=  1.3487746008e-02
       coeff_l4(26,83)=  1.2335284054e-01
       coeff_l4(26,84)= -3.3764164895e-02
       coeff_l4(26,85)=  2.7375760674e-01
       coeff_l4(26,86)=  2.4043334648e-02
       coeff_l4(26,87)=  1.0910157114e-01
       coeff_l4(26,88)=  1.1285580695e-01
       coeff_l4(26,89)=  6.1781384051e-02
       coeff_l4(26,90)=  1.8406784534e-01
       coeff_l4(26,91)=  1.3332097232e-01
       coeff_l4(26,92)=  5.4263036698e-02
       coeff_l4(26,93)= -2.7238987386e-02
       coeff_l4(26,94)= -3.1753656268e-01
       coeff_l4(26,95)=  2.0356379449e-02
       coeff_l4(26,96)= -3.8046589494e-01
       coeff_l4(26,97)= -3.1826749444e-01
       coeff_l4(26,98)=  2.7346771210e-02
       coeff_l4(26,99)=  1.5474349260e-01
       coeff_l4(26,100)= -6.1040494591e-02
       coeff_l4(26,101)= -1.9580567256e-02
       coeff_l4(26,102)=  1.1888479814e-02
       coeff_l4(26,103)=  7.7262741979e-04
       coeff_l4(26,104)=  1.6618371010e-01
       coeff_l4(26,105)= -4.3475278653e-03
       coeff_l4(27,1)=  7.3576050997e-01
       coeff_l4(27,2)=  6.4777553082e-01
       coeff_l4(27,3)=  5.0066173077e-01
       coeff_l4(27,4)=  7.2114765644e-01
       coeff_l4(27,5)=  5.4041934013e-01
       coeff_l4(27,6)=  4.4616383314e-01
       coeff_l4(27,7)=  8.1311351061e-01
       coeff_l4(27,8)=  6.5192407370e-01
       coeff_l4(27,9)= -4.8971271515e-01
       coeff_l4(27,10)= -2.0767870545e-01
       coeff_l4(27,11)= -4.7931957245e+00
       coeff_l4(27,12)= -4.7922182083e+00
       coeff_l4(27,13)= -4.1276578903e+00
       coeff_l4(27,14)=  5.4785680771e-01
       coeff_l4(27,15)= -1.7685223371e-02
       coeff_l4(27,16)=  1.1706066877e-01
       coeff_l4(27,17)= -5.3890716285e-02
       coeff_l4(27,18)= -3.4809401631e-01
       coeff_l4(27,19)=  2.1787661314e-01
       coeff_l4(27,20)=  4.4542588294e-02
       coeff_l4(27,21)= -3.5450737923e-02
       coeff_l4(27,22)=  3.4519180655e-02
       coeff_l4(27,23)=  7.1739017963e-02
       coeff_l4(27,24)= -4.3343435973e-02
       coeff_l4(27,25)= -2.5592681766e-01
       coeff_l4(27,26)=  2.5410622358e-02
       coeff_l4(27,27)=  2.1509051323e-02
       coeff_l4(27,28)= -9.8752334714e-02
       coeff_l4(27,29)= -1.5545982122e-01
       coeff_l4(27,30)=  1.9261658192e-01
       coeff_l4(27,31)=  2.8926357627e-01
       coeff_l4(27,32)=  1.3610944152e-01
       coeff_l4(27,33)= -1.2360998988e-01
       coeff_l4(27,34)= -1.7987178266e-01
       coeff_l4(27,35)=  4.0918614832e-04
       coeff_l4(27,36)=  2.0306797698e-02
       coeff_l4(27,37)= -5.5576536804e-02
       coeff_l4(27,38)=  1.5628108755e-02
       coeff_l4(27,39)=  3.1711209565e-02
       coeff_l4(27,40)= -8.5339970887e-02
       coeff_l4(27,41)= -5.5421035737e-02
       coeff_l4(27,42)= -1.0285402648e-02
       coeff_l4(27,43)=  1.1000252515e-01
       coeff_l4(27,44)=  1.0330778360e-01
       coeff_l4(27,45)=  4.8695169389e-02
       coeff_l4(27,46)=  1.6584466398e-01
       coeff_l4(27,47)= -5.5005270988e-02
       coeff_l4(27,48)=  1.4348399127e-03
       coeff_l4(27,49)=  1.0014787316e-01
       coeff_l4(27,50)= -4.2175748944e-01
       coeff_l4(27,51)=  1.3729722798e-01
       coeff_l4(27,52)=  9.0539187193e-02
       coeff_l4(27,53)= -1.0774270445e-01
       coeff_l4(27,54)= -2.8794175386e-01
       coeff_l4(27,55)= -5.5071704090e-02
       coeff_l4(27,56)= -2.4840934575e-01
       coeff_l4(27,57)= -1.7340794206e-01
       coeff_l4(27,58)= -1.0600168258e-01
       coeff_l4(27,59)= -5.5404421873e-03
       coeff_l4(27,60)= -1.9762547687e-02
       coeff_l4(27,61)= -4.2898106575e-01
       coeff_l4(27,62)=  7.1497052908e-02
       coeff_l4(27,63)=  1.6157582402e-01
       coeff_l4(27,64)= -7.7320798300e-03
       coeff_l4(27,65)=  1.5559628606e-01
       coeff_l4(27,66)=  1.2319578975e-01
       coeff_l4(27,67)= -3.0902199447e-02
       coeff_l4(27,68)= -1.8334451318e-01
       coeff_l4(27,69)= -1.0756479949e-01
       coeff_l4(27,70)= -1.0492944159e-02
       coeff_l4(27,71)= -2.2127306461e-01
       coeff_l4(27,72)= -2.2588934004e-01
       coeff_l4(27,73)=  1.9725680351e-01
       coeff_l4(27,74)= -3.6154499650e-01
       coeff_l4(27,75)= -1.5139976144e-01
       coeff_l4(27,76)= -1.9323588908e-01
       coeff_l4(27,77)= -8.4536015987e-02
       coeff_l4(27,78)= -6.8050049245e-02
       coeff_l4(27,79)=  1.7148397863e-01
       coeff_l4(27,80)= -1.5188993514e-01
       coeff_l4(27,81)=  2.3846903443e-01
       coeff_l4(27,82)= -4.0534473956e-02
       coeff_l4(27,83)=  1.4083768427e-01
       coeff_l4(27,84)= -5.6932043284e-02
       coeff_l4(27,85)=  6.9216258824e-02
       coeff_l4(27,86)= -1.4578943141e-02
       coeff_l4(27,87)= -4.3173022568e-02
       coeff_l4(27,88)= -1.2393046170e-01
       coeff_l4(27,89)= -1.4580898918e-02
       coeff_l4(27,90)=  8.8455185294e-02
       coeff_l4(27,91)=  1.2489580363e-01
       coeff_l4(27,92)=  6.7711895704e-01
       coeff_l4(27,93)= -7.3234248161e-01
       coeff_l4(27,94)= -2.6599259377e+00
       coeff_l4(27,95)=  4.0395751595e-01
       coeff_l4(27,96)= -1.6927635670e+00
       coeff_l4(27,97)= -1.5244637728e+00
       coeff_l4(27,98)=  5.0518041849e-01
       coeff_l4(27,99)= -1.3617585599e-01
       coeff_l4(27,100)=  5.6031483412e-01
       coeff_l4(27,101)= -5.6807219982e-02
       coeff_l4(27,102)= -1.2247567065e-02
       coeff_l4(27,103)=  3.1362831593e-01
       coeff_l4(27,104)=  6.6766709089e-01
       coeff_l4(27,105)= -9.6585057676e-02
       coeff_l4(28,1)=  8.4775918722e-01
       coeff_l4(28,2)=  8.7271761894e-01
       coeff_l4(28,3)=  7.2611105442e-01
       coeff_l4(28,4)=  8.6382156610e-01
       coeff_l4(28,5)=  6.3634401560e-01
       coeff_l4(28,6)=  4.2041286826e-01
       coeff_l4(28,7)=  1.2723339796e+00
       coeff_l4(28,8)=  7.6726955175e-01
       coeff_l4(28,9)= -4.6928599477e-01
       coeff_l4(28,10)= -4.2405724525e-01
       coeff_l4(28,11)= -3.4537260532e+00
       coeff_l4(28,12)= -3.5950055122e+00
       coeff_l4(28,13)= -3.0243732929e+00
       coeff_l4(28,14)=  2.1082141995e-01
       coeff_l4(28,15)=  1.5567724407e-01
       coeff_l4(28,16)=  5.6645657867e-02
       coeff_l4(28,17)= -6.4752921462e-02
       coeff_l4(28,18)= -3.0514213443e-01
       coeff_l4(28,19)=  2.1425338089e-01
       coeff_l4(28,20)= -1.3635887206e-01
       coeff_l4(28,21)=  1.9644898176e-01
       coeff_l4(28,22)= -2.0929520950e-02
       coeff_l4(28,23)=  8.3938412368e-02
       coeff_l4(28,24)=  3.0679026246e-01
       coeff_l4(28,25)=  3.1136363745e-01
       coeff_l4(28,26)= -3.5585922003e-01
       coeff_l4(28,27)= -2.1399474144e-01
       coeff_l4(28,28)= -1.8284694850e-01
       coeff_l4(28,29)= -2.1434459835e-02
       coeff_l4(28,30)= -8.3367258310e-02
       coeff_l4(28,31)=  1.7371644080e-01
       coeff_l4(28,32)=  3.6901965737e-02
       coeff_l4(28,33)=  1.8792835996e-02
       coeff_l4(28,34)= -1.5643937886e-01
       coeff_l4(28,35)= -4.7908365726e-02
       coeff_l4(28,36)=  1.3717384636e-01
       coeff_l4(28,37)=  3.6493498087e-01
       coeff_l4(28,38)= -4.0424773097e-01
       coeff_l4(28,39)=  1.2596671283e-01
       coeff_l4(28,40)= -3.7988699973e-02
       coeff_l4(28,41)=  8.6901962757e-02
       coeff_l4(28,42)=  1.4095480740e-01
       coeff_l4(28,43)=  3.7342417240e-01
       coeff_l4(28,44)= -1.0862906277e-01
       coeff_l4(28,45)=  7.8495725989e-02
       coeff_l4(28,46)= -3.0235961080e-01
       coeff_l4(28,47)= -2.8988131881e-01
       coeff_l4(28,48)= -2.6050630212e-01
       coeff_l4(28,49)=  1.8249031901e-01
       coeff_l4(28,50)= -1.5943720937e-01
       coeff_l4(28,51)= -3.2244320959e-02
       coeff_l4(28,52)=  2.5329223275e-01
       coeff_l4(28,53)= -3.3629152924e-02
       coeff_l4(28,54)=  1.2472385168e-01
       coeff_l4(28,55)=  1.2126068026e-01
       coeff_l4(28,56)= -2.1032722294e-01
       coeff_l4(28,57)= -5.5243593454e-01
       coeff_l4(28,58)= -8.5718385875e-02
       coeff_l4(28,59)=  2.3928344250e-01
       coeff_l4(28,60)= -3.0341908336e-01
       coeff_l4(28,61)= -3.5388803482e-01
       coeff_l4(28,62)= -1.1877329648e-01
       coeff_l4(28,63)=  1.6120253503e-01
       coeff_l4(28,64)= -2.0194491744e-01
       coeff_l4(28,65)=  1.9764104486e-01
       coeff_l4(28,66)=  1.7969383299e-01
       coeff_l4(28,67)=  1.0226834565e-01
       coeff_l4(28,68)= -6.5164619684e-01
       coeff_l4(28,69)=  2.2875337303e-01
       coeff_l4(28,70)=  3.3042111993e-01
       coeff_l4(28,71)= -3.5938972235e-01
       coeff_l4(28,72)= -2.7512845397e-01
       coeff_l4(28,73)= -8.6703687906e-02
       coeff_l4(28,74)= -7.1815490723e-01
       coeff_l4(28,75)=  4.3152061105e-01
       coeff_l4(28,76)= -7.7838379145e-01
       coeff_l4(28,77)= -1.2149415165e-01
       coeff_l4(28,78)= -1.8088054657e-01
       coeff_l4(28,79)=  2.4901486933e-01
       coeff_l4(28,80)=  7.5022457168e-03
       coeff_l4(28,81)=  6.5961885452e-01
       coeff_l4(28,82)=  2.0919904113e-01
       coeff_l4(28,83)=  2.1231980622e-01
       coeff_l4(28,84)= -1.5549284220e-01
       coeff_l4(28,85)= -3.2751888037e-01
       coeff_l4(28,86)= -2.7641999722e-01
       coeff_l4(28,87)=  1.4506882429e-01
       coeff_l4(28,88)= -7.1666222811e-01
       coeff_l4(28,89)= -1.4669771492e-01
       coeff_l4(28,90)= -1.2605170906e-01
       coeff_l4(28,91)=  2.5444459915e-01
       coeff_l4(28,92)=  3.7632942200e-01
       coeff_l4(28,93)= -6.0008949041e-01
       coeff_l4(28,94)= -1.5226502419e+00
       coeff_l4(28,95)=  6.1766106635e-02
       coeff_l4(28,96)= -9.5786613226e-01
       coeff_l4(28,97)= -1.1125023365e+00
       coeff_l4(28,98)=  4.0432402492e-01
       coeff_l4(28,99)=  7.8352972865e-02
       coeff_l4(28,100)=  3.9403146505e-01
       coeff_l4(28,101)=  4.1262373328e-02
       coeff_l4(28,102)= -2.2250694036e-01
       coeff_l4(28,103)=  1.5935181081e-01
       coeff_l4(28,104)=  5.1184829324e-02
       coeff_l4(28,105)= -2.8469714522e-01
       coeff_l4(29,1)=  1.4359470606e+00
       coeff_l4(29,2)=  1.8911244869e+00
       coeff_l4(29,3)=  1.7914922237e+00
       coeff_l4(29,4)=  1.7199457884e+00
       coeff_l4(29,5)=  1.7522652149e+00
       coeff_l4(29,6)=  1.5167455673e+00
       coeff_l4(29,7)=  1.0975965261e+00
       coeff_l4(29,8)=  1.0366662741e+00
       coeff_l4(29,9)=  3.2635101676e-01
       coeff_l4(29,10)=  7.4908083677e-01
       coeff_l4(29,11)= -1.7442378402e-01
       coeff_l4(29,12)= -8.8336533308e-01
       coeff_l4(29,13)= -6.6161371768e-02
       coeff_l4(29,14)=  1.1701400280e+00
       coeff_l4(29,15)=  2.4130567908e-01
       coeff_l4(29,16)=  1.9692715257e-02
       coeff_l4(29,17)= -2.0823492110e-01
       coeff_l4(29,18)=  2.8466524556e-02
       coeff_l4(29,19)= -1.7929734290e-01
       coeff_l4(29,20)= -2.8888472915e-01
       coeff_l4(29,21)= -5.0642108917e-01
       coeff_l4(29,22)=  2.0019675791e-01
       coeff_l4(29,23)= -4.8913087696e-02
       coeff_l4(29,24)=  1.2860988081e-01
       coeff_l4(29,25)=  1.5285916626e-01
       coeff_l4(29,26)=  3.1022205949e-01
       coeff_l4(29,27)=  2.8552955389e-01
       coeff_l4(29,28)= -6.2653750181e-01
       coeff_l4(29,29)= -1.8302598596e-01
       coeff_l4(29,30)=  3.8613706827e-01
       coeff_l4(29,31)= -6.2132114172e-01
       coeff_l4(29,32)=  5.9147363901e-01
       coeff_l4(29,33)=  5.2481766790e-02
       coeff_l4(29,34)= -1.6140677035e-01
       coeff_l4(29,35)=  2.0085199177e-01
       coeff_l4(29,36)=  1.5379507840e-01
       coeff_l4(29,37)= -1.2774133682e-01
       coeff_l4(29,38)=  6.9146001339e-01
       coeff_l4(29,39)= -4.9554419518e-01
       coeff_l4(29,40)= -2.0611454546e-01
       coeff_l4(29,41)=  5.0696450472e-01
       coeff_l4(29,42)=  5.0563311577e-01
       coeff_l4(29,43)= -1.3595296443e-01
       coeff_l4(29,44)=  2.2692075372e-01
       coeff_l4(29,45)= -9.1092753410e-01
       coeff_l4(29,46)=  1.7451554537e-01
       coeff_l4(29,47)= -3.3678776026e-01
       coeff_l4(29,48)= -1.5267309546e-01
       coeff_l4(29,49)=  3.4296423197e-01
       coeff_l4(29,50)=  8.2215197384e-02
       coeff_l4(29,51)= -1.8542613834e-02
       coeff_l4(29,52)= -4.2169696093e-01
       coeff_l4(29,53)= -5.5559158325e-02
       coeff_l4(29,54)= -3.5143899918e-01
       coeff_l4(29,55)= -1.1868088692e-01
       coeff_l4(29,56)=  3.2947623730e-01
       coeff_l4(29,57)=  4.2563411593e-01
       coeff_l4(29,58)= -2.0725202560e-01
       coeff_l4(29,59)=  2.9554900527e-01
       coeff_l4(29,60)= -8.9779101312e-02
       coeff_l4(29,61)= -4.3225926161e-01
       coeff_l4(29,62)=  4.8157534003e-01
       coeff_l4(29,63)= -3.5016927123e-01
       coeff_l4(29,64)=  3.8696295023e-01
       coeff_l4(29,65)=  4.9257084727e-02
       coeff_l4(29,66)= -2.9332092404e-01
       coeff_l4(29,67)=  1.3062597811e-01
       coeff_l4(29,68)= -1.1775308847e-01
       coeff_l4(29,69)= -3.3043447137e-01
       coeff_l4(29,70)=  3.3750975132e-01
       coeff_l4(29,71)= -4.9813473225e-01
       coeff_l4(29,72)=  1.0020457506e+00
       coeff_l4(29,73)=  3.7540428340e-02
       coeff_l4(29,74)=  5.5088967085e-02
       coeff_l4(29,75)= -5.1950168610e-01
       coeff_l4(29,76)=  3.4636938572e-01
       coeff_l4(29,77)= -3.5652484745e-02
       coeff_l4(29,78)= -1.7839235067e-01
       coeff_l4(29,79)= -4.4555646181e-01
       coeff_l4(29,80)= -2.7580094337e-01
       coeff_l4(29,81)=  4.0678676963e-01
       coeff_l4(29,82)=  7.1236306429e-01
       coeff_l4(29,83)= -2.4805189669e-01
       coeff_l4(29,84)= -4.1188782454e-01
       coeff_l4(29,85)=  1.0083154440e+00
       coeff_l4(29,86)= -6.4827036858e-01
       coeff_l4(29,87)=  4.3450617790e-01
       coeff_l4(29,88)=  1.8738111854e-01
       coeff_l4(29,89)= -2.7746438980e-01
       coeff_l4(29,90)= -1.5110030770e-01
       coeff_l4(29,91)=  1.4390702546e-01
       coeff_l4(29,92)= -6.4044333994e-02
       coeff_l4(29,93)= -6.3605986536e-02
       coeff_l4(29,94)= -1.3688489795e-02
       coeff_l4(29,95)= -3.7614351511e-01
       coeff_l4(29,96)=  5.1943108439e-02
       coeff_l4(29,97)=  4.1552883387e-01
       coeff_l4(29,98)= -3.5907065868e-01
       coeff_l4(29,99)= -4.1436541080e-01
       coeff_l4(29,100)=  2.7124118805e-01
       coeff_l4(29,101)= -6.6201579571e-01
       coeff_l4(29,102)= -4.1517171264e-01
       coeff_l4(29,103)=  1.8479776382e-01
       coeff_l4(29,104)=  7.0933520794e-01
       coeff_l4(29,105)= -2.6473492384e-01
       coeff_l4(30,1)= -4.5778149366e-01
       coeff_l4(30,2)= -7.6546758413e-02
       coeff_l4(30,3)=  1.1101611704e-01
       coeff_l4(30,4)=  3.7623107433e-02
       coeff_l4(30,5)= -7.7008329332e-02
       coeff_l4(30,6)= -5.0124782324e-01
       coeff_l4(30,7)= -5.4241347313e-01
       coeff_l4(30,8)= -7.6543927193e-01
       coeff_l4(30,9)= -1.8066976070e+00
       coeff_l4(30,10)= -2.2004518509e+00
       coeff_l4(30,11)= -9.5026159286e-01
       coeff_l4(30,12)= -6.4353346825e-01
       coeff_l4(30,13)= -1.3825486898e+00
       coeff_l4(30,14)= -2.1739668846e+00
       coeff_l4(30,15)=  2.7752605081e-01
       coeff_l4(30,16)= -1.9766120613e-01
       coeff_l4(30,17)= -9.2702940106e-02
       coeff_l4(30,18)= -1.5803773701e-01
       coeff_l4(30,19)=  3.2330060005e-01
       coeff_l4(30,20)=  6.2525159121e-01
       coeff_l4(30,21)= -8.0532056093e-01
       coeff_l4(30,22)=  1.4879231155e-01
       coeff_l4(30,23)= -3.7343731523e-01
       coeff_l4(30,24)=  4.2055276036e-01
       coeff_l4(30,25)= -1.0236736387e-01
       coeff_l4(30,26)=  3.2309746742e-01
       coeff_l4(30,27)=  3.2144024968e-01
       coeff_l4(30,28)= -3.7947994471e-01
       coeff_l4(30,29)= -5.0050277263e-02
       coeff_l4(30,30)=  3.3950677514e-01
       coeff_l4(30,31)= -1.6847209632e-01
       coeff_l4(30,32)= -2.5473368168e-01
       coeff_l4(30,33)= -7.3285073042e-02
       coeff_l4(30,34)= -1.6806250811e-01
       coeff_l4(30,35)=  6.1773523688e-02
       coeff_l4(30,36)= -2.9537320137e-02
       coeff_l4(30,37)= -2.5986140966e-01
       coeff_l4(30,38)= -1.6068612039e-01
       coeff_l4(30,39)= -1.7009387910e-01
       coeff_l4(30,40)= -1.5966974199e-01
       coeff_l4(30,41)=  3.2423582673e-01
       coeff_l4(30,42)=  1.5172046423e-01
       coeff_l4(30,43)= -6.0695856810e-02
       coeff_l4(30,44)= -2.9532435536e-01
       coeff_l4(30,45)= -3.2564142346e-01
       coeff_l4(30,46)= -9.1324532032e-01
       coeff_l4(30,47)= -5.2222095430e-02
       coeff_l4(30,48)= -1.8997341394e-01
       coeff_l4(30,49)= -1.7542849481e-01
       coeff_l4(30,50)= -7.7519440651e-01
       coeff_l4(30,51)=  3.0736276507e-01
       coeff_l4(30,52)=  2.1267640591e-01
       coeff_l4(30,53)=  3.9916780591e-01
       coeff_l4(30,54)=  2.5057503581e-01
       coeff_l4(30,55)= -3.4225523472e-01
       coeff_l4(30,56)=  2.7785739303e-01
       coeff_l4(30,57)=  1.4928959310e-02
       coeff_l4(30,58)= -3.2246088982e-01
       coeff_l4(30,59)= -1.9336010516e-01
       coeff_l4(30,60)= -5.3093403578e-01
       coeff_l4(30,61)= -6.2876677513e-01
       coeff_l4(30,62)= -4.2026090622e-01
       coeff_l4(30,63)=  2.6640754938e-01
       coeff_l4(30,64)=  1.2096216530e-01
       coeff_l4(30,65)= -1.8386860192e-01
       coeff_l4(30,66)= -7.6652146876e-02
       coeff_l4(30,67)= -5.6456619501e-01
       coeff_l4(30,68)=  3.5913676023e-01
       coeff_l4(30,69)=  1.1905185878e-01
       coeff_l4(30,70)= -1.2931878865e-01
       coeff_l4(30,71)=  1.5478250384e-01
       coeff_l4(30,72)= -4.5880046487e-01
       coeff_l4(30,73)= -2.2667264938e-01
       coeff_l4(30,74)=  1.2647274137e-01
       coeff_l4(30,75)= -2.2131809592e-01
       coeff_l4(30,76)=  2.1316137910e-01
       coeff_l4(30,77)=  3.6296492815e-01
       coeff_l4(30,78)=  1.4292824268e-01
       coeff_l4(30,79)=  3.4541955590e-01
       coeff_l4(30,80)= -2.1708099172e-02
       coeff_l4(30,81)= -2.4582304060e-02
       coeff_l4(30,82)= -2.5906586647e-01
       coeff_l4(30,83)= -7.5073532760e-02
       coeff_l4(30,84)=  3.9550983906e-01
       coeff_l4(30,85)=  1.4897701144e-01
       coeff_l4(30,86)=  2.8780240566e-02
       coeff_l4(30,87)= -1.3616639376e-01
       coeff_l4(30,88)=  1.9395095110e-01
       coeff_l4(30,89)=  3.1654399633e-01
       coeff_l4(30,90)= -1.5451769531e-01
       coeff_l4(30,91)=  1.7386919260e-01
       coeff_l4(30,92)= -1.8509232998e-01
       coeff_l4(30,93)= -2.2101241350e-01
       coeff_l4(30,94)=  1.1120362282e+00
       coeff_l4(30,95)= -7.3302197456e-01
       coeff_l4(30,96)=  8.8645778596e-02
       coeff_l4(30,97)=  7.7280199528e-01
       coeff_l4(30,98)= -4.3938088417e-01
       coeff_l4(30,99)=  1.1108806357e-03
       coeff_l4(30,100)=  6.6399298608e-02
       coeff_l4(30,101)= -4.4028130174e-01
       coeff_l4(30,102)= -7.0085960627e-01
       coeff_l4(30,103)= -1.4265565574e-01
       coeff_l4(30,104)=  2.4445680901e-02
       coeff_l4(30,105)= -7.0575410128e-01
       coeff_l4(31,1)= -7.4071604759e-03
       coeff_l4(31,2)=  1.4236938953e-01
       coeff_l4(31,3)= -3.0076199770e-01
       coeff_l4(31,4)= -2.9634609818e-01
       coeff_l4(31,5)= -3.6484116316e-01
       coeff_l4(31,6)= -2.3950012028e-01
       coeff_l4(31,7)= -6.9120931625e-01
       coeff_l4(31,8)= -4.1120761633e-01
       coeff_l4(31,9)= -2.5085434318e-01
       coeff_l4(31,10)= -6.7458367348e-01
       coeff_l4(31,11)=  7.6862347126e-01
       coeff_l4(31,12)=  2.7720612288e-01
       coeff_l4(31,13)= -3.1263913959e-02
       coeff_l4(31,14)=  1.5697275102e-01
       coeff_l4(31,15)=  6.2781527638e-02
       coeff_l4(31,16)=  2.2692497075e-01
       coeff_l4(31,17)=  7.2696073912e-03
       coeff_l4(31,18)= -1.8615756929e-01
       coeff_l4(31,19)=  8.1287540495e-02
       coeff_l4(31,20)=  2.1573266014e-02
       coeff_l4(31,21)=  6.4672231674e-02
       coeff_l4(31,22)= -1.0678567737e-01
       coeff_l4(31,23)=  3.3821231127e-01
       coeff_l4(31,24)= -1.0523840785e-01
       coeff_l4(31,25)=  7.8986965120e-02
       coeff_l4(31,26)= -1.4154105447e-02
       coeff_l4(31,27)= -3.1247103214e-01
       coeff_l4(31,28)=  2.0934985951e-02
       coeff_l4(31,29)= -2.6032444835e-01
       coeff_l4(31,30)=  4.7667194158e-02
       coeff_l4(31,31)=  6.6260196269e-02
       coeff_l4(31,32)= -3.5699343681e-01
       coeff_l4(31,33)=  2.6091624051e-02
       coeff_l4(31,34)=  1.0188826174e-01
       coeff_l4(31,35)=  1.0643280298e-01
       coeff_l4(31,36)= -1.2817102671e-01
       coeff_l4(31,37)=  1.4573415741e-02
       coeff_l4(31,38)=  2.0300719142e-01
       coeff_l4(31,39)=  6.1429303885e-01
       coeff_l4(31,40)= -1.7810289562e-01
       coeff_l4(31,41)=  2.3915448785e-01
       coeff_l4(31,42)=  5.5790018290e-02
       coeff_l4(31,43)= -1.9889031351e-01
       coeff_l4(31,44)= -6.2171872705e-02
       coeff_l4(31,45)= -2.3917210102e-01
       coeff_l4(31,46)=  2.4764135480e-02
       coeff_l4(31,47)= -9.9252397195e-03
       coeff_l4(31,48)= -1.6069805622e-01
       coeff_l4(31,49)=  1.5490206890e-02
       coeff_l4(31,50)=  1.8844481092e-03
       coeff_l4(31,51)= -1.2198180705e-01
       coeff_l4(31,52)= -1.8029353023e-01
       coeff_l4(31,53)= -6.7888014019e-02
       coeff_l4(31,54)= -9.0426914394e-02
       coeff_l4(31,55)= -3.9354342222e-01
       coeff_l4(31,56)=  4.1558131576e-01
       coeff_l4(31,57)= -9.5055669546e-02
       coeff_l4(31,58)=  5.6716311723e-02
       coeff_l4(31,59)= -5.5355370045e-01
       coeff_l4(31,60)=  2.1251302958e-01
       coeff_l4(31,61)=  3.5627025366e-01
       coeff_l4(31,62)=  1.1991788447e-01
       coeff_l4(31,63)= -2.2504851222e-01
       coeff_l4(31,64)=  4.2167514563e-02
       coeff_l4(31,65)= -8.9419424534e-02
       coeff_l4(31,66)=  2.2790784016e-02
       coeff_l4(31,67)= -1.7968222499e-02
       coeff_l4(31,68)=  6.9441080093e-02
       coeff_l4(31,69)= -1.5713094175e-01
       coeff_l4(31,70)= -1.9077439606e-01
       coeff_l4(31,71)=  2.4724067748e-01
       coeff_l4(31,72)=  2.1098522842e-01
       coeff_l4(31,73)= -3.2486996055e-01
       coeff_l4(31,74)= -3.9182400703e-01
       coeff_l4(31,75)=  9.5436006784e-02
       coeff_l4(31,76)= -2.9168820009e-02
       coeff_l4(31,77)= -2.4826727808e-01
       coeff_l4(31,78)=  4.3763461709e-01
       coeff_l4(31,79)=  1.6547414660e-01
       coeff_l4(31,80)= -2.8762209415e-01
       coeff_l4(31,81)=  3.1431179494e-02
       coeff_l4(31,82)= -9.2999681830e-02
       coeff_l4(31,83)= -2.4735225737e-01
       coeff_l4(31,84)= -1.1804855615e-01
       coeff_l4(31,85)= -1.6348260641e-01
       coeff_l4(31,86)= -2.0919037983e-02
       coeff_l4(31,87)=  9.5486141741e-02
       coeff_l4(31,88)=  2.8162521124e-01
       coeff_l4(31,89)= -3.2045948505e-01
       coeff_l4(31,90)= -2.0931532979e-01
       coeff_l4(31,91)= -3.1603157520e-01
       coeff_l4(31,92)=  3.6809366941e-01
       coeff_l4(31,93)= -2.3168580607e-02
       coeff_l4(31,94)=  5.2539074421e-01
       coeff_l4(31,95)= -4.9227261543e-01
       coeff_l4(31,96)=  2.8975895047e-01
       coeff_l4(31,97)=  2.0102268085e-02
       coeff_l4(31,98)= -1.5411482751e-01
       coeff_l4(31,99)= -2.9775993899e-03
       coeff_l4(31,100)= -5.2114468813e-01
       coeff_l4(31,101)= -4.7803786397e-01
       coeff_l4(31,102)= -2.3288281262e-01
       coeff_l4(31,103)= -4.2672097683e-01
       coeff_l4(31,104)=  1.4774148166e-01
       coeff_l4(31,105)=  5.5977500975e-02
       coeff_l4(32,1)= -4.5452705026e-01
       coeff_l4(32,2)= -2.5348389149e-01
       coeff_l4(32,3)= -4.8614826798e-01
       coeff_l4(32,4)= -2.7013444901e-01
       coeff_l4(32,5)= -2.6038286090e-01
       coeff_l4(32,6)= -3.3874401450e-01
       coeff_l4(32,7)= -4.4799509645e-01
       coeff_l4(32,8)= -2.5231063366e-01
       coeff_l4(32,9)= -8.9851364493e-02
       coeff_l4(32,10)= -2.5008741766e-02
       coeff_l4(32,11)=  3.5154196620e-01
       coeff_l4(32,12)=  1.7366677523e-01
       coeff_l4(32,13)=  2.1893565357e-01
       coeff_l4(32,14)= -2.6021229569e-03
       coeff_l4(32,15)=  1.2509034947e-02
       coeff_l4(32,16)=  1.3102514669e-02
       coeff_l4(32,17)= -1.5022893250e-01
       coeff_l4(32,18)= -1.8553781509e-01
       coeff_l4(32,19)=  2.9983960092e-02
       coeff_l4(32,20)= -8.2258962095e-02
       coeff_l4(32,21)=  4.1642361321e-03
       coeff_l4(32,22)=  1.5986867249e-01
       coeff_l4(32,23)=  2.5583740324e-02
       coeff_l4(32,24)= -7.6298275962e-03
       coeff_l4(32,25)= -1.9379083812e-01
       coeff_l4(32,26)=  1.5667380393e-01
       coeff_l4(32,27)=  5.3951740265e-02
       coeff_l4(32,28)=  7.2625026107e-02
       coeff_l4(32,29)=  7.1970805526e-02
       coeff_l4(32,30)=  6.6298834980e-02
       coeff_l4(32,31)= -3.0804134905e-02
       coeff_l4(32,32)= -1.0194977373e-01
       coeff_l4(32,33)=  6.0176659375e-02
       coeff_l4(32,34)= -2.8406063095e-02
       coeff_l4(32,35)=  3.4720625728e-02
       coeff_l4(32,36)=  5.1509030163e-02
       coeff_l4(32,37)=  1.1131559312e-01
       coeff_l4(32,38)=  1.5504246950e-01
       coeff_l4(32,39)= -5.5664253887e-04
       coeff_l4(32,40)= -9.6983574331e-02
       coeff_l4(32,41)= -3.7432756275e-02
       coeff_l4(32,42)=  1.8120993674e-01
       coeff_l4(32,43)= -3.2482031733e-02
       coeff_l4(32,44)= -1.4549373090e-01
       coeff_l4(32,45)= -2.7662530541e-02
       coeff_l4(32,46)=  6.6897198558e-02
       coeff_l4(32,47)= -2.0965988934e-01
       coeff_l4(32,48)=  2.1417881548e-01
       coeff_l4(32,49)= -7.1960993111e-02
       coeff_l4(32,50)= -1.0989534855e-01
       coeff_l4(32,51)=  1.3178990781e-01
       coeff_l4(32,52)=  1.4719240367e-01
       coeff_l4(32,53)=  1.5585230291e-01
       coeff_l4(32,54)=  4.6281319112e-02
       coeff_l4(32,55)=  1.2835465372e-01
       coeff_l4(32,56)=  3.1287740916e-02
       coeff_l4(32,57)= -1.3248664141e-01
       coeff_l4(32,58)= -8.2642734051e-02
       coeff_l4(32,59)=  1.0210303217e-01
       coeff_l4(32,60)= -9.9227391183e-02
       coeff_l4(32,61)= -2.5818517432e-02
       coeff_l4(32,62)= -1.3543160260e-01
       coeff_l4(32,63)= -9.6540987492e-02
       coeff_l4(32,64)=  1.1476270854e-01
       coeff_l4(32,65)= -2.6829589158e-02
       coeff_l4(32,66)=  2.8325865045e-02
       coeff_l4(32,67)=  1.3245490193e-01
       coeff_l4(32,68)=  1.2159408629e-01
       coeff_l4(32,69)= -9.3132093549e-02
       coeff_l4(32,70)=  4.4888079166e-02
       coeff_l4(32,71)=  1.8137738109e-02
       coeff_l4(32,72)= -4.6070072800e-02
       coeff_l4(32,73)= -1.8609847128e-01
       coeff_l4(32,74)= -5.2277289331e-02
       coeff_l4(32,75)= -7.9539939761e-02
       coeff_l4(32,76)=  7.7065132558e-02
       coeff_l4(32,77)= -2.5681862235e-01
       coeff_l4(32,78)= -5.9754930437e-02
       coeff_l4(32,79)= -5.3678888828e-02
       coeff_l4(32,80)= -9.3524895608e-02
       coeff_l4(32,81)=  6.8596705794e-02
       coeff_l4(32,82)=  6.3901752234e-02
       coeff_l4(32,83)= -2.5073677301e-02
       coeff_l4(32,84)=  1.2375375256e-02
       coeff_l4(32,85)=  3.3404771239e-02
       coeff_l4(32,86)= -1.8610644341e-01
       coeff_l4(32,87)= -4.9604303058e-05
       coeff_l4(32,88)=  2.9237344861e-02
       coeff_l4(32,89)= -7.8389376402e-02
       coeff_l4(32,90)=  2.6045853272e-02
       coeff_l4(32,91)=  8.8936604559e-02
       coeff_l4(32,92)= -7.5921919197e-03
       coeff_l4(32,93)=  1.9415523112e-01
       coeff_l4(32,94)=  3.2389184833e-01
       coeff_l4(32,95)= -3.3616419882e-02
       coeff_l4(32,96)=  5.3465377539e-02
       coeff_l4(32,97)=  3.4482102841e-02
       coeff_l4(32,98)=  1.3332891464e-01
       coeff_l4(32,99)= -3.1517598778e-02
       coeff_l4(32,100)=  2.9691210017e-02
       coeff_l4(32,101)= -5.5212557316e-02
       coeff_l4(32,102)= -4.5780975372e-02
       coeff_l4(32,103)=  1.0253631510e-02
       coeff_l4(32,104)=  1.3294355571e-01
       coeff_l4(32,105)= -2.8081241250e-01
       coeff_l4(33,1)=  5.3484812379e-02
       coeff_l4(33,2)=  2.2627216578e-01
       coeff_l4(33,3)=  1.9967086613e-01
       coeff_l4(33,4)=  1.5385881066e-01
       coeff_l4(33,5)=  9.5884472132e-02
       coeff_l4(33,6)=  2.2337263823e-01
       coeff_l4(33,7)=  9.1469779611e-02
       coeff_l4(33,8)=  1.1163169891e-01
       coeff_l4(33,9)= -2.6346999407e-01
       coeff_l4(33,10)= -1.5494072437e-01
       coeff_l4(33,11)= -3.2057204843e-01
       coeff_l4(33,12)= -1.2205963582e-01
       coeff_l4(33,13)= -3.0417627096e-01
       coeff_l4(33,14)= -3.0344930291e-01
       coeff_l4(33,15)= -1.0170571506e-01
       coeff_l4(33,16)= -2.6156315207e-01
       coeff_l4(33,17)= -1.3305145502e-01
       coeff_l4(33,18)= -1.2710250914e-01
       coeff_l4(33,19)= -1.6526636481e-01
       coeff_l4(33,20)= -1.2964913249e-01
       coeff_l4(33,21)=  9.3284465373e-02
       coeff_l4(33,22)= -9.0638078749e-02
       coeff_l4(33,23)= -7.2028301656e-02
       coeff_l4(33,24)=  3.0004639179e-02
       coeff_l4(33,25)=  3.1394939870e-02
       coeff_l4(33,26)=  5.4802853614e-02
       coeff_l4(33,27)=  1.2354658917e-02
       coeff_l4(33,28)= -1.2855961919e-01
       coeff_l4(33,29)= -1.6486626863e-01
       coeff_l4(33,30)=  2.0835326612e-01
       coeff_l4(33,31)= -1.0584447533e-02
       coeff_l4(33,32)= -3.7400271744e-02
       coeff_l4(33,33)=  1.7071230710e-01
       coeff_l4(33,34)= -5.0623141229e-02
       coeff_l4(33,35)=  1.7106412351e-01
       coeff_l4(33,36)= -5.5378094316e-02
       coeff_l4(33,37)= -5.2795320749e-02
       coeff_l4(33,38)=  3.5906761885e-02
       coeff_l4(33,39)= -1.2038166076e-01
       coeff_l4(33,40)= -8.8573254645e-02
       coeff_l4(33,41)=  6.9813609123e-02
       coeff_l4(33,42)= -4.9032822251e-02
       coeff_l4(33,43)=  1.7176398635e-01
       coeff_l4(33,44)=  1.0446188599e-01
       coeff_l4(33,45)= -4.2616248131e-02
       coeff_l4(33,46)= -6.7871272564e-02
       coeff_l4(33,47)= -1.3815641403e-01
       coeff_l4(33,48)=  5.8344334364e-02
       coeff_l4(33,49)= -4.0179599077e-02
       coeff_l4(33,50)= -1.3812458515e-01
       coeff_l4(33,51)=  1.2658791244e-01
       coeff_l4(33,52)= -1.3587963581e-01
       coeff_l4(33,53)=  2.0685367286e-01
       coeff_l4(33,54)=  3.1028641388e-02
       coeff_l4(33,55)=  1.0766706616e-01
       coeff_l4(33,56)=  9.7598969936e-02
       coeff_l4(33,57)=  9.9926725030e-02
       coeff_l4(33,58)=  1.4634536207e-01
       coeff_l4(33,59)= -5.1473930478e-02
       coeff_l4(33,60)= -8.1594754010e-03
       coeff_l4(33,61)= -1.7929705977e-01
       coeff_l4(33,62)= -8.0575942993e-02
       coeff_l4(33,63)= -9.0780273080e-02
       coeff_l4(33,64)= -5.1566180773e-03
       coeff_l4(33,65)=  1.3594412245e-02
       coeff_l4(33,66)= -1.7385925353e-01
       coeff_l4(33,67)= -6.7893043160e-02
       coeff_l4(33,68)=  4.2431965470e-02
       coeff_l4(33,69)=  7.6839886606e-02
       coeff_l4(33,70)=  7.4856087565e-02
       coeff_l4(33,71)=  2.0702121779e-02
       coeff_l4(33,72)= -8.6454644799e-02
       coeff_l4(33,73)= -6.4258269966e-02
       coeff_l4(33,74)=  1.3064833358e-02
       coeff_l4(33,75)=  1.1706959922e-03
       coeff_l4(33,76)= -7.3947757483e-03
       coeff_l4(33,77)=  5.2741322666e-02
       coeff_l4(33,78)= -7.4446313083e-02
       coeff_l4(33,79)= -2.4498556554e-01
       coeff_l4(33,80)=  1.8549276516e-02
       coeff_l4(33,81)= -1.7075957358e-01
       coeff_l4(33,82)= -3.4253962338e-02
       coeff_l4(33,83)= -5.1806934178e-02
       coeff_l4(33,84)= -5.2558360621e-03
       coeff_l4(33,85)=  3.9056442678e-02
       coeff_l4(33,86)= -1.3883554935e-01
       coeff_l4(33,87)=  3.6362987012e-02
       coeff_l4(33,88)=  1.4708489180e-01
       coeff_l4(33,89)=  1.0635506362e-01
       coeff_l4(33,90)= -1.0332762450e-01
       coeff_l4(33,91)=  8.8951386511e-02
       coeff_l4(33,92)= -1.9946038723e-01
       coeff_l4(33,93)=  4.7305382788e-02
       coeff_l4(33,94)=  5.2732143551e-02
       coeff_l4(33,95)= -1.1083194613e-01
       coeff_l4(33,96)=  1.6614931822e-01
       coeff_l4(33,97)=  1.3119071722e-01
       coeff_l4(33,98)=  1.4083223045e-01
       coeff_l4(33,99)=  1.0863860697e-01
       coeff_l4(33,100)= -1.4961491525e-01
       coeff_l4(33,101)= -2.5210892782e-02
       coeff_l4(33,102)= -2.0246368647e-01
       coeff_l4(33,103)=  1.2051752955e-01
       coeff_l4(33,104)=  5.6067403406e-02
       coeff_l4(33,105)= -1.8238326907e-01
       coeff_l4(34,1)= -3.8092106581e-01
       coeff_l4(34,2)= -1.0565607548e+00
       coeff_l4(34,3)= -7.6849007607e-01
       coeff_l4(34,4)= -8.1378650665e-01
       coeff_l4(34,5)= -1.0027042627e+00
       coeff_l4(34,6)= -9.8636853695e-01
       coeff_l4(34,7)= -8.6112666130e-01
       coeff_l4(34,8)= -8.7412357330e-01
       coeff_l4(34,9)=  7.6377414167e-02
       coeff_l4(34,10)= -2.5506147742e-01
       coeff_l4(34,11)= -1.7296822369e-01
       coeff_l4(34,12)= -1.0336597264e-01
       coeff_l4(34,13)= -7.5863506645e-03
       coeff_l4(34,14)= -2.1354158223e-01
       coeff_l4(34,15)= -2.4405334890e-01
       coeff_l4(34,16)= -1.4062786475e-02
       coeff_l4(34,17)= -3.5761568695e-02
       coeff_l4(34,18)=  6.8551190197e-02
       coeff_l4(34,19)= -2.4304042757e-01
       coeff_l4(34,20)= -2.6940962300e-02
       coeff_l4(34,21)=  2.0097951591e-01
       coeff_l4(34,22)=  1.9980265200e-01
       coeff_l4(34,23)= -2.5439482927e-01
       coeff_l4(34,24)=  3.6701041460e-01
       coeff_l4(34,25)=  1.1953667551e-01
       coeff_l4(34,26)=  1.9276086241e-02
       coeff_l4(34,27)= -3.1981521845e-01
       coeff_l4(34,28)=  2.3769838735e-02
       coeff_l4(34,29)=  9.2231601477e-02
       coeff_l4(34,30)= -6.2211763114e-02
       coeff_l4(34,31)=  1.5707422793e-01
       coeff_l4(34,32)= -1.8839496374e-01
       coeff_l4(34,33)= -6.7509530345e-04
       coeff_l4(34,34)= -4.3949443102e-01
       coeff_l4(34,35)= -1.1152045429e-01
       coeff_l4(34,36)=  1.5810781717e-01
       coeff_l4(34,37)=  7.4445814826e-03
       coeff_l4(34,38)= -2.8458511829e-01
       coeff_l4(34,39)=  2.0569625497e-01
       coeff_l4(34,40)= -1.1791793257e-01
       coeff_l4(34,41)=  2.5618475676e-01
       coeff_l4(34,42)=  6.8409256637e-02
       coeff_l4(34,43)= -2.4555213749e-02
       coeff_l4(34,44)=  2.9788941145e-01
       coeff_l4(34,45)= -1.6545413435e-01
       coeff_l4(34,46)= -2.1195916831e-01
       coeff_l4(34,47)=  1.7279849946e-01
       coeff_l4(34,48)= -6.6765020601e-03
       coeff_l4(34,49)= -2.1483173594e-02
       coeff_l4(34,50)=  2.6307213306e-01
       coeff_l4(34,51)=  9.7382143140e-02
       coeff_l4(34,52)=  2.9109221697e-01
       coeff_l4(34,53)= -1.2334159017e-01
       coeff_l4(34,54)= -9.8851015791e-03
       coeff_l4(34,55)=  1.1782437563e-01
       coeff_l4(34,56)= -1.7139171250e-03
       coeff_l4(34,57)= -1.7527928948e-01
       coeff_l4(34,58)= -1.8685953319e-01
       coeff_l4(34,59)=  2.6203626394e-01
       coeff_l4(34,60)= -1.2542544305e-01
       coeff_l4(34,61)=  1.4658352733e-01
       coeff_l4(34,62)=  2.6730126143e-01
       coeff_l4(34,63)=  4.6438261867e-01
       coeff_l4(34,64)=  2.1829059720e-01
       coeff_l4(34,65)=  9.6870541573e-02
       coeff_l4(34,66)=  2.0987853408e-01
       coeff_l4(34,67)=  3.7780448794e-02
       coeff_l4(34,68)=  8.5085079074e-02
       coeff_l4(34,69)= -2.3544874787e-01
       coeff_l4(34,70)=  1.5381315351e-01
       coeff_l4(34,71)=  4.2451191694e-02
       coeff_l4(34,72)= -3.1222862005e-01
       coeff_l4(34,73)= -4.6853828430e-01
       coeff_l4(34,74)=  3.9714440703e-01
       coeff_l4(34,75)= -9.3946538866e-02
       coeff_l4(34,76)= -3.8254368305e-01
       coeff_l4(34,77)=  6.6927425563e-02
       coeff_l4(34,78)= -8.4348186851e-02
       coeff_l4(34,79)= -1.4380259812e-01
       coeff_l4(34,80)= -3.5558193922e-02
       coeff_l4(34,81)=  2.2047223151e-01
       coeff_l4(34,82)= -2.2714262828e-02
       coeff_l4(34,83)= -4.3839791417e-01
       coeff_l4(34,84)=  1.9444127381e-01
       coeff_l4(34,85)=  4.1991329193e-01
       coeff_l4(34,86)= -2.1189591289e-01
       coeff_l4(34,87)=  2.9974663258e-01
       coeff_l4(34,88)= -2.0993740857e-01
       coeff_l4(34,89)=  3.3788645267e-01
       coeff_l4(34,90)= -4.5687928796e-02
       coeff_l4(34,91)= -6.3899919391e-02
       coeff_l4(34,92)=  2.4780014157e-01
       coeff_l4(34,93)= -1.8532356620e-01
       coeff_l4(34,94)=  3.9703956246e-01
       coeff_l4(34,95)= -8.4832832217e-02
       coeff_l4(34,96)= -3.2625761628e-01
       coeff_l4(34,97)=  2.3698951304e-01
       coeff_l4(34,98)= -3.1279936433e-02
       coeff_l4(34,99)= -9.8307840526e-02
       coeff_l4(34,100)= -2.8993651271e-01
       coeff_l4(34,101)=  3.3920899034e-01
       coeff_l4(34,102)=  9.4979152083e-02
       coeff_l4(34,103)=  1.7597129941e-01
       coeff_l4(34,104)=  1.6454817355e-01
       coeff_l4(34,105)= -2.2348791361e-01
       coeff_l4(35,1)= -5.4639242589e-03
       coeff_l4(35,2)=  5.5667944252e-02
       coeff_l4(35,3)=  1.6243998706e-01
       coeff_l4(35,4)=  4.9368899316e-02
       coeff_l4(35,5)=  3.5961676389e-02
       coeff_l4(35,6)=  1.0106006265e-01
       coeff_l4(35,7)=  3.6062918603e-02
       coeff_l4(35,8)= -1.3958744705e-01
       coeff_l4(35,9)= -1.1897895336e+00
       coeff_l4(35,10)= -3.9286011457e-01
       coeff_l4(35,11)= -1.7915499210e-01
       coeff_l4(35,12)= -2.9427525401e-01
       coeff_l4(35,13)= -8.2972896099e-01
       coeff_l4(35,14)= -5.3568625450e-01
       coeff_l4(35,15)= -6.7432701588e-02
       coeff_l4(35,16)=  1.0782751441e-01
       coeff_l4(35,17)=  8.3005912602e-02
       coeff_l4(35,18)=  2.7761127800e-02
       coeff_l4(35,19)=  1.1543997377e-01
       coeff_l4(35,20)= -9.2226110399e-02
       coeff_l4(35,21)=  2.2394161671e-02
       coeff_l4(35,22)= -1.4625999331e-01
       coeff_l4(35,23)=  2.5000387430e-01
       coeff_l4(35,24)=  1.0382666439e-01
       coeff_l4(35,25)=  1.2213017046e-01
       coeff_l4(35,26)=  2.0721620240e-04
       coeff_l4(35,27)= -5.5081687868e-02
       coeff_l4(35,28)=  1.1204552650e-01
       coeff_l4(35,29)=  1.7016861588e-02
       coeff_l4(35,30)= -6.0121901333e-03
       coeff_l4(35,31)= -7.6546087861e-02
       coeff_l4(35,32)= -1.0877285153e-01
       coeff_l4(35,33)=  1.4561804943e-02
       coeff_l4(35,34)=  5.2366599441e-02
       coeff_l4(35,35)=  3.6893185973e-01
       coeff_l4(35,36)=  2.9287940264e-01
       coeff_l4(35,37)=  2.1685385704e-01
       coeff_l4(35,38)= -1.9044569135e-01
       coeff_l4(35,39)=  4.0381804109e-01
       coeff_l4(35,40)=  4.1111268103e-02
       coeff_l4(35,41)= -9.3279264867e-02
       coeff_l4(35,42)= -4.6576716006e-02
       coeff_l4(35,43)= -1.9682820886e-02
       coeff_l4(35,44)= -3.2689356804e-01
       coeff_l4(35,45)= -2.1412478387e-01
       coeff_l4(35,46)= -7.0966653526e-02
       coeff_l4(35,47)=  4.1475299746e-02
       coeff_l4(35,48)= -1.0388379544e-01
       coeff_l4(35,49)=  1.4929693937e-01
       coeff_l4(35,50)= -6.9487437606e-02
       coeff_l4(35,51)=  1.3154010475e-01
       coeff_l4(35,52)= -4.9032278359e-02
       coeff_l4(35,53)=  9.8332688212e-02
       coeff_l4(35,54)=  3.5326695442e-01
       coeff_l4(35,55)=  3.1924743205e-02
       coeff_l4(35,56)= -8.1527248025e-02
       coeff_l4(35,57)= -3.0466869473e-01
       coeff_l4(35,58)= -2.6835508645e-02
       coeff_l4(35,59)= -7.4012905359e-02
       coeff_l4(35,60)= -1.3466921449e-01
       coeff_l4(35,61)=  1.5723271668e-01
       coeff_l4(35,62)=  6.5303772688e-02
       coeff_l4(35,63)= -6.8750426173e-02
       coeff_l4(35,64)=  1.9297455251e-01
       coeff_l4(35,65)=  3.8907371345e-04
       coeff_l4(35,66)=  1.1164996028e-01
       coeff_l4(35,67)= -8.2317747176e-02
       coeff_l4(35,68)= -2.0952054858e-01
       coeff_l4(35,69)= -2.5227841735e-01
       coeff_l4(35,70)= -2.1368555725e-01
       coeff_l4(35,71)= -6.1675034463e-02
       coeff_l4(35,72)= -1.1592438072e-01
       coeff_l4(35,73)=  2.8034478426e-01
       coeff_l4(35,74)=  7.6070927083e-02
       coeff_l4(35,75)= -3.2098114491e-01
       coeff_l4(35,76)=  7.0247359574e-02
       coeff_l4(35,77)=  5.7198449969e-02
       coeff_l4(35,78)= -8.0943681300e-02
       coeff_l4(35,79)= -1.5625028312e-01
       coeff_l4(35,80)=  2.2042661905e-02
       coeff_l4(35,81)=  1.5616376698e-01
       coeff_l4(35,82)=  1.6671809554e-01
       coeff_l4(35,83)= -8.0376088619e-02
       coeff_l4(35,84)=  1.9594399631e-01
       coeff_l4(35,85)=  4.9103435129e-02
       coeff_l4(35,86)= -3.5210620612e-02
       coeff_l4(35,87)=  3.1357595325e-01
       coeff_l4(35,88)= -2.5943756104e-01
       coeff_l4(35,89)=  3.2582014799e-02
       coeff_l4(35,90)= -2.8346332908e-01
       coeff_l4(35,91)=  9.3422159553e-02
       coeff_l4(35,92)= -4.1928064823e-01
       coeff_l4(35,93)=  2.5117745996e-01
       coeff_l4(35,94)= -7.0225340128e-01
       coeff_l4(35,95)=  3.2192781568e-01
       coeff_l4(35,96)= -2.8088769317e-01
       coeff_l4(35,97)=  6.8332299590e-02
       coeff_l4(35,98)= -1.3608509302e-01
       coeff_l4(35,99)=  1.3536110520e-01
       coeff_l4(35,100)=  2.1601080894e-02
       coeff_l4(35,101)= -8.6791822687e-03
       coeff_l4(35,102)= -1.3772980869e-01
       coeff_l4(35,103)=  1.2320739776e-01
       coeff_l4(35,104)=  2.6647397876e-01
       coeff_l4(35,105)=  3.4823378921e-01
       coeff_l4(36,1)= -8.3003556728e-01
       coeff_l4(36,2)= -8.1459385157e-01
       coeff_l4(36,3)= -8.8958281279e-01
       coeff_l4(36,4)= -7.2274297476e-01
       coeff_l4(36,5)= -7.1861410141e-01
       coeff_l4(36,6)= -8.8445210457e-01
       coeff_l4(36,7)= -4.2027163506e-01
       coeff_l4(36,8)= -2.7225184441e-01
       coeff_l4(36,9)= -5.7129698992e-01
       coeff_l4(36,10)= -6.1251604557e-01
       coeff_l4(36,11)=  1.2239544094e-01
       coeff_l4(36,12)=  1.2562564015e-01
       coeff_l4(36,13)=  4.6810176224e-02
       coeff_l4(36,14)= -4.0147024393e-01
       coeff_l4(36,15)= -1.1593544483e-01
       coeff_l4(36,16)=  3.5417053849e-02
       coeff_l4(36,17)=  2.4936856329e-01
       coeff_l4(36,18)= -2.3306882381e-01
       coeff_l4(36,19)= -1.9881922007e-01
       coeff_l4(36,20)=  5.7932477444e-02
       coeff_l4(36,21)= -2.6608961821e-01
       coeff_l4(36,22)= -1.1557857692e-01
       coeff_l4(36,23)=  1.7795535922e-01
       coeff_l4(36,24)= -1.1888540536e-01
       coeff_l4(36,25)=  4.1544729471e-01
       coeff_l4(36,26)= -4.4039222598e-01
       coeff_l4(36,27)=  1.5154080093e-01
       coeff_l4(36,28)= -2.5461205840e-01
       coeff_l4(36,29)=  8.8260203600e-02
       coeff_l4(36,30)=  4.5432567596e-02
       coeff_l4(36,31)=  6.8424202502e-02
       coeff_l4(36,32)= -4.3246755004e-01
       coeff_l4(36,33)=  1.7499974370e-01
       coeff_l4(36,34)=  2.1891981363e-01
       coeff_l4(36,35)= -8.6319647729e-02
       coeff_l4(36,36)=  1.3459275663e-01
       coeff_l4(36,37)=  1.0531719774e-01
       coeff_l4(36,38)= -1.2791915238e-01
       coeff_l4(36,39)=  2.1749917418e-02
       coeff_l4(36,40)=  1.3832022250e-01
       coeff_l4(36,41)=  2.7037171647e-02
       coeff_l4(36,42)= -3.1065380573e-01
       coeff_l4(36,43)= -3.0176810920e-02
       coeff_l4(36,44)= -2.1255013347e-01
       coeff_l4(36,45)=  4.6217352152e-01
       coeff_l4(36,46)=  1.4528658986e-01
       coeff_l4(36,47)=  1.0741686821e-01
       coeff_l4(36,48)=  1.0605067015e-01
       coeff_l4(36,49)=  2.8632357717e-01
       coeff_l4(36,50)= -2.1459649503e-01
       coeff_l4(36,51)=  1.2921988964e-01
       coeff_l4(36,52)= -6.7596144974e-02
       coeff_l4(36,53)=  5.3459588438e-02
       coeff_l4(36,54)=  8.9015327394e-02
       coeff_l4(36,55)= -3.8857045770e-01
       coeff_l4(36,56)=  1.9154118001e-01
       coeff_l4(36,57)= -3.0581146479e-01
       coeff_l4(36,58)= -1.1036710441e-01
       coeff_l4(36,59)= -3.7704467773e-01
       coeff_l4(36,60)=  2.5866442919e-01
       coeff_l4(36,61)=  2.7177382261e-02
       coeff_l4(36,62)=  8.9314570650e-03
       coeff_l4(36,63)=  1.9803036749e-01
       coeff_l4(36,64)= -1.3452923298e-01
       coeff_l4(36,65)=  2.5803947449e-01
       coeff_l4(36,66)=  5.8390378952e-02
       coeff_l4(36,67)= -2.7403637767e-02
       coeff_l4(36,68)= -3.6418566108e-01
       coeff_l4(36,69)= -3.1739422679e-01
       coeff_l4(36,70)= -1.6035063565e-01
       coeff_l4(36,71)=  1.0508805513e-02
       coeff_l4(36,72)=  2.8556886315e-01
       coeff_l4(36,73)= -1.6154272854e-01
       coeff_l4(36,74)= -6.2799870968e-02
       coeff_l4(36,75)= -2.9792448878e-01
       coeff_l4(36,76)=  6.4299449325e-02
       coeff_l4(36,77)= -2.6682403684e-01
       coeff_l4(36,78)=  8.4897018969e-02
       coeff_l4(36,79)=  1.6354057193e-01
       coeff_l4(36,80)= -8.3015426993e-02
       coeff_l4(36,81)=  3.7926244736e-01
       coeff_l4(36,82)=  4.3793432415e-02
       coeff_l4(36,83)=  3.9245927334e-01
       coeff_l4(36,84)=  2.8479129076e-01
       coeff_l4(36,85)= -4.6875810623e-01
       coeff_l4(36,86)=  1.8778273463e-01
       coeff_l4(36,87)=  2.1882148087e-01
       coeff_l4(36,88)=  2.2559893131e-01
       coeff_l4(36,89)= -1.3730156422e-01
       coeff_l4(36,90)= -9.7332909703e-02
       coeff_l4(36,91)= -5.0940282643e-02
       coeff_l4(36,92)= -4.9491945654e-02
       coeff_l4(36,93)= -1.6124913236e-04
       coeff_l4(36,94)=  6.4807921648e-02
       coeff_l4(36,95)= -3.5202089697e-02
       coeff_l4(36,96)=  5.4608386755e-01
       coeff_l4(36,97)=  1.7278787494e-01
       coeff_l4(36,98)= -1.2946382165e-01
       coeff_l4(36,99)=  1.0531563312e-02
       coeff_l4(36,100)= -2.0316998661e-01
       coeff_l4(36,101)=  2.3962895572e-01
       coeff_l4(36,102)= -1.9889891148e-02
       coeff_l4(36,103)=  7.8201524913e-02
       coeff_l4(36,104)= -2.6598706841e-01
       coeff_l4(36,105)=  3.2038226724e-01
       coeff_l4(37,1)= -6.8950384855e-01
       coeff_l4(37,2)= -4.6221956611e-01
       coeff_l4(37,3)= -4.6738326550e-01
       coeff_l4(37,4)= -4.3887540698e-01
       coeff_l4(37,5)= -3.0395141244e-01
       coeff_l4(37,6)= -5.4815417528e-01
       coeff_l4(37,7)= -3.9273241162e-01
       coeff_l4(37,8)= -5.0377124548e-01
       coeff_l4(37,9)=  2.2309416905e-02
       coeff_l4(37,10)= -2.2787776589e-01
       coeff_l4(37,11)=  1.2959453464e-01
       coeff_l4(37,12)=  5.2814733237e-02
       coeff_l4(37,13)= -6.3288547099e-02
       coeff_l4(37,14)= -4.1031502187e-02
       coeff_l4(37,15)=  1.9305093214e-02
       coeff_l4(37,16)=  2.7940344065e-02
       coeff_l4(37,17)= -2.3352141678e-01
       coeff_l4(37,18)= -6.7466646433e-02
       coeff_l4(37,19)= -9.0307906270e-02
       coeff_l4(37,20)=  1.2019249611e-02
       coeff_l4(37,21)= -1.2852041423e-01
       coeff_l4(37,22)= -1.2658540905e-01
       coeff_l4(37,23)=  2.2538591921e-01
       coeff_l4(37,24)= -8.1233464181e-02
       coeff_l4(37,25)=  1.1290097237e-01
       coeff_l4(37,26)= -4.0992543101e-02
       coeff_l4(37,27)= -6.1600029469e-02
       coeff_l4(37,28)= -2.7920106426e-02
       coeff_l4(37,29)=  6.4028248191e-02
       coeff_l4(37,30)=  1.1046068370e-01
       coeff_l4(37,31)=  8.0078549683e-02
       coeff_l4(37,32)= -6.3270784914e-02
       coeff_l4(37,33)=  1.1004568636e-01
       coeff_l4(37,34)=  1.8141414225e-01
       coeff_l4(37,35)= -4.0535420179e-02
       coeff_l4(37,36)=  8.7644547224e-02
       coeff_l4(37,37)= -8.4737271070e-02
       coeff_l4(37,38)=  5.3547803313e-02
       coeff_l4(37,39)=  8.3670370281e-02
       coeff_l4(37,40)= -2.4555856362e-02
       coeff_l4(37,41)= -5.7051077485e-02
       coeff_l4(37,42)=  1.4597079158e-01
       coeff_l4(37,43)= -1.5059255064e-01
       coeff_l4(37,44)= -5.7422574610e-02
       coeff_l4(37,45)= -3.3179733902e-02
       coeff_l4(37,46)=  1.2145074084e-02
       coeff_l4(37,47)= -1.4340473711e-01
       coeff_l4(37,48)= -1.1602157727e-02
       coeff_l4(37,49)=  7.0452556014e-02
       coeff_l4(37,50)=  1.5643587708e-01
       coeff_l4(37,51)=  1.1426269263e-01
       coeff_l4(37,52)= -8.1408336759e-02
       coeff_l4(37,53)= -2.0484942943e-02
       coeff_l4(37,54)=  1.2010925263e-01
       coeff_l4(37,55)= -7.7864699066e-02
       coeff_l4(37,56)=  5.7659823447e-02
       coeff_l4(37,57)=  1.2216648459e-01
       coeff_l4(37,58)= -1.1235433072e-01
       coeff_l4(37,59)= -5.1112852991e-02
       coeff_l4(37,60)= -6.7091345787e-02
       coeff_l4(37,61)= -3.5360321403e-02
       coeff_l4(37,62)=  2.3104959726e-01
       coeff_l4(37,63)= -4.1867770255e-02
       coeff_l4(37,64)= -3.2264411449e-02
       coeff_l4(37,65)=  9.8315365613e-02
       coeff_l4(37,66)= -1.3080433011e-01
       coeff_l4(37,67)= -8.2950338721e-02
       coeff_l4(37,68)=  1.0219423473e-01
       coeff_l4(37,69)=  1.0673952103e-01
       coeff_l4(37,70)= -7.5294017792e-02
       coeff_l4(37,71)=  1.5762437135e-02
       coeff_l4(37,72)=  1.0696379840e-01
       coeff_l4(37,73)= -6.6281452775e-02
       coeff_l4(37,74)= -1.4435257018e-01
       coeff_l4(37,75)=  8.6079649627e-02
       coeff_l4(37,76)=  9.2345334589e-02
       coeff_l4(37,77)= -6.6031418741e-02
       coeff_l4(37,78)= -1.8533010781e-01
       coeff_l4(37,79)= -6.9280102849e-02
       coeff_l4(37,80)= -7.1767605841e-02
       coeff_l4(37,81)=  5.0541337579e-02
       coeff_l4(37,82)=  3.4400939941e-02
       coeff_l4(37,83)=  1.4238315634e-03
       coeff_l4(37,84)= -1.1529793590e-01
       coeff_l4(37,85)=  9.2674784362e-02
       coeff_l4(37,86)=  6.0812849551e-02
       coeff_l4(37,87)= -6.9047681987e-02
       coeff_l4(37,88)= -1.3233402371e-01
       coeff_l4(37,89)=  1.0632303357e-01
       coeff_l4(37,90)= -1.0237699747e-01
       coeff_l4(37,91)= -1.2897656858e-01
       coeff_l4(37,92)= -1.1827217788e-01
       coeff_l4(37,93)= -3.7559218705e-02
       coeff_l4(37,94)=  7.9273335636e-02
       coeff_l4(37,95)= -5.3182546049e-02
       coeff_l4(37,96)=  3.8701355457e-02
       coeff_l4(37,97)= -2.9119547457e-02
       coeff_l4(37,98)=  1.3920013607e-01
       coeff_l4(37,99)= -1.2844410539e-01
       coeff_l4(37,100)= -3.7619654089e-02
       coeff_l4(37,101)= -7.9236678779e-02
       coeff_l4(37,102)=  1.4729526639e-01
       coeff_l4(37,103)=  1.9238215685e-01
       coeff_l4(37,104)= -1.2073723972e-01
       coeff_l4(37,105)=  8.2643553615e-03
       coeff_l4(38,1)=  1.6798558235e+00
       coeff_l4(38,2)=  1.8161728382e+00
       coeff_l4(38,3)=  1.9461988211e+00
       coeff_l4(38,4)=  1.5981794596e+00
       coeff_l4(38,5)=  1.7705948353e+00
       coeff_l4(38,6)=  2.1782448292e+00
       coeff_l4(38,7)=  2.1437222958e+00
       coeff_l4(38,8)=  1.9584451914e+00
       coeff_l4(38,9)=  1.2237906456e+00
       coeff_l4(38,10)=  1.2232515812e+00
       coeff_l4(38,11)=  8.6509728432e-01
       coeff_l4(38,12)=  7.5043606758e-01
       coeff_l4(38,13)=  8.3848381042e-01
       coeff_l4(38,14)=  1.7239177227e+00
       coeff_l4(38,15)= -1.5964289010e-01
       coeff_l4(38,16)= -3.1541445851e-01
       coeff_l4(38,17)= -1.2609201670e-01
       coeff_l4(38,18)=  4.3515384197e-02
       coeff_l4(38,19)= -6.8483686447e-01
       coeff_l4(38,20)= -4.6779987216e-01
       coeff_l4(38,21)= -1.8781812489e-01
       coeff_l4(38,22)=  1.3320912421e-01
       coeff_l4(38,23)=  2.5064843893e-01
       coeff_l4(38,24)= -4.1626778245e-01
       coeff_l4(38,25)=  9.7872447222e-03
       coeff_l4(38,26)= -5.9155076742e-01
       coeff_l4(38,27)=  2.8883842751e-02
       coeff_l4(38,28)= -3.6414015293e-01
       coeff_l4(38,29)= -1.1541278660e-01
       coeff_l4(38,30)= -5.5790686607e-01
       coeff_l4(38,31)= -9.0177536011e-01
       coeff_l4(38,32)= -3.8454037905e-01
       coeff_l4(38,33)= -2.3758299649e-02
       coeff_l4(38,34)= -5.5988132954e-01
       coeff_l4(38,35)= -3.1575370580e-02
       coeff_l4(38,36)=  1.0612881184e+00
       coeff_l4(38,37)=  5.6909382343e-02
       coeff_l4(38,38)=  3.9844876528e-01
       coeff_l4(38,39)= -4.6558481455e-01
       coeff_l4(38,40)= -1.0410310328e-01
       coeff_l4(38,41)= -8.7554179132e-02
       coeff_l4(38,42)=  4.7133129835e-01
       coeff_l4(38,43)=  5.8946959674e-02
       coeff_l4(38,44)=  1.3887888193e-01
       coeff_l4(38,45)= -6.5608763695e-01
       coeff_l4(38,46)=  5.3298473358e-01
       coeff_l4(38,47)= -4.4622290134e-01
       coeff_l4(38,48)= -5.8828628063e-01
       coeff_l4(38,49)= -5.1676910371e-02
       coeff_l4(38,50)= -1.4716498554e-01
       coeff_l4(38,51)=  1.1101393402e-01
       coeff_l4(38,52)= -6.8266324699e-02
       coeff_l4(38,53)= -1.7227713764e-01
       coeff_l4(38,54)= -3.4708797932e-01
       coeff_l4(38,55)=  7.5460329652e-02
       coeff_l4(38,56)= -2.0402102172e-01
       coeff_l4(38,57)=  2.6010930538e-01
       coeff_l4(38,58)=  3.6140176654e-01
       coeff_l4(38,59)= -4.0158252232e-03
       coeff_l4(38,60)=  3.0387541279e-02
       coeff_l4(38,61)= -1.6504283994e-02
       coeff_l4(38,62)= -3.9756888151e-01
       coeff_l4(38,63)= -1.1117558181e-01
       coeff_l4(38,64)=  8.0387371778e-01
       coeff_l4(38,65)= -2.3714409769e-01
       coeff_l4(38,66)= -4.6073850989e-01
       coeff_l4(38,67)= -3.5273265839e-01
       coeff_l4(38,68)= -5.5746185780e-01
       coeff_l4(38,69)=  2.4477415718e-03
       coeff_l4(38,70)= -2.4199633300e-01
       coeff_l4(38,71)= -4.2265763879e-01
       coeff_l4(38,72)=  1.0690367222e+00
       coeff_l4(38,73)=  2.0621961355e-01
       coeff_l4(38,74)= -2.9287928343e-01
       coeff_l4(38,75)=  5.9391293675e-02
       coeff_l4(38,76)=  1.8768157065e-01
       coeff_l4(38,77)= -5.4890072346e-01
       coeff_l4(38,78)=  3.3779823780e-01
       coeff_l4(38,79)= -7.1242165565e-01
       coeff_l4(38,80)= -3.4280604124e-01
       coeff_l4(38,81)=  1.2805999815e-01
       coeff_l4(38,82)=  4.4571614265e-01
       coeff_l4(38,83)= -3.7740474939e-01
       coeff_l4(38,84)= -3.2897728682e-01
       coeff_l4(38,85)=  9.1338062286e-01
       coeff_l4(38,86)= -3.2627031207e-01
       coeff_l4(38,87)=  2.7704492211e-01
       coeff_l4(38,88)= -4.7438759357e-02
       coeff_l4(38,89)= -3.4589192271e-01
       coeff_l4(38,90)=  1.3886496425e-01
       coeff_l4(38,91)=  8.0844599009e-01
       coeff_l4(38,92)=  2.0753689110e-01
       coeff_l4(38,93)=  1.8935891986e-01
       coeff_l4(38,94)= -6.2531483173e-01
       coeff_l4(38,95)=  1.1283840239e-01
       coeff_l4(38,96)= -2.1259486675e-01
       coeff_l4(38,97)=  1.4742837846e-01
       coeff_l4(38,98)= -6.6278988123e-01
       coeff_l4(38,99)=  7.1844112873e-01
       coeff_l4(38,100)=  7.1075796150e-03
       coeff_l4(38,101)= -1.3654530048e-01
       coeff_l4(38,102)=  1.6500650346e-01
       coeff_l4(38,103)=  1.1463859677e-01
       coeff_l4(38,104)=  1.8590298295e-01
       coeff_l4(38,105)= -2.3863317072e-01
       coeff_l4(39,1)= -6.9420105219e-01
       coeff_l4(39,2)= -2.9098102450e-01
       coeff_l4(39,3)= -5.2250158787e-01
       coeff_l4(39,4)= -5.3284734488e-01
       coeff_l4(39,5)= -4.0677487850e-01
       coeff_l4(39,6)= -6.0474491119e-01
       coeff_l4(39,7)= -4.6771380305e-01
       coeff_l4(39,8)= -4.4540074468e-01
       coeff_l4(39,9)=  6.0795627534e-02
       coeff_l4(39,10)= -3.1438535452e-01
       coeff_l4(39,11)=  3.0732753873e-01
       coeff_l4(39,12)=  2.9483690858e-01
       coeff_l4(39,13)= -3.8706492633e-02
       coeff_l4(39,14)=  6.8052820861e-02
       coeff_l4(39,15)=  2.6109459996e-01
       coeff_l4(39,16)=  1.3205508702e-02
       coeff_l4(39,17)= -2.6094095781e-02
       coeff_l4(39,18)=  9.0070225298e-02
       coeff_l4(39,19)= -1.6301710904e-01
       coeff_l4(39,20)= -1.3910730183e-01
       coeff_l4(39,21)= -9.5388382673e-02
       coeff_l4(39,22)= -1.7490626872e-01
       coeff_l4(39,23)=  2.4238614365e-02
       coeff_l4(39,24)= -4.2196601629e-02
       coeff_l4(39,25)= -7.5793214142e-02
       coeff_l4(39,26)=  2.6043182239e-02
       coeff_l4(39,27)=  1.8754319847e-01
       coeff_l4(39,28)=  7.7078044415e-02
       coeff_l4(39,29)= -1.5515106916e-01
       coeff_l4(39,30)=  5.0290379673e-02
       coeff_l4(39,31)= -1.2973520160e-01
       coeff_l4(39,32)= -7.2654679418e-02
       coeff_l4(39,33)=  2.9094485566e-02
       coeff_l4(39,34)=  2.6758924127e-01
       coeff_l4(39,35)=  1.3422757387e-01
       coeff_l4(39,36)= -6.1453420669e-02
       coeff_l4(39,37)= -4.0304888040e-02
       coeff_l4(39,38)= -1.0427019000e-01
       coeff_l4(39,39)= -2.7082461119e-01
       coeff_l4(39,40)=  5.6885774247e-03
       coeff_l4(39,41)=  2.5354007259e-02
       coeff_l4(39,42)= -1.9803471863e-01
       coeff_l4(39,43)=  1.3905173540e-01
       coeff_l4(39,44)= -1.0407702625e-01
       coeff_l4(39,45)=  3.1675696373e-02
       coeff_l4(39,46)=  2.0457759500e-02
       coeff_l4(39,47)= -7.6806902885e-02
       coeff_l4(39,48)= -1.3406430185e-01
       coeff_l4(39,49)= -4.3025780469e-02
       coeff_l4(39,50)= -1.2086711079e-01
       coeff_l4(39,51)=  7.6273322105e-02
       coeff_l4(39,52)=  4.7626666725e-02
       coeff_l4(39,53)=  1.4567790553e-02
       coeff_l4(39,54)=  1.3857743144e-01
       coeff_l4(39,55)= -3.1122576445e-02
       coeff_l4(39,56)=  1.1904463917e-01
       coeff_l4(39,57)=  2.1041201055e-01
       coeff_l4(39,58)=  1.6028074548e-02
       coeff_l4(39,59)= -3.5552196205e-02
       coeff_l4(39,60)=  2.0039059222e-01
       coeff_l4(39,61)= -6.9567419589e-02
       coeff_l4(39,62)=  1.4432862401e-01
       coeff_l4(39,63)= -1.0130675137e-01
       coeff_l4(39,64)= -1.3396234810e-01
       coeff_l4(39,65)=  7.9041182995e-02
       coeff_l4(39,66)=  2.0150206983e-02
       coeff_l4(39,67)= -7.5387507677e-02
       coeff_l4(39,68)= -3.1167671084e-02
       coeff_l4(39,69)=  1.4434601367e-01
       coeff_l4(39,70)=  4.1053536534e-01
       coeff_l4(39,71)= -5.3847126663e-02
       coeff_l4(39,72)= -1.7683174461e-02
       coeff_l4(39,73)= -5.1703199744e-02
       coeff_l4(39,74)= -4.7493990511e-02
       coeff_l4(39,75)= -2.1307909861e-02
       coeff_l4(39,76)=  1.7993827164e-01
       coeff_l4(39,77)= -2.7522268891e-01
       coeff_l4(39,78)= -4.9038946629e-02
       coeff_l4(39,79)= -8.4813609719e-02
       coeff_l4(39,80)= -9.2186145484e-02
       coeff_l4(39,81)= -1.0297513008e-01
       coeff_l4(39,82)=  1.4213140309e-01
       coeff_l4(39,83)=  9.9289044738e-02
       coeff_l4(39,84)= -7.7395759523e-02
       coeff_l4(39,85)=  8.9755401015e-02
       coeff_l4(39,86)= -2.8524515033e-01
       coeff_l4(39,87)= -1.7717079818e-01
       coeff_l4(39,88)=  8.8564813137e-02
       coeff_l4(39,89)= -2.6627668738e-01
       coeff_l4(39,90)= -7.6835653745e-03
       coeff_l4(39,91)=  4.1850406677e-02
       coeff_l4(39,92)= -2.2541649640e-02
       coeff_l4(39,93)=  7.5840823352e-02
       coeff_l4(39,94)=  2.9538056254e-01
       coeff_l4(39,95)=  2.5252959132e-01
       coeff_l4(39,96)=  2.5524336100e-01
       coeff_l4(39,97)=  3.4279605746e-01
       coeff_l4(39,98)= -5.5063400418e-02
       coeff_l4(39,99)= -1.1021984741e-02
       coeff_l4(39,100)=  1.7901708186e-01
       coeff_l4(39,101)=  2.3562227190e-01
       coeff_l4(39,102)=  2.9592299834e-03
       coeff_l4(39,103)=  6.6205680370e-02
       coeff_l4(39,104)= -4.5673292130e-02
       coeff_l4(39,105)= -9.5533475280e-02
       coeff_l4(40,1)=  1.0292580724e-01
       coeff_l4(40,2)=  5.4519754648e-01
       coeff_l4(40,3)=  4.1128587723e-01
       coeff_l4(40,4)=  2.4826142192e-01
       coeff_l4(40,5)=  2.5289353728e-01
       coeff_l4(40,6)=  3.1098607183e-01
       coeff_l4(40,7)= -1.9976094365e-02
       coeff_l4(40,8)= -1.1244388670e-01
       coeff_l4(40,9)=  3.9274864830e-03
       coeff_l4(40,10)=  1.6662068665e-01
       coeff_l4(40,11)= -2.2137207910e-02
       coeff_l4(40,12)= -1.2287665904e-01
       coeff_l4(40,13)= -1.7567744851e-01
       coeff_l4(40,14)=  2.6093900204e-01
       coeff_l4(40,15)=  2.4559517205e-01
       coeff_l4(40,16)= -1.5434724092e-01
       coeff_l4(40,17)= -1.8022927642e-01
       coeff_l4(40,18)= -3.0152279139e-01
       coeff_l4(40,19)= -5.0107955933e-02
       coeff_l4(40,20)=  6.4173109829e-02
       coeff_l4(40,21)= -1.9888609648e-01
       coeff_l4(40,22)=  3.1346697360e-02
       coeff_l4(40,23)=  4.5148455538e-03
       coeff_l4(40,24)=  1.6748003662e-01
       coeff_l4(40,25)= -8.3513833582e-02
       coeff_l4(40,26)=  2.7150276303e-01
       coeff_l4(40,27)=  1.7256760597e-01
       coeff_l4(40,28)= -3.1525239348e-02
       coeff_l4(40,29)= -1.9961543381e-01
       coeff_l4(40,30)=  1.1137295514e-01
       coeff_l4(40,31)= -1.7866890132e-01
       coeff_l4(40,32)=  1.1630541831e-01
       coeff_l4(40,33)= -3.2494626939e-02
       coeff_l4(40,34)= -1.6092866659e-01
       coeff_l4(40,35)= -3.8488585502e-02
       coeff_l4(40,36)=  6.9697029889e-02
       coeff_l4(40,37)=  7.5375966728e-02
       coeff_l4(40,38)=  1.5285211802e-01
       coeff_l4(40,39)=  8.1050723791e-02
       coeff_l4(40,40)= -1.3505743816e-02
       coeff_l4(40,41)=  3.7236854434e-02
       coeff_l4(40,42)=  2.1965610981e-01
       coeff_l4(40,43)= -1.1130259186e-01
       coeff_l4(40,44)= -1.1673402041e-01
       coeff_l4(40,45)= -2.9157939553e-01
       coeff_l4(40,46)= -8.4121085703e-02
       coeff_l4(40,47)=  1.5922129154e-02
       coeff_l4(40,48)= -1.0724309087e-01
       coeff_l4(40,49)=  1.1744651198e-01
       coeff_l4(40,50)= -5.8173302561e-02
       coeff_l4(40,51)=  3.2243826985e-01
       coeff_l4(40,52)= -4.8459600657e-02
       coeff_l4(40,53)=  8.6576543748e-02
       coeff_l4(40,54)= -2.4407759309e-01
       coeff_l4(40,55)=  3.1513947994e-02
       coeff_l4(40,56)= -2.2320072353e-01
       coeff_l4(40,57)=  1.2676411867e-01
       coeff_l4(40,58)=  8.6722984910e-02
       coeff_l4(40,59)=  2.7264925838e-01
       coeff_l4(40,60)= -2.2399770096e-02
       coeff_l4(40,61)= -2.3334565759e-01
       coeff_l4(40,62)= -7.6933078468e-02
       coeff_l4(40,63)= -1.2114000507e-02
       coeff_l4(40,64)=  3.3654946089e-01
       coeff_l4(40,65)= -1.9786236808e-02
       coeff_l4(40,66)=  3.1576754991e-03
       coeff_l4(40,67)=  1.4462230727e-02
       coeff_l4(40,68)= -7.5736694038e-02
       coeff_l4(40,69)=  3.9321705699e-02
       coeff_l4(40,70)=  1.6177283227e-01
       coeff_l4(40,71)= -9.4666279852e-02
       coeff_l4(40,72)=  1.1918312311e-01
       coeff_l4(40,73)= -6.2394142151e-02
       coeff_l4(40,74)= -1.8830423057e-01
       coeff_l4(40,75)= -3.0065819621e-01
       coeff_l4(40,76)=  2.0711514354e-01
       coeff_l4(40,77)= -8.7910994887e-02
       coeff_l4(40,78)= -2.0455764234e-01
       coeff_l4(40,79)= -1.8315647542e-01
       coeff_l4(40,80)= -2.2351932526e-01
       coeff_l4(40,81)=  6.8133341847e-04
       coeff_l4(40,82)=  1.3230001926e-01
       coeff_l4(40,83)= -1.3733462989e-01
       coeff_l4(40,84)= -3.1655329466e-01
       coeff_l4(40,85)=  3.4954902530e-01
       coeff_l4(40,86)=  7.5948484242e-02
       coeff_l4(40,87)=  1.1351321638e-01
       coeff_l4(40,88)= -2.0393948257e-01
       coeff_l4(40,89)=  7.6538622379e-02
       coeff_l4(40,90)= -3.7071455270e-02
       coeff_l4(40,91)=  9.5214573666e-03
       coeff_l4(40,92)= -1.2158662826e-01
       coeff_l4(40,93)= -1.2924899161e-01
       coeff_l4(40,94)=  1.2328606844e-01
       coeff_l4(40,95)=  2.2728297859e-02
       coeff_l4(40,96)=  1.0315608978e-01
       coeff_l4(40,97)=  1.8784303963e-01
       coeff_l4(40,98)= -1.0958962888e-01
       coeff_l4(40,99)= -5.1767475903e-02
       coeff_l4(40,100)=  4.9876414239e-02
       coeff_l4(40,101)= -2.1051426232e-01
       coeff_l4(40,102)= -5.6348282844e-02
       coeff_l4(40,103)= -8.7594695389e-02
       coeff_l4(40,104)=  4.0310047567e-02
       coeff_l4(40,105)= -2.2868801653e-01
       coeff_l4(41,1)=  2.6306307316e+00
       coeff_l4(41,2)=  2.5034115314e+00
       coeff_l4(41,3)=  2.4581723213e+00
       coeff_l4(41,4)=  2.9885663986e+00
       coeff_l4(41,5)=  1.9321144819e+00
       coeff_l4(41,6)=  1.7979613543e+00
       coeff_l4(41,7)=  1.8793126345e+00
       coeff_l4(41,8)=  1.5257560015e+00
       coeff_l4(41,9)=  1.1091479063e+00
       coeff_l4(41,10)=  9.9462956190e-01
       coeff_l4(41,11)=  1.9014811516e+00
       coeff_l4(41,12)=  2.0538077354e+00
       coeff_l4(41,13)=  1.6822021008e+00
       coeff_l4(41,14)=  1.1002634764e+00
       coeff_l4(41,15)= -5.4147714376e-01
       coeff_l4(41,16)=  1.0207607597e-01
       coeff_l4(41,17)= -1.2108578682e+00
       coeff_l4(41,18)= -4.0144985914e-01
       coeff_l4(41,19)= -4.9741658568e-01
       coeff_l4(41,20)= -4.7156640887e-01
       coeff_l4(41,21)= -1.8895536661e-01
       coeff_l4(41,22)=  1.5975350142e-01
       coeff_l4(41,23)=  5.9988123178e-01
       coeff_l4(41,24)= -3.6291342974e-01
       coeff_l4(41,25)= -1.3189269602e-01
       coeff_l4(41,26)= -4.9017393589e-01
       coeff_l4(41,27)= -9.9847547710e-02
       coeff_l4(41,28)=  3.7932825089e-01
       coeff_l4(41,29)=  2.0397609472e-01
       coeff_l4(41,30)=  1.3844861090e-01
       coeff_l4(41,31)= -4.6051260829e-01
       coeff_l4(41,32)= -3.4962552786e-01
       coeff_l4(41,33)= -2.2376655042e-01
       coeff_l4(41,34)=  3.1065702438e-01
       coeff_l4(41,35)= -2.2374808788e-01
       coeff_l4(41,36)=  4.7981354594e-01
       coeff_l4(41,37)=  2.3412983119e-01
       coeff_l4(41,38)= -2.1308134496e-01
       coeff_l4(41,39)=  1.4269074798e-01
       coeff_l4(41,40)=  4.6667867899e-01
       coeff_l4(41,41)= -4.9505140632e-02
       coeff_l4(41,42)= -4.1461986303e-01
       coeff_l4(41,43)= -5.5843752623e-01
       coeff_l4(41,44)= -6.4848387241e-01
       coeff_l4(41,45)= -2.5245189667e-01
       coeff_l4(41,46)=  5.9237855673e-01
       coeff_l4(41,47)=  5.6572091579e-01
       coeff_l4(41,48)= -4.0722757578e-01
       coeff_l4(41,49)=  2.3559826612e-01
       coeff_l4(41,50)=  1.8296612799e-01
       coeff_l4(41,51)= -1.4771102369e-01
       coeff_l4(41,52)=  5.9080260992e-01
       coeff_l4(41,53)=  4.6789672971e-01
       coeff_l4(41,54)=  6.8771317601e-02
       coeff_l4(41,55)=  4.5345455408e-02
       coeff_l4(41,56)= -3.0481866002e-01
       coeff_l4(41,57)= -3.0243888497e-01
       coeff_l4(41,58)= -4.7475945950e-01
       coeff_l4(41,59)= -1.6174161434e-01
       coeff_l4(41,60)=  5.2888435125e-01
       coeff_l4(41,61)=  4.9077086151e-03
       coeff_l4(41,62)= -6.5422616899e-02
       coeff_l4(41,63)= -1.9857639447e-02
       coeff_l4(41,64)=  3.4064683318e-01
       coeff_l4(41,65)= -2.8403648734e-01
       coeff_l4(41,66)= -5.5461919308e-01
       coeff_l4(41,67)= -2.9352191091e-01
       coeff_l4(41,68)= -4.1883337498e-01
       coeff_l4(41,69)= -4.4453430176e-01
       coeff_l4(41,70)= -4.2305791378e-01
       coeff_l4(41,71)=  9.7787909210e-02
       coeff_l4(41,72)=  4.2235791683e-01
       coeff_l4(41,73)=  9.5310248435e-02
       coeff_l4(41,74)=  1.0665147007e-01
       coeff_l4(41,75)= -3.8074986078e-03
       coeff_l4(41,76)= -5.1606875658e-01
       coeff_l4(41,77)= -3.2918244600e-01
       coeff_l4(41,78)=  6.4322844148e-02
       coeff_l4(41,79)= -3.8464260101e-01
       coeff_l4(41,80)= -5.4622429609e-01
       coeff_l4(41,81)= -1.4523161948e-01
       coeff_l4(41,82)= -6.0778582096e-01
       coeff_l4(41,83)=  1.4673489332e-01
       coeff_l4(41,84)= -4.7885560989e-01
       coeff_l4(41,85)= -2.9782998562e-01
       coeff_l4(41,86)=  1.8165779114e-01
       coeff_l4(41,87)=  2.3968489468e-01
       coeff_l4(41,88)= -1.1561226100e-01
       coeff_l4(41,89)=  2.2518000007e-01
       coeff_l4(41,90)=  1.7883940041e-01
       coeff_l4(41,91)= -3.1976294518e-01
       coeff_l4(41,92)= -5.5415067822e-02
       coeff_l4(41,93)=  4.0359008312e-01
       coeff_l4(41,94)= -6.3498407602e-01
       coeff_l4(41,95)=  2.6492470503e-01
       coeff_l4(41,96)=  1.0278149843e+00
       coeff_l4(41,97)= -8.4317259490e-02
       coeff_l4(41,98)= -3.4478148818e-01
       coeff_l4(41,99)= -9.4169616932e-04
       coeff_l4(41,100)=  5.3957378864e-01
       coeff_l4(41,101)=  7.8853702545e-01
       coeff_l4(41,102)=  4.2900344729e-01
       coeff_l4(41,103)=  7.3880709708e-02
       coeff_l4(41,104)= -3.0650913715e-01
       coeff_l4(41,105)=  2.5956293941e-01
       coeff_l4(42,1)= -3.5189440846e-01
       coeff_l4(42,2)= -2.3131911457e-01
       coeff_l4(42,3)= -9.5734216273e-02
       coeff_l4(42,4)= -1.9854782522e-01
       coeff_l4(42,5)= -3.2017195225e-01
       coeff_l4(42,6)= -2.8857716918e-01
       coeff_l4(42,7)= -1.8027886748e-01
       coeff_l4(42,8)= -2.8117191792e-01
       coeff_l4(42,9)=  1.9630098343e-01
       coeff_l4(42,10)=  2.0697498322e-01
       coeff_l4(42,11)=  3.7551259995e-01
       coeff_l4(42,12)=  3.4801885486e-01
       coeff_l4(42,13)=  2.5937977433e-01
       coeff_l4(42,14)=  1.6819401085e-01
       coeff_l4(42,15)=  8.6641065776e-02
       coeff_l4(42,16)= -1.1594817042e-01
       coeff_l4(42,17)= -1.0920441896e-01
       coeff_l4(42,18)=  1.9856756553e-02
       coeff_l4(42,19)=  5.5651817471e-02
       coeff_l4(42,20)=  1.5430729836e-02
       coeff_l4(42,21)= -1.1806235462e-01
       coeff_l4(42,22)= -1.7662273720e-02
       coeff_l4(42,23)=  1.7126850784e-01
       coeff_l4(42,24)= -1.2956625223e-01
       coeff_l4(42,25)=  1.1654045433e-01
       coeff_l4(42,26)=  1.2583760917e-01
       coeff_l4(42,27)=  6.8889848888e-02
       coeff_l4(42,28)= -9.7552416846e-03
       coeff_l4(42,29)=  1.2770244479e-01
       coeff_l4(42,30)= -4.0854658931e-02
       coeff_l4(42,31)=  3.9496596903e-02
       coeff_l4(42,32)= -1.0446807370e-02
       coeff_l4(42,33)= -3.5776674747e-02
       coeff_l4(42,34)= -9.4412025064e-03
       coeff_l4(42,35)=  7.8467033803e-02
       coeff_l4(42,36)=  8.5156537592e-02
       coeff_l4(42,37)= -8.0417795107e-03
       coeff_l4(42,38)= -2.4961242452e-02
       coeff_l4(42,39)=  8.2566551864e-02
       coeff_l4(42,40)=  1.2036063522e-01
       coeff_l4(42,41)= -1.2520614266e-01
       coeff_l4(42,42)=  1.6972776502e-02
       coeff_l4(42,43)= -1.0413797945e-01
       coeff_l4(42,44)= -1.0245088488e-01
       coeff_l4(42,45)=  1.2447013706e-01
       coeff_l4(42,46)= -4.1186112911e-02
       coeff_l4(42,47)=  1.1762355268e-01
       coeff_l4(42,48)=  3.5453815013e-02
       coeff_l4(42,49)=  4.1498385370e-02
       coeff_l4(42,50)=  3.1470460817e-03
       coeff_l4(42,51)=  1.0686203837e-02
       coeff_l4(42,52)= -1.3574528694e-01
       coeff_l4(42,53)= -9.2741325498e-02
       coeff_l4(42,54)= -1.0859327391e-02
       coeff_l4(42,55)=  2.1471114829e-02
       coeff_l4(42,56)=  1.4449290931e-01
       coeff_l4(42,57)=  1.2967364490e-01
       coeff_l4(42,58)= -1.0411098599e-01
       coeff_l4(42,59)= -1.1301840097e-01
       coeff_l4(42,60)= -7.0732727647e-02
       coeff_l4(42,61)= -9.6675358713e-02
       coeff_l4(42,62)=  3.8908105344e-02
       coeff_l4(42,63)= -7.5194932520e-02
       coeff_l4(42,64)=  6.5918169916e-02
       coeff_l4(42,65)= -1.2258122116e-01
       coeff_l4(42,66)= -1.6414254787e-04
       coeff_l4(42,67)=  7.4820645154e-02
       coeff_l4(42,68)=  5.3978085518e-02
       coeff_l4(42,69)= -1.6675922275e-01
       coeff_l4(42,70)=  1.4571996033e-01
       coeff_l4(42,71)= -1.2681138515e-01
       coeff_l4(42,72)=  4.9923855811e-02
       coeff_l4(42,73)=  1.1167556792e-01
       coeff_l4(42,74)= -1.7747031152e-01
       coeff_l4(42,75)= -1.4595034719e-01
       coeff_l4(42,76)=  6.7724168301e-02
       coeff_l4(42,77)= -8.4837824106e-02
       coeff_l4(42,78)= -1.4265160263e-01
       coeff_l4(42,79)= -1.0588306189e-01
       coeff_l4(42,80)= -1.1765663326e-01
       coeff_l4(42,81)= -1.2382852146e-03
       coeff_l4(42,82)=  1.1406551301e-01
       coeff_l4(42,83)= -8.6019366980e-02
       coeff_l4(42,84)= -1.0754057020e-01
       coeff_l4(42,85)=  8.9284814894e-02
       coeff_l4(42,86)=  9.2753224075e-02
       coeff_l4(42,87)= -4.9479641020e-03
       coeff_l4(42,88)= -1.0081493109e-01
       coeff_l4(42,89)=  2.1812861785e-02
       coeff_l4(42,90)= -1.0097542405e-01
       coeff_l4(42,91)= -8.6897298694e-02
       coeff_l4(42,92)=  4.9463640898e-02
       coeff_l4(42,93)= -6.2882713974e-02
       coeff_l4(42,94)=  1.0144414008e-01
       coeff_l4(42,95)= -1.0515645891e-01
       coeff_l4(42,96)= -1.2527978979e-02
       coeff_l4(42,97)=  8.9161448181e-02
       coeff_l4(42,98)=  9.6495747566e-02
       coeff_l4(42,99)= -2.9219122604e-02
       coeff_l4(42,100)= -2.2271286696e-02
       coeff_l4(42,101)= -1.4461603761e-01
       coeff_l4(42,102)= -1.9050447643e-01
       coeff_l4(42,103)= -3.3627148718e-02
       coeff_l4(42,104)= -3.3028952312e-03
       coeff_l4(42,105)= -1.3873690367e-01
       coeff_l4(43,1)=  1.4708267152e-01
       coeff_l4(43,2)=  3.7657430768e-01
       coeff_l4(43,3)=  7.7316045761e-02
       coeff_l4(43,4)=  2.3524586111e-02
       coeff_l4(43,5)=  1.6416764259e-01
       coeff_l4(43,6)= -3.5655606538e-02
       coeff_l4(43,7)=  1.3046115637e-01
       coeff_l4(43,8)= -2.3232080042e-01
       coeff_l4(43,9)=  2.8497889638e-01
       coeff_l4(43,10)= -7.6959379017e-02
       coeff_l4(43,11)=  7.7917158604e-02
       coeff_l4(43,12)=  1.9628056884e-01
       coeff_l4(43,13)= -2.5881493092e-01
       coeff_l4(43,14)=  4.8783856630e-01
       coeff_l4(43,15)=  2.0483729243e-01
       coeff_l4(43,16)=  8.4847241640e-02
       coeff_l4(43,17)=  3.4814319015e-01
       coeff_l4(43,18)= -2.3839344084e-01
       coeff_l4(43,19)= -7.2243846953e-02
       coeff_l4(43,20)= -8.2396812737e-02
       coeff_l4(43,21)= -3.1496141106e-02
       coeff_l4(43,22)=  1.5854088962e-01
       coeff_l4(43,23)= -1.2680515647e-01
       coeff_l4(43,24)=  4.7540584207e-01
       coeff_l4(43,25)=  1.1498159170e-01
       coeff_l4(43,26)=  3.0734011531e-01
       coeff_l4(43,27)=  1.0792422108e-03
       coeff_l4(43,28)= -1.6647373559e-03
       coeff_l4(43,29)= -7.7595961094e-01
       coeff_l4(43,30)=  6.9923475385e-02
       coeff_l4(43,31)= -7.2842255235e-02
       coeff_l4(43,32)=  2.9041397572e-01
       coeff_l4(43,33)= -3.9423578978e-01
       coeff_l4(43,34)= -4.5795238018e-01
       coeff_l4(43,35)= -8.0503195524e-02
       coeff_l4(43,36)= -7.2810955346e-02
       coeff_l4(43,37)= -1.7597949505e-01
       coeff_l4(43,38)=  1.0091212392e-01
       coeff_l4(43,39)= -2.4794347584e-01
       coeff_l4(43,40)= -1.3611455262e-01
       coeff_l4(43,41)=  3.6869981885e-01
       coeff_l4(43,42)=  6.3706332445e-01
       coeff_l4(43,43)=  5.9643322229e-01
       coeff_l4(43,44)= -3.9103224874e-01
       coeff_l4(43,45)=  1.2492314726e-01
       coeff_l4(43,46)= -2.8742811084e-01
       coeff_l4(43,47)= -3.8731566072e-01
       coeff_l4(43,48)= -3.1226390600e-01
       coeff_l4(43,49)=  1.7101502046e-02
       coeff_l4(43,50)= -3.5284480453e-01
       coeff_l4(43,51)=  3.4233629704e-01
       coeff_l4(43,52)= -2.1814802289e-01
       coeff_l4(43,53)=  2.7815263718e-02
       coeff_l4(43,54)=  2.2575744987e-01
       coeff_l4(43,55)= -2.3731234670e-01
       coeff_l4(43,56)=  9.4815976918e-02
       coeff_l4(43,57)=  3.7479165196e-01
       coeff_l4(43,58)=  7.8752879053e-03
       coeff_l4(43,59)= -5.5646556616e-01
       coeff_l4(43,60)= -1.9324137270e-01
       coeff_l4(43,61)= -3.1394562125e-01
       coeff_l4(43,62)=  1.2851871550e-01
       coeff_l4(43,63)= -3.7639069557e-01
       coeff_l4(43,64)=  2.0872646570e-01
       coeff_l4(43,65)=  4.5195057988e-02
       coeff_l4(43,66)= -5.1078045368e-01
       coeff_l4(43,67)=  2.7434125543e-01
       coeff_l4(43,68)= -1.4745651186e-01
       coeff_l4(43,69)=  7.5080908835e-02
       coeff_l4(43,70)=  2.0208375156e-01
       coeff_l4(43,71)= -2.5618055463e-01
       coeff_l4(43,72)=  3.2838705927e-02
       coeff_l4(43,73)= -2.1953292191e-01
       coeff_l4(43,74)= -1.4020198584e-01
       coeff_l4(43,75)=  3.6980599165e-01
       coeff_l4(43,76)=  7.5999675319e-03
       coeff_l4(43,77)=  2.1874485910e-01
       coeff_l4(43,78)=  2.5645700097e-01
       coeff_l4(43,79)=  2.2008128464e-01
       coeff_l4(43,80)= -2.1364298463e-01
       coeff_l4(43,81)=  3.7635844201e-02
       coeff_l4(43,82)=  3.5847160220e-01
       coeff_l4(43,83)=  5.8988478035e-02
       coeff_l4(43,84)=  5.8398704976e-02
       coeff_l4(43,85)=  1.5126119554e-01
       coeff_l4(43,86)= -4.8319745064e-01
       coeff_l4(43,87)=  3.4159727395e-02
       coeff_l4(43,88)= -1.1127571017e-01
       coeff_l4(43,89)= -6.9094717503e-01
       coeff_l4(43,90)= -2.9531031847e-01
       coeff_l4(43,91)=  6.3185346127e-01
       coeff_l4(43,92)= -1.0260825604e-01
       coeff_l4(43,93)=  3.7409838289e-02
       coeff_l4(43,94)=  1.9622349739e-01
       coeff_l4(43,95)= -6.8941339850e-02
       coeff_l4(43,96)= -3.8438025117e-01
       coeff_l4(43,97)=  2.6137727499e-01
       coeff_l4(43,98)= -4.5315265656e-02
       coeff_l4(43,99)=  3.6473533511e-01
       coeff_l4(43,100)= -2.4933642149e-01
       coeff_l4(43,101)= -4.3144278228e-02
       coeff_l4(43,102)= -1.5959832072e-01
       coeff_l4(43,103)= -5.0868368149e-01
       coeff_l4(43,104)=  2.3560509086e-02
       coeff_l4(43,105)= -8.2611376047e-01
       coeff_l4(44,1)= -4.4086033106e-01
       coeff_l4(44,2)= -1.4660902321e-01
       coeff_l4(44,3)= -3.4295088053e-01
       coeff_l4(44,4)=  2.6696384884e-03
       coeff_l4(44,5)= -1.1023457348e-01
       coeff_l4(44,6)= -8.1313177943e-02
       coeff_l4(44,7)= -1.1591543257e-01
       coeff_l4(44,8)= -1.4873319864e-01
       coeff_l4(44,9)=  1.6657130420e-01
       coeff_l4(44,10)=  2.9195973277e-01
       coeff_l4(44,11)=  2.2180791199e-01
       coeff_l4(44,12)=  3.0793094635e-01
       coeff_l4(44,13)=  2.7637718245e-02
       coeff_l4(44,14)=  6.3232564926e-01
       coeff_l4(44,15)=  3.4041947126e-01
       coeff_l4(44,16)=  5.8609694242e-02
       coeff_l4(44,17)=  8.0440685153e-02
       coeff_l4(44,18)= -1.4034560882e-02
       coeff_l4(44,19)=  7.0405386388e-02
       coeff_l4(44,20)=  1.5297732316e-02
       coeff_l4(44,21)= -1.0275996476e-01
       coeff_l4(44,22)=  1.8016304076e-01
       coeff_l4(44,23)=  5.4320119321e-02
       coeff_l4(44,24)= -1.2106393278e-01
       coeff_l4(44,25)=  1.4815120399e-01
       coeff_l4(44,26)=  5.5704772472e-02
       coeff_l4(44,27)= -3.2512787730e-02
       coeff_l4(44,28)=  3.6517858505e-02
       coeff_l4(44,29)= -3.1703997403e-02
       coeff_l4(44,30)=  8.9514337480e-02
       coeff_l4(44,31)= -2.8763136268e-01
       coeff_l4(44,32)=  1.4286416769e-01
       coeff_l4(44,33)=  9.5407381654e-02
       coeff_l4(44,34)=  1.1391017586e-01
       coeff_l4(44,35)=  1.3086555898e-01
       coeff_l4(44,36)= -4.1147358716e-02
       coeff_l4(44,37)= -8.3629958332e-02
       coeff_l4(44,38)=  1.4984830283e-02
       coeff_l4(44,39)= -2.7715384960e-01
       coeff_l4(44,40)=  9.6217788756e-02
       coeff_l4(44,41)= -1.7109246552e-01
       coeff_l4(44,42)=  1.5407551825e-01
       coeff_l4(44,43)= -7.7955879271e-02
       coeff_l4(44,44)= -2.5263065100e-01
       coeff_l4(44,45)=  8.5321977735e-02
       coeff_l4(44,46)= -2.5830984116e-02
       coeff_l4(44,47)=  1.9091053307e-01
       coeff_l4(44,48)=  1.2901178002e-01
       coeff_l4(44,49)=  8.5169523954e-02
       coeff_l4(44,50)=  3.4358371049e-02
       coeff_l4(44,51)=  1.6351071000e-01
       coeff_l4(44,52)= -1.1104128510e-01
       coeff_l4(44,53)=  2.7429662645e-02
       coeff_l4(44,54)= -4.3497372419e-02
       coeff_l4(44,55)=  1.1516202241e-01
       coeff_l4(44,56)= -2.8502358124e-02
       coeff_l4(44,57)= -1.5082428232e-02
       coeff_l4(44,58)= -1.8885619938e-02
       coeff_l4(44,59)=  3.5271509551e-03
       coeff_l4(44,60)= -2.1364496648e-01
       coeff_l4(44,61)= -1.6945226118e-02
       coeff_l4(44,62)=  2.6106575131e-01
       coeff_l4(44,63)= -2.1149031818e-01
       coeff_l4(44,64)=  7.5342759490e-02
       coeff_l4(44,65)= -2.2941706702e-02
       coeff_l4(44,66)=  1.9679257274e-01
       coeff_l4(44,67)=  2.5789609179e-02
       coeff_l4(44,68)=  2.9883062840e-01
       coeff_l4(44,69)=  2.5368618965e-01
       coeff_l4(44,70)=  3.3668628335e-01
       coeff_l4(44,71)= -1.5699823201e-01
       coeff_l4(44,72)= -1.7628391087e-01
       coeff_l4(44,73)= -1.1792009464e-03
       coeff_l4(44,74)=  8.5622482002e-02
       coeff_l4(44,75)=  1.1648697406e-01
       coeff_l4(44,76)=  7.2336018085e-02
       coeff_l4(44,77)= -1.4333344996e-01
       coeff_l4(44,78)=  2.9382601380e-01
       coeff_l4(44,79)=  1.3703122735e-01
       coeff_l4(44,80)=  5.5436562747e-02
       coeff_l4(44,81)=  3.6013576388e-01
       coeff_l4(44,82)=  1.0946229845e-01
       coeff_l4(44,83)= -1.4191469550e-01
       coeff_l4(44,84)= -2.6538378000e-01
       coeff_l4(44,85)=  2.0401586592e-01
       coeff_l4(44,86)= -2.5864058733e-01
       coeff_l4(44,87)= -1.5944419801e-01
       coeff_l4(44,88)= -4.3717358261e-02
       coeff_l4(44,89)=  3.1819202006e-02
       coeff_l4(44,90)=  1.2236678600e-01
       coeff_l4(44,91)=  3.1820422411e-01
       coeff_l4(44,92)= -1.5709103644e-01
       coeff_l4(44,93)= -3.6951504648e-02
       coeff_l4(44,94)=  1.2879060209e-01
       coeff_l4(44,95)=  2.2993332148e-01
       coeff_l4(44,96)=  2.6731571555e-01
       coeff_l4(44,97)=  2.8627258539e-01
       coeff_l4(44,98)=  2.9798656702e-01
       coeff_l4(44,99)= -2.1270345151e-01
       coeff_l4(44,100)= -6.5854400396e-02
       coeff_l4(44,101)= -3.6894697696e-02
       coeff_l4(44,102)= -1.2216810882e-01
       coeff_l4(44,103)= -1.0710234195e-01
       coeff_l4(44,104)= -2.2713899612e-01
       coeff_l4(44,105)= -3.5871729255e-02
       coeff_l4(45,1)=  7.2060948610e-01
       coeff_l4(45,2)= -4.1247618198e-01
       coeff_l4(45,3)=  3.8615974784e-01
       coeff_l4(45,4)= -4.9170100689e-01
       coeff_l4(45,5)= -4.3407836556e-01
       coeff_l4(45,6)= -2.4119646847e-01
       coeff_l4(45,7)=  2.1335450932e-02
       coeff_l4(45,8)= -2.4443987012e-01
       coeff_l4(45,9)= -3.5556012392e-01
       coeff_l4(45,10)=  2.0725303888e-01
       coeff_l4(45,11)=  4.3855956197e-01
       coeff_l4(45,12)=  2.5600388646e-01
       coeff_l4(45,13)= -4.4223687053e-01
       coeff_l4(45,14)= -8.3796340227e-01
       coeff_l4(45,15)= -5.0925850868e-02
       coeff_l4(45,16)= -2.3553836346e-01
       coeff_l4(45,17)= -5.3489398956e-01
       coeff_l4(45,18)= -3.6702445149e-01
       coeff_l4(45,19)=  2.0746691152e-02
       coeff_l4(45,20)=  2.9822304845e-01
       coeff_l4(45,21)= -2.9595899582e-01
       coeff_l4(45,22)= -8.2826234400e-02
       coeff_l4(45,23)= -6.0089212656e-01
       coeff_l4(45,24)=  4.8579812050e-01
       coeff_l4(45,25)= -1.3607063890e-01
       coeff_l4(45,26)=  3.4249743819e-01
       coeff_l4(45,27)= -4.2323616147e-01
       coeff_l4(45,28)= -2.7815359831e-01
       coeff_l4(45,29)= -2.9362851381e-01
       coeff_l4(45,30)=  5.6963211298e-01
       coeff_l4(45,31)=  2.6914975047e-01
       coeff_l4(45,32)=  4.4235107303e-01
       coeff_l4(45,33)=  3.0906325579e-01
       coeff_l4(45,34)=  1.5036989935e-03
       coeff_l4(45,35)= -8.9747466147e-02
       coeff_l4(45,36)= -3.4288698435e-01
       coeff_l4(45,37)=  2.7332028747e-01
       coeff_l4(45,38)= -7.0832565427e-02
       coeff_l4(45,39)=  2.8140470386e-01
       coeff_l4(45,40)= -3.9709559642e-03
       coeff_l4(45,41)=  7.3052734137e-01
       coeff_l4(45,42)=  3.4590929747e-02
       coeff_l4(45,43)= -3.9687573910e-01
       coeff_l4(45,44)= -9.9595040083e-02
       coeff_l4(45,45)= -5.6278593838e-02
       coeff_l4(45,46)= -1.9304011017e-02
       coeff_l4(45,47)=  1.2824186683e-01
       coeff_l4(45,48)= -2.3217214644e-01
       coeff_l4(45,49)= -1.0121040791e-01
       coeff_l4(45,50)=  4.4025678188e-02
       coeff_l4(45,51)=  3.7683016062e-01
       coeff_l4(45,52)=  5.3170734644e-01
       coeff_l4(45,53)=  2.5777801871e-02
       coeff_l4(45,54)=  2.0351823419e-02
       coeff_l4(45,55)=  9.9165990949e-02
       coeff_l4(45,56)=  6.9645807147e-02
       coeff_l4(45,57)= -2.4645677209e-01
       coeff_l4(45,58)= -1.7893108726e-01
       coeff_l4(45,59)= -1.1701393127e-01
       coeff_l4(45,60)= -4.1448551416e-01
       coeff_l4(45,61)= -2.4124011397e-01
       coeff_l4(45,62)= -1.4056532085e-01
       coeff_l4(45,63)= -3.9360773563e-01
       coeff_l4(45,64)= -2.9841572046e-01
       coeff_l4(45,65)= -2.6864787564e-02
       coeff_l4(45,66)=  2.3683258891e-01
       coeff_l4(45,67)=  1.3067404926e-01
       coeff_l4(45,68)= -7.9498827457e-02
       coeff_l4(45,69)= -6.0991551727e-02
       coeff_l4(45,70)= -2.6562950015e-01
       coeff_l4(45,71)= -2.6812469959e-01
       coeff_l4(45,72)=  2.0567961037e-01
       coeff_l4(45,73)=  1.2203208357e-01
       coeff_l4(45,74)=  3.2682001591e-01
       coeff_l4(45,75)=  1.2063556910e-01
       coeff_l4(45,76)=  4.9560147524e-01
       coeff_l4(45,77)=  3.1309416890e-01
       coeff_l4(45,78)= -2.2481544316e-01
       coeff_l4(45,79)= -7.1197323501e-02
       coeff_l4(45,80)= -6.0495406389e-01
       coeff_l4(45,81)=  2.9385429621e-01
       coeff_l4(45,82)= -5.4079286754e-02
       coeff_l4(45,83)= -2.6282414794e-01
       coeff_l4(45,84)=  1.7914861441e-01
       coeff_l4(45,85)= -2.8381469846e-01
       coeff_l4(45,86)= -1.4425154030e-01
       coeff_l4(45,87)=  2.5768858194e-01
       coeff_l4(45,88)=  1.8802741542e-02
       coeff_l4(45,89)=  6.9285228848e-02
       coeff_l4(45,90)= -2.1843835711e-01
       coeff_l4(45,91)= -2.4575930089e-02
       coeff_l4(45,92)=  6.2769994140e-02
       coeff_l4(45,93)=  1.3205921277e-02
       coeff_l4(45,94)=  1.0536814928e+00
       coeff_l4(45,95)= -4.2690446973e-01
       coeff_l4(45,96)=  1.2129963189e-01
       coeff_l4(45,97)=  1.0153839588e+00
       coeff_l4(45,98)=  2.6274690032e-01
       coeff_l4(45,99)=  1.0724420846e-01
       coeff_l4(45,100)= -3.9322903752e-01
       coeff_l4(45,101)= -6.7974799871e-01
       coeff_l4(45,102)= -5.0097137690e-01
       coeff_l4(45,103)=  1.7047618749e-03
       coeff_l4(45,104)= -1.3208292425e-01
       coeff_l4(45,105)= -3.9426624775e-01
       coeff_l4(46,1)=  7.3819851875e-01
       coeff_l4(46,2)=  4.5086014271e-01
       coeff_l4(46,3)=  5.5751729012e-01
       coeff_l4(46,4)=  6.1193919182e-01
       coeff_l4(46,5)=  4.3277391791e-01
       coeff_l4(46,6)=  5.7836526632e-01
       coeff_l4(46,7)=  6.0637813807e-01
       coeff_l4(46,8)=  3.6848920584e-01
       coeff_l4(46,9)=  7.9378110170e-01
       coeff_l4(46,10)=  7.9686176777e-01
       coeff_l4(46,11)=  5.2114397287e-01
       coeff_l4(46,12)=  8.0785036087e-01
       coeff_l4(46,13)=  5.7335972786e-01
       coeff_l4(46,14)=  6.5129441023e-01
       coeff_l4(46,15)= -3.6962932348e-01
       coeff_l4(46,16)= -1.2341281399e-02
       coeff_l4(46,17)= -7.4336841702e-02
       coeff_l4(46,18)=  2.0989383757e-01
       coeff_l4(46,19)= -9.0090874583e-03
       coeff_l4(46,20)= -5.1025670767e-01
       coeff_l4(46,21)=  2.4018788338e-01
       coeff_l4(46,22)= -1.8209201097e-01
       coeff_l4(46,23)=  6.1957668513e-02
       coeff_l4(46,24)= -2.7092131972e-01
       coeff_l4(46,25)= -1.0189710557e-01
       coeff_l4(46,26)= -2.0117451251e-01
       coeff_l4(46,27)=  3.8296841085e-02
       coeff_l4(46,28)=  8.5288062692e-02
       coeff_l4(46,29)=  1.2650576234e-01
       coeff_l4(46,30)= -2.1243020892e-01
       coeff_l4(46,31)=  2.6366770267e-01
       coeff_l4(46,32)=  4.7399912030e-02
       coeff_l4(46,33)=  3.1808969378e-01
       coeff_l4(46,34)=  9.4721421599e-02
       coeff_l4(46,35)=  1.1452477425e-01
       coeff_l4(46,36)= -1.8345777690e-01
       coeff_l4(46,37)=  2.0041163266e-01
       coeff_l4(46,38)=  1.6709835827e-01
       coeff_l4(46,39)=  2.0822634920e-02
       coeff_l4(46,40)= -4.1908763349e-02
       coeff_l4(46,41)= -2.4475625157e-01
       coeff_l4(46,42)= -4.8343798518e-01
       coeff_l4(46,43)= -1.9244368374e-01
       coeff_l4(46,44)=  4.7543842345e-02
       coeff_l4(46,45)=  3.5925969481e-01
       coeff_l4(46,46)=  4.0277349949e-01
       coeff_l4(46,47)=  5.9259802103e-02
       coeff_l4(46,48)=  2.1847510338e-01
       coeff_l4(46,49)= -6.5204181010e-04
       coeff_l4(46,50)=  1.8265895545e-01
       coeff_l4(46,51)= -4.1188618541e-01
       coeff_l4(46,52)=  2.0973052084e-01
       coeff_l4(46,53)= -4.7148960829e-01
       coeff_l4(46,54)=  1.1052194238e-01
       coeff_l4(46,55)=  1.7884527147e-01
       coeff_l4(46,56)= -4.8811636865e-02
       coeff_l4(46,57)=  1.3689456880e-01
       coeff_l4(46,58)=  3.2162380219e-01
       coeff_l4(46,59)=  7.4459545314e-02
       coeff_l4(46,60)=  2.1115447581e-01
       coeff_l4(46,61)=  4.1230893135e-01
       coeff_l4(46,62)=  7.0736795664e-02
       coeff_l4(46,63)= -3.5718038678e-02
       coeff_l4(46,64)= -3.4422792494e-02
       coeff_l4(46,65)=  2.8591090813e-02
       coeff_l4(46,66)= -7.4584797025e-02
       coeff_l4(46,67)=  3.4170553088e-01
       coeff_l4(46,68)=  7.7643446624e-02
       coeff_l4(46,69)= -9.8568134010e-02
       coeff_l4(46,70)=  9.6766397357e-02
       coeff_l4(46,71)=  1.8160586059e-01
       coeff_l4(46,72)=  3.6071889102e-02
       coeff_l4(46,73)=  2.1631723642e-01
       coeff_l4(46,74)= -8.8103383780e-02
       coeff_l4(46,75)=  5.0504095852e-02
       coeff_l4(46,76)= -2.5938078761e-02
       coeff_l4(46,77)=  2.1422129124e-02
       coeff_l4(46,78)= -2.8674265742e-01
       coeff_l4(46,79)=  1.5886391699e-01
       coeff_l4(46,80)= -3.7249105517e-03
       coeff_l4(46,81)=  1.2006509118e-02
       coeff_l4(46,82)=  1.8324495852e-01
       coeff_l4(46,83)=  1.5359559655e-01
       coeff_l4(46,84)= -1.6153654456e-01
       coeff_l4(46,85)= -5.3941190243e-01
       coeff_l4(46,86)=  2.4991378188e-01
       coeff_l4(46,87)= -4.3722966313e-01
       coeff_l4(46,88)= -9.4428732991e-02
       coeff_l4(46,89)= -3.8253754377e-02
       coeff_l4(46,90)=  2.2075732052e-01
       coeff_l4(46,91)= -5.8295994997e-02
       coeff_l4(46,92)=  4.7913786024e-02
       coeff_l4(46,93)= -3.0259920284e-02
       coeff_l4(46,94)= -7.0910602808e-01
       coeff_l4(46,95)=  3.4865868092e-01
       coeff_l4(46,96)=  2.5168508291e-01
       coeff_l4(46,97)= -4.8839035630e-01
       coeff_l4(46,98)=  2.0379127562e-01
       coeff_l4(46,99)=  2.1430487931e-01
       coeff_l4(46,100)=  2.6208594441e-01
       coeff_l4(46,101)=  9.4131059945e-02
       coeff_l4(46,102)= -1.4884462953e-01
       coeff_l4(46,103)=  1.1865263432e-01
       coeff_l4(46,104)=  7.9217642546e-02
       coeff_l4(46,105)=  1.5157781541e-01
       coeff_l4(47,1)=  5.5682426691e-01
       coeff_l4(47,2)=  3.3747145534e-01
       coeff_l4(47,3)=  6.5971732140e-01
       coeff_l4(47,4)=  6.1587989330e-01
       coeff_l4(47,5)=  4.9164435267e-01
       coeff_l4(47,6)=  4.5275893807e-01
       coeff_l4(47,7)=  4.4558838010e-01
       coeff_l4(47,8)=  5.5897086859e-01
       coeff_l4(47,9)= -3.6638894677e-01
       coeff_l4(47,10)= -3.3154365420e-01
       coeff_l4(47,11)= -2.8383636475e-01
       coeff_l4(47,12)=  8.4325984120e-02
       coeff_l4(47,13)=  2.2910608351e-01
       coeff_l4(47,14)= -3.7699139118e-01
       coeff_l4(47,15)=  3.9312350564e-03
       coeff_l4(47,16)= -1.0867518932e-01
       coeff_l4(47,17)= -1.3271404803e-01
       coeff_l4(47,18)= -2.2713501751e-01
       coeff_l4(47,19)= -2.8949734569e-01
       coeff_l4(47,20)= -2.4972863495e-01
       coeff_l4(47,21)= -7.5706861913e-02
       coeff_l4(47,22)= -1.5263524652e-01
       coeff_l4(47,23)=  2.0380894840e-01
       coeff_l4(47,24)= -1.7576968670e-01
       coeff_l4(47,25)=  1.3705834746e-01
       coeff_l4(47,26)= -2.6160132885e-01
       coeff_l4(47,27)=  1.1823083460e-01
       coeff_l4(47,28)= -2.7699294686e-01
       coeff_l4(47,29)=  3.4192752838e-01
       coeff_l4(47,30)= -2.2007304430e-01
       coeff_l4(47,31)= -1.8682700396e-01
       coeff_l4(47,32)=  3.8506248966e-03
       coeff_l4(47,33)=  1.5615634620e-01
       coeff_l4(47,34)= -4.7511272132e-02
       coeff_l4(47,35)= -2.2551891208e-01
       coeff_l4(47,36)=  8.1452213228e-02
       coeff_l4(47,37)=  3.9167601615e-02
       coeff_l4(47,38)= -1.5711946785e-01
       coeff_l4(47,39)= -5.7767022401e-02
       coeff_l4(47,40)= -1.1574331671e-01
       coeff_l4(47,41)=  2.1773407236e-02
       coeff_l4(47,42)= -4.7556694597e-02
       coeff_l4(47,43)= -2.2707896307e-02
       coeff_l4(47,44)=  3.4320154786e-01
       coeff_l4(47,45)= -9.9995113909e-02
       coeff_l4(47,46)=  2.9169062153e-02
       coeff_l4(47,47)= -2.0407265425e-01
       coeff_l4(47,48)= -8.1575736403e-02
       coeff_l4(47,49)= -1.0098863393e-01
       coeff_l4(47,50)=  4.5605120063e-01
       coeff_l4(47,51)=  4.0684479475e-01
       coeff_l4(47,52)=  3.5559743643e-02
       coeff_l4(47,53)=  2.6560080051e-01
       coeff_l4(47,54)= -8.2183167338e-02
       coeff_l4(47,55)=  2.5839075446e-01
       coeff_l4(47,56)= -2.2287331522e-01
       coeff_l4(47,57)=  2.1191671491e-01
       coeff_l4(47,58)=  2.5919565558e-01
       coeff_l4(47,59)= -3.0297806859e-01
       coeff_l4(47,60)= -7.8648693860e-02
       coeff_l4(47,61)= -3.9611762762e-01
       coeff_l4(47,62)= -3.4997785091e-01
       coeff_l4(47,63)=  8.2813195884e-02
       coeff_l4(47,64)= -1.7034201324e-01
       coeff_l4(47,65)= -7.5420245528e-02
       coeff_l4(47,66)= -2.8916180134e-02
       coeff_l4(47,67)= -1.0506680608e-01
       coeff_l4(47,68)= -8.5952669382e-02
       coeff_l4(47,69)= -3.7660697103e-01
       coeff_l4(47,70)= -3.5165938735e-01
       coeff_l4(47,71)=  2.6823163033e-01
       coeff_l4(47,72)= -2.1416958421e-02
       coeff_l4(47,73)= -5.7014338672e-03
       coeff_l4(47,74)= -2.1059423685e-02
       coeff_l4(47,75)=  5.7991053909e-02
       coeff_l4(47,76)=  2.7325227857e-01
       coeff_l4(47,77)=  1.1057133228e-01
       coeff_l4(47,78)=  1.7040055990e-01
       coeff_l4(47,79)= -5.0129723549e-01
       coeff_l4(47,80)=  7.2926416993e-02
       coeff_l4(47,81)= -2.1098296344e-01
       coeff_l4(47,82)= -1.3630783558e-01
       coeff_l4(47,83)= -1.1187770218e-01
       coeff_l4(47,84)=  9.7626604140e-02
       coeff_l4(47,85)= -1.2249392271e-01
       coeff_l4(47,86)= -1.8741331995e-02
       coeff_l4(47,87)=  2.0167877898e-02
       coeff_l4(47,88)=  7.6110824943e-02
       coeff_l4(47,89)=  3.6491748691e-01
       coeff_l4(47,90)=  9.6040457487e-02
       coeff_l4(47,91)=  2.1118676662e-01
       coeff_l4(47,92)=  2.1997103095e-01
       coeff_l4(47,93)=  6.4573034644e-02
       coeff_l4(47,94)=  1.8661715090e-01
       coeff_l4(47,95)=  1.9178444147e-01
       coeff_l4(47,96)=  2.6088517159e-02
       coeff_l4(47,97)=  2.9497477412e-01
       coeff_l4(47,98)= -2.2226992995e-02
       coeff_l4(47,99)=  8.0161668360e-02
       coeff_l4(47,100)=  8.6158387363e-02
       coeff_l4(47,101)=  1.9140127301e-01
       coeff_l4(47,102)=  1.7684935033e-01
       coeff_l4(47,103)= -7.7704526484e-02
       coeff_l4(47,104)=  4.6146776527e-02
       coeff_l4(47,105)= -2.6163029671e-01
       coeff_l4(48,1)=  1.1957510710e+00
       coeff_l4(48,2)=  1.3019330502e+00
       coeff_l4(48,3)=  1.0227143764e+00
       coeff_l4(48,4)=  1.1816359758e+00
       coeff_l4(48,5)=  1.1807858944e+00
       coeff_l4(48,6)=  7.3498994112e-01
       coeff_l4(48,7)=  7.5967943668e-01
       coeff_l4(48,8)=  7.2985076904e-01
       coeff_l4(48,9)=  7.1119733155e-02
       coeff_l4(48,10)=  3.4320005774e-01
       coeff_l4(48,11)=  6.8135544658e-02
       coeff_l4(48,12)= -1.7766262591e-01
       coeff_l4(48,13)= -9.5201700926e-02
       coeff_l4(48,14)=  3.7112200260e-01
       coeff_l4(48,15)= -2.1768261492e-01
       coeff_l4(48,16)= -3.0184540153e-01
       coeff_l4(48,17)= -4.2126871645e-02
       coeff_l4(48,18)=  1.8453146517e-01
       coeff_l4(48,19)=  1.1981358379e-01
       coeff_l4(48,20)=  8.7198555470e-02
       coeff_l4(48,21)=  1.2450202554e-01
       coeff_l4(48,22)= -2.7020029724e-02
       coeff_l4(48,23)= -9.8813034594e-02
       coeff_l4(48,24)=  2.9780685902e-02
       coeff_l4(48,25)=  6.6796116531e-02
       coeff_l4(48,26)=  2.5101915002e-01
       coeff_l4(48,27)= -3.1890928745e-01
       coeff_l4(48,28)=  3.0896955729e-01
       coeff_l4(48,29)=  1.0266762227e-01
       coeff_l4(48,30)= -2.9864028096e-02
       coeff_l4(48,31)=  3.5903912783e-01
       coeff_l4(48,32)=  2.9135382175e-01
       coeff_l4(48,33)= -3.5203528404e-01
       coeff_l4(48,34)= -5.6602682918e-02
       coeff_l4(48,35)= -1.0471962392e-01
       coeff_l4(48,36)= -1.2585332990e-01
       coeff_l4(48,37)=  2.0472115278e-01
       coeff_l4(48,38)= -6.5225601196e-02
       coeff_l4(48,39)=  1.8639947474e-01
       coeff_l4(48,40)= -2.8252702206e-02
       coeff_l4(48,41)=  1.0106331110e-01
       coeff_l4(48,42)=  3.3573406935e-01
       coeff_l4(48,43)= -1.4764954150e-01
       coeff_l4(48,44)=  2.8120255470e-01
       coeff_l4(48,45)= -5.5454108864e-02
       coeff_l4(48,46)= -1.6609098017e-01
       coeff_l4(48,47)= -2.3086237907e-01
       coeff_l4(48,48)=  1.1692938209e-01
       coeff_l4(48,49)= -1.7259402201e-02
       coeff_l4(48,50)= -2.5819912553e-01
       coeff_l4(48,51)=  1.6553136706e-01
       coeff_l4(48,52)=  1.1343222111e-01
       coeff_l4(48,53)=  3.6316862702e-01
       coeff_l4(48,54)=  1.4961373806e-01
       coeff_l4(48,55)= -1.7640164495e-01
       coeff_l4(48,56)=  7.5722247362e-02
       coeff_l4(48,57)=  1.3800098002e-01
       coeff_l4(48,58)=  2.2189085186e-01
       coeff_l4(48,59)=  7.5606280006e-03
       coeff_l4(48,60)= -2.2447496653e-01
       coeff_l4(48,61)= -2.0120391250e-01
       coeff_l4(48,62)=  1.2006884068e-01
       coeff_l4(48,63)=  3.5405713320e-01
       coeff_l4(48,64)=  1.1971287429e-01
       coeff_l4(48,65)=  1.2908366323e-01
       coeff_l4(48,66)= -2.3451648653e-01
       coeff_l4(48,67)= -4.6476125717e-02
       coeff_l4(48,68)= -2.3903505504e-01
       coeff_l4(48,69)= -1.0790339857e-01
       coeff_l4(48,70)=  3.3829279244e-02
       coeff_l4(48,71)= -1.2727759778e-02
       coeff_l4(48,72)=  3.7722572684e-01
       coeff_l4(48,73)=  8.6823813617e-03
       coeff_l4(48,74)= -2.3393074051e-02
       coeff_l4(48,75)=  1.2120576203e-01
       coeff_l4(48,76)=  4.0693065524e-01
       coeff_l4(48,77)= -9.8453439772e-02
       coeff_l4(48,78)= -2.8233754635e-01
       coeff_l4(48,79)= -1.9537068903e-01
       coeff_l4(48,80)=  1.5182882547e-01
       coeff_l4(48,81)= -1.1101150885e-02
       coeff_l4(48,82)= -1.1647079885e-01
       coeff_l4(48,83)= -3.6650497466e-02
       coeff_l4(48,84)= -1.7667354643e-01
       coeff_l4(48,85)= -3.6504742503e-01
       coeff_l4(48,86)= -4.0009651333e-02
       coeff_l4(48,87)= -1.3301999867e-01
       coeff_l4(48,88)=  1.7003199086e-02
       coeff_l4(48,89)=  1.5769484639e-01
       coeff_l4(48,90)=  1.0660748929e-02
       coeff_l4(48,91)= -1.3921971619e-01
       coeff_l4(48,92)= -9.2397846282e-02
       coeff_l4(48,93)= -1.6327327490e-01
       coeff_l4(48,94)=  4.4366266578e-02
       coeff_l4(48,95)= -1.5770150721e-01
       coeff_l4(48,96)= -1.0602200776e-01
       coeff_l4(48,97)=  1.1050651222e-01
       coeff_l4(48,98)=  6.3371174037e-02
       coeff_l4(48,99)=  1.7704416811e-01
       coeff_l4(48,100)= -5.2268565632e-03
       coeff_l4(48,101)=  6.8095654249e-02
       coeff_l4(48,102)= -3.8194563240e-02
       coeff_l4(48,103)=  2.9085034505e-02
       coeff_l4(48,104)= -4.3579310179e-02
       coeff_l4(48,105)=  9.2775739729e-02
       coeff_l4(49,1)=  2.9882889986e-01
       coeff_l4(49,2)=  4.3239477277e-01
       coeff_l4(49,3)=  4.3998399377e-01
       coeff_l4(49,4)=  2.9234543443e-01
       coeff_l4(49,5)=  1.6442734003e-01
       coeff_l4(49,6)=  5.4734206200e-01
       coeff_l4(49,7)= -2.8795641661e-01
       coeff_l4(49,8)=  1.9174076617e-01
       coeff_l4(49,9)= -3.3103256226e+00
       coeff_l4(49,10)= -2.3495552540e+00
       coeff_l4(49,11)= -5.7401041985e+00
       coeff_l4(49,12)= -5.8005070686e+00
       coeff_l4(49,13)= -5.1663560867e+00
       coeff_l4(49,14)= -8.9858603477e-01
       coeff_l4(49,15)=  2.5905039907e-01
       coeff_l4(49,16)=  1.2675027549e-01
       coeff_l4(49,17)=  1.7413498461e-01
       coeff_l4(49,18)= -1.6325771809e-02
       coeff_l4(49,19)=  3.7117695808e-01
       coeff_l4(49,20)= -1.4447623491e-01
       coeff_l4(49,21)= -7.4487403035e-02
       coeff_l4(49,22)=  3.3410783857e-02
       coeff_l4(49,23)= -5.2367323637e-01
       coeff_l4(49,24)=  2.8300365806e-01
       coeff_l4(49,25)= -1.5988968313e-01
       coeff_l4(49,26)= -2.1178659797e-01
       coeff_l4(49,27)= -9.5822647214e-02
       coeff_l4(49,28)=  2.4517866969e-01
       coeff_l4(49,29)= -8.1764727831e-02
       coeff_l4(49,30)=  1.6973896418e-03
       coeff_l4(49,31)=  4.0972030163e-01
       coeff_l4(49,32)= -8.1403441727e-02
       coeff_l4(49,33)= -3.8885310292e-01
       coeff_l4(49,34)=  2.6782804728e-01
       coeff_l4(49,35)= -2.0421560109e-01
       coeff_l4(49,36)=  2.1419219673e-01
       coeff_l4(49,37)= -2.3171113431e-01
       coeff_l4(49,38)=  2.0711991191e-01
       coeff_l4(49,39)= -4.2763259262e-02
       coeff_l4(49,40)=  1.8952886760e-01
       coeff_l4(49,41)= -3.0222457647e-01
       coeff_l4(49,42)= -2.1367941052e-02
       coeff_l4(49,43)= -6.0328364372e-01
       coeff_l4(49,44)= -1.0359185748e-02
       coeff_l4(49,45)= -4.9701303244e-01
       coeff_l4(49,46)=  3.4242406487e-01
       coeff_l4(49,47)=  2.7881604433e-01
       coeff_l4(49,48)=  7.4612587690e-02
       coeff_l4(49,49)=  8.5076108575e-02
       coeff_l4(49,50)=  8.6182542145e-02
       coeff_l4(49,51)=  4.0447976440e-02
       coeff_l4(49,52)=  4.6330943704e-02
       coeff_l4(49,53)= -2.2961360216e-01
       coeff_l4(49,54)= -1.6848807037e-01
       coeff_l4(49,55)= -6.0002531856e-02
       coeff_l4(49,56)=  2.2555297613e-01
       coeff_l4(49,57)=  4.2923542857e-01
       coeff_l4(49,58)=  2.7457514778e-02
       coeff_l4(49,59)=  3.2014784217e-01
       coeff_l4(49,60)=  1.3892014325e-01
       coeff_l4(49,61)=  2.1589590609e-01
       coeff_l4(49,62)=  1.2737444043e-01
       coeff_l4(49,63)= -1.9187724590e-01
       coeff_l4(49,64)=  4.2091047764e-01
       coeff_l4(49,65)= -2.9045358300e-01
       coeff_l4(49,66)= -8.7445840240e-02
       coeff_l4(49,67)=  1.3776628673e-01
       coeff_l4(49,68)=  1.7097806931e-01
       coeff_l4(49,69)= -1.8739905208e-02
       coeff_l4(49,70)= -2.2747257352e-01
       coeff_l4(49,71)=  1.8723312020e-01
       coeff_l4(49,72)=  1.0174417496e+00
       coeff_l4(49,73)= -1.1032824405e-02
       coeff_l4(49,74)=  3.7472093105e-01
       coeff_l4(49,75)=  4.8845309019e-01
       coeff_l4(49,76)=  2.8327882290e-01
       coeff_l4(49,77)= -3.2699570060e-01
       coeff_l4(49,78)=  2.2309811413e-01
       coeff_l4(49,79)= -5.2478569746e-01
       coeff_l4(49,80)= -6.2754660845e-02
       coeff_l4(49,81)= -2.1752005816e-01
       coeff_l4(49,82)=  9.8124854267e-02
       coeff_l4(49,83)= -3.9481568336e-01
       coeff_l4(49,84)=  5.7286605239e-02
       coeff_l4(49,85)=  8.6582072079e-02
       coeff_l4(49,86)=  1.1713054776e-01
       coeff_l4(49,87)=  9.5939479768e-02
       coeff_l4(49,88)=  3.2005277276e-01
       coeff_l4(49,89)= -6.9802904129e-01
       coeff_l4(49,90)= -3.2406741381e-01
       coeff_l4(49,91)=  1.2029156089e-01
       coeff_l4(49,92)= -2.5428217649e-01
       coeff_l4(49,93)= -6.7451637983e-01
       coeff_l4(49,94)= -3.5361201763e+00
       coeff_l4(49,95)=  4.6239566803e-01
       coeff_l4(49,96)= -2.0998871326e+00
       coeff_l4(49,97)= -2.0554211140e+00
       coeff_l4(49,98)=  6.1340034008e-01
       coeff_l4(49,99)=  1.8640181422e-01
       coeff_l4(49,100)=  8.2351469994e-01
       coeff_l4(49,101)= -5.0705736876e-01
       coeff_l4(49,102)= -3.1705822796e-02
       coeff_l4(49,103)=  4.5632925630e-01
       coeff_l4(49,104)=  6.9930309057e-01
       coeff_l4(49,105)=  2.5301194191e-01
       coeff_l4(50,1)=  1.1077234745e+00
       coeff_l4(50,2)=  7.5614851713e-01
       coeff_l4(50,3)=  7.1391350031e-01
       coeff_l4(50,4)=  6.2052685022e-01
       coeff_l4(50,5)=  8.3239096403e-01
       coeff_l4(50,6)=  6.4283454418e-01
       coeff_l4(50,7)=  1.7622429132e-01
       coeff_l4(50,8)=  8.2475316525e-01
       coeff_l4(50,9)=  2.6561862230e-01
       coeff_l4(50,10)=  1.6654722393e-01
       coeff_l4(50,11)=  1.3952568173e-01
       coeff_l4(50,12)= -1.3332395256e-01
       coeff_l4(50,13)=  2.9803800583e-01
       coeff_l4(50,14)= -3.8746967912e-02
       coeff_l4(50,15)= -1.4773310721e-01
       coeff_l4(50,16)=  2.2609372437e-01
       coeff_l4(50,17)=  1.1606093496e-01
       coeff_l4(50,18)= -1.4548854530e-01
       coeff_l4(50,19)= -4.7021076083e-02
       coeff_l4(50,20)=  3.2519198954e-02
       coeff_l4(50,21)=  3.4843137860e-01
       coeff_l4(50,22)=  2.2975249588e-01
       coeff_l4(50,23)= -3.9742863178e-01
       coeff_l4(50,24)=  3.0560076237e-01
       coeff_l4(50,25)=  1.2368912995e-01
       coeff_l4(50,26)= -4.2969811708e-02
       coeff_l4(50,27)= -4.7968804836e-02
       coeff_l4(50,28)= -2.3783241212e-01
       coeff_l4(50,29)=  1.2570057809e-01
       coeff_l4(50,30)= -3.5320362449e-01
       coeff_l4(50,31)=  1.2888637185e-01
       coeff_l4(50,32)=  1.3456638157e-01
       coeff_l4(50,33)= -3.6382603645e-01
       coeff_l4(50,34)= -3.1507572532e-01
       coeff_l4(50,35)=  6.2609622255e-03
       coeff_l4(50,36)=  1.9978296012e-02
       coeff_l4(50,37)= -6.7282984965e-03
       coeff_l4(50,38)=  1.5576204658e-01
       coeff_l4(50,39)=  7.9005874693e-02
       coeff_l4(50,40)= -9.3527160585e-02
       coeff_l4(50,41)= -1.9692660868e-01
       coeff_l4(50,42)=  2.8810107615e-03
       coeff_l4(50,43)=  7.9288743436e-03
       coeff_l4(50,44)=  1.2926740944e-01
       coeff_l4(50,45)= -2.0819911361e-01
       coeff_l4(50,46)= -3.7863022089e-01
       coeff_l4(50,47)=  4.2043146491e-01
       coeff_l4(50,48)=  1.4596015215e-01
       coeff_l4(50,49)=  2.8845217824e-01
       coeff_l4(50,50)=  1.2287457474e-02
       coeff_l4(50,51)= -1.6421645880e-01
       coeff_l4(50,52)= -1.2468355149e-01
       coeff_l4(50,53)= -2.3307281733e-01
       coeff_l4(50,54)= -3.9456818253e-02
       coeff_l4(50,55)=  4.8616304994e-01
       coeff_l4(50,56)= -4.5768518001e-02
       coeff_l4(50,57)=  8.8520295918e-02
       coeff_l4(50,58)=  3.6333523691e-02
       coeff_l4(50,59)= -2.3287515342e-01
       coeff_l4(50,60)= -1.0149989277e-01
       coeff_l4(50,61)=  8.3626791835e-02
       coeff_l4(50,62)= -2.6032308117e-02
       coeff_l4(50,63)=  5.1059890538e-02
       coeff_l4(50,64)=  9.6218764782e-02
       coeff_l4(50,65)= -8.0681607127e-02
       coeff_l4(50,66)=  2.8515881300e-01
       coeff_l4(50,67)= -1.8885353208e-01
       coeff_l4(50,68)= -3.2817136496e-02
       coeff_l4(50,69)=  1.6325208545e-01
       coeff_l4(50,70)=  1.3380022347e-01
       coeff_l4(50,71)=  3.8533478975e-02
       coeff_l4(50,72)= -4.5822210610e-02
       coeff_l4(50,73)= -1.3455049694e-01
       coeff_l4(50,74)=  3.6110389978e-02
       coeff_l4(50,75)=  3.6899319291e-01
       coeff_l4(50,76)=  4.9050807953e-02
       coeff_l4(50,77)= -5.4123014212e-02
       coeff_l4(50,78)=  4.7830935568e-02
       coeff_l4(50,79)= -3.4533682466e-01
       coeff_l4(50,80)=  1.1632764339e-01
       coeff_l4(50,81)=  2.6890325546e-01
       coeff_l4(50,82)= -1.2612232566e-01
       coeff_l4(50,83)= -2.0166935027e-01
       coeff_l4(50,84)= -9.8487205803e-02
       coeff_l4(50,85)= -3.3130804077e-03
       coeff_l4(50,86)=  2.0004931092e-01
       coeff_l4(50,87)= -2.4651768804e-01
       coeff_l4(50,88)=  9.8481804132e-02
       coeff_l4(50,89)= -1.8324425817e-01
       coeff_l4(50,90)= -2.7920552529e-03
       coeff_l4(50,91)= -2.5514388084e-01
       coeff_l4(50,92)=  2.3490263522e-01
       coeff_l4(50,93)= -1.6552738845e-01
       coeff_l4(50,94)=  4.0554353595e-01
       coeff_l4(50,95)=  1.3679745793e-01
       coeff_l4(50,96)= -4.0550627746e-03
       coeff_l4(50,97)= -5.3348030895e-02
       coeff_l4(50,98)= -2.0238716900e-01
       coeff_l4(50,99)= -5.0245895982e-02
       coeff_l4(50,100)=  8.9793466032e-02
       coeff_l4(50,101)=  1.1965122819e-01
       coeff_l4(50,102)= -1.6962809861e-01
       coeff_l4(50,103)= -6.2733568251e-02
       coeff_l4(50,104)=  9.3296319246e-02
       coeff_l4(50,105)= -1.1343713105e-01
       coeff_l4(51,1)= -1.2510854006e+00
       coeff_l4(51,2)= -9.8626172543e-01
       coeff_l4(51,3)= -6.9019144773e-01
       coeff_l4(51,4)= -8.4610402584e-01
       coeff_l4(51,5)= -5.6943202019e-01
       coeff_l4(51,6)= -8.2698357105e-01
       coeff_l4(51,7)= -3.3968892694e-01
       coeff_l4(51,8)= -8.7221282721e-01
       coeff_l4(51,9)= -4.1453874111e-01
       coeff_l4(51,10)= -5.4417818785e-01
       coeff_l4(51,11)= -1.5227156878e-01
       coeff_l4(51,12)= -2.2861143947e-01
       coeff_l4(51,13)= -9.9591299891e-02
       coeff_l4(51,14)= -3.0801182985e-01
       coeff_l4(51,15)=  1.4315928519e-01
       coeff_l4(51,16)= -5.4428435862e-02
       coeff_l4(51,17)= -6.9577261806e-02
       coeff_l4(51,18)= -1.6316650808e-01
       coeff_l4(51,19)= -4.0164086968e-02
       coeff_l4(51,20)=  1.3744303584e-01
       coeff_l4(51,21)= -3.4805437922e-01
       coeff_l4(51,22)= -1.6890294850e-01
       coeff_l4(51,23)=  1.8814609945e-01
       coeff_l4(51,24)= -2.0326441526e-01
       coeff_l4(51,25)= -1.0236977041e-01
       coeff_l4(51,26)=  8.7825931609e-02
       coeff_l4(51,27)=  2.6373901963e-01
       coeff_l4(51,28)= -1.8804976344e-01
       coeff_l4(51,29)=  5.8570101857e-02
       coeff_l4(51,30)= -1.6553714871e-02
       coeff_l4(51,31)= -2.8788867593e-01
       coeff_l4(51,32)=  1.1953545362e-01
       coeff_l4(51,33)= -2.0744336769e-02
       coeff_l4(51,34)=  1.7573139071e-01
       coeff_l4(51,35)=  1.9609791040e-01
       coeff_l4(51,36)= -6.5890089609e-03
       coeff_l4(51,37)=  1.4044328034e-01
       coeff_l4(51,38)=  1.1277356744e-01
       coeff_l4(51,39)= -1.5816356242e-01
       coeff_l4(51,40)= -7.9339511693e-02
       coeff_l4(51,41)=  1.2432666868e-01
       coeff_l4(51,42)=  2.0688256621e-01
       coeff_l4(51,43)= -2.4384979904e-01
       coeff_l4(51,44)= -1.3615255058e-01
       coeff_l4(51,45)= -6.7649908364e-02
       coeff_l4(51,46)=  1.2020774931e-01
       coeff_l4(51,47)= -1.0369911045e-01
       coeff_l4(51,48)= -2.1288999915e-01
       coeff_l4(51,49)=  2.1121424437e-01
       coeff_l4(51,50)= -2.7215507627e-01
       coeff_l4(51,51)= -2.5935439393e-02
       coeff_l4(51,52)= -3.3417192753e-03
       coeff_l4(51,53)=  1.4702464640e-01
       coeff_l4(51,54)=  2.4011332542e-02
       coeff_l4(51,55)= -3.7384468317e-01
       coeff_l4(51,56)=  1.2267381698e-01
       coeff_l4(51,57)= -3.5405065864e-02
       coeff_l4(51,58)= -1.5879562497e-01
       coeff_l4(51,59)= -2.2928416729e-01
       coeff_l4(51,60)=  1.1639263481e-01
       coeff_l4(51,61)= -1.4005598426e-01
       coeff_l4(51,62)= -7.8239649534e-02
       coeff_l4(51,63)= -3.2217627764e-01
       coeff_l4(51,64)=  1.7543731630e-01
       coeff_l4(51,65)= -3.4553375840e-01
       coeff_l4(51,66)= -1.0462801158e-01
       coeff_l4(51,67)= -1.3117076829e-02
       coeff_l4(51,68)= -1.0248643160e-01
       coeff_l4(51,69)= -1.4864278026e-02
       coeff_l4(51,70)= -1.3142368197e-01
       coeff_l4(51,71)= -4.3367370381e-04
       coeff_l4(51,72)=  1.8495872617e-01
       coeff_l4(51,73)=  3.0327519774e-01
       coeff_l4(51,74)=  7.6616361737e-02
       coeff_l4(51,75)= -6.9820724428e-02
       coeff_l4(51,76)= -1.1175215244e-01
       coeff_l4(51,77)=  1.5998525918e-01
       coeff_l4(51,78)= -1.0876052827e-01
       coeff_l4(51,79)=  3.6486750841e-01
       coeff_l4(51,80)= -4.4886502437e-03
       coeff_l4(51,81)= -4.2502760887e-02
       coeff_l4(51,82)=  1.0257086158e-01
       coeff_l4(51,83)=  3.3574399352e-01
       coeff_l4(51,84)= -7.6214186847e-02
       coeff_l4(51,85)= -1.4716516435e-01
       coeff_l4(51,86)=  1.0741847754e-01
       coeff_l4(51,87)= -5.2011210471e-02
       coeff_l4(51,88)=  5.7328134775e-02
       coeff_l4(51,89)= -6.8125493824e-02
       coeff_l4(51,90)=  1.0406496003e-02
       coeff_l4(51,91)= -7.6021365821e-02
       coeff_l4(51,92)= -3.4107971191e-01
       coeff_l4(51,93)=  3.1470763683e-01
       coeff_l4(51,94)=  2.7277576923e-01
       coeff_l4(51,95)= -1.8717715517e-02
       coeff_l4(51,96)=  7.2170548141e-02
       coeff_l4(51,97)= -2.6186665520e-02
       coeff_l4(51,98)= -1.4551423490e-01
       coeff_l4(51,99)=  7.7392145991e-02
       coeff_l4(51,100)=  1.0310338438e-01
       coeff_l4(51,101)=  1.3998168707e-01
       coeff_l4(51,102)= -2.2082793713e-01
       coeff_l4(51,103)= -9.2547170818e-02
       coeff_l4(51,104)= -1.1961992830e-01
       coeff_l4(51,105)=  1.4324706793e-01
       coeff_l4(52,1)= -3.9791950583e-01
       coeff_l4(52,2)= -2.9466924071e-01
       coeff_l4(52,3)= -4.6703696251e-01
       coeff_l4(52,4)= -4.1871601343e-01
       coeff_l4(52,5)= -5.8493906260e-01
       coeff_l4(52,6)= -4.0686005354e-01
       coeff_l4(52,7)= -6.6214907169e-01
       coeff_l4(52,8)= -2.7303960919e-01
       coeff_l4(52,9)= -2.6561555266e-01
       coeff_l4(52,10)= -4.0154051781e-01
       coeff_l4(52,11)=  9.5452412963e-02
       coeff_l4(52,12)=  3.9859566838e-02
       coeff_l4(52,13)= -1.4493896067e-01
       coeff_l4(52,14)= -3.9091482759e-02
       coeff_l4(52,15)= -6.9917842746e-02
       coeff_l4(52,16)= -1.4959631860e-01
       coeff_l4(52,17)= -1.8117201328e-01
       coeff_l4(52,18)= -7.8360766172e-02
       coeff_l4(52,19)=  2.1322318912e-01
       coeff_l4(52,20)= -2.0710492134e-01
       coeff_l4(52,21)=  2.6406142861e-02
       coeff_l4(52,22)= -9.1683723032e-02
       coeff_l4(52,23)=  2.0163424313e-01
       coeff_l4(52,24)=  7.6376095414e-02
       coeff_l4(52,25)= -8.8882230222e-02
       coeff_l4(52,26)=  1.7821314931e-01
       coeff_l4(52,27)=  2.8383889794e-01
       coeff_l4(52,28)= -1.0500544310e-01
       coeff_l4(52,29)= -5.6839838624e-02
       coeff_l4(52,30)=  3.0726385117e-01
       coeff_l4(52,31)= -1.2349776924e-01
       coeff_l4(52,32)=  1.5717621148e-01
       coeff_l4(52,33)=  1.4519657195e-01
       coeff_l4(52,34)= -9.8751515150e-02
       coeff_l4(52,35)=  9.8634965718e-02
       coeff_l4(52,36)= -5.4080415517e-02
       coeff_l4(52,37)= -4.2072031647e-02
       coeff_l4(52,38)= -6.0080055147e-02
       coeff_l4(52,39)= -2.1866354346e-01
       coeff_l4(52,40)=  8.9494280517e-02
       coeff_l4(52,41)=  1.0075882077e-01
       coeff_l4(52,42)=  1.2071558833e-01
       coeff_l4(52,43)=  7.5841464102e-02
       coeff_l4(52,44)= -2.6048296690e-01
       coeff_l4(52,45)= -1.4094572514e-02
       coeff_l4(52,46)=  5.5369541049e-02
       coeff_l4(52,47)=  1.1729303747e-01
       coeff_l4(52,48)= -5.0336487591e-02
       coeff_l4(52,49)=  6.8200230598e-02
       coeff_l4(52,50)=  4.0561702102e-02
       coeff_l4(52,51)=  2.6324683428e-01
       coeff_l4(52,52)=  1.8063789234e-02
       coeff_l4(52,53)= -9.5450431108e-02
       coeff_l4(52,54)= -3.3323030919e-02
       coeff_l4(52,55)= -1.4184911549e-01
       coeff_l4(52,56)= -3.1132867560e-02
       coeff_l4(52,57)= -3.3805221319e-01
       coeff_l4(52,58)= -2.6467859745e-02
       coeff_l4(52,59)=  3.4674558043e-01
       coeff_l4(52,60)= -2.4108289182e-01
       coeff_l4(52,61)=  1.6634816304e-02
       coeff_l4(52,62)=  8.5452936590e-02
       coeff_l4(52,63)= -1.7165130377e-01
       coeff_l4(52,64)=  1.1785767972e-01
       coeff_l4(52,65)= -7.9951500520e-03
       coeff_l4(52,66)= -2.1385448053e-02
       coeff_l4(52,67)=  1.5967647731e-01
       coeff_l4(52,68)=  1.1116807908e-01
       coeff_l4(52,69)=  6.3360013068e-02
       coeff_l4(52,70)=  1.5785518289e-01
       coeff_l4(52,71)=  1.5775761008e-01
       coeff_l4(52,72)= -2.5668191910e-01
       coeff_l4(52,73)=  7.5777083635e-02
       coeff_l4(52,74)=  1.4435487986e-01
       coeff_l4(52,75)=  1.5390883386e-01
       coeff_l4(52,76)=  1.3272851706e-01
       coeff_l4(52,77)= -3.9688266814e-02
       coeff_l4(52,78)=  1.6508503258e-01
       coeff_l4(52,79)=  5.9043977410e-02
       coeff_l4(52,80)=  1.2066791952e-01
       coeff_l4(52,81)= -1.0116685182e-01
       coeff_l4(52,82)=  1.2842389941e-01
       coeff_l4(52,83)=  3.5340160131e-02
       coeff_l4(52,84)= -1.6853344440e-01
       coeff_l4(52,85)=  9.2874683440e-02
       coeff_l4(52,86)= -1.5551665425e-01
       coeff_l4(52,87)= -3.8623255491e-01
       coeff_l4(52,88)= -2.2228553891e-01
       coeff_l4(52,89)=  5.3645733744e-02
       coeff_l4(52,90)=  4.5205804706e-01
       coeff_l4(52,91)= -1.0133841075e-02
       coeff_l4(52,92)=  2.4926941842e-02
       coeff_l4(52,93)=  4.8002183437e-02
       coeff_l4(52,94)= -1.4384986833e-02
       coeff_l4(52,95)=  4.6115167439e-02
       coeff_l4(52,96)=  9.3249201775e-02
       coeff_l4(52,97)=  2.6370060444e-01
       coeff_l4(52,98)=  1.5868283808e-01
       coeff_l4(52,99)=  1.4530731738e-01
       coeff_l4(52,100)= -2.2780418396e-02
       coeff_l4(52,101)=  3.1847158074e-01
       coeff_l4(52,102)=  4.8139948398e-02
       coeff_l4(52,103)=  6.9035127759e-02
       coeff_l4(52,104)=  9.2724196613e-02
       coeff_l4(52,105)= -5.5366002023e-02
       coeff_l4(53,1)= -9.7195863724e-01
       coeff_l4(53,2)= -9.8339492083e-01
       coeff_l4(53,3)= -1.0125056505e+00
       coeff_l4(53,4)= -9.7923862934e-01
       coeff_l4(53,5)= -8.9989048243e-01
       coeff_l4(53,6)= -9.6202558279e-01
       coeff_l4(53,7)= -9.6770918369e-01
       coeff_l4(53,8)= -1.0108994246e+00
       coeff_l4(53,9)= -1.4101402462e-01
       coeff_l4(53,10)= -4.2214697599e-01
       coeff_l4(53,11)=  5.6342915632e-03
       coeff_l4(53,12)= -1.7755717039e-01
       coeff_l4(53,13)=  7.4680350721e-02
       coeff_l4(53,14)= -1.1694864184e-01
       coeff_l4(53,15)=  1.1328041553e-01
       coeff_l4(53,16)= -1.3695985079e-01
       coeff_l4(53,17)=  1.2179689482e-02
       coeff_l4(53,18)= -1.6805265099e-02
       coeff_l4(53,19)= -1.9608306885e-01
       coeff_l4(53,20)= -1.7815424502e-01
       coeff_l4(53,21)= -6.2960140407e-02
       coeff_l4(53,22)= -8.7815187871e-02
       coeff_l4(53,23)= -1.4180848002e-01
       coeff_l4(53,24)=  7.8970260918e-02
       coeff_l4(53,25)= -1.9469080865e-01
       coeff_l4(53,26)= -4.9610845745e-02
       coeff_l4(53,27)= -8.3787478507e-02
       coeff_l4(53,28)= -1.4374944568e-01
       coeff_l4(53,29)=  2.4061987177e-02
       coeff_l4(53,30)=  2.9196031392e-02
       coeff_l4(53,31)= -2.0800417662e-01
       coeff_l4(53,32)= -1.0802727938e-01
       coeff_l4(53,33)=  2.1205721796e-01
       coeff_l4(53,34)= -6.0500402004e-02
       coeff_l4(53,35)=  1.2424992025e-01
       coeff_l4(53,36)=  1.8788121641e-01
       coeff_l4(53,37)= -3.6737382412e-02
       coeff_l4(53,38)=  2.8866928816e-01
       coeff_l4(53,39)= -1.8653659523e-01
       coeff_l4(53,40)=  1.6295390204e-02
       coeff_l4(53,41)= -9.4374209642e-02
       coeff_l4(53,42)=  2.2958444059e-01
       coeff_l4(53,43)= -5.1901950501e-03
       coeff_l4(53,44)= -1.2365987152e-01
       coeff_l4(53,45)=  2.0531676710e-02
       coeff_l4(53,46)= -1.0225576907e-01
       coeff_l4(53,47)= -2.2263617814e-01
       coeff_l4(53,48)=  1.0103759915e-01
       coeff_l4(53,49)=  6.6728919744e-02
       coeff_l4(53,50)= -6.7347511649e-02
       coeff_l4(53,51)= -1.4897552133e-01
       coeff_l4(53,52)=  3.7438273430e-02
       coeff_l4(53,53)=  1.8207065761e-01
       coeff_l4(53,54)= -3.2089769840e-02
       coeff_l4(53,55)= -1.0518768802e-02
       coeff_l4(53,56)=  3.7768121809e-02
       coeff_l4(53,57)=  6.4569443464e-02
       coeff_l4(53,58)= -1.8586322665e-02
       coeff_l4(53,59)= -1.6252469271e-02
       coeff_l4(53,60)=  8.1330597401e-02
       coeff_l4(53,61)= -5.0961069763e-02
       coeff_l4(53,62)= -1.0024097562e-01
       coeff_l4(53,63)= -2.8260743618e-01
       coeff_l4(53,64)= -6.8719796836e-02
       coeff_l4(53,65)= -8.2668408751e-02
       coeff_l4(53,66)= -1.7882518470e-01
       coeff_l4(53,67)=  2.5746004656e-02
       coeff_l4(53,68)=  1.8871312961e-02
       coeff_l4(53,69)=  2.0022855699e-01
       coeff_l4(53,70)=  2.7406769991e-01
       coeff_l4(53,71)= -1.1107232422e-01
       coeff_l4(53,72)=  3.2433841377e-02
       coeff_l4(53,73)= -1.5578871965e-01
       coeff_l4(53,74)= -7.6160483062e-02
       coeff_l4(53,75)=  7.8933194280e-02
       coeff_l4(53,76)= -5.8404304087e-02
       coeff_l4(53,77)= -2.2578129172e-01
       coeff_l4(53,78)=  2.3263230920e-01
       coeff_l4(53,79)= -8.9719742537e-02
       coeff_l4(53,80)=  9.0241096914e-02
       coeff_l4(53,81)= -5.9214454144e-02
       coeff_l4(53,82)= -1.4582556486e-01
       coeff_l4(53,83)=  1.3423185050e-01
       coeff_l4(53,84)= -1.1000648141e-01
       coeff_l4(53,85)=  2.1249407530e-01
       coeff_l4(53,86)=  2.8840875253e-02
       coeff_l4(53,87)=  4.5081865042e-02
       coeff_l4(53,88)= -2.0500469953e-02
       coeff_l4(53,89)= -2.3743680120e-01
       coeff_l4(53,90)= -9.7342878580e-02
       coeff_l4(53,91)= -3.8621868938e-02
       coeff_l4(53,92)= -3.6426011473e-02
       coeff_l4(53,93)= -1.3389495201e-02
       coeff_l4(53,94)=  1.9031713903e-01
       coeff_l4(53,95)=  1.0909105092e-01
       coeff_l4(53,96)=  9.1891750693e-02
       coeff_l4(53,97)=  2.8343653679e-01
       coeff_l4(53,98)= -2.5480243564e-01
       coeff_l4(53,99)= -2.5852795690e-02
       coeff_l4(53,100)= -1.6166426241e-01
       coeff_l4(53,101)=  7.5002573431e-04
       coeff_l4(53,102)=  4.9137271941e-02
       coeff_l4(53,103)= -2.4595785886e-02
       coeff_l4(53,104)= -7.7946200967e-02
       coeff_l4(53,105)= -3.7209370732e-01
       coeff_l4(54,1)= -9.0678894520e-01
       coeff_l4(54,2)= -8.7454038858e-01
       coeff_l4(54,3)= -9.9180006981e-01
       coeff_l4(54,4)= -9.8832035065e-01
       coeff_l4(54,5)= -9.3108856678e-01
       coeff_l4(54,6)= -9.9588012695e-01
       coeff_l4(54,7)= -7.9835492373e-01
       coeff_l4(54,8)= -1.0301979780e+00
       coeff_l4(54,9)= -1.1898807287e+00
       coeff_l4(54,10)= -1.0162649155e+00
       coeff_l4(54,11)= -2.1570889950e+00
       coeff_l4(54,12)= -1.8970611095e+00
       coeff_l4(54,13)= -1.9028636217e+00
       coeff_l4(54,14)= -5.4152309895e-01
       coeff_l4(54,15)=  1.2159297243e-02
       coeff_l4(54,16)= -2.7897831053e-02
       coeff_l4(54,17)=  2.2465614974e-01
       coeff_l4(54,18)= -1.9166342914e-01
       coeff_l4(54,19)=  1.6931433976e-01
       coeff_l4(54,20)=  2.1488471329e-01
       coeff_l4(54,21)= -2.9423114657e-01
       coeff_l4(54,22)=  4.6678941697e-02
       coeff_l4(54,23)= -2.0721988380e-01
       coeff_l4(54,24)=  4.2027715594e-02
       coeff_l4(54,25)=  1.9999788702e-01
       coeff_l4(54,26)= -7.7166343108e-03
       coeff_l4(54,27)= -1.2228219211e-01
       coeff_l4(54,28)= -1.4149211347e-01
       coeff_l4(54,29)=  1.5195073187e-01
       coeff_l4(54,30)= -5.3504973650e-02
       coeff_l4(54,31)=  1.6025348008e-01
       coeff_l4(54,32)=  5.4821267724e-02
       coeff_l4(54,33)=  6.0110785998e-03
       coeff_l4(54,34)= -1.3738030195e-01
       coeff_l4(54,35)=  1.8290352821e-01
       coeff_l4(54,36)=  2.3644450307e-01
       coeff_l4(54,37)= -1.1233598739e-01
       coeff_l4(54,38)=  2.3144101724e-02
       coeff_l4(54,39)= -1.9994024932e-01
       coeff_l4(54,40)= -3.1472269446e-02
       coeff_l4(54,41)=  1.2281792611e-01
       coeff_l4(54,42)=  2.0749194920e-01
       coeff_l4(54,43)=  2.8441343457e-02
       coeff_l4(54,44)= -4.3606426567e-02
       coeff_l4(54,45)= -1.2985992432e-01
       coeff_l4(54,46)= -2.1480754018e-01
       coeff_l4(54,47)= -5.7711787522e-03
       coeff_l4(54,48)= -1.5799613297e-01
       coeff_l4(54,49)= -2.2460326552e-01
       coeff_l4(54,50)= -8.0434501171e-02
       coeff_l4(54,51)=  1.7343029380e-01
       coeff_l4(54,52)=  1.2764459848e-01
       coeff_l4(54,53)= -2.5292026997e-01
       coeff_l4(54,54)=  8.3684638143e-02
       coeff_l4(54,55)=  3.7095263600e-02
       coeff_l4(54,56)= -3.4053359181e-02
       coeff_l4(54,57)= -1.7097415030e-01
       coeff_l4(54,58)= -1.2191744894e-01
       coeff_l4(54,59)=  9.2323772609e-02
       coeff_l4(54,60)= -2.4102060497e-01
       coeff_l4(54,61)=  4.5382969081e-02
       coeff_l4(54,62)=  4.4584862888e-02
       coeff_l4(54,63)=  1.0378787667e-01
       coeff_l4(54,64)=  1.3514250517e-01
       coeff_l4(54,65)= -1.4582155645e-01
       coeff_l4(54,66)=  6.0096364468e-02
       coeff_l4(54,67)=  3.9056785405e-02
       coeff_l4(54,68)= -8.4671936929e-02
       coeff_l4(54,69)=  3.1785473228e-01
       coeff_l4(54,70)=  8.4943063557e-02
       coeff_l4(54,71)=  1.6979028285e-01
       coeff_l4(54,72)= -3.1703186035e-01
       coeff_l4(54,73)= -3.9972141385e-02
       coeff_l4(54,74)= -1.0275011510e-01
       coeff_l4(54,75)=  9.3114972115e-02
       coeff_l4(54,76)= -2.9244739562e-02
       coeff_l4(54,77)= -1.1994183064e-01
       coeff_l4(54,78)= -9.6907734871e-02
       coeff_l4(54,79)=  1.1239746958e-01
       coeff_l4(54,80)=  3.4889284521e-02
       coeff_l4(54,81)=  1.4885705709e-01
       coeff_l4(54,82)=  3.3844059799e-03
       coeff_l4(54,83)=  1.1842455715e-03
       coeff_l4(54,84)=  2.3999987543e-01
       coeff_l4(54,85)=  5.6758094579e-02
       coeff_l4(54,86)= -1.1756158620e-01
       coeff_l4(54,87)= -8.6904488504e-02
       coeff_l4(54,88)= -4.2005278170e-02
       coeff_l4(54,89)= -1.4560164511e-01
       coeff_l4(54,90)= -1.9751684740e-02
       coeff_l4(54,91)=  3.8842564821e-01
       coeff_l4(54,92)=  3.0497115850e-01
       coeff_l4(54,93)= -4.9236860871e-01
       coeff_l4(54,94)= -7.5761628151e-01
       coeff_l4(54,95)= -2.7193127200e-02
       coeff_l4(54,96)= -7.8984451294e-01
       coeff_l4(54,97)= -6.0118985176e-01
       coeff_l4(54,98)=  3.6423850805e-02
       coeff_l4(54,99)=  1.6380999982e-01
       coeff_l4(54,100)=  1.6672660410e-01
       coeff_l4(54,101)= -6.4550764859e-02
       coeff_l4(54,102)= -3.4847047925e-01
       coeff_l4(54,103)=  2.6675119996e-01
       coeff_l4(54,104)= -4.2457389645e-03
       coeff_l4(54,105)= -1.2972037494e-01
       coeff_l4(55,1)=  5.3999662399e-01
       coeff_l4(55,2)=  6.6539782286e-01
       coeff_l4(55,3)=  4.7689273953e-01
       coeff_l4(55,4)=  3.5157382488e-01
       coeff_l4(55,5)=  4.1573679447e-01
       coeff_l4(55,6)=  1.1089209467e-01
       coeff_l4(55,7)=  2.9735600948e-01
       coeff_l4(55,8)=  1.3990972936e-01
       coeff_l4(55,9)=  2.0924709737e-01
       coeff_l4(55,10)=  3.8791573048e-01
       coeff_l4(55,11)= -3.1436151266e-01
       coeff_l4(55,12)= -1.4897526801e-01
       coeff_l4(55,13)= -2.7080622315e-01
       coeff_l4(55,14)=  3.6366930604e-01
       coeff_l4(55,15)=  1.3804295659e-01
       coeff_l4(55,16)= -1.0067327321e-01
       coeff_l4(55,17)= -3.8943219930e-02
       coeff_l4(55,18)= -2.0944003761e-01
       coeff_l4(55,19)= -3.9578586817e-02
       coeff_l4(55,20)=  1.3125053048e-01
       coeff_l4(55,21)= -1.4888432622e-01
       coeff_l4(55,22)=  1.8609760702e-01
       coeff_l4(55,23)= -1.6198052093e-02
       coeff_l4(55,24)=  2.8400883079e-01
       coeff_l4(55,25)=  8.6815513670e-02
       coeff_l4(55,26)=  1.7497159541e-01
       coeff_l4(55,27)=  9.5656499267e-02
       coeff_l4(55,28)=  1.1054282635e-01
       coeff_l4(55,29)= -1.5322566032e-01
       coeff_l4(55,30)=  8.8443100452e-02
       coeff_l4(55,31)=  1.0447768122e-01
       coeff_l4(55,32)= -2.4675698951e-02
       coeff_l4(55,33)= -3.1167952344e-02
       coeff_l4(55,34)=  1.0984463152e-03
       coeff_l4(55,35)= -2.4472689629e-01
       coeff_l4(55,36)=  3.4273296595e-01
       coeff_l4(55,37)= -1.5178273618e-01
       coeff_l4(55,38)= -2.0742298663e-01
       coeff_l4(55,39)=  2.6898561046e-02
       coeff_l4(55,40)=  1.2954145670e-01
       coeff_l4(55,41)= -1.6008634865e-01
       coeff_l4(55,42)=  1.9740109146e-01
       coeff_l4(55,43)= -8.3400748670e-02
       coeff_l4(55,44)=  8.6637377739e-02
       coeff_l4(55,45)= -1.7963124812e-01
       coeff_l4(55,46)= -1.7175728083e-01
       coeff_l4(55,47)=  1.2076073140e-01
       coeff_l4(55,48)=  3.1680080295e-01
       coeff_l4(55,49)= -3.5106517375e-02
       coeff_l4(55,50)= -1.5487080812e-01
       coeff_l4(55,51)=  2.7565771341e-01
       coeff_l4(55,52)= -8.8900119066e-02
       coeff_l4(55,53)=  4.4192591310e-01
       coeff_l4(55,54)=  3.6570083350e-02
       coeff_l4(55,55)=  1.4029924758e-02
       coeff_l4(55,56)=  1.2352257222e-01
       coeff_l4(55,57)= -4.9260672182e-02
       coeff_l4(55,58)= -7.4817612767e-02
       coeff_l4(55,59)=  3.6061640829e-02
       coeff_l4(55,60)= -1.1796032079e-02
       coeff_l4(55,61)= -3.7402096391e-01
       coeff_l4(55,62)= -4.0499272943e-01
       coeff_l4(55,63)=  1.0147240758e-01
       coeff_l4(55,64)=  2.9240095615e-01
       coeff_l4(55,65)= -1.5236061811e-01
       coeff_l4(55,66)= -2.4891969562e-01
       coeff_l4(55,67)= -8.2673341036e-02
       coeff_l4(55,68)= -4.1235457174e-03
       coeff_l4(55,69)= -8.1138446927e-02
       coeff_l4(55,70)= -5.1191903651e-02
       coeff_l4(55,71)= -1.4222557843e-01
       coeff_l4(55,72)=  1.0351904482e-01
       coeff_l4(55,73)= -2.5821271539e-01
       coeff_l4(55,74)=  1.9582949579e-01
       coeff_l4(55,75)= -2.1928797662e-01
       coeff_l4(55,76)=  4.0668669343e-01
       coeff_l4(55,77)=  8.1548709422e-03
       coeff_l4(55,78)= -2.6576942205e-01
       coeff_l4(55,79)= -1.9636979699e-01
       coeff_l4(55,80)= -2.9279831052e-01
       coeff_l4(55,81)=  5.4921481758e-02
       coeff_l4(55,82)=  5.2618995309e-02
       coeff_l4(55,83)=  7.6481178403e-02
       coeff_l4(55,84)= -1.5045286715e-01
       coeff_l4(55,85)=  1.8645560741e-01
       coeff_l4(55,86)=  3.8802627474e-02
       coeff_l4(55,87)=  3.3465635777e-01
       coeff_l4(55,88)= -1.3915228844e-01
       coeff_l4(55,89)=  8.7032325566e-02
       coeff_l4(55,90)= -9.1683596373e-02
       coeff_l4(55,91)=  8.0105587840e-02
       coeff_l4(55,92)= -2.4837760627e-01
       coeff_l4(55,93)= -7.6278038323e-02
       coeff_l4(55,94)=  4.4985964894e-01
       coeff_l4(55,95)= -3.4323051572e-01
       coeff_l4(55,96)=  9.1389395297e-02
       coeff_l4(55,97)=  2.9448232055e-01
       coeff_l4(55,98)=  8.4109283984e-02
       coeff_l4(55,99)=  1.5303149819e-01
       coeff_l4(55,100)= -2.5734680891e-01
       coeff_l4(55,101)=  1.2276917696e-01
       coeff_l4(55,102)= -7.3780864477e-02
       coeff_l4(55,103)=  2.0608489215e-01
       coeff_l4(55,104)= -1.8094411120e-02
       coeff_l4(55,105)= -2.2883109748e-01
       coeff_l4(56,1)= -3.1754145026e-01
       coeff_l4(56,2)= -2.6442572474e-01
       coeff_l4(56,3)= -4.1329604387e-01
       coeff_l4(56,4)= -7.3731824756e-02
       coeff_l4(56,5)= -3.5766106844e-01
       coeff_l4(56,6)= -2.1296700835e-01
       coeff_l4(56,7)= -2.1181769669e-01
       coeff_l4(56,8)= -3.6888310313e-01
       coeff_l4(56,9)= -3.0995184183e-01
       coeff_l4(56,10)= -1.8612070382e-01
       coeff_l4(56,11)= -1.8171355128e-01
       coeff_l4(56,12)= -2.8921970725e-01
       coeff_l4(56,13)= -3.0760771036e-01
       coeff_l4(56,14)= -3.1337957829e-02
       coeff_l4(56,15)=  8.8600464165e-02
       coeff_l4(56,16)= -2.0506016910e-02
       coeff_l4(56,17)= -2.6971319318e-01
       coeff_l4(56,18)= -1.6818208992e-01
       coeff_l4(56,19)= -3.4311745316e-02
       coeff_l4(56,20)= -9.1637201607e-02
       coeff_l4(56,21)= -1.4953915775e-01
       coeff_l4(56,22)= -3.2512728125e-02
       coeff_l4(56,23)=  1.3187023997e-01
       coeff_l4(56,24)=  7.0102773607e-02
       coeff_l4(56,25)= -6.3872247934e-02
       coeff_l4(56,26)=  1.5754658729e-02
       coeff_l4(56,27)=  2.0386485755e-01
       coeff_l4(56,28)= -1.6650074720e-01
       coeff_l4(56,29)= -9.8522342741e-02
       coeff_l4(56,30)=  1.3373318315e-01
       coeff_l4(56,31)= -2.0026145875e-01
       coeff_l4(56,32)= -1.1566589028e-01
       coeff_l4(56,33)=  6.4583674073e-02
       coeff_l4(56,34)= -1.3995635509e-01
       coeff_l4(56,35)=  2.3170360923e-01
       coeff_l4(56,36)= -2.1847981960e-02
       coeff_l4(56,37)=  1.3060435653e-01
       coeff_l4(56,38)= -5.0203435123e-02
       coeff_l4(56,39)= -2.3757261038e-01
       coeff_l4(56,40)=  1.1164134368e-02
       coeff_l4(56,41)=  2.9964295030e-01
       coeff_l4(56,42)= -1.2401196174e-02
       coeff_l4(56,43)=  4.9471743405e-02
       coeff_l4(56,44)= -1.8756225705e-01
       coeff_l4(56,45)=  1.9694857299e-02
       coeff_l4(56,46)=  1.6962902248e-01
       coeff_l4(56,47)= -2.5431578979e-02
       coeff_l4(56,48)=  9.6513755620e-02
       coeff_l4(56,49)= -2.4647109210e-02
       coeff_l4(56,50)=  1.4865200222e-01
       coeff_l4(56,51)= -1.0878852010e-01
       coeff_l4(56,52)= -9.6918128431e-02
       coeff_l4(56,53)=  5.7863224298e-02
       coeff_l4(56,54)=  1.0866448283e-01
       coeff_l4(56,55)=  2.1198511124e-02
       coeff_l4(56,56)=  2.1130980924e-02
       coeff_l4(56,57)=  1.1235478520e-01
       coeff_l4(56,58)=  3.6319620907e-02
       coeff_l4(56,59)= -7.0990934037e-03
       coeff_l4(56,60)=  1.7626491189e-01
       coeff_l4(56,61)=  1.2827593088e-01
       coeff_l4(56,62)= -3.7755101919e-02
       coeff_l4(56,63)= -1.6829696298e-01
       coeff_l4(56,64)=  1.4484594762e-01
       coeff_l4(56,65)=  5.0366658717e-02
       coeff_l4(56,66)= -3.2381251454e-02
       coeff_l4(56,67)=  5.2935015410e-02
       coeff_l4(56,68)=  9.6472255886e-02
       coeff_l4(56,69)= -1.3293403387e-01
       coeff_l4(56,70)= -4.8733349890e-02
       coeff_l4(56,71)= -2.7730667498e-03
       coeff_l4(56,72)=  1.0177596658e-01
       coeff_l4(56,73)= -3.4265998751e-02
       coeff_l4(56,74)= -1.6001911461e-01
       coeff_l4(56,75)= -1.2843938172e-01
       coeff_l4(56,76)= -6.7956014536e-03
       coeff_l4(56,77)= -3.1318361871e-03
       coeff_l4(56,78)=  9.6288695931e-02
       coeff_l4(56,79)=  6.2028020620e-02
       coeff_l4(56,80)= -7.6369427145e-02
       coeff_l4(56,81)= -9.9418342113e-02
       coeff_l4(56,82)= -3.4500408219e-04
       coeff_l4(56,83)=  6.9738894701e-02
       coeff_l4(56,84)= -1.6328170896e-01
       coeff_l4(56,85)=  3.7660900503e-02
       coeff_l4(56,86)=  5.2918898873e-03
       coeff_l4(56,87)=  2.3831872270e-02
       coeff_l4(56,88)=  7.8953513876e-03
       coeff_l4(56,89)= -1.3314031065e-01
       coeff_l4(56,90)=  6.8773329258e-02
       coeff_l4(56,91)=  1.6618406400e-02
       coeff_l4(56,92)=  5.6461621076e-02
       coeff_l4(56,93)= -6.1176080257e-02
       coeff_l4(56,94)= -1.6656632721e-01
       coeff_l4(56,95)=  2.5538876653e-02
       coeff_l4(56,96)=  4.7620356083e-02
       coeff_l4(56,97)= -2.6351147890e-01
       coeff_l4(56,98)= -1.4530616999e-01
       coeff_l4(56,99)=  3.5881944001e-02
       coeff_l4(56,100)=  1.6078065336e-01
       coeff_l4(56,101)=  3.0009595677e-02
       coeff_l4(56,102)=  1.1573064327e-01
       coeff_l4(56,103)=  1.2738758326e-01
       coeff_l4(56,104)=  8.9485123754e-02
       coeff_l4(56,105)= -1.3878038526e-01
       coeff_l4(57,1)= -7.3418420553e-01
       coeff_l4(57,2)= -6.2117636204e-01
       coeff_l4(57,3)= -5.4964864254e-01
       coeff_l4(57,4)= -6.1481380463e-01
       coeff_l4(57,5)= -7.0976084471e-01
       coeff_l4(57,6)= -5.7181799412e-01
       coeff_l4(57,7)= -4.7451820970e-01
       coeff_l4(57,8)= -6.1845862865e-01
       coeff_l4(57,9)= -2.2527308762e-01
       coeff_l4(57,10)= -9.1944836080e-02
       coeff_l4(57,11)= -3.3895161748e-01
       coeff_l4(57,12)= -3.1555229425e-01
       coeff_l4(57,13)= -6.4653478563e-02
       coeff_l4(57,14)= -6.3120923936e-02
       coeff_l4(57,15)=  1.1279218644e-01
       coeff_l4(57,16)= -1.7246863246e-01
       coeff_l4(57,17)=  6.4034038223e-03
       coeff_l4(57,18)=  2.7901316062e-02
       coeff_l4(57,19)= -1.6475901008e-01
       coeff_l4(57,20)= -4.9117378891e-02
       coeff_l4(57,21)= -1.7160251737e-01
       coeff_l4(57,22)= -7.5472772121e-02
       coeff_l4(57,23)= -5.4104138166e-02
       coeff_l4(57,24)= -2.0451020449e-02
       coeff_l4(57,25)=  2.3460173979e-02
       coeff_l4(57,26)=  1.4728315175e-01
       coeff_l4(57,27)=  9.5807991922e-02
       coeff_l4(57,28)= -9.2319808900e-02
       coeff_l4(57,29)= -1.5838880837e-01
       coeff_l4(57,30)=  1.8106120825e-01
       coeff_l4(57,31)= -1.2796035409e-01
       coeff_l4(57,32)= -2.6687372476e-02
       coeff_l4(57,33)=  6.9927409291e-02
       coeff_l4(57,34)=  1.0721377283e-01
       coeff_l4(57,35)=  1.6880454496e-02
       coeff_l4(57,36)= -8.4149604663e-04
       coeff_l4(57,37)=  1.0018764436e-01
       coeff_l4(57,38)= -1.5052604675e-01
       coeff_l4(57,39)=  7.0006601512e-02
       coeff_l4(57,40)= -1.0734522343e-01
       coeff_l4(57,41)=  1.8292337656e-01
       coeff_l4(57,42)=  1.7436674237e-01
       coeff_l4(57,43)=  4.4918665662e-03
       coeff_l4(57,44)= -1.7142516375e-01
       coeff_l4(57,45)=  7.7150315046e-02
       coeff_l4(57,46)=  1.6471254826e-01
       coeff_l4(57,47)= -1.8153874576e-01
       coeff_l4(57,48)= -1.1421197653e-01
       coeff_l4(57,49)= -1.1371895671e-01
       coeff_l4(57,50)= -1.0866712779e-01
       coeff_l4(57,51)=  7.7079259790e-03
       coeff_l4(57,52)= -9.4727650285e-02
       coeff_l4(57,53)=  6.1327207834e-02
       coeff_l4(57,54)=  1.2199169025e-02
       coeff_l4(57,55)=  6.4189737896e-04
       coeff_l4(57,56)= -4.1949559003e-02
       coeff_l4(57,57)= -4.6694848686e-02
       coeff_l4(57,58)= -2.9609993100e-02
       coeff_l4(57,59)= -2.7319677174e-02
       coeff_l4(57,60)=  1.4528702945e-02
       coeff_l4(57,61)=  2.8521796688e-02
       coeff_l4(57,62)=  1.2977072597e-01
       coeff_l4(57,63)= -1.6517898440e-01
       coeff_l4(57,64)=  1.6066411510e-02
       coeff_l4(57,65)=  3.3999759704e-02
       coeff_l4(57,66)= -1.4314495027e-01
       coeff_l4(57,67)= -3.0963754281e-02
       coeff_l4(57,68)=  1.0595102608e-01
       coeff_l4(57,69)=  1.8497195840e-01
       coeff_l4(57,70)= -9.2016369104e-02
       coeff_l4(57,71)= -7.1622632444e-02
       coeff_l4(57,72)=  7.9499566928e-03
       coeff_l4(57,73)= -1.0491991788e-01
       coeff_l4(57,74)= -9.7853869200e-02
       coeff_l4(57,75)= -6.7514911294e-02
       coeff_l4(57,76)=  5.5934406817e-02
       coeff_l4(57,77)=  1.0705770925e-02
       coeff_l4(57,78)= -5.6767776608e-02
       coeff_l4(57,79)= -7.8161291778e-02
       coeff_l4(57,80)=  9.0895570815e-02
       coeff_l4(57,81)= -8.0264434218e-02
       coeff_l4(57,82)=  4.2135506868e-02
       coeff_l4(57,83)= -5.2684757859e-02
       coeff_l4(57,84)= -5.2712097764e-02
       coeff_l4(57,85)= -1.6754847765e-01
       coeff_l4(57,86)=  1.0345841199e-01
       coeff_l4(57,87)= -1.1196119338e-01
       coeff_l4(57,88)=  3.3163364977e-02
       coeff_l4(57,89)= -2.0425613225e-01
       coeff_l4(57,90)=  1.7286227643e-01
       coeff_l4(57,91)= -6.3298203051e-02
       coeff_l4(57,92)= -9.2717483640e-02
       coeff_l4(57,93)= -1.0827872902e-01
       coeff_l4(57,94)=  1.4550073072e-02
       coeff_l4(57,95)=  1.5348805487e-01
       coeff_l4(57,96)= -2.2354730964e-01
       coeff_l4(57,97)= -1.3233090937e-01
       coeff_l4(57,98)= -1.9322286546e-01
       coeff_l4(57,99)=  1.6730965674e-01
       coeff_l4(57,100)=  1.0868853331e-01
       coeff_l4(57,101)= -8.5423956625e-04
       coeff_l4(57,102)= -1.1274714023e-01
       coeff_l4(57,103)=  7.3078379035e-02
       coeff_l4(57,104)=  1.4121592045e-01
       coeff_l4(57,105)= -8.8839069940e-04
       coeff_l4(58,1)= -5.9422183037e-01
       coeff_l4(58,2)= -1.2604960203e+00
       coeff_l4(58,3)= -1.3825743198e+00
       coeff_l4(58,4)= -1.0457904339e+00
       coeff_l4(58,5)= -1.8672825098e+00
       coeff_l4(58,6)= -1.3806856871e+00
       coeff_l4(58,7)= -1.3115980625e+00
       coeff_l4(58,8)= -1.9469175339e+00
       coeff_l4(58,9)= -1.0647560358e+00
       coeff_l4(58,10)= -8.1052058935e-01
       coeff_l4(58,11)= -1.0095063448e+00
       coeff_l4(58,12)= -5.1279783249e-01
       coeff_l4(58,13)= -1.3290221691e+00
       coeff_l4(58,14)=  1.6113368329e-03
       coeff_l4(58,15)= -1.5960200131e-01
       coeff_l4(58,16)= -3.2853513956e-02
       coeff_l4(58,17)=  4.5880323648e-01
       coeff_l4(58,18)= -1.4463901520e-01
       coeff_l4(58,19)=  5.4275763035e-01
       coeff_l4(58,20)=  2.6038822532e-01
       coeff_l4(58,21)= -5.1992513239e-02
       coeff_l4(58,22)= -1.1120776087e-01
       coeff_l4(58,23)= -3.1923469901e-01
       coeff_l4(58,24)=  2.0734171569e-01
       coeff_l4(58,25)= -3.6779597402e-02
       coeff_l4(58,26)= -1.3746240735e-01
       coeff_l4(58,27)= -1.8751984835e-01
       coeff_l4(58,28)=  3.6966854334e-01
       coeff_l4(58,29)=  1.6646319628e-01
       coeff_l4(58,30)= -1.8438145518e-01
       coeff_l4(58,31)=  6.5921920538e-01
       coeff_l4(58,32)=  1.4341963828e-01
       coeff_l4(58,33)= -2.5828066468e-01
       coeff_l4(58,34)=  3.1581057701e-03
       coeff_l4(58,35)=  2.0247530192e-02
       coeff_l4(58,36)= -2.9759505391e-01
       coeff_l4(58,37)=  4.2542763054e-02
       coeff_l4(58,38)= -7.2989813983e-02
       coeff_l4(58,39)= -1.4832443558e-02
       coeff_l4(58,40)= -2.4878183007e-01
       coeff_l4(58,41)= -5.5017136037e-02
       coeff_l4(58,42)= -2.2142780945e-02
       coeff_l4(58,43)= -3.5298969597e-02
       coeff_l4(58,44)= -1.9927230477e-01
       coeff_l4(58,45)= -3.7019643933e-02
       coeff_l4(58,46)= -7.7378195524e-01
       coeff_l4(58,47)= -1.2184026092e-01
       coeff_l4(58,48)=  4.2258670926e-01
       coeff_l4(58,49)= -4.8372349702e-03
       coeff_l4(58,50)= -4.6396875381e-01
       coeff_l4(58,51)=  1.7163832672e-03
       coeff_l4(58,52)= -2.1564771235e-01
       coeff_l4(58,53)=  8.6483851075e-02
       coeff_l4(58,54)=  4.0094456077e-01
       coeff_l4(58,55)=  7.0499621332e-02
       coeff_l4(58,56)=  8.4445513785e-02
       coeff_l4(58,57)=  2.1619501710e-01
       coeff_l4(58,58)=  6.7726415582e-03
       coeff_l4(58,59)= -1.7562385648e-02
       coeff_l4(58,60)=  1.1954925209e-01
       coeff_l4(58,61)=  2.5722587109e-01
       coeff_l4(58,62)= -2.2436626256e-01
       coeff_l4(58,63)=  5.2054088563e-02
       coeff_l4(58,64)= -2.8298053145e-01
       coeff_l4(58,65)=  2.3764485377e-04
       coeff_l4(58,66)= -4.4671508670e-01
       coeff_l4(58,67)=  3.8029614091e-01
       coeff_l4(58,68)=  1.0302106291e-01
       coeff_l4(58,69)= -2.1515676752e-02
       coeff_l4(58,70)=  1.1500351876e-01
       coeff_l4(58,71)=  2.8281131387e-01
       coeff_l4(58,72)= -3.1185466051e-01
       coeff_l4(58,73)= -2.8165188432e-01
       coeff_l4(58,74)=  1.7475919425e-01
       coeff_l4(58,75)= -1.7854520679e-01
       coeff_l4(58,76)=  1.5527544916e-01
       coeff_l4(58,77)= -6.0600444674e-02
       coeff_l4(58,78)= -2.0812584460e-01
       coeff_l4(58,79)=  1.1918845028e-01
       coeff_l4(58,80)=  1.9275946915e-01
       coeff_l4(58,81)=  3.0261832476e-01
       coeff_l4(58,82)=  8.9873326942e-03
       coeff_l4(58,83)=  2.7453878522e-01
       coeff_l4(58,84)=  3.9181017876e-01
       coeff_l4(58,85)= -2.7171102166e-01
       coeff_l4(58,86)=  4.6909440309e-02
       coeff_l4(58,87)= -5.3134225309e-02
       coeff_l4(58,88)= -3.7382009625e-01
       coeff_l4(58,89)= -2.6252192259e-01
       coeff_l4(58,90)= -3.0133512616e-01
       coeff_l4(58,91)=  1.4178371429e-01
       coeff_l4(58,92)= -5.0643521547e-01
       coeff_l4(58,93)= -5.5111682415e-01
       coeff_l4(58,94)= -1.7954928800e-02
       coeff_l4(58,95)= -1.3953661919e-01
       coeff_l4(58,96)=  3.5057481378e-02
       coeff_l4(58,97)= -1.9761715829e-01
       coeff_l4(58,98)= -1.8232204020e-01
       coeff_l4(58,99)= -1.8614278734e-01
       coeff_l4(58,100)= -4.9787808210e-02
       coeff_l4(58,101)= -1.6630837321e-01
       coeff_l4(58,102)=  4.8345214128e-01
       coeff_l4(58,103)=  1.3273354620e-02
       coeff_l4(58,104)= -1.1979189515e-01
       coeff_l4(58,105)= -2.8467342257e-01
       coeff_l4(59,1)=  7.5943315029e-01
       coeff_l4(59,2)=  1.6597733498e+00
       coeff_l4(59,3)=  1.3582471609e+00
       coeff_l4(59,4)=  1.1740794182e+00
       coeff_l4(59,5)=  1.2828961611e+00
       coeff_l4(59,6)=  1.3896591663e+00
       coeff_l4(59,7)=  8.7642794847e-01
       coeff_l4(59,8)=  6.8132668734e-01
       coeff_l4(59,9)=  3.8658094406e-01
       coeff_l4(59,10)=  5.0570863485e-01
       coeff_l4(59,11)=  6.3336324692e-01
       coeff_l4(59,12)=  7.8859466314e-01
       coeff_l4(59,13)=  4.8237034678e-01
       coeff_l4(59,14)=  4.5728132129e-01
       coeff_l4(59,15)=  2.7577984333e-01
       coeff_l4(59,16)= -1.1144523323e-01
       coeff_l4(59,17)= -1.7602369189e-01
       coeff_l4(59,18)=  6.5957792103e-02
       coeff_l4(59,19)=  1.5628860891e-01
       coeff_l4(59,20)=  5.9373710304e-02
       coeff_l4(59,21)= -2.4857942760e-01
       coeff_l4(59,22)= -1.5618185699e-01
       coeff_l4(59,23)=  2.7333021164e-01
       coeff_l4(59,24)= -4.4633498788e-01
       coeff_l4(59,25)= -1.0151229799e-01
       coeff_l4(59,26)= -9.9341697991e-02
       coeff_l4(59,27)=  1.0444603860e-01
       coeff_l4(59,28)=  1.2172919698e-02
       coeff_l4(59,29)= -5.3726587445e-02
       coeff_l4(59,30)=  1.2047632039e-01
       coeff_l4(59,31)= -1.7800691724e-01
       coeff_l4(59,32)= -4.2432978749e-02
       coeff_l4(59,33)=  1.7211464001e-03
       coeff_l4(59,34)=  2.6127839088e-01
       coeff_l4(59,35)= -6.8408772349e-02
       coeff_l4(59,36)=  2.8734171763e-02
       coeff_l4(59,37)= -3.0234867707e-02
       coeff_l4(59,38)= -3.7426430732e-02
       coeff_l4(59,39)= -2.0885975659e-01
       coeff_l4(59,40)=  2.5687891245e-01
       coeff_l4(59,41)= -3.0366603285e-02
       coeff_l4(59,42)= -1.4706908166e-01
       coeff_l4(59,43)=  5.5669225752e-02
       coeff_l4(59,44)= -2.0457810163e-01
       coeff_l4(59,45)=  2.2262845933e-01
       coeff_l4(59,46)=  4.2886239290e-01
       coeff_l4(59,47)= -3.1703424454e-01
       coeff_l4(59,48)=  4.9409367144e-02
       coeff_l4(59,49)= -3.8004238158e-02
       coeff_l4(59,50)= -2.4501120206e-03
       coeff_l4(59,51)= -2.2581361234e-02
       coeff_l4(59,52)= -7.4259854853e-02
       coeff_l4(59,53)= -1.9742922857e-02
       coeff_l4(59,54)= -2.4732133746e-01
       coeff_l4(59,55)= -3.7318760157e-01
       coeff_l4(59,56)=  4.1391488165e-02
       coeff_l4(59,57)=  2.0806424320e-02
       coeff_l4(59,58)=  1.6905465722e-01
       coeff_l4(59,59)= -7.8926309943e-02
       coeff_l4(59,60)=  6.3650280237e-02
       coeff_l4(59,61)=  1.3811071217e-01
       coeff_l4(59,62)= -5.7991158217e-02
       coeff_l4(59,63)= -1.9593341649e-01
       coeff_l4(59,64)= -3.1941244006e-01
       coeff_l4(59,65)=  4.3225266039e-02
       coeff_l4(59,66)= -1.4410060644e-01
       coeff_l4(59,67)=  1.1060314626e-01
       coeff_l4(59,68)=  2.3364715278e-02
       coeff_l4(59,69)=  6.6872879863e-02
       coeff_l4(59,70)= -3.7099006772e-01
       coeff_l4(59,71)= -4.0525409579e-01
       coeff_l4(59,72)=  2.4743670598e-02
       coeff_l4(59,73)=  2.4818116426e-01
       coeff_l4(59,74)= -4.9428489059e-02
       coeff_l4(59,75)= -1.9113960443e-03
       coeff_l4(59,76)=  7.7782392502e-02
       coeff_l4(59,77)= -8.6486242712e-02
       coeff_l4(59,78)=  6.6372968256e-02
       coeff_l4(59,79)=  9.4723150134e-02
       coeff_l4(59,80)= -4.8997312784e-02
       coeff_l4(59,81)= -1.5636634827e-01
       coeff_l4(59,82)= -1.5479820967e-01
       coeff_l4(59,83)=  1.4734952152e-01
       coeff_l4(59,84)= -1.3086226583e-01
       coeff_l4(59,85)= -2.1047092974e-01
       coeff_l4(59,86)= -1.5430142172e-02
       coeff_l4(59,87)= -1.3010475039e-01
       coeff_l4(59,88)=  2.0125040412e-01
       coeff_l4(59,89)= -1.9185298681e-01
       coeff_l4(59,90)=  9.9698834121e-02
       coeff_l4(59,91)=  5.4861333221e-02
       coeff_l4(59,92)=  5.9246908873e-02
       coeff_l4(59,93)=  3.8081476092e-01
       coeff_l4(59,94)= -3.3205783367e-01
       coeff_l4(59,95)=  1.6735221446e-01
       coeff_l4(59,96)=  4.7736769915e-01
       coeff_l4(59,97)= -7.4639089406e-02
       coeff_l4(59,98)=  1.5087820590e-01
       coeff_l4(59,99)= -7.7208511531e-02
       coeff_l4(59,100)= -1.5199248493e-01
       coeff_l4(59,101)= -1.3234586455e-02
       coeff_l4(59,102)= -4.9285784364e-02
       coeff_l4(59,103)= -5.5410559289e-03
       coeff_l4(59,104)= -1.6197285056e-01
       coeff_l4(59,105)=  3.5251402855e-01
       coeff_l4(60,1)= -1.7415708303e-01
       coeff_l4(60,2)= -2.2697278857e-01
       coeff_l4(60,3)= -6.2114375830e-01
       coeff_l4(60,4)= -3.9378806949e-02
       coeff_l4(60,5)= -7.1531876922e-02
       coeff_l4(60,6)= -5.8940476179e-01
       coeff_l4(60,7)=  3.5540202260e-01
       coeff_l4(60,8)= -3.7774339318e-01
       coeff_l4(60,9)=  3.9116084576e+00
       coeff_l4(60,10)=  3.0357451439e+00
       coeff_l4(60,11)=  8.2937297821e+00
       coeff_l4(60,12)=  8.5347490311e+00
       coeff_l4(60,13)=  7.2045168877e+00
       coeff_l4(60,14)=  7.9534298182e-01
       coeff_l4(60,15)= -1.3569629192e-01
       coeff_l4(60,16)= -2.7205514908e-01
       coeff_l4(60,17)= -6.8351432681e-02
       coeff_l4(60,18)= -8.3371497691e-02
       coeff_l4(60,19)= -3.2540386915e-01
       coeff_l4(60,20)=  3.0722809955e-02
       coeff_l4(60,21)= -7.6305694878e-02
       coeff_l4(60,22)= -6.1257842928e-02
       coeff_l4(60,23)=  8.8082695007e-01
       coeff_l4(60,24)= -3.2622766495e-01
       coeff_l4(60,25)=  2.7186101675e-01
       coeff_l4(60,26)=  4.4836887717e-01
       coeff_l4(60,27)=  8.0415336415e-03
       coeff_l4(60,28)= -3.5985782743e-01
       coeff_l4(60,29)=  7.1225002408e-02
       coeff_l4(60,30)= -4.5239333063e-02
       coeff_l4(60,31)= -3.5480812192e-01
       coeff_l4(60,32)= -2.7645040303e-02
       coeff_l4(60,33)=  2.7867266536e-01
       coeff_l4(60,34)= -2.1243907511e-01
       coeff_l4(60,35)=  3.8492321968e-01
       coeff_l4(60,36)= -7.8962907195e-02
       coeff_l4(60,37)=  2.2753056884e-01
       coeff_l4(60,38)= -4.9144002795e-01
       coeff_l4(60,39)=  2.6524689794e-01
       coeff_l4(60,40)= -3.1763365865e-01
       coeff_l4(60,41)=  4.2784467340e-01
       coeff_l4(60,42)=  2.6369005442e-01
       coeff_l4(60,43)=  8.6392104626e-01
       coeff_l4(60,44)=  2.7727647102e-04
       coeff_l4(60,45)=  9.5876502991e-01
       coeff_l4(60,46)= -2.3229938745e-01
       coeff_l4(60,47)= -5.6508541107e-01
       coeff_l4(60,48)= -2.2340545058e-01
       coeff_l4(60,49)=  3.7652883679e-02
       coeff_l4(60,50)= -8.2461476326e-02
       coeff_l4(60,51)= -4.0690273046e-02
       coeff_l4(60,52)= -1.1602604389e-01
       coeff_l4(60,53)=  2.8533145785e-01
       coeff_l4(60,54)=  1.2701778673e-02
       coeff_l4(60,55)=  1.1325306445e-01
       coeff_l4(60,56)= -3.1432113051e-01
       coeff_l4(60,57)= -5.9918630123e-01
       coeff_l4(60,58)=  1.2581919134e-01
       coeff_l4(60,59)=  2.2178715095e-02
       coeff_l4(60,60)= -1.2927167118e-01
       coeff_l4(60,61)= -9.1471426189e-02
       coeff_l4(60,62)=  5.4157458246e-02
       coeff_l4(60,63)=  3.6646720767e-01
       coeff_l4(60,64)= -6.5471881628e-01
       coeff_l4(60,65)=  2.8826376796e-01
       coeff_l4(60,66)=  4.2840139940e-03
       coeff_l4(60,67)= -2.2604225203e-02
       coeff_l4(60,68)= -3.3637946844e-01
       coeff_l4(60,69)=  2.5560371578e-02
       coeff_l4(60,70)=  4.5672062039e-01
       coeff_l4(60,71)= -1.8301852047e-01
       coeff_l4(60,72)= -9.2533248663e-01
       coeff_l4(60,73)=  9.3076892197e-02
       coeff_l4(60,74)= -4.8758235574e-01
       coeff_l4(60,75)= -4.9257215858e-01
       coeff_l4(60,76)= -7.1130508184e-01
       coeff_l4(60,77)=  4.6682009101e-01
       coeff_l4(60,78)= -3.0259209871e-01
       coeff_l4(60,79)=  5.7396805286e-01
       coeff_l4(60,80)=  3.1252479553e-01
       coeff_l4(60,81)=  4.0231668949e-01
       coeff_l4(60,82)= -8.9760012925e-03
       coeff_l4(60,83)=  3.4900897741e-01
       coeff_l4(60,84)= -1.6426381469e-01
       coeff_l4(60,85)= -2.8071519732e-01
       coeff_l4(60,86)= -1.0525398701e-01
       coeff_l4(60,87)=  6.8490371108e-02
       coeff_l4(60,88)= -8.6450344324e-01
       coeff_l4(60,89)=  7.6002532244e-01
       coeff_l4(60,90)=  4.0005230904e-01
       coeff_l4(60,91)= -1.5202736855e-01
       coeff_l4(60,92)= -2.7627947927e-01
       coeff_l4(60,93)=  1.3287205696e+00
       coeff_l4(60,94)=  4.4590744972e+00
       coeff_l4(60,95)= -2.4890631437e-01
       coeff_l4(60,96)=  3.4580931664e+00
       coeff_l4(60,97)=  2.8722944260e+00
       coeff_l4(60,98)= -3.8274151087e-01
       coeff_l4(60,99)= -3.7593671679e-01
       coeff_l4(60,100)= -1.2135381699e+00
       coeff_l4(60,101)=  8.6771905422e-01
       coeff_l4(60,102)=  5.1191097498e-01
       coeff_l4(60,103)= -8.0818134546e-01
       coeff_l4(60,104)= -8.9505308867e-01
       coeff_l4(60,105)=  1.9621738791e-01
       coeff_l4(61,1)=  3.3778656274e-02
       coeff_l4(61,2)=  2.5134173036e-01
       coeff_l4(61,3)=  2.4755652249e-01
       coeff_l4(61,4)=  2.4261195958e-01
       coeff_l4(61,5)=  1.1593060941e-01
       coeff_l4(61,6)=  1.2645593286e-01
       coeff_l4(61,7)=  1.6551408172e-01
       coeff_l4(61,8)=  1.0432383418e-01
       coeff_l4(61,9)=  2.4444634095e-02
       coeff_l4(61,10)=  1.1765176803e-01
       coeff_l4(61,11)=  2.6717954874e-01
       coeff_l4(61,12)=  2.6019403338e-01
       coeff_l4(61,13)=  3.2440799475e-01
       coeff_l4(61,14)=  2.3141731322e-01
       coeff_l4(61,15)= -2.7760658413e-03
       coeff_l4(61,16)= -5.9718459845e-02
       coeff_l4(61,17)= -1.7281312495e-02
       coeff_l4(61,18)=  1.5833836049e-02
       coeff_l4(61,19)= -1.7139075324e-02
       coeff_l4(61,20)=  1.3101787865e-01
       coeff_l4(61,21)=  1.6408026218e-01
       coeff_l4(61,22)=  7.8051090240e-02
       coeff_l4(61,23)= -8.1220418215e-02
       coeff_l4(61,24)=  1.1149831861e-01
       coeff_l4(61,25)= -5.2614547312e-02
       coeff_l4(61,26)= -8.6850360036e-02
       coeff_l4(61,27)= -8.7443679571e-02
       coeff_l4(61,28)= -8.6948998272e-02
       coeff_l4(61,29)= -6.2133409083e-02
       coeff_l4(61,30)=  1.4904311299e-01
       coeff_l4(61,31)=  7.9427868128e-02
       coeff_l4(61,32)= -5.1320556551e-02
       coeff_l4(61,33)=  6.3433018513e-03
       coeff_l4(61,34)=  1.2078653276e-01
       coeff_l4(61,35)= -2.2941760719e-02
       coeff_l4(61,36)= -1.9051688910e-01
       coeff_l4(61,37)= -3.0843432993e-02
       coeff_l4(61,38)= -1.4352293313e-01
       coeff_l4(61,39)=  4.0987860411e-03
       coeff_l4(61,40)=  3.2271463424e-02
       coeff_l4(61,41)= -2.2212810814e-02
       coeff_l4(61,42)=  4.9206413329e-02
       coeff_l4(61,43)=  1.2054055929e-01
       coeff_l4(61,44)=  1.3591293991e-01
       coeff_l4(61,45)= -1.0815157741e-01
       coeff_l4(61,46)=  1.1060243845e-01
       coeff_l4(61,47)= -8.8208846748e-02
       coeff_l4(61,48)=  3.2017964404e-03
       coeff_l4(61,49)=  2.2771570832e-02
       coeff_l4(61,50)= -8.6548954248e-02
       coeff_l4(61,51)=  2.7592696249e-02
       coeff_l4(61,52)= -1.1547092348e-01
       coeff_l4(61,53)= -2.9723967891e-03
       coeff_l4(61,54)=  1.2297996879e-01
       coeff_l4(61,55)= -1.1037388444e-01
       coeff_l4(61,56)= -1.0420987010e-01
       coeff_l4(61,57)=  1.4818011224e-01
       coeff_l4(61,58)=  5.0845447928e-02
       coeff_l4(61,59)= -3.6863200366e-02
       coeff_l4(61,60)=  4.8001939431e-03
       coeff_l4(61,61)=  1.1214048415e-01
       coeff_l4(61,62)=  1.4006221294e-01
       coeff_l4(61,63)=  8.1290654838e-02
       coeff_l4(61,64)=  3.2484073192e-02
       coeff_l4(61,65)=  4.9316793680e-02
       coeff_l4(61,66)= -1.3608115911e-01
       coeff_l4(61,67)= -1.3084253669e-01
       coeff_l4(61,68)= -1.2339010090e-01
       coeff_l4(61,69)=  6.1387125403e-02
       coeff_l4(61,70)= -7.4779249728e-02
       coeff_l4(61,71)= -8.5592875257e-03
       coeff_l4(61,72)=  1.6575796902e-01
       coeff_l4(61,73)= -7.0714928210e-02
       coeff_l4(61,74)=  1.4944459498e-01
       coeff_l4(61,75)= -9.5131732523e-02
       coeff_l4(61,76)=  8.8690906763e-02
       coeff_l4(61,77)= -7.2800517082e-02
       coeff_l4(61,78)= -1.3577842712e-01
       coeff_l4(61,79)= -1.7060469091e-01
       coeff_l4(61,80)= -8.3662465215e-02
       coeff_l4(61,81)= -6.5136879683e-02
       coeff_l4(61,82)=  7.7579240315e-03
       coeff_l4(61,83)=  1.0386983305e-01
       coeff_l4(61,84)= -1.0018394142e-01
       coeff_l4(61,85)= -1.6002523899e-01
       coeff_l4(61,86)=  7.0303358138e-02
       coeff_l4(61,87)=  9.8309796304e-03
       coeff_l4(61,88)= -4.2164493352e-02
       coeff_l4(61,89)=  1.2616811693e-01
       coeff_l4(61,90)= -1.5099091828e-01
       coeff_l4(61,91)=  3.6876152735e-03
       coeff_l4(61,92)=  1.0995052755e-01
       coeff_l4(61,93)= -6.5138712525e-02
       coeff_l4(61,94)=  2.5148463249e-01
       coeff_l4(61,95)= -1.6495168209e-01
       coeff_l4(61,96)= -1.3763605058e-01
       coeff_l4(61,97)= -6.1215437949e-02
       coeff_l4(61,98)=  2.0539005101e-01
       coeff_l4(61,99)=  2.3722076789e-02
       coeff_l4(61,100)=  1.0207025707e-01
       coeff_l4(61,101)=  2.7284637094e-02
       coeff_l4(61,102)= -1.8796098232e-01
       coeff_l4(61,103)=  2.1820233762e-01
       coeff_l4(61,104)= -8.2378260791e-02
       coeff_l4(61,105)= -6.4244069159e-02
       coeff_l4(62,1)= -4.7402763367e-01
       coeff_l4(62,2)= -6.0396397114e-01
       coeff_l4(62,3)= -5.6574243307e-01
       coeff_l4(62,4)= -5.2034157515e-01
       coeff_l4(62,5)= -4.6329027414e-01
       coeff_l4(62,6)= -6.2971353531e-01
       coeff_l4(62,7)= -3.0554899573e-01
       coeff_l4(62,8)= -4.6944326162e-01
       coeff_l4(62,9)=  1.3214325905e-01
       coeff_l4(62,10)=  9.4092048705e-02
       coeff_l4(62,11)=  1.1886944622e-01
       coeff_l4(62,12)=  2.3597221822e-02
       coeff_l4(62,13)=  2.2913765907e-01
       coeff_l4(62,14)=  6.4559921622e-02
       coeff_l4(62,15)=  1.8147674855e-03
       coeff_l4(62,16)= -1.4935408533e-01
       coeff_l4(62,17)= -5.9668265283e-02
       coeff_l4(62,18)=  8.4922537208e-03
       coeff_l4(62,19)= -1.8889191747e-01
       coeff_l4(62,20)=  1.3556337357e-01
       coeff_l4(62,21)= -1.7139703035e-01
       coeff_l4(62,22)= -1.1084844917e-01
       coeff_l4(62,23)=  1.6231292486e-01
       coeff_l4(62,24)=  1.0687392950e-01
       coeff_l4(62,25)= -3.2153341919e-02
       coeff_l4(62,26)= -9.7847975791e-02
       coeff_l4(62,27)=  1.4796020091e-01
       coeff_l4(62,28)= -4.0124472231e-02
       coeff_l4(62,29)= -1.0091782361e-01
       coeff_l4(62,30)=  3.6652371287e-02
       coeff_l4(62,31)=  3.2974302769e-02
       coeff_l4(62,32)= -1.3215665519e-01
       coeff_l4(62,33)=  1.3907524943e-01
       coeff_l4(62,34)=  4.8087477684e-02
       coeff_l4(62,35)=  1.1956010014e-01
       coeff_l4(62,36)=  1.2231405079e-01
       coeff_l4(62,37)= -4.9193076789e-02
       coeff_l4(62,38)=  1.0101619363e-01
       coeff_l4(62,39)= -2.0804937184e-01
       coeff_l4(62,40)=  1.2997029722e-01
       coeff_l4(62,41)=  1.5913788974e-01
       coeff_l4(62,42)= -1.6348743439e-01
       coeff_l4(62,43)=  5.7393603027e-02
       coeff_l4(62,44)=  3.0645476654e-02
       coeff_l4(62,45)= -8.0442741513e-02
       coeff_l4(62,46)= -6.6399551928e-02
       coeff_l4(62,47)= -1.1495686322e-01
       coeff_l4(62,48)=  3.4153051674e-02
       coeff_l4(62,49)=  8.2778103650e-02
       coeff_l4(62,50)=  8.9312888682e-02
       coeff_l4(62,51)= -1.5397937968e-02
       coeff_l4(62,52)=  6.2814205885e-02
       coeff_l4(62,53)= -1.2610864639e-01
       coeff_l4(62,54)=  1.2384323031e-01
       coeff_l4(62,55)= -4.3800108135e-02
       coeff_l4(62,56)= -9.5512710512e-02
       coeff_l4(62,57)= -8.1984050572e-02
       coeff_l4(62,58)= -4.9156774767e-03
       coeff_l4(62,59)=  1.6408731043e-01
       coeff_l4(62,60)=  8.2021705806e-02
       coeff_l4(62,61)= -3.6690149456e-02
       coeff_l4(62,62)=  3.6666113883e-02
       coeff_l4(62,63)=  1.4074136503e-02
       coeff_l4(62,64)=  4.8029299825e-02
       coeff_l4(62,65)=  1.3280886412e-01
       coeff_l4(62,66)=  1.3923233747e-01
       coeff_l4(62,67)=  2.0215300843e-02
       coeff_l4(62,68)= -6.8183451891e-02
       coeff_l4(62,69)=  1.0719208419e-01
       coeff_l4(62,70)=  1.7052622512e-02
       coeff_l4(62,71)= -1.2545394897e-01
       coeff_l4(62,72)=  1.0296975821e-01
       coeff_l4(62,73)=  9.7675353289e-02
       coeff_l4(62,74)= -6.0663990676e-02
       coeff_l4(62,75)=  1.3811253011e-01
       coeff_l4(62,76)= -9.5436023548e-03
       coeff_l4(62,77)= -1.0289725661e-01
       coeff_l4(62,78)= -9.6276156604e-02
       coeff_l4(62,79)= -6.7356787622e-02
       coeff_l4(62,80)=  1.3822424412e-01
       coeff_l4(62,81)=  1.8545730412e-01
       coeff_l4(62,82)= -2.7280723676e-02
       coeff_l4(62,83)= -1.6793338582e-02
       coeff_l4(62,84)=  1.0676857084e-02
       coeff_l4(62,85)= -6.9966413081e-02
       coeff_l4(62,86)= -1.6625578701e-01
       coeff_l4(62,87)=  9.8604470491e-02
       coeff_l4(62,88)= -1.2236905843e-01
       coeff_l4(62,89)=  9.7443103790e-02
       coeff_l4(62,90)=  1.6335135698e-01
       coeff_l4(62,91)= -6.0657497495e-02
       coeff_l4(62,92)=  4.1205547750e-02
       coeff_l4(62,93)= -8.4024764597e-02
       coeff_l4(62,94)=  2.4561496079e-01
       coeff_l4(62,95)= -8.4928356111e-02
       coeff_l4(62,96)= -7.6510816813e-02
       coeff_l4(62,97)= -1.0351734608e-01
       coeff_l4(62,98)=  7.1366503835e-02
       coeff_l4(62,99)= -4.8035018146e-02
       coeff_l4(62,100)= -4.0448378772e-02
       coeff_l4(62,101)=  2.7429681271e-02
       coeff_l4(62,102)=  6.3064768910e-02
       coeff_l4(62,103)=  1.1043199897e-01
       coeff_l4(62,104)=  4.8494163901e-02
       coeff_l4(62,105)= -2.5118678808e-01
       coeff_l4(63,1)= -3.0995571613e-01
       coeff_l4(63,2)= -3.4956327081e-01
       coeff_l4(63,3)=  2.0217441022e-02
       coeff_l4(63,4)= -2.5767892599e-01
       coeff_l4(63,5)= -2.6682925224e-01
       coeff_l4(63,6)= -4.1366326809e-01
       coeff_l4(63,7)= -9.6488632262e-02
       coeff_l4(63,8)= -1.8895918131e-01
       coeff_l4(63,9)=  5.3047955036e-01
       coeff_l4(63,10)=  3.3516409993e-01
       coeff_l4(63,11)= -4.8581838608e-01
       coeff_l4(63,12)= -8.1599527597e-01
       coeff_l4(63,13)=  5.8438920975e-01
       coeff_l4(63,14)=  2.8379237652e-01
       coeff_l4(63,15)= -2.1025866270e-01
       coeff_l4(63,16)=  4.1704501957e-02
       coeff_l4(63,17)= -3.2025918365e-02
       coeff_l4(63,18)= -2.2172807157e-01
       coeff_l4(63,19)=  1.6912942752e-02
       coeff_l4(63,20)= -1.4799377322e-01
       coeff_l4(63,21)=  2.0748186111e-01
       coeff_l4(63,22)= -2.3284794763e-02
       coeff_l4(63,23)= -3.6032384634e-01
       coeff_l4(63,24)= -1.7859350145e-01
       coeff_l4(63,25)=  5.7310611010e-03
       coeff_l4(63,26)=  6.7974500358e-02
       coeff_l4(63,27)=  3.5926771164e-01
       coeff_l4(63,28)= -4.2238968611e-01
       coeff_l4(63,29)= -9.2738056555e-03
       coeff_l4(63,30)=  1.6704468429e-01
       coeff_l4(63,31)=  8.9739702642e-02
       coeff_l4(63,32)= -7.0372261107e-02
       coeff_l4(63,33)=  1.9669799507e-01
       coeff_l4(63,34)=  2.3601619899e-01
       coeff_l4(63,35)= -5.6174758822e-02
       coeff_l4(63,36)= -3.2892206311e-01
       coeff_l4(63,37)= -2.2078624368e-01
       coeff_l4(63,38)= -7.3803693056e-02
       coeff_l4(63,39)= -1.9991129637e-01
       coeff_l4(63,40)=  1.6289929044e-04
       coeff_l4(63,41)=  3.5386246443e-01
       coeff_l4(63,42)= -1.2818729877e-01
       coeff_l4(63,43)= -3.3549451828e-01
       coeff_l4(63,44)=  4.1579096578e-03
       coeff_l4(63,45)= -3.3207339048e-01
       coeff_l4(63,46)= -3.3238676190e-01
       coeff_l4(63,47)= -1.0042699426e-01
       coeff_l4(63,48)= -4.1973668337e-01
       coeff_l4(63,49)=  4.2401183397e-02
       coeff_l4(63,50)=  5.8957147598e-01
       coeff_l4(63,51)= -1.0350946337e-01
       coeff_l4(63,52)= -2.9293006659e-01
       coeff_l4(63,53)= -9.5835946500e-02
       coeff_l4(63,54)=  8.9501313865e-02
       coeff_l4(63,55)=  1.0742271692e-01
       coeff_l4(63,56)= -2.3934879899e-01
       coeff_l4(63,57)= -2.2021038830e-01
       coeff_l4(63,58)= -4.3659350276e-01
       coeff_l4(63,59)= -1.9112634659e-01
       coeff_l4(63,60)=  7.9831168056e-02
       coeff_l4(63,61)= -1.2297838926e-01
       coeff_l4(63,62)=  2.0604154095e-02
       coeff_l4(63,63)=  4.0222637355e-02
       coeff_l4(63,64)= -3.7687104940e-01
       coeff_l4(63,65)=  2.1156513691e-01
       coeff_l4(63,66)= -1.4106968045e-01
       coeff_l4(63,67)= -3.9297452569e-01
       coeff_l4(63,68)= -2.9916802049e-01
       coeff_l4(63,69)=  9.3988671899e-02
       coeff_l4(63,70)= -6.4671248198e-02
       coeff_l4(63,71)=  1.2270334363e-01
       coeff_l4(63,72)= -1.7145636678e-01
       coeff_l4(63,73)= -5.1131552458e-01
       coeff_l4(63,74)= -9.7555741668e-02
       coeff_l4(63,75)= -3.6912307143e-01
       coeff_l4(63,76)=  7.9497829080e-02
       coeff_l4(63,77)=  7.6321445405e-02
       coeff_l4(63,78)= -3.1247492880e-02
       coeff_l4(63,79)=  2.1079470217e-01
       coeff_l4(63,80)= -1.0874097049e-01
       coeff_l4(63,81)=  2.9431203008e-01
       coeff_l4(63,82)= -4.5687073469e-01
       coeff_l4(63,83)= -1.6262394190e-01
       coeff_l4(63,84)=  1.5594193339e-01
       coeff_l4(63,85)=  2.4309283495e-01
       coeff_l4(63,86)= -2.6957818866e-01
       coeff_l4(63,87)=  2.4234887958e-01
       coeff_l4(63,88)= -1.3244952261e-01
       coeff_l4(63,89)=  1.7660273612e-01
       coeff_l4(63,90)= -8.0935060978e-03
       coeff_l4(63,91)=  2.2402203083e-01
       coeff_l4(63,92)=  9.3084883690e-01
       coeff_l4(63,93)= -3.1646989286e-02
       coeff_l4(63,94)=  1.9557851553e-01
       coeff_l4(63,95)= -9.1321063042e-01
       coeff_l4(63,96)= -1.1961974204e-01
       coeff_l4(63,97)=  4.9217604101e-03
       coeff_l4(63,98)= -2.5035670400e-01
       coeff_l4(63,99)=  9.1674871743e-02
       coeff_l4(63,100)= -3.6764597893e-01
       coeff_l4(63,101)= -5.9032344818e-01
       coeff_l4(63,102)= -3.4301560372e-02
       coeff_l4(63,103)= -2.9410091043e-01
       coeff_l4(63,104)= -1.4861869626e-02
       coeff_l4(63,105)= -8.3391612768e-01
       coeff_l4(64,1)= -4.6533554792e-01
       coeff_l4(64,2)= -1.3180679083e-01
       coeff_l4(64,3)= -2.4814434350e-02
       coeff_l4(64,4)=  7.9917587340e-02
       coeff_l4(64,5)=  6.8577177823e-02
       coeff_l4(64,6)=  1.0126323253e-01
       coeff_l4(64,7)= -1.7304231226e-01
       coeff_l4(64,8)= -2.2815835476e-01
       coeff_l4(64,9)=  1.7196834087e-01
       coeff_l4(64,10)=  8.7880920619e-03
       coeff_l4(64,11)=  2.5841283798e-01
       coeff_l4(64,12)=  6.5469819307e-01
       coeff_l4(64,13)=  3.9773961902e-01
       coeff_l4(64,14)=  2.9935079813e-01
       coeff_l4(64,15)=  3.5980346799e-01
       coeff_l4(64,16)=  1.6740474105e-01
       coeff_l4(64,17)=  4.8001885414e-01
       coeff_l4(64,18)=  9.3814410269e-02
       coeff_l4(64,19)=  6.2089230865e-02
       coeff_l4(64,20)=  6.6347710788e-02
       coeff_l4(64,21)= -9.9655225873e-02
       coeff_l4(64,22)=  1.3494761288e-01
       coeff_l4(64,23)= -3.0839049816e-01
       coeff_l4(64,24)=  2.0398894325e-02
       coeff_l4(64,25)=  1.6098640859e-01
       coeff_l4(64,26)=  2.0157076418e-01
       coeff_l4(64,27)=  1.1178753525e-01
       coeff_l4(64,28)= -2.1538905799e-01
       coeff_l4(64,29)= -1.9502261654e-02
       coeff_l4(64,30)=  1.5150474012e-01
       coeff_l4(64,31)=  3.7002004683e-02
       coeff_l4(64,32)= -1.7515413463e-02
       coeff_l4(64,33)= -2.9116939753e-02
       coeff_l4(64,34)=  2.4620180950e-02
       coeff_l4(64,35)= -5.4064024240e-02
       coeff_l4(64,36)= -1.6210047901e-01
       coeff_l4(64,37)= -1.1591573060e-01
       coeff_l4(64,38)=  7.8571803868e-02
       coeff_l4(64,39)= -2.6722264290e-01
       coeff_l4(64,40)= -1.9321705401e-01
       coeff_l4(64,41)= -1.9311119616e-01
       coeff_l4(64,42)=  1.8036204576e-01
       coeff_l4(64,43)= -4.4495169073e-02
       coeff_l4(64,44)=  1.9253768027e-01
       coeff_l4(64,45)=  1.4558984339e-01
       coeff_l4(64,46)=  6.5834179521e-02
       coeff_l4(64,47)= -9.3760490417e-02
       coeff_l4(64,48)=  3.7427376956e-02
       coeff_l4(64,49)=  5.2475589328e-03
       coeff_l4(64,50)= -3.6536920071e-01
       coeff_l4(64,51)=  1.1747550219e-02
       coeff_l4(64,52)= -1.2442617863e-01
       coeff_l4(64,53)=  1.1769241840e-01
       coeff_l4(64,54)= -4.4368929230e-03
       coeff_l4(64,55)=  2.9931163415e-02
       coeff_l4(64,56)= -4.9886040390e-02
       coeff_l4(64,57)= -3.9052221924e-02
       coeff_l4(64,58)=  6.4178705215e-02
       coeff_l4(64,59)= -7.5862795115e-02
       coeff_l4(64,60)= -3.3705413342e-01
       coeff_l4(64,61)= -1.4969667792e-01
       coeff_l4(64,62)=  1.0423927009e-01
       coeff_l4(64,63)= -1.0046017170e-01
       coeff_l4(64,64)= -3.6165508628e-01
       coeff_l4(64,65)= -1.3202923536e-01
       coeff_l4(64,66)=  2.6907888055e-01
       coeff_l4(64,67)= -1.3636791706e-01
       coeff_l4(64,68)=  1.0925593972e-01
       coeff_l4(64,69)=  1.7534342408e-01
       coeff_l4(64,70)=  2.4996812642e-01
       coeff_l4(64,71)= -2.5660473108e-01
       coeff_l4(64,72)= -2.3152689636e-01
       coeff_l4(64,73)= -5.2524387836e-02
       coeff_l4(64,74)=  2.4689617753e-01
       coeff_l4(64,75)=  2.3786506057e-01
       coeff_l4(64,76)=  4.6951737255e-02
       coeff_l4(64,77)= -2.4176359177e-01
       coeff_l4(64,78)=  6.0721412301e-02
       coeff_l4(64,79)=  9.4212755561e-02
       coeff_l4(64,80)=  1.5403698385e-01
       coeff_l4(64,81)=  6.9180332124e-02
       coeff_l4(64,82)=  4.3407011777e-02
       coeff_l4(64,83)= -6.0932487249e-02
       coeff_l4(64,84)= -7.4971236289e-02
       coeff_l4(64,85)=  8.3482220769e-02
       coeff_l4(64,86)= -2.4993939698e-01
       coeff_l4(64,87)= -3.3544123173e-02
       coeff_l4(64,88)=  1.1764497310e-01
       coeff_l4(64,89)= -1.3334596157e-01
       coeff_l4(64,90)= -2.4172559381e-02
       coeff_l4(64,91)=  2.0069555938e-01
       coeff_l4(64,92)=  1.7635241151e-01
       coeff_l4(64,93)=  1.2336128205e-01
       coeff_l4(64,94)=  5.7812958956e-01
       coeff_l4(64,95)= -5.9917245060e-02
       coeff_l4(64,96)=  1.4780418575e-01
       coeff_l4(64,97)=  3.1412914395e-01
       coeff_l4(64,98)=  2.1791879833e-01
       coeff_l4(64,99)= -4.0084578097e-02
       coeff_l4(64,100)= -3.2386603951e-01
       coeff_l4(64,101)= -1.4833271503e-02
       coeff_l4(64,102)=  1.5174940228e-01
       coeff_l4(64,103)= -7.9690620303e-02
       coeff_l4(64,104)= -7.5372166932e-02
       coeff_l4(64,105)= -2.1595914662e-01
       coeff_l4(65,1)= -5.1771140099e-01
       coeff_l4(65,2)= -5.1252007484e-01
       coeff_l4(65,3)= -6.8158608675e-01
       coeff_l4(65,4)= -6.1738938093e-01
       coeff_l4(65,5)= -6.4896512032e-01
       coeff_l4(65,6)= -6.4867377281e-01
       coeff_l4(65,7)= -5.9362119436e-01
       coeff_l4(65,8)= -3.3262512088e-01
       coeff_l4(65,9)= -2.7156880498e-01
       coeff_l4(65,10)= -1.3441208005e-01
       coeff_l4(65,11)= -2.2722850740e-01
       coeff_l4(65,12)= -1.2903012335e-01
       coeff_l4(65,13)=  5.5747244507e-02
       coeff_l4(65,14)= -2.2802197933e-01
       coeff_l4(65,15)=  2.1971379220e-01
       coeff_l4(65,16)=  8.3956167102e-02
       coeff_l4(65,17)= -8.6882211268e-02
       coeff_l4(65,18)=  1.2601076066e-01
       coeff_l4(65,19)= -4.4146105647e-02
       coeff_l4(65,20)= -3.0258839950e-02
       coeff_l4(65,21)= -5.9877622873e-02
       coeff_l4(65,22)=  1.5281449258e-01
       coeff_l4(65,23)=  5.9904921800e-02
       coeff_l4(65,24)=  1.1409813538e-02
       coeff_l4(65,25)= -1.2597399950e-01
       coeff_l4(65,26)=  1.4351132512e-01
       coeff_l4(65,27)=  7.7738448977e-02
       coeff_l4(65,28)= -1.5347859263e-01
       coeff_l4(65,29)= -6.8041943014e-02
       coeff_l4(65,30)= -7.6438121498e-02
       coeff_l4(65,31)= -1.4371246099e-01
       coeff_l4(65,32)=  8.6649157107e-02
       coeff_l4(65,33)=  6.1037275940e-02
       coeff_l4(65,34)=  3.5208232701e-02
       coeff_l4(65,35)=  8.4088519216e-02
       coeff_l4(65,36)=  7.2966255248e-02
       coeff_l4(65,37)= -7.6030552387e-02
       coeff_l4(65,38)= -1.2910275836e-04
       coeff_l4(65,39)= -1.0239263996e-02
       coeff_l4(65,40)=  1.3801071048e-01
       coeff_l4(65,41)= -2.0434718579e-02
       coeff_l4(65,42)= -1.1252231896e-01
       coeff_l4(65,43)=  1.0855349898e-01
       coeff_l4(65,44)= -1.5546526015e-01
       coeff_l4(65,45)=  6.9825291634e-02
       coeff_l4(65,46)= -4.6906560659e-02
       coeff_l4(65,47)=  1.3613775373e-01
       coeff_l4(65,48)=  2.8912736103e-02
       coeff_l4(65,49)= -1.3741892576e-01
       coeff_l4(65,50)=  8.3430092782e-03
       coeff_l4(65,51)=  2.3188408464e-02
       coeff_l4(65,52)=  5.5977106094e-02
       coeff_l4(65,53)=  1.4114518464e-01
       coeff_l4(65,54)=  4.1100814939e-02
       coeff_l4(65,55)=  2.1036639810e-02
       coeff_l4(65,56)= -4.8908784986e-02
       coeff_l4(65,57)=  1.6716320813e-01
       coeff_l4(65,58)= -1.3769169152e-01
       coeff_l4(65,59)=  1.2628771365e-01
       coeff_l4(65,60)= -1.2928879261e-01
       coeff_l4(65,61)=  8.3141118288e-02
       coeff_l4(65,62)=  2.3932972550e-01
       coeff_l4(65,63)=  5.2127536386e-02
       coeff_l4(65,64)= -1.5748091042e-01
       coeff_l4(65,65)=  8.9744366705e-02
       coeff_l4(65,66)= -5.6417457759e-02
       coeff_l4(65,67)=  9.9465355277e-02
       coeff_l4(65,68)= -1.5770340338e-02
       coeff_l4(65,69)= -1.4605680108e-01
       coeff_l4(65,70)= -1.2444521487e-01
       coeff_l4(65,71)=  3.9971932769e-02
       coeff_l4(65,72)=  1.0434461385e-01
       coeff_l4(65,73)= -4.0217161179e-02
       coeff_l4(65,74)=  3.0135735869e-02
       coeff_l4(65,75)= -7.8068464994e-02
       coeff_l4(65,76)= -3.6240130663e-02
       coeff_l4(65,77)=  1.1377384514e-01
       coeff_l4(65,78)= -6.9828115404e-02
       coeff_l4(65,79)=  1.4725528657e-01
       coeff_l4(65,80)= -8.2494452596e-02
       coeff_l4(65,81)=  2.8137139976e-02
       coeff_l4(65,82)= -7.9145722091e-02
       coeff_l4(65,83)= -1.8273618817e-01
       coeff_l4(65,84)= -7.2093896568e-02
       coeff_l4(65,85)= -1.4986330271e-01
       coeff_l4(65,86)= -1.6525873542e-01
       coeff_l4(65,87)=  1.1748835444e-01
       coeff_l4(65,88)= -5.9502996504e-02
       coeff_l4(65,89)=  5.6142944843e-02
       coeff_l4(65,90)=  1.2459669262e-02
       coeff_l4(65,91)= -7.6906792819e-02
       coeff_l4(65,92)= -1.1616003513e-01
       coeff_l4(65,93)= -1.2381865829e-01
       coeff_l4(65,94)=  2.3419417441e-02
       coeff_l4(65,95)=  2.5732752681e-01
       coeff_l4(65,96)= -1.1920864880e-01
       coeff_l4(65,97)= -2.3559106886e-01
       coeff_l4(65,98)=  8.7081365287e-02
       coeff_l4(65,99)=  3.8291264325e-02
       coeff_l4(65,100)= -1.1937897652e-01
       coeff_l4(65,101)=  9.2576980591e-02
       coeff_l4(65,102)= -9.4496179372e-03
       coeff_l4(65,103)= -5.0457637757e-02
       coeff_l4(65,104)=  9.0908117592e-02
       coeff_l4(65,105)=  1.0106351972e-01
       coeff_l4(66,1)= -8.2701939344e-01
       coeff_l4(66,2)= -7.1721750498e-01
       coeff_l4(66,3)= -6.0563760996e-01
       coeff_l4(66,4)= -8.3420646191e-01
       coeff_l4(66,5)= -6.7770558596e-01
       coeff_l4(66,6)= -6.9768881798e-01
       coeff_l4(66,7)= -5.2796322107e-01
       coeff_l4(66,8)= -6.9592297077e-01
       coeff_l4(66,9)= -2.9197121039e-02
       coeff_l4(66,10)= -3.5192888975e-01
       coeff_l4(66,11)=  1.3392375410e-01
       coeff_l4(66,12)= -8.4045492113e-02
       coeff_l4(66,13)=  1.7942832410e-01
       coeff_l4(66,14)= -3.6133214831e-01
       coeff_l4(66,15)=  6.1697177589e-02
       coeff_l4(66,16)= -6.8539686501e-02
       coeff_l4(66,17)= -1.5101459622e-01
       coeff_l4(66,18)=  2.0068199933e-01
       coeff_l4(66,19)= -2.1221810579e-01
       coeff_l4(66,20)=  5.6408088654e-02
       coeff_l4(66,21)=  8.4640599787e-02
       coeff_l4(66,22)= -1.0729409754e-01
       coeff_l4(66,23)= -1.5691213310e-02
       coeff_l4(66,24)= -9.6271201968e-02
       coeff_l4(66,25)=  1.0088075884e-02
       coeff_l4(66,26)= -1.3083431125e-01
       coeff_l4(66,27)=  9.9100075662e-02
       coeff_l4(66,28)=  1.0131672025e-02
       coeff_l4(66,29)=  1.0049056262e-01
       coeff_l4(66,30)= -4.0939878672e-02
       coeff_l4(66,31)= -1.1329182982e-01
       coeff_l4(66,32)= -7.0879422128e-02
       coeff_l4(66,33)=  2.5798422284e-03
       coeff_l4(66,34)=  3.5889163613e-02
       coeff_l4(66,35)=  5.7896994986e-03
       coeff_l4(66,36)= -5.6498542428e-02
       coeff_l4(66,37)=  5.5509291589e-02
       coeff_l4(66,38)= -1.2595473230e-01
       coeff_l4(66,39)= -6.9095872343e-02
       coeff_l4(66,40)=  9.6745096147e-02
       coeff_l4(66,41)=  1.8113154173e-01
       coeff_l4(66,42)= -5.3999114782e-02
       coeff_l4(66,43)= -1.0951633006e-01
       coeff_l4(66,44)= -1.4234326780e-01
       coeff_l4(66,45)=  2.0658930764e-02
       coeff_l4(66,46)= -1.2193911709e-02
       coeff_l4(66,47)= -5.2939116955e-02
       coeff_l4(66,48)=  4.4914553873e-03
       coeff_l4(66,49)=  8.0418959260e-02
       coeff_l4(66,50)=  6.4720965922e-02
       coeff_l4(66,51)= -1.0037312284e-02
       coeff_l4(66,52)= -1.1212541163e-01
       coeff_l4(66,53)=  1.6284735501e-01
       coeff_l4(66,54)= -8.0497793853e-02
       coeff_l4(66,55)= -1.0842927545e-01
       coeff_l4(66,56)= -8.5649751127e-02
       coeff_l4(66,57)= -1.5166787803e-01
       coeff_l4(66,58)= -1.8953353167e-01
       coeff_l4(66,59)=  7.5405150652e-02
       coeff_l4(66,60)=  5.6535080075e-02
       coeff_l4(66,61)= -9.7958855331e-02
       coeff_l4(66,62)=  1.7084643245e-02
       coeff_l4(66,63)= -2.1729110181e-01
       coeff_l4(66,64)= -5.2653782070e-02
       coeff_l4(66,65)=  7.4137218297e-02
       coeff_l4(66,66)=  1.1033558100e-01
       coeff_l4(66,67)= -2.5249620900e-02
       coeff_l4(66,68)=  1.5351548791e-01
       coeff_l4(66,69)= -2.1264718845e-02
       coeff_l4(66,70)=  2.2582054138e-02
       coeff_l4(66,71)=  3.4027621150e-02
       coeff_l4(66,72)=  7.6945722103e-02
       coeff_l4(66,73)= -1.1132466793e-01
       coeff_l4(66,74)= -2.8922464699e-02
       coeff_l4(66,75)= -8.8661544025e-02
       coeff_l4(66,76)= -9.7918726504e-02
       coeff_l4(66,77)=  3.6630339921e-02
       coeff_l4(66,78)= -3.4103188664e-02
       coeff_l4(66,79)=  2.9491780326e-02
       coeff_l4(66,80)= -3.4334313124e-02
       coeff_l4(66,81)=  6.0783494264e-02
       coeff_l4(66,82)= -5.6485235691e-02
       coeff_l4(66,83)=  6.5517850220e-02
       coeff_l4(66,84)= -6.0020253062e-02
       coeff_l4(66,85)=  3.7340138108e-02
       coeff_l4(66,86)=  2.1349566057e-02
       coeff_l4(66,87)= -1.2896274030e-01
       coeff_l4(66,88)=  1.1537109315e-01
       coeff_l4(66,89)= -9.3481399119e-02
       coeff_l4(66,90)= -1.6936796904e-01
       coeff_l4(66,91)= -9.2483624816e-02
       coeff_l4(66,92)= -1.1020943522e-01
       coeff_l4(66,93)=  1.0066729039e-01
       coeff_l4(66,94)=  2.9755230993e-02
       coeff_l4(66,95)= -6.0957062989e-02
       coeff_l4(66,96)=  4.2851623148e-02
       coeff_l4(66,97)=  1.6098207235e-01
       coeff_l4(66,98)= -5.9411816299e-02
       coeff_l4(66,99)=  7.3732584715e-02
       coeff_l4(66,100)=  8.3586290479e-02
       coeff_l4(66,101)=  2.8740020469e-02
       coeff_l4(66,102)=  1.8941518664e-01
       coeff_l4(66,103)= -3.1621064991e-03
       coeff_l4(66,104)= -6.5160460770e-02
       coeff_l4(66,105)= -1.3488978148e-01
       coeff_l4(67,1)=  1.6700074077e-01
       coeff_l4(67,2)= -2.2407354414e-01
       coeff_l4(67,3)= -9.0291008353e-02
       coeff_l4(67,4)=  1.4919256791e-02
       coeff_l4(67,5)=  5.2346348763e-01
       coeff_l4(67,6)= -1.3808617368e-02
       coeff_l4(67,7)=  1.1056136340e-01
       coeff_l4(67,8)=  9.2761874199e-01
       coeff_l4(67,9)= -1.7002105713e-02
       coeff_l4(67,10)=  1.2488203049e+00
       coeff_l4(67,11)=  3.9468148351e-01
       coeff_l4(67,12)= -2.1586987376e-01
       coeff_l4(67,13)=  4.0200788528e-02
       coeff_l4(67,14)= -3.4684577584e-01
       coeff_l4(67,15)= -1.0078788549e-01
       coeff_l4(67,16)=  1.9243776798e-01
       coeff_l4(67,17)= -4.8852139711e-01
       coeff_l4(67,18)=  8.2915079594e-01
       coeff_l4(67,19)=  5.8236843348e-01
       coeff_l4(67,20)= -4.8769405484e-01
       coeff_l4(67,21)=  3.0540791154e-01
       coeff_l4(67,22)=  1.3298594952e-01
       coeff_l4(67,23)= -3.7612235546e-01
       coeff_l4(67,24)=  1.2611757517e+00
       coeff_l4(67,25)= -1.0685440302e+00
       coeff_l4(67,26)=  2.1170775592e-01
       coeff_l4(67,27)= -1.4679322243e+00
       coeff_l4(67,28)=  3.3392678946e-02
       coeff_l4(67,29)=  4.0650075674e-01
       coeff_l4(67,30)= -4.8960441351e-01
       coeff_l4(67,31)=  2.7379164100e-01
       coeff_l4(67,32)=  2.0949600637e-01
       coeff_l4(67,33)= -1.0921000242e+00
       coeff_l4(67,34)= -1.5430566072e+00
       coeff_l4(67,35)= -4.9002861977e-01
       coeff_l4(67,36)=  2.2443021834e-01
       coeff_l4(67,37)= -5.6566160917e-01
       coeff_l4(67,38)= -2.7428603172e-01
       coeff_l4(67,39)= -6.2655115128e-01
       coeff_l4(67,40)= -2.4054329097e-01
       coeff_l4(67,41)= -1.0603533685e-01
       coeff_l4(67,42)=  1.6565507650e+00
       coeff_l4(67,43)= -3.6251363158e-01
       coeff_l4(67,44)=  1.4537386894e+00
       coeff_l4(67,45)= -7.4962073565e-01
       coeff_l4(67,46)= -8.2091474533e-01
       coeff_l4(67,47)= -2.9192453623e-01
       coeff_l4(67,48)=  4.8951470852e-01
       coeff_l4(67,49)= -6.7595922947e-01
       coeff_l4(67,50)= -9.7535437346e-01
       coeff_l4(67,51)=  4.7201001644e-01
       coeff_l4(67,52)=  4.6045374870e-01
       coeff_l4(67,53)= -2.4298630655e-01
       coeff_l4(67,54)= -9.3402194977e-01
       coeff_l4(67,55)=  1.2035759687e+00
       coeff_l4(67,56)= -9.7312021255e-01
       coeff_l4(67,57)=  1.0685260296e+00
       coeff_l4(67,58)=  4.7497478127e-01
       coeff_l4(67,59)= -6.5116339922e-01
       coeff_l4(67,60)= -2.1789731979e+00
       coeff_l4(67,61)= -2.5556153059e-01
       coeff_l4(67,62)=  2.8247976303e-01
       coeff_l4(67,63)=  8.8646131754e-01
       coeff_l4(67,64)= -1.8464550376e-01
       coeff_l4(67,65)= -4.3197849393e-01
       coeff_l4(67,66)= -2.0020659268e-01
       coeff_l4(67,67)=  6.2846839428e-02
       coeff_l4(67,68)=  7.9191946983e-01
       coeff_l4(67,69)=  9.4593548775e-01
       coeff_l4(67,70)=  7.3027998209e-01
       coeff_l4(67,71)= -5.9245622158e-01
       coeff_l4(67,72)=  2.3144926131e-01
       coeff_l4(67,73)=  1.1705292761e-01
       coeff_l4(67,74)=  9.0659201145e-01
       coeff_l4(67,75)=  3.8861656189e-01
       coeff_l4(67,76)=  5.8494514227e-01
       coeff_l4(67,77)=  1.3278349638e+00
       coeff_l4(67,78)=  3.0009093881e-01
       coeff_l4(67,79)=  4.3986457586e-01
       coeff_l4(67,80)=  1.8514344096e-01
       coeff_l4(67,81)= -1.2052330971e+00
       coeff_l4(67,82)= -1.7812118530e+00
       coeff_l4(67,83)= -3.5216414928e-01
       coeff_l4(67,84)= -2.4018684030e-01
       coeff_l4(67,85)=  8.9021068811e-01
       coeff_l4(67,86)= -1.4497344494e+00
       coeff_l4(67,87)=  1.0116926432e+00
       coeff_l4(67,88)= -1.0940076113e+00
       coeff_l4(67,89)= -4.2151501775e-01
       coeff_l4(67,90)= -1.0978150368e+00
       coeff_l4(67,91)=  1.2989451885e+00
       coeff_l4(67,92)=  1.5573536046e-02
       coeff_l4(67,93)=  1.2758471072e-01
       coeff_l4(67,94)=  1.3319988251e+00
       coeff_l4(67,95)= -1.3781503439e+00
       coeff_l4(67,96)=  6.4514689147e-02
       coeff_l4(67,97)= -4.4365718961e-01
       coeff_l4(67,98)= -3.2701313496e-01
       coeff_l4(67,99)= -4.2210790515e-01
       coeff_l4(67,100)= -1.1006662846e+00
       coeff_l4(67,101)=  5.0810956955e-01
       coeff_l4(67,102)= -4.2410367727e-01
       coeff_l4(67,103)= -5.3428554535e-01
       coeff_l4(67,104)=  4.4387346506e-01
       coeff_l4(67,105)= -1.1689194441e+00
       coeff_l4(68,1)=  5.9476888180e-01
       coeff_l4(68,2)= -3.0650565028e-01
       coeff_l4(68,3)= -2.8991878033e-01
       coeff_l4(68,4)=  2.1618041396e-01
       coeff_l4(68,5)= -2.2663231194e-01
       coeff_l4(68,6)= -2.7699798346e-02
       coeff_l4(68,7)=  2.8466942906e-01
       coeff_l4(68,8)=  4.1353476048e-01
       coeff_l4(68,9)=  2.8722992539e-01
       coeff_l4(68,10)=  8.2599258423e-01
       coeff_l4(68,11)=  4.7991225123e-01
       coeff_l4(68,12)= -1.0850808583e-02
       coeff_l4(68,13)=  1.2547925115e-01
       coeff_l4(68,14)=  5.8434963226e-01
       coeff_l4(68,15)=  1.7542858422e-01
       coeff_l4(68,16)=  1.6288644075e-01
       coeff_l4(68,17)= -2.1045909822e-01
       coeff_l4(68,18)= -2.3407985270e-01
       coeff_l4(68,19)=  1.5251730382e-01
       coeff_l4(68,20)=  8.3856813610e-02
       coeff_l4(68,21)=  3.2436329126e-01
       coeff_l4(68,22)= -9.7798846662e-02
       coeff_l4(68,23)=  9.1610096395e-02
       coeff_l4(68,24)=  3.5628566146e-01
       coeff_l4(68,25)= -5.5682981014e-01
       coeff_l4(68,26)=  2.0403608680e-01
       coeff_l4(68,27)= -1.2780423462e-01
       coeff_l4(68,28)=  2.5353786349e-01
       coeff_l4(68,29)=  3.3533433080e-01
       coeff_l4(68,30)= -3.5646554828e-01
       coeff_l4(68,31)= -2.7598115802e-01
       coeff_l4(68,32)=  5.4361581802e-01
       coeff_l4(68,33)= -3.8395613432e-02
       coeff_l4(68,34)= -1.9876131415e-01
       coeff_l4(68,35)=  2.0496245474e-02
       coeff_l4(68,36)=  7.7493511140e-02
       coeff_l4(68,37)=  1.8613019586e-01
       coeff_l4(68,38)= -1.4588432014e-01
       coeff_l4(68,39)=  2.6612967253e-01
       coeff_l4(68,40)= -4.5944225043e-02
       coeff_l4(68,41)= -5.8230179548e-01
       coeff_l4(68,42)= -3.7963721901e-02
       coeff_l4(68,43)= -3.5734820366e-01
       coeff_l4(68,44)=  3.7650573254e-01
       coeff_l4(68,45)= -3.5292440653e-01
       coeff_l4(68,46)= -9.7060389817e-02
       coeff_l4(68,47)= -2.8490711004e-02
       coeff_l4(68,48)=  5.8012727648e-02
       coeff_l4(68,49)=  2.1334308386e-01
       coeff_l4(68,50)=  1.4013932645e-01
       coeff_l4(68,51)= -1.0850240290e-01
       coeff_l4(68,52)= -3.0569633842e-01
       coeff_l4(68,53)= -3.2058298588e-01
       coeff_l4(68,54)= -1.2097059190e-01
       coeff_l4(68,55)=  1.6807806492e-01
       coeff_l4(68,56)= -4.1809689999e-01
       coeff_l4(68,57)=  1.7853488028e-01
       coeff_l4(68,58)= -4.4662919641e-01
       coeff_l4(68,59)=  2.2180072963e-01
       coeff_l4(68,60)= -3.9307693951e-03
       coeff_l4(68,61)= -5.5413167924e-02
       coeff_l4(68,62)=  1.2203612179e-01
       coeff_l4(68,63)=  3.2270628214e-01
       coeff_l4(68,64)=  7.3899686337e-02
       coeff_l4(68,65)= -4.3238271028e-02
       coeff_l4(68,66)=  3.5731562972e-01
       coeff_l4(68,67)= -2.2666162252e-01
       coeff_l4(68,68)= -1.8775504827e-01
       coeff_l4(68,69)=  1.3570903242e-01
       coeff_l4(68,70)= -1.0925445706e-01
       coeff_l4(68,71)= -3.9109584689e-01
       coeff_l4(68,72)=  2.0088208839e-02
       coeff_l4(68,73)= -1.6648285091e-01
       coeff_l4(68,74)= -1.6163402796e-01
       coeff_l4(68,75)= -2.3434135318e-01
       coeff_l4(68,76)= -4.1951432824e-02
       coeff_l4(68,77)=  2.8248399496e-01
       coeff_l4(68,78)= -4.4589240104e-02
       coeff_l4(68,79)= -4.2647626996e-01
       coeff_l4(68,80)= -5.6002110243e-02
       coeff_l4(68,81)= -1.1828637123e-01
       coeff_l4(68,82)= -4.3478336930e-01
       coeff_l4(68,83)= -4.8511961102e-01
       coeff_l4(68,84)= -2.3712249100e-01
       coeff_l4(68,85)=  3.3424851298e-01
       coeff_l4(68,86)=  3.2806301117e-01
       coeff_l4(68,87)=  2.5693359971e-01
       coeff_l4(68,88)=  1.0158035159e-01
       coeff_l4(68,89)=  4.9873080850e-01
       coeff_l4(68,90)= -1.8872131407e-01
       coeff_l4(68,91)= -5.7713901997e-01
       coeff_l4(68,92)=  1.6768772900e-01
       coeff_l4(68,93)= -1.0039854050e-01
       coeff_l4(68,94)= -8.6487770081e-02
       coeff_l4(68,95)= -5.1327973604e-02
       coeff_l4(68,96)= -5.4168768227e-02
       coeff_l4(68,97)=  3.4254491329e-01
       coeff_l4(68,98)= -4.2203325033e-01
       coeff_l4(68,99)= -3.3686730266e-01
       coeff_l4(68,100)= -1.0960228741e-02
       coeff_l4(68,101)= -3.0118364096e-01
       coeff_l4(68,102)=  1.9602607191e-01
       coeff_l4(68,103)=  6.1087053269e-02
       coeff_l4(68,104)= -2.9097899795e-01
       coeff_l4(68,105)=  4.4880571961e-01
       coeff_l4(69,1)=  1.2384065390e+00
       coeff_l4(69,2)=  1.0456380844e+00
       coeff_l4(69,3)=  8.3956891298e-01
       coeff_l4(69,4)=  1.0248285532e+00
       coeff_l4(69,5)=  1.0606681108e+00
       coeff_l4(69,6)=  7.5139415264e-01
       coeff_l4(69,7)=  2.2333393991e-01
       coeff_l4(69,8)= -2.0367037505e-02
       coeff_l4(69,9)=  7.9985219240e-01
       coeff_l4(69,10)=  8.8971024752e-01
       coeff_l4(69,11)=  5.5194759369e-01
       coeff_l4(69,12)=  4.9460425973e-01
       coeff_l4(69,13)=  4.7644311190e-01
       coeff_l4(69,14)=  5.5065488815e-01
       coeff_l4(69,15)= -1.1003478616e-01
       coeff_l4(69,16)=  9.8160915077e-02
       coeff_l4(69,17)=  9.9932909012e-02
       coeff_l4(69,18)= -7.2896517813e-02
       coeff_l4(69,19)= -2.0380200818e-02
       coeff_l4(69,20)= -8.3929225802e-02
       coeff_l4(69,21)=  1.7471520230e-02
       coeff_l4(69,22)= -3.4418493509e-02
       coeff_l4(69,23)= -6.2266401947e-03
       coeff_l4(69,24)=  1.2742814422e-01
       coeff_l4(69,25)=  1.0790904611e-01
       coeff_l4(69,26)=  7.4419174343e-03
       coeff_l4(69,27)= -7.2749815881e-02
       coeff_l4(69,28)= -5.7616882026e-02
       coeff_l4(69,29)=  7.4558444321e-02
       coeff_l4(69,30)=  1.2292270549e-02
       coeff_l4(69,31)= -4.5840758830e-02
       coeff_l4(69,32)= -6.3045606017e-02
       coeff_l4(69,33)=  1.9846816361e-01
       coeff_l4(69,34)= -3.2022833824e-02
       coeff_l4(69,35)=  9.7346208990e-02
       coeff_l4(69,36)=  6.5954066813e-02
       coeff_l4(69,37)=  5.2803311497e-02
       coeff_l4(69,38)= -6.8684600294e-02
       coeff_l4(69,39)= -1.5925951302e-02
       coeff_l4(69,40)= -6.9660663605e-02
       coeff_l4(69,41)=  1.9390361384e-02
       coeff_l4(69,42)=  1.5601338446e-01
       coeff_l4(69,43)= -5.9646733105e-02
       coeff_l4(69,44)= -1.6522262245e-02
       coeff_l4(69,45)= -1.5479806066e-01
       coeff_l4(69,46)= -6.7008234560e-02
       coeff_l4(69,47)= -2.0028090477e-01
       coeff_l4(69,48)= -7.0178352296e-02
       coeff_l4(69,49)= -2.3024567962e-01
       coeff_l4(69,50)= -9.1708183289e-02
       coeff_l4(69,51)=  9.6353851259e-02
       coeff_l4(69,52)= -1.4669661410e-02
       coeff_l4(69,53)=  2.8521683812e-01
       coeff_l4(69,54)=  2.2521582246e-01
       coeff_l4(69,55)= -4.4574737549e-02
       coeff_l4(69,56)=  7.5203575194e-02
       coeff_l4(69,57)= -1.2863366306e-01
       coeff_l4(69,58)= -6.9683998823e-02
       coeff_l4(69,59)=  2.4843581486e-03
       coeff_l4(69,60)= -5.9700533748e-02
       coeff_l4(69,61)= -7.4307493865e-02
       coeff_l4(69,62)=  1.4694256708e-02
       coeff_l4(69,63)= -6.8805269897e-02
       coeff_l4(69,64)=  1.4974428713e-01
       coeff_l4(69,65)= -1.0692770779e-01
       coeff_l4(69,66)=  3.4183785319e-02
       coeff_l4(69,67)=  1.0013873875e-01
       coeff_l4(69,68)= -9.5783062279e-02
       coeff_l4(69,69)= -8.2184337080e-02
       coeff_l4(69,70)=  1.2942203879e-01
       coeff_l4(69,71)= -1.0007950477e-02
       coeff_l4(69,72)=  9.0816266835e-02
       coeff_l4(69,73)= -1.7287051678e-01
       coeff_l4(69,74)=  1.3913728297e-01
       coeff_l4(69,75)= -1.2682484090e-01
       coeff_l4(69,76)= -4.9307938665e-02
       coeff_l4(69,77)=  6.0853928328e-02
       coeff_l4(69,78)= -1.4828529954e-01
       coeff_l4(69,79)=  1.2367258966e-01
       coeff_l4(69,80)= -3.3630423248e-02
       coeff_l4(69,81)= -9.7336575389e-02
       coeff_l4(69,82)=  1.3924714923e-01
       coeff_l4(69,83)=  3.3052738756e-02
       coeff_l4(69,84)=  1.6225637496e-01
       coeff_l4(69,85)=  5.4769653827e-02
       coeff_l4(69,86)= -1.8692705035e-01
       coeff_l4(69,87)=  1.7879310250e-01
       coeff_l4(69,88)= -1.4314949512e-01
       coeff_l4(69,89)=  1.6861218214e-01
       coeff_l4(69,90)= -8.7458193302e-03
       coeff_l4(69,91)=  7.8876644373e-02
       coeff_l4(69,92)= -1.2937998772e-01
       coeff_l4(69,93)= -2.0250326023e-02
       coeff_l4(69,94)=  4.0027856827e-01
       coeff_l4(69,95)= -2.3752808571e-01
       coeff_l4(69,96)= -1.9618460536e-01
       coeff_l4(69,97)= -1.6339147091e-01
       coeff_l4(69,98)= -1.9165156782e-01
       coeff_l4(69,99)= -2.6775904000e-02
       coeff_l4(69,100)= -1.8512981012e-02
       coeff_l4(69,101)= -1.4527130872e-02
       coeff_l4(69,102)= -2.0942753181e-02
       coeff_l4(69,103)= -1.4348571002e-01
       coeff_l4(69,104)=  5.6018181145e-02
       coeff_l4(69,105)= -1.9123025239e-01
       coeff_l4(70,1)= -1.4177787304e-01
       coeff_l4(70,2)=  1.8483911455e-01
       coeff_l4(70,3)=  6.0350164771e-02
       coeff_l4(70,4)= -1.4409741759e-01
       coeff_l4(70,5)= -5.6253854185e-02
       coeff_l4(70,6)= -1.5243989229e-01
       coeff_l4(70,7)=  6.1527319252e-02
       coeff_l4(70,8)=  1.0429218411e-01
       coeff_l4(70,9)= -1.4969393611e-01
       coeff_l4(70,10)= -2.5558707118e-01
       coeff_l4(70,11)= -1.1293444633e+00
       coeff_l4(70,12)= -1.1540945768e+00
       coeff_l4(70,13)= -1.1667659283e+00
       coeff_l4(70,14)=  1.0822274536e-01
       coeff_l4(70,15)=  3.4790277481e-02
       coeff_l4(70,16)=  8.5870541632e-02
       coeff_l4(70,17)= -1.4581945539e-01
       coeff_l4(70,18)= -1.2442557141e-02
       coeff_l4(70,19)=  6.6449530423e-02
       coeff_l4(70,20)=  1.5165899880e-02
       coeff_l4(70,21)=  1.6498893499e-02
       coeff_l4(70,22)= -2.1924355999e-02
       coeff_l4(70,23)=  1.0302692652e-01
       coeff_l4(70,24)=  1.1591340881e-03
       coeff_l4(70,25)=  8.8587487116e-03
       coeff_l4(70,26)=  4.4540753588e-03
       coeff_l4(70,27)=  1.4509519562e-02
       coeff_l4(70,28)=  2.6187784970e-02
       coeff_l4(70,29)= -2.1172057837e-02
       coeff_l4(70,30)= -1.9018488005e-02
       coeff_l4(70,31)= -2.4384877179e-03
       coeff_l4(70,32)=  3.2485548407e-02
       coeff_l4(70,33)=  1.3729108870e-01
       coeff_l4(70,34)= -4.1533291340e-02
       coeff_l4(70,35)= -1.4012737572e-01
       coeff_l4(70,36)=  1.3561588526e-01
       coeff_l4(70,37)=  8.7846415117e-03
       coeff_l4(70,38)=  1.2374348193e-01
       coeff_l4(70,39)= -1.3423858583e-01
       coeff_l4(70,40)=  1.0966394097e-01
       coeff_l4(70,41)= -9.4355702400e-02
       coeff_l4(70,42)=  1.6774111986e-01
       coeff_l4(70,43)= -2.5043448433e-02
       coeff_l4(70,44)= -1.7992137372e-01
       coeff_l4(70,45)= -1.2818264961e-01
       coeff_l4(70,46)=  1.0823951662e-01
       coeff_l4(70,47)= -4.5566294342e-02
       coeff_l4(70,48)= -3.9116924745e-04
       coeff_l4(70,49)=  2.1579414606e-02
       coeff_l4(70,50)= -1.2813460827e-01
       coeff_l4(70,51)=  2.0825064182e-01
       coeff_l4(70,52)= -6.9137357175e-02
       coeff_l4(70,53)= -4.9334496260e-02
       coeff_l4(70,54)=  3.8686275482e-02
       coeff_l4(70,55)= -7.9653546214e-02
       coeff_l4(70,56)= -2.7386432886e-01
       coeff_l4(70,57)=  7.5630903244e-02
       coeff_l4(70,58)= -1.4523610473e-01
       coeff_l4(70,59)=  2.9105121270e-02
       coeff_l4(70,60)= -5.1190126687e-02
       coeff_l4(70,61)= -6.1173655093e-02
       coeff_l4(70,62)=  1.2357982993e-01
       coeff_l4(70,63)=  1.6008131206e-01
       coeff_l4(70,64)=  7.7364601195e-02
       coeff_l4(70,65)= -8.6227282882e-02
       coeff_l4(70,66)=  9.8777264357e-02
       coeff_l4(70,67)= -6.4734801650e-02
       coeff_l4(70,68)=  1.8139183521e-02
       coeff_l4(70,69)= -1.6707439721e-01
       coeff_l4(70,70)=  1.1581093818e-01
       coeff_l4(70,71)= -4.0056258440e-02
       coeff_l4(70,72)=  6.7875795066e-03
       coeff_l4(70,73)= -2.0278874785e-02
       coeff_l4(70,74)=  1.0730235279e-01
       coeff_l4(70,75)= -5.5014312267e-02
       coeff_l4(70,76)= -2.3262176663e-02
       coeff_l4(70,77)= -1.3466108590e-02
       coeff_l4(70,78)= -1.8160331249e-01
       coeff_l4(70,79)= -7.8369274735e-02
       coeff_l4(70,80)= -1.3267296553e-01
       coeff_l4(70,81)=  9.7186356783e-02
       coeff_l4(70,82)= -1.0876500607e-01
       coeff_l4(70,83)=  1.2672002614e-01
       coeff_l4(70,84)= -1.4733545482e-01
       coeff_l4(70,85)=  9.4444856048e-02
       coeff_l4(70,86)=  6.3407614827e-02
       coeff_l4(70,87)=  6.9160386920e-02
       coeff_l4(70,88)= -6.2397513539e-02
       coeff_l4(70,89)= -2.8092587367e-02
       coeff_l4(70,90)=  1.9921652973e-02
       coeff_l4(70,91)=  2.6733472943e-01
       coeff_l4(70,92)=  2.0636330545e-01
       coeff_l4(70,93)= -1.5676690266e-02
       coeff_l4(70,94)= -4.7822409868e-01
       coeff_l4(70,95)=  1.0091274232e-01
       coeff_l4(70,96)= -5.1501965523e-01
       coeff_l4(70,97)= -3.5529425740e-01
       coeff_l4(70,98)=  5.1650747657e-02
       coeff_l4(70,99)= -1.3808323443e-01
       coeff_l4(70,100)=  3.2240886241e-02
       coeff_l4(70,101)= -2.0208890736e-01
       coeff_l4(70,102)= -1.3684944808e-01
       coeff_l4(70,103)= -3.9928570390e-02
       coeff_l4(70,104)=  1.0338249058e-01
       coeff_l4(70,105)= -2.3155208677e-02
       coeff_l4(71,1)=  1.5689836740e+00
       coeff_l4(71,2)=  1.6327542067e+00
       coeff_l4(71,3)=  1.8081796169e+00
       coeff_l4(71,4)=  1.2480204105e+00
       coeff_l4(71,5)=  1.6868981123e+00
       coeff_l4(71,6)=  1.8793412447e+00
       coeff_l4(71,7)=  1.9933221340e+00
       coeff_l4(71,8)=  1.9851801395e+00
       coeff_l4(71,9)=  1.3106102943e+00
       coeff_l4(71,10)=  1.1145852804e+00
       coeff_l4(71,11)=  5.8952796459e-01
       coeff_l4(71,12)=  6.3339114189e-01
       coeff_l4(71,13)=  8.3087694645e-01
       coeff_l4(71,14)=  1.0654064417e+00
       coeff_l4(71,15)= -9.3366101384e-02
       coeff_l4(71,16)= -1.1868768185e-01
       coeff_l4(71,17)=  1.6906996071e-01
       coeff_l4(71,18)=  4.6409928799e-01
       coeff_l4(71,19)=  4.4549098611e-01
       coeff_l4(71,20)=  9.3119479716e-02
       coeff_l4(71,21)=  4.0626126528e-01
       coeff_l4(71,22)= -2.7363535762e-01
       coeff_l4(71,23)=  3.6115598679e-01
       coeff_l4(71,24)=  8.4722049534e-02
       coeff_l4(71,25)=  3.4522615373e-02
       coeff_l4(71,26)=  1.7249615490e-01
       coeff_l4(71,27)= -2.2646084428e-01
       coeff_l4(71,28)=  6.4346365631e-02
       coeff_l4(71,29)=  1.3387320936e-01
       coeff_l4(71,30)= -4.8107466102e-01
       coeff_l4(71,31)= -5.8873772621e-02
       coeff_l4(71,32)= -6.3723489642e-02
       coeff_l4(71,33)= -4.1496742517e-02
       coeff_l4(71,34)= -1.0549020767e-01
       coeff_l4(71,35)= -3.1792566180e-01
       coeff_l4(71,36)=  3.1754076481e-01
       coeff_l4(71,37)=  8.8434610516e-03
       coeff_l4(71,38)= -1.3298507035e-01
       coeff_l4(71,39)=  2.2082138062e-01
       coeff_l4(71,40)=  9.2057801783e-02
       coeff_l4(71,41)= -7.4119216204e-01
       coeff_l4(71,42)= -2.5749793649e-01
       coeff_l4(71,43)= -1.7615649849e-02
       coeff_l4(71,44)=  1.0921732523e-02
       coeff_l4(71,45)=  3.7820002437e-01
       coeff_l4(71,46)= -6.8766556680e-02
       coeff_l4(71,47)=  8.3894439042e-02
       coeff_l4(71,48)=  1.5665502846e-01
       coeff_l4(71,49)= -1.8710446358e-01
       coeff_l4(71,50)=  7.0053979754e-02
       coeff_l4(71,51)= -1.7886583507e-01
       coeff_l4(71,52)=  3.0565140769e-02
       coeff_l4(71,53)= -4.1702955961e-01
       coeff_l4(71,54)= -3.9140909910e-02
       coeff_l4(71,55)= -1.2919045985e-01
       coeff_l4(71,56)= -2.7084362507e-01
       coeff_l4(71,57)= -2.6043584943e-01
       coeff_l4(71,58)=  1.6881795228e-01
       coeff_l4(71,59)= -2.4084834754e-01
       coeff_l4(71,60)= -6.0413260013e-02
       coeff_l4(71,61)= -5.3847640753e-02
       coeff_l4(71,62)= -8.1567570567e-02
       coeff_l4(71,63)=  4.3872269988e-01
       coeff_l4(71,64)= -3.9131846279e-02
       coeff_l4(71,65)= -1.2046775781e-02
       coeff_l4(71,66)=  5.3922516108e-01
       coeff_l4(71,67)=  5.6174062192e-02
       coeff_l4(71,68)= -1.2762121856e-01
       coeff_l4(71,69)= -4.1234776378e-02
       coeff_l4(71,70)= -2.2724863084e-04
       coeff_l4(71,71)= -1.8191204965e-01
       coeff_l4(71,72)= -3.7771627307e-01
       coeff_l4(71,73)=  2.7845087647e-01
       coeff_l4(71,74)= -1.0428816825e-01
       coeff_l4(71,75)=  1.4458242059e-01
       coeff_l4(71,76)= -2.8570130467e-01
       coeff_l4(71,77)=  1.8339920044e-01
       coeff_l4(71,78)=  2.8853905201e-01
       coeff_l4(71,79)=  6.7613668740e-02
       coeff_l4(71,80)=  1.4821723104e-01
       coeff_l4(71,81)=  1.0805664584e-02
       coeff_l4(71,82)= -6.9849371910e-02
       coeff_l4(71,83)= -1.3700300455e-01
       coeff_l4(71,84)= -2.7979398146e-02
       coeff_l4(71,85)= -1.7016673088e-01
       coeff_l4(71,86)=  3.8285031915e-01
       coeff_l4(71,87)= -4.8185724020e-01
       coeff_l4(71,88)=  4.7235310078e-02
       coeff_l4(71,89)=  2.0034494996e-01
       coeff_l4(71,90)= -5.5745895952e-02
       coeff_l4(71,91)= -3.2377798110e-02
       coeff_l4(71,92)=  1.5739809722e-02
       coeff_l4(71,93)= -4.1735917330e-02
       coeff_l4(71,94)= -2.5609111786e-01
       coeff_l4(71,95)=  2.3204040527e-01
       coeff_l4(71,96)= -1.4444214106e-01
       coeff_l4(71,97)= -6.7667432129e-02
       coeff_l4(71,98)=  4.3563011289e-01
       coeff_l4(71,99)=  1.3648383319e-01
       coeff_l4(71,100)= -2.2803060710e-03
       coeff_l4(71,101)=  3.2918270677e-02
       coeff_l4(71,102)= -1.2109635025e-01
       coeff_l4(71,103)=  1.9321002066e-01
       coeff_l4(71,104)= -2.2214370966e-01
       coeff_l4(71,105)=  5.8278584480e-01
       coeff_l4(72,1)=  1.2581855990e-02
       coeff_l4(72,2)=  4.1054669023e-01
       coeff_l4(72,3)=  3.3761879802e-01
       coeff_l4(72,4)=  1.8729345500e-01
       coeff_l4(72,5)=  1.8485587835e-01
       coeff_l4(72,6)=  1.8219944835e-01
       coeff_l4(72,7)=  2.7403897047e-01
       coeff_l4(72,8)=  1.8563541770e-01
       coeff_l4(72,9)=  3.2229563594e-01
       coeff_l4(72,10)=  1.5979623795e-01
       coeff_l4(72,11)=  4.4788429141e-01
       coeff_l4(72,12)=  1.1354586482e-01
       coeff_l4(72,13)=  3.0878376961e-01
       coeff_l4(72,14)=  3.5756179690e-01
       coeff_l4(72,15)=  1.4445427060e-01
       coeff_l4(72,16)= -9.7916767001e-02
       coeff_l4(72,17)= -2.2249588370e-01
       coeff_l4(72,18)= -7.1013271809e-02
       coeff_l4(72,19)= -1.1077338457e-01
       coeff_l4(72,20)=  1.4414055645e-01
       coeff_l4(72,21)=  7.2849117219e-02
       coeff_l4(72,22)=  1.5473113954e-01
       coeff_l4(72,23)= -4.4539961964e-03
       coeff_l4(72,24)=  9.8460502923e-02
       coeff_l4(72,25)= -2.1998735610e-03
       coeff_l4(72,26)=  2.9411802534e-03
       coeff_l4(72,27)= -4.5561876148e-02
       coeff_l4(72,28)= -1.0448685288e-01
       coeff_l4(72,29)= -6.6245131195e-02
       coeff_l4(72,30)= -5.7268459350e-03
       coeff_l4(72,31)= -1.2884655595e-01
       coeff_l4(72,32)= -5.9246741235e-02
       coeff_l4(72,33)=  1.4536705613e-01
       coeff_l4(72,34)=  1.2230072916e-01
       coeff_l4(72,35)= -4.7768972814e-02
       coeff_l4(72,36)= -7.2794497013e-02
       coeff_l4(72,37)= -1.3241578639e-01
       coeff_l4(72,38)=  3.1646955758e-02
       coeff_l4(72,39)= -1.5272974968e-01
       coeff_l4(72,40)= -7.8894056380e-02
       coeff_l4(72,41)=  1.5728272498e-01
       coeff_l4(72,42)=  4.2604591697e-02
       coeff_l4(72,43)=  5.1904287189e-02
       coeff_l4(72,44)= -1.9429696724e-02
       coeff_l4(72,45)=  1.1978363618e-02
       coeff_l4(72,46)= -1.7406681553e-02
       coeff_l4(72,47)= -1.6160129011e-01
       coeff_l4(72,48)= -6.8447768688e-02
       coeff_l4(72,49)= -1.5059342980e-01
       coeff_l4(72,50)= -1.0072149336e-01
       coeff_l4(72,51)= -2.0762838423e-02
       coeff_l4(72,52)= -1.2686799467e-01
       coeff_l4(72,53)=  1.6292431951e-01
       coeff_l4(72,54)=  1.1566832662e-01
       coeff_l4(72,55)= -5.2833192050e-02
       coeff_l4(72,56)= -9.0301528573e-02
       coeff_l4(72,57)=  1.0899996385e-02
       coeff_l4(72,58)=  1.9312418997e-01
       coeff_l4(72,59)=  5.5526815355e-02
       coeff_l4(72,60)= -5.5366721004e-02
       coeff_l4(72,61)= -3.8098238409e-02
       coeff_l4(72,62)=  1.1578860134e-01
       coeff_l4(72,63)= -3.0035007745e-02
       coeff_l4(72,64)= -5.7561945170e-02
       coeff_l4(72,65)= -7.7640831470e-02
       coeff_l4(72,66)= -1.0588231683e-01
       coeff_l4(72,67)=  4.1867196560e-02
       coeff_l4(72,68)=  5.3807357326e-03
       coeff_l4(72,69)=  2.1941589192e-02
       coeff_l4(72,70)=  9.6992239356e-02
       coeff_l4(72,71)= -2.1388922632e-01
       coeff_l4(72,72)=  1.9206318539e-03
       coeff_l4(72,73)=  2.5328682736e-02
       coeff_l4(72,74)=  7.0228047669e-02
       coeff_l4(72,75)= -7.1608841419e-02
       coeff_l4(72,76)=  1.7141495645e-01
       coeff_l4(72,77)= -1.1703173071e-01
       coeff_l4(72,78)= -3.9993498474e-02
       coeff_l4(72,79)=  1.1955949664e-01
       coeff_l4(72,80)= -4.6579085290e-02
       coeff_l4(72,81)=  2.5827126577e-02
       coeff_l4(72,82)=  9.3602322042e-02
       coeff_l4(72,83)= -7.9543087631e-03
       coeff_l4(72,84)=  7.6076351106e-02
       coeff_l4(72,85)=  6.6096335649e-02
       coeff_l4(72,86)= -8.9271046221e-02
       coeff_l4(72,87)=  1.3233493268e-01
       coeff_l4(72,88)= -7.1549601853e-02
       coeff_l4(72,89)= -1.2025481462e-01
       coeff_l4(72,90)= -5.4817892611e-02
       coeff_l4(72,91)= -1.5167392790e-01
       coeff_l4(72,92)=  5.6206129491e-02
       coeff_l4(72,93)=  5.0377681851e-02
       coeff_l4(72,94)=  7.2571799159e-02
       coeff_l4(72,95)=  1.0194417089e-01
       coeff_l4(72,96)=  5.1160704345e-02
       coeff_l4(72,97)=  4.4534787536e-02
       coeff_l4(72,98)=  1.5335422941e-02
       coeff_l4(72,99)=  1.0336618870e-01
       coeff_l4(72,100)=  7.3627322912e-02
       coeff_l4(72,101)=  1.1607562751e-01
       coeff_l4(72,102)= -2.2252611816e-01
       coeff_l4(72,103)=  9.3140386045e-02
       coeff_l4(72,104)= -7.0201836526e-02
       coeff_l4(72,105)= -1.3605261222e-02
       coeff_l4(73,1)= -8.0679887533e-01
       coeff_l4(73,2)= -1.2505139112e+00
       coeff_l4(73,3)= -1.1770741940e+00
       coeff_l4(73,4)= -8.8127505779e-01
       coeff_l4(73,5)= -1.3636403084e+00
       coeff_l4(73,6)= -1.1700211763e+00
       coeff_l4(73,7)= -5.7501852512e-01
       coeff_l4(73,8)= -5.0529104471e-01
       coeff_l4(73,9)= -9.1224384308e-01
       coeff_l4(73,10)= -5.7206046581e-01
       coeff_l4(73,11)= -2.5600069761e-01
       coeff_l4(73,12)=  1.5358144045e-01
       coeff_l4(73,13)= -4.2350980639e-01
       coeff_l4(73,14)= -1.6697099805e-01
       coeff_l4(73,15)= -3.1398099661e-01
       coeff_l4(73,16)=  1.9508823752e-01
       coeff_l4(73,17)= -2.5373250246e-01
       coeff_l4(73,18)= -6.0673761368e-01
       coeff_l4(73,19)=  1.1530602723e-01
       coeff_l4(73,20)= -1.1928711832e-01
       coeff_l4(73,21)= -1.8560886383e-01
       coeff_l4(73,22)=  1.0098620318e-02
       coeff_l4(73,23)= -2.3887906224e-02
       coeff_l4(73,24)=  7.3499172926e-02
       coeff_l4(73,25)= -7.9366423190e-02
       coeff_l4(73,26)=  7.7904082835e-02
       coeff_l4(73,27)=  2.4545733631e-01
       coeff_l4(73,28)= -2.7220672369e-01
       coeff_l4(73,29)=  2.0001831651e-01
       coeff_l4(73,30)=  7.2311051190e-02
       coeff_l4(73,31)=  8.1104584038e-02
       coeff_l4(73,32)=  2.6635891199e-01
       coeff_l4(73,33)= -2.1539227664e-01
       coeff_l4(73,34)=  1.7834428698e-02
       coeff_l4(73,35)= -3.2318094373e-01
       coeff_l4(73,36)=  2.5454762578e-01
       coeff_l4(73,37)= -1.2483877689e-01
       coeff_l4(73,38)= -3.4577497840e-01
       coeff_l4(73,39)= -2.4909043312e-01
       coeff_l4(73,40)= -8.4426425397e-02
       coeff_l4(73,41)=  4.2030686140e-01
       coeff_l4(73,42)=  4.1330593824e-01
       coeff_l4(73,43)= -4.1398027539e-01
       coeff_l4(73,44)= -6.9308362901e-02
       coeff_l4(73,45)= -5.6928187609e-01
       coeff_l4(73,46)=  7.6618869789e-03
       coeff_l4(73,47)=  5.9723699093e-01
       coeff_l4(73,48)=  1.8926078081e-01
       coeff_l4(73,49)= -6.1292082071e-02
       coeff_l4(73,50)= -3.2371273637e-01
       coeff_l4(73,51)=  2.2531466186e-01
       coeff_l4(73,52)= -8.7489068508e-02
       coeff_l4(73,53)=  3.4522774816e-01
       coeff_l4(73,54)= -1.8055209517e-01
       coeff_l4(73,55)=  4.9358299375e-01
       coeff_l4(73,56)=  2.8591284156e-01
       coeff_l4(73,57)= -2.8646789724e-04
       coeff_l4(73,58)= -1.2466462702e-01
       coeff_l4(73,59)=  3.1352511048e-01
       coeff_l4(73,60)= -9.5969196409e-03
       coeff_l4(73,61)=  9.7992651165e-02
       coeff_l4(73,62)= -3.9392420650e-01
       coeff_l4(73,63)=  3.5517939925e-01
       coeff_l4(73,64)= -1.0113943368e-01
       coeff_l4(73,65)= -5.5479586124e-01
       coeff_l4(73,66)= -3.5274687409e-01
       coeff_l4(73,67)=  5.7443171740e-01
       coeff_l4(73,68)=  1.9306164235e-02
       coeff_l4(73,69)= -1.0671705753e-01
       coeff_l4(73,70)=  6.0465067625e-02
       coeff_l4(73,71)= -4.2586290836e-01
       coeff_l4(73,72)=  2.1852519363e-02
       coeff_l4(73,73)=  1.0298470967e-02
       coeff_l4(73,74)=  8.8100515306e-02
       coeff_l4(73,75)=  5.9467101097e-01
       coeff_l4(73,76)=  9.3238726258e-02
       coeff_l4(73,77)=  2.9614561796e-01
       coeff_l4(73,78)=  5.8458793163e-01
       coeff_l4(73,79)= -1.9852116704e-01
       coeff_l4(73,80)=  3.0406248569e-01
       coeff_l4(73,81)= -2.9250010848e-01
       coeff_l4(73,82)= -1.7591013014e-01
       coeff_l4(73,83)=  1.0637549311e-01
       coeff_l4(73,84)= -5.5069759488e-02
       coeff_l4(73,85)= -3.5599249601e-01
       coeff_l4(73,86)= -1.3428522646e-01
       coeff_l4(73,87)= -7.6796576381e-02
       coeff_l4(73,88)= -4.5461633801e-01
       coeff_l4(73,89)= -5.6506872177e-01
       coeff_l4(73,90)=  3.8801819086e-02
       coeff_l4(73,91)= -2.8110963106e-01
       coeff_l4(73,92)= -5.3408384323e-01
       coeff_l4(73,93)=  2.4249339104e-01
       coeff_l4(73,94)=  1.8648482859e-01
       coeff_l4(73,95)=  1.4310900867e-01
       coeff_l4(73,96)=  6.7599141598e-01
       coeff_l4(73,97)=  1.2474570423e-01
       coeff_l4(73,98)= -1.9000211731e-02
       coeff_l4(73,99)= -8.4330677986e-01
       coeff_l4(73,100)= -1.9164009392e-01
       coeff_l4(73,101)=  4.3659368157e-01
       coeff_l4(73,102)= -4.5733323693e-01
       coeff_l4(73,103)= -3.4038326144e-01
       coeff_l4(73,104)= -2.3854985833e-01
       coeff_l4(73,105)= -4.9638982862e-02
       coeff_l4(74,1)=  1.3112333603e-02
       coeff_l4(74,2)= -3.0578199029e-01
       coeff_l4(74,3)=  2.3708974943e-02
       coeff_l4(74,4)= -3.8321875036e-02
       coeff_l4(74,5)= -1.2847115099e-01
       coeff_l4(74,6)= -5.1251590252e-01
       coeff_l4(74,7)=  1.9322094321e-01
       coeff_l4(74,8)= -3.1339004636e-01
       coeff_l4(74,9)= -5.3368616104e-01
       coeff_l4(74,10)= -8.2339072227e-01
       coeff_l4(74,11)= -4.5537722111e-01
       coeff_l4(74,12)= -5.1685076952e-01
       coeff_l4(74,13)= -9.6554493904e-01
       coeff_l4(74,14)= -4.4031745195e-01
       coeff_l4(74,15)=  4.2945280671e-02
       coeff_l4(74,16)= -1.4770905674e-01
       coeff_l4(74,17)= -5.6540790945e-02
       coeff_l4(74,18)= -1.1394272000e-01
       coeff_l4(74,19)= -1.3748064637e-01
       coeff_l4(74,20)= -9.1014280915e-02
       coeff_l4(74,21)= -6.9152653217e-02
       coeff_l4(74,22)=  1.4749704301e-01
       coeff_l4(74,23)= -1.3729688525e-01
       coeff_l4(74,24)= -5.3370080888e-02
       coeff_l4(74,25)= -3.3748692274e-01
       coeff_l4(74,26)=  2.3396641016e-01
       coeff_l4(74,27)= -9.8906546831e-02
       coeff_l4(74,28)=  3.4577354789e-01
       coeff_l4(74,29)=  1.7322200537e-01
       coeff_l4(74,30)= -2.5763431191e-01
       coeff_l4(74,31)= -4.1255366802e-01
       coeff_l4(74,32)= -3.8837146014e-02
       coeff_l4(74,33)=  8.0146573484e-02
       coeff_l4(74,34)= -3.3518540859e-01
       coeff_l4(74,35)= -2.7764168382e-01
       coeff_l4(74,36)=  3.0134189129e-01
       coeff_l4(74,37)= -3.2363119721e-01
       coeff_l4(74,38)= -4.0334725380e-01
       coeff_l4(74,39)=  3.6072221398e-01
       coeff_l4(74,40)=  2.7218949795e-01
       coeff_l4(74,41)= -1.7310164869e-02
       coeff_l4(74,42)=  9.5241390169e-02
       coeff_l4(74,43)=  3.5547930747e-02
       coeff_l4(74,44)=  3.7919738889e-01
       coeff_l4(74,45)=  1.4896798134e-01
       coeff_l4(74,46)=  1.6358867288e-01
       coeff_l4(74,47)= -2.8117302060e-01
       coeff_l4(74,48)= -3.4203967452e-01
       coeff_l4(74,49)= -2.7384027839e-01
       coeff_l4(74,50)=  2.9814147949e-01
       coeff_l4(74,51)= -8.1220574677e-02
       coeff_l4(74,52)=  4.4391471148e-01
       coeff_l4(74,53)=  5.2358341217e-01
       coeff_l4(74,54)= -7.7006138861e-02
       coeff_l4(74,55)=  6.4320427179e-01
       coeff_l4(74,56)= -5.0005979836e-02
       coeff_l4(74,57)=  2.4574048817e-01
       coeff_l4(74,58)=  6.6905976273e-03
       coeff_l4(74,59)= -2.6545765996e-01
       coeff_l4(74,60)= -6.2150287628e-01
       coeff_l4(74,61)= -5.0312280655e-01
       coeff_l4(74,62)=  2.4610602856e-01
       coeff_l4(74,63)=  6.7870318890e-02
       coeff_l4(74,64)= -9.2328421772e-02
       coeff_l4(74,65)= -3.2634347677e-01
       coeff_l4(74,66)=  2.0641664043e-02
       coeff_l4(74,67)= -3.4376275539e-01
       coeff_l4(74,68)= -3.7454620004e-01
       coeff_l4(74,69)=  4.5821398497e-02
       coeff_l4(74,70)=  9.9752925336e-02
       coeff_l4(74,71)= -4.1665059328e-01
       coeff_l4(74,72)= -1.0865757614e-01
       coeff_l4(74,73)= -1.6273555160e-01
       coeff_l4(74,74)= -6.1173415743e-03
       coeff_l4(74,75)= -1.4451660216e-01
       coeff_l4(74,76)= -1.8992149830e-01
       coeff_l4(74,77)=  2.8931757808e-01
       coeff_l4(74,78)=  6.8410150707e-02
       coeff_l4(74,79)= -2.3516038433e-02
       coeff_l4(74,80)=  3.6199748516e-01
       coeff_l4(74,81)= -3.7555661798e-01
       coeff_l4(74,82)= -6.1572366953e-01
       coeff_l4(74,83)=  2.5406092405e-02
       coeff_l4(74,84)= -9.0282700956e-02
       coeff_l4(74,85)=  1.4402469993e-01
       coeff_l4(74,86)= -3.9917573333e-01
       coeff_l4(74,87)=  1.9508374855e-02
       coeff_l4(74,88)=  3.7769037485e-01
       coeff_l4(74,89)=  5.2194543183e-02
       coeff_l4(74,90)=  4.0497094393e-02
       coeff_l4(74,91)= -7.0881783962e-01
       coeff_l4(74,92)=  5.4734075069e-01
       coeff_l4(74,93)=  1.4440223575e-01
       coeff_l4(74,94)=  6.9517552853e-01
       coeff_l4(74,95)= -3.5087293386e-01
       coeff_l4(74,96)=  6.9252133369e-01
       coeff_l4(74,97)=  3.1293684244e-01
       coeff_l4(74,98)=  8.9467763901e-02
       coeff_l4(74,99)= -3.7836587429e-01
       coeff_l4(74,100)= -3.4193816781e-01
       coeff_l4(74,101)=  1.7360749841e-01
       coeff_l4(74,102)= -4.3378978968e-01
       coeff_l4(74,103)= -6.3898563385e-01
       coeff_l4(74,104)= -1.9280117750e-01
       coeff_l4(74,105)=  2.2728363052e-02
       coeff_l4(75,1)= -9.8484802246e-01
       coeff_l4(75,2)= -7.0676547289e-01
       coeff_l4(75,3)= -7.5238329172e-01
       coeff_l4(75,4)= -9.3829500675e-01
       coeff_l4(75,5)= -4.8012402654e-01
       coeff_l4(75,6)= -8.3831858635e-01
       coeff_l4(75,7)= -9.1372841597e-01
       coeff_l4(75,8)= -7.5766134262e-01
       coeff_l4(75,9)= -3.9353966713e-01
       coeff_l4(75,10)= -5.8544868231e-01
       coeff_l4(75,11)= -4.1819319129e-01
       coeff_l4(75,12)= -2.9734382033e-01
       coeff_l4(75,13)= -2.3668472469e-01
       coeff_l4(75,14)= -2.2784969211e-01
       coeff_l4(75,15)=  7.0502638817e-02
       coeff_l4(75,16)= -1.6726641357e-01
       coeff_l4(75,17)= -5.8893963695e-02
       coeff_l4(75,18)=  4.2811024934e-02
       coeff_l4(75,19)= -1.2448149920e-01
       coeff_l4(75,20)= -2.6303234696e-01
       coeff_l4(75,21)= -2.0412899554e-01
       coeff_l4(75,22)=  6.0553591698e-02
       coeff_l4(75,23)= -1.4736433327e-01
       coeff_l4(75,24)=  8.6710728705e-02
       coeff_l4(75,25)=  1.8987582996e-02
       coeff_l4(75,26)=  1.6915814579e-01
       coeff_l4(75,27)=  2.8358206153e-01
       coeff_l4(75,28)= -9.0014142916e-03
       coeff_l4(75,29)= -3.2283535600e-01
       coeff_l4(75,30)=  3.1685981154e-01
       coeff_l4(75,31)= -1.9768406451e-01
       coeff_l4(75,32)=  2.8207799792e-01
       coeff_l4(75,33)=  2.8314924240e-01
       coeff_l4(75,34)=  3.9151605219e-02
       coeff_l4(75,35)=  2.7951177955e-01
       coeff_l4(75,36)= -1.5167829394e-01
       coeff_l4(75,37)=  2.0473086834e-01
       coeff_l4(75,38)=  3.1202983111e-02
       coeff_l4(75,39)= -2.3841662705e-01
       coeff_l4(75,40)=  2.4049815536e-01
       coeff_l4(75,41)=  2.4017682672e-01
       coeff_l4(75,42)=  5.6170828640e-02
       coeff_l4(75,43)=  1.4053784311e-01
       coeff_l4(75,44)= -2.9053896666e-01
       coeff_l4(75,45)= -3.3596858382e-01
       coeff_l4(75,46)= -1.3245303929e-01
       coeff_l4(75,47)= -2.4775835872e-01
       coeff_l4(75,48)=  7.5968384743e-02
       coeff_l4(75,49)= -5.7148333639e-02
       coeff_l4(75,50)= -1.2708079303e-03
       coeff_l4(75,51)=  1.2321907282e-01
       coeff_l4(75,52)=  3.9572276175e-02
       coeff_l4(75,53)=  4.1233691573e-01
       coeff_l4(75,54)=  3.2434578985e-02
       coeff_l4(75,55)=  1.1804132164e-01
       coeff_l4(75,56)= -5.3943336010e-02
       coeff_l4(75,57)= -1.4863948524e-01
       coeff_l4(75,58)= -8.9214282343e-04
       coeff_l4(75,59)=  1.3321569562e-01
       coeff_l4(75,60)=  6.4527527429e-03
       coeff_l4(75,61)=  1.0447526723e-01
       coeff_l4(75,62)=  1.0990339518e-01
       coeff_l4(75,63)= -2.6222705841e-01
       coeff_l4(75,64)=  3.1369686127e-01
       coeff_l4(75,65)= -3.0505442992e-02
       coeff_l4(75,66)=  1.4679735899e-01
       coeff_l4(75,67)= -2.3461854085e-02
       coeff_l4(75,68)= -5.2064582705e-03
       coeff_l4(75,69)= -8.4876632318e-03
       coeff_l4(75,70)=  3.0517721176e-01
       coeff_l4(75,71)=  2.2112980485e-02
       coeff_l4(75,72)= -4.1613006592e-01
       coeff_l4(75,73)= -7.0868730545e-02
       coeff_l4(75,74)=  1.9909720123e-01
       coeff_l4(75,75)=  3.0507832766e-01
       coeff_l4(75,76)=  6.9245502353e-02
       coeff_l4(75,77)= -1.6115695238e-01
       coeff_l4(75,78)=  1.0822742479e-03
       coeff_l4(75,79)=  3.0250283889e-03
       coeff_l4(75,80)=  2.3908823729e-01
       coeff_l4(75,81)=  9.3166097999e-02
       coeff_l4(75,82)=  3.1256529689e-01
       coeff_l4(75,83)= -6.2078149058e-03
       coeff_l4(75,84)=  4.2488925159e-02
       coeff_l4(75,85)=  1.3650819659e-01
       coeff_l4(75,86)= -1.8010987341e-01
       coeff_l4(75,87)= -1.5200787783e-01
       coeff_l4(75,88)= -2.9355683923e-01
       coeff_l4(75,89)= -1.4738559723e-01
       coeff_l4(75,90)=  1.6742587090e-02
       coeff_l4(75,91)= -1.4903740585e-01
       coeff_l4(75,92)= -1.7724540830e-01
       coeff_l4(75,93)=  1.9966457784e-01
       coeff_l4(75,94)= -4.3019339442e-02
       coeff_l4(75,95)= -9.0659506619e-02
       coeff_l4(75,96)= -2.5958491489e-02
       coeff_l4(75,97)=  1.8615199625e-01
       coeff_l4(75,98)=  1.0476025194e-01
       coeff_l4(75,99)=  1.2818710506e-01
       coeff_l4(75,100)=  3.5118910670e-01
       coeff_l4(75,101)=  1.7884950340e-01
       coeff_l4(75,102)= -1.8222200871e-01
       coeff_l4(75,103)=  4.9388942122e-01
       coeff_l4(75,104)= -1.5714672627e-03
       coeff_l4(75,105)= -2.3918238282e-01
       coeff_l4(76,1)= -1.9412402809e-01
       coeff_l4(76,2)= -1.9100528955e-01
       coeff_l4(76,3)=  7.4921153486e-02
       coeff_l4(76,4)= -2.2138433158e-01
       coeff_l4(76,5)= -2.0069402456e-01
       coeff_l4(76,6)=  1.2445291132e-01
       coeff_l4(76,7)= -9.2597827315e-02
       coeff_l4(76,8)=  1.4676690102e-01
       coeff_l4(76,9)= -7.7202491462e-02
       coeff_l4(76,10)= -6.6182959080e-01
       coeff_l4(76,11)= -2.1164910793e+00
       coeff_l4(76,12)= -2.3451006413e+00
       coeff_l4(76,13)= -1.2803597450e+00
       coeff_l4(76,14)= -8.3557538688e-02
       coeff_l4(76,15)= -8.8053807616e-02
       coeff_l4(76,16)=  1.5784524381e-01
       coeff_l4(76,17)= -1.8959322572e-01
       coeff_l4(76,18)= -1.7675410956e-02
       coeff_l4(76,19)= -5.9094805270e-02
       coeff_l4(76,20)=  1.5181243420e-01
       coeff_l4(76,21)=  4.9448389560e-02
       coeff_l4(76,22)=  1.6531229019e-01
       coeff_l4(76,23)= -5.0747394562e-01
       coeff_l4(76,24)=  3.8523834199e-02
       coeff_l4(76,25)= -1.8959285319e-01
       coeff_l4(76,26)= -2.4637545645e-01
       coeff_l4(76,27)=  1.5278963745e-01
       coeff_l4(76,28)=  1.2383931130e-01
       coeff_l4(76,29)= -9.7237147391e-02
       coeff_l4(76,30)=  6.3801229000e-02
       coeff_l4(76,31)= -1.3117407262e-01
       coeff_l4(76,32)=  8.6870804429e-02
       coeff_l4(76,33)=  6.1901644804e-03
       coeff_l4(76,34)= -6.0477342457e-02
       coeff_l4(76,35)= -2.9790931940e-01
       coeff_l4(76,36)= -3.5616067052e-01
       coeff_l4(76,37)=  7.6269991696e-02
       coeff_l4(76,38)=  3.0432891846e-01
       coeff_l4(76,39)= -2.7188560367e-01
       coeff_l4(76,40)=  1.3589854538e-01
       coeff_l4(76,41)= -5.3123917431e-02
       coeff_l4(76,42)= -2.3029173911e-01
       coeff_l4(76,43)= -2.4018095434e-01
       coeff_l4(76,44)=  3.2648392022e-02
       coeff_l4(76,45)= -4.4070202112e-01
       coeff_l4(76,46)= -1.1804868281e-01
       coeff_l4(76,47)=  3.4630948305e-01
       coeff_l4(76,48)=  1.3093951344e-01
       coeff_l4(76,49)= -2.4512235820e-01
       coeff_l4(76,50)= -1.9506190717e-01
       coeff_l4(76,51)= -3.3925471362e-03
       coeff_l4(76,52)=  1.2155954540e-01
       coeff_l4(76,53)=  7.7183932066e-02
       coeff_l4(76,54)=  1.0704541206e-01
       coeff_l4(76,55)= -1.7072482407e-01
       coeff_l4(76,56)=  1.5014135838e-01
       coeff_l4(76,57)=  3.0807638168e-01
       coeff_l4(76,58)= -1.9648170471e-01
       coeff_l4(76,59)= -5.8231347799e-01
       coeff_l4(76,60)=  3.4853242338e-02
       coeff_l4(76,61)= -3.9503525943e-02
       coeff_l4(76,62)= -2.2143274546e-01
       coeff_l4(76,63)= -2.6650345325e-01
       coeff_l4(76,64)=  2.1553593874e-01
       coeff_l4(76,65)= -9.4441175461e-02
       coeff_l4(76,66)=  2.3721870035e-02
       coeff_l4(76,67)= -9.6561491489e-02
       coeff_l4(76,68)=  3.0431622267e-01
       coeff_l4(76,69)= -1.1567276716e-01
       coeff_l4(76,70)= -3.2771459222e-01
       coeff_l4(76,71)=  2.6824197173e-01
       coeff_l4(76,72)=  1.3100689277e-02
       coeff_l4(76,73)= -1.9428230822e-01
       coeff_l4(76,74)=  1.3174772263e-01
       coeff_l4(76,75)=  3.6939035635e-03
       coeff_l4(76,76)=  3.6374974251e-01
       coeff_l4(76,77)= -3.3606749028e-02
       coeff_l4(76,78)=  1.9472974539e-01
       coeff_l4(76,79)=  8.5135966539e-02
       coeff_l4(76,80)= -4.8229745030e-01
       coeff_l4(76,81)= -9.3467801809e-02
       coeff_l4(76,82)= -2.2180444002e-01
       coeff_l4(76,83)=  5.2622444928e-02
       coeff_l4(76,84)=  1.7148420215e-01
       coeff_l4(76,85)=  3.4889313579e-01
       coeff_l4(76,86)= -1.5178807080e-01
       coeff_l4(76,87)= -2.0753517747e-01
       coeff_l4(76,88)=  6.2463408709e-01
       coeff_l4(76,89)=  7.4759908020e-03
       coeff_l4(76,90)= -4.6548143029e-02
       coeff_l4(76,91)=  1.7100863159e-01
       coeff_l4(76,92)=  5.7263761759e-01
       coeff_l4(76,93)= -6.2870281935e-01
       coeff_l4(76,94)= -1.8710555136e-01
       coeff_l4(76,95)= -6.2116968632e-01
       coeff_l4(76,96)= -9.0820604563e-01
       coeff_l4(76,97)= -6.9235187769e-01
       coeff_l4(76,98)= -2.8559091687e-01
       coeff_l4(76,99)=  9.1212913394e-02
       coeff_l4(76,100)=  4.2560797930e-01
       coeff_l4(76,101)= -2.0699989796e-01
       coeff_l4(76,102)= -4.2450913787e-01
       coeff_l4(76,103)=  1.8592205644e-01
       coeff_l4(76,104)= -5.4900296032e-02
       coeff_l4(76,105)= -9.1038674116e-01
       coeff_l4(77,1)= -4.3572434783e-01
       coeff_l4(77,2)= -4.7104346752e-01
       coeff_l4(77,3)= -1.9715687633e-01
       coeff_l4(77,4)= -3.8871863484e-01
       coeff_l4(77,5)= -2.6766407490e-01
       coeff_l4(77,6)= -3.4465545416e-01
       coeff_l4(77,7)= -6.5675330162e-01
       coeff_l4(77,8)= -7.8503507376e-01
       coeff_l4(77,9)= -6.4073115587e-01
       coeff_l4(77,10)= -4.9843859673e-01
       coeff_l4(77,11)= -1.3107656240e+00
       coeff_l4(77,12)= -1.6744894981e+00
       coeff_l4(77,13)= -9.7761464119e-01
       coeff_l4(77,14)= -7.7465724945e-01
       coeff_l4(77,15)= -2.9588237405e-01
       coeff_l4(77,16)=  8.8437655941e-03
       coeff_l4(77,17)= -2.9808960855e-02
       coeff_l4(77,18)= -2.7088251710e-01
       coeff_l4(77,19)= -2.7670168877e-01
       coeff_l4(77,20)=  9.6283137798e-02
       coeff_l4(77,21)= -1.4260829985e-01
       coeff_l4(77,22)=  9.9521242082e-02
       coeff_l4(77,23)=  3.0698293447e-01
       coeff_l4(77,24)=  1.5187172592e-01
       coeff_l4(77,25)=  1.8212667108e-01
       coeff_l4(77,26)= -1.8930850923e-01
       coeff_l4(77,27)=  1.7632254958e-01
       coeff_l4(77,28)=  7.8469842672e-02
       coeff_l4(77,29)=  8.9648388326e-02
       coeff_l4(77,30)= -1.4696967602e-01
       coeff_l4(77,31)=  5.1219914109e-02
       coeff_l4(77,32)= -7.5316107832e-03
       coeff_l4(77,33)=  2.9872467741e-02
       coeff_l4(77,34)= -1.1952143908e-01
       coeff_l4(77,35)=  2.1235749125e-01
       coeff_l4(77,36)=  1.5368102491e-01
       coeff_l4(77,37)= -5.3679864854e-02
       coeff_l4(77,38)= -1.2962672114e-01
       coeff_l4(77,39)=  1.2354029715e-01
       coeff_l4(77,40)= -2.5224930048e-01
       coeff_l4(77,41)=  3.0803304911e-01
       coeff_l4(77,42)=  2.8739589453e-01
       coeff_l4(77,43)= -9.9551370367e-03
       coeff_l4(77,44)= -4.7509890050e-02
       coeff_l4(77,45)= -1.7496259511e-01
       coeff_l4(77,46)= -1.0550893843e-01
       coeff_l4(77,47)= -4.8314803839e-01
       coeff_l4(77,48)= -1.0503607988e-01
       coeff_l4(77,49)=  7.4474424124e-02
       coeff_l4(77,50)=  1.9713094458e-02
       coeff_l4(77,51)= -8.0488540232e-02
       coeff_l4(77,52)=  6.7266590893e-02
       coeff_l4(77,53)= -2.4417753518e-01
       coeff_l4(77,54)=  2.0373153687e-01
       coeff_l4(77,55)= -2.4328979850e-01
       coeff_l4(77,56)= -7.1807973087e-02
       coeff_l4(77,57)= -1.2092577666e-01
       coeff_l4(77,58)=  2.0459507406e-01
       coeff_l4(77,59)=  1.5705791116e-01
       coeff_l4(77,60)=  2.7773207054e-02
       coeff_l4(77,61)= -1.3829874992e-01
       coeff_l4(77,62)= -4.0457103401e-02
       coeff_l4(77,63)=  3.4555036575e-02
       coeff_l4(77,64)= -2.8351115063e-02
       coeff_l4(77,65)= -1.2820671499e-01
       coeff_l4(77,66)=  7.1169063449e-02
       coeff_l4(77,67)=  1.1952925473e-01
       coeff_l4(77,68)= -1.9077882171e-01
       coeff_l4(77,69)= -6.1548557132e-02
       coeff_l4(77,70)=  3.5646717995e-02
       coeff_l4(77,71)=  1.5103310347e-02
       coeff_l4(77,72)=  6.8931281567e-02
       coeff_l4(77,73)=  4.2689353228e-02
       coeff_l4(77,74)= -1.0360332578e-01
       coeff_l4(77,75)= -3.5597693920e-01
       coeff_l4(77,76)= -6.0260808468e-01
       coeff_l4(77,77)= -6.2929198146e-02
       coeff_l4(77,78)= -3.8396576047e-01
       coeff_l4(77,79)=  3.9563691616e-01
       coeff_l4(77,80)= -2.5777570903e-02
       coeff_l4(77,81)=  1.1424599588e-01
       coeff_l4(77,82)=  1.9301423430e-01
       coeff_l4(77,83)=  6.5002307296e-02
       coeff_l4(77,84)= -1.7566873133e-01
       coeff_l4(77,85)=  4.4297050685e-02
       coeff_l4(77,86)= -6.8452507257e-02
       coeff_l4(77,87)=  1.0233525932e-01
       coeff_l4(77,88)= -4.7339317203e-01
       coeff_l4(77,89)=  1.8257060647e-01
       coeff_l4(77,90)= -2.3146952689e-01
       coeff_l4(77,91)= -1.4087539911e-01
       coeff_l4(77,92)= -4.1099780798e-01
       coeff_l4(77,93)=  3.1673151255e-01
       coeff_l4(77,94)=  1.1413764954e-01
       coeff_l4(77,95)= -1.9546245039e-01
       coeff_l4(77,96)= -7.1715670824e-01
       coeff_l4(77,97)= -3.5808950663e-01
       coeff_l4(77,98)= -2.0204862580e-02
       coeff_l4(77,99)= -3.7426356226e-02
       coeff_l4(77,100)=  1.8414126337e-01
       coeff_l4(77,101)= -2.0142425597e-01
       coeff_l4(77,102)=  1.0940542817e-01
       coeff_l4(77,103)= -2.0529097319e-01
       coeff_l4(77,104)=  6.2074963003e-02
       coeff_l4(77,105)=  7.9801417887e-03
       coeff_l4(78,1)= -6.5790563822e-01
       coeff_l4(78,2)= -1.3719309866e-01
       coeff_l4(78,3)= -3.0895113945e-01
       coeff_l4(78,4)= -1.5971219540e-01
       coeff_l4(78,5)= -1.7180389166e-01
       coeff_l4(78,6)= -1.7612364888e-01
       coeff_l4(78,7)= -4.5946884155e-01
       coeff_l4(78,8)= -4.2783474922e-01
       coeff_l4(78,9)=  2.1268701553e-01
       coeff_l4(78,10)=  1.5507473052e-01
       coeff_l4(78,11)=  1.4675517380e-01
       coeff_l4(78,12)=  1.8726135790e-01
       coeff_l4(78,13)=  8.7172649801e-02
       coeff_l4(78,14)=  2.0064702630e-01
       coeff_l4(78,15)=  3.4572538733e-01
       coeff_l4(78,16)= -4.0460456163e-02
       coeff_l4(78,17)= -1.2558462098e-02
       coeff_l4(78,18)= -7.9459622502e-02
       coeff_l4(78,19)= -4.9849908799e-02
       coeff_l4(78,20)= -1.2031175941e-01
       coeff_l4(78,21)= -1.0577578098e-01
       coeff_l4(78,22)=  6.3186436892e-02
       coeff_l4(78,23)=  1.8203371763e-01
       coeff_l4(78,24)=  6.8513102829e-02
       coeff_l4(78,25)=  2.7906140313e-02
       coeff_l4(78,26)=  4.2314630002e-02
       coeff_l4(78,27)=  1.9084610045e-01
       coeff_l4(78,28)= -1.5743669868e-01
       coeff_l4(78,29)=  1.1778406054e-01
       coeff_l4(78,30)=  4.4329136610e-02
       coeff_l4(78,31)= -2.1554680169e-01
       coeff_l4(78,32)=  1.1209757626e-01
       coeff_l4(78,33)=  1.9054187834e-01
       coeff_l4(78,34)= -8.7579272687e-02
       coeff_l4(78,35)=  7.7281117439e-02
       coeff_l4(78,36)= -4.5250216499e-03
       coeff_l4(78,37)=  1.7185096443e-01
       coeff_l4(78,38)=  1.0963003337e-01
       coeff_l4(78,39)= -1.1816822737e-01
       coeff_l4(78,40)=  1.0265655816e-01
       coeff_l4(78,41)= -2.5373888016e-01
       coeff_l4(78,42)= -1.4487700164e-01
       coeff_l4(78,43)=  1.1760991812e-01
       coeff_l4(78,44)= -1.3731155545e-02
       coeff_l4(78,45)= -2.7134647593e-02
       coeff_l4(78,46)=  2.1253436804e-01
       coeff_l4(78,47)= -2.9604998231e-01
       coeff_l4(78,48)= -5.0952676684e-02
       coeff_l4(78,49)= -7.5482069515e-03
       coeff_l4(78,50)=  1.0469838977e-01
       coeff_l4(78,51)= -4.4063970447e-02
       coeff_l4(78,52)=  4.7342326492e-02
       coeff_l4(78,53)=  4.6641115099e-02
       coeff_l4(78,54)=  1.4455497265e-01
       coeff_l4(78,55)= -5.8427147567e-02
       coeff_l4(78,56)= -1.2300361693e-01
       coeff_l4(78,57)=  1.6557419300e-01
       coeff_l4(78,58)=  1.2759327888e-01
       coeff_l4(78,59)=  1.8748170137e-01
       coeff_l4(78,60)=  1.7783561349e-01
       coeff_l4(78,61)= -9.0733610094e-02
       coeff_l4(78,62)=  1.8556737900e-01
       coeff_l4(78,63)= -2.3488520086e-01
       coeff_l4(78,64)= -4.1005786508e-02
       coeff_l4(78,65)= -8.6379712448e-03
       coeff_l4(78,66)= -1.7468412220e-01
       coeff_l4(78,67)= -1.4033872634e-02
       coeff_l4(78,68)= -1.8285833299e-01
       coeff_l4(78,69)= -3.8435783237e-02
       coeff_l4(78,70)=  8.2698844373e-02
       coeff_l4(78,71)=  9.3010701239e-02
       coeff_l4(78,72)=  1.2700234354e-01
       coeff_l4(78,73)=  3.5066064447e-02
       coeff_l4(78,74)=  1.5455043316e-01
       coeff_l4(78,75)= -8.9758783579e-02
       coeff_l4(78,76)=  6.8505024537e-03
       coeff_l4(78,77)= -4.4023327529e-02
       coeff_l4(78,78)= -1.3981151581e-01
       coeff_l4(78,79)= -5.3203318268e-02
       coeff_l4(78,80)= -2.1722478792e-02
       coeff_l4(78,81)= -2.4047441781e-01
       coeff_l4(78,82)=  4.4027365744e-02
       coeff_l4(78,83)= -6.0699112713e-02
       coeff_l4(78,84)=  3.5443391651e-02
       coeff_l4(78,85)=  1.7969477177e-01
       coeff_l4(78,86)=  1.4435602725e-01
       coeff_l4(78,87)= -1.0555991903e-02
       coeff_l4(78,88)=  1.4862188697e-01
       coeff_l4(78,89)=  2.0014615729e-02
       coeff_l4(78,90)=  8.5196495056e-02
       coeff_l4(78,91)=  1.6602163017e-01
       coeff_l4(78,92)= -1.2039208412e-01
       coeff_l4(78,93)=  3.2687809318e-02
       coeff_l4(78,94)= -1.8715719879e-01
       coeff_l4(78,95)=  6.1177551746e-02
       coeff_l4(78,96)=  1.5712089837e-01
       coeff_l4(78,97)= -2.1603167057e-01
       coeff_l4(78,98)= -2.1673078835e-01
       coeff_l4(78,99)=  1.6983257234e-01
       coeff_l4(78,100)= -7.2398044169e-02
       coeff_l4(78,101)=  6.4544253051e-02
       coeff_l4(78,102)=  1.4410705864e-01
       coeff_l4(78,103)=  4.9700908363e-02
       coeff_l4(78,104)= -1.0632170737e-01
       coeff_l4(78,105)= -7.6285516843e-03
       coeff_l4(79,1)= -3.7033629417e-01
       coeff_l4(79,2)= -2.4027051404e-02
       coeff_l4(79,3)= -3.5745701194e-01
       coeff_l4(79,4)= -4.6799877286e-01
       coeff_l4(79,5)= -4.9794068933e-01
       coeff_l4(79,6)= -7.2534823418e-01
       coeff_l4(79,7)= -6.9588202238e-01
       coeff_l4(79,8)= -4.5813578367e-01
       coeff_l4(79,9)= -3.0963912606e-01
       coeff_l4(79,10)= -3.4011042118e-01
       coeff_l4(79,11)=  2.5502112508e-01
       coeff_l4(79,12)= -5.9844471514e-02
       coeff_l4(79,13)=  1.3421452045e-01
       coeff_l4(79,14)= -8.3079643548e-02
       coeff_l4(79,15)=  1.3878495991e-01
       coeff_l4(79,16)= -3.3015465736e-01
       coeff_l4(79,17)= -2.6914641261e-01
       coeff_l4(79,18)=  1.4443910122e-01
       coeff_l4(79,19)=  9.3602957204e-03
       coeff_l4(79,20)= -1.8414337933e-01
       coeff_l4(79,21)=  2.6921844110e-02
       coeff_l4(79,22)=  5.7238969952e-02
       coeff_l4(79,23)=  2.8060004115e-02
       coeff_l4(79,24)=  1.1369437911e-02
       coeff_l4(79,25)=  1.3109999895e-01
       coeff_l4(79,26)=  1.8388745189e-01
       coeff_l4(79,27)=  1.5133099258e-01
       coeff_l4(79,28)= -2.9672136903e-01
       coeff_l4(79,29)=  1.2067627907e-01
       coeff_l4(79,30)=  2.2829897702e-01
       coeff_l4(79,31)= -8.2229368389e-02
       coeff_l4(79,32)=  1.4690344036e-01
       coeff_l4(79,33)=  1.7005522549e-01
       coeff_l4(79,34)=  3.8178214431e-01
       coeff_l4(79,35)=  1.6847786307e-01
       coeff_l4(79,36)= -1.6249372065e-01
       coeff_l4(79,37)=  2.5117075443e-01
       coeff_l4(79,38)=  6.8363368511e-02
       coeff_l4(79,39)=  9.5590092242e-03
       coeff_l4(79,40)= -1.1281814426e-01
       coeff_l4(79,41)=  3.2071220875e-01
       coeff_l4(79,42)=  8.4865055978e-02
       coeff_l4(79,43)= -4.5378353447e-02
       coeff_l4(79,44)=  1.7640562356e-01
       coeff_l4(79,45)= -2.9069060087e-01
       coeff_l4(79,46)=  9.6857860684e-02
       coeff_l4(79,47)= -7.1782261133e-02
       coeff_l4(79,48)= -3.3859947324e-01
       coeff_l4(79,49)=  3.2613405585e-01
       coeff_l4(79,50)= -1.2385910004e-01
       coeff_l4(79,51)=  3.8657169789e-02
       coeff_l4(79,52)=  3.2254688442e-02
       coeff_l4(79,53)=  1.8405240774e-01
       coeff_l4(79,54)= -7.1239970624e-02
       coeff_l4(79,55)=  3.7560511380e-02
       coeff_l4(79,56)=  1.6653303802e-01
       coeff_l4(79,57)= -2.6625633240e-01
       coeff_l4(79,58)= -1.3818332925e-02
       coeff_l4(79,59)=  1.6820449382e-02
       coeff_l4(79,60)= -1.2686988711e-01
       coeff_l4(79,61)= -1.4037762582e-01
       coeff_l4(79,62)=  9.5471940935e-02
       coeff_l4(79,63)= -1.0404604487e-02
       coeff_l4(79,64)= -1.3362944126e-01
       coeff_l4(79,65)= -1.7252774537e-01
       coeff_l4(79,66)=  6.5642833710e-02
       coeff_l4(79,67)= -1.0382640362e-01
       coeff_l4(79,68)=  4.4801622629e-02
       coeff_l4(79,69)= -7.4027977884e-02
       coeff_l4(79,70)=  4.6601537615e-02
       coeff_l4(79,71)=  1.4412726462e-01
       coeff_l4(79,72)= -2.1036170423e-01
       coeff_l4(79,73)= -2.9835628346e-02
       coeff_l4(79,74)=  7.2886839509e-02
       coeff_l4(79,75)=  1.3920022547e-01
       coeff_l4(79,76)=  1.5153230727e-01
       coeff_l4(79,77)= -4.3052750826e-01
       coeff_l4(79,78)=  2.1702288091e-01
       coeff_l4(79,79)= -8.8349886239e-02
       coeff_l4(79,80)=  6.9155730307e-02
       coeff_l4(79,81)= -1.3943941891e-01
       coeff_l4(79,82)= -5.5114831775e-02
       coeff_l4(79,83)= -1.2065254152e-01
       coeff_l4(79,84)=  4.7246702015e-02
       coeff_l4(79,85)= -2.2995019332e-02
       coeff_l4(79,86)= -2.2768057883e-01
       coeff_l4(79,87)= -6.1904869974e-02
       coeff_l4(79,88)=  1.7019207776e-01
       coeff_l4(79,89)= -8.9886039495e-02
       coeff_l4(79,90)=  2.5500129908e-02
       coeff_l4(79,91)= -1.4377871156e-01
       coeff_l4(79,92)=  1.3030147552e-01
       coeff_l4(79,93)=  1.8505910039e-01
       coeff_l4(79,94)=  3.2157021761e-01
       coeff_l4(79,95)=  7.7462501824e-02
       coeff_l4(79,96)= -1.8194336444e-02
       coeff_l4(79,97)= -1.3615751639e-02
       coeff_l4(79,98)=  4.3999306858e-02
       coeff_l4(79,99)=  2.5163522363e-01
       coeff_l4(79,100)=  5.2312340587e-02
       coeff_l4(79,101)= -1.3781617396e-02
       coeff_l4(79,102)=  3.8288682699e-02
       coeff_l4(79,103)=  7.4551396072e-02
       coeff_l4(79,104)=  1.4088799059e-01
       coeff_l4(79,105)= -3.2234344631e-02
       coeff_l4(80,1)=  3.3622041345e-02
       coeff_l4(80,2)= -1.4044766128e-01
       coeff_l4(80,3)=  1.7817462981e-01
       coeff_l4(80,4)= -1.8203882873e-01
       coeff_l4(80,5)=  1.2578807771e-01
       coeff_l4(80,6)= -2.3929609358e-01
       coeff_l4(80,7)=  8.7767295539e-02
       coeff_l4(80,8)=  1.6820307076e-01
       coeff_l4(80,9)= -4.1531151533e-01
       coeff_l4(80,10)= -4.0940459818e-02
       coeff_l4(80,11)= -1.3171181679e+00
       coeff_l4(80,12)= -1.3195487261e+00
       coeff_l4(80,13)= -1.5052012205e+00
       coeff_l4(80,14)=  4.6118187904e-01
       coeff_l4(80,15)= -1.3063926995e-01
       coeff_l4(80,16)=  4.1431322694e-02
       coeff_l4(80,17)=  4.2458735406e-02
       coeff_l4(80,18)=  2.4039943516e-01
       coeff_l4(80,19)= -1.2454501539e-01
       coeff_l4(80,20)=  7.0094801486e-02
       coeff_l4(80,21)=  2.7013406157e-01
       coeff_l4(80,22)= -1.7890185118e-01
       coeff_l4(80,23)=  7.9241611063e-02
       coeff_l4(80,24)= -1.3385715894e-02
       coeff_l4(80,25)=  1.1378069967e-01
       coeff_l4(80,26)= -1.0108902305e-01
       coeff_l4(80,27)= -9.7879961133e-02
       coeff_l4(80,28)=  5.2600316703e-02
       coeff_l4(80,29)=  1.7247569561e-01
       coeff_l4(80,30)= -5.1139861345e-02
       coeff_l4(80,31)=  3.7684396375e-03
       coeff_l4(80,32)=  1.3599064946e-01
       coeff_l4(80,33)=  2.7472689748e-01
       coeff_l4(80,34)= -8.8606722653e-02
       coeff_l4(80,35)=  3.8047790527e-02
       coeff_l4(80,36)=  1.8528507650e-01
       coeff_l4(80,37)= -7.6485469937e-02
       coeff_l4(80,38)=  8.3320066333e-02
       coeff_l4(80,39)= -9.7016997635e-02
       coeff_l4(80,40)=  3.5520724952e-02
       coeff_l4(80,41)= -1.6666592658e-01
       coeff_l4(80,42)= -1.5013894439e-01
       coeff_l4(80,43)= -5.1205564290e-02
       coeff_l4(80,44)= -5.8787735179e-04
       coeff_l4(80,45)= -1.8789270520e-01
       coeff_l4(80,46)= -4.0108419955e-02
       coeff_l4(80,47)=  1.1801469140e-02
       coeff_l4(80,48)=  1.3746657968e-01
       coeff_l4(80,49)=  1.4679121971e-01
       coeff_l4(80,50)=  2.6467904449e-01
       coeff_l4(80,51)= -8.8852038607e-03
       coeff_l4(80,52)= -1.1354221962e-02
       coeff_l4(80,53)= -1.7331689596e-01
       coeff_l4(80,54)=  1.4395682514e-01
       coeff_l4(80,55)=  1.2458484620e-01
       coeff_l4(80,56)= -6.1397660524e-02
       coeff_l4(80,57)= -1.6141830385e-01
       coeff_l4(80,58)=  5.2185889333e-02
       coeff_l4(80,59)=  1.2173408270e-01
       coeff_l4(80,60)= -4.6150289476e-02
       coeff_l4(80,61)= -2.3620167375e-01
       coeff_l4(80,62)= -9.2223763466e-02
       coeff_l4(80,63)=  5.8648422360e-02
       coeff_l4(80,64)=  7.5607545674e-02
       coeff_l4(80,65)=  1.4501643181e-01
       coeff_l4(80,66)=  1.5118138492e-01
       coeff_l4(80,67)=  2.5145511609e-04
       coeff_l4(80,68)= -9.2246808112e-02
       coeff_l4(80,69)=  1.5721035004e-01
       coeff_l4(80,70)=  7.5257584453e-02
       coeff_l4(80,71)= -4.5684903860e-01
       coeff_l4(80,72)= -3.2572802901e-01
       coeff_l4(80,73)=  1.1845122278e-01
       coeff_l4(80,74)=  5.7181236334e-03
       coeff_l4(80,75)= -5.5573139340e-02
       coeff_l4(80,76)=  1.9330847263e-01
       coeff_l4(80,77)= -2.3151157796e-01
       coeff_l4(80,78)= -1.5633088350e-01
       coeff_l4(80,79)= -8.3174400032e-02
       coeff_l4(80,80)=  1.8653859198e-01
       coeff_l4(80,81)= -1.9274702668e-01
       coeff_l4(80,82)=  1.8341507018e-01
       coeff_l4(80,83)=  1.3608117402e-01
       coeff_l4(80,84)= -3.5319305956e-02
       coeff_l4(80,85)= -1.6367632151e-01
       coeff_l4(80,86)=  1.7662015557e-01
       coeff_l4(80,87)= -5.7297986001e-02
       coeff_l4(80,88)=  9.8198734224e-02
       coeff_l4(80,89)= -3.8769960403e-02
       coeff_l4(80,90)= -1.4161704108e-02
       coeff_l4(80,91)= -2.0578160882e-01
       coeff_l4(80,92)=  3.0153357983e-01
       coeff_l4(80,93)= -3.6048638821e-01
       coeff_l4(80,94)= -9.8554754257e-01
       coeff_l4(80,95)=  3.4971883893e-01
       coeff_l4(80,96)= -1.0657362938e+00
       coeff_l4(80,97)= -3.5170680285e-01
       coeff_l4(80,98)= -5.8782115579e-02
       coeff_l4(80,99)=  2.0745249093e-01
       coeff_l4(80,100)=  2.6158848777e-04
       coeff_l4(80,101)= -3.3013668656e-01
       coeff_l4(80,102)= -3.5350042582e-01
       coeff_l4(80,103)=  3.6674273014e-01
       coeff_l4(80,104)=  3.8972610235e-01
       coeff_l4(80,105)=  3.6277079582e-01
       coeff_l4(81,1)=  4.5009222627e-01
       coeff_l4(81,2)=  1.0065389872e+00
       coeff_l4(81,3)=  1.1895564795e+00
       coeff_l4(81,4)=  1.0819272995e+00
       coeff_l4(81,5)=  1.2907552719e+00
       coeff_l4(81,6)=  1.1733454466e+00
       coeff_l4(81,7)=  1.5825033188e+00
       coeff_l4(81,8)=  1.6512783766e+00
       coeff_l4(81,9)=  3.4828659892e-01
       coeff_l4(81,10)=  8.6742997169e-02
       coeff_l4(81,11)=  2.5557836890e-01
       coeff_l4(81,12)= -3.2026162744e-01
       coeff_l4(81,13)=  2.8887847066e-01
       coeff_l4(81,14)= -3.2784464955e-01
       coeff_l4(81,15)= -3.9960961789e-02
       coeff_l4(81,16)= -2.7601200342e-01
       coeff_l4(81,17)= -4.9431064725e-01
       coeff_l4(81,18)=  3.6214822531e-01
       coeff_l4(81,19)= -4.6550247073e-01
       coeff_l4(81,20)= -4.4691894203e-02
       coeff_l4(81,21)=  2.9399195313e-01
       coeff_l4(81,22)= -3.1565934420e-01
       coeff_l4(81,23)=  6.5904036164e-02
       coeff_l4(81,24)=  2.4493236095e-02
       coeff_l4(81,25)=  1.3137918711e-01
       coeff_l4(81,26)=  4.9106210470e-01
       coeff_l4(81,27)=  4.5505940914e-01
       coeff_l4(81,28)= -1.2859191000e-01
       coeff_l4(81,29)= -7.6139010489e-02
       coeff_l4(81,30)= -2.9120072722e-02
       coeff_l4(81,31)= -4.4248354435e-01
       coeff_l4(81,32)=  1.1478362978e-01
       coeff_l4(81,33)=  2.8538531065e-01
       coeff_l4(81,34)=  1.9553717971e-01
       coeff_l4(81,35)=  2.4447846413e-01
       coeff_l4(81,36)=  6.8377003074e-02
       coeff_l4(81,37)=  2.7235859632e-01
       coeff_l4(81,38)= -5.0020072609e-02
       coeff_l4(81,39)= -1.6037738323e-01
       coeff_l4(81,40)=  2.4742943048e-01
       coeff_l4(81,41)=  3.0140915886e-02
       coeff_l4(81,42)= -4.0422860533e-02
       coeff_l4(81,43)=  6.3752436638e-01
       coeff_l4(81,44)= -4.1762682796e-01
       coeff_l4(81,45)= -5.5618431419e-02
       coeff_l4(81,46)=  2.3431888223e-01
       coeff_l4(81,47)= -4.3427503109e-01
       coeff_l4(81,48)= -3.9059326053e-01
       coeff_l4(81,49)= -4.0655843914e-02
       coeff_l4(81,50)=  3.0243429542e-01
       coeff_l4(81,51)= -8.3197675645e-02
       coeff_l4(81,52)=  1.6876019537e-01
       coeff_l4(81,53)=  4.4879478216e-01
       coeff_l4(81,54)= -3.1890773773e-01
       coeff_l4(81,55)= -1.6662961245e-01
       coeff_l4(81,56)= -1.7109621316e-02
       coeff_l4(81,57)= -3.9640957117e-01
       coeff_l4(81,58)=  1.2487684190e-01
       coeff_l4(81,59)=  4.6738451719e-01
       coeff_l4(81,60)=  6.4480251074e-01
       coeff_l4(81,61)=  3.3691251278e-01
       coeff_l4(81,62)=  1.9240061939e-01
       coeff_l4(81,63)= -2.6890721917e-01
       coeff_l4(81,64)=  9.5607382059e-01
       coeff_l4(81,65)=  4.6528011560e-01
       coeff_l4(81,66)= -1.9790055230e-02
       coeff_l4(81,67)= -2.4008958042e-01
       coeff_l4(81,68)= -1.9871311262e-02
       coeff_l4(81,69)= -2.7528718114e-02
       coeff_l4(81,70)=  1.3306328654e-01
       coeff_l4(81,71)=  1.7674477398e-01
       coeff_l4(81,72)= -6.7525140941e-02
       coeff_l4(81,73)= -2.3934568465e-01
       coeff_l4(81,74)= -6.3856177032e-02
       coeff_l4(81,75)=  2.1712990478e-02
       coeff_l4(81,76)=  1.8220517039e-01
       coeff_l4(81,77)= -1.0040344000e+00
       coeff_l4(81,78)=  4.8781689256e-02
       coeff_l4(81,79)= -4.8280730844e-01
       coeff_l4(81,80)= -5.3870308399e-01
       coeff_l4(81,81)=  3.0287143588e-01
       coeff_l4(81,82)=  7.6394486427e-01
       coeff_l4(81,83)= -8.5308760405e-02
       coeff_l4(81,84)=  1.6980236769e-01
       coeff_l4(81,85)=  4.3286272883e-01
       coeff_l4(81,86)= -4.2700782418e-01
       coeff_l4(81,87)=  1.1579886824e-01
       coeff_l4(81,88)= -6.6613800824e-02
       coeff_l4(81,89)= -1.3229086995e-01
       coeff_l4(81,90)=  4.9439501762e-01
       coeff_l4(81,91)= -1.6553384066e-01
       coeff_l4(81,92)=  3.6324205995e-01
       coeff_l4(81,93)=  7.9003289342e-02
       coeff_l4(81,94)= -7.1752846241e-01
       coeff_l4(81,95)= -2.0360991359e-01
       coeff_l4(81,96)= -3.0234241486e-01
       coeff_l4(81,97)= -4.6009531617e-01
       coeff_l4(81,98)=  6.6210669279e-01
       coeff_l4(81,99)=  4.6455964446e-01
       coeff_l4(81,100)=  7.0022672415e-01
       coeff_l4(81,101)= -2.9447942972e-01
       coeff_l4(81,102)=  1.2544496357e-01
       coeff_l4(81,103)=  5.5013400316e-01
       coeff_l4(81,104)=  5.6081324816e-01
       coeff_l4(81,105)= -1.2072154135e-01
       coeff_l4(82,1)=  8.6072154343e-02
       coeff_l4(82,2)=  1.6814383864e-01
       coeff_l4(82,3)= -6.1673396267e-03
       coeff_l4(82,4)=  1.0613464564e-01
       coeff_l4(82,5)=  2.1843329072e-01
       coeff_l4(82,6)= -8.9150143322e-04
       coeff_l4(82,7)=  8.5855968297e-02
       coeff_l4(82,8)=  8.4277331829e-02
       coeff_l4(82,9)=  2.0848858356e-01
       coeff_l4(82,10)=  9.2525996268e-02
       coeff_l4(82,11)=  6.6349491477e-02
       coeff_l4(82,12)= -2.9546441510e-02
       coeff_l4(82,13)=  1.6983930767e-01
       coeff_l4(82,14)=  4.1917353868e-01
       coeff_l4(82,15)=  1.7401167750e-01
       coeff_l4(82,16)=  1.7505909503e-01
       coeff_l4(82,17)=  5.8435466141e-02
       coeff_l4(82,18)=  1.0887600482e-01
       coeff_l4(82,19)= -7.8665852547e-02
       coeff_l4(82,20)=  3.5134259611e-02
       coeff_l4(82,21)=  1.1153570563e-01
       coeff_l4(82,22)=  1.1422666162e-01
       coeff_l4(82,23)= -4.3741095811e-02
       coeff_l4(82,24)= -1.2570357323e-01
       coeff_l4(82,25)= -1.6691336036e-01
       coeff_l4(82,26)= -1.7642697692e-01
       coeff_l4(82,27)=  1.0547231883e-01
       coeff_l4(82,28)= -2.1488764882e-01
       coeff_l4(82,29)=  2.7563046664e-02
       coeff_l4(82,30)= -5.9537243098e-02
       coeff_l4(82,31)=  3.4128770232e-02
       coeff_l4(82,32)=  1.7310927808e-01
       coeff_l4(82,33)=  7.6733559370e-02
       coeff_l4(82,34)=  1.1650984176e-02
       coeff_l4(82,35)= -9.4922453165e-02
       coeff_l4(82,36)=  1.0148023814e-01
       coeff_l4(82,37)= -3.9856429212e-03
       coeff_l4(82,38)=  4.2551122606e-02
       coeff_l4(82,39)= -1.3047988527e-02
       coeff_l4(82,40)= -1.0595583171e-01
       coeff_l4(82,41)=  1.3347594440e-01
       coeff_l4(82,42)=  1.1175139248e-01
       coeff_l4(82,43)= -1.1674626917e-01
       coeff_l4(82,44)=  1.6959276795e-01
       coeff_l4(82,45)= -1.7873246968e-01
       coeff_l4(82,46)= -1.2705653906e-01
       coeff_l4(82,47)= -1.2885595858e-01
       coeff_l4(82,48)= -9.4156593084e-02
       coeff_l4(82,49)=  3.6871824414e-02
       coeff_l4(82,50)=  7.3014408350e-02
       coeff_l4(82,51)=  5.3520951420e-02
       coeff_l4(82,52)=  2.5666302070e-02
       coeff_l4(82,53)= -6.2280859798e-02
       coeff_l4(82,54)=  5.1059857011e-02
       coeff_l4(82,55)=  1.8711429089e-02
       coeff_l4(82,56)= -1.5357105434e-01
       coeff_l4(82,57)= -9.0207559988e-03
       coeff_l4(82,58)=  8.9195156470e-03
       coeff_l4(82,59)=  8.8410139084e-02
       coeff_l4(82,60)= -7.6688513160e-02
       coeff_l4(82,61)=  7.6608769596e-02
       coeff_l4(82,62)= -9.8919831216e-02
       coeff_l4(82,63)=  1.4777195454e-01
       coeff_l4(82,64)=  1.1275728233e-02
       coeff_l4(82,65)= -9.5275968313e-02
       coeff_l4(82,66)= -1.2915837765e-01
       coeff_l4(82,67)= -9.1390714049e-02
       coeff_l4(82,68)= -6.8252682686e-02
       coeff_l4(82,69)= -6.6546253860e-02
       coeff_l4(82,70)=  8.7429210544e-02
       coeff_l4(82,71)=  1.1657068133e-01
       coeff_l4(82,72)= -8.9112989604e-02
       coeff_l4(82,73)= -1.6160892323e-02
       coeff_l4(82,74)= -1.0507766157e-01
       coeff_l4(82,75)= -3.4630980343e-02
       coeff_l4(82,76)=  3.8525655866e-02
       coeff_l4(82,77)= -8.6943864822e-02
       coeff_l4(82,78)=  1.0270627588e-01
       coeff_l4(82,79)= -8.3934426308e-02
       coeff_l4(82,80)= -1.2967050076e-01
       coeff_l4(82,81)=  1.0361400247e-01
       coeff_l4(82,82)= -1.2300404906e-01
       coeff_l4(82,83)= -6.4620736521e-04
       coeff_l4(82,84)= -1.0751616210e-01
       coeff_l4(82,85)=  2.1930587292e-01
       coeff_l4(82,86)= -3.0284373090e-02
       coeff_l4(82,87)=  1.4976894855e-01
       coeff_l4(82,88)=  6.3085101545e-02
       coeff_l4(82,89)= -1.1863692850e-01
       coeff_l4(82,90)= -4.9158595502e-02
       coeff_l4(82,91)=  2.8601247072e-01
       coeff_l4(82,92)= -1.2293414213e-02
       coeff_l4(82,93)= -3.6145497113e-02
       coeff_l4(82,94)=  1.5735542774e-01
       coeff_l4(82,95)=  5.2304465324e-02
       coeff_l4(82,96)=  8.6090825498e-02
       coeff_l4(82,97)=  1.3607278466e-01
       coeff_l4(82,98)=  2.6645988226e-02
       coeff_l4(82,99)=  8.5563786328e-02
       coeff_l4(82,100)= -1.1605991423e-01
       coeff_l4(82,101)= -1.1834013462e-01
       coeff_l4(82,102)= -1.3942670194e-04
       coeff_l4(82,103)=  9.8875738680e-02
       coeff_l4(82,104)=  1.8848236650e-02
       coeff_l4(82,105)= -3.5119283944e-02
       coeff_l4(83,1)=  4.8437803984e-01
       coeff_l4(83,2)=  1.6060131788e-01
       coeff_l4(83,3)= -1.3308452070e-01
       coeff_l4(83,4)=  9.0745396912e-02
       coeff_l4(83,5)=  1.1663366109e-01
       coeff_l4(83,6)=  2.5281524286e-02
       coeff_l4(83,7)=  4.3834980577e-02
       coeff_l4(83,8)= -2.6137402654e-01
       coeff_l4(83,9)= -1.0169077665e-01
       coeff_l4(83,10)= -1.9536636770e-02
       coeff_l4(83,11)=  2.7848866582e-01
       coeff_l4(83,12)= -2.6109629869e-01
       coeff_l4(83,13)= -2.0089742541e-01
       coeff_l4(83,14)=  3.0146354437e-01
       coeff_l4(83,15)=  6.5012597479e-03
       coeff_l4(83,16)=  2.5398728251e-01
       coeff_l4(83,17)= -1.8051171303e-01
       coeff_l4(83,18)=  8.7740860879e-02
       coeff_l4(83,19)= -1.5697984397e-01
       coeff_l4(83,20)= -3.4941482544e-01
       coeff_l4(83,21)=  4.1667541862e-01
       coeff_l4(83,22)= -1.4548022300e-02
       coeff_l4(83,23)=  1.0950508714e-01
       coeff_l4(83,24)=  1.5740214288e-01
       coeff_l4(83,25)= -5.0418619066e-02
       coeff_l4(83,26)=  2.6052904129e-01
       coeff_l4(83,27)= -4.3523198366e-01
       coeff_l4(83,28)=  6.9455765188e-02
       coeff_l4(83,29)= -3.6231097579e-01
       coeff_l4(83,30)=  3.0824893713e-01
       coeff_l4(83,31)= -3.2583570480e-01
       coeff_l4(83,32)=  2.4494542181e-01
       coeff_l4(83,33)= -2.1013116837e-01
       coeff_l4(83,34)= -3.8128721714e-01
       coeff_l4(83,35)=  1.2513167225e-02
       coeff_l4(83,36)= -1.2155050784e-01
       coeff_l4(83,37)=  4.0021178126e-01
       coeff_l4(83,38)= -2.7102190256e-01
       coeff_l4(83,39)=  6.8256326020e-02
       coeff_l4(83,40)=  2.0341807604e-01
       coeff_l4(83,41)= -1.5860131383e-01
       coeff_l4(83,42)=  6.9669701159e-02
       coeff_l4(83,43)=  3.1206062436e-01
       coeff_l4(83,44)= -1.3685297966e-01
       coeff_l4(83,45)=  5.7561293244e-02
       coeff_l4(83,46)= -2.3853124678e-01
       coeff_l4(83,47)= -3.1285482645e-01
       coeff_l4(83,48)=  1.3659250690e-03
       coeff_l4(83,49)=  1.4737576246e-01
       coeff_l4(83,50)=  2.6777383685e-01
       coeff_l4(83,51)= -6.4829535782e-02
       coeff_l4(83,52)= -8.4964148700e-02
       coeff_l4(83,53)=  1.0213454068e-01
       coeff_l4(83,54)=  2.7290772647e-02
       coeff_l4(83,55)=  3.9663544297e-01
       coeff_l4(83,56)= -4.8953220248e-02
       coeff_l4(83,57)= -3.6441257596e-01
       coeff_l4(83,58)=  5.0109989941e-02
       coeff_l4(83,59)=  2.2328600287e-01
       coeff_l4(83,60)= -1.0825045407e-01
       coeff_l4(83,61)=  2.2774995863e-01
       coeff_l4(83,62)=  1.8635687232e-01
       coeff_l4(83,63)= -1.9574663043e-01
       coeff_l4(83,64)=  4.2066931725e-01
       coeff_l4(83,65)= -5.3764473647e-02
       coeff_l4(83,66)= -1.1687206104e-02
       coeff_l4(83,67)= -2.8309887648e-01
       coeff_l4(83,68)=  1.3257040083e-01
       coeff_l4(83,69)=  3.4421768785e-01
       coeff_l4(83,70)= -1.8654814363e-01
       coeff_l4(83,71)= -5.8825548738e-02
       coeff_l4(83,72)= -2.8381064534e-01
       coeff_l4(83,73)= -6.4607751556e-03
       coeff_l4(83,74)=  7.6175607741e-02
       coeff_l4(83,75)=  1.8567523360e-01
       coeff_l4(83,76)= -1.9991286099e-01
       coeff_l4(83,77)=  2.4772130419e-03
       coeff_l4(83,78)=  2.4593560398e-01
       coeff_l4(83,79)= -3.6349795759e-02
       coeff_l4(83,80)=  1.2090268731e-01
       coeff_l4(83,81)= -1.9667723775e-01
       coeff_l4(83,82)= -1.2543322146e-01
       coeff_l4(83,83)=  4.1581671685e-02
       coeff_l4(83,84)= -7.6912239194e-02
       coeff_l4(83,85)= -2.2654823959e-02
       coeff_l4(83,86)= -6.4110952616e-01
       coeff_l4(83,87)= -7.5250618160e-02
       coeff_l4(83,88)= -1.8789893016e-02
       coeff_l4(83,89)=  3.2826790214e-01
       coeff_l4(83,90)=  5.0900600851e-02
       coeff_l4(83,91)= -1.1379451491e-02
       coeff_l4(83,92)= -2.6519843936e-01
       coeff_l4(83,93)=  7.8890629113e-02
       coeff_l4(83,94)=  5.0134080648e-01
       coeff_l4(83,95)= -4.0034526587e-01
       coeff_l4(83,96)= -9.1020911932e-02
       coeff_l4(83,97)= -1.5868179500e-01
       coeff_l4(83,98)=  1.8225590885e-01
       coeff_l4(83,99)=  4.9224302173e-02
       coeff_l4(83,100)=  7.9673320055e-02
       coeff_l4(83,101)=  1.2060035020e-01
       coeff_l4(83,102)= -2.7542662621e-01
       coeff_l4(83,103)= -1.8011160195e-01
       coeff_l4(83,104)=  3.2088235021e-01
       coeff_l4(83,105)= -2.3690994084e-01
       coeff_l4(84,1)= -1.1927939653e+00
       coeff_l4(84,2)= -9.9269378185e-01
       coeff_l4(84,3)= -8.0370318890e-01
       coeff_l4(84,4)= -1.3591800928e+00
       coeff_l4(84,5)= -1.0291575193e+00
       coeff_l4(84,6)= -9.8523914814e-01
       coeff_l4(84,7)= -1.7102875710e+00
       coeff_l4(84,8)= -1.2413653135e+00
       coeff_l4(84,9)= -6.1557728052e-01
       coeff_l4(84,10)= -2.7849200368e-01
       coeff_l4(84,11)=  5.7402831316e-01
       coeff_l4(84,12)=  7.1522213519e-02
       coeff_l4(84,13)=  4.8717400432e-01
       coeff_l4(84,14)= -3.5126516223e-01
       coeff_l4(84,15)= -2.2566993535e-01
       coeff_l4(84,16)=  5.8246951550e-02
       coeff_l4(84,17)= -1.2949686497e-02
       coeff_l4(84,18)=  3.0965974554e-02
       coeff_l4(84,19)= -1.7152739689e-02
       coeff_l4(84,20)=  3.1324556470e-01
       coeff_l4(84,21)= -6.3721582294e-02
       coeff_l4(84,22)=  7.3470436037e-02
       coeff_l4(84,23)= -2.8464907408e-01
       coeff_l4(84,24)= -3.2511815429e-02
       coeff_l4(84,25)=  3.7674966455e-01
       coeff_l4(84,26)=  1.5138904750e-01
       coeff_l4(84,27)=  2.1463489532e-01
       coeff_l4(84,28)= -1.4281907678e-01
       coeff_l4(84,29)= -6.0342650861e-02
       coeff_l4(84,30)=  2.2183094919e-01
       coeff_l4(84,31)=  1.3931716979e-01
       coeff_l4(84,32)= -1.3980592787e-01
       coeff_l4(84,33)=  3.5900163651e-01
       coeff_l4(84,34)=  5.1819157600e-01
       coeff_l4(84,35)=  1.6013956070e-01
       coeff_l4(84,36)= -4.9442484975e-01
       coeff_l4(84,37)=  2.4505762756e-01
       coeff_l4(84,38)=  3.3555462956e-02
       coeff_l4(84,39)=  2.4467915297e-01
       coeff_l4(84,40)= -2.6855733991e-01
       coeff_l4(84,41)= -2.5878814980e-02
       coeff_l4(84,42)= -6.5030050278e-01
       coeff_l4(84,43)= -3.7843158841e-01
       coeff_l4(84,44)= -7.3629729450e-02
       coeff_l4(84,45)=  1.3364949822e-01
       coeff_l4(84,46)=  4.2548793554e-01
       coeff_l4(84,47)= -2.0165090263e-01
       coeff_l4(84,48)= -7.5701189041e-01
       coeff_l4(84,49)=  7.6164022088e-02
       coeff_l4(84,50)=  2.9290330410e-01
       coeff_l4(84,51)=  3.2246869802e-01
       coeff_l4(84,52)= -2.2857549787e-01
       coeff_l4(84,53)=  5.3687542677e-01
       coeff_l4(84,54)=  3.2284179330e-01
       coeff_l4(84,55)= -2.8661888838e-01
       coeff_l4(84,56)= -3.5723738372e-02
       coeff_l4(84,57)= -1.4367097616e-01
       coeff_l4(84,58)=  8.5907680914e-03
       coeff_l4(84,59)= -3.1857392192e-01
       coeff_l4(84,60)= -1.5183810890e-01
       coeff_l4(84,61)= -4.7134172916e-01
       coeff_l4(84,62)= -1.1497698724e-02
       coeff_l4(84,63)= -4.3871733546e-01
       coeff_l4(84,64)= -4.0359976888e-01
       coeff_l4(84,65)=  4.3545457721e-01
       coeff_l4(84,66)= -1.4676581323e-01
       coeff_l4(84,67)= -6.5670585632e-01
       coeff_l4(84,68)=  2.2274349630e-01
       coeff_l4(84,69)= -1.0344296694e-01
       coeff_l4(84,70)= -2.1377600729e-01
       coeff_l4(84,71)=  2.5231531262e-01
       coeff_l4(84,72)= -7.3384225368e-01
       coeff_l4(84,73)= -1.2068331987e-01
       coeff_l4(84,74)= -4.6440345049e-01
       coeff_l4(84,75)= -8.6127072573e-01
       coeff_l4(84,76)= -1.0870124400e-01
       coeff_l4(84,77)= -2.2691901028e-01
       coeff_l4(84,78)= -3.5382273793e-01
       coeff_l4(84,79)=  4.7916406393e-01
       coeff_l4(84,80)= -3.5532182455e-01
       coeff_l4(84,81)=  1.1978358030e-01
       coeff_l4(84,82)=  7.7828846872e-02
       coeff_l4(84,83)=  4.7884824872e-01
       coeff_l4(84,84)=  3.9222151041e-01
       coeff_l4(84,85)=  2.5041055679e-01
       coeff_l4(84,86)=  1.5942409635e-01
       coeff_l4(84,87)=  8.6113047600e-01
       coeff_l4(84,88)=  2.7873855829e-01
       coeff_l4(84,89)=  8.4882169962e-01
       coeff_l4(84,90)=  1.9497132301e-01
       coeff_l4(84,91)=  3.8855844736e-01
       coeff_l4(84,92)= -1.7149136961e-01
       coeff_l4(84,93)= -1.7554004490e-01
       coeff_l4(84,94)=  1.5449810028e-01
       coeff_l4(84,95)= -1.7520537972e-01
       coeff_l4(84,96)=  2.6386484504e-01
       coeff_l4(84,97)=  5.0417375565e-01
       coeff_l4(84,98)=  3.1045308337e-02
       coeff_l4(84,99)=  1.1434233934e-01
       coeff_l4(84,100)= -1.5894867480e-01
       coeff_l4(84,101)= -8.5395283997e-02
       coeff_l4(84,102)=  3.0442520976e-01
       coeff_l4(84,103)= -7.7114515007e-02
       coeff_l4(84,104)= -2.5180438533e-02
       coeff_l4(84,105)= -7.2367526591e-02
       coeff_l4(85,1)=  5.5979001522e-01
       coeff_l4(85,2)=  8.5629820824e-01
       coeff_l4(85,3)=  9.9923080206e-01
       coeff_l4(85,4)=  7.1918833256e-01
       coeff_l4(85,5)=  1.2908803225e+00
       coeff_l4(85,6)=  3.5642823577e-01
       coeff_l4(85,7)=  1.5334467888e+00
       coeff_l4(85,8)=  8.3182054758e-01
       coeff_l4(85,9)= -3.8703572750e-01
       coeff_l4(85,10)= -2.2205407917e-01
       coeff_l4(85,11)= -5.2283447981e-01
       coeff_l4(85,12)= -3.0843454599e-01
       coeff_l4(85,13)= -1.9286239147e-01
       coeff_l4(85,14)= -7.0999270678e-01
       coeff_l4(85,15)=  1.4265432954e-01
       coeff_l4(85,16)=  2.5330014527e-02
       coeff_l4(85,17)=  1.2588873506e-01
       coeff_l4(85,18)= -5.4967129230e-01
       coeff_l4(85,19)=  3.2937639952e-01
       coeff_l4(85,20)=  2.0831005275e-01
       coeff_l4(85,21)= -1.1451275349e+00
       coeff_l4(85,22)=  5.9628003836e-01
       coeff_l4(85,23)=  1.3643915951e-01
       coeff_l4(85,24)= -3.1168994308e-01
       coeff_l4(85,25)=  6.7727440596e-01
       coeff_l4(85,26)= -4.5588576794e-01
       coeff_l4(85,27)=  9.4765502214e-01
       coeff_l4(85,28)= -2.8706499934e-01
       coeff_l4(85,29)=  7.7223533392e-01
       coeff_l4(85,30)=  4.2971050739e-01
       coeff_l4(85,31)=  4.4623503089e-01
       coeff_l4(85,32)= -9.3133163452e-01
       coeff_l4(85,33)= -5.3539976478e-02
       coeff_l4(85,34)=  9.1952914000e-01
       coeff_l4(85,35)= -2.0983643830e-01
       coeff_l4(85,36)= -3.5288113356e-01
       coeff_l4(85,37)=  2.1007703245e-01
       coeff_l4(85,38)= -9.5143866539e-01
       coeff_l4(85,39)= -1.9669018686e-01
       coeff_l4(85,40)=  1.7880108207e-02
       coeff_l4(85,41)=  1.8354812264e-01
       coeff_l4(85,42)=  1.4521175623e-01
       coeff_l4(85,43)= -1.0067573786e+00
       coeff_l4(85,44)=  6.9237035513e-01
       coeff_l4(85,45)=  2.0511411130e-01
       coeff_l4(85,46)=  4.9571567774e-01
       coeff_l4(85,47)=  1.4690853655e-01
       coeff_l4(85,48)=  4.8296350241e-01
       coeff_l4(85,49)= -1.9754062593e-01
       coeff_l4(85,50)= -5.6136715412e-01
       coeff_l4(85,51)=  4.4370833039e-01
       coeff_l4(85,52)= -9.0275481343e-02
       coeff_l4(85,53)= -4.3996497989e-02
       coeff_l4(85,54)=  5.4016470909e-01
       coeff_l4(85,55)= -6.2247108668e-02
       coeff_l4(85,56)=  6.0215997696e-01
       coeff_l4(85,57)=  2.2065603733e-01
       coeff_l4(85,58)= -4.0436949581e-02
       coeff_l4(85,59)= -7.5955766439e-01
       coeff_l4(85,60)= -7.2921401262e-01
       coeff_l4(85,61)= -4.7375163436e-01
       coeff_l4(85,62)= -6.2254172564e-01
       coeff_l4(85,63)= -2.6732695103e-01
       coeff_l4(85,64)= -1.2242065668e+00
       coeff_l4(85,65)= -5.6676369905e-01
       coeff_l4(85,66)= -2.7591294050e-01
       coeff_l4(85,67)=  1.7989718914e-01
       coeff_l4(85,68)=  2.0339664817e-01
       coeff_l4(85,69)= -4.4658130407e-01
       coeff_l4(85,70)= -4.9836510420e-01
       coeff_l4(85,71)=  2.6332375407e-01
       coeff_l4(85,72)=  1.0978954285e-01
       coeff_l4(85,73)=  6.4425903559e-01
       coeff_l4(85,74)=  5.9926253557e-01
       coeff_l4(85,75)= -6.6918486357e-01
       coeff_l4(85,76)= -4.2081794143e-01
       coeff_l4(85,77)=  1.4728480577e+00
       coeff_l4(85,78)= -7.5114572048e-01
       coeff_l4(85,79)=  1.2793111801e+00
       coeff_l4(85,80)= -3.0546978116e-01
       coeff_l4(85,81)= -7.8141820431e-01
       coeff_l4(85,82)= -6.8814808130e-01
       coeff_l4(85,83)=  3.2032170892e-01
       coeff_l4(85,84)=  3.9886140823e-01
       coeff_l4(85,85)= -4.8273167014e-01
       coeff_l4(85,86)=  1.5874302387e-01
       coeff_l4(85,87)=  9.1583210230e-01
       coeff_l4(85,88)=  1.9658625126e-01
       coeff_l4(85,89)=  9.0021973848e-01
       coeff_l4(85,90)= -1.7314289510e-01
       coeff_l4(85,91)=  1.0623084307e+00
       coeff_l4(85,92)=  8.1953299046e-01
       coeff_l4(85,93)= -5.1559603214e-01
       coeff_l4(85,94)=  1.3943166733e+00
       coeff_l4(85,95)=  1.1757831089e-02
       coeff_l4(85,96)=  2.7800956368e-01
       coeff_l4(85,97)=  2.0238262415e-01
       coeff_l4(85,98)=  2.8073027730e-02
       coeff_l4(85,99)= -3.2140198350e-01
       coeff_l4(85,100)= -8.5931634903e-01
       coeff_l4(85,101)=  2.5145870447e-01
       coeff_l4(85,102)= -5.9367084503e-01
       coeff_l4(85,103)= -9.5042592287e-01
       coeff_l4(85,104)= -7.0805633068e-01
       coeff_l4(85,105)= -6.9181054831e-01
       coeff_l4(86,1)= -1.1818814278e+00
       coeff_l4(86,2)= -1.0033706427e+00
       coeff_l4(86,3)= -1.3433698416e+00
       coeff_l4(86,4)= -1.0131113529e+00
       coeff_l4(86,5)= -1.2402328253e+00
       coeff_l4(86,6)= -7.8327953815e-01
       coeff_l4(86,7)= -4.4308438897e-01
       coeff_l4(86,8)= -4.4811092317e-02
       coeff_l4(86,9)= -2.4162730575e-01
       coeff_l4(86,10)= -5.7187432051e-01
       coeff_l4(86,11)= -4.5351915061e-02
       coeff_l4(86,12)= -4.0077230334e-01
       coeff_l4(86,13)= -4.6508172154e-01
       coeff_l4(86,14)=  6.2502771616e-02
       coeff_l4(86,15)=  1.0622771829e-01
       coeff_l4(86,16)=  2.5311252475e-01
       coeff_l4(86,17)=  7.6253063977e-02
       coeff_l4(86,18)=  2.3972408473e-01
       coeff_l4(86,19)=  5.3196195513e-02
       coeff_l4(86,20)= -2.4076532573e-02
       coeff_l4(86,21)=  7.0738688111e-02
       coeff_l4(86,22)=  7.0335194468e-02
       coeff_l4(86,23)= -9.6366524696e-02
       coeff_l4(86,24)= -5.4510440677e-02
       coeff_l4(86,25)= -8.8466621935e-02
       coeff_l4(86,26)=  1.6463810578e-02
       coeff_l4(86,27)= -1.1906092055e-02
       coeff_l4(86,28)=  2.7947857976e-01
       coeff_l4(86,29)= -2.1952392161e-01
       coeff_l4(86,30)=  1.7725168169e-01
       coeff_l4(86,31)= -1.1563274264e-01
       coeff_l4(86,32)= -1.5140861273e-01
       coeff_l4(86,33)=  7.6164022088e-02
       coeff_l4(86,34)= -9.9288985133e-02
       coeff_l4(86,35)= -1.5033151209e-01
       coeff_l4(86,36)=  1.4290325344e-01
       coeff_l4(86,37)=  7.2674010880e-03
       coeff_l4(86,38)=  3.0220636725e-01
       coeff_l4(86,39)= -2.5635915995e-01
       coeff_l4(86,40)=  1.5765203163e-02
       coeff_l4(86,41)= -3.5096380115e-01
       coeff_l4(86,42)=  3.3551406115e-02
       coeff_l4(86,43)=  3.0618423223e-01
       coeff_l4(86,44)= -3.0383747816e-01
       coeff_l4(86,45)= -3.0075871944e-01
       coeff_l4(86,46)=  1.2092591077e-01
       coeff_l4(86,47)= -4.2238477618e-02
       coeff_l4(86,48)=  3.1548416615e-01
       coeff_l4(86,49)= -3.1056961417e-01
       coeff_l4(86,50)=  1.3786314428e-01
       coeff_l4(86,51)=  4.6885337681e-02
       coeff_l4(86,52)= -2.3666615784e-01
       coeff_l4(86,53)= -2.0885011554e-01
       coeff_l4(86,54)= -1.4527250826e-01
       coeff_l4(86,55)=  2.2642457485e-01
       coeff_l4(86,56)=  2.4152573943e-01
       coeff_l4(86,57)= -3.0103385448e-02
       coeff_l4(86,58)=  1.6421408951e-01
       coeff_l4(86,59)= -3.4109674394e-02
       coeff_l4(86,60)= -1.1560259014e-01
       coeff_l4(86,61)=  2.2844430804e-01
       coeff_l4(86,62)=  5.4942250252e-02
       coeff_l4(86,63)= -3.7202969193e-01
       coeff_l4(86,64)=  1.8492776155e-01
       coeff_l4(86,65)= -1.0751298070e-01
       coeff_l4(86,66)= -1.4116850495e-01
       coeff_l4(86,67)=  9.5876241103e-03
       coeff_l4(86,68)=  6.7579969764e-02
       coeff_l4(86,69)=  8.7192893028e-02
       coeff_l4(86,70)=  3.1281191856e-02
       coeff_l4(86,71)= -2.4809662998e-01
       coeff_l4(86,72)= -2.4789823592e-01
       coeff_l4(86,73)= -1.5905271471e-01
       coeff_l4(86,74)=  1.8008886278e-01
       coeff_l4(86,75)=  4.3590247631e-01
       coeff_l4(86,76)=  4.4236525893e-01
       coeff_l4(86,77)= -4.2241367698e-01
       coeff_l4(86,78)=  3.1304228306e-01
       coeff_l4(86,79)= -2.4522544444e-01
       coeff_l4(86,80)=  2.0770367980e-01
       coeff_l4(86,81)= -1.2703487277e-01
       coeff_l4(86,82)=  9.6136376262e-02
       coeff_l4(86,83)= -2.5529298931e-02
       coeff_l4(86,84)= -7.7334433794e-02
       coeff_l4(86,85)=  3.4682843834e-02
       coeff_l4(86,86)= -3.2397824526e-01
       coeff_l4(86,87)= -2.1000975370e-01
       coeff_l4(86,88)= -4.0942545980e-02
       coeff_l4(86,89)= -1.7973861098e-01
       coeff_l4(86,90)=  4.1360691190e-02
       coeff_l4(86,91)=  1.2165865302e-01
       coeff_l4(86,92)= -1.3683116436e-01
       coeff_l4(86,93)= -3.0002992600e-02
       coeff_l4(86,94)= -1.2650796771e-01
       coeff_l4(86,95)=  8.2974344492e-02
       coeff_l4(86,96)=  1.3441641629e-01
       coeff_l4(86,97)=  9.1620080173e-02
       coeff_l4(86,98)= -5.0170559436e-02
       coeff_l4(86,99)= -2.3380288854e-02
       coeff_l4(86,100)= -4.3735612184e-02
       coeff_l4(86,101)= -9.2867061496e-02
       coeff_l4(86,102)= -2.0069645718e-02
       coeff_l4(86,103)=  1.0217285715e-02
       coeff_l4(86,104)= -8.4651604295e-02
       coeff_l4(86,105)= -1.2719716132e-01
       coeff_l4(87,1)= -1.3878871500e-01
       coeff_l4(87,2)=  1.3426515460e-01
       coeff_l4(87,3)=  6.8940632045e-02
       coeff_l4(87,4)= -6.8715368398e-03
       coeff_l4(87,5)= -5.2821587771e-02
       coeff_l4(87,6)=  2.7495643497e-01
       coeff_l4(87,7)= -1.4088532887e-02
       coeff_l4(87,8)= -2.0373265445e-01
       coeff_l4(87,9)= -7.2615280747e-02
       coeff_l4(87,10)= -4.2034128308e-01
       coeff_l4(87,11)= -2.2225294262e-02
       coeff_l4(87,12)=  5.4528467357e-02
       coeff_l4(87,13)= -2.3734697700e-01
       coeff_l4(87,14)=  2.4197056890e-01
       coeff_l4(87,15)= -2.6068918407e-02
       coeff_l4(87,16)= -7.5765788555e-02
       coeff_l4(87,17)= -9.0167403221e-02
       coeff_l4(87,18)=  2.9676130041e-02
       coeff_l4(87,19)= -5.6455772370e-02
       coeff_l4(87,20)=  4.9712866545e-02
       coeff_l4(87,21)=  1.3925065100e-01
       coeff_l4(87,22)= -5.2709326148e-02
       coeff_l4(87,23)= -5.4099038243e-03
       coeff_l4(87,24)=  8.6123561487e-03
       coeff_l4(87,25)= -5.7042445987e-02
       coeff_l4(87,26)= -1.0808676481e-01
       coeff_l4(87,27)= -3.3069644123e-02
       coeff_l4(87,28)=  2.8178631328e-03
       coeff_l4(87,29)=  8.5104899481e-03
       coeff_l4(87,30)=  2.8449565172e-01
       coeff_l4(87,31)= -3.2152023911e-01
       coeff_l4(87,32)= -2.7414904907e-02
       coeff_l4(87,33)=  2.4653089046e-01
       coeff_l4(87,34)= -1.0321344249e-02
       coeff_l4(87,35)= -4.7105757403e-04
       coeff_l4(87,36)=  6.0786705464e-02
       coeff_l4(87,37)= -3.0471973121e-02
       coeff_l4(87,38)=  5.9613175690e-02
       coeff_l4(87,39)= -1.8210319802e-02
       coeff_l4(87,40)= -5.4490700364e-02
       coeff_l4(87,41)= -8.8499099016e-02
       coeff_l4(87,42)= -9.8837517202e-02
       coeff_l4(87,43)=  2.5629652664e-02
       coeff_l4(87,44)= -3.4555710852e-02
       coeff_l4(87,45)= -5.7861011475e-02
       coeff_l4(87,46)= -9.5360219479e-02
       coeff_l4(87,47)=  2.6652019471e-02
       coeff_l4(87,48)=  8.9476428926e-02
       coeff_l4(87,49)=  2.0094232261e-01
       coeff_l4(87,50)= -1.3605982997e-02
       coeff_l4(87,51)= -4.5461244881e-02
       coeff_l4(87,52)=  3.1548637897e-02
       coeff_l4(87,53)=  6.5523153171e-03
       coeff_l4(87,54)=  2.9672868550e-02
       coeff_l4(87,55)=  1.2090984732e-01
       coeff_l4(87,56)= -9.4878181815e-02
       coeff_l4(87,57)=  4.8800662160e-02
       coeff_l4(87,58)= -1.3297152473e-03
       coeff_l4(87,59)=  3.5400189459e-02
       coeff_l4(87,60)=  9.9587701261e-03
       coeff_l4(87,61)=  7.0486180484e-02
       coeff_l4(87,62)=  1.0926205665e-01
       coeff_l4(87,63)= -5.8279260993e-02
       coeff_l4(87,64)= -1.0362228006e-01
       coeff_l4(87,65)= -1.1005074531e-01
       coeff_l4(87,66)= -7.3662258685e-02
       coeff_l4(87,67)= -1.5718413889e-01
       coeff_l4(87,68)= -9.6576951444e-02
       coeff_l4(87,69)= -1.6793183982e-01
       coeff_l4(87,70)=  1.3909186423e-01
       coeff_l4(87,71)=  4.5111554861e-01
       coeff_l4(87,72)= -5.7270199060e-02
       coeff_l4(87,73)=  2.1170949563e-02
       coeff_l4(87,74)= -6.8784311414e-02
       coeff_l4(87,75)= -1.8623209000e-01
       coeff_l4(87,76)= -4.3738283217e-02
       coeff_l4(87,77)=  5.6334342808e-02
       coeff_l4(87,78)= -1.2767194211e-01
       coeff_l4(87,79)=  2.1359004080e-01
       coeff_l4(87,80)= -4.8472493887e-02
       coeff_l4(87,81)= -4.0850747377e-02
       coeff_l4(87,82)= -4.7437477857e-02
       coeff_l4(87,83)=  1.2300041318e-01
       coeff_l4(87,84)=  4.4800907373e-02
       coeff_l4(87,85)=  1.9355069101e-01
       coeff_l4(87,86)= -7.8048557043e-02
       coeff_l4(87,87)= -1.3243117929e-01
       coeff_l4(87,88)= -7.0669595152e-03
       coeff_l4(87,89)=  4.3898355216e-02
       coeff_l4(87,90)= -5.4340776056e-02
       coeff_l4(87,91)= -1.5256552398e-01
       coeff_l4(87,92)=  4.0851019323e-02
       coeff_l4(87,93)=  7.2458073497e-02
       coeff_l4(87,94)=  5.4244148731e-01
       coeff_l4(87,95)=  4.7833933495e-03
       coeff_l4(87,96)=  2.0511594415e-01
       coeff_l4(87,97)=  3.0280333757e-01
       coeff_l4(87,98)=  9.4138182700e-02
       coeff_l4(87,99)= -2.3634068668e-02
       coeff_l4(87,100)= -1.5610137582e-01
       coeff_l4(87,101)=  2.2695710883e-02
       coeff_l4(87,102)=  2.0763662457e-01
       coeff_l4(87,103)= -3.0375337601e-01
       coeff_l4(87,104)= -8.1491507590e-02
       coeff_l4(87,105)= -2.8651168942e-01
       coeff_l4(88,1)=  1.1248027086e+00
       coeff_l4(88,2)=  1.2687836885e+00
       coeff_l4(88,3)=  1.2250614166e+00
       coeff_l4(88,4)=  1.2071648836e+00
       coeff_l4(88,5)=  1.3245443106e+00
       coeff_l4(88,6)=  1.5290833712e+00
       coeff_l4(88,7)=  2.0428800583e+00
       coeff_l4(88,8)=  2.0565743446e+00
       coeff_l4(88,9)= -2.4796715379e-01
       coeff_l4(88,10)= -7.9876869917e-01
       coeff_l4(88,11)= -3.3220559359e-01
       coeff_l4(88,12)= -3.2735061646e-01
       coeff_l4(88,13)=  1.0481281579e-01
       coeff_l4(88,14)= -6.2806200981e-01
       coeff_l4(88,15)= -4.5609369874e-02
       coeff_l4(88,16)=  2.0783881843e-01
       coeff_l4(88,17)=  5.3291406482e-02
       coeff_l4(88,18)=  1.0878739506e-01
       coeff_l4(88,19)= -2.6515603065e-02
       coeff_l4(88,20)= -2.8678357601e-01
       coeff_l4(88,21)=  2.2034046054e-01
       coeff_l4(88,22)= -2.9850411415e-01
       coeff_l4(88,23)=  1.5283958614e-01
       coeff_l4(88,24)= -1.4930807054e-01
       coeff_l4(88,25)= -3.8185775280e-02
       coeff_l4(88,26)=  1.0374389589e-02
       coeff_l4(88,27)=  2.3098234832e-01
       coeff_l4(88,28)= -1.6778886318e-01
       coeff_l4(88,29)= -3.2953076065e-02
       coeff_l4(88,30)= -1.3102732599e-01
       coeff_l4(88,31)=  1.6547678411e-01
       coeff_l4(88,32)=  2.3599345237e-02
       coeff_l4(88,33)= -4.9260552973e-02
       coeff_l4(88,34)=  9.5624238253e-02
       coeff_l4(88,35)=  7.7597387135e-02
       coeff_l4(88,36)= -1.5133985877e-01
       coeff_l4(88,37)= -3.4440180659e-01
       coeff_l4(88,38)= -3.9225623012e-02
       coeff_l4(88,39)= -3.0418032408e-01
       coeff_l4(88,40)= -2.2062224150e-01
       coeff_l4(88,41)= -1.7456585169e-01
       coeff_l4(88,42)=  1.4681832492e-01
       coeff_l4(88,43)=  3.2418897748e-01
       coeff_l4(88,44)= -1.7047123611e-01
       coeff_l4(88,45)= -5.2954010665e-02
       coeff_l4(88,46)=  5.7958666235e-02
       coeff_l4(88,47)= -3.2250189781e-01
       coeff_l4(88,48)=  2.0462036133e-01
       coeff_l4(88,49)= -9.2954367399e-02
       coeff_l4(88,50)=  5.1479831338e-02
       coeff_l4(88,51)= -4.6159821749e-01
       coeff_l4(88,52)=  6.0616757721e-02
       coeff_l4(88,53)= -1.2722018361e-01
       coeff_l4(88,54)= -2.0112060010e-01
       coeff_l4(88,55)= -8.8986262679e-02
       coeff_l4(88,56)=  3.6295759492e-03
       coeff_l4(88,57)=  5.4271627218e-02
       coeff_l4(88,58)=  1.9925406575e-01
       coeff_l4(88,59)=  3.3503714949e-02
       coeff_l4(88,60)=  1.9564063847e-01
       coeff_l4(88,61)=  2.2398446500e-01
       coeff_l4(88,62)= -6.6621415317e-02
       coeff_l4(88,63)=  7.1959622204e-02
       coeff_l4(88,64)=  3.7758615613e-01
       coeff_l4(88,65)= -2.8367751837e-01
       coeff_l4(88,66)=  3.1085434556e-01
       coeff_l4(88,67)=  2.1304857731e-01
       coeff_l4(88,68)=  1.4052036405e-01
       coeff_l4(88,69)=  6.2614515424e-02
       coeff_l4(88,70)=  3.4137642384e-01
       coeff_l4(88,71)= -2.7001976967e-01
       coeff_l4(88,72)=  8.2762666047e-02
       coeff_l4(88,73)=  1.2324633449e-01
       coeff_l4(88,74)=  2.4424944073e-02
       coeff_l4(88,75)=  4.2814937234e-01
       coeff_l4(88,76)= -1.3039404154e-01
       coeff_l4(88,77)=  2.9357336462e-02
       coeff_l4(88,78)=  5.0948619843e-01
       coeff_l4(88,79)= -2.5200781226e-01
       coeff_l4(88,80)=  5.6619328260e-01
       coeff_l4(88,81)=  2.6687595248e-01
       coeff_l4(88,82)=  3.4572407603e-01
       coeff_l4(88,83)= -3.8875302672e-01
       coeff_l4(88,84)= -1.8734559417e-01
       coeff_l4(88,85)= -2.3073464632e-01
       coeff_l4(88,86)= -1.4999189973e-01
       coeff_l4(88,87)= -2.7838167548e-01
       coeff_l4(88,88)= -2.7437743545e-01
       coeff_l4(88,89)= -2.1656562388e-01
       coeff_l4(88,90)=  2.1802479029e-01
       coeff_l4(88,91)= -2.9303738847e-02
       coeff_l4(88,92)=  4.0258431435e-01
       coeff_l4(88,93)= -6.4638420939e-02
       coeff_l4(88,94)=  9.8272472620e-01
       coeff_l4(88,95)=  9.0674251318e-02
       coeff_l4(88,96)=  2.0281419158e-01
       coeff_l4(88,97)=  7.7141964436e-01
       coeff_l4(88,98)= -7.8739374876e-02
       coeff_l4(88,99)= -2.2643624246e-01
       coeff_l4(88,100)= -1.0529441759e-02
       coeff_l4(88,101)= -3.8640677929e-02
       coeff_l4(88,102)=  4.2439382523e-03
       coeff_l4(88,103)=  1.2330398709e-01
       coeff_l4(88,104)= -1.6657651961e-01
       coeff_l4(88,105)= -1.2894088030e-01
       coeff_l4(89,1)= -6.2975215912e-01
       coeff_l4(89,2)= -8.8505357504e-01
       coeff_l4(89,3)= -7.4719524384e-01
       coeff_l4(89,4)= -6.7685526609e-01
       coeff_l4(89,5)= -1.0591927767e+00
       coeff_l4(89,6)= -6.8988877535e-01
       coeff_l4(89,7)= -1.4443264008e+00
       coeff_l4(89,8)= -4.6602413058e-01
       coeff_l4(89,9)= -1.2006473541e+00
       coeff_l4(89,10)= -1.4388486147e+00
       coeff_l4(89,11)= -5.6627511978e-01
       coeff_l4(89,12)= -3.2275515795e-01
       coeff_l4(89,13)= -4.1508007050e-01
       coeff_l4(89,14)= -1.1678814888e+00
       coeff_l4(89,15)=  4.4155783951e-02
       coeff_l4(89,16)=  1.0848961025e-01
       coeff_l4(89,17)=  3.8676187396e-01
       coeff_l4(89,18)= -3.8249516487e-01
       coeff_l4(89,19)=  1.9188612700e-01
       coeff_l4(89,20)=  3.8141512871e-01
       coeff_l4(89,21)= -1.8610063195e-01
       coeff_l4(89,22)=  1.8026976287e-01
       coeff_l4(89,23)= -4.6136122942e-01
       coeff_l4(89,24)=  1.6792148352e-01
       coeff_l4(89,25)=  1.0096736252e-01
       coeff_l4(89,26)=  2.4352318048e-01
       coeff_l4(89,27)=  3.6796781421e-01
       coeff_l4(89,28)= -1.3546715677e-01
       coeff_l4(89,29)= -1.3985572755e-01
       coeff_l4(89,30)=  3.3182370663e-01
       coeff_l4(89,31)=  7.2801280022e-01
       coeff_l4(89,32)=  2.2564980388e-01
       coeff_l4(89,33)=  3.1225967407e-01
       coeff_l4(89,34)=  2.9938462377e-01
       coeff_l4(89,35)= -1.3499172032e-01
       coeff_l4(89,36)= -3.9171832800e-01
       coeff_l4(89,37)= -1.4083029330e-01
       coeff_l4(89,38)= -4.0884528309e-02
       coeff_l4(89,39)=  3.2238587737e-01
       coeff_l4(89,40)= -6.5271906555e-02
       coeff_l4(89,41)=  5.6773805618e-01
       coeff_l4(89,42)= -5.7111158967e-02
       coeff_l4(89,43)= -2.6582738757e-01
       coeff_l4(89,44)=  9.5776200294e-02
       coeff_l4(89,45)= -6.4352512360e-02
       coeff_l4(89,46)= -2.7754271030e-01
       coeff_l4(89,47)=  1.3764141500e-01
       coeff_l4(89,48)= -3.2042738795e-01
       coeff_l4(89,49)= -7.3268568516e-01
       coeff_l4(89,50)= -2.5115706027e-02
       coeff_l4(89,51)=  5.2586948872e-01
       coeff_l4(89,52)= -4.9692225456e-01
       coeff_l4(89,53)=  2.6921594143e-01
       coeff_l4(89,54)=  3.9914074540e-01
       coeff_l4(89,55)= -2.8474247456e-01
       coeff_l4(89,56)=  2.7813234925e-01
       coeff_l4(89,57)= -9.2246860266e-02
       coeff_l4(89,58)= -4.9731028080e-01
       coeff_l4(89,59)=  3.7054914981e-02
       coeff_l4(89,60)= -2.5637254119e-01
       coeff_l4(89,61)= -3.6993494630e-01
       coeff_l4(89,62)= -2.4109071493e-01
       coeff_l4(89,63)= -7.4859552085e-02
       coeff_l4(89,64)=  7.7609522268e-03
       coeff_l4(89,65)=  1.6195505857e-01
       coeff_l4(89,66)=  1.6201229393e-01
       coeff_l4(89,67)=  2.7223473787e-01
       coeff_l4(89,68)=  4.2445921898e-01
       coeff_l4(89,69)=  2.5215733051e-01
       coeff_l4(89,70)= -1.0625887662e-01
       coeff_l4(89,71)=  4.7275662422e-01
       coeff_l4(89,72)= -4.4525906444e-01
       coeff_l4(89,73)= -2.2683800757e-01
       coeff_l4(89,74)=  1.0755429044e-02
       coeff_l4(89,75)= -2.1619972587e-01
       coeff_l4(89,76)=  2.0543602109e-01
       coeff_l4(89,77)=  1.5847131610e-01
       coeff_l4(89,78)= -3.2501661777e-01
       coeff_l4(89,79)=  1.1293054372e-01
       coeff_l4(89,80)= -3.7274760008e-01
       coeff_l4(89,81)= -2.2262181342e-01
       coeff_l4(89,82)=  1.4563585632e-02
       coeff_l4(89,83)=  6.6156131029e-01
       coeff_l4(89,84)=  1.0788866878e-01
       coeff_l4(89,85)= -1.9085301459e-01
       coeff_l4(89,86)= -1.2119527161e-01
       coeff_l4(89,87)=  5.5331412703e-02
       coeff_l4(89,88)= -4.8155099154e-02
       coeff_l4(89,89)= -6.0482114553e-02
       coeff_l4(89,90)= -8.1015691161e-02
       coeff_l4(89,91)=  7.7915504575e-02
       coeff_l4(89,92)=  1.1721450835e-01
       coeff_l4(89,93)= -2.2986648977e-01
       coeff_l4(89,94)=  3.7912586331e-01
       coeff_l4(89,95)= -2.6755118370e-01
       coeff_l4(89,96)= -5.4038256407e-01
       coeff_l4(89,97)=  3.6130923033e-01
       coeff_l4(89,98)=  1.7930473387e-01
       coeff_l4(89,99)= -1.1184598505e-01
       coeff_l4(89,100)= -1.9963651896e-01
       coeff_l4(89,101)= -1.2585398555e-01
       coeff_l4(89,102)= -6.8349450827e-01
       coeff_l4(89,103)= -1.1866250075e-02
       coeff_l4(89,104)= -3.6128848791e-01
       coeff_l4(89,105)= -3.6967894435e-01
       coeff_l4(90,1)=  6.5873825550e-01
       coeff_l4(90,2)=  8.5925894976e-01
       coeff_l4(90,3)=  8.6209833622e-01
       coeff_l4(90,4)=  9.2275416851e-01
       coeff_l4(90,5)=  6.8984347582e-01
       coeff_l4(90,6)=  5.9543478489e-01
       coeff_l4(90,7)=  8.1197464466e-01
       coeff_l4(90,8)=  7.3410874605e-01
       coeff_l4(90,9)= -1.5735663474e-01
       coeff_l4(90,10)=  1.5854935348e-01
       coeff_l4(90,11)= -8.7659919262e-01
       coeff_l4(90,12)= -1.1645112038e+00
       coeff_l4(90,13)= -7.0620274544e-01
       coeff_l4(90,14)=  3.9993306994e-01
       coeff_l4(90,15)= -5.9309415519e-02
       coeff_l4(90,16)= -7.6901413500e-02
       coeff_l4(90,17)=  3.0347472057e-02
       coeff_l4(90,18)=  1.2702654302e-01
       coeff_l4(90,19)=  1.0739126056e-01
       coeff_l4(90,20)= -6.4466334879e-02
       coeff_l4(90,21)=  8.3981655538e-02
       coeff_l4(90,22)=  4.5021451078e-03
       coeff_l4(90,23)= -1.4337699115e-01
       coeff_l4(90,24)=  2.5906211138e-01
       coeff_l4(90,25)= -1.8441109359e-01
       coeff_l4(90,26)=  1.5697856247e-01
       coeff_l4(90,27)=  2.1545821428e-01
       coeff_l4(90,28)=  1.4872570336e-01
       coeff_l4(90,29)=  2.8690230101e-02
       coeff_l4(90,30)=  2.8052929789e-02
       coeff_l4(90,31)=  1.2548039667e-02
       coeff_l4(90,32)=  1.5722675622e-01
       coeff_l4(90,33)= -5.2910368890e-02
       coeff_l4(90,34)= -1.3599772751e-01
       coeff_l4(90,35)= -1.7592400312e-01
       coeff_l4(90,36)=  7.9003036022e-02
       coeff_l4(90,37)= -6.9254790433e-03
       coeff_l4(90,38)= -1.2225361168e-01
       coeff_l4(90,39)=  6.3585087657e-02
       coeff_l4(90,40)= -2.3898135871e-02
       coeff_l4(90,41)= -2.4352957308e-01
       coeff_l4(90,42)=  2.3580467701e-01
       coeff_l4(90,43)=  1.6887186095e-02
       coeff_l4(90,44)= -8.9332416654e-02
       coeff_l4(90,45)=  5.4043088108e-02
       coeff_l4(90,46)=  1.3278281316e-02
       coeff_l4(90,47)=  2.2366873920e-01
       coeff_l4(90,48)= -2.7163954452e-02
       coeff_l4(90,49)= -7.3284320533e-02
       coeff_l4(90,50)=  1.7120568082e-02
       coeff_l4(90,51)=  2.4449416995e-01
       coeff_l4(90,52)= -7.7962018549e-02
       coeff_l4(90,53)=  8.2000702620e-02
       coeff_l4(90,54)=  1.1640860140e-01
       coeff_l4(90,55)=  1.1511450261e-01
       coeff_l4(90,56)= -5.6786987931e-02
       coeff_l4(90,57)=  8.4507174790e-02
       coeff_l4(90,58)= -6.5785072744e-02
       coeff_l4(90,59)= -3.6958444864e-02
       coeff_l4(90,60)=  1.0226748884e-02
       coeff_l4(90,61)= -1.8551388383e-01
       coeff_l4(90,62)= -8.1482477486e-02
       coeff_l4(90,63)= -1.8896766007e-01
       coeff_l4(90,64)=  1.9002415240e-01
       coeff_l4(90,65)= -4.1273642331e-02
       coeff_l4(90,66)=  2.0454554260e-01
       coeff_l4(90,67)=  7.3068246245e-02
       coeff_l4(90,68)= -3.4974340349e-02
       coeff_l4(90,69)= -1.9202676776e-04
       coeff_l4(90,70)=  1.3454052806e-01
       coeff_l4(90,71)=  1.4454520307e-02
       coeff_l4(90,72)= -1.3876809180e-01
       coeff_l4(90,73)=  1.0682658851e-01
       coeff_l4(90,74)= -2.4098090827e-02
       coeff_l4(90,75)= -1.3391378522e-01
       coeff_l4(90,76)=  3.8845926523e-02
       coeff_l4(90,77)= -5.7782724500e-02
       coeff_l4(90,78)=  4.4258803129e-02
       coeff_l4(90,79)=  1.0130916536e-01
       coeff_l4(90,80)= -1.8643452227e-01
       coeff_l4(90,81)=  3.4060537815e-02
       coeff_l4(90,82)= -2.0258642733e-02
       coeff_l4(90,83)=  6.7554466426e-02
       coeff_l4(90,84)= -2.0157128572e-02
       coeff_l4(90,85)=  2.3017166182e-02
       coeff_l4(90,86)= -6.8718574941e-02
       coeff_l4(90,87)=  3.1266242266e-02
       coeff_l4(90,88)= -6.4421802759e-02
       coeff_l4(90,89)= -7.9823501408e-02
       coeff_l4(90,90)= -3.1271312386e-02
       coeff_l4(90,91)=  1.9578626752e-01
       coeff_l4(90,92)=  9.5898605883e-02
       coeff_l4(90,93)=  1.1313065141e-01
       coeff_l4(90,94)= -3.2790613174e-01
       coeff_l4(90,95)= -4.3786089867e-02
       coeff_l4(90,96)= -3.2954025269e-01
       coeff_l4(90,97)= -1.8333519995e-01
       coeff_l4(90,98)=  2.5892749429e-01
       coeff_l4(90,99)= -1.8208651245e-01
       coeff_l4(90,100)=  1.1627330631e-01
       coeff_l4(90,101)=  6.4587660134e-02
       coeff_l4(90,102)= -1.1966180801e-01
       coeff_l4(90,103)=  5.2657570690e-02
       coeff_l4(90,104)=  3.3717167377e-01
       coeff_l4(90,105)=  1.6585869715e-02
       coeff_l4(91,1)=  4.2230537534e-01
       coeff_l4(91,2)=  1.2497497350e-01
       coeff_l4(91,3)=  1.3126733899e-01
       coeff_l4(91,4)=  1.6844828427e-01
       coeff_l4(91,5)=  1.5272729099e-01
       coeff_l4(91,6)=  3.8720551133e-01
       coeff_l4(91,7)= -1.6227762401e-01
       coeff_l4(91,8)=  4.4031071663e-01
       coeff_l4(91,9)= -1.9041062891e-01
       coeff_l4(91,10)=  2.5252825022e-01
       coeff_l4(91,11)= -5.0626736879e-01
       coeff_l4(91,12)= -1.2541635334e-01
       coeff_l4(91,13)= -6.7890053988e-01
       coeff_l4(91,14)=  7.5456565619e-01
       coeff_l4(91,15)=  1.8503744900e-01
       coeff_l4(91,16)= -1.6169582307e-01
       coeff_l4(91,17)= -7.1152627468e-02
       coeff_l4(91,18)=  1.8633091450e-01
       coeff_l4(91,19)=  1.5702345967e-01
       coeff_l4(91,20)=  9.9903792143e-02
       coeff_l4(91,21)= -1.3494113088e-01
       coeff_l4(91,22)= -1.8837490678e-01
       coeff_l4(91,23)= -1.0055035353e-02
       coeff_l4(91,24)=  3.5529755056e-02
       coeff_l4(91,25)= -1.5524294972e-01
       coeff_l4(91,26)=  1.1168389022e-01
       coeff_l4(91,27)=  6.0109630227e-02
       coeff_l4(91,28)= -6.3613183796e-02
       coeff_l4(91,29)=  1.7344993353e-01
       coeff_l4(91,30)=  9.4162248075e-02
       coeff_l4(91,31)=  2.0592698455e-01
       coeff_l4(91,32)=  9.3402519822e-02
       coeff_l4(91,33)= -1.8408478796e-01
       coeff_l4(91,34)= -7.2263471782e-02
       coeff_l4(91,35)= -1.4177618921e-01
       coeff_l4(91,36)= -4.1132789105e-02
       coeff_l4(91,37)= -2.8533178568e-01
       coeff_l4(91,38)=  2.0708033442e-01
       coeff_l4(91,39)= -1.1921647191e-01
       coeff_l4(91,40)= -4.1838053614e-02
       coeff_l4(91,41)= -1.0237698443e-02
       coeff_l4(91,42)=  1.5948955715e-01
       coeff_l4(91,43)=  1.2218025327e-01
       coeff_l4(91,44)=  4.6965685487e-01
       coeff_l4(91,45)=  4.3058580160e-01
       coeff_l4(91,46)=  1.8427322805e-01
       coeff_l4(91,47)= -1.2899640203e-01
       coeff_l4(91,48)=  2.3178091645e-01
       coeff_l4(91,49)= -3.1910765171e-01
       coeff_l4(91,50)=  2.7970281243e-01
       coeff_l4(91,51)= -1.0961607099e-01
       coeff_l4(91,52)=  1.9948484376e-02
       coeff_l4(91,53)= -2.5811544061e-01
       coeff_l4(91,54)= -4.2789503932e-01
       coeff_l4(91,55)=  5.8923076838e-02
       coeff_l4(91,56)=  7.5229324400e-02
       coeff_l4(91,57)=  4.1572076082e-01
       coeff_l4(91,58)=  1.0767179728e-01
       coeff_l4(91,59)=  5.3990346193e-01
       coeff_l4(91,60)=  1.1992223561e-01
       coeff_l4(91,61)= -2.2123125196e-01
       coeff_l4(91,62)=  8.8857933879e-03
       coeff_l4(91,63)=  2.1440930665e-01
       coeff_l4(91,64)= -3.1258262694e-02
       coeff_l4(91,65)= -9.8023883998e-02
       coeff_l4(91,66)= -9.6786968410e-02
       coeff_l4(91,67)=  2.5196319446e-02
       coeff_l4(91,68)=  2.3912614584e-01
       coeff_l4(91,69)=  4.0797013044e-01
       coeff_l4(91,70)=  3.8759684563e-01
       coeff_l4(91,71)= -1.4505052567e-01
       coeff_l4(91,72)=  6.2240068801e-03
       coeff_l4(91,73)=  1.9640329480e-01
       coeff_l4(91,74)= -1.2405002862e-01
       coeff_l4(91,75)=  6.7729640007e-01
       coeff_l4(91,76)=  1.9060151279e-01
       coeff_l4(91,77)= -2.6964509487e-01
       coeff_l4(91,78)=  1.2042786926e-01
       coeff_l4(91,79)=  1.3485373929e-02
       coeff_l4(91,80)=  1.3165925443e-01
       coeff_l4(91,81)= -3.2277676463e-01
       coeff_l4(91,82)=  7.8568175435e-02
       coeff_l4(91,83)= -1.3991837204e-01
       coeff_l4(91,84)= -4.5220082998e-01
       coeff_l4(91,85)=  6.9419048727e-02
       coeff_l4(91,86)=  5.5174242705e-02
       coeff_l4(91,87)= -3.3053633571e-01
       coeff_l4(91,88)=  2.9802745581e-01
       coeff_l4(91,89)= -4.1485047340e-01
       coeff_l4(91,90)=  2.6308420300e-01
       coeff_l4(91,91)= -3.6102825403e-01
       coeff_l4(91,92)=  1.0325212777e-01
       coeff_l4(91,93)= -3.0270549655e-01
       coeff_l4(91,94)= -6.3599252701e-01
       coeff_l4(91,95)=  6.3370597363e-01
       coeff_l4(91,96)= -1.9576432183e-02
       coeff_l4(91,97)= -4.9205785990e-01
       coeff_l4(91,98)=  2.4441459775e-01
       coeff_l4(91,99)= -5.8729812503e-02
       coeff_l4(91,100)=  2.5185108185e-01
       coeff_l4(91,101)=  9.6244983375e-02
       coeff_l4(91,102)=  4.1869822145e-01
       coeff_l4(91,103)=  1.7411097884e-01
       coeff_l4(91,104)= -2.0002683997e-01
       coeff_l4(91,105)=  4.7180894017e-01
       coeff_l4(92,1)= -4.9448039383e-02
       coeff_l4(92,2)= -8.7069869041e-02
       coeff_l4(92,3)= -1.7952194810e-01
       coeff_l4(92,4)=  8.1844270229e-02
       coeff_l4(92,5)= -2.3985937238e-01
       coeff_l4(92,6)= -2.8229498863e-01
       coeff_l4(92,7)=  5.8296319097e-02
       coeff_l4(92,8)=  1.5616906807e-02
       coeff_l4(92,9)=  7.7696137130e-02
       coeff_l4(92,10)= -8.4811523557e-02
       coeff_l4(92,11)= -1.0962870717e-01
       coeff_l4(92,12)=  1.6300176084e-01
       coeff_l4(92,13)= -1.8225792795e-02
       coeff_l4(92,14)= -9.8544925451e-02
       coeff_l4(92,15)=  6.5654844046e-02
       coeff_l4(92,16)= -3.4747581929e-02
       coeff_l4(92,17)= -7.3166163638e-03
       coeff_l4(92,18)=  5.9652380645e-02
       coeff_l4(92,19)= -2.5994284078e-02
       coeff_l4(92,20)=  9.8513565958e-02
       coeff_l4(92,21)= -5.7576119900e-02
       coeff_l4(92,22)=  9.6392616630e-02
       coeff_l4(92,23)=  7.6392628253e-02
       coeff_l4(92,24)=  1.1168741435e-01
       coeff_l4(92,25)=  3.2440844923e-02
       coeff_l4(92,26)= -1.4065517485e-01
       coeff_l4(92,27)= -4.6504955739e-02
       coeff_l4(92,28)= -8.5371747613e-02
       coeff_l4(92,29)= -9.0615294874e-02
       coeff_l4(92,30)=  8.4423974156e-02
       coeff_l4(92,31)= -6.7301578820e-02
       coeff_l4(92,32)=  4.7924868762e-02
       coeff_l4(92,33)=  2.5576878339e-02
       coeff_l4(92,34)=  5.5651158094e-02
       coeff_l4(92,35)=  1.7125634849e-01
       coeff_l4(92,36)=  5.0323877484e-02
       coeff_l4(92,37)=  1.1315175146e-01
       coeff_l4(92,38)= -7.3948457837e-02
       coeff_l4(92,39)= -1.8443870544e-01
       coeff_l4(92,40)=  6.7425884306e-02
       coeff_l4(92,41)=  5.5762320757e-02
       coeff_l4(92,42)= -2.6460830122e-02
       coeff_l4(92,43)=  1.8944520503e-02
       coeff_l4(92,44)= -1.4777156711e-01
       coeff_l4(92,45)= -2.2256909311e-01
       coeff_l4(92,46)= -7.8653968871e-02
       coeff_l4(92,47)= -4.0334064513e-02
       coeff_l4(92,48)=  1.0533774644e-01
       coeff_l4(92,49)= -1.9082379341e-01
       coeff_l4(92,50)= -9.2349678278e-02
       coeff_l4(92,51)= -5.8725021780e-02
       coeff_l4(92,52)=  2.3309603333e-02
       coeff_l4(92,53)=  1.4613935351e-01
       coeff_l4(92,54)=  1.8463641405e-01
       coeff_l4(92,55)= -8.5341215134e-02
       coeff_l4(92,56)= -1.3044920564e-01
       coeff_l4(92,57)=  1.7839245498e-01
       coeff_l4(92,58)= -6.4132630825e-02
       coeff_l4(92,59)=  5.8369137347e-02
       coeff_l4(92,60)= -5.6818123907e-02
       coeff_l4(92,61)= -1.5028831363e-01
       coeff_l4(92,62)=  1.5578568913e-02
       coeff_l4(92,63)= -2.1356427670e-01
       coeff_l4(92,64)=  1.1556996405e-01
       coeff_l4(92,65)= -8.8558308780e-02
       coeff_l4(92,66)=  1.4332605898e-01
       coeff_l4(92,67)= -4.8081381246e-03
       coeff_l4(92,68)= -9.5543088391e-03
       coeff_l4(92,69)=  9.2486307025e-02
       coeff_l4(92,70)=  9.2292450368e-02
       coeff_l4(92,71)= -6.9337621331e-02
       coeff_l4(92,72)=  1.0309732705e-01
       coeff_l4(92,73)=  1.1040548980e-01
       coeff_l4(92,74)=  1.8234723806e-01
       coeff_l4(92,75)= -1.1640913039e-01
       coeff_l4(92,76)=  6.8068571389e-02
       coeff_l4(92,77)=  7.4994433671e-03
       coeff_l4(92,78)= -5.0423998386e-02
       coeff_l4(92,79)=  3.5194110125e-02
       coeff_l4(92,80)= -8.3684809506e-02
       coeff_l4(92,81)= -3.2977465540e-02
       coeff_l4(92,82)= -7.7045455575e-02
       coeff_l4(92,83)= -1.6943421215e-02
       coeff_l4(92,84)= -1.6062335670e-01
       coeff_l4(92,85)=  1.0198310018e-01
       coeff_l4(92,86)= -1.0117274523e-01
       coeff_l4(92,87)=  1.7361978069e-02
       coeff_l4(92,88)= -1.6519194469e-02
       coeff_l4(92,89)= -1.1763675511e-01
       coeff_l4(92,90)= -1.2932883576e-02
       coeff_l4(92,91)=  3.0948102474e-02
       coeff_l4(92,92)= -2.9185991734e-02
       coeff_l4(92,93)=  2.4886816740e-01
       coeff_l4(92,94)=  2.6521688700e-01
       coeff_l4(92,95)=  2.7001867071e-02
       coeff_l4(92,96)=  1.4307899773e-01
       coeff_l4(92,97)=  6.9233290851e-02
       coeff_l4(92,98)=  4.8287127167e-02
       coeff_l4(92,99)=  2.9045825824e-02
       coeff_l4(92,100)= -1.6118369997e-01
       coeff_l4(92,101)=  5.3876969963e-02
       coeff_l4(92,102)= -7.2120763361e-02
       coeff_l4(92,103)= -4.8669300973e-02
       coeff_l4(92,104)=  1.0180013627e-01
       coeff_l4(92,105)= -1.8523672223e-01
       coeff_l4(93,1)=  8.9633530378e-01
       coeff_l4(93,2)= -3.6947202682e-01
       coeff_l4(93,3)=  2.2397677600e-01
       coeff_l4(93,4)= -1.5945796669e-01
       coeff_l4(93,5)= -2.0106530190e-01
       coeff_l4(93,6)= -2.5836542249e-01
       coeff_l4(93,7)=  1.2745523453e-01
       coeff_l4(93,8)= -5.1675356925e-02
       coeff_l4(93,9)= -2.1082425117e-01
       coeff_l4(93,10)= -3.6515071988e-01
       coeff_l4(93,11)=  1.6262361407e-01
       coeff_l4(93,12)=  2.3325335234e-02
       coeff_l4(93,13)=  4.9845898151e-01
       coeff_l4(93,14)= -4.0346857905e-01
       coeff_l4(93,15)= -1.4761517942e-01
       coeff_l4(93,16)=  2.8827854991e-01
       coeff_l4(93,17)= -1.5974505246e-01
       coeff_l4(93,18)=  1.4013642073e-01
       coeff_l4(93,19)= -2.4623995647e-02
       coeff_l4(93,20)= -7.2562448680e-02
       coeff_l4(93,21)=  3.7669217587e-01
       coeff_l4(93,22)=  7.6532443054e-03
       coeff_l4(93,23)= -4.8232063651e-01
       coeff_l4(93,24)=  3.0044093728e-01
       coeff_l4(93,25)= -2.4709406495e-01
       coeff_l4(93,26)=  2.4763515592e-01
       coeff_l4(93,27)= -5.8306330442e-01
       coeff_l4(93,28)=  9.2018403113e-02
       coeff_l4(93,29)=  1.6470539570e-01
       coeff_l4(93,30)=  1.3236640394e-01
       coeff_l4(93,31)=  9.0654149652e-02
       coeff_l4(93,32)=  3.5823974013e-01
       coeff_l4(93,33)= -2.2705186903e-01
       coeff_l4(93,34)=  1.1232623458e-01
       coeff_l4(93,35)= -3.3536955714e-01
       coeff_l4(93,36)=  2.1244728565e-01
       coeff_l4(93,37)= -2.8697863221e-01
       coeff_l4(93,38)= -3.2401472330e-01
       coeff_l4(93,39)=  4.4375926256e-01
       coeff_l4(93,40)=  1.0348399729e-01
       coeff_l4(93,41)=  2.1742689610e-01
       coeff_l4(93,42)= -2.9222872853e-01
       coeff_l4(93,43)=  5.5732414126e-02
       coeff_l4(93,44)=  6.0106861591e-01
       coeff_l4(93,45)=  8.5471555591e-02
       coeff_l4(93,46)= -1.1584197730e-01
       coeff_l4(93,47)=  4.4756734371e-01
       coeff_l4(93,48)= -4.6446278691e-02
       coeff_l4(93,49)= -9.7765371203e-02
       coeff_l4(93,50)=  3.3305031061e-01
       coeff_l4(93,51)=  4.8463925719e-02
       coeff_l4(93,52)=  2.7932837605e-01
       coeff_l4(93,53)= -6.4321048558e-02
       coeff_l4(93,54)= -1.9511261582e-01
       coeff_l4(93,55)=  4.8775374889e-01
       coeff_l4(93,56)= -1.9209510088e-01
       coeff_l4(93,57)= -2.8015395999e-01
       coeff_l4(93,58)= -4.0849986672e-01
       coeff_l4(93,59)=  4.8161914945e-01
       coeff_l4(93,60)= -3.9440000057e-01
       coeff_l4(93,61)=  2.7494138479e-01
       coeff_l4(93,62)=  2.3433449864e-01
       coeff_l4(93,63)=  4.3338665366e-01
       coeff_l4(93,64)= -2.9649671167e-02
       coeff_l4(93,65)=  1.8361373246e-01
       coeff_l4(93,66)= -5.8619815856e-02
       coeff_l4(93,67)= -1.0635314137e-01
       coeff_l4(93,68)=  2.7813711762e-01
       coeff_l4(93,69)= -2.0271424949e-01
       coeff_l4(93,70)= -2.0566050708e-01
       coeff_l4(93,71)=  1.4907036722e-01
       coeff_l4(93,72)= -9.5397956669e-02
       coeff_l4(93,73)= -2.9358151555e-01
       coeff_l4(93,74)= -2.6598080993e-02
       coeff_l4(93,75)=  2.7070486546e-01
       coeff_l4(93,76)= -2.0378810167e-01
       coeff_l4(93,77)=  4.4517406821e-01
       coeff_l4(93,78)=  2.3101392388e-01
       coeff_l4(93,79)= -6.6137951612e-01
       coeff_l4(93,80)=  1.9252105057e-01
       coeff_l4(93,81)=  1.2233351916e-01
       coeff_l4(93,82)= -2.2765444219e-01
       coeff_l4(93,83)= -5.3679007292e-01
       coeff_l4(93,84)=  5.0493204594e-01
       coeff_l4(93,85)= -8.9711040258e-02
       coeff_l4(93,86)= -2.2313122451e-01
       coeff_l4(93,87)=  2.4644125998e-01
       coeff_l4(93,88)= -2.3728372157e-01
       coeff_l4(93,89)=  4.9212351441e-01
       coeff_l4(93,90)=  2.7507731318e-01
       coeff_l4(93,91)=  5.4271467030e-02
       coeff_l4(93,92)=  1.1557483673e-01
       coeff_l4(93,93)= -6.7783437669e-02
       coeff_l4(93,94)=  7.9492676258e-01
       coeff_l4(93,95)= -3.9733275771e-01
       coeff_l4(93,96)= -3.0890446901e-01
       coeff_l4(93,97)=  3.9499169588e-01
       coeff_l4(93,98)=  4.5201030374e-01
       coeff_l4(93,99)= -3.4626424313e-01
       coeff_l4(93,100)= -1.5965712070e-01
       coeff_l4(93,101)=  2.1361047029e-01
       coeff_l4(93,102)= -1.3685736060e-01
       coeff_l4(93,103)= -1.3122831285e-01
       coeff_l4(93,104)=  4.2751541734e-01
       coeff_l4(93,105)= -3.0214890838e-01
       coeff_l4(94,1)= -5.5762451887e-01
       coeff_l4(94,2)= -4.6470922232e-01
       coeff_l4(94,3)= -4.1728514433e-01
       coeff_l4(94,4)= -4.0623977780e-01
       coeff_l4(94,5)= -4.9395936728e-01
       coeff_l4(94,6)= -4.8708122969e-01
       coeff_l4(94,7)= -6.4456954598e-02
       coeff_l4(94,8)= -4.3561545014e-01
       coeff_l4(94,9)= -5.7994246483e-01
       coeff_l4(94,10)= -4.0240007639e-01
       coeff_l4(94,11)= -6.6929852962e-01
       coeff_l4(94,12)= -4.7110873461e-01
       coeff_l4(94,13)= -3.9236313105e-01
       coeff_l4(94,14)= -6.4768075943e-02
       coeff_l4(94,15)=  3.3043453097e-01
       coeff_l4(94,16)=  6.7101441324e-02
       coeff_l4(94,17)=  1.1742597446e-02
       coeff_l4(94,18)= -7.0006057620e-02
       coeff_l4(94,19)= -2.5727078319e-01
       coeff_l4(94,20)=  1.4183627209e-03
       coeff_l4(94,21)= -3.3115938306e-01
       coeff_l4(94,22)= -8.2107849419e-02
       coeff_l4(94,23)=  2.8046908975e-01
       coeff_l4(94,24)=  1.0911250487e-02
       coeff_l4(94,25)=  3.6736685783e-02
       coeff_l4(94,26)= -1.8690957129e-01
       coeff_l4(94,27)=  1.9614164531e-01
       coeff_l4(94,28)= -1.5121412277e-01
       coeff_l4(94,29)= -8.8690198958e-02
       coeff_l4(94,30)=  3.2830968499e-02
       coeff_l4(94,31)= -1.7682872713e-01
       coeff_l4(94,32)=  1.8347804248e-01
       coeff_l4(94,33)=  3.6233976483e-02
       coeff_l4(94,34)=  2.4682755768e-01
       coeff_l4(94,35)=  1.8764395267e-02
       coeff_l4(94,36)= -1.7717593908e-01
       coeff_l4(94,37)= -6.3140429556e-02
       coeff_l4(94,38)=  9.5341324806e-02
       coeff_l4(94,39)= -8.7743751705e-02
       coeff_l4(94,40)= -1.1495277286e-02
       coeff_l4(94,41)=  1.7800495028e-01
       coeff_l4(94,42)=  9.1900520027e-02
       coeff_l4(94,43)= -1.6076052189e-01
       coeff_l4(94,44)= -2.7158844471e-01
       coeff_l4(94,45)=  2.8392422944e-02
       coeff_l4(94,46)=  7.2073869407e-02
       coeff_l4(94,47)= -4.9132008106e-02
       coeff_l4(94,48)= -2.7294701338e-01
       coeff_l4(94,49)=  2.9366850853e-01
       coeff_l4(94,50)= -3.9620053023e-02
       coeff_l4(94,51)= -4.1673526168e-02
       coeff_l4(94,52)=  3.8696736097e-02
       coeff_l4(94,53)=  2.9222492129e-02
       coeff_l4(94,54)= -2.1180747077e-02
       coeff_l4(94,55)= -1.5825343132e-01
       coeff_l4(94,56)=  1.9750565290e-01
       coeff_l4(94,57)= -1.6661594808e-01
       coeff_l4(94,58)=  8.6909398437e-02
       coeff_l4(94,59)= -1.8373754621e-01
       coeff_l4(94,60)= -5.5909812450e-02
       coeff_l4(94,61)= -3.8184252381e-01
       coeff_l4(94,62)= -6.3747242093e-02
       coeff_l4(94,63)= -1.9186255336e-01
       coeff_l4(94,64)= -9.5512442291e-02
       coeff_l4(94,65)= -1.2057770789e-01
       coeff_l4(94,66)= -7.3776505888e-02
       coeff_l4(94,67)=  1.2491790950e-01
       coeff_l4(94,68)=  7.3115691543e-02
       coeff_l4(94,69)=  6.1371587217e-02
       coeff_l4(94,70)= -1.5443292260e-01
       coeff_l4(94,71)= -3.1447678804e-01
       coeff_l4(94,72)=  2.5926095247e-01
       coeff_l4(94,73)=  3.2658207417e-01
       coeff_l4(94,74)= -1.9741987810e-02
       coeff_l4(94,75)= -1.9157355651e-02
       coeff_l4(94,76)= -1.7835739255e-01
       coeff_l4(94,77)=  7.9116271809e-03
       coeff_l4(94,78)= -3.2182753086e-01
       coeff_l4(94,79)=  8.4664382041e-02
       coeff_l4(94,80)= -6.0689158738e-02
       coeff_l4(94,81)= -2.7543354034e-01
       coeff_l4(94,82)= -1.8287993735e-03
       coeff_l4(94,83)=  1.6585722566e-01
       coeff_l4(94,84)= -3.5785846412e-02
       coeff_l4(94,85)= -2.6106408238e-01
       coeff_l4(94,86)=  1.9497182220e-02
       coeff_l4(94,87)= -6.9911025465e-02
       coeff_l4(94,88)= -4.6817097813e-02
       coeff_l4(94,89)= -9.0911418200e-02
       coeff_l4(94,90)= -1.4262218773e-01
       coeff_l4(94,91)=  7.1822598577e-02
       coeff_l4(94,92)=  5.1421321929e-02
       coeff_l4(94,93)= -1.0459165275e-01
       coeff_l4(94,94)=  6.2130179256e-02
       coeff_l4(94,95)= -7.4255637825e-02
       coeff_l4(94,96)= -3.3363702893e-01
       coeff_l4(94,97)= -3.2570308447e-01
       coeff_l4(94,98)= -1.5514410101e-02
       coeff_l4(94,99)= -1.5285062790e-01
       coeff_l4(94,100)=  2.6479896903e-01
       coeff_l4(94,101)= -1.0634586215e-01
       coeff_l4(94,102)= -1.0589186847e-01
       coeff_l4(94,103)=  4.5612588525e-01
       coeff_l4(94,104)=  2.9843340162e-03
       coeff_l4(94,105)=  9.7080320120e-02
       coeff_l4(95,1)=  5.7041913271e-01
       coeff_l4(95,2)=  4.0936043859e-01
       coeff_l4(95,3)=  4.7065815330e-01
       coeff_l4(95,4)=  3.3208745718e-01
       coeff_l4(95,5)=  3.7946099043e-01
       coeff_l4(95,6)=  2.2150693834e-01
       coeff_l4(95,7)=  2.5592404604e-01
       coeff_l4(95,8)=  1.9715295732e-01
       coeff_l4(95,9)=  2.0033074915e-01
       coeff_l4(95,10)=  1.4578248560e-01
       coeff_l4(95,11)= -6.5076395869e-02
       coeff_l4(95,12)=  7.0278562605e-02
       coeff_l4(95,13)= -4.1918619536e-03
       coeff_l4(95,14)=  1.3318455219e-01
       coeff_l4(95,15)= -1.1562676169e-02
       coeff_l4(95,16)= -2.3697556555e-01
       coeff_l4(95,17)=  1.4773592353e-02
       coeff_l4(95,18)= -7.0543400943e-02
       coeff_l4(95,19)= -3.8049854338e-02
       coeff_l4(95,20)= -4.5172683895e-02
       coeff_l4(95,21)= -6.5790437162e-02
       coeff_l4(95,22)=  1.7333745956e-01
       coeff_l4(95,23)=  2.6529204100e-02
       coeff_l4(95,24)=  3.2738709450e-01
       coeff_l4(95,25)= -8.8206857443e-02
       coeff_l4(95,26)=  5.3709335625e-02
       coeff_l4(95,27)= -9.8006501794e-02
       coeff_l4(95,28)=  2.3792514205e-01
       coeff_l4(95,29)= -4.4626016170e-02
       coeff_l4(95,30)= -2.4306742847e-01
       coeff_l4(95,31)=  9.2885665596e-02
       coeff_l4(95,32)=  1.7678348720e-01
       coeff_l4(95,33)= -9.5277167857e-02
       coeff_l4(95,34)= -1.4947071671e-01
       coeff_l4(95,35)=  1.0976131260e-01
       coeff_l4(95,36)= -3.8204506040e-02
       coeff_l4(95,37)=  1.1212126166e-01
       coeff_l4(95,38)=  1.7319575418e-03
       coeff_l4(95,39)=  9.5836721361e-02
       coeff_l4(95,40)= -1.0369358212e-01
       coeff_l4(95,41)=  1.8936821818e-01
       coeff_l4(95,42)=  3.4493210912e-01
       coeff_l4(95,43)= -8.2590930164e-02
       coeff_l4(95,44)=  1.6531883180e-01
       coeff_l4(95,45)= -2.3977744579e-01
       coeff_l4(95,46)=  4.3343123049e-02
       coeff_l4(95,47)= -5.0009898841e-02
       coeff_l4(95,48)=  1.7165745795e-01
       coeff_l4(95,49)= -3.8065299392e-02
       coeff_l4(95,50)= -6.4376860857e-02
       coeff_l4(95,51)=  2.4156770110e-01
       coeff_l4(95,52)= -9.0272761881e-03
       coeff_l4(95,53)=  1.3502833247e-01
       coeff_l4(95,54)=  1.3994728029e-01
       coeff_l4(95,55)=  2.0196322352e-02
       coeff_l4(95,56)= -8.9047573507e-02
       coeff_l4(95,57)=  6.4421482384e-02
       coeff_l4(95,58)= -7.2921209037e-02
       coeff_l4(95,59)=  2.7943793684e-02
       coeff_l4(95,60)= -5.3875666112e-02
       coeff_l4(95,61)= -3.5744199157e-01
       coeff_l4(95,62)=  8.7711632252e-02
       coeff_l4(95,63)=  1.6000953317e-01
       coeff_l4(95,64)=  2.2782243788e-01
       coeff_l4(95,65)=  4.7422084957e-02
       coeff_l4(95,66)= -1.4241074026e-01
       coeff_l4(95,67)= -6.7954599857e-02
       coeff_l4(95,68)=  1.0898789018e-01
       coeff_l4(95,69)= -2.1413058043e-01
       coeff_l4(95,70)= -1.3285483420e-01
       coeff_l4(95,71)=  1.1860510707e-01
       coeff_l4(95,72)=  1.7035791278e-01
       coeff_l4(95,73)=  2.4904185906e-02
       coeff_l4(95,74)=  1.4343744516e-01
       coeff_l4(95,75)= -1.0735332035e-02
       coeff_l4(95,76)=  2.4389252067e-01
       coeff_l4(95,77)=  1.0887226462e-01
       coeff_l4(95,78)= -2.7662375569e-01
       coeff_l4(95,79)= -1.9250597060e-01
       coeff_l4(95,80)=  8.2487113774e-02
       coeff_l4(95,81)=  1.3337804377e-01
       coeff_l4(95,82)=  6.4990453422e-02
       coeff_l4(95,83)= -5.7847112417e-02
       coeff_l4(95,84)= -2.4133136868e-01
       coeff_l4(95,85)= -1.1805954576e-01
       coeff_l4(95,86)=  5.3831484169e-02
       coeff_l4(95,87)= -5.9056989849e-02
       coeff_l4(95,88)=  5.0500843674e-02
       coeff_l4(95,89)=  1.0008718073e-01
       coeff_l4(95,90)= -8.6104460061e-02
       coeff_l4(95,91)= -2.8548823670e-02
       coeff_l4(95,92)=  1.0683442652e-01
       coeff_l4(95,93)=  3.3175367862e-02
       coeff_l4(95,94)=  2.2894442081e-01
       coeff_l4(95,95)= -5.8318704367e-02
       coeff_l4(95,96)= -7.9876765609e-02
       coeff_l4(95,97)=  1.8988938630e-01
       coeff_l4(95,98)=  2.0958195627e-01
       coeff_l4(95,99)= -2.1132703125e-01
       coeff_l4(95,100)=  5.3104929626e-02
       coeff_l4(95,101)= -1.6649814323e-02
       coeff_l4(95,102)= -2.1511985362e-01
       coeff_l4(95,103)=  1.7002224922e-02
       coeff_l4(95,104)=  4.9665369093e-02
       coeff_l4(95,105)=  8.6025439203e-02
       coeff_l4(96,1)=  5.2620702982e-01
       coeff_l4(96,2)=  3.8510823250e-01
       coeff_l4(96,3)=  3.8900172710e-01
       coeff_l4(96,4)=  3.1739297509e-01
       coeff_l4(96,5)=  1.2962685525e-01
       coeff_l4(96,6)=  3.3871996403e-01
       coeff_l4(96,7)=  2.5001881644e-02
       coeff_l4(96,8)= -7.4436612427e-02
       coeff_l4(96,9)=  2.4845303968e-02
       coeff_l4(96,10)=  2.2908799350e-01
       coeff_l4(96,11)= -3.3927208185e-01
       coeff_l4(96,12)= -5.1977550983e-01
       coeff_l4(96,13)= -2.6122117043e-01
       coeff_l4(96,14)=  8.8906139135e-02
       coeff_l4(96,15)=  9.1496907175e-02
       coeff_l4(96,16)= -3.4335281700e-02
       coeff_l4(96,17)=  2.8208610415e-01
       coeff_l4(96,18)= -1.8501456082e-01
       coeff_l4(96,19)= -1.4359383285e-01
       coeff_l4(96,20)= -1.0379168391e-01
       coeff_l4(96,21)=  5.1107630134e-02
       coeff_l4(96,22)=  2.0849113166e-01
       coeff_l4(96,23)= -8.9573003352e-02
       coeff_l4(96,24)=  2.7192205191e-01
       coeff_l4(96,25)= -2.3817762733e-01
       coeff_l4(96,26)=  1.2215369940e-01
       coeff_l4(96,27)=  2.5934422389e-02
       coeff_l4(96,28)=  2.4640293419e-01
       coeff_l4(96,29)=  8.5370756686e-02
       coeff_l4(96,30)=  3.8281437010e-02
       coeff_l4(96,31)=  1.8814311922e-01
       coeff_l4(96,32)= -2.5779238343e-01
       coeff_l4(96,33)=  1.5951704979e-01
       coeff_l4(96,34)= -1.7314195633e-01
       coeff_l4(96,35)= -3.4913010895e-02
       coeff_l4(96,36)=  3.8086912036e-01
       coeff_l4(96,37)=  2.6226565242e-02
       coeff_l4(96,38)= -4.1200101376e-01
       coeff_l4(96,39)= -9.7303181887e-02
       coeff_l4(96,40)=  1.6848519444e-02
       coeff_l4(96,41)=  2.4178336561e-01
       coeff_l4(96,42)=  5.6885730475e-02
       coeff_l4(96,43)= -3.6640670151e-03
       coeff_l4(96,44)= -1.3982942700e-01
       coeff_l4(96,45)= -3.9253732562e-01
       coeff_l4(96,46)= -4.8829853535e-01
       coeff_l4(96,47)= -1.1402163655e-01
       coeff_l4(96,48)=  1.4992554486e-01
       coeff_l4(96,49)=  1.4868918061e-01
       coeff_l4(96,50)= -1.6391530633e-01
       coeff_l4(96,51)=  8.7973147631e-02
       coeff_l4(96,52)= -6.7918151617e-02
       coeff_l4(96,53)= -1.3793134689e-01
       coeff_l4(96,54)=  2.8478503227e-01
       coeff_l4(96,55)=  2.7563935518e-01
       coeff_l4(96,56)=  7.8578211367e-02
       coeff_l4(96,57)= -4.9158368260e-02
       coeff_l4(96,58)= -2.4970525876e-02
       coeff_l4(96,59)= -1.5430474281e-01
       coeff_l4(96,60)=  9.8974503577e-02
       coeff_l4(96,61)= -3.0682563782e-01
       coeff_l4(96,62)= -8.3596944809e-02
       coeff_l4(96,63)=  1.2654702365e-01
       coeff_l4(96,64)=  2.0523932576e-01
       coeff_l4(96,65)= -2.1931868792e-01
       coeff_l4(96,66)= -5.7534885406e-01
       coeff_l4(96,67)=  2.0307271183e-01
       coeff_l4(96,68)=  3.1166136265e-01
       coeff_l4(96,69)=  4.5394852757e-02
       coeff_l4(96,70)=  5.4990005493e-01
       coeff_l4(96,71)=  6.2493704259e-02
       coeff_l4(96,72)=  1.4071637392e-01
       coeff_l4(96,73)= -2.4541029334e-01
       coeff_l4(96,74)=  1.9266290590e-02
       coeff_l4(96,75)= -2.8286191821e-01
       coeff_l4(96,76)= -1.0665699095e-01
       coeff_l4(96,77)= -2.9337626696e-01
       coeff_l4(96,78)= -3.9177168161e-02
       coeff_l4(96,79)= -4.7576659918e-01
       coeff_l4(96,80)= -2.6487916708e-02
       coeff_l4(96,81)=  5.2683163434e-02
       coeff_l4(96,82)=  3.0329704285e-01
       coeff_l4(96,83)=  3.0072471499e-01
       coeff_l4(96,84)=  9.0669490397e-02
       coeff_l4(96,85)= -2.2337105870e-01
       coeff_l4(96,86)=  4.0092185140e-02
       coeff_l4(96,87)=  3.6186572909e-01
       coeff_l4(96,88)= -3.4345242381e-01
       coeff_l4(96,89)= -6.2156725675e-02
       coeff_l4(96,90)= -1.9614298642e-01
       coeff_l4(96,91)=  2.2529943287e-01
       coeff_l4(96,92)= -1.3784699142e-01
       coeff_l4(96,93)= -4.9500063062e-02
       coeff_l4(96,94)=  6.6956681013e-01
       coeff_l4(96,95)= -4.5581316948e-01
       coeff_l4(96,96)= -3.8047024608e-01
       coeff_l4(96,97)=  5.6793171167e-01
       coeff_l4(96,98)=  4.5076679438e-02
       coeff_l4(96,99)= -2.4420402944e-02
       coeff_l4(96,100)=  2.4154721200e-01
       coeff_l4(96,101)=  7.7290758491e-02
       coeff_l4(96,102)= -4.1412556171e-01
       coeff_l4(96,103)=  3.9244714379e-01
       coeff_l4(96,104)=  3.9503636956e-01
       coeff_l4(96,105)= -3.1482794881e-01
       coeff_l4(97,1)= -6.6247344017e-01
       coeff_l4(97,2)= -4.3558165431e-01
       coeff_l4(97,3)= -4.9054670334e-01
       coeff_l4(97,4)= -5.0287240744e-01
       coeff_l4(97,5)= -5.0266695023e-01
       coeff_l4(97,6)= -3.7252193689e-01
       coeff_l4(97,7)= -5.3392791748e-01
       coeff_l4(97,8)= -5.4268491268e-01
       coeff_l4(97,9)= -5.7856634259e-02
       coeff_l4(97,10)= -9.2976242304e-02
       coeff_l4(97,11)=  3.2159835100e-01
       coeff_l4(97,12)=  1.5858441591e-02
       coeff_l4(97,13)=  5.0661634654e-02
       coeff_l4(97,14)=  5.1515035331e-02
       coeff_l4(97,15)=  2.2230906785e-01
       coeff_l4(97,16)= -1.5277111530e-01
       coeff_l4(97,17)= -1.7267544568e-01
       coeff_l4(97,18)= -1.6213761270e-01
       coeff_l4(97,19)=  7.4994922616e-03
       coeff_l4(97,20)= -2.0786966383e-01
       coeff_l4(97,21)= -8.8611774147e-02
       coeff_l4(97,22)=  1.2694208324e-01
       coeff_l4(97,23)=  7.9595789313e-02
       coeff_l4(97,24)= -1.0759783536e-01
       coeff_l4(97,25)= -1.2110018730e-01
       coeff_l4(97,26)=  8.0607064068e-02
       coeff_l4(97,27)=  8.6541853845e-02
       coeff_l4(97,28)= -1.1002824455e-01
       coeff_l4(97,29)= -9.3876853585e-02
       coeff_l4(97,30)=  3.2366465777e-02
       coeff_l4(97,31)= -9.2900589108e-02
       coeff_l4(97,32)= -1.0172793269e-01
       coeff_l4(97,33)= -3.1154997647e-02
       coeff_l4(97,34)= -1.9635209814e-02
       coeff_l4(97,35)=  9.7759895027e-02
       coeff_l4(97,36)=  1.4874282479e-01
       coeff_l4(97,37)=  8.4160082042e-02
       coeff_l4(97,38)=  1.0516230017e-01
       coeff_l4(97,39)= -1.9332641363e-01
       coeff_l4(97,40)= -1.8717773259e-01
       coeff_l4(97,41)=  1.6086813807e-01
       coeff_l4(97,42)=  5.1530241966e-02
       coeff_l4(97,43)=  3.8481958210e-02
       coeff_l4(97,44)= -1.0548119992e-01
       coeff_l4(97,45)= -1.4414192736e-01
       coeff_l4(97,46)=  1.8511986732e-01
       coeff_l4(97,47)= -1.7356844619e-03
       coeff_l4(97,48)= -4.5226402581e-02
       coeff_l4(97,49)=  1.9309489429e-01
       coeff_l4(97,50)= -1.3295224309e-01
       coeff_l4(97,51)= -6.7246057093e-02
       coeff_l4(97,52)=  8.8863037527e-02
       coeff_l4(97,53)=  1.2990647554e-01
       coeff_l4(97,54)= -8.6396560073e-02
       coeff_l4(97,55)=  6.1248213053e-02
       coeff_l4(97,56)= -9.8833829165e-02
       coeff_l4(97,57)=  4.4958051294e-02
       coeff_l4(97,58)=  5.0888736732e-03
       coeff_l4(97,59)=  1.3684129715e-01
       coeff_l4(97,60)=  1.7970894277e-01
       coeff_l4(97,61)= -1.5867166221e-01
       coeff_l4(97,62)=  1.8452140689e-01
       coeff_l4(97,63)= -4.7299675643e-02
       coeff_l4(97,64)=  1.3474680483e-02
       coeff_l4(97,65)= -6.3677527010e-02
       coeff_l4(97,66)= -5.2297826856e-02
       coeff_l4(97,67)= -5.6951850653e-02
       coeff_l4(97,68)= -1.0314443707e-01
       coeff_l4(97,69)=  9.3060657382e-02
       coeff_l4(97,70)=  2.3857493699e-01
       coeff_l4(97,71)= -5.9404496104e-02
       coeff_l4(97,72)= -9.0446121991e-02
       coeff_l4(97,73)= -2.5892348960e-02
       coeff_l4(97,74)=  1.0663231835e-02
       coeff_l4(97,75)= -1.7131263018e-01
       coeff_l4(97,76)= -5.7095699012e-02
       coeff_l4(97,77)= -2.8361427784e-01
       coeff_l4(97,78)= -3.6472652573e-03
       coeff_l4(97,79)=  1.6013336182e-01
       coeff_l4(97,80)=  8.2418106496e-02
       coeff_l4(97,81)=  1.7606562003e-02
       coeff_l4(97,82)= -8.0076709390e-02
       coeff_l4(97,83)=  8.9972754940e-03
       coeff_l4(97,84)=  9.9472604692e-02
       coeff_l4(97,85)= -4.0733743459e-02
       coeff_l4(97,86)= -2.2282673419e-01
       coeff_l4(97,87)=  9.2179052532e-02
       coeff_l4(97,88)=  7.2586131282e-03
       coeff_l4(97,89)=  3.9790030569e-02
       coeff_l4(97,90)=  1.1064828932e-01
       coeff_l4(97,91)= -3.1007314101e-02
       coeff_l4(97,92)= -1.1708448827e-01
       coeff_l4(97,93)=  5.0213083625e-02
       coeff_l4(97,94)=  7.8159101307e-02
       coeff_l4(97,95)=  7.9479003325e-03
       coeff_l4(97,96)=  8.2511611283e-02
       coeff_l4(97,97)=  6.0238748789e-02
       coeff_l4(97,98)=  4.7353919595e-02
       coeff_l4(97,99)=  8.9894451201e-02
       coeff_l4(97,100)=  8.2177393138e-02
       coeff_l4(97,101)=  1.1783756316e-01
       coeff_l4(97,102)= -1.3699360192e-01
       coeff_l4(97,103)=  1.3903899491e-01
       coeff_l4(97,104)= -1.2669384480e-01
       coeff_l4(97,105)= -2.1064962447e-01
       coeff_l4(98,1)= -8.3102720976e-01
       coeff_l4(98,2)= -5.9041523933e-01
       coeff_l4(98,3)= -7.2369742393e-01
       coeff_l4(98,4)= -9.1916251183e-01
       coeff_l4(98,5)= -6.1546158791e-01
       coeff_l4(98,6)= -7.4699431658e-01
       coeff_l4(98,7)= -3.0009075999e-01
       coeff_l4(98,8)= -2.6139506698e-01
       coeff_l4(98,9)= -1.0072600842e+00
       coeff_l4(98,10)= -1.0579524040e+00
       coeff_l4(98,11)= -8.3037918806e-01
       coeff_l4(98,12)= -8.5787540674e-01
       coeff_l4(98,13)= -8.3622270823e-01
       coeff_l4(98,14)= -8.5711115599e-01
       coeff_l4(98,15)= -1.0311350971e-01
       coeff_l4(98,16)=  1.4303904772e-01
       coeff_l4(98,17)=  2.7682176232e-01
       coeff_l4(98,18)= -1.8504850566e-01
       coeff_l4(98,19)=  1.4835532010e-01
       coeff_l4(98,20)=  3.4458961338e-02
       coeff_l4(98,21)= -9.0979568660e-02
       coeff_l4(98,22)= -2.1006830502e-03
       coeff_l4(98,23)=  8.5108630359e-02
       coeff_l4(98,24)= -4.5157108456e-02
       coeff_l4(98,25)=  3.2062208652e-01
       coeff_l4(98,26)= -1.9868020713e-01
       coeff_l4(98,27)=  2.2856749594e-01
       coeff_l4(98,28)= -6.3182659447e-02
       coeff_l4(98,29)=  1.9485820830e-01
       coeff_l4(98,30)= -4.0955871344e-02
       coeff_l4(98,31)=  3.9379292727e-01
       coeff_l4(98,32)= -6.1342400312e-01
       coeff_l4(98,33)=  8.5594646633e-02
       coeff_l4(98,34)=  3.3636397123e-01
       coeff_l4(98,35)= -5.7764705271e-02
       coeff_l4(98,36)=  9.5912151039e-02
       coeff_l4(98,37)=  3.1013626605e-02
       coeff_l4(98,38)= -1.5654093027e-01
       coeff_l4(98,39)=  7.9273819923e-02
       coeff_l4(98,40)=  1.6639467329e-02
       coeff_l4(98,41)=  1.3473604620e-01
       coeff_l4(98,42)=  1.4241366088e-01
       coeff_l4(98,43)= -4.4179081917e-02
       coeff_l4(98,44)=  7.8239232302e-02
       coeff_l4(98,45)=  1.5921187401e-01
       coeff_l4(98,46)= -1.3852775097e-01
       coeff_l4(98,47)=  1.8376152217e-01
       coeff_l4(98,48)= -2.7394285798e-01
       coeff_l4(98,49)= -1.3355961442e-01
       coeff_l4(98,50)= -1.2608563900e-01
       coeff_l4(98,51)= -1.0496055335e-01
       coeff_l4(98,52)= -1.6564041376e-01
       coeff_l4(98,53)=  1.9084542990e-01
       coeff_l4(98,54)=  3.3248719573e-01
       coeff_l4(98,55)= -3.0072310567e-01
       coeff_l4(98,56)=  7.1439929307e-02
       coeff_l4(98,57)= -1.8941572309e-01
       coeff_l4(98,58)= -4.2429286987e-02
       coeff_l4(98,59)= -5.7649677992e-01
       coeff_l4(98,60)=  4.9984689802e-02
       coeff_l4(98,61)= -2.4440184236e-01
       coeff_l4(98,62)= -1.2097733468e-01
       coeff_l4(98,63)=  7.5471818447e-02
       coeff_l4(98,64)= -1.9440203905e-01
       coeff_l4(98,65)=  3.0018661171e-02
       coeff_l4(98,66)= -6.8700239062e-02
       coeff_l4(98,67)= -1.0965732485e-01
       coeff_l4(98,68)= -5.5475361645e-02
       coeff_l4(98,69)= -3.0792185664e-01
       coeff_l4(98,70)= -1.8233318627e-01
       coeff_l4(98,71)=  2.4216769636e-01
       coeff_l4(98,72)=  2.1510887146e-01
       coeff_l4(98,73)=  1.9394274056e-01
       coeff_l4(98,74)=  1.1738005280e-01
       coeff_l4(98,75)= -3.3931669593e-01
       coeff_l4(98,76)=  2.0076343417e-01
       coeff_l4(98,77)=  1.4996729791e-01
       coeff_l4(98,78)= -3.0240169168e-01
       coeff_l4(98,79)=  4.0907233953e-01
       coeff_l4(98,80)= -1.7768266797e-01
       coeff_l4(98,81)= -1.5066893399e-01
       coeff_l4(98,82)=  1.7507612705e-01
       coeff_l4(98,83)=  3.3701995015e-01
       coeff_l4(98,84)=  2.6357239485e-01
       coeff_l4(98,85)= -3.1354966760e-01
       coeff_l4(98,86)=  3.6480018497e-01
       coeff_l4(98,87)=  1.7728394270e-01
       coeff_l4(98,88)=  3.3812391758e-01
       coeff_l4(98,89)= -6.2835000455e-02
       coeff_l4(98,90)= -7.4266950833e-04
       coeff_l4(98,91)=  6.2700025737e-02
       coeff_l4(98,92)= -5.5357474834e-02
       coeff_l4(98,93)= -4.3680986762e-01
       coeff_l4(98,94)= -2.6247236133e-01
       coeff_l4(98,95)= -6.9004394114e-02
       coeff_l4(98,96)= -6.3747793436e-02
       coeff_l4(98,97)= -4.1705986857e-01
       coeff_l4(98,98)=  2.1393375471e-02
       coeff_l4(98,99)= -9.3694590032e-02
       coeff_l4(98,100)= -7.2501555085e-02
       coeff_l4(98,101)= -2.9362339526e-02
       coeff_l4(98,102)=  5.7597182691e-02
       coeff_l4(98,103)=  1.4846112579e-02
       coeff_l4(98,104)=  9.1250702739e-02
       coeff_l4(98,105)=  1.1741866916e-01
       coeff_l4(99,1)= -4.7498270869e-01
       coeff_l4(99,2)= -3.7528896332e-01
       coeff_l4(99,3)= -3.1488788128e-01
       coeff_l4(99,4)= -6.3555914164e-01
       coeff_l4(99,5)= -2.1575503051e-01
       coeff_l4(99,6)= -1.5266893804e-01
       coeff_l4(99,7)= -2.3174037039e-01
       coeff_l4(99,8)= -5.0572645664e-01
       coeff_l4(99,9)= -1.2041368484e+00
       coeff_l4(99,10)= -1.1391937733e+00
       coeff_l4(99,11)= -1.2669914961e+00
       coeff_l4(99,12)= -9.6270936728e-01
       coeff_l4(99,13)= -1.3170161247e+00
       coeff_l4(99,14)= -5.3837913275e-01
       coeff_l4(99,15)= -2.5195514783e-02
       coeff_l4(99,16)= -1.4721745625e-02
       coeff_l4(99,17)=  3.1346064061e-02
       coeff_l4(99,18)=  1.5853738878e-03
       coeff_l4(99,19)=  6.7188307643e-02
       coeff_l4(99,20)= -1.2650477886e-01
       coeff_l4(99,21)= -2.3049864173e-01
       coeff_l4(99,22)=  3.3434537053e-01
       coeff_l4(99,23)= -1.2121534348e-01
       coeff_l4(99,24)= -9.0895578265e-02
       coeff_l4(99,25)= -3.3884710073e-01
       coeff_l4(99,26)= -1.9238743186e-01
       coeff_l4(99,27)=  7.6669640839e-02
       coeff_l4(99,28)=  5.5685527623e-02
       coeff_l4(99,29)= -2.3039075732e-01
       coeff_l4(99,30)=  6.8420745432e-02
       coeff_l4(99,31)=  1.6641159356e-01
       coeff_l4(99,32)= -5.7346388698e-02
       coeff_l4(99,33)=  2.0137603581e-01
       coeff_l4(99,34)=  2.9416495562e-01
       coeff_l4(99,35)= -8.3008095622e-02
       coeff_l4(99,36)=  4.9333363771e-01
       coeff_l4(99,37)=  7.7170126140e-02
       coeff_l4(99,38)=  3.4902533889e-01
       coeff_l4(99,39)= -3.7305986881e-01
       coeff_l4(99,40)= -1.8327238038e-02
       coeff_l4(99,41)= -1.3310097158e-01
       coeff_l4(99,42)= -1.1813481152e-01
       coeff_l4(99,43)= -2.5179880857e-01
       coeff_l4(99,44)= -2.6287543774e-01
       coeff_l4(99,45)= -1.4848083258e-01
       coeff_l4(99,46)= -1.4327342808e-01
       coeff_l4(99,47)=  3.8407287002e-01
       coeff_l4(99,48)=  1.3710509241e-01
       coeff_l4(99,49)= -2.1048037708e-01
       coeff_l4(99,50)= -9.3404807150e-02
       coeff_l4(99,51)=  9.3918330967e-02
       coeff_l4(99,52)=  4.5123510063e-02
       coeff_l4(99,53)= -1.2755163014e-01
       coeff_l4(99,54)=  4.3127790093e-02
       coeff_l4(99,55)= -2.4012710899e-02
       coeff_l4(99,56)=  1.8334756792e-01
       coeff_l4(99,57)=  1.4254391193e-01
       coeff_l4(99,58)= -2.5384199619e-01
       coeff_l4(99,59)= -1.0370976478e-01
       coeff_l4(99,60)= -7.8594386578e-02
       coeff_l4(99,61)=  4.4769819826e-02
       coeff_l4(99,62)=  2.3025622964e-01
       coeff_l4(99,63)=  7.2412662208e-02
       coeff_l4(99,64)=  3.2795989513e-01
       coeff_l4(99,65)=  2.5812008977e-01
       coeff_l4(99,66)=  4.9270957708e-02
       coeff_l4(99,67)= -2.2687509656e-01
       coeff_l4(99,68)= -9.4179973006e-02
       coeff_l4(99,69)= -2.2381355986e-02
       coeff_l4(99,70)= -6.6021688282e-02
       coeff_l4(99,71)=  1.0230670869e-01
       coeff_l4(99,72)=  1.7873017490e-01
       coeff_l4(99,73)= -1.4731552452e-02
       coeff_l4(99,74)= -1.0623378307e-01
       coeff_l4(99,75)=  1.4725534618e-01
       coeff_l4(99,76)=  5.1235222816e-01
       coeff_l4(99,77)= -4.4078253210e-02
       coeff_l4(99,78)= -1.2799327075e-01
       coeff_l4(99,79)= -8.7533414364e-01
       coeff_l4(99,80)=  2.7388581634e-01
       coeff_l4(99,81)= -5.2131843567e-01
       coeff_l4(99,82)= -1.4820583165e-01
       coeff_l4(99,83)=  1.0129523277e-01
       coeff_l4(99,84)= -2.7749907970e-01
       coeff_l4(99,85)= -8.8619299233e-02
       coeff_l4(99,86)=  3.2345968485e-01
       coeff_l4(99,87)= -2.8421509266e-01
       coeff_l4(99,88)=  6.1465311050e-01
       coeff_l4(99,89)= -7.9268757254e-03
       coeff_l4(99,90)=  1.7427000403e-01
       coeff_l4(99,91)= -2.4133933708e-02
       coeff_l4(99,92)= -4.0546724200e-01
       coeff_l4(99,93)= -1.7875355482e-01
       coeff_l4(99,94)= -7.6288712025e-01
       coeff_l4(99,95)=  4.2255127430e-01
       coeff_l4(99,96)= -9.3265151978e-01
       coeff_l4(99,97)= -5.5032926798e-01
       coeff_l4(99,98)= -8.6788438261e-02
       coeff_l4(99,99)= -1.5670485795e-02
       coeff_l4(99,100)=  4.7189411521e-01
       coeff_l4(99,101)= -1.9174361601e-02
       coeff_l4(99,102)= -8.1113986671e-02
       coeff_l4(99,103)=  3.2032215595e-01
       coeff_l4(99,104)= -7.4811220169e-02
       coeff_l4(99,105)=  2.6706799865e-01
       coeff_l4(100,1)=  3.5556045175e-01
       coeff_l4(100,2)=  3.0450308323e-01
       coeff_l4(100,3)=  3.6021307111e-01
       coeff_l4(100,4)=  5.7216234505e-02
       coeff_l4(100,5)=  1.4362928271e-01
       coeff_l4(100,6)=  2.3548118770e-01
       coeff_l4(100,7)=  5.3866840899e-02
       coeff_l4(100,8)=  3.6789286882e-02
       coeff_l4(100,9)=  1.3533200324e-01
       coeff_l4(100,10)=  3.2636091113e-01
       coeff_l4(100,11)= -1.5869076550e-01
       coeff_l4(100,12)=  1.1080545932e-01
       coeff_l4(100,13)=  3.2710004598e-02
       coeff_l4(100,14)= -2.6896789670e-02
       coeff_l4(100,15)= -8.7358534336e-02
       coeff_l4(100,16)= -2.2865180671e-01
       coeff_l4(100,17)=  6.1308681965e-02
       coeff_l4(100,18)= -9.7683109343e-02
       coeff_l4(100,19)=  4.4249199331e-02
       coeff_l4(100,20)=  2.3958054185e-01
       coeff_l4(100,21)=  1.6330882907e-01
       coeff_l4(100,22)= -1.9467522204e-01
       coeff_l4(100,23)=  5.0290238112e-02
       coeff_l4(100,24)=  4.0704645216e-02
       coeff_l4(100,25)= -1.3466666639e-01
       coeff_l4(100,26)=  1.1836796999e-01
       coeff_l4(100,27)= -1.8820753694e-01
       coeff_l4(100,28)=  6.2266409397e-02
       coeff_l4(100,29)= -9.2807233334e-02
       coeff_l4(100,30)= -2.1487675607e-01
       coeff_l4(100,31)=  2.6395428181e-01
       coeff_l4(100,32)= -2.7104282752e-02
       coeff_l4(100,33)= -3.3509150147e-02
       coeff_l4(100,34)=  1.2014155276e-02
       coeff_l4(100,35)=  1.0900198668e-01
       coeff_l4(100,36)=  2.5759758428e-03
       coeff_l4(100,37)=  3.3307053149e-02
       coeff_l4(100,38)=  1.0177568765e-03
       coeff_l4(100,39)=  1.7914563417e-01
       coeff_l4(100,40)=  1.0493057221e-01
       coeff_l4(100,41)= -4.2251762003e-02
       coeff_l4(100,42)=  1.3753519952e-01
       coeff_l4(100,43)= -7.6320995577e-03
       coeff_l4(100,44)= -5.2443128079e-03
       coeff_l4(100,45)= -1.3604657352e-01
       coeff_l4(100,46)= -5.7203732431e-02
       coeff_l4(100,47)= -1.0480438918e-01
       coeff_l4(100,48)=  1.2550683459e-03
       coeff_l4(100,49)= -6.6803842783e-02
       coeff_l4(100,50)= -2.7348470688e-01
       coeff_l4(100,51)=  5.6632217020e-02
       coeff_l4(100,52)=  7.2442047298e-02
       coeff_l4(100,53)=  1.1745516956e-01
       coeff_l4(100,54)=  1.3021217287e-01
       coeff_l4(100,55)=  7.2160825133e-02
       coeff_l4(100,56)= -2.0476058125e-02
       coeff_l4(100,57)= -2.7409117669e-02
       coeff_l4(100,58)= -3.0929379165e-02
       coeff_l4(100,59)= -4.1462060064e-02
       coeff_l4(100,60)= -1.3010779023e-01
       coeff_l4(100,61)= -3.6524778605e-01
       coeff_l4(100,62)= -2.2266780958e-02
       coeff_l4(100,63)=  1.1750702560e-01
       coeff_l4(100,64)=  1.2810704112e-01
       coeff_l4(100,65)= -4.5104507357e-02
       coeff_l4(100,66)= -1.3996087015e-02
       coeff_l4(100,67)= -8.3764590323e-02
       coeff_l4(100,68)=  8.1167325377e-02
       coeff_l4(100,69)= -2.0786967874e-01
       coeff_l4(100,70)=  8.0361731350e-02
       coeff_l4(100,71)= -1.5246173739e-01
       coeff_l4(100,72)=  1.2904210389e-01
       coeff_l4(100,73)=  1.6056077555e-02
       coeff_l4(100,74)=  1.6850464046e-01
       coeff_l4(100,75)=  4.5550383627e-02
       coeff_l4(100,76)=  2.0001542568e-01
       coeff_l4(100,77)=  3.0852826312e-02
       coeff_l4(100,78)= -2.5267241523e-02
       coeff_l4(100,79)=  3.3318638802e-02
       coeff_l4(100,80)= -7.0627242327e-02
       coeff_l4(100,81)=  2.0251503587e-01
       coeff_l4(100,82)=  7.5715929270e-02
       coeff_l4(100,83)= -8.8871121407e-02
       coeff_l4(100,84)= -3.1777465343e-01
       coeff_l4(100,85)= -3.0715751648e-01
       coeff_l4(100,86)= -1.1164713651e-01
       coeff_l4(100,87)=  6.2408745289e-03
       coeff_l4(100,88)=  1.2961778045e-01
       coeff_l4(100,89)=  1.1227469891e-01
       coeff_l4(100,90)= -1.2626415491e-01
       coeff_l4(100,91)= -1.1247076839e-01
       coeff_l4(100,92)=  7.4950627983e-02
       coeff_l4(100,93)=  3.0928345397e-02
       coeff_l4(100,94)=  3.4923991561e-01
       coeff_l4(100,95)= -1.1881054193e-01
       coeff_l4(100,96)=  6.4978711307e-02
       coeff_l4(100,97)=  3.4355205297e-01
       coeff_l4(100,98)=  8.5537225008e-02
       coeff_l4(100,99)=  7.5593814254e-02
       coeff_l4(100,100)= -3.8593482226e-02
       coeff_l4(100,101)= -1.3325984776e-01
       coeff_l4(100,102)= -1.9768787548e-02
       coeff_l4(100,103)= -7.9442724586e-02
       coeff_l4(100,104)=  2.0789261162e-01
       coeff_l4(100,105)= -1.2470500916e-01
       coeff_l4(101,1)= -8.8204175234e-01
       coeff_l4(101,2)= -1.0021321774e+00
       coeff_l4(101,3)= -1.3547110558e+00
       coeff_l4(101,4)= -8.6579185724e-01
       coeff_l4(101,5)= -1.0537265539e+00
       coeff_l4(101,6)= -1.2330456972e+00
       coeff_l4(101,7)= -5.8833879232e-01
       coeff_l4(101,8)= -9.7755801678e-01
       coeff_l4(101,9)= -9.2135232687e-01
       coeff_l4(101,10)= -8.5437297821e-01
       coeff_l4(101,11)= -1.9540478289e-01
       coeff_l4(101,12)= -2.3522056639e-01
       coeff_l4(101,13)= -4.1641700268e-01
       coeff_l4(101,14)= -7.5304365158e-01
       coeff_l4(101,15)=  1.2514892220e-01
       coeff_l4(101,16)= -4.5781457424e-01
       coeff_l4(101,17)= -1.7308229208e-01
       coeff_l4(101,18)=  3.2548274845e-02
       coeff_l4(101,19)= -1.2969408184e-02
       coeff_l4(101,20)=  1.3887454569e-01
       coeff_l4(101,21)= -7.2999484837e-02
       coeff_l4(101,22)= -4.3080642819e-02
       coeff_l4(101,23)=  4.9597971141e-02
       coeff_l4(101,24)=  1.0228207707e-01
       coeff_l4(101,25)= -4.6657517552e-02
       coeff_l4(101,26)= -6.2094997615e-02
       coeff_l4(101,27)= -2.7257275581e-01
       coeff_l4(101,28)=  1.6086202115e-02
       coeff_l4(101,29)= -3.5043511540e-02
       coeff_l4(101,30)= -1.4949239790e-01
       coeff_l4(101,31)= -1.0353728384e-01
       coeff_l4(101,32)= -2.9763418436e-01
       coeff_l4(101,33)=  1.6013035178e-01
       coeff_l4(101,34)= -3.1891521811e-01
       coeff_l4(101,35)=  2.0343932509e-01
       coeff_l4(101,36)= -1.7410842702e-02
       coeff_l4(101,37)=  8.9503161609e-02
       coeff_l4(101,38)= -2.4486945570e-01
       coeff_l4(101,39)= -1.5847837785e-03
       coeff_l4(101,40)=  1.5952393413e-02
       coeff_l4(101,41)=  2.7786797285e-01
       coeff_l4(101,42)=  7.5114861131e-02
       coeff_l4(101,43)=  1.6190513968e-01
       coeff_l4(101,44)= -2.3095825315e-01
       coeff_l4(101,45)=  1.6885258257e-01
       coeff_l4(101,46)= -5.1385247707e-01
       coeff_l4(101,47)= -5.2724622190e-02
       coeff_l4(101,48)=  1.3128271699e-01
       coeff_l4(101,49)= -2.8756770492e-01
       coeff_l4(101,50)= -4.3599855155e-02
       coeff_l4(101,51)=  4.1800594330e-01
       coeff_l4(101,52)=  2.2077347338e-01
       coeff_l4(101,53)= -5.4466444999e-02
       coeff_l4(101,54)=  2.2525103390e-01
       coeff_l4(101,55)=  9.4881154597e-02
       coeff_l4(101,56)= -5.5791459978e-02
       coeff_l4(101,57)= -1.4958196878e-01
       coeff_l4(101,58)= -7.5865209103e-02
       coeff_l4(101,59)=  1.8598423898e-01
       coeff_l4(101,60)=  1.9173923880e-02
       coeff_l4(101,61)=  1.3617697358e-01
       coeff_l4(101,62)= -6.5836034715e-02
       coeff_l4(101,63)= -5.3641755134e-02
       coeff_l4(101,64)= -1.4311039448e-01
       coeff_l4(101,65)= -8.6666516960e-02
       coeff_l4(101,66)=  6.3201844692e-02
       coeff_l4(101,67)= -7.3252029717e-02
       coeff_l4(101,68)= -9.2678949237e-02
       coeff_l4(101,69)=  8.1253677607e-02
       coeff_l4(101,70)=  1.3577270508e-01
       coeff_l4(101,71)= -7.3606312275e-02
       coeff_l4(101,72)= -4.9351584166e-02
       coeff_l4(101,73)= -1.0713738389e-02
       coeff_l4(101,74)=  1.8950819969e-01
       coeff_l4(101,75)= -1.7143803835e-01
       coeff_l4(101,76)=  9.7971744835e-02
       coeff_l4(101,77)=  2.5326716900e-01
       coeff_l4(101,78)= -7.0333465934e-02
       coeff_l4(101,79)= -5.8182898909e-02
       coeff_l4(101,80)=  2.1532431245e-01
       coeff_l4(101,81)=  2.3154873401e-02
       coeff_l4(101,82)=  2.3551067337e-02
       coeff_l4(101,83)=  5.4922811687e-02
       coeff_l4(101,84)= -3.0549190938e-02
       coeff_l4(101,85)= -1.1033004522e-01
       coeff_l4(101,86)= -1.1331220716e-01
       coeff_l4(101,87)=  1.4294364490e-02
       coeff_l4(101,88)=  1.9114131574e-03
       coeff_l4(101,89)=  6.7220233381e-02
       coeff_l4(101,90)=  2.1300046146e-01
       coeff_l4(101,91)=  2.2209486365e-01
       coeff_l4(101,92)=  2.0214582980e-01
       coeff_l4(101,93)= -2.0450951159e-01
       coeff_l4(101,94)=  5.0489628315e-01
       coeff_l4(101,95)= -2.7204972506e-01
       coeff_l4(101,96)=  4.5855370164e-01
       coeff_l4(101,97)=  2.4305477738e-01
       coeff_l4(101,98)= -5.4338872433e-01
       coeff_l4(101,99)=  2.2217853367e-01
       coeff_l4(101,100)= -1.6469244659e-01
       coeff_l4(101,101)=  2.7548164129e-01
       coeff_l4(101,102)= -5.5405713618e-02
       coeff_l4(101,103)= -1.0521224886e-01
       coeff_l4(101,104)= -5.5156350136e-02
       coeff_l4(101,105)= -1.6611775756e-01
       coeff_l4(102,1)= -5.2383291721e-01
       coeff_l4(102,2)= -5.3713202477e-01
       coeff_l4(102,3)= -3.7276878953e-01
       coeff_l4(102,4)= -5.4685622454e-01
       coeff_l4(102,5)= -3.9838427305e-01
       coeff_l4(102,6)= -2.7548268437e-01
       coeff_l4(102,7)= -2.6633727551e-01
       coeff_l4(102,8)= -3.8058760762e-01
       coeff_l4(102,9)= -5.1568776369e-01
       coeff_l4(102,10)= -5.3292179108e-01
       coeff_l4(102,11)= -6.8875902891e-01
       coeff_l4(102,12)= -5.5462378263e-01
       coeff_l4(102,13)= -4.1808512807e-01
       coeff_l4(102,14)= -9.8042815924e-02
       coeff_l4(102,15)=  2.5172331929e-01
       coeff_l4(102,16)= -1.5192060173e-01
       coeff_l4(102,17)= -1.1488275602e-02
       coeff_l4(102,18)= -2.1746250987e-01
       coeff_l4(102,19)= -7.4240036309e-02
       coeff_l4(102,20)= -9.4251617789e-02
       coeff_l4(102,21)= -7.2510829195e-03
       coeff_l4(102,22)= -8.5942797363e-02
       coeff_l4(102,23)=  2.4924319983e-01
       coeff_l4(102,24)= -1.7695014179e-01
       coeff_l4(102,25)= -1.1252748221e-01
       coeff_l4(102,26)= -1.3222430646e-01
       coeff_l4(102,27)= -8.3263866603e-02
       coeff_l4(102,28)= -8.7455727160e-02
       coeff_l4(102,29)=  4.2772870511e-02
       coeff_l4(102,30)=  1.2284998782e-02
       coeff_l4(102,31)= -4.0666979551e-01
       coeff_l4(102,32)=  1.5734413266e-01
       coeff_l4(102,33)=  4.5250371099e-02
       coeff_l4(102,34)=  1.2757393718e-01
       coeff_l4(102,35)=  1.2567012012e-01
       coeff_l4(102,36)= -1.4372922480e-01
       coeff_l4(102,37)= -1.0762005299e-01
       coeff_l4(102,38)=  2.3110076785e-01
       coeff_l4(102,39)=  2.7116328478e-02
       coeff_l4(102,40)=  1.1758174002e-01
       coeff_l4(102,41)=  5.3874645382e-02
       coeff_l4(102,42)= -3.1693649292e-01
       coeff_l4(102,43)=  3.4020029008e-02
       coeff_l4(102,44)= -1.7949812114e-01
       coeff_l4(102,45)=  2.1585499868e-02
       coeff_l4(102,46)=  1.5377773345e-01
       coeff_l4(102,47)= -3.0155190825e-01
       coeff_l4(102,48)= -1.4554755390e-01
       coeff_l4(102,49)=  9.4710953534e-02
       coeff_l4(102,50)= -1.5343521535e-01
       coeff_l4(102,51)= -4.3960992247e-02
       coeff_l4(102,52)=  1.0842940211e-01
       coeff_l4(102,53)=  2.5644096732e-01
       coeff_l4(102,54)= -7.2617745027e-03
       coeff_l4(102,55)= -7.5122907758e-02
       coeff_l4(102,56)=  2.5218049064e-02
       coeff_l4(102,57)=  1.5145978332e-01
       coeff_l4(102,58)=  2.2318466008e-01
       coeff_l4(102,59)= -1.0083875060e-01
       coeff_l4(102,60)=  1.0941594839e-01
       coeff_l4(102,61)=  7.8836590052e-02
       coeff_l4(102,62)= -7.9476073384e-02
       coeff_l4(102,63)= -2.7733308077e-01
       coeff_l4(102,64)=  2.5356224179e-01
       coeff_l4(102,65)= -1.4735530317e-01
       coeff_l4(102,66)=  1.3530726731e-01
       coeff_l4(102,67)= -3.6642163992e-02
       coeff_l4(102,68)=  1.1910147965e-01
       coeff_l4(102,69)=  4.5055791736e-02
       coeff_l4(102,70)= -6.0750219971e-02
       coeff_l4(102,71)= -1.6666987538e-01
       coeff_l4(102,72)=  4.3282859027e-02
       coeff_l4(102,73)=  1.3047069311e-01
       coeff_l4(102,74)= -9.3801423907e-02
       coeff_l4(102,75)= -4.9514966086e-03
       coeff_l4(102,76)=  4.1728183627e-02
       coeff_l4(102,77)= -1.1899302155e-01
       coeff_l4(102,78)=  3.2479441166e-01
       coeff_l4(102,79)=  8.2881748676e-02
       coeff_l4(102,80)=  1.2677235901e-01
       coeff_l4(102,81)= -5.2541472018e-02
       coeff_l4(102,82)= -2.2408537567e-02
       coeff_l4(102,83)=  1.4447052777e-01
       coeff_l4(102,84)= -2.1277915686e-02
       coeff_l4(102,85)= -4.7563418746e-02
       coeff_l4(102,86)= -6.0303594917e-02
       coeff_l4(102,87)= -2.5063446164e-01
       coeff_l4(102,88)=  1.1574935913e-01
       coeff_l4(102,89)=  1.8515259027e-02
       coeff_l4(102,90)=  5.2233245224e-02
       coeff_l4(102,91)= -1.4262188971e-01
       coeff_l4(102,92)=  1.2133633345e-01
       coeff_l4(102,93)= -4.9153227359e-02
       coeff_l4(102,94)= -2.4754546583e-01
       coeff_l4(102,95)=  1.8207088113e-01
       coeff_l4(102,96)=  2.1932916716e-02
       coeff_l4(102,97)= -2.3350554705e-01
       coeff_l4(102,98)= -1.6881084442e-01
       coeff_l4(102,99)=  1.1134643853e-01
       coeff_l4(102,100)=  1.3351646066e-01
       coeff_l4(102,101)= -3.3458016813e-02
       coeff_l4(102,102)=  9.2171579599e-02
       coeff_l4(102,103)= -1.0121018626e-02
       coeff_l4(102,104)=  1.2006839365e-01
       coeff_l4(102,105)=  1.2648085132e-02
       coeff_l4(103,1)= -8.0167484283e-01
       coeff_l4(103,2)= -9.0500634909e-01
       coeff_l4(103,3)= -8.7464427948e-01
       coeff_l4(103,4)= -8.2606893778e-01
       coeff_l4(103,5)= -8.0464643240e-01
       coeff_l4(103,6)= -3.4326520562e-01
       coeff_l4(103,7)= -3.5787162185e-01
       coeff_l4(103,8)= -4.0868070722e-01
       coeff_l4(103,9)= -3.9811131358e-01
       coeff_l4(103,10)= -4.1368454695e-01
       coeff_l4(103,11)=  2.8477421403e-01
       coeff_l4(103,12)=  7.5936466455e-02
       coeff_l4(103,13)=  2.3694414645e-02
       coeff_l4(103,14)=  1.4195182920e-01
       coeff_l4(103,15)= -2.3176478222e-02
       coeff_l4(103,16)=  2.0848843455e-01
       coeff_l4(103,17)=  1.3283554465e-02
       coeff_l4(103,18)=  1.9718585536e-02
       coeff_l4(103,19)=  1.5655882657e-01
       coeff_l4(103,20)= -8.3680011332e-02
       coeff_l4(103,21)=  1.7048749328e-01
       coeff_l4(103,22)= -1.8331189454e-01
       coeff_l4(103,23)= -7.5104333460e-02
       coeff_l4(103,24)=  1.6341397166e-01
       coeff_l4(103,25)=  1.4326354861e-01
       coeff_l4(103,26)= -3.1434837729e-02
       coeff_l4(103,27)= -1.3178293407e-01
       coeff_l4(103,28)= -1.6979932785e-01
       coeff_l4(103,29)= -2.0893983543e-01
       coeff_l4(103,30)=  4.5559827238e-02
       coeff_l4(103,31)= -7.0085659623e-02
       coeff_l4(103,32)= -1.3995263726e-02
       coeff_l4(103,33)=  1.8437872827e-01
       coeff_l4(103,34)= -5.8025784791e-02
       coeff_l4(103,35)= -1.9991245121e-02
       coeff_l4(103,36)=  3.3511176705e-02
       coeff_l4(103,37)=  7.7131681144e-02
       coeff_l4(103,38)=  3.0848744512e-01
       coeff_l4(103,39)= -7.4933826923e-02
       coeff_l4(103,40)= -1.6668693721e-01
       coeff_l4(103,41)= -5.1138694398e-03
       coeff_l4(103,42)= -2.5062757730e-01
       coeff_l4(103,43)= -2.4447977543e-02
       coeff_l4(103,44)= -2.5143668056e-01
       coeff_l4(103,45)= -2.8096688911e-02
       coeff_l4(103,46)= -1.9865301251e-01
       coeff_l4(103,47)=  1.2962819636e-01
       coeff_l4(103,48)=  6.1273641884e-02
       coeff_l4(103,49)=  1.3486115634e-01
       coeff_l4(103,50)= -8.3866916597e-02
       coeff_l4(103,51)=  6.7845813930e-02
       coeff_l4(103,52)=  2.6174113154e-03
       coeff_l4(103,53)= -4.3695488572e-01
       coeff_l4(103,54)= -2.6021337509e-01
       coeff_l4(103,55)= -8.2425937057e-02
       coeff_l4(103,56)= -1.7196543515e-01
       coeff_l4(103,57)= -6.0775574297e-02
       coeff_l4(103,58)=  9.6004344523e-02
       coeff_l4(103,59)=  1.4635291696e-01
       coeff_l4(103,60)=  1.4409694076e-01
       coeff_l4(103,61)=  2.0075874031e-01
       coeff_l4(103,62)= -1.1984396726e-01
       coeff_l4(103,63)= -1.9695927203e-01
       coeff_l4(103,64)=  1.1224360019e-01
       coeff_l4(103,65)=  1.3162322342e-01
       coeff_l4(103,66)=  5.0932694227e-02
       coeff_l4(103,67)=  2.1945081651e-01
       coeff_l4(103,68)= -5.8659870178e-02
       coeff_l4(103,69)= -7.5731903315e-02
       coeff_l4(103,70)=  3.1782022119e-01
       coeff_l4(103,71)= -2.0626214147e-01
       coeff_l4(103,72)= -2.4929150939e-02
       coeff_l4(103,73)= -4.5786869526e-01
       coeff_l4(103,74)= -8.1176050007e-02
       coeff_l4(103,75)=  1.6985827684e-01
       coeff_l4(103,76)=  4.6784296632e-02
       coeff_l4(103,77)= -3.8642546535e-01
       coeff_l4(103,78)=  1.9565372169e-01
       coeff_l4(103,79)=  5.7212587446e-02
       coeff_l4(103,80)=  3.3752191067e-01
       coeff_l4(103,81)=  3.9384520054e-01
       coeff_l4(103,82)=  1.7338110507e-01
       coeff_l4(103,83)=  2.4446302652e-01
       coeff_l4(103,84)=  9.9657543004e-02
       coeff_l4(103,85)= -8.9394979179e-02
       coeff_l4(103,86)= -1.8693721294e-01
       coeff_l4(103,87)= -2.4342986941e-01
       coeff_l4(103,88)= -1.8867781386e-02
       coeff_l4(103,89)= -3.6011844873e-01
       coeff_l4(103,90)= -1.6242402792e-01
       coeff_l4(103,91)=  1.9280327484e-02
       coeff_l4(103,92)=  3.5903602839e-03
       coeff_l4(103,93)=  1.0773088038e-01
       coeff_l4(103,94)=  1.3229373097e-01
       coeff_l4(103,95)=  7.2756476700e-02
       coeff_l4(103,96)=  2.2821085155e-01
       coeff_l4(103,97)=  1.2656772137e-01
       coeff_l4(103,98)=  1.4538734220e-02
       coeff_l4(103,99)= -1.0601061396e-02
       coeff_l4(103,100)= -1.0267407447e-01
       coeff_l4(103,101)= -3.6162108183e-02
       coeff_l4(103,102)=  1.0278017074e-01
       coeff_l4(103,103)=  7.9800881445e-02
       coeff_l4(103,104)= -2.4937441573e-02
       coeff_l4(103,105)=  6.4387261868e-02
       coeff_l4(104,1)= -4.5663505793e-01
       coeff_l4(104,2)= -1.1562547088e-01
       coeff_l4(104,3)= -3.9448258281e-01
       coeff_l4(104,4)=  1.2455376983e-01
       coeff_l4(104,5)= -1.9806052744e-01
       coeff_l4(104,6)= -2.2057871521e-01
       coeff_l4(104,7)= -3.8199394941e-01
       coeff_l4(104,8)= -3.1911128759e-01
       coeff_l4(104,9)= -1.6484773159e-01
       coeff_l4(104,10)=  1.2899325788e-01
       coeff_l4(104,11)=  4.6617224813e-02
       coeff_l4(104,12)=  2.8081841767e-02
       coeff_l4(104,13)= -1.5905048698e-02
       coeff_l4(104,14)=  3.1939510256e-02
       coeff_l4(104,15)=  2.6687049866e-01
       coeff_l4(104,16)= -1.1845421046e-01
       coeff_l4(104,17)= -4.7873002291e-01
       coeff_l4(104,18)= -1.7752699554e-01
       coeff_l4(104,19)= -2.3273911327e-02
       coeff_l4(104,20)=  6.8609640002e-02
       coeff_l4(104,21)= -3.6732065678e-01
       coeff_l4(104,22)=  1.3111995161e-01
       coeff_l4(104,23)= -1.9976338372e-02
       coeff_l4(104,24)=  1.4486092329e-01
       coeff_l4(104,25)=  9.3009509146e-02
       coeff_l4(104,26)= -9.6340648830e-02
       coeff_l4(104,27)= -4.7190867364e-02
       coeff_l4(104,28)= -7.9513676465e-02
       coeff_l4(104,29)= -1.3062749803e-01
       coeff_l4(104,30)=  2.7618184686e-01
       coeff_l4(104,31)=  4.2996790260e-02
       coeff_l4(104,32)= -1.1824570596e-01
       coeff_l4(104,33)= -1.1492344737e-01
       coeff_l4(104,34)=  2.1902641654e-01
       coeff_l4(104,35)= -5.8377161622e-02
       coeff_l4(104,36)=  1.6236878932e-01
       coeff_l4(104,37)= -2.0508354902e-01
       coeff_l4(104,38)= -1.4643505216e-01
       coeff_l4(104,39)= -3.2386124134e-02
       coeff_l4(104,40)=  1.3347899914e-01
       coeff_l4(104,41)=  6.8578541279e-02
       coeff_l4(104,42)=  2.0733170211e-01
       coeff_l4(104,43)= -4.6869564056e-01
       coeff_l4(104,44)= -8.9700289071e-02
       coeff_l4(104,45)= -3.5855215788e-01
       coeff_l4(104,46)= -1.2249241024e-01
       coeff_l4(104,47)= -1.5100736916e-01
       coeff_l4(104,48)=  1.2305781245e-01
       coeff_l4(104,49)=  3.9365984499e-02
       coeff_l4(104,50)= -1.0885493457e-01
       coeff_l4(104,51)= -1.0681434721e-01
       coeff_l4(104,52)= -1.9883116707e-02
       coeff_l4(104,53)=  5.0062597729e-03
       coeff_l4(104,54)=  2.6487573981e-02
       coeff_l4(104,55)=  1.7523464561e-01
       coeff_l4(104,56)= -1.9261285663e-01
       coeff_l4(104,57)=  1.3566461205e-01
       coeff_l4(104,58)= -1.2102921307e-01
       coeff_l4(104,59)=  2.6076257229e-01
       coeff_l4(104,60)= -7.7355258167e-02
       coeff_l4(104,61)= -7.5637519360e-02
       coeff_l4(104,62)=  3.9334103465e-01
       coeff_l4(104,63)=  3.3165330533e-03
       coeff_l4(104,64)= -4.7184303403e-03
       coeff_l4(104,65)= -1.7898383737e-01
       coeff_l4(104,66)= -2.4241667986e-01
       coeff_l4(104,67)= -9.5368782058e-03
       coeff_l4(104,68)= -9.2670075595e-02
       coeff_l4(104,69)=  1.7152750492e-01
       coeff_l4(104,70)=  1.1985819787e-01
       coeff_l4(104,71)=  7.3156833649e-02
       coeff_l4(104,72)= -1.3162321411e-02
       coeff_l4(104,73)=  3.4702509642e-01
       coeff_l4(104,74)= -2.2530271113e-01
       coeff_l4(104,75)=  1.8523329496e-01
       coeff_l4(104,76)=  7.5243726373e-02
       coeff_l4(104,77)= -1.2445021421e-01
       coeff_l4(104,78)=  2.0042185485e-01
       coeff_l4(104,79)= -1.9229564071e-01
       coeff_l4(104,80)= -2.2123864293e-01
       coeff_l4(104,81)=  1.6251415014e-01
       coeff_l4(104,82)=  1.8361151218e-01
       coeff_l4(104,83)= -3.3787393570e-01
       coeff_l4(104,84)= -2.0370952785e-01
       coeff_l4(104,85)=  2.5171464682e-01
       coeff_l4(104,86)= -5.8377616107e-02
       coeff_l4(104,87)=  3.5469028354e-01
       coeff_l4(104,88)=  1.2846912444e-01
       coeff_l4(104,89)= -6.9746360183e-02
       coeff_l4(104,90)=  2.1617203951e-01
       coeff_l4(104,91)=  2.2931370139e-01
       coeff_l4(104,92)=  9.2411302030e-03
       coeff_l4(104,93)=  1.1929764599e-01
       coeff_l4(104,94)=  2.6753529906e-01
       coeff_l4(104,95)= -1.2725923955e-01
       coeff_l4(104,96)= -1.8136343360e-01
       coeff_l4(104,97)=  1.1069653928e-01
       coeff_l4(104,98)= -5.1344579458e-01
       coeff_l4(104,99)= -1.9418869913e-01
       coeff_l4(104,100)= -6.5990805626e-02
       coeff_l4(104,101)= -2.5869840384e-01
       coeff_l4(104,102)= -5.5164659023e-01
       coeff_l4(104,103)= -2.9111239314e-01
       coeff_l4(104,104)= -1.6616733372e-01
       coeff_l4(104,105)= -4.8305022717e-01
       coeff_l4(105,1)=  2.0552656651e+00
       coeff_l4(105,2)=  1.6756120920e+00
       coeff_l4(105,3)=  1.8672415018e+00
       coeff_l4(105,4)=  1.9250266552e+00
       coeff_l4(105,5)=  1.7084605694e+00
       coeff_l4(105,6)=  1.2516129017e+00
       coeff_l4(105,7)=  3.0341594219e+00
       coeff_l4(105,8)=  2.4080605507e+00
       coeff_l4(105,9)=  1.2494007349e+00
       coeff_l4(105,10)=  2.1948847771e+00
       coeff_l4(105,11)=  8.7435144186e-01
       coeff_l4(105,12)=  5.4896068573e-01
       coeff_l4(105,13)=  4.2595127225e-01
       coeff_l4(105,14)=  4.0131711960e-01
       coeff_l4(105,15)=  2.0897163451e-01
       coeff_l4(105,16)= -2.7476835251e-01
       coeff_l4(105,17)= -2.4103949964e-01
       coeff_l4(105,18)=  2.3599953949e-01
       coeff_l4(105,19)= -1.4148148894e-01
       coeff_l4(105,20)=  3.0345872045e-02
       coeff_l4(105,21)= -7.7523536980e-02
       coeff_l4(105,22)= -2.4821342528e-01
       coeff_l4(105,23)= -1.0851454735e-01
       coeff_l4(105,24)=  1.9882073998e-01
       coeff_l4(105,25)= -1.5156507492e-01
       coeff_l4(105,26)=  4.4333127141e-01
       coeff_l4(105,27)= -3.4862822294e-01
       coeff_l4(105,28)= -1.2138977647e-01
       coeff_l4(105,29)= -2.9165867716e-02
       coeff_l4(105,30)=  3.1254347414e-02
       coeff_l4(105,31)= -3.9258134365e-01
       coeff_l4(105,32)=  1.2521044910e-01
       coeff_l4(105,33)=  4.0213975310e-01
       coeff_l4(105,34)=  3.0220255256e-01
       coeff_l4(105,35)=  2.8862920403e-01
       coeff_l4(105,36)=  1.0899443179e-01
       coeff_l4(105,37)=  7.2138272226e-02
       coeff_l4(105,38)= -6.9035038352e-02
       coeff_l4(105,39)=  6.2546241283e-01
       coeff_l4(105,40)= -5.3488463163e-02
       coeff_l4(105,41)=  2.6395794749e-01
       coeff_l4(105,42)= -3.3700436354e-01
       coeff_l4(105,43)=  1.3460010290e-01
       coeff_l4(105,44)= -1.6215364635e-01
       coeff_l4(105,45)= -8.0556400120e-02
       coeff_l4(105,46)= -1.3982646167e-01
       coeff_l4(105,47)=  2.6750195026e-01
       coeff_l4(105,48)= -4.0667334199e-01
       coeff_l4(105,49)=  1.1772355437e-01
       coeff_l4(105,50)= -2.8740052134e-02
       coeff_l4(105,51)=  1.7684377730e-01
       coeff_l4(105,52)=  2.8007230163e-01
       coeff_l4(105,53)= -4.7458410263e-02
       coeff_l4(105,54)=  9.1528534889e-02
       coeff_l4(105,55)= -3.4381622076e-01
       coeff_l4(105,56)=  1.4607635140e-01
       coeff_l4(105,57)= -5.8715426922e-01
       coeff_l4(105,58)= -1.2387069315e-01
       coeff_l4(105,59)=  3.0695575476e-01
       coeff_l4(105,60)=  1.4731742442e-01
       coeff_l4(105,61)= -1.8059799448e-02
       coeff_l4(105,62)=  2.2662423551e-01
       coeff_l4(105,63)=  5.0692867488e-02
       coeff_l4(105,64)=  1.2093460560e-01
       coeff_l4(105,65)=  1.0073204339e-01
       coeff_l4(105,66)=  2.5615561008e-01
       coeff_l4(105,67)= -3.1014123559e-01
       coeff_l4(105,68)= -2.9520738125e-01
       coeff_l4(105,69)= -1.1010452360e-01
       coeff_l4(105,70)= -1.6869424284e-01
       coeff_l4(105,71)=  2.8401869535e-01
       coeff_l4(105,72)= -1.3493080437e-01
       coeff_l4(105,73)=  8.3930335939e-02
       coeff_l4(105,74)=  3.0916884542e-01
       coeff_l4(105,75)= -1.1205217987e-01
       coeff_l4(105,76)= -2.1117392182e-01
       coeff_l4(105,77)=  2.0572093129e-01
       coeff_l4(105,78)=  3.5051342845e-01
       coeff_l4(105,79)=  5.4719686508e-01
       coeff_l4(105,80)= -1.4889803529e-01
       coeff_l4(105,81)= -1.3063950837e-01
       coeff_l4(105,82)= -5.1772946119e-01
       coeff_l4(105,83)= -1.8918763101e-01
       coeff_l4(105,84)= -3.6264684796e-01
       coeff_l4(105,85)=  3.3437615633e-01
       coeff_l4(105,86)=  2.9453873634e-01
       coeff_l4(105,87)=  7.6192170382e-02
       coeff_l4(105,88)=  2.1932505071e-02
       coeff_l4(105,89)=  6.9269910455e-02
       coeff_l4(105,90)= -7.3031216860e-01
       coeff_l4(105,91)= -6.0422050953e-01
       coeff_l4(105,92)= -3.2817608118e-01
       coeff_l4(105,93)=  2.2294303775e-01
       coeff_l4(105,94)= -5.0056904554e-01
       coeff_l4(105,95)= -5.9626936913e-01
       coeff_l4(105,96)= -3.0381190777e-01
       coeff_l4(105,97)= -6.4549195766e-01
       coeff_l4(105,98)=  8.5782885551e-02
       coeff_l4(105,99)=  4.3549543619e-01
       coeff_l4(105,100)= -2.8347018361e-01
       coeff_l4(105,101)= -3.2596036792e-01
       coeff_l4(105,102)= -2.3518423736e-01
       coeff_l4(105,103)= -2.3869822919e-01
       coeff_l4(105,104)=  5.5748850107e-02
       coeff_l4(105,105)=  1.0136040449e+00
       coeff_l4(106,1)=  3.3208787441e-01
       coeff_l4(106,2)=  2.7919226885e-01
       coeff_l4(106,3)=  4.6106928587e-01
       coeff_l4(106,4)=  9.4704508781e-02
       coeff_l4(106,5)= -1.1842244118e-01
       coeff_l4(106,6)=  2.6092860103e-01
       coeff_l4(106,7)=  2.6837679744e-01
       coeff_l4(106,8)=  3.2313835621e-01
       coeff_l4(106,9)=  6.1371606588e-01
       coeff_l4(106,10)=  6.1082267761e-01
       coeff_l4(106,11)= -9.3901753426e-02
       coeff_l4(106,12)=  3.4334129095e-01
       coeff_l4(106,13)=  2.2853156924e-01
       coeff_l4(106,14)=  6.5174996853e-01
       coeff_l4(106,15)= -2.6417130232e-01
       coeff_l4(106,16)=  1.3553567231e-01
       coeff_l4(106,17)=  2.3219053447e-01
       coeff_l4(106,18)= -1.3282148540e-01
       coeff_l4(106,19)=  2.2093282640e-01
       coeff_l4(106,20)= -1.9741542637e-01
       coeff_l4(106,21)= -6.8407177925e-02
       coeff_l4(106,22)=  4.0234324336e-01
       coeff_l4(106,23)= -2.3599290848e-01
       coeff_l4(106,24)= -3.0888924003e-01
       coeff_l4(106,25)=  1.5305966139e-02
       coeff_l4(106,26)= -2.8114783764e-01
       coeff_l4(106,27)= -2.8212839738e-02
       coeff_l4(106,28)= -7.0583425462e-02
       coeff_l4(106,29)=  7.1857117116e-02
       coeff_l4(106,30)= -1.7232157290e-01
       coeff_l4(106,31)=  1.4638291299e-01
       coeff_l4(106,32)=  1.2639218010e-02
       coeff_l4(106,33)=  7.8395240009e-02
       coeff_l4(106,34)= -1.6755001247e-01
       coeff_l4(106,35)= -5.2260909230e-02
       coeff_l4(106,36)=  2.3115612566e-01
       coeff_l4(106,37)= -1.8284551799e-01
       coeff_l4(106,38)= -1.4261041582e-01
       coeff_l4(106,39)= -8.0706691369e-03
       coeff_l4(106,40)= -3.1194525957e-01
       coeff_l4(106,41)=  5.0013595819e-01
       coeff_l4(106,42)= -4.2806394398e-02
       coeff_l4(106,43)=  8.0024763942e-02
       coeff_l4(106,44)=  1.6544002295e-01
       coeff_l4(106,45)=  6.8828925490e-02
       coeff_l4(106,46)= -8.0843895674e-02
       coeff_l4(106,47)= -4.9911397696e-01
       coeff_l4(106,48)=  5.1931146532e-02
       coeff_l4(106,49)= -3.7112613209e-03
       coeff_l4(106,50)=  1.9403275847e-01
       coeff_l4(106,51)= -1.0342057794e-01
       coeff_l4(106,52)= -3.1700870395e-01
       coeff_l4(106,53)=  3.2528495789e-01
       coeff_l4(106,54)=  2.8140875697e-01
       coeff_l4(106,55)=  3.6829692125e-01
       coeff_l4(106,56)=  1.4206758142e-01
       coeff_l4(106,57)=  2.6510328054e-01
       coeff_l4(106,58)=  1.3224361837e-01
       coeff_l4(106,59)=  1.4218991995e-01
       coeff_l4(106,60)= -4.8478720710e-03
       coeff_l4(106,61)= -8.1965532154e-03
       coeff_l4(106,62)=  6.0945268720e-02
       coeff_l4(106,63)=  6.8388828076e-03
       coeff_l4(106,64)=  3.6723563075e-01
       coeff_l4(106,65)=  2.2659361362e-01
       coeff_l4(106,66)= -1.9337230921e-01
       coeff_l4(106,67)= -1.6008344293e-01
       coeff_l4(106,68)=  2.1357581019e-01
       coeff_l4(106,69)= -4.7957396507e-01
       coeff_l4(106,70)= -4.2670503259e-02
       coeff_l4(106,71)=  4.4340193272e-01
       coeff_l4(106,72)=  1.3500091434e-01
       coeff_l4(106,73)= -1.5973529220e-01
       coeff_l4(106,74)= -8.8029956818e-01
       coeff_l4(106,75)=  1.9942151848e-03
       coeff_l4(106,76)= -3.3144831657e-02
       coeff_l4(106,77)= -2.9679268599e-01
       coeff_l4(106,78)= -3.0887499452e-01
       coeff_l4(106,79)= -3.7530162930e-01
       coeff_l4(106,80)= -1.8803474307e-01
       coeff_l4(106,81)= -2.3267558217e-01
       coeff_l4(106,82)=  1.9220985472e-01
       coeff_l4(106,83)=  3.2515013218e-01
       coeff_l4(106,84)=  3.2399851084e-01
       coeff_l4(106,85)=  1.0598673671e-01
       coeff_l4(106,86)= -2.8041258454e-01
       coeff_l4(106,87)= -3.6634677649e-01
       coeff_l4(106,88)=  3.0253171921e-01
       coeff_l4(106,89)= -1.1330917478e-01
       coeff_l4(106,90)= -1.7688657343e-01
       coeff_l4(106,91)=  2.5325027108e-01
       coeff_l4(106,92)= -1.5131472051e-01
       coeff_l4(106,93)= -5.9714818001e-01
       coeff_l4(106,94)= -6.2840873003e-01
       coeff_l4(106,95)=  3.2281477004e-02
       coeff_l4(106,96)= -6.2649846077e-01
       coeff_l4(106,97)= -3.8352397084e-01
       coeff_l4(106,98)= -2.9837593436e-01
       coeff_l4(106,99)=  7.5460207462e-01
       coeff_l4(106,100)=  3.0223211646e-01
       coeff_l4(106,101)=  2.1111777425e-01
       coeff_l4(106,102)= -3.1803399324e-02
       coeff_l4(106,103)=  6.5609380603e-02
       coeff_l4(106,104)=  1.8595911562e-01
       coeff_l4(106,105)=  1.5891036019e-02
       coeff_l4(107,1)=  5.3886926174e-01
       coeff_l4(107,2)=  3.5700914264e-01
       coeff_l4(107,3)=  5.4640114307e-01
       coeff_l4(107,4)=  5.0030547380e-01
       coeff_l4(107,5)=  2.5896790624e-01
       coeff_l4(107,6)=  3.8288348913e-01
       coeff_l4(107,7)=  1.3182466030e+00
       coeff_l4(107,8)=  1.5466119051e+00
       coeff_l4(107,9)=  8.3167695999e-01
       coeff_l4(107,10)=  8.1360721588e-01
       coeff_l4(107,11)=  7.4759787321e-01
       coeff_l4(107,12)=  1.3508087397e+00
       coeff_l4(107,13)=  8.6443620920e-01
       coeff_l4(107,14)=  8.3180814981e-01
       coeff_l4(107,15)=  2.1181114018e-01
       coeff_l4(107,16)= -5.6360626221e-01
       coeff_l4(107,17)= -1.9932720065e-01
       coeff_l4(107,18)=  6.9364197552e-02
       coeff_l4(107,19)=  7.0504799485e-02
       coeff_l4(107,20)=  3.4305218607e-02
       coeff_l4(107,21)= -5.7423273101e-03
       coeff_l4(107,22)= -1.3100852072e-01
       coeff_l4(107,23)=  2.1234756708e-01
       coeff_l4(107,24)=  5.6703789160e-03
       coeff_l4(107,25)= -4.4425971806e-02
       coeff_l4(107,26)=  4.2520567775e-02
       coeff_l4(107,27)=  1.4247104526e-01
       coeff_l4(107,28)=  1.1813982576e-01
       coeff_l4(107,29)=  1.2429296970e-01
       coeff_l4(107,30)= -1.6274730861e-01
       coeff_l4(107,31)= -2.0297078788e-01
       coeff_l4(107,32)= -2.6562297344e-01
       coeff_l4(107,33)= -5.9593167156e-02
       coeff_l4(107,34)= -8.9898802340e-02
       coeff_l4(107,35)= -1.4239320159e-01
       coeff_l4(107,36)= -4.3914249539e-01
       coeff_l4(107,37)=  6.2917284667e-02
       coeff_l4(107,38)=  1.9222813845e-01
       coeff_l4(107,39)=  3.5483300686e-01
       coeff_l4(107,40)= -1.5238682739e-02
       coeff_l4(107,41)= -1.2740203738e-01
       coeff_l4(107,42)= -1.8457441032e-01
       coeff_l4(107,43)=  1.2668807805e-01
       coeff_l4(107,44)=  7.9503014684e-02
       coeff_l4(107,45)=  1.5994088352e-01
       coeff_l4(107,46)= -2.6622748002e-02
       coeff_l4(107,47)= -1.7930515110e-02
       coeff_l4(107,48)= -2.5212243199e-02
       coeff_l4(107,49)=  3.4285566211e-01
       coeff_l4(107,50)= -2.2321766615e-01
       coeff_l4(107,51)= -1.1401321739e-01
       coeff_l4(107,52)=  3.3671185374e-02
       coeff_l4(107,53)=  1.5084032714e-01
       coeff_l4(107,54)=  1.7305765301e-02
       coeff_l4(107,55)= -1.5974289179e-01
       coeff_l4(107,56)= -1.4982035756e-01
       coeff_l4(107,57)=  1.2316209078e-01
       coeff_l4(107,58)=  8.8192395866e-02
       coeff_l4(107,59)= -2.5973460078e-01
       coeff_l4(107,60)=  6.1617925763e-02
       coeff_l4(107,61)=  4.0945153683e-02
       coeff_l4(107,62)= -7.5660735369e-02
       coeff_l4(107,63)= -1.7893509567e-01
       coeff_l4(107,64)=  1.9377073273e-02
       coeff_l4(107,65)=  1.5585912764e-01
       coeff_l4(107,66)=  2.5560167432e-01
       coeff_l4(107,67)= -6.8831242621e-02
       coeff_l4(107,68)=  1.2541380525e-01
       coeff_l4(107,69)=  8.7696082890e-02
       coeff_l4(107,70)= -2.9025855660e-01
       coeff_l4(107,71)= -2.3559257388e-01
       coeff_l4(107,72)= -8.3012171090e-02
       coeff_l4(107,73)=  1.1849162728e-01
       coeff_l4(107,74)= -1.7285679281e-01
       coeff_l4(107,75)= -2.7621516585e-01
       coeff_l4(107,76)=  7.2646737099e-02
       coeff_l4(107,77)= -3.8444668055e-02
       coeff_l4(107,78)=  2.1995399892e-01
       coeff_l4(107,79)=  2.5669264793e-01
       coeff_l4(107,80)=  6.1152558774e-02
       coeff_l4(107,81)=  3.3927637339e-01
       coeff_l4(107,82)= -1.4602245390e-01
       coeff_l4(107,83)=  2.0610161126e-01
       coeff_l4(107,84)= -1.8263836205e-01
       coeff_l4(107,85)=  8.0613657832e-02
       coeff_l4(107,86)=  1.5091820061e-01
       coeff_l4(107,87)= -2.0206703246e-01
       coeff_l4(107,88)=  2.0075760782e-01
       coeff_l4(107,89)=  4.5062988065e-03
       coeff_l4(107,90)= -2.7491220832e-01
       coeff_l4(107,91)= -1.1022086442e-01
       coeff_l4(107,92)=  2.1809925139e-01
       coeff_l4(107,93)= -4.5175027102e-02
       coeff_l4(107,94)= -5.1015228033e-01
       coeff_l4(107,95)=  4.9655178189e-01
       coeff_l4(107,96)=  7.4569237232e-01
       coeff_l4(107,97)=  4.5409506559e-01
       coeff_l4(107,98)=  2.1901689470e-01
       coeff_l4(107,99)= -9.8030604422e-03
       coeff_l4(107,100)=  8.7020702660e-02
       coeff_l4(107,101)= -9.8573677242e-02
       coeff_l4(107,102)=  2.1777303889e-02
       coeff_l4(107,103)=  2.5459167361e-01
       coeff_l4(107,104)= -2.3726998270e-01
       coeff_l4(107,105)=  4.4155165553e-01
       coeff_l4(108,1)= -7.1609699726e-01
       coeff_l4(108,2)= -1.0575202852e-01
       coeff_l4(108,3)= -1.5233461559e-01
       coeff_l4(108,4)= -3.5089111328e-01
       coeff_l4(108,5)= -3.1132649630e-02
       coeff_l4(108,6)= -3.8932141662e-01
       coeff_l4(108,7)=  3.6316189915e-02
       coeff_l4(108,8)= -2.3890294135e-01
       coeff_l4(108,9)=  1.2297911942e-01
       coeff_l4(108,10)=  1.5147434175e-01
       coeff_l4(108,11)= -1.0802166909e-01
       coeff_l4(108,12)=  1.0263884068e-01
       coeff_l4(108,13)= -6.9577962160e-02
       coeff_l4(108,14)=  3.3496254683e-01
       coeff_l4(108,15)=  4.3592458963e-01
       coeff_l4(108,16)= -1.4692935348e-01
       coeff_l4(108,17)= -1.6069799662e-01
       coeff_l4(108,18)=  1.7513068160e-03
       coeff_l4(108,19)= -1.2939077616e-01
       coeff_l4(108,20)=  1.9349247217e-02
       coeff_l4(108,21)= -1.3758972287e-01
       coeff_l4(108,22)= -6.6318079829e-02
       coeff_l4(108,23)=  1.8219283223e-01
       coeff_l4(108,24)=  1.3207277656e-01
       coeff_l4(108,25)= -4.7618571669e-02
       coeff_l4(108,26)= -8.5351394955e-04
       coeff_l4(108,27)= -7.0076063275e-02
       coeff_l4(108,28)= -7.4736266397e-03
       coeff_l4(108,29)= -3.0994176865e-01
       coeff_l4(108,30)=  1.4017966390e-01
       coeff_l4(108,31)= -1.0052396357e-01
       coeff_l4(108,32)=  1.9689917564e-01
       coeff_l4(108,33)=  8.0322317779e-02
       coeff_l4(108,34)=  2.7604101226e-02
       coeff_l4(108,35)=  1.4533990622e-01
       coeff_l4(108,36)=  1.2770158239e-02
       coeff_l4(108,37)=  7.7665328979e-02
       coeff_l4(108,38)= -2.0258778334e-01
       coeff_l4(108,39)= -8.2215413451e-02
       coeff_l4(108,40)= -3.8069076836e-02
       coeff_l4(108,41)=  2.3378087580e-01
       coeff_l4(108,42)=  6.9569334388e-02
       coeff_l4(108,43)=  7.0763975382e-02
       coeff_l4(108,44)= -8.8937446475e-02
       coeff_l4(108,45)= -1.7416736484e-01
       coeff_l4(108,46)=  6.8639352918e-02
       coeff_l4(108,47)= -2.3230408132e-01
       coeff_l4(108,48)=  1.3260419667e-01
       coeff_l4(108,49)=  1.7891354859e-01
       coeff_l4(108,50)= -3.3862861991e-01
       coeff_l4(108,51)=  1.5565758944e-01
       coeff_l4(108,52)= -1.1045612395e-01
       coeff_l4(108,53)=  2.3794272542e-01
       coeff_l4(108,54)= -3.0524831265e-02
       coeff_l4(108,55)= -3.2339358330e-01
       coeff_l4(108,56)=  2.0095066726e-01
       coeff_l4(108,57)= -7.0161953568e-02
       coeff_l4(108,58)=  6.0264891945e-03
       coeff_l4(108,59)= -2.4066813290e-01
       coeff_l4(108,60)= -5.6012056768e-02
       coeff_l4(108,61)= -3.1613942981e-01
       coeff_l4(108,62)=  1.7239783704e-01
       coeff_l4(108,63)= -2.4268625677e-01
       coeff_l4(108,64)=  2.9666608572e-01
       coeff_l4(108,65)=  5.1375735551e-02
       coeff_l4(108,66)= -1.6550062597e-01
       coeff_l4(108,67)=  9.4292491674e-02
       coeff_l4(108,68)= -2.2672837973e-01
       coeff_l4(108,69)= -1.5168868005e-01
       coeff_l4(108,70)=  1.5758754313e-01
       coeff_l4(108,71)= -3.9427254349e-02
       coeff_l4(108,72)=  1.9317106903e-01
       coeff_l4(108,73)= -5.7352393866e-02
       coeff_l4(108,74)=  1.8733562529e-01
       coeff_l4(108,75)=  1.1827056855e-01
       coeff_l4(108,76)= -9.5103289932e-03
       coeff_l4(108,77)=  1.9768097997e-01
       coeff_l4(108,78)= -3.1939578056e-01
       coeff_l4(108,79)=  1.1635514349e-01
       coeff_l4(108,80)= -2.5567561388e-01
       coeff_l4(108,81)= -2.4655015767e-01
       coeff_l4(108,82)=  1.8530102074e-01
       coeff_l4(108,83)=  1.9199959934e-01
       coeff_l4(108,84)= -2.3272095621e-01
       coeff_l4(108,85)= -7.2185665369e-02
       coeff_l4(108,86)= -4.0136784315e-02
       coeff_l4(108,87)=  1.8762354553e-01
       coeff_l4(108,88)= -1.1674245447e-01
       coeff_l4(108,89)= -6.1847731471e-02
       coeff_l4(108,90)= -1.4973376528e-04
       coeff_l4(108,91)=  1.3151089847e-01
       coeff_l4(108,92)= -3.0930313468e-01
       coeff_l4(108,93)= -5.6906145066e-02
       coeff_l4(108,94)=  2.5905567408e-01
       coeff_l4(108,95)= -1.1134592444e-01
       coeff_l4(108,96)= -7.6346002519e-02
       coeff_l4(108,97)=  1.3679440320e-01
       coeff_l4(108,98)= -1.5249067917e-02
       coeff_l4(108,99)= -1.1254486442e-01
       coeff_l4(108,100)=  2.0959319547e-02
       coeff_l4(108,101)= -3.7543948740e-02
       coeff_l4(108,102)= -2.5648674369e-01
       coeff_l4(108,103)=  1.2120489031e-02
       coeff_l4(108,104)=  1.4996866882e-01
       coeff_l4(108,105)=  5.0997406244e-02
       coeff_l4(109,1)= -7.0121091604e-01
       coeff_l4(109,2)= -6.2765812874e-01
       coeff_l4(109,3)= -7.5470370054e-01
       coeff_l4(109,4)= -6.4489018917e-01
       coeff_l4(109,5)= -5.7428354025e-01
       coeff_l4(109,6)= -7.4701160192e-01
       coeff_l4(109,7)= -5.9381884336e-01
       coeff_l4(109,8)= -6.0458821058e-01
       coeff_l4(109,9)= -1.7638234794e-01
       coeff_l4(109,10)=  2.9405826703e-02
       coeff_l4(109,11)=  2.9278727248e-02
       coeff_l4(109,12)=  1.6248434782e-01
       coeff_l4(109,13)= -2.6495143771e-02
       coeff_l4(109,14)= -7.0310533047e-02
       coeff_l4(109,15)=  1.4973336458e-01
       coeff_l4(109,16)=  4.3292264454e-03
       coeff_l4(109,17)= -7.5858302414e-02
       coeff_l4(109,18)=  7.7133215964e-02
       coeff_l4(109,19)= -2.1882450208e-02
       coeff_l4(109,20)=  2.2962102666e-02
       coeff_l4(109,21)=  8.8829852641e-02
       coeff_l4(109,22)= -1.0811139643e-01
       coeff_l4(109,23)=  1.5171751380e-01
       coeff_l4(109,24)=  9.8674625158e-02
       coeff_l4(109,25)=  8.8257305324e-02
       coeff_l4(109,26)= -6.9466263056e-02
       coeff_l4(109,27)=  3.2561633736e-02
       coeff_l4(109,28)= -9.7826130688e-02
       coeff_l4(109,29)= -2.4995566905e-01
       coeff_l4(109,30)= -4.5443885028e-02
       coeff_l4(109,31)= -1.6202627122e-01
       coeff_l4(109,32)=  1.3033053279e-01
       coeff_l4(109,33)=  1.8698039651e-01
       coeff_l4(109,34)= -5.6770099327e-03
       coeff_l4(109,35)=  9.3670502305e-02
       coeff_l4(109,36)=  1.5815013647e-01
       coeff_l4(109,37)= -1.2236939371e-01
       coeff_l4(109,38)=  8.7894767523e-02
       coeff_l4(109,39)= -4.0999997407e-02
       coeff_l4(109,40)=  1.3844023645e-01
       coeff_l4(109,41)= -1.4772574417e-02
       coeff_l4(109,42)= -8.6788684130e-02
       coeff_l4(109,43)= -1.2523418665e-01
       coeff_l4(109,44)= -4.7389198095e-02
       coeff_l4(109,45)=  6.0046266764e-02
       coeff_l4(109,46)=  1.2499643862e-01
       coeff_l4(109,47)=  6.5957389772e-02
       coeff_l4(109,48)= -6.9351084530e-02
       coeff_l4(109,49)=  2.3936597630e-02
       coeff_l4(109,50)=  1.2311562151e-01
       coeff_l4(109,51)= -1.1295481771e-01
       coeff_l4(109,52)=  6.3392356038e-02
       coeff_l4(109,53)=  9.6011206508e-02
       coeff_l4(109,54)=  1.6626788303e-02
       coeff_l4(109,55)=  9.7555786371e-02
       coeff_l4(109,56)= -9.0374760330e-02
       coeff_l4(109,57)= -5.3064305335e-02
       coeff_l4(109,58)= -1.1668393761e-01
       coeff_l4(109,59)=  6.5714180470e-02
       coeff_l4(109,60)= -4.1514255106e-02
       coeff_l4(109,61)=  4.9810711294e-02
       coeff_l4(109,62)=  3.6322046071e-02
       coeff_l4(109,63)= -1.5784034133e-01
       coeff_l4(109,64)= -4.7345887870e-02
       coeff_l4(109,65)= -1.3882501423e-01
       coeff_l4(109,66)= -1.3899764419e-01
       coeff_l4(109,67)=  9.7430340946e-02
       coeff_l4(109,68)= -8.0236352980e-02
       coeff_l4(109,69)= -6.7457683384e-02
       coeff_l4(109,70)=  5.9529189020e-02
       coeff_l4(109,71)=  8.7202534080e-02
       coeff_l4(109,72)= -1.3555408455e-02
       coeff_l4(109,73)= -4.6918645501e-02
       coeff_l4(109,74)=  4.3352302164e-02
       coeff_l4(109,75)=  6.7547038198e-02
       coeff_l4(109,76)=  2.2921321914e-02
       coeff_l4(109,77)= -2.0835258067e-01
       coeff_l4(109,78)= -2.3783342913e-02
       coeff_l4(109,79)= -1.9883846864e-02
       coeff_l4(109,80)= -2.7532760054e-02
       coeff_l4(109,81)=  1.4791152440e-02
       coeff_l4(109,82)= -6.0728646815e-02
       coeff_l4(109,83)=  1.1015643179e-01
       coeff_l4(109,84)= -9.4687640667e-03
       coeff_l4(109,85)= -5.0482456572e-03
       coeff_l4(109,86)= -8.8373780251e-02
       coeff_l4(109,87)= -1.6361465678e-02
       coeff_l4(109,88)= -6.3738532364e-02
       coeff_l4(109,89)=  6.0692876577e-02
       coeff_l4(109,90)=  1.7798710614e-02
       coeff_l4(109,91)=  6.1430968344e-02
       coeff_l4(109,92)= -1.5448154509e-01
       coeff_l4(109,93)=  7.3547899723e-02
       coeff_l4(109,94)=  2.2770580649e-01
       coeff_l4(109,95)=  3.9944358170e-02
       coeff_l4(109,96)=  1.4424039423e-01
       coeff_l4(109,97)=  1.0358330794e-02
       coeff_l4(109,98)= -1.1762646586e-01
       coeff_l4(109,99)= -2.7449756861e-02
       coeff_l4(109,100)= -1.9255371764e-02
       coeff_l4(109,101)= -6.4081184566e-02
       coeff_l4(109,102)= -1.1181773990e-01
       coeff_l4(109,103)= -1.0823846608e-01
       coeff_l4(109,104)=  9.7464071587e-03
       coeff_l4(109,105)= -1.2062860280e-01
       coeff_l4(110,1)= -8.6179912090e-01
       coeff_l4(110,2)= -9.7563832998e-01
       coeff_l4(110,3)= -8.7525165081e-01
       coeff_l4(110,4)= -6.3723909855e-01
       coeff_l4(110,5)= -7.9064393044e-01
       coeff_l4(110,6)= -8.4148001671e-01
       coeff_l4(110,7)= -6.6404205561e-01
       coeff_l4(110,8)= -5.9875041246e-01
       coeff_l4(110,9)= -4.5908492804e-01
       coeff_l4(110,10)= -1.2059619278e-01
       coeff_l4(110,11)=  2.7638237923e-02
       coeff_l4(110,12)= -2.6797462255e-02
       coeff_l4(110,13)= -1.6758140922e-01
       coeff_l4(110,14)= -3.6778920889e-01
       coeff_l4(110,15)= -1.2740285695e-01
       coeff_l4(110,16)= -1.7888958752e-01
       coeff_l4(110,17)= -4.3876364827e-02
       coeff_l4(110,18)=  1.1641598493e-01
       coeff_l4(110,19)= -7.3221050203e-02
       coeff_l4(110,20)=  4.8119749874e-02
       coeff_l4(110,21)= -1.7192219198e-01
       coeff_l4(110,22)=  3.0270320177e-01
       coeff_l4(110,23)= -6.8962313235e-02
       coeff_l4(110,24)=  1.9597059488e-01
       coeff_l4(110,25)= -1.9024867564e-02
       coeff_l4(110,26)= -9.8965667188e-02
       coeff_l4(110,27)=  5.5627524853e-03
       coeff_l4(110,28)= -9.3730203807e-02
       coeff_l4(110,29)=  9.5709770918e-02
       coeff_l4(110,30)= -4.1798904538e-02
       coeff_l4(110,31)=  7.2899684310e-02
       coeff_l4(110,32)= -1.1847125739e-01
       coeff_l4(110,33)=  8.3808138967e-02
       coeff_l4(110,34)=  8.2888685167e-02
       coeff_l4(110,35)= -7.0762991905e-02
       coeff_l4(110,36)= -1.7985482514e-01
       coeff_l4(110,37)=  1.8284603953e-02
       coeff_l4(110,38)=  1.7856112123e-01
       coeff_l4(110,39)= -1.2054260820e-01
       coeff_l4(110,40)= -6.4489364624e-02
       coeff_l4(110,41)=  1.7272222042e-01
       coeff_l4(110,42)=  2.2475701571e-01
       coeff_l4(110,43)=  1.3356147707e-01
       coeff_l4(110,44)=  1.2757441401e-01
       coeff_l4(110,45)= -8.8695324957e-02
       coeff_l4(110,46)=  1.0200402886e-01
       coeff_l4(110,47)= -1.2744635344e-01
       coeff_l4(110,48)=  1.2027440965e-01
       coeff_l4(110,49)= -2.2614178061e-01
       coeff_l4(110,50)= -1.0279063135e-01
       coeff_l4(110,51)=  9.9755860865e-02
       coeff_l4(110,52)= -1.9020815194e-01
       coeff_l4(110,53)=  3.6993262172e-01
       coeff_l4(110,54)=  1.5809553862e-01
       coeff_l4(110,55)= -1.1891831458e-01
       coeff_l4(110,56)=  6.9534189999e-02
       coeff_l4(110,57)=  5.5456329137e-02
       coeff_l4(110,58)=  9.6919551492e-02
       coeff_l4(110,59)=  6.0718555003e-02
       coeff_l4(110,60)= -3.2043319196e-02
       coeff_l4(110,61)=  2.8299292549e-02
       coeff_l4(110,62)= -2.1408338845e-01
       coeff_l4(110,63)=  1.2284818292e-01
       coeff_l4(110,64)= -1.0862082988e-01
       coeff_l4(110,65)=  3.5590875894e-02
       coeff_l4(110,66)= -4.3579198420e-02
       coeff_l4(110,67)= -2.5183321908e-02
       coeff_l4(110,68)=  2.3516757786e-01
       coeff_l4(110,69)=  2.5209680200e-01
       coeff_l4(110,70)=  2.2888347507e-01
       coeff_l4(110,71)=  7.0209845901e-02
       coeff_l4(110,72)=  4.9377892166e-02
       coeff_l4(110,73)=  4.1544832289e-02
       coeff_l4(110,74)=  2.4250972271e-01
       coeff_l4(110,75)=  5.2144024521e-02
       coeff_l4(110,76)=  7.1576990187e-02
       coeff_l4(110,77)= -4.3391339481e-02
       coeff_l4(110,78)= -1.8270160258e-01
       coeff_l4(110,79)=  1.1108691990e-01
       coeff_l4(110,80)= -1.8424052000e-01
       coeff_l4(110,81)= -5.1309574395e-02
       coeff_l4(110,82)= -1.6281731427e-01
       coeff_l4(110,83)= -1.0949810594e-01
       coeff_l4(110,84)= -2.7007862926e-02
       coeff_l4(110,85)=  8.7827295065e-02
       coeff_l4(110,86)= -1.0577491671e-01
       coeff_l4(110,87)=  1.0019635782e-02
       coeff_l4(110,88)=  1.1999368668e-01
       coeff_l4(110,89)=  3.5929415375e-02
       coeff_l4(110,90)=  1.2162096798e-01
       coeff_l4(110,91)=  1.5675593913e-01
       coeff_l4(110,92)=  2.4971555918e-02
       coeff_l4(110,93)=  1.2932434678e-01
       coeff_l4(110,94)=  1.9354464114e-01
       coeff_l4(110,95)=  1.3923655450e-01
       coeff_l4(110,96)=  1.2132943422e-01
       coeff_l4(110,97)= -1.7722180486e-01
       coeff_l4(110,98)=  7.5271315873e-02
       coeff_l4(110,99)=  2.2211962938e-01
       coeff_l4(110,100)= -1.5494425781e-02
       coeff_l4(110,101)= -1.8893189728e-01
       coeff_l4(110,102)=  8.2904458046e-02
       coeff_l4(110,103)=  1.4244528115e-01
       coeff_l4(110,104)=  1.8251372874e-01
       coeff_l4(110,105)= -1.8046995997e-01
       coeff_l4(111,1)= -2.3171560839e-02
       coeff_l4(111,2)= -7.7495835721e-03
       coeff_l4(111,3)=  1.5400777757e-01
       coeff_l4(111,4)=  1.5573307872e-01
       coeff_l4(111,5)= -1.1535969377e-01
       coeff_l4(111,6)= -7.6288878918e-03
       coeff_l4(111,7)=  3.2367554307e-01
       coeff_l4(111,8)= -7.0185787976e-02
       coeff_l4(111,9)= -1.2684135139e-01
       coeff_l4(111,10)=  2.5615155697e-01
       coeff_l4(111,11)= -3.0568921566e-01
       coeff_l4(111,12)=  1.3674242795e-01
       coeff_l4(111,13)= -1.3359676301e-01
       coeff_l4(111,14)=  2.3093988001e-01
       coeff_l4(111,15)=  2.4878552556e-01
       coeff_l4(111,16)= -6.2198411673e-02
       coeff_l4(111,17)=  5.9366829693e-02
       coeff_l4(111,18)= -1.5311349928e-01
       coeff_l4(111,19)= -2.9011669755e-01
       coeff_l4(111,20)=  1.2579992414e-01
       coeff_l4(111,21)= -2.2359922528e-01
       coeff_l4(111,22)= -5.3600173444e-02
       coeff_l4(111,23)=  3.6811619997e-01
       coeff_l4(111,24)= -1.4882808924e-01
       coeff_l4(111,25)=  5.2060622722e-02
       coeff_l4(111,26)= -2.2168716788e-01
       coeff_l4(111,27)=  8.7389238179e-02
       coeff_l4(111,28)=  5.0765015185e-02
       coeff_l4(111,29)=  1.3328534365e-01
       coeff_l4(111,30)=  5.4015789181e-02
       coeff_l4(111,31)=  3.0515234917e-02
       coeff_l4(111,32)= -9.6499674022e-02
       coeff_l4(111,33)=  1.8770588934e-01
       coeff_l4(111,34)=  3.6326363683e-01
       coeff_l4(111,35)=  1.7434021831e-01
       coeff_l4(111,36)= -1.9744360447e-01
       coeff_l4(111,37)= -2.1556515992e-01
       coeff_l4(111,38)= -3.1701061130e-01
       coeff_l4(111,39)=  8.2010075450e-02
       coeff_l4(111,40)= -1.8124590814e-01
       coeff_l4(111,41)=  1.8248254061e-01
       coeff_l4(111,42)=  9.0636067092e-02
       coeff_l4(111,43)= -2.1942728758e-01
       coeff_l4(111,44)= -1.1970966309e-01
       coeff_l4(111,45)=  1.0902007110e-02
       coeff_l4(111,46)=  1.4202697575e-01
       coeff_l4(111,47)= -8.0682709813e-03
       coeff_l4(111,48)=  2.5977373123e-01
       coeff_l4(111,49)=  9.0621694922e-02
       coeff_l4(111,50)= -2.2256191075e-01
       coeff_l4(111,51)=  1.0856702179e-01
       coeff_l4(111,52)=  1.1005751789e-02
       coeff_l4(111,53)=  2.4813842773e-01
       coeff_l4(111,54)=  6.8293482065e-02
       coeff_l4(111,55)= -1.2003585696e-01
       coeff_l4(111,56)=  1.3136607409e-01
       coeff_l4(111,57)= -1.9408406317e-01
       coeff_l4(111,58)= -1.1783950031e-01
       coeff_l4(111,59)= -1.1121097952e-01
       coeff_l4(111,60)=  7.2030395269e-02
       coeff_l4(111,61)= -4.2416724563e-01
       coeff_l4(111,62)= -9.2129766941e-02
       coeff_l4(111,63)= -4.0179047734e-02
       coeff_l4(111,64)= -5.0826035440e-02
       coeff_l4(111,65)= -2.9122796655e-01
       coeff_l4(111,66)= -1.5820123255e-01
       coeff_l4(111,67)=  2.7121260762e-01
       coeff_l4(111,68)= -2.4565946311e-02
       coeff_l4(111,69)=  2.2640483454e-02
       coeff_l4(111,70)= -7.7975437045e-02
       coeff_l4(111,71)= -1.2146969140e-01
       coeff_l4(111,72)=  3.8823571801e-01
       coeff_l4(111,73)=  2.5839176774e-01
       coeff_l4(111,74)=  4.6968376637e-01
       coeff_l4(111,75)= -1.1790806800e-01
       coeff_l4(111,76)=  1.1141212657e-02
       coeff_l4(111,77)=  3.8708651066e-01
       coeff_l4(111,78)= -3.8920360804e-01
       coeff_l4(111,79)=  7.9239286482e-02
       coeff_l4(111,80)= -3.4002035856e-02
       coeff_l4(111,81)= -2.2365581989e-01
       coeff_l4(111,82)=  1.8290220201e-01
       coeff_l4(111,83)=  1.2494590878e-01
       coeff_l4(111,84)= -3.8116660714e-01
       coeff_l4(111,85)= -6.0935342312e-01
       coeff_l4(111,86)= -8.7717518210e-02
       coeff_l4(111,87)=  1.6120085493e-02
       coeff_l4(111,88)= -4.5915074646e-02
       coeff_l4(111,89)= -3.8804192096e-02
       coeff_l4(111,90)= -1.6876776516e-01
       coeff_l4(111,91)= -4.4490639120e-02
       coeff_l4(111,92)=  8.7571755052e-02
       coeff_l4(111,93)=  2.0521370694e-02
       coeff_l4(111,94)=  4.1642490029e-01
       coeff_l4(111,95)= -3.5673916340e-02
       coeff_l4(111,96)= -2.0890882239e-02
       coeff_l4(111,97)=  7.5861781836e-02
       coeff_l4(111,98)=  9.2593885958e-02
       coeff_l4(111,99)= -2.9571390152e-01
       coeff_l4(111,100)=  3.8290783763e-01
       coeff_l4(111,101)=  1.0779115558e-01
       coeff_l4(111,102)= -1.9508950412e-01
       coeff_l4(111,103)=  4.2419540882e-01
       coeff_l4(111,104)=  1.0526197404e-01
       coeff_l4(111,105)=  3.0484849215e-01
       coeff_l4(112,1)=  1.1603616476e+00
       coeff_l4(112,2)=  4.2416217923e-01
       coeff_l4(112,3)=  1.1534435749e+00
       coeff_l4(112,4)=  8.3026582003e-01
       coeff_l4(112,5)=  1.1340601444e+00
       coeff_l4(112,6)=  9.2385077477e-01
       coeff_l4(112,7)=  1.8388733864e+00
       coeff_l4(112,8)=  1.7478020191e+00
       coeff_l4(112,9)=  8.1393533945e-01
       coeff_l4(112,10)=  8.3668959141e-01
       coeff_l4(112,11)=  1.0072172880e+00
       coeff_l4(112,12)=  1.0487859249e+00
       coeff_l4(112,13)=  7.5199401379e-01
       coeff_l4(112,14)=  8.5452109575e-01
       coeff_l4(112,15)= -3.0431115627e-01
       coeff_l4(112,16)=  3.7158036232e-01
       coeff_l4(112,17)= -1.0009460151e-01
       coeff_l4(112,18)=  4.3930605054e-01
       coeff_l4(112,19)=  3.4370085597e-01
       coeff_l4(112,20)=  3.9867978543e-02
       coeff_l4(112,21)=  8.2427293062e-01
       coeff_l4(112,22)= -3.0095207691e-01
       coeff_l4(112,23)= -1.0928332061e-01
       coeff_l4(112,24)=  7.8557297587e-02
       coeff_l4(112,25)=  1.6660892963e-01
       coeff_l4(112,26)=  2.0937186480e-01
       coeff_l4(112,27)= -5.3622347116e-01
       coeff_l4(112,28)=  3.6449965835e-01
       coeff_l4(112,29)=  4.1157567501e-01
       coeff_l4(112,30)= -3.5542857647e-01
       coeff_l4(112,31)= -2.9386830330e-01
       coeff_l4(112,32)=  1.0202404857e-01
       coeff_l4(112,33)= -4.0644273162e-01
       coeff_l4(112,34)= -4.2554989457e-02
       coeff_l4(112,35)= -2.6545631886e-01
       coeff_l4(112,36)=  5.9915147722e-02
       coeff_l4(112,37)= -1.2339405715e-01
       coeff_l4(112,38)=  2.7989798784e-01
       coeff_l4(112,39)=  3.1343102455e-01
       coeff_l4(112,40)= -9.6591852605e-02
       coeff_l4(112,41)= -5.4241561890e-01
       coeff_l4(112,42)= -8.3824738860e-02
       coeff_l4(112,43)=  1.4872725308e-01
       coeff_l4(112,44)=  1.8162137270e-01
       coeff_l4(112,45)=  2.0991887152e-01
       coeff_l4(112,46)=  2.4065631628e-01
       coeff_l4(112,47)=  4.7130286694e-02
       coeff_l4(112,48)=  8.2002662122e-02
       coeff_l4(112,49)=  1.1359216645e-02
       coeff_l4(112,50)=  4.1623631120e-01
       coeff_l4(112,51)= -1.5114541352e-01
       coeff_l4(112,52)= -2.7271371335e-02
       coeff_l4(112,53)=  1.7856247723e-01
       coeff_l4(112,54)= -1.8135787919e-02
       coeff_l4(112,55)=  2.6484921575e-01
       coeff_l4(112,56)= -2.8455460072e-01
       coeff_l4(112,57)=  3.3847592771e-02
       coeff_l4(112,58)= -1.1219524592e-01
       coeff_l4(112,59)=  4.1680401564e-01
       coeff_l4(112,60)= -5.1768943667e-02
       coeff_l4(112,61)= -3.9240169525e-01
       coeff_l4(112,62)=  1.9921836257e-01
       coeff_l4(112,63)=  4.2209500074e-01
       coeff_l4(112,64)= -1.5463958681e-01
       coeff_l4(112,65)=  3.1217187643e-01
       coeff_l4(112,66)=  3.3254760504e-01
       coeff_l4(112,67)= -4.9517411739e-02
       coeff_l4(112,68)= -4.9559181929e-01
       coeff_l4(112,69)= -4.6616679430e-01
       coeff_l4(112,70)= -5.1544293761e-02
       coeff_l4(112,71)= -2.4337448180e-01
       coeff_l4(112,72)=  1.3603793085e-01
       coeff_l4(112,73)= -3.5380479693e-01
       coeff_l4(112,74)=  1.2855578959e-01
       coeff_l4(112,75)= -1.1071314663e-01
       coeff_l4(112,76)=  1.8280981481e-01
       coeff_l4(112,77)=  1.2790299952e-01
       coeff_l4(112,78)= -1.1383344233e-01
       coeff_l4(112,79)=  5.3510092199e-02
       coeff_l4(112,80)= -3.2547779381e-02
       coeff_l4(112,81)= -2.0426382124e-01
       coeff_l4(112,82)= -3.4695467353e-01
       coeff_l4(112,83)= -5.0193768740e-01
       coeff_l4(112,84)= -7.6178282499e-02
       coeff_l4(112,85)= -2.4835930765e-01
       coeff_l4(112,86)=  1.2213382125e-01
       coeff_l4(112,87)=  1.3054679334e-01
       coeff_l4(112,88)=  2.2925561294e-02
       coeff_l4(112,89)=  1.5274052322e-01
       coeff_l4(112,90)=  4.8881571740e-02
       coeff_l4(112,91)= -5.9554231167e-01
       coeff_l4(112,92)=  2.4264824390e-01
       coeff_l4(112,93)=  1.7020235956e-01
       coeff_l4(112,94)= -1.2105246633e-01
       coeff_l4(112,95)=  5.6090605259e-01
       coeff_l4(112,96)=  4.2038789392e-01
       coeff_l4(112,97)= -1.5752948821e-02
       coeff_l4(112,98)=  1.6418090463e-01
       coeff_l4(112,99)=  2.5716710091e-01
       coeff_l4(112,100)=  4.7424558550e-02
       coeff_l4(112,101)=  4.3387770653e-01
       coeff_l4(112,102)=  6.3973551989e-01
       coeff_l4(112,103)=  5.4925840348e-02
       coeff_l4(112,104)= -1.4948399365e-01
       coeff_l4(112,105)=  6.0107833147e-01
       coeff_l4(113,1)= -5.9484755993e-01
       coeff_l4(113,2)= -4.5815566182e-01
       coeff_l4(113,3)= -3.4912756085e-01
       coeff_l4(113,4)= -9.7960555553e-01
       coeff_l4(113,5)= -3.4906197339e-02
       coeff_l4(113,6)= -2.1751374006e-01
       coeff_l4(113,7)= -6.4400744438e-01
       coeff_l4(113,8)= -3.6145740747e-01
       coeff_l4(113,9)=  1.3901516795e-01
       coeff_l4(113,10)=  7.1088790894e-01
       coeff_l4(113,11)=  9.0919888020e-01
       coeff_l4(113,12)=  4.4050446153e-01
       coeff_l4(113,13)= -2.7301910520e-01
       coeff_l4(113,14)=  1.1426627636e+00
       coeff_l4(113,15)=  2.7887737751e-01
       coeff_l4(113,16)= -7.3062932491e-01
       coeff_l4(113,17)=  2.5920274854e-01
       coeff_l4(113,18)= -1.8457573652e-01
       coeff_l4(113,19)=  4.2159238458e-01
       coeff_l4(113,20)=  5.9763032198e-01
       coeff_l4(113,21)= -2.9531818628e-01
       coeff_l4(113,22)=  3.3075433969e-01
       coeff_l4(113,23)= -4.5682084560e-01
       coeff_l4(113,24)=  3.2657590508e-01
       coeff_l4(113,25)= -2.9986941814e-01
       coeff_l4(113,26)=  2.3813626170e-01
       coeff_l4(113,27)= -9.5184899867e-02
       coeff_l4(113,28)=  1.7327690125e-01
       coeff_l4(113,29)= -3.4706011415e-01
       coeff_l4(113,30)=  3.2426497340e-01
       coeff_l4(113,31)= -5.6821882725e-01
       coeff_l4(113,32)=  9.7880798578e-01
       coeff_l4(113,33)= -1.3597613573e-01
       coeff_l4(113,34)= -4.5868197083e-01
       coeff_l4(113,35)=  4.0994644165e-01
       coeff_l4(113,36)=  2.7662417293e-01
       coeff_l4(113,37)=  8.4628555924e-03
       coeff_l4(113,38)=  1.4409293234e-01
       coeff_l4(113,39)= -1.3865582645e-01
       coeff_l4(113,40)= -1.1769470572e-01
       coeff_l4(113,41)=  1.7021195590e-01
       coeff_l4(113,42)=  8.8106137514e-01
       coeff_l4(113,43)=  3.6146694422e-01
       coeff_l4(113,44)=  1.3888119161e-01
       coeff_l4(113,45)= -5.4130756855e-01
       coeff_l4(113,46)= -4.3965843320e-01
       coeff_l4(113,47)= -3.5748377442e-01
       coeff_l4(113,48)=  3.7569709122e-02
       coeff_l4(113,49)=  5.4861438274e-01
       coeff_l4(113,50)= -5.4485201836e-02
       coeff_l4(113,51)=  9.2101180553e-01
       coeff_l4(113,52)= -4.6894788742e-01
       coeff_l4(113,53)=  5.4790359735e-01
       coeff_l4(113,54)=  7.5376898050e-02
       coeff_l4(113,55)= -1.3346199691e-01
       coeff_l4(113,56)= -3.2500615716e-01
       coeff_l4(113,57)=  6.7186072469e-02
       coeff_l4(113,58)= -7.9849988222e-02
       coeff_l4(113,59)=  3.6643229425e-02
       coeff_l4(113,60)= -1.6895301640e-01
       coeff_l4(113,61)=  8.8481806219e-02
       coeff_l4(113,62)= -4.5960139483e-02
       coeff_l4(113,63)=  7.6440647244e-02
       coeff_l4(113,64)=  1.0902335644e+00
       coeff_l4(113,65)=  1.0083095729e-01
       coeff_l4(113,66)=  3.2379832864e-01
       coeff_l4(113,67)= -2.9738569260e-01
       coeff_l4(113,68)= -3.8728219271e-01
       coeff_l4(113,69)= -6.5400987864e-02
       coeff_l4(113,70)=  4.4249381870e-02
       coeff_l4(113,71)=  5.1207286119e-01
       coeff_l4(113,72)=  6.5780299902e-01
       coeff_l4(113,73)=  4.2136999965e-01
       coeff_l4(113,74)= -3.5043485463e-02
       coeff_l4(113,75)=  9.1846629977e-02
       coeff_l4(113,76)= -3.6060538888e-01
       coeff_l4(113,77)= -1.3268776238e-01
       coeff_l4(113,78)=  3.0052602291e-01
       coeff_l4(113,79)= -8.4542267025e-02
       coeff_l4(113,80)= -6.9889284670e-02
       coeff_l4(113,81)=  3.1134110689e-01
       coeff_l4(113,82)=  4.8349049687e-01
       coeff_l4(113,83)= -3.2134652138e-01
       coeff_l4(113,84)= -3.6173316836e-01
       coeff_l4(113,85)=  6.4293766022e-01
       coeff_l4(113,86)= -8.1183922291e-01
       coeff_l4(113,87)=  1.6619172692e-01
       coeff_l4(113,88)= -3.6230701208e-01
       coeff_l4(113,89)= -1.8538621068e-01
       coeff_l4(113,90)= -9.8275035620e-01
       coeff_l4(113,91)=  4.0876787901e-01
       coeff_l4(113,92)= -4.7143372893e-01
       coeff_l4(113,93)= -1.5874628443e-03
       coeff_l4(113,94)=  3.5588341951e-01
       coeff_l4(113,95)= -9.1530889273e-01
       coeff_l4(113,96)= -8.2755379379e-02
       coeff_l4(113,97)=  2.9100894928e-01
       coeff_l4(113,98)= -1.7386819422e-01
       coeff_l4(113,99)= -2.6965186000e-01
       coeff_l4(113,100)= -3.9703834057e-01
       coeff_l4(113,101)=  3.7973690033e-01
       coeff_l4(113,102)= -5.2692413330e-01
       coeff_l4(113,103)= -5.7170981169e-01
       coeff_l4(113,104)=  6.4225025475e-02
       coeff_l4(113,105)= -8.5043884814e-02
       coeff_l4(114,1)= -9.2420864105e-01
       coeff_l4(114,2)= -8.4310799837e-01
       coeff_l4(114,3)= -6.7190557718e-01
       coeff_l4(114,4)= -6.0888904333e-01
       coeff_l4(114,5)= -6.9278091192e-01
       coeff_l4(114,6)= -7.0748484135e-01
       coeff_l4(114,7)= -5.7449102402e-01
       coeff_l4(114,8)= -5.1478964090e-01
       coeff_l4(114,9)= -9.7039502859e-01
       coeff_l4(114,10)= -1.0184555054e+00
       coeff_l4(114,11)= -1.6125206947e+00
       coeff_l4(114,12)= -1.7302756310e+00
       coeff_l4(114,13)= -1.6328287125e+00
       coeff_l4(114,14)= -3.8051629066e-01
       coeff_l4(114,15)=  5.4725777358e-02
       coeff_l4(114,16)= -1.2043832242e-01
       coeff_l4(114,17)=  2.0528878272e-01
       coeff_l4(114,18)=  1.4559766650e-01
       coeff_l4(114,19)=  1.1290679872e-01
       coeff_l4(114,20)=  2.7979305014e-02
       coeff_l4(114,21)= -6.5534189343e-02
       coeff_l4(114,22)= -8.9317768812e-02
       coeff_l4(114,23)= -1.6479679942e-01
       coeff_l4(114,24)= -9.3001134694e-02
       coeff_l4(114,25)= -1.8433995545e-02
       coeff_l4(114,26)=  2.4707233533e-02
       coeff_l4(114,27)= -1.4841939509e-01
       coeff_l4(114,28)=  2.3390730843e-02
       coeff_l4(114,29)= -2.2156776860e-02
       coeff_l4(114,30)=  1.2022939324e-01
       coeff_l4(114,31)= -1.3025143743e-01
       coeff_l4(114,32)= -1.3828122616e-01
       coeff_l4(114,33)=  5.9406463057e-02
       coeff_l4(114,34)= -1.2266062200e-01
       coeff_l4(114,35)=  9.8945096135e-02
       coeff_l4(114,36)= -2.3676313460e-02
       coeff_l4(114,37)=  1.4568728209e-01
       coeff_l4(114,38)= -1.4460861683e-01
       coeff_l4(114,39)= -5.0211247057e-02
       coeff_l4(114,40)= -1.0892285407e-01
       coeff_l4(114,41)=  2.1289750934e-01
       coeff_l4(114,42)=  1.3056616485e-01
       coeff_l4(114,43)=  2.6414703578e-02
       coeff_l4(114,44)= -3.9725478739e-02
       coeff_l4(114,45)= -1.2462617457e-01
       coeff_l4(114,46)= -1.7180928588e-01
       coeff_l4(114,47)= -1.4101487398e-01
       coeff_l4(114,48)= -3.5423394293e-02
       coeff_l4(114,49)= -2.1534308791e-01
       coeff_l4(114,50)=  1.2445785850e-01
       coeff_l4(114,51)=  4.2388003320e-02
       coeff_l4(114,52)= -2.7911689878e-01
       coeff_l4(114,53)= -6.0807794333e-02
       coeff_l4(114,54)= -1.2055445462e-01
       coeff_l4(114,55)=  1.5326511860e-01
       coeff_l4(114,56)= -3.5116743296e-02
       coeff_l4(114,57)=  7.4810981750e-02
       coeff_l4(114,58)=  1.0672818124e-01
       coeff_l4(114,59)=  1.4745928347e-01
       coeff_l4(114,60)=  6.6070631146e-02
       coeff_l4(114,61)= -7.0548206568e-02
       coeff_l4(114,62)= -1.2092217803e-01
       coeff_l4(114,63)=  6.7078933120e-02
       coeff_l4(114,64)=  6.0310166329e-02
       coeff_l4(114,65)=  1.3605679572e-01
       coeff_l4(114,66)= -2.0397835970e-01
       coeff_l4(114,67)= -2.8595745564e-02
       coeff_l4(114,68)=  1.7117129266e-01
       coeff_l4(114,69)=  2.0736299455e-02
       coeff_l4(114,70)=  5.9299558401e-02
       coeff_l4(114,71)=  2.2536674142e-01
       coeff_l4(114,72)= -8.7335072458e-02
       coeff_l4(114,73)=  9.6202000976e-02
       coeff_l4(114,74)= -3.8318369538e-02
       coeff_l4(114,75)= -6.2636785209e-02
       coeff_l4(114,76)=  1.1599609256e-01
       coeff_l4(114,77)= -2.9134762287e-01
       coeff_l4(114,78)=  9.9249176681e-02
       coeff_l4(114,79)= -3.4588340670e-02
       coeff_l4(114,80)= -9.0458570048e-03
       coeff_l4(114,81)= -2.1120925248e-01
       coeff_l4(114,82)=  1.7075109482e-01
       coeff_l4(114,83)=  1.9112877548e-01
       coeff_l4(114,84)=  1.5186224878e-01
       coeff_l4(114,85)=  2.3015777767e-01
       coeff_l4(114,86)=  1.3315268792e-02
       coeff_l4(114,87)= -1.0972998291e-01
       coeff_l4(114,88)= -4.9285326153e-02
       coeff_l4(114,89)= -1.2098215520e-01
       coeff_l4(114,90)=  1.8261709809e-01
       coeff_l4(114,91)=  2.4152874947e-01
       coeff_l4(114,92)=  2.9439359903e-01
       coeff_l4(114,93)= -4.1145101190e-01
       coeff_l4(114,94)= -9.3087947369e-01
       coeff_l4(114,95)= -2.1000535786e-01
       coeff_l4(114,96)= -5.4292434454e-01
       coeff_l4(114,97)= -5.4819703102e-01
       coeff_l4(114,98)= -1.0154610872e-01
       coeff_l4(114,99)=  3.1840249896e-01
       coeff_l4(114,100)=  3.7482187152e-01
       coeff_l4(114,101)= -2.6128345728e-01
       coeff_l4(114,102)= -7.3340550065e-02
       coeff_l4(114,103)=  1.0300611705e-01
       coeff_l4(114,104)=  2.0336538553e-01
       coeff_l4(114,105)=  1.2225974351e-01
       coeff_l4(115,1)=  2.5252957344e+00
       coeff_l4(115,2)=  1.4507801533e+00
       coeff_l4(115,3)=  5.7262635231e-01
       coeff_l4(115,4)=  1.1301367283e+00
       coeff_l4(115,5)=  1.3575786352e+00
       coeff_l4(115,6)=  8.2244545221e-01
       coeff_l4(115,7)=  2.2371219099e-01
       coeff_l4(115,8)=  3.1100338697e-01
       coeff_l4(115,9)=  3.2085523009e-01
       coeff_l4(115,10)=  5.0157278776e-01
       coeff_l4(115,11)=  6.9562721252e-01
       coeff_l4(115,12)= -1.7762123048e-01
       coeff_l4(115,13)=  2.7134034038e-01
       coeff_l4(115,14)=  1.1986687034e-01
       coeff_l4(115,15)= -3.0032744631e-02
       coeff_l4(115,16)=  1.7865084112e-01
       coeff_l4(115,17)=  1.3812488317e-01
       coeff_l4(115,18)=  1.2727379799e-01
       coeff_l4(115,19)=  3.7803605199e-01
       coeff_l4(115,20)= -4.2366787791e-01
       coeff_l4(115,21)=  6.8011772633e-01
       coeff_l4(115,22)=  3.6201852560e-01
       coeff_l4(115,23)= -2.2202250361e-01
       coeff_l4(115,24)=  3.8840088248e-01
       coeff_l4(115,25)= -2.0306488872e-01
       coeff_l4(115,26)=  5.0855362415e-01
       coeff_l4(115,27)= -3.1436240673e-01
       coeff_l4(115,28)= -2.0316176116e-01
       coeff_l4(115,29)=  6.3127107918e-02
       coeff_l4(115,30)=  4.4698961079e-02
       coeff_l4(115,31)=  4.2284303904e-01
       coeff_l4(115,32)=  2.6859059930e-01
       coeff_l4(115,33)= -2.7018937469e-01
       coeff_l4(115,34)= -3.4342414141e-01
       coeff_l4(115,35)=  1.5224686265e-01
       coeff_l4(115,36)= -1.4230181277e-01
       coeff_l4(115,37)= -1.1119350046e-01
       coeff_l4(115,38)=  2.0141826570e-01
       coeff_l4(115,39)=  1.8621031940e-01
       coeff_l4(115,40)= -4.6984377503e-01
       coeff_l4(115,41)= -3.7695731968e-02
       coeff_l4(115,42)=  3.7295842171e-01
       coeff_l4(115,43)= -1.4085428417e-01
       coeff_l4(115,44)=  6.0031461716e-01
       coeff_l4(115,45)= -5.6988537312e-01
       coeff_l4(115,46)= -7.5735241175e-02
       coeff_l4(115,47)=  2.1018050611e-01
       coeff_l4(115,48)=  6.0417073965e-01
       coeff_l4(115,49)= -6.0807240009e-01
       coeff_l4(115,50)= -1.3915601373e-01
       coeff_l4(115,51)=  8.2168065012e-02
       coeff_l4(115,52)= -2.0620198548e-01
       coeff_l4(115,53)= -1.7758971453e-01
       coeff_l4(115,54)= -4.9920897931e-02
       coeff_l4(115,55)=  6.1501067877e-01
       coeff_l4(115,56)= -2.3191426694e-01
       coeff_l4(115,57)= -2.1746456623e-01
       coeff_l4(115,58)= -1.6333289444e-01
       coeff_l4(115,59)=  3.8027274609e-01
       coeff_l4(115,60)= -4.8677709699e-01
       coeff_l4(115,61)= -1.5904435515e-01
       coeff_l4(115,62)=  1.7643898726e-01
       coeff_l4(115,63)=  4.1259744763e-01
       coeff_l4(115,64)= -2.8163233399e-01
       coeff_l4(115,65)= -5.0345800817e-02
       coeff_l4(115,66)=  5.8796513081e-01
       coeff_l4(115,67)=  6.9479838014e-02
       coeff_l4(115,68)=  2.6461017132e-01
       coeff_l4(115,69)=  4.0440264344e-01
       coeff_l4(115,70)=  2.8189411759e-01
       coeff_l4(115,71)= -1.3448554277e-01
       coeff_l4(115,72)=  8.6760528386e-02
       coeff_l4(115,73)=  2.5560316443e-01
       coeff_l4(115,74)=  2.1014529467e-01
       coeff_l4(115,75)=  2.1741257980e-02
       coeff_l4(115,76)=  1.7770215869e-01
       coeff_l4(115,77)=  3.4838718176e-01
       coeff_l4(115,78)= -8.0126725137e-02
       coeff_l4(115,79)=  4.4943679124e-02
       coeff_l4(115,80)=  4.5715850592e-01
       coeff_l4(115,81)=  5.9279795736e-02
       coeff_l4(115,82)= -2.2939550690e-03
       coeff_l4(115,83)= -4.5004463196e-01
       coeff_l4(115,84)=  8.2132622600e-02
       coeff_l4(115,85)=  8.8018335402e-02
       coeff_l4(115,86)= -1.3655330241e-01
       coeff_l4(115,87)=  1.3217465580e-01
       coeff_l4(115,88)=  1.5133722126e-01
       coeff_l4(115,89)=  1.7102304846e-02
       coeff_l4(115,90)=  1.9615550339e-01
       coeff_l4(115,91)=  1.6323676333e-02
       coeff_l4(115,92)=  2.9094639421e-01
       coeff_l4(115,93)= -3.9021231234e-02
       coeff_l4(115,94)=  3.6534345150e-01
       coeff_l4(115,95)= -1.1397551745e-01
       coeff_l4(115,96)= -4.4659450650e-01
       coeff_l4(115,97)=  5.8861806989e-02
       coeff_l4(115,98)= -5.7735061273e-03
       coeff_l4(115,99)=  3.4803394228e-02
       coeff_l4(115,100)= -7.6159752905e-02
       coeff_l4(115,101)= -1.5852980316e-01
       coeff_l4(115,102)=  3.6513549089e-01
       coeff_l4(115,103)= -2.9565387964e-01
       coeff_l4(115,104)=  2.6079168916e-01
       coeff_l4(115,105)= -4.1805210710e-01
       coeff_l4(116,1)=  1.9556730986e+00
       coeff_l4(116,2)=  1.6356465816e+00
       coeff_l4(116,3)=  1.3388932943e+00
       coeff_l4(116,4)=  1.7086272240e+00
       coeff_l4(116,5)=  1.3533631563e+00
       coeff_l4(116,6)=  1.7958937883e+00
       coeff_l4(116,7)=  1.6147217751e+00
       coeff_l4(116,8)=  1.7898654938e+00
       coeff_l4(116,9)=  1.4885684848e-01
       coeff_l4(116,10)= -3.7409506738e-02
       coeff_l4(116,11)=  1.3241851330e+00
       coeff_l4(116,12)=  9.8588764668e-01
       coeff_l4(116,13)=  1.1134909391e+00
       coeff_l4(116,14)= -2.9856750369e-01
       coeff_l4(116,15)=  2.6123070717e-01
       coeff_l4(116,16)= -6.5630033612e-02
       coeff_l4(116,17)=  1.4311715961e-02
       coeff_l4(116,18)=  6.9602453709e-01
       coeff_l4(116,19)= -2.2500361502e-01
       coeff_l4(116,20)=  1.8058110774e-01
       coeff_l4(116,21)=  2.1711027622e-01
       coeff_l4(116,22)= -3.9186128974e-01
       coeff_l4(116,23)=  5.0853717327e-01
       coeff_l4(116,24)= -6.5390512347e-02
       coeff_l4(116,25)=  2.4605226517e-01
       coeff_l4(116,26)=  2.3460341990e-01
       coeff_l4(116,27)= -2.9203307629e-01
       coeff_l4(116,28)=  3.5650271177e-01
       coeff_l4(116,29)= -2.7163842320e-01
       coeff_l4(116,30)=  3.4576116595e-03
       coeff_l4(116,31)=  4.0860536695e-01
       coeff_l4(116,32)= -1.3759638369e-01
       coeff_l4(116,33)= -1.4403776266e-02
       coeff_l4(116,34)= -5.3517647088e-02
       coeff_l4(116,35)= -5.7994954288e-02
       coeff_l4(116,36)= -2.0513544977e-01
       coeff_l4(116,37)=  1.1211456358e-01
       coeff_l4(116,38)=  7.2387196124e-02
       coeff_l4(116,39)=  5.7450383902e-01
       coeff_l4(116,40)=  4.3987289071e-01
       coeff_l4(116,41)= -4.6834877133e-01
       coeff_l4(116,42)= -2.4944083765e-02
       coeff_l4(116,43)=  6.5967047215e-01
       coeff_l4(116,44)=  7.8532636166e-02
       coeff_l4(116,45)=  1.1057373285e+00
       coeff_l4(116,46)=  7.2061997652e-01
       coeff_l4(116,47)=  7.6460373402e-01
       coeff_l4(116,48)= -2.9525479302e-02
       coeff_l4(116,49)=  1.9130040705e-01
       coeff_l4(116,50)=  4.5886301994e-01
       coeff_l4(116,51)= -5.4648286104e-01
       coeff_l4(116,52)=  4.3655633926e-01
       coeff_l4(116,53)= -4.9727645516e-01
       coeff_l4(116,54)= -4.9579742551e-01
       coeff_l4(116,55)= -1.4718818665e-01
       coeff_l4(116,56)=  1.6227918863e-01
       coeff_l4(116,57)= -7.2130881250e-02
       coeff_l4(116,58)=  5.3614246845e-01
       coeff_l4(116,59)= -1.4052888379e-02
       coeff_l4(116,60)= -8.9534677565e-02
       coeff_l4(116,61)=  4.0971961617e-01
       coeff_l4(116,62)=  6.1042565107e-01
       coeff_l4(116,63)= -4.2280355096e-01
       coeff_l4(116,64)= -8.8850820065e-01
       coeff_l4(116,65)=  3.8207417727e-01
       coeff_l4(116,66)=  3.5972860456e-01
       coeff_l4(116,67)= -1.2471298687e-02
       coeff_l4(116,68)= -1.9005404413e-01
       coeff_l4(116,69)=  6.3306540251e-01
       coeff_l4(116,70)=  4.2555250227e-02
       coeff_l4(116,71)= -1.5419788659e-01
       coeff_l4(116,72)=  3.3485077322e-02
       coeff_l4(116,73)=  4.1444161534e-01
       coeff_l4(116,74)= -6.4653530717e-02
       coeff_l4(116,75)=  3.7614968419e-01
       coeff_l4(116,76)= -8.2214608788e-02
       coeff_l4(116,77)=  2.1786858141e-01
       coeff_l4(116,78)=  2.2174833715e-01
       coeff_l4(116,79)=  3.6641022563e-01
       coeff_l4(116,80)=  4.3333882093e-01
       coeff_l4(116,81)= -1.2059520185e-01
       coeff_l4(116,82)= -8.7964713573e-02
       coeff_l4(116,83)= -4.5894905925e-02
       coeff_l4(116,84)= -9.0293258429e-02
       coeff_l4(116,85)= -1.0328108817e-01
       coeff_l4(116,86)=  5.5491799116e-01
       coeff_l4(116,87)=  5.2396960557e-02
       coeff_l4(116,88)=  1.3695475459e-01
       coeff_l4(116,89)=  7.9926900566e-02
       coeff_l4(116,90)=  5.2202004194e-01
       coeff_l4(116,91)= -6.3073539734e-01
       coeff_l4(116,92)= -1.8009342253e-01
       coeff_l4(116,93)=  4.6565228701e-01
       coeff_l4(116,94)= -2.6287456974e-02
       coeff_l4(116,95)=  6.2389826775e-01
       coeff_l4(116,96)=  4.3036392331e-01
       coeff_l4(116,97)= -5.1491791010e-01
       coeff_l4(116,98)=  1.2330841273e-01
       coeff_l4(116,99)= -2.0274330676e-01
       coeff_l4(116,100)=  2.2705656290e-01
       coeff_l4(116,101)= -5.0689060241e-02
       coeff_l4(116,102)=  3.9419591427e-01
       coeff_l4(116,103)= -2.3904967308e-01
       coeff_l4(116,104)= -1.0292858630e-01
       coeff_l4(116,105)=  8.9965081215e-01
       coeff_l4(117,1)=  4.1559451818e-01
       coeff_l4(117,2)=  4.7293344140e-01
       coeff_l4(117,3)=  4.5932814479e-01
       coeff_l4(117,4)=  7.3577612638e-01
       coeff_l4(117,5)=  6.6792678833e-01
       coeff_l4(117,6)=  7.3091822863e-01
       coeff_l4(117,7)=  9.1257381439e-01
       coeff_l4(117,8)=  7.2156924009e-01
       coeff_l4(117,9)=  3.0147746205e-01
       coeff_l4(117,10)=  5.4671859741e-01
       coeff_l4(117,11)=  4.8620113730e-01
       coeff_l4(117,12)=  7.0245772600e-01
       coeff_l4(117,13)=  7.1512073278e-01
       coeff_l4(117,14)=  2.5187787414e-01
       coeff_l4(117,15)= -1.2630198896e-01
       coeff_l4(117,16)= -3.6472633481e-02
       coeff_l4(117,17)= -2.7809613384e-03
       coeff_l4(117,18)= -2.0648242533e-01
       coeff_l4(117,19)=  1.0169108212e-01
       coeff_l4(117,20)= -1.8399049342e-01
       coeff_l4(117,21)=  8.4445901215e-02
       coeff_l4(117,22)= -1.5425325930e-01
       coeff_l4(117,23)=  3.8259953260e-02
       coeff_l4(117,24)= -2.0121183991e-01
       coeff_l4(117,25)= -7.8659497201e-02
       coeff_l4(117,26)= -2.1265983582e-01
       coeff_l4(117,27)=  1.5849251300e-02
       coeff_l4(117,28)= -7.0869095623e-02
       coeff_l4(117,29)=  1.7089676112e-02
       coeff_l4(117,30)= -1.7956361175e-01
       coeff_l4(117,31)= -3.8198783994e-02
       coeff_l4(117,32)= -1.2404290587e-01
       coeff_l4(117,33)= -1.7778028548e-01
       coeff_l4(117,34)=  7.1012437344e-02
       coeff_l4(117,35)=  3.4753254149e-03
       coeff_l4(117,36)=  1.6798482835e-01
       coeff_l4(117,37)=  8.8410437107e-02
       coeff_l4(117,38)=  2.8940254450e-01
       coeff_l4(117,39)= -2.5821167510e-03
       coeff_l4(117,40)=  8.3995476365e-02
       coeff_l4(117,41)= -1.3735763729e-01
       coeff_l4(117,42)= -1.1136633158e-01
       coeff_l4(117,43)=  1.9076567143e-02
       coeff_l4(117,44)=  1.8691714108e-01
       coeff_l4(117,45)= -2.4376569316e-02
       coeff_l4(117,46)=  2.1743908525e-02
       coeff_l4(117,47)= -1.2695679069e-01
       coeff_l4(117,48)=  3.0935123563e-01
       coeff_l4(117,49)=  1.0473116487e-01
       coeff_l4(117,50)= -7.9901136458e-02
       coeff_l4(117,51)= -2.3715767264e-01
       coeff_l4(117,52)=  3.4050634503e-01
       coeff_l4(117,53)= -2.2679367661e-01
       coeff_l4(117,54)= -1.5391421318e-01
       coeff_l4(117,55)=  1.2187787890e-01
       coeff_l4(117,56)=  5.6147422642e-02
       coeff_l4(117,57)= -1.1384204775e-01
       coeff_l4(117,58)=  9.6902273595e-02
       coeff_l4(117,59)=  1.1084002256e-01
       coeff_l4(117,60)= -4.0865115821e-02
       coeff_l4(117,61)=  3.2988704741e-02
       coeff_l4(117,62)= -8.0446965992e-02
       coeff_l4(117,63)= -1.3998182118e-01
       coeff_l4(117,64)= -1.8214541674e-01
       coeff_l4(117,65)= -3.6246764939e-03
       coeff_l4(117,66)=  1.7394612730e-01
       coeff_l4(117,67)= -8.5695512593e-02
       coeff_l4(117,68)= -5.4179318249e-02
       coeff_l4(117,69)= -2.0037373900e-01
       coeff_l4(117,70)=  2.1046085283e-02
       coeff_l4(117,71)= -1.6009733081e-01
       coeff_l4(117,72)= -1.1810203642e-01
       coeff_l4(117,73)=  4.6748523600e-03
       coeff_l4(117,74)= -1.4485917985e-01
       coeff_l4(117,75)=  1.1932047457e-01
       coeff_l4(117,76)= -2.4060184136e-02
       coeff_l4(117,77)=  1.5570230782e-01
       coeff_l4(117,78)= -1.8713413179e-01
       coeff_l4(117,79)=  2.6170450449e-01
       coeff_l4(117,80)= -1.1700434238e-01
       coeff_l4(117,81)=  1.4958967268e-01
       coeff_l4(117,82)=  1.1664920487e-02
       coeff_l4(117,83)= -1.8639120460e-01
       coeff_l4(117,84)= -7.4628025293e-02
       coeff_l4(117,85)= -4.9677852541e-02
       coeff_l4(117,86)=  1.7385702580e-02
       coeff_l4(117,87)= -1.0768635571e-01
       coeff_l4(117,88)=  4.6640560031e-02
       coeff_l4(117,89)= -1.7835721374e-02
       coeff_l4(117,90)= -1.4976255596e-01
       coeff_l4(117,91)=  3.1554734707e-01
       coeff_l4(117,92)=  9.9821992218e-02
       coeff_l4(117,93)= -5.4493371397e-02
       coeff_l4(117,94)=  1.2620230019e-01
       coeff_l4(117,95)= -1.5824091434e-01
       coeff_l4(117,96)= -4.3871939182e-02
       coeff_l4(117,97)=  2.5565616786e-02
       coeff_l4(117,98)= -1.0558129102e-01
       coeff_l4(117,99)= -1.2055106927e-03
       coeff_l4(117,100)= -2.7256307006e-01
       coeff_l4(117,101)=  1.1454693228e-01
       coeff_l4(117,102)=  1.4454612136e-01
       coeff_l4(117,103)=  2.2054453194e-01
       coeff_l4(117,104)= -4.1356813163e-02
       coeff_l4(117,105)= -1.8493857980e-01
       coeff_l4(118,1)=  4.4764056802e-01
       coeff_l4(118,2)=  4.3057706952e-01
       coeff_l4(118,3)=  1.0333834589e-01
       coeff_l4(118,4)=  2.8668045998e-01
       coeff_l4(118,5)= -7.3516017199e-01
       coeff_l4(118,6)= -6.1981004477e-01
       coeff_l4(118,7)= -6.0765489936e-02
       coeff_l4(118,8)= -8.8967448473e-01
       coeff_l4(118,9)= -1.1981543154e-01
       coeff_l4(118,10)=  3.6866247654e-01
       coeff_l4(118,11)= -1.6989935935e-01
       coeff_l4(118,12)= -5.4247479886e-02
       coeff_l4(118,13)=  9.0301483870e-02
       coeff_l4(118,14)=  1.8699248135e-01
       coeff_l4(118,15)= -1.4614833891e-01
       coeff_l4(118,16)= -1.1877358705e-01
       coeff_l4(118,17)=  2.2395293415e-01
       coeff_l4(118,18)= -5.6981712580e-01
       coeff_l4(118,19)= -2.7739927173e-01
       coeff_l4(118,20)=  1.1754073203e-02
       coeff_l4(118,21)= -5.2378702164e-01
       coeff_l4(118,22)= -1.1920060962e-01
       coeff_l4(118,23)= -7.1535639465e-02
       coeff_l4(118,24)= -2.0284570754e-01
       coeff_l4(118,25)=  1.9722199440e-01
       coeff_l4(118,26)= -1.4027038217e-01
       coeff_l4(118,27)=  2.3274078965e-01
       coeff_l4(118,28)=  2.3007710278e-01
       coeff_l4(118,29)= -1.4813403785e-01
       coeff_l4(118,30)=  5.1261216402e-01
       coeff_l4(118,31)= -1.6078151762e-01
       coeff_l4(118,32)=  4.5566353202e-01
       coeff_l4(118,33)=  4.2717230320e-01
       coeff_l4(118,34)=  1.1266808510e+00
       coeff_l4(118,35)=  3.9265897870e-01
       coeff_l4(118,36)=  1.1018184572e-01
       coeff_l4(118,37)=  2.8949600458e-01
       coeff_l4(118,38)= -3.3132031560e-01
       coeff_l4(118,39)= -1.4388820529e-01
       coeff_l4(118,40)=  1.5977889299e-02
       coeff_l4(118,41)=  7.8373438120e-01
       coeff_l4(118,42)=  2.4060299993e-01
       coeff_l4(118,43)= -5.9716922045e-01
       coeff_l4(118,44)= -4.6022376418e-01
       coeff_l4(118,45)= -1.9902471453e-02
       coeff_l4(118,46)= -2.4820810556e-01
       coeff_l4(118,47)= -3.4481048584e-01
       coeff_l4(118,48)= -1.0358655453e+00
       coeff_l4(118,49)=  2.0498962700e-01
       coeff_l4(118,50)=  1.6663648188e-01
       coeff_l4(118,51)=  6.4140915871e-01
       coeff_l4(118,52)= -4.3608371168e-02
       coeff_l4(118,53)=  8.4396487474e-01
       coeff_l4(118,54)=  6.6312128305e-01
       coeff_l4(118,55)= -4.3961536884e-01
       coeff_l4(118,56)=  9.4421207905e-02
       coeff_l4(118,57)= -1.8255056441e-01
       coeff_l4(118,58)= -2.8712505102e-01
       coeff_l4(118,59)=  1.2085422128e-01
       coeff_l4(118,60)=  2.1368888021e-01
       coeff_l4(118,61)= -3.5208088160e-01
       coeff_l4(118,62)= -1.4654447138e-01
       coeff_l4(118,63)= -4.7873643041e-01
       coeff_l4(118,64)=  2.5028806925e-01
       coeff_l4(118,65)= -3.4764894843e-01
       coeff_l4(118,66)= -8.3429419994e-01
       coeff_l4(118,67)= -1.5522997081e-01
       coeff_l4(118,68)= -3.2600274682e-01
       coeff_l4(118,69)= -6.2037062645e-01
       coeff_l4(118,70)=  2.1820944548e-01
       coeff_l4(118,71)=  3.6963430047e-01
       coeff_l4(118,72)=  9.6252515912e-02
       coeff_l4(118,73)= -5.5302506685e-01
       coeff_l4(118,74)=  4.8418045044e-02
       coeff_l4(118,75)= -6.6058337688e-01
       coeff_l4(118,76)= -1.3676737249e-01
       coeff_l4(118,77)= -6.6768802935e-04
       coeff_l4(118,78)= -1.6968010366e-01
       coeff_l4(118,79)=  1.3144348562e-01
       coeff_l4(118,80)= -4.9508523941e-01
       coeff_l4(118,81)= -5.4162645340e-01
       coeff_l4(118,82)=  5.5174136162e-01
       coeff_l4(118,83)=  4.6474260092e-01
       coeff_l4(118,84)=  3.1511697918e-02
       coeff_l4(118,85)= -7.0968642831e-02
       coeff_l4(118,86)=  1.1016047001e-01
       coeff_l4(118,87)=  5.2649927139e-01
       coeff_l4(118,88)= -3.8545477390e-01
       coeff_l4(118,89)=  4.3970242143e-01
       coeff_l4(118,90)= -2.7083296329e-02
       coeff_l4(118,91)= -4.7433310747e-01
       coeff_l4(118,92)= -4.8307156563e-01
       coeff_l4(118,93)= -2.6242306828e-01
       coeff_l4(118,94)=  3.1807088852e-01
       coeff_l4(118,95)= -3.4485197067e-01
       coeff_l4(118,96)= -3.7380224466e-01
       coeff_l4(118,97)=  2.1402066946e-01
       coeff_l4(118,98)= -3.2735839486e-01
       coeff_l4(118,99)=  1.4038385451e-01
       coeff_l4(118,100)=  5.0926464796e-01
       coeff_l4(118,101)=  1.4333875850e-02
       coeff_l4(118,102)=  2.1940362453e-01
       coeff_l4(118,103)=  3.3061921597e-01
       coeff_l4(118,104)=  1.0808534175e-01
       coeff_l4(118,105)= -3.6874881387e-01
       coeff_l4(119,1)= -8.4430098534e-02
       coeff_l4(119,2)= -1.2786236405e-01
       coeff_l4(119,3)=  9.2264721170e-03
       coeff_l4(119,4)= -1.1443117261e-01
       coeff_l4(119,5)= -2.4756385386e-01
       coeff_l4(119,6)=  1.9032649696e-02
       coeff_l4(119,7)= -1.9497114420e-01
       coeff_l4(119,8)= -1.6071005166e-01
       coeff_l4(119,9)= -1.0745976120e-01
       coeff_l4(119,10)= -2.0324328542e-01
       coeff_l4(119,11)=  1.1548542231e-01
       coeff_l4(119,12)= -2.4114642292e-02
       coeff_l4(119,13)=  2.6667186618e-01
       coeff_l4(119,14)=  5.8439940214e-02
       coeff_l4(119,15)=  1.9155141711e-01
       coeff_l4(119,16)= -1.9260551035e-01
       coeff_l4(119,17)= -1.9991654903e-02
       coeff_l4(119,18)=  8.4467111155e-03
       coeff_l4(119,19)= -1.5694145858e-01
       coeff_l4(119,20)= -7.5380280614e-02
       coeff_l4(119,21)= -1.2826997042e-01
       coeff_l4(119,22)= -6.8824172020e-02
       coeff_l4(119,23)= -6.3299626112e-02
       coeff_l4(119,24)=  3.8966145366e-02
       coeff_l4(119,25)= -1.0996226221e-01
       coeff_l4(119,26)= -4.4760997407e-03
       coeff_l4(119,27)=  8.9006587863e-02
       coeff_l4(119,28)= -2.1446844563e-02
       coeff_l4(119,29)= -8.3609372377e-02
       coeff_l4(119,30)= -6.2419301830e-03
       coeff_l4(119,31)=  7.1746490896e-02
       coeff_l4(119,32)= -3.3873394132e-02
       coeff_l4(119,33)=  1.2075883895e-01
       coeff_l4(119,34)=  2.5635831058e-02
       coeff_l4(119,35)=  1.5421783552e-02
       coeff_l4(119,36)= -4.0618341416e-02
       coeff_l4(119,37)= -1.3149346225e-02
       coeff_l4(119,38)= -1.0025232285e-01
       coeff_l4(119,39)= -4.9599643797e-02
       coeff_l4(119,40)= -1.9591741264e-01
       coeff_l4(119,41)=  1.6138593853e-01
       coeff_l4(119,42)= -3.2472226303e-03
       coeff_l4(119,43)= -7.8591026366e-02
       coeff_l4(119,44)= -1.8643904477e-02
       coeff_l4(119,45)= -6.7069254816e-02
       coeff_l4(119,46)=  1.5434290469e-01
       coeff_l4(119,47)= -2.7751676738e-02
       coeff_l4(119,48)= -1.1682476103e-01
       coeff_l4(119,49)= -1.3844715059e-01
       coeff_l4(119,50)=  9.6237741411e-02
       coeff_l4(119,51)= -6.6747479141e-02
       coeff_l4(119,52)= -4.1697715642e-04
       coeff_l4(119,53)=  1.3210980594e-01
       coeff_l4(119,54)=  5.4367423058e-02
       coeff_l4(119,55)= -9.1318868101e-02
       coeff_l4(119,56)=  4.7989141196e-02
       coeff_l4(119,57)=  1.0707064718e-01
       coeff_l4(119,58)= -4.7913197428e-02
       coeff_l4(119,59)=  6.6814556718e-02
       coeff_l4(119,60)= -9.7905136645e-03
       coeff_l4(119,61)=  1.9739313051e-02
       coeff_l4(119,62)=  1.0931672901e-01
       coeff_l4(119,63)=  5.0406074151e-03
       coeff_l4(119,64)=  1.1855927110e-01
       coeff_l4(119,65)=  9.7105056047e-03
       coeff_l4(119,66)=  1.3375805318e-01
       coeff_l4(119,67)=  2.4788757786e-02
       coeff_l4(119,68)= -1.2263532728e-01
       coeff_l4(119,69)= -1.4257544279e-01
       coeff_l4(119,70)= -3.7349087652e-03
       coeff_l4(119,71)= -1.5774758160e-01
       coeff_l4(119,72)= -7.2768703103e-02
       coeff_l4(119,73)= -9.2279858887e-02
       coeff_l4(119,74)=  1.6896706074e-02
       coeff_l4(119,75)= -4.6555098146e-02
       coeff_l4(119,76)=  2.1096948534e-02
       coeff_l4(119,77)=  6.7818574607e-02
       coeff_l4(119,78)= -1.0070738941e-01
       coeff_l4(119,79)=  1.1598435789e-01
       coeff_l4(119,80)= -1.4260175824e-01
       coeff_l4(119,81)= -6.8577885628e-02
       coeff_l4(119,82)= -4.3758969754e-02
       coeff_l4(119,83)= -3.0519964173e-02
       coeff_l4(119,84)= -1.1049726605e-01
       coeff_l4(119,85)= -3.5325892270e-02
       coeff_l4(119,86)=  4.5099869370e-02
       coeff_l4(119,87)= -7.9025648534e-02
       coeff_l4(119,88)=  1.1497683823e-01
       coeff_l4(119,89)=  1.0732179135e-01
       coeff_l4(119,90)= -7.7587708831e-02
       coeff_l4(119,91)= -1.9646828994e-02
       coeff_l4(119,92)= -2.4509290233e-02
       coeff_l4(119,93)=  1.6435140371e-01
       coeff_l4(119,94)= -3.2293181866e-02
       coeff_l4(119,95)= -5.8946684003e-02
       coeff_l4(119,96)=  1.9705096260e-02
       coeff_l4(119,97)= -3.4918673337e-02
       coeff_l4(119,98)=  4.6213127673e-02
       coeff_l4(119,99)=  7.0076867938e-02
       coeff_l4(119,100)= -3.6225400865e-02
       coeff_l4(119,101)=  5.8195527643e-02
       coeff_l4(119,102)=  1.5386399627e-01
       coeff_l4(119,103)=  1.2348821014e-01
       coeff_l4(119,104)=  1.9890198484e-02
       coeff_l4(119,105)=  2.9631860554e-02
       coeff_l4(120,1)=  9.5151938498e-02
       coeff_l4(120,2)=  2.2578808665e-01
       coeff_l4(120,3)=  2.1165896952e-01
       coeff_l4(120,4)=  1.3746725023e-01
       coeff_l4(120,5)=  1.7362558842e-01
       coeff_l4(120,6)=  2.8006958961e-01
       coeff_l4(120,7)=  4.8104920983e-01
       coeff_l4(120,8)=  3.5563129187e-01
       coeff_l4(120,9)= -1.4346010983e-01
       coeff_l4(120,10)=  4.2477782816e-02
       coeff_l4(120,11)= -1.2924610078e-01
       coeff_l4(120,12)=  7.5225390494e-02
       coeff_l4(120,13)=  5.7117749006e-02
       coeff_l4(120,14)=  1.4615114033e-01
       coeff_l4(120,15)=  1.1138441414e-01
       coeff_l4(120,16)=  5.1885522902e-02
       coeff_l4(120,17)=  6.1560153961e-02
       coeff_l4(120,18)= -3.1823817641e-02
       coeff_l4(120,19)= -1.2901289389e-02
       coeff_l4(120,20)= -1.0766264796e-01
       coeff_l4(120,21)= -6.2688149512e-02
       coeff_l4(120,22)=  7.5791761279e-02
       coeff_l4(120,23)=  5.6119100191e-03
       coeff_l4(120,24)=  1.1581918597e-01
       coeff_l4(120,25)=  1.3557679951e-01
       coeff_l4(120,26)=  1.8486815691e-01
       coeff_l4(120,27)=  1.0781269521e-01
       coeff_l4(120,28)= -3.5667056218e-03
       coeff_l4(120,29)= -2.1045839787e-01
       coeff_l4(120,30)=  8.3732806146e-02
       coeff_l4(120,31)= -5.8080006391e-02
       coeff_l4(120,32)=  1.3655576110e-01
       coeff_l4(120,33)=  3.9399270900e-03
       coeff_l4(120,34)= -8.7180398405e-02
       coeff_l4(120,35)=  2.4353114888e-02
       coeff_l4(120,36)= -1.5225017071e-01
       coeff_l4(120,37)=  1.6405323148e-01
       coeff_l4(120,38)= -1.7178769410e-01
       coeff_l4(120,39)=  1.9176962972e-01
       coeff_l4(120,40)=  4.6292208135e-02
       coeff_l4(120,41)=  7.2426050901e-02
       coeff_l4(120,42)=  1.3700661063e-01
       coeff_l4(120,43)= -2.5545129180e-01
       coeff_l4(120,44)= -1.4236815274e-01
       coeff_l4(120,45)=  7.7712796628e-02
       coeff_l4(120,46)=  9.8494574428e-02
       coeff_l4(120,47)= -6.7085817456e-02
       coeff_l4(120,48)=  5.3874615580e-02
       coeff_l4(120,49)=  9.2741213739e-02
       coeff_l4(120,50)=  2.2898290306e-02
       coeff_l4(120,51)= -8.3733059466e-02
       coeff_l4(120,52)=  1.4414745569e-01
       coeff_l4(120,53)=  9.0278737247e-02
       coeff_l4(120,54)=  1.4556598850e-02
       coeff_l4(120,55)= -2.5188627839e-01
       coeff_l4(120,56)=  4.3489109725e-02
       coeff_l4(120,57)= -1.1675510556e-01
       coeff_l4(120,58)= -1.2675400078e-01
       coeff_l4(120,59)= -1.1116051674e-01
       coeff_l4(120,60)= -1.2250591815e-01
       coeff_l4(120,61)= -1.0926377028e-02
       coeff_l4(120,62)=  9.9876172841e-02
       coeff_l4(120,63)= -2.4059077725e-02
       coeff_l4(120,64)= -5.4812800139e-02
       coeff_l4(120,65)= -6.0771960765e-02
       coeff_l4(120,66)= -2.0050749183e-01
       coeff_l4(120,67)=  1.2799826264e-01
       coeff_l4(120,68)= -1.3637101650e-01
       coeff_l4(120,69)= -5.1558822393e-02
       coeff_l4(120,70)=  7.0118971169e-02
       coeff_l4(120,71)=  9.6249319613e-02
       coeff_l4(120,72)=  1.3041690690e-03
       coeff_l4(120,73)=  1.0726407915e-01
       coeff_l4(120,74)= -6.0593903065e-02
       coeff_l4(120,75)=  1.2057924271e-01
       coeff_l4(120,76)=  4.4419001788e-02
       coeff_l4(120,77)= -4.1216652840e-02
       coeff_l4(120,78)= -6.6171281040e-02
       coeff_l4(120,79)= -9.6513656899e-03
       coeff_l4(120,80)=  9.2982068658e-02
       coeff_l4(120,81)=  3.0978916213e-02
       coeff_l4(120,82)= -4.3548937887e-02
       coeff_l4(120,83)=  1.2338386476e-01
       coeff_l4(120,84)=  7.3257200420e-02
       coeff_l4(120,85)= -2.2289508581e-01
       coeff_l4(120,86)= -5.9861835092e-02
       coeff_l4(120,87)= -1.7527738586e-02
       coeff_l4(120,88)= -1.5382406116e-01
       coeff_l4(120,89)=  2.1993252635e-01
       coeff_l4(120,90)= -2.5521691889e-02
       coeff_l4(120,91)= -1.9701117650e-02
       coeff_l4(120,92)=  1.7723802477e-02
       coeff_l4(120,93)= -1.7018656433e-01
       coeff_l4(120,94)=  2.5994741917e-01
       coeff_l4(120,95)= -4.9123421311e-02
       coeff_l4(120,96)= -1.1007172614e-01
       coeff_l4(120,97)= -5.0979431719e-02
       coeff_l4(120,98)=  1.9454862177e-01
       coeff_l4(120,99)=  9.6913002431e-02
       coeff_l4(120,100)= -1.0246322304e-01
       coeff_l4(120,101)=  8.7223812938e-02
       coeff_l4(120,102)= -1.5208393335e-01
       coeff_l4(120,103)=  3.4442741424e-02
       coeff_l4(120,104)= -1.3978581131e-01
       coeff_l4(120,105)=  9.1642491519e-02
       coeff_l4(121,1)=  9.0865892172e-01
       coeff_l4(121,2)=  1.7292108536e+00
       coeff_l4(121,3)=  1.1308149099e+00
       coeff_l4(121,4)=  1.6329990625e+00
       coeff_l4(121,5)=  1.5088393688e+00
       coeff_l4(121,6)=  1.6014932394e+00
       coeff_l4(121,7)=  8.3092623949e-01
       coeff_l4(121,8)=  1.0222548246e+00
       coeff_l4(121,9)= -4.1721126437e-01
       coeff_l4(121,10)= -5.2281463146e-01
       coeff_l4(121,11)= -1.6118717194e-01
       coeff_l4(121,12)=  4.4578763843e-01
       coeff_l4(121,13)=  3.5942107439e-01
       coeff_l4(121,14)= -1.3142319024e-01
       coeff_l4(121,15)= -1.0289013386e-01
       coeff_l4(121,16)= -7.8575136140e-03
       coeff_l4(121,17)=  9.7401231527e-02
       coeff_l4(121,18)=  1.6925567389e-01
       coeff_l4(121,19)= -3.1449413300e-01
       coeff_l4(121,20)= -1.7255364358e-01
       coeff_l4(121,21)= -1.9694602489e-01
       coeff_l4(121,22)= -1.1621664464e-01
       coeff_l4(121,23)=  5.7604163885e-01
       coeff_l4(121,24)= -3.9878654480e-01
       coeff_l4(121,25)=  4.8475142568e-02
       coeff_l4(121,26)= -4.3124717474e-01
       coeff_l4(121,27)=  2.8209280968e-01
       coeff_l4(121,28)=  2.4320770055e-02
       coeff_l4(121,29)= -1.4001119137e-01
       coeff_l4(121,30)=  1.6327212751e-01
       coeff_l4(121,31)=  1.0214870423e-01
       coeff_l4(121,32)= -4.8372548819e-01
       coeff_l4(121,33)=  2.5786095858e-01
       coeff_l4(121,34)= -6.7229166627e-02
       coeff_l4(121,35)=  1.9415117800e-01
       coeff_l4(121,36)= -9.3963816762e-02
       coeff_l4(121,37)= -1.7780707777e-01
       coeff_l4(121,38)=  2.6839223504e-01
       coeff_l4(121,39)= -3.0935513973e-01
       coeff_l4(121,40)= -5.7150624692e-02
       coeff_l4(121,41)= -1.1020087451e-01
       coeff_l4(121,42)= -5.8456491679e-02
       coeff_l4(121,43)= -3.9924898744e-01
       coeff_l4(121,44)=  5.4505642038e-04
       coeff_l4(121,45)=  7.4708437920e-01
       coeff_l4(121,46)=  1.5594542027e-01
       coeff_l4(121,47)= -5.2229147404e-02
       coeff_l4(121,48)= -3.1135445833e-01
       coeff_l4(121,49)= -1.5038903058e-01
       coeff_l4(121,50)=  1.4391487837e-01
       coeff_l4(121,51)= -2.6699185371e-02
       coeff_l4(121,52)=  6.2142502517e-02
       coeff_l4(121,53)= -4.8769614100e-01
       coeff_l4(121,54)= -1.5342743136e-02
       coeff_l4(121,55)= -1.8189795315e-01
       coeff_l4(121,56)= -2.0317289978e-02
       coeff_l4(121,57)=  3.5915285349e-01
       coeff_l4(121,58)=  6.2198676169e-02
       coeff_l4(121,59)=  4.7286613844e-03
       coeff_l4(121,60)=  4.1371816397e-01
       coeff_l4(121,61)= -3.5229560733e-01
       coeff_l4(121,62)=  2.1823346615e-01
       coeff_l4(121,63)=  2.6100838184e-01
       coeff_l4(121,64)= -3.0404824018e-01
       coeff_l4(121,65)=  1.9903725013e-02
       coeff_l4(121,66)= -2.7308776975e-01
       coeff_l4(121,67)=  3.3315647393e-02
       coeff_l4(121,68)= -1.7879636586e-01
       coeff_l4(121,69)=  2.0230822265e-01
       coeff_l4(121,70)=  3.7078538537e-01
       coeff_l4(121,71)=  3.6357459426e-01
       coeff_l4(121,72)=  1.6652397811e-01
       coeff_l4(121,73)=  1.2693071365e-01
       coeff_l4(121,74)= -4.0749418736e-01
       coeff_l4(121,75)= -6.9050326943e-02
       coeff_l4(121,76)= -2.2029688954e-01
       coeff_l4(121,77)=  3.0569595098e-01
       coeff_l4(121,78)= -3.2924675941e-01
       coeff_l4(121,79)=  4.5495185256e-01
       coeff_l4(121,80)=  2.3872609437e-01
       coeff_l4(121,81)= -3.7383207679e-01
       coeff_l4(121,82)=  1.0750438273e-01
       coeff_l4(121,83)=  8.6802095175e-02
       coeff_l4(121,84)=  7.9798430204e-02
       coeff_l4(121,85)= -1.2595537305e-01
       coeff_l4(121,86)=  1.5862122178e-01
       coeff_l4(121,87)= -1.6707596183e-01
       coeff_l4(121,88)= -4.1418660432e-02
       coeff_l4(121,89)=  1.5866796672e-01
       coeff_l4(121,90)=  3.5679477453e-01
       coeff_l4(121,91)=  1.1642845720e-01
       coeff_l4(121,92)= -3.7773540616e-01
       coeff_l4(121,93)=  1.9267961383e-01
       coeff_l4(121,94)= -3.5371717811e-01
       coeff_l4(121,95)=  4.5514586568e-01
       coeff_l4(121,96)=  1.7089348694e-04
       coeff_l4(121,97)=  8.2093782723e-02
       coeff_l4(121,98)= -2.5029185414e-01
       coeff_l4(121,99)= -1.1652790755e-01
       coeff_l4(121,100)= -9.1515861452e-02
       coeff_l4(121,101)=  1.0992832482e-01
       coeff_l4(121,102)=  4.1879323125e-01
       coeff_l4(121,103)= -2.8597783297e-02
       coeff_l4(121,104)= -3.7470772862e-01
       coeff_l4(121,105)= -6.1502911150e-02
       coeff_l4(122,1)= -4.0475818515e-01
       coeff_l4(122,2)= -2.4667543173e-01
       coeff_l4(122,3)= -1.2820185721e-01
       coeff_l4(122,4)= -2.2919134796e-01
       coeff_l4(122,5)= -3.3765268326e-01
       coeff_l4(122,6)= -1.1528143287e-01
       coeff_l4(122,7)= -1.7024500668e-01
       coeff_l4(122,8)= -2.5656542182e-01
       coeff_l4(122,9)=  1.5581897460e-02
       coeff_l4(122,10)= -1.4798092842e-01
       coeff_l4(122,11)=  1.1639325321e-01
       coeff_l4(122,12)= -1.5497910976e-01
       coeff_l4(122,13)=  1.3662774861e-01
       coeff_l4(122,14)=  3.0245799571e-02
       coeff_l4(122,15)=  1.5612639487e-01
       coeff_l4(122,16)= -1.6180813313e-01
       coeff_l4(122,17)= -1.2883010507e-01
       coeff_l4(122,18)=  1.5466213226e-01
       coeff_l4(122,19)= -2.3664665222e-01
       coeff_l4(122,20)=  7.8781843185e-02
       coeff_l4(122,21)= -5.1015850157e-02
       coeff_l4(122,22)= -9.2073857784e-02
       coeff_l4(122,23)=  2.0631350577e-01
       coeff_l4(122,24)= -1.3429385424e-01
       coeff_l4(122,25)=  3.3307775855e-02
       coeff_l4(122,26)=  6.7222453654e-02
       coeff_l4(122,27)=  5.6581106037e-02
       coeff_l4(122,28)= -8.1246711314e-02
       coeff_l4(122,29)=  1.1806435138e-01
       coeff_l4(122,30)=  2.2148384154e-01
       coeff_l4(122,31)= -1.6011434793e-01
       coeff_l4(122,32)=  5.2991006523e-02
       coeff_l4(122,33)= -5.0670299679e-02
       coeff_l4(122,34)=  1.9026710093e-01
       coeff_l4(122,35)=  4.8522677273e-02
       coeff_l4(122,36)=  7.7558062971e-02
       coeff_l4(122,37)=  4.6882282943e-02
       coeff_l4(122,38)=  8.9789822698e-02
       coeff_l4(122,39)= -1.1306060106e-01
       coeff_l4(122,40)=  2.2689266130e-02
       coeff_l4(122,41)=  4.2252600193e-02
       coeff_l4(122,42)=  8.9024737477e-02
       coeff_l4(122,43)=  4.6103898436e-02
       coeff_l4(122,44)=  3.5743083805e-02
       coeff_l4(122,45)=  3.4813862294e-02
       coeff_l4(122,46)=  1.9298210740e-01
       coeff_l4(122,47)=  1.2182643265e-01
       coeff_l4(122,48)= -5.1274813712e-02
       coeff_l4(122,49)=  1.2454733253e-01
       coeff_l4(122,50)= -1.3917931914e-01
       coeff_l4(122,51)= -9.3105860054e-02
       coeff_l4(122,52)= -9.6849933267e-02
       coeff_l4(122,53)= -6.9411724806e-02
       coeff_l4(122,54)=  1.6738134623e-01
       coeff_l4(122,55)= -1.1732029170e-01
       coeff_l4(122,56)= -2.6708709076e-02
       coeff_l4(122,57)=  1.7304765061e-02
       coeff_l4(122,58)=  3.6403656006e-02
       coeff_l4(122,59)= -9.9254567176e-03
       coeff_l4(122,60)= -9.2608734965e-02
       coeff_l4(122,61)=  9.3979306519e-02
       coeff_l4(122,62)=  3.3309783787e-02
       coeff_l4(122,63)= -1.0382462293e-01
       coeff_l4(122,64)= -1.6467553377e-01
       coeff_l4(122,65)=  1.0079309577e-03
       coeff_l4(122,66)=  7.4311330914e-02
       coeff_l4(122,67)=  1.0999186337e-01
       coeff_l4(122,68)=  1.2767946720e-01
       coeff_l4(122,69)=  1.1932728440e-01
       coeff_l4(122,70)= -1.7943454906e-02
       coeff_l4(122,71)= -1.0596036911e-01
       coeff_l4(122,72)=  9.2400282621e-02
       coeff_l4(122,73)=  1.1297307163e-01
       coeff_l4(122,74)=  1.2731088698e-01
       coeff_l4(122,75)=  7.1479134262e-02
       coeff_l4(122,76)=  4.4365052134e-02
       coeff_l4(122,77)=  3.5878289491e-02
       coeff_l4(122,78)= -1.0286216438e-01
       coeff_l4(122,79)= -6.7498655990e-03
       coeff_l4(122,80)=  1.2563142926e-02
       coeff_l4(122,81)=  1.3790041208e-01
       coeff_l4(122,82)=  4.2596742511e-02
       coeff_l4(122,83)=  6.5167270601e-02
       coeff_l4(122,84)=  1.0999152064e-01
       coeff_l4(122,85)= -1.3504470885e-01
       coeff_l4(122,86)=  1.0780809075e-01
       coeff_l4(122,87)=  9.9542878568e-02
       coeff_l4(122,88)=  1.6577728093e-01
       coeff_l4(122,89)=  5.9015251696e-02
       coeff_l4(122,90)=  9.5955371857e-02
       coeff_l4(122,91)= -8.4675975144e-02
       coeff_l4(122,92)=  3.4672055393e-02
       coeff_l4(122,93)= -5.6359045208e-02
       coeff_l4(122,94)=  1.9655419886e-01
       coeff_l4(122,95)= -5.4010711610e-03
       coeff_l4(122,96)= -2.2320441902e-02
       coeff_l4(122,97)= -1.1169271916e-01
       coeff_l4(122,98)= -3.3903922886e-02
       coeff_l4(122,99)=  6.3184045255e-02
       coeff_l4(122,100)=  4.5011557639e-02
       coeff_l4(122,101)= -1.7186704278e-01
       coeff_l4(122,102)= -6.6735029221e-02
       coeff_l4(122,103)=  1.0990468413e-01
       coeff_l4(122,104)= -2.1504244208e-01
       coeff_l4(122,105)= -3.0647043139e-02
       coeff_l4(123,1)= -2.2669681907e-01
       coeff_l4(123,2)= -2.6077710092e-02
       coeff_l4(123,3)= -2.4730859697e-01
       coeff_l4(123,4)= -2.2363977134e-01
       coeff_l4(123,5)= -9.5056563616e-02
       coeff_l4(123,6)= -2.4884581566e-02
       coeff_l4(123,7)= -2.2857968509e-01
       coeff_l4(123,8)= -2.5133067369e-01
       coeff_l4(123,9)= -5.0920568407e-02
       coeff_l4(123,10)= -8.4730803967e-02
       coeff_l4(123,11)=  1.5635220706e-01
       coeff_l4(123,12)=  1.8853118643e-02
       coeff_l4(123,13)=  7.8180760145e-02
       coeff_l4(123,14)= -1.0580182076e-01
       coeff_l4(123,15)=  5.3999222815e-02
       coeff_l4(123,16)=  2.8837643564e-02
       coeff_l4(123,17)= -1.8980541825e-01
       coeff_l4(123,18)=  1.8871025741e-01
       coeff_l4(123,19)=  4.5514285564e-02
       coeff_l4(123,20)= -1.4179846644e-01
       coeff_l4(123,21)= -1.5398821235e-01
       coeff_l4(123,22)= -8.9607246220e-02
       coeff_l4(123,23)=  2.2030635178e-01
       coeff_l4(123,24)=  1.3845451176e-02
       coeff_l4(123,25)= -1.0564676486e-03
       coeff_l4(123,26)= -5.9318039566e-02
       coeff_l4(123,27)=  1.4279250801e-01
       coeff_l4(123,28)=  1.5482904390e-02
       coeff_l4(123,29)= -5.4370556027e-02
       coeff_l4(123,30)=  1.6033042967e-01
       coeff_l4(123,31)=  3.9834327996e-02
       coeff_l4(123,32)=  4.4118113816e-02
       coeff_l4(123,33)=  2.8176084161e-02
       coeff_l4(123,34)=  6.5942004323e-02
       coeff_l4(123,35)= -3.3331964165e-02
       coeff_l4(123,36)= -1.0354726762e-01
       coeff_l4(123,37)= -3.1754784286e-02
       coeff_l4(123,38)= -4.2042408139e-02
       coeff_l4(123,39)= -7.2827599943e-02
       coeff_l4(123,40)= -1.5364098363e-02
       coeff_l4(123,41)=  7.7961742878e-02
       coeff_l4(123,42)= -8.0636991188e-03
       coeff_l4(123,43)=  1.2976779044e-01
       coeff_l4(123,44)=  1.0731405765e-01
       coeff_l4(123,45)= -1.2168896943e-01
       coeff_l4(123,46)= -2.5316081941e-02
       coeff_l4(123,47)= -1.6952839494e-01
       coeff_l4(123,48)= -4.6880606562e-02
       coeff_l4(123,49)=  1.6770569980e-01
       coeff_l4(123,50)= -4.5008141547e-02
       coeff_l4(123,51)= -1.4289296232e-02
       coeff_l4(123,52)= -9.0252362192e-02
       coeff_l4(123,53)=  1.6494046152e-01
       coeff_l4(123,54)= -1.0398267955e-01
       coeff_l4(123,55)=  8.0677486956e-02
       coeff_l4(123,56)= -4.9386698753e-02
       coeff_l4(123,57)=  1.5087270737e-01
       coeff_l4(123,58)=  6.3566155732e-02
       coeff_l4(123,59)= -9.9831230938e-02
       coeff_l4(123,60)=  1.5316165052e-02
       coeff_l4(123,61)=  9.5263414085e-02
       coeff_l4(123,62)=  2.0430567861e-01
       coeff_l4(123,63)= -1.6326148808e-01
       coeff_l4(123,64)=  4.6577356756e-02
       coeff_l4(123,65)= -5.3392564878e-03
       coeff_l4(123,66)=  3.2147478312e-02
       coeff_l4(123,67)= -3.7957325578e-02
       coeff_l4(123,68)= -2.1857960150e-02
       coeff_l4(123,69)= -9.3623295426e-02
       coeff_l4(123,70)=  1.4315567911e-01
       coeff_l4(123,71)=  2.6762360707e-02
       coeff_l4(123,72)= -1.2519101612e-02
       coeff_l4(123,73)= -6.9496527314e-02
       coeff_l4(123,74)= -6.3492774963e-02
       coeff_l4(123,75)= -1.0291222483e-01
       coeff_l4(123,76)= -7.4698574841e-02
       coeff_l4(123,77)=  7.6661273837e-02
       coeff_l4(123,78)= -1.1627891660e-01
       coeff_l4(123,79)=  1.7107544839e-01
       coeff_l4(123,80)= -1.2975205481e-01
       coeff_l4(123,81)= -8.9383795857e-02
       coeff_l4(123,82)= -9.9384605885e-02
       coeff_l4(123,83)=  7.8988149762e-02
       coeff_l4(123,84)=  9.1890357435e-02
       coeff_l4(123,85)=  2.7485593455e-04
       coeff_l4(123,86)= -1.1623313278e-01
       coeff_l4(123,87)=  1.2351018190e-01
       coeff_l4(123,88)=  2.6019699872e-02
       coeff_l4(123,89)=  1.0044837743e-01
       coeff_l4(123,90)= -9.5902375877e-02
       coeff_l4(123,91)= -5.6729853153e-02
       coeff_l4(123,92)=  6.5778829157e-02
       coeff_l4(123,93)=  7.2329775430e-03
       coeff_l4(123,94)=  1.6805905104e-01
       coeff_l4(123,95)=  3.3254817128e-02
       coeff_l4(123,96)=  1.9322079420e-01
       coeff_l4(123,97)=  1.2851101160e-01
       coeff_l4(123,98)= -1.4024819434e-01
       coeff_l4(123,99)= -1.4438167214e-01
       coeff_l4(123,100)= -1.6484643519e-01
       coeff_l4(123,101)=  1.1214560270e-01
       coeff_l4(123,102)= -6.1826188117e-02
       coeff_l4(123,103)=  1.6977506876e-01
       coeff_l4(123,104)= -1.2267468870e-01
       coeff_l4(123,105)=  7.9680362251e-04
       coeff_l4(124,1)= -7.2204309702e-01
       coeff_l4(124,2)= -2.6731795073e-01
       coeff_l4(124,3)= -5.8200806379e-01
       coeff_l4(124,4)= -4.0361839533e-01
       coeff_l4(124,5)= -5.9775847197e-01
       coeff_l4(124,6)= -4.0630912781e-01
       coeff_l4(124,7)= -6.8311661482e-01
       coeff_l4(124,8)= -7.3049056530e-01
       coeff_l4(124,9)= -4.1853037477e-01
       coeff_l4(124,10)= -5.3109771013e-01
       coeff_l4(124,11)= -7.6061390340e-02
       coeff_l4(124,12)= -2.9109984636e-01
       coeff_l4(124,13)= -8.6058259010e-02
       coeff_l4(124,14)= -1.2094198167e-01
       coeff_l4(124,15)=  2.0311314613e-02
       coeff_l4(124,16)=  5.6510206312e-02
       coeff_l4(124,17)= -1.2076101452e-01
       coeff_l4(124,18)= -1.9850169122e-01
       coeff_l4(124,19)=  3.4609124064e-02
       coeff_l4(124,20)= -2.4806655943e-02
       coeff_l4(124,21)=  1.5137660503e-01
       coeff_l4(124,22)=  3.6621436477e-02
       coeff_l4(124,23)= -1.7471356690e-01
       coeff_l4(124,24)=  7.6672837138e-02
       coeff_l4(124,25)= -1.5741144121e-01
       coeff_l4(124,26)=  6.0440357774e-03
       coeff_l4(124,27)=  6.0317624593e-04
       coeff_l4(124,28)= -2.1287970245e-01
       coeff_l4(124,29)= -3.5346560180e-02
       coeff_l4(124,30)=  1.2638103962e-01
       coeff_l4(124,31)= -1.8158158660e-01
       coeff_l4(124,32)= -1.6094965860e-02
       coeff_l4(124,33)=  2.6273770258e-02
       coeff_l4(124,34)= -1.3274076581e-01
       coeff_l4(124,35)=  1.8690937757e-01
       coeff_l4(124,36)= -2.3994456977e-02
       coeff_l4(124,37)=  6.4572513103e-02
       coeff_l4(124,38)=  1.4213053882e-01
       coeff_l4(124,39)= -5.7707719505e-02
       coeff_l4(124,40)= -2.0737011731e-01
       coeff_l4(124,41)=  1.0911949724e-01
       coeff_l4(124,42)=  8.2204900682e-02
       coeff_l4(124,43)= -2.1007731557e-02
       coeff_l4(124,44)= -9.2121407390e-02
       coeff_l4(124,45)= -4.2648736387e-02
       coeff_l4(124,46)= -2.0783054829e-01
       coeff_l4(124,47)=  2.8176503256e-03
       coeff_l4(124,48)= -4.6410653740e-02
       coeff_l4(124,49)= -8.2544095814e-02
       coeff_l4(124,50)=  2.9826901853e-02
       coeff_l4(124,51)= -7.9844258726e-02
       coeff_l4(124,52)=  1.4550562948e-02
       coeff_l4(124,53)= -5.7661369443e-02
       coeff_l4(124,54)=  7.0099413395e-02
       coeff_l4(124,55)=  1.7102076113e-01
       coeff_l4(124,56)=  2.1576153114e-02
       coeff_l4(124,57)=  5.1363840699e-02
       coeff_l4(124,58)=  1.7785491049e-01
       coeff_l4(124,59)= -7.7840022743e-02
       coeff_l4(124,60)= -2.2170852125e-01
       coeff_l4(124,61)= -2.1659815684e-02
       coeff_l4(124,62)=  2.3702524602e-02
       coeff_l4(124,63)= -6.6161714494e-02
       coeff_l4(124,64)=  4.6735834330e-02
       coeff_l4(124,65)=  1.1364693195e-01
       coeff_l4(124,66)= -1.5305015258e-03
       coeff_l4(124,67)= -1.0897208005e-01
       coeff_l4(124,68)= -2.2938430309e-02
       coeff_l4(124,69)=  2.4819964543e-02
       coeff_l4(124,70)=  9.7194530070e-02
       coeff_l4(124,71)=  2.4318026006e-01
       coeff_l4(124,72)= -1.3760511577e-01
       coeff_l4(124,73)=  2.8658522293e-02
       coeff_l4(124,74)=  5.1882617176e-02
       coeff_l4(124,75)=  9.0117916465e-02
       coeff_l4(124,76)= -6.7903012037e-02
       coeff_l4(124,77)= -2.5074169040e-01
       coeff_l4(124,78)=  6.8322904408e-02
       coeff_l4(124,79)=  9.7338231280e-03
       coeff_l4(124,80)=  1.0813128948e-01
       coeff_l4(124,81)= -8.7002314627e-02
       coeff_l4(124,82)=  2.3796010762e-02
       coeff_l4(124,83)=  9.8159998655e-02
       coeff_l4(124,84)=  4.5775771141e-02
       coeff_l4(124,85)=  1.8377022445e-01
       coeff_l4(124,86)= -2.0139363408e-01
       coeff_l4(124,87)=  9.3419894576e-02
       coeff_l4(124,88)= -3.9671357721e-02
       coeff_l4(124,89)=  3.2857436687e-02
       coeff_l4(124,90)= -4.9112886190e-02
       coeff_l4(124,91)=  1.7837484181e-01
       coeff_l4(124,92)= -1.2291987240e-01
       coeff_l4(124,93)=  6.7092247307e-02
       coeff_l4(124,94)=  1.5549409389e-01
       coeff_l4(124,95)= -2.0072825253e-01
       coeff_l4(124,96)= -1.8519768119e-01
       coeff_l4(124,97)=  1.5732194483e-01
       coeff_l4(124,98)= -1.5935006738e-01
       coeff_l4(124,99)=  2.7453646064e-01
       coeff_l4(124,100)= -6.4452439547e-02
       coeff_l4(124,101)=  1.8618791364e-03
       coeff_l4(124,102)= -2.9546411708e-02
       coeff_l4(124,103)=  3.7417139392e-03
       coeff_l4(124,104)= -5.3749155253e-02
       coeff_l4(124,105)= -2.5484833121e-01
       coeff_l4(125,1)= -7.6026833057e-01
       coeff_l4(125,2)= -5.4971277714e-01
       coeff_l4(125,3)= -8.4918761253e-01
       coeff_l4(125,4)= -4.5806974173e-01
       coeff_l4(125,5)= -3.8487350941e-01
       coeff_l4(125,6)= -5.5492067337e-01
       coeff_l4(125,7)= -1.4570887089e+00
       coeff_l4(125,8)= -1.5092914104e+00
       coeff_l4(125,9)= -5.3051263094e-01
       coeff_l4(125,10)= -6.7333275080e-01
       coeff_l4(125,11)= -4.1980439425e-01
       coeff_l4(125,12)= -4.7320172191e-01
       coeff_l4(125,13)= -6.7190450430e-01
       coeff_l4(125,14)= -1.7121128738e-01
       coeff_l4(125,15)= -7.2177968919e-02
       coeff_l4(125,16)= -1.7436149716e-01
       coeff_l4(125,17)=  3.1271561980e-02
       coeff_l4(125,18)= -1.2472849339e-01
       coeff_l4(125,19)= -2.0576095581e-01
       coeff_l4(125,20)= -5.2874900401e-02
       coeff_l4(125,21)= -3.0510732532e-01
       coeff_l4(125,22)=  8.1675037742e-02
       coeff_l4(125,23)=  9.0252421796e-02
       coeff_l4(125,24)= -1.4770631492e-01
       coeff_l4(125,25)=  1.4848846197e-01
       coeff_l4(125,26)= -1.3892406225e-01
       coeff_l4(125,27)=  1.8696391582e-01
       coeff_l4(125,28)= -1.7492395639e-01
       coeff_l4(125,29)= -2.4230731651e-03
       coeff_l4(125,30)=  6.6750988364e-02
       coeff_l4(125,31)=  1.9244016707e-01
       coeff_l4(125,32)=  2.4281908572e-01
       coeff_l4(125,33)=  1.6265118122e-01
       coeff_l4(125,34)= -1.5170434117e-01
       coeff_l4(125,35)=  2.6558125019e-01
       coeff_l4(125,36)=  5.9764511883e-02
       coeff_l4(125,37)= -5.4018381983e-02
       coeff_l4(125,38)= -1.1111427099e-01
       coeff_l4(125,39)= -1.6207200289e-01
       coeff_l4(125,40)= -1.9572885334e-01
       coeff_l4(125,41)=  1.9883915782e-01
       coeff_l4(125,42)=  2.3613621294e-01
       coeff_l4(125,43)= -3.1682658195e-01
       coeff_l4(125,44)= -1.2719394872e-03
       coeff_l4(125,45)=  8.0455951393e-02
       coeff_l4(125,46)= -1.1951373518e-01
       coeff_l4(125,47)= -1.5112289786e-01
       coeff_l4(125,48)=  1.6134273261e-02
       coeff_l4(125,49)=  1.1852078885e-01
       coeff_l4(125,50)= -1.1139344424e-01
       coeff_l4(125,51)= -1.1291363277e-02
       coeff_l4(125,52)=  5.4946404696e-01
       coeff_l4(125,53)=  9.5337152481e-02
       coeff_l4(125,54)= -5.3011812270e-02
       coeff_l4(125,55)= -6.6313855350e-02
       coeff_l4(125,56)= -6.7336484790e-02
       coeff_l4(125,57)=  2.8264433146e-01
       coeff_l4(125,58)= -4.8647154123e-02
       coeff_l4(125,59)=  1.4091783762e-01
       coeff_l4(125,60)=  3.2737863064e-01
       coeff_l4(125,61)= -1.5901766717e-01
       coeff_l4(125,62)=  9.9446298555e-03
       coeff_l4(125,63)=  1.8995103240e-01
       coeff_l4(125,64)=  1.5134225786e-01
       coeff_l4(125,65)=  4.6240635216e-02
       coeff_l4(125,66)= -1.1813832074e-01
       coeff_l4(125,67)=  1.4945160365e-03
       coeff_l4(125,68)= -2.8075748682e-01
       coeff_l4(125,69)=  5.8108422905e-02
       coeff_l4(125,70)=  4.7957950830e-01
       coeff_l4(125,71)=  5.8128845692e-01
       coeff_l4(125,72)=  2.6721465588e-01
       coeff_l4(125,73)=  5.2734702826e-02
       coeff_l4(125,74)=  8.8350683451e-02
       coeff_l4(125,75)= -8.2457765937e-02
       coeff_l4(125,76)= -2.7726536989e-01
       coeff_l4(125,77)=  1.4031849802e-01
       coeff_l4(125,78)= -1.0635873675e-01
       coeff_l4(125,79)=  6.0905087739e-02
       coeff_l4(125,80)= -2.5742825866e-01
       coeff_l4(125,81)= -2.1752977371e-01
       coeff_l4(125,82)=  8.4062859416e-02
       coeff_l4(125,83)= -1.1893564463e-01
       coeff_l4(125,84)= -1.8893377483e-01
       coeff_l4(125,85)=  2.6646441221e-01
       coeff_l4(125,86)=  1.3573704660e-01
       coeff_l4(125,87)=  1.5884670615e-01
       coeff_l4(125,88)=  2.3795953020e-02
       coeff_l4(125,89)= -3.0514287949e-01
       coeff_l4(125,90)= -1.4933095872e-01
       coeff_l4(125,91)= -1.4904633164e-01
       coeff_l4(125,92)=  4.0333453566e-02
       coeff_l4(125,93)= -1.8165463209e-01
       coeff_l4(125,94)= -7.1723498404e-02
       coeff_l4(125,95)= -1.5405642986e-01
       coeff_l4(125,96)= -4.4395413995e-01
       coeff_l4(125,97)= -7.0026293397e-02
       coeff_l4(125,98)= -1.9069841504e-01
       coeff_l4(125,99)=  3.3691608906e-01
       coeff_l4(125,100)= -9.0526692569e-02
       coeff_l4(125,101)=  2.9248344898e-01
       coeff_l4(125,102)=  7.7879346907e-02
       coeff_l4(125,103)=  2.6371666789e-01
       coeff_l4(125,104)=  9.7258336842e-02
       coeff_l4(125,105)= -1.1312890053e-01
!bias layer 0
       bias_l0=0.0d0
!bias layer 1
       bias_l1=0.0d0
!bias layer 2
       bias_l2=0.0d0
!bias layer 3
       bias_l3=0.0d0
!bias layer 4
       bias_l4=0.0d0

  endsubroutine assign_coeff_bias

      subroutine ev2gm2m(dist,v2b,g2b,imol,igrad,n)

      implicit none

      integer,intent(in) :: imol, igrad
      integer,intent(in) :: n
      double precision,intent(in)  :: dist(n*(n-1)/2)
      double precision,intent(out) :: v2b(14), g2b(14,3)
      double precision :: v(14), grad(14)
      double precision :: r
      integer :: i,j

      ! dist should be in bohr
      ! it is converted to be r in Ang
      ! v2b is in eV
      ! grad2b is in eV/bohr

      ! V0 is 120.17443 kcal/mol
      v2b = 120.17443d0/23.061d0
      g2b = 0.0d0
      do i=1,n*(n-1)/2
        r=dist(i)*0.529177d0
        call ev2gm2(r,v,grad,imol,igrad)
        v2b = v2b + v/23.061d0
        if (igrad.eq.1) then
          do j=1,14
            g2b(j,i) = grad(j)/(23.061d0/0.529177d0)
          enddo
        endif
      enddo

      end subroutine ev2gm2m

      subroutine ev2gm2(r,v,grad,imol,igrad) 
!**********************************************************************
!
! Compute the diatomic potential of groupd-state triplet O2
!
! References: J. Chem. Phys. 132, 074307 (2010)
!
! Input:  r      interatomic distance in Angstrom
! Output: V      potential in kcal/mol
!         grad   gradient (kcal/mol)/Angstrom
!
!**********************************************************************
      implicit none

      integer,intent(in) :: imol, igrad
      double precision,intent(in)  :: r
      double precision,intent(out) :: v(14), grad(14)
! Parameters of analytical even-tempered Gaussian expansions for the
! ground state potential energy curve of O2 CBS+SR+SO+CV.
! Units: alpha in Angstrom^-2, beta=dimensionless, as_k in milihartree.
      double precision :: alpha,beta,as(0:7)
      double precision :: gam, del
      integer :: i,j,k 
! Dispersion variables
      double precision :: dist(1),disp,dispdr(1)

      v=0.0d0
      grad=0.0d0

! list of considered states in 3A' symmetry
! V(1) - O2(3Sig_g^-) + O(3P)
! V(2) - O2(3Sig_g^-) + O(3P)
! V(3) - O2(3Del_u)   + O(3P)
! V(4) - O2(3Del_u)   + O(3P)
! V(5) - O2(3Del_u)   + O(3P)
! V(6) - O2(3Sig_u^+) + O(3P)


! Calculate dispersion correction for all states
        dist(1)=r
        call d3disp(dist,disp,dispdr,1,imol)

      do i=1,14
      if ((i.eq.1).or.(i.eq.2)) then
      ! Original parameters
      !      alpha = 0.785d0
      !      beta = 1.307d0
      !      as(0) = -2388.5641690d0
      !      as(1) = 18086.977116d0
      !      as(2) = -71760.197585d0
      !      as(3) = 154738.09175d0
      !      as(4) = -215074.85646d0
      !      as(5) = 214799.54567d0
      !      as(6) = -148395.42850d0
      !      as(7) = 73310.781453d0

      ! O2 X 3Sig_g^-
      ! Modified parameters for D3(BJ)
       alpha =  9.91088352d-1
       beta =   1.35561616d0
       as(0) =  1.02827423d3
       as(1) = -1.04225661d4
       as(2) =  2.62510588d4
       as(3) = -3.57598376d4
       as(4) =  3.45354087d4
       as(5) = -1.54421054d4
       as(6) =  5.10053268d3
       as(7) =  1.52168614d4
      else if ((i.eq.3).or.(i.eq.4).or.(i.eq.5)) then
      ! O2 a 1Del_g
       alpha =  0.926743129655067d0
       beta =   1.49310094503326d0
       as(0) = -306.73487963d0
       as(1) = -1390.05533975d0
       as(2) =  4128.81576751d0
       as(3) = -4290.68553855d0
       as(4) =  11743.6159167d0
       as(5) = -4286.12400273d0
       as(6) =  21029.43435387d0
       as(7) =  1807.06950125d0
      else if (i.eq.6) then
      ! O2 b 1Sig_g^+
       alpha =  0.859772042751452d0
       beta =   1.52376335851761d0
       as(0) = -191.49156299d0
       as(1) = -971.3168522d0
       as(2) =  2222.1876317d0
       as(3) = -885.28321436d0
       as(4) =  7119.87096329d0
       as(5) =  2330.45304051d0
       as(6) =  14359.28824246d0
       as(7) =  10675.50993141d0
      else if ((i.eq.7).or.(i.eq.8)) then
       ! mixed energy curve
       alpha =  2.19185728150371d0
       beta =   1.05793185102898d0
       as(0) = -2663791.40882274d0
       as(1) =  4890643.13480523d0
       as(2) =  18211797.6008347d0
       as(3) = -69478406.8831105d0
       as(4) =  88925293.4564620d0
       as(5) = -48996476.3149754d0
       as(6) =  7263885.80760647d0
       as(7) =  1859624.51906546d0
       gam = 15.5242851149696d0
       del = 39.7454754114259d0
      else if ((i.eq.9).or.(i.eq.10)) then
      ! O2 c 1Sig_u^-
       alpha =  0.950046799307d0
       beta =   1.642688451339d0
       as(0) = -483.125326d0
       as(1) =  956.00518d0
       as(2) =  1684.94566d0
       as(3) =  2347.10958d0
       as(4) =  18401.7933d0
       as(5) = -49552.6652d0
       as(6) = -367905.919d0
       as(7) =  47140027.3d0
      else if ((i.eq.11).or.(i.eq.12).or.(i.eq.13)) then
      ! O2 C 3Del_u
       alpha =  1.11399797348271d0
       beta =   1.63184883563828d0
       as(0) = -767.018283d0
       as(1) =  2893.42549d0
       as(2) = -3964.21584d0
       as(3) =  32389.5668d0
       as(4) = -203511.939d0
       as(5) =  2627822.41d0
       as(6) = -68125724.3d0
       as(7) =  4210349180d0
      else if (i.eq.14) then
      ! O2 A 3Sig_u^+
       alpha =  1.24888948675275d0
       beta =   1.61630730207906d0
       as(0) = -866.519522d0
       as(1) =  3532.65087d0
       as(2) = -2927.82317d0
       as(3) =  23357.3166d0
       as(4) = -113057.808d0
       as(5) =  1880718.34d0
       as(6) = -73167704.6d0
       as(7) =  6245674420d0
      endif

      do k=0,7
       v(i)= v(i) + as(k)*dexp(-alpha*beta**k*r**2)
      enddo
      if ((i.eq.7).or.(i.eq.8)) then
       v(i)= v(i) + gam*dexp(-del*(r-1.7d0)**2)
      endif
      !if ((i.eq.1).or.(i.eq.2)) then
! From milihartree to kcal/mol
       !v(i)=v(i)*627.509523475149d-3
      !endif

! Add D3 dispersion correction
        v(i)=v(i)+disp

! Compute the gradient if needed
      if (igrad.eq.1) then
       do k=0,7
        grad(i)=grad(i)-2.d0*as(k)*alpha*beta**k*r*dexp(-alpha*beta**k*r**2)
       enddo
       if ((i.eq.7).or.(i.eq.8)) then
        grad(i)=grad(i)-2.0d0*del*gam*(r-1.7d0)*dexp(-del*(r-1.7d0)**2)
       endif
       !if ((i.eq.1).or.(i.eq.2)) then
! Convert from milihartree/A to (kcal/mol)/A
        !grad(i)=grad(i)*627.509523475149d-3
       !endif
        grad(i)= grad(i) + dispdr(1)
      endif

      enddo

      return
      end subroutine ev2gm2

      subroutine d3disp(dist,disp,dispdr,igrad,imol)
!**********************************************************************
! Dispersion correction based on Grimme's D3(BJ) calculation for
! diatomic pairs
!
! Several subroutines of DFTD3 V3.1 Rev 1 by Grimme were merged into 
! subroutine edisp and they have been heavily modified to calculate only
! that dispersion energy correction that is needed.
!
! S. Grimme, J. Antony, S. Ehrlich and H. Krieg
! J. Chem. Phys, 132 (2010), 154104
! and 
! S. Grimme, S. Ehrlich and L. Goerigk, J. Comput. Chem, 32 (2011),
! 1456-1465
!
! The C6 values are fixed.
!
!**********************************************************************

      double precision cn(2),s6,s8,rs6,rs8
      double precision dist(1), e6(1), e8(1), disp, dispdr(1), c6(2)
      double precision e6dr(1),e8dr(1)
      integer iz(2), mxc(94), i, j, igrad
      double precision c6ab(94,94,5,5,3)
      double precision r2r4(94)
      double precision autoang,autokcal
      integer imol

      autoang =0.52917726d0
      autokcal=627.509541d0

! Generalized parameters for BJ damping from P. Verma, B. Wang, 
! L. E. Fernandez, and D. G. Truhlar, J. Phys. Chem. A 121, 2855 (2017)
      s6= 1.0d0
      s8= 2.0d0
      rs6= 0.5299d0
      rs8= 2.20d0

      do i=1,1
      dist(i)=dist(i)/autoang
      enddo

      if (imol.eq.4) then
! iz for O2 system
      iz(1)=8
      iz(2)=8

! C6 for O2 system
      c6(1)=12.8d0
      c6(2)=12.8d0
      else
! currently imol = 4 is used
      stop
      endif

! Calculate dispersion correction
      call edisp(94,5,2,dist,iz,mxc, &
           rs6,rs8,e6,e8,e6dr,e8dr,c6,0)

      disp = 0.0d0

      do i=1,1
      disp =disp + (-s6*e6(i)-s8*e8(i))*autokcal
      enddo

      if (igrad .eq. 1) then
      call edisp(94,5,2,dist,iz,mxc, &
           rs6,rs8,e6,e8,e6dr,e8dr,c6,1)

      dispdr(:) = 0.0d0

      do i=1,1
      dispdr(i) =dispdr(i) + (-s6*e6dr(i)-s8*e8dr(i))*autokcal/autoang
      enddo
      endif

      do i=1,1
      dist(i)=dist(i)*autoang
      enddo

      end subroutine d3disp

!**********************************************************************
! compute energy
!**********************************************************************
      subroutine edisp(max_elem,maxc,n,dist,iz,mxc, &
                 rs6,rs8,e6,e8,e6dr,e8dr,c6a,igrad)

      integer n,iz(2),max_elem,maxc,mxc(max_elem) 
      double precision dist(1),r2r4(max_elem),r0ab(max_elem,max_elem)
      double precision rs6,rs8,rcov(max_elem)
      double precision c6ab(max_elem,max_elem,maxc,maxc,3)
      double precision e6(1), e8(1), c6a(2), e6dr(1), e8dr(1)
       
      integer iat,jat,igrad
      double precision r,tmp,c6,c8,a1,a2
      double precision damp6,damp8
      double precision cn(n)
      double precision r2ab(n*n),cc6ab(n*n),dmp(n*n)
      integer step

      e6(:) =0.0d0
      e8(:) =0.0d0

      e6dr(:) =0.0d0
      e8dr(:) =0.0d0

      a1=rs6
      a2=rs8 

!  r2r4 =sqrt(0.5*r2r4(i)*dfloat(i)**0.5 ) with i=elementnumber
!  the large number of digits is just to keep the results consistent
!  with older versions. They should not imply any higher accuracy than
!  the old values
      r2r4(1:94)=(/ &
      2.00734898d0,  1.56637132d0,  5.01986934d0,  3.85379032d0, &
      3.64446594d0,  3.10492822d0,  2.71175247d0,  2.59361680d0, &
      2.38825250d0,  2.21522516d0,  6.58585536d0,  5.46295967d0, &
      5.65216669d0,  4.88284902d0,  4.29727576d0,  4.04108902d0, &
      3.72932356d0,  3.44677275d0,  7.97762753d0,  7.07623947d0, &
      6.60844053d0,  6.28791364d0,  6.07728703d0,  5.54643096d0, &
      5.80491167d0,  5.58415602d0,  5.41374528d0,  5.28497229d0, &
      5.22592821d0,  5.09817141d0,  6.12149689d0,  5.54083734d0, &
      5.06696878d0,  4.87005108d0,  4.59089647d0,  4.31176304d0, &
      9.55461698d0,  8.67396077d0,  7.97210197d0,  7.43439917d0, &
      6.58711862d0,  6.19536215d0,  6.01517290d0,  5.81623410d0, &
      5.65710424d0,  5.52640661d0,  5.44263305d0,  5.58285373d0, &
      7.02081898d0,  6.46815523d0,  5.98089120d0,  5.81686657d0, &
      5.53321815d0,  5.25477007d0, 11.02204549d0,  0.15679528d0, &
      9.35167836d0,  9.06926079d0,  8.97241155d0,  8.90092807d0, &
      8.85984840d0,  8.81736827d0,  8.79317710d0,  7.89969626d0, &
      8.80588454d0,  8.42439218d0,  8.54289262d0,  8.47583370d0, &
      8.45090888d0,  8.47339339d0,  7.83525634d0,  8.20702843d0, &
      7.70559063d0,  7.32755997d0,  7.03887381d0,  6.68978720d0, &
      6.05450052d0,  5.88752022d0,  5.70661499d0,  5.78450695d0, &
      7.79780729d0,  7.26443867d0,  6.78151984d0,  6.67883169d0, &
      6.39024318d0,  6.09527958d0, 11.79156076d0, 11.10997644d0, &
      9.51377795d0,  8.67197068d0,  8.77140725d0,  8.65402716d0, &
      8.53923501d0,  8.85024712d0 /)

! these new data are scaled with k2=4./3.  and converted a_0 via
! autoang=0.52917726d0
      rcov(1:94)=(/ &
       0.80628308d0, 1.15903197d0, 3.02356173d0, 2.36845659d0, &
       1.94011865d0, 1.88972601d0, 1.78894056d0, 1.58736983d0, &
       1.61256616d0, 1.68815527d0, 3.52748848d0, 3.14954334d0, &
       2.84718717d0, 2.62041997d0, 2.77159820d0, 2.57002732d0, &
       2.49443835d0, 2.41884923d0, 4.43455700d0, 3.88023730d0, &
       3.35111422d0, 3.07395437d0, 3.04875805d0, 2.77159820d0, &
       2.69600923d0, 2.62041997d0, 2.51963467d0, 2.49443835d0, &
       2.54483100d0, 2.74640188d0, 2.82199085d0, 2.74640188d0, &
       2.89757982d0, 2.77159820d0, 2.87238349d0, 2.94797246d0, &
       4.76210950d0, 4.20778980d0, 3.70386304d0, 3.50229216d0, &
       3.32591790d0, 3.12434702d0, 2.89757982d0, 2.84718717d0, &
       2.84718717d0, 2.72120556d0, 2.89757982d0, 3.09915070d0, &
       3.22513231d0, 3.17473967d0, 3.17473967d0, 3.09915070d0, &
       3.32591790d0, 3.30072128d0, 5.26603625d0, 4.43455700d0, &
       4.08180818d0, 3.70386304d0, 3.98102289d0, 3.95582657d0, &
       3.93062995d0, 3.90543362d0, 3.80464833d0, 3.82984466d0, &
       3.80464833d0, 3.77945201d0, 3.75425569d0, 3.75425569d0, &
       3.72905937d0, 3.85504098d0, 3.67866672d0, 3.45189952d0, &
       3.30072128d0, 3.09915070d0, 2.97316878d0, 2.92277614d0, &
       2.79679452d0, 2.82199085d0, 2.84718717d0, 3.32591790d0, &
       3.27552496d0, 3.27552496d0, 3.42670319d0, 3.30072128d0, &
       3.47709584d0, 3.57788113d0, 5.06446567d0, 4.56053862d0, &
       4.20778980d0, 3.98102289d0, 3.82984466d0, 3.85504098d0, &
       3.88023730d0, 3.90543362d0 /)

! DFT-D3
      step=0
      do iat=1,n-1
         do jat=iat+1,n
         step=step+1
         r=dist(step)
         c6=c6a(step)
! r2r4 stored in main as sqrt
         c8 =3.0d0*c6*r2r4(iz(iat))*r2r4(iz(jat))

! energy for BJ damping
          tmp=sqrt(c8/c6)
          e6(step)= c6/(r**6+(a1*tmp+a2)**6)
          e8(step)= c8/(r**8+(a1*tmp+a2)**8)
! calculate gradients
         if (igrad .eq. 1) then
! grad for BJ damping
          e6dr(step)=c6*(-6*r**5)/(r**6+(a1*tmp+a2)**6)**2
          e8dr(step)=c8*(-8*r**7)/(r**8+(a1*tmp+a2)**8)**2
         endif
         enddo
      enddo

      end subroutine edisp

