!***********************************************************************
!   System:                     O3
!   Functional form:            permutation restrained diabatization by 
!                               deep neural network
!   Common name:                O3_6_5Ap_2023 (6 quintet A' states)
!   Number of derivatives:      first derivatives of adiabatic surfaces
!   Number of bodies:           3
!   Number of surfaces:         6
!   Notes:                      -Diatomic potentials contain dispersion terms
!                               -Compile the subroutine requires LAPACK
!                                subroutine DSYEV
! 
!   Input:                      Cartesian coordinates in bohr
!   Output:                     Adiabatic energies (e) in hartrees,
!                               derivatives of adiabataic energies (g) in 
!                               hartrees/bohr, and NACs (h) in inverse bohrs.
!   Reference:
!   F. Badichi Akher, Y. Shu, Z. Varga, D. G. Truhlar, "Semiclassical multi-state
!   dynamics for six coupled 5A' states of O3", in preparation
!***********************************************************************

      subroutine dpem(x,igrad,u,ug)

      implicit none
      ! number of electronic state
      integer, parameter :: nstates=6
      integer, parameter :: natoms=3
      integer, intent(in) :: igrad
      double precision, intent(in) :: x(natoms,3)
      double precision, intent(out) :: u(nstates,nstates)
      double precision, intent(out) :: ug(nstates,nstates,natoms,3)

      double precision :: v(nstates), tx(natoms,3), tg(nstates,3,3)
      double precision :: td(nstates,nstates,3,3)
      double precision :: t(nstates,nstates)

      integer :: iatom, idir, j, istate, jstate
      !initialize 
      u=0.d0
      ug=0.d0

      ! transform from angstrom to bohr
      do iatom=1,natoms
        do idir=1,3
          tx(iatom,idir)=x(iatom,idir)/0.529177211
        enddo
      enddo
      tg=0.d0
      td=0.d0

      call energy_gradient(tx,v,tg,td,t,u,ug,natoms,nstates)

      u=u*27.211386
      ug=ug*51.422067

      end

 
  subroutine energy_gradient(x,e,g,h,t,u,ug,n,ns)
    ! n    - number of atoms: 3
    ! ns   - number of states: 6
    ! x    - Cartesian coordinates
    ! e    - adiabatic energies
    ! g    - derivatives of adiabatic energies w.r.t. Cartesians
    ! h    - nonadiabatic coupling vectors w.r.t. Cartesians
    ! t    - transformation matrix
    ! r    - internuclear distances
    ! gr   - derivatives of adiabatic energies w.r.t. internuclear
    ! distances
    ! v2b  - two-body energy contributions
    ! g2b  - derivatives of two-body energy contributions w.r.t.
    ! internuclear distances
    ! drdx - dr/dx matrix for converting derivatives

    implicit none
    integer, intent(in) :: n,ns
    real*8, intent(in) :: x(n,3)
    real*8, intent(inout) :: e(ns)
    real*8, intent(inout) :: g(ns,n,3)
    real*8, intent(inout) :: t(ns,ns)
    real*8, intent(inout) :: h(ns,ns,n,3)
    real*8, intent(inout) :: u(ns,ns)
    real*8, intent(inout) :: ug(ns,ns,n,3)

    ! layers and their sizes
    real*8, save :: coeff_l0(3,21), coeff_l1(21,36), coeff_l2(36,56), coeff_l3(56,56), coeff_l4(56,21)
    real*8, save :: bias_l0(21), bias_l1(36), bias_l2(56), bias_l3(56), bias_l4(21)
    real*8 :: r(n*(n-1)/2)
    real*8 :: gr(ns,n*(n-1)/2)
    real*8 :: hr(ns,ns,n*(n-1)/2)
    real*8 :: ur(ns,ns,n*(n-1)/2)
    real*8 :: v2b(ns),g2b(ns,n*(n-1)/2)
    real*8 :: drdx(n*(n-1)/2,3*n)
    integer :: i,j,k,l,m
    logical, save :: first_time_data=.true.

    ! In this version we used the new activation functional 
    ! So DPEM layer becomes:
    ! A(r)*B(r)*dpem_layer
    ! the f,a,b parameters are 2, -2, and 7.5
    ! A(r)=0.5+0.5*tanh(2*(r+2))
    ! B(r)=0.5+0.5*yanh(2*(-x+7.5))
    e=0.d0
    g=0.d0
    h=0.d0

    ! get ddnn coefficients and bias
    if(first_time_data) then
      call assign_coeff_bias(coeff_l0,coeff_l1,coeff_l2,coeff_l3,coeff_l4,bias_l0,bias_l1,bias_l2,bias_l3,bias_l4)
      first_time_data=.false.
    endif
    ! input x is in bohr
    call convert_xyz_to_r(x,r,n)
    ! calculate the two-body terms
    call ev2gm2m(r,v2b,g2b,4,1,n)
    ! forward propagate sfp to energy 
    call backprop(r,n,ns,e,gr,hr,u,ur,t,coeff_l0,coeff_l1,coeff_l2,coeff_l3,&
      &coeff_l4,bias_l0,bias_l1,bias_l2,bias_l3,bias_l4,v2b,g2b)
    ! convert from gradient_r to gradient_xyz
    call grad_r_to_xyz(g,drdx,gr,r,x,n,ns)
    call nac_r_to_xyz(h,drdx,hr,r,x,n,ns)
    call nac_r_to_xyz(ug,drdx,ur,r,x,n,ns)
    ! convert energy from eV to hartree (1/27.2107=0.0367502)
    e(:)=e(:)*0.0367502d0
    u(:,:)=u(:,:)*0.0367502d0
    ! convert grad and nac from eV/bohr to hartree/bohr (1/27.2107=0.0367502)
    g(:,:,:)=g(:,:,:)*0.0367502d0
    ug(:,:,:,:)=ug(:,:,:,:)*0.0367502d0

  endsubroutine energy_gradient

!===========================
! back propagation
  subroutine backprop(r,n,ns,e,g,h,u,ur,t,coeff_l0,coeff_l1,coeff_l2,coeff_l3,&
    &coeff_l4,bias_l0,bias_l1,bias_l2,bias_l3,bias_l4,v2b,g2b)
    implicit none
    integer, intent(in) :: n, ns
    real*8 :: r(n*(n-1)/2)
    real*8, intent(inout) :: e(ns), g(ns,n*(n-1)/2), h(ns,ns,n*(n-1)/2)
    real*8, intent(inout) :: t(ns,ns), u(ns,ns), ur(ns,ns,n*(n-1)/2)
    real*8, intent(in) :: coeff_l0(3,21), coeff_l1(21,36), coeff_l2(36,56), coeff_l3(56,56), coeff_l4(56,21)
    real*8, intent(in) :: bias_l0(21), bias_l1(36), bias_l2(56), bias_l3(56), bias_l4(21)

    real*8 :: a0(21), a1(36), a2(56), a3(56), a4(21)
    real*8 :: z0(21), z1(36), z2(56), z3(56), z4(21)
    real*8 :: g0(3,21), g1(3,36), g2(3,56), g3(3,56), g4(3,21), g4p(21,3)
    real*8 :: dpem(ns,ns), dvdu(ns,21)
    real*8 :: dudr(ns,ns,3), tmpmat(ns,ns)

    integer :: sum1
    integer :: i,j,k,l

    real*8 :: v2b(ns),g2b(ns,n*(n-1)/2)

    a0=0.0d0
    a1=0.0d0
    a2=0.0d0
    a3=0.0d0
    a4=0.0d0
    z0=0.0d0
    z1=0.0d0
    z2=0.0d0
    z3=0.0d0
    z4=0.0d0

    e=0.0d0
    g0=0.0d0
    g1=0.0d0
    g2=0.0d0
    g3=0.0d0
    g4=0.0d0

    call mproduct(r,coeff_l0,z0,1,3,21)
    z0=z0 !+bias_l0
    call activation_gelu(z0,a0,21)

    call mproduct(a0,coeff_l1,z1,1,21,36)
    z1=z1 !+bias_l1
    call activation_gelu(z1,a1,36)

    call mproduct(a1,coeff_l2,z2,1,36,56)
    z2=z2 !+bias_l2
    call activation_gelu(z2,a2,56)
    
    call mproduct(a2,coeff_l3,z3,1,56,56)
    z3=z3 !+bias_l3
    call activation_gelu(z3,a3,56)

    !diabatic layer, use activation functional 
    call mproduct(a3,coeff_l4,z4,1,56,21)
    z4=z4 !+bias_l4
    call activation_functional_heviside(z4,a4,21,r,n*(n-1)/2)

    !construction many-body diabatic potential energy matrix (mb-dpem)
    do i=1,ns
      dpem(i,i)=a4(i)
    enddo
    do i=1,ns
      do j=i+1,ns
        call sumover(ns,i,sum1)
        k=ns-1+sum1+j-i
        dpem(i,j)=a4(k)
        dpem(j,i)=dpem(i,j)
      enddo
    enddo

    !adding the two-body terms to mb-dpem
    do i=1,ns
      dpem(i,i)=dpem(i,i) + v2b(i)
    enddo

    u=dpem

    ! diagonalize dpem
    call diagonalize(ns,dpem,e,t)

    ! cmpute dvdu 
    do i=1,ns
      do j=1,ns
        dvdu(i,j)=t(j,i)*t(j,i)
      enddo
    enddo 
    do i=1,ns
      do j=1,ns
        do k=j+1,ns
          call sumover(ns,j,sum1)
          l=ns-1+sum1+k-j
          dvdu(i,l)=2.0*t(j,i)*t(k,i)
        enddo
      enddo
    enddo 

   ! compute gradients 
   g0=coeff_l0   
   do i=1,3
     call activation_derivative_gelu(z0, a0, g0(i,:), 21)
   enddo 

   call mproduct(g0,coeff_l1,g1,3,21,36)

   do i=1,3
     call activation_derivative_gelu(z1, a1, g1(i,:), 36)
   enddo   

   call mproduct(g1,coeff_l2,g2,3,36,56)
   do i=1,3
     call activation_derivative_gelu(z2, a2, g2(i,:), 56)
   enddo

   call mproduct(g2,coeff_l3,g3,3,56,56)
   do i=1,3
     call activation_derivative_gelu(z3, a3, g3(i,:), 56)
   enddo 

   call mproduct(g3,coeff_l4,g4,3,56,21)
   ! here to add actional functional derivative
   call activation_functional_derivative_terms(g4,z4,21,r,n*(n-1)/2)

   do i=1,3
     do j=1,21
       g4p(j,i)=g4(i,j)
     enddo
   enddo 
 
   do i=1,ns
     do j=1,3
     g4p(i,j) = g4p(i,j) + g2b(i,j)
     enddo
   enddo

   call mproduct(dvdu,g4p,g,ns,21,3)

   ! compute NACs
   ! g4p is the DPEM gradients 
   dudr(:,:,:)=0.d0
   do i=1,ns
     dudr(i,i,:)=g4p(i,:)
   enddo
   do i=1,ns
     do j=i+1,ns
        call sumover(ns,i,sum1)
        k=ns-1+sum1+j-i
        dudr(i,j,:)=g4p(k,:)
        dudr(j,i,:)=dudr(i,j,:)
     enddo
   enddo 

   ur=dudr

   h(:,:,:)=0.d0
   do i=1,3
     tmpmat(:,:)=dudr(:,:,i)
     ! C^T*dU*C
     tmpmat=matmul(transpose(t), matmul(tmpmat,t))
     ! F_ij = (C^T*dU*C)_ij / (V_j-V_i)
     do j=1,ns
       do k=j+1,ns
         if ((e(k)-e(j)) > 1.d-8) then 
           h(j,k,i)=tmpmat(j,k)/(e(k)-e(j))
         else 
           h(j,k,i)=tmpmat(j,k)/1.d-8
         endif
         h(k,j,i)=-h(j,k,i)
       enddo
     enddo
   enddo

  endsubroutine backprop

!=========================
!activation function, gelu
  subroutine activation_gelu(z, a, n)
    integer i,n
    real*8, intent(in) :: z(n)
    real*8, intent(inout) :: a(n)

    do i=1,n
      a(i)=0.5*z(i)*(1+erf(z(i)/sqrt(2.d0)))
    enddo

  end subroutine activation_gelu

  subroutine activation_derivative_gelu(z, a, b, n)
    integer i,n
    real*8, intent(in) :: z(n), a(n)
    real*8, intent(inout) :: b(n)

    do i=1,n
!     b(i)=b(i)*(0.5*erf(z(i)/sqrt(2.d0))+0.398942*exp(-z(i)**2/2)*z(i)+0.5)
      b(i)=b(i)*(a(i)/z(i) + 0.398942*z(i)*exp(-z(i)**2/2) )
    enddo

  end subroutine activation_derivative_gelu

!=============================
!activation functional heviside
! A(r)*B(r)*dpem_layer
! the f,a,b parameters are 2, -2, and 7.5
! A(r)=0.5+0.5*tanh(2*(r+2))
! B(r)=0.5+0.5*tanh(2*(-x+7.5))
  subroutine activation_functional_heviside(z, a, n, r, nbonds)
    integer, intent(in) :: n, nbonds
    real*8, intent(in) :: z(n)
    real*8, intent(in) :: r(nbonds)
    real*8, intent(inout) :: a(n)
    real*8 :: r_max, Ar, Br
    real*8, parameter :: af_f=2, af_a=-2, af_b=7.5

    integer :: i

    r_max=maxval(r,nbonds)
    Ar=0.5+0.5*tanh(af_f*(r_max-af_a))
    Br=0.5+0.5*tanh(af_f*(-r_max+af_b))
    do i=1,n
      a(i)=Ar*Br*z(i)
    enddo

  end subroutine activation_functional_heviside

!=============================
!add activation functional derivative terms
!call activation_functional_derivative_terms(g4,21,n*(n-1)/2,z4)
  subroutine activation_functional_derivative_terms(g, z, n, r, nbonds)
    integer, intent(in) :: n, nbonds
    real*8, intent(in) :: z(n)
    real*8, intent(in) :: r(nbonds)
    real*8, intent(inout) :: g(nbonds,n)

    real*8 :: r_max
    real*8 :: Ar, Br
    real*8, parameter :: af_f=2, af_a=-2, af_b=7.5
    real*8 :: dfdr(nbonds)

    integer :: i

    r_max=maxval(r,nbonds)
    Ar=0.5+0.5*tanh(af_f*(r_max-af_a))
    Br=0.5+0.5*tanh(af_f*(-r_max+af_b))

    if (r(1).ge.r(2) .and. r(1).ge.r(3)) then 
      dfdr(1)=(0.5+(0.5*sinh(15.0-2*r(1)))/cosh(15.0-2*r(1)))/(cosh(2.0*r(1)+4.0)**2)&
        &+ (-0.5-(0.5*sinh(2.0*r(1)+4.0))/(cosh(2.0*r(1)+4.0)))/((cosh(15.0-2*r(1)))**2)
    else 
      dfdr(1)=0.d0
    endif

    if (r(2).ge.r(1) .and. r(2).ge.r(3)) then
      dfdr(2)=(0.5+(0.5*sinh(15.0-2*r(2)))/cosh(15.0-2*r(2)))/(cosh(2.0*r(2)+4.0)**2)&
        & + (-0.5-(0.5*sinh(2.0*r(2)+4.0))/(cosh(2.0*r(2)+4.0)))/((cosh(15.0-2*r(2)))**2)
    else
      dfdr(2)=0.d0
    endif

    if (r(3).ge.r(2) .and. r(3).ge.r(1)) then
      dfdr(3)=(0.5+(0.5*sinh(15.0-2*r(3)))/cosh(15.0-2*r(3)))/(cosh(2.0*r(3)+4.0)**2)&
        & + (-0.5-(0.5*sinh(2.0*r(3)+4.0))/(cosh(2.0*r(3)+4.0)))/((cosh(15.0-2*r(3)))**2)
    else
      dfdr(3)=0.d0
    endif

    do i=1,nbonds
      g(i,:)=dfdr(i)*z(:)+Ar*Br*g(i,:)
    enddo

  end subroutine activation_functional_derivative_terms

!==============================
! used to compute index in DPEM
  subroutine sumover(a,b,sum1)
    integer, intent(in) :: a, b
    integer, intent(out) :: sum1
    integer :: num

    sum1=1-a
    do num=a, a-b+1, -1
      sum1=sum1+num
    enddo
  end subroutine sumover

!==============================
! matrix diagonalization
! subroutine diagonalize(n,A_ss,B_ss,U_ss)
  subroutine diagonalize(n,A_ss,EV_s,U_ss)
    implicit none
    ! parameters
    integer, intent(in) :: n
    real*8, intent(in) :: A_ss(n,n)
!   real*8, intent(out) :: B_ss(n,n)
    real*8, intent(out) :: U_ss(n,n)
    ! internal variables
!   real*8 :: EV_s(n)
    real*8, intent(out) :: EV_s(n)
    integer :: io,i
    real*8,allocatable :: lapack_work_d(:)
    integer, save :: lapack_lwork_d 
    logical, save :: first_time_diag=.true.

    U_ss=A_ss
    if (first_time_diag) then
      lapack_lwork_d= -1
      allocate( lapack_work_d(2) )
      call dsyev('V','L',n,U_ss,n,EV_s,lapack_work_d,lapack_lwork_d,io)
      lapack_lwork_d=int(lapack_work_d(1))
      deallocate ( lapack_work_d )
      first_time_diag=.false.
    endif
    allocate( lapack_work_d(lapack_lwork_d) )
    call dsyev('V','L',n,U_ss,n,EV_s,lapack_work_d,lapack_lwork_d,io)
    ! U_ss already holds the transformation matrix
    ! now  building the diagonal matrix A_ss
!   B_ss=0.d0
!   do i=1,n
!     B_ss(i,i)=EV_s(i)
!   enddo

    return

  end subroutine diagonalize

!==========================
! convert de/dr to de/dxyz
  subroutine grad_r_to_xyz(g,drdx,gr,r,x,n,ns)
    implicit none
    integer, intent(in) :: n, ns
    real*8, intent(in) :: x(n,3)
    real*8, intent(in) :: r(n*(n-1)/2)
    real*8, intent(in) :: gr(ns,n*(n-1)/2)
    real*8, intent(inout) :: g(ns,n,3)
    real*8, intent(inout) :: drdx(n*(n-1)/2,3*n)

    real*8 ::  gx(ns,3*n)
    integer :: i,j

    drdx=0.0d0

    ! compute drdx
    drdx(1,1)=(x(1,1)-x(2,1))/r(1)
    drdx(1,2)=(x(1,2)-x(2,2))/r(1)
    drdx(1,3)=(x(1,3)-x(2,3))/r(1)
    drdx(1,4)=-drdx(1,1)
    drdx(1,5)=-drdx(1,2)
    drdx(1,6)=-drdx(1,3)
    drdx(2,1)=(x(1,1)-x(3,1))/r(2)
    drdx(2,2)=(x(1,2)-x(3,2))/r(2)
    drdx(2,3)=(x(1,3)-x(3,3))/r(2)
    drdx(2,7)=-drdx(2,1)
    drdx(2,8)=-drdx(2,2)
    drdx(2,9)=-drdx(2,3)
    drdx(3,4)=(x(2,1)-x(3,1))/r(3)
    drdx(3,5)=(x(2,2)-x(3,2))/r(3)
    drdx(3,6)=(x(2,3)-x(3,3))/r(3)
    drdx(3,7)=-drdx(3,4)
    drdx(3,8)=-drdx(3,5)
    drdx(3,9)=-drdx(3,6)

    call mproduct(gr,drdx,gx,ns,n*(n-1)/2,3*n)

    do i=1,n
      g(:,i,1)=gx(:,3*(i-1)+1)
      g(:,i,2)=gx(:,3*(i-1)+2)
      g(:,i,3)=gx(:,3*(i-1)+3)
    enddo 

  end subroutine grad_r_to_xyz

!==========================
! convert d/dr to d/dxyz for NAC
  subroutine nac_r_to_xyz(h,drdx,hr,r,x,n,ns)
    implicit none
    integer, intent(in) :: n, ns
    real*8, intent(in) :: x(n,3)
    real*8, intent(in) :: r(n*(n-1)/2)
    real*8, intent(in) :: hr(ns,ns,n*(n-1)/2)
    real*8, intent(inout) :: h(ns,ns,n,3)
    real*8, intent(inout) :: drdx(n*(n-1)/2,3*n)

    real*8 ::  hr_vec(ns*ns,n*(n-1)/2)
    real*8 ::  hx(ns*ns,3*n)

    integer :: i,j,k,l

    drdx=0.0d0

    ! compute drdx
    drdx(1,1)=(x(1,1)-x(2,1))/r(1)
    drdx(1,2)=(x(1,2)-x(2,2))/r(1)
    drdx(1,3)=(x(1,3)-x(2,3))/r(1)
    drdx(1,4)=-drdx(1,1)
    drdx(1,5)=-drdx(1,2)
    drdx(1,6)=-drdx(1,3)
    drdx(2,1)=(x(1,1)-x(3,1))/r(2)
    drdx(2,2)=(x(1,2)-x(3,2))/r(2)
    drdx(2,3)=(x(1,3)-x(3,3))/r(2)
    drdx(2,7)=-drdx(2,1)
    drdx(2,8)=-drdx(2,2)
    drdx(2,9)=-drdx(2,3)
    drdx(3,4)=(x(2,1)-x(3,1))/r(3)
    drdx(3,5)=(x(2,2)-x(3,2))/r(3)
    drdx(3,6)=(x(2,3)-x(3,3))/r(3)
    drdx(3,7)=-drdx(3,4)
    drdx(3,8)=-drdx(3,5)
    drdx(3,9)=-drdx(3,6)

    do i=1,ns
      do j=1,ns
        k=(i-1)*ns+j
        hr_vec(k,:)=hr(i,j,:)
      enddo
    enddo
    call mproduct(hr_vec,drdx,hx,ns*ns,n*(n-1)/2,3*n)

    do i=1,n
      do j=1,ns
        do k=1,ns
          l=(j-1)*ns+k
          h(j,k,i,1)=hx(l,3*(i-1)+1)
          h(j,k,i,2)=hx(l,3*(i-1)+2)
          h(j,k,i,3)=hx(l,3*(i-1)+3)
        enddo
      enddo
    enddo

  end subroutine nac_r_to_xyz

!==========================
! convert xyz to r
  subroutine convert_xyz_to_r(x,r,n)
    implicit none
    integer, intent(in) :: n
    real*8, intent(in) :: x(n,3)
    real*8, intent(out) :: r(n*(n-1)/2)
    integer :: i, j, l

    l=1
    !pairs of distances
    do i=1,n
      do j=i+1,n
        r(l)=sqrt((x(i,1)-x(j,1))**2+(x(i,2)-x(j,2))**2+(x(i,3)-x(j,3))**2)
        l=l+1
      enddo
    enddo

  endsubroutine convert_xyz_to_r

!=========================
! matrix vector product 
  subroutine mproduct(a,b,c,l,m,n)
    implicit none
    integer, intent(in) :: l, m, n
    real*8, intent(in) :: a(l,m), b(m,n)
    real*8, intent(inout) :: c(l,n)

    integer :: i, j, k
  
    do i=1,l
      do j=1,n
        c(i,j)=0.d0
        do k=1,m
          c(i,j)=c(i,j)+a(i,k)*b(k,j)
        enddo
      enddo
    enddo
    
    return
  endsubroutine mproduct

!===========================
! assign coefficients
  subroutine assign_coeff_bias(coeff_l0,coeff_l1,coeff_l2,coeff_l3,coeff_l4,bias_l0,bias_l1,bias_l2,bias_l3,bias_l4)
    implicit none
    real*8, intent(inout) :: coeff_l0(3,21), coeff_l1(21,36), coeff_l2(36,56), coeff_l3(56,56), coeff_l4(56,21)
    real*8, intent(inout) :: bias_l0(21), bias_l1(36), bias_l2(56), bias_l3(56), bias_l4(21)

!coeff layer 0
       coeff_l0(1,1)= -1.0251729488e+00
       coeff_l0(1,2)= -2.5187647343e-01
       coeff_l0(1,3)= -8.9708417654e-01
       coeff_l0(1,4)= -1.4748250246e+00
       coeff_l0(1,5)= -1.3306060433e-01
       coeff_l0(1,6)= -8.6096882820e-01
       coeff_l0(1,7)= -9.4275331497e-01
       coeff_l0(1,8)= -4.1927242279e-01
       coeff_l0(1,9)=  4.4375699013e-02
       coeff_l0(1,10)= -1.3737808466e+00
       coeff_l0(1,11)= -4.0681984276e-02
       coeff_l0(1,12)= -1.9536374807e+00
       coeff_l0(1,13)= -9.0742409229e-01
       coeff_l0(1,14)=  3.4990528226e-01
       coeff_l0(1,15)= -8.3207339048e-01
       coeff_l0(1,16)=  3.7435021996e-01
       coeff_l0(1,17)= -1.9892479479e-01
       coeff_l0(1,18)= -1.6795969009e+00
       coeff_l0(1,19)= -3.9977049828e-01
       coeff_l0(1,20)=  5.2358323336e-01
       coeff_l0(1,21)= -1.4276874065e-01
       coeff_l0(2,1)= -1.5029629469e+00
       coeff_l0(2,2)=  7.4837088585e-01
       coeff_l0(2,3)= -6.9536685944e-01
       coeff_l0(2,4)= -8.7353157997e-01
       coeff_l0(2,5)=  9.3722319603e-01
       coeff_l0(2,6)=  4.4928920269e-01
       coeff_l0(2,7)= -5.5647611618e-01
       coeff_l0(2,8)= -1.1818692684e+00
       coeff_l0(2,9)=  4.8758277297e-01
       coeff_l0(2,10)= -1.6761893034e+00
       coeff_l0(2,11)= -7.4134483933e-02
       coeff_l0(2,12)= -9.1910719872e-01
       coeff_l0(2,13)= -1.1901957989e+00
       coeff_l0(2,14)= -6.6678851843e-01
       coeff_l0(2,15)=  1.2780059576e+00
       coeff_l0(2,16)=  6.4887744188e-01
       coeff_l0(2,17)= -2.3892303929e-02
       coeff_l0(2,18)=  4.9346137792e-02
       coeff_l0(2,19)=  1.0822886229e-01
       coeff_l0(2,20)= -1.7942533493e+00
       coeff_l0(2,21)= -5.0528999418e-02
       coeff_l0(3,1)= -8.9638912678e-01
       coeff_l0(3,2)=  2.0650127530e-01
       coeff_l0(3,3)= -6.0811567307e-01
       coeff_l0(3,4)= -5.1862168312e-01
       coeff_l0(3,5)= -1.7118071318e+00
       coeff_l0(3,6)=  6.9838541746e-01
       coeff_l0(3,7)=  6.2727296352e-01
       coeff_l0(3,8)= -6.0746061802e-01
       coeff_l0(3,9)=  6.3431304693e-01
       coeff_l0(3,10)= -1.3790383339e+00
       coeff_l0(3,11)= -8.5834898055e-02
       coeff_l0(3,12)= -9.9032062292e-01
       coeff_l0(3,13)= -7.8476428986e-01
       coeff_l0(3,14)= -7.7176558971e-01
       coeff_l0(3,15)= -8.0257743597e-01
       coeff_l0(3,16)= -1.3284335136e+00
       coeff_l0(3,17)= -3.2235756516e-01
       coeff_l0(3,18)=  2.8274756670e-01
       coeff_l0(3,19)=  9.9247741699e-01
       coeff_l0(3,20)=  8.8341730833e-01
       coeff_l0(3,21)= -1.0579357296e-01
!coeff layer 1
       coeff_l1(1,1)= -1.8967992067e-01
       coeff_l1(1,2)= -1.5737946332e-01
       coeff_l1(1,3)=  1.9270846248e-01
       coeff_l1(1,4)= -4.3512296677e-01
       coeff_l1(1,5)=  1.5229594707e-01
       coeff_l1(1,6)= -1.2635844946e-01
       coeff_l1(1,7)= -5.1014393568e-01
       coeff_l1(1,8)= -1.0704421997e-01
       coeff_l1(1,9)= -2.3858769238e-01
       coeff_l1(1,10)=  1.0142767429e-01
       coeff_l1(1,11)= -1.6098825634e-01
       coeff_l1(1,12)=  1.7147107422e-01
       coeff_l1(1,13)= -2.5970613956e-01
       coeff_l1(1,14)= -3.3216303587e-01
       coeff_l1(1,15)= -6.3076242805e-02
       coeff_l1(1,16)=  1.8842011690e-01
       coeff_l1(1,17)=  1.3432883658e-02
       coeff_l1(1,18)=  1.2022962421e-01
       coeff_l1(1,19)= -2.1466983855e-01
       coeff_l1(1,20)=  2.7087479830e-01
       coeff_l1(1,21)= -1.2076253444e-01
       coeff_l1(1,22)=  1.3329860568e-01
       coeff_l1(1,23)= -1.5489440411e-02
       coeff_l1(1,24)= -3.2419812679e-01
       coeff_l1(1,25)=  4.0257215500e-02
       coeff_l1(1,26)= -4.6008789539e-01
       coeff_l1(1,27)=  2.6422050595e-01
       coeff_l1(1,28)= -3.2170602679e-01
       coeff_l1(1,29)=  1.1701732874e-02
       coeff_l1(1,30)=  1.9903540611e-01
       coeff_l1(1,31)=  8.0514237285e-02
       coeff_l1(1,32)=  4.0591147542e-01
       coeff_l1(1,33)= -6.9268909283e-03
       coeff_l1(1,34)= -4.7529387474e-01
       coeff_l1(1,35)= -2.6019956917e-02
       coeff_l1(1,36)=  6.5212443471e-02
       coeff_l1(2,1)= -2.3026216030e-01
       coeff_l1(2,2)= -1.9800239801e-01
       coeff_l1(2,3)= -2.4878058434e+00
       coeff_l1(2,4)= -3.9372941852e-01
       coeff_l1(2,5)= -7.9271203279e-01
       coeff_l1(2,6)=  8.9580386877e-01
       coeff_l1(2,7)= -2.7956166267e+00
       coeff_l1(2,8)=  3.7470448017e-01
       coeff_l1(2,9)=  2.5222834945e-01
       coeff_l1(2,10)=  6.0379463434e-01
       coeff_l1(2,11)= -1.2410495281e+00
       coeff_l1(2,12)=  7.7746444941e-01
       coeff_l1(2,13)= -1.4115276337e+00
       coeff_l1(2,14)= -1.4465255737e+00
       coeff_l1(2,15)= -2.2584025860e+00
       coeff_l1(2,16)=  3.8986673951e-01
       coeff_l1(2,17)=  4.7316882014e-01
       coeff_l1(2,18)= -3.6337611079e-01
       coeff_l1(2,19)= -7.1973019838e-01
       coeff_l1(2,20)=  1.0930191278e+00
       coeff_l1(2,21)=  6.1406534910e-01
       coeff_l1(2,22)=  7.2125339508e-01
       coeff_l1(2,23)= -1.8816374540e+00
       coeff_l1(2,24)=  3.8171228766e-01
       coeff_l1(2,25)= -1.3755424321e-01
       coeff_l1(2,26)=  3.7722855806e-01
       coeff_l1(2,27)=  5.7164168358e-01
       coeff_l1(2,28)=  7.8357994556e-01
       coeff_l1(2,29)=  5.2943271585e-03
       coeff_l1(2,30)=  1.8245191872e-01
       coeff_l1(2,31)=  1.4760671854e+00
       coeff_l1(2,32)= -5.0000464916e-01
       coeff_l1(2,33)= -2.5552758574e-01
       coeff_l1(2,34)=  1.9958205521e-01
       coeff_l1(2,35)= -7.3196375370e-01
       coeff_l1(2,36)= -5.3850924969e-01
       coeff_l1(3,1)= -3.2465144992e-01
       coeff_l1(3,2)=  7.3242172599e-02
       coeff_l1(3,3)= -2.9009355232e-02
       coeff_l1(3,4)=  1.1696299165e-01
       coeff_l1(3,5)=  1.2685973942e-01
       coeff_l1(3,6)=  2.1643746644e-02
       coeff_l1(3,7)= -1.2191927433e-01
       coeff_l1(3,8)=  1.4772813022e-01
       coeff_l1(3,9)= -7.9628720880e-02
       coeff_l1(3,10)= -3.2027441263e-01
       coeff_l1(3,11)=  2.7274090797e-02
       coeff_l1(3,12)= -1.4562769234e-01
       coeff_l1(3,13)= -6.4919316769e-01
       coeff_l1(3,14)=  1.1408645660e-01
       coeff_l1(3,15)= -1.7574854195e-01
       coeff_l1(3,16)=  1.3141159713e-01
       coeff_l1(3,17)= -3.1253379583e-01
       coeff_l1(3,18)=  1.3689211011e-01
       coeff_l1(3,19)= -4.5425194502e-01
       coeff_l1(3,20)=  2.8916558623e-01
       coeff_l1(3,21)=  8.9421831071e-02
       coeff_l1(3,22)= -3.2481405139e-01
       coeff_l1(3,23)=  7.0166200399e-02
       coeff_l1(3,24)= -5.0706848502e-02
       coeff_l1(3,25)=  1.5613766015e-01
       coeff_l1(3,26)= -1.1855889112e-01
       coeff_l1(3,27)= -1.5904189646e-01
       coeff_l1(3,28)=  2.8095728159e-01
       coeff_l1(3,29)= -2.7877146006e-01
       coeff_l1(3,30)=  1.6507676244e-01
       coeff_l1(3,31)=  1.4201818407e-01
       coeff_l1(3,32)= -2.1868902445e-01
       coeff_l1(3,33)=  5.5865224451e-02
       coeff_l1(3,34)= -5.0302982330e-01
       coeff_l1(3,35)=  2.4747708440e-01
       coeff_l1(3,36)=  1.7069593072e-01
       coeff_l1(4,1)=  2.5777766481e-02
       coeff_l1(4,2)=  2.5280001760e-01
       coeff_l1(4,3)=  9.2095784843e-02
       coeff_l1(4,4)= -2.8689235449e-01
       coeff_l1(4,5)= -2.3694691062e-01
       coeff_l1(4,6)= -1.0427819192e-01
       coeff_l1(4,7)= -4.0793884546e-02
       coeff_l1(4,8)=  2.2747738659e-01
       coeff_l1(4,9)= -2.8170448542e-01
       coeff_l1(4,10)= -3.3415368199e-01
       coeff_l1(4,11)=  5.0646632910e-02
       coeff_l1(4,12)=  3.9605006576e-02
       coeff_l1(4,13)= -3.7268474698e-01
       coeff_l1(4,14)= -6.0989893973e-02
       coeff_l1(4,15)= -3.2091313601e-01
       coeff_l1(4,16)= -2.4260347709e-02
       coeff_l1(4,17)= -4.9890801311e-01
       coeff_l1(4,18)= -1.9254848361e-01
       coeff_l1(4,19)= -4.4614228606e-01
       coeff_l1(4,20)=  2.8770348430e-01
       coeff_l1(4,21)= -1.9520388450e-03
       coeff_l1(4,22)= -3.4768741578e-02
       coeff_l1(4,23)=  1.5312902397e-03
       coeff_l1(4,24)= -7.5470036268e-01
       coeff_l1(4,25)=  2.5524541736e-01
       coeff_l1(4,26)= -5.1942390203e-01
       coeff_l1(4,27)= -2.7015161514e-01
       coeff_l1(4,28)=  1.1967976391e-01
       coeff_l1(4,29)= -2.4622534215e-01
       coeff_l1(4,30)=  1.3900339603e-01
       coeff_l1(4,31)=  1.6819770634e-01
       coeff_l1(4,32)=  2.3951588571e-01
       coeff_l1(4,33)= -4.2658990622e-01
       coeff_l1(4,34)= -1.0517724752e+00
       coeff_l1(4,35)= -2.6281824708e-01
       coeff_l1(4,36)= -2.9833707213e-01
       coeff_l1(5,1)= -3.3056172729e-01
       coeff_l1(5,2)=  9.4947886467e-01
       coeff_l1(5,3)= -3.2658219337e-02
       coeff_l1(5,4)=  3.5001370311e-01
       coeff_l1(5,5)=  1.1246031523e+00
       coeff_l1(5,6)=  7.6337814331e-01
       coeff_l1(5,7)= -8.4122586250e-01
       coeff_l1(5,8)= -5.1497888565e-01
       coeff_l1(5,9)=  1.8001359701e+00
       coeff_l1(5,10)=  3.4412047267e-01
       coeff_l1(5,11)=  3.9583095908e-01
       coeff_l1(5,12)= -9.6033924818e-01
       coeff_l1(5,13)= -2.3528397083e+00
       coeff_l1(5,14)=  9.6833688021e-01
       coeff_l1(5,15)=  2.2505481541e-01
       coeff_l1(5,16)= -1.4488277435e+00
       coeff_l1(5,17)= -1.4923440218e+00
       coeff_l1(5,18)=  4.1231876612e-01
       coeff_l1(5,19)= -5.5434495211e-01
       coeff_l1(5,20)= -1.2526570559e+00
       coeff_l1(5,21)= -2.2191047668e+00
       coeff_l1(5,22)= -2.0145456493e-01
       coeff_l1(5,23)=  1.2487148046e+00
       coeff_l1(5,24)= -3.5280504227e+00
       coeff_l1(5,25)= -3.9073336124e-01
       coeff_l1(5,26)= -8.8634634018e-01
       coeff_l1(5,27)= -3.0937564373e-01
       coeff_l1(5,28)= -2.8054187298e+00
       coeff_l1(5,29)=  1.8659052253e-01
       coeff_l1(5,30)= -8.9666032791e-01
       coeff_l1(5,31)=  9.8381274939e-01
       coeff_l1(5,32)= -2.8212136030e-01
       coeff_l1(5,33)= -4.6411898732e-01
       coeff_l1(5,34)=  1.3882040977e+00
       coeff_l1(5,35)=  3.5993091762e-02
       coeff_l1(5,36)=  3.5732260346e-01
       coeff_l1(6,1)= -2.2038459778e+00
       coeff_l1(6,2)=  1.2334854603e+00
       coeff_l1(6,3)= -2.4731237888e+00
       coeff_l1(6,4)=  1.3853499889e+00
       coeff_l1(6,5)= -3.0418462753e+00
       coeff_l1(6,6)= -1.7524939030e-02
       coeff_l1(6,7)=  1.3413733244e-01
       coeff_l1(6,8)= -1.0696616173e+00
       coeff_l1(6,9)= -1.5118885040e+00
       coeff_l1(6,10)=  8.4870630503e-01
       coeff_l1(6,11)= -1.9574440718e+00
       coeff_l1(6,12)=  4.0461350232e-02
       coeff_l1(6,13)= -1.2457515299e-01
       coeff_l1(6,14)= -6.8934428692e-01
       coeff_l1(6,15)= -2.3035221100e+00
       coeff_l1(6,16)= -2.3659820557e+00
       coeff_l1(6,17)=  4.9447098374e-01
       coeff_l1(6,18)=  7.5006854534e-01
       coeff_l1(6,19)= -2.8546500206e-01
       coeff_l1(6,20)=  2.7079361677e-01
       coeff_l1(6,21)=  5.5419075489e-01
       coeff_l1(6,22)= -1.1210092306e+00
       coeff_l1(6,23)= -3.7812516689e+00
       coeff_l1(6,24)=  1.7450865507e+00
       coeff_l1(6,25)= -4.2324924469e-01
       coeff_l1(6,26)= -1.2956617773e-01
       coeff_l1(6,27)=  3.7159398198e-01
       coeff_l1(6,28)=  3.0557283759e-01
       coeff_l1(6,29)= -1.8188804388e-01
       coeff_l1(6,30)= -9.5399105549e-01
       coeff_l1(6,31)= -4.3129244447e-01
       coeff_l1(6,32)= -1.5671175718e-01
       coeff_l1(6,33)= -3.3136241436e+00
       coeff_l1(6,34)=  1.6945178509e+00
       coeff_l1(6,35)= -1.3457428217e+00
       coeff_l1(6,36)= -2.7125139236e+00
       coeff_l1(7,1)= -3.4660249949e-01
       coeff_l1(7,2)=  1.2277750969e+00
       coeff_l1(7,3)= -3.8599690795e-01
       coeff_l1(7,4)= -1.4662427902e+00
       coeff_l1(7,5)=  2.5500380993e+00
       coeff_l1(7,6)=  7.0343279839e-01
       coeff_l1(7,7)= -2.2870646417e-01
       coeff_l1(7,8)= -5.5903118849e-01
       coeff_l1(7,9)=  3.2911416888e-01
       coeff_l1(7,10)= -2.7808293700e-01
       coeff_l1(7,11)= -1.9289968908e-01
       coeff_l1(7,12)= -1.2798978388e-01
       coeff_l1(7,13)= -1.2124085426e+00
       coeff_l1(7,14)= -4.8376995325e-01
       coeff_l1(7,15)= -3.5315692425e-01
       coeff_l1(7,16)=  9.4703185558e-01
       coeff_l1(7,17)=  2.0483036339e-01
       coeff_l1(7,18)=  1.2572189793e-02
       coeff_l1(7,19)= -3.0649438500e-01
       coeff_l1(7,20)=  5.8266574144e-01
       coeff_l1(7,21)=  1.4668605328e+00
       coeff_l1(7,22)=  1.0134893656e+00
       coeff_l1(7,23)=  2.9074928164e-01
       coeff_l1(7,24)= -7.3775476217e-01
       coeff_l1(7,25)=  8.0378122628e-02
       coeff_l1(7,26)=  5.7258969545e-01
       coeff_l1(7,27)= -6.8409323692e-01
       coeff_l1(7,28)=  2.5932592154e-01
       coeff_l1(7,29)=  1.1456241459e-01
       coeff_l1(7,30)= -3.3583825827e-01
       coeff_l1(7,31)=  2.0236430168e+00
       coeff_l1(7,32)=  4.6211540699e-01
       coeff_l1(7,33)= -2.2323717177e-01
       coeff_l1(7,34)= -3.6413505077e+00
       coeff_l1(7,35)= -2.3785682023e-01
       coeff_l1(7,36)= -2.0062014461e-01
       coeff_l1(8,1)=  4.8132490367e-02
       coeff_l1(8,2)=  2.6630410552e-01
       coeff_l1(8,3)= -1.6266113520e-01
       coeff_l1(8,4)=  9.7291320562e-02
       coeff_l1(8,5)= -1.3823169470e-01
       coeff_l1(8,6)= -1.2810017169e-01
       coeff_l1(8,7)= -5.4618138820e-02
       coeff_l1(8,8)= -1.6781926155e-01
       coeff_l1(8,9)= -8.5961334407e-02
       coeff_l1(8,10)=  2.4678368866e-01
       coeff_l1(8,11)=  3.3398231864e-01
       coeff_l1(8,12)=  7.5518101454e-02
       coeff_l1(8,13)= -4.3027383089e-01
       coeff_l1(8,14)=  9.3397505581e-02
       coeff_l1(8,15)=  8.5360035300e-02
       coeff_l1(8,16)= -2.1238756180e-01
       coeff_l1(8,17)= -3.1100443006e-01
       coeff_l1(8,18)=  1.3045945764e-01
       coeff_l1(8,19)= -4.2829215527e-01
       coeff_l1(8,20)=  1.2929135561e-01
       coeff_l1(8,21)=  6.2411591411e-02
       coeff_l1(8,22)=  1.8241627142e-02
       coeff_l1(8,23)=  2.5640693307e-01
       coeff_l1(8,24)=  9.4320848584e-02
       coeff_l1(8,25)=  2.9568612576e-01
       coeff_l1(8,26)= -2.6770991087e-01
       coeff_l1(8,27)=  2.4181471765e-01
       coeff_l1(8,28)= -1.9757826626e-01
       coeff_l1(8,29)=  2.8688112274e-02
       coeff_l1(8,30)= -2.6433190703e-01
       coeff_l1(8,31)=  2.1098920703e-01
       coeff_l1(8,32)= -2.3104511201e-01
       coeff_l1(8,33)= -2.2677826881e-01
       coeff_l1(8,34)= -3.9615145326e-01
       coeff_l1(8,35)=  2.2230485454e-02
       coeff_l1(8,36)= -1.3508055359e-02
       coeff_l1(9,1)= -4.3611422181e-01
       coeff_l1(9,2)=  1.0387523174e+00
       coeff_l1(9,3)=  4.0430980921e-01
       coeff_l1(9,4)= -5.5463176221e-02
       coeff_l1(9,5)=  2.8418865800e-01
       coeff_l1(9,6)=  3.1486442685e-01
       coeff_l1(9,7)=  1.2963658571e+00
       coeff_l1(9,8)= -4.5572406054e-01
       coeff_l1(9,9)= -1.0162901878e+00
       coeff_l1(9,10)= -9.9550789595e-01
       coeff_l1(9,11)= -3.1033715606e-01
       coeff_l1(9,12)= -6.4544212818e-01
       coeff_l1(9,13)=  9.8023146391e-02
       coeff_l1(9,14)= -8.3813446760e-01
       coeff_l1(9,15)=  5.0937491655e-01
       coeff_l1(9,16)=  3.7940364331e-02
       coeff_l1(9,17)= -9.4192725420e-01
       coeff_l1(9,18)= -1.1866955757e+00
       coeff_l1(9,19)= -1.0576539040e+00
       coeff_l1(9,20)= -8.1384551525e-01
       coeff_l1(9,21)= -1.8095811605e+00
       coeff_l1(9,22)=  5.3720653057e-01
       coeff_l1(9,23)=  4.6812415123e-01
       coeff_l1(9,24)=  3.3660626411e-01
       coeff_l1(9,25)= -4.3645331264e-01
       coeff_l1(9,26)= -6.3067847490e-01
       coeff_l1(9,27)= -3.0869975686e-01
       coeff_l1(9,28)= -1.0426585674e+00
       coeff_l1(9,29)= -1.5670126677e-01
       coeff_l1(9,30)= -7.7959185839e-01
       coeff_l1(9,31)=  3.9390686154e-01
       coeff_l1(9,32)=  9.7668713331e-01
       coeff_l1(9,33)= -8.0035918951e-01
       coeff_l1(9,34)= -5.7889240980e-01
       coeff_l1(9,35)= -9.8953366280e-01
       coeff_l1(9,36)= -2.7210971713e-01
       coeff_l1(10,1)= -1.9592246413e-01
       coeff_l1(10,2)=  2.5618538260e-01
       coeff_l1(10,3)=  2.2253724933e-01
       coeff_l1(10,4)= -2.3266720772e-01
       coeff_l1(10,5)= -9.2569682747e-03
       coeff_l1(10,6)=  1.6822995245e-01
       coeff_l1(10,7)= -2.9524299502e-01
       coeff_l1(10,8)=  1.6437570751e-01
       coeff_l1(10,9)=  2.0293661952e-01
       coeff_l1(10,10)= -3.8166889548e-01
       coeff_l1(10,11)= -1.6434912384e-01
       coeff_l1(10,12)=  1.3551886380e-01
       coeff_l1(10,13)= -7.7163141966e-01
       coeff_l1(10,14)=  1.8480283022e-01
       coeff_l1(10,15)= -1.9833976030e-01
       coeff_l1(10,16)=  2.1948298812e-01
       coeff_l1(10,17)= -2.6986452937e-01
       coeff_l1(10,18)=  3.2650107145e-01
       coeff_l1(10,19)= -5.4807740450e-01
       coeff_l1(10,20)=  3.9683121443e-01
       coeff_l1(10,21)=  2.3893126845e-01
       coeff_l1(10,22)= -2.8797000647e-01
       coeff_l1(10,23)= -1.1120817065e-01
       coeff_l1(10,24)= -4.5367538929e-01
       coeff_l1(10,25)= -5.2630446851e-02
       coeff_l1(10,26)= -5.0471991301e-01
       coeff_l1(10,27)=  6.7956030369e-02
       coeff_l1(10,28)=  2.4720225483e-03
       coeff_l1(10,29)= -2.1400724351e-01
       coeff_l1(10,30)=  2.4385969341e-01
       coeff_l1(10,31)= -1.4363546669e-01
       coeff_l1(10,32)= -3.6604250781e-03
       coeff_l1(10,33)= -3.7234619260e-01
       coeff_l1(10,34)= -1.1617736816e+00
       coeff_l1(10,35)= -1.0714028776e-01
       coeff_l1(10,36)= -2.7300858498e-01
       coeff_l1(11,1)= -5.6122106314e-01
       coeff_l1(11,2)=  3.7068591118e+00
       coeff_l1(11,3)= -2.3140423000e-01
       coeff_l1(11,4)= -4.5895713568e-01
       coeff_l1(11,5)=  3.4013059735e-01
       coeff_l1(11,6)= -3.8679304719e-01
       coeff_l1(11,7)=  1.9860323668e+00
       coeff_l1(11,8)= -2.2944384813e-01
       coeff_l1(11,9)=  6.4193212986e-01
       coeff_l1(11,10)=  6.7823059857e-02
       coeff_l1(11,11)= -5.1702636480e-01
       coeff_l1(11,12)= -3.0644696951e-01
       coeff_l1(11,13)= -8.8996961713e-02
       coeff_l1(11,14)= -7.2186094522e-01
       coeff_l1(11,15)= -7.4319265783e-02
       coeff_l1(11,16)=  6.7365109921e-01
       coeff_l1(11,17)=  1.3015161455e-01
       coeff_l1(11,18)= -6.5190321207e-01
       coeff_l1(11,19)= -6.3854694366e-02
       coeff_l1(11,20)=  1.6838015616e-01
       coeff_l1(11,21)=  6.0871994495e-01
       coeff_l1(11,22)=  2.4980511665e+00
       coeff_l1(11,23)= -3.9081284404e-01
       coeff_l1(11,24)= -3.0942821503e-01
       coeff_l1(11,25)= -4.2510330677e-03
       coeff_l1(11,26)=  2.3710253835e-01
       coeff_l1(11,27)=  1.7180681229e-01
       coeff_l1(11,28)=  1.6974791884e-01
       coeff_l1(11,29)=  9.2718280852e-02
       coeff_l1(11,30)= -3.4256955981e-01
       coeff_l1(11,31)=  9.7063410282e-01
       coeff_l1(11,32)=  1.7225332260e+00
       coeff_l1(11,33)= -2.9968091846e-01
       coeff_l1(11,34)= -3.1036503315e+00
       coeff_l1(11,35)=  2.0552241802e-01
       coeff_l1(11,36)=  1.8127378076e-02
       coeff_l1(12,1)=  1.8553847075e-01
       coeff_l1(12,2)=  6.3024270535e-01
       coeff_l1(12,3)=  2.1715726703e-02
       coeff_l1(12,4)= -1.3274358213e-01
       coeff_l1(12,5)= -1.0312976837e+00
       coeff_l1(12,6)= -5.7026022673e-01
       coeff_l1(12,7)= -1.1727668047e+00
       coeff_l1(12,8)=  8.4072959423e-01
       coeff_l1(12,9)= -7.3831862211e-01
       coeff_l1(12,10)=  4.4568711519e-01
       coeff_l1(12,11)= -2.2144001722e-01
       coeff_l1(12,12)= -1.8839727342e-01
       coeff_l1(12,13)=  6.0117512941e-01
       coeff_l1(12,14)=  4.6129307151e-01
       coeff_l1(12,15)= -3.1963983178e-01
       coeff_l1(12,16)= -9.7898405790e-01
       coeff_l1(12,17)=  9.3906933069e-01
       coeff_l1(12,18)=  8.6547785997e-01
       coeff_l1(12,19)=  3.4005162120e-01
       coeff_l1(12,20)= -2.0060846210e-01
       coeff_l1(12,21)= -1.3915817440e-01
       coeff_l1(12,22)= -5.5524098873e-01
       coeff_l1(12,23)= -6.0865348577e-01
       coeff_l1(12,24)=  1.2445179224e+00
       coeff_l1(12,25)=  9.3688786030e-01
       coeff_l1(12,26)= -1.1363990307e+00
       coeff_l1(12,27)= -3.4873747826e-01
       coeff_l1(12,28)= -2.8374567628e-01
       coeff_l1(12,29)= -1.3687118888e-01
       coeff_l1(12,30)= -1.6939390451e-02
       coeff_l1(12,31)=  3.7688052654e-01
       coeff_l1(12,32)=  5.4042655230e-01
       coeff_l1(12,33)= -1.5240350962e+00
       coeff_l1(12,34)=  1.4271792173e+00
       coeff_l1(12,35)= -3.8495931774e-02
       coeff_l1(12,36)= -1.0195432901e+00
       coeff_l1(13,1)= -2.5119358301e-01
       coeff_l1(13,2)=  4.2415079474e-01
       coeff_l1(13,3)=  3.1170347705e-02
       coeff_l1(13,4)=  1.6387107968e-01
       coeff_l1(13,5)= -3.6754047871e-01
       coeff_l1(13,6)= -4.1305902600e-01
       coeff_l1(13,7)= -6.6463589668e-01
       coeff_l1(13,8)=  3.4626618028e-01
       coeff_l1(13,9)= -3.0972811580e-01
       coeff_l1(13,10)= -3.0594399571e-01
       coeff_l1(13,11)= -4.8729804158e-01
       coeff_l1(13,12)= -2.4806694686e-01
       coeff_l1(13,13)= -3.9319890738e-01
       coeff_l1(13,14)= -2.3877757788e-01
       coeff_l1(13,15)= -1.4785836637e-01
       coeff_l1(13,16)= -5.7308369875e-01
       coeff_l1(13,17)= -5.8615940809e-01
       coeff_l1(13,18)= -5.9969180822e-01
       coeff_l1(13,19)= -4.7503808141e-01
       coeff_l1(13,20)= -1.4091151953e-01
       coeff_l1(13,21)= -1.8318769336e-01
       coeff_l1(13,22)= -5.3679132462e-01
       coeff_l1(13,23)= -2.0585097373e-01
       coeff_l1(13,24)= -5.2004659176e-01
       coeff_l1(13,25)= -2.5292956829e-01
       coeff_l1(13,26)= -3.7762817740e-01
       coeff_l1(13,27)=  1.9710145891e-01
       coeff_l1(13,28)= -2.8262274340e-02
       coeff_l1(13,29)= -5.9751254320e-01
       coeff_l1(13,30)= -2.2391112149e-01
       coeff_l1(13,31)= -1.9267651439e-01
       coeff_l1(13,32)= -3.2065108418e-01
       coeff_l1(13,33)= -1.1481015682e+00
       coeff_l1(13,34)= -5.6398659945e-01
       coeff_l1(13,35)= -2.9560330510e-01
       coeff_l1(13,36)= -3.6915233731e-01
       coeff_l1(14,1)= -4.2571496218e-02
       coeff_l1(14,2)=  3.2162401080e-01
       coeff_l1(14,3)= -1.3835215569e-01
       coeff_l1(14,4)=  1.4815889299e-02
       coeff_l1(14,5)=  2.2639188170e-01
       coeff_l1(14,6)= -2.3354487121e-01
       coeff_l1(14,7)= -1.6680430621e-02
       coeff_l1(14,8)=  3.0425688624e-01
       coeff_l1(14,9)=  2.3995061219e-01
       coeff_l1(14,10)= -2.3207549751e-01
       coeff_l1(14,11)= -2.7244588360e-02
       coeff_l1(14,12)= -3.2078447938e-01
       coeff_l1(14,13)= -5.0613945723e-01
       coeff_l1(14,14)= -5.2948474884e-02
       coeff_l1(14,15)= -1.2547978759e-01
       coeff_l1(14,16)= -1.6745786369e-01
       coeff_l1(14,17)=  9.1800339520e-02
       coeff_l1(14,18)=  3.6773207784e-01
       coeff_l1(14,19)=  3.0103338882e-02
       coeff_l1(14,20)=  3.9186677337e-01
       coeff_l1(14,21)= -3.1274157763e-01
       coeff_l1(14,22)= -1.1001300067e-01
       coeff_l1(14,23)=  4.8681661487e-02
       coeff_l1(14,24)= -2.6875856519e-01
       coeff_l1(14,25)= -2.5298792124e-01
       coeff_l1(14,26)= -5.0042510033e-01
       coeff_l1(14,27)=  7.6261378825e-02
       coeff_l1(14,28)=  2.4984939396e-01
       coeff_l1(14,29)= -6.1664585024e-02
       coeff_l1(14,30)= -2.5990507007e-01
       coeff_l1(14,31)=  2.0223870873e-02
       coeff_l1(14,32)=  5.8943301439e-01
       coeff_l1(14,33)= -1.7763987184e-01
       coeff_l1(14,34)= -6.6128069162e-01
       coeff_l1(14,35)= -2.5016510487e-01
       coeff_l1(14,36)= -1.8525022268e-01
       coeff_l1(15,1)= -4.3341219425e-02
       coeff_l1(15,2)= -1.0427437723e-01
       coeff_l1(15,3)= -7.7338397503e-01
       coeff_l1(15,4)=  5.5848285556e-02
       coeff_l1(15,5)= -1.9207719564e+00
       coeff_l1(15,6)= -2.7414759994e-01
       coeff_l1(15,7)= -1.4153287411e+00
       coeff_l1(15,8)=  9.2393320799e-01
       coeff_l1(15,9)= -1.4715338945e+00
       coeff_l1(15,10)= -5.1350128651e-01
       coeff_l1(15,11)= -7.6617872715e-01
       coeff_l1(15,12)=  1.3699804544e+00
       coeff_l1(15,13)= -1.5169644356e+00
       coeff_l1(15,14)= -7.3928570747e-01
       coeff_l1(15,15)= -4.5103353262e-01
       coeff_l1(15,16)= -2.0678672194e-01
       coeff_l1(15,17)= -8.4158591926e-02
       coeff_l1(15,18)= -2.5205940008e-02
       coeff_l1(15,19)= -6.7977416515e-01
       coeff_l1(15,20)=  7.9245561361e-01
       coeff_l1(15,21)= -2.7961668968e+00
       coeff_l1(15,22)= -2.7753391862e-01
       coeff_l1(15,23)= -2.4660015106e+00
       coeff_l1(15,24)= -3.5184912384e-02
       coeff_l1(15,25)= -3.9756104350e-01
       coeff_l1(15,26)= -2.4752850533e+00
       coeff_l1(15,27)= -7.9799872637e-01
       coeff_l1(15,28)= -9.7706025839e-01
       coeff_l1(15,29)= -7.0708698034e-01
       coeff_l1(15,30)= -1.5807131529e+00
       coeff_l1(15,31)=  3.4218454361e-01
       coeff_l1(15,32)=  4.8715583980e-02
       coeff_l1(15,33)= -2.2014248371e+00
       coeff_l1(15,34)= -4.0536594391e-01
       coeff_l1(15,35)= -6.3340502977e-01
       coeff_l1(15,36)=  5.6646710634e-01
       coeff_l1(16,1)=  9.2671924829e-01
       coeff_l1(16,2)= -2.8777956963e-01
       coeff_l1(16,3)= -1.0752073526e+00
       coeff_l1(16,4)= -6.6385966539e-01
       coeff_l1(16,5)= -9.5106996596e-02
       coeff_l1(16,6)= -6.6313940287e-01
       coeff_l1(16,7)=  1.8317927122e+00
       coeff_l1(16,8)= -2.1490557492e-01
       coeff_l1(16,9)=  3.8498055935e-01
       coeff_l1(16,10)= -9.0697526932e-01
       coeff_l1(16,11)= -7.9594165087e-01
       coeff_l1(16,12)= -2.4016125500e-01
       coeff_l1(16,13)= -3.5519359112e+00
       coeff_l1(16,14)=  3.2248176634e-02
       coeff_l1(16,15)= -2.7034198865e-02
       coeff_l1(16,16)=  1.5977672338e+00
       coeff_l1(16,17)= -1.1955256462e+00
       coeff_l1(16,18)=  1.1974745989e+00
       coeff_l1(16,19)= -7.2880500555e-01
       coeff_l1(16,20)=  1.4457200468e-01
       coeff_l1(16,21)=  1.2446038723e+00
       coeff_l1(16,22)= -1.9438275695e-01
       coeff_l1(16,23)=  8.5784363747e-01
       coeff_l1(16,24)= -1.6741735935e+00
       coeff_l1(16,25)= -6.2903141975e-01
       coeff_l1(16,26)= -1.8352333307e+00
       coeff_l1(16,27)=  1.0562784225e-01
       coeff_l1(16,28)= -8.7226843834e-01
       coeff_l1(16,29)=  2.1904790774e-02
       coeff_l1(16,30)= -9.3890261650e-01
       coeff_l1(16,31)= -1.7187571526e+00
       coeff_l1(16,32)=  1.9005651772e-01
       coeff_l1(16,33)= -1.5237371922e+00
       coeff_l1(16,34)= -8.5747641325e-01
       coeff_l1(16,35)= -2.5623375177e-01
       coeff_l1(16,36)=  1.8927918375e-01
       coeff_l1(17,1)=  1.9987146556e-01
       coeff_l1(17,2)=  2.1896579266e+00
       coeff_l1(17,3)= -5.1873517036e-01
       coeff_l1(17,4)=  7.9098798335e-02
       coeff_l1(17,5)=  6.8760402501e-02
       coeff_l1(17,6)=  1.7230750322e+00
       coeff_l1(17,7)=  2.9606941342e-01
       coeff_l1(17,8)= -3.7884074450e-01
       coeff_l1(17,9)=  3.4697372466e-02
       coeff_l1(17,10)=  7.8831322491e-02
       coeff_l1(17,11)=  1.2615998089e-01
       coeff_l1(17,12)= -4.0267840028e-01
       coeff_l1(17,13)= -1.6096279025e-01
       coeff_l1(17,14)= -3.4575083852e-01
       coeff_l1(17,15)=  2.1843367815e-01
       coeff_l1(17,16)=  5.2838230133e-01
       coeff_l1(17,17)= -7.5922973454e-02
       coeff_l1(17,18)= -3.4676203132e-01
       coeff_l1(17,19)= -5.3136064671e-03
       coeff_l1(17,20)=  5.5422383547e-01
       coeff_l1(17,21)= -2.5845760107e-01
       coeff_l1(17,22)=  3.6861734390e+00
       coeff_l1(17,23)=  7.5818330050e-01
       coeff_l1(17,24)= -3.4753108025e-01
       coeff_l1(17,25)= -4.6713951230e-01
       coeff_l1(17,26)=  6.5325751901e-02
       coeff_l1(17,27)=  5.8157891035e-01
       coeff_l1(17,28)=  5.3020364046e-01
       coeff_l1(17,29)= -9.2673426867e-01
       coeff_l1(17,30)= -8.4474825859e-01
       coeff_l1(17,31)= -8.1875503063e-01
       coeff_l1(17,32)=  1.9725815058e+00
       coeff_l1(17,33)= -1.7020581663e-01
       coeff_l1(17,34)= -2.4679236412e+00
       coeff_l1(17,35)= -1.7358697951e-01
       coeff_l1(17,36)=  8.3637535572e-02
       coeff_l1(18,1)= -4.2581525445e-01
       coeff_l1(18,2)=  7.7425813675e-01
       coeff_l1(18,3)= -5.7094484568e-01
       coeff_l1(18,4)= -1.8434166908e+00
       coeff_l1(18,5)= -5.3017115593e-01
       coeff_l1(18,6)=  1.0842863321e+00
       coeff_l1(18,7)= -2.5841431618e+00
       coeff_l1(18,8)= -4.7291848063e-01
       coeff_l1(18,9)= -5.9453260899e-01
       coeff_l1(18,10)= -7.4375307560e-01
       coeff_l1(18,11)= -3.6350008845e-01
       coeff_l1(18,12)= -8.0608844757e-01
       coeff_l1(18,13)=  5.6237375736e-01
       coeff_l1(18,14)= -3.4205368161e-01
       coeff_l1(18,15)= -4.9403455853e-01
       coeff_l1(18,16)=  7.2041940689e-01
       coeff_l1(18,17)= -6.8886590004e-01
       coeff_l1(18,18)= -8.5988783836e-01
       coeff_l1(18,19)= -7.8518080711e-01
       coeff_l1(18,20)= -2.8816455603e-01
       coeff_l1(18,21)= -2.6945459843e+00
       coeff_l1(18,22)=  2.5349366665e+00
       coeff_l1(18,23)= -6.1208516359e-01
       coeff_l1(18,24)= -1.0260627270e+00
       coeff_l1(18,25)= -6.6297113895e-01
       coeff_l1(18,26)= -2.6918458939e-01
       coeff_l1(18,27)= -1.9058126211e-01
       coeff_l1(18,28)= -1.8183299899e-01
       coeff_l1(18,29)=  4.8674233258e-02
       coeff_l1(18,30)= -4.3902564049e-01
       coeff_l1(18,31)= -4.4887664914e-01
       coeff_l1(18,32)=  1.5030429363e+00
       coeff_l1(18,33)= -1.1585854292e+00
       coeff_l1(18,34)= -2.6810901165e+00
       coeff_l1(18,35)= -5.6226414442e-01
       coeff_l1(18,36)= -1.3652110100e-01
       coeff_l1(19,1)=  9.3170398474e-01
       coeff_l1(19,2)= -1.0857548714e+00
       coeff_l1(19,3)= -8.3891987801e-01
       coeff_l1(19,4)= -6.7148989439e-01
       coeff_l1(19,5)=  2.3985756934e-01
       coeff_l1(19,6)= -3.8744738698e-01
       coeff_l1(19,7)=  1.0001623631e+00
       coeff_l1(19,8)= -1.9705481529e+00
       coeff_l1(19,9)=  1.6507394314e+00
       coeff_l1(19,10)= -5.0779855251e-01
       coeff_l1(19,11)= -1.1236838102e+00
       coeff_l1(19,12)= -2.1604068279e+00
       coeff_l1(19,13)= -5.5210727453e-01
       coeff_l1(19,14)=  1.3642677069e+00
       coeff_l1(19,15)= -1.1804006100e+00
       coeff_l1(19,16)=  9.5553845167e-01
       coeff_l1(19,17)=  1.7814141512e-01
       coeff_l1(19,18)=  2.6543035638e-03
       coeff_l1(19,19)= -8.1295150518e-01
       coeff_l1(19,20)= -1.8783216476e+00
       coeff_l1(19,21)=  1.9945744276e+00
       coeff_l1(19,22)=  3.2940867543e-01
       coeff_l1(19,23)= -1.6671247780e-01
       coeff_l1(19,24)= -3.6810553074e+00
       coeff_l1(19,25)= -1.2188268900e+00
       coeff_l1(19,26)=  3.2435250282e-01
       coeff_l1(19,27)= -3.7728449702e-01
       coeff_l1(19,28)=  1.4115627110e-01
       coeff_l1(19,29)=  5.1510816813e-01
       coeff_l1(19,30)=  3.6931613088e-01
       coeff_l1(19,31)= -1.4056044817e+00
       coeff_l1(19,32)= -1.0057210922e-01
       coeff_l1(19,33)=  5.9756898880e-01
       coeff_l1(19,34)= -1.4134569168e+00
       coeff_l1(19,35)= -4.5937699080e-01
       coeff_l1(19,36)=  5.8054637909e-01
       coeff_l1(20,1)= -3.7407952547e-01
       coeff_l1(20,2)=  2.1820081770e-01
       coeff_l1(20,3)= -1.3723616600e+00
       coeff_l1(20,4)= -4.6688985825e-01
       coeff_l1(20,5)= -3.4255474806e-01
       coeff_l1(20,6)=  8.3700425923e-02
       coeff_l1(20,7)= -7.9820507765e-01
       coeff_l1(20,8)=  2.5677919388e-01
       coeff_l1(20,9)= -6.2240850925e-01
       coeff_l1(20,10)=  1.2297374010e+00
       coeff_l1(20,11)=  2.8116017580e-01
       coeff_l1(20,12)=  2.9618617892e-01
       coeff_l1(20,13)= -2.2067523003e+00
       coeff_l1(20,14)= -1.6672259569e-01
       coeff_l1(20,15)=  3.7951901555e-02
       coeff_l1(20,16)=  6.8299645185e-01
       coeff_l1(20,17)= -8.5267251730e-01
       coeff_l1(20,18)=  9.6530747414e-01
       coeff_l1(20,19)=  7.2811008431e-03
       coeff_l1(20,20)=  1.3022720814e+00
       coeff_l1(20,21)=  4.8131486773e-01
       coeff_l1(20,22)= -1.2247813493e-01
       coeff_l1(20,23)= -1.5581795573e-01
       coeff_l1(20,24)=  6.9800651073e-01
       coeff_l1(20,25)= -5.1309192181e-01
       coeff_l1(20,26)=  3.9296305180e-01
       coeff_l1(20,27)= -4.4679597020e-01
       coeff_l1(20,28)=  3.8048750162e-01
       coeff_l1(20,29)= -2.0115788281e-01
       coeff_l1(20,30)= -3.6620616913e-01
       coeff_l1(20,31)=  2.6644909382e-01
       coeff_l1(20,32)= -7.3125936091e-02
       coeff_l1(20,33)= -8.0467379093e-01
       coeff_l1(20,34)=  7.0772254467e-01
       coeff_l1(20,35)=  2.3724935949e-01
       coeff_l1(20,36)= -2.6185867190e-01
       coeff_l1(21,1)= -4.4615674019e-01
       coeff_l1(21,2)=  2.0791585445e+00
       coeff_l1(21,3)= -5.2251195908e-01
       coeff_l1(21,4)=  5.5120635033e-01
       coeff_l1(21,5)= -2.6808238029e-01
       coeff_l1(21,6)= -1.1545442045e-01
       coeff_l1(21,7)=  1.2893904448e+00
       coeff_l1(21,8)= -5.6162846088e-01
       coeff_l1(21,9)= -1.2999618053e-01
       coeff_l1(21,10)= -1.3413041830e-01
       coeff_l1(21,11)= -7.9985454679e-02
       coeff_l1(21,12)= -3.4752047062e-01
       coeff_l1(21,13)=  1.0759592056e-02
       coeff_l1(21,14)= -6.8087035418e-01
       coeff_l1(21,15)=  8.2512691617e-02
       coeff_l1(21,16)=  8.6677908897e-02
       coeff_l1(21,17)= -6.4252430201e-01
       coeff_l1(21,18)= -1.1597449780e+00
       coeff_l1(21,19)= -9.7823157907e-02
       coeff_l1(21,20)=  2.7474561334e-01
       coeff_l1(21,21)=  1.6262997687e-01
       coeff_l1(21,22)=  4.4061646461e+00
       coeff_l1(21,23)=  3.7297993898e-02
       coeff_l1(21,24)= -8.4694540501e-01
       coeff_l1(21,25)= -9.9185355008e-02
       coeff_l1(21,26)= -3.0593064427e-01
       coeff_l1(21,27)=  1.5761789680e-01
       coeff_l1(21,28)= -1.5633124113e-01
       coeff_l1(21,29)= -1.0035350323e+00
       coeff_l1(21,30)= -4.6251818538e-01
       coeff_l1(21,31)=  1.3267672062e-01
       coeff_l1(21,32)=  2.5493514538e+00
       coeff_l1(21,33)=  2.9504401609e-02
       coeff_l1(21,34)= -2.8669688702e+00
       coeff_l1(21,35)= -2.4951466918e-01
       coeff_l1(21,36)=  8.2809954882e-02
!coeff layer 2
       coeff_l2(1,1)= -6.0214847326e-02
       coeff_l2(1,2)= -2.4829465151e-01
       coeff_l2(1,3)= -1.3944929838e-01
       coeff_l2(1,4)= -3.2379689217e+00
       coeff_l2(1,5)=  3.8047397137e-01
       coeff_l2(1,6)= -5.4051343352e-02
       coeff_l2(1,7)=  6.3644990325e-02
       coeff_l2(1,8)=  1.3846759498e-01
       coeff_l2(1,9)=  3.9460593462e-01
       coeff_l2(1,10)=  5.9607261419e-01
       coeff_l2(1,11)= -4.4592864811e-02
       coeff_l2(1,12)=  3.3198952675e-01
       coeff_l2(1,13)= -6.1174994707e-01
       coeff_l2(1,14)=  4.0855330229e-01
       coeff_l2(1,15)= -7.8790926933e-01
       coeff_l2(1,16)= -1.0534534454e+00
       coeff_l2(1,17)= -3.4895113111e-01
       coeff_l2(1,18)= -8.8136643171e-01
       coeff_l2(1,19)= -5.4011029005e-01
       coeff_l2(1,20)=  8.3529651165e-01
       coeff_l2(1,21)=  1.8898460269e-01
       coeff_l2(1,22)= -4.0217757225e-01
       coeff_l2(1,23)= -1.3539739847e+00
       coeff_l2(1,24)=  1.4786607027e-01
       coeff_l2(1,25)= -2.7248868346e-01
       coeff_l2(1,26)= -4.5823377371e-01
       coeff_l2(1,27)=  2.0051489770e-01
       coeff_l2(1,28)= -2.7140319347e-01
       coeff_l2(1,29)= -1.1568694562e-01
       coeff_l2(1,30)= -1.1135645956e-01
       coeff_l2(1,31)=  2.1708066761e-01
       coeff_l2(1,32)=  2.9764196277e-01
       coeff_l2(1,33)=  7.3350048065e-01
       coeff_l2(1,34)=  6.6384983063e-01
       coeff_l2(1,35)= -2.9829326272e-01
       coeff_l2(1,36)= -6.0289108753e-01
       coeff_l2(1,37)= -9.8775342107e-02
       coeff_l2(1,38)= -3.8707011938e-01
       coeff_l2(1,39)=  2.9790815711e-01
       coeff_l2(1,40)= -3.6570581794e-01
       coeff_l2(1,41)= -1.8246696889e-01
       coeff_l2(1,42)= -1.7853094637e-01
       coeff_l2(1,43)=  2.8612768650e-01
       coeff_l2(1,44)=  2.2756345570e-01
       coeff_l2(1,45)= -3.8898456097e-01
       coeff_l2(1,46)= -1.2815840542e-01
       coeff_l2(1,47)=  2.1109549701e-01
       coeff_l2(1,48)= -2.2337526083e-01
       coeff_l2(1,49)=  3.5389786959e-01
       coeff_l2(1,50)= -1.1133303493e-01
       coeff_l2(1,51)=  3.6472478509e-01
       coeff_l2(1,52)= -3.7673553824e-01
       coeff_l2(1,53)= -2.9371479154e-01
       coeff_l2(1,54)= -5.4214602709e-01
       coeff_l2(1,55)=  3.1498856843e-02
       coeff_l2(1,56)=  1.3616406918e+00
       coeff_l2(2,1)= -1.8206171691e-01
       coeff_l2(2,2)= -1.7164978385e-01
       coeff_l2(2,3)=  3.7786138058e-01
       coeff_l2(2,4)=  5.0863653421e-01
       coeff_l2(2,5)=  2.9093018174e-01
       coeff_l2(2,6)= -1.0661461353e+00
       coeff_l2(2,7)= -2.1306235790e+00
       coeff_l2(2,8)= -3.7400195003e-01
       coeff_l2(2,9)=  1.1659269780e-01
       coeff_l2(2,10)=  4.3718487024e-01
       coeff_l2(2,11)= -1.1219829321e-01
       coeff_l2(2,12)= -2.4549348652e-01
       coeff_l2(2,13)=  2.0149819851e+00
       coeff_l2(2,14)=  7.7852763236e-02
       coeff_l2(2,15)= -1.2040500641e+00
       coeff_l2(2,16)= -1.3042792082e+00
       coeff_l2(2,17)= -1.2347677946e+00
       coeff_l2(2,18)=  1.2204518318e+00
       coeff_l2(2,19)= -5.9454121590e+00
       coeff_l2(2,20)= -2.0715723038e+00
       coeff_l2(2,21)= -1.4806969166e+00
       coeff_l2(2,22)= -2.0678575039e+00
       coeff_l2(2,23)= -2.0404603481e+00
       coeff_l2(2,24)=  1.4132513106e-01
       coeff_l2(2,25)=  1.2729945779e-01
       coeff_l2(2,26)=  2.9615029693e-01
       coeff_l2(2,27)= -1.3179746866e+00
       coeff_l2(2,28)=  9.1843861341e-01
       coeff_l2(2,29)= -6.8569099903e-01
       coeff_l2(2,30)=  1.1464414597e+00
       coeff_l2(2,31)= -1.3374761343e+00
       coeff_l2(2,32)= -9.6342730522e-01
       coeff_l2(2,33)=  2.4805815518e-01
       coeff_l2(2,34)= -2.2998952866e-01
       coeff_l2(2,35)= -1.6141337156e+00
       coeff_l2(2,36)=  5.0812542439e-01
       coeff_l2(2,37)=  1.8453530073e+00
       coeff_l2(2,38)= -2.6551660895e-01
       coeff_l2(2,39)= -9.9055022001e-01
       coeff_l2(2,40)=  4.3399295211e-01
       coeff_l2(2,41)=  1.2836226225e+00
       coeff_l2(2,42)= -1.0833451748e+00
       coeff_l2(2,43)= -4.3415108323e-01
       coeff_l2(2,44)= -5.0894761086e-01
       coeff_l2(2,45)= -3.1219837666e+00
       coeff_l2(2,46)= -4.1577529907e+00
       coeff_l2(2,47)= -4.0595802665e-01
       coeff_l2(2,48)= -1.3679962158e+00
       coeff_l2(2,49)= -3.0592885017e+00
       coeff_l2(2,50)=  2.5310134888e+00
       coeff_l2(2,51)=  4.7584068775e-01
       coeff_l2(2,52)= -5.5339223146e-01
       coeff_l2(2,53)= -1.5092074871e+00
       coeff_l2(2,54)= -1.4832575321e+00
       coeff_l2(2,55)= -4.9575588107e-01
       coeff_l2(2,56)= -9.7904813290e-01
       coeff_l2(3,1)=  8.3943732083e-02
       coeff_l2(3,2)=  7.1975253522e-02
       coeff_l2(3,3)=  5.1041554660e-02
       coeff_l2(3,4)=  5.5905926228e-01
       coeff_l2(3,5)=  4.7336596251e-01
       coeff_l2(3,6)= -1.1081655025e+00
       coeff_l2(3,7)= -2.2780558467e-01
       coeff_l2(3,8)= -2.0158074796e-01
       coeff_l2(3,9)=  3.3191896975e-02
       coeff_l2(3,10)=  2.4242241681e-01
       coeff_l2(3,11)= -1.5915185213e-01
       coeff_l2(3,12)=  3.8557714224e-01
       coeff_l2(3,13)= -5.9033489227e-01
       coeff_l2(3,14)= -4.5658390969e-02
       coeff_l2(3,15)=  2.0516121387e-01
       coeff_l2(3,16)= -8.7860083580e-01
       coeff_l2(3,17)= -7.8819721937e-01
       coeff_l2(3,18)= -2.0735716820e+00
       coeff_l2(3,19)= -1.0667178631e+00
       coeff_l2(3,20)=  8.1805028021e-02
       coeff_l2(3,21)= -2.9159389436e-02
       coeff_l2(3,22)= -9.9094671011e-01
       coeff_l2(3,23)= -1.3185553551e+00
       coeff_l2(3,24)=  1.5832775831e-01
       coeff_l2(3,25)=  5.0423777103e-01
       coeff_l2(3,26)= -4.9524649978e-01
       coeff_l2(3,27)=  1.1108539999e-01
       coeff_l2(3,28)= -4.1191151738e-01
       coeff_l2(3,29)=  2.7677777410e-01
       coeff_l2(3,30)= -8.2145220041e-01
       coeff_l2(3,31)= -4.9174454063e-02
       coeff_l2(3,32)=  3.1912231445e-01
       coeff_l2(3,33)=  1.3673216105e-02
       coeff_l2(3,34)=  2.7093318105e-01
       coeff_l2(3,35)= -7.1377633139e-03
       coeff_l2(3,36)= -2.1175464988e-01
       coeff_l2(3,37)=  3.7640741467e-01
       coeff_l2(3,38)= -1.1655616015e-01
       coeff_l2(3,39)= -2.2224232554e-01
       coeff_l2(3,40)= -1.9076526165e-01
       coeff_l2(3,41)= -7.3749321699e-01
       coeff_l2(3,42)= -2.6649683714e-01
       coeff_l2(3,43)= -9.8245991394e-03
       coeff_l2(3,44)=  1.3331459463e-01
       coeff_l2(3,45)= -2.1358222961e+00
       coeff_l2(3,46)= -1.8000681400e+00
       coeff_l2(3,47)= -1.3336674869e-01
       coeff_l2(3,48)=  2.1833045781e-01
       coeff_l2(3,49)= -1.1725276709e-02
       coeff_l2(3,50)=  4.8789924383e-01
       coeff_l2(3,51)= -3.5066625569e-03
       coeff_l2(3,52)=  1.5321163833e-01
       coeff_l2(3,53)= -1.4334447682e-01
       coeff_l2(3,54)=  1.6218383610e-01
       coeff_l2(3,55)=  2.0877343416e-01
       coeff_l2(3,56)= -4.3650886416e-01
       coeff_l2(4,1)= -3.1651444733e-02
       coeff_l2(4,2)=  6.1630642414e-01
       coeff_l2(4,3)= -2.1879601479e-01
       coeff_l2(4,4)=  6.8536841869e-01
       coeff_l2(4,5)= -1.5501798391e+00
       coeff_l2(4,6)= -9.4085806608e-01
       coeff_l2(4,7)= -2.0748646259e+00
       coeff_l2(4,8)=  4.7100138664e-01
       coeff_l2(4,9)=  2.3513196409e-01
       coeff_l2(4,10)=  1.2975987792e-01
       coeff_l2(4,11)= -1.1872401461e-02
       coeff_l2(4,12)=  2.4578765035e-01
       coeff_l2(4,13)=  5.8546155691e-01
       coeff_l2(4,14)=  5.5377449840e-02
       coeff_l2(4,15)= -1.1346278191e+00
       coeff_l2(4,16)= -7.8745180368e-01
       coeff_l2(4,17)=  1.1183451414e+00
       coeff_l2(4,18)=  1.5092440844e+00
       coeff_l2(4,19)= -3.7906982899e+00
       coeff_l2(4,20)= -1.3276009560e+00
       coeff_l2(4,21)= -1.3740049303e-01
       coeff_l2(4,22)= -1.6418873072e+00
       coeff_l2(4,23)= -3.2590931654e-01
       coeff_l2(4,24)=  2.5594899058e-01
       coeff_l2(4,25)= -2.4692539126e-02
       coeff_l2(4,26)=  2.6022449136e-01
       coeff_l2(4,27)= -6.4779639244e-01
       coeff_l2(4,28)= -2.5108110905e-01
       coeff_l2(4,29)=  3.2018387318e-01
       coeff_l2(4,30)= -9.1583120823e-01
       coeff_l2(4,31)= -1.2485802174e-01
       coeff_l2(4,32)=  1.6292850971e+00
       coeff_l2(4,33)= -9.9976837635e-01
       coeff_l2(4,34)=  6.6094078124e-02
       coeff_l2(4,35)= -1.9350689650e-01
       coeff_l2(4,36)= -3.4884855151e-01
       coeff_l2(4,37)=  2.1233255863e+00
       coeff_l2(4,38)=  1.9077259302e-01
       coeff_l2(4,39)= -2.5523307920e-01
       coeff_l2(4,40)=  2.3992998898e-01
       coeff_l2(4,41)=  6.6597893834e-02
       coeff_l2(4,42)=  2.4741865695e-01
       coeff_l2(4,43)=  3.6036542058e-01
       coeff_l2(4,44)= -5.2764421701e-01
       coeff_l2(4,45)=  1.9883546978e-02
       coeff_l2(4,46)= -2.2850189209e+00
       coeff_l2(4,47)=  2.8603824973e-01
       coeff_l2(4,48)= -3.6696842313e-01
       coeff_l2(4,49)= -1.5441817045e+00
       coeff_l2(4,50)=  7.3427730799e-01
       coeff_l2(4,51)=  1.6589267552e-01
       coeff_l2(4,52)=  8.5817463696e-02
       coeff_l2(4,53)= -9.5067419112e-02
       coeff_l2(4,54)= -1.5016375482e-01
       coeff_l2(4,55)= -2.3879621923e-01
       coeff_l2(4,56)= -8.4099912643e-01
       coeff_l2(5,1)=  2.0928010345e-01
       coeff_l2(5,2)=  3.5994067788e-02
       coeff_l2(5,3)=  2.1267022192e-01
       coeff_l2(5,4)=  4.8935988545e-01
       coeff_l2(5,5)= -1.9146172702e-01
       coeff_l2(5,6)=  1.6540038586e-01
       coeff_l2(5,7)=  1.7871631384e+00
       coeff_l2(5,8)=  6.3862431049e-01
       coeff_l2(5,9)=  2.9922944307e-01
       coeff_l2(5,10)=  8.9087885618e-01
       coeff_l2(5,11)= -1.5481694043e-01
       coeff_l2(5,12)=  2.3078864813e-01
       coeff_l2(5,13)= -1.4020080864e-01
       coeff_l2(5,14)=  1.4713538885e+00
       coeff_l2(5,15)= -2.3493050039e-01
       coeff_l2(5,16)= -3.4821643829e+00
       coeff_l2(5,17)= -1.0209609270e+00
       coeff_l2(5,18)=  2.3072656989e-01
       coeff_l2(5,19)=  1.3725111485e+00
       coeff_l2(5,20)=  1.1664069891e+00
       coeff_l2(5,21)=  6.8254083395e-02
       coeff_l2(5,22)= -1.6915651560e+00
       coeff_l2(5,23)=  9.2779332399e-01
       coeff_l2(5,24)= -1.0106348991e+00
       coeff_l2(5,25)= -4.0029844642e-01
       coeff_l2(5,26)=  1.6363456845e-02
       coeff_l2(5,27)=  6.0628443956e-01
       coeff_l2(5,28)= -8.0315995216e-01
       coeff_l2(5,29)= -3.4382283688e-02
       coeff_l2(5,30)=  2.0459695160e-01
       coeff_l2(5,31)=  5.4472762346e-01
       coeff_l2(5,32)= -3.4168457985e+00
       coeff_l2(5,33)=  7.3397231102e-01
       coeff_l2(5,34)=  5.0459164381e-01
       coeff_l2(5,35)= -6.2164944410e-01
       coeff_l2(5,36)= -2.6718502045e+00
       coeff_l2(5,37)= -3.8631120324e-01
       coeff_l2(5,38)= -3.2889696956e-01
       coeff_l2(5,39)= -1.2229655683e-01
       coeff_l2(5,40)=  1.6902241111e-01
       coeff_l2(5,41)=  9.9697649479e-01
       coeff_l2(5,42)= -2.6064026356e-01
       coeff_l2(5,43)= -1.7232465744e-01
       coeff_l2(5,44)=  4.8397496343e-01
       coeff_l2(5,45)=  1.4024924040e+00
       coeff_l2(5,46)=  7.5447863340e-01
       coeff_l2(5,47)= -8.6093729734e-01
       coeff_l2(5,48)= -9.6862062812e-02
       coeff_l2(5,49)=  2.2931795120e+00
       coeff_l2(5,50)= -3.1033936143e-01
       coeff_l2(5,51)=  1.2733912468e+00
       coeff_l2(5,52)=  3.8900077343e-01
       coeff_l2(5,53)= -7.9157650471e-01
       coeff_l2(5,54)=  2.5682765245e-01
       coeff_l2(5,55)=  9.8012810946e-01
       coeff_l2(5,56)=  1.2690325975e+00
       coeff_l2(6,1)=  5.5927968025e-01
       coeff_l2(6,2)= -5.4752618074e-01
       coeff_l2(6,3)= -6.2891834974e-01
       coeff_l2(6,4)=  2.0160832405e+00
       coeff_l2(6,5)= -1.1228599548e+00
       coeff_l2(6,6)=  7.5267636776e-01
       coeff_l2(6,7)= -1.3322386742e+00
       coeff_l2(6,8)= -3.4096211195e-01
       coeff_l2(6,9)=  2.0385930315e-02
       coeff_l2(6,10)= -5.4910957813e-01
       coeff_l2(6,11)=  5.3935968876e-01
       coeff_l2(6,12)=  2.2014802694e-01
       coeff_l2(6,13)= -3.0718407631e+00
       coeff_l2(6,14)= -2.2123308182e+00
       coeff_l2(6,15)=  3.5942506790e-01
       coeff_l2(6,16)= -5.2721887827e-01
       coeff_l2(6,17)=  1.2209240198e+00
       coeff_l2(6,18)= -1.5185666084e+00
       coeff_l2(6,19)= -6.5866909921e-02
       coeff_l2(6,20)=  1.1772711575e-01
       coeff_l2(6,21)= -6.6589236259e-02
       coeff_l2(6,22)=  4.4995996356e-01
       coeff_l2(6,23)= -1.0988317728e+00
       coeff_l2(6,24)= -3.0211663246e-01
       coeff_l2(6,25)= -4.7903484106e-01
       coeff_l2(6,26)= -1.6592223942e-01
       coeff_l2(6,27)= -4.0023833513e-01
       coeff_l2(6,28)= -4.9086970091e-01
       coeff_l2(6,29)= -6.9957204163e-02
       coeff_l2(6,30)= -2.2165274620e+00
       coeff_l2(6,31)=  3.7363049388e-01
       coeff_l2(6,32)=  5.6975322962e-01
       coeff_l2(6,33)= -3.4943607450e-01
       coeff_l2(6,34)=  6.2101721764e-01
       coeff_l2(6,35)= -4.3674442172e-01
       coeff_l2(6,36)=  5.1634383202e-01
       coeff_l2(6,37)=  2.3461389542e-01
       coeff_l2(6,38)=  3.2114341855e-01
       coeff_l2(6,39)=  4.9508711696e-01
       coeff_l2(6,40)= -8.6588330567e-02
       coeff_l2(6,41)= -2.0635368824e+00
       coeff_l2(6,42)=  4.1542553902e-01
       coeff_l2(6,43)=  1.3505867124e-01
       coeff_l2(6,44)= -4.7556403279e-01
       coeff_l2(6,45)= -2.8750107288e+00
       coeff_l2(6,46)= -1.8750838041e+00
       coeff_l2(6,47)=  1.0010169819e-02
       coeff_l2(6,48)= -8.2876503468e-01
       coeff_l2(6,49)= -2.0897108316e-01
       coeff_l2(6,50)= -1.2989969254e+00
       coeff_l2(6,51)=  2.0536777973e+00
       coeff_l2(6,52)= -2.6949769258e-01
       coeff_l2(6,53)= -7.9378509521e-01
       coeff_l2(6,54)= -8.0708128214e-01
       coeff_l2(6,55)= -6.0744124651e-01
       coeff_l2(6,56)=  1.6148108244e+00
       coeff_l2(7,1)=  1.1875056475e-01
       coeff_l2(7,2)= -7.7430903912e-02
       coeff_l2(7,3)= -3.6022562534e-02
       coeff_l2(7,4)=  3.7666794658e-01
       coeff_l2(7,5)= -2.3012906313e-01
       coeff_l2(7,6)=  2.6308718324e-01
       coeff_l2(7,7)=  1.1061851978e+00
       coeff_l2(7,8)= -3.0178582668e-01
       coeff_l2(7,9)= -4.7217994928e-02
       coeff_l2(7,10)= -2.4503202736e-01
       coeff_l2(7,11)=  9.8810926080e-02
       coeff_l2(7,12)=  1.1422803998e-01
       coeff_l2(7,13)= -1.2001742423e-01
       coeff_l2(7,14)=  1.4387952089e+00
       coeff_l2(7,15)= -2.6639513969e+00
       coeff_l2(7,16)= -1.3209542036e+00
       coeff_l2(7,17)= -3.9158317447e-01
       coeff_l2(7,18)=  1.0486440659e+00
       coeff_l2(7,19)=  1.3603199720e+00
       coeff_l2(7,20)=  7.3197120428e-01
       coeff_l2(7,21)= -8.8549000025e-01
       coeff_l2(7,22)= -9.4824028015e-01
       coeff_l2(7,23)= -1.5065006018e+00
       coeff_l2(7,24)= -3.9872890711e-01
       coeff_l2(7,25)= -4.8174542189e-01
       coeff_l2(7,26)= -9.3296337873e-03
       coeff_l2(7,27)=  4.2168161273e-01
       coeff_l2(7,28)= -2.7409595251e-01
       coeff_l2(7,29)= -3.2969093323e-01
       coeff_l2(7,30)=  6.6520422697e-01
       coeff_l2(7,31)= -7.9743963480e-01
       coeff_l2(7,32)= -1.8396283388e+00
       coeff_l2(7,33)= -1.3001999259e-01
       coeff_l2(7,34)=  1.6466382891e-02
       coeff_l2(7,35)=  2.1393498778e-01
       coeff_l2(7,36)=  9.9896445870e-02
       coeff_l2(7,37)= -9.6851751208e-02
       coeff_l2(7,38)= -3.7814980745e-01
       coeff_l2(7,39)=  1.1114814878e-01
       coeff_l2(7,40)= -5.1562264562e-02
       coeff_l2(7,41)=  8.7792968750e-01
       coeff_l2(7,42)= -5.3658522666e-02
       coeff_l2(7,43)= -1.5598738194e-01
       coeff_l2(7,44)=  2.4186854064e-01
       coeff_l2(7,45)=  7.4965721369e-01
       coeff_l2(7,46)=  9.2205941677e-01
       coeff_l2(7,47)= -2.1330021322e-01
       coeff_l2(7,48)= -3.4094664454e-01
       coeff_l2(7,49)=  1.2568047047e+00
       coeff_l2(7,50)= -2.4203453958e-01
       coeff_l2(7,51)=  3.4117537737e-01
       coeff_l2(7,52)=  3.3145433664e-01
       coeff_l2(7,53)=  7.1549016237e-01
       coeff_l2(7,54)=  1.7098626122e-02
       coeff_l2(7,55)=  7.2743266821e-01
       coeff_l2(7,56)= -4.2647438049e+00
       coeff_l2(8,1)= -1.0834223032e+00
       coeff_l2(8,2)= -1.2053333223e-01
       coeff_l2(8,3)=  1.8165194988e-01
       coeff_l2(8,4)= -7.7637004852e-01
       coeff_l2(8,5)=  3.7474110723e-01
       coeff_l2(8,6)= -6.3836061954e-01
       coeff_l2(8,7)=  1.7388059199e-01
       coeff_l2(8,8)=  1.1286780983e-01
       coeff_l2(8,9)=  4.7437846661e-01
       coeff_l2(8,10)=  3.9532512426e-01
       coeff_l2(8,11)=  4.1963177919e-01
       coeff_l2(8,12)= -4.3331533670e-02
       coeff_l2(8,13)= -7.5694873929e-02
       coeff_l2(8,14)= -1.3087219000e-01
       coeff_l2(8,15)= -3.9157113433e-01
       coeff_l2(8,16)= -8.2557660341e-01
       coeff_l2(8,17)= -1.0911968946e+00
       coeff_l2(8,18)= -2.8133077621e+00
       coeff_l2(8,19)=  1.8817275763e-01
       coeff_l2(8,20)=  5.4731857777e-01
       coeff_l2(8,21)= -3.2164666057e-01
       coeff_l2(8,22)= -7.4585491419e-01
       coeff_l2(8,23)= -4.4765132666e-01
       coeff_l2(8,24)=  6.6073894501e-01
       coeff_l2(8,25)=  5.5236911774e-01
       coeff_l2(8,26)= -6.0831236839e-01
       coeff_l2(8,27)= -5.9709813446e-02
       coeff_l2(8,28)= -3.1889375299e-02
       coeff_l2(8,29)=  2.0125491917e-01
       coeff_l2(8,30)=  1.7464140058e-01
       coeff_l2(8,31)= -6.0979318619e-01
       coeff_l2(8,32)=  6.4015753567e-02
       coeff_l2(8,33)= -3.2522919774e-01
       coeff_l2(8,34)= -5.3529828787e-01
       coeff_l2(8,35)=  2.9294034839e-01
       coeff_l2(8,36)=  1.7361637354e+00
       coeff_l2(8,37)= -6.7039445043e-02
       coeff_l2(8,38)= -4.4966048002e-01
       coeff_l2(8,39)=  4.4057205319e-01
       coeff_l2(8,40)= -4.0180793405e-01
       coeff_l2(8,41)= -2.9471912980e-01
       coeff_l2(8,42)= -1.3666656613e-01
       coeff_l2(8,43)=  2.5851956010e-01
       coeff_l2(8,44)= -1.1805967242e-01
       coeff_l2(8,45)= -1.0550661087e+00
       coeff_l2(8,46)= -4.5808851719e-01
       coeff_l2(8,47)=  5.5444800854e-01
       coeff_l2(8,48)=  2.9170441628e-01
       coeff_l2(8,49)= -1.2333469838e-01
       coeff_l2(8,50)= -3.1419771910e-01
       coeff_l2(8,51)=  6.5258510411e-02
       coeff_l2(8,52)=  8.0137783289e-01
       coeff_l2(8,53)=  1.8345090211e-04
       coeff_l2(8,54)= -1.5464609861e-01
       coeff_l2(8,55)= -3.1849691272e-01
       coeff_l2(8,56)= -6.1571043730e-01
       coeff_l2(9,1)= -1.7713035643e-01
       coeff_l2(9,2)= -3.3717432618e-01
       coeff_l2(9,3)=  3.4153389931e-01
       coeff_l2(9,4)= -1.0161724091e+00
       coeff_l2(9,5)= -9.5990681648e-01
       coeff_l2(9,6)= -4.5293055475e-02
       coeff_l2(9,7)=  1.8655106425e-01
       coeff_l2(9,8)= -1.2557263672e-01
       coeff_l2(9,9)= -2.7660134435e-01
       coeff_l2(9,10)=  2.1183722019e+00
       coeff_l2(9,11)=  6.0865378380e-01
       coeff_l2(9,12)=  6.9727754593e-01
       coeff_l2(9,13)= -4.5219385624e-01
       coeff_l2(9,14)=  2.5035402179e-01
       coeff_l2(9,15)= -3.3088114858e-01
       coeff_l2(9,16)= -6.2890845537e-01
       coeff_l2(9,17)= -1.1884969473e+00
       coeff_l2(9,18)= -6.0471154749e-02
       coeff_l2(9,19)= -2.6484596729e-01
       coeff_l2(9,20)=  8.5654211044e-01
       coeff_l2(9,21)= -4.0065097809e-01
       coeff_l2(9,22)= -7.0361793041e-01
       coeff_l2(9,23)= -3.3014705181e+00
       coeff_l2(9,24)=  8.7247699499e-02
       coeff_l2(9,25)=  6.0745561123e-01
       coeff_l2(9,26)= -6.8626099825e-01
       coeff_l2(9,27)= -3.1653863192e-01
       coeff_l2(9,28)=  2.7603495121e-01
       coeff_l2(9,29)=  1.5499932691e-02
       coeff_l2(9,30)= -1.0763537884e-01
       coeff_l2(9,31)= -7.6139545999e-03
       coeff_l2(9,32)= -1.7381192446e+00
       coeff_l2(9,33)= -1.0959453881e-01
       coeff_l2(9,34)=  1.7532702684e+00
       coeff_l2(9,35)= -2.5339302421e-01
       coeff_l2(9,36)=  4.5184338093e-01
       coeff_l2(9,37)= -2.9445514083e-01
       coeff_l2(9,38)= -8.3069348335e-01
       coeff_l2(9,39)= -4.8307448626e-01
       coeff_l2(9,40)=  1.1789722443e+00
       coeff_l2(9,41)=  1.3765465021e+00
       coeff_l2(9,42)= -2.6510164142e-01
       coeff_l2(9,43)= -9.5013439655e-01
       coeff_l2(9,44)=  4.5123851299e-01
       coeff_l2(9,45)= -9.5288938284e-01
       coeff_l2(9,46)= -1.4588508010e-01
       coeff_l2(9,47)= -2.7719479799e-01
       coeff_l2(9,48)= -1.1556777358e-01
       coeff_l2(9,49)= -2.4235734344e-01
       coeff_l2(9,50)=  1.3338476419e-01
       coeff_l2(9,51)=  5.6415927410e-01
       coeff_l2(9,52)=  4.6122565866e-01
       coeff_l2(9,53)=  7.3194354773e-01
       coeff_l2(9,54)= -7.8663244843e-02
       coeff_l2(9,55)=  7.6089508832e-02
       coeff_l2(9,56)=  2.2488067150e+00
       coeff_l2(10,1)= -3.0120828748e-01
       coeff_l2(10,2)=  5.9210622311e-01
       coeff_l2(10,3)=  1.6899682581e-01
       coeff_l2(10,4)= -2.1933434010e+00
       coeff_l2(10,5)=  3.8820415735e-01
       coeff_l2(10,6)= -1.5044007301e+00
       coeff_l2(10,7)= -1.5761756897e-01
       coeff_l2(10,8)=  4.6565070748e-01
       coeff_l2(10,9)=  2.3019863665e-01
       coeff_l2(10,10)=  5.2815043926e-01
       coeff_l2(10,11)= -3.8591008633e-02
       coeff_l2(10,12)=  2.7725270391e-01
       coeff_l2(10,13)= -2.7656301856e-01
       coeff_l2(10,14)=  1.3134016097e-01
       coeff_l2(10,15)= -6.3291800022e-01
       coeff_l2(10,16)= -1.1513423920e+00
       coeff_l2(10,17)= -9.4210171700e-01
       coeff_l2(10,18)= -1.0738612413e+00
       coeff_l2(10,19)= -5.6640219688e-01
       coeff_l2(10,20)= -2.3157736659e-01
       coeff_l2(10,21)=  7.4609969743e-03
       coeff_l2(10,22)= -1.4687682390e+00
       coeff_l2(10,23)= -1.3803021908e+00
       coeff_l2(10,24)=  1.5238107741e-01
       coeff_l2(10,25)=  1.4654092491e-01
       coeff_l2(10,26)= -3.1880840659e-01
       coeff_l2(10,27)= -7.3430068791e-02
       coeff_l2(10,28)= -1.4880350232e-01
       coeff_l2(10,29)=  3.9391770959e-01
       coeff_l2(10,30)= -7.0930671692e-01
       coeff_l2(10,31)= -2.1494258940e-01
       coeff_l2(10,32)= -1.0119966269e+00
       coeff_l2(10,33)=  1.4809425175e-01
       coeff_l2(10,34)=  3.2640717924e-02
       coeff_l2(10,35)=  4.0428772569e-02
       coeff_l2(10,36)= -1.4154789448e+00
       coeff_l2(10,37)= -6.0052293539e-01
       coeff_l2(10,38)= -2.7270367742e-01
       coeff_l2(10,39)=  1.8075264990e-01
       coeff_l2(10,40)=  2.4268110096e-01
       coeff_l2(10,41)= -1.1405130625e+00
       coeff_l2(10,42)= -8.6918696761e-03
       coeff_l2(10,43)=  5.6399029493e-01
       coeff_l2(10,44)=  4.6003647149e-02
       coeff_l2(10,45)= -1.5429292917e+00
       coeff_l2(10,46)= -2.5216194987e-01
       coeff_l2(10,47)= -2.5516951457e-02
       coeff_l2(10,48)=  2.0269744098e-01
       coeff_l2(10,49)=  9.7771726549e-02
       coeff_l2(10,50)= -8.5685372353e-02
       coeff_l2(10,51)=  4.5402017236e-01
       coeff_l2(10,52)=  2.7047902346e-01
       coeff_l2(10,53)= -3.3341017365e-01
       coeff_l2(10,54)=  1.3886289671e-02
       coeff_l2(10,55)= -4.4415521622e-01
       coeff_l2(10,56)=  1.1012521982e+00
       coeff_l2(11,1)= -3.6253529787e-01
       coeff_l2(11,2)= -3.3404019475e-01
       coeff_l2(11,3)= -3.6668342352e-01
       coeff_l2(11,4)= -1.3472656012e+00
       coeff_l2(11,5)= -1.2153636664e-02
       coeff_l2(11,6)= -1.2042820454e+00
       coeff_l2(11,7)= -4.9518290162e-01
       coeff_l2(11,8)=  3.6684498191e-02
       coeff_l2(11,9)= -1.2546381354e-01
       coeff_l2(11,10)=  1.8870282173e-01
       coeff_l2(11,11)= -3.0121192336e-01
       coeff_l2(11,12)=  1.2253012508e-01
       coeff_l2(11,13)= -5.4260098934e-01
       coeff_l2(11,14)=  2.1794578433e-01
       coeff_l2(11,15)=  1.6491951048e-01
       coeff_l2(11,16)= -8.0423408747e-01
       coeff_l2(11,17)= -3.6014163494e-01
       coeff_l2(11,18)= -1.7520046234e+00
       coeff_l2(11,19)= -5.9206765890e-01
       coeff_l2(11,20)= -6.9104790688e-02
       coeff_l2(11,21)=  2.1301531792e-01
       coeff_l2(11,22)= -1.0193271637e+00
       coeff_l2(11,23)= -1.3541043997e+00
       coeff_l2(11,24)=  1.3352117501e-02
       coeff_l2(11,25)=  2.6276865602e-01
       coeff_l2(11,26)= -2.7879574895e-01
       coeff_l2(11,27)=  3.2226109505e-01
       coeff_l2(11,28)= -5.8604866266e-01
       coeff_l2(11,29)= -1.3485060632e-01
       coeff_l2(11,30)= -3.5115391016e-01
       coeff_l2(11,31)= -4.9835669994e-01
       coeff_l2(11,32)= -1.2610712647e-01
       coeff_l2(11,33)=  1.0916934907e-01
       coeff_l2(11,34)=  2.0354409516e-01
       coeff_l2(11,35)= -3.8745164056e-04
       coeff_l2(11,36)= -1.1329734325e-01
       coeff_l2(11,37)= -4.6746313572e-01
       coeff_l2(11,38)=  2.1385405958e-01
       coeff_l2(11,39)= -2.8378725052e-01
       coeff_l2(11,40)=  1.0646697283e+00
       coeff_l2(11,41)= -3.9481577277e-01
       coeff_l2(11,42)=  1.2688302994e-01
       coeff_l2(11,43)=  9.8586007953e-02
       coeff_l2(11,44)=  6.9448560476e-02
       coeff_l2(11,45)= -1.9434038401e+00
       coeff_l2(11,46)= -2.6070931554e-01
       coeff_l2(11,47)=  4.1233465075e-02
       coeff_l2(11,48)=  3.8080865145e-01
       coeff_l2(11,49)=  1.4827977121e-01
       coeff_l2(11,50)=  9.9629059434e-02
       coeff_l2(11,51)= -2.0785957575e-01
       coeff_l2(11,52)=  2.7008721232e-01
       coeff_l2(11,53)= -5.0864082575e-01
       coeff_l2(11,54)=  1.7604118586e-01
       coeff_l2(11,55)=  3.9623242617e-01
       coeff_l2(11,56)= -1.6910457611e-01
       coeff_l2(12,1)= -4.3772238493e-01
       coeff_l2(12,2)=  2.3538495600e-01
       coeff_l2(12,3)=  2.7576634288e-01
       coeff_l2(12,4)= -2.2944967747e+00
       coeff_l2(12,5)=  4.8013401031e-01
       coeff_l2(12,6)= -1.0458259583e+00
       coeff_l2(12,7)= -1.2047513723e+00
       coeff_l2(12,8)= -2.3143546656e-02
       coeff_l2(12,9)=  3.1950473785e-01
       coeff_l2(12,10)=  4.5150896907e-01
       coeff_l2(12,11)= -2.4205906317e-02
       coeff_l2(12,12)=  1.2441695929e+00
       coeff_l2(12,13)=  2.5984477997e-01
       coeff_l2(12,14)=  6.9233742543e-03
       coeff_l2(12,15)= -6.8014085293e-01
       coeff_l2(12,16)= -1.3178173304e+00
       coeff_l2(12,17)= -4.2156893015e-01
       coeff_l2(12,18)= -1.1771445274e+00
       coeff_l2(12,19)= -1.0984063148e+00
       coeff_l2(12,20)= -1.6750870645e-01
       coeff_l2(12,21)= -6.6379539669e-02
       coeff_l2(12,22)= -1.1772636175e+00
       coeff_l2(12,23)= -7.5196832418e-01
       coeff_l2(12,24)=  6.6876620054e-01
       coeff_l2(12,25)=  5.4146367311e-01
       coeff_l2(12,26)= -3.0054035783e-01
       coeff_l2(12,27)= -3.9088189602e-01
       coeff_l2(12,28)= -1.2627819180e-01
       coeff_l2(12,29)=  5.1438983530e-02
       coeff_l2(12,30)= -1.3042124510e+00
       coeff_l2(12,31)= -3.8524869084e-01
       coeff_l2(12,32)=  1.0056799650e+00
       coeff_l2(12,33)= -1.5246774256e-01
       coeff_l2(12,34)=  5.0546997786e-01
       coeff_l2(12,35)= -3.6333775520e-01
       coeff_l2(12,36)=  1.0992118120e+00
       coeff_l2(12,37)=  7.1636921167e-01
       coeff_l2(12,38)= -3.3638622612e-02
       coeff_l2(12,39)= -1.8988277763e-02
       coeff_l2(12,40)= -1.1567164660e+00
       coeff_l2(12,41)= -5.1082181931e-01
       coeff_l2(12,42)= -6.5165415406e-02
       coeff_l2(12,43)= -3.5769987106e-01
       coeff_l2(12,44)=  1.4427043498e-01
       coeff_l2(12,45)= -1.2845146656e+00
       coeff_l2(12,46)= -2.4613113403e+00
       coeff_l2(12,47)=  1.1994662136e-01
       coeff_l2(12,48)=  1.2761650607e-02
       coeff_l2(12,49)= -1.8514531851e-01
       coeff_l2(12,50)= -5.0227540731e-01
       coeff_l2(12,51)= -8.8094276190e-01
       coeff_l2(12,52)= -2.2498214245e-01
       coeff_l2(12,53)= -4.2574617267e-01
       coeff_l2(12,54)= -6.0412168503e-01
       coeff_l2(12,55)= -9.6048384905e-01
       coeff_l2(12,56)=  1.0953353643e+00
       coeff_l2(13,1)=  2.1189558506e-01
       coeff_l2(13,2)=  3.3022388816e-01
       coeff_l2(13,3)=  8.2474184036e-01
       coeff_l2(13,4)=  1.9245787859e+00
       coeff_l2(13,5)= -1.3070466518e+00
       coeff_l2(13,6)=  1.3854022026e+00
       coeff_l2(13,7)=  2.7652886510e-01
       coeff_l2(13,8)= -4.1294139624e-01
       coeff_l2(13,9)= -3.3741027117e-01
       coeff_l2(13,10)=  2.5567147136e-01
       coeff_l2(13,11)= -9.9332809448e-01
       coeff_l2(13,12)= -3.1811070442e-01
       coeff_l2(13,13)=  7.2115588188e-01
       coeff_l2(13,14)= -1.2404384464e-01
       coeff_l2(13,15)=  6.7418146133e-01
       coeff_l2(13,16)= -1.6543161869e+00
       coeff_l2(13,17)=  8.8162791729e-01
       coeff_l2(13,18)=  3.7874643803e+00
       coeff_l2(13,19)= -6.9301050901e-01
       coeff_l2(13,20)= -1.5226984024e-01
       coeff_l2(13,21)=  1.8846136332e-01
       coeff_l2(13,22)=  4.1613522172e-01
       coeff_l2(13,23)=  6.4958244562e-01
       coeff_l2(13,24)= -5.9615474939e-01
       coeff_l2(13,25)=  7.1639835835e-02
       coeff_l2(13,26)=  2.6300504804e-01
       coeff_l2(13,27)= -2.4493360519e-01
       coeff_l2(13,28)=  6.7644870281e-01
       coeff_l2(13,29)= -3.7292647362e-01
       coeff_l2(13,30)= -1.6863089800e-01
       coeff_l2(13,31)= -3.5326567292e-01
       coeff_l2(13,32)=  6.0682117939e-01
       coeff_l2(13,33)=  4.5140951872e-01
       coeff_l2(13,34)= -3.1280821562e-01
       coeff_l2(13,35)=  3.5401824862e-02
       coeff_l2(13,36)= -4.1665399075e-01
       coeff_l2(13,37)=  1.6315153241e-01
       coeff_l2(13,38)= -5.1922649145e-01
       coeff_l2(13,39)= -5.3467732668e-01
       coeff_l2(13,40)= -6.7259502411e-01
       coeff_l2(13,41)=  4.2687293887e-01
       coeff_l2(13,42)= -1.6761314869e+00
       coeff_l2(13,43)=  4.5850825310e-01
       coeff_l2(13,44)=  3.8182982802e-01
       coeff_l2(13,45)=  1.0105198622e+00
       coeff_l2(13,46)= -1.4987078905e+00
       coeff_l2(13,47)= -3.7533089519e-02
       coeff_l2(13,48)= -3.1042712927e-01
       coeff_l2(13,49)=  5.8214628696e-01
       coeff_l2(13,50)= -4.6955367923e-01
       coeff_l2(13,51)= -3.6906450987e-01
       coeff_l2(13,52)= -3.5129821301e-01
       coeff_l2(13,53)= -4.4103133678e-01
       coeff_l2(13,54)= -1.0184090137e+00
       coeff_l2(13,55)=  4.5077937841e-01
       coeff_l2(13,56)=  9.1112178564e-01
       coeff_l2(14,1)= -9.5130205154e-01
       coeff_l2(14,2)= -4.3956121802e-01
       coeff_l2(14,3)=  9.4344300032e-01
       coeff_l2(14,4)= -6.5565443039e-01
       coeff_l2(14,5)=  1.2659012079e+00
       coeff_l2(14,6)= -9.2537057400e-01
       coeff_l2(14,7)= -7.0671254396e-01
       coeff_l2(14,8)=  3.7203484774e-01
       coeff_l2(14,9)=  3.2031694055e-01
       coeff_l2(14,10)= -7.2910875082e-01
       coeff_l2(14,11)= -4.8029983044e-01
       coeff_l2(14,12)=  1.3504771888e-01
       coeff_l2(14,13)=  7.0459380746e-02
       coeff_l2(14,14)= -5.1797162741e-02
       coeff_l2(14,15)=  2.1394005418e-01
       coeff_l2(14,16)= -1.0397944450e+00
       coeff_l2(14,17)= -2.8184759617e+00
       coeff_l2(14,18)= -2.0545756817e+00
       coeff_l2(14,19)= -3.0001929402e-01
       coeff_l2(14,20)= -8.9004629850e-01
       coeff_l2(14,21)=  5.0909805298e-01
       coeff_l2(14,22)= -2.0675513744e+00
       coeff_l2(14,23)= -5.6089067459e-01
       coeff_l2(14,24)= -4.4642579556e-01
       coeff_l2(14,25)= -2.4234138429e-02
       coeff_l2(14,26)= -5.2556300163e-01
       coeff_l2(14,27)= -3.0104538798e-01
       coeff_l2(14,28)= -3.1586205959e-01
       coeff_l2(14,29)= -3.6171483994e-01
       coeff_l2(14,30)= -1.3264122605e-01
       coeff_l2(14,31)= -4.5352968574e-01
       coeff_l2(14,32)= -3.3072713017e-01
       coeff_l2(14,33)=  9.7625917196e-01
       coeff_l2(14,34)= -1.3249733448e+00
       coeff_l2(14,35)= -7.8205829859e-01
       coeff_l2(14,36)= -4.8672416806e-01
       coeff_l2(14,37)= -9.4609880447e-01
       coeff_l2(14,38)=  5.3212636709e-01
       coeff_l2(14,39)= -6.4692576416e-03
       coeff_l2(14,40)=  5.0843554735e-01
       coeff_l2(14,41)= -4.1274914145e-01
       coeff_l2(14,42)=  6.4405687153e-02
       coeff_l2(14,43)= -2.7357944846e-01
       coeff_l2(14,44)=  6.5146094561e-01
       coeff_l2(14,45)= -2.2434608936e+00
       coeff_l2(14,46)= -1.5549896657e-01
       coeff_l2(14,47)=  1.3017759658e-02
       coeff_l2(14,48)=  1.2906195223e-01
       coeff_l2(14,49)= -9.7291392088e-01
       coeff_l2(14,50)=  1.6211971045e+00
       coeff_l2(14,51)= -1.0515940189e-01
       coeff_l2(14,52)= -5.8678179979e-01
       coeff_l2(14,53)= -8.1229281425e-01
       coeff_l2(14,54)=  9.5142740011e-01
       coeff_l2(14,55)= -1.0995533466e+00
       coeff_l2(14,56)=  2.5626471266e-02
       coeff_l2(15,1)= -6.7169034481e-01
       coeff_l2(15,2)= -6.6629357636e-02
       coeff_l2(15,3)= -9.9735051394e-02
       coeff_l2(15,4)= -7.2259646654e-01
       coeff_l2(15,5)= -1.6173848510e-01
       coeff_l2(15,6)= -1.4103569984e+00
       coeff_l2(15,7)= -4.4797480106e-01
       coeff_l2(15,8)= -2.5483995676e-01
       coeff_l2(15,9)=  2.1220071614e-01
       coeff_l2(15,10)=  1.3831555843e-03
       coeff_l2(15,11)= -3.9908179641e-01
       coeff_l2(15,12)=  2.3961398005e-01
       coeff_l2(15,13)= -2.9357832670e-01
       coeff_l2(15,14)= -3.8426987827e-02
       coeff_l2(15,15)= -4.9645956606e-02
       coeff_l2(15,16)= -8.3884525299e-01
       coeff_l2(15,17)= -2.4555100501e-01
       coeff_l2(15,18)= -1.8004807234e+00
       coeff_l2(15,19)= -7.6876264811e-01
       coeff_l2(15,20)= -3.3723947406e-01
       coeff_l2(15,21)=  1.6691231728e-01
       coeff_l2(15,22)= -8.4695845842e-01
       coeff_l2(15,23)= -1.4340660572e+00
       coeff_l2(15,24)=  9.2553719878e-02
       coeff_l2(15,25)=  1.8850453198e-01
       coeff_l2(15,26)= -5.6768053770e-01
       coeff_l2(15,27)=  3.7840731442e-02
       coeff_l2(15,28)= -1.6254119575e-01
       coeff_l2(15,29)=  3.7932506204e-01
       coeff_l2(15,30)= -6.6082012653e-01
       coeff_l2(15,31)= -4.3134164810e-01
       coeff_l2(15,32)= -2.0409461111e-02
       coeff_l2(15,33)=  8.0472476780e-02
       coeff_l2(15,34)= -2.0370669663e-01
       coeff_l2(15,35)= -1.4899228513e-01
       coeff_l2(15,36)=  1.8580372632e-01
       coeff_l2(15,37)= -4.4279447198e-01
       coeff_l2(15,38)= -1.4149600267e-01
       coeff_l2(15,39)= -5.2763823420e-02
       coeff_l2(15,40)=  3.6143290997e-01
       coeff_l2(15,41)= -5.8574265242e-01
       coeff_l2(15,42)= -1.2037295848e-01
       coeff_l2(15,43)=  3.9894759655e-02
       coeff_l2(15,44)= -1.8272668123e-02
       coeff_l2(15,45)= -2.3141891956e+00
       coeff_l2(15,46)= -1.4701523781e+00
       coeff_l2(15,47)= -4.3644887209e-01
       coeff_l2(15,48)=  1.7713361979e-01
       coeff_l2(15,49)=  1.8899211660e-02
       coeff_l2(15,50)=  5.3219344467e-02
       coeff_l2(15,51)= -5.3212959319e-02
       coeff_l2(15,52)=  3.9765408635e-01
       coeff_l2(15,53)= -2.8502961993e-01
       coeff_l2(15,54)=  1.0864315927e-01
       coeff_l2(15,55)=  2.6697957516e-01
       coeff_l2(15,56)= -3.4593385458e-01
       coeff_l2(16,1)=  2.8513243794e-01
       coeff_l2(16,2)= -1.2139595300e-01
       coeff_l2(16,3)= -1.0802362114e-02
       coeff_l2(16,4)=  2.3846905231e+00
       coeff_l2(16,5)= -1.2775427103e+00
       coeff_l2(16,6)=  4.0220756084e-02
       coeff_l2(16,7)= -1.3805243969e+00
       coeff_l2(16,8)= -7.1284204721e-01
       coeff_l2(16,9)= -9.5113769174e-02
       coeff_l2(16,10)= -2.4230505526e-01
       coeff_l2(16,11)=  7.7958337963e-02
       coeff_l2(16,12)= -3.3492341638e-01
       coeff_l2(16,13)= -2.0180535316e-01
       coeff_l2(16,14)= -1.1210143566e+00
       coeff_l2(16,15)=  4.0226405859e-01
       coeff_l2(16,16)= -1.3294432163e+00
       coeff_l2(16,17)= -1.9801538438e-03
       coeff_l2(16,18)= -2.2496855259e-01
       coeff_l2(16,19)= -3.1524865627e+00
       coeff_l2(16,20)= -1.0247486830e+00
       coeff_l2(16,21)=  6.3178837299e-03
       coeff_l2(16,22)= -4.0078783035e-01
       coeff_l2(16,23)= -1.0789618641e-01
       coeff_l2(16,24)=  1.6836996377e-01
       coeff_l2(16,25)=  1.1669772863e-01
       coeff_l2(16,26)= -4.2179595679e-02
       coeff_l2(16,27)= -4.4237270951e-01
       coeff_l2(16,28)= -5.5880206823e-01
       coeff_l2(16,29)=  4.6132481098e-01
       coeff_l2(16,30)= -2.1003159881e-01
       coeff_l2(16,31)=  2.7607887983e-01
       coeff_l2(16,32)=  7.4608951807e-01
       coeff_l2(16,33)= -3.3340658993e-02
       coeff_l2(16,34)= -1.7845898867e-01
       coeff_l2(16,35)= -8.1933921576e-01
       coeff_l2(16,36)=  6.3516639173e-02
       coeff_l2(16,37)=  9.2451763153e-01
       coeff_l2(16,38)=  8.2595288754e-02
       coeff_l2(16,39)= -3.7556439638e-01
       coeff_l2(16,40)=  3.4717440605e-01
       coeff_l2(16,41)=  1.4478586614e-01
       coeff_l2(16,42)= -7.0122677088e-01
       coeff_l2(16,43)= -1.3986815512e-01
       coeff_l2(16,44)= -1.3849568367e+00
       coeff_l2(16,45)=  9.3012994528e-01
       coeff_l2(16,46)= -4.0481290817e+00
       coeff_l2(16,47)= -1.4970719814e-02
       coeff_l2(16,48)= -8.9101457596e-01
       coeff_l2(16,49)= -1.5778552294e+00
       coeff_l2(16,50)= -2.4237948656e-01
       coeff_l2(16,51)=  1.0628471375e+00
       coeff_l2(16,52)= -5.7607477903e-01
       coeff_l2(16,53)= -4.4151484966e-01
       coeff_l2(16,54)= -1.5227929354e+00
       coeff_l2(16,55)= -3.4452167153e-01
       coeff_l2(16,56)=  6.4990216494e-01
       coeff_l2(17,1)=  1.7744915187e-01
       coeff_l2(17,2)= -6.6119700205e-04
       coeff_l2(17,3)= -1.5215548873e-01
       coeff_l2(17,4)=  2.4747042656e+00
       coeff_l2(17,5)= -9.3663316220e-03
       coeff_l2(17,6)=  8.8806289434e-01
       coeff_l2(17,7)=  1.8730713427e-01
       coeff_l2(17,8)=  3.0515730381e-01
       coeff_l2(17,9)=  1.7304185033e-01
       coeff_l2(17,10)=  1.2369910628e-01
       coeff_l2(17,11)=  1.1977542639e+00
       coeff_l2(17,12)=  6.2148910761e-01
       coeff_l2(17,13)=  1.0856959820e+00
       coeff_l2(17,14)=  7.2643339634e-01
       coeff_l2(17,15)=  3.0354982615e-01
       coeff_l2(17,16)= -2.7434697747e-01
       coeff_l2(17,17)= -1.0561012030e+00
       coeff_l2(17,18)= -5.5638242513e-02
       coeff_l2(17,19)= -1.0325787067e+00
       coeff_l2(17,20)=  1.6961593926e-01
       coeff_l2(17,21)= -1.6337335110e-02
       coeff_l2(17,22)=  1.7923392355e-01
       coeff_l2(17,23)=  3.7683686614e-01
       coeff_l2(17,24)=  4.0725231171e-01
       coeff_l2(17,25)=  3.1041133404e-01
       coeff_l2(17,26)=  7.3998552561e-01
       coeff_l2(17,27)= -3.4557810426e-01
       coeff_l2(17,28)=  1.2537517585e-02
       coeff_l2(17,29)=  1.0252023488e-01
       coeff_l2(17,30)=  1.1558330059e+00
       coeff_l2(17,31)= -2.9618099332e-01
       coeff_l2(17,32)= -1.0598017275e-01
       coeff_l2(17,33)=  6.1759471893e-02
       coeff_l2(17,34)=  1.2149785757e+00
       coeff_l2(17,35)=  1.4812989160e-02
       coeff_l2(17,36)= -3.1313553452e-01
       coeff_l2(17,37)= -1.5201912820e-01
       coeff_l2(17,38)= -3.6532044411e-01
       coeff_l2(17,39)= -3.9136540890e-01
       coeff_l2(17,40)= -1.1402648687e+00
       coeff_l2(17,41)= -1.1160959005e+00
       coeff_l2(17,42)= -6.5578520298e-02
       coeff_l2(17,43)=  9.5866680145e-02
       coeff_l2(17,44)=  3.2742589712e-01
       coeff_l2(17,45)= -2.1133171022e-01
       coeff_l2(17,46)= -3.1524914503e-01
       coeff_l2(17,47)=  5.1435959339e-01
       coeff_l2(17,48)=  7.1945287287e-02
       coeff_l2(17,49)= -1.7025014758e-01
       coeff_l2(17,50)= -8.9853838086e-02
       coeff_l2(17,51)= -4.8621544242e-01
       coeff_l2(17,52)=  4.7191175818e-01
       coeff_l2(17,53)=  3.7337452173e-01
       coeff_l2(17,54)= -1.5620813370e+00
       coeff_l2(17,55)=  1.1324146986e+00
       coeff_l2(17,56)=  7.4994611740e-01
       coeff_l2(18,1)= -1.3674817085e+00
       coeff_l2(18,2)= -6.9513529539e-01
       coeff_l2(18,3)=  4.6931508183e-01
       coeff_l2(18,4)= -1.2837675810e+00
       coeff_l2(18,5)=  3.0458023548e+00
       coeff_l2(18,6)= -9.5224839449e-01
       coeff_l2(18,7)=  2.1095937490e-01
       coeff_l2(18,8)=  1.7644160986e-01
       coeff_l2(18,9)= -3.1557399780e-02
       coeff_l2(18,10)=  3.9552461356e-02
       coeff_l2(18,11)= -1.1130009890e+00
       coeff_l2(18,12)= -7.3800230026e-01
       coeff_l2(18,13)=  8.9475616813e-02
       coeff_l2(18,14)=  4.8643346876e-02
       coeff_l2(18,15)= -1.6274432838e-01
       coeff_l2(18,16)= -1.2947790623e+00
       coeff_l2(18,17)= -5.2391189337e-01
       coeff_l2(18,18)= -1.9876751900e+00
       coeff_l2(18,19)= -2.0442891121e-01
       coeff_l2(18,20)=  6.7198555917e-03
       coeff_l2(18,21)=  9.9654369056e-02
       coeff_l2(18,22)= -4.1929572821e-01
       coeff_l2(18,23)= -1.0346456766e+00
       coeff_l2(18,24)=  3.1491243839e-01
       coeff_l2(18,25)= -8.2148456573e-01
       coeff_l2(18,26)= -7.7765583992e-01
       coeff_l2(18,27)=  5.0677660853e-02
       coeff_l2(18,28)= -2.0168209076e-01
       coeff_l2(18,29)=  3.9485839009e-01
       coeff_l2(18,30)= -6.7654573917e-01
       coeff_l2(18,31)= -4.2560473084e-01
       coeff_l2(18,32)= -7.0100322366e-02
       coeff_l2(18,33)=  1.9960014150e-02
       coeff_l2(18,34)= -3.7883749604e-01
       coeff_l2(18,35)= -4.8770725727e-01
       coeff_l2(18,36)=  1.5737379789e+00
       coeff_l2(18,37)=  6.9673091173e-01
       coeff_l2(18,38)= -3.3698147535e-01
       coeff_l2(18,39)=  5.0098937750e-01
       coeff_l2(18,40)= -2.2924170494e+00
       coeff_l2(18,41)= -1.1743739843e+00
       coeff_l2(18,42)= -2.1873535216e-01
       coeff_l2(18,43)= -2.7080100775e-01
       coeff_l2(18,44)=  2.2678703070e-01
       coeff_l2(18,45)= -1.5877078772e+00
       coeff_l2(18,46)= -6.0276210308e-01
       coeff_l2(18,47)= -3.5574647784e-01
       coeff_l2(18,48)=  8.5897423327e-02
       coeff_l2(18,49)=  2.7249303460e-01
       coeff_l2(18,50)= -3.4178438783e-01
       coeff_l2(18,51)=  6.1242461205e-01
       coeff_l2(18,52)=  1.5614110231e-01
       coeff_l2(18,53)= -6.0777252913e-01
       coeff_l2(18,54)= -1.5020412207e-01
       coeff_l2(18,55)= -1.8606114388e-01
       coeff_l2(18,56)=  1.8985381350e-02
       coeff_l2(19,1)=  5.1628384739e-02
       coeff_l2(19,2)=  6.2868654728e-02
       coeff_l2(19,3)=  3.7074688077e-01
       coeff_l2(19,4)= -2.4211861193e-01
       coeff_l2(19,5)=  1.3781464100e-01
       coeff_l2(19,6)=  4.4432371855e-01
       coeff_l2(19,7)= -2.9557284713e-01
       coeff_l2(19,8)= -3.8608331233e-02
       coeff_l2(19,9)=  7.1407589130e-03
       coeff_l2(19,10)= -3.1130021811e-01
       coeff_l2(19,11)= -5.3895348310e-01
       coeff_l2(19,12)=  2.1650972962e-01
       coeff_l2(19,13)=  5.8500164747e-01
       coeff_l2(19,14)= -2.1324636042e-01
       coeff_l2(19,15)=  4.7219640017e-01
       coeff_l2(19,16)=  2.8342637420e-01
       coeff_l2(19,17)= -6.6022914648e-01
       coeff_l2(19,18)=  3.9384230971e-01
       coeff_l2(19,19)= -3.6458492279e-01
       coeff_l2(19,20)= -9.0225294232e-02
       coeff_l2(19,21)=  1.3623462617e-01
       coeff_l2(19,22)=  1.0387588292e-02
       coeff_l2(19,23)=  1.2380849570e-01
       coeff_l2(19,24)=  2.4916218221e-01
       coeff_l2(19,25)=  1.6255013645e-01
       coeff_l2(19,26)=  2.6869538426e-01
       coeff_l2(19,27)= -9.4639062881e-02
       coeff_l2(19,28)= -1.3818490505e-01
       coeff_l2(19,29)=  1.6073742509e-01
       coeff_l2(19,30)=  2.4787281454e-01
       coeff_l2(19,31)= -1.1719710380e-01
       coeff_l2(19,32)=  2.1086524427e-01
       coeff_l2(19,33)=  3.4495553374e-01
       coeff_l2(19,34)= -2.0616362989e-01
       coeff_l2(19,35)= -1.4345820248e-01
       coeff_l2(19,36)=  1.3210642338e-01
       coeff_l2(19,37)=  2.1643888950e-01
       coeff_l2(19,38)=  3.0576759577e-01
       coeff_l2(19,39)=  7.8674376011e-02
       coeff_l2(19,40)= -2.8961312771e-01
       coeff_l2(19,41)=  3.4745684266e-01
       coeff_l2(19,42)=  1.2611036003e-01
       coeff_l2(19,43)=  1.4165882766e-01
       coeff_l2(19,44)=  2.2984920442e-01
       coeff_l2(19,45)= -1.2029236555e-01
       coeff_l2(19,46)=  1.0414676741e-02
       coeff_l2(19,47)=  1.0721488297e-01
       coeff_l2(19,48)= -1.7506611347e-01
       coeff_l2(19,49)= -8.0653682351e-02
       coeff_l2(19,50)=  3.4685099125e-01
       coeff_l2(19,51)= -1.9484988879e-03
       coeff_l2(19,52)= -1.4907872677e-01
       coeff_l2(19,53)=  3.3450424671e-01
       coeff_l2(19,54)= -1.8966847658e-01
       coeff_l2(19,55)= -1.8800917268e-01
       coeff_l2(19,56)=  1.2065610290e-01
       coeff_l2(20,1)=  3.6617267132e-01
       coeff_l2(20,2)= -2.4975825846e-01
       coeff_l2(20,3)= -5.5282449722e-01
       coeff_l2(20,4)= -1.5878157616e+00
       coeff_l2(20,5)= -1.5894879103e+00
       coeff_l2(20,6)= -1.0667201281e+00
       coeff_l2(20,7)= -1.3939914703e+00
       coeff_l2(20,8)=  5.1455199718e-01
       coeff_l2(20,9)=  6.9882464409e-01
       coeff_l2(20,10)=  1.3068497181e-01
       coeff_l2(20,11)=  4.2155060172e-01
       coeff_l2(20,12)=  6.2182283401e-01
       coeff_l2(20,13)=  4.8362368345e-01
       coeff_l2(20,14)= -2.4947037920e-02
       coeff_l2(20,15)= -2.0278547704e-01
       coeff_l2(20,16)= -8.4835398197e-01
       coeff_l2(20,17)= -7.8951632977e-01
       coeff_l2(20,18)= -1.9471719265e+00
       coeff_l2(20,19)= -1.7503453493e+00
       coeff_l2(20,20)= -3.9195099473e-01
       coeff_l2(20,21)=  3.9816549420e-01
       coeff_l2(20,22)= -7.0246741176e-02
       coeff_l2(20,23)=  7.1665489674e-01
       coeff_l2(20,24)=  2.3860648274e-01
       coeff_l2(20,25)= -2.7313226461e-01
       coeff_l2(20,26)=  7.0492690802e-01
       coeff_l2(20,27)= -1.2523937225e-01
       coeff_l2(20,28)= -3.1655073166e-01
       coeff_l2(20,29)=  1.7708662152e-01
       coeff_l2(20,30)= -8.1459736824e-01
       coeff_l2(20,31)= -1.0327560902e+00
       coeff_l2(20,32)=  1.3983399868e+00
       coeff_l2(20,33)=  2.2769081593e-01
       coeff_l2(20,34)=  7.9898542166e-01
       coeff_l2(20,35)= -9.5274651051e-01
       coeff_l2(20,36)=  1.1625565290e+00
       coeff_l2(20,37)=  8.3999119699e-02
       coeff_l2(20,38)= -1.1125502586e+00
       coeff_l2(20,39)= -9.0404945612e-01
       coeff_l2(20,40)=  8.8098078966e-01
       coeff_l2(20,41)= -1.0820320845e+00
       coeff_l2(20,42)=  3.8911413401e-02
       coeff_l2(20,43)=  9.9729178473e-03
       coeff_l2(20,44)=  6.2817353010e-01
       coeff_l2(20,45)= -1.2242969275e+00
       coeff_l2(20,46)= -1.8727394342e+00
       coeff_l2(20,47)=  2.1558146179e-01
       coeff_l2(20,48)= -6.9933705032e-02
       coeff_l2(20,49)= -5.0663554668e-01
       coeff_l2(20,50)=  3.4134247899e-01
       coeff_l2(20,51)=  5.5159259588e-02
       coeff_l2(20,52)= -1.9667305052e-01
       coeff_l2(20,53)= -6.5239772201e-02
       coeff_l2(20,54)= -7.7912193537e-01
       coeff_l2(20,55)= -1.0165731609e-01
       coeff_l2(20,56)= -1.4229676723e+00
       coeff_l2(21,1)=  4.7558620572e-02
       coeff_l2(21,2)=  2.7703553438e-02
       coeff_l2(21,3)=  5.0442282110e-02
       coeff_l2(21,4)=  7.6768380404e-01
       coeff_l2(21,5)=  1.2714761309e-02
       coeff_l2(21,6)= -4.0415748954e-02
       coeff_l2(21,7)=  5.9543401003e-01
       coeff_l2(21,8)=  2.1231079102e-01
       coeff_l2(21,9)= -4.3805003166e-01
       coeff_l2(21,10)=  7.3933891952e-02
       coeff_l2(21,11)= -1.9112546742e-01
       coeff_l2(21,12)= -5.6275397539e-02
       coeff_l2(21,13)=  2.3719187826e-02
       coeff_l2(21,14)=  2.1973920520e-03
       coeff_l2(21,15)=  5.9539884329e-01
       coeff_l2(21,16)= -1.5639955997e+00
       coeff_l2(21,17)= -7.4845172465e-02
       coeff_l2(21,18)= -1.0442671776e+01
       coeff_l2(21,19)=  5.4765272141e-01
       coeff_l2(21,20)=  3.1296768785e-01
       coeff_l2(21,21)=  2.7718871832e-01
       coeff_l2(21,22)= -3.4116964787e-02
       coeff_l2(21,23)=  6.1657875776e-01
       coeff_l2(21,24)=  1.1217122525e-01
       coeff_l2(21,25)= -2.0038782060e-01
       coeff_l2(21,26)= -7.7720060945e-02
       coeff_l2(21,27)=  5.3096145391e-01
       coeff_l2(21,28)= -4.0431857109e-01
       coeff_l2(21,29)=  5.1482832432e-01
       coeff_l2(21,30)=  3.9299419522e-01
       coeff_l2(21,31)= -2.2114245892e+00
       coeff_l2(21,32)= -1.8310263753e-01
       coeff_l2(21,33)=  5.3229708225e-02
       coeff_l2(21,34)= -2.7289895341e-02
       coeff_l2(21,35)= -2.1798168123e-01
       coeff_l2(21,36)= -6.0037393123e-02
       coeff_l2(21,37)= -1.9173258543e-01
       coeff_l2(21,38)= -2.1752695739e-01
       coeff_l2(21,39)=  7.6127544045e-02
       coeff_l2(21,40)=  3.3047739416e-03
       coeff_l2(21,41)=  1.7755824327e-01
       coeff_l2(21,42)= -4.5943748206e-02
       coeff_l2(21,43)=  7.2382211685e-02
       coeff_l2(21,44)= -1.2443134934e-01
       coeff_l2(21,45)=  1.2116280794e+00
       coeff_l2(21,46)=  3.7087804079e-01
       coeff_l2(21,47)=  1.3081811368e-01
       coeff_l2(21,48)=  5.9897344559e-02
       coeff_l2(21,49)=  3.3899840713e-01
       coeff_l2(21,50)= -3.6734005809e-01
       coeff_l2(21,51)= -3.1963281631e+00
       coeff_l2(21,52)=  3.3479064703e-02
       coeff_l2(21,53)= -7.3297113180e-02
       coeff_l2(21,54)=  3.4979742020e-02
       coeff_l2(21,55)=  3.7021914124e-01
       coeff_l2(21,56)=  4.7118371725e-01
       coeff_l2(22,1)= -5.3173226118e-01
       coeff_l2(22,2)= -9.8631203175e-02
       coeff_l2(22,3)= -1.1203873158e-01
       coeff_l2(22,4)= -6.2012939453e+00
       coeff_l2(22,5)=  3.0120787621e+00
       coeff_l2(22,6)= -3.6759811640e-01
       coeff_l2(22,7)= -1.5539296865e+00
       coeff_l2(22,8)=  4.0241947770e-01
       coeff_l2(22,9)= -3.8686880469e-01
       coeff_l2(22,10)= -7.3484754562e-01
       coeff_l2(22,11)=  2.3498509824e-01
       coeff_l2(22,12)= -6.4945757389e-01
       coeff_l2(22,13)=  1.1763031483e+00
       coeff_l2(22,14)= -1.0997784138e+00
       coeff_l2(22,15)= -1.6029156744e-01
       coeff_l2(22,16)= -7.1746206284e-01
       coeff_l2(22,17)=  1.2676620483e-01
       coeff_l2(22,18)=  3.3956825733e-01
       coeff_l2(22,19)= -1.3355574608e+00
       coeff_l2(22,20)= -6.6585963964e-01
       coeff_l2(22,21)= -8.0329728127e-01
       coeff_l2(22,22)= -8.5632967949e-01
       coeff_l2(22,23)=  1.6807343960e+00
       coeff_l2(22,24)= -6.4366757870e-01
       coeff_l2(22,25)= -1.0176005214e-01
       coeff_l2(22,26)=  2.5991049409e-01
       coeff_l2(22,27)= -1.5022708178e+00
       coeff_l2(22,28)=  1.7935715616e-01
       coeff_l2(22,29)= -8.3394497633e-01
       coeff_l2(22,30)= -2.1056115627e+00
       coeff_l2(22,31)= -1.6165246964e+00
       coeff_l2(22,32)= -1.9366154671e+00
       coeff_l2(22,33)= -3.0054894090e-01
       coeff_l2(22,34)= -9.2960131168e-01
       coeff_l2(22,35)= -1.0866595507e+00
       coeff_l2(22,36)= -4.6144524217e-01
       coeff_l2(22,37)=  1.3534787297e-01
       coeff_l2(22,38)=  8.9952760935e-01
       coeff_l2(22,39)= -5.9263437986e-01
       coeff_l2(22,40)= -5.8346588165e-02
       coeff_l2(22,41)= -1.0254321098e+00
       coeff_l2(22,42)=  6.1130857468e-01
       coeff_l2(22,43)=  4.7363780439e-02
       coeff_l2(22,44)= -1.2027526647e-01
       coeff_l2(22,45)= -2.2969391346e+00
       coeff_l2(22,46)=  1.4125652313e+00
       coeff_l2(22,47)= -7.2609198093e-01
       coeff_l2(22,48)= -1.5988781452e+00
       coeff_l2(22,49)= -1.4414367676e+00
       coeff_l2(22,50)=  2.6083593369e+00
       coeff_l2(22,51)= -2.4797668457e+00
       coeff_l2(22,52)=  3.0070251226e-01
       coeff_l2(22,53)=  4.9382064492e-02
       coeff_l2(22,54)=  4.6441501379e-01
       coeff_l2(22,55)= -1.1981439590e+00
       coeff_l2(22,56)= -2.6360473633e+00
       coeff_l2(23,1)= -1.5609544516e-01
       coeff_l2(23,2)= -1.4057475142e-02
       coeff_l2(23,3)=  1.9382253289e-01
       coeff_l2(23,4)= -3.1658661366e+00
       coeff_l2(23,5)= -3.7845920771e-02
       coeff_l2(23,6)= -1.7662150860e+00
       coeff_l2(23,7)=  9.7655844688e-01
       coeff_l2(23,8)= -5.4745255038e-03
       coeff_l2(23,9)= -5.6803530455e-01
       coeff_l2(23,10)= -5.7565617561e-01
       coeff_l2(23,11)= -2.5565913320e-01
       coeff_l2(23,12)=  4.1418430209e-01
       coeff_l2(23,13)=  1.6860415041e-01
       coeff_l2(23,14)=  1.4670777321e-01
       coeff_l2(23,15)=  4.3481954932e-01
       coeff_l2(23,16)= -3.8195374608e-01
       coeff_l2(23,17)= -1.7953996658e+00
       coeff_l2(23,18)=  3.8308742642e-01
       coeff_l2(23,19)=  2.8023189306e-01
       coeff_l2(23,20)=  6.0289841890e-01
       coeff_l2(23,21)=  1.8558266759e-01
       coeff_l2(23,22)= -1.9643636942e+00
       coeff_l2(23,23)= -6.3976836205e-01
       coeff_l2(23,24)=  3.9208200574e-01
       coeff_l2(23,25)=  5.7877909392e-02
       coeff_l2(23,26)= -2.0100231469e-01
       coeff_l2(23,27)= -3.0748618767e-02
       coeff_l2(23,28)= -1.9109147787e-01
       coeff_l2(23,29)=  5.7620372623e-02
       coeff_l2(23,30)= -7.8471535444e-01
       coeff_l2(23,31)=  3.2206088305e-01
       coeff_l2(23,32)= -4.4317710400e-01
       coeff_l2(23,33)= -1.9270551205e-01
       coeff_l2(23,34)= -2.0027533174e-01
       coeff_l2(23,35)=  3.4424358606e-01
       coeff_l2(23,36)= -2.4603555202e+00
       coeff_l2(23,37)=  5.4007351398e-01
       coeff_l2(23,38)= -1.1189734936e-01
       coeff_l2(23,39)= -6.7366869189e-03
       coeff_l2(23,40)= -1.1156748980e-01
       coeff_l2(23,41)=  5.6170042604e-02
       coeff_l2(23,42)=  8.0296032131e-02
       coeff_l2(23,43)= -2.9120373726e-01
       coeff_l2(23,44)= -1.6667880118e-02
       coeff_l2(23,45)= -1.5917487144e+00
       coeff_l2(23,46)=  8.2101517916e-01
       coeff_l2(23,47)=  2.8154495358e-01
       coeff_l2(23,48)= -4.1883956641e-02
       coeff_l2(23,49)=  5.6287676096e-01
       coeff_l2(23,50)=  4.9773332477e-01
       coeff_l2(23,51)= -7.0340472460e-01
       coeff_l2(23,52)=  8.5982006788e-01
       coeff_l2(23,53)= -3.6363968253e-01
       coeff_l2(23,54)= -6.7478820682e-02
       coeff_l2(23,55)=  8.4829258919e-01
       coeff_l2(23,56)= -1.8294496834e-01
       coeff_l2(24,1)=  2.4483051896e-01
       coeff_l2(24,2)= -1.3222434521e+00
       coeff_l2(24,3)=  4.0823981166e-01
       coeff_l2(24,4)=  4.1081571579e-01
       coeff_l2(24,5)= -5.3964555264e-01
       coeff_l2(24,6)=  2.8328368068e-01
       coeff_l2(24,7)= -6.5444374084e-01
       coeff_l2(24,8)=  4.8837351799e-01
       coeff_l2(24,9)=  2.5053903461e-01
       coeff_l2(24,10)= -1.0126239061e+00
       coeff_l2(24,11)= -5.3586256504e-01
       coeff_l2(24,12)= -8.1743085384e-01
       coeff_l2(24,13)=  6.7909753323e-01
       coeff_l2(24,14)= -2.2355918586e-01
       coeff_l2(24,15)=  8.3957362175e-01
       coeff_l2(24,16)= -1.1380354166e+00
       coeff_l2(24,17)= -9.1532725096e-01
       coeff_l2(24,18)=  3.0271620750e+00
       coeff_l2(24,19)= -9.2879784107e-01
       coeff_l2(24,20)= -5.7985252142e-01
       coeff_l2(24,21)=  7.2240179777e-01
       coeff_l2(24,22)= -3.1619358063e-01
       coeff_l2(24,23)= -9.8690259457e-01
       coeff_l2(24,24)= -1.9688333571e-01
       coeff_l2(24,25)=  3.7504279613e-01
       coeff_l2(24,26)=  7.1075111628e-01
       coeff_l2(24,27)= -4.5515474677e-01
       coeff_l2(24,28)=  7.2853541374e-01
       coeff_l2(24,29)= -3.0427843332e-01
       coeff_l2(24,30)=  2.2128249705e-01
       coeff_l2(24,31)= -1.6093230247e+00
       coeff_l2(24,32)= -3.0984634161e-01
       coeff_l2(24,33)=  3.5366490483e-01
       coeff_l2(24,34)= -8.4939426184e-01
       coeff_l2(24,35)= -1.2687909603e+00
       coeff_l2(24,36)= -4.2480909824e-01
       coeff_l2(24,37)=  2.7623641491e-01
       coeff_l2(24,38)=  7.3865103722e-01
       coeff_l2(24,39)= -1.0469147563e-01
       coeff_l2(24,40)=  4.9820253253e-01
       coeff_l2(24,41)=  1.1438909769e+00
       coeff_l2(24,42)= -1.7641055584e-01
       coeff_l2(24,43)=  1.4227066934e-01
       coeff_l2(24,44)=  9.2544215918e-01
       coeff_l2(24,45)= -9.7700893879e-01
       coeff_l2(24,46)= -1.2413799763e+00
       coeff_l2(24,47)= -4.1879188269e-02
       coeff_l2(24,48)= -3.8287901878e-01
       coeff_l2(24,49)= -8.3783566952e-01
       coeff_l2(24,50)= -4.7820675373e-01
       coeff_l2(24,51)=  1.4443334341e+00
       coeff_l2(24,52)=  7.7987059951e-02
       coeff_l2(24,53)=  3.5580703616e-01
       coeff_l2(24,54)= -1.2621295452e-01
       coeff_l2(24,55)= -7.4850499630e-01
       coeff_l2(24,56)= -3.5790090561e+00
       coeff_l2(25,1)= -1.1457078904e-01
       coeff_l2(25,2)= -2.1460369229e-01
       coeff_l2(25,3)= -4.2007375509e-02
       coeff_l2(25,4)= -4.0192243457e-01
       coeff_l2(25,5)=  3.7600874901e-01
       coeff_l2(25,6)= -1.4078439474e+00
       coeff_l2(25,7)= -3.6281657219e-01
       coeff_l2(25,8)= -2.3728670552e-02
       coeff_l2(25,9)=  4.4354531914e-02
       coeff_l2(25,10)= -1.7255020142e-01
       coeff_l2(25,11)= -3.7015753984e-01
       coeff_l2(25,12)= -8.0564394593e-02
       coeff_l2(25,13)= -5.6526255608e-01
       coeff_l2(25,14)=  2.8500652313e-01
       coeff_l2(25,15)=  3.4692566842e-02
       coeff_l2(25,16)= -8.2798266411e-01
       coeff_l2(25,17)= -5.4086190462e-01
       coeff_l2(25,18)= -1.8528501987e+00
       coeff_l2(25,19)= -5.3963953257e-01
       coeff_l2(25,20)= -3.6392948031e-01
       coeff_l2(25,21)=  1.2142643332e-01
       coeff_l2(25,22)= -4.1381031275e-01
       coeff_l2(25,23)= -6.7920118570e-01
       coeff_l2(25,24)=  1.9268433750e-01
       coeff_l2(25,25)=  3.1643658876e-01
       coeff_l2(25,26)= -4.6813935041e-01
       coeff_l2(25,27)= -2.5790652633e-01
       coeff_l2(25,28)= -1.6851475835e-01
       coeff_l2(25,29)=  7.2422020137e-02
       coeff_l2(25,30)= -4.7333380580e-01
       coeff_l2(25,31)= -2.1195127070e-01
       coeff_l2(25,32)=  8.8441774249e-02
       coeff_l2(25,33)= -5.9241730720e-02
       coeff_l2(25,34)=  2.3872864246e-01
       coeff_l2(25,35)= -1.0061170161e-01
       coeff_l2(25,36)= -1.8804438412e-01
       coeff_l2(25,37)= -3.7222349644e-01
       coeff_l2(25,38)= -1.5796533227e-01
       coeff_l2(25,39)=  2.3088114336e-02
       coeff_l2(25,40)= -5.5601835251e-01
       coeff_l2(25,41)= -1.6426925361e-01
       coeff_l2(25,42)= -4.5658347011e-01
       coeff_l2(25,43)= -1.4314880967e-01
       coeff_l2(25,44)=  1.6461393237e-01
       coeff_l2(25,45)= -1.4917730093e+00
       coeff_l2(25,46)= -7.1929657459e-01
       coeff_l2(25,47)= -3.3832040429e-01
       coeff_l2(25,48)= -2.2779671848e-01
       coeff_l2(25,49)= -2.2607867420e-01
       coeff_l2(25,50)=  8.1114369631e-01
       coeff_l2(25,51)=  6.1630092561e-02
       coeff_l2(25,52)= -3.3623823524e-01
       coeff_l2(25,53)=  4.8666194081e-02
       coeff_l2(25,54)= -1.6130989790e-01
       coeff_l2(25,55)= -1.9485552609e-01
       coeff_l2(25,56)= -8.8659167290e-01
       coeff_l2(26,1)= -1.0030134767e-01
       coeff_l2(26,2)=  1.5400458872e-01
       coeff_l2(26,3)=  3.7192577124e-01
       coeff_l2(26,4)= -7.1473580599e-01
       coeff_l2(26,5)=  1.1594245583e-01
       coeff_l2(26,6)= -5.6481063366e-01
       coeff_l2(26,7)= -4.4255286455e-01
       coeff_l2(26,8)=  2.8798583150e-01
       coeff_l2(26,9)=  2.3861438036e-01
       coeff_l2(26,10)=  9.3589527532e-03
       coeff_l2(26,11)= -5.4424780607e-01
       coeff_l2(26,12)=  1.1744649708e-01
       coeff_l2(26,13)=  8.0102682114e-01
       coeff_l2(26,14)=  3.6359801888e-01
       coeff_l2(26,15)=  2.0817130804e-01
       coeff_l2(26,16)=  8.9553058147e-01
       coeff_l2(26,17)=  5.7640150189e-02
       coeff_l2(26,18)=  2.8856511116e+00
       coeff_l2(26,19)= -7.3135608435e-01
       coeff_l2(26,20)= -8.5261803865e-01
       coeff_l2(26,21)= -8.2600378990e-01
       coeff_l2(26,22)= -1.7165684700e+00
       coeff_l2(26,23)= -9.7330242395e-01
       coeff_l2(26,24)=  5.7180441916e-02
       coeff_l2(26,25)=  6.8498820066e-01
       coeff_l2(26,26)=  3.5799586773e-01
       coeff_l2(26,27)= -4.3322160840e-01
       coeff_l2(26,28)= -2.4275878072e-01
       coeff_l2(26,29)=  6.1415065080e-02
       coeff_l2(26,30)= -3.7166514993e-01
       coeff_l2(26,31)= -5.2928602695e-01
       coeff_l2(26,32)= -1.3670244217e+00
       coeff_l2(26,33)=  5.0591671467e-01
       coeff_l2(26,34)= -6.8947565556e-01
       coeff_l2(26,35)=  2.6172375679e-01
       coeff_l2(26,36)=  5.7978731394e-01
       coeff_l2(26,37)=  4.4353231788e-01
       coeff_l2(26,38)=  6.5927368402e-01
       coeff_l2(26,39)= -4.3990290165e-01
       coeff_l2(26,40)= -2.8500482440e-01
       coeff_l2(26,41)=  7.8808683157e-01
       coeff_l2(26,42)= -1.5587604046e-01
       coeff_l2(26,43)=  4.0205362439e-01
       coeff_l2(26,44)=  6.3912469149e-01
       coeff_l2(26,45)=  1.0136006027e-01
       coeff_l2(26,46)= -5.7986193895e-01
       coeff_l2(26,47)=  1.7181871831e-01
       coeff_l2(26,48)=  2.6288527250e-01
       coeff_l2(26,49)= -5.8103740215e-01
       coeff_l2(26,50)=  1.3376468420e+00
       coeff_l2(26,51)=  1.6819241047e+00
       coeff_l2(26,52)=  8.3852224052e-03
       coeff_l2(26,53)= -5.5893128738e-03
       coeff_l2(26,54)= -4.5516800135e-02
       coeff_l2(26,55)= -8.8444560766e-01
       coeff_l2(26,56)= -3.1045289040e+00
       coeff_l2(27,1)= -1.1959671974e-01
       coeff_l2(27,2)=  1.3172498345e-01
       coeff_l2(27,3)=  1.6235610843e-01
       coeff_l2(27,4)= -2.4880020618e+00
       coeff_l2(27,5)=  1.4322735071e+00
       coeff_l2(27,6)= -4.5606753230e-01
       coeff_l2(27,7)= -8.6097723246e-01
       coeff_l2(27,8)= -4.1285414249e-02
       coeff_l2(27,9)=  3.5976728797e-01
       coeff_l2(27,10)=  2.3088233545e-02
       coeff_l2(27,11)= -4.7433492541e-01
       coeff_l2(27,12)=  2.5722411275e-01
       coeff_l2(27,13)=  7.4825149775e-01
       coeff_l2(27,14)=  1.5248866379e-01
       coeff_l2(27,15)= -5.5595320463e-01
       coeff_l2(27,16)= -4.6410247684e-01
       coeff_l2(27,17)= -7.0594888926e-01
       coeff_l2(27,18)=  3.4421381354e-01
       coeff_l2(27,19)= -1.6208819151e+00
       coeff_l2(27,20)= -8.2326662540e-01
       coeff_l2(27,21)= -9.1845788062e-02
       coeff_l2(27,22)= -1.2445356846e+00
       coeff_l2(27,23)= -1.0498918295e+00
       coeff_l2(27,24)= -6.7496347427e-01
       coeff_l2(27,25)= -2.3095360026e-02
       coeff_l2(27,26)=  7.8523419797e-02
       coeff_l2(27,27)= -4.0956941247e-01
       coeff_l2(27,28)=  1.2335060537e-01
       coeff_l2(27,29)=  1.9211661071e-02
       coeff_l2(27,30)= -7.8041774035e-01
       coeff_l2(27,31)= -7.1987253428e-01
       coeff_l2(27,32)= -7.2588467598e-01
       coeff_l2(27,33)=  1.8871346116e-01
       coeff_l2(27,34)=  3.8193935156e-01
       coeff_l2(27,35)= -2.8079780936e-01
       coeff_l2(27,36)=  2.8845316172e-01
       coeff_l2(27,37)=  3.8156136870e-01
       coeff_l2(27,38)= -3.6539486051e-01
       coeff_l2(27,39)= -7.3770266771e-01
       coeff_l2(27,40)= -4.7518411279e-01
       coeff_l2(27,41)=  8.8218945265e-01
       coeff_l2(27,42)= -6.0312628746e-01
       coeff_l2(27,43)= -1.1430348456e-01
       coeff_l2(27,44)=  3.0728161335e-01
       coeff_l2(27,45)= -8.2871532440e-01
       coeff_l2(27,46)= -8.6081492901e-01
       coeff_l2(27,47)=  5.1849168539e-01
       coeff_l2(27,48)=  1.0864943266e-01
       coeff_l2(27,49)= -9.3318462372e-01
       coeff_l2(27,50)=  2.3540978432e+00
       coeff_l2(27,51)= -9.5111846924e-02
       coeff_l2(27,52)=  2.1840831637e-01
       coeff_l2(27,53)= -3.4506997466e-01
       coeff_l2(27,54)= -1.0777153969e+00
       coeff_l2(27,55)= -9.0776908398e-01
       coeff_l2(27,56)= -6.3961803913e-01
       coeff_l2(28,1)=  3.2824996114e-01
       coeff_l2(28,2)= -2.2476591170e-02
       coeff_l2(28,3)=  2.5768529624e-02
       coeff_l2(28,4)= -4.2035982013e-01
       coeff_l2(28,5)=  2.3184068203e+00
       coeff_l2(28,6)= -1.7108111382e+00
       coeff_l2(28,7)=  6.9751197100e-01
       coeff_l2(28,8)=  3.7941849232e-01
       coeff_l2(28,9)= -5.6810867041e-02
       coeff_l2(28,10)= -8.1137561798e-01
       coeff_l2(28,11)=  4.0555042215e-03
       coeff_l2(28,12)= -4.2543992400e-01
       coeff_l2(28,13)=  4.2239609361e-01
       coeff_l2(28,14)=  6.5172499418e-01
       coeff_l2(28,15)= -7.4672645330e-01
       coeff_l2(28,16)= -3.1037172675e-01
       coeff_l2(28,17)= -2.4258184433e+00
       coeff_l2(28,18)= -2.0419175625e+00
       coeff_l2(28,19)= -3.8701289892e-01
       coeff_l2(28,20)=  1.0038311481e+00
       coeff_l2(28,21)= -5.1527645439e-02
       coeff_l2(28,22)= -1.4603869915e+00
       coeff_l2(28,23)= -8.6434468627e-02
       coeff_l2(28,24)=  2.9271746054e-02
       coeff_l2(28,25)= -7.6013155282e-02
       coeff_l2(28,26)= -2.9258814454e-01
       coeff_l2(28,27)= -1.3214425743e-01
       coeff_l2(28,28)= -6.4098525047e-01
       coeff_l2(28,29)=  7.6322264969e-02
       coeff_l2(28,30)=  1.3785398006e-01
       coeff_l2(28,31)= -6.5330219269e-01
       coeff_l2(28,32)= -7.7569492161e-02
       coeff_l2(28,33)=  4.1852006316e-01
       coeff_l2(28,34)= -1.9713990390e-02
       coeff_l2(28,35)= -5.2837848663e-02
       coeff_l2(28,36)= -5.6270217896e-01
       coeff_l2(28,37)= -4.6242532134e-01
       coeff_l2(28,38)= -7.3248761892e-01
       coeff_l2(28,39)= -1.5822038651e+00
       coeff_l2(28,40)=  3.5353344679e-01
       coeff_l2(28,41)= -1.0366019011e+00
       coeff_l2(28,42)= -6.6000390053e-01
       coeff_l2(28,43)=  4.7249540687e-01
       coeff_l2(28,44)= -4.6333399415e-01
       coeff_l2(28,45)= -7.7948051691e-01
       coeff_l2(28,46)= -3.7074357271e-01
       coeff_l2(28,47)=  4.0113475919e-01
       coeff_l2(28,48)= -2.8140786290e-01
       coeff_l2(28,49)= -2.3939874768e-01
       coeff_l2(28,50)=  1.2231758833e+00
       coeff_l2(28,51)= -4.2441725731e-01
       coeff_l2(28,52)=  2.2397856414e-01
       coeff_l2(28,53)= -7.4669122696e-02
       coeff_l2(28,54)= -1.5839308500e+00
       coeff_l2(28,55)=  8.4001725912e-01
       coeff_l2(28,56)=  1.0615903139e+00
       coeff_l2(29,1)= -3.8576010615e-02
       coeff_l2(29,2)=  1.6759464145e-01
       coeff_l2(29,3)=  1.1264438182e-01
       coeff_l2(29,4)= -2.3411958218e+00
       coeff_l2(29,5)= -4.6994206309e-01
       coeff_l2(29,6)= -4.9552184343e-01
       coeff_l2(29,7)= -9.6435427666e-01
       coeff_l2(29,8)=  5.4161334038e-01
       coeff_l2(29,9)=  2.7787479758e-01
       coeff_l2(29,10)= -2.1551427245e-01
       coeff_l2(29,11)=  2.1730446815e-01
       coeff_l2(29,12)= -6.2859818339e-02
       coeff_l2(29,13)=  2.9254186153e-01
       coeff_l2(29,14)= -5.6955331564e-01
       coeff_l2(29,15)= -1.1667593569e-01
       coeff_l2(29,16)=  1.0050247908e+00
       coeff_l2(29,17)= -8.4097528458e-01
       coeff_l2(29,18)= -2.2269554138e+00
       coeff_l2(29,19)= -9.8675823212e-01
       coeff_l2(29,20)= -8.6208784580e-01
       coeff_l2(29,21)=  1.0785464048e+00
       coeff_l2(29,22)=  3.9575695992e-01
       coeff_l2(29,23)=  2.6337473392e+00
       coeff_l2(29,24)= -1.4348341525e-01
       coeff_l2(29,25)=  6.3392168283e-01
       coeff_l2(29,26)=  2.4686463177e-01
       coeff_l2(29,27)= -6.0429888964e-01
       coeff_l2(29,28)=  4.5597195625e-01
       coeff_l2(29,29)=  6.6049583256e-02
       coeff_l2(29,30)= -4.3493482471e-01
       coeff_l2(29,31)= -8.9816862345e-01
       coeff_l2(29,32)=  1.4102603197e+00
       coeff_l2(29,33)= -6.7466482520e-02
       coeff_l2(29,34)= -2.3121580482e-01
       coeff_l2(29,35)= -9.3498569727e-01
       coeff_l2(29,36)= -7.8252039850e-02
       coeff_l2(29,37)=  5.2014231682e-01
       coeff_l2(29,38)=  1.6685435176e-01
       coeff_l2(29,39)= -4.3386894464e-01
       coeff_l2(29,40)= -1.0479079559e-02
       coeff_l2(29,41)=  2.2169296443e-01
       coeff_l2(29,42)= -4.4413775206e-01
       coeff_l2(29,43)=  1.3658688962e-01
       coeff_l2(29,44)=  2.4558332562e-01
       coeff_l2(29,45)=  2.9528853297e-01
       coeff_l2(29,46)=  2.3699136451e-02
       coeff_l2(29,47)= -1.7842124403e-01
       coeff_l2(29,48)=  4.3901944160e-01
       coeff_l2(29,49)= -1.9007234573e+00
       coeff_l2(29,50)=  1.1720073223e+00
       coeff_l2(29,51)= -6.5899932384e-01
       coeff_l2(29,52)= -6.0190993547e-01
       coeff_l2(29,53)= -3.6280238628e-01
       coeff_l2(29,54)=  4.1106319427e-01
       coeff_l2(29,55)= -6.6624230146e-01
       coeff_l2(29,56)= -5.4527006149e+00
       coeff_l2(30,1)=  5.0822919607e-01
       coeff_l2(30,2)=  5.8946514130e-01
       coeff_l2(30,3)=  2.2167704999e-01
       coeff_l2(30,4)= -1.5787295103e+00
       coeff_l2(30,5)=  7.0993649960e-01
       coeff_l2(30,6)= -1.0738815069e+00
       coeff_l2(30,7)= -6.0315697920e-04
       coeff_l2(30,8)= -1.6793367267e-01
       coeff_l2(30,9)=  8.1765174866e-02
       coeff_l2(30,10)=  2.0548932254e-01
       coeff_l2(30,11)= -1.1215559244e+00
       coeff_l2(30,12)= -4.6527048945e-01
       coeff_l2(30,13)= -5.5595308542e-01
       coeff_l2(30,14)= -5.1243826747e-02
       coeff_l2(30,15)=  4.0037444234e-01
       coeff_l2(30,16)= -1.6251289845e+00
       coeff_l2(30,17)= -9.1549813747e-01
       coeff_l2(30,18)= -1.4101805687e+00
       coeff_l2(30,19)= -4.0917256474e-01
       coeff_l2(30,20)=  3.3236876130e-02
       coeff_l2(30,21)= -1.0766071081e-01
       coeff_l2(30,22)= -1.4691185951e+00
       coeff_l2(30,23)= -1.8763967752e+00
       coeff_l2(30,24)=  8.2474507391e-02
       coeff_l2(30,25)= -5.9581352398e-03
       coeff_l2(30,26)= -5.4754817486e-01
       coeff_l2(30,27)=  2.2773213685e-01
       coeff_l2(30,28)= -5.3936696053e-01
       coeff_l2(30,29)=  4.8578515649e-02
       coeff_l2(30,30)= -7.4801492691e-01
       coeff_l2(30,31)= -6.0146771371e-02
       coeff_l2(30,32)= -1.4530332088e+00
       coeff_l2(30,33)=  2.2542674839e-01
       coeff_l2(30,34)=  5.1843839884e-01
       coeff_l2(30,35)= -2.4258990586e-01
       coeff_l2(30,36)= -1.4030139148e-01
       coeff_l2(30,37)=  1.8092222512e-01
       coeff_l2(30,38)= -8.8391232491e-01
       coeff_l2(30,39)=  3.3317312598e-01
       coeff_l2(30,40)=  6.3169747591e-01
       coeff_l2(30,41)= -4.8895108700e-01
       coeff_l2(30,42)= -2.6705196500e-01
       coeff_l2(30,43)=  5.6467682123e-01
       coeff_l2(30,44)=  3.3630605787e-02
       coeff_l2(30,45)= -2.0719988346e+00
       coeff_l2(30,46)= -1.5390962362e+00
       coeff_l2(30,47)= -4.7593590617e-01
       coeff_l2(30,48)= -4.8656795174e-02
       coeff_l2(30,49)=  3.9669370651e-01
       coeff_l2(30,50)= -2.3622204363e-01
       coeff_l2(30,51)= -3.5432338715e-01
       coeff_l2(30,52)=  8.7678128481e-01
       coeff_l2(30,53)= -4.2054021358e-01
       coeff_l2(30,54)= -3.9829269052e-01
       coeff_l2(30,55)=  3.8403627276e-01
       coeff_l2(30,56)= -2.8634113073e-01
       coeff_l2(31,1)= -6.6664479673e-02
       coeff_l2(31,2)=  3.6347556859e-02
       coeff_l2(31,3)=  2.3765017092e-01
       coeff_l2(31,4)= -3.9669829607e-01
       coeff_l2(31,5)= -1.3811625540e-01
       coeff_l2(31,6)= -2.8054383397e-01
       coeff_l2(31,7)=  1.8670965433e+00
       coeff_l2(31,8)= -1.3229371607e-01
       coeff_l2(31,9)= -2.8838264942e-01
       coeff_l2(31,10)= -3.7069220096e-02
       coeff_l2(31,11)= -2.8927028179e-01
       coeff_l2(31,12)=  4.2518611997e-02
       coeff_l2(31,13)=  5.0622588396e-01
       coeff_l2(31,14)=  1.8910504580e+00
       coeff_l2(31,15)= -9.6476143599e-01
       coeff_l2(31,16)= -3.2706943154e-01
       coeff_l2(31,17)= -9.9289709330e-01
       coeff_l2(31,18)=  1.1471365690e+00
       coeff_l2(31,19)=  2.3503401279e+00
       coeff_l2(31,20)=  8.5776948929e-01
       coeff_l2(31,21)= -2.3880898952e-01
       coeff_l2(31,22)= -5.0790238380e-01
       coeff_l2(31,23)= -1.9562988281e+00
       coeff_l2(31,24)= -1.7121580243e-01
       coeff_l2(31,25)=  6.4562015235e-02
       coeff_l2(31,26)= -1.1774577945e-01
       coeff_l2(31,27)=  9.4412255287e-01
       coeff_l2(31,28)= -1.6902342439e-01
       coeff_l2(31,29)= -4.0666162968e-01
       coeff_l2(31,30)=  1.5813486576e+00
       coeff_l2(31,31)= -7.1489977837e-01
       coeff_l2(31,32)= -1.2951524258e+00
       coeff_l2(31,33)= -1.2471234053e-01
       coeff_l2(31,34)= -1.7675565183e-01
       coeff_l2(31,35)=  7.0750004053e-01
       coeff_l2(31,36)= -2.8243243694e-01
       coeff_l2(31,37)= -3.6353108287e-01
       coeff_l2(31,38)= -7.0734888315e-01
       coeff_l2(31,39)=  2.1842211485e-01
       coeff_l2(31,40)= -1.3038694859e-01
       coeff_l2(31,41)=  1.5632269382e+00
       coeff_l2(31,42)= -2.2937211394e-01
       coeff_l2(31,43)= -5.2902463824e-02
       coeff_l2(31,44)=  3.0946257710e-01
       coeff_l2(31,45)= -1.3079038262e-01
       coeff_l2(31,46)=  2.5308892727e+00
       coeff_l2(31,47)=  2.1476478875e-01
       coeff_l2(31,48)= -3.8818094134e-01
       coeff_l2(31,49)=  1.2798378468e+00
       coeff_l2(31,50)= -8.5302740335e-02
       coeff_l2(31,51)= -1.1065403223e+00
       coeff_l2(31,52)=  3.2792100310e-01
       coeff_l2(31,53)=  9.4445383549e-01
       coeff_l2(31,54)=  8.2267850637e-01
       coeff_l2(31,55)=  1.0296425819e+00
       coeff_l2(31,56)= -1.4625095129e+00
       coeff_l2(32,1)= -3.9594873786e-01
       coeff_l2(32,2)=  2.7707150578e-01
       coeff_l2(32,3)=  3.3419141173e-01
       coeff_l2(32,4)= -1.4138417244e+00
       coeff_l2(32,5)=  1.2257741690e+00
       coeff_l2(32,6)= -1.9242424965e+00
       coeff_l2(32,7)= -6.6109336913e-02
       coeff_l2(32,8)= -3.2873156667e-01
       coeff_l2(32,9)= -6.1366760731e-01
       coeff_l2(32,10)=  7.2471374273e-01
       coeff_l2(32,11)= -1.0957871675e+00
       coeff_l2(32,12)=  3.0300742388e-01
       coeff_l2(32,13)= -5.5638039112e-01
       coeff_l2(32,14)= -7.2522098199e-03
       coeff_l2(32,15)= -3.4096187353e-01
       coeff_l2(32,16)= -1.8246350288e+00
       coeff_l2(32,17)= -1.5105844736e+00
       coeff_l2(32,18)= -3.0955188274e+00
       coeff_l2(32,19)=  7.5858330727e-01
       coeff_l2(32,20)=  2.9280513525e-02
       coeff_l2(32,21)= -8.4547197819e-01
       coeff_l2(32,22)= -1.1843876839e+00
       coeff_l2(32,23)= -5.3556293249e-01
       coeff_l2(32,24)= -8.7677635252e-02
       coeff_l2(32,25)= -2.8660359979e-01
       coeff_l2(32,26)= -7.6882737875e-01
       coeff_l2(32,27)= -8.7980425358e-01
       coeff_l2(32,28)= -1.1886688471e+00
       coeff_l2(32,29)= -8.9463233948e-01
       coeff_l2(32,30)=  1.1678941250e+00
       coeff_l2(32,31)= -8.8873475790e-01
       coeff_l2(32,32)= -7.0826895535e-02
       coeff_l2(32,33)=  4.2681418359e-02
       coeff_l2(32,34)=  4.3926361203e-01
       coeff_l2(32,35)= -1.1501896381e+00
       coeff_l2(32,36)= -7.0130991936e-01
       coeff_l2(32,37)=  2.1697124466e-02
       coeff_l2(32,38)= -1.3328753710e+00
       coeff_l2(32,39)=  8.5184592009e-01
       coeff_l2(32,40)= -4.8675027490e-01
       coeff_l2(32,41)=  2.9315122962e-01
       coeff_l2(32,42)= -2.2685162723e-01
       coeff_l2(32,43)=  1.4515696466e-01
       coeff_l2(32,44)= -8.1841528416e-01
       coeff_l2(32,45)= -2.6629629135e+00
       coeff_l2(32,46)=  2.4257588387e+00
       coeff_l2(32,47)=  7.4157828093e-01
       coeff_l2(32,48)= -9.0660685301e-01
       coeff_l2(32,49)= -4.1491553187e-01
       coeff_l2(32,50)=  1.7969741821e+00
       coeff_l2(32,51)= -2.3965899944e+00
       coeff_l2(32,52)=  3.0001586303e-02
       coeff_l2(32,53)= -8.8656103611e-01
       coeff_l2(32,54)=  1.4734468460e+00
       coeff_l2(32,55)= -3.3944541216e-01
       coeff_l2(32,56)=  7.5771939754e-01
       coeff_l2(33,1)= -4.6059262753e-01
       coeff_l2(33,2)= -1.1185487360e-01
       coeff_l2(33,3)=  8.4608811140e-01
       coeff_l2(33,4)=  4.5663535595e-01
       coeff_l2(33,5)=  3.9786240458e-01
       coeff_l2(33,6)= -4.5113611966e-02
       coeff_l2(33,7)= -1.1847428083e+00
       coeff_l2(33,8)= -2.7860558033e-01
       coeff_l2(33,9)= -5.2733409405e-01
       coeff_l2(33,10)= -2.2854878008e-01
       coeff_l2(33,11)=  1.5141765773e-01
       coeff_l2(33,12)= -1.4176931232e-02
       coeff_l2(33,13)=  2.9662489891e-01
       coeff_l2(33,14)= -2.4468845129e-01
       coeff_l2(33,15)=  4.5621052384e-01
       coeff_l2(33,16)= -1.7081373930e-01
       coeff_l2(33,17)= -1.4242850542e+00
       coeff_l2(33,18)=  7.3080867529e-01
       coeff_l2(33,19)= -2.8555891514e+00
       coeff_l2(33,20)= -9.1317719221e-01
       coeff_l2(33,21)= -4.1120779514e-01
       coeff_l2(33,22)=  1.1104624718e-01
       coeff_l2(33,23)=  6.2200874090e-03
       coeff_l2(33,24)=  3.6585241556e-01
       coeff_l2(33,25)=  8.2595223188e-01
       coeff_l2(33,26)= -2.1472187340e-01
       coeff_l2(33,27)= -6.2171429396e-01
       coeff_l2(33,28)=  2.1497257054e-01
       coeff_l2(33,29)= -8.4275376797e-01
       coeff_l2(33,30)=  7.0308059454e-02
       coeff_l2(33,31)= -5.9923309833e-02
       coeff_l2(33,32)=  5.1042717695e-01
       coeff_l2(33,33)=  1.1798518896e+00
       coeff_l2(33,34)= -8.2284098864e-01
       coeff_l2(33,35)=  1.6585314646e-02
       coeff_l2(33,36)=  5.3080695868e-01
       coeff_l2(33,37)=  7.4939095974e-01
       coeff_l2(33,38)=  1.2770985365e+00
       coeff_l2(33,39)= -9.0920323133e-01
       coeff_l2(33,40)= -1.2951017618e+00
       coeff_l2(33,41)=  2.4662411213e+00
       coeff_l2(33,42)= -7.6418238878e-01
       coeff_l2(33,43)= -1.0044586658e-01
       coeff_l2(33,44)=  3.4808591008e-01
       coeff_l2(33,45)= -7.4407541752e-01
       coeff_l2(33,46)= -8.7888056040e-01
       coeff_l2(33,47)=  5.5794692039e-01
       coeff_l2(33,48)= -4.6972590685e-01
       coeff_l2(33,49)= -6.7154824734e-01
       coeff_l2(33,50)=  1.3040094376e+00
       coeff_l2(33,51)= -2.7327978611e-01
       coeff_l2(33,52)= -5.1191562414e-01
       coeff_l2(33,53)= -2.7354028821e-01
       coeff_l2(33,54)=  1.3412927091e-01
       coeff_l2(33,55)= -7.6597869396e-01
       coeff_l2(33,56)= -8.1725746393e-01
       coeff_l2(34,1)= -7.6693487167e-01
       coeff_l2(34,2)= -1.1470545530e+00
       coeff_l2(34,3)=  3.0632594228e-01
       coeff_l2(34,4)=  4.3001890182e+00
       coeff_l2(34,5)= -8.7552055717e-02
       coeff_l2(34,6)=  1.3280164003e+00
       coeff_l2(34,7)= -7.0768606663e-01
       coeff_l2(34,8)=  7.0941942930e-01
       coeff_l2(34,9)= -6.5872585773e-01
       coeff_l2(34,10)= -3.0849914551e+00
       coeff_l2(34,11)= -1.4533890486e+00
       coeff_l2(34,12)= -2.0007459819e-01
       coeff_l2(34,13)=  1.5588041544e+00
       coeff_l2(34,14)= -8.5320127010e-01
       coeff_l2(34,15)=  3.0197086930e-01
       coeff_l2(34,16)= -1.5146259964e-01
       coeff_l2(34,17)=  2.2871559858e-01
       coeff_l2(34,18)= -3.2079327106e-01
       coeff_l2(34,19)= -1.1399493217e+00
       coeff_l2(34,20)= -7.7261215448e-01
       coeff_l2(34,21)=  4.2542451620e-01
       coeff_l2(34,22)= -9.1257399321e-01
       coeff_l2(34,23)=  2.0961754322e+00
       coeff_l2(34,24)=  1.2921692133e+00
       coeff_l2(34,25)=  1.4329843521e+00
       coeff_l2(34,26)=  3.5264766216e-01
       coeff_l2(34,27)= -8.4780144691e-01
       coeff_l2(34,28)=  5.6549930573e-01
       coeff_l2(34,29)=  7.9437702894e-02
       coeff_l2(34,30)=  9.1204844415e-02
       coeff_l2(34,31)= -1.4822822809e+00
       coeff_l2(34,32)= -2.5930765271e-01
       coeff_l2(34,33)=  1.6277337074e-01
       coeff_l2(34,34)= -1.9237767458e+00
       coeff_l2(34,35)= -1.1797151566e+00
       coeff_l2(34,36)=  2.0553457737e+00
       coeff_l2(34,37)=  4.1897228360e-01
       coeff_l2(34,38)=  3.4150764346e-01
       coeff_l2(34,39)= -4.3390050530e-01
       coeff_l2(34,40)= -1.3334478140e+00
       coeff_l2(34,41)= -2.8214657307e+00
       coeff_l2(34,42)= -3.8792565465e-01
       coeff_l2(34,43)= -1.2897691131e-01
       coeff_l2(34,44)=  1.6891947985e+00
       coeff_l2(34,45)=  6.4045298100e-01
       coeff_l2(34,46)= -1.8732517958e+00
       coeff_l2(34,47)= -4.3424513936e-01
       coeff_l2(34,48)=  2.2594767809e-01
       coeff_l2(34,49)= -1.7049947754e-02
       coeff_l2(34,50)=  7.8797996044e-01
       coeff_l2(34,51)=  7.7333682775e-01
       coeff_l2(34,52)= -9.7893834114e-01
       coeff_l2(34,53)=  1.4922815561e-01
       coeff_l2(34,54)=  5.0933521241e-02
       coeff_l2(34,55)= -7.4534708261e-01
       coeff_l2(34,56)= -3.4899726510e-01
       coeff_l2(35,1)=  1.6950930655e-01
       coeff_l2(35,2)= -2.1088929474e-01
       coeff_l2(35,3)= -4.7014083713e-02
       coeff_l2(35,4)= -5.6339514256e-01
       coeff_l2(35,5)=  1.2270481884e-01
       coeff_l2(35,6)= -1.0983166695e+00
       coeff_l2(35,7)=  2.6203364134e-01
       coeff_l2(35,8)=  8.0247901380e-02
       coeff_l2(35,9)=  4.8382248729e-02
       coeff_l2(35,10)= -1.9849261642e-01
       coeff_l2(35,11)=  9.2955216765e-02
       coeff_l2(35,12)=  2.6212009788e-01
       coeff_l2(35,13)= -7.0447927713e-01
       coeff_l2(35,14)=  1.6737654805e-02
       coeff_l2(35,15)= -2.0460318029e-01
       coeff_l2(35,16)= -9.4108486176e-01
       coeff_l2(35,17)= -1.0144623518e+00
       coeff_l2(35,18)= -1.4199627638e+00
       coeff_l2(35,19)= -3.2795414329e-01
       coeff_l2(35,20)= -6.4711928368e-02
       coeff_l2(35,21)=  2.0182363689e-01
       coeff_l2(35,22)= -1.1759129763e+00
       coeff_l2(35,23)= -1.4295334816e+00
       coeff_l2(35,24)=  4.7633014619e-02
       coeff_l2(35,25)= -1.9227245823e-02
       coeff_l2(35,26)= -4.5220738649e-01
       coeff_l2(35,27)=  1.7490221560e-01
       coeff_l2(35,28)= -1.3414256275e-01
       coeff_l2(35,29)= -6.2322609127e-02
       coeff_l2(35,30)= -7.2141927481e-01
       coeff_l2(35,31)=  3.3396162093e-02
       coeff_l2(35,32)=  2.7253237367e-01
       coeff_l2(35,33)=  6.5967053175e-02
       coeff_l2(35,34)= -1.3654951751e-01
       coeff_l2(35,35)= -1.7233243212e-02
       coeff_l2(35,36)= -1.6214298084e-02
       coeff_l2(35,37)= -3.9624324441e-01
       coeff_l2(35,38)= -7.5966291130e-02
       coeff_l2(35,39)= -1.8466404080e-01
       coeff_l2(35,40)=  5.1502758265e-01
       coeff_l2(35,41)= -7.4530088902e-01
       coeff_l2(35,42)= -1.0552120209e-01
       coeff_l2(35,43)=  3.3038657159e-02
       coeff_l2(35,44)=  1.1835961789e-01
       coeff_l2(35,45)= -1.4476095438e+00
       coeff_l2(35,46)= -6.5352523327e-01
       coeff_l2(35,47)= -2.0215563476e-01
       coeff_l2(35,48)=  7.1754299104e-02
       coeff_l2(35,49)=  1.3810914755e-01
       coeff_l2(35,50)= -4.6916061640e-01
       coeff_l2(35,51)= -3.0340337753e-01
       coeff_l2(35,52)=  1.3966907561e-01
       coeff_l2(35,53)= -1.5957354009e-01
       coeff_l2(35,54)= -1.9922541082e-01
       coeff_l2(35,55)=  9.5596343279e-02
       coeff_l2(35,56)= -1.3070818782e-01
       coeff_l2(36,1)=  2.7835443616e-01
       coeff_l2(36,2)=  3.1009334326e-01
       coeff_l2(36,3)=  3.6962673068e-01
       coeff_l2(36,4)= -1.0819293261e+00
       coeff_l2(36,5)= -7.4151498079e-01
       coeff_l2(36,6)=  4.4035330415e-01
       coeff_l2(36,7)=  4.7353848815e-01
       coeff_l2(36,8)= -2.1492791176e-01
       coeff_l2(36,9)=  1.6064651310e-01
       coeff_l2(36,10)=  2.5893849134e-01
       coeff_l2(36,11)= -2.2136659920e-01
       coeff_l2(36,12)= -1.1143956333e-01
       coeff_l2(36,13)=  6.4783650637e-01
       coeff_l2(36,14)= -1.0115369409e-01
       coeff_l2(36,15)=  5.3065937757e-01
       coeff_l2(36,16)= -3.7330749631e-01
       coeff_l2(36,17)=  1.3772159815e-01
       coeff_l2(36,18)=  8.5270947218e-01
       coeff_l2(36,19)=  1.0694134980e-01
       coeff_l2(36,20)= -2.5160303712e-01
       coeff_l2(36,21)=  1.7724265158e-01
       coeff_l2(36,22)= -1.3189918995e+00
       coeff_l2(36,23)= -3.1948396564e-01
       coeff_l2(36,24)= -5.1822048426e-01
       coeff_l2(36,25)=  2.6498067379e-01
       coeff_l2(36,26)= -8.0351746082e-01
       coeff_l2(36,27)= -1.0785062611e-01
       coeff_l2(36,28)=  1.3466788828e-01
       coeff_l2(36,29)=  4.7894995660e-02
       coeff_l2(36,30)= -5.0667369366e-01
       coeff_l2(36,31)=  2.5899988413e-01
       coeff_l2(36,32)= -1.9147351980e+00
       coeff_l2(36,33)=  2.1452966332e-01
       coeff_l2(36,34)=  2.6217883825e-01
       coeff_l2(36,35)=  2.8820288181e-01
       coeff_l2(36,36)= -5.5290156603e-01
       coeff_l2(36,37)= -2.5352087617e-01
       coeff_l2(36,38)=  2.9913258553e-01
       coeff_l2(36,39)= -1.7342999578e-01
       coeff_l2(36,40)=  1.1345818043e+00
       coeff_l2(36,41)= -7.2598224878e-01
       coeff_l2(36,42)=  4.2873084545e-02
       coeff_l2(36,43)=  7.6955860853e-01
       coeff_l2(36,44)= -8.9934244752e-02
       coeff_l2(36,45)= -3.6167570949e-01
       coeff_l2(36,46)= -6.4876711369e-01
       coeff_l2(36,47)=  4.2921924591e-01
       coeff_l2(36,48)=  3.2908210158e-01
       coeff_l2(36,49)=  4.8065462708e-01
       coeff_l2(36,50)=  8.5041716695e-02
       coeff_l2(36,51)=  2.4009312689e-01
       coeff_l2(36,52)=  7.7943247557e-01
       coeff_l2(36,53)= -2.0511411130e-01
       coeff_l2(36,54)=  8.6928844452e-02
       coeff_l2(36,55)= -3.4905996919e-01
       coeff_l2(36,56)= -2.2604623437e-01
!coeff layer 3
       coeff_l3(1,1)= -1.2572734058e-01
       coeff_l3(1,2)=  1.5467011929e-01
       coeff_l3(1,3)= -9.4145268202e-02
       coeff_l3(1,4)= -3.2647070289e-01
       coeff_l3(1,5)=  9.4622033834e-01
       coeff_l3(1,6)=  5.0720381737e-01
       coeff_l3(1,7)=  1.7494039237e-01
       coeff_l3(1,8)=  1.3526669741e+00
       coeff_l3(1,9)= -2.8868916631e-01
       coeff_l3(1,10)= -4.6409833431e-01
       coeff_l3(1,11)= -1.0706915855e+00
       coeff_l3(1,12)=  1.5348236263e-01
       coeff_l3(1,13)=  2.7214610949e-02
       coeff_l3(1,14)=  9.9641203880e-01
       coeff_l3(1,15)=  2.8132939339e-01
       coeff_l3(1,16)= -1.4800857008e-01
       coeff_l3(1,17)= -3.8929486275e-01
       coeff_l3(1,18)= -2.3438562453e-01
       coeff_l3(1,19)= -1.8240290880e-01
       coeff_l3(1,20)=  6.8453848362e-01
       coeff_l3(1,21)=  2.8362309933e-01
       coeff_l3(1,22)= -6.0293771327e-02
       coeff_l3(1,23)=  3.7518808246e-01
       coeff_l3(1,24)= -1.4500004053e+00
       coeff_l3(1,25)=  2.0182903111e-01
       coeff_l3(1,26)=  5.5836588144e-02
       coeff_l3(1,27)=  3.2002061605e-01
       coeff_l3(1,28)= -6.5078610182e-01
       coeff_l3(1,29)= -7.5688273646e-03
       coeff_l3(1,30)=  5.9543168545e-01
       coeff_l3(1,31)= -6.3293570280e-01
       coeff_l3(1,32)=  7.9285383224e-01
       coeff_l3(1,33)= -1.3975272179e+00
       coeff_l3(1,34)=  1.0693605989e-01
       coeff_l3(1,35)= -1.1710436344e+00
       coeff_l3(1,36)= -6.7359161377e-01
       coeff_l3(1,37)= -1.0519474000e-01
       coeff_l3(1,38)= -3.0015099049e-01
       coeff_l3(1,39)=  9.1818797588e-01
       coeff_l3(1,40)= -1.3651657104e+00
       coeff_l3(1,41)= -6.2870633602e-01
       coeff_l3(1,42)=  5.5443942547e-02
       coeff_l3(1,43)=  1.3158193231e-01
       coeff_l3(1,44)=  4.1102322936e-01
       coeff_l3(1,45)= -6.9889771938e-01
       coeff_l3(1,46)=  1.2414752506e-02
       coeff_l3(1,47)= -1.0045791417e-01
       coeff_l3(1,48)= -1.7352355719e+00
       coeff_l3(1,49)=  1.3320061564e-01
       coeff_l3(1,50)= -2.2106041014e-01
       coeff_l3(1,51)=  2.3096930981e+00
       coeff_l3(1,52)= -1.0853822231e+00
       coeff_l3(1,53)=  7.5735068321e-01
       coeff_l3(1,54)=  7.7183550596e-01
       coeff_l3(1,55)= -1.0326704979e+00
       coeff_l3(1,56)= -5.0712305307e-01
       coeff_l3(2,1)= -4.8428079486e-01
       coeff_l3(2,2)=  2.5076857209e-01
       coeff_l3(2,3)=  1.0166390985e-01
       coeff_l3(2,4)= -4.9806949496e-01
       coeff_l3(2,5)= -1.6452322006e+00
       coeff_l3(2,6)= -6.1453056335e-01
       coeff_l3(2,7)= -2.8868925571e-01
       coeff_l3(2,8)=  6.1583220959e-01
       coeff_l3(2,9)=  3.4801444411e-01
       coeff_l3(2,10)= -1.5624392033e-01
       coeff_l3(2,11)=  1.1680996418e-01
       coeff_l3(2,12)= -9.0655559301e-01
       coeff_l3(2,13)= -1.1472879648e+00
       coeff_l3(2,14)= -1.2062843144e-01
       coeff_l3(2,15)= -4.3706811965e-02
       coeff_l3(2,16)=  1.2053217888e+00
       coeff_l3(2,17)=  5.4759824276e-01
       coeff_l3(2,18)= -8.2185649872e-01
       coeff_l3(2,19)=  5.6312173605e-01
       coeff_l3(2,20)= -9.3502891064e-01
       coeff_l3(2,21)=  2.9779130220e-01
       coeff_l3(2,22)=  1.4813811779e+00
       coeff_l3(2,23)=  5.2848011255e-01
       coeff_l3(2,24)=  5.7965505123e-01
       coeff_l3(2,25)=  1.3804434538e+00
       coeff_l3(2,26)= -2.2839532793e-01
       coeff_l3(2,27)=  3.6470457911e-01
       coeff_l3(2,28)= -8.6115753651e-01
       coeff_l3(2,29)= -6.9726848602e-01
       coeff_l3(2,30)=  3.7569057941e-01
       coeff_l3(2,31)=  9.6918381751e-02
       coeff_l3(2,32)=  1.1341409683e+00
       coeff_l3(2,33)=  5.4112356901e-01
       coeff_l3(2,34)= -2.0944030583e-01
       coeff_l3(2,35)= -5.6960374117e-01
       coeff_l3(2,36)= -3.0344596505e-01
       coeff_l3(2,37)= -4.5371049643e-01
       coeff_l3(2,38)=  1.0806411505e+00
       coeff_l3(2,39)= -1.0874345303e+00
       coeff_l3(2,40)=  5.2185946703e-01
       coeff_l3(2,41)=  3.9804702997e-01
       coeff_l3(2,42)= -9.7225600481e-01
       coeff_l3(2,43)= -2.3725536466e-01
       coeff_l3(2,44)=  4.6061423421e-01
       coeff_l3(2,45)=  1.1746467091e-02
       coeff_l3(2,46)= -2.7062538266e-01
       coeff_l3(2,47)= -5.5350220203e-01
       coeff_l3(2,48)= -3.2532399893e-01
       coeff_l3(2,49)= -3.2377725840e-01
       coeff_l3(2,50)= -6.7134104669e-02
       coeff_l3(2,51)=  2.3820027709e-02
       coeff_l3(2,52)= -8.5202509165e-01
       coeff_l3(2,53)=  6.1702426523e-02
       coeff_l3(2,54)= -1.8560260534e+00
       coeff_l3(2,55)= -2.5791683793e-01
       coeff_l3(2,56)=  3.0922302604e-01
       coeff_l3(3,1)=  2.8726226091e-01
       coeff_l3(3,2)=  1.6055728495e-01
       coeff_l3(3,3)= -1.9029822946e-01
       coeff_l3(3,4)=  1.6488382593e-02
       coeff_l3(3,5)=  7.7887457609e-01
       coeff_l3(3,6)=  1.1994177848e-01
       coeff_l3(3,7)= -6.3938438892e-02
       coeff_l3(3,8)=  5.6969892979e-01
       coeff_l3(3,9)= -1.2135744095e+00
       coeff_l3(3,10)= -6.2489432096e-01
       coeff_l3(3,11)= -1.0874472857e+00
       coeff_l3(3,12)= -1.3042286038e-01
       coeff_l3(3,13)=  2.6505273581e-01
       coeff_l3(3,14)= -5.5328857899e-01
       coeff_l3(3,15)= -6.7099827528e-01
       coeff_l3(3,16)=  6.4746022224e-01
       coeff_l3(3,17)= -3.2026904821e-01
       coeff_l3(3,18)= -5.2442938089e-01
       coeff_l3(3,19)=  1.5974649787e-01
       coeff_l3(3,20)=  1.0324959755e+00
       coeff_l3(3,21)= -2.7096264064e-02
       coeff_l3(3,22)=  1.3180930614e+00
       coeff_l3(3,23)= -1.6278168559e-01
       coeff_l3(3,24)=  6.2612599134e-01
       coeff_l3(3,25)= -1.4344494045e-01
       coeff_l3(3,26)= -1.1381051540e+00
       coeff_l3(3,27)=  1.5814296007e+00
       coeff_l3(3,28)= -1.3662272692e-01
       coeff_l3(3,29)=  1.1144144833e-01
       coeff_l3(3,30)=  1.9255416095e-01
       coeff_l3(3,31)=  1.5268714726e-01
       coeff_l3(3,32)=  4.0271908045e-02
       coeff_l3(3,33)= -3.8531509042e-01
       coeff_l3(3,34)= -2.3417375982e-01
       coeff_l3(3,35)= -1.8520395458e-01
       coeff_l3(3,36)=  6.4051187038e-01
       coeff_l3(3,37)=  3.3319212496e-02
       coeff_l3(3,38)=  1.4130344614e-02
       coeff_l3(3,39)=  1.2807795405e-01
       coeff_l3(3,40)= -1.2549682856e+00
       coeff_l3(3,41)= -5.1998746395e-01
       coeff_l3(3,42)=  1.4726768732e+00
       coeff_l3(3,43)= -1.2852054834e-01
       coeff_l3(3,44)=  1.2849041820e-01
       coeff_l3(3,45)=  8.4207701683e-01
       coeff_l3(3,46)= -9.9754881859e-01
       coeff_l3(3,47)=  1.2422971427e-01
       coeff_l3(3,48)= -1.5068401396e-01
       coeff_l3(3,49)= -3.0455920100e-01
       coeff_l3(3,50)=  1.9164326191e+00
       coeff_l3(3,51)= -9.2961424589e-01
       coeff_l3(3,52)= -1.3781139851e+00
       coeff_l3(3,53)=  1.3470047154e-02
       coeff_l3(3,54)=  2.0863853395e-01
       coeff_l3(3,55)=  2.8831553459e-01
       coeff_l3(3,56)=  3.7514689565e-01
       coeff_l3(4,1)=  1.8874693662e-02
       coeff_l3(4,2)= -7.5275886059e-01
       coeff_l3(4,3)=  6.7410641909e-01
       coeff_l3(4,4)= -7.3000483215e-02
       coeff_l3(4,5)= -3.3756773919e-02
       coeff_l3(4,6)= -5.9578031301e-01
       coeff_l3(4,7)= -8.8041834533e-02
       coeff_l3(4,8)= -2.1179428697e-01
       coeff_l3(4,9)= -1.3784062862e-01
       coeff_l3(4,10)= -9.3188753817e-04
       coeff_l3(4,11)= -2.9095613956e+00
       coeff_l3(4,12)= -5.2302646637e-01
       coeff_l3(4,13)=  7.6413244009e-02
       coeff_l3(4,14)= -9.0375316143e-01
       coeff_l3(4,15)= -2.0510642231e-01
       coeff_l3(4,16)= -5.3996783495e-01
       coeff_l3(4,17)= -4.3499175459e-02
       coeff_l3(4,18)=  3.7561595440e-02
       coeff_l3(4,19)= -1.5765188932e+00
       coeff_l3(4,20)=  2.1123686433e-01
       coeff_l3(4,21)= -8.0334228277e-01
       coeff_l3(4,22)= -1.1401572227e+00
       coeff_l3(4,23)= -6.4310801029e-01
       coeff_l3(4,24)= -4.7800204158e-01
       coeff_l3(4,25)= -7.7807612419e+00
       coeff_l3(4,26)=  1.0199447870e+00
       coeff_l3(4,27)=  1.6640035808e-01
       coeff_l3(4,28)=  2.1106408536e-01
       coeff_l3(4,29)= -3.3464658260e+00
       coeff_l3(4,30)=  2.2076168656e-01
       coeff_l3(4,31)= -8.8068383932e-01
       coeff_l3(4,32)= -1.2791532278e-01
       coeff_l3(4,33)= -4.5916087925e-02
       coeff_l3(4,34)= -1.7970359325e-01
       coeff_l3(4,35)=  2.1816372871e-01
       coeff_l3(4,36)=  1.7407187819e-01
       coeff_l3(4,37)=  1.2955814600e-01
       coeff_l3(4,38)=  3.7647199631e-01
       coeff_l3(4,39)=  3.6586497445e-03
       coeff_l3(4,40)=  1.4230288565e-01
       coeff_l3(4,41)= -1.9429953769e-02
       coeff_l3(4,42)= -6.8845599890e-02
       coeff_l3(4,43)= -3.1397250295e-01
       coeff_l3(4,44)=  2.9210764170e-01
       coeff_l3(4,45)= -7.7490788698e-01
       coeff_l3(4,46)= -2.2054173052e-01
       coeff_l3(4,47)= -8.0547046661e-01
       coeff_l3(4,48)=  7.0470981300e-03
       coeff_l3(4,49)= -7.0580750704e-01
       coeff_l3(4,50)=  2.6351376437e-03
       coeff_l3(4,51)= -2.2291111946e+00
       coeff_l3(4,52)= -4.1195076704e-01
       coeff_l3(4,53)= -1.8504928350e+00
       coeff_l3(4,54)= -8.4152805805e-01
       coeff_l3(4,55)= -5.2405649424e-01
       coeff_l3(4,56)= -1.1903709918e-01
       coeff_l3(5,1)= -6.1487201601e-02
       coeff_l3(5,2)=  9.4680301845e-03
       coeff_l3(5,3)= -3.1484341621e-01
       coeff_l3(5,4)= -8.2482504845e-01
       coeff_l3(5,5)= -1.4854995906e-01
       coeff_l3(5,6)= -1.2265130281e+00
       coeff_l3(5,7)= -4.8085104674e-02
       coeff_l3(5,8)=  3.9859774709e-01
       coeff_l3(5,9)= -5.3145223856e-01
       coeff_l3(5,10)=  4.7154477239e-01
       coeff_l3(5,11)= -1.3546806574e-01
       coeff_l3(5,12)= -1.0468467139e-02
       coeff_l3(5,13)= -5.3148932755e-02
       coeff_l3(5,14)= -2.4938509464e+00
       coeff_l3(5,15)=  4.1878715158e-01
       coeff_l3(5,16)=  4.3052509427e-01
       coeff_l3(5,17)= -1.5370416641e-01
       coeff_l3(5,18)=  3.1014382839e-02
       coeff_l3(5,19)= -5.2226197720e-01
       coeff_l3(5,20)= -2.8828388453e-01
       coeff_l3(5,21)= -1.5481624007e-01
       coeff_l3(5,22)= -5.5893176794e-01
       coeff_l3(5,23)= -8.9421051741e-01
       coeff_l3(5,24)=  1.5092889965e-01
       coeff_l3(5,25)= -3.7629301548e+00
       coeff_l3(5,26)= -3.1068053246e+00
       coeff_l3(5,27)= -1.7032763362e-01
       coeff_l3(5,28)= -5.6810373068e-01
       coeff_l3(5,29)=  1.4071483612e+00
       coeff_l3(5,30)= -2.9233360663e-02
       coeff_l3(5,31)= -1.3942103386e+00
       coeff_l3(5,32)=  4.3971288949e-02
       coeff_l3(5,33)=  2.2741465271e-01
       coeff_l3(5,34)=  1.5385897458e-01
       coeff_l3(5,35)= -4.5463800430e-01
       coeff_l3(5,36)= -1.6391973197e-01
       coeff_l3(5,37)= -8.9435416460e-01
       coeff_l3(5,38)= -8.5549694300e-01
       coeff_l3(5,39)=  2.3401059210e-01
       coeff_l3(5,40)=  1.9620808959e-01
       coeff_l3(5,41)= -1.4275394380e-01
       coeff_l3(5,42)=  4.8073239625e-02
       coeff_l3(5,43)= -7.5840294361e-01
       coeff_l3(5,44)= -4.8788997531e-01
       coeff_l3(5,45)=  3.4423127770e-01
       coeff_l3(5,46)= -2.1645326167e-02
       coeff_l3(5,47)= -3.6690080166e-01
       coeff_l3(5,48)= -1.4772933722e-01
       coeff_l3(5,49)= -8.4412133694e-01
       coeff_l3(5,50)=  1.8537689745e-01
       coeff_l3(5,51)= -6.2425726652e-01
       coeff_l3(5,52)=  2.0544135571e-01
       coeff_l3(5,53)=  5.1164098084e-02
       coeff_l3(5,54)=  7.5227135420e-01
       coeff_l3(5,55)=  2.4843914807e-01
       coeff_l3(5,56)= -6.6915875673e-01
       coeff_l3(6,1)= -1.3941152096e+00
       coeff_l3(6,2)=  4.7251501679e-01
       coeff_l3(6,3)= -3.7136265635e-01
       coeff_l3(6,4)=  1.8443988264e-01
       coeff_l3(6,5)= -5.0811660290e-01
       coeff_l3(6,6)= -9.6542441845e-01
       coeff_l3(6,7)= -2.2425416112e-01
       coeff_l3(6,8)= -5.4780995846e-01
       coeff_l3(6,9)= -1.5884181857e-01
       coeff_l3(6,10)=  1.3773201406e-01
       coeff_l3(6,11)= -1.3362512589e+00
       coeff_l3(6,12)=  4.1031244397e-01
       coeff_l3(6,13)= -8.5779190063e-01
       coeff_l3(6,14)= -4.7243142128e-01
       coeff_l3(6,15)= -1.4372224808e+00
       coeff_l3(6,16)=  8.7802551687e-02
       coeff_l3(6,17)=  1.5190945566e-01
       coeff_l3(6,18)= -4.3551540375e-01
       coeff_l3(6,19)= -5.8804666996e-01
       coeff_l3(6,20)= -1.9782279432e-01
       coeff_l3(6,21)=  6.0255908966e-01
       coeff_l3(6,22)= -5.1411032677e-01
       coeff_l3(6,23)= -6.4526152611e-01
       coeff_l3(6,24)=  7.4074852467e-01
       coeff_l3(6,25)=  2.4715669155e+00
       coeff_l3(6,26)= -9.6067315340e-01
       coeff_l3(6,27)=  9.7030177712e-03
       coeff_l3(6,28)=  5.6648969650e-01
       coeff_l3(6,29)=  3.4881585836e-01
       coeff_l3(6,30)= -4.6830269694e-01
       coeff_l3(6,31)= -2.2761821747e-01
       coeff_l3(6,32)=  1.0986062884e-01
       coeff_l3(6,33)=  2.1876844764e-01
       coeff_l3(6,34)=  3.6708104610e-01
       coeff_l3(6,35)=  2.9274100065e-01
       coeff_l3(6,36)= -6.3078159094e-01
       coeff_l3(6,37)= -2.8920555115e-01
       coeff_l3(6,38)=  1.1062436104e+00
       coeff_l3(6,39)=  3.7574559450e-01
       coeff_l3(6,40)= -4.5292741060e-01
       coeff_l3(6,41)= -5.7511121035e-01
       coeff_l3(6,42)= -1.1264312267e+00
       coeff_l3(6,43)= -1.3807096481e+00
       coeff_l3(6,44)= -4.0155738592e-01
       coeff_l3(6,45)=  4.1515517235e+00
       coeff_l3(6,46)= -4.6382197738e-01
       coeff_l3(6,47)=  3.4201152623e-02
       coeff_l3(6,48)= -6.1938438565e-02
       coeff_l3(6,49)=  9.5429599285e-02
       coeff_l3(6,50)=  3.4121218324e-01
       coeff_l3(6,51)= -5.1066166162e-01
       coeff_l3(6,52)= -2.0675536990e-01
       coeff_l3(6,53)= -4.6157220006e-01
       coeff_l3(6,54)= -5.1524859667e-01
       coeff_l3(6,55)=  2.5171566010e-01
       coeff_l3(6,56)=  1.8512643576e+00
       coeff_l3(7,1)= -4.9247175455e-01
       coeff_l3(7,2)=  5.0251897424e-02
       coeff_l3(7,3)= -7.1247257292e-02
       coeff_l3(7,4)= -1.2141852826e-01
       coeff_l3(7,5)=  1.7108686268e-01
       coeff_l3(7,6)= -1.3583585620e-01
       coeff_l3(7,7)= -6.8480765820e-01
       coeff_l3(7,8)=  2.7848151326e-01
       coeff_l3(7,9)=  1.2996678054e-01
       coeff_l3(7,10)= -1.2525969744e-01
       coeff_l3(7,11)=  1.4362419844e+00
       coeff_l3(7,12)= -9.4107246399e-01
       coeff_l3(7,13)= -1.1935192347e+00
       coeff_l3(7,14)=  2.4864135683e-01
       coeff_l3(7,15)=  6.0959291458e-01
       coeff_l3(7,16)=  1.0170840472e-01
       coeff_l3(7,17)= -2.9077059031e-01
       coeff_l3(7,18)= -1.8381538987e-01
       coeff_l3(7,19)= -1.5481002331e+00
       coeff_l3(7,20)= -5.6277573109e-01
       coeff_l3(7,21)=  2.9844692349e-01
       coeff_l3(7,22)=  8.1450963020e-01
       coeff_l3(7,23)=  4.4365975261e-01
       coeff_l3(7,24)= -4.3955510855e-01
       coeff_l3(7,25)=  1.9836463928e+00
       coeff_l3(7,26)= -1.1184486151e+00
       coeff_l3(7,27)=  1.0732790828e-01
       coeff_l3(7,28)= -3.5861164331e-01
       coeff_l3(7,29)=  1.2384122610e+00
       coeff_l3(7,30)=  1.5149554014e+00
       coeff_l3(7,31)=  3.8560682535e-01
       coeff_l3(7,32)= -3.4435641766e-01
       coeff_l3(7,33)=  6.7771154642e-01
       coeff_l3(7,34)= -9.6500009298e-02
       coeff_l3(7,35)=  2.8065724298e-02
       coeff_l3(7,36)= -3.4807726741e-01
       coeff_l3(7,37)= -6.2159538269e-02
       coeff_l3(7,38)=  1.9847795963e+00
       coeff_l3(7,39)=  7.1481168270e-01
       coeff_l3(7,40)=  2.7609229088e-01
       coeff_l3(7,41)=  8.0478918552e-01
       coeff_l3(7,42)= -4.0448627472e+00
       coeff_l3(7,43)= -1.8460124731e+00
       coeff_l3(7,44)=  3.7224087119e-01
       coeff_l3(7,45)= -1.2687966824e+00
       coeff_l3(7,46)=  3.9565703273e-01
       coeff_l3(7,47)= -5.9267455339e-01
       coeff_l3(7,48)=  9.4649195671e-02
       coeff_l3(7,49)=  1.4558185637e-01
       coeff_l3(7,50)= -2.9519414529e-03
       coeff_l3(7,51)= -3.1060922146e+00
       coeff_l3(7,52)=  4.0087454021e-02
       coeff_l3(7,53)= -6.7643195391e-01
       coeff_l3(7,54)= -2.1879053116e+00
       coeff_l3(7,55)= -7.4926376343e-01
       coeff_l3(7,56)=  7.6401066780e-01
       coeff_l3(8,1)=  6.6373950243e-01
       coeff_l3(8,2)=  1.2727974681e-03
       coeff_l3(8,3)=  7.9059554264e-03
       coeff_l3(8,4)=  4.7291383147e-01
       coeff_l3(8,5)= -4.9061414599e-01
       coeff_l3(8,6)=  2.4766849354e-02
       coeff_l3(8,7)= -2.3630133271e-01
       coeff_l3(8,8)=  3.2943892293e-03
       coeff_l3(8,9)= -2.9402190447e-01
       coeff_l3(8,10)= -2.2774952650e-01
       coeff_l3(8,11)=  4.9157300591e-01
       coeff_l3(8,12)= -5.0099521875e-01
       coeff_l3(8,13)= -3.9151176810e-01
       coeff_l3(8,14)=  5.9345775843e-01
       coeff_l3(8,15)= -9.1914957762e-01
       coeff_l3(8,16)=  7.3564291000e-01
       coeff_l3(8,17)= -7.3200881481e-02
       coeff_l3(8,18)= -8.1449800730e-01
       coeff_l3(8,19)= -2.2309815884e-01
       coeff_l3(8,20)= -8.8316923380e-01
       coeff_l3(8,21)= -2.8930845857e-01
       coeff_l3(8,22)= -5.5431270599e-01
       coeff_l3(8,23)=  3.3096203208e-01
       coeff_l3(8,24)=  1.6174098849e-01
       coeff_l3(8,25)=  9.7826522589e-01
       coeff_l3(8,26)= -5.4956209660e-01
       coeff_l3(8,27)= -2.3422093689e-01
       coeff_l3(8,28)= -5.5605000257e-01
       coeff_l3(8,29)=  8.3756047487e-01
       coeff_l3(8,30)= -1.1415454149e+00
       coeff_l3(8,31)=  8.3884280920e-01
       coeff_l3(8,32)= -1.2341115624e-01
       coeff_l3(8,33)= -4.6295264363e-01
       coeff_l3(8,34)= -9.0623997152e-02
       coeff_l3(8,35)= -1.0466729403e+00
       coeff_l3(8,36)= -6.1753173359e-03
       coeff_l3(8,37)=  4.1487902403e-01
       coeff_l3(8,38)=  5.9892410040e-01
       coeff_l3(8,39)=  1.1750642061e+00
       coeff_l3(8,40)= -5.1028156281e-01
       coeff_l3(8,41)= -3.9704754949e-02
       coeff_l3(8,42)= -1.7128041387e-01
       coeff_l3(8,43)= -4.8106673360e-01
       coeff_l3(8,44)=  4.0670987964e-01
       coeff_l3(8,45)= -3.2877063751e-01
       coeff_l3(8,46)= -7.1075356007e-01
       coeff_l3(8,47)= -1.6062723100e-01
       coeff_l3(8,48)=  1.2508839369e-01
       coeff_l3(8,49)= -7.0843763649e-02
       coeff_l3(8,50)=  3.0313396454e-01
       coeff_l3(8,51)= -1.3660490513e-01
       coeff_l3(8,52)= -7.1271502972e-01
       coeff_l3(8,53)= -3.1352809072e-01
       coeff_l3(8,54)= -7.5210177898e-01
       coeff_l3(8,55)= -1.7221231759e-01
       coeff_l3(8,56)=  6.3924320042e-02
       coeff_l3(9,1)=  4.2555668950e-01
       coeff_l3(9,2)= -2.3062263429e-01
       coeff_l3(9,3)= -2.0766749978e-01
       coeff_l3(9,4)=  2.7920532227e-01
       coeff_l3(9,5)= -1.2064555883e+00
       coeff_l3(9,6)=  2.1785417199e-01
       coeff_l3(9,7)= -4.2008391023e-01
       coeff_l3(9,8)=  7.0222049952e-01
       coeff_l3(9,9)= -2.4905221164e-01
       coeff_l3(9,10)=  2.9969799519e-01
       coeff_l3(9,11)=  3.9896509051e-01
       coeff_l3(9,12)= -5.6707090139e-01
       coeff_l3(9,13)= -8.0062764883e-01
       coeff_l3(9,14)= -1.5760819614e-01
       coeff_l3(9,15)= -2.3885478079e-01
       coeff_l3(9,16)=  3.5472873598e-02
       coeff_l3(9,17)=  6.5075910091e-01
       coeff_l3(9,18)= -5.0446385145e-01
       coeff_l3(9,19)= -1.4692482352e-01
       coeff_l3(9,20)=  5.8899182826e-02
       coeff_l3(9,21)=  5.5432182550e-01
       coeff_l3(9,22)=  8.2219189405e-01
       coeff_l3(9,23)=  2.4998766184e-01
       coeff_l3(9,24)= -2.3666939139e-01
       coeff_l3(9,25)=  1.3335783780e-01
       coeff_l3(9,26)= -1.0307790041e+00
       coeff_l3(9,27)=  3.3639815450e-01
       coeff_l3(9,28)= -5.7297950983e-01
       coeff_l3(9,29)=  2.2676738445e-03
       coeff_l3(9,30)= -9.4842827320e-01
       coeff_l3(9,31)=  9.2988520861e-01
       coeff_l3(9,32)= -3.2312709093e-01
       coeff_l3(9,33)=  2.2091454268e-01
       coeff_l3(9,34)=  1.2918637693e-01
       coeff_l3(9,35)= -3.3341410756e-01
       coeff_l3(9,36)= -2.1986521780e-01
       coeff_l3(9,37)= -1.0095542669e-01
       coeff_l3(9,38)=  7.5237399340e-01
       coeff_l3(9,39)=  6.1549776793e-01
       coeff_l3(9,40)=  5.7024709880e-02
       coeff_l3(9,41)=  2.2316251695e-01
       coeff_l3(9,42)= -4.4977188110e-01
       coeff_l3(9,43)= -1.5773367137e-02
       coeff_l3(9,44)=  4.6644523740e-01
       coeff_l3(9,45)=  3.1119495630e-01
       coeff_l3(9,46)= -1.1245737225e-01
       coeff_l3(9,47)= -4.5664584637e-01
       coeff_l3(9,48)= -1.1480257511e+00
       coeff_l3(9,49)= -1.6052825749e-01
       coeff_l3(9,50)= -7.8287953138e-01
       coeff_l3(9,51)=  1.7194494605e-01
       coeff_l3(9,52)= -5.0271028280e-01
       coeff_l3(9,53)= -4.4967576861e-01
       coeff_l3(9,54)= -1.1323052645e+00
       coeff_l3(9,55)=  2.7812486887e-01
       coeff_l3(9,56)=  4.4993951917e-01
       coeff_l3(10,1)= -9.2178583145e-01
       coeff_l3(10,2)=  9.9214226007e-02
       coeff_l3(10,3)=  1.6820672899e-02
       coeff_l3(10,4)= -4.2064431310e-01
       coeff_l3(10,5)=  3.2957592607e-01
       coeff_l3(10,6)=  7.0183908939e-01
       coeff_l3(10,7)= -5.3346645832e-01
       coeff_l3(10,8)=  5.4371130466e-01
       coeff_l3(10,9)=  7.9255707562e-02
       coeff_l3(10,10)= -2.5826558471e-01
       coeff_l3(10,11)= -1.0107895732e-01
       coeff_l3(10,12)=  1.1066488922e-02
       coeff_l3(10,13)= -4.0844157338e-01
       coeff_l3(10,14)= -6.3468176126e-01
       coeff_l3(10,15)=  7.9193675518e-01
       coeff_l3(10,16)= -7.3484617472e-01
       coeff_l3(10,17)= -9.5731526613e-02
       coeff_l3(10,18)= -4.9905931950e-01
       coeff_l3(10,19)= -1.2688004971e-01
       coeff_l3(10,20)=  5.6969302893e-01
       coeff_l3(10,21)=  1.0576471090e+00
       coeff_l3(10,22)=  6.4296656847e-01
       coeff_l3(10,23)=  3.5867327452e-01
       coeff_l3(10,24)= -1.7015753686e-01
       coeff_l3(10,25)=  3.1565964222e+00
       coeff_l3(10,26)= -4.6525895596e-01
       coeff_l3(10,27)= -2.2687625885e-01
       coeff_l3(10,28)= -4.5887199044e-01
       coeff_l3(10,29)= -1.5974606574e-01
       coeff_l3(10,30)= -1.6921092570e-01
       coeff_l3(10,31)=  9.0261733532e-01
       coeff_l3(10,32)=  4.4778380543e-03
       coeff_l3(10,33)=  2.3816803470e-02
       coeff_l3(10,34)=  2.6044952869e-01
       coeff_l3(10,35)=  2.9509276152e-01
       coeff_l3(10,36)= -3.3756944537e-01
       coeff_l3(10,37)= -4.7910526395e-01
       coeff_l3(10,38)=  5.3828465939e-01
       coeff_l3(10,39)= -2.4234783649e-01
       coeff_l3(10,40)= -2.1371144056e-01
       coeff_l3(10,41)= -1.8537843227e-01
       coeff_l3(10,42)= -1.9788765907e-01
       coeff_l3(10,43)= -1.5971034765e-01
       coeff_l3(10,44)=  6.0773644596e-02
       coeff_l3(10,45)= -6.0109760612e-02
       coeff_l3(10,46)= -6.0430133343e-01
       coeff_l3(10,47)=  3.6763051152e-01
       coeff_l3(10,48)= -9.2411345243e-01
       coeff_l3(10,49)= -3.6790475249e-01
       coeff_l3(10,50)=  4.3976452947e-01
       coeff_l3(10,51)=  1.6022206545e+00
       coeff_l3(10,52)=  3.4550675750e-01
       coeff_l3(10,53)= -4.8600083590e-01
       coeff_l3(10,54)=  9.0801155567e-01
       coeff_l3(10,55)= -1.7688800395e-01
       coeff_l3(10,56)= -1.1879242212e-01
       coeff_l3(11,1)= -5.2008733153e-02
       coeff_l3(11,2)=  1.3810777664e+00
       coeff_l3(11,3)= -1.7476284504e-01
       coeff_l3(11,4)=  5.2553814650e-01
       coeff_l3(11,5)= -4.3780863285e-01
       coeff_l3(11,6)= -1.1140133142e+00
       coeff_l3(11,7)= -4.3540990353e-01
       coeff_l3(11,8)= -7.1797186136e-01
       coeff_l3(11,9)=  2.7507457137e-01
       coeff_l3(11,10)=  3.1903824210e-01
       coeff_l3(11,11)=  1.5689136982e+00
       coeff_l3(11,12)= -1.7035533190e+00
       coeff_l3(11,13)= -3.3673819900e-01
       coeff_l3(11,14)= -1.2999457121e-01
       coeff_l3(11,15)= -9.6793308854e-02
       coeff_l3(11,16)=  3.7886890769e-01
       coeff_l3(11,17)= -1.5626466274e+00
       coeff_l3(11,18)=  2.5309050083e-01
       coeff_l3(11,19)= -1.1559800804e-01
       coeff_l3(11,20)=  4.5395138860e-01
       coeff_l3(11,21)= -2.7170959115e-01
       coeff_l3(11,22)=  1.5314151347e-01
       coeff_l3(11,23)=  3.3561468124e-01
       coeff_l3(11,24)=  1.4197772741e-01
       coeff_l3(11,25)=  3.7974791527e+00
       coeff_l3(11,26)=  1.9348365068e+00
       coeff_l3(11,27)=  7.6620674133e-01
       coeff_l3(11,28)= -4.2302012444e-01
       coeff_l3(11,29)= -6.8120968342e-01
       coeff_l3(11,30)= -5.9958308935e-01
       coeff_l3(11,31)=  5.3310018778e-01
       coeff_l3(11,32)= -5.8107590675e-01
       coeff_l3(11,33)= -7.2436326742e-01
       coeff_l3(11,34)=  5.2045458555e-01
       coeff_l3(11,35)= -7.5331181288e-02
       coeff_l3(11,36)= -2.1542961895e-01
       coeff_l3(11,37)=  5.1668977737e-01
       coeff_l3(11,38)=  1.9509652257e-01
       coeff_l3(11,39)= -3.9552968740e-01
       coeff_l3(11,40)=  5.2285099030e-01
       coeff_l3(11,41)=  9.3625682592e-01
       coeff_l3(11,42)=  8.8854116201e-01
       coeff_l3(11,43)= -3.3868417144e-02
       coeff_l3(11,44)= -5.9881055355e-01
       coeff_l3(11,45)= -4.0040555596e-01
       coeff_l3(11,46)= -1.0946816951e-01
       coeff_l3(11,47)=  4.4593754411e-01
       coeff_l3(11,48)=  7.6330167055e-01
       coeff_l3(11,49)=  3.0542460084e-01
       coeff_l3(11,50)= -4.0309867263e-01
       coeff_l3(11,51)= -6.1267632246e-01
       coeff_l3(11,52)=  5.3538960218e-01
       coeff_l3(11,53)= -5.1829886436e-01
       coeff_l3(11,54)= -2.1558313370e+00
       coeff_l3(11,55)= -9.2452037334e-01
       coeff_l3(11,56)=  2.4627959356e-02
       coeff_l3(12,1)= -4.3724622577e-02
       coeff_l3(12,2)=  9.5676940680e-01
       coeff_l3(12,3)= -2.0224748552e-01
       coeff_l3(12,4)=  1.3507758081e-01
       coeff_l3(12,5)= -6.1720728874e-01
       coeff_l3(12,6)=  7.4984991550e-01
       coeff_l3(12,7)= -9.3931114674e-01
       coeff_l3(12,8)=  1.5345247984e+00
       coeff_l3(12,9)= -5.1579838991e-01
       coeff_l3(12,10)=  2.1672394127e-02
       coeff_l3(12,11)=  2.8411075473e-01
       coeff_l3(12,12)= -5.5449973792e-02
       coeff_l3(12,13)= -1.2321872711e+00
       coeff_l3(12,14)= -1.7845630646e-01
       coeff_l3(12,15)=  9.9280536175e-01
       coeff_l3(12,16)=  4.6828168631e-01
       coeff_l3(12,17)= -5.2926938981e-02
       coeff_l3(12,18)= -4.4746331871e-02
       coeff_l3(12,19)=  5.0247931480e-01
       coeff_l3(12,20)= -3.1602543592e-01
       coeff_l3(12,21)=  5.5712670088e-01
       coeff_l3(12,22)=  2.9596036673e-01
       coeff_l3(12,23)=  6.1320606619e-02
       coeff_l3(12,24)= -3.4469544888e-02
       coeff_l3(12,25)=  7.6261264086e-01
       coeff_l3(12,26)= -1.1318474263e-01
       coeff_l3(12,27)=  4.1758716106e-01
       coeff_l3(12,28)= -3.5115715861e-01
       coeff_l3(12,29)= -1.0382897854e+00
       coeff_l3(12,30)= -1.9454963207e+00
       coeff_l3(12,31)=  1.1240265369e+00
       coeff_l3(12,32)=  2.6185458899e-01
       coeff_l3(12,33)= -4.5970153809e-01
       coeff_l3(12,34)= -3.6248335242e-01
       coeff_l3(12,35)= -4.2297792435e-01
       coeff_l3(12,36)= -5.5050045252e-01
       coeff_l3(12,37)=  1.4217202365e-01
       coeff_l3(12,38)=  6.8729656935e-01
       coeff_l3(12,39)=  1.8682997227e+00
       coeff_l3(12,40)=  3.5453560948e-01
       coeff_l3(12,41)=  1.1623170972e-01
       coeff_l3(12,42)=  6.7304395139e-02
       coeff_l3(12,43)= -4.5571282506e-01
       coeff_l3(12,44)=  1.4920027554e-01
       coeff_l3(12,45)= -3.5616815090e-01
       coeff_l3(12,46)= -1.3940115273e-01
       coeff_l3(12,47)= -1.4463979006e-01
       coeff_l3(12,48)= -4.9681040645e-01
       coeff_l3(12,49)=  1.0035114735e-01
       coeff_l3(12,50)=  5.3575980663e-01
       coeff_l3(12,51)=  1.8047888279e+00
       coeff_l3(12,52)= -1.0787250996e+00
       coeff_l3(12,53)=  7.7058827877e-01
       coeff_l3(12,54)= -1.2204960585e+00
       coeff_l3(12,55)= -5.2593588829e-01
       coeff_l3(12,56)= -7.9063862562e-02
       coeff_l3(13,1)= -5.5375880003e-01
       coeff_l3(13,2)=  6.8022406101e-01
       coeff_l3(13,3)= -1.8049078062e-02
       coeff_l3(13,4)=  6.8549829721e-01
       coeff_l3(13,5)=  7.5383514166e-01
       coeff_l3(13,6)= -5.8053600788e-01
       coeff_l3(13,7)= -4.7053501010e-01
       coeff_l3(13,8)= -1.1278817654e+00
       coeff_l3(13,9)=  2.6190934703e-02
       coeff_l3(13,10)=  3.9044592530e-02
       coeff_l3(13,11)= -1.3331702948e+00
       coeff_l3(13,12)=  3.3093833923e-01
       coeff_l3(13,13)=  1.1116588116e+00
       coeff_l3(13,14)= -8.8642352819e-01
       coeff_l3(13,15)= -9.6005433798e-01
       coeff_l3(13,16)=  3.7152117491e-01
       coeff_l3(13,17)= -5.5864715576e-01
       coeff_l3(13,18)= -1.7242350578e+00
       coeff_l3(13,19)= -4.2113009095e-01
       coeff_l3(13,20)=  1.0788218975e+00
       coeff_l3(13,21)=  2.0784655213e-01
       coeff_l3(13,22)=  1.8697518110e+00
       coeff_l3(13,23)= -1.2678661346e+00
       coeff_l3(13,24)=  7.5300139189e-01
       coeff_l3(13,25)= -6.3568308949e-02
       coeff_l3(13,26)= -3.9769881964e-01
       coeff_l3(13,27)=  1.7242701054e+00
       coeff_l3(13,28)= -5.4190538824e-02
       coeff_l3(13,29)=  1.2293937206e+00
       coeff_l3(13,30)= -5.7155191898e-01
       coeff_l3(13,31)= -1.2036873102e+00
       coeff_l3(13,32)=  2.4216049910e-01
       coeff_l3(13,33)=  8.9548856020e-02
       coeff_l3(13,34)=  1.8516899645e-01
       coeff_l3(13,35)=  7.5107944012e-01
       coeff_l3(13,36)= -3.3561834693e-01
       coeff_l3(13,37)= -6.0947525501e-01
       coeff_l3(13,38)=  6.0029459000e-01
       coeff_l3(13,39)=  3.6730047315e-02
       coeff_l3(13,40)= -8.9624774456e-01
       coeff_l3(13,41)= -9.6839153767e-01
       coeff_l3(13,42)=  1.2168846130e+00
       coeff_l3(13,43)= -5.8513116837e-01
       coeff_l3(13,44)= -4.8636561632e-01
       coeff_l3(13,45)=  1.1086034775e+00
       coeff_l3(13,46)= -3.2094582915e-01
       coeff_l3(13,47)= -4.3185266852e-01
       coeff_l3(13,48)=  4.2272403836e-01
       coeff_l3(13,49)= -4.7739741206e-01
       coeff_l3(13,50)=  2.0131053925e+00
       coeff_l3(13,51)= -1.4065595865e+00
       coeff_l3(13,52)= -6.6737316549e-02
       coeff_l3(13,53)= -5.4428994656e-01
       coeff_l3(13,54)=  2.8956881166e-01
       coeff_l3(13,55)=  7.0172280073e-02
       coeff_l3(13,56)=  1.7958770990e+00
       coeff_l3(14,1)=  3.4715390205e-01
       coeff_l3(14,2)=  2.2666466236e-01
       coeff_l3(14,3)=  1.0233859718e-01
       coeff_l3(14,4)=  8.3815747499e-01
       coeff_l3(14,5)= -3.3261951804e-01
       coeff_l3(14,6)= -1.6016604006e-01
       coeff_l3(14,7)= -2.5915512443e-01
       coeff_l3(14,8)= -4.5748528838e-01
       coeff_l3(14,9)= -3.1832575798e-01
       coeff_l3(14,10)=  3.4774875641e-01
       coeff_l3(14,11)=  7.3687839508e-01
       coeff_l3(14,12)= -1.0231884718e+00
       coeff_l3(14,13)= -4.6646335721e-01
       coeff_l3(14,14)= -7.2381049395e-01
       coeff_l3(14,15)=  7.1778362989e-01
       coeff_l3(14,16)= -4.5166215301e-01
       coeff_l3(14,17)= -4.7041675448e-01
       coeff_l3(14,18)=  7.3978585005e-01
       coeff_l3(14,19)= -1.2792274356e-01
       coeff_l3(14,20)= -8.5795648396e-02
       coeff_l3(14,21)=  2.7703499794e-01
       coeff_l3(14,22)=  1.2153202295e-01
       coeff_l3(14,23)=  1.5681868792e-01
       coeff_l3(14,24)=  5.3465332836e-02
       coeff_l3(14,25)= -1.9633965492e+00
       coeff_l3(14,26)= -8.5900586843e-01
       coeff_l3(14,27)=  2.3507365584e-01
       coeff_l3(14,28)= -4.1223368049e-01
       coeff_l3(14,29)=  2.8602210805e-02
       coeff_l3(14,30)=  1.5138539672e-01
       coeff_l3(14,31)= -9.9157744646e-01
       coeff_l3(14,32)=  6.1551964283e-01
       coeff_l3(14,33)=  5.3008598089e-01
       coeff_l3(14,34)=  1.5529313684e-01
       coeff_l3(14,35)=  2.8047579527e-01
       coeff_l3(14,36)= -1.1154789925e+00
       coeff_l3(14,37)= -2.8281685710e-01
       coeff_l3(14,38)=  1.2350882292e+00
       coeff_l3(14,39)=  3.9813727140e-01
       coeff_l3(14,40)=  1.7436595261e-01
       coeff_l3(14,41)=  7.5505360961e-02
       coeff_l3(14,42)=  6.6679322720e-01
       coeff_l3(14,43)= -3.3670088649e-01
       coeff_l3(14,44)=  1.9424995780e-01
       coeff_l3(14,45)= -9.7142630816e-01
       coeff_l3(14,46)= -8.9277106524e-01
       coeff_l3(14,47)= -6.1161530018e-01
       coeff_l3(14,48)=  2.9771193862e-01
       coeff_l3(14,49)= -1.9661882520e-01
       coeff_l3(14,50)=  1.5510638952e+00
       coeff_l3(14,51)= -9.0635031462e-01
       coeff_l3(14,52)=  3.5056889057e-01
       coeff_l3(14,53)= -5.0001132488e-01
       coeff_l3(14,54)= -1.4839262962e+00
       coeff_l3(14,55)= -1.2998406887e+00
       coeff_l3(14,56)= -7.0960986614e-01
       coeff_l3(15,1)= -4.9123433232e-01
       coeff_l3(15,2)= -3.9387813210e-01
       coeff_l3(15,3)=  3.6834061146e-02
       coeff_l3(15,4)= -4.7381910682e-01
       coeff_l3(15,5)=  2.2242984176e-01
       coeff_l3(15,6)= -2.3619265854e-01
       coeff_l3(15,7)=  2.7899652719e-02
       coeff_l3(15,8)= -1.0862964392e-01
       coeff_l3(15,9)=  8.1755328178e-01
       coeff_l3(15,10)= -2.1901270375e-02
       coeff_l3(15,11)=  4.9335283041e-01
       coeff_l3(15,12)=  1.2952764034e+00
       coeff_l3(15,13)=  2.7158433199e-01
       coeff_l3(15,14)= -4.4134207070e-02
       coeff_l3(15,15)= -9.8164379597e-01
       coeff_l3(15,16)=  8.0566383898e-02
       coeff_l3(15,17)=  7.6858347654e-01
       coeff_l3(15,18)= -1.5169119835e-01
       coeff_l3(15,19)= -8.8847911358e-01
       coeff_l3(15,20)=  4.9394825101e-01
       coeff_l3(15,21)=  2.4316067994e-01
       coeff_l3(15,22)= -7.0352202654e-01
       coeff_l3(15,23)=  1.6462440789e-01
       coeff_l3(15,24)=  1.3290236704e-02
       coeff_l3(15,25)=  2.9437818527e+00
       coeff_l3(15,26)= -1.7366396189e+00
       coeff_l3(15,27)=  3.5162746906e-01
       coeff_l3(15,28)= -6.8883538246e-01
       coeff_l3(15,29)=  1.2438187599e+00
       coeff_l3(15,30)= -3.0627939105e-01
       coeff_l3(15,31)=  1.2016096115e+00
       coeff_l3(15,32)=  1.3150812685e-01
       coeff_l3(15,33)=  5.4040926695e-01
       coeff_l3(15,34)=  3.5042226315e-01
       coeff_l3(15,35)=  3.3400797844e-01
       coeff_l3(15,36)= -4.8049688339e-02
       coeff_l3(15,37)=  6.2073159963e-02
       coeff_l3(15,38)=  3.4462144971e-01
       coeff_l3(15,39)= -1.9596925378e-01
       coeff_l3(15,40)=  1.9776292145e-01
       coeff_l3(15,41)=  1.7532977462e-01
       coeff_l3(15,42)=  1.2257023156e-01
       coeff_l3(15,43)= -2.7415430546e-01
       coeff_l3(15,44)=  2.7789112926e-01
       coeff_l3(15,45)= -2.9888379574e-01
       coeff_l3(15,46)=  1.9055482745e-01
       coeff_l3(15,47)= -4.5780494809e-01
       coeff_l3(15,48)= -2.8567034006e-01
       coeff_l3(15,49)= -3.4489104152e-01
       coeff_l3(15,50)=  6.0993105173e-01
       coeff_l3(15,51)=  1.2163162231e-01
       coeff_l3(15,52)=  6.7461907864e-02
       coeff_l3(15,53)=  5.2156612277e-02
       coeff_l3(15,54)= -5.3379511833e-01
       coeff_l3(15,55)=  5.0059908628e-01
       coeff_l3(15,56)=  6.1567443609e-01
       coeff_l3(16,1)= -3.7133947015e-01
       coeff_l3(16,2)= -7.3411101103e-01
       coeff_l3(16,3)=  3.2402551174e-01
       coeff_l3(16,4)= -3.6591172218e-01
       coeff_l3(16,5)=  3.8681450486e-01
       coeff_l3(16,6)= -2.9280611873e-01
       coeff_l3(16,7)= -4.1826623678e-01
       coeff_l3(16,8)= -8.3258140087e-01
       coeff_l3(16,9)= -5.2186733484e-01
       coeff_l3(16,10)=  3.3111804724e-01
       coeff_l3(16,11)= -6.4969307184e-01
       coeff_l3(16,12)=  1.8260004520e+00
       coeff_l3(16,13)= -7.2884254158e-02
       coeff_l3(16,14)= -3.3603537083e-01
       coeff_l3(16,15)= -2.5987800956e-01
       coeff_l3(16,16)= -9.9550984800e-02
       coeff_l3(16,17)=  6.4449375868e-01
       coeff_l3(16,18)=  5.5077437311e-02
       coeff_l3(16,19)= -2.7317366004e-01
       coeff_l3(16,20)= -4.6625441313e-01
       coeff_l3(16,21)= -2.0544332266e-01
       coeff_l3(16,22)= -1.2403224409e-01
       coeff_l3(16,23)= -2.1859490871e-01
       coeff_l3(16,24)= -2.2833409905e-01
       coeff_l3(16,25)= -9.4062262774e-01
       coeff_l3(16,26)= -5.8802229166e-01
       coeff_l3(16,27)= -7.7621525526e-01
       coeff_l3(16,28)=  1.7598883808e-01
       coeff_l3(16,29)=  1.7451218367e+00
       coeff_l3(16,30)=  8.3106242120e-02
       coeff_l3(16,31)= -5.4128146172e-01
       coeff_l3(16,32)=  8.4422701597e-01
       coeff_l3(16,33)=  1.3320878148e-01
       coeff_l3(16,34)=  1.0901607573e-01
       coeff_l3(16,35)=  2.8513288498e-01
       coeff_l3(16,36)=  1.2125129998e-01
       coeff_l3(16,37)= -3.8919278979e-01
       coeff_l3(16,38)= -3.8369756937e-01
       coeff_l3(16,39)= -1.9753724337e-01
       coeff_l3(16,40)= -4.4247555733e-01
       coeff_l3(16,41)= -7.7564305067e-01
       coeff_l3(16,42)=  3.6340785027e-01
       coeff_l3(16,43)=  5.4983317852e-01
       coeff_l3(16,44)=  7.7845945954e-02
       coeff_l3(16,45)=  3.6885070801e-01
       coeff_l3(16,46)= -4.8294341564e-01
       coeff_l3(16,47)=  7.4242848158e-01
       coeff_l3(16,48)=  8.7807738781e-01
       coeff_l3(16,49)= -4.4492337853e-02
       coeff_l3(16,50)=  8.2292085886e-01
       coeff_l3(16,51)= -4.6897563338e-01
       coeff_l3(16,52)=  8.1135565042e-01
       coeff_l3(16,53)= -5.3936463594e-01
       coeff_l3(16,54)=  3.5729601383e+00
       coeff_l3(16,55)=  5.3864095360e-02
       coeff_l3(16,56)=  8.1516891718e-01
       coeff_l3(17,1)=  2.2460386157e-01
       coeff_l3(17,2)= -7.8498132527e-02
       coeff_l3(17,3)= -1.8210506439e-01
       coeff_l3(17,4)= -1.4307590723e+00
       coeff_l3(17,5)=  4.6979092062e-02
       coeff_l3(17,6)= -1.3130761385e+00
       coeff_l3(17,7)=  2.6979532838e-01
       coeff_l3(17,8)= -2.6549631357e-01
       coeff_l3(17,9)=  5.9357172251e-01
       coeff_l3(17,10)=  7.0521837473e-01
       coeff_l3(17,11)=  6.6696636379e-02
       coeff_l3(17,12)=  8.3032083511e-01
       coeff_l3(17,13)=  2.2703072429e-01
       coeff_l3(17,14)= -1.7091852427e-01
       coeff_l3(17,15)= -3.4510713816e-01
       coeff_l3(17,16)= -2.7940884233e-01
       coeff_l3(17,17)= -2.9163950682e-01
       coeff_l3(17,18)=  2.3239545524e-01
       coeff_l3(17,19)= -1.3023023307e-01
       coeff_l3(17,20)=  4.0491688251e-01
       coeff_l3(17,21)= -1.2413743734e+00
       coeff_l3(17,22)=  3.1147918105e-01
       coeff_l3(17,23)= -4.3604174256e-01
       coeff_l3(17,24)=  1.6073775291e-01
       coeff_l3(17,25)= -1.6284190416e+00
       coeff_l3(17,26)= -8.7939992547e-02
       coeff_l3(17,27)= -8.8333517313e-01
       coeff_l3(17,28)=  2.5798000395e-02
       coeff_l3(17,29)=  4.7313385010e+00
       coeff_l3(17,30)=  2.7103248239e-01
       coeff_l3(17,31)=  2.5262362957e+00
       coeff_l3(17,32)= -2.1516230702e-01
       coeff_l3(17,33)= -1.9562627375e-01
       coeff_l3(17,34)=  4.8178538680e-01
       coeff_l3(17,35)=  1.6517734528e+00
       coeff_l3(17,36)= -1.1841731519e-01
       coeff_l3(17,37)=  5.1299935579e-01
       coeff_l3(17,38)=  4.8123362660e-01
       coeff_l3(17,39)= -2.3786427453e-02
       coeff_l3(17,40)=  1.4538185298e-01
       coeff_l3(17,41)= -3.4391239285e-01
       coeff_l3(17,42)=  7.4515277147e-01
       coeff_l3(17,43)=  1.6768742800e+00
       coeff_l3(17,44)= -8.8040310144e-01
       coeff_l3(17,45)=  2.0097501278e+00
       coeff_l3(17,46)=  4.8526611924e-01
       coeff_l3(17,47)= -7.2445499897e-01
       coeff_l3(17,48)=  1.7655258179e+00
       coeff_l3(17,49)=  8.3069074154e-01
       coeff_l3(17,50)=  4.4420042634e-01
       coeff_l3(17,51)=  8.9625531435e-01
       coeff_l3(17,52)=  8.6662846804e-01
       coeff_l3(17,53)= -2.4023443460e-01
       coeff_l3(17,54)=  1.1843698025e+00
       coeff_l3(17,55)=  8.3697718382e-01
       coeff_l3(17,56)=  1.8807728291e+00
       coeff_l3(18,1)=  3.6584621668e-01
       coeff_l3(18,2)= -1.0418498516e+00
       coeff_l3(18,3)=  4.5511329174e-01
       coeff_l3(18,4)= -1.8252864480e-01
       coeff_l3(18,5)=  4.3935338035e-03
       coeff_l3(18,6)=  5.9918290377e-01
       coeff_l3(18,7)= -1.3314634562e-01
       coeff_l3(18,8)= -6.0350187123e-03
       coeff_l3(18,9)= -5.0086003542e-01
       coeff_l3(18,10)= -7.0463784039e-02
       coeff_l3(18,11)= -8.4334069490e-01
       coeff_l3(18,12)=  7.6078397036e-01
       coeff_l3(18,13)=  1.0704736412e-01
       coeff_l3(18,14)= -5.7408928871e-01
       coeff_l3(18,15)=  1.2763217092e-01
       coeff_l3(18,16)= -3.9036265016e-01
       coeff_l3(18,17)=  2.4577960372e-01
       coeff_l3(18,18)=  9.7498632967e-02
       coeff_l3(18,19)= -5.2788215876e-01
       coeff_l3(18,20)= -4.1163556278e-02
       coeff_l3(18,21)= -3.2424125075e-01
       coeff_l3(18,22)= -3.2105565071e-01
       coeff_l3(18,23)= -1.1180754900e+00
       coeff_l3(18,24)= -5.3681677580e-01
       coeff_l3(18,25)= -1.2612466812e+01
       coeff_l3(18,26)=  1.2531810999e+00
       coeff_l3(18,27)=  1.0080166161e-01
       coeff_l3(18,28)=  1.6496406496e-01
       coeff_l3(18,29)=  4.0003779531e-01
       coeff_l3(18,30)= -1.0097688437e-01
       coeff_l3(18,31)= -1.0238480568e+00
       coeff_l3(18,32)= -1.0511841625e-01
       coeff_l3(18,33)=  8.0871962011e-02
       coeff_l3(18,34)= -2.0588031411e-01
       coeff_l3(18,35)= -5.3815923631e-02
       coeff_l3(18,36)= -1.3188509643e-01
       coeff_l3(18,37)=  1.6230379045e-01
       coeff_l3(18,38)= -4.9725094438e-01
       coeff_l3(18,39)= -2.7585735917e-01
       coeff_l3(18,40)= -2.8798091412e-01
       coeff_l3(18,41)= -1.4110141993e+00
       coeff_l3(18,42)= -1.6443656385e-01
       coeff_l3(18,43)=  3.8489517570e-01
       coeff_l3(18,44)=  1.0070999712e-01
       coeff_l3(18,45)=  7.9696947336e-01
       coeff_l3(18,46)= -5.7350348681e-02
       coeff_l3(18,47)=  1.1281520128e-01
       coeff_l3(18,48)= -1.3529027700e+00
       coeff_l3(18,49)= -5.1757735014e-01
       coeff_l3(18,50)= -3.9741057158e-01
       coeff_l3(18,51)= -4.4734519720e-01
       coeff_l3(18,52)= -3.7619316578e-01
       coeff_l3(18,53)= -5.0212806463e-01
       coeff_l3(18,54)=  1.2966151237e+00
       coeff_l3(18,55)=  7.3755586147e-01
       coeff_l3(18,56)=  1.8276421353e-02
       coeff_l3(19,1)=  4.8394918442e-01
       coeff_l3(19,2)= -1.6639603376e+00
       coeff_l3(19,3)= -6.6538757086e-01
       coeff_l3(19,4)=  9.9973267317e-01
       coeff_l3(19,5)=  3.4055835009e-01
       coeff_l3(19,6)=  3.6360415816e-01
       coeff_l3(19,7)= -4.3466374278e-01
       coeff_l3(19,8)= -3.5388508439e-01
       coeff_l3(19,9)=  4.0631967783e-01
       coeff_l3(19,10)= -9.0998834372e-01
       coeff_l3(19,11)=  2.7877166271e+00
       coeff_l3(19,12)= -1.5641929209e-01
       coeff_l3(19,13)= -2.2615113854e-01
       coeff_l3(19,14)=  7.3028057814e-02
       coeff_l3(19,15)=  1.7395594716e-01
       coeff_l3(19,16)= -6.0305333138e-01
       coeff_l3(19,17)= -2.7338522673e-01
       coeff_l3(19,18)= -2.3771083355e+00
       coeff_l3(19,19)=  5.0046664476e-01
       coeff_l3(19,20)=  2.3290600628e-02
       coeff_l3(19,21)=  1.8071205914e-01
       coeff_l3(19,22)=  3.2378983498e+00
       coeff_l3(19,23)=  3.0969303846e-01
       coeff_l3(19,24)=  3.3214145899e-01
       coeff_l3(19,25)=  8.8861644268e-01
       coeff_l3(19,26)= -2.3435499668e+00
       coeff_l3(19,27)=  3.3217743039e-01
       coeff_l3(19,28)= -8.9674955606e-01
       coeff_l3(19,29)= -3.1079216003e+00
       coeff_l3(19,30)=  5.5921423435e-01
       coeff_l3(19,31)=  2.3439085484e+00
       coeff_l3(19,32)= -8.7845420837e-01
       coeff_l3(19,33)= -1.0029805899e+00
       coeff_l3(19,34)= -3.7916734815e-01
       coeff_l3(19,35)= -3.1984776258e-01
       coeff_l3(19,36)=  3.3302983642e-01
       coeff_l3(19,37)=  1.0722431540e-01
       coeff_l3(19,38)=  1.7696001530e+00
       coeff_l3(19,39)= -6.5094679594e-01
       coeff_l3(19,40)= -5.3353458643e-01
       coeff_l3(19,41)=  3.3510056138e-01
       coeff_l3(19,42)= -2.9297652245e+00
       coeff_l3(19,43)= -6.8868041039e-01
       coeff_l3(19,44)= -6.5289519727e-02
       coeff_l3(19,45)= -6.0599637032e-01
       coeff_l3(19,46)= -1.1438492537e+00
       coeff_l3(19,47)= -2.6528906822e-01
       coeff_l3(19,48)= -3.0957549810e-01
       coeff_l3(19,49)= -7.2912067175e-01
       coeff_l3(19,50)= -3.0993821621e+00
       coeff_l3(19,51)=  2.6244726777e-01
       coeff_l3(19,52)= -8.6113572121e-02
       coeff_l3(19,53)=  1.8301154375e+00
       coeff_l3(19,54)= -3.0053906441e+00
       coeff_l3(19,55)= -7.5135171413e-01
       coeff_l3(19,56)= -7.1968007088e-01
       coeff_l3(20,1)=  8.5040968657e-01
       coeff_l3(20,2)= -7.0165324211e-01
       coeff_l3(20,3)= -2.9892545938e-01
       coeff_l3(20,4)= -4.0800851583e-01
       coeff_l3(20,5)=  3.8686835766e-01
       coeff_l3(20,6)=  9.9961060286e-01
       coeff_l3(20,7)= -2.7121555805e-01
       coeff_l3(20,8)=  9.2283296585e-01
       coeff_l3(20,9)=  8.9332026243e-01
       coeff_l3(20,10)= -2.1336610615e-01
       coeff_l3(20,11)=  6.7114681005e-01
       coeff_l3(20,12)= -1.4228532091e-02
       coeff_l3(20,13)= -5.7055586576e-01
       coeff_l3(20,14)=  4.0921980143e-01
       coeff_l3(20,15)=  2.0399454236e-01
       coeff_l3(20,16)=  4.0127062798e-01
       coeff_l3(20,17)=  7.5252354145e-01
       coeff_l3(20,18)= -9.7588521242e-01
       coeff_l3(20,19)= -7.7167963982e-01
       coeff_l3(20,20)= -2.1546266973e-01
       coeff_l3(20,21)= -3.1968873739e-01
       coeff_l3(20,22)=  2.1421053410e+00
       coeff_l3(20,23)=  1.4648018405e-02
       coeff_l3(20,24)= -1.2039422244e-01
       coeff_l3(20,25)=  1.1170809269e+00
       coeff_l3(20,26)=  5.6593000889e-01
       coeff_l3(20,27)=  2.9724881053e-01
       coeff_l3(20,28)= -8.1266351044e-02
       coeff_l3(20,29)= -4.1946581006e-01
       coeff_l3(20,30)= -4.6535208821e-01
       coeff_l3(20,31)=  1.0597485304e+00
       coeff_l3(20,32)= -7.5344008207e-01
       coeff_l3(20,33)=  1.9275251031e-01
       coeff_l3(20,34)= -2.2887839377e-01
       coeff_l3(20,35)= -1.3372032642e+00
       coeff_l3(20,36)=  5.2125435323e-02
       coeff_l3(20,37)= -1.5468011796e-01
       coeff_l3(20,38)=  1.7652486265e-01
       coeff_l3(20,39)=  4.5995295048e-01
       coeff_l3(20,40)= -2.5247424841e-01
       coeff_l3(20,41)=  2.3305295408e-01
       coeff_l3(20,42)= -3.1092693806e+00
       coeff_l3(20,43)=  5.9258036315e-02
       coeff_l3(20,44)=  3.2758527994e-01
       coeff_l3(20,45)=  1.5937746763e+00
       coeff_l3(20,46)=  3.7274447083e-01
       coeff_l3(20,47)=  1.5886102617e-01
       coeff_l3(20,48)= -4.0362164378e-01
       coeff_l3(20,49)= -1.6931776702e-01
       coeff_l3(20,50)= -1.4932061434e+00
       coeff_l3(20,51)=  1.8381592035e+00
       coeff_l3(20,52)= -3.3593100309e-01
       coeff_l3(20,53)=  1.9186842442e+00
       coeff_l3(20,54)=  1.4699761868e+00
       coeff_l3(20,55)= -3.9088949561e-01
       coeff_l3(20,56)=  5.8723217249e-01
       coeff_l3(21,1)= -3.2826882601e-01
       coeff_l3(21,2)= -9.0420283377e-02
       coeff_l3(21,3)= -9.7541294992e-02
       coeff_l3(21,4)=  2.7249255776e-01
       coeff_l3(21,5)= -8.9514398575e-01
       coeff_l3(21,6)=  1.1213258654e-01
       coeff_l3(21,7)=  7.6892389916e-03
       coeff_l3(21,8)=  5.0673574209e-01
       coeff_l3(21,9)= -4.0753949434e-02
       coeff_l3(21,10)=  1.2441050261e-01
       coeff_l3(21,11)=  3.0272567272e-01
       coeff_l3(21,12)= -9.5647639036e-01
       coeff_l3(21,13)= -2.1559491158e+00
       coeff_l3(21,14)= -6.3141167164e-02
       coeff_l3(21,15)= -3.0667923391e-02
       coeff_l3(21,16)=  7.2305701673e-02
       coeff_l3(21,17)=  3.9534717798e-01
       coeff_l3(21,18)= -7.8268605471e-01
       coeff_l3(21,19)=  1.9370238483e-01
       coeff_l3(21,20)=  4.7380305827e-02
       coeff_l3(21,21)=  5.4025560617e-01
       coeff_l3(21,22)= -6.4271992445e-01
       coeff_l3(21,23)=  4.1767975688e-01
       coeff_l3(21,24)= -6.6724365950e-01
       coeff_l3(21,25)=  3.5940611362e-01
       coeff_l3(21,26)= -7.1016997099e-01
       coeff_l3(21,27)=  1.1997458935e+00
       coeff_l3(21,28)= -1.6603825986e-01
       coeff_l3(21,29)= -1.3505172729e-01
       coeff_l3(21,30)= -2.3415349424e-01
       coeff_l3(21,31)= -3.1825491786e-01
       coeff_l3(21,32)= -7.6114296913e-01
       coeff_l3(21,33)=  1.8512208760e-01
       coeff_l3(21,34)= -4.4692510366e-01
       coeff_l3(21,35)= -2.2179366648e-01
       coeff_l3(21,36)=  1.8463875353e-01
       coeff_l3(21,37)= -2.1964184940e-01
       coeff_l3(21,38)=  9.6035492420e-01
       coeff_l3(21,39)=  3.2950550318e-01
       coeff_l3(21,40)=  1.5102694929e-01
       coeff_l3(21,41)=  1.5590354800e-01
       coeff_l3(21,42)= -1.0532507896e+00
       coeff_l3(21,43)= -9.2712873220e-01
       coeff_l3(21,44)=  3.9531522989e-01
       coeff_l3(21,45)= -1.6365281343e+00
       coeff_l3(21,46)= -8.4345775843e-01
       coeff_l3(21,47)= -5.5722874403e-01
       coeff_l3(21,48)=  2.0588131621e-02
       coeff_l3(21,49)=  1.7107191682e-01
       coeff_l3(21,50)=  1.6646188498e-01
       coeff_l3(21,51)= -9.2603224516e-01
       coeff_l3(21,52)= -2.5067225099e-01
       coeff_l3(21,53)= -4.5592033863e-01
       coeff_l3(21,54)= -1.3144360781e+00
       coeff_l3(21,55)= -4.9092659354e-01
       coeff_l3(21,56)= -5.9978204966e-01
       coeff_l3(22,1)= -2.9940545559e-01
       coeff_l3(22,2)= -8.5237461329e-01
       coeff_l3(22,3)= -8.8004723191e-02
       coeff_l3(22,4)= -1.1416875757e-02
       coeff_l3(22,5)= -4.7188934684e-01
       coeff_l3(22,6)= -1.5305666924e+00
       coeff_l3(22,7)= -1.0709922314e+00
       coeff_l3(22,8)= -6.3529103994e-01
       coeff_l3(22,9)= -6.3218599558e-01
       coeff_l3(22,10)= -1.3377016783e-01
       coeff_l3(22,11)=  2.1404451132e-01
       coeff_l3(22,12)= -4.9750280380e-01
       coeff_l3(22,13)=  1.3311173022e-01
       coeff_l3(22,14)= -1.3558082581e+00
       coeff_l3(22,15)= -1.3616706133e+00
       coeff_l3(22,16)= -7.1915364265e-01
       coeff_l3(22,17)=  2.1404219791e-02
       coeff_l3(22,18)= -6.3970847987e-03
       coeff_l3(22,19)= -8.6026149988e-01
       coeff_l3(22,20)= -1.0157073848e-02
       coeff_l3(22,21)= -9.4732451439e-01
       coeff_l3(22,22)= -1.4779497385e+00
       coeff_l3(22,23)=  1.5298136510e-02
       coeff_l3(22,24)=  5.8526283503e-01
       coeff_l3(22,25)= -1.8285317421e+00
       coeff_l3(22,26)=  1.0246227980e+00
       coeff_l3(22,27)= -1.2563058138e+00
       coeff_l3(22,28)= -7.5473242998e-01
       coeff_l3(22,29)=  2.4566783905e+00
       coeff_l3(22,30)=  6.6892987490e-01
       coeff_l3(22,31)= -5.0133848190e-01
       coeff_l3(22,32)= -1.3581571169e-02
       coeff_l3(22,33)= -7.9371637106e-01
       coeff_l3(22,34)=  1.1368800402e+00
       coeff_l3(22,35)=  1.1894986629e+00
       coeff_l3(22,36)= -6.2225866318e-01
       coeff_l3(22,37)= -4.2771315575e-01
       coeff_l3(22,38)= -4.1753658652e-01
       coeff_l3(22,39)= -5.8280434459e-02
       coeff_l3(22,40)= -3.7769582868e-01
       coeff_l3(22,41)= -8.3834415674e-01
       coeff_l3(22,42)=  7.5875151157e-01
       coeff_l3(22,43)=  1.0195342302e+00
       coeff_l3(22,44)=  1.3535590470e-01
       coeff_l3(22,45)= -2.6545662880e+00
       coeff_l3(22,46)= -3.9390988648e-02
       coeff_l3(22,47)= -1.6229148209e-01
       coeff_l3(22,48)=  5.3559416533e-01
       coeff_l3(22,49)=  5.2891701460e-01
       coeff_l3(22,50)= -8.8102614880e-01
       coeff_l3(22,51)= -4.9568676949e-01
       coeff_l3(22,52)=  3.7011772394e-01
       coeff_l3(22,53)= -9.4789499044e-01
       coeff_l3(22,54)=  1.3160115480e+00
       coeff_l3(22,55)=  8.4864848852e-01
       coeff_l3(22,56)= -2.3658524454e-01
       coeff_l3(23,1)= -1.4496774971e-01
       coeff_l3(23,2)= -4.2674759030e-01
       coeff_l3(23,3)=  2.7158561349e-01
       coeff_l3(23,4)= -9.3661889434e-02
       coeff_l3(23,5)= -3.5074777901e-02
       coeff_l3(23,6)=  1.0141557455e+00
       coeff_l3(23,7)=  4.1548606008e-02
       coeff_l3(23,8)=  5.2138817310e-01
       coeff_l3(23,9)= -3.0487197638e-01
       coeff_l3(23,10)= -2.2842115723e-03
       coeff_l3(23,11)= -1.6461846828e+00
       coeff_l3(23,12)=  1.6988683939e+00
       coeff_l3(23,13)=  1.9139595330e-01
       coeff_l3(23,14)=  1.9599547982e-01
       coeff_l3(23,15)=  1.5649122000e-01
       coeff_l3(23,16)=  6.4085438848e-02
       coeff_l3(23,17)=  1.9175583497e-02
       coeff_l3(23,18)= -6.8690113723e-02
       coeff_l3(23,19)= -5.1400953531e-01
       coeff_l3(23,20)= -3.0145534873e-01
       coeff_l3(23,21)= -5.0610464811e-01
       coeff_l3(23,22)=  7.9777461290e-01
       coeff_l3(23,23)= -3.3757925034e-01
       coeff_l3(23,24)=  1.2111855745e+00
       coeff_l3(23,25)= -1.0974786282e+00
       coeff_l3(23,26)= -6.3299335539e-02
       coeff_l3(23,27)= -2.5597265363e-01
       coeff_l3(23,28)= -4.4959732890e-01
       coeff_l3(23,29)=  1.0733690262e+00
       coeff_l3(23,30)= -3.9124405384e-01
       coeff_l3(23,31)= -1.0723342896e+00
       coeff_l3(23,32)=  4.9329137802e-01
       coeff_l3(23,33)=  9.0488737822e-01
       coeff_l3(23,34)=  1.4954160154e-01
       coeff_l3(23,35)=  8.4900236130e-01
       coeff_l3(23,36)= -1.9594362378e-01
       coeff_l3(23,37)=  1.6795352101e-01
       coeff_l3(23,38)= -4.4900876284e-01
       coeff_l3(23,39)= -1.3811238110e-01
       coeff_l3(23,40)=  1.7103502154e-01
       coeff_l3(23,41)= -8.1248348951e-01
       coeff_l3(23,42)=  9.6800960600e-02
       coeff_l3(23,43)=  4.0109193325e-01
       coeff_l3(23,44)= -1.8904234748e-03
       coeff_l3(23,45)=  1.1563733816e+00
       coeff_l3(23,46)= -3.3055824041e-01
       coeff_l3(23,47)= -5.6349132210e-02
       coeff_l3(23,48)= -4.9837836623e-01
       coeff_l3(23,49)= -4.7500103712e-01
       coeff_l3(23,50)=  6.6555011272e-01
       coeff_l3(23,51)= -1.1529523134e-01
       coeff_l3(23,52)=  4.0461927652e-01
       coeff_l3(23,53)=  3.3482471108e-01
       coeff_l3(23,54)=  2.5992221832e+00
       coeff_l3(23,55)=  3.1526553631e-01
       coeff_l3(23,56)=  1.5002518892e+00
       coeff_l3(24,1)=  4.2482531071e-01
       coeff_l3(24,2)=  1.5619359910e-01
       coeff_l3(24,3)= -1.1779002100e-01
       coeff_l3(24,4)=  3.2157185674e-01
       coeff_l3(24,5)= -2.1291036904e-01
       coeff_l3(24,6)= -2.6970365644e-01
       coeff_l3(24,7)= -4.3336835504e-01
       coeff_l3(24,8)=  1.5261800960e-02
       coeff_l3(24,9)=  9.4687640667e-02
       coeff_l3(24,10)= -3.3673807979e-02
       coeff_l3(24,11)=  5.1348489523e-01
       coeff_l3(24,12)= -1.0516016483e+00
       coeff_l3(24,13)= -5.7787591219e-01
       coeff_l3(24,14)= -4.5912522823e-02
       coeff_l3(24,15)= -1.1086360216e+00
       coeff_l3(24,16)=  3.7027525902e-01
       coeff_l3(24,17)= -7.8055733442e-01
       coeff_l3(24,18)= -5.2002310753e-01
       coeff_l3(24,19)= -1.4208137989e-01
       coeff_l3(24,20)= -1.2169779092e-01
       coeff_l3(24,21)=  2.6039543748e-01
       coeff_l3(24,22)=  4.7051563859e-01
       coeff_l3(24,23)=  2.9129624367e-01
       coeff_l3(24,24)=  3.9620807767e-01
       coeff_l3(24,25)=  1.5589164495e+00
       coeff_l3(24,26)= -7.2637248039e-01
       coeff_l3(24,27)=  7.0547509193e-01
       coeff_l3(24,28)= -7.4709963799e-01
       coeff_l3(24,29)= -5.5352634192e-01
       coeff_l3(24,30)= -3.1038114429e-01
       coeff_l3(24,31)=  6.7877799273e-01
       coeff_l3(24,32)=  2.1458087862e-01
       coeff_l3(24,33)=  1.9407334924e-01
       coeff_l3(24,34)= -3.2623311877e-01
       coeff_l3(24,35)= -8.7182289362e-01
       coeff_l3(24,36)=  6.7378699780e-02
       coeff_l3(24,37)=  7.2450125217e-01
       coeff_l3(24,38)=  1.0944718122e+00
       coeff_l3(24,39)= -2.0155136585e+00
       coeff_l3(24,40)=  2.7727013826e-01
       coeff_l3(24,41)=  3.8076400757e-01
       coeff_l3(24,42)= -1.1308143139e+00
       coeff_l3(24,43)= -5.3005307913e-01
       coeff_l3(24,44)=  1.2199480087e-01
       coeff_l3(24,45)= -8.0467855930e-01
       coeff_l3(24,46)=  2.2301800549e-01
       coeff_l3(24,47)= -8.2700574398e-01
       coeff_l3(24,48)=  1.4950321615e-01
       coeff_l3(24,49)= -1.7819233239e-01
       coeff_l3(24,50)= -3.0833134055e-01
       coeff_l3(24,51)= -7.0289403200e-01
       coeff_l3(24,52)= -8.8863927126e-01
       coeff_l3(24,53)= -2.5256174803e-01
       coeff_l3(24,54)= -2.7140100002e+00
       coeff_l3(24,55)= -4.8946800828e-01
       coeff_l3(24,56)=  7.6824590564e-02
       coeff_l3(25,1)=  1.2420878410e+00
       coeff_l3(25,2)=  3.3573740721e-01
       coeff_l3(25,3)=  1.0742673278e-01
       coeff_l3(25,4)=  4.3641835451e-01
       coeff_l3(25,5)= -1.1091126204e+00
       coeff_l3(25,6)=  2.7802096680e-02
       coeff_l3(25,7)=  2.7224987745e-01
       coeff_l3(25,8)= -3.0872428417e-01
       coeff_l3(25,9)= -2.3053500056e-01
       coeff_l3(25,10)=  3.1742149591e-01
       coeff_l3(25,11)= -6.6723072529e-01
       coeff_l3(25,12)= -2.1697194874e-01
       coeff_l3(25,13)= -1.8130995035e+00
       coeff_l3(25,14)= -3.1541320682e-01
       coeff_l3(25,15)= -1.9232102633e+00
       coeff_l3(25,16)=  4.6838420630e-01
       coeff_l3(25,17)=  4.1256468743e-02
       coeff_l3(25,18)= -7.5980591774e-01
       coeff_l3(25,19)=  3.3796104789e-01
       coeff_l3(25,20)= -1.6816921532e-01
       coeff_l3(25,21)=  1.2117608786e+00
       coeff_l3(25,22)=  1.1071369648e+00
       coeff_l3(25,23)= -1.0037859529e-01
       coeff_l3(25,24)=  6.0128390789e-01
       coeff_l3(25,25)= -1.2733994722e+00
       coeff_l3(25,26)= -4.3999066949e-01
       coeff_l3(25,27)=  1.1567190886e+00
       coeff_l3(25,28)= -5.3974789381e-01
       coeff_l3(25,29)= -1.6374929249e-01
       coeff_l3(25,30)= -7.5686234236e-01
       coeff_l3(25,31)=  7.3659735918e-01
       coeff_l3(25,32)= -2.2339005023e-02
       coeff_l3(25,33)= -1.7214205861e-01
       coeff_l3(25,34)= -5.7090669870e-02
       coeff_l3(25,35)= -3.5655900836e-02
       coeff_l3(25,36)= -5.5450540781e-01
       coeff_l3(25,37)=  6.8763363361e-01
       coeff_l3(25,38)=  1.6736634970e+00
       coeff_l3(25,39)= -1.1822657585e+00
       coeff_l3(25,40)=  2.1457503736e-01
       coeff_l3(25,41)= -3.8782373071e-02
       coeff_l3(25,42)= -2.2099356651e+00
       coeff_l3(25,43)= -7.9870963097e-01
       coeff_l3(25,44)= -2.6862996817e-01
       coeff_l3(25,45)= -2.7581304312e-01
       coeff_l3(25,46)=  2.0867109299e-02
       coeff_l3(25,47)= -1.3756188154e+00
       coeff_l3(25,48)= -7.3625349998e-01
       coeff_l3(25,49)= -1.6992719471e-01
       coeff_l3(25,50)= -4.3534526229e-01
       coeff_l3(25,51)= -2.2720471025e-01
       coeff_l3(25,52)= -7.4198865891e-01
       coeff_l3(25,53)=  3.3807003498e-01
       coeff_l3(25,54)= -2.0725023746e+00
       coeff_l3(25,55)= -1.3928762078e-01
       coeff_l3(25,56)= -2.1996816993e-01
       coeff_l3(26,1)= -2.9678788781e-01
       coeff_l3(26,2)=  7.4040931463e-01
       coeff_l3(26,3)=  2.0929386839e-02
       coeff_l3(26,4)=  2.1833106875e-01
       coeff_l3(26,5)= -1.5242927074e+00
       coeff_l3(26,6)= -7.8863248229e-02
       coeff_l3(26,7)=  9.8341651261e-02
       coeff_l3(26,8)= -2.8949630260e-01
       coeff_l3(26,9)=  1.6886255145e-01
       coeff_l3(26,10)=  5.7191964239e-02
       coeff_l3(26,11)= -2.4075314403e-01
       coeff_l3(26,12)= -2.9392421246e-02
       coeff_l3(26,13)= -2.5123956800e-01
       coeff_l3(26,14)= -3.7785896659e-01
       coeff_l3(26,15)= -1.1457757652e-01
       coeff_l3(26,16)=  2.4409277737e-01
       coeff_l3(26,17)= -1.0962790996e-01
       coeff_l3(26,18)= -2.5204575062e-01
       coeff_l3(26,19)=  2.1034705639e-01
       coeff_l3(26,20)= -4.6804139018e-01
       coeff_l3(26,21)= -6.4137428999e-01
       coeff_l3(26,22)=  1.9784919918e-01
       coeff_l3(26,23)= -1.7727108300e-01
       coeff_l3(26,24)=  1.0827128887e+00
       coeff_l3(26,25)=  6.7154955864e-01
       coeff_l3(26,26)= -3.1733208895e-01
       coeff_l3(26,27)=  7.3754912615e-01
       coeff_l3(26,28)=  2.6014509797e-01
       coeff_l3(26,29)=  1.4117761850e+00
       coeff_l3(26,30)=  1.7128307372e-02
       coeff_l3(26,31)= -1.6723962501e-02
       coeff_l3(26,32)= -1.3002879918e-01
       coeff_l3(26,33)=  5.1995217800e-01
       coeff_l3(26,34)=  2.6075568795e-01
       coeff_l3(26,35)=  6.4797103405e-01
       coeff_l3(26,36)= -1.9605372846e-01
       coeff_l3(26,37)= -6.6657260060e-02
       coeff_l3(26,38)=  6.9692462683e-01
       coeff_l3(26,39)= -6.1527848244e-01
       coeff_l3(26,40)=  2.8430411220e-01
       coeff_l3(26,41)= -4.6268362552e-02
       coeff_l3(26,42)=  4.7455507517e-01
       coeff_l3(26,43)=  3.0603146553e-01
       coeff_l3(26,44)=  1.0346934199e-01
       coeff_l3(26,45)=  2.6605832577e-01
       coeff_l3(26,46)= -6.7494183779e-01
       coeff_l3(26,47)= -6.9199037552e-01
       coeff_l3(26,48)=  1.8016928434e+00
       coeff_l3(26,49)= -5.8528766036e-02
       coeff_l3(26,50)=  9.0273678303e-02
       coeff_l3(26,51)= -9.8221379519e-01
       coeff_l3(26,52)=  5.1643621922e-01
       coeff_l3(26,53)= -9.7710937262e-01
       coeff_l3(26,54)=  9.5834597945e-02
       coeff_l3(26,55)=  7.7357387543e-01
       coeff_l3(26,56)=  2.8498172760e-01
       coeff_l3(27,1)= -1.6283690929e-01
       coeff_l3(27,2)=  2.0611647516e-02
       coeff_l3(27,3)= -2.2282038629e-01
       coeff_l3(27,4)=  6.4910912514e-01
       coeff_l3(27,5)= -5.5936372280e-01
       coeff_l3(27,6)=  2.6865532994e-01
       coeff_l3(27,7)= -2.9902851582e-01
       coeff_l3(27,8)=  5.8965962380e-02
       coeff_l3(27,9)= -4.6495467424e-01
       coeff_l3(27,10)=  1.4805234969e-01
       coeff_l3(27,11)=  9.2688864470e-01
       coeff_l3(27,12)= -1.3865827322e+00
       coeff_l3(27,13)= -1.6364517212e+00
       coeff_l3(27,14)= -1.8935650587e-01
       coeff_l3(27,15)= -2.4932295084e-02
       coeff_l3(27,16)= -1.8304027617e-02
       coeff_l3(27,17)= -3.4071418643e-01
       coeff_l3(27,18)= -7.9424703121e-01
       coeff_l3(27,19)= -3.1441643834e-01
       coeff_l3(27,20)= -4.1446122527e-01
       coeff_l3(27,21)=  9.1396802664e-01
       coeff_l3(27,22)=  8.4202796221e-01
       coeff_l3(27,23)=  1.9229201972e-01
       coeff_l3(27,24)= -8.6275166273e-01
       coeff_l3(27,25)=  1.6249670982e+00
       coeff_l3(27,26)= -1.2237495184e+00
       coeff_l3(27,27)=  4.5165449381e-01
       coeff_l3(27,28)= -3.3828166127e-01
       coeff_l3(27,29)= -1.4245481491e+00
       coeff_l3(27,30)=  7.8099793196e-01
       coeff_l3(27,31)= -2.1606749296e-01
       coeff_l3(27,32)=  1.3871482015e-01
       coeff_l3(27,33)=  2.2218294442e-01
       coeff_l3(27,34)=  4.9348014593e-01
       coeff_l3(27,35)=  1.5054933727e-01
       coeff_l3(27,36)= -2.9987871647e-01
       coeff_l3(27,37)= -2.9038959742e-01
       coeff_l3(27,38)=  1.3242564201e+00
       coeff_l3(27,39)= -8.5018716753e-02
       coeff_l3(27,40)=  1.1610117555e-01
       coeff_l3(27,41)=  1.9056001306e-01
       coeff_l3(27,42)= -1.5853418112e+00
       coeff_l3(27,43)= -8.0804455280e-01
       coeff_l3(27,44)=  2.5867280364e-01
       coeff_l3(27,45)= -4.5537093282e-01
       coeff_l3(27,46)= -9.7566679120e-02
       coeff_l3(27,47)= -7.6269423962e-01
       coeff_l3(27,48)=  2.8651210666e-01
       coeff_l3(27,49)= -1.3535013795e-01
       coeff_l3(27,50)=  3.4340488911e-01
       coeff_l3(27,51)= -7.7309840918e-01
       coeff_l3(27,52)=  3.9058271796e-03
       coeff_l3(27,53)= -1.0164784193e+00
       coeff_l3(27,54)= -1.6413358450e+00
       coeff_l3(27,55)= -2.0367133617e-01
       coeff_l3(27,56)=  7.9090642929e-01
       coeff_l3(28,1)=  1.0284562409e-01
       coeff_l3(28,2)=  1.0529276729e-01
       coeff_l3(28,3)=  3.7249159813e-01
       coeff_l3(28,4)= -3.2226046920e-01
       coeff_l3(28,5)=  1.6742984951e-01
       coeff_l3(28,6)=  1.0525219887e-01
       coeff_l3(28,7)= -7.7854549885e-01
       coeff_l3(28,8)= -3.7959986925e-01
       coeff_l3(28,9)= -7.7446615696e-01
       coeff_l3(28,10)= -1.3531802222e-02
       coeff_l3(28,11)=  5.2567696571e-01
       coeff_l3(28,12)=  1.6172274947e-01
       coeff_l3(28,13)=  5.9439092875e-02
       coeff_l3(28,14)= -1.6219618917e-01
       coeff_l3(28,15)=  2.5857551023e-02
       coeff_l3(28,16)= -2.1610525250e-01
       coeff_l3(28,17)= -1.4789133333e-02
       coeff_l3(28,18)= -3.4729802608e-01
       coeff_l3(28,19)= -1.2566591799e-01
       coeff_l3(28,20)=  3.3630988002e-01
       coeff_l3(28,21)= -4.4015333056e-01
       coeff_l3(28,22)=  8.0320221186e-01
       coeff_l3(28,23)=  2.2401554883e-01
       coeff_l3(28,24)=  5.0603485107e-01
       coeff_l3(28,25)=  1.3594177961e+00
       coeff_l3(28,26)= -8.2883250713e-01
       coeff_l3(28,27)=  4.1240644455e-01
       coeff_l3(28,28)= -1.6003185511e-01
       coeff_l3(28,29)=  4.9574834108e-01
       coeff_l3(28,30)= -4.6831798553e-01
       coeff_l3(28,31)= -3.0236369371e-01
       coeff_l3(28,32)=  5.9729284048e-01
       coeff_l3(28,33)= -1.8344792724e-01
       coeff_l3(28,34)= -3.2230606675e-01
       coeff_l3(28,35)=  2.9258584976e-01
       coeff_l3(28,36)= -3.3899974823e-01
       coeff_l3(28,37)= -2.3440681398e-02
       coeff_l3(28,38)=  3.3713072538e-01
       coeff_l3(28,39)=  3.7398448586e-01
       coeff_l3(28,40)= -8.8747847080e-01
       coeff_l3(28,41)= -6.3928931952e-01
       coeff_l3(28,42)=  1.8597143888e-01
       coeff_l3(28,43)=  1.3073880970e-01
       coeff_l3(28,44)=  2.3789793253e-01
       coeff_l3(28,45)=  1.0814355612e+00
       coeff_l3(28,46)= -7.8916639090e-01
       coeff_l3(28,47)= -3.4816408157e-01
       coeff_l3(28,48)=  1.0163769722e+00
       coeff_l3(28,49)=  1.8279816210e-01
       coeff_l3(28,50)=  4.8734113574e-01
       coeff_l3(28,51)= -5.9523779154e-01
       coeff_l3(28,52)= -4.0386918932e-02
       coeff_l3(28,53)= -2.7415680885e-01
       coeff_l3(28,54)= -4.0493598580e-01
       coeff_l3(28,55)= -6.7527301610e-02
       coeff_l3(28,56)=  1.1063113213e+00
       coeff_l3(29,1)=  2.7341043949e-01
       coeff_l3(29,2)= -5.4693792015e-02
       coeff_l3(29,3)= -2.1540714800e-01
       coeff_l3(29,4)= -3.2327372581e-02
       coeff_l3(29,5)= -2.2237034142e-01
       coeff_l3(29,6)= -1.8132913113e-01
       coeff_l3(29,7)=  1.2033460662e-02
       coeff_l3(29,8)= -4.2185851932e-01
       coeff_l3(29,9)=  1.8067270517e-01
       coeff_l3(29,10)= -6.7769877613e-02
       coeff_l3(29,11)=  5.1065701246e-01
       coeff_l3(29,12)= -1.6984900236e+00
       coeff_l3(29,13)= -7.0361816883e-01
       coeff_l3(29,14)=  1.1513970047e-01
       coeff_l3(29,15)=  6.0457628220e-02
       coeff_l3(29,16)=  1.9545014203e-01
       coeff_l3(29,17)=  6.2059417367e-02
       coeff_l3(29,18)= -1.0409640074e+00
       coeff_l3(29,19)= -3.9724409580e-01
       coeff_l3(29,20)= -3.0462276936e-01
       coeff_l3(29,21)=  5.9544020891e-01
       coeff_l3(29,22)=  6.0767543316e-01
       coeff_l3(29,23)=  3.1588280201e-01
       coeff_l3(29,24)= -5.3637212515e-01
       coeff_l3(29,25)=  1.7930915952e-01
       coeff_l3(29,26)= -4.1997668147e-01
       coeff_l3(29,27)=  2.8574353456e-01
       coeff_l3(29,28)= -7.1060246229e-01
       coeff_l3(29,29)= -1.2133289576e+00
       coeff_l3(29,30)= -8.9044141769e-01
       coeff_l3(29,31)=  2.6128903031e-01
       coeff_l3(29,32)=  1.6182762384e-01
       coeff_l3(29,33)=  2.0836174488e-01
       coeff_l3(29,34)= -7.5509823859e-02
       coeff_l3(29,35)= -6.6586542130e-01
       coeff_l3(29,36)= -2.5045137852e-02
       coeff_l3(29,37)=  1.7267835140e-01
       coeff_l3(29,38)=  9.4577831030e-01
       coeff_l3(29,39)=  3.1734278798e-01
       coeff_l3(29,40)=  4.0810158849e-01
       coeff_l3(29,41)=  3.6938491464e-01
       coeff_l3(29,42)= -5.1826149225e-01
       coeff_l3(29,43)= -7.3869228363e-01
       coeff_l3(29,44)=  4.2416360974e-01
       coeff_l3(29,45)= -7.3177725077e-01
       coeff_l3(29,46)=  6.0690719634e-02
       coeff_l3(29,47)= -4.2823535204e-01
       coeff_l3(29,48)= -4.9414193630e-01
       coeff_l3(29,49)=  1.4322096109e-01
       coeff_l3(29,50)=  1.5613558888e-01
       coeff_l3(29,51)= -2.3962415755e-01
       coeff_l3(29,52)= -2.5367072225e-01
       coeff_l3(29,53)= -1.2366138399e-01
       coeff_l3(29,54)= -1.5356062651e+00
       coeff_l3(29,55)= -7.3540204763e-01
       coeff_l3(29,56)=  3.3732119948e-02
       coeff_l3(30,1)= -6.9959956408e-01
       coeff_l3(30,2)=  1.7217601836e-01
       coeff_l3(30,3)=  4.0665519238e-01
       coeff_l3(30,4)=  7.9586684704e-01
       coeff_l3(30,5)= -4.1340917349e-01
       coeff_l3(30,6)= -7.2316581011e-01
       coeff_l3(30,7)= -6.9137990475e-01
       coeff_l3(30,8)= -5.9262126684e-01
       coeff_l3(30,9)=  3.4358143806e-01
       coeff_l3(30,10)=  4.9878850579e-02
       coeff_l3(30,11)=  2.9825618863e-01
       coeff_l3(30,12)= -6.7577087879e-01
       coeff_l3(30,13)= -1.3023417443e-02
       coeff_l3(30,14)= -1.1124674082e+00
       coeff_l3(30,15)= -3.2513406873e-01
       coeff_l3(30,16)= -8.2428658009e-01
       coeff_l3(30,17)= -3.6623564363e-01
       coeff_l3(30,18)=  4.7382977605e-01
       coeff_l3(30,19)= -1.8984556198e+00
       coeff_l3(30,20)=  3.6002550274e-02
       coeff_l3(30,21)=  1.7137738466e+00
       coeff_l3(30,22)= -3.8938742876e-01
       coeff_l3(30,23)= -1.3314734399e-01
       coeff_l3(30,24)=  1.4962403774e+00
       coeff_l3(30,25)=  2.6739094257e+00
       coeff_l3(30,26)=  1.7701261044e+00
       coeff_l3(30,27)=  1.5123378038e+00
       coeff_l3(30,28)=  5.5096673965e-01
       coeff_l3(30,29)=  2.1096584797e+00
       coeff_l3(30,30)= -3.5386916995e-01
       coeff_l3(30,31)=  9.4017595053e-02
       coeff_l3(30,32)= -2.9660969973e-01
       coeff_l3(30,33)=  9.3899652362e-02
       coeff_l3(30,34)=  1.1051784456e-01
       coeff_l3(30,35)=  1.2838948965e+00
       coeff_l3(30,36)= -6.0581922531e-01
       coeff_l3(30,37)= -7.9999428988e-01
       coeff_l3(30,38)=  9.3961125612e-01
       coeff_l3(30,39)=  1.8091566861e-01
       coeff_l3(30,40)=  1.8508474529e-01
       coeff_l3(30,41)=  5.0839710236e-01
       coeff_l3(30,42)=  8.9002531767e-01
       coeff_l3(30,43)= -2.7571445704e-01
       coeff_l3(30,44)=  5.4078114033e-01
       coeff_l3(30,45)=  5.0887352228e-01
       coeff_l3(30,46)=  2.2678117454e-01
       coeff_l3(30,47)=  1.1200890541e+00
       coeff_l3(30,48)=  2.0260534286e+00
       coeff_l3(30,49)= -4.1502612829e-01
       coeff_l3(30,50)=  1.5456491709e+00
       coeff_l3(30,51)=  7.0235013962e-01
       coeff_l3(30,52)=  9.4765985012e-01
       coeff_l3(30,53)= -1.6327321529e+00
       coeff_l3(30,54)= -1.6098288298e+00
       coeff_l3(30,55)=  4.8616850376e-01
       coeff_l3(30,56)=  1.1802617311e+00
       coeff_l3(31,1)=  8.5521012545e-02
       coeff_l3(31,2)=  6.5338402987e-01
       coeff_l3(31,3)=  2.5379186869e-01
       coeff_l3(31,4)= -5.0348216295e-01
       coeff_l3(31,5)=  3.1349796057e-01
       coeff_l3(31,6)=  5.4264426231e-01
       coeff_l3(31,7)=  2.6241190732e-02
       coeff_l3(31,8)=  1.2120740861e-01
       coeff_l3(31,9)= -4.1395676136e-01
       coeff_l3(31,10)=  1.4022846520e-01
       coeff_l3(31,11)= -5.9975212812e-01
       coeff_l3(31,12)= -3.9477044344e-01
       coeff_l3(31,13)= -6.4577206969e-02
       coeff_l3(31,14)= -3.8801487535e-02
       coeff_l3(31,15)= -3.1520536542e-01
       coeff_l3(31,16)=  1.7359495163e-01
       coeff_l3(31,17)=  1.4154297113e-01
       coeff_l3(31,18)= -6.2423348427e-01
       coeff_l3(31,19)= -1.1030354500e+00
       coeff_l3(31,20)=  3.6983892322e-01
       coeff_l3(31,21)= -4.2096132040e-01
       coeff_l3(31,22)=  1.9837613106e+00
       coeff_l3(31,23)=  1.2325982749e-01
       coeff_l3(31,24)=  9.6112042665e-02
       coeff_l3(31,25)=  1.4332649708e+00
       coeff_l3(31,26)= -4.3769517541e-01
       coeff_l3(31,27)=  3.3790796995e-01
       coeff_l3(31,28)= -4.1871118546e-01
       coeff_l3(31,29)= -4.3745484948e-01
       coeff_l3(31,30)=  1.5571917593e-01
       coeff_l3(31,31)=  2.0468482375e-01
       coeff_l3(31,32)=  1.5752020478e-01
       coeff_l3(31,33)=  4.3307262659e-01
       coeff_l3(31,34)= -7.3335653543e-01
       coeff_l3(31,35)= -3.0219194293e-01
       coeff_l3(31,36)=  1.0283932835e-01
       coeff_l3(31,37)=  1.1456708610e-01
       coeff_l3(31,38)=  3.2967692614e-01
       coeff_l3(31,39)=  1.2276560068e-01
       coeff_l3(31,40)=  4.5112839341e-01
       coeff_l3(31,41)=  4.2538782954e-01
       coeff_l3(31,42)= -1.1383651495e+00
       coeff_l3(31,43)=  3.3009502292e-01
       coeff_l3(31,44)=  2.1588282660e-02
       coeff_l3(31,45)=  6.7627316713e-01
       coeff_l3(31,46)=  5.4344147444e-01
       coeff_l3(31,47)= -7.7296748757e-02
       coeff_l3(31,48)=  3.2692703605e-01
       coeff_l3(31,49)= -1.6231042147e-01
       coeff_l3(31,50)=  3.2220583409e-02
       coeff_l3(31,51)= -1.2750239670e-01
       coeff_l3(31,52)= -3.9696744084e-01
       coeff_l3(31,53)= -7.2318963706e-02
       coeff_l3(31,54)= -3.7989562750e-01
       coeff_l3(31,55)= -5.3875442594e-02
       coeff_l3(31,56)=  7.7752131224e-01
       coeff_l3(32,1)=  1.1094255745e-01
       coeff_l3(32,2)= -4.1144302487e-01
       coeff_l3(32,3)= -4.9800977111e-02
       coeff_l3(32,4)=  7.9833853245e-01
       coeff_l3(32,5)= -5.6043225527e-01
       coeff_l3(32,6)=  1.5265074968e+00
       coeff_l3(32,7)= -7.1975219250e-01
       coeff_l3(32,8)= -8.3419185877e-01
       coeff_l3(32,9)= -1.6639579833e-01
       coeff_l3(32,10)=  2.6095810533e-01
       coeff_l3(32,11)=  1.5803693533e+00
       coeff_l3(32,12)= -2.0196902752e+00
       coeff_l3(32,13)=  2.0616842806e-01
       coeff_l3(32,14)=  2.0623558760e-01
       coeff_l3(32,15)= -6.7030560970e-01
       coeff_l3(32,16)=  6.4555460215e-01
       coeff_l3(32,17)=  6.0996001959e-01
       coeff_l3(32,18)= -2.6276853681e-01
       coeff_l3(32,19)= -1.5935137868e-01
       coeff_l3(32,20)=  7.9684162140e-01
       coeff_l3(32,21)=  2.9790616035e-01
       coeff_l3(32,22)= -6.6957518458e-02
       coeff_l3(32,23)=  6.2463325262e-01
       coeff_l3(32,24)= -5.6160855293e-01
       coeff_l3(32,25)=  9.3475401402e-01
       coeff_l3(32,26)=  1.7285145819e-01
       coeff_l3(32,27)=  7.2132402658e-01
       coeff_l3(32,28)= -8.5565990210e-01
       coeff_l3(32,29)= -3.5243475437e+00
       coeff_l3(32,30)=  2.6694415137e-02
       coeff_l3(32,31)=  2.7503981590e+00
       coeff_l3(32,32)= -1.7858546972e-01
       coeff_l3(32,33)=  4.2728352547e-01
       coeff_l3(32,34)= -2.8269734979e-01
       coeff_l3(32,35)= -3.8684749603e-01
       coeff_l3(32,36)= -4.7073966265e-01
       coeff_l3(32,37)=  2.8106430173e-01
       coeff_l3(32,38)=  4.8267433047e-01
       coeff_l3(32,39)= -2.6236134768e-01
       coeff_l3(32,40)= -1.3169223070e-01
       coeff_l3(32,41)=  3.4675461054e-01
       coeff_l3(32,42)= -9.7434747219e-01
       coeff_l3(32,43)= -8.4596031904e-01
       coeff_l3(32,44)=  3.0291905999e-01
       coeff_l3(32,45)= -1.2857465744e+00
       coeff_l3(32,46)= -3.8234758377e-01
       coeff_l3(32,47)= -4.4372618198e-01
       coeff_l3(32,48)=  5.0644969940e-01
       coeff_l3(32,49)=  1.6445331275e-01
       coeff_l3(32,50)= -1.6498751640e+00
       coeff_l3(32,51)=  4.1911411285e-01
       coeff_l3(32,52)= -4.8394060135e-01
       coeff_l3(32,53)= -4.4941231608e-01
       coeff_l3(32,54)= -2.1865715981e+00
       coeff_l3(32,55)=  6.6613376141e-01
       coeff_l3(32,56)=  2.8724911809e-01
       coeff_l3(33,1)= -4.0790522099e-01
       coeff_l3(33,2)=  1.0633910894e+00
       coeff_l3(33,3)=  3.7879914045e-01
       coeff_l3(33,4)= -3.5310339928e-01
       coeff_l3(33,5)= -1.1049969196e+00
       coeff_l3(33,6)= -4.9073693156e-01
       coeff_l3(33,7)= -1.0753701925e+00
       coeff_l3(33,8)= -6.4728075266e-01
       coeff_l3(33,9)= -3.9374500513e-02
       coeff_l3(33,10)= -1.0171806440e-02
       coeff_l3(33,11)= -8.4554862976e-01
       coeff_l3(33,12)= -5.1720327139e-01
       coeff_l3(33,13)= -1.1976808310e+00
       coeff_l3(33,14)= -3.7748271227e-01
       coeff_l3(33,15)= -5.6453245878e-01
       coeff_l3(33,16)=  6.7779785395e-01
       coeff_l3(33,17)= -3.7250716239e-02
       coeff_l3(33,18)= -3.2512843609e-01
       coeff_l3(33,19)=  1.0251249373e-01
       coeff_l3(33,20)= -5.2754700184e-01
       coeff_l3(33,21)=  1.0507453233e-01
       coeff_l3(33,22)=  1.1324336529e+00
       coeff_l3(33,23)= -5.0999617577e-01
       coeff_l3(33,24)=  8.2966643572e-01
       coeff_l3(33,25)= -6.9712603092e-01
       coeff_l3(33,26)= -3.1483885646e-01
       coeff_l3(33,27)=  8.3359885216e-01
       coeff_l3(33,28)= -5.1530742645e-01
       coeff_l3(33,29)=  5.6132149696e-01
       coeff_l3(33,30)= -1.2792544365e+00
       coeff_l3(33,31)= -5.6726753712e-01
       coeff_l3(33,32)= -3.2002958655e-01
       coeff_l3(33,33)=  9.8156696558e-01
       coeff_l3(33,34)= -4.0802720189e-01
       coeff_l3(33,35)=  6.9982752204e-02
       coeff_l3(33,36)= -1.3994182348e+00
       coeff_l3(33,37)= -6.1772477627e-01
       coeff_l3(33,38)=  1.2912336588e+00
       coeff_l3(33,39)= -8.0975466967e-01
       coeff_l3(33,40)=  6.4408117533e-01
       coeff_l3(33,41)= -7.0081567764e-01
       coeff_l3(33,42)= -5.7335537672e-01
       coeff_l3(33,43)= -5.2024674416e-01
       coeff_l3(33,44)= -3.3433312178e-01
       coeff_l3(33,45)=  5.3939116001e-01
       coeff_l3(33,46)= -7.9724258184e-01
       coeff_l3(33,47)= -4.2610627413e-01
       coeff_l3(33,48)=  2.7327212691e-01
       coeff_l3(33,49)= -2.9430279136e-01
       coeff_l3(33,50)=  3.3353507519e-01
       coeff_l3(33,51)= -1.3764007092e+00
       coeff_l3(33,52)=  1.2393498793e-02
       coeff_l3(33,53)= -8.1530511379e-01
       coeff_l3(33,54)= -1.7108024359e+00
       coeff_l3(33,55)= -6.9795113802e-01
       coeff_l3(33,56)=  5.2519470453e-01
       coeff_l3(34,1)=  8.4842637181e-02
       coeff_l3(34,2)= -6.3214689493e-01
       coeff_l3(34,3)=  3.7588754296e-01
       coeff_l3(34,4)=  2.4345147610e-01
       coeff_l3(34,5)= -1.8167728558e-02
       coeff_l3(34,6)= -4.4912528247e-02
       coeff_l3(34,7)=  6.2347632647e-01
       coeff_l3(34,8)= -1.9027471542e-01
       coeff_l3(34,9)= -2.0360647142e-01
       coeff_l3(34,10)= -3.3569484949e-01
       coeff_l3(34,11)= -8.5866399109e-02
       coeff_l3(34,12)=  7.0899748243e-04
       coeff_l3(34,13)=  3.1366342306e-01
       coeff_l3(34,14)=  1.1609146595e+00
       coeff_l3(34,15)= -4.1071170568e-01
       coeff_l3(34,16)=  3.7313126028e-02
       coeff_l3(34,17)=  4.0168937296e-02
       coeff_l3(34,18)=  4.0505886078e-01
       coeff_l3(34,19)= -3.0777126551e-01
       coeff_l3(34,20)= -3.2690358162e-01
       coeff_l3(34,21)=  4.5729926229e-01
       coeff_l3(34,22)= -3.4597203135e-01
       coeff_l3(34,23)= -9.1730214655e-02
       coeff_l3(34,24)= -8.8740622997e-01
       coeff_l3(34,25)=  4.7288088799e+00
       coeff_l3(34,26)=  8.0843466520e-01
       coeff_l3(34,27)= -4.3832463026e-01
       coeff_l3(34,28)= -5.0995016098e-01
       coeff_l3(34,29)=  2.0714192092e-01
       coeff_l3(34,30)=  6.9675582647e-01
       coeff_l3(34,31)=  7.2879379988e-01
       coeff_l3(34,32)= -6.1287432909e-01
       coeff_l3(34,33)= -4.2578038573e-01
       coeff_l3(34,34)= -3.1652653217e-01
       coeff_l3(34,35)=  5.0831866264e-01
       coeff_l3(34,36)= -1.5429534018e-01
       coeff_l3(34,37)=  5.3674167395e-01
       coeff_l3(34,38)= -6.8520897627e-01
       coeff_l3(34,39)=  4.8407673836e-02
       coeff_l3(34,40)= -3.5741308331e-01
       coeff_l3(34,41)=  3.3279335499e-01
       coeff_l3(34,42)= -2.3236536980e-01
       coeff_l3(34,43)= -4.4821190834e-01
       coeff_l3(34,44)=  6.2792323530e-02
       coeff_l3(34,45)= -9.6579843760e-01
       coeff_l3(34,46)=  9.3009805679e-01
       coeff_l3(34,47)= -7.2348560207e-03
       coeff_l3(34,48)= -1.6524850130e+00
       coeff_l3(34,49)=  3.3264394850e-02
       coeff_l3(34,50)= -6.6110789776e-01
       coeff_l3(34,51)= -7.8010600805e-01
       coeff_l3(34,52)=  2.7826061845e-01
       coeff_l3(34,53)= -1.3011585921e-02
       coeff_l3(34,54)=  1.2091407776e+00
       coeff_l3(34,55)= -1.4183117449e-01
       coeff_l3(34,56)= -8.9632391930e-01
       coeff_l3(35,1)=  3.0033996701e-01
       coeff_l3(35,2)= -5.0365459919e-01
       coeff_l3(35,3)=  3.1927058101e-01
       coeff_l3(35,4)=  2.0340815187e-01
       coeff_l3(35,5)=  1.2326798588e-01
       coeff_l3(35,6)=  6.0025018454e-01
       coeff_l3(35,7)= -1.2587822974e-01
       coeff_l3(35,8)=  1.5412139893e-01
       coeff_l3(35,9)= -4.9456968904e-01
       coeff_l3(35,10)= -3.5195782781e-02
       coeff_l3(35,11)=  1.5538480133e-02
       coeff_l3(35,12)=  4.4328126311e-01
       coeff_l3(35,13)= -6.5670448542e-01
       coeff_l3(35,14)= -7.9681752250e-03
       coeff_l3(35,15)=  6.7969687283e-02
       coeff_l3(35,16)=  2.3872937262e-01
       coeff_l3(35,17)=  5.3384882212e-01
       coeff_l3(35,18)= -6.0088749975e-02
       coeff_l3(35,19)=  2.5846946239e-01
       coeff_l3(35,20)=  2.7769809589e-02
       coeff_l3(35,21)=  1.3321946561e-01
       coeff_l3(35,22)=  1.9278992712e-01
       coeff_l3(35,23)=  1.8484593928e-01
       coeff_l3(35,24)= -1.0703414679e+00
       coeff_l3(35,25)= -2.7085500956e-01
       coeff_l3(35,26)= -5.3621482849e-01
       coeff_l3(35,27)=  4.3796256185e-01
       coeff_l3(35,28)=  9.5435135067e-02
       coeff_l3(35,29)= -3.7788695097e-01
       coeff_l3(35,30)= -7.6759740710e-02
       coeff_l3(35,31)= -3.2240813971e-01
       coeff_l3(35,32)=  1.0203167796e-01
       coeff_l3(35,33)= -3.9810813963e-02
       coeff_l3(35,34)=  3.8777783513e-01
       coeff_l3(35,35)= -1.4397688210e-01
       coeff_l3(35,36)=  1.3947167993e-01
       coeff_l3(35,37)= -4.6124705672e-01
       coeff_l3(35,38)=  6.2023885548e-02
       coeff_l3(35,39)= -2.0706784725e-01
       coeff_l3(35,40)= -2.9418364167e-02
       coeff_l3(35,41)=  1.5759201488e-03
       coeff_l3(35,42)=  6.6952097416e-01
       coeff_l3(35,43)= -3.2946124673e-02
       coeff_l3(35,44)=  4.1985547543e-01
       coeff_l3(35,45)=  4.6014049649e-01
       coeff_l3(35,46)= -2.4673669040e-01
       coeff_l3(35,47)= -4.8909419775e-01
       coeff_l3(35,48)=  3.8753053546e-01
       coeff_l3(35,49)= -1.2337593734e-01
       coeff_l3(35,50)=  1.9813458622e-01
       coeff_l3(35,51)= -6.9219213724e-01
       coeff_l3(35,52)=  7.2314077616e-01
       coeff_l3(35,53)= -9.2883569002e-01
       coeff_l3(35,54)=  1.2053868771e+00
       coeff_l3(35,55)=  4.9161057919e-02
       coeff_l3(35,56)= -1.7792937160e-01
       coeff_l3(36,1)=  2.6510721445e-01
       coeff_l3(36,2)= -7.4666617438e-03
       coeff_l3(36,3)=  3.8845324516e-01
       coeff_l3(36,4)= -1.9428010285e-01
       coeff_l3(36,5)=  2.9850250483e-01
       coeff_l3(36,6)= -7.6615405083e-01
       coeff_l3(36,7)=  5.6331998110e-01
       coeff_l3(36,8)= -9.6859598160e-01
       coeff_l3(36,9)= -1.4141388237e-02
       coeff_l3(36,10)= -9.6229523420e-02
       coeff_l3(36,11)=  1.4413312674e+00
       coeff_l3(36,12)= -1.8023265898e-01
       coeff_l3(36,13)= -1.9083881378e-01
       coeff_l3(36,14)=  3.9296904206e-01
       coeff_l3(36,15)= -7.0819550753e-01
       coeff_l3(36,16)=  5.6009944528e-02
       coeff_l3(36,17)= -5.5901701562e-03
       coeff_l3(36,18)= -3.1573027372e-01
       coeff_l3(36,19)= -1.6692759097e-01
       coeff_l3(36,20)=  6.5036040545e-01
       coeff_l3(36,21)= -4.7381106764e-02
       coeff_l3(36,22)= -2.3533517122e-01
       coeff_l3(36,23)= -1.1889355630e-01
       coeff_l3(36,24)= -2.2709736228e-01
       coeff_l3(36,25)= -4.7037396431e+00
       coeff_l3(36,26)=  1.4160526991e+00
       coeff_l3(36,27)= -4.1113561392e-01
       coeff_l3(36,28)=  1.6290105879e-01
       coeff_l3(36,29)= -1.8085508049e-01
       coeff_l3(36,30)=  6.4662110806e-01
       coeff_l3(36,31)=  2.2199393809e-01
       coeff_l3(36,32)=  1.2688076124e-02
       coeff_l3(36,33)= -5.4017311335e-01
       coeff_l3(36,34)=  2.9910758138e-01
       coeff_l3(36,35)= -7.1812190115e-02
       coeff_l3(36,36)=  2.3307245970e-01
       coeff_l3(36,37)= -2.2933213413e-01
       coeff_l3(36,38)= -5.7284617424e-01
       coeff_l3(36,39)= -5.6966102123e-01
       coeff_l3(36,40)= -7.7768745832e-03
       coeff_l3(36,41)=  5.4628103971e-01
       coeff_l3(36,42)=  1.8079876900e-01
       coeff_l3(36,43)=  8.2930646837e-02
       coeff_l3(36,44)= -6.4372017980e-02
       coeff_l3(36,45)= -5.5041681975e-02
       coeff_l3(36,46)=  3.1581923366e-01
       coeff_l3(36,47)= -4.1146355867e-01
       coeff_l3(36,48)= -1.4500524104e-01
       coeff_l3(36,49)=  5.7974565029e-01
       coeff_l3(36,50)= -1.9466237724e-01
       coeff_l3(36,51)= -2.0261771977e-01
       coeff_l3(36,52)= -4.9924570322e-01
       coeff_l3(36,53)= -5.2798187733e-01
       coeff_l3(36,54)= -8.7428647280e-01
       coeff_l3(36,55)= -1.9938690960e-01
       coeff_l3(36,56)= -6.6752296686e-01
       coeff_l3(37,1)=  5.6101459265e-01
       coeff_l3(37,2)= -7.5593262911e-02
       coeff_l3(37,3)= -2.3681202531e-01
       coeff_l3(37,4)= -3.1514707208e-01
       coeff_l3(37,5)=  1.4651037753e-01
       coeff_l3(37,6)= -9.2553071678e-02
       coeff_l3(37,7)= -2.5887456536e-01
       coeff_l3(37,8)=  4.6411967278e-01
       coeff_l3(37,9)= -1.5646111965e-01
       coeff_l3(37,10)= -3.6139994860e-01
       coeff_l3(37,11)=  9.0815149248e-02
       coeff_l3(37,12)= -4.9045002460e-01
       coeff_l3(37,13)= -3.9721611887e-02
       coeff_l3(37,14)=  5.8666604757e-01
       coeff_l3(37,15)=  5.1631855965e-01
       coeff_l3(37,16)=  3.6525750160e-01
       coeff_l3(37,17)= -2.4882167578e-01
       coeff_l3(37,18)=  1.6460070014e-01
       coeff_l3(37,19)=  5.7071920484e-02
       coeff_l3(37,20)= -2.1723330021e-01
       coeff_l3(37,21)= -4.5134282112e-01
       coeff_l3(37,22)= -8.6864602566e-01
       coeff_l3(37,23)= -4.8921412230e-01
       coeff_l3(37,24)= -4.2847341299e-01
       coeff_l3(37,25)= -4.5017367601e-01
       coeff_l3(37,26)= -1.5128417015e+00
       coeff_l3(37,27)= -2.2590825334e-02
       coeff_l3(37,28)= -1.5790876746e-01
       coeff_l3(37,29)=  2.0948851109e+00
       coeff_l3(37,30)=  2.5926057249e-02
       coeff_l3(37,31)=  2.8524222970e-01
       coeff_l3(37,32)=  1.1625885218e-01
       coeff_l3(37,33)= -1.3157565892e-01
       coeff_l3(37,34)=  1.3171409070e-01
       coeff_l3(37,35)= -2.3030221462e-01
       coeff_l3(37,36)= -3.1526184082e-01
       coeff_l3(37,37)= -4.0727546811e-01
       coeff_l3(37,38)= -3.0520188808e+00
       coeff_l3(37,39)=  4.4403654337e-01
       coeff_l3(37,40)=  1.5501160920e-01
       coeff_l3(37,41)= -3.5630950332e-01
       coeff_l3(37,42)=  4.2855873704e-02
       coeff_l3(37,43)= -7.3528027534e-01
       coeff_l3(37,44)= -1.4685125649e-01
       coeff_l3(37,45)= -1.5673710108e+00
       coeff_l3(37,46)=  7.1946956217e-02
       coeff_l3(37,47)= -2.4974970520e-01
       coeff_l3(37,48)= -3.3267441392e-01
       coeff_l3(37,49)= -1.3163724542e-01
       coeff_l3(37,50)=  2.3169271648e-01
       coeff_l3(37,51)= -5.7976919413e-01
       coeff_l3(37,52)=  4.6814668179e-01
       coeff_l3(37,53)= -6.2294441462e-01
       coeff_l3(37,54)= -1.9579030275e+00
       coeff_l3(37,55)= -1.4651034772e-01
       coeff_l3(37,56)=  2.4539852142e-01
       coeff_l3(38,1)= -2.2519113123e-01
       coeff_l3(38,2)=  1.6456505656e-01
       coeff_l3(38,3)= -4.1808092594e-01
       coeff_l3(38,4)= -1.8991383910e-01
       coeff_l3(38,5)= -1.0765077919e-01
       coeff_l3(38,6)= -5.8432048559e-01
       coeff_l3(38,7)=  7.7315592766e-01
       coeff_l3(38,8)= -5.0956183672e-01
       coeff_l3(38,9)= -2.1209299564e-01
       coeff_l3(38,10)=  2.7148017287e-01
       coeff_l3(38,11)=  1.0492893457e+00
       coeff_l3(38,12)= -1.5851310492e+00
       coeff_l3(38,13)= -7.2891020775e-01
       coeff_l3(38,14)= -3.0863374472e-01
       coeff_l3(38,15)= -5.6585764885e-01
       coeff_l3(38,16)=  3.1345427036e-01
       coeff_l3(38,17)= -4.9227744341e-01
       coeff_l3(38,18)= -9.2977797985e-01
       coeff_l3(38,19)=  3.0405882001e-01
       coeff_l3(38,20)=  5.4070490599e-01
       coeff_l3(38,21)=  2.1342140436e-01
       coeff_l3(38,22)= -1.2252039909e+00
       coeff_l3(38,23)=  2.3359850049e-01
       coeff_l3(38,24)=  3.8354200125e-01
       coeff_l3(38,25)=  3.9343709946e+00
       coeff_l3(38,26)=  2.3597204685e-01
       coeff_l3(38,27)=  3.1427434087e-01
       coeff_l3(38,28)=  4.9847128987e-01
       coeff_l3(38,29)= -1.3469627500e-01
       coeff_l3(38,30)=  5.3924924135e-01
       coeff_l3(38,31)=  9.5140033960e-01
       coeff_l3(38,32)= -9.7284239531e-01
       coeff_l3(38,33)= -8.6310434341e-01
       coeff_l3(38,34)=  5.7958143950e-01
       coeff_l3(38,35)= -6.4016658068e-01
       coeff_l3(38,36)=  1.3581854105e-01
       coeff_l3(38,37)= -3.7284651399e-01
       coeff_l3(38,38)=  8.4668314457e-01
       coeff_l3(38,39)=  8.5272185504e-02
       coeff_l3(38,40)=  1.8482792377e-01
       coeff_l3(38,41)= -4.5313104987e-02
       coeff_l3(38,42)=  1.0269446373e+00
       coeff_l3(38,43)=  3.2598239183e-01
       coeff_l3(38,44)=  1.7529472709e-01
       coeff_l3(38,45)= -8.1914252043e-01
       coeff_l3(38,46)=  2.3610638082e-01
       coeff_l3(38,47)= -1.5048080683e+00
       coeff_l3(38,48)=  2.3531749249e+00
       coeff_l3(38,49)=  5.1823675632e-01
       coeff_l3(38,50)= -5.0006043911e-01
       coeff_l3(38,51)= -1.3739265203e+00
       coeff_l3(38,52)=  8.7515905499e-02
       coeff_l3(38,53)= -4.7376221418e-01
       coeff_l3(38,54)= -1.7778887749e+00
       coeff_l3(38,55)=  5.2432632446e-01
       coeff_l3(38,56)= -2.3066614568e-01
       coeff_l3(39,1)=  8.9549176395e-02
       coeff_l3(39,2)= -3.9157348871e-01
       coeff_l3(39,3)= -4.2452356219e-01
       coeff_l3(39,4)= -3.3843642473e-01
       coeff_l3(39,5)=  8.1263846159e-01
       coeff_l3(39,6)=  1.3307614625e-01
       coeff_l3(39,7)=  7.8952199221e-01
       coeff_l3(39,8)= -1.0465710610e-01
       coeff_l3(39,9)=  4.0105858445e-01
       coeff_l3(39,10)= -9.5237262547e-02
       coeff_l3(39,11)= -1.2935751677e+00
       coeff_l3(39,12)=  1.2836318016e+00
       coeff_l3(39,13)=  1.4927997589e+00
       coeff_l3(39,14)=  1.2817336321e+00
       coeff_l3(39,15)= -5.9285122156e-01
       coeff_l3(39,16)= -9.3214046955e-01
       coeff_l3(39,17)=  6.5660482645e-01
       coeff_l3(39,18)= -5.4857802391e-01
       coeff_l3(39,19)=  1.5994527936e-01
       coeff_l3(39,20)= -1.1618459225e+00
       coeff_l3(39,21)= -1.0134395361e+00
       coeff_l3(39,22)=  6.8213723600e-02
       coeff_l3(39,23)= -5.5553668737e-01
       coeff_l3(39,24)= -2.3145927489e-01
       coeff_l3(39,25)= -9.6821326017e-01
       coeff_l3(39,26)=  2.7665829659e+00
       coeff_l3(39,27)=  1.1287508160e-01
       coeff_l3(39,28)= -1.5388789177e+00
       coeff_l3(39,29)= -2.3698780537e+00
       coeff_l3(39,30)= -4.4811028242e-01
       coeff_l3(39,31)= -5.2073156834e-01
       coeff_l3(39,32)=  3.7264561653e-01
       coeff_l3(39,33)=  5.3039616346e-01
       coeff_l3(39,34)=  6.2988126278e-01
       coeff_l3(39,35)= -4.5225271583e-01
       coeff_l3(39,36)= -5.5228941143e-02
       coeff_l3(39,37)= -1.4531439543e-01
       coeff_l3(39,38)= -2.8442945480e+00
       coeff_l3(39,39)= -7.2230613232e-01
       coeff_l3(39,40)=  5.0465714931e-01
       coeff_l3(39,41)= -5.3507244587e-01
       coeff_l3(39,42)=  2.1101012826e-02
       coeff_l3(39,43)=  7.3989009857e-01
       coeff_l3(39,44)= -7.8098040819e-01
       coeff_l3(39,45)=  6.8343919516e-01
       coeff_l3(39,46)= -3.8771224022e-01
       coeff_l3(39,47)=  2.1807430685e-01
       coeff_l3(39,48)= -2.2609822750e+00
       coeff_l3(39,49)=  6.5258026123e-01
       coeff_l3(39,50)=  1.2631417811e-01
       coeff_l3(39,51)=  1.1518665552e+00
       coeff_l3(39,52)= -4.2500147223e-01
       coeff_l3(39,53)=  8.7320536375e-01
       coeff_l3(39,54)=  1.8899186850e+00
       coeff_l3(39,55)=  1.2783269584e-01
       coeff_l3(39,56)= -3.0913046002e-01
       coeff_l3(40,1)= -4.8538950086e-01
       coeff_l3(40,2)=  1.3598012924e-01
       coeff_l3(40,3)= -5.8656209707e-01
       coeff_l3(40,4)= -4.3777275085e-01
       coeff_l3(40,5)= -4.7837510705e-01
       coeff_l3(40,6)= -9.2364856973e-03
       coeff_l3(40,7)=  5.6574088335e-01
       coeff_l3(40,8)=  6.2785506248e-02
       coeff_l3(40,9)= -8.1746065617e-01
       coeff_l3(40,10)= -1.9104984403e-01
       coeff_l3(40,11)= -5.9284228086e-01
       coeff_l3(40,12)=  6.7148223519e-02
       coeff_l3(40,13)= -3.2814717293e-01
       coeff_l3(40,14)= -2.3254880682e-02
       coeff_l3(40,15)=  3.1908649951e-02
       coeff_l3(40,16)= -3.1818494201e-01
       coeff_l3(40,17)=  2.8102824092e-01
       coeff_l3(40,18)=  6.8332985044e-02
       coeff_l3(40,19)=  7.3718440533e-01
       coeff_l3(40,20)= -2.2887866199e-01
       coeff_l3(40,21)=  1.3617950678e+00
       coeff_l3(40,22)=  1.6419854760e-01
       coeff_l3(40,23)= -1.3866020739e-01
       coeff_l3(40,24)=  4.5717456937e-01
       coeff_l3(40,25)=  4.5273866653e+00
       coeff_l3(40,26)= -1.8802990019e-01
       coeff_l3(40,27)=  2.6732632518e-01
       coeff_l3(40,28)= -1.0412807465e+00
       coeff_l3(40,29)= -7.6487529278e-01
       coeff_l3(40,30)= -3.4825321287e-02
       coeff_l3(40,31)= -7.8483885527e-01
       coeff_l3(40,32)= -2.5325390697e-01
       coeff_l3(40,33)= -4.7249577940e-02
       coeff_l3(40,34)= -7.1863096952e-01
       coeff_l3(40,35)= -3.3601298928e-01
       coeff_l3(40,36)= -6.4643269777e-01
       coeff_l3(40,37)= -5.2993184328e-01
       coeff_l3(40,38)=  4.7484317422e-01
       coeff_l3(40,39)=  3.0517017469e-02
       coeff_l3(40,40)= -2.3500195146e-01
       coeff_l3(40,41)= -7.1099418402e-01
       coeff_l3(40,42)= -1.2537631989e+00
       coeff_l3(40,43)=  1.5634125471e-01
       coeff_l3(40,44)=  4.5773915946e-02
       coeff_l3(40,45)= -2.7886456251e-01
       coeff_l3(40,46)=  3.6092537642e-01
       coeff_l3(40,47)= -1.1062580161e-02
       coeff_l3(40,48)= -1.1248036623e+00
       coeff_l3(40,49)= -2.7695557475e-01
       coeff_l3(40,50)= -4.5430576801e-01
       coeff_l3(40,51)=  2.4882290363e+00
       coeff_l3(40,52)= -4.1896235943e-01
       coeff_l3(40,53)=  9.1648423672e-01
       coeff_l3(40,54)=  1.5579141676e-02
       coeff_l3(40,55)= -1.3727955520e-01
       coeff_l3(40,56)= -8.1541073322e-01
       coeff_l3(41,1)= -2.1688000858e-01
       coeff_l3(41,2)= -5.0599569082e-01
       coeff_l3(41,3)= -4.0129099041e-02
       coeff_l3(41,4)=  1.5743280947e-01
       coeff_l3(41,5)= -6.6959649324e-02
       coeff_l3(41,6)=  3.9470821619e-02
       coeff_l3(41,7)= -6.2630981207e-02
       coeff_l3(41,8)= -1.9217169285e-01
       coeff_l3(41,9)= -2.5373309851e-01
       coeff_l3(41,10)= -2.3096358776e-01
       coeff_l3(41,11)= -1.9370709658e+00
       coeff_l3(41,12)=  3.5117283463e-01
       coeff_l3(41,13)= -3.6507710814e-02
       coeff_l3(41,14)=  2.0814150572e-01
       coeff_l3(41,15)= -1.9939897954e-01
       coeff_l3(41,16)=  4.4855721295e-02
       coeff_l3(41,17)=  6.1260432005e-02
       coeff_l3(41,18)=  5.7865209877e-02
       coeff_l3(41,19)= -1.3096222878e+00
       coeff_l3(41,20)=  5.8161150664e-02
       coeff_l3(41,21)= -2.0184329152e-01
       coeff_l3(41,22)= -1.4053410292e-01
       coeff_l3(41,23)= -6.3694441319e-01
       coeff_l3(41,24)= -4.5758119226e-01
       coeff_l3(41,25)= -5.6076536179e+00
       coeff_l3(41,26)=  1.0039985180e+00
       coeff_l3(41,27)=  3.7497088779e-03
       coeff_l3(41,28)=  4.6475085616e-01
       coeff_l3(41,29)=  8.9258588850e-02
       coeff_l3(41,30)=  4.8865865916e-02
       coeff_l3(41,31)= -1.7866252363e-01
       coeff_l3(41,32)= -1.3231620193e-01
       coeff_l3(41,33)= -5.3525978327e-01
       coeff_l3(41,34)= -5.4573225975e-01
       coeff_l3(41,35)=  1.7113214731e-01
       coeff_l3(41,36)=  2.5362470746e-01
       coeff_l3(41,37)= -1.6946339607e-01
       coeff_l3(41,38)= -2.5839152932e-01
       coeff_l3(41,39)= -8.5476778448e-02
       coeff_l3(41,40)=  6.9898068905e-03
       coeff_l3(41,41)= -1.1762083769e+00
       coeff_l3(41,42)=  1.3295334578e-01
       coeff_l3(41,43)= -2.4927501380e-01
       coeff_l3(41,44)=  5.3206318617e-01
       coeff_l3(41,45)=  2.7846276760e+00
       coeff_l3(41,46)= -5.8232732117e-03
       coeff_l3(41,47)=  6.0969211161e-02
       coeff_l3(41,48)=  4.1115097702e-02
       coeff_l3(41,49)= -5.5292057991e-01
       coeff_l3(41,50)=  1.1862974614e-01
       coeff_l3(41,51)= -1.5026973486e+00
       coeff_l3(41,52)=  1.2096158415e-01
       coeff_l3(41,53)= -3.2329535484e-01
       coeff_l3(41,54)=  1.3582119942e+00
       coeff_l3(41,55)=  4.5741659403e-01
       coeff_l3(41,56)=  9.0138483047e-01
       coeff_l3(42,1)= -6.2734138966e-01
       coeff_l3(42,2)=  4.3435248733e-01
       coeff_l3(42,3)= -2.3462149501e-01
       coeff_l3(42,4)=  4.2478168011e-01
       coeff_l3(42,5)= -7.3590874672e-01
       coeff_l3(42,6)= -2.1521008015e-01
       coeff_l3(42,7)= -8.3684116602e-01
       coeff_l3(42,8)= -1.3144698925e-02
       coeff_l3(42,9)= -1.9362936914e-01
       coeff_l3(42,10)=  3.8568085432e-01
       coeff_l3(42,11)= -8.0810016394e-01
       coeff_l3(42,12)= -1.4223507643e+00
       coeff_l3(42,13)=  6.0668802261e-01
       coeff_l3(42,14)= -9.1240328550e-01
       coeff_l3(42,15)=  7.4141561985e-01
       coeff_l3(42,16)=  9.1542202234e-01
       coeff_l3(42,17)= -1.0291770846e-01
       coeff_l3(42,18)=  7.9182541370e-01
       coeff_l3(42,19)=  8.7306171656e-01
       coeff_l3(42,20)= -4.5644182712e-02
       coeff_l3(42,21)=  1.3114345074e-01
       coeff_l3(42,22)=  1.0807659626e+00
       coeff_l3(42,23)=  1.5141229145e-02
       coeff_l3(42,24)=  1.5512028933e+00
       coeff_l3(42,25)=  1.1916078329e+00
       coeff_l3(42,26)= -1.3064324856e-01
       coeff_l3(42,27)=  8.9908379316e-01
       coeff_l3(42,28)= -2.4723121524e-01
       coeff_l3(42,29)= -4.5449948311e-01
       coeff_l3(42,30)=  1.1001460254e-01
       coeff_l3(42,31)= -8.6890244484e-01
       coeff_l3(42,32)=  2.3541228473e-01
       coeff_l3(42,33)=  1.4328093529e+00
       coeff_l3(42,34)=  1.9579383731e-01
       coeff_l3(42,35)=  7.5823175907e-01
       coeff_l3(42,36)= -2.9759582877e-01
       coeff_l3(42,37)= -9.6768569201e-03
       coeff_l3(42,38)=  3.1819066405e-01
       coeff_l3(42,39)= -1.5415510535e-01
       coeff_l3(42,40)=  3.6969739199e-01
       coeff_l3(42,41)=  7.6340466738e-02
       coeff_l3(42,42)=  6.8196064234e-01
       coeff_l3(42,43)=  7.3617690802e-01
       coeff_l3(42,44)= -2.0511502028e-01
       coeff_l3(42,45)=  7.7786274254e-02
       coeff_l3(42,46)=  1.5249228477e-01
       coeff_l3(42,47)= -6.2253005803e-02
       coeff_l3(42,48)=  1.5151602030e+00
       coeff_l3(42,49)=  3.1567803025e-01
       coeff_l3(42,50)=  8.6468321085e-01
       coeff_l3(42,51)=  5.9055936337e-01
       coeff_l3(42,52)=  1.3416124880e-01
       coeff_l3(42,53)= -4.5185902715e-01
       coeff_l3(42,54)= -6.3021981716e-01
       coeff_l3(42,55)=  6.7843005061e-02
       coeff_l3(42,56)=  1.8537817895e-01
       coeff_l3(43,1)= -2.9448738694e-01
       coeff_l3(43,2)=  5.0569218397e-01
       coeff_l3(43,3)=  1.4651042223e-01
       coeff_l3(43,4)= -1.5983548760e-01
       coeff_l3(43,5)= -1.3021236658e+00
       coeff_l3(43,6)=  1.0158359259e-01
       coeff_l3(43,7)= -6.3505196571e-01
       coeff_l3(43,8)= -8.7474721670e-01
       coeff_l3(43,9)=  9.2794895172e-02
       coeff_l3(43,10)=  3.6599910259e-01
       coeff_l3(43,11)=  1.0078454763e-01
       coeff_l3(43,12)= -5.0401705503e-01
       coeff_l3(43,13)= -5.4007023573e-01
       coeff_l3(43,14)= -5.8444142342e-01
       coeff_l3(43,15)= -3.7181860209e-01
       coeff_l3(43,16)= -1.1850095540e-01
       coeff_l3(43,17)= -7.9583466053e-01
       coeff_l3(43,18)= -4.7450050712e-02
       coeff_l3(43,19)=  2.3071165383e-01
       coeff_l3(43,20)=  1.3594861031e+00
       coeff_l3(43,21)=  9.5522332191e-01
       coeff_l3(43,22)=  8.6158710718e-01
       coeff_l3(43,23)=  1.4657737315e-01
       coeff_l3(43,24)=  3.0255734921e-01
       coeff_l3(43,25)=  1.0861226320e+00
       coeff_l3(43,26)= -1.4439827204e-01
       coeff_l3(43,27)=  9.9625116587e-01
       coeff_l3(43,28)= -7.7582645416e-01
       coeff_l3(43,29)=  1.3059555292e+00
       coeff_l3(43,30)= -7.9753053188e-01
       coeff_l3(43,31)= -6.3570410013e-01
       coeff_l3(43,32)= -2.7306428552e-01
       coeff_l3(43,33)=  3.7844303250e-01
       coeff_l3(43,34)=  1.3321815431e-01
       coeff_l3(43,35)= -5.0337100029e-01
       coeff_l3(43,36)= -2.7116385102e-01
       coeff_l3(43,37)= -1.7400091887e-01
       coeff_l3(43,38)=  5.6205195189e-01
       coeff_l3(43,39)= -5.5865442753e-01
       coeff_l3(43,40)=  5.5045980215e-01
       coeff_l3(43,41)=  1.5422436595e-01
       coeff_l3(43,42)=  5.6430071592e-01
       coeff_l3(43,43)= -2.7922704816e-01
       coeff_l3(43,44)=  1.8437774479e-01
       coeff_l3(43,45)=  3.4800502658e-01
       coeff_l3(43,46)= -1.1448499560e-01
       coeff_l3(43,47)=  1.4001391828e-02
       coeff_l3(43,48)=  2.3395396769e-01
       coeff_l3(43,49)=  2.0487883594e-03
       coeff_l3(43,50)=  5.6080895662e-01
       coeff_l3(43,51)=  4.9794062972e-01
       coeff_l3(43,52)= -2.1114934981e-01
       coeff_l3(43,53)= -6.7410331964e-01
       coeff_l3(43,54)= -2.4680912495e-01
       coeff_l3(43,55)=  1.5487901866e-01
       coeff_l3(43,56)=  1.9215711951e-01
       coeff_l3(44,1)= -9.8714101315e-01
       coeff_l3(44,2)= -3.3409065008e-01
       coeff_l3(44,3)= -4.7872728109e-01
       coeff_l3(44,4)=  1.7650525272e-01
       coeff_l3(44,5)= -4.9669462442e-01
       coeff_l3(44,6)=  3.0954124406e-02
       coeff_l3(44,7)=  1.6661152244e-01
       coeff_l3(44,8)= -1.8129250407e-01
       coeff_l3(44,9)= -7.3134419508e-03
       coeff_l3(44,10)= -9.7874432802e-02
       coeff_l3(44,11)=  2.1488957107e-01
       coeff_l3(44,12)= -9.1731131077e-01
       coeff_l3(44,13)= -1.7403444052e+00
       coeff_l3(44,14)=  3.5870385170e-01
       coeff_l3(44,15)= -7.6550060511e-01
       coeff_l3(44,16)=  3.3451698255e-04
       coeff_l3(44,17)= -3.6594763398e-01
       coeff_l3(44,18)= -3.1745779514e-01
       coeff_l3(44,19)= -1.4131973684e-01
       coeff_l3(44,20)= -2.4950788915e-01
       coeff_l3(44,21)=  3.0388873816e-01
       coeff_l3(44,22)=  9.2502264306e-03
       coeff_l3(44,23)=  4.3831011653e-01
       coeff_l3(44,24)= -6.8984799087e-02
       coeff_l3(44,25)=  1.7878985405e-01
       coeff_l3(44,26)=  3.4252354503e-01
       coeff_l3(44,27)=  3.3412918448e-01
       coeff_l3(44,28)= -1.6252517700e-01
       coeff_l3(44,29)=  2.3716008663e-01
       coeff_l3(44,30)=  5.9664624929e-01
       coeff_l3(44,31)= -8.5425912403e-04
       coeff_l3(44,32)= -4.0653032064e-01
       coeff_l3(44,33)= -1.0165961832e-01
       coeff_l3(44,34)= -3.2306984067e-01
       coeff_l3(44,35)= -4.2855477333e-01
       coeff_l3(44,36)=  4.5215022564e-01
       coeff_l3(44,37)=  4.9758696556e-01
       coeff_l3(44,38)=  3.4864613414e-01
       coeff_l3(44,39)= -3.4308731556e-01
       coeff_l3(44,40)= -6.3993752003e-01
       coeff_l3(44,41)=  2.3578980565e-01
       coeff_l3(44,42)= -4.3069717288e-01
       coeff_l3(44,43)= -1.0560425930e-02
       coeff_l3(44,44)=  2.4787813425e-01
       coeff_l3(44,45)= -6.0740500689e-01
       coeff_l3(44,46)=  5.5063301325e-01
       coeff_l3(44,47)= -2.1745404601e-01
       coeff_l3(44,48)=  1.6420522332e-01
       coeff_l3(44,49)=  1.8302120268e-01
       coeff_l3(44,50)= -4.0819266438e-01
       coeff_l3(44,51)=  2.9704034328e-02
       coeff_l3(44,52)= -9.4579917192e-01
       coeff_l3(44,53)=  2.4084797502e-01
       coeff_l3(44,54)= -2.1456110477e+00
       coeff_l3(44,55)= -1.7687967420e-01
       coeff_l3(44,56)= -4.0837529302e-01
       coeff_l3(45,1)= -1.0334589481e+00
       coeff_l3(45,2)= -9.9081918597e-02
       coeff_l3(45,3)=  3.6939916015e-01
       coeff_l3(45,4)= -7.4178203940e-02
       coeff_l3(45,5)= -2.1798928082e-01
       coeff_l3(45,6)= -9.4139540195e-01
       coeff_l3(45,7)= -1.0852559805e+00
       coeff_l3(45,8)= -6.1336028576e-01
       coeff_l3(45,9)= -8.3500778675e-01
       coeff_l3(45,10)=  2.4711513519e-01
       coeff_l3(45,11)= -1.4405825138e+00
       coeff_l3(45,12)=  9.3084603548e-01
       coeff_l3(45,13)= -7.6444365084e-02
       coeff_l3(45,14)= -1.1725319624e+00
       coeff_l3(45,15)= -1.0776160955e+00
       coeff_l3(45,16)= -1.3321335614e-01
       coeff_l3(45,17)=  6.4827543497e-01
       coeff_l3(45,18)= -1.7158795893e-01
       coeff_l3(45,19)= -8.4508132935e-01
       coeff_l3(45,20)= -1.2183409184e-02
       coeff_l3(45,21)=  2.1919222176e-01
       coeff_l3(45,22)= -2.4301709235e-01
       coeff_l3(45,23)= -9.5705729723e-01
       coeff_l3(45,24)=  4.2847174406e-01
       coeff_l3(45,25)= -1.2284673452e+00
       coeff_l3(45,26)= -6.1863464117e-01
       coeff_l3(45,27)= -1.6093202829e+00
       coeff_l3(45,28)= -1.2895943224e-01
       coeff_l3(45,29)= -2.2957672179e-01
       coeff_l3(45,30)=  1.7521873116e-01
       coeff_l3(45,31)= -1.6165664196e+00
       coeff_l3(45,32)=  3.2179978490e-01
       coeff_l3(45,33)=  2.5504264981e-02
       coeff_l3(45,34)=  4.0162053704e-01
       coeff_l3(45,35)=  4.1538542509e-01
       coeff_l3(45,36)= -3.7927427888e-01
       coeff_l3(45,37)= -6.6497504711e-01
       coeff_l3(45,38)= -7.4568040669e-02
       coeff_l3(45,39)= -5.9238404036e-02
       coeff_l3(45,40)= -9.7694855928e-01
       coeff_l3(45,41)= -9.4226527214e-01
       coeff_l3(45,42)= -8.6599504948e-01
       coeff_l3(45,43)= -1.0756136179e+00
       coeff_l3(45,44)= -7.6577520370e-01
       coeff_l3(45,45)=  2.8106093407e+00
       coeff_l3(45,46)= -7.5132423639e-01
       coeff_l3(45,47)=  6.0357892513e-01
       coeff_l3(45,48)= -4.1757032275e-01
       coeff_l3(45,49)=  1.0424192995e-02
       coeff_l3(45,50)=  8.3359420300e-01
       coeff_l3(45,51)= -1.3032926321e+00
       coeff_l3(45,52)=  1.1036657095e+00
       coeff_l3(45,53)= -1.1114037037e+00
       coeff_l3(45,54)=  1.7850722075e+00
       coeff_l3(45,55)=  1.6669027507e-01
       coeff_l3(45,56)=  1.5659943819e+00
       coeff_l3(46,1)=  6.0290843248e-01
       coeff_l3(46,2)=  8.0122685432e-01
       coeff_l3(46,3)= -2.0994208753e-01
       coeff_l3(46,4)=  2.1576751024e-02
       coeff_l3(46,5)= -2.5660168380e-02
       coeff_l3(46,6)=  1.3924373388e+00
       coeff_l3(46,7)= -2.0035903156e-01
       coeff_l3(46,8)= -4.4900199771e-01
       coeff_l3(46,9)= -9.7634768486e-01
       coeff_l3(46,10)=  3.6977559328e-01
       coeff_l3(46,11)= -3.0095963478e+00
       coeff_l3(46,12)=  1.1368350983e+00
       coeff_l3(46,13)=  4.3019391596e-02
       coeff_l3(46,14)= -2.3451011181e+00
       coeff_l3(46,15)= -2.2404952347e-01
       coeff_l3(46,16)= -6.0472041368e-01
       coeff_l3(46,17)=  4.1746076941e-01
       coeff_l3(46,18)= -8.4940660000e-01
       coeff_l3(46,19)= -1.0182708502e+00
       coeff_l3(46,20)= -6.5544348955e-01
       coeff_l3(46,21)=  1.2468147278e+00
       coeff_l3(46,22)=  2.4297347069e+00
       coeff_l3(46,23)= -1.6856386662e+00
       coeff_l3(46,24)=  1.7404261827e+00
       coeff_l3(46,25)= -1.8668385744e+00
       coeff_l3(46,26)= -5.9062104225e+00
       coeff_l3(46,27)=  6.2125456333e-01
       coeff_l3(46,28)=  8.7805831432e-01
       coeff_l3(46,29)=  3.0120294094e+00
       coeff_l3(46,30)= -4.6183580160e-01
       coeff_l3(46,31)= -4.1273803711e+00
       coeff_l3(46,32)= -1.0575375557e+00
       coeff_l3(46,33)= -5.7644009590e-02
       coeff_l3(46,34)=  1.6139937937e-01
       coeff_l3(46,35)=  3.2921913266e-01
       coeff_l3(46,36)=  2.9683317989e-02
       coeff_l3(46,37)= -4.1381040215e-01
       coeff_l3(46,38)=  2.6381126046e-01
       coeff_l3(46,39)=  1.8741975725e-01
       coeff_l3(46,40)= -4.7991088033e-01
       coeff_l3(46,41)= -9.5388484001e-01
       coeff_l3(46,42)= -1.5614584088e-01
       coeff_l3(46,43)=  3.5235348344e-01
       coeff_l3(46,44)= -1.5136934519e+00
       coeff_l3(46,45)=  4.7759124637e-01
       coeff_l3(46,46)= -5.4390519857e-02
       coeff_l3(46,47)= -1.2465722114e-01
       coeff_l3(46,48)= -9.4585859776e-01
       coeff_l3(46,49)= -1.4862307310e+00
       coeff_l3(46,50)= -1.2411116362e+00
       coeff_l3(46,51)= -1.6332166195e+00
       coeff_l3(46,52)= -4.2514270544e-01
       coeff_l3(46,53)=  4.2318686843e-01
       coeff_l3(46,54)=  5.5931687355e-01
       coeff_l3(46,55)=  2.9985675216e-01
       coeff_l3(46,56)= -2.1548824385e-02
       coeff_l3(47,1)=  4.7503271699e-01
       coeff_l3(47,2)=  1.4512372017e+00
       coeff_l3(47,3)= -2.7629038692e-01
       coeff_l3(47,4)= -1.4319497347e-01
       coeff_l3(47,5)= -4.9592962861e-01
       coeff_l3(47,6)= -2.6413321495e-01
       coeff_l3(47,7)= -5.1569700241e-01
       coeff_l3(47,8)=  2.0991231501e-01
       coeff_l3(47,9)=  2.5817123055e-01
       coeff_l3(47,10)= -3.0282223225e-01
       coeff_l3(47,11)=  1.1989760399e+00
       coeff_l3(47,12)= -1.4325977564e+00
       coeff_l3(47,13)= -9.4509142637e-01
       coeff_l3(47,14)=  2.0454624295e-01
       coeff_l3(47,15)= -4.4543081522e-01
       coeff_l3(47,16)=  1.8987374008e-01
       coeff_l3(47,17)= -6.3739860058e-01
       coeff_l3(47,18)=  3.9606946707e-01
       coeff_l3(47,19)= -3.5499486327e-01
       coeff_l3(47,20)=  1.5762590170e+00
       coeff_l3(47,21)=  1.2911323309e+00
       coeff_l3(47,22)= -1.7286859453e-01
       coeff_l3(47,23)=  5.0855606794e-01
       coeff_l3(47,24)= -2.3344244063e-01
       coeff_l3(47,25)= -1.0365872830e-01
       coeff_l3(47,26)=  2.5490680337e-01
       coeff_l3(47,27)=  1.4184112549e+00
       coeff_l3(47,28)=  5.1475858688e-01
       coeff_l3(47,29)=  1.5168334544e-01
       coeff_l3(47,30)= -6.3826456666e-02
       coeff_l3(47,31)=  1.0415430069e+00
       coeff_l3(47,32)= -7.4279256165e-02
       coeff_l3(47,33)=  8.7224967778e-02
       coeff_l3(47,34)=  1.6070248187e-01
       coeff_l3(47,35)=  2.3970097303e-01
       coeff_l3(47,36)= -1.0192101002e+00
       coeff_l3(47,37)= -4.8278349638e-01
       coeff_l3(47,38)=  6.9295144081e-01
       coeff_l3(47,39)= -1.7620228231e-01
       coeff_l3(47,40)=  1.1252771318e-01
       coeff_l3(47,41)=  2.2102597356e-01
       coeff_l3(47,42)=  1.1057558656e-01
       coeff_l3(47,43)= -6.8232989311e-01
       coeff_l3(47,44)= -2.2570595145e-02
       coeff_l3(47,45)= -5.5586463213e-01
       coeff_l3(47,46)= -5.2480387688e-01
       coeff_l3(47,47)= -4.8275921494e-02
       coeff_l3(47,48)= -9.5856350660e-01
       coeff_l3(47,49)=  2.5547511876e-02
       coeff_l3(47,50)= -2.5253781676e-01
       coeff_l3(47,51)=  5.9070503712e-01
       coeff_l3(47,52)=  4.5726418495e-01
       coeff_l3(47,53)= -3.4749162197e-01
       coeff_l3(47,54)= -1.6211621761e+00
       coeff_l3(47,55)= -8.5902965069e-01
       coeff_l3(47,56)=  1.2056823075e-01
       coeff_l3(48,1)= -3.5617396235e-02
       coeff_l3(48,2)=  1.9944077730e-01
       coeff_l3(48,3)= -9.0976208448e-02
       coeff_l3(48,4)= -1.4701026678e-01
       coeff_l3(48,5)= -1.1355662346e+00
       coeff_l3(48,6)= -3.4125312231e-03
       coeff_l3(48,7)=  5.8485586196e-02
       coeff_l3(48,8)=  1.7204605043e-01
       coeff_l3(48,9)= -3.4634623677e-02
       coeff_l3(48,10)= -7.3014102876e-02
       coeff_l3(48,11)=  4.9834847450e-01
       coeff_l3(48,12)= -1.3753689528e+00
       coeff_l3(48,13)= -1.4629777670e+00
       coeff_l3(48,14)=  3.5335806012e-01
       coeff_l3(48,15)=  3.4514498711e-01
       coeff_l3(48,16)=  3.3390876651e-01
       coeff_l3(48,17)= -1.9136010110e-01
       coeff_l3(48,18)= -3.9594620466e-01
       coeff_l3(48,19)=  1.6472880542e-01
       coeff_l3(48,20)= -1.0893958807e-01
       coeff_l3(48,21)=  9.2864620686e-01
       coeff_l3(48,22)=  1.8328306079e-01
       coeff_l3(48,23)=  1.2774467468e-01
       coeff_l3(48,24)= -7.8177386522e-01
       coeff_l3(48,25)=  2.0960113406e-01
       coeff_l3(48,26)= -5.4797548056e-01
       coeff_l3(48,27)=  6.0252511501e-01
       coeff_l3(48,28)=  2.5878580287e-02
       coeff_l3(48,29)= -1.0576598644e+00
       coeff_l3(48,30)= -6.2559568882e-01
       coeff_l3(48,31)= -8.0264687538e-02
       coeff_l3(48,32)= -5.0270900130e-02
       coeff_l3(48,33)=  2.2127765417e-01
       coeff_l3(48,34)= -1.6218146682e-01
       coeff_l3(48,35)= -5.7106119394e-01
       coeff_l3(48,36)=  2.5105229020e-01
       coeff_l3(48,37)= -6.6647498170e-04
       coeff_l3(48,38)=  6.6494446993e-01
       coeff_l3(48,39)=  1.0567295551e-01
       coeff_l3(48,40)=  5.1980000734e-01
       coeff_l3(48,41)=  5.2821564674e-01
       coeff_l3(48,42)= -5.3713399172e-01
       coeff_l3(48,43)= -9.6501088142e-01
       coeff_l3(48,44)=  2.6879921556e-01
       coeff_l3(48,45)= -1.1643103361e+00
       coeff_l3(48,46)= -1.0356029868e-01
       coeff_l3(48,47)= -1.1296701431e+00
       coeff_l3(48,48)= -1.0815284401e-01
       coeff_l3(48,49)=  2.7786725760e-01
       coeff_l3(48,50)= -3.0672803521e-01
       coeff_l3(48,51)= -3.7851384282e-01
       coeff_l3(48,52)= -4.0321221948e-01
       coeff_l3(48,53)= -1.9842934608e-01
       coeff_l3(48,54)= -1.4210031033e+00
       coeff_l3(48,55)= -3.4104076028e-01
       coeff_l3(48,56)= -2.1983130276e-01
       coeff_l3(49,1)= -6.7644113302e-01
       coeff_l3(49,2)=  3.6847490072e-01
       coeff_l3(49,3)=  5.6606046855e-03
       coeff_l3(49,4)= -4.5845266432e-02
       coeff_l3(49,5)= -2.5041949749e-01
       coeff_l3(49,6)=  2.6189342141e-01
       coeff_l3(49,7)= -1.9262826443e-01
       coeff_l3(49,8)= -4.9061608315e-01
       coeff_l3(49,9)= -3.3768609166e-01
       coeff_l3(49,10)=  6.2431409955e-02
       coeff_l3(49,11)=  1.4186786413e+00
       coeff_l3(49,12)= -1.0501399040e+00
       coeff_l3(49,13)= -1.9742484093e+00
       coeff_l3(49,14)=  5.0491690636e-01
       coeff_l3(49,15)= -1.1835446358e+00
       coeff_l3(49,16)=  4.9715366215e-02
       coeff_l3(49,17)= -1.0495370626e+00
       coeff_l3(49,18)= -7.6492583752e-01
       coeff_l3(49,19)= -1.0360301733e+00
       coeff_l3(49,20)=  6.5685063601e-02
       coeff_l3(49,21)=  5.4624862969e-02
       coeff_l3(49,22)=  4.2198160291e-01
       coeff_l3(49,23)=  3.1152403355e-01
       coeff_l3(49,24)= -8.2127308846e-01
       coeff_l3(49,25)=  2.2297217846e+00
       coeff_l3(49,26)= -7.2166281939e-01
       coeff_l3(49,27)=  1.4817321673e-02
       coeff_l3(49,28)= -2.4359293282e-01
       coeff_l3(49,29)= -7.3566550016e-01
       coeff_l3(49,30)=  1.7851916552e+00
       coeff_l3(49,31)=  5.6352907419e-01
       coeff_l3(49,32)= -1.0187550783e+00
       coeff_l3(49,33)=  1.2105674297e-01
       coeff_l3(49,34)=  1.1120423675e-01
       coeff_l3(49,35)= -1.4133803546e-01
       coeff_l3(49,36)= -3.4463602304e-01
       coeff_l3(49,37)=  1.7879976332e-01
       coeff_l3(49,38)=  1.1615939140e+00
       coeff_l3(49,39)=  3.0729541183e-01
       coeff_l3(49,40)= -2.1521382034e-01
       coeff_l3(49,41)=  4.9018734694e-01
       coeff_l3(49,42)= -3.1988494396e+00
       coeff_l3(49,43)= -1.3823016882e+00
       coeff_l3(49,44)=  2.8794127703e-01
       coeff_l3(49,45)=  9.0344041586e-01
       coeff_l3(49,46)=  4.6095713973e-01
       coeff_l3(49,47)=  1.7912784591e-02
       coeff_l3(49,48)=  6.7191559076e-01
       coeff_l3(49,49)=  4.8469252884e-02
       coeff_l3(49,50)=  1.1142252684e+00
       coeff_l3(49,51)= -1.7613729239e+00
       coeff_l3(49,52)= -6.9110369682e-01
       coeff_l3(49,53)= -1.7366757393e+00
       coeff_l3(49,54)= -1.4508004189e+00
       coeff_l3(49,55)= -8.3386182785e-02
       coeff_l3(49,56)=  7.2288829088e-01
       coeff_l3(50,1)= -4.6267190576e-01
       coeff_l3(50,2)=  4.9273647368e-02
       coeff_l3(50,3)= -1.1103795767e+00
       coeff_l3(50,4)=  3.1494238973e-01
       coeff_l3(50,5)= -2.3109102622e-02
       coeff_l3(50,6)= -6.3881754875e-02
       coeff_l3(50,7)=  9.3523941934e-02
       coeff_l3(50,8)= -5.5591928959e-01
       coeff_l3(50,9)=  2.3220443726e-01
       coeff_l3(50,10)= -3.6462196708e-01
       coeff_l3(50,11)= -5.6916117668e-01
       coeff_l3(50,12)= -7.1749174595e-01
       coeff_l3(50,13)= -4.6186279505e-03
       coeff_l3(50,14)= -2.3640580177e+00
       coeff_l3(50,15)= -5.0619512796e-01
       coeff_l3(50,16)= -4.4127050042e-01
       coeff_l3(50,17)=  1.1450735480e-01
       coeff_l3(50,18)= -1.7450900376e-01
       coeff_l3(50,19)= -1.3296562433e-01
       coeff_l3(50,20)=  2.4664956331e-01
       coeff_l3(50,21)=  2.5071904063e-01
       coeff_l3(50,22)=  6.8214607239e-01
       coeff_l3(50,23)= -1.1915687323e+00
       coeff_l3(50,24)=  6.6562443972e-02
       coeff_l3(50,25)=  5.4491358995e-01
       coeff_l3(50,26)= -4.8426852226e+00
       coeff_l3(50,27)=  1.2058757991e-01
       coeff_l3(50,28)=  7.1850471199e-02
       coeff_l3(50,29)= -2.3683423996e+00
       coeff_l3(50,30)= -4.7351937741e-02
       coeff_l3(50,31)= -2.0970958471e-01
       coeff_l3(50,32)= -1.0768014193e-01
       coeff_l3(50,33)= -1.2310975045e-01
       coeff_l3(50,34)= -2.6908001304e-01
       coeff_l3(50,35)=  1.0776096582e-01
       coeff_l3(50,36)= -3.3703975379e-02
       coeff_l3(50,37)=  3.0697247386e-01
       coeff_l3(50,38)= -4.0685763359e+00
       coeff_l3(50,39)= -3.6444374919e-01
       coeff_l3(50,40)= -2.3344215751e-01
       coeff_l3(50,41)=  3.9872296154e-02
       coeff_l3(50,42)= -5.2127707750e-02
       coeff_l3(50,43)= -1.6076475382e+00
       coeff_l3(50,44)= -7.4435019493e-01
       coeff_l3(50,45)= -1.1613820791e+00
       coeff_l3(50,46)= -6.2344919890e-02
       coeff_l3(50,47)= -9.9475216866e-01
       coeff_l3(50,48)=  5.2031163126e-02
       coeff_l3(50,49)= -1.3280366659e+00
       coeff_l3(50,50)= -2.2489377856e-01
       coeff_l3(50,51)=  2.8961780667e-01
       coeff_l3(50,52)= -5.5758774281e-01
       coeff_l3(50,53)=  3.4535542130e-02
       coeff_l3(50,54)= -2.1627388000e+00
       coeff_l3(50,55)= -3.3622834086e-01
       coeff_l3(50,56)= -8.4456020594e-01
       coeff_l3(51,1)= -2.4650371075e-01
       coeff_l3(51,2)=  1.6434444487e-01
       coeff_l3(51,3)=  1.6329666972e-01
       coeff_l3(51,4)= -3.0303308368e-01
       coeff_l3(51,5)=  1.0658841580e-02
       coeff_l3(51,6)= -5.8273203671e-02
       coeff_l3(51,7)=  9.0743945912e-03
       coeff_l3(51,8)= -9.8367571831e-02
       coeff_l3(51,9)= -2.2789998353e-01
       coeff_l3(51,10)= -4.2921599001e-02
       coeff_l3(51,11)= -8.2734709978e-01
       coeff_l3(51,12)= -4.3974310160e-01
       coeff_l3(51,13)= -7.9424992204e-02
       coeff_l3(51,14)= -1.0665552616e+00
       coeff_l3(51,15)= -7.2944380343e-02
       coeff_l3(51,16)=  4.1007405519e-01
       coeff_l3(51,17)= -7.9966641963e-02
       coeff_l3(51,18)= -1.8066653982e-02
       coeff_l3(51,19)= -7.3410344124e-01
       coeff_l3(51,20)=  2.4965601042e-02
       coeff_l3(51,21)= -1.6988455057e+00
       coeff_l3(51,22)= -9.3443769217e-01
       coeff_l3(51,23)=  2.1806703508e-01
       coeff_l3(51,24)=  5.8709162474e-01
       coeff_l3(51,25)= -1.3603067398e+00
       coeff_l3(51,26)= -6.2109333277e-01
       coeff_l3(51,27)=  9.0365158394e-03
       coeff_l3(51,28)=  1.8911384046e-01
       coeff_l3(51,29)= -9.5395773649e-02
       coeff_l3(51,30)=  6.3000760972e-02
       coeff_l3(51,31)= -1.8053685427e+00
       coeff_l3(51,32)=  9.8836921155e-02
       coeff_l3(51,33)= -6.2411284447e-01
       coeff_l3(51,34)=  4.5033700764e-02
       coeff_l3(51,35)= -1.7521946132e-01
       coeff_l3(51,36)=  2.5342291594e-01
       coeff_l3(51,37)=  4.7510638833e-02
       coeff_l3(51,38)= -5.5354565382e-01
       coeff_l3(51,39)= -1.0877233744e-01
       coeff_l3(51,40)= -8.7629728019e-02
       coeff_l3(51,41)=  6.4974337816e-02
       coeff_l3(51,42)=  7.9615898430e-02
       coeff_l3(51,43)=  6.6956982017e-02
       coeff_l3(51,44)=  3.5326030850e-01
       coeff_l3(51,45)= -3.0655846000e-01
       coeff_l3(51,46)= -5.0865225494e-02
       coeff_l3(51,47)=  5.0425034761e-01
       coeff_l3(51,48)=  5.0596004725e-01
       coeff_l3(51,49)=  7.3984235525e-02
       coeff_l3(51,50)=  2.7748948336e-01
       coeff_l3(51,51)= -9.5512098074e-01
       coeff_l3(51,52)=  1.4917001128e-01
       coeff_l3(51,53)=  1.1428916454e-01
       coeff_l3(51,54)=  2.6766784489e-02
       coeff_l3(51,55)= -4.9699869007e-02
       coeff_l3(51,56)=  4.5180818439e-01
       coeff_l3(52,1)=  7.2759169340e-01
       coeff_l3(52,2)=  3.8121944666e-01
       coeff_l3(52,3)= -2.6663360000e-01
       coeff_l3(52,4)=  7.5578653812e-01
       coeff_l3(52,5)=  6.7524850368e-01
       coeff_l3(52,6)=  5.3815513849e-01
       coeff_l3(52,7)=  2.7433341742e-01
       coeff_l3(52,8)=  1.1274113320e-02
       coeff_l3(52,9)= -4.2933991551e-01
       coeff_l3(52,10)= -7.5719170272e-02
       coeff_l3(52,11)=  3.3493820578e-02
       coeff_l3(52,12)=  7.7380284667e-02
       coeff_l3(52,13)= -1.9070807695e+00
       coeff_l3(52,14)=  2.1211215854e-01
       coeff_l3(52,15)= -1.5306429705e-03
       coeff_l3(52,16)= -2.2192832828e-01
       coeff_l3(52,17)= -8.7254393101e-01
       coeff_l3(52,18)= -2.1228823662e+00
       coeff_l3(52,19)= -2.9663360119e-01
       coeff_l3(52,20)=  7.5206834078e-01
       coeff_l3(52,21)=  8.2214838266e-01
       coeff_l3(52,22)=  4.4757595658e-01
       coeff_l3(52,23)=  3.2793527842e-01
       coeff_l3(52,24)= -1.2440251112e+00
       coeff_l3(52,25)=  3.8143536448e-01
       coeff_l3(52,26)=  2.5734180212e-01
       coeff_l3(52,27)=  5.8523714542e-01
       coeff_l3(52,28)= -1.0468181372e+00
       coeff_l3(52,29)= -2.5957667828e-01
       coeff_l3(52,30)=  4.5052659512e-01
       coeff_l3(52,31)=  2.9339230061e-01
       coeff_l3(52,32)=  2.3715235293e-01
       coeff_l3(52,33)= -1.2932269275e-01
       coeff_l3(52,34)= -3.5899049044e-01
       coeff_l3(52,35)= -5.4254525900e-01
       coeff_l3(52,36)= -3.4960547090e-01
       coeff_l3(52,37)= -2.1477672458e-01
       coeff_l3(52,38)=  8.6107969284e-01
       coeff_l3(52,39)=  1.1502467394e+00
       coeff_l3(52,40)= -2.2526006401e-01
       coeff_l3(52,41)= -1.2195528299e-01
       coeff_l3(52,42)= -4.5960119367e-01
       coeff_l3(52,43)= -6.6514229774e-01
       coeff_l3(52,44)=  2.8323909640e-01
       coeff_l3(52,45)= -7.9255634546e-01
       coeff_l3(52,46)=  4.8694267869e-01
       coeff_l3(52,47)= -8.4962880611e-01
       coeff_l3(52,48)=  4.1840501130e-02
       coeff_l3(52,49)= -2.2196686268e-01
       coeff_l3(52,50)= -1.0196573734e+00
       coeff_l3(52,51)= -2.1415480971e-01
       coeff_l3(52,52)= -4.4478264451e-01
       coeff_l3(52,53)= -1.0095445812e-01
       coeff_l3(52,54)= -1.5661818981e+00
       coeff_l3(52,55)= -2.9512563348e-01
       coeff_l3(52,56)=  7.7919811010e-02
       coeff_l3(53,1)= -1.4186517894e-01
       coeff_l3(53,2)= -7.3818850517e-01
       coeff_l3(53,3)=  2.4191856384e-01
       coeff_l3(53,4)= -1.7114605010e-01
       coeff_l3(53,5)= -2.8862875700e-01
       coeff_l3(53,6)= -1.9979371130e-01
       coeff_l3(53,7)=  2.3389559239e-02
       coeff_l3(53,8)=  1.9432094693e-01
       coeff_l3(53,9)= -3.6637571454e-01
       coeff_l3(53,10)= -2.1365056746e-03
       coeff_l3(53,11)= -3.2100379467e-01
       coeff_l3(53,12)= -6.5781897306e-01
       coeff_l3(53,13)= -1.7317850590e+00
       coeff_l3(53,14)= -4.1071072221e-01
       coeff_l3(53,15)= -1.0154930353e+00
       coeff_l3(53,16)= -2.1527811885e-02
       coeff_l3(53,17)=  3.1898084283e-01
       coeff_l3(53,18)= -3.9233067632e-01
       coeff_l3(53,19)= -6.2828111649e-01
       coeff_l3(53,20)= -2.6372939348e-01
       coeff_l3(53,21)= -2.7325019240e-01
       coeff_l3(53,22)=  2.9417103529e-01
       coeff_l3(53,23)= -1.3826544583e-01
       coeff_l3(53,24)= -2.2909528017e-01
       coeff_l3(53,25)= -6.1967190355e-02
       coeff_l3(53,26)= -1.0994569063e+00
       coeff_l3(53,27)= -1.9040885568e-01
       coeff_l3(53,28)=  2.4332797527e-01
       coeff_l3(53,29)=  6.5869623423e-01
       coeff_l3(53,30)= -5.3191244602e-01
       coeff_l3(53,31)= -1.8247669935e-01
       coeff_l3(53,32)=  6.2380295992e-01
       coeff_l3(53,33)=  1.2482618541e-01
       coeff_l3(53,34)=  7.9373367131e-02
       coeff_l3(53,35)=  3.3885624260e-02
       coeff_l3(53,36)=  3.0252116919e-01
       coeff_l3(53,37)=  1.3697040081e-01
       coeff_l3(53,38)=  2.0817328990e-01
       coeff_l3(53,39)=  2.2339503467e-01
       coeff_l3(53,40)= -4.3185636401e-01
       coeff_l3(53,41)=  3.3179637045e-02
       coeff_l3(53,42)= -9.9922192097e-01
       coeff_l3(53,43)=  1.0254399851e-02
       coeff_l3(53,44)=  7.6861523092e-02
       coeff_l3(53,45)= -2.0399884880e-01
       coeff_l3(53,46)=  7.6839197427e-03
       coeff_l3(53,47)=  3.5925763845e-01
       coeff_l3(53,48)=  7.1098434925e-01
       coeff_l3(53,49)= -1.9121713936e-01
       coeff_l3(53,50)=  8.5251998901e-01
       coeff_l3(53,51)= -9.6375453472e-01
       coeff_l3(53,52)=  5.1701432467e-01
       coeff_l3(53,53)= -1.6052178144e+00
       coeff_l3(53,54)=  5.0209504366e-01
       coeff_l3(53,55)=  3.3808496594e-01
       coeff_l3(53,56)=  1.8574793637e-01
       coeff_l3(54,1)= -1.3944025040e+00
       coeff_l3(54,2)=  3.8738080859e-01
       coeff_l3(54,3)= -1.7567981780e-01
       coeff_l3(54,4)= -3.5727661848e-01
       coeff_l3(54,5)= -6.1914819479e-01
       coeff_l3(54,6)= -1.4069835842e-01
       coeff_l3(54,7)= -1.2864309549e+00
       coeff_l3(54,8)=  2.6654878259e-01
       coeff_l3(54,9)= -6.1418157816e-01
       coeff_l3(54,10)=  2.8482779860e-01
       coeff_l3(54,11)= -2.0759572983e+00
       coeff_l3(54,12)= -1.4228587151e+00
       coeff_l3(54,13)=  1.8806114793e-01
       coeff_l3(54,14)= -1.4322685003e+00
       coeff_l3(54,15)=  1.2515964918e-02
       coeff_l3(54,16)=  1.5786342323e-02
       coeff_l3(54,17)= -1.2897016108e-01
       coeff_l3(54,18)= -4.1872087121e-01
       coeff_l3(54,19)=  6.6451442242e-01
       coeff_l3(54,20)= -1.8822643161e-01
       coeff_l3(54,21)=  1.5696475506e+00
       coeff_l3(54,22)=  6.7069971561e-01
       coeff_l3(54,23)= -2.6064028963e-02
       coeff_l3(54,24)=  2.3720409870e+00
       coeff_l3(54,25)=  2.2154183388e+00
       coeff_l3(54,26)=  2.4678574875e-02
       coeff_l3(54,27)=  9.3918925524e-01
       coeff_l3(54,28)= -3.9529788494e-01
       coeff_l3(54,29)= -1.3318854570e+00
       coeff_l3(54,30)= -5.3939074278e-01
       coeff_l3(54,31)= -1.5751050711e+00
       coeff_l3(54,32)= -9.1643637419e-01
       coeff_l3(54,33)=  9.4696462154e-01
       coeff_l3(54,34)=  7.8789329529e-01
       coeff_l3(54,35)=  5.1555026323e-02
       coeff_l3(54,36)=  1.5832886100e-01
       coeff_l3(54,37)= -6.0253751278e-01
       coeff_l3(54,38)= -9.2424541712e-02
       coeff_l3(54,39)=  7.7829159796e-02
       coeff_l3(54,40)= -2.3521009088e-01
       coeff_l3(54,41)= -9.0882048011e-02
       coeff_l3(54,42)= -3.3557918668e-01
       coeff_l3(54,43)=  1.2572866678e+00
       coeff_l3(54,44)= -4.9093735218e-01
       coeff_l3(54,45)=  5.9215538204e-02
       coeff_l3(54,46)= -4.0994626284e-01
       coeff_l3(54,47)=  1.1917322874e+00
       coeff_l3(54,48)=  2.3680897057e-01
       coeff_l3(54,49)= -7.6028123498e-02
       coeff_l3(54,50)= -5.4244494438e-01
       coeff_l3(54,51)=  4.6016898006e-02
       coeff_l3(54,52)=  1.8710768223e+00
       coeff_l3(54,53)=  9.7828370333e-01
       coeff_l3(54,54)= -1.4155236483e+00
       coeff_l3(54,55)= -2.6575511694e-01
       coeff_l3(54,56)= -2.2817337513e-01
       coeff_l3(55,1)= -3.9436089993e-01
       coeff_l3(55,2)=  2.0684306324e-01
       coeff_l3(55,3)=  2.8568932787e-02
       coeff_l3(55,4)=  7.7745515108e-01
       coeff_l3(55,5)= -2.3769973218e-01
       coeff_l3(55,6)=  8.2750272751e-01
       coeff_l3(55,7)=  1.2005491182e-02
       coeff_l3(55,8)=  1.5304239988e+00
       coeff_l3(55,9)=  6.1238247156e-01
       coeff_l3(55,10)=  1.3294647448e-02
       coeff_l3(55,11)=  9.6345371008e-01
       coeff_l3(55,12)= -3.0875527859e-01
       coeff_l3(55,13)= -4.3191501498e-01
       coeff_l3(55,14)=  2.1230423450e-01
       coeff_l3(55,15)=  7.4549609423e-01
       coeff_l3(55,16)=  6.4164787531e-01
       coeff_l3(55,17)= -3.8801684976e-01
       coeff_l3(55,18)=  3.7168139219e-01
       coeff_l3(55,19)= -1.3823609352e+00
       coeff_l3(55,20)= -3.3376762271e-01
       coeff_l3(55,21)= -3.2652008533e-01
       coeff_l3(55,22)=  8.7611711025e-01
       coeff_l3(55,23)=  2.1478812397e-01
       coeff_l3(55,24)=  1.8260532618e-01
       coeff_l3(55,25)=  3.7293756008e+00
       coeff_l3(55,26)=  1.2332901955e+00
       coeff_l3(55,27)= -2.5266087055e-01
       coeff_l3(55,28)= -5.5170720816e-01
       coeff_l3(55,29)= -6.0132527351e-01
       coeff_l3(55,30)=  5.6868255138e-01
       coeff_l3(55,31)=  7.1291947365e-01
       coeff_l3(55,32)= -2.2856180370e-01
       coeff_l3(55,33)= -1.8700760603e-01
       coeff_l3(55,34)= -6.1481320858e-01
       coeff_l3(55,35)=  1.2873640060e+00
       coeff_l3(55,36)=  6.9008588791e-01
       coeff_l3(55,37)=  4.7757393122e-01
       coeff_l3(55,38)=  1.2835651636e+00
       coeff_l3(55,39)=  8.7767076492e-01
       coeff_l3(55,40)=  4.1098853946e-01
       coeff_l3(55,41)=  7.6110261679e-01
       coeff_l3(55,42)= -2.2827954292e+00
       coeff_l3(55,43)= -4.7745436430e-01
       coeff_l3(55,44)=  2.6140087843e-01
       coeff_l3(55,45)= -8.1784325838e-01
       coeff_l3(55,46)=  3.6858239770e-01
       coeff_l3(55,47)= -1.0455406457e-01
       coeff_l3(55,48)=  5.8770966530e-01
       coeff_l3(55,49)= -2.4496731162e-01
       coeff_l3(55,50)= -1.0703635216e+00
       coeff_l3(55,51)=  1.8102092743e+00
       coeff_l3(55,52)= -1.3378129900e-01
       coeff_l3(55,53)=  1.1441905499e+00
       coeff_l3(55,54)= -1.8822649121e-01
       coeff_l3(55,55)= -5.9293168783e-01
       coeff_l3(55,56)=  2.4645558000e-01
       coeff_l3(56,1)=  1.7636337876e-01
       coeff_l3(56,2)=  4.7613555193e-01
       coeff_l3(56,3)= -3.8081891835e-02
       coeff_l3(56,4)= -9.4521868229e-01
       coeff_l3(56,5)= -2.6166203618e-01
       coeff_l3(56,6)= -1.1377401352e+00
       coeff_l3(56,7)=  6.1593508720e-01
       coeff_l3(56,8)= -3.9117100835e-01
       coeff_l3(56,9)=  4.8904311657e-01
       coeff_l3(56,10)=  2.4234656990e-01
       coeff_l3(56,11)=  1.1318591833e+00
       coeff_l3(56,12)= -6.9232445955e-01
       coeff_l3(56,13)=  1.3655391335e-01
       coeff_l3(56,14)=  1.7734782398e-01
       coeff_l3(56,15)=  6.7490421236e-02
       coeff_l3(56,16)=  3.4996384382e-01
       coeff_l3(56,17)= -6.2469506264e-01
       coeff_l3(56,18)=  1.5543413162e-01
       coeff_l3(56,19)=  4.2756798863e-01
       coeff_l3(56,20)=  1.1579662561e-01
       coeff_l3(56,21)=  9.7914600372e-01
       coeff_l3(56,22)=  1.6424782574e-01
       coeff_l3(56,23)= -5.2283480763e-02
       coeff_l3(56,24)=  1.5975536406e-01
       coeff_l3(56,25)=  6.4669680595e+00
       coeff_l3(56,26)= -3.8768991828e-01
       coeff_l3(56,27)= -2.2538393736e-01
       coeff_l3(56,28)= -2.7584233880e-01
       coeff_l3(56,29)= -1.7600039244e+00
       coeff_l3(56,30)=  5.2619643509e-02
       coeff_l3(56,31)=  2.8940561414e-01
       coeff_l3(56,32)= -5.6982451677e-01
       coeff_l3(56,33)=  2.5882318616e-01
       coeff_l3(56,34)= -5.2602037787e-02
       coeff_l3(56,35)= -2.8168392181e-01
       coeff_l3(56,36)= -2.4068856239e-01
       coeff_l3(56,37)= -1.9105280936e-01
       coeff_l3(56,38)=  5.7628595829e-01
       coeff_l3(56,39)=  2.9715567827e-01
       coeff_l3(56,40)=  1.3767802715e-01
       coeff_l3(56,41)=  7.9472142458e-01
       coeff_l3(56,42)= -7.5396424532e-01
       coeff_l3(56,43)=  5.4979201406e-02
       coeff_l3(56,44)=  8.3677008748e-02
       coeff_l3(56,45)= -1.4852005243e+00
       coeff_l3(56,46)=  4.6950814128e-01
       coeff_l3(56,47)=  1.2851166725e-01
       coeff_l3(56,48)= -7.1606296301e-01
       coeff_l3(56,49)=  1.1952488124e-01
       coeff_l3(56,50)= -2.3162299395e-01
       coeff_l3(56,51)=  2.0464901924e+00
       coeff_l3(56,52)= -5.3374007344e-02
       coeff_l3(56,53)=  3.8849362731e-01
       coeff_l3(56,54)= -5.2704232931e-01
       coeff_l3(56,55)= -7.1472167969e-02
       coeff_l3(56,56)= -4.8589304090e-01
!coeff layer 4
       coeff_l4(1,1)= -1.7349704504e+00
       coeff_l4(1,2)= -1.5955097675e+00
       coeff_l4(1,3)=  6.0204476118e-01
       coeff_l4(1,4)= -1.5637793541e+00
       coeff_l4(1,5)= -5.6599134207e-01
       coeff_l4(1,6)=  1.9348560274e-01
       coeff_l4(1,7)= -6.0866594315e-02
       coeff_l4(1,8)= -2.3405668139e-01
       coeff_l4(1,9)= -7.3042559624e-01
       coeff_l4(1,10)=  1.1824267916e-02
       coeff_l4(1,11)=  7.1254742146e-01
       coeff_l4(1,12)= -7.4062901735e-01
       coeff_l4(1,13)= -1.2897798792e-02
       coeff_l4(1,14)=  3.6971330643e-01
       coeff_l4(1,15)= -4.1274210811e-01
       coeff_l4(1,16)= -3.5945177078e-01
       coeff_l4(1,17)=  4.8596689105e-01
       coeff_l4(1,18)= -1.4418603480e-01
       coeff_l4(1,19)=  2.9711753130e-01
       coeff_l4(1,20)=  1.0169739723e+00
       coeff_l4(1,21)=  1.8959288299e-01
       coeff_l4(2,1)= -1.9345518351e+00
       coeff_l4(2,2)= -1.8272399902e+00
       coeff_l4(2,3)= -3.0981054902e-01
       coeff_l4(2,4)= -1.1786558628e+00
       coeff_l4(2,5)= -1.3299764395e+00
       coeff_l4(2,6)= -7.8626447916e-01
       coeff_l4(2,7)= -1.0194794834e-01
       coeff_l4(2,8)= -2.6904332638e-01
       coeff_l4(2,9)=  3.5105288029e-01
       coeff_l4(2,10)=  4.4594910741e-01
       coeff_l4(2,11)= -2.5285360217e-01
       coeff_l4(2,12)=  1.0545911789e+00
       coeff_l4(2,13)=  9.7209684551e-02
       coeff_l4(2,14)= -3.2051807642e-01
       coeff_l4(2,15)= -5.1694238186e-01
       coeff_l4(2,16)=  1.4447848499e-01
       coeff_l4(2,17)= -3.2866570354e-01
       coeff_l4(2,18)= -1.4678780735e-01
       coeff_l4(2,19)=  3.3065553755e-02
       coeff_l4(2,20)= -4.4006845355e-01
       coeff_l4(2,21)=  3.3425740898e-02
       coeff_l4(3,1)= -9.4376164675e-01
       coeff_l4(3,2)= -7.6659071445e-01
       coeff_l4(3,3)= -2.9542404413e-01
       coeff_l4(3,4)= -3.4530296922e-02
       coeff_l4(3,5)=  1.1779879779e-01
       coeff_l4(3,6)= -7.3098582029e-01
       coeff_l4(3,7)=  4.3460708112e-02
       coeff_l4(3,8)= -6.8190556765e-01
       coeff_l4(3,9)=  1.9953092560e-02
       coeff_l4(3,10)= -6.4528220892e-01
       coeff_l4(3,11)= -1.8630963564e-01
       coeff_l4(3,12)= -2.7955707908e-01
       coeff_l4(3,13)=  4.7743937373e-01
       coeff_l4(3,14)=  5.2715057135e-01
       coeff_l4(3,15)= -2.0828139782e-01
       coeff_l4(3,16)=  4.0419459343e-01
       coeff_l4(3,17)= -8.6990915239e-02
       coeff_l4(3,18)= -3.1576547027e-01
       coeff_l4(3,19)= -5.1472224295e-03
       coeff_l4(3,20)= -5.8965208009e-03
       coeff_l4(3,21)= -3.5394239426e-01
       coeff_l4(4,1)= -1.3438678980e+00
       coeff_l4(4,2)= -1.5944049358e+00
       coeff_l4(4,3)= -1.4755780697e+00
       coeff_l4(4,4)= -1.7894955873e+00
       coeff_l4(4,5)= -1.6423709393e+00
       coeff_l4(4,6)= -1.8855736256e+00
       coeff_l4(4,7)=  4.7286519408e-01
       coeff_l4(4,8)= -4.2035269737e-01
       coeff_l4(4,9)=  4.0999504924e-01
       coeff_l4(4,10)=  3.0234912038e-01
       coeff_l4(4,11)= -9.0724480152e-01
       coeff_l4(4,12)=  1.8000997305e+00
       coeff_l4(4,13)=  1.7454171553e-02
       coeff_l4(4,14)= -8.5856571794e-02
       coeff_l4(4,15)= -2.6299309731e-01
       coeff_l4(4,16)=  2.7814701200e-01
       coeff_l4(4,17)=  5.2604377270e-01
       coeff_l4(4,18)= -5.4241199046e-02
       coeff_l4(4,19)= -3.5786128044e-01
       coeff_l4(4,20)= -2.3030455410e-01
       coeff_l4(4,21)= -4.5135399699e-01
       coeff_l4(5,1)=  4.0252828598e+00
       coeff_l4(5,2)=  2.1214096546e+00
       coeff_l4(5,3)= -7.9703223705e-01
       coeff_l4(5,4)= -1.2566114664e+00
       coeff_l4(5,5)= -3.8909520954e-02
       coeff_l4(5,6)= -2.1683857441e+00
       coeff_l4(5,7)=  6.9428555667e-02
       coeff_l4(5,8)=  4.9905151129e-01
       coeff_l4(5,9)= -9.6871244907e-01
       coeff_l4(5,10)= -1.2403604388e-01
       coeff_l4(5,11)=  1.4243931770e+00
       coeff_l4(5,12)= -2.1361086369e+00
       coeff_l4(5,13)= -2.9405719042e-01
       coeff_l4(5,14)=  3.9026860148e-02
       coeff_l4(5,15)=  3.6193263531e-01
       coeff_l4(5,16)=  3.2345706224e-01
       coeff_l4(5,17)=  2.0360175520e-02
       coeff_l4(5,18)= -1.0162018239e-01
       coeff_l4(5,19)= -2.3313531280e-01
       coeff_l4(5,20)=  1.3779525757e+00
       coeff_l4(5,21)=  8.5151650012e-02
       coeff_l4(6,1)= -1.4879195690e+00
       coeff_l4(6,2)= -1.9572432041e+00
       coeff_l4(6,3)= -1.0461517572e+00
       coeff_l4(6,4)= -6.5116143227e-01
       coeff_l4(6,5)= -1.2385679483e+00
       coeff_l4(6,6)= -7.9065042734e-01
       coeff_l4(6,7)=  2.8779762983e-01
       coeff_l4(6,8)= -1.7702966928e-02
       coeff_l4(6,9)=  3.1769597530e-01
       coeff_l4(6,10)=  5.2109056711e-01
       coeff_l4(6,11)= -6.6731166840e-01
       coeff_l4(6,12)=  1.1165982485e+00
       coeff_l4(6,13)=  2.4105932564e-02
       coeff_l4(6,14)= -5.8835071325e-01
       coeff_l4(6,15)= -8.3727025986e-01
       coeff_l4(6,16)= -5.0442898273e-01
       coeff_l4(6,17)=  1.0363299400e-01
       coeff_l4(6,18)=  6.2048632652e-02
       coeff_l4(6,19)=  2.4507668614e-01
       coeff_l4(6,20)= -2.6406434178e-01
       coeff_l4(6,21)=  3.8143748045e-01
       coeff_l4(7,1)= -1.6314120293e+00
       coeff_l4(7,2)=  5.1539224386e-01
       coeff_l4(7,3)=  1.4760756493e-01
       coeff_l4(7,4)=  1.0404117107e+00
       coeff_l4(7,5)=  1.4121325314e-01
       coeff_l4(7,6)=  1.6885153055e+00
       coeff_l4(7,7)=  2.8546464443e-01
       coeff_l4(7,8)=  3.7706318498e-01
       coeff_l4(7,9)=  3.6626389623e-01
       coeff_l4(7,10)= -1.2824945152e-01
       coeff_l4(7,11)= -3.0900260806e-01
       coeff_l4(7,12)=  1.5631740093e+00
       coeff_l4(7,13)= -4.5472684503e-01
       coeff_l4(7,14)= -6.6061913967e-02
       coeff_l4(7,15)= -5.9037703276e-01
       coeff_l4(7,16)=  1.0445132107e-01
       coeff_l4(7,17)= -2.6785504073e-02
       coeff_l4(7,18)= -5.2801299840e-02
       coeff_l4(7,19)= -2.6999613643e-01
       coeff_l4(7,20)= -4.1070315242e-01
       coeff_l4(7,21)= -4.9656647444e-01
       coeff_l4(8,1)= -9.5270794630e-01
       coeff_l4(8,2)= -3.0757257342e-01
       coeff_l4(8,3)= -2.4105472863e-01
       coeff_l4(8,4)= -8.9151751995e-01
       coeff_l4(8,5)= -3.1948274374e-01
       coeff_l4(8,6)=  3.5290595889e-01
       coeff_l4(8,7)=  2.5709185004e-01
       coeff_l4(8,8)= -2.8952527791e-02
       coeff_l4(8,9)=  7.7104711533e-01
       coeff_l4(8,10)= -3.7556204200e-01
       coeff_l4(8,11)= -5.1988387108e-01
       coeff_l4(8,12)=  2.9698433876e+00
       coeff_l4(8,13)= -6.6071093082e-02
       coeff_l4(8,14)=  1.9972674549e-02
       coeff_l4(8,15)=  1.3833428919e-01
       coeff_l4(8,16)= -9.0752428770e-01
       coeff_l4(8,17)=  1.5280084312e-01
       coeff_l4(8,18)=  3.2848647237e-01
       coeff_l4(8,19)= -3.7164190412e-01
       coeff_l4(8,20)= -2.4839565754e+00
       coeff_l4(8,21)=  7.8534501791e-01
       coeff_l4(9,1)= -2.2105078697e+00
       coeff_l4(9,2)= -1.2586374283e+00
       coeff_l4(9,3)= -3.4123459458e-01
       coeff_l4(9,4)=  2.8241154552e-01
       coeff_l4(9,5)= -7.4063104391e-01
       coeff_l4(9,6)= -4.4504743814e-01
       coeff_l4(9,7)=  3.6970120668e-01
       coeff_l4(9,8)=  1.7941494286e-01
       coeff_l4(9,9)=  8.1069701910e-01
       coeff_l4(9,10)= -1.0590858012e-01
       coeff_l4(9,11)= -7.6262283325e-01
       coeff_l4(9,12)=  1.1059700251e+00
       coeff_l4(9,13)= -6.5166169405e-01
       coeff_l4(9,14)=  3.6826539040e-01
       coeff_l4(9,15)= -9.4535320997e-01
       coeff_l4(9,16)= -9.2172540724e-02
       coeff_l4(9,17)=  5.3201103210e-01
       coeff_l4(9,18)=  2.3832653463e-01
       coeff_l4(9,19)= -2.9410937428e-01
       coeff_l4(9,20)= -4.8131769896e-01
       coeff_l4(9,21)=  2.1689419448e-01
       coeff_l4(10,1)= -5.5638980865e-01
       coeff_l4(10,2)= -3.4927409887e-01
       coeff_l4(10,3)=  5.6751120090e-01
       coeff_l4(10,4)=  8.4941124916e-01
       coeff_l4(10,5)=  3.5485464334e-01
       coeff_l4(10,6)=  1.9767837226e-01
       coeff_l4(10,7)=  3.7130308151e-01
       coeff_l4(10,8)= -3.7217441201e-01
       coeff_l4(10,9)=  3.8411098719e-01
       coeff_l4(10,10)=  7.6969265938e-01
       coeff_l4(10,11)=  1.9999568164e-01
       coeff_l4(10,12)=  6.5391367674e-01
       coeff_l4(10,13)= -2.7133971453e-01
       coeff_l4(10,14)= -5.0478750467e-01
       coeff_l4(10,15)= -5.6894177198e-01
       coeff_l4(10,16)= -2.2847749293e-01
       coeff_l4(10,17)=  7.6474475861e-01
       coeff_l4(10,18)= -2.5385397673e-01
       coeff_l4(10,19)= -3.7087902427e-01
       coeff_l4(10,20)= -1.6885815561e-01
       coeff_l4(10,21)= -1.2582647800e-01
       coeff_l4(11,1)= -2.6233386993e+00
       coeff_l4(11,2)= -3.6906731129e+00
       coeff_l4(11,3)= -1.4763902426e+00
       coeff_l4(11,4)= -3.3884349465e-01
       coeff_l4(11,5)= -1.7904398441e+00
       coeff_l4(11,6)= -1.9505112171e+00
       coeff_l4(11,7)=  9.9401956797e-01
       coeff_l4(11,8)=  2.5420245528e-01
       coeff_l4(11,9)=  2.9279336333e-01
       coeff_l4(11,10)=  1.0090808868e+00
       coeff_l4(11,11)=  4.3835237622e-01
       coeff_l4(11,12)=  2.1660923958e+00
       coeff_l4(11,13)= -1.4146170616e+00
       coeff_l4(11,14)= -1.6064342260e+00
       coeff_l4(11,15)=  1.9656354189e-01
       coeff_l4(11,16)=  7.3860090971e-01
       coeff_l4(11,17)=  1.2552529573e-01
       coeff_l4(11,18)= -1.7220035195e-01
       coeff_l4(11,19)=  3.0703452229e-01
       coeff_l4(11,20)=  6.7830115557e-01
       coeff_l4(11,21)= -1.1320260167e-01
       coeff_l4(12,1)=  3.0292820930e+00
       coeff_l4(12,2)=  2.2855198383e+00
       coeff_l4(12,3)=  1.1195510626e-01
       coeff_l4(12,4)=  6.7461162806e-01
       coeff_l4(12,5)=  8.2818722725e-01
       coeff_l4(12,6)=  7.9060053825e-01
       coeff_l4(12,7)=  2.5327104330e-01
       coeff_l4(12,8)=  1.1217607558e-01
       coeff_l4(12,9)= -1.3396187127e-01
       coeff_l4(12,10)= -2.0700295269e-01
       coeff_l4(12,11)=  1.2601631880e-01
       coeff_l4(12,12)= -1.1605314016e+00
       coeff_l4(12,13)= -2.1977122128e-01
       coeff_l4(12,14)= -4.5905917883e-01
       coeff_l4(12,15)=  5.4745578766e-01
       coeff_l4(12,16)=  2.2795006633e-01
       coeff_l4(12,17)= -1.2205213308e-01
       coeff_l4(12,18)= -6.8075114489e-01
       coeff_l4(12,19)= -9.4879806042e-01
       coeff_l4(12,20)= -6.6008263826e-01
       coeff_l4(12,21)= -3.1366690993e-02
       coeff_l4(13,1)=  3.0501685143e+00
       coeff_l4(13,2)=  2.6466276646e+00
       coeff_l4(13,3)=  1.0466300249e+00
       coeff_l4(13,4)=  1.6777577400e+00
       coeff_l4(13,5)=  1.4658800364e+00
       coeff_l4(13,6)=  1.9300166368e+00
       coeff_l4(13,7)= -1.5871340409e-02
       coeff_l4(13,8)= -6.8365120888e-01
       coeff_l4(13,9)=  4.9697467685e-01
       coeff_l4(13,10)=  5.5280911922e-01
       coeff_l4(13,11)= -6.8169248104e-01
       coeff_l4(13,12)=  1.0970139503e+00
       coeff_l4(13,13)=  1.0304246098e-01
       coeff_l4(13,14)= -2.9214558005e-01
       coeff_l4(13,15)=  5.9264644980e-02
       coeff_l4(13,16)=  4.2599657178e-01
       coeff_l4(13,17)=  2.2014009953e-01
       coeff_l4(13,18)=  4.7129873186e-02
       coeff_l4(13,19)=  5.1049351692e-01
       coeff_l4(13,20)= -1.3426870108e-01
       coeff_l4(13,21)= -1.5925295651e-01
       coeff_l4(14,1)= -1.9358508587e+00
       coeff_l4(14,2)= -1.4821082354e+00
       coeff_l4(14,3)= -1.5134079754e-01
       coeff_l4(14,4)= -5.6379395723e-01
       coeff_l4(14,5)= -3.5065576434e-01
       coeff_l4(14,6)= -8.4911715984e-01
       coeff_l4(14,7)= -2.5777751207e-01
       coeff_l4(14,8)=  9.8661571741e-02
       coeff_l4(14,9)=  3.5981088877e-02
       coeff_l4(14,10)=  3.8381847739e-01
       coeff_l4(14,11)= -2.9382896423e-01
       coeff_l4(14,12)=  9.2032885551e-01
       coeff_l4(14,13)=  1.3630510867e-01
       coeff_l4(14,14)= -3.5415622592e-01
       coeff_l4(14,15)= -8.3884879947e-02
       coeff_l4(14,16)=  2.0411993563e-01
       coeff_l4(14,17)= -3.8556915522e-01
       coeff_l4(14,18)=  1.4130291343e-01
       coeff_l4(14,19)=  9.8947042227e-01
       coeff_l4(14,20)= -1.6469685733e-01
       coeff_l4(14,21)=  2.2239887714e-01
       coeff_l4(15,1)=  2.6760551333e-01
       coeff_l4(15,2)=  1.6212676466e-01
       coeff_l4(15,3)=  9.9169725180e-01
       coeff_l4(15,4)= -9.1555297375e-01
       coeff_l4(15,5)=  2.7628913522e-01
       coeff_l4(15,6)=  2.2616209984e+00
       coeff_l4(15,7)= -4.9871364236e-01
       coeff_l4(15,8)= -3.2515341043e-01
       coeff_l4(15,9)=  1.6502002478e+00
       coeff_l4(15,10)= -3.6798276007e-02
       coeff_l4(15,11)= -2.4379882216e-01
       coeff_l4(15,12)= -1.0216250271e-01
       coeff_l4(15,13)= -2.4137231708e-01
       coeff_l4(15,14)=  3.3011439443e-01
       coeff_l4(15,15)= -7.1598803997e-01
       coeff_l4(15,16)= -9.4631141424e-01
       coeff_l4(15,17)= -3.9439473301e-02
       coeff_l4(15,18)=  3.0060194433e-02
       coeff_l4(15,19)= -1.3889701664e-01
       coeff_l4(15,20)= -1.9874116182e+00
       coeff_l4(15,21)= -4.2376047373e-01
       coeff_l4(16,1)= -3.1942496300e+00
       coeff_l4(16,2)= -1.8885926008e+00
       coeff_l4(16,3)= -1.0014344454e+00
       coeff_l4(16,4)= -3.0426490307e-01
       coeff_l4(16,5)= -1.9485385418e+00
       coeff_l4(16,6)=  1.5874730051e-01
       coeff_l4(16,7)= -8.6301505566e-01
       coeff_l4(16,8)= -5.2295768261e-01
       coeff_l4(16,9)= -1.1072268337e-01
       coeff_l4(16,10)=  1.1863373965e-01
       coeff_l4(16,11)= -1.3501709700e+00
       coeff_l4(16,12)= -2.3085458577e-01
       coeff_l4(16,13)=  2.9325681925e-01
       coeff_l4(16,14)=  3.2339695841e-02
       coeff_l4(16,15)= -2.4797993898e-01
       coeff_l4(16,16)=  4.3131867051e-01
       coeff_l4(16,17)=  1.9806730747e-01
       coeff_l4(16,18)= -3.8983625174e-01
       coeff_l4(16,19)= -2.8272721171e-01
       coeff_l4(16,20)=  4.7486349940e-01
       coeff_l4(16,21)= -2.2831271589e-01
       coeff_l4(17,1)= -6.6269069910e-01
       coeff_l4(17,2)=  1.1246676445e+00
       coeff_l4(17,3)=  1.7311826944e+00
       coeff_l4(17,4)= -2.0105589181e-02
       coeff_l4(17,5)=  3.9597588778e-01
       coeff_l4(17,6)= -5.5107361078e-01
       coeff_l4(17,7)= -1.0060819332e-03
       coeff_l4(17,8)=  4.3137508631e-01
       coeff_l4(17,9)= -1.2593775988e-01
       coeff_l4(17,10)= -5.6611114740e-01
       coeff_l4(17,11)= -4.5474833250e-01
       coeff_l4(17,12)= -4.3885478377e-01
       coeff_l4(17,13)= -2.0719258487e-01
       coeff_l4(17,14)=  7.8849935532e-01
       coeff_l4(17,15)=  7.4681919813e-01
       coeff_l4(17,16)= -2.4143721163e-01
       coeff_l4(17,17)= -5.5465650558e-01
       coeff_l4(17,18)=  9.5223098993e-01
       coeff_l4(17,19)= -1.1510003358e-02
       coeff_l4(17,20)= -8.1897753477e-01
       coeff_l4(17,21)=  2.1493309736e-01
       coeff_l4(18,1)=  1.4386217594e+00
       coeff_l4(18,2)=  1.0937131643e+00
       coeff_l4(18,3)=  7.9975199699e-01
       coeff_l4(18,4)= -1.0621209145e+00
       coeff_l4(18,5)=  5.7034718990e-01
       coeff_l4(18,6)=  8.6966812611e-01
       coeff_l4(18,7)=  3.7984329462e-01
       coeff_l4(18,8)=  4.9786403775e-01
       coeff_l4(18,9)=  1.9102670252e-01
       coeff_l4(18,10)= -6.0256427526e-01
       coeff_l4(18,11)=  1.0624608994e+00
       coeff_l4(18,12)= -2.4609193802e+00
       coeff_l4(18,13)= -2.2578839958e-01
       coeff_l4(18,14)= -1.7461080849e-01
       coeff_l4(18,15)=  4.3209180236e-01
       coeff_l4(18,16)= -6.4061492682e-01
       coeff_l4(18,17)=  2.0730726421e-01
       coeff_l4(18,18)=  3.7395197153e-01
       coeff_l4(18,19)= -2.8781023622e-01
       coeff_l4(18,20)=  9.0563875437e-01
       coeff_l4(18,21)= -9.2304274440e-02
       coeff_l4(19,1)=  1.1920108795e+00
       coeff_l4(19,2)=  9.0365165472e-01
       coeff_l4(19,3)=  1.2018543482e+00
       coeff_l4(19,4)=  1.6858917475e+00
       coeff_l4(19,5)=  1.9132182598e+00
       coeff_l4(19,6)=  1.3104662895e+00
       coeff_l4(19,7)= -3.1312841177e-01
       coeff_l4(19,8)=  1.9887359440e-01
       coeff_l4(19,9)= -1.5598835051e-01
       coeff_l4(19,10)= -1.9309827685e-01
       coeff_l4(19,11)= -1.9048014656e-02
       coeff_l4(19,12)= -6.6343396902e-01
       coeff_l4(19,13)=  3.2934537530e-01
       coeff_l4(19,14)=  9.6433806419e-01
       coeff_l4(19,15)=  3.7392419577e-01
       coeff_l4(19,16)=  4.8893326521e-01
       coeff_l4(19,17)=  2.2387872636e-01
       coeff_l4(19,18)= -3.3757740259e-01
       coeff_l4(19,19)= -6.3211905956e-01
       coeff_l4(19,20)=  4.9849450588e-01
       coeff_l4(19,21)=  2.8102576733e-01
       coeff_l4(20,1)= -1.2512441874e+00
       coeff_l4(20,2)= -1.5904569626e+00
       coeff_l4(20,3)=  4.9034011364e-01
       coeff_l4(20,4)=  4.4609433413e-01
       coeff_l4(20,5)=  1.0060364753e-01
       coeff_l4(20,6)=  3.0776908621e-02
       coeff_l4(20,7)= -3.7260364741e-02
       coeff_l4(20,8)=  5.9694075584e-01
       coeff_l4(20,9)=  2.1679571271e-01
       coeff_l4(20,10)= -2.5271511078e-01
       coeff_l4(20,11)=  3.2261629105e+00
       coeff_l4(20,12)= -2.2349097729e+00
       coeff_l4(20,13)=  3.3142343163e-01
       coeff_l4(20,14)= -7.5282501057e-03
       coeff_l4(20,15)=  2.9844403267e-01
       coeff_l4(20,16)=  2.1309596300e-01
       coeff_l4(20,17)= -5.6424140930e-02
       coeff_l4(20,18)= -1.3607060909e-01
       coeff_l4(20,19)=  4.6459458768e-02
       coeff_l4(20,20)=  8.5032619536e-02
       coeff_l4(20,21)=  1.4758108556e-01
       coeff_l4(21,1)= -2.5051200390e+00
       coeff_l4(21,2)= -2.1291065216e+00
       coeff_l4(21,3)= -6.8831592798e-01
       coeff_l4(21,4)= -9.4909393787e-01
       coeff_l4(21,5)= -1.4377553463e+00
       coeff_l4(21,6)= -5.3899794817e-01
       coeff_l4(21,7)=  1.4169191942e-02
       coeff_l4(21,8)= -3.5395026207e-01
       coeff_l4(21,9)=  8.5561209917e-01
       coeff_l4(21,10)=  1.6060596704e-01
       coeff_l4(21,11)= -5.9101200104e-01
       coeff_l4(21,12)=  1.0303466320e+00
       coeff_l4(21,13)= -4.1129317880e-01
       coeff_l4(21,14)= -3.7735846639e-01
       coeff_l4(21,15)= -9.0324866772e-01
       coeff_l4(21,16)=  8.2801751792e-02
       coeff_l4(21,17)=  1.8574285507e-01
       coeff_l4(21,18)=  4.1424527764e-02
       coeff_l4(21,19)=  8.7601602077e-01
       coeff_l4(21,20)= -1.3684014557e-03
       coeff_l4(21,21)=  4.3850582838e-01
       coeff_l4(22,1)=  2.1656622887e+00
       coeff_l4(22,2)=  2.6276612282e+00
       coeff_l4(22,3)=  1.3750978708e+00
       coeff_l4(22,4)=  1.7786765099e+00
       coeff_l4(22,5)=  1.2562221289e+00
       coeff_l4(22,6)=  1.9050477743e+00
       coeff_l4(22,7)= -3.7039881945e-01
       coeff_l4(22,8)=  1.2345541269e-01
       coeff_l4(22,9)= -4.1808393598e-01
       coeff_l4(22,10)= -5.0969642401e-01
       coeff_l4(22,11)= -1.4228153229e-01
       coeff_l4(22,12)= -8.9491099119e-01
       coeff_l4(22,13)=  2.1257083118e-01
       coeff_l4(22,14)=  6.1505734921e-01
       coeff_l4(22,15)=  6.4507895708e-01
       coeff_l4(22,16)=  7.1831279993e-01
       coeff_l4(22,17)= -2.1250694990e-01
       coeff_l4(22,18)=  2.9526960850e-01
       coeff_l4(22,19)= -1.9928251207e-01
       coeff_l4(22,20)=  2.6420649886e-01
       coeff_l4(22,21)= -2.0340022445e-01
       coeff_l4(23,1)= -1.1887664795e+00
       coeff_l4(23,2)= -1.0345098972e+00
       coeff_l4(23,3)=  1.0647650063e-01
       coeff_l4(23,4)=  4.3014699221e-01
       coeff_l4(23,5)=  1.1609263718e-01
       coeff_l4(23,6)=  1.3855218887e-01
       coeff_l4(23,7)=  4.8465812206e-01
       coeff_l4(23,8)=  2.9074138403e-01
       coeff_l4(23,9)=  4.1426229477e-01
       coeff_l4(23,10)=  2.2054493427e-01
       coeff_l4(23,11)=  1.3843658566e-01
       coeff_l4(23,12)= -5.4637055844e-02
       coeff_l4(23,13)= -3.3533733338e-02
       coeff_l4(23,14)= -9.4657480717e-01
       coeff_l4(23,15)= -2.4463252723e-01
       coeff_l4(23,16)= -8.3742745221e-02
       coeff_l4(23,17)= -1.7612737417e-01
       coeff_l4(23,18)=  2.0378355682e-01
       coeff_l4(23,19)=  9.8009258509e-02
       coeff_l4(23,20)=  4.3113976717e-01
       coeff_l4(23,21)=  8.8413674384e-03
       coeff_l4(24,1)=  3.5752889514e-01
       coeff_l4(24,2)=  1.3885204494e-01
       coeff_l4(24,3)= -5.8564490080e-01
       coeff_l4(24,4)=  4.3670734763e-01
       coeff_l4(24,5)=  5.2571916580e-01
       coeff_l4(24,6)= -8.7918078899e-01
       coeff_l4(24,7)=  2.2233514488e-01
       coeff_l4(24,8)=  2.1416036785e-01
       coeff_l4(24,9)=  4.7245627642e-01
       coeff_l4(24,10)=  1.5829604864e-01
       coeff_l4(24,11)= -7.6059454679e-01
       coeff_l4(24,12)= -5.4640915245e-02
       coeff_l4(24,13)= -5.5725181103e-01
       coeff_l4(24,14)=  2.6117455959e-01
       coeff_l4(24,15)= -4.9721494317e-01
       coeff_l4(24,16)= -3.5194227099e-01
       coeff_l4(24,17)= -2.9936340451e-01
       coeff_l4(24,18)=  4.7792446613e-01
       coeff_l4(24,19)= -3.0659455061e-01
       coeff_l4(24,20)=  3.3090829849e-01
       coeff_l4(24,21)=  3.3954748511e-01
       coeff_l4(25,1)= -4.0085186958e+00
       coeff_l4(25,2)= -2.6554412842e+00
       coeff_l4(25,3)= -1.7248206139e+00
       coeff_l4(25,4)= -1.9822309017e+00
       coeff_l4(25,5)= -3.0052928925e+00
       coeff_l4(25,6)= -8.0033302307e-01
       coeff_l4(25,7)= -2.1597996354e-01
       coeff_l4(25,8)= -3.7389427423e-01
       coeff_l4(25,9)=  8.3672833443e-01
       coeff_l4(25,10)=  9.4677907228e-01
       coeff_l4(25,11)= -6.1358970404e-01
       coeff_l4(25,12)=  2.5987586975e+00
       coeff_l4(25,13)= -7.8706961870e-01
       coeff_l4(25,14)= -6.4426785707e-01
       coeff_l4(25,15)= -1.5870848894e+00
       coeff_l4(25,16)= -4.0468436480e-01
       coeff_l4(25,17)=  4.4952053577e-02
       coeff_l4(25,18)=  3.8911983371e-01
       coeff_l4(25,19)=  1.9307466745e+00
       coeff_l4(25,20)= -9.5205444098e-01
       coeff_l4(25,21)= -5.8813160658e-01
       coeff_l4(26,1)= -2.4772570133e+00
       coeff_l4(26,2)= -2.5055289268e+00
       coeff_l4(26,3)= -1.1271177530e+00
       coeff_l4(26,4)= -1.1746426821e+00
       coeff_l4(26,5)= -1.9209524393e+00
       coeff_l4(26,6)= -1.9758791924e+00
       coeff_l4(26,7)=  4.3485647440e-01
       coeff_l4(26,8)= -1.9326919690e-02
       coeff_l4(26,9)=  1.1456079781e-01
       coeff_l4(26,10)=  7.0792895555e-01
       coeff_l4(26,11)= -1.4461770654e-01
       coeff_l4(26,12)=  1.9315528870e+00
       coeff_l4(26,13)= -3.7727057934e-01
       coeff_l4(26,14)= -1.1462078989e-01
       coeff_l4(26,15)= -8.3568000793e-01
       coeff_l4(26,16)=  4.7402136028e-02
       coeff_l4(26,17)=  8.8611143827e-01
       coeff_l4(26,18)= -1.2339766026e+00
       coeff_l4(26,19)=  1.8334342539e-01
       coeff_l4(26,20)= -7.3490244150e-01
       coeff_l4(26,21)=  3.6218229681e-02
       coeff_l4(27,1)= -1.3428851366e+00
       coeff_l4(27,2)= -1.3051474094e-01
       coeff_l4(27,3)=  1.0820069313e+00
       coeff_l4(27,4)=  1.3269985914e+00
       coeff_l4(27,5)=  3.0556270480e-01
       coeff_l4(27,6)=  6.0662579536e-01
       coeff_l4(27,7)=  3.9712324739e-02
       coeff_l4(27,8)=  5.1519054174e-01
       coeff_l4(27,9)= -1.0673574209e+00
       coeff_l4(27,10)= -2.2253932431e-02
       coeff_l4(27,11)=  2.1032919884e+00
       coeff_l4(27,12)= -6.8363852799e-02
       coeff_l4(27,13)=  3.2923230529e-01
       coeff_l4(27,14)= -4.9334776402e-01
       coeff_l4(27,15)=  4.9711048603e-01
       coeff_l4(27,16)= -2.7710387111e-01
       coeff_l4(27,17)= -3.3209715039e-02
       coeff_l4(27,18)=  9.1963328421e-02
       coeff_l4(27,19)=  1.2030607462e-01
       coeff_l4(27,20)= -1.1223927736e+00
       coeff_l4(27,21)= -2.2211021185e-01
       coeff_l4(28,1)= -5.1267105341e-01
       coeff_l4(28,2)= -3.7002074718e-01
       coeff_l4(28,3)=  7.4447280169e-01
       coeff_l4(28,4)= -4.6092936397e-01
       coeff_l4(28,5)=  1.3462363183e-01
       coeff_l4(28,6)= -1.3425334692e+00
       coeff_l4(28,7)=  3.4442293644e-01
       coeff_l4(28,8)=  3.0301383138e-01
       coeff_l4(28,9)=  1.4039919376e+00
       coeff_l4(28,10)=  2.5451302528e-01
       coeff_l4(28,11)= -1.3455321081e-02
       coeff_l4(28,12)=  1.9354896247e-01
       coeff_l4(28,13)= -3.7431320548e-01
       coeff_l4(28,14)=  3.7224803120e-02
       coeff_l4(28,15)= -4.0953457355e-02
       coeff_l4(28,16)= -2.9500264674e-02
       coeff_l4(28,17)= -2.1517528594e-01
       coeff_l4(28,18)= -4.2110332847e-01
       coeff_l4(28,19)=  1.7727741599e-01
       coeff_l4(28,20)= -6.8545764685e-01
       coeff_l4(28,21)= -1.8377099931e-01
       coeff_l4(29,1)=  3.8225147724e+00
       coeff_l4(29,2)=  3.4708197117e+00
       coeff_l4(29,3)=  2.5603959560e+00
       coeff_l4(29,4)=  2.8018302917e+00
       coeff_l4(29,5)=  2.6042652130e+00
       coeff_l4(29,6)=  2.7146594524e+00
       coeff_l4(29,7)= -8.0201970413e-03
       coeff_l4(29,8)=  3.5687395930e-01
       coeff_l4(29,9)=  3.9832521230e-02
       coeff_l4(29,10)= -3.4322029352e-01
       coeff_l4(29,11)= -1.1950095743e-01
       coeff_l4(29,12)= -1.8910613060e+00
       coeff_l4(29,13)=  8.3036816120e-01
       coeff_l4(29,14)=  6.7509418726e-01
       coeff_l4(29,15)=  7.5723993778e-01
       coeff_l4(29,16)=  1.4918807149e-01
       coeff_l4(29,17)= -1.4045752585e-01
       coeff_l4(29,18)=  6.5020591021e-01
       coeff_l4(29,19)= -5.8763706684e-01
       coeff_l4(29,20)=  5.2645587921e-01
       coeff_l4(29,21)=  2.1952725947e-01
       coeff_l4(30,1)=  2.2241854668e+00
       coeff_l4(30,2)=  2.6947815418e+00
       coeff_l4(30,3)= -2.1621961594e+00
       coeff_l4(30,4)=  1.2121152878e+00
       coeff_l4(30,5)=  4.3752980232e-01
       coeff_l4(30,6)= -2.5335175991e+00
       coeff_l4(30,7)=  2.0603418350e-01
       coeff_l4(30,8)= -2.5960367918e-01
       coeff_l4(30,9)=  7.7143484354e-01
       coeff_l4(30,10)=  1.8077491224e-01
       coeff_l4(30,11)= -1.5167042017e+00
       coeff_l4(30,12)=  7.9481327534e-01
       coeff_l4(30,13)=  4.2718384415e-02
       coeff_l4(30,14)= -7.4093304574e-02
       coeff_l4(30,15)=  1.9352394342e-01
       coeff_l4(30,16)= -3.1739148498e-01
       coeff_l4(30,17)= -4.7651585191e-03
       coeff_l4(30,18)= -1.8710615113e-02
       coeff_l4(30,19)= -1.6145442426e-01
       coeff_l4(30,20)= -7.1999281645e-01
       coeff_l4(30,21)= -3.4593276214e-03
       coeff_l4(31,1)= -1.9284793139e+00
       coeff_l4(31,2)= -1.8936659098e+00
       coeff_l4(31,3)= -1.7373106480e+00
       coeff_l4(31,4)= -2.0411787033e+00
       coeff_l4(31,5)= -1.9445248842e+00
       coeff_l4(31,6)= -2.3850715160e+00
       coeff_l4(31,7)=  5.6564885378e-01
       coeff_l4(31,8)=  4.5402385294e-02
       coeff_l4(31,9)= -7.2492636740e-02
       coeff_l4(31,10)=  3.5481113940e-02
       coeff_l4(31,11)= -7.2239583731e-01
       coeff_l4(31,12)=  2.0208225250e+00
       coeff_l4(31,13)= -5.6052929163e-01
       coeff_l4(31,14)=  5.8330398798e-01
       coeff_l4(31,15)= -2.3585948348e-01
       coeff_l4(31,16)= -1.2589529157e-01
       coeff_l4(31,17)=  7.3810386658e-01
       coeff_l4(31,18)= -4.3511262536e-01
       coeff_l4(31,19)= -1.7198525369e-01
       coeff_l4(31,20)= -6.3694328070e-01
       coeff_l4(31,21)= -5.9314268827e-01
       coeff_l4(32,1)=  6.6343444586e-01
       coeff_l4(32,2)= -1.8945643902e+00
       coeff_l4(32,3)=  4.8003104329e-01
       coeff_l4(32,4)= -2.0813672543e+00
       coeff_l4(32,5)= -5.9810096025e-01
       coeff_l4(32,6)=  1.0335509777e+00
       coeff_l4(32,7)=  7.4732974172e-02
       coeff_l4(32,8)= -9.5161533356e-01
       coeff_l4(32,9)= -1.8115533888e-01
       coeff_l4(32,10)= -1.4782688022e-01
       coeff_l4(32,11)= -1.2664875984e+00
       coeff_l4(32,12)= -6.7260879278e-01
       coeff_l4(32,13)=  6.8945288658e-01
       coeff_l4(32,14)= -1.1138836294e-01
       coeff_l4(32,15)= -2.2068594396e-01
       coeff_l4(32,16)=  4.2533852160e-02
       coeff_l4(32,17)=  3.5281318426e-01
       coeff_l4(32,18)= -3.0879360437e-01
       coeff_l4(32,19)= -3.8339316845e-01
       coeff_l4(32,20)=  1.6874331236e+00
       coeff_l4(32,21)= -1.1535359174e-01
       coeff_l4(33,1)= -1.7767966986e+00
       coeff_l4(33,2)= -1.3775398731e+00
       coeff_l4(33,3)=  2.8139337897e-01
       coeff_l4(33,4)= -1.5304064378e-02
       coeff_l4(33,5)= -5.1863622665e-01
       coeff_l4(33,6)= -1.3612286747e-01
       coeff_l4(33,7)=  3.5929256678e-01
       coeff_l4(33,8)= -3.2996577024e-01
       coeff_l4(33,9)= -1.0993430614e+00
       coeff_l4(33,10)=  8.3684585989e-02
       coeff_l4(33,11)= -3.0356475711e-01
       coeff_l4(33,12)=  1.4400689602e+00
       coeff_l4(33,13)=  5.0240713358e-01
       coeff_l4(33,14)= -6.2240219116e-01
       coeff_l4(33,15)=  3.0361136794e-01
       coeff_l4(33,16)=  5.2743816376e-01
       coeff_l4(33,17)=  3.4139242768e-01
       coeff_l4(33,18)= -1.8147714436e-01
       coeff_l4(33,19)= -1.8496499956e-01
       coeff_l4(33,20)= -2.7586027980e-01
       coeff_l4(33,21)= -1.7606841028e-01
       coeff_l4(34,1)=  9.7867017984e-01
       coeff_l4(34,2)=  5.6631952524e-01
       coeff_l4(34,3)=  5.6946597993e-02
       coeff_l4(34,4)= -1.9180412591e-01
       coeff_l4(34,5)= -1.5583060682e-01
       coeff_l4(34,6)=  2.1944675446e+00
       coeff_l4(34,7)= -2.7028864250e-02
       coeff_l4(34,8)= -1.1184438467e+00
       coeff_l4(34,9)=  1.2977369130e-01
       coeff_l4(34,10)= -1.3859023154e-01
       coeff_l4(34,11)=  3.4470680356e-01
       coeff_l4(34,12)=  7.7309972048e-01
       coeff_l4(34,13)=  3.9657557011e-01
       coeff_l4(34,14)= -2.7558827400e-01
       coeff_l4(34,15)=  2.1334441379e-02
       coeff_l4(34,16)= -2.1142278612e-01
       coeff_l4(34,17)=  3.5194167495e-01
       coeff_l4(34,18)= -1.4845627546e+00
       coeff_l4(34,19)=  3.8008531928e-01
       coeff_l4(34,20)= -3.9722093940e-01
       coeff_l4(34,21)= -1.1949350834e+00
       coeff_l4(35,1)=  1.3086947203e+00
       coeff_l4(35,2)=  3.3010504246e+00
       coeff_l4(35,3)=  4.0716153383e-01
       coeff_l4(35,4)=  1.0668627024e+00
       coeff_l4(35,5)=  1.3005081415e+00
       coeff_l4(35,6)=  1.0886511803e+00
       coeff_l4(35,7)= -4.9279746413e-01
       coeff_l4(35,8)=  6.5574598312e-01
       coeff_l4(35,9)=  5.1720702648e-01
       coeff_l4(35,10)= -5.9479266405e-01
       coeff_l4(35,11)= -8.2316792011e-01
       coeff_l4(35,12)= -6.1547756195e-01
       coeff_l4(35,13)= -4.5876353979e-01
       coeff_l4(35,14)= -4.4886136055e-01
       coeff_l4(35,15)= -5.3261511028e-02
       coeff_l4(35,16)= -7.2068035603e-01
       coeff_l4(35,17)= -1.2695621252e+00
       coeff_l4(35,18)=  8.1250560284e-01
       coeff_l4(35,19)=  1.0374458134e-01
       coeff_l4(35,20)= -2.5304672122e-01
       coeff_l4(35,21)=  5.4708009958e-01
       coeff_l4(36,1)= -1.3548270464e+00
       coeff_l4(36,2)= -1.3361958265e+00
       coeff_l4(36,3)= -1.8406965733e+00
       coeff_l4(36,4)=  2.1441471577e+00
       coeff_l4(36,5)= -5.4784238338e-01
       coeff_l4(36,6)= -8.3717495203e-02
       coeff_l4(36,7)= -2.8783851862e-01
       coeff_l4(36,8)=  5.9941822290e-01
       coeff_l4(36,9)= -5.5009442568e-01
       coeff_l4(36,10)=  8.1689214706e-01
       coeff_l4(36,11)= -1.8646614552e+00
       coeff_l4(36,12)=  1.3912961483e+00
       coeff_l4(36,13)=  6.9114439189e-02
       coeff_l4(36,14)= -4.6397867799e-01
       coeff_l4(36,15)=  4.2203748226e-01
       coeff_l4(36,16)=  1.0159218311e+00
       coeff_l4(36,17)=  1.2077546865e-01
       coeff_l4(36,18)= -5.4320018739e-02
       coeff_l4(36,19)=  2.4115453660e-01
       coeff_l4(36,20)=  4.6262764931e-01
       coeff_l4(36,21)= -3.4499561787e-01
       coeff_l4(37,1)= -4.8661047220e-01
       coeff_l4(37,2)= -1.1256099939e+00
       coeff_l4(37,3)= -1.5333373547e+00
       coeff_l4(37,4)= -1.5069524050e+00
       coeff_l4(37,5)= -1.7687762976e+00
       coeff_l4(37,6)=  9.1350235045e-02
       coeff_l4(37,7)=  1.7730543613e+00
       coeff_l4(37,8)= -2.8620508313e-01
       coeff_l4(37,9)= -6.8336613476e-02
       coeff_l4(37,10)= -4.2904183269e-02
       coeff_l4(37,11)= -4.6727052331e-01
       coeff_l4(37,12)=  5.1445609331e-01
       coeff_l4(37,13)= -2.9203665257e-01
       coeff_l4(37,14)=  2.8501042724e-01
       coeff_l4(37,15)= -2.5127387047e-01
       coeff_l4(37,16)= -3.0643549562e-01
       coeff_l4(37,17)=  4.6973648667e-01
       coeff_l4(37,18)=  3.1149038672e-01
       coeff_l4(37,19)=  2.0343504846e-01
       coeff_l4(37,20)= -1.2993423641e-01
       coeff_l4(37,21)=  4.1793560982e-01
       coeff_l4(38,1)= -2.4409623146e+00
       coeff_l4(38,2)= -2.1894757748e+00
       coeff_l4(38,3)= -1.1547497511e+00
       coeff_l4(38,4)= -1.3455319405e+00
       coeff_l4(38,5)= -2.1107690334e+00
       coeff_l4(38,6)= -6.7263340950e-01
       coeff_l4(38,7)=  4.8849058151e-01
       coeff_l4(38,8)= -1.1720336229e-01
       coeff_l4(38,9)=  5.4772090912e-01
       coeff_l4(38,10)=  9.3856644630e-01
       coeff_l4(38,11)= -1.1487256289e+00
       coeff_l4(38,12)=  1.2279247046e+00
       coeff_l4(38,13)= -6.4752227068e-01
       coeff_l4(38,14)= -1.1849319935e+00
       coeff_l4(38,15)= -9.3169534206e-01
       coeff_l4(38,16)= -5.9484469891e-01
       coeff_l4(38,17)= -1.3288591802e-01
       coeff_l4(38,18)=  2.7908262610e-01
       coeff_l4(38,19)=  4.0736418962e-01
       coeff_l4(38,20)= -1.0909594297e+00
       coeff_l4(38,21)=  5.6614339352e-01
       coeff_l4(39,1)=  4.5728123188e-01
       coeff_l4(39,2)= -2.7894327641e+00
       coeff_l4(39,3)=  6.8311476707e-01
       coeff_l4(39,4)= -3.2060170174e+00
       coeff_l4(39,5)= -5.1275569201e-01
       coeff_l4(39,6)= -1.1163971573e-01
       coeff_l4(39,7)=  5.6452149153e-01
       coeff_l4(39,8)=  1.9938264787e-01
       coeff_l4(39,9)= -1.6042411327e+00
       coeff_l4(39,10)=  1.8373217434e-02
       coeff_l4(39,11)= -1.8724179268e-01
       coeff_l4(39,12)=  1.3663798571e+00
       coeff_l4(39,13)=  1.8372006714e-01
       coeff_l4(39,14)= -2.8137231246e-02
       coeff_l4(39,15)=  4.7932928801e-01
       coeff_l4(39,16)=  6.5897911787e-01
       coeff_l4(39,17)=  6.0159865767e-02
       coeff_l4(39,18)=  9.5248324797e-03
       coeff_l4(39,19)=  1.3129687309e-01
       coeff_l4(39,20)=  1.4918239117e+00
       coeff_l4(39,21)=  3.4728735685e-01
       coeff_l4(40,1)= -1.0094588995e+00
       coeff_l4(40,2)= -1.4297608137e+00
       coeff_l4(40,3)= -2.6323347092e+00
       coeff_l4(40,4)=  2.0203237534e+00
       coeff_l4(40,5)= -1.4855864644e-01
       coeff_l4(40,6)=  7.9265162349e-02
       coeff_l4(40,7)=  1.6802372038e-01
       coeff_l4(40,8)= -2.8988167644e-02
       coeff_l4(40,9)= -7.5391435623e-01
       coeff_l4(40,10)=  6.4873564243e-01
       coeff_l4(40,11)=  1.2506388426e+00
       coeff_l4(40,12)= -1.2216144800e+00
       coeff_l4(40,13)=  1.6343211755e-02
       coeff_l4(40,14)=  1.0549782217e-01
       coeff_l4(40,15)= -4.4794553518e-01
       coeff_l4(40,16)= -3.4447714686e-01
       coeff_l4(40,17)= -1.5988519788e-01
       coeff_l4(40,18)= -1.0625170469e+00
       coeff_l4(40,19)=  8.8377840817e-02
       coeff_l4(40,20)=  7.7928072214e-01
       coeff_l4(40,21)=  2.8283056617e-01
       coeff_l4(41,1)= -1.0301508904e+00
       coeff_l4(41,2)=  5.5985502899e-02
       coeff_l4(41,3)= -8.6600489914e-02
       coeff_l4(41,4)= -9.3668542802e-02
       coeff_l4(41,5)= -1.2876533270e+00
       coeff_l4(41,6)= -2.6617553830e-01
       coeff_l4(41,7)=  4.7523546964e-02
       coeff_l4(41,8)=  8.0003690720e-01
       coeff_l4(41,9)= -6.1842149496e-01
       coeff_l4(41,10)=  2.7697944641e-01
       coeff_l4(41,11)=  3.4098899364e-01
       coeff_l4(41,12)=  1.0302224159e+00
       coeff_l4(41,13)= -1.6687046289e+00
       coeff_l4(41,14)= -1.4696468413e-01
       coeff_l4(41,15)= -5.9240262955e-02
       coeff_l4(41,16)= -1.5608516335e-01
       coeff_l4(41,17)= -5.3983473778e-01
       coeff_l4(41,18)=  7.5111991167e-01
       coeff_l4(41,19)= -2.0894804597e-01
       coeff_l4(41,20)= -8.0069921911e-02
       coeff_l4(41,21)= -3.1913769245e-01
       coeff_l4(42,1)=  2.2951765060e+00
       coeff_l4(42,2)=  2.3051009178e+00
       coeff_l4(42,3)=  1.1213393211e+00
       coeff_l4(42,4)=  1.1350241899e+00
       coeff_l4(42,5)=  1.5837280750e+00
       coeff_l4(42,6)=  6.0329645872e-01
       coeff_l4(42,7)= -3.0910542607e-01
       coeff_l4(42,8)=  2.7018949389e-01
       coeff_l4(42,9)= -5.1900672913e-01
       coeff_l4(42,10)= -4.0960320830e-01
       coeff_l4(42,11)= -2.4839803576e-01
       coeff_l4(42,12)= -8.4233260155e-01
       coeff_l4(42,13)=  1.5384604037e-01
       coeff_l4(42,14)=  1.9044414163e-01
       coeff_l4(42,15)=  3.8408789039e-01
       coeff_l4(42,16)= -5.9560466558e-02
       coeff_l4(42,17)= -2.8801780939e-01
       coeff_l4(42,18)=  6.2935918570e-01
       coeff_l4(42,19)= -3.7892204523e-01
       coeff_l4(42,20)=  8.4317706525e-02
       coeff_l4(42,21)=  5.3587186337e-01
       coeff_l4(43,1)= -1.4602820873e+00
       coeff_l4(43,2)= -1.6046100855e+00
       coeff_l4(43,3)= -7.3763984442e-01
       coeff_l4(43,4)= -7.4312329292e-01
       coeff_l4(43,5)= -8.6575126648e-01
       coeff_l4(43,6)= -1.4661027193e+00
       coeff_l4(43,7)= -4.4147360325e-01
       coeff_l4(43,8)= -9.4417288899e-02
       coeff_l4(43,9)= -2.7501535416e-01
       coeff_l4(43,10)=  5.3186994791e-01
       coeff_l4(43,11)=  1.9331499934e-02
       coeff_l4(43,12)=  9.8280894756e-01
       coeff_l4(43,13)= -2.3559387028e-01
       coeff_l4(43,14)= -2.5002086163e-01
       coeff_l4(43,15)=  3.4544348717e-01
       coeff_l4(43,16)= -5.1935422421e-01
       coeff_l4(43,17)=  1.4803455770e-01
       coeff_l4(43,18)=  4.6310237050e-01
       coeff_l4(43,19)=  1.7955979705e-01
       coeff_l4(43,20)= -1.8607823551e-01
       coeff_l4(43,21)=  3.2495999336e-01
       coeff_l4(44,1)= -1.7550332546e+00
       coeff_l4(44,2)= -9.8535209894e-01
       coeff_l4(44,3)= -5.9331005812e-01
       coeff_l4(44,4)= -2.4825456738e-01
       coeff_l4(44,5)= -1.5581721067e-01
       coeff_l4(44,6)= -6.6517752409e-01
       coeff_l4(44,7)= -3.9378851652e-01
       coeff_l4(44,8)=  2.9408007860e-01
       coeff_l4(44,9)=  7.5021080673e-02
       coeff_l4(44,10)= -1.9582290202e-02
       coeff_l4(44,11)=  9.2073474079e-03
       coeff_l4(44,12)=  1.1308956891e-01
       coeff_l4(44,13)= -1.9685870409e-01
       coeff_l4(44,14)= -2.4517033249e-02
       coeff_l4(44,15)= -4.1801106930e-01
       coeff_l4(44,16)=  3.4784048796e-01
       coeff_l4(44,17)= -4.0326038003e-01
       coeff_l4(44,18)=  3.0480694771e-01
       coeff_l4(44,19)= -2.8020665050e-01
       coeff_l4(44,20)= -1.5174499154e-01
       coeff_l4(44,21)= -1.6555818915e-01
       coeff_l4(45,1)=  4.4298014641e+00
       coeff_l4(45,2)=  3.8074798584e+00
       coeff_l4(45,3)=  1.1485006809e+00
       coeff_l4(45,4)=  8.2482445240e-01
       coeff_l4(45,5)=  1.7692558765e+00
       coeff_l4(45,6)=  1.3832784891e+00
       coeff_l4(45,7)= -3.4836772084e-01
       coeff_l4(45,8)=  5.2179050446e-01
       coeff_l4(45,9)= -1.6033464670e-01
       coeff_l4(45,10)= -8.1064975262e-01
       coeff_l4(45,11)=  5.7058727741e-01
       coeff_l4(45,12)= -1.4305219650e+00
       coeff_l4(45,13)=  5.0931453705e-01
       coeff_l4(45,14)=  5.2974492311e-01
       coeff_l4(45,15)=  6.0079675913e-01
       coeff_l4(45,16)= -3.5771477968e-02
       coeff_l4(45,17)= -2.4211758375e-01
       coeff_l4(45,18)=  2.5668990612e-01
       coeff_l4(45,19)= -4.4752749801e-01
       coeff_l4(45,20)=  1.0317051411e+00
       coeff_l4(45,21)=  2.8306722641e-01
       coeff_l4(46,1)= -3.3068633080e-01
       coeff_l4(46,2)=  6.1436057091e-01
       coeff_l4(46,3)= -2.8719391823e+00
       coeff_l4(46,4)= -9.5832443237e-01
       coeff_l4(46,5)= -8.3222168684e-01
       coeff_l4(46,6)= -1.6316579580e+00
       coeff_l4(46,7)= -2.6830923557e-01
       coeff_l4(46,8)= -4.6776521206e-01
       coeff_l4(46,9)=  2.3437123001e-01
       coeff_l4(46,10)=  1.0911454260e-01
       coeff_l4(46,11)= -6.6766220331e-01
       coeff_l4(46,12)=  6.0673242807e-01
       coeff_l4(46,13)=  1.0445886105e-01
       coeff_l4(46,14)=  7.0416271687e-02
       coeff_l4(46,15)= -2.4225471914e-01
       coeff_l4(46,16)= -1.2436698377e-01
       coeff_l4(46,17)=  7.9508550465e-02
       coeff_l4(46,18)=  5.3798668087e-02
       coeff_l4(46,19)=  6.0892450809e-01
       coeff_l4(46,20)= -1.6294957697e-01
       coeff_l4(46,21)=  7.3742434382e-02
       coeff_l4(47,1)=  9.1433793306e-01
       coeff_l4(47,2)=  2.0355820656e+00
       coeff_l4(47,3)= -1.9709788263e-02
       coeff_l4(47,4)= -7.9980485141e-02
       coeff_l4(47,5)=  1.4467738569e-01
       coeff_l4(47,6)=  5.8875888586e-01
       coeff_l4(47,7)=  8.9439049363e-02
       coeff_l4(47,8)=  8.9447313547e-01
       coeff_l4(47,9)=  1.6781875491e-01
       coeff_l4(47,10)= -3.3259880543e-01
       coeff_l4(47,11)= -9.7822964191e-01
       coeff_l4(47,12)= -4.4944345951e-01
       coeff_l4(47,13)= -1.3824380934e-01
       coeff_l4(47,14)=  7.1167176962e-01
       coeff_l4(47,15)=  3.5165959597e-01
       coeff_l4(47,16)=  5.9089076519e-01
       coeff_l4(47,17)= -4.2217394710e-01
       coeff_l4(47,18)=  4.5216313004e-01
       coeff_l4(47,19)= -9.6578204632e-01
       coeff_l4(47,20)=  1.4954812825e-01
       coeff_l4(47,21)= -2.5643894076e-01
       coeff_l4(48,1)= -1.2809476852e+00
       coeff_l4(48,2)=  1.2363698483e+00
       coeff_l4(48,3)=  6.0723584145e-02
       coeff_l4(48,4)=  1.7690099776e-01
       coeff_l4(48,5)= -3.7625488639e-01
       coeff_l4(48,6)= -1.3297058344e+00
       coeff_l4(48,7)= -2.1393081546e-01
       coeff_l4(48,8)=  1.4452482462e+00
       coeff_l4(48,9)= -2.2017002106e-02
       coeff_l4(48,10)=  6.6952407360e-01
       coeff_l4(48,11)= -9.3123918772e-01
       coeff_l4(48,12)= -3.0236557126e-01
       coeff_l4(48,13)=  9.3843829632e-01
       coeff_l4(48,14)= -1.1973317014e-03
       coeff_l4(48,15)= -7.5323112309e-02
       coeff_l4(48,16)= -6.5494012833e-01
       coeff_l4(48,17)= -1.6571564674e+00
       coeff_l4(48,18)=  9.1682499647e-01
       coeff_l4(48,19)= -2.1738921106e-01
       coeff_l4(48,20)= -4.9511575699e-01
       coeff_l4(48,21)=  4.5509806275e-01
       coeff_l4(49,1)=  7.8514540195e-01
       coeff_l4(49,2)=  3.9760151505e-01
       coeff_l4(49,3)=  7.8959673643e-01
       coeff_l4(49,4)=  4.3705061078e-01
       coeff_l4(49,5)=  1.5197141469e-01
       coeff_l4(49,6)=  4.3160352111e-01
       coeff_l4(49,7)=  4.6767745167e-02
       coeff_l4(49,8)=  2.4990009144e-02
       coeff_l4(49,9)=  2.2612354159e-01
       coeff_l4(49,10)= -6.2018412352e-01
       coeff_l4(49,11)=  5.8718991280e-01
       coeff_l4(49,12)= -1.0792572498e+00
       coeff_l4(49,13)= -4.3161368370e-01
       coeff_l4(49,14)=  3.4328418970e-01
       coeff_l4(49,15)=  5.4808241129e-01
       coeff_l4(49,16)=  2.1098883450e-01
       coeff_l4(49,17)= -4.4250789285e-01
       coeff_l4(49,18)=  1.1790463924e+00
       coeff_l4(49,19)=  1.6624894738e-01
       coeff_l4(49,20)= -4.7735768557e-01
       coeff_l4(49,21)= -6.4851182699e-01
       coeff_l4(50,1)= -1.7743065357e+00
       coeff_l4(50,2)= -1.5720088482e+00
       coeff_l4(50,3)=  6.8681502342e-01
       coeff_l4(50,4)= -2.1379169822e-01
       coeff_l4(50,5)= -5.1919043064e-01
       coeff_l4(50,6)= -2.1248345375e+00
       coeff_l4(50,7)=  1.3850462437e-01
       coeff_l4(50,8)=  3.7771505117e-01
       coeff_l4(50,9)=  1.2465373278e+00
       coeff_l4(50,10)=  4.6665862203e-01
       coeff_l4(50,11)= -1.1004232615e-01
       coeff_l4(50,12)=  1.9123069048e+00
       coeff_l4(50,13)= -6.1197239161e-01
       coeff_l4(50,14)= -2.9481686652e-02
       coeff_l4(50,15)= -1.7240813375e-01
       coeff_l4(50,16)= -3.9383125305e-01
       coeff_l4(50,17)=  3.8041591644e-02
       coeff_l4(50,18)= -5.4692041874e-01
       coeff_l4(50,19)=  2.6592352986e-01
       coeff_l4(50,20)= -1.0592248440e+00
       coeff_l4(50,21)= -3.3451351523e-01
       coeff_l4(51,1)= -1.2563241720e+00
       coeff_l4(51,2)= -7.1086555719e-01
       coeff_l4(51,3)= -1.1144345999e+00
       coeff_l4(51,4)= -7.7701455355e-01
       coeff_l4(51,5)= -1.3234441280e+00
       coeff_l4(51,6)= -9.7611898184e-01
       coeff_l4(51,7)=  1.8320073187e-01
       coeff_l4(51,8)= -7.1665868163e-02
       coeff_l4(51,9)= -1.2056792527e-01
       coeff_l4(51,10)=  3.4149202704e-01
       coeff_l4(51,11)= -2.7319869399e-01
       coeff_l4(51,12)=  8.5812705755e-01
       coeff_l4(51,13)= -4.4105628133e-01
       coeff_l4(51,14)= -2.1705685556e-01
       coeff_l4(51,15)= -7.2795116901e-01
       coeff_l4(51,16)= -1.6090433300e-01
       coeff_l4(51,17)=  1.2289962173e-01
       coeff_l4(51,18)=  9.1298080981e-02
       coeff_l4(51,19)=  5.5368274450e-01
       coeff_l4(51,20)= -4.2391067743e-01
       coeff_l4(51,21)= -7.4901282787e-02
       coeff_l4(52,1)=  2.5744276047e+00
       coeff_l4(52,2)=  8.5351485014e-01
       coeff_l4(52,3)=  7.4982696772e-01
       coeff_l4(52,4)=  1.0618960857e+00
       coeff_l4(52,5)=  7.1170520782e-01
       coeff_l4(52,6)=  1.2522163391e+00
       coeff_l4(52,7)=  9.2130762339e-01
       coeff_l4(52,8)=  3.0218020082e-01
       coeff_l4(52,9)= -7.2681105137e-01
       coeff_l4(52,10)= -2.6276987791e-01
       coeff_l4(52,11)=  6.6090655327e-01
       coeff_l4(52,12)= -1.2627276182e+00
       coeff_l4(52,13)= -7.7885860205e-01
       coeff_l4(52,14)=  3.7536889315e-01
       coeff_l4(52,15)= -1.3626408577e-01
       coeff_l4(52,16)=  6.3694232702e-01
       coeff_l4(52,17)=  7.1590298414e-01
       coeff_l4(52,18)=  1.1309399456e-01
       coeff_l4(52,19)= -3.0663943291e-01
       coeff_l4(52,20)=  7.3535287380e-01
       coeff_l4(52,21)=  2.1498357877e-02
       coeff_l4(53,1)= -1.7867730856e+00
       coeff_l4(53,2)= -1.7667531967e+00
       coeff_l4(53,3)= -4.9858263135e-01
       coeff_l4(53,4)= -1.6527277231e+00
       coeff_l4(53,5)= -6.0167711973e-01
       coeff_l4(53,6)= -1.2565886974e+00
       coeff_l4(53,7)=  1.5790797770e-01
       coeff_l4(53,8)= -3.2877492905e-01
       coeff_l4(53,9)=  3.9308965206e-01
       coeff_l4(53,10)=  2.8022080660e-01
       coeff_l4(53,11)=  1.0484684706e+00
       coeff_l4(53,12)=  8.3907407522e-01
       coeff_l4(53,13)=  9.3997381628e-02
       coeff_l4(53,14)= -5.1335448027e-01
       coeff_l4(53,15)= -4.6207857132e-01
       coeff_l4(53,16)= -6.1007225513e-01
       coeff_l4(53,17)=  3.9091449976e-01
       coeff_l4(53,18)= -6.0739505291e-01
       coeff_l4(53,19)=  4.6088111401e-01
       coeff_l4(53,20)= -4.3761906028e-01
       coeff_l4(53,21)=  1.8172030151e-01
       coeff_l4(54,1)=  3.9223148823e+00
       coeff_l4(54,2)=  5.0862736702e+00
       coeff_l4(54,3)=  1.2844684124e+00
       coeff_l4(54,4)=  2.0681333542e+00
       coeff_l4(54,5)=  2.6451902390e+00
       coeff_l4(54,6)=  2.7005996704e+00
       coeff_l4(54,7)= -7.0125722885e-01
       coeff_l4(54,8)=  6.1386179924e-01
       coeff_l4(54,9)= -1.2997038662e-01
       coeff_l4(54,10)= -1.1401250362e+00
       coeff_l4(54,11)=  5.4260998964e-01
       coeff_l4(54,12)= -2.7649736404e+00
       coeff_l4(54,13)=  1.1057358980e+00
       coeff_l4(54,14)=  4.3297287822e-01
       coeff_l4(54,15)=  1.3819441795e+00
       coeff_l4(54,16)=  3.1604403257e-01
       coeff_l4(54,17)= -9.2968386412e-01
       coeff_l4(54,18)=  1.4390199184e+00
       coeff_l4(54,19)= -6.9703614712e-01
       coeff_l4(54,20)=  1.8329900503e+00
       coeff_l4(54,21)= -6.7061048746e-01
       coeff_l4(55,1)=  1.0195692778e+00
       coeff_l4(55,2)=  7.3064994812e-01
       coeff_l4(55,3)= -1.0784001350e+00
       coeff_l4(55,4)=  2.2037296295e+00
       coeff_l4(55,5)=  5.9648627043e-01
       coeff_l4(55,6)= -2.1000978351e-01
       coeff_l4(55,7)=  4.6154150367e-01
       coeff_l4(55,8)=  1.0676934570e-01
       coeff_l4(55,9)=  2.2932134569e-01
       coeff_l4(55,10)= -7.7344447374e-02
       coeff_l4(55,11)= -1.4585281610e+00
       coeff_l4(55,12)=  1.7478238344e+00
       coeff_l4(55,13)=  8.6739800870e-02
       coeff_l4(55,14)=  3.9165747166e-01
       coeff_l4(55,15)= -1.2795878947e-01
       coeff_l4(55,16)=  4.5006594062e-01
       coeff_l4(55,17)=  1.1685959101e+00
       coeff_l4(55,18)= -3.4276470542e-01
       coeff_l4(55,19)= -4.7552981973e-01
       coeff_l4(55,20)=  1.5609996021e-01
       coeff_l4(55,21)= -5.4049164057e-01
       coeff_l4(56,1)=  2.2460639477e-01
       coeff_l4(56,2)=  1.9938559532e+00
       coeff_l4(56,3)=  2.9806798697e-01
       coeff_l4(56,4)=  1.3764810562e-01
       coeff_l4(56,5)=  1.1957950145e-01
       coeff_l4(56,6)=  1.2013952732e+00
       coeff_l4(56,7)=  2.3609159887e-01
       coeff_l4(56,8)= -2.5195658207e-01
       coeff_l4(56,9)=  1.3540825844e+00
       coeff_l4(56,10)= -1.7516105175e+00
       coeff_l4(56,11)= -2.6554718614e-01
       coeff_l4(56,12)= -8.6767560244e-01
       coeff_l4(56,13)= -5.7757806778e-01
       coeff_l4(56,14)= -8.4251266718e-01
       coeff_l4(56,15)= -6.9793885946e-01
       coeff_l4(56,16)= -1.6644358635e+00
       coeff_l4(56,17)=  2.7796775103e-02
       coeff_l4(56,18)=  1.2703007460e+00
       coeff_l4(56,19)=  9.9875319004e-01
       coeff_l4(56,20)=  1.7988856137e-01
       coeff_l4(56,21)=  8.0497972667e-02
!bias layer 0
       bias_l0=0.0d0
!bias layer 1
       bias_l1=0.0d0
!bias layer 2
       bias_l2=0.0d0
!bias layer 3
       bias_l3=0.0d0
!bias layer 4
       bias_l4=0.0d0

  endsubroutine assign_coeff_bias

      subroutine ev2gm2m(dist,v2b,g2b,imol,igrad,n)

      implicit none

      integer,intent(in) :: imol, igrad
      integer,intent(in) :: n
      double precision,intent(in)  :: dist(n*(n-1)/2)
      double precision,intent(out) :: v2b(6), g2b(6,3)
      double precision :: v(6), grad(6)
      double precision :: r
      integer :: i,j

      ! dist should be in bohr
      ! it is converted to be r in Ang
      ! v2b is in eV
      ! grad2b is in eV/bohr

      ! V0 is 120.17443 kcal/mol
      v2b = 120.17443d0/23.061d0
      g2b = 0.0d0
      do i=1,n*(n-1)/2
       r=dist(i)*0.529177d0
        call ev2gm2(r,v,grad,imol,igrad)
        v2b = v2b + v/23.061d0
        if (igrad.eq.1) then
          do j=1,6
            g2b(j,i) = grad(j)/(23.061d0/0.529177d0)
          enddo
        endif
      enddo

      end subroutine ev2gm2m

      subroutine ev2gm2(r,v,grad,imol,igrad) 
!**********************************************************************
!
! Compute the diatomic potential of groupd-state triplet O2
!
! References: J. Chem. Phys. 132, 074307 (2010)
!
! Input:  r      interatomic distance in Angstrom
! Output: V      potential in kcal/mol
!         grad   gradient (kcal/mol)/Angstrom
!
!**********************************************************************
      implicit none

      integer,intent(in) :: imol, igrad
      double precision,intent(in)  :: r
      double precision,intent(out) :: v(6), grad(6)
! Parameters of analytical even-tempered Gaussian expansions for the
! ground state potential energy curve of O2 CBS+SR+SO+CV.
! Units: alpha in Angstrom^-2, beta=dimensionless, as_k in milihartree.
      double precision :: alpha,beta,as(0:7)
      double precision :: gam, del
      integer :: i,j,k 
! Dispersion variables
      double precision :: dist(1),disp,dispdr(1)

      v=0.0d0
      grad=0.0d0

! list of considered states in 3A' symmetry
! V(1) - O2(3Sig_g^-) + O(3P)
! V(2) - O2(3Sig_g^-) + O(3P)
! V(3) - O2(3Del_u)   + O(3P)
! V(4) - O2(3Del_u)   + O(3P)
! V(5) - O2(3Del_u)   + O(3P)
! V(6) - O2(3Sig_u^+) + O(3P)


! Calculate dispersion correction for all states
        dist(1)=r
        call d3disp(dist,disp,dispdr,1,imol)

      do i=1,6
      if ((i.eq.1).or.(i.eq.2)) then
      ! Original parameters
      !      alpha = 0.785d0
      !      beta = 1.307d0
      !      as(0) = -2388.5641690d0
      !      as(1) = 18086.977116d0
      !      as(2) = -71760.197585d0
      !      as(3) = 154738.09175d0
      !      as(4) = -215074.85646d0
      !      as(5) = 214799.54567d0
      !      as(6) = -148395.42850d0
      !      as(7) = 73310.781453d0

      ! O2 X 3Sig_g^-
      ! Modified parameters for D3(BJ)
       alpha =  9.91088352d-1
       beta =   1.35561616d0
       as(0) =  1.02827423d3
       as(1) = -1.04225661d4
       as(2) =  2.62510588d4
       as(3) = -3.57598376d4
       as(4) =  3.45354087d4
       as(5) = -1.54421054d4
       as(6) =  5.10053268d3
       as(7) =  1.52168614d4
      else if ((i.eq.3).or.(i.eq.4).or.(i.eq.5)) then
      ! O2 C 3Del_u
       alpha =  1.1139979735d0
       beta =   1.6318488356d0
       as(0) = -7.6701828300d2
       as(1) =  2.8934254900d3
       as(2) = -3.9642158400d3
       as(3) =  3.2389566800d4
       as(4) = -2.0351193900d5
       as(5) =  2.6278224100d6
       as(6) = -6.8125724300d7
       as(7) =  4.2103491800d9
      else if (i.eq.6) then
      ! O2 A 3Sig_u^+
       alpha =  1.2488894868d0
       beta =   1.6163073021d0
       as(0) = -8.6651952200d2
       as(1) =  3.5326508700d3
       as(2) = -2.9278231700d3
       as(3) =  2.3357316600d4
       as(4) = -1.1305780800d5
       as(5) =  1.8807183400d6
       as(6) = -7.3167704600d7
       as(7) =  6.2456744200d9
      endif

      do k=0,7
       v(i)= v(i) + as(k)*dexp(-alpha*beta**k*r**2)
      enddo
      !if ((i.eq.1).or.(i.eq.2)) then
! From milihartree to kcal/mol
       !v(i)=v(i)*627.509523475149d-3
      !endif

! Add D3 dispersion correction
        v(i)=v(i)+disp

! Compute the gradient if needed
      if (igrad.eq.1) then
       do k=0,7
        grad(i)=grad(i)-2.d0*as(k)*alpha*beta**k*r*dexp(-alpha*beta**k*r**2)
       enddo
       !if ((i.eq.1).or.(i.eq.2)) then
! Convert from milihartree/A to (kcal/mol)/A
        !grad(i)=grad(i)*627.509523475149d-3
       !endif
        grad(i)= grad(i) + dispdr(1)
      endif

      enddo

      return
      end subroutine ev2gm2

      subroutine d3disp(dist,disp,dispdr,igrad,imol)
!**********************************************************************
! Dispersion correction based on Grimme's D3(BJ) calculation for
! diatomic pairs
!
! Several subroutines of DFTD3 V3.1 Rev 1 by Grimme were merged into 
! subroutine edisp and they have been heavily modified to calculate only
! that dispersion energy correction that is needed.
!
! S. Grimme, J. Antony, S. Ehrlich and H. Krieg
! J. Chem. Phys, 132 (2010), 154104
! and 
! S. Grimme, S. Ehrlich and L. Goerigk, J. Comput. Chem, 32 (2011),
! 1456-1465
!
! The C6 values are fixed.
!
!**********************************************************************

      double precision cn(2),s6,s8,rs6,rs8
      double precision dist(1), e6(1), e8(1), disp, dispdr(1), c6(2)
      double precision e6dr(1),e8dr(1)
      integer iz(2), mxc(94), i, j, igrad
      double precision c6ab(94,94,5,5,3)
      double precision r2r4(94)
      double precision autoang,autokcal
      integer imol

      autoang =0.52917726d0
      autokcal=627.509541d0

! Generalized parameters for BJ damping from P. Verma, B. Wang, 
! L. E. Fernandez, and D. G. Truhlar, J. Phys. Chem. A 121, 2855 (2017)
      s6= 1.0d0
      s8= 2.0d0
      rs6= 0.5299d0
      rs8= 2.20d0

      do i=1,1
      dist(i)=dist(i)/autoang
      enddo

      if (imol.eq.4) then
! iz for O2 system
      iz(1)=8
      iz(2)=8

! C6 for O2 system
      c6(1)=12.8d0
      c6(2)=12.8d0
      else
! currently imol = 4 is used
      stop
      endif

! Calculate dispersion correction
      call edisp(94,5,2,dist,iz,mxc, &
           rs6,rs8,e6,e8,e6dr,e8dr,c6,0)

      disp = 0.0d0

      do i=1,1
      disp =disp + (-s6*e6(i)-s8*e8(i))*autokcal
      enddo

      if (igrad .eq. 1) then
      call edisp(94,5,2,dist,iz,mxc, &
           rs6,rs8,e6,e8,e6dr,e8dr,c6,1)

      dispdr(:) = 0.0d0

      do i=1,1
      dispdr(i) =dispdr(i) + (-s6*e6dr(i)-s8*e8dr(i))*autokcal/autoang
      enddo
      endif

      do i=1,1
      dist(i)=dist(i)*autoang
      enddo

      end subroutine d3disp

!**********************************************************************
! compute energy
!**********************************************************************
      subroutine edisp(max_elem,maxc,n,dist,iz,mxc, &
                 rs6,rs8,e6,e8,e6dr,e8dr,c6a,igrad)

      integer n,iz(2),max_elem,maxc,mxc(max_elem) 
      double precision dist(1),r2r4(max_elem),r0ab(max_elem,max_elem)
      double precision rs6,rs8,rcov(max_elem)
      double precision c6ab(max_elem,max_elem,maxc,maxc,3)
      double precision e6(1), e8(1), c6a(2), e6dr(1), e8dr(1)
       
      integer iat,jat,igrad
      double precision r,tmp,c6,c8,a1,a2
      double precision damp6,damp8
      double precision cn(n)
      double precision r2ab(n*n),cc6ab(n*n),dmp(n*n)
      integer step

      e6(:) =0.0d0
      e8(:) =0.0d0

      e6dr(:) =0.0d0
      e8dr(:) =0.0d0

      a1=rs6
      a2=rs8 

!  r2r4 =sqrt(0.5*r2r4(i)*dfloat(i)**0.5 ) with i=elementnumber
!  the large number of digits is just to keep the results consistent
!  with older versions. They should not imply any higher accuracy than
!  the old values
      r2r4(1:94)=(/ &
      2.00734898d0,  1.56637132d0,  5.01986934d0,  3.85379032d0, &
      3.64446594d0,  3.10492822d0,  2.71175247d0,  2.59361680d0, &
      2.38825250d0,  2.21522516d0,  6.58585536d0,  5.46295967d0, &
      5.65216669d0,  4.88284902d0,  4.29727576d0,  4.04108902d0, &
      3.72932356d0,  3.44677275d0,  7.97762753d0,  7.07623947d0, &
      6.60844053d0,  6.28791364d0,  6.07728703d0,  5.54643096d0, &
      5.80491167d0,  5.58415602d0,  5.41374528d0,  5.28497229d0, &
      5.22592821d0,  5.09817141d0,  6.12149689d0,  5.54083734d0, &
      5.06696878d0,  4.87005108d0,  4.59089647d0,  4.31176304d0, &
      9.55461698d0,  8.67396077d0,  7.97210197d0,  7.43439917d0, &
      6.58711862d0,  6.19536215d0,  6.01517290d0,  5.81623410d0, &
      5.65710424d0,  5.52640661d0,  5.44263305d0,  5.58285373d0, &
      7.02081898d0,  6.46815523d0,  5.98089120d0,  5.81686657d0, &
      5.53321815d0,  5.25477007d0, 11.02204549d0,  0.15679528d0, &
      9.35167836d0,  9.06926079d0,  8.97241155d0,  8.90092807d0, &
      8.85984840d0,  8.81736827d0,  8.79317710d0,  7.89969626d0, &
      8.80588454d0,  8.42439218d0,  8.54289262d0,  8.47583370d0, &
      8.45090888d0,  8.47339339d0,  7.83525634d0,  8.20702843d0, &
      7.70559063d0,  7.32755997d0,  7.03887381d0,  6.68978720d0, &
      6.05450052d0,  5.88752022d0,  5.70661499d0,  5.78450695d0, &
      7.79780729d0,  7.26443867d0,  6.78151984d0,  6.67883169d0, &
      6.39024318d0,  6.09527958d0, 11.79156076d0, 11.10997644d0, &
      9.51377795d0,  8.67197068d0,  8.77140725d0,  8.65402716d0, &
      8.53923501d0,  8.85024712d0 /)

! these new data are scaled with k2=4./3.  and converted a_0 via
! autoang=0.52917726d0
      rcov(1:94)=(/ &
       0.80628308d0, 1.15903197d0, 3.02356173d0, 2.36845659d0, &
       1.94011865d0, 1.88972601d0, 1.78894056d0, 1.58736983d0, &
       1.61256616d0, 1.68815527d0, 3.52748848d0, 3.14954334d0, &
       2.84718717d0, 2.62041997d0, 2.77159820d0, 2.57002732d0, &
       2.49443835d0, 2.41884923d0, 4.43455700d0, 3.88023730d0, &
       3.35111422d0, 3.07395437d0, 3.04875805d0, 2.77159820d0, &
       2.69600923d0, 2.62041997d0, 2.51963467d0, 2.49443835d0, &
       2.54483100d0, 2.74640188d0, 2.82199085d0, 2.74640188d0, &
       2.89757982d0, 2.77159820d0, 2.87238349d0, 2.94797246d0, &
       4.76210950d0, 4.20778980d0, 3.70386304d0, 3.50229216d0, &
       3.32591790d0, 3.12434702d0, 2.89757982d0, 2.84718717d0, &
       2.84718717d0, 2.72120556d0, 2.89757982d0, 3.09915070d0, &
       3.22513231d0, 3.17473967d0, 3.17473967d0, 3.09915070d0, &
       3.32591790d0, 3.30072128d0, 5.26603625d0, 4.43455700d0, &
       4.08180818d0, 3.70386304d0, 3.98102289d0, 3.95582657d0, &
       3.93062995d0, 3.90543362d0, 3.80464833d0, 3.82984466d0, &
       3.80464833d0, 3.77945201d0, 3.75425569d0, 3.75425569d0, &
       3.72905937d0, 3.85504098d0, 3.67866672d0, 3.45189952d0, &
       3.30072128d0, 3.09915070d0, 2.97316878d0, 2.92277614d0, &
       2.79679452d0, 2.82199085d0, 2.84718717d0, 3.32591790d0, &
       3.27552496d0, 3.27552496d0, 3.42670319d0, 3.30072128d0, &
       3.47709584d0, 3.57788113d0, 5.06446567d0, 4.56053862d0, &
       4.20778980d0, 3.98102289d0, 3.82984466d0, 3.85504098d0, &
       3.88023730d0, 3.90543362d0 /)

! DFT-D3
      step=0
      do iat=1,n-1
         do jat=iat+1,n
         step=step+1
         r=dist(step)
         c6=c6a(step)
! r2r4 stored in main as sqrt
         c8 =3.0d0*c6*r2r4(iz(iat))*r2r4(iz(jat))

! energy for BJ damping
          tmp=sqrt(c8/c6)
          e6(step)= c6/(r**6+(a1*tmp+a2)**6)
          e8(step)= c8/(r**8+(a1*tmp+a2)**8)
! calculate gradients
         if (igrad .eq. 1) then
! grad for BJ damping
          e6dr(step)=c6*(-6*r**5)/(r**6+(a1*tmp+a2)**6)**2
          e8dr(step)=c8*(-8*r**7)/(r**8+(a1*tmp+a2)**8)**2
         endif
         enddo
      enddo

      end subroutine edisp


