!********************************************************************** 
!   System:                     OH3 (lowest 3 state)
!   Functional form:            permutationally invariant polynomials
!   Common name:                OH3 three states PES
!   Number of derivatives:      3
!   Number of bodies:           4
!   Number of electronic surfaces: 3
!   Interface: Section-2
!
!   References:  Yinan Shu, Joanna Kryven, Antonio Gustavo Sampaio de Oliveira Filho,
!                Steven L. Mielke, Linyao Zhang, Guo-Liang Song, Shaohong L. Li, 
!                Rubén Meana-Pañeda, Bina Fu, Joel M. Bowman, and Donald G. Truhlar
!                to be published
!
!   Notes:    -PES of OH3
!             -Including diabatic states, diabatic couplings, and transformed adiabatic states
!
!**********************************************************************

!***************************************************************************
! PES program for OH3 
!
! Authors: Yinan Shu, Antonio Gustavo Sampaio de Oliveira Filho
! Dec. 2018
!
! Reference for this potential energy surface:
!   "Direct  Diabatization and Analytic Representation of Coupled 
!    Potential Energy Surfaces and Couplings for the Reactive 
!    Quenching of the Excited 2Σ+ State of OH by Molecular Hydrogen"
!   by Yinan Shu, Joanna Kryven, Antonio Gustavo Sampaio de Oliveira Filho,
!      Steven L. Mielke, Guo-Liang Song, Shaohong L. Li, Rubén Meana-Pañeda,
!      Bina Fu, Joel Bowman, and Donald G. Truhlar
!   to be published
!
!***************************************************************************

!===========================================================================
! *def oh3_pes
!  1. Main PES program for calculating:
!     energies and gradients for diabatic states and diabatic couplings
!     energies, gradients and non-adiabatic couplings for adiabatic states 
!  2. The input file: ****.xyz (**** is a file name, for example, OH3;) 
!     !!!THE UNIT SHOULD BE ANGSTROM !!!
!     the format of ****.xyz should be a standard xyz format that is readbale
!     by Molden with 3 hydrogen atoms first, the oxygen at last, for example: 
!     4
!    
!     H  X Y Z 
!     H  X Y Z
!     H  X Y Z
!     O  X Y Z
!  3. Executing the program with the input, the program will generate 2 output 
!     they correspond to: ****.out and ****.tmp
!     ****.out has a detailed print out of energies, gradients and nonadiabatic 
!     couplings
!     ****.tmp has a three line simple  print of diabatic states, diabatic couplings and
!     adiabatic states in the following format
!     u(1,1) u(2,2) u(3,3)
!     u(1,2) u(1,3) u(2,3)
!     a(1)   a(2)   a(3)
!  4. The zero of energy is: -76.80042447 hartree
!
!  ==========================definition of variables========================
!
!  4. xyz: the xyz coordinate
!          unit: in angstroms
!          xyz is a 4 by 3 matrix
!  5.  u : the diabatic matrix
!          unit: in hartrees
!          u is a 3 by 3 matrix
!          notice u(i,j)=u(j,i) for i not equal to j 
!          the diabatic states are diagonal elements of u 
!          the diabatic couplings are off-diagonal elements of u 
!  6.  gu: the gradients of diabatic matrix w.r.t. the xyz coordinates
!          unit: in hartrees/angstrom
!          gu is a 4 by 3 by 3 by 3 matrix
!          "gu(:,:,i,j)" is the gradient of u(i,j)
!  7.  a : the adiabatic states
!          unit: in hartrees
!          a is a 3 element vector 
!  8.  ga: the gradient of adiabatic states w.r.t. the xyz coordinates
!          unit: in hartrees/angstrom
!          ga is a 4 by 3 by 3 matrix
!          "ga(:,:,1)" is the gradient of the ground adiabatic state
!          "ga(:,:,2)" is the gradient of the first excited adiabatic state
!          "ga(:,:,3)" is the gradient of the second excited adiabatic state
!  9.  f : the non-adiabatic couplings
!          unit: in angstrom**(-1)
!          f is a 4 by 3 by 3 by 3 matrix
!          "f(:,:,i,j)" is the non-adiabatic coupling of state i and j
!          f(:,:,i,j)=-f(:,:,j,i), and f(:,:,i,i)=0 
!  10. rv: the diabatic to adiabatic rotation matrix 
!===========================================================================

  subroutine pes(x,igrad,p,g,d)

  implicit none
  ! number of electronic state
  integer, parameter :: nstates=3
  integer, parameter :: natoms=4
  integer, intent(in) :: igrad
  double precision, intent(in) :: x(natoms,3)
  double precision, intent(out) :: p(nstates), g(nstates,natoms,3)
  double precision, intent(out) :: d(nstates,nstates,natoms,3) 

  character (len=1) :: symb
  integer :: iatom,idir,istate,jstate
  integer :: i,j,k,l
  real*8 :: u11, u22, u33
  real*8 :: u12, u13, u23
  real*8 :: s12, s23
  real*8 :: xyz(4,3)
  character(len=32)::fname, bname
  real*8 :: du11dx(4,3), du22dx(4,3), du33dx(4,3)
  real*8 :: du12dx(4,3), du13dx(4,3), du23dx(4,3)
  real*8 :: gu(4,3,3,3)
  real*8 :: u(3,3),rv(3,3),a(3)
  real*8 :: ga(4,3,3),f(4,3,3,3)
  real*8 :: tmpgu(3,3),tmpga(3,3)


   xyz=x

!compute diabatic states
  call u11_prepot_8
  call u11_pot_8(xyz,u11,du11dx)
  call u22_prepot_8
  call u22_pot_8(xyz,u22,du22dx)
  call u33_prepot_8
  call u33_pot_8(xyz,u33,du33dx)

!compute diabatic couplings
  call u12_order10
  call u13_order12
  call u23_order10
  call u12_dc_10(xyz,u12,s12)
  call u13_dc_12(xyz,u13)
  call u23_dc_10(xyz,u23,s23)

!compute the adiabatic states and eigenvectors
  u(1,1)=u11
  u(1,2)=u12
  u(1,3)=u13
  u(2,1)=u12
  u(2,2)=u22
  u(2,3)=u23
  u(3,1)=u13
  u(3,2)=u23
  u(3,3)=u33
  call dsyevj3(u,rv,a)
  call order(a,rv)


  p=(a+76.80042447)*27.211386

  if (igrad.ne.0) then 

  !compute diabatic couplings derivatives
  call u12_dvdx(xyz,u12,s12,du12dx)
  call u13_dvdx(xyz,u13,du13dx)
  call u23_dvdx(xyz,u23,s23,du23dx)

  !assign dudx to gu

  do i=1,4
  do j=1,3
  gu(i,j,1,1)=du11dx(i,j)
  gu(i,j,1,2)=du12dx(i,j)
  gu(i,j,1,3)=du13dx(i,j)
  gu(i,j,2,1)=du12dx(i,j)
  gu(i,j,2,2)=du22dx(i,j)
  gu(i,j,2,3)=du23dx(i,j)
  gu(i,j,3,1)=du13dx(i,j)
  gu(i,j,3,2)=du23dx(i,j)
  gu(i,j,3,3)=du33dx(i,j)
  enddo
  enddo

!initialize the ga and f matrices
  do i=1,4
  do j=1,3
  do k=1,3
  do l=1,3
  ga(i,j,k)=0.000
  f(i,j,k,l)=0.000
  enddo
  enddo
  enddo
  enddo

!compute the adiabatic states gradients,nonadiabatic coupling vectors
  do i=1,4
  do j=1,3
  tmpgu(1,1)=du11dx(i,j)
  tmpgu(1,2)=du12dx(i,j)
  tmpgu(1,3)=du13dx(i,j)
  tmpgu(2,1)=du12dx(i,j)
  tmpgu(2,2)=du22dx(i,j)
  tmpgu(2,3)=du23dx(i,j)
  tmpgu(3,1)=du13dx(i,j)
  tmpgu(3,2)=du23dx(i,j)
  tmpgu(3,3)=du33dx(i,j)
  !C^T*dU*C
  tmpga = matmul(transpose(rv), matmul(tmpgu, rv))
  !dV_i = (C^T*dU*C)_ii
  do k=1,3
  ga(i,j,k)=tmpga(k,k)
  enddo
  !F_ij = (C^T*dU*C)_ij / (V_j-V_i)
  do k=1,2
  do l=k+1,3
  f(i,j,k,l)=tmpga(k,l)/(a(l)-a(k))
  f(i,j,l,k)=-f(i,j,k,l)
  enddo
  enddo
  enddo
  enddo

    do iatom=1,4
    do idir=1,3
    do istate=1,3
      g(istate,iatom,idir)=ga(iatom,idir,istate)*27.211386
    enddo
    enddo
    enddo 

    do iatom=1,4
    do idir=1,3
    do istate=1,3
    do jstate=1,3
      d(istate,jstate,iatom,idir)=f(iatom,idir,istate,jstate)
    enddo
    enddo
    enddo
    enddo

  endif 

end
!===========================================================================
! *def u11_prepot_8
! subtoutine of computing u11 and gradient of u11
!===========================================================================
subroutine u11_prepot_8
implicit none
real*8 :: xyz(4,3)
real*8 :: v, r12, r13, r14, r23, r24, r34
real*8 :: a1, a2, b1, b2, oh, hh, ra1, ra2, rb1, rb2, grad_hh, grad_oh
integer, parameter :: ncoeff=546
real*8 :: coeff(ncoeff), p(ncoeff)
real*8 :: dpdx1(ncoeff), dpdx2(ncoeff), dpdx3(ncoeff) ,dpdx4(ncoeff), dpdx5(ncoeff) ,dpdx6(ncoeff)
real*8 :: x(6), re_hh, re_oh, ra_hh, ra_oh, rb_hh, rb_oh
real*8 :: dx1, dx2, dx3, dx4, dx5, dx6, dr1, dr2, dr3, dr4, dr5, dr6, dxyz(4,3)
integer ::  ncall
data ncall/1/
save ncall, coeff, a1, a2, b1, b2, ra1, ra2, rb1, rb2

if(ncall.eq.1) then

a1 = 8.0000000000000004E-01
a2 = 1.2000000000000000E+00
b1 = 1.0000000000000000E+00
b2 = 1.5000000000000000E+00
ra1 = 1.2000000000000000E+00
ra2 = 1.5000000000000000E+00
rb1 = 1.2000000000000000E+00
rb2 = 1.5000000000000000E+00
coeff(   1)=-7.646858051765578E+01
coeff(   2)=-2.1451400770501184E+01
coeff(   3)= 2.7072769910035312E+01
coeff(   4)=-9.6786173287777544E+00
coeff(   5)= 1.9445844168656353E+02
coeff(   6)=-2.6958034145985209E+01
coeff(   7)=-1.1214911772372406E+02
coeff(   8)=-2.1520189713114114E+01
coeff(   9)=-5.0821778951829977E+01
coeff(  10)= 5.2857820203265426E+01
coeff(  11)= 3.3739299488002587E+01
coeff(  12)=-4.0010770933905640E+01
coeff(  13)=-2.1550757476775729E+01
coeff(  14)= 1.0143636074663927E+01
coeff(  15)= 1.2392398721485830E+02
coeff(  16)= 9.9905777001253554E+01
coeff(  17)=-4.4337836671427141E+01
coeff(  18)= 1.0871513250412187E+02
coeff(  19)=-2.5147064869294664E+02
coeff(  20)=-8.7518162077413688E+00
coeff(  21)= 3.1816723481277087E+01
coeff(  22)= 1.2329518615269220E+02
coeff(  23)= 5.2497095626408068E+01
coeff(  24)=-1.0052107065182868E+02
coeff(  25)= 2.1769980003316164E+01
coeff(  26)= 8.1208052676071617E+01
coeff(  27)=-9.9356882305119036E+00
coeff(  28)=-1.9279304254449602E+01
coeff(  29)= 1.0466833289796725E+01
coeff(  30)=-3.1192096568627875E+01
coeff(  31)=-6.0900453068926097E+01
coeff(  32)=-1.3015659691708775E+01
coeff(  33)= 1.6093275902142093E+01
coeff(  34)= 3.5095151475466174E+01
coeff(  35)= 2.6712876075734332E+01
coeff(  36)= 3.3170057348093369E+00
coeff(  37)=-1.8408135917546993E+00
coeff(  38)=-1.2642926767010195E+01
coeff(  39)=-1.5539783947628968E+02
coeff(  40)=-6.6217417938541004E+01
coeff(  41)=-8.8567571342523109E+01
coeff(  42)=-1.2899360076868084E+02
coeff(  43)= 3.4142698575530339E+00
coeff(  44)= 7.4746779526150334E+01
coeff(  45)=-1.0693814554304920E+02
coeff(  46)=-1.0887005462022944E+02
coeff(  47)= 2.8727143160447469E+01
coeff(  48)= 1.6815952119614384E+01
coeff(  49)= 8.7358119648557988E+00
coeff(  50)= 5.9975339527871000E+01
coeff(  51)=-2.0412429181273328E+01
coeff(  52)= 2.9651238342024516E+02
coeff(  53)= 9.9039855637680219E+01
coeff(  54)= 1.4967297194083754E+01
coeff(  55)= 3.8699968071208929E+01
coeff(  56)=-1.2593066537629421E+02
coeff(  57)=-2.8590141743518110E+01
coeff(  58)=-4.2864098370334027E+01
coeff(  59)=-2.7103549434748203E+01
coeff(  60)= 1.2373509200119641E+02
coeff(  61)= 4.9869133969110351E+00
coeff(  62)=-1.1805543263273419E+01
coeff(  63)=-7.9482611808111926E+01
coeff(  64)= 5.2936923059192864E-01
coeff(  65)= 4.3203752270986229E+00
coeff(  66)= 3.3854960804777974E+01
coeff(  67)=-2.9915392598474625E+01
coeff(  68)=-5.5523741145874936E+00
coeff(  69)=-6.0715537715632983E+00
coeff(  70)=-1.0758031350335282E+01
coeff(  71)=-1.3617130671796353E+01
coeff(  72)= 1.1116191452322997E+01
coeff(  73)= 1.3445670189406714E+01
coeff(  74)=-1.7788520676524620E+01
coeff(  75)= 1.5740275118473471E+01
coeff(  76)= 3.2429010505674164E+01
coeff(  77)=-4.1858062773991689E-01
coeff(  78)=-5.1513882270067790E-01
coeff(  79)=-2.1470749522967395E+01
coeff(  80)=-1.6960494805480501E+01
coeff(  81)=-1.4012093071809634E+01
coeff(  82)= 7.0090106430881400E+00
coeff(  83)=-3.1457364560208569E+00
coeff(  84)= 1.4578772245811967E+02
coeff(  85)= 2.8589486926965236E+01
coeff(  86)=-2.8944288667062921E+00
coeff(  87)= 1.0406967292807153E+02
coeff(  88)=-2.6541502394047725E+01
coeff(  89)= 2.2487250548703770E+00
coeff(  90)= 1.2496274321020175E+01
coeff(  91)= 5.9797084678526716E+01
coeff(  92)=-4.7286445210137362E-01
coeff(  93)= 8.2479934889757317E+01
coeff(  94)= 5.1987299188569853E+00
coeff(  95)= 1.0531196932561376E+02
coeff(  96)= 7.4701036803503300E+01
coeff(  97)= 9.5828558351475266E+00
coeff(  98)=-4.1825664303996419E+01
coeff(  99)=-5.1643616063223199E+01
coeff( 100)= 4.4569876054547990E+01
coeff( 101)= 4.4202482897060150E+01
coeff( 102)= 8.3369581957505460E+01
coeff( 103)=-2.4663220634064512E+01
coeff( 104)=-6.7754886842269197E+00
coeff( 105)=-2.2199211620764615E+01
coeff( 106)= 1.1215418710973012E+01
coeff( 107)=-3.5619066065550371E+01
coeff( 108)=-8.0961388443564601E+00
coeff( 109)=-4.5052684988021511E+01
coeff( 110)= 2.0127093457514277E+01
coeff( 111)= 3.3328621562467546E+01
coeff( 112)=-5.9043568391585524E+00
coeff( 113)= 6.0846150198060514E+00
coeff( 114)= 2.2932563040525977E+00
coeff( 115)= 5.1819537472624786E+00
coeff( 116)=-2.3828398061128084E+01
coeff( 117)= 5.3980514712282250E+00
coeff( 118)= 7.2791252249636589E+00
coeff( 119)=-1.8086371472115971E+01
coeff( 120)= 9.7052340480061670E+00
coeff( 121)=-2.6507551547246942E+02
coeff( 122)=-1.1327372526212795E+02
coeff( 123)=-3.1490524048640172E+00
coeff( 124)=-5.8887459180836856E+00
coeff( 125)=-5.4074810227907868E+01
coeff( 126)= 1.0654491416466700E+01
coeff( 127)= 4.2007317711924244E+01
coeff( 128)= 1.7080869007356540E+01
coeff( 129)= 6.7112227594890275E+00
coeff( 130)= 1.1097211829147483E+01
coeff( 131)=-7.9287773484068623E+01
coeff( 132)=-6.1523465933947232E+00
coeff( 133)=-2.3790965035537649E+01
coeff( 134)= 5.9060709915259075E+01
coeff( 135)=-1.5993217388437463E+01
coeff( 136)= 2.3299459740426773E+01
coeff( 137)=-1.8612873914940693E+01
coeff( 138)=-2.7825722054210758E+00
coeff( 139)=-3.1586896355080434E+01
coeff( 140)= 2.9128435634203889E+01
coeff( 141)=-5.1888167924576418E+00
coeff( 142)= 1.3145318484083539E+00
coeff( 143)= 1.5363826220437933E+00
coeff( 144)= 2.2694086829712706E+01
coeff( 145)= 1.4098427849026914E+01
coeff( 146)= 4.4091876193338209E-01
coeff( 147)=-1.2720152614840877E+01
coeff( 148)=-6.0546043307014319E+00
coeff( 149)= 1.9892806225929536E+00
coeff( 150)= 2.9543403454378092E+00
coeff( 151)= 6.1143131204900723E+00
coeff( 152)= 3.5231087297525767E+00
coeff( 153)= 2.0590805786078246E+00
coeff( 154)= 5.7641563272828753E+00
coeff( 155)=-4.7338004809005669E+00
coeff( 156)= 8.2484338233681793E+00
coeff( 157)= 1.7366379402479307E+01
coeff( 158)=-1.4697762531789877E+01
coeff( 159)=-1.9587811348178374E+01
coeff( 160)=-6.4471963980509308E+00
coeff( 161)= 6.4935526436097191E-01
coeff( 162)=-4.1797074799381813E+00
coeff( 163)= 1.1896578837119614E+01
coeff( 164)= 2.1681616714782976E+01
coeff( 165)= 1.5408833531505687E+01
coeff( 166)=-7.5707278350564042E+00
coeff( 167)=-3.0083065326571390E+00
coeff( 168)=-5.2255848921048935E+00
coeff( 169)= 2.6684308347172627E+00
coeff( 170)=-6.2409404414882388E+01
coeff( 171)= 1.1210790558069709E+01
coeff( 172)= 2.9751852030017325E+00
coeff( 173)=-1.5134603758607419E+01
coeff( 174)=-7.2380498950535070E+00
coeff( 175)=-2.7646967180988788E+01
coeff( 176)= 1.0623077069745648E+01
coeff( 177)=-3.7551545363263052E+01
coeff( 178)=-4.4032492648247761E+00
coeff( 179)= 1.6468066518844864E+01
coeff( 180)= 4.8192795531577239E+00
coeff( 181)= 1.8299284005083756E+00
coeff( 182)=-1.2288292879708557E+01
coeff( 183)=-1.3671372353149547E+00
coeff( 184)= 1.2547997698608627E+01
coeff( 185)=-1.2339085708554373E+01
coeff( 186)=-9.7994379447302666E+00
coeff( 187)= 7.6392025105089107E+00
coeff( 188)=-4.4501804348894058E-01
coeff( 189)=-1.5183175020093389E+01
coeff( 190)=-2.8081615527073925E+01
coeff( 191)=-7.1441768693258378E+00
coeff( 192)=-4.6821999369351621E+00
coeff( 193)= 2.4943584139146999E+01
coeff( 194)=-1.7556413872485493E+01
coeff( 195)= 1.5914182080543289E+00
coeff( 196)=-4.7559160138663380E+01
coeff( 197)=-2.0306439574082432E+01
coeff( 198)=-1.4256109674043635E+00
coeff( 199)= 1.2460491168908206E+01
coeff( 200)= 1.8813697014528966E+01
coeff( 201)= 1.6642542467884518E+01
coeff( 202)=-7.9568250550216906E+00
coeff( 203)=-3.4646821994052224E+00
coeff( 204)=-2.5234505269710994E+01
coeff( 205)=-4.6153875161082134E+01
coeff( 206)=-1.6063038951433381E+01
coeff( 207)= 2.4343812296031263E+01
coeff( 208)= 1.2344289518434865E+01
coeff( 209)=-3.6975710585226738E+00
coeff( 210)= 1.0526114432608818E+01
coeff( 211)=-2.3563196227467316E+01
coeff( 212)= 1.3053291031685326E+01
coeff( 213)= 1.2805669220687932E+01
coeff( 214)= 8.0076374406510595E+00
coeff( 215)= 1.0505537068758564E-02
coeff( 216)= 2.4895940938924507E+01
coeff( 217)= 2.1404095127101774E-01
coeff( 218)=-1.7823885585131919E+01
coeff( 219)=-5.5399298202147973E-01
coeff( 220)=-5.4763892180312190E+00
coeff( 221)= 1.1984198444009674E+01
coeff( 222)= 7.3730209446267700E+00
coeff( 223)=-1.9680207875117790E+00
coeff( 224)= 2.8377261371857427E+00
coeff( 225)=-4.8070383229036260E+00
coeff( 226)= 2.2844963615908265E+00
coeff( 227)= 1.2658506251405598E+01
coeff( 228)= 1.6312024094806885E+00
coeff( 229)= 4.5727478139706399E+00
coeff( 230)= 6.3617469435543956E+00
coeff( 231)=-9.1501785015362671E+00
coeff( 232)= 8.9390435184438957E+00
coeff( 233)=-9.6596109077473926E+00
coeff( 234)= 1.5108437868313884E+00
coeff( 235)= 5.7707604863430060E+00
coeff( 236)=-6.3255556354490263E+00
coeff( 237)=-7.1024879835461414E+00
coeff( 238)= 3.1000772974024313E+00
coeff( 239)= 2.9019743978382930E+00
coeff( 240)=-3.1239380155867789E+00
coeff( 241)=-2.0543724028463908E+00
coeff( 242)=-1.3836083225355381E+01
coeff( 243)= 1.9002060558849903E+00
coeff( 244)= 5.3954556897219073E+00
coeff( 245)= 1.4784287137564838E+00
coeff( 246)= 7.1356643822921598E+01
coeff( 247)= 4.3064156898134399E+01
coeff( 248)= 1.0810901114348741E+01
coeff( 249)=-4.1808151740923263E+00
coeff( 250)= 3.0487055219510149E+00
coeff( 251)= 1.8739414937640849E+01
coeff( 252)= 1.7780943839448710E+01
coeff( 253)=-3.2868162639815232E+01
coeff( 254)=-3.0555231502778124E+01
coeff( 255)= 1.0392548871429746E+01
coeff( 256)= 1.1778824499099123E+00
coeff( 257)=-7.3243982978228175E+00
coeff( 258)=-1.1121414614126035E+01
coeff( 259)= 2.1756572098420673E+01
coeff( 260)=-5.0589105342434693E-01
coeff( 261)= 1.2819917922635245E+01
coeff( 262)= 1.8892591516648373E+01
coeff( 263)=-1.2683029807006999E+01
coeff( 264)= 1.4952995771777555E+01
coeff( 265)=-9.6158877440374724E-01
coeff( 266)= 2.3670614466352610E+01
coeff( 267)= 1.4585484117506242E+00
coeff( 268)= 1.9118400092032548E+01
coeff( 269)=-1.1623215822422608E+01
coeff( 270)=-7.0349101221540451E+00
coeff( 271)=-1.4369391744639737E-01
coeff( 272)=-4.5169773100394179E+00
coeff( 273)=-5.5135454079923640E+00
coeff( 274)=-1.4734897982829931E+01
coeff( 275)=-6.4336996113096996E+00
coeff( 276)=-7.8577762286676700E+00
coeff( 277)= 5.0069598886986606E+00
coeff( 278)=-4.0595045641423217E+00
coeff( 279)= 1.7202499727198293E+00
coeff( 280)= 7.1248757827933840E-01
coeff( 281)=-5.2657918649247542E+00
coeff( 282)= 2.8227668944642685E+00
coeff( 283)=-5.9144692553602152E+00
coeff( 284)= 3.2879629876102916E+00
coeff( 285)=-2.0611424918868240E+00
coeff( 286)=-2.9197762537331606E-01
coeff( 287)=-2.2345887707076004E+00
coeff( 288)=-3.0874271648414839E+00
coeff( 289)= 3.2063624037350564E+00
coeff( 290)=-5.8522968797090735E-01
coeff( 291)= 7.3060422145086630E+00
coeff( 292)= 3.4016431893376065E+00
coeff( 293)= 1.1760736989871817E-02
coeff( 294)=-4.1874187711745403E+00
coeff( 295)=-2.1423794915135295E+00
coeff( 296)= 2.0399584698794332E-01
coeff( 297)=-7.5350126880069657E-01
coeff( 298)= 1.5634316171524154E+00
coeff( 299)=-5.1070187073488107E+00
coeff( 300)=-4.5238278398695204E+00
coeff( 301)= 3.7576048429978761E+00
coeff( 302)= 2.8941672788648085E+00
coeff( 303)= 2.6694760951485428E+00
coeff( 304)=-1.1047615443535070E+00
coeff( 305)= 4.8948636683201014E+00
coeff( 306)=-7.4265084246515052E+00
coeff( 307)=-1.0686387044871417E+01
coeff( 308)= 1.5381741020556396E+00
coeff( 309)=-2.6228936858493928E+00
coeff( 310)= 4.3919563852810555E+00
coeff( 311)= 3.7572851618324847E+00
coeff( 312)= 1.4566883734258063E+00
coeff( 313)=-1.4695822675705548E+00
coeff( 314)= 1.6475139208516541E+01
coeff( 315)= 5.7926857275737422E+00
coeff( 316)=-6.1721803595672178E+00
coeff( 317)=-2.7775893042430058E+00
coeff( 318)=-2.6703315530767715E+00
coeff( 319)= 8.8352938667989314E+00
coeff( 320)=-3.2990722793547960E+00
coeff( 321)= 3.0754034261671626E+00
coeff( 322)= 2.9436466178210980E+00
coeff( 323)=-4.6600612514066775E-02
coeff( 324)= 3.7049602780404625E+00
coeff( 325)=-1.6093340523444843E-01
coeff( 326)=-1.0118565172200441E+01
coeff( 327)= 7.8237381228650511E+00
coeff( 328)= 7.6707258052540519E+00
coeff( 329)=-1.6122467608800234E-01
coeff( 330)=-3.5317378463537232E+00
coeff( 331)=-3.3145170296824413E+00
coeff( 332)=-4.8449882527260115E+00
coeff( 333)=-4.3095625430631852E+00
coeff( 334)= 9.2987141773065556E+00
coeff( 335)=-3.2401862047763652E+00
coeff( 336)= 5.7915747076774551E+00
coeff( 337)=-8.4092505355132197E-01
coeff( 338)= 3.6322994320548165E+00
coeff( 339)=-2.3271527233864493E+00
coeff( 340)= 8.5558785914603135E+00
coeff( 341)= 7.6916730337736912E+00
coeff( 342)=-3.6416393255873429E+00
coeff( 343)=-7.4336461290671121E+00
coeff( 344)=-3.5137371531007151E+00
coeff( 345)=-3.3537170474326925E+00
coeff( 346)=-2.1506145033629962E+00
coeff( 347)=-4.9652933136108377E+00
coeff( 348)= 3.7974020978251941E+00
coeff( 349)=-8.6428737435873981E-02
coeff( 350)= 3.9131905409374546E-01
coeff( 351)= 1.2419906696563059E+00
coeff( 352)=-1.1652661206333956E+00
coeff( 353)=-2.4553587029835215E+00
coeff( 354)= 4.7004347056570293E+00
coeff( 355)=-2.2074219481168478E+00
coeff( 356)= 2.6661695805063306E+00
coeff( 357)=-8.4064794881249405E-02
coeff( 358)= 2.6322740039169457E+00
coeff( 359)= 5.6013801028172017E+00
coeff( 360)= 3.3913686327375911E+00
coeff( 361)=-6.9374603349803809E+00
coeff( 362)=-3.3217258757963219E+00
coeff( 363)= 1.7198676773315182E+01
coeff( 364)=-2.4575355173160700E+00
coeff( 365)=-8.3094617611827122E+00
coeff( 366)= 3.8369574009725333E+00
coeff( 367)= 4.4748506031980604E+00
coeff( 368)= 1.2627060882074685E+01
coeff( 369)= 2.3989148731636813E+00
coeff( 370)= 1.2119956566997350E+00
coeff( 371)=-6.8286828171228073E+00
coeff( 372)= 2.9389717973772149E+00
coeff( 373)=-6.9120289956490053E+00
coeff( 374)= 1.0057925627549296E+00
coeff( 375)= 8.9308322344809898E-03
coeff( 376)= 1.7644879331637719E-01
coeff( 377)= 1.8342578427208760E+00
coeff( 378)= 7.4209003312179203E+00
coeff( 379)= 3.2013426351395058E+00
coeff( 380)= 5.3966396905494101E+00
coeff( 381)=-1.2780286813740838E-01
coeff( 382)=-1.1877367216301854E+01
coeff( 383)=-3.6991021404674669E+00
coeff( 384)= 1.3809815510984458E+00
coeff( 385)=-7.8497751611612998E-01
coeff( 386)= 1.0183587195946244E+00
coeff( 387)= 8.5731486622256448E+00
coeff( 388)=-1.1379340447696649E+00
coeff( 389)=-2.0855847698739156E+00
coeff( 390)= 1.0775969277044453E-02
coeff( 391)=-6.5914106178066989E-01
coeff( 392)=-9.2673872421555159E-01
coeff( 393)=-1.0128261782245929E+00
coeff( 394)=-7.2563464813149512E+00
coeff( 395)=-2.3503865307080778E+00
coeff( 396)=-4.2080573374488868E+00
coeff( 397)= 3.9165633212619051E+00
coeff( 398)=-1.0507963161956173E+00
coeff( 399)= 1.7323744037277844E+00
coeff( 400)=-4.3322323596184722E+00
coeff( 401)=-1.4010719537562111E+00
coeff( 402)= 1.3519217742500196E+00
coeff( 403)= 3.4020768002591653E+00
coeff( 404)=-3.4675267233881111E+00
coeff( 405)= 1.1697409226202538E-01
coeff( 406)= 3.0828707652666631E+00
coeff( 407)= 1.6390269349492415E-01
coeff( 408)= 3.7157222234298883E+00
coeff( 409)=-4.9534768075684177E+00
coeff( 410)= 4.1447524368804201E-01
coeff( 411)=-2.7639496951788427E+00
coeff( 412)=-3.3807561439420377E+00
coeff( 413)=-3.5349840588693371E+00
coeff( 414)= 1.6852420235974586E+00
coeff( 415)= 6.3523656411424079E-01
coeff( 416)= 1.7808846577170245E-01
coeff( 417)=-2.9239520958512917E+00
coeff( 418)= 1.5125729859481993E+00
coeff( 419)= 5.1152787399756416E-01
coeff( 420)= 2.2705643193189129E+00
coeff( 421)=-4.5906578034386780E-01
coeff( 422)= 1.3542212345412479E+00
coeff( 423)=-1.0620190356114125E+00
coeff( 424)= 2.9888834644758955E+00
coeff( 425)=-3.7631074095914037E+00
coeff( 426)= 1.5480471534601153E-01
coeff( 427)=-4.3915205713293309E-01
coeff( 428)= 2.8000991074438475E+00
coeff( 429)=-7.6486336498548768E-01
coeff( 430)=-1.5302268553987461E-01
coeff( 431)=-4.5234316096532495E-01
coeff( 432)= 3.7198186113327375E-01
coeff( 433)=-2.0904573704208440E+00
coeff( 434)= 1.5805049539886062E+00
coeff( 435)= 1.3945543705574448E-01
coeff( 436)= 2.2819830657818949E+00
coeff( 437)=-7.5405727289377733E-01
coeff( 438)= 4.3744197468035493E+00
coeff( 439)=-1.3600279994513302E+00
coeff( 440)= 1.8553472841360508E+00
coeff( 441)= 1.1042468392083333E+00
coeff( 442)=-3.7404249935243832E+00
coeff( 443)= 1.2207986322436779E+00
coeff( 444)=-2.3319493576568067E+00
coeff( 445)=-4.1180717862015243E-01
coeff( 446)=-9.7981319537978928E-01
coeff( 447)= 1.6223782069150867E+00
coeff( 448)=-9.9891836608911377E-01
coeff( 449)= 6.4983536852920942E-01
coeff( 450)= 6.9192195638984577E-01
coeff( 451)= 6.6257797250040862E-01
coeff( 452)= 8.4964349327214483E-01
coeff( 453)=-7.8410506838468541E-01
coeff( 454)=-6.8946126971145466E-02
coeff( 455)=-1.2586916640649732E+00
coeff( 456)=-1.0730422595305047E+01
coeff( 457)=-4.6763041811352046E+00
coeff( 458)=-5.0375775939881109E+00
coeff( 459)= 1.2150661136680387E+00
coeff( 460)= 3.5379355707302362E-01
coeff( 461)=-9.9672076826340861E-01
coeff( 462)=-1.8649457707177290E+00
coeff( 463)=-5.9991705417903702E+00
coeff( 464)= 3.7529604146977413E+00
coeff( 465)= 2.4062726406842376E+00
coeff( 466)= 6.6643280467484107E+00
coeff( 467)=-2.3356721856137326E+00
coeff( 468)=-2.6826593561993777E+00
coeff( 469)= 1.4020042348504917E+00
coeff( 470)= 3.5627421259890326E+00
coeff( 471)=-2.0044711764879075E+00
coeff( 472)= 7.6823317004191971E-01
coeff( 473)=-1.9359325441039839E+00
coeff( 474)=-5.1999484972402001E+00
coeff( 475)=-1.8874487622123317E+00
coeff( 476)=-3.1742343067395140E+00
coeff( 477)= 2.6572605670432177E+00
coeff( 478)=-3.7487967585068329E+00
coeff( 479)=-7.5423523834793071E-01
coeff( 480)=-6.3394443735795347E+00
coeff( 481)= 4.7987601815766601E-01
coeff( 482)=-4.7082667003307774E+00
coeff( 483)=-4.4598624034833750E-01
coeff( 484)= 2.1605820130449849E+00
coeff( 485)= 1.2960997680521985E-01
coeff( 486)= 2.2343146862920853E+00
coeff( 487)= 3.9960467247862325E-01
coeff( 488)= 1.3128971465433754E+00
coeff( 489)= 2.7489980955561686E+00
coeff( 490)= 3.8892800414360018E-01
coeff( 491)= 1.1011289815592056E+00
coeff( 492)=-8.1279866936746648E-01
coeff( 493)= 3.0114028833436408E+00
coeff( 494)= 1.4679517712119456E+00
coeff( 495)= 1.3924662749697752E+00
coeff( 496)=-2.0725271587482692E+00
coeff( 497)=-1.5116032452236583E+00
coeff( 498)=-3.3480184221678411E-01
coeff( 499)= 7.9394282301754937E-02
coeff( 500)=-1.7131210335719438E-01
coeff( 501)=-1.8897633262615901E-01
coeff( 502)=-1.4147556335337430E+00
coeff( 503)= 2.1763805077751175E+00
coeff( 504)= 1.8349084256783137E+00
coeff( 505)= 1.3460724444807384E+00
coeff( 506)=-7.6610072382679906E-01
coeff( 507)=-2.3974275912417897E-01
coeff( 508)=-1.9526647968297985E+00
coeff( 509)=-2.2255779842252460E-01
coeff( 510)= 2.8345354754667658E+00
coeff( 511)= 6.3044761020266871E-01
coeff( 512)= 1.4068160124706537E+00
coeff( 513)= 1.3313805979282831E+00
coeff( 514)=-3.9455766512818718E-01
coeff( 515)= 3.2709348396677418E-01
coeff( 516)= 1.0249307321924420E-02
coeff( 517)=-7.8199385003182953E-01
coeff( 518)= 4.9071477126619895E-01
coeff( 519)=-1.6591319473773249E+00
coeff( 520)= 4.9443004619039765E-01
coeff( 521)=-1.5714885779755483E+00
coeff( 522)=-5.1614437788804957E-01
coeff( 523)= 7.3279551071548499E-01
coeff( 524)= 5.6052223443042770E-01
coeff( 525)=-3.3803617491710330E-01
coeff( 526)= 2.3270199030519821E-02
coeff( 527)=-4.6112113146101230E-01
coeff( 528)= 2.5264078680857388E-01
coeff( 529)= 6.3041838375475323E-01
coeff( 530)= 8.2007903174975361E-01
coeff( 531)= 2.0663547182936795E-01
coeff( 532)= 1.4190373554806732E+00
coeff( 533)=-1.0184838653088000E+00
coeff( 534)=-6.6517985012305225E-01
coeff( 535)= 5.7428997790407987E-01
coeff( 536)=-1.4682807603958181E+00
coeff( 537)= 2.3396360835273962E+00
coeff( 538)= 1.6343193954714834E+00
coeff( 539)= 6.1784492888477838E-01
coeff( 540)= 2.1322527271211919E-01
coeff( 541)=-1.7213955315359724E+00
coeff( 542)=-1.6615028417775612E-01
coeff( 543)=-7.6331918773094073E-02
coeff( 544)=-8.4156187292745444E-01
coeff( 545)=-2.1293079811341056E-01
coeff( 546)= 2.3807205965201941E-01
  return
end if

entry u11_pot_8(xyz,v, dxyz)

r12=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r13=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r14=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r23=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r24=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r34=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

x(1)=exp(-((r12-ra1))/a1 -((r12-rb1)**2)/b1)
x(2)=exp(-((r13-ra1))/a1 -((r13-rb1)**2)/b1)
x(3)=exp(-((r14-ra2))/a2 -((r14-rb2)**2)/b2)
x(4)=exp(-((r23-ra1))/a1 -((r23-rb1)**2)/b1)
x(5)=exp(-((r24-ra2))/a2 -((r24-rb2)**2)/b2)
x(6)=exp(-((r34-ra2))/a2 -((r34-rb2)**2)/b2)
call ev_pol_con_8(x,p)
v=dot_product(coeff,p) +(oh(r14)+oh(r24)+oh(r34)+hh(r12)+hh(r13)+hh(r23))





call grad_ev_pol_con_8(x,dpdx1,dpdx2,dpdx3,dpdx4,dpdx5,dpdx6)

dx1 = dot_product(coeff,dpdx1)
dx2 = dot_product(coeff,dpdx2)
dx3 = dot_product(coeff,dpdx3)
dx4 = dot_product(coeff,dpdx4)
dx5 = dot_product(coeff,dpdx5)
dx6 = dot_product(coeff,dpdx6)

dr1=-x(1)*(1.0d0/(a1)+2.0d0*(r12-rb1)/(b1))*dx1 + grad_hh(r12)
dr2=-x(2)*(1.0d0/(a1)+2.0d0*(r13-rb1)/(b1))*dx2 + grad_hh(r13)
dr3=-x(3)*(1.0d0/(a2)+2.0d0*(r14-rb2)/(b2))*dx3 + grad_oh(r14)
dr4=-x(4)*(1.0d0/(a1)+2.0d0*(r23-rb1)/(b1))*dx4 + grad_hh(r23)
dr5=-x(5)*(1.0d0/(a2)+2.0d0*(r24-rb2)/(b2))*dx5 + grad_oh(r24)
dr6=-x(6)*(1.0d0/(a2)+2.0d0*(r34-rb2)/(b2))*dx6 + grad_oh(r34)

dxyz(1,1)=dr1/r12*(xyz(1,1)-xyz(2,1))+dr2/r13*(xyz(1,1)-xyz(3,1))+dr3/r14*(xyz(1,1)-xyz(4,1))
dxyz(1,2)=dr1/r12*(xyz(1,2)-xyz(2,2))+dr2/r13*(xyz(1,2)-xyz(3,2))+dr3/r14*(xyz(1,2)-xyz(4,2))
dxyz(1,3)=dr1/r12*(xyz(1,3)-xyz(2,3))+dr2/r13*(xyz(1,3)-xyz(3,3))+dr3/r14*(xyz(1,3)-xyz(4,3))

dxyz(2,1)=-dr1/r12*(xyz(1,1)-xyz(2,1))+dr4/r23*(xyz(2,1)-xyz(3,1))+dr5/r24*(xyz(2,1)-xyz(4,1))
dxyz(2,2)=-dr1/r12*(xyz(1,2)-xyz(2,2))+dr4/r23*(xyz(2,2)-xyz(3,2))+dr5/r24*(xyz(2,2)-xyz(4,2))
dxyz(2,3)=-dr1/r12*(xyz(1,3)-xyz(2,3))+dr4/r23*(xyz(2,3)-xyz(3,3))+dr5/r24*(xyz(2,3)-xyz(4,3))

dxyz(3,1)=-dr2/r13*(xyz(1,1)-xyz(3,1))-dr4/r23*(xyz(2,1)-xyz(3,1))+dr6/r34*(xyz(3,1)-xyz(4,1))
dxyz(3,2)=-dr2/r13*(xyz(1,2)-xyz(3,2))-dr4/r23*(xyz(2,2)-xyz(3,2))+dr6/r34*(xyz(3,2)-xyz(4,2))
dxyz(3,3)=-dr2/r13*(xyz(1,3)-xyz(3,3))-dr4/r23*(xyz(2,3)-xyz(3,3))+dr6/r34*(xyz(3,3)-xyz(4,3))

dxyz(4,1)=-dr3/r14*(xyz(1,1)-xyz(4,1))-dr5/r24*(xyz(2,1)-xyz(4,1))-dr6/r34*(xyz(3,1)-xyz(4,1))
dxyz(4,2)=-dr3/r14*(xyz(1,2)-xyz(4,2))-dr5/r24*(xyz(2,2)-xyz(4,2))-dr6/r34*(xyz(3,2)-xyz(4,2))
dxyz(4,3)=-dr3/r14*(xyz(1,3)-xyz(4,3))-dr5/r24*(xyz(2,3)-xyz(4,3))-dr6/r34*(xyz(3,3)-xyz(4,3))

return

end  subroutine

!===========================================================================
! *def u22_prepot_8
! subtoutine of computing u11 and gradient of u11
!===========================================================================
subroutine u22_prepot_8
implicit none
real*8 :: xyz(4,3)
real*8 :: v, r12, r13, r14, r23, r24, r34
real*8 :: a1, a2, b1, b2, oh, hh, ra1, ra2, rb1, rb2, grad_hh, grad_oh
integer, parameter :: ncoeff=546
real*8 :: coeff(ncoeff), p(ncoeff)
real*8 :: dpdx1(ncoeff), dpdx2(ncoeff), dpdx3(ncoeff) ,dpdx4(ncoeff), dpdx5(ncoeff) ,dpdx6(ncoeff)
real*8 :: x(6), re_hh, re_oh, ra_hh, ra_oh, rb_hh, rb_oh
integer ::  ncall
real*8 :: dx1, dx2, dx3, dx4, dx5, dx6, dr1, dr2, dr3, dr4, dr5, dr6, dxyz(4,3)
data ncall/1/
save ncall, coeff, a1, a2, b1, b2, ra1, ra2, rb1, rb2

if(ncall.eq.1) then

a1 = 8.0000000000000004E-01
a2 = 1.2000000000000000E+00
b1 = 1.0000000000000000E+00
b2 = 1.5000000000000000E+00
ra1 = 1.2000000000000000E+00
ra2 = 1.5000000000000000E+00
rb1 = 1.2000000000000000E+00
rb2 = 1.5000000000000000E+00
coeff(   1)=-7.646858051765578E+01
coeff(   2)=-1.6067137962709705E+01
coeff(   3)= 2.5733835881848961E+01
coeff(   4)=-8.8724603704328118E+00
coeff(   5)= 7.2120788815263367E+01
coeff(   6)=-2.9555223430316911E+01
coeff(   7)=-7.2602720862545553E+01
coeff(   8)=-2.3338685055226989E+01
coeff(   9)=-1.0632080905169293E+01
coeff(  10)= 2.5863515495314601E+01
coeff(  11)= 3.7095995975929213E+01
coeff(  12)=-3.6751706748464073E+01
coeff(  13)=-2.5258730680088590E+01
coeff(  14)= 7.8523026370290285E+00
coeff(  15)= 9.5893099047704808E+01
coeff(  16)= 1.1210729619454887E+02
coeff(  17)=-1.1852682893935565E+01
coeff(  18)= 1.0542680466575565E+01
coeff(  19)=-9.7767325945928107E+01
coeff(  20)= 1.2157702458099786E+01
coeff(  21)= 2.5121867924023540E+01
coeff(  22)= 4.5663947156584854E+01
coeff(  23)= 3.5779654225305535E+01
coeff(  24)=-1.1466749648611430E+00
coeff(  25)= 1.4346174703008664E+01
coeff(  26)= 9.1412502505061326E+01
coeff(  27)=-1.7592872072115444E+00
coeff(  28)= 9.7605440466391080E+00
coeff(  29)= 1.2747958544429531E+01
coeff(  30)=-2.3957085385299507E+01
coeff(  31)=-6.4758703442797000E+01
coeff(  32)=-4.3640942770948648E+01
coeff(  33)= 2.9529394902720341E+01
coeff(  34)= 2.4656009444126269E+01
coeff(  35)= 4.6711923799485895E+00
coeff(  36)= 1.0594119139180586E+01
coeff(  37)= 6.7170211608770725E-01
coeff(  38)=-3.0526320414592703E+01
coeff(  39)= 7.7280920288055910E+00
coeff(  40)=-4.8906040868222284E+01
coeff(  41)=-4.7154922607872223E+01
coeff(  42)=-1.2087157080352381E+02
coeff(  43)= 7.1624774017810431E+00
coeff(  44)= 1.1091980983503245E+01
coeff(  45)=-2.7776839567972900E+01
coeff(  46)=-1.1159748113554022E+02
coeff(  47)=-2.0483062575232598E+01
coeff(  48)=-4.0545699659756615E+01
coeff(  49)=-8.2004799613096324E+00
coeff(  50)= 1.6769903163893982E+00
coeff(  51)= 1.6864200621875483E+01
coeff(  52)= 9.2646640488573553E+01
coeff(  53)= 7.4527965415557773E+01
coeff(  54)=-6.8701113152148139E+00
coeff(  55)=-1.4485706188745766E+00
coeff(  56)=-8.7719832204302630E+00
coeff(  57)=-7.4739817444467693E+00
coeff(  58)=-2.5383144267306550E+00
coeff(  59)=-1.0744479963600963E+01
coeff(  60)=-3.0197203916473381E+00
coeff(  61)= 1.2880991126254548E+01
coeff(  62)= 5.0999905799035057E+00
coeff(  63)=-6.5584081592549552E+01
coeff(  64)=-3.0480817379355858E+00
coeff(  65)=-1.2275829086682556E+01
coeff(  66)= 8.3721900136081047E+00
coeff(  67)= 7.4220769597759473E-01
coeff(  68)=-1.9103288792876292E+01
coeff(  69)= 1.4226434246312619E+01
coeff(  70)=-2.3335045144495719E+00
coeff(  71)=-2.6141311114379810E+01
coeff(  72)= 1.2735353331324436E+01
coeff(  73)=-6.5314781027909108E+00
coeff(  74)=-8.8403552623857919E+00
coeff(  75)= 1.1240221953262846E+01
coeff(  76)= 5.5295032580527348E+01
coeff(  77)= 2.9939251966395918E+01
coeff(  78)=-2.5084146687247319E+01
coeff(  79)=-5.0977301134050590E+00
coeff(  80)=-1.5653163695127247E+01
coeff(  81)=-7.5268788314499009E+00
coeff(  82)=-1.3511439050971514E+00
coeff(  83)=-4.7732489954293102E+00
coeff(  84)=-3.3545602199327007E+00
coeff(  85)= 2.5400039118366529E+01
coeff(  86)=-1.9174199151119407E+00
coeff(  87)=-2.2835874185655349E+00
coeff(  88)= 1.3908841268942558E+01
coeff(  89)= 1.2434927981028626E+01
coeff(  90)=-2.5758805269489795E+01
coeff(  91)= 1.5510367365734943E+01
coeff(  92)= 2.3766904930257265E+01
coeff(  93)= 1.3620758699981565E+01
coeff(  94)=-6.9866830246876663E+00
coeff(  95)= 1.1142777987713387E+02
coeff(  96)= 5.2681735268263637E+01
coeff(  97)=-1.2347722976065954E+01
coeff(  98)=-2.7527640583256307E+01
coeff(  99)=-6.9528443141005836E-01
coeff( 100)= 2.1094193299388696E+01
coeff( 101)= 4.4267247066755218E+01
coeff( 102)= 6.4183389428947919E+01
coeff( 103)= 2.6660718495754439E+00
coeff( 104)= 2.1313958591096497E+01
coeff( 105)= 3.7832176750279068E+01
coeff( 106)= 2.6916294435946167E+01
coeff( 107)= 8.0032779091228754E+00
coeff( 108)= 8.8742183429588035E+00
coeff( 109)= 6.1338980504148086E+00
coeff( 110)=-1.1151298051961229E+01
coeff( 111)=-2.0728802934447987E-01
coeff( 112)=-9.6784901789246334E+00
coeff( 113)=-7.8123241612377923E+00
coeff( 114)= 3.5521013879276886E+01
coeff( 115)= 1.5769416912753508E+01
coeff( 116)= 5.8161250774115834E+00
coeff( 117)= 2.4008632565893993E+01
coeff( 118)=-3.1847541354514752E+00
coeff( 119)=-2.3048864883579543E+00
coeff( 120)=-2.0793475342085721E+01
coeff( 121)=-6.1921528991351522E+01
coeff( 122)=-4.5170962646082202E+01
coeff( 123)=-4.0446948454365739E+01
coeff( 124)= 3.6878702567516530E+00
coeff( 125)= 3.1058413385384043E+00
coeff( 126)=-1.5620241962181428E+01
coeff( 127)=-1.5557138040990765E+01
coeff( 128)= 1.2064568898226279E+01
coeff( 129)= 5.5004543690933163E+00
coeff( 130)= 3.0889467350059259E+00
coeff( 131)=-1.0671005531565449E+01
coeff( 132)=-6.9544652685906572E+00
coeff( 133)=-2.2971818391201310E+01
coeff( 134)= 4.0339821077228642E+01
coeff( 135)=-1.3322961596740448E+01
coeff( 136)= 7.1887718020088558E+00
coeff( 137)=-6.1393369763672681E+00
coeff( 138)=-8.8000677138407735E+00
coeff( 139)= 3.4751188251957412E+00
coeff( 140)= 7.0577473906060750E-01
coeff( 141)=-7.0617842004896403E-01
coeff( 142)=-7.1183286786473410E+00
coeff( 143)=-1.1990982974693475E+01
coeff( 144)=-1.8789067671325133E+00
coeff( 145)= 1.8968600203324673E+01
coeff( 146)=-7.2429165450514237E+00
coeff( 147)= 8.8871633925345890E+00
coeff( 148)=-9.1046972579606766E+00
coeff( 149)= 1.3056982876501171E+00
coeff( 150)= 6.3427258711785059E+00
coeff( 151)= 1.2324517433299667E+01
coeff( 152)=-7.0738304252525328E+00
coeff( 153)= 4.2331858417599921E+00
coeff( 154)= 3.9813794543839456E+00
coeff( 155)=-1.1514703811193336E+01
coeff( 156)= 1.6231698952881191E+00
coeff( 157)= 6.5259323720360154E+00
coeff( 158)= 1.2962043248678108E+00
coeff( 159)=-2.1784349767942420E+01
coeff( 160)=-3.1684376816872074E+01
coeff( 161)=-9.2631750483660831E+00
coeff( 162)= 1.2310183254515197E+01
coeff( 163)= 3.4689909278130417E+00
coeff( 164)= 4.9197778688761380E+00
coeff( 165)= 2.5232917312766716E+00
coeff( 166)= 6.9582628302451459E+00
coeff( 167)= 2.0201276965540331E+00
coeff( 168)=-2.1944757021629449E+00
coeff( 169)= 4.4990143456056515E+00
coeff( 170)=-3.6634770254405140E+00
coeff( 171)= 2.5987619355140561E+00
coeff( 172)=-1.9380989937134679E+01
coeff( 173)=-6.8001457934974663E-01
coeff( 174)= 1.1221484835862926E+01
coeff( 175)= 1.1845295303732705E+01
coeff( 176)=-7.9353291511905244E+00
coeff( 177)=-8.2238055391770732E+00
coeff( 178)=-1.5283405648885187E+01
coeff( 179)=-6.2276699092199852E+00
coeff( 180)=-1.2101155065421228E+01
coeff( 181)=-1.0305843659808019E+01
coeff( 182)= 4.3547816898892968E+00
coeff( 183)= 1.3290770082161760E+01
coeff( 184)= 1.9394192743134475E+01
coeff( 185)= 7.8059247748438887E+00
coeff( 186)=-1.6727586605073668E+01
coeff( 187)= 1.4404907104822477E+00
coeff( 188)=-5.6884079454107805E+00
coeff( 189)= 1.5492899967129151E+01
coeff( 190)=-1.2584600335061258E+00
coeff( 191)=-1.1624855081063815E+01
coeff( 192)= 3.5215299808146430E+00
coeff( 193)=-1.5686926446348051E+00
coeff( 194)= 7.4152053203112596E+00
coeff( 195)=-2.7187358744417436E+00
coeff( 196)=-3.9125031289963715E+01
coeff( 197)=-1.3848167102938591E+01
coeff( 198)= 1.2970478921391217E+00
coeff( 199)= 1.6073862922679623E+01
coeff( 200)= 5.6722235918658637E+00
coeff( 201)= 6.7153828827927455E+00
coeff( 202)=-6.4905351161414959E+00
coeff( 203)=-6.9963471177044303E+00
coeff( 204)=-1.9338549742927771E+01
coeff( 205)=-4.7226213228645136E+01
coeff( 206)= 6.7920004231799567E+00
coeff( 207)= 9.5264541836225067E+00
coeff( 208)=-3.2482743463543917E+00
coeff( 209)=-5.0604962281652650E+00
coeff( 210)=-7.7279723849683748E+00
coeff( 211)=-1.3162224249823645E+01
coeff( 212)= 6.6412621823537341E+00
coeff( 213)=-6.4048905405326853E+00
coeff( 214)=-3.3566174415975993E+00
coeff( 215)=-5.1461879287889980E+00
coeff( 216)=-4.7737970983435707E+00
coeff( 217)= 3.8330255058624281E+00
coeff( 218)= 1.6898320680108375E-01
coeff( 219)=-1.0136966251132682E+00
coeff( 220)= 1.7501101253939173E+00
coeff( 221)= 1.0498695960535390E+01
coeff( 222)=-9.9897402979455485E-01
coeff( 223)=-1.8461788317702752E+01
coeff( 224)= 9.8022712887061081E-01
coeff( 225)=-8.1751147278772045E+00
coeff( 226)=-4.7450263165025817E+00
coeff( 227)=-6.9711788299716098E+00
coeff( 228)=-9.7226459523173734E+00
coeff( 229)=-1.6524943685172080E-01
coeff( 230)=-9.8160563463620818E+00
coeff( 231)= 8.4965321968161056E-01
coeff( 232)= 2.1507284252910708E+00
coeff( 233)= 1.0606518581842547E+01
coeff( 234)=-4.5860734789533186E+00
coeff( 235)= 2.9434136352074969E+00
coeff( 236)=-2.9067198524381426E+00
coeff( 237)= 2.9335924037086141E+00
coeff( 238)= 1.4039526942485105E+00
coeff( 239)=-5.0780123898041518E+00
coeff( 240)=-3.5659237533742942E+00
coeff( 241)= 2.5476807486387201E+00
coeff( 242)=-8.3445630331531984E+00
coeff( 243)= 1.0358544050968264E+00
coeff( 244)= 2.2685758807465071E+00
coeff( 245)= 6.9797546509251465E+00
coeff( 246)= 1.6434134498559427E+01
coeff( 247)= 9.8869183000300112E+00
coeff( 248)= 1.7336240115505198E+01
coeff( 249)= 1.1351154920110963E+01
coeff( 250)=-1.0611488926193966E+00
coeff( 251)=-1.2277521745108133E+00
coeff( 252)= 5.9079299420409548E+00
coeff( 253)= 3.6159038463623863E+00
coeff( 254)= 7.5896455362485611E+00
coeff( 255)= 9.1388792702114952E+00
coeff( 256)=-6.0786213498593185E+00
coeff( 257)=-4.3740981622882957E+00
coeff( 258)=-6.8319463093971500E-01
coeff( 259)= 5.6985517864837263E+00
coeff( 260)=-3.0887422455378799E-01
coeff( 261)= 6.0953399154473189E+00
coeff( 262)= 1.0066644701322732E+01
coeff( 263)=-4.1038936741978915E+00
coeff( 264)= 1.3281845074612320E-01
coeff( 265)= 1.8716551261779246E-01
coeff( 266)= 3.3485393688150809E+00
coeff( 267)= 6.6504789977652585E+00
coeff( 268)=-1.8712712515501029E+00
coeff( 269)=-2.1711224280346864E+00
coeff( 270)=-5.6516546554133433E-03
coeff( 271)= 5.7675616294682985E+00
coeff( 272)= 1.0576873937909308E+01
coeff( 273)=-4.8690989240168570E+00
coeff( 274)= 1.6964663360631795E+00
coeff( 275)=-5.3171156714775289E+00
coeff( 276)= 3.5883331605903663E+00
coeff( 277)=-4.4551270271253846E-01
coeff( 278)= 2.9002579318127437E+00
coeff( 279)= 2.9626503124986409E-01
coeff( 280)= 8.7353546939114934E-01
coeff( 281)=-3.0880424700267253E+00
coeff( 282)= 4.8461591221091727E+00
coeff( 283)=-1.3042617658308415E+00
coeff( 284)=-5.6594547766938810E+00
coeff( 285)= 6.9153279357995290E-01
coeff( 286)=-3.1292053307152640E-01
coeff( 287)=-1.4447024656407415E+00
coeff( 288)=-2.9448213636740710E+00
coeff( 289)= 5.3873564534368326E+00
coeff( 290)=-1.1933505909714877E+00
coeff( 291)=-4.4695252181079864E-02
coeff( 292)=-2.4893678652892848E-01
coeff( 293)=-2.4603056691985672E+00
coeff( 294)=-2.1703368206796338E+00
coeff( 295)= 1.8881968840061507E+00
coeff( 296)=-9.5397328850995178E-01
coeff( 297)= 7.1059561552863404E-01
coeff( 298)= 3.2778773685485061E+00
coeff( 299)=-1.1986334137983714E+00
coeff( 300)=-3.2084426749398465E+00
coeff( 301)=-2.4734965682829961E+00
coeff( 302)= 5.3380297099599812E+00
coeff( 303)= 9.1902952662177810E+00
coeff( 304)= 2.8052178334020489E-01
coeff( 305)=-3.3347946922752643E+00
coeff( 306)=-1.7078850538089758E+00
coeff( 307)=-2.5057349101442467E-01
coeff( 308)=-2.7344662643253335E+00
coeff( 309)= 3.2096061840867629E-01
coeff( 310)=-1.5037060846705066E+00
coeff( 311)=-1.4828690174740317E-01
coeff( 312)= 1.2601585952154226E+00
coeff( 313)=-1.6210463398034751E+00
coeff( 314)=-1.0955354186674793E+00
coeff( 315)= 5.2232603604661474E+00
coeff( 316)=-4.3442748575966936E+00
coeff( 317)=-3.5521873423653350E-01
coeff( 318)= 4.8671758620451372E-01
coeff( 319)= 4.2827653983391549E+00
coeff( 320)= 3.3918193010254631E+00
coeff( 321)=-1.4242287066642567E-01
coeff( 322)=-2.5861687133485622E+00
coeff( 323)=-5.6299861514556264E+00
coeff( 324)= 6.7604696097230177E-01
coeff( 325)= 4.9738750351339750E-01
coeff( 326)=-5.0283153488679089E-01
coeff( 327)= 5.2136523899689746E-01
coeff( 328)= 3.8361067664546713E+00
coeff( 329)= 1.6442026203134280E-01
coeff( 330)=-2.8222442488235626E+00
coeff( 331)= 4.0655091580241436E+00
coeff( 332)= 2.9372769037310249E+00
coeff( 333)= 3.1829704192022792E+00
coeff( 334)=-2.2433655194514706E+00
coeff( 335)=-3.6427160050652918E-01
coeff( 336)= 2.5776951086382032E+00
coeff( 337)= 2.4037860206686446E-01
coeff( 338)=-5.0158195920228978E+00
coeff( 339)=-3.6699179717497645E+00
coeff( 340)=-4.1139687788287305E-01
coeff( 341)= 1.7581338807569686E+00
coeff( 342)=-2.8049551085930338E+00
coeff( 343)=-5.3719233243080267E-01
coeff( 344)= 8.4922178648130160E+00
coeff( 345)= 1.9031246404813682E+00
coeff( 346)= 4.6897285392188284E+00
coeff( 347)= 3.1663729602611701E+00
coeff( 348)=-1.8371196651911881E+00
coeff( 349)=-4.6617408446099908E+00
coeff( 350)=-4.3415914054702602E+00
coeff( 351)= 2.1341348948882505E-01
coeff( 352)=-1.3634116775660987E+00
coeff( 353)= 2.7918295057255837E-01
coeff( 354)= 2.9181689888136519E-01
coeff( 355)=-3.2769140271775195E+00
coeff( 356)= 1.4797864996159011E+00
coeff( 357)=-1.8021817350635387E+00
coeff( 358)=-5.9286092109378830E-01
coeff( 359)= 1.5117643540260299E+00
coeff( 360)=-2.9660368389002589E+00
coeff( 361)= 5.5815280466035171E+00
coeff( 362)=-2.6124443716615851E+00
coeff( 363)= 4.7360015671826412E+00
coeff( 364)=-1.3682529592917989E+00
coeff( 365)=-5.5403696018989130E+00
coeff( 366)= 4.6891095867881472E+00
coeff( 367)= 9.8054040811037382E+00
coeff( 368)= 3.4356498263987292E+00
coeff( 369)= 2.6537244597113383E+00
coeff( 370)= 1.8421247379192602E+00
coeff( 371)=-3.7236410568787583E+00
coeff( 372)=-1.2355898384472774E+00
coeff( 373)=-1.2868093118114963E+00
coeff( 374)=-1.4592648924213232E+00
coeff( 375)= 2.0846729047422281E+00
coeff( 376)=-2.0752127813913618E-01
coeff( 377)= 1.5340319208705069E+00
coeff( 378)= 1.5546285224255897E+01
coeff( 379)=-3.4718493182396362E+00
coeff( 380)= 3.3669768246108682E+00
coeff( 381)=-3.9538521946113279E+00
coeff( 382)=-2.9430177148318912E+00
coeff( 383)=-2.7621210598342367E+00
coeff( 384)= 2.3386492046059066E+00
coeff( 385)= 3.3453140039970269E-02
coeff( 386)=-2.3554770975066939E+00
coeff( 387)= 3.1762816479034890E+00
coeff( 388)=-4.3181058821585934E+00
coeff( 389)= 1.8213524491824726E+00
coeff( 390)= 2.0025874464356495E+00
coeff( 391)= 1.1021329065163294E+00
coeff( 392)=-7.5403578293582174E-01
coeff( 393)= 8.0092846752200256E-02
coeff( 394)= 3.4508493469722297E-02
coeff( 395)=-5.2854102061999209E-01
coeff( 396)=-4.3539771492768192E-01
coeff( 397)= 2.3947145754392499E-01
coeff( 398)= 2.5361500544830626E+00
coeff( 399)=-9.2140837079952886E-01
coeff( 400)=-1.6402881649873624E+00
coeff( 401)=-7.0274826907319063E+00
coeff( 402)= 3.3488144580618462E+00
coeff( 403)= 2.1097514784600038E+00
coeff( 404)= 9.2084861275897967E-01
coeff( 405)=-1.6545496200004803E+00
coeff( 406)=-7.0471665687496898E-01
coeff( 407)= 2.3096978051585615E+00
coeff( 408)= 9.9105931617876941E-01
coeff( 409)= 2.0065064076392689E+00
coeff( 410)= 9.5174052829468048E-01
coeff( 411)= 3.4427236869705191E-01
coeff( 412)=-1.6018494139790074E+00
coeff( 413)= 3.7316892050167256E+00
coeff( 414)=-8.1204515311262304E-01
coeff( 415)=-8.9722161372928155E-01
coeff( 416)=-1.9834947268917921E-01
coeff( 417)= 7.4501015085063305E-01
coeff( 418)=-2.2297422892248600E+00
coeff( 419)= 5.7568097471514945E-01
coeff( 420)= 7.5449589053364075E-01
coeff( 421)=-3.7326250657434018E-01
coeff( 422)= 3.4404049552467675E+00
coeff( 423)= 1.5658588692790811E+00
coeff( 424)= 5.8508117212121313E-01
coeff( 425)= 1.3854078721524754E+00
coeff( 426)= 4.4094964592280361E-01
coeff( 427)=-1.0846468902791115E+00
coeff( 428)=-1.8833551367012507E+00
coeff( 429)= 6.1896922148176459E-01
coeff( 430)= 7.6978973149395580E-01
coeff( 431)= 5.2881186094524735E-01
coeff( 432)= 2.5636928830221806E-01
coeff( 433)=-4.0183465074291075E-02
coeff( 434)= 2.7213806048550088E-03
coeff( 435)= 1.7700082627288658E+00
coeff( 436)=-1.0055367390433374E+00
coeff( 437)=-7.6514765763405199E-02
coeff( 438)= 1.6675649744119823E+00
coeff( 439)= 3.5911423571547407E-01
coeff( 440)=-1.1047264934263519E+00
coeff( 441)= 1.6889285340383702E+00
coeff( 442)= 1.2370965676721870E+00
coeff( 443)= 3.2263244426809062E-01
coeff( 444)= 3.6513076050559135E-01
coeff( 445)=-1.6404848588330128E+00
coeff( 446)=-2.0994078503211977E+00
coeff( 447)=-4.9782514317924148E-01
coeff( 448)= 1.6665884120556326E-01
coeff( 449)= 1.6355694567577876E+00
coeff( 450)=-9.9255786222167772E-03
coeff( 451)= 1.0648130341022449E-01
coeff( 452)= 1.0101877372301855E+00
coeff( 453)=-2.3168755810477129E-01
coeff( 454)= 2.9486479868433257E-01
coeff( 455)=-1.7212686348711275E+00
coeff( 456)=-2.2889041090201374E+00
coeff( 457)=-1.4636000897927144E+00
coeff( 458)=-1.5841906427963881E+00
coeff( 459)=-3.3365516848658086E+00
coeff( 460)=-7.9110619320147235E-01
coeff( 461)= 1.9384591458626513E-01
coeff( 462)= 2.3416883851913328E-01
coeff( 463)=-1.0928756006362774E+00
coeff( 464)=-3.4579263653980101E+00
coeff( 465)=-1.6502134981024268E+00
coeff( 466)= 6.7108543534950571E-01
coeff( 467)= 1.7069559889832517E-01
coeff( 468)=-3.8168868978911746E-01
coeff( 469)=-7.0027594059277376E-01
coeff( 470)=-6.7783438427418763E-01
coeff( 471)=-5.6788593681456678E-01
coeff( 472)= 1.0004323091404546E-01
coeff( 473)= 4.3133265108805668E-01
coeff( 474)=-6.2764851655720127E-02
coeff( 475)=-1.2424638783231092E+00
coeff( 476)=-2.1896891317356806E+00
coeff( 477)= 6.7532669892566555E-01
coeff( 478)= 1.9552807475556502E+00
coeff( 479)= 4.8450298839369105E-01
coeff( 480)=-9.5252157130869944E-02
coeff( 481)=-7.6943525409728963E-01
coeff( 482)=-2.4228341047130819E-01
coeff( 483)= 9.9165346957574696E-01
coeff( 484)= 7.1720643325714950E-01
coeff( 485)=-2.4443750660576277E-01
coeff( 486)=-3.9745480856188764E+00
coeff( 487)=-4.1979419186959077E-02
coeff( 488)=-1.3340326256804819E-01
coeff( 489)=-5.5597907371076205E-01
coeff( 490)= 1.2665759506910830E+00
coeff( 491)=-1.1034890600303380E+00
coeff( 492)=-4.3728262443885846E-01
coeff( 493)= 2.8422143322836740E-01
coeff( 494)= 5.2174090882710433E-01
coeff( 495)=-8.9695859525790578E-01
coeff( 496)=-1.4973222225610643E+00
coeff( 497)=-1.0403835739815332E+00
coeff( 498)=-3.7810905133959288E-01
coeff( 499)= 3.6152315891001402E-01
coeff( 500)=-3.2214478268925478E-01
coeff( 501)= 7.5706438890489336E-03
coeff( 502)= 2.2464385597077099E+00
coeff( 503)=-6.3931157487057655E-01
coeff( 504)=-6.0224680041951373E-01
coeff( 505)=-5.0285307590094164E-01
coeff( 506)=-4.1892234338191683E-02
coeff( 507)=-5.7946743431514358E-01
coeff( 508)=-4.6655227530837351E-01
coeff( 509)=-4.5643979071917123E-01
coeff( 510)= 1.0666466524021809E+00
coeff( 511)=-2.1847858945345966E-01
coeff( 512)= 9.3103017450500952E-02
coeff( 513)= 7.4876558882631228E-01
coeff( 514)= 4.7073737133512022E-01
coeff( 515)=-4.9362430562357718E-01
coeff( 516)= 2.5352224443583105E-01
coeff( 517)= 1.5598242811729682E-01
coeff( 518)=-8.8442151226879248E-02
coeff( 519)=-8.6581763494635766E-01
coeff( 520)= 2.6679072892433792E-01
coeff( 521)=-2.1530763641612624E-01
coeff( 522)= 5.6010965736210294E-01
coeff( 523)= 6.9288394930122543E-01
coeff( 524)= 6.2464420464015291E-01
coeff( 525)=-2.9011412844334811E-01
coeff( 526)= 8.1823692947392029E-02
coeff( 527)=-1.5983850481542072E-01
coeff( 528)=-4.6165109949220995E-01
coeff( 529)= 2.3031447505530067E-01
coeff( 530)= 6.9704571460165510E-01
coeff( 531)= 6.0141954947257459E-01
coeff( 532)= 7.9290392129011233E-01
coeff( 533)=-1.0555648424147315E+00
coeff( 534)=-8.1147798589080034E-01
coeff( 535)= 2.6670214014220400E-01
coeff( 536)= 9.3039976586282880E-01
coeff( 537)= 6.1080266734766259E-03
coeff( 538)=-2.9382722775513498E-01
coeff( 539)= 4.8895957478754848E-01
coeff( 540)=-1.6506967097574166E-01
coeff( 541)= 6.2799897944211180E-01
coeff( 542)=-1.3202306591466567E-01
coeff( 543)= 1.1859822367615025E-01
coeff( 544)=-2.3968567217838699E-01
coeff( 545)=-2.4009078792555746E-01
coeff( 546)= 2.5386361788460277E-01
  return
end if

entry u22_pot_8(xyz,v, dxyz)

r12=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r13=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r14=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r23=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r24=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r34=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

re_hh = 0.741
re_oh = 0.970

ra_hh=ra1
ra_oh=ra2
rb_hh=rb1
rb_oh=rb2


x(1)=exp(-((r12-ra1))/a1 -((r12-rb1)**2)/b1)
x(2)=exp(-((r13-ra1))/a1 -((r13-rb1)**2)/b1)
x(3)=exp(-((r14-ra2))/a2 -((r14-rb2)**2)/b2)
x(4)=exp(-((r23-ra1))/a1 -((r23-rb1)**2)/b1)
x(5)=exp(-((r24-ra2))/a2 -((r24-rb2)**2)/b2)
x(6)=exp(-((r34-ra2))/a2 -((r34-rb2)**2)/b2)
call ev_pol_con_8(x,p) 
v=dot_product(coeff,p)  +((oh(r14)+oh(r24)+oh(r34))*0.9960924723525706+hh(r12)+hh(r13)+hh(r23))


call grad_ev_pol_con_8(x,dpdx1,dpdx2,dpdx3,dpdx4,dpdx5,dpdx6)

dx1 = dot_product(coeff,dpdx1)
dx2 = dot_product(coeff,dpdx2)
dx3 = dot_product(coeff,dpdx3)
dx4 = dot_product(coeff,dpdx4)
dx5 = dot_product(coeff,dpdx5)
dx6 = dot_product(coeff,dpdx6)

dr1=-x(1)*(1.0d0/(a1)+2.0d0*(r12-rb1)/(b1))*dx1 + grad_hh(r12)
dr2=-x(2)*(1.0d0/(a1)+2.0d0*(r13-rb1)/(b1))*dx2 + grad_hh(r13)
dr3=-x(3)*(1.0d0/(a2)+2.0d0*(r14-rb2)/(b2))*dx3 + grad_oh(r14)*0.9960924723525706
dr4=-x(4)*(1.0d0/(a1)+2.0d0*(r23-rb1)/(b1))*dx4 + grad_hh(r23)
dr5=-x(5)*(1.0d0/(a2)+2.0d0*(r24-rb2)/(b2))*dx5 + grad_oh(r24)*0.9960924723525706
dr6=-x(6)*(1.0d0/(a2)+2.0d0*(r34-rb2)/(b2))*dx6 + grad_oh(r34)*0.9960924723525706

dxyz(1,1)=dr1/r12*(xyz(1,1)-xyz(2,1))+dr2/r13*(xyz(1,1)-xyz(3,1))+dr3/r14*(xyz(1,1)-xyz(4,1))
dxyz(1,2)=dr1/r12*(xyz(1,2)-xyz(2,2))+dr2/r13*(xyz(1,2)-xyz(3,2))+dr3/r14*(xyz(1,2)-xyz(4,2))
dxyz(1,3)=dr1/r12*(xyz(1,3)-xyz(2,3))+dr2/r13*(xyz(1,3)-xyz(3,3))+dr3/r14*(xyz(1,3)-xyz(4,3))

dxyz(2,1)=-dr1/r12*(xyz(1,1)-xyz(2,1))+dr4/r23*(xyz(2,1)-xyz(3,1))+dr5/r24*(xyz(2,1)-xyz(4,1))
dxyz(2,2)=-dr1/r12*(xyz(1,2)-xyz(2,2))+dr4/r23*(xyz(2,2)-xyz(3,2))+dr5/r24*(xyz(2,2)-xyz(4,2))
dxyz(2,3)=-dr1/r12*(xyz(1,3)-xyz(2,3))+dr4/r23*(xyz(2,3)-xyz(3,3))+dr5/r24*(xyz(2,3)-xyz(4,3))

dxyz(3,1)=-dr2/r13*(xyz(1,1)-xyz(3,1))-dr4/r23*(xyz(2,1)-xyz(3,1))+dr6/r34*(xyz(3,1)-xyz(4,1))
dxyz(3,2)=-dr2/r13*(xyz(1,2)-xyz(3,2))-dr4/r23*(xyz(2,2)-xyz(3,2))+dr6/r34*(xyz(3,2)-xyz(4,2))
dxyz(3,3)=-dr2/r13*(xyz(1,3)-xyz(3,3))-dr4/r23*(xyz(2,3)-xyz(3,3))+dr6/r34*(xyz(3,3)-xyz(4,3))

dxyz(4,1)=-dr3/r14*(xyz(1,1)-xyz(4,1))-dr5/r24*(xyz(2,1)-xyz(4,1))-dr6/r34*(xyz(3,1)-xyz(4,1))
dxyz(4,2)=-dr3/r14*(xyz(1,2)-xyz(4,2))-dr5/r24*(xyz(2,2)-xyz(4,2))-dr6/r34*(xyz(3,2)-xyz(4,2))
dxyz(4,3)=-dr3/r14*(xyz(1,3)-xyz(4,3))-dr5/r24*(xyz(2,3)-xyz(4,3))-dr6/r34*(xyz(3,3)-xyz(4,3))

return
end  subroutine

!===========================================================================
! *def u11_prepot_8
! subtoutine of computing u11 and gradient of u11
!===========================================================================
subroutine u33_prepot_8
implicit none
real*8 :: xyz(4,3)
real*8 :: v, r12, r13, r14, r23, r24, r34
real*8 :: a1, a2, b1, b2, ohstar, hh, ra1, ra2, rb1, rb2, grad_hh, grad_ohstar
integer, parameter :: ncoeff=546
real*8 :: coeff(ncoeff), p(ncoeff)
real*8 :: dpdx1(ncoeff), dpdx2(ncoeff), dpdx3(ncoeff) ,dpdx4(ncoeff), dpdx5(ncoeff) ,dpdx6(ncoeff)
real*8 :: x(6), re_hh, re_oh, ra_hh, ra_oh, rb_hh, rb_oh
real*8 :: dx1, dx2, dx3, dx4, dx5, dx6, dr1, dr2, dr3, dr4, dr5, dr6, dxyz(4,3)
integer ::  ncall
data ncall/1/
save ncall, coeff, a1, a2, b1, b2, ra1, ra2, rb1, rb2

if(ncall.eq.1) then

a1 = 8.0000000000000004E-01
a2 = 1.2000000000000000E+00
b1 = 1.0000000000000000E+00
b2 = 1.5000000000000000E+00
ra1 = 1.2000000000000000E+00
ra2 = 1.3999999999999999E+00
rb1 = 1.2000000000000000E+00
rb2 = 1.3999999999999999E+00
coeff(   1)=-7.646858051765578E+01
coeff(   2)=-3.8448256931152436E+01
coeff(   3)= 5.9347562318317824E+00
coeff(   4)=-1.5584957116077254E+01
coeff(   5)= 2.0478496457019961E+02
coeff(   6)= 2.1080496202282077E+01
coeff(   7)=-9.5002540876343900E+01
coeff(   8)=-1.1636026185443935E+00
coeff(   9)= 1.8305203428744260E+01
coeff(  10)= 4.5003199515212188E+01
coeff(  11)= 8.5303243408379444E+01
coeff(  12)=-4.7171578184915823E+00
coeff(  13)=-9.3169798077713057E+00
coeff(  14)= 2.5978561562750496E+01
coeff(  15)= 1.4689735037049552E+01
coeff(  16)= 5.3200338032622625E+01
coeff(  17)= 1.5510847142174455E+00
coeff(  18)=-2.3459206220628644E+01
coeff(  19)=-3.0647701575092844E+02
coeff(  20)=-3.4162544257127976E+01
coeff(  21)=-7.1718787285732802E+01
coeff(  22)= 1.2351023410099882E+02
coeff(  23)= 6.0274071703251835E+00
coeff(  24)=-9.6369166347023466E+01
coeff(  25)= 6.3225440969548785E+00
coeff(  26)= 3.7305144132672496E+01
coeff(  27)=-1.1995042743450879E+01
coeff(  28)=-2.9324452383886637E+01
coeff(  29)=-2.0192891452779882E+01
coeff(  30)=-1.3421499953292155E+01
coeff(  31)=-1.2937554639983406E+02
coeff(  32)=-8.4180689131934784E+01
coeff(  33)= 3.7525289899164406E+01
coeff(  34)= 2.5403823682355791E+00
coeff(  35)=-2.7506961157083929E+01
coeff(  36)= 7.9099119725627256E+00
coeff(  37)=-1.5879455268417829E+01
coeff(  38)=-1.0464839976653052E+02
coeff(  39)=-1.4356716820501518E+01
coeff(  40)= 1.0468027212949156E+02
coeff(  41)=-3.0169886131603363E+01
coeff(  42)=-5.0722472483607497E+00
coeff(  43)=-7.3102000940364178E+01
coeff(  44)= 8.9592669545018609E+01
coeff(  45)= 8.3417263468523132E+01
coeff(  46)= 3.6784831085201766E+01
coeff(  47)=-2.2109176357536604E+01
coeff(  48)= 1.9942228389186551E+00
coeff(  49)= 1.2533108175787749E+01
coeff(  50)= 2.9010069183655940E+01
coeff(  51)= 5.6403474547749051E+00
coeff(  52)= 2.9934447393937006E+02
coeff(  53)= 2.5128806159116922E+02
coeff(  54)= 2.2276314148603930E+01
coeff(  55)= 3.7513844943895798E+01
coeff(  56)=-1.0577985552401077E+02
coeff(  57)= 8.0357546089389871E+01
coeff(  58)=-8.0888241972141202E+01
coeff(  59)= 1.5325357815681921E+01
coeff(  60)= 1.0673293807428285E+01
coeff(  61)= 1.0168408927083403E+01
coeff(  62)=-1.3079924171204873E+01
coeff(  63)=-6.8852441764337229E+01
coeff(  64)=-3.0043076102966104E+01
coeff(  65)= 8.1663598844797498E+00
coeff(  66)= 1.2867551175464005E+01
coeff(  67)= 3.2471380326615275E+01
coeff(  68)=-8.1250998219460069E-01
coeff(  69)= 8.0046508123270819E+00
coeff(  70)=-1.1675555235602857E+01
coeff(  71)=-1.3257999929051314E+00
coeff(  72)= 1.9752498028596690E+01
coeff(  73)= 1.7229824717147416E+01
coeff(  74)= 2.9864267610307810E+01
coeff(  75)= 3.5209117307884199E+00
coeff(  76)= 7.9925228017528354E+01
coeff(  77)= 4.1176031718693608E+01
coeff(  78)=-7.8495543520821784E+01
coeff(  79)= 1.6323378783124287E+01
coeff(  80)= 3.8651365498935668E+00
coeff(  81)= 1.1011213428401216E+01
coeff(  82)=-6.8342760658918920E+00
coeff(  83)=-7.6222335248491391E-01
coeff(  84)= 4.2572253089391033E+01
coeff(  85)= 9.9875102505886332E+01
coeff(  86)=-4.2193947474226711E+01
coeff(  87)=-8.5901384147563000E+00
coeff(  88)= 2.8233427509555860E+00
coeff(  89)= 3.2761039535056661E+01
coeff(  90)=-3.9340616390028906E+01
coeff(  91)=-1.0373050484967024E+02
coeff(  92)=-3.6997725666612126E+01
coeff(  93)= 6.0107691073437856E+01
coeff(  94)=-3.7811784207125996E+00
coeff(  95)=-1.5096420548338067E+01
coeff(  96)=-5.8789441200060402E+01
coeff(  97)= 5.8785636929456665E+01
coeff(  98)= 1.6326264574992685E+01
coeff(  99)=-8.7164304785791387E+01
coeff( 100)=-1.1328635065710920E+02
coeff( 101)=-7.3200071767449742E+01
coeff( 102)= 2.0456957277710446E+00
coeff( 103)= 5.8169402463698852E+01
coeff( 104)= 1.0347825054885622E+01
coeff( 105)= 1.4673464221546006E+01
coeff( 106)=-1.5347069439208607E+01
coeff( 107)=-7.2959379841246417E+00
coeff( 108)=-4.7007921538451685E+00
coeff( 109)=-5.6895226438792470E+00
coeff( 110)= 9.5680715775427938E+00
coeff( 111)=-2.5895961591502100E+01
coeff( 112)=-8.6639246304641020E+00
coeff( 113)=-3.9180050453953058E+01
coeff( 114)= 1.2277176341118597E+01
coeff( 115)=-2.1957287847868030E+00
coeff( 116)=-1.4974736798454945E+01
coeff( 117)=-3.0086977741100249E+01
coeff( 118)= 1.6295306600729170E+01
coeff( 119)= 6.6786035122258358E-01
coeff( 120)=-1.3687301900986141E+01
coeff( 121)=-2.2030854014998380E+02
coeff( 122)=-1.4372879808813741E+02
coeff( 123)=-1.2923803260246990E+02
coeff( 124)=-1.4344197841683805E+01
coeff( 125)= 3.1140605603893079E+01
coeff( 126)=-5.5612908575968802E+01
coeff( 127)= 3.4597704674287158E+01
coeff( 128)=-2.5688356510669749E+01
coeff( 129)= 3.4406120573241630E+01
coeff( 130)=-2.1667383160930978E+01
coeff( 131)= 1.0663441028542762E+02
coeff( 132)=-1.5678564936010099E+01
coeff( 133)= 8.9085366302450915E+00
coeff( 134)= 3.4067136363297209E+01
coeff( 135)= 4.1293910447748878E+00
coeff( 136)= 6.4997070221301158E+01
coeff( 137)= 4.4661847130822721E+01
coeff( 138)=-7.9193390826300170E-01
coeff( 139)=-1.2489417176755806E+01
coeff( 140)=-2.2311376855622356E+01
coeff( 141)=-2.1158367872375983E+01
coeff( 142)= 1.1852605879757508E+01
coeff( 143)= 9.3294541689021404E+00
coeff( 144)=-1.6952046146639523E+00
coeff( 145)= 3.3388278512878689E+00
coeff( 146)=-7.4916093579753573E+00
coeff( 147)= 3.5880954089934169E+00
coeff( 148)=-1.0654618489948827E+01
coeff( 149)= 7.9257786880877745E+00
coeff( 150)=-3.6553471037216374E+00
coeff( 151)= 9.0368317336938020E+00
coeff( 152)= 1.5587840886191717E-01
coeff( 153)= 2.9764804148825506E+00
coeff( 154)= 2.7027218776798625E+00
coeff( 155)=-1.1736367543204457E+01
coeff( 156)=-8.3546480686564255E+00
coeff( 157)=-1.8667524986684107E+01
coeff( 158)=-5.6448836272559246E+00
coeff( 159)=-5.1324386815412574E+01
coeff( 160)=-6.0840838272474569E+00
coeff( 161)=-7.4417048205184804E+00
coeff( 162)= 5.8378496510914090E+01
coeff( 163)=-2.7519582070875117E+00
coeff( 164)=-3.2660263728395350E+01
coeff( 165)=-2.8815308804922783E+00
coeff( 166)= 1.1399129224814873E+01
coeff( 167)=-1.1472688611000374E+01
coeff( 168)= 2.4095520791237997E+00
coeff( 169)= 1.1376945718908253E+01
coeff( 170)=-1.6281655149063980E+01
coeff( 171)=-1.9723803197882304E+00
coeff( 172)=-5.7525240451886688E+01
coeff( 173)=-2.3934727976612670E+01
coeff( 174)=-1.6073316836450275E+01
coeff( 175)=-9.2152849344177810E+00
coeff( 176)= 7.0564307574276214E+01
coeff( 177)= 3.2057803212023401E+01
coeff( 178)=-8.5822472521235760E+00
coeff( 179)= 4.8427750834425343E-01
coeff( 180)=-2.0360176853051534E+01
coeff( 181)=-4.5699635027672363E+00
coeff( 182)=-2.5442933843354279E+01
coeff( 183)= 1.1329366514772506E+01
coeff( 184)= 1.4972412890785769E+01
coeff( 185)= 2.6893496151411238E+01
coeff( 186)= 1.9031894639428240E+00
coeff( 187)=-5.1341195448976800E+00
coeff( 188)=-5.8329263165962137E+00
coeff( 189)= 1.5408464915237856E+01
coeff( 190)= 4.5651628422285121E+01
coeff( 191)= 1.9167910331746203E+01
coeff( 192)=-2.1392211702222057E+00
coeff( 193)= 1.0619815609902377E+01
coeff( 194)=-1.4471977245168652E+01
coeff( 195)= 9.2584430205531199E+00
coeff( 196)= 2.8658619410498659E+01
coeff( 197)= 3.3449118802285959E+01
coeff( 198)=-1.7051121763695530E+01
coeff( 199)=-2.3736841778651367E+01
coeff( 200)= 1.2006907911652064E+01
coeff( 201)= 1.3321890655752057E+01
coeff( 202)= 5.4332460701740750E+01
coeff( 203)= 3.2062987760447250E+01
coeff( 204)= 3.0838735493853875E+01
coeff( 205)= 6.2585936889944183E+00
coeff( 206)=-2.5173060107610194E+01
coeff( 207)=-3.7760579098680594E+01
coeff( 208)=-2.4767312116805325E+01
coeff( 209)= 1.2685808845831726E+01
coeff( 210)=-9.7225422999402404E+00
coeff( 211)= 1.0063673440882171E+01
coeff( 212)=-1.2360440537665076E+01
coeff( 213)= 1.2944596434284454E+01
coeff( 214)= 2.9397919995720843E+00
coeff( 215)= 5.9043114482378591E+00
coeff( 216)=-3.7705862133443064E+00
coeff( 217)= 4.7948327031867244E-01
coeff( 218)= 6.8806518551328608E+00
coeff( 219)= 3.7785299332640931E-01
coeff( 220)= 1.8084681605195978E+01
coeff( 221)= 5.4166211154512567E+00
coeff( 222)= 1.4967820823367470E+01
coeff( 223)=-1.0431177930157205E+01
coeff( 224)=-3.3891629186260315E+00
coeff( 225)=-1.9015517307645242E+00
coeff( 226)= 1.3334079256142092E+01
coeff( 227)=-2.7587603718808191E+00
coeff( 228)= 1.3109249164857653E+01
coeff( 229)=-1.4520093632759032E+01
coeff( 230)= 8.0357891845559397E+00
coeff( 231)=-1.0213258603065642E+01
coeff( 232)=-4.4579280779159189E+00
coeff( 233)= 4.0300426940412670E+00
coeff( 234)=-3.0667980753802717E+00
coeff( 235)=-3.5641816611265433E+00
coeff( 236)= 4.2239877548476601E+00
coeff( 237)= 3.5066004866800058E+00
coeff( 238)= 1.8597711977532172E+00
coeff( 239)=-4.4002140585058243E+00
coeff( 240)= 9.5152517726537189E-01
coeff( 241)= 9.2279572212359380E+00
coeff( 242)= 1.6889488149999682E+01
coeff( 243)=-2.9730988015262336E+00
coeff( 244)=-5.9365416339430359E+00
coeff( 245)= 2.0991164185251692E+00
coeff( 246)= 6.8734904554934943E+01
coeff( 247)= 2.9121059465073820E+01
coeff( 248)= 3.9985864821113729E+01
coeff( 249)= 3.7097579989740737E+01
coeff( 250)= 8.0312284979777395E+00
coeff( 251)=-2.4335284098354791E+01
coeff( 252)=-6.5544561101875143E+00
coeff( 253)= 2.7576629051631627E+01
coeff( 254)= 3.2134714842874040E+01
coeff( 255)=-2.5018016181008864E+01
coeff( 256)=-4.4706286675739815E+00
coeff( 257)=-3.3698724650393834E+00
coeff( 258)= 1.1906970263351496E+01
coeff( 259)=-7.7374711518659737E+01
coeff( 260)= 6.3388116974037985E+00
coeff( 261)=-1.3520117599770241E+00
coeff( 262)=-1.2158018642266327E+01
coeff( 263)=-2.0287439396991132E+01
coeff( 264)=-9.1042930025275326E-01
coeff( 265)=-1.5407273356388069E+01
coeff( 266)=-2.5366330810400061E+01
coeff( 267)= 5.7734866834646681E-01
coeff( 268)=-4.5541397449081297E-02
coeff( 269)= 9.2834816956993524E+00
coeff( 270)=-1.1073041455569916E+01
coeff( 271)= 1.8177402507004399E+00
coeff( 272)=-1.4035787682172822E+00
coeff( 273)= 6.5138000006779189E+00
coeff( 274)= 1.1797508184005149E+01
coeff( 275)=-1.0057951915702960E+01
coeff( 276)= 4.8524390113855037E-01
coeff( 277)=-7.2055284390639605E+00
coeff( 278)= 1.2122250404699228E+01
coeff( 279)=-7.1289520672441773E+00
coeff( 280)= 2.0277910041301843E+00
coeff( 281)= 6.1403595922970364E+00
coeff( 282)=-4.7546851162429169E+00
coeff( 283)=-4.8832989901362032E+00
coeff( 284)=-8.0397665437975760E+00
coeff( 285)=-5.4110341163665776E+00
coeff( 286)=-5.6050706087195525E+00
coeff( 287)= 3.0822405623268878E+00
coeff( 288)= 2.8511945324266299E+00
coeff( 289)= 6.2352633042054322E+00
coeff( 290)= 1.6534145195771548E+00
coeff( 291)= 6.5878940547722609E+00
coeff( 292)= 3.7874692210260397E-01
coeff( 293)= 4.3858106437200490E+00
coeff( 294)=-3.5731338732340059E+00
coeff( 295)= 6.3331265598684663E+00
coeff( 296)= 5.9507898683682807E-03
coeff( 297)=-3.0257469424846972E+00
coeff( 298)= 7.8081476151475360E-01
coeff( 299)=-2.9428547266534588E-01
coeff( 300)= 6.8242607924109082E+00
coeff( 301)= 2.4630983937785751E+00
coeff( 302)= 1.1149800536988030E+01
coeff( 303)=-1.1757723215143953E+01
coeff( 304)=-1.4506095164481929E+00
coeff( 305)=-1.6637529338191154E+01
coeff( 306)=-1.1374970210817613E+01
coeff( 307)= 2.3254750204740063E+01
coeff( 308)=-7.8600587811999245E-02
coeff( 309)= 3.6362532612750291E+00
coeff( 310)=-7.2524904184855385E+00
coeff( 311)= 3.0260154226141434E+00
coeff( 312)= 4.2738265459812269E-01
coeff( 313)=-6.7210675915619031E+00
coeff( 314)= 5.7821805134369608E-01
coeff( 315)= 9.1591018136146296E-01
coeff( 316)= 7.6079053670714920E+00
coeff( 317)= 3.2970307031051433E+00
coeff( 318)=-1.7288302677017762E+00
coeff( 319)=-7.2284405952481112E+00
coeff( 320)= 7.3422595478201984E+00
coeff( 321)= 1.2776535317857209E+00
coeff( 322)= 1.1405689882566204E+01
coeff( 323)= 4.9728067525218220E+00
coeff( 324)=-7.6169966934985283E+00
coeff( 325)= 1.5921376136961307E+00
coeff( 326)=-1.5215783918689535E+01
coeff( 327)=-1.2924578598815611E+01
coeff( 328)= 5.8632256450176479E-01
coeff( 329)=-8.8894943928266124E-01
coeff( 330)=-8.0073995535153664E-01
coeff( 331)= 5.9438036866853849E+00
coeff( 332)= 1.7820389134330876E+00
coeff( 333)= 2.9621985520326408E+00
coeff( 334)=-2.4584991696723928E+00
coeff( 335)= 2.6901038641257689E+00
coeff( 336)=-1.0104268587557299E+00
coeff( 337)= 9.3577158150972846E+00
coeff( 338)=-5.8843324151213361E+00
coeff( 339)=-5.3630603406222130E+00
coeff( 340)=-7.9997341392911858E+00
coeff( 341)=-5.3824395602891215E+00
coeff( 342)= 3.2990267976545828E-01
coeff( 343)= 1.7362807239288769E+00
coeff( 344)= 8.0662761411012767E+00
coeff( 345)= 3.3126201394911767E+00
coeff( 346)= 4.8174002868156207E-01
coeff( 347)= 5.3421658583665277E+00
coeff( 348)=-1.2591414044441285E+00
coeff( 349)= 2.1573016855900979E+00
coeff( 350)=-1.3336050984705796E+00
coeff( 351)=-1.0473666047687551E+00
coeff( 352)=-5.9489421300999390E-01
coeff( 353)= 6.9929375428285512E-01
coeff( 354)=-1.2061214562742109E+00
coeff( 355)= 2.4019501502113645E+00
coeff( 356)=-2.5381579964093670E-01
coeff( 357)=-1.8294997736235903E+00
coeff( 358)=-6.6881944077709043E+00
coeff( 359)=-4.8119990665314729E+00
coeff( 360)=-1.1725216471689937E+00
coeff( 361)= 8.2216305182407035E-01
coeff( 362)= 1.3566307839009424E+00
coeff( 363)=-7.3394879008878302E+00
coeff( 364)=-3.2269923367124327E+00
coeff( 365)= 5.7045332589363866E+00
coeff( 366)=-3.3263511004498536E+00
coeff( 367)=-1.4140602820778942E+00
coeff( 368)=-1.0502021736037005E+01
coeff( 369)=-5.3234478384585868E+00
coeff( 370)=-1.8848961171168646E-02
coeff( 371)= 9.9200846016681421E+00
coeff( 372)=-4.1815784704692387E+00
coeff( 373)=-1.2942808913269956E+00
coeff( 374)= 4.6643882281417479E+00
coeff( 375)=-7.9598819531060307E+00
coeff( 376)=-6.7520475162486120E+00
coeff( 377)=-6.9829130545211058E+00
coeff( 378)=-1.0034285278666655E+01
coeff( 379)=-7.0781705625039582E-01
coeff( 380)= 3.9436515443427007E+00
coeff( 381)= 3.8913045346975355E+00
coeff( 382)= 1.1940597678487133E+01
coeff( 383)= 6.1097521229717779E+00
coeff( 384)=-8.7298274234450102E-01
coeff( 385)=-3.7291852473357872E+00
coeff( 386)= 9.0277684295954314E-02
coeff( 387)=-5.0983114827960438E+00
coeff( 388)= 1.0141419721264890E+00
coeff( 389)=-5.7350299060199461E+00
coeff( 390)= 5.6780248958372317E+00
coeff( 391)= 2.4116011010951675E+00
coeff( 392)=-1.6370714865185396E+00
coeff( 393)=-3.2833621169325218E+00
coeff( 394)= 3.8267916192465576E+00
coeff( 395)=-4.3943561432197559E-01
coeff( 396)= 7.2888843194628876E+00
coeff( 397)=-2.9079908791063032E+00
coeff( 398)= 1.6226549278130971E+00
coeff( 399)=-3.3127185009559277E+00
coeff( 400)=-2.5839199865255278E+00
coeff( 401)=-2.2726342818166609E-01
coeff( 402)=-3.5421348194089957E+00
coeff( 403)=-4.5513161109458284E+00
coeff( 404)= 3.9908872812186829E+00
coeff( 405)= 4.5106303306504247E-01
coeff( 406)=-5.3421478657072425E+00
coeff( 407)= 5.8214185485623915E+00
coeff( 408)=-5.3155635415386246E+00
coeff( 409)= 3.2539986125926190E+00
coeff( 410)= 1.6996447638781831E-02
coeff( 411)=-7.4193962564861049E-01
coeff( 412)= 8.7675490447674846E+00
coeff( 413)= 2.8808629273634927E+00
coeff( 414)=-2.7680844077931361E+00
coeff( 415)=-4.9751635074848739E+00
coeff( 416)= 2.0741924172013864E+00
coeff( 417)= 7.5737664930299542E-01
coeff( 418)=-5.6495988755499860E-01
coeff( 419)= 1.1629340799717023E+00
coeff( 420)=-5.2049611421062880E-01
coeff( 421)= 8.7729086544613677E-01
coeff( 422)= 6.5659790422783249E-01
coeff( 423)= 1.7458731112522110E+00
coeff( 424)= 6.6806445145171578E-01
coeff( 425)= 4.7901688301314205E-01
coeff( 426)=-2.1715987308864784E+00
coeff( 427)=-2.0563479980063657E+00
coeff( 428)=-3.3982017177905894E+00
coeff( 429)= 1.0293041664853098E+00
coeff( 430)= 2.8483767288521338E+00
coeff( 431)= 8.5727309080621772E-01
coeff( 432)= 2.2367852586117212E+00
coeff( 433)= 7.6012683473762532E-01
coeff( 434)=-1.5488711017492334E+00
coeff( 435)=-1.5408171014810932E+00
coeff( 436)=-5.6346128932813384E+00
coeff( 437)=-1.9995928773667704E+00
coeff( 438)=-6.2922933559044427E+00
coeff( 439)= 1.0495761411243938E+00
coeff( 440)=-2.7496171620067202E+00
coeff( 441)= 3.6277375229774682E+00
coeff( 442)= 5.0862035813291344E+00
coeff( 443)=-1.0516850470905288E+00
coeff( 444)=-1.5663741571783504E-01
coeff( 445)=-7.0362343187240661E-01
coeff( 446)=-5.1954518120110382E-01
coeff( 447)=-1.7920661082894414E+00
coeff( 448)=-1.8361062941397768E-01
coeff( 449)= 1.4677468225782788E+00
coeff( 450)= 1.4425373535546366E+00
coeff( 451)= 8.9033576949600024E-01
coeff( 452)=-5.2483450651880281E-01
coeff( 453)=-1.4212406016577979E+00
coeff( 454)= 6.1952476618845942E-01
coeff( 455)= 5.1237013538107301E-01
coeff( 456)=-8.5471671913874054E+00
coeff( 457)=-1.0137569976907347E+01
coeff( 458)=-1.9243866870903965E+00
coeff( 459)=-5.1206011300806766E+00
coeff( 460)=-4.2204748125277813E+00
coeff( 461)=-1.9393068704635943E+00
coeff( 462)= 4.1300688478532566E+00
coeff( 463)= 5.8818210735963667E+00
coeff( 464)=-4.1069663699095873E+00
coeff( 465)=-3.5476896648967138E+00
coeff( 466)=-4.2060897162669777E+00
coeff( 467)= 4.9254444191547320E+00
coeff( 468)= 2.1932167821986148E+00
coeff( 469)=-8.0249841937850663E-01
coeff( 470)=-2.3632153860451459E+00
coeff( 471)= 1.5459750090784969E+01
coeff( 472)=-1.3143384556817070E+00
coeff( 473)= 1.2022917115489558E+00
coeff( 474)= 4.5527784863356739E+00
coeff( 475)=-3.7710456828064798E+00
coeff( 476)= 5.6689136378178961E-01
coeff( 477)= 7.3748580324229609E+00
coeff( 478)= 2.3098236262812062E+00
coeff( 479)=-6.5554792003307694E-01
coeff( 480)= 5.5261102066966634E+00
coeff( 481)= 3.6727655009424159E-01
coeff( 482)= 1.2448273095992461E+00
coeff( 483)= 4.7930315522029487E-01
coeff( 484)= 3.0773456284981751E+00
coeff( 485)=-1.4511962892944019E-01
coeff( 486)=-3.9776579068266229E+00
coeff( 487)= 6.1835774540881816E-01
coeff( 488)= 7.7055421528095847E-01
coeff( 489)=-5.0020278456810727E+00
coeff( 490)= 1.1784267167561937E+00
coeff( 491)=-8.8987478315636226E-01
coeff( 492)=-4.7808056737807381E-01
coeff( 493)=-3.3691660296079511E+00
coeff( 494)=-2.6265723419735343E-01
coeff( 495)=-8.7492870931696942E-01
coeff( 496)= 3.6442600203873581E+00
coeff( 497)= 1.9103077804053479E+00
coeff( 498)= 1.3438398442987906E-01
coeff( 499)= 1.0551166438806825E+00
coeff( 500)= 3.0782099063846251E-02
coeff( 501)= 1.7246852536596422E+00
coeff( 502)= 5.3778344331685013E-01
coeff( 503)=-2.2329829239294585E+00
coeff( 504)=-1.6760798022951113E+00
coeff( 505)=-4.6501834200164699E-01
coeff( 506)= 2.6572374926462561E+00
coeff( 507)=-2.9323003964805388E+00
coeff( 508)= 5.5664509766479775E-01
coeff( 509)=-1.5940284657348112E+00
coeff( 510)=-1.6565172589031978E+00
coeff( 511)=-9.4469559211774079E-01
coeff( 512)= 2.7142062336330586E-01
coeff( 513)=-9.7993861235367585E-01
coeff( 514)= 1.1074879054907552E+00
coeff( 515)= 1.5158517360745397E-01
coeff( 516)= 1.1380039830759467E+00
coeff( 517)=-4.5637146763668796E-01
coeff( 518)= 2.3247297578076254E-01
coeff( 519)=-6.2034783967496593E-03
coeff( 520)=-5.5722818079045522E-01
coeff( 521)=-1.5292587358926935E+00
coeff( 522)= 2.4307993087409718E-02
coeff( 523)=-7.7106553494871971E-02
coeff( 524)= 8.1111705017087987E-01
coeff( 525)=-1.5870165133142693E+00
coeff( 526)=-3.3413795100971089E-01
coeff( 527)= 1.9516137300104580E-01
coeff( 528)=-2.2651422070348451E-01
coeff( 529)= 1.4574178070772534E-01
coeff( 530)=-1.8977544146844305E+00
coeff( 531)= 1.3425367678023903E-01
coeff( 532)=-6.5263250578002259E+00
coeff( 533)= 3.0007585659700986E+00
coeff( 534)= 2.2506719143720071E+00
coeff( 535)= 7.8934470807734003E-01
coeff( 536)= 1.0167237589363380E+00
coeff( 537)= 4.0528836000784514E+00
coeff( 538)=-3.0617340877585759E+00
coeff( 539)=-2.5630176558402664E+00
coeff( 540)=-5.5901211792054900E-02
coeff( 541)= 2.6784697915804521E+00
coeff( 542)=-1.0002594864968397E+00
coeff( 543)= 6.4829621383442682E-01
coeff( 544)=-5.8982680699660468E-03
coeff( 545)=-1.5338598743207985E-01
coeff( 546)= 1.3558093973838310E+00
  return
end if

entry u33_pot_8(xyz,v, dxyz)

r12=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r13=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r14=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r23=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r24=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r34=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

re_hh = 0.741
re_oh = 0.970

ra_hh=ra1
ra_oh=ra2
rb_hh=rb1
rb_oh=rb2

x(1)=exp(-((r12-ra1))/a1 -((r12-rb1)**2)/b1)
x(2)=exp(-((r13-ra1))/a1 -((r13-rb1)**2)/b1)
x(3)=exp(-((r14-ra2))/a2 -((r14-rb2)**2)/b2)
x(4)=exp(-((r23-ra1))/a1 -((r23-rb1)**2)/b1)
x(5)=exp(-((r24-ra2))/a2 -((r24-rb2)**2)/b2)
x(6)=exp(-((r34-ra2))/a2 -((r34-rb2)**2)/b2)

call ev_pol_con_8(x,p)
v=dot_product(coeff,p) + (ohstar(r14)+ohstar(r24)+ohstar(r34)+hh(r12)+hh(r13)+hh(r23))





call grad_ev_pol_con_8(x,dpdx1,dpdx2,dpdx3,dpdx4,dpdx5,dpdx6)

dx1 = dot_product(coeff,dpdx1)
dx2 = dot_product(coeff,dpdx2)
dx3 = dot_product(coeff,dpdx3)
dx4 = dot_product(coeff,dpdx4)
dx5 = dot_product(coeff,dpdx5)
dx6 = dot_product(coeff,dpdx6)

dr1=-x(1)*(1.0d0/(a1)+2.0d0*(r12-rb1)/(b1))*dx1 + grad_hh(r12)
dr2=-x(2)*(1.0d0/(a1)+2.0d0*(r13-rb1)/(b1))*dx2 + grad_hh(r13)
dr3=-x(3)*(1.0d0/(a2)+2.0d0*(r14-rb2)/(b2))*dx3 + grad_ohstar(r14)
dr4=-x(4)*(1.0d0/(a1)+2.0d0*(r23-rb1)/(b1))*dx4 + grad_hh(r23)
dr5=-x(5)*(1.0d0/(a2)+2.0d0*(r24-rb2)/(b2))*dx5 + grad_ohstar(r24)
dr6=-x(6)*(1.0d0/(a2)+2.0d0*(r34-rb2)/(b2))*dx6 + grad_ohstar(r34)

dxyz(1,1)=dr1/r12*(xyz(1,1)-xyz(2,1))+dr2/r13*(xyz(1,1)-xyz(3,1))+dr3/r14*(xyz(1,1)-xyz(4,1))
dxyz(1,2)=dr1/r12*(xyz(1,2)-xyz(2,2))+dr2/r13*(xyz(1,2)-xyz(3,2))+dr3/r14*(xyz(1,2)-xyz(4,2))
dxyz(1,3)=dr1/r12*(xyz(1,3)-xyz(2,3))+dr2/r13*(xyz(1,3)-xyz(3,3))+dr3/r14*(xyz(1,3)-xyz(4,3))

dxyz(2,1)=-dr1/r12*(xyz(1,1)-xyz(2,1))+dr4/r23*(xyz(2,1)-xyz(3,1))+dr5/r24*(xyz(2,1)-xyz(4,1))
dxyz(2,2)=-dr1/r12*(xyz(1,2)-xyz(2,2))+dr4/r23*(xyz(2,2)-xyz(3,2))+dr5/r24*(xyz(2,2)-xyz(4,2))
dxyz(2,3)=-dr1/r12*(xyz(1,3)-xyz(2,3))+dr4/r23*(xyz(2,3)-xyz(3,3))+dr5/r24*(xyz(2,3)-xyz(4,3))

dxyz(3,1)=-dr2/r13*(xyz(1,1)-xyz(3,1))-dr4/r23*(xyz(2,1)-xyz(3,1))+dr6/r34*(xyz(3,1)-xyz(4,1))
dxyz(3,2)=-dr2/r13*(xyz(1,2)-xyz(3,2))-dr4/r23*(xyz(2,2)-xyz(3,2))+dr6/r34*(xyz(3,2)-xyz(4,2))
dxyz(3,3)=-dr2/r13*(xyz(1,3)-xyz(3,3))-dr4/r23*(xyz(2,3)-xyz(3,3))+dr6/r34*(xyz(3,3)-xyz(4,3))

dxyz(4,1)=-dr3/r14*(xyz(1,1)-xyz(4,1))-dr5/r24*(xyz(2,1)-xyz(4,1))-dr6/r34*(xyz(3,1)-xyz(4,1))
dxyz(4,2)=-dr3/r14*(xyz(1,2)-xyz(4,2))-dr5/r24*(xyz(2,2)-xyz(4,2))-dr6/r34*(xyz(3,2)-xyz(4,2))
dxyz(4,3)=-dr3/r14*(xyz(1,3)-xyz(4,3))-dr5/r24*(xyz(2,3)-xyz(4,3))-dr6/r34*(xyz(3,3)-xyz(4,3))

return
end  subroutine

real*8 function d3bj_hh(r)
implicit none
real*8, intent(in) :: r
real*8 :: s6, s8, C6au, C8au, C6, C8, a1, a2, R0, fR0

s6 = 1.0
s8 = 2.0

C6au = 7.5915999999999997
C8au = 91.76991785999999

C6 = C6au*0.021958709
C8 = C8au*0.0061490647

a1 = 0.5299
a2 = 2.2

R0 = sqrt(C8/C6)
fR0 = a1*R0 + a2

d3bj_hh = -s6*C6/(r**6 + fR0**6) -s8*C8/(r**8 + fR0**8)

end function d3bj_hh

real*8 function grad_d3bj_hh(r)
implicit none
real*8, intent(in) :: r
real*8 :: s6, s8, C6au, C8au, C6, C8, a1, a2, R0, fR0

s6 = 1.0
s8 = 2.0

C6au = 7.5915999999999997
C8au = 91.76991785999999

C6 = C6au*0.021958709
C8 = C8au*0.0061490647

a1 = 0.5299
a2 = 2.2

R0 = sqrt(C8/C6)
fR0 = a1*R0 + a2

grad_d3bj_hh = 6.0*s6*C6*r**5/(r**6 + fR0**6)**2  + 8.0*s8*C8*r**7/(r**8 + fR0**8)**2

end function grad_d3bj_hh

real*8 function d3bj_oh(r)
implicit none
real*8, intent(in) :: r
real*8 :: s6, s8, C6au, C8au, C6, C8, a1, a2, R0, fR0

s6 = 1.0
s8 = 2.0

C6au = 10.84963549802481
C8au = 169.4591809288624

C6 = C6au*0.021958709
C8 = C8au*0.0061490647

a1 = 0.5299
a2 = 2.2

R0 = sqrt(C8/C6)
fR0 = a1*R0 + a2

d3bj_oh = -s6*C6/(r**6 + fR0**6) -s8*C8/(r**8 + fR0**8)

end function d3bj_oh

real*8 function grad_d3bj_oh(r)
implicit none
real*8, intent(in) :: r
real*8 :: s6, s8, C6au, C8au, C6, C8, a1, a2, R0, fR0

s6 = 1.0
s8 = 2.0

C6au = 10.84963549802481
C8au = 169.4591809288624

C6 = C6au*0.021958709
C8 = C8au*0.0061490647

a1 = 0.5299
a2 = 2.2

R0 = sqrt(C8/C6)
fR0 = a1*R0 + a2

grad_d3bj_oh = 6.0*s6*C6*r**5/(r**6 + fR0**6)**2 + 8.0*s8*C8*r**7/(r**8 + fR0**8)**2

end function grad_d3bj_oh

real*8 function hh(r)
implicit none
real*8 :: d3bj_hh
real*8, intent(in) :: r
real*8 :: vsr, alpha, beta, c(8), p(8)
integer :: i

alpha = 0.53931748805004220
beta = 1.6459680933202987

c(1) = -0.10896821437780629
c(2) = -0.29798048492728535
c(3) = 0.20089464558076681
c(4) = -8.2216773488458517E-002
c(5) = 0.15456941633221136
c(6) = 0.29533240911519809
c(7) = 0.22790629347946670
c(8) = 0.93346028818320326

do i=0, 7
    p(i+1) = exp(-alpha*beta**i*r**2)
end do

vsr = dot_product(c, p)

hh = vsr + d3bj_hh(r)
end function hh

real*8 function grad_hh(r)
implicit none
real*8 :: grad_d3bj_hh
real*8, intent(in) :: r
real*8 :: vsr, alpha, beta, c(8), p(8)
integer :: i

alpha = 0.53931748805004220
beta = 1.6459680933202987

c(1) = -0.10896821437780629
c(2) = -0.29798048492728535
c(3) = 0.20089464558076681
c(4) = -8.2216773488458517E-002
c(5) = 0.15456941633221136
c(6) = 0.29533240911519809
c(7) = 0.22790629347946670
c(8) = 0.93346028818320326

do i=0, 7
p(i+1) = -2.0*alpha*beta**i*r*exp(-alpha*beta**i*r**2)
end do

vsr = dot_product(c, p)

grad_hh = vsr + grad_d3bj_hh(r)

end function grad_hh

real*8 function oh(r)
implicit none
real*8 :: d3bj_oh
real*8, intent(in) :: r
real*8 :: vsr, alpha, beta, c(8), p(8)
integer :: i

alpha = 0.51045358916334960
beta = 1.5433662455565513

c(1) = -3.3264921812231049E-002
c(2) = -0.19210399441400144
c(3) = -0.92587310862479089
c(4) = 1.9962515830881151
c(5) = -2.3610928168877425
c(6) = 4.0512644189749327
c(7) = -2.1935688479612669
c(8) = 6.4929855978115452

do i=0, 7
    p(i+1) = exp(-alpha*beta**i*r**2)
end do

vsr = dot_product(c, p)
oh = vsr + d3bj_oh(r)

end function oh 

real*8 function grad_oh(r)
implicit none
real*8 :: grad_d3bj_oh
real*8, intent(in) :: r
real*8 :: vsr, alpha, beta, c(8), p(8)
integer :: i

alpha = 0.51045358916334960
beta = 1.5433662455565513

c(1) = -3.3264921812231049E-002
c(2) = -0.19210399441400144
c(3) = -0.92587310862479089
c(4) = 1.9962515830881151
c(5) = -2.3610928168877425
c(6) = 4.0512644189749327
c(7) = -2.1935688479612669
c(8) = 6.4929855978115452

do i=0, 7
    p(i+1) = -2.0*alpha*beta**i*r*exp(-alpha*beta**i*r**2)
end do

vsr = dot_product(c, p)
grad_oh = vsr + grad_d3bj_oh(r)

end function grad_oh 

real*8 function ohstar(r)
implicit none
real*8, intent(in) :: r
integer :: i
real*8 :: alr, blr, vsr, vlr, fs
real*8, dimension(3) :: plr, clr
real*8 :: asr, bsr
real*8, dimension(6) :: psr, csr


alr = 2.0991414262152035
blr = 0.55368872045847861
clr(1) = 4.8925016098582264
clr(2) = -0.25648455026027650
clr(3) = 0.19261955256047161

do i = 1, 3
    plr(i) = exp(-alr*blr**(i-1)*r**2)
end do

vlr = dot_product(clr, plr)

asr = 0.22510201499513907
bsr = 2.2657615066074954

csr(1) = 0.35455513969718444    
csr(2) = -0.52912871421810181     
csr(3) = -6.0697181582250126E-002
csr(4) = 0.47976112924154690     
csr(5) = 2.3310416022954366     
csr(6) = 5.6903894590457540

do i = 1, 6
    psr(i) = exp(-asr*bsr**(i-1)*r**2)
end do

vsr = dot_product(csr, psr)

fs = 0.5*(1.0 - tanh(10.0*(r-1.577)))

ohstar = fs*vsr + (1.0 - fs)*vlr

end function ohstar

real*8 function grad_ohstar(r)
implicit none
real*8, intent(in) :: r
integer :: i
real*8 :: alr, blr, vsr, vlr, fs, gsr, glr, gfs
real*8, dimension(3) :: plr, clr, pglr
real*8 :: asr, bsr
real*8, dimension(6) :: psr, csr, pgsr


alr = 2.0991414262152035
blr = 0.55368872045847861
clr(1) = 4.8925016098582264
clr(2) = -0.25648455026027650
clr(3) = 0.19261955256047161

do i = 1, 3
    plr(i) = exp(-alr*blr**(i-1)*r**2)
    pglr(i) = -2.0*alr*blr**(i-1)*r*exp(-alr*blr**(i-1)*r**2)
end do

vlr = dot_product(clr, plr)
glr = dot_product(clr, pglr)

asr = 0.22510201499513907
bsr = 2.2657615066074954

csr(1) = 0.35455513969718444    
csr(2) = -0.52912871421810181     
csr(3) = -6.0697181582250126E-002
csr(4) = 0.47976112924154690     
csr(5) = 2.3310416022954366     
csr(6) = 5.6903894590457540

do i = 1, 6
    psr(i) = exp(-asr*bsr**(i-1)*r**2)
    pgsr(i) = -2.0*asr*bsr**(i-1)*r*exp(-asr*bsr**(i-1)*r**2)
end do

vsr = dot_product(csr, psr)
gsr = dot_product(csr, pgsr)

fs = 0.5*(1.0 - tanh(10.0*(r-1.577)))
gfs = -0.5*10.0/(cosh(10.0*(r-1.577)))**2

grad_ohstar = gfs*(vsr - vlr) + fs*gsr + (1.0 - fs)*glr

end function grad_ohstar

!===========================================================================
! *def ev_pol_con_8(x,q)
! subtoutine of evaluating polynomials at 8th order
!===========================================================================
subroutine ev_pol_con_8(x,p)
implicit none
real*8,dimension(1:6),intent(in)::x
real*8,dimension(1:546),intent(out)::p
real*8::x1,x2,x3,x4,x5,x6

x1=x(1)
x2=x(2)
x3=x(3)
x4=x(4)
x5=x(5)
x6=x(6)

p(1) = 1.d+0
p(2) = x5*x6+x3*x6+x3*x5
p(3) = x4*x6+x2*x6+x4*x5+x1*x5+x2*x3+x1*x3
p(4) = x2*x4+x1*x4+x1*x2
p(5) = x3*x5*x6
p(6) = x2*x5*x6+x1*x5*x6+x3*x4*x6+x1*x3*x6+x3*x4*x5+x2*x3*x5
p(7) = x4*x5*x6+x2*x3*x6+x1*x3*x5
p(8) = x1*x4*x6+x1*x2*x6+x2*x4*x5+x1*x2*x5+x2*x3*x4+x1*x3*x4
p(9) = x1*x2*x4
p(10) = x2*x4*x6+x1*x4*x5+x1*x2*x3
p(11) = x5*x6**2+x3*x6**2+x5**2*x6+x3**2*x6+x3*x5**2+x3**2*x5
p(12) = x4*x6**2+x2*x6**2+x4*x5**2+x1*x5**2+x2*x3**2+x1*x3**2
p(13) = x4**2*x6+x2**2*x6+x4**2*x5+x1**2*x5+x2**2*x3+x1**2*x3
p(14) = x2*x4**2+x1*x4**2+x2**2*x4+x1**2*x4+x1*x2**2+x1**2*x2
p(15) = x3*x4*x5*x6+x2*x3*x5*x6+x1*x3*x5*x6
p(16) = x1*x2*x5*x6+x1*x3*x4*x6+x2*x3*x4*x5
p(17) = x2*x4*x5*x6+x1*x4*x5*x6+x2*x3*x4*x6+x1*x2*x3*x6+x1*x3*x4*&
&x5+x1*x2*x3*x5
p(18) = x1*x2*x4*x6+x1*x2*x4*x5+x1*x2*x3*x4
p(19) = x3*x5*x6**2+x3*x5**2*x6+x3**2*x5*x6
p(20) = x1*x5*x6**2+x1*x3*x6**2+x2*x5**2*x6+x3**2*x4*x6+x2*x3*x5*&
&*2+x3**2*x4*x5
p(21) = x2*x5*x6**2+x3*x4*x6**2+x1*x5**2*x6+x1*x3**2*x6+x3*x4*x5*&
&*2+x2*x3**2*x5
p(22) = x4*x5*x6**2+x2*x3*x6**2+x4*x5**2*x6+x2*x3**2*x6+x1*x3*x5*&
&*2+x1*x3**2*x5
p(23) = x1*x4*x6**2+x1*x2*x6**2+x2*x4*x5**2+x1*x2*x5**2+x2*x3**2*&
&x4+x1*x3**2*x4
p(24) = x2*x4*x6**2+x1*x4*x5**2+x1*x2*x3**2
p(25) = x2**2*x5*x6+x1**2*x5*x6+x3*x4**2*x6+x1**2*x3*x6+x3*x4**2*&
&x5+x2**2*x3*x5
p(26) = x4**2*x5*x6+x2**2*x3*x6+x1**2*x3*x5
p(27) = x1**2*x4*x6+x1**2*x2*x6+x2**2*x4*x5+x1*x2**2*x5+x2*x3*x4*&
&*2+x1*x3*x4**2
p(28) = x1*x4**2*x6+x1*x2**2*x6+x2*x4**2*x5+x1**2*x2*x5+x2**2*x3*&
&x4+x1**2*x3*x4
p(29) = x1*x2*x4**2+x1*x2**2*x4+x1**2*x2*x4
p(30) = x2*x4**2*x6+x2**2*x4*x6+x1*x4**2*x5+x1**2*x4*x5+x1*x2**2*&
&x3+x1**2*x2*x3
p(31) = x5**2*x6**2+x3**2*x6**2+x3**2*x5**2
p(32) = x5*x6**3+x3*x6**3+x5**3*x6+x3**3*x6+x3*x5**3+x3**3*x5
p(33) = x4*x6**3+x2*x6**3+x4*x5**3+x1*x5**3+x2*x3**3+x1*x3**3
p(34) = x4**2*x6**2+x2**2*x6**2+x4**2*x5**2+x1**2*x5**2+x2**2*x3*&
&*2+x1**2*x3**2
p(35) = x2**2*x4**2+x1**2*x4**2+x1**2*x2**2
p(36) = x4**3*x6+x2**3*x6+x4**3*x5+x1**3*x5+x2**3*x3+x1**3*x3
p(37) = x2*x4**3+x1*x4**3+x2**3*x4+x1**3*x4+x1*x2**3+x1**3*x2
p(38) = x2*x3*x4*x5*x6+x1*x3*x4*x5*x6+x1*x2*x3*x5*x6
p(39) = x1*x2*x4*x5*x6+x1*x2*x3*x4*x6+x1*x2*x3*x4*x5
p(40) = x1*x3*x5*x6**2+x2*x3*x5**2*x6+x3**2*x4*x5*x6
p(41) = x3*x4*x5*x6**2+x2*x3*x5*x6**2+x3*x4*x5**2*x6+x1*x3*x5**2*&
&x6+x2*x3**2*x5*x6+x1*x3**2*x5*x6
p(42) = x1*x2*x5*x6**2+x1*x3*x4*x6**2+x1*x2*x5**2*x6+x1*x3**2*x4*&
&x6+x2*x3*x4*x5**2+x2*x3**2*x4*x5
p(43) = x1*x4*x5*x6**2+x1*x2*x3*x6**2+x2*x4*x5**2*x6+x2*x3**2*x4*&
&x6+x1*x2*x3*x5**2+x1*x3**2*x4*x5
p(44) = x2*x4*x5*x6**2+x2*x3*x4*x6**2+x1*x4*x5**2*x6+x1*x2*x3**2*&
&x6+x1*x3*x4*x5**2+x1*x2*x3**2*x5
p(45) = x1*x2*x4*x6**2+x1*x2*x4*x5**2+x1*x2*x3**2*x4
p(46) = x3*x4**2*x5*x6+x2**2*x3*x5*x6+x1**2*x3*x5*x6
p(47) = x1*x2**2*x5*x6+x1**2*x2*x5*x6+x1*x3*x4**2*x6+x1**2*x3*x4*&
&x6+x2*x3*x4**2*x5+x2**2*x3*x4*x5
p(48) = x1**2*x2*x4*x6+x1*x2**2*x4*x5+x1*x2*x3*x4**2
p(49) = x2**2*x4*x5*x6+x1**2*x4*x5*x6+x2*x3*x4**2*x6+x1**2*x2*x3*&
&x6+x1*x3*x4**2*x5+x1*x2**2*x3*x5
p(50) = x2*x4**2*x5*x6+x1*x4**2*x5*x6+x2**2*x3*x4*x6+x1*x2**2*x3*&
&x6+x1**2*x3*x4*x5+x1**2*x2*x3*x5
p(51) = x1*x2*x4**2*x6+x1*x2**2*x4*x6+x1*x2*x4**2*x5+x1**2*x2*x4*&
&x5+x1*x2**2*x3*x4+x1**2*x2*x3*x4
p(52) = x3*x5**2*x6**2+x3**2*x5*x6**2+x3**2*x5**2*x6
p(53) = x3*x5*x6**3+x3*x5**3*x6+x3**3*x5*x6
p(54) = x1*x5*x6**3+x1*x3*x6**3+x2*x5**3*x6+x3**3*x4*x6+x2*x3*x5*&
&*3+x3**3*x4*x5
p(55) = x2*x5**2*x6**2+x1*x5**2*x6**2+x3**2*x4*x6**2+x1*x3**2*x6*&
&*2+x3**2*x4*x5**2+x2*x3**2*x5**2
p(56) = x4*x5**2*x6**2+x2*x3**2*x6**2+x1*x3**2*x5**2
p(57) = x2*x5*x6**3+x3*x4*x6**3+x1*x5**3*x6+x1*x3**3*x6+x3*x4*x5*&
&*3+x2*x3**3*x5
p(58) = x4*x5*x6**3+x2*x3*x6**3+x4*x5**3*x6+x2*x3**3*x6+x1*x3*x5*&
&*3+x1*x3**3*x5
p(59) = x1*x4*x6**3+x1*x2*x6**3+x2*x4*x5**3+x1*x2*x5**3+x2*x3**3*&
&x4+x1*x3**3*x4
p(60) = x2*x4*x6**3+x1*x4*x5**3+x1*x2*x3**3
p(61) = x1**2*x5*x6**2+x1**2*x3*x6**2+x2**2*x5**2*x6+x3**2*x4**2*&
&x6+x2**2*x3*x5**2+x3**2*x4**2*x5
p(62) = x2**2*x5*x6**2+x3*x4**2*x6**2+x1**2*x5**2*x6+x1**2*x3**2*&
&x6+x3*x4**2*x5**2+x2**2*x3**2*x5
p(63) = x4**2*x5*x6**2+x2**2*x3*x6**2+x4**2*x5**2*x6+x2**2*x3**2*&
&x6+x1**2*x3*x5**2+x1**2*x3**2*x5
p(64) = x1**2*x4*x6**2+x1**2*x2*x6**2+x2**2*x4*x5**2+x1*x2**2*x5*&
&*2+x2*x3**2*x4**2+x1*x3**2*x4**2
p(65) = x1*x4**2*x6**2+x1*x2**2*x6**2+x2*x4**2*x5**2+x1**2*x2*x5*&
&*2+x2**2*x3**2*x4+x1**2*x3**2*x4
p(66) = x2*x4**2*x6**2+x2**2*x4*x6**2+x1*x4**2*x5**2+x1**2*x4*x5*&
&*2+x1*x2**2*x3**2+x1**2*x2*x3**2
p(67) = x1**2*x4**2*x6+x1**2*x2**2*x6+x2**2*x4**2*x5+x1**2*x2**2*&
&x5+x2**2*x3*x4**2+x1**2*x3*x4**2
p(68) = x1*x2**2*x4**2+x1**2*x2*x4**2+x1**2*x2**2*x4
p(69) = x2**2*x4**2*x6+x1**2*x4**2*x5+x1**2*x2**2*x3
p(70) = x2**3*x5*x6+x1**3*x5*x6+x3*x4**3*x6+x1**3*x3*x6+x3*x4**3*&
&x5+x2**3*x3*x5
p(71) = x4**3*x5*x6+x2**3*x3*x6+x1**3*x3*x5
p(72) = x1**3*x4*x6+x1**3*x2*x6+x2**3*x4*x5+x1*x2**3*x5+x2*x3*x4*&
&*3+x1*x3*x4**3
p(73) = x1*x4**3*x6+x1*x2**3*x6+x2*x4**3*x5+x1**3*x2*x5+x2**3*x3*&
&x4+x1**3*x3*x4
p(74) = x1*x2*x4**3+x1*x2**3*x4+x1**3*x2*x4
p(75) = x2*x4**3*x6+x2**3*x4*x6+x1*x4**3*x5+x1**3*x4*x5+x1*x2**3*&
&x3+x1**3*x2*x3
p(76) = x5**2*x6**3+x3**2*x6**3+x5**3*x6**2+x3**3*x6**2+x3**2*x5*&
&*3+x3**3*x5**2
p(77) = x5*x6**4+x3*x6**4+x5**4*x6+x3**4*x6+x3*x5**4+x3**4*x5
p(78) = x4*x6**4+x2*x6**4+x4*x5**4+x1*x5**4+x2*x3**4+x1*x3**4
p(79) = x4**2*x6**3+x2**2*x6**3+x4**2*x5**3+x1**2*x5**3+x2**2*x3*&
&*3+x1**2*x3**3
p(80) = x4**3*x6**2+x2**3*x6**2+x4**3*x5**2+x1**3*x5**2+x2**3*x3*&
&*2+x1**3*x3**2
p(81) = x2**2*x4**3+x1**2*x4**3+x2**3*x4**2+x1**3*x4**2+x1**2*x2*&
&*3+x1**3*x2**2
p(82) = x4**4*x6+x2**4*x6+x4**4*x5+x1**4*x5+x2**4*x3+x1**4*x3
p(83) = x2*x4**4+x1*x4**4+x2**4*x4+x1**4*x4+x1*x2**4+x1**4*x2
p(84) = x1*x2*x3*x4*x5*x6
p(85) = x1*x3*x4*x5*x6**2+x1*x2*x3*x5*x6**2+x2*x3*x4*x5**2*x6+x1&
&*x2*x3*x5**2*x6+x2*x3**2*x4*x5*x6+x1*x3**2*x4*x5*x6
p(86) = x2*x3*x4*x5*x6**2+x1*x3*x4*x5**2*x6+x1*x2*x3**2*x5*x6
p(87) = x1*x2*x4*x5*x6**2+x1*x2*x3*x4*x6**2+x1*x2*x4*x5**2*x6+x1&
&*x2*x3**2*x4*x6+x1*x2*x3*x4*x5**2+x1*x2*x3**2*x4*x5
p(88) = x2*x3*x4**2*x5*x6+x1*x3*x4**2*x5*x6+x2**2*x3*x4*x5*x6+x1&
&**2*x3*x4*x5*x6+x1*x2**2*x3*x5*x6+x1**2*x2*x3*x5*x6
p(89) = x1*x2**2*x4*x5*x6+x1**2*x2*x4*x5*x6+x1*x2*x3*x4**2*x6+x1&
&**2*x2*x3*x4*x6+x1*x2*x3*x4**2*x5+x1*x2**2*x3*x4*x5
p(90) = x1*x2*x4**2*x5*x6+x1*x2**2*x3*x4*x6+x1**2*x2*x3*x4*x5
p(91) = x1*x3*x5*x6**3+x2*x3*x5**3*x6+x3**3*x4*x5*x6
p(92) = x2*x3*x5**2*x6**2+x1*x3*x5**2*x6**2+x3**2*x4*x5*x6**2+x1&
&*x3**2*x5*x6**2+x3**2*x4*x5**2*x6+x2*x3**2*x5**2*x6
p(93) = x3*x4*x5**2*x6**2+x2*x3**2*x5*x6**2+x1*x3**2*x5**2*x6
p(94) = x3*x4*x5*x6**3+x2*x3*x5*x6**3+x3*x4*x5**3*x6+x1*x3*x5**3&
&*x6+x2*x3**3*x5*x6+x1*x3**3*x5*x6
p(95) = x1*x2*x5**2*x6**2+x1*x3**2*x4*x6**2+x2*x3**2*x4*x5**2
p(96) = x1*x2*x5*x6**3+x1*x3*x4*x6**3+x1*x2*x5**3*x6+x1*x3**3*x4&
&*x6+x2*x3*x4*x5**3+x2*x3**3*x4*x5
p(97) = x1*x4*x5*x6**3+x1*x2*x3*x6**3+x2*x4*x5**3*x6+x2*x3**3*x4&
&*x6+x1*x2*x3*x5**3+x1*x3**3*x4*x5
p(98) = x2*x4*x5**2*x6**2+x1*x4*x5**2*x6**2+x2*x3**2*x4*x6**2+x1&
&*x2*x3**2*x6**2+x1*x3**2*x4*x5**2+x1*x2*x3**2*x5**2
p(99) = x2*x4*x5*x6**3+x2*x3*x4*x6**3+x1*x4*x5**3*x6+x1*x2*x3**3&
&*x6+x1*x3*x4*x5**3+x1*x2*x3**3*x5
p(100) = x1*x2*x4*x6**3+x1*x2*x4*x5**3+x1*x2*x3**3*x4
p(101) = x1**2*x3*x5*x6**2+x2**2*x3*x5**2*x6+x3**2*x4**2*x5*x6
p(102) = x3*x4**2*x5*x6**2+x2**2*x3*x5*x6**2+x3*x4**2*x5**2*x6+x1&
&**2*x3*x5**2*x6+x2**2*x3**2*x5*x6+x1**2*x3**2*x5*x6
p(103) = x1**2*x2*x5*x6**2+x1**2*x3*x4*x6**2+x1*x2**2*x5**2*x6+x1&
&*x3**2*x4**2*x6+x2**2*x3*x4*x5**2+x2*x3**2*x4**2*x5
p(104) = x1*x2**2*x5*x6**2+x1*x3*x4**2*x6**2+x1**2*x2*x5**2*x6+x1&
&**2*x3**2*x4*x6+x2*x3*x4**2*x5**2+x2**2*x3**2*x4*x5
p(105) = x1**2*x2*x4*x6**2+x1*x2**2*x4*x5**2+x1*x2*x3**2*x4**2
p(106) = x1**2*x4*x5*x6**2+x1**2*x2*x3*x6**2+x2**2*x4*x5**2*x6+x2&
&*x3**2*x4**2*x6+x1*x2**2*x3*x5**2+x1*x3**2*x4**2*x5
p(107) = x1*x4**2*x5*x6**2+x1*x2**2*x3*x6**2+x2*x4**2*x5**2*x6+x2&
&**2*x3**2*x4*x6+x1**2*x2*x3*x5**2+x1**2*x3**2*x4*x5
p(108) = x2**2*x4*x5*x6**2+x2*x3*x4**2*x6**2+x1**2*x4*x5**2*x6+x1&
&**2*x2*x3**2*x6+x1*x3*x4**2*x5**2+x1*x2**2*x3**2*x5
p(109) = x2*x4**2*x5*x6**2+x2**2*x3*x4*x6**2+x1*x4**2*x5**2*x6+x1&
&*x2**2*x3**2*x6+x1**2*x3*x4*x5**2+x1**2*x2*x3**2*x5
p(110) = x1*x2*x4**2*x6**2+x1*x2**2*x4*x6**2+x1*x2*x4**2*x5**2+x1&
&**2*x2*x4*x5**2+x1*x2**2*x3**2*x4+x1**2*x2*x3**2*x4
p(111) = x1**2*x2**2*x5*x6+x1**2*x3*x4**2*x6+x2**2*x3*x4**2*x5
p(112) = x1**2*x2*x4**2*x6+x1**2*x2**2*x4*x6+x1*x2**2*x4**2*x5+x1&
&**2*x2**2*x4*x5+x1*x2**2*x3*x4**2+x1**2*x2*x3*x4**2
p(113) = x2**2*x4**2*x5*x6+x1**2*x4**2*x5*x6+x2**2*x3*x4**2*x6+x1&
&**2*x2**2*x3*x6+x1**2*x3*x4**2*x5+x1**2*x2**2*x3*x5
p(114) = x1*x2**2*x4**2*x6+x1**2*x2*x4**2*x5+x1**2*x2**2*x3*x4
p(115) = x3*x4**3*x5*x6+x2**3*x3*x5*x6+x1**3*x3*x5*x6
p(116) = x1*x2**3*x5*x6+x1**3*x2*x5*x6+x1*x3*x4**3*x6+x1**3*x3*x4&
&*x6+x2*x3*x4**3*x5+x2**3*x3*x4*x5
p(117) = x1**3*x2*x4*x6+x1*x2**3*x4*x5+x1*x2*x3*x4**3
p(118) = x2**3*x4*x5*x6+x1**3*x4*x5*x6+x2*x3*x4**3*x6+x1**3*x2*x3&
&*x6+x1*x3*x4**3*x5+x1*x2**3*x3*x5
p(119) = x2*x4**3*x5*x6+x1*x4**3*x5*x6+x2**3*x3*x4*x6+x1*x2**3*x3&
&*x6+x1**3*x3*x4*x5+x1**3*x2*x3*x5
p(120) = x1*x2*x4**3*x6+x1*x2**3*x4*x6+x1*x2*x4**3*x5+x1**3*x2*x4&
&*x5+x1*x2**3*x3*x4+x1**3*x2*x3*x4
p(121) = x3**2*x5**2*x6**2
p(122) = x3*x5**2*x6**3+x3**2*x5*x6**3+x3*x5**3*x6**2+x3**3*x5*x6&
&**2+x3**2*x5**3*x6+x3**3*x5**2*x6
p(123) = x3*x5*x6**4+x3*x5**4*x6+x3**4*x5*x6
p(124) = x1*x5*x6**4+x1*x3*x6**4+x2*x5**4*x6+x3**4*x4*x6+x2*x3*x5&
&**4+x3**4*x4*x5
p(125) = x1*x5**2*x6**3+x1*x3**2*x6**3+x2*x5**3*x6**2+x3**3*x4*x6&
&**2+x2*x3**2*x5**3+x3**3*x4*x5**2
p(126) = x2*x5**2*x6**3+x3**2*x4*x6**3+x1*x5**3*x6**2+x1*x3**3*x6&
&**2+x3**2*x4*x5**3+x2*x3**3*x5**2
p(127) = x4*x5**2*x6**3+x2*x3**2*x6**3+x4*x5**3*x6**2+x2*x3**3*x6&
&**2+x1*x3**2*x5**3+x1*x3**3*x5**2
p(128) = x2*x5*x6**4+x3*x4*x6**4+x1*x5**4*x6+x1*x3**4*x6+x3*x4*x5&
&**4+x2*x3**4*x5
p(129) = x4*x5*x6**4+x2*x3*x6**4+x4*x5**4*x6+x2*x3**4*x6+x1*x3*x5&
&**4+x1*x3**4*x5
p(130) = x1*x4*x6**4+x1*x2*x6**4+x2*x4*x5**4+x1*x2*x5**4+x2*x3**4&
&*x4+x1*x3**4*x4
p(131) = x2*x4*x6**4+x1*x4*x5**4+x1*x2*x3**4
p(132) = x1**2*x5*x6**3+x1**2*x3*x6**3+x2**2*x5**3*x6+x3**3*x4**2&
&*x6+x2**2*x3*x5**3+x3**3*x4**2*x5
p(133) = x2**2*x5**2*x6**2+x1**2*x5**2*x6**2+x3**2*x4**2*x6**2+x1&
&**2*x3**2*x6**2+x3**2*x4**2*x5**2+x2**2*x3**2*x5**2
p(134) = x4**2*x5**2*x6**2+x2**2*x3**2*x6**2+x1**2*x3**2*x5**2
p(135) = x2**2*x5*x6**3+x3*x4**2*x6**3+x1**2*x5**3*x6+x1**2*x3**3&
&*x6+x3*x4**2*x5**3+x2**2*x3**3*x5
p(136) = x4**2*x5*x6**3+x2**2*x3*x6**3+x4**2*x5**3*x6+x2**2*x3**3&
&*x6+x1**2*x3*x5**3+x1**2*x3**3*x5
p(137) = x1**2*x4*x6**3+x1**2*x2*x6**3+x2**2*x4*x5**3+x1*x2**2*x5&
&**3+x2*x3**3*x4**2+x1*x3**3*x4**2
p(138) = x1*x4**2*x6**3+x1*x2**2*x6**3+x2*x4**2*x5**3+x1**2*x2*x5&
&**3+x2**2*x3**3*x4+x1**2*x3**3*x4
p(139) = x2*x4**2*x6**3+x2**2*x4*x6**3+x1*x4**2*x5**3+x1**2*x4*x5&
&**3+x1*x2**2*x3**3+x1**2*x2*x3**3
p(140) = x1**2*x4**2*x6**2+x1**2*x2**2*x6**2+x2**2*x4**2*x5**2+x1&
&**2*x2**2*x5**2+x2**2*x3**2*x4**2+x1**2*x3**2*x4**2
p(141) = x1**2*x2**2*x4**2
p(142) = x2**2*x4**2*x6**2+x1**2*x4**2*x5**2+x1**2*x2**2*x3**2
p(143) = x1**3*x5*x6**2+x1**3*x3*x6**2+x2**3*x5**2*x6+x3**2*x4**3&
&*x6+x2**3*x3*x5**2+x3**2*x4**3*x5
p(144) = x2**3*x5*x6**2+x3*x4**3*x6**2+x1**3*x5**2*x6+x1**3*x3**2&
&*x6+x3*x4**3*x5**2+x2**3*x3**2*x5
p(145) = x4**3*x5*x6**2+x2**3*x3*x6**2+x4**3*x5**2*x6+x2**3*x3**2&
&*x6+x1**3*x3*x5**2+x1**3*x3**2*x5
p(146) = x1**3*x4*x6**2+x1**3*x2*x6**2+x2**3*x4*x5**2+x1*x2**3*x5&
&**2+x2*x3**2*x4**3+x1*x3**2*x4**3
p(147) = x1*x4**3*x6**2+x1*x2**3*x6**2+x2*x4**3*x5**2+x1**3*x2*x5&
&**2+x2**3*x3**2*x4+x1**3*x3**2*x4
p(148) = x2*x4**3*x6**2+x2**3*x4*x6**2+x1*x4**3*x5**2+x1**3*x4*x5&
&**2+x1*x2**3*x3**2+x1**3*x2*x3**2
p(149) = x1**3*x4**2*x6+x1**3*x2**2*x6+x2**3*x4**2*x5+x1**2*x2**3&
&*x5+x2**2*x3*x4**3+x1**2*x3*x4**3
p(150) = x1**2*x4**3*x6+x1**2*x2**3*x6+x2**2*x4**3*x5+x1**3*x2**2&
&*x5+x2**3*x3*x4**2+x1**3*x3*x4**2
p(151) = x1*x2**2*x4**3+x1**2*x2*x4**3+x1*x2**3*x4**2+x1**3*x2*x4&
&**2+x1**2*x2**3*x4+x1**3*x2**2*x4
p(152) = x2**2*x4**3*x6+x2**3*x4**2*x6+x1**2*x4**3*x5+x1**3*x4**2&
&*x5+x1**2*x2**3*x3+x1**3*x2**2*x3
p(153) = x2**4*x5*x6+x1**4*x5*x6+x3*x4**4*x6+x1**4*x3*x6+x3*x4**4&
&*x5+x2**4*x3*x5
p(154) = x4**4*x5*x6+x2**4*x3*x6+x1**4*x3*x5
p(155) = x1**4*x4*x6+x1**4*x2*x6+x2**4*x4*x5+x1*x2**4*x5+x2*x3*x4&
&**4+x1*x3*x4**4
p(156) = x1*x4**4*x6+x1*x2**4*x6+x2*x4**4*x5+x1**4*x2*x5+x2**4*x3&
&*x4+x1**4*x3*x4
p(157) = x1*x2*x4**4+x1*x2**4*x4+x1**4*x2*x4
p(158) = x2*x4**4*x6+x2**4*x4*x6+x1*x4**4*x5+x1**4*x4*x5+x1*x2**4&
&*x3+x1**4*x2*x3
p(159) = x5**3*x6**3+x3**3*x6**3+x3**3*x5**3
p(160) = x5**2*x6**4+x3**2*x6**4+x5**4*x6**2+x3**4*x6**2+x3**2*x5&
&**4+x3**4*x5**2
p(161) = x5*x6**5+x3*x6**5+x5**5*x6+x3**5*x6+x3*x5**5+x3**5*x5
p(162) = x4*x6**5+x2*x6**5+x4*x5**5+x1*x5**5+x2*x3**5+x1*x3**5
p(163) = x4**2*x6**4+x2**2*x6**4+x4**2*x5**4+x1**2*x5**4+x2**2*x3&
&**4+x1**2*x3**4
p(164) = x4**3*x6**3+x2**3*x6**3+x4**3*x5**3+x1**3*x5**3+x2**3*x3&
&**3+x1**3*x3**3
p(165) = x2**3*x4**3+x1**3*x4**3+x1**3*x2**3
p(166) = x4**4*x6**2+x2**4*x6**2+x4**4*x5**2+x1**4*x5**2+x2**4*x3&
&**2+x1**4*x3**2
p(167) = x2**2*x4**4+x1**2*x4**4+x2**4*x4**2+x1**4*x4**2+x1**2*x2&
&**4+x1**4*x2**2
p(168) = x4**5*x6+x2**5*x6+x4**5*x5+x1**5*x5+x2**5*x3+x1**5*x3
p(169) = x2*x4**5+x1*x4**5+x2**5*x4+x1**5*x4+x1*x2**5+x1**5*x2
p(170) = x1*x2*x3*x4*x5*x6**2+x1*x2*x3*x4*x5**2*x6+x1*x2*x3**2*x4&
&*x5*x6
p(171) = x1*x2*x3*x4**2*x5*x6+x1*x2**2*x3*x4*x5*x6+x1**2*x2*x3*x4&
&*x5*x6
p(172) = x1*x2*x3*x5**2*x6**2+x1*x3**2*x4*x5*x6**2+x2*x3**2*x4*x5&
&**2*x6
p(173) = x1*x3*x4*x5*x6**3+x1*x2*x3*x5*x6**3+x2*x3*x4*x5**3*x6+x1&
&*x2*x3*x5**3*x6+x2*x3**3*x4*x5*x6+x1*x3**3*x4*x5*x6
p(174) = x2*x3*x4*x5**2*x6**2+x1*x3*x4*x5**2*x6**2+x2*x3**2*x4*x5&
&*x6**2+x1*x2*x3**2*x5*x6**2+x1*x3**2*x4*x5**2*x6+x1*x2*x3**2*x5**&
&2*x6
p(175) = x1*x2*x4*x5**2*x6**2+x1*x2*x3**2*x4*x6**2+x1*x2*x3**2*x4&
&*x5**2
p(176) = x2*x3*x4*x5*x6**3+x1*x3*x4*x5**3*x6+x1*x2*x3**3*x5*x6
p(177) = x1*x2*x4*x5*x6**3+x1*x2*x3*x4*x6**3+x1*x2*x4*x5**3*x6+x1&
&*x2*x3**3*x4*x6+x1*x2*x3*x4*x5**3+x1*x2*x3**3*x4*x5
p(178) = x1**2*x3*x4*x5*x6**2+x1**2*x2*x3*x5*x6**2+x2**2*x3*x4*x5&
&**2*x6+x1*x2**2*x3*x5**2*x6+x2*x3**2*x4**2*x5*x6+x1*x3**2*x4**2*x&
&5*x6
p(179) = x1*x3*x4**2*x5*x6**2+x1*x2**2*x3*x5*x6**2+x2*x3*x4**2*x5&
&**2*x6+x1**2*x2*x3*x5**2*x6+x2**2*x3**2*x4*x5*x6+x1**2*x3**2*x4*x&
&5*x6
p(180) = x1**2*x2*x4*x5*x6**2+x1**2*x2*x3*x4*x6**2+x1*x2**2*x4*x5&
&**2*x6+x1*x2*x3**2*x4**2*x6+x1*x2**2*x3*x4*x5**2+x1*x2*x3**2*x4**&
&2*x5
p(181) = x2*x3*x4**2*x5*x6**2+x2**2*x3*x4*x5*x6**2+x1*x3*x4**2*x5&
&**2*x6+x1**2*x3*x4*x5**2*x6+x1*x2**2*x3**2*x5*x6+x1**2*x2*x3**2*x&
&5*x6
p(182) = x1*x2**2*x4*x5*x6**2+x1*x2*x3*x4**2*x6**2+x1**2*x2*x4*x5&
&**2*x6+x1**2*x2*x3**2*x4*x6+x1*x2*x3*x4**2*x5**2+x1*x2**2*x3**2*x&
&4*x5
p(183) = x1*x2*x4**2*x5*x6**2+x1*x2**2*x3*x4*x6**2+x1*x2*x4**2*x5&
&**2*x6+x1*x2**2*x3**2*x4*x6+x1**2*x2*x3*x4*x5**2+x1**2*x2*x3**2*x&
&4*x5
p(184) = x1**2*x2**2*x4*x5*x6+x1**2*x2*x3*x4**2*x6+x1*x2**2*x3*x4&
&**2*x5
p(185) = x2**2*x3*x4**2*x5*x6+x1**2*x3*x4**2*x5*x6+x1**2*x2**2*x3&
&*x5*x6
p(186) = x1*x2**2*x4**2*x5*x6+x1**2*x2*x4**2*x5*x6+x1*x2**2*x3*x4&
&**2*x6+x1**2*x2**2*x3*x4*x6+x1**2*x2*x3*x4**2*x5+x1**2*x2**2*x3*x&
&4*x5
p(187) = x2*x3*x4**3*x5*x6+x1*x3*x4**3*x5*x6+x2**3*x3*x4*x5*x6+x1&
&**3*x3*x4*x5*x6+x1*x2**3*x3*x5*x6+x1**3*x2*x3*x5*x6
p(188) = x1*x2**3*x4*x5*x6+x1**3*x2*x4*x5*x6+x1*x2*x3*x4**3*x6+x1&
&**3*x2*x3*x4*x6+x1*x2*x3*x4**3*x5+x1*x2**3*x3*x4*x5
p(189) = x1*x2*x4**3*x5*x6+x1*x2**3*x3*x4*x6+x1**3*x2*x3*x4*x5
p(190) = x1*x3*x5*x6**4+x2*x3*x5**4*x6+x3**4*x4*x5*x6
p(191) = x1*x3*x5**2*x6**3+x1*x3**2*x5*x6**3+x2*x3*x5**3*x6**2+x3&
&**3*x4*x5*x6**2+x2*x3**2*x5**3*x6+x3**3*x4*x5**2*x6
p(192) = x3**2*x4*x5**2*x6**2+x2*x3**2*x5**2*x6**2+x1*x3**2*x5**2&
&*x6**2
p(193) = x2*x3*x5**2*x6**3+x3**2*x4*x5*x6**3+x1*x3*x5**3*x6**2+x1&
&*x3**3*x5*x6**2+x3**2*x4*x5**3*x6+x2*x3**3*x5**2*x6
p(194) = x3*x4*x5**2*x6**3+x2*x3**2*x5*x6**3+x3*x4*x5**3*x6**2+x2&
&*x3**3*x5*x6**2+x1*x3**2*x5**3*x6+x1*x3**3*x5**2*x6
p(195) = x3*x4*x5*x6**4+x2*x3*x5*x6**4+x3*x4*x5**4*x6+x1*x3*x5**4&
&*x6+x2*x3**4*x5*x6+x1*x3**4*x5*x6
p(196) = x1*x2*x5**2*x6**3+x1*x3**2*x4*x6**3+x1*x2*x5**3*x6**2+x1&
&*x3**3*x4*x6**2+x2*x3**2*x4*x5**3+x2*x3**3*x4*x5**2
p(197) = x1*x2*x5*x6**4+x1*x3*x4*x6**4+x1*x2*x5**4*x6+x1*x3**4*x4&
&*x6+x2*x3*x4*x5**4+x2*x3**4*x4*x5
p(198) = x1*x4*x5*x6**4+x1*x2*x3*x6**4+x2*x4*x5**4*x6+x2*x3**4*x4&
&*x6+x1*x2*x3*x5**4+x1*x3**4*x4*x5
p(199) = x1*x4*x5**2*x6**3+x1*x2*x3**2*x6**3+x2*x4*x5**3*x6**2+x2&
&*x3**3*x4*x6**2+x1*x2*x3**2*x5**3+x1*x3**3*x4*x5**2
p(200) = x2*x4*x5**2*x6**3+x2*x3**2*x4*x6**3+x1*x4*x5**3*x6**2+x1&
&*x2*x3**3*x6**2+x1*x3**2*x4*x5**3+x1*x2*x3**3*x5**2
p(201) = x2*x4*x5*x6**4+x2*x3*x4*x6**4+x1*x4*x5**4*x6+x1*x2*x3**4&
&*x6+x1*x3*x4*x5**4+x1*x2*x3**4*x5
p(202) = x1*x2*x4*x6**4+x1*x2*x4*x5**4+x1*x2*x3**4*x4
p(203) = x1**2*x3*x5*x6**3+x2**2*x3*x5**3*x6+x3**3*x4**2*x5*x6
p(204) = x2**2*x3*x5**2*x6**2+x1**2*x3*x5**2*x6**2+x3**2*x4**2*x5&
&*x6**2+x1**2*x3**2*x5*x6**2+x3**2*x4**2*x5**2*x6+x2**2*x3**2*x5**&
&2*x6
p(205) = x3*x4**2*x5**2*x6**2+x2**2*x3**2*x5*x6**2+x1**2*x3**2*x5&
&**2*x6
p(206) = x3*x4**2*x5*x6**3+x2**2*x3*x5*x6**3+x3*x4**2*x5**3*x6+x1&
&**2*x3*x5**3*x6+x2**2*x3**3*x5*x6+x1**2*x3**3*x5*x6
p(207) = x1**2*x2*x5*x6**3+x1**2*x3*x4*x6**3+x1*x2**2*x5**3*x6+x1&
&*x3**3*x4**2*x6+x2**2*x3*x4*x5**3+x2*x3**3*x4**2*x5
p(208) = x1*x2**2*x5**2*x6**2+x1**2*x2*x5**2*x6**2+x1*x3**2*x4**2&
&*x6**2+x1**2*x3**2*x4*x6**2+x2*x3**2*x4**2*x5**2+x2**2*x3**2*x4*x&
&5**2
p(209) = x1*x2**2*x5*x6**3+x1*x3*x4**2*x6**3+x1**2*x2*x5**3*x6+x1&
&**2*x3**3*x4*x6+x2*x3*x4**2*x5**3+x2**2*x3**3*x4*x5
p(210) = x1**2*x2*x4*x6**3+x1*x2**2*x4*x5**3+x1*x2*x3**3*x4**2
p(211) = x1**2*x4*x5*x6**3+x1**2*x2*x3*x6**3+x2**2*x4*x5**3*x6+x2&
&*x3**3*x4**2*x6+x1*x2**2*x3*x5**3+x1*x3**3*x4**2*x5
p(212) = x1*x4**2*x5*x6**3+x1*x2**2*x3*x6**3+x2*x4**2*x5**3*x6+x2&
&**2*x3**3*x4*x6+x1**2*x2*x3*x5**3+x1**2*x3**3*x4*x5
p(213) = x2**2*x4*x5**2*x6**2+x1**2*x4*x5**2*x6**2+x2*x3**2*x4**2&
&*x6**2+x1**2*x2*x3**2*x6**2+x1*x3**2*x4**2*x5**2+x1*x2**2*x3**2*x&
&5**2
p(214) = x2*x4**2*x5**2*x6**2+x1*x4**2*x5**2*x6**2+x2**2*x3**2*x4&
&*x6**2+x1*x2**2*x3**2*x6**2+x1**2*x3**2*x4*x5**2+x1**2*x2*x3**2*x&
&5**2
p(215) = x2**2*x4*x5*x6**3+x2*x3*x4**2*x6**3+x1**2*x4*x5**3*x6+x1&
&**2*x2*x3**3*x6+x1*x3*x4**2*x5**3+x1*x2**2*x3**3*x5
p(216) = x2*x4**2*x5*x6**3+x2**2*x3*x4*x6**3+x1*x4**2*x5**3*x6+x1&
&*x2**2*x3**3*x6+x1**2*x3*x4*x5**3+x1**2*x2*x3**3*x5
p(217) = x1*x2*x4**2*x6**3+x1*x2**2*x4*x6**3+x1*x2*x4**2*x5**3+x1&
&**2*x2*x4*x5**3+x1*x2**2*x3**3*x4+x1**2*x2*x3**3*x4
p(218) = x1**2*x2**2*x5*x6**2+x1**2*x3*x4**2*x6**2+x1**2*x2**2*x5&
&**2*x6+x1**2*x3**2*x4**2*x6+x2**2*x3*x4**2*x5**2+x2**2*x3**2*x4**&
&2*x5
p(219) = x1**2*x2*x4**2*x6**2+x1**2*x2**2*x4*x6**2+x1*x2**2*x4**2&
&*x5**2+x1**2*x2**2*x4*x5**2+x1*x2**2*x3**2*x4**2+x1**2*x2*x3**2*x&
&4**2
p(220) = x1**2*x4**2*x5*x6**2+x1**2*x2**2*x3*x6**2+x2**2*x4**2*x5&
&**2*x6+x2**2*x3**2*x4**2*x6+x1**2*x2**2*x3*x5**2+x1**2*x3**2*x4**&
&2*x5
p(221) = x1**2*x2**2*x4**2*x6+x1**2*x2**2*x4**2*x5+x1**2*x2**2*x3&
&*x4**2
p(222) = x2**2*x4**2*x5*x6**2+x2**2*x3*x4**2*x6**2+x1**2*x4**2*x5&
&**2*x6+x1**2*x2**2*x3**2*x6+x1**2*x3*x4**2*x5**2+x1**2*x2**2*x3**&
&2*x5
p(223) = x1*x2**2*x4**2*x6**2+x1**2*x2*x4**2*x5**2+x1**2*x2**2*x3&
&**2*x4
p(224) = x1**3*x3*x5*x6**2+x2**3*x3*x5**2*x6+x3**2*x4**3*x5*x6
p(225) = x3*x4**3*x5*x6**2+x2**3*x3*x5*x6**2+x3*x4**3*x5**2*x6+x1&
&**3*x3*x5**2*x6+x2**3*x3**2*x5*x6+x1**3*x3**2*x5*x6
p(226) = x1**3*x2*x5*x6**2+x1**3*x3*x4*x6**2+x1*x2**3*x5**2*x6+x1&
&*x3**2*x4**3*x6+x2**3*x3*x4*x5**2+x2*x3**2*x4**3*x5
p(227) = x1*x2**3*x5*x6**2+x1*x3*x4**3*x6**2+x1**3*x2*x5**2*x6+x1&
&**3*x3**2*x4*x6+x2*x3*x4**3*x5**2+x2**3*x3**2*x4*x5
p(228) = x1**3*x2*x4*x6**2+x1*x2**3*x4*x5**2+x1*x2*x3**2*x4**3
p(229) = x1**3*x4*x5*x6**2+x1**3*x2*x3*x6**2+x2**3*x4*x5**2*x6+x2&
&*x3**2*x4**3*x6+x1*x2**3*x3*x5**2+x1*x3**2*x4**3*x5
p(230) = x1*x4**3*x5*x6**2+x1*x2**3*x3*x6**2+x2*x4**3*x5**2*x6+x2&
&**3*x3**2*x4*x6+x1**3*x2*x3*x5**2+x1**3*x3**2*x4*x5
p(231) = x2**3*x4*x5*x6**2+x2*x3*x4**3*x6**2+x1**3*x4*x5**2*x6+x1&
&**3*x2*x3**2*x6+x1*x3*x4**3*x5**2+x1*x2**3*x3**2*x5
p(232) = x2*x4**3*x5*x6**2+x2**3*x3*x4*x6**2+x1*x4**3*x5**2*x6+x1&
&*x2**3*x3**2*x6+x1**3*x3*x4*x5**2+x1**3*x2*x3**2*x5
p(233) = x1*x2*x4**3*x6**2+x1*x2**3*x4*x6**2+x1*x2*x4**3*x5**2+x1&
&**3*x2*x4*x5**2+x1*x2**3*x3**2*x4+x1**3*x2*x3**2*x4
p(234) = x1**2*x2**3*x5*x6+x1**3*x2**2*x5*x6+x1**2*x3*x4**3*x6+x1&
&**3*x3*x4**2*x6+x2**2*x3*x4**3*x5+x2**3*x3*x4**2*x5
p(235) = x1**3*x2*x4**2*x6+x1**3*x2**2*x4*x6+x1*x2**3*x4**2*x5+x1&
&**2*x2**3*x4*x5+x1*x2**2*x3*x4**3+x1**2*x2*x3*x4**3
p(236) = x1**2*x2*x4**3*x6+x1**2*x2**3*x4*x6+x1*x2**2*x4**3*x5+x1&
&**3*x2**2*x4*x5+x1*x2**3*x3*x4**2+x1**3*x2*x3*x4**2
p(237) = x2**3*x4**2*x5*x6+x1**3*x4**2*x5*x6+x2**2*x3*x4**3*x6+x1&
&**3*x2**2*x3*x6+x1**2*x3*x4**3*x5+x1**2*x2**3*x3*x5
p(238) = x2**2*x4**3*x5*x6+x1**2*x4**3*x5*x6+x2**3*x3*x4**2*x6+x1&
&**2*x2**3*x3*x6+x1**3*x3*x4**2*x5+x1**3*x2**2*x3*x5
p(239) = x1*x2**2*x4**3*x6+x1*x2**3*x4**2*x6+x1**2*x2*x4**3*x5+x1&
&**3*x2*x4**2*x5+x1**2*x2**3*x3*x4+x1**3*x2**2*x3*x4
p(240) = x3*x4**4*x5*x6+x2**4*x3*x5*x6+x1**4*x3*x5*x6
p(241) = x1*x2**4*x5*x6+x1**4*x2*x5*x6+x1*x3*x4**4*x6+x1**4*x3*x4&
&*x6+x2*x3*x4**4*x5+x2**4*x3*x4*x5
p(242) = x1**4*x2*x4*x6+x1*x2**4*x4*x5+x1*x2*x3*x4**4
p(243) = x2**4*x4*x5*x6+x1**4*x4*x5*x6+x2*x3*x4**4*x6+x1**4*x2*x3&
&*x6+x1*x3*x4**4*x5+x1*x2**4*x3*x5
p(244) = x2*x4**4*x5*x6+x1*x4**4*x5*x6+x2**4*x3*x4*x6+x1*x2**4*x3&
&*x6+x1**4*x3*x4*x5+x1**4*x2*x3*x5
p(245) = x1*x2*x4**4*x6+x1*x2**4*x4*x6+x1*x2*x4**4*x5+x1**4*x2*x4&
&*x5+x1*x2**4*x3*x4+x1**4*x2*x3*x4
p(246) = x3**2*x5**2*x6**3+x3**2*x5**3*x6**2+x3**3*x5**2*x6**2
p(247) = x3*x5**3*x6**3+x3**3*x5*x6**3+x3**3*x5**3*x6
p(248) = x3*x5**2*x6**4+x3**2*x5*x6**4+x3*x5**4*x6**2+x3**4*x5*x6&
&**2+x3**2*x5**4*x6+x3**4*x5**2*x6
p(249) = x3*x5*x6**5+x3*x5**5*x6+x3**5*x5*x6
p(250) = x1*x5*x6**5+x1*x3*x6**5+x2*x5**5*x6+x3**5*x4*x6+x2*x3*x5&
&**5+x3**5*x4*x5
p(251) = x1*x5**2*x6**4+x1*x3**2*x6**4+x2*x5**4*x6**2+x3**4*x4*x6&
&**2+x2*x3**2*x5**4+x3**4*x4*x5**2
p(252) = x2*x5**3*x6**3+x1*x5**3*x6**3+x3**3*x4*x6**3+x1*x3**3*x6&
&**3+x3**3*x4*x5**3+x2*x3**3*x5**3
p(253) = x4*x5**3*x6**3+x2*x3**3*x6**3+x1*x3**3*x5**3
p(254) = x2*x5**2*x6**4+x3**2*x4*x6**4+x1*x5**4*x6**2+x1*x3**4*x6&
&**2+x3**2*x4*x5**4+x2*x3**4*x5**2
p(255) = x4*x5**2*x6**4+x2*x3**2*x6**4+x4*x5**4*x6**2+x2*x3**4*x6&
&**2+x1*x3**2*x5**4+x1*x3**4*x5**2
p(256) = x2*x5*x6**5+x3*x4*x6**5+x1*x5**5*x6+x1*x3**5*x6+x3*x4*x5&
&**5+x2*x3**5*x5
p(257) = x4*x5*x6**5+x2*x3*x6**5+x4*x5**5*x6+x2*x3**5*x6+x1*x3*x5&
&**5+x1*x3**5*x5
p(258) = x1*x4*x6**5+x1*x2*x6**5+x2*x4*x5**5+x1*x2*x5**5+x2*x3**5&
&*x4+x1*x3**5*x4
p(259) = x2*x4*x6**5+x1*x4*x5**5+x1*x2*x3**5
p(260) = x1**2*x5*x6**4+x1**2*x3*x6**4+x2**2*x5**4*x6+x3**4*x4**2&
&*x6+x2**2*x3*x5**4+x3**4*x4**2*x5
p(261) = x1**2*x5**2*x6**3+x1**2*x3**2*x6**3+x2**2*x5**3*x6**2+x3&
&**3*x4**2*x6**2+x2**2*x3**2*x5**3+x3**3*x4**2*x5**2
p(262) = x2**2*x5**2*x6**3+x3**2*x4**2*x6**3+x1**2*x5**3*x6**2+x1&
&**2*x3**3*x6**2+x3**2*x4**2*x5**3+x2**2*x3**3*x5**2
p(263) = x4**2*x5**2*x6**3+x2**2*x3**2*x6**3+x4**2*x5**3*x6**2+x2&
&**2*x3**3*x6**2+x1**2*x3**2*x5**3+x1**2*x3**3*x5**2
p(264) = x2**2*x5*x6**4+x3*x4**2*x6**4+x1**2*x5**4*x6+x1**2*x3**4&
&*x6+x3*x4**2*x5**4+x2**2*x3**4*x5
p(265) = x4**2*x5*x6**4+x2**2*x3*x6**4+x4**2*x5**4*x6+x2**2*x3**4&
&*x6+x1**2*x3*x5**4+x1**2*x3**4*x5
p(266) = x1**2*x4*x6**4+x1**2*x2*x6**4+x2**2*x4*x5**4+x1*x2**2*x5&
&**4+x2*x3**4*x4**2+x1*x3**4*x4**2
p(267) = x1*x4**2*x6**4+x1*x2**2*x6**4+x2*x4**2*x5**4+x1**2*x2*x5&
&**4+x2**2*x3**4*x4+x1**2*x3**4*x4
p(268) = x2*x4**2*x6**4+x2**2*x4*x6**4+x1*x4**2*x5**4+x1**2*x4*x5&
&**4+x1*x2**2*x3**4+x1**2*x2*x3**4
p(269) = x1**2*x4**2*x6**3+x1**2*x2**2*x6**3+x2**2*x4**2*x5**3+x1&
&**2*x2**2*x5**3+x2**2*x3**3*x4**2+x1**2*x3**3*x4**2
p(270) = x2**2*x4**2*x6**3+x1**2*x4**2*x5**3+x1**2*x2**2*x3**3
p(271) = x1**3*x5*x6**3+x1**3*x3*x6**3+x2**3*x5**3*x6+x3**3*x4**3&
&*x6+x2**3*x3*x5**3+x3**3*x4**3*x5
p(272) = x2**3*x5**2*x6**2+x1**3*x5**2*x6**2+x3**2*x4**3*x6**2+x1&
&**3*x3**2*x6**2+x3**2*x4**3*x5**2+x2**3*x3**2*x5**2
p(273) = x4**3*x5**2*x6**2+x2**3*x3**2*x6**2+x1**3*x3**2*x5**2
p(274) = x2**3*x5*x6**3+x3*x4**3*x6**3+x1**3*x5**3*x6+x1**3*x3**3&
&*x6+x3*x4**3*x5**3+x2**3*x3**3*x5
p(275) = x4**3*x5*x6**3+x2**3*x3*x6**3+x4**3*x5**3*x6+x2**3*x3**3&
&*x6+x1**3*x3*x5**3+x1**3*x3**3*x5
p(276) = x1**3*x4*x6**3+x1**3*x2*x6**3+x2**3*x4*x5**3+x1*x2**3*x5&
&**3+x2*x3**3*x4**3+x1*x3**3*x4**3
p(277) = x1*x4**3*x6**3+x1*x2**3*x6**3+x2*x4**3*x5**3+x1**3*x2*x5&
&**3+x2**3*x3**3*x4+x1**3*x3**3*x4
p(278) = x2*x4**3*x6**3+x2**3*x4*x6**3+x1*x4**3*x5**3+x1**3*x4*x5&
&**3+x1*x2**3*x3**3+x1**3*x2*x3**3
p(279) = x1**3*x4**2*x6**2+x1**3*x2**2*x6**2+x2**3*x4**2*x5**2+x1&
&**2*x2**3*x5**2+x2**2*x3**2*x4**3+x1**2*x3**2*x4**3
p(280) = x1**2*x4**3*x6**2+x1**2*x2**3*x6**2+x2**2*x4**3*x5**2+x1&
&**3*x2**2*x5**2+x2**3*x3**2*x4**2+x1**3*x3**2*x4**2
p(281) = x1**2*x2**2*x4**3+x1**2*x2**3*x4**2+x1**3*x2**2*x4**2
p(282) = x2**2*x4**3*x6**2+x2**3*x4**2*x6**2+x1**2*x4**3*x5**2+x1&
&**3*x4**2*x5**2+x1**2*x2**3*x3**2+x1**3*x2**2*x3**2
p(283) = x1**3*x4**3*x6+x1**3*x2**3*x6+x2**3*x4**3*x5+x1**3*x2**3&
&*x5+x2**3*x3*x4**3+x1**3*x3*x4**3
p(284) = x1*x2**3*x4**3+x1**3*x2*x4**3+x1**3*x2**3*x4
p(285) = x2**3*x4**3*x6+x1**3*x4**3*x5+x1**3*x2**3*x3
p(286) = x1**4*x5*x6**2+x1**4*x3*x6**2+x2**4*x5**2*x6+x3**2*x4**4&
&*x6+x2**4*x3*x5**2+x3**2*x4**4*x5
p(287) = x2**4*x5*x6**2+x3*x4**4*x6**2+x1**4*x5**2*x6+x1**4*x3**2&
&*x6+x3*x4**4*x5**2+x2**4*x3**2*x5
p(288) = x4**4*x5*x6**2+x2**4*x3*x6**2+x4**4*x5**2*x6+x2**4*x3**2&
&*x6+x1**4*x3*x5**2+x1**4*x3**2*x5
p(289) = x1**4*x4*x6**2+x1**4*x2*x6**2+x2**4*x4*x5**2+x1*x2**4*x5&
&**2+x2*x3**2*x4**4+x1*x3**2*x4**4
p(290) = x1*x4**4*x6**2+x1*x2**4*x6**2+x2*x4**4*x5**2+x1**4*x2*x5&
&**2+x2**4*x3**2*x4+x1**4*x3**2*x4
p(291) = x2*x4**4*x6**2+x2**4*x4*x6**2+x1*x4**4*x5**2+x1**4*x4*x5&
&**2+x1*x2**4*x3**2+x1**4*x2*x3**2
p(292) = x1**4*x4**2*x6+x1**4*x2**2*x6+x2**4*x4**2*x5+x1**2*x2**4&
&*x5+x2**2*x3*x4**4+x1**2*x3*x4**4
p(293) = x1**2*x4**4*x6+x1**2*x2**4*x6+x2**2*x4**4*x5+x1**4*x2**2&
&*x5+x2**4*x3*x4**2+x1**4*x3*x4**2
p(294) = x1*x2**2*x4**4+x1**2*x2*x4**4+x1*x2**4*x4**2+x1**4*x2*x4&
&**2+x1**2*x2**4*x4+x1**4*x2**2*x4
p(295) = x2**2*x4**4*x6+x2**4*x4**2*x6+x1**2*x4**4*x5+x1**4*x4**2&
&*x5+x1**2*x2**4*x3+x1**4*x2**2*x3
p(296) = x2**5*x5*x6+x1**5*x5*x6+x3*x4**5*x6+x1**5*x3*x6+x3*x4**5&
&*x5+x2**5*x3*x5
p(297) = x4**5*x5*x6+x2**5*x3*x6+x1**5*x3*x5
p(298) = x1**5*x4*x6+x1**5*x2*x6+x2**5*x4*x5+x1*x2**5*x5+x2*x3*x4&
&**5+x1*x3*x4**5
p(299) = x1*x4**5*x6+x1*x2**5*x6+x2*x4**5*x5+x1**5*x2*x5+x2**5*x3&
&*x4+x1**5*x3*x4
p(300) = x1*x2*x4**5+x1*x2**5*x4+x1**5*x2*x4
p(301) = x2*x4**5*x6+x2**5*x4*x6+x1*x4**5*x5+x1**5*x4*x5+x1*x2**5&
&*x3+x1**5*x2*x3
p(302) = x5**3*x6**4+x3**3*x6**4+x5**4*x6**3+x3**4*x6**3+x3**3*x5&
&**4+x3**4*x5**3
p(303) = x5**2*x6**5+x3**2*x6**5+x5**5*x6**2+x3**5*x6**2+x3**2*x5&
&**5+x3**5*x5**2
p(304) = x5*x6**6+x3*x6**6+x5**6*x6+x3**6*x6+x3*x5**6+x3**6*x5
p(305) = x4*x6**6+x2*x6**6+x4*x5**6+x1*x5**6+x2*x3**6+x1*x3**6
p(306) = x4**2*x6**5+x2**2*x6**5+x4**2*x5**5+x1**2*x5**5+x2**2*x3&
&**5+x1**2*x3**5
p(307) = x4**3*x6**4+x2**3*x6**4+x4**3*x5**4+x1**3*x5**4+x2**3*x3&
&**4+x1**3*x3**4
p(308) = x4**4*x6**3+x2**4*x6**3+x4**4*x5**3+x1**4*x5**3+x2**4*x3&
&**3+x1**4*x3**3
p(309) = x2**3*x4**4+x1**3*x4**4+x2**4*x4**3+x1**4*x4**3+x1**3*x2&
&**4+x1**4*x2**3
p(310) = x4**5*x6**2+x2**5*x6**2+x4**5*x5**2+x1**5*x5**2+x2**5*x3&
&**2+x1**5*x3**2
p(311) = x2**2*x4**5+x1**2*x4**5+x2**5*x4**2+x1**5*x4**2+x1**2*x2&
&**5+x1**5*x2**2
p(312) = x4**6*x6+x2**6*x6+x4**6*x5+x1**6*x5+x2**6*x3+x1**6*x3
p(313) = x2*x4**6+x1*x4**6+x2**6*x4+x1**6*x4+x1*x2**6+x1**6*x2
p(314) = x1*x2*x3*x4*x5**2*x6**2+x1*x2*x3**2*x4*x5*x6**2+x1*x2*x3&
&**2*x4*x5**2*x6
p(315) = x1*x2*x3*x4*x5*x6**3+x1*x2*x3*x4*x5**3*x6+x1*x2*x3**3*x4&
&*x5*x6
p(316) = x1**2*x2*x3*x4*x5*x6**2+x1*x2**2*x3*x4*x5**2*x6+x1*x2*x3&
&**2*x4**2*x5*x6
p(317) = x1*x2*x3*x4**2*x5*x6**2+x1*x2**2*x3*x4*x5*x6**2+x1*x2*x3&
&*x4**2*x5**2*x6+x1**2*x2*x3*x4*x5**2*x6+x1*x2**2*x3**2*x4*x5*x6+x&
&1**2*x2*x3**2*x4*x5*x6
p(318) = x1*x2**2*x3*x4**2*x5*x6+x1**2*x2*x3*x4**2*x5*x6+x1**2*x2&
&**2*x3*x4*x5*x6
p(319) = x1*x2*x3*x4**3*x5*x6+x1*x2**3*x3*x4*x5*x6+x1**3*x2*x3*x4&
&*x5*x6
p(320) = x1*x2*x3*x5**2*x6**3+x1*x3**2*x4*x5*x6**3+x1*x2*x3*x5**3&
&*x6**2+x1*x3**3*x4*x5*x6**2+x2*x3**2*x4*x5**3*x6+x2*x3**3*x4*x5**&
&2*x6
p(321) = x1*x3*x4*x5*x6**4+x1*x2*x3*x5*x6**4+x2*x3*x4*x5**4*x6+x1&
&*x2*x3*x5**4*x6+x2*x3**4*x4*x5*x6+x1*x3**4*x4*x5*x6
p(322) = x2*x3**2*x4*x5**2*x6**2+x1*x3**2*x4*x5**2*x6**2+x1*x2*x3&
&**2*x5**2*x6**2
p(323) = x1*x3*x4*x5**2*x6**3+x1*x2*x3**2*x5*x6**3+x2*x3*x4*x5**3&
&*x6**2+x2*x3**3*x4*x5*x6**2+x1*x2*x3**2*x5**3*x6+x1*x3**3*x4*x5**&
&2*x6
p(324) = x2*x3*x4*x5**2*x6**3+x2*x3**2*x4*x5*x6**3+x1*x3*x4*x5**3&
&*x6**2+x1*x2*x3**3*x5*x6**2+x1*x3**2*x4*x5**3*x6+x1*x2*x3**3*x5**&
&2*x6
p(325) = x1*x2*x4*x5**2*x6**3+x1*x2*x3**2*x4*x6**3+x1*x2*x4*x5**3&
&*x6**2+x1*x2*x3**3*x4*x6**2+x1*x2*x3**2*x4*x5**3+x1*x2*x3**3*x4*x&
&5**2
p(326) = x2*x3*x4*x5*x6**4+x1*x3*x4*x5**4*x6+x1*x2*x3**4*x5*x6
p(327) = x1*x2*x4*x5*x6**4+x1*x2*x3*x4*x6**4+x1*x2*x4*x5**4*x6+x1&
&*x2*x3**4*x4*x6+x1*x2*x3*x4*x5**4+x1*x2*x3**4*x4*x5
p(328) = x1**2*x3*x4*x5*x6**3+x1**2*x2*x3*x5*x6**3+x2**2*x3*x4*x5&
&**3*x6+x1*x2**2*x3*x5**3*x6+x2*x3**3*x4**2*x5*x6+x1*x3**3*x4**2*x&
&5*x6
p(329) = x1*x2**2*x3*x5**2*x6**2+x1**2*x2*x3*x5**2*x6**2+x1*x3**2&
&*x4**2*x5*x6**2+x1**2*x3**2*x4*x5*x6**2+x2*x3**2*x4**2*x5**2*x6+x&
&2**2*x3**2*x4*x5**2*x6
p(330) = x1*x3*x4**2*x5*x6**3+x1*x2**2*x3*x5*x6**3+x2*x3*x4**2*x5&
&**3*x6+x1**2*x2*x3*x5**3*x6+x2**2*x3**3*x4*x5*x6+x1**2*x3**3*x4*x&
&5*x6
p(331) = x1**2*x2*x4*x5*x6**3+x1**2*x2*x3*x4*x6**3+x1*x2**2*x4*x5&
&**3*x6+x1*x2*x3**3*x4**2*x6+x1*x2**2*x3*x4*x5**3+x1*x2*x3**3*x4**&
&2*x5
p(332) = x2**2*x3*x4*x5**2*x6**2+x1**2*x3*x4*x5**2*x6**2+x2*x3**2&
&*x4**2*x5*x6**2+x1**2*x2*x3**2*x5*x6**2+x1*x3**2*x4**2*x5**2*x6+x&
&1*x2**2*x3**2*x5**2*x6
p(333) = x2*x3*x4**2*x5**2*x6**2+x1*x3*x4**2*x5**2*x6**2+x2**2*x3&
&**2*x4*x5*x6**2+x1*x2**2*x3**2*x5*x6**2+x1**2*x3**2*x4*x5**2*x6+x&
&1**2*x2*x3**2*x5**2*x6
p(334) = x1*x2**2*x4*x5**2*x6**2+x1**2*x2*x4*x5**2*x6**2+x1*x2*x3&
&**2*x4**2*x6**2+x1**2*x2*x3**2*x4*x6**2+x1*x2*x3**2*x4**2*x5**2+x&
&1*x2**2*x3**2*x4*x5**2
p(335) = x1*x2*x4**2*x5**2*x6**2+x1*x2**2*x3**2*x4*x6**2+x1**2*x2&
&*x3**2*x4*x5**2
p(336) = x2*x3*x4**2*x5*x6**3+x2**2*x3*x4*x5*x6**3+x1*x3*x4**2*x5&
&**3*x6+x1**2*x3*x4*x5**3*x6+x1*x2**2*x3**3*x5*x6+x1**2*x2*x3**3*x&
&5*x6
p(337) = x1*x2**2*x4*x5*x6**3+x1*x2*x3*x4**2*x6**3+x1**2*x2*x4*x5&
&**3*x6+x1**2*x2*x3**3*x4*x6+x1*x2*x3*x4**2*x5**3+x1*x2**2*x3**3*x&
&4*x5
p(338) = x1*x2*x4**2*x5*x6**3+x1*x2**2*x3*x4*x6**3+x1*x2*x4**2*x5&
&**3*x6+x1*x2**2*x3**3*x4*x6+x1**2*x2*x3*x4*x5**3+x1**2*x2*x3**3*x&
&4*x5
p(339) = x1**2*x2**2*x4*x5*x6**2+x1**2*x2*x3*x4**2*x6**2+x1**2*x2&
&**2*x4*x5**2*x6+x1**2*x2*x3**2*x4**2*x6+x1*x2**2*x3*x4**2*x5**2+x&
&1*x2**2*x3**2*x4**2*x5
p(340) = x1**2*x3*x4**2*x5*x6**2+x1**2*x2**2*x3*x5*x6**2+x2**2*x3&
&*x4**2*x5**2*x6+x1**2*x2**2*x3*x5**2*x6+x2**2*x3**2*x4**2*x5*x6+x&
&1**2*x3**2*x4**2*x5*x6
p(341) = x1**2*x2*x4**2*x5*x6**2+x1**2*x2**2*x3*x4*x6**2+x1*x2**2&
&*x4**2*x5**2*x6+x1*x2**2*x3**2*x4**2*x6+x1**2*x2**2*x3*x4*x5**2+x&
&1**2*x2*x3**2*x4**2*x5
p(342) = x1**2*x2**2*x4**2*x5*x6+x1**2*x2**2*x3*x4**2*x6+x1**2*x2&
&**2*x3*x4**2*x5
p(343) = x2**2*x3*x4**2*x5*x6**2+x1**2*x3*x4**2*x5**2*x6+x1**2*x2&
&**2*x3**2*x5*x6
p(344) = x1*x2**2*x4**2*x5*x6**2+x1*x2**2*x3*x4**2*x6**2+x1**2*x2&
&*x4**2*x5**2*x6+x1**2*x2**2*x3**2*x4*x6+x1**2*x2*x3*x4**2*x5**2+x&
&1**2*x2**2*x3**2*x4*x5
p(345) = x1**3*x3*x4*x5*x6**2+x1**3*x2*x3*x5*x6**2+x2**3*x3*x4*x5&
&**2*x6+x1*x2**3*x3*x5**2*x6+x2*x3**2*x4**3*x5*x6+x1*x3**2*x4**3*x&
&5*x6
p(346) = x1*x3*x4**3*x5*x6**2+x1*x2**3*x3*x5*x6**2+x2*x3*x4**3*x5&
&**2*x6+x1**3*x2*x3*x5**2*x6+x2**3*x3**2*x4*x5*x6+x1**3*x3**2*x4*x&
&5*x6
p(347) = x1**3*x2*x4*x5*x6**2+x1**3*x2*x3*x4*x6**2+x1*x2**3*x4*x5&
&**2*x6+x1*x2*x3**2*x4**3*x6+x1*x2**3*x3*x4*x5**2+x1*x2*x3**2*x4**&
&3*x5
p(348) = x2*x3*x4**3*x5*x6**2+x2**3*x3*x4*x5*x6**2+x1*x3*x4**3*x5&
&**2*x6+x1**3*x3*x4*x5**2*x6+x1*x2**3*x3**2*x5*x6+x1**3*x2*x3**2*x&
&5*x6
p(349) = x1*x2**3*x4*x5*x6**2+x1*x2*x3*x4**3*x6**2+x1**3*x2*x4*x5&
&**2*x6+x1**3*x2*x3**2*x4*x6+x1*x2*x3*x4**3*x5**2+x1*x2**3*x3**2*x&
&4*x5
p(350) = x1*x2*x4**3*x5*x6**2+x1*x2**3*x3*x4*x6**2+x1*x2*x4**3*x5&
&**2*x6+x1*x2**3*x3**2*x4*x6+x1**3*x2*x3*x4*x5**2+x1**3*x2*x3**2*x&
&4*x5
p(351) = x1**2*x2**3*x4*x5*x6+x1**3*x2**2*x4*x5*x6+x1**2*x2*x3*x4&
&**3*x6+x1**3*x2*x3*x4**2*x6+x1*x2**2*x3*x4**3*x5+x1*x2**3*x3*x4**&
&2*x5
p(352) = x2**2*x3*x4**3*x5*x6+x1**2*x3*x4**3*x5*x6+x2**3*x3*x4**2&
&*x5*x6+x1**3*x3*x4**2*x5*x6+x1**2*x2**3*x3*x5*x6+x1**3*x2**2*x3*x&
&5*x6
p(353) = x1*x2**3*x4**2*x5*x6+x1**3*x2*x4**2*x5*x6+x1*x2**2*x3*x4&
&**3*x6+x1**3*x2**2*x3*x4*x6+x1**2*x2*x3*x4**3*x5+x1**2*x2**3*x3*x&
&4*x5
p(354) = x1*x2**2*x4**3*x5*x6+x1**2*x2*x4**3*x5*x6+x1*x2**3*x3*x4&
&**2*x6+x1**2*x2**3*x3*x4*x6+x1**3*x2*x3*x4**2*x5+x1**3*x2**2*x3*x&
&4*x5
p(355) = x2*x3*x4**4*x5*x6+x1*x3*x4**4*x5*x6+x2**4*x3*x4*x5*x6+x1&
&**4*x3*x4*x5*x6+x1*x2**4*x3*x5*x6+x1**4*x2*x3*x5*x6
p(356) = x1*x2**4*x4*x5*x6+x1**4*x2*x4*x5*x6+x1*x2*x3*x4**4*x6+x1&
&**4*x2*x3*x4*x6+x1*x2*x3*x4**4*x5+x1*x2**4*x3*x4*x5
p(357) = x1*x2*x4**4*x5*x6+x1*x2**4*x3*x4*x6+x1**4*x2*x3*x4*x5
p(358) = x1*x3*x5*x6**5+x2*x3*x5**5*x6+x3**5*x4*x5*x6
p(359) = x1*x3*x5**2*x6**4+x1*x3**2*x5*x6**4+x2*x3*x5**4*x6**2+x3&
&**4*x4*x5*x6**2+x2*x3**2*x5**4*x6+x3**4*x4*x5**2*x6
p(360) = x1*x3**2*x5**2*x6**3+x2*x3**2*x5**3*x6**2+x3**3*x4*x5**2&
&*x6**2
p(361) = x2*x3*x5**3*x6**3+x1*x3*x5**3*x6**3+x3**3*x4*x5*x6**3+x1&
&*x3**3*x5*x6**3+x3**3*x4*x5**3*x6+x2*x3**3*x5**3*x6
p(362) = x3**2*x4*x5**2*x6**3+x2*x3**2*x5**2*x6**3+x3**2*x4*x5**3&
&*x6**2+x1*x3**2*x5**3*x6**2+x2*x3**3*x5**2*x6**2+x1*x3**3*x5**2*x&
&6**2
p(363) = x3*x4*x5**3*x6**3+x2*x3**3*x5*x6**3+x1*x3**3*x5**3*x6
p(364) = x2*x3*x5**2*x6**4+x3**2*x4*x5*x6**4+x1*x3*x5**4*x6**2+x1&
&*x3**4*x5*x6**2+x3**2*x4*x5**4*x6+x2*x3**4*x5**2*x6
p(365) = x3*x4*x5**2*x6**4+x2*x3**2*x5*x6**4+x3*x4*x5**4*x6**2+x2&
&*x3**4*x5*x6**2+x1*x3**2*x5**4*x6+x1*x3**4*x5**2*x6
p(366) = x3*x4*x5*x6**5+x2*x3*x5*x6**5+x3*x4*x5**5*x6+x1*x3*x5**5&
&*x6+x2*x3**5*x5*x6+x1*x3**5*x5*x6
p(367) = x1*x2*x5**3*x6**3+x1*x3**3*x4*x6**3+x2*x3**3*x4*x5**3
p(368) = x1*x2*x5**2*x6**4+x1*x3**2*x4*x6**4+x1*x2*x5**4*x6**2+x1&
&*x3**4*x4*x6**2+x2*x3**2*x4*x5**4+x2*x3**4*x4*x5**2
p(369) = x1*x2*x5*x6**5+x1*x3*x4*x6**5+x1*x2*x5**5*x6+x1*x3**5*x4&
&*x6+x2*x3*x4*x5**5+x2*x3**5*x4*x5
p(370) = x1*x4*x5*x6**5+x1*x2*x3*x6**5+x2*x4*x5**5*x6+x2*x3**5*x4&
&*x6+x1*x2*x3*x5**5+x1*x3**5*x4*x5
p(371) = x1*x4*x5**2*x6**4+x1*x2*x3**2*x6**4+x2*x4*x5**4*x6**2+x2&
&*x3**4*x4*x6**2+x1*x2*x3**2*x5**4+x1*x3**4*x4*x5**2
p(372) = x2*x4*x5**3*x6**3+x1*x4*x5**3*x6**3+x2*x3**3*x4*x6**3+x1&
&*x2*x3**3*x6**3+x1*x3**3*x4*x5**3+x1*x2*x3**3*x5**3
p(373) = x2*x4*x5**2*x6**4+x2*x3**2*x4*x6**4+x1*x4*x5**4*x6**2+x1&
&*x2*x3**4*x6**2+x1*x3**2*x4*x5**4+x1*x2*x3**4*x5**2
p(374) = x2*x4*x5*x6**5+x2*x3*x4*x6**5+x1*x4*x5**5*x6+x1*x2*x3**5&
&*x6+x1*x3*x4*x5**5+x1*x2*x3**5*x5
p(375) = x1*x2*x4*x6**5+x1*x2*x4*x5**5+x1*x2*x3**5*x4
p(376) = x1**2*x3*x5*x6**4+x2**2*x3*x5**4*x6+x3**4*x4**2*x5*x6
p(377) = x1**2*x3*x5**2*x6**3+x1**2*x3**2*x5*x6**3+x2**2*x3*x5**3&
&*x6**2+x3**3*x4**2*x5*x6**2+x2**2*x3**2*x5**3*x6+x3**3*x4**2*x5**&
&2*x6
p(378) = x3**2*x4**2*x5**2*x6**2+x2**2*x3**2*x5**2*x6**2+x1**2*x3&
&**2*x5**2*x6**2
p(379) = x2**2*x3*x5**2*x6**3+x3**2*x4**2*x5*x6**3+x1**2*x3*x5**3&
&*x6**2+x1**2*x3**3*x5*x6**2+x3**2*x4**2*x5**3*x6+x2**2*x3**3*x5**&
&2*x6
p(380) = x3*x4**2*x5**2*x6**3+x2**2*x3**2*x5*x6**3+x3*x4**2*x5**3&
&*x6**2+x2**2*x3**3*x5*x6**2+x1**2*x3**2*x5**3*x6+x1**2*x3**3*x5**&
&2*x6
p(381) = x3*x4**2*x5*x6**4+x2**2*x3*x5*x6**4+x3*x4**2*x5**4*x6+x1&
&**2*x3*x5**4*x6+x2**2*x3**4*x5*x6+x1**2*x3**4*x5*x6
p(382) = x1**2*x2*x5*x6**4+x1**2*x3*x4*x6**4+x1*x2**2*x5**4*x6+x1&
&*x3**4*x4**2*x6+x2**2*x3*x4*x5**4+x2*x3**4*x4**2*x5
p(383) = x1**2*x2*x5**2*x6**3+x1**2*x3**2*x4*x6**3+x1*x2**2*x5**3&
&*x6**2+x1*x3**3*x4**2*x6**2+x2**2*x3**2*x4*x5**3+x2*x3**3*x4**2*x&
&5**2
p(384) = x1*x2**2*x5**2*x6**3+x1*x3**2*x4**2*x6**3+x1**2*x2*x5**3&
&*x6**2+x1**2*x3**3*x4*x6**2+x2*x3**2*x4**2*x5**3+x2**2*x3**3*x4*x&
&5**2
p(385) = x1*x2**2*x5*x6**4+x1*x3*x4**2*x6**4+x1**2*x2*x5**4*x6+x1&
&**2*x3**4*x4*x6+x2*x3*x4**2*x5**4+x2**2*x3**4*x4*x5
p(386) = x1**2*x2*x4*x6**4+x1*x2**2*x4*x5**4+x1*x2*x3**4*x4**2
p(387) = x1**2*x4*x5*x6**4+x1**2*x2*x3*x6**4+x2**2*x4*x5**4*x6+x2&
&*x3**4*x4**2*x6+x1*x2**2*x3*x5**4+x1*x3**4*x4**2*x5
p(388) = x1*x4**2*x5*x6**4+x1*x2**2*x3*x6**4+x2*x4**2*x5**4*x6+x2&
&**2*x3**4*x4*x6+x1**2*x2*x3*x5**4+x1**2*x3**4*x4*x5
p(389) = x1**2*x4*x5**2*x6**3+x1**2*x2*x3**2*x6**3+x2**2*x4*x5**3&
&*x6**2+x2*x3**3*x4**2*x6**2+x1*x2**2*x3**2*x5**3+x1*x3**3*x4**2*x&
&5**2
p(390) = x1*x4**2*x5**2*x6**3+x1*x2**2*x3**2*x6**3+x2*x4**2*x5**3&
&*x6**2+x2**2*x3**3*x4*x6**2+x1**2*x2*x3**2*x5**3+x1**2*x3**3*x4*x&
&5**2
p(391) = x2**2*x4*x5**2*x6**3+x2*x3**2*x4**2*x6**3+x1**2*x4*x5**3&
&*x6**2+x1**2*x2*x3**3*x6**2+x1*x3**2*x4**2*x5**3+x1*x2**2*x3**3*x&
&5**2
p(392) = x2*x4**2*x5**2*x6**3+x2**2*x3**2*x4*x6**3+x1*x4**2*x5**3&
&*x6**2+x1*x2**2*x3**3*x6**2+x1**2*x3**2*x4*x5**3+x1**2*x2*x3**3*x&
&5**2
p(393) = x2**2*x4*x5*x6**4+x2*x3*x4**2*x6**4+x1**2*x4*x5**4*x6+x1&
&**2*x2*x3**4*x6+x1*x3*x4**2*x5**4+x1*x2**2*x3**4*x5
p(394) = x2*x4**2*x5*x6**4+x2**2*x3*x4*x6**4+x1*x4**2*x5**4*x6+x1&
&*x2**2*x3**4*x6+x1**2*x3*x4*x5**4+x1**2*x2*x3**4*x5
p(395) = x1*x2*x4**2*x6**4+x1*x2**2*x4*x6**4+x1*x2*x4**2*x5**4+x1&
&**2*x2*x4*x5**4+x1*x2**2*x3**4*x4+x1**2*x2*x3**4*x4
p(396) = x1**2*x2**2*x5**2*x6**2+x1**2*x3**2*x4**2*x6**2+x2**2*x3&
&**2*x4**2*x5**2
p(397) = x1**2*x2**2*x5*x6**3+x1**2*x3*x4**2*x6**3+x1**2*x2**2*x5&
&**3*x6+x1**2*x3**3*x4**2*x6+x2**2*x3*x4**2*x5**3+x2**2*x3**3*x4**&
&2*x5
p(398) = x1**2*x2*x4**2*x6**3+x1**2*x2**2*x4*x6**3+x1*x2**2*x4**2&
&*x5**3+x1**2*x2**2*x4*x5**3+x1*x2**2*x3**3*x4**2+x1**2*x2*x3**3*x&
&4**2
p(399) = x1**2*x4**2*x5*x6**3+x1**2*x2**2*x3*x6**3+x2**2*x4**2*x5&
&**3*x6+x2**2*x3**3*x4**2*x6+x1**2*x2**2*x3*x5**3+x1**2*x3**3*x4**&
&2*x5
p(400) = x2**2*x4**2*x5**2*x6**2+x1**2*x4**2*x5**2*x6**2+x2**2*x3&
&**2*x4**2*x6**2+x1**2*x2**2*x3**2*x6**2+x1**2*x3**2*x4**2*x5**2+x&
&1**2*x2**2*x3**2*x5**2
p(401) = x1**2*x2**2*x4**2*x6**2+x1**2*x2**2*x4**2*x5**2+x1**2*x2&
&**2*x3**2*x4**2
p(402) = x2**2*x4**2*x5*x6**3+x2**2*x3*x4**2*x6**3+x1**2*x4**2*x5&
&**3*x6+x1**2*x2**2*x3**3*x6+x1**2*x3*x4**2*x5**3+x1**2*x2**2*x3**&
&3*x5
p(403) = x1*x2**2*x4**2*x6**3+x1**2*x2*x4**2*x5**3+x1**2*x2**2*x3&
&**3*x4
p(404) = x1**3*x3*x5*x6**3+x2**3*x3*x5**3*x6+x3**3*x4**3*x5*x6
p(405) = x2**3*x3*x5**2*x6**2+x1**3*x3*x5**2*x6**2+x3**2*x4**3*x5&
&*x6**2+x1**3*x3**2*x5*x6**2+x3**2*x4**3*x5**2*x6+x2**3*x3**2*x5**&
&2*x6
p(406) = x3*x4**3*x5**2*x6**2+x2**3*x3**2*x5*x6**2+x1**3*x3**2*x5&
&**2*x6
p(407) = x3*x4**3*x5*x6**3+x2**3*x3*x5*x6**3+x3*x4**3*x5**3*x6+x1&
&**3*x3*x5**3*x6+x2**3*x3**3*x5*x6+x1**3*x3**3*x5*x6
p(408) = x1**3*x2*x5*x6**3+x1**3*x3*x4*x6**3+x1*x2**3*x5**3*x6+x1&
&*x3**3*x4**3*x6+x2**3*x3*x4*x5**3+x2*x3**3*x4**3*x5
p(409) = x1*x2**3*x5**2*x6**2+x1**3*x2*x5**2*x6**2+x1*x3**2*x4**3&
&*x6**2+x1**3*x3**2*x4*x6**2+x2*x3**2*x4**3*x5**2+x2**3*x3**2*x4*x&
&5**2
p(410) = x1*x2**3*x5*x6**3+x1*x3*x4**3*x6**3+x1**3*x2*x5**3*x6+x1&
&**3*x3**3*x4*x6+x2*x3*x4**3*x5**3+x2**3*x3**3*x4*x5
p(411) = x1**3*x2*x4*x6**3+x1*x2**3*x4*x5**3+x1*x2*x3**3*x4**3
p(412) = x1**3*x4*x5*x6**3+x1**3*x2*x3*x6**3+x2**3*x4*x5**3*x6+x2&
&*x3**3*x4**3*x6+x1*x2**3*x3*x5**3+x1*x3**3*x4**3*x5
p(413) = x1*x4**3*x5*x6**3+x1*x2**3*x3*x6**3+x2*x4**3*x5**3*x6+x2&
&**3*x3**3*x4*x6+x1**3*x2*x3*x5**3+x1**3*x3**3*x4*x5
p(414) = x2**3*x4*x5**2*x6**2+x1**3*x4*x5**2*x6**2+x2*x3**2*x4**3&
&*x6**2+x1**3*x2*x3**2*x6**2+x1*x3**2*x4**3*x5**2+x1*x2**3*x3**2*x&
&5**2
p(415) = x2*x4**3*x5**2*x6**2+x1*x4**3*x5**2*x6**2+x2**3*x3**2*x4&
&*x6**2+x1*x2**3*x3**2*x6**2+x1**3*x3**2*x4*x5**2+x1**3*x2*x3**2*x&
&5**2
p(416) = x2**3*x4*x5*x6**3+x2*x3*x4**3*x6**3+x1**3*x4*x5**3*x6+x1&
&**3*x2*x3**3*x6+x1*x3*x4**3*x5**3+x1*x2**3*x3**3*x5
p(417) = x2*x4**3*x5*x6**3+x2**3*x3*x4*x6**3+x1*x4**3*x5**3*x6+x1&
&*x2**3*x3**3*x6+x1**3*x3*x4*x5**3+x1**3*x2*x3**3*x5
p(418) = x1*x2*x4**3*x6**3+x1*x2**3*x4*x6**3+x1*x2*x4**3*x5**3+x1&
&**3*x2*x4*x5**3+x1*x2**3*x3**3*x4+x1**3*x2*x3**3*x4
p(419) = x1**3*x2**2*x5*x6**2+x1**3*x3*x4**2*x6**2+x1**2*x2**3*x5&
&**2*x6+x1**2*x3**2*x4**3*x6+x2**3*x3*x4**2*x5**2+x2**2*x3**2*x4**&
&3*x5
p(420) = x1**2*x2**3*x5*x6**2+x1**2*x3*x4**3*x6**2+x1**3*x2**2*x5&
&**2*x6+x1**3*x3**2*x4**2*x6+x2**2*x3*x4**3*x5**2+x2**3*x3**2*x4**&
&2*x5
p(421) = x1**3*x2*x4**2*x6**2+x1**3*x2**2*x4*x6**2+x1*x2**3*x4**2&
&*x5**2+x1**2*x2**3*x4*x5**2+x1*x2**2*x3**2*x4**3+x1**2*x2*x3**2*x&
&4**3
p(422) = x1**2*x2*x4**3*x6**2+x1**2*x2**3*x4*x6**2+x1*x2**2*x4**3&
&*x5**2+x1**3*x2**2*x4*x5**2+x1*x2**3*x3**2*x4**2+x1**3*x2*x3**2*x&
&4**2
p(423) = x1**3*x2**2*x4**2*x6+x1**2*x2**3*x4**2*x5+x1**2*x2**2*x3&
&*x4**3
p(424) = x1**3*x4**2*x5*x6**2+x1**3*x2**2*x3*x6**2+x2**3*x4**2*x5&
&**2*x6+x2**2*x3**2*x4**3*x6+x1**2*x2**3*x3*x5**2+x1**2*x3**2*x4**&
&3*x5
p(425) = x1**2*x4**3*x5*x6**2+x1**2*x2**3*x3*x6**2+x2**2*x4**3*x5&
&**2*x6+x2**3*x3**2*x4**2*x6+x1**3*x2**2*x3*x5**2+x1**3*x3**2*x4**&
&2*x5
p(426) = x1**2*x2**2*x4**3*x6+x1**2*x2**3*x4**2*x6+x1**2*x2**2*x4&
&**3*x5+x1**3*x2**2*x4**2*x5+x1**2*x2**3*x3*x4**2+x1**3*x2**2*x3*x&
&4**2
p(427) = x2**3*x4**2*x5*x6**2+x2**2*x3*x4**3*x6**2+x1**3*x4**2*x5&
&**2*x6+x1**3*x2**2*x3**2*x6+x1**2*x3*x4**3*x5**2+x1**2*x2**3*x3**&
&2*x5
p(428) = x2**2*x4**3*x5*x6**2+x2**3*x3*x4**2*x6**2+x1**2*x4**3*x5&
&**2*x6+x1**2*x2**3*x3**2*x6+x1**3*x3*x4**2*x5**2+x1**3*x2**2*x3**&
&2*x5
p(429) = x1*x2**2*x4**3*x6**2+x1*x2**3*x4**2*x6**2+x1**2*x2*x4**3&
&*x5**2+x1**3*x2*x4**2*x5**2+x1**2*x2**3*x3**2*x4+x1**3*x2**2*x3**&
&2*x4
p(430) = x1**3*x2**3*x5*x6+x1**3*x3*x4**3*x6+x2**3*x3*x4**3*x5
p(431) = x1**3*x2*x4**3*x6+x1**3*x2**3*x4*x6+x1*x2**3*x4**3*x5+x1&
&**3*x2**3*x4*x5+x1*x2**3*x3*x4**3+x1**3*x2*x3*x4**3
p(432) = x2**3*x4**3*x5*x6+x1**3*x4**3*x5*x6+x2**3*x3*x4**3*x6+x1&
&**3*x2**3*x3*x6+x1**3*x3*x4**3*x5+x1**3*x2**3*x3*x5
p(433) = x1*x2**3*x4**3*x6+x1**3*x2*x4**3*x5+x1**3*x2**3*x3*x4
p(434) = x1**4*x3*x5*x6**2+x2**4*x3*x5**2*x6+x3**2*x4**4*x5*x6
p(435) = x3*x4**4*x5*x6**2+x2**4*x3*x5*x6**2+x3*x4**4*x5**2*x6+x1&
&**4*x3*x5**2*x6+x2**4*x3**2*x5*x6+x1**4*x3**2*x5*x6
p(436) = x1**4*x2*x5*x6**2+x1**4*x3*x4*x6**2+x1*x2**4*x5**2*x6+x1&
&*x3**2*x4**4*x6+x2**4*x3*x4*x5**2+x2*x3**2*x4**4*x5
p(437) = x1*x2**4*x5*x6**2+x1*x3*x4**4*x6**2+x1**4*x2*x5**2*x6+x1&
&**4*x3**2*x4*x6+x2*x3*x4**4*x5**2+x2**4*x3**2*x4*x5
p(438) = x1**4*x2*x4*x6**2+x1*x2**4*x4*x5**2+x1*x2*x3**2*x4**4
p(439) = x1**4*x4*x5*x6**2+x1**4*x2*x3*x6**2+x2**4*x4*x5**2*x6+x2&
&*x3**2*x4**4*x6+x1*x2**4*x3*x5**2+x1*x3**2*x4**4*x5
p(440) = x1*x4**4*x5*x6**2+x1*x2**4*x3*x6**2+x2*x4**4*x5**2*x6+x2&
&**4*x3**2*x4*x6+x1**4*x2*x3*x5**2+x1**4*x3**2*x4*x5
p(441) = x2**4*x4*x5*x6**2+x2*x3*x4**4*x6**2+x1**4*x4*x5**2*x6+x1&
&**4*x2*x3**2*x6+x1*x3*x4**4*x5**2+x1*x2**4*x3**2*x5
p(442) = x2*x4**4*x5*x6**2+x2**4*x3*x4*x6**2+x1*x4**4*x5**2*x6+x1&
&*x2**4*x3**2*x6+x1**4*x3*x4*x5**2+x1**4*x2*x3**2*x5
p(443) = x1*x2*x4**4*x6**2+x1*x2**4*x4*x6**2+x1*x2*x4**4*x5**2+x1&
&**4*x2*x4*x5**2+x1*x2**4*x3**2*x4+x1**4*x2*x3**2*x4
p(444) = x1**2*x2**4*x5*x6+x1**4*x2**2*x5*x6+x1**2*x3*x4**4*x6+x1&
&**4*x3*x4**2*x6+x2**2*x3*x4**4*x5+x2**4*x3*x4**2*x5
p(445) = x1**4*x2*x4**2*x6+x1**4*x2**2*x4*x6+x1*x2**4*x4**2*x5+x1&
&**2*x2**4*x4*x5+x1*x2**2*x3*x4**4+x1**2*x2*x3*x4**4
p(446) = x1**2*x2*x4**4*x6+x1**2*x2**4*x4*x6+x1*x2**2*x4**4*x5+x1&
&**4*x2**2*x4*x5+x1*x2**4*x3*x4**2+x1**4*x2*x3*x4**2
p(447) = x2**4*x4**2*x5*x6+x1**4*x4**2*x5*x6+x2**2*x3*x4**4*x6+x1&
&**4*x2**2*x3*x6+x1**2*x3*x4**4*x5+x1**2*x2**4*x3*x5
p(448) = x2**2*x4**4*x5*x6+x1**2*x4**4*x5*x6+x2**4*x3*x4**2*x6+x1&
&**2*x2**4*x3*x6+x1**4*x3*x4**2*x5+x1**4*x2**2*x3*x5
p(449) = x1*x2**2*x4**4*x6+x1*x2**4*x4**2*x6+x1**2*x2*x4**4*x5+x1&
&**4*x2*x4**2*x5+x1**2*x2**4*x3*x4+x1**4*x2**2*x3*x4
p(450) = x3*x4**5*x5*x6+x2**5*x3*x5*x6+x1**5*x3*x5*x6
p(451) = x1*x2**5*x5*x6+x1**5*x2*x5*x6+x1*x3*x4**5*x6+x1**5*x3*x4&
&*x6+x2*x3*x4**5*x5+x2**5*x3*x4*x5
p(452) = x1**5*x2*x4*x6+x1*x2**5*x4*x5+x1*x2*x3*x4**5
p(453) = x2**5*x4*x5*x6+x1**5*x4*x5*x6+x2*x3*x4**5*x6+x1**5*x2*x3&
&*x6+x1*x3*x4**5*x5+x1*x2**5*x3*x5
p(454) = x2*x4**5*x5*x6+x1*x4**5*x5*x6+x2**5*x3*x4*x6+x1*x2**5*x3&
&*x6+x1**5*x3*x4*x5+x1**5*x2*x3*x5
p(455) = x1*x2*x4**5*x6+x1*x2**5*x4*x6+x1*x2*x4**5*x5+x1**5*x2*x4&
&*x5+x1*x2**5*x3*x4+x1**5*x2*x3*x4
p(456) = x3**2*x5**3*x6**3+x3**3*x5**2*x6**3+x3**3*x5**3*x6**2
p(457) = x3**2*x5**2*x6**4+x3**2*x5**4*x6**2+x3**4*x5**2*x6**2
p(458) = x3*x5**3*x6**4+x3**3*x5*x6**4+x3*x5**4*x6**3+x3**4*x5*x6&
&**3+x3**3*x5**4*x6+x3**4*x5**3*x6
p(459) = x3*x5**2*x6**5+x3**2*x5*x6**5+x3*x5**5*x6**2+x3**5*x5*x6&
&**2+x3**2*x5**5*x6+x3**5*x5**2*x6
p(460) = x3*x5*x6**6+x3*x5**6*x6+x3**6*x5*x6
p(461) = x1*x5*x6**6+x1*x3*x6**6+x2*x5**6*x6+x3**6*x4*x6+x2*x3*x5&
&**6+x3**6*x4*x5
p(462) = x1*x5**2*x6**5+x1*x3**2*x6**5+x2*x5**5*x6**2+x3**5*x4*x6&
&**2+x2*x3**2*x5**5+x3**5*x4*x5**2
p(463) = x1*x5**3*x6**4+x1*x3**3*x6**4+x2*x5**4*x6**3+x3**4*x4*x6&
&**3+x2*x3**3*x5**4+x3**4*x4*x5**3
p(464) = x2*x5**3*x6**4+x3**3*x4*x6**4+x1*x5**4*x6**3+x1*x3**4*x6&
&**3+x3**3*x4*x5**4+x2*x3**4*x5**3
p(465) = x4*x5**3*x6**4+x2*x3**3*x6**4+x4*x5**4*x6**3+x2*x3**4*x6&
&**3+x1*x3**3*x5**4+x1*x3**4*x5**3
p(466) = x2*x5**2*x6**5+x3**2*x4*x6**5+x1*x5**5*x6**2+x1*x3**5*x6&
&**2+x3**2*x4*x5**5+x2*x3**5*x5**2
p(467) = x4*x5**2*x6**5+x2*x3**2*x6**5+x4*x5**5*x6**2+x2*x3**5*x6&
&**2+x1*x3**2*x5**5+x1*x3**5*x5**2
p(468) = x2*x5*x6**6+x3*x4*x6**6+x1*x5**6*x6+x1*x3**6*x6+x3*x4*x5&
&**6+x2*x3**6*x5
p(469) = x4*x5*x6**6+x2*x3*x6**6+x4*x5**6*x6+x2*x3**6*x6+x1*x3*x5&
&**6+x1*x3**6*x5
p(470) = x1*x4*x6**6+x1*x2*x6**6+x2*x4*x5**6+x1*x2*x5**6+x2*x3**6&
&*x4+x1*x3**6*x4
p(471) = x2*x4*x6**6+x1*x4*x5**6+x1*x2*x3**6
p(472) = x1**2*x5*x6**5+x1**2*x3*x6**5+x2**2*x5**5*x6+x3**5*x4**2&
&*x6+x2**2*x3*x5**5+x3**5*x4**2*x5
p(473) = x1**2*x5**2*x6**4+x1**2*x3**2*x6**4+x2**2*x5**4*x6**2+x3&
&**4*x4**2*x6**2+x2**2*x3**2*x5**4+x3**4*x4**2*x5**2
p(474) = x2**2*x5**3*x6**3+x1**2*x5**3*x6**3+x3**3*x4**2*x6**3+x1&
&**2*x3**3*x6**3+x3**3*x4**2*x5**3+x2**2*x3**3*x5**3
p(475) = x4**2*x5**3*x6**3+x2**2*x3**3*x6**3+x1**2*x3**3*x5**3
p(476) = x2**2*x5**2*x6**4+x3**2*x4**2*x6**4+x1**2*x5**4*x6**2+x1&
&**2*x3**4*x6**2+x3**2*x4**2*x5**4+x2**2*x3**4*x5**2
p(477) = x4**2*x5**2*x6**4+x2**2*x3**2*x6**4+x4**2*x5**4*x6**2+x2&
&**2*x3**4*x6**2+x1**2*x3**2*x5**4+x1**2*x3**4*x5**2
p(478) = x2**2*x5*x6**5+x3*x4**2*x6**5+x1**2*x5**5*x6+x1**2*x3**5&
&*x6+x3*x4**2*x5**5+x2**2*x3**5*x5
p(479) = x4**2*x5*x6**5+x2**2*x3*x6**5+x4**2*x5**5*x6+x2**2*x3**5&
&*x6+x1**2*x3*x5**5+x1**2*x3**5*x5
p(480) = x1**2*x4*x6**5+x1**2*x2*x6**5+x2**2*x4*x5**5+x1*x2**2*x5&
&**5+x2*x3**5*x4**2+x1*x3**5*x4**2
p(481) = x1*x4**2*x6**5+x1*x2**2*x6**5+x2*x4**2*x5**5+x1**2*x2*x5&
&**5+x2**2*x3**5*x4+x1**2*x3**5*x4
p(482) = x2*x4**2*x6**5+x2**2*x4*x6**5+x1*x4**2*x5**5+x1**2*x4*x5&
&**5+x1*x2**2*x3**5+x1**2*x2*x3**5
p(483) = x1**2*x4**2*x6**4+x1**2*x2**2*x6**4+x2**2*x4**2*x5**4+x1&
&**2*x2**2*x5**4+x2**2*x3**4*x4**2+x1**2*x3**4*x4**2
p(484) = x2**2*x4**2*x6**4+x1**2*x4**2*x5**4+x1**2*x2**2*x3**4
p(485) = x1**3*x5*x6**4+x1**3*x3*x6**4+x2**3*x5**4*x6+x3**4*x4**3&
&*x6+x2**3*x3*x5**4+x3**4*x4**3*x5
p(486) = x1**3*x5**2*x6**3+x1**3*x3**2*x6**3+x2**3*x5**3*x6**2+x3&
&**3*x4**3*x6**2+x2**3*x3**2*x5**3+x3**3*x4**3*x5**2
p(487) = x2**3*x5**2*x6**3+x3**2*x4**3*x6**3+x1**3*x5**3*x6**2+x1&
&**3*x3**3*x6**2+x3**2*x4**3*x5**3+x2**3*x3**3*x5**2
p(488) = x4**3*x5**2*x6**3+x2**3*x3**2*x6**3+x4**3*x5**3*x6**2+x2&
&**3*x3**3*x6**2+x1**3*x3**2*x5**3+x1**3*x3**3*x5**2
p(489) = x2**3*x5*x6**4+x3*x4**3*x6**4+x1**3*x5**4*x6+x1**3*x3**4&
&*x6+x3*x4**3*x5**4+x2**3*x3**4*x5
p(490) = x4**3*x5*x6**4+x2**3*x3*x6**4+x4**3*x5**4*x6+x2**3*x3**4&
&*x6+x1**3*x3*x5**4+x1**3*x3**4*x5
p(491) = x1**3*x4*x6**4+x1**3*x2*x6**4+x2**3*x4*x5**4+x1*x2**3*x5&
&**4+x2*x3**4*x4**3+x1*x3**4*x4**3
p(492) = x1*x4**3*x6**4+x1*x2**3*x6**4+x2*x4**3*x5**4+x1**3*x2*x5&
&**4+x2**3*x3**4*x4+x1**3*x3**4*x4
p(493) = x2*x4**3*x6**4+x2**3*x4*x6**4+x1*x4**3*x5**4+x1**3*x4*x5&
&**4+x1*x2**3*x3**4+x1**3*x2*x3**4
p(494) = x1**3*x4**2*x6**3+x1**3*x2**2*x6**3+x2**3*x4**2*x5**3+x1&
&**2*x2**3*x5**3+x2**2*x3**3*x4**3+x1**2*x3**3*x4**3
p(495) = x1**2*x4**3*x6**3+x1**2*x2**3*x6**3+x2**2*x4**3*x5**3+x1&
&**3*x2**2*x5**3+x2**3*x3**3*x4**2+x1**3*x3**3*x4**2
p(496) = x2**2*x4**3*x6**3+x2**3*x4**2*x6**3+x1**2*x4**3*x5**3+x1&
&**3*x4**2*x5**3+x1**2*x2**3*x3**3+x1**3*x2**2*x3**3
p(497) = x1**3*x4**3*x6**2+x1**3*x2**3*x6**2+x2**3*x4**3*x5**2+x1&
&**3*x2**3*x5**2+x2**3*x3**2*x4**3+x1**3*x3**2*x4**3
p(498) = x1**2*x2**3*x4**3+x1**3*x2**2*x4**3+x1**3*x2**3*x4**2
p(499) = x2**3*x4**3*x6**2+x1**3*x4**3*x5**2+x1**3*x2**3*x3**2
p(500) = x1**4*x5*x6**3+x1**4*x3*x6**3+x2**4*x5**3*x6+x3**3*x4**4&
&*x6+x2**4*x3*x5**3+x3**3*x4**4*x5
p(501) = x2**4*x5**2*x6**2+x1**4*x5**2*x6**2+x3**2*x4**4*x6**2+x1&
&**4*x3**2*x6**2+x3**2*x4**4*x5**2+x2**4*x3**2*x5**2
p(502) = x4**4*x5**2*x6**2+x2**4*x3**2*x6**2+x1**4*x3**2*x5**2
p(503) = x2**4*x5*x6**3+x3*x4**4*x6**3+x1**4*x5**3*x6+x1**4*x3**3&
&*x6+x3*x4**4*x5**3+x2**4*x3**3*x5
p(504) = x4**4*x5*x6**3+x2**4*x3*x6**3+x4**4*x5**3*x6+x2**4*x3**3&
&*x6+x1**4*x3*x5**3+x1**4*x3**3*x5
p(505) = x1**4*x4*x6**3+x1**4*x2*x6**3+x2**4*x4*x5**3+x1*x2**4*x5&
&**3+x2*x3**3*x4**4+x1*x3**3*x4**4
p(506) = x1*x4**4*x6**3+x1*x2**4*x6**3+x2*x4**4*x5**3+x1**4*x2*x5&
&**3+x2**4*x3**3*x4+x1**4*x3**3*x4
p(507) = x2*x4**4*x6**3+x2**4*x4*x6**3+x1*x4**4*x5**3+x1**4*x4*x5&
&**3+x1*x2**4*x3**3+x1**4*x2*x3**3
p(508) = x1**4*x4**2*x6**2+x1**4*x2**2*x6**2+x2**4*x4**2*x5**2+x1&
&**2*x2**4*x5**2+x2**2*x3**2*x4**4+x1**2*x3**2*x4**4
p(509) = x1**2*x4**4*x6**2+x1**2*x2**4*x6**2+x2**2*x4**4*x5**2+x1&
&**4*x2**2*x5**2+x2**4*x3**2*x4**2+x1**4*x3**2*x4**2
p(510) = x1**2*x2**2*x4**4+x1**2*x2**4*x4**2+x1**4*x2**2*x4**2
p(511) = x2**2*x4**4*x6**2+x2**4*x4**2*x6**2+x1**2*x4**4*x5**2+x1&
&**4*x4**2*x5**2+x1**2*x2**4*x3**2+x1**4*x2**2*x3**2
p(512) = x1**4*x4**3*x6+x1**4*x2**3*x6+x2**4*x4**3*x5+x1**3*x2**4&
&*x5+x2**3*x3*x4**4+x1**3*x3*x4**4
p(513) = x1**3*x4**4*x6+x1**3*x2**4*x6+x2**3*x4**4*x5+x1**4*x2**3&
&*x5+x2**4*x3*x4**3+x1**4*x3*x4**3
p(514) = x1*x2**3*x4**4+x1**3*x2*x4**4+x1*x2**4*x4**3+x1**4*x2*x4&
&**3+x1**3*x2**4*x4+x1**4*x2**3*x4
p(515) = x2**3*x4**4*x6+x2**4*x4**3*x6+x1**3*x4**4*x5+x1**4*x4**3&
&*x5+x1**3*x2**4*x3+x1**4*x2**3*x3
p(516) = x1**5*x5*x6**2+x1**5*x3*x6**2+x2**5*x5**2*x6+x3**2*x4**5&
&*x6+x2**5*x3*x5**2+x3**2*x4**5*x5
p(517) = x2**5*x5*x6**2+x3*x4**5*x6**2+x1**5*x5**2*x6+x1**5*x3**2&
&*x6+x3*x4**5*x5**2+x2**5*x3**2*x5
p(518) = x4**5*x5*x6**2+x2**5*x3*x6**2+x4**5*x5**2*x6+x2**5*x3**2&
&*x6+x1**5*x3*x5**2+x1**5*x3**2*x5
p(519) = x1**5*x4*x6**2+x1**5*x2*x6**2+x2**5*x4*x5**2+x1*x2**5*x5&
&**2+x2*x3**2*x4**5+x1*x3**2*x4**5
p(520) = x1*x4**5*x6**2+x1*x2**5*x6**2+x2*x4**5*x5**2+x1**5*x2*x5&
&**2+x2**5*x3**2*x4+x1**5*x3**2*x4
p(521) = x2*x4**5*x6**2+x2**5*x4*x6**2+x1*x4**5*x5**2+x1**5*x4*x5&
&**2+x1*x2**5*x3**2+x1**5*x2*x3**2
p(522) = x1**5*x4**2*x6+x1**5*x2**2*x6+x2**5*x4**2*x5+x1**2*x2**5&
&*x5+x2**2*x3*x4**5+x1**2*x3*x4**5
p(523) = x1**2*x4**5*x6+x1**2*x2**5*x6+x2**2*x4**5*x5+x1**5*x2**2&
&*x5+x2**5*x3*x4**2+x1**5*x3*x4**2
p(524) = x1*x2**2*x4**5+x1**2*x2*x4**5+x1*x2**5*x4**2+x1**5*x2*x4&
&**2+x1**2*x2**5*x4+x1**5*x2**2*x4
p(525) = x2**2*x4**5*x6+x2**5*x4**2*x6+x1**2*x4**5*x5+x1**5*x4**2&
&*x5+x1**2*x2**5*x3+x1**5*x2**2*x3
p(526) = x2**6*x5*x6+x1**6*x5*x6+x3*x4**6*x6+x1**6*x3*x6+x3*x4**6&
&*x5+x2**6*x3*x5
p(527) = x4**6*x5*x6+x2**6*x3*x6+x1**6*x3*x5
p(528) = x1**6*x4*x6+x1**6*x2*x6+x2**6*x4*x5+x1*x2**6*x5+x2*x3*x4&
&**6+x1*x3*x4**6
p(529) = x1*x4**6*x6+x1*x2**6*x6+x2*x4**6*x5+x1**6*x2*x5+x2**6*x3&
&*x4+x1**6*x3*x4
p(530) = x1*x2*x4**6+x1*x2**6*x4+x1**6*x2*x4
p(531) = x2*x4**6*x6+x2**6*x4*x6+x1*x4**6*x5+x1**6*x4*x5+x1*x2**6&
&*x3+x1**6*x2*x3
p(532) = x5**4*x6**4+x3**4*x6**4+x3**4*x5**4
p(533) = x5**3*x6**5+x3**3*x6**5+x5**5*x6**3+x3**5*x6**3+x3**3*x5&
&**5+x3**5*x5**3
p(534) = x5**2*x6**6+x3**2*x6**6+x5**6*x6**2+x3**6*x6**2+x3**2*x5&
&**6+x3**6*x5**2
p(535) = x5*x6**7+x3*x6**7+x5**7*x6+x3**7*x6+x3*x5**7+x3**7*x5
p(536) = x4*x6**7+x2*x6**7+x4*x5**7+x1*x5**7+x2*x3**7+x1*x3**7
p(537) = x4**2*x6**6+x2**2*x6**6+x4**2*x5**6+x1**2*x5**6+x2**2*x3&
&**6+x1**2*x3**6
p(538) = x4**3*x6**5+x2**3*x6**5+x4**3*x5**5+x1**3*x5**5+x2**3*x3&
&**5+x1**3*x3**5
p(539) = x4**4*x6**4+x2**4*x6**4+x4**4*x5**4+x1**4*x5**4+x2**4*x3&
&**4+x1**4*x3**4
p(540) = x2**4*x4**4+x1**4*x4**4+x1**4*x2**4
p(541) = x4**5*x6**3+x2**5*x6**3+x4**5*x5**3+x1**5*x5**3+x2**5*x3&
&**3+x1**5*x3**3
p(542) = x2**3*x4**5+x1**3*x4**5+x2**5*x4**3+x1**5*x4**3+x1**3*x2&
&**5+x1**5*x2**3
p(543) = x4**6*x6**2+x2**6*x6**2+x4**6*x5**2+x1**6*x5**2+x2**6*x3&
&**2+x1**6*x3**2
p(544) = x2**2*x4**6+x1**2*x4**6+x2**6*x4**2+x1**6*x4**2+x1**2*x2&
&**6+x1**6*x2**2
p(545) = x4**7*x6+x2**7*x6+x4**7*x5+x1**7*x5+x2**7*x3+x1**7*x3
p(546) = x2*x4**7+x1*x4**7+x2**7*x4+x1**7*x4+x1*x2**7+x1**7*x2
end subroutine ev_pol_con_8

!===========================================================================
! *def grad_ev_pol_con_8(x,dpdx1,dpdx2,dpdx3,dpdx4,dpdx5,dpdx6)
! subtoutine of evaluating polynomials gradient at 8th order
!===========================================================================
subroutine grad_ev_pol_con_8(x,dpdx1,dpdx2,dpdx3,dpdx4,dpdx5,dpdx6)
implicit none
real*8,dimension(1:6),intent(in)::x
real*8,dimension(1:546),intent(out)::dpdx1,dpdx2,dpdx3,dpdx4,dpdx5,dpdx6
real*8::x1,x2,x3,x4,x5,x6

x1=x(1)
x2=x(2)
x3=x(3)
x4=x(4)
x5=x(5)
x6=x(6)

dpdx1(1) = 0
dpdx1(2) = 0
dpdx1(3) = x5+x3
dpdx1(4) = x4+x2
dpdx1(5) = 0
dpdx1(6) = x5*x6+x3*x6
dpdx1(7) = x3*x5
dpdx1(8) = x4*x6+x2*x6+x2*x5+x3*x4
dpdx1(9) = x2*x4
dpdx1(10) = x4*x5+x2*x3
dpdx1(11) = 0
dpdx1(12) = x5**2+x3**2
dpdx1(13) = 2*x1*x5+2*x1*x3
dpdx1(14) = x4**2+2*x1*x4+x2**2+2*x1*x2
dpdx1(15) = x3*x5*x6
dpdx1(16) = x2*x5*x6+x3*x4*x6
dpdx1(17) = x4*x5*x6+x2*x3*x6+x3*x4*x5+x2*x3*x5
dpdx1(18) = x2*x4*x6+x2*x4*x5+x2*x3*x4
dpdx1(19) = 0
dpdx1(20) = x5*x6**2+x3*x6**2
dpdx1(21) = x5**2*x6+x3**2*x6
dpdx1(22) = x3*x5**2+x3**2*x5
dpdx1(23) = x4*x6**2+x2*x6**2+x2*x5**2+x3**2*x4
dpdx1(24) = x4*x5**2+x2*x3**2
dpdx1(25) = 2*x1*x5*x6+2*x1*x3*x6
dpdx1(26) = 2*x1*x3*x5
dpdx1(27) = 2*x1*x4*x6+2*x1*x2*x6+x2**2*x5+x3*x4**2
dpdx1(28) = x4**2*x6+x2**2*x6+2*x1*x2*x5+2*x1*x3*x4
dpdx1(29) = x2*x4**2+x2**2*x4+2*x1*x2*x4
dpdx1(30) = x4**2*x5+2*x1*x4*x5+x2**2*x3+2*x1*x2*x3
dpdx1(31) = 0
dpdx1(32) = 0
dpdx1(33) = x5**3+x3**3
dpdx1(34) = 2*x1*x5**2+2*x1*x3**2
dpdx1(35) = 2*x1*x4**2+2*x1*x2**2
dpdx1(36) = 3*x1**2*x5+3*x1**2*x3
dpdx1(37) = x4**3+3*x1**2*x4+x2**3+3*x1**2*x2
dpdx1(38) = x3*x4*x5*x6+x2*x3*x5*x6
dpdx1(39) = x2*x4*x5*x6+x2*x3*x4*x6+x2*x3*x4*x5
dpdx1(40) = x3*x5*x6**2
dpdx1(41) = x3*x5**2*x6+x3**2*x5*x6
dpdx1(42) = x2*x5*x6**2+x3*x4*x6**2+x2*x5**2*x6+x3**2*x4*x6
dpdx1(43) = x4*x5*x6**2+x2*x3*x6**2+x2*x3*x5**2+x3**2*x4*x5
dpdx1(44) = x4*x5**2*x6+x2*x3**2*x6+x3*x4*x5**2+x2*x3**2*x5
dpdx1(45) = x2*x4*x6**2+x2*x4*x5**2+x2*x3**2*x4
dpdx1(46) = 2*x1*x3*x5*x6
dpdx1(47) = x2**2*x5*x6+2*x1*x2*x5*x6+x3*x4**2*x6+2*x1*x3*x4*x6
dpdx1(48) = 2*x1*x2*x4*x6+x2**2*x4*x5+x2*x3*x4**2
dpdx1(49) = 2*x1*x4*x5*x6+2*x1*x2*x3*x6+x3*x4**2*x5+x2**2*x3*x5
dpdx1(50) = x4**2*x5*x6+x2**2*x3*x6+2*x1*x3*x4*x5+2*x1*x2*x3*x5
dpdx1(51) = x2*x4**2*x6+x2**2*x4*x6+x2*x4**2*x5+2*x1*x2*x4*x5+x2*&
&*2*x3*x4+2*x1*x2*x3*x4
dpdx1(52) = 0
dpdx1(53) = 0
dpdx1(54) = x5*x6**3+x3*x6**3
dpdx1(55) = x5**2*x6**2+x3**2*x6**2
dpdx1(56) = x3**2*x5**2
dpdx1(57) = x5**3*x6+x3**3*x6
dpdx1(58) = x3*x5**3+x3**3*x5
dpdx1(59) = x4*x6**3+x2*x6**3+x2*x5**3+x3**3*x4
dpdx1(60) = x4*x5**3+x2*x3**3
dpdx1(61) = 2*x1*x5*x6**2+2*x1*x3*x6**2
dpdx1(62) = 2*x1*x5**2*x6+2*x1*x3**2*x6
dpdx1(63) = 2*x1*x3*x5**2+2*x1*x3**2*x5
dpdx1(64) = 2*x1*x4*x6**2+2*x1*x2*x6**2+x2**2*x5**2+x3**2*x4**2
dpdx1(65) = x4**2*x6**2+x2**2*x6**2+2*x1*x2*x5**2+2*x1*x3**2*x4
dpdx1(66) = x4**2*x5**2+2*x1*x4*x5**2+x2**2*x3**2+2*x1*x2*x3**2
dpdx1(67) = 2*x1*x4**2*x6+2*x1*x2**2*x6+2*x1*x2**2*x5+2*x1*x3*x4*&
&*2
dpdx1(68) = x2**2*x4**2+2*x1*x2*x4**2+2*x1*x2**2*x4
dpdx1(69) = 2*x1*x4**2*x5+2*x1*x2**2*x3
dpdx1(70) = 3*x1**2*x5*x6+3*x1**2*x3*x6
dpdx1(71) = 3*x1**2*x3*x5
dpdx1(72) = 3*x1**2*x4*x6+3*x1**2*x2*x6+x2**3*x5+x3*x4**3
dpdx1(73) = x4**3*x6+x2**3*x6+3*x1**2*x2*x5+3*x1**2*x3*x4
dpdx1(74) = x2*x4**3+x2**3*x4+3*x1**2*x2*x4
dpdx1(75) = x4**3*x5+3*x1**2*x4*x5+x2**3*x3+3*x1**2*x2*x3
dpdx1(76) = 0
dpdx1(77) = 0
dpdx1(78) = x5**4+x3**4
dpdx1(79) = 2*x1*x5**3+2*x1*x3**3
dpdx1(80) = 3*x1**2*x5**2+3*x1**2*x3**2
dpdx1(81) = 2*x1*x4**3+3*x1**2*x4**2+2*x1*x2**3+3*x1**2*x2**2
dpdx1(82) = 4*x1**3*x5+4*x1**3*x3
dpdx1(83) = x4**4+4*x1**3*x4+x2**4+4*x1**3*x2
dpdx1(84) = x2*x3*x4*x5*x6
dpdx1(85) = x3*x4*x5*x6**2+x2*x3*x5*x6**2+x2*x3*x5**2*x6+x3**2*x4&
&*x5*x6
dpdx1(86) = x3*x4*x5**2*x6+x2*x3**2*x5*x6
dpdx1(87) = x2*x4*x5*x6**2+x2*x3*x4*x6**2+x2*x4*x5**2*x6+x2*x3**2&
&*x4*x6+x2*x3*x4*x5**2+x2*x3**2*x4*x5
dpdx1(88) = x3*x4**2*x5*x6+2*x1*x3*x4*x5*x6+x2**2*x3*x5*x6+2*x1*x&
&2*x3*x5*x6
dpdx1(89) = x2**2*x4*x5*x6+2*x1*x2*x4*x5*x6+x2*x3*x4**2*x6+2*x1*x&
&2*x3*x4*x6+x2*x3*x4**2*x5+x2**2*x3*x4*x5
dpdx1(90) = x2*x4**2*x5*x6+x2**2*x3*x4*x6+2*x1*x2*x3*x4*x5
dpdx1(91) = x3*x5*x6**3
dpdx1(92) = x3*x5**2*x6**2+x3**2*x5*x6**2
dpdx1(93) = x3**2*x5**2*x6
dpdx1(94) = x3*x5**3*x6+x3**3*x5*x6
dpdx1(95) = x2*x5**2*x6**2+x3**2*x4*x6**2
dpdx1(96) = x2*x5*x6**3+x3*x4*x6**3+x2*x5**3*x6+x3**3*x4*x6
dpdx1(97) = x4*x5*x6**3+x2*x3*x6**3+x2*x3*x5**3+x3**3*x4*x5
dpdx1(98) = x4*x5**2*x6**2+x2*x3**2*x6**2+x3**2*x4*x5**2+x2*x3**2&
&*x5**2
dpdx1(99) = x4*x5**3*x6+x2*x3**3*x6+x3*x4*x5**3+x2*x3**3*x5
dpdx1(100) = x2*x4*x6**3+x2*x4*x5**3+x2*x3**3*x4
dpdx1(101) = 2*x1*x3*x5*x6**2
dpdx1(102) = 2*x1*x3*x5**2*x6+2*x1*x3**2*x5*x6
dpdx1(103) = 2*x1*x2*x5*x6**2+2*x1*x3*x4*x6**2+x2**2*x5**2*x6+x3*&
&*2*x4**2*x6
dpdx1(104) = x2**2*x5*x6**2+x3*x4**2*x6**2+2*x1*x2*x5**2*x6+2*x1*&
&x3**2*x4*x6
dpdx1(105) = 2*x1*x2*x4*x6**2+x2**2*x4*x5**2+x2*x3**2*x4**2
dpdx1(106) = 2*x1*x4*x5*x6**2+2*x1*x2*x3*x6**2+x2**2*x3*x5**2+x3*&
&*2*x4**2*x5
dpdx1(107) = x4**2*x5*x6**2+x2**2*x3*x6**2+2*x1*x2*x3*x5**2+2*x1*&
&x3**2*x4*x5
dpdx1(108) = 2*x1*x4*x5**2*x6+2*x1*x2*x3**2*x6+x3*x4**2*x5**2+x2*&
&*2*x3**2*x5
dpdx1(109) = x4**2*x5**2*x6+x2**2*x3**2*x6+2*x1*x3*x4*x5**2+2*x1*&
&x2*x3**2*x5
dpdx1(110) = x2*x4**2*x6**2+x2**2*x4*x6**2+x2*x4**2*x5**2+2*x1*x2&
&*x4*x5**2+x2**2*x3**2*x4+2*x1*x2*x3**2*x4
dpdx1(111) = 2*x1*x2**2*x5*x6+2*x1*x3*x4**2*x6
dpdx1(112) = 2*x1*x2*x4**2*x6+2*x1*x2**2*x4*x6+x2**2*x4**2*x5+2*x&
&1*x2**2*x4*x5+x2**2*x3*x4**2+2*x1*x2*x3*x4**2
dpdx1(113) = 2*x1*x4**2*x5*x6+2*x1*x2**2*x3*x6+2*x1*x3*x4**2*x5+2&
&*x1*x2**2*x3*x5
dpdx1(114) = x2**2*x4**2*x6+2*x1*x2*x4**2*x5+2*x1*x2**2*x3*x4
dpdx1(115) = 3*x1**2*x3*x5*x6
dpdx1(116) = x2**3*x5*x6+3*x1**2*x2*x5*x6+x3*x4**3*x6+3*x1**2*x3*&
&x4*x6
dpdx1(117) = 3*x1**2*x2*x4*x6+x2**3*x4*x5+x2*x3*x4**3
dpdx1(118) = 3*x1**2*x4*x5*x6+3*x1**2*x2*x3*x6+x3*x4**3*x5+x2**3*&
&x3*x5
dpdx1(119) = x4**3*x5*x6+x2**3*x3*x6+3*x1**2*x3*x4*x5+3*x1**2*x2*&
&x3*x5
dpdx1(120) = x2*x4**3*x6+x2**3*x4*x6+x2*x4**3*x5+3*x1**2*x2*x4*x5&
&+x2**3*x3*x4+3*x1**2*x2*x3*x4
dpdx1(121) = 0
dpdx1(122) = 0
dpdx1(123) = 0
dpdx1(124) = x5*x6**4+x3*x6**4
dpdx1(125) = x5**2*x6**3+x3**2*x6**3
dpdx1(126) = x5**3*x6**2+x3**3*x6**2
dpdx1(127) = x3**2*x5**3+x3**3*x5**2
dpdx1(128) = x5**4*x6+x3**4*x6
dpdx1(129) = x3*x5**4+x3**4*x5
dpdx1(130) = x4*x6**4+x2*x6**4+x2*x5**4+x3**4*x4
dpdx1(131) = x4*x5**4+x2*x3**4
dpdx1(132) = 2*x1*x5*x6**3+2*x1*x3*x6**3
dpdx1(133) = 2*x1*x5**2*x6**2+2*x1*x3**2*x6**2
dpdx1(134) = 2*x1*x3**2*x5**2
dpdx1(135) = 2*x1*x5**3*x6+2*x1*x3**3*x6
dpdx1(136) = 2*x1*x3*x5**3+2*x1*x3**3*x5
dpdx1(137) = 2*x1*x4*x6**3+2*x1*x2*x6**3+x2**2*x5**3+x3**3*x4**2
dpdx1(138) = x4**2*x6**3+x2**2*x6**3+2*x1*x2*x5**3+2*x1*x3**3*x4
dpdx1(139) = x4**2*x5**3+2*x1*x4*x5**3+x2**2*x3**3+2*x1*x2*x3**3
dpdx1(140) = 2*x1*x4**2*x6**2+2*x1*x2**2*x6**2+2*x1*x2**2*x5**2+2&
&*x1*x3**2*x4**2
dpdx1(141) = 2*x1*x2**2*x4**2
dpdx1(142) = 2*x1*x4**2*x5**2+2*x1*x2**2*x3**2
dpdx1(143) = 3*x1**2*x5*x6**2+3*x1**2*x3*x6**2
dpdx1(144) = 3*x1**2*x5**2*x6+3*x1**2*x3**2*x6
dpdx1(145) = 3*x1**2*x3*x5**2+3*x1**2*x3**2*x5
dpdx1(146) = 3*x1**2*x4*x6**2+3*x1**2*x2*x6**2+x2**3*x5**2+x3**2*&
&x4**3
dpdx1(147) = x4**3*x6**2+x2**3*x6**2+3*x1**2*x2*x5**2+3*x1**2*x3*&
&*2*x4
dpdx1(148) = x4**3*x5**2+3*x1**2*x4*x5**2+x2**3*x3**2+3*x1**2*x2*&
&x3**2
dpdx1(149) = 3*x1**2*x4**2*x6+3*x1**2*x2**2*x6+2*x1*x2**3*x5+2*x1&
&*x3*x4**3
dpdx1(150) = 2*x1*x4**3*x6+2*x1*x2**3*x6+3*x1**2*x2**2*x5+3*x1**2&
&*x3*x4**2
dpdx1(151) = x2**2*x4**3+2*x1*x2*x4**3+x2**3*x4**2+3*x1**2*x2*x4*&
&*2+2*x1*x2**3*x4+3*x1**2*x2**2*x4
dpdx1(152) = 2*x1*x4**3*x5+3*x1**2*x4**2*x5+2*x1*x2**3*x3+3*x1**2&
&*x2**2*x3
dpdx1(153) = 4*x1**3*x5*x6+4*x1**3*x3*x6
dpdx1(154) = 4*x1**3*x3*x5
dpdx1(155) = 4*x1**3*x4*x6+4*x1**3*x2*x6+x2**4*x5+x3*x4**4
dpdx1(156) = x4**4*x6+x2**4*x6+4*x1**3*x2*x5+4*x1**3*x3*x4
dpdx1(157) = x2*x4**4+x2**4*x4+4*x1**3*x2*x4
dpdx1(158) = x4**4*x5+4*x1**3*x4*x5+x2**4*x3+4*x1**3*x2*x3
dpdx1(159) = 0
dpdx1(160) = 0
dpdx1(161) = 0
dpdx1(162) = x5**5+x3**5
dpdx1(163) = 2*x1*x5**4+2*x1*x3**4
dpdx1(164) = 3*x1**2*x5**3+3*x1**2*x3**3
dpdx1(165) = 3*x1**2*x4**3+3*x1**2*x2**3
dpdx1(166) = 4*x1**3*x5**2+4*x1**3*x3**2
dpdx1(167) = 2*x1*x4**4+4*x1**3*x4**2+2*x1*x2**4+4*x1**3*x2**2
dpdx1(168) = 5*x1**4*x5+5*x1**4*x3
dpdx1(169) = x4**5+5*x1**4*x4+x2**5+5*x1**4*x2
dpdx1(170) = x2*x3*x4*x5*x6**2+x2*x3*x4*x5**2*x6+x2*x3**2*x4*x5*x&
&6
dpdx1(171) = x2*x3*x4**2*x5*x6+x2**2*x3*x4*x5*x6+2*x1*x2*x3*x4*x5&
&*x6
dpdx1(172) = x2*x3*x5**2*x6**2+x3**2*x4*x5*x6**2
dpdx1(173) = x3*x4*x5*x6**3+x2*x3*x5*x6**3+x2*x3*x5**3*x6+x3**3*x&
&4*x5*x6
dpdx1(174) = x3*x4*x5**2*x6**2+x2*x3**2*x5*x6**2+x3**2*x4*x5**2*x&
&6+x2*x3**2*x5**2*x6
dpdx1(175) = x2*x4*x5**2*x6**2+x2*x3**2*x4*x6**2+x2*x3**2*x4*x5**&
&2
dpdx1(176) = x3*x4*x5**3*x6+x2*x3**3*x5*x6
dpdx1(177) = x2*x4*x5*x6**3+x2*x3*x4*x6**3+x2*x4*x5**3*x6+x2*x3**&
&3*x4*x6+x2*x3*x4*x5**3+x2*x3**3*x4*x5
dpdx1(178) = 2*x1*x3*x4*x5*x6**2+2*x1*x2*x3*x5*x6**2+x2**2*x3*x5*&
&*2*x6+x3**2*x4**2*x5*x6
dpdx1(179) = x3*x4**2*x5*x6**2+x2**2*x3*x5*x6**2+2*x1*x2*x3*x5**2&
&*x6+2*x1*x3**2*x4*x5*x6
dpdx1(180) = 2*x1*x2*x4*x5*x6**2+2*x1*x2*x3*x4*x6**2+x2**2*x4*x5*&
&*2*x6+x2*x3**2*x4**2*x6+x2**2*x3*x4*x5**2+x2*x3**2*x4**2*x5
dpdx1(181) = x3*x4**2*x5**2*x6+2*x1*x3*x4*x5**2*x6+x2**2*x3**2*x5&
&*x6+2*x1*x2*x3**2*x5*x6
dpdx1(182) = x2**2*x4*x5*x6**2+x2*x3*x4**2*x6**2+2*x1*x2*x4*x5**2&
&*x6+2*x1*x2*x3**2*x4*x6+x2*x3*x4**2*x5**2+x2**2*x3**2*x4*x5
dpdx1(183) = x2*x4**2*x5*x6**2+x2**2*x3*x4*x6**2+x2*x4**2*x5**2*x&
&6+x2**2*x3**2*x4*x6+2*x1*x2*x3*x4*x5**2+2*x1*x2*x3**2*x4*x5
dpdx1(184) = 2*x1*x2**2*x4*x5*x6+2*x1*x2*x3*x4**2*x6+x2**2*x3*x4*&
&*2*x5
dpdx1(185) = 2*x1*x3*x4**2*x5*x6+2*x1*x2**2*x3*x5*x6
dpdx1(186) = x2**2*x4**2*x5*x6+2*x1*x2*x4**2*x5*x6+x2**2*x3*x4**2&
&*x6+2*x1*x2**2*x3*x4*x6+2*x1*x2*x3*x4**2*x5+2*x1*x2**2*x3*x4*x5
dpdx1(187) = x3*x4**3*x5*x6+3*x1**2*x3*x4*x5*x6+x2**3*x3*x5*x6+3*&
&x1**2*x2*x3*x5*x6
dpdx1(188) = x2**3*x4*x5*x6+3*x1**2*x2*x4*x5*x6+x2*x3*x4**3*x6+3*&
&x1**2*x2*x3*x4*x6+x2*x3*x4**3*x5+x2**3*x3*x4*x5
dpdx1(189) = x2*x4**3*x5*x6+x2**3*x3*x4*x6+3*x1**2*x2*x3*x4*x5
dpdx1(190) = x3*x5*x6**4
dpdx1(191) = x3*x5**2*x6**3+x3**2*x5*x6**3
dpdx1(192) = x3**2*x5**2*x6**2
dpdx1(193) = x3*x5**3*x6**2+x3**3*x5*x6**2
dpdx1(194) = x3**2*x5**3*x6+x3**3*x5**2*x6
dpdx1(195) = x3*x5**4*x6+x3**4*x5*x6
dpdx1(196) = x2*x5**2*x6**3+x3**2*x4*x6**3+x2*x5**3*x6**2+x3**3*x&
&4*x6**2
dpdx1(197) = x2*x5*x6**4+x3*x4*x6**4+x2*x5**4*x6+x3**4*x4*x6
dpdx1(198) = x4*x5*x6**4+x2*x3*x6**4+x2*x3*x5**4+x3**4*x4*x5
dpdx1(199) = x4*x5**2*x6**3+x2*x3**2*x6**3+x2*x3**2*x5**3+x3**3*x&
&4*x5**2
dpdx1(200) = x4*x5**3*x6**2+x2*x3**3*x6**2+x3**2*x4*x5**3+x2*x3**&
&3*x5**2
dpdx1(201) = x4*x5**4*x6+x2*x3**4*x6+x3*x4*x5**4+x2*x3**4*x5
dpdx1(202) = x2*x4*x6**4+x2*x4*x5**4+x2*x3**4*x4
dpdx1(203) = 2*x1*x3*x5*x6**3
dpdx1(204) = 2*x1*x3*x5**2*x6**2+2*x1*x3**2*x5*x6**2
dpdx1(205) = 2*x1*x3**2*x5**2*x6
dpdx1(206) = 2*x1*x3*x5**3*x6+2*x1*x3**3*x5*x6
dpdx1(207) = 2*x1*x2*x5*x6**3+2*x1*x3*x4*x6**3+x2**2*x5**3*x6+x3*&
&*3*x4**2*x6
dpdx1(208) = x2**2*x5**2*x6**2+2*x1*x2*x5**2*x6**2+x3**2*x4**2*x6&
&**2+2*x1*x3**2*x4*x6**2
dpdx1(209) = x2**2*x5*x6**3+x3*x4**2*x6**3+2*x1*x2*x5**3*x6+2*x1*&
&x3**3*x4*x6
dpdx1(210) = 2*x1*x2*x4*x6**3+x2**2*x4*x5**3+x2*x3**3*x4**2
dpdx1(211) = 2*x1*x4*x5*x6**3+2*x1*x2*x3*x6**3+x2**2*x3*x5**3+x3*&
&*3*x4**2*x5
dpdx1(212) = x4**2*x5*x6**3+x2**2*x3*x6**3+2*x1*x2*x3*x5**3+2*x1*&
&x3**3*x4*x5
dpdx1(213) = 2*x1*x4*x5**2*x6**2+2*x1*x2*x3**2*x6**2+x3**2*x4**2*&
&x5**2+x2**2*x3**2*x5**2
dpdx1(214) = x4**2*x5**2*x6**2+x2**2*x3**2*x6**2+2*x1*x3**2*x4*x5&
&**2+2*x1*x2*x3**2*x5**2
dpdx1(215) = 2*x1*x4*x5**3*x6+2*x1*x2*x3**3*x6+x3*x4**2*x5**3+x2*&
&*2*x3**3*x5
dpdx1(216) = x4**2*x5**3*x6+x2**2*x3**3*x6+2*x1*x3*x4*x5**3+2*x1*&
&x2*x3**3*x5
dpdx1(217) = x2*x4**2*x6**3+x2**2*x4*x6**3+x2*x4**2*x5**3+2*x1*x2&
&*x4*x5**3+x2**2*x3**3*x4+2*x1*x2*x3**3*x4
dpdx1(218) = 2*x1*x2**2*x5*x6**2+2*x1*x3*x4**2*x6**2+2*x1*x2**2*x&
&5**2*x6+2*x1*x3**2*x4**2*x6
dpdx1(219) = 2*x1*x2*x4**2*x6**2+2*x1*x2**2*x4*x6**2+x2**2*x4**2*&
&x5**2+2*x1*x2**2*x4*x5**2+x2**2*x3**2*x4**2+2*x1*x2*x3**2*x4**2
dpdx1(220) = 2*x1*x4**2*x5*x6**2+2*x1*x2**2*x3*x6**2+2*x1*x2**2*x&
&3*x5**2+2*x1*x3**2*x4**2*x5
dpdx1(221) = 2*x1*x2**2*x4**2*x6+2*x1*x2**2*x4**2*x5+2*x1*x2**2*x&
&3*x4**2
dpdx1(222) = 2*x1*x4**2*x5**2*x6+2*x1*x2**2*x3**2*x6+2*x1*x3*x4**&
&2*x5**2+2*x1*x2**2*x3**2*x5
dpdx1(223) = x2**2*x4**2*x6**2+2*x1*x2*x4**2*x5**2+2*x1*x2**2*x3*&
&*2*x4
dpdx1(224) = 3*x1**2*x3*x5*x6**2
dpdx1(225) = 3*x1**2*x3*x5**2*x6+3*x1**2*x3**2*x5*x6
dpdx1(226) = 3*x1**2*x2*x5*x6**2+3*x1**2*x3*x4*x6**2+x2**3*x5**2*&
&x6+x3**2*x4**3*x6
dpdx1(227) = x2**3*x5*x6**2+x3*x4**3*x6**2+3*x1**2*x2*x5**2*x6+3*&
&x1**2*x3**2*x4*x6
dpdx1(228) = 3*x1**2*x2*x4*x6**2+x2**3*x4*x5**2+x2*x3**2*x4**3
dpdx1(229) = 3*x1**2*x4*x5*x6**2+3*x1**2*x2*x3*x6**2+x2**3*x3*x5*&
&*2+x3**2*x4**3*x5
dpdx1(230) = x4**3*x5*x6**2+x2**3*x3*x6**2+3*x1**2*x2*x3*x5**2+3*&
&x1**2*x3**2*x4*x5
dpdx1(231) = 3*x1**2*x4*x5**2*x6+3*x1**2*x2*x3**2*x6+x3*x4**3*x5*&
&*2+x2**3*x3**2*x5
dpdx1(232) = x4**3*x5**2*x6+x2**3*x3**2*x6+3*x1**2*x3*x4*x5**2+3*&
&x1**2*x2*x3**2*x5
dpdx1(233) = x2*x4**3*x6**2+x2**3*x4*x6**2+x2*x4**3*x5**2+3*x1**2&
&*x2*x4*x5**2+x2**3*x3**2*x4+3*x1**2*x2*x3**2*x4
dpdx1(234) = 2*x1*x2**3*x5*x6+3*x1**2*x2**2*x5*x6+2*x1*x3*x4**3*x&
&6+3*x1**2*x3*x4**2*x6
dpdx1(235) = 3*x1**2*x2*x4**2*x6+3*x1**2*x2**2*x4*x6+x2**3*x4**2*&
&x5+2*x1*x2**3*x4*x5+x2**2*x3*x4**3+2*x1*x2*x3*x4**3
dpdx1(236) = 2*x1*x2*x4**3*x6+2*x1*x2**3*x4*x6+x2**2*x4**3*x5+3*x&
&1**2*x2**2*x4*x5+x2**3*x3*x4**2+3*x1**2*x2*x3*x4**2
dpdx1(237) = 3*x1**2*x4**2*x5*x6+3*x1**2*x2**2*x3*x6+2*x1*x3*x4**&
&3*x5+2*x1*x2**3*x3*x5
dpdx1(238) = 2*x1*x4**3*x5*x6+2*x1*x2**3*x3*x6+3*x1**2*x3*x4**2*x&
&5+3*x1**2*x2**2*x3*x5
dpdx1(239) = x2**2*x4**3*x6+x2**3*x4**2*x6+2*x1*x2*x4**3*x5+3*x1*&
&*2*x2*x4**2*x5+2*x1*x2**3*x3*x4+3*x1**2*x2**2*x3*x4
dpdx1(240) = 4*x1**3*x3*x5*x6
dpdx1(241) = x2**4*x5*x6+4*x1**3*x2*x5*x6+x3*x4**4*x6+4*x1**3*x3*&
&x4*x6
dpdx1(242) = 4*x1**3*x2*x4*x6+x2**4*x4*x5+x2*x3*x4**4
dpdx1(243) = 4*x1**3*x4*x5*x6+4*x1**3*x2*x3*x6+x3*x4**4*x5+x2**4*&
&x3*x5
dpdx1(244) = x4**4*x5*x6+x2**4*x3*x6+4*x1**3*x3*x4*x5+4*x1**3*x2*&
&x3*x5
dpdx1(245) = x2*x4**4*x6+x2**4*x4*x6+x2*x4**4*x5+4*x1**3*x2*x4*x5&
&+x2**4*x3*x4+4*x1**3*x2*x3*x4
dpdx1(246) = 0
dpdx1(247) = 0
dpdx1(248) = 0
dpdx1(249) = 0
dpdx1(250) = x5*x6**5+x3*x6**5
dpdx1(251) = x5**2*x6**4+x3**2*x6**4
dpdx1(252) = x5**3*x6**3+x3**3*x6**3
dpdx1(253) = x3**3*x5**3
dpdx1(254) = x5**4*x6**2+x3**4*x6**2
dpdx1(255) = x3**2*x5**4+x3**4*x5**2
dpdx1(256) = x5**5*x6+x3**5*x6
dpdx1(257) = x3*x5**5+x3**5*x5
dpdx1(258) = x4*x6**5+x2*x6**5+x2*x5**5+x3**5*x4
dpdx1(259) = x4*x5**5+x2*x3**5
dpdx1(260) = 2*x1*x5*x6**4+2*x1*x3*x6**4
dpdx1(261) = 2*x1*x5**2*x6**3+2*x1*x3**2*x6**3
dpdx1(262) = 2*x1*x5**3*x6**2+2*x1*x3**3*x6**2
dpdx1(263) = 2*x1*x3**2*x5**3+2*x1*x3**3*x5**2
dpdx1(264) = 2*x1*x5**4*x6+2*x1*x3**4*x6
dpdx1(265) = 2*x1*x3*x5**4+2*x1*x3**4*x5
dpdx1(266) = 2*x1*x4*x6**4+2*x1*x2*x6**4+x2**2*x5**4+x3**4*x4**2
dpdx1(267) = x4**2*x6**4+x2**2*x6**4+2*x1*x2*x5**4+2*x1*x3**4*x4
dpdx1(268) = x4**2*x5**4+2*x1*x4*x5**4+x2**2*x3**4+2*x1*x2*x3**4
dpdx1(269) = 2*x1*x4**2*x6**3+2*x1*x2**2*x6**3+2*x1*x2**2*x5**3+2&
&*x1*x3**3*x4**2
dpdx1(270) = 2*x1*x4**2*x5**3+2*x1*x2**2*x3**3
dpdx1(271) = 3*x1**2*x5*x6**3+3*x1**2*x3*x6**3
dpdx1(272) = 3*x1**2*x5**2*x6**2+3*x1**2*x3**2*x6**2
dpdx1(273) = 3*x1**2*x3**2*x5**2
dpdx1(274) = 3*x1**2*x5**3*x6+3*x1**2*x3**3*x6
dpdx1(275) = 3*x1**2*x3*x5**3+3*x1**2*x3**3*x5
dpdx1(276) = 3*x1**2*x4*x6**3+3*x1**2*x2*x6**3+x2**3*x5**3+x3**3*&
&x4**3
dpdx1(277) = x4**3*x6**3+x2**3*x6**3+3*x1**2*x2*x5**3+3*x1**2*x3*&
&*3*x4
dpdx1(278) = x4**3*x5**3+3*x1**2*x4*x5**3+x2**3*x3**3+3*x1**2*x2*&
&x3**3
dpdx1(279) = 3*x1**2*x4**2*x6**2+3*x1**2*x2**2*x6**2+2*x1*x2**3*x&
&5**2+2*x1*x3**2*x4**3
dpdx1(280) = 2*x1*x4**3*x6**2+2*x1*x2**3*x6**2+3*x1**2*x2**2*x5**&
&2+3*x1**2*x3**2*x4**2
dpdx1(281) = 2*x1*x2**2*x4**3+2*x1*x2**3*x4**2+3*x1**2*x2**2*x4**&
&2
dpdx1(282) = 2*x1*x4**3*x5**2+3*x1**2*x4**2*x5**2+2*x1*x2**3*x3**&
&2+3*x1**2*x2**2*x3**2
dpdx1(283) = 3*x1**2*x4**3*x6+3*x1**2*x2**3*x6+3*x1**2*x2**3*x5+3&
&*x1**2*x3*x4**3
dpdx1(284) = x2**3*x4**3+3*x1**2*x2*x4**3+3*x1**2*x2**3*x4
dpdx1(285) = 3*x1**2*x4**3*x5+3*x1**2*x2**3*x3
dpdx1(286) = 4*x1**3*x5*x6**2+4*x1**3*x3*x6**2
dpdx1(287) = 4*x1**3*x5**2*x6+4*x1**3*x3**2*x6
dpdx1(288) = 4*x1**3*x3*x5**2+4*x1**3*x3**2*x5
dpdx1(289) = 4*x1**3*x4*x6**2+4*x1**3*x2*x6**2+x2**4*x5**2+x3**2*&
&x4**4
dpdx1(290) = x4**4*x6**2+x2**4*x6**2+4*x1**3*x2*x5**2+4*x1**3*x3*&
&*2*x4
dpdx1(291) = x4**4*x5**2+4*x1**3*x4*x5**2+x2**4*x3**2+4*x1**3*x2*&
&x3**2
dpdx1(292) = 4*x1**3*x4**2*x6+4*x1**3*x2**2*x6+2*x1*x2**4*x5+2*x1&
&*x3*x4**4
dpdx1(293) = 2*x1*x4**4*x6+2*x1*x2**4*x6+4*x1**3*x2**2*x5+4*x1**3&
&*x3*x4**2
dpdx1(294) = x2**2*x4**4+2*x1*x2*x4**4+x2**4*x4**2+4*x1**3*x2*x4*&
&*2+2*x1*x2**4*x4+4*x1**3*x2**2*x4
dpdx1(295) = 2*x1*x4**4*x5+4*x1**3*x4**2*x5+2*x1*x2**4*x3+4*x1**3&
&*x2**2*x3
dpdx1(296) = 5*x1**4*x5*x6+5*x1**4*x3*x6
dpdx1(297) = 5*x1**4*x3*x5
dpdx1(298) = 5*x1**4*x4*x6+5*x1**4*x2*x6+x2**5*x5+x3*x4**5
dpdx1(299) = x4**5*x6+x2**5*x6+5*x1**4*x2*x5+5*x1**4*x3*x4
dpdx1(300) = x2*x4**5+x2**5*x4+5*x1**4*x2*x4
dpdx1(301) = x4**5*x5+5*x1**4*x4*x5+x2**5*x3+5*x1**4*x2*x3
dpdx1(302) = 0
dpdx1(303) = 0
dpdx1(304) = 0
dpdx1(305) = x5**6+x3**6
dpdx1(306) = 2*x1*x5**5+2*x1*x3**5
dpdx1(307) = 3*x1**2*x5**4+3*x1**2*x3**4
dpdx1(308) = 4*x1**3*x5**3+4*x1**3*x3**3
dpdx1(309) = 3*x1**2*x4**4+4*x1**3*x4**3+3*x1**2*x2**4+4*x1**3*x2&
&**3
dpdx1(310) = 5*x1**4*x5**2+5*x1**4*x3**2
dpdx1(311) = 2*x1*x4**5+5*x1**4*x4**2+2*x1*x2**5+5*x1**4*x2**2
dpdx1(312) = 6*x1**5*x5+6*x1**5*x3
dpdx1(313) = x4**6+6*x1**5*x4+x2**6+6*x1**5*x2
dpdx1(314) = x2*x3*x4*x5**2*x6**2+x2*x3**2*x4*x5*x6**2+x2*x3**2*x&
&4*x5**2*x6
dpdx1(315) = x2*x3*x4*x5*x6**3+x2*x3*x4*x5**3*x6+x2*x3**3*x4*x5*x&
&6
dpdx1(316) = 2*x1*x2*x3*x4*x5*x6**2+x2**2*x3*x4*x5**2*x6+x2*x3**2&
&*x4**2*x5*x6
dpdx1(317) = x2*x3*x4**2*x5*x6**2+x2**2*x3*x4*x5*x6**2+x2*x3*x4**&
&2*x5**2*x6+2*x1*x2*x3*x4*x5**2*x6+x2**2*x3**2*x4*x5*x6+2*x1*x2*x3&
&**2*x4*x5*x6
dpdx1(318) = x2**2*x3*x4**2*x5*x6+2*x1*x2*x3*x4**2*x5*x6+2*x1*x2*&
&*2*x3*x4*x5*x6
dpdx1(319) = x2*x3*x4**3*x5*x6+x2**3*x3*x4*x5*x6+3*x1**2*x2*x3*x4&
&*x5*x6
dpdx1(320) = x2*x3*x5**2*x6**3+x3**2*x4*x5*x6**3+x2*x3*x5**3*x6**&
&2+x3**3*x4*x5*x6**2
dpdx1(321) = x3*x4*x5*x6**4+x2*x3*x5*x6**4+x2*x3*x5**4*x6+x3**4*x&
&4*x5*x6
dpdx1(322) = x3**2*x4*x5**2*x6**2+x2*x3**2*x5**2*x6**2
dpdx1(323) = x3*x4*x5**2*x6**3+x2*x3**2*x5*x6**3+x2*x3**2*x5**3*x&
&6+x3**3*x4*x5**2*x6
dpdx1(324) = x3*x4*x5**3*x6**2+x2*x3**3*x5*x6**2+x3**2*x4*x5**3*x&
&6+x2*x3**3*x5**2*x6
dpdx1(325) = x2*x4*x5**2*x6**3+x2*x3**2*x4*x6**3+x2*x4*x5**3*x6**&
&2+x2*x3**3*x4*x6**2+x2*x3**2*x4*x5**3+x2*x3**3*x4*x5**2
dpdx1(326) = x3*x4*x5**4*x6+x2*x3**4*x5*x6
dpdx1(327) = x2*x4*x5*x6**4+x2*x3*x4*x6**4+x2*x4*x5**4*x6+x2*x3**&
&4*x4*x6+x2*x3*x4*x5**4+x2*x3**4*x4*x5
dpdx1(328) = 2*x1*x3*x4*x5*x6**3+2*x1*x2*x3*x5*x6**3+x2**2*x3*x5*&
&*3*x6+x3**3*x4**2*x5*x6
dpdx1(329) = x2**2*x3*x5**2*x6**2+2*x1*x2*x3*x5**2*x6**2+x3**2*x4&
&**2*x5*x6**2+2*x1*x3**2*x4*x5*x6**2
dpdx1(330) = x3*x4**2*x5*x6**3+x2**2*x3*x5*x6**3+2*x1*x2*x3*x5**3&
&*x6+2*x1*x3**3*x4*x5*x6
dpdx1(331) = 2*x1*x2*x4*x5*x6**3+2*x1*x2*x3*x4*x6**3+x2**2*x4*x5*&
&*3*x6+x2*x3**3*x4**2*x6+x2**2*x3*x4*x5**3+x2*x3**3*x4**2*x5
dpdx1(332) = 2*x1*x3*x4*x5**2*x6**2+2*x1*x2*x3**2*x5*x6**2+x3**2*&
&x4**2*x5**2*x6+x2**2*x3**2*x5**2*x6
dpdx1(333) = x3*x4**2*x5**2*x6**2+x2**2*x3**2*x5*x6**2+2*x1*x3**2&
&*x4*x5**2*x6+2*x1*x2*x3**2*x5**2*x6
dpdx1(334) = x2**2*x4*x5**2*x6**2+2*x1*x2*x4*x5**2*x6**2+x2*x3**2&
&*x4**2*x6**2+2*x1*x2*x3**2*x4*x6**2+x2*x3**2*x4**2*x5**2+x2**2*x3&
&**2*x4*x5**2
dpdx1(335) = x2*x4**2*x5**2*x6**2+x2**2*x3**2*x4*x6**2+2*x1*x2*x3&
&**2*x4*x5**2
dpdx1(336) = x3*x4**2*x5**3*x6+2*x1*x3*x4*x5**3*x6+x2**2*x3**3*x5&
&*x6+2*x1*x2*x3**3*x5*x6
dpdx1(337) = x2**2*x4*x5*x6**3+x2*x3*x4**2*x6**3+2*x1*x2*x4*x5**3&
&*x6+2*x1*x2*x3**3*x4*x6+x2*x3*x4**2*x5**3+x2**2*x3**3*x4*x5
dpdx1(338) = x2*x4**2*x5*x6**3+x2**2*x3*x4*x6**3+x2*x4**2*x5**3*x&
&6+x2**2*x3**3*x4*x6+2*x1*x2*x3*x4*x5**3+2*x1*x2*x3**3*x4*x5
dpdx1(339) = 2*x1*x2**2*x4*x5*x6**2+2*x1*x2*x3*x4**2*x6**2+2*x1*x&
&2**2*x4*x5**2*x6+2*x1*x2*x3**2*x4**2*x6+x2**2*x3*x4**2*x5**2+x2**&
&2*x3**2*x4**2*x5
dpdx1(340) = 2*x1*x3*x4**2*x5*x6**2+2*x1*x2**2*x3*x5*x6**2+2*x1*x&
&2**2*x3*x5**2*x6+2*x1*x3**2*x4**2*x5*x6
dpdx1(341) = 2*x1*x2*x4**2*x5*x6**2+2*x1*x2**2*x3*x4*x6**2+x2**2*&
&x4**2*x5**2*x6+x2**2*x3**2*x4**2*x6+2*x1*x2**2*x3*x4*x5**2+2*x1*x&
&2*x3**2*x4**2*x5
dpdx1(342) = 2*x1*x2**2*x4**2*x5*x6+2*x1*x2**2*x3*x4**2*x6+2*x1*x&
&2**2*x3*x4**2*x5
dpdx1(343) = 2*x1*x3*x4**2*x5**2*x6+2*x1*x2**2*x3**2*x5*x6
dpdx1(344) = x2**2*x4**2*x5*x6**2+x2**2*x3*x4**2*x6**2+2*x1*x2*x4&
&**2*x5**2*x6+2*x1*x2**2*x3**2*x4*x6+2*x1*x2*x3*x4**2*x5**2+2*x1*x&
&2**2*x3**2*x4*x5
dpdx1(345) = 3*x1**2*x3*x4*x5*x6**2+3*x1**2*x2*x3*x5*x6**2+x2**3*&
&x3*x5**2*x6+x3**2*x4**3*x5*x6
dpdx1(346) = x3*x4**3*x5*x6**2+x2**3*x3*x5*x6**2+3*x1**2*x2*x3*x5&
&**2*x6+3*x1**2*x3**2*x4*x5*x6
dpdx1(347) = 3*x1**2*x2*x4*x5*x6**2+3*x1**2*x2*x3*x4*x6**2+x2**3*&
&x4*x5**2*x6+x2*x3**2*x4**3*x6+x2**3*x3*x4*x5**2+x2*x3**2*x4**3*x5
dpdx1(348) = x3*x4**3*x5**2*x6+3*x1**2*x3*x4*x5**2*x6+x2**3*x3**2&
&*x5*x6+3*x1**2*x2*x3**2*x5*x6
dpdx1(349) = x2**3*x4*x5*x6**2+x2*x3*x4**3*x6**2+3*x1**2*x2*x4*x5&
&**2*x6+3*x1**2*x2*x3**2*x4*x6+x2*x3*x4**3*x5**2+x2**3*x3**2*x4*x5
dpdx1(350) = x2*x4**3*x5*x6**2+x2**3*x3*x4*x6**2+x2*x4**3*x5**2*x&
&6+x2**3*x3**2*x4*x6+3*x1**2*x2*x3*x4*x5**2+3*x1**2*x2*x3**2*x4*x5
dpdx1(351) = 2*x1*x2**3*x4*x5*x6+3*x1**2*x2**2*x4*x5*x6+2*x1*x2*x&
&3*x4**3*x6+3*x1**2*x2*x3*x4**2*x6+x2**2*x3*x4**3*x5+x2**3*x3*x4**&
&2*x5
dpdx1(352) = 2*x1*x3*x4**3*x5*x6+3*x1**2*x3*x4**2*x5*x6+2*x1*x2**&
&3*x3*x5*x6+3*x1**2*x2**2*x3*x5*x6
dpdx1(353) = x2**3*x4**2*x5*x6+3*x1**2*x2*x4**2*x5*x6+x2**2*x3*x4&
&**3*x6+3*x1**2*x2**2*x3*x4*x6+2*x1*x2*x3*x4**3*x5+2*x1*x2**3*x3*x&
&4*x5
dpdx1(354) = x2**2*x4**3*x5*x6+2*x1*x2*x4**3*x5*x6+x2**3*x3*x4**2&
&*x6+2*x1*x2**3*x3*x4*x6+3*x1**2*x2*x3*x4**2*x5+3*x1**2*x2**2*x3*x&
&4*x5
dpdx1(355) = x3*x4**4*x5*x6+4*x1**3*x3*x4*x5*x6+x2**4*x3*x5*x6+4*&
&x1**3*x2*x3*x5*x6
dpdx1(356) = x2**4*x4*x5*x6+4*x1**3*x2*x4*x5*x6+x2*x3*x4**4*x6+4*&
&x1**3*x2*x3*x4*x6+x2*x3*x4**4*x5+x2**4*x3*x4*x5
dpdx1(357) = x2*x4**4*x5*x6+x2**4*x3*x4*x6+4*x1**3*x2*x3*x4*x5
dpdx1(358) = x3*x5*x6**5
dpdx1(359) = x3*x5**2*x6**4+x3**2*x5*x6**4
dpdx1(360) = x3**2*x5**2*x6**3
dpdx1(361) = x3*x5**3*x6**3+x3**3*x5*x6**3
dpdx1(362) = x3**2*x5**3*x6**2+x3**3*x5**2*x6**2
dpdx1(363) = x3**3*x5**3*x6
dpdx1(364) = x3*x5**4*x6**2+x3**4*x5*x6**2
dpdx1(365) = x3**2*x5**4*x6+x3**4*x5**2*x6
dpdx1(366) = x3*x5**5*x6+x3**5*x5*x6
dpdx1(367) = x2*x5**3*x6**3+x3**3*x4*x6**3
dpdx1(368) = x2*x5**2*x6**4+x3**2*x4*x6**4+x2*x5**4*x6**2+x3**4*x&
&4*x6**2
dpdx1(369) = x2*x5*x6**5+x3*x4*x6**5+x2*x5**5*x6+x3**5*x4*x6
dpdx1(370) = x4*x5*x6**5+x2*x3*x6**5+x2*x3*x5**5+x3**5*x4*x5
dpdx1(371) = x4*x5**2*x6**4+x2*x3**2*x6**4+x2*x3**2*x5**4+x3**4*x&
&4*x5**2
dpdx1(372) = x4*x5**3*x6**3+x2*x3**3*x6**3+x3**3*x4*x5**3+x2*x3**&
&3*x5**3
dpdx1(373) = x4*x5**4*x6**2+x2*x3**4*x6**2+x3**2*x4*x5**4+x2*x3**&
&4*x5**2
dpdx1(374) = x4*x5**5*x6+x2*x3**5*x6+x3*x4*x5**5+x2*x3**5*x5
dpdx1(375) = x2*x4*x6**5+x2*x4*x5**5+x2*x3**5*x4
dpdx1(376) = 2*x1*x3*x5*x6**4
dpdx1(377) = 2*x1*x3*x5**2*x6**3+2*x1*x3**2*x5*x6**3
dpdx1(378) = 2*x1*x3**2*x5**2*x6**2
dpdx1(379) = 2*x1*x3*x5**3*x6**2+2*x1*x3**3*x5*x6**2
dpdx1(380) = 2*x1*x3**2*x5**3*x6+2*x1*x3**3*x5**2*x6
dpdx1(381) = 2*x1*x3*x5**4*x6+2*x1*x3**4*x5*x6
dpdx1(382) = 2*x1*x2*x5*x6**4+2*x1*x3*x4*x6**4+x2**2*x5**4*x6+x3*&
&*4*x4**2*x6
dpdx1(383) = 2*x1*x2*x5**2*x6**3+2*x1*x3**2*x4*x6**3+x2**2*x5**3*&
&x6**2+x3**3*x4**2*x6**2
dpdx1(384) = x2**2*x5**2*x6**3+x3**2*x4**2*x6**3+2*x1*x2*x5**3*x6&
&**2+2*x1*x3**3*x4*x6**2
dpdx1(385) = x2**2*x5*x6**4+x3*x4**2*x6**4+2*x1*x2*x5**4*x6+2*x1*&
&x3**4*x4*x6
dpdx1(386) = 2*x1*x2*x4*x6**4+x2**2*x4*x5**4+x2*x3**4*x4**2
dpdx1(387) = 2*x1*x4*x5*x6**4+2*x1*x2*x3*x6**4+x2**2*x3*x5**4+x3*&
&*4*x4**2*x5
dpdx1(388) = x4**2*x5*x6**4+x2**2*x3*x6**4+2*x1*x2*x3*x5**4+2*x1*&
&x3**4*x4*x5
dpdx1(389) = 2*x1*x4*x5**2*x6**3+2*x1*x2*x3**2*x6**3+x2**2*x3**2*&
&x5**3+x3**3*x4**2*x5**2
dpdx1(390) = x4**2*x5**2*x6**3+x2**2*x3**2*x6**3+2*x1*x2*x3**2*x5&
&**3+2*x1*x3**3*x4*x5**2
dpdx1(391) = 2*x1*x4*x5**3*x6**2+2*x1*x2*x3**3*x6**2+x3**2*x4**2*&
&x5**3+x2**2*x3**3*x5**2
dpdx1(392) = x4**2*x5**3*x6**2+x2**2*x3**3*x6**2+2*x1*x3**2*x4*x5&
&**3+2*x1*x2*x3**3*x5**2
dpdx1(393) = 2*x1*x4*x5**4*x6+2*x1*x2*x3**4*x6+x3*x4**2*x5**4+x2*&
&*2*x3**4*x5
dpdx1(394) = x4**2*x5**4*x6+x2**2*x3**4*x6+2*x1*x3*x4*x5**4+2*x1*&
&x2*x3**4*x5
dpdx1(395) = x2*x4**2*x6**4+x2**2*x4*x6**4+x2*x4**2*x5**4+2*x1*x2&
&*x4*x5**4+x2**2*x3**4*x4+2*x1*x2*x3**4*x4
dpdx1(396) = 2*x1*x2**2*x5**2*x6**2+2*x1*x3**2*x4**2*x6**2
dpdx1(397) = 2*x1*x2**2*x5*x6**3+2*x1*x3*x4**2*x6**3+2*x1*x2**2*x&
&5**3*x6+2*x1*x3**3*x4**2*x6
dpdx1(398) = 2*x1*x2*x4**2*x6**3+2*x1*x2**2*x4*x6**3+x2**2*x4**2*&
&x5**3+2*x1*x2**2*x4*x5**3+x2**2*x3**3*x4**2+2*x1*x2*x3**3*x4**2
dpdx1(399) = 2*x1*x4**2*x5*x6**3+2*x1*x2**2*x3*x6**3+2*x1*x2**2*x&
&3*x5**3+2*x1*x3**3*x4**2*x5
dpdx1(400) = 2*x1*x4**2*x5**2*x6**2+2*x1*x2**2*x3**2*x6**2+2*x1*x&
&3**2*x4**2*x5**2+2*x1*x2**2*x3**2*x5**2
dpdx1(401) = 2*x1*x2**2*x4**2*x6**2+2*x1*x2**2*x4**2*x5**2+2*x1*x&
&2**2*x3**2*x4**2
dpdx1(402) = 2*x1*x4**2*x5**3*x6+2*x1*x2**2*x3**3*x6+2*x1*x3*x4**&
&2*x5**3+2*x1*x2**2*x3**3*x5
dpdx1(403) = x2**2*x4**2*x6**3+2*x1*x2*x4**2*x5**3+2*x1*x2**2*x3*&
&*3*x4
dpdx1(404) = 3*x1**2*x3*x5*x6**3
dpdx1(405) = 3*x1**2*x3*x5**2*x6**2+3*x1**2*x3**2*x5*x6**2
dpdx1(406) = 3*x1**2*x3**2*x5**2*x6
dpdx1(407) = 3*x1**2*x3*x5**3*x6+3*x1**2*x3**3*x5*x6
dpdx1(408) = 3*x1**2*x2*x5*x6**3+3*x1**2*x3*x4*x6**3+x2**3*x5**3*&
&x6+x3**3*x4**3*x6
dpdx1(409) = x2**3*x5**2*x6**2+3*x1**2*x2*x5**2*x6**2+x3**2*x4**3&
&*x6**2+3*x1**2*x3**2*x4*x6**2
dpdx1(410) = x2**3*x5*x6**3+x3*x4**3*x6**3+3*x1**2*x2*x5**3*x6+3*&
&x1**2*x3**3*x4*x6
dpdx1(411) = 3*x1**2*x2*x4*x6**3+x2**3*x4*x5**3+x2*x3**3*x4**3
dpdx1(412) = 3*x1**2*x4*x5*x6**3+3*x1**2*x2*x3*x6**3+x2**3*x3*x5*&
&*3+x3**3*x4**3*x5
dpdx1(413) = x4**3*x5*x6**3+x2**3*x3*x6**3+3*x1**2*x2*x3*x5**3+3*&
&x1**2*x3**3*x4*x5
dpdx1(414) = 3*x1**2*x4*x5**2*x6**2+3*x1**2*x2*x3**2*x6**2+x3**2*&
&x4**3*x5**2+x2**3*x3**2*x5**2
dpdx1(415) = x4**3*x5**2*x6**2+x2**3*x3**2*x6**2+3*x1**2*x3**2*x4&
&*x5**2+3*x1**2*x2*x3**2*x5**2
dpdx1(416) = 3*x1**2*x4*x5**3*x6+3*x1**2*x2*x3**3*x6+x3*x4**3*x5*&
&*3+x2**3*x3**3*x5
dpdx1(417) = x4**3*x5**3*x6+x2**3*x3**3*x6+3*x1**2*x3*x4*x5**3+3*&
&x1**2*x2*x3**3*x5
dpdx1(418) = x2*x4**3*x6**3+x2**3*x4*x6**3+x2*x4**3*x5**3+3*x1**2&
&*x2*x4*x5**3+x2**3*x3**3*x4+3*x1**2*x2*x3**3*x4
dpdx1(419) = 3*x1**2*x2**2*x5*x6**2+3*x1**2*x3*x4**2*x6**2+2*x1*x&
&2**3*x5**2*x6+2*x1*x3**2*x4**3*x6
dpdx1(420) = 2*x1*x2**3*x5*x6**2+2*x1*x3*x4**3*x6**2+3*x1**2*x2**&
&2*x5**2*x6+3*x1**2*x3**2*x4**2*x6
dpdx1(421) = 3*x1**2*x2*x4**2*x6**2+3*x1**2*x2**2*x4*x6**2+x2**3*&
&x4**2*x5**2+2*x1*x2**3*x4*x5**2+x2**2*x3**2*x4**3+2*x1*x2*x3**2*x&
&4**3
dpdx1(422) = 2*x1*x2*x4**3*x6**2+2*x1*x2**3*x4*x6**2+x2**2*x4**3*&
&x5**2+3*x1**2*x2**2*x4*x5**2+x2**3*x3**2*x4**2+3*x1**2*x2*x3**2*x&
&4**2
dpdx1(423) = 3*x1**2*x2**2*x4**2*x6+2*x1*x2**3*x4**2*x5+2*x1*x2**&
&2*x3*x4**3
dpdx1(424) = 3*x1**2*x4**2*x5*x6**2+3*x1**2*x2**2*x3*x6**2+2*x1*x&
&2**3*x3*x5**2+2*x1*x3**2*x4**3*x5
dpdx1(425) = 2*x1*x4**3*x5*x6**2+2*x1*x2**3*x3*x6**2+3*x1**2*x2**&
&2*x3*x5**2+3*x1**2*x3**2*x4**2*x5
dpdx1(426) = 2*x1*x2**2*x4**3*x6+2*x1*x2**3*x4**2*x6+2*x1*x2**2*x&
&4**3*x5+3*x1**2*x2**2*x4**2*x5+2*x1*x2**3*x3*x4**2+3*x1**2*x2**2*&
&x3*x4**2
dpdx1(427) = 3*x1**2*x4**2*x5**2*x6+3*x1**2*x2**2*x3**2*x6+2*x1*x&
&3*x4**3*x5**2+2*x1*x2**3*x3**2*x5
dpdx1(428) = 2*x1*x4**3*x5**2*x6+2*x1*x2**3*x3**2*x6+3*x1**2*x3*x&
&4**2*x5**2+3*x1**2*x2**2*x3**2*x5
dpdx1(429) = x2**2*x4**3*x6**2+x2**3*x4**2*x6**2+2*x1*x2*x4**3*x5&
&**2+3*x1**2*x2*x4**2*x5**2+2*x1*x2**3*x3**2*x4+3*x1**2*x2**2*x3**&
&2*x4
dpdx1(430) = 3*x1**2*x2**3*x5*x6+3*x1**2*x3*x4**3*x6
dpdx1(431) = 3*x1**2*x2*x4**3*x6+3*x1**2*x2**3*x4*x6+x2**3*x4**3*&
&x5+3*x1**2*x2**3*x4*x5+x2**3*x3*x4**3+3*x1**2*x2*x3*x4**3
dpdx1(432) = 3*x1**2*x4**3*x5*x6+3*x1**2*x2**3*x3*x6+3*x1**2*x3*x&
&4**3*x5+3*x1**2*x2**3*x3*x5
dpdx1(433) = x2**3*x4**3*x6+3*x1**2*x2*x4**3*x5+3*x1**2*x2**3*x3*&
&x4
dpdx1(434) = 4*x1**3*x3*x5*x6**2
dpdx1(435) = 4*x1**3*x3*x5**2*x6+4*x1**3*x3**2*x5*x6
dpdx1(436) = 4*x1**3*x2*x5*x6**2+4*x1**3*x3*x4*x6**2+x2**4*x5**2*&
&x6+x3**2*x4**4*x6
dpdx1(437) = x2**4*x5*x6**2+x3*x4**4*x6**2+4*x1**3*x2*x5**2*x6+4*&
&x1**3*x3**2*x4*x6
dpdx1(438) = 4*x1**3*x2*x4*x6**2+x2**4*x4*x5**2+x2*x3**2*x4**4
dpdx1(439) = 4*x1**3*x4*x5*x6**2+4*x1**3*x2*x3*x6**2+x2**4*x3*x5*&
&*2+x3**2*x4**4*x5
dpdx1(440) = x4**4*x5*x6**2+x2**4*x3*x6**2+4*x1**3*x2*x3*x5**2+4*&
&x1**3*x3**2*x4*x5
dpdx1(441) = 4*x1**3*x4*x5**2*x6+4*x1**3*x2*x3**2*x6+x3*x4**4*x5*&
&*2+x2**4*x3**2*x5
dpdx1(442) = x4**4*x5**2*x6+x2**4*x3**2*x6+4*x1**3*x3*x4*x5**2+4*&
&x1**3*x2*x3**2*x5
dpdx1(443) = x2*x4**4*x6**2+x2**4*x4*x6**2+x2*x4**4*x5**2+4*x1**3&
&*x2*x4*x5**2+x2**4*x3**2*x4+4*x1**3*x2*x3**2*x4
dpdx1(444) = 2*x1*x2**4*x5*x6+4*x1**3*x2**2*x5*x6+2*x1*x3*x4**4*x&
&6+4*x1**3*x3*x4**2*x6
dpdx1(445) = 4*x1**3*x2*x4**2*x6+4*x1**3*x2**2*x4*x6+x2**4*x4**2*&
&x5+2*x1*x2**4*x4*x5+x2**2*x3*x4**4+2*x1*x2*x3*x4**4
dpdx1(446) = 2*x1*x2*x4**4*x6+2*x1*x2**4*x4*x6+x2**2*x4**4*x5+4*x&
&1**3*x2**2*x4*x5+x2**4*x3*x4**2+4*x1**3*x2*x3*x4**2
dpdx1(447) = 4*x1**3*x4**2*x5*x6+4*x1**3*x2**2*x3*x6+2*x1*x3*x4**&
&4*x5+2*x1*x2**4*x3*x5
dpdx1(448) = 2*x1*x4**4*x5*x6+2*x1*x2**4*x3*x6+4*x1**3*x3*x4**2*x&
&5+4*x1**3*x2**2*x3*x5
dpdx1(449) = x2**2*x4**4*x6+x2**4*x4**2*x6+2*x1*x2*x4**4*x5+4*x1*&
&*3*x2*x4**2*x5+2*x1*x2**4*x3*x4+4*x1**3*x2**2*x3*x4
dpdx1(450) = 5*x1**4*x3*x5*x6
dpdx1(451) = x2**5*x5*x6+5*x1**4*x2*x5*x6+x3*x4**5*x6+5*x1**4*x3*&
&x4*x6
dpdx1(452) = 5*x1**4*x2*x4*x6+x2**5*x4*x5+x2*x3*x4**5
dpdx1(453) = 5*x1**4*x4*x5*x6+5*x1**4*x2*x3*x6+x3*x4**5*x5+x2**5*&
&x3*x5
dpdx1(454) = x4**5*x5*x6+x2**5*x3*x6+5*x1**4*x3*x4*x5+5*x1**4*x2*&
&x3*x5
dpdx1(455) = x2*x4**5*x6+x2**5*x4*x6+x2*x4**5*x5+5*x1**4*x2*x4*x5&
&+x2**5*x3*x4+5*x1**4*x2*x3*x4
dpdx1(456) = 0
dpdx1(457) = 0
dpdx1(458) = 0
dpdx1(459) = 0
dpdx1(460) = 0
dpdx1(461) = x5*x6**6+x3*x6**6
dpdx1(462) = x5**2*x6**5+x3**2*x6**5
dpdx1(463) = x5**3*x6**4+x3**3*x6**4
dpdx1(464) = x5**4*x6**3+x3**4*x6**3
dpdx1(465) = x3**3*x5**4+x3**4*x5**3
dpdx1(466) = x5**5*x6**2+x3**5*x6**2
dpdx1(467) = x3**2*x5**5+x3**5*x5**2
dpdx1(468) = x5**6*x6+x3**6*x6
dpdx1(469) = x3*x5**6+x3**6*x5
dpdx1(470) = x4*x6**6+x2*x6**6+x2*x5**6+x3**6*x4
dpdx1(471) = x4*x5**6+x2*x3**6
dpdx1(472) = 2*x1*x5*x6**5+2*x1*x3*x6**5
dpdx1(473) = 2*x1*x5**2*x6**4+2*x1*x3**2*x6**4
dpdx1(474) = 2*x1*x5**3*x6**3+2*x1*x3**3*x6**3
dpdx1(475) = 2*x1*x3**3*x5**3
dpdx1(476) = 2*x1*x5**4*x6**2+2*x1*x3**4*x6**2
dpdx1(477) = 2*x1*x3**2*x5**4+2*x1*x3**4*x5**2
dpdx1(478) = 2*x1*x5**5*x6+2*x1*x3**5*x6
dpdx1(479) = 2*x1*x3*x5**5+2*x1*x3**5*x5
dpdx1(480) = 2*x1*x4*x6**5+2*x1*x2*x6**5+x2**2*x5**5+x3**5*x4**2
dpdx1(481) = x4**2*x6**5+x2**2*x6**5+2*x1*x2*x5**5+2*x1*x3**5*x4
dpdx1(482) = x4**2*x5**5+2*x1*x4*x5**5+x2**2*x3**5+2*x1*x2*x3**5
dpdx1(483) = 2*x1*x4**2*x6**4+2*x1*x2**2*x6**4+2*x1*x2**2*x5**4+2&
&*x1*x3**4*x4**2
dpdx1(484) = 2*x1*x4**2*x5**4+2*x1*x2**2*x3**4
dpdx1(485) = 3*x1**2*x5*x6**4+3*x1**2*x3*x6**4
dpdx1(486) = 3*x1**2*x5**2*x6**3+3*x1**2*x3**2*x6**3
dpdx1(487) = 3*x1**2*x5**3*x6**2+3*x1**2*x3**3*x6**2
dpdx1(488) = 3*x1**2*x3**2*x5**3+3*x1**2*x3**3*x5**2
dpdx1(489) = 3*x1**2*x5**4*x6+3*x1**2*x3**4*x6
dpdx1(490) = 3*x1**2*x3*x5**4+3*x1**2*x3**4*x5
dpdx1(491) = 3*x1**2*x4*x6**4+3*x1**2*x2*x6**4+x2**3*x5**4+x3**4*&
&x4**3
dpdx1(492) = x4**3*x6**4+x2**3*x6**4+3*x1**2*x2*x5**4+3*x1**2*x3*&
&*4*x4
dpdx1(493) = x4**3*x5**4+3*x1**2*x4*x5**4+x2**3*x3**4+3*x1**2*x2*&
&x3**4
dpdx1(494) = 3*x1**2*x4**2*x6**3+3*x1**2*x2**2*x6**3+2*x1*x2**3*x&
&5**3+2*x1*x3**3*x4**3
dpdx1(495) = 2*x1*x4**3*x6**3+2*x1*x2**3*x6**3+3*x1**2*x2**2*x5**&
&3+3*x1**2*x3**3*x4**2
dpdx1(496) = 2*x1*x4**3*x5**3+3*x1**2*x4**2*x5**3+2*x1*x2**3*x3**&
&3+3*x1**2*x2**2*x3**3
dpdx1(497) = 3*x1**2*x4**3*x6**2+3*x1**2*x2**3*x6**2+3*x1**2*x2**&
&3*x5**2+3*x1**2*x3**2*x4**3
dpdx1(498) = 2*x1*x2**3*x4**3+3*x1**2*x2**2*x4**3+3*x1**2*x2**3*x&
&4**2
dpdx1(499) = 3*x1**2*x4**3*x5**2+3*x1**2*x2**3*x3**2
dpdx1(500) = 4*x1**3*x5*x6**3+4*x1**3*x3*x6**3
dpdx1(501) = 4*x1**3*x5**2*x6**2+4*x1**3*x3**2*x6**2
dpdx1(502) = 4*x1**3*x3**2*x5**2
dpdx1(503) = 4*x1**3*x5**3*x6+4*x1**3*x3**3*x6
dpdx1(504) = 4*x1**3*x3*x5**3+4*x1**3*x3**3*x5
dpdx1(505) = 4*x1**3*x4*x6**3+4*x1**3*x2*x6**3+x2**4*x5**3+x3**3*&
&x4**4
dpdx1(506) = x4**4*x6**3+x2**4*x6**3+4*x1**3*x2*x5**3+4*x1**3*x3*&
&*3*x4
dpdx1(507) = x4**4*x5**3+4*x1**3*x4*x5**3+x2**4*x3**3+4*x1**3*x2*&
&x3**3
dpdx1(508) = 4*x1**3*x4**2*x6**2+4*x1**3*x2**2*x6**2+2*x1*x2**4*x&
&5**2+2*x1*x3**2*x4**4
dpdx1(509) = 2*x1*x4**4*x6**2+2*x1*x2**4*x6**2+4*x1**3*x2**2*x5**&
&2+4*x1**3*x3**2*x4**2
dpdx1(510) = 2*x1*x2**2*x4**4+2*x1*x2**4*x4**2+4*x1**3*x2**2*x4**&
&2
dpdx1(511) = 2*x1*x4**4*x5**2+4*x1**3*x4**2*x5**2+2*x1*x2**4*x3**&
&2+4*x1**3*x2**2*x3**2
dpdx1(512) = 4*x1**3*x4**3*x6+4*x1**3*x2**3*x6+3*x1**2*x2**4*x5+3&
&*x1**2*x3*x4**4
dpdx1(513) = 3*x1**2*x4**4*x6+3*x1**2*x2**4*x6+4*x1**3*x2**3*x5+4&
&*x1**3*x3*x4**3
dpdx1(514) = x2**3*x4**4+3*x1**2*x2*x4**4+x2**4*x4**3+4*x1**3*x2*&
&x4**3+3*x1**2*x2**4*x4+4*x1**3*x2**3*x4
dpdx1(515) = 3*x1**2*x4**4*x5+4*x1**3*x4**3*x5+3*x1**2*x2**4*x3+4&
&*x1**3*x2**3*x3
dpdx1(516) = 5*x1**4*x5*x6**2+5*x1**4*x3*x6**2
dpdx1(517) = 5*x1**4*x5**2*x6+5*x1**4*x3**2*x6
dpdx1(518) = 5*x1**4*x3*x5**2+5*x1**4*x3**2*x5
dpdx1(519) = 5*x1**4*x4*x6**2+5*x1**4*x2*x6**2+x2**5*x5**2+x3**2*&
&x4**5
dpdx1(520) = x4**5*x6**2+x2**5*x6**2+5*x1**4*x2*x5**2+5*x1**4*x3*&
&*2*x4
dpdx1(521) = x4**5*x5**2+5*x1**4*x4*x5**2+x2**5*x3**2+5*x1**4*x2*&
&x3**2
dpdx1(522) = 5*x1**4*x4**2*x6+5*x1**4*x2**2*x6+2*x1*x2**5*x5+2*x1&
&*x3*x4**5
dpdx1(523) = 2*x1*x4**5*x6+2*x1*x2**5*x6+5*x1**4*x2**2*x5+5*x1**4&
&*x3*x4**2
dpdx1(524) = x2**2*x4**5+2*x1*x2*x4**5+x2**5*x4**2+5*x1**4*x2*x4*&
&*2+2*x1*x2**5*x4+5*x1**4*x2**2*x4
dpdx1(525) = 2*x1*x4**5*x5+5*x1**4*x4**2*x5+2*x1*x2**5*x3+5*x1**4&
&*x2**2*x3
dpdx1(526) = 6*x1**5*x5*x6+6*x1**5*x3*x6
dpdx1(527) = 6*x1**5*x3*x5
dpdx1(528) = 6*x1**5*x4*x6+6*x1**5*x2*x6+x2**6*x5+x3*x4**6
dpdx1(529) = x4**6*x6+x2**6*x6+6*x1**5*x2*x5+6*x1**5*x3*x4
dpdx1(530) = x2*x4**6+x2**6*x4+6*x1**5*x2*x4
dpdx1(531) = x4**6*x5+6*x1**5*x4*x5+x2**6*x3+6*x1**5*x2*x3
dpdx1(532) = 0
dpdx1(533) = 0
dpdx1(534) = 0
dpdx1(535) = 0
dpdx1(536) = x5**7+x3**7
dpdx1(537) = 2*x1*x5**6+2*x1*x3**6
dpdx1(538) = 3*x1**2*x5**5+3*x1**2*x3**5
dpdx1(539) = 4*x1**3*x5**4+4*x1**3*x3**4
dpdx1(540) = 4*x1**3*x4**4+4*x1**3*x2**4
dpdx1(541) = 5*x1**4*x5**3+5*x1**4*x3**3
dpdx1(542) = 3*x1**2*x4**5+5*x1**4*x4**3+3*x1**2*x2**5+5*x1**4*x2&
&**3
dpdx1(543) = 6*x1**5*x5**2+6*x1**5*x3**2
dpdx1(544) = 2*x1*x4**6+6*x1**5*x4**2+2*x1*x2**6+6*x1**5*x2**2
dpdx1(545) = 7*x1**6*x5+7*x1**6*x3
dpdx1(546) = x4**7+7*x1**6*x4+x2**7+7*x1**6*x2
dpdx2(1) = 0
dpdx2(2) = 0
dpdx2(3) = x6+x3
dpdx2(4) = x4+x1
dpdx2(5) = 0
dpdx2(6) = x5*x6+x3*x5
dpdx2(7) = x3*x6
dpdx2(8) = x1*x6+x4*x5+x1*x5+x3*x4
dpdx2(9) = x1*x4
dpdx2(10) = x4*x6+x1*x3
dpdx2(11) = 0
dpdx2(12) = x6**2+x3**2
dpdx2(13) = 2*x2*x6+2*x2*x3
dpdx2(14) = x4**2+2*x2*x4+2*x1*x2+x1**2
dpdx2(15) = x3*x5*x6
dpdx2(16) = x1*x5*x6+x3*x4*x5
dpdx2(17) = x4*x5*x6+x3*x4*x6+x1*x3*x6+x1*x3*x5
dpdx2(18) = x1*x4*x6+x1*x4*x5+x1*x3*x4
dpdx2(19) = 0
dpdx2(20) = x5**2*x6+x3*x5**2
dpdx2(21) = x5*x6**2+x3**2*x5
dpdx2(22) = x3*x6**2+x3**2*x6
dpdx2(23) = x1*x6**2+x4*x5**2+x1*x5**2+x3**2*x4
dpdx2(24) = x4*x6**2+x1*x3**2
dpdx2(25) = 2*x2*x5*x6+2*x2*x3*x5
dpdx2(26) = 2*x2*x3*x6
dpdx2(27) = x1**2*x6+2*x2*x4*x5+2*x1*x2*x5+x3*x4**2
dpdx2(28) = 2*x1*x2*x6+x4**2*x5+x1**2*x5+2*x2*x3*x4
dpdx2(29) = x1*x4**2+2*x1*x2*x4+x1**2*x4
dpdx2(30) = x4**2*x6+2*x2*x4*x6+2*x1*x2*x3+x1**2*x3
dpdx2(31) = 0
dpdx2(32) = 0
dpdx2(33) = x6**3+x3**3
dpdx2(34) = 2*x2*x6**2+2*x2*x3**2
dpdx2(35) = 2*x2*x4**2+2*x1**2*x2
dpdx2(36) = 3*x2**2*x6+3*x2**2*x3
dpdx2(37) = x4**3+3*x2**2*x4+3*x1*x2**2+x1**3
dpdx2(38) = x3*x4*x5*x6+x1*x3*x5*x6
dpdx2(39) = x1*x4*x5*x6+x1*x3*x4*x6+x1*x3*x4*x5
dpdx2(40) = x3*x5**2*x6
dpdx2(41) = x3*x5*x6**2+x3**2*x5*x6
dpdx2(42) = x1*x5*x6**2+x1*x5**2*x6+x3*x4*x5**2+x3**2*x4*x5
dpdx2(43) = x1*x3*x6**2+x4*x5**2*x6+x3**2*x4*x6+x1*x3*x5**2
dpdx2(44) = x4*x5*x6**2+x3*x4*x6**2+x1*x3**2*x6+x1*x3**2*x5
dpdx2(45) = x1*x4*x6**2+x1*x4*x5**2+x1*x3**2*x4
dpdx2(46) = 2*x2*x3*x5*x6
dpdx2(47) = 2*x1*x2*x5*x6+x1**2*x5*x6+x3*x4**2*x5+2*x2*x3*x4*x5
dpdx2(48) = x1**2*x4*x6+2*x1*x2*x4*x5+x1*x3*x4**2
dpdx2(49) = 2*x2*x4*x5*x6+x3*x4**2*x6+x1**2*x3*x6+2*x1*x2*x3*x5
dpdx2(50) = x4**2*x5*x6+2*x2*x3*x4*x6+2*x1*x2*x3*x6+x1**2*x3*x5
dpdx2(51) = x1*x4**2*x6+2*x1*x2*x4*x6+x1*x4**2*x5+x1**2*x4*x5+2*x&
&1*x2*x3*x4+x1**2*x3*x4
dpdx2(52) = 0
dpdx2(53) = 0
dpdx2(54) = x5**3*x6+x3*x5**3
dpdx2(55) = x5**2*x6**2+x3**2*x5**2
dpdx2(56) = x3**2*x6**2
dpdx2(57) = x5*x6**3+x3**3*x5
dpdx2(58) = x3*x6**3+x3**3*x6
dpdx2(59) = x1*x6**3+x4*x5**3+x1*x5**3+x3**3*x4
dpdx2(60) = x4*x6**3+x1*x3**3
dpdx2(61) = 2*x2*x5**2*x6+2*x2*x3*x5**2
dpdx2(62) = 2*x2*x5*x6**2+2*x2*x3**2*x5
dpdx2(63) = 2*x2*x3*x6**2+2*x2*x3**2*x6
dpdx2(64) = x1**2*x6**2+2*x2*x4*x5**2+2*x1*x2*x5**2+x3**2*x4**2
dpdx2(65) = 2*x1*x2*x6**2+x4**2*x5**2+x1**2*x5**2+2*x2*x3**2*x4
dpdx2(66) = x4**2*x6**2+2*x2*x4*x6**2+2*x1*x2*x3**2+x1**2*x3**2
dpdx2(67) = 2*x1**2*x2*x6+2*x2*x4**2*x5+2*x1**2*x2*x5+2*x2*x3*x4*&
&*2
dpdx2(68) = 2*x1*x2*x4**2+x1**2*x4**2+2*x1**2*x2*x4
dpdx2(69) = 2*x2*x4**2*x6+2*x1**2*x2*x3
dpdx2(70) = 3*x2**2*x5*x6+3*x2**2*x3*x5
dpdx2(71) = 3*x2**2*x3*x6
dpdx2(72) = x1**3*x6+3*x2**2*x4*x5+3*x1*x2**2*x5+x3*x4**3
dpdx2(73) = 3*x1*x2**2*x6+x4**3*x5+x1**3*x5+3*x2**2*x3*x4
dpdx2(74) = x1*x4**3+3*x1*x2**2*x4+x1**3*x4
dpdx2(75) = x4**3*x6+3*x2**2*x4*x6+3*x1*x2**2*x3+x1**3*x3
dpdx2(76) = 0
dpdx2(77) = 0
dpdx2(78) = x6**4+x3**4
dpdx2(79) = 2*x2*x6**3+2*x2*x3**3
dpdx2(80) = 3*x2**2*x6**2+3*x2**2*x3**2
dpdx2(81) = 2*x2*x4**3+3*x2**2*x4**2+3*x1**2*x2**2+2*x1**3*x2
dpdx2(82) = 4*x2**3*x6+4*x2**3*x3
dpdx2(83) = x4**4+4*x2**3*x4+4*x1*x2**3+x1**4
dpdx2(84) = x1*x3*x4*x5*x6
dpdx2(85) = x1*x3*x5*x6**2+x3*x4*x5**2*x6+x1*x3*x5**2*x6+x3**2*x4&
&*x5*x6
dpdx2(86) = x3*x4*x5*x6**2+x1*x3**2*x5*x6
dpdx2(87) = x1*x4*x5*x6**2+x1*x3*x4*x6**2+x1*x4*x5**2*x6+x1*x3**2&
&*x4*x6+x1*x3*x4*x5**2+x1*x3**2*x4*x5
dpdx2(88) = x3*x4**2*x5*x6+2*x2*x3*x4*x5*x6+2*x1*x2*x3*x5*x6+x1**&
&2*x3*x5*x6
dpdx2(89) = 2*x1*x2*x4*x5*x6+x1**2*x4*x5*x6+x1*x3*x4**2*x6+x1**2*&
&x3*x4*x6+x1*x3*x4**2*x5+2*x1*x2*x3*x4*x5
dpdx2(90) = x1*x4**2*x5*x6+2*x1*x2*x3*x4*x6+x1**2*x3*x4*x5
dpdx2(91) = x3*x5**3*x6
dpdx2(92) = x3*x5**2*x6**2+x3**2*x5**2*x6
dpdx2(93) = x3**2*x5*x6**2
dpdx2(94) = x3*x5*x6**3+x3**3*x5*x6
dpdx2(95) = x1*x5**2*x6**2+x3**2*x4*x5**2
dpdx2(96) = x1*x5*x6**3+x1*x5**3*x6+x3*x4*x5**3+x3**3*x4*x5
dpdx2(97) = x1*x3*x6**3+x4*x5**3*x6+x3**3*x4*x6+x1*x3*x5**3
dpdx2(98) = x4*x5**2*x6**2+x3**2*x4*x6**2+x1*x3**2*x6**2+x1*x3**2&
&*x5**2
dpdx2(99) = x4*x5*x6**3+x3*x4*x6**3+x1*x3**3*x6+x1*x3**3*x5
dpdx2(100) = x1*x4*x6**3+x1*x4*x5**3+x1*x3**3*x4
dpdx2(101) = 2*x2*x3*x5**2*x6
dpdx2(102) = 2*x2*x3*x5*x6**2+2*x2*x3**2*x5*x6
dpdx2(103) = x1**2*x5*x6**2+2*x1*x2*x5**2*x6+2*x2*x3*x4*x5**2+x3*&
&*2*x4**2*x5
dpdx2(104) = 2*x1*x2*x5*x6**2+x1**2*x5**2*x6+x3*x4**2*x5**2+2*x2*&
&x3**2*x4*x5
dpdx2(105) = x1**2*x4*x6**2+2*x1*x2*x4*x5**2+x1*x3**2*x4**2
dpdx2(106) = x1**2*x3*x6**2+2*x2*x4*x5**2*x6+x3**2*x4**2*x6+2*x1*&
&x2*x3*x5**2
dpdx2(107) = 2*x1*x2*x3*x6**2+x4**2*x5**2*x6+2*x2*x3**2*x4*x6+x1*&
&*2*x3*x5**2
dpdx2(108) = 2*x2*x4*x5*x6**2+x3*x4**2*x6**2+x1**2*x3**2*x6+2*x1*&
&x2*x3**2*x5
dpdx2(109) = x4**2*x5*x6**2+2*x2*x3*x4*x6**2+2*x1*x2*x3**2*x6+x1*&
&*2*x3**2*x5
dpdx2(110) = x1*x4**2*x6**2+2*x1*x2*x4*x6**2+x1*x4**2*x5**2+x1**2&
&*x4*x5**2+2*x1*x2*x3**2*x4+x1**2*x3**2*x4
dpdx2(111) = 2*x1**2*x2*x5*x6+2*x2*x3*x4**2*x5
dpdx2(112) = x1**2*x4**2*x6+2*x1**2*x2*x4*x6+2*x1*x2*x4**2*x5+2*x&
&1**2*x2*x4*x5+2*x1*x2*x3*x4**2+x1**2*x3*x4**2
dpdx2(113) = 2*x2*x4**2*x5*x6+2*x2*x3*x4**2*x6+2*x1**2*x2*x3*x6+2&
&*x1**2*x2*x3*x5
dpdx2(114) = 2*x1*x2*x4**2*x6+x1**2*x4**2*x5+2*x1**2*x2*x3*x4
dpdx2(115) = 3*x2**2*x3*x5*x6
dpdx2(116) = 3*x1*x2**2*x5*x6+x1**3*x5*x6+x3*x4**3*x5+3*x2**2*x3*&
&x4*x5
dpdx2(117) = x1**3*x4*x6+3*x1*x2**2*x4*x5+x1*x3*x4**3
dpdx2(118) = 3*x2**2*x4*x5*x6+x3*x4**3*x6+x1**3*x3*x6+3*x1*x2**2*&
&x3*x5
dpdx2(119) = x4**3*x5*x6+3*x2**2*x3*x4*x6+3*x1*x2**2*x3*x6+x1**3*&
&x3*x5
dpdx2(120) = x1*x4**3*x6+3*x1*x2**2*x4*x6+x1*x4**3*x5+x1**3*x4*x5&
&+3*x1*x2**2*x3*x4+x1**3*x3*x4
dpdx2(121) = 0
dpdx2(122) = 0
dpdx2(123) = 0
dpdx2(124) = x5**4*x6+x3*x5**4
dpdx2(125) = x5**3*x6**2+x3**2*x5**3
dpdx2(126) = x5**2*x6**3+x3**3*x5**2
dpdx2(127) = x3**2*x6**3+x3**3*x6**2
dpdx2(128) = x5*x6**4+x3**4*x5
dpdx2(129) = x3*x6**4+x3**4*x6
dpdx2(130) = x1*x6**4+x4*x5**4+x1*x5**4+x3**4*x4
dpdx2(131) = x4*x6**4+x1*x3**4
dpdx2(132) = 2*x2*x5**3*x6+2*x2*x3*x5**3
dpdx2(133) = 2*x2*x5**2*x6**2+2*x2*x3**2*x5**2
dpdx2(134) = 2*x2*x3**2*x6**2
dpdx2(135) = 2*x2*x5*x6**3+2*x2*x3**3*x5
dpdx2(136) = 2*x2*x3*x6**3+2*x2*x3**3*x6
dpdx2(137) = x1**2*x6**3+2*x2*x4*x5**3+2*x1*x2*x5**3+x3**3*x4**2
dpdx2(138) = 2*x1*x2*x6**3+x4**2*x5**3+x1**2*x5**3+2*x2*x3**3*x4
dpdx2(139) = x4**2*x6**3+2*x2*x4*x6**3+2*x1*x2*x3**3+x1**2*x3**3
dpdx2(140) = 2*x1**2*x2*x6**2+2*x2*x4**2*x5**2+2*x1**2*x2*x5**2+2&
&*x2*x3**2*x4**2
dpdx2(141) = 2*x1**2*x2*x4**2
dpdx2(142) = 2*x2*x4**2*x6**2+2*x1**2*x2*x3**2
dpdx2(143) = 3*x2**2*x5**2*x6+3*x2**2*x3*x5**2
dpdx2(144) = 3*x2**2*x5*x6**2+3*x2**2*x3**2*x5
dpdx2(145) = 3*x2**2*x3*x6**2+3*x2**2*x3**2*x6
dpdx2(146) = x1**3*x6**2+3*x2**2*x4*x5**2+3*x1*x2**2*x5**2+x3**2*&
&x4**3
dpdx2(147) = 3*x1*x2**2*x6**2+x4**3*x5**2+x1**3*x5**2+3*x2**2*x3*&
&*2*x4
dpdx2(148) = x4**3*x6**2+3*x2**2*x4*x6**2+3*x1*x2**2*x3**2+x1**3*&
&x3**2
dpdx2(149) = 2*x1**3*x2*x6+3*x2**2*x4**2*x5+3*x1**2*x2**2*x5+2*x2&
&*x3*x4**3
dpdx2(150) = 3*x1**2*x2**2*x6+2*x2*x4**3*x5+2*x1**3*x2*x5+3*x2**2&
&*x3*x4**2
dpdx2(151) = 2*x1*x2*x4**3+x1**2*x4**3+3*x1*x2**2*x4**2+x1**3*x4*&
&*2+3*x1**2*x2**2*x4+2*x1**3*x2*x4
dpdx2(152) = 2*x2*x4**3*x6+3*x2**2*x4**2*x6+3*x1**2*x2**2*x3+2*x1&
&**3*x2*x3
dpdx2(153) = 4*x2**3*x5*x6+4*x2**3*x3*x5
dpdx2(154) = 4*x2**3*x3*x6
dpdx2(155) = x1**4*x6+4*x2**3*x4*x5+4*x1*x2**3*x5+x3*x4**4
dpdx2(156) = 4*x1*x2**3*x6+x4**4*x5+x1**4*x5+4*x2**3*x3*x4
dpdx2(157) = x1*x4**4+4*x1*x2**3*x4+x1**4*x4
dpdx2(158) = x4**4*x6+4*x2**3*x4*x6+4*x1*x2**3*x3+x1**4*x3
dpdx2(159) = 0
dpdx2(160) = 0
dpdx2(161) = 0
dpdx2(162) = x6**5+x3**5
dpdx2(163) = 2*x2*x6**4+2*x2*x3**4
dpdx2(164) = 3*x2**2*x6**3+3*x2**2*x3**3
dpdx2(165) = 3*x2**2*x4**3+3*x1**3*x2**2
dpdx2(166) = 4*x2**3*x6**2+4*x2**3*x3**2
dpdx2(167) = 2*x2*x4**4+4*x2**3*x4**2+4*x1**2*x2**3+2*x1**4*x2
dpdx2(168) = 5*x2**4*x6+5*x2**4*x3
dpdx2(169) = x4**5+5*x2**4*x4+5*x1*x2**4+x1**5
dpdx2(170) = x1*x3*x4*x5*x6**2+x1*x3*x4*x5**2*x6+x1*x3**2*x4*x5*x&
&6
dpdx2(171) = x1*x3*x4**2*x5*x6+2*x1*x2*x3*x4*x5*x6+x1**2*x3*x4*x5&
&*x6
dpdx2(172) = x1*x3*x5**2*x6**2+x3**2*x4*x5**2*x6
dpdx2(173) = x1*x3*x5*x6**3+x3*x4*x5**3*x6+x1*x3*x5**3*x6+x3**3*x&
&4*x5*x6
dpdx2(174) = x3*x4*x5**2*x6**2+x3**2*x4*x5*x6**2+x1*x3**2*x5*x6**&
&2+x1*x3**2*x5**2*x6
dpdx2(175) = x1*x4*x5**2*x6**2+x1*x3**2*x4*x6**2+x1*x3**2*x4*x5**&
&2
dpdx2(176) = x3*x4*x5*x6**3+x1*x3**3*x5*x6
dpdx2(177) = x1*x4*x5*x6**3+x1*x3*x4*x6**3+x1*x4*x5**3*x6+x1*x3**&
&3*x4*x6+x1*x3*x4*x5**3+x1*x3**3*x4*x5
dpdx2(178) = x1**2*x3*x5*x6**2+2*x2*x3*x4*x5**2*x6+2*x1*x2*x3*x5*&
&*2*x6+x3**2*x4**2*x5*x6
dpdx2(179) = 2*x1*x2*x3*x5*x6**2+x3*x4**2*x5**2*x6+x1**2*x3*x5**2&
&*x6+2*x2*x3**2*x4*x5*x6
dpdx2(180) = x1**2*x4*x5*x6**2+x1**2*x3*x4*x6**2+2*x1*x2*x4*x5**2&
&*x6+x1*x3**2*x4**2*x6+2*x1*x2*x3*x4*x5**2+x1*x3**2*x4**2*x5
dpdx2(181) = x3*x4**2*x5*x6**2+2*x2*x3*x4*x5*x6**2+2*x1*x2*x3**2*&
&x5*x6+x1**2*x3**2*x5*x6
dpdx2(182) = 2*x1*x2*x4*x5*x6**2+x1*x3*x4**2*x6**2+x1**2*x4*x5**2&
&*x6+x1**2*x3**2*x4*x6+x1*x3*x4**2*x5**2+2*x1*x2*x3**2*x4*x5
dpdx2(183) = x1*x4**2*x5*x6**2+2*x1*x2*x3*x4*x6**2+x1*x4**2*x5**2&
&*x6+2*x1*x2*x3**2*x4*x6+x1**2*x3*x4*x5**2+x1**2*x3**2*x4*x5
dpdx2(184) = 2*x1**2*x2*x4*x5*x6+x1**2*x3*x4**2*x6+2*x1*x2*x3*x4*&
&*2*x5
dpdx2(185) = 2*x2*x3*x4**2*x5*x6+2*x1**2*x2*x3*x5*x6
dpdx2(186) = 2*x1*x2*x4**2*x5*x6+x1**2*x4**2*x5*x6+2*x1*x2*x3*x4*&
&*2*x6+2*x1**2*x2*x3*x4*x6+x1**2*x3*x4**2*x5+2*x1**2*x2*x3*x4*x5
dpdx2(187) = x3*x4**3*x5*x6+3*x2**2*x3*x4*x5*x6+3*x1*x2**2*x3*x5*&
&x6+x1**3*x3*x5*x6
dpdx2(188) = 3*x1*x2**2*x4*x5*x6+x1**3*x4*x5*x6+x1*x3*x4**3*x6+x1&
&**3*x3*x4*x6+x1*x3*x4**3*x5+3*x1*x2**2*x3*x4*x5
dpdx2(189) = x1*x4**3*x5*x6+3*x1*x2**2*x3*x4*x6+x1**3*x3*x4*x5
dpdx2(190) = x3*x5**4*x6
dpdx2(191) = x3*x5**3*x6**2+x3**2*x5**3*x6
dpdx2(192) = x3**2*x5**2*x6**2
dpdx2(193) = x3*x5**2*x6**3+x3**3*x5**2*x6
dpdx2(194) = x3**2*x5*x6**3+x3**3*x5*x6**2
dpdx2(195) = x3*x5*x6**4+x3**4*x5*x6
dpdx2(196) = x1*x5**2*x6**3+x1*x5**3*x6**2+x3**2*x4*x5**3+x3**3*x&
&4*x5**2
dpdx2(197) = x1*x5*x6**4+x1*x5**4*x6+x3*x4*x5**4+x3**4*x4*x5
dpdx2(198) = x1*x3*x6**4+x4*x5**4*x6+x3**4*x4*x6+x1*x3*x5**4
dpdx2(199) = x1*x3**2*x6**3+x4*x5**3*x6**2+x3**3*x4*x6**2+x1*x3**&
&2*x5**3
dpdx2(200) = x4*x5**2*x6**3+x3**2*x4*x6**3+x1*x3**3*x6**2+x1*x3**&
&3*x5**2
dpdx2(201) = x4*x5*x6**4+x3*x4*x6**4+x1*x3**4*x6+x1*x3**4*x5
dpdx2(202) = x1*x4*x6**4+x1*x4*x5**4+x1*x3**4*x4
dpdx2(203) = 2*x2*x3*x5**3*x6
dpdx2(204) = 2*x2*x3*x5**2*x6**2+2*x2*x3**2*x5**2*x6
dpdx2(205) = 2*x2*x3**2*x5*x6**2
dpdx2(206) = 2*x2*x3*x5*x6**3+2*x2*x3**3*x5*x6
dpdx2(207) = x1**2*x5*x6**3+2*x1*x2*x5**3*x6+2*x2*x3*x4*x5**3+x3*&
&*3*x4**2*x5
dpdx2(208) = 2*x1*x2*x5**2*x6**2+x1**2*x5**2*x6**2+x3**2*x4**2*x5&
&**2+2*x2*x3**2*x4*x5**2
dpdx2(209) = 2*x1*x2*x5*x6**3+x1**2*x5**3*x6+x3*x4**2*x5**3+2*x2*&
&x3**3*x4*x5
dpdx2(210) = x1**2*x4*x6**3+2*x1*x2*x4*x5**3+x1*x3**3*x4**2
dpdx2(211) = x1**2*x3*x6**3+2*x2*x4*x5**3*x6+x3**3*x4**2*x6+2*x1*&
&x2*x3*x5**3
dpdx2(212) = 2*x1*x2*x3*x6**3+x4**2*x5**3*x6+2*x2*x3**3*x4*x6+x1*&
&*2*x3*x5**3
dpdx2(213) = 2*x2*x4*x5**2*x6**2+x3**2*x4**2*x6**2+x1**2*x3**2*x6&
&**2+2*x1*x2*x3**2*x5**2
dpdx2(214) = x4**2*x5**2*x6**2+2*x2*x3**2*x4*x6**2+2*x1*x2*x3**2*&
&x6**2+x1**2*x3**2*x5**2
dpdx2(215) = 2*x2*x4*x5*x6**3+x3*x4**2*x6**3+x1**2*x3**3*x6+2*x1*&
&x2*x3**3*x5
dpdx2(216) = x4**2*x5*x6**3+2*x2*x3*x4*x6**3+2*x1*x2*x3**3*x6+x1*&
&*2*x3**3*x5
dpdx2(217) = x1*x4**2*x6**3+2*x1*x2*x4*x6**3+x1*x4**2*x5**3+x1**2&
&*x4*x5**3+2*x1*x2*x3**3*x4+x1**2*x3**3*x4
dpdx2(218) = 2*x1**2*x2*x5*x6**2+2*x1**2*x2*x5**2*x6+2*x2*x3*x4**&
&2*x5**2+2*x2*x3**2*x4**2*x5
dpdx2(219) = x1**2*x4**2*x6**2+2*x1**2*x2*x4*x6**2+2*x1*x2*x4**2*&
&x5**2+2*x1**2*x2*x4*x5**2+2*x1*x2*x3**2*x4**2+x1**2*x3**2*x4**2
dpdx2(220) = 2*x1**2*x2*x3*x6**2+2*x2*x4**2*x5**2*x6+2*x2*x3**2*x&
&4**2*x6+2*x1**2*x2*x3*x5**2
dpdx2(221) = 2*x1**2*x2*x4**2*x6+2*x1**2*x2*x4**2*x5+2*x1**2*x2*x&
&3*x4**2
dpdx2(222) = 2*x2*x4**2*x5*x6**2+2*x2*x3*x4**2*x6**2+2*x1**2*x2*x&
&3**2*x6+2*x1**2*x2*x3**2*x5
dpdx2(223) = 2*x1*x2*x4**2*x6**2+x1**2*x4**2*x5**2+2*x1**2*x2*x3*&
&*2*x4
dpdx2(224) = 3*x2**2*x3*x5**2*x6
dpdx2(225) = 3*x2**2*x3*x5*x6**2+3*x2**2*x3**2*x5*x6
dpdx2(226) = x1**3*x5*x6**2+3*x1*x2**2*x5**2*x6+3*x2**2*x3*x4*x5*&
&*2+x3**2*x4**3*x5
dpdx2(227) = 3*x1*x2**2*x5*x6**2+x1**3*x5**2*x6+x3*x4**3*x5**2+3*&
&x2**2*x3**2*x4*x5
dpdx2(228) = x1**3*x4*x6**2+3*x1*x2**2*x4*x5**2+x1*x3**2*x4**3
dpdx2(229) = x1**3*x3*x6**2+3*x2**2*x4*x5**2*x6+x3**2*x4**3*x6+3*&
&x1*x2**2*x3*x5**2
dpdx2(230) = 3*x1*x2**2*x3*x6**2+x4**3*x5**2*x6+3*x2**2*x3**2*x4*&
&x6+x1**3*x3*x5**2
dpdx2(231) = 3*x2**2*x4*x5*x6**2+x3*x4**3*x6**2+x1**3*x3**2*x6+3*&
&x1*x2**2*x3**2*x5
dpdx2(232) = x4**3*x5*x6**2+3*x2**2*x3*x4*x6**2+3*x1*x2**2*x3**2*&
&x6+x1**3*x3**2*x5
dpdx2(233) = x1*x4**3*x6**2+3*x1*x2**2*x4*x6**2+x1*x4**3*x5**2+x1&
&**3*x4*x5**2+3*x1*x2**2*x3**2*x4+x1**3*x3**2*x4
dpdx2(234) = 3*x1**2*x2**2*x5*x6+2*x1**3*x2*x5*x6+2*x2*x3*x4**3*x&
&5+3*x2**2*x3*x4**2*x5
dpdx2(235) = x1**3*x4**2*x6+2*x1**3*x2*x4*x6+3*x1*x2**2*x4**2*x5+&
&3*x1**2*x2**2*x4*x5+2*x1*x2*x3*x4**3+x1**2*x3*x4**3
dpdx2(236) = x1**2*x4**3*x6+3*x1**2*x2**2*x4*x6+2*x1*x2*x4**3*x5+&
&2*x1**3*x2*x4*x5+3*x1*x2**2*x3*x4**2+x1**3*x3*x4**2
dpdx2(237) = 3*x2**2*x4**2*x5*x6+2*x2*x3*x4**3*x6+2*x1**3*x2*x3*x&
&6+3*x1**2*x2**2*x3*x5
dpdx2(238) = 2*x2*x4**3*x5*x6+3*x2**2*x3*x4**2*x6+3*x1**2*x2**2*x&
&3*x6+2*x1**3*x2*x3*x5
dpdx2(239) = 2*x1*x2*x4**3*x6+3*x1*x2**2*x4**2*x6+x1**2*x4**3*x5+&
&x1**3*x4**2*x5+3*x1**2*x2**2*x3*x4+2*x1**3*x2*x3*x4
dpdx2(240) = 4*x2**3*x3*x5*x6
dpdx2(241) = 4*x1*x2**3*x5*x6+x1**4*x5*x6+x3*x4**4*x5+4*x2**3*x3*&
&x4*x5
dpdx2(242) = x1**4*x4*x6+4*x1*x2**3*x4*x5+x1*x3*x4**4
dpdx2(243) = 4*x2**3*x4*x5*x6+x3*x4**4*x6+x1**4*x3*x6+4*x1*x2**3*&
&x3*x5
dpdx2(244) = x4**4*x5*x6+4*x2**3*x3*x4*x6+4*x1*x2**3*x3*x6+x1**4*&
&x3*x5
dpdx2(245) = x1*x4**4*x6+4*x1*x2**3*x4*x6+x1*x4**4*x5+x1**4*x4*x5&
&+4*x1*x2**3*x3*x4+x1**4*x3*x4
dpdx2(246) = 0
dpdx2(247) = 0
dpdx2(248) = 0
dpdx2(249) = 0
dpdx2(250) = x5**5*x6+x3*x5**5
dpdx2(251) = x5**4*x6**2+x3**2*x5**4
dpdx2(252) = x5**3*x6**3+x3**3*x5**3
dpdx2(253) = x3**3*x6**3
dpdx2(254) = x5**2*x6**4+x3**4*x5**2
dpdx2(255) = x3**2*x6**4+x3**4*x6**2
dpdx2(256) = x5*x6**5+x3**5*x5
dpdx2(257) = x3*x6**5+x3**5*x6
dpdx2(258) = x1*x6**5+x4*x5**5+x1*x5**5+x3**5*x4
dpdx2(259) = x4*x6**5+x1*x3**5
dpdx2(260) = 2*x2*x5**4*x6+2*x2*x3*x5**4
dpdx2(261) = 2*x2*x5**3*x6**2+2*x2*x3**2*x5**3
dpdx2(262) = 2*x2*x5**2*x6**3+2*x2*x3**3*x5**2
dpdx2(263) = 2*x2*x3**2*x6**3+2*x2*x3**3*x6**2
dpdx2(264) = 2*x2*x5*x6**4+2*x2*x3**4*x5
dpdx2(265) = 2*x2*x3*x6**4+2*x2*x3**4*x6
dpdx2(266) = x1**2*x6**4+2*x2*x4*x5**4+2*x1*x2*x5**4+x3**4*x4**2
dpdx2(267) = 2*x1*x2*x6**4+x4**2*x5**4+x1**2*x5**4+2*x2*x3**4*x4
dpdx2(268) = x4**2*x6**4+2*x2*x4*x6**4+2*x1*x2*x3**4+x1**2*x3**4
dpdx2(269) = 2*x1**2*x2*x6**3+2*x2*x4**2*x5**3+2*x1**2*x2*x5**3+2&
&*x2*x3**3*x4**2
dpdx2(270) = 2*x2*x4**2*x6**3+2*x1**2*x2*x3**3
dpdx2(271) = 3*x2**2*x5**3*x6+3*x2**2*x3*x5**3
dpdx2(272) = 3*x2**2*x5**2*x6**2+3*x2**2*x3**2*x5**2
dpdx2(273) = 3*x2**2*x3**2*x6**2
dpdx2(274) = 3*x2**2*x5*x6**3+3*x2**2*x3**3*x5
dpdx2(275) = 3*x2**2*x3*x6**3+3*x2**2*x3**3*x6
dpdx2(276) = x1**3*x6**3+3*x2**2*x4*x5**3+3*x1*x2**2*x5**3+x3**3*&
&x4**3
dpdx2(277) = 3*x1*x2**2*x6**3+x4**3*x5**3+x1**3*x5**3+3*x2**2*x3*&
&*3*x4
dpdx2(278) = x4**3*x6**3+3*x2**2*x4*x6**3+3*x1*x2**2*x3**3+x1**3*&
&x3**3
dpdx2(279) = 2*x1**3*x2*x6**2+3*x2**2*x4**2*x5**2+3*x1**2*x2**2*x&
&5**2+2*x2*x3**2*x4**3
dpdx2(280) = 3*x1**2*x2**2*x6**2+2*x2*x4**3*x5**2+2*x1**3*x2*x5**&
&2+3*x2**2*x3**2*x4**2
dpdx2(281) = 2*x1**2*x2*x4**3+3*x1**2*x2**2*x4**2+2*x1**3*x2*x4**&
&2
dpdx2(282) = 2*x2*x4**3*x6**2+3*x2**2*x4**2*x6**2+3*x1**2*x2**2*x&
&3**2+2*x1**3*x2*x3**2
dpdx2(283) = 3*x1**3*x2**2*x6+3*x2**2*x4**3*x5+3*x1**3*x2**2*x5+3&
&*x2**2*x3*x4**3
dpdx2(284) = 3*x1*x2**2*x4**3+x1**3*x4**3+3*x1**3*x2**2*x4
dpdx2(285) = 3*x2**2*x4**3*x6+3*x1**3*x2**2*x3
dpdx2(286) = 4*x2**3*x5**2*x6+4*x2**3*x3*x5**2
dpdx2(287) = 4*x2**3*x5*x6**2+4*x2**3*x3**2*x5
dpdx2(288) = 4*x2**3*x3*x6**2+4*x2**3*x3**2*x6
dpdx2(289) = x1**4*x6**2+4*x2**3*x4*x5**2+4*x1*x2**3*x5**2+x3**2*&
&x4**4
dpdx2(290) = 4*x1*x2**3*x6**2+x4**4*x5**2+x1**4*x5**2+4*x2**3*x3*&
&*2*x4
dpdx2(291) = x4**4*x6**2+4*x2**3*x4*x6**2+4*x1*x2**3*x3**2+x1**4*&
&x3**2
dpdx2(292) = 2*x1**4*x2*x6+4*x2**3*x4**2*x5+4*x1**2*x2**3*x5+2*x2&
&*x3*x4**4
dpdx2(293) = 4*x1**2*x2**3*x6+2*x2*x4**4*x5+2*x1**4*x2*x5+4*x2**3&
&*x3*x4**2
dpdx2(294) = 2*x1*x2*x4**4+x1**2*x4**4+4*x1*x2**3*x4**2+x1**4*x4*&
&*2+4*x1**2*x2**3*x4+2*x1**4*x2*x4
dpdx2(295) = 2*x2*x4**4*x6+4*x2**3*x4**2*x6+4*x1**2*x2**3*x3+2*x1&
&**4*x2*x3
dpdx2(296) = 5*x2**4*x5*x6+5*x2**4*x3*x5
dpdx2(297) = 5*x2**4*x3*x6
dpdx2(298) = x1**5*x6+5*x2**4*x4*x5+5*x1*x2**4*x5+x3*x4**5
dpdx2(299) = 5*x1*x2**4*x6+x4**5*x5+x1**5*x5+5*x2**4*x3*x4
dpdx2(300) = x1*x4**5+5*x1*x2**4*x4+x1**5*x4
dpdx2(301) = x4**5*x6+5*x2**4*x4*x6+5*x1*x2**4*x3+x1**5*x3
dpdx2(302) = 0
dpdx2(303) = 0
dpdx2(304) = 0
dpdx2(305) = x6**6+x3**6
dpdx2(306) = 2*x2*x6**5+2*x2*x3**5
dpdx2(307) = 3*x2**2*x6**4+3*x2**2*x3**4
dpdx2(308) = 4*x2**3*x6**3+4*x2**3*x3**3
dpdx2(309) = 3*x2**2*x4**4+4*x2**3*x4**3+4*x1**3*x2**3+3*x1**4*x2&
&**2
dpdx2(310) = 5*x2**4*x6**2+5*x2**4*x3**2
dpdx2(311) = 2*x2*x4**5+5*x2**4*x4**2+5*x1**2*x2**4+2*x1**5*x2
dpdx2(312) = 6*x2**5*x6+6*x2**5*x3
dpdx2(313) = x4**6+6*x2**5*x4+6*x1*x2**5+x1**6
dpdx2(314) = x1*x3*x4*x5**2*x6**2+x1*x3**2*x4*x5*x6**2+x1*x3**2*x&
&4*x5**2*x6
dpdx2(315) = x1*x3*x4*x5*x6**3+x1*x3*x4*x5**3*x6+x1*x3**3*x4*x5*x&
&6
dpdx2(316) = x1**2*x3*x4*x5*x6**2+2*x1*x2*x3*x4*x5**2*x6+x1*x3**2&
&*x4**2*x5*x6
dpdx2(317) = x1*x3*x4**2*x5*x6**2+2*x1*x2*x3*x4*x5*x6**2+x1*x3*x4&
&**2*x5**2*x6+x1**2*x3*x4*x5**2*x6+2*x1*x2*x3**2*x4*x5*x6+x1**2*x3&
&**2*x4*x5*x6
dpdx2(318) = 2*x1*x2*x3*x4**2*x5*x6+x1**2*x3*x4**2*x5*x6+2*x1**2*&
&x2*x3*x4*x5*x6
dpdx2(319) = x1*x3*x4**3*x5*x6+3*x1*x2**2*x3*x4*x5*x6+x1**3*x3*x4&
&*x5*x6
dpdx2(320) = x1*x3*x5**2*x6**3+x1*x3*x5**3*x6**2+x3**2*x4*x5**3*x&
&6+x3**3*x4*x5**2*x6
dpdx2(321) = x1*x3*x5*x6**4+x3*x4*x5**4*x6+x1*x3*x5**4*x6+x3**4*x&
&4*x5*x6
dpdx2(322) = x3**2*x4*x5**2*x6**2+x1*x3**2*x5**2*x6**2
dpdx2(323) = x1*x3**2*x5*x6**3+x3*x4*x5**3*x6**2+x3**3*x4*x5*x6**&
&2+x1*x3**2*x5**3*x6
dpdx2(324) = x3*x4*x5**2*x6**3+x3**2*x4*x5*x6**3+x1*x3**3*x5*x6**&
&2+x1*x3**3*x5**2*x6
dpdx2(325) = x1*x4*x5**2*x6**3+x1*x3**2*x4*x6**3+x1*x4*x5**3*x6**&
&2+x1*x3**3*x4*x6**2+x1*x3**2*x4*x5**3+x1*x3**3*x4*x5**2
dpdx2(326) = x3*x4*x5*x6**4+x1*x3**4*x5*x6
dpdx2(327) = x1*x4*x5*x6**4+x1*x3*x4*x6**4+x1*x4*x5**4*x6+x1*x3**&
&4*x4*x6+x1*x3*x4*x5**4+x1*x3**4*x4*x5
dpdx2(328) = x1**2*x3*x5*x6**3+2*x2*x3*x4*x5**3*x6+2*x1*x2*x3*x5*&
&*3*x6+x3**3*x4**2*x5*x6
dpdx2(329) = 2*x1*x2*x3*x5**2*x6**2+x1**2*x3*x5**2*x6**2+x3**2*x4&
&**2*x5**2*x6+2*x2*x3**2*x4*x5**2*x6
dpdx2(330) = 2*x1*x2*x3*x5*x6**3+x3*x4**2*x5**3*x6+x1**2*x3*x5**3&
&*x6+2*x2*x3**3*x4*x5*x6
dpdx2(331) = x1**2*x4*x5*x6**3+x1**2*x3*x4*x6**3+2*x1*x2*x4*x5**3&
&*x6+x1*x3**3*x4**2*x6+2*x1*x2*x3*x4*x5**3+x1*x3**3*x4**2*x5
dpdx2(332) = 2*x2*x3*x4*x5**2*x6**2+x3**2*x4**2*x5*x6**2+x1**2*x3&
&**2*x5*x6**2+2*x1*x2*x3**2*x5**2*x6
dpdx2(333) = x3*x4**2*x5**2*x6**2+2*x2*x3**2*x4*x5*x6**2+2*x1*x2*&
&x3**2*x5*x6**2+x1**2*x3**2*x5**2*x6
dpdx2(334) = 2*x1*x2*x4*x5**2*x6**2+x1**2*x4*x5**2*x6**2+x1*x3**2&
&*x4**2*x6**2+x1**2*x3**2*x4*x6**2+x1*x3**2*x4**2*x5**2+2*x1*x2*x3&
&**2*x4*x5**2
dpdx2(335) = x1*x4**2*x5**2*x6**2+2*x1*x2*x3**2*x4*x6**2+x1**2*x3&
&**2*x4*x5**2
dpdx2(336) = x3*x4**2*x5*x6**3+2*x2*x3*x4*x5*x6**3+2*x1*x2*x3**3*&
&x5*x6+x1**2*x3**3*x5*x6
dpdx2(337) = 2*x1*x2*x4*x5*x6**3+x1*x3*x4**2*x6**3+x1**2*x4*x5**3&
&*x6+x1**2*x3**3*x4*x6+x1*x3*x4**2*x5**3+2*x1*x2*x3**3*x4*x5
dpdx2(338) = x1*x4**2*x5*x6**3+2*x1*x2*x3*x4*x6**3+x1*x4**2*x5**3&
&*x6+2*x1*x2*x3**3*x4*x6+x1**2*x3*x4*x5**3+x1**2*x3**3*x4*x5
dpdx2(339) = 2*x1**2*x2*x4*x5*x6**2+x1**2*x3*x4**2*x6**2+2*x1**2*&
&x2*x4*x5**2*x6+x1**2*x3**2*x4**2*x6+2*x1*x2*x3*x4**2*x5**2+2*x1*x&
&2*x3**2*x4**2*x5
dpdx2(340) = 2*x1**2*x2*x3*x5*x6**2+2*x2*x3*x4**2*x5**2*x6+2*x1**&
&2*x2*x3*x5**2*x6+2*x2*x3**2*x4**2*x5*x6
dpdx2(341) = x1**2*x4**2*x5*x6**2+2*x1**2*x2*x3*x4*x6**2+2*x1*x2*&
&x4**2*x5**2*x6+2*x1*x2*x3**2*x4**2*x6+2*x1**2*x2*x3*x4*x5**2+x1**&
&2*x3**2*x4**2*x5
dpdx2(342) = 2*x1**2*x2*x4**2*x5*x6+2*x1**2*x2*x3*x4**2*x6+2*x1**&
&2*x2*x3*x4**2*x5
dpdx2(343) = 2*x2*x3*x4**2*x5*x6**2+2*x1**2*x2*x3**2*x5*x6
dpdx2(344) = 2*x1*x2*x4**2*x5*x6**2+2*x1*x2*x3*x4**2*x6**2+x1**2*&
&x4**2*x5**2*x6+2*x1**2*x2*x3**2*x4*x6+x1**2*x3*x4**2*x5**2+2*x1**&
&2*x2*x3**2*x4*x5
dpdx2(345) = x1**3*x3*x5*x6**2+3*x2**2*x3*x4*x5**2*x6+3*x1*x2**2*&
&x3*x5**2*x6+x3**2*x4**3*x5*x6
dpdx2(346) = 3*x1*x2**2*x3*x5*x6**2+x3*x4**3*x5**2*x6+x1**3*x3*x5&
&**2*x6+3*x2**2*x3**2*x4*x5*x6
dpdx2(347) = x1**3*x4*x5*x6**2+x1**3*x3*x4*x6**2+3*x1*x2**2*x4*x5&
&**2*x6+x1*x3**2*x4**3*x6+3*x1*x2**2*x3*x4*x5**2+x1*x3**2*x4**3*x5
dpdx2(348) = x3*x4**3*x5*x6**2+3*x2**2*x3*x4*x5*x6**2+3*x1*x2**2*&
&x3**2*x5*x6+x1**3*x3**2*x5*x6
dpdx2(349) = 3*x1*x2**2*x4*x5*x6**2+x1*x3*x4**3*x6**2+x1**3*x4*x5&
&**2*x6+x1**3*x3**2*x4*x6+x1*x3*x4**3*x5**2+3*x1*x2**2*x3**2*x4*x5
dpdx2(350) = x1*x4**3*x5*x6**2+3*x1*x2**2*x3*x4*x6**2+x1*x4**3*x5&
&**2*x6+3*x1*x2**2*x3**2*x4*x6+x1**3*x3*x4*x5**2+x1**3*x3**2*x4*x5
dpdx2(351) = 3*x1**2*x2**2*x4*x5*x6+2*x1**3*x2*x4*x5*x6+x1**2*x3*&
&x4**3*x6+x1**3*x3*x4**2*x6+2*x1*x2*x3*x4**3*x5+3*x1*x2**2*x3*x4**&
&2*x5
dpdx2(352) = 2*x2*x3*x4**3*x5*x6+3*x2**2*x3*x4**2*x5*x6+3*x1**2*x&
&2**2*x3*x5*x6+2*x1**3*x2*x3*x5*x6
dpdx2(353) = 3*x1*x2**2*x4**2*x5*x6+x1**3*x4**2*x5*x6+2*x1*x2*x3*&
&x4**3*x6+2*x1**3*x2*x3*x4*x6+x1**2*x3*x4**3*x5+3*x1**2*x2**2*x3*x&
&4*x5
dpdx2(354) = 2*x1*x2*x4**3*x5*x6+x1**2*x4**3*x5*x6+3*x1*x2**2*x3*&
&x4**2*x6+3*x1**2*x2**2*x3*x4*x6+x1**3*x3*x4**2*x5+2*x1**3*x2*x3*x&
&4*x5
dpdx2(355) = x3*x4**4*x5*x6+4*x2**3*x3*x4*x5*x6+4*x1*x2**3*x3*x5*&
&x6+x1**4*x3*x5*x6
dpdx2(356) = 4*x1*x2**3*x4*x5*x6+x1**4*x4*x5*x6+x1*x3*x4**4*x6+x1&
&**4*x3*x4*x6+x1*x3*x4**4*x5+4*x1*x2**3*x3*x4*x5
dpdx2(357) = x1*x4**4*x5*x6+4*x1*x2**3*x3*x4*x6+x1**4*x3*x4*x5
dpdx2(358) = x3*x5**5*x6
dpdx2(359) = x3*x5**4*x6**2+x3**2*x5**4*x6
dpdx2(360) = x3**2*x5**3*x6**2
dpdx2(361) = x3*x5**3*x6**3+x3**3*x5**3*x6
dpdx2(362) = x3**2*x5**2*x6**3+x3**3*x5**2*x6**2
dpdx2(363) = x3**3*x5*x6**3
dpdx2(364) = x3*x5**2*x6**4+x3**4*x5**2*x6
dpdx2(365) = x3**2*x5*x6**4+x3**4*x5*x6**2
dpdx2(366) = x3*x5*x6**5+x3**5*x5*x6
dpdx2(367) = x1*x5**3*x6**3+x3**3*x4*x5**3
dpdx2(368) = x1*x5**2*x6**4+x1*x5**4*x6**2+x3**2*x4*x5**4+x3**4*x&
&4*x5**2
dpdx2(369) = x1*x5*x6**5+x1*x5**5*x6+x3*x4*x5**5+x3**5*x4*x5
dpdx2(370) = x1*x3*x6**5+x4*x5**5*x6+x3**5*x4*x6+x1*x3*x5**5
dpdx2(371) = x1*x3**2*x6**4+x4*x5**4*x6**2+x3**4*x4*x6**2+x1*x3**&
&2*x5**4
dpdx2(372) = x4*x5**3*x6**3+x3**3*x4*x6**3+x1*x3**3*x6**3+x1*x3**&
&3*x5**3
dpdx2(373) = x4*x5**2*x6**4+x3**2*x4*x6**4+x1*x3**4*x6**2+x1*x3**&
&4*x5**2
dpdx2(374) = x4*x5*x6**5+x3*x4*x6**5+x1*x3**5*x6+x1*x3**5*x5
dpdx2(375) = x1*x4*x6**5+x1*x4*x5**5+x1*x3**5*x4
dpdx2(376) = 2*x2*x3*x5**4*x6
dpdx2(377) = 2*x2*x3*x5**3*x6**2+2*x2*x3**2*x5**3*x6
dpdx2(378) = 2*x2*x3**2*x5**2*x6**2
dpdx2(379) = 2*x2*x3*x5**2*x6**3+2*x2*x3**3*x5**2*x6
dpdx2(380) = 2*x2*x3**2*x5*x6**3+2*x2*x3**3*x5*x6**2
dpdx2(381) = 2*x2*x3*x5*x6**4+2*x2*x3**4*x5*x6
dpdx2(382) = x1**2*x5*x6**4+2*x1*x2*x5**4*x6+2*x2*x3*x4*x5**4+x3*&
&*4*x4**2*x5
dpdx2(383) = x1**2*x5**2*x6**3+2*x1*x2*x5**3*x6**2+2*x2*x3**2*x4*&
&x5**3+x3**3*x4**2*x5**2
dpdx2(384) = 2*x1*x2*x5**2*x6**3+x1**2*x5**3*x6**2+x3**2*x4**2*x5&
&**3+2*x2*x3**3*x4*x5**2
dpdx2(385) = 2*x1*x2*x5*x6**4+x1**2*x5**4*x6+x3*x4**2*x5**4+2*x2*&
&x3**4*x4*x5
dpdx2(386) = x1**2*x4*x6**4+2*x1*x2*x4*x5**4+x1*x3**4*x4**2
dpdx2(387) = x1**2*x3*x6**4+2*x2*x4*x5**4*x6+x3**4*x4**2*x6+2*x1*&
&x2*x3*x5**4
dpdx2(388) = 2*x1*x2*x3*x6**4+x4**2*x5**4*x6+2*x2*x3**4*x4*x6+x1*&
&*2*x3*x5**4
dpdx2(389) = x1**2*x3**2*x6**3+2*x2*x4*x5**3*x6**2+x3**3*x4**2*x6&
&**2+2*x1*x2*x3**2*x5**3
dpdx2(390) = 2*x1*x2*x3**2*x6**3+x4**2*x5**3*x6**2+2*x2*x3**3*x4*&
&x6**2+x1**2*x3**2*x5**3
dpdx2(391) = 2*x2*x4*x5**2*x6**3+x3**2*x4**2*x6**3+x1**2*x3**3*x6&
&**2+2*x1*x2*x3**3*x5**2
dpdx2(392) = x4**2*x5**2*x6**3+2*x2*x3**2*x4*x6**3+2*x1*x2*x3**3*&
&x6**2+x1**2*x3**3*x5**2
dpdx2(393) = 2*x2*x4*x5*x6**4+x3*x4**2*x6**4+x1**2*x3**4*x6+2*x1*&
&x2*x3**4*x5
dpdx2(394) = x4**2*x5*x6**4+2*x2*x3*x4*x6**4+2*x1*x2*x3**4*x6+x1*&
&*2*x3**4*x5
dpdx2(395) = x1*x4**2*x6**4+2*x1*x2*x4*x6**4+x1*x4**2*x5**4+x1**2&
&*x4*x5**4+2*x1*x2*x3**4*x4+x1**2*x3**4*x4
dpdx2(396) = 2*x1**2*x2*x5**2*x6**2+2*x2*x3**2*x4**2*x5**2
dpdx2(397) = 2*x1**2*x2*x5*x6**3+2*x1**2*x2*x5**3*x6+2*x2*x3*x4**&
&2*x5**3+2*x2*x3**3*x4**2*x5
dpdx2(398) = x1**2*x4**2*x6**3+2*x1**2*x2*x4*x6**3+2*x1*x2*x4**2*&
&x5**3+2*x1**2*x2*x4*x5**3+2*x1*x2*x3**3*x4**2+x1**2*x3**3*x4**2
dpdx2(399) = 2*x1**2*x2*x3*x6**3+2*x2*x4**2*x5**3*x6+2*x2*x3**3*x&
&4**2*x6+2*x1**2*x2*x3*x5**3
dpdx2(400) = 2*x2*x4**2*x5**2*x6**2+2*x2*x3**2*x4**2*x6**2+2*x1**&
&2*x2*x3**2*x6**2+2*x1**2*x2*x3**2*x5**2
dpdx2(401) = 2*x1**2*x2*x4**2*x6**2+2*x1**2*x2*x4**2*x5**2+2*x1**&
&2*x2*x3**2*x4**2
dpdx2(402) = 2*x2*x4**2*x5*x6**3+2*x2*x3*x4**2*x6**3+2*x1**2*x2*x&
&3**3*x6+2*x1**2*x2*x3**3*x5
dpdx2(403) = 2*x1*x2*x4**2*x6**3+x1**2*x4**2*x5**3+2*x1**2*x2*x3*&
&*3*x4
dpdx2(404) = 3*x2**2*x3*x5**3*x6
dpdx2(405) = 3*x2**2*x3*x5**2*x6**2+3*x2**2*x3**2*x5**2*x6
dpdx2(406) = 3*x2**2*x3**2*x5*x6**2
dpdx2(407) = 3*x2**2*x3*x5*x6**3+3*x2**2*x3**3*x5*x6
dpdx2(408) = x1**3*x5*x6**3+3*x1*x2**2*x5**3*x6+3*x2**2*x3*x4*x5*&
&*3+x3**3*x4**3*x5
dpdx2(409) = 3*x1*x2**2*x5**2*x6**2+x1**3*x5**2*x6**2+x3**2*x4**3&
&*x5**2+3*x2**2*x3**2*x4*x5**2
dpdx2(410) = 3*x1*x2**2*x5*x6**3+x1**3*x5**3*x6+x3*x4**3*x5**3+3*&
&x2**2*x3**3*x4*x5
dpdx2(411) = x1**3*x4*x6**3+3*x1*x2**2*x4*x5**3+x1*x3**3*x4**3
dpdx2(412) = x1**3*x3*x6**3+3*x2**2*x4*x5**3*x6+x3**3*x4**3*x6+3*&
&x1*x2**2*x3*x5**3
dpdx2(413) = 3*x1*x2**2*x3*x6**3+x4**3*x5**3*x6+3*x2**2*x3**3*x4*&
&x6+x1**3*x3*x5**3
dpdx2(414) = 3*x2**2*x4*x5**2*x6**2+x3**2*x4**3*x6**2+x1**3*x3**2&
&*x6**2+3*x1*x2**2*x3**2*x5**2
dpdx2(415) = x4**3*x5**2*x6**2+3*x2**2*x3**2*x4*x6**2+3*x1*x2**2*&
&x3**2*x6**2+x1**3*x3**2*x5**2
dpdx2(416) = 3*x2**2*x4*x5*x6**3+x3*x4**3*x6**3+x1**3*x3**3*x6+3*&
&x1*x2**2*x3**3*x5
dpdx2(417) = x4**3*x5*x6**3+3*x2**2*x3*x4*x6**3+3*x1*x2**2*x3**3*&
&x6+x1**3*x3**3*x5
dpdx2(418) = x1*x4**3*x6**3+3*x1*x2**2*x4*x6**3+x1*x4**3*x5**3+x1&
&**3*x4*x5**3+3*x1*x2**2*x3**3*x4+x1**3*x3**3*x4
dpdx2(419) = 2*x1**3*x2*x5*x6**2+3*x1**2*x2**2*x5**2*x6+3*x2**2*x&
&3*x4**2*x5**2+2*x2*x3**2*x4**3*x5
dpdx2(420) = 3*x1**2*x2**2*x5*x6**2+2*x1**3*x2*x5**2*x6+2*x2*x3*x&
&4**3*x5**2+3*x2**2*x3**2*x4**2*x5
dpdx2(421) = x1**3*x4**2*x6**2+2*x1**3*x2*x4*x6**2+3*x1*x2**2*x4*&
&*2*x5**2+3*x1**2*x2**2*x4*x5**2+2*x1*x2*x3**2*x4**3+x1**2*x3**2*x&
&4**3
dpdx2(422) = x1**2*x4**3*x6**2+3*x1**2*x2**2*x4*x6**2+2*x1*x2*x4*&
&*3*x5**2+2*x1**3*x2*x4*x5**2+3*x1*x2**2*x3**2*x4**2+x1**3*x3**2*x&
&4**2
dpdx2(423) = 2*x1**3*x2*x4**2*x6+3*x1**2*x2**2*x4**2*x5+2*x1**2*x&
&2*x3*x4**3
dpdx2(424) = 2*x1**3*x2*x3*x6**2+3*x2**2*x4**2*x5**2*x6+2*x2*x3**&
&2*x4**3*x6+3*x1**2*x2**2*x3*x5**2
dpdx2(425) = 3*x1**2*x2**2*x3*x6**2+2*x2*x4**3*x5**2*x6+3*x2**2*x&
&3**2*x4**2*x6+2*x1**3*x2*x3*x5**2
dpdx2(426) = 2*x1**2*x2*x4**3*x6+3*x1**2*x2**2*x4**2*x6+2*x1**2*x&
&2*x4**3*x5+2*x1**3*x2*x4**2*x5+3*x1**2*x2**2*x3*x4**2+2*x1**3*x2*&
&x3*x4**2
dpdx2(427) = 3*x2**2*x4**2*x5*x6**2+2*x2*x3*x4**3*x6**2+2*x1**3*x&
&2*x3**2*x6+3*x1**2*x2**2*x3**2*x5
dpdx2(428) = 2*x2*x4**3*x5*x6**2+3*x2**2*x3*x4**2*x6**2+3*x1**2*x&
&2**2*x3**2*x6+2*x1**3*x2*x3**2*x5
dpdx2(429) = 2*x1*x2*x4**3*x6**2+3*x1*x2**2*x4**2*x6**2+x1**2*x4*&
&*3*x5**2+x1**3*x4**2*x5**2+3*x1**2*x2**2*x3**2*x4+2*x1**3*x2*x3**&
&2*x4
dpdx2(430) = 3*x1**3*x2**2*x5*x6+3*x2**2*x3*x4**3*x5
dpdx2(431) = x1**3*x4**3*x6+3*x1**3*x2**2*x4*x6+3*x1*x2**2*x4**3*&
&x5+3*x1**3*x2**2*x4*x5+3*x1*x2**2*x3*x4**3+x1**3*x3*x4**3
dpdx2(432) = 3*x2**2*x4**3*x5*x6+3*x2**2*x3*x4**3*x6+3*x1**3*x2**&
&2*x3*x6+3*x1**3*x2**2*x3*x5
dpdx2(433) = 3*x1*x2**2*x4**3*x6+x1**3*x4**3*x5+3*x1**3*x2**2*x3*&
&x4
dpdx2(434) = 4*x2**3*x3*x5**2*x6
dpdx2(435) = 4*x2**3*x3*x5*x6**2+4*x2**3*x3**2*x5*x6
dpdx2(436) = x1**4*x5*x6**2+4*x1*x2**3*x5**2*x6+4*x2**3*x3*x4*x5*&
&*2+x3**2*x4**4*x5
dpdx2(437) = 4*x1*x2**3*x5*x6**2+x1**4*x5**2*x6+x3*x4**4*x5**2+4*&
&x2**3*x3**2*x4*x5
dpdx2(438) = x1**4*x4*x6**2+4*x1*x2**3*x4*x5**2+x1*x3**2*x4**4
dpdx2(439) = x1**4*x3*x6**2+4*x2**3*x4*x5**2*x6+x3**2*x4**4*x6+4*&
&x1*x2**3*x3*x5**2
dpdx2(440) = 4*x1*x2**3*x3*x6**2+x4**4*x5**2*x6+4*x2**3*x3**2*x4*&
&x6+x1**4*x3*x5**2
dpdx2(441) = 4*x2**3*x4*x5*x6**2+x3*x4**4*x6**2+x1**4*x3**2*x6+4*&
&x1*x2**3*x3**2*x5
dpdx2(442) = x4**4*x5*x6**2+4*x2**3*x3*x4*x6**2+4*x1*x2**3*x3**2*&
&x6+x1**4*x3**2*x5
dpdx2(443) = x1*x4**4*x6**2+4*x1*x2**3*x4*x6**2+x1*x4**4*x5**2+x1&
&**4*x4*x5**2+4*x1*x2**3*x3**2*x4+x1**4*x3**2*x4
dpdx2(444) = 4*x1**2*x2**3*x5*x6+2*x1**4*x2*x5*x6+2*x2*x3*x4**4*x&
&5+4*x2**3*x3*x4**2*x5
dpdx2(445) = x1**4*x4**2*x6+2*x1**4*x2*x4*x6+4*x1*x2**3*x4**2*x5+&
&4*x1**2*x2**3*x4*x5+2*x1*x2*x3*x4**4+x1**2*x3*x4**4
dpdx2(446) = x1**2*x4**4*x6+4*x1**2*x2**3*x4*x6+2*x1*x2*x4**4*x5+&
&2*x1**4*x2*x4*x5+4*x1*x2**3*x3*x4**2+x1**4*x3*x4**2
dpdx2(447) = 4*x2**3*x4**2*x5*x6+2*x2*x3*x4**4*x6+2*x1**4*x2*x3*x&
&6+4*x1**2*x2**3*x3*x5
dpdx2(448) = 2*x2*x4**4*x5*x6+4*x2**3*x3*x4**2*x6+4*x1**2*x2**3*x&
&3*x6+2*x1**4*x2*x3*x5
dpdx2(449) = 2*x1*x2*x4**4*x6+4*x1*x2**3*x4**2*x6+x1**2*x4**4*x5+&
&x1**4*x4**2*x5+4*x1**2*x2**3*x3*x4+2*x1**4*x2*x3*x4
dpdx2(450) = 5*x2**4*x3*x5*x6
dpdx2(451) = 5*x1*x2**4*x5*x6+x1**5*x5*x6+x3*x4**5*x5+5*x2**4*x3*&
&x4*x5
dpdx2(452) = x1**5*x4*x6+5*x1*x2**4*x4*x5+x1*x3*x4**5
dpdx2(453) = 5*x2**4*x4*x5*x6+x3*x4**5*x6+x1**5*x3*x6+5*x1*x2**4*&
&x3*x5
dpdx2(454) = x4**5*x5*x6+5*x2**4*x3*x4*x6+5*x1*x2**4*x3*x6+x1**5*&
&x3*x5
dpdx2(455) = x1*x4**5*x6+5*x1*x2**4*x4*x6+x1*x4**5*x5+x1**5*x4*x5&
&+5*x1*x2**4*x3*x4+x1**5*x3*x4
dpdx2(456) = 0
dpdx2(457) = 0
dpdx2(458) = 0
dpdx2(459) = 0
dpdx2(460) = 0
dpdx2(461) = x5**6*x6+x3*x5**6
dpdx2(462) = x5**5*x6**2+x3**2*x5**5
dpdx2(463) = x5**4*x6**3+x3**3*x5**4
dpdx2(464) = x5**3*x6**4+x3**4*x5**3
dpdx2(465) = x3**3*x6**4+x3**4*x6**3
dpdx2(466) = x5**2*x6**5+x3**5*x5**2
dpdx2(467) = x3**2*x6**5+x3**5*x6**2
dpdx2(468) = x5*x6**6+x3**6*x5
dpdx2(469) = x3*x6**6+x3**6*x6
dpdx2(470) = x1*x6**6+x4*x5**6+x1*x5**6+x3**6*x4
dpdx2(471) = x4*x6**6+x1*x3**6
dpdx2(472) = 2*x2*x5**5*x6+2*x2*x3*x5**5
dpdx2(473) = 2*x2*x5**4*x6**2+2*x2*x3**2*x5**4
dpdx2(474) = 2*x2*x5**3*x6**3+2*x2*x3**3*x5**3
dpdx2(475) = 2*x2*x3**3*x6**3
dpdx2(476) = 2*x2*x5**2*x6**4+2*x2*x3**4*x5**2
dpdx2(477) = 2*x2*x3**2*x6**4+2*x2*x3**4*x6**2
dpdx2(478) = 2*x2*x5*x6**5+2*x2*x3**5*x5
dpdx2(479) = 2*x2*x3*x6**5+2*x2*x3**5*x6
dpdx2(480) = x1**2*x6**5+2*x2*x4*x5**5+2*x1*x2*x5**5+x3**5*x4**2
dpdx2(481) = 2*x1*x2*x6**5+x4**2*x5**5+x1**2*x5**5+2*x2*x3**5*x4
dpdx2(482) = x4**2*x6**5+2*x2*x4*x6**5+2*x1*x2*x3**5+x1**2*x3**5
dpdx2(483) = 2*x1**2*x2*x6**4+2*x2*x4**2*x5**4+2*x1**2*x2*x5**4+2&
&*x2*x3**4*x4**2
dpdx2(484) = 2*x2*x4**2*x6**4+2*x1**2*x2*x3**4
dpdx2(485) = 3*x2**2*x5**4*x6+3*x2**2*x3*x5**4
dpdx2(486) = 3*x2**2*x5**3*x6**2+3*x2**2*x3**2*x5**3
dpdx2(487) = 3*x2**2*x5**2*x6**3+3*x2**2*x3**3*x5**2
dpdx2(488) = 3*x2**2*x3**2*x6**3+3*x2**2*x3**3*x6**2
dpdx2(489) = 3*x2**2*x5*x6**4+3*x2**2*x3**4*x5
dpdx2(490) = 3*x2**2*x3*x6**4+3*x2**2*x3**4*x6
dpdx2(491) = x1**3*x6**4+3*x2**2*x4*x5**4+3*x1*x2**2*x5**4+x3**4*&
&x4**3
dpdx2(492) = 3*x1*x2**2*x6**4+x4**3*x5**4+x1**3*x5**4+3*x2**2*x3*&
&*4*x4
dpdx2(493) = x4**3*x6**4+3*x2**2*x4*x6**4+3*x1*x2**2*x3**4+x1**3*&
&x3**4
dpdx2(494) = 2*x1**3*x2*x6**3+3*x2**2*x4**2*x5**3+3*x1**2*x2**2*x&
&5**3+2*x2*x3**3*x4**3
dpdx2(495) = 3*x1**2*x2**2*x6**3+2*x2*x4**3*x5**3+2*x1**3*x2*x5**&
&3+3*x2**2*x3**3*x4**2
dpdx2(496) = 2*x2*x4**3*x6**3+3*x2**2*x4**2*x6**3+3*x1**2*x2**2*x&
&3**3+2*x1**3*x2*x3**3
dpdx2(497) = 3*x1**3*x2**2*x6**2+3*x2**2*x4**3*x5**2+3*x1**3*x2**&
&2*x5**2+3*x2**2*x3**2*x4**3
dpdx2(498) = 3*x1**2*x2**2*x4**3+2*x1**3*x2*x4**3+3*x1**3*x2**2*x&
&4**2
dpdx2(499) = 3*x2**2*x4**3*x6**2+3*x1**3*x2**2*x3**2
dpdx2(500) = 4*x2**3*x5**3*x6+4*x2**3*x3*x5**3
dpdx2(501) = 4*x2**3*x5**2*x6**2+4*x2**3*x3**2*x5**2
dpdx2(502) = 4*x2**3*x3**2*x6**2
dpdx2(503) = 4*x2**3*x5*x6**3+4*x2**3*x3**3*x5
dpdx2(504) = 4*x2**3*x3*x6**3+4*x2**3*x3**3*x6
dpdx2(505) = x1**4*x6**3+4*x2**3*x4*x5**3+4*x1*x2**3*x5**3+x3**3*&
&x4**4
dpdx2(506) = 4*x1*x2**3*x6**3+x4**4*x5**3+x1**4*x5**3+4*x2**3*x3*&
&*3*x4
dpdx2(507) = x4**4*x6**3+4*x2**3*x4*x6**3+4*x1*x2**3*x3**3+x1**4*&
&x3**3
dpdx2(508) = 2*x1**4*x2*x6**2+4*x2**3*x4**2*x5**2+4*x1**2*x2**3*x&
&5**2+2*x2*x3**2*x4**4
dpdx2(509) = 4*x1**2*x2**3*x6**2+2*x2*x4**4*x5**2+2*x1**4*x2*x5**&
&2+4*x2**3*x3**2*x4**2
dpdx2(510) = 2*x1**2*x2*x4**4+4*x1**2*x2**3*x4**2+2*x1**4*x2*x4**&
&2
dpdx2(511) = 2*x2*x4**4*x6**2+4*x2**3*x4**2*x6**2+4*x1**2*x2**3*x&
&3**2+2*x1**4*x2*x3**2
dpdx2(512) = 3*x1**4*x2**2*x6+4*x2**3*x4**3*x5+4*x1**3*x2**3*x5+3&
&*x2**2*x3*x4**4
dpdx2(513) = 4*x1**3*x2**3*x6+3*x2**2*x4**4*x5+3*x1**4*x2**2*x5+4&
&*x2**3*x3*x4**3
dpdx2(514) = 3*x1*x2**2*x4**4+x1**3*x4**4+4*x1*x2**3*x4**3+x1**4*&
&x4**3+4*x1**3*x2**3*x4+3*x1**4*x2**2*x4
dpdx2(515) = 3*x2**2*x4**4*x6+4*x2**3*x4**3*x6+4*x1**3*x2**3*x3+3&
&*x1**4*x2**2*x3
dpdx2(516) = 5*x2**4*x5**2*x6+5*x2**4*x3*x5**2
dpdx2(517) = 5*x2**4*x5*x6**2+5*x2**4*x3**2*x5
dpdx2(518) = 5*x2**4*x3*x6**2+5*x2**4*x3**2*x6
dpdx2(519) = x1**5*x6**2+5*x2**4*x4*x5**2+5*x1*x2**4*x5**2+x3**2*&
&x4**5
dpdx2(520) = 5*x1*x2**4*x6**2+x4**5*x5**2+x1**5*x5**2+5*x2**4*x3*&
&*2*x4
dpdx2(521) = x4**5*x6**2+5*x2**4*x4*x6**2+5*x1*x2**4*x3**2+x1**5*&
&x3**2
dpdx2(522) = 2*x1**5*x2*x6+5*x2**4*x4**2*x5+5*x1**2*x2**4*x5+2*x2&
&*x3*x4**5
dpdx2(523) = 5*x1**2*x2**4*x6+2*x2*x4**5*x5+2*x1**5*x2*x5+5*x2**4&
&*x3*x4**2
dpdx2(524) = 2*x1*x2*x4**5+x1**2*x4**5+5*x1*x2**4*x4**2+x1**5*x4*&
&*2+5*x1**2*x2**4*x4+2*x1**5*x2*x4
dpdx2(525) = 2*x2*x4**5*x6+5*x2**4*x4**2*x6+5*x1**2*x2**4*x3+2*x1&
&**5*x2*x3
dpdx2(526) = 6*x2**5*x5*x6+6*x2**5*x3*x5
dpdx2(527) = 6*x2**5*x3*x6
dpdx2(528) = x1**6*x6+6*x2**5*x4*x5+6*x1*x2**5*x5+x3*x4**6
dpdx2(529) = 6*x1*x2**5*x6+x4**6*x5+x1**6*x5+6*x2**5*x3*x4
dpdx2(530) = x1*x4**6+6*x1*x2**5*x4+x1**6*x4
dpdx2(531) = x4**6*x6+6*x2**5*x4*x6+6*x1*x2**5*x3+x1**6*x3
dpdx2(532) = 0
dpdx2(533) = 0
dpdx2(534) = 0
dpdx2(535) = 0
dpdx2(536) = x6**7+x3**7
dpdx2(537) = 2*x2*x6**6+2*x2*x3**6
dpdx2(538) = 3*x2**2*x6**5+3*x2**2*x3**5
dpdx2(539) = 4*x2**3*x6**4+4*x2**3*x3**4
dpdx2(540) = 4*x2**3*x4**4+4*x1**4*x2**3
dpdx2(541) = 5*x2**4*x6**3+5*x2**4*x3**3
dpdx2(542) = 3*x2**2*x4**5+5*x2**4*x4**3+5*x1**3*x2**4+3*x1**5*x2&
&**2
dpdx2(543) = 6*x2**5*x6**2+6*x2**5*x3**2
dpdx2(544) = 2*x2*x4**6+6*x2**5*x4**2+6*x1**2*x2**5+2*x1**6*x2
dpdx2(545) = 7*x2**6*x6+7*x2**6*x3
dpdx2(546) = x4**7+7*x2**6*x4+7*x1*x2**6+x1**7
dpdx3(1) = 0
dpdx3(2) = x6+x5
dpdx3(3) = x2+x1
dpdx3(4) = 0
dpdx3(5) = x5*x6
dpdx3(6) = x4*x6+x1*x6+x4*x5+x2*x5
dpdx3(7) = x2*x6+x1*x5
dpdx3(8) = x2*x4+x1*x4
dpdx3(9) = 0
dpdx3(10) = x1*x2
dpdx3(11) = x6**2+2*x3*x6+x5**2+2*x3*x5
dpdx3(12) = 2*x2*x3+2*x1*x3
dpdx3(13) = x2**2+x1**2
dpdx3(14) = 0
dpdx3(15) = x4*x5*x6+x2*x5*x6+x1*x5*x6
dpdx3(16) = x1*x4*x6+x2*x4*x5
dpdx3(17) = x2*x4*x6+x1*x2*x6+x1*x4*x5+x1*x2*x5
dpdx3(18) = x1*x2*x4
dpdx3(19) = x5*x6**2+x5**2*x6+2*x3*x5*x6
dpdx3(20) = x1*x6**2+2*x3*x4*x6+x2*x5**2+2*x3*x4*x5
dpdx3(21) = x4*x6**2+2*x1*x3*x6+x4*x5**2+2*x2*x3*x5
dpdx3(22) = x2*x6**2+2*x2*x3*x6+x1*x5**2+2*x1*x3*x5
dpdx3(23) = 2*x2*x3*x4+2*x1*x3*x4
dpdx3(24) = 2*x1*x2*x3
dpdx3(25) = x4**2*x6+x1**2*x6+x4**2*x5+x2**2*x5
dpdx3(26) = x2**2*x6+x1**2*x5
dpdx3(27) = x2*x4**2+x1*x4**2
dpdx3(28) = x2**2*x4+x1**2*x4
dpdx3(29) = 0
dpdx3(30) = x1*x2**2+x1**2*x2
dpdx3(31) = 2*x3*x6**2+2*x3*x5**2
dpdx3(32) = x6**3+3*x3**2*x6+x5**3+3*x3**2*x5
dpdx3(33) = 3*x2*x3**2+3*x1*x3**2
dpdx3(34) = 2*x2**2*x3+2*x1**2*x3
dpdx3(35) = 0
dpdx3(36) = x2**3+x1**3
dpdx3(37) = 0
dpdx3(38) = x2*x4*x5*x6+x1*x4*x5*x6+x1*x2*x5*x6
dpdx3(39) = x1*x2*x4*x6+x1*x2*x4*x5
dpdx3(40) = x1*x5*x6**2+x2*x5**2*x6+2*x3*x4*x5*x6
dpdx3(41) = x4*x5*x6**2+x2*x5*x6**2+x4*x5**2*x6+x1*x5**2*x6+2*x2*&
&x3*x5*x6+2*x1*x3*x5*x6
dpdx3(42) = x1*x4*x6**2+2*x1*x3*x4*x6+x2*x4*x5**2+2*x2*x3*x4*x5
dpdx3(43) = x1*x2*x6**2+2*x2*x3*x4*x6+x1*x2*x5**2+2*x1*x3*x4*x5
dpdx3(44) = x2*x4*x6**2+2*x1*x2*x3*x6+x1*x4*x5**2+2*x1*x2*x3*x5
dpdx3(45) = 2*x1*x2*x3*x4
dpdx3(46) = x4**2*x5*x6+x2**2*x5*x6+x1**2*x5*x6
dpdx3(47) = x1*x4**2*x6+x1**2*x4*x6+x2*x4**2*x5+x2**2*x4*x5
dpdx3(48) = x1*x2*x4**2
dpdx3(49) = x2*x4**2*x6+x1**2*x2*x6+x1*x4**2*x5+x1*x2**2*x5
dpdx3(50) = x2**2*x4*x6+x1*x2**2*x6+x1**2*x4*x5+x1**2*x2*x5
dpdx3(51) = x1*x2**2*x4+x1**2*x2*x4
dpdx3(52) = x5**2*x6**2+2*x3*x5*x6**2+2*x3*x5**2*x6
dpdx3(53) = x5*x6**3+x5**3*x6+3*x3**2*x5*x6
dpdx3(54) = x1*x6**3+3*x3**2*x4*x6+x2*x5**3+3*x3**2*x4*x5
dpdx3(55) = 2*x3*x4*x6**2+2*x1*x3*x6**2+2*x3*x4*x5**2+2*x2*x3*x5*&
&*2
dpdx3(56) = 2*x2*x3*x6**2+2*x1*x3*x5**2
dpdx3(57) = x4*x6**3+3*x1*x3**2*x6+x4*x5**3+3*x2*x3**2*x5
dpdx3(58) = x2*x6**3+3*x2*x3**2*x6+x1*x5**3+3*x1*x3**2*x5
dpdx3(59) = 3*x2*x3**2*x4+3*x1*x3**2*x4
dpdx3(60) = 3*x1*x2*x3**2
dpdx3(61) = x1**2*x6**2+2*x3*x4**2*x6+x2**2*x5**2+2*x3*x4**2*x5
dpdx3(62) = x4**2*x6**2+2*x1**2*x3*x6+x4**2*x5**2+2*x2**2*x3*x5
dpdx3(63) = x2**2*x6**2+2*x2**2*x3*x6+x1**2*x5**2+2*x1**2*x3*x5
dpdx3(64) = 2*x2*x3*x4**2+2*x1*x3*x4**2
dpdx3(65) = 2*x2**2*x3*x4+2*x1**2*x3*x4
dpdx3(66) = 2*x1*x2**2*x3+2*x1**2*x2*x3
dpdx3(67) = x2**2*x4**2+x1**2*x4**2
dpdx3(68) = 0
dpdx3(69) = x1**2*x2**2
dpdx3(70) = x4**3*x6+x1**3*x6+x4**3*x5+x2**3*x5
dpdx3(71) = x2**3*x6+x1**3*x5
dpdx3(72) = x2*x4**3+x1*x4**3
dpdx3(73) = x2**3*x4+x1**3*x4
dpdx3(74) = 0
dpdx3(75) = x1*x2**3+x1**3*x2
dpdx3(76) = 2*x3*x6**3+3*x3**2*x6**2+2*x3*x5**3+3*x3**2*x5**2
dpdx3(77) = x6**4+4*x3**3*x6+x5**4+4*x3**3*x5
dpdx3(78) = 4*x2*x3**3+4*x1*x3**3
dpdx3(79) = 3*x2**2*x3**2+3*x1**2*x3**2
dpdx3(80) = 2*x2**3*x3+2*x1**3*x3
dpdx3(81) = 0
dpdx3(82) = x2**4+x1**4
dpdx3(83) = 0
dpdx3(84) = x1*x2*x4*x5*x6
dpdx3(85) = x1*x4*x5*x6**2+x1*x2*x5*x6**2+x2*x4*x5**2*x6+x1*x2*x5&
&**2*x6+2*x2*x3*x4*x5*x6+2*x1*x3*x4*x5*x6
dpdx3(86) = x2*x4*x5*x6**2+x1*x4*x5**2*x6+2*x1*x2*x3*x5*x6
dpdx3(87) = x1*x2*x4*x6**2+2*x1*x2*x3*x4*x6+x1*x2*x4*x5**2+2*x1*x&
&2*x3*x4*x5
dpdx3(88) = x2*x4**2*x5*x6+x1*x4**2*x5*x6+x2**2*x4*x5*x6+x1**2*x4&
&*x5*x6+x1*x2**2*x5*x6+x1**2*x2*x5*x6
dpdx3(89) = x1*x2*x4**2*x6+x1**2*x2*x4*x6+x1*x2*x4**2*x5+x1*x2**2&
&*x4*x5
dpdx3(90) = x1*x2**2*x4*x6+x1**2*x2*x4*x5
dpdx3(91) = x1*x5*x6**3+x2*x5**3*x6+3*x3**2*x4*x5*x6
dpdx3(92) = x2*x5**2*x6**2+x1*x5**2*x6**2+2*x3*x4*x5*x6**2+2*x1*x&
&3*x5*x6**2+2*x3*x4*x5**2*x6+2*x2*x3*x5**2*x6
dpdx3(93) = x4*x5**2*x6**2+2*x2*x3*x5*x6**2+2*x1*x3*x5**2*x6
dpdx3(94) = x4*x5*x6**3+x2*x5*x6**3+x4*x5**3*x6+x1*x5**3*x6+3*x2*&
&x3**2*x5*x6+3*x1*x3**2*x5*x6
dpdx3(95) = 2*x1*x3*x4*x6**2+2*x2*x3*x4*x5**2
dpdx3(96) = x1*x4*x6**3+3*x1*x3**2*x4*x6+x2*x4*x5**3+3*x2*x3**2*x&
&4*x5
dpdx3(97) = x1*x2*x6**3+3*x2*x3**2*x4*x6+x1*x2*x5**3+3*x1*x3**2*x&
&4*x5
dpdx3(98) = 2*x2*x3*x4*x6**2+2*x1*x2*x3*x6**2+2*x1*x3*x4*x5**2+2*&
&x1*x2*x3*x5**2
dpdx3(99) = x2*x4*x6**3+3*x1*x2*x3**2*x6+x1*x4*x5**3+3*x1*x2*x3**&
&2*x5
dpdx3(100) = 3*x1*x2*x3**2*x4
dpdx3(101) = x1**2*x5*x6**2+x2**2*x5**2*x6+2*x3*x4**2*x5*x6
dpdx3(102) = x4**2*x5*x6**2+x2**2*x5*x6**2+x4**2*x5**2*x6+x1**2*x&
&5**2*x6+2*x2**2*x3*x5*x6+2*x1**2*x3*x5*x6
dpdx3(103) = x1**2*x4*x6**2+2*x1*x3*x4**2*x6+x2**2*x4*x5**2+2*x2*&
&x3*x4**2*x5
dpdx3(104) = x1*x4**2*x6**2+2*x1**2*x3*x4*x6+x2*x4**2*x5**2+2*x2*&
&*2*x3*x4*x5
dpdx3(105) = 2*x1*x2*x3*x4**2
dpdx3(106) = x1**2*x2*x6**2+2*x2*x3*x4**2*x6+x1*x2**2*x5**2+2*x1*&
&x3*x4**2*x5
dpdx3(107) = x1*x2**2*x6**2+2*x2**2*x3*x4*x6+x1**2*x2*x5**2+2*x1*&
&*2*x3*x4*x5
dpdx3(108) = x2*x4**2*x6**2+2*x1**2*x2*x3*x6+x1*x4**2*x5**2+2*x1*&
&x2**2*x3*x5
dpdx3(109) = x2**2*x4*x6**2+2*x1*x2**2*x3*x6+x1**2*x4*x5**2+2*x1*&
&*2*x2*x3*x5
dpdx3(110) = 2*x1*x2**2*x3*x4+2*x1**2*x2*x3*x4
dpdx3(111) = x1**2*x4**2*x6+x2**2*x4**2*x5
dpdx3(112) = x1*x2**2*x4**2+x1**2*x2*x4**2
dpdx3(113) = x2**2*x4**2*x6+x1**2*x2**2*x6+x1**2*x4**2*x5+x1**2*x&
&2**2*x5
dpdx3(114) = x1**2*x2**2*x4
dpdx3(115) = x4**3*x5*x6+x2**3*x5*x6+x1**3*x5*x6
dpdx3(116) = x1*x4**3*x6+x1**3*x4*x6+x2*x4**3*x5+x2**3*x4*x5
dpdx3(117) = x1*x2*x4**3
dpdx3(118) = x2*x4**3*x6+x1**3*x2*x6+x1*x4**3*x5+x1*x2**3*x5
dpdx3(119) = x2**3*x4*x6+x1*x2**3*x6+x1**3*x4*x5+x1**3*x2*x5
dpdx3(120) = x1*x2**3*x4+x1**3*x2*x4
dpdx3(121) = 2*x3*x5**2*x6**2
dpdx3(122) = x5**2*x6**3+2*x3*x5*x6**3+x5**3*x6**2+3*x3**2*x5*x6*&
&*2+2*x3*x5**3*x6+3*x3**2*x5**2*x6
dpdx3(123) = x5*x6**4+x5**4*x6+4*x3**3*x5*x6
dpdx3(124) = x1*x6**4+4*x3**3*x4*x6+x2*x5**4+4*x3**3*x4*x5
dpdx3(125) = 2*x1*x3*x6**3+3*x3**2*x4*x6**2+2*x2*x3*x5**3+3*x3**2&
&*x4*x5**2
dpdx3(126) = 2*x3*x4*x6**3+3*x1*x3**2*x6**2+2*x3*x4*x5**3+3*x2*x3&
&**2*x5**2
dpdx3(127) = 2*x2*x3*x6**3+3*x2*x3**2*x6**2+2*x1*x3*x5**3+3*x1*x3&
&**2*x5**2
dpdx3(128) = x4*x6**4+4*x1*x3**3*x6+x4*x5**4+4*x2*x3**3*x5
dpdx3(129) = x2*x6**4+4*x2*x3**3*x6+x1*x5**4+4*x1*x3**3*x5
dpdx3(130) = 4*x2*x3**3*x4+4*x1*x3**3*x4
dpdx3(131) = 4*x1*x2*x3**3
dpdx3(132) = x1**2*x6**3+3*x3**2*x4**2*x6+x2**2*x5**3+3*x3**2*x4*&
&*2*x5
dpdx3(133) = 2*x3*x4**2*x6**2+2*x1**2*x3*x6**2+2*x3*x4**2*x5**2+2&
&*x2**2*x3*x5**2
dpdx3(134) = 2*x2**2*x3*x6**2+2*x1**2*x3*x5**2
dpdx3(135) = x4**2*x6**3+3*x1**2*x3**2*x6+x4**2*x5**3+3*x2**2*x3*&
&*2*x5
dpdx3(136) = x2**2*x6**3+3*x2**2*x3**2*x6+x1**2*x5**3+3*x1**2*x3*&
&*2*x5
dpdx3(137) = 3*x2*x3**2*x4**2+3*x1*x3**2*x4**2
dpdx3(138) = 3*x2**2*x3**2*x4+3*x1**2*x3**2*x4
dpdx3(139) = 3*x1*x2**2*x3**2+3*x1**2*x2*x3**2
dpdx3(140) = 2*x2**2*x3*x4**2+2*x1**2*x3*x4**2
dpdx3(141) = 0
dpdx3(142) = 2*x1**2*x2**2*x3
dpdx3(143) = x1**3*x6**2+2*x3*x4**3*x6+x2**3*x5**2+2*x3*x4**3*x5
dpdx3(144) = x4**3*x6**2+2*x1**3*x3*x6+x4**3*x5**2+2*x2**3*x3*x5
dpdx3(145) = x2**3*x6**2+2*x2**3*x3*x6+x1**3*x5**2+2*x1**3*x3*x5
dpdx3(146) = 2*x2*x3*x4**3+2*x1*x3*x4**3
dpdx3(147) = 2*x2**3*x3*x4+2*x1**3*x3*x4
dpdx3(148) = 2*x1*x2**3*x3+2*x1**3*x2*x3
dpdx3(149) = x2**2*x4**3+x1**2*x4**3
dpdx3(150) = x2**3*x4**2+x1**3*x4**2
dpdx3(151) = 0
dpdx3(152) = x1**2*x2**3+x1**3*x2**2
dpdx3(153) = x4**4*x6+x1**4*x6+x4**4*x5+x2**4*x5
dpdx3(154) = x2**4*x6+x1**4*x5
dpdx3(155) = x2*x4**4+x1*x4**4
dpdx3(156) = x2**4*x4+x1**4*x4
dpdx3(157) = 0
dpdx3(158) = x1*x2**4+x1**4*x2
dpdx3(159) = 3*x3**2*x6**3+3*x3**2*x5**3
dpdx3(160) = 2*x3*x6**4+4*x3**3*x6**2+2*x3*x5**4+4*x3**3*x5**2
dpdx3(161) = x6**5+5*x3**4*x6+x5**5+5*x3**4*x5
dpdx3(162) = 5*x2*x3**4+5*x1*x3**4
dpdx3(163) = 4*x2**2*x3**3+4*x1**2*x3**3
dpdx3(164) = 3*x2**3*x3**2+3*x1**3*x3**2
dpdx3(165) = 0
dpdx3(166) = 2*x2**4*x3+2*x1**4*x3
dpdx3(167) = 0
dpdx3(168) = x2**5+x1**5
dpdx3(169) = 0
dpdx3(170) = x1*x2*x4*x5*x6**2+x1*x2*x4*x5**2*x6+2*x1*x2*x3*x4*x5&
&*x6
dpdx3(171) = x1*x2*x4**2*x5*x6+x1*x2**2*x4*x5*x6+x1**2*x2*x4*x5*x&
&6
dpdx3(172) = x1*x2*x5**2*x6**2+2*x1*x3*x4*x5*x6**2+2*x2*x3*x4*x5*&
&*2*x6
dpdx3(173) = x1*x4*x5*x6**3+x1*x2*x5*x6**3+x2*x4*x5**3*x6+x1*x2*x&
&5**3*x6+3*x2*x3**2*x4*x5*x6+3*x1*x3**2*x4*x5*x6
dpdx3(174) = x2*x4*x5**2*x6**2+x1*x4*x5**2*x6**2+2*x2*x3*x4*x5*x6&
&**2+2*x1*x2*x3*x5*x6**2+2*x1*x3*x4*x5**2*x6+2*x1*x2*x3*x5**2*x6
dpdx3(175) = 2*x1*x2*x3*x4*x6**2+2*x1*x2*x3*x4*x5**2
dpdx3(176) = x2*x4*x5*x6**3+x1*x4*x5**3*x6+3*x1*x2*x3**2*x5*x6
dpdx3(177) = x1*x2*x4*x6**3+3*x1*x2*x3**2*x4*x6+x1*x2*x4*x5**3+3*&
&x1*x2*x3**2*x4*x5
dpdx3(178) = x1**2*x4*x5*x6**2+x1**2*x2*x5*x6**2+x2**2*x4*x5**2*x&
&6+x1*x2**2*x5**2*x6+2*x2*x3*x4**2*x5*x6+2*x1*x3*x4**2*x5*x6
dpdx3(179) = x1*x4**2*x5*x6**2+x1*x2**2*x5*x6**2+x2*x4**2*x5**2*x&
&6+x1**2*x2*x5**2*x6+2*x2**2*x3*x4*x5*x6+2*x1**2*x3*x4*x5*x6
dpdx3(180) = x1**2*x2*x4*x6**2+2*x1*x2*x3*x4**2*x6+x1*x2**2*x4*x5&
&**2+2*x1*x2*x3*x4**2*x5
dpdx3(181) = x2*x4**2*x5*x6**2+x2**2*x4*x5*x6**2+x1*x4**2*x5**2*x&
&6+x1**2*x4*x5**2*x6+2*x1*x2**2*x3*x5*x6+2*x1**2*x2*x3*x5*x6
dpdx3(182) = x1*x2*x4**2*x6**2+2*x1**2*x2*x3*x4*x6+x1*x2*x4**2*x5&
&**2+2*x1*x2**2*x3*x4*x5
dpdx3(183) = x1*x2**2*x4*x6**2+2*x1*x2**2*x3*x4*x6+x1**2*x2*x4*x5&
&**2+2*x1**2*x2*x3*x4*x5
dpdx3(184) = x1**2*x2*x4**2*x6+x1*x2**2*x4**2*x5
dpdx3(185) = x2**2*x4**2*x5*x6+x1**2*x4**2*x5*x6+x1**2*x2**2*x5*x&
&6
dpdx3(186) = x1*x2**2*x4**2*x6+x1**2*x2**2*x4*x6+x1**2*x2*x4**2*x&
&5+x1**2*x2**2*x4*x5
dpdx3(187) = x2*x4**3*x5*x6+x1*x4**3*x5*x6+x2**3*x4*x5*x6+x1**3*x&
&4*x5*x6+x1*x2**3*x5*x6+x1**3*x2*x5*x6
dpdx3(188) = x1*x2*x4**3*x6+x1**3*x2*x4*x6+x1*x2*x4**3*x5+x1*x2**&
&3*x4*x5
dpdx3(189) = x1*x2**3*x4*x6+x1**3*x2*x4*x5
dpdx3(190) = x1*x5*x6**4+x2*x5**4*x6+4*x3**3*x4*x5*x6
dpdx3(191) = x1*x5**2*x6**3+2*x1*x3*x5*x6**3+x2*x5**3*x6**2+3*x3*&
&*2*x4*x5*x6**2+2*x2*x3*x5**3*x6+3*x3**2*x4*x5**2*x6
dpdx3(192) = 2*x3*x4*x5**2*x6**2+2*x2*x3*x5**2*x6**2+2*x1*x3*x5**&
&2*x6**2
dpdx3(193) = x2*x5**2*x6**3+2*x3*x4*x5*x6**3+x1*x5**3*x6**2+3*x1*&
&x3**2*x5*x6**2+2*x3*x4*x5**3*x6+3*x2*x3**2*x5**2*x6
dpdx3(194) = x4*x5**2*x6**3+2*x2*x3*x5*x6**3+x4*x5**3*x6**2+3*x2*&
&x3**2*x5*x6**2+2*x1*x3*x5**3*x6+3*x1*x3**2*x5**2*x6
dpdx3(195) = x4*x5*x6**4+x2*x5*x6**4+x4*x5**4*x6+x1*x5**4*x6+4*x2&
&*x3**3*x5*x6+4*x1*x3**3*x5*x6
dpdx3(196) = 2*x1*x3*x4*x6**3+3*x1*x3**2*x4*x6**2+2*x2*x3*x4*x5**&
&3+3*x2*x3**2*x4*x5**2
dpdx3(197) = x1*x4*x6**4+4*x1*x3**3*x4*x6+x2*x4*x5**4+4*x2*x3**3*&
&x4*x5
dpdx3(198) = x1*x2*x6**4+4*x2*x3**3*x4*x6+x1*x2*x5**4+4*x1*x3**3*&
&x4*x5
dpdx3(199) = 2*x1*x2*x3*x6**3+3*x2*x3**2*x4*x6**2+2*x1*x2*x3*x5**&
&3+3*x1*x3**2*x4*x5**2
dpdx3(200) = 2*x2*x3*x4*x6**3+3*x1*x2*x3**2*x6**2+2*x1*x3*x4*x5**&
&3+3*x1*x2*x3**2*x5**2
dpdx3(201) = x2*x4*x6**4+4*x1*x2*x3**3*x6+x1*x4*x5**4+4*x1*x2*x3*&
&*3*x5
dpdx3(202) = 4*x1*x2*x3**3*x4
dpdx3(203) = x1**2*x5*x6**3+x2**2*x5**3*x6+3*x3**2*x4**2*x5*x6
dpdx3(204) = x2**2*x5**2*x6**2+x1**2*x5**2*x6**2+2*x3*x4**2*x5*x6&
&**2+2*x1**2*x3*x5*x6**2+2*x3*x4**2*x5**2*x6+2*x2**2*x3*x5**2*x6
dpdx3(205) = x4**2*x5**2*x6**2+2*x2**2*x3*x5*x6**2+2*x1**2*x3*x5*&
&*2*x6
dpdx3(206) = x4**2*x5*x6**3+x2**2*x5*x6**3+x4**2*x5**3*x6+x1**2*x&
&5**3*x6+3*x2**2*x3**2*x5*x6+3*x1**2*x3**2*x5*x6
dpdx3(207) = x1**2*x4*x6**3+3*x1*x3**2*x4**2*x6+x2**2*x4*x5**3+3*&
&x2*x3**2*x4**2*x5
dpdx3(208) = 2*x1*x3*x4**2*x6**2+2*x1**2*x3*x4*x6**2+2*x2*x3*x4**&
&2*x5**2+2*x2**2*x3*x4*x5**2
dpdx3(209) = x1*x4**2*x6**3+3*x1**2*x3**2*x4*x6+x2*x4**2*x5**3+3*&
&x2**2*x3**2*x4*x5
dpdx3(210) = 3*x1*x2*x3**2*x4**2
dpdx3(211) = x1**2*x2*x6**3+3*x2*x3**2*x4**2*x6+x1*x2**2*x5**3+3*&
&x1*x3**2*x4**2*x5
dpdx3(212) = x1*x2**2*x6**3+3*x2**2*x3**2*x4*x6+x1**2*x2*x5**3+3*&
&x1**2*x3**2*x4*x5
dpdx3(213) = 2*x2*x3*x4**2*x6**2+2*x1**2*x2*x3*x6**2+2*x1*x3*x4**&
&2*x5**2+2*x1*x2**2*x3*x5**2
dpdx3(214) = 2*x2**2*x3*x4*x6**2+2*x1*x2**2*x3*x6**2+2*x1**2*x3*x&
&4*x5**2+2*x1**2*x2*x3*x5**2
dpdx3(215) = x2*x4**2*x6**3+3*x1**2*x2*x3**2*x6+x1*x4**2*x5**3+3*&
&x1*x2**2*x3**2*x5
dpdx3(216) = x2**2*x4*x6**3+3*x1*x2**2*x3**2*x6+x1**2*x4*x5**3+3*&
&x1**2*x2*x3**2*x5
dpdx3(217) = 3*x1*x2**2*x3**2*x4+3*x1**2*x2*x3**2*x4
dpdx3(218) = x1**2*x4**2*x6**2+2*x1**2*x3*x4**2*x6+x2**2*x4**2*x5&
&**2+2*x2**2*x3*x4**2*x5
dpdx3(219) = 2*x1*x2**2*x3*x4**2+2*x1**2*x2*x3*x4**2
dpdx3(220) = x1**2*x2**2*x6**2+2*x2**2*x3*x4**2*x6+x1**2*x2**2*x5&
&**2+2*x1**2*x3*x4**2*x5
dpdx3(221) = x1**2*x2**2*x4**2
dpdx3(222) = x2**2*x4**2*x6**2+2*x1**2*x2**2*x3*x6+x1**2*x4**2*x5&
&**2+2*x1**2*x2**2*x3*x5
dpdx3(223) = 2*x1**2*x2**2*x3*x4
dpdx3(224) = x1**3*x5*x6**2+x2**3*x5**2*x6+2*x3*x4**3*x5*x6
dpdx3(225) = x4**3*x5*x6**2+x2**3*x5*x6**2+x4**3*x5**2*x6+x1**3*x&
&5**2*x6+2*x2**3*x3*x5*x6+2*x1**3*x3*x5*x6
dpdx3(226) = x1**3*x4*x6**2+2*x1*x3*x4**3*x6+x2**3*x4*x5**2+2*x2*&
&x3*x4**3*x5
dpdx3(227) = x1*x4**3*x6**2+2*x1**3*x3*x4*x6+x2*x4**3*x5**2+2*x2*&
&*3*x3*x4*x5
dpdx3(228) = 2*x1*x2*x3*x4**3
dpdx3(229) = x1**3*x2*x6**2+2*x2*x3*x4**3*x6+x1*x2**3*x5**2+2*x1*&
&x3*x4**3*x5
dpdx3(230) = x1*x2**3*x6**2+2*x2**3*x3*x4*x6+x1**3*x2*x5**2+2*x1*&
&*3*x3*x4*x5
dpdx3(231) = x2*x4**3*x6**2+2*x1**3*x2*x3*x6+x1*x4**3*x5**2+2*x1*&
&x2**3*x3*x5
dpdx3(232) = x2**3*x4*x6**2+2*x1*x2**3*x3*x6+x1**3*x4*x5**2+2*x1*&
&*3*x2*x3*x5
dpdx3(233) = 2*x1*x2**3*x3*x4+2*x1**3*x2*x3*x4
dpdx3(234) = x1**2*x4**3*x6+x1**3*x4**2*x6+x2**2*x4**3*x5+x2**3*x&
&4**2*x5
dpdx3(235) = x1*x2**2*x4**3+x1**2*x2*x4**3
dpdx3(236) = x1*x2**3*x4**2+x1**3*x2*x4**2
dpdx3(237) = x2**2*x4**3*x6+x1**3*x2**2*x6+x1**2*x4**3*x5+x1**2*x&
&2**3*x5
dpdx3(238) = x2**3*x4**2*x6+x1**2*x2**3*x6+x1**3*x4**2*x5+x1**3*x&
&2**2*x5
dpdx3(239) = x1**2*x2**3*x4+x1**3*x2**2*x4
dpdx3(240) = x4**4*x5*x6+x2**4*x5*x6+x1**4*x5*x6
dpdx3(241) = x1*x4**4*x6+x1**4*x4*x6+x2*x4**4*x5+x2**4*x4*x5
dpdx3(242) = x1*x2*x4**4
dpdx3(243) = x2*x4**4*x6+x1**4*x2*x6+x1*x4**4*x5+x1*x2**4*x5
dpdx3(244) = x2**4*x4*x6+x1*x2**4*x6+x1**4*x4*x5+x1**4*x2*x5
dpdx3(245) = x1*x2**4*x4+x1**4*x2*x4
dpdx3(246) = 2*x3*x5**2*x6**3+2*x3*x5**3*x6**2+3*x3**2*x5**2*x6**&
&2
dpdx3(247) = x5**3*x6**3+3*x3**2*x5*x6**3+3*x3**2*x5**3*x6
dpdx3(248) = x5**2*x6**4+2*x3*x5*x6**4+x5**4*x6**2+4*x3**3*x5*x6*&
&*2+2*x3*x5**4*x6+4*x3**3*x5**2*x6
dpdx3(249) = x5*x6**5+x5**5*x6+5*x3**4*x5*x6
dpdx3(250) = x1*x6**5+5*x3**4*x4*x6+x2*x5**5+5*x3**4*x4*x5
dpdx3(251) = 2*x1*x3*x6**4+4*x3**3*x4*x6**2+2*x2*x3*x5**4+4*x3**3&
&*x4*x5**2
dpdx3(252) = 3*x3**2*x4*x6**3+3*x1*x3**2*x6**3+3*x3**2*x4*x5**3+3&
&*x2*x3**2*x5**3
dpdx3(253) = 3*x2*x3**2*x6**3+3*x1*x3**2*x5**3
dpdx3(254) = 2*x3*x4*x6**4+4*x1*x3**3*x6**2+2*x3*x4*x5**4+4*x2*x3&
&**3*x5**2
dpdx3(255) = 2*x2*x3*x6**4+4*x2*x3**3*x6**2+2*x1*x3*x5**4+4*x1*x3&
&**3*x5**2
dpdx3(256) = x4*x6**5+5*x1*x3**4*x6+x4*x5**5+5*x2*x3**4*x5
dpdx3(257) = x2*x6**5+5*x2*x3**4*x6+x1*x5**5+5*x1*x3**4*x5
dpdx3(258) = 5*x2*x3**4*x4+5*x1*x3**4*x4
dpdx3(259) = 5*x1*x2*x3**4
dpdx3(260) = x1**2*x6**4+4*x3**3*x4**2*x6+x2**2*x5**4+4*x3**3*x4*&
&*2*x5
dpdx3(261) = 2*x1**2*x3*x6**3+3*x3**2*x4**2*x6**2+2*x2**2*x3*x5**&
&3+3*x3**2*x4**2*x5**2
dpdx3(262) = 2*x3*x4**2*x6**3+3*x1**2*x3**2*x6**2+2*x3*x4**2*x5**&
&3+3*x2**2*x3**2*x5**2
dpdx3(263) = 2*x2**2*x3*x6**3+3*x2**2*x3**2*x6**2+2*x1**2*x3*x5**&
&3+3*x1**2*x3**2*x5**2
dpdx3(264) = x4**2*x6**4+4*x1**2*x3**3*x6+x4**2*x5**4+4*x2**2*x3*&
&*3*x5
dpdx3(265) = x2**2*x6**4+4*x2**2*x3**3*x6+x1**2*x5**4+4*x1**2*x3*&
&*3*x5
dpdx3(266) = 4*x2*x3**3*x4**2+4*x1*x3**3*x4**2
dpdx3(267) = 4*x2**2*x3**3*x4+4*x1**2*x3**3*x4
dpdx3(268) = 4*x1*x2**2*x3**3+4*x1**2*x2*x3**3
dpdx3(269) = 3*x2**2*x3**2*x4**2+3*x1**2*x3**2*x4**2
dpdx3(270) = 3*x1**2*x2**2*x3**2
dpdx3(271) = x1**3*x6**3+3*x3**2*x4**3*x6+x2**3*x5**3+3*x3**2*x4*&
&*3*x5
dpdx3(272) = 2*x3*x4**3*x6**2+2*x1**3*x3*x6**2+2*x3*x4**3*x5**2+2&
&*x2**3*x3*x5**2
dpdx3(273) = 2*x2**3*x3*x6**2+2*x1**3*x3*x5**2
dpdx3(274) = x4**3*x6**3+3*x1**3*x3**2*x6+x4**3*x5**3+3*x2**3*x3*&
&*2*x5
dpdx3(275) = x2**3*x6**3+3*x2**3*x3**2*x6+x1**3*x5**3+3*x1**3*x3*&
&*2*x5
dpdx3(276) = 3*x2*x3**2*x4**3+3*x1*x3**2*x4**3
dpdx3(277) = 3*x2**3*x3**2*x4+3*x1**3*x3**2*x4
dpdx3(278) = 3*x1*x2**3*x3**2+3*x1**3*x2*x3**2
dpdx3(279) = 2*x2**2*x3*x4**3+2*x1**2*x3*x4**3
dpdx3(280) = 2*x2**3*x3*x4**2+2*x1**3*x3*x4**2
dpdx3(281) = 0
dpdx3(282) = 2*x1**2*x2**3*x3+2*x1**3*x2**2*x3
dpdx3(283) = x2**3*x4**3+x1**3*x4**3
dpdx3(284) = 0
dpdx3(285) = x1**3*x2**3
dpdx3(286) = x1**4*x6**2+2*x3*x4**4*x6+x2**4*x5**2+2*x3*x4**4*x5
dpdx3(287) = x4**4*x6**2+2*x1**4*x3*x6+x4**4*x5**2+2*x2**4*x3*x5
dpdx3(288) = x2**4*x6**2+2*x2**4*x3*x6+x1**4*x5**2+2*x1**4*x3*x5
dpdx3(289) = 2*x2*x3*x4**4+2*x1*x3*x4**4
dpdx3(290) = 2*x2**4*x3*x4+2*x1**4*x3*x4
dpdx3(291) = 2*x1*x2**4*x3+2*x1**4*x2*x3
dpdx3(292) = x2**2*x4**4+x1**2*x4**4
dpdx3(293) = x2**4*x4**2+x1**4*x4**2
dpdx3(294) = 0
dpdx3(295) = x1**2*x2**4+x1**4*x2**2
dpdx3(296) = x4**5*x6+x1**5*x6+x4**5*x5+x2**5*x5
dpdx3(297) = x2**5*x6+x1**5*x5
dpdx3(298) = x2*x4**5+x1*x4**5
dpdx3(299) = x2**5*x4+x1**5*x4
dpdx3(300) = 0
dpdx3(301) = x1*x2**5+x1**5*x2
dpdx3(302) = 3*x3**2*x6**4+4*x3**3*x6**3+3*x3**2*x5**4+4*x3**3*x5&
&**3
dpdx3(303) = 2*x3*x6**5+5*x3**4*x6**2+2*x3*x5**5+5*x3**4*x5**2
dpdx3(304) = x6**6+6*x3**5*x6+x5**6+6*x3**5*x5
dpdx3(305) = 6*x2*x3**5+6*x1*x3**5
dpdx3(306) = 5*x2**2*x3**4+5*x1**2*x3**4
dpdx3(307) = 4*x2**3*x3**3+4*x1**3*x3**3
dpdx3(308) = 3*x2**4*x3**2+3*x1**4*x3**2
dpdx3(309) = 0
dpdx3(310) = 2*x2**5*x3+2*x1**5*x3
dpdx3(311) = 0
dpdx3(312) = x2**6+x1**6
dpdx3(313) = 0
dpdx3(314) = x1*x2*x4*x5**2*x6**2+2*x1*x2*x3*x4*x5*x6**2+2*x1*x2*&
&x3*x4*x5**2*x6
dpdx3(315) = x1*x2*x4*x5*x6**3+x1*x2*x4*x5**3*x6+3*x1*x2*x3**2*x4&
&*x5*x6
dpdx3(316) = x1**2*x2*x4*x5*x6**2+x1*x2**2*x4*x5**2*x6+2*x1*x2*x3&
&*x4**2*x5*x6
dpdx3(317) = x1*x2*x4**2*x5*x6**2+x1*x2**2*x4*x5*x6**2+x1*x2*x4**&
&2*x5**2*x6+x1**2*x2*x4*x5**2*x6+2*x1*x2**2*x3*x4*x5*x6+2*x1**2*x2&
&*x3*x4*x5*x6
dpdx3(318) = x1*x2**2*x4**2*x5*x6+x1**2*x2*x4**2*x5*x6+x1**2*x2**&
&2*x4*x5*x6
dpdx3(319) = x1*x2*x4**3*x5*x6+x1*x2**3*x4*x5*x6+x1**3*x2*x4*x5*x&
&6
dpdx3(320) = x1*x2*x5**2*x6**3+2*x1*x3*x4*x5*x6**3+x1*x2*x5**3*x6&
&**2+3*x1*x3**2*x4*x5*x6**2+2*x2*x3*x4*x5**3*x6+3*x2*x3**2*x4*x5**&
&2*x6
dpdx3(321) = x1*x4*x5*x6**4+x1*x2*x5*x6**4+x2*x4*x5**4*x6+x1*x2*x&
&5**4*x6+4*x2*x3**3*x4*x5*x6+4*x1*x3**3*x4*x5*x6
dpdx3(322) = 2*x2*x3*x4*x5**2*x6**2+2*x1*x3*x4*x5**2*x6**2+2*x1*x&
&2*x3*x5**2*x6**2
dpdx3(323) = x1*x4*x5**2*x6**3+2*x1*x2*x3*x5*x6**3+x2*x4*x5**3*x6&
&**2+3*x2*x3**2*x4*x5*x6**2+2*x1*x2*x3*x5**3*x6+3*x1*x3**2*x4*x5**&
&2*x6
dpdx3(324) = x2*x4*x5**2*x6**3+2*x2*x3*x4*x5*x6**3+x1*x4*x5**3*x6&
&**2+3*x1*x2*x3**2*x5*x6**2+2*x1*x3*x4*x5**3*x6+3*x1*x2*x3**2*x5**&
&2*x6
dpdx3(325) = 2*x1*x2*x3*x4*x6**3+3*x1*x2*x3**2*x4*x6**2+2*x1*x2*x&
&3*x4*x5**3+3*x1*x2*x3**2*x4*x5**2
dpdx3(326) = x2*x4*x5*x6**4+x1*x4*x5**4*x6+4*x1*x2*x3**3*x5*x6
dpdx3(327) = x1*x2*x4*x6**4+4*x1*x2*x3**3*x4*x6+x1*x2*x4*x5**4+4*&
&x1*x2*x3**3*x4*x5
dpdx3(328) = x1**2*x4*x5*x6**3+x1**2*x2*x5*x6**3+x2**2*x4*x5**3*x&
&6+x1*x2**2*x5**3*x6+3*x2*x3**2*x4**2*x5*x6+3*x1*x3**2*x4**2*x5*x6
dpdx3(329) = x1*x2**2*x5**2*x6**2+x1**2*x2*x5**2*x6**2+2*x1*x3*x4&
&**2*x5*x6**2+2*x1**2*x3*x4*x5*x6**2+2*x2*x3*x4**2*x5**2*x6+2*x2**&
&2*x3*x4*x5**2*x6
dpdx3(330) = x1*x4**2*x5*x6**3+x1*x2**2*x5*x6**3+x2*x4**2*x5**3*x&
&6+x1**2*x2*x5**3*x6+3*x2**2*x3**2*x4*x5*x6+3*x1**2*x3**2*x4*x5*x6
dpdx3(331) = x1**2*x2*x4*x6**3+3*x1*x2*x3**2*x4**2*x6+x1*x2**2*x4&
&*x5**3+3*x1*x2*x3**2*x4**2*x5
dpdx3(332) = x2**2*x4*x5**2*x6**2+x1**2*x4*x5**2*x6**2+2*x2*x3*x4&
&**2*x5*x6**2+2*x1**2*x2*x3*x5*x6**2+2*x1*x3*x4**2*x5**2*x6+2*x1*x&
&2**2*x3*x5**2*x6
dpdx3(333) = x2*x4**2*x5**2*x6**2+x1*x4**2*x5**2*x6**2+2*x2**2*x3&
&*x4*x5*x6**2+2*x1*x2**2*x3*x5*x6**2+2*x1**2*x3*x4*x5**2*x6+2*x1**&
&2*x2*x3*x5**2*x6
dpdx3(334) = 2*x1*x2*x3*x4**2*x6**2+2*x1**2*x2*x3*x4*x6**2+2*x1*x&
&2*x3*x4**2*x5**2+2*x1*x2**2*x3*x4*x5**2
dpdx3(335) = 2*x1*x2**2*x3*x4*x6**2+2*x1**2*x2*x3*x4*x5**2
dpdx3(336) = x2*x4**2*x5*x6**3+x2**2*x4*x5*x6**3+x1*x4**2*x5**3*x&
&6+x1**2*x4*x5**3*x6+3*x1*x2**2*x3**2*x5*x6+3*x1**2*x2*x3**2*x5*x6
dpdx3(337) = x1*x2*x4**2*x6**3+3*x1**2*x2*x3**2*x4*x6+x1*x2*x4**2&
&*x5**3+3*x1*x2**2*x3**2*x4*x5
dpdx3(338) = x1*x2**2*x4*x6**3+3*x1*x2**2*x3**2*x4*x6+x1**2*x2*x4&
&*x5**3+3*x1**2*x2*x3**2*x4*x5
dpdx3(339) = x1**2*x2*x4**2*x6**2+2*x1**2*x2*x3*x4**2*x6+x1*x2**2&
&*x4**2*x5**2+2*x1*x2**2*x3*x4**2*x5
dpdx3(340) = x1**2*x4**2*x5*x6**2+x1**2*x2**2*x5*x6**2+x2**2*x4**&
&2*x5**2*x6+x1**2*x2**2*x5**2*x6+2*x2**2*x3*x4**2*x5*x6+2*x1**2*x3&
&*x4**2*x5*x6
dpdx3(341) = x1**2*x2**2*x4*x6**2+2*x1*x2**2*x3*x4**2*x6+x1**2*x2&
&**2*x4*x5**2+2*x1**2*x2*x3*x4**2*x5
dpdx3(342) = x1**2*x2**2*x4**2*x6+x1**2*x2**2*x4**2*x5
dpdx3(343) = x2**2*x4**2*x5*x6**2+x1**2*x4**2*x5**2*x6+2*x1**2*x2&
&**2*x3*x5*x6
dpdx3(344) = x1*x2**2*x4**2*x6**2+2*x1**2*x2**2*x3*x4*x6+x1**2*x2&
&*x4**2*x5**2+2*x1**2*x2**2*x3*x4*x5
dpdx3(345) = x1**3*x4*x5*x6**2+x1**3*x2*x5*x6**2+x2**3*x4*x5**2*x&
&6+x1*x2**3*x5**2*x6+2*x2*x3*x4**3*x5*x6+2*x1*x3*x4**3*x5*x6
dpdx3(346) = x1*x4**3*x5*x6**2+x1*x2**3*x5*x6**2+x2*x4**3*x5**2*x&
&6+x1**3*x2*x5**2*x6+2*x2**3*x3*x4*x5*x6+2*x1**3*x3*x4*x5*x6
dpdx3(347) = x1**3*x2*x4*x6**2+2*x1*x2*x3*x4**3*x6+x1*x2**3*x4*x5&
&**2+2*x1*x2*x3*x4**3*x5
dpdx3(348) = x2*x4**3*x5*x6**2+x2**3*x4*x5*x6**2+x1*x4**3*x5**2*x&
&6+x1**3*x4*x5**2*x6+2*x1*x2**3*x3*x5*x6+2*x1**3*x2*x3*x5*x6
dpdx3(349) = x1*x2*x4**3*x6**2+2*x1**3*x2*x3*x4*x6+x1*x2*x4**3*x5&
&**2+2*x1*x2**3*x3*x4*x5
dpdx3(350) = x1*x2**3*x4*x6**2+2*x1*x2**3*x3*x4*x6+x1**3*x2*x4*x5&
&**2+2*x1**3*x2*x3*x4*x5
dpdx3(351) = x1**2*x2*x4**3*x6+x1**3*x2*x4**2*x6+x1*x2**2*x4**3*x&
&5+x1*x2**3*x4**2*x5
dpdx3(352) = x2**2*x4**3*x5*x6+x1**2*x4**3*x5*x6+x2**3*x4**2*x5*x&
&6+x1**3*x4**2*x5*x6+x1**2*x2**3*x5*x6+x1**3*x2**2*x5*x6
dpdx3(353) = x1*x2**2*x4**3*x6+x1**3*x2**2*x4*x6+x1**2*x2*x4**3*x&
&5+x1**2*x2**3*x4*x5
dpdx3(354) = x1*x2**3*x4**2*x6+x1**2*x2**3*x4*x6+x1**3*x2*x4**2*x&
&5+x1**3*x2**2*x4*x5
dpdx3(355) = x2*x4**4*x5*x6+x1*x4**4*x5*x6+x2**4*x4*x5*x6+x1**4*x&
&4*x5*x6+x1*x2**4*x5*x6+x1**4*x2*x5*x6
dpdx3(356) = x1*x2*x4**4*x6+x1**4*x2*x4*x6+x1*x2*x4**4*x5+x1*x2**&
&4*x4*x5
dpdx3(357) = x1*x2**4*x4*x6+x1**4*x2*x4*x5
dpdx3(358) = x1*x5*x6**5+x2*x5**5*x6+5*x3**4*x4*x5*x6
dpdx3(359) = x1*x5**2*x6**4+2*x1*x3*x5*x6**4+x2*x5**4*x6**2+4*x3*&
&*3*x4*x5*x6**2+2*x2*x3*x5**4*x6+4*x3**3*x4*x5**2*x6
dpdx3(360) = 2*x1*x3*x5**2*x6**3+2*x2*x3*x5**3*x6**2+3*x3**2*x4*x&
&5**2*x6**2
dpdx3(361) = x2*x5**3*x6**3+x1*x5**3*x6**3+3*x3**2*x4*x5*x6**3+3*&
&x1*x3**2*x5*x6**3+3*x3**2*x4*x5**3*x6+3*x2*x3**2*x5**3*x6
dpdx3(362) = 2*x3*x4*x5**2*x6**3+2*x2*x3*x5**2*x6**3+2*x3*x4*x5**&
&3*x6**2+2*x1*x3*x5**3*x6**2+3*x2*x3**2*x5**2*x6**2+3*x1*x3**2*x5*&
&*2*x6**2
dpdx3(363) = x4*x5**3*x6**3+3*x2*x3**2*x5*x6**3+3*x1*x3**2*x5**3*&
&x6
dpdx3(364) = x2*x5**2*x6**4+2*x3*x4*x5*x6**4+x1*x5**4*x6**2+4*x1*&
&x3**3*x5*x6**2+2*x3*x4*x5**4*x6+4*x2*x3**3*x5**2*x6
dpdx3(365) = x4*x5**2*x6**4+2*x2*x3*x5*x6**4+x4*x5**4*x6**2+4*x2*&
&x3**3*x5*x6**2+2*x1*x3*x5**4*x6+4*x1*x3**3*x5**2*x6
dpdx3(366) = x4*x5*x6**5+x2*x5*x6**5+x4*x5**5*x6+x1*x5**5*x6+5*x2&
&*x3**4*x5*x6+5*x1*x3**4*x5*x6
dpdx3(367) = 3*x1*x3**2*x4*x6**3+3*x2*x3**2*x4*x5**3
dpdx3(368) = 2*x1*x3*x4*x6**4+4*x1*x3**3*x4*x6**2+2*x2*x3*x4*x5**&
&4+4*x2*x3**3*x4*x5**2
dpdx3(369) = x1*x4*x6**5+5*x1*x3**4*x4*x6+x2*x4*x5**5+5*x2*x3**4*&
&x4*x5
dpdx3(370) = x1*x2*x6**5+5*x2*x3**4*x4*x6+x1*x2*x5**5+5*x1*x3**4*&
&x4*x5
dpdx3(371) = 2*x1*x2*x3*x6**4+4*x2*x3**3*x4*x6**2+2*x1*x2*x3*x5**&
&4+4*x1*x3**3*x4*x5**2
dpdx3(372) = 3*x2*x3**2*x4*x6**3+3*x1*x2*x3**2*x6**3+3*x1*x3**2*x&
&4*x5**3+3*x1*x2*x3**2*x5**3
dpdx3(373) = 2*x2*x3*x4*x6**4+4*x1*x2*x3**3*x6**2+2*x1*x3*x4*x5**&
&4+4*x1*x2*x3**3*x5**2
dpdx3(374) = x2*x4*x6**5+5*x1*x2*x3**4*x6+x1*x4*x5**5+5*x1*x2*x3*&
&*4*x5
dpdx3(375) = 5*x1*x2*x3**4*x4
dpdx3(376) = x1**2*x5*x6**4+x2**2*x5**4*x6+4*x3**3*x4**2*x5*x6
dpdx3(377) = x1**2*x5**2*x6**3+2*x1**2*x3*x5*x6**3+x2**2*x5**3*x6&
&**2+3*x3**2*x4**2*x5*x6**2+2*x2**2*x3*x5**3*x6+3*x3**2*x4**2*x5**&
&2*x6
dpdx3(378) = 2*x3*x4**2*x5**2*x6**2+2*x2**2*x3*x5**2*x6**2+2*x1**&
&2*x3*x5**2*x6**2
dpdx3(379) = x2**2*x5**2*x6**3+2*x3*x4**2*x5*x6**3+x1**2*x5**3*x6&
&**2+3*x1**2*x3**2*x5*x6**2+2*x3*x4**2*x5**3*x6+3*x2**2*x3**2*x5**&
&2*x6
dpdx3(380) = x4**2*x5**2*x6**3+2*x2**2*x3*x5*x6**3+x4**2*x5**3*x6&
&**2+3*x2**2*x3**2*x5*x6**2+2*x1**2*x3*x5**3*x6+3*x1**2*x3**2*x5**&
&2*x6
dpdx3(381) = x4**2*x5*x6**4+x2**2*x5*x6**4+x4**2*x5**4*x6+x1**2*x&
&5**4*x6+4*x2**2*x3**3*x5*x6+4*x1**2*x3**3*x5*x6
dpdx3(382) = x1**2*x4*x6**4+4*x1*x3**3*x4**2*x6+x2**2*x4*x5**4+4*&
&x2*x3**3*x4**2*x5
dpdx3(383) = 2*x1**2*x3*x4*x6**3+3*x1*x3**2*x4**2*x6**2+2*x2**2*x&
&3*x4*x5**3+3*x2*x3**2*x4**2*x5**2
dpdx3(384) = 2*x1*x3*x4**2*x6**3+3*x1**2*x3**2*x4*x6**2+2*x2*x3*x&
&4**2*x5**3+3*x2**2*x3**2*x4*x5**2
dpdx3(385) = x1*x4**2*x6**4+4*x1**2*x3**3*x4*x6+x2*x4**2*x5**4+4*&
&x2**2*x3**3*x4*x5
dpdx3(386) = 4*x1*x2*x3**3*x4**2
dpdx3(387) = x1**2*x2*x6**4+4*x2*x3**3*x4**2*x6+x1*x2**2*x5**4+4*&
&x1*x3**3*x4**2*x5
dpdx3(388) = x1*x2**2*x6**4+4*x2**2*x3**3*x4*x6+x1**2*x2*x5**4+4*&
&x1**2*x3**3*x4*x5
dpdx3(389) = 2*x1**2*x2*x3*x6**3+3*x2*x3**2*x4**2*x6**2+2*x1*x2**&
&2*x3*x5**3+3*x1*x3**2*x4**2*x5**2
dpdx3(390) = 2*x1*x2**2*x3*x6**3+3*x2**2*x3**2*x4*x6**2+2*x1**2*x&
&2*x3*x5**3+3*x1**2*x3**2*x4*x5**2
dpdx3(391) = 2*x2*x3*x4**2*x6**3+3*x1**2*x2*x3**2*x6**2+2*x1*x3*x&
&4**2*x5**3+3*x1*x2**2*x3**2*x5**2
dpdx3(392) = 2*x2**2*x3*x4*x6**3+3*x1*x2**2*x3**2*x6**2+2*x1**2*x&
&3*x4*x5**3+3*x1**2*x2*x3**2*x5**2
dpdx3(393) = x2*x4**2*x6**4+4*x1**2*x2*x3**3*x6+x1*x4**2*x5**4+4*&
&x1*x2**2*x3**3*x5
dpdx3(394) = x2**2*x4*x6**4+4*x1*x2**2*x3**3*x6+x1**2*x4*x5**4+4*&
&x1**2*x2*x3**3*x5
dpdx3(395) = 4*x1*x2**2*x3**3*x4+4*x1**2*x2*x3**3*x4
dpdx3(396) = 2*x1**2*x3*x4**2*x6**2+2*x2**2*x3*x4**2*x5**2
dpdx3(397) = x1**2*x4**2*x6**3+3*x1**2*x3**2*x4**2*x6+x2**2*x4**2&
&*x5**3+3*x2**2*x3**2*x4**2*x5
dpdx3(398) = 3*x1*x2**2*x3**2*x4**2+3*x1**2*x2*x3**2*x4**2
dpdx3(399) = x1**2*x2**2*x6**3+3*x2**2*x3**2*x4**2*x6+x1**2*x2**2&
&*x5**3+3*x1**2*x3**2*x4**2*x5
dpdx3(400) = 2*x2**2*x3*x4**2*x6**2+2*x1**2*x2**2*x3*x6**2+2*x1**&
&2*x3*x4**2*x5**2+2*x1**2*x2**2*x3*x5**2
dpdx3(401) = 2*x1**2*x2**2*x3*x4**2
dpdx3(402) = x2**2*x4**2*x6**3+3*x1**2*x2**2*x3**2*x6+x1**2*x4**2&
&*x5**3+3*x1**2*x2**2*x3**2*x5
dpdx3(403) = 3*x1**2*x2**2*x3**2*x4
dpdx3(404) = x1**3*x5*x6**3+x2**3*x5**3*x6+3*x3**2*x4**3*x5*x6
dpdx3(405) = x2**3*x5**2*x6**2+x1**3*x5**2*x6**2+2*x3*x4**3*x5*x6&
&**2+2*x1**3*x3*x5*x6**2+2*x3*x4**3*x5**2*x6+2*x2**3*x3*x5**2*x6
dpdx3(406) = x4**3*x5**2*x6**2+2*x2**3*x3*x5*x6**2+2*x1**3*x3*x5*&
&*2*x6
dpdx3(407) = x4**3*x5*x6**3+x2**3*x5*x6**3+x4**3*x5**3*x6+x1**3*x&
&5**3*x6+3*x2**3*x3**2*x5*x6+3*x1**3*x3**2*x5*x6
dpdx3(408) = x1**3*x4*x6**3+3*x1*x3**2*x4**3*x6+x2**3*x4*x5**3+3*&
&x2*x3**2*x4**3*x5
dpdx3(409) = 2*x1*x3*x4**3*x6**2+2*x1**3*x3*x4*x6**2+2*x2*x3*x4**&
&3*x5**2+2*x2**3*x3*x4*x5**2
dpdx3(410) = x1*x4**3*x6**3+3*x1**3*x3**2*x4*x6+x2*x4**3*x5**3+3*&
&x2**3*x3**2*x4*x5
dpdx3(411) = 3*x1*x2*x3**2*x4**3
dpdx3(412) = x1**3*x2*x6**3+3*x2*x3**2*x4**3*x6+x1*x2**3*x5**3+3*&
&x1*x3**2*x4**3*x5
dpdx3(413) = x1*x2**3*x6**3+3*x2**3*x3**2*x4*x6+x1**3*x2*x5**3+3*&
&x1**3*x3**2*x4*x5
dpdx3(414) = 2*x2*x3*x4**3*x6**2+2*x1**3*x2*x3*x6**2+2*x1*x3*x4**&
&3*x5**2+2*x1*x2**3*x3*x5**2
dpdx3(415) = 2*x2**3*x3*x4*x6**2+2*x1*x2**3*x3*x6**2+2*x1**3*x3*x&
&4*x5**2+2*x1**3*x2*x3*x5**2
dpdx3(416) = x2*x4**3*x6**3+3*x1**3*x2*x3**2*x6+x1*x4**3*x5**3+3*&
&x1*x2**3*x3**2*x5
dpdx3(417) = x2**3*x4*x6**3+3*x1*x2**3*x3**2*x6+x1**3*x4*x5**3+3*&
&x1**3*x2*x3**2*x5
dpdx3(418) = 3*x1*x2**3*x3**2*x4+3*x1**3*x2*x3**2*x4
dpdx3(419) = x1**3*x4**2*x6**2+2*x1**2*x3*x4**3*x6+x2**3*x4**2*x5&
&**2+2*x2**2*x3*x4**3*x5
dpdx3(420) = x1**2*x4**3*x6**2+2*x1**3*x3*x4**2*x6+x2**2*x4**3*x5&
&**2+2*x2**3*x3*x4**2*x5
dpdx3(421) = 2*x1*x2**2*x3*x4**3+2*x1**2*x2*x3*x4**3
dpdx3(422) = 2*x1*x2**3*x3*x4**2+2*x1**3*x2*x3*x4**2
dpdx3(423) = x1**2*x2**2*x4**3
dpdx3(424) = x1**3*x2**2*x6**2+2*x2**2*x3*x4**3*x6+x1**2*x2**3*x5&
&**2+2*x1**2*x3*x4**3*x5
dpdx3(425) = x1**2*x2**3*x6**2+2*x2**3*x3*x4**2*x6+x1**3*x2**2*x5&
&**2+2*x1**3*x3*x4**2*x5
dpdx3(426) = x1**2*x2**3*x4**2+x1**3*x2**2*x4**2
dpdx3(427) = x2**2*x4**3*x6**2+2*x1**3*x2**2*x3*x6+x1**2*x4**3*x5&
&**2+2*x1**2*x2**3*x3*x5
dpdx3(428) = x2**3*x4**2*x6**2+2*x1**2*x2**3*x3*x6+x1**3*x4**2*x5&
&**2+2*x1**3*x2**2*x3*x5
dpdx3(429) = 2*x1**2*x2**3*x3*x4+2*x1**3*x2**2*x3*x4
dpdx3(430) = x1**3*x4**3*x6+x2**3*x4**3*x5
dpdx3(431) = x1*x2**3*x4**3+x1**3*x2*x4**3
dpdx3(432) = x2**3*x4**3*x6+x1**3*x2**3*x6+x1**3*x4**3*x5+x1**3*x&
&2**3*x5
dpdx3(433) = x1**3*x2**3*x4
dpdx3(434) = x1**4*x5*x6**2+x2**4*x5**2*x6+2*x3*x4**4*x5*x6
dpdx3(435) = x4**4*x5*x6**2+x2**4*x5*x6**2+x4**4*x5**2*x6+x1**4*x&
&5**2*x6+2*x2**4*x3*x5*x6+2*x1**4*x3*x5*x6
dpdx3(436) = x1**4*x4*x6**2+2*x1*x3*x4**4*x6+x2**4*x4*x5**2+2*x2*&
&x3*x4**4*x5
dpdx3(437) = x1*x4**4*x6**2+2*x1**4*x3*x4*x6+x2*x4**4*x5**2+2*x2*&
&*4*x3*x4*x5
dpdx3(438) = 2*x1*x2*x3*x4**4
dpdx3(439) = x1**4*x2*x6**2+2*x2*x3*x4**4*x6+x1*x2**4*x5**2+2*x1*&
&x3*x4**4*x5
dpdx3(440) = x1*x2**4*x6**2+2*x2**4*x3*x4*x6+x1**4*x2*x5**2+2*x1*&
&*4*x3*x4*x5
dpdx3(441) = x2*x4**4*x6**2+2*x1**4*x2*x3*x6+x1*x4**4*x5**2+2*x1*&
&x2**4*x3*x5
dpdx3(442) = x2**4*x4*x6**2+2*x1*x2**4*x3*x6+x1**4*x4*x5**2+2*x1*&
&*4*x2*x3*x5
dpdx3(443) = 2*x1*x2**4*x3*x4+2*x1**4*x2*x3*x4
dpdx3(444) = x1**2*x4**4*x6+x1**4*x4**2*x6+x2**2*x4**4*x5+x2**4*x&
&4**2*x5
dpdx3(445) = x1*x2**2*x4**4+x1**2*x2*x4**4
dpdx3(446) = x1*x2**4*x4**2+x1**4*x2*x4**2
dpdx3(447) = x2**2*x4**4*x6+x1**4*x2**2*x6+x1**2*x4**4*x5+x1**2*x&
&2**4*x5
dpdx3(448) = x2**4*x4**2*x6+x1**2*x2**4*x6+x1**4*x4**2*x5+x1**4*x&
&2**2*x5
dpdx3(449) = x1**2*x2**4*x4+x1**4*x2**2*x4
dpdx3(450) = x4**5*x5*x6+x2**5*x5*x6+x1**5*x5*x6
dpdx3(451) = x1*x4**5*x6+x1**5*x4*x6+x2*x4**5*x5+x2**5*x4*x5
dpdx3(452) = x1*x2*x4**5
dpdx3(453) = x2*x4**5*x6+x1**5*x2*x6+x1*x4**5*x5+x1*x2**5*x5
dpdx3(454) = x2**5*x4*x6+x1*x2**5*x6+x1**5*x4*x5+x1**5*x2*x5
dpdx3(455) = x1*x2**5*x4+x1**5*x2*x4
dpdx3(456) = 2*x3*x5**3*x6**3+3*x3**2*x5**2*x6**3+3*x3**2*x5**3*x&
&6**2
dpdx3(457) = 2*x3*x5**2*x6**4+2*x3*x5**4*x6**2+4*x3**3*x5**2*x6**&
&2
dpdx3(458) = x5**3*x6**4+3*x3**2*x5*x6**4+x5**4*x6**3+4*x3**3*x5*&
&x6**3+3*x3**2*x5**4*x6+4*x3**3*x5**3*x6
dpdx3(459) = x5**2*x6**5+2*x3*x5*x6**5+x5**5*x6**2+5*x3**4*x5*x6*&
&*2+2*x3*x5**5*x6+5*x3**4*x5**2*x6
dpdx3(460) = x5*x6**6+x5**6*x6+6*x3**5*x5*x6
dpdx3(461) = x1*x6**6+6*x3**5*x4*x6+x2*x5**6+6*x3**5*x4*x5
dpdx3(462) = 2*x1*x3*x6**5+5*x3**4*x4*x6**2+2*x2*x3*x5**5+5*x3**4&
&*x4*x5**2
dpdx3(463) = 3*x1*x3**2*x6**4+4*x3**3*x4*x6**3+3*x2*x3**2*x5**4+4&
&*x3**3*x4*x5**3
dpdx3(464) = 3*x3**2*x4*x6**4+4*x1*x3**3*x6**3+3*x3**2*x4*x5**4+4&
&*x2*x3**3*x5**3
dpdx3(465) = 3*x2*x3**2*x6**4+4*x2*x3**3*x6**3+3*x1*x3**2*x5**4+4&
&*x1*x3**3*x5**3
dpdx3(466) = 2*x3*x4*x6**5+5*x1*x3**4*x6**2+2*x3*x4*x5**5+5*x2*x3&
&**4*x5**2
dpdx3(467) = 2*x2*x3*x6**5+5*x2*x3**4*x6**2+2*x1*x3*x5**5+5*x1*x3&
&**4*x5**2
dpdx3(468) = x4*x6**6+6*x1*x3**5*x6+x4*x5**6+6*x2*x3**5*x5
dpdx3(469) = x2*x6**6+6*x2*x3**5*x6+x1*x5**6+6*x1*x3**5*x5
dpdx3(470) = 6*x2*x3**5*x4+6*x1*x3**5*x4
dpdx3(471) = 6*x1*x2*x3**5
dpdx3(472) = x1**2*x6**5+5*x3**4*x4**2*x6+x2**2*x5**5+5*x3**4*x4*&
&*2*x5
dpdx3(473) = 2*x1**2*x3*x6**4+4*x3**3*x4**2*x6**2+2*x2**2*x3*x5**&
&4+4*x3**3*x4**2*x5**2
dpdx3(474) = 3*x3**2*x4**2*x6**3+3*x1**2*x3**2*x6**3+3*x3**2*x4**&
&2*x5**3+3*x2**2*x3**2*x5**3
dpdx3(475) = 3*x2**2*x3**2*x6**3+3*x1**2*x3**2*x5**3
dpdx3(476) = 2*x3*x4**2*x6**4+4*x1**2*x3**3*x6**2+2*x3*x4**2*x5**&
&4+4*x2**2*x3**3*x5**2
dpdx3(477) = 2*x2**2*x3*x6**4+4*x2**2*x3**3*x6**2+2*x1**2*x3*x5**&
&4+4*x1**2*x3**3*x5**2
dpdx3(478) = x4**2*x6**5+5*x1**2*x3**4*x6+x4**2*x5**5+5*x2**2*x3*&
&*4*x5
dpdx3(479) = x2**2*x6**5+5*x2**2*x3**4*x6+x1**2*x5**5+5*x1**2*x3*&
&*4*x5
dpdx3(480) = 5*x2*x3**4*x4**2+5*x1*x3**4*x4**2
dpdx3(481) = 5*x2**2*x3**4*x4+5*x1**2*x3**4*x4
dpdx3(482) = 5*x1*x2**2*x3**4+5*x1**2*x2*x3**4
dpdx3(483) = 4*x2**2*x3**3*x4**2+4*x1**2*x3**3*x4**2
dpdx3(484) = 4*x1**2*x2**2*x3**3
dpdx3(485) = x1**3*x6**4+4*x3**3*x4**3*x6+x2**3*x5**4+4*x3**3*x4*&
&*3*x5
dpdx3(486) = 2*x1**3*x3*x6**3+3*x3**2*x4**3*x6**2+2*x2**3*x3*x5**&
&3+3*x3**2*x4**3*x5**2
dpdx3(487) = 2*x3*x4**3*x6**3+3*x1**3*x3**2*x6**2+2*x3*x4**3*x5**&
&3+3*x2**3*x3**2*x5**2
dpdx3(488) = 2*x2**3*x3*x6**3+3*x2**3*x3**2*x6**2+2*x1**3*x3*x5**&
&3+3*x1**3*x3**2*x5**2
dpdx3(489) = x4**3*x6**4+4*x1**3*x3**3*x6+x4**3*x5**4+4*x2**3*x3*&
&*3*x5
dpdx3(490) = x2**3*x6**4+4*x2**3*x3**3*x6+x1**3*x5**4+4*x1**3*x3*&
&*3*x5
dpdx3(491) = 4*x2*x3**3*x4**3+4*x1*x3**3*x4**3
dpdx3(492) = 4*x2**3*x3**3*x4+4*x1**3*x3**3*x4
dpdx3(493) = 4*x1*x2**3*x3**3+4*x1**3*x2*x3**3
dpdx3(494) = 3*x2**2*x3**2*x4**3+3*x1**2*x3**2*x4**3
dpdx3(495) = 3*x2**3*x3**2*x4**2+3*x1**3*x3**2*x4**2
dpdx3(496) = 3*x1**2*x2**3*x3**2+3*x1**3*x2**2*x3**2
dpdx3(497) = 2*x2**3*x3*x4**3+2*x1**3*x3*x4**3
dpdx3(498) = 0
dpdx3(499) = 2*x1**3*x2**3*x3
dpdx3(500) = x1**4*x6**3+3*x3**2*x4**4*x6+x2**4*x5**3+3*x3**2*x4*&
&*4*x5
dpdx3(501) = 2*x3*x4**4*x6**2+2*x1**4*x3*x6**2+2*x3*x4**4*x5**2+2&
&*x2**4*x3*x5**2
dpdx3(502) = 2*x2**4*x3*x6**2+2*x1**4*x3*x5**2
dpdx3(503) = x4**4*x6**3+3*x1**4*x3**2*x6+x4**4*x5**3+3*x2**4*x3*&
&*2*x5
dpdx3(504) = x2**4*x6**3+3*x2**4*x3**2*x6+x1**4*x5**3+3*x1**4*x3*&
&*2*x5
dpdx3(505) = 3*x2*x3**2*x4**4+3*x1*x3**2*x4**4
dpdx3(506) = 3*x2**4*x3**2*x4+3*x1**4*x3**2*x4
dpdx3(507) = 3*x1*x2**4*x3**2+3*x1**4*x2*x3**2
dpdx3(508) = 2*x2**2*x3*x4**4+2*x1**2*x3*x4**4
dpdx3(509) = 2*x2**4*x3*x4**2+2*x1**4*x3*x4**2
dpdx3(510) = 0
dpdx3(511) = 2*x1**2*x2**4*x3+2*x1**4*x2**2*x3
dpdx3(512) = x2**3*x4**4+x1**3*x4**4
dpdx3(513) = x2**4*x4**3+x1**4*x4**3
dpdx3(514) = 0
dpdx3(515) = x1**3*x2**4+x1**4*x2**3
dpdx3(516) = x1**5*x6**2+2*x3*x4**5*x6+x2**5*x5**2+2*x3*x4**5*x5
dpdx3(517) = x4**5*x6**2+2*x1**5*x3*x6+x4**5*x5**2+2*x2**5*x3*x5
dpdx3(518) = x2**5*x6**2+2*x2**5*x3*x6+x1**5*x5**2+2*x1**5*x3*x5
dpdx3(519) = 2*x2*x3*x4**5+2*x1*x3*x4**5
dpdx3(520) = 2*x2**5*x3*x4+2*x1**5*x3*x4
dpdx3(521) = 2*x1*x2**5*x3+2*x1**5*x2*x3
dpdx3(522) = x2**2*x4**5+x1**2*x4**5
dpdx3(523) = x2**5*x4**2+x1**5*x4**2
dpdx3(524) = 0
dpdx3(525) = x1**2*x2**5+x1**5*x2**2
dpdx3(526) = x4**6*x6+x1**6*x6+x4**6*x5+x2**6*x5
dpdx3(527) = x2**6*x6+x1**6*x5
dpdx3(528) = x2*x4**6+x1*x4**6
dpdx3(529) = x2**6*x4+x1**6*x4
dpdx3(530) = 0
dpdx3(531) = x1*x2**6+x1**6*x2
dpdx3(532) = 4*x3**3*x6**4+4*x3**3*x5**4
dpdx3(533) = 3*x3**2*x6**5+5*x3**4*x6**3+3*x3**2*x5**5+5*x3**4*x5&
&**3
dpdx3(534) = 2*x3*x6**6+6*x3**5*x6**2+2*x3*x5**6+6*x3**5*x5**2
dpdx3(535) = x6**7+7*x3**6*x6+x5**7+7*x3**6*x5
dpdx3(536) = 7*x2*x3**6+7*x1*x3**6
dpdx3(537) = 6*x2**2*x3**5+6*x1**2*x3**5
dpdx3(538) = 5*x2**3*x3**4+5*x1**3*x3**4
dpdx3(539) = 4*x2**4*x3**3+4*x1**4*x3**3
dpdx3(540) = 0
dpdx3(541) = 3*x2**5*x3**2+3*x1**5*x3**2
dpdx3(542) = 0
dpdx3(543) = 2*x2**6*x3+2*x1**6*x3
dpdx3(544) = 0
dpdx3(545) = x2**7+x1**7
dpdx3(546) = 0
dpdx4(1) = 0
dpdx4(2) = 0
dpdx4(3) = x6+x5
dpdx4(4) = x2+x1
dpdx4(5) = 0
dpdx4(6) = x3*x6+x3*x5
dpdx4(7) = x5*x6
dpdx4(8) = x1*x6+x2*x5+x2*x3+x1*x3
dpdx4(9) = x1*x2
dpdx4(10) = x2*x6+x1*x5
dpdx4(11) = 0
dpdx4(12) = x6**2+x5**2
dpdx4(13) = 2*x4*x6+2*x4*x5
dpdx4(14) = 2*x2*x4+2*x1*x4+x2**2+x1**2
dpdx4(15) = x3*x5*x6
dpdx4(16) = x1*x3*x6+x2*x3*x5
dpdx4(17) = x2*x5*x6+x1*x5*x6+x2*x3*x6+x1*x3*x5
dpdx4(18) = x1*x2*x6+x1*x2*x5+x1*x2*x3
dpdx4(19) = 0
dpdx4(20) = x3**2*x6+x3**2*x5
dpdx4(21) = x3*x6**2+x3*x5**2
dpdx4(22) = x5*x6**2+x5**2*x6
dpdx4(23) = x1*x6**2+x2*x5**2+x2*x3**2+x1*x3**2
dpdx4(24) = x2*x6**2+x1*x5**2
dpdx4(25) = 2*x3*x4*x6+2*x3*x4*x5
dpdx4(26) = 2*x4*x5*x6
dpdx4(27) = x1**2*x6+x2**2*x5+2*x2*x3*x4+2*x1*x3*x4
dpdx4(28) = 2*x1*x4*x6+2*x2*x4*x5+x2**2*x3+x1**2*x3
dpdx4(29) = 2*x1*x2*x4+x1*x2**2+x1**2*x2
dpdx4(30) = 2*x2*x4*x6+x2**2*x6+2*x1*x4*x5+x1**2*x5
dpdx4(31) = 0
dpdx4(32) = 0
dpdx4(33) = x6**3+x5**3
dpdx4(34) = 2*x4*x6**2+2*x4*x5**2
dpdx4(35) = 2*x2**2*x4+2*x1**2*x4
dpdx4(36) = 3*x4**2*x6+3*x4**2*x5
dpdx4(37) = 3*x2*x4**2+3*x1*x4**2+x2**3+x1**3
dpdx4(38) = x2*x3*x5*x6+x1*x3*x5*x6
dpdx4(39) = x1*x2*x5*x6+x1*x2*x3*x6+x1*x2*x3*x5
dpdx4(40) = x3**2*x5*x6
dpdx4(41) = x3*x5*x6**2+x3*x5**2*x6
dpdx4(42) = x1*x3*x6**2+x1*x3**2*x6+x2*x3*x5**2+x2*x3**2*x5
dpdx4(43) = x1*x5*x6**2+x2*x5**2*x6+x2*x3**2*x6+x1*x3**2*x5
dpdx4(44) = x2*x5*x6**2+x2*x3*x6**2+x1*x5**2*x6+x1*x3*x5**2
dpdx4(45) = x1*x2*x6**2+x1*x2*x5**2+x1*x2*x3**2
dpdx4(46) = 2*x3*x4*x5*x6
dpdx4(47) = 2*x1*x3*x4*x6+x1**2*x3*x6+2*x2*x3*x4*x5+x2**2*x3*x5
dpdx4(48) = x1**2*x2*x6+x1*x2**2*x5+2*x1*x2*x3*x4
dpdx4(49) = x2**2*x5*x6+x1**2*x5*x6+2*x2*x3*x4*x6+2*x1*x3*x4*x5
dpdx4(50) = 2*x2*x4*x5*x6+2*x1*x4*x5*x6+x2**2*x3*x6+x1**2*x3*x5
dpdx4(51) = 2*x1*x2*x4*x6+x1*x2**2*x6+2*x1*x2*x4*x5+x1**2*x2*x5+x&
&1*x2**2*x3+x1**2*x2*x3
dpdx4(52) = 0
dpdx4(53) = 0
dpdx4(54) = x3**3*x6+x3**3*x5
dpdx4(55) = x3**2*x6**2+x3**2*x5**2
dpdx4(56) = x5**2*x6**2
dpdx4(57) = x3*x6**3+x3*x5**3
dpdx4(58) = x5*x6**3+x5**3*x6
dpdx4(59) = x1*x6**3+x2*x5**3+x2*x3**3+x1*x3**3
dpdx4(60) = x2*x6**3+x1*x5**3
dpdx4(61) = 2*x3**2*x4*x6+2*x3**2*x4*x5
dpdx4(62) = 2*x3*x4*x6**2+2*x3*x4*x5**2
dpdx4(63) = 2*x4*x5*x6**2+2*x4*x5**2*x6
dpdx4(64) = x1**2*x6**2+x2**2*x5**2+2*x2*x3**2*x4+2*x1*x3**2*x4
dpdx4(65) = 2*x1*x4*x6**2+2*x2*x4*x5**2+x2**2*x3**2+x1**2*x3**2
dpdx4(66) = 2*x2*x4*x6**2+x2**2*x6**2+2*x1*x4*x5**2+x1**2*x5**2
dpdx4(67) = 2*x1**2*x4*x6+2*x2**2*x4*x5+2*x2**2*x3*x4+2*x1**2*x3*&
&x4
dpdx4(68) = 2*x1*x2**2*x4+2*x1**2*x2*x4+x1**2*x2**2
dpdx4(69) = 2*x2**2*x4*x6+2*x1**2*x4*x5
dpdx4(70) = 3*x3*x4**2*x6+3*x3*x4**2*x5
dpdx4(71) = 3*x4**2*x5*x6
dpdx4(72) = x1**3*x6+x2**3*x5+3*x2*x3*x4**2+3*x1*x3*x4**2
dpdx4(73) = 3*x1*x4**2*x6+3*x2*x4**2*x5+x2**3*x3+x1**3*x3
dpdx4(74) = 3*x1*x2*x4**2+x1*x2**3+x1**3*x2
dpdx4(75) = 3*x2*x4**2*x6+x2**3*x6+3*x1*x4**2*x5+x1**3*x5
dpdx4(76) = 0
dpdx4(77) = 0
dpdx4(78) = x6**4+x5**4
dpdx4(79) = 2*x4*x6**3+2*x4*x5**3
dpdx4(80) = 3*x4**2*x6**2+3*x4**2*x5**2
dpdx4(81) = 3*x2**2*x4**2+3*x1**2*x4**2+2*x2**3*x4+2*x1**3*x4
dpdx4(82) = 4*x4**3*x6+4*x4**3*x5
dpdx4(83) = 4*x2*x4**3+4*x1*x4**3+x2**4+x1**4
dpdx4(84) = x1*x2*x3*x5*x6
dpdx4(85) = x1*x3*x5*x6**2+x2*x3*x5**2*x6+x2*x3**2*x5*x6+x1*x3**2&
&*x5*x6
dpdx4(86) = x2*x3*x5*x6**2+x1*x3*x5**2*x6
dpdx4(87) = x1*x2*x5*x6**2+x1*x2*x3*x6**2+x1*x2*x5**2*x6+x1*x2*x3&
&**2*x6+x1*x2*x3*x5**2+x1*x2*x3**2*x5
dpdx4(88) = 2*x2*x3*x4*x5*x6+2*x1*x3*x4*x5*x6+x2**2*x3*x5*x6+x1**&
&2*x3*x5*x6
dpdx4(89) = x1*x2**2*x5*x6+x1**2*x2*x5*x6+2*x1*x2*x3*x4*x6+x1**2*&
&x2*x3*x6+2*x1*x2*x3*x4*x5+x1*x2**2*x3*x5
dpdx4(90) = 2*x1*x2*x4*x5*x6+x1*x2**2*x3*x6+x1**2*x2*x3*x5
dpdx4(91) = x3**3*x5*x6
dpdx4(92) = x3**2*x5*x6**2+x3**2*x5**2*x6
dpdx4(93) = x3*x5**2*x6**2
dpdx4(94) = x3*x5*x6**3+x3*x5**3*x6
dpdx4(95) = x1*x3**2*x6**2+x2*x3**2*x5**2
dpdx4(96) = x1*x3*x6**3+x1*x3**3*x6+x2*x3*x5**3+x2*x3**3*x5
dpdx4(97) = x1*x5*x6**3+x2*x5**3*x6+x2*x3**3*x6+x1*x3**3*x5
dpdx4(98) = x2*x5**2*x6**2+x1*x5**2*x6**2+x2*x3**2*x6**2+x1*x3**2&
&*x5**2
dpdx4(99) = x2*x5*x6**3+x2*x3*x6**3+x1*x5**3*x6+x1*x3*x5**3
dpdx4(100) = x1*x2*x6**3+x1*x2*x5**3+x1*x2*x3**3
dpdx4(101) = 2*x3**2*x4*x5*x6
dpdx4(102) = 2*x3*x4*x5*x6**2+2*x3*x4*x5**2*x6
dpdx4(103) = x1**2*x3*x6**2+2*x1*x3**2*x4*x6+x2**2*x3*x5**2+2*x2*&
&x3**2*x4*x5
dpdx4(104) = 2*x1*x3*x4*x6**2+x1**2*x3**2*x6+2*x2*x3*x4*x5**2+x2*&
&*2*x3**2*x5
dpdx4(105) = x1**2*x2*x6**2+x1*x2**2*x5**2+2*x1*x2*x3**2*x4
dpdx4(106) = x1**2*x5*x6**2+x2**2*x5**2*x6+2*x2*x3**2*x4*x6+2*x1*&
&x3**2*x4*x5
dpdx4(107) = 2*x1*x4*x5*x6**2+2*x2*x4*x5**2*x6+x2**2*x3**2*x6+x1*&
&*2*x3**2*x5
dpdx4(108) = x2**2*x5*x6**2+2*x2*x3*x4*x6**2+x1**2*x5**2*x6+2*x1*&
&x3*x4*x5**2
dpdx4(109) = 2*x2*x4*x5*x6**2+x2**2*x3*x6**2+2*x1*x4*x5**2*x6+x1*&
&*2*x3*x5**2
dpdx4(110) = 2*x1*x2*x4*x6**2+x1*x2**2*x6**2+2*x1*x2*x4*x5**2+x1*&
&*2*x2*x5**2+x1*x2**2*x3**2+x1**2*x2*x3**2
dpdx4(111) = 2*x1**2*x3*x4*x6+2*x2**2*x3*x4*x5
dpdx4(112) = 2*x1**2*x2*x4*x6+x1**2*x2**2*x6+2*x1*x2**2*x4*x5+x1*&
&*2*x2**2*x5+2*x1*x2**2*x3*x4+2*x1**2*x2*x3*x4
dpdx4(113) = 2*x2**2*x4*x5*x6+2*x1**2*x4*x5*x6+2*x2**2*x3*x4*x6+2&
&*x1**2*x3*x4*x5
dpdx4(114) = 2*x1*x2**2*x4*x6+2*x1**2*x2*x4*x5+x1**2*x2**2*x3
dpdx4(115) = 3*x3*x4**2*x5*x6
dpdx4(116) = 3*x1*x3*x4**2*x6+x1**3*x3*x6+3*x2*x3*x4**2*x5+x2**3*&
&x3*x5
dpdx4(117) = x1**3*x2*x6+x1*x2**3*x5+3*x1*x2*x3*x4**2
dpdx4(118) = x2**3*x5*x6+x1**3*x5*x6+3*x2*x3*x4**2*x6+3*x1*x3*x4*&
&*2*x5
dpdx4(119) = 3*x2*x4**2*x5*x6+3*x1*x4**2*x5*x6+x2**3*x3*x6+x1**3*&
&x3*x5
dpdx4(120) = 3*x1*x2*x4**2*x6+x1*x2**3*x6+3*x1*x2*x4**2*x5+x1**3*&
&x2*x5+x1*x2**3*x3+x1**3*x2*x3
dpdx4(121) = 0
dpdx4(122) = 0
dpdx4(123) = 0
dpdx4(124) = x3**4*x6+x3**4*x5
dpdx4(125) = x3**3*x6**2+x3**3*x5**2
dpdx4(126) = x3**2*x6**3+x3**2*x5**3
dpdx4(127) = x5**2*x6**3+x5**3*x6**2
dpdx4(128) = x3*x6**4+x3*x5**4
dpdx4(129) = x5*x6**4+x5**4*x6
dpdx4(130) = x1*x6**4+x2*x5**4+x2*x3**4+x1*x3**4
dpdx4(131) = x2*x6**4+x1*x5**4
dpdx4(132) = 2*x3**3*x4*x6+2*x3**3*x4*x5
dpdx4(133) = 2*x3**2*x4*x6**2+2*x3**2*x4*x5**2
dpdx4(134) = 2*x4*x5**2*x6**2
dpdx4(135) = 2*x3*x4*x6**3+2*x3*x4*x5**3
dpdx4(136) = 2*x4*x5*x6**3+2*x4*x5**3*x6
dpdx4(137) = x1**2*x6**3+x2**2*x5**3+2*x2*x3**3*x4+2*x1*x3**3*x4
dpdx4(138) = 2*x1*x4*x6**3+2*x2*x4*x5**3+x2**2*x3**3+x1**2*x3**3
dpdx4(139) = 2*x2*x4*x6**3+x2**2*x6**3+2*x1*x4*x5**3+x1**2*x5**3
dpdx4(140) = 2*x1**2*x4*x6**2+2*x2**2*x4*x5**2+2*x2**2*x3**2*x4+2&
&*x1**2*x3**2*x4
dpdx4(141) = 2*x1**2*x2**2*x4
dpdx4(142) = 2*x2**2*x4*x6**2+2*x1**2*x4*x5**2
dpdx4(143) = 3*x3**2*x4**2*x6+3*x3**2*x4**2*x5
dpdx4(144) = 3*x3*x4**2*x6**2+3*x3*x4**2*x5**2
dpdx4(145) = 3*x4**2*x5*x6**2+3*x4**2*x5**2*x6
dpdx4(146) = x1**3*x6**2+x2**3*x5**2+3*x2*x3**2*x4**2+3*x1*x3**2*&
&x4**2
dpdx4(147) = 3*x1*x4**2*x6**2+3*x2*x4**2*x5**2+x2**3*x3**2+x1**3*&
&x3**2
dpdx4(148) = 3*x2*x4**2*x6**2+x2**3*x6**2+3*x1*x4**2*x5**2+x1**3*&
&x5**2
dpdx4(149) = 2*x1**3*x4*x6+2*x2**3*x4*x5+3*x2**2*x3*x4**2+3*x1**2&
&*x3*x4**2
dpdx4(150) = 3*x1**2*x4**2*x6+3*x2**2*x4**2*x5+2*x2**3*x3*x4+2*x1&
&**3*x3*x4
dpdx4(151) = 3*x1*x2**2*x4**2+3*x1**2*x2*x4**2+2*x1*x2**3*x4+2*x1&
&**3*x2*x4+x1**2*x2**3+x1**3*x2**2
dpdx4(152) = 3*x2**2*x4**2*x6+2*x2**3*x4*x6+3*x1**2*x4**2*x5+2*x1&
&**3*x4*x5
dpdx4(153) = 4*x3*x4**3*x6+4*x3*x4**3*x5
dpdx4(154) = 4*x4**3*x5*x6
dpdx4(155) = x1**4*x6+x2**4*x5+4*x2*x3*x4**3+4*x1*x3*x4**3
dpdx4(156) = 4*x1*x4**3*x6+4*x2*x4**3*x5+x2**4*x3+x1**4*x3
dpdx4(157) = 4*x1*x2*x4**3+x1*x2**4+x1**4*x2
dpdx4(158) = 4*x2*x4**3*x6+x2**4*x6+4*x1*x4**3*x5+x1**4*x5
dpdx4(159) = 0
dpdx4(160) = 0
dpdx4(161) = 0
dpdx4(162) = x6**5+x5**5
dpdx4(163) = 2*x4*x6**4+2*x4*x5**4
dpdx4(164) = 3*x4**2*x6**3+3*x4**2*x5**3
dpdx4(165) = 3*x2**3*x4**2+3*x1**3*x4**2
dpdx4(166) = 4*x4**3*x6**2+4*x4**3*x5**2
dpdx4(167) = 4*x2**2*x4**3+4*x1**2*x4**3+2*x2**4*x4+2*x1**4*x4
dpdx4(168) = 5*x4**4*x6+5*x4**4*x5
dpdx4(169) = 5*x2*x4**4+5*x1*x4**4+x2**5+x1**5
dpdx4(170) = x1*x2*x3*x5*x6**2+x1*x2*x3*x5**2*x6+x1*x2*x3**2*x5*x&
&6
dpdx4(171) = 2*x1*x2*x3*x4*x5*x6+x1*x2**2*x3*x5*x6+x1**2*x2*x3*x5&
&*x6
dpdx4(172) = x1*x3**2*x5*x6**2+x2*x3**2*x5**2*x6
dpdx4(173) = x1*x3*x5*x6**3+x2*x3*x5**3*x6+x2*x3**3*x5*x6+x1*x3**&
&3*x5*x6
dpdx4(174) = x2*x3*x5**2*x6**2+x1*x3*x5**2*x6**2+x2*x3**2*x5*x6**&
&2+x1*x3**2*x5**2*x6
dpdx4(175) = x1*x2*x5**2*x6**2+x1*x2*x3**2*x6**2+x1*x2*x3**2*x5**&
&2
dpdx4(176) = x2*x3*x5*x6**3+x1*x3*x5**3*x6
dpdx4(177) = x1*x2*x5*x6**3+x1*x2*x3*x6**3+x1*x2*x5**3*x6+x1*x2*x&
&3**3*x6+x1*x2*x3*x5**3+x1*x2*x3**3*x5
dpdx4(178) = x1**2*x3*x5*x6**2+x2**2*x3*x5**2*x6+2*x2*x3**2*x4*x5&
&*x6+2*x1*x3**2*x4*x5*x6
dpdx4(179) = 2*x1*x3*x4*x5*x6**2+2*x2*x3*x4*x5**2*x6+x2**2*x3**2*&
&x5*x6+x1**2*x3**2*x5*x6
dpdx4(180) = x1**2*x2*x5*x6**2+x1**2*x2*x3*x6**2+x1*x2**2*x5**2*x&
&6+2*x1*x2*x3**2*x4*x6+x1*x2**2*x3*x5**2+2*x1*x2*x3**2*x4*x5
dpdx4(181) = 2*x2*x3*x4*x5*x6**2+x2**2*x3*x5*x6**2+2*x1*x3*x4*x5*&
&*2*x6+x1**2*x3*x5**2*x6
dpdx4(182) = x1*x2**2*x5*x6**2+2*x1*x2*x3*x4*x6**2+x1**2*x2*x5**2&
&*x6+x1**2*x2*x3**2*x6+2*x1*x2*x3*x4*x5**2+x1*x2**2*x3**2*x5
dpdx4(183) = 2*x1*x2*x4*x5*x6**2+x1*x2**2*x3*x6**2+2*x1*x2*x4*x5*&
&*2*x6+x1*x2**2*x3**2*x6+x1**2*x2*x3*x5**2+x1**2*x2*x3**2*x5
dpdx4(184) = x1**2*x2**2*x5*x6+2*x1**2*x2*x3*x4*x6+2*x1*x2**2*x3*&
&x4*x5
dpdx4(185) = 2*x2**2*x3*x4*x5*x6+2*x1**2*x3*x4*x5*x6
dpdx4(186) = 2*x1*x2**2*x4*x5*x6+2*x1**2*x2*x4*x5*x6+2*x1*x2**2*x&
&3*x4*x6+x1**2*x2**2*x3*x6+2*x1**2*x2*x3*x4*x5+x1**2*x2**2*x3*x5
dpdx4(187) = 3*x2*x3*x4**2*x5*x6+3*x1*x3*x4**2*x5*x6+x2**3*x3*x5*&
&x6+x1**3*x3*x5*x6
dpdx4(188) = x1*x2**3*x5*x6+x1**3*x2*x5*x6+3*x1*x2*x3*x4**2*x6+x1&
&**3*x2*x3*x6+3*x1*x2*x3*x4**2*x5+x1*x2**3*x3*x5
dpdx4(189) = 3*x1*x2*x4**2*x5*x6+x1*x2**3*x3*x6+x1**3*x2*x3*x5
dpdx4(190) = x3**4*x5*x6
dpdx4(191) = x3**3*x5*x6**2+x3**3*x5**2*x6
dpdx4(192) = x3**2*x5**2*x6**2
dpdx4(193) = x3**2*x5*x6**3+x3**2*x5**3*x6
dpdx4(194) = x3*x5**2*x6**3+x3*x5**3*x6**2
dpdx4(195) = x3*x5*x6**4+x3*x5**4*x6
dpdx4(196) = x1*x3**2*x6**3+x1*x3**3*x6**2+x2*x3**2*x5**3+x2*x3**&
&3*x5**2
dpdx4(197) = x1*x3*x6**4+x1*x3**4*x6+x2*x3*x5**4+x2*x3**4*x5
dpdx4(198) = x1*x5*x6**4+x2*x5**4*x6+x2*x3**4*x6+x1*x3**4*x5
dpdx4(199) = x1*x5**2*x6**3+x2*x5**3*x6**2+x2*x3**3*x6**2+x1*x3**&
&3*x5**2
dpdx4(200) = x2*x5**2*x6**3+x2*x3**2*x6**3+x1*x5**3*x6**2+x1*x3**&
&2*x5**3
dpdx4(201) = x2*x5*x6**4+x2*x3*x6**4+x1*x5**4*x6+x1*x3*x5**4
dpdx4(202) = x1*x2*x6**4+x1*x2*x5**4+x1*x2*x3**4
dpdx4(203) = 2*x3**3*x4*x5*x6
dpdx4(204) = 2*x3**2*x4*x5*x6**2+2*x3**2*x4*x5**2*x6
dpdx4(205) = 2*x3*x4*x5**2*x6**2
dpdx4(206) = 2*x3*x4*x5*x6**3+2*x3*x4*x5**3*x6
dpdx4(207) = x1**2*x3*x6**3+2*x1*x3**3*x4*x6+x2**2*x3*x5**3+2*x2*&
&x3**3*x4*x5
dpdx4(208) = 2*x1*x3**2*x4*x6**2+x1**2*x3**2*x6**2+2*x2*x3**2*x4*&
&x5**2+x2**2*x3**2*x5**2
dpdx4(209) = 2*x1*x3*x4*x6**3+x1**2*x3**3*x6+2*x2*x3*x4*x5**3+x2*&
&*2*x3**3*x5
dpdx4(210) = x1**2*x2*x6**3+x1*x2**2*x5**3+2*x1*x2*x3**3*x4
dpdx4(211) = x1**2*x5*x6**3+x2**2*x5**3*x6+2*x2*x3**3*x4*x6+2*x1*&
&x3**3*x4*x5
dpdx4(212) = 2*x1*x4*x5*x6**3+2*x2*x4*x5**3*x6+x2**2*x3**3*x6+x1*&
&*2*x3**3*x5
dpdx4(213) = x2**2*x5**2*x6**2+x1**2*x5**2*x6**2+2*x2*x3**2*x4*x6&
&**2+2*x1*x3**2*x4*x5**2
dpdx4(214) = 2*x2*x4*x5**2*x6**2+2*x1*x4*x5**2*x6**2+x2**2*x3**2*&
&x6**2+x1**2*x3**2*x5**2
dpdx4(215) = x2**2*x5*x6**3+2*x2*x3*x4*x6**3+x1**2*x5**3*x6+2*x1*&
&x3*x4*x5**3
dpdx4(216) = 2*x2*x4*x5*x6**3+x2**2*x3*x6**3+2*x1*x4*x5**3*x6+x1*&
&*2*x3*x5**3
dpdx4(217) = 2*x1*x2*x4*x6**3+x1*x2**2*x6**3+2*x1*x2*x4*x5**3+x1*&
&*2*x2*x5**3+x1*x2**2*x3**3+x1**2*x2*x3**3
dpdx4(218) = 2*x1**2*x3*x4*x6**2+2*x1**2*x3**2*x4*x6+2*x2**2*x3*x&
&4*x5**2+2*x2**2*x3**2*x4*x5
dpdx4(219) = 2*x1**2*x2*x4*x6**2+x1**2*x2**2*x6**2+2*x1*x2**2*x4*&
&x5**2+x1**2*x2**2*x5**2+2*x1*x2**2*x3**2*x4+2*x1**2*x2*x3**2*x4
dpdx4(220) = 2*x1**2*x4*x5*x6**2+2*x2**2*x4*x5**2*x6+2*x2**2*x3**&
&2*x4*x6+2*x1**2*x3**2*x4*x5
dpdx4(221) = 2*x1**2*x2**2*x4*x6+2*x1**2*x2**2*x4*x5+2*x1**2*x2**&
&2*x3*x4
dpdx4(222) = 2*x2**2*x4*x5*x6**2+2*x2**2*x3*x4*x6**2+2*x1**2*x4*x&
&5**2*x6+2*x1**2*x3*x4*x5**2
dpdx4(223) = 2*x1*x2**2*x4*x6**2+2*x1**2*x2*x4*x5**2+x1**2*x2**2*&
&x3**2
dpdx4(224) = 3*x3**2*x4**2*x5*x6
dpdx4(225) = 3*x3*x4**2*x5*x6**2+3*x3*x4**2*x5**2*x6
dpdx4(226) = x1**3*x3*x6**2+3*x1*x3**2*x4**2*x6+x2**3*x3*x5**2+3*&
&x2*x3**2*x4**2*x5
dpdx4(227) = 3*x1*x3*x4**2*x6**2+x1**3*x3**2*x6+3*x2*x3*x4**2*x5*&
&*2+x2**3*x3**2*x5
dpdx4(228) = x1**3*x2*x6**2+x1*x2**3*x5**2+3*x1*x2*x3**2*x4**2
dpdx4(229) = x1**3*x5*x6**2+x2**3*x5**2*x6+3*x2*x3**2*x4**2*x6+3*&
&x1*x3**2*x4**2*x5
dpdx4(230) = 3*x1*x4**2*x5*x6**2+3*x2*x4**2*x5**2*x6+x2**3*x3**2*&
&x6+x1**3*x3**2*x5
dpdx4(231) = x2**3*x5*x6**2+3*x2*x3*x4**2*x6**2+x1**3*x5**2*x6+3*&
&x1*x3*x4**2*x5**2
dpdx4(232) = 3*x2*x4**2*x5*x6**2+x2**3*x3*x6**2+3*x1*x4**2*x5**2*&
&x6+x1**3*x3*x5**2
dpdx4(233) = 3*x1*x2*x4**2*x6**2+x1*x2**3*x6**2+3*x1*x2*x4**2*x5*&
&*2+x1**3*x2*x5**2+x1*x2**3*x3**2+x1**3*x2*x3**2
dpdx4(234) = 3*x1**2*x3*x4**2*x6+2*x1**3*x3*x4*x6+3*x2**2*x3*x4**&
&2*x5+2*x2**3*x3*x4*x5
dpdx4(235) = 2*x1**3*x2*x4*x6+x1**3*x2**2*x6+2*x1*x2**3*x4*x5+x1*&
&*2*x2**3*x5+3*x1*x2**2*x3*x4**2+3*x1**2*x2*x3*x4**2
dpdx4(236) = 3*x1**2*x2*x4**2*x6+x1**2*x2**3*x6+3*x1*x2**2*x4**2*&
&x5+x1**3*x2**2*x5+2*x1*x2**3*x3*x4+2*x1**3*x2*x3*x4
dpdx4(237) = 2*x2**3*x4*x5*x6+2*x1**3*x4*x5*x6+3*x2**2*x3*x4**2*x&
&6+3*x1**2*x3*x4**2*x5
dpdx4(238) = 3*x2**2*x4**2*x5*x6+3*x1**2*x4**2*x5*x6+2*x2**3*x3*x&
&4*x6+2*x1**3*x3*x4*x5
dpdx4(239) = 3*x1*x2**2*x4**2*x6+2*x1*x2**3*x4*x6+3*x1**2*x2*x4**&
&2*x5+2*x1**3*x2*x4*x5+x1**2*x2**3*x3+x1**3*x2**2*x3
dpdx4(240) = 4*x3*x4**3*x5*x6
dpdx4(241) = 4*x1*x3*x4**3*x6+x1**4*x3*x6+4*x2*x3*x4**3*x5+x2**4*&
&x3*x5
dpdx4(242) = x1**4*x2*x6+x1*x2**4*x5+4*x1*x2*x3*x4**3
dpdx4(243) = x2**4*x5*x6+x1**4*x5*x6+4*x2*x3*x4**3*x6+4*x1*x3*x4*&
&*3*x5
dpdx4(244) = 4*x2*x4**3*x5*x6+4*x1*x4**3*x5*x6+x2**4*x3*x6+x1**4*&
&x3*x5
dpdx4(245) = 4*x1*x2*x4**3*x6+x1*x2**4*x6+4*x1*x2*x4**3*x5+x1**4*&
&x2*x5+x1*x2**4*x3+x1**4*x2*x3
dpdx4(246) = 0
dpdx4(247) = 0
dpdx4(248) = 0
dpdx4(249) = 0
dpdx4(250) = x3**5*x6+x3**5*x5
dpdx4(251) = x3**4*x6**2+x3**4*x5**2
dpdx4(252) = x3**3*x6**3+x3**3*x5**3
dpdx4(253) = x5**3*x6**3
dpdx4(254) = x3**2*x6**4+x3**2*x5**4
dpdx4(255) = x5**2*x6**4+x5**4*x6**2
dpdx4(256) = x3*x6**5+x3*x5**5
dpdx4(257) = x5*x6**5+x5**5*x6
dpdx4(258) = x1*x6**5+x2*x5**5+x2*x3**5+x1*x3**5
dpdx4(259) = x2*x6**5+x1*x5**5
dpdx4(260) = 2*x3**4*x4*x6+2*x3**4*x4*x5
dpdx4(261) = 2*x3**3*x4*x6**2+2*x3**3*x4*x5**2
dpdx4(262) = 2*x3**2*x4*x6**3+2*x3**2*x4*x5**3
dpdx4(263) = 2*x4*x5**2*x6**3+2*x4*x5**3*x6**2
dpdx4(264) = 2*x3*x4*x6**4+2*x3*x4*x5**4
dpdx4(265) = 2*x4*x5*x6**4+2*x4*x5**4*x6
dpdx4(266) = x1**2*x6**4+x2**2*x5**4+2*x2*x3**4*x4+2*x1*x3**4*x4
dpdx4(267) = 2*x1*x4*x6**4+2*x2*x4*x5**4+x2**2*x3**4+x1**2*x3**4
dpdx4(268) = 2*x2*x4*x6**4+x2**2*x6**4+2*x1*x4*x5**4+x1**2*x5**4
dpdx4(269) = 2*x1**2*x4*x6**3+2*x2**2*x4*x5**3+2*x2**2*x3**3*x4+2&
&*x1**2*x3**3*x4
dpdx4(270) = 2*x2**2*x4*x6**3+2*x1**2*x4*x5**3
dpdx4(271) = 3*x3**3*x4**2*x6+3*x3**3*x4**2*x5
dpdx4(272) = 3*x3**2*x4**2*x6**2+3*x3**2*x4**2*x5**2
dpdx4(273) = 3*x4**2*x5**2*x6**2
dpdx4(274) = 3*x3*x4**2*x6**3+3*x3*x4**2*x5**3
dpdx4(275) = 3*x4**2*x5*x6**3+3*x4**2*x5**3*x6
dpdx4(276) = x1**3*x6**3+x2**3*x5**3+3*x2*x3**3*x4**2+3*x1*x3**3*&
&x4**2
dpdx4(277) = 3*x1*x4**2*x6**3+3*x2*x4**2*x5**3+x2**3*x3**3+x1**3*&
&x3**3
dpdx4(278) = 3*x2*x4**2*x6**3+x2**3*x6**3+3*x1*x4**2*x5**3+x1**3*&
&x5**3
dpdx4(279) = 2*x1**3*x4*x6**2+2*x2**3*x4*x5**2+3*x2**2*x3**2*x4**&
&2+3*x1**2*x3**2*x4**2
dpdx4(280) = 3*x1**2*x4**2*x6**2+3*x2**2*x4**2*x5**2+2*x2**3*x3**&
&2*x4+2*x1**3*x3**2*x4
dpdx4(281) = 3*x1**2*x2**2*x4**2+2*x1**2*x2**3*x4+2*x1**3*x2**2*x&
&4
dpdx4(282) = 3*x2**2*x4**2*x6**2+2*x2**3*x4*x6**2+3*x1**2*x4**2*x&
&5**2+2*x1**3*x4*x5**2
dpdx4(283) = 3*x1**3*x4**2*x6+3*x2**3*x4**2*x5+3*x2**3*x3*x4**2+3&
&*x1**3*x3*x4**2
dpdx4(284) = 3*x1*x2**3*x4**2+3*x1**3*x2*x4**2+x1**3*x2**3
dpdx4(285) = 3*x2**3*x4**2*x6+3*x1**3*x4**2*x5
dpdx4(286) = 4*x3**2*x4**3*x6+4*x3**2*x4**3*x5
dpdx4(287) = 4*x3*x4**3*x6**2+4*x3*x4**3*x5**2
dpdx4(288) = 4*x4**3*x5*x6**2+4*x4**3*x5**2*x6
dpdx4(289) = x1**4*x6**2+x2**4*x5**2+4*x2*x3**2*x4**3+4*x1*x3**2*&
&x4**3
dpdx4(290) = 4*x1*x4**3*x6**2+4*x2*x4**3*x5**2+x2**4*x3**2+x1**4*&
&x3**2
dpdx4(291) = 4*x2*x4**3*x6**2+x2**4*x6**2+4*x1*x4**3*x5**2+x1**4*&
&x5**2
dpdx4(292) = 2*x1**4*x4*x6+2*x2**4*x4*x5+4*x2**2*x3*x4**3+4*x1**2&
&*x3*x4**3
dpdx4(293) = 4*x1**2*x4**3*x6+4*x2**2*x4**3*x5+2*x2**4*x3*x4+2*x1&
&**4*x3*x4
dpdx4(294) = 4*x1*x2**2*x4**3+4*x1**2*x2*x4**3+2*x1*x2**4*x4+2*x1&
&**4*x2*x4+x1**2*x2**4+x1**4*x2**2
dpdx4(295) = 4*x2**2*x4**3*x6+2*x2**4*x4*x6+4*x1**2*x4**3*x5+2*x1&
&**4*x4*x5
dpdx4(296) = 5*x3*x4**4*x6+5*x3*x4**4*x5
dpdx4(297) = 5*x4**4*x5*x6
dpdx4(298) = x1**5*x6+x2**5*x5+5*x2*x3*x4**4+5*x1*x3*x4**4
dpdx4(299) = 5*x1*x4**4*x6+5*x2*x4**4*x5+x2**5*x3+x1**5*x3
dpdx4(300) = 5*x1*x2*x4**4+x1*x2**5+x1**5*x2
dpdx4(301) = 5*x2*x4**4*x6+x2**5*x6+5*x1*x4**4*x5+x1**5*x5
dpdx4(302) = 0
dpdx4(303) = 0
dpdx4(304) = 0
dpdx4(305) = x6**6+x5**6
dpdx4(306) = 2*x4*x6**5+2*x4*x5**5
dpdx4(307) = 3*x4**2*x6**4+3*x4**2*x5**4
dpdx4(308) = 4*x4**3*x6**3+4*x4**3*x5**3
dpdx4(309) = 4*x2**3*x4**3+4*x1**3*x4**3+3*x2**4*x4**2+3*x1**4*x4&
&**2
dpdx4(310) = 5*x4**4*x6**2+5*x4**4*x5**2
dpdx4(311) = 5*x2**2*x4**4+5*x1**2*x4**4+2*x2**5*x4+2*x1**5*x4
dpdx4(312) = 6*x4**5*x6+6*x4**5*x5
dpdx4(313) = 6*x2*x4**5+6*x1*x4**5+x2**6+x1**6
dpdx4(314) = x1*x2*x3*x5**2*x6**2+x1*x2*x3**2*x5*x6**2+x1*x2*x3**&
&2*x5**2*x6
dpdx4(315) = x1*x2*x3*x5*x6**3+x1*x2*x3*x5**3*x6+x1*x2*x3**3*x5*x&
&6
dpdx4(316) = x1**2*x2*x3*x5*x6**2+x1*x2**2*x3*x5**2*x6+2*x1*x2*x3&
&**2*x4*x5*x6
dpdx4(317) = 2*x1*x2*x3*x4*x5*x6**2+x1*x2**2*x3*x5*x6**2+2*x1*x2*&
&x3*x4*x5**2*x6+x1**2*x2*x3*x5**2*x6+x1*x2**2*x3**2*x5*x6+x1**2*x2&
&*x3**2*x5*x6
dpdx4(318) = 2*x1*x2**2*x3*x4*x5*x6+2*x1**2*x2*x3*x4*x5*x6+x1**2*&
&x2**2*x3*x5*x6
dpdx4(319) = 3*x1*x2*x3*x4**2*x5*x6+x1*x2**3*x3*x5*x6+x1**3*x2*x3&
&*x5*x6
dpdx4(320) = x1*x3**2*x5*x6**3+x1*x3**3*x5*x6**2+x2*x3**2*x5**3*x&
&6+x2*x3**3*x5**2*x6
dpdx4(321) = x1*x3*x5*x6**4+x2*x3*x5**4*x6+x2*x3**4*x5*x6+x1*x3**&
&4*x5*x6
dpdx4(322) = x2*x3**2*x5**2*x6**2+x1*x3**2*x5**2*x6**2
dpdx4(323) = x1*x3*x5**2*x6**3+x2*x3*x5**3*x6**2+x2*x3**3*x5*x6**&
&2+x1*x3**3*x5**2*x6
dpdx4(324) = x2*x3*x5**2*x6**3+x2*x3**2*x5*x6**3+x1*x3*x5**3*x6**&
&2+x1*x3**2*x5**3*x6
dpdx4(325) = x1*x2*x5**2*x6**3+x1*x2*x3**2*x6**3+x1*x2*x5**3*x6**&
&2+x1*x2*x3**3*x6**2+x1*x2*x3**2*x5**3+x1*x2*x3**3*x5**2
dpdx4(326) = x2*x3*x5*x6**4+x1*x3*x5**4*x6
dpdx4(327) = x1*x2*x5*x6**4+x1*x2*x3*x6**4+x1*x2*x5**4*x6+x1*x2*x&
&3**4*x6+x1*x2*x3*x5**4+x1*x2*x3**4*x5
dpdx4(328) = x1**2*x3*x5*x6**3+x2**2*x3*x5**3*x6+2*x2*x3**3*x4*x5&
&*x6+2*x1*x3**3*x4*x5*x6
dpdx4(329) = 2*x1*x3**2*x4*x5*x6**2+x1**2*x3**2*x5*x6**2+2*x2*x3*&
&*2*x4*x5**2*x6+x2**2*x3**2*x5**2*x6
dpdx4(330) = 2*x1*x3*x4*x5*x6**3+2*x2*x3*x4*x5**3*x6+x2**2*x3**3*&
&x5*x6+x1**2*x3**3*x5*x6
dpdx4(331) = x1**2*x2*x5*x6**3+x1**2*x2*x3*x6**3+x1*x2**2*x5**3*x&
&6+2*x1*x2*x3**3*x4*x6+x1*x2**2*x3*x5**3+2*x1*x2*x3**3*x4*x5
dpdx4(332) = x2**2*x3*x5**2*x6**2+x1**2*x3*x5**2*x6**2+2*x2*x3**2&
&*x4*x5*x6**2+2*x1*x3**2*x4*x5**2*x6
dpdx4(333) = 2*x2*x3*x4*x5**2*x6**2+2*x1*x3*x4*x5**2*x6**2+x2**2*&
&x3**2*x5*x6**2+x1**2*x3**2*x5**2*x6
dpdx4(334) = x1*x2**2*x5**2*x6**2+x1**2*x2*x5**2*x6**2+2*x1*x2*x3&
&**2*x4*x6**2+x1**2*x2*x3**2*x6**2+2*x1*x2*x3**2*x4*x5**2+x1*x2**2&
&*x3**2*x5**2
dpdx4(335) = 2*x1*x2*x4*x5**2*x6**2+x1*x2**2*x3**2*x6**2+x1**2*x2&
&*x3**2*x5**2
dpdx4(336) = 2*x2*x3*x4*x5*x6**3+x2**2*x3*x5*x6**3+2*x1*x3*x4*x5*&
&*3*x6+x1**2*x3*x5**3*x6
dpdx4(337) = x1*x2**2*x5*x6**3+2*x1*x2*x3*x4*x6**3+x1**2*x2*x5**3&
&*x6+x1**2*x2*x3**3*x6+2*x1*x2*x3*x4*x5**3+x1*x2**2*x3**3*x5
dpdx4(338) = 2*x1*x2*x4*x5*x6**3+x1*x2**2*x3*x6**3+2*x1*x2*x4*x5*&
&*3*x6+x1*x2**2*x3**3*x6+x1**2*x2*x3*x5**3+x1**2*x2*x3**3*x5
dpdx4(339) = x1**2*x2**2*x5*x6**2+2*x1**2*x2*x3*x4*x6**2+x1**2*x2&
&**2*x5**2*x6+2*x1**2*x2*x3**2*x4*x6+2*x1*x2**2*x3*x4*x5**2+2*x1*x&
&2**2*x3**2*x4*x5
dpdx4(340) = 2*x1**2*x3*x4*x5*x6**2+2*x2**2*x3*x4*x5**2*x6+2*x2**&
&2*x3**2*x4*x5*x6+2*x1**2*x3**2*x4*x5*x6
dpdx4(341) = 2*x1**2*x2*x4*x5*x6**2+x1**2*x2**2*x3*x6**2+2*x1*x2*&
&*2*x4*x5**2*x6+2*x1*x2**2*x3**2*x4*x6+x1**2*x2**2*x3*x5**2+2*x1**&
&2*x2*x3**2*x4*x5
dpdx4(342) = 2*x1**2*x2**2*x4*x5*x6+2*x1**2*x2**2*x3*x4*x6+2*x1**&
&2*x2**2*x3*x4*x5
dpdx4(343) = 2*x2**2*x3*x4*x5*x6**2+2*x1**2*x3*x4*x5**2*x6
dpdx4(344) = 2*x1*x2**2*x4*x5*x6**2+2*x1*x2**2*x3*x4*x6**2+2*x1**&
&2*x2*x4*x5**2*x6+x1**2*x2**2*x3**2*x6+2*x1**2*x2*x3*x4*x5**2+x1**&
&2*x2**2*x3**2*x5
dpdx4(345) = x1**3*x3*x5*x6**2+x2**3*x3*x5**2*x6+3*x2*x3**2*x4**2&
&*x5*x6+3*x1*x3**2*x4**2*x5*x6
dpdx4(346) = 3*x1*x3*x4**2*x5*x6**2+3*x2*x3*x4**2*x5**2*x6+x2**3*&
&x3**2*x5*x6+x1**3*x3**2*x5*x6
dpdx4(347) = x1**3*x2*x5*x6**2+x1**3*x2*x3*x6**2+x1*x2**3*x5**2*x&
&6+3*x1*x2*x3**2*x4**2*x6+x1*x2**3*x3*x5**2+3*x1*x2*x3**2*x4**2*x5
dpdx4(348) = 3*x2*x3*x4**2*x5*x6**2+x2**3*x3*x5*x6**2+3*x1*x3*x4*&
&*2*x5**2*x6+x1**3*x3*x5**2*x6
dpdx4(349) = x1*x2**3*x5*x6**2+3*x1*x2*x3*x4**2*x6**2+x1**3*x2*x5&
&**2*x6+x1**3*x2*x3**2*x6+3*x1*x2*x3*x4**2*x5**2+x1*x2**3*x3**2*x5
dpdx4(350) = 3*x1*x2*x4**2*x5*x6**2+x1*x2**3*x3*x6**2+3*x1*x2*x4*&
&*2*x5**2*x6+x1*x2**3*x3**2*x6+x1**3*x2*x3*x5**2+x1**3*x2*x3**2*x5
dpdx4(351) = x1**2*x2**3*x5*x6+x1**3*x2**2*x5*x6+3*x1**2*x2*x3*x4&
&**2*x6+2*x1**3*x2*x3*x4*x6+3*x1*x2**2*x3*x4**2*x5+2*x1*x2**3*x3*x&
&4*x5
dpdx4(352) = 3*x2**2*x3*x4**2*x5*x6+3*x1**2*x3*x4**2*x5*x6+2*x2**&
&3*x3*x4*x5*x6+2*x1**3*x3*x4*x5*x6
dpdx4(353) = 2*x1*x2**3*x4*x5*x6+2*x1**3*x2*x4*x5*x6+3*x1*x2**2*x&
&3*x4**2*x6+x1**3*x2**2*x3*x6+3*x1**2*x2*x3*x4**2*x5+x1**2*x2**3*x&
&3*x5
dpdx4(354) = 3*x1*x2**2*x4**2*x5*x6+3*x1**2*x2*x4**2*x5*x6+2*x1*x&
&2**3*x3*x4*x6+x1**2*x2**3*x3*x6+2*x1**3*x2*x3*x4*x5+x1**3*x2**2*x&
&3*x5
dpdx4(355) = 4*x2*x3*x4**3*x5*x6+4*x1*x3*x4**3*x5*x6+x2**4*x3*x5*&
&x6+x1**4*x3*x5*x6
dpdx4(356) = x1*x2**4*x5*x6+x1**4*x2*x5*x6+4*x1*x2*x3*x4**3*x6+x1&
&**4*x2*x3*x6+4*x1*x2*x3*x4**3*x5+x1*x2**4*x3*x5
dpdx4(357) = 4*x1*x2*x4**3*x5*x6+x1*x2**4*x3*x6+x1**4*x2*x3*x5
dpdx4(358) = x3**5*x5*x6
dpdx4(359) = x3**4*x5*x6**2+x3**4*x5**2*x6
dpdx4(360) = x3**3*x5**2*x6**2
dpdx4(361) = x3**3*x5*x6**3+x3**3*x5**3*x6
dpdx4(362) = x3**2*x5**2*x6**3+x3**2*x5**3*x6**2
dpdx4(363) = x3*x5**3*x6**3
dpdx4(364) = x3**2*x5*x6**4+x3**2*x5**4*x6
dpdx4(365) = x3*x5**2*x6**4+x3*x5**4*x6**2
dpdx4(366) = x3*x5*x6**5+x3*x5**5*x6
dpdx4(367) = x1*x3**3*x6**3+x2*x3**3*x5**3
dpdx4(368) = x1*x3**2*x6**4+x1*x3**4*x6**2+x2*x3**2*x5**4+x2*x3**&
&4*x5**2
dpdx4(369) = x1*x3*x6**5+x1*x3**5*x6+x2*x3*x5**5+x2*x3**5*x5
dpdx4(370) = x1*x5*x6**5+x2*x5**5*x6+x2*x3**5*x6+x1*x3**5*x5
dpdx4(371) = x1*x5**2*x6**4+x2*x5**4*x6**2+x2*x3**4*x6**2+x1*x3**&
&4*x5**2
dpdx4(372) = x2*x5**3*x6**3+x1*x5**3*x6**3+x2*x3**3*x6**3+x1*x3**&
&3*x5**3
dpdx4(373) = x2*x5**2*x6**4+x2*x3**2*x6**4+x1*x5**4*x6**2+x1*x3**&
&2*x5**4
dpdx4(374) = x2*x5*x6**5+x2*x3*x6**5+x1*x5**5*x6+x1*x3*x5**5
dpdx4(375) = x1*x2*x6**5+x1*x2*x5**5+x1*x2*x3**5
dpdx4(376) = 2*x3**4*x4*x5*x6
dpdx4(377) = 2*x3**3*x4*x5*x6**2+2*x3**3*x4*x5**2*x6
dpdx4(378) = 2*x3**2*x4*x5**2*x6**2
dpdx4(379) = 2*x3**2*x4*x5*x6**3+2*x3**2*x4*x5**3*x6
dpdx4(380) = 2*x3*x4*x5**2*x6**3+2*x3*x4*x5**3*x6**2
dpdx4(381) = 2*x3*x4*x5*x6**4+2*x3*x4*x5**4*x6
dpdx4(382) = x1**2*x3*x6**4+2*x1*x3**4*x4*x6+x2**2*x3*x5**4+2*x2*&
&x3**4*x4*x5
dpdx4(383) = x1**2*x3**2*x6**3+2*x1*x3**3*x4*x6**2+x2**2*x3**2*x5&
&**3+2*x2*x3**3*x4*x5**2
dpdx4(384) = 2*x1*x3**2*x4*x6**3+x1**2*x3**3*x6**2+2*x2*x3**2*x4*&
&x5**3+x2**2*x3**3*x5**2
dpdx4(385) = 2*x1*x3*x4*x6**4+x1**2*x3**4*x6+2*x2*x3*x4*x5**4+x2*&
&*2*x3**4*x5
dpdx4(386) = x1**2*x2*x6**4+x1*x2**2*x5**4+2*x1*x2*x3**4*x4
dpdx4(387) = x1**2*x5*x6**4+x2**2*x5**4*x6+2*x2*x3**4*x4*x6+2*x1*&
&x3**4*x4*x5
dpdx4(388) = 2*x1*x4*x5*x6**4+2*x2*x4*x5**4*x6+x2**2*x3**4*x6+x1*&
&*2*x3**4*x5
dpdx4(389) = x1**2*x5**2*x6**3+x2**2*x5**3*x6**2+2*x2*x3**3*x4*x6&
&**2+2*x1*x3**3*x4*x5**2
dpdx4(390) = 2*x1*x4*x5**2*x6**3+2*x2*x4*x5**3*x6**2+x2**2*x3**3*&
&x6**2+x1**2*x3**3*x5**2
dpdx4(391) = x2**2*x5**2*x6**3+2*x2*x3**2*x4*x6**3+x1**2*x5**3*x6&
&**2+2*x1*x3**2*x4*x5**3
dpdx4(392) = 2*x2*x4*x5**2*x6**3+x2**2*x3**2*x6**3+2*x1*x4*x5**3*&
&x6**2+x1**2*x3**2*x5**3
dpdx4(393) = x2**2*x5*x6**4+2*x2*x3*x4*x6**4+x1**2*x5**4*x6+2*x1*&
&x3*x4*x5**4
dpdx4(394) = 2*x2*x4*x5*x6**4+x2**2*x3*x6**4+2*x1*x4*x5**4*x6+x1*&
&*2*x3*x5**4
dpdx4(395) = 2*x1*x2*x4*x6**4+x1*x2**2*x6**4+2*x1*x2*x4*x5**4+x1*&
&*2*x2*x5**4+x1*x2**2*x3**4+x1**2*x2*x3**4
dpdx4(396) = 2*x1**2*x3**2*x4*x6**2+2*x2**2*x3**2*x4*x5**2
dpdx4(397) = 2*x1**2*x3*x4*x6**3+2*x1**2*x3**3*x4*x6+2*x2**2*x3*x&
&4*x5**3+2*x2**2*x3**3*x4*x5
dpdx4(398) = 2*x1**2*x2*x4*x6**3+x1**2*x2**2*x6**3+2*x1*x2**2*x4*&
&x5**3+x1**2*x2**2*x5**3+2*x1*x2**2*x3**3*x4+2*x1**2*x2*x3**3*x4
dpdx4(399) = 2*x1**2*x4*x5*x6**3+2*x2**2*x4*x5**3*x6+2*x2**2*x3**&
&3*x4*x6+2*x1**2*x3**3*x4*x5
dpdx4(400) = 2*x2**2*x4*x5**2*x6**2+2*x1**2*x4*x5**2*x6**2+2*x2**&
&2*x3**2*x4*x6**2+2*x1**2*x3**2*x4*x5**2
dpdx4(401) = 2*x1**2*x2**2*x4*x6**2+2*x1**2*x2**2*x4*x5**2+2*x1**&
&2*x2**2*x3**2*x4
dpdx4(402) = 2*x2**2*x4*x5*x6**3+2*x2**2*x3*x4*x6**3+2*x1**2*x4*x&
&5**3*x6+2*x1**2*x3*x4*x5**3
dpdx4(403) = 2*x1*x2**2*x4*x6**3+2*x1**2*x2*x4*x5**3+x1**2*x2**2*&
&x3**3
dpdx4(404) = 3*x3**3*x4**2*x5*x6
dpdx4(405) = 3*x3**2*x4**2*x5*x6**2+3*x3**2*x4**2*x5**2*x6
dpdx4(406) = 3*x3*x4**2*x5**2*x6**2
dpdx4(407) = 3*x3*x4**2*x5*x6**3+3*x3*x4**2*x5**3*x6
dpdx4(408) = x1**3*x3*x6**3+3*x1*x3**3*x4**2*x6+x2**3*x3*x5**3+3*&
&x2*x3**3*x4**2*x5
dpdx4(409) = 3*x1*x3**2*x4**2*x6**2+x1**3*x3**2*x6**2+3*x2*x3**2*&
&x4**2*x5**2+x2**3*x3**2*x5**2
dpdx4(410) = 3*x1*x3*x4**2*x6**3+x1**3*x3**3*x6+3*x2*x3*x4**2*x5*&
&*3+x2**3*x3**3*x5
dpdx4(411) = x1**3*x2*x6**3+x1*x2**3*x5**3+3*x1*x2*x3**3*x4**2
dpdx4(412) = x1**3*x5*x6**3+x2**3*x5**3*x6+3*x2*x3**3*x4**2*x6+3*&
&x1*x3**3*x4**2*x5
dpdx4(413) = 3*x1*x4**2*x5*x6**3+3*x2*x4**2*x5**3*x6+x2**3*x3**3*&
&x6+x1**3*x3**3*x5
dpdx4(414) = x2**3*x5**2*x6**2+x1**3*x5**2*x6**2+3*x2*x3**2*x4**2&
&*x6**2+3*x1*x3**2*x4**2*x5**2
dpdx4(415) = 3*x2*x4**2*x5**2*x6**2+3*x1*x4**2*x5**2*x6**2+x2**3*&
&x3**2*x6**2+x1**3*x3**2*x5**2
dpdx4(416) = x2**3*x5*x6**3+3*x2*x3*x4**2*x6**3+x1**3*x5**3*x6+3*&
&x1*x3*x4**2*x5**3
dpdx4(417) = 3*x2*x4**2*x5*x6**3+x2**3*x3*x6**3+3*x1*x4**2*x5**3*&
&x6+x1**3*x3*x5**3
dpdx4(418) = 3*x1*x2*x4**2*x6**3+x1*x2**3*x6**3+3*x1*x2*x4**2*x5*&
&*3+x1**3*x2*x5**3+x1*x2**3*x3**3+x1**3*x2*x3**3
dpdx4(419) = 2*x1**3*x3*x4*x6**2+3*x1**2*x3**2*x4**2*x6+2*x2**3*x&
&3*x4*x5**2+3*x2**2*x3**2*x4**2*x5
dpdx4(420) = 3*x1**2*x3*x4**2*x6**2+2*x1**3*x3**2*x4*x6+3*x2**2*x&
&3*x4**2*x5**2+2*x2**3*x3**2*x4*x5
dpdx4(421) = 2*x1**3*x2*x4*x6**2+x1**3*x2**2*x6**2+2*x1*x2**3*x4*&
&x5**2+x1**2*x2**3*x5**2+3*x1*x2**2*x3**2*x4**2+3*x1**2*x2*x3**2*x&
&4**2
dpdx4(422) = 3*x1**2*x2*x4**2*x6**2+x1**2*x2**3*x6**2+3*x1*x2**2*&
&x4**2*x5**2+x1**3*x2**2*x5**2+2*x1*x2**3*x3**2*x4+2*x1**3*x2*x3**&
&2*x4
dpdx4(423) = 2*x1**3*x2**2*x4*x6+2*x1**2*x2**3*x4*x5+3*x1**2*x2**&
&2*x3*x4**2
dpdx4(424) = 2*x1**3*x4*x5*x6**2+2*x2**3*x4*x5**2*x6+3*x2**2*x3**&
&2*x4**2*x6+3*x1**2*x3**2*x4**2*x5
dpdx4(425) = 3*x1**2*x4**2*x5*x6**2+3*x2**2*x4**2*x5**2*x6+2*x2**&
&3*x3**2*x4*x6+2*x1**3*x3**2*x4*x5
dpdx4(426) = 3*x1**2*x2**2*x4**2*x6+2*x1**2*x2**3*x4*x6+3*x1**2*x&
&2**2*x4**2*x5+2*x1**3*x2**2*x4*x5+2*x1**2*x2**3*x3*x4+2*x1**3*x2*&
&*2*x3*x4
dpdx4(427) = 2*x2**3*x4*x5*x6**2+3*x2**2*x3*x4**2*x6**2+2*x1**3*x&
&4*x5**2*x6+3*x1**2*x3*x4**2*x5**2
dpdx4(428) = 3*x2**2*x4**2*x5*x6**2+2*x2**3*x3*x4*x6**2+3*x1**2*x&
&4**2*x5**2*x6+2*x1**3*x3*x4*x5**2
dpdx4(429) = 3*x1*x2**2*x4**2*x6**2+2*x1*x2**3*x4*x6**2+3*x1**2*x&
&2*x4**2*x5**2+2*x1**3*x2*x4*x5**2+x1**2*x2**3*x3**2+x1**3*x2**2*x&
&3**2
dpdx4(430) = 3*x1**3*x3*x4**2*x6+3*x2**3*x3*x4**2*x5
dpdx4(431) = 3*x1**3*x2*x4**2*x6+x1**3*x2**3*x6+3*x1*x2**3*x4**2*&
&x5+x1**3*x2**3*x5+3*x1*x2**3*x3*x4**2+3*x1**3*x2*x3*x4**2
dpdx4(432) = 3*x2**3*x4**2*x5*x6+3*x1**3*x4**2*x5*x6+3*x2**3*x3*x&
&4**2*x6+3*x1**3*x3*x4**2*x5
dpdx4(433) = 3*x1*x2**3*x4**2*x6+3*x1**3*x2*x4**2*x5+x1**3*x2**3*&
&x3
dpdx4(434) = 4*x3**2*x4**3*x5*x6
dpdx4(435) = 4*x3*x4**3*x5*x6**2+4*x3*x4**3*x5**2*x6
dpdx4(436) = x1**4*x3*x6**2+4*x1*x3**2*x4**3*x6+x2**4*x3*x5**2+4*&
&x2*x3**2*x4**3*x5
dpdx4(437) = 4*x1*x3*x4**3*x6**2+x1**4*x3**2*x6+4*x2*x3*x4**3*x5*&
&*2+x2**4*x3**2*x5
dpdx4(438) = x1**4*x2*x6**2+x1*x2**4*x5**2+4*x1*x2*x3**2*x4**3
dpdx4(439) = x1**4*x5*x6**2+x2**4*x5**2*x6+4*x2*x3**2*x4**3*x6+4*&
&x1*x3**2*x4**3*x5
dpdx4(440) = 4*x1*x4**3*x5*x6**2+4*x2*x4**3*x5**2*x6+x2**4*x3**2*&
&x6+x1**4*x3**2*x5
dpdx4(441) = x2**4*x5*x6**2+4*x2*x3*x4**3*x6**2+x1**4*x5**2*x6+4*&
&x1*x3*x4**3*x5**2
dpdx4(442) = 4*x2*x4**3*x5*x6**2+x2**4*x3*x6**2+4*x1*x4**3*x5**2*&
&x6+x1**4*x3*x5**2
dpdx4(443) = 4*x1*x2*x4**3*x6**2+x1*x2**4*x6**2+4*x1*x2*x4**3*x5*&
&*2+x1**4*x2*x5**2+x1*x2**4*x3**2+x1**4*x2*x3**2
dpdx4(444) = 4*x1**2*x3*x4**3*x6+2*x1**4*x3*x4*x6+4*x2**2*x3*x4**&
&3*x5+2*x2**4*x3*x4*x5
dpdx4(445) = 2*x1**4*x2*x4*x6+x1**4*x2**2*x6+2*x1*x2**4*x4*x5+x1*&
&*2*x2**4*x5+4*x1*x2**2*x3*x4**3+4*x1**2*x2*x3*x4**3
dpdx4(446) = 4*x1**2*x2*x4**3*x6+x1**2*x2**4*x6+4*x1*x2**2*x4**3*&
&x5+x1**4*x2**2*x5+2*x1*x2**4*x3*x4+2*x1**4*x2*x3*x4
dpdx4(447) = 2*x2**4*x4*x5*x6+2*x1**4*x4*x5*x6+4*x2**2*x3*x4**3*x&
&6+4*x1**2*x3*x4**3*x5
dpdx4(448) = 4*x2**2*x4**3*x5*x6+4*x1**2*x4**3*x5*x6+2*x2**4*x3*x&
&4*x6+2*x1**4*x3*x4*x5
dpdx4(449) = 4*x1*x2**2*x4**3*x6+2*x1*x2**4*x4*x6+4*x1**2*x2*x4**&
&3*x5+2*x1**4*x2*x4*x5+x1**2*x2**4*x3+x1**4*x2**2*x3
dpdx4(450) = 5*x3*x4**4*x5*x6
dpdx4(451) = 5*x1*x3*x4**4*x6+x1**5*x3*x6+5*x2*x3*x4**4*x5+x2**5*&
&x3*x5
dpdx4(452) = x1**5*x2*x6+x1*x2**5*x5+5*x1*x2*x3*x4**4
dpdx4(453) = x2**5*x5*x6+x1**5*x5*x6+5*x2*x3*x4**4*x6+5*x1*x3*x4*&
&*4*x5
dpdx4(454) = 5*x2*x4**4*x5*x6+5*x1*x4**4*x5*x6+x2**5*x3*x6+x1**5*&
&x3*x5
dpdx4(455) = 5*x1*x2*x4**4*x6+x1*x2**5*x6+5*x1*x2*x4**4*x5+x1**5*&
&x2*x5+x1*x2**5*x3+x1**5*x2*x3
dpdx4(456) = 0
dpdx4(457) = 0
dpdx4(458) = 0
dpdx4(459) = 0
dpdx4(460) = 0
dpdx4(461) = x3**6*x6+x3**6*x5
dpdx4(462) = x3**5*x6**2+x3**5*x5**2
dpdx4(463) = x3**4*x6**3+x3**4*x5**3
dpdx4(464) = x3**3*x6**4+x3**3*x5**4
dpdx4(465) = x5**3*x6**4+x5**4*x6**3
dpdx4(466) = x3**2*x6**5+x3**2*x5**5
dpdx4(467) = x5**2*x6**5+x5**5*x6**2
dpdx4(468) = x3*x6**6+x3*x5**6
dpdx4(469) = x5*x6**6+x5**6*x6
dpdx4(470) = x1*x6**6+x2*x5**6+x2*x3**6+x1*x3**6
dpdx4(471) = x2*x6**6+x1*x5**6
dpdx4(472) = 2*x3**5*x4*x6+2*x3**5*x4*x5
dpdx4(473) = 2*x3**4*x4*x6**2+2*x3**4*x4*x5**2
dpdx4(474) = 2*x3**3*x4*x6**3+2*x3**3*x4*x5**3
dpdx4(475) = 2*x4*x5**3*x6**3
dpdx4(476) = 2*x3**2*x4*x6**4+2*x3**2*x4*x5**4
dpdx4(477) = 2*x4*x5**2*x6**4+2*x4*x5**4*x6**2
dpdx4(478) = 2*x3*x4*x6**5+2*x3*x4*x5**5
dpdx4(479) = 2*x4*x5*x6**5+2*x4*x5**5*x6
dpdx4(480) = x1**2*x6**5+x2**2*x5**5+2*x2*x3**5*x4+2*x1*x3**5*x4
dpdx4(481) = 2*x1*x4*x6**5+2*x2*x4*x5**5+x2**2*x3**5+x1**2*x3**5
dpdx4(482) = 2*x2*x4*x6**5+x2**2*x6**5+2*x1*x4*x5**5+x1**2*x5**5
dpdx4(483) = 2*x1**2*x4*x6**4+2*x2**2*x4*x5**4+2*x2**2*x3**4*x4+2&
&*x1**2*x3**4*x4
dpdx4(484) = 2*x2**2*x4*x6**4+2*x1**2*x4*x5**4
dpdx4(485) = 3*x3**4*x4**2*x6+3*x3**4*x4**2*x5
dpdx4(486) = 3*x3**3*x4**2*x6**2+3*x3**3*x4**2*x5**2
dpdx4(487) = 3*x3**2*x4**2*x6**3+3*x3**2*x4**2*x5**3
dpdx4(488) = 3*x4**2*x5**2*x6**3+3*x4**2*x5**3*x6**2
dpdx4(489) = 3*x3*x4**2*x6**4+3*x3*x4**2*x5**4
dpdx4(490) = 3*x4**2*x5*x6**4+3*x4**2*x5**4*x6
dpdx4(491) = x1**3*x6**4+x2**3*x5**4+3*x2*x3**4*x4**2+3*x1*x3**4*&
&x4**2
dpdx4(492) = 3*x1*x4**2*x6**4+3*x2*x4**2*x5**4+x2**3*x3**4+x1**3*&
&x3**4
dpdx4(493) = 3*x2*x4**2*x6**4+x2**3*x6**4+3*x1*x4**2*x5**4+x1**3*&
&x5**4
dpdx4(494) = 2*x1**3*x4*x6**3+2*x2**3*x4*x5**3+3*x2**2*x3**3*x4**&
&2+3*x1**2*x3**3*x4**2
dpdx4(495) = 3*x1**2*x4**2*x6**3+3*x2**2*x4**2*x5**3+2*x2**3*x3**&
&3*x4+2*x1**3*x3**3*x4
dpdx4(496) = 3*x2**2*x4**2*x6**3+2*x2**3*x4*x6**3+3*x1**2*x4**2*x&
&5**3+2*x1**3*x4*x5**3
dpdx4(497) = 3*x1**3*x4**2*x6**2+3*x2**3*x4**2*x5**2+3*x2**3*x3**&
&2*x4**2+3*x1**3*x3**2*x4**2
dpdx4(498) = 3*x1**2*x2**3*x4**2+3*x1**3*x2**2*x4**2+2*x1**3*x2**&
&3*x4
dpdx4(499) = 3*x2**3*x4**2*x6**2+3*x1**3*x4**2*x5**2
dpdx4(500) = 4*x3**3*x4**3*x6+4*x3**3*x4**3*x5
dpdx4(501) = 4*x3**2*x4**3*x6**2+4*x3**2*x4**3*x5**2
dpdx4(502) = 4*x4**3*x5**2*x6**2
dpdx4(503) = 4*x3*x4**3*x6**3+4*x3*x4**3*x5**3
dpdx4(504) = 4*x4**3*x5*x6**3+4*x4**3*x5**3*x6
dpdx4(505) = x1**4*x6**3+x2**4*x5**3+4*x2*x3**3*x4**3+4*x1*x3**3*&
&x4**3
dpdx4(506) = 4*x1*x4**3*x6**3+4*x2*x4**3*x5**3+x2**4*x3**3+x1**4*&
&x3**3
dpdx4(507) = 4*x2*x4**3*x6**3+x2**4*x6**3+4*x1*x4**3*x5**3+x1**4*&
&x5**3
dpdx4(508) = 2*x1**4*x4*x6**2+2*x2**4*x4*x5**2+4*x2**2*x3**2*x4**&
&3+4*x1**2*x3**2*x4**3
dpdx4(509) = 4*x1**2*x4**3*x6**2+4*x2**2*x4**3*x5**2+2*x2**4*x3**&
&2*x4+2*x1**4*x3**2*x4
dpdx4(510) = 4*x1**2*x2**2*x4**3+2*x1**2*x2**4*x4+2*x1**4*x2**2*x&
&4
dpdx4(511) = 4*x2**2*x4**3*x6**2+2*x2**4*x4*x6**2+4*x1**2*x4**3*x&
&5**2+2*x1**4*x4*x5**2
dpdx4(512) = 3*x1**4*x4**2*x6+3*x2**4*x4**2*x5+4*x2**3*x3*x4**3+4&
&*x1**3*x3*x4**3
dpdx4(513) = 4*x1**3*x4**3*x6+4*x2**3*x4**3*x5+3*x2**4*x3*x4**2+3&
&*x1**4*x3*x4**2
dpdx4(514) = 4*x1*x2**3*x4**3+4*x1**3*x2*x4**3+3*x1*x2**4*x4**2+3&
&*x1**4*x2*x4**2+x1**3*x2**4+x1**4*x2**3
dpdx4(515) = 4*x2**3*x4**3*x6+3*x2**4*x4**2*x6+4*x1**3*x4**3*x5+3&
&*x1**4*x4**2*x5
dpdx4(516) = 5*x3**2*x4**4*x6+5*x3**2*x4**4*x5
dpdx4(517) = 5*x3*x4**4*x6**2+5*x3*x4**4*x5**2
dpdx4(518) = 5*x4**4*x5*x6**2+5*x4**4*x5**2*x6
dpdx4(519) = x1**5*x6**2+x2**5*x5**2+5*x2*x3**2*x4**4+5*x1*x3**2*&
&x4**4
dpdx4(520) = 5*x1*x4**4*x6**2+5*x2*x4**4*x5**2+x2**5*x3**2+x1**5*&
&x3**2
dpdx4(521) = 5*x2*x4**4*x6**2+x2**5*x6**2+5*x1*x4**4*x5**2+x1**5*&
&x5**2
dpdx4(522) = 2*x1**5*x4*x6+2*x2**5*x4*x5+5*x2**2*x3*x4**4+5*x1**2&
&*x3*x4**4
dpdx4(523) = 5*x1**2*x4**4*x6+5*x2**2*x4**4*x5+2*x2**5*x3*x4+2*x1&
&**5*x3*x4
dpdx4(524) = 5*x1*x2**2*x4**4+5*x1**2*x2*x4**4+2*x1*x2**5*x4+2*x1&
&**5*x2*x4+x1**2*x2**5+x1**5*x2**2
dpdx4(525) = 5*x2**2*x4**4*x6+2*x2**5*x4*x6+5*x1**2*x4**4*x5+2*x1&
&**5*x4*x5
dpdx4(526) = 6*x3*x4**5*x6+6*x3*x4**5*x5
dpdx4(527) = 6*x4**5*x5*x6
dpdx4(528) = x1**6*x6+x2**6*x5+6*x2*x3*x4**5+6*x1*x3*x4**5
dpdx4(529) = 6*x1*x4**5*x6+6*x2*x4**5*x5+x2**6*x3+x1**6*x3
dpdx4(530) = 6*x1*x2*x4**5+x1*x2**6+x1**6*x2
dpdx4(531) = 6*x2*x4**5*x6+x2**6*x6+6*x1*x4**5*x5+x1**6*x5
dpdx4(532) = 0
dpdx4(533) = 0
dpdx4(534) = 0
dpdx4(535) = 0
dpdx4(536) = x6**7+x5**7
dpdx4(537) = 2*x4*x6**6+2*x4*x5**6
dpdx4(538) = 3*x4**2*x6**5+3*x4**2*x5**5
dpdx4(539) = 4*x4**3*x6**4+4*x4**3*x5**4
dpdx4(540) = 4*x2**4*x4**3+4*x1**4*x4**3
dpdx4(541) = 5*x4**4*x6**3+5*x4**4*x5**3
dpdx4(542) = 5*x2**3*x4**4+5*x1**3*x4**4+3*x2**5*x4**2+3*x1**5*x4&
&**2
dpdx4(543) = 6*x4**5*x6**2+6*x4**5*x5**2
dpdx4(544) = 6*x2**2*x4**5+6*x1**2*x4**5+2*x2**6*x4+2*x1**6*x4
dpdx4(545) = 7*x4**6*x6+7*x4**6*x5
dpdx4(546) = 7*x2*x4**6+7*x1*x4**6+x2**7+x1**7
dpdx5(1) = 0
dpdx5(2) = x6+x3
dpdx5(3) = x4+x1
dpdx5(4) = 0
dpdx5(5) = x3*x6
dpdx5(6) = x2*x6+x1*x6+x3*x4+x2*x3
dpdx5(7) = x4*x6+x1*x3
dpdx5(8) = x2*x4+x1*x2
dpdx5(9) = 0
dpdx5(10) = x1*x4
dpdx5(11) = x6**2+2*x5*x6+2*x3*x5+x3**2
dpdx5(12) = 2*x4*x5+2*x1*x5
dpdx5(13) = x4**2+x1**2
dpdx5(14) = 0
dpdx5(15) = x3*x4*x6+x2*x3*x6+x1*x3*x6
dpdx5(16) = x1*x2*x6+x2*x3*x4
dpdx5(17) = x2*x4*x6+x1*x4*x6+x1*x3*x4+x1*x2*x3
dpdx5(18) = x1*x2*x4
dpdx5(19) = x3*x6**2+2*x3*x5*x6+x3**2*x6
dpdx5(20) = x1*x6**2+2*x2*x5*x6+2*x2*x3*x5+x3**2*x4
dpdx5(21) = x2*x6**2+2*x1*x5*x6+2*x3*x4*x5+x2*x3**2
dpdx5(22) = x4*x6**2+2*x4*x5*x6+2*x1*x3*x5+x1*x3**2
dpdx5(23) = 2*x2*x4*x5+2*x1*x2*x5
dpdx5(24) = 2*x1*x4*x5
dpdx5(25) = x2**2*x6+x1**2*x6+x3*x4**2+x2**2*x3
dpdx5(26) = x4**2*x6+x1**2*x3
dpdx5(27) = x2**2*x4+x1*x2**2
dpdx5(28) = x2*x4**2+x1**2*x2
dpdx5(29) = 0
dpdx5(30) = x1*x4**2+x1**2*x4
dpdx5(31) = 2*x5*x6**2+2*x3**2*x5
dpdx5(32) = x6**3+3*x5**2*x6+3*x3*x5**2+x3**3
dpdx5(33) = 3*x4*x5**2+3*x1*x5**2
dpdx5(34) = 2*x4**2*x5+2*x1**2*x5
dpdx5(35) = 0
dpdx5(36) = x4**3+x1**3
dpdx5(37) = 0
dpdx5(38) = x2*x3*x4*x6+x1*x3*x4*x6+x1*x2*x3*x6
dpdx5(39) = x1*x2*x4*x6+x1*x2*x3*x4
dpdx5(40) = x1*x3*x6**2+2*x2*x3*x5*x6+x3**2*x4*x6
dpdx5(41) = x3*x4*x6**2+x2*x3*x6**2+2*x3*x4*x5*x6+2*x1*x3*x5*x6+x&
&2*x3**2*x6+x1*x3**2*x6
dpdx5(42) = x1*x2*x6**2+2*x1*x2*x5*x6+2*x2*x3*x4*x5+x2*x3**2*x4
dpdx5(43) = x1*x4*x6**2+2*x2*x4*x5*x6+2*x1*x2*x3*x5+x1*x3**2*x4
dpdx5(44) = x2*x4*x6**2+2*x1*x4*x5*x6+2*x1*x3*x4*x5+x1*x2*x3**2
dpdx5(45) = 2*x1*x2*x4*x5
dpdx5(46) = x3*x4**2*x6+x2**2*x3*x6+x1**2*x3*x6
dpdx5(47) = x1*x2**2*x6+x1**2*x2*x6+x2*x3*x4**2+x2**2*x3*x4
dpdx5(48) = x1*x2**2*x4
dpdx5(49) = x2**2*x4*x6+x1**2*x4*x6+x1*x3*x4**2+x1*x2**2*x3
dpdx5(50) = x2*x4**2*x6+x1*x4**2*x6+x1**2*x3*x4+x1**2*x2*x3
dpdx5(51) = x1*x2*x4**2+x1**2*x2*x4
dpdx5(52) = 2*x3*x5*x6**2+x3**2*x6**2+2*x3**2*x5*x6
dpdx5(53) = x3*x6**3+3*x3*x5**2*x6+x3**3*x6
dpdx5(54) = x1*x6**3+3*x2*x5**2*x6+3*x2*x3*x5**2+x3**3*x4
dpdx5(55) = 2*x2*x5*x6**2+2*x1*x5*x6**2+2*x3**2*x4*x5+2*x2*x3**2*&
&x5
dpdx5(56) = 2*x4*x5*x6**2+2*x1*x3**2*x5
dpdx5(57) = x2*x6**3+3*x1*x5**2*x6+3*x3*x4*x5**2+x2*x3**3
dpdx5(58) = x4*x6**3+3*x4*x5**2*x6+3*x1*x3*x5**2+x1*x3**3
dpdx5(59) = 3*x2*x4*x5**2+3*x1*x2*x5**2
dpdx5(60) = 3*x1*x4*x5**2
dpdx5(61) = x1**2*x6**2+2*x2**2*x5*x6+2*x2**2*x3*x5+x3**2*x4**2
dpdx5(62) = x2**2*x6**2+2*x1**2*x5*x6+2*x3*x4**2*x5+x2**2*x3**2
dpdx5(63) = x4**2*x6**2+2*x4**2*x5*x6+2*x1**2*x3*x5+x1**2*x3**2
dpdx5(64) = 2*x2**2*x4*x5+2*x1*x2**2*x5
dpdx5(65) = 2*x2*x4**2*x5+2*x1**2*x2*x5
dpdx5(66) = 2*x1*x4**2*x5+2*x1**2*x4*x5
dpdx5(67) = x2**2*x4**2+x1**2*x2**2
dpdx5(68) = 0
dpdx5(69) = x1**2*x4**2
dpdx5(70) = x2**3*x6+x1**3*x6+x3*x4**3+x2**3*x3
dpdx5(71) = x4**3*x6+x1**3*x3
dpdx5(72) = x2**3*x4+x1*x2**3
dpdx5(73) = x2*x4**3+x1**3*x2
dpdx5(74) = 0
dpdx5(75) = x1*x4**3+x1**3*x4
dpdx5(76) = 2*x5*x6**3+3*x5**2*x6**2+3*x3**2*x5**2+2*x3**3*x5
dpdx5(77) = x6**4+4*x5**3*x6+4*x3*x5**3+x3**4
dpdx5(78) = 4*x4*x5**3+4*x1*x5**3
dpdx5(79) = 3*x4**2*x5**2+3*x1**2*x5**2
dpdx5(80) = 2*x4**3*x5+2*x1**3*x5
dpdx5(81) = 0
dpdx5(82) = x4**4+x1**4
dpdx5(83) = 0
dpdx5(84) = x1*x2*x3*x4*x6
dpdx5(85) = x1*x3*x4*x6**2+x1*x2*x3*x6**2+2*x2*x3*x4*x5*x6+2*x1*x&
&2*x3*x5*x6+x2*x3**2*x4*x6+x1*x3**2*x4*x6
dpdx5(86) = x2*x3*x4*x6**2+2*x1*x3*x4*x5*x6+x1*x2*x3**2*x6
dpdx5(87) = x1*x2*x4*x6**2+2*x1*x2*x4*x5*x6+2*x1*x2*x3*x4*x5+x1*x&
&2*x3**2*x4
dpdx5(88) = x2*x3*x4**2*x6+x1*x3*x4**2*x6+x2**2*x3*x4*x6+x1**2*x3&
&*x4*x6+x1*x2**2*x3*x6+x1**2*x2*x3*x6
dpdx5(89) = x1*x2**2*x4*x6+x1**2*x2*x4*x6+x1*x2*x3*x4**2+x1*x2**2&
&*x3*x4
dpdx5(90) = x1*x2*x4**2*x6+x1**2*x2*x3*x4
dpdx5(91) = x1*x3*x6**3+3*x2*x3*x5**2*x6+x3**3*x4*x6
dpdx5(92) = 2*x2*x3*x5*x6**2+2*x1*x3*x5*x6**2+x3**2*x4*x6**2+x1*x&
&3**2*x6**2+2*x3**2*x4*x5*x6+2*x2*x3**2*x5*x6
dpdx5(93) = 2*x3*x4*x5*x6**2+x2*x3**2*x6**2+2*x1*x3**2*x5*x6
dpdx5(94) = x3*x4*x6**3+x2*x3*x6**3+3*x3*x4*x5**2*x6+3*x1*x3*x5**&
&2*x6+x2*x3**3*x6+x1*x3**3*x6
dpdx5(95) = 2*x1*x2*x5*x6**2+2*x2*x3**2*x4*x5
dpdx5(96) = x1*x2*x6**3+3*x1*x2*x5**2*x6+3*x2*x3*x4*x5**2+x2*x3**&
&3*x4
dpdx5(97) = x1*x4*x6**3+3*x2*x4*x5**2*x6+3*x1*x2*x3*x5**2+x1*x3**&
&3*x4
dpdx5(98) = 2*x2*x4*x5*x6**2+2*x1*x4*x5*x6**2+2*x1*x3**2*x4*x5+2*&
&x1*x2*x3**2*x5
dpdx5(99) = x2*x4*x6**3+3*x1*x4*x5**2*x6+3*x1*x3*x4*x5**2+x1*x2*x&
&3**3
dpdx5(100) = 3*x1*x2*x4*x5**2
dpdx5(101) = x1**2*x3*x6**2+2*x2**2*x3*x5*x6+x3**2*x4**2*x6
dpdx5(102) = x3*x4**2*x6**2+x2**2*x3*x6**2+2*x3*x4**2*x5*x6+2*x1*&
&*2*x3*x5*x6+x2**2*x3**2*x6+x1**2*x3**2*x6
dpdx5(103) = x1**2*x2*x6**2+2*x1*x2**2*x5*x6+2*x2**2*x3*x4*x5+x2*&
&x3**2*x4**2
dpdx5(104) = x1*x2**2*x6**2+2*x1**2*x2*x5*x6+2*x2*x3*x4**2*x5+x2*&
&*2*x3**2*x4
dpdx5(105) = 2*x1*x2**2*x4*x5
dpdx5(106) = x1**2*x4*x6**2+2*x2**2*x4*x5*x6+2*x1*x2**2*x3*x5+x1*&
&x3**2*x4**2
dpdx5(107) = x1*x4**2*x6**2+2*x2*x4**2*x5*x6+2*x1**2*x2*x3*x5+x1*&
&*2*x3**2*x4
dpdx5(108) = x2**2*x4*x6**2+2*x1**2*x4*x5*x6+2*x1*x3*x4**2*x5+x1*&
&x2**2*x3**2
dpdx5(109) = x2*x4**2*x6**2+2*x1*x4**2*x5*x6+2*x1**2*x3*x4*x5+x1*&
&*2*x2*x3**2
dpdx5(110) = 2*x1*x2*x4**2*x5+2*x1**2*x2*x4*x5
dpdx5(111) = x1**2*x2**2*x6+x2**2*x3*x4**2
dpdx5(112) = x1*x2**2*x4**2+x1**2*x2**2*x4
dpdx5(113) = x2**2*x4**2*x6+x1**2*x4**2*x6+x1**2*x3*x4**2+x1**2*x&
&2**2*x3
dpdx5(114) = x1**2*x2*x4**2
dpdx5(115) = x3*x4**3*x6+x2**3*x3*x6+x1**3*x3*x6
dpdx5(116) = x1*x2**3*x6+x1**3*x2*x6+x2*x3*x4**3+x2**3*x3*x4
dpdx5(117) = x1*x2**3*x4
dpdx5(118) = x2**3*x4*x6+x1**3*x4*x6+x1*x3*x4**3+x1*x2**3*x3
dpdx5(119) = x2*x4**3*x6+x1*x4**3*x6+x1**3*x3*x4+x1**3*x2*x3
dpdx5(120) = x1*x2*x4**3+x1**3*x2*x4
dpdx5(121) = 2*x3**2*x5*x6**2
dpdx5(122) = 2*x3*x5*x6**3+x3**2*x6**3+3*x3*x5**2*x6**2+x3**3*x6*&
&*2+3*x3**2*x5**2*x6+2*x3**3*x5*x6
dpdx5(123) = x3*x6**4+4*x3*x5**3*x6+x3**4*x6
dpdx5(124) = x1*x6**4+4*x2*x5**3*x6+4*x2*x3*x5**3+x3**4*x4
dpdx5(125) = 2*x1*x5*x6**3+3*x2*x5**2*x6**2+3*x2*x3**2*x5**2+2*x3&
&**3*x4*x5
dpdx5(126) = 2*x2*x5*x6**3+3*x1*x5**2*x6**2+3*x3**2*x4*x5**2+2*x2&
&*x3**3*x5
dpdx5(127) = 2*x4*x5*x6**3+3*x4*x5**2*x6**2+3*x1*x3**2*x5**2+2*x1&
&*x3**3*x5
dpdx5(128) = x2*x6**4+4*x1*x5**3*x6+4*x3*x4*x5**3+x2*x3**4
dpdx5(129) = x4*x6**4+4*x4*x5**3*x6+4*x1*x3*x5**3+x1*x3**4
dpdx5(130) = 4*x2*x4*x5**3+4*x1*x2*x5**3
dpdx5(131) = 4*x1*x4*x5**3
dpdx5(132) = x1**2*x6**3+3*x2**2*x5**2*x6+3*x2**2*x3*x5**2+x3**3*&
&x4**2
dpdx5(133) = 2*x2**2*x5*x6**2+2*x1**2*x5*x6**2+2*x3**2*x4**2*x5+2&
&*x2**2*x3**2*x5
dpdx5(134) = 2*x4**2*x5*x6**2+2*x1**2*x3**2*x5
dpdx5(135) = x2**2*x6**3+3*x1**2*x5**2*x6+3*x3*x4**2*x5**2+x2**2*&
&x3**3
dpdx5(136) = x4**2*x6**3+3*x4**2*x5**2*x6+3*x1**2*x3*x5**2+x1**2*&
&x3**3
dpdx5(137) = 3*x2**2*x4*x5**2+3*x1*x2**2*x5**2
dpdx5(138) = 3*x2*x4**2*x5**2+3*x1**2*x2*x5**2
dpdx5(139) = 3*x1*x4**2*x5**2+3*x1**2*x4*x5**2
dpdx5(140) = 2*x2**2*x4**2*x5+2*x1**2*x2**2*x5
dpdx5(141) = 0
dpdx5(142) = 2*x1**2*x4**2*x5
dpdx5(143) = x1**3*x6**2+2*x2**3*x5*x6+2*x2**3*x3*x5+x3**2*x4**3
dpdx5(144) = x2**3*x6**2+2*x1**3*x5*x6+2*x3*x4**3*x5+x2**3*x3**2
dpdx5(145) = x4**3*x6**2+2*x4**3*x5*x6+2*x1**3*x3*x5+x1**3*x3**2
dpdx5(146) = 2*x2**3*x4*x5+2*x1*x2**3*x5
dpdx5(147) = 2*x2*x4**3*x5+2*x1**3*x2*x5
dpdx5(148) = 2*x1*x4**3*x5+2*x1**3*x4*x5
dpdx5(149) = x2**3*x4**2+x1**2*x2**3
dpdx5(150) = x2**2*x4**3+x1**3*x2**2
dpdx5(151) = 0
dpdx5(152) = x1**2*x4**3+x1**3*x4**2
dpdx5(153) = x2**4*x6+x1**4*x6+x3*x4**4+x2**4*x3
dpdx5(154) = x4**4*x6+x1**4*x3
dpdx5(155) = x2**4*x4+x1*x2**4
dpdx5(156) = x2*x4**4+x1**4*x2
dpdx5(157) = 0
dpdx5(158) = x1*x4**4+x1**4*x4
dpdx5(159) = 3*x5**2*x6**3+3*x3**3*x5**2
dpdx5(160) = 2*x5*x6**4+4*x5**3*x6**2+4*x3**2*x5**3+2*x3**4*x5
dpdx5(161) = x6**5+5*x5**4*x6+5*x3*x5**4+x3**5
dpdx5(162) = 5*x4*x5**4+5*x1*x5**4
dpdx5(163) = 4*x4**2*x5**3+4*x1**2*x5**3
dpdx5(164) = 3*x4**3*x5**2+3*x1**3*x5**2
dpdx5(165) = 0
dpdx5(166) = 2*x4**4*x5+2*x1**4*x5
dpdx5(167) = 0
dpdx5(168) = x4**5+x1**5
dpdx5(169) = 0
dpdx5(170) = x1*x2*x3*x4*x6**2+2*x1*x2*x3*x4*x5*x6+x1*x2*x3**2*x4&
&*x6
dpdx5(171) = x1*x2*x3*x4**2*x6+x1*x2**2*x3*x4*x6+x1**2*x2*x3*x4*x&
&6
dpdx5(172) = 2*x1*x2*x3*x5*x6**2+x1*x3**2*x4*x6**2+2*x2*x3**2*x4*&
&x5*x6
dpdx5(173) = x1*x3*x4*x6**3+x1*x2*x3*x6**3+3*x2*x3*x4*x5**2*x6+3*&
&x1*x2*x3*x5**2*x6+x2*x3**3*x4*x6+x1*x3**3*x4*x6
dpdx5(174) = 2*x2*x3*x4*x5*x6**2+2*x1*x3*x4*x5*x6**2+x2*x3**2*x4*&
&x6**2+x1*x2*x3**2*x6**2+2*x1*x3**2*x4*x5*x6+2*x1*x2*x3**2*x5*x6
dpdx5(175) = 2*x1*x2*x4*x5*x6**2+2*x1*x2*x3**2*x4*x5
dpdx5(176) = x2*x3*x4*x6**3+3*x1*x3*x4*x5**2*x6+x1*x2*x3**3*x6
dpdx5(177) = x1*x2*x4*x6**3+3*x1*x2*x4*x5**2*x6+3*x1*x2*x3*x4*x5*&
&*2+x1*x2*x3**3*x4
dpdx5(178) = x1**2*x3*x4*x6**2+x1**2*x2*x3*x6**2+2*x2**2*x3*x4*x5&
&*x6+2*x1*x2**2*x3*x5*x6+x2*x3**2*x4**2*x6+x1*x3**2*x4**2*x6
dpdx5(179) = x1*x3*x4**2*x6**2+x1*x2**2*x3*x6**2+2*x2*x3*x4**2*x5&
&*x6+2*x1**2*x2*x3*x5*x6+x2**2*x3**2*x4*x6+x1**2*x3**2*x4*x6
dpdx5(180) = x1**2*x2*x4*x6**2+2*x1*x2**2*x4*x5*x6+2*x1*x2**2*x3*&
&x4*x5+x1*x2*x3**2*x4**2
dpdx5(181) = x2*x3*x4**2*x6**2+x2**2*x3*x4*x6**2+2*x1*x3*x4**2*x5&
&*x6+2*x1**2*x3*x4*x5*x6+x1*x2**2*x3**2*x6+x1**2*x2*x3**2*x6
dpdx5(182) = x1*x2**2*x4*x6**2+2*x1**2*x2*x4*x5*x6+2*x1*x2*x3*x4*&
&*2*x5+x1*x2**2*x3**2*x4
dpdx5(183) = x1*x2*x4**2*x6**2+2*x1*x2*x4**2*x5*x6+2*x1**2*x2*x3*&
&x4*x5+x1**2*x2*x3**2*x4
dpdx5(184) = x1**2*x2**2*x4*x6+x1*x2**2*x3*x4**2
dpdx5(185) = x2**2*x3*x4**2*x6+x1**2*x3*x4**2*x6+x1**2*x2**2*x3*x&
&6
dpdx5(186) = x1*x2**2*x4**2*x6+x1**2*x2*x4**2*x6+x1**2*x2*x3*x4**&
&2+x1**2*x2**2*x3*x4
dpdx5(187) = x2*x3*x4**3*x6+x1*x3*x4**3*x6+x2**3*x3*x4*x6+x1**3*x&
&3*x4*x6+x1*x2**3*x3*x6+x1**3*x2*x3*x6
dpdx5(188) = x1*x2**3*x4*x6+x1**3*x2*x4*x6+x1*x2*x3*x4**3+x1*x2**&
&3*x3*x4
dpdx5(189) = x1*x2*x4**3*x6+x1**3*x2*x3*x4
dpdx5(190) = x1*x3*x6**4+4*x2*x3*x5**3*x6+x3**4*x4*x6
dpdx5(191) = 2*x1*x3*x5*x6**3+x1*x3**2*x6**3+3*x2*x3*x5**2*x6**2+&
&x3**3*x4*x6**2+3*x2*x3**2*x5**2*x6+2*x3**3*x4*x5*x6
dpdx5(192) = 2*x3**2*x4*x5*x6**2+2*x2*x3**2*x5*x6**2+2*x1*x3**2*x&
&5*x6**2
dpdx5(193) = 2*x2*x3*x5*x6**3+x3**2*x4*x6**3+3*x1*x3*x5**2*x6**2+&
&x1*x3**3*x6**2+3*x3**2*x4*x5**2*x6+2*x2*x3**3*x5*x6
dpdx5(194) = 2*x3*x4*x5*x6**3+x2*x3**2*x6**3+3*x3*x4*x5**2*x6**2+&
&x2*x3**3*x6**2+3*x1*x3**2*x5**2*x6+2*x1*x3**3*x5*x6
dpdx5(195) = x3*x4*x6**4+x2*x3*x6**4+4*x3*x4*x5**3*x6+4*x1*x3*x5*&
&*3*x6+x2*x3**4*x6+x1*x3**4*x6
dpdx5(196) = 2*x1*x2*x5*x6**3+3*x1*x2*x5**2*x6**2+3*x2*x3**2*x4*x&
&5**2+2*x2*x3**3*x4*x5
dpdx5(197) = x1*x2*x6**4+4*x1*x2*x5**3*x6+4*x2*x3*x4*x5**3+x2*x3*&
&*4*x4
dpdx5(198) = x1*x4*x6**4+4*x2*x4*x5**3*x6+4*x1*x2*x3*x5**3+x1*x3*&
&*4*x4
dpdx5(199) = 2*x1*x4*x5*x6**3+3*x2*x4*x5**2*x6**2+3*x1*x2*x3**2*x&
&5**2+2*x1*x3**3*x4*x5
dpdx5(200) = 2*x2*x4*x5*x6**3+3*x1*x4*x5**2*x6**2+3*x1*x3**2*x4*x&
&5**2+2*x1*x2*x3**3*x5
dpdx5(201) = x2*x4*x6**4+4*x1*x4*x5**3*x6+4*x1*x3*x4*x5**3+x1*x2*&
&x3**4
dpdx5(202) = 4*x1*x2*x4*x5**3
dpdx5(203) = x1**2*x3*x6**3+3*x2**2*x3*x5**2*x6+x3**3*x4**2*x6
dpdx5(204) = 2*x2**2*x3*x5*x6**2+2*x1**2*x3*x5*x6**2+x3**2*x4**2*&
&x6**2+x1**2*x3**2*x6**2+2*x3**2*x4**2*x5*x6+2*x2**2*x3**2*x5*x6
dpdx5(205) = 2*x3*x4**2*x5*x6**2+x2**2*x3**2*x6**2+2*x1**2*x3**2*&
&x5*x6
dpdx5(206) = x3*x4**2*x6**3+x2**2*x3*x6**3+3*x3*x4**2*x5**2*x6+3*&
&x1**2*x3*x5**2*x6+x2**2*x3**3*x6+x1**2*x3**3*x6
dpdx5(207) = x1**2*x2*x6**3+3*x1*x2**2*x5**2*x6+3*x2**2*x3*x4*x5*&
&*2+x2*x3**3*x4**2
dpdx5(208) = 2*x1*x2**2*x5*x6**2+2*x1**2*x2*x5*x6**2+2*x2*x3**2*x&
&4**2*x5+2*x2**2*x3**2*x4*x5
dpdx5(209) = x1*x2**2*x6**3+3*x1**2*x2*x5**2*x6+3*x2*x3*x4**2*x5*&
&*2+x2**2*x3**3*x4
dpdx5(210) = 3*x1*x2**2*x4*x5**2
dpdx5(211) = x1**2*x4*x6**3+3*x2**2*x4*x5**2*x6+3*x1*x2**2*x3*x5*&
&*2+x1*x3**3*x4**2
dpdx5(212) = x1*x4**2*x6**3+3*x2*x4**2*x5**2*x6+3*x1**2*x2*x3*x5*&
&*2+x1**2*x3**3*x4
dpdx5(213) = 2*x2**2*x4*x5*x6**2+2*x1**2*x4*x5*x6**2+2*x1*x3**2*x&
&4**2*x5+2*x1*x2**2*x3**2*x5
dpdx5(214) = 2*x2*x4**2*x5*x6**2+2*x1*x4**2*x5*x6**2+2*x1**2*x3**&
&2*x4*x5+2*x1**2*x2*x3**2*x5
dpdx5(215) = x2**2*x4*x6**3+3*x1**2*x4*x5**2*x6+3*x1*x3*x4**2*x5*&
&*2+x1*x2**2*x3**3
dpdx5(216) = x2*x4**2*x6**3+3*x1*x4**2*x5**2*x6+3*x1**2*x3*x4*x5*&
&*2+x1**2*x2*x3**3
dpdx5(217) = 3*x1*x2*x4**2*x5**2+3*x1**2*x2*x4*x5**2
dpdx5(218) = x1**2*x2**2*x6**2+2*x1**2*x2**2*x5*x6+2*x2**2*x3*x4*&
&*2*x5+x2**2*x3**2*x4**2
dpdx5(219) = 2*x1*x2**2*x4**2*x5+2*x1**2*x2**2*x4*x5
dpdx5(220) = x1**2*x4**2*x6**2+2*x2**2*x4**2*x5*x6+2*x1**2*x2**2*&
&x3*x5+x1**2*x3**2*x4**2
dpdx5(221) = x1**2*x2**2*x4**2
dpdx5(222) = x2**2*x4**2*x6**2+2*x1**2*x4**2*x5*x6+2*x1**2*x3*x4*&
&*2*x5+x1**2*x2**2*x3**2
dpdx5(223) = 2*x1**2*x2*x4**2*x5
dpdx5(224) = x1**3*x3*x6**2+2*x2**3*x3*x5*x6+x3**2*x4**3*x6
dpdx5(225) = x3*x4**3*x6**2+x2**3*x3*x6**2+2*x3*x4**3*x5*x6+2*x1*&
&*3*x3*x5*x6+x2**3*x3**2*x6+x1**3*x3**2*x6
dpdx5(226) = x1**3*x2*x6**2+2*x1*x2**3*x5*x6+2*x2**3*x3*x4*x5+x2*&
&x3**2*x4**3
dpdx5(227) = x1*x2**3*x6**2+2*x1**3*x2*x5*x6+2*x2*x3*x4**3*x5+x2*&
&*3*x3**2*x4
dpdx5(228) = 2*x1*x2**3*x4*x5
dpdx5(229) = x1**3*x4*x6**2+2*x2**3*x4*x5*x6+2*x1*x2**3*x3*x5+x1*&
&x3**2*x4**3
dpdx5(230) = x1*x4**3*x6**2+2*x2*x4**3*x5*x6+2*x1**3*x2*x3*x5+x1*&
&*3*x3**2*x4
dpdx5(231) = x2**3*x4*x6**2+2*x1**3*x4*x5*x6+2*x1*x3*x4**3*x5+x1*&
&x2**3*x3**2
dpdx5(232) = x2*x4**3*x6**2+2*x1*x4**3*x5*x6+2*x1**3*x3*x4*x5+x1*&
&*3*x2*x3**2
dpdx5(233) = 2*x1*x2*x4**3*x5+2*x1**3*x2*x4*x5
dpdx5(234) = x1**2*x2**3*x6+x1**3*x2**2*x6+x2**2*x3*x4**3+x2**3*x&
&3*x4**2
dpdx5(235) = x1*x2**3*x4**2+x1**2*x2**3*x4
dpdx5(236) = x1*x2**2*x4**3+x1**3*x2**2*x4
dpdx5(237) = x2**3*x4**2*x6+x1**3*x4**2*x6+x1**2*x3*x4**3+x1**2*x&
&2**3*x3
dpdx5(238) = x2**2*x4**3*x6+x1**2*x4**3*x6+x1**3*x3*x4**2+x1**3*x&
&2**2*x3
dpdx5(239) = x1**2*x2*x4**3+x1**3*x2*x4**2
dpdx5(240) = x3*x4**4*x6+x2**4*x3*x6+x1**4*x3*x6
dpdx5(241) = x1*x2**4*x6+x1**4*x2*x6+x2*x3*x4**4+x2**4*x3*x4
dpdx5(242) = x1*x2**4*x4
dpdx5(243) = x2**4*x4*x6+x1**4*x4*x6+x1*x3*x4**4+x1*x2**4*x3
dpdx5(244) = x2*x4**4*x6+x1*x4**4*x6+x1**4*x3*x4+x1**4*x2*x3
dpdx5(245) = x1*x2*x4**4+x1**4*x2*x4
dpdx5(246) = 2*x3**2*x5*x6**3+3*x3**2*x5**2*x6**2+2*x3**3*x5*x6**&
&2
dpdx5(247) = 3*x3*x5**2*x6**3+x3**3*x6**3+3*x3**3*x5**2*x6
dpdx5(248) = 2*x3*x5*x6**4+x3**2*x6**4+4*x3*x5**3*x6**2+x3**4*x6*&
&*2+4*x3**2*x5**3*x6+2*x3**4*x5*x6
dpdx5(249) = x3*x6**5+5*x3*x5**4*x6+x3**5*x6
dpdx5(250) = x1*x6**5+5*x2*x5**4*x6+5*x2*x3*x5**4+x3**5*x4
dpdx5(251) = 2*x1*x5*x6**4+4*x2*x5**3*x6**2+4*x2*x3**2*x5**3+2*x3&
&**4*x4*x5
dpdx5(252) = 3*x2*x5**2*x6**3+3*x1*x5**2*x6**3+3*x3**3*x4*x5**2+3&
&*x2*x3**3*x5**2
dpdx5(253) = 3*x4*x5**2*x6**3+3*x1*x3**3*x5**2
dpdx5(254) = 2*x2*x5*x6**4+4*x1*x5**3*x6**2+4*x3**2*x4*x5**3+2*x2&
&*x3**4*x5
dpdx5(255) = 2*x4*x5*x6**4+4*x4*x5**3*x6**2+4*x1*x3**2*x5**3+2*x1&
&*x3**4*x5
dpdx5(256) = x2*x6**5+5*x1*x5**4*x6+5*x3*x4*x5**4+x2*x3**5
dpdx5(257) = x4*x6**5+5*x4*x5**4*x6+5*x1*x3*x5**4+x1*x3**5
dpdx5(258) = 5*x2*x4*x5**4+5*x1*x2*x5**4
dpdx5(259) = 5*x1*x4*x5**4
dpdx5(260) = x1**2*x6**4+4*x2**2*x5**3*x6+4*x2**2*x3*x5**3+x3**4*&
&x4**2
dpdx5(261) = 2*x1**2*x5*x6**3+3*x2**2*x5**2*x6**2+3*x2**2*x3**2*x&
&5**2+2*x3**3*x4**2*x5
dpdx5(262) = 2*x2**2*x5*x6**3+3*x1**2*x5**2*x6**2+3*x3**2*x4**2*x&
&5**2+2*x2**2*x3**3*x5
dpdx5(263) = 2*x4**2*x5*x6**3+3*x4**2*x5**2*x6**2+3*x1**2*x3**2*x&
&5**2+2*x1**2*x3**3*x5
dpdx5(264) = x2**2*x6**4+4*x1**2*x5**3*x6+4*x3*x4**2*x5**3+x2**2*&
&x3**4
dpdx5(265) = x4**2*x6**4+4*x4**2*x5**3*x6+4*x1**2*x3*x5**3+x1**2*&
&x3**4
dpdx5(266) = 4*x2**2*x4*x5**3+4*x1*x2**2*x5**3
dpdx5(267) = 4*x2*x4**2*x5**3+4*x1**2*x2*x5**3
dpdx5(268) = 4*x1*x4**2*x5**3+4*x1**2*x4*x5**3
dpdx5(269) = 3*x2**2*x4**2*x5**2+3*x1**2*x2**2*x5**2
dpdx5(270) = 3*x1**2*x4**2*x5**2
dpdx5(271) = x1**3*x6**3+3*x2**3*x5**2*x6+3*x2**3*x3*x5**2+x3**3*&
&x4**3
dpdx5(272) = 2*x2**3*x5*x6**2+2*x1**3*x5*x6**2+2*x3**2*x4**3*x5+2&
&*x2**3*x3**2*x5
dpdx5(273) = 2*x4**3*x5*x6**2+2*x1**3*x3**2*x5
dpdx5(274) = x2**3*x6**3+3*x1**3*x5**2*x6+3*x3*x4**3*x5**2+x2**3*&
&x3**3
dpdx5(275) = x4**3*x6**3+3*x4**3*x5**2*x6+3*x1**3*x3*x5**2+x1**3*&
&x3**3
dpdx5(276) = 3*x2**3*x4*x5**2+3*x1*x2**3*x5**2
dpdx5(277) = 3*x2*x4**3*x5**2+3*x1**3*x2*x5**2
dpdx5(278) = 3*x1*x4**3*x5**2+3*x1**3*x4*x5**2
dpdx5(279) = 2*x2**3*x4**2*x5+2*x1**2*x2**3*x5
dpdx5(280) = 2*x2**2*x4**3*x5+2*x1**3*x2**2*x5
dpdx5(281) = 0
dpdx5(282) = 2*x1**2*x4**3*x5+2*x1**3*x4**2*x5
dpdx5(283) = x2**3*x4**3+x1**3*x2**3
dpdx5(284) = 0
dpdx5(285) = x1**3*x4**3
dpdx5(286) = x1**4*x6**2+2*x2**4*x5*x6+2*x2**4*x3*x5+x3**2*x4**4
dpdx5(287) = x2**4*x6**2+2*x1**4*x5*x6+2*x3*x4**4*x5+x2**4*x3**2
dpdx5(288) = x4**4*x6**2+2*x4**4*x5*x6+2*x1**4*x3*x5+x1**4*x3**2
dpdx5(289) = 2*x2**4*x4*x5+2*x1*x2**4*x5
dpdx5(290) = 2*x2*x4**4*x5+2*x1**4*x2*x5
dpdx5(291) = 2*x1*x4**4*x5+2*x1**4*x4*x5
dpdx5(292) = x2**4*x4**2+x1**2*x2**4
dpdx5(293) = x2**2*x4**4+x1**4*x2**2
dpdx5(294) = 0
dpdx5(295) = x1**2*x4**4+x1**4*x4**2
dpdx5(296) = x2**5*x6+x1**5*x6+x3*x4**5+x2**5*x3
dpdx5(297) = x4**5*x6+x1**5*x3
dpdx5(298) = x2**5*x4+x1*x2**5
dpdx5(299) = x2*x4**5+x1**5*x2
dpdx5(300) = 0
dpdx5(301) = x1*x4**5+x1**5*x4
dpdx5(302) = 3*x5**2*x6**4+4*x5**3*x6**3+4*x3**3*x5**3+3*x3**4*x5&
&**2
dpdx5(303) = 2*x5*x6**5+5*x5**4*x6**2+5*x3**2*x5**4+2*x3**5*x5
dpdx5(304) = x6**6+6*x5**5*x6+6*x3*x5**5+x3**6
dpdx5(305) = 6*x4*x5**5+6*x1*x5**5
dpdx5(306) = 5*x4**2*x5**4+5*x1**2*x5**4
dpdx5(307) = 4*x4**3*x5**3+4*x1**3*x5**3
dpdx5(308) = 3*x4**4*x5**2+3*x1**4*x5**2
dpdx5(309) = 0
dpdx5(310) = 2*x4**5*x5+2*x1**5*x5
dpdx5(311) = 0
dpdx5(312) = x4**6+x1**6
dpdx5(313) = 0
dpdx5(314) = 2*x1*x2*x3*x4*x5*x6**2+x1*x2*x3**2*x4*x6**2+2*x1*x2*&
&x3**2*x4*x5*x6
dpdx5(315) = x1*x2*x3*x4*x6**3+3*x1*x2*x3*x4*x5**2*x6+x1*x2*x3**3&
&*x4*x6
dpdx5(316) = x1**2*x2*x3*x4*x6**2+2*x1*x2**2*x3*x4*x5*x6+x1*x2*x3&
&**2*x4**2*x6
dpdx5(317) = x1*x2*x3*x4**2*x6**2+x1*x2**2*x3*x4*x6**2+2*x1*x2*x3&
&*x4**2*x5*x6+2*x1**2*x2*x3*x4*x5*x6+x1*x2**2*x3**2*x4*x6+x1**2*x2&
&*x3**2*x4*x6
dpdx5(318) = x1*x2**2*x3*x4**2*x6+x1**2*x2*x3*x4**2*x6+x1**2*x2**&
&2*x3*x4*x6
dpdx5(319) = x1*x2*x3*x4**3*x6+x1*x2**3*x3*x4*x6+x1**3*x2*x3*x4*x&
&6
dpdx5(320) = 2*x1*x2*x3*x5*x6**3+x1*x3**2*x4*x6**3+3*x1*x2*x3*x5*&
&*2*x6**2+x1*x3**3*x4*x6**2+3*x2*x3**2*x4*x5**2*x6+2*x2*x3**3*x4*x&
&5*x6
dpdx5(321) = x1*x3*x4*x6**4+x1*x2*x3*x6**4+4*x2*x3*x4*x5**3*x6+4*&
&x1*x2*x3*x5**3*x6+x2*x3**4*x4*x6+x1*x3**4*x4*x6
dpdx5(322) = 2*x2*x3**2*x4*x5*x6**2+2*x1*x3**2*x4*x5*x6**2+2*x1*x&
&2*x3**2*x5*x6**2
dpdx5(323) = 2*x1*x3*x4*x5*x6**3+x1*x2*x3**2*x6**3+3*x2*x3*x4*x5*&
&*2*x6**2+x2*x3**3*x4*x6**2+3*x1*x2*x3**2*x5**2*x6+2*x1*x3**3*x4*x&
&5*x6
dpdx5(324) = 2*x2*x3*x4*x5*x6**3+x2*x3**2*x4*x6**3+3*x1*x3*x4*x5*&
&*2*x6**2+x1*x2*x3**3*x6**2+3*x1*x3**2*x4*x5**2*x6+2*x1*x2*x3**3*x&
&5*x6
dpdx5(325) = 2*x1*x2*x4*x5*x6**3+3*x1*x2*x4*x5**2*x6**2+3*x1*x2*x&
&3**2*x4*x5**2+2*x1*x2*x3**3*x4*x5
dpdx5(326) = x2*x3*x4*x6**4+4*x1*x3*x4*x5**3*x6+x1*x2*x3**4*x6
dpdx5(327) = x1*x2*x4*x6**4+4*x1*x2*x4*x5**3*x6+4*x1*x2*x3*x4*x5*&
&*3+x1*x2*x3**4*x4
dpdx5(328) = x1**2*x3*x4*x6**3+x1**2*x2*x3*x6**3+3*x2**2*x3*x4*x5&
&**2*x6+3*x1*x2**2*x3*x5**2*x6+x2*x3**3*x4**2*x6+x1*x3**3*x4**2*x6
dpdx5(329) = 2*x1*x2**2*x3*x5*x6**2+2*x1**2*x2*x3*x5*x6**2+x1*x3*&
&*2*x4**2*x6**2+x1**2*x3**2*x4*x6**2+2*x2*x3**2*x4**2*x5*x6+2*x2**&
&2*x3**2*x4*x5*x6
dpdx5(330) = x1*x3*x4**2*x6**3+x1*x2**2*x3*x6**3+3*x2*x3*x4**2*x5&
&**2*x6+3*x1**2*x2*x3*x5**2*x6+x2**2*x3**3*x4*x6+x1**2*x3**3*x4*x6
dpdx5(331) = x1**2*x2*x4*x6**3+3*x1*x2**2*x4*x5**2*x6+3*x1*x2**2*&
&x3*x4*x5**2+x1*x2*x3**3*x4**2
dpdx5(332) = 2*x2**2*x3*x4*x5*x6**2+2*x1**2*x3*x4*x5*x6**2+x2*x3*&
&*2*x4**2*x6**2+x1**2*x2*x3**2*x6**2+2*x1*x3**2*x4**2*x5*x6+2*x1*x&
&2**2*x3**2*x5*x6
dpdx5(333) = 2*x2*x3*x4**2*x5*x6**2+2*x1*x3*x4**2*x5*x6**2+x2**2*&
&x3**2*x4*x6**2+x1*x2**2*x3**2*x6**2+2*x1**2*x3**2*x4*x5*x6+2*x1**&
&2*x2*x3**2*x5*x6
dpdx5(334) = 2*x1*x2**2*x4*x5*x6**2+2*x1**2*x2*x4*x5*x6**2+2*x1*x&
&2*x3**2*x4**2*x5+2*x1*x2**2*x3**2*x4*x5
dpdx5(335) = 2*x1*x2*x4**2*x5*x6**2+2*x1**2*x2*x3**2*x4*x5
dpdx5(336) = x2*x3*x4**2*x6**3+x2**2*x3*x4*x6**3+3*x1*x3*x4**2*x5&
&**2*x6+3*x1**2*x3*x4*x5**2*x6+x1*x2**2*x3**3*x6+x1**2*x2*x3**3*x6
dpdx5(337) = x1*x2**2*x4*x6**3+3*x1**2*x2*x4*x5**2*x6+3*x1*x2*x3*&
&x4**2*x5**2+x1*x2**2*x3**3*x4
dpdx5(338) = x1*x2*x4**2*x6**3+3*x1*x2*x4**2*x5**2*x6+3*x1**2*x2*&
&x3*x4*x5**2+x1**2*x2*x3**3*x4
dpdx5(339) = x1**2*x2**2*x4*x6**2+2*x1**2*x2**2*x4*x5*x6+2*x1*x2*&
&*2*x3*x4**2*x5+x1*x2**2*x3**2*x4**2
dpdx5(340) = x1**2*x3*x4**2*x6**2+x1**2*x2**2*x3*x6**2+2*x2**2*x3&
&*x4**2*x5*x6+2*x1**2*x2**2*x3*x5*x6+x2**2*x3**2*x4**2*x6+x1**2*x3&
&**2*x4**2*x6
dpdx5(341) = x1**2*x2*x4**2*x6**2+2*x1*x2**2*x4**2*x5*x6+2*x1**2*&
&x2**2*x3*x4*x5+x1**2*x2*x3**2*x4**2
dpdx5(342) = x1**2*x2**2*x4**2*x6+x1**2*x2**2*x3*x4**2
dpdx5(343) = x2**2*x3*x4**2*x6**2+2*x1**2*x3*x4**2*x5*x6+x1**2*x2&
&**2*x3**2*x6
dpdx5(344) = x1*x2**2*x4**2*x6**2+2*x1**2*x2*x4**2*x5*x6+2*x1**2*&
&x2*x3*x4**2*x5+x1**2*x2**2*x3**2*x4
dpdx5(345) = x1**3*x3*x4*x6**2+x1**3*x2*x3*x6**2+2*x2**3*x3*x4*x5&
&*x6+2*x1*x2**3*x3*x5*x6+x2*x3**2*x4**3*x6+x1*x3**2*x4**3*x6
dpdx5(346) = x1*x3*x4**3*x6**2+x1*x2**3*x3*x6**2+2*x2*x3*x4**3*x5&
&*x6+2*x1**3*x2*x3*x5*x6+x2**3*x3**2*x4*x6+x1**3*x3**2*x4*x6
dpdx5(347) = x1**3*x2*x4*x6**2+2*x1*x2**3*x4*x5*x6+2*x1*x2**3*x3*&
&x4*x5+x1*x2*x3**2*x4**3
dpdx5(348) = x2*x3*x4**3*x6**2+x2**3*x3*x4*x6**2+2*x1*x3*x4**3*x5&
&*x6+2*x1**3*x3*x4*x5*x6+x1*x2**3*x3**2*x6+x1**3*x2*x3**2*x6
dpdx5(349) = x1*x2**3*x4*x6**2+2*x1**3*x2*x4*x5*x6+2*x1*x2*x3*x4*&
&*3*x5+x1*x2**3*x3**2*x4
dpdx5(350) = x1*x2*x4**3*x6**2+2*x1*x2*x4**3*x5*x6+2*x1**3*x2*x3*&
&x4*x5+x1**3*x2*x3**2*x4
dpdx5(351) = x1**2*x2**3*x4*x6+x1**3*x2**2*x4*x6+x1*x2**2*x3*x4**&
&3+x1*x2**3*x3*x4**2
dpdx5(352) = x2**2*x3*x4**3*x6+x1**2*x3*x4**3*x6+x2**3*x3*x4**2*x&
&6+x1**3*x3*x4**2*x6+x1**2*x2**3*x3*x6+x1**3*x2**2*x3*x6
dpdx5(353) = x1*x2**3*x4**2*x6+x1**3*x2*x4**2*x6+x1**2*x2*x3*x4**&
&3+x1**2*x2**3*x3*x4
dpdx5(354) = x1*x2**2*x4**3*x6+x1**2*x2*x4**3*x6+x1**3*x2*x3*x4**&
&2+x1**3*x2**2*x3*x4
dpdx5(355) = x2*x3*x4**4*x6+x1*x3*x4**4*x6+x2**4*x3*x4*x6+x1**4*x&
&3*x4*x6+x1*x2**4*x3*x6+x1**4*x2*x3*x6
dpdx5(356) = x1*x2**4*x4*x6+x1**4*x2*x4*x6+x1*x2*x3*x4**4+x1*x2**&
&4*x3*x4
dpdx5(357) = x1*x2*x4**4*x6+x1**4*x2*x3*x4
dpdx5(358) = x1*x3*x6**5+5*x2*x3*x5**4*x6+x3**5*x4*x6
dpdx5(359) = 2*x1*x3*x5*x6**4+x1*x3**2*x6**4+4*x2*x3*x5**3*x6**2+&
&x3**4*x4*x6**2+4*x2*x3**2*x5**3*x6+2*x3**4*x4*x5*x6
dpdx5(360) = 2*x1*x3**2*x5*x6**3+3*x2*x3**2*x5**2*x6**2+2*x3**3*x&
&4*x5*x6**2
dpdx5(361) = 3*x2*x3*x5**2*x6**3+3*x1*x3*x5**2*x6**3+x3**3*x4*x6*&
&*3+x1*x3**3*x6**3+3*x3**3*x4*x5**2*x6+3*x2*x3**3*x5**2*x6
dpdx5(362) = 2*x3**2*x4*x5*x6**3+2*x2*x3**2*x5*x6**3+3*x3**2*x4*x&
&5**2*x6**2+3*x1*x3**2*x5**2*x6**2+2*x2*x3**3*x5*x6**2+2*x1*x3**3*&
&x5*x6**2
dpdx5(363) = 3*x3*x4*x5**2*x6**3+x2*x3**3*x6**3+3*x1*x3**3*x5**2*&
&x6
dpdx5(364) = 2*x2*x3*x5*x6**4+x3**2*x4*x6**4+4*x1*x3*x5**3*x6**2+&
&x1*x3**4*x6**2+4*x3**2*x4*x5**3*x6+2*x2*x3**4*x5*x6
dpdx5(365) = 2*x3*x4*x5*x6**4+x2*x3**2*x6**4+4*x3*x4*x5**3*x6**2+&
&x2*x3**4*x6**2+4*x1*x3**2*x5**3*x6+2*x1*x3**4*x5*x6
dpdx5(366) = x3*x4*x6**5+x2*x3*x6**5+5*x3*x4*x5**4*x6+5*x1*x3*x5*&
&*4*x6+x2*x3**5*x6+x1*x3**5*x6
dpdx5(367) = 3*x1*x2*x5**2*x6**3+3*x2*x3**3*x4*x5**2
dpdx5(368) = 2*x1*x2*x5*x6**4+4*x1*x2*x5**3*x6**2+4*x2*x3**2*x4*x&
&5**3+2*x2*x3**4*x4*x5
dpdx5(369) = x1*x2*x6**5+5*x1*x2*x5**4*x6+5*x2*x3*x4*x5**4+x2*x3*&
&*5*x4
dpdx5(370) = x1*x4*x6**5+5*x2*x4*x5**4*x6+5*x1*x2*x3*x5**4+x1*x3*&
&*5*x4
dpdx5(371) = 2*x1*x4*x5*x6**4+4*x2*x4*x5**3*x6**2+4*x1*x2*x3**2*x&
&5**3+2*x1*x3**4*x4*x5
dpdx5(372) = 3*x2*x4*x5**2*x6**3+3*x1*x4*x5**2*x6**3+3*x1*x3**3*x&
&4*x5**2+3*x1*x2*x3**3*x5**2
dpdx5(373) = 2*x2*x4*x5*x6**4+4*x1*x4*x5**3*x6**2+4*x1*x3**2*x4*x&
&5**3+2*x1*x2*x3**4*x5
dpdx5(374) = x2*x4*x6**5+5*x1*x4*x5**4*x6+5*x1*x3*x4*x5**4+x1*x2*&
&x3**5
dpdx5(375) = 5*x1*x2*x4*x5**4
dpdx5(376) = x1**2*x3*x6**4+4*x2**2*x3*x5**3*x6+x3**4*x4**2*x6
dpdx5(377) = 2*x1**2*x3*x5*x6**3+x1**2*x3**2*x6**3+3*x2**2*x3*x5*&
&*2*x6**2+x3**3*x4**2*x6**2+3*x2**2*x3**2*x5**2*x6+2*x3**3*x4**2*x&
&5*x6
dpdx5(378) = 2*x3**2*x4**2*x5*x6**2+2*x2**2*x3**2*x5*x6**2+2*x1**&
&2*x3**2*x5*x6**2
dpdx5(379) = 2*x2**2*x3*x5*x6**3+x3**2*x4**2*x6**3+3*x1**2*x3*x5*&
&*2*x6**2+x1**2*x3**3*x6**2+3*x3**2*x4**2*x5**2*x6+2*x2**2*x3**3*x&
&5*x6
dpdx5(380) = 2*x3*x4**2*x5*x6**3+x2**2*x3**2*x6**3+3*x3*x4**2*x5*&
&*2*x6**2+x2**2*x3**3*x6**2+3*x1**2*x3**2*x5**2*x6+2*x1**2*x3**3*x&
&5*x6
dpdx5(381) = x3*x4**2*x6**4+x2**2*x3*x6**4+4*x3*x4**2*x5**3*x6+4*&
&x1**2*x3*x5**3*x6+x2**2*x3**4*x6+x1**2*x3**4*x6
dpdx5(382) = x1**2*x2*x6**4+4*x1*x2**2*x5**3*x6+4*x2**2*x3*x4*x5*&
&*3+x2*x3**4*x4**2
dpdx5(383) = 2*x1**2*x2*x5*x6**3+3*x1*x2**2*x5**2*x6**2+3*x2**2*x&
&3**2*x4*x5**2+2*x2*x3**3*x4**2*x5
dpdx5(384) = 2*x1*x2**2*x5*x6**3+3*x1**2*x2*x5**2*x6**2+3*x2*x3**&
&2*x4**2*x5**2+2*x2**2*x3**3*x4*x5
dpdx5(385) = x1*x2**2*x6**4+4*x1**2*x2*x5**3*x6+4*x2*x3*x4**2*x5*&
&*3+x2**2*x3**4*x4
dpdx5(386) = 4*x1*x2**2*x4*x5**3
dpdx5(387) = x1**2*x4*x6**4+4*x2**2*x4*x5**3*x6+4*x1*x2**2*x3*x5*&
&*3+x1*x3**4*x4**2
dpdx5(388) = x1*x4**2*x6**4+4*x2*x4**2*x5**3*x6+4*x1**2*x2*x3*x5*&
&*3+x1**2*x3**4*x4
dpdx5(389) = 2*x1**2*x4*x5*x6**3+3*x2**2*x4*x5**2*x6**2+3*x1*x2**&
&2*x3**2*x5**2+2*x1*x3**3*x4**2*x5
dpdx5(390) = 2*x1*x4**2*x5*x6**3+3*x2*x4**2*x5**2*x6**2+3*x1**2*x&
&2*x3**2*x5**2+2*x1**2*x3**3*x4*x5
dpdx5(391) = 2*x2**2*x4*x5*x6**3+3*x1**2*x4*x5**2*x6**2+3*x1*x3**&
&2*x4**2*x5**2+2*x1*x2**2*x3**3*x5
dpdx5(392) = 2*x2*x4**2*x5*x6**3+3*x1*x4**2*x5**2*x6**2+3*x1**2*x&
&3**2*x4*x5**2+2*x1**2*x2*x3**3*x5
dpdx5(393) = x2**2*x4*x6**4+4*x1**2*x4*x5**3*x6+4*x1*x3*x4**2*x5*&
&*3+x1*x2**2*x3**4
dpdx5(394) = x2*x4**2*x6**4+4*x1*x4**2*x5**3*x6+4*x1**2*x3*x4*x5*&
&*3+x1**2*x2*x3**4
dpdx5(395) = 4*x1*x2*x4**2*x5**3+4*x1**2*x2*x4*x5**3
dpdx5(396) = 2*x1**2*x2**2*x5*x6**2+2*x2**2*x3**2*x4**2*x5
dpdx5(397) = x1**2*x2**2*x6**3+3*x1**2*x2**2*x5**2*x6+3*x2**2*x3*&
&x4**2*x5**2+x2**2*x3**3*x4**2
dpdx5(398) = 3*x1*x2**2*x4**2*x5**2+3*x1**2*x2**2*x4*x5**2
dpdx5(399) = x1**2*x4**2*x6**3+3*x2**2*x4**2*x5**2*x6+3*x1**2*x2*&
&*2*x3*x5**2+x1**2*x3**3*x4**2
dpdx5(400) = 2*x2**2*x4**2*x5*x6**2+2*x1**2*x4**2*x5*x6**2+2*x1**&
&2*x3**2*x4**2*x5+2*x1**2*x2**2*x3**2*x5
dpdx5(401) = 2*x1**2*x2**2*x4**2*x5
dpdx5(402) = x2**2*x4**2*x6**3+3*x1**2*x4**2*x5**2*x6+3*x1**2*x3*&
&x4**2*x5**2+x1**2*x2**2*x3**3
dpdx5(403) = 3*x1**2*x2*x4**2*x5**2
dpdx5(404) = x1**3*x3*x6**3+3*x2**3*x3*x5**2*x6+x3**3*x4**3*x6
dpdx5(405) = 2*x2**3*x3*x5*x6**2+2*x1**3*x3*x5*x6**2+x3**2*x4**3*&
&x6**2+x1**3*x3**2*x6**2+2*x3**2*x4**3*x5*x6+2*x2**3*x3**2*x5*x6
dpdx5(406) = 2*x3*x4**3*x5*x6**2+x2**3*x3**2*x6**2+2*x1**3*x3**2*&
&x5*x6
dpdx5(407) = x3*x4**3*x6**3+x2**3*x3*x6**3+3*x3*x4**3*x5**2*x6+3*&
&x1**3*x3*x5**2*x6+x2**3*x3**3*x6+x1**3*x3**3*x6
dpdx5(408) = x1**3*x2*x6**3+3*x1*x2**3*x5**2*x6+3*x2**3*x3*x4*x5*&
&*2+x2*x3**3*x4**3
dpdx5(409) = 2*x1*x2**3*x5*x6**2+2*x1**3*x2*x5*x6**2+2*x2*x3**2*x&
&4**3*x5+2*x2**3*x3**2*x4*x5
dpdx5(410) = x1*x2**3*x6**3+3*x1**3*x2*x5**2*x6+3*x2*x3*x4**3*x5*&
&*2+x2**3*x3**3*x4
dpdx5(411) = 3*x1*x2**3*x4*x5**2
dpdx5(412) = x1**3*x4*x6**3+3*x2**3*x4*x5**2*x6+3*x1*x2**3*x3*x5*&
&*2+x1*x3**3*x4**3
dpdx5(413) = x1*x4**3*x6**3+3*x2*x4**3*x5**2*x6+3*x1**3*x2*x3*x5*&
&*2+x1**3*x3**3*x4
dpdx5(414) = 2*x2**3*x4*x5*x6**2+2*x1**3*x4*x5*x6**2+2*x1*x3**2*x&
&4**3*x5+2*x1*x2**3*x3**2*x5
dpdx5(415) = 2*x2*x4**3*x5*x6**2+2*x1*x4**3*x5*x6**2+2*x1**3*x3**&
&2*x4*x5+2*x1**3*x2*x3**2*x5
dpdx5(416) = x2**3*x4*x6**3+3*x1**3*x4*x5**2*x6+3*x1*x3*x4**3*x5*&
&*2+x1*x2**3*x3**3
dpdx5(417) = x2*x4**3*x6**3+3*x1*x4**3*x5**2*x6+3*x1**3*x3*x4*x5*&
&*2+x1**3*x2*x3**3
dpdx5(418) = 3*x1*x2*x4**3*x5**2+3*x1**3*x2*x4*x5**2
dpdx5(419) = x1**3*x2**2*x6**2+2*x1**2*x2**3*x5*x6+2*x2**3*x3*x4*&
&*2*x5+x2**2*x3**2*x4**3
dpdx5(420) = x1**2*x2**3*x6**2+2*x1**3*x2**2*x5*x6+2*x2**2*x3*x4*&
&*3*x5+x2**3*x3**2*x4**2
dpdx5(421) = 2*x1*x2**3*x4**2*x5+2*x1**2*x2**3*x4*x5
dpdx5(422) = 2*x1*x2**2*x4**3*x5+2*x1**3*x2**2*x4*x5
dpdx5(423) = x1**2*x2**3*x4**2
dpdx5(424) = x1**3*x4**2*x6**2+2*x2**3*x4**2*x5*x6+2*x1**2*x2**3*&
&x3*x5+x1**2*x3**2*x4**3
dpdx5(425) = x1**2*x4**3*x6**2+2*x2**2*x4**3*x5*x6+2*x1**3*x2**2*&
&x3*x5+x1**3*x3**2*x4**2
dpdx5(426) = x1**2*x2**2*x4**3+x1**3*x2**2*x4**2
dpdx5(427) = x2**3*x4**2*x6**2+2*x1**3*x4**2*x5*x6+2*x1**2*x3*x4*&
&*3*x5+x1**2*x2**3*x3**2
dpdx5(428) = x2**2*x4**3*x6**2+2*x1**2*x4**3*x5*x6+2*x1**3*x3*x4*&
&*2*x5+x1**3*x2**2*x3**2
dpdx5(429) = 2*x1**2*x2*x4**3*x5+2*x1**3*x2*x4**2*x5
dpdx5(430) = x1**3*x2**3*x6+x2**3*x3*x4**3
dpdx5(431) = x1*x2**3*x4**3+x1**3*x2**3*x4
dpdx5(432) = x2**3*x4**3*x6+x1**3*x4**3*x6+x1**3*x3*x4**3+x1**3*x&
&2**3*x3
dpdx5(433) = x1**3*x2*x4**3
dpdx5(434) = x1**4*x3*x6**2+2*x2**4*x3*x5*x6+x3**2*x4**4*x6
dpdx5(435) = x3*x4**4*x6**2+x2**4*x3*x6**2+2*x3*x4**4*x5*x6+2*x1*&
&*4*x3*x5*x6+x2**4*x3**2*x6+x1**4*x3**2*x6
dpdx5(436) = x1**4*x2*x6**2+2*x1*x2**4*x5*x6+2*x2**4*x3*x4*x5+x2*&
&x3**2*x4**4
dpdx5(437) = x1*x2**4*x6**2+2*x1**4*x2*x5*x6+2*x2*x3*x4**4*x5+x2*&
&*4*x3**2*x4
dpdx5(438) = 2*x1*x2**4*x4*x5
dpdx5(439) = x1**4*x4*x6**2+2*x2**4*x4*x5*x6+2*x1*x2**4*x3*x5+x1*&
&x3**2*x4**4
dpdx5(440) = x1*x4**4*x6**2+2*x2*x4**4*x5*x6+2*x1**4*x2*x3*x5+x1*&
&*4*x3**2*x4
dpdx5(441) = x2**4*x4*x6**2+2*x1**4*x4*x5*x6+2*x1*x3*x4**4*x5+x1*&
&x2**4*x3**2
dpdx5(442) = x2*x4**4*x6**2+2*x1*x4**4*x5*x6+2*x1**4*x3*x4*x5+x1*&
&*4*x2*x3**2
dpdx5(443) = 2*x1*x2*x4**4*x5+2*x1**4*x2*x4*x5
dpdx5(444) = x1**2*x2**4*x6+x1**4*x2**2*x6+x2**2*x3*x4**4+x2**4*x&
&3*x4**2
dpdx5(445) = x1*x2**4*x4**2+x1**2*x2**4*x4
dpdx5(446) = x1*x2**2*x4**4+x1**4*x2**2*x4
dpdx5(447) = x2**4*x4**2*x6+x1**4*x4**2*x6+x1**2*x3*x4**4+x1**2*x&
&2**4*x3
dpdx5(448) = x2**2*x4**4*x6+x1**2*x4**4*x6+x1**4*x3*x4**2+x1**4*x&
&2**2*x3
dpdx5(449) = x1**2*x2*x4**4+x1**4*x2*x4**2
dpdx5(450) = x3*x4**5*x6+x2**5*x3*x6+x1**5*x3*x6
dpdx5(451) = x1*x2**5*x6+x1**5*x2*x6+x2*x3*x4**5+x2**5*x3*x4
dpdx5(452) = x1*x2**5*x4
dpdx5(453) = x2**5*x4*x6+x1**5*x4*x6+x1*x3*x4**5+x1*x2**5*x3
dpdx5(454) = x2*x4**5*x6+x1*x4**5*x6+x1**5*x3*x4+x1**5*x2*x3
dpdx5(455) = x1*x2*x4**5+x1**5*x2*x4
dpdx5(456) = 3*x3**2*x5**2*x6**3+2*x3**3*x5*x6**3+3*x3**3*x5**2*x&
&6**2
dpdx5(457) = 2*x3**2*x5*x6**4+4*x3**2*x5**3*x6**2+2*x3**4*x5*x6**&
&2
dpdx5(458) = 3*x3*x5**2*x6**4+x3**3*x6**4+4*x3*x5**3*x6**3+x3**4*&
&x6**3+4*x3**3*x5**3*x6+3*x3**4*x5**2*x6
dpdx5(459) = 2*x3*x5*x6**5+x3**2*x6**5+5*x3*x5**4*x6**2+x3**5*x6*&
&*2+5*x3**2*x5**4*x6+2*x3**5*x5*x6
dpdx5(460) = x3*x6**6+6*x3*x5**5*x6+x3**6*x6
dpdx5(461) = x1*x6**6+6*x2*x5**5*x6+6*x2*x3*x5**5+x3**6*x4
dpdx5(462) = 2*x1*x5*x6**5+5*x2*x5**4*x6**2+5*x2*x3**2*x5**4+2*x3&
&**5*x4*x5
dpdx5(463) = 3*x1*x5**2*x6**4+4*x2*x5**3*x6**3+4*x2*x3**3*x5**3+3&
&*x3**4*x4*x5**2
dpdx5(464) = 3*x2*x5**2*x6**4+4*x1*x5**3*x6**3+4*x3**3*x4*x5**3+3&
&*x2*x3**4*x5**2
dpdx5(465) = 3*x4*x5**2*x6**4+4*x4*x5**3*x6**3+4*x1*x3**3*x5**3+3&
&*x1*x3**4*x5**2
dpdx5(466) = 2*x2*x5*x6**5+5*x1*x5**4*x6**2+5*x3**2*x4*x5**4+2*x2&
&*x3**5*x5
dpdx5(467) = 2*x4*x5*x6**5+5*x4*x5**4*x6**2+5*x1*x3**2*x5**4+2*x1&
&*x3**5*x5
dpdx5(468) = x2*x6**6+6*x1*x5**5*x6+6*x3*x4*x5**5+x2*x3**6
dpdx5(469) = x4*x6**6+6*x4*x5**5*x6+6*x1*x3*x5**5+x1*x3**6
dpdx5(470) = 6*x2*x4*x5**5+6*x1*x2*x5**5
dpdx5(471) = 6*x1*x4*x5**5
dpdx5(472) = x1**2*x6**5+5*x2**2*x5**4*x6+5*x2**2*x3*x5**4+x3**5*&
&x4**2
dpdx5(473) = 2*x1**2*x5*x6**4+4*x2**2*x5**3*x6**2+4*x2**2*x3**2*x&
&5**3+2*x3**4*x4**2*x5
dpdx5(474) = 3*x2**2*x5**2*x6**3+3*x1**2*x5**2*x6**3+3*x3**3*x4**&
&2*x5**2+3*x2**2*x3**3*x5**2
dpdx5(475) = 3*x4**2*x5**2*x6**3+3*x1**2*x3**3*x5**2
dpdx5(476) = 2*x2**2*x5*x6**4+4*x1**2*x5**3*x6**2+4*x3**2*x4**2*x&
&5**3+2*x2**2*x3**4*x5
dpdx5(477) = 2*x4**2*x5*x6**4+4*x4**2*x5**3*x6**2+4*x1**2*x3**2*x&
&5**3+2*x1**2*x3**4*x5
dpdx5(478) = x2**2*x6**5+5*x1**2*x5**4*x6+5*x3*x4**2*x5**4+x2**2*&
&x3**5
dpdx5(479) = x4**2*x6**5+5*x4**2*x5**4*x6+5*x1**2*x3*x5**4+x1**2*&
&x3**5
dpdx5(480) = 5*x2**2*x4*x5**4+5*x1*x2**2*x5**4
dpdx5(481) = 5*x2*x4**2*x5**4+5*x1**2*x2*x5**4
dpdx5(482) = 5*x1*x4**2*x5**4+5*x1**2*x4*x5**4
dpdx5(483) = 4*x2**2*x4**2*x5**3+4*x1**2*x2**2*x5**3
dpdx5(484) = 4*x1**2*x4**2*x5**3
dpdx5(485) = x1**3*x6**4+4*x2**3*x5**3*x6+4*x2**3*x3*x5**3+x3**4*&
&x4**3
dpdx5(486) = 2*x1**3*x5*x6**3+3*x2**3*x5**2*x6**2+3*x2**3*x3**2*x&
&5**2+2*x3**3*x4**3*x5
dpdx5(487) = 2*x2**3*x5*x6**3+3*x1**3*x5**2*x6**2+3*x3**2*x4**3*x&
&5**2+2*x2**3*x3**3*x5
dpdx5(488) = 2*x4**3*x5*x6**3+3*x4**3*x5**2*x6**2+3*x1**3*x3**2*x&
&5**2+2*x1**3*x3**3*x5
dpdx5(489) = x2**3*x6**4+4*x1**3*x5**3*x6+4*x3*x4**3*x5**3+x2**3*&
&x3**4
dpdx5(490) = x4**3*x6**4+4*x4**3*x5**3*x6+4*x1**3*x3*x5**3+x1**3*&
&x3**4
dpdx5(491) = 4*x2**3*x4*x5**3+4*x1*x2**3*x5**3
dpdx5(492) = 4*x2*x4**3*x5**3+4*x1**3*x2*x5**3
dpdx5(493) = 4*x1*x4**3*x5**3+4*x1**3*x4*x5**3
dpdx5(494) = 3*x2**3*x4**2*x5**2+3*x1**2*x2**3*x5**2
dpdx5(495) = 3*x2**2*x4**3*x5**2+3*x1**3*x2**2*x5**2
dpdx5(496) = 3*x1**2*x4**3*x5**2+3*x1**3*x4**2*x5**2
dpdx5(497) = 2*x2**3*x4**3*x5+2*x1**3*x2**3*x5
dpdx5(498) = 0
dpdx5(499) = 2*x1**3*x4**3*x5
dpdx5(500) = x1**4*x6**3+3*x2**4*x5**2*x6+3*x2**4*x3*x5**2+x3**3*&
&x4**4
dpdx5(501) = 2*x2**4*x5*x6**2+2*x1**4*x5*x6**2+2*x3**2*x4**4*x5+2&
&*x2**4*x3**2*x5
dpdx5(502) = 2*x4**4*x5*x6**2+2*x1**4*x3**2*x5
dpdx5(503) = x2**4*x6**3+3*x1**4*x5**2*x6+3*x3*x4**4*x5**2+x2**4*&
&x3**3
dpdx5(504) = x4**4*x6**3+3*x4**4*x5**2*x6+3*x1**4*x3*x5**2+x1**4*&
&x3**3
dpdx5(505) = 3*x2**4*x4*x5**2+3*x1*x2**4*x5**2
dpdx5(506) = 3*x2*x4**4*x5**2+3*x1**4*x2*x5**2
dpdx5(507) = 3*x1*x4**4*x5**2+3*x1**4*x4*x5**2
dpdx5(508) = 2*x2**4*x4**2*x5+2*x1**2*x2**4*x5
dpdx5(509) = 2*x2**2*x4**4*x5+2*x1**4*x2**2*x5
dpdx5(510) = 0
dpdx5(511) = 2*x1**2*x4**4*x5+2*x1**4*x4**2*x5
dpdx5(512) = x2**4*x4**3+x1**3*x2**4
dpdx5(513) = x2**3*x4**4+x1**4*x2**3
dpdx5(514) = 0
dpdx5(515) = x1**3*x4**4+x1**4*x4**3
dpdx5(516) = x1**5*x6**2+2*x2**5*x5*x6+2*x2**5*x3*x5+x3**2*x4**5
dpdx5(517) = x2**5*x6**2+2*x1**5*x5*x6+2*x3*x4**5*x5+x2**5*x3**2
dpdx5(518) = x4**5*x6**2+2*x4**5*x5*x6+2*x1**5*x3*x5+x1**5*x3**2
dpdx5(519) = 2*x2**5*x4*x5+2*x1*x2**5*x5
dpdx5(520) = 2*x2*x4**5*x5+2*x1**5*x2*x5
dpdx5(521) = 2*x1*x4**5*x5+2*x1**5*x4*x5
dpdx5(522) = x2**5*x4**2+x1**2*x2**5
dpdx5(523) = x2**2*x4**5+x1**5*x2**2
dpdx5(524) = 0
dpdx5(525) = x1**2*x4**5+x1**5*x4**2
dpdx5(526) = x2**6*x6+x1**6*x6+x3*x4**6+x2**6*x3
dpdx5(527) = x4**6*x6+x1**6*x3
dpdx5(528) = x2**6*x4+x1*x2**6
dpdx5(529) = x2*x4**6+x1**6*x2
dpdx5(530) = 0
dpdx5(531) = x1*x4**6+x1**6*x4
dpdx5(532) = 4*x5**3*x6**4+4*x3**4*x5**3
dpdx5(533) = 3*x5**2*x6**5+5*x5**4*x6**3+5*x3**3*x5**4+3*x3**5*x5&
&**2
dpdx5(534) = 2*x5*x6**6+6*x5**5*x6**2+6*x3**2*x5**5+2*x3**6*x5
dpdx5(535) = x6**7+7*x5**6*x6+7*x3*x5**6+x3**7
dpdx5(536) = 7*x4*x5**6+7*x1*x5**6
dpdx5(537) = 6*x4**2*x5**5+6*x1**2*x5**5
dpdx5(538) = 5*x4**3*x5**4+5*x1**3*x5**4
dpdx5(539) = 4*x4**4*x5**3+4*x1**4*x5**3
dpdx5(540) = 0
dpdx5(541) = 3*x4**5*x5**2+3*x1**5*x5**2
dpdx5(542) = 0
dpdx5(543) = 2*x4**6*x5+2*x1**6*x5
dpdx5(544) = 0
dpdx5(545) = x4**7+x1**7
dpdx5(546) = 0
dpdx6(1) = 0
dpdx6(2) = x5+x3
dpdx6(3) = x4+x2
dpdx6(4) = 0
dpdx6(5) = x3*x5
dpdx6(6) = x2*x5+x1*x5+x3*x4+x1*x3
dpdx6(7) = x4*x5+x2*x3
dpdx6(8) = x1*x4+x1*x2
dpdx6(9) = 0
dpdx6(10) = x2*x4
dpdx6(11) = 2*x5*x6+2*x3*x6+x5**2+x3**2
dpdx6(12) = 2*x4*x6+2*x2*x6
dpdx6(13) = x4**2+x2**2
dpdx6(14) = 0
dpdx6(15) = x3*x4*x5+x2*x3*x5+x1*x3*x5
dpdx6(16) = x1*x2*x5+x1*x3*x4
dpdx6(17) = x2*x4*x5+x1*x4*x5+x2*x3*x4+x1*x2*x3
dpdx6(18) = x1*x2*x4
dpdx6(19) = 2*x3*x5*x6+x3*x5**2+x3**2*x5
dpdx6(20) = 2*x1*x5*x6+2*x1*x3*x6+x2*x5**2+x3**2*x4
dpdx6(21) = 2*x2*x5*x6+2*x3*x4*x6+x1*x5**2+x1*x3**2
dpdx6(22) = 2*x4*x5*x6+2*x2*x3*x6+x4*x5**2+x2*x3**2
dpdx6(23) = 2*x1*x4*x6+2*x1*x2*x6
dpdx6(24) = 2*x2*x4*x6
dpdx6(25) = x2**2*x5+x1**2*x5+x3*x4**2+x1**2*x3
dpdx6(26) = x4**2*x5+x2**2*x3
dpdx6(27) = x1**2*x4+x1**2*x2
dpdx6(28) = x1*x4**2+x1*x2**2
dpdx6(29) = 0
dpdx6(30) = x2*x4**2+x2**2*x4
dpdx6(31) = 2*x5**2*x6+2*x3**2*x6
dpdx6(32) = 3*x5*x6**2+3*x3*x6**2+x5**3+x3**3
dpdx6(33) = 3*x4*x6**2+3*x2*x6**2
dpdx6(34) = 2*x4**2*x6+2*x2**2*x6
dpdx6(35) = 0
dpdx6(36) = x4**3+x2**3
dpdx6(37) = 0
dpdx6(38) = x2*x3*x4*x5+x1*x3*x4*x5+x1*x2*x3*x5
dpdx6(39) = x1*x2*x4*x5+x1*x2*x3*x4
dpdx6(40) = 2*x1*x3*x5*x6+x2*x3*x5**2+x3**2*x4*x5
dpdx6(41) = 2*x3*x4*x5*x6+2*x2*x3*x5*x6+x3*x4*x5**2+x1*x3*x5**2+x&
&2*x3**2*x5+x1*x3**2*x5
dpdx6(42) = 2*x1*x2*x5*x6+2*x1*x3*x4*x6+x1*x2*x5**2+x1*x3**2*x4
dpdx6(43) = 2*x1*x4*x5*x6+2*x1*x2*x3*x6+x2*x4*x5**2+x2*x3**2*x4
dpdx6(44) = 2*x2*x4*x5*x6+2*x2*x3*x4*x6+x1*x4*x5**2+x1*x2*x3**2
dpdx6(45) = 2*x1*x2*x4*x6
dpdx6(46) = x3*x4**2*x5+x2**2*x3*x5+x1**2*x3*x5
dpdx6(47) = x1*x2**2*x5+x1**2*x2*x5+x1*x3*x4**2+x1**2*x3*x4
dpdx6(48) = x1**2*x2*x4
dpdx6(49) = x2**2*x4*x5+x1**2*x4*x5+x2*x3*x4**2+x1**2*x2*x3
dpdx6(50) = x2*x4**2*x5+x1*x4**2*x5+x2**2*x3*x4+x1*x2**2*x3
dpdx6(51) = x1*x2*x4**2+x1*x2**2*x4
dpdx6(52) = 2*x3*x5**2*x6+2*x3**2*x5*x6+x3**2*x5**2
dpdx6(53) = 3*x3*x5*x6**2+x3*x5**3+x3**3*x5
dpdx6(54) = 3*x1*x5*x6**2+3*x1*x3*x6**2+x2*x5**3+x3**3*x4
dpdx6(55) = 2*x2*x5**2*x6+2*x1*x5**2*x6+2*x3**2*x4*x6+2*x1*x3**2*&
&x6
dpdx6(56) = 2*x4*x5**2*x6+2*x2*x3**2*x6
dpdx6(57) = 3*x2*x5*x6**2+3*x3*x4*x6**2+x1*x5**3+x1*x3**3
dpdx6(58) = 3*x4*x5*x6**2+3*x2*x3*x6**2+x4*x5**3+x2*x3**3
dpdx6(59) = 3*x1*x4*x6**2+3*x1*x2*x6**2
dpdx6(60) = 3*x2*x4*x6**2
dpdx6(61) = 2*x1**2*x5*x6+2*x1**2*x3*x6+x2**2*x5**2+x3**2*x4**2
dpdx6(62) = 2*x2**2*x5*x6+2*x3*x4**2*x6+x1**2*x5**2+x1**2*x3**2
dpdx6(63) = 2*x4**2*x5*x6+2*x2**2*x3*x6+x4**2*x5**2+x2**2*x3**2
dpdx6(64) = 2*x1**2*x4*x6+2*x1**2*x2*x6
dpdx6(65) = 2*x1*x4**2*x6+2*x1*x2**2*x6
dpdx6(66) = 2*x2*x4**2*x6+2*x2**2*x4*x6
dpdx6(67) = x1**2*x4**2+x1**2*x2**2
dpdx6(68) = 0
dpdx6(69) = x2**2*x4**2
dpdx6(70) = x2**3*x5+x1**3*x5+x3*x4**3+x1**3*x3
dpdx6(71) = x4**3*x5+x2**3*x3
dpdx6(72) = x1**3*x4+x1**3*x2
dpdx6(73) = x1*x4**3+x1*x2**3
dpdx6(74) = 0
dpdx6(75) = x2*x4**3+x2**3*x4
dpdx6(76) = 3*x5**2*x6**2+3*x3**2*x6**2+2*x5**3*x6+2*x3**3*x6
dpdx6(77) = 4*x5*x6**3+4*x3*x6**3+x5**4+x3**4
dpdx6(78) = 4*x4*x6**3+4*x2*x6**3
dpdx6(79) = 3*x4**2*x6**2+3*x2**2*x6**2
dpdx6(80) = 2*x4**3*x6+2*x2**3*x6
dpdx6(81) = 0
dpdx6(82) = x4**4+x2**4
dpdx6(83) = 0
dpdx6(84) = x1*x2*x3*x4*x5
dpdx6(85) = 2*x1*x3*x4*x5*x6+2*x1*x2*x3*x5*x6+x2*x3*x4*x5**2+x1*x&
&2*x3*x5**2+x2*x3**2*x4*x5+x1*x3**2*x4*x5
dpdx6(86) = 2*x2*x3*x4*x5*x6+x1*x3*x4*x5**2+x1*x2*x3**2*x5
dpdx6(87) = 2*x1*x2*x4*x5*x6+2*x1*x2*x3*x4*x6+x1*x2*x4*x5**2+x1*x&
&2*x3**2*x4
dpdx6(88) = x2*x3*x4**2*x5+x1*x3*x4**2*x5+x2**2*x3*x4*x5+x1**2*x3&
&*x4*x5+x1*x2**2*x3*x5+x1**2*x2*x3*x5
dpdx6(89) = x1*x2**2*x4*x5+x1**2*x2*x4*x5+x1*x2*x3*x4**2+x1**2*x2&
&*x3*x4
dpdx6(90) = x1*x2*x4**2*x5+x1*x2**2*x3*x4
dpdx6(91) = 3*x1*x3*x5*x6**2+x2*x3*x5**3+x3**3*x4*x5
dpdx6(92) = 2*x2*x3*x5**2*x6+2*x1*x3*x5**2*x6+2*x3**2*x4*x5*x6+2*&
&x1*x3**2*x5*x6+x3**2*x4*x5**2+x2*x3**2*x5**2
dpdx6(93) = 2*x3*x4*x5**2*x6+2*x2*x3**2*x5*x6+x1*x3**2*x5**2
dpdx6(94) = 3*x3*x4*x5*x6**2+3*x2*x3*x5*x6**2+x3*x4*x5**3+x1*x3*x&
&5**3+x2*x3**3*x5+x1*x3**3*x5
dpdx6(95) = 2*x1*x2*x5**2*x6+2*x1*x3**2*x4*x6
dpdx6(96) = 3*x1*x2*x5*x6**2+3*x1*x3*x4*x6**2+x1*x2*x5**3+x1*x3**&
&3*x4
dpdx6(97) = 3*x1*x4*x5*x6**2+3*x1*x2*x3*x6**2+x2*x4*x5**3+x2*x3**&
&3*x4
dpdx6(98) = 2*x2*x4*x5**2*x6+2*x1*x4*x5**2*x6+2*x2*x3**2*x4*x6+2*&
&x1*x2*x3**2*x6
dpdx6(99) = 3*x2*x4*x5*x6**2+3*x2*x3*x4*x6**2+x1*x4*x5**3+x1*x2*x&
&3**3
dpdx6(100) = 3*x1*x2*x4*x6**2
dpdx6(101) = 2*x1**2*x3*x5*x6+x2**2*x3*x5**2+x3**2*x4**2*x5
dpdx6(102) = 2*x3*x4**2*x5*x6+2*x2**2*x3*x5*x6+x3*x4**2*x5**2+x1*&
&*2*x3*x5**2+x2**2*x3**2*x5+x1**2*x3**2*x5
dpdx6(103) = 2*x1**2*x2*x5*x6+2*x1**2*x3*x4*x6+x1*x2**2*x5**2+x1*&
&x3**2*x4**2
dpdx6(104) = 2*x1*x2**2*x5*x6+2*x1*x3*x4**2*x6+x1**2*x2*x5**2+x1*&
&*2*x3**2*x4
dpdx6(105) = 2*x1**2*x2*x4*x6
dpdx6(106) = 2*x1**2*x4*x5*x6+2*x1**2*x2*x3*x6+x2**2*x4*x5**2+x2*&
&x3**2*x4**2
dpdx6(107) = 2*x1*x4**2*x5*x6+2*x1*x2**2*x3*x6+x2*x4**2*x5**2+x2*&
&*2*x3**2*x4
dpdx6(108) = 2*x2**2*x4*x5*x6+2*x2*x3*x4**2*x6+x1**2*x4*x5**2+x1*&
&*2*x2*x3**2
dpdx6(109) = 2*x2*x4**2*x5*x6+2*x2**2*x3*x4*x6+x1*x4**2*x5**2+x1*&
&x2**2*x3**2
dpdx6(110) = 2*x1*x2*x4**2*x6+2*x1*x2**2*x4*x6
dpdx6(111) = x1**2*x2**2*x5+x1**2*x3*x4**2
dpdx6(112) = x1**2*x2*x4**2+x1**2*x2**2*x4
dpdx6(113) = x2**2*x4**2*x5+x1**2*x4**2*x5+x2**2*x3*x4**2+x1**2*x&
&2**2*x3
dpdx6(114) = x1*x2**2*x4**2
dpdx6(115) = x3*x4**3*x5+x2**3*x3*x5+x1**3*x3*x5
dpdx6(116) = x1*x2**3*x5+x1**3*x2*x5+x1*x3*x4**3+x1**3*x3*x4
dpdx6(117) = x1**3*x2*x4
dpdx6(118) = x2**3*x4*x5+x1**3*x4*x5+x2*x3*x4**3+x1**3*x2*x3
dpdx6(119) = x2*x4**3*x5+x1*x4**3*x5+x2**3*x3*x4+x1*x2**3*x3
dpdx6(120) = x1*x2*x4**3+x1*x2**3*x4
dpdx6(121) = 2*x3**2*x5**2*x6
dpdx6(122) = 3*x3*x5**2*x6**2+3*x3**2*x5*x6**2+2*x3*x5**3*x6+2*x3&
&**3*x5*x6+x3**2*x5**3+x3**3*x5**2
dpdx6(123) = 4*x3*x5*x6**3+x3*x5**4+x3**4*x5
dpdx6(124) = 4*x1*x5*x6**3+4*x1*x3*x6**3+x2*x5**4+x3**4*x4
dpdx6(125) = 3*x1*x5**2*x6**2+3*x1*x3**2*x6**2+2*x2*x5**3*x6+2*x3&
&**3*x4*x6
dpdx6(126) = 3*x2*x5**2*x6**2+3*x3**2*x4*x6**2+2*x1*x5**3*x6+2*x1&
&*x3**3*x6
dpdx6(127) = 3*x4*x5**2*x6**2+3*x2*x3**2*x6**2+2*x4*x5**3*x6+2*x2&
&*x3**3*x6
dpdx6(128) = 4*x2*x5*x6**3+4*x3*x4*x6**3+x1*x5**4+x1*x3**4
dpdx6(129) = 4*x4*x5*x6**3+4*x2*x3*x6**3+x4*x5**4+x2*x3**4
dpdx6(130) = 4*x1*x4*x6**3+4*x1*x2*x6**3
dpdx6(131) = 4*x2*x4*x6**3
dpdx6(132) = 3*x1**2*x5*x6**2+3*x1**2*x3*x6**2+x2**2*x5**3+x3**3*&
&x4**2
dpdx6(133) = 2*x2**2*x5**2*x6+2*x1**2*x5**2*x6+2*x3**2*x4**2*x6+2&
&*x1**2*x3**2*x6
dpdx6(134) = 2*x4**2*x5**2*x6+2*x2**2*x3**2*x6
dpdx6(135) = 3*x2**2*x5*x6**2+3*x3*x4**2*x6**2+x1**2*x5**3+x1**2*&
&x3**3
dpdx6(136) = 3*x4**2*x5*x6**2+3*x2**2*x3*x6**2+x4**2*x5**3+x2**2*&
&x3**3
dpdx6(137) = 3*x1**2*x4*x6**2+3*x1**2*x2*x6**2
dpdx6(138) = 3*x1*x4**2*x6**2+3*x1*x2**2*x6**2
dpdx6(139) = 3*x2*x4**2*x6**2+3*x2**2*x4*x6**2
dpdx6(140) = 2*x1**2*x4**2*x6+2*x1**2*x2**2*x6
dpdx6(141) = 0
dpdx6(142) = 2*x2**2*x4**2*x6
dpdx6(143) = 2*x1**3*x5*x6+2*x1**3*x3*x6+x2**3*x5**2+x3**2*x4**3
dpdx6(144) = 2*x2**3*x5*x6+2*x3*x4**3*x6+x1**3*x5**2+x1**3*x3**2
dpdx6(145) = 2*x4**3*x5*x6+2*x2**3*x3*x6+x4**3*x5**2+x2**3*x3**2
dpdx6(146) = 2*x1**3*x4*x6+2*x1**3*x2*x6
dpdx6(147) = 2*x1*x4**3*x6+2*x1*x2**3*x6
dpdx6(148) = 2*x2*x4**3*x6+2*x2**3*x4*x6
dpdx6(149) = x1**3*x4**2+x1**3*x2**2
dpdx6(150) = x1**2*x4**3+x1**2*x2**3
dpdx6(151) = 0
dpdx6(152) = x2**2*x4**3+x2**3*x4**2
dpdx6(153) = x2**4*x5+x1**4*x5+x3*x4**4+x1**4*x3
dpdx6(154) = x4**4*x5+x2**4*x3
dpdx6(155) = x1**4*x4+x1**4*x2
dpdx6(156) = x1*x4**4+x1*x2**4
dpdx6(157) = 0
dpdx6(158) = x2*x4**4+x2**4*x4
dpdx6(159) = 3*x5**3*x6**2+3*x3**3*x6**2
dpdx6(160) = 4*x5**2*x6**3+4*x3**2*x6**3+2*x5**4*x6+2*x3**4*x6
dpdx6(161) = 5*x5*x6**4+5*x3*x6**4+x5**5+x3**5
dpdx6(162) = 5*x4*x6**4+5*x2*x6**4
dpdx6(163) = 4*x4**2*x6**3+4*x2**2*x6**3
dpdx6(164) = 3*x4**3*x6**2+3*x2**3*x6**2
dpdx6(165) = 0
dpdx6(166) = 2*x4**4*x6+2*x2**4*x6
dpdx6(167) = 0
dpdx6(168) = x4**5+x2**5
dpdx6(169) = 0
dpdx6(170) = 2*x1*x2*x3*x4*x5*x6+x1*x2*x3*x4*x5**2+x1*x2*x3**2*x4&
&*x5
dpdx6(171) = x1*x2*x3*x4**2*x5+x1*x2**2*x3*x4*x5+x1**2*x2*x3*x4*x&
&5
dpdx6(172) = 2*x1*x2*x3*x5**2*x6+2*x1*x3**2*x4*x5*x6+x2*x3**2*x4*&
&x5**2
dpdx6(173) = 3*x1*x3*x4*x5*x6**2+3*x1*x2*x3*x5*x6**2+x2*x3*x4*x5*&
&*3+x1*x2*x3*x5**3+x2*x3**3*x4*x5+x1*x3**3*x4*x5
dpdx6(174) = 2*x2*x3*x4*x5**2*x6+2*x1*x3*x4*x5**2*x6+2*x2*x3**2*x&
&4*x5*x6+2*x1*x2*x3**2*x5*x6+x1*x3**2*x4*x5**2+x1*x2*x3**2*x5**2
dpdx6(175) = 2*x1*x2*x4*x5**2*x6+2*x1*x2*x3**2*x4*x6
dpdx6(176) = 3*x2*x3*x4*x5*x6**2+x1*x3*x4*x5**3+x1*x2*x3**3*x5
dpdx6(177) = 3*x1*x2*x4*x5*x6**2+3*x1*x2*x3*x4*x6**2+x1*x2*x4*x5*&
&*3+x1*x2*x3**3*x4
dpdx6(178) = 2*x1**2*x3*x4*x5*x6+2*x1**2*x2*x3*x5*x6+x2**2*x3*x4*&
&x5**2+x1*x2**2*x3*x5**2+x2*x3**2*x4**2*x5+x1*x3**2*x4**2*x5
dpdx6(179) = 2*x1*x3*x4**2*x5*x6+2*x1*x2**2*x3*x5*x6+x2*x3*x4**2*&
&x5**2+x1**2*x2*x3*x5**2+x2**2*x3**2*x4*x5+x1**2*x3**2*x4*x5
dpdx6(180) = 2*x1**2*x2*x4*x5*x6+2*x1**2*x2*x3*x4*x6+x1*x2**2*x4*&
&x5**2+x1*x2*x3**2*x4**2
dpdx6(181) = 2*x2*x3*x4**2*x5*x6+2*x2**2*x3*x4*x5*x6+x1*x3*x4**2*&
&x5**2+x1**2*x3*x4*x5**2+x1*x2**2*x3**2*x5+x1**2*x2*x3**2*x5
dpdx6(182) = 2*x1*x2**2*x4*x5*x6+2*x1*x2*x3*x4**2*x6+x1**2*x2*x4*&
&x5**2+x1**2*x2*x3**2*x4
dpdx6(183) = 2*x1*x2*x4**2*x5*x6+2*x1*x2**2*x3*x4*x6+x1*x2*x4**2*&
&x5**2+x1*x2**2*x3**2*x4
dpdx6(184) = x1**2*x2**2*x4*x5+x1**2*x2*x3*x4**2
dpdx6(185) = x2**2*x3*x4**2*x5+x1**2*x3*x4**2*x5+x1**2*x2**2*x3*x&
&5
dpdx6(186) = x1*x2**2*x4**2*x5+x1**2*x2*x4**2*x5+x1*x2**2*x3*x4**&
&2+x1**2*x2**2*x3*x4
dpdx6(187) = x2*x3*x4**3*x5+x1*x3*x4**3*x5+x2**3*x3*x4*x5+x1**3*x&
&3*x4*x5+x1*x2**3*x3*x5+x1**3*x2*x3*x5
dpdx6(188) = x1*x2**3*x4*x5+x1**3*x2*x4*x5+x1*x2*x3*x4**3+x1**3*x&
&2*x3*x4
dpdx6(189) = x1*x2*x4**3*x5+x1*x2**3*x3*x4
dpdx6(190) = 4*x1*x3*x5*x6**3+x2*x3*x5**4+x3**4*x4*x5
dpdx6(191) = 3*x1*x3*x5**2*x6**2+3*x1*x3**2*x5*x6**2+2*x2*x3*x5**&
&3*x6+2*x3**3*x4*x5*x6+x2*x3**2*x5**3+x3**3*x4*x5**2
dpdx6(192) = 2*x3**2*x4*x5**2*x6+2*x2*x3**2*x5**2*x6+2*x1*x3**2*x&
&5**2*x6
dpdx6(193) = 3*x2*x3*x5**2*x6**2+3*x3**2*x4*x5*x6**2+2*x1*x3*x5**&
&3*x6+2*x1*x3**3*x5*x6+x3**2*x4*x5**3+x2*x3**3*x5**2
dpdx6(194) = 3*x3*x4*x5**2*x6**2+3*x2*x3**2*x5*x6**2+2*x3*x4*x5**&
&3*x6+2*x2*x3**3*x5*x6+x1*x3**2*x5**3+x1*x3**3*x5**2
dpdx6(195) = 4*x3*x4*x5*x6**3+4*x2*x3*x5*x6**3+x3*x4*x5**4+x1*x3*&
&x5**4+x2*x3**4*x5+x1*x3**4*x5
dpdx6(196) = 3*x1*x2*x5**2*x6**2+3*x1*x3**2*x4*x6**2+2*x1*x2*x5**&
&3*x6+2*x1*x3**3*x4*x6
dpdx6(197) = 4*x1*x2*x5*x6**3+4*x1*x3*x4*x6**3+x1*x2*x5**4+x1*x3*&
&*4*x4
dpdx6(198) = 4*x1*x4*x5*x6**3+4*x1*x2*x3*x6**3+x2*x4*x5**4+x2*x3*&
&*4*x4
dpdx6(199) = 3*x1*x4*x5**2*x6**2+3*x1*x2*x3**2*x6**2+2*x2*x4*x5**&
&3*x6+2*x2*x3**3*x4*x6
dpdx6(200) = 3*x2*x4*x5**2*x6**2+3*x2*x3**2*x4*x6**2+2*x1*x4*x5**&
&3*x6+2*x1*x2*x3**3*x6
dpdx6(201) = 4*x2*x4*x5*x6**3+4*x2*x3*x4*x6**3+x1*x4*x5**4+x1*x2*&
&x3**4
dpdx6(202) = 4*x1*x2*x4*x6**3
dpdx6(203) = 3*x1**2*x3*x5*x6**2+x2**2*x3*x5**3+x3**3*x4**2*x5
dpdx6(204) = 2*x2**2*x3*x5**2*x6+2*x1**2*x3*x5**2*x6+2*x3**2*x4**&
&2*x5*x6+2*x1**2*x3**2*x5*x6+x3**2*x4**2*x5**2+x2**2*x3**2*x5**2
dpdx6(205) = 2*x3*x4**2*x5**2*x6+2*x2**2*x3**2*x5*x6+x1**2*x3**2*&
&x5**2
dpdx6(206) = 3*x3*x4**2*x5*x6**2+3*x2**2*x3*x5*x6**2+x3*x4**2*x5*&
&*3+x1**2*x3*x5**3+x2**2*x3**3*x5+x1**2*x3**3*x5
dpdx6(207) = 3*x1**2*x2*x5*x6**2+3*x1**2*x3*x4*x6**2+x1*x2**2*x5*&
&*3+x1*x3**3*x4**2
dpdx6(208) = 2*x1*x2**2*x5**2*x6+2*x1**2*x2*x5**2*x6+2*x1*x3**2*x&
&4**2*x6+2*x1**2*x3**2*x4*x6
dpdx6(209) = 3*x1*x2**2*x5*x6**2+3*x1*x3*x4**2*x6**2+x1**2*x2*x5*&
&*3+x1**2*x3**3*x4
dpdx6(210) = 3*x1**2*x2*x4*x6**2
dpdx6(211) = 3*x1**2*x4*x5*x6**2+3*x1**2*x2*x3*x6**2+x2**2*x4*x5*&
&*3+x2*x3**3*x4**2
dpdx6(212) = 3*x1*x4**2*x5*x6**2+3*x1*x2**2*x3*x6**2+x2*x4**2*x5*&
&*3+x2**2*x3**3*x4
dpdx6(213) = 2*x2**2*x4*x5**2*x6+2*x1**2*x4*x5**2*x6+2*x2*x3**2*x&
&4**2*x6+2*x1**2*x2*x3**2*x6
dpdx6(214) = 2*x2*x4**2*x5**2*x6+2*x1*x4**2*x5**2*x6+2*x2**2*x3**&
&2*x4*x6+2*x1*x2**2*x3**2*x6
dpdx6(215) = 3*x2**2*x4*x5*x6**2+3*x2*x3*x4**2*x6**2+x1**2*x4*x5*&
&*3+x1**2*x2*x3**3
dpdx6(216) = 3*x2*x4**2*x5*x6**2+3*x2**2*x3*x4*x6**2+x1*x4**2*x5*&
&*3+x1*x2**2*x3**3
dpdx6(217) = 3*x1*x2*x4**2*x6**2+3*x1*x2**2*x4*x6**2
dpdx6(218) = 2*x1**2*x2**2*x5*x6+2*x1**2*x3*x4**2*x6+x1**2*x2**2*&
&x5**2+x1**2*x3**2*x4**2
dpdx6(219) = 2*x1**2*x2*x4**2*x6+2*x1**2*x2**2*x4*x6
dpdx6(220) = 2*x1**2*x4**2*x5*x6+2*x1**2*x2**2*x3*x6+x2**2*x4**2*&
&x5**2+x2**2*x3**2*x4**2
dpdx6(221) = x1**2*x2**2*x4**2
dpdx6(222) = 2*x2**2*x4**2*x5*x6+2*x2**2*x3*x4**2*x6+x1**2*x4**2*&
&x5**2+x1**2*x2**2*x3**2
dpdx6(223) = 2*x1*x2**2*x4**2*x6
dpdx6(224) = 2*x1**3*x3*x5*x6+x2**3*x3*x5**2+x3**2*x4**3*x5
dpdx6(225) = 2*x3*x4**3*x5*x6+2*x2**3*x3*x5*x6+x3*x4**3*x5**2+x1*&
&*3*x3*x5**2+x2**3*x3**2*x5+x1**3*x3**2*x5
dpdx6(226) = 2*x1**3*x2*x5*x6+2*x1**3*x3*x4*x6+x1*x2**3*x5**2+x1*&
&x3**2*x4**3
dpdx6(227) = 2*x1*x2**3*x5*x6+2*x1*x3*x4**3*x6+x1**3*x2*x5**2+x1*&
&*3*x3**2*x4
dpdx6(228) = 2*x1**3*x2*x4*x6
dpdx6(229) = 2*x1**3*x4*x5*x6+2*x1**3*x2*x3*x6+x2**3*x4*x5**2+x2*&
&x3**2*x4**3
dpdx6(230) = 2*x1*x4**3*x5*x6+2*x1*x2**3*x3*x6+x2*x4**3*x5**2+x2*&
&*3*x3**2*x4
dpdx6(231) = 2*x2**3*x4*x5*x6+2*x2*x3*x4**3*x6+x1**3*x4*x5**2+x1*&
&*3*x2*x3**2
dpdx6(232) = 2*x2*x4**3*x5*x6+2*x2**3*x3*x4*x6+x1*x4**3*x5**2+x1*&
&x2**3*x3**2
dpdx6(233) = 2*x1*x2*x4**3*x6+2*x1*x2**3*x4*x6
dpdx6(234) = x1**2*x2**3*x5+x1**3*x2**2*x5+x1**2*x3*x4**3+x1**3*x&
&3*x4**2
dpdx6(235) = x1**3*x2*x4**2+x1**3*x2**2*x4
dpdx6(236) = x1**2*x2*x4**3+x1**2*x2**3*x4
dpdx6(237) = x2**3*x4**2*x5+x1**3*x4**2*x5+x2**2*x3*x4**3+x1**3*x&
&2**2*x3
dpdx6(238) = x2**2*x4**3*x5+x1**2*x4**3*x5+x2**3*x3*x4**2+x1**2*x&
&2**3*x3
dpdx6(239) = x1*x2**2*x4**3+x1*x2**3*x4**2
dpdx6(240) = x3*x4**4*x5+x2**4*x3*x5+x1**4*x3*x5
dpdx6(241) = x1*x2**4*x5+x1**4*x2*x5+x1*x3*x4**4+x1**4*x3*x4
dpdx6(242) = x1**4*x2*x4
dpdx6(243) = x2**4*x4*x5+x1**4*x4*x5+x2*x3*x4**4+x1**4*x2*x3
dpdx6(244) = x2*x4**4*x5+x1*x4**4*x5+x2**4*x3*x4+x1*x2**4*x3
dpdx6(245) = x1*x2*x4**4+x1*x2**4*x4
dpdx6(246) = 3*x3**2*x5**2*x6**2+2*x3**2*x5**3*x6+2*x3**3*x5**2*x&
&6
dpdx6(247) = 3*x3*x5**3*x6**2+3*x3**3*x5*x6**2+x3**3*x5**3
dpdx6(248) = 4*x3*x5**2*x6**3+4*x3**2*x5*x6**3+2*x3*x5**4*x6+2*x3&
&**4*x5*x6+x3**2*x5**4+x3**4*x5**2
dpdx6(249) = 5*x3*x5*x6**4+x3*x5**5+x3**5*x5
dpdx6(250) = 5*x1*x5*x6**4+5*x1*x3*x6**4+x2*x5**5+x3**5*x4
dpdx6(251) = 4*x1*x5**2*x6**3+4*x1*x3**2*x6**3+2*x2*x5**4*x6+2*x3&
&**4*x4*x6
dpdx6(252) = 3*x2*x5**3*x6**2+3*x1*x5**3*x6**2+3*x3**3*x4*x6**2+3&
&*x1*x3**3*x6**2
dpdx6(253) = 3*x4*x5**3*x6**2+3*x2*x3**3*x6**2
dpdx6(254) = 4*x2*x5**2*x6**3+4*x3**2*x4*x6**3+2*x1*x5**4*x6+2*x1&
&*x3**4*x6
dpdx6(255) = 4*x4*x5**2*x6**3+4*x2*x3**2*x6**3+2*x4*x5**4*x6+2*x2&
&*x3**4*x6
dpdx6(256) = 5*x2*x5*x6**4+5*x3*x4*x6**4+x1*x5**5+x1*x3**5
dpdx6(257) = 5*x4*x5*x6**4+5*x2*x3*x6**4+x4*x5**5+x2*x3**5
dpdx6(258) = 5*x1*x4*x6**4+5*x1*x2*x6**4
dpdx6(259) = 5*x2*x4*x6**4
dpdx6(260) = 4*x1**2*x5*x6**3+4*x1**2*x3*x6**3+x2**2*x5**4+x3**4*&
&x4**2
dpdx6(261) = 3*x1**2*x5**2*x6**2+3*x1**2*x3**2*x6**2+2*x2**2*x5**&
&3*x6+2*x3**3*x4**2*x6
dpdx6(262) = 3*x2**2*x5**2*x6**2+3*x3**2*x4**2*x6**2+2*x1**2*x5**&
&3*x6+2*x1**2*x3**3*x6
dpdx6(263) = 3*x4**2*x5**2*x6**2+3*x2**2*x3**2*x6**2+2*x4**2*x5**&
&3*x6+2*x2**2*x3**3*x6
dpdx6(264) = 4*x2**2*x5*x6**3+4*x3*x4**2*x6**3+x1**2*x5**4+x1**2*&
&x3**4
dpdx6(265) = 4*x4**2*x5*x6**3+4*x2**2*x3*x6**3+x4**2*x5**4+x2**2*&
&x3**4
dpdx6(266) = 4*x1**2*x4*x6**3+4*x1**2*x2*x6**3
dpdx6(267) = 4*x1*x4**2*x6**3+4*x1*x2**2*x6**3
dpdx6(268) = 4*x2*x4**2*x6**3+4*x2**2*x4*x6**3
dpdx6(269) = 3*x1**2*x4**2*x6**2+3*x1**2*x2**2*x6**2
dpdx6(270) = 3*x2**2*x4**2*x6**2
dpdx6(271) = 3*x1**3*x5*x6**2+3*x1**3*x3*x6**2+x2**3*x5**3+x3**3*&
&x4**3
dpdx6(272) = 2*x2**3*x5**2*x6+2*x1**3*x5**2*x6+2*x3**2*x4**3*x6+2&
&*x1**3*x3**2*x6
dpdx6(273) = 2*x4**3*x5**2*x6+2*x2**3*x3**2*x6
dpdx6(274) = 3*x2**3*x5*x6**2+3*x3*x4**3*x6**2+x1**3*x5**3+x1**3*&
&x3**3
dpdx6(275) = 3*x4**3*x5*x6**2+3*x2**3*x3*x6**2+x4**3*x5**3+x2**3*&
&x3**3
dpdx6(276) = 3*x1**3*x4*x6**2+3*x1**3*x2*x6**2
dpdx6(277) = 3*x1*x4**3*x6**2+3*x1*x2**3*x6**2
dpdx6(278) = 3*x2*x4**3*x6**2+3*x2**3*x4*x6**2
dpdx6(279) = 2*x1**3*x4**2*x6+2*x1**3*x2**2*x6
dpdx6(280) = 2*x1**2*x4**3*x6+2*x1**2*x2**3*x6
dpdx6(281) = 0
dpdx6(282) = 2*x2**2*x4**3*x6+2*x2**3*x4**2*x6
dpdx6(283) = x1**3*x4**3+x1**3*x2**3
dpdx6(284) = 0
dpdx6(285) = x2**3*x4**3
dpdx6(286) = 2*x1**4*x5*x6+2*x1**4*x3*x6+x2**4*x5**2+x3**2*x4**4
dpdx6(287) = 2*x2**4*x5*x6+2*x3*x4**4*x6+x1**4*x5**2+x1**4*x3**2
dpdx6(288) = 2*x4**4*x5*x6+2*x2**4*x3*x6+x4**4*x5**2+x2**4*x3**2
dpdx6(289) = 2*x1**4*x4*x6+2*x1**4*x2*x6
dpdx6(290) = 2*x1*x4**4*x6+2*x1*x2**4*x6
dpdx6(291) = 2*x2*x4**4*x6+2*x2**4*x4*x6
dpdx6(292) = x1**4*x4**2+x1**4*x2**2
dpdx6(293) = x1**2*x4**4+x1**2*x2**4
dpdx6(294) = 0
dpdx6(295) = x2**2*x4**4+x2**4*x4**2
dpdx6(296) = x2**5*x5+x1**5*x5+x3*x4**5+x1**5*x3
dpdx6(297) = x4**5*x5+x2**5*x3
dpdx6(298) = x1**5*x4+x1**5*x2
dpdx6(299) = x1*x4**5+x1*x2**5
dpdx6(300) = 0
dpdx6(301) = x2*x4**5+x2**5*x4
dpdx6(302) = 4*x5**3*x6**3+4*x3**3*x6**3+3*x5**4*x6**2+3*x3**4*x6&
&**2
dpdx6(303) = 5*x5**2*x6**4+5*x3**2*x6**4+2*x5**5*x6+2*x3**5*x6
dpdx6(304) = 6*x5*x6**5+6*x3*x6**5+x5**6+x3**6
dpdx6(305) = 6*x4*x6**5+6*x2*x6**5
dpdx6(306) = 5*x4**2*x6**4+5*x2**2*x6**4
dpdx6(307) = 4*x4**3*x6**3+4*x2**3*x6**3
dpdx6(308) = 3*x4**4*x6**2+3*x2**4*x6**2
dpdx6(309) = 0
dpdx6(310) = 2*x4**5*x6+2*x2**5*x6
dpdx6(311) = 0
dpdx6(312) = x4**6+x2**6
dpdx6(313) = 0
dpdx6(314) = 2*x1*x2*x3*x4*x5**2*x6+2*x1*x2*x3**2*x4*x5*x6+x1*x2*&
&x3**2*x4*x5**2
dpdx6(315) = 3*x1*x2*x3*x4*x5*x6**2+x1*x2*x3*x4*x5**3+x1*x2*x3**3&
&*x4*x5
dpdx6(316) = 2*x1**2*x2*x3*x4*x5*x6+x1*x2**2*x3*x4*x5**2+x1*x2*x3&
&**2*x4**2*x5
dpdx6(317) = 2*x1*x2*x3*x4**2*x5*x6+2*x1*x2**2*x3*x4*x5*x6+x1*x2*&
&x3*x4**2*x5**2+x1**2*x2*x3*x4*x5**2+x1*x2**2*x3**2*x4*x5+x1**2*x2&
&*x3**2*x4*x5
dpdx6(318) = x1*x2**2*x3*x4**2*x5+x1**2*x2*x3*x4**2*x5+x1**2*x2**&
&2*x3*x4*x5
dpdx6(319) = x1*x2*x3*x4**3*x5+x1*x2**3*x3*x4*x5+x1**3*x2*x3*x4*x&
&5
dpdx6(320) = 3*x1*x2*x3*x5**2*x6**2+3*x1*x3**2*x4*x5*x6**2+2*x1*x&
&2*x3*x5**3*x6+2*x1*x3**3*x4*x5*x6+x2*x3**2*x4*x5**3+x2*x3**3*x4*x&
&5**2
dpdx6(321) = 4*x1*x3*x4*x5*x6**3+4*x1*x2*x3*x5*x6**3+x2*x3*x4*x5*&
&*4+x1*x2*x3*x5**4+x2*x3**4*x4*x5+x1*x3**4*x4*x5
dpdx6(322) = 2*x2*x3**2*x4*x5**2*x6+2*x1*x3**2*x4*x5**2*x6+2*x1*x&
&2*x3**2*x5**2*x6
dpdx6(323) = 3*x1*x3*x4*x5**2*x6**2+3*x1*x2*x3**2*x5*x6**2+2*x2*x&
&3*x4*x5**3*x6+2*x2*x3**3*x4*x5*x6+x1*x2*x3**2*x5**3+x1*x3**3*x4*x&
&5**2
dpdx6(324) = 3*x2*x3*x4*x5**2*x6**2+3*x2*x3**2*x4*x5*x6**2+2*x1*x&
&3*x4*x5**3*x6+2*x1*x2*x3**3*x5*x6+x1*x3**2*x4*x5**3+x1*x2*x3**3*x&
&5**2
dpdx6(325) = 3*x1*x2*x4*x5**2*x6**2+3*x1*x2*x3**2*x4*x6**2+2*x1*x&
&2*x4*x5**3*x6+2*x1*x2*x3**3*x4*x6
dpdx6(326) = 4*x2*x3*x4*x5*x6**3+x1*x3*x4*x5**4+x1*x2*x3**4*x5
dpdx6(327) = 4*x1*x2*x4*x5*x6**3+4*x1*x2*x3*x4*x6**3+x1*x2*x4*x5*&
&*4+x1*x2*x3**4*x4
dpdx6(328) = 3*x1**2*x3*x4*x5*x6**2+3*x1**2*x2*x3*x5*x6**2+x2**2*&
&x3*x4*x5**3+x1*x2**2*x3*x5**3+x2*x3**3*x4**2*x5+x1*x3**3*x4**2*x5
dpdx6(329) = 2*x1*x2**2*x3*x5**2*x6+2*x1**2*x2*x3*x5**2*x6+2*x1*x&
&3**2*x4**2*x5*x6+2*x1**2*x3**2*x4*x5*x6+x2*x3**2*x4**2*x5**2+x2**&
&2*x3**2*x4*x5**2
dpdx6(330) = 3*x1*x3*x4**2*x5*x6**2+3*x1*x2**2*x3*x5*x6**2+x2*x3*&
&x4**2*x5**3+x1**2*x2*x3*x5**3+x2**2*x3**3*x4*x5+x1**2*x3**3*x4*x5
dpdx6(331) = 3*x1**2*x2*x4*x5*x6**2+3*x1**2*x2*x3*x4*x6**2+x1*x2*&
&*2*x4*x5**3+x1*x2*x3**3*x4**2
dpdx6(332) = 2*x2**2*x3*x4*x5**2*x6+2*x1**2*x3*x4*x5**2*x6+2*x2*x&
&3**2*x4**2*x5*x6+2*x1**2*x2*x3**2*x5*x6+x1*x3**2*x4**2*x5**2+x1*x&
&2**2*x3**2*x5**2
dpdx6(333) = 2*x2*x3*x4**2*x5**2*x6+2*x1*x3*x4**2*x5**2*x6+2*x2**&
&2*x3**2*x4*x5*x6+2*x1*x2**2*x3**2*x5*x6+x1**2*x3**2*x4*x5**2+x1**&
&2*x2*x3**2*x5**2
dpdx6(334) = 2*x1*x2**2*x4*x5**2*x6+2*x1**2*x2*x4*x5**2*x6+2*x1*x&
&2*x3**2*x4**2*x6+2*x1**2*x2*x3**2*x4*x6
dpdx6(335) = 2*x1*x2*x4**2*x5**2*x6+2*x1*x2**2*x3**2*x4*x6
dpdx6(336) = 3*x2*x3*x4**2*x5*x6**2+3*x2**2*x3*x4*x5*x6**2+x1*x3*&
&x4**2*x5**3+x1**2*x3*x4*x5**3+x1*x2**2*x3**3*x5+x1**2*x2*x3**3*x5
dpdx6(337) = 3*x1*x2**2*x4*x5*x6**2+3*x1*x2*x3*x4**2*x6**2+x1**2*&
&x2*x4*x5**3+x1**2*x2*x3**3*x4
dpdx6(338) = 3*x1*x2*x4**2*x5*x6**2+3*x1*x2**2*x3*x4*x6**2+x1*x2*&
&x4**2*x5**3+x1*x2**2*x3**3*x4
dpdx6(339) = 2*x1**2*x2**2*x4*x5*x6+2*x1**2*x2*x3*x4**2*x6+x1**2*&
&x2**2*x4*x5**2+x1**2*x2*x3**2*x4**2
dpdx6(340) = 2*x1**2*x3*x4**2*x5*x6+2*x1**2*x2**2*x3*x5*x6+x2**2*&
&x3*x4**2*x5**2+x1**2*x2**2*x3*x5**2+x2**2*x3**2*x4**2*x5+x1**2*x3&
&**2*x4**2*x5
dpdx6(341) = 2*x1**2*x2*x4**2*x5*x6+2*x1**2*x2**2*x3*x4*x6+x1*x2*&
&*2*x4**2*x5**2+x1*x2**2*x3**2*x4**2
dpdx6(342) = x1**2*x2**2*x4**2*x5+x1**2*x2**2*x3*x4**2
dpdx6(343) = 2*x2**2*x3*x4**2*x5*x6+x1**2*x3*x4**2*x5**2+x1**2*x2&
&**2*x3**2*x5
dpdx6(344) = 2*x1*x2**2*x4**2*x5*x6+2*x1*x2**2*x3*x4**2*x6+x1**2*&
&x2*x4**2*x5**2+x1**2*x2**2*x3**2*x4
dpdx6(345) = 2*x1**3*x3*x4*x5*x6+2*x1**3*x2*x3*x5*x6+x2**3*x3*x4*&
&x5**2+x1*x2**3*x3*x5**2+x2*x3**2*x4**3*x5+x1*x3**2*x4**3*x5
dpdx6(346) = 2*x1*x3*x4**3*x5*x6+2*x1*x2**3*x3*x5*x6+x2*x3*x4**3*&
&x5**2+x1**3*x2*x3*x5**2+x2**3*x3**2*x4*x5+x1**3*x3**2*x4*x5
dpdx6(347) = 2*x1**3*x2*x4*x5*x6+2*x1**3*x2*x3*x4*x6+x1*x2**3*x4*&
&x5**2+x1*x2*x3**2*x4**3
dpdx6(348) = 2*x2*x3*x4**3*x5*x6+2*x2**3*x3*x4*x5*x6+x1*x3*x4**3*&
&x5**2+x1**3*x3*x4*x5**2+x1*x2**3*x3**2*x5+x1**3*x2*x3**2*x5
dpdx6(349) = 2*x1*x2**3*x4*x5*x6+2*x1*x2*x3*x4**3*x6+x1**3*x2*x4*&
&x5**2+x1**3*x2*x3**2*x4
dpdx6(350) = 2*x1*x2*x4**3*x5*x6+2*x1*x2**3*x3*x4*x6+x1*x2*x4**3*&
&x5**2+x1*x2**3*x3**2*x4
dpdx6(351) = x1**2*x2**3*x4*x5+x1**3*x2**2*x4*x5+x1**2*x2*x3*x4**&
&3+x1**3*x2*x3*x4**2
dpdx6(352) = x2**2*x3*x4**3*x5+x1**2*x3*x4**3*x5+x2**3*x3*x4**2*x&
&5+x1**3*x3*x4**2*x5+x1**2*x2**3*x3*x5+x1**3*x2**2*x3*x5
dpdx6(353) = x1*x2**3*x4**2*x5+x1**3*x2*x4**2*x5+x1*x2**2*x3*x4**&
&3+x1**3*x2**2*x3*x4
dpdx6(354) = x1*x2**2*x4**3*x5+x1**2*x2*x4**3*x5+x1*x2**3*x3*x4**&
&2+x1**2*x2**3*x3*x4
dpdx6(355) = x2*x3*x4**4*x5+x1*x3*x4**4*x5+x2**4*x3*x4*x5+x1**4*x&
&3*x4*x5+x1*x2**4*x3*x5+x1**4*x2*x3*x5
dpdx6(356) = x1*x2**4*x4*x5+x1**4*x2*x4*x5+x1*x2*x3*x4**4+x1**4*x&
&2*x3*x4
dpdx6(357) = x1*x2*x4**4*x5+x1*x2**4*x3*x4
dpdx6(358) = 5*x1*x3*x5*x6**4+x2*x3*x5**5+x3**5*x4*x5
dpdx6(359) = 4*x1*x3*x5**2*x6**3+4*x1*x3**2*x5*x6**3+2*x2*x3*x5**&
&4*x6+2*x3**4*x4*x5*x6+x2*x3**2*x5**4+x3**4*x4*x5**2
dpdx6(360) = 3*x1*x3**2*x5**2*x6**2+2*x2*x3**2*x5**3*x6+2*x3**3*x&
&4*x5**2*x6
dpdx6(361) = 3*x2*x3*x5**3*x6**2+3*x1*x3*x5**3*x6**2+3*x3**3*x4*x&
&5*x6**2+3*x1*x3**3*x5*x6**2+x3**3*x4*x5**3+x2*x3**3*x5**3
dpdx6(362) = 3*x3**2*x4*x5**2*x6**2+3*x2*x3**2*x5**2*x6**2+2*x3**&
&2*x4*x5**3*x6+2*x1*x3**2*x5**3*x6+2*x2*x3**3*x5**2*x6+2*x1*x3**3*&
&x5**2*x6
dpdx6(363) = 3*x3*x4*x5**3*x6**2+3*x2*x3**3*x5*x6**2+x1*x3**3*x5*&
&*3
dpdx6(364) = 4*x2*x3*x5**2*x6**3+4*x3**2*x4*x5*x6**3+2*x1*x3*x5**&
&4*x6+2*x1*x3**4*x5*x6+x3**2*x4*x5**4+x2*x3**4*x5**2
dpdx6(365) = 4*x3*x4*x5**2*x6**3+4*x2*x3**2*x5*x6**3+2*x3*x4*x5**&
&4*x6+2*x2*x3**4*x5*x6+x1*x3**2*x5**4+x1*x3**4*x5**2
dpdx6(366) = 5*x3*x4*x5*x6**4+5*x2*x3*x5*x6**4+x3*x4*x5**5+x1*x3*&
&x5**5+x2*x3**5*x5+x1*x3**5*x5
dpdx6(367) = 3*x1*x2*x5**3*x6**2+3*x1*x3**3*x4*x6**2
dpdx6(368) = 4*x1*x2*x5**2*x6**3+4*x1*x3**2*x4*x6**3+2*x1*x2*x5**&
&4*x6+2*x1*x3**4*x4*x6
dpdx6(369) = 5*x1*x2*x5*x6**4+5*x1*x3*x4*x6**4+x1*x2*x5**5+x1*x3*&
&*5*x4
dpdx6(370) = 5*x1*x4*x5*x6**4+5*x1*x2*x3*x6**4+x2*x4*x5**5+x2*x3*&
&*5*x4
dpdx6(371) = 4*x1*x4*x5**2*x6**3+4*x1*x2*x3**2*x6**3+2*x2*x4*x5**&
&4*x6+2*x2*x3**4*x4*x6
dpdx6(372) = 3*x2*x4*x5**3*x6**2+3*x1*x4*x5**3*x6**2+3*x2*x3**3*x&
&4*x6**2+3*x1*x2*x3**3*x6**2
dpdx6(373) = 4*x2*x4*x5**2*x6**3+4*x2*x3**2*x4*x6**3+2*x1*x4*x5**&
&4*x6+2*x1*x2*x3**4*x6
dpdx6(374) = 5*x2*x4*x5*x6**4+5*x2*x3*x4*x6**4+x1*x4*x5**5+x1*x2*&
&x3**5
dpdx6(375) = 5*x1*x2*x4*x6**4
dpdx6(376) = 4*x1**2*x3*x5*x6**3+x2**2*x3*x5**4+x3**4*x4**2*x5
dpdx6(377) = 3*x1**2*x3*x5**2*x6**2+3*x1**2*x3**2*x5*x6**2+2*x2**&
&2*x3*x5**3*x6+2*x3**3*x4**2*x5*x6+x2**2*x3**2*x5**3+x3**3*x4**2*x&
&5**2
dpdx6(378) = 2*x3**2*x4**2*x5**2*x6+2*x2**2*x3**2*x5**2*x6+2*x1**&
&2*x3**2*x5**2*x6
dpdx6(379) = 3*x2**2*x3*x5**2*x6**2+3*x3**2*x4**2*x5*x6**2+2*x1**&
&2*x3*x5**3*x6+2*x1**2*x3**3*x5*x6+x3**2*x4**2*x5**3+x2**2*x3**3*x&
&5**2
dpdx6(380) = 3*x3*x4**2*x5**2*x6**2+3*x2**2*x3**2*x5*x6**2+2*x3*x&
&4**2*x5**3*x6+2*x2**2*x3**3*x5*x6+x1**2*x3**2*x5**3+x1**2*x3**3*x&
&5**2
dpdx6(381) = 4*x3*x4**2*x5*x6**3+4*x2**2*x3*x5*x6**3+x3*x4**2*x5*&
&*4+x1**2*x3*x5**4+x2**2*x3**4*x5+x1**2*x3**4*x5
dpdx6(382) = 4*x1**2*x2*x5*x6**3+4*x1**2*x3*x4*x6**3+x1*x2**2*x5*&
&*4+x1*x3**4*x4**2
dpdx6(383) = 3*x1**2*x2*x5**2*x6**2+3*x1**2*x3**2*x4*x6**2+2*x1*x&
&2**2*x5**3*x6+2*x1*x3**3*x4**2*x6
dpdx6(384) = 3*x1*x2**2*x5**2*x6**2+3*x1*x3**2*x4**2*x6**2+2*x1**&
&2*x2*x5**3*x6+2*x1**2*x3**3*x4*x6
dpdx6(385) = 4*x1*x2**2*x5*x6**3+4*x1*x3*x4**2*x6**3+x1**2*x2*x5*&
&*4+x1**2*x3**4*x4
dpdx6(386) = 4*x1**2*x2*x4*x6**3
dpdx6(387) = 4*x1**2*x4*x5*x6**3+4*x1**2*x2*x3*x6**3+x2**2*x4*x5*&
&*4+x2*x3**4*x4**2
dpdx6(388) = 4*x1*x4**2*x5*x6**3+4*x1*x2**2*x3*x6**3+x2*x4**2*x5*&
&*4+x2**2*x3**4*x4
dpdx6(389) = 3*x1**2*x4*x5**2*x6**2+3*x1**2*x2*x3**2*x6**2+2*x2**&
&2*x4*x5**3*x6+2*x2*x3**3*x4**2*x6
dpdx6(390) = 3*x1*x4**2*x5**2*x6**2+3*x1*x2**2*x3**2*x6**2+2*x2*x&
&4**2*x5**3*x6+2*x2**2*x3**3*x4*x6
dpdx6(391) = 3*x2**2*x4*x5**2*x6**2+3*x2*x3**2*x4**2*x6**2+2*x1**&
&2*x4*x5**3*x6+2*x1**2*x2*x3**3*x6
dpdx6(392) = 3*x2*x4**2*x5**2*x6**2+3*x2**2*x3**2*x4*x6**2+2*x1*x&
&4**2*x5**3*x6+2*x1*x2**2*x3**3*x6
dpdx6(393) = 4*x2**2*x4*x5*x6**3+4*x2*x3*x4**2*x6**3+x1**2*x4*x5*&
&*4+x1**2*x2*x3**4
dpdx6(394) = 4*x2*x4**2*x5*x6**3+4*x2**2*x3*x4*x6**3+x1*x4**2*x5*&
&*4+x1*x2**2*x3**4
dpdx6(395) = 4*x1*x2*x4**2*x6**3+4*x1*x2**2*x4*x6**3
dpdx6(396) = 2*x1**2*x2**2*x5**2*x6+2*x1**2*x3**2*x4**2*x6
dpdx6(397) = 3*x1**2*x2**2*x5*x6**2+3*x1**2*x3*x4**2*x6**2+x1**2*&
&x2**2*x5**3+x1**2*x3**3*x4**2
dpdx6(398) = 3*x1**2*x2*x4**2*x6**2+3*x1**2*x2**2*x4*x6**2
dpdx6(399) = 3*x1**2*x4**2*x5*x6**2+3*x1**2*x2**2*x3*x6**2+x2**2*&
&x4**2*x5**3+x2**2*x3**3*x4**2
dpdx6(400) = 2*x2**2*x4**2*x5**2*x6+2*x1**2*x4**2*x5**2*x6+2*x2**&
&2*x3**2*x4**2*x6+2*x1**2*x2**2*x3**2*x6
dpdx6(401) = 2*x1**2*x2**2*x4**2*x6
dpdx6(402) = 3*x2**2*x4**2*x5*x6**2+3*x2**2*x3*x4**2*x6**2+x1**2*&
&x4**2*x5**3+x1**2*x2**2*x3**3
dpdx6(403) = 3*x1*x2**2*x4**2*x6**2
dpdx6(404) = 3*x1**3*x3*x5*x6**2+x2**3*x3*x5**3+x3**3*x4**3*x5
dpdx6(405) = 2*x2**3*x3*x5**2*x6+2*x1**3*x3*x5**2*x6+2*x3**2*x4**&
&3*x5*x6+2*x1**3*x3**2*x5*x6+x3**2*x4**3*x5**2+x2**3*x3**2*x5**2
dpdx6(406) = 2*x3*x4**3*x5**2*x6+2*x2**3*x3**2*x5*x6+x1**3*x3**2*&
&x5**2
dpdx6(407) = 3*x3*x4**3*x5*x6**2+3*x2**3*x3*x5*x6**2+x3*x4**3*x5*&
&*3+x1**3*x3*x5**3+x2**3*x3**3*x5+x1**3*x3**3*x5
dpdx6(408) = 3*x1**3*x2*x5*x6**2+3*x1**3*x3*x4*x6**2+x1*x2**3*x5*&
&*3+x1*x3**3*x4**3
dpdx6(409) = 2*x1*x2**3*x5**2*x6+2*x1**3*x2*x5**2*x6+2*x1*x3**2*x&
&4**3*x6+2*x1**3*x3**2*x4*x6
dpdx6(410) = 3*x1*x2**3*x5*x6**2+3*x1*x3*x4**3*x6**2+x1**3*x2*x5*&
&*3+x1**3*x3**3*x4
dpdx6(411) = 3*x1**3*x2*x4*x6**2
dpdx6(412) = 3*x1**3*x4*x5*x6**2+3*x1**3*x2*x3*x6**2+x2**3*x4*x5*&
&*3+x2*x3**3*x4**3
dpdx6(413) = 3*x1*x4**3*x5*x6**2+3*x1*x2**3*x3*x6**2+x2*x4**3*x5*&
&*3+x2**3*x3**3*x4
dpdx6(414) = 2*x2**3*x4*x5**2*x6+2*x1**3*x4*x5**2*x6+2*x2*x3**2*x&
&4**3*x6+2*x1**3*x2*x3**2*x6
dpdx6(415) = 2*x2*x4**3*x5**2*x6+2*x1*x4**3*x5**2*x6+2*x2**3*x3**&
&2*x4*x6+2*x1*x2**3*x3**2*x6
dpdx6(416) = 3*x2**3*x4*x5*x6**2+3*x2*x3*x4**3*x6**2+x1**3*x4*x5*&
&*3+x1**3*x2*x3**3
dpdx6(417) = 3*x2*x4**3*x5*x6**2+3*x2**3*x3*x4*x6**2+x1*x4**3*x5*&
&*3+x1*x2**3*x3**3
dpdx6(418) = 3*x1*x2*x4**3*x6**2+3*x1*x2**3*x4*x6**2
dpdx6(419) = 2*x1**3*x2**2*x5*x6+2*x1**3*x3*x4**2*x6+x1**2*x2**3*&
&x5**2+x1**2*x3**2*x4**3
dpdx6(420) = 2*x1**2*x2**3*x5*x6+2*x1**2*x3*x4**3*x6+x1**3*x2**2*&
&x5**2+x1**3*x3**2*x4**2
dpdx6(421) = 2*x1**3*x2*x4**2*x6+2*x1**3*x2**2*x4*x6
dpdx6(422) = 2*x1**2*x2*x4**3*x6+2*x1**2*x2**3*x4*x6
dpdx6(423) = x1**3*x2**2*x4**2
dpdx6(424) = 2*x1**3*x4**2*x5*x6+2*x1**3*x2**2*x3*x6+x2**3*x4**2*&
&x5**2+x2**2*x3**2*x4**3
dpdx6(425) = 2*x1**2*x4**3*x5*x6+2*x1**2*x2**3*x3*x6+x2**2*x4**3*&
&x5**2+x2**3*x3**2*x4**2
dpdx6(426) = x1**2*x2**2*x4**3+x1**2*x2**3*x4**2
dpdx6(427) = 2*x2**3*x4**2*x5*x6+2*x2**2*x3*x4**3*x6+x1**3*x4**2*&
&x5**2+x1**3*x2**2*x3**2
dpdx6(428) = 2*x2**2*x4**3*x5*x6+2*x2**3*x3*x4**2*x6+x1**2*x4**3*&
&x5**2+x1**2*x2**3*x3**2
dpdx6(429) = 2*x1*x2**2*x4**3*x6+2*x1*x2**3*x4**2*x6
dpdx6(430) = x1**3*x2**3*x5+x1**3*x3*x4**3
dpdx6(431) = x1**3*x2*x4**3+x1**3*x2**3*x4
dpdx6(432) = x2**3*x4**3*x5+x1**3*x4**3*x5+x2**3*x3*x4**3+x1**3*x&
&2**3*x3
dpdx6(433) = x1*x2**3*x4**3
dpdx6(434) = 2*x1**4*x3*x5*x6+x2**4*x3*x5**2+x3**2*x4**4*x5
dpdx6(435) = 2*x3*x4**4*x5*x6+2*x2**4*x3*x5*x6+x3*x4**4*x5**2+x1*&
&*4*x3*x5**2+x2**4*x3**2*x5+x1**4*x3**2*x5
dpdx6(436) = 2*x1**4*x2*x5*x6+2*x1**4*x3*x4*x6+x1*x2**4*x5**2+x1*&
&x3**2*x4**4
dpdx6(437) = 2*x1*x2**4*x5*x6+2*x1*x3*x4**4*x6+x1**4*x2*x5**2+x1*&
&*4*x3**2*x4
dpdx6(438) = 2*x1**4*x2*x4*x6
dpdx6(439) = 2*x1**4*x4*x5*x6+2*x1**4*x2*x3*x6+x2**4*x4*x5**2+x2*&
&x3**2*x4**4
dpdx6(440) = 2*x1*x4**4*x5*x6+2*x1*x2**4*x3*x6+x2*x4**4*x5**2+x2*&
&*4*x3**2*x4
dpdx6(441) = 2*x2**4*x4*x5*x6+2*x2*x3*x4**4*x6+x1**4*x4*x5**2+x1*&
&*4*x2*x3**2
dpdx6(442) = 2*x2*x4**4*x5*x6+2*x2**4*x3*x4*x6+x1*x4**4*x5**2+x1*&
&x2**4*x3**2
dpdx6(443) = 2*x1*x2*x4**4*x6+2*x1*x2**4*x4*x6
dpdx6(444) = x1**2*x2**4*x5+x1**4*x2**2*x5+x1**2*x3*x4**4+x1**4*x&
&3*x4**2
dpdx6(445) = x1**4*x2*x4**2+x1**4*x2**2*x4
dpdx6(446) = x1**2*x2*x4**4+x1**2*x2**4*x4
dpdx6(447) = x2**4*x4**2*x5+x1**4*x4**2*x5+x2**2*x3*x4**4+x1**4*x&
&2**2*x3
dpdx6(448) = x2**2*x4**4*x5+x1**2*x4**4*x5+x2**4*x3*x4**2+x1**2*x&
&2**4*x3
dpdx6(449) = x1*x2**2*x4**4+x1*x2**4*x4**2
dpdx6(450) = x3*x4**5*x5+x2**5*x3*x5+x1**5*x3*x5
dpdx6(451) = x1*x2**5*x5+x1**5*x2*x5+x1*x3*x4**5+x1**5*x3*x4
dpdx6(452) = x1**5*x2*x4
dpdx6(453) = x2**5*x4*x5+x1**5*x4*x5+x2*x3*x4**5+x1**5*x2*x3
dpdx6(454) = x2*x4**5*x5+x1*x4**5*x5+x2**5*x3*x4+x1*x2**5*x3
dpdx6(455) = x1*x2*x4**5+x1*x2**5*x4
dpdx6(456) = 3*x3**2*x5**3*x6**2+3*x3**3*x5**2*x6**2+2*x3**3*x5**&
&3*x6
dpdx6(457) = 4*x3**2*x5**2*x6**3+2*x3**2*x5**4*x6+2*x3**4*x5**2*x&
&6
dpdx6(458) = 4*x3*x5**3*x6**3+4*x3**3*x5*x6**3+3*x3*x5**4*x6**2+3&
&*x3**4*x5*x6**2+x3**3*x5**4+x3**4*x5**3
dpdx6(459) = 5*x3*x5**2*x6**4+5*x3**2*x5*x6**4+2*x3*x5**5*x6+2*x3&
&**5*x5*x6+x3**2*x5**5+x3**5*x5**2
dpdx6(460) = 6*x3*x5*x6**5+x3*x5**6+x3**6*x5
dpdx6(461) = 6*x1*x5*x6**5+6*x1*x3*x6**5+x2*x5**6+x3**6*x4
dpdx6(462) = 5*x1*x5**2*x6**4+5*x1*x3**2*x6**4+2*x2*x5**5*x6+2*x3&
&**5*x4*x6
dpdx6(463) = 4*x1*x5**3*x6**3+4*x1*x3**3*x6**3+3*x2*x5**4*x6**2+3&
&*x3**4*x4*x6**2
dpdx6(464) = 4*x2*x5**3*x6**3+4*x3**3*x4*x6**3+3*x1*x5**4*x6**2+3&
&*x1*x3**4*x6**2
dpdx6(465) = 4*x4*x5**3*x6**3+4*x2*x3**3*x6**3+3*x4*x5**4*x6**2+3&
&*x2*x3**4*x6**2
dpdx6(466) = 5*x2*x5**2*x6**4+5*x3**2*x4*x6**4+2*x1*x5**5*x6+2*x1&
&*x3**5*x6
dpdx6(467) = 5*x4*x5**2*x6**4+5*x2*x3**2*x6**4+2*x4*x5**5*x6+2*x2&
&*x3**5*x6
dpdx6(468) = 6*x2*x5*x6**5+6*x3*x4*x6**5+x1*x5**6+x1*x3**6
dpdx6(469) = 6*x4*x5*x6**5+6*x2*x3*x6**5+x4*x5**6+x2*x3**6
dpdx6(470) = 6*x1*x4*x6**5+6*x1*x2*x6**5
dpdx6(471) = 6*x2*x4*x6**5
dpdx6(472) = 5*x1**2*x5*x6**4+5*x1**2*x3*x6**4+x2**2*x5**5+x3**5*&
&x4**2
dpdx6(473) = 4*x1**2*x5**2*x6**3+4*x1**2*x3**2*x6**3+2*x2**2*x5**&
&4*x6+2*x3**4*x4**2*x6
dpdx6(474) = 3*x2**2*x5**3*x6**2+3*x1**2*x5**3*x6**2+3*x3**3*x4**&
&2*x6**2+3*x1**2*x3**3*x6**2
dpdx6(475) = 3*x4**2*x5**3*x6**2+3*x2**2*x3**3*x6**2
dpdx6(476) = 4*x2**2*x5**2*x6**3+4*x3**2*x4**2*x6**3+2*x1**2*x5**&
&4*x6+2*x1**2*x3**4*x6
dpdx6(477) = 4*x4**2*x5**2*x6**3+4*x2**2*x3**2*x6**3+2*x4**2*x5**&
&4*x6+2*x2**2*x3**4*x6
dpdx6(478) = 5*x2**2*x5*x6**4+5*x3*x4**2*x6**4+x1**2*x5**5+x1**2*&
&x3**5
dpdx6(479) = 5*x4**2*x5*x6**4+5*x2**2*x3*x6**4+x4**2*x5**5+x2**2*&
&x3**5
dpdx6(480) = 5*x1**2*x4*x6**4+5*x1**2*x2*x6**4
dpdx6(481) = 5*x1*x4**2*x6**4+5*x1*x2**2*x6**4
dpdx6(482) = 5*x2*x4**2*x6**4+5*x2**2*x4*x6**4
dpdx6(483) = 4*x1**2*x4**2*x6**3+4*x1**2*x2**2*x6**3
dpdx6(484) = 4*x2**2*x4**2*x6**3
dpdx6(485) = 4*x1**3*x5*x6**3+4*x1**3*x3*x6**3+x2**3*x5**4+x3**4*&
&x4**3
dpdx6(486) = 3*x1**3*x5**2*x6**2+3*x1**3*x3**2*x6**2+2*x2**3*x5**&
&3*x6+2*x3**3*x4**3*x6
dpdx6(487) = 3*x2**3*x5**2*x6**2+3*x3**2*x4**3*x6**2+2*x1**3*x5**&
&3*x6+2*x1**3*x3**3*x6
dpdx6(488) = 3*x4**3*x5**2*x6**2+3*x2**3*x3**2*x6**2+2*x4**3*x5**&
&3*x6+2*x2**3*x3**3*x6
dpdx6(489) = 4*x2**3*x5*x6**3+4*x3*x4**3*x6**3+x1**3*x5**4+x1**3*&
&x3**4
dpdx6(490) = 4*x4**3*x5*x6**3+4*x2**3*x3*x6**3+x4**3*x5**4+x2**3*&
&x3**4
dpdx6(491) = 4*x1**3*x4*x6**3+4*x1**3*x2*x6**3
dpdx6(492) = 4*x1*x4**3*x6**3+4*x1*x2**3*x6**3
dpdx6(493) = 4*x2*x4**3*x6**3+4*x2**3*x4*x6**3
dpdx6(494) = 3*x1**3*x4**2*x6**2+3*x1**3*x2**2*x6**2
dpdx6(495) = 3*x1**2*x4**3*x6**2+3*x1**2*x2**3*x6**2
dpdx6(496) = 3*x2**2*x4**3*x6**2+3*x2**3*x4**2*x6**2
dpdx6(497) = 2*x1**3*x4**3*x6+2*x1**3*x2**3*x6
dpdx6(498) = 0
dpdx6(499) = 2*x2**3*x4**3*x6
dpdx6(500) = 3*x1**4*x5*x6**2+3*x1**4*x3*x6**2+x2**4*x5**3+x3**3*&
&x4**4
dpdx6(501) = 2*x2**4*x5**2*x6+2*x1**4*x5**2*x6+2*x3**2*x4**4*x6+2&
&*x1**4*x3**2*x6
dpdx6(502) = 2*x4**4*x5**2*x6+2*x2**4*x3**2*x6
dpdx6(503) = 3*x2**4*x5*x6**2+3*x3*x4**4*x6**2+x1**4*x5**3+x1**4*&
&x3**3
dpdx6(504) = 3*x4**4*x5*x6**2+3*x2**4*x3*x6**2+x4**4*x5**3+x2**4*&
&x3**3
dpdx6(505) = 3*x1**4*x4*x6**2+3*x1**4*x2*x6**2
dpdx6(506) = 3*x1*x4**4*x6**2+3*x1*x2**4*x6**2
dpdx6(507) = 3*x2*x4**4*x6**2+3*x2**4*x4*x6**2
dpdx6(508) = 2*x1**4*x4**2*x6+2*x1**4*x2**2*x6
dpdx6(509) = 2*x1**2*x4**4*x6+2*x1**2*x2**4*x6
dpdx6(510) = 0
dpdx6(511) = 2*x2**2*x4**4*x6+2*x2**4*x4**2*x6
dpdx6(512) = x1**4*x4**3+x1**4*x2**3
dpdx6(513) = x1**3*x4**4+x1**3*x2**4
dpdx6(514) = 0
dpdx6(515) = x2**3*x4**4+x2**4*x4**3
dpdx6(516) = 2*x1**5*x5*x6+2*x1**5*x3*x6+x2**5*x5**2+x3**2*x4**5
dpdx6(517) = 2*x2**5*x5*x6+2*x3*x4**5*x6+x1**5*x5**2+x1**5*x3**2
dpdx6(518) = 2*x4**5*x5*x6+2*x2**5*x3*x6+x4**5*x5**2+x2**5*x3**2
dpdx6(519) = 2*x1**5*x4*x6+2*x1**5*x2*x6
dpdx6(520) = 2*x1*x4**5*x6+2*x1*x2**5*x6
dpdx6(521) = 2*x2*x4**5*x6+2*x2**5*x4*x6
dpdx6(522) = x1**5*x4**2+x1**5*x2**2
dpdx6(523) = x1**2*x4**5+x1**2*x2**5
dpdx6(524) = 0
dpdx6(525) = x2**2*x4**5+x2**5*x4**2
dpdx6(526) = x2**6*x5+x1**6*x5+x3*x4**6+x1**6*x3
dpdx6(527) = x4**6*x5+x2**6*x3
dpdx6(528) = x1**6*x4+x1**6*x2
dpdx6(529) = x1*x4**6+x1*x2**6
dpdx6(530) = 0
dpdx6(531) = x2*x4**6+x2**6*x4
dpdx6(532) = 4*x5**4*x6**3+4*x3**4*x6**3
dpdx6(533) = 5*x5**3*x6**4+5*x3**3*x6**4+3*x5**5*x6**2+3*x3**5*x6&
&**2
dpdx6(534) = 6*x5**2*x6**5+6*x3**2*x6**5+2*x5**6*x6+2*x3**6*x6
dpdx6(535) = 7*x5*x6**6+7*x3*x6**6+x5**7+x3**7
dpdx6(536) = 7*x4*x6**6+7*x2*x6**6
dpdx6(537) = 6*x4**2*x6**5+6*x2**2*x6**5
dpdx6(538) = 5*x4**3*x6**4+5*x2**3*x6**4
dpdx6(539) = 4*x4**4*x6**3+4*x2**4*x6**3
dpdx6(540) = 0
dpdx6(541) = 3*x4**5*x6**2+3*x2**5*x6**2
dpdx6(542) = 0
dpdx6(543) = 2*x4**6*x6+2*x2**6*x6
dpdx6(544) = 0
dpdx6(545) = x4**7+x2**7
dpdx6(546) = 0

end subroutine grad_ev_pol_con_8
!===========================================================================
! *def u12_order10
! subtoutine of computing u12
!===========================================================================
subroutine u12_order10
implicit none
real*8 :: xyz(4,3)
real*8 :: v, r12, r13, r14, r23, r24, r34
real*8 :: s
real*8 :: a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer, parameter :: ncoeff=1300
real*8 :: coeff(ncoeff), q(ncoeff)
real*8 :: x(6)
integer ::  ncall
data ncall/1/
save ncall, coeff, a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer :: i, j
real*8 :: rrel(3,3),cross(3)
real*8 :: volume

if(ncall.eq.1) then
a1=0.610
a2=0.800
b1=1.300
b2=1.300
ra1=0.802
ra2=0.702
rb1=0.802
rb2=0.702
coeff(1)=-0.145119363753D+04
coeff(2)=0.130004701981D+05
coeff(3)=0.531779879233D+04
coeff(4)=0.105986030196D+05
coeff(5)=0.362169613171D+03
coeff(6)=-0.107491480704D+04
coeff(7)=0.780510828777D+03
coeff(8)=-0.236060288363D+05
coeff(9)=-0.433662039748D+05
coeff(10)=-0.214220336456D+05
coeff(11)=0.122746448315D+05
coeff(12)=-0.177342221377D+05
coeff(13)=-0.488724609595D+04
coeff(14)=-0.216957399632D+05
coeff(15)=-0.474323012830D+04
coeff(16)=0.161137897848D+05
coeff(17)=0.243201785821D+05
coeff(18)=-0.639199811760D+05
coeff(19)=-0.792729644532D+05
coeff(20)=0.583987774463D+05
coeff(21)=0.417154311098D+04
coeff(22)=0.298709598914D+04
coeff(23)=0.211227737871D+04
coeff(24)=0.866569330761D+05
coeff(25)=-0.681255618420D+04
coeff(26)=-0.131104659383D+05
coeff(27)=0.353331873694D+05
coeff(28)=-0.317860023242D+03
coeff(29)=0.367046700357D+05
coeff(30)=0.279489069856D+05
coeff(31)=-0.322879696744D+05
coeff(32)=0.321752596981D+05
coeff(33)=0.869536541706D+05
coeff(34)=0.419223827847D+05
coeff(35)=-0.106728708986D+04
coeff(36)=-0.174670463552D+06
coeff(37)=0.102664683418D+06
coeff(38)=-0.432313300486D+03
coeff(39)=0.416213307041D+05
coeff(40)=0.845509346017D+04
coeff(41)=0.288099082715D+05
coeff(42)=0.239512859270D+05
coeff(43)=0.121332392893D+05
coeff(44)=-0.178300146292D+05
coeff(45)=-0.385303576098D+05
coeff(46)=-0.371724484123D+05
coeff(47)=0.353112572228D+05
coeff(48)=0.329806443491D+05
coeff(49)=0.688008918405D+05
coeff(50)=-0.924470232090D+05
coeff(51)=-0.201361005640D+05
coeff(52)=-0.106899897494D+06
coeff(53)=-0.109978328918D+06
coeff(54)=0.129408354518D+06
coeff(55)=0.278313370319D+06
coeff(56)=-0.189224442609D+06
coeff(57)=0.936028806331D+05
coeff(58)=-0.123793186128D+05
coeff(59)=0.339876130560D+03
coeff(60)=-0.848135236758D+05
coeff(61)=-0.758438013846D+05
coeff(62)=0.104005091266D+06
coeff(63)=0.418729082258D+05
coeff(64)=-0.681563458173D+05
coeff(65)=-0.455771128879D+05
coeff(66)=0.211009387655D+06
coeff(67)=0.611623009616D+05
coeff(68)=-0.167686134252D+05
coeff(69)=0.681913247406D+05
coeff(70)=-0.727299094665D+05
coeff(71)=0.373768444377D+05
coeff(72)=0.966908065169D+05
coeff(73)=-0.594435023770D+05
coeff(74)=-0.820968840152D+05
coeff(75)=0.436514535336D+04
coeff(76)=0.557538909853D+05
coeff(77)=0.871720778990D+05
coeff(78)=-0.108338179331D+06
coeff(79)=-0.757529546811D+04
coeff(80)=-0.130480634478D+06
coeff(81)=0.132665598228D+05
coeff(82)=-0.158426809056D+05
coeff(83)=-0.302622257411D+05
coeff(84)=-0.529638176958D+05
coeff(85)=0.450804491848D+05
coeff(86)=-0.778229226901D+05
coeff(87)=-0.812093444870D+05
coeff(88)=0.367540005663D+05
coeff(89)=-0.997702201187D+05
coeff(90)=-0.617797441863D+05
coeff(91)=-0.341548774138D+05
coeff(92)=0.144259351105D+05
coeff(93)=-0.984183783441D+05
coeff(94)=-0.290418354362D+05
coeff(95)=0.393943399545D+06
coeff(96)=-0.186404648221D+06
coeff(97)=-0.448378304289D+05
coeff(98)=0.227151633554D+05
coeff(99)=-0.101181674935D+06
coeff(100)=-0.135522143029D+05
coeff(101)=-0.463018380645D+05
coeff(102)=0.200917103449D+05
coeff(103)=-0.368025308254D+05
coeff(104)=-0.187934717221D+04
coeff(105)=0.383449919883D+05
coeff(106)=0.254135460480D+05
coeff(107)=-0.255386853412D+05
coeff(108)=-0.201287450771D+04
coeff(109)=0.305960879240D+05
coeff(110)=-0.172558087250D+05
coeff(111)=0.266270290599D+04
coeff(112)=0.476042597640D+05
coeff(113)=0.477159582988D+05
coeff(114)=0.997695938348D+05
coeff(115)=0.228351057777D+05
coeff(116)=0.130078969073D+06
coeff(117)=-0.285907747776D+05
coeff(118)=-0.341039972202D+05
coeff(119)=-0.283208753007D+06
coeff(120)=0.161009012031D+06
coeff(121)=-0.256482379108D+06
coeff(122)=0.354289521449D+06
coeff(123)=0.122211187862D+06
coeff(124)=0.129118528795D+06
coeff(125)=-0.455756258795D+05
coeff(126)=-0.309409522175D+06
coeff(127)=0.157114532296D+06
coeff(128)=-0.247591271554D+06
coeff(129)=0.217930294007D+05
coeff(130)=0.103325087473D+05
coeff(131)=-0.388538977504D+04
coeff(132)=-0.297428885686D+05
coeff(133)=-0.207633554257D+05
coeff(134)=-0.196154397756D+05
coeff(135)=-0.298058906183D+05
coeff(136)=-0.281135137078D+05
coeff(137)=-0.214858521791D+05
coeff(138)=0.540020194002D+05
coeff(139)=-0.921839462361D+05
coeff(140)=0.130585966459D+06
coeff(141)=-0.603433372873D+05
coeff(142)=-0.216863879996D+05
coeff(143)=-0.264408317364D+05
coeff(144)=0.868170481859D+05
coeff(145)=-0.401533046565D+05
coeff(146)=-0.755000266240D+05
coeff(147)=-0.768572769152D+04
coeff(148)=-0.716106691270D+05
coeff(149)=0.734708558274D+05
coeff(150)=-0.919912283760D+05
coeff(151)=-0.268054929909D+05
coeff(152)=0.829364604204D+04
coeff(153)=0.125270985035D+05
coeff(154)=0.241694707818D+05
coeff(155)=-0.123231782202D+05
coeff(156)=-0.364757645947D+05
coeff(157)=0.219184743203D+05
coeff(158)=-0.573265786677D+05
coeff(159)=-0.244573577864D+05
coeff(160)=0.257842811111D+05
coeff(161)=0.384740030593D+05
coeff(162)=-0.101841329664D+05
coeff(163)=-0.258370145291D+05
coeff(164)=-0.430466110869D+05
coeff(165)=-0.461437384994D+05
coeff(166)=-0.511834954253D+05
coeff(167)=-0.110621433975D+05
coeff(168)=0.100434508682D+06
coeff(169)=-0.969866732992D+05
coeff(170)=0.330689626559D+05
coeff(171)=0.578404683230D+05
coeff(172)=0.885887893839D+04
coeff(173)=0.247739723969D+05
coeff(174)=0.329737294954D+05
coeff(175)=0.157373087846D+05
coeff(176)=0.762490469519D+05
coeff(177)=-0.413398769870D+05
coeff(178)=0.141809736619D+05
coeff(179)=0.175435122549D+05
coeff(180)=-0.880702670510D+04
coeff(181)=-0.332403578257D+05
coeff(182)=-0.165052917098D+05
coeff(183)=0.105933975017D+06
coeff(184)=-0.224598460416D+05
coeff(185)=-0.860061904764D+05
coeff(186)=-0.333083653458D+05
coeff(187)=0.124835965345D+06
coeff(188)=-0.752618738531D+04
coeff(189)=0.779133072514D+05
coeff(190)=-0.119877956657D+05
coeff(191)=0.114956587718D+05
coeff(192)=-0.621052976484D+05
coeff(193)=0.116376492673D+06
coeff(194)=-0.102173635949D+05
coeff(195)=0.122557541579D+06
coeff(196)=-0.586795241058D+05
coeff(197)=0.159305933523D+06
coeff(198)=-0.586770401650D+05
coeff(199)=-0.532565570700D+05
coeff(200)=0.382905924978D+06
coeff(201)=0.820763080485D+05
coeff(202)=-0.840777178290D+05
coeff(203)=-0.490608160036D+05
coeff(204)=0.697252367627D+05
coeff(205)=-0.313141677656D+05
coeff(206)=-0.241752965235D+06
coeff(207)=-0.725003725779D+05
coeff(208)=-0.136211757321D+05
coeff(209)=-0.311965755621D+05
coeff(210)=0.329898891363D+05
coeff(211)=0.566955628471D+05
coeff(212)=0.305802206818D+05
coeff(213)=-0.119256175814D+05
coeff(214)=0.168893840604D+05
coeff(215)=0.261997085677D+05
coeff(216)=-0.115223553524D+05
coeff(217)=0.519914169164D+05
coeff(218)=-0.968482005677D+04
coeff(219)=-0.335549915600D+05
coeff(220)=-0.317524707140D+05
coeff(221)=-0.298058715624D+05
coeff(222)=0.109769579002D+06
coeff(223)=-0.829243714078D+05
coeff(224)=-0.182497139202D+05
coeff(225)=0.203794861869D+05
coeff(226)=0.781753121295D+05
coeff(227)=0.771489615211D+05
coeff(228)=0.161752660400D+05
coeff(229)=-0.111062318160D+06
coeff(230)=0.716278152372D+05
coeff(231)=-0.971470180872D+05
coeff(232)=-0.378924359358D+05
coeff(233)=-0.636103076382D+04
coeff(234)=-0.870213392028D+05
coeff(235)=-0.949098032808D+05
coeff(236)=-0.206769170424D+05
coeff(237)=0.149258008647D+06
coeff(238)=-0.235183440754D+06
coeff(239)=0.964555127128D+05
coeff(240)=-0.135457569033D+06
coeff(241)=-0.459840418940D+06
coeff(242)=-0.908166446305D+05
coeff(243)=0.779857744308D+05
coeff(244)=0.246152921536D+04
coeff(245)=0.199028061377D+06
coeff(246)=-0.224988736842D+04
coeff(247)=0.204581234124D+06
coeff(248)=-0.195284533002D+06
coeff(249)=0.154110044095D+06
coeff(250)=-0.308693484630D+05
coeff(251)=-0.704501450094D+05
coeff(252)=0.624735002618D+05
coeff(253)=-0.149446371678D+03
coeff(254)=0.923976618764D+05
coeff(255)=-0.458307360816D+05
coeff(256)=0.333163447741D+05
coeff(257)=0.789473841701D+02
coeff(258)=-0.222649838129D+05
coeff(259)=0.217354568085D+04
coeff(260)=0.409077530451D+05
coeff(261)=0.361098903901D+05
coeff(262)=0.528491133714D+05
coeff(263)=0.797610873579D+04
coeff(264)=-0.405900388300D+04
coeff(265)=0.117377007230D+06
coeff(266)=0.443932137543D+05
coeff(267)=-0.162076771352D+05
coeff(268)=0.585508981024D+02
coeff(269)=0.204387069755D+05
coeff(270)=0.211783378456D+05
coeff(271)=0.246506190547D+05
coeff(272)=-0.453294970723D+05
coeff(273)=0.106680995040D+06
coeff(274)=-0.582272142660D+05
coeff(275)=0.123711624251D+05
coeff(276)=0.464203877453D+05
coeff(277)=-0.401140520394D+05
coeff(278)=-0.286382682789D+05
coeff(279)=-0.409319545964D+05
coeff(280)=0.360982670139D+04
coeff(281)=0.896766912202D+05
coeff(282)=-0.450584647327D+05
coeff(283)=-0.312992668269D+05
coeff(284)=-0.222514090256D+04
coeff(285)=0.377516843357D+04
coeff(286)=-0.200384669582D+06
coeff(287)=-0.226190808950D+05
coeff(288)=0.119820075705D+04
coeff(289)=0.948617465528D+04
coeff(290)=0.887146151424D+05
coeff(291)=-0.985715566141D+05
coeff(292)=0.183811647121D+06
coeff(293)=-0.121585196138D+06
coeff(294)=0.441977109126D+05
coeff(295)=0.658163965219D+05
coeff(296)=0.128138084231D+06
coeff(297)=0.878894170210D+05
coeff(298)=-0.103641368854D+06
coeff(299)=-0.946836719904D+04
coeff(300)=-0.287870205403D+05
coeff(301)=-0.693766844494D+05
coeff(302)=-0.502476881260D+05
coeff(303)=-0.469272727271D+05
coeff(304)=0.165547642676D+05
coeff(305)=0.198603362742D+05
coeff(306)=-0.185267513228D+05
coeff(307)=-0.309015560994D+04
coeff(308)=0.217325322905D+05
coeff(309)=0.483425418490D+05
coeff(310)=0.257636631691D+05
coeff(311)=-0.406231642516D+05
coeff(312)=0.254354735524D+05
coeff(313)=-0.419050634389D+05
coeff(314)=-0.432216874752D+05
coeff(315)=0.866824557796D+05
coeff(316)=0.380558256218D+05
coeff(317)=0.445231346719D+05
coeff(318)=-0.665310135090D+04
coeff(319)=0.240501730545D+05
coeff(320)=-0.374407629140D+05
coeff(321)=-0.226070023676D+05
coeff(322)=0.371062853243D+05
coeff(323)=-0.163831846127D+05
coeff(324)=-0.304114216221D+05
coeff(325)=0.678682147819D+05
coeff(326)=-0.383713159660D+05
coeff(327)=0.665675969775D+05
coeff(328)=-0.983657709598D+05
coeff(329)=0.171682474162D+05
coeff(330)=-0.119395841716D+04
coeff(331)=0.122616364681D+05
coeff(332)=-0.615192096074D+04
coeff(333)=0.519572991754D+05
coeff(334)=0.208650868110D+05
coeff(335)=-0.170129821868D+05
coeff(336)=0.645989464270D+04
coeff(337)=0.231953805459D+05
coeff(338)=0.197111221139D+05
coeff(339)=-0.624370026983D+05
coeff(340)=0.507337885995D+05
coeff(341)=0.579486872465D+05
coeff(342)=-0.352394947761D+05
coeff(343)=0.663521365581D+05
coeff(344)=-0.495848834007D+04
coeff(345)=0.189493806892D+05
coeff(346)=-0.898997205342D+04
coeff(347)=0.694542525882D+05
coeff(348)=-0.290416294182D+05
coeff(349)=0.435747154319D+04
coeff(350)=0.218602102454D+05
coeff(351)=0.629086528607D+05
coeff(352)=-0.226651970600D+05
coeff(353)=0.689778551582D+04
coeff(354)=-0.172985783837D+05
coeff(355)=0.526191428372D+05
coeff(356)=0.470516379883D+05
coeff(357)=0.314410427064D+05
coeff(358)=-0.258184283647D+05
coeff(359)=-0.459805234755D+05
coeff(360)=0.501555216204D+05
coeff(361)=-0.121766555862D+06
coeff(362)=0.130781933491D+05
coeff(363)=-0.193471943901D+05
coeff(364)=0.665212863845D+05
coeff(365)=-0.609100517649D+05
coeff(366)=0.505048646007D+05
coeff(367)=0.787106282364D+04
coeff(368)=-0.169519155133D+05
coeff(369)=-0.622774879623D+05
coeff(370)=-0.236267792639D+05
coeff(371)=0.446448086210D+05
coeff(372)=0.161918795164D+06
coeff(373)=-0.178145335326D+06
coeff(374)=-0.596637453263D+05
coeff(375)=-0.447926457514D+05
coeff(376)=-0.141738091029D+06
coeff(377)=0.198137947759D+06
coeff(378)=-0.427488987787D+05
coeff(379)=0.262644206664D+05
coeff(380)=0.189122236546D+04
coeff(381)=0.375381802310D+05
coeff(382)=-0.575129319493D+05
coeff(383)=-0.101279276820D+05
coeff(384)=0.819159957624D+05
coeff(385)=0.601529765993D+05
coeff(386)=-0.378344655944D+05
coeff(387)=0.138320323525D+05
coeff(388)=0.546867895582D+05
coeff(389)=0.329651701694D+04
coeff(390)=-0.442005553302D+04
coeff(391)=-0.342218746016D+05
coeff(392)=0.108870964452D+06
coeff(393)=-0.614817927771D+05
coeff(394)=0.123795654871D+05
coeff(395)=-0.317640998991D+05
coeff(396)=0.367041703444D+05
coeff(397)=0.261581477452D+05
coeff(398)=-0.111677825144D+06
coeff(399)=0.102047205252D+06
coeff(400)=0.293022321341D+05
coeff(401)=-0.668984254612D+04
coeff(402)=-0.909425159311D+03
coeff(403)=-0.523918039489D+05
coeff(404)=0.322255518032D+05
coeff(405)=-0.296479724182D+05
coeff(406)=-0.160691855684D+04
coeff(407)=0.399475060861D+04
coeff(408)=0.633238590299D+05
coeff(409)=-0.171046011977D+05
coeff(410)=-0.798701001813D+05
coeff(411)=-0.580704745273D+05
coeff(412)=0.326758555791D+05
coeff(413)=0.130943008236D+05
coeff(414)=0.524065897069D+05
coeff(415)=0.797687625542D+05
coeff(416)=0.281301757671D+05
coeff(417)=-0.253877237954D+05
coeff(418)=-0.359453151088D+03
coeff(419)=0.575013158140D+05
coeff(420)=-0.681309050357D+05
coeff(421)=-0.610688627092D+05
coeff(422)=-0.722269011305D+05
coeff(423)=-0.169810903002D+06
coeff(424)=-0.174810439988D+05
coeff(425)=-0.902836070899D+04
coeff(426)=-0.650673209847D+05
coeff(427)=-0.619403610897D+05
coeff(428)=0.494951394212D+05
coeff(429)=0.448878172679D+05
coeff(430)=-0.178253462552D+05
coeff(431)=-0.256486747692D+05
coeff(432)=0.178659556317D+05
coeff(433)=-0.790343799026D+04
coeff(434)=0.442809404830D+05
coeff(435)=-0.146532861560D+06
coeff(436)=0.225445014291D+05
coeff(437)=-0.182731890958D+05
coeff(438)=-0.646397943288D+05
coeff(439)=0.213868227022D+04
coeff(440)=-0.330805107928D+05
coeff(441)=0.675443684816D+05
coeff(442)=0.170406523019D+05
coeff(443)=0.223401222992D+06
coeff(444)=0.132569665724D+05
coeff(445)=0.462314922677D+05
coeff(446)=-0.105154925035D+06
coeff(447)=-0.758643445704D+05
coeff(448)=0.941445308255D+05
coeff(449)=-0.872441344685D+05
coeff(450)=-0.856605248637D+04
coeff(451)=-0.559346908276D+05
coeff(452)=-0.725205085363D+05
coeff(453)=0.424742281870D+04
coeff(454)=0.945446767889D+05
coeff(455)=-0.178999029653D+06
coeff(456)=0.113325979728D+06
coeff(457)=-0.125428487170D+06
coeff(458)=-0.488743026234D+05
coeff(459)=-0.422137182217D+05
coeff(460)=0.291815663743D+05
coeff(461)=0.119661849475D+06
coeff(462)=0.156431200002D+05
coeff(463)=0.174198286155D+06
coeff(464)=0.298687612882D+05
coeff(465)=0.368901792575D+05
coeff(466)=-0.493418809132D+05
coeff(467)=0.218345408642D+04
coeff(468)=-0.730796838070D+03
coeff(469)=-0.990059809504D+04
coeff(470)=0.195637365909D+05
coeff(471)=-0.498378218300D+05
coeff(472)=-0.248580585109D+05
coeff(473)=0.926650612448D+04
coeff(474)=0.508255461341D+05
coeff(475)=0.583228731704D+05
coeff(476)=-0.129425080252D+05
coeff(477)=0.274022213505D+05
coeff(478)=0.480510215598D+05
coeff(479)=-0.211601260276D+05
coeff(480)=0.545168786686D+05
coeff(481)=-0.198461295709D+04
coeff(482)=-0.404707525915D+04
coeff(483)=-0.380599679924D+05
coeff(484)=-0.732721980100D+04
coeff(485)=0.287613745037D+05
coeff(486)=-0.762770495466D+05
coeff(487)=-0.398352156190D+04
coeff(488)=-0.305681527099D+05
coeff(489)=0.120837987822D+05
coeff(490)=-0.370978067049D+03
coeff(491)=-0.840517516300D+03
coeff(492)=0.353163180926D+05
coeff(493)=-0.111950401859D+05
coeff(494)=-0.157585500727D+05
coeff(495)=0.139211088406D+05
coeff(496)=0.910544810345D+05
coeff(497)=0.353814577596D+05
coeff(498)=-0.463831030867D+04
coeff(499)=0.399663253515D+05
coeff(500)=0.286405584581D+04
coeff(501)=0.343394543616D+04
coeff(502)=0.441370700175D+05
coeff(503)=-0.961351505166D+04
coeff(504)=0.997517646402D+05
coeff(505)=-0.618946254701D+05
coeff(506)=-0.465979205805D+05
coeff(507)=-0.168160075564D+05
coeff(508)=0.513004715840D+05
coeff(509)=-0.399738518724D+05
coeff(510)=0.155173671063D+05
coeff(511)=-0.845620240831D+05
coeff(512)=-0.696536735026D+04
coeff(513)=-0.322851265687D+05
coeff(514)=0.136724733402D+05
coeff(515)=-0.869464038903D+05
coeff(516)=-0.562568294541D+04
coeff(517)=-0.106706653005D+05
coeff(518)=0.366837266153D+05
coeff(519)=0.596629669880D+04
coeff(520)=-0.616112631376D+05
coeff(521)=-0.180204029614D+05
coeff(522)=-0.182880186003D+05
coeff(523)=-0.677978114280D+05
coeff(524)=-0.242662098864D+05
coeff(525)=-0.976612228686D+05
coeff(526)=0.508822931839D+05
coeff(527)=-0.509249306252D+05
coeff(528)=0.276260334680D+05
coeff(529)=0.170027332092D+05
coeff(530)=0.385024794420D+05
coeff(531)=-0.585100012364D+04
coeff(532)=0.412204772446D+05
coeff(533)=0.106792332262D+06
coeff(534)=0.109646838720D+06
coeff(535)=-0.403332939451D+05
coeff(536)=-0.479845160993D+05
coeff(537)=-0.216878931285D+05
coeff(538)=-0.371537344725D+05
coeff(539)=-0.310652910833D+04
coeff(540)=0.107312648066D+06
coeff(541)=0.168308125674D+05
coeff(542)=0.407915978080D+05
coeff(543)=0.640611309053D+05
coeff(544)=0.264970574970D+05
coeff(545)=-0.483334911731D+05
coeff(546)=-0.655848586392D+04
coeff(547)=-0.963161588829D+05
coeff(548)=0.135148585514D+06
coeff(549)=-0.684691055317D+05
coeff(550)=0.285771181076D+05
coeff(551)=0.197262290012D+05
coeff(552)=0.505340319326D+05
coeff(553)=0.775166124074D+05
coeff(554)=0.335096779243D+05
coeff(555)=-0.443082530570D+05
coeff(556)=0.445573827973D+05
coeff(557)=-0.428789573479D+05
coeff(558)=0.786600027106D+04
coeff(559)=0.425889195663D+05
coeff(560)=-0.151725865297D+05
coeff(561)=0.136164400850D+05
coeff(562)=-0.394763019519D+05
coeff(563)=0.454623169466D+05
coeff(564)=0.315345594716D+05
coeff(565)=-0.121027355927D+05
coeff(566)=-0.178635602836D+05
coeff(567)=-0.196190023284D+05
coeff(568)=-0.694867391030D+04
coeff(569)=-0.393910448578D+05
coeff(570)=0.501652986172D+05
coeff(571)=-0.113557331053D+05
coeff(572)=0.129017622225D+05
coeff(573)=0.618126618130D+05
coeff(574)=-0.906969339481D+05
coeff(575)=-0.498539126801D+05
coeff(576)=0.121574664973D+05
coeff(577)=-0.189134088698D+05
coeff(578)=0.593025332560D+04
coeff(579)=-0.370964471617D+05
coeff(580)=0.191516502110D+05
coeff(581)=0.504409061502D+05
coeff(582)=0.273979288792D+05
coeff(583)=-0.406327394123D+05
coeff(584)=0.435321381239D+05
coeff(585)=0.512268876457D+05
coeff(586)=-0.584920595593D+05
coeff(587)=-0.112156229035D+05
coeff(588)=-0.301299902087D+05
coeff(589)=0.437114976597D+05
coeff(590)=0.161910961790D+05
coeff(591)=-0.475257984177D+05
coeff(592)=0.571421023776D+05
coeff(593)=-0.728916940090D+04
coeff(594)=-0.457146923322D+05
coeff(595)=0.157985593971D+04
coeff(596)=-0.537599251626D+04
coeff(597)=-0.337951104944D+05
coeff(598)=-0.762661783827D+05
coeff(599)=-0.748739718976D+04
coeff(600)=-0.346674440137D+04
coeff(601)=-0.889565159429D+05
coeff(602)=-0.904988645102D+04
coeff(603)=0.479243863887D+05
coeff(604)=0.628875402606D+05
coeff(605)=0.787433073877D+05
coeff(606)=0.257005758508D+05
coeff(607)=-0.632649328273D+04
coeff(608)=-0.211829266422D+05
coeff(609)=-0.226204138190D+05
coeff(610)=0.542986688244D+05
coeff(611)=-0.847373513070D+04
coeff(612)=-0.635005036559D+05
coeff(613)=0.244714008154D+05
coeff(614)=-0.241218800210D+05
coeff(615)=0.762423647481D+05
coeff(616)=0.107918576702D+06
coeff(617)=-0.257099002372D+05
coeff(618)=-0.617230787515D+05
coeff(619)=-0.900719988051D+05
coeff(620)=-0.536487369083D+05
coeff(621)=0.123774934656D+06
coeff(622)=-0.560032833894D+04
coeff(623)=-0.499555020878D+05
coeff(624)=-0.138533384853D+06
coeff(625)=0.881571593486D+05
coeff(626)=-0.418938845308D+05
coeff(627)=-0.696435035878D+05
coeff(628)=-0.370556642778D+05
coeff(629)=0.353737928775D+05
coeff(630)=-0.281693543992D+05
coeff(631)=-0.583636513501D+05
coeff(632)=0.655450066242D+05
coeff(633)=-0.234878950082D+05
coeff(634)=-0.113408316689D+05
coeff(635)=0.998298145878D+04
coeff(636)=0.271308764369D+05
coeff(637)=0.387268635507D+05
coeff(638)=-0.434328574695D+04
coeff(639)=0.288123074993D+05
coeff(640)=0.497729379589D+04
coeff(641)=0.173248654270D+05
coeff(642)=-0.328305820465D+05
coeff(643)=-0.703607591027D+05
coeff(644)=0.801725810474D+05
coeff(645)=-0.167999739942D+06
coeff(646)=0.136559397119D+05
coeff(647)=-0.123546451580D+05
coeff(648)=-0.154844716932D+05
coeff(649)=-0.508533687326D+05
coeff(650)=0.385810776403D+04
coeff(651)=-0.441587445404D+05
coeff(652)=-0.168648664356D+06
coeff(653)=0.134638961020D+05
coeff(654)=0.814968358617D+04
coeff(655)=-0.497030159844D+05
coeff(656)=0.129712977729D+05
coeff(657)=-0.398495374201D+05
coeff(658)=-0.632209295399D+04
coeff(659)=-0.680119769820D+05
coeff(660)=-0.444011269834D+05
coeff(661)=-0.552967451521D+05
coeff(662)=0.848134519061D+04
coeff(663)=-0.319550199337D+05
coeff(664)=0.810981621005D+05
coeff(665)=0.482905293722D+05
coeff(666)=-0.283113455316D+05
coeff(667)=-0.894063156494D+04
coeff(668)=-0.341042932225D+05
coeff(669)=-0.751261314400D+04
coeff(670)=0.145358558080D+05
coeff(671)=0.178775621227D+05
coeff(672)=0.690515570133D+05
coeff(673)=-0.922213809480D+05
coeff(674)=0.649953612643D+05
coeff(675)=-0.342993988890D+04
coeff(676)=-0.314461511944D+05
coeff(677)=0.614693268992D+05
coeff(678)=-0.750136979671D+05
coeff(679)=-0.129621081091D+06
coeff(680)=0.241481272927D+06
coeff(681)=-0.170353235748D+06
coeff(682)=-0.175924995041D+05
coeff(683)=-0.124960408704D+06
coeff(684)=-0.235302077697D+05
coeff(685)=-0.651590421300D+05
coeff(686)=-0.435452633050D+05
coeff(687)=0.989980760302D+05
coeff(688)=0.757252442890D+05
coeff(689)=-0.247915894257D+05
coeff(690)=0.591962982647D+05
coeff(691)=-0.894646691214D+05
coeff(692)=-0.108621809056D+05
coeff(693)=0.911789761888D+04
coeff(694)=0.280145193480D+05
coeff(695)=-0.519079148497D+05
coeff(696)=0.985511210377D+05
coeff(697)=-0.664903108655D+05
coeff(698)=0.559621835800D+05
coeff(699)=-0.133158322805D+06
coeff(700)=0.220574461734D+05
coeff(701)=0.471869655662D+05
coeff(702)=-0.918927547743D+05
coeff(703)=-0.530346014206D+05
coeff(704)=0.105014634695D+06
coeff(705)=0.679537183626D+05
coeff(706)=-0.451796353605D+05
coeff(707)=0.178934526284D+05
coeff(708)=0.195423705366D+06
coeff(709)=0.811130850671D+05
coeff(710)=0.388875131331D+04
coeff(711)=-0.152022519052D+05
coeff(712)=0.687551807433D+04
coeff(713)=0.349774665440D+05
coeff(714)=0.590702499017D+04
coeff(715)=0.129882845590D+05
coeff(716)=-0.307831465640D+05
coeff(717)=0.127768908928D+05
coeff(718)=0.197403948387D+05
coeff(719)=-0.100389294783D+04
coeff(720)=-0.409409238752D+05
coeff(721)=0.316118995087D+05
coeff(722)=-0.295264001704D+05
coeff(723)=-0.118676326526D+05
coeff(724)=0.696885670084D+05
coeff(725)=0.473433620507D+05
coeff(726)=-0.238220107074D+05
coeff(727)=-0.267775581552D+04
coeff(728)=-0.335228905500D+05
coeff(729)=-0.478983036948D+05
coeff(730)=-0.348732499543D+05
coeff(731)=-0.362847146401D+05
coeff(732)=0.739874982275D+04
coeff(733)=0.373705611566D+05
coeff(734)=0.116999096247D+05
coeff(735)=0.130869120673D+05
coeff(736)=0.322484587158D+05
coeff(737)=0.251523372560D+05
coeff(738)=0.829363145318D+05
coeff(739)=0.217481282419D+04
coeff(740)=0.327662087009D+05
coeff(741)=0.135661783542D+05
coeff(742)=-0.427229701283D+05
coeff(743)=0.504803132024D+05
coeff(744)=0.732193685857D+05
coeff(745)=-0.148916986863D+05
coeff(746)=0.947475690523D+04
coeff(747)=0.671688942577D+05
coeff(748)=-0.582065374430D+04
coeff(749)=0.618177897893D+05
coeff(750)=-0.292931413420D+05
coeff(751)=0.244359621287D+05
coeff(752)=0.134706922369D+06
coeff(753)=0.211344979514D+06
coeff(754)=0.607941215573D+05
coeff(755)=0.309997883629D+04
coeff(756)=0.126443079075D+05
coeff(757)=0.324766332237D+05
coeff(758)=0.462997551939D+05
coeff(759)=-0.306594085760D+05
coeff(760)=0.656775661607D+05
coeff(761)=-0.498409530754D+05
coeff(762)=-0.108077060861D+05
coeff(763)=-0.167168144522D+06
coeff(764)=0.172777463473D+06
coeff(765)=-0.564823010415D+05
coeff(766)=0.197250231290D+06
coeff(767)=-0.964593640471D+04
coeff(768)=0.796979338506D+03
coeff(769)=0.278511583035D+06
coeff(770)=0.989347338808D+05
coeff(771)=0.148156864354D+06
coeff(772)=-0.610118756324D+05
coeff(773)=-0.112055095722D+06
coeff(774)=-0.442594854925D+05
coeff(775)=-0.248302492797D+06
coeff(776)=0.289271196664D+04
coeff(777)=-0.232149576829D+06
coeff(778)=0.143858253421D+06
coeff(779)=-0.185494101952D+06
coeff(780)=-0.115596414294D+06
coeff(781)=0.325619765936D+06
coeff(782)=0.196321904817D+06
coeff(783)=0.453433933511D+05
coeff(784)=-0.284619858737D+05
coeff(785)=0.781826760403D+05
coeff(786)=-0.115009194037D+06
coeff(787)=0.131506479185D+06
coeff(788)=-0.154785682641D+06
coeff(789)=0.508339221469D+05
coeff(790)=-0.181659529794D+06
coeff(791)=0.399145686993D+05
coeff(792)=0.418147313342D+05
coeff(793)=-0.971672742196D+05
coeff(794)=-0.780445151008D+05
coeff(795)=-0.150627212395D+06
coeff(796)=-0.157517689268D+05
coeff(797)=-0.548700975016D+04
coeff(798)=0.206726550473D+05
coeff(799)=-0.686608662670D+03
coeff(800)=0.344248497802D+03
coeff(801)=-0.455425578235D+04
coeff(802)=0.775632314076D+04
coeff(803)=-0.498993799351D+05
coeff(804)=-0.453554417271D+05
coeff(805)=-0.267478039896D+05
coeff(806)=-0.399733128374D+05
coeff(807)=0.101815819090D+05
coeff(808)=0.275709888669D+05
coeff(809)=0.913701201992D+05
coeff(810)=-0.236336538289D+05
coeff(811)=0.115754303498D+05
coeff(812)=-0.107433156249D+05
coeff(813)=-0.100917135895D+05
coeff(814)=0.152727139577D+05
coeff(815)=-0.319998880913D+05
coeff(816)=-0.257432467908D+05
coeff(817)=-0.176448596156D+05
coeff(818)=0.196328085023D+05
coeff(819)=0.598272958197D+05
coeff(820)=-0.117822163409D+05
coeff(821)=-0.748205233661D+05
coeff(822)=-0.828738599174D+04
coeff(823)=-0.485230713329D+05
coeff(824)=0.697564672088D+05
coeff(825)=-0.604548073942D+05
coeff(826)=-0.729938560264D+05
coeff(827)=0.668377932074D+04
coeff(828)=-0.116373076926D+05
coeff(829)=-0.157795143470D+05
coeff(830)=-0.259332558520D+04
coeff(831)=0.185560995770D+05
coeff(832)=0.596750327272D+05
coeff(833)=0.315478870563D+05
coeff(834)=-0.590760261364D+05
coeff(835)=0.912858535080D+04
coeff(836)=0.364158247469D+04
coeff(837)=0.223745593576D+05
coeff(838)=0.346825273973D+05
coeff(839)=-0.536937594152D+05
coeff(840)=0.575473791264D+04
coeff(841)=0.535644253730D+04
coeff(842)=0.991769106127D+04
coeff(843)=-0.312785365833D+05
coeff(844)=0.167521157693D+04
coeff(845)=-0.560453680975D+05
coeff(846)=0.669264452947D+05
coeff(847)=-0.659631591142D+05
coeff(848)=0.715752331602D+05
coeff(849)=-0.604035980006D+04
coeff(850)=0.754557424405D+04
coeff(851)=-0.393657373021D+05
coeff(852)=-0.392885815515D+05
coeff(853)=-0.656947003716D+05
coeff(854)=-0.259202330660D+05
coeff(855)=-0.429918442611D+05
coeff(856)=0.440565914566D+05
coeff(857)=0.425483250879D+05
coeff(858)=0.114951268306D+05
coeff(859)=0.410744596746D+05
coeff(860)=0.394690048857D+05
coeff(861)=0.190244925118D+05
coeff(862)=-0.105588981736D+06
coeff(863)=-0.560701468792D+05
coeff(864)=-0.140932530193D+05
coeff(865)=-0.753558109289D+05
coeff(866)=-0.206577732953D+05
coeff(867)=0.523559791237D+05
coeff(868)=-0.453135359541D+05
coeff(869)=0.287231962160D+05
coeff(870)=-0.344704668506D+04
coeff(871)=-0.502219201019D+05
coeff(872)=-0.712229347885D+05
coeff(873)=-0.638827590891D+05
coeff(874)=0.196588221376D+06
coeff(875)=0.600432739313D+05
coeff(876)=0.139672083479D+05
coeff(877)=-0.138877515046D+03
coeff(878)=0.623547298031D+05
coeff(879)=0.631722813094D+05
coeff(880)=0.254955538885D+05
coeff(881)=-0.569848054654D+05
coeff(882)=-0.219636039111D+05
coeff(883)=0.188195694480D+05
coeff(884)=0.692610727746D+05
coeff(885)=-0.454414724717D+05
coeff(886)=0.211493166173D+05
coeff(887)=0.540259057548D+05
coeff(888)=0.359494191614D+05
coeff(889)=-0.264072500131D+05
coeff(890)=-0.152792594314D+06
coeff(891)=-0.260126210169D+05
coeff(892)=0.505986404189D+05
coeff(893)=0.435282902504D+05
coeff(894)=0.993319588957D+05
coeff(895)=-0.292590782162D+05
coeff(896)=0.636560894416D+05
coeff(897)=-0.600798701957D+05
coeff(898)=-0.421740952414D+05
coeff(899)=0.483694221579D+05
coeff(900)=-0.137924639470D+05
coeff(901)=0.122377397976D+05
coeff(902)=-0.128395460460D+06
coeff(903)=0.838380398633D+05
coeff(904)=-0.464395648263D+05
coeff(905)=0.638846142085D+05
coeff(906)=0.116437051041D+06
coeff(907)=-0.401933469087D+04
coeff(908)=0.208322304958D+05
coeff(909)=-0.171526670076D+05
coeff(910)=-0.348637476618D+05
coeff(911)=0.418928383348D+05
coeff(912)=-0.427229410645D+05
coeff(913)=0.373389636209D+05
coeff(914)=0.121746981081D+04
coeff(915)=0.741497309740D+05
coeff(916)=0.138598246251D+05
coeff(917)=0.796885510635D+04
coeff(918)=-0.167561570518D+05
coeff(919)=-0.372195754517D+05
coeff(920)=-0.345160801031D+05
coeff(921)=0.221540610699D+05
coeff(922)=0.654700511693D+04
coeff(923)=0.633593243468D+04
coeff(924)=0.869932437601D+05
coeff(925)=0.363728973355D+05
coeff(926)=0.529866142648D+05
coeff(927)=-0.961554109145D+04
coeff(928)=0.913757620174D+04
coeff(929)=0.149849962895D+05
coeff(930)=0.416688600377D+05
coeff(931)=-0.103711985364D+06
coeff(932)=-0.196571802005D+05
coeff(933)=-0.897453802591D+05
coeff(934)=0.972338157533D+05
coeff(935)=-0.965597155859D+05
coeff(936)=-0.121575495355D+05
coeff(937)=-0.156690762580D+06
coeff(938)=0.635789607670D+05
coeff(939)=0.191616919997D+05
coeff(940)=-0.285883424512D+05
coeff(941)=-0.968193252966D+05
coeff(942)=0.774647959776D+05
coeff(943)=-0.576938925638D+05
coeff(944)=-0.594265603225D+05
coeff(945)=-0.755328562314D+05
coeff(946)=0.605953339220D+04
coeff(947)=0.201058810296D+05
coeff(948)=-0.256578415501D+05
coeff(949)=0.103262245233D+05
coeff(950)=-0.761395951613D+05
coeff(951)=0.962632912875D+05
coeff(952)=0.818835383850D+04
coeff(953)=-0.410370350125D+05
coeff(954)=-0.151810965215D+06
coeff(955)=-0.372361305428D+04
coeff(956)=0.122790589845D+06
coeff(957)=0.218513941952D+05
coeff(958)=0.278723019513D+05
coeff(959)=-0.538686909851D+05
coeff(960)=0.396272437562D+04
coeff(961)=-0.236348998171D+04
coeff(962)=-0.237215493028D+05
coeff(963)=-0.663395065200D+04
coeff(964)=0.877436549799D+04
coeff(965)=0.741321396309D+05
coeff(966)=-0.103970307086D+05
coeff(967)=-0.202391600281D+05
coeff(968)=-0.129271225659D+05
coeff(969)=0.426379620029D+05
coeff(970)=-0.245009561963D+04
coeff(971)=-0.136583551373D+04
coeff(972)=-0.187973174657D+05
coeff(973)=0.148618629873D+05
coeff(974)=-0.310294075471D+04
coeff(975)=0.134300404078D+05
coeff(976)=-0.625680960604D+04
coeff(977)=-0.188646765780D+05
coeff(978)=0.111557411603D+05
coeff(979)=0.705548261551D+03
coeff(980)=-0.161734961975D+05
coeff(981)=0.144091985576D+05
coeff(982)=-0.271255537400D+04
coeff(983)=0.511487487142D+05
coeff(984)=-0.773484403209D+05
coeff(985)=0.257677911463D+05
coeff(986)=0.169811504981D+04
coeff(987)=0.180211557673D+05
coeff(988)=-0.799254847493D+04
coeff(989)=-0.295351769719D+05
coeff(990)=0.653300679953D+05
coeff(991)=0.278364909076D+05
coeff(992)=-0.820641687557D+05
coeff(993)=-0.682911951575D+04
coeff(994)=0.322057128170D+05
coeff(995)=0.399125715926D+05
coeff(996)=-0.212998950327D+05
coeff(997)=-0.306998855590D+05
coeff(998)=0.181538546281D+05
coeff(999)=0.174083816133D+05
coeff(1000)=-0.189079274214D+05
coeff(1001)=-0.504881184228D+04
coeff(1002)=0.840413662963D+04
coeff(1003)=0.620966224798D+04
coeff(1004)=-0.142031734295D+05
coeff(1005)=-0.624308650025D+04
coeff(1006)=0.101980172246D+05
coeff(1007)=-0.193414472174D+05
coeff(1008)=0.124736994950D+05
coeff(1009)=0.453116033907D+05
coeff(1010)=0.324533601980D+05
coeff(1011)=-0.220242958207D+05
coeff(1012)=0.516558148155D+04
coeff(1013)=0.342447144685D+05
coeff(1014)=0.167375371534D+05
coeff(1015)=0.213970698149D+05
coeff(1016)=-0.216457920851D+05
coeff(1017)=-0.983802264048D+04
coeff(1018)=-0.404640601435D+05
coeff(1019)=-0.240311265025D+05
coeff(1020)=-0.642180986150D+05
coeff(1021)=-0.266799810402D+04
coeff(1022)=0.103522551874D+05
coeff(1023)=-0.261769069910D+05
coeff(1024)=0.948071892633D+05
coeff(1025)=-0.258924842014D+04
coeff(1026)=0.908259693698D+03
coeff(1027)=-0.159212110344D+03
coeff(1028)=0.236800269388D+05
coeff(1029)=-0.284101072179D+05
coeff(1030)=0.552113386487D+05
coeff(1031)=0.380635650382D+04
coeff(1032)=-0.380280445939D+05
coeff(1033)=0.389979033999D+04
coeff(1034)=-0.584141486491D+05
coeff(1035)=0.412651017634D+05
coeff(1036)=0.590621783246D+04
coeff(1037)=0.837643797872D+05
coeff(1038)=0.265394683783D+05
coeff(1039)=-0.340421419532D+05
coeff(1040)=0.278069635020D+05
coeff(1041)=-0.295380592217D+05
coeff(1042)=0.718228924334D+04
coeff(1043)=0.118302503619D+05
coeff(1044)=0.871865388057D+05
coeff(1045)=-0.367044892246D+05
coeff(1046)=0.106396647678D+06
coeff(1047)=-0.349337693558D+04
coeff(1048)=-0.104319326247D+06
coeff(1049)=-0.630980450323D+05
coeff(1050)=0.381583174699D+05
coeff(1051)=0.196620882400D+05
coeff(1052)=0.545468690897D+04
coeff(1053)=0.610291904551D+05
coeff(1054)=-0.428472213748D+05
coeff(1055)=0.434714515801D+05
coeff(1056)=-0.387525995535D+04
coeff(1057)=0.254633947466D+04
coeff(1058)=-0.727461810403D+04
coeff(1059)=0.345238989211D+05
coeff(1060)=-0.514484239877D+05
coeff(1061)=0.312228013202D+05
coeff(1062)=-0.203702910901D+05
coeff(1063)=-0.238380425491D+05
coeff(1064)=-0.464990887160D+05
coeff(1065)=-0.129933756899D+04
coeff(1066)=-0.383269984933D+05
coeff(1067)=0.186492688421D+05
coeff(1068)=-0.129570222358D+05
coeff(1069)=-0.863726562006D+05
coeff(1070)=0.721485944589D+05
coeff(1071)=0.108159135633D+05
coeff(1072)=-0.604215177604D+05
coeff(1073)=-0.169341803985D+05
coeff(1074)=-0.445873806213D+05
coeff(1075)=-0.348122603242D+05
coeff(1076)=0.277617259840D+05
coeff(1077)=-0.108993946434D+05
coeff(1078)=0.139046238700D+05
coeff(1079)=0.442866125328D+05
coeff(1080)=0.387221814599D+05
coeff(1081)=0.872842069725D+05
coeff(1082)=0.874462788054D+03
coeff(1083)=-0.109522741600D+06
coeff(1084)=-0.147361316402D+05
coeff(1085)=-0.390637648072D+05
coeff(1086)=-0.241698555744D+05
coeff(1087)=0.709034426329D+05
coeff(1088)=-0.229348298470D+05
coeff(1089)=-0.526309717641D+04
coeff(1090)=-0.971038940271D+05
coeff(1091)=0.205390559846D+05
coeff(1092)=0.144159109206D+06
coeff(1093)=-0.569069758764D+05
coeff(1094)=0.570015454992D+04
coeff(1095)=-0.138532439720D+06
coeff(1096)=0.254262626111D+05
coeff(1097)=0.680103210257D+05
coeff(1098)=0.188666444441D+05
coeff(1099)=0.454351227441D+05
coeff(1100)=-0.307220127060D+05
coeff(1101)=-0.133709170846D+05
coeff(1102)=-0.510466440796D+05
coeff(1103)=-0.132349985870D+05
coeff(1104)=-0.154470490918D+05
coeff(1105)=-0.823167618242D+05
coeff(1106)=0.110514326782D+06
coeff(1107)=0.214742515219D+05
coeff(1108)=-0.902198790831D+05
coeff(1109)=-0.111465453591D+06
coeff(1110)=0.393689725980D+04
coeff(1111)=0.580276629944D+05
coeff(1112)=0.171782798076D+05
coeff(1113)=0.142400199585D+06
coeff(1114)=-0.978349980513D+04
coeff(1115)=-0.582668362333D+05
coeff(1116)=-0.165222875992D+06
coeff(1117)=0.139310043173D+06
coeff(1118)=0.866418628348D+05
coeff(1119)=-0.891258034324D+05
coeff(1120)=0.978128637816D+05
coeff(1121)=-0.420168916441D+05
coeff(1122)=0.306857393693D+05
coeff(1123)=0.403655052305D+05
coeff(1124)=0.479905387253D+05
coeff(1125)=0.139430448134D+06
coeff(1126)=-0.561231812856D+05
coeff(1127)=0.720753076763D+04
coeff(1128)=0.390465293996D+05
coeff(1129)=-0.214775694081D+05
coeff(1130)=-0.307327430203D+05
coeff(1131)=0.399764033976D+05
coeff(1132)=-0.119320483579D+06
coeff(1133)=-0.118344315105D+06
coeff(1134)=-0.186625150563D+05
coeff(1135)=0.117941846559D+05
coeff(1136)=0.428301103388D+05
coeff(1137)=0.377104085372D+05
coeff(1138)=0.536451366723D+05
coeff(1139)=-0.276889490136D+05
coeff(1140)=-0.329899742015D+05
coeff(1141)=-0.193374781083D+05
coeff(1142)=0.567111297990D+05
coeff(1143)=-0.228821744732D+05
coeff(1144)=0.106480122693D+05
coeff(1145)=0.179129621337D+05
coeff(1146)=-0.125035771402D+05
coeff(1147)=0.703173330557D+05
coeff(1148)=0.515268538517D+05
coeff(1149)=-0.131725675295D+05
coeff(1150)=-0.406356162856D+05
coeff(1151)=0.952220407406D+05
coeff(1152)=-0.415343057286D+05
coeff(1153)=0.119804422199D+05
coeff(1154)=0.386703228304D+05
coeff(1155)=-0.115120772176D+06
coeff(1156)=-0.446112881068D+05
coeff(1157)=0.815240891026D+05
coeff(1158)=-0.955880179857D+05
coeff(1159)=0.259734434729D+06
coeff(1160)=0.897450541157D+05
coeff(1161)=0.995005645696D+05
coeff(1162)=0.291756610349D+05
coeff(1163)=0.169984151059D+06
coeff(1164)=0.586921027592D+05
coeff(1165)=-0.314657758535D+05
coeff(1166)=-0.306085371692D+05
coeff(1167)=-0.295587555003D+05
coeff(1168)=-0.164495936263D+05
coeff(1169)=0.810211289119D+05
coeff(1170)=0.327252247503D+05
coeff(1171)=-0.470823150084D+05
coeff(1172)=-0.549302086241D+05
coeff(1173)=-0.390698781928D+05
coeff(1174)=-0.597115125539D+05
coeff(1175)=0.103896967337D+05
coeff(1176)=-0.299468672645D+05
coeff(1177)=0.167409506385D+05
coeff(1178)=-0.305135439882D+05
coeff(1179)=-0.274057959967D+05
coeff(1180)=-0.856787737646D+05
coeff(1181)=-0.874337720016D+04
coeff(1182)=-0.286410361794D+05
coeff(1183)=-0.579703883300D+05
coeff(1184)=0.145673723557D+05
coeff(1185)=0.298427471061D+05
coeff(1186)=-0.552772930738D+05
coeff(1187)=-0.493314709786D+05
coeff(1188)=-0.102379116963D+05
coeff(1189)=0.417508989731D+04
coeff(1190)=0.899935301099D+04
coeff(1191)=-0.291064021206D+04
coeff(1192)=-0.138830266578D+05
coeff(1193)=-0.162975198580D+04
coeff(1194)=0.807889744536D+03
coeff(1195)=0.551374027749D+04
coeff(1196)=-0.192015526542D+04
coeff(1197)=-0.771763490976D+04
coeff(1198)=-0.308448780861D+04
coeff(1199)=0.657668427330D+04
coeff(1200)=0.159989110098D+05
coeff(1201)=-0.731716525770D+04
coeff(1202)=-0.335138187526D+04
coeff(1203)=0.117420343326D+05
coeff(1204)=-0.158757523369D+05
coeff(1205)=-0.150423503200D+05
coeff(1206)=0.499770672840D+04
coeff(1207)=-0.711820475743D+02
coeff(1208)=0.647645533745D+04
coeff(1209)=0.585356372676D+04
coeff(1210)=0.148404168290D+05
coeff(1211)=0.136876875186D+05
coeff(1212)=-0.768579192718D+04
coeff(1213)=-0.126543987467D+05
coeff(1214)=0.191689562911D+05
coeff(1215)=-0.292548146186D+04
coeff(1216)=-0.148621123891D+05
coeff(1217)=-0.708476486520D+04
coeff(1218)=-0.406373615313D+04
coeff(1219)=0.214103438123D+04
coeff(1220)=-0.648563202613D+04
coeff(1221)=0.771309675633D+03
coeff(1222)=-0.194976269416D+05
coeff(1223)=0.175756473183D+05
coeff(1224)=-0.123724130883D+05
coeff(1225)=0.154492844946D+05
coeff(1226)=-0.346829400346D+05
coeff(1227)=-0.353519124278D+05
coeff(1228)=-0.456164770464D+04
coeff(1229)=0.283656625580D+05
coeff(1230)=-0.173539044111D+05
coeff(1231)=-0.219730086037D+05
coeff(1232)=-0.396029826496D+05
coeff(1233)=0.182754012317D+05
coeff(1234)=0.315086200278D+05
coeff(1235)=-0.323390052230D+05
coeff(1236)=-0.532829047164D+05
coeff(1237)=-0.377567411301D+05
coeff(1238)=0.605213389961D+05
coeff(1239)=0.204855129165D+05
coeff(1240)=-0.700118513380D+05
coeff(1241)=-0.820381069087D+04
coeff(1242)=-0.258004616804D+04
coeff(1243)=-0.213791905303D+05
coeff(1244)=0.670328381095D+05
coeff(1245)=0.265842753345D+05
coeff(1246)=0.385749534078D+03
coeff(1247)=-0.600830430978D+05
coeff(1248)=-0.362232945579D+05
coeff(1249)=-0.469467626973D+05
coeff(1250)=-0.132687805035D+06
coeff(1251)=0.775207666472D+05
coeff(1252)=0.459175082061D+05
coeff(1253)=-0.462588125033D+05
coeff(1254)=0.482129695126D+05
coeff(1255)=-0.556218818837D+05
coeff(1256)=-0.746137448267D+05
coeff(1257)=0.109665964546D+04
coeff(1258)=0.414695528474D+05
coeff(1259)=-0.960721639726D+05
coeff(1260)=-0.420314386542D+04
coeff(1261)=0.133357741445D+05
coeff(1262)=0.157992099914D+06
coeff(1263)=-0.106483937666D+05
coeff(1264)=0.876406998194D+04
coeff(1265)=-0.167530575962D+06
coeff(1266)=0.992841488783D+03
coeff(1267)=-0.140492895999D+05
coeff(1268)=-0.487945932139D+05
coeff(1269)=-0.123537140801D+06
coeff(1270)=-0.726248247198D+05
coeff(1271)=-0.466615635836D+05
coeff(1272)=0.277759768622D+05
coeff(1273)=0.639181242789D+05
coeff(1274)=0.768523015741D+05
coeff(1275)=0.968826271734D+05
coeff(1276)=-0.226081612198D+05
coeff(1277)=0.761188102782D+05
coeff(1278)=0.270673255370D+05
coeff(1279)=0.135044319961D+06
coeff(1280)=0.634880616087D+05
coeff(1281)=-0.101354945336D+06
coeff(1282)=-0.624711674837D+05
coeff(1283)=0.103867366179D+05
coeff(1284)=0.288537921994D+05
coeff(1285)=-0.678220162351D+05
coeff(1286)=0.165364630974D+05
coeff(1287)=-0.302321154992D+05
coeff(1288)=0.353657185324D+05
coeff(1289)=0.601431874268D+04
coeff(1290)=0.104715900327D+06
coeff(1291)=-0.140851597540D+05
coeff(1292)=-0.334914617343D+05
coeff(1293)=0.266503642850D+05
coeff(1294)=0.393455668236D+05
coeff(1295)=0.252800052193D+05
coeff(1296)=-0.204826277796D+04
coeff(1297)=0.578119735678D+04
coeff(1298)=-0.191356487320D+04
coeff(1299)=-0.231510814719D+04
coeff(1300)=-0.113833184845D+02
  return
end if

entry u12_dc_10(xyz,v,s)

r12=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r13=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r14=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r23=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r24=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r34=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

x(1)=exp(-((r12-ra1))/a1 -((r12-rb1)**2)/b1)
x(2)=exp(-((r13-ra1))/a1 -((r13-rb1)**2)/b1)
x(3)=exp(-((r14-ra2))/a2 -((r14-rb2)**2)/b2)
x(4)=exp(-((r23-ra1))/a1 -((r23-rb1)**2)/b1)
x(5)=exp(-((r24-ra2))/a2 -((r24-rb2)**2)/b2)
x(6)=exp(-((r34-ra2))/a2 -((r34-rb2)**2)/b2)

do i=1,3
do j=1,3
rrel(i,j)=xyz(i,j)-xyz(4,j)
enddo
enddo

!compute r2xr3
cross(1)=rrel(2,2)*rrel(3,3)-rrel(2,3)*rrel(3,2)
cross(2)=-rrel(2,1)*rrel(3,3)+rrel(2,3)*rrel(3,1)
cross(3)=rrel(2,1)*rrel(3,2)-rrel(2,2)*rrel(3,1)
!compute p*r1*(r2xr3)
volume=0.00000
do i=1,3
volume=volume+rrel(1,i)*cross(i)
enddo
volume=tanh(15.4*volume)

call ev_pol_con_10(x,q)
v=dot_product(coeff,q)*volume/627.509474
s=dot_product(coeff,q)/627.509474

end  subroutine

!===========================================================================
! *def u13_order12
! subtoutine of computing u13
!===========================================================================
subroutine u13_order12
implicit none
real*8 :: xyz(4,3)
real*8 :: v, r12, r13, r14, r23, r24, r34
real*8 :: a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer, parameter :: ncoeff=3075
real*8 :: coeff(ncoeff), q(ncoeff)
real*8 :: x(6)
integer ::  ncall
data ncall/1/
save ncall, coeff, a1, a2, b1, b2, ra1, ra2, rb1, rb2

if(ncall.eq.1) then
a1=1.2561573
a2=1.1028022
b1=1.75
b2=1.61
ra1=1.196078
ra2=0.9333333
rb1=1.196078
rb2=0.9333333
coeff(1)=0.231115191881D+04
coeff(2)=0.766914172340D+04
coeff(3)=-0.557136124868D+04
coeff(4)=-0.285959979588D+04
coeff(5)=0.392680364188D+03
coeff(6)=-0.258600088240D+04
coeff(7)=-0.738930854520D+04
coeff(8)=-0.569480495279D+04
coeff(9)=0.278961865807D+04
coeff(10)=0.169344540759D+05
coeff(11)=-0.892714283177D+04
coeff(12)=0.581826911803D+04
coeff(13)=0.779309372717D+04
coeff(14)=0.138882801292D+05
coeff(15)=-0.109722406393D+04
coeff(16)=-0.272257153066D+04
coeff(17)=0.741726637872D+04
coeff(18)=0.306803738871D+04
coeff(19)=0.131329843696D+04
coeff(20)=-0.224688468756D+04
coeff(21)=0.551678306305D+04
coeff(22)=0.167202457181D+05
coeff(23)=0.451561471761D+04
coeff(24)=0.753446773078D+04
coeff(25)=-0.173266567650D+04
coeff(26)=-0.203774267068D+05
coeff(27)=-0.109856750064D+05
coeff(28)=-0.218575649186D+05
coeff(29)=-0.388794475986D+04
coeff(30)=-0.664560881872D+04
coeff(31)=0.103875992380D+05
coeff(32)=-0.761410788291D+04
coeff(33)=0.233045657571D+04
coeff(34)=-0.969339433218D+04
coeff(35)=-0.165481322351D+05
coeff(36)=0.854349804722D+04
coeff(37)=0.247970621031D+04
coeff(38)=-0.280118846613D+04
coeff(39)=0.115799405813D+04
coeff(40)=0.785819096606D+04
coeff(41)=-0.118294718031D+05
coeff(42)=-0.248800913717D+04
coeff(43)=-0.597880272425D+04
coeff(44)=0.506061830100D+03
coeff(45)=0.765991446455D+04
coeff(46)=-0.233552771757D+04
coeff(47)=-0.569240836104D+04
coeff(48)=0.308197467374D+04
coeff(49)=0.284250957199D+04
coeff(50)=-0.107820265240D+04
coeff(51)=-0.145499549445D+03
coeff(52)=-0.357028658899D+04
coeff(53)=-0.126803300125D+05
coeff(54)=-0.160409945363D+04
coeff(55)=-0.421315455714D+04
coeff(56)=0.731118479712D+04
coeff(57)=-0.317137601678D+04
coeff(58)=-0.774649845069D+04
coeff(59)=0.260717511909D+04
coeff(60)=-0.621274257657D+04
coeff(61)=0.417525751798D+04
coeff(62)=0.134588412182D+04
coeff(63)=0.836271486759D+03
coeff(64)=0.120988074835D+05
coeff(65)=-0.313375481450D+04
coeff(66)=0.112807269607D+05
coeff(67)=0.314490082818D+04
coeff(68)=-0.403245736345D+04
coeff(69)=-0.880257524573D+04
coeff(70)=0.164902202985D+05
coeff(71)=-0.108591722489D+05
coeff(72)=0.138703381122D+05
coeff(73)=0.725572955712D+04
coeff(74)=0.939990477056D+04
coeff(75)=-0.758802967075D+04
coeff(76)=-0.267397820247D+04
coeff(77)=0.122446503968D+05
coeff(78)=0.737507467719D+03
coeff(79)=-0.815875819889D+03
coeff(80)=0.327768006014D+04
coeff(81)=-0.467343182343D+04
coeff(82)=-0.151662106004D+05
coeff(83)=0.105237603572D+05
coeff(84)=0.419156522595D+04
coeff(85)=0.477503779515D+04
coeff(86)=-0.911620340692D+04
coeff(87)=0.335469575465D+04
coeff(88)=0.255102999820D+04
coeff(89)=-0.715020317241D+03
coeff(90)=0.984672028211D+04
coeff(91)=-0.892867071101D+03
coeff(92)=-0.255018622701D+04
coeff(93)=-0.504303336431D+04
coeff(94)=-0.430470260282D+04
coeff(95)=0.188859162278D+04
coeff(96)=0.165124499470D+04
coeff(97)=0.780627145376D+04
coeff(98)=0.498621141531D+04
coeff(99)=-0.141051561213D+05
coeff(100)=-0.605278631998D+04
coeff(101)=0.761997239316D+04
coeff(102)=0.280347105486D+04
coeff(103)=-0.137636364883D+04
coeff(104)=-0.237193599448D+04
coeff(105)=-0.383680506689D+04
coeff(106)=-0.450805757132D+04
coeff(107)=-0.194526481172D+05
coeff(108)=0.275470154762D+03
coeff(109)=0.104932067454D+05
coeff(110)=-0.327373650803D+04
coeff(111)=-0.203568159828D+04
coeff(112)=-0.162113695044D+05
coeff(113)=0.443497903714D+04
coeff(114)=0.853857219410D+04
coeff(115)=-0.536936779124D+04
coeff(116)=-0.906039972601D+03
coeff(117)=-0.909745633723D+04
coeff(118)=0.204780221141D+04
coeff(119)=0.702363629920D+04
coeff(120)=-0.341571710626D+04
coeff(121)=-0.150496955166D+04
coeff(122)=-0.520063526368D+04
coeff(123)=-0.659390204403D+04
coeff(124)=0.190233553161D+05
coeff(125)=0.513593060963D+04
coeff(126)=0.146882929229D+04
coeff(127)=-0.685132288287D+04
coeff(128)=0.516100037676D+04
coeff(129)=-0.144597490267D+05
coeff(130)=-0.239972949982D+04
coeff(131)=-0.378877930972D+04
coeff(132)=-0.140059251763D+05
coeff(133)=-0.197625033073D+04
coeff(134)=0.595289854817D+03
coeff(135)=0.118753545142D+05
coeff(136)=0.737261434489D+04
coeff(137)=-0.383501550068D+04
coeff(138)=-0.515056765371D+03
coeff(139)=0.613978132086D+04
coeff(140)=0.302217566551D+04
coeff(141)=-0.180353901441D+04
coeff(142)=0.391677387570D+04
coeff(143)=-0.130016437859D+05
coeff(144)=0.187795745055D+05
coeff(145)=-0.173983278076D+03
coeff(146)=0.534545489608D+04
coeff(147)=0.971405523781D+03
coeff(148)=0.961808007466D+04
coeff(149)=0.562998726186D+02
coeff(150)=-0.919348695306D+04
coeff(151)=0.775402621581D+03
coeff(152)=0.101763029192D+05
coeff(153)=-0.938143340451D+04
coeff(154)=0.111153066998D+05
coeff(155)=-0.859019540453D+04
coeff(156)=0.864659377827D+04
coeff(157)=0.519697639912D+04
coeff(158)=0.364547477110D+04
coeff(159)=-0.736997142548D+04
coeff(160)=-0.186749550910D+05
coeff(161)=0.604874456187D+04
coeff(162)=-0.122211291701D+04
coeff(163)=-0.294162327731D+04
coeff(164)=-0.420035686294D+04
coeff(165)=0.853923347403D+04
coeff(166)=0.143318037423D+05
coeff(167)=-0.630846023881D+04
coeff(168)=-0.146454200572D+04
coeff(169)=0.101762789563D+05
coeff(170)=0.503052453662D+04
coeff(171)=0.767854748445D+04
coeff(172)=0.142494227204D+05
coeff(173)=-0.245413684461D+04
coeff(174)=-0.584654180178D+04
coeff(175)=0.829590327047D+04
coeff(176)=0.189812133668D+03
coeff(177)=0.398468416352D+04
coeff(178)=0.352794964226D+04
coeff(179)=-0.608131465401D+04
coeff(180)=0.357999901038D+04
coeff(181)=0.154193271966D+05
coeff(182)=0.147485786525D+04
coeff(183)=-0.194342330463D+04
coeff(184)=0.109379776172D+05
coeff(185)=-0.956071244778D+04
coeff(186)=0.734653073497D+04
coeff(187)=0.778479814775D+04
coeff(188)=0.913886697071D+04
coeff(189)=0.403228995940D+04
coeff(190)=-0.805183798073D+03
coeff(191)=0.112306748428D+04
coeff(192)=0.126659387937D+05
coeff(193)=-0.394837666692D+04
coeff(194)=-0.918155239264D+04
coeff(195)=0.959268331348D+03
coeff(196)=-0.512891439022D+04
coeff(197)=0.123239481758D+04
coeff(198)=0.546870019149D+04
coeff(199)=-0.113262506270D+04
coeff(200)=-0.318635595355D+04
coeff(201)=-0.521357649783D+04
coeff(202)=0.105343013291D+04
coeff(203)=0.807469055972D+04
coeff(204)=0.204468287206D+04
coeff(205)=0.201179888348D+04
coeff(206)=-0.563075630944D+04
coeff(207)=0.993422804099D+04
coeff(208)=0.972283624671D+04
coeff(209)=-0.108760971468D+05
coeff(210)=0.335515602124D+04
coeff(211)=-0.809749502608D+04
coeff(212)=0.382575449336D+04
coeff(213)=0.498219476531D+04
coeff(214)=-0.286887359377D+04
coeff(215)=-0.647056923202D+04
coeff(216)=0.132645614561D+05
coeff(217)=0.274384061379D+03
coeff(218)=0.138539727442D+04
coeff(219)=0.854408729135D+04
coeff(220)=0.944467094366D+03
coeff(221)=0.523899770841D+04
coeff(222)=0.110304820651D+04
coeff(223)=0.262893012816D+04
coeff(224)=-0.281776957112D+04
coeff(225)=0.693524442480D+03
coeff(226)=0.975786820914D+04
coeff(227)=-0.107373990553D+04
coeff(228)=-0.325008895275D+03
coeff(229)=0.353057819245D+03
coeff(230)=-0.877105385411D+04
coeff(231)=-0.167448772530D+04
coeff(232)=0.507717252429D+04
coeff(233)=0.302903907542D+04
coeff(234)=-0.716758184260D+04
coeff(235)=0.731146873877D+04
coeff(236)=-0.114378704009D+05
coeff(237)=-0.366675994623D+04
coeff(238)=-0.700762080242D+03
coeff(239)=-0.512766931270D+04
coeff(240)=0.143349823624D+05
coeff(241)=0.482038293609D+04
coeff(242)=0.117445479454D+05
coeff(243)=0.762307957193D+04
coeff(244)=-0.163041402741D+04
coeff(245)=-0.792387654616D+04
coeff(246)=0.766436505269D+04
coeff(247)=-0.310264525878D+04
coeff(248)=-0.711880688064D+04
coeff(249)=0.127519452818D+05
coeff(250)=-0.609157218671D+04
coeff(251)=0.288296346959D+04
coeff(252)=0.353870882029D+04
coeff(253)=0.239465577481D+04
coeff(254)=-0.599198703187D+04
coeff(255)=0.980562589901D+04
coeff(256)=-0.788346942204D+04
coeff(257)=-0.989760604999D+03
coeff(258)=-0.479098231560D+04
coeff(259)=-0.115115742444D+05
coeff(260)=-0.450602097318D+04
coeff(261)=-0.823284348312D+04
coeff(262)=0.661736080389D+04
coeff(263)=-0.462350424825D+04
coeff(264)=0.426626534236D+04
coeff(265)=-0.193727059203D+04
coeff(266)=0.611367734036D+04
coeff(267)=-0.210337937126D+05
coeff(268)=0.233197669025D+04
coeff(269)=-0.512594276958D+04
coeff(270)=0.103077657483D+05
coeff(271)=-0.349020967464D+04
coeff(272)=-0.515467907460D+04
coeff(273)=0.643448729535D+04
coeff(274)=0.350564643192D+03
coeff(275)=-0.354583019920D+04
coeff(276)=0.968468494493D+04
coeff(277)=-0.147158649648D+05
coeff(278)=0.122244371810D+05
coeff(279)=-0.751806407624D+04
coeff(280)=-0.132770738935D+05
coeff(281)=0.279897004632D+04
coeff(282)=0.811655078888D+03
coeff(283)=0.199110954262D+04
coeff(284)=0.700983188732D+04
coeff(285)=-0.148646086819D+04
coeff(286)=0.201612968054D+03
coeff(287)=-0.402239656004D+04
coeff(288)=0.521055351892D+04
coeff(289)=-0.998591067508D+04
coeff(290)=-0.261661062308D+04
coeff(291)=-0.273583691114D+04
coeff(292)=-0.118677984602D+05
coeff(293)=0.681878690751D+04
coeff(294)=-0.338569792706D+04
coeff(295)=-0.997190806337D+04
coeff(296)=0.505622827380D+04
coeff(297)=-0.317741386504D+04
coeff(298)=-0.512155805627D+04
coeff(299)=-0.134428199634D+04
coeff(300)=-0.742900669302D+04
coeff(301)=0.298953618374D+04
coeff(302)=0.149231837929D+05
coeff(303)=-0.233911020036D+03
coeff(304)=-0.517390442191D+04
coeff(305)=-0.782989525052D+04
coeff(306)=-0.389897148069D+04
coeff(307)=0.664908651747D+04
coeff(308)=0.962263884894D+03
coeff(309)=-0.897175125164D+02
coeff(310)=0.357462552986D+04
coeff(311)=-0.447139906049D+04
coeff(312)=0.284331721480D+04
coeff(313)=-0.677798944877D+04
coeff(314)=-0.687688539579D+04
coeff(315)=0.130241750841D+04
coeff(316)=0.361377810274D+04
coeff(317)=-0.480299659088D+04
coeff(318)=0.109179525937D+05
coeff(319)=0.246017681909D+04
coeff(320)=0.916447237102D+03
coeff(321)=-0.257316756335D+04
coeff(322)=0.478674236090D+04
coeff(323)=-0.109286096162D+05
coeff(324)=-0.272505689285D+04
coeff(325)=-0.511388658005D+04
coeff(326)=-0.253900577805D+03
coeff(327)=0.202581822955D+04
coeff(328)=-0.999071883011D+04
coeff(329)=-0.345218669818D+04
coeff(330)=-0.899687803036D+04
coeff(331)=-0.257713120088D+04
coeff(332)=-0.242611355131D+04
coeff(333)=-0.117068471402D+05
coeff(334)=-0.522917759287D+04
coeff(335)=-0.101430963680D+05
coeff(336)=-0.120340967998D+05
coeff(337)=0.797871594413D+04
coeff(338)=-0.805163495681D+03
coeff(339)=0.919341390953D+03
coeff(340)=-0.722836796153D+03
coeff(341)=-0.676878142225D+04
coeff(342)=0.809707050944D+04
coeff(343)=0.112831197730D+05
coeff(344)=0.444642874944D+04
coeff(345)=0.128699536156D+05
coeff(346)=0.129523290751D+05
coeff(347)=-0.560100231061D+04
coeff(348)=-0.124456726326D+04
coeff(349)=0.433274775624D+04
coeff(350)=-0.990731876641D+04
coeff(351)=0.211649627502D+04
coeff(352)=0.532088715707D+04
coeff(353)=0.161638275567D+05
coeff(354)=-0.592437862482D+04
coeff(355)=0.117294519059D+05
coeff(356)=0.563162655939D+04
coeff(357)=-0.626571156027D+04
coeff(358)=0.756729961437D+04
coeff(359)=0.302944987400D+04
coeff(360)=-0.598641762471D+04
coeff(361)=-0.336940886740D+04
coeff(362)=-0.175678062791D+04
coeff(363)=0.104097521108D+05
coeff(364)=-0.105028016226D+05
coeff(365)=-0.562030700352D+04
coeff(366)=0.483301642983D+03
coeff(367)=-0.231277484934D+04
coeff(368)=-0.162914199738D+05
coeff(369)=-0.102368762949D+05
coeff(370)=0.451825924198D+02
coeff(371)=-0.356146060236D+04
coeff(372)=-0.175392121722D+05
coeff(373)=-0.414356805773D+04
coeff(374)=-0.683057972086D+04
coeff(375)=0.670551095326D+04
coeff(376)=0.671088707765D+04
coeff(377)=-0.587832922336D+04
coeff(378)=-0.337700746041D+03
coeff(379)=-0.576758487790D+03
coeff(380)=-0.926220324770D+03
coeff(381)=0.398261359624D+04
coeff(382)=0.421184646603D+04
coeff(383)=-0.732451624504D+03
coeff(384)=0.143273564265D+04
coeff(385)=-0.366144408329D+04
coeff(386)=-0.750412054506D+04
coeff(387)=0.974836632974D+04
coeff(388)=0.196301524168D+04
coeff(389)=0.119738252032D+04
coeff(390)=0.100230986023D+04
coeff(391)=0.585790718458D+04
coeff(392)=-0.608963257368D+04
coeff(393)=-0.326989505978D+04
coeff(394)=-0.110762507822D+04
coeff(395)=-0.499160128333D+04
coeff(396)=-0.592720733518D+04
coeff(397)=0.407240484572D+04
coeff(398)=-0.247703236136D+04
coeff(399)=-0.115930360584D+04
coeff(400)=-0.304302744949D+04
coeff(401)=-0.388433867384D+04
coeff(402)=-0.739084982578D+04
coeff(403)=0.405396836089D+04
coeff(404)=0.267646319316D+04
coeff(405)=0.101732419554D+04
coeff(406)=0.502448975733D+03
coeff(407)=0.155376199527D+04
coeff(408)=-0.709808299765D+03
coeff(409)=-0.433202946484D+04
coeff(410)=-0.201983322845D+04
coeff(411)=0.922734039206D+04
coeff(412)=0.551356249867D+04
coeff(413)=0.431100441562D+04
coeff(414)=0.207853237660D+03
coeff(415)=0.129796561587D+05
coeff(416)=0.467983920689D+04
coeff(417)=-0.110218743557D+05
coeff(418)=-0.502742072962D+04
coeff(419)=0.270082623898D+03
coeff(420)=-0.367446228496D+04
coeff(421)=-0.130057796112D+03
coeff(422)=0.200889817518D+04
coeff(423)=0.290172171224D+04
coeff(424)=0.293313754024D+03
coeff(425)=-0.171323631243D+04
coeff(426)=-0.776801715553D+03
coeff(427)=0.733041585996D+04
coeff(428)=0.838929542211D+04
coeff(429)=-0.200309191247D+04
coeff(430)=0.238983334747D+04
coeff(431)=-0.339781451348D+04
coeff(432)=0.964749812762D+04
coeff(433)=-0.183736210552D+04
coeff(434)=0.330714496319D+03
coeff(435)=-0.205635507839D+04
coeff(436)=0.253785601297D+04
coeff(437)=0.459470517014D+04
coeff(438)=0.660129764918D+04
coeff(439)=-0.124229843649D+05
coeff(440)=-0.575258441894D+04
coeff(441)=0.532430252073D+04
coeff(442)=0.224990849360D+04
coeff(443)=0.223436889489D+04
coeff(444)=0.288674878373D+04
coeff(445)=-0.104915136970D+05
coeff(446)=0.148757110095D+04
coeff(447)=0.102699638406D+05
coeff(448)=0.549168456821D+04
coeff(449)=-0.647189634920D+03
coeff(450)=0.362898030070D+04
coeff(451)=-0.214088218259D+04
coeff(452)=-0.667611271350D+03
coeff(453)=-0.316269938391D+04
coeff(454)=-0.290319253802D+04
coeff(455)=0.257215621701D+04
coeff(456)=-0.105866947931D+04
coeff(457)=0.509479175366D+04
coeff(458)=-0.461279250104D+04
coeff(459)=-0.128242881766D+05
coeff(460)=-0.566220020098D+04
coeff(461)=-0.331279158431D+01
coeff(462)=0.612254366257D+03
coeff(463)=-0.874791188626D+04
coeff(464)=0.148917843455D+05
coeff(465)=0.490197704016D+04
coeff(466)=0.337331505875D+04
coeff(467)=-0.105164507704D+04
coeff(468)=0.119857594397D+04
coeff(469)=-0.232591257100D+04
coeff(470)=-0.412932140800D+04
coeff(471)=-0.302829629023D+04
coeff(472)=-0.113353147048D+04
coeff(473)=0.438548675926D+04
coeff(474)=-0.101004632549D+05
coeff(475)=0.579114552638D+04
coeff(476)=-0.558979869855D+04
coeff(477)=0.151789890031D+04
coeff(478)=-0.338329090169D+04
coeff(479)=0.471660824188D+04
coeff(480)=0.451144379669D+04
coeff(481)=-0.678133193575D+03
coeff(482)=0.118592896124D+05
coeff(483)=0.393715456885D+03
coeff(484)=-0.152043773399D+04
coeff(485)=-0.570861371708D+04
coeff(486)=-0.198810947248D+05
coeff(487)=-0.803695508444D+04
coeff(488)=-0.654189820620D+04
coeff(489)=0.590843642492D+04
coeff(490)=0.576597386026D+04
coeff(491)=0.114816712643D+05
coeff(492)=0.769982254991D+04
coeff(493)=-0.277936615940D+04
coeff(494)=-0.124830744389D+05
coeff(495)=0.453876625825D+04
coeff(496)=0.542726355444D+04
coeff(497)=0.422090808507D+04
coeff(498)=-0.392628135266D+04
coeff(499)=0.118495901402D+05
coeff(500)=-0.237800947728D+04
coeff(501)=-0.181247002008D+04
coeff(502)=-0.222779795824D+04
coeff(503)=-0.173818349601D+05
coeff(504)=0.102380673084D+05
coeff(505)=0.294369683033D+04
coeff(506)=-0.468465416519D+04
coeff(507)=-0.128971184195D+05
coeff(508)=-0.424423277013D+04
coeff(509)=0.580908049901D+04
coeff(510)=0.467392765529D+04
coeff(511)=-0.528946392911D+04
coeff(512)=0.104277939188D+05
coeff(513)=0.492163203011D+04
coeff(514)=0.932920759980D+03
coeff(515)=-0.428709741255D+04
coeff(516)=-0.131442674447D+04
coeff(517)=-0.612008258673D+04
coeff(518)=-0.131694669104D+04
coeff(519)=0.372309717482D+04
coeff(520)=-0.101316688762D+05
coeff(521)=0.507030781290D+02
coeff(522)=0.399747171413D+04
coeff(523)=-0.643512311396D+04
coeff(524)=0.701328620569D+04
coeff(525)=-0.257165943583D+04
coeff(526)=0.394214504519D+04
coeff(527)=-0.512644987871D+04
coeff(528)=-0.151669392543D+04
coeff(529)=-0.928199023693D+04
coeff(530)=-0.967616519684D+04
coeff(531)=-0.976423594965D+04
coeff(532)=-0.144853513306D+04
coeff(533)=0.435665117729D+04
coeff(534)=0.103739792126D+05
coeff(535)=-0.665900804985D+04
coeff(536)=0.152974198010D+04
coeff(537)=0.614757113374D+03
coeff(538)=-0.165521447637D+04
coeff(539)=-0.112376976136D+05
coeff(540)=0.925226331746D+04
coeff(541)=-0.708149528119D+03
coeff(542)=-0.677088157097D+04
coeff(543)=0.279219344633D+04
coeff(544)=0.302706649909D+04
coeff(545)=-0.219677382838D+04
coeff(546)=-0.567867992993D+04
coeff(547)=-0.420975882524D+04
coeff(548)=-0.549180100037D+04
coeff(549)=0.653544986866D+04
coeff(550)=0.598679514268D+04
coeff(551)=-0.525759621400D+04
coeff(552)=0.706517086891D+03
coeff(553)=-0.988196250857D+04
coeff(554)=-0.185055479924D+04
coeff(555)=0.123294057187D+04
coeff(556)=0.450956685565D+04
coeff(557)=0.797056386913D+04
coeff(558)=0.589936597868D+04
coeff(559)=0.477800469641D+04
coeff(560)=0.138087699407D+04
coeff(561)=0.413516367129D+04
coeff(562)=-0.640119118088D+04
coeff(563)=-0.121206003670D+05
coeff(564)=0.942841494720D+04
coeff(565)=-0.577873636759D+04
coeff(566)=0.497524224451D+04
coeff(567)=0.115198049742D+05
coeff(568)=-0.355422439403D+04
coeff(569)=-0.367575850060D+04
coeff(570)=-0.348063662876D+04
coeff(571)=0.120470770684D+04
coeff(572)=0.226441501546D+04
coeff(573)=0.109956248325D+05
coeff(574)=0.922803849525D+03
coeff(575)=0.112058463549D+05
coeff(576)=-0.346647008757D+04
coeff(577)=0.858701872055D+04
coeff(578)=-0.305675760534D+04
coeff(579)=0.131329175686D+04
coeff(580)=-0.209610306936D+04
coeff(581)=-0.114320779566D+04
coeff(582)=-0.316039764821D+04
coeff(583)=-0.345488334363D+04
coeff(584)=0.372965981378D+04
coeff(585)=0.194375710769D+03
coeff(586)=-0.938648081976D+04
coeff(587)=-0.110046252845D+04
coeff(588)=0.680123163162D+04
coeff(589)=-0.466715696586D+04
coeff(590)=0.514920578524D+03
coeff(591)=-0.131286469326D+05
coeff(592)=-0.732072075647D+04
coeff(593)=0.532228831528D+04
coeff(594)=-0.115865083722D+05
coeff(595)=0.149257856174D+05
coeff(596)=-0.168098780114D+04
coeff(597)=-0.285540910826D+04
coeff(598)=0.967477126912D+04
coeff(599)=-0.780557338642D+04
coeff(600)=-0.704596757552D+04
coeff(601)=-0.435394178429D+04
coeff(602)=-0.501707709058D+04
coeff(603)=0.520422033787D+04
coeff(604)=-0.279711713756D+04
coeff(605)=0.474723241559D+04
coeff(606)=-0.738631755362D+04
coeff(607)=-0.710607351193D+04
coeff(608)=0.523688348644D+02
coeff(609)=-0.776016679427D+04
coeff(610)=0.150002470716D+05
coeff(611)=-0.117592992935D+04
coeff(612)=0.105207717662D+05
coeff(613)=0.690962142928D+04
coeff(614)=-0.855070363181D+03
coeff(615)=-0.868525832927D+04
coeff(616)=-0.227077940298D+04
coeff(617)=-0.964318129685D+03
coeff(618)=0.217950179152D+04
coeff(619)=-0.292503470493D+04
coeff(620)=0.211018083344D+04
coeff(621)=-0.841595302876D+04
coeff(622)=0.309512354726D+04
coeff(623)=0.164085101992D+04
coeff(624)=-0.854892896641D+04
coeff(625)=-0.757194934732D+04
coeff(626)=-0.705003546728D+04
coeff(627)=-0.173052182925D+04
coeff(628)=-0.483143667115D+04
coeff(629)=0.412345813097D+04
coeff(630)=-0.135170942746D+05
coeff(631)=-0.465241394158D+04
coeff(632)=0.227143813542D+04
coeff(633)=-0.429355370090D+04
coeff(634)=-0.694983042812D+04
coeff(635)=-0.245944490349D+04
coeff(636)=0.201940908827D+04
coeff(637)=-0.101904241516D+05
coeff(638)=-0.671821825304D+04
coeff(639)=0.546470606239D+03
coeff(640)=-0.150665026928D+04
coeff(641)=0.284526096075D+04
coeff(642)=0.123452239640D+05
coeff(643)=-0.789576144942D+04
coeff(644)=0.298957260681D+04
coeff(645)=0.280624826922D+04
coeff(646)=-0.455210513691D+04
coeff(647)=-0.112831260688D+05
coeff(648)=-0.782431991110D+04
coeff(649)=-0.415361521807D+03
coeff(650)=-0.210020069400D+05
coeff(651)=0.306221481131D+04
coeff(652)=-0.264961665318D+04
coeff(653)=-0.206484592671D+04
coeff(654)=-0.145866099583D+04
coeff(655)=0.337663958149D+04
coeff(656)=-0.285622754553D+04
coeff(657)=-0.979522176450D+04
coeff(658)=-0.684045213335D+04
coeff(659)=-0.424431889976D+04
coeff(660)=-0.248743259015D+04
coeff(661)=0.393392804008D+04
coeff(662)=-0.458953182906D+04
coeff(663)=0.897590627988D+04
coeff(664)=0.701358365662D+04
coeff(665)=-0.101741558507D+05
coeff(666)=0.107258348853D+05
coeff(667)=-0.435734514059D+04
coeff(668)=0.423416421155D+04
coeff(669)=0.277534935866D+04
coeff(670)=0.305054035920D+04
coeff(671)=0.789775758618D+04
coeff(672)=-0.280322896847D+04
coeff(673)=-0.427435790968D+03
coeff(674)=-0.736912469447D+04
coeff(675)=-0.376016705886D+04
coeff(676)=-0.115494243755D+05
coeff(677)=0.106358111412D+04
coeff(678)=0.322092028883D+04
coeff(679)=0.679247575058D+04
coeff(680)=-0.404538814727D+04
coeff(681)=0.726425721164D+04
coeff(682)=-0.389729932035D+04
coeff(683)=0.340943553541D+04
coeff(684)=0.274535957698D+04
coeff(685)=0.223104237153D+04
coeff(686)=0.136958311923D+04
coeff(687)=-0.743951552125D+04
coeff(688)=-0.338294752725D+04
coeff(689)=-0.164006003801D+04
coeff(690)=-0.125523391888D+05
coeff(691)=-0.651942488718D+04
coeff(692)=0.191514434169D+04
coeff(693)=0.100655473954D+04
coeff(694)=-0.553574236074D+04
coeff(695)=0.634906205737D+03
coeff(696)=0.276101755052D+04
coeff(697)=-0.350572747978D+04
coeff(698)=-0.407322667203D+04
coeff(699)=-0.589440744773D+04
coeff(700)=0.334071490563D+04
coeff(701)=0.130995782753D+05
coeff(702)=-0.976847566923D+04
coeff(703)=-0.264466549768D+04
coeff(704)=0.250988550656D+04
coeff(705)=-0.700006659584D+03
coeff(706)=-0.216556200451D+04
coeff(707)=0.247551845467D+04
coeff(708)=0.168641504471D+04
coeff(709)=-0.258323769273D+04
coeff(710)=-0.541143717417D+04
coeff(711)=0.728585492875D+04
coeff(712)=0.427386177732D+04
coeff(713)=-0.106750896803D+05
coeff(714)=-0.142850613283D+03
coeff(715)=-0.263529188273D+04
coeff(716)=0.377556606652D+04
coeff(717)=-0.982795811636D+04
coeff(718)=-0.824475921561D+04
coeff(719)=0.137051468704D+04
coeff(720)=-0.370986713729D+04
coeff(721)=-0.352036254891D+04
coeff(722)=-0.975808537844D+04
coeff(723)=0.651740040158D+03
coeff(724)=-0.153690088291D+04
coeff(725)=-0.973893745856D+02
coeff(726)=-0.553852549037D+04
coeff(727)=-0.530614463678D+04
coeff(728)=-0.748335532491D+03
coeff(729)=0.884311575859D+04
coeff(730)=-0.297126532127D+04
coeff(731)=0.206537876659D+04
coeff(732)=-0.954938796958D+04
coeff(733)=0.809069493124D+04
coeff(734)=-0.558692747128D+04
coeff(735)=-0.656526463008D+04
coeff(736)=0.870198811183D+04
coeff(737)=-0.681130777785D+04
coeff(738)=0.307973525122D+04
coeff(739)=-0.356735711676D+04
coeff(740)=0.872597305909D+03
coeff(741)=-0.980942837246D+04
coeff(742)=-0.278677588989D+04
coeff(743)=0.754758925814D+04
coeff(744)=-0.509299977146D+02
coeff(745)=0.462727427190D+04
coeff(746)=0.479052206231D+04
coeff(747)=-0.415297816338D+04
coeff(748)=-0.156715750477D+05
coeff(749)=-0.814579575814D+04
coeff(750)=0.695600072644D+04
coeff(751)=-0.359110735359D+04
coeff(752)=0.335327469921D+04
coeff(753)=-0.509235080128D+03
coeff(754)=0.278037079199D+04
coeff(755)=0.775923802367D+04
coeff(756)=0.756927318692D+04
coeff(757)=-0.263167803815D+03
coeff(758)=0.194424717176D+04
coeff(759)=-0.288987261312D+04
coeff(760)=0.587531319676D+04
coeff(761)=0.430629444306D+04
coeff(762)=0.271480207423D+04
coeff(763)=0.639212296754D+04
coeff(764)=0.792432310989D+04
coeff(765)=-0.195295765085D+04
coeff(766)=-0.128242786323D+04
coeff(767)=0.104128368367D+05
coeff(768)=-0.876075324104D+04
coeff(769)=-0.519989686458D+04
coeff(770)=-0.546525845897D+04
coeff(771)=-0.130127851572D+05
coeff(772)=-0.950320737179D+04
coeff(773)=-0.464012650897D+04
coeff(774)=0.981851082986D+04
coeff(775)=-0.174579524396D+04
coeff(776)=-0.268634700455D+04
coeff(777)=0.395257784460D+03
coeff(778)=-0.332217772151D+04
coeff(779)=0.504606163046D+04
coeff(780)=-0.358541379303D+04
coeff(781)=-0.303273652245D+04
coeff(782)=-0.129612512931D+05
coeff(783)=0.221140070262D+04
coeff(784)=0.951033860475D+04
coeff(785)=-0.193913189444D+04
coeff(786)=0.204103757226D+04
coeff(787)=-0.628230770108D+04
coeff(788)=0.743920510754D+04
coeff(789)=0.819257874701D+04
coeff(790)=-0.530538497886D+04
coeff(791)=0.298161364374D+03
coeff(792)=-0.784128312893D+04
coeff(793)=-0.267205865318D+04
coeff(794)=-0.193359211020D+04
coeff(795)=0.396957577886D+04
coeff(796)=0.660668188339D+03
coeff(797)=-0.126215470138D+03
coeff(798)=0.824460442796D+04
coeff(799)=0.225315195262D+04
coeff(800)=0.379367134428D+04
coeff(801)=0.290426328497D+04
coeff(802)=0.141520734054D+04
coeff(803)=0.775642917664D+04
coeff(804)=0.229450887198D+04
coeff(805)=-0.133899618737D+04
coeff(806)=0.446680934548D+04
coeff(807)=-0.374127162031D+04
coeff(808)=0.106633827274D+05
coeff(809)=0.262002534000D+04
coeff(810)=-0.452863155011D+04
coeff(811)=0.295506486283D+04
coeff(812)=-0.638555874579D+03
coeff(813)=-0.272546059283D+04
coeff(814)=-0.540752896847D+04
coeff(815)=-0.585061615308D+04
coeff(816)=0.922095979244D+04
coeff(817)=0.147617815370D+05
coeff(818)=0.101034239835D+04
coeff(819)=0.947685255929D+04
coeff(820)=0.801541223486D+04
coeff(821)=-0.647989099176D+04
coeff(822)=-0.212350164140D+04
coeff(823)=-0.272176498744D+04
coeff(824)=0.300099934399D+04
coeff(825)=0.138824270864D+05
coeff(826)=0.267061800049D+04
coeff(827)=0.111806789711D+04
coeff(828)=0.102915461171D+05
coeff(829)=0.219579973505D+04
coeff(830)=-0.459472167798D+04
coeff(831)=-0.188228462317D+03
coeff(832)=0.513279919919D+04
coeff(833)=-0.301275224088D+04
coeff(834)=-0.664504888529D+04
coeff(835)=-0.610197374185D+04
coeff(836)=-0.286121282547D+04
coeff(837)=-0.178115607597D+04
coeff(838)=0.102791973680D+05
coeff(839)=0.112713507198D+05
coeff(840)=0.584457322183D+04
coeff(841)=0.119771166267D+05
coeff(842)=-0.118503492995D+05
coeff(843)=-0.694422566207D+04
coeff(844)=0.217492887291D+04
coeff(845)=0.129004024373D+05
coeff(846)=-0.222169950982D+04
coeff(847)=0.788680835146D+04
coeff(848)=0.997298265230D+04
coeff(849)=0.379079569071D+04
coeff(850)=0.817122445064D+04
coeff(851)=0.397323039823D+04
coeff(852)=0.713483101991D+04
coeff(853)=-0.216934070888D+04
coeff(854)=0.653974256115D+04
coeff(855)=-0.609788063551D+04
coeff(856)=0.381003601879D+04
coeff(857)=-0.700317672785D+04
coeff(858)=0.998756039552D+04
coeff(859)=-0.831236762894D+04
coeff(860)=0.792532580514D+04
coeff(861)=0.382321349022D+04
coeff(862)=-0.679004254383D+04
coeff(863)=0.285663132035D+03
coeff(864)=0.822658893877D+04
coeff(865)=0.102424775348D+05
coeff(866)=0.159884945353D+04
coeff(867)=0.204434946734D+03
coeff(868)=-0.533266287986D+03
coeff(869)=0.320950003907D+04
coeff(870)=0.280716423750D+04
coeff(871)=0.471839691400D+04
coeff(872)=0.889972710163D+04
coeff(873)=-0.265790643956D+04
coeff(874)=-0.202664584324D+04
coeff(875)=-0.569547987307D+04
coeff(876)=0.671767707614D+03
coeff(877)=0.869460935812D+04
coeff(878)=0.988720916354D+04
coeff(879)=-0.601472587480D+04
coeff(880)=0.349204404453D+03
coeff(881)=-0.753548693973D+02
coeff(882)=0.308469046149D+04
coeff(883)=0.291910720061D+04
coeff(884)=-0.630352947539D+03
coeff(885)=0.488293530602D+04
coeff(886)=-0.353068417057D+04
coeff(887)=0.590739329596D+04
coeff(888)=0.478991817466D+04
coeff(889)=0.992129875234D+03
coeff(890)=-0.156803711161D+04
coeff(891)=-0.143442721054D+04
coeff(892)=-0.242352395278D+04
coeff(893)=0.562322089421D+04
coeff(894)=-0.883899856622D+04
coeff(895)=-0.695702076187D+04
coeff(896)=-0.512381134672D+04
coeff(897)=-0.147367294571D+04
coeff(898)=0.551307289832D+04
coeff(899)=0.786679294680D+03
coeff(900)=0.315247322067D+04
coeff(901)=0.130227759863D+05
coeff(902)=0.114358171410D+05
coeff(903)=0.500211442360D+04
coeff(904)=-0.910372641427D+04
coeff(905)=0.182130394194D+04
coeff(906)=-0.241587256617D+04
coeff(907)=0.168808754538D+04
coeff(908)=0.579441031766D+04
coeff(909)=0.867445737007D+04
coeff(910)=-0.707632615879D+04
coeff(911)=0.227270158206D+05
coeff(912)=-0.541542133598D+04
coeff(913)=-0.733168219317D+04
coeff(914)=-0.591872489067D+04
coeff(915)=0.252010526535D+04
coeff(916)=-0.209300291968D+04
coeff(917)=0.407845210260D+04
coeff(918)=0.419643151333D+04
coeff(919)=-0.325852982982D+04
coeff(920)=0.510685250321D+04
coeff(921)=0.518245440335D+04
coeff(922)=-0.570021026484D+04
coeff(923)=0.379948451007D+04
coeff(924)=-0.442249273133D+04
coeff(925)=0.580855730437D+04
coeff(926)=0.502010690985D+04
coeff(927)=-0.305050758201D+04
coeff(928)=0.867991295795D+04
coeff(929)=0.559326365983D+04
coeff(930)=-0.500898977298D+04
coeff(931)=-0.793557040479D+04
coeff(932)=0.404586797503D+04
coeff(933)=0.142150513650D+04
coeff(934)=0.429711236763D+04
coeff(935)=0.526297596091D+04
coeff(936)=-0.949943573404D+04
coeff(937)=-0.205519313087D+04
coeff(938)=0.753542910706D+04
coeff(939)=0.574409858897D+04
coeff(940)=-0.955986352121D+04
coeff(941)=0.456488464513D+04
coeff(942)=-0.210564784285D+04
coeff(943)=0.257422574710D+03
coeff(944)=-0.106905890325D+05
coeff(945)=0.308842427905D+04
coeff(946)=0.833984940358D+04
coeff(947)=0.912028273264D+04
coeff(948)=-0.810555848269D+04
coeff(949)=-0.483269815993D+04
coeff(950)=0.574568456115D+04
coeff(951)=0.141517702335D+05
coeff(952)=0.703444990347D+04
coeff(953)=-0.287568821299D+04
coeff(954)=0.804698210028D+04
coeff(955)=-0.133199111567D+05
coeff(956)=0.249776018972D+04
coeff(957)=0.569737767322D+04
coeff(958)=0.437450130895D+04
coeff(959)=0.495175432194D+04
coeff(960)=-0.720343229582D+04
coeff(961)=-0.126998562278D+04
coeff(962)=0.123777503315D+04
coeff(963)=0.538968319814D+04
coeff(964)=0.767485674359D+04
coeff(965)=0.265035979842D+03
coeff(966)=0.167772353047D+04
coeff(967)=0.524454681460D+04
coeff(968)=-0.784295447690D+04
coeff(969)=-0.189515469697D+04
coeff(970)=0.717761674230D+04
coeff(971)=-0.360126588367D+04
coeff(972)=0.102846102590D+04
coeff(973)=0.127775215098D+05
coeff(974)=-0.605513000715D+04
coeff(975)=-0.111285869821D+05
coeff(976)=0.381051647050D+04
coeff(977)=-0.352072991933D+04
coeff(978)=0.355751511875D+04
coeff(979)=0.603107269606D+04
coeff(980)=0.127851783080D+05
coeff(981)=-0.617125995335D+04
coeff(982)=0.442474301313D+04
coeff(983)=-0.245973399467D+04
coeff(984)=0.512752527447D+04
coeff(985)=0.117309897192D+04
coeff(986)=-0.133697598331D+05
coeff(987)=-0.667031465200D+03
coeff(988)=-0.373520290201D+04
coeff(989)=0.178060872301D+04
coeff(990)=-0.694635900645D+04
coeff(991)=0.939363808099D+03
coeff(992)=0.629577778806D+03
coeff(993)=0.516056411738D+04
coeff(994)=0.717701575839D+04
coeff(995)=0.100593340573D+05
coeff(996)=0.230626260414D+04
coeff(997)=-0.957130532753D+04
coeff(998)=-0.285005330782D+04
coeff(999)=0.124493495290D+05
coeff(1000)=0.945064236383D+03
coeff(1001)=-0.578672574022D+04
coeff(1002)=-0.427118594762D+04
coeff(1003)=0.391852195358D+04
coeff(1004)=-0.813363423605D+03
coeff(1005)=0.318014535478D+04
coeff(1006)=-0.374719008288D+04
coeff(1007)=0.819384594366D+04
coeff(1008)=-0.189522845089D+04
coeff(1009)=0.576025440810D+04
coeff(1010)=0.192212695453D+05
coeff(1011)=0.543469609604D+04
coeff(1012)=0.580127298769D+04
coeff(1013)=0.489374093904D+04
coeff(1014)=-0.376111987565D+04
coeff(1015)=0.461687584317D+04
coeff(1016)=0.328803103839D+04
coeff(1017)=-0.229968138850D+04
coeff(1018)=-0.620419881856D+04
coeff(1019)=-0.525652997779D+03
coeff(1020)=0.327227587872D+04
coeff(1021)=0.253707565470D+04
coeff(1022)=-0.334928172243D+04
coeff(1023)=0.782625390101D+04
coeff(1024)=-0.985881222838D+04
coeff(1025)=0.204081846270D+04
coeff(1026)=0.453389975426D+04
coeff(1027)=-0.456626543357D+04
coeff(1028)=0.189409092297D+04
coeff(1029)=0.314088410647D+04
coeff(1030)=0.610718464025D+03
coeff(1031)=-0.723017576220D+04
coeff(1032)=0.623727629909D+04
coeff(1033)=0.335384541471D+03
coeff(1034)=0.105631482287D+05
coeff(1035)=0.247765690656D+04
coeff(1036)=-0.169512919708D+04
coeff(1037)=0.359349838163D+04
coeff(1038)=0.176925097854D+04
coeff(1039)=0.310667250563D+04
coeff(1040)=-0.770660066194D+03
coeff(1041)=0.958602908404D+03
coeff(1042)=0.345226644350D+04
coeff(1043)=-0.879995734430D+03
coeff(1044)=0.845929756141D+04
coeff(1045)=-0.194938793075D+04
coeff(1046)=0.893069982656D+04
coeff(1047)=0.828106019934D+04
coeff(1048)=-0.150641530452D+04
coeff(1049)=0.151264831575D+04
coeff(1050)=-0.915136150947D+04
coeff(1051)=0.958415279892D+04
coeff(1052)=0.147249402109D+03
coeff(1053)=-0.213900662337D+04
coeff(1054)=0.358524977064D+04
coeff(1055)=0.187355180411D+04
coeff(1056)=-0.353365883952D+04
coeff(1057)=-0.627078890138D+04
coeff(1058)=-0.204378301640D+03
coeff(1059)=-0.244674184190D+04
coeff(1060)=0.798174181138D+04
coeff(1061)=0.572830176836D+03
coeff(1062)=-0.617746892913D+04
coeff(1063)=-0.821081693071D+04
coeff(1064)=-0.451424190069D+02
coeff(1065)=0.353317610779D+04
coeff(1066)=0.300629956925D+04
coeff(1067)=0.790675725632D+04
coeff(1068)=-0.239099974580D+04
coeff(1069)=-0.116354531758D+04
coeff(1070)=0.453215588238D+03
coeff(1071)=-0.116468864211D+05
coeff(1072)=-0.528922835154D+04
coeff(1073)=-0.387580845216D+04
coeff(1074)=0.182527233118D+04
coeff(1075)=-0.496405443691D+04
coeff(1076)=-0.486956843070D+04
coeff(1077)=0.260940959538D+02
coeff(1078)=0.136800422967D+05
coeff(1079)=-0.161162497257D+04
coeff(1080)=0.357442860490D+03
coeff(1081)=-0.142142608472D+04
coeff(1082)=-0.132690842410D+04
coeff(1083)=0.815454678979D+04
coeff(1084)=-0.396476548147D+04
coeff(1085)=-0.805323996033D+04
coeff(1086)=-0.224315565296D+04
coeff(1087)=0.391939852726D+04
coeff(1088)=0.720122787602D+04
coeff(1089)=0.356176305339D+03
coeff(1090)=-0.895627069821D+04
coeff(1091)=-0.952440035330D+04
coeff(1092)=0.755570074290D+04
coeff(1093)=-0.736686400686D+04
coeff(1094)=-0.337351338527D+04
coeff(1095)=-0.107163369312D+05
coeff(1096)=-0.945343401525D+04
coeff(1097)=0.121699288951D+04
coeff(1098)=0.771428346266D+03
coeff(1099)=-0.549005135830D+04
coeff(1100)=-0.607655644706D+04
coeff(1101)=0.510264050014D+04
coeff(1102)=0.306368719492D+04
coeff(1103)=-0.577071045307D+04
coeff(1104)=0.299434300801D+04
coeff(1105)=-0.359554029914D+04
coeff(1106)=0.139914122652D+05
coeff(1107)=-0.101119393571D+04
coeff(1108)=0.984273668153D+04
coeff(1109)=0.480271351758D+04
coeff(1110)=0.116062111816D+05
coeff(1111)=0.324571349754D+04
coeff(1112)=-0.584861538414D+04
coeff(1113)=0.267578509629D+04
coeff(1114)=0.132121793027D+03
coeff(1115)=0.465902637063D+04
coeff(1116)=-0.252640617873D+04
coeff(1117)=0.401167365355D+04
coeff(1118)=0.757152864356D+04
coeff(1119)=0.330240202253D+04
coeff(1120)=-0.959408445212D+03
coeff(1121)=-0.167284961315D+05
coeff(1122)=0.868394451665D+03
coeff(1123)=0.379150461103D+04
coeff(1124)=0.514418417803D+03
coeff(1125)=0.118398341870D+04
coeff(1126)=0.413449341812D+04
coeff(1127)=0.794925679096D+04
coeff(1128)=0.108548792026D+05
coeff(1129)=0.164175263450D+04
coeff(1130)=0.637495548019D+03
coeff(1131)=-0.202616128012D+04
coeff(1132)=-0.919143684902D+04
coeff(1133)=0.165237533639D+02
coeff(1134)=0.327930363047D+04
coeff(1135)=-0.562356377043D+04
coeff(1136)=-0.789491499791D+03
coeff(1137)=-0.436216644438D+04
coeff(1138)=-0.756968170070D+03
coeff(1139)=-0.473918009082D+04
coeff(1140)=-0.726997133185D+04
coeff(1141)=0.230922343439D+04
coeff(1142)=0.885581364063D+04
coeff(1143)=-0.545788977784D+04
coeff(1144)=0.513501948297D+04
coeff(1145)=0.656746020845D+04
coeff(1146)=0.154683958464D+05
coeff(1147)=0.709788749439D+04
coeff(1148)=0.793428067530D+03
coeff(1149)=-0.269716977064D+04
coeff(1150)=-0.459144704304D+04
coeff(1151)=0.562742795863D+04
coeff(1152)=-0.317804461070D+04
coeff(1153)=-0.823353029636D+04
coeff(1154)=-0.773321940022D+04
coeff(1155)=0.453377921988D+04
coeff(1156)=0.304088474681D+04
coeff(1157)=0.874372652337D+04
coeff(1158)=0.485461958091D+04
coeff(1159)=0.387871984645D+04
coeff(1160)=0.399050536442D+04
coeff(1161)=-0.174791386350D+04
coeff(1162)=0.957532700751D+04
coeff(1163)=0.416971107005D+04
coeff(1164)=0.391618864811D+04
coeff(1165)=-0.978522832773D+03
coeff(1166)=-0.721526706054D+03
coeff(1167)=0.848764298271D+04
coeff(1168)=-0.206499049527D+04
coeff(1169)=0.547065472640D+04
coeff(1170)=0.770589248590D+03
coeff(1171)=0.241623391197D+04
coeff(1172)=-0.229974724372D+04
coeff(1173)=0.138681619035D+04
coeff(1174)=0.507959766574D+04
coeff(1175)=-0.966139601785D+04
coeff(1176)=-0.150539275134D+04
coeff(1177)=0.343334110060D+04
coeff(1178)=0.638885168763D+04
coeff(1179)=0.463724142555D+04
coeff(1180)=-0.258116295042D+04
coeff(1181)=0.279902682476D+04
coeff(1182)=0.160947266647D+03
coeff(1183)=0.551920266709D+04
coeff(1184)=0.443296785239D+04
coeff(1185)=-0.291929491191D+04
coeff(1186)=-0.430096264617D+03
coeff(1187)=0.950481489367D+03
coeff(1188)=-0.405108701649D+04
coeff(1189)=0.908635884910D+04
coeff(1190)=0.746567912648D+04
coeff(1191)=0.363002690812D+04
coeff(1192)=-0.164930528042D+05
coeff(1193)=-0.455329891243D+04
coeff(1194)=-0.419559988394D+04
coeff(1195)=0.172512443562D+03
coeff(1196)=-0.578793759562D+04
coeff(1197)=-0.640598951451D+04
coeff(1198)=-0.523452484265D+04
coeff(1199)=-0.344832551101D+03
coeff(1200)=-0.983756582241D+04
coeff(1201)=-0.978886918093D+04
coeff(1202)=0.333732034496D+03
coeff(1203)=0.972345080370D+03
coeff(1204)=-0.272312686431D+04
coeff(1205)=-0.756256780785D+04
coeff(1206)=-0.800246087590D+04
coeff(1207)=-0.429769566768D+04
coeff(1208)=0.182123229929D+04
coeff(1209)=0.148318880125D+03
coeff(1210)=0.777156694537D+03
coeff(1211)=0.584365428286D+04
coeff(1212)=-0.271881849171D+04
coeff(1213)=0.495407423078D+04
coeff(1214)=-0.435881025252D+04
coeff(1215)=0.134304180390D+04
coeff(1216)=0.108668265220D+05
coeff(1217)=-0.569176831446D+04
coeff(1218)=0.196998785383D+04
coeff(1219)=-0.160214160970D+04
coeff(1220)=0.451647095073D+04
coeff(1221)=-0.309378359991D+04
coeff(1222)=0.143922763214D+04
coeff(1223)=-0.334537553968D+04
coeff(1224)=-0.141031963727D+05
coeff(1225)=-0.539115412017D+04
coeff(1226)=0.136776747142D+05
coeff(1227)=0.116797814503D+05
coeff(1228)=-0.110410447976D+05
coeff(1229)=0.472618513777D+04
coeff(1230)=0.231050117586D+04
coeff(1231)=-0.622968952770D+04
coeff(1232)=-0.772574448311D+04
coeff(1233)=0.425821980070D+04
coeff(1234)=0.224058549297D+04
coeff(1235)=0.750065253617D+02
coeff(1236)=0.539209286783D+04
coeff(1237)=-0.393326209091D+04
coeff(1238)=0.466419153600D+04
coeff(1239)=0.175642802687D+03
coeff(1240)=0.334515830087D+04
coeff(1241)=0.974609393095D+03
coeff(1242)=0.383786898150D+04
coeff(1243)=-0.362933393565D+04
coeff(1244)=-0.933973088559D+04
coeff(1245)=-0.762556172203D+04
coeff(1246)=-0.382919176211D+03
coeff(1247)=-0.537785138681D+04
coeff(1248)=0.935754223766D+03
coeff(1249)=-0.559644042259D+04
coeff(1250)=-0.114295594663D+04
coeff(1251)=-0.139250799794D+04
coeff(1252)=0.882429273181D+04
coeff(1253)=-0.507295541346D+03
coeff(1254)=0.231488147006D+04
coeff(1255)=0.569797885330D+04
coeff(1256)=0.114151735156D+04
coeff(1257)=0.313408598741D+04
coeff(1258)=-0.659531797285D+04
coeff(1259)=0.151345494806D+05
coeff(1260)=0.433475423394D+04
coeff(1261)=0.259643701557D+04
coeff(1262)=0.134215713478D+05
coeff(1263)=0.282633033745D+04
coeff(1264)=0.131134035607D+04
coeff(1265)=-0.962949799793D+04
coeff(1266)=-0.120354034719D+05
coeff(1267)=-0.751529791503D+04
coeff(1268)=-0.134563411431D+04
coeff(1269)=-0.450697560650D+04
coeff(1270)=-0.597326236841D+04
coeff(1271)=0.398586239055D+04
coeff(1272)=-0.410891063110D+04
coeff(1273)=0.340856334045D+04
coeff(1274)=-0.294805704801D+04
coeff(1275)=-0.847014670150D+04
coeff(1276)=-0.756898533905D+04
coeff(1277)=-0.624659568005D+03
coeff(1278)=-0.256502611822D+04
coeff(1279)=0.557277797937D+04
coeff(1280)=-0.106207927457D+04
coeff(1281)=-0.206304003634D+04
coeff(1282)=-0.548460427043D+03
coeff(1283)=0.640192963858D+04
coeff(1284)=0.590060749142D+04
coeff(1285)=0.241641538523D+04
coeff(1286)=0.493457060737D+04
coeff(1287)=0.922277191829D+03
coeff(1288)=0.664381529872D+04
coeff(1289)=0.138339518080D+04
coeff(1290)=0.530840520284D+04
coeff(1291)=0.925498979219D+04
coeff(1292)=0.297010446685D+04
coeff(1293)=-0.985640208013D+03
coeff(1294)=0.308389558246D+03
coeff(1295)=0.116820821184D+05
coeff(1296)=-0.439192767591D+04
coeff(1297)=-0.133841147805D+05
coeff(1298)=-0.293445674856D+04
coeff(1299)=0.732628882156D+03
coeff(1300)=0.517493609066D+04
coeff(1301)=-0.234217427433D+04
coeff(1302)=0.242016745258D+04
coeff(1303)=0.420472874247D+03
coeff(1304)=-0.167903443033D+04
coeff(1305)=0.898766311618D+04
coeff(1306)=0.860276834364D+04
coeff(1307)=-0.309708791139D+04
coeff(1308)=-0.263615270963D+04
coeff(1309)=-0.109330071430D+05
coeff(1310)=-0.888922015390D+03
coeff(1311)=0.516777764983D+04
coeff(1312)=-0.102234118516D+05
coeff(1313)=0.394344894177D+04
coeff(1314)=0.563625616572D+04
coeff(1315)=-0.147138873310D+05
coeff(1316)=0.121446062423D+05
coeff(1317)=-0.316038565621D+04
coeff(1318)=-0.155289407316D+05
coeff(1319)=-0.179498583723D+04
coeff(1320)=0.122780746410D+04
coeff(1321)=-0.143995273964D+05
coeff(1322)=-0.465926890207D+04
coeff(1323)=-0.270835709953D+04
coeff(1324)=-0.223491138461D+04
coeff(1325)=0.664462986684D+04
coeff(1326)=-0.149223960556D+04
coeff(1327)=0.134567389586D+05
coeff(1328)=0.864681710270D+03
coeff(1329)=0.347633007260D+04
coeff(1330)=-0.272699937702D+04
coeff(1331)=-0.157926787442D+04
coeff(1332)=0.698990961703D+04
coeff(1333)=0.413420122018D+04
coeff(1334)=-0.174410641516D+04
coeff(1335)=-0.895374226874D+03
coeff(1336)=-0.147141933206D+05
coeff(1337)=-0.348908801996D+04
coeff(1338)=-0.468716800981D+04
coeff(1339)=-0.457054580265D+04
coeff(1340)=-0.688936797939D+04
coeff(1341)=0.849030776179D+04
coeff(1342)=0.123123601580D+05
coeff(1343)=-0.551408438594D+03
coeff(1344)=0.512845110423D+04
coeff(1345)=0.866084975552D+03
coeff(1346)=-0.739267685508D+04
coeff(1347)=-0.608759653713D+04
coeff(1348)=-0.973582716808D+04
coeff(1349)=-0.371975012136D+03
coeff(1350)=0.825445144573D+04
coeff(1351)=-0.462039629769D+03
coeff(1352)=-0.572899423753D+04
coeff(1353)=-0.121490419521D+05
coeff(1354)=-0.554596895094D+04
coeff(1355)=-0.330959800888D+03
coeff(1356)=-0.451250538243D+04
coeff(1357)=0.337182334532D+04
coeff(1358)=0.562792516382D+04
coeff(1359)=0.983625109055D+04
coeff(1360)=0.861771590727D+04
coeff(1361)=-0.576799707553D+03
coeff(1362)=-0.418755692181D+04
coeff(1363)=-0.195223482539D+03
coeff(1364)=-0.623962345921D+04
coeff(1365)=0.131339021683D+04
coeff(1366)=-0.371836186055D+04
coeff(1367)=-0.414285723438D+03
coeff(1368)=0.848791062149D+04
coeff(1369)=-0.925134893483D+04
coeff(1370)=-0.610137716166D+04
coeff(1371)=0.518613961539D+04
coeff(1372)=-0.214604945481D+05
coeff(1373)=-0.391962354880D+04
coeff(1374)=-0.125182868417D+03
coeff(1375)=-0.871946094587D+04
coeff(1376)=-0.129229657774D+04
coeff(1377)=-0.145163103921D+05
coeff(1378)=-0.939540989641D+04
coeff(1379)=0.152945442432D+05
coeff(1380)=-0.417290793689D+04
coeff(1381)=0.158658586022D+04
coeff(1382)=0.443694101594D+04
coeff(1383)=-0.328687642292D+02
coeff(1384)=0.851431799292D+04
coeff(1385)=-0.336185862304D+04
coeff(1386)=0.106494372247D+05
coeff(1387)=0.561406113475D+04
coeff(1388)=-0.683504232979D+04
coeff(1389)=-0.782259103868D+03
coeff(1390)=0.747550774696D+04
coeff(1391)=-0.144716873412D+05
coeff(1392)=0.167493895106D+04
coeff(1393)=-0.483236843505D+04
coeff(1394)=0.173470638966D+04
coeff(1395)=-0.119452400558D+05
coeff(1396)=-0.110174763877D+05
coeff(1397)=-0.671750523320D+04
coeff(1398)=0.424731824400D+04
coeff(1399)=-0.348413204050D+04
coeff(1400)=0.274277511929D+04
coeff(1401)=0.963771409067D+04
coeff(1402)=-0.341680920971D+04
coeff(1403)=-0.601717860291D+03
coeff(1404)=-0.254430789823D+04
coeff(1405)=-0.394639048421D+03
coeff(1406)=-0.122391377577D+05
coeff(1407)=0.282830271424D+03
coeff(1408)=-0.635064662072D+04
coeff(1409)=-0.461944434339D+04
coeff(1410)=-0.566023673419D+04
coeff(1411)=0.586482669571D+03
coeff(1412)=0.957996401019D+04
coeff(1413)=0.525254798411D+03
coeff(1414)=0.213176946554D+04
coeff(1415)=0.140446956129D+04
coeff(1416)=-0.126578057716D+05
coeff(1417)=-0.562606008031D+04
coeff(1418)=-0.129807015934D+04
coeff(1419)=0.984998897287D+04
coeff(1420)=0.979676392336D+04
coeff(1421)=-0.681945365966D+04
coeff(1422)=0.111794569939D+05
coeff(1423)=0.532730099031D+04
coeff(1424)=0.265163692158D+04
coeff(1425)=-0.803485569898D+04
coeff(1426)=-0.933008505373D+02
coeff(1427)=0.853901990903D+04
coeff(1428)=-0.349936978467D+04
coeff(1429)=-0.355554051282D+04
coeff(1430)=-0.971131216580D+04
coeff(1431)=-0.235863502142D+03
coeff(1432)=-0.684849066859D+04
coeff(1433)=-0.799265076579D+02
coeff(1434)=-0.425568529897D+04
coeff(1435)=0.635340525531D+04
coeff(1436)=-0.232631944419D+04
coeff(1437)=0.340982242217D+03
coeff(1438)=-0.914565221885D+04
coeff(1439)=-0.452333424098D+04
coeff(1440)=0.923555645729D+04
coeff(1441)=-0.502342987565D+04
coeff(1442)=0.430168140336D+04
coeff(1443)=0.521823180230D+04
coeff(1444)=-0.425169798907D+02
coeff(1445)=0.381362475660D+04
coeff(1446)=-0.903124077531D+04
coeff(1447)=-0.572991562008D+03
coeff(1448)=0.351471326222D+04
coeff(1449)=-0.567046460220D+04
coeff(1450)=-0.138549344064D+05
coeff(1451)=-0.209501525301D+04
coeff(1452)=-0.681026364529D+04
coeff(1453)=-0.242199834251D+04
coeff(1454)=0.872031934441D+04
coeff(1455)=0.233878635072D+04
coeff(1456)=0.119575718951D+05
coeff(1457)=-0.634024021865D+04
coeff(1458)=0.565680125788D+04
coeff(1459)=0.934545790506D+03
coeff(1460)=0.187957196579D+04
coeff(1461)=0.194753712361D+04
coeff(1462)=0.421641769750D+04
coeff(1463)=0.433440395754D+04
coeff(1464)=-0.575721176440D+03
coeff(1465)=0.116794022847D+05
coeff(1466)=0.274180810892D+04
coeff(1467)=0.134650750733D+05
coeff(1468)=0.158009546852D+05
coeff(1469)=0.479693234858D+04
coeff(1470)=-0.729789467621D+04
coeff(1471)=-0.100397894764D+05
coeff(1472)=-0.575633529457D+04
coeff(1473)=0.302523072616D+04
coeff(1474)=-0.191934753000D+04
coeff(1475)=0.275691582556D+04
coeff(1476)=-0.442122679603D+04
coeff(1477)=-0.109610430700D+04
coeff(1478)=0.360271528486D+04
coeff(1479)=-0.242412250973D+04
coeff(1480)=0.393096917237D+04
coeff(1481)=-0.216482780416D+04
coeff(1482)=-0.164520426914D+04
coeff(1483)=-0.867674139536D+04
coeff(1484)=-0.181915028109D+04
coeff(1485)=0.153362432949D+05
coeff(1486)=-0.942214515143D+04
coeff(1487)=-0.619611681691D+04
coeff(1488)=-0.113972272884D+04
coeff(1489)=-0.580984997808D+04
coeff(1490)=-0.750499012952D+02
coeff(1491)=-0.836620807200D+04
coeff(1492)=0.598496003974D+04
coeff(1493)=0.102655581228D+03
coeff(1494)=0.210520143576D+04
coeff(1495)=-0.708460476393D+04
coeff(1496)=0.216328681227D+03
coeff(1497)=0.460336563641D+04
coeff(1498)=0.493545660431D+04
coeff(1499)=-0.917518590434D+04
coeff(1500)=-0.123571129553D+05
coeff(1501)=0.750113363872D+04
coeff(1502)=-0.698698367587D+04
coeff(1503)=0.131082908093D+05
coeff(1504)=-0.455369200705D+04
coeff(1505)=-0.208130666699D+04
coeff(1506)=0.786585432645D+04
coeff(1507)=0.671883283750D+03
coeff(1508)=-0.420060241965D+04
coeff(1509)=-0.698638805078D+04
coeff(1510)=-0.919497375588D+04
coeff(1511)=-0.232927665468D+04
coeff(1512)=0.827268667121D+03
coeff(1513)=0.290681756690D+04
coeff(1514)=0.779656094472D+04
coeff(1515)=-0.819750827551D+04
coeff(1516)=0.116560829465D+05
coeff(1517)=-0.514648251217D+04
coeff(1518)=-0.130150166784D+05
coeff(1519)=0.252638297341D+04
coeff(1520)=0.224409624290D+04
coeff(1521)=0.101751867398D+04
coeff(1522)=-0.116686770944D+05
coeff(1523)=-0.129302748856D+05
coeff(1524)=-0.268276498938D+04
coeff(1525)=0.494221045077D+04
coeff(1526)=0.176083210012D+04
coeff(1527)=-0.311949225904D+04
coeff(1528)=-0.211326041017D+05
coeff(1529)=-0.122759741286D+04
coeff(1530)=-0.700556078358D+04
coeff(1531)=0.751732551121D+04
coeff(1532)=-0.174022229575D+05
coeff(1533)=0.124447098222D+05
coeff(1534)=0.121536671425D+05
coeff(1535)=-0.491388836003D+04
coeff(1536)=-0.671461355092D+04
coeff(1537)=-0.950189217450D+03
coeff(1538)=-0.216104258687D+04
coeff(1539)=0.507493152696D+04
coeff(1540)=0.206483529351D+03
coeff(1541)=0.167641116370D+05
coeff(1542)=-0.153527330496D+04
coeff(1543)=0.566484160241D+04
coeff(1544)=-0.683543226750D+04
coeff(1545)=0.887705416422D+04
coeff(1546)=-0.975766555954D+04
coeff(1547)=-0.445625563656D+04
coeff(1548)=0.360709779834D+04
coeff(1549)=-0.403128179178D+04
coeff(1550)=-0.477238585326D+04
coeff(1551)=0.541367550109D+04
coeff(1552)=0.228257777726D+04
coeff(1553)=-0.122171771864D+05
coeff(1554)=-0.259629621848D+03
coeff(1555)=-0.524094496805D+03
coeff(1556)=-0.619454422217D+04
coeff(1557)=0.264640632535D+04
coeff(1558)=-0.231888231670D+03
coeff(1559)=0.606417214752D+04
coeff(1560)=0.741457735439D+04
coeff(1561)=-0.301698054162D+04
coeff(1562)=-0.836704337136D+04
coeff(1563)=0.147321318994D+04
coeff(1564)=0.364237458740D+04
coeff(1565)=-0.518467287940D+04
coeff(1566)=0.116695543626D+04
coeff(1567)=0.282266433427D+03
coeff(1568)=-0.167059012846D+05
coeff(1569)=-0.597943620623D+04
coeff(1570)=-0.254234628327D+03
coeff(1571)=-0.116630380795D+05
coeff(1572)=-0.541939870989D+04
coeff(1573)=-0.583037650003D+04
coeff(1574)=0.251789579137D+04
coeff(1575)=-0.519514152788D+03
coeff(1576)=0.117430651965D+04
coeff(1577)=-0.630178539281D+04
coeff(1578)=-0.126293151820D+04
coeff(1579)=-0.325658002551D+04
coeff(1580)=0.921048820863D+02
coeff(1581)=0.113400619427D+05
coeff(1582)=0.494461762372D+04
coeff(1583)=-0.114344879171D+05
coeff(1584)=-0.219705409885D+04
coeff(1585)=-0.741960906091D+04
coeff(1586)=0.514386939980D+03
coeff(1587)=-0.737800213509D+04
coeff(1588)=-0.261254080918D+04
coeff(1589)=0.658252609017D+04
coeff(1590)=0.121060342775D+05
coeff(1591)=0.455313390400D+04
coeff(1592)=-0.144692429904D+04
coeff(1593)=0.700482473951D+04
coeff(1594)=-0.102641891514D+05
coeff(1595)=0.643540711175D+04
coeff(1596)=0.880011890010D+04
coeff(1597)=0.437495119590D+03
coeff(1598)=-0.115465439481D+05
coeff(1599)=-0.287558146319D+04
coeff(1600)=-0.617568600771D+04
coeff(1601)=0.326172274873D+04
coeff(1602)=-0.240810867664D+03
coeff(1603)=-0.109197223212D+05
coeff(1604)=0.754011686957D+04
coeff(1605)=0.510379888754D+04
coeff(1606)=-0.584453528775D+04
coeff(1607)=0.293001634989D+04
coeff(1608)=-0.101532368694D+05
coeff(1609)=-0.491964130047D+04
coeff(1610)=-0.178342245771D+05
coeff(1611)=-0.234802138237D+03
coeff(1612)=-0.679628058148D+04
coeff(1613)=0.444146213824D+04
coeff(1614)=0.180638989507D+05
coeff(1615)=-0.199388832456D+04
coeff(1616)=0.412452858602D+04
coeff(1617)=-0.696358988372D+03
coeff(1618)=-0.477378715792D+04
coeff(1619)=-0.170494705636D+05
coeff(1620)=-0.758921746321D+04
coeff(1621)=0.552978540710D+04
coeff(1622)=0.117435896090D+04
coeff(1623)=-0.389094773607D+03
coeff(1624)=-0.477425035260D+04
coeff(1625)=0.309615148623D+04
coeff(1626)=0.650678347775D+04
coeff(1627)=-0.278819188335D+04
coeff(1628)=-0.146259150452D+05
coeff(1629)=-0.658677755998D+04
coeff(1630)=-0.235355749962D+05
coeff(1631)=-0.528229205415D+04
coeff(1632)=-0.319806565492D+04
coeff(1633)=0.688537029994D+04
coeff(1634)=-0.368592477906D+04
coeff(1635)=0.762338519689D+04
coeff(1636)=-0.137192383053D+04
coeff(1637)=-0.660957262804D+04
coeff(1638)=0.114599912712D+05
coeff(1639)=-0.859238160860D+04
coeff(1640)=0.207009519650D+04
coeff(1641)=0.128044378874D+05
coeff(1642)=0.252078792907D+04
coeff(1643)=-0.689664802868D+04
coeff(1644)=-0.331427953975D+03
coeff(1645)=0.145449840145D+05
coeff(1646)=-0.358281937315D+04
coeff(1647)=-0.813167222507D+04
coeff(1648)=0.192381698995D+04
coeff(1649)=-0.824371175361D+04
coeff(1650)=-0.645177555174D+04
coeff(1651)=-0.847465147938D+03
coeff(1652)=-0.121442166194D+05
coeff(1653)=0.889008053490D+03
coeff(1654)=-0.120327880786D+05
coeff(1655)=0.473745587074D+04
coeff(1656)=0.141046293558D+04
coeff(1657)=-0.112060445372D+03
coeff(1658)=0.620265622084D+04
coeff(1659)=-0.146175442743D+04
coeff(1660)=0.122276438802D+05
coeff(1661)=-0.839236572534D+04
coeff(1662)=0.622078475631D+03
coeff(1663)=0.189320948375D+04
coeff(1664)=-0.108216661868D+04
coeff(1665)=-0.803437929993D+04
coeff(1666)=-0.732183383671D+04
coeff(1667)=-0.137349608644D+05
coeff(1668)=-0.441838884492D+04
coeff(1669)=0.178209214376D+05
coeff(1670)=0.334235845730D+03
coeff(1671)=0.205627268700D+04
coeff(1672)=0.104975793713D+05
coeff(1673)=-0.919718324080D+04
coeff(1674)=0.937842298893D+04
coeff(1675)=0.135901985345D+05
coeff(1676)=0.129810850286D+05
coeff(1677)=0.123661470504D+05
coeff(1678)=0.259727059350D+04
coeff(1679)=0.244147622220D+04
coeff(1680)=-0.554131392463D+04
coeff(1681)=0.110910792093D+04
coeff(1682)=0.312731184272D+04
coeff(1683)=0.220255538104D+04
coeff(1684)=0.518412221615D+04
coeff(1685)=0.116050277680D+04
coeff(1686)=0.125506496645D+04
coeff(1687)=0.631420884731D+04
coeff(1688)=0.223418709410D+04
coeff(1689)=-0.290830886977D+03
coeff(1690)=0.118645923843D+05
coeff(1691)=-0.332349034232D+04
coeff(1692)=0.217415438107D+04
coeff(1693)=0.132045229634D+05
coeff(1694)=0.103288940231D+05
coeff(1695)=-0.291474515219D+04
coeff(1696)=-0.179624069189D+04
coeff(1697)=-0.475754141866D+04
coeff(1698)=-0.136786497046D+05
coeff(1699)=-0.596672865019D+04
coeff(1700)=-0.970082518174D+04
coeff(1701)=-0.554182752630D+03
coeff(1702)=-0.174976998778D+04
coeff(1703)=0.281739051468D+04
coeff(1704)=0.344762073607D+03
coeff(1705)=0.154026061322D+04
coeff(1706)=-0.430365110010D+04
coeff(1707)=-0.160604371928D+04
coeff(1708)=0.377939007713D+04
coeff(1709)=-0.836799157820D+04
coeff(1710)=0.110768398612D+04
coeff(1711)=0.547501726882D+04
coeff(1712)=-0.129280612421D+04
coeff(1713)=-0.524959139234D+04
coeff(1714)=0.632607144280D+04
coeff(1715)=-0.205774553284D+04
coeff(1716)=0.150889695978D+04
coeff(1717)=-0.924446357412D+03
coeff(1718)=0.102268032630D+04
coeff(1719)=-0.564873003248D+03
coeff(1720)=0.161239632144D+04
coeff(1721)=0.378782933300D+04
coeff(1722)=0.175448871291D+04
coeff(1723)=0.283969567928D+04
coeff(1724)=-0.759229804886D+04
coeff(1725)=-0.309161993012D+04
coeff(1726)=0.103864208928D+05
coeff(1727)=0.807875898991D+04
coeff(1728)=-0.486439190665D+04
coeff(1729)=0.182697654556D+04
coeff(1730)=-0.315418188227D+04
coeff(1731)=0.663939540515D+04
coeff(1732)=0.127233730457D+04
coeff(1733)=0.316306673807D+04
coeff(1734)=0.144727409003D+04
coeff(1735)=-0.902776100429D+03
coeff(1736)=0.456993915396D+04
coeff(1737)=-0.242237422306D+04
coeff(1738)=-0.206461511666D+05
coeff(1739)=-0.931458521748D+04
coeff(1740)=0.723029124580D+04
coeff(1741)=0.290597400685D+03
coeff(1742)=-0.627100307835D+04
coeff(1743)=-0.100460669801D+04
coeff(1744)=0.626462650985D+04
coeff(1745)=0.205230713004D+04
coeff(1746)=-0.385840376210D+04
coeff(1747)=-0.447468957449D+04
coeff(1748)=0.257483960600D+04
coeff(1749)=0.213991776789D+05
coeff(1750)=0.762369630988D+03
coeff(1751)=-0.113465264308D+04
coeff(1752)=0.625081120488D+04
coeff(1753)=0.883023015006D+04
coeff(1754)=-0.920862491463D+04
coeff(1755)=-0.238459332674D+04
coeff(1756)=-0.400076139119D+04
coeff(1757)=-0.826336930354D+04
coeff(1758)=-0.867330315247D+04
coeff(1759)=-0.515397494918D+04
coeff(1760)=-0.330852339056D+04
coeff(1761)=0.189217260920D+04
coeff(1762)=0.690786718500D+04
coeff(1763)=0.540918817852D+04
coeff(1764)=-0.541136281602D+03
coeff(1765)=0.543816641647D+04
coeff(1766)=0.135302848929D+05
coeff(1767)=0.751952631983D+04
coeff(1768)=0.111208286977D+05
coeff(1769)=0.470636984761D+04
coeff(1770)=-0.947047861329D+04
coeff(1771)=-0.357221112814D+04
coeff(1772)=-0.443636779394D+04
coeff(1773)=-0.222818531708D+04
coeff(1774)=0.386283748330D+04
coeff(1775)=0.775397852809D+03
coeff(1776)=0.841598131928D+04
coeff(1777)=-0.608844538971D+04
coeff(1778)=0.464540176461D+04
coeff(1779)=0.789910089632D+04
coeff(1780)=0.103174283257D+04
coeff(1781)=0.864266182495D+03
coeff(1782)=-0.355202871939D+04
coeff(1783)=-0.165224007536D+04
coeff(1784)=0.692711571970D+03
coeff(1785)=0.247182216301D+04
coeff(1786)=-0.120141541059D+04
coeff(1787)=0.671131551824D+03
coeff(1788)=-0.114586282934D+04
coeff(1789)=0.650719507430D+04
coeff(1790)=-0.905077506415D+03
coeff(1791)=0.851866620263D+04
coeff(1792)=-0.174436618603D+04
coeff(1793)=0.748210542782D+03
coeff(1794)=-0.158441247803D+05
coeff(1795)=0.230140069529D+04
coeff(1796)=-0.967173484099D+04
coeff(1797)=-0.135542460299D+04
coeff(1798)=-0.708952439356D+04
coeff(1799)=-0.100848777747D+05
coeff(1800)=-0.113755804272D+05
coeff(1801)=0.680457621951D+04
coeff(1802)=-0.731670466936D+04
coeff(1803)=0.802769502803D+04
coeff(1804)=0.903158393929D+03
coeff(1805)=0.190500025234D+04
coeff(1806)=-0.200672309248D+04
coeff(1807)=0.350807983163D+03
coeff(1808)=-0.107807773526D+05
coeff(1809)=0.100943939565D+05
coeff(1810)=-0.372286132836D+04
coeff(1811)=0.133490032418D+05
coeff(1812)=0.907597671738D+04
coeff(1813)=0.109516949830D+05
coeff(1814)=-0.320051372419D+04
coeff(1815)=-0.861573556228D+04
coeff(1816)=-0.894527035435D+04
coeff(1817)=0.809470931679D+03
coeff(1818)=0.419589029333D+04
coeff(1819)=-0.477500369138D+04
coeff(1820)=-0.105269437146D+04
coeff(1821)=0.457102748122D+04
coeff(1822)=-0.740134144617D+03
coeff(1823)=0.106109749345D+05
coeff(1824)=-0.232704888211D+04
coeff(1825)=-0.772609440690D+04
coeff(1826)=-0.380748979624D+04
coeff(1827)=0.506113586150D+04
coeff(1828)=0.638717219194D+04
coeff(1829)=-0.105039462022D+04
coeff(1830)=0.754448044687D+04
coeff(1831)=0.609239430727D+04
coeff(1832)=-0.925475859581D+04
coeff(1833)=0.711364124899D+04
coeff(1834)=-0.119947715791D+05
coeff(1835)=0.231051321230D+04
coeff(1836)=-0.363575443930D+04
coeff(1837)=-0.373504736818D+04
coeff(1838)=0.394659600657D+04
coeff(1839)=0.841171018794D+04
coeff(1840)=0.297050140394D+04
coeff(1841)=-0.496951854472D+03
coeff(1842)=0.173409595632D+04
coeff(1843)=0.109439472037D+05
coeff(1844)=0.330157405894D+04
coeff(1845)=0.343940351365D+04
coeff(1846)=-0.306212009289D+04
coeff(1847)=-0.897313866472D+03
coeff(1848)=-0.187511900413D+04
coeff(1849)=-0.619390981555D+04
coeff(1850)=0.131607764016D+05
coeff(1851)=0.100958947011D+05
coeff(1852)=0.325098596316D+04
coeff(1853)=0.657712662279D+04
coeff(1854)=-0.690554153754D+04
coeff(1855)=0.109415146422D+04
coeff(1856)=0.722106462168D+04
coeff(1857)=-0.467014386721D+04
coeff(1858)=-0.661038525477D+04
coeff(1859)=-0.636451840079D+04
coeff(1860)=-0.895418453741D+04
coeff(1861)=0.306204702260D+04
coeff(1862)=0.493504550538D+04
coeff(1863)=-0.463527358110D+03
coeff(1864)=-0.263402618634D+04
coeff(1865)=0.923143600573D+04
coeff(1866)=0.865746412922D+04
coeff(1867)=0.952668077449D+04
coeff(1868)=-0.431583681897D+04
coeff(1869)=0.422243140232D+04
coeff(1870)=0.565084729049D+04
coeff(1871)=-0.418594917854D+04
coeff(1872)=0.139672610496D+04
coeff(1873)=0.598816692796D+04
coeff(1874)=0.953216173475D+04
coeff(1875)=-0.402111070886D+04
coeff(1876)=0.124964280346D+04
coeff(1877)=-0.220606217222D+04
coeff(1878)=-0.400513254165D+04
coeff(1879)=-0.285729090251D+04
coeff(1880)=-0.117210172472D+05
coeff(1881)=-0.507016218372D+04
coeff(1882)=0.952980152954D+04
coeff(1883)=0.872532052631D+03
coeff(1884)=-0.137870597653D+05
coeff(1885)=0.264019350745D+04
coeff(1886)=0.381586854384D+03
coeff(1887)=-0.676674724743D+04
coeff(1888)=-0.110069009282D+04
coeff(1889)=0.273784201076D+03
coeff(1890)=-0.863640385109D+04
coeff(1891)=-0.537710024275D+04
coeff(1892)=-0.571668160537D+04
coeff(1893)=-0.372324130898D+03
coeff(1894)=-0.133245852249D+05
coeff(1895)=0.495270703532D+03
coeff(1896)=0.323542881333D+04
coeff(1897)=-0.240252950564D+04
coeff(1898)=-0.220455528345D+03
coeff(1899)=-0.701746618267D+04
coeff(1900)=0.110279124302D+05
coeff(1901)=0.949495090969D+04
coeff(1902)=-0.790138926952D+03
coeff(1903)=0.995774892314D+04
coeff(1904)=0.119522868927D+05
coeff(1905)=0.227102675691D+04
coeff(1906)=0.143145982183D+04
coeff(1907)=0.114174171480D+05
coeff(1908)=0.164183894190D+03
coeff(1909)=0.379041913690D+03
coeff(1910)=0.505859850858D+04
coeff(1911)=0.118172197318D+04
coeff(1912)=0.163723826050D+05
coeff(1913)=-0.647567130722D+03
coeff(1914)=0.184340309760D+04
coeff(1915)=0.973824793450D+04
coeff(1916)=0.100732418276D+04
coeff(1917)=-0.293065275316D+04
coeff(1918)=-0.908171107466D+04
coeff(1919)=0.838781994273D+04
coeff(1920)=0.834157866639D+04
coeff(1921)=-0.146033906366D+04
coeff(1922)=0.278369007651D+04
coeff(1923)=-0.959661801944D+04
coeff(1924)=0.730966518788D+04
coeff(1925)=-0.219376094343D+04
coeff(1926)=0.774174936274D+04
coeff(1927)=0.362344973512D+04
coeff(1928)=-0.738972445507D+04
coeff(1929)=0.802046201133D+04
coeff(1930)=-0.274235884235D+04
coeff(1931)=0.468618162255D+04
coeff(1932)=-0.274372700096D+04
coeff(1933)=0.654053391660D+04
coeff(1934)=0.206501900451D+03
coeff(1935)=-0.835530467568D+04
coeff(1936)=-0.859722906201D+04
coeff(1937)=0.116529369266D+04
coeff(1938)=0.125856471475D+05
coeff(1939)=-0.247813621150D+04
coeff(1940)=-0.939202063690D+04
coeff(1941)=0.635450621055D+04
coeff(1942)=-0.394819945693D+04
coeff(1943)=0.165023111358D+05
coeff(1944)=0.322213592327D+04
coeff(1945)=-0.249331028160D+04
coeff(1946)=0.108589293281D+05
coeff(1947)=-0.114805088261D+05
coeff(1948)=0.621511406300D+04
coeff(1949)=0.391378484139D+04
coeff(1950)=-0.470434339916D+03
coeff(1951)=-0.138644559460D+05
coeff(1952)=0.465858460434D+04
coeff(1953)=-0.383570922327D+04
coeff(1954)=-0.497103693905D+04
coeff(1955)=-0.578524487655D+04
coeff(1956)=0.800508737552D+04
coeff(1957)=-0.379643445550D+03
coeff(1958)=0.408600889939D+04
coeff(1959)=-0.497673469819D+04
coeff(1960)=-0.583443598729D+04
coeff(1961)=-0.133918931950D+05
coeff(1962)=-0.957332790580D+04
coeff(1963)=0.310071563419D+04
coeff(1964)=-0.104703818055D+04
coeff(1965)=-0.585447455440D+04
coeff(1966)=-0.290751199991D+04
coeff(1967)=0.700906563998D+04
coeff(1968)=-0.117864971304D+05
coeff(1969)=0.177307948324D+04
coeff(1970)=0.533435158910D+03
coeff(1971)=-0.852120062992D+04
coeff(1972)=-0.920078524800D+03
coeff(1973)=-0.991652055589D+03
coeff(1974)=-0.144964334376D+04
coeff(1975)=0.609352667883D+04
coeff(1976)=-0.583945056103D+04
coeff(1977)=-0.365633235479D+04
coeff(1978)=-0.508764077184D+04
coeff(1979)=-0.101249791770D+05
coeff(1980)=-0.168829638904D+05
coeff(1981)=0.177486408008D+05
coeff(1982)=0.126746839309D+05
coeff(1983)=-0.985920121289D+04
coeff(1984)=0.195814550747D+04
coeff(1985)=0.688737159122D+04
coeff(1986)=0.446507357689D+04
coeff(1987)=0.320471350708D+04
coeff(1988)=0.938329481080D+04
coeff(1989)=0.217096129837D+04
coeff(1990)=0.718302795592D+04
coeff(1991)=-0.287717621969D+04
coeff(1992)=0.631879507384D+04
coeff(1993)=0.388390159008D+04
coeff(1994)=-0.667810455235D+03
coeff(1995)=0.374206593847D+03
coeff(1996)=-0.609848669437D+04
coeff(1997)=-0.886993585068D+04
coeff(1998)=-0.899530355882D+04
coeff(1999)=0.367366107626D+04
coeff(2000)=0.582310872967D+04
coeff(2001)=0.155287183586D+05
coeff(2002)=-0.483058353476D+04
coeff(2003)=-0.172088230390D+04
coeff(2004)=0.255788495859D+04
coeff(2005)=-0.677070147579D+04
coeff(2006)=-0.216352227632D+05
coeff(2007)=-0.509350755403D+04
coeff(2008)=0.232876880114D+03
coeff(2009)=-0.110202389097D+05
coeff(2010)=0.523717329935D+04
coeff(2011)=0.594002971909D+04
coeff(2012)=-0.512599984662D+04
coeff(2013)=-0.576642068912D+04
coeff(2014)=0.110634328257D+05
coeff(2015)=-0.411915655898D+04
coeff(2016)=-0.161157984040D+05
coeff(2017)=0.869862405023D+04
coeff(2018)=0.552721128178D+04
coeff(2019)=0.975138318264D+04
coeff(2020)=-0.450412911639D+04
coeff(2021)=-0.814345485838D+04
coeff(2022)=-0.347968325155D+04
coeff(2023)=-0.264650983984D+04
coeff(2024)=0.112564778490D+05
coeff(2025)=-0.140236747732D+04
coeff(2026)=-0.252628711331D+04
coeff(2027)=-0.539940451963D+03
coeff(2028)=0.101317832882D+05
coeff(2029)=-0.536131849015D+04
coeff(2030)=0.101615492620D+04
coeff(2031)=-0.820078529633D+04
coeff(2032)=-0.511724051109D+04
coeff(2033)=-0.634273954920D+04
coeff(2034)=-0.142540331018D+05
coeff(2035)=0.140584263335D+05
coeff(2036)=0.404255816662D+04
coeff(2037)=0.191533110831D+05
coeff(2038)=-0.124196189482D+05
coeff(2039)=-0.117085019131D+04
coeff(2040)=0.787805200994D+04
coeff(2041)=0.723109380276D+04
coeff(2042)=0.355903714518D+04
coeff(2043)=0.118187026527D+05
coeff(2044)=-0.550167068428D+04
coeff(2045)=-0.469403266370D+04
coeff(2046)=0.717362427703D+04
coeff(2047)=-0.573039806408D+04
coeff(2048)=0.916924572609D+04
coeff(2049)=-0.298793143147D+04
coeff(2050)=-0.900881914669D+04
coeff(2051)=-0.531243469540D+04
coeff(2052)=0.197986980078D+04
coeff(2053)=0.345153779926D+04
coeff(2054)=0.199200484062D+05
coeff(2055)=0.246206817711D+04
coeff(2056)=-0.151574629638D+05
coeff(2057)=0.423755231106D+04
coeff(2058)=-0.449439593107D+04
coeff(2059)=0.378016014610D+04
coeff(2060)=-0.356434124441D+04
coeff(2061)=-0.121472732282D+04
coeff(2062)=-0.231792659968D+05
coeff(2063)=0.188959001341D+04
coeff(2064)=0.161420609754D+04
coeff(2065)=0.925165647553D+04
coeff(2066)=0.808005001104D+04
coeff(2067)=0.209168080365D+04
coeff(2068)=0.239957787592D+04
coeff(2069)=-0.391453998197D+04
coeff(2070)=0.179379979202D+04
coeff(2071)=-0.735048033411D+04
coeff(2072)=0.177005192909D+04
coeff(2073)=0.106887669421D+05
coeff(2074)=0.822846970005D+04
coeff(2075)=-0.337722832271D+04
coeff(2076)=-0.585607385908D+03
coeff(2077)=0.653842550586D+04
coeff(2078)=0.215590092812D+05
coeff(2079)=-0.203821152337D+05
coeff(2080)=0.235927477067D+05
coeff(2081)=0.491522847310D+04
coeff(2082)=0.366112261353D+04
coeff(2083)=0.407715714826D+04
coeff(2084)=0.265609375708D+04
coeff(2085)=-0.674951561045D+04
coeff(2086)=-0.162809003226D+04
coeff(2087)=0.315707916903D+04
coeff(2088)=-0.104016235853D+05
coeff(2089)=0.123904382347D+05
coeff(2090)=0.148677712399D+04
coeff(2091)=0.125801917689D+05
coeff(2092)=0.367899809351D+04
coeff(2093)=0.560790104249D+04
coeff(2094)=-0.795846405960D+04
coeff(2095)=-0.123460297909D+05
coeff(2096)=0.323341287776D+04
coeff(2097)=-0.744438200478D+04
coeff(2098)=-0.992335305707D+04
coeff(2099)=-0.933167177964D+04
coeff(2100)=-0.106119734496D+05
coeff(2101)=-0.273099588987D+04
coeff(2102)=-0.167685904055D+05
coeff(2103)=0.412773531630D+04
coeff(2104)=-0.442060815724D+04
coeff(2105)=-0.163563007885D+04
coeff(2106)=0.277422149718D+04
coeff(2107)=-0.716338790434D+04
coeff(2108)=-0.394234718139D+04
coeff(2109)=-0.582777779492D+03
coeff(2110)=0.567950472757D+03
coeff(2111)=0.742134653118D+04
coeff(2112)=0.512374390697D+04
coeff(2113)=-0.182366435933D+05
coeff(2114)=-0.667875440266D+04
coeff(2115)=0.679975673315D+04
coeff(2116)=-0.621727896049D+04
coeff(2117)=-0.567299958567D+04
coeff(2118)=0.676532699382D+03
coeff(2119)=-0.109310691898D+03
coeff(2120)=0.176083557732D+03
coeff(2121)=0.640655528251D+04
coeff(2122)=-0.116225324406D+05
coeff(2123)=0.108631707934D+05
coeff(2124)=0.108096053288D+05
coeff(2125)=-0.581540885282D+04
coeff(2126)=-0.121105500540D+05
coeff(2127)=0.138614872646D+04
coeff(2128)=-0.992239980401D+04
coeff(2129)=0.928780380839D+04
coeff(2130)=0.978326158725D+04
coeff(2131)=0.108471766816D+05
coeff(2132)=0.220106484402D+04
coeff(2133)=0.108848913419D+05
coeff(2134)=0.116762843436D+05
coeff(2135)=-0.124386205018D+05
coeff(2136)=-0.759169589508D+04
coeff(2137)=-0.667680709719D+04
coeff(2138)=-0.799433572837D+04
coeff(2139)=0.852730031718D+04
coeff(2140)=-0.976943715084D+04
coeff(2141)=0.667197544214D+04
coeff(2142)=0.116182047953D+05
coeff(2143)=0.618750540201D+04
coeff(2144)=-0.564999075578D+04
coeff(2145)=-0.110114992110D+05
coeff(2146)=-0.573129867333D+03
coeff(2147)=-0.519676347614D+04
coeff(2148)=0.170337030493D+04
coeff(2149)=0.110710568494D+05
coeff(2150)=0.198714974372D+04
coeff(2151)=-0.695600058780D+03
coeff(2152)=-0.560808189160D+04
coeff(2153)=-0.130006963919D+05
coeff(2154)=0.109325894033D+05
coeff(2155)=0.189437644251D+03
coeff(2156)=-0.800346488176D+04
coeff(2157)=-0.234383331032D+04
coeff(2158)=-0.116330997435D+05
coeff(2159)=0.352767327559D+04
coeff(2160)=0.219113288168D+04
coeff(2161)=-0.893822139646D+04
coeff(2162)=0.114105342719D+05
coeff(2163)=0.319799285776D+04
coeff(2164)=-0.330077781573D+04
coeff(2165)=0.861115514457D+04
coeff(2166)=-0.719845405123D+04
coeff(2167)=-0.264442295453D+04
coeff(2168)=0.886885410470D+04
coeff(2169)=-0.106139525506D+04
coeff(2170)=-0.596523740065D+03
coeff(2171)=-0.845386688668D+03
coeff(2172)=-0.640260355739D+04
coeff(2173)=0.668184119493D+04
coeff(2174)=-0.476779936382D+04
coeff(2175)=0.557695177828D+04
coeff(2176)=0.649752019288D+04
coeff(2177)=0.655867722357D+04
coeff(2178)=0.465203844201D+04
coeff(2179)=-0.499718541289D+04
coeff(2180)=0.524702329220D+04
coeff(2181)=-0.418867912259D+04
coeff(2182)=-0.176667978336D+03
coeff(2183)=0.389487410266D+04
coeff(2184)=-0.458300032274D+03
coeff(2185)=0.554724286253D+04
coeff(2186)=-0.486206524050D+04
coeff(2187)=-0.308515585963D+04
coeff(2188)=0.138617206513D+04
coeff(2189)=0.340312069260D+04
coeff(2190)=0.336473176990D+04
coeff(2191)=-0.558696348721D+04
coeff(2192)=0.182310244338D+04
coeff(2193)=-0.106930233945D+05
coeff(2194)=0.167706681919D+04
coeff(2195)=0.390783680877D+04
coeff(2196)=0.637953872749D+04
coeff(2197)=0.406727393812D+04
coeff(2198)=0.587840058706D+04
coeff(2199)=-0.735836615977D+02
coeff(2200)=-0.332261694477D+04
coeff(2201)=0.194645857720D+05
coeff(2202)=-0.110762387283D+04
coeff(2203)=-0.160946877014D+05
coeff(2204)=0.340772798413D+04
coeff(2205)=0.486796895062D+03
coeff(2206)=-0.773103448452D+04
coeff(2207)=-0.555683367022D+04
coeff(2208)=-0.738425444944D+04
coeff(2209)=-0.120044867984D+05
coeff(2210)=0.180268483773D+04
coeff(2211)=-0.352003075178D+04
coeff(2212)=-0.168897979771D+05
coeff(2213)=-0.776739298139D+04
coeff(2214)=-0.413648812493D+04
coeff(2215)=-0.126373815879D+04
coeff(2216)=0.180534732004D+04
coeff(2217)=0.855509806468D+04
coeff(2218)=0.274966555502D+04
coeff(2219)=-0.116206915711D+05
coeff(2220)=0.284092664692D+03
coeff(2221)=0.651655204190D+04
coeff(2222)=0.106456472132D+04
coeff(2223)=-0.379239132023D+04
coeff(2224)=0.881858753505D+04
coeff(2225)=-0.284278932944D+03
coeff(2226)=0.271838894373D+04
coeff(2227)=-0.312116685888D+04
coeff(2228)=-0.608991232805D+04
coeff(2229)=-0.480581211440D+04
coeff(2230)=-0.662752460156D+04
coeff(2231)=-0.287080085273D+03
coeff(2232)=-0.283700826994D+04
coeff(2233)=0.244556664418D+04
coeff(2234)=-0.684809169353D+04
coeff(2235)=-0.192682407156D+04
coeff(2236)=-0.362513316172D+04
coeff(2237)=-0.602700574208D+04
coeff(2238)=0.736838292496D+04
coeff(2239)=0.843075322202D+04
coeff(2240)=-0.526550413479D+04
coeff(2241)=0.854361148400D+04
coeff(2242)=0.722424102577D+04
coeff(2243)=-0.741921003710D+04
coeff(2244)=0.323873879230D+04
coeff(2245)=0.771388977350D+04
coeff(2246)=0.172342731981D+05
coeff(2247)=-0.135756862345D+04
coeff(2248)=-0.983104180056D+04
coeff(2249)=-0.647572065397D+04
coeff(2250)=0.258492420569D+04
coeff(2251)=-0.439505717989D+03
coeff(2252)=-0.230294564247D+03
coeff(2253)=-0.843944849732D+03
coeff(2254)=0.460058070183D+04
coeff(2255)=0.108144968094D+04
coeff(2256)=-0.841700066764D+03
coeff(2257)=-0.415302271310D+04
coeff(2258)=-0.176376882608D+05
coeff(2259)=0.849647652564D+04
coeff(2260)=-0.148602446268D+04
coeff(2261)=-0.167400258742D+05
coeff(2262)=0.190589390211D+05
coeff(2263)=-0.290739419386D+04
coeff(2264)=0.192473545901D+05
coeff(2265)=0.122318412359D+05
coeff(2266)=0.673751646134D+03
coeff(2267)=0.108587473939D+05
coeff(2268)=-0.489800212989D+04
coeff(2269)=0.125859751037D+05
coeff(2270)=-0.626365224458D+04
coeff(2271)=-0.914656190823D+04
coeff(2272)=0.734355882088D+04
coeff(2273)=-0.374309424915D+04
coeff(2274)=0.155064319292D+05
coeff(2275)=0.635369872989D+04
coeff(2276)=0.950219454627D+04
coeff(2277)=0.107214767996D+04
coeff(2278)=0.509469781668D+04
coeff(2279)=0.588660734311D+04
coeff(2280)=-0.634597235213D+04
coeff(2281)=0.107361112844D+04
coeff(2282)=0.378439528480D+04
coeff(2283)=0.258145641724D+04
coeff(2284)=-0.224282719819D+04
coeff(2285)=-0.130056640054D+05
coeff(2286)=0.229081546098D+04
coeff(2287)=0.989220336444D+04
coeff(2288)=-0.855236049668D+03
coeff(2289)=-0.450689588446D+04
coeff(2290)=-0.402580219426D+04
coeff(2291)=0.873352639281D+04
coeff(2292)=0.292021895776D+04
coeff(2293)=-0.334618794698D+04
coeff(2294)=0.377849128660D+04
coeff(2295)=0.847019061075D+04
coeff(2296)=-0.368487811133D+04
coeff(2297)=0.219662894226D+04
coeff(2298)=0.189071996754D+04
coeff(2299)=-0.159352266909D+04
coeff(2300)=0.123134803417D+04
coeff(2301)=0.921371625020D+04
coeff(2302)=-0.786583447696D+04
coeff(2303)=0.864844911222D+04
coeff(2304)=-0.613543424451D+03
coeff(2305)=-0.528789346519D+04
coeff(2306)=0.232914385778D+04
coeff(2307)=0.199379971756D+04
coeff(2308)=-0.369210334868D+04
coeff(2309)=0.415869593965D+04
coeff(2310)=0.872017415235D+03
coeff(2311)=-0.360742724988D+04
coeff(2312)=0.454108914943D+04
coeff(2313)=-0.946083537922D+03
coeff(2314)=-0.901251203045D+04
coeff(2315)=0.861147220829D+04
coeff(2316)=0.365112831765D+04
coeff(2317)=-0.512528684601D+04
coeff(2318)=-0.803104040750D+04
coeff(2319)=-0.379791949755D+04
coeff(2320)=0.367454513294D+04
coeff(2321)=-0.118730670383D+05
coeff(2322)=0.125958579993D+05
coeff(2323)=0.519192100294D+04
coeff(2324)=-0.890112196719D+04
coeff(2325)=-0.625188743975D+04
coeff(2326)=-0.395553056700D+03
coeff(2327)=-0.150426720837D+04
coeff(2328)=-0.715826591105D+04
coeff(2329)=0.229550909297D+04
coeff(2330)=-0.102692121949D+05
coeff(2331)=-0.568171347380D+04
coeff(2332)=-0.208886181342D+05
coeff(2333)=0.113966677934D+05
coeff(2334)=-0.107289804135D+05
coeff(2335)=-0.344484176099D+04
coeff(2336)=0.434480872295D+04
coeff(2337)=-0.142869203668D+04
coeff(2338)=-0.860957215620D+03
coeff(2339)=-0.706243033780D+04
coeff(2340)=-0.729087517429D+04
coeff(2341)=0.136055969607D+05
coeff(2342)=0.992611075205D+03
coeff(2343)=0.452923430855D+04
coeff(2344)=0.263997949113D+04
coeff(2345)=0.107555702302D+05
coeff(2346)=0.849747976479D+03
coeff(2347)=0.157159041318D+04
coeff(2348)=0.104715725197D+04
coeff(2349)=0.160087925850D+05
coeff(2350)=0.307948976109D+04
coeff(2351)=0.535255951244D+04
coeff(2352)=-0.496747686721D+04
coeff(2353)=-0.566985652253D+04
coeff(2354)=-0.324601937656D+04
coeff(2355)=0.664594080008D+04
coeff(2356)=-0.701898359720D+04
coeff(2357)=0.236996657683D+03
coeff(2358)=-0.106417256151D+05
coeff(2359)=-0.108537208600D+05
coeff(2360)=-0.155858035272D+05
coeff(2361)=0.107083728938D+04
coeff(2362)=0.439427962371D+03
coeff(2363)=0.147943098384D+05
coeff(2364)=-0.164401458008D+05
coeff(2365)=-0.171875002083D+05
coeff(2366)=-0.241531104548D+03
coeff(2367)=-0.452856042792D+04
coeff(2368)=0.335542068088D+04
coeff(2369)=-0.542367378784D+04
coeff(2370)=0.152312503883D+05
coeff(2371)=-0.113789425821D+04
coeff(2372)=-0.406262099688D+04
coeff(2373)=0.233081762042D+04
coeff(2374)=-0.258867020762D+04
coeff(2375)=-0.738775237911D+04
coeff(2376)=-0.510571080076D+04
coeff(2377)=0.328761587604D+04
coeff(2378)=-0.219133824383D+04
coeff(2379)=-0.118065164302D+05
coeff(2380)=-0.351245238817D+04
coeff(2381)=-0.187996365027D+04
coeff(2382)=0.212447110850D+05
coeff(2383)=-0.646548258069D+04
coeff(2384)=0.107038674808D+05
coeff(2385)=-0.407548342516D+03
coeff(2386)=-0.369121156707D+04
coeff(2387)=0.191721102729D+05
coeff(2388)=0.271952422134D+04
coeff(2389)=0.651244103875D+04
coeff(2390)=0.231428321823D+04
coeff(2391)=0.987920943463D+02
coeff(2392)=-0.186687011609D+04
coeff(2393)=0.601492385707D+04
coeff(2394)=-0.300858477519D+04
coeff(2395)=0.930781574875D+03
coeff(2396)=-0.109533725467D+05
coeff(2397)=-0.527415014322D+04
coeff(2398)=0.101364140928D+05
coeff(2399)=0.558615509200D+04
coeff(2400)=-0.334865257707D+04
coeff(2401)=0.399608989504D+04
coeff(2402)=-0.439303711083D+04
coeff(2403)=-0.720044116547D+04
coeff(2404)=-0.744251210889D+04
coeff(2405)=0.467671824356D+04
coeff(2406)=-0.436182040297D+03
coeff(2407)=0.125575115811D+05
coeff(2408)=0.481162995290D+04
coeff(2409)=0.445822292789D+04
coeff(2410)=0.243916437978D+04
coeff(2411)=-0.137344174720D+05
coeff(2412)=-0.249501318781D+04
coeff(2413)=-0.901469288270D+04
coeff(2414)=0.114324232279D+05
coeff(2415)=0.195289246164D+05
coeff(2416)=-0.580781641325D+04
coeff(2417)=0.146676668115D+04
coeff(2418)=-0.257216306888D+04
coeff(2419)=0.478182015489D+03
coeff(2420)=-0.797520815793D+03
coeff(2421)=-0.117136245469D+04
coeff(2422)=0.547941335741D+03
coeff(2423)=0.400085528885D+04
coeff(2424)=-0.137544872477D+03
coeff(2425)=0.158680615701D+05
coeff(2426)=0.121628548288D+05
coeff(2427)=-0.137293473350D+05
coeff(2428)=0.140113514571D+05
coeff(2429)=-0.424035566122D+03
coeff(2430)=0.404599183783D+03
coeff(2431)=0.287083099853D+04
coeff(2432)=-0.114436418330D+04
coeff(2433)=0.586238742615D+04
coeff(2434)=0.619782692185D+04
coeff(2435)=0.764900138923D+02
coeff(2436)=-0.114066219607D+04
coeff(2437)=0.624007962122D+04
coeff(2438)=0.124507086324D+05
coeff(2439)=-0.214630588513D+04
coeff(2440)=0.255773710701D+04
coeff(2441)=0.442830221626D+04
coeff(2442)=-0.143430936514D+05
coeff(2443)=0.385409827376D+04
coeff(2444)=0.472368547142D+04
coeff(2445)=0.404347927458D+04
coeff(2446)=0.559724377425D+04
coeff(2447)=-0.129295051880D+04
coeff(2448)=0.120329086245D+05
coeff(2449)=-0.215600330554D+05
coeff(2450)=0.181854522849D+05
coeff(2451)=0.635523328872D+04
coeff(2452)=-0.645725336655D+04
coeff(2453)=-0.129116103550D+05
coeff(2454)=0.864786970966D+04
coeff(2455)=-0.304891714287D+04
coeff(2456)=-0.985345893531D+04
coeff(2457)=-0.114094387710D+05
coeff(2458)=0.387218638856D+04
coeff(2459)=0.174983172001D+05
coeff(2460)=-0.826310800012D+04
coeff(2461)=0.737505104100D+04
coeff(2462)=-0.724581223587D+04
coeff(2463)=0.165206397019D+04
coeff(2464)=0.516609503380D+03
coeff(2465)=-0.203566402191D+04
coeff(2466)=0.203321381340D+04
coeff(2467)=-0.251043052753D+04
coeff(2468)=0.809138384000D+04
coeff(2469)=-0.543030253728D+04
coeff(2470)=0.114070747916D+04
coeff(2471)=-0.132633337697D+05
coeff(2472)=0.297030553365D+03
coeff(2473)=0.187489506161D+04
coeff(2474)=-0.754182950937D+04
coeff(2475)=0.209531511577D+04
coeff(2476)=0.950055034614D+04
coeff(2477)=0.122731716672D+05
coeff(2478)=-0.347977848091D+04
coeff(2479)=0.521513386393D+04
coeff(2480)=0.447543236279D+04
coeff(2481)=-0.942512343257D+04
coeff(2482)=0.843017673471D+04
coeff(2483)=-0.140832375711D+05
coeff(2484)=-0.230502543616D+04
coeff(2485)=-0.600538168977D+04
coeff(2486)=0.148851454767D+05
coeff(2487)=0.207560714570D+03
coeff(2488)=-0.830420671245D+04
coeff(2489)=-0.839912913628D+04
coeff(2490)=0.365703321987D+04
coeff(2491)=0.266296835801D+04
coeff(2492)=0.732095226084D+04
coeff(2493)=-0.867835951493D+04
coeff(2494)=-0.245951620642D+04
coeff(2495)=-0.819897292819D+04
coeff(2496)=-0.720560961358D+03
coeff(2497)=0.366013923104D+04
coeff(2498)=-0.419889945762D+04
coeff(2499)=0.277186272938D+04
coeff(2500)=0.459290651764D+04
coeff(2501)=-0.115551588365D+04
coeff(2502)=0.550752132826D+04
coeff(2503)=-0.627601304653D+04
coeff(2504)=0.470974894898D+04
coeff(2505)=-0.411730089549D+04
coeff(2506)=0.103966016257D+04
coeff(2507)=0.358590450157D+04
coeff(2508)=0.724038091102D+04
coeff(2509)=-0.879565385961D+04
coeff(2510)=0.146322365196D+05
coeff(2511)=0.273382641894D+04
coeff(2512)=-0.348094937077D+04
coeff(2513)=0.491667299003D+04
coeff(2514)=-0.596789967479D+04
coeff(2515)=0.453662580912D+04
coeff(2516)=0.413138656824D+04
coeff(2517)=-0.522448436743D+04
coeff(2518)=-0.111599595392D+05
coeff(2519)=-0.250013729394D+04
coeff(2520)=0.111937827375D+05
coeff(2521)=-0.214902624900D+04
coeff(2522)=0.703348839757D+04
coeff(2523)=-0.201739358469D+04
coeff(2524)=0.141746759283D+05
coeff(2525)=-0.394564081018D+04
coeff(2526)=-0.267745947120D+04
coeff(2527)=0.111397822948D+05
coeff(2528)=0.141888890384D+05
coeff(2529)=-0.216535842003D+04
coeff(2530)=-0.632346904770D+04
coeff(2531)=0.992758177543D+04
coeff(2532)=0.492779464016D+04
coeff(2533)=0.893557927267D+04
coeff(2534)=-0.507905047574D+02
coeff(2535)=-0.213818225778D+04
coeff(2536)=0.410479477293D+04
coeff(2537)=0.906471851023D+04
coeff(2538)=-0.113701257105D+05
coeff(2539)=0.156262402726D+04
coeff(2540)=0.543137567677D+04
coeff(2541)=-0.742491634313D+04
coeff(2542)=-0.159665141152D+05
coeff(2543)=-0.106353074454D+05
coeff(2544)=-0.154526411250D+05
coeff(2545)=0.117895674147D+04
coeff(2546)=0.246027792511D+05
coeff(2547)=0.617756663319D+04
coeff(2548)=-0.115338321686D+05
coeff(2549)=-0.435730716763D+04
coeff(2550)=0.941003635749D+04
coeff(2551)=-0.361844612470D+03
coeff(2552)=-0.443601883619D+04
coeff(2553)=0.767434338468D+04
coeff(2554)=-0.148834520060D+05
coeff(2555)=0.437097463342D+04
coeff(2556)=0.669260106317D+04
coeff(2557)=0.116277678779D+05
coeff(2558)=0.496295740845D+03
coeff(2559)=0.688037963870D+04
coeff(2560)=0.108704913734D+05
coeff(2561)=-0.120464581601D+05
coeff(2562)=-0.437999739876D+04
coeff(2563)=0.800063034293D+02
coeff(2564)=0.230840792390D+04
coeff(2565)=-0.516224190138D+04
coeff(2566)=0.680907936179D+04
coeff(2567)=-0.953795121864D+04
coeff(2568)=0.853354495267D+03
coeff(2569)=0.113882445865D+05
coeff(2570)=-0.384603081992D+04
coeff(2571)=0.508770403077D+03
coeff(2572)=-0.647364356151D+04
coeff(2573)=-0.253769122464D+04
coeff(2574)=0.186454025653D+05
coeff(2575)=0.272468159143D+04
coeff(2576)=-0.116227578261D+05
coeff(2577)=-0.569262669561D+04
coeff(2578)=-0.664906098549D+04
coeff(2579)=0.956186812112D+04
coeff(2580)=-0.105914737227D+05
coeff(2581)=0.491333007968D+04
coeff(2582)=-0.490157111881D+04
coeff(2583)=-0.897158247999D+04
coeff(2584)=-0.123115281751D+04
coeff(2585)=-0.136029928646D+05
coeff(2586)=-0.699702918790D+04
coeff(2587)=-0.485751531066D+03
coeff(2588)=-0.563563037502D+04
coeff(2589)=-0.138197011422D+03
coeff(2590)=0.428841246390D+04
coeff(2591)=-0.293535782584D+04
coeff(2592)=-0.322822491780D+04
coeff(2593)=-0.511225160223D+04
coeff(2594)=0.618738416419D+04
coeff(2595)=0.201781442739D+05
coeff(2596)=-0.175228901441D+04
coeff(2597)=-0.643131921891D+04
coeff(2598)=-0.506095194070D+04
coeff(2599)=0.139416503625D+04
coeff(2600)=-0.841489063911D+04
coeff(2601)=0.866651814487D+04
coeff(2602)=-0.801039273819D+04
coeff(2603)=-0.410336110999D+04
coeff(2604)=-0.387003778482D+04
coeff(2605)=0.113981353029D+05
coeff(2606)=0.853310332023D+04
coeff(2607)=-0.930494791942D+04
coeff(2608)=-0.144942346284D+05
coeff(2609)=-0.118132263819D+04
coeff(2610)=-0.500955974924D+04
coeff(2611)=0.549733466843D+03
coeff(2612)=-0.120207445849D+05
coeff(2613)=-0.262647704922D+04
coeff(2614)=0.483270073149D+04
coeff(2615)=-0.982408168313D+04
coeff(2616)=0.364412834538D+02
coeff(2617)=0.233958513709D+05
coeff(2618)=0.161640201198D+05
coeff(2619)=-0.603842782320D+04
coeff(2620)=0.416915365604D+04
coeff(2621)=-0.696250583501D+04
coeff(2622)=0.168254313709D+05
coeff(2623)=0.467684177152D+04
coeff(2624)=0.526800970355D+04
coeff(2625)=0.516890420447D+04
coeff(2626)=0.688168665801D+03
coeff(2627)=-0.342868569491D+04
coeff(2628)=-0.202781038294D+05
coeff(2629)=0.134765799697D+05
coeff(2630)=-0.798595641997D+04
coeff(2631)=0.247407037051D+03
coeff(2632)=-0.100280711777D+05
coeff(2633)=-0.180002639611D+04
coeff(2634)=0.488220310021D+04
coeff(2635)=0.754287805630D+04
coeff(2636)=0.255583941245D+04
coeff(2637)=0.544858651357D+04
coeff(2638)=-0.219424168565D+04
coeff(2639)=-0.434569652748D+03
coeff(2640)=-0.801794982602D+04
coeff(2641)=0.971316270959D+04
coeff(2642)=-0.126113340281D+05
coeff(2643)=0.665252171559D+04
coeff(2644)=0.988608619828D+04
coeff(2645)=-0.112728834611D+05
coeff(2646)=-0.303448288698D+04
coeff(2647)=-0.836927384050D+04
coeff(2648)=0.253979326710D+04
coeff(2649)=0.124999464793D+04
coeff(2650)=0.191101547163D+05
coeff(2651)=0.115269405097D+05
coeff(2652)=-0.501939661463D+04
coeff(2653)=-0.644269454366D+04
coeff(2654)=-0.390225894562D+04
coeff(2655)=0.114163131953D+05
coeff(2656)=0.149036106252D+05
coeff(2657)=-0.307553567111D+04
coeff(2658)=-0.614845911340D+04
coeff(2659)=-0.267402994102D+04
coeff(2660)=-0.117119325120D+05
coeff(2661)=-0.427111567293D+04
coeff(2662)=0.580266262172D+04
coeff(2663)=-0.113038067893D+05
coeff(2664)=0.110807672599D+05
coeff(2665)=-0.247014426339D+04
coeff(2666)=-0.740185887935D+03
coeff(2667)=0.663668825817D+03
coeff(2668)=0.100027879550D+05
coeff(2669)=-0.267738672406D+04
coeff(2670)=-0.514205329016D+03
coeff(2671)=-0.546092694312D+04
coeff(2672)=0.672402277676D+04
coeff(2673)=0.249576867354D+04
coeff(2674)=0.153391759903D+05
coeff(2675)=-0.883804936417D+04
coeff(2676)=0.898622938720D+04
coeff(2677)=-0.666034752917D+04
coeff(2678)=-0.118076590001D+05
coeff(2679)=-0.142106464113D+05
coeff(2680)=0.130717395110D+05
coeff(2681)=0.143747143221D+04
coeff(2682)=-0.122268855918D+04
coeff(2683)=-0.184397111321D+05
coeff(2684)=-0.268855565597D+04
coeff(2685)=-0.416091564964D+04
coeff(2686)=-0.471443915102D+04
coeff(2687)=0.150717570051D+04
coeff(2688)=0.240453291254D+04
coeff(2689)=-0.516021415276D+03
coeff(2690)=-0.133401273493D+05
coeff(2691)=0.149867432364D+05
coeff(2692)=0.792093691265D+04
coeff(2693)=0.200619849768D+04
coeff(2694)=-0.105961268973D+05
coeff(2695)=0.115671952486D+05
coeff(2696)=-0.148175504857D+05
coeff(2697)=0.997844434126D+04
coeff(2698)=0.276457351773D+04
coeff(2699)=0.106011590707D+05
coeff(2700)=-0.176501544935D+05
coeff(2701)=0.216935140962D+04
coeff(2702)=-0.964483997545D+04
coeff(2703)=-0.855674686227D+04
coeff(2704)=-0.693932479223D+04
coeff(2705)=-0.191955334296D+04
coeff(2706)=-0.886635316105D+04
coeff(2707)=0.297309363654D+04
coeff(2708)=0.737854425190D+04
coeff(2709)=0.525141147413D+04
coeff(2710)=0.206500640520D+04
coeff(2711)=-0.856491872413D+04
coeff(2712)=-0.107990427843D+05
coeff(2713)=0.577024177988D+04
coeff(2714)=-0.376982298307D+03
coeff(2715)=-0.489161778838D+04
coeff(2716)=0.102068228220D+05
coeff(2717)=-0.801093344719D+04
coeff(2718)=-0.823951892775D+04
coeff(2719)=0.490831549237D+04
coeff(2720)=-0.461674662121D+04
coeff(2721)=-0.478152439540D+03
coeff(2722)=0.580367963506D+04
coeff(2723)=0.752883751135D+04
coeff(2724)=-0.424276092846D+04
coeff(2725)=0.377668838024D+04
coeff(2726)=0.140830259337D+05
coeff(2727)=-0.690043719576D+04
coeff(2728)=-0.103276029125D+05
coeff(2729)=-0.282870195343D+04
coeff(2730)=-0.379791392172D+04
coeff(2731)=0.132695957252D+05
coeff(2732)=-0.130074590344D+05
coeff(2733)=0.527033036852D+04
coeff(2734)=0.923225667263D+04
coeff(2735)=0.601242985809D+04
coeff(2736)=-0.897144643463D+04
coeff(2737)=0.675827191001D+04
coeff(2738)=0.657740806825D+03
coeff(2739)=0.130975204598D+05
coeff(2740)=0.151559458031D+05
coeff(2741)=0.908789775830D+04
coeff(2742)=-0.108624498414D+05
coeff(2743)=0.515602252411D+04
coeff(2744)=0.746587548833D+04
coeff(2745)=0.530330752960D+04
coeff(2746)=0.104138504168D+04
coeff(2747)=0.137531045929D+04
coeff(2748)=-0.125342599733D+04
coeff(2749)=0.440748104389D+04
coeff(2750)=-0.433930292514D+04
coeff(2751)=0.345687905695D+04
coeff(2752)=-0.621746384705D+04
coeff(2753)=-0.975170654359D+04
coeff(2754)=-0.808954110299D+04
coeff(2755)=-0.155248965075D+04
coeff(2756)=0.140164418611D+05
coeff(2757)=0.519495609462D+03
coeff(2758)=-0.115634430577D+05
coeff(2759)=-0.996053060832D+03
coeff(2760)=0.127792939784D+05
coeff(2761)=-0.319736200378D+03
coeff(2762)=-0.152827348145D+04
coeff(2763)=-0.473070366299D+04
coeff(2764)=-0.952916396964D+04
coeff(2765)=0.870800577184D+04
coeff(2766)=0.428190160061D+04
coeff(2767)=0.473758784835D+04
coeff(2768)=0.373376401520D+04
coeff(2769)=-0.238746663335D+04
coeff(2770)=0.991542779059D+03
coeff(2771)=0.645560079448D+04
coeff(2772)=0.637864938861D+04
coeff(2773)=0.808715006788D+03
coeff(2774)=0.401102839405D+04
coeff(2775)=0.476783336684D+04
coeff(2776)=0.665716898419D+04
coeff(2777)=0.345066554621D+04
coeff(2778)=0.355270154514D+04
coeff(2779)=-0.469645828908D+04
coeff(2780)=0.363194879240D+04
coeff(2781)=-0.654562770718D+04
coeff(2782)=0.346383360031D+04
coeff(2783)=0.563798295485D+04
coeff(2784)=-0.140632653805D+05
coeff(2785)=-0.668179257171D+04
coeff(2786)=-0.241276782241D+05
coeff(2787)=-0.210191362557D+04
coeff(2788)=0.129502770076D+05
coeff(2789)=0.969616280747D+04
coeff(2790)=0.912284021541D+04
coeff(2791)=0.348091571048D+04
coeff(2792)=-0.399947959250D+03
coeff(2793)=-0.161360349718D+04
coeff(2794)=-0.483785947334D+04
coeff(2795)=0.869427418427D+04
coeff(2796)=0.108945451539D+05
coeff(2797)=-0.136005471616D+04
coeff(2798)=0.656381793531D+04
coeff(2799)=-0.779073740342D+04
coeff(2800)=-0.107575733691D+05
coeff(2801)=0.634434035401D+04
coeff(2802)=0.190941325587D+05
coeff(2803)=-0.150358004701D+05
coeff(2804)=-0.576037097272D+04
coeff(2805)=0.979959240746D+04
coeff(2806)=-0.743640475581D+04
coeff(2807)=-0.117981954559D+05
coeff(2808)=-0.552676529709D+04
coeff(2809)=0.136045537443D+05
coeff(2810)=-0.197732741965D+04
coeff(2811)=-0.128546638169D+04
coeff(2812)=0.515165603094D+04
coeff(2813)=-0.378277270360D+04
coeff(2814)=-0.164210111099D+04
coeff(2815)=-0.167868753811D+04
coeff(2816)=-0.958613393884D+04
coeff(2817)=-0.153241837609D+05
coeff(2818)=-0.108800616764D+04
coeff(2819)=-0.118022399475D+04
coeff(2820)=-0.180626610057D+04
coeff(2821)=0.105671900418D+05
coeff(2822)=-0.420300914002D+04
coeff(2823)=-0.132382094084D+05
coeff(2824)=0.506762603650D+04
coeff(2825)=-0.418056790740D+04
coeff(2826)=0.210640700374D+05
coeff(2827)=0.679055260818D+04
coeff(2828)=0.824001689076D+04
coeff(2829)=-0.544422228478D+03
coeff(2830)=-0.353179929676D+03
coeff(2831)=0.104799365585D+04
coeff(2832)=-0.152344614568D+05
coeff(2833)=-0.946824347081D+04
coeff(2834)=-0.746950209753D+04
coeff(2835)=0.421388223867D+04
coeff(2836)=-0.339856339477D+04
coeff(2837)=0.216359992102D+04
coeff(2838)=0.444507058278D+04
coeff(2839)=0.146994531577D+03
coeff(2840)=0.101740240044D+05
coeff(2841)=0.514714176147D+03
coeff(2842)=-0.209919546455D+04
coeff(2843)=-0.246321090336D+04
coeff(2844)=0.168032483006D+04
coeff(2845)=-0.118207976133D+04
coeff(2846)=-0.742396904390D+03
coeff(2847)=-0.674079791581D+04
coeff(2848)=-0.136474060026D+04
coeff(2849)=-0.384941881613D+04
coeff(2850)=-0.190681198550D+05
coeff(2851)=-0.125761511328D+05
coeff(2852)=0.559414605479D+04
coeff(2853)=-0.259857488292D+04
coeff(2854)=-0.153161223451D+05
coeff(2855)=0.596729152245D+04
coeff(2856)=0.756677693808D+02
coeff(2857)=0.474329129569D+04
coeff(2858)=-0.860871329075D+04
coeff(2859)=0.282360433952D+04
coeff(2860)=-0.596430026879D+04
coeff(2861)=0.125223127161D+04
coeff(2862)=-0.887748165992D+04
coeff(2863)=-0.122080376133D+04
coeff(2864)=0.118982457233D+04
coeff(2865)=-0.104857229001D+04
coeff(2866)=0.647203428924D+04
coeff(2867)=0.504654247578D+03
coeff(2868)=-0.448489755251D+04
coeff(2869)=0.264602312520D+04
coeff(2870)=-0.155823397001D+05
coeff(2871)=-0.164822125792D+05
coeff(2872)=-0.803547863522D+02
coeff(2873)=0.445120296150D+04
coeff(2874)=-0.662483345904D+03
coeff(2875)=0.187082607883D+04
coeff(2876)=-0.940869120069D+04
coeff(2877)=-0.426559803629D+04
coeff(2878)=-0.744580748565D+04
coeff(2879)=0.105507648565D+05
coeff(2880)=-0.100055778859D+05
coeff(2881)=-0.172316323280D+05
coeff(2882)=-0.754564976538D+04
coeff(2883)=-0.132217036729D+04
coeff(2884)=0.168111916003D+04
coeff(2885)=-0.435607193903D+04
coeff(2886)=0.369373603929D+04
coeff(2887)=0.990383789464D+03
coeff(2888)=0.151888684764D+04
coeff(2889)=0.464125881291D+04
coeff(2890)=0.645564071125D+04
coeff(2891)=0.256430241926D+04
coeff(2892)=-0.855724024500D+04
coeff(2893)=-0.495212869081D+04
coeff(2894)=0.665488389869D+04
coeff(2895)=-0.591887799500D+04
coeff(2896)=0.162572249521D+04
coeff(2897)=0.204517561709D+05
coeff(2898)=0.107560129301D+05
coeff(2899)=0.276475240810D+04
coeff(2900)=0.512492337561D+04
coeff(2901)=-0.698546006178D+04
coeff(2902)=-0.159978832302D+04
coeff(2903)=0.292252428732D+04
coeff(2904)=0.725825025933D+04
coeff(2905)=0.103807656761D+05
coeff(2906)=0.672299603308D+04
coeff(2907)=0.425711365178D+04
coeff(2908)=0.103953964036D+05
coeff(2909)=-0.203135264351D+05
coeff(2910)=-0.308844540460D+04
coeff(2911)=0.315189294840D+04
coeff(2912)=0.107648413450D+05
coeff(2913)=-0.155860521821D+05
coeff(2914)=0.297511502556D+04
coeff(2915)=0.344412573855D+04
coeff(2916)=-0.568904769457D+04
coeff(2917)=0.123874464667D+04
coeff(2918)=-0.207241878173D+04
coeff(2919)=-0.486141001507D+04
coeff(2920)=0.729565013629D+03
coeff(2921)=0.971142132447D+04
coeff(2922)=-0.129477462164D+05
coeff(2923)=-0.940368671895D+03
coeff(2924)=-0.420786997771D+04
coeff(2925)=0.635548037741D+03
coeff(2926)=0.503028085294D+04
coeff(2927)=0.868098188842D+04
coeff(2928)=-0.104555940060D+05
coeff(2929)=-0.246200084987D+04
coeff(2930)=0.353109284040D+04
coeff(2931)=-0.181513955071D+04
coeff(2932)=0.107307416397D+03
coeff(2933)=0.125167771936D+04
coeff(2934)=0.260111795518D+04
coeff(2935)=-0.364851353175D+04
coeff(2936)=-0.589046564021D+04
coeff(2937)=-0.952092919274D+04
coeff(2938)=0.838668596453D+04
coeff(2939)=-0.415690341991D+04
coeff(2940)=0.434101560799D+04
coeff(2941)=0.879085684267D+04
coeff(2942)=-0.135983160525D+05
coeff(2943)=-0.718745191837D+04
coeff(2944)=-0.102216029465D+03
coeff(2945)=-0.304848962988D+03
coeff(2946)=0.490979458814D+03
coeff(2947)=-0.344938289973D+04
coeff(2948)=-0.987956063977D+02
coeff(2949)=-0.639933654874D+03
coeff(2950)=-0.602104129022D+03
coeff(2951)=0.191401066315D+04
coeff(2952)=0.113414167115D+04
coeff(2953)=0.473422341021D+04
coeff(2954)=0.126370882150D+05
coeff(2955)=-0.106122354278D+05
coeff(2956)=0.999835024040D+04
coeff(2957)=-0.661662478035D+03
coeff(2958)=-0.977656156285D+03
coeff(2959)=0.146662693820D+03
coeff(2960)=-0.438384116921D+04
coeff(2961)=0.805891321635D+03
coeff(2962)=0.842589645203D+03
coeff(2963)=-0.941362509512D+03
coeff(2964)=0.394587009411D+03
coeff(2965)=-0.591860171874D+04
coeff(2966)=0.920059312543D+04
coeff(2967)=-0.457666906316D+04
coeff(2968)=-0.113963011670D+05
coeff(2969)=0.448029699853D+04
coeff(2970)=0.934316604747D+03
coeff(2971)=-0.613510150520D+04
coeff(2972)=-0.659414394878D+04
coeff(2973)=0.136260804468D+05
coeff(2974)=-0.915620484184D+04
coeff(2975)=0.832358262641D+03
coeff(2976)=0.599261555783D+04
coeff(2977)=0.335667072345D+04
coeff(2978)=-0.111378659363D+05
coeff(2979)=0.339882902623D+04
coeff(2980)=-0.282221838215D+04
coeff(2981)=0.381227253380D+04
coeff(2982)=0.387845298383D+04
coeff(2983)=-0.104323667688D+05
coeff(2984)=0.538700425518D+04
coeff(2985)=0.346043349484D+04
coeff(2986)=0.694813799387D+04
coeff(2987)=0.100579333189D+05
coeff(2988)=-0.956888827319D+04
coeff(2989)=0.108090840177D+05
coeff(2990)=-0.845251734073D+04
coeff(2991)=0.614162135603D+03
coeff(2992)=-0.369466156543D+04
coeff(2993)=0.109558232615D+04
coeff(2994)=-0.890912539741D+04
coeff(2995)=0.145184701539D+05
coeff(2996)=-0.349280184124D+04
coeff(2997)=0.402263014881D+04
coeff(2998)=0.746183195688D+04
coeff(2999)=0.297160509710D+03
coeff(3000)=0.162816850524D+05
coeff(3001)=0.106038050387D+05
coeff(3002)=-0.735708162672D+04
coeff(3003)=-0.116301123459D+05
coeff(3004)=-0.144233007208D+03
coeff(3005)=-0.463247342187D+04
coeff(3006)=-0.288948330950D+03
coeff(3007)=0.103763157398D+04
coeff(3008)=0.120238408052D+04
coeff(3009)=0.103679316514D+04
coeff(3010)=0.389757076471D+04
coeff(3011)=0.181603210190D+03
coeff(3012)=-0.581898734955D+04
coeff(3013)=0.938465375830D+04
coeff(3014)=-0.609151175208D+04
coeff(3015)=-0.261841069857D+04
coeff(3016)=-0.536242211603D+04
coeff(3017)=-0.858291047730D+04
coeff(3018)=0.208693837485D+04
coeff(3019)=0.716605479987D+04
coeff(3020)=-0.372190338253D+04
coeff(3021)=-0.704626801249D+04
coeff(3022)=-0.513952881342D+04
coeff(3023)=0.120162901753D+05
coeff(3024)=0.122606067313D+05
coeff(3025)=0.390377375363D+04
coeff(3026)=-0.241095488456D+04
coeff(3027)=-0.154057949507D+04
coeff(3028)=-0.958093385628D+04
coeff(3029)=-0.144365708750D+04
coeff(3030)=0.354338327271D+04
coeff(3031)=0.559271507152D+04
coeff(3032)=-0.399567356581D+04
coeff(3033)=-0.106395663644D+03
coeff(3034)=0.446122583621D+04
coeff(3035)=-0.553198418501D+04
coeff(3036)=0.125976474444D+04
coeff(3037)=-0.139956793631D+03
coeff(3038)=-0.758742270506D+03
coeff(3039)=0.365055788574D+04
coeff(3040)=0.862718313760D+04
coeff(3041)=0.203880072571D+03
coeff(3042)=-0.431919606706D+04
coeff(3043)=-0.532616141733D+04
coeff(3044)=0.951978409353D+04
coeff(3045)=-0.335728223861D+04
coeff(3046)=-0.288093064032D+03
coeff(3047)=-0.169445099807D+04
coeff(3048)=0.115416540610D+05
coeff(3049)=0.961010530525D+03
coeff(3050)=0.782362986665D+04
coeff(3051)=0.946336174095D+04
coeff(3052)=-0.125156240309D+03
coeff(3053)=-0.300615737287D+04
coeff(3054)=0.124176354015D+04
coeff(3055)=0.677281024944D+04
coeff(3056)=0.356253448083D+04
coeff(3057)=-0.133655923915D+04
coeff(3058)=0.843325848664D+04
coeff(3059)=-0.441452139007D+04
coeff(3060)=-0.585045404066D+04
coeff(3061)=-0.962176393305D+04
coeff(3062)=-0.202594905257D+04
coeff(3063)=0.201668467269D+04
coeff(3064)=-0.967897118394D+04
coeff(3065)=-0.470442354947D+03
coeff(3066)=-0.753482112043D+04
coeff(3067)=0.257254974279D+04
coeff(3068)=0.320224115745D+04
coeff(3069)=-0.301810378477D+04
coeff(3070)=-0.658456548452D+04
coeff(3071)=0.381023977154D+04
coeff(3072)=0.781053171351D+03
coeff(3073)=0.168237890692D+03
coeff(3074)=0.346953457933D+04
coeff(3075)=0.868229972210D+03
  return
end if

entry u13_dc_12(xyz,v)

r12=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r13=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r14=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r23=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r24=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r34=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

x(1)=exp(-((r12-ra1))/a1 -((r12-rb1)**2)/b1)
x(2)=exp(-((r13-ra1))/a1 -((r13-rb1)**2)/b1)
x(3)=exp(-((r14-ra2))/a2 -((r14-rb2)**2)/b2)
x(4)=exp(-((r23-ra1))/a1 -((r23-rb1)**2)/b1)
x(5)=exp(-((r24-ra2))/a2 -((r24-rb2)**2)/b2)
x(6)=exp(-((r34-ra2))/a2 -((r34-rb2)**2)/b2)

call ev_pol_con_12(x,q)
v=dot_product(coeff,q)/627.509474

end  subroutine


!===========================================================================
! *def u23_order10
! subtoutine of computing u23
!===========================================================================
subroutine u23_order10
implicit none
real*8 :: xyz(4,3)
real*8 :: v, r12, r13, r14, r23, r24, r34
real*8 :: s
real*8 :: a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer, parameter :: ncoeff=1300
real*8 :: coeff(ncoeff), q(ncoeff)
real*8 :: x(6)
integer ::  ncall
data ncall/1/
save ncall, coeff, a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer :: i, j
real*8 :: rrel(3,3),cross(3)
real*8 :: volume

if(ncall.eq.1) then
a1=1.323343
a2=1.09
b1=1.823343
b2=1.633190
ra1=1.113545
ra2=1.001178
rb1=1.113545
rb2=1.001178
coeff(1)=0.211647933777D+03
coeff(2)=0.418652112410D+03
coeff(3)=-0.804367714422D+03
coeff(4)=0.225274094195D+04
coeff(5)=-0.219286930355D+03
coeff(6)=-0.661002101653D+03
coeff(7)=-0.211035907398D+04
coeff(8)=-0.725168884355D+04
coeff(9)=0.817671816103D+02
coeff(10)=0.106113450336D+03
coeff(11)=-0.282535164976D+03
coeff(12)=-0.333754938494D+03
coeff(13)=-0.120987812011D+04
coeff(14)=0.471328667932D+04
coeff(15)=-0.384876246075D+04
coeff(16)=0.605116484146D+03
coeff(17)=0.601027152770D+04
coeff(18)=-0.476723869130D+04
coeff(19)=0.242850620352D+04
coeff(20)=-0.285066775847D+04
coeff(21)=0.246324630474D+04
coeff(22)=0.132728275082D+05
coeff(23)=0.240787551653D+04
coeff(24)=0.126999899000D+04
coeff(25)=0.200285824450D+05
coeff(26)=0.919112928831D+04
coeff(27)=0.497043700144D+04
coeff(28)=-0.109627473995D+05
coeff(29)=-0.140272171362D+04
coeff(30)=-0.173348331752D+04
coeff(31)=0.620938257312D+04
coeff(32)=0.130663291877D+05
coeff(33)=-0.404330571225D+04
coeff(34)=0.579617572582D+04
coeff(35)=-0.184590489416D+04
coeff(36)=0.101033461549D+05
coeff(37)=-0.825753568497D+04
coeff(38)=0.172464407115D+04
coeff(39)=-0.411425194165D+04
coeff(40)=0.222094534618D+04
coeff(41)=0.355796799193D+04
coeff(42)=0.689415348021D+04
coeff(43)=0.273155016647D+04
coeff(44)=0.353283844254D+03
coeff(45)=0.580886568523D+03
coeff(46)=0.110312632929D+04
coeff(47)=-0.168011521747D+05
coeff(48)=0.353418462670D+04
coeff(49)=-0.597989062336D+04
coeff(50)=0.508881039273D+04
coeff(51)=-0.664521211452D+04
coeff(52)=-0.268344104178D+03
coeff(53)=-0.105728580807D+05
coeff(54)=0.771426546054D+04
coeff(55)=-0.341939447216D+04
coeff(56)=0.100930435911D+05
coeff(57)=-0.711877607607D+04
coeff(58)=-0.155504503582D+05
coeff(59)=0.835283348744D+03
coeff(60)=-0.186167419259D+04
coeff(61)=-0.750394455066D+04
coeff(62)=0.765511384346D+04
coeff(63)=-0.185411921215D+05
coeff(64)=-0.144536025045D+04
coeff(65)=-0.887098455427D+04
coeff(66)=0.124321031826D+05
coeff(67)=0.391319268725D+04
coeff(68)=-0.662324600924D+04
coeff(69)=-0.103554030463D+04
coeff(70)=-0.427258160978D+04
coeff(71)=0.103948501982D+05
coeff(72)=0.112145935635D+05
coeff(73)=0.194938398901D+04
coeff(74)=-0.267000591512D+04
coeff(75)=0.385988255201D+04
coeff(76)=0.127706936283D+04
coeff(77)=-0.263199545820D+04
coeff(78)=-0.686790537395D+03
coeff(79)=0.209004269958D+04
coeff(80)=0.194908838388D+04
coeff(81)=0.697248336014D+03
coeff(82)=-0.787561824300D+03
coeff(83)=0.136735048798D+05
coeff(84)=-0.766393273674D+04
coeff(85)=-0.101612857385D+05
coeff(86)=-0.150458102806D+04
coeff(87)=0.184303879523D+04
coeff(88)=-0.367174434265D+04
coeff(89)=0.551217093762D+04
coeff(90)=0.923713748717D+04
coeff(91)=-0.706482698652D+04
coeff(92)=0.116392028325D+05
coeff(93)=-0.123138913147D+05
coeff(94)=0.229741326621D+04
coeff(95)=-0.185523989821D+05
coeff(96)=0.735262156221D+03
coeff(97)=-0.188220469491D+04
coeff(98)=0.152275516236D+04
coeff(99)=-0.252592730525D+04
coeff(100)=-0.793757417199D+03
coeff(101)=-0.604976728408D+04
coeff(102)=-0.473025909522D+04
coeff(103)=0.119720935477D+04
coeff(104)=-0.100981520248D+05
coeff(105)=-0.576899334276D+04
coeff(106)=0.156200727204D+04
coeff(107)=-0.274205695884D+04
coeff(108)=0.633972997294D+03
coeff(109)=-0.259040638406D+04
coeff(110)=0.122688507486D+05
coeff(111)=0.478804307974D+04
coeff(112)=0.283075109758D+04
coeff(113)=-0.424552038963D+04
coeff(114)=0.144203753127D+05
coeff(115)=-0.196998725896D+05
coeff(116)=-0.553738511755D+04
coeff(117)=0.215276876578D+04
coeff(118)=-0.706079086225D+04
coeff(119)=-0.519623096936D+04
coeff(120)=-0.142795317441D+05
coeff(121)=0.954385693868D+04
coeff(122)=-0.442470764446D+04
coeff(123)=0.606437371054D+04
coeff(124)=0.108881484463D+05
coeff(125)=-0.175602933727D+04
coeff(126)=0.785776658911D+04
coeff(127)=-0.291073754103D+04
coeff(128)=0.345211843353D+04
coeff(129)=0.107743052941D+05
coeff(130)=-0.215505790698D+04
coeff(131)=0.188194518691D+03
coeff(132)=0.872981484426D+03
coeff(133)=0.299779542950D+04
coeff(134)=-0.261024752855D+04
coeff(135)=0.878325161608D+04
coeff(136)=0.460601048750D+03
coeff(137)=0.522471819592D+04
coeff(138)=0.146941038866D+04
coeff(139)=0.279900256878D+04
coeff(140)=0.112714763249D+05
coeff(141)=-0.178205322597D+05
coeff(142)=-0.539917259722D+04
coeff(143)=-0.981118612930D+03
coeff(144)=-0.120445498339D+04
coeff(145)=-0.348130291156D+04
coeff(146)=-0.947561120555D+04
coeff(147)=-0.131298777309D+05
coeff(148)=0.556092647580D+04
coeff(149)=-0.311322310319D+04
coeff(150)=0.240348339527D+04
coeff(151)=-0.478893631390D+04
coeff(152)=0.311968286004D+04
coeff(153)=0.438658740177D+04
coeff(154)=-0.292615985156D+04
coeff(155)=-0.621092323259D+04
coeff(156)=-0.598689221259D+03
coeff(157)=0.347654294197D+04
coeff(158)=-0.870267145852D+04
coeff(159)=0.812791865812D+04
coeff(160)=-0.254792083823D+04
coeff(161)=0.205957895951D+04
coeff(162)=-0.713320343822D+04
coeff(163)=-0.634971269641D+04
coeff(164)=0.538967619613D+04
coeff(165)=-0.165210899194D+05
coeff(166)=0.679901128832D+04
coeff(167)=-0.516311416596D+04
coeff(168)=0.794222896084D+04
coeff(169)=-0.154874682939D+05
coeff(170)=0.100318912256D+05
coeff(171)=0.346978627199D+04
coeff(172)=0.116221827650D+05
coeff(173)=0.560756804188D+04
coeff(174)=0.316031261953D+04
coeff(175)=0.414298251791D+04
coeff(176)=0.160412337074D+05
coeff(177)=0.454731869840D+03
coeff(178)=-0.122844689389D+05
coeff(179)=0.137654384409D+04
coeff(180)=0.931808457513D+04
coeff(181)=-0.152469108535D+04
coeff(182)=0.266113779776D+04
coeff(183)=0.136662893273D+04
coeff(184)=0.114760133250D+05
coeff(185)=-0.238414696255D+04
coeff(186)=0.207915965250D+04
coeff(187)=0.491672928547D+03
coeff(188)=0.734091831023D+04
coeff(189)=-0.755417432975D+03
coeff(190)=0.218453948263D+04
coeff(191)=0.246277741312D+04
coeff(192)=-0.110715819015D+05
coeff(193)=-0.408131312641D+04
coeff(194)=0.562529183644D+04
coeff(195)=0.107035640219D+04
coeff(196)=-0.677016433357D+04
coeff(197)=0.241895037447D+04
coeff(198)=-0.575839298458D+03
coeff(199)=-0.135187541063D+04
coeff(200)=-0.657723802413D+03
coeff(201)=-0.405564754766D+03
coeff(202)=-0.332119647538D+04
coeff(203)=0.960171882108D+03
coeff(204)=0.232759520158D+04
coeff(205)=0.642924381807D+03
coeff(206)=0.553557927391D+04
coeff(207)=0.203385937440D+04
coeff(208)=0.349003755629D+04
coeff(209)=0.355042183210D+03
coeff(210)=0.316626114661D+04
coeff(211)=0.473853116624D+04
coeff(212)=-0.129504460989D+05
coeff(213)=0.368125207289D+04
coeff(214)=0.574335833529D+04
coeff(215)=0.389473193093D+04
coeff(216)=0.167202822270D+04
coeff(217)=-0.576006077404D+04
coeff(218)=0.780310610992D+04
coeff(219)=0.807696701473D+04
coeff(220)=-0.440896415389D+04
coeff(221)=0.138322229034D+05
coeff(222)=-0.452686681693D+04
coeff(223)=-0.434638074044D+04
coeff(224)=-0.308603324545D+03
coeff(225)=-0.723025644983D+04
coeff(226)=-0.359902103425D+03
coeff(227)=0.406868213350D+04
coeff(228)=-0.159758027308D+05
coeff(229)=0.148758514419D+04
coeff(230)=-0.256542121670D+04
coeff(231)=-0.311093213226D+03
coeff(232)=-0.591962023401D+04
coeff(233)=0.148939018654D+04
coeff(234)=-0.413251508685D+03
coeff(235)=0.555500105784D+04
coeff(236)=0.765072996785D+04
coeff(237)=-0.884574285652D+04
coeff(238)=-0.414364207318D+04
coeff(239)=-0.728446604557D+04
coeff(240)=-0.845308636007D+04
coeff(241)=0.310447105166D+04
coeff(242)=0.106841260226D+05
coeff(243)=0.147478816499D+05
coeff(244)=0.841171326337D+04
coeff(245)=0.159992483035D+05
coeff(246)=0.725297584139D+04
coeff(247)=0.886090494099D+04
coeff(248)=0.611443618164D+04
coeff(249)=0.153353447193D+04
coeff(250)=-0.788960988038D+04
coeff(251)=-0.784454960466D+04
coeff(252)=-0.161622519920D+05
coeff(253)=-0.102015921946D+05
coeff(254)=-0.607360092697D+02
coeff(255)=0.854274218915D+04
coeff(256)=0.140381109964D+05
coeff(257)=-0.206174182222D+04
coeff(258)=0.423124172580D+04
coeff(259)=0.942142641629D+04
coeff(260)=0.205581209087D+04
coeff(261)=0.901453524210D+04
coeff(262)=0.180229898607D+04
coeff(263)=-0.386934853468D+03
coeff(264)=-0.800325152020D+04
coeff(265)=0.565997170365D+04
coeff(266)=0.126471659788D+05
coeff(267)=0.805151518303D+04
coeff(268)=0.108212838455D+04
coeff(269)=0.729853295141D+04
coeff(270)=0.100135608074D+05
coeff(271)=-0.880886706967D+04
coeff(272)=-0.837210665616D+04
coeff(273)=-0.449345286055D+04
coeff(274)=-0.177837788615D+05
coeff(275)=0.182702185665D+05
coeff(276)=-0.160222908995D+05
coeff(277)=0.129168301491D+04
coeff(278)=0.319596826661D+04
coeff(279)=-0.131920566025D+05
coeff(280)=0.731313548438D+04
coeff(281)=0.816640344635D+04
coeff(282)=0.433445562636D+04
coeff(283)=0.824596342164D+04
coeff(284)=-0.154377440831D+05
coeff(285)=-0.477307507237D+04
coeff(286)=0.452942370033D+04
coeff(287)=0.590203366024D+04
coeff(288)=-0.289510470722D+03
coeff(289)=-0.139160274135D+04
coeff(290)=0.105245266484D+05
coeff(291)=0.785584524316D+04
coeff(292)=0.223672228790D+05
coeff(293)=-0.436107265931D+04
coeff(294)=0.116336896901D+05
coeff(295)=-0.813986863027D+04
coeff(296)=-0.475570277913D+04
coeff(297)=0.249291600262D+04
coeff(298)=-0.538572940697D+04
coeff(299)=0.552887073802D+04
coeff(300)=-0.798317794628D+04
coeff(301)=-0.120015511652D+05
coeff(302)=0.351650039286D+03
coeff(303)=-0.286610951224D+04
coeff(304)=0.367640304557D+04
coeff(305)=0.858368715570D+04
coeff(306)=0.244231932091D+04
coeff(307)=-0.378682460787D+04
coeff(308)=0.566566446860D+04
coeff(309)=-0.101254057870D+04
coeff(310)=-0.138162992855D+05
coeff(311)=-0.274266905393D+03
coeff(312)=-0.823543799736D+04
coeff(313)=0.715464088703D+04
coeff(314)=0.509591752960D+02
coeff(315)=-0.824180102796D+04
coeff(316)=-0.690228199982D+04
coeff(317)=0.113246333455D+05
coeff(318)=-0.276814306789D+04
coeff(319)=0.326769237427D+03
coeff(320)=-0.164441481232D+05
coeff(321)=-0.986558519261D+04
coeff(322)=0.305965192374D+04
coeff(323)=-0.124560876199D+05
coeff(324)=0.849489854222D+04
coeff(325)=0.469058641683D+03
coeff(326)=-0.116951694384D+05
coeff(327)=0.571771043791D+04
coeff(328)=0.474221866009D+04
coeff(329)=-0.759154554584D+04
coeff(330)=-0.131061280042D+05
coeff(331)=-0.729920552840D+02
coeff(332)=-0.145264422615D+04
coeff(333)=0.705033575472D+04
coeff(334)=-0.222145101851D+05
coeff(335)=0.287824498452D+03
coeff(336)=-0.221683588814D+05
coeff(337)=0.756143447516D+04
coeff(338)=0.458056630042D+04
coeff(339)=-0.867309337733D+04
coeff(340)=-0.107336508124D+05
coeff(341)=0.866305865689D+04
coeff(342)=-0.581597567509D+04
coeff(343)=0.844041055541D+04
coeff(344)=0.203377361346D+05
coeff(345)=0.711676455951D+04
coeff(346)=0.364017873449D+04
coeff(347)=-0.108011526702D+05
coeff(348)=0.519329835896D+04
coeff(349)=0.619089433742D+04
coeff(350)=-0.160352972025D+03
coeff(351)=0.204102074699D+04
coeff(352)=0.462952659815D+04
coeff(353)=-0.174449003851D+04
coeff(354)=-0.818605191606D+04
coeff(355)=-0.610481151342D+04
coeff(356)=-0.112642250003D+05
coeff(357)=0.686114901052D+04
coeff(358)=0.142904853960D+05
coeff(359)=-0.128404935854D+04
coeff(360)=-0.171062434296D+05
coeff(361)=-0.176312862883D+05
coeff(362)=-0.119828828939D+05
coeff(363)=0.639521126413D+02
coeff(364)=0.239363257075D+04
coeff(365)=0.453656969486D+04
coeff(366)=-0.108137745343D+05
coeff(367)=-0.840364144300D+04
coeff(368)=-0.215492842075D+05
coeff(369)=0.344354167334D+04
coeff(370)=0.143783801040D+05
coeff(371)=-0.536000152024D+04
coeff(372)=0.211345975677D+05
coeff(373)=-0.806203928141D+04
coeff(374)=-0.210797992535D+05
coeff(375)=0.999089844288D+04
coeff(376)=-0.105690119328D+05
coeff(377)=0.172853580329D+05
coeff(378)=-0.433679926462D+04
coeff(379)=-0.535161827053D+04
coeff(380)=-0.840193528856D+04
coeff(381)=-0.190539209474D+04
coeff(382)=-0.143886737250D+05
coeff(383)=-0.100139826101D+05
coeff(384)=-0.844941014067D+04
coeff(385)=0.145484458925D+05
coeff(386)=0.183327242970D+05
coeff(387)=-0.676718070256D+04
coeff(388)=0.173802012846D+05
coeff(389)=0.939246818437D+04
coeff(390)=0.142021784978D+05
coeff(391)=0.707745594318D+04
coeff(392)=-0.211739852425D+04
coeff(393)=-0.188108120328D+05
coeff(394)=-0.484980266354D+04
coeff(395)=-0.123953177645D+05
coeff(396)=0.873501246793D+04
coeff(397)=0.553405589926D+03
coeff(398)=0.754060449979D+03
coeff(399)=0.111996669509D+05
coeff(400)=0.200051534993D+04
coeff(401)=0.127710247734D+04
coeff(402)=-0.125373177765D+05
coeff(403)=-0.656523010610D+04
coeff(404)=0.646306478580D+04
coeff(405)=0.112981824053D+05
coeff(406)=-0.280228269632D+04
coeff(407)=0.328632583173D+04
coeff(408)=0.233380197654D+04
coeff(409)=0.366412004611D+04
coeff(410)=0.129976681422D+05
coeff(411)=0.227785831967D+04
coeff(412)=-0.120094595817D+05
coeff(413)=0.548446295435D+04
coeff(414)=0.317344873326D+04
coeff(415)=-0.251869258369D+04
coeff(416)=0.957076310380D+04
coeff(417)=-0.122866996563D+05
coeff(418)=0.357626019981D+04
coeff(419)=-0.440521223410D+04
coeff(420)=-0.155980313909D+05
coeff(421)=0.104578600400D+04
coeff(422)=-0.196283935829D+04
coeff(423)=-0.912349341551D+04
coeff(424)=0.233026839176D+05
coeff(425)=-0.147602401182D+05
coeff(426)=0.920608639025D+04
coeff(427)=0.165008763402D+05
coeff(428)=0.316407236740D+04
coeff(429)=0.910398242444D+04
coeff(430)=0.156282911603D+05
coeff(431)=0.455217929966D+04
coeff(432)=0.798240300074D+04
coeff(433)=0.154665164917D+05
coeff(434)=0.139841221317D+05
coeff(435)=0.458278504293D+04
coeff(436)=-0.679518151250D+04
coeff(437)=0.194182259734D+05
coeff(438)=-0.238501077119D+05
coeff(439)=0.886479228604D+04
coeff(440)=0.264169841465D+04
coeff(441)=-0.229987570672D+05
coeff(442)=-0.127540242560D+05
coeff(443)=-0.300810458674D+05
coeff(444)=-0.864180651060D+04
coeff(445)=-0.980703968326D+04
coeff(446)=-0.228755424442D+04
coeff(447)=0.217531067958D+04
coeff(448)=0.115993875227D+05
coeff(449)=-0.487824241576D+04
coeff(450)=-0.109613645685D+05
coeff(451)=0.254098826690D+04
coeff(452)=-0.100839480665D+05
coeff(453)=0.429231269927D+04
coeff(454)=0.292273435699D+04
coeff(455)=0.126346109158D+05
coeff(456)=-0.237644290969D+05
coeff(457)=0.105326862772D+04
coeff(458)=-0.302816292293D+04
coeff(459)=0.412727756457D+04
coeff(460)=0.886107923464D+03
coeff(461)=0.141398086812D+05
coeff(462)=0.800086199879D+04
coeff(463)=0.498135622939D+03
coeff(464)=-0.788789165755D+03
coeff(465)=-0.180635476261D+05
coeff(466)=-0.897848085429D+04
coeff(467)=0.100606399871D+04
coeff(468)=0.681136882137D+03
coeff(469)=-0.187729372963D+05
coeff(470)=0.184423732262D+04
coeff(471)=0.983166494245D+03
coeff(472)=0.580498294935D+04
coeff(473)=-0.126398977329D+04
coeff(474)=0.439141428450D+04
coeff(475)=-0.411009358289D+04
coeff(476)=-0.771397094847D+04
coeff(477)=-0.429193217326D+04
coeff(478)=-0.191592567348D+05
coeff(479)=0.894429393011D+04
coeff(480)=-0.126904798807D+05
coeff(481)=-0.165571069346D+04
coeff(482)=-0.207200394755D+04
coeff(483)=0.630824428987D+04
coeff(484)=-0.435174256335D+04
coeff(485)=-0.114745830451D+05
coeff(486)=-0.455622390054D+04
coeff(487)=-0.690479840274D+03
coeff(488)=-0.473220191997D+04
coeff(489)=0.703479673537D+03
coeff(490)=-0.783603365840D+04
coeff(491)=0.167820931382D+04
coeff(492)=0.177186082080D+05
coeff(493)=0.214306690681D+04
coeff(494)=0.168583319253D+05
coeff(495)=0.947840388226D+04
coeff(496)=0.612836408675D+04
coeff(497)=0.521221663863D+04
coeff(498)=0.106458341247D+05
coeff(499)=0.815957531447D+04
coeff(500)=-0.120798820834D+05
coeff(501)=-0.956952273876D+04
coeff(502)=-0.101369679988D+04
coeff(503)=0.111132300983D+05
coeff(504)=-0.102473205255D+05
coeff(505)=0.112851009001D+05
coeff(506)=-0.115440500701D+05
coeff(507)=-0.172001455301D+05
coeff(508)=-0.166044091076D+05
coeff(509)=0.749391286538D+03
coeff(510)=0.153501944690D+04
coeff(511)=-0.997667070870D+04
coeff(512)=0.232737402211D+05
coeff(513)=-0.435062157950D+04
coeff(514)=-0.217558654723D+05
coeff(515)=0.348611177666D+05
coeff(516)=-0.156927617719D+05
coeff(517)=-0.125865995280D+05
coeff(518)=0.186517132941D+05
coeff(519)=-0.132020323908D+05
coeff(520)=-0.176288988619D+05
coeff(521)=-0.305531009490D+03
coeff(522)=0.623617611314D+04
coeff(523)=-0.837797397973D+04
coeff(524)=-0.179746775710D+05
coeff(525)=-0.561533897337D+04
coeff(526)=-0.618604464695D+04
coeff(527)=0.261085526115D+04
coeff(528)=0.204208516366D+05
coeff(529)=-0.477701707198D+04
coeff(530)=-0.266492460457D+05
coeff(531)=0.170334076647D+05
coeff(532)=0.132801148467D+05
coeff(533)=-0.181550747259D+05
coeff(534)=-0.978944666325D+04
coeff(535)=0.313562856072D+05
coeff(536)=-0.145130702723D+04
coeff(537)=0.168325674218D+05
coeff(538)=-0.263364229793D+05
coeff(539)=0.184847781026D+05
coeff(540)=0.293564118237D+04
coeff(541)=-0.246252936069D+05
coeff(542)=0.290794529088D+05
coeff(543)=0.636594336636D+04
coeff(544)=0.132739897757D+05
coeff(545)=-0.133812044415D+05
coeff(546)=0.162886661655D+05
coeff(547)=-0.463700947082D+04
coeff(548)=0.515614934580D+04
coeff(549)=0.354888572547D+04
coeff(550)=-0.145059638747D+05
coeff(551)=0.466921132932D+04
coeff(552)=0.386740914864D+04
coeff(553)=-0.697827875099D+04
coeff(554)=0.176438500807D+05
coeff(555)=0.106558354391D+05
coeff(556)=0.261274365953D+04
coeff(557)=-0.420974163639D+04
coeff(558)=-0.728521599619D+04
coeff(559)=-0.612577254167D+03
coeff(560)=-0.543557715697D+04
coeff(561)=0.556722228047D+03
coeff(562)=-0.608813999052D+02
coeff(563)=-0.367500294410D+04
coeff(564)=0.117402749050D+04
coeff(565)=-0.155769267580D+04
coeff(566)=0.568853574951D+04
coeff(567)=-0.647004497778D+04
coeff(568)=-0.324623328204D+04
coeff(569)=0.132009997628D+05
coeff(570)=-0.203726537156D+04
coeff(571)=-0.401145805680D+04
coeff(572)=0.120596594370D+05
coeff(573)=-0.114762040099D+05
coeff(574)=0.872064521477D+04
coeff(575)=-0.200875008999D+04
coeff(576)=0.419485468969D+04
coeff(577)=-0.602820092804D+03
coeff(578)=0.631528595479D+04
coeff(579)=-0.155241702912D+05
coeff(580)=0.168820242377D+05
coeff(581)=0.886230038929D+03
coeff(582)=-0.208653861199D+04
coeff(583)=0.133962521259D+05
coeff(584)=-0.249012426367D+04
coeff(585)=0.845227655056D+04
coeff(586)=0.658683618767D+03
coeff(587)=-0.100794609516D+04
coeff(588)=-0.209497412460D+04
coeff(589)=-0.197737710699D+05
coeff(590)=0.450968347457D+04
coeff(591)=-0.109298801468D+05
coeff(592)=0.316033488971D+05
coeff(593)=-0.328458633346D+05
coeff(594)=0.127152043804D+05
coeff(595)=0.172032495043D+05
coeff(596)=0.114098517100D+05
coeff(597)=-0.144024892546D+05
coeff(598)=-0.387881031585D+04
coeff(599)=0.438059911701D+04
coeff(600)=-0.110301734768D+04
coeff(601)=-0.361306212856D+03
coeff(602)=-0.797605956309D+04
coeff(603)=0.583319540842D+04
coeff(604)=0.702046903300D+04
coeff(605)=0.165602273819D+05
coeff(606)=-0.133534530658D+05
coeff(607)=0.256779407543D+05
coeff(608)=-0.769682057365D+04
coeff(609)=-0.480210976179D+04
coeff(610)=0.557238359195D+04
coeff(611)=-0.238862939170D+05
coeff(612)=-0.777213341411D+04
coeff(613)=-0.195705269576D+05
coeff(614)=0.303438866902D+04
coeff(615)=0.276291738073D+03
coeff(616)=0.907443392981D+03
coeff(617)=-0.483319606829D+04
coeff(618)=0.121802135862D+04
coeff(619)=-0.330447664147D+04
coeff(620)=-0.159026294155D+04
coeff(621)=0.127596968062D+05
coeff(622)=0.774597199000D+04
coeff(623)=-0.528962243324D+04
coeff(624)=0.113721135797D+05
coeff(625)=0.229427572720D+04
coeff(626)=0.235051218796D+05
coeff(627)=-0.788853686334D+04
coeff(628)=0.955043977662D+04
coeff(629)=0.154142110402D+05
coeff(630)=-0.110429591521D+05
coeff(631)=0.961846631464D+04
coeff(632)=0.213307552388D+05
coeff(633)=0.251489343934D+05
coeff(634)=0.201727645937D+05
coeff(635)=-0.113761316538D+05
coeff(636)=-0.589297276243D+04
coeff(637)=-0.132146074272D+05
coeff(638)=-0.750025452072D+04
coeff(639)=-0.104244721684D+05
coeff(640)=0.174299992183D+04
coeff(641)=-0.163305780372D+05
coeff(642)=0.211731189345D+05
coeff(643)=0.111405574535D+05
coeff(644)=-0.358404804042D+04
coeff(645)=0.651401016379D+04
coeff(646)=-0.896152207253D+04
coeff(647)=0.118088494122D+05
coeff(648)=-0.863753325552D+04
coeff(649)=0.151731639811D+04
coeff(650)=0.118637278718D+05
coeff(651)=0.193910286642D+04
coeff(652)=0.144030069211D+05
coeff(653)=0.951288784882D+04
coeff(654)=-0.111792875525D+05
coeff(655)=0.208200443921D+05
coeff(656)=0.318184280250D+05
coeff(657)=-0.698782627044D+04
coeff(658)=-0.698925909244D+03
coeff(659)=0.260191676613D+03
coeff(660)=0.738479353076D+04
coeff(661)=-0.140249306175D+05
coeff(662)=0.276278691115D+04
coeff(663)=0.127195357524D+05
coeff(664)=-0.667177657508D+04
coeff(665)=-0.132167256563D+04
coeff(666)=0.990876855094D+04
coeff(667)=0.107457623664D+05
coeff(668)=-0.122359355646D+05
coeff(669)=-0.105359643063D+05
coeff(670)=-0.651323058933D+04
coeff(671)=-0.789451596992D+04
coeff(672)=-0.681479860718D+04
coeff(673)=0.490773657682D+04
coeff(674)=-0.902138238377D+03
coeff(675)=0.108774913269D+04
coeff(676)=-0.134979133748D+05
coeff(677)=-0.341592648972D+05
coeff(678)=-0.188881173371D+05
coeff(679)=0.188553626586D+05
coeff(680)=0.516328345700D+04
coeff(681)=0.185583590878D+04
coeff(682)=0.513267361280D+04
coeff(683)=-0.904233111870D+04
coeff(684)=-0.360503976380D+04
coeff(685)=0.178275215763D+05
coeff(686)=-0.124901776611D+05
coeff(687)=-0.150473574552D+05
coeff(688)=-0.744133658492D+04
coeff(689)=0.650267811704D+03
coeff(690)=-0.749581230756D+04
coeff(691)=0.838508727874D+04
coeff(692)=0.729998096071D+04
coeff(693)=0.475484034866D+04
coeff(694)=-0.737834775200D+04
coeff(695)=0.636859075484D+02
coeff(696)=0.160101985552D+05
coeff(697)=0.128631393471D+05
coeff(698)=-0.777640785191D+04
coeff(699)=-0.998849518212D+04
coeff(700)=-0.595614832118D+04
coeff(701)=-0.823954824150D+04
coeff(702)=0.598047636268D+04
coeff(703)=0.148708335510D+05
coeff(704)=0.121484442805D+05
coeff(705)=0.929781315285D+04
coeff(706)=-0.941422205119D+03
coeff(707)=-0.206225373002D+04
coeff(708)=-0.845693184896D+04
coeff(709)=-0.152391624697D+05
coeff(710)=0.277561444170D+04
coeff(711)=-0.123834198881D+04
coeff(712)=0.102477394815D+05
coeff(713)=-0.762719564387D+04
coeff(714)=0.935747489719D+04
coeff(715)=0.612925562927D+04
coeff(716)=-0.903447818938D+04
coeff(717)=-0.108413723177D+04
coeff(718)=-0.302379961043D+04
coeff(719)=0.247113907394D+04
coeff(720)=-0.860289358822D+04
coeff(721)=0.253015017396D+05
coeff(722)=-0.330795533739D+03
coeff(723)=-0.190556978729D+05
coeff(724)=-0.118104447811D+05
coeff(725)=0.100209550412D+04
coeff(726)=0.635953798583D+04
coeff(727)=-0.643337616566D+04
coeff(728)=-0.278942425818D+04
coeff(729)=-0.849777019108D+04
coeff(730)=0.163734093994D+05
coeff(731)=-0.372578662399D+04
coeff(732)=0.973547648607D+04
coeff(733)=-0.533739334679D+04
coeff(734)=0.102542624640D+04
coeff(735)=0.211245759946D+05
coeff(736)=0.116253844900D+05
coeff(737)=-0.950050456377D+04
coeff(738)=0.120135625007D+05
coeff(739)=-0.151093122050D+04
coeff(740)=-0.545696940306D+04
coeff(741)=0.345706166411D+04
coeff(742)=-0.481457845502D+05
coeff(743)=0.857811435232D+03
coeff(744)=0.351524495527D+03
coeff(745)=-0.936368639331D+04
coeff(746)=-0.145717285474D+05
coeff(747)=-0.626301355850D+04
coeff(748)=0.190027903852D+04
coeff(749)=-0.121099039136D+05
coeff(750)=-0.336347129047D+05
coeff(751)=0.310019121505D+04
coeff(752)=0.133760683236D+05
coeff(753)=-0.722833067710D+04
coeff(754)=0.141234678875D+04
coeff(755)=-0.647632220327D+04
coeff(756)=0.226052041885D+05
coeff(757)=0.182570266564D+05
coeff(758)=0.100428107927D+05
coeff(759)=-0.859108243013D+04
coeff(760)=-0.672285542465D+04
coeff(761)=0.894720176331D+04
coeff(762)=0.118833299748D+05
coeff(763)=0.276888599085D+03
coeff(764)=-0.681856208739D+04
coeff(765)=0.160376328325D+05
coeff(766)=0.281587725867D+03
coeff(767)=-0.128100263145D+05
coeff(768)=-0.133030126058D+05
coeff(769)=0.182478312181D+05
coeff(770)=0.185576613079D+05
coeff(771)=-0.338814595879D+05
coeff(772)=-0.188586132712D+05
coeff(773)=0.140781241150D+05
coeff(774)=0.184973664066D+05
coeff(775)=0.157151785925D+05
coeff(776)=-0.498012032863D+04
coeff(777)=0.252479902676D+02
coeff(778)=0.603467387459D+04
coeff(779)=0.183561117910D+05
coeff(780)=0.216796169959D+04
coeff(781)=0.127175084558D+05
coeff(782)=-0.740755872422D+02
coeff(783)=-0.428943300008D+04
coeff(784)=0.738505411884D+04
coeff(785)=-0.867082150183D+04
coeff(786)=-0.103901842247D+05
coeff(787)=-0.125016865836D+05
coeff(788)=0.161761476384D+05
coeff(789)=-0.437266783788D+04
coeff(790)=-0.182481677788D+04
coeff(791)=-0.303234231261D+04
coeff(792)=0.704599110454D+04
coeff(793)=-0.551168564435D+04
coeff(794)=-0.170002596878D+04
coeff(795)=0.173088561623D+04
coeff(796)=-0.983597277196D+03
coeff(797)=0.108163564425D+05
coeff(798)=0.167712798474D+04
coeff(799)=0.133460876828D+03
coeff(800)=-0.108641280057D+05
coeff(801)=0.159262328053D+05
coeff(802)=0.783489595595D+04
coeff(803)=-0.132279072990D+05
coeff(804)=0.187811867663D+05
coeff(805)=0.596877397479D+04
coeff(806)=-0.724252563702D+04
coeff(807)=-0.121281518163D+05
coeff(808)=0.845471239079D+04
coeff(809)=0.605729916056D+04
coeff(810)=0.126763630960D+05
coeff(811)=-0.151596689550D+05
coeff(812)=0.148143229077D+05
coeff(813)=-0.207010208121D+05
coeff(814)=-0.123105128358D+05
coeff(815)=-0.134757570205D+04
coeff(816)=-0.206837083977D+05
coeff(817)=-0.138812775116D+05
coeff(818)=0.204816239594D+05
coeff(819)=0.531344190849D+04
coeff(820)=0.109040431825D+05
coeff(821)=-0.813618893189D+04
coeff(822)=0.177798614730D+05
coeff(823)=-0.246316571747D+04
coeff(824)=0.740041242244D+04
coeff(825)=-0.925375152013D+04
coeff(826)=0.108744287690D+05
coeff(827)=-0.853288004799D+04
coeff(828)=-0.916176267691D+04
coeff(829)=-0.597783573661D+04
coeff(830)=-0.293233673907D+04
coeff(831)=0.879041103546D+04
coeff(832)=0.830201457672D+04
coeff(833)=0.246061738377D+05
coeff(834)=-0.966552892389D+04
coeff(835)=-0.143974788509D+05
coeff(836)=0.118213653326D+05
coeff(837)=0.585317871490D+04
coeff(838)=0.122468197450D+05
coeff(839)=-0.164404554322D+05
coeff(840)=-0.633189017259D+04
coeff(841)=-0.507421887053D+04
coeff(842)=0.528436467740D+04
coeff(843)=0.233261508067D+04
coeff(844)=-0.440073914388D+04
coeff(845)=0.933114397181D+04
coeff(846)=-0.738530830929D+04
coeff(847)=-0.114564542041D+04
coeff(848)=-0.230465460379D+05
coeff(849)=0.663071416953D+04
coeff(850)=0.147865028430D+05
coeff(851)=-0.858585929337D+04
coeff(852)=0.132054552457D+04
coeff(853)=-0.549899804536D+04
coeff(854)=0.585264907620D+04
coeff(855)=-0.717933231748D+04
coeff(856)=-0.310173809354D+05
coeff(857)=0.195510265657D+05
coeff(858)=-0.958731214834D+04
coeff(859)=-0.166743910214D+05
coeff(860)=0.364168160303D+04
coeff(861)=0.174084932942D+05
coeff(862)=-0.200720586768D+02
coeff(863)=-0.147115753478D+04
coeff(864)=0.534620276156D+04
coeff(865)=-0.234348041867D+04
coeff(866)=-0.233676378509D+04
coeff(867)=-0.176009510718D+05
coeff(868)=-0.206334852632D+04
coeff(869)=0.643624026998D+04
coeff(870)=-0.106498847200D+05
coeff(871)=-0.733232888792D+04
coeff(872)=-0.687800376643D+04
coeff(873)=0.295426763110D+04
coeff(874)=0.121172292296D+05
coeff(875)=0.394221626292D+04
coeff(876)=0.584279488469D+04
coeff(877)=-0.174182056693D+05
coeff(878)=0.220643721464D+04
coeff(879)=0.217081342067D+05
coeff(880)=-0.230336390390D+04
coeff(881)=-0.704333621999D+04
coeff(882)=0.106632271710D+04
coeff(883)=0.983598417454D+04
coeff(884)=-0.533828694372D+03
coeff(885)=-0.228418138058D+04
coeff(886)=0.160709323817D+05
coeff(887)=0.260350411568D+05
coeff(888)=0.670228959185D+04
coeff(889)=-0.949079191017D+04
coeff(890)=0.222282344460D+05
coeff(891)=0.125639665229D+05
coeff(892)=0.160766989120D+05
coeff(893)=-0.785291851172D+04
coeff(894)=0.310374654095D+04
coeff(895)=0.130961698965D+05
coeff(896)=0.262244548440D+05
coeff(897)=-0.177461610180D+05
coeff(898)=-0.759308065051D+04
coeff(899)=0.323355895680D+05
coeff(900)=-0.724516590198D+04
coeff(901)=0.148006838386D+05
coeff(902)=-0.377562638608D+05
coeff(903)=0.149191649065D+05
coeff(904)=-0.364819546764D+04
coeff(905)=-0.484498176328D+04
coeff(906)=-0.195590000782D+05
coeff(907)=0.846429800932D+04
coeff(908)=-0.243318664334D+05
coeff(909)=-0.137568575079D+05
coeff(910)=0.127429781226D+05
coeff(911)=0.209229029722D+05
coeff(912)=-0.435748008792D+05
coeff(913)=0.304879168545D+05
coeff(914)=-0.307474481662D+05
coeff(915)=0.118917704130D+05
coeff(916)=-0.176685116730D+04
coeff(917)=-0.966241385978D+04
coeff(918)=0.137890159969D+05
coeff(919)=-0.135763408943D+04
coeff(920)=-0.264446147378D+05
coeff(921)=0.750446406788D+04
coeff(922)=0.182822594156D+05
coeff(923)=-0.182049353450D+05
coeff(924)=-0.266217060153D+05
coeff(925)=0.727470127923D+03
coeff(926)=-0.183004028246D+05
coeff(927)=-0.983004520033D+04
coeff(928)=-0.222047877224D+05
coeff(929)=0.980115367601D+04
coeff(930)=0.367972129503D+04
coeff(931)=0.412529961693D+03
coeff(932)=-0.103836813096D+05
coeff(933)=0.347197160454D+04
coeff(934)=-0.575314716883D+04
coeff(935)=0.730408030062D+04
coeff(936)=-0.126329551331D+05
coeff(937)=0.169068580730D+05
coeff(938)=0.101562286455D+05
coeff(939)=0.311132455467D+04
coeff(940)=0.458648214123D+03
coeff(941)=0.189482981332D+05
coeff(942)=0.124888386420D+05
coeff(943)=-0.748479353427D+04
coeff(944)=-0.498029559341D+04
coeff(945)=0.689294048143D+04
coeff(946)=-0.878310106226D+04
coeff(947)=0.803015394242D+04
coeff(948)=-0.557339689062D+03
coeff(949)=0.201899090274D+05
coeff(950)=-0.269924686401D+04
coeff(951)=0.180129198111D+05
coeff(952)=-0.206569349514D+04
coeff(953)=0.813845870751D+04
coeff(954)=0.439236520870D+04
coeff(955)=-0.198718218962D+05
coeff(956)=-0.191264944162D+05
coeff(957)=-0.149635509248D+05
coeff(958)=0.663327018605D+03
coeff(959)=0.162319690889D+05
coeff(960)=0.205791587715D+04
coeff(961)=0.256802188401D+04
coeff(962)=0.383294107828D+04
coeff(963)=-0.163810108098D+05
coeff(964)=-0.296656726741D+03
coeff(965)=0.313787934364D+04
coeff(966)=0.302066454484D+05
coeff(967)=-0.101842350569D+05
coeff(968)=0.163745614962D+04
coeff(969)=0.892208520456D+04
coeff(970)=-0.115577298546D+05
coeff(971)=0.888331360468D+03
coeff(972)=-0.533090439299D+04
coeff(973)=0.580010130727D+04
coeff(974)=-0.150014413420D+04
coeff(975)=-0.279863512923D+04
coeff(976)=-0.136756893463D+05
coeff(977)=0.197946485725D+05
coeff(978)=-0.721999069250D+04
coeff(979)=-0.494933650044D+04
coeff(980)=0.272936108704D+04
coeff(981)=-0.394015942004D+04
coeff(982)=0.434487546724D+04
coeff(983)=-0.166841789125D+04
coeff(984)=-0.717922398479D+04
coeff(985)=-0.431668115803D+04
coeff(986)=0.209094115703D+04
coeff(987)=0.723263575895D+04
coeff(988)=0.141692973518D+04
coeff(989)=0.248807553301D+04
coeff(990)=0.581004434858D+04
coeff(991)=-0.434751689230D+04
coeff(992)=0.290506905394D+04
coeff(993)=-0.590388370019D+04
coeff(994)=-0.294171108409D+05
coeff(995)=0.211928010106D+05
coeff(996)=-0.105633790746D+05
coeff(997)=0.299241032041D+04
coeff(998)=0.136440123905D+04
coeff(999)=0.158929081419D+04
coeff(1000)=0.289119265109D+04
coeff(1001)=0.950627792267D+04
coeff(1002)=-0.370144114574D+04
coeff(1003)=-0.262332336930D+04
coeff(1004)=-0.215362881593D+04
coeff(1005)=0.808532532867D+04
coeff(1006)=0.390256126577D+04
coeff(1007)=-0.236134034905D+04
coeff(1008)=-0.163013747730D+05
coeff(1009)=0.954349389159D+04
coeff(1010)=0.289678300995D+04
coeff(1011)=-0.142416639060D+05
coeff(1012)=0.206968611050D+05
coeff(1013)=0.640372889811D+04
coeff(1014)=0.309087677074D+04
coeff(1015)=0.998741832720D+04
coeff(1016)=0.430689625952D+04
coeff(1017)=-0.134552328492D+05
coeff(1018)=-0.369628153343D+04
coeff(1019)=-0.623737249603D+04
coeff(1020)=-0.117934048204D+04
coeff(1021)=-0.156204171538D+04
coeff(1022)=0.277939267678D+03
coeff(1023)=-0.542505729304D+03
coeff(1024)=0.259494401914D+05
coeff(1025)=-0.136848837294D+05
coeff(1026)=0.164554422247D+05
coeff(1027)=0.271222077622D+03
coeff(1028)=0.109767854894D+05
coeff(1029)=0.181460166841D+05
coeff(1030)=-0.688231860866D+04
coeff(1031)=-0.145088696515D+05
coeff(1032)=-0.665572265355D+04
coeff(1033)=0.397156117761D+04
coeff(1034)=0.262213168083D+05
coeff(1035)=-0.232940963715D+05
coeff(1036)=0.579305048476D+04
coeff(1037)=0.916257192739D+04
coeff(1038)=0.934581063558D+04
coeff(1039)=-0.138497468554D+05
coeff(1040)=0.351404687520D+04
coeff(1041)=-0.675405804466D+04
coeff(1042)=0.396817531866D+04
coeff(1043)=-0.180978681407D+05
coeff(1044)=0.927107762457D+04
coeff(1045)=0.485030028734D+03
coeff(1046)=-0.180910383072D+05
coeff(1047)=0.110538712218D+05
coeff(1048)=-0.746120174053D+04
coeff(1049)=-0.182137464083D+05
coeff(1050)=0.198648243025D+05
coeff(1051)=-0.238261818502D+05
coeff(1052)=-0.298920455466D+05
coeff(1053)=-0.698573302489D+04
coeff(1054)=0.177719716839D+05
coeff(1055)=-0.720059094357D+04
coeff(1056)=-0.211198364047D+05
coeff(1057)=0.179285349829D+04
coeff(1058)=0.580119253974D+04
coeff(1059)=0.148027935062D+05
coeff(1060)=-0.510544789769D+04
coeff(1061)=-0.128005487841D+05
coeff(1062)=0.639195535186D+04
coeff(1063)=-0.214420795216D+05
coeff(1064)=0.469324923984D+04
coeff(1065)=-0.159479080215D+05
coeff(1066)=-0.620919504050D+04
coeff(1067)=-0.118010762971D+05
coeff(1068)=0.140640318423D+05
coeff(1069)=-0.165558238880D+05
coeff(1070)=-0.132232186395D+05
coeff(1071)=0.779058215784D+04
coeff(1072)=0.729279229078D+04
coeff(1073)=0.516469572775D+04
coeff(1074)=-0.128981566784D+05
coeff(1075)=0.371200197911D+04
coeff(1076)=0.627992116259D+04
coeff(1077)=0.127262808071D+05
coeff(1078)=-0.670763979614D+04
coeff(1079)=0.163003306635D+05
coeff(1080)=0.278944602510D+04
coeff(1081)=-0.612452139012D+04
coeff(1082)=0.275062660142D+05
coeff(1083)=-0.159040671465D+05
coeff(1084)=0.217721913937D+05
coeff(1085)=-0.580446105600D+04
coeff(1086)=0.713439192609D+04
coeff(1087)=-0.196740108958D+05
coeff(1088)=0.471607619774D+04
coeff(1089)=0.166724453164D+05
coeff(1090)=-0.108998111399D+05
coeff(1091)=-0.566318967661D+04
coeff(1092)=0.772382867827D+04
coeff(1093)=-0.613935079165D+04
coeff(1094)=-0.887007852446D+04
coeff(1095)=0.897621600591D+03
coeff(1096)=0.134229779392D+05
coeff(1097)=-0.142000752304D+05
coeff(1098)=0.306793314335D+05
coeff(1099)=0.287823386253D+05
coeff(1100)=0.259568848608D+05
coeff(1101)=-0.321410968620D+05
coeff(1102)=0.532659594351D+04
coeff(1103)=0.199719279484D+05
coeff(1104)=-0.322258089668D+04
coeff(1105)=0.863012522533D+04
coeff(1106)=0.706129822064D+03
coeff(1107)=0.161214456427D+05
coeff(1108)=0.708936358803D+04
coeff(1109)=-0.376958918655D+03
coeff(1110)=0.657728511725D+04
coeff(1111)=-0.462700382558D+04
coeff(1112)=-0.352809795812D+04
coeff(1113)=0.155973473172D+05
coeff(1114)=0.406637234131D+04
coeff(1115)=-0.203645280705D+05
coeff(1116)=-0.596292310659D+04
coeff(1117)=-0.230481459797D+05
coeff(1118)=0.390985772351D+04
coeff(1119)=0.169226535054D+05
coeff(1120)=0.172756976176D+04
coeff(1121)=-0.215459887422D+05
coeff(1122)=0.187711830264D+05
coeff(1123)=-0.112374626557D+05
coeff(1124)=-0.194824720412D+05
coeff(1125)=-0.226022116401D+04
coeff(1126)=0.867446001053D+03
coeff(1127)=0.128186492483D+05
coeff(1128)=0.124840315576D+05
coeff(1129)=0.550406472343D+04
coeff(1130)=0.119152597845D+04
coeff(1131)=-0.637070892365D+04
coeff(1132)=0.600935131025D+04
coeff(1133)=-0.556730077970D+04
coeff(1134)=-0.352010105301D+05
coeff(1135)=0.166020178922D+05
coeff(1136)=-0.496976514094D+04
coeff(1137)=-0.530895796165D+04
coeff(1138)=0.837617536066D+04
coeff(1139)=0.438359346713D+04
coeff(1140)=0.136667666388D+03
coeff(1141)=-0.546426357950D+04
coeff(1142)=0.816480709816D+04
coeff(1143)=-0.177448128106D+05
coeff(1144)=-0.192115529969D+05
coeff(1145)=-0.320956318103D+05
coeff(1146)=0.232105939754D+05
coeff(1147)=-0.885644253599D+04
coeff(1148)=-0.836960403637D+04
coeff(1149)=-0.107589560276D+04
coeff(1150)=-0.975445230041D+04
coeff(1151)=-0.574949739136D+04
coeff(1152)=0.127307332207D+05
coeff(1153)=-0.247742457500D+04
coeff(1154)=0.511882462297D+04
coeff(1155)=0.147071558605D+05
coeff(1156)=0.125299172375D+05
coeff(1157)=0.614567544305D+04
coeff(1158)=-0.199335344529D+05
coeff(1159)=-0.397636911853D+04
coeff(1160)=-0.777495995925D+04
coeff(1161)=0.145301431439D+04
coeff(1162)=0.152713071690D+05
coeff(1163)=-0.434539091780D+04
coeff(1164)=0.455724178529D+03
coeff(1165)=0.458278449400D+04
coeff(1166)=0.843415635779D+04
coeff(1167)=-0.187910538325D+04
coeff(1168)=0.239295679558D+04
coeff(1169)=-0.446696408587D+04
coeff(1170)=0.309970728144D+04
coeff(1171)=-0.348823418159D+04
coeff(1172)=0.180310490908D+05
coeff(1173)=-0.294898274968D+04
coeff(1174)=-0.905901956507D+04
coeff(1175)=-0.521375984610D+04
coeff(1176)=0.605251654143D+04
coeff(1177)=0.220895271851D+04
coeff(1178)=-0.758760717181D+04
coeff(1179)=0.585143816827D+04
coeff(1180)=-0.137789212803D+04
coeff(1181)=-0.239029038662D+04
coeff(1182)=-0.333503972138D+04
coeff(1183)=-0.241281794836D+04
coeff(1184)=-0.357956509589D+04
coeff(1185)=0.223626196399D+04
coeff(1186)=0.389755178454D+04
coeff(1187)=0.389764113508D+04
coeff(1188)=-0.238667351071D+04
coeff(1189)=0.707851749774D+04
coeff(1190)=-0.651353207427D+03
coeff(1191)=-0.609431340566D+04
coeff(1192)=0.161550123952D+04
coeff(1193)=0.210751921346D+03
coeff(1194)=-0.143021645831D+04
coeff(1195)=0.225790942648D+04
coeff(1196)=-0.131779036335D+04
coeff(1197)=0.405116085389D+04
coeff(1198)=-0.234751844895D+03
coeff(1199)=-0.458142999806D+04
coeff(1200)=0.611319536148D+04
coeff(1201)=-0.491681491757D+04
coeff(1202)=-0.577195655147D+04
coeff(1203)=0.628087766074D+04
coeff(1204)=0.557122467492D+04
coeff(1205)=0.117496110445D+04
coeff(1206)=-0.140154626835D+04
coeff(1207)=0.110539103571D+04
coeff(1208)=0.123795522639D+04
coeff(1209)=0.406368035714D+04
coeff(1210)=-0.128509371972D+05
coeff(1211)=0.377862342268D+04
coeff(1212)=-0.174161543961D+04
coeff(1213)=0.654181856639D+04
coeff(1214)=0.528164648645D+04
coeff(1215)=0.245069895339D+04
coeff(1216)=-0.693938861307D+03
coeff(1217)=-0.887216704986D+04
coeff(1218)=-0.629338411811D+04
coeff(1219)=0.415606391532D+04
coeff(1220)=-0.456314274040D+04
coeff(1221)=-0.211618665779D+04
coeff(1222)=0.129543235611D+04
coeff(1223)=-0.626699811537D+03
coeff(1224)=0.156494766207D+05
coeff(1225)=0.690405290432D+04
coeff(1226)=-0.477233452943D+04
coeff(1227)=-0.377115546869D+04
coeff(1228)=-0.948963054923D+03
coeff(1229)=0.706667175896D+04
coeff(1230)=0.602228007235D+04
coeff(1231)=0.585266724721D+03
coeff(1232)=0.375958472092D+04
coeff(1233)=0.964440293393D+04
coeff(1234)=-0.208203816777D+04
coeff(1235)=0.353238998085D+04
coeff(1236)=-0.513655602613D+03
coeff(1237)=-0.326105155700D+04
coeff(1238)=-0.229737774101D+04
coeff(1239)=-0.977195859781D+04
coeff(1240)=-0.397355021636D+04
coeff(1241)=0.584338284366D+04
coeff(1242)=0.197034903929D+02
coeff(1243)=-0.819677809246D+04
coeff(1244)=0.129155504347D+04
coeff(1245)=-0.107966510371D+02
coeff(1246)=0.160396721091D+04
coeff(1247)=0.551831995563D+04
coeff(1248)=-0.677627236828D+04
coeff(1249)=-0.555607309988D+04
coeff(1250)=0.364498641716D+04
coeff(1251)=0.376413776618D+04
coeff(1252)=0.360722380928D+04
coeff(1253)=-0.756064954870D+04
coeff(1254)=0.697650650146D+04
coeff(1255)=0.213647808336D+04
coeff(1256)=-0.605367708237D+04
coeff(1257)=-0.400944171487D+04
coeff(1258)=0.737496660054D+04
coeff(1259)=0.823087768265D+03
coeff(1260)=-0.492795083989D+04
coeff(1261)=-0.269029439713D+04
coeff(1262)=-0.548466707933D+04
coeff(1263)=0.210744790046D+04
coeff(1264)=-0.495556794293D+04
coeff(1265)=-0.246270643095D+04
coeff(1266)=0.109431019601D+04
coeff(1267)=0.770656718899D+04
coeff(1268)=-0.558985159522D+04
coeff(1269)=-0.215915300180D+04
coeff(1270)=-0.907566744532D+04
coeff(1271)=0.101572502306D+05
coeff(1272)=0.137744497180D+05
coeff(1273)=-0.337959936048D+04
coeff(1274)=-0.729418530143D+04
coeff(1275)=0.111743357562D+04
coeff(1276)=0.206165866161D+04
coeff(1277)=0.469464878724D+04
coeff(1278)=-0.341710681496D+04
coeff(1279)=-0.420554627978D+04
coeff(1280)=-0.358775241294D+04
coeff(1281)=-0.151990633273D+03
coeff(1282)=0.168278666683D+04
coeff(1283)=0.270327162100D+04
coeff(1284)=-0.261008589882D+04
coeff(1285)=0.203358695112D+04
coeff(1286)=0.101821694932D+04
coeff(1287)=0.464351483262D+04
coeff(1288)=-0.413955044988D+04
coeff(1289)=0.114119491513D+04
coeff(1290)=-0.265602016495D+02
coeff(1291)=0.866199251434D+03
coeff(1292)=-0.314785590221D+04
coeff(1293)=0.136708688425D+04
coeff(1294)=-0.171593325786D+03
coeff(1295)=-0.658135716656D+03
coeff(1296)=-0.986024736002D+04
coeff(1297)=0.918316658746D+04
coeff(1298)=-0.575756119788D+04
coeff(1299)=0.333394907145D+03
coeff(1300)=-0.469641168166D+02
  return
end if

entry u23_dc_10(xyz,v,s)

r12=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r13=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r14=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r23=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r24=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r34=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

!x(1)=exp(-((r12-ra1))/a1)
!x(2)=exp(-((r13-ra1))/a1)
!x(3)=exp(-((r14-ra2))/a2)
!x(4)=exp(-((r23-ra1))/a1)
!x(5)=exp(-((r24-ra2))/a2)
!x(6)=exp(-((r34-ra2))/a2)
 
x(1)=exp(-((r12-ra1))/a1 -((r12-rb1)**2)/b1)
x(2)=exp(-((r13-ra1))/a1 -((r13-rb1)**2)/b1)
x(3)=exp(-((r14-ra2))/a2 -((r14-rb2)**2)/b2)
x(4)=exp(-((r23-ra1))/a1 -((r23-rb1)**2)/b1)
x(5)=exp(-((r24-ra2))/a2 -((r24-rb2)**2)/b2)
x(6)=exp(-((r34-ra2))/a2 -((r34-rb2)**2)/b2)

do i=1,3
do j=1,3
rrel(i,j)=xyz(i,j)-xyz(4,j)
enddo
enddo

!compute r2xr3
cross(1)=rrel(2,2)*rrel(3,3)-rrel(2,3)*rrel(3,2)
cross(2)=-rrel(2,1)*rrel(3,3)+rrel(2,3)*rrel(3,1)
cross(3)=rrel(2,1)*rrel(3,2)-rrel(2,2)*rrel(3,1)
!compute p*r1*(r2xr3)
volume=0.00000
do i=1,3
volume=volume+rrel(1,i)*cross(i)
enddo
volume=tanh(15.4*volume)

call ev_pol_con_10(x,q)
v=dot_product(coeff,q)*volume/627.509474
s=dot_product(coeff,q)/627.509474

end  subroutine

!===========================================================================
! *def ev_pol_con_12(x,q)
! subtoutine of evaluating polynomials at 12th order
!===========================================================================
subroutine ev_pol_con_12(x,q)
implicit none
integer :: j
real*8,dimension(1:6),intent(in)::x
real*8,dimension(1:3075),intent(out)::q
real*8,dimension(0:32) :: rm
real*8,dimension(0:3392) :: p
real*8 :: bm1(3394), bm2(3076)

rm(0) = 1.0D0
rm(1) = x(4)
rm(2) = x(2)
rm(3) = x(1)
rm(4) = x(6)
rm(5) = x(5)
rm(6) = x(3)
rm(7) = rm(1)*rm(2)
rm(8) = rm(1)*rm(3)
rm(9) = rm(2)*rm(3)
rm(10) = rm(3)*rm(4)
rm(11) = rm(2)*rm(5)
rm(12) = rm(1)*rm(6)
rm(13) = rm(4)*rm(5)
rm(14) = rm(4)*rm(6)
rm(15) = rm(5)*rm(6)
rm(16) = rm(1)*rm(9)
rm(17) = rm(1)*rm(10)
rm(18) = rm(2)*rm(10)
rm(19) = rm(1)*rm(11)
rm(20) = rm(3)*rm(11)
rm(21) = rm(2)*rm(12)
rm(22) = rm(3)*rm(12)
rm(23) = rm(2)*rm(13)
rm(24) = rm(3)*rm(13)
rm(25) = rm(1)*rm(14)
rm(26) = rm(3)*rm(14)
rm(27) = rm(1)*rm(15)
rm(28) = rm(2)*rm(15)
rm(29) = rm(4)*rm(15)
rm(30) = rm(2)*rm(24)
rm(31) = rm(1)*rm(26)
rm(32) = rm(1)*rm(28)

p(0) = rm(0)
p(1) = rm(1) + rm(2) + rm(3)
p(2) = rm(4) + rm(5) + rm(6)
p(3) = rm(7) + rm(8) + rm(9)
p(4) = rm(10) + rm(11) + rm(12)
p(5) = p(1)*p(2) - p(4)
p(6) = rm(13) + rm(14) + rm(15)
p(7) = p(1)*p(1) - p(3) - p(3)
p(8) = p(2)*p(2) - p(6) - p(6)
p(9) = rm(16)
p(10) = rm(17) + rm(18) + rm(19) + rm(20) + rm(21) + rm(22)
p(11) = p(2)*p(3) - p(10)
p(12) = rm(23) + rm(24) + rm(25) + rm(26) + rm(27) + rm(28)
p(13) = p(1)*p(6) - p(12)
p(14) = rm(29)
p(15) = p(1)*p(3) - p(9) - p(9) - p(9)
p(16) = p(1)*p(4) - p(10)
p(17) = p(2)*p(7) - p(16)
p(18) = p(2)*p(4) - p(12)
p(19) = p(1)*p(8) - p(18)
p(20) = p(2)*p(6) - p(14) - p(14) - p(14)
p(21) = p(1)*p(7) - p(15)
p(22) = p(2)*p(8) - p(20)
p(23) = p(9)*p(2)
p(24) = rm(30) + rm(31) + rm(32)
p(25) = p(3)*p(6) - p(24)
p(26) = p(14)*p(1)
p(27) = p(9)*p(1)
p(28) = p(3)*p(4) - p(23)
p(29) = p(1)*p(10) - p(23) - p(28) - p(23)
p(30) = p(1)*p(11) - p(23)
p(31) = p(1)*p(12) - p(25) - p(24) - p(24)
p(32) = p(1)*p(13) - p(25)
p(33) = p(4)*p(5) - p(25) - p(31)
p(34) = p(2)*p(11) - p(25)
p(35) = p(4)*p(6) - p(26)
p(36) = p(2)*p(12) - p(26) - p(35) - p(26)
p(37) = p(2)*p(13) - p(26)
p(38) = p(14)*p(2)
p(39) = p(3)*p(3) - p(27) - p(27)
p(40) = p(3)*p(7) - p(27)
p(41) = p(1)*p(16) - p(28)
p(42) = p(2)*p(21) - p(41)
p(43) = p(1)*p(18) - p(33)
p(44) = p(7)*p(8) - p(43)
p(45) = p(6)*p(6) - p(38) - p(38)
p(46) = p(2)*p(18) - p(35)
p(47) = p(1)*p(22) - p(46)
p(48) = p(6)*p(8) - p(38)
p(49) = p(1)*p(21) - p(40)
p(50) = p(2)*p(22) - p(48)
p(51) = p(9)*p(6)
p(52) = p(14)*p(3)
p(53) = p(9)*p(4)
p(54) = p(9)*p(5)
p(55) = p(1)*p(24) - p(51)
p(56) = p(3)*p(12) - p(51) - p(55) - p(51)
p(57) = p(3)*p(13) - p(51)
p(58) = p(14)*p(7)
p(59) = p(9)*p(8)
p(60) = p(2)*p(24) - p(52)
p(61) = p(4)*p(13) - p(58)
p(62) = p(6)*p(11) - p(52)
p(63) = p(14)*p(4)
p(64) = p(14)*p(5)
p(65) = p(9)*p(3)
p(66) = p(9)*p(7)
p(67) = p(3)*p(16) - p(53)
p(68) = p(4)*p(15) - p(54) - p(67)
p(69) = p(2)*p(39) - p(68)
p(70) = p(1)*p(29) - p(54) - p(68)
p(71) = p(7)*p(11) - p(53)
p(72) = p(1)*p(31) - p(56) - p(55)
p(73) = p(1)*p(32) - p(57)
p(74) = p(3)*p(18) - p(59)
p(75) = p(2)*p(29) - p(57) - p(55)
p(76) = p(1)*p(34) - p(59)
p(77) = p(6)*p(16) - p(58)
p(78) = p(1)*p(36) - p(62) - p(60)
p(79) = p(2)*p(32) - p(58)
p(80) = p(6)*p(12) - p(64) - p(63) - p(63)
p(81) = p(1)*p(45) - p(80)
p(82) = p(14)*p(6)
p(83) = p(2)*p(33) - p(61) - p(60)
p(84) = p(2)*p(34) - p(62)
p(85) = p(6)*p(18) - p(63)
p(86) = p(2)*p(36) - p(64) - p(80)
p(87) = p(8)*p(13) - p(63)
p(88) = p(14)*p(8)
p(89) = p(1)*p(39) - p(65)
p(90) = p(3)*p(21) - p(66)
p(91) = p(1)*p(41) - p(67)
p(92) = p(2)*p(49) - p(91)
p(93) = p(1)*p(43) - p(74)
p(94) = p(8)*p(21) - p(93)
p(95) = p(1)*p(46) - p(83)
p(96) = p(7)*p(22) - p(95)
p(97) = p(2)*p(45) - p(82)
p(98) = p(2)*p(46) - p(85)
p(99) = p(1)*p(50) - p(98)
p(100) = p(6)*p(22) - p(88)
p(101) = p(1)*p(49) - p(90)
p(102) = p(2)*p(50) - p(100)
p(103) = p(9)*p(14)
p(104) = p(9)*p(12)
p(105) = p(9)*p(13)
p(106) = p(14)*p(15)
p(107) = p(9)*p(20)
p(108) = p(14)*p(10)
p(109) = p(14)*p(11)
p(110) = p(9)*p(16)
p(111) = p(9)*p(10)
p(112) = p(9)*p(11)
p(113) = p(9)*p(17)
p(114) = p(3)*p(24) - p(104)
p(115) = p(7)*p(24) - p(105)
p(116) = p(3)*p(31) - p(104) - p(115)
p(117) = p(6)*p(39) - p(114)
p(118) = p(3)*p(32) - p(105)
p(119) = p(14)*p(21)
p(120) = p(9)*p(18)
p(121) = p(9)*p(19)
p(122) = p(4)*p(24) - p(103) - p(103) - p(103)
p(123) = p(1)*p(60) - p(107) - p(122)
p(124) = p(13)*p(16) - p(119)
p(125) = p(2)*p(56) - p(106) - p(124)
p(126) = p(4)*p(32) - p(119)
p(127) = p(1)*p(62) - p(107) - p(125)
p(128) = p(14)*p(16)
p(129) = p(14)*p(17)
p(130) = p(6)*p(24) - p(108)
p(131) = p(3)*p(45) - p(130)
p(132) = p(14)*p(12)
p(133) = p(14)*p(13)
p(134) = p(9)*p(22)
p(135) = p(8)*p(24) - p(109)
p(136) = p(13)*p(18) - p(128)
p(137) = p(6)*p(34) - p(109)
p(138) = p(14)*p(18)
p(139) = p(14)*p(19)
p(140) = p(9)*p(9)
p(141) = p(9)*p(15)
p(142) = p(9)*p(21)
p(143) = p(3)*p(41) - p(110)
p(144) = p(4)*p(39) - p(112)
p(145) = p(1)*p(68) - p(111) - p(144)
p(146) = p(1)*p(69) - p(112)
p(147) = p(1)*p(70) - p(113) - p(145)
p(148) = p(11)*p(21) - p(110)
p(149) = p(1)*p(72) - p(116) - p(115)
p(150) = p(1)*p(73) - p(118)
p(151) = p(3)*p(43) - p(120)
p(152) = p(15)*p(18) - p(121) - p(151)
p(153) = p(2)*p(69) - p(117)
p(154) = p(1)*p(75) - p(121) - p(152)
p(155) = p(7)*p(34) - p(120)
p(156) = p(6)*p(41) - p(119)
p(157) = p(1)*p(78) - p(125) - p(123)
p(158) = p(2)*p(73) - p(119)
p(159) = p(1)*p(80) - p(131) - p(130) - p(130)
p(160) = p(1)*p(81) - p(131)
p(161) = p(14)*p(14)
p(162) = p(3)*p(46) - p(134)
p(163) = p(2)*p(75) - p(126) - p(123)
p(164) = p(1)*p(84) - p(134)
p(165) = p(6)*p(43) - p(128)
p(166) = p(1)*p(86) - p(137) - p(135)
p(167) = p(8)*p(32) - p(128)
p(168) = p(4)*p(45) - p(133)
p(169) = p(2)*p(80) - p(132) - p(168)
p(170) = p(2)*p(81) - p(133)
p(171) = p(14)*p(20)
p(172) = p(2)*p(83) - p(136) - p(135)
p(173) = p(2)*p(84) - p(137)
p(174) = p(6)*p(46) - p(138)
p(175) = p(2)*p(86) - p(139) - p(169)
p(176) = p(13)*p(22) - p(138)
p(177) = p(14)*p(22)
p(178) = p(3)*p(39) - p(141)
p(179) = p(1)*p(89) - p(141) - p(178) - p(178)
p(180) = p(3)*p(49) - p(142)
p(181) = p(1)*p(91) - p(143)
p(182) = p(2)*p(101) - p(181)
p(183) = p(1)*p(93) - p(151)
p(184) = p(8)*p(49) - p(183)
p(185) = p(1)*p(95) - p(162)
p(186) = p(21)*p(22) - p(185)
p(187) = p(6)*p(45) - p(171)
p(188) = p(1)*p(98) - p(172)
p(189) = p(7)*p(50) - p(188)
p(190) = p(2)*p(97) - p(171) - p(187) - p(187)
p(191) = p(2)*p(98) - p(174)
p(192) = p(1)*p(102) - p(191)
p(193) = p(6)*p(50) - p(177)
p(194) = p(1)*p(101) - p(180)
p(195) = p(2)*p(102) - p(193)
p(196) = p(9)*p(26)
p(197) = p(9)*p(38)
p(198) = p(9)*p(24)
p(199) = p(9)*p(31)
p(200) = p(9)*p(25)
p(201) = p(9)*p(32)
p(202) = p(14)*p(39)
p(203) = p(14)*p(40)
p(204) = p(9)*p(35)
p(205) = p(9)*p(36)
p(206) = p(9)*p(37)
p(207) = p(14)*p(28)
p(208) = p(14)*p(29)
p(209) = p(14)*p(30)
p(210) = p(9)*p(45)
p(211) = p(14)*p(24)
p(212) = p(14)*p(25)
p(213) = p(9)*p(48)
p(214) = p(14)*p(33)
p(215) = p(14)*p(34)
p(216) = p(9)*p(41)
p(217) = p(9)*p(28)
p(218) = p(9)*p(23)
p(219) = p(9)*p(29)
p(220) = p(9)*p(30)
p(221) = p(9)*p(42)
p(222) = p(1)*p(114) - p(198)
p(223) = p(21)*p(24) - p(201)
p(224) = p(3)*p(72) - p(199) - p(223)
p(225) = p(1)*p(116) - p(199) - p(224)
p(226) = p(13)*p(39) - p(198)
p(227) = p(3)*p(73) - p(201)
p(228) = p(14)*p(49)
p(229) = p(9)*p(43)
p(230) = p(9)*p(33)
p(231) = p(9)*p(34)
p(232) = p(9)*p(44)
p(233) = p(16)*p(24) - p(196)
p(234) = p(2)*p(114) - p(202)
p(235) = p(1)*p(123) - p(205) - p(234)
p(236) = p(13)*p(41) - p(228)
p(237) = p(2)*p(116) - p(203) - p(236)
p(238) = p(16)*p(32) - p(228)
p(239) = p(6)*p(69) - p(202)
p(240) = p(4)*p(73) - p(228)
p(241) = p(11)*p(32) - p(196)
p(242) = p(14)*p(41)
p(243) = p(14)*p(42)
p(244) = p(1)*p(130) - p(210)
p(245) = p(4)*p(62) - p(213) - p(209)
p(246) = p(3)*p(81) - p(210)
p(247) = p(14)*p(31)
p(248) = p(14)*p(32)
p(249) = p(14)*p(26)
p(250) = p(9)*p(46)
p(251) = p(9)*p(47)
p(252) = p(18)*p(24) - p(197)
p(253) = p(1)*p(135) - p(213) - p(252)
p(254) = p(13)*p(43) - p(242)
p(255) = p(2)*p(125) - p(209) - p(245)
p(256) = p(18)*p(32) - p(242)
p(257) = p(13)*p(34) - p(197)
p(258) = p(14)*p(43)
p(259) = p(14)*p(44)
p(260) = p(2)*p(130) - p(211)
p(261) = p(4)*p(81) - p(248)
p(262) = p(11)*p(45) - p(211)
p(263) = p(14)*p(35)
p(264) = p(14)*p(36)
p(265) = p(14)*p(37)
p(266) = p(9)*p(50)
p(267) = p(22)*p(24) - p(215)
p(268) = p(13)*p(46) - p(258)
p(269) = p(6)*p(84) - p(215)
p(270) = p(14)*p(46)
p(271) = p(14)*p(47)
p(272) = p(9)*p(27)
p(273) = p(9)*p(39)
p(274) = p(9)*p(40)
p(275) = p(9)*p(49)
p(276) = p(3)*p(91) - p(216)
p(277) = p(16)*p(39) - p(218)
p(278) = p(1)*p(144) - p(217) - p(277)
p(279) = p(2)*p(178) - p(278)
p(280) = p(1)*p(145) - p(219) - p(278)
p(281) = p(7)*p(69) - p(218)
p(282) = p(1)*p(147) - p(221) - p(280)
p(283) = p(11)*p(49) - p(216)
p(284) = p(1)*p(149) - p(224) - p(223)
p(285) = p(1)*p(150) - p(227)
p(286) = p(3)*p(93) - p(229)
p(287) = p(18)*p(39) - p(231)
p(288) = p(1)*p(152) - p(230) - p(287)
p(289) = p(1)*p(153) - p(231)
p(290) = p(1)*p(154) - p(232) - p(288)
p(291) = p(21)*p(34) - p(229)
p(292) = p(6)*p(91) - p(228)
p(293) = p(1)*p(157) - p(237) - p(235)
p(294) = p(2)*p(150) - p(228)
p(295) = p(1)*p(159) - p(245) - p(244)
p(296) = p(1)*p(160) - p(246)
p(297) = p(3)*p(95) - p(250)
p(298) = p(2)*p(152) - p(238) - p(234)
p(299) = p(2)*p(153) - p(239)
p(300) = p(1)*p(163) - p(251) - p(298)
p(301) = p(7)*p(84) - p(250)
p(302) = p(6)*p(93) - p(242)
p(303) = p(1)*p(166) - p(255) - p(253)
p(304) = p(8)*p(73) - p(242)
p(305) = p(16)*p(45) - p(248)
p(306) = p(1)*p(169) - p(262) - p(260)
p(307) = p(2)*p(160) - p(248)
p(308) = p(14)*p(38)
p(309) = p(6)*p(80) - p(264) - p(263)
p(310) = p(1)*p(187) - p(309)
p(311) = p(14)*p(45)
p(312) = p(3)*p(98) - p(266)
p(313) = p(2)*p(163) - p(256) - p(253)
p(314) = p(1)*p(173) - p(266)
p(315) = p(6)*p(95) - p(258)
p(316) = p(1)*p(175) - p(269) - p(267)
p(317) = p(22)*p(32) - p(258)
p(318) = p(18)*p(45) - p(249)
p(319) = p(2)*p(169) - p(264) - p(309)
p(320) = p(8)*p(81) - p(249)
p(321) = p(14)*p(48)
p(322) = p(2)*p(172) - p(268) - p(267)
p(323) = p(2)*p(173) - p(269)
p(324) = p(6)*p(98) - p(270)
p(325) = p(2)*p(175) - p(271) - p(319)
p(326) = p(13)*p(50) - p(270)
p(327) = p(14)*p(50)
p(328) = p(1)*p(178) - p(273)
p(329) = p(21)*p(39) - p(272)
p(330) = p(3)*p(101) - p(275)
p(331) = p(1)*p(181) - p(276)
p(332) = p(2)*p(194) - p(331)
p(333) = p(1)*p(183) - p(286)
p(334) = p(8)*p(101) - p(333)
p(335) = p(1)*p(185) - p(297)
p(336) = p(22)*p(49) - p(335)
p(337) = p(1)*p(188) - p(312)
p(338) = p(21)*p(50) - p(337)
p(339) = p(2)*p(187) - p(311)
p(340) = p(1)*p(191) - p(322)
p(341) = p(7)*p(102) - p(340)
p(342) = p(22)*p(45) - p(308)
p(343) = p(2)*p(191) - p(324)
p(344) = p(1)*p(195) - p(343)
p(345) = p(6)*p(102) - p(327)
p(346) = p(1)*p(194) - p(330)
p(347) = p(2)*p(195) - p(345)
p(348) = p(9)*p(52)
p(349) = p(9)*p(58)
p(350) = p(9)*p(63)
p(351) = p(9)*p(64)
p(352) = p(9)*p(82)
p(353) = p(9)*p(88)
p(354) = p(9)*p(55)
p(355) = p(9)*p(72)
p(356) = p(9)*p(51)
p(357) = p(9)*p(56)
p(358) = p(9)*p(57)
p(359) = p(9)*p(73)
p(360) = p(14)*p(89)
p(361) = p(14)*p(90)
p(362) = p(9)*p(77)
p(363) = p(9)*p(60)
p(364) = p(9)*p(78)
p(365) = p(9)*p(61)
p(366) = p(9)*p(62)
p(367) = p(9)*p(79)
p(368) = p(14)*p(67)
p(369) = p(14)*p(68)
p(370) = p(14)*p(69)
p(371) = p(14)*p(70)
p(372) = p(14)*p(71)
p(373) = p(9)*p(80)
p(374) = p(9)*p(81)
p(375) = p(14)*p(55)
p(376) = p(14)*p(56)
p(377) = p(14)*p(57)
p(378) = p(14)*p(52)
p(379) = p(9)*p(85)
p(380) = p(9)*p(86)
p(381) = p(9)*p(87)
p(382) = p(14)*p(74)
p(383) = p(14)*p(75)
p(384) = p(14)*p(76)
p(385) = p(9)*p(97)
p(386) = p(14)*p(60)
p(387) = p(14)*p(61)
p(388) = p(14)*p(62)
p(389) = p(9)*p(100)
p(390) = p(14)*p(83)
p(391) = p(14)*p(84)
p(392) = p(9)*p(91)
p(393) = p(9)*p(67)
p(394) = p(9)*p(53)
p(395) = p(9)*p(68)
p(396) = p(9)*p(54)
p(397) = p(9)*p(69)
p(398) = p(9)*p(70)
p(399) = p(9)*p(71)
p(400) = p(9)*p(92)
p(401) = p(3)*p(114) - p(354)
p(402) = p(7)*p(114) - p(356)
p(403) = p(24)*p(49) - p(359)
p(404) = p(3)*p(149) - p(355) - p(403)
p(405) = p(1)*p(224) - p(355) - p(404)
p(406) = p(6)*p(178) - p(401)
p(407) = p(32)*p(39) - p(356)
p(408) = p(3)*p(150) - p(359)
p(409) = p(14)*p(101)
p(410) = p(9)*p(93)
p(411) = p(9)*p(74)
p(412) = p(9)*p(59)
p(413) = p(9)*p(75)
p(414) = p(9)*p(76)
p(415) = p(9)*p(94)
p(416) = p(24)*p(41) - p(349)
p(417) = p(4)*p(114) - p(348)
p(418) = p(1)*p(234) - p(363) - p(417)
p(419) = p(1)*p(235) - p(364) - p(418)
p(420) = p(13)*p(91) - p(409)
p(421) = p(2)*p(224) - p(361) - p(420)
p(422) = p(32)*p(41) - p(409)
p(423) = p(1)*p(237) - p(364) - p(421)
p(424) = p(16)*p(73) - p(409)
p(425) = p(13)*p(69) - p(348)
p(426) = p(4)*p(150) - p(409)
p(427) = p(11)*p(73) - p(349)
p(428) = p(14)*p(91)
p(429) = p(14)*p(92)
p(430) = p(3)*p(130) - p(373)
p(431) = p(7)*p(130) - p(374)
p(432) = p(3)*p(159) - p(373) - p(431)
p(433) = p(39)*p(45) - p(430)
p(434) = p(3)*p(160) - p(374)
p(435) = p(14)*p(72)
p(436) = p(14)*p(73)
p(437) = p(14)*p(58)
p(438) = p(9)*p(95)
p(439) = p(9)*p(83)
p(440) = p(9)*p(84)
p(441) = p(9)*p(96)
p(442) = p(24)*p(43) - p(350)
p(443) = p(8)*p(114) - p(370)
p(444) = p(1)*p(253) - p(380) - p(443)
p(445) = p(13)*p(93) - p(428)
p(446) = p(2)*p(237) - p(372) - p(432)
p(447) = p(32)*p(43) - p(428)
p(448) = p(6)*p(153) - p(370)
p(449) = p(18)*p(73) - p(428)
p(450) = p(32)*p(34) - p(350)
p(451) = p(14)*p(93)
p(452) = p(14)*p(94)
p(453) = p(4)*p(130) - p(352)
p(454) = p(1)*p(260) - p(385) - p(453)
p(455) = p(16)*p(81) - p(436)
p(456) = p(2)*p(245) - p(376) - p(455)
p(457) = p(4)*p(160) - p(436)
p(458) = p(11)*p(81) - p(352)
p(459) = p(14)*p(77)
p(460) = p(14)*p(78)
p(461) = p(14)*p(79)
p(462) = p(14)*p(63)
p(463) = p(14)*p(64)
p(464) = p(6)*p(130) - p(386)
p(465) = p(3)*p(187) - p(464)
p(466) = p(14)*p(80)
p(467) = p(14)*p(81)
p(468) = p(9)*p(98)
p(469) = p(9)*p(99)
p(470) = p(24)*p(46) - p(353)
p(471) = p(1)*p(267) - p(389) - p(470)
p(472) = p(13)*p(95) - p(451)
p(473) = p(2)*p(255) - p(384) - p(456)
p(474) = p(32)*p(46) - p(451)
p(475) = p(13)*p(84) - p(353)
p(476) = p(14)*p(95)
p(477) = p(14)*p(96)
p(478) = p(8)*p(130) - p(378)
p(479) = p(18)*p(81) - p(437)
p(480) = p(34)*p(45) - p(378)
p(481) = p(14)*p(85)
p(482) = p(14)*p(86)
p(483) = p(14)*p(87)
p(484) = p(9)*p(102)
p(485) = p(24)*p(50) - p(391)
p(486) = p(13)*p(98) - p(476)
p(487) = p(6)*p(173) - p(391)
p(488) = p(14)*p(98)
p(489) = p(14)*p(99)
p(490) = p(9)*p(65)
p(491) = p(9)*p(66)
p(492) = p(9)*p(89)
p(493) = p(9)*p(90)
p(494) = p(9)*p(101)
p(495) = p(3)*p(181) - p(392)
p(496) = p(39)*p(41) - p(394)
p(497) = p(4)*p(178) - p(397)
p(498) = p(1)*p(278) - p(395) - p(497)
p(499) = p(1)*p(279) - p(397)
p(500) = p(1)*p(280) - p(398) - p(498)
p(501) = p(21)*p(69) - p(394)
p(502) = p(1)*p(282) - p(400) - p(500)
p(503) = p(11)*p(101) - p(392)
p(504) = p(1)*p(284) - p(404) - p(403)
p(505) = p(1)*p(285) - p(408)
p(506) = p(3)*p(183) - p(410)
p(507) = p(39)*p(43) - p(412)
p(508) = p(1)*p(287) - p(411) - p(507)
p(509) = p(2)*p(279) - p(406)
p(510) = p(1)*p(288) - p(413) - p(508)
p(511) = p(7)*p(153) - p(412)
p(512) = p(1)*p(290) - p(415) - p(510)
p(513) = p(34)*p(49) - p(410)
p(514) = p(6)*p(181) - p(409)
p(515) = p(1)*p(293) - p(421) - p(419)
p(516) = p(2)*p(285) - p(409)
p(517) = p(1)*p(295) - p(432) - p(431)
p(518) = p(1)*p(296) - p(434)
p(519) = p(3)*p(185) - p(438)
p(520) = p(39)*p(46) - p(440)
p(521) = p(1)*p(298) - p(439) - p(520)
p(522) = p(1)*p(299) - p(440)
p(523) = p(1)*p(300) - p(441) - p(521)
p(524) = p(21)*p(84) - p(438)
p(525) = p(6)*p(183) - p(428)
p(526) = p(1)*p(303) - p(446) - p(444)
p(527) = p(8)*p(150) - p(428)
p(528) = p(41)*p(45) - p(436)
p(529) = p(1)*p(306) - p(456) - p(454)
p(530) = p(2)*p(296) - p(436)
p(531) = p(4)*p(169) - p(388) - p(478)
p(532) = p(1)*p(310) - p(465)
p(533) = p(14)*p(82)
p(534) = p(3)*p(188) - p(468)
p(535) = p(2)*p(298) - p(447) - p(443)
p(536) = p(2)*p(299) - p(448)
p(537) = p(1)*p(313) - p(469) - p(535)
p(538) = p(7)*p(173) - p(468)
p(539) = p(6)*p(185) - p(451)
p(540) = p(1)*p(316) - p(473) - p(471)
p(541) = p(22)*p(73) - p(451)
p(542) = p(43)*p(45) - p(437)
p(543) = p(1)*p(319) - p(480) - p(478)
p(544) = p(8)*p(160) - p(437)
p(545) = p(14)*p(88)
p(546) = p(4)*p(187) - p(467)
p(547) = p(2)*p(309) - p(466) - p(546)
p(548) = p(2)*p(310) - p(467)
p(549) = p(14)*p(97)
p(550) = p(3)*p(191) - p(484)
p(551) = p(2)*p(313) - p(474) - p(471)
p(552) = p(1)*p(323) - p(484)
p(553) = p(6)*p(188) - p(476)
p(554) = p(1)*p(325) - p(487) - p(485)
p(555) = p(32)*p(50) - p(476)
p(556) = p(45)*p(46) - p(462)
p(557) = p(2)*p(319) - p(482) - p(547)
p(558) = p(22)*p(81) - p(462)
p(559) = p(14)*p(100)
p(560) = p(2)*p(322) - p(486) - p(485)
p(561) = p(2)*p(323) - p(487)
p(562) = p(6)*p(191) - p(488)
p(563) = p(2)*p(325) - p(489) - p(557)
p(564) = p(13)*p(102) - p(488)
p(565) = p(14)*p(102)
p(566) = p(3)*p(178) - p(492)
p(567) = p(7)*p(178) - p(490)
p(568) = p(39)*p(49) - p(491)
p(569) = p(3)*p(194) - p(494)
p(570) = p(1)*p(331) - p(495)
p(571) = p(2)*p(346) - p(570)
p(572) = p(1)*p(333) - p(506)
p(573) = p(8)*p(194) - p(572)
p(574) = p(1)*p(335) - p(519)
p(575) = p(22)*p(101) - p(574)
p(576) = p(1)*p(337) - p(534)
p(577) = p(49)*p(50) - p(576)
p(578) = p(6)*p(187) - p(549)
p(579) = p(1)*p(340) - p(550)
p(580) = p(21)*p(102) - p(579)
p(581) = p(8)*p(187) - p(533)
p(582) = p(1)*p(343) - p(560)
p(583) = p(7)*p(195) - p(582)
p(584) = p(45)*p(50) - p(545)
p(585) = p(2)*p(343) - p(562)
p(586) = p(1)*p(347) - p(585)
p(587) = p(6)*p(195) - p(565)
p(588) = p(1)*p(346) - p(569)
p(589) = p(2)*p(347) - p(587)
p(590) = p(9)*p(103)
p(591) = p(9)*p(106)
p(592) = p(9)*p(119)
p(593) = p(9)*p(128)
p(594) = p(9)*p(108)
p(595) = p(9)*p(109)
p(596) = p(9)*p(129)
p(597) = p(9)*p(132)
p(598) = p(9)*p(133)
p(599) = p(9)*p(161)
p(600) = p(9)*p(138)
p(601) = p(9)*p(139)
p(602) = p(9)*p(171)
p(603) = p(9)*p(177)
p(604) = p(9)*p(114)
p(605) = p(9)*p(115)
p(606) = p(9)*p(149)
p(607) = p(9)*p(104)
p(608) = p(9)*p(116)
p(609) = p(9)*p(105)
p(610) = p(9)*p(117)
p(611) = p(9)*p(118)
p(612) = p(9)*p(150)
p(613) = p(14)*p(178)
p(614) = p(14)*p(179)
p(615) = p(14)*p(180)
p(616) = p(9)*p(156)
p(617) = p(9)*p(122)
p(618) = p(9)*p(123)
p(619) = p(9)*p(157)
p(620) = p(9)*p(124)
p(621) = p(9)*p(107)
p(622) = p(9)*p(125)
p(623) = p(9)*p(126)
p(624) = p(9)*p(127)
p(625) = p(9)*p(158)
p(626) = p(14)*p(143)
p(627) = p(14)*p(144)
p(628) = p(14)*p(145)
p(629) = p(14)*p(146)
p(630) = p(14)*p(147)
p(631) = p(14)*p(148)
p(632) = p(9)*p(130)
p(633) = p(9)*p(159)
p(634) = p(9)*p(131)
p(635) = p(9)*p(160)
p(636) = p(14)*p(114)
p(637) = p(14)*p(115)
p(638) = p(14)*p(116)
p(639) = p(14)*p(117)
p(640) = p(14)*p(118)
p(641) = p(14)*p(106)
p(642) = p(9)*p(165)
p(643) = p(9)*p(135)
p(644) = p(9)*p(166)
p(645) = p(9)*p(136)
p(646) = p(9)*p(137)
p(647) = p(9)*p(167)
p(648) = p(14)*p(151)
p(649) = p(14)*p(152)
p(650) = p(14)*p(153)
p(651) = p(14)*p(154)
p(652) = p(14)*p(155)
p(653) = p(9)*p(168)
p(654) = p(9)*p(169)
p(655) = p(9)*p(170)
p(656) = p(14)*p(122)
p(657) = p(14)*p(123)
p(658) = p(14)*p(124)
p(659) = p(14)*p(125)
p(660) = p(14)*p(126)
p(661) = p(14)*p(127)
p(662) = p(14)*p(108)
p(663) = p(14)*p(109)
p(664) = p(9)*p(187)
p(665) = p(14)*p(130)
p(666) = p(14)*p(131)
p(667) = p(9)*p(174)
p(668) = p(9)*p(175)
p(669) = p(9)*p(176)
p(670) = p(14)*p(162)
p(671) = p(14)*p(163)
p(672) = p(14)*p(164)
p(673) = p(9)*p(190)
p(674) = p(14)*p(135)
p(675) = p(14)*p(136)
p(676) = p(14)*p(137)
p(677) = p(9)*p(193)
p(678) = p(14)*p(172)
p(679) = p(14)*p(173)
p(680) = p(9)*p(181)
p(681) = p(9)*p(143)
p(682) = p(9)*p(110)
p(683) = p(9)*p(144)
p(684) = p(9)*p(111)
p(685) = p(9)*p(112)
p(686) = p(9)*p(145)
p(687) = p(9)*p(113)
p(688) = p(9)*p(146)
p(689) = p(9)*p(147)
p(690) = p(9)*p(148)
p(691) = p(9)*p(182)
p(692) = p(1)*p(401) - p(604)
p(693) = p(21)*p(114) - p(609)
p(694) = p(24)*p(101) - p(612)
p(695) = p(3)*p(284) - p(606) - p(694)
p(696) = p(1)*p(404) - p(606) - p(695)
p(697) = p(1)*p(405) - p(608) - p(696)
p(698) = p(13)*p(178) - p(604)
p(699) = p(39)*p(73) - p(609)
p(700) = p(3)*p(285) - p(612)
p(701) = p(14)*p(194)
p(702) = p(9)*p(183)
p(703) = p(9)*p(151)
p(704) = p(9)*p(120)
p(705) = p(9)*p(152)
p(706) = p(9)*p(121)
p(707) = p(9)*p(153)
p(708) = p(9)*p(154)
p(709) = p(9)*p(155)
p(710) = p(9)*p(184)
p(711) = p(24)*p(91) - p(592)
p(712) = p(1)*p(416) - p(616) - p(711)
p(713) = p(2)*p(401) - p(613)
p(714) = p(1)*p(418) - p(618) - p(713)
p(715) = p(1)*p(419) - p(619) - p(714)
p(716) = p(13)*p(181) - p(701)
p(717) = p(2)*p(404) - p(615) - p(716)
p(718) = p(32)*p(91) - p(701)
p(719) = p(1)*p(421) - p(619) - p(717)
p(720) = p(41)*p(73) - p(701)
p(721) = p(6)*p(279) - p(613)
p(722) = p(16)*p(150) - p(701)
p(723) = p(1)*p(425) - p(624) - p(721)
p(724) = p(4)*p(285) - p(701)
p(725) = p(11)*p(150) - p(592)
p(726) = p(14)*p(181)
p(727) = p(14)*p(182)
p(728) = p(1)*p(430) - p(632)
p(729) = p(21)*p(130) - p(635)
p(730) = p(3)*p(295) - p(633) - p(729)
p(731) = p(1)*p(432) - p(633) - p(730)
p(732) = p(39)*p(81) - p(632)
p(733) = p(3)*p(296) - p(635)
p(734) = p(14)*p(149)
p(735) = p(14)*p(150)
p(736) = p(14)*p(119)
p(737) = p(9)*p(185)
p(738) = p(9)*p(162)
p(739) = p(9)*p(134)
p(740) = p(9)*p(163)
p(741) = p(9)*p(164)
p(742) = p(9)*p(186)
p(743) = p(24)*p(93) - p(593)
p(744) = p(18)*p(114) - p(595)
p(745) = p(1)*p(443) - p(643) - p(744)
p(746) = p(1)*p(444) - p(644) - p(745)
p(747) = p(13)*p(183) - p(726)
p(748) = p(2)*p(421) - p(631) - p(730)
p(749) = p(32)*p(93) - p(726)
p(750) = p(1)*p(446) - p(644) - p(748)
p(751) = p(43)*p(73) - p(726)
p(752) = p(13)*p(153) - p(595)
p(753) = p(18)*p(150) - p(726)
p(754) = p(34)*p(73) - p(593)
p(755) = p(14)*p(183)
p(756) = p(14)*p(184)
p(757) = p(16)*p(130) - p(598)
p(758) = p(2)*p(430) - p(636)
p(759) = p(1)*p(454) - p(654) - p(758)
p(760) = p(41)*p(81) - p(735)
p(761) = p(2)*p(432) - p(638) - p(760)
p(762) = p(16)*p(160) - p(735)
p(763) = p(45)*p(69) - p(636)
p(764) = p(4)*p(296) - p(735)
p(765) = p(11)*p(160) - p(598)
p(766) = p(14)*p(156)
p(767) = p(14)*p(157)
p(768) = p(14)*p(158)
p(769) = p(14)*p(128)
p(770) = p(14)*p(129)
p(771) = p(1)*p(464) - p(664)
p(772) = p(4)*p(262) - p(673) - p(661)
p(773) = p(3)*p(310) - p(664)
p(774) = p(14)*p(159)
p(775) = p(14)*p(160)
p(776) = p(14)*p(132)
p(777) = p(14)*p(133)
p(778) = p(9)*p(188)
p(779) = p(9)*p(172)
p(780) = p(9)*p(173)
p(781) = p(9)*p(189)
p(782) = p(24)*p(95) - p(600)
p(783) = p(22)*p(114) - p(650)
p(784) = p(1)*p(471) - p(668) - p(783)
p(785) = p(13)*p(185) - p(755)
p(786) = p(2)*p(446) - p(652) - p(761)
p(787) = p(32)*p(95) - p(755)
p(788) = p(6)*p(299) - p(650)
p(789) = p(46)*p(73) - p(755)
p(790) = p(32)*p(84) - p(600)
p(791) = p(14)*p(185)
p(792) = p(14)*p(186)
p(793) = p(2)*p(453) - p(656) - p(771)
p(794) = p(1)*p(478) - p(673) - p(793)
p(795) = p(43)*p(81) - p(736)
p(796) = p(2)*p(456) - p(659) - p(772)
p(797) = p(18)*p(160) - p(736)
p(798) = p(1)*p(480) - p(673) - p(796)
p(799) = p(14)*p(165)
p(800) = p(14)*p(166)
p(801) = p(14)*p(167)
p(802) = p(14)*p(138)
p(803) = p(14)*p(139)
p(804) = p(2)*p(464) - p(665)
p(805) = p(4)*p(310) - p(775)
p(806) = p(11)*p(187) - p(665)
p(807) = p(14)*p(168)
p(808) = p(14)*p(169)
p(809) = p(14)*p(170)
p(810) = p(9)*p(191)
p(811) = p(9)*p(192)
p(812) = p(24)*p(98) - p(603)
p(813) = p(1)*p(485) - p(677) - p(812)
p(814) = p(13)*p(188) - p(791)
p(815) = p(2)*p(473) - p(672) - p(796)
p(816) = p(32)*p(98) - p(791)
p(817) = p(13)*p(173) - p(603)
p(818) = p(14)*p(188)
p(819) = p(14)*p(189)
p(820) = p(22)*p(130) - p(663)
p(821) = p(46)*p(81) - p(769)
p(822) = p(45)*p(84) - p(663)
p(823) = p(14)*p(174)
p(824) = p(14)*p(175)
p(825) = p(14)*p(176)
p(826) = p(9)*p(195)
p(827) = p(24)*p(102) - p(679)
p(828) = p(13)*p(191) - p(818)
p(829) = p(6)*p(323) - p(679)
p(830) = p(14)*p(191)
p(831) = p(14)*p(192)
p(832) = p(9)*p(140)
p(833) = p(9)*p(141)
p(834) = p(9)*p(178)
p(835) = p(9)*p(142)
p(836) = p(9)*p(179)
p(837) = p(9)*p(180)
p(838) = p(9)*p(194)
p(839) = p(3)*p(331) - p(680)
p(840) = p(39)*p(91) - p(682)
p(841) = p(16)*p(178) - p(685)
p(842) = p(1)*p(497) - p(683) - p(841)
p(843) = p(2)*p(566) - p(842)
p(844) = p(1)*p(498) - p(686) - p(842)
p(845) = p(7)*p(279) - p(685)
p(846) = p(1)*p(500) - p(689) - p(844)
p(847) = p(49)*p(69) - p(682)
p(848) = p(1)*p(502) - p(691) - p(846)
p(849) = p(11)*p(194) - p(680)
p(850) = p(1)*p(504) - p(695) - p(694)
p(851) = p(1)*p(505) - p(700)
p(852) = p(3)*p(333) - p(702)
p(853) = p(39)*p(93) - p(704)
p(854) = p(18)*p(178) - p(707)
p(855) = p(1)*p(508) - p(705) - p(854)
p(856) = p(1)*p(509) - p(707)
p(857) = p(1)*p(510) - p(708) - p(855)
p(858) = p(21)*p(153) - p(704)
p(859) = p(1)*p(512) - p(710) - p(857)
p(860) = p(34)*p(101) - p(702)
p(861) = p(6)*p(331) - p(701)
p(862) = p(1)*p(515) - p(717) - p(715)
p(863) = p(2)*p(505) - p(701)
p(864) = p(1)*p(517) - p(730) - p(729)
p(865) = p(1)*p(518) - p(733)
p(866) = p(3)*p(335) - p(737)
p(867) = p(39)*p(95) - p(739)
p(868) = p(1)*p(520) - p(738) - p(867)
p(869) = p(2)*p(509) - p(721)
p(870) = p(1)*p(521) - p(740) - p(868)
p(871) = p(7)*p(299) - p(739)
p(872) = p(1)*p(523) - p(742) - p(870)
p(873) = p(49)*p(84) - p(737)
p(874) = p(6)*p(333) - p(726)
p(875) = p(1)*p(526) - p(748) - p(746)
p(876) = p(8)*p(285) - p(726)
p(877) = p(45)*p(91) - p(735)
p(878) = p(1)*p(529) - p(761) - p(759)
p(879) = p(2)*p(518) - p(735)
p(880) = p(1)*p(531) - p(772) - p(771)
p(881) = p(1)*p(532) - p(773)
p(882) = p(14)*p(161)
p(883) = p(3)*p(337) - p(778)
p(884) = p(39)*p(98) - p(780)
p(885) = p(1)*p(535) - p(779) - p(884)
p(886) = p(1)*p(536) - p(780)
p(887) = p(1)*p(537) - p(781) - p(885)
p(888) = p(21)*p(173) - p(778)
p(889) = p(6)*p(335) - p(755)
p(890) = p(1)*p(540) - p(786) - p(784)
p(891) = p(22)*p(150) - p(755)
p(892) = p(45)*p(93) - p(736)
p(893) = p(1)*p(543) - p(796) - p(794)
p(894) = p(8)*p(296) - p(736)
p(895) = p(16)*p(187) - p(775)
p(896) = p(1)*p(547) - p(806) - p(804)
p(897) = p(2)*p(532) - p(775)
p(898) = p(14)*p(171)
p(899) = p(6)*p(309) - p(808) - p(807)
p(900) = p(1)*p(578) - p(899)
p(901) = p(14)*p(187)
p(902) = p(3)*p(340) - p(810)
p(903) = p(2)*p(535) - p(787) - p(783)
p(904) = p(2)*p(536) - p(788)
p(905) = p(1)*p(551) - p(811) - p(903)
p(906) = p(7)*p(323) - p(810)
p(907) = p(6)*p(337) - p(791)
p(908) = p(1)*p(554) - p(815) - p(813)
p(909) = p(50)*p(73) - p(791)
p(910) = p(45)*p(95) - p(769)
p(911) = p(1)*p(557) - p(822) - p(820)
p(912) = p(22)*p(160) - p(769)
p(913) = p(14)*p(177)
p(914) = p(18)*p(187) - p(777)
p(915) = p(2)*p(547) - p(808) - p(899)
p(916) = p(8)*p(310) - p(777)
p(917) = p(14)*p(190)
p(918) = p(3)*p(343) - p(826)
p(919) = p(2)*p(551) - p(816) - p(813)
p(920) = p(1)*p(561) - p(826)
p(921) = p(6)*p(340) - p(818)
p(922) = p(1)*p(563) - p(829) - p(827)
p(923) = p(32)*p(102) - p(818)
p(924) = p(45)*p(98) - p(802)
p(925) = p(2)*p(557) - p(824) - p(915)
p(926) = p(50)*p(81) - p(802)
p(927) = p(14)*p(193)
p(928) = p(2)*p(560) - p(828) - p(827)
p(929) = p(2)*p(561) - p(829)
p(930) = p(6)*p(343) - p(830)
p(931) = p(2)*p(563) - p(831) - p(925)
p(932) = p(13)*p(195) - p(830)
p(933) = p(14)*p(195)
p(934) = p(1)*p(566) - p(834)
p(935) = p(1)*p(567) - p(836) - p(934)
p(936) = p(39)*p(101) - p(835)
p(937) = p(3)*p(346) - p(838)
p(938) = p(1)*p(570) - p(839)
p(939) = p(2)*p(588) - p(938)
p(940) = p(1)*p(572) - p(852)
p(941) = p(8)*p(346) - p(940)
p(942) = p(1)*p(574) - p(866)
p(943) = p(22)*p(194) - p(942)
p(944) = p(1)*p(576) - p(883)
p(945) = p(50)*p(101) - p(944)
p(946) = p(1)*p(579) - p(902)
p(947) = p(49)*p(102) - p(946)
p(948) = p(2)*p(578) - p(901)
p(949) = p(1)*p(582) - p(918)
p(950) = p(21)*p(195) - p(949)
p(951) = p(2)*p(581) - p(917) - p(948)
p(952) = p(1)*p(585) - p(928)
p(953) = p(7)*p(347) - p(952)
p(954) = p(45)*p(102) - p(913)
p(955) = p(2)*p(585) - p(930)
p(956) = p(1)*p(589) - p(955)
p(957) = p(6)*p(347) - p(933)
p(958) = p(1)*p(588) - p(937)
p(959) = p(2)*p(589) - p(957)
p(960) = p(9)*p(196)
p(961) = p(9)*p(202)
p(962) = p(9)*p(203)
p(963) = p(9)*p(228)
p(964) = p(9)*p(242)
p(965) = p(9)*p(207)
p(966) = p(9)*p(197)
p(967) = p(9)*p(208)
p(968) = p(9)*p(209)
p(969) = p(9)*p(243)
p(970) = p(9)*p(211)
p(971) = p(9)*p(247)
p(972) = p(9)*p(212)
p(973) = p(9)*p(248)
p(974) = p(9)*p(249)
p(975) = p(9)*p(258)
p(976) = p(9)*p(214)
p(977) = p(9)*p(215)
p(978) = p(9)*p(259)
p(979) = p(9)*p(263)
p(980) = p(9)*p(264)
p(981) = p(9)*p(265)
p(982) = p(9)*p(308)
p(983) = p(9)*p(311)
p(984) = p(9)*p(270)
p(985) = p(9)*p(271)
p(986) = p(9)*p(321)
p(987) = p(9)*p(327)
p(988) = p(9)*p(222)
p(989) = p(9)*p(223)
p(990) = p(9)*p(284)
p(991) = p(9)*p(198)
p(992) = p(9)*p(199)
p(993) = p(9)*p(224)
p(994) = p(9)*p(200)
p(995) = p(9)*p(225)
p(996) = p(9)*p(201)
p(997) = p(9)*p(226)
p(998) = p(9)*p(227)
p(999) = p(9)*p(285)
p(1000) = p(14)*p(328)
p(1001) = p(14)*p(329)
p(1002) = p(14)*p(330)
p(1003) = p(9)*p(292)
p(1004) = p(9)*p(233)
p(1005) = p(9)*p(234)
p(1006) = p(9)*p(235)
p(1007) = p(9)*p(293)
p(1008) = p(9)*p(236)
p(1009) = p(9)*p(204)
p(1010) = p(9)*p(205)
p(1011) = p(9)*p(237)
p(1012) = p(9)*p(238)
p(1013) = p(9)*p(206)
p(1014) = p(9)*p(239)
p(1015) = p(9)*p(240)
p(1016) = p(9)*p(241)
p(1017) = p(9)*p(294)
p(1018) = p(14)*p(276)
p(1019) = p(14)*p(277)
p(1020) = p(14)*p(278)
p(1021) = p(14)*p(279)
p(1022) = p(14)*p(280)
p(1023) = p(14)*p(281)
p(1024) = p(14)*p(282)
p(1025) = p(14)*p(283)
p(1026) = p(9)*p(244)
p(1027) = p(9)*p(295)
p(1028) = p(9)*p(210)
p(1029) = p(9)*p(245)
p(1030) = p(9)*p(246)
p(1031) = p(9)*p(296)
p(1032) = p(14)*p(222)
p(1033) = p(14)*p(223)
p(1034) = p(14)*p(224)
p(1035) = p(14)*p(225)
p(1036) = p(14)*p(226)
p(1037) = p(14)*p(227)
p(1038) = p(14)*p(202)
p(1039) = p(14)*p(203)
p(1040) = p(9)*p(302)
p(1041) = p(9)*p(252)
p(1042) = p(9)*p(253)
p(1043) = p(9)*p(303)
p(1044) = p(9)*p(254)
p(1045) = p(9)*p(213)
p(1046) = p(9)*p(255)
p(1047) = p(9)*p(256)
p(1048) = p(9)*p(257)
p(1049) = p(9)*p(304)
p(1050) = p(14)*p(286)
p(1051) = p(14)*p(287)
p(1052) = p(14)*p(288)
p(1053) = p(14)*p(289)
p(1054) = p(14)*p(290)
p(1055) = p(14)*p(291)
p(1056) = p(9)*p(305)
p(1057) = p(9)*p(260)
p(1058) = p(9)*p(306)
p(1059) = p(9)*p(261)
p(1060) = p(9)*p(262)
p(1061) = p(9)*p(307)
p(1062) = p(14)*p(233)
p(1063) = p(14)*p(234)
p(1064) = p(14)*p(235)
p(1065) = p(14)*p(236)
p(1066) = p(14)*p(237)
p(1067) = p(14)*p(238)
p(1068) = p(14)*p(239)
p(1069) = p(14)*p(240)
p(1070) = p(14)*p(241)
p(1071) = p(14)*p(207)
p(1072) = p(14)*p(208)
p(1073) = p(14)*p(209)
p(1074) = p(9)*p(309)
p(1075) = p(9)*p(310)
p(1076) = p(14)*p(244)
p(1077) = p(14)*p(245)
p(1078) = p(14)*p(246)
p(1079) = p(14)*p(211)
p(1080) = p(14)*p(212)
p(1081) = p(9)*p(315)
p(1082) = p(9)*p(267)
p(1083) = p(9)*p(316)
p(1084) = p(9)*p(268)
p(1085) = p(9)*p(269)
p(1086) = p(9)*p(317)
p(1087) = p(14)*p(297)
p(1088) = p(14)*p(298)
p(1089) = p(14)*p(299)
p(1090) = p(14)*p(300)
p(1091) = p(14)*p(301)
p(1092) = p(9)*p(318)
p(1093) = p(9)*p(319)
p(1094) = p(9)*p(320)
p(1095) = p(14)*p(252)
p(1096) = p(14)*p(253)
p(1097) = p(14)*p(254)
p(1098) = p(14)*p(255)
p(1099) = p(14)*p(256)
p(1100) = p(14)*p(257)
p(1101) = p(14)*p(214)
p(1102) = p(14)*p(215)
p(1103) = p(9)*p(339)
p(1104) = p(14)*p(260)
p(1105) = p(14)*p(261)
p(1106) = p(14)*p(262)
p(1107) = p(9)*p(324)
p(1108) = p(9)*p(325)
p(1109) = p(9)*p(326)
p(1110) = p(14)*p(312)
p(1111) = p(14)*p(313)
p(1112) = p(14)*p(314)
p(1113) = p(9)*p(342)
p(1114) = p(14)*p(267)
p(1115) = p(14)*p(268)
p(1116) = p(14)*p(269)
p(1117) = p(9)*p(345)
p(1118) = p(14)*p(322)
p(1119) = p(14)*p(323)
p(1120) = p(9)*p(331)
p(1121) = p(9)*p(276)
p(1122) = p(9)*p(216)
p(1123) = p(9)*p(277)
p(1124) = p(9)*p(217)
p(1125) = p(9)*p(218)
p(1126) = p(9)*p(278)
p(1127) = p(9)*p(219)
p(1128) = p(9)*p(220)
p(1129) = p(9)*p(279)
p(1130) = p(9)*p(280)
p(1131) = p(9)*p(221)
p(1132) = p(9)*p(281)
p(1133) = p(9)*p(282)
p(1134) = p(9)*p(283)
p(1135) = p(9)*p(332)
p(1136) = p(3)*p(401) - p(988)
p(1137) = p(7)*p(401) - p(991)
p(1138) = p(49)*p(114) - p(996)
p(1139) = p(24)*p(194) - p(999)
p(1140) = p(3)*p(504) - p(990) - p(1139)
p(1141) = p(1)*p(695) - p(990) - p(1140)
p(1142) = p(1)*p(696) - p(993) - p(1141)
p(1143) = p(6)*p(566) - p(1136)
p(1144) = p(32)*p(178) - p(991)
p(1145) = p(39)*p(150) - p(996)
p(1146) = p(3)*p(505) - p(999)
p(1147) = p(14)*p(346)
p(1148) = p(9)*p(333)
p(1149) = p(9)*p(286)
p(1150) = p(9)*p(229)
p(1151) = p(9)*p(287)
p(1152) = p(9)*p(230)
p(1153) = p(9)*p(231)
p(1154) = p(9)*p(288)
p(1155) = p(9)*p(232)
p(1156) = p(9)*p(289)
p(1157) = p(9)*p(290)
p(1158) = p(9)*p(291)
p(1159) = p(9)*p(334)
p(1160) = p(24)*p(181) - p(963)
p(1161) = p(41)*p(114) - p(960)
p(1162) = p(4)*p(401) - p(961)
p(1163) = p(1)*p(713) - p(1005) - p(1162)
p(1164) = p(1)*p(714) - p(1006) - p(1163)
p(1165) = p(1)*p(715) - p(1007) - p(1164)
p(1166) = p(13)*p(331) - p(1147)
p(1167) = p(2)*p(695) - p(1002) - p(1166)
p(1168) = p(32)*p(181) - p(1147)
p(1169) = p(1)*p(717) - p(1007) - p(1167)
p(1170) = p(73)*p(91) - p(1147)
p(1171) = p(1)*p(719) - p(1011) - p(1169)
p(1172) = p(41)*p(150) - p(1147)
p(1173) = p(13)*p(279) - p(961)
p(1174) = p(16)*p(285) - p(1147)
p(1175) = p(69)*p(73) - p(960)
p(1176) = p(4)*p(505) - p(1147)
p(1177) = p(11)*p(285) - p(963)
p(1178) = p(14)*p(331)
p(1179) = p(14)*p(332)
p(1180) = p(3)*p(430) - p(1026)
p(1181) = p(7)*p(430) - p(1028)
p(1182) = p(49)*p(130) - p(1031)
p(1183) = p(3)*p(517) - p(1027) - p(1182)
p(1184) = p(1)*p(730) - p(1027) - p(1183)
p(1185) = p(45)*p(178) - p(1180)
p(1186) = p(39)*p(160) - p(1028)
p(1187) = p(3)*p(518) - p(1031)
p(1188) = p(14)*p(284)
p(1189) = p(14)*p(285)
p(1190) = p(14)*p(228)
p(1191) = p(9)*p(335)
p(1192) = p(9)*p(297)
p(1193) = p(9)*p(250)
p(1194) = p(9)*p(298)
p(1195) = p(9)*p(251)
p(1196) = p(9)*p(299)
p(1197) = p(9)*p(300)
p(1198) = p(9)*p(301)
p(1199) = p(9)*p(336)
p(1200) = p(24)*p(183) - p(964)
p(1201) = p(43)*p(114) - p(966)
p(1202) = p(8)*p(401) - p(1021)
p(1203) = p(1)*p(745) - p(1042) - p(1202)
p(1204) = p(1)*p(746) - p(1043) - p(1203)
p(1205) = p(13)*p(333) - p(1178)
p(1206) = p(2)*p(717) - p(1025) - p(1183)
p(1207) = p(32)*p(183) - p(1178)
p(1208) = p(1)*p(748) - p(1043) - p(1206)
p(1209) = p(73)*p(93) - p(1178)
p(1210) = p(6)*p(509) - p(1021)
p(1211) = p(43)*p(150) - p(1178)
p(1212) = p(32)*p(153) - p(966)
p(1213) = p(18)*p(285) - p(1178)
p(1214) = p(34)*p(150) - p(964)
p(1215) = p(14)*p(333)
p(1216) = p(14)*p(334)
p(1217) = p(41)*p(130) - p(973)
p(1218) = p(4)*p(430) - p(970)
p(1219) = p(1)*p(758) - p(1057) - p(1218)
p(1220) = p(1)*p(759) - p(1058) - p(1219)
p(1221) = p(81)*p(91) - p(1189)
p(1222) = p(2)*p(730) - p(1034) - p(1221)
p(1223) = p(41)*p(160) - p(1189)
p(1224) = p(1)*p(761) - p(1058) - p(1222)
p(1225) = p(16)*p(296) - p(1189)
p(1226) = p(69)*p(81) - p(970)
p(1227) = p(4)*p(518) - p(1189)
p(1228) = p(11)*p(296) - p(973)
p(1229) = p(14)*p(292)
p(1230) = p(14)*p(293)
p(1231) = p(14)*p(294)
p(1232) = p(14)*p(242)
p(1233) = p(14)*p(243)
p(1234) = p(3)*p(464) - p(1074)
p(1235) = p(7)*p(464) - p(1075)
p(1236) = p(3)*p(531) - p(1074) - p(1235)
p(1237) = p(39)*p(187) - p(1234)
p(1238) = p(3)*p(532) - p(1075)
p(1239) = p(14)*p(295)
p(1240) = p(14)*p(296)
p(1241) = p(14)*p(247)
p(1242) = p(14)*p(248)
p(1243) = p(14)*p(249)
p(1244) = p(9)*p(337)
p(1245) = p(9)*p(312)
p(1246) = p(9)*p(266)
p(1247) = p(9)*p(313)
p(1248) = p(9)*p(314)
p(1249) = p(9)*p(338)
p(1250) = p(24)*p(185) - p(975)
p(1251) = p(46)*p(114) - p(977)
p(1252) = p(1)*p(783) - p(1082) - p(1251)
p(1253) = p(1)*p(784) - p(1083) - p(1252)
p(1254) = p(13)*p(335) - p(1215)
p(1255) = p(2)*p(748) - p(1055) - p(1222)
p(1256) = p(32)*p(185) - p(1215)
p(1257) = p(1)*p(786) - p(1083) - p(1255)
p(1258) = p(73)*p(95) - p(1215)
p(1259) = p(13)*p(299) - p(977)
p(1260) = p(46)*p(150) - p(1215)
p(1261) = p(73)*p(84) - p(975)
p(1262) = p(14)*p(335)
p(1263) = p(14)*p(336)
p(1264) = p(43)*p(130) - p(974)
p(1265) = p(8)*p(430) - p(1038)
p(1266) = p(1)*p(794) - p(1093) - p(1265)
p(1267) = p(81)*p(93) - p(1190)
p(1268) = p(2)*p(761) - p(1066) - p(1236)
p(1269) = p(43)*p(160) - p(1190)
p(1270) = p(45)*p(153) - p(1038)
p(1271) = p(18)*p(296) - p(1190)
p(1272) = p(34)*p(160) - p(974)
p(1273) = p(14)*p(302)
p(1274) = p(14)*p(303)
p(1275) = p(14)*p(304)
p(1276) = p(14)*p(258)
p(1277) = p(14)*p(259)
p(1278) = p(4)*p(464) - p(983)
p(1279) = p(1)*p(804) - p(1103) - p(1278)
p(1280) = p(16)*p(310) - p(1240)
p(1281) = p(2)*p(772) - p(1077) - p(1280)
p(1282) = p(4)*p(532) - p(1240)
p(1283) = p(11)*p(310) - p(983)
p(1284) = p(14)*p(305)
p(1285) = p(14)*p(306)
p(1286) = p(14)*p(307)
p(1287) = p(14)*p(263)
p(1288) = p(14)*p(264)
p(1289) = p(14)*p(265)
p(1290) = p(6)*p(464) - p(1104)
p(1291) = p(3)*p(578) - p(1290)
p(1292) = p(14)*p(309)
p(1293) = p(14)*p(310)
p(1294) = p(9)*p(340)
p(1295) = p(9)*p(322)
p(1296) = p(9)*p(323)
p(1297) = p(9)*p(341)
p(1298) = p(24)*p(188) - p(984)
p(1299) = p(50)*p(114) - p(1089)
p(1300) = p(1)*p(813) - p(1108) - p(1299)
p(1301) = p(13)*p(337) - p(1262)
p(1302) = p(2)*p(786) - p(1091) - p(1268)
p(1303) = p(32)*p(188) - p(1262)
p(1304) = p(6)*p(536) - p(1089)
p(1305) = p(73)*p(98) - p(1262)
p(1306) = p(32)*p(173) - p(984)
p(1307) = p(14)*p(337)
p(1308) = p(14)*p(338)
p(1309) = p(46)*p(130) - p(982)
p(1310) = p(1)*p(820) - p(1113) - p(1309)
p(1311) = p(81)*p(95) - p(1232)
p(1312) = p(2)*p(796) - p(1098) - p(1281)
p(1313) = p(46)*p(160) - p(1232)
p(1314) = p(81)*p(84) - p(982)
p(1315) = p(14)*p(315)
p(1316) = p(14)*p(316)
p(1317) = p(14)*p(317)
p(1318) = p(14)*p(270)
p(1319) = p(14)*p(271)
p(1320) = p(8)*p(464) - p(1079)
p(1321) = p(18)*p(310) - p(1242)
p(1322) = p(34)*p(187) - p(1079)
p(1323) = p(14)*p(318)
p(1324) = p(14)*p(319)
p(1325) = p(14)*p(320)
p(1326) = p(9)*p(343)
p(1327) = p(9)*p(344)
p(1328) = p(24)*p(191) - p(987)
p(1329) = p(1)*p(827) - p(1117) - p(1328)
p(1330) = p(13)*p(340) - p(1307)
p(1331) = p(2)*p(815) - p(1112) - p(1312)
p(1332) = p(32)*p(191) - p(1307)
p(1333) = p(13)*p(323) - p(987)
p(1334) = p(14)*p(340)
p(1335) = p(14)*p(341)
p(1336) = p(50)*p(130) - p(1102)
p(1337) = p(81)*p(98) - p(1276)
p(1338) = p(45)*p(173) - p(1102)
p(1339) = p(14)*p(324)
p(1340) = p(14)*p(325)
p(1341) = p(14)*p(326)
p(1342) = p(9)*p(347)
p(1343) = p(24)*p(195) - p(1119)
p(1344) = p(13)*p(343) - p(1334)
p(1345) = p(6)*p(561) - p(1119)
p(1346) = p(14)*p(343)
p(1347) = p(14)*p(344)
p(1348) = p(9)*p(272)
p(1349) = p(9)*p(273)
p(1350) = p(9)*p(274)
p(1351) = p(9)*p(328)
p(1352) = p(9)*p(275)
p(1353) = p(9)*p(329)
p(1354) = p(9)*p(330)
p(1355) = p(9)*p(346)
p(1356) = p(3)*p(570) - p(1120)
p(1357) = p(39)*p(181) - p(1122)
p(1358) = p(41)*p(178) - p(1125)
p(1359) = p(4)*p(566) - p(1129)
p(1360) = p(1)*p(842) - p(1126) - p(1359)
p(1361) = p(1)*p(843) - p(1129)
p(1362) = p(1)*p(844) - p(1130) - p(1360)
p(1363) = p(21)*p(279) - p(1125)
p(1364) = p(1)*p(846) - p(1133) - p(1362)
p(1365) = p(69)*p(101) - p(1122)
p(1366) = p(1)*p(848) - p(1135) - p(1364)
p(1367) = p(11)*p(346) - p(1120)
p(1368) = p(1)*p(850) - p(1140) - p(1139)
p(1369) = p(1)*p(851) - p(1146)
p(1370) = p(3)*p(572) - p(1148)
p(1371) = p(39)*p(183) - p(1150)
p(1372) = p(43)*p(178) - p(1153)
p(1373) = p(1)*p(854) - p(1151) - p(1372)
p(1374) = p(2)*p(843) - p(1143)
p(1375) = p(1)*p(855) - p(1154) - p(1373)
p(1376) = p(7)*p(509) - p(1153)
p(1377) = p(1)*p(857) - p(1157) - p(1375)
p(1378) = p(49)*p(153) - p(1150)
p(1379) = p(1)*p(859) - p(1159) - p(1377)
p(1380) = p(34)*p(194) - p(1148)
p(1381) = p(6)*p(570) - p(1147)
p(1382) = p(1)*p(862) - p(1167) - p(1165)
p(1383) = p(2)*p(851) - p(1147)
p(1384) = p(1)*p(864) - p(1183) - p(1182)
p(1385) = p(1)*p(865) - p(1187)
p(1386) = p(3)*p(574) - p(1191)
p(1387) = p(39)*p(185) - p(1193)
p(1388) = p(46)*p(178) - p(1196)
p(1389) = p(1)*p(868) - p(1194) - p(1388)
p(1390) = p(1)*p(869) - p(1196)
p(1391) = p(1)*p(870) - p(1197) - p(1389)
p(1392) = p(21)*p(299) - p(1193)
p(1393) = p(1)*p(872) - p(1199) - p(1391)
p(1394) = p(84)*p(101) - p(1191)
p(1395) = p(6)*p(572) - p(1178)
p(1396) = p(1)*p(875) - p(1206) - p(1204)
p(1397) = p(8)*p(505) - p(1178)
p(1398) = p(45)*p(181) - p(1189)
p(1399) = p(1)*p(878) - p(1222) - p(1220)
p(1400) = p(2)*p(865) - p(1189)
p(1401) = p(1)*p(880) - p(1236) - p(1235)
p(1402) = p(1)*p(881) - p(1238)
p(1403) = p(3)*p(576) - p(1244)
p(1404) = p(39)*p(188) - p(1246)
p(1405) = p(1)*p(884) - p(1245) - p(1404)
p(1406) = p(2)*p(869) - p(1210)
p(1407) = p(1)*p(885) - p(1247) - p(1405)
p(1408) = p(7)*p(536) - p(1246)
p(1409) = p(1)*p(887) - p(1249) - p(1407)
p(1410) = p(49)*p(173) - p(1244)
p(1411) = p(6)*p(574) - p(1215)
p(1412) = p(1)*p(890) - p(1255) - p(1253)
p(1413) = p(22)*p(285) - p(1215)
p(1414) = p(45)*p(183) - p(1190)
p(1415) = p(1)*p(893) - p(1268) - p(1266)
p(1416) = p(8)*p(518) - p(1190)
p(1417) = p(41)*p(187) - p(1240)
p(1418) = p(1)*p(896) - p(1281) - p(1279)
p(1419) = p(2)*p(881) - p(1240)
p(1420) = p(14)*p(308)
p(1421) = p(4)*p(547) - p(1106) - p(1320)
p(1422) = p(1)*p(900) - p(1291)
p(1423) = p(14)*p(311)
p(1424) = p(3)*p(579) - p(1294)
p(1425) = p(39)*p(191) - p(1296)
p(1426) = p(1)*p(903) - p(1295) - p(1425)
p(1427) = p(1)*p(904) - p(1296)
p(1428) = p(1)*p(905) - p(1297) - p(1426)
p(1429) = p(21)*p(323) - p(1294)
p(1430) = p(6)*p(576) - p(1262)
p(1431) = p(1)*p(908) - p(1302) - p(1300)
p(1432) = p(50)*p(150) - p(1262)
p(1433) = p(45)*p(185) - p(1232)
p(1434) = p(1)*p(911) - p(1312) - p(1310)
p(1435) = p(22)*p(296) - p(1232)
p(1436) = p(43)*p(187) - p(1242)
p(1437) = p(1)*p(915) - p(1322) - p(1320)
p(1438) = p(8)*p(532) - p(1242)
p(1439) = p(14)*p(321)
p(1440) = p(4)*p(578) - p(1293)
p(1441) = p(2)*p(899) - p(1292) - p(1440)
p(1442) = p(2)*p(900) - p(1293)
p(1443) = p(14)*p(339)
p(1444) = p(3)*p(582) - p(1326)
p(1445) = p(2)*p(903) - p(1303) - p(1299)
p(1446) = p(2)*p(904) - p(1304)
p(1447) = p(1)*p(919) - p(1327) - p(1445)
p(1448) = p(7)*p(561) - p(1326)
p(1449) = p(6)*p(579) - p(1307)
p(1450) = p(1)*p(922) - p(1331) - p(1329)
p(1451) = p(73)*p(102) - p(1307)
p(1452) = p(45)*p(188) - p(1276)
p(1453) = p(1)*p(925) - p(1338) - p(1336)
p(1454) = p(50)*p(160) - p(1276)
p(1455) = p(14)*p(327)
p(1456) = p(46)*p(187) - p(1243)
p(1457) = p(2)*p(915) - p(1324) - p(1441)
p(1458) = p(22)*p(310) - p(1243)
p(1459) = p(14)*p(342)
p(1460) = p(3)*p(585) - p(1342)
p(1461) = p(2)*p(919) - p(1332) - p(1329)
p(1462) = p(1)*p(929) - p(1342)
p(1463) = p(6)*p(582) - p(1334)
p(1464) = p(1)*p(931) - p(1345) - p(1343)
p(1465) = p(32)*p(195) - p(1334)
p(1466) = p(45)*p(191) - p(1318)
p(1467) = p(2)*p(925) - p(1340) - p(1457)
p(1468) = p(81)*p(102) - p(1318)
p(1469) = p(14)*p(345)
p(1470) = p(2)*p(928) - p(1344) - p(1343)
p(1471) = p(2)*p(929) - p(1345)
p(1472) = p(6)*p(585) - p(1346)
p(1473) = p(2)*p(931) - p(1347) - p(1467)
p(1474) = p(13)*p(347) - p(1346)
p(1475) = p(14)*p(347)
p(1476) = p(3)*p(566) - p(1351)
p(1477) = p(7)*p(566) - p(1349)
p(1478) = p(49)*p(178) - p(1348)
p(1479) = p(39)*p(194) - p(1352)
p(1480) = p(3)*p(588) - p(1355)
p(1481) = p(1)*p(938) - p(1356)
p(1482) = p(2)*p(958) - p(1481)
p(1483) = p(1)*p(940) - p(1370)
p(1484) = p(8)*p(588) - p(1483)
p(1485) = p(1)*p(942) - p(1386)
p(1486) = p(22)*p(346) - p(1485)
p(1487) = p(1)*p(944) - p(1403)
p(1488) = p(50)*p(194) - p(1487)
p(1489) = p(1)*p(946) - p(1424)
p(1490) = p(101)*p(102) - p(1489)
p(1491) = p(6)*p(578) - p(1443)
p(1492) = p(1)*p(949) - p(1444)
p(1493) = p(49)*p(195) - p(1492)
p(1494) = p(8)*p(578) - p(1423)
p(1495) = p(1)*p(952) - p(1460)
p(1496) = p(21)*p(347) - p(1495)
p(1497) = p(50)*p(187) - p(1420)
p(1498) = p(1)*p(955) - p(1470)
p(1499) = p(7)*p(589) - p(1498)
p(1500) = p(45)*p(195) - p(1455)
p(1501) = p(2)*p(955) - p(1472)
p(1502) = p(1)*p(959) - p(1501)
p(1503) = p(6)*p(589) - p(1475)
p(1504) = p(1)*p(958) - p(1480)
p(1505) = p(2)*p(959) - p(1503)
p(1506) = p(9)*p(348)
p(1507) = p(9)*p(349)
p(1508) = p(9)*p(360)
p(1509) = p(9)*p(361)
p(1510) = p(9)*p(409)
p(1511) = p(9)*p(428)
p(1512) = p(9)*p(368)
p(1513) = p(9)*p(350)
p(1514) = p(9)*p(369)
p(1515) = p(9)*p(351)
p(1516) = p(9)*p(370)
p(1517) = p(9)*p(371)
p(1518) = p(9)*p(372)
p(1519) = p(9)*p(429)
p(1520) = p(9)*p(375)
p(1521) = p(9)*p(435)
p(1522) = p(9)*p(352)
p(1523) = p(9)*p(376)
p(1524) = p(9)*p(377)
p(1525) = p(9)*p(436)
p(1526) = p(9)*p(378)
p(1527) = p(9)*p(437)
p(1528) = p(9)*p(451)
p(1529) = p(9)*p(382)
p(1530) = p(9)*p(353)
p(1531) = p(9)*p(383)
p(1532) = p(9)*p(384)
p(1533) = p(9)*p(452)
p(1534) = p(9)*p(459)
p(1535) = p(9)*p(386)
p(1536) = p(9)*p(460)
p(1537) = p(9)*p(387)
p(1538) = p(9)*p(388)
p(1539) = p(9)*p(461)
p(1540) = p(9)*p(462)
p(1541) = p(9)*p(463)
p(1542) = p(9)*p(466)
p(1543) = p(9)*p(467)
p(1544) = p(9)*p(533)
p(1545) = p(9)*p(476)
p(1546) = p(9)*p(390)
p(1547) = p(9)*p(391)
p(1548) = p(9)*p(477)
p(1549) = p(9)*p(481)
p(1550) = p(9)*p(482)
p(1551) = p(9)*p(483)
p(1552) = p(9)*p(545)
p(1553) = p(9)*p(549)
p(1554) = p(9)*p(488)
p(1555) = p(9)*p(489)
p(1556) = p(9)*p(559)
p(1557) = p(9)*p(565)
p(1558) = p(9)*p(401)
p(1559) = p(9)*p(402)
p(1560) = p(9)*p(403)
p(1561) = p(9)*p(504)
p(1562) = p(9)*p(354)
p(1563) = p(9)*p(355)
p(1564) = p(9)*p(404)
p(1565) = p(9)*p(356)
p(1566) = p(9)*p(357)
p(1567) = p(9)*p(405)
p(1568) = p(9)*p(358)
p(1569) = p(9)*p(406)
p(1570) = p(9)*p(359)
p(1571) = p(9)*p(407)
p(1572) = p(9)*p(408)
p(1573) = p(9)*p(505)
p(1574) = p(14)*p(566)
p(1575) = p(14)*p(567)
p(1576) = p(14)*p(568)
p(1577) = p(14)*p(569)
p(1578) = p(9)*p(514)
p(1579) = p(9)*p(416)
p(1580) = p(9)*p(417)
p(1581) = p(9)*p(418)
p(1582) = p(9)*p(419)
p(1583) = p(9)*p(515)
p(1584) = p(9)*p(420)
p(1585) = p(9)*p(362)
p(1586) = p(9)*p(363)
p(1587) = p(9)*p(364)
p(1588) = p(9)*p(421)
p(1589) = p(9)*p(422)
p(1590) = p(9)*p(365)
p(1591) = p(9)*p(366)
p(1592) = p(9)*p(423)
p(1593) = p(9)*p(424)
p(1594) = p(9)*p(367)
p(1595) = p(9)*p(425)
p(1596) = p(9)*p(426)
p(1597) = p(9)*p(427)
p(1598) = p(9)*p(516)
p(1599) = p(14)*p(495)
p(1600) = p(14)*p(496)
p(1601) = p(14)*p(497)
p(1602) = p(14)*p(498)
p(1603) = p(14)*p(499)
p(1604) = p(14)*p(500)
p(1605) = p(14)*p(501)
p(1606) = p(14)*p(502)
p(1607) = p(14)*p(503)
p(1608) = p(9)*p(430)
p(1609) = p(9)*p(431)
p(1610) = p(9)*p(517)
p(1611) = p(9)*p(373)
p(1612) = p(9)*p(432)
p(1613) = p(9)*p(374)
p(1614) = p(9)*p(433)
p(1615) = p(9)*p(434)
p(1616) = p(9)*p(518)
p(1617) = p(14)*p(401)
p(1618) = p(14)*p(402)
p(1619) = p(14)*p(403)
p(1620) = p(14)*p(404)
p(1621) = p(14)*p(405)
p(1622) = p(14)*p(406)
p(1623) = p(14)*p(407)
p(1624) = p(14)*p(408)
p(1625) = p(14)*p(360)
p(1626) = p(14)*p(361)
p(1627) = p(9)*p(525)
p(1628) = p(9)*p(442)
p(1629) = p(9)*p(443)
p(1630) = p(9)*p(444)
p(1631) = p(9)*p(526)
p(1632) = p(9)*p(445)
p(1633) = p(9)*p(379)
p(1634) = p(9)*p(380)
p(1635) = p(9)*p(446)
p(1636) = p(9)*p(447)
p(1637) = p(9)*p(381)
p(1638) = p(9)*p(448)
p(1639) = p(9)*p(449)
p(1640) = p(9)*p(450)
p(1641) = p(9)*p(527)
p(1642) = p(14)*p(506)
p(1643) = p(14)*p(507)
p(1644) = p(14)*p(508)
p(1645) = p(14)*p(509)
p(1646) = p(14)*p(510)
p(1647) = p(14)*p(511)
p(1648) = p(14)*p(512)
p(1649) = p(14)*p(513)
p(1650) = p(9)*p(528)
p(1651) = p(9)*p(453)
p(1652) = p(9)*p(454)
p(1653) = p(9)*p(529)
p(1654) = p(9)*p(455)
p(1655) = p(9)*p(385)
p(1656) = p(9)*p(456)
p(1657) = p(9)*p(457)
p(1658) = p(9)*p(458)
p(1659) = p(9)*p(530)
p(1660) = p(14)*p(416)
p(1661) = p(14)*p(417)
p(1662) = p(14)*p(418)
p(1663) = p(14)*p(419)
p(1664) = p(14)*p(420)
p(1665) = p(14)*p(421)
p(1666) = p(14)*p(422)
p(1667) = p(14)*p(423)
p(1668) = p(14)*p(424)
p(1669) = p(14)*p(425)
p(1670) = p(14)*p(426)
p(1671) = p(14)*p(427)
p(1672) = p(14)*p(368)
p(1673) = p(14)*p(369)
p(1674) = p(14)*p(370)
p(1675) = p(14)*p(371)
p(1676) = p(14)*p(372)
p(1677) = p(9)*p(464)
p(1678) = p(9)*p(531)
p(1679) = p(9)*p(465)
p(1680) = p(9)*p(532)
p(1681) = p(14)*p(430)
p(1682) = p(14)*p(431)
p(1683) = p(14)*p(432)
p(1684) = p(14)*p(433)
p(1685) = p(14)*p(434)
p(1686) = p(14)*p(375)
p(1687) = p(14)*p(376)
p(1688) = p(14)*p(377)
p(1689) = p(14)*p(378)
p(1690) = p(9)*p(539)
p(1691) = p(9)*p(470)
p(1692) = p(9)*p(471)
p(1693) = p(9)*p(540)
p(1694) = p(9)*p(472)
p(1695) = p(9)*p(389)
p(1696) = p(9)*p(473)
p(1697) = p(9)*p(474)
p(1698) = p(9)*p(475)
p(1699) = p(9)*p(541)
p(1700) = p(14)*p(519)
p(1701) = p(14)*p(520)
p(1702) = p(14)*p(521)
p(1703) = p(14)*p(522)
p(1704) = p(14)*p(523)
p(1705) = p(14)*p(524)
p(1706) = p(9)*p(542)
p(1707) = p(9)*p(478)
p(1708) = p(9)*p(543)
p(1709) = p(9)*p(479)
p(1710) = p(9)*p(480)
p(1711) = p(9)*p(544)
p(1712) = p(14)*p(442)
p(1713) = p(14)*p(443)
p(1714) = p(14)*p(444)
p(1715) = p(14)*p(445)
p(1716) = p(14)*p(446)
p(1717) = p(14)*p(447)
p(1718) = p(14)*p(448)
p(1719) = p(14)*p(449)
p(1720) = p(14)*p(450)
p(1721) = p(14)*p(382)
p(1722) = p(14)*p(383)
p(1723) = p(14)*p(384)
p(1724) = p(9)*p(546)
p(1725) = p(9)*p(547)
p(1726) = p(9)*p(548)
p(1727) = p(14)*p(453)
p(1728) = p(14)*p(454)
p(1729) = p(14)*p(455)
p(1730) = p(14)*p(456)
p(1731) = p(14)*p(457)
p(1732) = p(14)*p(458)
p(1733) = p(14)*p(386)
p(1734) = p(14)*p(387)
p(1735) = p(14)*p(388)
p(1736) = p(9)*p(578)
p(1737) = p(14)*p(464)
p(1738) = p(14)*p(465)
p(1739) = p(9)*p(553)
p(1740) = p(9)*p(485)
p(1741) = p(9)*p(554)
p(1742) = p(9)*p(486)
p(1743) = p(9)*p(487)
p(1744) = p(9)*p(555)
p(1745) = p(14)*p(534)
p(1746) = p(14)*p(535)
p(1747) = p(14)*p(536)
p(1748) = p(14)*p(537)
p(1749) = p(14)*p(538)
p(1750) = p(9)*p(556)
p(1751) = p(9)*p(557)
p(1752) = p(9)*p(558)
p(1753) = p(14)*p(470)
p(1754) = p(14)*p(471)
p(1755) = p(14)*p(472)
p(1756) = p(14)*p(473)
p(1757) = p(14)*p(474)
p(1758) = p(14)*p(475)
p(1759) = p(14)*p(390)
p(1760) = p(14)*p(391)
p(1761) = p(9)*p(581)
p(1762) = p(14)*p(478)
p(1763) = p(14)*p(479)
p(1764) = p(14)*p(480)
p(1765) = p(9)*p(562)
p(1766) = p(9)*p(563)
p(1767) = p(9)*p(564)
p(1768) = p(14)*p(550)
p(1769) = p(14)*p(551)
p(1770) = p(14)*p(552)
p(1771) = p(9)*p(584)
p(1772) = p(14)*p(485)
p(1773) = p(14)*p(486)
p(1774) = p(14)*p(487)
p(1775) = p(9)*p(587)
p(1776) = p(14)*p(560)
p(1777) = p(14)*p(561)
p(1778) = p(9)*p(570)
p(1779) = p(9)*p(495)
p(1780) = p(9)*p(392)
p(1781) = p(9)*p(496)
p(1782) = p(9)*p(393)
p(1783) = p(9)*p(394)
p(1784) = p(9)*p(497)
p(1785) = p(9)*p(395)
p(1786) = p(9)*p(396)
p(1787) = p(9)*p(397)
p(1788) = p(9)*p(498)
p(1789) = p(9)*p(398)
p(1790) = p(9)*p(399)
p(1791) = p(9)*p(499)
p(1792) = p(9)*p(500)
p(1793) = p(9)*p(400)
p(1794) = p(9)*p(501)
p(1795) = p(9)*p(502)
p(1796) = p(9)*p(503)
p(1797) = p(9)*p(571)
p(1798) = p(1)*p(1136) - p(1558)
p(1799) = p(21)*p(401) - p(1565)
p(1800) = p(101)*p(114) - p(1570)
p(1801) = p(24)*p(346) - p(1573)
p(1802) = p(3)*p(850) - p(1561) - p(1801)
p(1803) = p(1)*p(1140) - p(1561) - p(1802)
p(1804) = p(1)*p(1141) - p(1564) - p(1803)
p(1805) = p(1)*p(1142) - p(1567) - p(1804)
p(1806) = p(13)*p(566) - p(1558)
p(1807) = p(73)*p(178) - p(1565)
p(1808) = p(39)*p(285) - p(1570)
p(1809) = p(3)*p(851) - p(1573)
p(1810) = p(14)*p(588)
p(1811) = p(9)*p(572)
p(1812) = p(9)*p(506)
p(1813) = p(9)*p(410)
p(1814) = p(9)*p(507)
p(1815) = p(9)*p(411)
p(1816) = p(9)*p(412)
p(1817) = p(9)*p(508)
p(1818) = p(9)*p(413)
p(1819) = p(9)*p(414)
p(1820) = p(9)*p(509)
p(1821) = p(9)*p(510)
p(1822) = p(9)*p(415)
p(1823) = p(9)*p(511)
p(1824) = p(9)*p(512)
p(1825) = p(9)*p(513)
p(1826) = p(9)*p(573)
p(1827) = p(24)*p(331) - p(1510)
p(1828) = p(91)*p(114) - p(1507)
p(1829) = p(16)*p(401) - p(1506)
p(1830) = p(2)*p(1136) - p(1574)
p(1831) = p(1)*p(1163) - p(1581) - p(1830)
p(1832) = p(1)*p(1164) - p(1582) - p(1831)
p(1833) = p(1)*p(1165) - p(1583) - p(1832)
p(1834) = p(13)*p(570) - p(1810)
p(1835) = p(2)*p(1140) - p(1577) - p(1834)
p(1836) = p(32)*p(331) - p(1810)
p(1837) = p(1)*p(1167) - p(1583) - p(1835)
p(1838) = p(73)*p(181) - p(1810)
p(1839) = p(1)*p(1169) - p(1588) - p(1837)
p(1840) = p(91)*p(150) - p(1810)
p(1841) = p(6)*p(843) - p(1574)
p(1842) = p(41)*p(285) - p(1810)
p(1843) = p(32)*p(279) - p(1506)
p(1844) = p(16)*p(505) - p(1810)
p(1845) = p(69)*p(150) - p(1507)
p(1846) = p(4)*p(851) - p(1810)
p(1847) = p(11)*p(505) - p(1510)
p(1848) = p(14)*p(570)
p(1849) = p(14)*p(571)
p(1850) = p(1)*p(1180) - p(1608)
p(1851) = p(21)*p(430) - p(1613)
p(1852) = p(101)*p(130) - p(1616)
p(1853) = p(3)*p(864) - p(1610) - p(1852)
p(1854) = p(1)*p(1183) - p(1610) - p(1853)
p(1855) = p(1)*p(1184) - p(1612) - p(1854)
p(1856) = p(81)*p(178) - p(1608)
p(1857) = p(39)*p(296) - p(1613)
p(1858) = p(3)*p(865) - p(1616)
p(1859) = p(14)*p(504)
p(1860) = p(14)*p(505)
p(1861) = p(14)*p(409)
p(1862) = p(9)*p(574)
p(1863) = p(9)*p(519)
p(1864) = p(9)*p(438)
p(1865) = p(9)*p(520)
p(1866) = p(9)*p(439)
p(1867) = p(9)*p(440)
p(1868) = p(9)*p(521)
p(1869) = p(9)*p(441)
p(1870) = p(9)*p(522)
p(1871) = p(9)*p(523)
p(1872) = p(9)*p(524)
p(1873) = p(9)*p(575)
p(1874) = p(24)*p(333) - p(1511)
p(1875) = p(93)*p(114) - p(1513)
p(1876) = p(18)*p(401) - p(1516)
p(1877) = p(1)*p(1202) - p(1629) - p(1876)
p(1878) = p(1)*p(1203) - p(1630) - p(1877)
p(1879) = p(1)*p(1204) - p(1631) - p(1878)
p(1880) = p(13)*p(572) - p(1848)
p(1881) = p(2)*p(1167) - p(1607) - p(1853)
p(1882) = p(32)*p(333) - p(1848)
p(1883) = p(1)*p(1206) - p(1631) - p(1881)
p(1884) = p(73)*p(183) - p(1848)
p(1885) = p(1)*p(1208) - p(1635) - p(1883)
p(1886) = p(93)*p(150) - p(1848)
p(1887) = p(13)*p(509) - p(1516)
p(1888) = p(43)*p(285) - p(1848)
p(1889) = p(73)*p(153) - p(1513)
p(1890) = p(18)*p(505) - p(1848)
p(1891) = p(34)*p(285) - p(1511)
p(1892) = p(14)*p(572)
p(1893) = p(14)*p(573)
p(1894) = p(91)*p(130) - p(1525)
p(1895) = p(16)*p(430) - p(1522)
p(1896) = p(2)*p(1180) - p(1617)
p(1897) = p(1)*p(1219) - p(1652) - p(1896)
p(1898) = p(1)*p(1220) - p(1653) - p(1897)
p(1899) = p(81)*p(181) - p(1860)
p(1900) = p(2)*p(1183) - p(1620) - p(1899)
p(1901) = p(91)*p(160) - p(1860)
p(1902) = p(1)*p(1222) - p(1653) - p(1900)
p(1903) = p(41)*p(296) - p(1860)
p(1904) = p(45)*p(279) - p(1617)
p(1905) = p(16)*p(518) - p(1860)
p(1906) = p(69)*p(160) - p(1522)
p(1907) = p(4)*p(865) - p(1860)
p(1908) = p(11)*p(518) - p(1525)
p(1909) = p(14)*p(514)
p(1910) = p(14)*p(515)
p(1911) = p(14)*p(516)
p(1912) = p(14)*p(428)
p(1913) = p(14)*p(429)
p(1914) = p(1)*p(1234) - p(1677)
p(1915) = p(21)*p(464) - p(1680)
p(1916) = p(3)*p(880) - p(1678) - p(1915)
p(1917) = p(1)*p(1236) - p(1678) - p(1916)
p(1918) = p(39)*p(310) - p(1677)
p(1919) = p(3)*p(881) - p(1680)
p(1920) = p(14)*p(517)
p(1921) = p(14)*p(518)
p(1922) = p(14)*p(435)
p(1923) = p(14)*p(436)
p(1924) = p(14)*p(437)
p(1925) = p(9)*p(576)
p(1926) = p(9)*p(534)
p(1927) = p(9)*p(468)
p(1928) = p(9)*p(535)
p(1929) = p(9)*p(469)
p(1930) = p(9)*p(536)
p(1931) = p(9)*p(537)
p(1932) = p(9)*p(538)
p(1933) = p(9)*p(577)
p(1934) = p(24)*p(335) - p(1528)
p(1935) = p(95)*p(114) - p(1530)
p(1936) = p(22)*p(401) - p(1645)
p(1937) = p(1)*p(1252) - p(1692) - p(1936)
p(1938) = p(1)*p(1253) - p(1693) - p(1937)
p(1939) = p(13)*p(574) - p(1892)
p(1940) = p(2)*p(1206) - p(1649) - p(1900)
p(1941) = p(32)*p(335) - p(1892)
p(1942) = p(1)*p(1255) - p(1693) - p(1940)
p(1943) = p(73)*p(185) - p(1892)
p(1944) = p(6)*p(869) - p(1645)
p(1945) = p(95)*p(150) - p(1892)
p(1946) = p(32)*p(299) - p(1530)
p(1947) = p(46)*p(285) - p(1892)
p(1948) = p(84)*p(150) - p(1528)
p(1949) = p(14)*p(574)
p(1950) = p(14)*p(575)
p(1951) = p(93)*p(130) - p(1527)
p(1952) = p(18)*p(430) - p(1526)
p(1953) = p(1)*p(1265) - p(1707) - p(1952)
p(1954) = p(1)*p(1266) - p(1708) - p(1953)
p(1955) = p(81)*p(183) - p(1861)
p(1956) = p(2)*p(1222) - p(1665) - p(1916)
p(1957) = p(93)*p(160) - p(1861)
p(1958) = p(1)*p(1268) - p(1708) - p(1956)
p(1959) = p(43)*p(296) - p(1861)
p(1960) = p(81)*p(153) - p(1526)
p(1961) = p(18)*p(518) - p(1861)
p(1962) = p(34)*p(296) - p(1527)
p(1963) = p(14)*p(525)
p(1964) = p(14)*p(526)
p(1965) = p(14)*p(527)
p(1966) = p(14)*p(451)
p(1967) = p(14)*p(452)
p(1968) = p(16)*p(464) - p(1543)
p(1969) = p(2)*p(1234) - p(1681)
p(1970) = p(1)*p(1279) - p(1725) - p(1969)
p(1971) = p(41)*p(310) - p(1921)
p(1972) = p(2)*p(1236) - p(1683) - p(1971)
p(1973) = p(16)*p(532) - p(1921)
p(1974) = p(69)*p(187) - p(1681)
p(1975) = p(4)*p(881) - p(1921)
p(1976) = p(11)*p(532) - p(1543)
p(1977) = p(14)*p(528)
p(1978) = p(14)*p(529)
p(1979) = p(14)*p(530)
p(1980) = p(14)*p(459)
p(1981) = p(14)*p(460)
p(1982) = p(14)*p(461)
p(1983) = p(14)*p(462)
p(1984) = p(14)*p(463)
p(1985) = p(1)*p(1290) - p(1736)
p(1986) = p(4)*p(806) - p(1761) - p(1732)
p(1987) = p(3)*p(900) - p(1736)
p(1988) = p(14)*p(531)
p(1989) = p(14)*p(532)
p(1990) = p(14)*p(466)
p(1991) = p(14)*p(467)
p(1992) = p(9)*p(579)
p(1993) = p(9)*p(550)
p(1994) = p(9)*p(484)
p(1995) = p(9)*p(551)
p(1996) = p(9)*p(552)
p(1997) = p(9)*p(580)
p(1998) = p(24)*p(337) - p(1545)
p(1999) = p(98)*p(114) - p(1547)
p(2000) = p(1)*p(1299) - p(1740) - p(1999)
p(2001) = p(1)*p(1300) - p(1741) - p(2000)
p(2002) = p(13)*p(576) - p(1949)
p(2003) = p(2)*p(1255) - p(1705) - p(1956)
p(2004) = p(32)*p(337) - p(1949)
p(2005) = p(1)*p(1302) - p(1741) - p(2003)
p(2006) = p(73)*p(188) - p(1949)
p(2007) = p(13)*p(536) - p(1547)
p(2008) = p(98)*p(150) - p(1949)
p(2009) = p(73)*p(173) - p(1545)
p(2010) = p(14)*p(576)
p(2011) = p(14)*p(577)
p(2012) = p(95)*p(130) - p(1540)
p(2013) = p(22)*p(430) - p(1674)
p(2014) = p(1)*p(1310) - p(1751) - p(2013)
p(2015) = p(81)*p(185) - p(1912)
p(2016) = p(2)*p(1268) - p(1716) - p(1972)
p(2017) = p(95)*p(160) - p(1912)
p(2018) = p(45)*p(299) - p(1674)
p(2019) = p(46)*p(296) - p(1912)
p(2020) = p(84)*p(160) - p(1540)
p(2021) = p(14)*p(539)
p(2022) = p(14)*p(540)
p(2023) = p(14)*p(541)
p(2024) = p(14)*p(476)
p(2025) = p(14)*p(477)
p(2026) = p(18)*p(464) - p(1544)
p(2027) = p(1)*p(1320) - p(1761) - p(2026)
p(2028) = p(43)*p(310) - p(1923)
p(2029) = p(2)*p(1281) - p(1730) - p(1986)
p(2030) = p(18)*p(532) - p(1923)
p(2031) = p(34)*p(310) - p(1544)
p(2032) = p(14)*p(542)
p(2033) = p(14)*p(543)
p(2034) = p(14)*p(544)
p(2035) = p(14)*p(481)
p(2036) = p(14)*p(482)
p(2037) = p(14)*p(483)
p(2038) = p(2)*p(1290) - p(1737)
p(2039) = p(4)*p(900) - p(1989)
p(2040) = p(11)*p(578) - p(1737)
p(2041) = p(14)*p(546)
p(2042) = p(14)*p(547)
p(2043) = p(14)*p(548)
p(2044) = p(9)*p(582)
p(2045) = p(9)*p(560)
p(2046) = p(9)*p(561)
p(2047) = p(9)*p(583)
p(2048) = p(24)*p(340) - p(1554)
p(2049) = p(102)*p(114) - p(1747)
p(2050) = p(1)*p(1329) - p(1766) - p(2049)
p(2051) = p(13)*p(579) - p(2010)
p(2052) = p(2)*p(1302) - p(1749) - p(2016)
p(2053) = p(32)*p(340) - p(2010)
p(2054) = p(6)*p(904) - p(1747)
p(2055) = p(73)*p(191) - p(2010)
p(2056) = p(32)*p(323) - p(1554)
p(2057) = p(14)*p(579)
p(2058) = p(14)*p(580)
p(2059) = p(98)*p(130) - p(1552)
p(2060) = p(1)*p(1336) - p(1771) - p(2059)
p(2061) = p(81)*p(188) - p(1966)
p(2062) = p(2)*p(1312) - p(1756) - p(2029)
p(2063) = p(98)*p(160) - p(1966)
p(2064) = p(81)*p(173) - p(1552)
p(2065) = p(14)*p(553)
p(2066) = p(14)*p(554)
p(2067) = p(14)*p(555)
p(2068) = p(14)*p(488)
p(2069) = p(14)*p(489)
p(2070) = p(22)*p(464) - p(1689)
p(2071) = p(46)*p(310) - p(1924)
p(2072) = p(84)*p(187) - p(1689)
p(2073) = p(14)*p(556)
p(2074) = p(14)*p(557)
p(2075) = p(14)*p(558)
p(2076) = p(9)*p(585)
p(2077) = p(9)*p(586)
p(2078) = p(24)*p(343) - p(1557)
p(2079) = p(1)*p(1343) - p(1775) - p(2078)
p(2080) = p(13)*p(582) - p(2057)
p(2081) = p(2)*p(1331) - p(1770) - p(2062)
p(2082) = p(32)*p(343) - p(2057)
p(2083) = p(13)*p(561) - p(1557)
p(2084) = p(14)*p(582)
p(2085) = p(14)*p(583)
p(2086) = p(102)*p(130) - p(1760)
p(2087) = p(81)*p(191) - p(2024)
p(2088) = p(45)*p(323) - p(1760)
p(2089) = p(14)*p(562)
p(2090) = p(14)*p(563)
p(2091) = p(14)*p(564)
p(2092) = p(9)*p(589)
p(2093) = p(24)*p(347) - p(1777)
p(2094) = p(13)*p(585) - p(2084)
p(2095) = p(6)*p(929) - p(1777)
p(2096) = p(14)*p(585)
p(2097) = p(14)*p(586)
p(2098) = p(9)*p(490)
p(2099) = p(9)*p(491)
p(2100) = p(9)*p(492)
p(2101) = p(9)*p(566)
p(2102) = p(9)*p(493)
p(2103) = p(9)*p(567)
p(2104) = p(9)*p(494)
p(2105) = p(9)*p(568)
p(2106) = p(9)*p(569)
p(2107) = p(9)*p(588)
p(2108) = p(3)*p(938) - p(1778)
p(2109) = p(39)*p(331) - p(1780)
p(2110) = p(91)*p(178) - p(1783)
p(2111) = p(16)*p(566) - p(1787)
p(2112) = p(1)*p(1359) - p(1784) - p(2111)
p(2113) = p(2)*p(1476) - p(2112)
p(2114) = p(1)*p(1360) - p(1788) - p(2112)
p(2115) = p(7)*p(843) - p(1787)
p(2116) = p(1)*p(1362) - p(1792) - p(2114)
p(2117) = p(49)*p(279) - p(1783)
p(2118) = p(1)*p(1364) - p(1795) - p(2116)
p(2119) = p(69)*p(194) - p(1780)
p(2120) = p(1)*p(1366) - p(1797) - p(2118)
p(2121) = p(11)*p(588) - p(1778)
p(2122) = p(1)*p(1368) - p(1802) - p(1801)
p(2123) = p(1)*p(1369) - p(1809)
p(2124) = p(3)*p(940) - p(1811)
p(2125) = p(39)*p(333) - p(1813)
p(2126) = p(93)*p(178) - p(1816)
p(2127) = p(18)*p(566) - p(1820)
p(2128) = p(1)*p(1373) - p(1817) - p(2127)
p(2129) = p(1)*p(1374) - p(1820)
p(2130) = p(1)*p(1375) - p(1821) - p(2128)
p(2131) = p(21)*p(509) - p(1816)
p(2132) = p(1)*p(1377) - p(1824) - p(2130)
p(2133) = p(101)*p(153) - p(1813)
p(2134) = p(1)*p(1379) - p(1826) - p(2132)
p(2135) = p(34)*p(346) - p(1811)
p(2136) = p(6)*p(938) - p(1810)
p(2137) = p(1)*p(1382) - p(1835) - p(1833)
p(2138) = p(2)*p(1369) - p(1810)
p(2139) = p(1)*p(1384) - p(1853) - p(1852)
p(2140) = p(1)*p(1385) - p(1858)
p(2141) = p(3)*p(942) - p(1862)
p(2142) = p(39)*p(335) - p(1864)
p(2143) = p(95)*p(178) - p(1867)
p(2144) = p(1)*p(1388) - p(1865) - p(2143)
p(2145) = p(2)*p(1374) - p(1841)
p(2146) = p(1)*p(1389) - p(1868) - p(2144)
p(2147) = p(7)*p(869) - p(1867)
p(2148) = p(1)*p(1391) - p(1871) - p(2146)
p(2149) = p(49)*p(299) - p(1864)
p(2150) = p(1)*p(1393) - p(1873) - p(2148)
p(2151) = p(84)*p(194) - p(1862)
p(2152) = p(6)*p(940) - p(1848)
p(2153) = p(1)*p(1396) - p(1881) - p(1879)
p(2154) = p(8)*p(851) - p(1848)
p(2155) = p(45)*p(331) - p(1860)
p(2156) = p(1)*p(1399) - p(1900) - p(1898)
p(2157) = p(2)*p(1385) - p(1860)
p(2158) = p(1)*p(1401) - p(1916) - p(1915)
p(2159) = p(1)*p(1402) - p(1919)
p(2160) = p(3)*p(944) - p(1925)
p(2161) = p(39)*p(337) - p(1927)
p(2162) = p(98)*p(178) - p(1930)
p(2163) = p(1)*p(1405) - p(1928) - p(2162)
p(2164) = p(1)*p(1406) - p(1930)
p(2165) = p(1)*p(1407) - p(1931) - p(2163)
p(2166) = p(21)*p(536) - p(1927)
p(2167) = p(1)*p(1409) - p(1933) - p(2165)
p(2168) = p(101)*p(173) - p(1925)
p(2169) = p(6)*p(942) - p(1892)
p(2170) = p(1)*p(1412) - p(1940) - p(1938)
p(2171) = p(22)*p(505) - p(1892)
p(2172) = p(45)*p(333) - p(1861)
p(2173) = p(1)*p(1415) - p(1956) - p(1954)
p(2174) = p(8)*p(865) - p(1861)
p(2175) = p(91)*p(187) - p(1921)
p(2176) = p(1)*p(1418) - p(1972) - p(1970)
p(2177) = p(2)*p(1402) - p(1921)
p(2178) = p(1)*p(1421) - p(1986) - p(1985)
p(2179) = p(1)*p(1422) - p(1987)
p(2180) = p(14)*p(533)
p(2181) = p(3)*p(946) - p(1992)
p(2182) = p(39)*p(340) - p(1994)
p(2183) = p(1)*p(1425) - p(1993) - p(2182)
p(2184) = p(2)*p(1406) - p(1944)
p(2185) = p(1)*p(1426) - p(1995) - p(2183)
p(2186) = p(7)*p(904) - p(1994)
p(2187) = p(1)*p(1428) - p(1997) - p(2185)
p(2188) = p(49)*p(323) - p(1992)
p(2189) = p(6)*p(944) - p(1949)
p(2190) = p(1)*p(1431) - p(2003) - p(2001)
p(2191) = p(50)*p(285) - p(1949)
p(2192) = p(45)*p(335) - p(1912)
p(2193) = p(1)*p(1434) - p(2016) - p(2014)
p(2194) = p(22)*p(518) - p(1912)
p(2195) = p(93)*p(187) - p(1923)
p(2196) = p(1)*p(1437) - p(2029) - p(2027)
p(2197) = p(8)*p(881) - p(1923)
p(2198) = p(14)*p(545)
p(2199) = p(16)*p(578) - p(1989)
p(2200) = p(1)*p(1441) - p(2040) - p(2038)
p(2201) = p(2)*p(1422) - p(1989)
p(2202) = p(14)*p(549)
p(2203) = p(6)*p(899) - p(2042) - p(2041)
p(2204) = p(1)*p(1491) - p(2203)
p(2205) = p(14)*p(578)
p(2206) = p(3)*p(949) - p(2044)
p(2207) = p(39)*p(343) - p(2046)
p(2208) = p(1)*p(1445) - p(2045) - p(2207)
p(2209) = p(1)*p(1446) - p(2046)
p(2210) = p(1)*p(1447) - p(2047) - p(2208)
p(2211) = p(21)*p(561) - p(2044)
p(2212) = p(6)*p(946) - p(2010)
p(2213) = p(1)*p(1450) - p(2052) - p(2050)
p(2214) = p(102)*p(150) - p(2010)
p(2215) = p(45)*p(337) - p(1966)
p(2216) = p(1)*p(1453) - p(2062) - p(2060)
p(2217) = p(50)*p(296) - p(1966)
p(2218) = p(95)*p(187) - p(1924)
p(2219) = p(1)*p(1457) - p(2072) - p(2070)
p(2220) = p(22)*p(532) - p(1924)
p(2221) = p(14)*p(559)
p(2222) = p(18)*p(578) - p(1991)
p(2223) = p(2)*p(1441) - p(2042) - p(2203)
p(2224) = p(8)*p(900) - p(1991)
p(2225) = p(14)*p(581)
p(2226) = p(3)*p(952) - p(2076)
p(2227) = p(2)*p(1445) - p(2053) - p(2049)
p(2228) = p(2)*p(1446) - p(2054)
p(2229) = p(1)*p(1461) - p(2077) - p(2227)
p(2230) = p(7)*p(929) - p(2076)
p(2231) = p(6)*p(949) - p(2057)
p(2232) = p(1)*p(1464) - p(2081) - p(2079)
p(2233) = p(73)*p(195) - p(2057)
p(2234) = p(45)*p(340) - p(2024)
p(2235) = p(1)*p(1467) - p(2088) - p(2086)
p(2236) = p(102)*p(160) - p(2024)
p(2237) = p(14)*p(565)
p(2238) = p(98)*p(187) - p(1983)
p(2239) = p(2)*p(1457) - p(2074) - p(2223)
p(2240) = p(50)*p(310) - p(1983)
p(2241) = p(14)*p(584)
p(2242) = p(3)*p(955) - p(2092)
p(2243) = p(2)*p(1461) - p(2082) - p(2079)
p(2244) = p(1)*p(1471) - p(2092)
p(2245) = p(6)*p(952) - p(2084)
p(2246) = p(1)*p(1473) - p(2095) - p(2093)
p(2247) = p(32)*p(347) - p(2084)
p(2248) = p(45)*p(343) - p(2068)
p(2249) = p(2)*p(1467) - p(2090) - p(2239)
p(2250) = p(81)*p(195) - p(2068)
p(2251) = p(14)*p(587)
p(2252) = p(2)*p(1470) - p(2094) - p(2093)
p(2253) = p(2)*p(1471) - p(2095)
p(2254) = p(6)*p(955) - p(2096)
p(2255) = p(2)*p(1473) - p(2097) - p(2249)
p(2256) = p(13)*p(589) - p(2096)
p(2257) = p(14)*p(589)
p(2258) = p(1)*p(1476) - p(2101)
p(2259) = p(21)*p(566) - p(2098)
p(2260) = p(101)*p(178) - p(2099)
p(2261) = p(39)*p(346) - p(2104)
p(2262) = p(3)*p(958) - p(2107)
p(2263) = p(1)*p(1481) - p(2108)
p(2264) = p(2)*p(1504) - p(2263)
p(2265) = p(1)*p(1483) - p(2124)
p(2266) = p(8)*p(958) - p(2265)
p(2267) = p(1)*p(1485) - p(2141)
p(2268) = p(22)*p(588) - p(2267)
p(2269) = p(1)*p(1487) - p(2160)
p(2270) = p(50)*p(346) - p(2269)
p(2271) = p(1)*p(1489) - p(2181)
p(2272) = p(102)*p(194) - p(2271)
p(2273) = p(1)*p(1492) - p(2206)
p(2274) = p(101)*p(195) - p(2273)
p(2275) = p(2)*p(1491) - p(2205)
p(2276) = p(1)*p(1495) - p(2226)
p(2277) = p(49)*p(347) - p(2276)
p(2278) = p(22)*p(578) - p(2180)
p(2279) = p(1)*p(1498) - p(2242)
p(2280) = p(21)*p(589) - p(2279)
p(2281) = p(102)*p(187) - p(2198)
p(2282) = p(1)*p(1501) - p(2252)
p(2283) = p(7)*p(959) - p(2282)
p(2284) = p(45)*p(347) - p(2237)
p(2285) = p(2)*p(1501) - p(2254)
p(2286) = p(1)*p(1505) - p(2285)
p(2287) = p(6)*p(959) - p(2257)
p(2288) = p(1)*p(1504) - p(2262)
p(2289) = p(2)*p(1505) - p(2287)
p(2290) = p(9)*p(590)
p(2291) = p(9)*p(591)
p(2292) = p(9)*p(613)
p(2293) = p(9)*p(592)
p(2294) = p(9)*p(614)
p(2295) = p(9)*p(615)
p(2296) = p(9)*p(701)
p(2297) = p(9)*p(726)
p(2298) = p(9)*p(626)
p(2299) = p(9)*p(593)
p(2300) = p(9)*p(627)
p(2301) = p(9)*p(594)
p(2302) = p(9)*p(595)
p(2303) = p(9)*p(628)
p(2304) = p(9)*p(596)
p(2305) = p(9)*p(629)
p(2306) = p(9)*p(630)
p(2307) = p(9)*p(631)
p(2308) = p(9)*p(727)
p(2309) = p(9)*p(636)
p(2310) = p(9)*p(637)
p(2311) = p(9)*p(734)
p(2312) = p(9)*p(597)
p(2313) = p(9)*p(638)
p(2314) = p(9)*p(598)
p(2315) = p(9)*p(639)
p(2316) = p(9)*p(640)
p(2317) = p(9)*p(735)
p(2318) = p(9)*p(599)
p(2319) = p(9)*p(641)
p(2320) = p(9)*p(736)
p(2321) = p(9)*p(755)
p(2322) = p(9)*p(648)
p(2323) = p(9)*p(600)
p(2324) = p(9)*p(649)
p(2325) = p(9)*p(601)
p(2326) = p(9)*p(650)
p(2327) = p(9)*p(651)
p(2328) = p(9)*p(652)
p(2329) = p(9)*p(756)
p(2330) = p(9)*p(766)
p(2331) = p(9)*p(656)
p(2332) = p(9)*p(657)
p(2333) = p(9)*p(767)
p(2334) = p(9)*p(658)
p(2335) = p(9)*p(602)
p(2336) = p(9)*p(659)
p(2337) = p(9)*p(660)
p(2338) = p(9)*p(661)
p(2339) = p(9)*p(768)
p(2340) = p(9)*p(769)
p(2341) = p(9)*p(662)
p(2342) = p(9)*p(663)
p(2343) = p(9)*p(770)
p(2344) = p(9)*p(665)
p(2345) = p(9)*p(774)
p(2346) = p(9)*p(666)
p(2347) = p(9)*p(775)
p(2348) = p(9)*p(776)
p(2349) = p(9)*p(777)
p(2350) = p(9)*p(882)
p(2351) = p(9)*p(791)
p(2352) = p(9)*p(670)
p(2353) = p(9)*p(603)
p(2354) = p(9)*p(671)
p(2355) = p(9)*p(672)
p(2356) = p(9)*p(792)
p(2357) = p(9)*p(799)
p(2358) = p(9)*p(674)
p(2359) = p(9)*p(800)
p(2360) = p(9)*p(675)
p(2361) = p(9)*p(676)
p(2362) = p(9)*p(801)
p(2363) = p(9)*p(802)
p(2364) = p(9)*p(803)
p(2365) = p(9)*p(807)
p(2366) = p(9)*p(808)
p(2367) = p(9)*p(809)
p(2368) = p(9)*p(898)
p(2369) = p(9)*p(901)
p(2370) = p(9)*p(818)
p(2371) = p(9)*p(678)
p(2372) = p(9)*p(679)
p(2373) = p(9)*p(819)
p(2374) = p(9)*p(823)
p(2375) = p(9)*p(824)
p(2376) = p(9)*p(825)
p(2377) = p(9)*p(913)
p(2378) = p(9)*p(917)
p(2379) = p(9)*p(830)
p(2380) = p(9)*p(831)
p(2381) = p(9)*p(927)
p(2382) = p(9)*p(933)
p(2383) = p(9)*p(692)
p(2384) = p(9)*p(693)
p(2385) = p(9)*p(694)
p(2386) = p(9)*p(850)
p(2387) = p(9)*p(604)
p(2388) = p(9)*p(605)
p(2389) = p(9)*p(606)
p(2390) = p(9)*p(695)
p(2391) = p(9)*p(607)
p(2392) = p(9)*p(608)
p(2393) = p(9)*p(696)
p(2394) = p(9)*p(609)
p(2395) = p(9)*p(610)
p(2396) = p(9)*p(697)
p(2397) = p(9)*p(611)
p(2398) = p(9)*p(698)
p(2399) = p(9)*p(612)
p(2400) = p(9)*p(699)
p(2401) = p(9)*p(700)
p(2402) = p(9)*p(851)
p(2403) = p(14)*p(934)
p(2404) = p(14)*p(935)
p(2405) = p(14)*p(936)
p(2406) = p(14)*p(937)
p(2407) = p(9)*p(861)
p(2408) = p(9)*p(711)
p(2409) = p(9)*p(712)
p(2410) = p(9)*p(713)
p(2411) = p(9)*p(714)
p(2412) = p(9)*p(715)
p(2413) = p(9)*p(862)
p(2414) = p(9)*p(716)
p(2415) = p(9)*p(616)
p(2416) = p(9)*p(617)
p(2417) = p(9)*p(618)
p(2418) = p(9)*p(619)
p(2419) = p(9)*p(717)
p(2420) = p(9)*p(718)
p(2421) = p(9)*p(620)
p(2422) = p(9)*p(621)
p(2423) = p(9)*p(622)
p(2424) = p(9)*p(719)
p(2425) = p(9)*p(720)
p(2426) = p(9)*p(623)
p(2427) = p(9)*p(624)
p(2428) = p(9)*p(721)
p(2429) = p(9)*p(722)
p(2430) = p(9)*p(625)
p(2431) = p(9)*p(723)
p(2432) = p(9)*p(724)
p(2433) = p(9)*p(725)
p(2434) = p(9)*p(863)
p(2435) = p(14)*p(839)
p(2436) = p(14)*p(840)
p(2437) = p(14)*p(841)
p(2438) = p(14)*p(842)
p(2439) = p(14)*p(843)
p(2440) = p(14)*p(844)
p(2441) = p(14)*p(845)
p(2442) = p(14)*p(846)
p(2443) = p(14)*p(847)
p(2444) = p(14)*p(848)
p(2445) = p(14)*p(849)
p(2446) = p(9)*p(728)
p(2447) = p(9)*p(729)
p(2448) = p(9)*p(864)
p(2449) = p(9)*p(632)
p(2450) = p(9)*p(633)
p(2451) = p(9)*p(730)
p(2452) = p(9)*p(634)
p(2453) = p(9)*p(731)
p(2454) = p(9)*p(635)
p(2455) = p(9)*p(732)
p(2456) = p(9)*p(733)
p(2457) = p(9)*p(865)
p(2458) = p(14)*p(692)
p(2459) = p(14)*p(693)
p(2460) = p(14)*p(694)
p(2461) = p(14)*p(695)
p(2462) = p(14)*p(696)
p(2463) = p(14)*p(697)
p(2464) = p(14)*p(698)
p(2465) = p(14)*p(699)
p(2466) = p(14)*p(700)
p(2467) = p(14)*p(613)
p(2468) = p(14)*p(614)
p(2469) = p(14)*p(615)
p(2470) = p(9)*p(874)
p(2471) = p(9)*p(743)
p(2472) = p(9)*p(744)
p(2473) = p(9)*p(745)
p(2474) = p(9)*p(746)
p(2475) = p(9)*p(875)
p(2476) = p(9)*p(747)
p(2477) = p(9)*p(642)
p(2478) = p(9)*p(643)
p(2479) = p(9)*p(644)
p(2480) = p(9)*p(748)
p(2481) = p(9)*p(749)
p(2482) = p(9)*p(645)
p(2483) = p(9)*p(646)
p(2484) = p(9)*p(750)
p(2485) = p(9)*p(751)
p(2486) = p(9)*p(647)
p(2487) = p(9)*p(752)
p(2488) = p(9)*p(753)
p(2489) = p(9)*p(754)
p(2490) = p(9)*p(876)
p(2491) = p(14)*p(852)
p(2492) = p(14)*p(853)
p(2493) = p(14)*p(854)
p(2494) = p(14)*p(855)
p(2495) = p(14)*p(856)
p(2496) = p(14)*p(857)
p(2497) = p(14)*p(858)
p(2498) = p(14)*p(859)
p(2499) = p(14)*p(860)
p(2500) = p(9)*p(877)
p(2501) = p(9)*p(757)
p(2502) = p(9)*p(758)
p(2503) = p(9)*p(759)
p(2504) = p(9)*p(878)
p(2505) = p(9)*p(760)
p(2506) = p(9)*p(653)
p(2507) = p(9)*p(654)
p(2508) = p(9)*p(761)
p(2509) = p(9)*p(762)
p(2510) = p(9)*p(655)
p(2511) = p(9)*p(763)
p(2512) = p(9)*p(764)
p(2513) = p(9)*p(765)
p(2514) = p(9)*p(879)
p(2515) = p(14)*p(711)
p(2516) = p(14)*p(712)
p(2517) = p(14)*p(713)
p(2518) = p(14)*p(714)
p(2519) = p(14)*p(715)
p(2520) = p(14)*p(716)
p(2521) = p(14)*p(717)
p(2522) = p(14)*p(718)
p(2523) = p(14)*p(719)
p(2524) = p(14)*p(720)
p(2525) = p(14)*p(721)
p(2526) = p(14)*p(722)
p(2527) = p(14)*p(723)
p(2528) = p(14)*p(724)
p(2529) = p(14)*p(725)
p(2530) = p(14)*p(626)
p(2531) = p(14)*p(627)
p(2532) = p(14)*p(628)
p(2533) = p(14)*p(629)
p(2534) = p(14)*p(630)
p(2535) = p(14)*p(631)
p(2536) = p(9)*p(771)
p(2537) = p(9)*p(880)
p(2538) = p(9)*p(664)
p(2539) = p(9)*p(772)
p(2540) = p(9)*p(773)
p(2541) = p(9)*p(881)
p(2542) = p(14)*p(728)
p(2543) = p(14)*p(729)
p(2544) = p(14)*p(730)
p(2545) = p(14)*p(731)
p(2546) = p(14)*p(732)
p(2547) = p(14)*p(733)
p(2548) = p(14)*p(636)
p(2549) = p(14)*p(637)
p(2550) = p(14)*p(638)
p(2551) = p(14)*p(639)
p(2552) = p(14)*p(640)
p(2553) = p(14)*p(641)
p(2554) = p(9)*p(889)
p(2555) = p(9)*p(782)
p(2556) = p(9)*p(783)
p(2557) = p(9)*p(784)
p(2558) = p(9)*p(890)
p(2559) = p(9)*p(785)
p(2560) = p(9)*p(667)
p(2561) = p(9)*p(668)
p(2562) = p(9)*p(786)
p(2563) = p(9)*p(787)
p(2564) = p(9)*p(669)
p(2565) = p(9)*p(788)
p(2566) = p(9)*p(789)
p(2567) = p(9)*p(790)
p(2568) = p(9)*p(891)
p(2569) = p(14)*p(866)
p(2570) = p(14)*p(867)
p(2571) = p(14)*p(868)
p(2572) = p(14)*p(869)
p(2573) = p(14)*p(870)
p(2574) = p(14)*p(871)
p(2575) = p(14)*p(872)
p(2576) = p(14)*p(873)
p(2577) = p(9)*p(892)
p(2578) = p(9)*p(793)
p(2579) = p(9)*p(794)
p(2580) = p(9)*p(893)
p(2581) = p(9)*p(795)
p(2582) = p(9)*p(673)
p(2583) = p(9)*p(796)
p(2584) = p(9)*p(797)
p(2585) = p(9)*p(798)
p(2586) = p(9)*p(894)
p(2587) = p(14)*p(743)
p(2588) = p(14)*p(744)
p(2589) = p(14)*p(745)
p(2590) = p(14)*p(746)
p(2591) = p(14)*p(747)
p(2592) = p(14)*p(748)
p(2593) = p(14)*p(749)
p(2594) = p(14)*p(750)
p(2595) = p(14)*p(751)
p(2596) = p(14)*p(752)
p(2597) = p(14)*p(753)
p(2598) = p(14)*p(754)
p(2599) = p(14)*p(648)
p(2600) = p(14)*p(649)
p(2601) = p(14)*p(650)
p(2602) = p(14)*p(651)
p(2603) = p(14)*p(652)
p(2604) = p(9)*p(895)
p(2605) = p(9)*p(804)
p(2606) = p(9)*p(896)
p(2607) = p(9)*p(805)
p(2608) = p(9)*p(806)
p(2609) = p(9)*p(897)
p(2610) = p(14)*p(757)
p(2611) = p(14)*p(758)
p(2612) = p(14)*p(759)
p(2613) = p(14)*p(760)
p(2614) = p(14)*p(761)
p(2615) = p(14)*p(762)
p(2616) = p(14)*p(763)
p(2617) = p(14)*p(764)
p(2618) = p(14)*p(765)
p(2619) = p(14)*p(656)
p(2620) = p(14)*p(657)
p(2621) = p(14)*p(658)
p(2622) = p(14)*p(659)
p(2623) = p(14)*p(660)
p(2624) = p(14)*p(661)
p(2625) = p(14)*p(662)
p(2626) = p(14)*p(663)
p(2627) = p(9)*p(899)
p(2628) = p(9)*p(900)
p(2629) = p(14)*p(771)
p(2630) = p(14)*p(772)
p(2631) = p(14)*p(773)
p(2632) = p(14)*p(665)
p(2633) = p(14)*p(666)
p(2634) = p(9)*p(907)
p(2635) = p(9)*p(812)
p(2636) = p(9)*p(813)
p(2637) = p(9)*p(908)
p(2638) = p(9)*p(814)
p(2639) = p(9)*p(677)
p(2640) = p(9)*p(815)
p(2641) = p(9)*p(816)
p(2642) = p(9)*p(817)
p(2643) = p(9)*p(909)
p(2644) = p(14)*p(883)
p(2645) = p(14)*p(884)
p(2646) = p(14)*p(885)
p(2647) = p(14)*p(886)
p(2648) = p(14)*p(887)
p(2649) = p(14)*p(888)
p(2650) = p(9)*p(910)
p(2651) = p(9)*p(820)
p(2652) = p(9)*p(911)
p(2653) = p(9)*p(821)
p(2654) = p(9)*p(822)
p(2655) = p(9)*p(912)
p(2656) = p(14)*p(782)
p(2657) = p(14)*p(783)
p(2658) = p(14)*p(784)
p(2659) = p(14)*p(785)
p(2660) = p(14)*p(786)
p(2661) = p(14)*p(787)
p(2662) = p(14)*p(788)
p(2663) = p(14)*p(789)
p(2664) = p(14)*p(790)
p(2665) = p(14)*p(670)
p(2666) = p(14)*p(671)
p(2667) = p(14)*p(672)
p(2668) = p(9)*p(914)
p(2669) = p(9)*p(915)
p(2670) = p(9)*p(916)
p(2671) = p(14)*p(793)
p(2672) = p(14)*p(794)
p(2673) = p(14)*p(795)
p(2674) = p(14)*p(796)
p(2675) = p(14)*p(797)
p(2676) = p(14)*p(798)
p(2677) = p(14)*p(674)
p(2678) = p(14)*p(675)
p(2679) = p(14)*p(676)
p(2680) = p(9)*p(948)
p(2681) = p(14)*p(804)
p(2682) = p(14)*p(805)
p(2683) = p(14)*p(806)
p(2684) = p(9)*p(921)
p(2685) = p(9)*p(827)
p(2686) = p(9)*p(922)
p(2687) = p(9)*p(828)
p(2688) = p(9)*p(829)
p(2689) = p(9)*p(923)
p(2690) = p(14)*p(902)
p(2691) = p(14)*p(903)
p(2692) = p(14)*p(904)
p(2693) = p(14)*p(905)
p(2694) = p(14)*p(906)
p(2695) = p(9)*p(924)
p(2696) = p(9)*p(925)
p(2697) = p(9)*p(926)
p(2698) = p(14)*p(812)
p(2699) = p(14)*p(813)
p(2700) = p(14)*p(814)
p(2701) = p(14)*p(815)
p(2702) = p(14)*p(816)
p(2703) = p(14)*p(817)
p(2704) = p(14)*p(678)
p(2705) = p(14)*p(679)
p(2706) = p(9)*p(951)
p(2707) = p(14)*p(820)
p(2708) = p(14)*p(821)
p(2709) = p(14)*p(822)
p(2710) = p(9)*p(930)
p(2711) = p(9)*p(931)
p(2712) = p(9)*p(932)
p(2713) = p(14)*p(918)
p(2714) = p(14)*p(919)
p(2715) = p(14)*p(920)
p(2716) = p(9)*p(954)
p(2717) = p(14)*p(827)
p(2718) = p(14)*p(828)
p(2719) = p(14)*p(829)
p(2720) = p(9)*p(957)
p(2721) = p(14)*p(928)
p(2722) = p(14)*p(929)
p(2723) = p(9)*p(938)
p(2724) = p(9)*p(839)
p(2725) = p(9)*p(680)
p(2726) = p(9)*p(840)
p(2727) = p(9)*p(681)
p(2728) = p(9)*p(682)
p(2729) = p(9)*p(841)
p(2730) = p(9)*p(683)
p(2731) = p(9)*p(684)
p(2732) = p(9)*p(685)
p(2733) = p(9)*p(842)
p(2734) = p(9)*p(686)
p(2735) = p(9)*p(687)
p(2736) = p(9)*p(688)
p(2737) = p(9)*p(843)
p(2738) = p(9)*p(844)
p(2739) = p(9)*p(689)
p(2740) = p(9)*p(690)
p(2741) = p(9)*p(845)
p(2742) = p(9)*p(846)
p(2743) = p(9)*p(691)
p(2744) = p(9)*p(847)
p(2745) = p(9)*p(848)
p(2746) = p(9)*p(849)
p(2747) = p(9)*p(939)
p(2748) = p(3)*p(1136) - p(2383)
p(2749) = p(7)*p(1136) - p(2387)
p(2750) = p(49)*p(401) - p(2394)
p(2751) = p(114)*p(194) - p(2399)
p(2752) = p(24)*p(588) - p(2402)
p(2753) = p(3)*p(1368) - p(2386) - p(2752)
p(2754) = p(1)*p(1802) - p(2386) - p(2753)
p(2755) = p(1)*p(1803) - p(2390) - p(2754)
p(2756) = p(1)*p(1804) - p(2393) - p(2755)
p(2757) = p(6)*p(1476) - p(2748)
p(2758) = p(32)*p(566) - p(2387)
p(2759) = p(150)*p(178) - p(2394)
p(2760) = p(39)*p(505) - p(2399)
p(2761) = p(3)*p(1369) - p(2402)
p(2762) = p(14)*p(958)
p(2763) = p(9)*p(940)
p(2764) = p(9)*p(852)
p(2765) = p(9)*p(702)
p(2766) = p(9)*p(853)
p(2767) = p(9)*p(703)
p(2768) = p(9)*p(704)
p(2769) = p(9)*p(854)
p(2770) = p(9)*p(705)
p(2771) = p(9)*p(706)
p(2772) = p(9)*p(707)
p(2773) = p(9)*p(855)
p(2774) = p(9)*p(708)
p(2775) = p(9)*p(709)
p(2776) = p(9)*p(856)
p(2777) = p(9)*p(857)
p(2778) = p(9)*p(710)
p(2779) = p(9)*p(858)
p(2780) = p(9)*p(859)
p(2781) = p(9)*p(860)
p(2782) = p(9)*p(941)
p(2783) = p(24)*p(570) - p(2296)
p(2784) = p(114)*p(181) - p(2293)
p(2785) = p(1)*p(1828) - p(2408) - p(2784)
p(2786) = p(4)*p(1136) - p(2292)
p(2787) = p(1)*p(1830) - p(2410) - p(2786)
p(2788) = p(1)*p(1831) - p(2411) - p(2787)
p(2789) = p(1)*p(1832) - p(2412) - p(2788)
p(2790) = p(1)*p(1833) - p(2413) - p(2789)
p(2791) = p(13)*p(938) - p(2762)
p(2792) = p(2)*p(1802) - p(2406) - p(2791)
p(2793) = p(32)*p(570) - p(2762)
p(2794) = p(1)*p(1835) - p(2413) - p(2792)
p(2795) = p(73)*p(331) - p(2762)
p(2796) = p(1)*p(1837) - p(2419) - p(2794)
p(2797) = p(150)*p(181) - p(2762)
p(2798) = p(1)*p(1839) - p(2424) - p(2796)
p(2799) = p(91)*p(285) - p(2762)
p(2800) = p(13)*p(843) - p(2292)
p(2801) = p(41)*p(505) - p(2762)
p(2802) = p(1)*p(1843) - p(2431) - p(2800)
p(2803) = p(16)*p(851) - p(2762)
p(2804) = p(69)*p(285) - p(2293)
p(2805) = p(4)*p(1369) - p(2762)
p(2806) = p(11)*p(851) - p(2296)
p(2807) = p(14)*p(938)
p(2808) = p(14)*p(939)
p(2809) = p(3)*p(1180) - p(2446)
p(2810) = p(7)*p(1180) - p(2449)
p(2811) = p(49)*p(430) - p(2454)
p(2812) = p(130)*p(194) - p(2457)
p(2813) = p(3)*p(1384) - p(2448) - p(2812)
p(2814) = p(1)*p(1853) - p(2448) - p(2813)
p(2815) = p(1)*p(1854) - p(2451) - p(2814)
p(2816) = p(45)*p(566) - p(2809)
p(2817) = p(160)*p(178) - p(2449)
p(2818) = p(39)*p(518) - p(2454)
p(2819) = p(3)*p(1385) - p(2457)
p(2820) = p(14)*p(850)
p(2821) = p(14)*p(851)
p(2822) = p(14)*p(701)
p(2823) = p(9)*p(942)
p(2824) = p(9)*p(866)
p(2825) = p(9)*p(737)
p(2826) = p(9)*p(867)
p(2827) = p(9)*p(738)
p(2828) = p(9)*p(739)
p(2829) = p(9)*p(868)
p(2830) = p(9)*p(740)
p(2831) = p(9)*p(741)
p(2832) = p(9)*p(869)
p(2833) = p(9)*p(870)
p(2834) = p(9)*p(742)
p(2835) = p(9)*p(871)
p(2836) = p(9)*p(872)
p(2837) = p(9)*p(873)
p(2838) = p(9)*p(943)
p(2839) = p(24)*p(572) - p(2297)
p(2840) = p(114)*p(183) - p(2299)
p(2841) = p(43)*p(401) - p(2302)
p(2842) = p(8)*p(1136) - p(2439)
p(2843) = p(1)*p(1877) - p(2473) - p(2842)
p(2844) = p(1)*p(1878) - p(2474) - p(2843)
p(2845) = p(1)*p(1879) - p(2475) - p(2844)
p(2846) = p(13)*p(940) - p(2807)
p(2847) = p(2)*p(1835) - p(2445) - p(2813)
p(2848) = p(32)*p(572) - p(2807)
p(2849) = p(1)*p(1881) - p(2475) - p(2847)
p(2850) = p(73)*p(333) - p(2807)
p(2851) = p(1)*p(1883) - p(2480) - p(2849)
p(2852) = p(150)*p(183) - p(2807)
p(2853) = p(6)*p(1374) - p(2439)
p(2854) = p(93)*p(285) - p(2807)
p(2855) = p(32)*p(509) - p(2302)
p(2856) = p(43)*p(505) - p(2807)
p(2857) = p(150)*p(153) - p(2299)
p(2858) = p(18)*p(851) - p(2807)
p(2859) = p(34)*p(505) - p(2297)
p(2860) = p(14)*p(940)
p(2861) = p(14)*p(941)
p(2862) = p(130)*p(181) - p(2317)
p(2863) = p(41)*p(430) - p(2314)
p(2864) = p(4)*p(1180) - p(2309)
p(2865) = p(1)*p(1896) - p(2502) - p(2864)
p(2866) = p(1)*p(1897) - p(2503) - p(2865)
p(2867) = p(1)*p(1898) - p(2504) - p(2866)
p(2868) = p(81)*p(331) - p(2821)
p(2869) = p(2)*p(1853) - p(2461) - p(2868)
p(2870) = p(160)*p(181) - p(2821)
p(2871) = p(1)*p(1900) - p(2504) - p(2869)
p(2872) = p(91)*p(296) - p(2821)
p(2873) = p(1)*p(1902) - p(2508) - p(2871)
p(2874) = p(41)*p(518) - p(2821)
p(2875) = p(81)*p(279) - p(2309)
p(2876) = p(16)*p(865) - p(2821)
p(2877) = p(69)*p(296) - p(2314)
p(2878) = p(4)*p(1385) - p(2821)
p(2879) = p(11)*p(865) - p(2317)
p(2880) = p(14)*p(861)
p(2881) = p(14)*p(862)
p(2882) = p(14)*p(863)
p(2883) = p(14)*p(726)
p(2884) = p(14)*p(727)
p(2885) = p(3)*p(1234) - p(2536)
p(2886) = p(7)*p(1234) - p(2538)
p(2887) = p(49)*p(464) - p(2541)
p(2888) = p(3)*p(1401) - p(2537) - p(2887)
p(2889) = p(1)*p(1916) - p(2537) - p(2888)
p(2890) = p(178)*p(187) - p(2885)
p(2891) = p(39)*p(532) - p(2538)
p(2892) = p(3)*p(1402) - p(2541)
p(2893) = p(14)*p(864)
p(2894) = p(14)*p(865)
p(2895) = p(14)*p(734)
p(2896) = p(14)*p(735)
p(2897) = p(14)*p(736)
p(2898) = p(9)*p(944)
p(2899) = p(9)*p(883)
p(2900) = p(9)*p(778)
p(2901) = p(9)*p(884)
p(2902) = p(9)*p(779)
p(2903) = p(9)*p(780)
p(2904) = p(9)*p(885)
p(2905) = p(9)*p(781)
p(2906) = p(9)*p(886)
p(2907) = p(9)*p(887)
p(2908) = p(9)*p(888)
p(2909) = p(9)*p(945)
p(2910) = p(24)*p(574) - p(2321)
p(2911) = p(114)*p(185) - p(2323)
p(2912) = p(46)*p(401) - p(2326)
p(2913) = p(1)*p(1936) - p(2556) - p(2912)
p(2914) = p(1)*p(1937) - p(2557) - p(2913)
p(2915) = p(1)*p(1938) - p(2558) - p(2914)
p(2916) = p(13)*p(942) - p(2860)
p(2917) = p(2)*p(1881) - p(2499) - p(2869)
p(2918) = p(32)*p(574) - p(2860)
p(2919) = p(1)*p(1940) - p(2558) - p(2917)
p(2920) = p(73)*p(335) - p(2860)
p(2921) = p(1)*p(1942) - p(2562) - p(2919)
p(2922) = p(150)*p(185) - p(2860)
p(2923) = p(13)*p(869) - p(2326)
p(2924) = p(95)*p(285) - p(2860)
p(2925) = p(73)*p(299) - p(2323)
p(2926) = p(46)*p(505) - p(2860)
p(2927) = p(84)*p(285) - p(2321)
p(2928) = p(14)*p(942)
p(2929) = p(14)*p(943)
p(2930) = p(130)*p(183) - p(2320)
p(2931) = p(1)*p(1951) - p(2577) - p(2930)
p(2932) = p(8)*p(1180) - p(2467)
p(2933) = p(1)*p(1953) - p(2579) - p(2932)
p(2934) = p(1)*p(1954) - p(2580) - p(2933)
p(2935) = p(81)*p(333) - p(2822)
p(2936) = p(2)*p(1900) - p(2521) - p(2888)
p(2937) = p(160)*p(183) - p(2822)
p(2938) = p(1)*p(1956) - p(2580) - p(2936)
p(2939) = p(93)*p(296) - p(2822)
p(2940) = p(45)*p(509) - p(2467)
p(2941) = p(43)*p(518) - p(2822)
p(2942) = p(1)*p(1960) - p(2585) - p(2940)
p(2943) = p(18)*p(865) - p(2822)
p(2944) = p(34)*p(518) - p(2320)
p(2945) = p(14)*p(874)
p(2946) = p(14)*p(875)
p(2947) = p(14)*p(876)
p(2948) = p(14)*p(755)
p(2949) = p(14)*p(756)
p(2950) = p(41)*p(464) - p(2347)
p(2951) = p(4)*p(1234) - p(2344)
p(2952) = p(1)*p(1969) - p(2605) - p(2951)
p(2953) = p(1)*p(1970) - p(2606) - p(2952)
p(2954) = p(91)*p(310) - p(2894)
p(2955) = p(2)*p(1916) - p(2544) - p(2954)
p(2956) = p(41)*p(532) - p(2894)
p(2957) = p(1)*p(1972) - p(2606) - p(2955)
p(2958) = p(16)*p(881) - p(2894)
p(2959) = p(69)*p(310) - p(2344)
p(2960) = p(4)*p(1402) - p(2894)
p(2961) = p(11)*p(881) - p(2347)
p(2962) = p(14)*p(877)
p(2963) = p(14)*p(878)
p(2964) = p(14)*p(879)
p(2965) = p(14)*p(766)
p(2966) = p(14)*p(767)
p(2967) = p(14)*p(768)
p(2968) = p(14)*p(769)
p(2969) = p(14)*p(770)
p(2970) = p(3)*p(1290) - p(2627)
p(2971) = p(7)*p(1290) - p(2628)
p(2972) = p(3)*p(1421) - p(2627) - p(2971)
p(2973) = p(39)*p(578) - p(2970)
p(2974) = p(3)*p(1422) - p(2628)
p(2975) = p(14)*p(880)
p(2976) = p(14)*p(881)
p(2977) = p(14)*p(774)
p(2978) = p(14)*p(775)
p(2979) = p(14)*p(776)
p(2980) = p(14)*p(777)
p(2981) = p(9)*p(946)
p(2982) = p(9)*p(902)
p(2983) = p(9)*p(810)
p(2984) = p(9)*p(903)
p(2985) = p(9)*p(811)
p(2986) = p(9)*p(904)
p(2987) = p(9)*p(905)
p(2988) = p(9)*p(906)
p(2989) = p(9)*p(947)
p(2990) = p(24)*p(576) - p(2351)
p(2991) = p(114)*p(188) - p(2353)
p(2992) = p(50)*p(401) - p(2572)
p(2993) = p(1)*p(2000) - p(2636) - p(2992)
p(2994) = p(1)*p(2001) - p(2637) - p(2993)
p(2995) = p(13)*p(944) - p(2928)
p(2996) = p(2)*p(1940) - p(2576) - p(2936)
p(2997) = p(32)*p(576) - p(2928)
p(2998) = p(1)*p(2003) - p(2637) - p(2996)
p(2999) = p(73)*p(337) - p(2928)
p(3000) = p(6)*p(1406) - p(2572)
p(3001) = p(150)*p(188) - p(2928)
p(3002) = p(32)*p(536) - p(2353)
p(3003) = p(98)*p(285) - p(2928)
p(3004) = p(150)*p(173) - p(2351)
p(3005) = p(14)*p(944)
p(3006) = p(14)*p(945)
p(3007) = p(130)*p(185) - p(2340)
p(3008) = p(46)*p(430) - p(2342)
p(3009) = p(1)*p(2013) - p(2651) - p(3008)
p(3010) = p(1)*p(2014) - p(2652) - p(3009)
p(3011) = p(81)*p(335) - p(2883)
p(3012) = p(2)*p(1956) - p(2592) - p(2955)
p(3013) = p(160)*p(185) - p(2883)
p(3014) = p(1)*p(2016) - p(2652) - p(3012)
p(3015) = p(95)*p(296) - p(2883)
p(3016) = p(81)*p(299) - p(2342)
p(3017) = p(46)*p(518) - p(2883)
p(3018) = p(84)*p(296) - p(2340)
p(3019) = p(14)*p(889)
p(3020) = p(14)*p(890)
p(3021) = p(14)*p(891)
p(3022) = p(14)*p(791)
p(3023) = p(14)*p(792)
p(3024) = p(43)*p(464) - p(2349)
p(3025) = p(8)*p(1234) - p(2548)
p(3026) = p(1)*p(2027) - p(2669) - p(3025)
p(3027) = p(93)*p(310) - p(2896)
p(3028) = p(2)*p(1972) - p(2614) - p(2972)
p(3029) = p(43)*p(532) - p(2896)
p(3030) = p(153)*p(187) - p(2548)
p(3031) = p(18)*p(881) - p(2896)
p(3032) = p(34)*p(532) - p(2349)
p(3033) = p(14)*p(892)
p(3034) = p(14)*p(893)
p(3035) = p(14)*p(894)
p(3036) = p(14)*p(799)
p(3037) = p(14)*p(800)
p(3038) = p(14)*p(801)
p(3039) = p(14)*p(802)
p(3040) = p(14)*p(803)
p(3041) = p(4)*p(1290) - p(2369)
p(3042) = p(1)*p(2038) - p(2680) - p(3041)
p(3043) = p(16)*p(900) - p(2976)
p(3044) = p(2)*p(1986) - p(2630) - p(3043)
p(3045) = p(4)*p(1422) - p(2976)
p(3046) = p(11)*p(900) - p(2369)
p(3047) = p(14)*p(895)
p(3048) = p(14)*p(896)
p(3049) = p(14)*p(897)
p(3050) = p(14)*p(807)
p(3051) = p(14)*p(808)
p(3052) = p(14)*p(809)
p(3053) = p(6)*p(1290) - p(2681)
p(3054) = p(3)*p(1491) - p(3053)
p(3055) = p(14)*p(899)
p(3056) = p(14)*p(900)
p(3057) = p(9)*p(949)
p(3058) = p(9)*p(918)
p(3059) = p(9)*p(826)
p(3060) = p(9)*p(919)
p(3061) = p(9)*p(920)
p(3062) = p(9)*p(950)
p(3063) = p(24)*p(579) - p(2370)
p(3064) = p(114)*p(191) - p(2372)
p(3065) = p(1)*p(2049) - p(2685) - p(3064)
p(3066) = p(1)*p(2050) - p(2686) - p(3065)
p(3067) = p(13)*p(946) - p(3005)
p(3068) = p(2)*p(2003) - p(2649) - p(3012)
p(3069) = p(32)*p(579) - p(3005)
p(3070) = p(1)*p(2052) - p(2686) - p(3068)
p(3071) = p(73)*p(340) - p(3005)
p(3072) = p(13)*p(904) - p(2372)
p(3073) = p(150)*p(191) - p(3005)
p(3074) = p(73)*p(323) - p(2370)
p(3075) = p(14)*p(946)
p(3076) = p(14)*p(947)
p(3077) = p(130)*p(188) - p(2363)
p(3078) = p(50)*p(430) - p(2601)
p(3079) = p(1)*p(2060) - p(2696) - p(3078)
p(3080) = p(81)*p(337) - p(2948)
p(3081) = p(2)*p(2016) - p(2660) - p(3028)
p(3082) = p(160)*p(188) - p(2948)
p(3083) = p(45)*p(536) - p(2601)
p(3084) = p(98)*p(296) - p(2948)
p(3085) = p(160)*p(173) - p(2363)
p(3086) = p(14)*p(907)
p(3087) = p(14)*p(908)
p(3088) = p(14)*p(909)
p(3089) = p(14)*p(818)
p(3090) = p(14)*p(819)
p(3091) = p(2)*p(2026) - p(2671) - p(3041)
p(3092) = p(1)*p(2070) - p(2706) - p(3091)
p(3093) = p(95)*p(310) - p(2897)
p(3094) = p(2)*p(2029) - p(2674) - p(3044)
p(3095) = p(46)*p(532) - p(2897)
p(3096) = p(1)*p(2072) - p(2706) - p(3094)
p(3097) = p(14)*p(910)
p(3098) = p(14)*p(911)
p(3099) = p(14)*p(912)
p(3100) = p(14)*p(823)
p(3101) = p(14)*p(824)
p(3102) = p(14)*p(825)
p(3103) = p(8)*p(1290) - p(2632)
p(3104) = p(18)*p(900) - p(2978)
p(3105) = p(34)*p(578) - p(2632)
p(3106) = p(14)*p(914)
p(3107) = p(14)*p(915)
p(3108) = p(14)*p(916)
p(3109) = p(9)*p(952)
p(3110) = p(9)*p(928)
p(3111) = p(9)*p(929)
p(3112) = p(9)*p(953)
p(3113) = p(24)*p(582) - p(2379)
p(3114) = p(114)*p(195) - p(2692)
p(3115) = p(1)*p(2079) - p(2711) - p(3114)
p(3116) = p(13)*p(949) - p(3075)
p(3117) = p(2)*p(2052) - p(2694) - p(3081)
p(3118) = p(32)*p(582) - p(3075)
p(3119) = p(6)*p(1446) - p(2692)
p(3120) = p(73)*p(343) - p(3075)
p(3121) = p(32)*p(561) - p(2379)
p(3122) = p(14)*p(949)
p(3123) = p(14)*p(950)
p(3124) = p(130)*p(191) - p(2377)
p(3125) = p(1)*p(2086) - p(2716) - p(3124)
p(3126) = p(81)*p(340) - p(3022)
p(3127) = p(2)*p(2062) - p(2701) - p(3094)
p(3128) = p(160)*p(191) - p(3022)
p(3129) = p(81)*p(323) - p(2377)
p(3130) = p(14)*p(921)
p(3131) = p(14)*p(922)
p(3132) = p(14)*p(923)
p(3133) = p(14)*p(830)
p(3134) = p(14)*p(831)
p(3135) = p(50)*p(464) - p(2626)
p(3136) = p(98)*p(310) - p(2968)
p(3137) = p(173)*p(187) - p(2626)
p(3138) = p(14)*p(924)
p(3139) = p(14)*p(925)
p(3140) = p(14)*p(926)
p(3141) = p(9)*p(955)
p(3142) = p(9)*p(956)
p(3143) = p(24)*p(585) - p(2382)
p(3144) = p(1)*p(2093) - p(2720) - p(3143)
p(3145) = p(13)*p(952) - p(3122)
p(3146) = p(2)*p(2081) - p(2715) - p(3127)
p(3147) = p(32)*p(585) - p(3122)
p(3148) = p(13)*p(929) - p(2382)
p(3149) = p(14)*p(952)
p(3150) = p(14)*p(953)
p(3151) = p(130)*p(195) - p(2705)
p(3152) = p(81)*p(343) - p(3089)
p(3153) = p(45)*p(561) - p(2705)
p(3154) = p(14)*p(930)
p(3155) = p(14)*p(931)
p(3156) = p(14)*p(932)
p(3157) = p(9)*p(959)
p(3158) = p(24)*p(589) - p(2722)
p(3159) = p(13)*p(955) - p(3149)
p(3160) = p(6)*p(1471) - p(2722)
p(3161) = p(14)*p(955)
p(3162) = p(14)*p(956)
p(3163) = p(9)*p(832)
p(3164) = p(9)*p(833)
p(3165) = p(9)*p(834)
p(3166) = p(9)*p(835)
p(3167) = p(9)*p(836)
p(3168) = p(9)*p(934)
p(3169) = p(9)*p(837)
p(3170) = p(9)*p(935)
p(3171) = p(9)*p(838)
p(3172) = p(9)*p(936)
p(3173) = p(9)*p(937)
p(3174) = p(9)*p(958)
p(3175) = p(3)*p(1481) - p(2723)
p(3176) = p(39)*p(570) - p(2725)
p(3177) = p(178)*p(181) - p(2728)
p(3178) = p(41)*p(566) - p(2732)
p(3179) = p(4)*p(1476) - p(2737)
p(3180) = p(1)*p(2112) - p(2733) - p(3179)
p(3181) = p(1)*p(2113) - p(2737)
p(3182) = p(1)*p(2114) - p(2738) - p(3180)
p(3183) = p(21)*p(843) - p(2732)
p(3184) = p(1)*p(2116) - p(2742) - p(3182)
p(3185) = p(101)*p(279) - p(2728)
p(3186) = p(1)*p(2118) - p(2745) - p(3184)
p(3187) = p(69)*p(346) - p(2725)
p(3188) = p(1)*p(2120) - p(2747) - p(3186)
p(3189) = p(11)*p(958) - p(2723)
p(3190) = p(1)*p(2122) - p(2753) - p(2752)
p(3191) = p(1)*p(2123) - p(2761)
p(3192) = p(3)*p(1483) - p(2763)
p(3193) = p(39)*p(572) - p(2765)
p(3194) = p(178)*p(183) - p(2768)
p(3195) = p(43)*p(566) - p(2772)
p(3196) = p(1)*p(2127) - p(2769) - p(3195)
p(3197) = p(2)*p(2113) - p(2757)
p(3198) = p(1)*p(2128) - p(2773) - p(3196)
p(3199) = p(7)*p(1374) - p(2772)
p(3200) = p(1)*p(2130) - p(2777) - p(3198)
p(3201) = p(49)*p(509) - p(2768)
p(3202) = p(1)*p(2132) - p(2780) - p(3200)
p(3203) = p(153)*p(194) - p(2765)
p(3204) = p(1)*p(2134) - p(2782) - p(3202)
p(3205) = p(34)*p(588) - p(2763)
p(3206) = p(6)*p(1481) - p(2762)
p(3207) = p(1)*p(2137) - p(2792) - p(2790)
p(3208) = p(2)*p(2123) - p(2762)
p(3209) = p(1)*p(2139) - p(2813) - p(2812)
p(3210) = p(1)*p(2140) - p(2819)
p(3211) = p(3)*p(1485) - p(2823)
p(3212) = p(39)*p(574) - p(2825)
p(3213) = p(178)*p(185) - p(2828)
p(3214) = p(46)*p(566) - p(2832)
p(3215) = p(1)*p(2144) - p(2829) - p(3214)
p(3216) = p(1)*p(2145) - p(2832)
p(3217) = p(1)*p(2146) - p(2833) - p(3215)
p(3218) = p(21)*p(869) - p(2828)
p(3219) = p(1)*p(2148) - p(2836) - p(3217)
p(3220) = p(101)*p(299) - p(2825)
p(3221) = p(1)*p(2150) - p(2838) - p(3219)
p(3222) = p(84)*p(346) - p(2823)
p(3223) = p(6)*p(1483) - p(2807)
p(3224) = p(1)*p(2153) - p(2847) - p(2845)
p(3225) = p(8)*p(1369) - p(2807)
p(3226) = p(45)*p(570) - p(2821)
p(3227) = p(1)*p(2156) - p(2869) - p(2867)
p(3228) = p(2)*p(2140) - p(2821)
p(3229) = p(1)*p(2158) - p(2888) - p(2887)
p(3230) = p(1)*p(2159) - p(2892)
p(3231) = p(3)*p(1487) - p(2898)
p(3232) = p(39)*p(576) - p(2900)
p(3233) = p(178)*p(188) - p(2903)
p(3234) = p(1)*p(2162) - p(2901) - p(3233)
p(3235) = p(2)*p(2145) - p(2853)
p(3236) = p(1)*p(2163) - p(2904) - p(3234)
p(3237) = p(7)*p(1406) - p(2903)
p(3238) = p(1)*p(2165) - p(2907) - p(3236)
p(3239) = p(49)*p(536) - p(2900)
p(3240) = p(1)*p(2167) - p(2909) - p(3238)
p(3241) = p(173)*p(194) - p(2898)
p(3242) = p(6)*p(1485) - p(2860)
p(3243) = p(1)*p(2170) - p(2917) - p(2915)
p(3244) = p(22)*p(851) - p(2860)
p(3245) = p(45)*p(572) - p(2822)
p(3246) = p(1)*p(2173) - p(2936) - p(2934)
p(3247) = p(8)*p(1385) - p(2822)
p(3248) = p(181)*p(187) - p(2894)
p(3249) = p(1)*p(2176) - p(2955) - p(2953)
p(3250) = p(2)*p(2159) - p(2894)
p(3251) = p(1)*p(2178) - p(2972) - p(2971)
p(3252) = p(1)*p(2179) - p(2974)
p(3253) = p(14)*p(882)
p(3254) = p(3)*p(1489) - p(2981)
p(3255) = p(39)*p(579) - p(2983)
p(3256) = p(178)*p(191) - p(2986)
p(3257) = p(1)*p(2183) - p(2984) - p(3256)
p(3258) = p(1)*p(2184) - p(2986)
p(3259) = p(1)*p(2185) - p(2987) - p(3257)
p(3260) = p(21)*p(904) - p(2983)
p(3261) = p(1)*p(2187) - p(2989) - p(3259)
p(3262) = p(101)*p(323) - p(2981)
p(3263) = p(6)*p(1487) - p(2928)
p(3264) = p(1)*p(2190) - p(2996) - p(2994)
p(3265) = p(50)*p(505) - p(2928)
p(3266) = p(45)*p(574) - p(2883)
p(3267) = p(1)*p(2193) - p(3012) - p(3010)
p(3268) = p(22)*p(865) - p(2883)
p(3269) = p(183)*p(187) - p(2896)
p(3270) = p(1)*p(2196) - p(3028) - p(3026)
p(3271) = p(8)*p(1402) - p(2896)
p(3272) = p(41)*p(578) - p(2976)
p(3273) = p(1)*p(2200) - p(3044) - p(3042)
p(3274) = p(2)*p(2179) - p(2976)
p(3275) = p(14)*p(898)
p(3276) = p(4)*p(1441) - p(2683) - p(3103)
p(3277) = p(1)*p(2204) - p(3054)
p(3278) = p(14)*p(901)
p(3279) = p(3)*p(1492) - p(3057)
p(3280) = p(39)*p(582) - p(3059)
p(3281) = p(1)*p(2207) - p(3058) - p(3280)
p(3282) = p(2)*p(2184) - p(3000)
p(3283) = p(1)*p(2208) - p(3060) - p(3281)
p(3284) = p(7)*p(1446) - p(3059)
p(3285) = p(1)*p(2210) - p(3062) - p(3283)
p(3286) = p(49)*p(561) - p(3057)
p(3287) = p(6)*p(1489) - p(3005)
p(3288) = p(1)*p(2213) - p(3068) - p(3066)
p(3289) = p(102)*p(285) - p(3005)
p(3290) = p(45)*p(576) - p(2948)
p(3291) = p(1)*p(2216) - p(3081) - p(3079)
p(3292) = p(50)*p(518) - p(2948)
p(3293) = p(185)*p(187) - p(2897)
p(3294) = p(1)*p(2219) - p(3094) - p(3092)
p(3295) = p(22)*p(881) - p(2897)
p(3296) = p(14)*p(913)
p(3297) = p(43)*p(578) - p(2978)
p(3298) = p(1)*p(2223) - p(3105) - p(3103)
p(3299) = p(8)*p(1422) - p(2978)
p(3300) = p(14)*p(917)
p(3301) = p(4)*p(1491) - p(3056)
p(3302) = p(2)*p(2203) - p(3055) - p(3301)
p(3303) = p(2)*p(2204) - p(3056)
p(3304) = p(14)*p(948)
p(3305) = p(3)*p(1495) - p(3109)
p(3306) = p(39)*p(585) - p(3111)
p(3307) = p(1)*p(2227) - p(3110) - p(3306)
p(3308) = p(1)*p(2228) - p(3111)
p(3309) = p(1)*p(2229) - p(3112) - p(3307)
p(3310) = p(21)*p(929) - p(3109)
p(3311) = p(6)*p(1492) - p(3075)
p(3312) = p(1)*p(2232) - p(3117) - p(3115)
p(3313) = p(150)*p(195) - p(3075)
p(3314) = p(45)*p(579) - p(3022)
p(3315) = p(1)*p(2235) - p(3127) - p(3125)
p(3316) = p(102)*p(296) - p(3022)
p(3317) = p(187)*p(188) - p(2968)
p(3318) = p(1)*p(2239) - p(3137) - p(3135)
p(3319) = p(50)*p(532) - p(2968)
p(3320) = p(14)*p(927)
p(3321) = p(46)*p(578) - p(2980)
p(3322) = p(2)*p(2223) - p(3107) - p(3302)
p(3323) = p(22)*p(900) - p(2980)
p(3324) = p(14)*p(951)
p(3325) = p(3)*p(1498) - p(3141)
p(3326) = p(2)*p(2227) - p(3118) - p(3114)
p(3327) = p(2)*p(2228) - p(3119)
p(3328) = p(1)*p(2243) - p(3142) - p(3326)
p(3329) = p(7)*p(1471) - p(3141)
p(3330) = p(6)*p(1495) - p(3122)
p(3331) = p(1)*p(2246) - p(3146) - p(3144)
p(3332) = p(73)*p(347) - p(3122)
p(3333) = p(45)*p(582) - p(3089)
p(3334) = p(1)*p(2249) - p(3153) - p(3151)
p(3335) = p(160)*p(195) - p(3089)
p(3336) = p(14)*p(933)
p(3337) = p(187)*p(191) - p(3039)
p(3338) = p(2)*p(2239) - p(3139) - p(3322)
p(3339) = p(102)*p(310) - p(3039)
p(3340) = p(14)*p(954)
p(3341) = p(3)*p(1501) - p(3157)
p(3342) = p(2)*p(2243) - p(3147) - p(3144)
p(3343) = p(1)*p(2253) - p(3157)
p(3344) = p(6)*p(1498) - p(3149)
p(3345) = p(1)*p(2255) - p(3160) - p(3158)
p(3346) = p(32)*p(589) - p(3149)
p(3347) = p(45)*p(585) - p(3133)
p(3348) = p(2)*p(2249) - p(3155) - p(3338)
p(3349) = p(81)*p(347) - p(3133)
p(3350) = p(14)*p(957)
p(3351) = p(2)*p(2252) - p(3159) - p(3158)
p(3352) = p(2)*p(2253) - p(3160)
p(3353) = p(6)*p(1501) - p(3161)
p(3354) = p(2)*p(2255) - p(3162) - p(3348)
p(3355) = p(13)*p(959) - p(3161)
p(3356) = p(14)*p(959)
p(3357) = p(3)*p(1476) - p(3168)
p(3358) = p(7)*p(1476) - p(3165)
p(3359) = p(1)*p(2259) - p(3170) - p(3358)
p(3360) = p(178)*p(194) - p(3166)
p(3361) = p(39)*p(588) - p(3171)
p(3362) = p(3)*p(1504) - p(3174)
p(3363) = p(1)*p(2263) - p(3175)
p(3364) = p(2)*p(2288) - p(3363)
p(3365) = p(1)*p(2265) - p(3192)
p(3366) = p(8)*p(1504) - p(3365)
p(3367) = p(1)*p(2267) - p(3211)
p(3368) = p(22)*p(958) - p(3367)
p(3369) = p(1)*p(2269) - p(3231)
p(3370) = p(50)*p(588) - p(3369)
p(3371) = p(1)*p(2271) - p(3254)
p(3372) = p(102)*p(346) - p(3371)
p(3373) = p(1)*p(2273) - p(3279)
p(3374) = p(194)*p(195) - p(3373)
p(3375) = p(6)*p(1491) - p(3304)
p(3376) = p(1)*p(2276) - p(3305)
p(3377) = p(101)*p(347) - p(3376)
p(3378) = p(8)*p(1491) - p(3278)
p(3379) = p(1)*p(2279) - p(3325)
p(3380) = p(49)*p(589) - p(3379)
p(3381) = p(2)*p(2278) - p(3324) - p(3378)
p(3382) = p(1)*p(2282) - p(3341)
p(3383) = p(21)*p(959) - p(3382)
p(3384) = p(187)*p(195) - p(3296)
p(3385) = p(1)*p(2285) - p(3351)
p(3386) = p(7)*p(1505) - p(3385)
p(3387) = p(45)*p(589) - p(3336)
p(3388) = p(2)*p(2285) - p(3353)
p(3389) = p(1)*p(2289) - p(3388)
p(3390) = p(6)*p(1505) - p(3356)
p(3391) = p(1)*p(2288) - p(3362)
p(3392) = p(2)*p(2289) - p(3390)

do j=1,3393
  bm1(j)=p(j-1)
enddo
 
bm2(1)=bm1(4)
bm2(2)=bm1(10)
bm2(3)=bm1(11)
bm2(4)=bm1(12)
bm2(5)=bm1(13)
bm2(6)=bm1(15)
bm2(7)=bm1(16)
bm2(8)=bm1(24)
bm2(9)=bm1(25)
bm2(10)=bm1(26)
bm2(11)=bm1(27)
bm2(12)=bm1(28)
bm2(13)=bm1(29)
bm2(14)=bm1(30)
bm2(15)=bm1(31)
bm2(16)=bm1(32)
bm2(17)=bm1(34)
bm2(18)=bm1(35)
bm2(19)=bm1(36)
bm2(20)=bm1(37)
bm2(21)=bm1(39)
bm2(22)=bm1(40)
bm2(23)=bm1(41)
bm2(24)=bm1(52)
bm2(25)=bm1(53)
bm2(26)=bm1(54)
bm2(27)=bm1(55)
bm2(28)=bm1(56)
bm2(29)=bm1(57)
bm2(30)=bm1(58)
bm2(31)=bm1(59)
bm2(32)=bm1(60)
bm2(33)=bm1(61)
bm2(34)=bm1(62)
bm2(35)=bm1(63)
bm2(36)=bm1(64)
bm2(37)=bm1(65)
bm2(38)=bm1(66)
bm2(39)=bm1(67)
bm2(40)=bm1(68)
bm2(41)=bm1(69)
bm2(42)=bm1(70)
bm2(43)=bm1(71)
bm2(44)=bm1(72)
bm2(45)=bm1(73)
bm2(46)=bm1(75)
bm2(47)=bm1(76)
bm2(48)=bm1(77)
bm2(49)=bm1(78)
bm2(50)=bm1(79)
bm2(51)=bm1(81)
bm2(52)=bm1(83)
bm2(53)=bm1(84)
bm2(54)=bm1(85)
bm2(55)=bm1(86)
bm2(56)=bm1(87)
bm2(57)=bm1(89)
bm2(58)=bm1(90)
bm2(59)=bm1(91)
bm2(60)=bm1(104)
bm2(61)=bm1(105)
bm2(62)=bm1(106)
bm2(63)=bm1(107)
bm2(64)=bm1(108)
bm2(65)=bm1(109)
bm2(66)=bm1(110)
bm2(67)=bm1(111)
bm2(68)=bm1(112)
bm2(69)=bm1(113)
bm2(70)=bm1(114)
bm2(71)=bm1(115)
bm2(72)=bm1(116)
bm2(73)=bm1(117)
bm2(74)=bm1(118)
bm2(75)=bm1(119)
bm2(76)=bm1(120)
bm2(77)=bm1(121)
bm2(78)=bm1(122)
bm2(79)=bm1(123)
bm2(80)=bm1(124)
bm2(81)=bm1(125)
bm2(82)=bm1(126)
bm2(83)=bm1(127)
bm2(84)=bm1(128)
bm2(85)=bm1(129)
bm2(86)=bm1(130)
bm2(87)=bm1(131)
bm2(88)=bm1(132)
bm2(89)=bm1(133)
bm2(90)=bm1(134)
bm2(91)=bm1(135)
bm2(92)=bm1(136)
bm2(93)=bm1(137)
bm2(94)=bm1(138)
bm2(95)=bm1(139)
bm2(96)=bm1(140)
bm2(97)=bm1(141)
bm2(98)=bm1(142)
bm2(99)=bm1(143)
bm2(100)=bm1(144)
bm2(101)=bm1(145)
bm2(102)=bm1(146)
bm2(103)=bm1(147)
bm2(104)=bm1(148)
bm2(105)=bm1(149)
bm2(106)=bm1(150)
bm2(107)=bm1(152)
bm2(108)=bm1(153)
bm2(109)=bm1(154)
bm2(110)=bm1(155)
bm2(111)=bm1(156)
bm2(112)=bm1(157)
bm2(113)=bm1(158)
bm2(114)=bm1(160)
bm2(115)=bm1(162)
bm2(116)=bm1(163)
bm2(117)=bm1(164)
bm2(118)=bm1(165)
bm2(119)=bm1(166)
bm2(120)=bm1(167)
bm2(121)=bm1(169)
bm2(122)=bm1(170)
bm2(123)=bm1(172)
bm2(124)=bm1(173)
bm2(125)=bm1(174)
bm2(126)=bm1(175)
bm2(127)=bm1(176)
bm2(128)=bm1(178)
bm2(129)=bm1(179)
bm2(130)=bm1(180)
bm2(131)=bm1(181)
bm2(132)=bm1(197)
bm2(133)=bm1(198)
bm2(134)=bm1(199)
bm2(135)=bm1(200)
bm2(136)=bm1(201)
bm2(137)=bm1(202)
bm2(138)=bm1(203)
bm2(139)=bm1(204)
bm2(140)=bm1(205)
bm2(141)=bm1(206)
bm2(142)=bm1(207)
bm2(143)=bm1(208)
bm2(144)=bm1(209)
bm2(145)=bm1(210)
bm2(146)=bm1(211)
bm2(147)=bm1(212)
bm2(148)=bm1(213)
bm2(149)=bm1(214)
bm2(150)=bm1(215)
bm2(151)=bm1(216)
bm2(152)=bm1(217)
bm2(153)=bm1(218)
bm2(154)=bm1(219)
bm2(155)=bm1(220)
bm2(156)=bm1(221)
bm2(157)=bm1(222)
bm2(158)=bm1(223)
bm2(159)=bm1(224)
bm2(160)=bm1(225)
bm2(161)=bm1(226)
bm2(162)=bm1(227)
bm2(163)=bm1(228)
bm2(164)=bm1(229)
bm2(165)=bm1(230)
bm2(166)=bm1(231)
bm2(167)=bm1(232)
bm2(168)=bm1(233)
bm2(169)=bm1(234)
bm2(170)=bm1(235)
bm2(171)=bm1(236)
bm2(172)=bm1(237)
bm2(173)=bm1(238)
bm2(174)=bm1(239)
bm2(175)=bm1(240)
bm2(176)=bm1(241)
bm2(177)=bm1(242)
bm2(178)=bm1(243)
bm2(179)=bm1(244)
bm2(180)=bm1(245)
bm2(181)=bm1(246)
bm2(182)=bm1(247)
bm2(183)=bm1(248)
bm2(184)=bm1(249)
bm2(185)=bm1(250)
bm2(186)=bm1(251)
bm2(187)=bm1(252)
bm2(188)=bm1(253)
bm2(189)=bm1(254)
bm2(190)=bm1(255)
bm2(191)=bm1(256)
bm2(192)=bm1(257)
bm2(193)=bm1(258)
bm2(194)=bm1(259)
bm2(195)=bm1(260)
bm2(196)=bm1(261)
bm2(197)=bm1(262)
bm2(198)=bm1(263)
bm2(199)=bm1(264)
bm2(200)=bm1(265)
bm2(201)=bm1(266)
bm2(202)=bm1(267)
bm2(203)=bm1(268)
bm2(204)=bm1(269)
bm2(205)=bm1(270)
bm2(206)=bm1(271)
bm2(207)=bm1(272)
bm2(208)=bm1(273)
bm2(209)=bm1(274)
bm2(210)=bm1(275)
bm2(211)=bm1(276)
bm2(212)=bm1(277)
bm2(213)=bm1(278)
bm2(214)=bm1(279)
bm2(215)=bm1(280)
bm2(216)=bm1(281)
bm2(217)=bm1(282)
bm2(218)=bm1(283)
bm2(219)=bm1(284)
bm2(220)=bm1(285)
bm2(221)=bm1(287)
bm2(222)=bm1(288)
bm2(223)=bm1(289)
bm2(224)=bm1(290)
bm2(225)=bm1(291)
bm2(226)=bm1(292)
bm2(227)=bm1(293)
bm2(228)=bm1(294)
bm2(229)=bm1(296)
bm2(230)=bm1(298)
bm2(231)=bm1(299)
bm2(232)=bm1(300)
bm2(233)=bm1(301)
bm2(234)=bm1(302)
bm2(235)=bm1(303)
bm2(236)=bm1(304)
bm2(237)=bm1(306)
bm2(238)=bm1(307)
bm2(239)=bm1(309)
bm2(240)=bm1(310)
bm2(241)=bm1(312)
bm2(242)=bm1(313)
bm2(243)=bm1(314)
bm2(244)=bm1(315)
bm2(245)=bm1(316)
bm2(246)=bm1(317)
bm2(247)=bm1(319)
bm2(248)=bm1(320)
bm2(249)=bm1(322)
bm2(250)=bm1(323)
bm2(251)=bm1(324)
bm2(252)=bm1(325)
bm2(253)=bm1(326)
bm2(254)=bm1(328)
bm2(255)=bm1(329)
bm2(256)=bm1(330)
bm2(257)=bm1(331)
bm2(258)=bm1(349)
bm2(259)=bm1(350)
bm2(260)=bm1(351)
bm2(261)=bm1(352)
bm2(262)=bm1(353)
bm2(263)=bm1(354)
bm2(264)=bm1(355)
bm2(265)=bm1(356)
bm2(266)=bm1(357)
bm2(267)=bm1(358)
bm2(268)=bm1(359)
bm2(269)=bm1(360)
bm2(270)=bm1(361)
bm2(271)=bm1(362)
bm2(272)=bm1(363)
bm2(273)=bm1(364)
bm2(274)=bm1(365)
bm2(275)=bm1(366)
bm2(276)=bm1(367)
bm2(277)=bm1(368)
bm2(278)=bm1(369)
bm2(279)=bm1(370)
bm2(280)=bm1(371)
bm2(281)=bm1(372)
bm2(282)=bm1(373)
bm2(283)=bm1(374)
bm2(284)=bm1(375)
bm2(285)=bm1(376)
bm2(286)=bm1(377)
bm2(287)=bm1(378)
bm2(288)=bm1(379)
bm2(289)=bm1(380)
bm2(290)=bm1(381)
bm2(291)=bm1(382)
bm2(292)=bm1(383)
bm2(293)=bm1(384)
bm2(294)=bm1(385)
bm2(295)=bm1(386)
bm2(296)=bm1(387)
bm2(297)=bm1(388)
bm2(298)=bm1(389)
bm2(299)=bm1(390)
bm2(300)=bm1(391)
bm2(301)=bm1(392)
bm2(302)=bm1(393)
bm2(303)=bm1(394)
bm2(304)=bm1(395)
bm2(305)=bm1(396)
bm2(306)=bm1(397)
bm2(307)=bm1(398)
bm2(308)=bm1(399)
bm2(309)=bm1(400)
bm2(310)=bm1(401)
bm2(311)=bm1(402)
bm2(312)=bm1(403)
bm2(313)=bm1(404)
bm2(314)=bm1(405)
bm2(315)=bm1(406)
bm2(316)=bm1(407)
bm2(317)=bm1(408)
bm2(318)=bm1(409)
bm2(319)=bm1(410)
bm2(320)=bm1(411)
bm2(321)=bm1(412)
bm2(322)=bm1(413)
bm2(323)=bm1(414)
bm2(324)=bm1(415)
bm2(325)=bm1(416)
bm2(326)=bm1(417)
bm2(327)=bm1(418)
bm2(328)=bm1(419)
bm2(329)=bm1(420)
bm2(330)=bm1(421)
bm2(331)=bm1(422)
bm2(332)=bm1(423)
bm2(333)=bm1(424)
bm2(334)=bm1(425)
bm2(335)=bm1(426)
bm2(336)=bm1(427)
bm2(337)=bm1(428)
bm2(338)=bm1(429)
bm2(339)=bm1(430)
bm2(340)=bm1(431)
bm2(341)=bm1(432)
bm2(342)=bm1(433)
bm2(343)=bm1(434)
bm2(344)=bm1(435)
bm2(345)=bm1(436)
bm2(346)=bm1(437)
bm2(347)=bm1(438)
bm2(348)=bm1(439)
bm2(349)=bm1(440)
bm2(350)=bm1(441)
bm2(351)=bm1(442)
bm2(352)=bm1(443)
bm2(353)=bm1(444)
bm2(354)=bm1(445)
bm2(355)=bm1(446)
bm2(356)=bm1(447)
bm2(357)=bm1(448)
bm2(358)=bm1(449)
bm2(359)=bm1(450)
bm2(360)=bm1(451)
bm2(361)=bm1(452)
bm2(362)=bm1(453)
bm2(363)=bm1(454)
bm2(364)=bm1(455)
bm2(365)=bm1(456)
bm2(366)=bm1(457)
bm2(367)=bm1(458)
bm2(368)=bm1(459)
bm2(369)=bm1(460)
bm2(370)=bm1(461)
bm2(371)=bm1(462)
bm2(372)=bm1(463)
bm2(373)=bm1(464)
bm2(374)=bm1(465)
bm2(375)=bm1(466)
bm2(376)=bm1(467)
bm2(377)=bm1(468)
bm2(378)=bm1(469)
bm2(379)=bm1(470)
bm2(380)=bm1(471)
bm2(381)=bm1(472)
bm2(382)=bm1(473)
bm2(383)=bm1(474)
bm2(384)=bm1(475)
bm2(385)=bm1(476)
bm2(386)=bm1(477)
bm2(387)=bm1(478)
bm2(388)=bm1(479)
bm2(389)=bm1(480)
bm2(390)=bm1(481)
bm2(391)=bm1(482)
bm2(392)=bm1(483)
bm2(393)=bm1(484)
bm2(394)=bm1(485)
bm2(395)=bm1(486)
bm2(396)=bm1(487)
bm2(397)=bm1(488)
bm2(398)=bm1(489)
bm2(399)=bm1(490)
bm2(400)=bm1(491)
bm2(401)=bm1(492)
bm2(402)=bm1(493)
bm2(403)=bm1(494)
bm2(404)=bm1(495)
bm2(405)=bm1(496)
bm2(406)=bm1(497)
bm2(407)=bm1(498)
bm2(408)=bm1(499)
bm2(409)=bm1(500)
bm2(410)=bm1(501)
bm2(411)=bm1(502)
bm2(412)=bm1(503)
bm2(413)=bm1(504)
bm2(414)=bm1(505)
bm2(415)=bm1(507)
bm2(416)=bm1(508)
bm2(417)=bm1(509)
bm2(418)=bm1(510)
bm2(419)=bm1(511)
bm2(420)=bm1(512)
bm2(421)=bm1(513)
bm2(422)=bm1(514)
bm2(423)=bm1(515)
bm2(424)=bm1(516)
bm2(425)=bm1(518)
bm2(426)=bm1(520)
bm2(427)=bm1(521)
bm2(428)=bm1(522)
bm2(429)=bm1(523)
bm2(430)=bm1(524)
bm2(431)=bm1(525)
bm2(432)=bm1(526)
bm2(433)=bm1(527)
bm2(434)=bm1(529)
bm2(435)=bm1(530)
bm2(436)=bm1(532)
bm2(437)=bm1(534)
bm2(438)=bm1(535)
bm2(439)=bm1(536)
bm2(440)=bm1(537)
bm2(441)=bm1(538)
bm2(442)=bm1(539)
bm2(443)=bm1(540)
bm2(444)=bm1(541)
bm2(445)=bm1(543)
bm2(446)=bm1(544)
bm2(447)=bm1(546)
bm2(448)=bm1(547)
bm2(449)=bm1(548)
bm2(450)=bm1(550)
bm2(451)=bm1(551)
bm2(452)=bm1(552)
bm2(453)=bm1(553)
bm2(454)=bm1(554)
bm2(455)=bm1(555)
bm2(456)=bm1(557)
bm2(457)=bm1(558)
bm2(458)=bm1(560)
bm2(459)=bm1(561)
bm2(460)=bm1(562)
bm2(461)=bm1(563)
bm2(462)=bm1(564)
bm2(463)=bm1(566)
bm2(464)=bm1(567)
bm2(465)=bm1(568)
bm2(466)=bm1(569)
bm2(467)=bm1(570)
bm2(468)=bm1(591)
bm2(469)=bm1(592)
bm2(470)=bm1(593)
bm2(471)=bm1(594)
bm2(472)=bm1(595)
bm2(473)=bm1(596)
bm2(474)=bm1(597)
bm2(475)=bm1(598)
bm2(476)=bm1(599)
bm2(477)=bm1(600)
bm2(478)=bm1(601)
bm2(479)=bm1(602)
bm2(480)=bm1(603)
bm2(481)=bm1(604)
bm2(482)=bm1(605)
bm2(483)=bm1(606)
bm2(484)=bm1(607)
bm2(485)=bm1(608)
bm2(486)=bm1(609)
bm2(487)=bm1(610)
bm2(488)=bm1(611)
bm2(489)=bm1(612)
bm2(490)=bm1(613)
bm2(491)=bm1(614)
bm2(492)=bm1(615)
bm2(493)=bm1(616)
bm2(494)=bm1(617)
bm2(495)=bm1(618)
bm2(496)=bm1(619)
bm2(497)=bm1(620)
bm2(498)=bm1(621)
bm2(499)=bm1(622)
bm2(500)=bm1(623)
bm2(501)=bm1(624)
bm2(502)=bm1(625)
bm2(503)=bm1(626)
bm2(504)=bm1(627)
bm2(505)=bm1(628)
bm2(506)=bm1(629)
bm2(507)=bm1(630)
bm2(508)=bm1(631)
bm2(509)=bm1(632)
bm2(510)=bm1(633)
bm2(511)=bm1(634)
bm2(512)=bm1(635)
bm2(513)=bm1(636)
bm2(514)=bm1(637)
bm2(515)=bm1(638)
bm2(516)=bm1(639)
bm2(517)=bm1(640)
bm2(518)=bm1(641)
bm2(519)=bm1(642)
bm2(520)=bm1(643)
bm2(521)=bm1(644)
bm2(522)=bm1(645)
bm2(523)=bm1(646)
bm2(524)=bm1(647)
bm2(525)=bm1(648)
bm2(526)=bm1(649)
bm2(527)=bm1(650)
bm2(528)=bm1(651)
bm2(529)=bm1(652)
bm2(530)=bm1(653)
bm2(531)=bm1(654)
bm2(532)=bm1(655)
bm2(533)=bm1(656)
bm2(534)=bm1(657)
bm2(535)=bm1(658)
bm2(536)=bm1(659)
bm2(537)=bm1(660)
bm2(538)=bm1(661)
bm2(539)=bm1(662)
bm2(540)=bm1(663)
bm2(541)=bm1(664)
bm2(542)=bm1(665)
bm2(543)=bm1(666)
bm2(544)=bm1(667)
bm2(545)=bm1(668)
bm2(546)=bm1(669)
bm2(547)=bm1(670)
bm2(548)=bm1(671)
bm2(549)=bm1(672)
bm2(550)=bm1(673)
bm2(551)=bm1(674)
bm2(552)=bm1(675)
bm2(553)=bm1(676)
bm2(554)=bm1(677)
bm2(555)=bm1(678)
bm2(556)=bm1(679)
bm2(557)=bm1(680)
bm2(558)=bm1(681)
bm2(559)=bm1(682)
bm2(560)=bm1(683)
bm2(561)=bm1(684)
bm2(562)=bm1(685)
bm2(563)=bm1(686)
bm2(564)=bm1(687)
bm2(565)=bm1(688)
bm2(566)=bm1(689)
bm2(567)=bm1(690)
bm2(568)=bm1(691)
bm2(569)=bm1(692)
bm2(570)=bm1(693)
bm2(571)=bm1(694)
bm2(572)=bm1(695)
bm2(573)=bm1(696)
bm2(574)=bm1(697)
bm2(575)=bm1(698)
bm2(576)=bm1(699)
bm2(577)=bm1(700)
bm2(578)=bm1(701)
bm2(579)=bm1(702)
bm2(580)=bm1(703)
bm2(581)=bm1(704)
bm2(582)=bm1(705)
bm2(583)=bm1(706)
bm2(584)=bm1(707)
bm2(585)=bm1(708)
bm2(586)=bm1(709)
bm2(587)=bm1(710)
bm2(588)=bm1(711)
bm2(589)=bm1(712)
bm2(590)=bm1(713)
bm2(591)=bm1(714)
bm2(592)=bm1(715)
bm2(593)=bm1(716)
bm2(594)=bm1(717)
bm2(595)=bm1(718)
bm2(596)=bm1(719)
bm2(597)=bm1(720)
bm2(598)=bm1(721)
bm2(599)=bm1(722)
bm2(600)=bm1(723)
bm2(601)=bm1(724)
bm2(602)=bm1(725)
bm2(603)=bm1(726)
bm2(604)=bm1(727)
bm2(605)=bm1(728)
bm2(606)=bm1(729)
bm2(607)=bm1(730)
bm2(608)=bm1(731)
bm2(609)=bm1(732)
bm2(610)=bm1(733)
bm2(611)=bm1(734)
bm2(612)=bm1(735)
bm2(613)=bm1(736)
bm2(614)=bm1(737)
bm2(615)=bm1(738)
bm2(616)=bm1(739)
bm2(617)=bm1(740)
bm2(618)=bm1(741)
bm2(619)=bm1(742)
bm2(620)=bm1(743)
bm2(621)=bm1(744)
bm2(622)=bm1(745)
bm2(623)=bm1(746)
bm2(624)=bm1(747)
bm2(625)=bm1(748)
bm2(626)=bm1(749)
bm2(627)=bm1(750)
bm2(628)=bm1(751)
bm2(629)=bm1(752)
bm2(630)=bm1(753)
bm2(631)=bm1(754)
bm2(632)=bm1(755)
bm2(633)=bm1(756)
bm2(634)=bm1(757)
bm2(635)=bm1(758)
bm2(636)=bm1(759)
bm2(637)=bm1(760)
bm2(638)=bm1(761)
bm2(639)=bm1(762)
bm2(640)=bm1(763)
bm2(641)=bm1(764)
bm2(642)=bm1(765)
bm2(643)=bm1(766)
bm2(644)=bm1(767)
bm2(645)=bm1(768)
bm2(646)=bm1(769)
bm2(647)=bm1(770)
bm2(648)=bm1(771)
bm2(649)=bm1(772)
bm2(650)=bm1(773)
bm2(651)=bm1(774)
bm2(652)=bm1(775)
bm2(653)=bm1(776)
bm2(654)=bm1(777)
bm2(655)=bm1(778)
bm2(656)=bm1(779)
bm2(657)=bm1(780)
bm2(658)=bm1(781)
bm2(659)=bm1(782)
bm2(660)=bm1(783)
bm2(661)=bm1(784)
bm2(662)=bm1(785)
bm2(663)=bm1(786)
bm2(664)=bm1(787)
bm2(665)=bm1(788)
bm2(666)=bm1(789)
bm2(667)=bm1(790)
bm2(668)=bm1(791)
bm2(669)=bm1(792)
bm2(670)=bm1(793)
bm2(671)=bm1(794)
bm2(672)=bm1(795)
bm2(673)=bm1(796)
bm2(674)=bm1(797)
bm2(675)=bm1(798)
bm2(676)=bm1(799)
bm2(677)=bm1(800)
bm2(678)=bm1(801)
bm2(679)=bm1(802)
bm2(680)=bm1(803)
bm2(681)=bm1(804)
bm2(682)=bm1(805)
bm2(683)=bm1(806)
bm2(684)=bm1(807)
bm2(685)=bm1(808)
bm2(686)=bm1(809)
bm2(687)=bm1(810)
bm2(688)=bm1(811)
bm2(689)=bm1(812)
bm2(690)=bm1(813)
bm2(691)=bm1(814)
bm2(692)=bm1(815)
bm2(693)=bm1(816)
bm2(694)=bm1(817)
bm2(695)=bm1(818)
bm2(696)=bm1(819)
bm2(697)=bm1(820)
bm2(698)=bm1(821)
bm2(699)=bm1(822)
bm2(700)=bm1(823)
bm2(701)=bm1(824)
bm2(702)=bm1(825)
bm2(703)=bm1(826)
bm2(704)=bm1(827)
bm2(705)=bm1(828)
bm2(706)=bm1(829)
bm2(707)=bm1(830)
bm2(708)=bm1(831)
bm2(709)=bm1(832)
bm2(710)=bm1(833)
bm2(711)=bm1(834)
bm2(712)=bm1(835)
bm2(713)=bm1(836)
bm2(714)=bm1(837)
bm2(715)=bm1(838)
bm2(716)=bm1(839)
bm2(717)=bm1(840)
bm2(718)=bm1(841)
bm2(719)=bm1(842)
bm2(720)=bm1(843)
bm2(721)=bm1(844)
bm2(722)=bm1(845)
bm2(723)=bm1(846)
bm2(724)=bm1(847)
bm2(725)=bm1(848)
bm2(726)=bm1(849)
bm2(727)=bm1(850)
bm2(728)=bm1(851)
bm2(729)=bm1(853)
bm2(730)=bm1(854)
bm2(731)=bm1(855)
bm2(732)=bm1(856)
bm2(733)=bm1(857)
bm2(734)=bm1(858)
bm2(735)=bm1(859)
bm2(736)=bm1(860)
bm2(737)=bm1(861)
bm2(738)=bm1(862)
bm2(739)=bm1(863)
bm2(740)=bm1(865)
bm2(741)=bm1(867)
bm2(742)=bm1(868)
bm2(743)=bm1(869)
bm2(744)=bm1(870)
bm2(745)=bm1(871)
bm2(746)=bm1(872)
bm2(747)=bm1(873)
bm2(748)=bm1(874)
bm2(749)=bm1(875)
bm2(750)=bm1(876)
bm2(751)=bm1(878)
bm2(752)=bm1(879)
bm2(753)=bm1(881)
bm2(754)=bm1(883)
bm2(755)=bm1(884)
bm2(756)=bm1(885)
bm2(757)=bm1(886)
bm2(758)=bm1(887)
bm2(759)=bm1(888)
bm2(760)=bm1(889)
bm2(761)=bm1(890)
bm2(762)=bm1(891)
bm2(763)=bm1(893)
bm2(764)=bm1(894)
bm2(765)=bm1(896)
bm2(766)=bm1(897)
bm2(767)=bm1(899)
bm2(768)=bm1(900)
bm2(769)=bm1(902)
bm2(770)=bm1(903)
bm2(771)=bm1(904)
bm2(772)=bm1(905)
bm2(773)=bm1(906)
bm2(774)=bm1(907)
bm2(775)=bm1(908)
bm2(776)=bm1(909)
bm2(777)=bm1(911)
bm2(778)=bm1(912)
bm2(779)=bm1(914)
bm2(780)=bm1(915)
bm2(781)=bm1(916)
bm2(782)=bm1(918)
bm2(783)=bm1(919)
bm2(784)=bm1(920)
bm2(785)=bm1(921)
bm2(786)=bm1(922)
bm2(787)=bm1(923)
bm2(788)=bm1(925)
bm2(789)=bm1(926)
bm2(790)=bm1(928)
bm2(791)=bm1(929)
bm2(792)=bm1(930)
bm2(793)=bm1(931)
bm2(794)=bm1(932)
bm2(795)=bm1(934)
bm2(796)=bm1(935)
bm2(797)=bm1(936)
bm2(798)=bm1(937)
bm2(799)=bm1(938)
bm2(800)=bm1(961)
bm2(801)=bm1(962)
bm2(802)=bm1(963)
bm2(803)=bm1(964)
bm2(804)=bm1(965)
bm2(805)=bm1(966)
bm2(806)=bm1(967)
bm2(807)=bm1(968)
bm2(808)=bm1(969)
bm2(809)=bm1(970)
bm2(810)=bm1(971)
bm2(811)=bm1(972)
bm2(812)=bm1(973)
bm2(813)=bm1(974)
bm2(814)=bm1(975)
bm2(815)=bm1(976)
bm2(816)=bm1(977)
bm2(817)=bm1(978)
bm2(818)=bm1(979)
bm2(819)=bm1(980)
bm2(820)=bm1(981)
bm2(821)=bm1(982)
bm2(822)=bm1(983)
bm2(823)=bm1(984)
bm2(824)=bm1(985)
bm2(825)=bm1(986)
bm2(826)=bm1(987)
bm2(827)=bm1(988)
bm2(828)=bm1(989)
bm2(829)=bm1(990)
bm2(830)=bm1(991)
bm2(831)=bm1(992)
bm2(832)=bm1(993)
bm2(833)=bm1(994)
bm2(834)=bm1(995)
bm2(835)=bm1(996)
bm2(836)=bm1(997)
bm2(837)=bm1(998)
bm2(838)=bm1(999)
bm2(839)=bm1(1000)
bm2(840)=bm1(1001)
bm2(841)=bm1(1002)
bm2(842)=bm1(1003)
bm2(843)=bm1(1004)
bm2(844)=bm1(1005)
bm2(845)=bm1(1006)
bm2(846)=bm1(1007)
bm2(847)=bm1(1008)
bm2(848)=bm1(1009)
bm2(849)=bm1(1010)
bm2(850)=bm1(1011)
bm2(851)=bm1(1012)
bm2(852)=bm1(1013)
bm2(853)=bm1(1014)
bm2(854)=bm1(1015)
bm2(855)=bm1(1016)
bm2(856)=bm1(1017)
bm2(857)=bm1(1018)
bm2(858)=bm1(1019)
bm2(859)=bm1(1020)
bm2(860)=bm1(1021)
bm2(861)=bm1(1022)
bm2(862)=bm1(1023)
bm2(863)=bm1(1024)
bm2(864)=bm1(1025)
bm2(865)=bm1(1026)
bm2(866)=bm1(1027)
bm2(867)=bm1(1028)
bm2(868)=bm1(1029)
bm2(869)=bm1(1030)
bm2(870)=bm1(1031)
bm2(871)=bm1(1032)
bm2(872)=bm1(1033)
bm2(873)=bm1(1034)
bm2(874)=bm1(1035)
bm2(875)=bm1(1036)
bm2(876)=bm1(1037)
bm2(877)=bm1(1038)
bm2(878)=bm1(1039)
bm2(879)=bm1(1040)
bm2(880)=bm1(1041)
bm2(881)=bm1(1042)
bm2(882)=bm1(1043)
bm2(883)=bm1(1044)
bm2(884)=bm1(1045)
bm2(885)=bm1(1046)
bm2(886)=bm1(1047)
bm2(887)=bm1(1048)
bm2(888)=bm1(1049)
bm2(889)=bm1(1050)
bm2(890)=bm1(1051)
bm2(891)=bm1(1052)
bm2(892)=bm1(1053)
bm2(893)=bm1(1054)
bm2(894)=bm1(1055)
bm2(895)=bm1(1056)
bm2(896)=bm1(1057)
bm2(897)=bm1(1058)
bm2(898)=bm1(1059)
bm2(899)=bm1(1060)
bm2(900)=bm1(1061)
bm2(901)=bm1(1062)
bm2(902)=bm1(1063)
bm2(903)=bm1(1064)
bm2(904)=bm1(1065)
bm2(905)=bm1(1066)
bm2(906)=bm1(1067)
bm2(907)=bm1(1068)
bm2(908)=bm1(1069)
bm2(909)=bm1(1070)
bm2(910)=bm1(1071)
bm2(911)=bm1(1072)
bm2(912)=bm1(1073)
bm2(913)=bm1(1074)
bm2(914)=bm1(1075)
bm2(915)=bm1(1076)
bm2(916)=bm1(1077)
bm2(917)=bm1(1078)
bm2(918)=bm1(1079)
bm2(919)=bm1(1080)
bm2(920)=bm1(1081)
bm2(921)=bm1(1082)
bm2(922)=bm1(1083)
bm2(923)=bm1(1084)
bm2(924)=bm1(1085)
bm2(925)=bm1(1086)
bm2(926)=bm1(1087)
bm2(927)=bm1(1088)
bm2(928)=bm1(1089)
bm2(929)=bm1(1090)
bm2(930)=bm1(1091)
bm2(931)=bm1(1092)
bm2(932)=bm1(1093)
bm2(933)=bm1(1094)
bm2(934)=bm1(1095)
bm2(935)=bm1(1096)
bm2(936)=bm1(1097)
bm2(937)=bm1(1098)
bm2(938)=bm1(1099)
bm2(939)=bm1(1100)
bm2(940)=bm1(1101)
bm2(941)=bm1(1102)
bm2(942)=bm1(1103)
bm2(943)=bm1(1104)
bm2(944)=bm1(1105)
bm2(945)=bm1(1106)
bm2(946)=bm1(1107)
bm2(947)=bm1(1108)
bm2(948)=bm1(1109)
bm2(949)=bm1(1110)
bm2(950)=bm1(1111)
bm2(951)=bm1(1112)
bm2(952)=bm1(1113)
bm2(953)=bm1(1114)
bm2(954)=bm1(1115)
bm2(955)=bm1(1116)
bm2(956)=bm1(1117)
bm2(957)=bm1(1118)
bm2(958)=bm1(1119)
bm2(959)=bm1(1120)
bm2(960)=bm1(1121)
bm2(961)=bm1(1122)
bm2(962)=bm1(1123)
bm2(963)=bm1(1124)
bm2(964)=bm1(1125)
bm2(965)=bm1(1126)
bm2(966)=bm1(1127)
bm2(967)=bm1(1128)
bm2(968)=bm1(1129)
bm2(969)=bm1(1130)
bm2(970)=bm1(1131)
bm2(971)=bm1(1132)
bm2(972)=bm1(1133)
bm2(973)=bm1(1134)
bm2(974)=bm1(1135)
bm2(975)=bm1(1136)
bm2(976)=bm1(1137)
bm2(977)=bm1(1138)
bm2(978)=bm1(1139)
bm2(979)=bm1(1140)
bm2(980)=bm1(1141)
bm2(981)=bm1(1142)
bm2(982)=bm1(1143)
bm2(983)=bm1(1144)
bm2(984)=bm1(1145)
bm2(985)=bm1(1146)
bm2(986)=bm1(1147)
bm2(987)=bm1(1148)
bm2(988)=bm1(1149)
bm2(989)=bm1(1150)
bm2(990)=bm1(1151)
bm2(991)=bm1(1152)
bm2(992)=bm1(1153)
bm2(993)=bm1(1154)
bm2(994)=bm1(1155)
bm2(995)=bm1(1156)
bm2(996)=bm1(1157)
bm2(997)=bm1(1158)
bm2(998)=bm1(1159)
bm2(999)=bm1(1160)
bm2(1000)=bm1(1161)
bm2(1001)=bm1(1162)
bm2(1002)=bm1(1163)
bm2(1003)=bm1(1164)
bm2(1004)=bm1(1165)
bm2(1005)=bm1(1166)
bm2(1006)=bm1(1167)
bm2(1007)=bm1(1168)
bm2(1008)=bm1(1169)
bm2(1009)=bm1(1170)
bm2(1010)=bm1(1171)
bm2(1011)=bm1(1172)
bm2(1012)=bm1(1173)
bm2(1013)=bm1(1174)
bm2(1014)=bm1(1175)
bm2(1015)=bm1(1176)
bm2(1016)=bm1(1177)
bm2(1017)=bm1(1178)
bm2(1018)=bm1(1179)
bm2(1019)=bm1(1180)
bm2(1020)=bm1(1181)
bm2(1021)=bm1(1182)
bm2(1022)=bm1(1183)
bm2(1023)=bm1(1184)
bm2(1024)=bm1(1185)
bm2(1025)=bm1(1186)
bm2(1026)=bm1(1187)
bm2(1027)=bm1(1188)
bm2(1028)=bm1(1189)
bm2(1029)=bm1(1190)
bm2(1030)=bm1(1191)
bm2(1031)=bm1(1192)
bm2(1032)=bm1(1193)
bm2(1033)=bm1(1194)
bm2(1034)=bm1(1195)
bm2(1035)=bm1(1196)
bm2(1036)=bm1(1197)
bm2(1037)=bm1(1198)
bm2(1038)=bm1(1199)
bm2(1039)=bm1(1200)
bm2(1040)=bm1(1201)
bm2(1041)=bm1(1202)
bm2(1042)=bm1(1203)
bm2(1043)=bm1(1204)
bm2(1044)=bm1(1205)
bm2(1045)=bm1(1206)
bm2(1046)=bm1(1207)
bm2(1047)=bm1(1208)
bm2(1048)=bm1(1209)
bm2(1049)=bm1(1210)
bm2(1050)=bm1(1211)
bm2(1051)=bm1(1212)
bm2(1052)=bm1(1213)
bm2(1053)=bm1(1214)
bm2(1054)=bm1(1215)
bm2(1055)=bm1(1216)
bm2(1056)=bm1(1217)
bm2(1057)=bm1(1218)
bm2(1058)=bm1(1219)
bm2(1059)=bm1(1220)
bm2(1060)=bm1(1221)
bm2(1061)=bm1(1222)
bm2(1062)=bm1(1223)
bm2(1063)=bm1(1224)
bm2(1064)=bm1(1225)
bm2(1065)=bm1(1226)
bm2(1066)=bm1(1227)
bm2(1067)=bm1(1228)
bm2(1068)=bm1(1229)
bm2(1069)=bm1(1230)
bm2(1070)=bm1(1231)
bm2(1071)=bm1(1232)
bm2(1072)=bm1(1233)
bm2(1073)=bm1(1234)
bm2(1074)=bm1(1235)
bm2(1075)=bm1(1236)
bm2(1076)=bm1(1237)
bm2(1077)=bm1(1238)
bm2(1078)=bm1(1239)
bm2(1079)=bm1(1240)
bm2(1080)=bm1(1241)
bm2(1081)=bm1(1242)
bm2(1082)=bm1(1243)
bm2(1083)=bm1(1244)
bm2(1084)=bm1(1245)
bm2(1085)=bm1(1246)
bm2(1086)=bm1(1247)
bm2(1087)=bm1(1248)
bm2(1088)=bm1(1249)
bm2(1089)=bm1(1250)
bm2(1090)=bm1(1251)
bm2(1091)=bm1(1252)
bm2(1092)=bm1(1253)
bm2(1093)=bm1(1254)
bm2(1094)=bm1(1255)
bm2(1095)=bm1(1256)
bm2(1096)=bm1(1257)
bm2(1097)=bm1(1258)
bm2(1098)=bm1(1259)
bm2(1099)=bm1(1260)
bm2(1100)=bm1(1261)
bm2(1101)=bm1(1262)
bm2(1102)=bm1(1263)
bm2(1103)=bm1(1264)
bm2(1104)=bm1(1265)
bm2(1105)=bm1(1266)
bm2(1106)=bm1(1267)
bm2(1107)=bm1(1268)
bm2(1108)=bm1(1269)
bm2(1109)=bm1(1270)
bm2(1110)=bm1(1271)
bm2(1111)=bm1(1272)
bm2(1112)=bm1(1273)
bm2(1113)=bm1(1274)
bm2(1114)=bm1(1275)
bm2(1115)=bm1(1276)
bm2(1116)=bm1(1277)
bm2(1117)=bm1(1278)
bm2(1118)=bm1(1279)
bm2(1119)=bm1(1280)
bm2(1120)=bm1(1281)
bm2(1121)=bm1(1282)
bm2(1122)=bm1(1283)
bm2(1123)=bm1(1284)
bm2(1124)=bm1(1285)
bm2(1125)=bm1(1286)
bm2(1126)=bm1(1287)
bm2(1127)=bm1(1288)
bm2(1128)=bm1(1289)
bm2(1129)=bm1(1290)
bm2(1130)=bm1(1291)
bm2(1131)=bm1(1292)
bm2(1132)=bm1(1293)
bm2(1133)=bm1(1294)
bm2(1134)=bm1(1295)
bm2(1135)=bm1(1296)
bm2(1136)=bm1(1297)
bm2(1137)=bm1(1298)
bm2(1138)=bm1(1299)
bm2(1139)=bm1(1300)
bm2(1140)=bm1(1301)
bm2(1141)=bm1(1302)
bm2(1142)=bm1(1303)
bm2(1143)=bm1(1304)
bm2(1144)=bm1(1305)
bm2(1145)=bm1(1306)
bm2(1146)=bm1(1307)
bm2(1147)=bm1(1308)
bm2(1148)=bm1(1309)
bm2(1149)=bm1(1310)
bm2(1150)=bm1(1311)
bm2(1151)=bm1(1312)
bm2(1152)=bm1(1313)
bm2(1153)=bm1(1314)
bm2(1154)=bm1(1315)
bm2(1155)=bm1(1316)
bm2(1156)=bm1(1317)
bm2(1157)=bm1(1318)
bm2(1158)=bm1(1319)
bm2(1159)=bm1(1320)
bm2(1160)=bm1(1321)
bm2(1161)=bm1(1322)
bm2(1162)=bm1(1323)
bm2(1163)=bm1(1324)
bm2(1164)=bm1(1325)
bm2(1165)=bm1(1326)
bm2(1166)=bm1(1327)
bm2(1167)=bm1(1328)
bm2(1168)=bm1(1329)
bm2(1169)=bm1(1330)
bm2(1170)=bm1(1331)
bm2(1171)=bm1(1332)
bm2(1172)=bm1(1333)
bm2(1173)=bm1(1334)
bm2(1174)=bm1(1335)
bm2(1175)=bm1(1336)
bm2(1176)=bm1(1337)
bm2(1177)=bm1(1338)
bm2(1178)=bm1(1339)
bm2(1179)=bm1(1340)
bm2(1180)=bm1(1341)
bm2(1181)=bm1(1342)
bm2(1182)=bm1(1343)
bm2(1183)=bm1(1344)
bm2(1184)=bm1(1345)
bm2(1185)=bm1(1346)
bm2(1186)=bm1(1347)
bm2(1187)=bm1(1348)
bm2(1188)=bm1(1349)
bm2(1189)=bm1(1350)
bm2(1190)=bm1(1351)
bm2(1191)=bm1(1352)
bm2(1192)=bm1(1353)
bm2(1193)=bm1(1354)
bm2(1194)=bm1(1355)
bm2(1195)=bm1(1356)
bm2(1196)=bm1(1357)
bm2(1197)=bm1(1358)
bm2(1198)=bm1(1359)
bm2(1199)=bm1(1360)
bm2(1200)=bm1(1361)
bm2(1201)=bm1(1362)
bm2(1202)=bm1(1363)
bm2(1203)=bm1(1364)
bm2(1204)=bm1(1365)
bm2(1205)=bm1(1366)
bm2(1206)=bm1(1367)
bm2(1207)=bm1(1368)
bm2(1208)=bm1(1369)
bm2(1209)=bm1(1371)
bm2(1210)=bm1(1372)
bm2(1211)=bm1(1373)
bm2(1212)=bm1(1374)
bm2(1213)=bm1(1375)
bm2(1214)=bm1(1376)
bm2(1215)=bm1(1377)
bm2(1216)=bm1(1378)
bm2(1217)=bm1(1379)
bm2(1218)=bm1(1380)
bm2(1219)=bm1(1381)
bm2(1220)=bm1(1382)
bm2(1221)=bm1(1383)
bm2(1222)=bm1(1385)
bm2(1223)=bm1(1387)
bm2(1224)=bm1(1388)
bm2(1225)=bm1(1389)
bm2(1226)=bm1(1390)
bm2(1227)=bm1(1391)
bm2(1228)=bm1(1392)
bm2(1229)=bm1(1393)
bm2(1230)=bm1(1394)
bm2(1231)=bm1(1395)
bm2(1232)=bm1(1396)
bm2(1233)=bm1(1397)
bm2(1234)=bm1(1399)
bm2(1235)=bm1(1400)
bm2(1236)=bm1(1402)
bm2(1237)=bm1(1404)
bm2(1238)=bm1(1405)
bm2(1239)=bm1(1406)
bm2(1240)=bm1(1407)
bm2(1241)=bm1(1408)
bm2(1242)=bm1(1409)
bm2(1243)=bm1(1410)
bm2(1244)=bm1(1411)
bm2(1245)=bm1(1412)
bm2(1246)=bm1(1413)
bm2(1247)=bm1(1415)
bm2(1248)=bm1(1416)
bm2(1249)=bm1(1418)
bm2(1250)=bm1(1419)
bm2(1251)=bm1(1421)
bm2(1252)=bm1(1422)
bm2(1253)=bm1(1424)
bm2(1254)=bm1(1425)
bm2(1255)=bm1(1426)
bm2(1256)=bm1(1427)
bm2(1257)=bm1(1428)
bm2(1258)=bm1(1429)
bm2(1259)=bm1(1430)
bm2(1260)=bm1(1431)
bm2(1261)=bm1(1432)
bm2(1262)=bm1(1434)
bm2(1263)=bm1(1435)
bm2(1264)=bm1(1437)
bm2(1265)=bm1(1438)
bm2(1266)=bm1(1440)
bm2(1267)=bm1(1441)
bm2(1268)=bm1(1442)
bm2(1269)=bm1(1444)
bm2(1270)=bm1(1445)
bm2(1271)=bm1(1446)
bm2(1272)=bm1(1447)
bm2(1273)=bm1(1448)
bm2(1274)=bm1(1449)
bm2(1275)=bm1(1450)
bm2(1276)=bm1(1451)
bm2(1277)=bm1(1453)
bm2(1278)=bm1(1454)
bm2(1279)=bm1(1456)
bm2(1280)=bm1(1457)
bm2(1281)=bm1(1458)
bm2(1282)=bm1(1460)
bm2(1283)=bm1(1461)
bm2(1284)=bm1(1462)
bm2(1285)=bm1(1463)
bm2(1286)=bm1(1464)
bm2(1287)=bm1(1465)
bm2(1288)=bm1(1467)
bm2(1289)=bm1(1468)
bm2(1290)=bm1(1470)
bm2(1291)=bm1(1471)
bm2(1292)=bm1(1472)
bm2(1293)=bm1(1473)
bm2(1294)=bm1(1474)
bm2(1295)=bm1(1476)
bm2(1296)=bm1(1477)
bm2(1297)=bm1(1478)
bm2(1298)=bm1(1479)
bm2(1299)=bm1(1480)
bm2(1300)=bm1(1481)
bm2(1301)=bm1(1507)
bm2(1302)=bm1(1508)
bm2(1303)=bm1(1509)
bm2(1304)=bm1(1510)
bm2(1305)=bm1(1511)
bm2(1306)=bm1(1512)
bm2(1307)=bm1(1513)
bm2(1308)=bm1(1514)
bm2(1309)=bm1(1515)
bm2(1310)=bm1(1516)
bm2(1311)=bm1(1517)
bm2(1312)=bm1(1518)
bm2(1313)=bm1(1519)
bm2(1314)=bm1(1520)
bm2(1315)=bm1(1521)
bm2(1316)=bm1(1522)
bm2(1317)=bm1(1523)
bm2(1318)=bm1(1524)
bm2(1319)=bm1(1525)
bm2(1320)=bm1(1526)
bm2(1321)=bm1(1527)
bm2(1322)=bm1(1528)
bm2(1323)=bm1(1529)
bm2(1324)=bm1(1530)
bm2(1325)=bm1(1531)
bm2(1326)=bm1(1532)
bm2(1327)=bm1(1533)
bm2(1328)=bm1(1534)
bm2(1329)=bm1(1535)
bm2(1330)=bm1(1536)
bm2(1331)=bm1(1537)
bm2(1332)=bm1(1538)
bm2(1333)=bm1(1539)
bm2(1334)=bm1(1540)
bm2(1335)=bm1(1541)
bm2(1336)=bm1(1542)
bm2(1337)=bm1(1543)
bm2(1338)=bm1(1544)
bm2(1339)=bm1(1545)
bm2(1340)=bm1(1546)
bm2(1341)=bm1(1547)
bm2(1342)=bm1(1548)
bm2(1343)=bm1(1549)
bm2(1344)=bm1(1550)
bm2(1345)=bm1(1551)
bm2(1346)=bm1(1552)
bm2(1347)=bm1(1553)
bm2(1348)=bm1(1554)
bm2(1349)=bm1(1555)
bm2(1350)=bm1(1556)
bm2(1351)=bm1(1557)
bm2(1352)=bm1(1558)
bm2(1353)=bm1(1559)
bm2(1354)=bm1(1560)
bm2(1355)=bm1(1561)
bm2(1356)=bm1(1562)
bm2(1357)=bm1(1563)
bm2(1358)=bm1(1564)
bm2(1359)=bm1(1565)
bm2(1360)=bm1(1566)
bm2(1361)=bm1(1567)
bm2(1362)=bm1(1568)
bm2(1363)=bm1(1569)
bm2(1364)=bm1(1570)
bm2(1365)=bm1(1571)
bm2(1366)=bm1(1572)
bm2(1367)=bm1(1573)
bm2(1368)=bm1(1574)
bm2(1369)=bm1(1575)
bm2(1370)=bm1(1576)
bm2(1371)=bm1(1577)
bm2(1372)=bm1(1578)
bm2(1373)=bm1(1579)
bm2(1374)=bm1(1580)
bm2(1375)=bm1(1581)
bm2(1376)=bm1(1582)
bm2(1377)=bm1(1583)
bm2(1378)=bm1(1584)
bm2(1379)=bm1(1585)
bm2(1380)=bm1(1586)
bm2(1381)=bm1(1587)
bm2(1382)=bm1(1588)
bm2(1383)=bm1(1589)
bm2(1384)=bm1(1590)
bm2(1385)=bm1(1591)
bm2(1386)=bm1(1592)
bm2(1387)=bm1(1593)
bm2(1388)=bm1(1594)
bm2(1389)=bm1(1595)
bm2(1390)=bm1(1596)
bm2(1391)=bm1(1597)
bm2(1392)=bm1(1598)
bm2(1393)=bm1(1599)
bm2(1394)=bm1(1600)
bm2(1395)=bm1(1601)
bm2(1396)=bm1(1602)
bm2(1397)=bm1(1603)
bm2(1398)=bm1(1604)
bm2(1399)=bm1(1605)
bm2(1400)=bm1(1606)
bm2(1401)=bm1(1607)
bm2(1402)=bm1(1608)
bm2(1403)=bm1(1609)
bm2(1404)=bm1(1610)
bm2(1405)=bm1(1611)
bm2(1406)=bm1(1612)
bm2(1407)=bm1(1613)
bm2(1408)=bm1(1614)
bm2(1409)=bm1(1615)
bm2(1410)=bm1(1616)
bm2(1411)=bm1(1617)
bm2(1412)=bm1(1618)
bm2(1413)=bm1(1619)
bm2(1414)=bm1(1620)
bm2(1415)=bm1(1621)
bm2(1416)=bm1(1622)
bm2(1417)=bm1(1623)
bm2(1418)=bm1(1624)
bm2(1419)=bm1(1625)
bm2(1420)=bm1(1626)
bm2(1421)=bm1(1627)
bm2(1422)=bm1(1628)
bm2(1423)=bm1(1629)
bm2(1424)=bm1(1630)
bm2(1425)=bm1(1631)
bm2(1426)=bm1(1632)
bm2(1427)=bm1(1633)
bm2(1428)=bm1(1634)
bm2(1429)=bm1(1635)
bm2(1430)=bm1(1636)
bm2(1431)=bm1(1637)
bm2(1432)=bm1(1638)
bm2(1433)=bm1(1639)
bm2(1434)=bm1(1640)
bm2(1435)=bm1(1641)
bm2(1436)=bm1(1642)
bm2(1437)=bm1(1643)
bm2(1438)=bm1(1644)
bm2(1439)=bm1(1645)
bm2(1440)=bm1(1646)
bm2(1441)=bm1(1647)
bm2(1442)=bm1(1648)
bm2(1443)=bm1(1649)
bm2(1444)=bm1(1650)
bm2(1445)=bm1(1651)
bm2(1446)=bm1(1652)
bm2(1447)=bm1(1653)
bm2(1448)=bm1(1654)
bm2(1449)=bm1(1655)
bm2(1450)=bm1(1656)
bm2(1451)=bm1(1657)
bm2(1452)=bm1(1658)
bm2(1453)=bm1(1659)
bm2(1454)=bm1(1660)
bm2(1455)=bm1(1661)
bm2(1456)=bm1(1662)
bm2(1457)=bm1(1663)
bm2(1458)=bm1(1664)
bm2(1459)=bm1(1665)
bm2(1460)=bm1(1666)
bm2(1461)=bm1(1667)
bm2(1462)=bm1(1668)
bm2(1463)=bm1(1669)
bm2(1464)=bm1(1670)
bm2(1465)=bm1(1671)
bm2(1466)=bm1(1672)
bm2(1467)=bm1(1673)
bm2(1468)=bm1(1674)
bm2(1469)=bm1(1675)
bm2(1470)=bm1(1676)
bm2(1471)=bm1(1677)
bm2(1472)=bm1(1678)
bm2(1473)=bm1(1679)
bm2(1474)=bm1(1680)
bm2(1475)=bm1(1681)
bm2(1476)=bm1(1682)
bm2(1477)=bm1(1683)
bm2(1478)=bm1(1684)
bm2(1479)=bm1(1685)
bm2(1480)=bm1(1686)
bm2(1481)=bm1(1687)
bm2(1482)=bm1(1688)
bm2(1483)=bm1(1689)
bm2(1484)=bm1(1690)
bm2(1485)=bm1(1691)
bm2(1486)=bm1(1692)
bm2(1487)=bm1(1693)
bm2(1488)=bm1(1694)
bm2(1489)=bm1(1695)
bm2(1490)=bm1(1696)
bm2(1491)=bm1(1697)
bm2(1492)=bm1(1698)
bm2(1493)=bm1(1699)
bm2(1494)=bm1(1700)
bm2(1495)=bm1(1701)
bm2(1496)=bm1(1702)
bm2(1497)=bm1(1703)
bm2(1498)=bm1(1704)
bm2(1499)=bm1(1705)
bm2(1500)=bm1(1706)
bm2(1501)=bm1(1707)
bm2(1502)=bm1(1708)
bm2(1503)=bm1(1709)
bm2(1504)=bm1(1710)
bm2(1505)=bm1(1711)
bm2(1506)=bm1(1712)
bm2(1507)=bm1(1713)
bm2(1508)=bm1(1714)
bm2(1509)=bm1(1715)
bm2(1510)=bm1(1716)
bm2(1511)=bm1(1717)
bm2(1512)=bm1(1718)
bm2(1513)=bm1(1719)
bm2(1514)=bm1(1720)
bm2(1515)=bm1(1721)
bm2(1516)=bm1(1722)
bm2(1517)=bm1(1723)
bm2(1518)=bm1(1724)
bm2(1519)=bm1(1725)
bm2(1520)=bm1(1726)
bm2(1521)=bm1(1727)
bm2(1522)=bm1(1728)
bm2(1523)=bm1(1729)
bm2(1524)=bm1(1730)
bm2(1525)=bm1(1731)
bm2(1526)=bm1(1732)
bm2(1527)=bm1(1733)
bm2(1528)=bm1(1734)
bm2(1529)=bm1(1735)
bm2(1530)=bm1(1736)
bm2(1531)=bm1(1737)
bm2(1532)=bm1(1738)
bm2(1533)=bm1(1739)
bm2(1534)=bm1(1740)
bm2(1535)=bm1(1741)
bm2(1536)=bm1(1742)
bm2(1537)=bm1(1743)
bm2(1538)=bm1(1744)
bm2(1539)=bm1(1745)
bm2(1540)=bm1(1746)
bm2(1541)=bm1(1747)
bm2(1542)=bm1(1748)
bm2(1543)=bm1(1749)
bm2(1544)=bm1(1750)
bm2(1545)=bm1(1751)
bm2(1546)=bm1(1752)
bm2(1547)=bm1(1753)
bm2(1548)=bm1(1754)
bm2(1549)=bm1(1755)
bm2(1550)=bm1(1756)
bm2(1551)=bm1(1757)
bm2(1552)=bm1(1758)
bm2(1553)=bm1(1759)
bm2(1554)=bm1(1760)
bm2(1555)=bm1(1761)
bm2(1556)=bm1(1762)
bm2(1557)=bm1(1763)
bm2(1558)=bm1(1764)
bm2(1559)=bm1(1765)
bm2(1560)=bm1(1766)
bm2(1561)=bm1(1767)
bm2(1562)=bm1(1768)
bm2(1563)=bm1(1769)
bm2(1564)=bm1(1770)
bm2(1565)=bm1(1771)
bm2(1566)=bm1(1772)
bm2(1567)=bm1(1773)
bm2(1568)=bm1(1774)
bm2(1569)=bm1(1775)
bm2(1570)=bm1(1776)
bm2(1571)=bm1(1777)
bm2(1572)=bm1(1778)
bm2(1573)=bm1(1779)
bm2(1574)=bm1(1780)
bm2(1575)=bm1(1781)
bm2(1576)=bm1(1782)
bm2(1577)=bm1(1783)
bm2(1578)=bm1(1784)
bm2(1579)=bm1(1785)
bm2(1580)=bm1(1786)
bm2(1581)=bm1(1787)
bm2(1582)=bm1(1788)
bm2(1583)=bm1(1789)
bm2(1584)=bm1(1790)
bm2(1585)=bm1(1791)
bm2(1586)=bm1(1792)
bm2(1587)=bm1(1793)
bm2(1588)=bm1(1794)
bm2(1589)=bm1(1795)
bm2(1590)=bm1(1796)
bm2(1591)=bm1(1797)
bm2(1592)=bm1(1798)
bm2(1593)=bm1(1799)
bm2(1594)=bm1(1800)
bm2(1595)=bm1(1801)
bm2(1596)=bm1(1802)
bm2(1597)=bm1(1803)
bm2(1598)=bm1(1804)
bm2(1599)=bm1(1805)
bm2(1600)=bm1(1806)
bm2(1601)=bm1(1807)
bm2(1602)=bm1(1808)
bm2(1603)=bm1(1809)
bm2(1604)=bm1(1810)
bm2(1605)=bm1(1811)
bm2(1606)=bm1(1812)
bm2(1607)=bm1(1813)
bm2(1608)=bm1(1814)
bm2(1609)=bm1(1815)
bm2(1610)=bm1(1816)
bm2(1611)=bm1(1817)
bm2(1612)=bm1(1818)
bm2(1613)=bm1(1819)
bm2(1614)=bm1(1820)
bm2(1615)=bm1(1821)
bm2(1616)=bm1(1822)
bm2(1617)=bm1(1823)
bm2(1618)=bm1(1824)
bm2(1619)=bm1(1825)
bm2(1620)=bm1(1826)
bm2(1621)=bm1(1827)
bm2(1622)=bm1(1828)
bm2(1623)=bm1(1829)
bm2(1624)=bm1(1830)
bm2(1625)=bm1(1831)
bm2(1626)=bm1(1832)
bm2(1627)=bm1(1833)
bm2(1628)=bm1(1834)
bm2(1629)=bm1(1835)
bm2(1630)=bm1(1836)
bm2(1631)=bm1(1837)
bm2(1632)=bm1(1838)
bm2(1633)=bm1(1839)
bm2(1634)=bm1(1840)
bm2(1635)=bm1(1841)
bm2(1636)=bm1(1842)
bm2(1637)=bm1(1843)
bm2(1638)=bm1(1844)
bm2(1639)=bm1(1845)
bm2(1640)=bm1(1846)
bm2(1641)=bm1(1847)
bm2(1642)=bm1(1848)
bm2(1643)=bm1(1849)
bm2(1644)=bm1(1850)
bm2(1645)=bm1(1851)
bm2(1646)=bm1(1852)
bm2(1647)=bm1(1853)
bm2(1648)=bm1(1854)
bm2(1649)=bm1(1855)
bm2(1650)=bm1(1856)
bm2(1651)=bm1(1857)
bm2(1652)=bm1(1858)
bm2(1653)=bm1(1859)
bm2(1654)=bm1(1860)
bm2(1655)=bm1(1861)
bm2(1656)=bm1(1862)
bm2(1657)=bm1(1863)
bm2(1658)=bm1(1864)
bm2(1659)=bm1(1865)
bm2(1660)=bm1(1866)
bm2(1661)=bm1(1867)
bm2(1662)=bm1(1868)
bm2(1663)=bm1(1869)
bm2(1664)=bm1(1870)
bm2(1665)=bm1(1871)
bm2(1666)=bm1(1872)
bm2(1667)=bm1(1873)
bm2(1668)=bm1(1874)
bm2(1669)=bm1(1875)
bm2(1670)=bm1(1876)
bm2(1671)=bm1(1877)
bm2(1672)=bm1(1878)
bm2(1673)=bm1(1879)
bm2(1674)=bm1(1880)
bm2(1675)=bm1(1881)
bm2(1676)=bm1(1882)
bm2(1677)=bm1(1883)
bm2(1678)=bm1(1884)
bm2(1679)=bm1(1885)
bm2(1680)=bm1(1886)
bm2(1681)=bm1(1887)
bm2(1682)=bm1(1888)
bm2(1683)=bm1(1889)
bm2(1684)=bm1(1890)
bm2(1685)=bm1(1891)
bm2(1686)=bm1(1892)
bm2(1687)=bm1(1893)
bm2(1688)=bm1(1894)
bm2(1689)=bm1(1895)
bm2(1690)=bm1(1896)
bm2(1691)=bm1(1897)
bm2(1692)=bm1(1898)
bm2(1693)=bm1(1899)
bm2(1694)=bm1(1900)
bm2(1695)=bm1(1901)
bm2(1696)=bm1(1902)
bm2(1697)=bm1(1903)
bm2(1698)=bm1(1904)
bm2(1699)=bm1(1905)
bm2(1700)=bm1(1906)
bm2(1701)=bm1(1907)
bm2(1702)=bm1(1908)
bm2(1703)=bm1(1909)
bm2(1704)=bm1(1910)
bm2(1705)=bm1(1911)
bm2(1706)=bm1(1912)
bm2(1707)=bm1(1913)
bm2(1708)=bm1(1914)
bm2(1709)=bm1(1915)
bm2(1710)=bm1(1916)
bm2(1711)=bm1(1917)
bm2(1712)=bm1(1918)
bm2(1713)=bm1(1919)
bm2(1714)=bm1(1920)
bm2(1715)=bm1(1921)
bm2(1716)=bm1(1922)
bm2(1717)=bm1(1923)
bm2(1718)=bm1(1924)
bm2(1719)=bm1(1925)
bm2(1720)=bm1(1926)
bm2(1721)=bm1(1927)
bm2(1722)=bm1(1928)
bm2(1723)=bm1(1929)
bm2(1724)=bm1(1930)
bm2(1725)=bm1(1931)
bm2(1726)=bm1(1932)
bm2(1727)=bm1(1933)
bm2(1728)=bm1(1934)
bm2(1729)=bm1(1935)
bm2(1730)=bm1(1936)
bm2(1731)=bm1(1937)
bm2(1732)=bm1(1938)
bm2(1733)=bm1(1939)
bm2(1734)=bm1(1940)
bm2(1735)=bm1(1941)
bm2(1736)=bm1(1942)
bm2(1737)=bm1(1943)
bm2(1738)=bm1(1944)
bm2(1739)=bm1(1945)
bm2(1740)=bm1(1946)
bm2(1741)=bm1(1947)
bm2(1742)=bm1(1948)
bm2(1743)=bm1(1949)
bm2(1744)=bm1(1950)
bm2(1745)=bm1(1951)
bm2(1746)=bm1(1952)
bm2(1747)=bm1(1953)
bm2(1748)=bm1(1954)
bm2(1749)=bm1(1955)
bm2(1750)=bm1(1956)
bm2(1751)=bm1(1957)
bm2(1752)=bm1(1958)
bm2(1753)=bm1(1959)
bm2(1754)=bm1(1960)
bm2(1755)=bm1(1961)
bm2(1756)=bm1(1962)
bm2(1757)=bm1(1963)
bm2(1758)=bm1(1964)
bm2(1759)=bm1(1965)
bm2(1760)=bm1(1966)
bm2(1761)=bm1(1967)
bm2(1762)=bm1(1968)
bm2(1763)=bm1(1969)
bm2(1764)=bm1(1970)
bm2(1765)=bm1(1971)
bm2(1766)=bm1(1972)
bm2(1767)=bm1(1973)
bm2(1768)=bm1(1974)
bm2(1769)=bm1(1975)
bm2(1770)=bm1(1976)
bm2(1771)=bm1(1977)
bm2(1772)=bm1(1978)
bm2(1773)=bm1(1979)
bm2(1774)=bm1(1980)
bm2(1775)=bm1(1981)
bm2(1776)=bm1(1982)
bm2(1777)=bm1(1983)
bm2(1778)=bm1(1984)
bm2(1779)=bm1(1985)
bm2(1780)=bm1(1986)
bm2(1781)=bm1(1987)
bm2(1782)=bm1(1988)
bm2(1783)=bm1(1989)
bm2(1784)=bm1(1990)
bm2(1785)=bm1(1991)
bm2(1786)=bm1(1992)
bm2(1787)=bm1(1993)
bm2(1788)=bm1(1994)
bm2(1789)=bm1(1995)
bm2(1790)=bm1(1996)
bm2(1791)=bm1(1997)
bm2(1792)=bm1(1998)
bm2(1793)=bm1(1999)
bm2(1794)=bm1(2000)
bm2(1795)=bm1(2001)
bm2(1796)=bm1(2002)
bm2(1797)=bm1(2003)
bm2(1798)=bm1(2004)
bm2(1799)=bm1(2005)
bm2(1800)=bm1(2006)
bm2(1801)=bm1(2007)
bm2(1802)=bm1(2008)
bm2(1803)=bm1(2009)
bm2(1804)=bm1(2010)
bm2(1805)=bm1(2011)
bm2(1806)=bm1(2012)
bm2(1807)=bm1(2013)
bm2(1808)=bm1(2014)
bm2(1809)=bm1(2015)
bm2(1810)=bm1(2016)
bm2(1811)=bm1(2017)
bm2(1812)=bm1(2018)
bm2(1813)=bm1(2019)
bm2(1814)=bm1(2020)
bm2(1815)=bm1(2021)
bm2(1816)=bm1(2022)
bm2(1817)=bm1(2023)
bm2(1818)=bm1(2024)
bm2(1819)=bm1(2025)
bm2(1820)=bm1(2026)
bm2(1821)=bm1(2027)
bm2(1822)=bm1(2028)
bm2(1823)=bm1(2029)
bm2(1824)=bm1(2030)
bm2(1825)=bm1(2031)
bm2(1826)=bm1(2032)
bm2(1827)=bm1(2033)
bm2(1828)=bm1(2034)
bm2(1829)=bm1(2035)
bm2(1830)=bm1(2036)
bm2(1831)=bm1(2037)
bm2(1832)=bm1(2038)
bm2(1833)=bm1(2039)
bm2(1834)=bm1(2040)
bm2(1835)=bm1(2041)
bm2(1836)=bm1(2042)
bm2(1837)=bm1(2043)
bm2(1838)=bm1(2044)
bm2(1839)=bm1(2045)
bm2(1840)=bm1(2046)
bm2(1841)=bm1(2047)
bm2(1842)=bm1(2048)
bm2(1843)=bm1(2049)
bm2(1844)=bm1(2050)
bm2(1845)=bm1(2051)
bm2(1846)=bm1(2052)
bm2(1847)=bm1(2053)
bm2(1848)=bm1(2054)
bm2(1849)=bm1(2055)
bm2(1850)=bm1(2056)
bm2(1851)=bm1(2057)
bm2(1852)=bm1(2058)
bm2(1853)=bm1(2059)
bm2(1854)=bm1(2060)
bm2(1855)=bm1(2061)
bm2(1856)=bm1(2062)
bm2(1857)=bm1(2063)
bm2(1858)=bm1(2064)
bm2(1859)=bm1(2065)
bm2(1860)=bm1(2066)
bm2(1861)=bm1(2067)
bm2(1862)=bm1(2068)
bm2(1863)=bm1(2069)
bm2(1864)=bm1(2070)
bm2(1865)=bm1(2071)
bm2(1866)=bm1(2072)
bm2(1867)=bm1(2073)
bm2(1868)=bm1(2074)
bm2(1869)=bm1(2075)
bm2(1870)=bm1(2076)
bm2(1871)=bm1(2077)
bm2(1872)=bm1(2078)
bm2(1873)=bm1(2079)
bm2(1874)=bm1(2080)
bm2(1875)=bm1(2081)
bm2(1876)=bm1(2082)
bm2(1877)=bm1(2083)
bm2(1878)=bm1(2084)
bm2(1879)=bm1(2085)
bm2(1880)=bm1(2086)
bm2(1881)=bm1(2087)
bm2(1882)=bm1(2088)
bm2(1883)=bm1(2089)
bm2(1884)=bm1(2090)
bm2(1885)=bm1(2091)
bm2(1886)=bm1(2092)
bm2(1887)=bm1(2093)
bm2(1888)=bm1(2094)
bm2(1889)=bm1(2095)
bm2(1890)=bm1(2096)
bm2(1891)=bm1(2097)
bm2(1892)=bm1(2098)
bm2(1893)=bm1(2099)
bm2(1894)=bm1(2100)
bm2(1895)=bm1(2101)
bm2(1896)=bm1(2102)
bm2(1897)=bm1(2103)
bm2(1898)=bm1(2104)
bm2(1899)=bm1(2105)
bm2(1900)=bm1(2106)
bm2(1901)=bm1(2107)
bm2(1902)=bm1(2108)
bm2(1903)=bm1(2109)
bm2(1904)=bm1(2110)
bm2(1905)=bm1(2111)
bm2(1906)=bm1(2112)
bm2(1907)=bm1(2113)
bm2(1908)=bm1(2114)
bm2(1909)=bm1(2115)
bm2(1910)=bm1(2116)
bm2(1911)=bm1(2117)
bm2(1912)=bm1(2118)
bm2(1913)=bm1(2119)
bm2(1914)=bm1(2120)
bm2(1915)=bm1(2121)
bm2(1916)=bm1(2122)
bm2(1917)=bm1(2123)
bm2(1918)=bm1(2125)
bm2(1919)=bm1(2126)
bm2(1920)=bm1(2127)
bm2(1921)=bm1(2128)
bm2(1922)=bm1(2129)
bm2(1923)=bm1(2130)
bm2(1924)=bm1(2131)
bm2(1925)=bm1(2132)
bm2(1926)=bm1(2133)
bm2(1927)=bm1(2134)
bm2(1928)=bm1(2135)
bm2(1929)=bm1(2136)
bm2(1930)=bm1(2137)
bm2(1931)=bm1(2138)
bm2(1932)=bm1(2140)
bm2(1933)=bm1(2142)
bm2(1934)=bm1(2143)
bm2(1935)=bm1(2144)
bm2(1936)=bm1(2145)
bm2(1937)=bm1(2146)
bm2(1938)=bm1(2147)
bm2(1939)=bm1(2148)
bm2(1940)=bm1(2149)
bm2(1941)=bm1(2150)
bm2(1942)=bm1(2151)
bm2(1943)=bm1(2152)
bm2(1944)=bm1(2153)
bm2(1945)=bm1(2154)
bm2(1946)=bm1(2156)
bm2(1947)=bm1(2157)
bm2(1948)=bm1(2159)
bm2(1949)=bm1(2161)
bm2(1950)=bm1(2162)
bm2(1951)=bm1(2163)
bm2(1952)=bm1(2164)
bm2(1953)=bm1(2165)
bm2(1954)=bm1(2166)
bm2(1955)=bm1(2167)
bm2(1956)=bm1(2168)
bm2(1957)=bm1(2169)
bm2(1958)=bm1(2170)
bm2(1959)=bm1(2171)
bm2(1960)=bm1(2173)
bm2(1961)=bm1(2174)
bm2(1962)=bm1(2176)
bm2(1963)=bm1(2177)
bm2(1964)=bm1(2179)
bm2(1965)=bm1(2181)
bm2(1966)=bm1(2182)
bm2(1967)=bm1(2183)
bm2(1968)=bm1(2184)
bm2(1969)=bm1(2185)
bm2(1970)=bm1(2186)
bm2(1971)=bm1(2187)
bm2(1972)=bm1(2188)
bm2(1973)=bm1(2189)
bm2(1974)=bm1(2190)
bm2(1975)=bm1(2191)
bm2(1976)=bm1(2193)
bm2(1977)=bm1(2194)
bm2(1978)=bm1(2196)
bm2(1979)=bm1(2197)
bm2(1980)=bm1(2199)
bm2(1981)=bm1(2200)
bm2(1982)=bm1(2201)
bm2(1983)=bm1(2203)
bm2(1984)=bm1(2204)
bm2(1985)=bm1(2206)
bm2(1986)=bm1(2207)
bm2(1987)=bm1(2208)
bm2(1988)=bm1(2209)
bm2(1989)=bm1(2210)
bm2(1990)=bm1(2211)
bm2(1991)=bm1(2212)
bm2(1992)=bm1(2213)
bm2(1993)=bm1(2214)
bm2(1994)=bm1(2216)
bm2(1995)=bm1(2217)
bm2(1996)=bm1(2219)
bm2(1997)=bm1(2220)
bm2(1998)=bm1(2222)
bm2(1999)=bm1(2223)
bm2(2000)=bm1(2224)
bm2(2001)=bm1(2226)
bm2(2002)=bm1(2227)
bm2(2003)=bm1(2228)
bm2(2004)=bm1(2229)
bm2(2005)=bm1(2230)
bm2(2006)=bm1(2231)
bm2(2007)=bm1(2232)
bm2(2008)=bm1(2233)
bm2(2009)=bm1(2235)
bm2(2010)=bm1(2236)
bm2(2011)=bm1(2238)
bm2(2012)=bm1(2239)
bm2(2013)=bm1(2240)
bm2(2014)=bm1(2242)
bm2(2015)=bm1(2243)
bm2(2016)=bm1(2244)
bm2(2017)=bm1(2245)
bm2(2018)=bm1(2246)
bm2(2019)=bm1(2247)
bm2(2020)=bm1(2249)
bm2(2021)=bm1(2250)
bm2(2022)=bm1(2252)
bm2(2023)=bm1(2253)
bm2(2024)=bm1(2254)
bm2(2025)=bm1(2255)
bm2(2026)=bm1(2256)
bm2(2027)=bm1(2258)
bm2(2028)=bm1(2259)
bm2(2029)=bm1(2260)
bm2(2030)=bm1(2261)
bm2(2031)=bm1(2262)
bm2(2032)=bm1(2263)
bm2(2033)=bm1(2291)
bm2(2034)=bm1(2292)
bm2(2035)=bm1(2293)
bm2(2036)=bm1(2294)
bm2(2037)=bm1(2295)
bm2(2038)=bm1(2296)
bm2(2039)=bm1(2297)
bm2(2040)=bm1(2298)
bm2(2041)=bm1(2299)
bm2(2042)=bm1(2300)
bm2(2043)=bm1(2301)
bm2(2044)=bm1(2302)
bm2(2045)=bm1(2303)
bm2(2046)=bm1(2304)
bm2(2047)=bm1(2305)
bm2(2048)=bm1(2306)
bm2(2049)=bm1(2307)
bm2(2050)=bm1(2308)
bm2(2051)=bm1(2309)
bm2(2052)=bm1(2310)
bm2(2053)=bm1(2311)
bm2(2054)=bm1(2312)
bm2(2055)=bm1(2313)
bm2(2056)=bm1(2314)
bm2(2057)=bm1(2315)
bm2(2058)=bm1(2316)
bm2(2059)=bm1(2317)
bm2(2060)=bm1(2318)
bm2(2061)=bm1(2319)
bm2(2062)=bm1(2320)
bm2(2063)=bm1(2321)
bm2(2064)=bm1(2322)
bm2(2065)=bm1(2323)
bm2(2066)=bm1(2324)
bm2(2067)=bm1(2325)
bm2(2068)=bm1(2326)
bm2(2069)=bm1(2327)
bm2(2070)=bm1(2328)
bm2(2071)=bm1(2329)
bm2(2072)=bm1(2330)
bm2(2073)=bm1(2331)
bm2(2074)=bm1(2332)
bm2(2075)=bm1(2333)
bm2(2076)=bm1(2334)
bm2(2077)=bm1(2335)
bm2(2078)=bm1(2336)
bm2(2079)=bm1(2337)
bm2(2080)=bm1(2338)
bm2(2081)=bm1(2339)
bm2(2082)=bm1(2340)
bm2(2083)=bm1(2341)
bm2(2084)=bm1(2342)
bm2(2085)=bm1(2343)
bm2(2086)=bm1(2344)
bm2(2087)=bm1(2345)
bm2(2088)=bm1(2346)
bm2(2089)=bm1(2347)
bm2(2090)=bm1(2348)
bm2(2091)=bm1(2349)
bm2(2092)=bm1(2350)
bm2(2093)=bm1(2351)
bm2(2094)=bm1(2352)
bm2(2095)=bm1(2353)
bm2(2096)=bm1(2354)
bm2(2097)=bm1(2355)
bm2(2098)=bm1(2356)
bm2(2099)=bm1(2357)
bm2(2100)=bm1(2358)
bm2(2101)=bm1(2359)
bm2(2102)=bm1(2360)
bm2(2103)=bm1(2361)
bm2(2104)=bm1(2362)
bm2(2105)=bm1(2363)
bm2(2106)=bm1(2364)
bm2(2107)=bm1(2365)
bm2(2108)=bm1(2366)
bm2(2109)=bm1(2367)
bm2(2110)=bm1(2368)
bm2(2111)=bm1(2369)
bm2(2112)=bm1(2370)
bm2(2113)=bm1(2371)
bm2(2114)=bm1(2372)
bm2(2115)=bm1(2373)
bm2(2116)=bm1(2374)
bm2(2117)=bm1(2375)
bm2(2118)=bm1(2376)
bm2(2119)=bm1(2377)
bm2(2120)=bm1(2378)
bm2(2121)=bm1(2379)
bm2(2122)=bm1(2380)
bm2(2123)=bm1(2381)
bm2(2124)=bm1(2382)
bm2(2125)=bm1(2383)
bm2(2126)=bm1(2384)
bm2(2127)=bm1(2385)
bm2(2128)=bm1(2386)
bm2(2129)=bm1(2387)
bm2(2130)=bm1(2388)
bm2(2131)=bm1(2389)
bm2(2132)=bm1(2390)
bm2(2133)=bm1(2391)
bm2(2134)=bm1(2392)
bm2(2135)=bm1(2393)
bm2(2136)=bm1(2394)
bm2(2137)=bm1(2395)
bm2(2138)=bm1(2396)
bm2(2139)=bm1(2397)
bm2(2140)=bm1(2398)
bm2(2141)=bm1(2399)
bm2(2142)=bm1(2400)
bm2(2143)=bm1(2401)
bm2(2144)=bm1(2402)
bm2(2145)=bm1(2403)
bm2(2146)=bm1(2404)
bm2(2147)=bm1(2405)
bm2(2148)=bm1(2406)
bm2(2149)=bm1(2407)
bm2(2150)=bm1(2408)
bm2(2151)=bm1(2409)
bm2(2152)=bm1(2410)
bm2(2153)=bm1(2411)
bm2(2154)=bm1(2412)
bm2(2155)=bm1(2413)
bm2(2156)=bm1(2414)
bm2(2157)=bm1(2415)
bm2(2158)=bm1(2416)
bm2(2159)=bm1(2417)
bm2(2160)=bm1(2418)
bm2(2161)=bm1(2419)
bm2(2162)=bm1(2420)
bm2(2163)=bm1(2421)
bm2(2164)=bm1(2422)
bm2(2165)=bm1(2423)
bm2(2166)=bm1(2424)
bm2(2167)=bm1(2425)
bm2(2168)=bm1(2426)
bm2(2169)=bm1(2427)
bm2(2170)=bm1(2428)
bm2(2171)=bm1(2429)
bm2(2172)=bm1(2430)
bm2(2173)=bm1(2431)
bm2(2174)=bm1(2432)
bm2(2175)=bm1(2433)
bm2(2176)=bm1(2434)
bm2(2177)=bm1(2435)
bm2(2178)=bm1(2436)
bm2(2179)=bm1(2437)
bm2(2180)=bm1(2438)
bm2(2181)=bm1(2439)
bm2(2182)=bm1(2440)
bm2(2183)=bm1(2441)
bm2(2184)=bm1(2442)
bm2(2185)=bm1(2443)
bm2(2186)=bm1(2444)
bm2(2187)=bm1(2445)
bm2(2188)=bm1(2446)
bm2(2189)=bm1(2447)
bm2(2190)=bm1(2448)
bm2(2191)=bm1(2449)
bm2(2192)=bm1(2450)
bm2(2193)=bm1(2451)
bm2(2194)=bm1(2452)
bm2(2195)=bm1(2453)
bm2(2196)=bm1(2454)
bm2(2197)=bm1(2455)
bm2(2198)=bm1(2456)
bm2(2199)=bm1(2457)
bm2(2200)=bm1(2458)
bm2(2201)=bm1(2459)
bm2(2202)=bm1(2460)
bm2(2203)=bm1(2461)
bm2(2204)=bm1(2462)
bm2(2205)=bm1(2463)
bm2(2206)=bm1(2464)
bm2(2207)=bm1(2465)
bm2(2208)=bm1(2466)
bm2(2209)=bm1(2467)
bm2(2210)=bm1(2468)
bm2(2211)=bm1(2469)
bm2(2212)=bm1(2470)
bm2(2213)=bm1(2471)
bm2(2214)=bm1(2472)
bm2(2215)=bm1(2473)
bm2(2216)=bm1(2474)
bm2(2217)=bm1(2475)
bm2(2218)=bm1(2476)
bm2(2219)=bm1(2477)
bm2(2220)=bm1(2478)
bm2(2221)=bm1(2479)
bm2(2222)=bm1(2480)
bm2(2223)=bm1(2481)
bm2(2224)=bm1(2482)
bm2(2225)=bm1(2483)
bm2(2226)=bm1(2484)
bm2(2227)=bm1(2485)
bm2(2228)=bm1(2486)
bm2(2229)=bm1(2487)
bm2(2230)=bm1(2488)
bm2(2231)=bm1(2489)
bm2(2232)=bm1(2490)
bm2(2233)=bm1(2491)
bm2(2234)=bm1(2492)
bm2(2235)=bm1(2493)
bm2(2236)=bm1(2494)
bm2(2237)=bm1(2495)
bm2(2238)=bm1(2496)
bm2(2239)=bm1(2497)
bm2(2240)=bm1(2498)
bm2(2241)=bm1(2499)
bm2(2242)=bm1(2500)
bm2(2243)=bm1(2501)
bm2(2244)=bm1(2502)
bm2(2245)=bm1(2503)
bm2(2246)=bm1(2504)
bm2(2247)=bm1(2505)
bm2(2248)=bm1(2506)
bm2(2249)=bm1(2507)
bm2(2250)=bm1(2508)
bm2(2251)=bm1(2509)
bm2(2252)=bm1(2510)
bm2(2253)=bm1(2511)
bm2(2254)=bm1(2512)
bm2(2255)=bm1(2513)
bm2(2256)=bm1(2514)
bm2(2257)=bm1(2515)
bm2(2258)=bm1(2516)
bm2(2259)=bm1(2517)
bm2(2260)=bm1(2518)
bm2(2261)=bm1(2519)
bm2(2262)=bm1(2520)
bm2(2263)=bm1(2521)
bm2(2264)=bm1(2522)
bm2(2265)=bm1(2523)
bm2(2266)=bm1(2524)
bm2(2267)=bm1(2525)
bm2(2268)=bm1(2526)
bm2(2269)=bm1(2527)
bm2(2270)=bm1(2528)
bm2(2271)=bm1(2529)
bm2(2272)=bm1(2530)
bm2(2273)=bm1(2531)
bm2(2274)=bm1(2532)
bm2(2275)=bm1(2533)
bm2(2276)=bm1(2534)
bm2(2277)=bm1(2535)
bm2(2278)=bm1(2536)
bm2(2279)=bm1(2537)
bm2(2280)=bm1(2538)
bm2(2281)=bm1(2539)
bm2(2282)=bm1(2540)
bm2(2283)=bm1(2541)
bm2(2284)=bm1(2542)
bm2(2285)=bm1(2543)
bm2(2286)=bm1(2544)
bm2(2287)=bm1(2545)
bm2(2288)=bm1(2546)
bm2(2289)=bm1(2547)
bm2(2290)=bm1(2548)
bm2(2291)=bm1(2549)
bm2(2292)=bm1(2550)
bm2(2293)=bm1(2551)
bm2(2294)=bm1(2552)
bm2(2295)=bm1(2553)
bm2(2296)=bm1(2554)
bm2(2297)=bm1(2555)
bm2(2298)=bm1(2556)
bm2(2299)=bm1(2557)
bm2(2300)=bm1(2558)
bm2(2301)=bm1(2559)
bm2(2302)=bm1(2560)
bm2(2303)=bm1(2561)
bm2(2304)=bm1(2562)
bm2(2305)=bm1(2563)
bm2(2306)=bm1(2564)
bm2(2307)=bm1(2565)
bm2(2308)=bm1(2566)
bm2(2309)=bm1(2567)
bm2(2310)=bm1(2568)
bm2(2311)=bm1(2569)
bm2(2312)=bm1(2570)
bm2(2313)=bm1(2571)
bm2(2314)=bm1(2572)
bm2(2315)=bm1(2573)
bm2(2316)=bm1(2574)
bm2(2317)=bm1(2575)
bm2(2318)=bm1(2576)
bm2(2319)=bm1(2577)
bm2(2320)=bm1(2578)
bm2(2321)=bm1(2579)
bm2(2322)=bm1(2580)
bm2(2323)=bm1(2581)
bm2(2324)=bm1(2582)
bm2(2325)=bm1(2583)
bm2(2326)=bm1(2584)
bm2(2327)=bm1(2585)
bm2(2328)=bm1(2586)
bm2(2329)=bm1(2587)
bm2(2330)=bm1(2588)
bm2(2331)=bm1(2589)
bm2(2332)=bm1(2590)
bm2(2333)=bm1(2591)
bm2(2334)=bm1(2592)
bm2(2335)=bm1(2593)
bm2(2336)=bm1(2594)
bm2(2337)=bm1(2595)
bm2(2338)=bm1(2596)
bm2(2339)=bm1(2597)
bm2(2340)=bm1(2598)
bm2(2341)=bm1(2599)
bm2(2342)=bm1(2600)
bm2(2343)=bm1(2601)
bm2(2344)=bm1(2602)
bm2(2345)=bm1(2603)
bm2(2346)=bm1(2604)
bm2(2347)=bm1(2605)
bm2(2348)=bm1(2606)
bm2(2349)=bm1(2607)
bm2(2350)=bm1(2608)
bm2(2351)=bm1(2609)
bm2(2352)=bm1(2610)
bm2(2353)=bm1(2611)
bm2(2354)=bm1(2612)
bm2(2355)=bm1(2613)
bm2(2356)=bm1(2614)
bm2(2357)=bm1(2615)
bm2(2358)=bm1(2616)
bm2(2359)=bm1(2617)
bm2(2360)=bm1(2618)
bm2(2361)=bm1(2619)
bm2(2362)=bm1(2620)
bm2(2363)=bm1(2621)
bm2(2364)=bm1(2622)
bm2(2365)=bm1(2623)
bm2(2366)=bm1(2624)
bm2(2367)=bm1(2625)
bm2(2368)=bm1(2626)
bm2(2369)=bm1(2627)
bm2(2370)=bm1(2628)
bm2(2371)=bm1(2629)
bm2(2372)=bm1(2630)
bm2(2373)=bm1(2631)
bm2(2374)=bm1(2632)
bm2(2375)=bm1(2633)
bm2(2376)=bm1(2634)
bm2(2377)=bm1(2635)
bm2(2378)=bm1(2636)
bm2(2379)=bm1(2637)
bm2(2380)=bm1(2638)
bm2(2381)=bm1(2639)
bm2(2382)=bm1(2640)
bm2(2383)=bm1(2641)
bm2(2384)=bm1(2642)
bm2(2385)=bm1(2643)
bm2(2386)=bm1(2644)
bm2(2387)=bm1(2645)
bm2(2388)=bm1(2646)
bm2(2389)=bm1(2647)
bm2(2390)=bm1(2648)
bm2(2391)=bm1(2649)
bm2(2392)=bm1(2650)
bm2(2393)=bm1(2651)
bm2(2394)=bm1(2652)
bm2(2395)=bm1(2653)
bm2(2396)=bm1(2654)
bm2(2397)=bm1(2655)
bm2(2398)=bm1(2656)
bm2(2399)=bm1(2657)
bm2(2400)=bm1(2658)
bm2(2401)=bm1(2659)
bm2(2402)=bm1(2660)
bm2(2403)=bm1(2661)
bm2(2404)=bm1(2662)
bm2(2405)=bm1(2663)
bm2(2406)=bm1(2664)
bm2(2407)=bm1(2665)
bm2(2408)=bm1(2666)
bm2(2409)=bm1(2667)
bm2(2410)=bm1(2668)
bm2(2411)=bm1(2669)
bm2(2412)=bm1(2670)
bm2(2413)=bm1(2671)
bm2(2414)=bm1(2672)
bm2(2415)=bm1(2673)
bm2(2416)=bm1(2674)
bm2(2417)=bm1(2675)
bm2(2418)=bm1(2676)
bm2(2419)=bm1(2677)
bm2(2420)=bm1(2678)
bm2(2421)=bm1(2679)
bm2(2422)=bm1(2680)
bm2(2423)=bm1(2681)
bm2(2424)=bm1(2682)
bm2(2425)=bm1(2683)
bm2(2426)=bm1(2684)
bm2(2427)=bm1(2685)
bm2(2428)=bm1(2686)
bm2(2429)=bm1(2687)
bm2(2430)=bm1(2688)
bm2(2431)=bm1(2689)
bm2(2432)=bm1(2690)
bm2(2433)=bm1(2691)
bm2(2434)=bm1(2692)
bm2(2435)=bm1(2693)
bm2(2436)=bm1(2694)
bm2(2437)=bm1(2695)
bm2(2438)=bm1(2696)
bm2(2439)=bm1(2697)
bm2(2440)=bm1(2698)
bm2(2441)=bm1(2699)
bm2(2442)=bm1(2700)
bm2(2443)=bm1(2701)
bm2(2444)=bm1(2702)
bm2(2445)=bm1(2703)
bm2(2446)=bm1(2704)
bm2(2447)=bm1(2705)
bm2(2448)=bm1(2706)
bm2(2449)=bm1(2707)
bm2(2450)=bm1(2708)
bm2(2451)=bm1(2709)
bm2(2452)=bm1(2710)
bm2(2453)=bm1(2711)
bm2(2454)=bm1(2712)
bm2(2455)=bm1(2713)
bm2(2456)=bm1(2714)
bm2(2457)=bm1(2715)
bm2(2458)=bm1(2716)
bm2(2459)=bm1(2717)
bm2(2460)=bm1(2718)
bm2(2461)=bm1(2719)
bm2(2462)=bm1(2720)
bm2(2463)=bm1(2721)
bm2(2464)=bm1(2722)
bm2(2465)=bm1(2723)
bm2(2466)=bm1(2724)
bm2(2467)=bm1(2725)
bm2(2468)=bm1(2726)
bm2(2469)=bm1(2727)
bm2(2470)=bm1(2728)
bm2(2471)=bm1(2729)
bm2(2472)=bm1(2730)
bm2(2473)=bm1(2731)
bm2(2474)=bm1(2732)
bm2(2475)=bm1(2733)
bm2(2476)=bm1(2734)
bm2(2477)=bm1(2735)
bm2(2478)=bm1(2736)
bm2(2479)=bm1(2737)
bm2(2480)=bm1(2738)
bm2(2481)=bm1(2739)
bm2(2482)=bm1(2740)
bm2(2483)=bm1(2741)
bm2(2484)=bm1(2742)
bm2(2485)=bm1(2743)
bm2(2486)=bm1(2744)
bm2(2487)=bm1(2745)
bm2(2488)=bm1(2746)
bm2(2489)=bm1(2747)
bm2(2490)=bm1(2748)
bm2(2491)=bm1(2749)
bm2(2492)=bm1(2750)
bm2(2493)=bm1(2751)
bm2(2494)=bm1(2752)
bm2(2495)=bm1(2753)
bm2(2496)=bm1(2754)
bm2(2497)=bm1(2755)
bm2(2498)=bm1(2756)
bm2(2499)=bm1(2757)
bm2(2500)=bm1(2758)
bm2(2501)=bm1(2759)
bm2(2502)=bm1(2760)
bm2(2503)=bm1(2761)
bm2(2504)=bm1(2762)
bm2(2505)=bm1(2763)
bm2(2506)=bm1(2764)
bm2(2507)=bm1(2765)
bm2(2508)=bm1(2766)
bm2(2509)=bm1(2767)
bm2(2510)=bm1(2768)
bm2(2511)=bm1(2769)
bm2(2512)=bm1(2770)
bm2(2513)=bm1(2771)
bm2(2514)=bm1(2772)
bm2(2515)=bm1(2773)
bm2(2516)=bm1(2774)
bm2(2517)=bm1(2775)
bm2(2518)=bm1(2776)
bm2(2519)=bm1(2777)
bm2(2520)=bm1(2778)
bm2(2521)=bm1(2779)
bm2(2522)=bm1(2780)
bm2(2523)=bm1(2781)
bm2(2524)=bm1(2782)
bm2(2525)=bm1(2783)
bm2(2526)=bm1(2784)
bm2(2527)=bm1(2785)
bm2(2528)=bm1(2786)
bm2(2529)=bm1(2787)
bm2(2530)=bm1(2788)
bm2(2531)=bm1(2789)
bm2(2532)=bm1(2790)
bm2(2533)=bm1(2791)
bm2(2534)=bm1(2792)
bm2(2535)=bm1(2793)
bm2(2536)=bm1(2794)
bm2(2537)=bm1(2795)
bm2(2538)=bm1(2796)
bm2(2539)=bm1(2797)
bm2(2540)=bm1(2798)
bm2(2541)=bm1(2799)
bm2(2542)=bm1(2800)
bm2(2543)=bm1(2801)
bm2(2544)=bm1(2802)
bm2(2545)=bm1(2803)
bm2(2546)=bm1(2804)
bm2(2547)=bm1(2805)
bm2(2548)=bm1(2806)
bm2(2549)=bm1(2807)
bm2(2550)=bm1(2808)
bm2(2551)=bm1(2809)
bm2(2552)=bm1(2810)
bm2(2553)=bm1(2811)
bm2(2554)=bm1(2812)
bm2(2555)=bm1(2813)
bm2(2556)=bm1(2814)
bm2(2557)=bm1(2815)
bm2(2558)=bm1(2816)
bm2(2559)=bm1(2817)
bm2(2560)=bm1(2818)
bm2(2561)=bm1(2819)
bm2(2562)=bm1(2820)
bm2(2563)=bm1(2821)
bm2(2564)=bm1(2822)
bm2(2565)=bm1(2823)
bm2(2566)=bm1(2824)
bm2(2567)=bm1(2825)
bm2(2568)=bm1(2826)
bm2(2569)=bm1(2827)
bm2(2570)=bm1(2828)
bm2(2571)=bm1(2829)
bm2(2572)=bm1(2830)
bm2(2573)=bm1(2831)
bm2(2574)=bm1(2832)
bm2(2575)=bm1(2833)
bm2(2576)=bm1(2834)
bm2(2577)=bm1(2835)
bm2(2578)=bm1(2836)
bm2(2579)=bm1(2837)
bm2(2580)=bm1(2838)
bm2(2581)=bm1(2839)
bm2(2582)=bm1(2840)
bm2(2583)=bm1(2841)
bm2(2584)=bm1(2842)
bm2(2585)=bm1(2843)
bm2(2586)=bm1(2844)
bm2(2587)=bm1(2845)
bm2(2588)=bm1(2846)
bm2(2589)=bm1(2847)
bm2(2590)=bm1(2848)
bm2(2591)=bm1(2849)
bm2(2592)=bm1(2850)
bm2(2593)=bm1(2851)
bm2(2594)=bm1(2852)
bm2(2595)=bm1(2853)
bm2(2596)=bm1(2854)
bm2(2597)=bm1(2855)
bm2(2598)=bm1(2856)
bm2(2599)=bm1(2857)
bm2(2600)=bm1(2858)
bm2(2601)=bm1(2859)
bm2(2602)=bm1(2860)
bm2(2603)=bm1(2861)
bm2(2604)=bm1(2862)
bm2(2605)=bm1(2863)
bm2(2606)=bm1(2864)
bm2(2607)=bm1(2865)
bm2(2608)=bm1(2866)
bm2(2609)=bm1(2867)
bm2(2610)=bm1(2868)
bm2(2611)=bm1(2869)
bm2(2612)=bm1(2870)
bm2(2613)=bm1(2871)
bm2(2614)=bm1(2872)
bm2(2615)=bm1(2873)
bm2(2616)=bm1(2874)
bm2(2617)=bm1(2875)
bm2(2618)=bm1(2876)
bm2(2619)=bm1(2877)
bm2(2620)=bm1(2878)
bm2(2621)=bm1(2879)
bm2(2622)=bm1(2880)
bm2(2623)=bm1(2881)
bm2(2624)=bm1(2882)
bm2(2625)=bm1(2883)
bm2(2626)=bm1(2884)
bm2(2627)=bm1(2885)
bm2(2628)=bm1(2886)
bm2(2629)=bm1(2887)
bm2(2630)=bm1(2888)
bm2(2631)=bm1(2889)
bm2(2632)=bm1(2890)
bm2(2633)=bm1(2891)
bm2(2634)=bm1(2892)
bm2(2635)=bm1(2893)
bm2(2636)=bm1(2894)
bm2(2637)=bm1(2895)
bm2(2638)=bm1(2896)
bm2(2639)=bm1(2897)
bm2(2640)=bm1(2898)
bm2(2641)=bm1(2899)
bm2(2642)=bm1(2900)
bm2(2643)=bm1(2901)
bm2(2644)=bm1(2902)
bm2(2645)=bm1(2903)
bm2(2646)=bm1(2904)
bm2(2647)=bm1(2905)
bm2(2648)=bm1(2906)
bm2(2649)=bm1(2907)
bm2(2650)=bm1(2908)
bm2(2651)=bm1(2909)
bm2(2652)=bm1(2910)
bm2(2653)=bm1(2911)
bm2(2654)=bm1(2912)
bm2(2655)=bm1(2913)
bm2(2656)=bm1(2914)
bm2(2657)=bm1(2915)
bm2(2658)=bm1(2916)
bm2(2659)=bm1(2917)
bm2(2660)=bm1(2918)
bm2(2661)=bm1(2919)
bm2(2662)=bm1(2920)
bm2(2663)=bm1(2921)
bm2(2664)=bm1(2922)
bm2(2665)=bm1(2923)
bm2(2666)=bm1(2924)
bm2(2667)=bm1(2925)
bm2(2668)=bm1(2926)
bm2(2669)=bm1(2927)
bm2(2670)=bm1(2928)
bm2(2671)=bm1(2929)
bm2(2672)=bm1(2930)
bm2(2673)=bm1(2931)
bm2(2674)=bm1(2932)
bm2(2675)=bm1(2933)
bm2(2676)=bm1(2934)
bm2(2677)=bm1(2935)
bm2(2678)=bm1(2936)
bm2(2679)=bm1(2937)
bm2(2680)=bm1(2938)
bm2(2681)=bm1(2939)
bm2(2682)=bm1(2940)
bm2(2683)=bm1(2941)
bm2(2684)=bm1(2942)
bm2(2685)=bm1(2943)
bm2(2686)=bm1(2944)
bm2(2687)=bm1(2945)
bm2(2688)=bm1(2946)
bm2(2689)=bm1(2947)
bm2(2690)=bm1(2948)
bm2(2691)=bm1(2949)
bm2(2692)=bm1(2950)
bm2(2693)=bm1(2951)
bm2(2694)=bm1(2952)
bm2(2695)=bm1(2953)
bm2(2696)=bm1(2954)
bm2(2697)=bm1(2955)
bm2(2698)=bm1(2956)
bm2(2699)=bm1(2957)
bm2(2700)=bm1(2958)
bm2(2701)=bm1(2959)
bm2(2702)=bm1(2960)
bm2(2703)=bm1(2961)
bm2(2704)=bm1(2962)
bm2(2705)=bm1(2963)
bm2(2706)=bm1(2964)
bm2(2707)=bm1(2965)
bm2(2708)=bm1(2966)
bm2(2709)=bm1(2967)
bm2(2710)=bm1(2968)
bm2(2711)=bm1(2969)
bm2(2712)=bm1(2970)
bm2(2713)=bm1(2971)
bm2(2714)=bm1(2972)
bm2(2715)=bm1(2973)
bm2(2716)=bm1(2974)
bm2(2717)=bm1(2975)
bm2(2718)=bm1(2976)
bm2(2719)=bm1(2977)
bm2(2720)=bm1(2978)
bm2(2721)=bm1(2979)
bm2(2722)=bm1(2980)
bm2(2723)=bm1(2981)
bm2(2724)=bm1(2982)
bm2(2725)=bm1(2983)
bm2(2726)=bm1(2984)
bm2(2727)=bm1(2985)
bm2(2728)=bm1(2986)
bm2(2729)=bm1(2987)
bm2(2730)=bm1(2988)
bm2(2731)=bm1(2989)
bm2(2732)=bm1(2990)
bm2(2733)=bm1(2991)
bm2(2734)=bm1(2992)
bm2(2735)=bm1(2993)
bm2(2736)=bm1(2994)
bm2(2737)=bm1(2995)
bm2(2738)=bm1(2996)
bm2(2739)=bm1(2997)
bm2(2740)=bm1(2998)
bm2(2741)=bm1(2999)
bm2(2742)=bm1(3000)
bm2(2743)=bm1(3001)
bm2(2744)=bm1(3002)
bm2(2745)=bm1(3003)
bm2(2746)=bm1(3004)
bm2(2747)=bm1(3005)
bm2(2748)=bm1(3006)
bm2(2749)=bm1(3007)
bm2(2750)=bm1(3008)
bm2(2751)=bm1(3009)
bm2(2752)=bm1(3010)
bm2(2753)=bm1(3011)
bm2(2754)=bm1(3012)
bm2(2755)=bm1(3013)
bm2(2756)=bm1(3014)
bm2(2757)=bm1(3015)
bm2(2758)=bm1(3016)
bm2(2759)=bm1(3017)
bm2(2760)=bm1(3018)
bm2(2761)=bm1(3019)
bm2(2762)=bm1(3020)
bm2(2763)=bm1(3021)
bm2(2764)=bm1(3022)
bm2(2765)=bm1(3023)
bm2(2766)=bm1(3024)
bm2(2767)=bm1(3025)
bm2(2768)=bm1(3026)
bm2(2769)=bm1(3027)
bm2(2770)=bm1(3028)
bm2(2771)=bm1(3029)
bm2(2772)=bm1(3030)
bm2(2773)=bm1(3031)
bm2(2774)=bm1(3032)
bm2(2775)=bm1(3033)
bm2(2776)=bm1(3034)
bm2(2777)=bm1(3035)
bm2(2778)=bm1(3036)
bm2(2779)=bm1(3037)
bm2(2780)=bm1(3038)
bm2(2781)=bm1(3039)
bm2(2782)=bm1(3040)
bm2(2783)=bm1(3041)
bm2(2784)=bm1(3042)
bm2(2785)=bm1(3043)
bm2(2786)=bm1(3044)
bm2(2787)=bm1(3045)
bm2(2788)=bm1(3046)
bm2(2789)=bm1(3047)
bm2(2790)=bm1(3048)
bm2(2791)=bm1(3049)
bm2(2792)=bm1(3050)
bm2(2793)=bm1(3051)
bm2(2794)=bm1(3052)
bm2(2795)=bm1(3053)
bm2(2796)=bm1(3054)
bm2(2797)=bm1(3055)
bm2(2798)=bm1(3056)
bm2(2799)=bm1(3057)
bm2(2800)=bm1(3058)
bm2(2801)=bm1(3059)
bm2(2802)=bm1(3060)
bm2(2803)=bm1(3061)
bm2(2804)=bm1(3062)
bm2(2805)=bm1(3063)
bm2(2806)=bm1(3064)
bm2(2807)=bm1(3065)
bm2(2808)=bm1(3066)
bm2(2809)=bm1(3067)
bm2(2810)=bm1(3068)
bm2(2811)=bm1(3069)
bm2(2812)=bm1(3070)
bm2(2813)=bm1(3071)
bm2(2814)=bm1(3072)
bm2(2815)=bm1(3073)
bm2(2816)=bm1(3074)
bm2(2817)=bm1(3075)
bm2(2818)=bm1(3076)
bm2(2819)=bm1(3077)
bm2(2820)=bm1(3078)
bm2(2821)=bm1(3079)
bm2(2822)=bm1(3080)
bm2(2823)=bm1(3081)
bm2(2824)=bm1(3082)
bm2(2825)=bm1(3083)
bm2(2826)=bm1(3084)
bm2(2827)=bm1(3085)
bm2(2828)=bm1(3086)
bm2(2829)=bm1(3087)
bm2(2830)=bm1(3088)
bm2(2831)=bm1(3089)
bm2(2832)=bm1(3090)
bm2(2833)=bm1(3091)
bm2(2834)=bm1(3092)
bm2(2835)=bm1(3093)
bm2(2836)=bm1(3094)
bm2(2837)=bm1(3095)
bm2(2838)=bm1(3096)
bm2(2839)=bm1(3097)
bm2(2840)=bm1(3098)
bm2(2841)=bm1(3099)
bm2(2842)=bm1(3100)
bm2(2843)=bm1(3101)
bm2(2844)=bm1(3102)
bm2(2845)=bm1(3103)
bm2(2846)=bm1(3104)
bm2(2847)=bm1(3105)
bm2(2848)=bm1(3106)
bm2(2849)=bm1(3107)
bm2(2850)=bm1(3108)
bm2(2851)=bm1(3109)
bm2(2852)=bm1(3110)
bm2(2853)=bm1(3111)
bm2(2854)=bm1(3112)
bm2(2855)=bm1(3113)
bm2(2856)=bm1(3114)
bm2(2857)=bm1(3115)
bm2(2858)=bm1(3116)
bm2(2859)=bm1(3117)
bm2(2860)=bm1(3118)
bm2(2861)=bm1(3119)
bm2(2862)=bm1(3120)
bm2(2863)=bm1(3121)
bm2(2864)=bm1(3122)
bm2(2865)=bm1(3123)
bm2(2866)=bm1(3124)
bm2(2867)=bm1(3125)
bm2(2868)=bm1(3126)
bm2(2869)=bm1(3127)
bm2(2870)=bm1(3128)
bm2(2871)=bm1(3129)
bm2(2872)=bm1(3130)
bm2(2873)=bm1(3131)
bm2(2874)=bm1(3132)
bm2(2875)=bm1(3133)
bm2(2876)=bm1(3134)
bm2(2877)=bm1(3135)
bm2(2878)=bm1(3136)
bm2(2879)=bm1(3137)
bm2(2880)=bm1(3138)
bm2(2881)=bm1(3139)
bm2(2882)=bm1(3140)
bm2(2883)=bm1(3141)
bm2(2884)=bm1(3142)
bm2(2885)=bm1(3143)
bm2(2886)=bm1(3144)
bm2(2887)=bm1(3145)
bm2(2888)=bm1(3146)
bm2(2889)=bm1(3147)
bm2(2890)=bm1(3148)
bm2(2891)=bm1(3149)
bm2(2892)=bm1(3150)
bm2(2893)=bm1(3151)
bm2(2894)=bm1(3152)
bm2(2895)=bm1(3153)
bm2(2896)=bm1(3154)
bm2(2897)=bm1(3155)
bm2(2898)=bm1(3156)
bm2(2899)=bm1(3157)
bm2(2900)=bm1(3158)
bm2(2901)=bm1(3159)
bm2(2902)=bm1(3160)
bm2(2903)=bm1(3161)
bm2(2904)=bm1(3162)
bm2(2905)=bm1(3163)
bm2(2906)=bm1(3164)
bm2(2907)=bm1(3165)
bm2(2908)=bm1(3166)
bm2(2909)=bm1(3167)
bm2(2910)=bm1(3168)
bm2(2911)=bm1(3169)
bm2(2912)=bm1(3170)
bm2(2913)=bm1(3171)
bm2(2914)=bm1(3172)
bm2(2915)=bm1(3173)
bm2(2916)=bm1(3174)
bm2(2917)=bm1(3175)
bm2(2918)=bm1(3176)
bm2(2919)=bm1(3177)
bm2(2920)=bm1(3178)
bm2(2921)=bm1(3179)
bm2(2922)=bm1(3180)
bm2(2923)=bm1(3181)
bm2(2924)=bm1(3182)
bm2(2925)=bm1(3183)
bm2(2926)=bm1(3184)
bm2(2927)=bm1(3185)
bm2(2928)=bm1(3186)
bm2(2929)=bm1(3187)
bm2(2930)=bm1(3188)
bm2(2931)=bm1(3189)
bm2(2932)=bm1(3190)
bm2(2933)=bm1(3191)
bm2(2934)=bm1(3193)
bm2(2935)=bm1(3194)
bm2(2936)=bm1(3195)
bm2(2937)=bm1(3196)
bm2(2938)=bm1(3197)
bm2(2939)=bm1(3198)
bm2(2940)=bm1(3199)
bm2(2941)=bm1(3200)
bm2(2942)=bm1(3201)
bm2(2943)=bm1(3202)
bm2(2944)=bm1(3203)
bm2(2945)=bm1(3204)
bm2(2946)=bm1(3205)
bm2(2947)=bm1(3206)
bm2(2948)=bm1(3207)
bm2(2949)=bm1(3208)
bm2(2950)=bm1(3210)
bm2(2951)=bm1(3212)
bm2(2952)=bm1(3213)
bm2(2953)=bm1(3214)
bm2(2954)=bm1(3215)
bm2(2955)=bm1(3216)
bm2(2956)=bm1(3217)
bm2(2957)=bm1(3218)
bm2(2958)=bm1(3219)
bm2(2959)=bm1(3220)
bm2(2960)=bm1(3221)
bm2(2961)=bm1(3222)
bm2(2962)=bm1(3223)
bm2(2963)=bm1(3224)
bm2(2964)=bm1(3225)
bm2(2965)=bm1(3227)
bm2(2966)=bm1(3228)
bm2(2967)=bm1(3230)
bm2(2968)=bm1(3232)
bm2(2969)=bm1(3233)
bm2(2970)=bm1(3234)
bm2(2971)=bm1(3235)
bm2(2972)=bm1(3236)
bm2(2973)=bm1(3237)
bm2(2974)=bm1(3238)
bm2(2975)=bm1(3239)
bm2(2976)=bm1(3240)
bm2(2977)=bm1(3241)
bm2(2978)=bm1(3242)
bm2(2979)=bm1(3243)
bm2(2980)=bm1(3244)
bm2(2981)=bm1(3246)
bm2(2982)=bm1(3247)
bm2(2983)=bm1(3249)
bm2(2984)=bm1(3250)
bm2(2985)=bm1(3252)
bm2(2986)=bm1(3254)
bm2(2987)=bm1(3255)
bm2(2988)=bm1(3256)
bm2(2989)=bm1(3257)
bm2(2990)=bm1(3258)
bm2(2991)=bm1(3259)
bm2(2992)=bm1(3260)
bm2(2993)=bm1(3261)
bm2(2994)=bm1(3262)
bm2(2995)=bm1(3263)
bm2(2996)=bm1(3264)
bm2(2997)=bm1(3265)
bm2(2998)=bm1(3267)
bm2(2999)=bm1(3268)
bm2(3000)=bm1(3270)
bm2(3001)=bm1(3271)
bm2(3002)=bm1(3273)
bm2(3003)=bm1(3274)
bm2(3004)=bm1(3276)
bm2(3005)=bm1(3277)
bm2(3006)=bm1(3279)
bm2(3007)=bm1(3280)
bm2(3008)=bm1(3281)
bm2(3009)=bm1(3282)
bm2(3010)=bm1(3283)
bm2(3011)=bm1(3284)
bm2(3012)=bm1(3285)
bm2(3013)=bm1(3286)
bm2(3014)=bm1(3287)
bm2(3015)=bm1(3288)
bm2(3016)=bm1(3289)
bm2(3017)=bm1(3291)
bm2(3018)=bm1(3292)
bm2(3019)=bm1(3294)
bm2(3020)=bm1(3295)
bm2(3021)=bm1(3297)
bm2(3022)=bm1(3298)
bm2(3023)=bm1(3299)
bm2(3024)=bm1(3301)
bm2(3025)=bm1(3302)
bm2(3026)=bm1(3303)
bm2(3027)=bm1(3305)
bm2(3028)=bm1(3306)
bm2(3029)=bm1(3307)
bm2(3030)=bm1(3308)
bm2(3031)=bm1(3309)
bm2(3032)=bm1(3310)
bm2(3033)=bm1(3311)
bm2(3034)=bm1(3312)
bm2(3035)=bm1(3313)
bm2(3036)=bm1(3315)
bm2(3037)=bm1(3316)
bm2(3038)=bm1(3318)
bm2(3039)=bm1(3319)
bm2(3040)=bm1(3321)
bm2(3041)=bm1(3322)
bm2(3042)=bm1(3323)
bm2(3043)=bm1(3325)
bm2(3044)=bm1(3326)
bm2(3045)=bm1(3327)
bm2(3046)=bm1(3328)
bm2(3047)=bm1(3329)
bm2(3048)=bm1(3330)
bm2(3049)=bm1(3331)
bm2(3050)=bm1(3332)
bm2(3051)=bm1(3334)
bm2(3052)=bm1(3335)
bm2(3053)=bm1(3337)
bm2(3054)=bm1(3338)
bm2(3055)=bm1(3339)
bm2(3056)=bm1(3341)
bm2(3057)=bm1(3342)
bm2(3058)=bm1(3343)
bm2(3059)=bm1(3344)
bm2(3060)=bm1(3345)
bm2(3061)=bm1(3346)
bm2(3062)=bm1(3348)
bm2(3063)=bm1(3349)
bm2(3064)=bm1(3351)
bm2(3065)=bm1(3352)
bm2(3066)=bm1(3353)
bm2(3067)=bm1(3354)
bm2(3068)=bm1(3355)
bm2(3069)=bm1(3357)
bm2(3070)=bm1(3358)
bm2(3071)=bm1(3359)
bm2(3072)=bm1(3360)
bm2(3073)=bm1(3361)
bm2(3074)=bm1(3362)
bm2(3075)=bm1(3363)

do j=1,3075
  q(j)=bm2(j)
enddo

end subroutine ev_pol_con_12

!===========================================================================
! *def ev_pol_con_10(x,q)
! subtoutine of evaluating polynomials at 10th order
!===========================================================================
subroutine ev_pol_con_10(x,q)
implicit none
integer :: i, j
real*8,dimension(1:6),intent(in)::x
real*8,dimension(1:1300),intent(out)::q
real*8,dimension(0:32) :: rm
real*8,dimension(0:1506) :: p
real*8 :: bm1(1506), bm2(1300)

rm(0) = 1.0D0
rm(1) = x(4)
rm(2) = x(2)
rm(3) = x(1)
rm(4) = x(6)
rm(5) = x(5)
rm(6) = x(3)
rm(7) = rm(1)*rm(2)
rm(8) = rm(1)*rm(3)
rm(9) = rm(2)*rm(3)
rm(10) = rm(3)*rm(4)
rm(11) = rm(2)*rm(5)
rm(12) = rm(1)*rm(6)
rm(13) = rm(4)*rm(5)
rm(14) = rm(4)*rm(6)
rm(15) = rm(5)*rm(6)
rm(16) = rm(1)*rm(9)
rm(17) = rm(1)*rm(10)
rm(18) = rm(2)*rm(10)
rm(19) = rm(1)*rm(11)
rm(20) = rm(3)*rm(11)
rm(21) = rm(2)*rm(12)
rm(22) = rm(3)*rm(12)
rm(23) = rm(2)*rm(13)
rm(24) = rm(3)*rm(13)
rm(25) = rm(1)*rm(14)
rm(26) = rm(3)*rm(14)
rm(27) = rm(1)*rm(15)
rm(28) = rm(2)*rm(15)
rm(29) = rm(4)*rm(15)
rm(30) = rm(2)*rm(24)
rm(31) = rm(1)*rm(26)
rm(32) = rm(1)*rm(28)

p(0) = rm(0)
p(1) = rm(1) + rm(2) + rm(3)
p(2) = rm(4) + rm(5) + rm(6)
p(3) = rm(7) + rm(8) + rm(9)
p(4) = rm(10) + rm(11) + rm(12)
p(5) = p(1)*p(2) - p(4)
p(6) = rm(13) + rm(14) + rm(15)
p(7) = p(1)*p(1) - p(3) - p(3)
p(8) = p(2)*p(2) - p(6) - p(6)
p(9) = rm(16)
p(10) = rm(17) + rm(18) + rm(19) + rm(20) + rm(21) + rm(22)
p(11) = p(2)*p(3) - p(10)
p(12) = rm(23) + rm(24) + rm(25) + rm(26) + rm(27) + rm(28)
p(13) = p(1)*p(6) - p(12)
p(14) = rm(29)
p(15) = p(1)*p(3) - p(9) - p(9) - p(9)
p(16) = p(1)*p(4) - p(10)
p(17) = p(2)*p(7) - p(16)
p(18) = p(2)*p(4) - p(12)
p(19) = p(1)*p(8) - p(18)
p(20) = p(2)*p(6) - p(14) - p(14) - p(14)
p(21) = p(1)*p(7) - p(15)
p(22) = p(2)*p(8) - p(20)
p(23) = p(9)*p(2)
p(24) = rm(30) + rm(31) + rm(32)
p(25) = p(3)*p(6) - p(24)
p(26) = p(14)*p(1)
p(27) = p(9)*p(1)
p(28) = p(3)*p(4) - p(23)
p(29) = p(1)*p(10) - p(23) - p(28) - p(23)
p(30) = p(1)*p(11) - p(23)
p(31) = p(1)*p(12) - p(25) - p(24) - p(24)
p(32) = p(1)*p(13) - p(25)
p(33) = p(4)*p(5) - p(25) - p(31)
p(34) = p(2)*p(11) - p(25)
p(35) = p(4)*p(6) - p(26)
p(36) = p(2)*p(12) - p(26) - p(35) - p(26)
p(37) = p(2)*p(13) - p(26)
p(38) = p(14)*p(2)
p(39) = p(3)*p(3) - p(27) - p(27)
p(40) = p(3)*p(7) - p(27)
p(41) = p(1)*p(16) - p(28)
p(42) = p(2)*p(21) - p(41)
p(43) = p(1)*p(18) - p(33)
p(44) = p(7)*p(8) - p(43)
p(45) = p(6)*p(6) - p(38) - p(38)
p(46) = p(2)*p(18) - p(35)
p(47) = p(1)*p(22) - p(46)
p(48) = p(6)*p(8) - p(38)
p(49) = p(1)*p(21) - p(40)
p(50) = p(2)*p(22) - p(48)
p(51) = p(9)*p(6)
p(52) = p(14)*p(3)
p(53) = p(9)*p(4)
p(54) = p(9)*p(5)
p(55) = p(1)*p(24) - p(51)
p(56) = p(3)*p(12) - p(51) - p(55) - p(51)
p(57) = p(3)*p(13) - p(51)
p(58) = p(14)*p(7)
p(59) = p(9)*p(8)
p(60) = p(2)*p(24) - p(52)
p(61) = p(4)*p(13) - p(58)
p(62) = p(6)*p(11) - p(52)
p(63) = p(14)*p(4)
p(64) = p(14)*p(5)
p(65) = p(9)*p(3)
p(66) = p(9)*p(7)
p(67) = p(3)*p(16) - p(53)
p(68) = p(4)*p(15) - p(54) - p(67)
p(69) = p(2)*p(39) - p(68)
p(70) = p(1)*p(29) - p(54) - p(68)
p(71) = p(7)*p(11) - p(53)
p(72) = p(1)*p(31) - p(56) - p(55)
p(73) = p(1)*p(32) - p(57)
p(74) = p(3)*p(18) - p(59)
p(75) = p(2)*p(29) - p(57) - p(55)
p(76) = p(1)*p(34) - p(59)
p(77) = p(6)*p(16) - p(58)
p(78) = p(1)*p(36) - p(62) - p(60)
p(79) = p(2)*p(32) - p(58)
p(80) = p(6)*p(12) - p(64) - p(63) - p(63)
p(81) = p(1)*p(45) - p(80)
p(82) = p(14)*p(6)
p(83) = p(2)*p(33) - p(61) - p(60)
p(84) = p(2)*p(34) - p(62)
p(85) = p(6)*p(18) - p(63)
p(86) = p(2)*p(36) - p(64) - p(80)
p(87) = p(8)*p(13) - p(63)
p(88) = p(14)*p(8)
p(89) = p(1)*p(39) - p(65)
p(90) = p(3)*p(21) - p(66)
p(91) = p(1)*p(41) - p(67)
p(92) = p(2)*p(49) - p(91)
p(93) = p(1)*p(43) - p(74)
p(94) = p(8)*p(21) - p(93)
p(95) = p(1)*p(46) - p(83)
p(96) = p(7)*p(22) - p(95)
p(97) = p(2)*p(45) - p(82)
p(98) = p(2)*p(46) - p(85)
p(99) = p(1)*p(50) - p(98)
p(100) = p(6)*p(22) - p(88)
p(101) = p(1)*p(49) - p(90)
p(102) = p(2)*p(50) - p(100)
p(103) = p(9)*p(14)
p(104) = p(9)*p(12)
p(105) = p(9)*p(13)
p(106) = p(14)*p(15)
p(107) = p(9)*p(20)
p(108) = p(14)*p(10)
p(109) = p(14)*p(11)
p(110) = p(9)*p(16)
p(111) = p(9)*p(10)
p(112) = p(9)*p(11)
p(113) = p(9)*p(17)
p(114) = p(3)*p(24) - p(104)
p(115) = p(7)*p(24) - p(105)
p(116) = p(3)*p(31) - p(104) - p(115)
p(117) = p(6)*p(39) - p(114)
p(118) = p(3)*p(32) - p(105)
p(119) = p(14)*p(21)
p(120) = p(9)*p(18)
p(121) = p(9)*p(19)
p(122) = p(4)*p(24) - p(103) - p(103) - p(103)
p(123) = p(1)*p(60) - p(107) - p(122)
p(124) = p(13)*p(16) - p(119)
p(125) = p(2)*p(56) - p(106) - p(124)
p(126) = p(4)*p(32) - p(119)
p(127) = p(1)*p(62) - p(107) - p(125)
p(128) = p(14)*p(16)
p(129) = p(14)*p(17)
p(130) = p(6)*p(24) - p(108)
p(131) = p(3)*p(45) - p(130)
p(132) = p(14)*p(12)
p(133) = p(14)*p(13)
p(134) = p(9)*p(22)
p(135) = p(8)*p(24) - p(109)
p(136) = p(13)*p(18) - p(128)
p(137) = p(6)*p(34) - p(109)
p(138) = p(14)*p(18)
p(139) = p(14)*p(19)
p(140) = p(9)*p(9)
p(141) = p(9)*p(15)
p(142) = p(9)*p(21)
p(143) = p(3)*p(41) - p(110)
p(144) = p(4)*p(39) - p(112)
p(145) = p(1)*p(68) - p(111) - p(144)
p(146) = p(1)*p(69) - p(112)
p(147) = p(1)*p(70) - p(113) - p(145)
p(148) = p(11)*p(21) - p(110)
p(149) = p(1)*p(72) - p(116) - p(115)
p(150) = p(1)*p(73) - p(118)
p(151) = p(3)*p(43) - p(120)
p(152) = p(15)*p(18) - p(121) - p(151)
p(153) = p(2)*p(69) - p(117)
p(154) = p(1)*p(75) - p(121) - p(152)
p(155) = p(7)*p(34) - p(120)
p(156) = p(6)*p(41) - p(119)
p(157) = p(1)*p(78) - p(125) - p(123)
p(158) = p(2)*p(73) - p(119)
p(159) = p(1)*p(80) - p(131) - p(130) - p(130)
p(160) = p(1)*p(81) - p(131)
p(161) = p(14)*p(14)
p(162) = p(3)*p(46) - p(134)
p(163) = p(2)*p(75) - p(126) - p(123)
p(164) = p(1)*p(84) - p(134)
p(165) = p(6)*p(43) - p(128)
p(166) = p(1)*p(86) - p(137) - p(135)
p(167) = p(8)*p(32) - p(128)
p(168) = p(4)*p(45) - p(133)
p(169) = p(2)*p(80) - p(132) - p(168)
p(170) = p(2)*p(81) - p(133)
p(171) = p(14)*p(20)
p(172) = p(2)*p(83) - p(136) - p(135)
p(173) = p(2)*p(84) - p(137)
p(174) = p(6)*p(46) - p(138)
p(175) = p(2)*p(86) - p(139) - p(169)
p(176) = p(13)*p(22) - p(138)
p(177) = p(14)*p(22)
p(178) = p(3)*p(39) - p(141)
p(179) = p(1)*p(89) - p(141) - p(178) - p(178)
p(180) = p(3)*p(49) - p(142)
p(181) = p(1)*p(91) - p(143)
p(182) = p(2)*p(101) - p(181)
p(183) = p(1)*p(93) - p(151)
p(184) = p(8)*p(49) - p(183)
p(185) = p(1)*p(95) - p(162)
p(186) = p(21)*p(22) - p(185)
p(187) = p(6)*p(45) - p(171)
p(188) = p(1)*p(98) - p(172)
p(189) = p(7)*p(50) - p(188)
p(190) = p(2)*p(97) - p(171) - p(187) - p(187)
p(191) = p(2)*p(98) - p(174)
p(192) = p(1)*p(102) - p(191)
p(193) = p(6)*p(50) - p(177)
p(194) = p(1)*p(101) - p(180)
p(195) = p(2)*p(102) - p(193)
p(196) = p(9)*p(26)
p(197) = p(9)*p(38)
p(198) = p(9)*p(24)
p(199) = p(9)*p(31)
p(200) = p(9)*p(25)
p(201) = p(9)*p(32)
p(202) = p(14)*p(39)
p(203) = p(14)*p(40)
p(204) = p(9)*p(35)
p(205) = p(9)*p(36)
p(206) = p(9)*p(37)
p(207) = p(14)*p(28)
p(208) = p(14)*p(29)
p(209) = p(14)*p(30)
p(210) = p(9)*p(45)
p(211) = p(14)*p(24)
p(212) = p(14)*p(25)
p(213) = p(9)*p(48)
p(214) = p(14)*p(33)
p(215) = p(14)*p(34)
p(216) = p(9)*p(41)
p(217) = p(9)*p(28)
p(218) = p(9)*p(23)
p(219) = p(9)*p(29)
p(220) = p(9)*p(30)
p(221) = p(9)*p(42)
p(222) = p(1)*p(114) - p(198)
p(223) = p(21)*p(24) - p(201)
p(224) = p(3)*p(72) - p(199) - p(223)
p(225) = p(1)*p(116) - p(199) - p(224)
p(226) = p(13)*p(39) - p(198)
p(227) = p(3)*p(73) - p(201)
p(228) = p(14)*p(49)
p(229) = p(9)*p(43)
p(230) = p(9)*p(33)
p(231) = p(9)*p(34)
p(232) = p(9)*p(44)
p(233) = p(16)*p(24) - p(196)
p(234) = p(2)*p(114) - p(202)
p(235) = p(1)*p(123) - p(205) - p(234)
p(236) = p(13)*p(41) - p(228)
p(237) = p(2)*p(116) - p(203) - p(236)
p(238) = p(16)*p(32) - p(228)
p(239) = p(6)*p(69) - p(202)
p(240) = p(4)*p(73) - p(228)
p(241) = p(11)*p(32) - p(196)
p(242) = p(14)*p(41)
p(243) = p(14)*p(42)
p(244) = p(1)*p(130) - p(210)
p(245) = p(4)*p(62) - p(213) - p(209)
p(246) = p(3)*p(81) - p(210)
p(247) = p(14)*p(31)
p(248) = p(14)*p(32)
p(249) = p(14)*p(26)
p(250) = p(9)*p(46)
p(251) = p(9)*p(47)
p(252) = p(18)*p(24) - p(197)
p(253) = p(1)*p(135) - p(213) - p(252)
p(254) = p(13)*p(43) - p(242)
p(255) = p(2)*p(125) - p(209) - p(245)
p(256) = p(18)*p(32) - p(242)
p(257) = p(13)*p(34) - p(197)
p(258) = p(14)*p(43)
p(259) = p(14)*p(44)
p(260) = p(2)*p(130) - p(211)
p(261) = p(4)*p(81) - p(248)
p(262) = p(11)*p(45) - p(211)
p(263) = p(14)*p(35)
p(264) = p(14)*p(36)
p(265) = p(14)*p(37)
p(266) = p(9)*p(50)
p(267) = p(22)*p(24) - p(215)
p(268) = p(13)*p(46) - p(258)
p(269) = p(6)*p(84) - p(215)
p(270) = p(14)*p(46)
p(271) = p(14)*p(47)
p(272) = p(9)*p(27)
p(273) = p(9)*p(39)
p(274) = p(9)*p(40)
p(275) = p(9)*p(49)
p(276) = p(3)*p(91) - p(216)
p(277) = p(16)*p(39) - p(218)
p(278) = p(1)*p(144) - p(217) - p(277)
p(279) = p(2)*p(178) - p(278)
p(280) = p(1)*p(145) - p(219) - p(278)
p(281) = p(7)*p(69) - p(218)
p(282) = p(1)*p(147) - p(221) - p(280)
p(283) = p(11)*p(49) - p(216)
p(284) = p(1)*p(149) - p(224) - p(223)
p(285) = p(1)*p(150) - p(227)
p(286) = p(3)*p(93) - p(229)
p(287) = p(18)*p(39) - p(231)
p(288) = p(1)*p(152) - p(230) - p(287)
p(289) = p(1)*p(153) - p(231)
p(290) = p(1)*p(154) - p(232) - p(288)
p(291) = p(21)*p(34) - p(229)
p(292) = p(6)*p(91) - p(228)
p(293) = p(1)*p(157) - p(237) - p(235)
p(294) = p(2)*p(150) - p(228)
p(295) = p(1)*p(159) - p(245) - p(244)
p(296) = p(1)*p(160) - p(246)
p(297) = p(3)*p(95) - p(250)
p(298) = p(2)*p(152) - p(238) - p(234)
p(299) = p(2)*p(153) - p(239)
p(300) = p(1)*p(163) - p(251) - p(298)
p(301) = p(7)*p(84) - p(250)
p(302) = p(6)*p(93) - p(242)
p(303) = p(1)*p(166) - p(255) - p(253)
p(304) = p(8)*p(73) - p(242)
p(305) = p(16)*p(45) - p(248)
p(306) = p(1)*p(169) - p(262) - p(260)
p(307) = p(2)*p(160) - p(248)
p(308) = p(14)*p(38)
p(309) = p(6)*p(80) - p(264) - p(263)
p(310) = p(1)*p(187) - p(309)
p(311) = p(14)*p(45)
p(312) = p(3)*p(98) - p(266)
p(313) = p(2)*p(163) - p(256) - p(253)
p(314) = p(1)*p(173) - p(266)
p(315) = p(6)*p(95) - p(258)
p(316) = p(1)*p(175) - p(269) - p(267)
p(317) = p(22)*p(32) - p(258)
p(318) = p(18)*p(45) - p(249)
p(319) = p(2)*p(169) - p(264) - p(309)
p(320) = p(8)*p(81) - p(249)
p(321) = p(14)*p(48)
p(322) = p(2)*p(172) - p(268) - p(267)
p(323) = p(2)*p(173) - p(269)
p(324) = p(6)*p(98) - p(270)
p(325) = p(2)*p(175) - p(271) - p(319)
p(326) = p(13)*p(50) - p(270)
p(327) = p(14)*p(50)
p(328) = p(1)*p(178) - p(273)
p(329) = p(21)*p(39) - p(272)
p(330) = p(3)*p(101) - p(275)
p(331) = p(1)*p(181) - p(276)
p(332) = p(2)*p(194) - p(331)
p(333) = p(1)*p(183) - p(286)
p(334) = p(8)*p(101) - p(333)
p(335) = p(1)*p(185) - p(297)
p(336) = p(22)*p(49) - p(335)
p(337) = p(1)*p(188) - p(312)
p(338) = p(21)*p(50) - p(337)
p(339) = p(2)*p(187) - p(311)
p(340) = p(1)*p(191) - p(322)
p(341) = p(7)*p(102) - p(340)
p(342) = p(22)*p(45) - p(308)
p(343) = p(2)*p(191) - p(324)
p(344) = p(1)*p(195) - p(343)
p(345) = p(6)*p(102) - p(327)
p(346) = p(1)*p(194) - p(330)
p(347) = p(2)*p(195) - p(345)
p(348) = p(9)*p(52)
p(349) = p(9)*p(58)
p(350) = p(9)*p(63)
p(351) = p(9)*p(64)
p(352) = p(9)*p(82)
p(353) = p(9)*p(88)
p(354) = p(9)*p(55)
p(355) = p(9)*p(72)
p(356) = p(9)*p(51)
p(357) = p(9)*p(56)
p(358) = p(9)*p(57)
p(359) = p(9)*p(73)
p(360) = p(14)*p(89)
p(361) = p(14)*p(90)
p(362) = p(9)*p(77)
p(363) = p(9)*p(60)
p(364) = p(9)*p(78)
p(365) = p(9)*p(61)
p(366) = p(9)*p(62)
p(367) = p(9)*p(79)
p(368) = p(14)*p(67)
p(369) = p(14)*p(68)
p(370) = p(14)*p(69)
p(371) = p(14)*p(70)
p(372) = p(14)*p(71)
p(373) = p(9)*p(80)
p(374) = p(9)*p(81)
p(375) = p(14)*p(55)
p(376) = p(14)*p(56)
p(377) = p(14)*p(57)
p(378) = p(14)*p(52)
p(379) = p(9)*p(85)
p(380) = p(9)*p(86)
p(381) = p(9)*p(87)
p(382) = p(14)*p(74)
p(383) = p(14)*p(75)
p(384) = p(14)*p(76)
p(385) = p(9)*p(97)
p(386) = p(14)*p(60)
p(387) = p(14)*p(61)
p(388) = p(14)*p(62)
p(389) = p(9)*p(100)
p(390) = p(14)*p(83)
p(391) = p(14)*p(84)
p(392) = p(9)*p(91)
p(393) = p(9)*p(67)
p(394) = p(9)*p(53)
p(395) = p(9)*p(68)
p(396) = p(9)*p(54)
p(397) = p(9)*p(69)
p(398) = p(9)*p(70)
p(399) = p(9)*p(71)
p(400) = p(9)*p(92)
p(401) = p(3)*p(114) - p(354)
p(402) = p(7)*p(114) - p(356)
p(403) = p(24)*p(49) - p(359)
p(404) = p(3)*p(149) - p(355) - p(403)
p(405) = p(1)*p(224) - p(355) - p(404)
p(406) = p(6)*p(178) - p(401)
p(407) = p(32)*p(39) - p(356)
p(408) = p(3)*p(150) - p(359)
p(409) = p(14)*p(101)
p(410) = p(9)*p(93)
p(411) = p(9)*p(74)
p(412) = p(9)*p(59)
p(413) = p(9)*p(75)
p(414) = p(9)*p(76)
p(415) = p(9)*p(94)
p(416) = p(24)*p(41) - p(349)
p(417) = p(4)*p(114) - p(348)
p(418) = p(1)*p(234) - p(363) - p(417)
p(419) = p(1)*p(235) - p(364) - p(418)
p(420) = p(13)*p(91) - p(409)
p(421) = p(2)*p(224) - p(361) - p(420)
p(422) = p(32)*p(41) - p(409)
p(423) = p(1)*p(237) - p(364) - p(421)
p(424) = p(16)*p(73) - p(409)
p(425) = p(13)*p(69) - p(348)
p(426) = p(4)*p(150) - p(409)
p(427) = p(11)*p(73) - p(349)
p(428) = p(14)*p(91)
p(429) = p(14)*p(92)
p(430) = p(3)*p(130) - p(373)
p(431) = p(7)*p(130) - p(374)
p(432) = p(3)*p(159) - p(373) - p(431)
p(433) = p(39)*p(45) - p(430)
p(434) = p(3)*p(160) - p(374)
p(435) = p(14)*p(72)
p(436) = p(14)*p(73)
p(437) = p(14)*p(58)
p(438) = p(9)*p(95)
p(439) = p(9)*p(83)
p(440) = p(9)*p(84)
p(441) = p(9)*p(96)
p(442) = p(24)*p(43) - p(350)
p(443) = p(8)*p(114) - p(370)
p(444) = p(1)*p(253) - p(380) - p(443)
p(445) = p(13)*p(93) - p(428)
p(446) = p(2)*p(237) - p(372) - p(432)
p(447) = p(32)*p(43) - p(428)
p(448) = p(6)*p(153) - p(370)
p(449) = p(18)*p(73) - p(428)
p(450) = p(32)*p(34) - p(350)
p(451) = p(14)*p(93)
p(452) = p(14)*p(94)
p(453) = p(4)*p(130) - p(352)
p(454) = p(1)*p(260) - p(385) - p(453)
p(455) = p(16)*p(81) - p(436)
p(456) = p(2)*p(245) - p(376) - p(455)
p(457) = p(4)*p(160) - p(436)
p(458) = p(11)*p(81) - p(352)
p(459) = p(14)*p(77)
p(460) = p(14)*p(78)
p(461) = p(14)*p(79)
p(462) = p(14)*p(63)
p(463) = p(14)*p(64)
p(464) = p(6)*p(130) - p(386)
p(465) = p(3)*p(187) - p(464)
p(466) = p(14)*p(80)
p(467) = p(14)*p(81)
p(468) = p(9)*p(98)
p(469) = p(9)*p(99)
p(470) = p(24)*p(46) - p(353)
p(471) = p(1)*p(267) - p(389) - p(470)
p(472) = p(13)*p(95) - p(451)
p(473) = p(2)*p(255) - p(384) - p(456)
p(474) = p(32)*p(46) - p(451)
p(475) = p(13)*p(84) - p(353)
p(476) = p(14)*p(95)
p(477) = p(14)*p(96)
p(478) = p(8)*p(130) - p(378)
p(479) = p(18)*p(81) - p(437)
p(480) = p(34)*p(45) - p(378)
p(481) = p(14)*p(85)
p(482) = p(14)*p(86)
p(483) = p(14)*p(87)
p(484) = p(9)*p(102)
p(485) = p(24)*p(50) - p(391)
p(486) = p(13)*p(98) - p(476)
p(487) = p(6)*p(173) - p(391)
p(488) = p(14)*p(98)
p(489) = p(14)*p(99)
p(490) = p(9)*p(65)
p(491) = p(9)*p(66)
p(492) = p(9)*p(89)
p(493) = p(9)*p(90)
p(494) = p(9)*p(101)
p(495) = p(3)*p(181) - p(392)
p(496) = p(39)*p(41) - p(394)
p(497) = p(4)*p(178) - p(397)
p(498) = p(1)*p(278) - p(395) - p(497)
p(499) = p(1)*p(279) - p(397)
p(500) = p(1)*p(280) - p(398) - p(498)
p(501) = p(21)*p(69) - p(394)
p(502) = p(1)*p(282) - p(400) - p(500)
p(503) = p(11)*p(101) - p(392)
p(504) = p(1)*p(284) - p(404) - p(403)
p(505) = p(1)*p(285) - p(408)
p(506) = p(3)*p(183) - p(410)
p(507) = p(39)*p(43) - p(412)
p(508) = p(1)*p(287) - p(411) - p(507)
p(509) = p(2)*p(279) - p(406)
p(510) = p(1)*p(288) - p(413) - p(508)
p(511) = p(7)*p(153) - p(412)
p(512) = p(1)*p(290) - p(415) - p(510)
p(513) = p(34)*p(49) - p(410)
p(514) = p(6)*p(181) - p(409)
p(515) = p(1)*p(293) - p(421) - p(419)
p(516) = p(2)*p(285) - p(409)
p(517) = p(1)*p(295) - p(432) - p(431)
p(518) = p(1)*p(296) - p(434)
p(519) = p(3)*p(185) - p(438)
p(520) = p(39)*p(46) - p(440)
p(521) = p(1)*p(298) - p(439) - p(520)
p(522) = p(1)*p(299) - p(440)
p(523) = p(1)*p(300) - p(441) - p(521)
p(524) = p(21)*p(84) - p(438)
p(525) = p(6)*p(183) - p(428)
p(526) = p(1)*p(303) - p(446) - p(444)
p(527) = p(8)*p(150) - p(428)
p(528) = p(41)*p(45) - p(436)
p(529) = p(1)*p(306) - p(456) - p(454)
p(530) = p(2)*p(296) - p(436)
p(531) = p(4)*p(169) - p(388) - p(478)
p(532) = p(1)*p(310) - p(465)
p(533) = p(14)*p(82)
p(534) = p(3)*p(188) - p(468)
p(535) = p(2)*p(298) - p(447) - p(443)
p(536) = p(2)*p(299) - p(448)
p(537) = p(1)*p(313) - p(469) - p(535)
p(538) = p(7)*p(173) - p(468)
p(539) = p(6)*p(185) - p(451)
p(540) = p(1)*p(316) - p(473) - p(471)
p(541) = p(22)*p(73) - p(451)
p(542) = p(43)*p(45) - p(437)
p(543) = p(1)*p(319) - p(480) - p(478)
p(544) = p(8)*p(160) - p(437)
p(545) = p(14)*p(88)
p(546) = p(4)*p(187) - p(467)
p(547) = p(2)*p(309) - p(466) - p(546)
p(548) = p(2)*p(310) - p(467)
p(549) = p(14)*p(97)
p(550) = p(3)*p(191) - p(484)
p(551) = p(2)*p(313) - p(474) - p(471)
p(552) = p(1)*p(323) - p(484)
p(553) = p(6)*p(188) - p(476)
p(554) = p(1)*p(325) - p(487) - p(485)
p(555) = p(32)*p(50) - p(476)
p(556) = p(45)*p(46) - p(462)
p(557) = p(2)*p(319) - p(482) - p(547)
p(558) = p(22)*p(81) - p(462)
p(559) = p(14)*p(100)
p(560) = p(2)*p(322) - p(486) - p(485)
p(561) = p(2)*p(323) - p(487)
p(562) = p(6)*p(191) - p(488)
p(563) = p(2)*p(325) - p(489) - p(557)
p(564) = p(13)*p(102) - p(488)
p(565) = p(14)*p(102)
p(566) = p(3)*p(178) - p(492)
p(567) = p(7)*p(178) - p(490)
p(568) = p(39)*p(49) - p(491)
p(569) = p(3)*p(194) - p(494)
p(570) = p(1)*p(331) - p(495)
p(571) = p(2)*p(346) - p(570)
p(572) = p(1)*p(333) - p(506)
p(573) = p(8)*p(194) - p(572)
p(574) = p(1)*p(335) - p(519)
p(575) = p(22)*p(101) - p(574)
p(576) = p(1)*p(337) - p(534)
p(577) = p(49)*p(50) - p(576)
p(578) = p(6)*p(187) - p(549)
p(579) = p(1)*p(340) - p(550)
p(580) = p(21)*p(102) - p(579)
p(581) = p(8)*p(187) - p(533)
p(582) = p(1)*p(343) - p(560)
p(583) = p(7)*p(195) - p(582)
p(584) = p(45)*p(50) - p(545)
p(585) = p(2)*p(343) - p(562)
p(586) = p(1)*p(347) - p(585)
p(587) = p(6)*p(195) - p(565)
p(588) = p(1)*p(346) - p(569)
p(589) = p(2)*p(347) - p(587)
p(590) = p(9)*p(103)
p(591) = p(9)*p(106)
p(592) = p(9)*p(119)
p(593) = p(9)*p(128)
p(594) = p(9)*p(108)
p(595) = p(9)*p(109)
p(596) = p(9)*p(129)
p(597) = p(9)*p(132)
p(598) = p(9)*p(133)
p(599) = p(9)*p(161)
p(600) = p(9)*p(138)
p(601) = p(9)*p(139)
p(602) = p(9)*p(171)
p(603) = p(9)*p(177)
p(604) = p(9)*p(114)
p(605) = p(9)*p(115)
p(606) = p(9)*p(149)
p(607) = p(9)*p(104)
p(608) = p(9)*p(116)
p(609) = p(9)*p(105)
p(610) = p(9)*p(117)
p(611) = p(9)*p(118)
p(612) = p(9)*p(150)
p(613) = p(14)*p(178)
p(614) = p(14)*p(179)
p(615) = p(14)*p(180)
p(616) = p(9)*p(156)
p(617) = p(9)*p(122)
p(618) = p(9)*p(123)
p(619) = p(9)*p(157)
p(620) = p(9)*p(124)
p(621) = p(9)*p(107)
p(622) = p(9)*p(125)
p(623) = p(9)*p(126)
p(624) = p(9)*p(127)
p(625) = p(9)*p(158)
p(626) = p(14)*p(143)
p(627) = p(14)*p(144)
p(628) = p(14)*p(145)
p(629) = p(14)*p(146)
p(630) = p(14)*p(147)
p(631) = p(14)*p(148)
p(632) = p(9)*p(130)
p(633) = p(9)*p(159)
p(634) = p(9)*p(131)
p(635) = p(9)*p(160)
p(636) = p(14)*p(114)
p(637) = p(14)*p(115)
p(638) = p(14)*p(116)
p(639) = p(14)*p(117)
p(640) = p(14)*p(118)
p(641) = p(14)*p(106)
p(642) = p(9)*p(165)
p(643) = p(9)*p(135)
p(644) = p(9)*p(166)
p(645) = p(9)*p(136)
p(646) = p(9)*p(137)
p(647) = p(9)*p(167)
p(648) = p(14)*p(151)
p(649) = p(14)*p(152)
p(650) = p(14)*p(153)
p(651) = p(14)*p(154)
p(652) = p(14)*p(155)
p(653) = p(9)*p(168)
p(654) = p(9)*p(169)
p(655) = p(9)*p(170)
p(656) = p(14)*p(122)
p(657) = p(14)*p(123)
p(658) = p(14)*p(124)
p(659) = p(14)*p(125)
p(660) = p(14)*p(126)
p(661) = p(14)*p(127)
p(662) = p(14)*p(108)
p(663) = p(14)*p(109)
p(664) = p(9)*p(187)
p(665) = p(14)*p(130)
p(666) = p(14)*p(131)
p(667) = p(9)*p(174)
p(668) = p(9)*p(175)
p(669) = p(9)*p(176)
p(670) = p(14)*p(162)
p(671) = p(14)*p(163)
p(672) = p(14)*p(164)
p(673) = p(9)*p(190)
p(674) = p(14)*p(135)
p(675) = p(14)*p(136)
p(676) = p(14)*p(137)
p(677) = p(9)*p(193)
p(678) = p(14)*p(172)
p(679) = p(14)*p(173)
p(680) = p(9)*p(181)
p(681) = p(9)*p(143)
p(682) = p(9)*p(110)
p(683) = p(9)*p(144)
p(684) = p(9)*p(111)
p(685) = p(9)*p(112)
p(686) = p(9)*p(145)
p(687) = p(9)*p(113)
p(688) = p(9)*p(146)
p(689) = p(9)*p(147)
p(690) = p(9)*p(148)
p(691) = p(9)*p(182)
p(692) = p(1)*p(401) - p(604)
p(693) = p(21)*p(114) - p(609)
p(694) = p(24)*p(101) - p(612)
p(695) = p(3)*p(284) - p(606) - p(694)
p(696) = p(1)*p(404) - p(606) - p(695)
p(697) = p(1)*p(405) - p(608) - p(696)
p(698) = p(13)*p(178) - p(604)
p(699) = p(39)*p(73) - p(609)
p(700) = p(3)*p(285) - p(612)
p(701) = p(14)*p(194)
p(702) = p(9)*p(183)
p(703) = p(9)*p(151)
p(704) = p(9)*p(120)
p(705) = p(9)*p(152)
p(706) = p(9)*p(121)
p(707) = p(9)*p(153)
p(708) = p(9)*p(154)
p(709) = p(9)*p(155)
p(710) = p(9)*p(184)
p(711) = p(24)*p(91) - p(592)
p(712) = p(1)*p(416) - p(616) - p(711)
p(713) = p(2)*p(401) - p(613)
p(714) = p(1)*p(418) - p(618) - p(713)
p(715) = p(1)*p(419) - p(619) - p(714)
p(716) = p(13)*p(181) - p(701)
p(717) = p(2)*p(404) - p(615) - p(716)
p(718) = p(32)*p(91) - p(701)
p(719) = p(1)*p(421) - p(619) - p(717)
p(720) = p(41)*p(73) - p(701)
p(721) = p(6)*p(279) - p(613)
p(722) = p(16)*p(150) - p(701)
p(723) = p(1)*p(425) - p(624) - p(721)
p(724) = p(4)*p(285) - p(701)
p(725) = p(11)*p(150) - p(592)
p(726) = p(14)*p(181)
p(727) = p(14)*p(182)
p(728) = p(1)*p(430) - p(632)
p(729) = p(21)*p(130) - p(635)
p(730) = p(3)*p(295) - p(633) - p(729)
p(731) = p(1)*p(432) - p(633) - p(730)
p(732) = p(39)*p(81) - p(632)
p(733) = p(3)*p(296) - p(635)
p(734) = p(14)*p(149)
p(735) = p(14)*p(150)
p(736) = p(14)*p(119)
p(737) = p(9)*p(185)
p(738) = p(9)*p(162)
p(739) = p(9)*p(134)
p(740) = p(9)*p(163)
p(741) = p(9)*p(164)
p(742) = p(9)*p(186)
p(743) = p(24)*p(93) - p(593)
p(744) = p(18)*p(114) - p(595)
p(745) = p(1)*p(443) - p(643) - p(744)
p(746) = p(1)*p(444) - p(644) - p(745)
p(747) = p(13)*p(183) - p(726)
p(748) = p(2)*p(421) - p(631) - p(730)
p(749) = p(32)*p(93) - p(726)
p(750) = p(1)*p(446) - p(644) - p(748)
p(751) = p(43)*p(73) - p(726)
p(752) = p(13)*p(153) - p(595)
p(753) = p(18)*p(150) - p(726)
p(754) = p(34)*p(73) - p(593)
p(755) = p(14)*p(183)
p(756) = p(14)*p(184)
p(757) = p(16)*p(130) - p(598)
p(758) = p(2)*p(430) - p(636)
p(759) = p(1)*p(454) - p(654) - p(758)
p(760) = p(41)*p(81) - p(735)
p(761) = p(2)*p(432) - p(638) - p(760)
p(762) = p(16)*p(160) - p(735)
p(763) = p(45)*p(69) - p(636)
p(764) = p(4)*p(296) - p(735)
p(765) = p(11)*p(160) - p(598)
p(766) = p(14)*p(156)
p(767) = p(14)*p(157)
p(768) = p(14)*p(158)
p(769) = p(14)*p(128)
p(770) = p(14)*p(129)
p(771) = p(1)*p(464) - p(664)
p(772) = p(4)*p(262) - p(673) - p(661)
p(773) = p(3)*p(310) - p(664)
p(774) = p(14)*p(159)
p(775) = p(14)*p(160)
p(776) = p(14)*p(132)
p(777) = p(14)*p(133)
p(778) = p(9)*p(188)
p(779) = p(9)*p(172)
p(780) = p(9)*p(173)
p(781) = p(9)*p(189)
p(782) = p(24)*p(95) - p(600)
p(783) = p(22)*p(114) - p(650)
p(784) = p(1)*p(471) - p(668) - p(783)
p(785) = p(13)*p(185) - p(755)
p(786) = p(2)*p(446) - p(652) - p(761)
p(787) = p(32)*p(95) - p(755)
p(788) = p(6)*p(299) - p(650)
p(789) = p(46)*p(73) - p(755)
p(790) = p(32)*p(84) - p(600)
p(791) = p(14)*p(185)
p(792) = p(14)*p(186)
p(793) = p(2)*p(453) - p(656) - p(771)
p(794) = p(1)*p(478) - p(673) - p(793)
p(795) = p(43)*p(81) - p(736)
p(796) = p(2)*p(456) - p(659) - p(772)
p(797) = p(18)*p(160) - p(736)
p(798) = p(1)*p(480) - p(673) - p(796)
p(799) = p(14)*p(165)
p(800) = p(14)*p(166)
p(801) = p(14)*p(167)
p(802) = p(14)*p(138)
p(803) = p(14)*p(139)
p(804) = p(2)*p(464) - p(665)
p(805) = p(4)*p(310) - p(775)
p(806) = p(11)*p(187) - p(665)
p(807) = p(14)*p(168)
p(808) = p(14)*p(169)
p(809) = p(14)*p(170)
p(810) = p(9)*p(191)
p(811) = p(9)*p(192)
p(812) = p(24)*p(98) - p(603)
p(813) = p(1)*p(485) - p(677) - p(812)
p(814) = p(13)*p(188) - p(791)
p(815) = p(2)*p(473) - p(672) - p(796)
p(816) = p(32)*p(98) - p(791)
p(817) = p(13)*p(173) - p(603)
p(818) = p(14)*p(188)
p(819) = p(14)*p(189)
p(820) = p(22)*p(130) - p(663)
p(821) = p(46)*p(81) - p(769)
p(822) = p(45)*p(84) - p(663)
p(823) = p(14)*p(174)
p(824) = p(14)*p(175)
p(825) = p(14)*p(176)
p(826) = p(9)*p(195)
p(827) = p(24)*p(102) - p(679)
p(828) = p(13)*p(191) - p(818)
p(829) = p(6)*p(323) - p(679)
p(830) = p(14)*p(191)
p(831) = p(14)*p(192)
p(832) = p(9)*p(140)
p(833) = p(9)*p(141)
p(834) = p(9)*p(178)
p(835) = p(9)*p(142)
p(836) = p(9)*p(179)
p(837) = p(9)*p(180)
p(838) = p(9)*p(194)
p(839) = p(3)*p(331) - p(680)
p(840) = p(39)*p(91) - p(682)
p(841) = p(16)*p(178) - p(685)
p(842) = p(1)*p(497) - p(683) - p(841)
p(843) = p(2)*p(566) - p(842)
p(844) = p(1)*p(498) - p(686) - p(842)
p(845) = p(7)*p(279) - p(685)
p(846) = p(1)*p(500) - p(689) - p(844)
p(847) = p(49)*p(69) - p(682)
p(848) = p(1)*p(502) - p(691) - p(846)
p(849) = p(11)*p(194) - p(680)
p(850) = p(1)*p(504) - p(695) - p(694)
p(851) = p(1)*p(505) - p(700)
p(852) = p(3)*p(333) - p(702)
p(853) = p(39)*p(93) - p(704)
p(854) = p(18)*p(178) - p(707)
p(855) = p(1)*p(508) - p(705) - p(854)
p(856) = p(1)*p(509) - p(707)
p(857) = p(1)*p(510) - p(708) - p(855)
p(858) = p(21)*p(153) - p(704)
p(859) = p(1)*p(512) - p(710) - p(857)
p(860) = p(34)*p(101) - p(702)
p(861) = p(6)*p(331) - p(701)
p(862) = p(1)*p(515) - p(717) - p(715)
p(863) = p(2)*p(505) - p(701)
p(864) = p(1)*p(517) - p(730) - p(729)
p(865) = p(1)*p(518) - p(733)
p(866) = p(3)*p(335) - p(737)
p(867) = p(39)*p(95) - p(739)
p(868) = p(1)*p(520) - p(738) - p(867)
p(869) = p(2)*p(509) - p(721)
p(870) = p(1)*p(521) - p(740) - p(868)
p(871) = p(7)*p(299) - p(739)
p(872) = p(1)*p(523) - p(742) - p(870)
p(873) = p(49)*p(84) - p(737)
p(874) = p(6)*p(333) - p(726)
p(875) = p(1)*p(526) - p(748) - p(746)
p(876) = p(8)*p(285) - p(726)
p(877) = p(45)*p(91) - p(735)
p(878) = p(1)*p(529) - p(761) - p(759)
p(879) = p(2)*p(518) - p(735)
p(880) = p(1)*p(531) - p(772) - p(771)
p(881) = p(1)*p(532) - p(773)
p(882) = p(14)*p(161)
p(883) = p(3)*p(337) - p(778)
p(884) = p(39)*p(98) - p(780)
p(885) = p(1)*p(535) - p(779) - p(884)
p(886) = p(1)*p(536) - p(780)
p(887) = p(1)*p(537) - p(781) - p(885)
p(888) = p(21)*p(173) - p(778)
p(889) = p(6)*p(335) - p(755)
p(890) = p(1)*p(540) - p(786) - p(784)
p(891) = p(22)*p(150) - p(755)
p(892) = p(45)*p(93) - p(736)
p(893) = p(1)*p(543) - p(796) - p(794)
p(894) = p(8)*p(296) - p(736)
p(895) = p(16)*p(187) - p(775)
p(896) = p(1)*p(547) - p(806) - p(804)
p(897) = p(2)*p(532) - p(775)
p(898) = p(14)*p(171)
p(899) = p(6)*p(309) - p(808) - p(807)
p(900) = p(1)*p(578) - p(899)
p(901) = p(14)*p(187)
p(902) = p(3)*p(340) - p(810)
p(903) = p(2)*p(535) - p(787) - p(783)
p(904) = p(2)*p(536) - p(788)
p(905) = p(1)*p(551) - p(811) - p(903)
p(906) = p(7)*p(323) - p(810)
p(907) = p(6)*p(337) - p(791)
p(908) = p(1)*p(554) - p(815) - p(813)
p(909) = p(50)*p(73) - p(791)
p(910) = p(45)*p(95) - p(769)
p(911) = p(1)*p(557) - p(822) - p(820)
p(912) = p(22)*p(160) - p(769)
p(913) = p(14)*p(177)
p(914) = p(18)*p(187) - p(777)
p(915) = p(2)*p(547) - p(808) - p(899)
p(916) = p(8)*p(310) - p(777)
p(917) = p(14)*p(190)
p(918) = p(3)*p(343) - p(826)
p(919) = p(2)*p(551) - p(816) - p(813)
p(920) = p(1)*p(561) - p(826)
p(921) = p(6)*p(340) - p(818)
p(922) = p(1)*p(563) - p(829) - p(827)
p(923) = p(32)*p(102) - p(818)
p(924) = p(45)*p(98) - p(802)
p(925) = p(2)*p(557) - p(824) - p(915)
p(926) = p(50)*p(81) - p(802)
p(927) = p(14)*p(193)
p(928) = p(2)*p(560) - p(828) - p(827)
p(929) = p(2)*p(561) - p(829)
p(930) = p(6)*p(343) - p(830)
p(931) = p(2)*p(563) - p(831) - p(925)
p(932) = p(13)*p(195) - p(830)
p(933) = p(14)*p(195)
p(934) = p(1)*p(566) - p(834)
p(935) = p(1)*p(567) - p(836) - p(934)
p(936) = p(39)*p(101) - p(835)
p(937) = p(3)*p(346) - p(838)
p(938) = p(1)*p(570) - p(839)
p(939) = p(2)*p(588) - p(938)
p(940) = p(1)*p(572) - p(852)
p(941) = p(8)*p(346) - p(940)
p(942) = p(1)*p(574) - p(866)
p(943) = p(22)*p(194) - p(942)
p(944) = p(1)*p(576) - p(883)
p(945) = p(50)*p(101) - p(944)
p(946) = p(1)*p(579) - p(902)
p(947) = p(49)*p(102) - p(946)
p(948) = p(2)*p(578) - p(901)
p(949) = p(1)*p(582) - p(918)
p(950) = p(21)*p(195) - p(949)
p(951) = p(2)*p(581) - p(917) - p(948)
p(952) = p(1)*p(585) - p(928)
p(953) = p(7)*p(347) - p(952)
p(954) = p(45)*p(102) - p(913)
p(955) = p(2)*p(585) - p(930)
p(956) = p(1)*p(589) - p(955)
p(957) = p(6)*p(347) - p(933)
p(958) = p(1)*p(588) - p(937)
p(959) = p(2)*p(589) - p(957)
p(960) = p(9)*p(196)
p(961) = p(9)*p(202)
p(962) = p(9)*p(203)
p(963) = p(9)*p(228)
p(964) = p(9)*p(242)
p(965) = p(9)*p(207)
p(966) = p(9)*p(197)
p(967) = p(9)*p(208)
p(968) = p(9)*p(209)
p(969) = p(9)*p(243)
p(970) = p(9)*p(211)
p(971) = p(9)*p(247)
p(972) = p(9)*p(212)
p(973) = p(9)*p(248)
p(974) = p(9)*p(249)
p(975) = p(9)*p(258)
p(976) = p(9)*p(214)
p(977) = p(9)*p(215)
p(978) = p(9)*p(259)
p(979) = p(9)*p(263)
p(980) = p(9)*p(264)
p(981) = p(9)*p(265)
p(982) = p(9)*p(308)
p(983) = p(9)*p(311)
p(984) = p(9)*p(270)
p(985) = p(9)*p(271)
p(986) = p(9)*p(321)
p(987) = p(9)*p(327)
p(988) = p(9)*p(222)
p(989) = p(9)*p(223)
p(990) = p(9)*p(284)
p(991) = p(9)*p(198)
p(992) = p(9)*p(199)
p(993) = p(9)*p(224)
p(994) = p(9)*p(200)
p(995) = p(9)*p(225)
p(996) = p(9)*p(201)
p(997) = p(9)*p(226)
p(998) = p(9)*p(227)
p(999) = p(9)*p(285)
p(1000) = p(14)*p(328)
p(1001) = p(14)*p(329)
p(1002) = p(14)*p(330)
p(1003) = p(9)*p(292)
p(1004) = p(9)*p(233)
p(1005) = p(9)*p(234)
p(1006) = p(9)*p(235)
p(1007) = p(9)*p(293)
p(1008) = p(9)*p(236)
p(1009) = p(9)*p(204)
p(1010) = p(9)*p(205)
p(1011) = p(9)*p(237)
p(1012) = p(9)*p(238)
p(1013) = p(9)*p(206)
p(1014) = p(9)*p(239)
p(1015) = p(9)*p(240)
p(1016) = p(9)*p(241)
p(1017) = p(9)*p(294)
p(1018) = p(14)*p(276)
p(1019) = p(14)*p(277)
p(1020) = p(14)*p(278)
p(1021) = p(14)*p(279)
p(1022) = p(14)*p(280)
p(1023) = p(14)*p(281)
p(1024) = p(14)*p(282)
p(1025) = p(14)*p(283)
p(1026) = p(9)*p(244)
p(1027) = p(9)*p(295)
p(1028) = p(9)*p(210)
p(1029) = p(9)*p(245)
p(1030) = p(9)*p(246)
p(1031) = p(9)*p(296)
p(1032) = p(14)*p(222)
p(1033) = p(14)*p(223)
p(1034) = p(14)*p(224)
p(1035) = p(14)*p(225)
p(1036) = p(14)*p(226)
p(1037) = p(14)*p(227)
p(1038) = p(14)*p(202)
p(1039) = p(14)*p(203)
p(1040) = p(9)*p(302)
p(1041) = p(9)*p(252)
p(1042) = p(9)*p(253)
p(1043) = p(9)*p(303)
p(1044) = p(9)*p(254)
p(1045) = p(9)*p(213)
p(1046) = p(9)*p(255)
p(1047) = p(9)*p(256)
p(1048) = p(9)*p(257)
p(1049) = p(9)*p(304)
p(1050) = p(14)*p(286)
p(1051) = p(14)*p(287)
p(1052) = p(14)*p(288)
p(1053) = p(14)*p(289)
p(1054) = p(14)*p(290)
p(1055) = p(14)*p(291)
p(1056) = p(9)*p(305)
p(1057) = p(9)*p(260)
p(1058) = p(9)*p(306)
p(1059) = p(9)*p(261)
p(1060) = p(9)*p(262)
p(1061) = p(9)*p(307)
p(1062) = p(14)*p(233)
p(1063) = p(14)*p(234)
p(1064) = p(14)*p(235)
p(1065) = p(14)*p(236)
p(1066) = p(14)*p(237)
p(1067) = p(14)*p(238)
p(1068) = p(14)*p(239)
p(1069) = p(14)*p(240)
p(1070) = p(14)*p(241)
p(1071) = p(14)*p(207)
p(1072) = p(14)*p(208)
p(1073) = p(14)*p(209)
p(1074) = p(9)*p(309)
p(1075) = p(9)*p(310)
p(1076) = p(14)*p(244)
p(1077) = p(14)*p(245)
p(1078) = p(14)*p(246)
p(1079) = p(14)*p(211)
p(1080) = p(14)*p(212)
p(1081) = p(9)*p(315)
p(1082) = p(9)*p(267)
p(1083) = p(9)*p(316)
p(1084) = p(9)*p(268)
p(1085) = p(9)*p(269)
p(1086) = p(9)*p(317)
p(1087) = p(14)*p(297)
p(1088) = p(14)*p(298)
p(1089) = p(14)*p(299)
p(1090) = p(14)*p(300)
p(1091) = p(14)*p(301)
p(1092) = p(9)*p(318)
p(1093) = p(9)*p(319)
p(1094) = p(9)*p(320)
p(1095) = p(14)*p(252)
p(1096) = p(14)*p(253)
p(1097) = p(14)*p(254)
p(1098) = p(14)*p(255)
p(1099) = p(14)*p(256)
p(1100) = p(14)*p(257)
p(1101) = p(14)*p(214)
p(1102) = p(14)*p(215)
p(1103) = p(9)*p(339)
p(1104) = p(14)*p(260)
p(1105) = p(14)*p(261)
p(1106) = p(14)*p(262)
p(1107) = p(9)*p(324)
p(1108) = p(9)*p(325)
p(1109) = p(9)*p(326)
p(1110) = p(14)*p(312)
p(1111) = p(14)*p(313)
p(1112) = p(14)*p(314)
p(1113) = p(9)*p(342)
p(1114) = p(14)*p(267)
p(1115) = p(14)*p(268)
p(1116) = p(14)*p(269)
p(1117) = p(9)*p(345)
p(1118) = p(14)*p(322)
p(1119) = p(14)*p(323)
p(1120) = p(9)*p(331)
p(1121) = p(9)*p(276)
p(1122) = p(9)*p(216)
p(1123) = p(9)*p(277)
p(1124) = p(9)*p(217)
p(1125) = p(9)*p(218)
p(1126) = p(9)*p(278)
p(1127) = p(9)*p(219)
p(1128) = p(9)*p(220)
p(1129) = p(9)*p(279)
p(1130) = p(9)*p(280)
p(1131) = p(9)*p(221)
p(1132) = p(9)*p(281)
p(1133) = p(9)*p(282)
p(1134) = p(9)*p(283)
p(1135) = p(9)*p(332)
p(1136) = p(3)*p(401) - p(988)
p(1137) = p(7)*p(401) - p(991)
p(1138) = p(49)*p(114) - p(996)
p(1139) = p(24)*p(194) - p(999)
p(1140) = p(3)*p(504) - p(990) - p(1139)
p(1141) = p(1)*p(695) - p(990) - p(1140)
p(1142) = p(1)*p(696) - p(993) - p(1141)
p(1143) = p(6)*p(566) - p(1136)
p(1144) = p(32)*p(178) - p(991)
p(1145) = p(39)*p(150) - p(996)
p(1146) = p(3)*p(505) - p(999)
p(1147) = p(14)*p(346)
p(1148) = p(9)*p(333)
p(1149) = p(9)*p(286)
p(1150) = p(9)*p(229)
p(1151) = p(9)*p(287)
p(1152) = p(9)*p(230)
p(1153) = p(9)*p(231)
p(1154) = p(9)*p(288)
p(1155) = p(9)*p(232)
p(1156) = p(9)*p(289)
p(1157) = p(9)*p(290)
p(1158) = p(9)*p(291)
p(1159) = p(9)*p(334)
p(1160) = p(24)*p(181) - p(963)
p(1161) = p(41)*p(114) - p(960)
p(1162) = p(4)*p(401) - p(961)
p(1163) = p(1)*p(713) - p(1005) - p(1162)
p(1164) = p(1)*p(714) - p(1006) - p(1163)
p(1165) = p(1)*p(715) - p(1007) - p(1164)
p(1166) = p(13)*p(331) - p(1147)
p(1167) = p(2)*p(695) - p(1002) - p(1166)
p(1168) = p(32)*p(181) - p(1147)
p(1169) = p(1)*p(717) - p(1007) - p(1167)
p(1170) = p(73)*p(91) - p(1147)
p(1171) = p(1)*p(719) - p(1011) - p(1169)
p(1172) = p(41)*p(150) - p(1147)
p(1173) = p(13)*p(279) - p(961)
p(1174) = p(16)*p(285) - p(1147)
p(1175) = p(69)*p(73) - p(960)
p(1176) = p(4)*p(505) - p(1147)
p(1177) = p(11)*p(285) - p(963)
p(1178) = p(14)*p(331)
p(1179) = p(14)*p(332)
p(1180) = p(3)*p(430) - p(1026)
p(1181) = p(7)*p(430) - p(1028)
p(1182) = p(49)*p(130) - p(1031)
p(1183) = p(3)*p(517) - p(1027) - p(1182)
p(1184) = p(1)*p(730) - p(1027) - p(1183)
p(1185) = p(45)*p(178) - p(1180)
p(1186) = p(39)*p(160) - p(1028)
p(1187) = p(3)*p(518) - p(1031)
p(1188) = p(14)*p(284)
p(1189) = p(14)*p(285)
p(1190) = p(14)*p(228)
p(1191) = p(9)*p(335)
p(1192) = p(9)*p(297)
p(1193) = p(9)*p(250)
p(1194) = p(9)*p(298)
p(1195) = p(9)*p(251)
p(1196) = p(9)*p(299)
p(1197) = p(9)*p(300)
p(1198) = p(9)*p(301)
p(1199) = p(9)*p(336)
p(1200) = p(24)*p(183) - p(964)
p(1201) = p(43)*p(114) - p(966)
p(1202) = p(8)*p(401) - p(1021)
p(1203) = p(1)*p(745) - p(1042) - p(1202)
p(1204) = p(1)*p(746) - p(1043) - p(1203)
p(1205) = p(13)*p(333) - p(1178)
p(1206) = p(2)*p(717) - p(1025) - p(1183)
p(1207) = p(32)*p(183) - p(1178)
p(1208) = p(1)*p(748) - p(1043) - p(1206)
p(1209) = p(73)*p(93) - p(1178)
p(1210) = p(6)*p(509) - p(1021)
p(1211) = p(43)*p(150) - p(1178)
p(1212) = p(32)*p(153) - p(966)
p(1213) = p(18)*p(285) - p(1178)
p(1214) = p(34)*p(150) - p(964)
p(1215) = p(14)*p(333)
p(1216) = p(14)*p(334)
p(1217) = p(41)*p(130) - p(973)
p(1218) = p(4)*p(430) - p(970)
p(1219) = p(1)*p(758) - p(1057) - p(1218)
p(1220) = p(1)*p(759) - p(1058) - p(1219)
p(1221) = p(81)*p(91) - p(1189)
p(1222) = p(2)*p(730) - p(1034) - p(1221)
p(1223) = p(41)*p(160) - p(1189)
p(1224) = p(1)*p(761) - p(1058) - p(1222)
p(1225) = p(16)*p(296) - p(1189)
p(1226) = p(69)*p(81) - p(970)
p(1227) = p(4)*p(518) - p(1189)
p(1228) = p(11)*p(296) - p(973)
p(1229) = p(14)*p(292)
p(1230) = p(14)*p(293)
p(1231) = p(14)*p(294)
p(1232) = p(14)*p(242)
p(1233) = p(14)*p(243)
p(1234) = p(3)*p(464) - p(1074)
p(1235) = p(7)*p(464) - p(1075)
p(1236) = p(3)*p(531) - p(1074) - p(1235)
p(1237) = p(39)*p(187) - p(1234)
p(1238) = p(3)*p(532) - p(1075)
p(1239) = p(14)*p(295)
p(1240) = p(14)*p(296)
p(1241) = p(14)*p(247)
p(1242) = p(14)*p(248)
p(1243) = p(14)*p(249)
p(1244) = p(9)*p(337)
p(1245) = p(9)*p(312)
p(1246) = p(9)*p(266)
p(1247) = p(9)*p(313)
p(1248) = p(9)*p(314)
p(1249) = p(9)*p(338)
p(1250) = p(24)*p(185) - p(975)
p(1251) = p(46)*p(114) - p(977)
p(1252) = p(1)*p(783) - p(1082) - p(1251)
p(1253) = p(1)*p(784) - p(1083) - p(1252)
p(1254) = p(13)*p(335) - p(1215)
p(1255) = p(2)*p(748) - p(1055) - p(1222)
p(1256) = p(32)*p(185) - p(1215)
p(1257) = p(1)*p(786) - p(1083) - p(1255)
p(1258) = p(73)*p(95) - p(1215)
p(1259) = p(13)*p(299) - p(977)
p(1260) = p(46)*p(150) - p(1215)
p(1261) = p(73)*p(84) - p(975)
p(1262) = p(14)*p(335)
p(1263) = p(14)*p(336)
p(1264) = p(43)*p(130) - p(974)
p(1265) = p(8)*p(430) - p(1038)
p(1266) = p(1)*p(794) - p(1093) - p(1265)
p(1267) = p(81)*p(93) - p(1190)
p(1268) = p(2)*p(761) - p(1066) - p(1236)
p(1269) = p(43)*p(160) - p(1190)
p(1270) = p(45)*p(153) - p(1038)
p(1271) = p(18)*p(296) - p(1190)
p(1272) = p(34)*p(160) - p(974)
p(1273) = p(14)*p(302)
p(1274) = p(14)*p(303)
p(1275) = p(14)*p(304)
p(1276) = p(14)*p(258)
p(1277) = p(14)*p(259)
p(1278) = p(4)*p(464) - p(983)
p(1279) = p(1)*p(804) - p(1103) - p(1278)
p(1280) = p(16)*p(310) - p(1240)
p(1281) = p(2)*p(772) - p(1077) - p(1280)
p(1282) = p(4)*p(532) - p(1240)
p(1283) = p(11)*p(310) - p(983)
p(1284) = p(14)*p(305)
p(1285) = p(14)*p(306)
p(1286) = p(14)*p(307)
p(1287) = p(14)*p(263)
p(1288) = p(14)*p(264)
p(1289) = p(14)*p(265)
p(1290) = p(6)*p(464) - p(1104)
p(1291) = p(3)*p(578) - p(1290)
p(1292) = p(14)*p(309)
p(1293) = p(14)*p(310)
p(1294) = p(9)*p(340)
p(1295) = p(9)*p(322)
p(1296) = p(9)*p(323)
p(1297) = p(9)*p(341)
p(1298) = p(24)*p(188) - p(984)
p(1299) = p(50)*p(114) - p(1089)
p(1300) = p(1)*p(813) - p(1108) - p(1299)
p(1301) = p(13)*p(337) - p(1262)
p(1302) = p(2)*p(786) - p(1091) - p(1268)
p(1303) = p(32)*p(188) - p(1262)
p(1304) = p(6)*p(536) - p(1089)
p(1305) = p(73)*p(98) - p(1262)
p(1306) = p(32)*p(173) - p(984)
p(1307) = p(14)*p(337)
p(1308) = p(14)*p(338)
p(1309) = p(46)*p(130) - p(982)
p(1310) = p(1)*p(820) - p(1113) - p(1309)
p(1311) = p(81)*p(95) - p(1232)
p(1312) = p(2)*p(796) - p(1098) - p(1281)
p(1313) = p(46)*p(160) - p(1232)
p(1314) = p(81)*p(84) - p(982)
p(1315) = p(14)*p(315)
p(1316) = p(14)*p(316)
p(1317) = p(14)*p(317)
p(1318) = p(14)*p(270)
p(1319) = p(14)*p(271)
p(1320) = p(8)*p(464) - p(1079)
p(1321) = p(18)*p(310) - p(1242)
p(1322) = p(34)*p(187) - p(1079)
p(1323) = p(14)*p(318)
p(1324) = p(14)*p(319)
p(1325) = p(14)*p(320)
p(1326) = p(9)*p(343)
p(1327) = p(9)*p(344)
p(1328) = p(24)*p(191) - p(987)
p(1329) = p(1)*p(827) - p(1117) - p(1328)
p(1330) = p(13)*p(340) - p(1307)
p(1331) = p(2)*p(815) - p(1112) - p(1312)
p(1332) = p(32)*p(191) - p(1307)
p(1333) = p(13)*p(323) - p(987)
p(1334) = p(14)*p(340)
p(1335) = p(14)*p(341)
p(1336) = p(50)*p(130) - p(1102)
p(1337) = p(81)*p(98) - p(1276)
p(1338) = p(45)*p(173) - p(1102)
p(1339) = p(14)*p(324)
p(1340) = p(14)*p(325)
p(1341) = p(14)*p(326)
p(1342) = p(9)*p(347)
p(1343) = p(24)*p(195) - p(1119)
p(1344) = p(13)*p(343) - p(1334)
p(1345) = p(6)*p(561) - p(1119)
p(1346) = p(14)*p(343)
p(1347) = p(14)*p(344)
p(1348) = p(9)*p(272)
p(1349) = p(9)*p(273)
p(1350) = p(9)*p(274)
p(1351) = p(9)*p(328)
p(1352) = p(9)*p(275)
p(1353) = p(9)*p(329)
p(1354) = p(9)*p(330)
p(1355) = p(9)*p(346)
p(1356) = p(3)*p(570) - p(1120)
p(1357) = p(39)*p(181) - p(1122)
p(1358) = p(41)*p(178) - p(1125)
p(1359) = p(4)*p(566) - p(1129)
p(1360) = p(1)*p(842) - p(1126) - p(1359)
p(1361) = p(1)*p(843) - p(1129)
p(1362) = p(1)*p(844) - p(1130) - p(1360)
p(1363) = p(21)*p(279) - p(1125)
p(1364) = p(1)*p(846) - p(1133) - p(1362)
p(1365) = p(69)*p(101) - p(1122)
p(1366) = p(1)*p(848) - p(1135) - p(1364)
p(1367) = p(11)*p(346) - p(1120)
p(1368) = p(1)*p(850) - p(1140) - p(1139)
p(1369) = p(1)*p(851) - p(1146)
p(1370) = p(3)*p(572) - p(1148)
p(1371) = p(39)*p(183) - p(1150)
p(1372) = p(43)*p(178) - p(1153)
p(1373) = p(1)*p(854) - p(1151) - p(1372)
p(1374) = p(2)*p(843) - p(1143)
p(1375) = p(1)*p(855) - p(1154) - p(1373)
p(1376) = p(7)*p(509) - p(1153)
p(1377) = p(1)*p(857) - p(1157) - p(1375)
p(1378) = p(49)*p(153) - p(1150)
p(1379) = p(1)*p(859) - p(1159) - p(1377)
p(1380) = p(34)*p(194) - p(1148)
p(1381) = p(6)*p(570) - p(1147)
p(1382) = p(1)*p(862) - p(1167) - p(1165)
p(1383) = p(2)*p(851) - p(1147)
p(1384) = p(1)*p(864) - p(1183) - p(1182)
p(1385) = p(1)*p(865) - p(1187)
p(1386) = p(3)*p(574) - p(1191)
p(1387) = p(39)*p(185) - p(1193)
p(1388) = p(46)*p(178) - p(1196)
p(1389) = p(1)*p(868) - p(1194) - p(1388)
p(1390) = p(1)*p(869) - p(1196)
p(1391) = p(1)*p(870) - p(1197) - p(1389)
p(1392) = p(21)*p(299) - p(1193)
p(1393) = p(1)*p(872) - p(1199) - p(1391)
p(1394) = p(84)*p(101) - p(1191)
p(1395) = p(6)*p(572) - p(1178)
p(1396) = p(1)*p(875) - p(1206) - p(1204)
p(1397) = p(8)*p(505) - p(1178)
p(1398) = p(45)*p(181) - p(1189)
p(1399) = p(1)*p(878) - p(1222) - p(1220)
p(1400) = p(2)*p(865) - p(1189)
p(1401) = p(1)*p(880) - p(1236) - p(1235)
p(1402) = p(1)*p(881) - p(1238)
p(1403) = p(3)*p(576) - p(1244)
p(1404) = p(39)*p(188) - p(1246)
p(1405) = p(1)*p(884) - p(1245) - p(1404)
p(1406) = p(2)*p(869) - p(1210)
p(1407) = p(1)*p(885) - p(1247) - p(1405)
p(1408) = p(7)*p(536) - p(1246)
p(1409) = p(1)*p(887) - p(1249) - p(1407)
p(1410) = p(49)*p(173) - p(1244)
p(1411) = p(6)*p(574) - p(1215)
p(1412) = p(1)*p(890) - p(1255) - p(1253)
p(1413) = p(22)*p(285) - p(1215)
p(1414) = p(45)*p(183) - p(1190)
p(1415) = p(1)*p(893) - p(1268) - p(1266)
p(1416) = p(8)*p(518) - p(1190)
p(1417) = p(41)*p(187) - p(1240)
p(1418) = p(1)*p(896) - p(1281) - p(1279)
p(1419) = p(2)*p(881) - p(1240)
p(1420) = p(14)*p(308)
p(1421) = p(4)*p(547) - p(1106) - p(1320)
p(1422) = p(1)*p(900) - p(1291)
p(1423) = p(14)*p(311)
p(1424) = p(3)*p(579) - p(1294)
p(1425) = p(39)*p(191) - p(1296)
p(1426) = p(1)*p(903) - p(1295) - p(1425)
p(1427) = p(1)*p(904) - p(1296)
p(1428) = p(1)*p(905) - p(1297) - p(1426)
p(1429) = p(21)*p(323) - p(1294)
p(1430) = p(6)*p(576) - p(1262)
p(1431) = p(1)*p(908) - p(1302) - p(1300)
p(1432) = p(50)*p(150) - p(1262)
p(1433) = p(45)*p(185) - p(1232)
p(1434) = p(1)*p(911) - p(1312) - p(1310)
p(1435) = p(22)*p(296) - p(1232)
p(1436) = p(43)*p(187) - p(1242)
p(1437) = p(1)*p(915) - p(1322) - p(1320)
p(1438) = p(8)*p(532) - p(1242)
p(1439) = p(14)*p(321)
p(1440) = p(4)*p(578) - p(1293)
p(1441) = p(2)*p(899) - p(1292) - p(1440)
p(1442) = p(2)*p(900) - p(1293)
p(1443) = p(14)*p(339)
p(1444) = p(3)*p(582) - p(1326)
p(1445) = p(2)*p(903) - p(1303) - p(1299)
p(1446) = p(2)*p(904) - p(1304)
p(1447) = p(1)*p(919) - p(1327) - p(1445)
p(1448) = p(7)*p(561) - p(1326)
p(1449) = p(6)*p(579) - p(1307)
p(1450) = p(1)*p(922) - p(1331) - p(1329)
p(1451) = p(73)*p(102) - p(1307)
p(1452) = p(45)*p(188) - p(1276)
p(1453) = p(1)*p(925) - p(1338) - p(1336)
p(1454) = p(50)*p(160) - p(1276)
p(1455) = p(14)*p(327)
p(1456) = p(46)*p(187) - p(1243)
p(1457) = p(2)*p(915) - p(1324) - p(1441)
p(1458) = p(22)*p(310) - p(1243)
p(1459) = p(14)*p(342)
p(1460) = p(3)*p(585) - p(1342)
p(1461) = p(2)*p(919) - p(1332) - p(1329)
p(1462) = p(1)*p(929) - p(1342)
p(1463) = p(6)*p(582) - p(1334)
p(1464) = p(1)*p(931) - p(1345) - p(1343)
p(1465) = p(32)*p(195) - p(1334)
p(1466) = p(45)*p(191) - p(1318)
p(1467) = p(2)*p(925) - p(1340) - p(1457)
p(1468) = p(81)*p(102) - p(1318)
p(1469) = p(14)*p(345)
p(1470) = p(2)*p(928) - p(1344) - p(1343)
p(1471) = p(2)*p(929) - p(1345)
p(1472) = p(6)*p(585) - p(1346)
p(1473) = p(2)*p(931) - p(1347) - p(1467)
p(1474) = p(13)*p(347) - p(1346)
p(1475) = p(14)*p(347)
p(1476) = p(3)*p(566) - p(1351)
p(1477) = p(7)*p(566) - p(1349)
p(1478) = p(49)*p(178) - p(1348)
p(1479) = p(39)*p(194) - p(1352)
p(1480) = p(3)*p(588) - p(1355)
p(1481) = p(1)*p(938) - p(1356)
p(1482) = p(2)*p(958) - p(1481)
p(1483) = p(1)*p(940) - p(1370)
p(1484) = p(8)*p(588) - p(1483)
p(1485) = p(1)*p(942) - p(1386)
p(1486) = p(22)*p(346) - p(1485)
p(1487) = p(1)*p(944) - p(1403)
p(1488) = p(50)*p(194) - p(1487)
p(1489) = p(1)*p(946) - p(1424)
p(1490) = p(101)*p(102) - p(1489)
p(1491) = p(6)*p(578) - p(1443)
p(1492) = p(1)*p(949) - p(1444)
p(1493) = p(49)*p(195) - p(1492)
p(1494) = p(8)*p(578) - p(1423)
p(1495) = p(1)*p(952) - p(1460)
p(1496) = p(21)*p(347) - p(1495)
p(1497) = p(50)*p(187) - p(1420)
p(1498) = p(1)*p(955) - p(1470)
p(1499) = p(7)*p(589) - p(1498)
p(1500) = p(45)*p(195) - p(1455)
p(1501) = p(2)*p(955) - p(1472)
p(1502) = p(1)*p(959) - p(1501)
p(1503) = p(6)*p(589) - p(1475)
p(1504) = p(1)*p(958) - p(1480)
p(1505) = p(2)*p(959) - p(1503)

do j=1,1506
  bm1(j)=p(j-1)
enddo

bm2(1)=bm1(4)
bm2(2)=bm1(10)
bm2(3)=bm1(11)
bm2(4)=bm1(12)
bm2(5)=bm1(13)
bm2(6)=bm1(15)
bm2(7)=bm1(16)
bm2(8)=bm1(24)
bm2(9)=bm1(25)
bm2(10)=bm1(26)
bm2(11)=bm1(27)
bm2(12)=bm1(28)
bm2(13)=bm1(29)
bm2(14)=bm1(30)
bm2(15)=bm1(31)
bm2(16)=bm1(32)
bm2(17)=bm1(34)
bm2(18)=bm1(35)
bm2(19)=bm1(36)
bm2(20)=bm1(37)
bm2(21)=bm1(39)
bm2(22)=bm1(40)
bm2(23)=bm1(41)
bm2(24)=bm1(52)
bm2(25)=bm1(53)
bm2(26)=bm1(54)
bm2(27)=bm1(55)
bm2(28)=bm1(56)
bm2(29)=bm1(57)
bm2(30)=bm1(58)
bm2(31)=bm1(59)
bm2(32)=bm1(60)
bm2(33)=bm1(61)
bm2(34)=bm1(62)
bm2(35)=bm1(63)
bm2(36)=bm1(64)
bm2(37)=bm1(65)
bm2(38)=bm1(66)
bm2(39)=bm1(67)
bm2(40)=bm1(68)
bm2(41)=bm1(69)
bm2(42)=bm1(70)
bm2(43)=bm1(71)
bm2(44)=bm1(72)
bm2(45)=bm1(73)
bm2(46)=bm1(75)
bm2(47)=bm1(76)
bm2(48)=bm1(77)
bm2(49)=bm1(78)
bm2(50)=bm1(79)
bm2(51)=bm1(81)
bm2(52)=bm1(83)
bm2(53)=bm1(84)
bm2(54)=bm1(85)
bm2(55)=bm1(86)
bm2(56)=bm1(87)
bm2(57)=bm1(89)
bm2(58)=bm1(90)
bm2(59)=bm1(91)
bm2(60)=bm1(104)
bm2(61)=bm1(105)
bm2(62)=bm1(106)
bm2(63)=bm1(107)
bm2(64)=bm1(108)
bm2(65)=bm1(109)
bm2(66)=bm1(110)
bm2(67)=bm1(111)
bm2(68)=bm1(112)
bm2(69)=bm1(113)
bm2(70)=bm1(114)
bm2(71)=bm1(115)
bm2(72)=bm1(116)
bm2(73)=bm1(117)
bm2(74)=bm1(118)
bm2(75)=bm1(119)
bm2(76)=bm1(120)
bm2(77)=bm1(121)
bm2(78)=bm1(122)
bm2(79)=bm1(123)
bm2(80)=bm1(124)
bm2(81)=bm1(125)
bm2(82)=bm1(126)
bm2(83)=bm1(127)
bm2(84)=bm1(128)
bm2(85)=bm1(129)
bm2(86)=bm1(130)
bm2(87)=bm1(131)
bm2(88)=bm1(132)
bm2(89)=bm1(133)
bm2(90)=bm1(134)
bm2(91)=bm1(135)
bm2(92)=bm1(136)
bm2(93)=bm1(137)
bm2(94)=bm1(138)
bm2(95)=bm1(139)
bm2(96)=bm1(140)
bm2(97)=bm1(141)
bm2(98)=bm1(142)
bm2(99)=bm1(143)
bm2(100)=bm1(144)
bm2(101)=bm1(145)
bm2(102)=bm1(146)
bm2(103)=bm1(147)
bm2(104)=bm1(148)
bm2(105)=bm1(149)
bm2(106)=bm1(150)
bm2(107)=bm1(152)
bm2(108)=bm1(153)
bm2(109)=bm1(154)
bm2(110)=bm1(155)
bm2(111)=bm1(156)
bm2(112)=bm1(157)
bm2(113)=bm1(158)
bm2(114)=bm1(160)
bm2(115)=bm1(162)
bm2(116)=bm1(163)
bm2(117)=bm1(164)
bm2(118)=bm1(165)
bm2(119)=bm1(166)
bm2(120)=bm1(167)
bm2(121)=bm1(169)
bm2(122)=bm1(170)
bm2(123)=bm1(172)
bm2(124)=bm1(173)
bm2(125)=bm1(174)
bm2(126)=bm1(175)
bm2(127)=bm1(176)
bm2(128)=bm1(178)
bm2(129)=bm1(179)
bm2(130)=bm1(180)
bm2(131)=bm1(181)
bm2(132)=bm1(197)
bm2(133)=bm1(198)
bm2(134)=bm1(199)
bm2(135)=bm1(200)
bm2(136)=bm1(201)
bm2(137)=bm1(202)
bm2(138)=bm1(203)
bm2(139)=bm1(204)
bm2(140)=bm1(205)
bm2(141)=bm1(206)
bm2(142)=bm1(207)
bm2(143)=bm1(208)
bm2(144)=bm1(209)
bm2(145)=bm1(210)
bm2(146)=bm1(211)
bm2(147)=bm1(212)
bm2(148)=bm1(213)
bm2(149)=bm1(214)
bm2(150)=bm1(215)
bm2(151)=bm1(216)
bm2(152)=bm1(217)
bm2(153)=bm1(218)
bm2(154)=bm1(219)
bm2(155)=bm1(220)
bm2(156)=bm1(221)
bm2(157)=bm1(222)
bm2(158)=bm1(223)
bm2(159)=bm1(224)
bm2(160)=bm1(225)
bm2(161)=bm1(226)
bm2(162)=bm1(227)
bm2(163)=bm1(228)
bm2(164)=bm1(229)
bm2(165)=bm1(230)
bm2(166)=bm1(231)
bm2(167)=bm1(232)
bm2(168)=bm1(233)
bm2(169)=bm1(234)
bm2(170)=bm1(235)
bm2(171)=bm1(236)
bm2(172)=bm1(237)
bm2(173)=bm1(238)
bm2(174)=bm1(239)
bm2(175)=bm1(240)
bm2(176)=bm1(241)
bm2(177)=bm1(242)
bm2(178)=bm1(243)
bm2(179)=bm1(244)
bm2(180)=bm1(245)
bm2(181)=bm1(246)
bm2(182)=bm1(247)
bm2(183)=bm1(248)
bm2(184)=bm1(249)
bm2(185)=bm1(250)
bm2(186)=bm1(251)
bm2(187)=bm1(252)
bm2(188)=bm1(253)
bm2(189)=bm1(254)
bm2(190)=bm1(255)
bm2(191)=bm1(256)
bm2(192)=bm1(257)
bm2(193)=bm1(258)
bm2(194)=bm1(259)
bm2(195)=bm1(260)
bm2(196)=bm1(261)
bm2(197)=bm1(262)
bm2(198)=bm1(263)
bm2(199)=bm1(264)
bm2(200)=bm1(265)
bm2(201)=bm1(266)
bm2(202)=bm1(267)
bm2(203)=bm1(268)
bm2(204)=bm1(269)
bm2(205)=bm1(270)
bm2(206)=bm1(271)
bm2(207)=bm1(272)
bm2(208)=bm1(273)
bm2(209)=bm1(274)
bm2(210)=bm1(275)
bm2(211)=bm1(276)
bm2(212)=bm1(277)
bm2(213)=bm1(278)
bm2(214)=bm1(279)
bm2(215)=bm1(280)
bm2(216)=bm1(281)
bm2(217)=bm1(282)
bm2(218)=bm1(283)
bm2(219)=bm1(284)
bm2(220)=bm1(285)
bm2(221)=bm1(287)
bm2(222)=bm1(288)
bm2(223)=bm1(289)
bm2(224)=bm1(290)
bm2(225)=bm1(291)
bm2(226)=bm1(292)
bm2(227)=bm1(293)
bm2(228)=bm1(294)
bm2(229)=bm1(296)
bm2(230)=bm1(298)
bm2(231)=bm1(299)
bm2(232)=bm1(300)
bm2(233)=bm1(301)
bm2(234)=bm1(302)
bm2(235)=bm1(303)
bm2(236)=bm1(304)
bm2(237)=bm1(306)
bm2(238)=bm1(307)
bm2(239)=bm1(309)
bm2(240)=bm1(310)
bm2(241)=bm1(312)
bm2(242)=bm1(313)
bm2(243)=bm1(314)
bm2(244)=bm1(315)
bm2(245)=bm1(316)
bm2(246)=bm1(317)
bm2(247)=bm1(319)
bm2(248)=bm1(320)
bm2(249)=bm1(322)
bm2(250)=bm1(323)
bm2(251)=bm1(324)
bm2(252)=bm1(325)
bm2(253)=bm1(326)
bm2(254)=bm1(328)
bm2(255)=bm1(329)
bm2(256)=bm1(330)
bm2(257)=bm1(331)
bm2(258)=bm1(349)
bm2(259)=bm1(350)
bm2(260)=bm1(351)
bm2(261)=bm1(352)
bm2(262)=bm1(353)
bm2(263)=bm1(354)
bm2(264)=bm1(355)
bm2(265)=bm1(356)
bm2(266)=bm1(357)
bm2(267)=bm1(358)
bm2(268)=bm1(359)
bm2(269)=bm1(360)
bm2(270)=bm1(361)
bm2(271)=bm1(362)
bm2(272)=bm1(363)
bm2(273)=bm1(364)
bm2(274)=bm1(365)
bm2(275)=bm1(366)
bm2(276)=bm1(367)
bm2(277)=bm1(368)
bm2(278)=bm1(369)
bm2(279)=bm1(370)
bm2(280)=bm1(371)
bm2(281)=bm1(372)
bm2(282)=bm1(373)
bm2(283)=bm1(374)
bm2(284)=bm1(375)
bm2(285)=bm1(376)
bm2(286)=bm1(377)
bm2(287)=bm1(378)
bm2(288)=bm1(379)
bm2(289)=bm1(380)
bm2(290)=bm1(381)
bm2(291)=bm1(382)
bm2(292)=bm1(383)
bm2(293)=bm1(384)
bm2(294)=bm1(385)
bm2(295)=bm1(386)
bm2(296)=bm1(387)
bm2(297)=bm1(388)
bm2(298)=bm1(389)
bm2(299)=bm1(390)
bm2(300)=bm1(391)
bm2(301)=bm1(392)
bm2(302)=bm1(393)
bm2(303)=bm1(394)
bm2(304)=bm1(395)
bm2(305)=bm1(396)
bm2(306)=bm1(397)
bm2(307)=bm1(398)
bm2(308)=bm1(399)
bm2(309)=bm1(400)
bm2(310)=bm1(401)
bm2(311)=bm1(402)
bm2(312)=bm1(403)
bm2(313)=bm1(404)
bm2(314)=bm1(405)
bm2(315)=bm1(406)
bm2(316)=bm1(407)
bm2(317)=bm1(408)
bm2(318)=bm1(409)
bm2(319)=bm1(410)
bm2(320)=bm1(411)
bm2(321)=bm1(412)
bm2(322)=bm1(413)
bm2(323)=bm1(414)
bm2(324)=bm1(415)
bm2(325)=bm1(416)
bm2(326)=bm1(417)
bm2(327)=bm1(418)
bm2(328)=bm1(419)
bm2(329)=bm1(420)
bm2(330)=bm1(421)
bm2(331)=bm1(422)
bm2(332)=bm1(423)
bm2(333)=bm1(424)
bm2(334)=bm1(425)
bm2(335)=bm1(426)
bm2(336)=bm1(427)
bm2(337)=bm1(428)
bm2(338)=bm1(429)
bm2(339)=bm1(430)
bm2(340)=bm1(431)
bm2(341)=bm1(432)
bm2(342)=bm1(433)
bm2(343)=bm1(434)
bm2(344)=bm1(435)
bm2(345)=bm1(436)
bm2(346)=bm1(437)
bm2(347)=bm1(438)
bm2(348)=bm1(439)
bm2(349)=bm1(440)
bm2(350)=bm1(441)
bm2(351)=bm1(442)
bm2(352)=bm1(443)
bm2(353)=bm1(444)
bm2(354)=bm1(445)
bm2(355)=bm1(446)
bm2(356)=bm1(447)
bm2(357)=bm1(448)
bm2(358)=bm1(449)
bm2(359)=bm1(450)
bm2(360)=bm1(451)
bm2(361)=bm1(452)
bm2(362)=bm1(453)
bm2(363)=bm1(454)
bm2(364)=bm1(455)
bm2(365)=bm1(456)
bm2(366)=bm1(457)
bm2(367)=bm1(458)
bm2(368)=bm1(459)
bm2(369)=bm1(460)
bm2(370)=bm1(461)
bm2(371)=bm1(462)
bm2(372)=bm1(463)
bm2(373)=bm1(464)
bm2(374)=bm1(465)
bm2(375)=bm1(466)
bm2(376)=bm1(467)
bm2(377)=bm1(468)
bm2(378)=bm1(469)
bm2(379)=bm1(470)
bm2(380)=bm1(471)
bm2(381)=bm1(472)
bm2(382)=bm1(473)
bm2(383)=bm1(474)
bm2(384)=bm1(475)
bm2(385)=bm1(476)
bm2(386)=bm1(477)
bm2(387)=bm1(478)
bm2(388)=bm1(479)
bm2(389)=bm1(480)
bm2(390)=bm1(481)
bm2(391)=bm1(482)
bm2(392)=bm1(483)
bm2(393)=bm1(484)
bm2(394)=bm1(485)
bm2(395)=bm1(486)
bm2(396)=bm1(487)
bm2(397)=bm1(488)
bm2(398)=bm1(489)
bm2(399)=bm1(490)
bm2(400)=bm1(491)
bm2(401)=bm1(492)
bm2(402)=bm1(493)
bm2(403)=bm1(494)
bm2(404)=bm1(495)
bm2(405)=bm1(496)
bm2(406)=bm1(497)
bm2(407)=bm1(498)
bm2(408)=bm1(499)
bm2(409)=bm1(500)
bm2(410)=bm1(501)
bm2(411)=bm1(502)
bm2(412)=bm1(503)
bm2(413)=bm1(504)
bm2(414)=bm1(505)
bm2(415)=bm1(507)
bm2(416)=bm1(508)
bm2(417)=bm1(509)
bm2(418)=bm1(510)
bm2(419)=bm1(511)
bm2(420)=bm1(512)
bm2(421)=bm1(513)
bm2(422)=bm1(514)
bm2(423)=bm1(515)
bm2(424)=bm1(516)
bm2(425)=bm1(518)
bm2(426)=bm1(520)
bm2(427)=bm1(521)
bm2(428)=bm1(522)
bm2(429)=bm1(523)
bm2(430)=bm1(524)
bm2(431)=bm1(525)
bm2(432)=bm1(526)
bm2(433)=bm1(527)
bm2(434)=bm1(529)
bm2(435)=bm1(530)
bm2(436)=bm1(532)
bm2(437)=bm1(534)
bm2(438)=bm1(535)
bm2(439)=bm1(536)
bm2(440)=bm1(537)
bm2(441)=bm1(538)
bm2(442)=bm1(539)
bm2(443)=bm1(540)
bm2(444)=bm1(541)
bm2(445)=bm1(543)
bm2(446)=bm1(544)
bm2(447)=bm1(546)
bm2(448)=bm1(547)
bm2(449)=bm1(548)
bm2(450)=bm1(550)
bm2(451)=bm1(551)
bm2(452)=bm1(552)
bm2(453)=bm1(553)
bm2(454)=bm1(554)
bm2(455)=bm1(555)
bm2(456)=bm1(557)
bm2(457)=bm1(558)
bm2(458)=bm1(560)
bm2(459)=bm1(561)
bm2(460)=bm1(562)
bm2(461)=bm1(563)
bm2(462)=bm1(564)
bm2(463)=bm1(566)
bm2(464)=bm1(567)
bm2(465)=bm1(568)
bm2(466)=bm1(569)
bm2(467)=bm1(570)
bm2(468)=bm1(591)
bm2(469)=bm1(592)
bm2(470)=bm1(593)
bm2(471)=bm1(594)
bm2(472)=bm1(595)
bm2(473)=bm1(596)
bm2(474)=bm1(597)
bm2(475)=bm1(598)
bm2(476)=bm1(599)
bm2(477)=bm1(600)
bm2(478)=bm1(601)
bm2(479)=bm1(602)
bm2(480)=bm1(603)
bm2(481)=bm1(604)
bm2(482)=bm1(605)
bm2(483)=bm1(606)
bm2(484)=bm1(607)
bm2(485)=bm1(608)
bm2(486)=bm1(609)
bm2(487)=bm1(610)
bm2(488)=bm1(611)
bm2(489)=bm1(612)
bm2(490)=bm1(613)
bm2(491)=bm1(614)
bm2(492)=bm1(615)
bm2(493)=bm1(616)
bm2(494)=bm1(617)
bm2(495)=bm1(618)
bm2(496)=bm1(619)
bm2(497)=bm1(620)
bm2(498)=bm1(621)
bm2(499)=bm1(622)
bm2(500)=bm1(623)
bm2(501)=bm1(624)
bm2(502)=bm1(625)
bm2(503)=bm1(626)
bm2(504)=bm1(627)
bm2(505)=bm1(628)
bm2(506)=bm1(629)
bm2(507)=bm1(630)
bm2(508)=bm1(631)
bm2(509)=bm1(632)
bm2(510)=bm1(633)
bm2(511)=bm1(634)
bm2(512)=bm1(635)
bm2(513)=bm1(636)
bm2(514)=bm1(637)
bm2(515)=bm1(638)
bm2(516)=bm1(639)
bm2(517)=bm1(640)
bm2(518)=bm1(641)
bm2(519)=bm1(642)
bm2(520)=bm1(643)
bm2(521)=bm1(644)
bm2(522)=bm1(645)
bm2(523)=bm1(646)
bm2(524)=bm1(647)
bm2(525)=bm1(648)
bm2(526)=bm1(649)
bm2(527)=bm1(650)
bm2(528)=bm1(651)
bm2(529)=bm1(652)
bm2(530)=bm1(653)
bm2(531)=bm1(654)
bm2(532)=bm1(655)
bm2(533)=bm1(656)
bm2(534)=bm1(657)
bm2(535)=bm1(658)
bm2(536)=bm1(659)
bm2(537)=bm1(660)
bm2(538)=bm1(661)
bm2(539)=bm1(662)
bm2(540)=bm1(663)
bm2(541)=bm1(664)
bm2(542)=bm1(665)
bm2(543)=bm1(666)
bm2(544)=bm1(667)
bm2(545)=bm1(668)
bm2(546)=bm1(669)
bm2(547)=bm1(670)
bm2(548)=bm1(671)
bm2(549)=bm1(672)
bm2(550)=bm1(673)
bm2(551)=bm1(674)
bm2(552)=bm1(675)
bm2(553)=bm1(676)
bm2(554)=bm1(677)
bm2(555)=bm1(678)
bm2(556)=bm1(679)
bm2(557)=bm1(680)
bm2(558)=bm1(681)
bm2(559)=bm1(682)
bm2(560)=bm1(683)
bm2(561)=bm1(684)
bm2(562)=bm1(685)
bm2(563)=bm1(686)
bm2(564)=bm1(687)
bm2(565)=bm1(688)
bm2(566)=bm1(689)
bm2(567)=bm1(690)
bm2(568)=bm1(691)
bm2(569)=bm1(692)
bm2(570)=bm1(693)
bm2(571)=bm1(694)
bm2(572)=bm1(695)
bm2(573)=bm1(696)
bm2(574)=bm1(697)
bm2(575)=bm1(698)
bm2(576)=bm1(699)
bm2(577)=bm1(700)
bm2(578)=bm1(701)
bm2(579)=bm1(702)
bm2(580)=bm1(703)
bm2(581)=bm1(704)
bm2(582)=bm1(705)
bm2(583)=bm1(706)
bm2(584)=bm1(707)
bm2(585)=bm1(708)
bm2(586)=bm1(709)
bm2(587)=bm1(710)
bm2(588)=bm1(711)
bm2(589)=bm1(712)
bm2(590)=bm1(713)
bm2(591)=bm1(714)
bm2(592)=bm1(715)
bm2(593)=bm1(716)
bm2(594)=bm1(717)
bm2(595)=bm1(718)
bm2(596)=bm1(719)
bm2(597)=bm1(720)
bm2(598)=bm1(721)
bm2(599)=bm1(722)
bm2(600)=bm1(723)
bm2(601)=bm1(724)
bm2(602)=bm1(725)
bm2(603)=bm1(726)
bm2(604)=bm1(727)
bm2(605)=bm1(728)
bm2(606)=bm1(729)
bm2(607)=bm1(730)
bm2(608)=bm1(731)
bm2(609)=bm1(732)
bm2(610)=bm1(733)
bm2(611)=bm1(734)
bm2(612)=bm1(735)
bm2(613)=bm1(736)
bm2(614)=bm1(737)
bm2(615)=bm1(738)
bm2(616)=bm1(739)
bm2(617)=bm1(740)
bm2(618)=bm1(741)
bm2(619)=bm1(742)
bm2(620)=bm1(743)
bm2(621)=bm1(744)
bm2(622)=bm1(745)
bm2(623)=bm1(746)
bm2(624)=bm1(747)
bm2(625)=bm1(748)
bm2(626)=bm1(749)
bm2(627)=bm1(750)
bm2(628)=bm1(751)
bm2(629)=bm1(752)
bm2(630)=bm1(753)
bm2(631)=bm1(754)
bm2(632)=bm1(755)
bm2(633)=bm1(756)
bm2(634)=bm1(757)
bm2(635)=bm1(758)
bm2(636)=bm1(759)
bm2(637)=bm1(760)
bm2(638)=bm1(761)
bm2(639)=bm1(762)
bm2(640)=bm1(763)
bm2(641)=bm1(764)
bm2(642)=bm1(765)
bm2(643)=bm1(766)
bm2(644)=bm1(767)
bm2(645)=bm1(768)
bm2(646)=bm1(769)
bm2(647)=bm1(770)
bm2(648)=bm1(771)
bm2(649)=bm1(772)
bm2(650)=bm1(773)
bm2(651)=bm1(774)
bm2(652)=bm1(775)
bm2(653)=bm1(776)
bm2(654)=bm1(777)
bm2(655)=bm1(778)
bm2(656)=bm1(779)
bm2(657)=bm1(780)
bm2(658)=bm1(781)
bm2(659)=bm1(782)
bm2(660)=bm1(783)
bm2(661)=bm1(784)
bm2(662)=bm1(785)
bm2(663)=bm1(786)
bm2(664)=bm1(787)
bm2(665)=bm1(788)
bm2(666)=bm1(789)
bm2(667)=bm1(790)
bm2(668)=bm1(791)
bm2(669)=bm1(792)
bm2(670)=bm1(793)
bm2(671)=bm1(794)
bm2(672)=bm1(795)
bm2(673)=bm1(796)
bm2(674)=bm1(797)
bm2(675)=bm1(798)
bm2(676)=bm1(799)
bm2(677)=bm1(800)
bm2(678)=bm1(801)
bm2(679)=bm1(802)
bm2(680)=bm1(803)
bm2(681)=bm1(804)
bm2(682)=bm1(805)
bm2(683)=bm1(806)
bm2(684)=bm1(807)
bm2(685)=bm1(808)
bm2(686)=bm1(809)
bm2(687)=bm1(810)
bm2(688)=bm1(811)
bm2(689)=bm1(812)
bm2(690)=bm1(813)
bm2(691)=bm1(814)
bm2(692)=bm1(815)
bm2(693)=bm1(816)
bm2(694)=bm1(817)
bm2(695)=bm1(818)
bm2(696)=bm1(819)
bm2(697)=bm1(820)
bm2(698)=bm1(821)
bm2(699)=bm1(822)
bm2(700)=bm1(823)
bm2(701)=bm1(824)
bm2(702)=bm1(825)
bm2(703)=bm1(826)
bm2(704)=bm1(827)
bm2(705)=bm1(828)
bm2(706)=bm1(829)
bm2(707)=bm1(830)
bm2(708)=bm1(831)
bm2(709)=bm1(832)
bm2(710)=bm1(833)
bm2(711)=bm1(834)
bm2(712)=bm1(835)
bm2(713)=bm1(836)
bm2(714)=bm1(837)
bm2(715)=bm1(838)
bm2(716)=bm1(839)
bm2(717)=bm1(840)
bm2(718)=bm1(841)
bm2(719)=bm1(842)
bm2(720)=bm1(843)
bm2(721)=bm1(844)
bm2(722)=bm1(845)
bm2(723)=bm1(846)
bm2(724)=bm1(847)
bm2(725)=bm1(848)
bm2(726)=bm1(849)
bm2(727)=bm1(850)
bm2(728)=bm1(851)
bm2(729)=bm1(853)
bm2(730)=bm1(854)
bm2(731)=bm1(855)
bm2(732)=bm1(856)
bm2(733)=bm1(857)
bm2(734)=bm1(858)
bm2(735)=bm1(859)
bm2(736)=bm1(860)
bm2(737)=bm1(861)
bm2(738)=bm1(862)
bm2(739)=bm1(863)
bm2(740)=bm1(865)
bm2(741)=bm1(867)
bm2(742)=bm1(868)
bm2(743)=bm1(869)
bm2(744)=bm1(870)
bm2(745)=bm1(871)
bm2(746)=bm1(872)
bm2(747)=bm1(873)
bm2(748)=bm1(874)
bm2(749)=bm1(875)
bm2(750)=bm1(876)
bm2(751)=bm1(878)
bm2(752)=bm1(879)
bm2(753)=bm1(881)
bm2(754)=bm1(883)
bm2(755)=bm1(884)
bm2(756)=bm1(885)
bm2(757)=bm1(886)
bm2(758)=bm1(887)
bm2(759)=bm1(888)
bm2(760)=bm1(889)
bm2(761)=bm1(890)
bm2(762)=bm1(891)
bm2(763)=bm1(893)
bm2(764)=bm1(894)
bm2(765)=bm1(896)
bm2(766)=bm1(897)
bm2(767)=bm1(899)
bm2(768)=bm1(900)
bm2(769)=bm1(902)
bm2(770)=bm1(903)
bm2(771)=bm1(904)
bm2(772)=bm1(905)
bm2(773)=bm1(906)
bm2(774)=bm1(907)
bm2(775)=bm1(908)
bm2(776)=bm1(909)
bm2(777)=bm1(911)
bm2(778)=bm1(912)
bm2(779)=bm1(914)
bm2(780)=bm1(915)
bm2(781)=bm1(916)
bm2(782)=bm1(918)
bm2(783)=bm1(919)
bm2(784)=bm1(920)
bm2(785)=bm1(921)
bm2(786)=bm1(922)
bm2(787)=bm1(923)
bm2(788)=bm1(925)
bm2(789)=bm1(926)
bm2(790)=bm1(928)
bm2(791)=bm1(929)
bm2(792)=bm1(930)
bm2(793)=bm1(931)
bm2(794)=bm1(932)
bm2(795)=bm1(934)
bm2(796)=bm1(935)
bm2(797)=bm1(936)
bm2(798)=bm1(937)
bm2(799)=bm1(938)
bm2(800)=bm1(961)
bm2(801)=bm1(962)
bm2(802)=bm1(963)
bm2(803)=bm1(964)
bm2(804)=bm1(965)
bm2(805)=bm1(966)
bm2(806)=bm1(967)
bm2(807)=bm1(968)
bm2(808)=bm1(969)
bm2(809)=bm1(970)
bm2(810)=bm1(971)
bm2(811)=bm1(972)
bm2(812)=bm1(973)
bm2(813)=bm1(974)
bm2(814)=bm1(975)
bm2(815)=bm1(976)
bm2(816)=bm1(977)
bm2(817)=bm1(978)
bm2(818)=bm1(979)
bm2(819)=bm1(980)
bm2(820)=bm1(981)
bm2(821)=bm1(982)
bm2(822)=bm1(983)
bm2(823)=bm1(984)
bm2(824)=bm1(985)
bm2(825)=bm1(986)
bm2(826)=bm1(987)
bm2(827)=bm1(988)
bm2(828)=bm1(989)
bm2(829)=bm1(990)
bm2(830)=bm1(991)
bm2(831)=bm1(992)
bm2(832)=bm1(993)
bm2(833)=bm1(994)
bm2(834)=bm1(995)
bm2(835)=bm1(996)
bm2(836)=bm1(997)
bm2(837)=bm1(998)
bm2(838)=bm1(999)
bm2(839)=bm1(1000)
bm2(840)=bm1(1001)
bm2(841)=bm1(1002)
bm2(842)=bm1(1003)
bm2(843)=bm1(1004)
bm2(844)=bm1(1005)
bm2(845)=bm1(1006)
bm2(846)=bm1(1007)
bm2(847)=bm1(1008)
bm2(848)=bm1(1009)
bm2(849)=bm1(1010)
bm2(850)=bm1(1011)
bm2(851)=bm1(1012)
bm2(852)=bm1(1013)
bm2(853)=bm1(1014)
bm2(854)=bm1(1015)
bm2(855)=bm1(1016)
bm2(856)=bm1(1017)
bm2(857)=bm1(1018)
bm2(858)=bm1(1019)
bm2(859)=bm1(1020)
bm2(860)=bm1(1021)
bm2(861)=bm1(1022)
bm2(862)=bm1(1023)
bm2(863)=bm1(1024)
bm2(864)=bm1(1025)
bm2(865)=bm1(1026)
bm2(866)=bm1(1027)
bm2(867)=bm1(1028)
bm2(868)=bm1(1029)
bm2(869)=bm1(1030)
bm2(870)=bm1(1031)
bm2(871)=bm1(1032)
bm2(872)=bm1(1033)
bm2(873)=bm1(1034)
bm2(874)=bm1(1035)
bm2(875)=bm1(1036)
bm2(876)=bm1(1037)
bm2(877)=bm1(1038)
bm2(878)=bm1(1039)
bm2(879)=bm1(1040)
bm2(880)=bm1(1041)
bm2(881)=bm1(1042)
bm2(882)=bm1(1043)
bm2(883)=bm1(1044)
bm2(884)=bm1(1045)
bm2(885)=bm1(1046)
bm2(886)=bm1(1047)
bm2(887)=bm1(1048)
bm2(888)=bm1(1049)
bm2(889)=bm1(1050)
bm2(890)=bm1(1051)
bm2(891)=bm1(1052)
bm2(892)=bm1(1053)
bm2(893)=bm1(1054)
bm2(894)=bm1(1055)
bm2(895)=bm1(1056)
bm2(896)=bm1(1057)
bm2(897)=bm1(1058)
bm2(898)=bm1(1059)
bm2(899)=bm1(1060)
bm2(900)=bm1(1061)
bm2(901)=bm1(1062)
bm2(902)=bm1(1063)
bm2(903)=bm1(1064)
bm2(904)=bm1(1065)
bm2(905)=bm1(1066)
bm2(906)=bm1(1067)
bm2(907)=bm1(1068)
bm2(908)=bm1(1069)
bm2(909)=bm1(1070)
bm2(910)=bm1(1071)
bm2(911)=bm1(1072)
bm2(912)=bm1(1073)
bm2(913)=bm1(1074)
bm2(914)=bm1(1075)
bm2(915)=bm1(1076)
bm2(916)=bm1(1077)
bm2(917)=bm1(1078)
bm2(918)=bm1(1079)
bm2(919)=bm1(1080)
bm2(920)=bm1(1081)
bm2(921)=bm1(1082)
bm2(922)=bm1(1083)
bm2(923)=bm1(1084)
bm2(924)=bm1(1085)
bm2(925)=bm1(1086)
bm2(926)=bm1(1087)
bm2(927)=bm1(1088)
bm2(928)=bm1(1089)
bm2(929)=bm1(1090)
bm2(930)=bm1(1091)
bm2(931)=bm1(1092)
bm2(932)=bm1(1093)
bm2(933)=bm1(1094)
bm2(934)=bm1(1095)
bm2(935)=bm1(1096)
bm2(936)=bm1(1097)
bm2(937)=bm1(1098)
bm2(938)=bm1(1099)
bm2(939)=bm1(1100)
bm2(940)=bm1(1101)
bm2(941)=bm1(1102)
bm2(942)=bm1(1103)
bm2(943)=bm1(1104)
bm2(944)=bm1(1105)
bm2(945)=bm1(1106)
bm2(946)=bm1(1107)
bm2(947)=bm1(1108)
bm2(948)=bm1(1109)
bm2(949)=bm1(1110)
bm2(950)=bm1(1111)
bm2(951)=bm1(1112)
bm2(952)=bm1(1113)
bm2(953)=bm1(1114)
bm2(954)=bm1(1115)
bm2(955)=bm1(1116)
bm2(956)=bm1(1117)
bm2(957)=bm1(1118)
bm2(958)=bm1(1119)
bm2(959)=bm1(1120)
bm2(960)=bm1(1121)
bm2(961)=bm1(1122)
bm2(962)=bm1(1123)
bm2(963)=bm1(1124)
bm2(964)=bm1(1125)
bm2(965)=bm1(1126)
bm2(966)=bm1(1127)
bm2(967)=bm1(1128)
bm2(968)=bm1(1129)
bm2(969)=bm1(1130)
bm2(970)=bm1(1131)
bm2(971)=bm1(1132)
bm2(972)=bm1(1133)
bm2(973)=bm1(1134)
bm2(974)=bm1(1135)
bm2(975)=bm1(1136)
bm2(976)=bm1(1137)
bm2(977)=bm1(1138)
bm2(978)=bm1(1139)
bm2(979)=bm1(1140)
bm2(980)=bm1(1141)
bm2(981)=bm1(1142)
bm2(982)=bm1(1143)
bm2(983)=bm1(1144)
bm2(984)=bm1(1145)
bm2(985)=bm1(1146)
bm2(986)=bm1(1147)
bm2(987)=bm1(1148)
bm2(988)=bm1(1149)
bm2(989)=bm1(1150)
bm2(990)=bm1(1151)
bm2(991)=bm1(1152)
bm2(992)=bm1(1153)
bm2(993)=bm1(1154)
bm2(994)=bm1(1155)
bm2(995)=bm1(1156)
bm2(996)=bm1(1157)
bm2(997)=bm1(1158)
bm2(998)=bm1(1159)
bm2(999)=bm1(1160)
bm2(1000)=bm1(1161)
bm2(1001)=bm1(1162)
bm2(1002)=bm1(1163)
bm2(1003)=bm1(1164)
bm2(1004)=bm1(1165)
bm2(1005)=bm1(1166)
bm2(1006)=bm1(1167)
bm2(1007)=bm1(1168)
bm2(1008)=bm1(1169)
bm2(1009)=bm1(1170)
bm2(1010)=bm1(1171)
bm2(1011)=bm1(1172)
bm2(1012)=bm1(1173)
bm2(1013)=bm1(1174)
bm2(1014)=bm1(1175)
bm2(1015)=bm1(1176)
bm2(1016)=bm1(1177)
bm2(1017)=bm1(1178)
bm2(1018)=bm1(1179)
bm2(1019)=bm1(1180)
bm2(1020)=bm1(1181)
bm2(1021)=bm1(1182)
bm2(1022)=bm1(1183)
bm2(1023)=bm1(1184)
bm2(1024)=bm1(1185)
bm2(1025)=bm1(1186)
bm2(1026)=bm1(1187)
bm2(1027)=bm1(1188)
bm2(1028)=bm1(1189)
bm2(1029)=bm1(1190)
bm2(1030)=bm1(1191)
bm2(1031)=bm1(1192)
bm2(1032)=bm1(1193)
bm2(1033)=bm1(1194)
bm2(1034)=bm1(1195)
bm2(1035)=bm1(1196)
bm2(1036)=bm1(1197)
bm2(1037)=bm1(1198)
bm2(1038)=bm1(1199)
bm2(1039)=bm1(1200)
bm2(1040)=bm1(1201)
bm2(1041)=bm1(1202)
bm2(1042)=bm1(1203)
bm2(1043)=bm1(1204)
bm2(1044)=bm1(1205)
bm2(1045)=bm1(1206)
bm2(1046)=bm1(1207)
bm2(1047)=bm1(1208)
bm2(1048)=bm1(1209)
bm2(1049)=bm1(1210)
bm2(1050)=bm1(1211)
bm2(1051)=bm1(1212)
bm2(1052)=bm1(1213)
bm2(1053)=bm1(1214)
bm2(1054)=bm1(1215)
bm2(1055)=bm1(1216)
bm2(1056)=bm1(1217)
bm2(1057)=bm1(1218)
bm2(1058)=bm1(1219)
bm2(1059)=bm1(1220)
bm2(1060)=bm1(1221)
bm2(1061)=bm1(1222)
bm2(1062)=bm1(1223)
bm2(1063)=bm1(1224)
bm2(1064)=bm1(1225)
bm2(1065)=bm1(1226)
bm2(1066)=bm1(1227)
bm2(1067)=bm1(1228)
bm2(1068)=bm1(1229)
bm2(1069)=bm1(1230)
bm2(1070)=bm1(1231)
bm2(1071)=bm1(1232)
bm2(1072)=bm1(1233)
bm2(1073)=bm1(1234)
bm2(1074)=bm1(1235)
bm2(1075)=bm1(1236)
bm2(1076)=bm1(1237)
bm2(1077)=bm1(1238)
bm2(1078)=bm1(1239)
bm2(1079)=bm1(1240)
bm2(1080)=bm1(1241)
bm2(1081)=bm1(1242)
bm2(1082)=bm1(1243)
bm2(1083)=bm1(1244)
bm2(1084)=bm1(1245)
bm2(1085)=bm1(1246)
bm2(1086)=bm1(1247)
bm2(1087)=bm1(1248)
bm2(1088)=bm1(1249)
bm2(1089)=bm1(1250)
bm2(1090)=bm1(1251)
bm2(1091)=bm1(1252)
bm2(1092)=bm1(1253)
bm2(1093)=bm1(1254)
bm2(1094)=bm1(1255)
bm2(1095)=bm1(1256)
bm2(1096)=bm1(1257)
bm2(1097)=bm1(1258)
bm2(1098)=bm1(1259)
bm2(1099)=bm1(1260)
bm2(1100)=bm1(1261)
bm2(1101)=bm1(1262)
bm2(1102)=bm1(1263)
bm2(1103)=bm1(1264)
bm2(1104)=bm1(1265)
bm2(1105)=bm1(1266)
bm2(1106)=bm1(1267)
bm2(1107)=bm1(1268)
bm2(1108)=bm1(1269)
bm2(1109)=bm1(1270)
bm2(1110)=bm1(1271)
bm2(1111)=bm1(1272)
bm2(1112)=bm1(1273)
bm2(1113)=bm1(1274)
bm2(1114)=bm1(1275)
bm2(1115)=bm1(1276)
bm2(1116)=bm1(1277)
bm2(1117)=bm1(1278)
bm2(1118)=bm1(1279)
bm2(1119)=bm1(1280)
bm2(1120)=bm1(1281)
bm2(1121)=bm1(1282)
bm2(1122)=bm1(1283)
bm2(1123)=bm1(1284)
bm2(1124)=bm1(1285)
bm2(1125)=bm1(1286)
bm2(1126)=bm1(1287)
bm2(1127)=bm1(1288)
bm2(1128)=bm1(1289)
bm2(1129)=bm1(1290)
bm2(1130)=bm1(1291)
bm2(1131)=bm1(1292)
bm2(1132)=bm1(1293)
bm2(1133)=bm1(1294)
bm2(1134)=bm1(1295)
bm2(1135)=bm1(1296)
bm2(1136)=bm1(1297)
bm2(1137)=bm1(1298)
bm2(1138)=bm1(1299)
bm2(1139)=bm1(1300)
bm2(1140)=bm1(1301)
bm2(1141)=bm1(1302)
bm2(1142)=bm1(1303)
bm2(1143)=bm1(1304)
bm2(1144)=bm1(1305)
bm2(1145)=bm1(1306)
bm2(1146)=bm1(1307)
bm2(1147)=bm1(1308)
bm2(1148)=bm1(1309)
bm2(1149)=bm1(1310)
bm2(1150)=bm1(1311)
bm2(1151)=bm1(1312)
bm2(1152)=bm1(1313)
bm2(1153)=bm1(1314)
bm2(1154)=bm1(1315)
bm2(1155)=bm1(1316)
bm2(1156)=bm1(1317)
bm2(1157)=bm1(1318)
bm2(1158)=bm1(1319)
bm2(1159)=bm1(1320)
bm2(1160)=bm1(1321)
bm2(1161)=bm1(1322)
bm2(1162)=bm1(1323)
bm2(1163)=bm1(1324)
bm2(1164)=bm1(1325)
bm2(1165)=bm1(1326)
bm2(1166)=bm1(1327)
bm2(1167)=bm1(1328)
bm2(1168)=bm1(1329)
bm2(1169)=bm1(1330)
bm2(1170)=bm1(1331)
bm2(1171)=bm1(1332)
bm2(1172)=bm1(1333)
bm2(1173)=bm1(1334)
bm2(1174)=bm1(1335)
bm2(1175)=bm1(1336)
bm2(1176)=bm1(1337)
bm2(1177)=bm1(1338)
bm2(1178)=bm1(1339)
bm2(1179)=bm1(1340)
bm2(1180)=bm1(1341)
bm2(1181)=bm1(1342)
bm2(1182)=bm1(1343)
bm2(1183)=bm1(1344)
bm2(1184)=bm1(1345)
bm2(1185)=bm1(1346)
bm2(1186)=bm1(1347)
bm2(1187)=bm1(1348)
bm2(1188)=bm1(1349)
bm2(1189)=bm1(1350)
bm2(1190)=bm1(1351)
bm2(1191)=bm1(1352)
bm2(1192)=bm1(1353)
bm2(1193)=bm1(1354)
bm2(1194)=bm1(1355)
bm2(1195)=bm1(1356)
bm2(1196)=bm1(1357)
bm2(1197)=bm1(1358)
bm2(1198)=bm1(1359)
bm2(1199)=bm1(1360)
bm2(1200)=bm1(1361)
bm2(1201)=bm1(1362)
bm2(1202)=bm1(1363)
bm2(1203)=bm1(1364)
bm2(1204)=bm1(1365)
bm2(1205)=bm1(1366)
bm2(1206)=bm1(1367)
bm2(1207)=bm1(1368)
bm2(1208)=bm1(1369)
bm2(1209)=bm1(1371)
bm2(1210)=bm1(1372)
bm2(1211)=bm1(1373)
bm2(1212)=bm1(1374)
bm2(1213)=bm1(1375)
bm2(1214)=bm1(1376)
bm2(1215)=bm1(1377)
bm2(1216)=bm1(1378)
bm2(1217)=bm1(1379)
bm2(1218)=bm1(1380)
bm2(1219)=bm1(1381)
bm2(1220)=bm1(1382)
bm2(1221)=bm1(1383)
bm2(1222)=bm1(1385)
bm2(1223)=bm1(1387)
bm2(1224)=bm1(1388)
bm2(1225)=bm1(1389)
bm2(1226)=bm1(1390)
bm2(1227)=bm1(1391)
bm2(1228)=bm1(1392)
bm2(1229)=bm1(1393)
bm2(1230)=bm1(1394)
bm2(1231)=bm1(1395)
bm2(1232)=bm1(1396)
bm2(1233)=bm1(1397)
bm2(1234)=bm1(1399)
bm2(1235)=bm1(1400)
bm2(1236)=bm1(1402)
bm2(1237)=bm1(1404)
bm2(1238)=bm1(1405)
bm2(1239)=bm1(1406)
bm2(1240)=bm1(1407)
bm2(1241)=bm1(1408)
bm2(1242)=bm1(1409)
bm2(1243)=bm1(1410)
bm2(1244)=bm1(1411)
bm2(1245)=bm1(1412)
bm2(1246)=bm1(1413)
bm2(1247)=bm1(1415)
bm2(1248)=bm1(1416)
bm2(1249)=bm1(1418)
bm2(1250)=bm1(1419)
bm2(1251)=bm1(1421)
bm2(1252)=bm1(1422)
bm2(1253)=bm1(1424)
bm2(1254)=bm1(1425)
bm2(1255)=bm1(1426)
bm2(1256)=bm1(1427)
bm2(1257)=bm1(1428)
bm2(1258)=bm1(1429)
bm2(1259)=bm1(1430)
bm2(1260)=bm1(1431)
bm2(1261)=bm1(1432)
bm2(1262)=bm1(1434)
bm2(1263)=bm1(1435)
bm2(1264)=bm1(1437)
bm2(1265)=bm1(1438)
bm2(1266)=bm1(1440)
bm2(1267)=bm1(1441)
bm2(1268)=bm1(1442)
bm2(1269)=bm1(1444)
bm2(1270)=bm1(1445)
bm2(1271)=bm1(1446)
bm2(1272)=bm1(1447)
bm2(1273)=bm1(1448)
bm2(1274)=bm1(1449)
bm2(1275)=bm1(1450)
bm2(1276)=bm1(1451)
bm2(1277)=bm1(1453)
bm2(1278)=bm1(1454)
bm2(1279)=bm1(1456)
bm2(1280)=bm1(1457)
bm2(1281)=bm1(1458)
bm2(1282)=bm1(1460)
bm2(1283)=bm1(1461)
bm2(1284)=bm1(1462)
bm2(1285)=bm1(1463)
bm2(1286)=bm1(1464)
bm2(1287)=bm1(1465)
bm2(1288)=bm1(1467)
bm2(1289)=bm1(1468)
bm2(1290)=bm1(1470)
bm2(1291)=bm1(1471)
bm2(1292)=bm1(1472)
bm2(1293)=bm1(1473)
bm2(1294)=bm1(1474)
bm2(1295)=bm1(1476)
bm2(1296)=bm1(1477)
bm2(1297)=bm1(1478)
bm2(1298)=bm1(1479)
bm2(1299)=bm1(1480)
bm2(1300)=bm1(1481)

do j=1,1300
  q(j)=bm2(j)
enddo

end subroutine ev_pol_con_10
!===========================================================================
! *def DSYEVJ3(A, Q, W)
!  subroutine of diagonalizing a 3 by 3 matrix
!  A is the input matirx
!  Q is the eigenvector matrix
!  W is the eigenvalue
!===========================================================================
SUBROUTINE DSYEVJ3(A, Q, W)
      DOUBLE PRECISION A(3,3)
      DOUBLE PRECISION Q(3,3)
      DOUBLE PRECISION W(3)

      INTEGER          N
      PARAMETER        ( N = 3 )
    
      DOUBLE PRECISION SD, SO
      DOUBLE PRECISION S, C, T
      DOUBLE PRECISION G, H, Z, THETA
      DOUBLE PRECISION THRESH
      INTEGER          I, X, Y, R

      DO 10 X = 1, N
        Q(X,X) = 1.0D0
        DO 11, Y = 1, X-1
          Q(X, Y) = 0.0D0
          Q(Y, X) = 0.0D0
   11   CONTINUE
   10 CONTINUE

      DO 20 X = 1, N
        W(X) = A(X, X)
   20 CONTINUE

      SD = 0.0D0
      DO 30 X = 1, N
        SD = SD + ABS(W(X))
   30 CONTINUE
      SD = SD**2
 
      DO 40 I = 1, 50
        SO = 0.0D0
        DO 50 X = 1, N
          DO 51 Y = X+1, N
            SO = SO + ABS(A(X, Y))
   51     CONTINUE
   50   CONTINUE
        IF (SO .EQ. 0.0D0) THEN
          RETURN
        END IF

        IF (I .LT. 4) THEN
          THRESH = 0.2D0 * SO / N**2
        ELSE
          THRESH = 0.0D0
        END IF

        DO 60 X = 1, N
          DO 61 Y = X+1, N
            G = 100.0D0 * ( ABS(A(X, Y)) )
            IF ( I .GT. 4 .AND. ABS(W(X)) + G .EQ. ABS(W(X)) .AND. ABS(W(Y)) + G .EQ. ABS(W(Y)) ) THEN
              A(X, Y) = 0.0D0
            ELSE IF (ABS(A(X, Y)) .GT. THRESH) THEN
              H = W(Y) - W(X)
              IF ( ABS(H) + G .EQ. ABS(H) ) THEN
                T = A(X, Y) / H
              ELSE
                THETA = 0.5D0 * H / A(X, Y)
                IF (THETA .LT. 0.0D0) THEN
                  T = -1.0D0 / (SQRT(1.0D0 + THETA**2) - THETA)
                ELSE
                  T = 1.0D0 / (SQRT(1.0D0 + THETA**2) + THETA)
                END IF
              END IF

              C = 1.0D0 / SQRT( 1.0D0 + T**2 )
              S = T * C
              Z = T * A(X, Y)
              
              A(X, Y) = 0.0D0
              W(X)    = W(X) - Z
              W(Y)    = W(Y) + Z
              DO 70 R = 1, X-1
                T       = A(R, X)
                A(R, X) = C * T - S * A(R, Y)
                A(R, Y) = S * T + C * A(R, Y)
   70         CONTINUE
              DO 80, R = X+1, Y-1
                T       = A(X, R)
                A(X, R) = C * T - S * A(R, Y)
                A(R, Y) = S * T + C * A(R, Y)
   80         CONTINUE
              DO 90, R = Y+1, N
                T       = A(X, R)
                A(X, R) = C * T - S * A(Y, R)
                A(Y, R) = S * T + C * A(Y, R)
   90         CONTINUE

              DO 100, R = 1, N
                T       = Q(R, X)
                Q(R, X) = C * T - S * Q(R, Y)
                Q(R, Y) = S * T + C * Q(R, Y)
  100         CONTINUE
            END IF
   61     CONTINUE
   60   CONTINUE
   40 CONTINUE

      PRINT *, "DSYEVJ3: No convergence."
            
END SUBROUTINE

!===========================================================================
! *def order(a,rv)
!  subroutine of ordering three eigenstates
!  a is the eigenvalue
!  rv is the eigenvector
!===========================================================================
subroutine order(a,rv)
double precision a(3)
double precision rv(3,3)
double precision tmpv(3,3)
double precision s0, s1, s2
integer i, j

do i=1,3
do j=1,3
tmpv(i,j)=rv(i,j)
enddo
enddo

    if (a(1) < a(2)) then
      if (a(1) < a(3)) then
        if (a(2) < a(3)) then
          s0=a(1)
          s1=a(2)
          s2=a(3)
          do i=1,3
            rv(i,1)=tmpv(i,1)
          enddo
          do i=1,3
            rv(i,2)=tmpv(i,2)
          enddo
          do i=1,3
            rv(i,3)=tmpv(i,3)
          enddo
        else
          s0=a(1)
          s1=a(3)
          s2=a(2)
          do i=1,3
            rv(i,1)=tmpv(i,1)
          enddo
          do i=1,3
            rv(i,2)=tmpv(i,3)
          enddo
          do i=1,3
            rv(i,3)=tmpv(i,2)
          enddo
        end if
      else
          s0=a(3)
          s1=a(1)
          s2=a(2)
          do i=1,3
            rv(i,1)=tmpv(i,3)
          enddo
          do i=1,3
            rv(i,2)=tmpv(i,1)
          enddo
          do i=1,3
            rv(i,3)=tmpv(i,2)
          enddo
      end if
    else
      if (a(2) < a(3)) then
        if (a(1) < a(3)) then
          s0=a(2)
          s1=a(1)
          s2=a(3)
          do i=1,3
            rv(i,1)=tmpv(i,2)
          enddo
          do i=1,3
            rv(i,2)=tmpv(i,1)
          enddo
          do i=1,3
            rv(i,3)=tmpv(i,3)
          enddo
        else
          s0=a(2)
          s1=a(3)
          s2=a(1)
          do i=1,3
            rv(i,1)=tmpv(i,2)
          enddo
          do i=1,3
            rv(i,2)=tmpv(i,3)
          enddo
          do i=1,3
            rv(i,3)=tmpv(i,1)
          enddo
        end if
      else
          s0=a(3)
          s1=a(2)
          s2=a(1)
          do i=1,3
            rv(i,1)=tmpv(i,3)
          enddo
          do i=1,3
            rv(i,2)=tmpv(i,2)
          enddo
          do i=1,3
            rv(i,3)=tmpv(i,1)
          enddo
      end if
    end if

a(1)=s0
a(2)=s1
a(3)=s2
end subroutine
!===========================================================================
! *def u12_dvdx(xyz,u12,s12,du12dx)
!  main subtoutine of evaluating derivative of u12 w.r.t. the xyz coordinates
!  xyz is the coordinate
!  u12 is u12
!  s12 is u12 devided by the hyperbolic function
!  du12dx is the gradient of u12
!===========================================================================
subroutine u12_dvdx(xyz,u12,s12,du12dx)
implicit none
integer :: i,j,k
real*8 :: du12dx(4,3),xyz(4,3)
real*8 :: u12,s12
real*8 :: coord(12), r(6)
real*8 :: dudx(12)
real*8 :: dvdx(12), dvdr(6), drdx(6,12)
real*8 :: dtdx(12)
real*8 :: volume

! Initialize dVdX(12)
      do i=1,12
        dVdX(i)=0.0d0
      enddo

! Initialize dRdX(6,12)
      do i=1,6
        do j=1,12
          dRdX(i,j)=0.0d0
        enddo
      enddo

! Initialize dVdR(6)
      do i=1,6
        dVdR(i)=0.0d0
      enddo

! Assign xyz to coord
      do i=1,4
      do j=1,3
        k=3*(i-1)+j
        coord(k)=xyz(i,j)
      enddo
      enddo

! Compute distances r
r(1)=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r(2)=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r(3)=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r(4)=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r(5)=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r(6)=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

! Call EvdVdR to evaluate dVdR(6)
      Call EvU12O10dVdR(coord,r,dvdr)

! Call EvdRdX to evaluate dRdX(6,12)
      Call EvdRdX(coord,r,drdx)

! Calculate dVdX by using chain rule: dV/dXi=(dV/dRj)*(dRj/dXi), j=1 to 6
      do i=1,12
        do j=1,6
          dVdX(i)=dVdX(i) + dVdR(j)*dRdX(j,i)
        enddo
      enddo

      do i=1,12
          dVdX(i)=dVdX(i)/627.509474
      enddo

  call P_volume(xyz,volume)

! Now compute numerical dtdx
  call Num_dtdx(xyz,dtdx)

! Chain rule again, because U=tanh*V
! dUdX=dtanhdX*V+tanh*dVdX
do i=1,12
  dudx(i)=0
enddo

do i=1,12
  dudx(i)=dtdx(i)*s12+volume*dvdx(i)
enddo


! Assign dvdx back to du12dx
      do i=1,4
      do j=1,3
        k=3*(i-1)+j
        du12dx(i,j)=dudx(k)
      enddo
      enddo

      return

end subroutine u12_dvdx 

!===========================================================================
! *def EvU12O10dVdR(coord,r,dvdr)
!  subtoutine of evaluating derivative of u12 w.r.t. the 6 nuclear distances
!  coord is the xyz coordinate 
!  r is the nuclear distances
!  dvdr is the derivative of u12 w.r.t. the 6 nuclear distances
!===========================================================================
subroutine EvU12O10dVdR(coord,r,dvdr)
integer :: i,j
real*8 :: coord(12),r(6),dvdr(6)
real*8 :: a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer, parameter :: ncoeff=1300
real*8 :: coeff(ncoeff)
real*8,dimension(0:32) :: rm
real*8,dimension(0:32) :: dm
real*8,dimension(0:1506) :: p
real*8,dimension(0:1506) :: dp
real*8 :: x(6),dmsdr(6,6)
real*8 :: dbm1(1506), dbm2(1300)

! Assign coefficients first
a1=0.610
a2=0.800
b1=1.300
b2=1.300
ra1=0.802
ra2=0.702
rb1=0.802
rb2=0.702
coeff(1)=0.150475946155D+03
coeff(2)=0.149915195877D+04
coeff(3)=-0.543046449767D+03
coeff(4)=-0.770110818424D+03
coeff(5)=0.872830595681D+03
coeff(6)=0.323286596435D+03
coeff(7)=-0.469728701570D+03
coeff(8)=0.186753873162D+04
coeff(9)=0.638857933683D+03
coeff(10)=-0.183536219349D+04
coeff(11)=0.148126978045D+04
coeff(12)=-0.260203987883D+04
coeff(13)=0.815003249175D+03
coeff(14)=-0.272290964683D+02
coeff(15)=0.233311870011D+04
coeff(16)=0.277061379005D+03
coeff(17)=0.522365795891D+03
coeff(18)=0.551604367771D+01
coeff(19)=-0.437609409702D+04
coeff(20)=-0.112567934602D+04
coeff(21)=-0.820714544993D+03
coeff(22)=0.154204384971D+04
coeff(23)=0.481971242021D+03
coeff(24)=0.215448749696D+04
coeff(25)=-0.109519698742D+04
coeff(26)=-0.176040670091D+04
coeff(27)=-0.255739741850D+04
coeff(28)=0.250585924438D+03
coeff(29)=0.190379219496D+04
coeff(30)=-0.164329764847D+04
coeff(31)=-0.181300546275D+04
coeff(32)=-0.376807894255D+03
coeff(33)=0.400842181148D+02
coeff(34)=0.144241784754D+04
coeff(35)=0.240241890030D+04
coeff(36)=-0.266810135127D+04
coeff(37)=0.170123163736D+04
coeff(38)=0.132046480227D+04
coeff(39)=0.253421175765D+04
coeff(40)=-0.121912275229D+04
coeff(41)=0.158681686442D+03
coeff(42)=0.254554061211D+03
coeff(43)=0.118713899864D+04
coeff(44)=-0.250259849827D+04
coeff(45)=-0.922683889307D+03
coeff(46)=0.256042888308D+03
coeff(47)=-0.193972761750D+04
coeff(48)=-0.847482592607D+03
coeff(49)=-0.390873710041D+04
coeff(50)=0.322276369506D+04
coeff(51)=-0.229161176311D+04
coeff(52)=-0.265748890260D+04
coeff(53)=0.119478899535D+04
coeff(54)=0.176676180030D+03
coeff(55)=0.106289720608D+05
coeff(56)=0.142497844429D+04
coeff(57)=0.441596932955D+04
coeff(58)=-0.134489545951D+04
coeff(59)=0.263307314781D+02
coeff(60)=0.222889679446D+03
coeff(61)=0.396640924407D+03
coeff(62)=-0.105132656584D+04
coeff(63)=-0.933810623149D+02
coeff(64)=0.100238518254D+04
coeff(65)=-0.213553077991D+03
coeff(66)=-0.524110717947D+02
coeff(67)=0.525031304633D+03
coeff(68)=-0.214440759232D+03
coeff(69)=0.484715769129D+03
coeff(70)=0.264436924640D+04
coeff(71)=0.471083814804D+02
coeff(72)=-0.187402072485D+04
coeff(73)=-0.332564372363D+03
coeff(74)=0.106235447112D+04
coeff(75)=0.660611758971D+03
coeff(76)=0.379572143607D+03
coeff(77)=0.207349276932D+03
coeff(78)=-0.133788480879D+04
coeff(79)=-0.712555218103D+03
coeff(80)=0.105934356687D+04
coeff(81)=-0.159151177614D+02
coeff(82)=-0.163802476967D+02
coeff(83)=-0.673833881386D+03
coeff(84)=-0.248833674422D+03
coeff(85)=-0.182061136147D+04
coeff(86)=-0.191065777431D+03
coeff(87)=-0.684897319901D+03
coeff(88)=0.151414516714D+04
coeff(89)=-0.679651957206D+03
coeff(90)=0.963008073012D+03
coeff(91)=-0.266352391255D+02
coeff(92)=-0.211552089270D+03
coeff(93)=-0.147696540277D+02
coeff(94)=0.420155159425D+02
coeff(95)=0.160755386003D+04
coeff(96)=-0.239688283000D+04
coeff(97)=0.118352874371D+03
coeff(98)=0.662666538613D+03
coeff(99)=-0.748127995583D+03
coeff(100)=0.130849504809D+04
coeff(101)=0.378642695634D+03
coeff(102)=-0.121424807082D+04
coeff(103)=-0.172235346001D+04
coeff(104)=0.543510582039D+03
coeff(105)=0.200116433729D+03
coeff(106)=0.101958099670D+04
coeff(107)=-0.264505753890D+03
coeff(108)=0.156553844950D+03
coeff(109)=0.612703001097D+02
coeff(110)=0.948585795188D+03
coeff(111)=0.114840525409D+02
coeff(112)=0.489755736522D+04
coeff(113)=-0.364221369260D+04
coeff(114)=0.392809782312D+04
coeff(115)=0.472300350087D+03
coeff(116)=-0.473013677641D+03
coeff(117)=0.303153880427D+03
coeff(118)=0.152538695845D+03
coeff(119)=-0.136157901441D+04
coeff(120)=0.380362422125D+03
coeff(121)=0.188978532271D+04
coeff(122)=0.286094877779D+04
coeff(123)=0.389667082447D+03
coeff(124)=-0.128644586156D+04
coeff(125)=-0.422196872572D+03
coeff(126)=-0.444574026993D+04
coeff(127)=-0.386064555226D+04
coeff(128)=-0.175130063625D+04
coeff(129)=0.413429920890D+03
coeff(130)=0.432892982768D+02
coeff(131)=-0.843827990656D+03
coeff(132)=-0.115001849651D+03
coeff(133)=0.245973776456D+03
coeff(134)=-0.863700274500D+03
coeff(135)=0.474031929275D+03
coeff(136)=-0.481309100517D+03
coeff(137)=-0.249230511775D+03
coeff(138)=-0.248846467532D+03
coeff(139)=0.469365037125D+03
coeff(140)=0.102213314392D+04
coeff(141)=-0.207795743405D+03
coeff(142)=-0.143441935639D+04
coeff(143)=-0.632448073752D+03
coeff(144)=0.644917388571D+03
coeff(145)=0.141129870448D+04
coeff(146)=-0.157567066577D+03
coeff(147)=0.498427089231D+03
coeff(148)=-0.111890453727D+04
coeff(149)=0.205940986554D+03
coeff(150)=-0.224847836214D+03
coeff(151)=-0.618238160746D+03
coeff(152)=0.149760129815D+03
coeff(153)=0.358294122728D+03
coeff(154)=0.842063676902D+03
coeff(155)=0.964481429083D+03
coeff(156)=0.130454599334D+03
coeff(157)=0.471743090237D+03
coeff(158)=-0.105055219157D+04
coeff(159)=-0.180097968896D+03
coeff(160)=-0.294631776928D+03
coeff(161)=0.563251137184D+03
coeff(162)=0.675340219985D+03
coeff(163)=0.176065995699D+04
coeff(164)=0.894824409615D+03
coeff(165)=0.369324586323D+02
coeff(166)=0.135751147379D+04
coeff(167)=0.128904251078D+04
coeff(168)=0.534416036498D+03
coeff(169)=-0.469183278059D+03
coeff(170)=-0.238351968855D+03
coeff(171)=-0.127488741170D+03
coeff(172)=-0.546533005703D+03
coeff(173)=-0.937997949726D+03
coeff(174)=-0.894267834123D+03
coeff(175)=-0.121115728447D+04
coeff(176)=-0.699250946523D+03
coeff(177)=0.169012603058D+04
coeff(178)=0.454270754025D+03
coeff(179)=-0.338457674601D+01
coeff(180)=-0.649536341499D+03
coeff(181)=-0.103283315632D+04
coeff(182)=-0.487374674810D+03
coeff(183)=0.284894519583D+04
coeff(184)=-0.664350799282D+03
coeff(185)=-0.948976242332D+03
coeff(186)=0.182423651337D+03
coeff(187)=-0.670333333310D+03
coeff(188)=0.350501822823D+03
coeff(189)=0.943257719835D+03
coeff(190)=-0.228878153767D+03
coeff(191)=-0.232348285732D+03
coeff(192)=-0.483347783815D+03
coeff(193)=-0.337215130963D+03
coeff(194)=-0.101315142781D+04
coeff(195)=-0.605891336085D+03
coeff(196)=-0.679260410601D+03
coeff(197)=0.253272802818D+03
coeff(198)=-0.374925163426D+02
coeff(199)=-0.316621067224D+00
coeff(200)=0.414767613826D+02
coeff(201)=0.132300773937D+04
coeff(202)=0.296174123517D+03
coeff(203)=-0.133144984047D+04
coeff(204)=-0.555440829412D+03
coeff(205)=-0.128389244059D+04
coeff(206)=0.181133664888D+04
coeff(207)=-0.218662930603D+04
coeff(208)=-0.187496180186D+04
coeff(209)=-0.974310070781D+03
coeff(210)=-0.144648628356D+04
coeff(211)=-0.625041000113D+03
coeff(212)=-0.337627958059D+03
coeff(213)=0.118082233118D+04
coeff(214)=-0.561549799948D+03
coeff(215)=-0.694457744326D+03
coeff(216)=-0.182685220544D+03
coeff(217)=0.135491538756D+04
coeff(218)=-0.156812880940D+04
coeff(219)=0.344690479196D+03
coeff(220)=-0.128898043944D+04
coeff(221)=-0.298167484454D+03
coeff(222)=0.451959744286D+03
coeff(223)=-0.914332481905D+03
coeff(224)=-0.412075837244D+03
coeff(225)=0.725603914184D+03
coeff(226)=0.157716810796D+04
coeff(227)=0.320228709895D+04
coeff(228)=-0.708854457599D+03
coeff(229)=0.967472716874D+03
coeff(230)=-0.500455803926D+03
coeff(231)=0.563015606617D+03
coeff(232)=0.302995542696D+03
coeff(233)=0.106890510122D+04
coeff(234)=0.484235329310D+02
coeff(235)=-0.623505976175D+03
coeff(236)=-0.189495391973D+04
coeff(237)=0.483528040896D+03
coeff(238)=0.109860747237D+04
coeff(239)=0.219377864968D+04
coeff(240)=-0.188650569125D+04
coeff(241)=-0.253841268076D+04
coeff(242)=-0.219746925170D+03
coeff(243)=0.150904526753D+03
coeff(244)=0.461324240357D+03
coeff(245)=-0.176749634612D+04
coeff(246)=-0.178848848654D+04
coeff(247)=-0.101182175728D+04
coeff(248)=0.236025634629D+04
coeff(249)=0.136627012761D+04
coeff(250)=-0.971301033623D+03
coeff(251)=-0.194813342700D+03
coeff(252)=-0.568330713226D+04
coeff(253)=-0.634564122654D+03
coeff(254)=-0.346755599865D+04
coeff(255)=0.121657241884D+04
coeff(256)=-0.100511174054D+03
coeff(257)=0.137945047535D+04
coeff(258)=-0.103319465780D+03
coeff(259)=-0.413429373664D+03
coeff(260)=-0.926194923919D+02
coeff(261)=-0.322242810833D+02
coeff(262)=0.135658607666D+03
coeff(263)=0.355035114527D+03
coeff(264)=-0.993652025850D+03
coeff(265)=0.584329372977D+02
coeff(266)=0.189823640888D+02
coeff(267)=-0.471744562756D+03
coeff(268)=0.965301017120D+02
coeff(269)=-0.248404122677D+03
coeff(270)=-0.976432255897D+02
coeff(271)=-0.188024867432D+03
coeff(272)=0.733455826952D+03
coeff(273)=0.178406526998D+02
coeff(274)=-0.233773599044D+03
coeff(275)=-0.274715590947D+03
coeff(276)=0.531528564451D+02
coeff(277)=-0.105732978982D+04
coeff(278)=-0.180944993584D+03
coeff(279)=-0.652562305526D+03
coeff(280)=-0.174458543879D+03
coeff(281)=0.974970278958D+03
coeff(282)=0.199076108649D+04
coeff(283)=0.488975905120D+03
coeff(284)=0.360282372573D+02
coeff(285)=0.511691027775D+03
coeff(286)=0.629115681141D+03
coeff(287)=-0.179059156101D+03
coeff(288)=-0.178656006005D+03
coeff(289)=0.764784770430D+03
coeff(290)=0.523391171536D+03
coeff(291)=-0.113482489203D+03
coeff(292)=-0.148837299343D+03
coeff(293)=0.426484338597D+03
coeff(294)=0.955358205174D+03
coeff(295)=-0.389026458266D+03
coeff(296)=0.130103662094D+04
coeff(297)=-0.147643310982D+04
coeff(298)=-0.146906165109D+04
coeff(299)=-0.578499063819D+03
coeff(300)=-0.325832090557D+03
coeff(301)=-0.663801352331D+03
coeff(302)=-0.211121022866D+02
coeff(303)=-0.633158628038D+03
coeff(304)=-0.240889741393D+03
coeff(305)=-0.493855498598D+03
coeff(306)=-0.110078328585D+04
coeff(307)=-0.189643896595D+03
coeff(308)=-0.768225404470D+03
coeff(309)=-0.635598179602D+03
coeff(310)=-0.952764381219D+03
coeff(311)=-0.666355787573D+03
coeff(312)=0.117970963615D+04
coeff(313)=0.402382386483D+03
coeff(314)=-0.102312150904D+04
coeff(315)=0.244524513628D+03
coeff(316)=0.330698088887D+03
coeff(317)=0.922154755689D+03
coeff(318)=-0.360555583875D+03
coeff(319)=0.637775674181D+03
coeff(320)=-0.101390551294D+04
coeff(321)=0.896359484114D+02
coeff(322)=0.781417196463D+03
coeff(323)=0.113931331202D+04
coeff(324)=0.737761849924D+03
coeff(325)=-0.847728024247D+03
coeff(326)=0.661473608483D+03
coeff(327)=-0.466356022206D+03
coeff(328)=-0.105973578384D+04
coeff(329)=0.677402676391D+03
coeff(330)=0.912809054529D+03
coeff(331)=0.140338112274D+03
coeff(332)=-0.232789708436D+03
coeff(333)=-0.171597031167D+04
coeff(334)=-0.149993918097D+04
coeff(335)=-0.334108276298D+03
coeff(336)=-0.456584304975D+03
coeff(337)=0.995974891332D+03
coeff(338)=0.140847014771D+03
coeff(339)=-0.165968995384D+04
coeff(340)=-0.751476761985D+02
coeff(341)=-0.601025716091D+03
coeff(342)=-0.162480968804D+04
coeff(343)=-0.315830694864D+03
coeff(344)=-0.204849947174D+02
coeff(345)=0.282709056085D+04
coeff(346)=-0.120456189181D+04
coeff(347)=-0.661836016680D+03
coeff(348)=-0.901624148646D+02
coeff(349)=0.211495583093D+03
coeff(350)=0.590958634609D+03
coeff(351)=-0.304588767050D+03
coeff(352)=0.578719951097D+03
coeff(353)=0.714862816798D+03
coeff(354)=0.505064797102D+03
coeff(355)=-0.237805001321D+03
coeff(356)=-0.182346201339D+03
coeff(357)=0.229379867994D+03
coeff(358)=0.326361911311D+02
coeff(359)=-0.157597785197D+03
coeff(360)=0.474665279441D+03
coeff(361)=-0.147146948041D+04
coeff(362)=0.106049519236D+04
coeff(363)=-0.231804162587D+03
coeff(364)=-0.405159110304D+03
coeff(365)=-0.668459104362D+03
coeff(366)=-0.217950254734D+03
coeff(367)=-0.210334298946D+03
coeff(368)=-0.565859543448D+03
coeff(369)=0.115005680314D+04
coeff(370)=0.160631042444D+04
coeff(371)=0.537255177611D+03
coeff(372)=-0.138313124133D+04
coeff(373)=-0.190835696667D+03
coeff(374)=-0.504033330385D+02
coeff(375)=-0.109379219134D+04
coeff(376)=-0.843602716754D+03
coeff(377)=0.142128637336D+03
coeff(378)=0.178463221216D+03
coeff(379)=0.343668380619D+02
coeff(380)=0.670098996097D+03
coeff(381)=-0.463114084593D+03
coeff(382)=0.558976402182D+02
coeff(383)=-0.594943668193D+03
coeff(384)=-0.618993728658D+03
coeff(385)=-0.147383861758D+03
coeff(386)=0.751652000327D+02
coeff(387)=-0.100837526579D+04
coeff(388)=-0.409721534466D+03
coeff(389)=0.384390126364D+03
coeff(390)=0.269033643332D+03
coeff(391)=-0.205920488493D+03
coeff(392)=0.148136859639D+04
coeff(393)=0.432263466272D+03
coeff(394)=0.293975277915D+03
coeff(395)=-0.556668119133D+03
coeff(396)=0.471795678462D+03
coeff(397)=-0.698680859087D+03
coeff(398)=0.169388940440D+04
coeff(399)=0.115752805755D+02
coeff(400)=0.118436807064D+04
coeff(401)=0.132672907464D+04
coeff(402)=-0.251067013300D+02
coeff(403)=0.165131346608D+04
coeff(404)=0.372576624138D+03
coeff(405)=-0.785426280013D+03
coeff(406)=-0.287195774047D+03
coeff(407)=-0.555445317666D+03
coeff(408)=0.754587601130D+03
coeff(409)=0.302572589832D+03
coeff(410)=-0.673245663881D+02
coeff(411)=0.155845377194D+04
coeff(412)=-0.427159023160D+03
coeff(413)=0.378738454644D+03
coeff(414)=-0.104087482394D+04
coeff(415)=-0.249365983670D+03
coeff(416)=-0.179646906585D+03
coeff(417)=0.176939527284D+03
coeff(418)=0.435928992243D+03
coeff(419)=-0.393477674189D+03
coeff(420)=-0.486038393166D+03
coeff(421)=-0.266132804384D+02
coeff(422)=-0.291629145912D+02
coeff(423)=-0.817696537504D+03
coeff(424)=0.111635091393D+04
coeff(425)=-0.893008255908D+03
coeff(426)=-0.410845088250D+03
coeff(427)=-0.447832016317D+02
coeff(428)=-0.103806138234D+04
coeff(429)=-0.739645886377D+03
coeff(430)=-0.717443615482D+03
coeff(431)=0.118975090937D+04
coeff(432)=-0.288155560030D+04
coeff(433)=0.153045055353D+04
coeff(434)=-0.266497765237D+04
coeff(435)=-0.142400071644D+02
coeff(436)=-0.252599371312D+04
coeff(437)=0.113005524593D+04
coeff(438)=0.470418021630D+03
coeff(439)=0.907964817045D+03
coeff(440)=0.683818675760D+03
coeff(441)=0.622925783314D+03
coeff(442)=-0.852546844814D+03
coeff(443)=0.509967262927D+03
coeff(444)=0.179423415936D+03
coeff(445)=-0.633185977681D+03
coeff(446)=-0.103353209848D+04
coeff(447)=0.111285134504D+04
coeff(448)=-0.334440263428D+04
coeff(449)=-0.236242341811D+04
coeff(450)=-0.360469919146D+04
coeff(451)=0.259689629116D+03
coeff(452)=0.961728406595D+02
coeff(453)=0.900409806595D+03
coeff(454)=0.257378144607D+04
coeff(455)=0.806045645309D+02
coeff(456)=0.117175081856D+04
coeff(457)=0.383371621518D+04
coeff(458)=0.255808646711D+04
coeff(459)=0.591352063498D+03
coeff(460)=-0.621110017827D+02
coeff(461)=0.959867782575D+03
coeff(462)=0.254631725969D+04
coeff(463)=-0.118625995678D+04
coeff(464)=-0.483080133890D+03
coeff(465)=-0.107343641387D+04
coeff(466)=0.690387365044D+02
coeff(467)=-0.932452163590D+03
coeff(468)=0.365918413650D+02
coeff(469)=0.525118634615D+02
coeff(470)=-0.404201552721D+03
coeff(471)=-0.332659297313D+03
coeff(472)=-0.206838232759D+03
coeff(473)=-0.179797383135D+03
coeff(474)=-0.624626953475D+03
coeff(475)=-0.104977413937D+03
coeff(476)=0.242028069557D-03
coeff(477)=0.522368690413D+02
coeff(478)=0.128620377990D+02
coeff(479)=0.538858073591D+03
coeff(480)=0.438478766038D+03
coeff(481)=0.332669604562D+03
coeff(482)=0.197689543840D+02
coeff(483)=0.738623583307D+01
coeff(484)=-0.212985339998D+03
coeff(485)=0.323669252832D+02
coeff(486)=-0.253453068887D+03
coeff(487)=-0.265191392416D+02
coeff(488)=-0.112237551746D+02
coeff(489)=0.304201258073D+02
coeff(490)=-0.648733783966D+02
coeff(491)=0.317419560502D+03
coeff(492)=0.439763535817D+03
coeff(493)=-0.737907894251D+03
coeff(494)=0.552642518147D+03
coeff(495)=0.173596785653D+03
coeff(496)=0.298557213774D+03
coeff(497)=-0.228791883593D+03
coeff(498)=0.801197904984D+02
coeff(499)=0.385822738351D+03
coeff(500)=0.377342668574D+03
coeff(501)=-0.299606194610D+03
coeff(502)=0.337183359362D+02
coeff(503)=-0.817333036449D+03
coeff(504)=0.138149352005D+03
coeff(505)=-0.189459871711D+03
coeff(506)=-0.782643778274D+02
coeff(507)=-0.508644056063D+03
coeff(508)=0.117125099531D+03
coeff(509)=0.924633801990D+03
coeff(510)=0.483773947378D+03
coeff(511)=0.202361059625D+03
coeff(512)=0.627164588667D+03
coeff(513)=0.112782823470D+03
coeff(514)=-0.267204869364D+03
coeff(515)=0.350529888885D+03
coeff(516)=0.112660255134D+04
coeff(517)=-0.302907602505D+03
coeff(518)=-0.940440768743D+01
coeff(519)=0.155655232099D+03
coeff(520)=0.136870314523D+03
coeff(521)=0.396661918013D+03
coeff(522)=0.330934123143D+03
coeff(523)=0.241243989685D+03
coeff(524)=0.743593798081D+03
coeff(525)=0.276254094829D+03
coeff(526)=-0.211587912835D+03
coeff(527)=0.146792507628D+03
coeff(528)=0.177232604378D+03
coeff(529)=0.900441922770D+03
coeff(530)=0.110467325008D+04
coeff(531)=0.238758529874D+03
coeff(532)=0.496372460062D+03
coeff(533)=0.600886405369D+03
coeff(534)=0.765423992744D+03
coeff(535)=0.449895222515D+03
coeff(536)=0.210649717773D+03
coeff(537)=0.606830630534D+03
coeff(538)=-0.472299085390D+03
coeff(539)=-0.348693586283D+03
coeff(540)=0.279585498484D+03
coeff(541)=-0.310936716245D+03
coeff(542)=-0.242705264516D+03
coeff(543)=0.967104334191D+03
coeff(544)=-0.127097811972D+04
coeff(545)=-0.243480426077D+03
coeff(546)=0.575221626803D+03
coeff(547)=0.171893864311D+03
coeff(548)=-0.910556940773D+02
coeff(549)=0.233353879295D+03
coeff(550)=0.725893416792D+03
coeff(551)=-0.472714499811D+03
coeff(552)=0.134009489700D+04
coeff(553)=-0.137915579597D+04
coeff(554)=-0.833895877959D+03
coeff(555)=-0.996702030977D+03
coeff(556)=-0.170120790639D+03
coeff(557)=-0.410227481145D+03
coeff(558)=0.990524820038D+03
coeff(559)=0.815592455871D+03
coeff(560)=0.141278491724D+03
coeff(561)=-0.301034707811D+03
coeff(562)=0.159061106537D+03
coeff(563)=0.391079363327D+03
coeff(564)=-0.499602064028D+02
coeff(565)=0.827837417511D+03
coeff(566)=0.798219697194D+03
coeff(567)=0.161083316227D+03
coeff(568)=0.259022873063D+03
coeff(569)=-0.978843770230D+03
coeff(570)=0.551908134209D+03
coeff(571)=0.168932474935D+04
coeff(572)=0.769045972305D+03
coeff(573)=-0.770379976744D+03
coeff(574)=-0.298115084363D+03
coeff(575)=0.145343939205D+03
coeff(576)=0.349946818479D+03
coeff(577)=0.331304819092D+03
coeff(578)=-0.164764169462D+04
coeff(579)=0.279777235658D+03
coeff(580)=-0.108685616224D+04
coeff(581)=-0.881321667995D+03
coeff(582)=-0.265451828407D+03
coeff(583)=-0.629828960887D+02
coeff(584)=-0.698563865157D+03
coeff(585)=-0.509873799547D+02
coeff(586)=0.930932239163D+02
coeff(587)=-0.357143658567D+03
coeff(588)=-0.859912628780D+03
coeff(589)=0.887231884992D+03
coeff(590)=0.454012605720D+03
coeff(591)=-0.858531420340D+03
coeff(592)=0.417242087488D+02
coeff(593)=0.599426637209D+03
coeff(594)=0.133797809631D+04
coeff(595)=0.335349169637D+03
coeff(596)=0.725974621425D+03
coeff(597)=-0.835553821681D+03
coeff(598)=-0.286016317928D+03
coeff(599)=-0.681029743481D+01
coeff(600)=-0.144578512034D+04
coeff(601)=-0.585270895006D+03
coeff(602)=-0.241497240074D+03
coeff(603)=-0.314340066517D+03
coeff(604)=0.106053799910D+03
coeff(605)=-0.201690768613D+04
coeff(606)=0.474533455587D+01
coeff(607)=0.216640528130D+03
coeff(608)=0.232713651778D+03
coeff(609)=-0.343793930831D+03
coeff(610)=0.541762813687D+03
coeff(611)=-0.666828038692D+03
coeff(612)=0.602024956719D+03
coeff(613)=-0.137202852308D+04
coeff(614)=-0.154770895223D+04
coeff(615)=-0.128760645340D+03
coeff(616)=-0.159327498231D+03
coeff(617)=-0.198605019134D+03
coeff(618)=-0.228624428616D+03
coeff(619)=0.540921203853D+02
coeff(620)=-0.224549434579D+03
coeff(621)=0.534993789700D+03
coeff(622)=0.546052272661D+02
coeff(623)=0.302615328300D+03
coeff(624)=0.630765027768D+03
coeff(625)=0.980884373985D+03
coeff(626)=0.365238145917D+03
coeff(627)=0.304475665886D+03
coeff(628)=-0.805206040939D+03
coeff(629)=0.254706034809D+03
coeff(630)=0.657543394027D+02
coeff(631)=0.124967709698D+03
coeff(632)=-0.657540288174D+03
coeff(633)=-0.158220446348D+04
coeff(634)=0.108052474836D+04
coeff(635)=-0.443786361965D+03
coeff(636)=0.475167707212D+03
coeff(637)=-0.546235492051D+03
coeff(638)=-0.141971963354D+04
coeff(639)=-0.461160623939D+03
coeff(640)=0.103272982975D+04
coeff(641)=0.509099447849D+03
coeff(642)=0.584227417235D+03
coeff(643)=-0.569366216876D+03
coeff(644)=0.199374883715D+03
coeff(645)=0.859683178371D+03
coeff(646)=0.553987933980D+03
coeff(647)=-0.123186439708D+04
coeff(648)=-0.725573212963D+03
coeff(649)=-0.381294009519D+03
coeff(650)=0.213563011217D+03
coeff(651)=-0.667017012041D+03
coeff(652)=-0.168969158336D+03
coeff(653)=0.711288174471D+03
coeff(654)=-0.107271938158D+04
coeff(655)=0.238933361162D+03
coeff(656)=0.139778130981D+03
coeff(657)=0.252226757503D+03
coeff(658)=0.226142995020D+03
coeff(659)=0.541111127210D+02
coeff(660)=0.657105534091D+03
coeff(661)=0.698530758095D+03
coeff(662)=-0.108058079828D+03
coeff(663)=-0.617901210669D+03
coeff(664)=-0.215700333057D+03
coeff(665)=0.673943021906D+03
coeff(666)=0.113848776487D+04
coeff(667)=0.264977024501D+03
coeff(668)=-0.124604234931D+03
coeff(669)=-0.689377510852D+03
coeff(670)=-0.196854796485D+03
coeff(671)=-0.319829780119D+03
coeff(672)=-0.593107013507D+03
coeff(673)=0.758736573226D+02
coeff(674)=0.888808250252D+03
coeff(675)=0.429493211714D+02
coeff(676)=-0.381906575916D+02
coeff(677)=0.116972003483D+04
coeff(678)=0.478516709046D+03
coeff(679)=0.856217184634D+03
coeff(680)=-0.986364223676D+03
coeff(681)=0.178767698045D+03
coeff(682)=0.523809201464D+03
coeff(683)=-0.113575563929D+04
coeff(684)=-0.487148114738D+03
coeff(685)=-0.944098272127D+03
coeff(686)=0.150220219500D+04
coeff(687)=-0.720436446012D+03
coeff(688)=-0.177947652342D+03
coeff(689)=0.424828142051D+02
coeff(690)=0.931966532686D+03
coeff(691)=-0.714180972071D+03
coeff(692)=0.504435705797D+03
coeff(693)=-0.497916806078D+03
coeff(694)=-0.577677990360D+03
coeff(695)=0.585812584571D+03
coeff(696)=0.707543946063D+03
coeff(697)=-0.997177810642D+03
coeff(698)=0.353851348894D+03
coeff(699)=0.907954259367D+03
coeff(700)=0.125582844084D+04
coeff(701)=-0.543520266561D+03
coeff(702)=0.166236712455D+04
coeff(703)=0.213476068297D+02
coeff(704)=-0.114024123795D+02
coeff(705)=0.728059963024D+03
coeff(706)=0.882611539100D+03
coeff(707)=-0.328916214272D+03
coeff(708)=0.809717204798D+03
coeff(709)=0.163046778047D+04
coeff(710)=-0.182141087191D+03
coeff(711)=-0.450517111028D+03
coeff(712)=-0.295917699321D+03
coeff(713)=0.170580027196D+03
coeff(714)=0.906419787426D+02
coeff(715)=-0.739830446862D+03
coeff(716)=-0.222632642021D+03
coeff(717)=0.314056536597D+03
coeff(718)=-0.823262006994D+02
coeff(719)=-0.625318601685D+03
coeff(720)=-0.421496302676D+03
coeff(721)=-0.503063568461D+03
coeff(722)=0.103221713844D+04
coeff(723)=-0.109809566561D+04
coeff(724)=0.636489700284D+03
coeff(725)=-0.521219823808D+03
coeff(726)=0.926152123883D+03
coeff(727)=-0.291030207895D+03
coeff(728)=0.186808638322D+04
coeff(729)=0.171012733592D+04
coeff(730)=-0.481551500625D+03
coeff(731)=0.485690530014D+02
coeff(732)=0.132812684295D+04
coeff(733)=0.107492195907D+04
coeff(734)=0.397876374019D+03
coeff(735)=-0.689281556341D+03
coeff(736)=0.601534199527D+03
coeff(737)=-0.124303024120D+04
coeff(738)=-0.489607475424D+03
coeff(739)=0.126931412389D+04
coeff(740)=-0.179401970779D+04
coeff(741)=-0.251649465541D+03
coeff(742)=-0.790638655575D+02
coeff(743)=-0.435223192255D+03
coeff(744)=0.549820910279D+02
coeff(745)=-0.108610518051D+04
coeff(746)=-0.788713401230D+03
coeff(747)=-0.105618482408D+04
coeff(748)=0.118510504021D+03
coeff(749)=-0.274663934121D+04
coeff(750)=0.164890085894D+04
coeff(751)=-0.154941285454D+04
coeff(752)=0.538111512543D+02
coeff(753)=-0.177680011572D+04
coeff(754)=0.148582013090D+03
coeff(755)=0.481882329225D+03
coeff(756)=-0.154736489732D+03
coeff(757)=-0.454090952898D+02
coeff(758)=-0.358426584146D+03
coeff(759)=-0.103235171276D+04
coeff(760)=0.679240776795D+03
coeff(761)=-0.766166345566D+02
coeff(762)=0.185102784219D+04
coeff(763)=-0.123491098286D+04
coeff(764)=0.261836558561D+03
coeff(765)=-0.136886410890D+04
coeff(766)=-0.100279092336D+04
coeff(767)=0.541937558995D+03
coeff(768)=-0.235443591045D+04
coeff(769)=-0.461492502925D+03
coeff(770)=0.784630901073D+03
coeff(771)=0.561250230095D+03
coeff(772)=0.801012578576D+03
coeff(773)=0.524807706192D+03
coeff(774)=-0.148399815006D+04
coeff(775)=0.246003987438D+04
coeff(776)=0.165287377656D+04
coeff(777)=0.266795329886D+03
coeff(778)=-0.148095688192D+04
coeff(779)=-0.843070995466D+03
coeff(780)=0.199879812206D+03
coeff(781)=0.345317857919D+03
coeff(782)=-0.664190598455D+03
coeff(783)=0.136362632498D+03
coeff(784)=0.658896977779D+02
coeff(785)=0.889719253087D+03
coeff(786)=0.400434433461D+04
coeff(787)=0.856846463040D+03
coeff(788)=0.251393287702D+04
coeff(789)=0.175085589962D+04
coeff(790)=0.251921041250D+04
coeff(791)=0.123360926953D+04
coeff(792)=-0.110055213533D+03
coeff(793)=0.431942466765D+04
coeff(794)=0.208767993601D+04
coeff(795)=0.106613625687D+04
coeff(796)=0.372104003653D+03
coeff(797)=0.601369483446D+02
coeff(798)=0.200920016149D+03
coeff(799)=0.277207263135D+03
coeff(800)=0.927144844437D+02
coeff(801)=0.271226941179D+03
coeff(802)=0.524514578020D+03
coeff(803)=-0.650135576799D+02
coeff(804)=-0.451206139384D+03
coeff(805)=-0.499361832115D+03
coeff(806)=-0.411449702480D+03
coeff(807)=-0.738653065679D+03
coeff(808)=-0.986454259592D+03
coeff(809)=-0.122272039113D+04
coeff(810)=-0.295615045239D+03
coeff(811)=-0.837556823457D+03
coeff(812)=-0.628597600891D+03
coeff(813)=-0.331132711225D+03
coeff(814)=-0.183194225560D+03
coeff(815)=-0.444480398529D+03
coeff(816)=-0.175176921801D+03
coeff(817)=-0.105423242435D+03
coeff(818)=-0.176189847392D+03
coeff(819)=-0.330230536963D+03
coeff(820)=-0.130486316919D+03
coeff(821)=-0.356041268565D+02
coeff(822)=0.163642406050D+03
coeff(823)=0.153942673965D+03
coeff(824)=-0.209373640875D+03
coeff(825)=0.443590712637D+03
coeff(826)=0.346337482168D+03
coeff(827)=0.187905914637D+03
coeff(828)=0.304699038008D+03
coeff(829)=0.620219812636D+03
coeff(830)=-0.129654627937D+04
coeff(831)=-0.102216886428D+03
coeff(832)=0.374875783660D+03
coeff(833)=0.773773201348D+03
coeff(834)=-0.458751230482D+03
coeff(835)=0.253934410726D+03
coeff(836)=-0.261069396850D+03
coeff(837)=-0.331585275898D+03
coeff(838)=-0.116612739871D+03
coeff(839)=0.937117254771D+03
coeff(840)=0.120098293433D+04
coeff(841)=0.815301603773D+03
coeff(842)=-0.241443849623D+03
coeff(843)=-0.288602696895D+03
coeff(844)=0.111361303341D+03
coeff(845)=0.144970210905D+03
coeff(846)=0.269206829809D+03
coeff(847)=-0.285891743724D+03
coeff(848)=0.643209385200D+02
coeff(849)=0.716712095647D+02
coeff(850)=0.280209481273D+03
coeff(851)=0.793985098025D+03
coeff(852)=-0.195505047956D+03
coeff(853)=-0.140172816805D+03
coeff(854)=0.723981786382D+03
coeff(855)=-0.480943068447D+03
coeff(856)=0.222015286513D+03
coeff(857)=0.420956514458D+03
coeff(858)=0.307364616126D+03
coeff(859)=0.207523253082D+03
coeff(860)=0.915950920808D+02
coeff(861)=-0.475655261963D+03
coeff(862)=-0.283075868414D+03
coeff(863)=-0.801464644002D+03
coeff(864)=-0.116948289696D+04
coeff(865)=-0.536822217844D+03
coeff(866)=0.502716770113D+03
coeff(867)=-0.648997203158D+02
coeff(868)=0.105472793604D+03
coeff(869)=0.306360496681D+03
coeff(870)=0.471279700323D+02
coeff(871)=0.733561548948D+02
coeff(872)=-0.469845005039D+03
coeff(873)=-0.621455416395D+02
coeff(874)=0.823156844254D+03
coeff(875)=-0.359796385044D+03
coeff(876)=-0.367923282677D+03
coeff(877)=-0.103326235057D+04
coeff(878)=-0.235489931256D+03
coeff(879)=-0.308078719491D+03
coeff(880)=-0.765161566126D+03
coeff(881)=-0.437638060780D+03
coeff(882)=-0.106332158472D+03
coeff(883)=-0.243248108228D+03
coeff(884)=-0.285559045557D+03
coeff(885)=-0.112119397259D+03
coeff(886)=0.145033800328D+03
coeff(887)=-0.161047369244D+03
coeff(888)=0.338032007824D+02
coeff(889)=0.130637173531D+03
coeff(890)=-0.286776981492D+03
coeff(891)=0.787414281120D+02
coeff(892)=0.750565575346D+03
coeff(893)=-0.332108280638D+03
coeff(894)=0.929361215282D+02
coeff(895)=-0.336051266631D+03
coeff(896)=-0.383312359662D+03
coeff(897)=0.502019135804D+03
coeff(898)=-0.175059562862D+03
coeff(899)=0.410984308532D+03
coeff(900)=0.373817801160D+03
coeff(901)=0.668340943360D+03
coeff(902)=0.122117276437D+02
coeff(903)=-0.239519582093D+03
coeff(904)=-0.995417839484D+02
coeff(905)=0.238020967528D+03
coeff(906)=0.476149248165D+03
coeff(907)=-0.165401079179D+03
coeff(908)=-0.116970029080D+03
coeff(909)=-0.285639292870D+03
coeff(910)=-0.615352954341D+03
coeff(911)=0.270151837483D+03
coeff(912)=-0.746757735512D+02
coeff(913)=-0.857382056801D+02
coeff(914)=-0.252166052815D+03
coeff(915)=0.159376181883D+03
coeff(916)=0.617895184118D+03
coeff(917)=0.197130836071D+03
coeff(918)=-0.547874830220D+03
coeff(919)=0.582665660651D+03
coeff(920)=0.711080842558D+02
coeff(921)=-0.107740578026D+04
coeff(922)=-0.396371511245D+02
coeff(923)=0.479706703231D+03
coeff(924)=-0.102237482166D+03
coeff(925)=0.672588793195D+03
coeff(926)=0.761916466012D+03
coeff(927)=-0.803143670448D+03
coeff(928)=0.192599263767D+03
coeff(929)=0.338548269064D+03
coeff(930)=0.802178163716D+03
coeff(931)=0.367029892371D+03
coeff(932)=-0.600254755740D+03
coeff(933)=0.273893303906D+03
coeff(934)=0.424742139817D+03
coeff(935)=0.548113527265D+03
coeff(936)=0.276801360073D+03
coeff(937)=-0.330207447319D+03
coeff(938)=0.613068894602D+03
coeff(939)=-0.390373509814D+03
coeff(940)=-0.126634842931D+03
coeff(941)=0.585030284288D+03
coeff(942)=-0.162492738462D+03
coeff(943)=-0.508378858559D+03
coeff(944)=0.210164861635D+04
coeff(945)=-0.871939782827D+03
coeff(946)=-0.315384340100D+03
coeff(947)=-0.132322827181D+04
coeff(948)=0.291174224600D+03
coeff(949)=-0.128047687477D+03
coeff(950)=-0.437848629777D+03
coeff(951)=0.184590853120D+03
coeff(952)=0.679251442751D+03
coeff(953)=-0.599235770529D+03
coeff(954)=0.121689455598D+04
coeff(955)=-0.121135359652D+04
coeff(956)=-0.113157764562D+03
coeff(957)=-0.136932962826D+04
coeff(958)=-0.249990656582D+03
coeff(959)=-0.362141447012D+03
coeff(960)=0.954909513802D+02
coeff(961)=-0.618952270773D+03
coeff(962)=-0.482372081528D+03
coeff(963)=-0.299650465442D+03
coeff(964)=0.240714680153D+03
coeff(965)=-0.476841547889D+01
coeff(966)=0.580593038910D+03
coeff(967)=0.503374070046D+03
coeff(968)=-0.979479263630D+03
coeff(969)=-0.461095296467D+03
coeff(970)=0.292633652079D+03
coeff(971)=-0.174253956196D+02
coeff(972)=-0.587399302206D-01
coeff(973)=-0.594325175029D+03
coeff(974)=0.244262286457D+03
coeff(975)=0.930093915018D+03
coeff(976)=-0.330514683933D+03
coeff(977)=-0.267399444656D+03
coeff(978)=-0.663532828741D+03
coeff(979)=0.529475636311D+02
coeff(980)=0.692475132359D+03
coeff(981)=-0.450951886832D+03
coeff(982)=0.683134057457D+03
coeff(983)=-0.743341950962D+03
coeff(984)=-0.132161650051D+04
coeff(985)=0.103656708222D+04
coeff(986)=-0.114026479500D+03
coeff(987)=-0.627086902515D+03
coeff(988)=-0.173981750943D+03
coeff(989)=0.766783078443D+03
coeff(990)=0.595320853593D+02
coeff(991)=-0.725135833509D+02
coeff(992)=0.993364982474D+02
coeff(993)=0.106735166846D+03
coeff(994)=0.250711806801D+03
coeff(995)=0.158357529606D+03
coeff(996)=0.655853393933D+02
coeff(997)=-0.206846925873D+03
coeff(998)=-0.320805820610D+03
coeff(999)=0.121960134011D+03
coeff(1000)=-0.125259271305D+04
coeff(1001)=-0.970239105686D+02
coeff(1002)=-0.717231294268D+03
coeff(1003)=-0.721254033908D+03
coeff(1004)=-0.684168862514D+03
coeff(1005)=-0.177074053683D+03
coeff(1006)=-0.712911322977D+03
coeff(1007)=0.169554036003D+03
coeff(1008)=0.186329580370D+03
coeff(1009)=0.354823205588D+03
coeff(1010)=0.134281332279D+03
coeff(1011)=0.127147138605D+04
coeff(1012)=-0.742909062941D+03
coeff(1013)=0.774063706641D+03
coeff(1014)=-0.160343569865D+04
coeff(1015)=-0.281592167414D+03
coeff(1016)=0.231391528739D+04
coeff(1017)=0.106464935771D+04
coeff(1018)=0.878483460052D+03
coeff(1019)=0.374848465564D+03
coeff(1020)=0.647956134940D+02
coeff(1021)=0.427845226508D+03
coeff(1022)=0.670975541993D+03
coeff(1023)=0.170300918543D+04
coeff(1024)=0.558328470066D+03
coeff(1025)=0.809257274823D+03
coeff(1026)=0.447347763836D+00
coeff(1027)=-0.703975683640D+03
coeff(1028)=0.415318914732D+03
coeff(1029)=0.275257804982D+03
coeff(1030)=-0.234309344607D+04
coeff(1031)=0.603296325504D+03
coeff(1032)=0.665135536839D+03
coeff(1033)=-0.658951484424D+03
coeff(1034)=-0.459337125278D+03
coeff(1035)=-0.152121769900D+04
coeff(1036)=0.529715692276D+02
coeff(1037)=0.242378574648D+03
coeff(1038)=0.402106329719D+03
coeff(1039)=0.129644444912D+04
coeff(1040)=-0.117989038663D+04
coeff(1041)=-0.507796085142D+03
coeff(1042)=-0.429237799864D+03
coeff(1043)=0.482816601373D+03
coeff(1044)=-0.609406163332D+03
coeff(1045)=0.342695046759D+03
coeff(1046)=-0.814577920542D+03
coeff(1047)=0.270105256282D+03
coeff(1048)=-0.722406787007D+03
coeff(1049)=0.389751442946D+03
coeff(1050)=-0.619709740600D+03
coeff(1051)=-0.106902569341D+03
coeff(1052)=-0.105536386523D+04
coeff(1053)=0.498848840109D+03
coeff(1054)=-0.185690001884D+04
coeff(1055)=0.217200407883D+03
coeff(1056)=0.273691697919D+04
coeff(1057)=-0.194963895820D+03
coeff(1058)=0.291626213185D+03
coeff(1059)=0.461040315789D+03
coeff(1060)=-0.555208700965D+03
coeff(1061)=0.184077749186D+02
coeff(1062)=0.399990069689D+03
coeff(1063)=0.379256366215D+03
coeff(1064)=-0.270465360484D+03
coeff(1065)=0.188256213807D+04
coeff(1066)=0.535091414367D+03
coeff(1067)=0.591180417975D+03
coeff(1068)=-0.112296252782D+04
coeff(1069)=-0.217917777911D+03
coeff(1070)=-0.105793803555D+04
coeff(1071)=0.116128096490D+04
coeff(1072)=-0.216106492659D+04
coeff(1073)=-0.289269625358D+04
coeff(1074)=0.174949370003D+03
coeff(1075)=-0.949839884406D+03
coeff(1076)=-0.379395054276D+03
coeff(1077)=0.101134657190D+04
coeff(1078)=-0.437542252320D+02
coeff(1079)=-0.110165714142D+04
coeff(1080)=0.129962217069D+04
coeff(1081)=-0.191446646430D+04
coeff(1082)=0.135810618941D+02
coeff(1083)=0.331214402700D+02
coeff(1084)=0.576367416710D+03
coeff(1085)=0.667657817603D+03
coeff(1086)=-0.518920954910D+03
coeff(1087)=-0.259554618384D+03
coeff(1088)=-0.348896416716D+03
coeff(1089)=0.143297932656D+04
coeff(1090)=-0.271336607294D+03
coeff(1091)=-0.191640763668D+03
coeff(1092)=0.829115052051D+03
coeff(1093)=0.139152147968D+03
coeff(1094)=-0.266948247344D+03
coeff(1095)=-0.633333495983D+02
coeff(1096)=-0.219955001024D+03
coeff(1097)=0.122660331265D+03
coeff(1098)=0.111372139453D+04
coeff(1099)=0.514389282453D+03
coeff(1100)=0.102497239613D+04
coeff(1101)=-0.953680968063D+03
coeff(1102)=-0.214342426188D+03
coeff(1103)=-0.120575731124D+04
coeff(1104)=-0.931307904854D+03
coeff(1105)=0.339869930835D+03
coeff(1106)=-0.792432573590D+03
coeff(1107)=-0.168126806096D+04
coeff(1108)=0.515800168018D+03
coeff(1109)=0.144410595224D+04
coeff(1110)=0.112467833364D+04
coeff(1111)=0.121766276924D+04
coeff(1112)=-0.455143192261D+03
coeff(1113)=0.719692749364D+03
coeff(1114)=-0.169830550434D+04
coeff(1115)=0.145938863130D+04
coeff(1116)=-0.713931334232D+03
coeff(1117)=-0.490506343010D+03
coeff(1118)=-0.652068780660D+03
coeff(1119)=-0.387458264341D+03
coeff(1120)=0.845297329863D+03
coeff(1121)=0.161162012774D+04
coeff(1122)=-0.720247232625D+03
coeff(1123)=-0.538303300460D+03
coeff(1124)=-0.103742148085D+03
coeff(1125)=0.142310130763D+03
coeff(1126)=0.166676494292D+04
coeff(1127)=-0.263693738628D+03
coeff(1128)=0.924434177060D+02
coeff(1129)=0.571820697777D+03
coeff(1130)=0.713981919246D+03
coeff(1131)=-0.923847579264D+03
coeff(1132)=0.210519596155D+04
coeff(1133)=-0.514974203606D+03
coeff(1134)=-0.117857261231D+03
coeff(1135)=-0.735030158977D+02
coeff(1136)=-0.222980758394D+03
coeff(1137)=-0.196406221528D+03
coeff(1138)=0.190657603939D+03
coeff(1139)=0.401852459769D+03
coeff(1140)=-0.117139449522D+03
coeff(1141)=-0.163751064695D+04
coeff(1142)=-0.153339835406D+03
coeff(1143)=0.642423822218D+03
coeff(1144)=0.174844725942D+04
coeff(1145)=0.570065461080D+03
coeff(1146)=0.879536788923D+02
coeff(1147)=-0.252144894545D+03
coeff(1148)=-0.153729619233D+04
coeff(1149)=-0.707867313732D+03
coeff(1150)=-0.416619860285D+03
coeff(1151)=0.678815254377D+03
coeff(1152)=0.173552916277D+04
coeff(1153)=-0.921046063642D+02
coeff(1154)=0.611720894161D+03
coeff(1155)=0.886096772474D+03
coeff(1156)=-0.852382602560D+03
coeff(1157)=0.938798754022D+03
coeff(1158)=-0.736561449508D+03
coeff(1159)=-0.342356415500D+03
coeff(1160)=0.126989722595D+04
coeff(1161)=-0.130957513946D+04
coeff(1162)=0.328536066313D+03
coeff(1163)=-0.820329491763D+03
coeff(1164)=0.294438946523D+04
coeff(1165)=-0.106231614770D+04
coeff(1166)=-0.843126449605D+03
coeff(1167)=-0.519693030491D+03
coeff(1168)=0.844220248113D+03
coeff(1169)=-0.464595587317D+03
coeff(1170)=0.448798177772D+03
coeff(1171)=-0.856091874815D+03
coeff(1172)=-0.992775576462D+03
coeff(1173)=0.103085662401D+04
coeff(1174)=-0.101155407545D+03
coeff(1175)=-0.676269486512D+03
coeff(1176)=0.902882093934D+03
coeff(1177)=0.106830383514D+03
coeff(1178)=0.122310479650D+04
coeff(1179)=-0.192209415884D+04
coeff(1180)=0.353757018026D+03
coeff(1181)=-0.778589603889D+03
coeff(1182)=-0.580647609988D+03
coeff(1183)=0.118065722908D+04
coeff(1184)=-0.107178906684D+04
coeff(1185)=-0.120238181628D+04
coeff(1186)=-0.152996072528D+04
coeff(1187)=0.907953316496D+03
coeff(1188)=0.477086026465D+03
coeff(1189)=0.486225383397D+03
coeff(1190)=-0.710872672942D+03
coeff(1191)=-0.943225378009D+02
coeff(1192)=0.389120377156D+03
coeff(1193)=0.340563212395D+03
coeff(1194)=-0.112174936774D+02
coeff(1195)=0.367533100891D+02
coeff(1196)=0.714534904258D+01
coeff(1197)=-0.334357798518D+03
coeff(1198)=0.120460911760D+04
coeff(1199)=-0.231097763961D+03
coeff(1200)=-0.585461426661D+03
coeff(1201)=0.140360971716D+03
coeff(1202)=0.620950242988D+02
coeff(1203)=0.612574784885D+03
coeff(1204)=-0.510296861997D+03
coeff(1205)=-0.427926710416D+03
coeff(1206)=-0.242041793289D+03
coeff(1207)=0.814949788954D+02
coeff(1208)=-0.595965395831D+03
coeff(1209)=-0.557125617096D+03
coeff(1210)=0.193009899997D+03
coeff(1211)=-0.116750128521D+03
coeff(1212)=-0.570209373090D+03
coeff(1213)=-0.664242607855D+02
coeff(1214)=-0.719996042631D+02
coeff(1215)=-0.415317429964D+03
coeff(1216)=-0.688229273360D+02
coeff(1217)=0.411304431583D+03
coeff(1218)=-0.139555753994D+03
coeff(1219)=0.694721998796D+03
coeff(1220)=0.514263050619D+01
coeff(1221)=-0.488708701883D+03
coeff(1222)=-0.769053336325D+03
coeff(1223)=-0.815727679989D+02
coeff(1224)=0.318968158167D+03
coeff(1225)=0.128819748244D+03
coeff(1226)=0.822073703471D+03
coeff(1227)=0.984910975193D+03
coeff(1228)=0.805672219665D+03
coeff(1229)=-0.224587598859D+03
coeff(1230)=-0.232486523037D+03
coeff(1231)=-0.158885189981D+04
coeff(1232)=0.144246240684D+04
coeff(1233)=-0.102632943582D+04
coeff(1234)=0.326742913973D+04
coeff(1235)=-0.666988053692D+03
coeff(1236)=0.133953133729D+04
coeff(1237)=-0.682777355141D+03
coeff(1238)=-0.147145373147D+03
coeff(1239)=-0.796869187136D+02
coeff(1240)=-0.266657683070D+03
coeff(1241)=-0.359067127852D+02
coeff(1242)=0.172057677142D+03
coeff(1243)=0.415366757219D+03
coeff(1244)=0.160785025708D+04
coeff(1245)=0.120686548630D+04
coeff(1246)=-0.160349352273D+04
coeff(1247)=0.280540939297D+04
coeff(1248)=-0.233830375929D+03
coeff(1249)=0.847276705812D+03
coeff(1250)=0.195549411148D+04
coeff(1251)=-0.109562839622D+03
coeff(1252)=0.269596240467D+04
coeff(1253)=0.243451724422D+03
coeff(1254)=0.784620623665D+03
coeff(1255)=-0.105819991557D+04
coeff(1256)=0.200172317484D+03
coeff(1257)=-0.313950448945D+03
coeff(1258)=-0.999567157808D+03
coeff(1259)=0.199499797997D+03
coeff(1260)=-0.329945348255D+03
coeff(1261)=-0.127972977831D+04
coeff(1262)=-0.683832820977D+03
coeff(1263)=-0.407416222977D+03
coeff(1264)=0.173479424099D+04
coeff(1265)=0.189892968926D+04
coeff(1266)=-0.176744219466D+04
coeff(1267)=0.105637171190D+04
coeff(1268)=0.112656167898D+04
coeff(1269)=0.277854301027D+04
coeff(1270)=-0.324241341895D+03
coeff(1271)=-0.211843827775D+03
coeff(1272)=0.583890138512D+03
coeff(1273)=0.582008589218D+03
coeff(1274)=-0.197004014016D+04
coeff(1275)=-0.254067838532D+04
coeff(1276)=0.189010269550D+04
coeff(1277)=-0.149995674875D+04
coeff(1278)=-0.399523419879D+04
coeff(1279)=-0.329555930847D+04
coeff(1280)=0.246450042423D+04
coeff(1281)=-0.326766021864D+03
coeff(1282)=0.211926447872D+03
coeff(1283)=-0.990965418264D+03
coeff(1284)=-0.510123575326D+03
coeff(1285)=0.394752463149D+03
coeff(1286)=-0.241776577295D+04
coeff(1287)=-0.658481163770D+03
coeff(1288)=-0.222919933245D+04
coeff(1289)=-0.476457014935D+04
coeff(1290)=-0.151508454071D+04
coeff(1291)=-0.463257079945D+03
coeff(1292)=0.480148398862D+03
coeff(1293)=-0.189182523234D+04
coeff(1294)=-0.144577933756D+04
coeff(1295)=0.193330313399D+03
coeff(1296)=0.468050994089D+03
coeff(1297)=-0.381534687101D+03
coeff(1298)=0.637127888051D+02
coeff(1299)=-0.426152258364D+02
coeff(1300)=-0.308744829468D+02

! Calculate MEG
x(1)=exp(-((r(1)-ra1))/a1 -((r(1)-rb1)**2)/b1)
x(2)=exp(-((r(2)-ra1))/a1 -((r(2)-rb1)**2)/b1)
x(3)=exp(-((r(3)-ra2))/a2 -((r(3)-rb2)**2)/b2)
x(4)=exp(-((r(4)-ra1))/a1 -((r(4)-rb1)**2)/b1)
x(5)=exp(-((r(5)-ra2))/a2 -((r(5)-rb2)**2)/b2)
x(6)=exp(-((r(6)-ra2))/a2 -((r(6)-rb2)**2)/b2)

! Calculate dMEG/dr(6,6) for giving R(6)
! dmsdr(i,j)=0  i!=j
! Initialize dmsdr
      do i=1,6
        do j=1,6
          dmsdr(i,j)=0.0d0
        enddo
      enddo
dmsdr(1,1)=(-2.0d0*(r(1)-rb1)/b1-1/a1)*Exp(-(r(1)-ra1)/a1-((r(1)-rb1)**2.0d0)/b1)
dmsdr(2,2)=(-2.0d0*(r(2)-rb1)/b1-1/a1)*Exp(-(r(2)-ra1)/a1-((r(2)-rb1)**2.0d0)/b1)
dmsdr(3,3)=(-2.0d0*(r(3)-rb2)/b2-1/a2)*Exp(-(r(3)-ra2)/a2-((r(3)-rb2)**2.0d0)/b2)
dmsdr(4,4)=(-2.0d0*(r(4)-rb1)/b1-1/a1)*Exp(-(r(4)-ra1)/a1-((r(4)-rb1)**2.0d0)/b1)
dmsdr(5,5)=(-2.0d0*(r(5)-rb2)/b2-1/a2)*Exp(-(r(5)-ra2)/a2-((r(5)-rb2)**2.0d0)/b2)
dmsdr(6,6)=(-2.0d0*(r(6)-rb2)/b2-1/a2)*Exp(-(r(6)-ra2)/a2-((r(6)-rb2)**2.0d0)/b2)

rm(0) = 1.0D0
rm(1) = x(4)
rm(2) = x(2)
rm(3) = x(1)
rm(4) = x(6)
rm(5) = x(5)
rm(6) = x(3)
rm(7) = rm(1)*rm(2)
rm(8) = rm(1)*rm(3)
rm(9) = rm(2)*rm(3)
rm(10) = rm(3)*rm(4)
rm(11) = rm(2)*rm(5)
rm(12) = rm(1)*rm(6)
rm(13) = rm(4)*rm(5)
rm(14) = rm(4)*rm(6)
rm(15) = rm(5)*rm(6)
rm(16) = rm(1)*rm(9)
rm(17) = rm(1)*rm(10)
rm(18) = rm(2)*rm(10)
rm(19) = rm(1)*rm(11)
rm(20) = rm(3)*rm(11)
rm(21) = rm(2)*rm(12)
rm(22) = rm(3)*rm(12)
rm(23) = rm(2)*rm(13)
rm(24) = rm(3)*rm(13)
rm(25) = rm(1)*rm(14)
rm(26) = rm(3)*rm(14)
rm(27) = rm(1)*rm(15)
rm(28) = rm(2)*rm(15)
rm(29) = rm(4)*rm(15)
rm(30) = rm(2)*rm(24)
rm(31) = rm(1)*rm(26)
rm(32) = rm(1)*rm(28)

p(0) = rm(0)
p(1) = rm(1) + rm(2) + rm(3)
p(2) = rm(4) + rm(5) + rm(6)
p(3) = rm(7) + rm(8) + rm(9)
p(4) = rm(10) + rm(11) + rm(12)
p(5) = p(1)*p(2) - p(4)
p(6) = rm(13) + rm(14) + rm(15)
p(7) = p(1)*p(1) - p(3) - p(3)
p(8) = p(2)*p(2) - p(6) - p(6)
p(9) = rm(16)
p(10) = rm(17) + rm(18) + rm(19) + rm(20) + rm(21) + rm(22)
p(11) = p(2)*p(3) - p(10)
p(12) = rm(23) + rm(24) + rm(25) + rm(26) + rm(27) + rm(28)
p(13) = p(1)*p(6) - p(12)
p(14) = rm(29)
p(15) = p(1)*p(3) - p(9) - p(9) - p(9)
p(16) = p(1)*p(4) - p(10)
p(17) = p(2)*p(7) - p(16)
p(18) = p(2)*p(4) - p(12)
p(19) = p(1)*p(8) - p(18)
p(20) = p(2)*p(6) - p(14) - p(14) - p(14)
p(21) = p(1)*p(7) - p(15)
p(22) = p(2)*p(8) - p(20)
p(23) = p(9)*p(2)
p(24) = rm(30) + rm(31) + rm(32)
p(25) = p(3)*p(6) - p(24)
p(26) = p(14)*p(1)
p(27) = p(9)*p(1)
p(28) = p(3)*p(4) - p(23)
p(29) = p(1)*p(10) - p(23) - p(28) - p(23)
p(30) = p(1)*p(11) - p(23)
p(31) = p(1)*p(12) - p(25) - p(24) - p(24)
p(32) = p(1)*p(13) - p(25)
p(33) = p(4)*p(5) - p(25) - p(31)
p(34) = p(2)*p(11) - p(25)
p(35) = p(4)*p(6) - p(26)
p(36) = p(2)*p(12) - p(26) - p(35) - p(26)
p(37) = p(2)*p(13) - p(26)
p(38) = p(14)*p(2)
p(39) = p(3)*p(3) - p(27) - p(27)
p(40) = p(3)*p(7) - p(27)
p(41) = p(1)*p(16) - p(28)
p(42) = p(2)*p(21) - p(41)
p(43) = p(1)*p(18) - p(33)
p(44) = p(7)*p(8) - p(43)
p(45) = p(6)*p(6) - p(38) - p(38)
p(46) = p(2)*p(18) - p(35)
p(47) = p(1)*p(22) - p(46)
p(48) = p(6)*p(8) - p(38)
p(49) = p(1)*p(21) - p(40)
p(50) = p(2)*p(22) - p(48)
p(51) = p(9)*p(6)
p(52) = p(14)*p(3)
p(53) = p(9)*p(4)
p(54) = p(9)*p(5)
p(55) = p(1)*p(24) - p(51)
p(56) = p(3)*p(12) - p(51) - p(55) - p(51)
p(57) = p(3)*p(13) - p(51)
p(58) = p(14)*p(7)
p(59) = p(9)*p(8)
p(60) = p(2)*p(24) - p(52)
p(61) = p(4)*p(13) - p(58)
p(62) = p(6)*p(11) - p(52)
p(63) = p(14)*p(4)
p(64) = p(14)*p(5)
p(65) = p(9)*p(3)
p(66) = p(9)*p(7)
p(67) = p(3)*p(16) - p(53)
p(68) = p(4)*p(15) - p(54) - p(67)
p(69) = p(2)*p(39) - p(68)
p(70) = p(1)*p(29) - p(54) - p(68)
p(71) = p(7)*p(11) - p(53)
p(72) = p(1)*p(31) - p(56) - p(55)
p(73) = p(1)*p(32) - p(57)
p(74) = p(3)*p(18) - p(59)
p(75) = p(2)*p(29) - p(57) - p(55)
p(76) = p(1)*p(34) - p(59)
p(77) = p(6)*p(16) - p(58)
p(78) = p(1)*p(36) - p(62) - p(60)
p(79) = p(2)*p(32) - p(58)
p(80) = p(6)*p(12) - p(64) - p(63) - p(63)
p(81) = p(1)*p(45) - p(80)
p(82) = p(14)*p(6)
p(83) = p(2)*p(33) - p(61) - p(60)
p(84) = p(2)*p(34) - p(62)
p(85) = p(6)*p(18) - p(63)
p(86) = p(2)*p(36) - p(64) - p(80)
p(87) = p(8)*p(13) - p(63)
p(88) = p(14)*p(8)
p(89) = p(1)*p(39) - p(65)
p(90) = p(3)*p(21) - p(66)
p(91) = p(1)*p(41) - p(67)
p(92) = p(2)*p(49) - p(91)
p(93) = p(1)*p(43) - p(74)
p(94) = p(8)*p(21) - p(93)
p(95) = p(1)*p(46) - p(83)
p(96) = p(7)*p(22) - p(95)
p(97) = p(2)*p(45) - p(82)
p(98) = p(2)*p(46) - p(85)
p(99) = p(1)*p(50) - p(98)
p(100) = p(6)*p(22) - p(88)
p(101) = p(1)*p(49) - p(90)
p(102) = p(2)*p(50) - p(100)
p(103) = p(9)*p(14)
p(104) = p(9)*p(12)
p(105) = p(9)*p(13)
p(106) = p(14)*p(15)
p(107) = p(9)*p(20)
p(108) = p(14)*p(10)
p(109) = p(14)*p(11)
p(110) = p(9)*p(16)
p(111) = p(9)*p(10)
p(112) = p(9)*p(11)
p(113) = p(9)*p(17)
p(114) = p(3)*p(24) - p(104)
p(115) = p(7)*p(24) - p(105)
p(116) = p(3)*p(31) - p(104) - p(115)
p(117) = p(6)*p(39) - p(114)
p(118) = p(3)*p(32) - p(105)
p(119) = p(14)*p(21)
p(120) = p(9)*p(18)
p(121) = p(9)*p(19)
p(122) = p(4)*p(24) - p(103) - p(103) - p(103)
p(123) = p(1)*p(60) - p(107) - p(122)
p(124) = p(13)*p(16) - p(119)
p(125) = p(2)*p(56) - p(106) - p(124)
p(126) = p(4)*p(32) - p(119)
p(127) = p(1)*p(62) - p(107) - p(125)
p(128) = p(14)*p(16)
p(129) = p(14)*p(17)
p(130) = p(6)*p(24) - p(108)
p(131) = p(3)*p(45) - p(130)
p(132) = p(14)*p(12)
p(133) = p(14)*p(13)
p(134) = p(9)*p(22)
p(135) = p(8)*p(24) - p(109)
p(136) = p(13)*p(18) - p(128)
p(137) = p(6)*p(34) - p(109)
p(138) = p(14)*p(18)
p(139) = p(14)*p(19)
p(140) = p(9)*p(9)
p(141) = p(9)*p(15)
p(142) = p(9)*p(21)
p(143) = p(3)*p(41) - p(110)
p(144) = p(4)*p(39) - p(112)
p(145) = p(1)*p(68) - p(111) - p(144)
p(146) = p(1)*p(69) - p(112)
p(147) = p(1)*p(70) - p(113) - p(145)
p(148) = p(11)*p(21) - p(110)
p(149) = p(1)*p(72) - p(116) - p(115)
p(150) = p(1)*p(73) - p(118)
p(151) = p(3)*p(43) - p(120)
p(152) = p(15)*p(18) - p(121) - p(151)
p(153) = p(2)*p(69) - p(117)
p(154) = p(1)*p(75) - p(121) - p(152)
p(155) = p(7)*p(34) - p(120)
p(156) = p(6)*p(41) - p(119)
p(157) = p(1)*p(78) - p(125) - p(123)
p(158) = p(2)*p(73) - p(119)
p(159) = p(1)*p(80) - p(131) - p(130) - p(130)
p(160) = p(1)*p(81) - p(131)
p(161) = p(14)*p(14)
p(162) = p(3)*p(46) - p(134)
p(163) = p(2)*p(75) - p(126) - p(123)
p(164) = p(1)*p(84) - p(134)
p(165) = p(6)*p(43) - p(128)
p(166) = p(1)*p(86) - p(137) - p(135)
p(167) = p(8)*p(32) - p(128)
p(168) = p(4)*p(45) - p(133)
p(169) = p(2)*p(80) - p(132) - p(168)
p(170) = p(2)*p(81) - p(133)
p(171) = p(14)*p(20)
p(172) = p(2)*p(83) - p(136) - p(135)
p(173) = p(2)*p(84) - p(137)
p(174) = p(6)*p(46) - p(138)
p(175) = p(2)*p(86) - p(139) - p(169)
p(176) = p(13)*p(22) - p(138)
p(177) = p(14)*p(22)
p(178) = p(3)*p(39) - p(141)
p(179) = p(1)*p(89) - p(141) - p(178) - p(178)
p(180) = p(3)*p(49) - p(142)
p(181) = p(1)*p(91) - p(143)
p(182) = p(2)*p(101) - p(181)
p(183) = p(1)*p(93) - p(151)
p(184) = p(8)*p(49) - p(183)
p(185) = p(1)*p(95) - p(162)
p(186) = p(21)*p(22) - p(185)
p(187) = p(6)*p(45) - p(171)
p(188) = p(1)*p(98) - p(172)
p(189) = p(7)*p(50) - p(188)
p(190) = p(2)*p(97) - p(171) - p(187) - p(187)
p(191) = p(2)*p(98) - p(174)
p(192) = p(1)*p(102) - p(191)
p(193) = p(6)*p(50) - p(177)
p(194) = p(1)*p(101) - p(180)
p(195) = p(2)*p(102) - p(193)
p(196) = p(9)*p(26)
p(197) = p(9)*p(38)
p(198) = p(9)*p(24)
p(199) = p(9)*p(31)
p(200) = p(9)*p(25)
p(201) = p(9)*p(32)
p(202) = p(14)*p(39)
p(203) = p(14)*p(40)
p(204) = p(9)*p(35)
p(205) = p(9)*p(36)
p(206) = p(9)*p(37)
p(207) = p(14)*p(28)
p(208) = p(14)*p(29)
p(209) = p(14)*p(30)
p(210) = p(9)*p(45)
p(211) = p(14)*p(24)
p(212) = p(14)*p(25)
p(213) = p(9)*p(48)
p(214) = p(14)*p(33)
p(215) = p(14)*p(34)
p(216) = p(9)*p(41)
p(217) = p(9)*p(28)
p(218) = p(9)*p(23)
p(219) = p(9)*p(29)
p(220) = p(9)*p(30)
p(221) = p(9)*p(42)
p(222) = p(1)*p(114) - p(198)
p(223) = p(21)*p(24) - p(201)
p(224) = p(3)*p(72) - p(199) - p(223)
p(225) = p(1)*p(116) - p(199) - p(224)
p(226) = p(13)*p(39) - p(198)
p(227) = p(3)*p(73) - p(201)
p(228) = p(14)*p(49)
p(229) = p(9)*p(43)
p(230) = p(9)*p(33)
p(231) = p(9)*p(34)
p(232) = p(9)*p(44)
p(233) = p(16)*p(24) - p(196)
p(234) = p(2)*p(114) - p(202)
p(235) = p(1)*p(123) - p(205) - p(234)
p(236) = p(13)*p(41) - p(228)
p(237) = p(2)*p(116) - p(203) - p(236)
p(238) = p(16)*p(32) - p(228)
p(239) = p(6)*p(69) - p(202)
p(240) = p(4)*p(73) - p(228)
p(241) = p(11)*p(32) - p(196)
p(242) = p(14)*p(41)
p(243) = p(14)*p(42)
p(244) = p(1)*p(130) - p(210)
p(245) = p(4)*p(62) - p(213) - p(209)
p(246) = p(3)*p(81) - p(210)
p(247) = p(14)*p(31)
p(248) = p(14)*p(32)
p(249) = p(14)*p(26)
p(250) = p(9)*p(46)
p(251) = p(9)*p(47)
p(252) = p(18)*p(24) - p(197)
p(253) = p(1)*p(135) - p(213) - p(252)
p(254) = p(13)*p(43) - p(242)
p(255) = p(2)*p(125) - p(209) - p(245)
p(256) = p(18)*p(32) - p(242)
p(257) = p(13)*p(34) - p(197)
p(258) = p(14)*p(43)
p(259) = p(14)*p(44)
p(260) = p(2)*p(130) - p(211)
p(261) = p(4)*p(81) - p(248)
p(262) = p(11)*p(45) - p(211)
p(263) = p(14)*p(35)
p(264) = p(14)*p(36)
p(265) = p(14)*p(37)
p(266) = p(9)*p(50)
p(267) = p(22)*p(24) - p(215)
p(268) = p(13)*p(46) - p(258)
p(269) = p(6)*p(84) - p(215)
p(270) = p(14)*p(46)
p(271) = p(14)*p(47)
p(272) = p(9)*p(27)
p(273) = p(9)*p(39)
p(274) = p(9)*p(40)
p(275) = p(9)*p(49)
p(276) = p(3)*p(91) - p(216)
p(277) = p(16)*p(39) - p(218)
p(278) = p(1)*p(144) - p(217) - p(277)
p(279) = p(2)*p(178) - p(278)
p(280) = p(1)*p(145) - p(219) - p(278)
p(281) = p(7)*p(69) - p(218)
p(282) = p(1)*p(147) - p(221) - p(280)
p(283) = p(11)*p(49) - p(216)
p(284) = p(1)*p(149) - p(224) - p(223)
p(285) = p(1)*p(150) - p(227)
p(286) = p(3)*p(93) - p(229)
p(287) = p(18)*p(39) - p(231)
p(288) = p(1)*p(152) - p(230) - p(287)
p(289) = p(1)*p(153) - p(231)
p(290) = p(1)*p(154) - p(232) - p(288)
p(291) = p(21)*p(34) - p(229)
p(292) = p(6)*p(91) - p(228)
p(293) = p(1)*p(157) - p(237) - p(235)
p(294) = p(2)*p(150) - p(228)
p(295) = p(1)*p(159) - p(245) - p(244)
p(296) = p(1)*p(160) - p(246)
p(297) = p(3)*p(95) - p(250)
p(298) = p(2)*p(152) - p(238) - p(234)
p(299) = p(2)*p(153) - p(239)
p(300) = p(1)*p(163) - p(251) - p(298)
p(301) = p(7)*p(84) - p(250)
p(302) = p(6)*p(93) - p(242)
p(303) = p(1)*p(166) - p(255) - p(253)
p(304) = p(8)*p(73) - p(242)
p(305) = p(16)*p(45) - p(248)
p(306) = p(1)*p(169) - p(262) - p(260)
p(307) = p(2)*p(160) - p(248)
p(308) = p(14)*p(38)
p(309) = p(6)*p(80) - p(264) - p(263)
p(310) = p(1)*p(187) - p(309)
p(311) = p(14)*p(45)
p(312) = p(3)*p(98) - p(266)
p(313) = p(2)*p(163) - p(256) - p(253)
p(314) = p(1)*p(173) - p(266)
p(315) = p(6)*p(95) - p(258)
p(316) = p(1)*p(175) - p(269) - p(267)
p(317) = p(22)*p(32) - p(258)
p(318) = p(18)*p(45) - p(249)
p(319) = p(2)*p(169) - p(264) - p(309)
p(320) = p(8)*p(81) - p(249)
p(321) = p(14)*p(48)
p(322) = p(2)*p(172) - p(268) - p(267)
p(323) = p(2)*p(173) - p(269)
p(324) = p(6)*p(98) - p(270)
p(325) = p(2)*p(175) - p(271) - p(319)
p(326) = p(13)*p(50) - p(270)
p(327) = p(14)*p(50)
p(328) = p(1)*p(178) - p(273)
p(329) = p(21)*p(39) - p(272)
p(330) = p(3)*p(101) - p(275)
p(331) = p(1)*p(181) - p(276)
p(332) = p(2)*p(194) - p(331)
p(333) = p(1)*p(183) - p(286)
p(334) = p(8)*p(101) - p(333)
p(335) = p(1)*p(185) - p(297)
p(336) = p(22)*p(49) - p(335)
p(337) = p(1)*p(188) - p(312)
p(338) = p(21)*p(50) - p(337)
p(339) = p(2)*p(187) - p(311)
p(340) = p(1)*p(191) - p(322)
p(341) = p(7)*p(102) - p(340)
p(342) = p(22)*p(45) - p(308)
p(343) = p(2)*p(191) - p(324)
p(344) = p(1)*p(195) - p(343)
p(345) = p(6)*p(102) - p(327)
p(346) = p(1)*p(194) - p(330)
p(347) = p(2)*p(195) - p(345)
p(348) = p(9)*p(52)
p(349) = p(9)*p(58)
p(350) = p(9)*p(63)
p(351) = p(9)*p(64)
p(352) = p(9)*p(82)
p(353) = p(9)*p(88)
p(354) = p(9)*p(55)
p(355) = p(9)*p(72)
p(356) = p(9)*p(51)
p(357) = p(9)*p(56)
p(358) = p(9)*p(57)
p(359) = p(9)*p(73)
p(360) = p(14)*p(89)
p(361) = p(14)*p(90)
p(362) = p(9)*p(77)
p(363) = p(9)*p(60)
p(364) = p(9)*p(78)
p(365) = p(9)*p(61)
p(366) = p(9)*p(62)
p(367) = p(9)*p(79)
p(368) = p(14)*p(67)
p(369) = p(14)*p(68)
p(370) = p(14)*p(69)
p(371) = p(14)*p(70)
p(372) = p(14)*p(71)
p(373) = p(9)*p(80)
p(374) = p(9)*p(81)
p(375) = p(14)*p(55)
p(376) = p(14)*p(56)
p(377) = p(14)*p(57)
p(378) = p(14)*p(52)
p(379) = p(9)*p(85)
p(380) = p(9)*p(86)
p(381) = p(9)*p(87)
p(382) = p(14)*p(74)
p(383) = p(14)*p(75)
p(384) = p(14)*p(76)
p(385) = p(9)*p(97)
p(386) = p(14)*p(60)
p(387) = p(14)*p(61)
p(388) = p(14)*p(62)
p(389) = p(9)*p(100)
p(390) = p(14)*p(83)
p(391) = p(14)*p(84)
p(392) = p(9)*p(91)
p(393) = p(9)*p(67)
p(394) = p(9)*p(53)
p(395) = p(9)*p(68)
p(396) = p(9)*p(54)
p(397) = p(9)*p(69)
p(398) = p(9)*p(70)
p(399) = p(9)*p(71)
p(400) = p(9)*p(92)
p(401) = p(3)*p(114) - p(354)
p(402) = p(7)*p(114) - p(356)
p(403) = p(24)*p(49) - p(359)
p(404) = p(3)*p(149) - p(355) - p(403)
p(405) = p(1)*p(224) - p(355) - p(404)
p(406) = p(6)*p(178) - p(401)
p(407) = p(32)*p(39) - p(356)
p(408) = p(3)*p(150) - p(359)
p(409) = p(14)*p(101)
p(410) = p(9)*p(93)
p(411) = p(9)*p(74)
p(412) = p(9)*p(59)
p(413) = p(9)*p(75)
p(414) = p(9)*p(76)
p(415) = p(9)*p(94)
p(416) = p(24)*p(41) - p(349)
p(417) = p(4)*p(114) - p(348)
p(418) = p(1)*p(234) - p(363) - p(417)
p(419) = p(1)*p(235) - p(364) - p(418)
p(420) = p(13)*p(91) - p(409)
p(421) = p(2)*p(224) - p(361) - p(420)
p(422) = p(32)*p(41) - p(409)
p(423) = p(1)*p(237) - p(364) - p(421)
p(424) = p(16)*p(73) - p(409)
p(425) = p(13)*p(69) - p(348)
p(426) = p(4)*p(150) - p(409)
p(427) = p(11)*p(73) - p(349)
p(428) = p(14)*p(91)
p(429) = p(14)*p(92)
p(430) = p(3)*p(130) - p(373)
p(431) = p(7)*p(130) - p(374)
p(432) = p(3)*p(159) - p(373) - p(431)
p(433) = p(39)*p(45) - p(430)
p(434) = p(3)*p(160) - p(374)
p(435) = p(14)*p(72)
p(436) = p(14)*p(73)
p(437) = p(14)*p(58)
p(438) = p(9)*p(95)
p(439) = p(9)*p(83)
p(440) = p(9)*p(84)
p(441) = p(9)*p(96)
p(442) = p(24)*p(43) - p(350)
p(443) = p(8)*p(114) - p(370)
p(444) = p(1)*p(253) - p(380) - p(443)
p(445) = p(13)*p(93) - p(428)
p(446) = p(2)*p(237) - p(372) - p(432)
p(447) = p(32)*p(43) - p(428)
p(448) = p(6)*p(153) - p(370)
p(449) = p(18)*p(73) - p(428)
p(450) = p(32)*p(34) - p(350)
p(451) = p(14)*p(93)
p(452) = p(14)*p(94)
p(453) = p(4)*p(130) - p(352)
p(454) = p(1)*p(260) - p(385) - p(453)
p(455) = p(16)*p(81) - p(436)
p(456) = p(2)*p(245) - p(376) - p(455)
p(457) = p(4)*p(160) - p(436)
p(458) = p(11)*p(81) - p(352)
p(459) = p(14)*p(77)
p(460) = p(14)*p(78)
p(461) = p(14)*p(79)
p(462) = p(14)*p(63)
p(463) = p(14)*p(64)
p(464) = p(6)*p(130) - p(386)
p(465) = p(3)*p(187) - p(464)
p(466) = p(14)*p(80)
p(467) = p(14)*p(81)
p(468) = p(9)*p(98)
p(469) = p(9)*p(99)
p(470) = p(24)*p(46) - p(353)
p(471) = p(1)*p(267) - p(389) - p(470)
p(472) = p(13)*p(95) - p(451)
p(473) = p(2)*p(255) - p(384) - p(456)
p(474) = p(32)*p(46) - p(451)
p(475) = p(13)*p(84) - p(353)
p(476) = p(14)*p(95)
p(477) = p(14)*p(96)
p(478) = p(8)*p(130) - p(378)
p(479) = p(18)*p(81) - p(437)
p(480) = p(34)*p(45) - p(378)
p(481) = p(14)*p(85)
p(482) = p(14)*p(86)
p(483) = p(14)*p(87)
p(484) = p(9)*p(102)
p(485) = p(24)*p(50) - p(391)
p(486) = p(13)*p(98) - p(476)
p(487) = p(6)*p(173) - p(391)
p(488) = p(14)*p(98)
p(489) = p(14)*p(99)
p(490) = p(9)*p(65)
p(491) = p(9)*p(66)
p(492) = p(9)*p(89)
p(493) = p(9)*p(90)
p(494) = p(9)*p(101)
p(495) = p(3)*p(181) - p(392)
p(496) = p(39)*p(41) - p(394)
p(497) = p(4)*p(178) - p(397)
p(498) = p(1)*p(278) - p(395) - p(497)
p(499) = p(1)*p(279) - p(397)
p(500) = p(1)*p(280) - p(398) - p(498)
p(501) = p(21)*p(69) - p(394)
p(502) = p(1)*p(282) - p(400) - p(500)
p(503) = p(11)*p(101) - p(392)
p(504) = p(1)*p(284) - p(404) - p(403)
p(505) = p(1)*p(285) - p(408)
p(506) = p(3)*p(183) - p(410)
p(507) = p(39)*p(43) - p(412)
p(508) = p(1)*p(287) - p(411) - p(507)
p(509) = p(2)*p(279) - p(406)
p(510) = p(1)*p(288) - p(413) - p(508)
p(511) = p(7)*p(153) - p(412)
p(512) = p(1)*p(290) - p(415) - p(510)
p(513) = p(34)*p(49) - p(410)
p(514) = p(6)*p(181) - p(409)
p(515) = p(1)*p(293) - p(421) - p(419)
p(516) = p(2)*p(285) - p(409)
p(517) = p(1)*p(295) - p(432) - p(431)
p(518) = p(1)*p(296) - p(434)
p(519) = p(3)*p(185) - p(438)
p(520) = p(39)*p(46) - p(440)
p(521) = p(1)*p(298) - p(439) - p(520)
p(522) = p(1)*p(299) - p(440)
p(523) = p(1)*p(300) - p(441) - p(521)
p(524) = p(21)*p(84) - p(438)
p(525) = p(6)*p(183) - p(428)
p(526) = p(1)*p(303) - p(446) - p(444)
p(527) = p(8)*p(150) - p(428)
p(528) = p(41)*p(45) - p(436)
p(529) = p(1)*p(306) - p(456) - p(454)
p(530) = p(2)*p(296) - p(436)
p(531) = p(4)*p(169) - p(388) - p(478)
p(532) = p(1)*p(310) - p(465)
p(533) = p(14)*p(82)
p(534) = p(3)*p(188) - p(468)
p(535) = p(2)*p(298) - p(447) - p(443)
p(536) = p(2)*p(299) - p(448)
p(537) = p(1)*p(313) - p(469) - p(535)
p(538) = p(7)*p(173) - p(468)
p(539) = p(6)*p(185) - p(451)
p(540) = p(1)*p(316) - p(473) - p(471)
p(541) = p(22)*p(73) - p(451)
p(542) = p(43)*p(45) - p(437)
p(543) = p(1)*p(319) - p(480) - p(478)
p(544) = p(8)*p(160) - p(437)
p(545) = p(14)*p(88)
p(546) = p(4)*p(187) - p(467)
p(547) = p(2)*p(309) - p(466) - p(546)
p(548) = p(2)*p(310) - p(467)
p(549) = p(14)*p(97)
p(550) = p(3)*p(191) - p(484)
p(551) = p(2)*p(313) - p(474) - p(471)
p(552) = p(1)*p(323) - p(484)
p(553) = p(6)*p(188) - p(476)
p(554) = p(1)*p(325) - p(487) - p(485)
p(555) = p(32)*p(50) - p(476)
p(556) = p(45)*p(46) - p(462)
p(557) = p(2)*p(319) - p(482) - p(547)
p(558) = p(22)*p(81) - p(462)
p(559) = p(14)*p(100)
p(560) = p(2)*p(322) - p(486) - p(485)
p(561) = p(2)*p(323) - p(487)
p(562) = p(6)*p(191) - p(488)
p(563) = p(2)*p(325) - p(489) - p(557)
p(564) = p(13)*p(102) - p(488)
p(565) = p(14)*p(102)
p(566) = p(3)*p(178) - p(492)
p(567) = p(7)*p(178) - p(490)
p(568) = p(39)*p(49) - p(491)
p(569) = p(3)*p(194) - p(494)
p(570) = p(1)*p(331) - p(495)
p(571) = p(2)*p(346) - p(570)
p(572) = p(1)*p(333) - p(506)
p(573) = p(8)*p(194) - p(572)
p(574) = p(1)*p(335) - p(519)
p(575) = p(22)*p(101) - p(574)
p(576) = p(1)*p(337) - p(534)
p(577) = p(49)*p(50) - p(576)
p(578) = p(6)*p(187) - p(549)
p(579) = p(1)*p(340) - p(550)
p(580) = p(21)*p(102) - p(579)
p(581) = p(8)*p(187) - p(533)
p(582) = p(1)*p(343) - p(560)
p(583) = p(7)*p(195) - p(582)
p(584) = p(45)*p(50) - p(545)
p(585) = p(2)*p(343) - p(562)
p(586) = p(1)*p(347) - p(585)
p(587) = p(6)*p(195) - p(565)
p(588) = p(1)*p(346) - p(569)
p(589) = p(2)*p(347) - p(587)
p(590) = p(9)*p(103)
p(591) = p(9)*p(106)
p(592) = p(9)*p(119)
p(593) = p(9)*p(128)
p(594) = p(9)*p(108)
p(595) = p(9)*p(109)
p(596) = p(9)*p(129)
p(597) = p(9)*p(132)
p(598) = p(9)*p(133)
p(599) = p(9)*p(161)
p(600) = p(9)*p(138)
p(601) = p(9)*p(139)
p(602) = p(9)*p(171)
p(603) = p(9)*p(177)
p(604) = p(9)*p(114)
p(605) = p(9)*p(115)
p(606) = p(9)*p(149)
p(607) = p(9)*p(104)
p(608) = p(9)*p(116)
p(609) = p(9)*p(105)
p(610) = p(9)*p(117)
p(611) = p(9)*p(118)
p(612) = p(9)*p(150)
p(613) = p(14)*p(178)
p(614) = p(14)*p(179)
p(615) = p(14)*p(180)
p(616) = p(9)*p(156)
p(617) = p(9)*p(122)
p(618) = p(9)*p(123)
p(619) = p(9)*p(157)
p(620) = p(9)*p(124)
p(621) = p(9)*p(107)
p(622) = p(9)*p(125)
p(623) = p(9)*p(126)
p(624) = p(9)*p(127)
p(625) = p(9)*p(158)
p(626) = p(14)*p(143)
p(627) = p(14)*p(144)
p(628) = p(14)*p(145)
p(629) = p(14)*p(146)
p(630) = p(14)*p(147)
p(631) = p(14)*p(148)
p(632) = p(9)*p(130)
p(633) = p(9)*p(159)
p(634) = p(9)*p(131)
p(635) = p(9)*p(160)
p(636) = p(14)*p(114)
p(637) = p(14)*p(115)
p(638) = p(14)*p(116)
p(639) = p(14)*p(117)
p(640) = p(14)*p(118)
p(641) = p(14)*p(106)
p(642) = p(9)*p(165)
p(643) = p(9)*p(135)
p(644) = p(9)*p(166)
p(645) = p(9)*p(136)
p(646) = p(9)*p(137)
p(647) = p(9)*p(167)
p(648) = p(14)*p(151)
p(649) = p(14)*p(152)
p(650) = p(14)*p(153)
p(651) = p(14)*p(154)
p(652) = p(14)*p(155)
p(653) = p(9)*p(168)
p(654) = p(9)*p(169)
p(655) = p(9)*p(170)
p(656) = p(14)*p(122)
p(657) = p(14)*p(123)
p(658) = p(14)*p(124)
p(659) = p(14)*p(125)
p(660) = p(14)*p(126)
p(661) = p(14)*p(127)
p(662) = p(14)*p(108)
p(663) = p(14)*p(109)
p(664) = p(9)*p(187)
p(665) = p(14)*p(130)
p(666) = p(14)*p(131)
p(667) = p(9)*p(174)
p(668) = p(9)*p(175)
p(669) = p(9)*p(176)
p(670) = p(14)*p(162)
p(671) = p(14)*p(163)
p(672) = p(14)*p(164)
p(673) = p(9)*p(190)
p(674) = p(14)*p(135)
p(675) = p(14)*p(136)
p(676) = p(14)*p(137)
p(677) = p(9)*p(193)
p(678) = p(14)*p(172)
p(679) = p(14)*p(173)
p(680) = p(9)*p(181)
p(681) = p(9)*p(143)
p(682) = p(9)*p(110)
p(683) = p(9)*p(144)
p(684) = p(9)*p(111)
p(685) = p(9)*p(112)
p(686) = p(9)*p(145)
p(687) = p(9)*p(113)
p(688) = p(9)*p(146)
p(689) = p(9)*p(147)
p(690) = p(9)*p(148)
p(691) = p(9)*p(182)
p(692) = p(1)*p(401) - p(604)
p(693) = p(21)*p(114) - p(609)
p(694) = p(24)*p(101) - p(612)
p(695) = p(3)*p(284) - p(606) - p(694)
p(696) = p(1)*p(404) - p(606) - p(695)
p(697) = p(1)*p(405) - p(608) - p(696)
p(698) = p(13)*p(178) - p(604)
p(699) = p(39)*p(73) - p(609)
p(700) = p(3)*p(285) - p(612)
p(701) = p(14)*p(194)
p(702) = p(9)*p(183)
p(703) = p(9)*p(151)
p(704) = p(9)*p(120)
p(705) = p(9)*p(152)
p(706) = p(9)*p(121)
p(707) = p(9)*p(153)
p(708) = p(9)*p(154)
p(709) = p(9)*p(155)
p(710) = p(9)*p(184)
p(711) = p(24)*p(91) - p(592)
p(712) = p(1)*p(416) - p(616) - p(711)
p(713) = p(2)*p(401) - p(613)
p(714) = p(1)*p(418) - p(618) - p(713)
p(715) = p(1)*p(419) - p(619) - p(714)
p(716) = p(13)*p(181) - p(701)
p(717) = p(2)*p(404) - p(615) - p(716)
p(718) = p(32)*p(91) - p(701)
p(719) = p(1)*p(421) - p(619) - p(717)
p(720) = p(41)*p(73) - p(701)
p(721) = p(6)*p(279) - p(613)
p(722) = p(16)*p(150) - p(701)
p(723) = p(1)*p(425) - p(624) - p(721)
p(724) = p(4)*p(285) - p(701)
p(725) = p(11)*p(150) - p(592)
p(726) = p(14)*p(181)
p(727) = p(14)*p(182)
p(728) = p(1)*p(430) - p(632)
p(729) = p(21)*p(130) - p(635)
p(730) = p(3)*p(295) - p(633) - p(729)
p(731) = p(1)*p(432) - p(633) - p(730)
p(732) = p(39)*p(81) - p(632)
p(733) = p(3)*p(296) - p(635)
p(734) = p(14)*p(149)
p(735) = p(14)*p(150)
p(736) = p(14)*p(119)
p(737) = p(9)*p(185)
p(738) = p(9)*p(162)
p(739) = p(9)*p(134)
p(740) = p(9)*p(163)
p(741) = p(9)*p(164)
p(742) = p(9)*p(186)
p(743) = p(24)*p(93) - p(593)
p(744) = p(18)*p(114) - p(595)
p(745) = p(1)*p(443) - p(643) - p(744)
p(746) = p(1)*p(444) - p(644) - p(745)
p(747) = p(13)*p(183) - p(726)
p(748) = p(2)*p(421) - p(631) - p(730)
p(749) = p(32)*p(93) - p(726)
p(750) = p(1)*p(446) - p(644) - p(748)
p(751) = p(43)*p(73) - p(726)
p(752) = p(13)*p(153) - p(595)
p(753) = p(18)*p(150) - p(726)
p(754) = p(34)*p(73) - p(593)
p(755) = p(14)*p(183)
p(756) = p(14)*p(184)
p(757) = p(16)*p(130) - p(598)
p(758) = p(2)*p(430) - p(636)
p(759) = p(1)*p(454) - p(654) - p(758)
p(760) = p(41)*p(81) - p(735)
p(761) = p(2)*p(432) - p(638) - p(760)
p(762) = p(16)*p(160) - p(735)
p(763) = p(45)*p(69) - p(636)
p(764) = p(4)*p(296) - p(735)
p(765) = p(11)*p(160) - p(598)
p(766) = p(14)*p(156)
p(767) = p(14)*p(157)
p(768) = p(14)*p(158)
p(769) = p(14)*p(128)
p(770) = p(14)*p(129)
p(771) = p(1)*p(464) - p(664)
p(772) = p(4)*p(262) - p(673) - p(661)
p(773) = p(3)*p(310) - p(664)
p(774) = p(14)*p(159)
p(775) = p(14)*p(160)
p(776) = p(14)*p(132)
p(777) = p(14)*p(133)
p(778) = p(9)*p(188)
p(779) = p(9)*p(172)
p(780) = p(9)*p(173)
p(781) = p(9)*p(189)
p(782) = p(24)*p(95) - p(600)
p(783) = p(22)*p(114) - p(650)
p(784) = p(1)*p(471) - p(668) - p(783)
p(785) = p(13)*p(185) - p(755)
p(786) = p(2)*p(446) - p(652) - p(761)
p(787) = p(32)*p(95) - p(755)
p(788) = p(6)*p(299) - p(650)
p(789) = p(46)*p(73) - p(755)
p(790) = p(32)*p(84) - p(600)
p(791) = p(14)*p(185)
p(792) = p(14)*p(186)
p(793) = p(2)*p(453) - p(656) - p(771)
p(794) = p(1)*p(478) - p(673) - p(793)
p(795) = p(43)*p(81) - p(736)
p(796) = p(2)*p(456) - p(659) - p(772)
p(797) = p(18)*p(160) - p(736)
p(798) = p(1)*p(480) - p(673) - p(796)
p(799) = p(14)*p(165)
p(800) = p(14)*p(166)
p(801) = p(14)*p(167)
p(802) = p(14)*p(138)
p(803) = p(14)*p(139)
p(804) = p(2)*p(464) - p(665)
p(805) = p(4)*p(310) - p(775)
p(806) = p(11)*p(187) - p(665)
p(807) = p(14)*p(168)
p(808) = p(14)*p(169)
p(809) = p(14)*p(170)
p(810) = p(9)*p(191)
p(811) = p(9)*p(192)
p(812) = p(24)*p(98) - p(603)
p(813) = p(1)*p(485) - p(677) - p(812)
p(814) = p(13)*p(188) - p(791)
p(815) = p(2)*p(473) - p(672) - p(796)
p(816) = p(32)*p(98) - p(791)
p(817) = p(13)*p(173) - p(603)
p(818) = p(14)*p(188)
p(819) = p(14)*p(189)
p(820) = p(22)*p(130) - p(663)
p(821) = p(46)*p(81) - p(769)
p(822) = p(45)*p(84) - p(663)
p(823) = p(14)*p(174)
p(824) = p(14)*p(175)
p(825) = p(14)*p(176)
p(826) = p(9)*p(195)
p(827) = p(24)*p(102) - p(679)
p(828) = p(13)*p(191) - p(818)
p(829) = p(6)*p(323) - p(679)
p(830) = p(14)*p(191)
p(831) = p(14)*p(192)
p(832) = p(9)*p(140)
p(833) = p(9)*p(141)
p(834) = p(9)*p(178)
p(835) = p(9)*p(142)
p(836) = p(9)*p(179)
p(837) = p(9)*p(180)
p(838) = p(9)*p(194)
p(839) = p(3)*p(331) - p(680)
p(840) = p(39)*p(91) - p(682)
p(841) = p(16)*p(178) - p(685)
p(842) = p(1)*p(497) - p(683) - p(841)
p(843) = p(2)*p(566) - p(842)
p(844) = p(1)*p(498) - p(686) - p(842)
p(845) = p(7)*p(279) - p(685)
p(846) = p(1)*p(500) - p(689) - p(844)
p(847) = p(49)*p(69) - p(682)
p(848) = p(1)*p(502) - p(691) - p(846)
p(849) = p(11)*p(194) - p(680)
p(850) = p(1)*p(504) - p(695) - p(694)
p(851) = p(1)*p(505) - p(700)
p(852) = p(3)*p(333) - p(702)
p(853) = p(39)*p(93) - p(704)
p(854) = p(18)*p(178) - p(707)
p(855) = p(1)*p(508) - p(705) - p(854)
p(856) = p(1)*p(509) - p(707)
p(857) = p(1)*p(510) - p(708) - p(855)
p(858) = p(21)*p(153) - p(704)
p(859) = p(1)*p(512) - p(710) - p(857)
p(860) = p(34)*p(101) - p(702)
p(861) = p(6)*p(331) - p(701)
p(862) = p(1)*p(515) - p(717) - p(715)
p(863) = p(2)*p(505) - p(701)
p(864) = p(1)*p(517) - p(730) - p(729)
p(865) = p(1)*p(518) - p(733)
p(866) = p(3)*p(335) - p(737)
p(867) = p(39)*p(95) - p(739)
p(868) = p(1)*p(520) - p(738) - p(867)
p(869) = p(2)*p(509) - p(721)
p(870) = p(1)*p(521) - p(740) - p(868)
p(871) = p(7)*p(299) - p(739)
p(872) = p(1)*p(523) - p(742) - p(870)
p(873) = p(49)*p(84) - p(737)
p(874) = p(6)*p(333) - p(726)
p(875) = p(1)*p(526) - p(748) - p(746)
p(876) = p(8)*p(285) - p(726)
p(877) = p(45)*p(91) - p(735)
p(878) = p(1)*p(529) - p(761) - p(759)
p(879) = p(2)*p(518) - p(735)
p(880) = p(1)*p(531) - p(772) - p(771)
p(881) = p(1)*p(532) - p(773)
p(882) = p(14)*p(161)
p(883) = p(3)*p(337) - p(778)
p(884) = p(39)*p(98) - p(780)
p(885) = p(1)*p(535) - p(779) - p(884)
p(886) = p(1)*p(536) - p(780)
p(887) = p(1)*p(537) - p(781) - p(885)
p(888) = p(21)*p(173) - p(778)
p(889) = p(6)*p(335) - p(755)
p(890) = p(1)*p(540) - p(786) - p(784)
p(891) = p(22)*p(150) - p(755)
p(892) = p(45)*p(93) - p(736)
p(893) = p(1)*p(543) - p(796) - p(794)
p(894) = p(8)*p(296) - p(736)
p(895) = p(16)*p(187) - p(775)
p(896) = p(1)*p(547) - p(806) - p(804)
p(897) = p(2)*p(532) - p(775)
p(898) = p(14)*p(171)
p(899) = p(6)*p(309) - p(808) - p(807)
p(900) = p(1)*p(578) - p(899)
p(901) = p(14)*p(187)
p(902) = p(3)*p(340) - p(810)
p(903) = p(2)*p(535) - p(787) - p(783)
p(904) = p(2)*p(536) - p(788)
p(905) = p(1)*p(551) - p(811) - p(903)
p(906) = p(7)*p(323) - p(810)
p(907) = p(6)*p(337) - p(791)
p(908) = p(1)*p(554) - p(815) - p(813)
p(909) = p(50)*p(73) - p(791)
p(910) = p(45)*p(95) - p(769)
p(911) = p(1)*p(557) - p(822) - p(820)
p(912) = p(22)*p(160) - p(769)
p(913) = p(14)*p(177)
p(914) = p(18)*p(187) - p(777)
p(915) = p(2)*p(547) - p(808) - p(899)
p(916) = p(8)*p(310) - p(777)
p(917) = p(14)*p(190)
p(918) = p(3)*p(343) - p(826)
p(919) = p(2)*p(551) - p(816) - p(813)
p(920) = p(1)*p(561) - p(826)
p(921) = p(6)*p(340) - p(818)
p(922) = p(1)*p(563) - p(829) - p(827)
p(923) = p(32)*p(102) - p(818)
p(924) = p(45)*p(98) - p(802)
p(925) = p(2)*p(557) - p(824) - p(915)
p(926) = p(50)*p(81) - p(802)
p(927) = p(14)*p(193)
p(928) = p(2)*p(560) - p(828) - p(827)
p(929) = p(2)*p(561) - p(829)
p(930) = p(6)*p(343) - p(830)
p(931) = p(2)*p(563) - p(831) - p(925)
p(932) = p(13)*p(195) - p(830)
p(933) = p(14)*p(195)
p(934) = p(1)*p(566) - p(834)
p(935) = p(1)*p(567) - p(836) - p(934)
p(936) = p(39)*p(101) - p(835)
p(937) = p(3)*p(346) - p(838)
p(938) = p(1)*p(570) - p(839)
p(939) = p(2)*p(588) - p(938)
p(940) = p(1)*p(572) - p(852)
p(941) = p(8)*p(346) - p(940)
p(942) = p(1)*p(574) - p(866)
p(943) = p(22)*p(194) - p(942)
p(944) = p(1)*p(576) - p(883)
p(945) = p(50)*p(101) - p(944)
p(946) = p(1)*p(579) - p(902)
p(947) = p(49)*p(102) - p(946)
p(948) = p(2)*p(578) - p(901)
p(949) = p(1)*p(582) - p(918)
p(950) = p(21)*p(195) - p(949)
p(951) = p(2)*p(581) - p(917) - p(948)
p(952) = p(1)*p(585) - p(928)
p(953) = p(7)*p(347) - p(952)
p(954) = p(45)*p(102) - p(913)
p(955) = p(2)*p(585) - p(930)
p(956) = p(1)*p(589) - p(955)
p(957) = p(6)*p(347) - p(933)
p(958) = p(1)*p(588) - p(937)
p(959) = p(2)*p(589) - p(957)
p(960) = p(9)*p(196)
p(961) = p(9)*p(202)
p(962) = p(9)*p(203)
p(963) = p(9)*p(228)
p(964) = p(9)*p(242)
p(965) = p(9)*p(207)
p(966) = p(9)*p(197)
p(967) = p(9)*p(208)
p(968) = p(9)*p(209)
p(969) = p(9)*p(243)
p(970) = p(9)*p(211)
p(971) = p(9)*p(247)
p(972) = p(9)*p(212)
p(973) = p(9)*p(248)
p(974) = p(9)*p(249)
p(975) = p(9)*p(258)
p(976) = p(9)*p(214)
p(977) = p(9)*p(215)
p(978) = p(9)*p(259)
p(979) = p(9)*p(263)
p(980) = p(9)*p(264)
p(981) = p(9)*p(265)
p(982) = p(9)*p(308)
p(983) = p(9)*p(311)
p(984) = p(9)*p(270)
p(985) = p(9)*p(271)
p(986) = p(9)*p(321)
p(987) = p(9)*p(327)
p(988) = p(9)*p(222)
p(989) = p(9)*p(223)
p(990) = p(9)*p(284)
p(991) = p(9)*p(198)
p(992) = p(9)*p(199)
p(993) = p(9)*p(224)
p(994) = p(9)*p(200)
p(995) = p(9)*p(225)
p(996) = p(9)*p(201)
p(997) = p(9)*p(226)
p(998) = p(9)*p(227)
p(999) = p(9)*p(285)
p(1000) = p(14)*p(328)
p(1001) = p(14)*p(329)
p(1002) = p(14)*p(330)
p(1003) = p(9)*p(292)
p(1004) = p(9)*p(233)
p(1005) = p(9)*p(234)
p(1006) = p(9)*p(235)
p(1007) = p(9)*p(293)
p(1008) = p(9)*p(236)
p(1009) = p(9)*p(204)
p(1010) = p(9)*p(205)
p(1011) = p(9)*p(237)
p(1012) = p(9)*p(238)
p(1013) = p(9)*p(206)
p(1014) = p(9)*p(239)
p(1015) = p(9)*p(240)
p(1016) = p(9)*p(241)
p(1017) = p(9)*p(294)
p(1018) = p(14)*p(276)
p(1019) = p(14)*p(277)
p(1020) = p(14)*p(278)
p(1021) = p(14)*p(279)
p(1022) = p(14)*p(280)
p(1023) = p(14)*p(281)
p(1024) = p(14)*p(282)
p(1025) = p(14)*p(283)
p(1026) = p(9)*p(244)
p(1027) = p(9)*p(295)
p(1028) = p(9)*p(210)
p(1029) = p(9)*p(245)
p(1030) = p(9)*p(246)
p(1031) = p(9)*p(296)
p(1032) = p(14)*p(222)
p(1033) = p(14)*p(223)
p(1034) = p(14)*p(224)
p(1035) = p(14)*p(225)
p(1036) = p(14)*p(226)
p(1037) = p(14)*p(227)
p(1038) = p(14)*p(202)
p(1039) = p(14)*p(203)
p(1040) = p(9)*p(302)
p(1041) = p(9)*p(252)
p(1042) = p(9)*p(253)
p(1043) = p(9)*p(303)
p(1044) = p(9)*p(254)
p(1045) = p(9)*p(213)
p(1046) = p(9)*p(255)
p(1047) = p(9)*p(256)
p(1048) = p(9)*p(257)
p(1049) = p(9)*p(304)
p(1050) = p(14)*p(286)
p(1051) = p(14)*p(287)
p(1052) = p(14)*p(288)
p(1053) = p(14)*p(289)
p(1054) = p(14)*p(290)
p(1055) = p(14)*p(291)
p(1056) = p(9)*p(305)
p(1057) = p(9)*p(260)
p(1058) = p(9)*p(306)
p(1059) = p(9)*p(261)
p(1060) = p(9)*p(262)
p(1061) = p(9)*p(307)
p(1062) = p(14)*p(233)
p(1063) = p(14)*p(234)
p(1064) = p(14)*p(235)
p(1065) = p(14)*p(236)
p(1066) = p(14)*p(237)
p(1067) = p(14)*p(238)
p(1068) = p(14)*p(239)
p(1069) = p(14)*p(240)
p(1070) = p(14)*p(241)
p(1071) = p(14)*p(207)
p(1072) = p(14)*p(208)
p(1073) = p(14)*p(209)
p(1074) = p(9)*p(309)
p(1075) = p(9)*p(310)
p(1076) = p(14)*p(244)
p(1077) = p(14)*p(245)
p(1078) = p(14)*p(246)
p(1079) = p(14)*p(211)
p(1080) = p(14)*p(212)
p(1081) = p(9)*p(315)
p(1082) = p(9)*p(267)
p(1083) = p(9)*p(316)
p(1084) = p(9)*p(268)
p(1085) = p(9)*p(269)
p(1086) = p(9)*p(317)
p(1087) = p(14)*p(297)
p(1088) = p(14)*p(298)
p(1089) = p(14)*p(299)
p(1090) = p(14)*p(300)
p(1091) = p(14)*p(301)
p(1092) = p(9)*p(318)
p(1093) = p(9)*p(319)
p(1094) = p(9)*p(320)
p(1095) = p(14)*p(252)
p(1096) = p(14)*p(253)
p(1097) = p(14)*p(254)
p(1098) = p(14)*p(255)
p(1099) = p(14)*p(256)
p(1100) = p(14)*p(257)
p(1101) = p(14)*p(214)
p(1102) = p(14)*p(215)
p(1103) = p(9)*p(339)
p(1104) = p(14)*p(260)
p(1105) = p(14)*p(261)
p(1106) = p(14)*p(262)
p(1107) = p(9)*p(324)
p(1108) = p(9)*p(325)
p(1109) = p(9)*p(326)
p(1110) = p(14)*p(312)
p(1111) = p(14)*p(313)
p(1112) = p(14)*p(314)
p(1113) = p(9)*p(342)
p(1114) = p(14)*p(267)
p(1115) = p(14)*p(268)
p(1116) = p(14)*p(269)
p(1117) = p(9)*p(345)
p(1118) = p(14)*p(322)
p(1119) = p(14)*p(323)
p(1120) = p(9)*p(331)
p(1121) = p(9)*p(276)
p(1122) = p(9)*p(216)
p(1123) = p(9)*p(277)
p(1124) = p(9)*p(217)
p(1125) = p(9)*p(218)
p(1126) = p(9)*p(278)
p(1127) = p(9)*p(219)
p(1128) = p(9)*p(220)
p(1129) = p(9)*p(279)
p(1130) = p(9)*p(280)
p(1131) = p(9)*p(221)
p(1132) = p(9)*p(281)
p(1133) = p(9)*p(282)
p(1134) = p(9)*p(283)
p(1135) = p(9)*p(332)
p(1136) = p(3)*p(401) - p(988)
p(1137) = p(7)*p(401) - p(991)
p(1138) = p(49)*p(114) - p(996)
p(1139) = p(24)*p(194) - p(999)
p(1140) = p(3)*p(504) - p(990) - p(1139)
p(1141) = p(1)*p(695) - p(990) - p(1140)
p(1142) = p(1)*p(696) - p(993) - p(1141)
p(1143) = p(6)*p(566) - p(1136)
p(1144) = p(32)*p(178) - p(991)
p(1145) = p(39)*p(150) - p(996)
p(1146) = p(3)*p(505) - p(999)
p(1147) = p(14)*p(346)
p(1148) = p(9)*p(333)
p(1149) = p(9)*p(286)
p(1150) = p(9)*p(229)
p(1151) = p(9)*p(287)
p(1152) = p(9)*p(230)
p(1153) = p(9)*p(231)
p(1154) = p(9)*p(288)
p(1155) = p(9)*p(232)
p(1156) = p(9)*p(289)
p(1157) = p(9)*p(290)
p(1158) = p(9)*p(291)
p(1159) = p(9)*p(334)
p(1160) = p(24)*p(181) - p(963)
p(1161) = p(41)*p(114) - p(960)
p(1162) = p(4)*p(401) - p(961)
p(1163) = p(1)*p(713) - p(1005) - p(1162)
p(1164) = p(1)*p(714) - p(1006) - p(1163)
p(1165) = p(1)*p(715) - p(1007) - p(1164)
p(1166) = p(13)*p(331) - p(1147)
p(1167) = p(2)*p(695) - p(1002) - p(1166)
p(1168) = p(32)*p(181) - p(1147)
p(1169) = p(1)*p(717) - p(1007) - p(1167)
p(1170) = p(73)*p(91) - p(1147)
p(1171) = p(1)*p(719) - p(1011) - p(1169)
p(1172) = p(41)*p(150) - p(1147)
p(1173) = p(13)*p(279) - p(961)
p(1174) = p(16)*p(285) - p(1147)
p(1175) = p(69)*p(73) - p(960)
p(1176) = p(4)*p(505) - p(1147)
p(1177) = p(11)*p(285) - p(963)
p(1178) = p(14)*p(331)
p(1179) = p(14)*p(332)
p(1180) = p(3)*p(430) - p(1026)
p(1181) = p(7)*p(430) - p(1028)
p(1182) = p(49)*p(130) - p(1031)
p(1183) = p(3)*p(517) - p(1027) - p(1182)
p(1184) = p(1)*p(730) - p(1027) - p(1183)
p(1185) = p(45)*p(178) - p(1180)
p(1186) = p(39)*p(160) - p(1028)
p(1187) = p(3)*p(518) - p(1031)
p(1188) = p(14)*p(284)
p(1189) = p(14)*p(285)
p(1190) = p(14)*p(228)
p(1191) = p(9)*p(335)
p(1192) = p(9)*p(297)
p(1193) = p(9)*p(250)
p(1194) = p(9)*p(298)
p(1195) = p(9)*p(251)
p(1196) = p(9)*p(299)
p(1197) = p(9)*p(300)
p(1198) = p(9)*p(301)
p(1199) = p(9)*p(336)
p(1200) = p(24)*p(183) - p(964)
p(1201) = p(43)*p(114) - p(966)
p(1202) = p(8)*p(401) - p(1021)
p(1203) = p(1)*p(745) - p(1042) - p(1202)
p(1204) = p(1)*p(746) - p(1043) - p(1203)
p(1205) = p(13)*p(333) - p(1178)
p(1206) = p(2)*p(717) - p(1025) - p(1183)
p(1207) = p(32)*p(183) - p(1178)
p(1208) = p(1)*p(748) - p(1043) - p(1206)
p(1209) = p(73)*p(93) - p(1178)
p(1210) = p(6)*p(509) - p(1021)
p(1211) = p(43)*p(150) - p(1178)
p(1212) = p(32)*p(153) - p(966)
p(1213) = p(18)*p(285) - p(1178)
p(1214) = p(34)*p(150) - p(964)
p(1215) = p(14)*p(333)
p(1216) = p(14)*p(334)
p(1217) = p(41)*p(130) - p(973)
p(1218) = p(4)*p(430) - p(970)
p(1219) = p(1)*p(758) - p(1057) - p(1218)
p(1220) = p(1)*p(759) - p(1058) - p(1219)
p(1221) = p(81)*p(91) - p(1189)
p(1222) = p(2)*p(730) - p(1034) - p(1221)
p(1223) = p(41)*p(160) - p(1189)
p(1224) = p(1)*p(761) - p(1058) - p(1222)
p(1225) = p(16)*p(296) - p(1189)
p(1226) = p(69)*p(81) - p(970)
p(1227) = p(4)*p(518) - p(1189)
p(1228) = p(11)*p(296) - p(973)
p(1229) = p(14)*p(292)
p(1230) = p(14)*p(293)
p(1231) = p(14)*p(294)
p(1232) = p(14)*p(242)
p(1233) = p(14)*p(243)
p(1234) = p(3)*p(464) - p(1074)
p(1235) = p(7)*p(464) - p(1075)
p(1236) = p(3)*p(531) - p(1074) - p(1235)
p(1237) = p(39)*p(187) - p(1234)
p(1238) = p(3)*p(532) - p(1075)
p(1239) = p(14)*p(295)
p(1240) = p(14)*p(296)
p(1241) = p(14)*p(247)
p(1242) = p(14)*p(248)
p(1243) = p(14)*p(249)
p(1244) = p(9)*p(337)
p(1245) = p(9)*p(312)
p(1246) = p(9)*p(266)
p(1247) = p(9)*p(313)
p(1248) = p(9)*p(314)
p(1249) = p(9)*p(338)
p(1250) = p(24)*p(185) - p(975)
p(1251) = p(46)*p(114) - p(977)
p(1252) = p(1)*p(783) - p(1082) - p(1251)
p(1253) = p(1)*p(784) - p(1083) - p(1252)
p(1254) = p(13)*p(335) - p(1215)
p(1255) = p(2)*p(748) - p(1055) - p(1222)
p(1256) = p(32)*p(185) - p(1215)
p(1257) = p(1)*p(786) - p(1083) - p(1255)
p(1258) = p(73)*p(95) - p(1215)
p(1259) = p(13)*p(299) - p(977)
p(1260) = p(46)*p(150) - p(1215)
p(1261) = p(73)*p(84) - p(975)
p(1262) = p(14)*p(335)
p(1263) = p(14)*p(336)
p(1264) = p(43)*p(130) - p(974)
p(1265) = p(8)*p(430) - p(1038)
p(1266) = p(1)*p(794) - p(1093) - p(1265)
p(1267) = p(81)*p(93) - p(1190)
p(1268) = p(2)*p(761) - p(1066) - p(1236)
p(1269) = p(43)*p(160) - p(1190)
p(1270) = p(45)*p(153) - p(1038)
p(1271) = p(18)*p(296) - p(1190)
p(1272) = p(34)*p(160) - p(974)
p(1273) = p(14)*p(302)
p(1274) = p(14)*p(303)
p(1275) = p(14)*p(304)
p(1276) = p(14)*p(258)
p(1277) = p(14)*p(259)
p(1278) = p(4)*p(464) - p(983)
p(1279) = p(1)*p(804) - p(1103) - p(1278)
p(1280) = p(16)*p(310) - p(1240)
p(1281) = p(2)*p(772) - p(1077) - p(1280)
p(1282) = p(4)*p(532) - p(1240)
p(1283) = p(11)*p(310) - p(983)
p(1284) = p(14)*p(305)
p(1285) = p(14)*p(306)
p(1286) = p(14)*p(307)
p(1287) = p(14)*p(263)
p(1288) = p(14)*p(264)
p(1289) = p(14)*p(265)
p(1290) = p(6)*p(464) - p(1104)
p(1291) = p(3)*p(578) - p(1290)
p(1292) = p(14)*p(309)
p(1293) = p(14)*p(310)
p(1294) = p(9)*p(340)
p(1295) = p(9)*p(322)
p(1296) = p(9)*p(323)
p(1297) = p(9)*p(341)
p(1298) = p(24)*p(188) - p(984)
p(1299) = p(50)*p(114) - p(1089)
p(1300) = p(1)*p(813) - p(1108) - p(1299)
p(1301) = p(13)*p(337) - p(1262)
p(1302) = p(2)*p(786) - p(1091) - p(1268)
p(1303) = p(32)*p(188) - p(1262)
p(1304) = p(6)*p(536) - p(1089)
p(1305) = p(73)*p(98) - p(1262)
p(1306) = p(32)*p(173) - p(984)
p(1307) = p(14)*p(337)
p(1308) = p(14)*p(338)
p(1309) = p(46)*p(130) - p(982)
p(1310) = p(1)*p(820) - p(1113) - p(1309)
p(1311) = p(81)*p(95) - p(1232)
p(1312) = p(2)*p(796) - p(1098) - p(1281)
p(1313) = p(46)*p(160) - p(1232)
p(1314) = p(81)*p(84) - p(982)
p(1315) = p(14)*p(315)
p(1316) = p(14)*p(316)
p(1317) = p(14)*p(317)
p(1318) = p(14)*p(270)
p(1319) = p(14)*p(271)
p(1320) = p(8)*p(464) - p(1079)
p(1321) = p(18)*p(310) - p(1242)
p(1322) = p(34)*p(187) - p(1079)
p(1323) = p(14)*p(318)
p(1324) = p(14)*p(319)
p(1325) = p(14)*p(320)
p(1326) = p(9)*p(343)
p(1327) = p(9)*p(344)
p(1328) = p(24)*p(191) - p(987)
p(1329) = p(1)*p(827) - p(1117) - p(1328)
p(1330) = p(13)*p(340) - p(1307)
p(1331) = p(2)*p(815) - p(1112) - p(1312)
p(1332) = p(32)*p(191) - p(1307)
p(1333) = p(13)*p(323) - p(987)
p(1334) = p(14)*p(340)
p(1335) = p(14)*p(341)
p(1336) = p(50)*p(130) - p(1102)
p(1337) = p(81)*p(98) - p(1276)
p(1338) = p(45)*p(173) - p(1102)
p(1339) = p(14)*p(324)
p(1340) = p(14)*p(325)
p(1341) = p(14)*p(326)
p(1342) = p(9)*p(347)
p(1343) = p(24)*p(195) - p(1119)
p(1344) = p(13)*p(343) - p(1334)
p(1345) = p(6)*p(561) - p(1119)
p(1346) = p(14)*p(343)
p(1347) = p(14)*p(344)
p(1348) = p(9)*p(272)
p(1349) = p(9)*p(273)
p(1350) = p(9)*p(274)
p(1351) = p(9)*p(328)
p(1352) = p(9)*p(275)
p(1353) = p(9)*p(329)
p(1354) = p(9)*p(330)
p(1355) = p(9)*p(346)
p(1356) = p(3)*p(570) - p(1120)
p(1357) = p(39)*p(181) - p(1122)
p(1358) = p(41)*p(178) - p(1125)
p(1359) = p(4)*p(566) - p(1129)
p(1360) = p(1)*p(842) - p(1126) - p(1359)
p(1361) = p(1)*p(843) - p(1129)
p(1362) = p(1)*p(844) - p(1130) - p(1360)
p(1363) = p(21)*p(279) - p(1125)
p(1364) = p(1)*p(846) - p(1133) - p(1362)
p(1365) = p(69)*p(101) - p(1122)
p(1366) = p(1)*p(848) - p(1135) - p(1364)
p(1367) = p(11)*p(346) - p(1120)
p(1368) = p(1)*p(850) - p(1140) - p(1139)
p(1369) = p(1)*p(851) - p(1146)
p(1370) = p(3)*p(572) - p(1148)
p(1371) = p(39)*p(183) - p(1150)
p(1372) = p(43)*p(178) - p(1153)
p(1373) = p(1)*p(854) - p(1151) - p(1372)
p(1374) = p(2)*p(843) - p(1143)
p(1375) = p(1)*p(855) - p(1154) - p(1373)
p(1376) = p(7)*p(509) - p(1153)
p(1377) = p(1)*p(857) - p(1157) - p(1375)
p(1378) = p(49)*p(153) - p(1150)
p(1379) = p(1)*p(859) - p(1159) - p(1377)
p(1380) = p(34)*p(194) - p(1148)
p(1381) = p(6)*p(570) - p(1147)
p(1382) = p(1)*p(862) - p(1167) - p(1165)
p(1383) = p(2)*p(851) - p(1147)
p(1384) = p(1)*p(864) - p(1183) - p(1182)
p(1385) = p(1)*p(865) - p(1187)
p(1386) = p(3)*p(574) - p(1191)
p(1387) = p(39)*p(185) - p(1193)
p(1388) = p(46)*p(178) - p(1196)
p(1389) = p(1)*p(868) - p(1194) - p(1388)
p(1390) = p(1)*p(869) - p(1196)
p(1391) = p(1)*p(870) - p(1197) - p(1389)
p(1392) = p(21)*p(299) - p(1193)
p(1393) = p(1)*p(872) - p(1199) - p(1391)
p(1394) = p(84)*p(101) - p(1191)
p(1395) = p(6)*p(572) - p(1178)
p(1396) = p(1)*p(875) - p(1206) - p(1204)
p(1397) = p(8)*p(505) - p(1178)
p(1398) = p(45)*p(181) - p(1189)
p(1399) = p(1)*p(878) - p(1222) - p(1220)
p(1400) = p(2)*p(865) - p(1189)
p(1401) = p(1)*p(880) - p(1236) - p(1235)
p(1402) = p(1)*p(881) - p(1238)
p(1403) = p(3)*p(576) - p(1244)
p(1404) = p(39)*p(188) - p(1246)
p(1405) = p(1)*p(884) - p(1245) - p(1404)
p(1406) = p(2)*p(869) - p(1210)
p(1407) = p(1)*p(885) - p(1247) - p(1405)
p(1408) = p(7)*p(536) - p(1246)
p(1409) = p(1)*p(887) - p(1249) - p(1407)
p(1410) = p(49)*p(173) - p(1244)
p(1411) = p(6)*p(574) - p(1215)
p(1412) = p(1)*p(890) - p(1255) - p(1253)
p(1413) = p(22)*p(285) - p(1215)
p(1414) = p(45)*p(183) - p(1190)
p(1415) = p(1)*p(893) - p(1268) - p(1266)
p(1416) = p(8)*p(518) - p(1190)
p(1417) = p(41)*p(187) - p(1240)
p(1418) = p(1)*p(896) - p(1281) - p(1279)
p(1419) = p(2)*p(881) - p(1240)
p(1420) = p(14)*p(308)
p(1421) = p(4)*p(547) - p(1106) - p(1320)
p(1422) = p(1)*p(900) - p(1291)
p(1423) = p(14)*p(311)
p(1424) = p(3)*p(579) - p(1294)
p(1425) = p(39)*p(191) - p(1296)
p(1426) = p(1)*p(903) - p(1295) - p(1425)
p(1427) = p(1)*p(904) - p(1296)
p(1428) = p(1)*p(905) - p(1297) - p(1426)
p(1429) = p(21)*p(323) - p(1294)
p(1430) = p(6)*p(576) - p(1262)
p(1431) = p(1)*p(908) - p(1302) - p(1300)
p(1432) = p(50)*p(150) - p(1262)
p(1433) = p(45)*p(185) - p(1232)
p(1434) = p(1)*p(911) - p(1312) - p(1310)
p(1435) = p(22)*p(296) - p(1232)
p(1436) = p(43)*p(187) - p(1242)
p(1437) = p(1)*p(915) - p(1322) - p(1320)
p(1438) = p(8)*p(532) - p(1242)
p(1439) = p(14)*p(321)
p(1440) = p(4)*p(578) - p(1293)
p(1441) = p(2)*p(899) - p(1292) - p(1440)
p(1442) = p(2)*p(900) - p(1293)
p(1443) = p(14)*p(339)
p(1444) = p(3)*p(582) - p(1326)
p(1445) = p(2)*p(903) - p(1303) - p(1299)
p(1446) = p(2)*p(904) - p(1304)
p(1447) = p(1)*p(919) - p(1327) - p(1445)
p(1448) = p(7)*p(561) - p(1326)
p(1449) = p(6)*p(579) - p(1307)
p(1450) = p(1)*p(922) - p(1331) - p(1329)
p(1451) = p(73)*p(102) - p(1307)
p(1452) = p(45)*p(188) - p(1276)
p(1453) = p(1)*p(925) - p(1338) - p(1336)
p(1454) = p(50)*p(160) - p(1276)
p(1455) = p(14)*p(327)
p(1456) = p(46)*p(187) - p(1243)
p(1457) = p(2)*p(915) - p(1324) - p(1441)
p(1458) = p(22)*p(310) - p(1243)
p(1459) = p(14)*p(342)
p(1460) = p(3)*p(585) - p(1342)
p(1461) = p(2)*p(919) - p(1332) - p(1329)
p(1462) = p(1)*p(929) - p(1342)
p(1463) = p(6)*p(582) - p(1334)
p(1464) = p(1)*p(931) - p(1345) - p(1343)
p(1465) = p(32)*p(195) - p(1334)
p(1466) = p(45)*p(191) - p(1318)
p(1467) = p(2)*p(925) - p(1340) - p(1457)
p(1468) = p(81)*p(102) - p(1318)
p(1469) = p(14)*p(345)
p(1470) = p(2)*p(928) - p(1344) - p(1343)
p(1471) = p(2)*p(929) - p(1345)
p(1472) = p(6)*p(585) - p(1346)
p(1473) = p(2)*p(931) - p(1347) - p(1467)
p(1474) = p(13)*p(347) - p(1346)
p(1475) = p(14)*p(347)
p(1476) = p(3)*p(566) - p(1351)
p(1477) = p(7)*p(566) - p(1349)
p(1478) = p(49)*p(178) - p(1348)
p(1479) = p(39)*p(194) - p(1352)
p(1480) = p(3)*p(588) - p(1355)
p(1481) = p(1)*p(938) - p(1356)
p(1482) = p(2)*p(958) - p(1481)
p(1483) = p(1)*p(940) - p(1370)
p(1484) = p(8)*p(588) - p(1483)
p(1485) = p(1)*p(942) - p(1386)
p(1486) = p(22)*p(346) - p(1485)
p(1487) = p(1)*p(944) - p(1403)
p(1488) = p(50)*p(194) - p(1487)
p(1489) = p(1)*p(946) - p(1424)
p(1490) = p(101)*p(102) - p(1489)
p(1491) = p(6)*p(578) - p(1443)
p(1492) = p(1)*p(949) - p(1444)
p(1493) = p(49)*p(195) - p(1492)
p(1494) = p(8)*p(578) - p(1423)
p(1495) = p(1)*p(952) - p(1460)
p(1496) = p(21)*p(347) - p(1495)
p(1497) = p(50)*p(187) - p(1420)
p(1498) = p(1)*p(955) - p(1470)
p(1499) = p(7)*p(589) - p(1498)
p(1500) = p(45)*p(195) - p(1455)
p(1501) = p(2)*p(955) - p(1472)
p(1502) = p(1)*p(959) - p(1501)
p(1503) = p(6)*p(589) - p(1475)
p(1504) = p(1)*p(958) - p(1480)
p(1505) = p(2)*p(959) - p(1503)


do i=1,6
dm(0) = 0.0D0
dm(1) = dmsdr(i,4)
dm(2) = dmsdr(i,2)
dm(3) = dmsdr(i,1)
dm(4) = dmsdr(i,6)
dm(5) = dmsdr(i,5)
dm(6) = dmsdr(i,3)
dm(7) = dm(1)*rm(2) + rm(1)*dm(2)
dm(8) = dm(1)*rm(3) + rm(1)*dm(3)
dm(9) = dm(2)*rm(3) + rm(2)*dm(3)
dm(10) = dm(3)*rm(4) + rm(3)*dm(4)
dm(11) = dm(2)*rm(5) + rm(2)*dm(5)
dm(12) = dm(1)*rm(6) + rm(1)*dm(6)
dm(13) = dm(4)*rm(5) + rm(4)*dm(5)
dm(14) = dm(4)*rm(6) + rm(4)*dm(6)
dm(15) = dm(5)*rm(6) + rm(5)*dm(6)
dm(16) = dm(1)*rm(9) + rm(1)*dm(9)
dm(17) = dm(1)*rm(10) + rm(1)*dm(10)
dm(18) = dm(2)*rm(10) + rm(2)*dm(10)
dm(19) = dm(1)*rm(11) + rm(1)*dm(11)
dm(20) = dm(3)*rm(11) + rm(3)*dm(11)
dm(21) = dm(2)*rm(12) + rm(2)*dm(12)
dm(22) = dm(3)*rm(12) + rm(3)*dm(12)
dm(23) = dm(2)*rm(13) + rm(2)*dm(13)
dm(24) = dm(3)*rm(13) + rm(3)*dm(13)
dm(25) = dm(1)*rm(14) + rm(1)*dm(14)
dm(26) = dm(3)*rm(14) + rm(3)*dm(14)
dm(27) = dm(1)*rm(15) + rm(1)*dm(15)
dm(28) = dm(2)*rm(15) + rm(2)*dm(15)
dm(29) = dm(4)*rm(15) + rm(4)*dm(15)
dm(30) = dm(2)*rm(24) + rm(2)*dm(24)
dm(31) = dm(1)*rm(26) + rm(1)*dm(26)
dm(32) = dm(1)*rm(28) + rm(1)*dm(28)

dp(0) = dm(0)
dp(1) = dm(1) + dm(2) + dm(3)
dp(2) = dm(4) + dm(5) + dm(6)
dp(3) = dm(7) + dm(8) + dm(9)
dp(4) = dm(10) + dm(11) + dm(12)
dp(5) = dp(1)*p(2) + p(1)*dp(2) - dp(4)
dp(6) = dm(13) + dm(14) + dm(15)
dp(7) = dp(1)*p(1) + p(1)*dp(1) - dp(3) - dp(3)
dp(8) = dp(2)*p(2) + p(2)*dp(2) - dp(6) - dp(6)
dp(9) = dm(16)
dp(10) = dm(17) + dm(18) + dm(19) + dm(20) + dm(21) + dm(22)
dp(11) = dp(2)*p(3) + p(2)*dp(3) - dp(10)
dp(12) = dm(23) + dm(24) + dm(25) + dm(26) + dm(27) + dm(28)
dp(13) = dp(1)*p(6) + p(1)*dp(6) - dp(12)
dp(14) = dm(29)
dp(15) = dp(1)*p(3) + p(1)*dp(3) - dp(9) - dp(9) - dp(9)
dp(16) = dp(1)*p(4) + p(1)*dp(4) - dp(10)
dp(17) = dp(2)*p(7) + p(2)*dp(7) - dp(16)
dp(18) = dp(2)*p(4) + p(2)*dp(4) - dp(12)
dp(19) = dp(1)*p(8) + p(1)*dp(8) - dp(18)
dp(20) = dp(2)*p(6) + p(2)*dp(6) - dp(14) - dp(14) - dp(14)
dp(21) = dp(1)*p(7) + p(1)*dp(7) - dp(15)
dp(22) = dp(2)*p(8) + p(2)*dp(8) - dp(20)
dp(23) = dp(9)*p(2) + p(9)*dp(2)
dp(24) = dm(30) + dm(31) + dm(32)
dp(25) = dp(3)*p(6) + p(3)*dp(6) - dp(24)
dp(26) = dp(14)*p(1) + p(14)*dp(1)
dp(27) = dp(9)*p(1) + p(9)*dp(1)
dp(28) = dp(3)*p(4) + p(3)*dp(4) - dp(23)
dp(29) = dp(1)*p(10) + p(1)*dp(10) - dp(23) - dp(28) - dp(23)
dp(30) = dp(1)*p(11) + p(1)*dp(11) - dp(23)
dp(31) = dp(1)*p(12) + p(1)*dp(12) - dp(25) - dp(24) - dp(24)
dp(32) = dp(1)*p(13) + p(1)*dp(13) - dp(25)
dp(33) = dp(4)*p(5) + p(4)*dp(5) - dp(25) - dp(31)
dp(34) = dp(2)*p(11) + p(2)*dp(11) - dp(25)
dp(35) = dp(4)*p(6) + p(4)*dp(6) - dp(26)
dp(36) = dp(2)*p(12) + p(2)*dp(12) - dp(26) - dp(35) - dp(26)
dp(37) = dp(2)*p(13) + p(2)*dp(13) - dp(26)
dp(38) = dp(14)*p(2) + p(14)*dp(2)
dp(39) = dp(3)*p(3) + p(3)*dp(3) - dp(27) - dp(27)
dp(40) = dp(3)*p(7) + p(3)*dp(7) - dp(27)
dp(41) = dp(1)*p(16) + p(1)*dp(16) - dp(28)
dp(42) = dp(2)*p(21) + p(2)*dp(21) - dp(41)
dp(43) = dp(1)*p(18) + p(1)*dp(18) - dp(33)
dp(44) = dp(7)*p(8) + p(7)*dp(8) - dp(43)
dp(45) = dp(6)*p(6) + p(6)*dp(6) - dp(38) - dp(38)
dp(46) = dp(2)*p(18) + p(2)*dp(18) - dp(35)
dp(47) = dp(1)*p(22) + p(1)*dp(22) - dp(46)
dp(48) = dp(6)*p(8) + p(6)*dp(8) - dp(38)
dp(49) = dp(1)*p(21) + p(1)*dp(21) - dp(40)
dp(50) = dp(2)*p(22) + p(2)*dp(22) - dp(48)
dp(51) = dp(9)*p(6) + p(9)*dp(6)
dp(52) = dp(14)*p(3) + p(14)*dp(3)
dp(53) = dp(9)*p(4) + p(9)*dp(4)
dp(54) = dp(9)*p(5) + p(9)*dp(5)
dp(55) = dp(1)*p(24) + p(1)*dp(24) - dp(51)
dp(56) = dp(3)*p(12) + p(3)*dp(12) - dp(51) - dp(55) - dp(51)
dp(57) = dp(3)*p(13) + p(3)*dp(13) - dp(51)
dp(58) = dp(14)*p(7) + p(14)*dp(7)
dp(59) = dp(9)*p(8) + p(9)*dp(8)
dp(60) = dp(2)*p(24) + p(2)*dp(24) - dp(52)
dp(61) = dp(4)*p(13) + p(4)*dp(13) - dp(58)
dp(62) = dp(6)*p(11) + p(6)*dp(11) - dp(52)
dp(63) = dp(14)*p(4) + p(14)*dp(4)
dp(64) = dp(14)*p(5) + p(14)*dp(5)
dp(65) = dp(9)*p(3) + p(9)*dp(3)
dp(66) = dp(9)*p(7) + p(9)*dp(7)
dp(67) = dp(3)*p(16) + p(3)*dp(16) - dp(53)
dp(68) = dp(4)*p(15) + p(4)*dp(15) - dp(54) - dp(67)
dp(69) = dp(2)*p(39) + p(2)*dp(39) - dp(68)
dp(70) = dp(1)*p(29) + p(1)*dp(29) - dp(54) - dp(68)
dp(71) = dp(7)*p(11) + p(7)*dp(11) - dp(53)
dp(72) = dp(1)*p(31) + p(1)*dp(31) - dp(56) - dp(55)
dp(73) = dp(1)*p(32) + p(1)*dp(32) - dp(57)
dp(74) = dp(3)*p(18) + p(3)*dp(18) - dp(59)
dp(75) = dp(2)*p(29) + p(2)*dp(29) - dp(57) - dp(55)
dp(76) = dp(1)*p(34) + p(1)*dp(34) - dp(59)
dp(77) = dp(6)*p(16) + p(6)*dp(16) - dp(58)
dp(78) = dp(1)*p(36) + p(1)*dp(36) - dp(62) - dp(60)
dp(79) = dp(2)*p(32) + p(2)*dp(32) - dp(58)
dp(80) = dp(6)*p(12) + p(6)*dp(12) - dp(64) - dp(63) - dp(63)
dp(81) = dp(1)*p(45) + p(1)*dp(45) - dp(80)
dp(82) = dp(14)*p(6) + p(14)*dp(6)
dp(83) = dp(2)*p(33) + p(2)*dp(33) - dp(61) - dp(60)
dp(84) = dp(2)*p(34) + p(2)*dp(34) - dp(62)
dp(85) = dp(6)*p(18) + p(6)*dp(18) - dp(63)
dp(86) = dp(2)*p(36) + p(2)*dp(36) - dp(64) - dp(80)
dp(87) = dp(8)*p(13) + p(8)*dp(13) - dp(63)
dp(88) = dp(14)*p(8) + p(14)*dp(8)
dp(89) = dp(1)*p(39) + p(1)*dp(39) - dp(65)
dp(90) = dp(3)*p(21) + p(3)*dp(21) - dp(66)
dp(91) = dp(1)*p(41) + p(1)*dp(41) - dp(67)
dp(92) = dp(2)*p(49) + p(2)*dp(49) - dp(91)
dp(93) = dp(1)*p(43) + p(1)*dp(43) - dp(74)
dp(94) = dp(8)*p(21) + p(8)*dp(21) - dp(93)
dp(95) = dp(1)*p(46) + p(1)*dp(46) - dp(83)
dp(96) = dp(7)*p(22) + p(7)*dp(22) - dp(95)
dp(97) = dp(2)*p(45) + p(2)*dp(45) - dp(82)
dp(98) = dp(2)*p(46) + p(2)*dp(46) - dp(85)
dp(99) = dp(1)*p(50) + p(1)*dp(50) - dp(98)
dp(100) = dp(6)*p(22) + p(6)*dp(22) - dp(88)
dp(101) = dp(1)*p(49) + p(1)*dp(49) - dp(90)
dp(102) = dp(2)*p(50) + p(2)*dp(50) - dp(100)
dp(103) = dp(9)*p(14) + p(9)*dp(14)
dp(104) = dp(9)*p(12) + p(9)*dp(12)
dp(105) = dp(9)*p(13) + p(9)*dp(13)
dp(106) = dp(14)*p(15) + p(14)*dp(15)
dp(107) = dp(9)*p(20) + p(9)*dp(20)
dp(108) = dp(14)*p(10) + p(14)*dp(10)
dp(109) = dp(14)*p(11) + p(14)*dp(11)
dp(110) = dp(9)*p(16) + p(9)*dp(16)
dp(111) = dp(9)*p(10) + p(9)*dp(10)
dp(112) = dp(9)*p(11) + p(9)*dp(11)
dp(113) = dp(9)*p(17) + p(9)*dp(17)
dp(114) = dp(3)*p(24) + p(3)*dp(24) - dp(104)
dp(115) = dp(7)*p(24) + p(7)*dp(24) - dp(105)
dp(116) = dp(3)*p(31) + p(3)*dp(31) - dp(104) - dp(115)
dp(117) = dp(6)*p(39) + p(6)*dp(39) - dp(114)
dp(118) = dp(3)*p(32) + p(3)*dp(32) - dp(105)
dp(119) = dp(14)*p(21) + p(14)*dp(21)
dp(120) = dp(9)*p(18) + p(9)*dp(18)
dp(121) = dp(9)*p(19) + p(9)*dp(19)
dp(122) = dp(4)*p(24) + p(4)*dp(24) - dp(103) - dp(103) - dp(103)
dp(123) = dp(1)*p(60) + p(1)*dp(60) - dp(107) - dp(122)
dp(124) = dp(13)*p(16) + p(13)*dp(16) - dp(119)
dp(125) = dp(2)*p(56) + p(2)*dp(56) - dp(106) - dp(124)
dp(126) = dp(4)*p(32) + p(4)*dp(32) - dp(119)
dp(127) = dp(1)*p(62) + p(1)*dp(62) - dp(107) - dp(125)
dp(128) = dp(14)*p(16) + p(14)*dp(16)
dp(129) = dp(14)*p(17) + p(14)*dp(17)
dp(130) = dp(6)*p(24) + p(6)*dp(24) - dp(108)
dp(131) = dp(3)*p(45) + p(3)*dp(45) - dp(130)
dp(132) = dp(14)*p(12) + p(14)*dp(12)
dp(133) = dp(14)*p(13) + p(14)*dp(13)
dp(134) = dp(9)*p(22) + p(9)*dp(22)
dp(135) = dp(8)*p(24) + p(8)*dp(24) - dp(109)
dp(136) = dp(13)*p(18) + p(13)*dp(18) - dp(128)
dp(137) = dp(6)*p(34) + p(6)*dp(34) - dp(109)
dp(138) = dp(14)*p(18) + p(14)*dp(18)
dp(139) = dp(14)*p(19) + p(14)*dp(19)
dp(140) = dp(9)*p(9) + p(9)*dp(9)
dp(141) = dp(9)*p(15) + p(9)*dp(15)
dp(142) = dp(9)*p(21) + p(9)*dp(21)
dp(143) = dp(3)*p(41) + p(3)*dp(41) - dp(110)
dp(144) = dp(4)*p(39) + p(4)*dp(39) - dp(112)
dp(145) = dp(1)*p(68) + p(1)*dp(68) - dp(111) - dp(144)
dp(146) = dp(1)*p(69) + p(1)*dp(69) - dp(112)
dp(147) = dp(1)*p(70) + p(1)*dp(70) - dp(113) - dp(145)
dp(148) = dp(11)*p(21) + p(11)*dp(21) - dp(110)
dp(149) = dp(1)*p(72) + p(1)*dp(72) - dp(116) - dp(115)
dp(150) = dp(1)*p(73) + p(1)*dp(73) - dp(118)
dp(151) = dp(3)*p(43) + p(3)*dp(43) - dp(120)
dp(152) = dp(15)*p(18) + p(15)*dp(18) - dp(121) - dp(151)
dp(153) = dp(2)*p(69) + p(2)*dp(69) - dp(117)
dp(154) = dp(1)*p(75) + p(1)*dp(75) - dp(121) - dp(152)
dp(155) = dp(7)*p(34) + p(7)*dp(34) - dp(120)
dp(156) = dp(6)*p(41) + p(6)*dp(41) - dp(119)
dp(157) = dp(1)*p(78) + p(1)*dp(78) - dp(125) - dp(123)
dp(158) = dp(2)*p(73) + p(2)*dp(73) - dp(119)
dp(159) = dp(1)*p(80) + p(1)*dp(80) - dp(131) - dp(130) - dp(130)
dp(160) = dp(1)*p(81) + p(1)*dp(81) - dp(131)
dp(161) = dp(14)*p(14) + p(14)*dp(14)
dp(162) = dp(3)*p(46) + p(3)*dp(46) - dp(134)
dp(163) = dp(2)*p(75) + p(2)*dp(75) - dp(126) - dp(123)
dp(164) = dp(1)*p(84) + p(1)*dp(84) - dp(134)
dp(165) = dp(6)*p(43) + p(6)*dp(43) - dp(128)
dp(166) = dp(1)*p(86) + p(1)*dp(86) - dp(137) - dp(135)
dp(167) = dp(8)*p(32) + p(8)*dp(32) - dp(128)
dp(168) = dp(4)*p(45) + p(4)*dp(45) - dp(133)
dp(169) = dp(2)*p(80) + p(2)*dp(80) - dp(132) - dp(168)
dp(170) = dp(2)*p(81) + p(2)*dp(81) - dp(133)
dp(171) = dp(14)*p(20) + p(14)*dp(20)
dp(172) = dp(2)*p(83) + p(2)*dp(83) - dp(136) - dp(135)
dp(173) = dp(2)*p(84) + p(2)*dp(84) - dp(137)
dp(174) = dp(6)*p(46) + p(6)*dp(46) - dp(138)
dp(175) = dp(2)*p(86) + p(2)*dp(86) - dp(139) - dp(169)
dp(176) = dp(13)*p(22) + p(13)*dp(22) - dp(138)
dp(177) = dp(14)*p(22) + p(14)*dp(22)
dp(178) = dp(3)*p(39) + p(3)*dp(39) - dp(141)
dp(179) = dp(1)*p(89) + p(1)*dp(89) - dp(141) - dp(178) - dp(178)
dp(180) = dp(3)*p(49) + p(3)*dp(49) - dp(142)
dp(181) = dp(1)*p(91) + p(1)*dp(91) - dp(143)
dp(182) = dp(2)*p(101) + p(2)*dp(101) - dp(181)
dp(183) = dp(1)*p(93) + p(1)*dp(93) - dp(151)
dp(184) = dp(8)*p(49) + p(8)*dp(49) - dp(183)
dp(185) = dp(1)*p(95) + p(1)*dp(95) - dp(162)
dp(186) = dp(21)*p(22) + p(21)*dp(22) - dp(185)
dp(187) = dp(6)*p(45) + p(6)*dp(45) - dp(171)
dp(188) = dp(1)*p(98) + p(1)*dp(98) - dp(172)
dp(189) = dp(7)*p(50) + p(7)*dp(50) - dp(188)
dp(190) = dp(2)*p(97) + p(2)*dp(97) - dp(171) - dp(187) - dp(187)
dp(191) = dp(2)*p(98) + p(2)*dp(98) - dp(174)
dp(192) = dp(1)*p(102) + p(1)*dp(102) - dp(191)
dp(193) = dp(6)*p(50) + p(6)*dp(50) - dp(177)
dp(194) = dp(1)*p(101) + p(1)*dp(101) - dp(180)
dp(195) = dp(2)*p(102) + p(2)*dp(102) - dp(193)
dp(196) = dp(9)*p(26) + p(9)*dp(26)
dp(197) = dp(9)*p(38) + p(9)*dp(38)
dp(198) = dp(9)*p(24) + p(9)*dp(24)
dp(199) = dp(9)*p(31) + p(9)*dp(31)
dp(200) = dp(9)*p(25) + p(9)*dp(25)
dp(201) = dp(9)*p(32) + p(9)*dp(32)
dp(202) = dp(14)*p(39) + p(14)*dp(39)
dp(203) = dp(14)*p(40) + p(14)*dp(40)
dp(204) = dp(9)*p(35) + p(9)*dp(35)
dp(205) = dp(9)*p(36) + p(9)*dp(36)
dp(206) = dp(9)*p(37) + p(9)*dp(37)
dp(207) = dp(14)*p(28) + p(14)*dp(28)
dp(208) = dp(14)*p(29) + p(14)*dp(29)
dp(209) = dp(14)*p(30) + p(14)*dp(30)
dp(210) = dp(9)*p(45) + p(9)*dp(45)
dp(211) = dp(14)*p(24) + p(14)*dp(24)
dp(212) = dp(14)*p(25) + p(14)*dp(25)
dp(213) = dp(9)*p(48) + p(9)*dp(48)
dp(214) = dp(14)*p(33) + p(14)*dp(33)
dp(215) = dp(14)*p(34) + p(14)*dp(34)
dp(216) = dp(9)*p(41) + p(9)*dp(41)
dp(217) = dp(9)*p(28) + p(9)*dp(28)
dp(218) = dp(9)*p(23) + p(9)*dp(23)
dp(219) = dp(9)*p(29) + p(9)*dp(29)
dp(220) = dp(9)*p(30) + p(9)*dp(30)
dp(221) = dp(9)*p(42) + p(9)*dp(42)
dp(222) = dp(1)*p(114) + p(1)*dp(114) - dp(198)
dp(223) = dp(21)*p(24) + p(21)*dp(24) - dp(201)
dp(224) = dp(3)*p(72) + p(3)*dp(72) - dp(199) - dp(223)
dp(225) = dp(1)*p(116) + p(1)*dp(116) - dp(199) - dp(224)
dp(226) = dp(13)*p(39) + p(13)*dp(39) - dp(198)
dp(227) = dp(3)*p(73) + p(3)*dp(73) - dp(201)
dp(228) = dp(14)*p(49) + p(14)*dp(49)
dp(229) = dp(9)*p(43) + p(9)*dp(43)
dp(230) = dp(9)*p(33) + p(9)*dp(33)
dp(231) = dp(9)*p(34) + p(9)*dp(34)
dp(232) = dp(9)*p(44) + p(9)*dp(44)
dp(233) = dp(16)*p(24) + p(16)*dp(24) - dp(196)
dp(234) = dp(2)*p(114) + p(2)*dp(114) - dp(202)
dp(235) = dp(1)*p(123) + p(1)*dp(123) - dp(205) - dp(234)
dp(236) = dp(13)*p(41) + p(13)*dp(41) - dp(228)
dp(237) = dp(2)*p(116) + p(2)*dp(116) - dp(203) - dp(236)
dp(238) = dp(16)*p(32) + p(16)*dp(32) - dp(228)
dp(239) = dp(6)*p(69) + p(6)*dp(69) - dp(202)
dp(240) = dp(4)*p(73) + p(4)*dp(73) - dp(228)
dp(241) = dp(11)*p(32) + p(11)*dp(32) - dp(196)
dp(242) = dp(14)*p(41) + p(14)*dp(41)
dp(243) = dp(14)*p(42) + p(14)*dp(42)
dp(244) = dp(1)*p(130) + p(1)*dp(130) - dp(210)
dp(245) = dp(4)*p(62) + p(4)*dp(62) - dp(213) - dp(209)
dp(246) = dp(3)*p(81) + p(3)*dp(81) - dp(210)
dp(247) = dp(14)*p(31) + p(14)*dp(31)
dp(248) = dp(14)*p(32) + p(14)*dp(32)
dp(249) = dp(14)*p(26) + p(14)*dp(26)
dp(250) = dp(9)*p(46) + p(9)*dp(46)
dp(251) = dp(9)*p(47) + p(9)*dp(47)
dp(252) = dp(18)*p(24) + p(18)*dp(24) - dp(197)
dp(253) = dp(1)*p(135) + p(1)*dp(135) - dp(213) - dp(252)
dp(254) = dp(13)*p(43) + p(13)*dp(43) - dp(242)
dp(255) = dp(2)*p(125) + p(2)*dp(125) - dp(209) - dp(245)
dp(256) = dp(18)*p(32) + p(18)*dp(32) - dp(242)
dp(257) = dp(13)*p(34) + p(13)*dp(34) - dp(197)
dp(258) = dp(14)*p(43) + p(14)*dp(43)
dp(259) = dp(14)*p(44) + p(14)*dp(44)
dp(260) = dp(2)*p(130) + p(2)*dp(130) - dp(211)
dp(261) = dp(4)*p(81) + p(4)*dp(81) - dp(248)
dp(262) = dp(11)*p(45) + p(11)*dp(45) - dp(211)
dp(263) = dp(14)*p(35) + p(14)*dp(35)
dp(264) = dp(14)*p(36) + p(14)*dp(36)
dp(265) = dp(14)*p(37) + p(14)*dp(37)
dp(266) = dp(9)*p(50) + p(9)*dp(50)
dp(267) = dp(22)*p(24) + p(22)*dp(24) - dp(215)
dp(268) = dp(13)*p(46) + p(13)*dp(46) - dp(258)
dp(269) = dp(6)*p(84) + p(6)*dp(84) - dp(215)
dp(270) = dp(14)*p(46) + p(14)*dp(46)
dp(271) = dp(14)*p(47) + p(14)*dp(47)
dp(272) = dp(9)*p(27) + p(9)*dp(27)
dp(273) = dp(9)*p(39) + p(9)*dp(39)
dp(274) = dp(9)*p(40) + p(9)*dp(40)
dp(275) = dp(9)*p(49) + p(9)*dp(49)
dp(276) = dp(3)*p(91) + p(3)*dp(91) - dp(216)
dp(277) = dp(16)*p(39) + p(16)*dp(39) - dp(218)
dp(278) = dp(1)*p(144) + p(1)*dp(144) - dp(217) - dp(277)
dp(279) = dp(2)*p(178) + p(2)*dp(178) - dp(278)
dp(280) = dp(1)*p(145) + p(1)*dp(145) - dp(219) - dp(278)
dp(281) = dp(7)*p(69) + p(7)*dp(69) - dp(218)
dp(282) = dp(1)*p(147) + p(1)*dp(147) - dp(221) - dp(280)
dp(283) = dp(11)*p(49) + p(11)*dp(49) - dp(216)
dp(284) = dp(1)*p(149) + p(1)*dp(149) - dp(224) - dp(223)
dp(285) = dp(1)*p(150) + p(1)*dp(150) - dp(227)
dp(286) = dp(3)*p(93) + p(3)*dp(93) - dp(229)
dp(287) = dp(18)*p(39) + p(18)*dp(39) - dp(231)
dp(288) = dp(1)*p(152) + p(1)*dp(152) - dp(230) - dp(287)
dp(289) = dp(1)*p(153) + p(1)*dp(153) - dp(231)
dp(290) = dp(1)*p(154) + p(1)*dp(154) - dp(232) - dp(288)
dp(291) = dp(21)*p(34) + p(21)*dp(34) - dp(229)
dp(292) = dp(6)*p(91) + p(6)*dp(91) - dp(228)
dp(293) = dp(1)*p(157) + p(1)*dp(157) - dp(237) - dp(235)
dp(294) = dp(2)*p(150) + p(2)*dp(150) - dp(228)
dp(295) = dp(1)*p(159) + p(1)*dp(159) - dp(245) - dp(244)
dp(296) = dp(1)*p(160) + p(1)*dp(160) - dp(246)
dp(297) = dp(3)*p(95) + p(3)*dp(95) - dp(250)
dp(298) = dp(2)*p(152) + p(2)*dp(152) - dp(238) - dp(234)
dp(299) = dp(2)*p(153) + p(2)*dp(153) - dp(239)
dp(300) = dp(1)*p(163) + p(1)*dp(163) - dp(251) - dp(298)
dp(301) = dp(7)*p(84) + p(7)*dp(84) - dp(250)
dp(302) = dp(6)*p(93) + p(6)*dp(93) - dp(242)
dp(303) = dp(1)*p(166) + p(1)*dp(166) - dp(255) - dp(253)
dp(304) = dp(8)*p(73) + p(8)*dp(73) - dp(242)
dp(305) = dp(16)*p(45) + p(16)*dp(45) - dp(248)
dp(306) = dp(1)*p(169) + p(1)*dp(169) - dp(262) - dp(260)
dp(307) = dp(2)*p(160) + p(2)*dp(160) - dp(248)
dp(308) = dp(14)*p(38) + p(14)*dp(38)
dp(309) = dp(6)*p(80) + p(6)*dp(80) - dp(264) - dp(263)
dp(310) = dp(1)*p(187) + p(1)*dp(187) - dp(309)
dp(311) = dp(14)*p(45) + p(14)*dp(45)
dp(312) = dp(3)*p(98) + p(3)*dp(98) - dp(266)
dp(313) = dp(2)*p(163) + p(2)*dp(163) - dp(256) - dp(253)
dp(314) = dp(1)*p(173) + p(1)*dp(173) - dp(266)
dp(315) = dp(6)*p(95) + p(6)*dp(95) - dp(258)
dp(316) = dp(1)*p(175) + p(1)*dp(175) - dp(269) - dp(267)
dp(317) = dp(22)*p(32) + p(22)*dp(32) - dp(258)
dp(318) = dp(18)*p(45) + p(18)*dp(45) - dp(249)
dp(319) = dp(2)*p(169) + p(2)*dp(169) - dp(264) - dp(309)
dp(320) = dp(8)*p(81) + p(8)*dp(81) - dp(249)
dp(321) = dp(14)*p(48) + p(14)*dp(48)
dp(322) = dp(2)*p(172) + p(2)*dp(172) - dp(268) - dp(267)
dp(323) = dp(2)*p(173) + p(2)*dp(173) - dp(269)
dp(324) = dp(6)*p(98) + p(6)*dp(98) - dp(270)
dp(325) = dp(2)*p(175) + p(2)*dp(175) - dp(271) - dp(319)
dp(326) = dp(13)*p(50) + p(13)*dp(50) - dp(270)
dp(327) = dp(14)*p(50) + p(14)*dp(50)
dp(328) = dp(1)*p(178) + p(1)*dp(178) - dp(273)
dp(329) = dp(21)*p(39) + p(21)*dp(39) - dp(272)
dp(330) = dp(3)*p(101) + p(3)*dp(101) - dp(275)
dp(331) = dp(1)*p(181) + p(1)*dp(181) - dp(276)
dp(332) = dp(2)*p(194) + p(2)*dp(194) - dp(331)
dp(333) = dp(1)*p(183) + p(1)*dp(183) - dp(286)
dp(334) = dp(8)*p(101) + p(8)*dp(101) - dp(333)
dp(335) = dp(1)*p(185) + p(1)*dp(185) - dp(297)
dp(336) = dp(22)*p(49) + p(22)*dp(49) - dp(335)
dp(337) = dp(1)*p(188) + p(1)*dp(188) - dp(312)
dp(338) = dp(21)*p(50) + p(21)*dp(50) - dp(337)
dp(339) = dp(2)*p(187) + p(2)*dp(187) - dp(311)
dp(340) = dp(1)*p(191) + p(1)*dp(191) - dp(322)
dp(341) = dp(7)*p(102) + p(7)*dp(102) - dp(340)
dp(342) = dp(22)*p(45) + p(22)*dp(45) - dp(308)
dp(343) = dp(2)*p(191) + p(2)*dp(191) - dp(324)
dp(344) = dp(1)*p(195) + p(1)*dp(195) - dp(343)
dp(345) = dp(6)*p(102) + p(6)*dp(102) - dp(327)
dp(346) = dp(1)*p(194) + p(1)*dp(194) - dp(330)
dp(347) = dp(2)*p(195) + p(2)*dp(195) - dp(345)
dp(348) = dp(9)*p(52) + p(9)*dp(52)
dp(349) = dp(9)*p(58) + p(9)*dp(58)
dp(350) = dp(9)*p(63) + p(9)*dp(63)
dp(351) = dp(9)*p(64) + p(9)*dp(64)
dp(352) = dp(9)*p(82) + p(9)*dp(82)
dp(353) = dp(9)*p(88) + p(9)*dp(88)
dp(354) = dp(9)*p(55) + p(9)*dp(55)
dp(355) = dp(9)*p(72) + p(9)*dp(72)
dp(356) = dp(9)*p(51) + p(9)*dp(51)
dp(357) = dp(9)*p(56) + p(9)*dp(56)
dp(358) = dp(9)*p(57) + p(9)*dp(57)
dp(359) = dp(9)*p(73) + p(9)*dp(73)
dp(360) = dp(14)*p(89) + p(14)*dp(89)
dp(361) = dp(14)*p(90) + p(14)*dp(90)
dp(362) = dp(9)*p(77) + p(9)*dp(77)
dp(363) = dp(9)*p(60) + p(9)*dp(60)
dp(364) = dp(9)*p(78) + p(9)*dp(78)
dp(365) = dp(9)*p(61) + p(9)*dp(61)
dp(366) = dp(9)*p(62) + p(9)*dp(62)
dp(367) = dp(9)*p(79) + p(9)*dp(79)
dp(368) = dp(14)*p(67) + p(14)*dp(67)
dp(369) = dp(14)*p(68) + p(14)*dp(68)
dp(370) = dp(14)*p(69) + p(14)*dp(69)
dp(371) = dp(14)*p(70) + p(14)*dp(70)
dp(372) = dp(14)*p(71) + p(14)*dp(71)
dp(373) = dp(9)*p(80) + p(9)*dp(80)
dp(374) = dp(9)*p(81) + p(9)*dp(81)
dp(375) = dp(14)*p(55) + p(14)*dp(55)
dp(376) = dp(14)*p(56) + p(14)*dp(56)
dp(377) = dp(14)*p(57) + p(14)*dp(57)
dp(378) = dp(14)*p(52) + p(14)*dp(52)
dp(379) = dp(9)*p(85) + p(9)*dp(85)
dp(380) = dp(9)*p(86) + p(9)*dp(86)
dp(381) = dp(9)*p(87) + p(9)*dp(87)
dp(382) = dp(14)*p(74) + p(14)*dp(74)
dp(383) = dp(14)*p(75) + p(14)*dp(75)
dp(384) = dp(14)*p(76) + p(14)*dp(76)
dp(385) = dp(9)*p(97) + p(9)*dp(97)
dp(386) = dp(14)*p(60) + p(14)*dp(60)
dp(387) = dp(14)*p(61) + p(14)*dp(61)
dp(388) = dp(14)*p(62) + p(14)*dp(62)
dp(389) = dp(9)*p(100) + p(9)*dp(100)
dp(390) = dp(14)*p(83) + p(14)*dp(83)
dp(391) = dp(14)*p(84) + p(14)*dp(84)
dp(392) = dp(9)*p(91) + p(9)*dp(91)
dp(393) = dp(9)*p(67) + p(9)*dp(67)
dp(394) = dp(9)*p(53) + p(9)*dp(53)
dp(395) = dp(9)*p(68) + p(9)*dp(68)
dp(396) = dp(9)*p(54) + p(9)*dp(54)
dp(397) = dp(9)*p(69) + p(9)*dp(69)
dp(398) = dp(9)*p(70) + p(9)*dp(70)
dp(399) = dp(9)*p(71) + p(9)*dp(71)
dp(400) = dp(9)*p(92) + p(9)*dp(92)
dp(401) = dp(3)*p(114) + p(3)*dp(114) - dp(354)
dp(402) = dp(7)*p(114) + p(7)*dp(114) - dp(356)
dp(403) = dp(24)*p(49) + p(24)*dp(49) - dp(359)
dp(404) = dp(3)*p(149) + p(3)*dp(149) - dp(355) - dp(403)
dp(405) = dp(1)*p(224) + p(1)*dp(224) - dp(355) - dp(404)
dp(406) = dp(6)*p(178) + p(6)*dp(178) - dp(401)
dp(407) = dp(32)*p(39) + p(32)*dp(39) - dp(356)
dp(408) = dp(3)*p(150) + p(3)*dp(150) - dp(359)
dp(409) = dp(14)*p(101) + p(14)*dp(101)
dp(410) = dp(9)*p(93) + p(9)*dp(93)
dp(411) = dp(9)*p(74) + p(9)*dp(74)
dp(412) = dp(9)*p(59) + p(9)*dp(59)
dp(413) = dp(9)*p(75) + p(9)*dp(75)
dp(414) = dp(9)*p(76) + p(9)*dp(76)
dp(415) = dp(9)*p(94) + p(9)*dp(94)
dp(416) = dp(24)*p(41) + p(24)*dp(41) - dp(349)
dp(417) = dp(4)*p(114) + p(4)*dp(114) - dp(348)
dp(418) = dp(1)*p(234) + p(1)*dp(234) - dp(363) - dp(417)
dp(419) = dp(1)*p(235) + p(1)*dp(235) - dp(364) - dp(418)
dp(420) = dp(13)*p(91) + p(13)*dp(91) - dp(409)
dp(421) = dp(2)*p(224) + p(2)*dp(224) - dp(361) - dp(420)
dp(422) = dp(32)*p(41) + p(32)*dp(41) - dp(409)
dp(423) = dp(1)*p(237) + p(1)*dp(237) - dp(364) - dp(421)
dp(424) = dp(16)*p(73) + p(16)*dp(73) - dp(409)
dp(425) = dp(13)*p(69) + p(13)*dp(69) - dp(348)
dp(426) = dp(4)*p(150) + p(4)*dp(150) - dp(409)
dp(427) = dp(11)*p(73) + p(11)*dp(73) - dp(349)
dp(428) = dp(14)*p(91) + p(14)*dp(91)
dp(429) = dp(14)*p(92) + p(14)*dp(92)
dp(430) = dp(3)*p(130) + p(3)*dp(130) - dp(373)
dp(431) = dp(7)*p(130) + p(7)*dp(130) - dp(374)
dp(432) = dp(3)*p(159) + p(3)*dp(159) - dp(373) - dp(431)
dp(433) = dp(39)*p(45) + p(39)*dp(45) - dp(430)
dp(434) = dp(3)*p(160) + p(3)*dp(160) - dp(374)
dp(435) = dp(14)*p(72) + p(14)*dp(72)
dp(436) = dp(14)*p(73) + p(14)*dp(73)
dp(437) = dp(14)*p(58) + p(14)*dp(58)
dp(438) = dp(9)*p(95) + p(9)*dp(95)
dp(439) = dp(9)*p(83) + p(9)*dp(83)
dp(440) = dp(9)*p(84) + p(9)*dp(84)
dp(441) = dp(9)*p(96) + p(9)*dp(96)
dp(442) = dp(24)*p(43) + p(24)*dp(43) - dp(350)
dp(443) = dp(8)*p(114) + p(8)*dp(114) - dp(370)
dp(444) = dp(1)*p(253) + p(1)*dp(253) - dp(380) - dp(443)
dp(445) = dp(13)*p(93) + p(13)*dp(93) - dp(428)
dp(446) = dp(2)*p(237) + p(2)*dp(237) - dp(372) - dp(432)
dp(447) = dp(32)*p(43) + p(32)*dp(43) - dp(428)
dp(448) = dp(6)*p(153) + p(6)*dp(153) - dp(370)
dp(449) = dp(18)*p(73) + p(18)*dp(73) - dp(428)
dp(450) = dp(32)*p(34) + p(32)*dp(34) - dp(350)
dp(451) = dp(14)*p(93) + p(14)*dp(93)
dp(452) = dp(14)*p(94) + p(14)*dp(94)
dp(453) = dp(4)*p(130) + p(4)*dp(130) - dp(352)
dp(454) = dp(1)*p(260) + p(1)*dp(260) - dp(385) - dp(453)
dp(455) = dp(16)*p(81) + p(16)*dp(81) - dp(436)
dp(456) = dp(2)*p(245) + p(2)*dp(245) - dp(376) - dp(455)
dp(457) = dp(4)*p(160) + p(4)*dp(160) - dp(436)
dp(458) = dp(11)*p(81) + p(11)*dp(81) - dp(352)
dp(459) = dp(14)*p(77) + p(14)*dp(77)
dp(460) = dp(14)*p(78) + p(14)*dp(78)
dp(461) = dp(14)*p(79) + p(14)*dp(79)
dp(462) = dp(14)*p(63) + p(14)*dp(63)
dp(463) = dp(14)*p(64) + p(14)*dp(64)
dp(464) = dp(6)*p(130) + p(6)*dp(130) - dp(386)
dp(465) = dp(3)*p(187) + p(3)*dp(187) - dp(464)
dp(466) = dp(14)*p(80) + p(14)*dp(80)
dp(467) = dp(14)*p(81) + p(14)*dp(81)
dp(468) = dp(9)*p(98) + p(9)*dp(98)
dp(469) = dp(9)*p(99) + p(9)*dp(99)
dp(470) = dp(24)*p(46) + p(24)*dp(46) - dp(353)
dp(471) = dp(1)*p(267) + p(1)*dp(267) - dp(389) - dp(470)
dp(472) = dp(13)*p(95) + p(13)*dp(95) - dp(451)
dp(473) = dp(2)*p(255) + p(2)*dp(255) - dp(384) - dp(456)
dp(474) = dp(32)*p(46) + p(32)*dp(46) - dp(451)
dp(475) = dp(13)*p(84) + p(13)*dp(84) - dp(353)
dp(476) = dp(14)*p(95) + p(14)*dp(95)
dp(477) = dp(14)*p(96) + p(14)*dp(96)
dp(478) = dp(8)*p(130) + p(8)*dp(130) - dp(378)
dp(479) = dp(18)*p(81) + p(18)*dp(81) - dp(437)
dp(480) = dp(34)*p(45) + p(34)*dp(45) - dp(378)
dp(481) = dp(14)*p(85) + p(14)*dp(85)
dp(482) = dp(14)*p(86) + p(14)*dp(86)
dp(483) = dp(14)*p(87) + p(14)*dp(87)
dp(484) = dp(9)*p(102) + p(9)*dp(102)
dp(485) = dp(24)*p(50) + p(24)*dp(50) - dp(391)
dp(486) = dp(13)*p(98) + p(13)*dp(98) - dp(476)
dp(487) = dp(6)*p(173) + p(6)*dp(173) - dp(391)
dp(488) = dp(14)*p(98) + p(14)*dp(98)
dp(489) = dp(14)*p(99) + p(14)*dp(99)
dp(490) = dp(9)*p(65) + p(9)*dp(65)
dp(491) = dp(9)*p(66) + p(9)*dp(66)
dp(492) = dp(9)*p(89) + p(9)*dp(89)
dp(493) = dp(9)*p(90) + p(9)*dp(90)
dp(494) = dp(9)*p(101) + p(9)*dp(101)
dp(495) = dp(3)*p(181) + p(3)*dp(181) - dp(392)
dp(496) = dp(39)*p(41) + p(39)*dp(41) - dp(394)
dp(497) = dp(4)*p(178) + p(4)*dp(178) - dp(397)
dp(498) = dp(1)*p(278) + p(1)*dp(278) - dp(395) - dp(497)
dp(499) = dp(1)*p(279) + p(1)*dp(279) - dp(397)
dp(500) = dp(1)*p(280) + p(1)*dp(280) - dp(398) - dp(498)
dp(501) = dp(21)*p(69) + p(21)*dp(69) - dp(394)
dp(502) = dp(1)*p(282) + p(1)*dp(282) - dp(400) - dp(500)
dp(503) = dp(11)*p(101) + p(11)*dp(101) - dp(392)
dp(504) = dp(1)*p(284) + p(1)*dp(284) - dp(404) - dp(403)
dp(505) = dp(1)*p(285) + p(1)*dp(285) - dp(408)
dp(506) = dp(3)*p(183) + p(3)*dp(183) - dp(410)
dp(507) = dp(39)*p(43) + p(39)*dp(43) - dp(412)
dp(508) = dp(1)*p(287) + p(1)*dp(287) - dp(411) - dp(507)
dp(509) = dp(2)*p(279) + p(2)*dp(279) - dp(406)
dp(510) = dp(1)*p(288) + p(1)*dp(288) - dp(413) - dp(508)
dp(511) = dp(7)*p(153) + p(7)*dp(153) - dp(412)
dp(512) = dp(1)*p(290) + p(1)*dp(290) - dp(415) - dp(510)
dp(513) = dp(34)*p(49) + p(34)*dp(49) - dp(410)
dp(514) = dp(6)*p(181) + p(6)*dp(181) - dp(409)
dp(515) = dp(1)*p(293) + p(1)*dp(293) - dp(421) - dp(419)
dp(516) = dp(2)*p(285) + p(2)*dp(285) - dp(409)
dp(517) = dp(1)*p(295) + p(1)*dp(295) - dp(432) - dp(431)
dp(518) = dp(1)*p(296) + p(1)*dp(296) - dp(434)
dp(519) = dp(3)*p(185) + p(3)*dp(185) - dp(438)
dp(520) = dp(39)*p(46) + p(39)*dp(46) - dp(440)
dp(521) = dp(1)*p(298) + p(1)*dp(298) - dp(439) - dp(520)
dp(522) = dp(1)*p(299) + p(1)*dp(299) - dp(440)
dp(523) = dp(1)*p(300) + p(1)*dp(300) - dp(441) - dp(521)
dp(524) = dp(21)*p(84) + p(21)*dp(84) - dp(438)
dp(525) = dp(6)*p(183) + p(6)*dp(183) - dp(428)
dp(526) = dp(1)*p(303) + p(1)*dp(303) - dp(446) - dp(444)
dp(527) = dp(8)*p(150) + p(8)*dp(150) - dp(428)
dp(528) = dp(41)*p(45) + p(41)*dp(45) - dp(436)
dp(529) = dp(1)*p(306) + p(1)*dp(306) - dp(456) - dp(454)
dp(530) = dp(2)*p(296) + p(2)*dp(296) - dp(436)
dp(531) = dp(4)*p(169) + p(4)*dp(169) - dp(388) - dp(478)
dp(532) = dp(1)*p(310) + p(1)*dp(310) - dp(465)
dp(533) = dp(14)*p(82) + p(14)*dp(82)
dp(534) = dp(3)*p(188) + p(3)*dp(188) - dp(468)
dp(535) = dp(2)*p(298) + p(2)*dp(298) - dp(447) - dp(443)
dp(536) = dp(2)*p(299) + p(2)*dp(299) - dp(448)
dp(537) = dp(1)*p(313) + p(1)*dp(313) - dp(469) - dp(535)
dp(538) = dp(7)*p(173) + p(7)*dp(173) - dp(468)
dp(539) = dp(6)*p(185) + p(6)*dp(185) - dp(451)
dp(540) = dp(1)*p(316) + p(1)*dp(316) - dp(473) - dp(471)
dp(541) = dp(22)*p(73) + p(22)*dp(73) - dp(451)
dp(542) = dp(43)*p(45) + p(43)*dp(45) - dp(437)
dp(543) = dp(1)*p(319) + p(1)*dp(319) - dp(480) - dp(478)
dp(544) = dp(8)*p(160) + p(8)*dp(160) - dp(437)
dp(545) = dp(14)*p(88) + p(14)*dp(88)
dp(546) = dp(4)*p(187) + p(4)*dp(187) - dp(467)
dp(547) = dp(2)*p(309) + p(2)*dp(309) - dp(466) - dp(546)
dp(548) = dp(2)*p(310) + p(2)*dp(310) - dp(467)
dp(549) = dp(14)*p(97) + p(14)*dp(97)
dp(550) = dp(3)*p(191) + p(3)*dp(191) - dp(484)
dp(551) = dp(2)*p(313) + p(2)*dp(313) - dp(474) - dp(471)
dp(552) = dp(1)*p(323) + p(1)*dp(323) - dp(484)
dp(553) = dp(6)*p(188) + p(6)*dp(188) - dp(476)
dp(554) = dp(1)*p(325) + p(1)*dp(325) - dp(487) - dp(485)
dp(555) = dp(32)*p(50) + p(32)*dp(50) - dp(476)
dp(556) = dp(45)*p(46) + p(45)*dp(46) - dp(462)
dp(557) = dp(2)*p(319) + p(2)*dp(319) - dp(482) - dp(547)
dp(558) = dp(22)*p(81) + p(22)*dp(81) - dp(462)
dp(559) = dp(14)*p(100) + p(14)*dp(100)
dp(560) = dp(2)*p(322) + p(2)*dp(322) - dp(486) - dp(485)
dp(561) = dp(2)*p(323) + p(2)*dp(323) - dp(487)
dp(562) = dp(6)*p(191) + p(6)*dp(191) - dp(488)
dp(563) = dp(2)*p(325) + p(2)*dp(325) - dp(489) - dp(557)
dp(564) = dp(13)*p(102) + p(13)*dp(102) - dp(488)
dp(565) = dp(14)*p(102) + p(14)*dp(102)
dp(566) = dp(3)*p(178) + p(3)*dp(178) - dp(492)
dp(567) = dp(7)*p(178) + p(7)*dp(178) - dp(490)
dp(568) = dp(39)*p(49) + p(39)*dp(49) - dp(491)
dp(569) = dp(3)*p(194) + p(3)*dp(194) - dp(494)
dp(570) = dp(1)*p(331) + p(1)*dp(331) - dp(495)
dp(571) = dp(2)*p(346) + p(2)*dp(346) - dp(570)
dp(572) = dp(1)*p(333) + p(1)*dp(333) - dp(506)
dp(573) = dp(8)*p(194) + p(8)*dp(194) - dp(572)
dp(574) = dp(1)*p(335) + p(1)*dp(335) - dp(519)
dp(575) = dp(22)*p(101) + p(22)*dp(101) - dp(574)
dp(576) = dp(1)*p(337) + p(1)*dp(337) - dp(534)
dp(577) = dp(49)*p(50) + p(49)*dp(50) - dp(576)
dp(578) = dp(6)*p(187) + p(6)*dp(187) - dp(549)
dp(579) = dp(1)*p(340) + p(1)*dp(340) - dp(550)
dp(580) = dp(21)*p(102) + p(21)*dp(102) - dp(579)
dp(581) = dp(8)*p(187) + p(8)*dp(187) - dp(533)
dp(582) = dp(1)*p(343) + p(1)*dp(343) - dp(560)
dp(583) = dp(7)*p(195) + p(7)*dp(195) - dp(582)
dp(584) = dp(45)*p(50) + p(45)*dp(50) - dp(545)
dp(585) = dp(2)*p(343) + p(2)*dp(343) - dp(562)
dp(586) = dp(1)*p(347) + p(1)*dp(347) - dp(585)
dp(587) = dp(6)*p(195) + p(6)*dp(195) - dp(565)
dp(588) = dp(1)*p(346) + p(1)*dp(346) - dp(569)
dp(589) = dp(2)*p(347) + p(2)*dp(347) - dp(587)
dp(590) = dp(9)*p(103) + p(9)*dp(103)
dp(591) = dp(9)*p(106) + p(9)*dp(106)
dp(592) = dp(9)*p(119) + p(9)*dp(119)
dp(593) = dp(9)*p(128) + p(9)*dp(128)
dp(594) = dp(9)*p(108) + p(9)*dp(108)
dp(595) = dp(9)*p(109) + p(9)*dp(109)
dp(596) = dp(9)*p(129) + p(9)*dp(129)
dp(597) = dp(9)*p(132) + p(9)*dp(132)
dp(598) = dp(9)*p(133) + p(9)*dp(133)
dp(599) = dp(9)*p(161) + p(9)*dp(161)
dp(600) = dp(9)*p(138) + p(9)*dp(138)
dp(601) = dp(9)*p(139) + p(9)*dp(139)
dp(602) = dp(9)*p(171) + p(9)*dp(171)
dp(603) = dp(9)*p(177) + p(9)*dp(177)
dp(604) = dp(9)*p(114) + p(9)*dp(114)
dp(605) = dp(9)*p(115) + p(9)*dp(115)
dp(606) = dp(9)*p(149) + p(9)*dp(149)
dp(607) = dp(9)*p(104) + p(9)*dp(104)
dp(608) = dp(9)*p(116) + p(9)*dp(116)
dp(609) = dp(9)*p(105) + p(9)*dp(105)
dp(610) = dp(9)*p(117) + p(9)*dp(117)
dp(611) = dp(9)*p(118) + p(9)*dp(118)
dp(612) = dp(9)*p(150) + p(9)*dp(150)
dp(613) = dp(14)*p(178) + p(14)*dp(178)
dp(614) = dp(14)*p(179) + p(14)*dp(179)
dp(615) = dp(14)*p(180) + p(14)*dp(180)
dp(616) = dp(9)*p(156) + p(9)*dp(156)
dp(617) = dp(9)*p(122) + p(9)*dp(122)
dp(618) = dp(9)*p(123) + p(9)*dp(123)
dp(619) = dp(9)*p(157) + p(9)*dp(157)
dp(620) = dp(9)*p(124) + p(9)*dp(124)
dp(621) = dp(9)*p(107) + p(9)*dp(107)
dp(622) = dp(9)*p(125) + p(9)*dp(125)
dp(623) = dp(9)*p(126) + p(9)*dp(126)
dp(624) = dp(9)*p(127) + p(9)*dp(127)
dp(625) = dp(9)*p(158) + p(9)*dp(158)
dp(626) = dp(14)*p(143) + p(14)*dp(143)
dp(627) = dp(14)*p(144) + p(14)*dp(144)
dp(628) = dp(14)*p(145) + p(14)*dp(145)
dp(629) = dp(14)*p(146) + p(14)*dp(146)
dp(630) = dp(14)*p(147) + p(14)*dp(147)
dp(631) = dp(14)*p(148) + p(14)*dp(148)
dp(632) = dp(9)*p(130) + p(9)*dp(130)
dp(633) = dp(9)*p(159) + p(9)*dp(159)
dp(634) = dp(9)*p(131) + p(9)*dp(131)
dp(635) = dp(9)*p(160) + p(9)*dp(160)
dp(636) = dp(14)*p(114) + p(14)*dp(114)
dp(637) = dp(14)*p(115) + p(14)*dp(115)
dp(638) = dp(14)*p(116) + p(14)*dp(116)
dp(639) = dp(14)*p(117) + p(14)*dp(117)
dp(640) = dp(14)*p(118) + p(14)*dp(118)
dp(641) = dp(14)*p(106) + p(14)*dp(106)
dp(642) = dp(9)*p(165) + p(9)*dp(165)
dp(643) = dp(9)*p(135) + p(9)*dp(135)
dp(644) = dp(9)*p(166) + p(9)*dp(166)
dp(645) = dp(9)*p(136) + p(9)*dp(136)
dp(646) = dp(9)*p(137) + p(9)*dp(137)
dp(647) = dp(9)*p(167) + p(9)*dp(167)
dp(648) = dp(14)*p(151) + p(14)*dp(151)
dp(649) = dp(14)*p(152) + p(14)*dp(152)
dp(650) = dp(14)*p(153) + p(14)*dp(153)
dp(651) = dp(14)*p(154) + p(14)*dp(154)
dp(652) = dp(14)*p(155) + p(14)*dp(155)
dp(653) = dp(9)*p(168) + p(9)*dp(168)
dp(654) = dp(9)*p(169) + p(9)*dp(169)
dp(655) = dp(9)*p(170) + p(9)*dp(170)
dp(656) = dp(14)*p(122) + p(14)*dp(122)
dp(657) = dp(14)*p(123) + p(14)*dp(123)
dp(658) = dp(14)*p(124) + p(14)*dp(124)
dp(659) = dp(14)*p(125) + p(14)*dp(125)
dp(660) = dp(14)*p(126) + p(14)*dp(126)
dp(661) = dp(14)*p(127) + p(14)*dp(127)
dp(662) = dp(14)*p(108) + p(14)*dp(108)
dp(663) = dp(14)*p(109) + p(14)*dp(109)
dp(664) = dp(9)*p(187) + p(9)*dp(187)
dp(665) = dp(14)*p(130) + p(14)*dp(130)
dp(666) = dp(14)*p(131) + p(14)*dp(131)
dp(667) = dp(9)*p(174) + p(9)*dp(174)
dp(668) = dp(9)*p(175) + p(9)*dp(175)
dp(669) = dp(9)*p(176) + p(9)*dp(176)
dp(670) = dp(14)*p(162) + p(14)*dp(162)
dp(671) = dp(14)*p(163) + p(14)*dp(163)
dp(672) = dp(14)*p(164) + p(14)*dp(164)
dp(673) = dp(9)*p(190) + p(9)*dp(190)
dp(674) = dp(14)*p(135) + p(14)*dp(135)
dp(675) = dp(14)*p(136) + p(14)*dp(136)
dp(676) = dp(14)*p(137) + p(14)*dp(137)
dp(677) = dp(9)*p(193) + p(9)*dp(193)
dp(678) = dp(14)*p(172) + p(14)*dp(172)
dp(679) = dp(14)*p(173) + p(14)*dp(173)
dp(680) = dp(9)*p(181) + p(9)*dp(181)
dp(681) = dp(9)*p(143) + p(9)*dp(143)
dp(682) = dp(9)*p(110) + p(9)*dp(110)
dp(683) = dp(9)*p(144) + p(9)*dp(144)
dp(684) = dp(9)*p(111) + p(9)*dp(111)
dp(685) = dp(9)*p(112) + p(9)*dp(112)
dp(686) = dp(9)*p(145) + p(9)*dp(145)
dp(687) = dp(9)*p(113) + p(9)*dp(113)
dp(688) = dp(9)*p(146) + p(9)*dp(146)
dp(689) = dp(9)*p(147) + p(9)*dp(147)
dp(690) = dp(9)*p(148) + p(9)*dp(148)
dp(691) = dp(9)*p(182) + p(9)*dp(182)
dp(692) = dp(1)*p(401) + p(1)*dp(401) - dp(604)
dp(693) = dp(21)*p(114) + p(21)*dp(114) - dp(609)
dp(694) = dp(24)*p(101) + p(24)*dp(101) - dp(612)
dp(695) = dp(3)*p(284) + p(3)*dp(284) - dp(606) - dp(694)
dp(696) = dp(1)*p(404) + p(1)*dp(404) - dp(606) - dp(695)
dp(697) = dp(1)*p(405) + p(1)*dp(405) - dp(608) - dp(696)
dp(698) = dp(13)*p(178) + p(13)*dp(178) - dp(604)
dp(699) = dp(39)*p(73) + p(39)*dp(73) - dp(609)
dp(700) = dp(3)*p(285) + p(3)*dp(285) - dp(612)
dp(701) = dp(14)*p(194) + p(14)*dp(194)
dp(702) = dp(9)*p(183) + p(9)*dp(183)
dp(703) = dp(9)*p(151) + p(9)*dp(151)
dp(704) = dp(9)*p(120) + p(9)*dp(120)
dp(705) = dp(9)*p(152) + p(9)*dp(152)
dp(706) = dp(9)*p(121) + p(9)*dp(121)
dp(707) = dp(9)*p(153) + p(9)*dp(153)
dp(708) = dp(9)*p(154) + p(9)*dp(154)
dp(709) = dp(9)*p(155) + p(9)*dp(155)
dp(710) = dp(9)*p(184) + p(9)*dp(184)
dp(711) = dp(24)*p(91) + p(24)*dp(91) - dp(592)
dp(712) = dp(1)*p(416) + p(1)*dp(416) - dp(616) - dp(711)
dp(713) = dp(2)*p(401) + p(2)*dp(401) - dp(613)
dp(714) = dp(1)*p(418) + p(1)*dp(418) - dp(618) - dp(713)
dp(715) = dp(1)*p(419) + p(1)*dp(419) - dp(619) - dp(714)
dp(716) = dp(13)*p(181) + p(13)*dp(181) - dp(701)
dp(717) = dp(2)*p(404) + p(2)*dp(404) - dp(615) - dp(716)
dp(718) = dp(32)*p(91) + p(32)*dp(91) - dp(701)
dp(719) = dp(1)*p(421) + p(1)*dp(421) - dp(619) - dp(717)
dp(720) = dp(41)*p(73) + p(41)*dp(73) - dp(701)
dp(721) = dp(6)*p(279) + p(6)*dp(279) - dp(613)
dp(722) = dp(16)*p(150) + p(16)*dp(150) - dp(701)
dp(723) = dp(1)*p(425) + p(1)*dp(425) - dp(624) - dp(721)
dp(724) = dp(4)*p(285) + p(4)*dp(285) - dp(701)
dp(725) = dp(11)*p(150) + p(11)*dp(150) - dp(592)
dp(726) = dp(14)*p(181) + p(14)*dp(181)
dp(727) = dp(14)*p(182) + p(14)*dp(182)
dp(728) = dp(1)*p(430) + p(1)*dp(430) - dp(632)
dp(729) = dp(21)*p(130) + p(21)*dp(130) - dp(635)
dp(730) = dp(3)*p(295) + p(3)*dp(295) - dp(633) - dp(729)
dp(731) = dp(1)*p(432) + p(1)*dp(432) - dp(633) - dp(730)
dp(732) = dp(39)*p(81) + p(39)*dp(81) - dp(632)
dp(733) = dp(3)*p(296) + p(3)*dp(296) - dp(635)
dp(734) = dp(14)*p(149) + p(14)*dp(149)
dp(735) = dp(14)*p(150) + p(14)*dp(150)
dp(736) = dp(14)*p(119) + p(14)*dp(119)
dp(737) = dp(9)*p(185) + p(9)*dp(185)
dp(738) = dp(9)*p(162) + p(9)*dp(162)
dp(739) = dp(9)*p(134) + p(9)*dp(134)
dp(740) = dp(9)*p(163) + p(9)*dp(163)
dp(741) = dp(9)*p(164) + p(9)*dp(164)
dp(742) = dp(9)*p(186) + p(9)*dp(186)
dp(743) = dp(24)*p(93) + p(24)*dp(93) - dp(593)
dp(744) = dp(18)*p(114) + p(18)*dp(114) - dp(595)
dp(745) = dp(1)*p(443) + p(1)*dp(443) - dp(643) - dp(744)
dp(746) = dp(1)*p(444) + p(1)*dp(444) - dp(644) - dp(745)
dp(747) = dp(13)*p(183) + p(13)*dp(183) - dp(726)
dp(748) = dp(2)*p(421) + p(2)*dp(421) - dp(631) - dp(730)
dp(749) = dp(32)*p(93) + p(32)*dp(93) - dp(726)
dp(750) = dp(1)*p(446) + p(1)*dp(446) - dp(644) - dp(748)
dp(751) = dp(43)*p(73) + p(43)*dp(73) - dp(726)
dp(752) = dp(13)*p(153) + p(13)*dp(153) - dp(595)
dp(753) = dp(18)*p(150) + p(18)*dp(150) - dp(726)
dp(754) = dp(34)*p(73) + p(34)*dp(73) - dp(593)
dp(755) = dp(14)*p(183) + p(14)*dp(183)
dp(756) = dp(14)*p(184) + p(14)*dp(184)
dp(757) = dp(16)*p(130) + p(16)*dp(130) - dp(598)
dp(758) = dp(2)*p(430) + p(2)*dp(430) - dp(636)
dp(759) = dp(1)*p(454) + p(1)*dp(454) - dp(654) - dp(758)
dp(760) = dp(41)*p(81) + p(41)*dp(81) - dp(735)
dp(761) = dp(2)*p(432) + p(2)*dp(432) - dp(638) - dp(760)
dp(762) = dp(16)*p(160) + p(16)*dp(160) - dp(735)
dp(763) = dp(45)*p(69) + p(45)*dp(69) - dp(636)
dp(764) = dp(4)*p(296) + p(4)*dp(296) - dp(735)
dp(765) = dp(11)*p(160) + p(11)*dp(160) - dp(598)
dp(766) = dp(14)*p(156) + p(14)*dp(156)
dp(767) = dp(14)*p(157) + p(14)*dp(157)
dp(768) = dp(14)*p(158) + p(14)*dp(158)
dp(769) = dp(14)*p(128) + p(14)*dp(128)
dp(770) = dp(14)*p(129) + p(14)*dp(129)
dp(771) = dp(1)*p(464) + p(1)*dp(464) - dp(664)
dp(772) = dp(4)*p(262) + p(4)*dp(262) - dp(673) - dp(661)
dp(773) = dp(3)*p(310) + p(3)*dp(310) - dp(664)
dp(774) = dp(14)*p(159) + p(14)*dp(159)
dp(775) = dp(14)*p(160) + p(14)*dp(160)
dp(776) = dp(14)*p(132) + p(14)*dp(132)
dp(777) = dp(14)*p(133) + p(14)*dp(133)
dp(778) = dp(9)*p(188) + p(9)*dp(188)
dp(779) = dp(9)*p(172) + p(9)*dp(172)
dp(780) = dp(9)*p(173) + p(9)*dp(173)
dp(781) = dp(9)*p(189) + p(9)*dp(189)
dp(782) = dp(24)*p(95) + p(24)*dp(95) - dp(600)
dp(783) = dp(22)*p(114) + p(22)*dp(114) - dp(650)
dp(784) = dp(1)*p(471) + p(1)*dp(471) - dp(668) - dp(783)
dp(785) = dp(13)*p(185) + p(13)*dp(185) - dp(755)
dp(786) = dp(2)*p(446) + p(2)*dp(446) - dp(652) - dp(761)
dp(787) = dp(32)*p(95) + p(32)*dp(95) - dp(755)
dp(788) = dp(6)*p(299) + p(6)*dp(299) - dp(650)
dp(789) = dp(46)*p(73) + p(46)*dp(73) - dp(755)
dp(790) = dp(32)*p(84) + p(32)*dp(84) - dp(600)
dp(791) = dp(14)*p(185) + p(14)*dp(185)
dp(792) = dp(14)*p(186) + p(14)*dp(186)
dp(793) = dp(2)*p(453) + p(2)*dp(453) - dp(656) - dp(771)
dp(794) = dp(1)*p(478) + p(1)*dp(478) - dp(673) - dp(793)
dp(795) = dp(43)*p(81) + p(43)*dp(81) - dp(736)
dp(796) = dp(2)*p(456) + p(2)*dp(456) - dp(659) - dp(772)
dp(797) = dp(18)*p(160) + p(18)*dp(160) - dp(736)
dp(798) = dp(1)*p(480) + p(1)*dp(480) - dp(673) - dp(796)
dp(799) = dp(14)*p(165) + p(14)*dp(165)
dp(800) = dp(14)*p(166) + p(14)*dp(166)
dp(801) = dp(14)*p(167) + p(14)*dp(167)
dp(802) = dp(14)*p(138) + p(14)*dp(138)
dp(803) = dp(14)*p(139) + p(14)*dp(139)
dp(804) = dp(2)*p(464) + p(2)*dp(464) - dp(665)
dp(805) = dp(4)*p(310) + p(4)*dp(310) - dp(775)
dp(806) = dp(11)*p(187) + p(11)*dp(187) - dp(665)
dp(807) = dp(14)*p(168) + p(14)*dp(168)
dp(808) = dp(14)*p(169) + p(14)*dp(169)
dp(809) = dp(14)*p(170) + p(14)*dp(170)
dp(810) = dp(9)*p(191) + p(9)*dp(191)
dp(811) = dp(9)*p(192) + p(9)*dp(192)
dp(812) = dp(24)*p(98) + p(24)*dp(98) - dp(603)
dp(813) = dp(1)*p(485) + p(1)*dp(485) - dp(677) - dp(812)
dp(814) = dp(13)*p(188) + p(13)*dp(188) - dp(791)
dp(815) = dp(2)*p(473) + p(2)*dp(473) - dp(672) - dp(796)
dp(816) = dp(32)*p(98) + p(32)*dp(98) - dp(791)
dp(817) = dp(13)*p(173) + p(13)*dp(173) - dp(603)
dp(818) = dp(14)*p(188) + p(14)*dp(188)
dp(819) = dp(14)*p(189) + p(14)*dp(189)
dp(820) = dp(22)*p(130) + p(22)*dp(130) - dp(663)
dp(821) = dp(46)*p(81) + p(46)*dp(81) - dp(769)
dp(822) = dp(45)*p(84) + p(45)*dp(84) - dp(663)
dp(823) = dp(14)*p(174) + p(14)*dp(174)
dp(824) = dp(14)*p(175) + p(14)*dp(175)
dp(825) = dp(14)*p(176) + p(14)*dp(176)
dp(826) = dp(9)*p(195) + p(9)*dp(195)
dp(827) = dp(24)*p(102) + p(24)*dp(102) - dp(679)
dp(828) = dp(13)*p(191) + p(13)*dp(191) - dp(818)
dp(829) = dp(6)*p(323) + p(6)*dp(323) - dp(679)
dp(830) = dp(14)*p(191) + p(14)*dp(191)
dp(831) = dp(14)*p(192) + p(14)*dp(192)
dp(832) = dp(9)*p(140) + p(9)*dp(140)
dp(833) = dp(9)*p(141) + p(9)*dp(141)
dp(834) = dp(9)*p(178) + p(9)*dp(178)
dp(835) = dp(9)*p(142) + p(9)*dp(142)
dp(836) = dp(9)*p(179) + p(9)*dp(179)
dp(837) = dp(9)*p(180) + p(9)*dp(180)
dp(838) = dp(9)*p(194) + p(9)*dp(194)
dp(839) = dp(3)*p(331) + p(3)*dp(331) - dp(680)
dp(840) = dp(39)*p(91) + p(39)*dp(91) - dp(682)
dp(841) = dp(16)*p(178) + p(16)*dp(178) - dp(685)
dp(842) = dp(1)*p(497) + p(1)*dp(497) - dp(683) - dp(841)
dp(843) = dp(2)*p(566) + p(2)*dp(566) - dp(842)
dp(844) = dp(1)*p(498) + p(1)*dp(498) - dp(686) - dp(842)
dp(845) = dp(7)*p(279) + p(7)*dp(279) - dp(685)
dp(846) = dp(1)*p(500) + p(1)*dp(500) - dp(689) - dp(844)
dp(847) = dp(49)*p(69) + p(49)*dp(69) - dp(682)
dp(848) = dp(1)*p(502) + p(1)*dp(502) - dp(691) - dp(846)
dp(849) = dp(11)*p(194) + p(11)*dp(194) - dp(680)
dp(850) = dp(1)*p(504) + p(1)*dp(504) - dp(695) - dp(694)
dp(851) = dp(1)*p(505) + p(1)*dp(505) - dp(700)
dp(852) = dp(3)*p(333) + p(3)*dp(333) - dp(702)
dp(853) = dp(39)*p(93) + p(39)*dp(93) - dp(704)
dp(854) = dp(18)*p(178) + p(18)*dp(178) - dp(707)
dp(855) = dp(1)*p(508) + p(1)*dp(508) - dp(705) - dp(854)
dp(856) = dp(1)*p(509) + p(1)*dp(509) - dp(707)
dp(857) = dp(1)*p(510) + p(1)*dp(510) - dp(708) - dp(855)
dp(858) = dp(21)*p(153) + p(21)*dp(153) - dp(704)
dp(859) = dp(1)*p(512) + p(1)*dp(512) - dp(710) - dp(857)
dp(860) = dp(34)*p(101) + p(34)*dp(101) - dp(702)
dp(861) = dp(6)*p(331) + p(6)*dp(331) - dp(701)
dp(862) = dp(1)*p(515) + p(1)*dp(515) - dp(717) - dp(715)
dp(863) = dp(2)*p(505) + p(2)*dp(505) - dp(701)
dp(864) = dp(1)*p(517) + p(1)*dp(517) - dp(730) - dp(729)
dp(865) = dp(1)*p(518) + p(1)*dp(518) - dp(733)
dp(866) = dp(3)*p(335) + p(3)*dp(335) - dp(737)
dp(867) = dp(39)*p(95) + p(39)*dp(95) - dp(739)
dp(868) = dp(1)*p(520) + p(1)*dp(520) - dp(738) - dp(867)
dp(869) = dp(2)*p(509) + p(2)*dp(509) - dp(721)
dp(870) = dp(1)*p(521) + p(1)*dp(521) - dp(740) - dp(868)
dp(871) = dp(7)*p(299) + p(7)*dp(299) - dp(739)
dp(872) = dp(1)*p(523) + p(1)*dp(523) - dp(742) - dp(870)
dp(873) = dp(49)*p(84) + p(49)*dp(84) - dp(737)
dp(874) = dp(6)*p(333) + p(6)*dp(333) - dp(726)
dp(875) = dp(1)*p(526) + p(1)*dp(526) - dp(748) - dp(746)
dp(876) = dp(8)*p(285) + p(8)*dp(285) - dp(726)
dp(877) = dp(45)*p(91) + p(45)*dp(91) - dp(735)
dp(878) = dp(1)*p(529) + p(1)*dp(529) - dp(761) - dp(759)
dp(879) = dp(2)*p(518) + p(2)*dp(518) - dp(735)
dp(880) = dp(1)*p(531) + p(1)*dp(531) - dp(772) - dp(771)
dp(881) = dp(1)*p(532) + p(1)*dp(532) - dp(773)
dp(882) = dp(14)*p(161) + p(14)*dp(161)
dp(883) = dp(3)*p(337) + p(3)*dp(337) - dp(778)
dp(884) = dp(39)*p(98) + p(39)*dp(98) - dp(780)
dp(885) = dp(1)*p(535) + p(1)*dp(535) - dp(779) - dp(884)
dp(886) = dp(1)*p(536) + p(1)*dp(536) - dp(780)
dp(887) = dp(1)*p(537) + p(1)*dp(537) - dp(781) - dp(885)
dp(888) = dp(21)*p(173) + p(21)*dp(173) - dp(778)
dp(889) = dp(6)*p(335) + p(6)*dp(335) - dp(755)
dp(890) = dp(1)*p(540) + p(1)*dp(540) - dp(786) - dp(784)
dp(891) = dp(22)*p(150) + p(22)*dp(150) - dp(755)
dp(892) = dp(45)*p(93) + p(45)*dp(93) - dp(736)
dp(893) = dp(1)*p(543) + p(1)*dp(543) - dp(796) - dp(794)
dp(894) = dp(8)*p(296) + p(8)*dp(296) - dp(736)
dp(895) = dp(16)*p(187) + p(16)*dp(187) - dp(775)
dp(896) = dp(1)*p(547) + p(1)*dp(547) - dp(806) - dp(804)
dp(897) = dp(2)*p(532) + p(2)*dp(532) - dp(775)
dp(898) = dp(14)*p(171) + p(14)*dp(171)
dp(899) = dp(6)*p(309) + p(6)*dp(309) - dp(808) - dp(807)
dp(900) = dp(1)*p(578) + p(1)*dp(578) - dp(899)
dp(901) = dp(14)*p(187) + p(14)*dp(187)
dp(902) = dp(3)*p(340) + p(3)*dp(340) - dp(810)
dp(903) = dp(2)*p(535) + p(2)*dp(535) - dp(787) - dp(783)
dp(904) = dp(2)*p(536) + p(2)*dp(536) - dp(788)
dp(905) = dp(1)*p(551) + p(1)*dp(551) - dp(811) - dp(903)
dp(906) = dp(7)*p(323) + p(7)*dp(323) - dp(810)
dp(907) = dp(6)*p(337) + p(6)*dp(337) - dp(791)
dp(908) = dp(1)*p(554) + p(1)*dp(554) - dp(815) - dp(813)
dp(909) = dp(50)*p(73) + p(50)*dp(73) - dp(791)
dp(910) = dp(45)*p(95) + p(45)*dp(95) - dp(769)
dp(911) = dp(1)*p(557) + p(1)*dp(557) - dp(822) - dp(820)
dp(912) = dp(22)*p(160) + p(22)*dp(160) - dp(769)
dp(913) = dp(14)*p(177) + p(14)*dp(177)
dp(914) = dp(18)*p(187) + p(18)*dp(187) - dp(777)
dp(915) = dp(2)*p(547) + p(2)*dp(547) - dp(808) - dp(899)
dp(916) = dp(8)*p(310) + p(8)*dp(310) - dp(777)
dp(917) = dp(14)*p(190) + p(14)*dp(190)
dp(918) = dp(3)*p(343) + p(3)*dp(343) - dp(826)
dp(919) = dp(2)*p(551) + p(2)*dp(551) - dp(816) - dp(813)
dp(920) = dp(1)*p(561) + p(1)*dp(561) - dp(826)
dp(921) = dp(6)*p(340) + p(6)*dp(340) - dp(818)
dp(922) = dp(1)*p(563) + p(1)*dp(563) - dp(829) - dp(827)
dp(923) = dp(32)*p(102) + p(32)*dp(102) - dp(818)
dp(924) = dp(45)*p(98) + p(45)*dp(98) - dp(802)
dp(925) = dp(2)*p(557) + p(2)*dp(557) - dp(824) - dp(915)
dp(926) = dp(50)*p(81) + p(50)*dp(81) - dp(802)
dp(927) = dp(14)*p(193) + p(14)*dp(193)
dp(928) = dp(2)*p(560) + p(2)*dp(560) - dp(828) - dp(827)
dp(929) = dp(2)*p(561) + p(2)*dp(561) - dp(829)
dp(930) = dp(6)*p(343) + p(6)*dp(343) - dp(830)
dp(931) = dp(2)*p(563) + p(2)*dp(563) - dp(831) - dp(925)
dp(932) = dp(13)*p(195) + p(13)*dp(195) - dp(830)
dp(933) = dp(14)*p(195) + p(14)*dp(195)
dp(934) = dp(1)*p(566) + p(1)*dp(566) - dp(834)
dp(935) = dp(1)*p(567) + p(1)*dp(567) - dp(836) - dp(934)
dp(936) = dp(39)*p(101) + p(39)*dp(101) - dp(835)
dp(937) = dp(3)*p(346) + p(3)*dp(346) - dp(838)
dp(938) = dp(1)*p(570) + p(1)*dp(570) - dp(839)
dp(939) = dp(2)*p(588) + p(2)*dp(588) - dp(938)
dp(940) = dp(1)*p(572) + p(1)*dp(572) - dp(852)
dp(941) = dp(8)*p(346) + p(8)*dp(346) - dp(940)
dp(942) = dp(1)*p(574) + p(1)*dp(574) - dp(866)
dp(943) = dp(22)*p(194) + p(22)*dp(194) - dp(942)
dp(944) = dp(1)*p(576) + p(1)*dp(576) - dp(883)
dp(945) = dp(50)*p(101) + p(50)*dp(101) - dp(944)
dp(946) = dp(1)*p(579) + p(1)*dp(579) - dp(902)
dp(947) = dp(49)*p(102) + p(49)*dp(102) - dp(946)
dp(948) = dp(2)*p(578) + p(2)*dp(578) - dp(901)
dp(949) = dp(1)*p(582) + p(1)*dp(582) - dp(918)
dp(950) = dp(21)*p(195) + p(21)*dp(195) - dp(949)
dp(951) = dp(2)*p(581) + p(2)*dp(581) - dp(917) - dp(948)
dp(952) = dp(1)*p(585) + p(1)*dp(585) - dp(928)
dp(953) = dp(7)*p(347) + p(7)*dp(347) - dp(952)
dp(954) = dp(45)*p(102) + p(45)*dp(102) - dp(913)
dp(955) = dp(2)*p(585) + p(2)*dp(585) - dp(930)
dp(956) = dp(1)*p(589) + p(1)*dp(589) - dp(955)
dp(957) = dp(6)*p(347) + p(6)*dp(347) - dp(933)
dp(958) = dp(1)*p(588) + p(1)*dp(588) - dp(937)
dp(959) = dp(2)*p(589) + p(2)*dp(589) - dp(957)
dp(960) = dp(9)*p(196) + p(9)*dp(196)
dp(961) = dp(9)*p(202) + p(9)*dp(202)
dp(962) = dp(9)*p(203) + p(9)*dp(203)
dp(963) = dp(9)*p(228) + p(9)*dp(228)
dp(964) = dp(9)*p(242) + p(9)*dp(242)
dp(965) = dp(9)*p(207) + p(9)*dp(207)
dp(966) = dp(9)*p(197) + p(9)*dp(197)
dp(967) = dp(9)*p(208) + p(9)*dp(208)
dp(968) = dp(9)*p(209) + p(9)*dp(209)
dp(969) = dp(9)*p(243) + p(9)*dp(243)
dp(970) = dp(9)*p(211) + p(9)*dp(211)
dp(971) = dp(9)*p(247) + p(9)*dp(247)
dp(972) = dp(9)*p(212) + p(9)*dp(212)
dp(973) = dp(9)*p(248) + p(9)*dp(248)
dp(974) = dp(9)*p(249) + p(9)*dp(249)
dp(975) = dp(9)*p(258) + p(9)*dp(258)
dp(976) = dp(9)*p(214) + p(9)*dp(214)
dp(977) = dp(9)*p(215) + p(9)*dp(215)
dp(978) = dp(9)*p(259) + p(9)*dp(259)
dp(979) = dp(9)*p(263) + p(9)*dp(263)
dp(980) = dp(9)*p(264) + p(9)*dp(264)
dp(981) = dp(9)*p(265) + p(9)*dp(265)
dp(982) = dp(9)*p(308) + p(9)*dp(308)
dp(983) = dp(9)*p(311) + p(9)*dp(311)
dp(984) = dp(9)*p(270) + p(9)*dp(270)
dp(985) = dp(9)*p(271) + p(9)*dp(271)
dp(986) = dp(9)*p(321) + p(9)*dp(321)
dp(987) = dp(9)*p(327) + p(9)*dp(327)
dp(988) = dp(9)*p(222) + p(9)*dp(222)
dp(989) = dp(9)*p(223) + p(9)*dp(223)
dp(990) = dp(9)*p(284) + p(9)*dp(284)
dp(991) = dp(9)*p(198) + p(9)*dp(198)
dp(992) = dp(9)*p(199) + p(9)*dp(199)
dp(993) = dp(9)*p(224) + p(9)*dp(224)
dp(994) = dp(9)*p(200) + p(9)*dp(200)
dp(995) = dp(9)*p(225) + p(9)*dp(225)
dp(996) = dp(9)*p(201) + p(9)*dp(201)
dp(997) = dp(9)*p(226) + p(9)*dp(226)
dp(998) = dp(9)*p(227) + p(9)*dp(227)
dp(999) = dp(9)*p(285) + p(9)*dp(285)
dp(1000) = dp(14)*p(328) + p(14)*dp(328)
dp(1001) = dp(14)*p(329) + p(14)*dp(329)
dp(1002) = dp(14)*p(330) + p(14)*dp(330)
dp(1003) = dp(9)*p(292) + p(9)*dp(292)
dp(1004) = dp(9)*p(233) + p(9)*dp(233)
dp(1005) = dp(9)*p(234) + p(9)*dp(234)
dp(1006) = dp(9)*p(235) + p(9)*dp(235)
dp(1007) = dp(9)*p(293) + p(9)*dp(293)
dp(1008) = dp(9)*p(236) + p(9)*dp(236)
dp(1009) = dp(9)*p(204) + p(9)*dp(204)
dp(1010) = dp(9)*p(205) + p(9)*dp(205)
dp(1011) = dp(9)*p(237) + p(9)*dp(237)
dp(1012) = dp(9)*p(238) + p(9)*dp(238)
dp(1013) = dp(9)*p(206) + p(9)*dp(206)
dp(1014) = dp(9)*p(239) + p(9)*dp(239)
dp(1015) = dp(9)*p(240) + p(9)*dp(240)
dp(1016) = dp(9)*p(241) + p(9)*dp(241)
dp(1017) = dp(9)*p(294) + p(9)*dp(294)
dp(1018) = dp(14)*p(276) + p(14)*dp(276)
dp(1019) = dp(14)*p(277) + p(14)*dp(277)
dp(1020) = dp(14)*p(278) + p(14)*dp(278)
dp(1021) = dp(14)*p(279) + p(14)*dp(279)
dp(1022) = dp(14)*p(280) + p(14)*dp(280)
dp(1023) = dp(14)*p(281) + p(14)*dp(281)
dp(1024) = dp(14)*p(282) + p(14)*dp(282)
dp(1025) = dp(14)*p(283) + p(14)*dp(283)
dp(1026) = dp(9)*p(244) + p(9)*dp(244)
dp(1027) = dp(9)*p(295) + p(9)*dp(295)
dp(1028) = dp(9)*p(210) + p(9)*dp(210)
dp(1029) = dp(9)*p(245) + p(9)*dp(245)
dp(1030) = dp(9)*p(246) + p(9)*dp(246)
dp(1031) = dp(9)*p(296) + p(9)*dp(296)
dp(1032) = dp(14)*p(222) + p(14)*dp(222)
dp(1033) = dp(14)*p(223) + p(14)*dp(223)
dp(1034) = dp(14)*p(224) + p(14)*dp(224)
dp(1035) = dp(14)*p(225) + p(14)*dp(225)
dp(1036) = dp(14)*p(226) + p(14)*dp(226)
dp(1037) = dp(14)*p(227) + p(14)*dp(227)
dp(1038) = dp(14)*p(202) + p(14)*dp(202)
dp(1039) = dp(14)*p(203) + p(14)*dp(203)
dp(1040) = dp(9)*p(302) + p(9)*dp(302)
dp(1041) = dp(9)*p(252) + p(9)*dp(252)
dp(1042) = dp(9)*p(253) + p(9)*dp(253)
dp(1043) = dp(9)*p(303) + p(9)*dp(303)
dp(1044) = dp(9)*p(254) + p(9)*dp(254)
dp(1045) = dp(9)*p(213) + p(9)*dp(213)
dp(1046) = dp(9)*p(255) + p(9)*dp(255)
dp(1047) = dp(9)*p(256) + p(9)*dp(256)
dp(1048) = dp(9)*p(257) + p(9)*dp(257)
dp(1049) = dp(9)*p(304) + p(9)*dp(304)
dp(1050) = dp(14)*p(286) + p(14)*dp(286)
dp(1051) = dp(14)*p(287) + p(14)*dp(287)
dp(1052) = dp(14)*p(288) + p(14)*dp(288)
dp(1053) = dp(14)*p(289) + p(14)*dp(289)
dp(1054) = dp(14)*p(290) + p(14)*dp(290)
dp(1055) = dp(14)*p(291) + p(14)*dp(291)
dp(1056) = dp(9)*p(305) + p(9)*dp(305)
dp(1057) = dp(9)*p(260) + p(9)*dp(260)
dp(1058) = dp(9)*p(306) + p(9)*dp(306)
dp(1059) = dp(9)*p(261) + p(9)*dp(261)
dp(1060) = dp(9)*p(262) + p(9)*dp(262)
dp(1061) = dp(9)*p(307) + p(9)*dp(307)
dp(1062) = dp(14)*p(233) + p(14)*dp(233)
dp(1063) = dp(14)*p(234) + p(14)*dp(234)
dp(1064) = dp(14)*p(235) + p(14)*dp(235)
dp(1065) = dp(14)*p(236) + p(14)*dp(236)
dp(1066) = dp(14)*p(237) + p(14)*dp(237)
dp(1067) = dp(14)*p(238) + p(14)*dp(238)
dp(1068) = dp(14)*p(239) + p(14)*dp(239)
dp(1069) = dp(14)*p(240) + p(14)*dp(240)
dp(1070) = dp(14)*p(241) + p(14)*dp(241)
dp(1071) = dp(14)*p(207) + p(14)*dp(207)
dp(1072) = dp(14)*p(208) + p(14)*dp(208)
dp(1073) = dp(14)*p(209) + p(14)*dp(209)
dp(1074) = dp(9)*p(309) + p(9)*dp(309)
dp(1075) = dp(9)*p(310) + p(9)*dp(310)
dp(1076) = dp(14)*p(244) + p(14)*dp(244)
dp(1077) = dp(14)*p(245) + p(14)*dp(245)
dp(1078) = dp(14)*p(246) + p(14)*dp(246)
dp(1079) = dp(14)*p(211) + p(14)*dp(211)
dp(1080) = dp(14)*p(212) + p(14)*dp(212)
dp(1081) = dp(9)*p(315) + p(9)*dp(315)
dp(1082) = dp(9)*p(267) + p(9)*dp(267)
dp(1083) = dp(9)*p(316) + p(9)*dp(316)
dp(1084) = dp(9)*p(268) + p(9)*dp(268)
dp(1085) = dp(9)*p(269) + p(9)*dp(269)
dp(1086) = dp(9)*p(317) + p(9)*dp(317)
dp(1087) = dp(14)*p(297) + p(14)*dp(297)
dp(1088) = dp(14)*p(298) + p(14)*dp(298)
dp(1089) = dp(14)*p(299) + p(14)*dp(299)
dp(1090) = dp(14)*p(300) + p(14)*dp(300)
dp(1091) = dp(14)*p(301) + p(14)*dp(301)
dp(1092) = dp(9)*p(318) + p(9)*dp(318)
dp(1093) = dp(9)*p(319) + p(9)*dp(319)
dp(1094) = dp(9)*p(320) + p(9)*dp(320)
dp(1095) = dp(14)*p(252) + p(14)*dp(252)
dp(1096) = dp(14)*p(253) + p(14)*dp(253)
dp(1097) = dp(14)*p(254) + p(14)*dp(254)
dp(1098) = dp(14)*p(255) + p(14)*dp(255)
dp(1099) = dp(14)*p(256) + p(14)*dp(256)
dp(1100) = dp(14)*p(257) + p(14)*dp(257)
dp(1101) = dp(14)*p(214) + p(14)*dp(214)
dp(1102) = dp(14)*p(215) + p(14)*dp(215)
dp(1103) = dp(9)*p(339) + p(9)*dp(339)
dp(1104) = dp(14)*p(260) + p(14)*dp(260)
dp(1105) = dp(14)*p(261) + p(14)*dp(261)
dp(1106) = dp(14)*p(262) + p(14)*dp(262)
dp(1107) = dp(9)*p(324) + p(9)*dp(324)
dp(1108) = dp(9)*p(325) + p(9)*dp(325)
dp(1109) = dp(9)*p(326) + p(9)*dp(326)
dp(1110) = dp(14)*p(312) + p(14)*dp(312)
dp(1111) = dp(14)*p(313) + p(14)*dp(313)
dp(1112) = dp(14)*p(314) + p(14)*dp(314)
dp(1113) = dp(9)*p(342) + p(9)*dp(342)
dp(1114) = dp(14)*p(267) + p(14)*dp(267)
dp(1115) = dp(14)*p(268) + p(14)*dp(268)
dp(1116) = dp(14)*p(269) + p(14)*dp(269)
dp(1117) = dp(9)*p(345) + p(9)*dp(345)
dp(1118) = dp(14)*p(322) + p(14)*dp(322)
dp(1119) = dp(14)*p(323) + p(14)*dp(323)
dp(1120) = dp(9)*p(331) + p(9)*dp(331)
dp(1121) = dp(9)*p(276) + p(9)*dp(276)
dp(1122) = dp(9)*p(216) + p(9)*dp(216)
dp(1123) = dp(9)*p(277) + p(9)*dp(277)
dp(1124) = dp(9)*p(217) + p(9)*dp(217)
dp(1125) = dp(9)*p(218) + p(9)*dp(218)
dp(1126) = dp(9)*p(278) + p(9)*dp(278)
dp(1127) = dp(9)*p(219) + p(9)*dp(219)
dp(1128) = dp(9)*p(220) + p(9)*dp(220)
dp(1129) = dp(9)*p(279) + p(9)*dp(279)
dp(1130) = dp(9)*p(280) + p(9)*dp(280)
dp(1131) = dp(9)*p(221) + p(9)*dp(221)
dp(1132) = dp(9)*p(281) + p(9)*dp(281)
dp(1133) = dp(9)*p(282) + p(9)*dp(282)
dp(1134) = dp(9)*p(283) + p(9)*dp(283)
dp(1135) = dp(9)*p(332) + p(9)*dp(332)
dp(1136) = dp(3)*p(401) + p(3)*dp(401) - dp(988)
dp(1137) = dp(7)*p(401) + p(7)*dp(401) - dp(991)
dp(1138) = dp(49)*p(114) + p(49)*dp(114) - dp(996)
dp(1139) = dp(24)*p(194) + p(24)*dp(194) - dp(999)
dp(1140) = dp(3)*p(504) + p(3)*dp(504) - dp(990) - dp(1139)
dp(1141) = dp(1)*p(695) + p(1)*dp(695) - dp(990) - dp(1140)
dp(1142) = dp(1)*p(696) + p(1)*dp(696) - dp(993) - dp(1141)
dp(1143) = dp(6)*p(566) + p(6)*dp(566) - dp(1136)
dp(1144) = dp(32)*p(178) + p(32)*dp(178) - dp(991)
dp(1145) = dp(39)*p(150) + p(39)*dp(150) - dp(996)
dp(1146) = dp(3)*p(505) + p(3)*dp(505) - dp(999)
dp(1147) = dp(14)*p(346) + p(14)*dp(346)
dp(1148) = dp(9)*p(333) + p(9)*dp(333)
dp(1149) = dp(9)*p(286) + p(9)*dp(286)
dp(1150) = dp(9)*p(229) + p(9)*dp(229)
dp(1151) = dp(9)*p(287) + p(9)*dp(287)
dp(1152) = dp(9)*p(230) + p(9)*dp(230)
dp(1153) = dp(9)*p(231) + p(9)*dp(231)
dp(1154) = dp(9)*p(288) + p(9)*dp(288)
dp(1155) = dp(9)*p(232) + p(9)*dp(232)
dp(1156) = dp(9)*p(289) + p(9)*dp(289)
dp(1157) = dp(9)*p(290) + p(9)*dp(290)
dp(1158) = dp(9)*p(291) + p(9)*dp(291)
dp(1159) = dp(9)*p(334) + p(9)*dp(334)
dp(1160) = dp(24)*p(181) + p(24)*dp(181) - dp(963)
dp(1161) = dp(41)*p(114) + p(41)*dp(114) - dp(960)
dp(1162) = dp(4)*p(401) + p(4)*dp(401) - dp(961)
dp(1163) = dp(1)*p(713) + p(1)*dp(713) - dp(1005) - dp(1162)
dp(1164) = dp(1)*p(714) + p(1)*dp(714) - dp(1006) - dp(1163)
dp(1165) = dp(1)*p(715) + p(1)*dp(715) - dp(1007) - dp(1164)
dp(1166) = dp(13)*p(331) + p(13)*dp(331) - dp(1147)
dp(1167) = dp(2)*p(695) + p(2)*dp(695) - dp(1002) - dp(1166)
dp(1168) = dp(32)*p(181) + p(32)*dp(181) - dp(1147)
dp(1169) = dp(1)*p(717) + p(1)*dp(717) - dp(1007) - dp(1167)
dp(1170) = dp(73)*p(91) + p(73)*dp(91) - dp(1147)
dp(1171) = dp(1)*p(719) + p(1)*dp(719) - dp(1011) - dp(1169)
dp(1172) = dp(41)*p(150) + p(41)*dp(150) - dp(1147)
dp(1173) = dp(13)*p(279) + p(13)*dp(279) - dp(961)
dp(1174) = dp(16)*p(285) + p(16)*dp(285) - dp(1147)
dp(1175) = dp(69)*p(73) + p(69)*dp(73) - dp(960)
dp(1176) = dp(4)*p(505) + p(4)*dp(505) - dp(1147)
dp(1177) = dp(11)*p(285) + p(11)*dp(285) - dp(963)
dp(1178) = dp(14)*p(331) + p(14)*dp(331)
dp(1179) = dp(14)*p(332) + p(14)*dp(332)
dp(1180) = dp(3)*p(430) + p(3)*dp(430) - dp(1026)
dp(1181) = dp(7)*p(430) + p(7)*dp(430) - dp(1028)
dp(1182) = dp(49)*p(130) + p(49)*dp(130) - dp(1031)
dp(1183) = dp(3)*p(517) + p(3)*dp(517) - dp(1027) - dp(1182)
dp(1184) = dp(1)*p(730) + p(1)*dp(730) - dp(1027) - dp(1183)
dp(1185) = dp(45)*p(178) + p(45)*dp(178) - dp(1180)
dp(1186) = dp(39)*p(160) + p(39)*dp(160) - dp(1028)
dp(1187) = dp(3)*p(518) + p(3)*dp(518) - dp(1031)
dp(1188) = dp(14)*p(284) + p(14)*dp(284)
dp(1189) = dp(14)*p(285) + p(14)*dp(285)
dp(1190) = dp(14)*p(228) + p(14)*dp(228)
dp(1191) = dp(9)*p(335) + p(9)*dp(335)
dp(1192) = dp(9)*p(297) + p(9)*dp(297)
dp(1193) = dp(9)*p(250) + p(9)*dp(250)
dp(1194) = dp(9)*p(298) + p(9)*dp(298)
dp(1195) = dp(9)*p(251) + p(9)*dp(251)
dp(1196) = dp(9)*p(299) + p(9)*dp(299)
dp(1197) = dp(9)*p(300) + p(9)*dp(300)
dp(1198) = dp(9)*p(301) + p(9)*dp(301)
dp(1199) = dp(9)*p(336) + p(9)*dp(336)
dp(1200) = dp(24)*p(183) + p(24)*dp(183) - dp(964)
dp(1201) = dp(43)*p(114) + p(43)*dp(114) - dp(966)
dp(1202) = dp(8)*p(401) + p(8)*dp(401) - dp(1021)
dp(1203) = dp(1)*p(745) + p(1)*dp(745) - dp(1042) - dp(1202)
dp(1204) = dp(1)*p(746) + p(1)*dp(746) - dp(1043) - dp(1203)
dp(1205) = dp(13)*p(333) + p(13)*dp(333) - dp(1178)
dp(1206) = dp(2)*p(717) + p(2)*dp(717) - dp(1025) - dp(1183)
dp(1207) = dp(32)*p(183) + p(32)*dp(183) - dp(1178)
dp(1208) = dp(1)*p(748) + p(1)*dp(748) - dp(1043) - dp(1206)
dp(1209) = dp(73)*p(93) + p(73)*dp(93) - dp(1178)
dp(1210) = dp(6)*p(509) + p(6)*dp(509) - dp(1021)
dp(1211) = dp(43)*p(150) + p(43)*dp(150) - dp(1178)
dp(1212) = dp(32)*p(153) + p(32)*dp(153) - dp(966)
dp(1213) = dp(18)*p(285) + p(18)*dp(285) - dp(1178)
dp(1214) = dp(34)*p(150) + p(34)*dp(150) - dp(964)
dp(1215) = dp(14)*p(333) + p(14)*dp(333)
dp(1216) = dp(14)*p(334) + p(14)*dp(334)
dp(1217) = dp(41)*p(130) + p(41)*dp(130) - dp(973)
dp(1218) = dp(4)*p(430) + p(4)*dp(430) - dp(970)
dp(1219) = dp(1)*p(758) + p(1)*dp(758) - dp(1057) - dp(1218)
dp(1220) = dp(1)*p(759) + p(1)*dp(759) - dp(1058) - dp(1219)
dp(1221) = dp(81)*p(91) + p(81)*dp(91) - dp(1189)
dp(1222) = dp(2)*p(730) + p(2)*dp(730) - dp(1034) - dp(1221)
dp(1223) = dp(41)*p(160) + p(41)*dp(160) - dp(1189)
dp(1224) = dp(1)*p(761) + p(1)*dp(761) - dp(1058) - dp(1222)
dp(1225) = dp(16)*p(296) + p(16)*dp(296) - dp(1189)
dp(1226) = dp(69)*p(81) + p(69)*dp(81) - dp(970)
dp(1227) = dp(4)*p(518) + p(4)*dp(518) - dp(1189)
dp(1228) = dp(11)*p(296) + p(11)*dp(296) - dp(973)
dp(1229) = dp(14)*p(292) + p(14)*dp(292)
dp(1230) = dp(14)*p(293) + p(14)*dp(293)
dp(1231) = dp(14)*p(294) + p(14)*dp(294)
dp(1232) = dp(14)*p(242) + p(14)*dp(242)
dp(1233) = dp(14)*p(243) + p(14)*dp(243)
dp(1234) = dp(3)*p(464) + p(3)*dp(464) - dp(1074)
dp(1235) = dp(7)*p(464) + p(7)*dp(464) - dp(1075)
dp(1236) = dp(3)*p(531) + p(3)*dp(531) - dp(1074) - dp(1235)
dp(1237) = dp(39)*p(187) + p(39)*dp(187) - dp(1234)
dp(1238) = dp(3)*p(532) + p(3)*dp(532) - dp(1075)
dp(1239) = dp(14)*p(295) + p(14)*dp(295)
dp(1240) = dp(14)*p(296) + p(14)*dp(296)
dp(1241) = dp(14)*p(247) + p(14)*dp(247)
dp(1242) = dp(14)*p(248) + p(14)*dp(248)
dp(1243) = dp(14)*p(249) + p(14)*dp(249)
dp(1244) = dp(9)*p(337) + p(9)*dp(337)
dp(1245) = dp(9)*p(312) + p(9)*dp(312)
dp(1246) = dp(9)*p(266) + p(9)*dp(266)
dp(1247) = dp(9)*p(313) + p(9)*dp(313)
dp(1248) = dp(9)*p(314) + p(9)*dp(314)
dp(1249) = dp(9)*p(338) + p(9)*dp(338)
dp(1250) = dp(24)*p(185) + p(24)*dp(185) - dp(975)
dp(1251) = dp(46)*p(114) + p(46)*dp(114) - dp(977)
dp(1252) = dp(1)*p(783) + p(1)*dp(783) - dp(1082) - dp(1251)
dp(1253) = dp(1)*p(784) + p(1)*dp(784) - dp(1083) - dp(1252)
dp(1254) = dp(13)*p(335) + p(13)*dp(335) - dp(1215)
dp(1255) = dp(2)*p(748) + p(2)*dp(748) - dp(1055) - dp(1222)
dp(1256) = dp(32)*p(185) + p(32)*dp(185) - dp(1215)
dp(1257) = dp(1)*p(786) + p(1)*dp(786) - dp(1083) - dp(1255)
dp(1258) = dp(73)*p(95) + p(73)*dp(95) - dp(1215)
dp(1259) = dp(13)*p(299) + p(13)*dp(299) - dp(977)
dp(1260) = dp(46)*p(150) + p(46)*dp(150) - dp(1215)
dp(1261) = dp(73)*p(84) + p(73)*dp(84) - dp(975)
dp(1262) = dp(14)*p(335) + p(14)*dp(335)
dp(1263) = dp(14)*p(336) + p(14)*dp(336)
dp(1264) = dp(43)*p(130) + p(43)*dp(130) - dp(974)
dp(1265) = dp(8)*p(430) + p(8)*dp(430) - dp(1038)
dp(1266) = dp(1)*p(794) + p(1)*dp(794) - dp(1093) - dp(1265)
dp(1267) = dp(81)*p(93) + p(81)*dp(93) - dp(1190)
dp(1268) = dp(2)*p(761) + p(2)*dp(761) - dp(1066) - dp(1236)
dp(1269) = dp(43)*p(160) + p(43)*dp(160) - dp(1190)
dp(1270) = dp(45)*p(153) + p(45)*dp(153) - dp(1038)
dp(1271) = dp(18)*p(296) + p(18)*dp(296) - dp(1190)
dp(1272) = dp(34)*p(160) + p(34)*dp(160) - dp(974)
dp(1273) = dp(14)*p(302) + p(14)*dp(302)
dp(1274) = dp(14)*p(303) + p(14)*dp(303)
dp(1275) = dp(14)*p(304) + p(14)*dp(304)
dp(1276) = dp(14)*p(258) + p(14)*dp(258)
dp(1277) = dp(14)*p(259) + p(14)*dp(259)
dp(1278) = dp(4)*p(464) + p(4)*dp(464) - dp(983)
dp(1279) = dp(1)*p(804) + p(1)*dp(804) - dp(1103) - dp(1278)
dp(1280) = dp(16)*p(310) + p(16)*dp(310) - dp(1240)
dp(1281) = dp(2)*p(772) + p(2)*dp(772) - dp(1077) - dp(1280)
dp(1282) = dp(4)*p(532) + p(4)*dp(532) - dp(1240)
dp(1283) = dp(11)*p(310) + p(11)*dp(310) - dp(983)
dp(1284) = dp(14)*p(305) + p(14)*dp(305)
dp(1285) = dp(14)*p(306) + p(14)*dp(306)
dp(1286) = dp(14)*p(307) + p(14)*dp(307)
dp(1287) = dp(14)*p(263) + p(14)*dp(263)
dp(1288) = dp(14)*p(264) + p(14)*dp(264)
dp(1289) = dp(14)*p(265) + p(14)*dp(265)
dp(1290) = dp(6)*p(464) + p(6)*dp(464) - dp(1104)
dp(1291) = dp(3)*p(578) + p(3)*dp(578) - dp(1290)
dp(1292) = dp(14)*p(309) + p(14)*dp(309)
dp(1293) = dp(14)*p(310) + p(14)*dp(310)
dp(1294) = dp(9)*p(340) + p(9)*dp(340)
dp(1295) = dp(9)*p(322) + p(9)*dp(322)
dp(1296) = dp(9)*p(323) + p(9)*dp(323)
dp(1297) = dp(9)*p(341) + p(9)*dp(341)
dp(1298) = dp(24)*p(188) + p(24)*dp(188) - dp(984)
dp(1299) = dp(50)*p(114) + p(50)*dp(114) - dp(1089)
dp(1300) = dp(1)*p(813) + p(1)*dp(813) - dp(1108) - dp(1299)
dp(1301) = dp(13)*p(337) + p(13)*dp(337) - dp(1262)
dp(1302) = dp(2)*p(786) + p(2)*dp(786) - dp(1091) - dp(1268)
dp(1303) = dp(32)*p(188) + p(32)*dp(188) - dp(1262)
dp(1304) = dp(6)*p(536) + p(6)*dp(536) - dp(1089)
dp(1305) = dp(73)*p(98) + p(73)*dp(98) - dp(1262)
dp(1306) = dp(32)*p(173) + p(32)*dp(173) - dp(984)
dp(1307) = dp(14)*p(337) + p(14)*dp(337)
dp(1308) = dp(14)*p(338) + p(14)*dp(338)
dp(1309) = dp(46)*p(130) + p(46)*dp(130) - dp(982)
dp(1310) = dp(1)*p(820) + p(1)*dp(820) - dp(1113) - dp(1309)
dp(1311) = dp(81)*p(95) + p(81)*dp(95) - dp(1232)
dp(1312) = dp(2)*p(796) + p(2)*dp(796) - dp(1098) - dp(1281)
dp(1313) = dp(46)*p(160) + p(46)*dp(160) - dp(1232)
dp(1314) = dp(81)*p(84) + p(81)*dp(84) - dp(982)
dp(1315) = dp(14)*p(315) + p(14)*dp(315)
dp(1316) = dp(14)*p(316) + p(14)*dp(316)
dp(1317) = dp(14)*p(317) + p(14)*dp(317)
dp(1318) = dp(14)*p(270) + p(14)*dp(270)
dp(1319) = dp(14)*p(271) + p(14)*dp(271)
dp(1320) = dp(8)*p(464) + p(8)*dp(464) - dp(1079)
dp(1321) = dp(18)*p(310) + p(18)*dp(310) - dp(1242)
dp(1322) = dp(34)*p(187) + p(34)*dp(187) - dp(1079)
dp(1323) = dp(14)*p(318) + p(14)*dp(318)
dp(1324) = dp(14)*p(319) + p(14)*dp(319)
dp(1325) = dp(14)*p(320) + p(14)*dp(320)
dp(1326) = dp(9)*p(343) + p(9)*dp(343)
dp(1327) = dp(9)*p(344) + p(9)*dp(344)
dp(1328) = dp(24)*p(191) + p(24)*dp(191) - dp(987)
dp(1329) = dp(1)*p(827) + p(1)*dp(827) - dp(1117) - dp(1328)
dp(1330) = dp(13)*p(340) + p(13)*dp(340) - dp(1307)
dp(1331) = dp(2)*p(815) + p(2)*dp(815) - dp(1112) - dp(1312)
dp(1332) = dp(32)*p(191) + p(32)*dp(191) - dp(1307)
dp(1333) = dp(13)*p(323) + p(13)*dp(323) - dp(987)
dp(1334) = dp(14)*p(340) + p(14)*dp(340)
dp(1335) = dp(14)*p(341) + p(14)*dp(341)
dp(1336) = dp(50)*p(130) + p(50)*dp(130) - dp(1102)
dp(1337) = dp(81)*p(98) + p(81)*dp(98) - dp(1276)
dp(1338) = dp(45)*p(173) + p(45)*dp(173) - dp(1102)
dp(1339) = dp(14)*p(324) + p(14)*dp(324)
dp(1340) = dp(14)*p(325) + p(14)*dp(325)
dp(1341) = dp(14)*p(326) + p(14)*dp(326)
dp(1342) = dp(9)*p(347) + p(9)*dp(347)
dp(1343) = dp(24)*p(195) + p(24)*dp(195) - dp(1119)
dp(1344) = dp(13)*p(343) + p(13)*dp(343) - dp(1334)
dp(1345) = dp(6)*p(561) + p(6)*dp(561) - dp(1119)
dp(1346) = dp(14)*p(343) + p(14)*dp(343)
dp(1347) = dp(14)*p(344) + p(14)*dp(344)
dp(1348) = dp(9)*p(272) + p(9)*dp(272)
dp(1349) = dp(9)*p(273) + p(9)*dp(273)
dp(1350) = dp(9)*p(274) + p(9)*dp(274)
dp(1351) = dp(9)*p(328) + p(9)*dp(328)
dp(1352) = dp(9)*p(275) + p(9)*dp(275)
dp(1353) = dp(9)*p(329) + p(9)*dp(329)
dp(1354) = dp(9)*p(330) + p(9)*dp(330)
dp(1355) = dp(9)*p(346) + p(9)*dp(346)
dp(1356) = dp(3)*p(570) + p(3)*dp(570) - dp(1120)
dp(1357) = dp(39)*p(181) + p(39)*dp(181) - dp(1122)
dp(1358) = dp(41)*p(178) + p(41)*dp(178) - dp(1125)
dp(1359) = dp(4)*p(566) + p(4)*dp(566) - dp(1129)
dp(1360) = dp(1)*p(842) + p(1)*dp(842) - dp(1126) - dp(1359)
dp(1361) = dp(1)*p(843) + p(1)*dp(843) - dp(1129)
dp(1362) = dp(1)*p(844) + p(1)*dp(844) - dp(1130) - dp(1360)
dp(1363) = dp(21)*p(279) + p(21)*dp(279) - dp(1125)
dp(1364) = dp(1)*p(846) + p(1)*dp(846) - dp(1133) - dp(1362)
dp(1365) = dp(69)*p(101) + p(69)*dp(101) - dp(1122)
dp(1366) = dp(1)*p(848) + p(1)*dp(848) - dp(1135) - dp(1364)
dp(1367) = dp(11)*p(346) + p(11)*dp(346) - dp(1120)
dp(1368) = dp(1)*p(850) + p(1)*dp(850) - dp(1140) - dp(1139)
dp(1369) = dp(1)*p(851) + p(1)*dp(851) - dp(1146)
dp(1370) = dp(3)*p(572) + p(3)*dp(572) - dp(1148)
dp(1371) = dp(39)*p(183) + p(39)*dp(183) - dp(1150)
dp(1372) = dp(43)*p(178) + p(43)*dp(178) - dp(1153)
dp(1373) = dp(1)*p(854) + p(1)*dp(854) - dp(1151) - dp(1372)
dp(1374) = dp(2)*p(843) + p(2)*dp(843) - dp(1143)
dp(1375) = dp(1)*p(855) + p(1)*dp(855) - dp(1154) - dp(1373)
dp(1376) = dp(7)*p(509) + p(7)*dp(509) - dp(1153)
dp(1377) = dp(1)*p(857) + p(1)*dp(857) - dp(1157) - dp(1375)
dp(1378) = dp(49)*p(153) + p(49)*dp(153) - dp(1150)
dp(1379) = dp(1)*p(859) + p(1)*dp(859) - dp(1159) - dp(1377)
dp(1380) = dp(34)*p(194) + p(34)*dp(194) - dp(1148)
dp(1381) = dp(6)*p(570) + p(6)*dp(570) - dp(1147)
dp(1382) = dp(1)*p(862) + p(1)*dp(862) - dp(1167) - dp(1165)
dp(1383) = dp(2)*p(851) + p(2)*dp(851) - dp(1147)
dp(1384) = dp(1)*p(864) + p(1)*dp(864) - dp(1183) - dp(1182)
dp(1385) = dp(1)*p(865) + p(1)*dp(865) - dp(1187)
dp(1386) = dp(3)*p(574) + p(3)*dp(574) - dp(1191)
dp(1387) = dp(39)*p(185) + p(39)*dp(185) - dp(1193)
dp(1388) = dp(46)*p(178) + p(46)*dp(178) - dp(1196)
dp(1389) = dp(1)*p(868) + p(1)*dp(868) - dp(1194) - dp(1388)
dp(1390) = dp(1)*p(869) + p(1)*dp(869) - dp(1196)
dp(1391) = dp(1)*p(870) + p(1)*dp(870) - dp(1197) - dp(1389)
dp(1392) = dp(21)*p(299) + p(21)*dp(299) - dp(1193)
dp(1393) = dp(1)*p(872) + p(1)*dp(872) - dp(1199) - dp(1391)
dp(1394) = dp(84)*p(101) + p(84)*dp(101) - dp(1191)
dp(1395) = dp(6)*p(572) + p(6)*dp(572) - dp(1178)
dp(1396) = dp(1)*p(875) + p(1)*dp(875) - dp(1206) - dp(1204)
dp(1397) = dp(8)*p(505) + p(8)*dp(505) - dp(1178)
dp(1398) = dp(45)*p(181) + p(45)*dp(181) - dp(1189)
dp(1399) = dp(1)*p(878) + p(1)*dp(878) - dp(1222) - dp(1220)
dp(1400) = dp(2)*p(865) + p(2)*dp(865) - dp(1189)
dp(1401) = dp(1)*p(880) + p(1)*dp(880) - dp(1236) - dp(1235)
dp(1402) = dp(1)*p(881) + p(1)*dp(881) - dp(1238)
dp(1403) = dp(3)*p(576) + p(3)*dp(576) - dp(1244)
dp(1404) = dp(39)*p(188) + p(39)*dp(188) - dp(1246)
dp(1405) = dp(1)*p(884) + p(1)*dp(884) - dp(1245) - dp(1404)
dp(1406) = dp(2)*p(869) + p(2)*dp(869) - dp(1210)
dp(1407) = dp(1)*p(885) + p(1)*dp(885) - dp(1247) - dp(1405)
dp(1408) = dp(7)*p(536) + p(7)*dp(536) - dp(1246)
dp(1409) = dp(1)*p(887) + p(1)*dp(887) - dp(1249) - dp(1407)
dp(1410) = dp(49)*p(173) + p(49)*dp(173) - dp(1244)
dp(1411) = dp(6)*p(574) + p(6)*dp(574) - dp(1215)
dp(1412) = dp(1)*p(890) + p(1)*dp(890) - dp(1255) - dp(1253)
dp(1413) = dp(22)*p(285) + p(22)*dp(285) - dp(1215)
dp(1414) = dp(45)*p(183) + p(45)*dp(183) - dp(1190)
dp(1415) = dp(1)*p(893) + p(1)*dp(893) - dp(1268) - dp(1266)
dp(1416) = dp(8)*p(518) + p(8)*dp(518) - dp(1190)
dp(1417) = dp(41)*p(187) + p(41)*dp(187) - dp(1240)
dp(1418) = dp(1)*p(896) + p(1)*dp(896) - dp(1281) - dp(1279)
dp(1419) = dp(2)*p(881) + p(2)*dp(881) - dp(1240)
dp(1420) = dp(14)*p(308) + p(14)*dp(308)
dp(1421) = dp(4)*p(547) + p(4)*dp(547) - dp(1106) - dp(1320)
dp(1422) = dp(1)*p(900) + p(1)*dp(900) - dp(1291)
dp(1423) = dp(14)*p(311) + p(14)*dp(311)
dp(1424) = dp(3)*p(579) + p(3)*dp(579) - dp(1294)
dp(1425) = dp(39)*p(191) + p(39)*dp(191) - dp(1296)
dp(1426) = dp(1)*p(903) + p(1)*dp(903) - dp(1295) - dp(1425)
dp(1427) = dp(1)*p(904) + p(1)*dp(904) - dp(1296)
dp(1428) = dp(1)*p(905) + p(1)*dp(905) - dp(1297) - dp(1426)
dp(1429) = dp(21)*p(323) + p(21)*dp(323) - dp(1294)
dp(1430) = dp(6)*p(576) + p(6)*dp(576) - dp(1262)
dp(1431) = dp(1)*p(908) + p(1)*dp(908) - dp(1302) - dp(1300)
dp(1432) = dp(50)*p(150) + p(50)*dp(150) - dp(1262)
dp(1433) = dp(45)*p(185) + p(45)*dp(185) - dp(1232)
dp(1434) = dp(1)*p(911) + p(1)*dp(911) - dp(1312) - dp(1310)
dp(1435) = dp(22)*p(296) + p(22)*dp(296) - dp(1232)
dp(1436) = dp(43)*p(187) + p(43)*dp(187) - dp(1242)
dp(1437) = dp(1)*p(915) + p(1)*dp(915) - dp(1322) - dp(1320)
dp(1438) = dp(8)*p(532) + p(8)*dp(532) - dp(1242)
dp(1439) = dp(14)*p(321) + p(14)*dp(321)
dp(1440) = dp(4)*p(578) + p(4)*dp(578) - dp(1293)
dp(1441) = dp(2)*p(899) + p(2)*dp(899) - dp(1292) - dp(1440)
dp(1442) = dp(2)*p(900) + p(2)*dp(900) - dp(1293)
dp(1443) = dp(14)*p(339) + p(14)*dp(339)
dp(1444) = dp(3)*p(582) + p(3)*dp(582) - dp(1326)
dp(1445) = dp(2)*p(903) + p(2)*dp(903) - dp(1303) - dp(1299)
dp(1446) = dp(2)*p(904) + p(2)*dp(904) - dp(1304)
dp(1447) = dp(1)*p(919) + p(1)*dp(919) - dp(1327) - dp(1445)
dp(1448) = dp(7)*p(561) + p(7)*dp(561) - dp(1326)
dp(1449) = dp(6)*p(579) + p(6)*dp(579) - dp(1307)
dp(1450) = dp(1)*p(922) + p(1)*dp(922) - dp(1331) - dp(1329)
dp(1451) = dp(73)*p(102) + p(73)*dp(102) - dp(1307)
dp(1452) = dp(45)*p(188) + p(45)*dp(188) - dp(1276)
dp(1453) = dp(1)*p(925) + p(1)*dp(925) - dp(1338) - dp(1336)
dp(1454) = dp(50)*p(160) + p(50)*dp(160) - dp(1276)
dp(1455) = dp(14)*p(327) + p(14)*dp(327)
dp(1456) = dp(46)*p(187) + p(46)*dp(187) - dp(1243)
dp(1457) = dp(2)*p(915) + p(2)*dp(915) - dp(1324) - dp(1441)
dp(1458) = dp(22)*p(310) + p(22)*dp(310) - dp(1243)
dp(1459) = dp(14)*p(342) + p(14)*dp(342)
dp(1460) = dp(3)*p(585) + p(3)*dp(585) - dp(1342)
dp(1461) = dp(2)*p(919) + p(2)*dp(919) - dp(1332) - dp(1329)
dp(1462) = dp(1)*p(929) + p(1)*dp(929) - dp(1342)
dp(1463) = dp(6)*p(582) + p(6)*dp(582) - dp(1334)
dp(1464) = dp(1)*p(931) + p(1)*dp(931) - dp(1345) - dp(1343)
dp(1465) = dp(32)*p(195) + p(32)*dp(195) - dp(1334)
dp(1466) = dp(45)*p(191) + p(45)*dp(191) - dp(1318)
dp(1467) = dp(2)*p(925) + p(2)*dp(925) - dp(1340) - dp(1457)
dp(1468) = dp(81)*p(102) + p(81)*dp(102) - dp(1318)
dp(1469) = dp(14)*p(345) + p(14)*dp(345)
dp(1470) = dp(2)*p(928) + p(2)*dp(928) - dp(1344) - dp(1343)
dp(1471) = dp(2)*p(929) + p(2)*dp(929) - dp(1345)
dp(1472) = dp(6)*p(585) + p(6)*dp(585) - dp(1346)
dp(1473) = dp(2)*p(931) + p(2)*dp(931) - dp(1347) - dp(1467)
dp(1474) = dp(13)*p(347) + p(13)*dp(347) - dp(1346)
dp(1475) = dp(14)*p(347) + p(14)*dp(347)
dp(1476) = dp(3)*p(566) + p(3)*dp(566) - dp(1351)
dp(1477) = dp(7)*p(566) + p(7)*dp(566) - dp(1349)
dp(1478) = dp(49)*p(178) + p(49)*dp(178) - dp(1348)
dp(1479) = dp(39)*p(194) + p(39)*dp(194) - dp(1352)
dp(1480) = dp(3)*p(588) + p(3)*dp(588) - dp(1355)
dp(1481) = dp(1)*p(938) + p(1)*dp(938) - dp(1356)
dp(1482) = dp(2)*p(958) + p(2)*dp(958) - dp(1481)
dp(1483) = dp(1)*p(940) + p(1)*dp(940) - dp(1370)
dp(1484) = dp(8)*p(588) + p(8)*dp(588) - dp(1483)
dp(1485) = dp(1)*p(942) + p(1)*dp(942) - dp(1386)
dp(1486) = dp(22)*p(346) + p(22)*dp(346) - dp(1485)
dp(1487) = dp(1)*p(944) + p(1)*dp(944) - dp(1403)
dp(1488) = dp(50)*p(194) + p(50)*dp(194) - dp(1487)
dp(1489) = dp(1)*p(946) + p(1)*dp(946) - dp(1424)
dp(1490) = dp(101)*p(102) + p(101)*dp(102) - dp(1489)
dp(1491) = dp(6)*p(578) + p(6)*dp(578) - dp(1443)
dp(1492) = dp(1)*p(949) + p(1)*dp(949) - dp(1444)
dp(1493) = dp(49)*p(195) + p(49)*dp(195) - dp(1492)
dp(1494) = dp(8)*p(578) + p(8)*dp(578) - dp(1423)
dp(1495) = dp(1)*p(952) + p(1)*dp(952) - dp(1460)
dp(1496) = dp(21)*p(347) + p(21)*dp(347) - dp(1495)
dp(1497) = dp(50)*p(187) + p(50)*dp(187) - dp(1420)
dp(1498) = dp(1)*p(955) + p(1)*dp(955) - dp(1470)
dp(1499) = dp(7)*p(589) + p(7)*dp(589) - dp(1498)
dp(1500) = dp(45)*p(195) + p(45)*dp(195) - dp(1455)
dp(1501) = dp(2)*p(955) + p(2)*dp(955) - dp(1472)
dp(1502) = dp(1)*p(959) + p(1)*dp(959) - dp(1501)
dp(1503) = dp(6)*p(589) + p(6)*dp(589) - dp(1475)
dp(1504) = dp(1)*p(958) + p(1)*dp(958) - dp(1480)
dp(1505) = dp(2)*p(959) + p(2)*dp(959) - dp(1503)

do j=1,1506
  dbm1(j)=dp(j-1)
enddo

dbm2(1)=dbm1(4)
dbm2(2)=dbm1(10)
dbm2(3)=dbm1(11)
dbm2(4)=dbm1(12)
dbm2(5)=dbm1(13)
dbm2(6)=dbm1(15)
dbm2(7)=dbm1(16)
dbm2(8)=dbm1(24)
dbm2(9)=dbm1(25)
dbm2(10)=dbm1(26)
dbm2(11)=dbm1(27)
dbm2(12)=dbm1(28)
dbm2(13)=dbm1(29)
dbm2(14)=dbm1(30)
dbm2(15)=dbm1(31)
dbm2(16)=dbm1(32)
dbm2(17)=dbm1(34)
dbm2(18)=dbm1(35)
dbm2(19)=dbm1(36)
dbm2(20)=dbm1(37)
dbm2(21)=dbm1(39)
dbm2(22)=dbm1(40)
dbm2(23)=dbm1(41)
dbm2(24)=dbm1(52)
dbm2(25)=dbm1(53)
dbm2(26)=dbm1(54)
dbm2(27)=dbm1(55)
dbm2(28)=dbm1(56)
dbm2(29)=dbm1(57)
dbm2(30)=dbm1(58)
dbm2(31)=dbm1(59)
dbm2(32)=dbm1(60)
dbm2(33)=dbm1(61)
dbm2(34)=dbm1(62)
dbm2(35)=dbm1(63)
dbm2(36)=dbm1(64)
dbm2(37)=dbm1(65)
dbm2(38)=dbm1(66)
dbm2(39)=dbm1(67)
dbm2(40)=dbm1(68)
dbm2(41)=dbm1(69)
dbm2(42)=dbm1(70)
dbm2(43)=dbm1(71)
dbm2(44)=dbm1(72)
dbm2(45)=dbm1(73)
dbm2(46)=dbm1(75)
dbm2(47)=dbm1(76)
dbm2(48)=dbm1(77)
dbm2(49)=dbm1(78)
dbm2(50)=dbm1(79)
dbm2(51)=dbm1(81)
dbm2(52)=dbm1(83)
dbm2(53)=dbm1(84)
dbm2(54)=dbm1(85)
dbm2(55)=dbm1(86)
dbm2(56)=dbm1(87)
dbm2(57)=dbm1(89)
dbm2(58)=dbm1(90)
dbm2(59)=dbm1(91)
dbm2(60)=dbm1(104)
dbm2(61)=dbm1(105)
dbm2(62)=dbm1(106)
dbm2(63)=dbm1(107)
dbm2(64)=dbm1(108)
dbm2(65)=dbm1(109)
dbm2(66)=dbm1(110)
dbm2(67)=dbm1(111)
dbm2(68)=dbm1(112)
dbm2(69)=dbm1(113)
dbm2(70)=dbm1(114)
dbm2(71)=dbm1(115)
dbm2(72)=dbm1(116)
dbm2(73)=dbm1(117)
dbm2(74)=dbm1(118)
dbm2(75)=dbm1(119)
dbm2(76)=dbm1(120)
dbm2(77)=dbm1(121)
dbm2(78)=dbm1(122)
dbm2(79)=dbm1(123)
dbm2(80)=dbm1(124)
dbm2(81)=dbm1(125)
dbm2(82)=dbm1(126)
dbm2(83)=dbm1(127)
dbm2(84)=dbm1(128)
dbm2(85)=dbm1(129)
dbm2(86)=dbm1(130)
dbm2(87)=dbm1(131)
dbm2(88)=dbm1(132)
dbm2(89)=dbm1(133)
dbm2(90)=dbm1(134)
dbm2(91)=dbm1(135)
dbm2(92)=dbm1(136)
dbm2(93)=dbm1(137)
dbm2(94)=dbm1(138)
dbm2(95)=dbm1(139)
dbm2(96)=dbm1(140)
dbm2(97)=dbm1(141)
dbm2(98)=dbm1(142)
dbm2(99)=dbm1(143)
dbm2(100)=dbm1(144)
dbm2(101)=dbm1(145)
dbm2(102)=dbm1(146)
dbm2(103)=dbm1(147)
dbm2(104)=dbm1(148)
dbm2(105)=dbm1(149)
dbm2(106)=dbm1(150)
dbm2(107)=dbm1(152)
dbm2(108)=dbm1(153)
dbm2(109)=dbm1(154)
dbm2(110)=dbm1(155)
dbm2(111)=dbm1(156)
dbm2(112)=dbm1(157)
dbm2(113)=dbm1(158)
dbm2(114)=dbm1(160)
dbm2(115)=dbm1(162)
dbm2(116)=dbm1(163)
dbm2(117)=dbm1(164)
dbm2(118)=dbm1(165)
dbm2(119)=dbm1(166)
dbm2(120)=dbm1(167)
dbm2(121)=dbm1(169)
dbm2(122)=dbm1(170)
dbm2(123)=dbm1(172)
dbm2(124)=dbm1(173)
dbm2(125)=dbm1(174)
dbm2(126)=dbm1(175)
dbm2(127)=dbm1(176)
dbm2(128)=dbm1(178)
dbm2(129)=dbm1(179)
dbm2(130)=dbm1(180)
dbm2(131)=dbm1(181)
dbm2(132)=dbm1(197)
dbm2(133)=dbm1(198)
dbm2(134)=dbm1(199)
dbm2(135)=dbm1(200)
dbm2(136)=dbm1(201)
dbm2(137)=dbm1(202)
dbm2(138)=dbm1(203)
dbm2(139)=dbm1(204)
dbm2(140)=dbm1(205)
dbm2(141)=dbm1(206)
dbm2(142)=dbm1(207)
dbm2(143)=dbm1(208)
dbm2(144)=dbm1(209)
dbm2(145)=dbm1(210)
dbm2(146)=dbm1(211)
dbm2(147)=dbm1(212)
dbm2(148)=dbm1(213)
dbm2(149)=dbm1(214)
dbm2(150)=dbm1(215)
dbm2(151)=dbm1(216)
dbm2(152)=dbm1(217)
dbm2(153)=dbm1(218)
dbm2(154)=dbm1(219)
dbm2(155)=dbm1(220)
dbm2(156)=dbm1(221)
dbm2(157)=dbm1(222)
dbm2(158)=dbm1(223)
dbm2(159)=dbm1(224)
dbm2(160)=dbm1(225)
dbm2(161)=dbm1(226)
dbm2(162)=dbm1(227)
dbm2(163)=dbm1(228)
dbm2(164)=dbm1(229)
dbm2(165)=dbm1(230)
dbm2(166)=dbm1(231)
dbm2(167)=dbm1(232)
dbm2(168)=dbm1(233)
dbm2(169)=dbm1(234)
dbm2(170)=dbm1(235)
dbm2(171)=dbm1(236)
dbm2(172)=dbm1(237)
dbm2(173)=dbm1(238)
dbm2(174)=dbm1(239)
dbm2(175)=dbm1(240)
dbm2(176)=dbm1(241)
dbm2(177)=dbm1(242)
dbm2(178)=dbm1(243)
dbm2(179)=dbm1(244)
dbm2(180)=dbm1(245)
dbm2(181)=dbm1(246)
dbm2(182)=dbm1(247)
dbm2(183)=dbm1(248)
dbm2(184)=dbm1(249)
dbm2(185)=dbm1(250)
dbm2(186)=dbm1(251)
dbm2(187)=dbm1(252)
dbm2(188)=dbm1(253)
dbm2(189)=dbm1(254)
dbm2(190)=dbm1(255)
dbm2(191)=dbm1(256)
dbm2(192)=dbm1(257)
dbm2(193)=dbm1(258)
dbm2(194)=dbm1(259)
dbm2(195)=dbm1(260)
dbm2(196)=dbm1(261)
dbm2(197)=dbm1(262)
dbm2(198)=dbm1(263)
dbm2(199)=dbm1(264)
dbm2(200)=dbm1(265)
dbm2(201)=dbm1(266)
dbm2(202)=dbm1(267)
dbm2(203)=dbm1(268)
dbm2(204)=dbm1(269)
dbm2(205)=dbm1(270)
dbm2(206)=dbm1(271)
dbm2(207)=dbm1(272)
dbm2(208)=dbm1(273)
dbm2(209)=dbm1(274)
dbm2(210)=dbm1(275)
dbm2(211)=dbm1(276)
dbm2(212)=dbm1(277)
dbm2(213)=dbm1(278)
dbm2(214)=dbm1(279)
dbm2(215)=dbm1(280)
dbm2(216)=dbm1(281)
dbm2(217)=dbm1(282)
dbm2(218)=dbm1(283)
dbm2(219)=dbm1(284)
dbm2(220)=dbm1(285)
dbm2(221)=dbm1(287)
dbm2(222)=dbm1(288)
dbm2(223)=dbm1(289)
dbm2(224)=dbm1(290)
dbm2(225)=dbm1(291)
dbm2(226)=dbm1(292)
dbm2(227)=dbm1(293)
dbm2(228)=dbm1(294)
dbm2(229)=dbm1(296)
dbm2(230)=dbm1(298)
dbm2(231)=dbm1(299)
dbm2(232)=dbm1(300)
dbm2(233)=dbm1(301)
dbm2(234)=dbm1(302)
dbm2(235)=dbm1(303)
dbm2(236)=dbm1(304)
dbm2(237)=dbm1(306)
dbm2(238)=dbm1(307)
dbm2(239)=dbm1(309)
dbm2(240)=dbm1(310)
dbm2(241)=dbm1(312)
dbm2(242)=dbm1(313)
dbm2(243)=dbm1(314)
dbm2(244)=dbm1(315)
dbm2(245)=dbm1(316)
dbm2(246)=dbm1(317)
dbm2(247)=dbm1(319)
dbm2(248)=dbm1(320)
dbm2(249)=dbm1(322)
dbm2(250)=dbm1(323)
dbm2(251)=dbm1(324)
dbm2(252)=dbm1(325)
dbm2(253)=dbm1(326)
dbm2(254)=dbm1(328)
dbm2(255)=dbm1(329)
dbm2(256)=dbm1(330)
dbm2(257)=dbm1(331)
dbm2(258)=dbm1(349)
dbm2(259)=dbm1(350)
dbm2(260)=dbm1(351)
dbm2(261)=dbm1(352)
dbm2(262)=dbm1(353)
dbm2(263)=dbm1(354)
dbm2(264)=dbm1(355)
dbm2(265)=dbm1(356)
dbm2(266)=dbm1(357)
dbm2(267)=dbm1(358)
dbm2(268)=dbm1(359)
dbm2(269)=dbm1(360)
dbm2(270)=dbm1(361)
dbm2(271)=dbm1(362)
dbm2(272)=dbm1(363)
dbm2(273)=dbm1(364)
dbm2(274)=dbm1(365)
dbm2(275)=dbm1(366)
dbm2(276)=dbm1(367)
dbm2(277)=dbm1(368)
dbm2(278)=dbm1(369)
dbm2(279)=dbm1(370)
dbm2(280)=dbm1(371)
dbm2(281)=dbm1(372)
dbm2(282)=dbm1(373)
dbm2(283)=dbm1(374)
dbm2(284)=dbm1(375)
dbm2(285)=dbm1(376)
dbm2(286)=dbm1(377)
dbm2(287)=dbm1(378)
dbm2(288)=dbm1(379)
dbm2(289)=dbm1(380)
dbm2(290)=dbm1(381)
dbm2(291)=dbm1(382)
dbm2(292)=dbm1(383)
dbm2(293)=dbm1(384)
dbm2(294)=dbm1(385)
dbm2(295)=dbm1(386)
dbm2(296)=dbm1(387)
dbm2(297)=dbm1(388)
dbm2(298)=dbm1(389)
dbm2(299)=dbm1(390)
dbm2(300)=dbm1(391)
dbm2(301)=dbm1(392)
dbm2(302)=dbm1(393)
dbm2(303)=dbm1(394)
dbm2(304)=dbm1(395)
dbm2(305)=dbm1(396)
dbm2(306)=dbm1(397)
dbm2(307)=dbm1(398)
dbm2(308)=dbm1(399)
dbm2(309)=dbm1(400)
dbm2(310)=dbm1(401)
dbm2(311)=dbm1(402)
dbm2(312)=dbm1(403)
dbm2(313)=dbm1(404)
dbm2(314)=dbm1(405)
dbm2(315)=dbm1(406)
dbm2(316)=dbm1(407)
dbm2(317)=dbm1(408)
dbm2(318)=dbm1(409)
dbm2(319)=dbm1(410)
dbm2(320)=dbm1(411)
dbm2(321)=dbm1(412)
dbm2(322)=dbm1(413)
dbm2(323)=dbm1(414)
dbm2(324)=dbm1(415)
dbm2(325)=dbm1(416)
dbm2(326)=dbm1(417)
dbm2(327)=dbm1(418)
dbm2(328)=dbm1(419)
dbm2(329)=dbm1(420)
dbm2(330)=dbm1(421)
dbm2(331)=dbm1(422)
dbm2(332)=dbm1(423)
dbm2(333)=dbm1(424)
dbm2(334)=dbm1(425)
dbm2(335)=dbm1(426)
dbm2(336)=dbm1(427)
dbm2(337)=dbm1(428)
dbm2(338)=dbm1(429)
dbm2(339)=dbm1(430)
dbm2(340)=dbm1(431)
dbm2(341)=dbm1(432)
dbm2(342)=dbm1(433)
dbm2(343)=dbm1(434)
dbm2(344)=dbm1(435)
dbm2(345)=dbm1(436)
dbm2(346)=dbm1(437)
dbm2(347)=dbm1(438)
dbm2(348)=dbm1(439)
dbm2(349)=dbm1(440)
dbm2(350)=dbm1(441)
dbm2(351)=dbm1(442)
dbm2(352)=dbm1(443)
dbm2(353)=dbm1(444)
dbm2(354)=dbm1(445)
dbm2(355)=dbm1(446)
dbm2(356)=dbm1(447)
dbm2(357)=dbm1(448)
dbm2(358)=dbm1(449)
dbm2(359)=dbm1(450)
dbm2(360)=dbm1(451)
dbm2(361)=dbm1(452)
dbm2(362)=dbm1(453)
dbm2(363)=dbm1(454)
dbm2(364)=dbm1(455)
dbm2(365)=dbm1(456)
dbm2(366)=dbm1(457)
dbm2(367)=dbm1(458)
dbm2(368)=dbm1(459)
dbm2(369)=dbm1(460)
dbm2(370)=dbm1(461)
dbm2(371)=dbm1(462)
dbm2(372)=dbm1(463)
dbm2(373)=dbm1(464)
dbm2(374)=dbm1(465)
dbm2(375)=dbm1(466)
dbm2(376)=dbm1(467)
dbm2(377)=dbm1(468)
dbm2(378)=dbm1(469)
dbm2(379)=dbm1(470)
dbm2(380)=dbm1(471)
dbm2(381)=dbm1(472)
dbm2(382)=dbm1(473)
dbm2(383)=dbm1(474)
dbm2(384)=dbm1(475)
dbm2(385)=dbm1(476)
dbm2(386)=dbm1(477)
dbm2(387)=dbm1(478)
dbm2(388)=dbm1(479)
dbm2(389)=dbm1(480)
dbm2(390)=dbm1(481)
dbm2(391)=dbm1(482)
dbm2(392)=dbm1(483)
dbm2(393)=dbm1(484)
dbm2(394)=dbm1(485)
dbm2(395)=dbm1(486)
dbm2(396)=dbm1(487)
dbm2(397)=dbm1(488)
dbm2(398)=dbm1(489)
dbm2(399)=dbm1(490)
dbm2(400)=dbm1(491)
dbm2(401)=dbm1(492)
dbm2(402)=dbm1(493)
dbm2(403)=dbm1(494)
dbm2(404)=dbm1(495)
dbm2(405)=dbm1(496)
dbm2(406)=dbm1(497)
dbm2(407)=dbm1(498)
dbm2(408)=dbm1(499)
dbm2(409)=dbm1(500)
dbm2(410)=dbm1(501)
dbm2(411)=dbm1(502)
dbm2(412)=dbm1(503)
dbm2(413)=dbm1(504)
dbm2(414)=dbm1(505)
dbm2(415)=dbm1(507)
dbm2(416)=dbm1(508)
dbm2(417)=dbm1(509)
dbm2(418)=dbm1(510)
dbm2(419)=dbm1(511)
dbm2(420)=dbm1(512)
dbm2(421)=dbm1(513)
dbm2(422)=dbm1(514)
dbm2(423)=dbm1(515)
dbm2(424)=dbm1(516)
dbm2(425)=dbm1(518)
dbm2(426)=dbm1(520)
dbm2(427)=dbm1(521)
dbm2(428)=dbm1(522)
dbm2(429)=dbm1(523)
dbm2(430)=dbm1(524)
dbm2(431)=dbm1(525)
dbm2(432)=dbm1(526)
dbm2(433)=dbm1(527)
dbm2(434)=dbm1(529)
dbm2(435)=dbm1(530)
dbm2(436)=dbm1(532)
dbm2(437)=dbm1(534)
dbm2(438)=dbm1(535)
dbm2(439)=dbm1(536)
dbm2(440)=dbm1(537)
dbm2(441)=dbm1(538)
dbm2(442)=dbm1(539)
dbm2(443)=dbm1(540)
dbm2(444)=dbm1(541)
dbm2(445)=dbm1(543)
dbm2(446)=dbm1(544)
dbm2(447)=dbm1(546)
dbm2(448)=dbm1(547)
dbm2(449)=dbm1(548)
dbm2(450)=dbm1(550)
dbm2(451)=dbm1(551)
dbm2(452)=dbm1(552)
dbm2(453)=dbm1(553)
dbm2(454)=dbm1(554)
dbm2(455)=dbm1(555)
dbm2(456)=dbm1(557)
dbm2(457)=dbm1(558)
dbm2(458)=dbm1(560)
dbm2(459)=dbm1(561)
dbm2(460)=dbm1(562)
dbm2(461)=dbm1(563)
dbm2(462)=dbm1(564)
dbm2(463)=dbm1(566)
dbm2(464)=dbm1(567)
dbm2(465)=dbm1(568)
dbm2(466)=dbm1(569)
dbm2(467)=dbm1(570)
dbm2(468)=dbm1(591)
dbm2(469)=dbm1(592)
dbm2(470)=dbm1(593)
dbm2(471)=dbm1(594)
dbm2(472)=dbm1(595)
dbm2(473)=dbm1(596)
dbm2(474)=dbm1(597)
dbm2(475)=dbm1(598)
dbm2(476)=dbm1(599)
dbm2(477)=dbm1(600)
dbm2(478)=dbm1(601)
dbm2(479)=dbm1(602)
dbm2(480)=dbm1(603)
dbm2(481)=dbm1(604)
dbm2(482)=dbm1(605)
dbm2(483)=dbm1(606)
dbm2(484)=dbm1(607)
dbm2(485)=dbm1(608)
dbm2(486)=dbm1(609)
dbm2(487)=dbm1(610)
dbm2(488)=dbm1(611)
dbm2(489)=dbm1(612)
dbm2(490)=dbm1(613)
dbm2(491)=dbm1(614)
dbm2(492)=dbm1(615)
dbm2(493)=dbm1(616)
dbm2(494)=dbm1(617)
dbm2(495)=dbm1(618)
dbm2(496)=dbm1(619)
dbm2(497)=dbm1(620)
dbm2(498)=dbm1(621)
dbm2(499)=dbm1(622)
dbm2(500)=dbm1(623)
dbm2(501)=dbm1(624)
dbm2(502)=dbm1(625)
dbm2(503)=dbm1(626)
dbm2(504)=dbm1(627)
dbm2(505)=dbm1(628)
dbm2(506)=dbm1(629)
dbm2(507)=dbm1(630)
dbm2(508)=dbm1(631)
dbm2(509)=dbm1(632)
dbm2(510)=dbm1(633)
dbm2(511)=dbm1(634)
dbm2(512)=dbm1(635)
dbm2(513)=dbm1(636)
dbm2(514)=dbm1(637)
dbm2(515)=dbm1(638)
dbm2(516)=dbm1(639)
dbm2(517)=dbm1(640)
dbm2(518)=dbm1(641)
dbm2(519)=dbm1(642)
dbm2(520)=dbm1(643)
dbm2(521)=dbm1(644)
dbm2(522)=dbm1(645)
dbm2(523)=dbm1(646)
dbm2(524)=dbm1(647)
dbm2(525)=dbm1(648)
dbm2(526)=dbm1(649)
dbm2(527)=dbm1(650)
dbm2(528)=dbm1(651)
dbm2(529)=dbm1(652)
dbm2(530)=dbm1(653)
dbm2(531)=dbm1(654)
dbm2(532)=dbm1(655)
dbm2(533)=dbm1(656)
dbm2(534)=dbm1(657)
dbm2(535)=dbm1(658)
dbm2(536)=dbm1(659)
dbm2(537)=dbm1(660)
dbm2(538)=dbm1(661)
dbm2(539)=dbm1(662)
dbm2(540)=dbm1(663)
dbm2(541)=dbm1(664)
dbm2(542)=dbm1(665)
dbm2(543)=dbm1(666)
dbm2(544)=dbm1(667)
dbm2(545)=dbm1(668)
dbm2(546)=dbm1(669)
dbm2(547)=dbm1(670)
dbm2(548)=dbm1(671)
dbm2(549)=dbm1(672)
dbm2(550)=dbm1(673)
dbm2(551)=dbm1(674)
dbm2(552)=dbm1(675)
dbm2(553)=dbm1(676)
dbm2(554)=dbm1(677)
dbm2(555)=dbm1(678)
dbm2(556)=dbm1(679)
dbm2(557)=dbm1(680)
dbm2(558)=dbm1(681)
dbm2(559)=dbm1(682)
dbm2(560)=dbm1(683)
dbm2(561)=dbm1(684)
dbm2(562)=dbm1(685)
dbm2(563)=dbm1(686)
dbm2(564)=dbm1(687)
dbm2(565)=dbm1(688)
dbm2(566)=dbm1(689)
dbm2(567)=dbm1(690)
dbm2(568)=dbm1(691)
dbm2(569)=dbm1(692)
dbm2(570)=dbm1(693)
dbm2(571)=dbm1(694)
dbm2(572)=dbm1(695)
dbm2(573)=dbm1(696)
dbm2(574)=dbm1(697)
dbm2(575)=dbm1(698)
dbm2(576)=dbm1(699)
dbm2(577)=dbm1(700)
dbm2(578)=dbm1(701)
dbm2(579)=dbm1(702)
dbm2(580)=dbm1(703)
dbm2(581)=dbm1(704)
dbm2(582)=dbm1(705)
dbm2(583)=dbm1(706)
dbm2(584)=dbm1(707)
dbm2(585)=dbm1(708)
dbm2(586)=dbm1(709)
dbm2(587)=dbm1(710)
dbm2(588)=dbm1(711)
dbm2(589)=dbm1(712)
dbm2(590)=dbm1(713)
dbm2(591)=dbm1(714)
dbm2(592)=dbm1(715)
dbm2(593)=dbm1(716)
dbm2(594)=dbm1(717)
dbm2(595)=dbm1(718)
dbm2(596)=dbm1(719)
dbm2(597)=dbm1(720)
dbm2(598)=dbm1(721)
dbm2(599)=dbm1(722)
dbm2(600)=dbm1(723)
dbm2(601)=dbm1(724)
dbm2(602)=dbm1(725)
dbm2(603)=dbm1(726)
dbm2(604)=dbm1(727)
dbm2(605)=dbm1(728)
dbm2(606)=dbm1(729)
dbm2(607)=dbm1(730)
dbm2(608)=dbm1(731)
dbm2(609)=dbm1(732)
dbm2(610)=dbm1(733)
dbm2(611)=dbm1(734)
dbm2(612)=dbm1(735)
dbm2(613)=dbm1(736)
dbm2(614)=dbm1(737)
dbm2(615)=dbm1(738)
dbm2(616)=dbm1(739)
dbm2(617)=dbm1(740)
dbm2(618)=dbm1(741)
dbm2(619)=dbm1(742)
dbm2(620)=dbm1(743)
dbm2(621)=dbm1(744)
dbm2(622)=dbm1(745)
dbm2(623)=dbm1(746)
dbm2(624)=dbm1(747)
dbm2(625)=dbm1(748)
dbm2(626)=dbm1(749)
dbm2(627)=dbm1(750)
dbm2(628)=dbm1(751)
dbm2(629)=dbm1(752)
dbm2(630)=dbm1(753)
dbm2(631)=dbm1(754)
dbm2(632)=dbm1(755)
dbm2(633)=dbm1(756)
dbm2(634)=dbm1(757)
dbm2(635)=dbm1(758)
dbm2(636)=dbm1(759)
dbm2(637)=dbm1(760)
dbm2(638)=dbm1(761)
dbm2(639)=dbm1(762)
dbm2(640)=dbm1(763)
dbm2(641)=dbm1(764)
dbm2(642)=dbm1(765)
dbm2(643)=dbm1(766)
dbm2(644)=dbm1(767)
dbm2(645)=dbm1(768)
dbm2(646)=dbm1(769)
dbm2(647)=dbm1(770)
dbm2(648)=dbm1(771)
dbm2(649)=dbm1(772)
dbm2(650)=dbm1(773)
dbm2(651)=dbm1(774)
dbm2(652)=dbm1(775)
dbm2(653)=dbm1(776)
dbm2(654)=dbm1(777)
dbm2(655)=dbm1(778)
dbm2(656)=dbm1(779)
dbm2(657)=dbm1(780)
dbm2(658)=dbm1(781)
dbm2(659)=dbm1(782)
dbm2(660)=dbm1(783)
dbm2(661)=dbm1(784)
dbm2(662)=dbm1(785)
dbm2(663)=dbm1(786)
dbm2(664)=dbm1(787)
dbm2(665)=dbm1(788)
dbm2(666)=dbm1(789)
dbm2(667)=dbm1(790)
dbm2(668)=dbm1(791)
dbm2(669)=dbm1(792)
dbm2(670)=dbm1(793)
dbm2(671)=dbm1(794)
dbm2(672)=dbm1(795)
dbm2(673)=dbm1(796)
dbm2(674)=dbm1(797)
dbm2(675)=dbm1(798)
dbm2(676)=dbm1(799)
dbm2(677)=dbm1(800)
dbm2(678)=dbm1(801)
dbm2(679)=dbm1(802)
dbm2(680)=dbm1(803)
dbm2(681)=dbm1(804)
dbm2(682)=dbm1(805)
dbm2(683)=dbm1(806)
dbm2(684)=dbm1(807)
dbm2(685)=dbm1(808)
dbm2(686)=dbm1(809)
dbm2(687)=dbm1(810)
dbm2(688)=dbm1(811)
dbm2(689)=dbm1(812)
dbm2(690)=dbm1(813)
dbm2(691)=dbm1(814)
dbm2(692)=dbm1(815)
dbm2(693)=dbm1(816)
dbm2(694)=dbm1(817)
dbm2(695)=dbm1(818)
dbm2(696)=dbm1(819)
dbm2(697)=dbm1(820)
dbm2(698)=dbm1(821)
dbm2(699)=dbm1(822)
dbm2(700)=dbm1(823)
dbm2(701)=dbm1(824)
dbm2(702)=dbm1(825)
dbm2(703)=dbm1(826)
dbm2(704)=dbm1(827)
dbm2(705)=dbm1(828)
dbm2(706)=dbm1(829)
dbm2(707)=dbm1(830)
dbm2(708)=dbm1(831)
dbm2(709)=dbm1(832)
dbm2(710)=dbm1(833)
dbm2(711)=dbm1(834)
dbm2(712)=dbm1(835)
dbm2(713)=dbm1(836)
dbm2(714)=dbm1(837)
dbm2(715)=dbm1(838)
dbm2(716)=dbm1(839)
dbm2(717)=dbm1(840)
dbm2(718)=dbm1(841)
dbm2(719)=dbm1(842)
dbm2(720)=dbm1(843)
dbm2(721)=dbm1(844)
dbm2(722)=dbm1(845)
dbm2(723)=dbm1(846)
dbm2(724)=dbm1(847)
dbm2(725)=dbm1(848)
dbm2(726)=dbm1(849)
dbm2(727)=dbm1(850)
dbm2(728)=dbm1(851)
dbm2(729)=dbm1(853)
dbm2(730)=dbm1(854)
dbm2(731)=dbm1(855)
dbm2(732)=dbm1(856)
dbm2(733)=dbm1(857)
dbm2(734)=dbm1(858)
dbm2(735)=dbm1(859)
dbm2(736)=dbm1(860)
dbm2(737)=dbm1(861)
dbm2(738)=dbm1(862)
dbm2(739)=dbm1(863)
dbm2(740)=dbm1(865)
dbm2(741)=dbm1(867)
dbm2(742)=dbm1(868)
dbm2(743)=dbm1(869)
dbm2(744)=dbm1(870)
dbm2(745)=dbm1(871)
dbm2(746)=dbm1(872)
dbm2(747)=dbm1(873)
dbm2(748)=dbm1(874)
dbm2(749)=dbm1(875)
dbm2(750)=dbm1(876)
dbm2(751)=dbm1(878)
dbm2(752)=dbm1(879)
dbm2(753)=dbm1(881)
dbm2(754)=dbm1(883)
dbm2(755)=dbm1(884)
dbm2(756)=dbm1(885)
dbm2(757)=dbm1(886)
dbm2(758)=dbm1(887)
dbm2(759)=dbm1(888)
dbm2(760)=dbm1(889)
dbm2(761)=dbm1(890)
dbm2(762)=dbm1(891)
dbm2(763)=dbm1(893)
dbm2(764)=dbm1(894)
dbm2(765)=dbm1(896)
dbm2(766)=dbm1(897)
dbm2(767)=dbm1(899)
dbm2(768)=dbm1(900)
dbm2(769)=dbm1(902)
dbm2(770)=dbm1(903)
dbm2(771)=dbm1(904)
dbm2(772)=dbm1(905)
dbm2(773)=dbm1(906)
dbm2(774)=dbm1(907)
dbm2(775)=dbm1(908)
dbm2(776)=dbm1(909)
dbm2(777)=dbm1(911)
dbm2(778)=dbm1(912)
dbm2(779)=dbm1(914)
dbm2(780)=dbm1(915)
dbm2(781)=dbm1(916)
dbm2(782)=dbm1(918)
dbm2(783)=dbm1(919)
dbm2(784)=dbm1(920)
dbm2(785)=dbm1(921)
dbm2(786)=dbm1(922)
dbm2(787)=dbm1(923)
dbm2(788)=dbm1(925)
dbm2(789)=dbm1(926)
dbm2(790)=dbm1(928)
dbm2(791)=dbm1(929)
dbm2(792)=dbm1(930)
dbm2(793)=dbm1(931)
dbm2(794)=dbm1(932)
dbm2(795)=dbm1(934)
dbm2(796)=dbm1(935)
dbm2(797)=dbm1(936)
dbm2(798)=dbm1(937)
dbm2(799)=dbm1(938)
dbm2(800)=dbm1(961)
dbm2(801)=dbm1(962)
dbm2(802)=dbm1(963)
dbm2(803)=dbm1(964)
dbm2(804)=dbm1(965)
dbm2(805)=dbm1(966)
dbm2(806)=dbm1(967)
dbm2(807)=dbm1(968)
dbm2(808)=dbm1(969)
dbm2(809)=dbm1(970)
dbm2(810)=dbm1(971)
dbm2(811)=dbm1(972)
dbm2(812)=dbm1(973)
dbm2(813)=dbm1(974)
dbm2(814)=dbm1(975)
dbm2(815)=dbm1(976)
dbm2(816)=dbm1(977)
dbm2(817)=dbm1(978)
dbm2(818)=dbm1(979)
dbm2(819)=dbm1(980)
dbm2(820)=dbm1(981)
dbm2(821)=dbm1(982)
dbm2(822)=dbm1(983)
dbm2(823)=dbm1(984)
dbm2(824)=dbm1(985)
dbm2(825)=dbm1(986)
dbm2(826)=dbm1(987)
dbm2(827)=dbm1(988)
dbm2(828)=dbm1(989)
dbm2(829)=dbm1(990)
dbm2(830)=dbm1(991)
dbm2(831)=dbm1(992)
dbm2(832)=dbm1(993)
dbm2(833)=dbm1(994)
dbm2(834)=dbm1(995)
dbm2(835)=dbm1(996)
dbm2(836)=dbm1(997)
dbm2(837)=dbm1(998)
dbm2(838)=dbm1(999)
dbm2(839)=dbm1(1000)
dbm2(840)=dbm1(1001)
dbm2(841)=dbm1(1002)
dbm2(842)=dbm1(1003)
dbm2(843)=dbm1(1004)
dbm2(844)=dbm1(1005)
dbm2(845)=dbm1(1006)
dbm2(846)=dbm1(1007)
dbm2(847)=dbm1(1008)
dbm2(848)=dbm1(1009)
dbm2(849)=dbm1(1010)
dbm2(850)=dbm1(1011)
dbm2(851)=dbm1(1012)
dbm2(852)=dbm1(1013)
dbm2(853)=dbm1(1014)
dbm2(854)=dbm1(1015)
dbm2(855)=dbm1(1016)
dbm2(856)=dbm1(1017)
dbm2(857)=dbm1(1018)
dbm2(858)=dbm1(1019)
dbm2(859)=dbm1(1020)
dbm2(860)=dbm1(1021)
dbm2(861)=dbm1(1022)
dbm2(862)=dbm1(1023)
dbm2(863)=dbm1(1024)
dbm2(864)=dbm1(1025)
dbm2(865)=dbm1(1026)
dbm2(866)=dbm1(1027)
dbm2(867)=dbm1(1028)
dbm2(868)=dbm1(1029)
dbm2(869)=dbm1(1030)
dbm2(870)=dbm1(1031)
dbm2(871)=dbm1(1032)
dbm2(872)=dbm1(1033)
dbm2(873)=dbm1(1034)
dbm2(874)=dbm1(1035)
dbm2(875)=dbm1(1036)
dbm2(876)=dbm1(1037)
dbm2(877)=dbm1(1038)
dbm2(878)=dbm1(1039)
dbm2(879)=dbm1(1040)
dbm2(880)=dbm1(1041)
dbm2(881)=dbm1(1042)
dbm2(882)=dbm1(1043)
dbm2(883)=dbm1(1044)
dbm2(884)=dbm1(1045)
dbm2(885)=dbm1(1046)
dbm2(886)=dbm1(1047)
dbm2(887)=dbm1(1048)
dbm2(888)=dbm1(1049)
dbm2(889)=dbm1(1050)
dbm2(890)=dbm1(1051)
dbm2(891)=dbm1(1052)
dbm2(892)=dbm1(1053)
dbm2(893)=dbm1(1054)
dbm2(894)=dbm1(1055)
dbm2(895)=dbm1(1056)
dbm2(896)=dbm1(1057)
dbm2(897)=dbm1(1058)
dbm2(898)=dbm1(1059)
dbm2(899)=dbm1(1060)
dbm2(900)=dbm1(1061)
dbm2(901)=dbm1(1062)
dbm2(902)=dbm1(1063)
dbm2(903)=dbm1(1064)
dbm2(904)=dbm1(1065)
dbm2(905)=dbm1(1066)
dbm2(906)=dbm1(1067)
dbm2(907)=dbm1(1068)
dbm2(908)=dbm1(1069)
dbm2(909)=dbm1(1070)
dbm2(910)=dbm1(1071)
dbm2(911)=dbm1(1072)
dbm2(912)=dbm1(1073)
dbm2(913)=dbm1(1074)
dbm2(914)=dbm1(1075)
dbm2(915)=dbm1(1076)
dbm2(916)=dbm1(1077)
dbm2(917)=dbm1(1078)
dbm2(918)=dbm1(1079)
dbm2(919)=dbm1(1080)
dbm2(920)=dbm1(1081)
dbm2(921)=dbm1(1082)
dbm2(922)=dbm1(1083)
dbm2(923)=dbm1(1084)
dbm2(924)=dbm1(1085)
dbm2(925)=dbm1(1086)
dbm2(926)=dbm1(1087)
dbm2(927)=dbm1(1088)
dbm2(928)=dbm1(1089)
dbm2(929)=dbm1(1090)
dbm2(930)=dbm1(1091)
dbm2(931)=dbm1(1092)
dbm2(932)=dbm1(1093)
dbm2(933)=dbm1(1094)
dbm2(934)=dbm1(1095)
dbm2(935)=dbm1(1096)
dbm2(936)=dbm1(1097)
dbm2(937)=dbm1(1098)
dbm2(938)=dbm1(1099)
dbm2(939)=dbm1(1100)
dbm2(940)=dbm1(1101)
dbm2(941)=dbm1(1102)
dbm2(942)=dbm1(1103)
dbm2(943)=dbm1(1104)
dbm2(944)=dbm1(1105)
dbm2(945)=dbm1(1106)
dbm2(946)=dbm1(1107)
dbm2(947)=dbm1(1108)
dbm2(948)=dbm1(1109)
dbm2(949)=dbm1(1110)
dbm2(950)=dbm1(1111)
dbm2(951)=dbm1(1112)
dbm2(952)=dbm1(1113)
dbm2(953)=dbm1(1114)
dbm2(954)=dbm1(1115)
dbm2(955)=dbm1(1116)
dbm2(956)=dbm1(1117)
dbm2(957)=dbm1(1118)
dbm2(958)=dbm1(1119)
dbm2(959)=dbm1(1120)
dbm2(960)=dbm1(1121)
dbm2(961)=dbm1(1122)
dbm2(962)=dbm1(1123)
dbm2(963)=dbm1(1124)
dbm2(964)=dbm1(1125)
dbm2(965)=dbm1(1126)
dbm2(966)=dbm1(1127)
dbm2(967)=dbm1(1128)
dbm2(968)=dbm1(1129)
dbm2(969)=dbm1(1130)
dbm2(970)=dbm1(1131)
dbm2(971)=dbm1(1132)
dbm2(972)=dbm1(1133)
dbm2(973)=dbm1(1134)
dbm2(974)=dbm1(1135)
dbm2(975)=dbm1(1136)
dbm2(976)=dbm1(1137)
dbm2(977)=dbm1(1138)
dbm2(978)=dbm1(1139)
dbm2(979)=dbm1(1140)
dbm2(980)=dbm1(1141)
dbm2(981)=dbm1(1142)
dbm2(982)=dbm1(1143)
dbm2(983)=dbm1(1144)
dbm2(984)=dbm1(1145)
dbm2(985)=dbm1(1146)
dbm2(986)=dbm1(1147)
dbm2(987)=dbm1(1148)
dbm2(988)=dbm1(1149)
dbm2(989)=dbm1(1150)
dbm2(990)=dbm1(1151)
dbm2(991)=dbm1(1152)
dbm2(992)=dbm1(1153)
dbm2(993)=dbm1(1154)
dbm2(994)=dbm1(1155)
dbm2(995)=dbm1(1156)
dbm2(996)=dbm1(1157)
dbm2(997)=dbm1(1158)
dbm2(998)=dbm1(1159)
dbm2(999)=dbm1(1160)
dbm2(1000)=dbm1(1161)
dbm2(1001)=dbm1(1162)
dbm2(1002)=dbm1(1163)
dbm2(1003)=dbm1(1164)
dbm2(1004)=dbm1(1165)
dbm2(1005)=dbm1(1166)
dbm2(1006)=dbm1(1167)
dbm2(1007)=dbm1(1168)
dbm2(1008)=dbm1(1169)
dbm2(1009)=dbm1(1170)
dbm2(1010)=dbm1(1171)
dbm2(1011)=dbm1(1172)
dbm2(1012)=dbm1(1173)
dbm2(1013)=dbm1(1174)
dbm2(1014)=dbm1(1175)
dbm2(1015)=dbm1(1176)
dbm2(1016)=dbm1(1177)
dbm2(1017)=dbm1(1178)
dbm2(1018)=dbm1(1179)
dbm2(1019)=dbm1(1180)
dbm2(1020)=dbm1(1181)
dbm2(1021)=dbm1(1182)
dbm2(1022)=dbm1(1183)
dbm2(1023)=dbm1(1184)
dbm2(1024)=dbm1(1185)
dbm2(1025)=dbm1(1186)
dbm2(1026)=dbm1(1187)
dbm2(1027)=dbm1(1188)
dbm2(1028)=dbm1(1189)
dbm2(1029)=dbm1(1190)
dbm2(1030)=dbm1(1191)
dbm2(1031)=dbm1(1192)
dbm2(1032)=dbm1(1193)
dbm2(1033)=dbm1(1194)
dbm2(1034)=dbm1(1195)
dbm2(1035)=dbm1(1196)
dbm2(1036)=dbm1(1197)
dbm2(1037)=dbm1(1198)
dbm2(1038)=dbm1(1199)
dbm2(1039)=dbm1(1200)
dbm2(1040)=dbm1(1201)
dbm2(1041)=dbm1(1202)
dbm2(1042)=dbm1(1203)
dbm2(1043)=dbm1(1204)
dbm2(1044)=dbm1(1205)
dbm2(1045)=dbm1(1206)
dbm2(1046)=dbm1(1207)
dbm2(1047)=dbm1(1208)
dbm2(1048)=dbm1(1209)
dbm2(1049)=dbm1(1210)
dbm2(1050)=dbm1(1211)
dbm2(1051)=dbm1(1212)
dbm2(1052)=dbm1(1213)
dbm2(1053)=dbm1(1214)
dbm2(1054)=dbm1(1215)
dbm2(1055)=dbm1(1216)
dbm2(1056)=dbm1(1217)
dbm2(1057)=dbm1(1218)
dbm2(1058)=dbm1(1219)
dbm2(1059)=dbm1(1220)
dbm2(1060)=dbm1(1221)
dbm2(1061)=dbm1(1222)
dbm2(1062)=dbm1(1223)
dbm2(1063)=dbm1(1224)
dbm2(1064)=dbm1(1225)
dbm2(1065)=dbm1(1226)
dbm2(1066)=dbm1(1227)
dbm2(1067)=dbm1(1228)
dbm2(1068)=dbm1(1229)
dbm2(1069)=dbm1(1230)
dbm2(1070)=dbm1(1231)
dbm2(1071)=dbm1(1232)
dbm2(1072)=dbm1(1233)
dbm2(1073)=dbm1(1234)
dbm2(1074)=dbm1(1235)
dbm2(1075)=dbm1(1236)
dbm2(1076)=dbm1(1237)
dbm2(1077)=dbm1(1238)
dbm2(1078)=dbm1(1239)
dbm2(1079)=dbm1(1240)
dbm2(1080)=dbm1(1241)
dbm2(1081)=dbm1(1242)
dbm2(1082)=dbm1(1243)
dbm2(1083)=dbm1(1244)
dbm2(1084)=dbm1(1245)
dbm2(1085)=dbm1(1246)
dbm2(1086)=dbm1(1247)
dbm2(1087)=dbm1(1248)
dbm2(1088)=dbm1(1249)
dbm2(1089)=dbm1(1250)
dbm2(1090)=dbm1(1251)
dbm2(1091)=dbm1(1252)
dbm2(1092)=dbm1(1253)
dbm2(1093)=dbm1(1254)
dbm2(1094)=dbm1(1255)
dbm2(1095)=dbm1(1256)
dbm2(1096)=dbm1(1257)
dbm2(1097)=dbm1(1258)
dbm2(1098)=dbm1(1259)
dbm2(1099)=dbm1(1260)
dbm2(1100)=dbm1(1261)
dbm2(1101)=dbm1(1262)
dbm2(1102)=dbm1(1263)
dbm2(1103)=dbm1(1264)
dbm2(1104)=dbm1(1265)
dbm2(1105)=dbm1(1266)
dbm2(1106)=dbm1(1267)
dbm2(1107)=dbm1(1268)
dbm2(1108)=dbm1(1269)
dbm2(1109)=dbm1(1270)
dbm2(1110)=dbm1(1271)
dbm2(1111)=dbm1(1272)
dbm2(1112)=dbm1(1273)
dbm2(1113)=dbm1(1274)
dbm2(1114)=dbm1(1275)
dbm2(1115)=dbm1(1276)
dbm2(1116)=dbm1(1277)
dbm2(1117)=dbm1(1278)
dbm2(1118)=dbm1(1279)
dbm2(1119)=dbm1(1280)
dbm2(1120)=dbm1(1281)
dbm2(1121)=dbm1(1282)
dbm2(1122)=dbm1(1283)
dbm2(1123)=dbm1(1284)
dbm2(1124)=dbm1(1285)
dbm2(1125)=dbm1(1286)
dbm2(1126)=dbm1(1287)
dbm2(1127)=dbm1(1288)
dbm2(1128)=dbm1(1289)
dbm2(1129)=dbm1(1290)
dbm2(1130)=dbm1(1291)
dbm2(1131)=dbm1(1292)
dbm2(1132)=dbm1(1293)
dbm2(1133)=dbm1(1294)
dbm2(1134)=dbm1(1295)
dbm2(1135)=dbm1(1296)
dbm2(1136)=dbm1(1297)
dbm2(1137)=dbm1(1298)
dbm2(1138)=dbm1(1299)
dbm2(1139)=dbm1(1300)
dbm2(1140)=dbm1(1301)
dbm2(1141)=dbm1(1302)
dbm2(1142)=dbm1(1303)
dbm2(1143)=dbm1(1304)
dbm2(1144)=dbm1(1305)
dbm2(1145)=dbm1(1306)
dbm2(1146)=dbm1(1307)
dbm2(1147)=dbm1(1308)
dbm2(1148)=dbm1(1309)
dbm2(1149)=dbm1(1310)
dbm2(1150)=dbm1(1311)
dbm2(1151)=dbm1(1312)
dbm2(1152)=dbm1(1313)
dbm2(1153)=dbm1(1314)
dbm2(1154)=dbm1(1315)
dbm2(1155)=dbm1(1316)
dbm2(1156)=dbm1(1317)
dbm2(1157)=dbm1(1318)
dbm2(1158)=dbm1(1319)
dbm2(1159)=dbm1(1320)
dbm2(1160)=dbm1(1321)
dbm2(1161)=dbm1(1322)
dbm2(1162)=dbm1(1323)
dbm2(1163)=dbm1(1324)
dbm2(1164)=dbm1(1325)
dbm2(1165)=dbm1(1326)
dbm2(1166)=dbm1(1327)
dbm2(1167)=dbm1(1328)
dbm2(1168)=dbm1(1329)
dbm2(1169)=dbm1(1330)
dbm2(1170)=dbm1(1331)
dbm2(1171)=dbm1(1332)
dbm2(1172)=dbm1(1333)
dbm2(1173)=dbm1(1334)
dbm2(1174)=dbm1(1335)
dbm2(1175)=dbm1(1336)
dbm2(1176)=dbm1(1337)
dbm2(1177)=dbm1(1338)
dbm2(1178)=dbm1(1339)
dbm2(1179)=dbm1(1340)
dbm2(1180)=dbm1(1341)
dbm2(1181)=dbm1(1342)
dbm2(1182)=dbm1(1343)
dbm2(1183)=dbm1(1344)
dbm2(1184)=dbm1(1345)
dbm2(1185)=dbm1(1346)
dbm2(1186)=dbm1(1347)
dbm2(1187)=dbm1(1348)
dbm2(1188)=dbm1(1349)
dbm2(1189)=dbm1(1350)
dbm2(1190)=dbm1(1351)
dbm2(1191)=dbm1(1352)
dbm2(1192)=dbm1(1353)
dbm2(1193)=dbm1(1354)
dbm2(1194)=dbm1(1355)
dbm2(1195)=dbm1(1356)
dbm2(1196)=dbm1(1357)
dbm2(1197)=dbm1(1358)
dbm2(1198)=dbm1(1359)
dbm2(1199)=dbm1(1360)
dbm2(1200)=dbm1(1361)
dbm2(1201)=dbm1(1362)
dbm2(1202)=dbm1(1363)
dbm2(1203)=dbm1(1364)
dbm2(1204)=dbm1(1365)
dbm2(1205)=dbm1(1366)
dbm2(1206)=dbm1(1367)
dbm2(1207)=dbm1(1368)
dbm2(1208)=dbm1(1369)
dbm2(1209)=dbm1(1371)
dbm2(1210)=dbm1(1372)
dbm2(1211)=dbm1(1373)
dbm2(1212)=dbm1(1374)
dbm2(1213)=dbm1(1375)
dbm2(1214)=dbm1(1376)
dbm2(1215)=dbm1(1377)
dbm2(1216)=dbm1(1378)
dbm2(1217)=dbm1(1379)
dbm2(1218)=dbm1(1380)
dbm2(1219)=dbm1(1381)
dbm2(1220)=dbm1(1382)
dbm2(1221)=dbm1(1383)
dbm2(1222)=dbm1(1385)
dbm2(1223)=dbm1(1387)
dbm2(1224)=dbm1(1388)
dbm2(1225)=dbm1(1389)
dbm2(1226)=dbm1(1390)
dbm2(1227)=dbm1(1391)
dbm2(1228)=dbm1(1392)
dbm2(1229)=dbm1(1393)
dbm2(1230)=dbm1(1394)
dbm2(1231)=dbm1(1395)
dbm2(1232)=dbm1(1396)
dbm2(1233)=dbm1(1397)
dbm2(1234)=dbm1(1399)
dbm2(1235)=dbm1(1400)
dbm2(1236)=dbm1(1402)
dbm2(1237)=dbm1(1404)
dbm2(1238)=dbm1(1405)
dbm2(1239)=dbm1(1406)
dbm2(1240)=dbm1(1407)
dbm2(1241)=dbm1(1408)
dbm2(1242)=dbm1(1409)
dbm2(1243)=dbm1(1410)
dbm2(1244)=dbm1(1411)
dbm2(1245)=dbm1(1412)
dbm2(1246)=dbm1(1413)
dbm2(1247)=dbm1(1415)
dbm2(1248)=dbm1(1416)
dbm2(1249)=dbm1(1418)
dbm2(1250)=dbm1(1419)
dbm2(1251)=dbm1(1421)
dbm2(1252)=dbm1(1422)
dbm2(1253)=dbm1(1424)
dbm2(1254)=dbm1(1425)
dbm2(1255)=dbm1(1426)
dbm2(1256)=dbm1(1427)
dbm2(1257)=dbm1(1428)
dbm2(1258)=dbm1(1429)
dbm2(1259)=dbm1(1430)
dbm2(1260)=dbm1(1431)
dbm2(1261)=dbm1(1432)
dbm2(1262)=dbm1(1434)
dbm2(1263)=dbm1(1435)
dbm2(1264)=dbm1(1437)
dbm2(1265)=dbm1(1438)
dbm2(1266)=dbm1(1440)
dbm2(1267)=dbm1(1441)
dbm2(1268)=dbm1(1442)
dbm2(1269)=dbm1(1444)
dbm2(1270)=dbm1(1445)
dbm2(1271)=dbm1(1446)
dbm2(1272)=dbm1(1447)
dbm2(1273)=dbm1(1448)
dbm2(1274)=dbm1(1449)
dbm2(1275)=dbm1(1450)
dbm2(1276)=dbm1(1451)
dbm2(1277)=dbm1(1453)
dbm2(1278)=dbm1(1454)
dbm2(1279)=dbm1(1456)
dbm2(1280)=dbm1(1457)
dbm2(1281)=dbm1(1458)
dbm2(1282)=dbm1(1460)
dbm2(1283)=dbm1(1461)
dbm2(1284)=dbm1(1462)
dbm2(1285)=dbm1(1463)
dbm2(1286)=dbm1(1464)
dbm2(1287)=dbm1(1465)
dbm2(1288)=dbm1(1467)
dbm2(1289)=dbm1(1468)
dbm2(1290)=dbm1(1470)
dbm2(1291)=dbm1(1471)
dbm2(1292)=dbm1(1472)
dbm2(1293)=dbm1(1473)
dbm2(1294)=dbm1(1474)
dbm2(1295)=dbm1(1476)
dbm2(1296)=dbm1(1477)
dbm2(1297)=dbm1(1478)
dbm2(1298)=dbm1(1479)
dbm2(1299)=dbm1(1480)
dbm2(1300)=dbm1(1481)

dVdR(i)=dVdR(i) + dot_product(coeff,dbm2)

end do

end subroutine EvU12O10dVdR

!===========================================================================
! *def EvdRdX(coord,r,dvdr)
!  subtoutine of evaluating derivative of 6 nuclear distances w.r.t. xyz 
!  coord is the xyz coordinate 
!  r is the nuclear distances
!  drdx is the derivative of 6 nuclear distances w.r.t. xyz 
!===========================================================================
subroutine EvdRdX(coord,r,drdx)
real*8 :: coord(12),r(6),drdx(6,12)

! Start to calculate the non-zero dRdX
! dr1dx
      dRdX(1,1)=(coord(1)-coord(4))/r(1)
      dRdX(1,2)=(coord(2)-coord(5))/r(1)
      dRdX(1,3)=(coord(3)-coord(6))/r(1)
      dRdX(1,4)=-dRdX(1,1)
      dRdX(1,5)=-dRdX(1,2)
      dRdX(1,6)=-dRdX(1,3)

! dr2dx
      dRdX(2,1)=(coord(1)-coord(7))/r(2)
      dRdX(2,2)=(coord(2)-coord(8))/r(2)
      dRdX(2,3)=(coord(3)-coord(9))/r(2)
      dRdX(2,7)=-dRdX(2,1)
      dRdX(2,8)=-dRdX(2,2)
      dRdX(2,9)=-dRdX(2,3)

! dr3dx
      dRdX(3,1)=(coord(1)-coord(10))/r(3)
      dRdX(3,2)=(coord(2)-coord(11))/r(3)
      dRdX(3,3)=(coord(3)-coord(12))/r(3)
      dRdX(3,10)=-dRdX(3,1)
      dRdX(3,11)=-dRdX(3,2)
      dRdX(3,12)=-dRdX(3,3)

! dr4dx
      dRdX(4,4)=(coord(4)-coord(7))/r(4)
      dRdX(4,5)=(coord(5)-coord(8))/r(4)
      dRdX(4,6)=(coord(6)-coord(9))/r(4)
      dRdX(4,7)=-dRdX(4,4)
      dRdX(4,8)=-dRdX(4,5)
      dRdX(4,9)=-dRdX(4,6)

! dr5dx
      dRdX(5,4)=(coord(4)-coord(10))/r(5)
      dRdX(5,5)=(coord(5)-coord(11))/r(5)
      dRdX(5,6)=(coord(6)-coord(12))/r(5)
      dRdX(5,10)=-dRdX(5,4)
      dRdX(5,11)=-dRdX(5,5)
      dRdX(5,12)=-dRdX(5,6)

! dr6dx
      dRdX(6,7)=(coord(7)-coord(10))/r(6)
      dRdX(6,8)=(coord(8)-coord(11))/r(6)
      dRdX(6,9)=(coord(9)-coord(12))/r(6)
      dRdX(6,10)=-dRdX(6,7)
      dRdX(6,11)=-dRdX(6,8)
      dRdX(6,12)=-dRdX(6,9)
! Finish the calculation of non-zero dRdX

      return

end subroutine EvdRdX

!===========================================================================
! *def Num_dtdx(xyz,dtdx)
!  subtoutine of evaluating derivative of hyperbolic function
!  coord is the xyz coordinate 
!  dtdx is the derivative of hyperbolic function
!===========================================================================
subroutine Num_dtdx(xyz,dtdx)
implicit none
integer :: i,j,k,m,n
real*8 :: xyz(4,3)
real*8 :: dtdx(12)
real*8 :: xyz1(4,3), xyz2(4,3)
real*8 :: volume1, volume2

  do i=1,12
    dtdx(i)=0.000
  enddo

  do i=1,4
  do j=1,3
    do m=1,4
    do n=1,3
      xyz1(m,n)=xyz(m,n)
      xyz2(m,n)=xyz(m,n)
    enddo
    enddo
    k=(i-1)*3+j
    xyz1(i,j)=xyz(i,j)+0.001
    call P_volume(xyz1,volume1)
    xyz2(i,j)=xyz(i,j)-0.001
    call P_volume(xyz2,volume2)
    dtdx(k)=(volume1-volume2)/0.002
  enddo
  enddo
end subroutine Num_dtdx

!===========================================================================
! *def P_volume(xyz,volume) 
!  subtoutine of evaluating hyperbolic function
!  coord is the xyz coordinate 
!  volume is the hyperbolic function
!===========================================================================
subroutine P_volume(xyz,volume)
implicit none
integer :: i,j,k
real*8 :: volume
real*8 :: xyz(4,3)
real*8 :: rrel(3,3),cross(3)
  volume=0.0000
  do i=1,3
  do j=1,3
    rrel(i,j)=xyz(i,j)-xyz(4,j)
  enddo
  enddo
  cross(1)=rrel(2,2)*rrel(3,3)-rrel(2,3)*rrel(3,2)
  cross(2)=-rrel(2,1)*rrel(3,3)+rrel(2,3)*rrel(3,1)
  cross(3)=rrel(2,1)*rrel(3,2)-rrel(2,2)*rrel(3,1)
  do i=1,3
    volume=volume+rrel(1,i)*cross(i)
  enddo
  volume=tanh(15.4*volume)
end subroutine P_volume
!===========================================================================
! *def u13_dvdx(xyz,u13,du13dx)
!  main subtoutine of evaluating derivative of u13 w.r.t. the xyz coordinates
!  xyz is the coordinate
!  u13 is u13
!  du13dx is the gradient of u13
!===========================================================================
subroutine u13_dvdx(xyz,u13,du13dx)
implicit none
integer :: i,j,k
real*8 :: du13dx(4,3),xyz(4,3)
real*8 :: u13
real*8 :: coord(12), r(6)
real*8 :: dudx(12)
real*8 :: dvdx(12), dvdr(6), drdx(6,12)
real*8 :: volume

! Initialize dVdX(12)
      do i=1,12
        dVdX(i)=0.0d0
      enddo

! Initialize dRdX(6,12)
      do i=1,6
        do j=1,12
          dRdX(i,j)=0.0d0
        enddo
      enddo

! Initialize dVdR(6)
      do i=1,6
        dVdR(i)=0.0d0
      enddo

! Assign xyz to coord
      do i=1,4
      do j=1,3
        k=3*(i-1)+j
        coord(k)=xyz(i,j)
      enddo
      enddo

! Compute distances r
r(1)=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r(2)=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r(3)=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r(4)=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r(5)=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r(6)=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

! Call EvdVdR to evaluate dVdR(6)
      Call EvU13O10dVdR(coord,r,dvdr)

! Call EvdRdX to evaluate dRdX(6,12)
      Call EvdRdX(coord,r,drdx)

! Calculate dVdX by using chain rule: dV/dXi=(dV/dRj)*(dRj/dXi), j=1 to 6
      do i=1,12
        do j=1,6
          dVdX(i)=dVdX(i) + dVdR(j)*dRdX(j,i)
        enddo
      enddo

      do i=1,12
        dudx(i)=dvdx(i)/627.509474
      enddo

! Assign dvdx back to du13dvdx
      do i=1,4
      do j=1,3
        k=3*(i-1)+j
        du13dx(i,j)=dudx(k)
      enddo
      enddo

      return

end subroutine u13_dvdx 

!===========================================================================
! *def EvU13O10dVdR(coord,r,dvdr)
!  subtoutine of evaluating derivative of u12 w.r.t. the 6 nuclear distances
!  coord is the xyz coordinate 
!  r is the nuclear distances
!  dvdr is the derivative of u13 w.r.t. the 6 nuclear distances
!===========================================================================
subroutine EvU13O10dVdR(coord,r,dvdr)
integer, parameter:: ddp=kind(0.d0)
integer :: i,j
real(ddp) :: coord(12),r(6),dvdr(6)
real(ddp) :: a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer, parameter :: ncoeff=3075
real(ddp) :: coeff(ncoeff)
real(ddp),dimension(0:32) :: rm
real(ddp),dimension(0:32) :: dm
real(ddp),dimension(0:3392) :: p
real(ddp),dimension(0:3392) :: dp
real(ddp) :: x(6),dmsdr(6,6)
real(ddp) :: dbm1(3393), dbm2(3075)

! Assign coefficients first
a1=1.2561573
a2=1.1028022
b1=1.75
b2=1.61
ra1=1.196078
ra2=0.9333333
rb1=1.196078
rb2=0.9333333
coeff(1)=0.231115191881D+04
coeff(2)=0.766914172340D+04
coeff(3)=-0.557136124868D+04
coeff(4)=-0.285959979588D+04
coeff(5)=0.392680364188D+03
coeff(6)=-0.258600088240D+04
coeff(7)=-0.738930854520D+04
coeff(8)=-0.569480495279D+04
coeff(9)=0.278961865807D+04
coeff(10)=0.169344540759D+05
coeff(11)=-0.892714283177D+04
coeff(12)=0.581826911803D+04
coeff(13)=0.779309372717D+04
coeff(14)=0.138882801292D+05
coeff(15)=-0.109722406393D+04
coeff(16)=-0.272257153066D+04
coeff(17)=0.741726637872D+04
coeff(18)=0.306803738871D+04
coeff(19)=0.131329843696D+04
coeff(20)=-0.224688468756D+04
coeff(21)=0.551678306305D+04
coeff(22)=0.167202457181D+05
coeff(23)=0.451561471761D+04
coeff(24)=0.753446773078D+04
coeff(25)=-0.173266567650D+04
coeff(26)=-0.203774267068D+05
coeff(27)=-0.109856750064D+05
coeff(28)=-0.218575649186D+05
coeff(29)=-0.388794475986D+04
coeff(30)=-0.664560881872D+04
coeff(31)=0.103875992380D+05
coeff(32)=-0.761410788291D+04
coeff(33)=0.233045657571D+04
coeff(34)=-0.969339433218D+04
coeff(35)=-0.165481322351D+05
coeff(36)=0.854349804722D+04
coeff(37)=0.247970621031D+04
coeff(38)=-0.280118846613D+04
coeff(39)=0.115799405813D+04
coeff(40)=0.785819096606D+04
coeff(41)=-0.118294718031D+05
coeff(42)=-0.248800913717D+04
coeff(43)=-0.597880272425D+04
coeff(44)=0.506061830100D+03
coeff(45)=0.765991446455D+04
coeff(46)=-0.233552771757D+04
coeff(47)=-0.569240836104D+04
coeff(48)=0.308197467374D+04
coeff(49)=0.284250957199D+04
coeff(50)=-0.107820265240D+04
coeff(51)=-0.145499549445D+03
coeff(52)=-0.357028658899D+04
coeff(53)=-0.126803300125D+05
coeff(54)=-0.160409945363D+04
coeff(55)=-0.421315455714D+04
coeff(56)=0.731118479712D+04
coeff(57)=-0.317137601678D+04
coeff(58)=-0.774649845069D+04
coeff(59)=0.260717511909D+04
coeff(60)=-0.621274257657D+04
coeff(61)=0.417525751798D+04
coeff(62)=0.134588412182D+04
coeff(63)=0.836271486759D+03
coeff(64)=0.120988074835D+05
coeff(65)=-0.313375481450D+04
coeff(66)=0.112807269607D+05
coeff(67)=0.314490082818D+04
coeff(68)=-0.403245736345D+04
coeff(69)=-0.880257524573D+04
coeff(70)=0.164902202985D+05
coeff(71)=-0.108591722489D+05
coeff(72)=0.138703381122D+05
coeff(73)=0.725572955712D+04
coeff(74)=0.939990477056D+04
coeff(75)=-0.758802967075D+04
coeff(76)=-0.267397820247D+04
coeff(77)=0.122446503968D+05
coeff(78)=0.737507467719D+03
coeff(79)=-0.815875819889D+03
coeff(80)=0.327768006014D+04
coeff(81)=-0.467343182343D+04
coeff(82)=-0.151662106004D+05
coeff(83)=0.105237603572D+05
coeff(84)=0.419156522595D+04
coeff(85)=0.477503779515D+04
coeff(86)=-0.911620340692D+04
coeff(87)=0.335469575465D+04
coeff(88)=0.255102999820D+04
coeff(89)=-0.715020317241D+03
coeff(90)=0.984672028211D+04
coeff(91)=-0.892867071101D+03
coeff(92)=-0.255018622701D+04
coeff(93)=-0.504303336431D+04
coeff(94)=-0.430470260282D+04
coeff(95)=0.188859162278D+04
coeff(96)=0.165124499470D+04
coeff(97)=0.780627145376D+04
coeff(98)=0.498621141531D+04
coeff(99)=-0.141051561213D+05
coeff(100)=-0.605278631998D+04
coeff(101)=0.761997239316D+04
coeff(102)=0.280347105486D+04
coeff(103)=-0.137636364883D+04
coeff(104)=-0.237193599448D+04
coeff(105)=-0.383680506689D+04
coeff(106)=-0.450805757132D+04
coeff(107)=-0.194526481172D+05
coeff(108)=0.275470154762D+03
coeff(109)=0.104932067454D+05
coeff(110)=-0.327373650803D+04
coeff(111)=-0.203568159828D+04
coeff(112)=-0.162113695044D+05
coeff(113)=0.443497903714D+04
coeff(114)=0.853857219410D+04
coeff(115)=-0.536936779124D+04
coeff(116)=-0.906039972601D+03
coeff(117)=-0.909745633723D+04
coeff(118)=0.204780221141D+04
coeff(119)=0.702363629920D+04
coeff(120)=-0.341571710626D+04
coeff(121)=-0.150496955166D+04
coeff(122)=-0.520063526368D+04
coeff(123)=-0.659390204403D+04
coeff(124)=0.190233553161D+05
coeff(125)=0.513593060963D+04
coeff(126)=0.146882929229D+04
coeff(127)=-0.685132288287D+04
coeff(128)=0.516100037676D+04
coeff(129)=-0.144597490267D+05
coeff(130)=-0.239972949982D+04
coeff(131)=-0.378877930972D+04
coeff(132)=-0.140059251763D+05
coeff(133)=-0.197625033073D+04
coeff(134)=0.595289854817D+03
coeff(135)=0.118753545142D+05
coeff(136)=0.737261434489D+04
coeff(137)=-0.383501550068D+04
coeff(138)=-0.515056765371D+03
coeff(139)=0.613978132086D+04
coeff(140)=0.302217566551D+04
coeff(141)=-0.180353901441D+04
coeff(142)=0.391677387570D+04
coeff(143)=-0.130016437859D+05
coeff(144)=0.187795745055D+05
coeff(145)=-0.173983278076D+03
coeff(146)=0.534545489608D+04
coeff(147)=0.971405523781D+03
coeff(148)=0.961808007466D+04
coeff(149)=0.562998726186D+02
coeff(150)=-0.919348695306D+04
coeff(151)=0.775402621581D+03
coeff(152)=0.101763029192D+05
coeff(153)=-0.938143340451D+04
coeff(154)=0.111153066998D+05
coeff(155)=-0.859019540453D+04
coeff(156)=0.864659377827D+04
coeff(157)=0.519697639912D+04
coeff(158)=0.364547477110D+04
coeff(159)=-0.736997142548D+04
coeff(160)=-0.186749550910D+05
coeff(161)=0.604874456187D+04
coeff(162)=-0.122211291701D+04
coeff(163)=-0.294162327731D+04
coeff(164)=-0.420035686294D+04
coeff(165)=0.853923347403D+04
coeff(166)=0.143318037423D+05
coeff(167)=-0.630846023881D+04
coeff(168)=-0.146454200572D+04
coeff(169)=0.101762789563D+05
coeff(170)=0.503052453662D+04
coeff(171)=0.767854748445D+04
coeff(172)=0.142494227204D+05
coeff(173)=-0.245413684461D+04
coeff(174)=-0.584654180178D+04
coeff(175)=0.829590327047D+04
coeff(176)=0.189812133668D+03
coeff(177)=0.398468416352D+04
coeff(178)=0.352794964226D+04
coeff(179)=-0.608131465401D+04
coeff(180)=0.357999901038D+04
coeff(181)=0.154193271966D+05
coeff(182)=0.147485786525D+04
coeff(183)=-0.194342330463D+04
coeff(184)=0.109379776172D+05
coeff(185)=-0.956071244778D+04
coeff(186)=0.734653073497D+04
coeff(187)=0.778479814775D+04
coeff(188)=0.913886697071D+04
coeff(189)=0.403228995940D+04
coeff(190)=-0.805183798073D+03
coeff(191)=0.112306748428D+04
coeff(192)=0.126659387937D+05
coeff(193)=-0.394837666692D+04
coeff(194)=-0.918155239264D+04
coeff(195)=0.959268331348D+03
coeff(196)=-0.512891439022D+04
coeff(197)=0.123239481758D+04
coeff(198)=0.546870019149D+04
coeff(199)=-0.113262506270D+04
coeff(200)=-0.318635595355D+04
coeff(201)=-0.521357649783D+04
coeff(202)=0.105343013291D+04
coeff(203)=0.807469055972D+04
coeff(204)=0.204468287206D+04
coeff(205)=0.201179888348D+04
coeff(206)=-0.563075630944D+04
coeff(207)=0.993422804099D+04
coeff(208)=0.972283624671D+04
coeff(209)=-0.108760971468D+05
coeff(210)=0.335515602124D+04
coeff(211)=-0.809749502608D+04
coeff(212)=0.382575449336D+04
coeff(213)=0.498219476531D+04
coeff(214)=-0.286887359377D+04
coeff(215)=-0.647056923202D+04
coeff(216)=0.132645614561D+05
coeff(217)=0.274384061379D+03
coeff(218)=0.138539727442D+04
coeff(219)=0.854408729135D+04
coeff(220)=0.944467094366D+03
coeff(221)=0.523899770841D+04
coeff(222)=0.110304820651D+04
coeff(223)=0.262893012816D+04
coeff(224)=-0.281776957112D+04
coeff(225)=0.693524442480D+03
coeff(226)=0.975786820914D+04
coeff(227)=-0.107373990553D+04
coeff(228)=-0.325008895275D+03
coeff(229)=0.353057819245D+03
coeff(230)=-0.877105385411D+04
coeff(231)=-0.167448772530D+04
coeff(232)=0.507717252429D+04
coeff(233)=0.302903907542D+04
coeff(234)=-0.716758184260D+04
coeff(235)=0.731146873877D+04
coeff(236)=-0.114378704009D+05
coeff(237)=-0.366675994623D+04
coeff(238)=-0.700762080242D+03
coeff(239)=-0.512766931270D+04
coeff(240)=0.143349823624D+05
coeff(241)=0.482038293609D+04
coeff(242)=0.117445479454D+05
coeff(243)=0.762307957193D+04
coeff(244)=-0.163041402741D+04
coeff(245)=-0.792387654616D+04
coeff(246)=0.766436505269D+04
coeff(247)=-0.310264525878D+04
coeff(248)=-0.711880688064D+04
coeff(249)=0.127519452818D+05
coeff(250)=-0.609157218671D+04
coeff(251)=0.288296346959D+04
coeff(252)=0.353870882029D+04
coeff(253)=0.239465577481D+04
coeff(254)=-0.599198703187D+04
coeff(255)=0.980562589901D+04
coeff(256)=-0.788346942204D+04
coeff(257)=-0.989760604999D+03
coeff(258)=-0.479098231560D+04
coeff(259)=-0.115115742444D+05
coeff(260)=-0.450602097318D+04
coeff(261)=-0.823284348312D+04
coeff(262)=0.661736080389D+04
coeff(263)=-0.462350424825D+04
coeff(264)=0.426626534236D+04
coeff(265)=-0.193727059203D+04
coeff(266)=0.611367734036D+04
coeff(267)=-0.210337937126D+05
coeff(268)=0.233197669025D+04
coeff(269)=-0.512594276958D+04
coeff(270)=0.103077657483D+05
coeff(271)=-0.349020967464D+04
coeff(272)=-0.515467907460D+04
coeff(273)=0.643448729535D+04
coeff(274)=0.350564643192D+03
coeff(275)=-0.354583019920D+04
coeff(276)=0.968468494493D+04
coeff(277)=-0.147158649648D+05
coeff(278)=0.122244371810D+05
coeff(279)=-0.751806407624D+04
coeff(280)=-0.132770738935D+05
coeff(281)=0.279897004632D+04
coeff(282)=0.811655078888D+03
coeff(283)=0.199110954262D+04
coeff(284)=0.700983188732D+04
coeff(285)=-0.148646086819D+04
coeff(286)=0.201612968054D+03
coeff(287)=-0.402239656004D+04
coeff(288)=0.521055351892D+04
coeff(289)=-0.998591067508D+04
coeff(290)=-0.261661062308D+04
coeff(291)=-0.273583691114D+04
coeff(292)=-0.118677984602D+05
coeff(293)=0.681878690751D+04
coeff(294)=-0.338569792706D+04
coeff(295)=-0.997190806337D+04
coeff(296)=0.505622827380D+04
coeff(297)=-0.317741386504D+04
coeff(298)=-0.512155805627D+04
coeff(299)=-0.134428199634D+04
coeff(300)=-0.742900669302D+04
coeff(301)=0.298953618374D+04
coeff(302)=0.149231837929D+05
coeff(303)=-0.233911020036D+03
coeff(304)=-0.517390442191D+04
coeff(305)=-0.782989525052D+04
coeff(306)=-0.389897148069D+04
coeff(307)=0.664908651747D+04
coeff(308)=0.962263884894D+03
coeff(309)=-0.897175125164D+02
coeff(310)=0.357462552986D+04
coeff(311)=-0.447139906049D+04
coeff(312)=0.284331721480D+04
coeff(313)=-0.677798944877D+04
coeff(314)=-0.687688539579D+04
coeff(315)=0.130241750841D+04
coeff(316)=0.361377810274D+04
coeff(317)=-0.480299659088D+04
coeff(318)=0.109179525937D+05
coeff(319)=0.246017681909D+04
coeff(320)=0.916447237102D+03
coeff(321)=-0.257316756335D+04
coeff(322)=0.478674236090D+04
coeff(323)=-0.109286096162D+05
coeff(324)=-0.272505689285D+04
coeff(325)=-0.511388658005D+04
coeff(326)=-0.253900577805D+03
coeff(327)=0.202581822955D+04
coeff(328)=-0.999071883011D+04
coeff(329)=-0.345218669818D+04
coeff(330)=-0.899687803036D+04
coeff(331)=-0.257713120088D+04
coeff(332)=-0.242611355131D+04
coeff(333)=-0.117068471402D+05
coeff(334)=-0.522917759287D+04
coeff(335)=-0.101430963680D+05
coeff(336)=-0.120340967998D+05
coeff(337)=0.797871594413D+04
coeff(338)=-0.805163495681D+03
coeff(339)=0.919341390953D+03
coeff(340)=-0.722836796153D+03
coeff(341)=-0.676878142225D+04
coeff(342)=0.809707050944D+04
coeff(343)=0.112831197730D+05
coeff(344)=0.444642874944D+04
coeff(345)=0.128699536156D+05
coeff(346)=0.129523290751D+05
coeff(347)=-0.560100231061D+04
coeff(348)=-0.124456726326D+04
coeff(349)=0.433274775624D+04
coeff(350)=-0.990731876641D+04
coeff(351)=0.211649627502D+04
coeff(352)=0.532088715707D+04
coeff(353)=0.161638275567D+05
coeff(354)=-0.592437862482D+04
coeff(355)=0.117294519059D+05
coeff(356)=0.563162655939D+04
coeff(357)=-0.626571156027D+04
coeff(358)=0.756729961437D+04
coeff(359)=0.302944987400D+04
coeff(360)=-0.598641762471D+04
coeff(361)=-0.336940886740D+04
coeff(362)=-0.175678062791D+04
coeff(363)=0.104097521108D+05
coeff(364)=-0.105028016226D+05
coeff(365)=-0.562030700352D+04
coeff(366)=0.483301642983D+03
coeff(367)=-0.231277484934D+04
coeff(368)=-0.162914199738D+05
coeff(369)=-0.102368762949D+05
coeff(370)=0.451825924198D+02
coeff(371)=-0.356146060236D+04
coeff(372)=-0.175392121722D+05
coeff(373)=-0.414356805773D+04
coeff(374)=-0.683057972086D+04
coeff(375)=0.670551095326D+04
coeff(376)=0.671088707765D+04
coeff(377)=-0.587832922336D+04
coeff(378)=-0.337700746041D+03
coeff(379)=-0.576758487790D+03
coeff(380)=-0.926220324770D+03
coeff(381)=0.398261359624D+04
coeff(382)=0.421184646603D+04
coeff(383)=-0.732451624504D+03
coeff(384)=0.143273564265D+04
coeff(385)=-0.366144408329D+04
coeff(386)=-0.750412054506D+04
coeff(387)=0.974836632974D+04
coeff(388)=0.196301524168D+04
coeff(389)=0.119738252032D+04
coeff(390)=0.100230986023D+04
coeff(391)=0.585790718458D+04
coeff(392)=-0.608963257368D+04
coeff(393)=-0.326989505978D+04
coeff(394)=-0.110762507822D+04
coeff(395)=-0.499160128333D+04
coeff(396)=-0.592720733518D+04
coeff(397)=0.407240484572D+04
coeff(398)=-0.247703236136D+04
coeff(399)=-0.115930360584D+04
coeff(400)=-0.304302744949D+04
coeff(401)=-0.388433867384D+04
coeff(402)=-0.739084982578D+04
coeff(403)=0.405396836089D+04
coeff(404)=0.267646319316D+04
coeff(405)=0.101732419554D+04
coeff(406)=0.502448975733D+03
coeff(407)=0.155376199527D+04
coeff(408)=-0.709808299765D+03
coeff(409)=-0.433202946484D+04
coeff(410)=-0.201983322845D+04
coeff(411)=0.922734039206D+04
coeff(412)=0.551356249867D+04
coeff(413)=0.431100441562D+04
coeff(414)=0.207853237660D+03
coeff(415)=0.129796561587D+05
coeff(416)=0.467983920689D+04
coeff(417)=-0.110218743557D+05
coeff(418)=-0.502742072962D+04
coeff(419)=0.270082623898D+03
coeff(420)=-0.367446228496D+04
coeff(421)=-0.130057796112D+03
coeff(422)=0.200889817518D+04
coeff(423)=0.290172171224D+04
coeff(424)=0.293313754024D+03
coeff(425)=-0.171323631243D+04
coeff(426)=-0.776801715553D+03
coeff(427)=0.733041585996D+04
coeff(428)=0.838929542211D+04
coeff(429)=-0.200309191247D+04
coeff(430)=0.238983334747D+04
coeff(431)=-0.339781451348D+04
coeff(432)=0.964749812762D+04
coeff(433)=-0.183736210552D+04
coeff(434)=0.330714496319D+03
coeff(435)=-0.205635507839D+04
coeff(436)=0.253785601297D+04
coeff(437)=0.459470517014D+04
coeff(438)=0.660129764918D+04
coeff(439)=-0.124229843649D+05
coeff(440)=-0.575258441894D+04
coeff(441)=0.532430252073D+04
coeff(442)=0.224990849360D+04
coeff(443)=0.223436889489D+04
coeff(444)=0.288674878373D+04
coeff(445)=-0.104915136970D+05
coeff(446)=0.148757110095D+04
coeff(447)=0.102699638406D+05
coeff(448)=0.549168456821D+04
coeff(449)=-0.647189634920D+03
coeff(450)=0.362898030070D+04
coeff(451)=-0.214088218259D+04
coeff(452)=-0.667611271350D+03
coeff(453)=-0.316269938391D+04
coeff(454)=-0.290319253802D+04
coeff(455)=0.257215621701D+04
coeff(456)=-0.105866947931D+04
coeff(457)=0.509479175366D+04
coeff(458)=-0.461279250104D+04
coeff(459)=-0.128242881766D+05
coeff(460)=-0.566220020098D+04
coeff(461)=-0.331279158431D+01
coeff(462)=0.612254366257D+03
coeff(463)=-0.874791188626D+04
coeff(464)=0.148917843455D+05
coeff(465)=0.490197704016D+04
coeff(466)=0.337331505875D+04
coeff(467)=-0.105164507704D+04
coeff(468)=0.119857594397D+04
coeff(469)=-0.232591257100D+04
coeff(470)=-0.412932140800D+04
coeff(471)=-0.302829629023D+04
coeff(472)=-0.113353147048D+04
coeff(473)=0.438548675926D+04
coeff(474)=-0.101004632549D+05
coeff(475)=0.579114552638D+04
coeff(476)=-0.558979869855D+04
coeff(477)=0.151789890031D+04
coeff(478)=-0.338329090169D+04
coeff(479)=0.471660824188D+04
coeff(480)=0.451144379669D+04
coeff(481)=-0.678133193575D+03
coeff(482)=0.118592896124D+05
coeff(483)=0.393715456885D+03
coeff(484)=-0.152043773399D+04
coeff(485)=-0.570861371708D+04
coeff(486)=-0.198810947248D+05
coeff(487)=-0.803695508444D+04
coeff(488)=-0.654189820620D+04
coeff(489)=0.590843642492D+04
coeff(490)=0.576597386026D+04
coeff(491)=0.114816712643D+05
coeff(492)=0.769982254991D+04
coeff(493)=-0.277936615940D+04
coeff(494)=-0.124830744389D+05
coeff(495)=0.453876625825D+04
coeff(496)=0.542726355444D+04
coeff(497)=0.422090808507D+04
coeff(498)=-0.392628135266D+04
coeff(499)=0.118495901402D+05
coeff(500)=-0.237800947728D+04
coeff(501)=-0.181247002008D+04
coeff(502)=-0.222779795824D+04
coeff(503)=-0.173818349601D+05
coeff(504)=0.102380673084D+05
coeff(505)=0.294369683033D+04
coeff(506)=-0.468465416519D+04
coeff(507)=-0.128971184195D+05
coeff(508)=-0.424423277013D+04
coeff(509)=0.580908049901D+04
coeff(510)=0.467392765529D+04
coeff(511)=-0.528946392911D+04
coeff(512)=0.104277939188D+05
coeff(513)=0.492163203011D+04
coeff(514)=0.932920759980D+03
coeff(515)=-0.428709741255D+04
coeff(516)=-0.131442674447D+04
coeff(517)=-0.612008258673D+04
coeff(518)=-0.131694669104D+04
coeff(519)=0.372309717482D+04
coeff(520)=-0.101316688762D+05
coeff(521)=0.507030781290D+02
coeff(522)=0.399747171413D+04
coeff(523)=-0.643512311396D+04
coeff(524)=0.701328620569D+04
coeff(525)=-0.257165943583D+04
coeff(526)=0.394214504519D+04
coeff(527)=-0.512644987871D+04
coeff(528)=-0.151669392543D+04
coeff(529)=-0.928199023693D+04
coeff(530)=-0.967616519684D+04
coeff(531)=-0.976423594965D+04
coeff(532)=-0.144853513306D+04
coeff(533)=0.435665117729D+04
coeff(534)=0.103739792126D+05
coeff(535)=-0.665900804985D+04
coeff(536)=0.152974198010D+04
coeff(537)=0.614757113374D+03
coeff(538)=-0.165521447637D+04
coeff(539)=-0.112376976136D+05
coeff(540)=0.925226331746D+04
coeff(541)=-0.708149528119D+03
coeff(542)=-0.677088157097D+04
coeff(543)=0.279219344633D+04
coeff(544)=0.302706649909D+04
coeff(545)=-0.219677382838D+04
coeff(546)=-0.567867992993D+04
coeff(547)=-0.420975882524D+04
coeff(548)=-0.549180100037D+04
coeff(549)=0.653544986866D+04
coeff(550)=0.598679514268D+04
coeff(551)=-0.525759621400D+04
coeff(552)=0.706517086891D+03
coeff(553)=-0.988196250857D+04
coeff(554)=-0.185055479924D+04
coeff(555)=0.123294057187D+04
coeff(556)=0.450956685565D+04
coeff(557)=0.797056386913D+04
coeff(558)=0.589936597868D+04
coeff(559)=0.477800469641D+04
coeff(560)=0.138087699407D+04
coeff(561)=0.413516367129D+04
coeff(562)=-0.640119118088D+04
coeff(563)=-0.121206003670D+05
coeff(564)=0.942841494720D+04
coeff(565)=-0.577873636759D+04
coeff(566)=0.497524224451D+04
coeff(567)=0.115198049742D+05
coeff(568)=-0.355422439403D+04
coeff(569)=-0.367575850060D+04
coeff(570)=-0.348063662876D+04
coeff(571)=0.120470770684D+04
coeff(572)=0.226441501546D+04
coeff(573)=0.109956248325D+05
coeff(574)=0.922803849525D+03
coeff(575)=0.112058463549D+05
coeff(576)=-0.346647008757D+04
coeff(577)=0.858701872055D+04
coeff(578)=-0.305675760534D+04
coeff(579)=0.131329175686D+04
coeff(580)=-0.209610306936D+04
coeff(581)=-0.114320779566D+04
coeff(582)=-0.316039764821D+04
coeff(583)=-0.345488334363D+04
coeff(584)=0.372965981378D+04
coeff(585)=0.194375710769D+03
coeff(586)=-0.938648081976D+04
coeff(587)=-0.110046252845D+04
coeff(588)=0.680123163162D+04
coeff(589)=-0.466715696586D+04
coeff(590)=0.514920578524D+03
coeff(591)=-0.131286469326D+05
coeff(592)=-0.732072075647D+04
coeff(593)=0.532228831528D+04
coeff(594)=-0.115865083722D+05
coeff(595)=0.149257856174D+05
coeff(596)=-0.168098780114D+04
coeff(597)=-0.285540910826D+04
coeff(598)=0.967477126912D+04
coeff(599)=-0.780557338642D+04
coeff(600)=-0.704596757552D+04
coeff(601)=-0.435394178429D+04
coeff(602)=-0.501707709058D+04
coeff(603)=0.520422033787D+04
coeff(604)=-0.279711713756D+04
coeff(605)=0.474723241559D+04
coeff(606)=-0.738631755362D+04
coeff(607)=-0.710607351193D+04
coeff(608)=0.523688348644D+02
coeff(609)=-0.776016679427D+04
coeff(610)=0.150002470716D+05
coeff(611)=-0.117592992935D+04
coeff(612)=0.105207717662D+05
coeff(613)=0.690962142928D+04
coeff(614)=-0.855070363181D+03
coeff(615)=-0.868525832927D+04
coeff(616)=-0.227077940298D+04
coeff(617)=-0.964318129685D+03
coeff(618)=0.217950179152D+04
coeff(619)=-0.292503470493D+04
coeff(620)=0.211018083344D+04
coeff(621)=-0.841595302876D+04
coeff(622)=0.309512354726D+04
coeff(623)=0.164085101992D+04
coeff(624)=-0.854892896641D+04
coeff(625)=-0.757194934732D+04
coeff(626)=-0.705003546728D+04
coeff(627)=-0.173052182925D+04
coeff(628)=-0.483143667115D+04
coeff(629)=0.412345813097D+04
coeff(630)=-0.135170942746D+05
coeff(631)=-0.465241394158D+04
coeff(632)=0.227143813542D+04
coeff(633)=-0.429355370090D+04
coeff(634)=-0.694983042812D+04
coeff(635)=-0.245944490349D+04
coeff(636)=0.201940908827D+04
coeff(637)=-0.101904241516D+05
coeff(638)=-0.671821825304D+04
coeff(639)=0.546470606239D+03
coeff(640)=-0.150665026928D+04
coeff(641)=0.284526096075D+04
coeff(642)=0.123452239640D+05
coeff(643)=-0.789576144942D+04
coeff(644)=0.298957260681D+04
coeff(645)=0.280624826922D+04
coeff(646)=-0.455210513691D+04
coeff(647)=-0.112831260688D+05
coeff(648)=-0.782431991110D+04
coeff(649)=-0.415361521807D+03
coeff(650)=-0.210020069400D+05
coeff(651)=0.306221481131D+04
coeff(652)=-0.264961665318D+04
coeff(653)=-0.206484592671D+04
coeff(654)=-0.145866099583D+04
coeff(655)=0.337663958149D+04
coeff(656)=-0.285622754553D+04
coeff(657)=-0.979522176450D+04
coeff(658)=-0.684045213335D+04
coeff(659)=-0.424431889976D+04
coeff(660)=-0.248743259015D+04
coeff(661)=0.393392804008D+04
coeff(662)=-0.458953182906D+04
coeff(663)=0.897590627988D+04
coeff(664)=0.701358365662D+04
coeff(665)=-0.101741558507D+05
coeff(666)=0.107258348853D+05
coeff(667)=-0.435734514059D+04
coeff(668)=0.423416421155D+04
coeff(669)=0.277534935866D+04
coeff(670)=0.305054035920D+04
coeff(671)=0.789775758618D+04
coeff(672)=-0.280322896847D+04
coeff(673)=-0.427435790968D+03
coeff(674)=-0.736912469447D+04
coeff(675)=-0.376016705886D+04
coeff(676)=-0.115494243755D+05
coeff(677)=0.106358111412D+04
coeff(678)=0.322092028883D+04
coeff(679)=0.679247575058D+04
coeff(680)=-0.404538814727D+04
coeff(681)=0.726425721164D+04
coeff(682)=-0.389729932035D+04
coeff(683)=0.340943553541D+04
coeff(684)=0.274535957698D+04
coeff(685)=0.223104237153D+04
coeff(686)=0.136958311923D+04
coeff(687)=-0.743951552125D+04
coeff(688)=-0.338294752725D+04
coeff(689)=-0.164006003801D+04
coeff(690)=-0.125523391888D+05
coeff(691)=-0.651942488718D+04
coeff(692)=0.191514434169D+04
coeff(693)=0.100655473954D+04
coeff(694)=-0.553574236074D+04
coeff(695)=0.634906205737D+03
coeff(696)=0.276101755052D+04
coeff(697)=-0.350572747978D+04
coeff(698)=-0.407322667203D+04
coeff(699)=-0.589440744773D+04
coeff(700)=0.334071490563D+04
coeff(701)=0.130995782753D+05
coeff(702)=-0.976847566923D+04
coeff(703)=-0.264466549768D+04
coeff(704)=0.250988550656D+04
coeff(705)=-0.700006659584D+03
coeff(706)=-0.216556200451D+04
coeff(707)=0.247551845467D+04
coeff(708)=0.168641504471D+04
coeff(709)=-0.258323769273D+04
coeff(710)=-0.541143717417D+04
coeff(711)=0.728585492875D+04
coeff(712)=0.427386177732D+04
coeff(713)=-0.106750896803D+05
coeff(714)=-0.142850613283D+03
coeff(715)=-0.263529188273D+04
coeff(716)=0.377556606652D+04
coeff(717)=-0.982795811636D+04
coeff(718)=-0.824475921561D+04
coeff(719)=0.137051468704D+04
coeff(720)=-0.370986713729D+04
coeff(721)=-0.352036254891D+04
coeff(722)=-0.975808537844D+04
coeff(723)=0.651740040158D+03
coeff(724)=-0.153690088291D+04
coeff(725)=-0.973893745856D+02
coeff(726)=-0.553852549037D+04
coeff(727)=-0.530614463678D+04
coeff(728)=-0.748335532491D+03
coeff(729)=0.884311575859D+04
coeff(730)=-0.297126532127D+04
coeff(731)=0.206537876659D+04
coeff(732)=-0.954938796958D+04
coeff(733)=0.809069493124D+04
coeff(734)=-0.558692747128D+04
coeff(735)=-0.656526463008D+04
coeff(736)=0.870198811183D+04
coeff(737)=-0.681130777785D+04
coeff(738)=0.307973525122D+04
coeff(739)=-0.356735711676D+04
coeff(740)=0.872597305909D+03
coeff(741)=-0.980942837246D+04
coeff(742)=-0.278677588989D+04
coeff(743)=0.754758925814D+04
coeff(744)=-0.509299977146D+02
coeff(745)=0.462727427190D+04
coeff(746)=0.479052206231D+04
coeff(747)=-0.415297816338D+04
coeff(748)=-0.156715750477D+05
coeff(749)=-0.814579575814D+04
coeff(750)=0.695600072644D+04
coeff(751)=-0.359110735359D+04
coeff(752)=0.335327469921D+04
coeff(753)=-0.509235080128D+03
coeff(754)=0.278037079199D+04
coeff(755)=0.775923802367D+04
coeff(756)=0.756927318692D+04
coeff(757)=-0.263167803815D+03
coeff(758)=0.194424717176D+04
coeff(759)=-0.288987261312D+04
coeff(760)=0.587531319676D+04
coeff(761)=0.430629444306D+04
coeff(762)=0.271480207423D+04
coeff(763)=0.639212296754D+04
coeff(764)=0.792432310989D+04
coeff(765)=-0.195295765085D+04
coeff(766)=-0.128242786323D+04
coeff(767)=0.104128368367D+05
coeff(768)=-0.876075324104D+04
coeff(769)=-0.519989686458D+04
coeff(770)=-0.546525845897D+04
coeff(771)=-0.130127851572D+05
coeff(772)=-0.950320737179D+04
coeff(773)=-0.464012650897D+04
coeff(774)=0.981851082986D+04
coeff(775)=-0.174579524396D+04
coeff(776)=-0.268634700455D+04
coeff(777)=0.395257784460D+03
coeff(778)=-0.332217772151D+04
coeff(779)=0.504606163046D+04
coeff(780)=-0.358541379303D+04
coeff(781)=-0.303273652245D+04
coeff(782)=-0.129612512931D+05
coeff(783)=0.221140070262D+04
coeff(784)=0.951033860475D+04
coeff(785)=-0.193913189444D+04
coeff(786)=0.204103757226D+04
coeff(787)=-0.628230770108D+04
coeff(788)=0.743920510754D+04
coeff(789)=0.819257874701D+04
coeff(790)=-0.530538497886D+04
coeff(791)=0.298161364374D+03
coeff(792)=-0.784128312893D+04
coeff(793)=-0.267205865318D+04
coeff(794)=-0.193359211020D+04
coeff(795)=0.396957577886D+04
coeff(796)=0.660668188339D+03
coeff(797)=-0.126215470138D+03
coeff(798)=0.824460442796D+04
coeff(799)=0.225315195262D+04
coeff(800)=0.379367134428D+04
coeff(801)=0.290426328497D+04
coeff(802)=0.141520734054D+04
coeff(803)=0.775642917664D+04
coeff(804)=0.229450887198D+04
coeff(805)=-0.133899618737D+04
coeff(806)=0.446680934548D+04
coeff(807)=-0.374127162031D+04
coeff(808)=0.106633827274D+05
coeff(809)=0.262002534000D+04
coeff(810)=-0.452863155011D+04
coeff(811)=0.295506486283D+04
coeff(812)=-0.638555874579D+03
coeff(813)=-0.272546059283D+04
coeff(814)=-0.540752896847D+04
coeff(815)=-0.585061615308D+04
coeff(816)=0.922095979244D+04
coeff(817)=0.147617815370D+05
coeff(818)=0.101034239835D+04
coeff(819)=0.947685255929D+04
coeff(820)=0.801541223486D+04
coeff(821)=-0.647989099176D+04
coeff(822)=-0.212350164140D+04
coeff(823)=-0.272176498744D+04
coeff(824)=0.300099934399D+04
coeff(825)=0.138824270864D+05
coeff(826)=0.267061800049D+04
coeff(827)=0.111806789711D+04
coeff(828)=0.102915461171D+05
coeff(829)=0.219579973505D+04
coeff(830)=-0.459472167798D+04
coeff(831)=-0.188228462317D+03
coeff(832)=0.513279919919D+04
coeff(833)=-0.301275224088D+04
coeff(834)=-0.664504888529D+04
coeff(835)=-0.610197374185D+04
coeff(836)=-0.286121282547D+04
coeff(837)=-0.178115607597D+04
coeff(838)=0.102791973680D+05
coeff(839)=0.112713507198D+05
coeff(840)=0.584457322183D+04
coeff(841)=0.119771166267D+05
coeff(842)=-0.118503492995D+05
coeff(843)=-0.694422566207D+04
coeff(844)=0.217492887291D+04
coeff(845)=0.129004024373D+05
coeff(846)=-0.222169950982D+04
coeff(847)=0.788680835146D+04
coeff(848)=0.997298265230D+04
coeff(849)=0.379079569071D+04
coeff(850)=0.817122445064D+04
coeff(851)=0.397323039823D+04
coeff(852)=0.713483101991D+04
coeff(853)=-0.216934070888D+04
coeff(854)=0.653974256115D+04
coeff(855)=-0.609788063551D+04
coeff(856)=0.381003601879D+04
coeff(857)=-0.700317672785D+04
coeff(858)=0.998756039552D+04
coeff(859)=-0.831236762894D+04
coeff(860)=0.792532580514D+04
coeff(861)=0.382321349022D+04
coeff(862)=-0.679004254383D+04
coeff(863)=0.285663132035D+03
coeff(864)=0.822658893877D+04
coeff(865)=0.102424775348D+05
coeff(866)=0.159884945353D+04
coeff(867)=0.204434946734D+03
coeff(868)=-0.533266287986D+03
coeff(869)=0.320950003907D+04
coeff(870)=0.280716423750D+04
coeff(871)=0.471839691400D+04
coeff(872)=0.889972710163D+04
coeff(873)=-0.265790643956D+04
coeff(874)=-0.202664584324D+04
coeff(875)=-0.569547987307D+04
coeff(876)=0.671767707614D+03
coeff(877)=0.869460935812D+04
coeff(878)=0.988720916354D+04
coeff(879)=-0.601472587480D+04
coeff(880)=0.349204404453D+03
coeff(881)=-0.753548693973D+02
coeff(882)=0.308469046149D+04
coeff(883)=0.291910720061D+04
coeff(884)=-0.630352947539D+03
coeff(885)=0.488293530602D+04
coeff(886)=-0.353068417057D+04
coeff(887)=0.590739329596D+04
coeff(888)=0.478991817466D+04
coeff(889)=0.992129875234D+03
coeff(890)=-0.156803711161D+04
coeff(891)=-0.143442721054D+04
coeff(892)=-0.242352395278D+04
coeff(893)=0.562322089421D+04
coeff(894)=-0.883899856622D+04
coeff(895)=-0.695702076187D+04
coeff(896)=-0.512381134672D+04
coeff(897)=-0.147367294571D+04
coeff(898)=0.551307289832D+04
coeff(899)=0.786679294680D+03
coeff(900)=0.315247322067D+04
coeff(901)=0.130227759863D+05
coeff(902)=0.114358171410D+05
coeff(903)=0.500211442360D+04
coeff(904)=-0.910372641427D+04
coeff(905)=0.182130394194D+04
coeff(906)=-0.241587256617D+04
coeff(907)=0.168808754538D+04
coeff(908)=0.579441031766D+04
coeff(909)=0.867445737007D+04
coeff(910)=-0.707632615879D+04
coeff(911)=0.227270158206D+05
coeff(912)=-0.541542133598D+04
coeff(913)=-0.733168219317D+04
coeff(914)=-0.591872489067D+04
coeff(915)=0.252010526535D+04
coeff(916)=-0.209300291968D+04
coeff(917)=0.407845210260D+04
coeff(918)=0.419643151333D+04
coeff(919)=-0.325852982982D+04
coeff(920)=0.510685250321D+04
coeff(921)=0.518245440335D+04
coeff(922)=-0.570021026484D+04
coeff(923)=0.379948451007D+04
coeff(924)=-0.442249273133D+04
coeff(925)=0.580855730437D+04
coeff(926)=0.502010690985D+04
coeff(927)=-0.305050758201D+04
coeff(928)=0.867991295795D+04
coeff(929)=0.559326365983D+04
coeff(930)=-0.500898977298D+04
coeff(931)=-0.793557040479D+04
coeff(932)=0.404586797503D+04
coeff(933)=0.142150513650D+04
coeff(934)=0.429711236763D+04
coeff(935)=0.526297596091D+04
coeff(936)=-0.949943573404D+04
coeff(937)=-0.205519313087D+04
coeff(938)=0.753542910706D+04
coeff(939)=0.574409858897D+04
coeff(940)=-0.955986352121D+04
coeff(941)=0.456488464513D+04
coeff(942)=-0.210564784285D+04
coeff(943)=0.257422574710D+03
coeff(944)=-0.106905890325D+05
coeff(945)=0.308842427905D+04
coeff(946)=0.833984940358D+04
coeff(947)=0.912028273264D+04
coeff(948)=-0.810555848269D+04
coeff(949)=-0.483269815993D+04
coeff(950)=0.574568456115D+04
coeff(951)=0.141517702335D+05
coeff(952)=0.703444990347D+04
coeff(953)=-0.287568821299D+04
coeff(954)=0.804698210028D+04
coeff(955)=-0.133199111567D+05
coeff(956)=0.249776018972D+04
coeff(957)=0.569737767322D+04
coeff(958)=0.437450130895D+04
coeff(959)=0.495175432194D+04
coeff(960)=-0.720343229582D+04
coeff(961)=-0.126998562278D+04
coeff(962)=0.123777503315D+04
coeff(963)=0.538968319814D+04
coeff(964)=0.767485674359D+04
coeff(965)=0.265035979842D+03
coeff(966)=0.167772353047D+04
coeff(967)=0.524454681460D+04
coeff(968)=-0.784295447690D+04
coeff(969)=-0.189515469697D+04
coeff(970)=0.717761674230D+04
coeff(971)=-0.360126588367D+04
coeff(972)=0.102846102590D+04
coeff(973)=0.127775215098D+05
coeff(974)=-0.605513000715D+04
coeff(975)=-0.111285869821D+05
coeff(976)=0.381051647050D+04
coeff(977)=-0.352072991933D+04
coeff(978)=0.355751511875D+04
coeff(979)=0.603107269606D+04
coeff(980)=0.127851783080D+05
coeff(981)=-0.617125995335D+04
coeff(982)=0.442474301313D+04
coeff(983)=-0.245973399467D+04
coeff(984)=0.512752527447D+04
coeff(985)=0.117309897192D+04
coeff(986)=-0.133697598331D+05
coeff(987)=-0.667031465200D+03
coeff(988)=-0.373520290201D+04
coeff(989)=0.178060872301D+04
coeff(990)=-0.694635900645D+04
coeff(991)=0.939363808099D+03
coeff(992)=0.629577778806D+03
coeff(993)=0.516056411738D+04
coeff(994)=0.717701575839D+04
coeff(995)=0.100593340573D+05
coeff(996)=0.230626260414D+04
coeff(997)=-0.957130532753D+04
coeff(998)=-0.285005330782D+04
coeff(999)=0.124493495290D+05
coeff(1000)=0.945064236383D+03
coeff(1001)=-0.578672574022D+04
coeff(1002)=-0.427118594762D+04
coeff(1003)=0.391852195358D+04
coeff(1004)=-0.813363423605D+03
coeff(1005)=0.318014535478D+04
coeff(1006)=-0.374719008288D+04
coeff(1007)=0.819384594366D+04
coeff(1008)=-0.189522845089D+04
coeff(1009)=0.576025440810D+04
coeff(1010)=0.192212695453D+05
coeff(1011)=0.543469609604D+04
coeff(1012)=0.580127298769D+04
coeff(1013)=0.489374093904D+04
coeff(1014)=-0.376111987565D+04
coeff(1015)=0.461687584317D+04
coeff(1016)=0.328803103839D+04
coeff(1017)=-0.229968138850D+04
coeff(1018)=-0.620419881856D+04
coeff(1019)=-0.525652997779D+03
coeff(1020)=0.327227587872D+04
coeff(1021)=0.253707565470D+04
coeff(1022)=-0.334928172243D+04
coeff(1023)=0.782625390101D+04
coeff(1024)=-0.985881222838D+04
coeff(1025)=0.204081846270D+04
coeff(1026)=0.453389975426D+04
coeff(1027)=-0.456626543357D+04
coeff(1028)=0.189409092297D+04
coeff(1029)=0.314088410647D+04
coeff(1030)=0.610718464025D+03
coeff(1031)=-0.723017576220D+04
coeff(1032)=0.623727629909D+04
coeff(1033)=0.335384541471D+03
coeff(1034)=0.105631482287D+05
coeff(1035)=0.247765690656D+04
coeff(1036)=-0.169512919708D+04
coeff(1037)=0.359349838163D+04
coeff(1038)=0.176925097854D+04
coeff(1039)=0.310667250563D+04
coeff(1040)=-0.770660066194D+03
coeff(1041)=0.958602908404D+03
coeff(1042)=0.345226644350D+04
coeff(1043)=-0.879995734430D+03
coeff(1044)=0.845929756141D+04
coeff(1045)=-0.194938793075D+04
coeff(1046)=0.893069982656D+04
coeff(1047)=0.828106019934D+04
coeff(1048)=-0.150641530452D+04
coeff(1049)=0.151264831575D+04
coeff(1050)=-0.915136150947D+04
coeff(1051)=0.958415279892D+04
coeff(1052)=0.147249402109D+03
coeff(1053)=-0.213900662337D+04
coeff(1054)=0.358524977064D+04
coeff(1055)=0.187355180411D+04
coeff(1056)=-0.353365883952D+04
coeff(1057)=-0.627078890138D+04
coeff(1058)=-0.204378301640D+03
coeff(1059)=-0.244674184190D+04
coeff(1060)=0.798174181138D+04
coeff(1061)=0.572830176836D+03
coeff(1062)=-0.617746892913D+04
coeff(1063)=-0.821081693071D+04
coeff(1064)=-0.451424190069D+02
coeff(1065)=0.353317610779D+04
coeff(1066)=0.300629956925D+04
coeff(1067)=0.790675725632D+04
coeff(1068)=-0.239099974580D+04
coeff(1069)=-0.116354531758D+04
coeff(1070)=0.453215588238D+03
coeff(1071)=-0.116468864211D+05
coeff(1072)=-0.528922835154D+04
coeff(1073)=-0.387580845216D+04
coeff(1074)=0.182527233118D+04
coeff(1075)=-0.496405443691D+04
coeff(1076)=-0.486956843070D+04
coeff(1077)=0.260940959538D+02
coeff(1078)=0.136800422967D+05
coeff(1079)=-0.161162497257D+04
coeff(1080)=0.357442860490D+03
coeff(1081)=-0.142142608472D+04
coeff(1082)=-0.132690842410D+04
coeff(1083)=0.815454678979D+04
coeff(1084)=-0.396476548147D+04
coeff(1085)=-0.805323996033D+04
coeff(1086)=-0.224315565296D+04
coeff(1087)=0.391939852726D+04
coeff(1088)=0.720122787602D+04
coeff(1089)=0.356176305339D+03
coeff(1090)=-0.895627069821D+04
coeff(1091)=-0.952440035330D+04
coeff(1092)=0.755570074290D+04
coeff(1093)=-0.736686400686D+04
coeff(1094)=-0.337351338527D+04
coeff(1095)=-0.107163369312D+05
coeff(1096)=-0.945343401525D+04
coeff(1097)=0.121699288951D+04
coeff(1098)=0.771428346266D+03
coeff(1099)=-0.549005135830D+04
coeff(1100)=-0.607655644706D+04
coeff(1101)=0.510264050014D+04
coeff(1102)=0.306368719492D+04
coeff(1103)=-0.577071045307D+04
coeff(1104)=0.299434300801D+04
coeff(1105)=-0.359554029914D+04
coeff(1106)=0.139914122652D+05
coeff(1107)=-0.101119393571D+04
coeff(1108)=0.984273668153D+04
coeff(1109)=0.480271351758D+04
coeff(1110)=0.116062111816D+05
coeff(1111)=0.324571349754D+04
coeff(1112)=-0.584861538414D+04
coeff(1113)=0.267578509629D+04
coeff(1114)=0.132121793027D+03
coeff(1115)=0.465902637063D+04
coeff(1116)=-0.252640617873D+04
coeff(1117)=0.401167365355D+04
coeff(1118)=0.757152864356D+04
coeff(1119)=0.330240202253D+04
coeff(1120)=-0.959408445212D+03
coeff(1121)=-0.167284961315D+05
coeff(1122)=0.868394451665D+03
coeff(1123)=0.379150461103D+04
coeff(1124)=0.514418417803D+03
coeff(1125)=0.118398341870D+04
coeff(1126)=0.413449341812D+04
coeff(1127)=0.794925679096D+04
coeff(1128)=0.108548792026D+05
coeff(1129)=0.164175263450D+04
coeff(1130)=0.637495548019D+03
coeff(1131)=-0.202616128012D+04
coeff(1132)=-0.919143684902D+04
coeff(1133)=0.165237533639D+02
coeff(1134)=0.327930363047D+04
coeff(1135)=-0.562356377043D+04
coeff(1136)=-0.789491499791D+03
coeff(1137)=-0.436216644438D+04
coeff(1138)=-0.756968170070D+03
coeff(1139)=-0.473918009082D+04
coeff(1140)=-0.726997133185D+04
coeff(1141)=0.230922343439D+04
coeff(1142)=0.885581364063D+04
coeff(1143)=-0.545788977784D+04
coeff(1144)=0.513501948297D+04
coeff(1145)=0.656746020845D+04
coeff(1146)=0.154683958464D+05
coeff(1147)=0.709788749439D+04
coeff(1148)=0.793428067530D+03
coeff(1149)=-0.269716977064D+04
coeff(1150)=-0.459144704304D+04
coeff(1151)=0.562742795863D+04
coeff(1152)=-0.317804461070D+04
coeff(1153)=-0.823353029636D+04
coeff(1154)=-0.773321940022D+04
coeff(1155)=0.453377921988D+04
coeff(1156)=0.304088474681D+04
coeff(1157)=0.874372652337D+04
coeff(1158)=0.485461958091D+04
coeff(1159)=0.387871984645D+04
coeff(1160)=0.399050536442D+04
coeff(1161)=-0.174791386350D+04
coeff(1162)=0.957532700751D+04
coeff(1163)=0.416971107005D+04
coeff(1164)=0.391618864811D+04
coeff(1165)=-0.978522832773D+03
coeff(1166)=-0.721526706054D+03
coeff(1167)=0.848764298271D+04
coeff(1168)=-0.206499049527D+04
coeff(1169)=0.547065472640D+04
coeff(1170)=0.770589248590D+03
coeff(1171)=0.241623391197D+04
coeff(1172)=-0.229974724372D+04
coeff(1173)=0.138681619035D+04
coeff(1174)=0.507959766574D+04
coeff(1175)=-0.966139601785D+04
coeff(1176)=-0.150539275134D+04
coeff(1177)=0.343334110060D+04
coeff(1178)=0.638885168763D+04
coeff(1179)=0.463724142555D+04
coeff(1180)=-0.258116295042D+04
coeff(1181)=0.279902682476D+04
coeff(1182)=0.160947266647D+03
coeff(1183)=0.551920266709D+04
coeff(1184)=0.443296785239D+04
coeff(1185)=-0.291929491191D+04
coeff(1186)=-0.430096264617D+03
coeff(1187)=0.950481489367D+03
coeff(1188)=-0.405108701649D+04
coeff(1189)=0.908635884910D+04
coeff(1190)=0.746567912648D+04
coeff(1191)=0.363002690812D+04
coeff(1192)=-0.164930528042D+05
coeff(1193)=-0.455329891243D+04
coeff(1194)=-0.419559988394D+04
coeff(1195)=0.172512443562D+03
coeff(1196)=-0.578793759562D+04
coeff(1197)=-0.640598951451D+04
coeff(1198)=-0.523452484265D+04
coeff(1199)=-0.344832551101D+03
coeff(1200)=-0.983756582241D+04
coeff(1201)=-0.978886918093D+04
coeff(1202)=0.333732034496D+03
coeff(1203)=0.972345080370D+03
coeff(1204)=-0.272312686431D+04
coeff(1205)=-0.756256780785D+04
coeff(1206)=-0.800246087590D+04
coeff(1207)=-0.429769566768D+04
coeff(1208)=0.182123229929D+04
coeff(1209)=0.148318880125D+03
coeff(1210)=0.777156694537D+03
coeff(1211)=0.584365428286D+04
coeff(1212)=-0.271881849171D+04
coeff(1213)=0.495407423078D+04
coeff(1214)=-0.435881025252D+04
coeff(1215)=0.134304180390D+04
coeff(1216)=0.108668265220D+05
coeff(1217)=-0.569176831446D+04
coeff(1218)=0.196998785383D+04
coeff(1219)=-0.160214160970D+04
coeff(1220)=0.451647095073D+04
coeff(1221)=-0.309378359991D+04
coeff(1222)=0.143922763214D+04
coeff(1223)=-0.334537553968D+04
coeff(1224)=-0.141031963727D+05
coeff(1225)=-0.539115412017D+04
coeff(1226)=0.136776747142D+05
coeff(1227)=0.116797814503D+05
coeff(1228)=-0.110410447976D+05
coeff(1229)=0.472618513777D+04
coeff(1230)=0.231050117586D+04
coeff(1231)=-0.622968952770D+04
coeff(1232)=-0.772574448311D+04
coeff(1233)=0.425821980070D+04
coeff(1234)=0.224058549297D+04
coeff(1235)=0.750065253617D+02
coeff(1236)=0.539209286783D+04
coeff(1237)=-0.393326209091D+04
coeff(1238)=0.466419153600D+04
coeff(1239)=0.175642802687D+03
coeff(1240)=0.334515830087D+04
coeff(1241)=0.974609393095D+03
coeff(1242)=0.383786898150D+04
coeff(1243)=-0.362933393565D+04
coeff(1244)=-0.933973088559D+04
coeff(1245)=-0.762556172203D+04
coeff(1246)=-0.382919176211D+03
coeff(1247)=-0.537785138681D+04
coeff(1248)=0.935754223766D+03
coeff(1249)=-0.559644042259D+04
coeff(1250)=-0.114295594663D+04
coeff(1251)=-0.139250799794D+04
coeff(1252)=0.882429273181D+04
coeff(1253)=-0.507295541346D+03
coeff(1254)=0.231488147006D+04
coeff(1255)=0.569797885330D+04
coeff(1256)=0.114151735156D+04
coeff(1257)=0.313408598741D+04
coeff(1258)=-0.659531797285D+04
coeff(1259)=0.151345494806D+05
coeff(1260)=0.433475423394D+04
coeff(1261)=0.259643701557D+04
coeff(1262)=0.134215713478D+05
coeff(1263)=0.282633033745D+04
coeff(1264)=0.131134035607D+04
coeff(1265)=-0.962949799793D+04
coeff(1266)=-0.120354034719D+05
coeff(1267)=-0.751529791503D+04
coeff(1268)=-0.134563411431D+04
coeff(1269)=-0.450697560650D+04
coeff(1270)=-0.597326236841D+04
coeff(1271)=0.398586239055D+04
coeff(1272)=-0.410891063110D+04
coeff(1273)=0.340856334045D+04
coeff(1274)=-0.294805704801D+04
coeff(1275)=-0.847014670150D+04
coeff(1276)=-0.756898533905D+04
coeff(1277)=-0.624659568005D+03
coeff(1278)=-0.256502611822D+04
coeff(1279)=0.557277797937D+04
coeff(1280)=-0.106207927457D+04
coeff(1281)=-0.206304003634D+04
coeff(1282)=-0.548460427043D+03
coeff(1283)=0.640192963858D+04
coeff(1284)=0.590060749142D+04
coeff(1285)=0.241641538523D+04
coeff(1286)=0.493457060737D+04
coeff(1287)=0.922277191829D+03
coeff(1288)=0.664381529872D+04
coeff(1289)=0.138339518080D+04
coeff(1290)=0.530840520284D+04
coeff(1291)=0.925498979219D+04
coeff(1292)=0.297010446685D+04
coeff(1293)=-0.985640208013D+03
coeff(1294)=0.308389558246D+03
coeff(1295)=0.116820821184D+05
coeff(1296)=-0.439192767591D+04
coeff(1297)=-0.133841147805D+05
coeff(1298)=-0.293445674856D+04
coeff(1299)=0.732628882156D+03
coeff(1300)=0.517493609066D+04
coeff(1301)=-0.234217427433D+04
coeff(1302)=0.242016745258D+04
coeff(1303)=0.420472874247D+03
coeff(1304)=-0.167903443033D+04
coeff(1305)=0.898766311618D+04
coeff(1306)=0.860276834364D+04
coeff(1307)=-0.309708791139D+04
coeff(1308)=-0.263615270963D+04
coeff(1309)=-0.109330071430D+05
coeff(1310)=-0.888922015390D+03
coeff(1311)=0.516777764983D+04
coeff(1312)=-0.102234118516D+05
coeff(1313)=0.394344894177D+04
coeff(1314)=0.563625616572D+04
coeff(1315)=-0.147138873310D+05
coeff(1316)=0.121446062423D+05
coeff(1317)=-0.316038565621D+04
coeff(1318)=-0.155289407316D+05
coeff(1319)=-0.179498583723D+04
coeff(1320)=0.122780746410D+04
coeff(1321)=-0.143995273964D+05
coeff(1322)=-0.465926890207D+04
coeff(1323)=-0.270835709953D+04
coeff(1324)=-0.223491138461D+04
coeff(1325)=0.664462986684D+04
coeff(1326)=-0.149223960556D+04
coeff(1327)=0.134567389586D+05
coeff(1328)=0.864681710270D+03
coeff(1329)=0.347633007260D+04
coeff(1330)=-0.272699937702D+04
coeff(1331)=-0.157926787442D+04
coeff(1332)=0.698990961703D+04
coeff(1333)=0.413420122018D+04
coeff(1334)=-0.174410641516D+04
coeff(1335)=-0.895374226874D+03
coeff(1336)=-0.147141933206D+05
coeff(1337)=-0.348908801996D+04
coeff(1338)=-0.468716800981D+04
coeff(1339)=-0.457054580265D+04
coeff(1340)=-0.688936797939D+04
coeff(1341)=0.849030776179D+04
coeff(1342)=0.123123601580D+05
coeff(1343)=-0.551408438594D+03
coeff(1344)=0.512845110423D+04
coeff(1345)=0.866084975552D+03
coeff(1346)=-0.739267685508D+04
coeff(1347)=-0.608759653713D+04
coeff(1348)=-0.973582716808D+04
coeff(1349)=-0.371975012136D+03
coeff(1350)=0.825445144573D+04
coeff(1351)=-0.462039629769D+03
coeff(1352)=-0.572899423753D+04
coeff(1353)=-0.121490419521D+05
coeff(1354)=-0.554596895094D+04
coeff(1355)=-0.330959800888D+03
coeff(1356)=-0.451250538243D+04
coeff(1357)=0.337182334532D+04
coeff(1358)=0.562792516382D+04
coeff(1359)=0.983625109055D+04
coeff(1360)=0.861771590727D+04
coeff(1361)=-0.576799707553D+03
coeff(1362)=-0.418755692181D+04
coeff(1363)=-0.195223482539D+03
coeff(1364)=-0.623962345921D+04
coeff(1365)=0.131339021683D+04
coeff(1366)=-0.371836186055D+04
coeff(1367)=-0.414285723438D+03
coeff(1368)=0.848791062149D+04
coeff(1369)=-0.925134893483D+04
coeff(1370)=-0.610137716166D+04
coeff(1371)=0.518613961539D+04
coeff(1372)=-0.214604945481D+05
coeff(1373)=-0.391962354880D+04
coeff(1374)=-0.125182868417D+03
coeff(1375)=-0.871946094587D+04
coeff(1376)=-0.129229657774D+04
coeff(1377)=-0.145163103921D+05
coeff(1378)=-0.939540989641D+04
coeff(1379)=0.152945442432D+05
coeff(1380)=-0.417290793689D+04
coeff(1381)=0.158658586022D+04
coeff(1382)=0.443694101594D+04
coeff(1383)=-0.328687642292D+02
coeff(1384)=0.851431799292D+04
coeff(1385)=-0.336185862304D+04
coeff(1386)=0.106494372247D+05
coeff(1387)=0.561406113475D+04
coeff(1388)=-0.683504232979D+04
coeff(1389)=-0.782259103868D+03
coeff(1390)=0.747550774696D+04
coeff(1391)=-0.144716873412D+05
coeff(1392)=0.167493895106D+04
coeff(1393)=-0.483236843505D+04
coeff(1394)=0.173470638966D+04
coeff(1395)=-0.119452400558D+05
coeff(1396)=-0.110174763877D+05
coeff(1397)=-0.671750523320D+04
coeff(1398)=0.424731824400D+04
coeff(1399)=-0.348413204050D+04
coeff(1400)=0.274277511929D+04
coeff(1401)=0.963771409067D+04
coeff(1402)=-0.341680920971D+04
coeff(1403)=-0.601717860291D+03
coeff(1404)=-0.254430789823D+04
coeff(1405)=-0.394639048421D+03
coeff(1406)=-0.122391377577D+05
coeff(1407)=0.282830271424D+03
coeff(1408)=-0.635064662072D+04
coeff(1409)=-0.461944434339D+04
coeff(1410)=-0.566023673419D+04
coeff(1411)=0.586482669571D+03
coeff(1412)=0.957996401019D+04
coeff(1413)=0.525254798411D+03
coeff(1414)=0.213176946554D+04
coeff(1415)=0.140446956129D+04
coeff(1416)=-0.126578057716D+05
coeff(1417)=-0.562606008031D+04
coeff(1418)=-0.129807015934D+04
coeff(1419)=0.984998897287D+04
coeff(1420)=0.979676392336D+04
coeff(1421)=-0.681945365966D+04
coeff(1422)=0.111794569939D+05
coeff(1423)=0.532730099031D+04
coeff(1424)=0.265163692158D+04
coeff(1425)=-0.803485569898D+04
coeff(1426)=-0.933008505373D+02
coeff(1427)=0.853901990903D+04
coeff(1428)=-0.349936978467D+04
coeff(1429)=-0.355554051282D+04
coeff(1430)=-0.971131216580D+04
coeff(1431)=-0.235863502142D+03
coeff(1432)=-0.684849066859D+04
coeff(1433)=-0.799265076579D+02
coeff(1434)=-0.425568529897D+04
coeff(1435)=0.635340525531D+04
coeff(1436)=-0.232631944419D+04
coeff(1437)=0.340982242217D+03
coeff(1438)=-0.914565221885D+04
coeff(1439)=-0.452333424098D+04
coeff(1440)=0.923555645729D+04
coeff(1441)=-0.502342987565D+04
coeff(1442)=0.430168140336D+04
coeff(1443)=0.521823180230D+04
coeff(1444)=-0.425169798907D+02
coeff(1445)=0.381362475660D+04
coeff(1446)=-0.903124077531D+04
coeff(1447)=-0.572991562008D+03
coeff(1448)=0.351471326222D+04
coeff(1449)=-0.567046460220D+04
coeff(1450)=-0.138549344064D+05
coeff(1451)=-0.209501525301D+04
coeff(1452)=-0.681026364529D+04
coeff(1453)=-0.242199834251D+04
coeff(1454)=0.872031934441D+04
coeff(1455)=0.233878635072D+04
coeff(1456)=0.119575718951D+05
coeff(1457)=-0.634024021865D+04
coeff(1458)=0.565680125788D+04
coeff(1459)=0.934545790506D+03
coeff(1460)=0.187957196579D+04
coeff(1461)=0.194753712361D+04
coeff(1462)=0.421641769750D+04
coeff(1463)=0.433440395754D+04
coeff(1464)=-0.575721176440D+03
coeff(1465)=0.116794022847D+05
coeff(1466)=0.274180810892D+04
coeff(1467)=0.134650750733D+05
coeff(1468)=0.158009546852D+05
coeff(1469)=0.479693234858D+04
coeff(1470)=-0.729789467621D+04
coeff(1471)=-0.100397894764D+05
coeff(1472)=-0.575633529457D+04
coeff(1473)=0.302523072616D+04
coeff(1474)=-0.191934753000D+04
coeff(1475)=0.275691582556D+04
coeff(1476)=-0.442122679603D+04
coeff(1477)=-0.109610430700D+04
coeff(1478)=0.360271528486D+04
coeff(1479)=-0.242412250973D+04
coeff(1480)=0.393096917237D+04
coeff(1481)=-0.216482780416D+04
coeff(1482)=-0.164520426914D+04
coeff(1483)=-0.867674139536D+04
coeff(1484)=-0.181915028109D+04
coeff(1485)=0.153362432949D+05
coeff(1486)=-0.942214515143D+04
coeff(1487)=-0.619611681691D+04
coeff(1488)=-0.113972272884D+04
coeff(1489)=-0.580984997808D+04
coeff(1490)=-0.750499012952D+02
coeff(1491)=-0.836620807200D+04
coeff(1492)=0.598496003974D+04
coeff(1493)=0.102655581228D+03
coeff(1494)=0.210520143576D+04
coeff(1495)=-0.708460476393D+04
coeff(1496)=0.216328681227D+03
coeff(1497)=0.460336563641D+04
coeff(1498)=0.493545660431D+04
coeff(1499)=-0.917518590434D+04
coeff(1500)=-0.123571129553D+05
coeff(1501)=0.750113363872D+04
coeff(1502)=-0.698698367587D+04
coeff(1503)=0.131082908093D+05
coeff(1504)=-0.455369200705D+04
coeff(1505)=-0.208130666699D+04
coeff(1506)=0.786585432645D+04
coeff(1507)=0.671883283750D+03
coeff(1508)=-0.420060241965D+04
coeff(1509)=-0.698638805078D+04
coeff(1510)=-0.919497375588D+04
coeff(1511)=-0.232927665468D+04
coeff(1512)=0.827268667121D+03
coeff(1513)=0.290681756690D+04
coeff(1514)=0.779656094472D+04
coeff(1515)=-0.819750827551D+04
coeff(1516)=0.116560829465D+05
coeff(1517)=-0.514648251217D+04
coeff(1518)=-0.130150166784D+05
coeff(1519)=0.252638297341D+04
coeff(1520)=0.224409624290D+04
coeff(1521)=0.101751867398D+04
coeff(1522)=-0.116686770944D+05
coeff(1523)=-0.129302748856D+05
coeff(1524)=-0.268276498938D+04
coeff(1525)=0.494221045077D+04
coeff(1526)=0.176083210012D+04
coeff(1527)=-0.311949225904D+04
coeff(1528)=-0.211326041017D+05
coeff(1529)=-0.122759741286D+04
coeff(1530)=-0.700556078358D+04
coeff(1531)=0.751732551121D+04
coeff(1532)=-0.174022229575D+05
coeff(1533)=0.124447098222D+05
coeff(1534)=0.121536671425D+05
coeff(1535)=-0.491388836003D+04
coeff(1536)=-0.671461355092D+04
coeff(1537)=-0.950189217450D+03
coeff(1538)=-0.216104258687D+04
coeff(1539)=0.507493152696D+04
coeff(1540)=0.206483529351D+03
coeff(1541)=0.167641116370D+05
coeff(1542)=-0.153527330496D+04
coeff(1543)=0.566484160241D+04
coeff(1544)=-0.683543226750D+04
coeff(1545)=0.887705416422D+04
coeff(1546)=-0.975766555954D+04
coeff(1547)=-0.445625563656D+04
coeff(1548)=0.360709779834D+04
coeff(1549)=-0.403128179178D+04
coeff(1550)=-0.477238585326D+04
coeff(1551)=0.541367550109D+04
coeff(1552)=0.228257777726D+04
coeff(1553)=-0.122171771864D+05
coeff(1554)=-0.259629621848D+03
coeff(1555)=-0.524094496805D+03
coeff(1556)=-0.619454422217D+04
coeff(1557)=0.264640632535D+04
coeff(1558)=-0.231888231670D+03
coeff(1559)=0.606417214752D+04
coeff(1560)=0.741457735439D+04
coeff(1561)=-0.301698054162D+04
coeff(1562)=-0.836704337136D+04
coeff(1563)=0.147321318994D+04
coeff(1564)=0.364237458740D+04
coeff(1565)=-0.518467287940D+04
coeff(1566)=0.116695543626D+04
coeff(1567)=0.282266433427D+03
coeff(1568)=-0.167059012846D+05
coeff(1569)=-0.597943620623D+04
coeff(1570)=-0.254234628327D+03
coeff(1571)=-0.116630380795D+05
coeff(1572)=-0.541939870989D+04
coeff(1573)=-0.583037650003D+04
coeff(1574)=0.251789579137D+04
coeff(1575)=-0.519514152788D+03
coeff(1576)=0.117430651965D+04
coeff(1577)=-0.630178539281D+04
coeff(1578)=-0.126293151820D+04
coeff(1579)=-0.325658002551D+04
coeff(1580)=0.921048820863D+02
coeff(1581)=0.113400619427D+05
coeff(1582)=0.494461762372D+04
coeff(1583)=-0.114344879171D+05
coeff(1584)=-0.219705409885D+04
coeff(1585)=-0.741960906091D+04
coeff(1586)=0.514386939980D+03
coeff(1587)=-0.737800213509D+04
coeff(1588)=-0.261254080918D+04
coeff(1589)=0.658252609017D+04
coeff(1590)=0.121060342775D+05
coeff(1591)=0.455313390400D+04
coeff(1592)=-0.144692429904D+04
coeff(1593)=0.700482473951D+04
coeff(1594)=-0.102641891514D+05
coeff(1595)=0.643540711175D+04
coeff(1596)=0.880011890010D+04
coeff(1597)=0.437495119590D+03
coeff(1598)=-0.115465439481D+05
coeff(1599)=-0.287558146319D+04
coeff(1600)=-0.617568600771D+04
coeff(1601)=0.326172274873D+04
coeff(1602)=-0.240810867664D+03
coeff(1603)=-0.109197223212D+05
coeff(1604)=0.754011686957D+04
coeff(1605)=0.510379888754D+04
coeff(1606)=-0.584453528775D+04
coeff(1607)=0.293001634989D+04
coeff(1608)=-0.101532368694D+05
coeff(1609)=-0.491964130047D+04
coeff(1610)=-0.178342245771D+05
coeff(1611)=-0.234802138237D+03
coeff(1612)=-0.679628058148D+04
coeff(1613)=0.444146213824D+04
coeff(1614)=0.180638989507D+05
coeff(1615)=-0.199388832456D+04
coeff(1616)=0.412452858602D+04
coeff(1617)=-0.696358988372D+03
coeff(1618)=-0.477378715792D+04
coeff(1619)=-0.170494705636D+05
coeff(1620)=-0.758921746321D+04
coeff(1621)=0.552978540710D+04
coeff(1622)=0.117435896090D+04
coeff(1623)=-0.389094773607D+03
coeff(1624)=-0.477425035260D+04
coeff(1625)=0.309615148623D+04
coeff(1626)=0.650678347775D+04
coeff(1627)=-0.278819188335D+04
coeff(1628)=-0.146259150452D+05
coeff(1629)=-0.658677755998D+04
coeff(1630)=-0.235355749962D+05
coeff(1631)=-0.528229205415D+04
coeff(1632)=-0.319806565492D+04
coeff(1633)=0.688537029994D+04
coeff(1634)=-0.368592477906D+04
coeff(1635)=0.762338519689D+04
coeff(1636)=-0.137192383053D+04
coeff(1637)=-0.660957262804D+04
coeff(1638)=0.114599912712D+05
coeff(1639)=-0.859238160860D+04
coeff(1640)=0.207009519650D+04
coeff(1641)=0.128044378874D+05
coeff(1642)=0.252078792907D+04
coeff(1643)=-0.689664802868D+04
coeff(1644)=-0.331427953975D+03
coeff(1645)=0.145449840145D+05
coeff(1646)=-0.358281937315D+04
coeff(1647)=-0.813167222507D+04
coeff(1648)=0.192381698995D+04
coeff(1649)=-0.824371175361D+04
coeff(1650)=-0.645177555174D+04
coeff(1651)=-0.847465147938D+03
coeff(1652)=-0.121442166194D+05
coeff(1653)=0.889008053490D+03
coeff(1654)=-0.120327880786D+05
coeff(1655)=0.473745587074D+04
coeff(1656)=0.141046293558D+04
coeff(1657)=-0.112060445372D+03
coeff(1658)=0.620265622084D+04
coeff(1659)=-0.146175442743D+04
coeff(1660)=0.122276438802D+05
coeff(1661)=-0.839236572534D+04
coeff(1662)=0.622078475631D+03
coeff(1663)=0.189320948375D+04
coeff(1664)=-0.108216661868D+04
coeff(1665)=-0.803437929993D+04
coeff(1666)=-0.732183383671D+04
coeff(1667)=-0.137349608644D+05
coeff(1668)=-0.441838884492D+04
coeff(1669)=0.178209214376D+05
coeff(1670)=0.334235845730D+03
coeff(1671)=0.205627268700D+04
coeff(1672)=0.104975793713D+05
coeff(1673)=-0.919718324080D+04
coeff(1674)=0.937842298893D+04
coeff(1675)=0.135901985345D+05
coeff(1676)=0.129810850286D+05
coeff(1677)=0.123661470504D+05
coeff(1678)=0.259727059350D+04
coeff(1679)=0.244147622220D+04
coeff(1680)=-0.554131392463D+04
coeff(1681)=0.110910792093D+04
coeff(1682)=0.312731184272D+04
coeff(1683)=0.220255538104D+04
coeff(1684)=0.518412221615D+04
coeff(1685)=0.116050277680D+04
coeff(1686)=0.125506496645D+04
coeff(1687)=0.631420884731D+04
coeff(1688)=0.223418709410D+04
coeff(1689)=-0.290830886977D+03
coeff(1690)=0.118645923843D+05
coeff(1691)=-0.332349034232D+04
coeff(1692)=0.217415438107D+04
coeff(1693)=0.132045229634D+05
coeff(1694)=0.103288940231D+05
coeff(1695)=-0.291474515219D+04
coeff(1696)=-0.179624069189D+04
coeff(1697)=-0.475754141866D+04
coeff(1698)=-0.136786497046D+05
coeff(1699)=-0.596672865019D+04
coeff(1700)=-0.970082518174D+04
coeff(1701)=-0.554182752630D+03
coeff(1702)=-0.174976998778D+04
coeff(1703)=0.281739051468D+04
coeff(1704)=0.344762073607D+03
coeff(1705)=0.154026061322D+04
coeff(1706)=-0.430365110010D+04
coeff(1707)=-0.160604371928D+04
coeff(1708)=0.377939007713D+04
coeff(1709)=-0.836799157820D+04
coeff(1710)=0.110768398612D+04
coeff(1711)=0.547501726882D+04
coeff(1712)=-0.129280612421D+04
coeff(1713)=-0.524959139234D+04
coeff(1714)=0.632607144280D+04
coeff(1715)=-0.205774553284D+04
coeff(1716)=0.150889695978D+04
coeff(1717)=-0.924446357412D+03
coeff(1718)=0.102268032630D+04
coeff(1719)=-0.564873003248D+03
coeff(1720)=0.161239632144D+04
coeff(1721)=0.378782933300D+04
coeff(1722)=0.175448871291D+04
coeff(1723)=0.283969567928D+04
coeff(1724)=-0.759229804886D+04
coeff(1725)=-0.309161993012D+04
coeff(1726)=0.103864208928D+05
coeff(1727)=0.807875898991D+04
coeff(1728)=-0.486439190665D+04
coeff(1729)=0.182697654556D+04
coeff(1730)=-0.315418188227D+04
coeff(1731)=0.663939540515D+04
coeff(1732)=0.127233730457D+04
coeff(1733)=0.316306673807D+04
coeff(1734)=0.144727409003D+04
coeff(1735)=-0.902776100429D+03
coeff(1736)=0.456993915396D+04
coeff(1737)=-0.242237422306D+04
coeff(1738)=-0.206461511666D+05
coeff(1739)=-0.931458521748D+04
coeff(1740)=0.723029124580D+04
coeff(1741)=0.290597400685D+03
coeff(1742)=-0.627100307835D+04
coeff(1743)=-0.100460669801D+04
coeff(1744)=0.626462650985D+04
coeff(1745)=0.205230713004D+04
coeff(1746)=-0.385840376210D+04
coeff(1747)=-0.447468957449D+04
coeff(1748)=0.257483960600D+04
coeff(1749)=0.213991776789D+05
coeff(1750)=0.762369630988D+03
coeff(1751)=-0.113465264308D+04
coeff(1752)=0.625081120488D+04
coeff(1753)=0.883023015006D+04
coeff(1754)=-0.920862491463D+04
coeff(1755)=-0.238459332674D+04
coeff(1756)=-0.400076139119D+04
coeff(1757)=-0.826336930354D+04
coeff(1758)=-0.867330315247D+04
coeff(1759)=-0.515397494918D+04
coeff(1760)=-0.330852339056D+04
coeff(1761)=0.189217260920D+04
coeff(1762)=0.690786718500D+04
coeff(1763)=0.540918817852D+04
coeff(1764)=-0.541136281602D+03
coeff(1765)=0.543816641647D+04
coeff(1766)=0.135302848929D+05
coeff(1767)=0.751952631983D+04
coeff(1768)=0.111208286977D+05
coeff(1769)=0.470636984761D+04
coeff(1770)=-0.947047861329D+04
coeff(1771)=-0.357221112814D+04
coeff(1772)=-0.443636779394D+04
coeff(1773)=-0.222818531708D+04
coeff(1774)=0.386283748330D+04
coeff(1775)=0.775397852809D+03
coeff(1776)=0.841598131928D+04
coeff(1777)=-0.608844538971D+04
coeff(1778)=0.464540176461D+04
coeff(1779)=0.789910089632D+04
coeff(1780)=0.103174283257D+04
coeff(1781)=0.864266182495D+03
coeff(1782)=-0.355202871939D+04
coeff(1783)=-0.165224007536D+04
coeff(1784)=0.692711571970D+03
coeff(1785)=0.247182216301D+04
coeff(1786)=-0.120141541059D+04
coeff(1787)=0.671131551824D+03
coeff(1788)=-0.114586282934D+04
coeff(1789)=0.650719507430D+04
coeff(1790)=-0.905077506415D+03
coeff(1791)=0.851866620263D+04
coeff(1792)=-0.174436618603D+04
coeff(1793)=0.748210542782D+03
coeff(1794)=-0.158441247803D+05
coeff(1795)=0.230140069529D+04
coeff(1796)=-0.967173484099D+04
coeff(1797)=-0.135542460299D+04
coeff(1798)=-0.708952439356D+04
coeff(1799)=-0.100848777747D+05
coeff(1800)=-0.113755804272D+05
coeff(1801)=0.680457621951D+04
coeff(1802)=-0.731670466936D+04
coeff(1803)=0.802769502803D+04
coeff(1804)=0.903158393929D+03
coeff(1805)=0.190500025234D+04
coeff(1806)=-0.200672309248D+04
coeff(1807)=0.350807983163D+03
coeff(1808)=-0.107807773526D+05
coeff(1809)=0.100943939565D+05
coeff(1810)=-0.372286132836D+04
coeff(1811)=0.133490032418D+05
coeff(1812)=0.907597671738D+04
coeff(1813)=0.109516949830D+05
coeff(1814)=-0.320051372419D+04
coeff(1815)=-0.861573556228D+04
coeff(1816)=-0.894527035435D+04
coeff(1817)=0.809470931679D+03
coeff(1818)=0.419589029333D+04
coeff(1819)=-0.477500369138D+04
coeff(1820)=-0.105269437146D+04
coeff(1821)=0.457102748122D+04
coeff(1822)=-0.740134144617D+03
coeff(1823)=0.106109749345D+05
coeff(1824)=-0.232704888211D+04
coeff(1825)=-0.772609440690D+04
coeff(1826)=-0.380748979624D+04
coeff(1827)=0.506113586150D+04
coeff(1828)=0.638717219194D+04
coeff(1829)=-0.105039462022D+04
coeff(1830)=0.754448044687D+04
coeff(1831)=0.609239430727D+04
coeff(1832)=-0.925475859581D+04
coeff(1833)=0.711364124899D+04
coeff(1834)=-0.119947715791D+05
coeff(1835)=0.231051321230D+04
coeff(1836)=-0.363575443930D+04
coeff(1837)=-0.373504736818D+04
coeff(1838)=0.394659600657D+04
coeff(1839)=0.841171018794D+04
coeff(1840)=0.297050140394D+04
coeff(1841)=-0.496951854472D+03
coeff(1842)=0.173409595632D+04
coeff(1843)=0.109439472037D+05
coeff(1844)=0.330157405894D+04
coeff(1845)=0.343940351365D+04
coeff(1846)=-0.306212009289D+04
coeff(1847)=-0.897313866472D+03
coeff(1848)=-0.187511900413D+04
coeff(1849)=-0.619390981555D+04
coeff(1850)=0.131607764016D+05
coeff(1851)=0.100958947011D+05
coeff(1852)=0.325098596316D+04
coeff(1853)=0.657712662279D+04
coeff(1854)=-0.690554153754D+04
coeff(1855)=0.109415146422D+04
coeff(1856)=0.722106462168D+04
coeff(1857)=-0.467014386721D+04
coeff(1858)=-0.661038525477D+04
coeff(1859)=-0.636451840079D+04
coeff(1860)=-0.895418453741D+04
coeff(1861)=0.306204702260D+04
coeff(1862)=0.493504550538D+04
coeff(1863)=-0.463527358110D+03
coeff(1864)=-0.263402618634D+04
coeff(1865)=0.923143600573D+04
coeff(1866)=0.865746412922D+04
coeff(1867)=0.952668077449D+04
coeff(1868)=-0.431583681897D+04
coeff(1869)=0.422243140232D+04
coeff(1870)=0.565084729049D+04
coeff(1871)=-0.418594917854D+04
coeff(1872)=0.139672610496D+04
coeff(1873)=0.598816692796D+04
coeff(1874)=0.953216173475D+04
coeff(1875)=-0.402111070886D+04
coeff(1876)=0.124964280346D+04
coeff(1877)=-0.220606217222D+04
coeff(1878)=-0.400513254165D+04
coeff(1879)=-0.285729090251D+04
coeff(1880)=-0.117210172472D+05
coeff(1881)=-0.507016218372D+04
coeff(1882)=0.952980152954D+04
coeff(1883)=0.872532052631D+03
coeff(1884)=-0.137870597653D+05
coeff(1885)=0.264019350745D+04
coeff(1886)=0.381586854384D+03
coeff(1887)=-0.676674724743D+04
coeff(1888)=-0.110069009282D+04
coeff(1889)=0.273784201076D+03
coeff(1890)=-0.863640385109D+04
coeff(1891)=-0.537710024275D+04
coeff(1892)=-0.571668160537D+04
coeff(1893)=-0.372324130898D+03
coeff(1894)=-0.133245852249D+05
coeff(1895)=0.495270703532D+03
coeff(1896)=0.323542881333D+04
coeff(1897)=-0.240252950564D+04
coeff(1898)=-0.220455528345D+03
coeff(1899)=-0.701746618267D+04
coeff(1900)=0.110279124302D+05
coeff(1901)=0.949495090969D+04
coeff(1902)=-0.790138926952D+03
coeff(1903)=0.995774892314D+04
coeff(1904)=0.119522868927D+05
coeff(1905)=0.227102675691D+04
coeff(1906)=0.143145982183D+04
coeff(1907)=0.114174171480D+05
coeff(1908)=0.164183894190D+03
coeff(1909)=0.379041913690D+03
coeff(1910)=0.505859850858D+04
coeff(1911)=0.118172197318D+04
coeff(1912)=0.163723826050D+05
coeff(1913)=-0.647567130722D+03
coeff(1914)=0.184340309760D+04
coeff(1915)=0.973824793450D+04
coeff(1916)=0.100732418276D+04
coeff(1917)=-0.293065275316D+04
coeff(1918)=-0.908171107466D+04
coeff(1919)=0.838781994273D+04
coeff(1920)=0.834157866639D+04
coeff(1921)=-0.146033906366D+04
coeff(1922)=0.278369007651D+04
coeff(1923)=-0.959661801944D+04
coeff(1924)=0.730966518788D+04
coeff(1925)=-0.219376094343D+04
coeff(1926)=0.774174936274D+04
coeff(1927)=0.362344973512D+04
coeff(1928)=-0.738972445507D+04
coeff(1929)=0.802046201133D+04
coeff(1930)=-0.274235884235D+04
coeff(1931)=0.468618162255D+04
coeff(1932)=-0.274372700096D+04
coeff(1933)=0.654053391660D+04
coeff(1934)=0.206501900451D+03
coeff(1935)=-0.835530467568D+04
coeff(1936)=-0.859722906201D+04
coeff(1937)=0.116529369266D+04
coeff(1938)=0.125856471475D+05
coeff(1939)=-0.247813621150D+04
coeff(1940)=-0.939202063690D+04
coeff(1941)=0.635450621055D+04
coeff(1942)=-0.394819945693D+04
coeff(1943)=0.165023111358D+05
coeff(1944)=0.322213592327D+04
coeff(1945)=-0.249331028160D+04
coeff(1946)=0.108589293281D+05
coeff(1947)=-0.114805088261D+05
coeff(1948)=0.621511406300D+04
coeff(1949)=0.391378484139D+04
coeff(1950)=-0.470434339916D+03
coeff(1951)=-0.138644559460D+05
coeff(1952)=0.465858460434D+04
coeff(1953)=-0.383570922327D+04
coeff(1954)=-0.497103693905D+04
coeff(1955)=-0.578524487655D+04
coeff(1956)=0.800508737552D+04
coeff(1957)=-0.379643445550D+03
coeff(1958)=0.408600889939D+04
coeff(1959)=-0.497673469819D+04
coeff(1960)=-0.583443598729D+04
coeff(1961)=-0.133918931950D+05
coeff(1962)=-0.957332790580D+04
coeff(1963)=0.310071563419D+04
coeff(1964)=-0.104703818055D+04
coeff(1965)=-0.585447455440D+04
coeff(1966)=-0.290751199991D+04
coeff(1967)=0.700906563998D+04
coeff(1968)=-0.117864971304D+05
coeff(1969)=0.177307948324D+04
coeff(1970)=0.533435158910D+03
coeff(1971)=-0.852120062992D+04
coeff(1972)=-0.920078524800D+03
coeff(1973)=-0.991652055589D+03
coeff(1974)=-0.144964334376D+04
coeff(1975)=0.609352667883D+04
coeff(1976)=-0.583945056103D+04
coeff(1977)=-0.365633235479D+04
coeff(1978)=-0.508764077184D+04
coeff(1979)=-0.101249791770D+05
coeff(1980)=-0.168829638904D+05
coeff(1981)=0.177486408008D+05
coeff(1982)=0.126746839309D+05
coeff(1983)=-0.985920121289D+04
coeff(1984)=0.195814550747D+04
coeff(1985)=0.688737159122D+04
coeff(1986)=0.446507357689D+04
coeff(1987)=0.320471350708D+04
coeff(1988)=0.938329481080D+04
coeff(1989)=0.217096129837D+04
coeff(1990)=0.718302795592D+04
coeff(1991)=-0.287717621969D+04
coeff(1992)=0.631879507384D+04
coeff(1993)=0.388390159008D+04
coeff(1994)=-0.667810455235D+03
coeff(1995)=0.374206593847D+03
coeff(1996)=-0.609848669437D+04
coeff(1997)=-0.886993585068D+04
coeff(1998)=-0.899530355882D+04
coeff(1999)=0.367366107626D+04
coeff(2000)=0.582310872967D+04
coeff(2001)=0.155287183586D+05
coeff(2002)=-0.483058353476D+04
coeff(2003)=-0.172088230390D+04
coeff(2004)=0.255788495859D+04
coeff(2005)=-0.677070147579D+04
coeff(2006)=-0.216352227632D+05
coeff(2007)=-0.509350755403D+04
coeff(2008)=0.232876880114D+03
coeff(2009)=-0.110202389097D+05
coeff(2010)=0.523717329935D+04
coeff(2011)=0.594002971909D+04
coeff(2012)=-0.512599984662D+04
coeff(2013)=-0.576642068912D+04
coeff(2014)=0.110634328257D+05
coeff(2015)=-0.411915655898D+04
coeff(2016)=-0.161157984040D+05
coeff(2017)=0.869862405023D+04
coeff(2018)=0.552721128178D+04
coeff(2019)=0.975138318264D+04
coeff(2020)=-0.450412911639D+04
coeff(2021)=-0.814345485838D+04
coeff(2022)=-0.347968325155D+04
coeff(2023)=-0.264650983984D+04
coeff(2024)=0.112564778490D+05
coeff(2025)=-0.140236747732D+04
coeff(2026)=-0.252628711331D+04
coeff(2027)=-0.539940451963D+03
coeff(2028)=0.101317832882D+05
coeff(2029)=-0.536131849015D+04
coeff(2030)=0.101615492620D+04
coeff(2031)=-0.820078529633D+04
coeff(2032)=-0.511724051109D+04
coeff(2033)=-0.634273954920D+04
coeff(2034)=-0.142540331018D+05
coeff(2035)=0.140584263335D+05
coeff(2036)=0.404255816662D+04
coeff(2037)=0.191533110831D+05
coeff(2038)=-0.124196189482D+05
coeff(2039)=-0.117085019131D+04
coeff(2040)=0.787805200994D+04
coeff(2041)=0.723109380276D+04
coeff(2042)=0.355903714518D+04
coeff(2043)=0.118187026527D+05
coeff(2044)=-0.550167068428D+04
coeff(2045)=-0.469403266370D+04
coeff(2046)=0.717362427703D+04
coeff(2047)=-0.573039806408D+04
coeff(2048)=0.916924572609D+04
coeff(2049)=-0.298793143147D+04
coeff(2050)=-0.900881914669D+04
coeff(2051)=-0.531243469540D+04
coeff(2052)=0.197986980078D+04
coeff(2053)=0.345153779926D+04
coeff(2054)=0.199200484062D+05
coeff(2055)=0.246206817711D+04
coeff(2056)=-0.151574629638D+05
coeff(2057)=0.423755231106D+04
coeff(2058)=-0.449439593107D+04
coeff(2059)=0.378016014610D+04
coeff(2060)=-0.356434124441D+04
coeff(2061)=-0.121472732282D+04
coeff(2062)=-0.231792659968D+05
coeff(2063)=0.188959001341D+04
coeff(2064)=0.161420609754D+04
coeff(2065)=0.925165647553D+04
coeff(2066)=0.808005001104D+04
coeff(2067)=0.209168080365D+04
coeff(2068)=0.239957787592D+04
coeff(2069)=-0.391453998197D+04
coeff(2070)=0.179379979202D+04
coeff(2071)=-0.735048033411D+04
coeff(2072)=0.177005192909D+04
coeff(2073)=0.106887669421D+05
coeff(2074)=0.822846970005D+04
coeff(2075)=-0.337722832271D+04
coeff(2076)=-0.585607385908D+03
coeff(2077)=0.653842550586D+04
coeff(2078)=0.215590092812D+05
coeff(2079)=-0.203821152337D+05
coeff(2080)=0.235927477067D+05
coeff(2081)=0.491522847310D+04
coeff(2082)=0.366112261353D+04
coeff(2083)=0.407715714826D+04
coeff(2084)=0.265609375708D+04
coeff(2085)=-0.674951561045D+04
coeff(2086)=-0.162809003226D+04
coeff(2087)=0.315707916903D+04
coeff(2088)=-0.104016235853D+05
coeff(2089)=0.123904382347D+05
coeff(2090)=0.148677712399D+04
coeff(2091)=0.125801917689D+05
coeff(2092)=0.367899809351D+04
coeff(2093)=0.560790104249D+04
coeff(2094)=-0.795846405960D+04
coeff(2095)=-0.123460297909D+05
coeff(2096)=0.323341287776D+04
coeff(2097)=-0.744438200478D+04
coeff(2098)=-0.992335305707D+04
coeff(2099)=-0.933167177964D+04
coeff(2100)=-0.106119734496D+05
coeff(2101)=-0.273099588987D+04
coeff(2102)=-0.167685904055D+05
coeff(2103)=0.412773531630D+04
coeff(2104)=-0.442060815724D+04
coeff(2105)=-0.163563007885D+04
coeff(2106)=0.277422149718D+04
coeff(2107)=-0.716338790434D+04
coeff(2108)=-0.394234718139D+04
coeff(2109)=-0.582777779492D+03
coeff(2110)=0.567950472757D+03
coeff(2111)=0.742134653118D+04
coeff(2112)=0.512374390697D+04
coeff(2113)=-0.182366435933D+05
coeff(2114)=-0.667875440266D+04
coeff(2115)=0.679975673315D+04
coeff(2116)=-0.621727896049D+04
coeff(2117)=-0.567299958567D+04
coeff(2118)=0.676532699382D+03
coeff(2119)=-0.109310691898D+03
coeff(2120)=0.176083557732D+03
coeff(2121)=0.640655528251D+04
coeff(2122)=-0.116225324406D+05
coeff(2123)=0.108631707934D+05
coeff(2124)=0.108096053288D+05
coeff(2125)=-0.581540885282D+04
coeff(2126)=-0.121105500540D+05
coeff(2127)=0.138614872646D+04
coeff(2128)=-0.992239980401D+04
coeff(2129)=0.928780380839D+04
coeff(2130)=0.978326158725D+04
coeff(2131)=0.108471766816D+05
coeff(2132)=0.220106484402D+04
coeff(2133)=0.108848913419D+05
coeff(2134)=0.116762843436D+05
coeff(2135)=-0.124386205018D+05
coeff(2136)=-0.759169589508D+04
coeff(2137)=-0.667680709719D+04
coeff(2138)=-0.799433572837D+04
coeff(2139)=0.852730031718D+04
coeff(2140)=-0.976943715084D+04
coeff(2141)=0.667197544214D+04
coeff(2142)=0.116182047953D+05
coeff(2143)=0.618750540201D+04
coeff(2144)=-0.564999075578D+04
coeff(2145)=-0.110114992110D+05
coeff(2146)=-0.573129867333D+03
coeff(2147)=-0.519676347614D+04
coeff(2148)=0.170337030493D+04
coeff(2149)=0.110710568494D+05
coeff(2150)=0.198714974372D+04
coeff(2151)=-0.695600058780D+03
coeff(2152)=-0.560808189160D+04
coeff(2153)=-0.130006963919D+05
coeff(2154)=0.109325894033D+05
coeff(2155)=0.189437644251D+03
coeff(2156)=-0.800346488176D+04
coeff(2157)=-0.234383331032D+04
coeff(2158)=-0.116330997435D+05
coeff(2159)=0.352767327559D+04
coeff(2160)=0.219113288168D+04
coeff(2161)=-0.893822139646D+04
coeff(2162)=0.114105342719D+05
coeff(2163)=0.319799285776D+04
coeff(2164)=-0.330077781573D+04
coeff(2165)=0.861115514457D+04
coeff(2166)=-0.719845405123D+04
coeff(2167)=-0.264442295453D+04
coeff(2168)=0.886885410470D+04
coeff(2169)=-0.106139525506D+04
coeff(2170)=-0.596523740065D+03
coeff(2171)=-0.845386688668D+03
coeff(2172)=-0.640260355739D+04
coeff(2173)=0.668184119493D+04
coeff(2174)=-0.476779936382D+04
coeff(2175)=0.557695177828D+04
coeff(2176)=0.649752019288D+04
coeff(2177)=0.655867722357D+04
coeff(2178)=0.465203844201D+04
coeff(2179)=-0.499718541289D+04
coeff(2180)=0.524702329220D+04
coeff(2181)=-0.418867912259D+04
coeff(2182)=-0.176667978336D+03
coeff(2183)=0.389487410266D+04
coeff(2184)=-0.458300032274D+03
coeff(2185)=0.554724286253D+04
coeff(2186)=-0.486206524050D+04
coeff(2187)=-0.308515585963D+04
coeff(2188)=0.138617206513D+04
coeff(2189)=0.340312069260D+04
coeff(2190)=0.336473176990D+04
coeff(2191)=-0.558696348721D+04
coeff(2192)=0.182310244338D+04
coeff(2193)=-0.106930233945D+05
coeff(2194)=0.167706681919D+04
coeff(2195)=0.390783680877D+04
coeff(2196)=0.637953872749D+04
coeff(2197)=0.406727393812D+04
coeff(2198)=0.587840058706D+04
coeff(2199)=-0.735836615977D+02
coeff(2200)=-0.332261694477D+04
coeff(2201)=0.194645857720D+05
coeff(2202)=-0.110762387283D+04
coeff(2203)=-0.160946877014D+05
coeff(2204)=0.340772798413D+04
coeff(2205)=0.486796895062D+03
coeff(2206)=-0.773103448452D+04
coeff(2207)=-0.555683367022D+04
coeff(2208)=-0.738425444944D+04
coeff(2209)=-0.120044867984D+05
coeff(2210)=0.180268483773D+04
coeff(2211)=-0.352003075178D+04
coeff(2212)=-0.168897979771D+05
coeff(2213)=-0.776739298139D+04
coeff(2214)=-0.413648812493D+04
coeff(2215)=-0.126373815879D+04
coeff(2216)=0.180534732004D+04
coeff(2217)=0.855509806468D+04
coeff(2218)=0.274966555502D+04
coeff(2219)=-0.116206915711D+05
coeff(2220)=0.284092664692D+03
coeff(2221)=0.651655204190D+04
coeff(2222)=0.106456472132D+04
coeff(2223)=-0.379239132023D+04
coeff(2224)=0.881858753505D+04
coeff(2225)=-0.284278932944D+03
coeff(2226)=0.271838894373D+04
coeff(2227)=-0.312116685888D+04
coeff(2228)=-0.608991232805D+04
coeff(2229)=-0.480581211440D+04
coeff(2230)=-0.662752460156D+04
coeff(2231)=-0.287080085273D+03
coeff(2232)=-0.283700826994D+04
coeff(2233)=0.244556664418D+04
coeff(2234)=-0.684809169353D+04
coeff(2235)=-0.192682407156D+04
coeff(2236)=-0.362513316172D+04
coeff(2237)=-0.602700574208D+04
coeff(2238)=0.736838292496D+04
coeff(2239)=0.843075322202D+04
coeff(2240)=-0.526550413479D+04
coeff(2241)=0.854361148400D+04
coeff(2242)=0.722424102577D+04
coeff(2243)=-0.741921003710D+04
coeff(2244)=0.323873879230D+04
coeff(2245)=0.771388977350D+04
coeff(2246)=0.172342731981D+05
coeff(2247)=-0.135756862345D+04
coeff(2248)=-0.983104180056D+04
coeff(2249)=-0.647572065397D+04
coeff(2250)=0.258492420569D+04
coeff(2251)=-0.439505717989D+03
coeff(2252)=-0.230294564247D+03
coeff(2253)=-0.843944849732D+03
coeff(2254)=0.460058070183D+04
coeff(2255)=0.108144968094D+04
coeff(2256)=-0.841700066764D+03
coeff(2257)=-0.415302271310D+04
coeff(2258)=-0.176376882608D+05
coeff(2259)=0.849647652564D+04
coeff(2260)=-0.148602446268D+04
coeff(2261)=-0.167400258742D+05
coeff(2262)=0.190589390211D+05
coeff(2263)=-0.290739419386D+04
coeff(2264)=0.192473545901D+05
coeff(2265)=0.122318412359D+05
coeff(2266)=0.673751646134D+03
coeff(2267)=0.108587473939D+05
coeff(2268)=-0.489800212989D+04
coeff(2269)=0.125859751037D+05
coeff(2270)=-0.626365224458D+04
coeff(2271)=-0.914656190823D+04
coeff(2272)=0.734355882088D+04
coeff(2273)=-0.374309424915D+04
coeff(2274)=0.155064319292D+05
coeff(2275)=0.635369872989D+04
coeff(2276)=0.950219454627D+04
coeff(2277)=0.107214767996D+04
coeff(2278)=0.509469781668D+04
coeff(2279)=0.588660734311D+04
coeff(2280)=-0.634597235213D+04
coeff(2281)=0.107361112844D+04
coeff(2282)=0.378439528480D+04
coeff(2283)=0.258145641724D+04
coeff(2284)=-0.224282719819D+04
coeff(2285)=-0.130056640054D+05
coeff(2286)=0.229081546098D+04
coeff(2287)=0.989220336444D+04
coeff(2288)=-0.855236049668D+03
coeff(2289)=-0.450689588446D+04
coeff(2290)=-0.402580219426D+04
coeff(2291)=0.873352639281D+04
coeff(2292)=0.292021895776D+04
coeff(2293)=-0.334618794698D+04
coeff(2294)=0.377849128660D+04
coeff(2295)=0.847019061075D+04
coeff(2296)=-0.368487811133D+04
coeff(2297)=0.219662894226D+04
coeff(2298)=0.189071996754D+04
coeff(2299)=-0.159352266909D+04
coeff(2300)=0.123134803417D+04
coeff(2301)=0.921371625020D+04
coeff(2302)=-0.786583447696D+04
coeff(2303)=0.864844911222D+04
coeff(2304)=-0.613543424451D+03
coeff(2305)=-0.528789346519D+04
coeff(2306)=0.232914385778D+04
coeff(2307)=0.199379971756D+04
coeff(2308)=-0.369210334868D+04
coeff(2309)=0.415869593965D+04
coeff(2310)=0.872017415235D+03
coeff(2311)=-0.360742724988D+04
coeff(2312)=0.454108914943D+04
coeff(2313)=-0.946083537922D+03
coeff(2314)=-0.901251203045D+04
coeff(2315)=0.861147220829D+04
coeff(2316)=0.365112831765D+04
coeff(2317)=-0.512528684601D+04
coeff(2318)=-0.803104040750D+04
coeff(2319)=-0.379791949755D+04
coeff(2320)=0.367454513294D+04
coeff(2321)=-0.118730670383D+05
coeff(2322)=0.125958579993D+05
coeff(2323)=0.519192100294D+04
coeff(2324)=-0.890112196719D+04
coeff(2325)=-0.625188743975D+04
coeff(2326)=-0.395553056700D+03
coeff(2327)=-0.150426720837D+04
coeff(2328)=-0.715826591105D+04
coeff(2329)=0.229550909297D+04
coeff(2330)=-0.102692121949D+05
coeff(2331)=-0.568171347380D+04
coeff(2332)=-0.208886181342D+05
coeff(2333)=0.113966677934D+05
coeff(2334)=-0.107289804135D+05
coeff(2335)=-0.344484176099D+04
coeff(2336)=0.434480872295D+04
coeff(2337)=-0.142869203668D+04
coeff(2338)=-0.860957215620D+03
coeff(2339)=-0.706243033780D+04
coeff(2340)=-0.729087517429D+04
coeff(2341)=0.136055969607D+05
coeff(2342)=0.992611075205D+03
coeff(2343)=0.452923430855D+04
coeff(2344)=0.263997949113D+04
coeff(2345)=0.107555702302D+05
coeff(2346)=0.849747976479D+03
coeff(2347)=0.157159041318D+04
coeff(2348)=0.104715725197D+04
coeff(2349)=0.160087925850D+05
coeff(2350)=0.307948976109D+04
coeff(2351)=0.535255951244D+04
coeff(2352)=-0.496747686721D+04
coeff(2353)=-0.566985652253D+04
coeff(2354)=-0.324601937656D+04
coeff(2355)=0.664594080008D+04
coeff(2356)=-0.701898359720D+04
coeff(2357)=0.236996657683D+03
coeff(2358)=-0.106417256151D+05
coeff(2359)=-0.108537208600D+05
coeff(2360)=-0.155858035272D+05
coeff(2361)=0.107083728938D+04
coeff(2362)=0.439427962371D+03
coeff(2363)=0.147943098384D+05
coeff(2364)=-0.164401458008D+05
coeff(2365)=-0.171875002083D+05
coeff(2366)=-0.241531104548D+03
coeff(2367)=-0.452856042792D+04
coeff(2368)=0.335542068088D+04
coeff(2369)=-0.542367378784D+04
coeff(2370)=0.152312503883D+05
coeff(2371)=-0.113789425821D+04
coeff(2372)=-0.406262099688D+04
coeff(2373)=0.233081762042D+04
coeff(2374)=-0.258867020762D+04
coeff(2375)=-0.738775237911D+04
coeff(2376)=-0.510571080076D+04
coeff(2377)=0.328761587604D+04
coeff(2378)=-0.219133824383D+04
coeff(2379)=-0.118065164302D+05
coeff(2380)=-0.351245238817D+04
coeff(2381)=-0.187996365027D+04
coeff(2382)=0.212447110850D+05
coeff(2383)=-0.646548258069D+04
coeff(2384)=0.107038674808D+05
coeff(2385)=-0.407548342516D+03
coeff(2386)=-0.369121156707D+04
coeff(2387)=0.191721102729D+05
coeff(2388)=0.271952422134D+04
coeff(2389)=0.651244103875D+04
coeff(2390)=0.231428321823D+04
coeff(2391)=0.987920943463D+02
coeff(2392)=-0.186687011609D+04
coeff(2393)=0.601492385707D+04
coeff(2394)=-0.300858477519D+04
coeff(2395)=0.930781574875D+03
coeff(2396)=-0.109533725467D+05
coeff(2397)=-0.527415014322D+04
coeff(2398)=0.101364140928D+05
coeff(2399)=0.558615509200D+04
coeff(2400)=-0.334865257707D+04
coeff(2401)=0.399608989504D+04
coeff(2402)=-0.439303711083D+04
coeff(2403)=-0.720044116547D+04
coeff(2404)=-0.744251210889D+04
coeff(2405)=0.467671824356D+04
coeff(2406)=-0.436182040297D+03
coeff(2407)=0.125575115811D+05
coeff(2408)=0.481162995290D+04
coeff(2409)=0.445822292789D+04
coeff(2410)=0.243916437978D+04
coeff(2411)=-0.137344174720D+05
coeff(2412)=-0.249501318781D+04
coeff(2413)=-0.901469288270D+04
coeff(2414)=0.114324232279D+05
coeff(2415)=0.195289246164D+05
coeff(2416)=-0.580781641325D+04
coeff(2417)=0.146676668115D+04
coeff(2418)=-0.257216306888D+04
coeff(2419)=0.478182015489D+03
coeff(2420)=-0.797520815793D+03
coeff(2421)=-0.117136245469D+04
coeff(2422)=0.547941335741D+03
coeff(2423)=0.400085528885D+04
coeff(2424)=-0.137544872477D+03
coeff(2425)=0.158680615701D+05
coeff(2426)=0.121628548288D+05
coeff(2427)=-0.137293473350D+05
coeff(2428)=0.140113514571D+05
coeff(2429)=-0.424035566122D+03
coeff(2430)=0.404599183783D+03
coeff(2431)=0.287083099853D+04
coeff(2432)=-0.114436418330D+04
coeff(2433)=0.586238742615D+04
coeff(2434)=0.619782692185D+04
coeff(2435)=0.764900138923D+02
coeff(2436)=-0.114066219607D+04
coeff(2437)=0.624007962122D+04
coeff(2438)=0.124507086324D+05
coeff(2439)=-0.214630588513D+04
coeff(2440)=0.255773710701D+04
coeff(2441)=0.442830221626D+04
coeff(2442)=-0.143430936514D+05
coeff(2443)=0.385409827376D+04
coeff(2444)=0.472368547142D+04
coeff(2445)=0.404347927458D+04
coeff(2446)=0.559724377425D+04
coeff(2447)=-0.129295051880D+04
coeff(2448)=0.120329086245D+05
coeff(2449)=-0.215600330554D+05
coeff(2450)=0.181854522849D+05
coeff(2451)=0.635523328872D+04
coeff(2452)=-0.645725336655D+04
coeff(2453)=-0.129116103550D+05
coeff(2454)=0.864786970966D+04
coeff(2455)=-0.304891714287D+04
coeff(2456)=-0.985345893531D+04
coeff(2457)=-0.114094387710D+05
coeff(2458)=0.387218638856D+04
coeff(2459)=0.174983172001D+05
coeff(2460)=-0.826310800012D+04
coeff(2461)=0.737505104100D+04
coeff(2462)=-0.724581223587D+04
coeff(2463)=0.165206397019D+04
coeff(2464)=0.516609503380D+03
coeff(2465)=-0.203566402191D+04
coeff(2466)=0.203321381340D+04
coeff(2467)=-0.251043052753D+04
coeff(2468)=0.809138384000D+04
coeff(2469)=-0.543030253728D+04
coeff(2470)=0.114070747916D+04
coeff(2471)=-0.132633337697D+05
coeff(2472)=0.297030553365D+03
coeff(2473)=0.187489506161D+04
coeff(2474)=-0.754182950937D+04
coeff(2475)=0.209531511577D+04
coeff(2476)=0.950055034614D+04
coeff(2477)=0.122731716672D+05
coeff(2478)=-0.347977848091D+04
coeff(2479)=0.521513386393D+04
coeff(2480)=0.447543236279D+04
coeff(2481)=-0.942512343257D+04
coeff(2482)=0.843017673471D+04
coeff(2483)=-0.140832375711D+05
coeff(2484)=-0.230502543616D+04
coeff(2485)=-0.600538168977D+04
coeff(2486)=0.148851454767D+05
coeff(2487)=0.207560714570D+03
coeff(2488)=-0.830420671245D+04
coeff(2489)=-0.839912913628D+04
coeff(2490)=0.365703321987D+04
coeff(2491)=0.266296835801D+04
coeff(2492)=0.732095226084D+04
coeff(2493)=-0.867835951493D+04
coeff(2494)=-0.245951620642D+04
coeff(2495)=-0.819897292819D+04
coeff(2496)=-0.720560961358D+03
coeff(2497)=0.366013923104D+04
coeff(2498)=-0.419889945762D+04
coeff(2499)=0.277186272938D+04
coeff(2500)=0.459290651764D+04
coeff(2501)=-0.115551588365D+04
coeff(2502)=0.550752132826D+04
coeff(2503)=-0.627601304653D+04
coeff(2504)=0.470974894898D+04
coeff(2505)=-0.411730089549D+04
coeff(2506)=0.103966016257D+04
coeff(2507)=0.358590450157D+04
coeff(2508)=0.724038091102D+04
coeff(2509)=-0.879565385961D+04
coeff(2510)=0.146322365196D+05
coeff(2511)=0.273382641894D+04
coeff(2512)=-0.348094937077D+04
coeff(2513)=0.491667299003D+04
coeff(2514)=-0.596789967479D+04
coeff(2515)=0.453662580912D+04
coeff(2516)=0.413138656824D+04
coeff(2517)=-0.522448436743D+04
coeff(2518)=-0.111599595392D+05
coeff(2519)=-0.250013729394D+04
coeff(2520)=0.111937827375D+05
coeff(2521)=-0.214902624900D+04
coeff(2522)=0.703348839757D+04
coeff(2523)=-0.201739358469D+04
coeff(2524)=0.141746759283D+05
coeff(2525)=-0.394564081018D+04
coeff(2526)=-0.267745947120D+04
coeff(2527)=0.111397822948D+05
coeff(2528)=0.141888890384D+05
coeff(2529)=-0.216535842003D+04
coeff(2530)=-0.632346904770D+04
coeff(2531)=0.992758177543D+04
coeff(2532)=0.492779464016D+04
coeff(2533)=0.893557927267D+04
coeff(2534)=-0.507905047574D+02
coeff(2535)=-0.213818225778D+04
coeff(2536)=0.410479477293D+04
coeff(2537)=0.906471851023D+04
coeff(2538)=-0.113701257105D+05
coeff(2539)=0.156262402726D+04
coeff(2540)=0.543137567677D+04
coeff(2541)=-0.742491634313D+04
coeff(2542)=-0.159665141152D+05
coeff(2543)=-0.106353074454D+05
coeff(2544)=-0.154526411250D+05
coeff(2545)=0.117895674147D+04
coeff(2546)=0.246027792511D+05
coeff(2547)=0.617756663319D+04
coeff(2548)=-0.115338321686D+05
coeff(2549)=-0.435730716763D+04
coeff(2550)=0.941003635749D+04
coeff(2551)=-0.361844612470D+03
coeff(2552)=-0.443601883619D+04
coeff(2553)=0.767434338468D+04
coeff(2554)=-0.148834520060D+05
coeff(2555)=0.437097463342D+04
coeff(2556)=0.669260106317D+04
coeff(2557)=0.116277678779D+05
coeff(2558)=0.496295740845D+03
coeff(2559)=0.688037963870D+04
coeff(2560)=0.108704913734D+05
coeff(2561)=-0.120464581601D+05
coeff(2562)=-0.437999739876D+04
coeff(2563)=0.800063034293D+02
coeff(2564)=0.230840792390D+04
coeff(2565)=-0.516224190138D+04
coeff(2566)=0.680907936179D+04
coeff(2567)=-0.953795121864D+04
coeff(2568)=0.853354495267D+03
coeff(2569)=0.113882445865D+05
coeff(2570)=-0.384603081992D+04
coeff(2571)=0.508770403077D+03
coeff(2572)=-0.647364356151D+04
coeff(2573)=-0.253769122464D+04
coeff(2574)=0.186454025653D+05
coeff(2575)=0.272468159143D+04
coeff(2576)=-0.116227578261D+05
coeff(2577)=-0.569262669561D+04
coeff(2578)=-0.664906098549D+04
coeff(2579)=0.956186812112D+04
coeff(2580)=-0.105914737227D+05
coeff(2581)=0.491333007968D+04
coeff(2582)=-0.490157111881D+04
coeff(2583)=-0.897158247999D+04
coeff(2584)=-0.123115281751D+04
coeff(2585)=-0.136029928646D+05
coeff(2586)=-0.699702918790D+04
coeff(2587)=-0.485751531066D+03
coeff(2588)=-0.563563037502D+04
coeff(2589)=-0.138197011422D+03
coeff(2590)=0.428841246390D+04
coeff(2591)=-0.293535782584D+04
coeff(2592)=-0.322822491780D+04
coeff(2593)=-0.511225160223D+04
coeff(2594)=0.618738416419D+04
coeff(2595)=0.201781442739D+05
coeff(2596)=-0.175228901441D+04
coeff(2597)=-0.643131921891D+04
coeff(2598)=-0.506095194070D+04
coeff(2599)=0.139416503625D+04
coeff(2600)=-0.841489063911D+04
coeff(2601)=0.866651814487D+04
coeff(2602)=-0.801039273819D+04
coeff(2603)=-0.410336110999D+04
coeff(2604)=-0.387003778482D+04
coeff(2605)=0.113981353029D+05
coeff(2606)=0.853310332023D+04
coeff(2607)=-0.930494791942D+04
coeff(2608)=-0.144942346284D+05
coeff(2609)=-0.118132263819D+04
coeff(2610)=-0.500955974924D+04
coeff(2611)=0.549733466843D+03
coeff(2612)=-0.120207445849D+05
coeff(2613)=-0.262647704922D+04
coeff(2614)=0.483270073149D+04
coeff(2615)=-0.982408168313D+04
coeff(2616)=0.364412834538D+02
coeff(2617)=0.233958513709D+05
coeff(2618)=0.161640201198D+05
coeff(2619)=-0.603842782320D+04
coeff(2620)=0.416915365604D+04
coeff(2621)=-0.696250583501D+04
coeff(2622)=0.168254313709D+05
coeff(2623)=0.467684177152D+04
coeff(2624)=0.526800970355D+04
coeff(2625)=0.516890420447D+04
coeff(2626)=0.688168665801D+03
coeff(2627)=-0.342868569491D+04
coeff(2628)=-0.202781038294D+05
coeff(2629)=0.134765799697D+05
coeff(2630)=-0.798595641997D+04
coeff(2631)=0.247407037051D+03
coeff(2632)=-0.100280711777D+05
coeff(2633)=-0.180002639611D+04
coeff(2634)=0.488220310021D+04
coeff(2635)=0.754287805630D+04
coeff(2636)=0.255583941245D+04
coeff(2637)=0.544858651357D+04
coeff(2638)=-0.219424168565D+04
coeff(2639)=-0.434569652748D+03
coeff(2640)=-0.801794982602D+04
coeff(2641)=0.971316270959D+04
coeff(2642)=-0.126113340281D+05
coeff(2643)=0.665252171559D+04
coeff(2644)=0.988608619828D+04
coeff(2645)=-0.112728834611D+05
coeff(2646)=-0.303448288698D+04
coeff(2647)=-0.836927384050D+04
coeff(2648)=0.253979326710D+04
coeff(2649)=0.124999464793D+04
coeff(2650)=0.191101547163D+05
coeff(2651)=0.115269405097D+05
coeff(2652)=-0.501939661463D+04
coeff(2653)=-0.644269454366D+04
coeff(2654)=-0.390225894562D+04
coeff(2655)=0.114163131953D+05
coeff(2656)=0.149036106252D+05
coeff(2657)=-0.307553567111D+04
coeff(2658)=-0.614845911340D+04
coeff(2659)=-0.267402994102D+04
coeff(2660)=-0.117119325120D+05
coeff(2661)=-0.427111567293D+04
coeff(2662)=0.580266262172D+04
coeff(2663)=-0.113038067893D+05
coeff(2664)=0.110807672599D+05
coeff(2665)=-0.247014426339D+04
coeff(2666)=-0.740185887935D+03
coeff(2667)=0.663668825817D+03
coeff(2668)=0.100027879550D+05
coeff(2669)=-0.267738672406D+04
coeff(2670)=-0.514205329016D+03
coeff(2671)=-0.546092694312D+04
coeff(2672)=0.672402277676D+04
coeff(2673)=0.249576867354D+04
coeff(2674)=0.153391759903D+05
coeff(2675)=-0.883804936417D+04
coeff(2676)=0.898622938720D+04
coeff(2677)=-0.666034752917D+04
coeff(2678)=-0.118076590001D+05
coeff(2679)=-0.142106464113D+05
coeff(2680)=0.130717395110D+05
coeff(2681)=0.143747143221D+04
coeff(2682)=-0.122268855918D+04
coeff(2683)=-0.184397111321D+05
coeff(2684)=-0.268855565597D+04
coeff(2685)=-0.416091564964D+04
coeff(2686)=-0.471443915102D+04
coeff(2687)=0.150717570051D+04
coeff(2688)=0.240453291254D+04
coeff(2689)=-0.516021415276D+03
coeff(2690)=-0.133401273493D+05
coeff(2691)=0.149867432364D+05
coeff(2692)=0.792093691265D+04
coeff(2693)=0.200619849768D+04
coeff(2694)=-0.105961268973D+05
coeff(2695)=0.115671952486D+05
coeff(2696)=-0.148175504857D+05
coeff(2697)=0.997844434126D+04
coeff(2698)=0.276457351773D+04
coeff(2699)=0.106011590707D+05
coeff(2700)=-0.176501544935D+05
coeff(2701)=0.216935140962D+04
coeff(2702)=-0.964483997545D+04
coeff(2703)=-0.855674686227D+04
coeff(2704)=-0.693932479223D+04
coeff(2705)=-0.191955334296D+04
coeff(2706)=-0.886635316105D+04
coeff(2707)=0.297309363654D+04
coeff(2708)=0.737854425190D+04
coeff(2709)=0.525141147413D+04
coeff(2710)=0.206500640520D+04
coeff(2711)=-0.856491872413D+04
coeff(2712)=-0.107990427843D+05
coeff(2713)=0.577024177988D+04
coeff(2714)=-0.376982298307D+03
coeff(2715)=-0.489161778838D+04
coeff(2716)=0.102068228220D+05
coeff(2717)=-0.801093344719D+04
coeff(2718)=-0.823951892775D+04
coeff(2719)=0.490831549237D+04
coeff(2720)=-0.461674662121D+04
coeff(2721)=-0.478152439540D+03
coeff(2722)=0.580367963506D+04
coeff(2723)=0.752883751135D+04
coeff(2724)=-0.424276092846D+04
coeff(2725)=0.377668838024D+04
coeff(2726)=0.140830259337D+05
coeff(2727)=-0.690043719576D+04
coeff(2728)=-0.103276029125D+05
coeff(2729)=-0.282870195343D+04
coeff(2730)=-0.379791392172D+04
coeff(2731)=0.132695957252D+05
coeff(2732)=-0.130074590344D+05
coeff(2733)=0.527033036852D+04
coeff(2734)=0.923225667263D+04
coeff(2735)=0.601242985809D+04
coeff(2736)=-0.897144643463D+04
coeff(2737)=0.675827191001D+04
coeff(2738)=0.657740806825D+03
coeff(2739)=0.130975204598D+05
coeff(2740)=0.151559458031D+05
coeff(2741)=0.908789775830D+04
coeff(2742)=-0.108624498414D+05
coeff(2743)=0.515602252411D+04
coeff(2744)=0.746587548833D+04
coeff(2745)=0.530330752960D+04
coeff(2746)=0.104138504168D+04
coeff(2747)=0.137531045929D+04
coeff(2748)=-0.125342599733D+04
coeff(2749)=0.440748104389D+04
coeff(2750)=-0.433930292514D+04
coeff(2751)=0.345687905695D+04
coeff(2752)=-0.621746384705D+04
coeff(2753)=-0.975170654359D+04
coeff(2754)=-0.808954110299D+04
coeff(2755)=-0.155248965075D+04
coeff(2756)=0.140164418611D+05
coeff(2757)=0.519495609462D+03
coeff(2758)=-0.115634430577D+05
coeff(2759)=-0.996053060832D+03
coeff(2760)=0.127792939784D+05
coeff(2761)=-0.319736200378D+03
coeff(2762)=-0.152827348145D+04
coeff(2763)=-0.473070366299D+04
coeff(2764)=-0.952916396964D+04
coeff(2765)=0.870800577184D+04
coeff(2766)=0.428190160061D+04
coeff(2767)=0.473758784835D+04
coeff(2768)=0.373376401520D+04
coeff(2769)=-0.238746663335D+04
coeff(2770)=0.991542779059D+03
coeff(2771)=0.645560079448D+04
coeff(2772)=0.637864938861D+04
coeff(2773)=0.808715006788D+03
coeff(2774)=0.401102839405D+04
coeff(2775)=0.476783336684D+04
coeff(2776)=0.665716898419D+04
coeff(2777)=0.345066554621D+04
coeff(2778)=0.355270154514D+04
coeff(2779)=-0.469645828908D+04
coeff(2780)=0.363194879240D+04
coeff(2781)=-0.654562770718D+04
coeff(2782)=0.346383360031D+04
coeff(2783)=0.563798295485D+04
coeff(2784)=-0.140632653805D+05
coeff(2785)=-0.668179257171D+04
coeff(2786)=-0.241276782241D+05
coeff(2787)=-0.210191362557D+04
coeff(2788)=0.129502770076D+05
coeff(2789)=0.969616280747D+04
coeff(2790)=0.912284021541D+04
coeff(2791)=0.348091571048D+04
coeff(2792)=-0.399947959250D+03
coeff(2793)=-0.161360349718D+04
coeff(2794)=-0.483785947334D+04
coeff(2795)=0.869427418427D+04
coeff(2796)=0.108945451539D+05
coeff(2797)=-0.136005471616D+04
coeff(2798)=0.656381793531D+04
coeff(2799)=-0.779073740342D+04
coeff(2800)=-0.107575733691D+05
coeff(2801)=0.634434035401D+04
coeff(2802)=0.190941325587D+05
coeff(2803)=-0.150358004701D+05
coeff(2804)=-0.576037097272D+04
coeff(2805)=0.979959240746D+04
coeff(2806)=-0.743640475581D+04
coeff(2807)=-0.117981954559D+05
coeff(2808)=-0.552676529709D+04
coeff(2809)=0.136045537443D+05
coeff(2810)=-0.197732741965D+04
coeff(2811)=-0.128546638169D+04
coeff(2812)=0.515165603094D+04
coeff(2813)=-0.378277270360D+04
coeff(2814)=-0.164210111099D+04
coeff(2815)=-0.167868753811D+04
coeff(2816)=-0.958613393884D+04
coeff(2817)=-0.153241837609D+05
coeff(2818)=-0.108800616764D+04
coeff(2819)=-0.118022399475D+04
coeff(2820)=-0.180626610057D+04
coeff(2821)=0.105671900418D+05
coeff(2822)=-0.420300914002D+04
coeff(2823)=-0.132382094084D+05
coeff(2824)=0.506762603650D+04
coeff(2825)=-0.418056790740D+04
coeff(2826)=0.210640700374D+05
coeff(2827)=0.679055260818D+04
coeff(2828)=0.824001689076D+04
coeff(2829)=-0.544422228478D+03
coeff(2830)=-0.353179929676D+03
coeff(2831)=0.104799365585D+04
coeff(2832)=-0.152344614568D+05
coeff(2833)=-0.946824347081D+04
coeff(2834)=-0.746950209753D+04
coeff(2835)=0.421388223867D+04
coeff(2836)=-0.339856339477D+04
coeff(2837)=0.216359992102D+04
coeff(2838)=0.444507058278D+04
coeff(2839)=0.146994531577D+03
coeff(2840)=0.101740240044D+05
coeff(2841)=0.514714176147D+03
coeff(2842)=-0.209919546455D+04
coeff(2843)=-0.246321090336D+04
coeff(2844)=0.168032483006D+04
coeff(2845)=-0.118207976133D+04
coeff(2846)=-0.742396904390D+03
coeff(2847)=-0.674079791581D+04
coeff(2848)=-0.136474060026D+04
coeff(2849)=-0.384941881613D+04
coeff(2850)=-0.190681198550D+05
coeff(2851)=-0.125761511328D+05
coeff(2852)=0.559414605479D+04
coeff(2853)=-0.259857488292D+04
coeff(2854)=-0.153161223451D+05
coeff(2855)=0.596729152245D+04
coeff(2856)=0.756677693808D+02
coeff(2857)=0.474329129569D+04
coeff(2858)=-0.860871329075D+04
coeff(2859)=0.282360433952D+04
coeff(2860)=-0.596430026879D+04
coeff(2861)=0.125223127161D+04
coeff(2862)=-0.887748165992D+04
coeff(2863)=-0.122080376133D+04
coeff(2864)=0.118982457233D+04
coeff(2865)=-0.104857229001D+04
coeff(2866)=0.647203428924D+04
coeff(2867)=0.504654247578D+03
coeff(2868)=-0.448489755251D+04
coeff(2869)=0.264602312520D+04
coeff(2870)=-0.155823397001D+05
coeff(2871)=-0.164822125792D+05
coeff(2872)=-0.803547863522D+02
coeff(2873)=0.445120296150D+04
coeff(2874)=-0.662483345904D+03
coeff(2875)=0.187082607883D+04
coeff(2876)=-0.940869120069D+04
coeff(2877)=-0.426559803629D+04
coeff(2878)=-0.744580748565D+04
coeff(2879)=0.105507648565D+05
coeff(2880)=-0.100055778859D+05
coeff(2881)=-0.172316323280D+05
coeff(2882)=-0.754564976538D+04
coeff(2883)=-0.132217036729D+04
coeff(2884)=0.168111916003D+04
coeff(2885)=-0.435607193903D+04
coeff(2886)=0.369373603929D+04
coeff(2887)=0.990383789464D+03
coeff(2888)=0.151888684764D+04
coeff(2889)=0.464125881291D+04
coeff(2890)=0.645564071125D+04
coeff(2891)=0.256430241926D+04
coeff(2892)=-0.855724024500D+04
coeff(2893)=-0.495212869081D+04
coeff(2894)=0.665488389869D+04
coeff(2895)=-0.591887799500D+04
coeff(2896)=0.162572249521D+04
coeff(2897)=0.204517561709D+05
coeff(2898)=0.107560129301D+05
coeff(2899)=0.276475240810D+04
coeff(2900)=0.512492337561D+04
coeff(2901)=-0.698546006178D+04
coeff(2902)=-0.159978832302D+04
coeff(2903)=0.292252428732D+04
coeff(2904)=0.725825025933D+04
coeff(2905)=0.103807656761D+05
coeff(2906)=0.672299603308D+04
coeff(2907)=0.425711365178D+04
coeff(2908)=0.103953964036D+05
coeff(2909)=-0.203135264351D+05
coeff(2910)=-0.308844540460D+04
coeff(2911)=0.315189294840D+04
coeff(2912)=0.107648413450D+05
coeff(2913)=-0.155860521821D+05
coeff(2914)=0.297511502556D+04
coeff(2915)=0.344412573855D+04
coeff(2916)=-0.568904769457D+04
coeff(2917)=0.123874464667D+04
coeff(2918)=-0.207241878173D+04
coeff(2919)=-0.486141001507D+04
coeff(2920)=0.729565013629D+03
coeff(2921)=0.971142132447D+04
coeff(2922)=-0.129477462164D+05
coeff(2923)=-0.940368671895D+03
coeff(2924)=-0.420786997771D+04
coeff(2925)=0.635548037741D+03
coeff(2926)=0.503028085294D+04
coeff(2927)=0.868098188842D+04
coeff(2928)=-0.104555940060D+05
coeff(2929)=-0.246200084987D+04
coeff(2930)=0.353109284040D+04
coeff(2931)=-0.181513955071D+04
coeff(2932)=0.107307416397D+03
coeff(2933)=0.125167771936D+04
coeff(2934)=0.260111795518D+04
coeff(2935)=-0.364851353175D+04
coeff(2936)=-0.589046564021D+04
coeff(2937)=-0.952092919274D+04
coeff(2938)=0.838668596453D+04
coeff(2939)=-0.415690341991D+04
coeff(2940)=0.434101560799D+04
coeff(2941)=0.879085684267D+04
coeff(2942)=-0.135983160525D+05
coeff(2943)=-0.718745191837D+04
coeff(2944)=-0.102216029465D+03
coeff(2945)=-0.304848962988D+03
coeff(2946)=0.490979458814D+03
coeff(2947)=-0.344938289973D+04
coeff(2948)=-0.987956063977D+02
coeff(2949)=-0.639933654874D+03
coeff(2950)=-0.602104129022D+03
coeff(2951)=0.191401066315D+04
coeff(2952)=0.113414167115D+04
coeff(2953)=0.473422341021D+04
coeff(2954)=0.126370882150D+05
coeff(2955)=-0.106122354278D+05
coeff(2956)=0.999835024040D+04
coeff(2957)=-0.661662478035D+03
coeff(2958)=-0.977656156285D+03
coeff(2959)=0.146662693820D+03
coeff(2960)=-0.438384116921D+04
coeff(2961)=0.805891321635D+03
coeff(2962)=0.842589645203D+03
coeff(2963)=-0.941362509512D+03
coeff(2964)=0.394587009411D+03
coeff(2965)=-0.591860171874D+04
coeff(2966)=0.920059312543D+04
coeff(2967)=-0.457666906316D+04
coeff(2968)=-0.113963011670D+05
coeff(2969)=0.448029699853D+04
coeff(2970)=0.934316604747D+03
coeff(2971)=-0.613510150520D+04
coeff(2972)=-0.659414394878D+04
coeff(2973)=0.136260804468D+05
coeff(2974)=-0.915620484184D+04
coeff(2975)=0.832358262641D+03
coeff(2976)=0.599261555783D+04
coeff(2977)=0.335667072345D+04
coeff(2978)=-0.111378659363D+05
coeff(2979)=0.339882902623D+04
coeff(2980)=-0.282221838215D+04
coeff(2981)=0.381227253380D+04
coeff(2982)=0.387845298383D+04
coeff(2983)=-0.104323667688D+05
coeff(2984)=0.538700425518D+04
coeff(2985)=0.346043349484D+04
coeff(2986)=0.694813799387D+04
coeff(2987)=0.100579333189D+05
coeff(2988)=-0.956888827319D+04
coeff(2989)=0.108090840177D+05
coeff(2990)=-0.845251734073D+04
coeff(2991)=0.614162135603D+03
coeff(2992)=-0.369466156543D+04
coeff(2993)=0.109558232615D+04
coeff(2994)=-0.890912539741D+04
coeff(2995)=0.145184701539D+05
coeff(2996)=-0.349280184124D+04
coeff(2997)=0.402263014881D+04
coeff(2998)=0.746183195688D+04
coeff(2999)=0.297160509710D+03
coeff(3000)=0.162816850524D+05
coeff(3001)=0.106038050387D+05
coeff(3002)=-0.735708162672D+04
coeff(3003)=-0.116301123459D+05
coeff(3004)=-0.144233007208D+03
coeff(3005)=-0.463247342187D+04
coeff(3006)=-0.288948330950D+03
coeff(3007)=0.103763157398D+04
coeff(3008)=0.120238408052D+04
coeff(3009)=0.103679316514D+04
coeff(3010)=0.389757076471D+04
coeff(3011)=0.181603210190D+03
coeff(3012)=-0.581898734955D+04
coeff(3013)=0.938465375830D+04
coeff(3014)=-0.609151175208D+04
coeff(3015)=-0.261841069857D+04
coeff(3016)=-0.536242211603D+04
coeff(3017)=-0.858291047730D+04
coeff(3018)=0.208693837485D+04
coeff(3019)=0.716605479987D+04
coeff(3020)=-0.372190338253D+04
coeff(3021)=-0.704626801249D+04
coeff(3022)=-0.513952881342D+04
coeff(3023)=0.120162901753D+05
coeff(3024)=0.122606067313D+05
coeff(3025)=0.390377375363D+04
coeff(3026)=-0.241095488456D+04
coeff(3027)=-0.154057949507D+04
coeff(3028)=-0.958093385628D+04
coeff(3029)=-0.144365708750D+04
coeff(3030)=0.354338327271D+04
coeff(3031)=0.559271507152D+04
coeff(3032)=-0.399567356581D+04
coeff(3033)=-0.106395663644D+03
coeff(3034)=0.446122583621D+04
coeff(3035)=-0.553198418501D+04
coeff(3036)=0.125976474444D+04
coeff(3037)=-0.139956793631D+03
coeff(3038)=-0.758742270506D+03
coeff(3039)=0.365055788574D+04
coeff(3040)=0.862718313760D+04
coeff(3041)=0.203880072571D+03
coeff(3042)=-0.431919606706D+04
coeff(3043)=-0.532616141733D+04
coeff(3044)=0.951978409353D+04
coeff(3045)=-0.335728223861D+04
coeff(3046)=-0.288093064032D+03
coeff(3047)=-0.169445099807D+04
coeff(3048)=0.115416540610D+05
coeff(3049)=0.961010530525D+03
coeff(3050)=0.782362986665D+04
coeff(3051)=0.946336174095D+04
coeff(3052)=-0.125156240309D+03
coeff(3053)=-0.300615737287D+04
coeff(3054)=0.124176354015D+04
coeff(3055)=0.677281024944D+04
coeff(3056)=0.356253448083D+04
coeff(3057)=-0.133655923915D+04
coeff(3058)=0.843325848664D+04
coeff(3059)=-0.441452139007D+04
coeff(3060)=-0.585045404066D+04
coeff(3061)=-0.962176393305D+04
coeff(3062)=-0.202594905257D+04
coeff(3063)=0.201668467269D+04
coeff(3064)=-0.967897118394D+04
coeff(3065)=-0.470442354947D+03
coeff(3066)=-0.753482112043D+04
coeff(3067)=0.257254974279D+04
coeff(3068)=0.320224115745D+04
coeff(3069)=-0.301810378477D+04
coeff(3070)=-0.658456548452D+04
coeff(3071)=0.381023977154D+04
coeff(3072)=0.781053171351D+03
coeff(3073)=0.168237890692D+03
coeff(3074)=0.346953457933D+04
coeff(3075)=0.868229972210D+03

! Calculate MEG
x(1)=exp(-((r(1)-ra1))/a1 -((r(1)-rb1)**2)/b1)
x(2)=exp(-((r(2)-ra1))/a1 -((r(2)-rb1)**2)/b1)
x(3)=exp(-((r(3)-ra2))/a2 -((r(3)-rb2)**2)/b2)
x(4)=exp(-((r(4)-ra1))/a1 -((r(4)-rb1)**2)/b1)
x(5)=exp(-((r(5)-ra2))/a2 -((r(5)-rb2)**2)/b2)
x(6)=exp(-((r(6)-ra2))/a2 -((r(6)-rb2)**2)/b2)

! Calculate dMEG/dr(6,6) for giving R(6)
! dmsdr(i,j)=0  i!=j
! Initialize dmsdr
      do i=1,6
        do j=1,6
          dmsdr(i,j)=0.0d0
        enddo
      enddo
dmsdr(1,1)=(-2.0d0*(r(1)-rb1)/b1-1/a1)*Exp(-(r(1)-ra1)/a1-((r(1)-rb1)**2.0d0)/b1)
dmsdr(2,2)=(-2.0d0*(r(2)-rb1)/b1-1/a1)*Exp(-(r(2)-ra1)/a1-((r(2)-rb1)**2.0d0)/b1)
dmsdr(3,3)=(-2.0d0*(r(3)-rb2)/b2-1/a2)*Exp(-(r(3)-ra2)/a2-((r(3)-rb2)**2.0d0)/b2)
dmsdr(4,4)=(-2.0d0*(r(4)-rb1)/b1-1/a1)*Exp(-(r(4)-ra1)/a1-((r(4)-rb1)**2.0d0)/b1)
dmsdr(5,5)=(-2.0d0*(r(5)-rb2)/b2-1/a2)*Exp(-(r(5)-ra2)/a2-((r(5)-rb2)**2.0d0)/b2)
dmsdr(6,6)=(-2.0d0*(r(6)-rb2)/b2-1/a2)*Exp(-(r(6)-ra2)/a2-((r(6)-rb2)**2.0d0)/b2)

rm(0) = 1.0D0
rm(1) = x(4)
rm(2) = x(2)
rm(3) = x(1)
rm(4) = x(6)
rm(5) = x(5)
rm(6) = x(3)
rm(7) = rm(1)*rm(2)
rm(8) = rm(1)*rm(3)
rm(9) = rm(2)*rm(3)
rm(10) = rm(3)*rm(4)
rm(11) = rm(2)*rm(5)
rm(12) = rm(1)*rm(6)
rm(13) = rm(4)*rm(5)
rm(14) = rm(4)*rm(6)
rm(15) = rm(5)*rm(6)
rm(16) = rm(1)*rm(9)
rm(17) = rm(1)*rm(10)
rm(18) = rm(2)*rm(10)
rm(19) = rm(1)*rm(11)
rm(20) = rm(3)*rm(11)
rm(21) = rm(2)*rm(12)
rm(22) = rm(3)*rm(12)
rm(23) = rm(2)*rm(13)
rm(24) = rm(3)*rm(13)
rm(25) = rm(1)*rm(14)
rm(26) = rm(3)*rm(14)
rm(27) = rm(1)*rm(15)
rm(28) = rm(2)*rm(15)
rm(29) = rm(4)*rm(15)
rm(30) = rm(2)*rm(24)
rm(31) = rm(1)*rm(26)
rm(32) = rm(1)*rm(28)

p(0) = rm(0)
p(1) = rm(1) + rm(2) + rm(3)
p(2) = rm(4) + rm(5) + rm(6)
p(3) = rm(7) + rm(8) + rm(9)
p(4) = rm(10) + rm(11) + rm(12)
p(5) = p(1)*p(2) - p(4)
p(6) = rm(13) + rm(14) + rm(15)
p(7) = p(1)*p(1) - p(3) - p(3)
p(8) = p(2)*p(2) - p(6) - p(6)
p(9) = rm(16)
p(10) = rm(17) + rm(18) + rm(19) + rm(20) + rm(21) + rm(22)
p(11) = p(2)*p(3) - p(10)
p(12) = rm(23) + rm(24) + rm(25) + rm(26) + rm(27) + rm(28)
p(13) = p(1)*p(6) - p(12)
p(14) = rm(29)
p(15) = p(1)*p(3) - p(9) - p(9) - p(9)
p(16) = p(1)*p(4) - p(10)
p(17) = p(2)*p(7) - p(16)
p(18) = p(2)*p(4) - p(12)
p(19) = p(1)*p(8) - p(18)
p(20) = p(2)*p(6) - p(14) - p(14) - p(14)
p(21) = p(1)*p(7) - p(15)
p(22) = p(2)*p(8) - p(20)
p(23) = p(9)*p(2)
p(24) = rm(30) + rm(31) + rm(32)
p(25) = p(3)*p(6) - p(24)
p(26) = p(14)*p(1)
p(27) = p(9)*p(1)
p(28) = p(3)*p(4) - p(23)
p(29) = p(1)*p(10) - p(23) - p(28) - p(23)
p(30) = p(1)*p(11) - p(23)
p(31) = p(1)*p(12) - p(25) - p(24) - p(24)
p(32) = p(1)*p(13) - p(25)
p(33) = p(4)*p(5) - p(25) - p(31)
p(34) = p(2)*p(11) - p(25)
p(35) = p(4)*p(6) - p(26)
p(36) = p(2)*p(12) - p(26) - p(35) - p(26)
p(37) = p(2)*p(13) - p(26)
p(38) = p(14)*p(2)
p(39) = p(3)*p(3) - p(27) - p(27)
p(40) = p(3)*p(7) - p(27)
p(41) = p(1)*p(16) - p(28)
p(42) = p(2)*p(21) - p(41)
p(43) = p(1)*p(18) - p(33)
p(44) = p(7)*p(8) - p(43)
p(45) = p(6)*p(6) - p(38) - p(38)
p(46) = p(2)*p(18) - p(35)
p(47) = p(1)*p(22) - p(46)
p(48) = p(6)*p(8) - p(38)
p(49) = p(1)*p(21) - p(40)
p(50) = p(2)*p(22) - p(48)
p(51) = p(9)*p(6)
p(52) = p(14)*p(3)
p(53) = p(9)*p(4)
p(54) = p(9)*p(5)
p(55) = p(1)*p(24) - p(51)
p(56) = p(3)*p(12) - p(51) - p(55) - p(51)
p(57) = p(3)*p(13) - p(51)
p(58) = p(14)*p(7)
p(59) = p(9)*p(8)
p(60) = p(2)*p(24) - p(52)
p(61) = p(4)*p(13) - p(58)
p(62) = p(6)*p(11) - p(52)
p(63) = p(14)*p(4)
p(64) = p(14)*p(5)
p(65) = p(9)*p(3)
p(66) = p(9)*p(7)
p(67) = p(3)*p(16) - p(53)
p(68) = p(4)*p(15) - p(54) - p(67)
p(69) = p(2)*p(39) - p(68)
p(70) = p(1)*p(29) - p(54) - p(68)
p(71) = p(7)*p(11) - p(53)
p(72) = p(1)*p(31) - p(56) - p(55)
p(73) = p(1)*p(32) - p(57)
p(74) = p(3)*p(18) - p(59)
p(75) = p(2)*p(29) - p(57) - p(55)
p(76) = p(1)*p(34) - p(59)
p(77) = p(6)*p(16) - p(58)
p(78) = p(1)*p(36) - p(62) - p(60)
p(79) = p(2)*p(32) - p(58)
p(80) = p(6)*p(12) - p(64) - p(63) - p(63)
p(81) = p(1)*p(45) - p(80)
p(82) = p(14)*p(6)
p(83) = p(2)*p(33) - p(61) - p(60)
p(84) = p(2)*p(34) - p(62)
p(85) = p(6)*p(18) - p(63)
p(86) = p(2)*p(36) - p(64) - p(80)
p(87) = p(8)*p(13) - p(63)
p(88) = p(14)*p(8)
p(89) = p(1)*p(39) - p(65)
p(90) = p(3)*p(21) - p(66)
p(91) = p(1)*p(41) - p(67)
p(92) = p(2)*p(49) - p(91)
p(93) = p(1)*p(43) - p(74)
p(94) = p(8)*p(21) - p(93)
p(95) = p(1)*p(46) - p(83)
p(96) = p(7)*p(22) - p(95)
p(97) = p(2)*p(45) - p(82)
p(98) = p(2)*p(46) - p(85)
p(99) = p(1)*p(50) - p(98)
p(100) = p(6)*p(22) - p(88)
p(101) = p(1)*p(49) - p(90)
p(102) = p(2)*p(50) - p(100)
p(103) = p(9)*p(14)
p(104) = p(9)*p(12)
p(105) = p(9)*p(13)
p(106) = p(14)*p(15)
p(107) = p(9)*p(20)
p(108) = p(14)*p(10)
p(109) = p(14)*p(11)
p(110) = p(9)*p(16)
p(111) = p(9)*p(10)
p(112) = p(9)*p(11)
p(113) = p(9)*p(17)
p(114) = p(3)*p(24) - p(104)
p(115) = p(7)*p(24) - p(105)
p(116) = p(3)*p(31) - p(104) - p(115)
p(117) = p(6)*p(39) - p(114)
p(118) = p(3)*p(32) - p(105)
p(119) = p(14)*p(21)
p(120) = p(9)*p(18)
p(121) = p(9)*p(19)
p(122) = p(4)*p(24) - p(103) - p(103) - p(103)
p(123) = p(1)*p(60) - p(107) - p(122)
p(124) = p(13)*p(16) - p(119)
p(125) = p(2)*p(56) - p(106) - p(124)
p(126) = p(4)*p(32) - p(119)
p(127) = p(1)*p(62) - p(107) - p(125)
p(128) = p(14)*p(16)
p(129) = p(14)*p(17)
p(130) = p(6)*p(24) - p(108)
p(131) = p(3)*p(45) - p(130)
p(132) = p(14)*p(12)
p(133) = p(14)*p(13)
p(134) = p(9)*p(22)
p(135) = p(8)*p(24) - p(109)
p(136) = p(13)*p(18) - p(128)
p(137) = p(6)*p(34) - p(109)
p(138) = p(14)*p(18)
p(139) = p(14)*p(19)
p(140) = p(9)*p(9)
p(141) = p(9)*p(15)
p(142) = p(9)*p(21)
p(143) = p(3)*p(41) - p(110)
p(144) = p(4)*p(39) - p(112)
p(145) = p(1)*p(68) - p(111) - p(144)
p(146) = p(1)*p(69) - p(112)
p(147) = p(1)*p(70) - p(113) - p(145)
p(148) = p(11)*p(21) - p(110)
p(149) = p(1)*p(72) - p(116) - p(115)
p(150) = p(1)*p(73) - p(118)
p(151) = p(3)*p(43) - p(120)
p(152) = p(15)*p(18) - p(121) - p(151)
p(153) = p(2)*p(69) - p(117)
p(154) = p(1)*p(75) - p(121) - p(152)
p(155) = p(7)*p(34) - p(120)
p(156) = p(6)*p(41) - p(119)
p(157) = p(1)*p(78) - p(125) - p(123)
p(158) = p(2)*p(73) - p(119)
p(159) = p(1)*p(80) - p(131) - p(130) - p(130)
p(160) = p(1)*p(81) - p(131)
p(161) = p(14)*p(14)
p(162) = p(3)*p(46) - p(134)
p(163) = p(2)*p(75) - p(126) - p(123)
p(164) = p(1)*p(84) - p(134)
p(165) = p(6)*p(43) - p(128)
p(166) = p(1)*p(86) - p(137) - p(135)
p(167) = p(8)*p(32) - p(128)
p(168) = p(4)*p(45) - p(133)
p(169) = p(2)*p(80) - p(132) - p(168)
p(170) = p(2)*p(81) - p(133)
p(171) = p(14)*p(20)
p(172) = p(2)*p(83) - p(136) - p(135)
p(173) = p(2)*p(84) - p(137)
p(174) = p(6)*p(46) - p(138)
p(175) = p(2)*p(86) - p(139) - p(169)
p(176) = p(13)*p(22) - p(138)
p(177) = p(14)*p(22)
p(178) = p(3)*p(39) - p(141)
p(179) = p(1)*p(89) - p(141) - p(178) - p(178)
p(180) = p(3)*p(49) - p(142)
p(181) = p(1)*p(91) - p(143)
p(182) = p(2)*p(101) - p(181)
p(183) = p(1)*p(93) - p(151)
p(184) = p(8)*p(49) - p(183)
p(185) = p(1)*p(95) - p(162)
p(186) = p(21)*p(22) - p(185)
p(187) = p(6)*p(45) - p(171)
p(188) = p(1)*p(98) - p(172)
p(189) = p(7)*p(50) - p(188)
p(190) = p(2)*p(97) - p(171) - p(187) - p(187)
p(191) = p(2)*p(98) - p(174)
p(192) = p(1)*p(102) - p(191)
p(193) = p(6)*p(50) - p(177)
p(194) = p(1)*p(101) - p(180)
p(195) = p(2)*p(102) - p(193)
p(196) = p(9)*p(26)
p(197) = p(9)*p(38)
p(198) = p(9)*p(24)
p(199) = p(9)*p(31)
p(200) = p(9)*p(25)
p(201) = p(9)*p(32)
p(202) = p(14)*p(39)
p(203) = p(14)*p(40)
p(204) = p(9)*p(35)
p(205) = p(9)*p(36)
p(206) = p(9)*p(37)
p(207) = p(14)*p(28)
p(208) = p(14)*p(29)
p(209) = p(14)*p(30)
p(210) = p(9)*p(45)
p(211) = p(14)*p(24)
p(212) = p(14)*p(25)
p(213) = p(9)*p(48)
p(214) = p(14)*p(33)
p(215) = p(14)*p(34)
p(216) = p(9)*p(41)
p(217) = p(9)*p(28)
p(218) = p(9)*p(23)
p(219) = p(9)*p(29)
p(220) = p(9)*p(30)
p(221) = p(9)*p(42)
p(222) = p(1)*p(114) - p(198)
p(223) = p(21)*p(24) - p(201)
p(224) = p(3)*p(72) - p(199) - p(223)
p(225) = p(1)*p(116) - p(199) - p(224)
p(226) = p(13)*p(39) - p(198)
p(227) = p(3)*p(73) - p(201)
p(228) = p(14)*p(49)
p(229) = p(9)*p(43)
p(230) = p(9)*p(33)
p(231) = p(9)*p(34)
p(232) = p(9)*p(44)
p(233) = p(16)*p(24) - p(196)
p(234) = p(2)*p(114) - p(202)
p(235) = p(1)*p(123) - p(205) - p(234)
p(236) = p(13)*p(41) - p(228)
p(237) = p(2)*p(116) - p(203) - p(236)
p(238) = p(16)*p(32) - p(228)
p(239) = p(6)*p(69) - p(202)
p(240) = p(4)*p(73) - p(228)
p(241) = p(11)*p(32) - p(196)
p(242) = p(14)*p(41)
p(243) = p(14)*p(42)
p(244) = p(1)*p(130) - p(210)
p(245) = p(4)*p(62) - p(213) - p(209)
p(246) = p(3)*p(81) - p(210)
p(247) = p(14)*p(31)
p(248) = p(14)*p(32)
p(249) = p(14)*p(26)
p(250) = p(9)*p(46)
p(251) = p(9)*p(47)
p(252) = p(18)*p(24) - p(197)
p(253) = p(1)*p(135) - p(213) - p(252)
p(254) = p(13)*p(43) - p(242)
p(255) = p(2)*p(125) - p(209) - p(245)
p(256) = p(18)*p(32) - p(242)
p(257) = p(13)*p(34) - p(197)
p(258) = p(14)*p(43)
p(259) = p(14)*p(44)
p(260) = p(2)*p(130) - p(211)
p(261) = p(4)*p(81) - p(248)
p(262) = p(11)*p(45) - p(211)
p(263) = p(14)*p(35)
p(264) = p(14)*p(36)
p(265) = p(14)*p(37)
p(266) = p(9)*p(50)
p(267) = p(22)*p(24) - p(215)
p(268) = p(13)*p(46) - p(258)
p(269) = p(6)*p(84) - p(215)
p(270) = p(14)*p(46)
p(271) = p(14)*p(47)
p(272) = p(9)*p(27)
p(273) = p(9)*p(39)
p(274) = p(9)*p(40)
p(275) = p(9)*p(49)
p(276) = p(3)*p(91) - p(216)
p(277) = p(16)*p(39) - p(218)
p(278) = p(1)*p(144) - p(217) - p(277)
p(279) = p(2)*p(178) - p(278)
p(280) = p(1)*p(145) - p(219) - p(278)
p(281) = p(7)*p(69) - p(218)
p(282) = p(1)*p(147) - p(221) - p(280)
p(283) = p(11)*p(49) - p(216)
p(284) = p(1)*p(149) - p(224) - p(223)
p(285) = p(1)*p(150) - p(227)
p(286) = p(3)*p(93) - p(229)
p(287) = p(18)*p(39) - p(231)
p(288) = p(1)*p(152) - p(230) - p(287)
p(289) = p(1)*p(153) - p(231)
p(290) = p(1)*p(154) - p(232) - p(288)
p(291) = p(21)*p(34) - p(229)
p(292) = p(6)*p(91) - p(228)
p(293) = p(1)*p(157) - p(237) - p(235)
p(294) = p(2)*p(150) - p(228)
p(295) = p(1)*p(159) - p(245) - p(244)
p(296) = p(1)*p(160) - p(246)
p(297) = p(3)*p(95) - p(250)
p(298) = p(2)*p(152) - p(238) - p(234)
p(299) = p(2)*p(153) - p(239)
p(300) = p(1)*p(163) - p(251) - p(298)
p(301) = p(7)*p(84) - p(250)
p(302) = p(6)*p(93) - p(242)
p(303) = p(1)*p(166) - p(255) - p(253)
p(304) = p(8)*p(73) - p(242)
p(305) = p(16)*p(45) - p(248)
p(306) = p(1)*p(169) - p(262) - p(260)
p(307) = p(2)*p(160) - p(248)
p(308) = p(14)*p(38)
p(309) = p(6)*p(80) - p(264) - p(263)
p(310) = p(1)*p(187) - p(309)
p(311) = p(14)*p(45)
p(312) = p(3)*p(98) - p(266)
p(313) = p(2)*p(163) - p(256) - p(253)
p(314) = p(1)*p(173) - p(266)
p(315) = p(6)*p(95) - p(258)
p(316) = p(1)*p(175) - p(269) - p(267)
p(317) = p(22)*p(32) - p(258)
p(318) = p(18)*p(45) - p(249)
p(319) = p(2)*p(169) - p(264) - p(309)
p(320) = p(8)*p(81) - p(249)
p(321) = p(14)*p(48)
p(322) = p(2)*p(172) - p(268) - p(267)
p(323) = p(2)*p(173) - p(269)
p(324) = p(6)*p(98) - p(270)
p(325) = p(2)*p(175) - p(271) - p(319)
p(326) = p(13)*p(50) - p(270)
p(327) = p(14)*p(50)
p(328) = p(1)*p(178) - p(273)
p(329) = p(21)*p(39) - p(272)
p(330) = p(3)*p(101) - p(275)
p(331) = p(1)*p(181) - p(276)
p(332) = p(2)*p(194) - p(331)
p(333) = p(1)*p(183) - p(286)
p(334) = p(8)*p(101) - p(333)
p(335) = p(1)*p(185) - p(297)
p(336) = p(22)*p(49) - p(335)
p(337) = p(1)*p(188) - p(312)
p(338) = p(21)*p(50) - p(337)
p(339) = p(2)*p(187) - p(311)
p(340) = p(1)*p(191) - p(322)
p(341) = p(7)*p(102) - p(340)
p(342) = p(22)*p(45) - p(308)
p(343) = p(2)*p(191) - p(324)
p(344) = p(1)*p(195) - p(343)
p(345) = p(6)*p(102) - p(327)
p(346) = p(1)*p(194) - p(330)
p(347) = p(2)*p(195) - p(345)
p(348) = p(9)*p(52)
p(349) = p(9)*p(58)
p(350) = p(9)*p(63)
p(351) = p(9)*p(64)
p(352) = p(9)*p(82)
p(353) = p(9)*p(88)
p(354) = p(9)*p(55)
p(355) = p(9)*p(72)
p(356) = p(9)*p(51)
p(357) = p(9)*p(56)
p(358) = p(9)*p(57)
p(359) = p(9)*p(73)
p(360) = p(14)*p(89)
p(361) = p(14)*p(90)
p(362) = p(9)*p(77)
p(363) = p(9)*p(60)
p(364) = p(9)*p(78)
p(365) = p(9)*p(61)
p(366) = p(9)*p(62)
p(367) = p(9)*p(79)
p(368) = p(14)*p(67)
p(369) = p(14)*p(68)
p(370) = p(14)*p(69)
p(371) = p(14)*p(70)
p(372) = p(14)*p(71)
p(373) = p(9)*p(80)
p(374) = p(9)*p(81)
p(375) = p(14)*p(55)
p(376) = p(14)*p(56)
p(377) = p(14)*p(57)
p(378) = p(14)*p(52)
p(379) = p(9)*p(85)
p(380) = p(9)*p(86)
p(381) = p(9)*p(87)
p(382) = p(14)*p(74)
p(383) = p(14)*p(75)
p(384) = p(14)*p(76)
p(385) = p(9)*p(97)
p(386) = p(14)*p(60)
p(387) = p(14)*p(61)
p(388) = p(14)*p(62)
p(389) = p(9)*p(100)
p(390) = p(14)*p(83)
p(391) = p(14)*p(84)
p(392) = p(9)*p(91)
p(393) = p(9)*p(67)
p(394) = p(9)*p(53)
p(395) = p(9)*p(68)
p(396) = p(9)*p(54)
p(397) = p(9)*p(69)
p(398) = p(9)*p(70)
p(399) = p(9)*p(71)
p(400) = p(9)*p(92)
p(401) = p(3)*p(114) - p(354)
p(402) = p(7)*p(114) - p(356)
p(403) = p(24)*p(49) - p(359)
p(404) = p(3)*p(149) - p(355) - p(403)
p(405) = p(1)*p(224) - p(355) - p(404)
p(406) = p(6)*p(178) - p(401)
p(407) = p(32)*p(39) - p(356)
p(408) = p(3)*p(150) - p(359)
p(409) = p(14)*p(101)
p(410) = p(9)*p(93)
p(411) = p(9)*p(74)
p(412) = p(9)*p(59)
p(413) = p(9)*p(75)
p(414) = p(9)*p(76)
p(415) = p(9)*p(94)
p(416) = p(24)*p(41) - p(349)
p(417) = p(4)*p(114) - p(348)
p(418) = p(1)*p(234) - p(363) - p(417)
p(419) = p(1)*p(235) - p(364) - p(418)
p(420) = p(13)*p(91) - p(409)
p(421) = p(2)*p(224) - p(361) - p(420)
p(422) = p(32)*p(41) - p(409)
p(423) = p(1)*p(237) - p(364) - p(421)
p(424) = p(16)*p(73) - p(409)
p(425) = p(13)*p(69) - p(348)
p(426) = p(4)*p(150) - p(409)
p(427) = p(11)*p(73) - p(349)
p(428) = p(14)*p(91)
p(429) = p(14)*p(92)
p(430) = p(3)*p(130) - p(373)
p(431) = p(7)*p(130) - p(374)
p(432) = p(3)*p(159) - p(373) - p(431)
p(433) = p(39)*p(45) - p(430)
p(434) = p(3)*p(160) - p(374)
p(435) = p(14)*p(72)
p(436) = p(14)*p(73)
p(437) = p(14)*p(58)
p(438) = p(9)*p(95)
p(439) = p(9)*p(83)
p(440) = p(9)*p(84)
p(441) = p(9)*p(96)
p(442) = p(24)*p(43) - p(350)
p(443) = p(8)*p(114) - p(370)
p(444) = p(1)*p(253) - p(380) - p(443)
p(445) = p(13)*p(93) - p(428)
p(446) = p(2)*p(237) - p(372) - p(432)
p(447) = p(32)*p(43) - p(428)
p(448) = p(6)*p(153) - p(370)
p(449) = p(18)*p(73) - p(428)
p(450) = p(32)*p(34) - p(350)
p(451) = p(14)*p(93)
p(452) = p(14)*p(94)
p(453) = p(4)*p(130) - p(352)
p(454) = p(1)*p(260) - p(385) - p(453)
p(455) = p(16)*p(81) - p(436)
p(456) = p(2)*p(245) - p(376) - p(455)
p(457) = p(4)*p(160) - p(436)
p(458) = p(11)*p(81) - p(352)
p(459) = p(14)*p(77)
p(460) = p(14)*p(78)
p(461) = p(14)*p(79)
p(462) = p(14)*p(63)
p(463) = p(14)*p(64)
p(464) = p(6)*p(130) - p(386)
p(465) = p(3)*p(187) - p(464)
p(466) = p(14)*p(80)
p(467) = p(14)*p(81)
p(468) = p(9)*p(98)
p(469) = p(9)*p(99)
p(470) = p(24)*p(46) - p(353)
p(471) = p(1)*p(267) - p(389) - p(470)
p(472) = p(13)*p(95) - p(451)
p(473) = p(2)*p(255) - p(384) - p(456)
p(474) = p(32)*p(46) - p(451)
p(475) = p(13)*p(84) - p(353)
p(476) = p(14)*p(95)
p(477) = p(14)*p(96)
p(478) = p(8)*p(130) - p(378)
p(479) = p(18)*p(81) - p(437)
p(480) = p(34)*p(45) - p(378)
p(481) = p(14)*p(85)
p(482) = p(14)*p(86)
p(483) = p(14)*p(87)
p(484) = p(9)*p(102)
p(485) = p(24)*p(50) - p(391)
p(486) = p(13)*p(98) - p(476)
p(487) = p(6)*p(173) - p(391)
p(488) = p(14)*p(98)
p(489) = p(14)*p(99)
p(490) = p(9)*p(65)
p(491) = p(9)*p(66)
p(492) = p(9)*p(89)
p(493) = p(9)*p(90)
p(494) = p(9)*p(101)
p(495) = p(3)*p(181) - p(392)
p(496) = p(39)*p(41) - p(394)
p(497) = p(4)*p(178) - p(397)
p(498) = p(1)*p(278) - p(395) - p(497)
p(499) = p(1)*p(279) - p(397)
p(500) = p(1)*p(280) - p(398) - p(498)
p(501) = p(21)*p(69) - p(394)
p(502) = p(1)*p(282) - p(400) - p(500)
p(503) = p(11)*p(101) - p(392)
p(504) = p(1)*p(284) - p(404) - p(403)
p(505) = p(1)*p(285) - p(408)
p(506) = p(3)*p(183) - p(410)
p(507) = p(39)*p(43) - p(412)
p(508) = p(1)*p(287) - p(411) - p(507)
p(509) = p(2)*p(279) - p(406)
p(510) = p(1)*p(288) - p(413) - p(508)
p(511) = p(7)*p(153) - p(412)
p(512) = p(1)*p(290) - p(415) - p(510)
p(513) = p(34)*p(49) - p(410)
p(514) = p(6)*p(181) - p(409)
p(515) = p(1)*p(293) - p(421) - p(419)
p(516) = p(2)*p(285) - p(409)
p(517) = p(1)*p(295) - p(432) - p(431)
p(518) = p(1)*p(296) - p(434)
p(519) = p(3)*p(185) - p(438)
p(520) = p(39)*p(46) - p(440)
p(521) = p(1)*p(298) - p(439) - p(520)
p(522) = p(1)*p(299) - p(440)
p(523) = p(1)*p(300) - p(441) - p(521)
p(524) = p(21)*p(84) - p(438)
p(525) = p(6)*p(183) - p(428)
p(526) = p(1)*p(303) - p(446) - p(444)
p(527) = p(8)*p(150) - p(428)
p(528) = p(41)*p(45) - p(436)
p(529) = p(1)*p(306) - p(456) - p(454)
p(530) = p(2)*p(296) - p(436)
p(531) = p(4)*p(169) - p(388) - p(478)
p(532) = p(1)*p(310) - p(465)
p(533) = p(14)*p(82)
p(534) = p(3)*p(188) - p(468)
p(535) = p(2)*p(298) - p(447) - p(443)
p(536) = p(2)*p(299) - p(448)
p(537) = p(1)*p(313) - p(469) - p(535)
p(538) = p(7)*p(173) - p(468)
p(539) = p(6)*p(185) - p(451)
p(540) = p(1)*p(316) - p(473) - p(471)
p(541) = p(22)*p(73) - p(451)
p(542) = p(43)*p(45) - p(437)
p(543) = p(1)*p(319) - p(480) - p(478)
p(544) = p(8)*p(160) - p(437)
p(545) = p(14)*p(88)
p(546) = p(4)*p(187) - p(467)
p(547) = p(2)*p(309) - p(466) - p(546)
p(548) = p(2)*p(310) - p(467)
p(549) = p(14)*p(97)
p(550) = p(3)*p(191) - p(484)
p(551) = p(2)*p(313) - p(474) - p(471)
p(552) = p(1)*p(323) - p(484)
p(553) = p(6)*p(188) - p(476)
p(554) = p(1)*p(325) - p(487) - p(485)
p(555) = p(32)*p(50) - p(476)
p(556) = p(45)*p(46) - p(462)
p(557) = p(2)*p(319) - p(482) - p(547)
p(558) = p(22)*p(81) - p(462)
p(559) = p(14)*p(100)
p(560) = p(2)*p(322) - p(486) - p(485)
p(561) = p(2)*p(323) - p(487)
p(562) = p(6)*p(191) - p(488)
p(563) = p(2)*p(325) - p(489) - p(557)
p(564) = p(13)*p(102) - p(488)
p(565) = p(14)*p(102)
p(566) = p(3)*p(178) - p(492)
p(567) = p(7)*p(178) - p(490)
p(568) = p(39)*p(49) - p(491)
p(569) = p(3)*p(194) - p(494)
p(570) = p(1)*p(331) - p(495)
p(571) = p(2)*p(346) - p(570)
p(572) = p(1)*p(333) - p(506)
p(573) = p(8)*p(194) - p(572)
p(574) = p(1)*p(335) - p(519)
p(575) = p(22)*p(101) - p(574)
p(576) = p(1)*p(337) - p(534)
p(577) = p(49)*p(50) - p(576)
p(578) = p(6)*p(187) - p(549)
p(579) = p(1)*p(340) - p(550)
p(580) = p(21)*p(102) - p(579)
p(581) = p(8)*p(187) - p(533)
p(582) = p(1)*p(343) - p(560)
p(583) = p(7)*p(195) - p(582)
p(584) = p(45)*p(50) - p(545)
p(585) = p(2)*p(343) - p(562)
p(586) = p(1)*p(347) - p(585)
p(587) = p(6)*p(195) - p(565)
p(588) = p(1)*p(346) - p(569)
p(589) = p(2)*p(347) - p(587)
p(590) = p(9)*p(103)
p(591) = p(9)*p(106)
p(592) = p(9)*p(119)
p(593) = p(9)*p(128)
p(594) = p(9)*p(108)
p(595) = p(9)*p(109)
p(596) = p(9)*p(129)
p(597) = p(9)*p(132)
p(598) = p(9)*p(133)
p(599) = p(9)*p(161)
p(600) = p(9)*p(138)
p(601) = p(9)*p(139)
p(602) = p(9)*p(171)
p(603) = p(9)*p(177)
p(604) = p(9)*p(114)
p(605) = p(9)*p(115)
p(606) = p(9)*p(149)
p(607) = p(9)*p(104)
p(608) = p(9)*p(116)
p(609) = p(9)*p(105)
p(610) = p(9)*p(117)
p(611) = p(9)*p(118)
p(612) = p(9)*p(150)
p(613) = p(14)*p(178)
p(614) = p(14)*p(179)
p(615) = p(14)*p(180)
p(616) = p(9)*p(156)
p(617) = p(9)*p(122)
p(618) = p(9)*p(123)
p(619) = p(9)*p(157)
p(620) = p(9)*p(124)
p(621) = p(9)*p(107)
p(622) = p(9)*p(125)
p(623) = p(9)*p(126)
p(624) = p(9)*p(127)
p(625) = p(9)*p(158)
p(626) = p(14)*p(143)
p(627) = p(14)*p(144)
p(628) = p(14)*p(145)
p(629) = p(14)*p(146)
p(630) = p(14)*p(147)
p(631) = p(14)*p(148)
p(632) = p(9)*p(130)
p(633) = p(9)*p(159)
p(634) = p(9)*p(131)
p(635) = p(9)*p(160)
p(636) = p(14)*p(114)
p(637) = p(14)*p(115)
p(638) = p(14)*p(116)
p(639) = p(14)*p(117)
p(640) = p(14)*p(118)
p(641) = p(14)*p(106)
p(642) = p(9)*p(165)
p(643) = p(9)*p(135)
p(644) = p(9)*p(166)
p(645) = p(9)*p(136)
p(646) = p(9)*p(137)
p(647) = p(9)*p(167)
p(648) = p(14)*p(151)
p(649) = p(14)*p(152)
p(650) = p(14)*p(153)
p(651) = p(14)*p(154)
p(652) = p(14)*p(155)
p(653) = p(9)*p(168)
p(654) = p(9)*p(169)
p(655) = p(9)*p(170)
p(656) = p(14)*p(122)
p(657) = p(14)*p(123)
p(658) = p(14)*p(124)
p(659) = p(14)*p(125)
p(660) = p(14)*p(126)
p(661) = p(14)*p(127)
p(662) = p(14)*p(108)
p(663) = p(14)*p(109)
p(664) = p(9)*p(187)
p(665) = p(14)*p(130)
p(666) = p(14)*p(131)
p(667) = p(9)*p(174)
p(668) = p(9)*p(175)
p(669) = p(9)*p(176)
p(670) = p(14)*p(162)
p(671) = p(14)*p(163)
p(672) = p(14)*p(164)
p(673) = p(9)*p(190)
p(674) = p(14)*p(135)
p(675) = p(14)*p(136)
p(676) = p(14)*p(137)
p(677) = p(9)*p(193)
p(678) = p(14)*p(172)
p(679) = p(14)*p(173)
p(680) = p(9)*p(181)
p(681) = p(9)*p(143)
p(682) = p(9)*p(110)
p(683) = p(9)*p(144)
p(684) = p(9)*p(111)
p(685) = p(9)*p(112)
p(686) = p(9)*p(145)
p(687) = p(9)*p(113)
p(688) = p(9)*p(146)
p(689) = p(9)*p(147)
p(690) = p(9)*p(148)
p(691) = p(9)*p(182)
p(692) = p(1)*p(401) - p(604)
p(693) = p(21)*p(114) - p(609)
p(694) = p(24)*p(101) - p(612)
p(695) = p(3)*p(284) - p(606) - p(694)
p(696) = p(1)*p(404) - p(606) - p(695)
p(697) = p(1)*p(405) - p(608) - p(696)
p(698) = p(13)*p(178) - p(604)
p(699) = p(39)*p(73) - p(609)
p(700) = p(3)*p(285) - p(612)
p(701) = p(14)*p(194)
p(702) = p(9)*p(183)
p(703) = p(9)*p(151)
p(704) = p(9)*p(120)
p(705) = p(9)*p(152)
p(706) = p(9)*p(121)
p(707) = p(9)*p(153)
p(708) = p(9)*p(154)
p(709) = p(9)*p(155)
p(710) = p(9)*p(184)
p(711) = p(24)*p(91) - p(592)
p(712) = p(1)*p(416) - p(616) - p(711)
p(713) = p(2)*p(401) - p(613)
p(714) = p(1)*p(418) - p(618) - p(713)
p(715) = p(1)*p(419) - p(619) - p(714)
p(716) = p(13)*p(181) - p(701)
p(717) = p(2)*p(404) - p(615) - p(716)
p(718) = p(32)*p(91) - p(701)
p(719) = p(1)*p(421) - p(619) - p(717)
p(720) = p(41)*p(73) - p(701)
p(721) = p(6)*p(279) - p(613)
p(722) = p(16)*p(150) - p(701)
p(723) = p(1)*p(425) - p(624) - p(721)
p(724) = p(4)*p(285) - p(701)
p(725) = p(11)*p(150) - p(592)
p(726) = p(14)*p(181)
p(727) = p(14)*p(182)
p(728) = p(1)*p(430) - p(632)
p(729) = p(21)*p(130) - p(635)
p(730) = p(3)*p(295) - p(633) - p(729)
p(731) = p(1)*p(432) - p(633) - p(730)
p(732) = p(39)*p(81) - p(632)
p(733) = p(3)*p(296) - p(635)
p(734) = p(14)*p(149)
p(735) = p(14)*p(150)
p(736) = p(14)*p(119)
p(737) = p(9)*p(185)
p(738) = p(9)*p(162)
p(739) = p(9)*p(134)
p(740) = p(9)*p(163)
p(741) = p(9)*p(164)
p(742) = p(9)*p(186)
p(743) = p(24)*p(93) - p(593)
p(744) = p(18)*p(114) - p(595)
p(745) = p(1)*p(443) - p(643) - p(744)
p(746) = p(1)*p(444) - p(644) - p(745)
p(747) = p(13)*p(183) - p(726)
p(748) = p(2)*p(421) - p(631) - p(730)
p(749) = p(32)*p(93) - p(726)
p(750) = p(1)*p(446) - p(644) - p(748)
p(751) = p(43)*p(73) - p(726)
p(752) = p(13)*p(153) - p(595)
p(753) = p(18)*p(150) - p(726)
p(754) = p(34)*p(73) - p(593)
p(755) = p(14)*p(183)
p(756) = p(14)*p(184)
p(757) = p(16)*p(130) - p(598)
p(758) = p(2)*p(430) - p(636)
p(759) = p(1)*p(454) - p(654) - p(758)
p(760) = p(41)*p(81) - p(735)
p(761) = p(2)*p(432) - p(638) - p(760)
p(762) = p(16)*p(160) - p(735)
p(763) = p(45)*p(69) - p(636)
p(764) = p(4)*p(296) - p(735)
p(765) = p(11)*p(160) - p(598)
p(766) = p(14)*p(156)
p(767) = p(14)*p(157)
p(768) = p(14)*p(158)
p(769) = p(14)*p(128)
p(770) = p(14)*p(129)
p(771) = p(1)*p(464) - p(664)
p(772) = p(4)*p(262) - p(673) - p(661)
p(773) = p(3)*p(310) - p(664)
p(774) = p(14)*p(159)
p(775) = p(14)*p(160)
p(776) = p(14)*p(132)
p(777) = p(14)*p(133)
p(778) = p(9)*p(188)
p(779) = p(9)*p(172)
p(780) = p(9)*p(173)
p(781) = p(9)*p(189)
p(782) = p(24)*p(95) - p(600)
p(783) = p(22)*p(114) - p(650)
p(784) = p(1)*p(471) - p(668) - p(783)
p(785) = p(13)*p(185) - p(755)
p(786) = p(2)*p(446) - p(652) - p(761)
p(787) = p(32)*p(95) - p(755)
p(788) = p(6)*p(299) - p(650)
p(789) = p(46)*p(73) - p(755)
p(790) = p(32)*p(84) - p(600)
p(791) = p(14)*p(185)
p(792) = p(14)*p(186)
p(793) = p(2)*p(453) - p(656) - p(771)
p(794) = p(1)*p(478) - p(673) - p(793)
p(795) = p(43)*p(81) - p(736)
p(796) = p(2)*p(456) - p(659) - p(772)
p(797) = p(18)*p(160) - p(736)
p(798) = p(1)*p(480) - p(673) - p(796)
p(799) = p(14)*p(165)
p(800) = p(14)*p(166)
p(801) = p(14)*p(167)
p(802) = p(14)*p(138)
p(803) = p(14)*p(139)
p(804) = p(2)*p(464) - p(665)
p(805) = p(4)*p(310) - p(775)
p(806) = p(11)*p(187) - p(665)
p(807) = p(14)*p(168)
p(808) = p(14)*p(169)
p(809) = p(14)*p(170)
p(810) = p(9)*p(191)
p(811) = p(9)*p(192)
p(812) = p(24)*p(98) - p(603)
p(813) = p(1)*p(485) - p(677) - p(812)
p(814) = p(13)*p(188) - p(791)
p(815) = p(2)*p(473) - p(672) - p(796)
p(816) = p(32)*p(98) - p(791)
p(817) = p(13)*p(173) - p(603)
p(818) = p(14)*p(188)
p(819) = p(14)*p(189)
p(820) = p(22)*p(130) - p(663)
p(821) = p(46)*p(81) - p(769)
p(822) = p(45)*p(84) - p(663)
p(823) = p(14)*p(174)
p(824) = p(14)*p(175)
p(825) = p(14)*p(176)
p(826) = p(9)*p(195)
p(827) = p(24)*p(102) - p(679)
p(828) = p(13)*p(191) - p(818)
p(829) = p(6)*p(323) - p(679)
p(830) = p(14)*p(191)
p(831) = p(14)*p(192)
p(832) = p(9)*p(140)
p(833) = p(9)*p(141)
p(834) = p(9)*p(178)
p(835) = p(9)*p(142)
p(836) = p(9)*p(179)
p(837) = p(9)*p(180)
p(838) = p(9)*p(194)
p(839) = p(3)*p(331) - p(680)
p(840) = p(39)*p(91) - p(682)
p(841) = p(16)*p(178) - p(685)
p(842) = p(1)*p(497) - p(683) - p(841)
p(843) = p(2)*p(566) - p(842)
p(844) = p(1)*p(498) - p(686) - p(842)
p(845) = p(7)*p(279) - p(685)
p(846) = p(1)*p(500) - p(689) - p(844)
p(847) = p(49)*p(69) - p(682)
p(848) = p(1)*p(502) - p(691) - p(846)
p(849) = p(11)*p(194) - p(680)
p(850) = p(1)*p(504) - p(695) - p(694)
p(851) = p(1)*p(505) - p(700)
p(852) = p(3)*p(333) - p(702)
p(853) = p(39)*p(93) - p(704)
p(854) = p(18)*p(178) - p(707)
p(855) = p(1)*p(508) - p(705) - p(854)
p(856) = p(1)*p(509) - p(707)
p(857) = p(1)*p(510) - p(708) - p(855)
p(858) = p(21)*p(153) - p(704)
p(859) = p(1)*p(512) - p(710) - p(857)
p(860) = p(34)*p(101) - p(702)
p(861) = p(6)*p(331) - p(701)
p(862) = p(1)*p(515) - p(717) - p(715)
p(863) = p(2)*p(505) - p(701)
p(864) = p(1)*p(517) - p(730) - p(729)
p(865) = p(1)*p(518) - p(733)
p(866) = p(3)*p(335) - p(737)
p(867) = p(39)*p(95) - p(739)
p(868) = p(1)*p(520) - p(738) - p(867)
p(869) = p(2)*p(509) - p(721)
p(870) = p(1)*p(521) - p(740) - p(868)
p(871) = p(7)*p(299) - p(739)
p(872) = p(1)*p(523) - p(742) - p(870)
p(873) = p(49)*p(84) - p(737)
p(874) = p(6)*p(333) - p(726)
p(875) = p(1)*p(526) - p(748) - p(746)
p(876) = p(8)*p(285) - p(726)
p(877) = p(45)*p(91) - p(735)
p(878) = p(1)*p(529) - p(761) - p(759)
p(879) = p(2)*p(518) - p(735)
p(880) = p(1)*p(531) - p(772) - p(771)
p(881) = p(1)*p(532) - p(773)
p(882) = p(14)*p(161)
p(883) = p(3)*p(337) - p(778)
p(884) = p(39)*p(98) - p(780)
p(885) = p(1)*p(535) - p(779) - p(884)
p(886) = p(1)*p(536) - p(780)
p(887) = p(1)*p(537) - p(781) - p(885)
p(888) = p(21)*p(173) - p(778)
p(889) = p(6)*p(335) - p(755)
p(890) = p(1)*p(540) - p(786) - p(784)
p(891) = p(22)*p(150) - p(755)
p(892) = p(45)*p(93) - p(736)
p(893) = p(1)*p(543) - p(796) - p(794)
p(894) = p(8)*p(296) - p(736)
p(895) = p(16)*p(187) - p(775)
p(896) = p(1)*p(547) - p(806) - p(804)
p(897) = p(2)*p(532) - p(775)
p(898) = p(14)*p(171)
p(899) = p(6)*p(309) - p(808) - p(807)
p(900) = p(1)*p(578) - p(899)
p(901) = p(14)*p(187)
p(902) = p(3)*p(340) - p(810)
p(903) = p(2)*p(535) - p(787) - p(783)
p(904) = p(2)*p(536) - p(788)
p(905) = p(1)*p(551) - p(811) - p(903)
p(906) = p(7)*p(323) - p(810)
p(907) = p(6)*p(337) - p(791)
p(908) = p(1)*p(554) - p(815) - p(813)
p(909) = p(50)*p(73) - p(791)
p(910) = p(45)*p(95) - p(769)
p(911) = p(1)*p(557) - p(822) - p(820)
p(912) = p(22)*p(160) - p(769)
p(913) = p(14)*p(177)
p(914) = p(18)*p(187) - p(777)
p(915) = p(2)*p(547) - p(808) - p(899)
p(916) = p(8)*p(310) - p(777)
p(917) = p(14)*p(190)
p(918) = p(3)*p(343) - p(826)
p(919) = p(2)*p(551) - p(816) - p(813)
p(920) = p(1)*p(561) - p(826)
p(921) = p(6)*p(340) - p(818)
p(922) = p(1)*p(563) - p(829) - p(827)
p(923) = p(32)*p(102) - p(818)
p(924) = p(45)*p(98) - p(802)
p(925) = p(2)*p(557) - p(824) - p(915)
p(926) = p(50)*p(81) - p(802)
p(927) = p(14)*p(193)
p(928) = p(2)*p(560) - p(828) - p(827)
p(929) = p(2)*p(561) - p(829)
p(930) = p(6)*p(343) - p(830)
p(931) = p(2)*p(563) - p(831) - p(925)
p(932) = p(13)*p(195) - p(830)
p(933) = p(14)*p(195)
p(934) = p(1)*p(566) - p(834)
p(935) = p(1)*p(567) - p(836) - p(934)
p(936) = p(39)*p(101) - p(835)
p(937) = p(3)*p(346) - p(838)
p(938) = p(1)*p(570) - p(839)
p(939) = p(2)*p(588) - p(938)
p(940) = p(1)*p(572) - p(852)
p(941) = p(8)*p(346) - p(940)
p(942) = p(1)*p(574) - p(866)
p(943) = p(22)*p(194) - p(942)
p(944) = p(1)*p(576) - p(883)
p(945) = p(50)*p(101) - p(944)
p(946) = p(1)*p(579) - p(902)
p(947) = p(49)*p(102) - p(946)
p(948) = p(2)*p(578) - p(901)
p(949) = p(1)*p(582) - p(918)
p(950) = p(21)*p(195) - p(949)
p(951) = p(2)*p(581) - p(917) - p(948)
p(952) = p(1)*p(585) - p(928)
p(953) = p(7)*p(347) - p(952)
p(954) = p(45)*p(102) - p(913)
p(955) = p(2)*p(585) - p(930)
p(956) = p(1)*p(589) - p(955)
p(957) = p(6)*p(347) - p(933)
p(958) = p(1)*p(588) - p(937)
p(959) = p(2)*p(589) - p(957)
p(960) = p(9)*p(196)
p(961) = p(9)*p(202)
p(962) = p(9)*p(203)
p(963) = p(9)*p(228)
p(964) = p(9)*p(242)
p(965) = p(9)*p(207)
p(966) = p(9)*p(197)
p(967) = p(9)*p(208)
p(968) = p(9)*p(209)
p(969) = p(9)*p(243)
p(970) = p(9)*p(211)
p(971) = p(9)*p(247)
p(972) = p(9)*p(212)
p(973) = p(9)*p(248)
p(974) = p(9)*p(249)
p(975) = p(9)*p(258)
p(976) = p(9)*p(214)
p(977) = p(9)*p(215)
p(978) = p(9)*p(259)
p(979) = p(9)*p(263)
p(980) = p(9)*p(264)
p(981) = p(9)*p(265)
p(982) = p(9)*p(308)
p(983) = p(9)*p(311)
p(984) = p(9)*p(270)
p(985) = p(9)*p(271)
p(986) = p(9)*p(321)
p(987) = p(9)*p(327)
p(988) = p(9)*p(222)
p(989) = p(9)*p(223)
p(990) = p(9)*p(284)
p(991) = p(9)*p(198)
p(992) = p(9)*p(199)
p(993) = p(9)*p(224)
p(994) = p(9)*p(200)
p(995) = p(9)*p(225)
p(996) = p(9)*p(201)
p(997) = p(9)*p(226)
p(998) = p(9)*p(227)
p(999) = p(9)*p(285)
p(1000) = p(14)*p(328)
p(1001) = p(14)*p(329)
p(1002) = p(14)*p(330)
p(1003) = p(9)*p(292)
p(1004) = p(9)*p(233)
p(1005) = p(9)*p(234)
p(1006) = p(9)*p(235)
p(1007) = p(9)*p(293)
p(1008) = p(9)*p(236)
p(1009) = p(9)*p(204)
p(1010) = p(9)*p(205)
p(1011) = p(9)*p(237)
p(1012) = p(9)*p(238)
p(1013) = p(9)*p(206)
p(1014) = p(9)*p(239)
p(1015) = p(9)*p(240)
p(1016) = p(9)*p(241)
p(1017) = p(9)*p(294)
p(1018) = p(14)*p(276)
p(1019) = p(14)*p(277)
p(1020) = p(14)*p(278)
p(1021) = p(14)*p(279)
p(1022) = p(14)*p(280)
p(1023) = p(14)*p(281)
p(1024) = p(14)*p(282)
p(1025) = p(14)*p(283)
p(1026) = p(9)*p(244)
p(1027) = p(9)*p(295)
p(1028) = p(9)*p(210)
p(1029) = p(9)*p(245)
p(1030) = p(9)*p(246)
p(1031) = p(9)*p(296)
p(1032) = p(14)*p(222)
p(1033) = p(14)*p(223)
p(1034) = p(14)*p(224)
p(1035) = p(14)*p(225)
p(1036) = p(14)*p(226)
p(1037) = p(14)*p(227)
p(1038) = p(14)*p(202)
p(1039) = p(14)*p(203)
p(1040) = p(9)*p(302)
p(1041) = p(9)*p(252)
p(1042) = p(9)*p(253)
p(1043) = p(9)*p(303)
p(1044) = p(9)*p(254)
p(1045) = p(9)*p(213)
p(1046) = p(9)*p(255)
p(1047) = p(9)*p(256)
p(1048) = p(9)*p(257)
p(1049) = p(9)*p(304)
p(1050) = p(14)*p(286)
p(1051) = p(14)*p(287)
p(1052) = p(14)*p(288)
p(1053) = p(14)*p(289)
p(1054) = p(14)*p(290)
p(1055) = p(14)*p(291)
p(1056) = p(9)*p(305)
p(1057) = p(9)*p(260)
p(1058) = p(9)*p(306)
p(1059) = p(9)*p(261)
p(1060) = p(9)*p(262)
p(1061) = p(9)*p(307)
p(1062) = p(14)*p(233)
p(1063) = p(14)*p(234)
p(1064) = p(14)*p(235)
p(1065) = p(14)*p(236)
p(1066) = p(14)*p(237)
p(1067) = p(14)*p(238)
p(1068) = p(14)*p(239)
p(1069) = p(14)*p(240)
p(1070) = p(14)*p(241)
p(1071) = p(14)*p(207)
p(1072) = p(14)*p(208)
p(1073) = p(14)*p(209)
p(1074) = p(9)*p(309)
p(1075) = p(9)*p(310)
p(1076) = p(14)*p(244)
p(1077) = p(14)*p(245)
p(1078) = p(14)*p(246)
p(1079) = p(14)*p(211)
p(1080) = p(14)*p(212)
p(1081) = p(9)*p(315)
p(1082) = p(9)*p(267)
p(1083) = p(9)*p(316)
p(1084) = p(9)*p(268)
p(1085) = p(9)*p(269)
p(1086) = p(9)*p(317)
p(1087) = p(14)*p(297)
p(1088) = p(14)*p(298)
p(1089) = p(14)*p(299)
p(1090) = p(14)*p(300)
p(1091) = p(14)*p(301)
p(1092) = p(9)*p(318)
p(1093) = p(9)*p(319)
p(1094) = p(9)*p(320)
p(1095) = p(14)*p(252)
p(1096) = p(14)*p(253)
p(1097) = p(14)*p(254)
p(1098) = p(14)*p(255)
p(1099) = p(14)*p(256)
p(1100) = p(14)*p(257)
p(1101) = p(14)*p(214)
p(1102) = p(14)*p(215)
p(1103) = p(9)*p(339)
p(1104) = p(14)*p(260)
p(1105) = p(14)*p(261)
p(1106) = p(14)*p(262)
p(1107) = p(9)*p(324)
p(1108) = p(9)*p(325)
p(1109) = p(9)*p(326)
p(1110) = p(14)*p(312)
p(1111) = p(14)*p(313)
p(1112) = p(14)*p(314)
p(1113) = p(9)*p(342)
p(1114) = p(14)*p(267)
p(1115) = p(14)*p(268)
p(1116) = p(14)*p(269)
p(1117) = p(9)*p(345)
p(1118) = p(14)*p(322)
p(1119) = p(14)*p(323)
p(1120) = p(9)*p(331)
p(1121) = p(9)*p(276)
p(1122) = p(9)*p(216)
p(1123) = p(9)*p(277)
p(1124) = p(9)*p(217)
p(1125) = p(9)*p(218)
p(1126) = p(9)*p(278)
p(1127) = p(9)*p(219)
p(1128) = p(9)*p(220)
p(1129) = p(9)*p(279)
p(1130) = p(9)*p(280)
p(1131) = p(9)*p(221)
p(1132) = p(9)*p(281)
p(1133) = p(9)*p(282)
p(1134) = p(9)*p(283)
p(1135) = p(9)*p(332)
p(1136) = p(3)*p(401) - p(988)
p(1137) = p(7)*p(401) - p(991)
p(1138) = p(49)*p(114) - p(996)
p(1139) = p(24)*p(194) - p(999)
p(1140) = p(3)*p(504) - p(990) - p(1139)
p(1141) = p(1)*p(695) - p(990) - p(1140)
p(1142) = p(1)*p(696) - p(993) - p(1141)
p(1143) = p(6)*p(566) - p(1136)
p(1144) = p(32)*p(178) - p(991)
p(1145) = p(39)*p(150) - p(996)
p(1146) = p(3)*p(505) - p(999)
p(1147) = p(14)*p(346)
p(1148) = p(9)*p(333)
p(1149) = p(9)*p(286)
p(1150) = p(9)*p(229)
p(1151) = p(9)*p(287)
p(1152) = p(9)*p(230)
p(1153) = p(9)*p(231)
p(1154) = p(9)*p(288)
p(1155) = p(9)*p(232)
p(1156) = p(9)*p(289)
p(1157) = p(9)*p(290)
p(1158) = p(9)*p(291)
p(1159) = p(9)*p(334)
p(1160) = p(24)*p(181) - p(963)
p(1161) = p(41)*p(114) - p(960)
p(1162) = p(4)*p(401) - p(961)
p(1163) = p(1)*p(713) - p(1005) - p(1162)
p(1164) = p(1)*p(714) - p(1006) - p(1163)
p(1165) = p(1)*p(715) - p(1007) - p(1164)
p(1166) = p(13)*p(331) - p(1147)
p(1167) = p(2)*p(695) - p(1002) - p(1166)
p(1168) = p(32)*p(181) - p(1147)
p(1169) = p(1)*p(717) - p(1007) - p(1167)
p(1170) = p(73)*p(91) - p(1147)
p(1171) = p(1)*p(719) - p(1011) - p(1169)
p(1172) = p(41)*p(150) - p(1147)
p(1173) = p(13)*p(279) - p(961)
p(1174) = p(16)*p(285) - p(1147)
p(1175) = p(69)*p(73) - p(960)
p(1176) = p(4)*p(505) - p(1147)
p(1177) = p(11)*p(285) - p(963)
p(1178) = p(14)*p(331)
p(1179) = p(14)*p(332)
p(1180) = p(3)*p(430) - p(1026)
p(1181) = p(7)*p(430) - p(1028)
p(1182) = p(49)*p(130) - p(1031)
p(1183) = p(3)*p(517) - p(1027) - p(1182)
p(1184) = p(1)*p(730) - p(1027) - p(1183)
p(1185) = p(45)*p(178) - p(1180)
p(1186) = p(39)*p(160) - p(1028)
p(1187) = p(3)*p(518) - p(1031)
p(1188) = p(14)*p(284)
p(1189) = p(14)*p(285)
p(1190) = p(14)*p(228)
p(1191) = p(9)*p(335)
p(1192) = p(9)*p(297)
p(1193) = p(9)*p(250)
p(1194) = p(9)*p(298)
p(1195) = p(9)*p(251)
p(1196) = p(9)*p(299)
p(1197) = p(9)*p(300)
p(1198) = p(9)*p(301)
p(1199) = p(9)*p(336)
p(1200) = p(24)*p(183) - p(964)
p(1201) = p(43)*p(114) - p(966)
p(1202) = p(8)*p(401) - p(1021)
p(1203) = p(1)*p(745) - p(1042) - p(1202)
p(1204) = p(1)*p(746) - p(1043) - p(1203)
p(1205) = p(13)*p(333) - p(1178)
p(1206) = p(2)*p(717) - p(1025) - p(1183)
p(1207) = p(32)*p(183) - p(1178)
p(1208) = p(1)*p(748) - p(1043) - p(1206)
p(1209) = p(73)*p(93) - p(1178)
p(1210) = p(6)*p(509) - p(1021)
p(1211) = p(43)*p(150) - p(1178)
p(1212) = p(32)*p(153) - p(966)
p(1213) = p(18)*p(285) - p(1178)
p(1214) = p(34)*p(150) - p(964)
p(1215) = p(14)*p(333)
p(1216) = p(14)*p(334)
p(1217) = p(41)*p(130) - p(973)
p(1218) = p(4)*p(430) - p(970)
p(1219) = p(1)*p(758) - p(1057) - p(1218)
p(1220) = p(1)*p(759) - p(1058) - p(1219)
p(1221) = p(81)*p(91) - p(1189)
p(1222) = p(2)*p(730) - p(1034) - p(1221)
p(1223) = p(41)*p(160) - p(1189)
p(1224) = p(1)*p(761) - p(1058) - p(1222)
p(1225) = p(16)*p(296) - p(1189)
p(1226) = p(69)*p(81) - p(970)
p(1227) = p(4)*p(518) - p(1189)
p(1228) = p(11)*p(296) - p(973)
p(1229) = p(14)*p(292)
p(1230) = p(14)*p(293)
p(1231) = p(14)*p(294)
p(1232) = p(14)*p(242)
p(1233) = p(14)*p(243)
p(1234) = p(3)*p(464) - p(1074)
p(1235) = p(7)*p(464) - p(1075)
p(1236) = p(3)*p(531) - p(1074) - p(1235)
p(1237) = p(39)*p(187) - p(1234)
p(1238) = p(3)*p(532) - p(1075)
p(1239) = p(14)*p(295)
p(1240) = p(14)*p(296)
p(1241) = p(14)*p(247)
p(1242) = p(14)*p(248)
p(1243) = p(14)*p(249)
p(1244) = p(9)*p(337)
p(1245) = p(9)*p(312)
p(1246) = p(9)*p(266)
p(1247) = p(9)*p(313)
p(1248) = p(9)*p(314)
p(1249) = p(9)*p(338)
p(1250) = p(24)*p(185) - p(975)
p(1251) = p(46)*p(114) - p(977)
p(1252) = p(1)*p(783) - p(1082) - p(1251)
p(1253) = p(1)*p(784) - p(1083) - p(1252)
p(1254) = p(13)*p(335) - p(1215)
p(1255) = p(2)*p(748) - p(1055) - p(1222)
p(1256) = p(32)*p(185) - p(1215)
p(1257) = p(1)*p(786) - p(1083) - p(1255)
p(1258) = p(73)*p(95) - p(1215)
p(1259) = p(13)*p(299) - p(977)
p(1260) = p(46)*p(150) - p(1215)
p(1261) = p(73)*p(84) - p(975)
p(1262) = p(14)*p(335)
p(1263) = p(14)*p(336)
p(1264) = p(43)*p(130) - p(974)
p(1265) = p(8)*p(430) - p(1038)
p(1266) = p(1)*p(794) - p(1093) - p(1265)
p(1267) = p(81)*p(93) - p(1190)
p(1268) = p(2)*p(761) - p(1066) - p(1236)
p(1269) = p(43)*p(160) - p(1190)
p(1270) = p(45)*p(153) - p(1038)
p(1271) = p(18)*p(296) - p(1190)
p(1272) = p(34)*p(160) - p(974)
p(1273) = p(14)*p(302)
p(1274) = p(14)*p(303)
p(1275) = p(14)*p(304)
p(1276) = p(14)*p(258)
p(1277) = p(14)*p(259)
p(1278) = p(4)*p(464) - p(983)
p(1279) = p(1)*p(804) - p(1103) - p(1278)
p(1280) = p(16)*p(310) - p(1240)
p(1281) = p(2)*p(772) - p(1077) - p(1280)
p(1282) = p(4)*p(532) - p(1240)
p(1283) = p(11)*p(310) - p(983)
p(1284) = p(14)*p(305)
p(1285) = p(14)*p(306)
p(1286) = p(14)*p(307)
p(1287) = p(14)*p(263)
p(1288) = p(14)*p(264)
p(1289) = p(14)*p(265)
p(1290) = p(6)*p(464) - p(1104)
p(1291) = p(3)*p(578) - p(1290)
p(1292) = p(14)*p(309)
p(1293) = p(14)*p(310)
p(1294) = p(9)*p(340)
p(1295) = p(9)*p(322)
p(1296) = p(9)*p(323)
p(1297) = p(9)*p(341)
p(1298) = p(24)*p(188) - p(984)
p(1299) = p(50)*p(114) - p(1089)
p(1300) = p(1)*p(813) - p(1108) - p(1299)
p(1301) = p(13)*p(337) - p(1262)
p(1302) = p(2)*p(786) - p(1091) - p(1268)
p(1303) = p(32)*p(188) - p(1262)
p(1304) = p(6)*p(536) - p(1089)
p(1305) = p(73)*p(98) - p(1262)
p(1306) = p(32)*p(173) - p(984)
p(1307) = p(14)*p(337)
p(1308) = p(14)*p(338)
p(1309) = p(46)*p(130) - p(982)
p(1310) = p(1)*p(820) - p(1113) - p(1309)
p(1311) = p(81)*p(95) - p(1232)
p(1312) = p(2)*p(796) - p(1098) - p(1281)
p(1313) = p(46)*p(160) - p(1232)
p(1314) = p(81)*p(84) - p(982)
p(1315) = p(14)*p(315)
p(1316) = p(14)*p(316)
p(1317) = p(14)*p(317)
p(1318) = p(14)*p(270)
p(1319) = p(14)*p(271)
p(1320) = p(8)*p(464) - p(1079)
p(1321) = p(18)*p(310) - p(1242)
p(1322) = p(34)*p(187) - p(1079)
p(1323) = p(14)*p(318)
p(1324) = p(14)*p(319)
p(1325) = p(14)*p(320)
p(1326) = p(9)*p(343)
p(1327) = p(9)*p(344)
p(1328) = p(24)*p(191) - p(987)
p(1329) = p(1)*p(827) - p(1117) - p(1328)
p(1330) = p(13)*p(340) - p(1307)
p(1331) = p(2)*p(815) - p(1112) - p(1312)
p(1332) = p(32)*p(191) - p(1307)
p(1333) = p(13)*p(323) - p(987)
p(1334) = p(14)*p(340)
p(1335) = p(14)*p(341)
p(1336) = p(50)*p(130) - p(1102)
p(1337) = p(81)*p(98) - p(1276)
p(1338) = p(45)*p(173) - p(1102)
p(1339) = p(14)*p(324)
p(1340) = p(14)*p(325)
p(1341) = p(14)*p(326)
p(1342) = p(9)*p(347)
p(1343) = p(24)*p(195) - p(1119)
p(1344) = p(13)*p(343) - p(1334)
p(1345) = p(6)*p(561) - p(1119)
p(1346) = p(14)*p(343)
p(1347) = p(14)*p(344)
p(1348) = p(9)*p(272)
p(1349) = p(9)*p(273)
p(1350) = p(9)*p(274)
p(1351) = p(9)*p(328)
p(1352) = p(9)*p(275)
p(1353) = p(9)*p(329)
p(1354) = p(9)*p(330)
p(1355) = p(9)*p(346)
p(1356) = p(3)*p(570) - p(1120)
p(1357) = p(39)*p(181) - p(1122)
p(1358) = p(41)*p(178) - p(1125)
p(1359) = p(4)*p(566) - p(1129)
p(1360) = p(1)*p(842) - p(1126) - p(1359)
p(1361) = p(1)*p(843) - p(1129)
p(1362) = p(1)*p(844) - p(1130) - p(1360)
p(1363) = p(21)*p(279) - p(1125)
p(1364) = p(1)*p(846) - p(1133) - p(1362)
p(1365) = p(69)*p(101) - p(1122)
p(1366) = p(1)*p(848) - p(1135) - p(1364)
p(1367) = p(11)*p(346) - p(1120)
p(1368) = p(1)*p(850) - p(1140) - p(1139)
p(1369) = p(1)*p(851) - p(1146)
p(1370) = p(3)*p(572) - p(1148)
p(1371) = p(39)*p(183) - p(1150)
p(1372) = p(43)*p(178) - p(1153)
p(1373) = p(1)*p(854) - p(1151) - p(1372)
p(1374) = p(2)*p(843) - p(1143)
p(1375) = p(1)*p(855) - p(1154) - p(1373)
p(1376) = p(7)*p(509) - p(1153)
p(1377) = p(1)*p(857) - p(1157) - p(1375)
p(1378) = p(49)*p(153) - p(1150)
p(1379) = p(1)*p(859) - p(1159) - p(1377)
p(1380) = p(34)*p(194) - p(1148)
p(1381) = p(6)*p(570) - p(1147)
p(1382) = p(1)*p(862) - p(1167) - p(1165)
p(1383) = p(2)*p(851) - p(1147)
p(1384) = p(1)*p(864) - p(1183) - p(1182)
p(1385) = p(1)*p(865) - p(1187)
p(1386) = p(3)*p(574) - p(1191)
p(1387) = p(39)*p(185) - p(1193)
p(1388) = p(46)*p(178) - p(1196)
p(1389) = p(1)*p(868) - p(1194) - p(1388)
p(1390) = p(1)*p(869) - p(1196)
p(1391) = p(1)*p(870) - p(1197) - p(1389)
p(1392) = p(21)*p(299) - p(1193)
p(1393) = p(1)*p(872) - p(1199) - p(1391)
p(1394) = p(84)*p(101) - p(1191)
p(1395) = p(6)*p(572) - p(1178)
p(1396) = p(1)*p(875) - p(1206) - p(1204)
p(1397) = p(8)*p(505) - p(1178)
p(1398) = p(45)*p(181) - p(1189)
p(1399) = p(1)*p(878) - p(1222) - p(1220)
p(1400) = p(2)*p(865) - p(1189)
p(1401) = p(1)*p(880) - p(1236) - p(1235)
p(1402) = p(1)*p(881) - p(1238)
p(1403) = p(3)*p(576) - p(1244)
p(1404) = p(39)*p(188) - p(1246)
p(1405) = p(1)*p(884) - p(1245) - p(1404)
p(1406) = p(2)*p(869) - p(1210)
p(1407) = p(1)*p(885) - p(1247) - p(1405)
p(1408) = p(7)*p(536) - p(1246)
p(1409) = p(1)*p(887) - p(1249) - p(1407)
p(1410) = p(49)*p(173) - p(1244)
p(1411) = p(6)*p(574) - p(1215)
p(1412) = p(1)*p(890) - p(1255) - p(1253)
p(1413) = p(22)*p(285) - p(1215)
p(1414) = p(45)*p(183) - p(1190)
p(1415) = p(1)*p(893) - p(1268) - p(1266)
p(1416) = p(8)*p(518) - p(1190)
p(1417) = p(41)*p(187) - p(1240)
p(1418) = p(1)*p(896) - p(1281) - p(1279)
p(1419) = p(2)*p(881) - p(1240)
p(1420) = p(14)*p(308)
p(1421) = p(4)*p(547) - p(1106) - p(1320)
p(1422) = p(1)*p(900) - p(1291)
p(1423) = p(14)*p(311)
p(1424) = p(3)*p(579) - p(1294)
p(1425) = p(39)*p(191) - p(1296)
p(1426) = p(1)*p(903) - p(1295) - p(1425)
p(1427) = p(1)*p(904) - p(1296)
p(1428) = p(1)*p(905) - p(1297) - p(1426)
p(1429) = p(21)*p(323) - p(1294)
p(1430) = p(6)*p(576) - p(1262)
p(1431) = p(1)*p(908) - p(1302) - p(1300)
p(1432) = p(50)*p(150) - p(1262)
p(1433) = p(45)*p(185) - p(1232)
p(1434) = p(1)*p(911) - p(1312) - p(1310)
p(1435) = p(22)*p(296) - p(1232)
p(1436) = p(43)*p(187) - p(1242)
p(1437) = p(1)*p(915) - p(1322) - p(1320)
p(1438) = p(8)*p(532) - p(1242)
p(1439) = p(14)*p(321)
p(1440) = p(4)*p(578) - p(1293)
p(1441) = p(2)*p(899) - p(1292) - p(1440)
p(1442) = p(2)*p(900) - p(1293)
p(1443) = p(14)*p(339)
p(1444) = p(3)*p(582) - p(1326)
p(1445) = p(2)*p(903) - p(1303) - p(1299)
p(1446) = p(2)*p(904) - p(1304)
p(1447) = p(1)*p(919) - p(1327) - p(1445)
p(1448) = p(7)*p(561) - p(1326)
p(1449) = p(6)*p(579) - p(1307)
p(1450) = p(1)*p(922) - p(1331) - p(1329)
p(1451) = p(73)*p(102) - p(1307)
p(1452) = p(45)*p(188) - p(1276)
p(1453) = p(1)*p(925) - p(1338) - p(1336)
p(1454) = p(50)*p(160) - p(1276)
p(1455) = p(14)*p(327)
p(1456) = p(46)*p(187) - p(1243)
p(1457) = p(2)*p(915) - p(1324) - p(1441)
p(1458) = p(22)*p(310) - p(1243)
p(1459) = p(14)*p(342)
p(1460) = p(3)*p(585) - p(1342)
p(1461) = p(2)*p(919) - p(1332) - p(1329)
p(1462) = p(1)*p(929) - p(1342)
p(1463) = p(6)*p(582) - p(1334)
p(1464) = p(1)*p(931) - p(1345) - p(1343)
p(1465) = p(32)*p(195) - p(1334)
p(1466) = p(45)*p(191) - p(1318)
p(1467) = p(2)*p(925) - p(1340) - p(1457)
p(1468) = p(81)*p(102) - p(1318)
p(1469) = p(14)*p(345)
p(1470) = p(2)*p(928) - p(1344) - p(1343)
p(1471) = p(2)*p(929) - p(1345)
p(1472) = p(6)*p(585) - p(1346)
p(1473) = p(2)*p(931) - p(1347) - p(1467)
p(1474) = p(13)*p(347) - p(1346)
p(1475) = p(14)*p(347)
p(1476) = p(3)*p(566) - p(1351)
p(1477) = p(7)*p(566) - p(1349)
p(1478) = p(49)*p(178) - p(1348)
p(1479) = p(39)*p(194) - p(1352)
p(1480) = p(3)*p(588) - p(1355)
p(1481) = p(1)*p(938) - p(1356)
p(1482) = p(2)*p(958) - p(1481)
p(1483) = p(1)*p(940) - p(1370)
p(1484) = p(8)*p(588) - p(1483)
p(1485) = p(1)*p(942) - p(1386)
p(1486) = p(22)*p(346) - p(1485)
p(1487) = p(1)*p(944) - p(1403)
p(1488) = p(50)*p(194) - p(1487)
p(1489) = p(1)*p(946) - p(1424)
p(1490) = p(101)*p(102) - p(1489)
p(1491) = p(6)*p(578) - p(1443)
p(1492) = p(1)*p(949) - p(1444)
p(1493) = p(49)*p(195) - p(1492)
p(1494) = p(8)*p(578) - p(1423)
p(1495) = p(1)*p(952) - p(1460)
p(1496) = p(21)*p(347) - p(1495)
p(1497) = p(50)*p(187) - p(1420)
p(1498) = p(1)*p(955) - p(1470)
p(1499) = p(7)*p(589) - p(1498)
p(1500) = p(45)*p(195) - p(1455)
p(1501) = p(2)*p(955) - p(1472)
p(1502) = p(1)*p(959) - p(1501)
p(1503) = p(6)*p(589) - p(1475)
p(1504) = p(1)*p(958) - p(1480)
p(1505) = p(2)*p(959) - p(1503)
p(1506) = p(9)*p(348)
p(1507) = p(9)*p(349)
p(1508) = p(9)*p(360)
p(1509) = p(9)*p(361)
p(1510) = p(9)*p(409)
p(1511) = p(9)*p(428)
p(1512) = p(9)*p(368)
p(1513) = p(9)*p(350)
p(1514) = p(9)*p(369)
p(1515) = p(9)*p(351)
p(1516) = p(9)*p(370)
p(1517) = p(9)*p(371)
p(1518) = p(9)*p(372)
p(1519) = p(9)*p(429)
p(1520) = p(9)*p(375)
p(1521) = p(9)*p(435)
p(1522) = p(9)*p(352)
p(1523) = p(9)*p(376)
p(1524) = p(9)*p(377)
p(1525) = p(9)*p(436)
p(1526) = p(9)*p(378)
p(1527) = p(9)*p(437)
p(1528) = p(9)*p(451)
p(1529) = p(9)*p(382)
p(1530) = p(9)*p(353)
p(1531) = p(9)*p(383)
p(1532) = p(9)*p(384)
p(1533) = p(9)*p(452)
p(1534) = p(9)*p(459)
p(1535) = p(9)*p(386)
p(1536) = p(9)*p(460)
p(1537) = p(9)*p(387)
p(1538) = p(9)*p(388)
p(1539) = p(9)*p(461)
p(1540) = p(9)*p(462)
p(1541) = p(9)*p(463)
p(1542) = p(9)*p(466)
p(1543) = p(9)*p(467)
p(1544) = p(9)*p(533)
p(1545) = p(9)*p(476)
p(1546) = p(9)*p(390)
p(1547) = p(9)*p(391)
p(1548) = p(9)*p(477)
p(1549) = p(9)*p(481)
p(1550) = p(9)*p(482)
p(1551) = p(9)*p(483)
p(1552) = p(9)*p(545)
p(1553) = p(9)*p(549)
p(1554) = p(9)*p(488)
p(1555) = p(9)*p(489)
p(1556) = p(9)*p(559)
p(1557) = p(9)*p(565)
p(1558) = p(9)*p(401)
p(1559) = p(9)*p(402)
p(1560) = p(9)*p(403)
p(1561) = p(9)*p(504)
p(1562) = p(9)*p(354)
p(1563) = p(9)*p(355)
p(1564) = p(9)*p(404)
p(1565) = p(9)*p(356)
p(1566) = p(9)*p(357)
p(1567) = p(9)*p(405)
p(1568) = p(9)*p(358)
p(1569) = p(9)*p(406)
p(1570) = p(9)*p(359)
p(1571) = p(9)*p(407)
p(1572) = p(9)*p(408)
p(1573) = p(9)*p(505)
p(1574) = p(14)*p(566)
p(1575) = p(14)*p(567)
p(1576) = p(14)*p(568)
p(1577) = p(14)*p(569)
p(1578) = p(9)*p(514)
p(1579) = p(9)*p(416)
p(1580) = p(9)*p(417)
p(1581) = p(9)*p(418)
p(1582) = p(9)*p(419)
p(1583) = p(9)*p(515)
p(1584) = p(9)*p(420)
p(1585) = p(9)*p(362)
p(1586) = p(9)*p(363)
p(1587) = p(9)*p(364)
p(1588) = p(9)*p(421)
p(1589) = p(9)*p(422)
p(1590) = p(9)*p(365)
p(1591) = p(9)*p(366)
p(1592) = p(9)*p(423)
p(1593) = p(9)*p(424)
p(1594) = p(9)*p(367)
p(1595) = p(9)*p(425)
p(1596) = p(9)*p(426)
p(1597) = p(9)*p(427)
p(1598) = p(9)*p(516)
p(1599) = p(14)*p(495)
p(1600) = p(14)*p(496)
p(1601) = p(14)*p(497)
p(1602) = p(14)*p(498)
p(1603) = p(14)*p(499)
p(1604) = p(14)*p(500)
p(1605) = p(14)*p(501)
p(1606) = p(14)*p(502)
p(1607) = p(14)*p(503)
p(1608) = p(9)*p(430)
p(1609) = p(9)*p(431)
p(1610) = p(9)*p(517)
p(1611) = p(9)*p(373)
p(1612) = p(9)*p(432)
p(1613) = p(9)*p(374)
p(1614) = p(9)*p(433)
p(1615) = p(9)*p(434)
p(1616) = p(9)*p(518)
p(1617) = p(14)*p(401)
p(1618) = p(14)*p(402)
p(1619) = p(14)*p(403)
p(1620) = p(14)*p(404)
p(1621) = p(14)*p(405)
p(1622) = p(14)*p(406)
p(1623) = p(14)*p(407)
p(1624) = p(14)*p(408)
p(1625) = p(14)*p(360)
p(1626) = p(14)*p(361)
p(1627) = p(9)*p(525)
p(1628) = p(9)*p(442)
p(1629) = p(9)*p(443)
p(1630) = p(9)*p(444)
p(1631) = p(9)*p(526)
p(1632) = p(9)*p(445)
p(1633) = p(9)*p(379)
p(1634) = p(9)*p(380)
p(1635) = p(9)*p(446)
p(1636) = p(9)*p(447)
p(1637) = p(9)*p(381)
p(1638) = p(9)*p(448)
p(1639) = p(9)*p(449)
p(1640) = p(9)*p(450)
p(1641) = p(9)*p(527)
p(1642) = p(14)*p(506)
p(1643) = p(14)*p(507)
p(1644) = p(14)*p(508)
p(1645) = p(14)*p(509)
p(1646) = p(14)*p(510)
p(1647) = p(14)*p(511)
p(1648) = p(14)*p(512)
p(1649) = p(14)*p(513)
p(1650) = p(9)*p(528)
p(1651) = p(9)*p(453)
p(1652) = p(9)*p(454)
p(1653) = p(9)*p(529)
p(1654) = p(9)*p(455)
p(1655) = p(9)*p(385)
p(1656) = p(9)*p(456)
p(1657) = p(9)*p(457)
p(1658) = p(9)*p(458)
p(1659) = p(9)*p(530)
p(1660) = p(14)*p(416)
p(1661) = p(14)*p(417)
p(1662) = p(14)*p(418)
p(1663) = p(14)*p(419)
p(1664) = p(14)*p(420)
p(1665) = p(14)*p(421)
p(1666) = p(14)*p(422)
p(1667) = p(14)*p(423)
p(1668) = p(14)*p(424)
p(1669) = p(14)*p(425)
p(1670) = p(14)*p(426)
p(1671) = p(14)*p(427)
p(1672) = p(14)*p(368)
p(1673) = p(14)*p(369)
p(1674) = p(14)*p(370)
p(1675) = p(14)*p(371)
p(1676) = p(14)*p(372)
p(1677) = p(9)*p(464)
p(1678) = p(9)*p(531)
p(1679) = p(9)*p(465)
p(1680) = p(9)*p(532)
p(1681) = p(14)*p(430)
p(1682) = p(14)*p(431)
p(1683) = p(14)*p(432)
p(1684) = p(14)*p(433)
p(1685) = p(14)*p(434)
p(1686) = p(14)*p(375)
p(1687) = p(14)*p(376)
p(1688) = p(14)*p(377)
p(1689) = p(14)*p(378)
p(1690) = p(9)*p(539)
p(1691) = p(9)*p(470)
p(1692) = p(9)*p(471)
p(1693) = p(9)*p(540)
p(1694) = p(9)*p(472)
p(1695) = p(9)*p(389)
p(1696) = p(9)*p(473)
p(1697) = p(9)*p(474)
p(1698) = p(9)*p(475)
p(1699) = p(9)*p(541)
p(1700) = p(14)*p(519)
p(1701) = p(14)*p(520)
p(1702) = p(14)*p(521)
p(1703) = p(14)*p(522)
p(1704) = p(14)*p(523)
p(1705) = p(14)*p(524)
p(1706) = p(9)*p(542)
p(1707) = p(9)*p(478)
p(1708) = p(9)*p(543)
p(1709) = p(9)*p(479)
p(1710) = p(9)*p(480)
p(1711) = p(9)*p(544)
p(1712) = p(14)*p(442)
p(1713) = p(14)*p(443)
p(1714) = p(14)*p(444)
p(1715) = p(14)*p(445)
p(1716) = p(14)*p(446)
p(1717) = p(14)*p(447)
p(1718) = p(14)*p(448)
p(1719) = p(14)*p(449)
p(1720) = p(14)*p(450)
p(1721) = p(14)*p(382)
p(1722) = p(14)*p(383)
p(1723) = p(14)*p(384)
p(1724) = p(9)*p(546)
p(1725) = p(9)*p(547)
p(1726) = p(9)*p(548)
p(1727) = p(14)*p(453)
p(1728) = p(14)*p(454)
p(1729) = p(14)*p(455)
p(1730) = p(14)*p(456)
p(1731) = p(14)*p(457)
p(1732) = p(14)*p(458)
p(1733) = p(14)*p(386)
p(1734) = p(14)*p(387)
p(1735) = p(14)*p(388)
p(1736) = p(9)*p(578)
p(1737) = p(14)*p(464)
p(1738) = p(14)*p(465)
p(1739) = p(9)*p(553)
p(1740) = p(9)*p(485)
p(1741) = p(9)*p(554)
p(1742) = p(9)*p(486)
p(1743) = p(9)*p(487)
p(1744) = p(9)*p(555)
p(1745) = p(14)*p(534)
p(1746) = p(14)*p(535)
p(1747) = p(14)*p(536)
p(1748) = p(14)*p(537)
p(1749) = p(14)*p(538)
p(1750) = p(9)*p(556)
p(1751) = p(9)*p(557)
p(1752) = p(9)*p(558)
p(1753) = p(14)*p(470)
p(1754) = p(14)*p(471)
p(1755) = p(14)*p(472)
p(1756) = p(14)*p(473)
p(1757) = p(14)*p(474)
p(1758) = p(14)*p(475)
p(1759) = p(14)*p(390)
p(1760) = p(14)*p(391)
p(1761) = p(9)*p(581)
p(1762) = p(14)*p(478)
p(1763) = p(14)*p(479)
p(1764) = p(14)*p(480)
p(1765) = p(9)*p(562)
p(1766) = p(9)*p(563)
p(1767) = p(9)*p(564)
p(1768) = p(14)*p(550)
p(1769) = p(14)*p(551)
p(1770) = p(14)*p(552)
p(1771) = p(9)*p(584)
p(1772) = p(14)*p(485)
p(1773) = p(14)*p(486)
p(1774) = p(14)*p(487)
p(1775) = p(9)*p(587)
p(1776) = p(14)*p(560)
p(1777) = p(14)*p(561)
p(1778) = p(9)*p(570)
p(1779) = p(9)*p(495)
p(1780) = p(9)*p(392)
p(1781) = p(9)*p(496)
p(1782) = p(9)*p(393)
p(1783) = p(9)*p(394)
p(1784) = p(9)*p(497)
p(1785) = p(9)*p(395)
p(1786) = p(9)*p(396)
p(1787) = p(9)*p(397)
p(1788) = p(9)*p(498)
p(1789) = p(9)*p(398)
p(1790) = p(9)*p(399)
p(1791) = p(9)*p(499)
p(1792) = p(9)*p(500)
p(1793) = p(9)*p(400)
p(1794) = p(9)*p(501)
p(1795) = p(9)*p(502)
p(1796) = p(9)*p(503)
p(1797) = p(9)*p(571)
p(1798) = p(1)*p(1136) - p(1558)
p(1799) = p(21)*p(401) - p(1565)
p(1800) = p(101)*p(114) - p(1570)
p(1801) = p(24)*p(346) - p(1573)
p(1802) = p(3)*p(850) - p(1561) - p(1801)
p(1803) = p(1)*p(1140) - p(1561) - p(1802)
p(1804) = p(1)*p(1141) - p(1564) - p(1803)
p(1805) = p(1)*p(1142) - p(1567) - p(1804)
p(1806) = p(13)*p(566) - p(1558)
p(1807) = p(73)*p(178) - p(1565)
p(1808) = p(39)*p(285) - p(1570)
p(1809) = p(3)*p(851) - p(1573)
p(1810) = p(14)*p(588)
p(1811) = p(9)*p(572)
p(1812) = p(9)*p(506)
p(1813) = p(9)*p(410)
p(1814) = p(9)*p(507)
p(1815) = p(9)*p(411)
p(1816) = p(9)*p(412)
p(1817) = p(9)*p(508)
p(1818) = p(9)*p(413)
p(1819) = p(9)*p(414)
p(1820) = p(9)*p(509)
p(1821) = p(9)*p(510)
p(1822) = p(9)*p(415)
p(1823) = p(9)*p(511)
p(1824) = p(9)*p(512)
p(1825) = p(9)*p(513)
p(1826) = p(9)*p(573)
p(1827) = p(24)*p(331) - p(1510)
p(1828) = p(91)*p(114) - p(1507)
p(1829) = p(16)*p(401) - p(1506)
p(1830) = p(2)*p(1136) - p(1574)
p(1831) = p(1)*p(1163) - p(1581) - p(1830)
p(1832) = p(1)*p(1164) - p(1582) - p(1831)
p(1833) = p(1)*p(1165) - p(1583) - p(1832)
p(1834) = p(13)*p(570) - p(1810)
p(1835) = p(2)*p(1140) - p(1577) - p(1834)
p(1836) = p(32)*p(331) - p(1810)
p(1837) = p(1)*p(1167) - p(1583) - p(1835)
p(1838) = p(73)*p(181) - p(1810)
p(1839) = p(1)*p(1169) - p(1588) - p(1837)
p(1840) = p(91)*p(150) - p(1810)
p(1841) = p(6)*p(843) - p(1574)
p(1842) = p(41)*p(285) - p(1810)
p(1843) = p(32)*p(279) - p(1506)
p(1844) = p(16)*p(505) - p(1810)
p(1845) = p(69)*p(150) - p(1507)
p(1846) = p(4)*p(851) - p(1810)
p(1847) = p(11)*p(505) - p(1510)
p(1848) = p(14)*p(570)
p(1849) = p(14)*p(571)
p(1850) = p(1)*p(1180) - p(1608)
p(1851) = p(21)*p(430) - p(1613)
p(1852) = p(101)*p(130) - p(1616)
p(1853) = p(3)*p(864) - p(1610) - p(1852)
p(1854) = p(1)*p(1183) - p(1610) - p(1853)
p(1855) = p(1)*p(1184) - p(1612) - p(1854)
p(1856) = p(81)*p(178) - p(1608)
p(1857) = p(39)*p(296) - p(1613)
p(1858) = p(3)*p(865) - p(1616)
p(1859) = p(14)*p(504)
p(1860) = p(14)*p(505)
p(1861) = p(14)*p(409)
p(1862) = p(9)*p(574)
p(1863) = p(9)*p(519)
p(1864) = p(9)*p(438)
p(1865) = p(9)*p(520)
p(1866) = p(9)*p(439)
p(1867) = p(9)*p(440)
p(1868) = p(9)*p(521)
p(1869) = p(9)*p(441)
p(1870) = p(9)*p(522)
p(1871) = p(9)*p(523)
p(1872) = p(9)*p(524)
p(1873) = p(9)*p(575)
p(1874) = p(24)*p(333) - p(1511)
p(1875) = p(93)*p(114) - p(1513)
p(1876) = p(18)*p(401) - p(1516)
p(1877) = p(1)*p(1202) - p(1629) - p(1876)
p(1878) = p(1)*p(1203) - p(1630) - p(1877)
p(1879) = p(1)*p(1204) - p(1631) - p(1878)
p(1880) = p(13)*p(572) - p(1848)
p(1881) = p(2)*p(1167) - p(1607) - p(1853)
p(1882) = p(32)*p(333) - p(1848)
p(1883) = p(1)*p(1206) - p(1631) - p(1881)
p(1884) = p(73)*p(183) - p(1848)
p(1885) = p(1)*p(1208) - p(1635) - p(1883)
p(1886) = p(93)*p(150) - p(1848)
p(1887) = p(13)*p(509) - p(1516)
p(1888) = p(43)*p(285) - p(1848)
p(1889) = p(73)*p(153) - p(1513)
p(1890) = p(18)*p(505) - p(1848)
p(1891) = p(34)*p(285) - p(1511)
p(1892) = p(14)*p(572)
p(1893) = p(14)*p(573)
p(1894) = p(91)*p(130) - p(1525)
p(1895) = p(16)*p(430) - p(1522)
p(1896) = p(2)*p(1180) - p(1617)
p(1897) = p(1)*p(1219) - p(1652) - p(1896)
p(1898) = p(1)*p(1220) - p(1653) - p(1897)
p(1899) = p(81)*p(181) - p(1860)
p(1900) = p(2)*p(1183) - p(1620) - p(1899)
p(1901) = p(91)*p(160) - p(1860)
p(1902) = p(1)*p(1222) - p(1653) - p(1900)
p(1903) = p(41)*p(296) - p(1860)
p(1904) = p(45)*p(279) - p(1617)
p(1905) = p(16)*p(518) - p(1860)
p(1906) = p(69)*p(160) - p(1522)
p(1907) = p(4)*p(865) - p(1860)
p(1908) = p(11)*p(518) - p(1525)
p(1909) = p(14)*p(514)
p(1910) = p(14)*p(515)
p(1911) = p(14)*p(516)
p(1912) = p(14)*p(428)
p(1913) = p(14)*p(429)
p(1914) = p(1)*p(1234) - p(1677)
p(1915) = p(21)*p(464) - p(1680)
p(1916) = p(3)*p(880) - p(1678) - p(1915)
p(1917) = p(1)*p(1236) - p(1678) - p(1916)
p(1918) = p(39)*p(310) - p(1677)
p(1919) = p(3)*p(881) - p(1680)
p(1920) = p(14)*p(517)
p(1921) = p(14)*p(518)
p(1922) = p(14)*p(435)
p(1923) = p(14)*p(436)
p(1924) = p(14)*p(437)
p(1925) = p(9)*p(576)
p(1926) = p(9)*p(534)
p(1927) = p(9)*p(468)
p(1928) = p(9)*p(535)
p(1929) = p(9)*p(469)
p(1930) = p(9)*p(536)
p(1931) = p(9)*p(537)
p(1932) = p(9)*p(538)
p(1933) = p(9)*p(577)
p(1934) = p(24)*p(335) - p(1528)
p(1935) = p(95)*p(114) - p(1530)
p(1936) = p(22)*p(401) - p(1645)
p(1937) = p(1)*p(1252) - p(1692) - p(1936)
p(1938) = p(1)*p(1253) - p(1693) - p(1937)
p(1939) = p(13)*p(574) - p(1892)
p(1940) = p(2)*p(1206) - p(1649) - p(1900)
p(1941) = p(32)*p(335) - p(1892)
p(1942) = p(1)*p(1255) - p(1693) - p(1940)
p(1943) = p(73)*p(185) - p(1892)
p(1944) = p(6)*p(869) - p(1645)
p(1945) = p(95)*p(150) - p(1892)
p(1946) = p(32)*p(299) - p(1530)
p(1947) = p(46)*p(285) - p(1892)
p(1948) = p(84)*p(150) - p(1528)
p(1949) = p(14)*p(574)
p(1950) = p(14)*p(575)
p(1951) = p(93)*p(130) - p(1527)
p(1952) = p(18)*p(430) - p(1526)
p(1953) = p(1)*p(1265) - p(1707) - p(1952)
p(1954) = p(1)*p(1266) - p(1708) - p(1953)
p(1955) = p(81)*p(183) - p(1861)
p(1956) = p(2)*p(1222) - p(1665) - p(1916)
p(1957) = p(93)*p(160) - p(1861)
p(1958) = p(1)*p(1268) - p(1708) - p(1956)
p(1959) = p(43)*p(296) - p(1861)
p(1960) = p(81)*p(153) - p(1526)
p(1961) = p(18)*p(518) - p(1861)
p(1962) = p(34)*p(296) - p(1527)
p(1963) = p(14)*p(525)
p(1964) = p(14)*p(526)
p(1965) = p(14)*p(527)
p(1966) = p(14)*p(451)
p(1967) = p(14)*p(452)
p(1968) = p(16)*p(464) - p(1543)
p(1969) = p(2)*p(1234) - p(1681)
p(1970) = p(1)*p(1279) - p(1725) - p(1969)
p(1971) = p(41)*p(310) - p(1921)
p(1972) = p(2)*p(1236) - p(1683) - p(1971)
p(1973) = p(16)*p(532) - p(1921)
p(1974) = p(69)*p(187) - p(1681)
p(1975) = p(4)*p(881) - p(1921)
p(1976) = p(11)*p(532) - p(1543)
p(1977) = p(14)*p(528)
p(1978) = p(14)*p(529)
p(1979) = p(14)*p(530)
p(1980) = p(14)*p(459)
p(1981) = p(14)*p(460)
p(1982) = p(14)*p(461)
p(1983) = p(14)*p(462)
p(1984) = p(14)*p(463)
p(1985) = p(1)*p(1290) - p(1736)
p(1986) = p(4)*p(806) - p(1761) - p(1732)
p(1987) = p(3)*p(900) - p(1736)
p(1988) = p(14)*p(531)
p(1989) = p(14)*p(532)
p(1990) = p(14)*p(466)
p(1991) = p(14)*p(467)
p(1992) = p(9)*p(579)
p(1993) = p(9)*p(550)
p(1994) = p(9)*p(484)
p(1995) = p(9)*p(551)
p(1996) = p(9)*p(552)
p(1997) = p(9)*p(580)
p(1998) = p(24)*p(337) - p(1545)
p(1999) = p(98)*p(114) - p(1547)
p(2000) = p(1)*p(1299) - p(1740) - p(1999)
p(2001) = p(1)*p(1300) - p(1741) - p(2000)
p(2002) = p(13)*p(576) - p(1949)
p(2003) = p(2)*p(1255) - p(1705) - p(1956)
p(2004) = p(32)*p(337) - p(1949)
p(2005) = p(1)*p(1302) - p(1741) - p(2003)
p(2006) = p(73)*p(188) - p(1949)
p(2007) = p(13)*p(536) - p(1547)
p(2008) = p(98)*p(150) - p(1949)
p(2009) = p(73)*p(173) - p(1545)
p(2010) = p(14)*p(576)
p(2011) = p(14)*p(577)
p(2012) = p(95)*p(130) - p(1540)
p(2013) = p(22)*p(430) - p(1674)
p(2014) = p(1)*p(1310) - p(1751) - p(2013)
p(2015) = p(81)*p(185) - p(1912)
p(2016) = p(2)*p(1268) - p(1716) - p(1972)
p(2017) = p(95)*p(160) - p(1912)
p(2018) = p(45)*p(299) - p(1674)
p(2019) = p(46)*p(296) - p(1912)
p(2020) = p(84)*p(160) - p(1540)
p(2021) = p(14)*p(539)
p(2022) = p(14)*p(540)
p(2023) = p(14)*p(541)
p(2024) = p(14)*p(476)
p(2025) = p(14)*p(477)
p(2026) = p(18)*p(464) - p(1544)
p(2027) = p(1)*p(1320) - p(1761) - p(2026)
p(2028) = p(43)*p(310) - p(1923)
p(2029) = p(2)*p(1281) - p(1730) - p(1986)
p(2030) = p(18)*p(532) - p(1923)
p(2031) = p(34)*p(310) - p(1544)
p(2032) = p(14)*p(542)
p(2033) = p(14)*p(543)
p(2034) = p(14)*p(544)
p(2035) = p(14)*p(481)
p(2036) = p(14)*p(482)
p(2037) = p(14)*p(483)
p(2038) = p(2)*p(1290) - p(1737)
p(2039) = p(4)*p(900) - p(1989)
p(2040) = p(11)*p(578) - p(1737)
p(2041) = p(14)*p(546)
p(2042) = p(14)*p(547)
p(2043) = p(14)*p(548)
p(2044) = p(9)*p(582)
p(2045) = p(9)*p(560)
p(2046) = p(9)*p(561)
p(2047) = p(9)*p(583)
p(2048) = p(24)*p(340) - p(1554)
p(2049) = p(102)*p(114) - p(1747)
p(2050) = p(1)*p(1329) - p(1766) - p(2049)
p(2051) = p(13)*p(579) - p(2010)
p(2052) = p(2)*p(1302) - p(1749) - p(2016)
p(2053) = p(32)*p(340) - p(2010)
p(2054) = p(6)*p(904) - p(1747)
p(2055) = p(73)*p(191) - p(2010)
p(2056) = p(32)*p(323) - p(1554)
p(2057) = p(14)*p(579)
p(2058) = p(14)*p(580)
p(2059) = p(98)*p(130) - p(1552)
p(2060) = p(1)*p(1336) - p(1771) - p(2059)
p(2061) = p(81)*p(188) - p(1966)
p(2062) = p(2)*p(1312) - p(1756) - p(2029)
p(2063) = p(98)*p(160) - p(1966)
p(2064) = p(81)*p(173) - p(1552)
p(2065) = p(14)*p(553)
p(2066) = p(14)*p(554)
p(2067) = p(14)*p(555)
p(2068) = p(14)*p(488)
p(2069) = p(14)*p(489)
p(2070) = p(22)*p(464) - p(1689)
p(2071) = p(46)*p(310) - p(1924)
p(2072) = p(84)*p(187) - p(1689)
p(2073) = p(14)*p(556)
p(2074) = p(14)*p(557)
p(2075) = p(14)*p(558)
p(2076) = p(9)*p(585)
p(2077) = p(9)*p(586)
p(2078) = p(24)*p(343) - p(1557)
p(2079) = p(1)*p(1343) - p(1775) - p(2078)
p(2080) = p(13)*p(582) - p(2057)
p(2081) = p(2)*p(1331) - p(1770) - p(2062)
p(2082) = p(32)*p(343) - p(2057)
p(2083) = p(13)*p(561) - p(1557)
p(2084) = p(14)*p(582)
p(2085) = p(14)*p(583)
p(2086) = p(102)*p(130) - p(1760)
p(2087) = p(81)*p(191) - p(2024)
p(2088) = p(45)*p(323) - p(1760)
p(2089) = p(14)*p(562)
p(2090) = p(14)*p(563)
p(2091) = p(14)*p(564)
p(2092) = p(9)*p(589)
p(2093) = p(24)*p(347) - p(1777)
p(2094) = p(13)*p(585) - p(2084)
p(2095) = p(6)*p(929) - p(1777)
p(2096) = p(14)*p(585)
p(2097) = p(14)*p(586)
p(2098) = p(9)*p(490)
p(2099) = p(9)*p(491)
p(2100) = p(9)*p(492)
p(2101) = p(9)*p(566)
p(2102) = p(9)*p(493)
p(2103) = p(9)*p(567)
p(2104) = p(9)*p(494)
p(2105) = p(9)*p(568)
p(2106) = p(9)*p(569)
p(2107) = p(9)*p(588)
p(2108) = p(3)*p(938) - p(1778)
p(2109) = p(39)*p(331) - p(1780)
p(2110) = p(91)*p(178) - p(1783)
p(2111) = p(16)*p(566) - p(1787)
p(2112) = p(1)*p(1359) - p(1784) - p(2111)
p(2113) = p(2)*p(1476) - p(2112)
p(2114) = p(1)*p(1360) - p(1788) - p(2112)
p(2115) = p(7)*p(843) - p(1787)
p(2116) = p(1)*p(1362) - p(1792) - p(2114)
p(2117) = p(49)*p(279) - p(1783)
p(2118) = p(1)*p(1364) - p(1795) - p(2116)
p(2119) = p(69)*p(194) - p(1780)
p(2120) = p(1)*p(1366) - p(1797) - p(2118)
p(2121) = p(11)*p(588) - p(1778)
p(2122) = p(1)*p(1368) - p(1802) - p(1801)
p(2123) = p(1)*p(1369) - p(1809)
p(2124) = p(3)*p(940) - p(1811)
p(2125) = p(39)*p(333) - p(1813)
p(2126) = p(93)*p(178) - p(1816)
p(2127) = p(18)*p(566) - p(1820)
p(2128) = p(1)*p(1373) - p(1817) - p(2127)
p(2129) = p(1)*p(1374) - p(1820)
p(2130) = p(1)*p(1375) - p(1821) - p(2128)
p(2131) = p(21)*p(509) - p(1816)
p(2132) = p(1)*p(1377) - p(1824) - p(2130)
p(2133) = p(101)*p(153) - p(1813)
p(2134) = p(1)*p(1379) - p(1826) - p(2132)
p(2135) = p(34)*p(346) - p(1811)
p(2136) = p(6)*p(938) - p(1810)
p(2137) = p(1)*p(1382) - p(1835) - p(1833)
p(2138) = p(2)*p(1369) - p(1810)
p(2139) = p(1)*p(1384) - p(1853) - p(1852)
p(2140) = p(1)*p(1385) - p(1858)
p(2141) = p(3)*p(942) - p(1862)
p(2142) = p(39)*p(335) - p(1864)
p(2143) = p(95)*p(178) - p(1867)
p(2144) = p(1)*p(1388) - p(1865) - p(2143)
p(2145) = p(2)*p(1374) - p(1841)
p(2146) = p(1)*p(1389) - p(1868) - p(2144)
p(2147) = p(7)*p(869) - p(1867)
p(2148) = p(1)*p(1391) - p(1871) - p(2146)
p(2149) = p(49)*p(299) - p(1864)
p(2150) = p(1)*p(1393) - p(1873) - p(2148)
p(2151) = p(84)*p(194) - p(1862)
p(2152) = p(6)*p(940) - p(1848)
p(2153) = p(1)*p(1396) - p(1881) - p(1879)
p(2154) = p(8)*p(851) - p(1848)
p(2155) = p(45)*p(331) - p(1860)
p(2156) = p(1)*p(1399) - p(1900) - p(1898)
p(2157) = p(2)*p(1385) - p(1860)
p(2158) = p(1)*p(1401) - p(1916) - p(1915)
p(2159) = p(1)*p(1402) - p(1919)
p(2160) = p(3)*p(944) - p(1925)
p(2161) = p(39)*p(337) - p(1927)
p(2162) = p(98)*p(178) - p(1930)
p(2163) = p(1)*p(1405) - p(1928) - p(2162)
p(2164) = p(1)*p(1406) - p(1930)
p(2165) = p(1)*p(1407) - p(1931) - p(2163)
p(2166) = p(21)*p(536) - p(1927)
p(2167) = p(1)*p(1409) - p(1933) - p(2165)
p(2168) = p(101)*p(173) - p(1925)
p(2169) = p(6)*p(942) - p(1892)
p(2170) = p(1)*p(1412) - p(1940) - p(1938)
p(2171) = p(22)*p(505) - p(1892)
p(2172) = p(45)*p(333) - p(1861)
p(2173) = p(1)*p(1415) - p(1956) - p(1954)
p(2174) = p(8)*p(865) - p(1861)
p(2175) = p(91)*p(187) - p(1921)
p(2176) = p(1)*p(1418) - p(1972) - p(1970)
p(2177) = p(2)*p(1402) - p(1921)
p(2178) = p(1)*p(1421) - p(1986) - p(1985)
p(2179) = p(1)*p(1422) - p(1987)
p(2180) = p(14)*p(533)
p(2181) = p(3)*p(946) - p(1992)
p(2182) = p(39)*p(340) - p(1994)
p(2183) = p(1)*p(1425) - p(1993) - p(2182)
p(2184) = p(2)*p(1406) - p(1944)
p(2185) = p(1)*p(1426) - p(1995) - p(2183)
p(2186) = p(7)*p(904) - p(1994)
p(2187) = p(1)*p(1428) - p(1997) - p(2185)
p(2188) = p(49)*p(323) - p(1992)
p(2189) = p(6)*p(944) - p(1949)
p(2190) = p(1)*p(1431) - p(2003) - p(2001)
p(2191) = p(50)*p(285) - p(1949)
p(2192) = p(45)*p(335) - p(1912)
p(2193) = p(1)*p(1434) - p(2016) - p(2014)
p(2194) = p(22)*p(518) - p(1912)
p(2195) = p(93)*p(187) - p(1923)
p(2196) = p(1)*p(1437) - p(2029) - p(2027)
p(2197) = p(8)*p(881) - p(1923)
p(2198) = p(14)*p(545)
p(2199) = p(16)*p(578) - p(1989)
p(2200) = p(1)*p(1441) - p(2040) - p(2038)
p(2201) = p(2)*p(1422) - p(1989)
p(2202) = p(14)*p(549)
p(2203) = p(6)*p(899) - p(2042) - p(2041)
p(2204) = p(1)*p(1491) - p(2203)
p(2205) = p(14)*p(578)
p(2206) = p(3)*p(949) - p(2044)
p(2207) = p(39)*p(343) - p(2046)
p(2208) = p(1)*p(1445) - p(2045) - p(2207)
p(2209) = p(1)*p(1446) - p(2046)
p(2210) = p(1)*p(1447) - p(2047) - p(2208)
p(2211) = p(21)*p(561) - p(2044)
p(2212) = p(6)*p(946) - p(2010)
p(2213) = p(1)*p(1450) - p(2052) - p(2050)
p(2214) = p(102)*p(150) - p(2010)
p(2215) = p(45)*p(337) - p(1966)
p(2216) = p(1)*p(1453) - p(2062) - p(2060)
p(2217) = p(50)*p(296) - p(1966)
p(2218) = p(95)*p(187) - p(1924)
p(2219) = p(1)*p(1457) - p(2072) - p(2070)
p(2220) = p(22)*p(532) - p(1924)
p(2221) = p(14)*p(559)
p(2222) = p(18)*p(578) - p(1991)
p(2223) = p(2)*p(1441) - p(2042) - p(2203)
p(2224) = p(8)*p(900) - p(1991)
p(2225) = p(14)*p(581)
p(2226) = p(3)*p(952) - p(2076)
p(2227) = p(2)*p(1445) - p(2053) - p(2049)
p(2228) = p(2)*p(1446) - p(2054)
p(2229) = p(1)*p(1461) - p(2077) - p(2227)
p(2230) = p(7)*p(929) - p(2076)
p(2231) = p(6)*p(949) - p(2057)
p(2232) = p(1)*p(1464) - p(2081) - p(2079)
p(2233) = p(73)*p(195) - p(2057)
p(2234) = p(45)*p(340) - p(2024)
p(2235) = p(1)*p(1467) - p(2088) - p(2086)
p(2236) = p(102)*p(160) - p(2024)
p(2237) = p(14)*p(565)
p(2238) = p(98)*p(187) - p(1983)
p(2239) = p(2)*p(1457) - p(2074) - p(2223)
p(2240) = p(50)*p(310) - p(1983)
p(2241) = p(14)*p(584)
p(2242) = p(3)*p(955) - p(2092)
p(2243) = p(2)*p(1461) - p(2082) - p(2079)
p(2244) = p(1)*p(1471) - p(2092)
p(2245) = p(6)*p(952) - p(2084)
p(2246) = p(1)*p(1473) - p(2095) - p(2093)
p(2247) = p(32)*p(347) - p(2084)
p(2248) = p(45)*p(343) - p(2068)
p(2249) = p(2)*p(1467) - p(2090) - p(2239)
p(2250) = p(81)*p(195) - p(2068)
p(2251) = p(14)*p(587)
p(2252) = p(2)*p(1470) - p(2094) - p(2093)
p(2253) = p(2)*p(1471) - p(2095)
p(2254) = p(6)*p(955) - p(2096)
p(2255) = p(2)*p(1473) - p(2097) - p(2249)
p(2256) = p(13)*p(589) - p(2096)
p(2257) = p(14)*p(589)
p(2258) = p(1)*p(1476) - p(2101)
p(2259) = p(21)*p(566) - p(2098)
p(2260) = p(101)*p(178) - p(2099)
p(2261) = p(39)*p(346) - p(2104)
p(2262) = p(3)*p(958) - p(2107)
p(2263) = p(1)*p(1481) - p(2108)
p(2264) = p(2)*p(1504) - p(2263)
p(2265) = p(1)*p(1483) - p(2124)
p(2266) = p(8)*p(958) - p(2265)
p(2267) = p(1)*p(1485) - p(2141)
p(2268) = p(22)*p(588) - p(2267)
p(2269) = p(1)*p(1487) - p(2160)
p(2270) = p(50)*p(346) - p(2269)
p(2271) = p(1)*p(1489) - p(2181)
p(2272) = p(102)*p(194) - p(2271)
p(2273) = p(1)*p(1492) - p(2206)
p(2274) = p(101)*p(195) - p(2273)
p(2275) = p(2)*p(1491) - p(2205)
p(2276) = p(1)*p(1495) - p(2226)
p(2277) = p(49)*p(347) - p(2276)
p(2278) = p(22)*p(578) - p(2180)
p(2279) = p(1)*p(1498) - p(2242)
p(2280) = p(21)*p(589) - p(2279)
p(2281) = p(102)*p(187) - p(2198)
p(2282) = p(1)*p(1501) - p(2252)
p(2283) = p(7)*p(959) - p(2282)
p(2284) = p(45)*p(347) - p(2237)
p(2285) = p(2)*p(1501) - p(2254)
p(2286) = p(1)*p(1505) - p(2285)
p(2287) = p(6)*p(959) - p(2257)
p(2288) = p(1)*p(1504) - p(2262)
p(2289) = p(2)*p(1505) - p(2287)
p(2290) = p(9)*p(590)
p(2291) = p(9)*p(591)
p(2292) = p(9)*p(613)
p(2293) = p(9)*p(592)
p(2294) = p(9)*p(614)
p(2295) = p(9)*p(615)
p(2296) = p(9)*p(701)
p(2297) = p(9)*p(726)
p(2298) = p(9)*p(626)
p(2299) = p(9)*p(593)
p(2300) = p(9)*p(627)
p(2301) = p(9)*p(594)
p(2302) = p(9)*p(595)
p(2303) = p(9)*p(628)
p(2304) = p(9)*p(596)
p(2305) = p(9)*p(629)
p(2306) = p(9)*p(630)
p(2307) = p(9)*p(631)
p(2308) = p(9)*p(727)
p(2309) = p(9)*p(636)
p(2310) = p(9)*p(637)
p(2311) = p(9)*p(734)
p(2312) = p(9)*p(597)
p(2313) = p(9)*p(638)
p(2314) = p(9)*p(598)
p(2315) = p(9)*p(639)
p(2316) = p(9)*p(640)
p(2317) = p(9)*p(735)
p(2318) = p(9)*p(599)
p(2319) = p(9)*p(641)
p(2320) = p(9)*p(736)
p(2321) = p(9)*p(755)
p(2322) = p(9)*p(648)
p(2323) = p(9)*p(600)
p(2324) = p(9)*p(649)
p(2325) = p(9)*p(601)
p(2326) = p(9)*p(650)
p(2327) = p(9)*p(651)
p(2328) = p(9)*p(652)
p(2329) = p(9)*p(756)
p(2330) = p(9)*p(766)
p(2331) = p(9)*p(656)
p(2332) = p(9)*p(657)
p(2333) = p(9)*p(767)
p(2334) = p(9)*p(658)
p(2335) = p(9)*p(602)
p(2336) = p(9)*p(659)
p(2337) = p(9)*p(660)
p(2338) = p(9)*p(661)
p(2339) = p(9)*p(768)
p(2340) = p(9)*p(769)
p(2341) = p(9)*p(662)
p(2342) = p(9)*p(663)
p(2343) = p(9)*p(770)
p(2344) = p(9)*p(665)
p(2345) = p(9)*p(774)
p(2346) = p(9)*p(666)
p(2347) = p(9)*p(775)
p(2348) = p(9)*p(776)
p(2349) = p(9)*p(777)
p(2350) = p(9)*p(882)
p(2351) = p(9)*p(791)
p(2352) = p(9)*p(670)
p(2353) = p(9)*p(603)
p(2354) = p(9)*p(671)
p(2355) = p(9)*p(672)
p(2356) = p(9)*p(792)
p(2357) = p(9)*p(799)
p(2358) = p(9)*p(674)
p(2359) = p(9)*p(800)
p(2360) = p(9)*p(675)
p(2361) = p(9)*p(676)
p(2362) = p(9)*p(801)
p(2363) = p(9)*p(802)
p(2364) = p(9)*p(803)
p(2365) = p(9)*p(807)
p(2366) = p(9)*p(808)
p(2367) = p(9)*p(809)
p(2368) = p(9)*p(898)
p(2369) = p(9)*p(901)
p(2370) = p(9)*p(818)
p(2371) = p(9)*p(678)
p(2372) = p(9)*p(679)
p(2373) = p(9)*p(819)
p(2374) = p(9)*p(823)
p(2375) = p(9)*p(824)
p(2376) = p(9)*p(825)
p(2377) = p(9)*p(913)
p(2378) = p(9)*p(917)
p(2379) = p(9)*p(830)
p(2380) = p(9)*p(831)
p(2381) = p(9)*p(927)
p(2382) = p(9)*p(933)
p(2383) = p(9)*p(692)
p(2384) = p(9)*p(693)
p(2385) = p(9)*p(694)
p(2386) = p(9)*p(850)
p(2387) = p(9)*p(604)
p(2388) = p(9)*p(605)
p(2389) = p(9)*p(606)
p(2390) = p(9)*p(695)
p(2391) = p(9)*p(607)
p(2392) = p(9)*p(608)
p(2393) = p(9)*p(696)
p(2394) = p(9)*p(609)
p(2395) = p(9)*p(610)
p(2396) = p(9)*p(697)
p(2397) = p(9)*p(611)
p(2398) = p(9)*p(698)
p(2399) = p(9)*p(612)
p(2400) = p(9)*p(699)
p(2401) = p(9)*p(700)
p(2402) = p(9)*p(851)
p(2403) = p(14)*p(934)
p(2404) = p(14)*p(935)
p(2405) = p(14)*p(936)
p(2406) = p(14)*p(937)
p(2407) = p(9)*p(861)
p(2408) = p(9)*p(711)
p(2409) = p(9)*p(712)
p(2410) = p(9)*p(713)
p(2411) = p(9)*p(714)
p(2412) = p(9)*p(715)
p(2413) = p(9)*p(862)
p(2414) = p(9)*p(716)
p(2415) = p(9)*p(616)
p(2416) = p(9)*p(617)
p(2417) = p(9)*p(618)
p(2418) = p(9)*p(619)
p(2419) = p(9)*p(717)
p(2420) = p(9)*p(718)
p(2421) = p(9)*p(620)
p(2422) = p(9)*p(621)
p(2423) = p(9)*p(622)
p(2424) = p(9)*p(719)
p(2425) = p(9)*p(720)
p(2426) = p(9)*p(623)
p(2427) = p(9)*p(624)
p(2428) = p(9)*p(721)
p(2429) = p(9)*p(722)
p(2430) = p(9)*p(625)
p(2431) = p(9)*p(723)
p(2432) = p(9)*p(724)
p(2433) = p(9)*p(725)
p(2434) = p(9)*p(863)
p(2435) = p(14)*p(839)
p(2436) = p(14)*p(840)
p(2437) = p(14)*p(841)
p(2438) = p(14)*p(842)
p(2439) = p(14)*p(843)
p(2440) = p(14)*p(844)
p(2441) = p(14)*p(845)
p(2442) = p(14)*p(846)
p(2443) = p(14)*p(847)
p(2444) = p(14)*p(848)
p(2445) = p(14)*p(849)
p(2446) = p(9)*p(728)
p(2447) = p(9)*p(729)
p(2448) = p(9)*p(864)
p(2449) = p(9)*p(632)
p(2450) = p(9)*p(633)
p(2451) = p(9)*p(730)
p(2452) = p(9)*p(634)
p(2453) = p(9)*p(731)
p(2454) = p(9)*p(635)
p(2455) = p(9)*p(732)
p(2456) = p(9)*p(733)
p(2457) = p(9)*p(865)
p(2458) = p(14)*p(692)
p(2459) = p(14)*p(693)
p(2460) = p(14)*p(694)
p(2461) = p(14)*p(695)
p(2462) = p(14)*p(696)
p(2463) = p(14)*p(697)
p(2464) = p(14)*p(698)
p(2465) = p(14)*p(699)
p(2466) = p(14)*p(700)
p(2467) = p(14)*p(613)
p(2468) = p(14)*p(614)
p(2469) = p(14)*p(615)
p(2470) = p(9)*p(874)
p(2471) = p(9)*p(743)
p(2472) = p(9)*p(744)
p(2473) = p(9)*p(745)
p(2474) = p(9)*p(746)
p(2475) = p(9)*p(875)
p(2476) = p(9)*p(747)
p(2477) = p(9)*p(642)
p(2478) = p(9)*p(643)
p(2479) = p(9)*p(644)
p(2480) = p(9)*p(748)
p(2481) = p(9)*p(749)
p(2482) = p(9)*p(645)
p(2483) = p(9)*p(646)
p(2484) = p(9)*p(750)
p(2485) = p(9)*p(751)
p(2486) = p(9)*p(647)
p(2487) = p(9)*p(752)
p(2488) = p(9)*p(753)
p(2489) = p(9)*p(754)
p(2490) = p(9)*p(876)
p(2491) = p(14)*p(852)
p(2492) = p(14)*p(853)
p(2493) = p(14)*p(854)
p(2494) = p(14)*p(855)
p(2495) = p(14)*p(856)
p(2496) = p(14)*p(857)
p(2497) = p(14)*p(858)
p(2498) = p(14)*p(859)
p(2499) = p(14)*p(860)
p(2500) = p(9)*p(877)
p(2501) = p(9)*p(757)
p(2502) = p(9)*p(758)
p(2503) = p(9)*p(759)
p(2504) = p(9)*p(878)
p(2505) = p(9)*p(760)
p(2506) = p(9)*p(653)
p(2507) = p(9)*p(654)
p(2508) = p(9)*p(761)
p(2509) = p(9)*p(762)
p(2510) = p(9)*p(655)
p(2511) = p(9)*p(763)
p(2512) = p(9)*p(764)
p(2513) = p(9)*p(765)
p(2514) = p(9)*p(879)
p(2515) = p(14)*p(711)
p(2516) = p(14)*p(712)
p(2517) = p(14)*p(713)
p(2518) = p(14)*p(714)
p(2519) = p(14)*p(715)
p(2520) = p(14)*p(716)
p(2521) = p(14)*p(717)
p(2522) = p(14)*p(718)
p(2523) = p(14)*p(719)
p(2524) = p(14)*p(720)
p(2525) = p(14)*p(721)
p(2526) = p(14)*p(722)
p(2527) = p(14)*p(723)
p(2528) = p(14)*p(724)
p(2529) = p(14)*p(725)
p(2530) = p(14)*p(626)
p(2531) = p(14)*p(627)
p(2532) = p(14)*p(628)
p(2533) = p(14)*p(629)
p(2534) = p(14)*p(630)
p(2535) = p(14)*p(631)
p(2536) = p(9)*p(771)
p(2537) = p(9)*p(880)
p(2538) = p(9)*p(664)
p(2539) = p(9)*p(772)
p(2540) = p(9)*p(773)
p(2541) = p(9)*p(881)
p(2542) = p(14)*p(728)
p(2543) = p(14)*p(729)
p(2544) = p(14)*p(730)
p(2545) = p(14)*p(731)
p(2546) = p(14)*p(732)
p(2547) = p(14)*p(733)
p(2548) = p(14)*p(636)
p(2549) = p(14)*p(637)
p(2550) = p(14)*p(638)
p(2551) = p(14)*p(639)
p(2552) = p(14)*p(640)
p(2553) = p(14)*p(641)
p(2554) = p(9)*p(889)
p(2555) = p(9)*p(782)
p(2556) = p(9)*p(783)
p(2557) = p(9)*p(784)
p(2558) = p(9)*p(890)
p(2559) = p(9)*p(785)
p(2560) = p(9)*p(667)
p(2561) = p(9)*p(668)
p(2562) = p(9)*p(786)
p(2563) = p(9)*p(787)
p(2564) = p(9)*p(669)
p(2565) = p(9)*p(788)
p(2566) = p(9)*p(789)
p(2567) = p(9)*p(790)
p(2568) = p(9)*p(891)
p(2569) = p(14)*p(866)
p(2570) = p(14)*p(867)
p(2571) = p(14)*p(868)
p(2572) = p(14)*p(869)
p(2573) = p(14)*p(870)
p(2574) = p(14)*p(871)
p(2575) = p(14)*p(872)
p(2576) = p(14)*p(873)
p(2577) = p(9)*p(892)
p(2578) = p(9)*p(793)
p(2579) = p(9)*p(794)
p(2580) = p(9)*p(893)
p(2581) = p(9)*p(795)
p(2582) = p(9)*p(673)
p(2583) = p(9)*p(796)
p(2584) = p(9)*p(797)
p(2585) = p(9)*p(798)
p(2586) = p(9)*p(894)
p(2587) = p(14)*p(743)
p(2588) = p(14)*p(744)
p(2589) = p(14)*p(745)
p(2590) = p(14)*p(746)
p(2591) = p(14)*p(747)
p(2592) = p(14)*p(748)
p(2593) = p(14)*p(749)
p(2594) = p(14)*p(750)
p(2595) = p(14)*p(751)
p(2596) = p(14)*p(752)
p(2597) = p(14)*p(753)
p(2598) = p(14)*p(754)
p(2599) = p(14)*p(648)
p(2600) = p(14)*p(649)
p(2601) = p(14)*p(650)
p(2602) = p(14)*p(651)
p(2603) = p(14)*p(652)
p(2604) = p(9)*p(895)
p(2605) = p(9)*p(804)
p(2606) = p(9)*p(896)
p(2607) = p(9)*p(805)
p(2608) = p(9)*p(806)
p(2609) = p(9)*p(897)
p(2610) = p(14)*p(757)
p(2611) = p(14)*p(758)
p(2612) = p(14)*p(759)
p(2613) = p(14)*p(760)
p(2614) = p(14)*p(761)
p(2615) = p(14)*p(762)
p(2616) = p(14)*p(763)
p(2617) = p(14)*p(764)
p(2618) = p(14)*p(765)
p(2619) = p(14)*p(656)
p(2620) = p(14)*p(657)
p(2621) = p(14)*p(658)
p(2622) = p(14)*p(659)
p(2623) = p(14)*p(660)
p(2624) = p(14)*p(661)
p(2625) = p(14)*p(662)
p(2626) = p(14)*p(663)
p(2627) = p(9)*p(899)
p(2628) = p(9)*p(900)
p(2629) = p(14)*p(771)
p(2630) = p(14)*p(772)
p(2631) = p(14)*p(773)
p(2632) = p(14)*p(665)
p(2633) = p(14)*p(666)
p(2634) = p(9)*p(907)
p(2635) = p(9)*p(812)
p(2636) = p(9)*p(813)
p(2637) = p(9)*p(908)
p(2638) = p(9)*p(814)
p(2639) = p(9)*p(677)
p(2640) = p(9)*p(815)
p(2641) = p(9)*p(816)
p(2642) = p(9)*p(817)
p(2643) = p(9)*p(909)
p(2644) = p(14)*p(883)
p(2645) = p(14)*p(884)
p(2646) = p(14)*p(885)
p(2647) = p(14)*p(886)
p(2648) = p(14)*p(887)
p(2649) = p(14)*p(888)
p(2650) = p(9)*p(910)
p(2651) = p(9)*p(820)
p(2652) = p(9)*p(911)
p(2653) = p(9)*p(821)
p(2654) = p(9)*p(822)
p(2655) = p(9)*p(912)
p(2656) = p(14)*p(782)
p(2657) = p(14)*p(783)
p(2658) = p(14)*p(784)
p(2659) = p(14)*p(785)
p(2660) = p(14)*p(786)
p(2661) = p(14)*p(787)
p(2662) = p(14)*p(788)
p(2663) = p(14)*p(789)
p(2664) = p(14)*p(790)
p(2665) = p(14)*p(670)
p(2666) = p(14)*p(671)
p(2667) = p(14)*p(672)
p(2668) = p(9)*p(914)
p(2669) = p(9)*p(915)
p(2670) = p(9)*p(916)
p(2671) = p(14)*p(793)
p(2672) = p(14)*p(794)
p(2673) = p(14)*p(795)
p(2674) = p(14)*p(796)
p(2675) = p(14)*p(797)
p(2676) = p(14)*p(798)
p(2677) = p(14)*p(674)
p(2678) = p(14)*p(675)
p(2679) = p(14)*p(676)
p(2680) = p(9)*p(948)
p(2681) = p(14)*p(804)
p(2682) = p(14)*p(805)
p(2683) = p(14)*p(806)
p(2684) = p(9)*p(921)
p(2685) = p(9)*p(827)
p(2686) = p(9)*p(922)
p(2687) = p(9)*p(828)
p(2688) = p(9)*p(829)
p(2689) = p(9)*p(923)
p(2690) = p(14)*p(902)
p(2691) = p(14)*p(903)
p(2692) = p(14)*p(904)
p(2693) = p(14)*p(905)
p(2694) = p(14)*p(906)
p(2695) = p(9)*p(924)
p(2696) = p(9)*p(925)
p(2697) = p(9)*p(926)
p(2698) = p(14)*p(812)
p(2699) = p(14)*p(813)
p(2700) = p(14)*p(814)
p(2701) = p(14)*p(815)
p(2702) = p(14)*p(816)
p(2703) = p(14)*p(817)
p(2704) = p(14)*p(678)
p(2705) = p(14)*p(679)
p(2706) = p(9)*p(951)
p(2707) = p(14)*p(820)
p(2708) = p(14)*p(821)
p(2709) = p(14)*p(822)
p(2710) = p(9)*p(930)
p(2711) = p(9)*p(931)
p(2712) = p(9)*p(932)
p(2713) = p(14)*p(918)
p(2714) = p(14)*p(919)
p(2715) = p(14)*p(920)
p(2716) = p(9)*p(954)
p(2717) = p(14)*p(827)
p(2718) = p(14)*p(828)
p(2719) = p(14)*p(829)
p(2720) = p(9)*p(957)
p(2721) = p(14)*p(928)
p(2722) = p(14)*p(929)
p(2723) = p(9)*p(938)
p(2724) = p(9)*p(839)
p(2725) = p(9)*p(680)
p(2726) = p(9)*p(840)
p(2727) = p(9)*p(681)
p(2728) = p(9)*p(682)
p(2729) = p(9)*p(841)
p(2730) = p(9)*p(683)
p(2731) = p(9)*p(684)
p(2732) = p(9)*p(685)
p(2733) = p(9)*p(842)
p(2734) = p(9)*p(686)
p(2735) = p(9)*p(687)
p(2736) = p(9)*p(688)
p(2737) = p(9)*p(843)
p(2738) = p(9)*p(844)
p(2739) = p(9)*p(689)
p(2740) = p(9)*p(690)
p(2741) = p(9)*p(845)
p(2742) = p(9)*p(846)
p(2743) = p(9)*p(691)
p(2744) = p(9)*p(847)
p(2745) = p(9)*p(848)
p(2746) = p(9)*p(849)
p(2747) = p(9)*p(939)
p(2748) = p(3)*p(1136) - p(2383)
p(2749) = p(7)*p(1136) - p(2387)
p(2750) = p(49)*p(401) - p(2394)
p(2751) = p(114)*p(194) - p(2399)
p(2752) = p(24)*p(588) - p(2402)
p(2753) = p(3)*p(1368) - p(2386) - p(2752)
p(2754) = p(1)*p(1802) - p(2386) - p(2753)
p(2755) = p(1)*p(1803) - p(2390) - p(2754)
p(2756) = p(1)*p(1804) - p(2393) - p(2755)
p(2757) = p(6)*p(1476) - p(2748)
p(2758) = p(32)*p(566) - p(2387)
p(2759) = p(150)*p(178) - p(2394)
p(2760) = p(39)*p(505) - p(2399)
p(2761) = p(3)*p(1369) - p(2402)
p(2762) = p(14)*p(958)
p(2763) = p(9)*p(940)
p(2764) = p(9)*p(852)
p(2765) = p(9)*p(702)
p(2766) = p(9)*p(853)
p(2767) = p(9)*p(703)
p(2768) = p(9)*p(704)
p(2769) = p(9)*p(854)
p(2770) = p(9)*p(705)
p(2771) = p(9)*p(706)
p(2772) = p(9)*p(707)
p(2773) = p(9)*p(855)
p(2774) = p(9)*p(708)
p(2775) = p(9)*p(709)
p(2776) = p(9)*p(856)
p(2777) = p(9)*p(857)
p(2778) = p(9)*p(710)
p(2779) = p(9)*p(858)
p(2780) = p(9)*p(859)
p(2781) = p(9)*p(860)
p(2782) = p(9)*p(941)
p(2783) = p(24)*p(570) - p(2296)
p(2784) = p(114)*p(181) - p(2293)
p(2785) = p(1)*p(1828) - p(2408) - p(2784)
p(2786) = p(4)*p(1136) - p(2292)
p(2787) = p(1)*p(1830) - p(2410) - p(2786)
p(2788) = p(1)*p(1831) - p(2411) - p(2787)
p(2789) = p(1)*p(1832) - p(2412) - p(2788)
p(2790) = p(1)*p(1833) - p(2413) - p(2789)
p(2791) = p(13)*p(938) - p(2762)
p(2792) = p(2)*p(1802) - p(2406) - p(2791)
p(2793) = p(32)*p(570) - p(2762)
p(2794) = p(1)*p(1835) - p(2413) - p(2792)
p(2795) = p(73)*p(331) - p(2762)
p(2796) = p(1)*p(1837) - p(2419) - p(2794)
p(2797) = p(150)*p(181) - p(2762)
p(2798) = p(1)*p(1839) - p(2424) - p(2796)
p(2799) = p(91)*p(285) - p(2762)
p(2800) = p(13)*p(843) - p(2292)
p(2801) = p(41)*p(505) - p(2762)
p(2802) = p(1)*p(1843) - p(2431) - p(2800)
p(2803) = p(16)*p(851) - p(2762)
p(2804) = p(69)*p(285) - p(2293)
p(2805) = p(4)*p(1369) - p(2762)
p(2806) = p(11)*p(851) - p(2296)
p(2807) = p(14)*p(938)
p(2808) = p(14)*p(939)
p(2809) = p(3)*p(1180) - p(2446)
p(2810) = p(7)*p(1180) - p(2449)
p(2811) = p(49)*p(430) - p(2454)
p(2812) = p(130)*p(194) - p(2457)
p(2813) = p(3)*p(1384) - p(2448) - p(2812)
p(2814) = p(1)*p(1853) - p(2448) - p(2813)
p(2815) = p(1)*p(1854) - p(2451) - p(2814)
p(2816) = p(45)*p(566) - p(2809)
p(2817) = p(160)*p(178) - p(2449)
p(2818) = p(39)*p(518) - p(2454)
p(2819) = p(3)*p(1385) - p(2457)
p(2820) = p(14)*p(850)
p(2821) = p(14)*p(851)
p(2822) = p(14)*p(701)
p(2823) = p(9)*p(942)
p(2824) = p(9)*p(866)
p(2825) = p(9)*p(737)
p(2826) = p(9)*p(867)
p(2827) = p(9)*p(738)
p(2828) = p(9)*p(739)
p(2829) = p(9)*p(868)
p(2830) = p(9)*p(740)
p(2831) = p(9)*p(741)
p(2832) = p(9)*p(869)
p(2833) = p(9)*p(870)
p(2834) = p(9)*p(742)
p(2835) = p(9)*p(871)
p(2836) = p(9)*p(872)
p(2837) = p(9)*p(873)
p(2838) = p(9)*p(943)
p(2839) = p(24)*p(572) - p(2297)
p(2840) = p(114)*p(183) - p(2299)
p(2841) = p(43)*p(401) - p(2302)
p(2842) = p(8)*p(1136) - p(2439)
p(2843) = p(1)*p(1877) - p(2473) - p(2842)
p(2844) = p(1)*p(1878) - p(2474) - p(2843)
p(2845) = p(1)*p(1879) - p(2475) - p(2844)
p(2846) = p(13)*p(940) - p(2807)
p(2847) = p(2)*p(1835) - p(2445) - p(2813)
p(2848) = p(32)*p(572) - p(2807)
p(2849) = p(1)*p(1881) - p(2475) - p(2847)
p(2850) = p(73)*p(333) - p(2807)
p(2851) = p(1)*p(1883) - p(2480) - p(2849)
p(2852) = p(150)*p(183) - p(2807)
p(2853) = p(6)*p(1374) - p(2439)
p(2854) = p(93)*p(285) - p(2807)
p(2855) = p(32)*p(509) - p(2302)
p(2856) = p(43)*p(505) - p(2807)
p(2857) = p(150)*p(153) - p(2299)
p(2858) = p(18)*p(851) - p(2807)
p(2859) = p(34)*p(505) - p(2297)
p(2860) = p(14)*p(940)
p(2861) = p(14)*p(941)
p(2862) = p(130)*p(181) - p(2317)
p(2863) = p(41)*p(430) - p(2314)
p(2864) = p(4)*p(1180) - p(2309)
p(2865) = p(1)*p(1896) - p(2502) - p(2864)
p(2866) = p(1)*p(1897) - p(2503) - p(2865)
p(2867) = p(1)*p(1898) - p(2504) - p(2866)
p(2868) = p(81)*p(331) - p(2821)
p(2869) = p(2)*p(1853) - p(2461) - p(2868)
p(2870) = p(160)*p(181) - p(2821)
p(2871) = p(1)*p(1900) - p(2504) - p(2869)
p(2872) = p(91)*p(296) - p(2821)
p(2873) = p(1)*p(1902) - p(2508) - p(2871)
p(2874) = p(41)*p(518) - p(2821)
p(2875) = p(81)*p(279) - p(2309)
p(2876) = p(16)*p(865) - p(2821)
p(2877) = p(69)*p(296) - p(2314)
p(2878) = p(4)*p(1385) - p(2821)
p(2879) = p(11)*p(865) - p(2317)
p(2880) = p(14)*p(861)
p(2881) = p(14)*p(862)
p(2882) = p(14)*p(863)
p(2883) = p(14)*p(726)
p(2884) = p(14)*p(727)
p(2885) = p(3)*p(1234) - p(2536)
p(2886) = p(7)*p(1234) - p(2538)
p(2887) = p(49)*p(464) - p(2541)
p(2888) = p(3)*p(1401) - p(2537) - p(2887)
p(2889) = p(1)*p(1916) - p(2537) - p(2888)
p(2890) = p(178)*p(187) - p(2885)
p(2891) = p(39)*p(532) - p(2538)
p(2892) = p(3)*p(1402) - p(2541)
p(2893) = p(14)*p(864)
p(2894) = p(14)*p(865)
p(2895) = p(14)*p(734)
p(2896) = p(14)*p(735)
p(2897) = p(14)*p(736)
p(2898) = p(9)*p(944)
p(2899) = p(9)*p(883)
p(2900) = p(9)*p(778)
p(2901) = p(9)*p(884)
p(2902) = p(9)*p(779)
p(2903) = p(9)*p(780)
p(2904) = p(9)*p(885)
p(2905) = p(9)*p(781)
p(2906) = p(9)*p(886)
p(2907) = p(9)*p(887)
p(2908) = p(9)*p(888)
p(2909) = p(9)*p(945)
p(2910) = p(24)*p(574) - p(2321)
p(2911) = p(114)*p(185) - p(2323)
p(2912) = p(46)*p(401) - p(2326)
p(2913) = p(1)*p(1936) - p(2556) - p(2912)
p(2914) = p(1)*p(1937) - p(2557) - p(2913)
p(2915) = p(1)*p(1938) - p(2558) - p(2914)
p(2916) = p(13)*p(942) - p(2860)
p(2917) = p(2)*p(1881) - p(2499) - p(2869)
p(2918) = p(32)*p(574) - p(2860)
p(2919) = p(1)*p(1940) - p(2558) - p(2917)
p(2920) = p(73)*p(335) - p(2860)
p(2921) = p(1)*p(1942) - p(2562) - p(2919)
p(2922) = p(150)*p(185) - p(2860)
p(2923) = p(13)*p(869) - p(2326)
p(2924) = p(95)*p(285) - p(2860)
p(2925) = p(73)*p(299) - p(2323)
p(2926) = p(46)*p(505) - p(2860)
p(2927) = p(84)*p(285) - p(2321)
p(2928) = p(14)*p(942)
p(2929) = p(14)*p(943)
p(2930) = p(130)*p(183) - p(2320)
p(2931) = p(1)*p(1951) - p(2577) - p(2930)
p(2932) = p(8)*p(1180) - p(2467)
p(2933) = p(1)*p(1953) - p(2579) - p(2932)
p(2934) = p(1)*p(1954) - p(2580) - p(2933)
p(2935) = p(81)*p(333) - p(2822)
p(2936) = p(2)*p(1900) - p(2521) - p(2888)
p(2937) = p(160)*p(183) - p(2822)
p(2938) = p(1)*p(1956) - p(2580) - p(2936)
p(2939) = p(93)*p(296) - p(2822)
p(2940) = p(45)*p(509) - p(2467)
p(2941) = p(43)*p(518) - p(2822)
p(2942) = p(1)*p(1960) - p(2585) - p(2940)
p(2943) = p(18)*p(865) - p(2822)
p(2944) = p(34)*p(518) - p(2320)
p(2945) = p(14)*p(874)
p(2946) = p(14)*p(875)
p(2947) = p(14)*p(876)
p(2948) = p(14)*p(755)
p(2949) = p(14)*p(756)
p(2950) = p(41)*p(464) - p(2347)
p(2951) = p(4)*p(1234) - p(2344)
p(2952) = p(1)*p(1969) - p(2605) - p(2951)
p(2953) = p(1)*p(1970) - p(2606) - p(2952)
p(2954) = p(91)*p(310) - p(2894)
p(2955) = p(2)*p(1916) - p(2544) - p(2954)
p(2956) = p(41)*p(532) - p(2894)
p(2957) = p(1)*p(1972) - p(2606) - p(2955)
p(2958) = p(16)*p(881) - p(2894)
p(2959) = p(69)*p(310) - p(2344)
p(2960) = p(4)*p(1402) - p(2894)
p(2961) = p(11)*p(881) - p(2347)
p(2962) = p(14)*p(877)
p(2963) = p(14)*p(878)
p(2964) = p(14)*p(879)
p(2965) = p(14)*p(766)
p(2966) = p(14)*p(767)
p(2967) = p(14)*p(768)
p(2968) = p(14)*p(769)
p(2969) = p(14)*p(770)
p(2970) = p(3)*p(1290) - p(2627)
p(2971) = p(7)*p(1290) - p(2628)
p(2972) = p(3)*p(1421) - p(2627) - p(2971)
p(2973) = p(39)*p(578) - p(2970)
p(2974) = p(3)*p(1422) - p(2628)
p(2975) = p(14)*p(880)
p(2976) = p(14)*p(881)
p(2977) = p(14)*p(774)
p(2978) = p(14)*p(775)
p(2979) = p(14)*p(776)
p(2980) = p(14)*p(777)
p(2981) = p(9)*p(946)
p(2982) = p(9)*p(902)
p(2983) = p(9)*p(810)
p(2984) = p(9)*p(903)
p(2985) = p(9)*p(811)
p(2986) = p(9)*p(904)
p(2987) = p(9)*p(905)
p(2988) = p(9)*p(906)
p(2989) = p(9)*p(947)
p(2990) = p(24)*p(576) - p(2351)
p(2991) = p(114)*p(188) - p(2353)
p(2992) = p(50)*p(401) - p(2572)
p(2993) = p(1)*p(2000) - p(2636) - p(2992)
p(2994) = p(1)*p(2001) - p(2637) - p(2993)
p(2995) = p(13)*p(944) - p(2928)
p(2996) = p(2)*p(1940) - p(2576) - p(2936)
p(2997) = p(32)*p(576) - p(2928)
p(2998) = p(1)*p(2003) - p(2637) - p(2996)
p(2999) = p(73)*p(337) - p(2928)
p(3000) = p(6)*p(1406) - p(2572)
p(3001) = p(150)*p(188) - p(2928)
p(3002) = p(32)*p(536) - p(2353)
p(3003) = p(98)*p(285) - p(2928)
p(3004) = p(150)*p(173) - p(2351)
p(3005) = p(14)*p(944)
p(3006) = p(14)*p(945)
p(3007) = p(130)*p(185) - p(2340)
p(3008) = p(46)*p(430) - p(2342)
p(3009) = p(1)*p(2013) - p(2651) - p(3008)
p(3010) = p(1)*p(2014) - p(2652) - p(3009)
p(3011) = p(81)*p(335) - p(2883)
p(3012) = p(2)*p(1956) - p(2592) - p(2955)
p(3013) = p(160)*p(185) - p(2883)
p(3014) = p(1)*p(2016) - p(2652) - p(3012)
p(3015) = p(95)*p(296) - p(2883)
p(3016) = p(81)*p(299) - p(2342)
p(3017) = p(46)*p(518) - p(2883)
p(3018) = p(84)*p(296) - p(2340)
p(3019) = p(14)*p(889)
p(3020) = p(14)*p(890)
p(3021) = p(14)*p(891)
p(3022) = p(14)*p(791)
p(3023) = p(14)*p(792)
p(3024) = p(43)*p(464) - p(2349)
p(3025) = p(8)*p(1234) - p(2548)
p(3026) = p(1)*p(2027) - p(2669) - p(3025)
p(3027) = p(93)*p(310) - p(2896)
p(3028) = p(2)*p(1972) - p(2614) - p(2972)
p(3029) = p(43)*p(532) - p(2896)
p(3030) = p(153)*p(187) - p(2548)
p(3031) = p(18)*p(881) - p(2896)
p(3032) = p(34)*p(532) - p(2349)
p(3033) = p(14)*p(892)
p(3034) = p(14)*p(893)
p(3035) = p(14)*p(894)
p(3036) = p(14)*p(799)
p(3037) = p(14)*p(800)
p(3038) = p(14)*p(801)
p(3039) = p(14)*p(802)
p(3040) = p(14)*p(803)
p(3041) = p(4)*p(1290) - p(2369)
p(3042) = p(1)*p(2038) - p(2680) - p(3041)
p(3043) = p(16)*p(900) - p(2976)
p(3044) = p(2)*p(1986) - p(2630) - p(3043)
p(3045) = p(4)*p(1422) - p(2976)
p(3046) = p(11)*p(900) - p(2369)
p(3047) = p(14)*p(895)
p(3048) = p(14)*p(896)
p(3049) = p(14)*p(897)
p(3050) = p(14)*p(807)
p(3051) = p(14)*p(808)
p(3052) = p(14)*p(809)
p(3053) = p(6)*p(1290) - p(2681)
p(3054) = p(3)*p(1491) - p(3053)
p(3055) = p(14)*p(899)
p(3056) = p(14)*p(900)
p(3057) = p(9)*p(949)
p(3058) = p(9)*p(918)
p(3059) = p(9)*p(826)
p(3060) = p(9)*p(919)
p(3061) = p(9)*p(920)
p(3062) = p(9)*p(950)
p(3063) = p(24)*p(579) - p(2370)
p(3064) = p(114)*p(191) - p(2372)
p(3065) = p(1)*p(2049) - p(2685) - p(3064)
p(3066) = p(1)*p(2050) - p(2686) - p(3065)
p(3067) = p(13)*p(946) - p(3005)
p(3068) = p(2)*p(2003) - p(2649) - p(3012)
p(3069) = p(32)*p(579) - p(3005)
p(3070) = p(1)*p(2052) - p(2686) - p(3068)
p(3071) = p(73)*p(340) - p(3005)
p(3072) = p(13)*p(904) - p(2372)
p(3073) = p(150)*p(191) - p(3005)
p(3074) = p(73)*p(323) - p(2370)
p(3075) = p(14)*p(946)
p(3076) = p(14)*p(947)
p(3077) = p(130)*p(188) - p(2363)
p(3078) = p(50)*p(430) - p(2601)
p(3079) = p(1)*p(2060) - p(2696) - p(3078)
p(3080) = p(81)*p(337) - p(2948)
p(3081) = p(2)*p(2016) - p(2660) - p(3028)
p(3082) = p(160)*p(188) - p(2948)
p(3083) = p(45)*p(536) - p(2601)
p(3084) = p(98)*p(296) - p(2948)
p(3085) = p(160)*p(173) - p(2363)
p(3086) = p(14)*p(907)
p(3087) = p(14)*p(908)
p(3088) = p(14)*p(909)
p(3089) = p(14)*p(818)
p(3090) = p(14)*p(819)
p(3091) = p(2)*p(2026) - p(2671) - p(3041)
p(3092) = p(1)*p(2070) - p(2706) - p(3091)
p(3093) = p(95)*p(310) - p(2897)
p(3094) = p(2)*p(2029) - p(2674) - p(3044)
p(3095) = p(46)*p(532) - p(2897)
p(3096) = p(1)*p(2072) - p(2706) - p(3094)
p(3097) = p(14)*p(910)
p(3098) = p(14)*p(911)
p(3099) = p(14)*p(912)
p(3100) = p(14)*p(823)
p(3101) = p(14)*p(824)
p(3102) = p(14)*p(825)
p(3103) = p(8)*p(1290) - p(2632)
p(3104) = p(18)*p(900) - p(2978)
p(3105) = p(34)*p(578) - p(2632)
p(3106) = p(14)*p(914)
p(3107) = p(14)*p(915)
p(3108) = p(14)*p(916)
p(3109) = p(9)*p(952)
p(3110) = p(9)*p(928)
p(3111) = p(9)*p(929)
p(3112) = p(9)*p(953)
p(3113) = p(24)*p(582) - p(2379)
p(3114) = p(114)*p(195) - p(2692)
p(3115) = p(1)*p(2079) - p(2711) - p(3114)
p(3116) = p(13)*p(949) - p(3075)
p(3117) = p(2)*p(2052) - p(2694) - p(3081)
p(3118) = p(32)*p(582) - p(3075)
p(3119) = p(6)*p(1446) - p(2692)
p(3120) = p(73)*p(343) - p(3075)
p(3121) = p(32)*p(561) - p(2379)
p(3122) = p(14)*p(949)
p(3123) = p(14)*p(950)
p(3124) = p(130)*p(191) - p(2377)
p(3125) = p(1)*p(2086) - p(2716) - p(3124)
p(3126) = p(81)*p(340) - p(3022)
p(3127) = p(2)*p(2062) - p(2701) - p(3094)
p(3128) = p(160)*p(191) - p(3022)
p(3129) = p(81)*p(323) - p(2377)
p(3130) = p(14)*p(921)
p(3131) = p(14)*p(922)
p(3132) = p(14)*p(923)
p(3133) = p(14)*p(830)
p(3134) = p(14)*p(831)
p(3135) = p(50)*p(464) - p(2626)
p(3136) = p(98)*p(310) - p(2968)
p(3137) = p(173)*p(187) - p(2626)
p(3138) = p(14)*p(924)
p(3139) = p(14)*p(925)
p(3140) = p(14)*p(926)
p(3141) = p(9)*p(955)
p(3142) = p(9)*p(956)
p(3143) = p(24)*p(585) - p(2382)
p(3144) = p(1)*p(2093) - p(2720) - p(3143)
p(3145) = p(13)*p(952) - p(3122)
p(3146) = p(2)*p(2081) - p(2715) - p(3127)
p(3147) = p(32)*p(585) - p(3122)
p(3148) = p(13)*p(929) - p(2382)
p(3149) = p(14)*p(952)
p(3150) = p(14)*p(953)
p(3151) = p(130)*p(195) - p(2705)
p(3152) = p(81)*p(343) - p(3089)
p(3153) = p(45)*p(561) - p(2705)
p(3154) = p(14)*p(930)
p(3155) = p(14)*p(931)
p(3156) = p(14)*p(932)
p(3157) = p(9)*p(959)
p(3158) = p(24)*p(589) - p(2722)
p(3159) = p(13)*p(955) - p(3149)
p(3160) = p(6)*p(1471) - p(2722)
p(3161) = p(14)*p(955)
p(3162) = p(14)*p(956)
p(3163) = p(9)*p(832)
p(3164) = p(9)*p(833)
p(3165) = p(9)*p(834)
p(3166) = p(9)*p(835)
p(3167) = p(9)*p(836)
p(3168) = p(9)*p(934)
p(3169) = p(9)*p(837)
p(3170) = p(9)*p(935)
p(3171) = p(9)*p(838)
p(3172) = p(9)*p(936)
p(3173) = p(9)*p(937)
p(3174) = p(9)*p(958)
p(3175) = p(3)*p(1481) - p(2723)
p(3176) = p(39)*p(570) - p(2725)
p(3177) = p(178)*p(181) - p(2728)
p(3178) = p(41)*p(566) - p(2732)
p(3179) = p(4)*p(1476) - p(2737)
p(3180) = p(1)*p(2112) - p(2733) - p(3179)
p(3181) = p(1)*p(2113) - p(2737)
p(3182) = p(1)*p(2114) - p(2738) - p(3180)
p(3183) = p(21)*p(843) - p(2732)
p(3184) = p(1)*p(2116) - p(2742) - p(3182)
p(3185) = p(101)*p(279) - p(2728)
p(3186) = p(1)*p(2118) - p(2745) - p(3184)
p(3187) = p(69)*p(346) - p(2725)
p(3188) = p(1)*p(2120) - p(2747) - p(3186)
p(3189) = p(11)*p(958) - p(2723)
p(3190) = p(1)*p(2122) - p(2753) - p(2752)
p(3191) = p(1)*p(2123) - p(2761)
p(3192) = p(3)*p(1483) - p(2763)
p(3193) = p(39)*p(572) - p(2765)
p(3194) = p(178)*p(183) - p(2768)
p(3195) = p(43)*p(566) - p(2772)
p(3196) = p(1)*p(2127) - p(2769) - p(3195)
p(3197) = p(2)*p(2113) - p(2757)
p(3198) = p(1)*p(2128) - p(2773) - p(3196)
p(3199) = p(7)*p(1374) - p(2772)
p(3200) = p(1)*p(2130) - p(2777) - p(3198)
p(3201) = p(49)*p(509) - p(2768)
p(3202) = p(1)*p(2132) - p(2780) - p(3200)
p(3203) = p(153)*p(194) - p(2765)
p(3204) = p(1)*p(2134) - p(2782) - p(3202)
p(3205) = p(34)*p(588) - p(2763)
p(3206) = p(6)*p(1481) - p(2762)
p(3207) = p(1)*p(2137) - p(2792) - p(2790)
p(3208) = p(2)*p(2123) - p(2762)
p(3209) = p(1)*p(2139) - p(2813) - p(2812)
p(3210) = p(1)*p(2140) - p(2819)
p(3211) = p(3)*p(1485) - p(2823)
p(3212) = p(39)*p(574) - p(2825)
p(3213) = p(178)*p(185) - p(2828)
p(3214) = p(46)*p(566) - p(2832)
p(3215) = p(1)*p(2144) - p(2829) - p(3214)
p(3216) = p(1)*p(2145) - p(2832)
p(3217) = p(1)*p(2146) - p(2833) - p(3215)
p(3218) = p(21)*p(869) - p(2828)
p(3219) = p(1)*p(2148) - p(2836) - p(3217)
p(3220) = p(101)*p(299) - p(2825)
p(3221) = p(1)*p(2150) - p(2838) - p(3219)
p(3222) = p(84)*p(346) - p(2823)
p(3223) = p(6)*p(1483) - p(2807)
p(3224) = p(1)*p(2153) - p(2847) - p(2845)
p(3225) = p(8)*p(1369) - p(2807)
p(3226) = p(45)*p(570) - p(2821)
p(3227) = p(1)*p(2156) - p(2869) - p(2867)
p(3228) = p(2)*p(2140) - p(2821)
p(3229) = p(1)*p(2158) - p(2888) - p(2887)
p(3230) = p(1)*p(2159) - p(2892)
p(3231) = p(3)*p(1487) - p(2898)
p(3232) = p(39)*p(576) - p(2900)
p(3233) = p(178)*p(188) - p(2903)
p(3234) = p(1)*p(2162) - p(2901) - p(3233)
p(3235) = p(2)*p(2145) - p(2853)
p(3236) = p(1)*p(2163) - p(2904) - p(3234)
p(3237) = p(7)*p(1406) - p(2903)
p(3238) = p(1)*p(2165) - p(2907) - p(3236)
p(3239) = p(49)*p(536) - p(2900)
p(3240) = p(1)*p(2167) - p(2909) - p(3238)
p(3241) = p(173)*p(194) - p(2898)
p(3242) = p(6)*p(1485) - p(2860)
p(3243) = p(1)*p(2170) - p(2917) - p(2915)
p(3244) = p(22)*p(851) - p(2860)
p(3245) = p(45)*p(572) - p(2822)
p(3246) = p(1)*p(2173) - p(2936) - p(2934)
p(3247) = p(8)*p(1385) - p(2822)
p(3248) = p(181)*p(187) - p(2894)
p(3249) = p(1)*p(2176) - p(2955) - p(2953)
p(3250) = p(2)*p(2159) - p(2894)
p(3251) = p(1)*p(2178) - p(2972) - p(2971)
p(3252) = p(1)*p(2179) - p(2974)
p(3253) = p(14)*p(882)
p(3254) = p(3)*p(1489) - p(2981)
p(3255) = p(39)*p(579) - p(2983)
p(3256) = p(178)*p(191) - p(2986)
p(3257) = p(1)*p(2183) - p(2984) - p(3256)
p(3258) = p(1)*p(2184) - p(2986)
p(3259) = p(1)*p(2185) - p(2987) - p(3257)
p(3260) = p(21)*p(904) - p(2983)
p(3261) = p(1)*p(2187) - p(2989) - p(3259)
p(3262) = p(101)*p(323) - p(2981)
p(3263) = p(6)*p(1487) - p(2928)
p(3264) = p(1)*p(2190) - p(2996) - p(2994)
p(3265) = p(50)*p(505) - p(2928)
p(3266) = p(45)*p(574) - p(2883)
p(3267) = p(1)*p(2193) - p(3012) - p(3010)
p(3268) = p(22)*p(865) - p(2883)
p(3269) = p(183)*p(187) - p(2896)
p(3270) = p(1)*p(2196) - p(3028) - p(3026)
p(3271) = p(8)*p(1402) - p(2896)
p(3272) = p(41)*p(578) - p(2976)
p(3273) = p(1)*p(2200) - p(3044) - p(3042)
p(3274) = p(2)*p(2179) - p(2976)
p(3275) = p(14)*p(898)
p(3276) = p(4)*p(1441) - p(2683) - p(3103)
p(3277) = p(1)*p(2204) - p(3054)
p(3278) = p(14)*p(901)
p(3279) = p(3)*p(1492) - p(3057)
p(3280) = p(39)*p(582) - p(3059)
p(3281) = p(1)*p(2207) - p(3058) - p(3280)
p(3282) = p(2)*p(2184) - p(3000)
p(3283) = p(1)*p(2208) - p(3060) - p(3281)
p(3284) = p(7)*p(1446) - p(3059)
p(3285) = p(1)*p(2210) - p(3062) - p(3283)
p(3286) = p(49)*p(561) - p(3057)
p(3287) = p(6)*p(1489) - p(3005)
p(3288) = p(1)*p(2213) - p(3068) - p(3066)
p(3289) = p(102)*p(285) - p(3005)
p(3290) = p(45)*p(576) - p(2948)
p(3291) = p(1)*p(2216) - p(3081) - p(3079)
p(3292) = p(50)*p(518) - p(2948)
p(3293) = p(185)*p(187) - p(2897)
p(3294) = p(1)*p(2219) - p(3094) - p(3092)
p(3295) = p(22)*p(881) - p(2897)
p(3296) = p(14)*p(913)
p(3297) = p(43)*p(578) - p(2978)
p(3298) = p(1)*p(2223) - p(3105) - p(3103)
p(3299) = p(8)*p(1422) - p(2978)
p(3300) = p(14)*p(917)
p(3301) = p(4)*p(1491) - p(3056)
p(3302) = p(2)*p(2203) - p(3055) - p(3301)
p(3303) = p(2)*p(2204) - p(3056)
p(3304) = p(14)*p(948)
p(3305) = p(3)*p(1495) - p(3109)
p(3306) = p(39)*p(585) - p(3111)
p(3307) = p(1)*p(2227) - p(3110) - p(3306)
p(3308) = p(1)*p(2228) - p(3111)
p(3309) = p(1)*p(2229) - p(3112) - p(3307)
p(3310) = p(21)*p(929) - p(3109)
p(3311) = p(6)*p(1492) - p(3075)
p(3312) = p(1)*p(2232) - p(3117) - p(3115)
p(3313) = p(150)*p(195) - p(3075)
p(3314) = p(45)*p(579) - p(3022)
p(3315) = p(1)*p(2235) - p(3127) - p(3125)
p(3316) = p(102)*p(296) - p(3022)
p(3317) = p(187)*p(188) - p(2968)
p(3318) = p(1)*p(2239) - p(3137) - p(3135)
p(3319) = p(50)*p(532) - p(2968)
p(3320) = p(14)*p(927)
p(3321) = p(46)*p(578) - p(2980)
p(3322) = p(2)*p(2223) - p(3107) - p(3302)
p(3323) = p(22)*p(900) - p(2980)
p(3324) = p(14)*p(951)
p(3325) = p(3)*p(1498) - p(3141)
p(3326) = p(2)*p(2227) - p(3118) - p(3114)
p(3327) = p(2)*p(2228) - p(3119)
p(3328) = p(1)*p(2243) - p(3142) - p(3326)
p(3329) = p(7)*p(1471) - p(3141)
p(3330) = p(6)*p(1495) - p(3122)
p(3331) = p(1)*p(2246) - p(3146) - p(3144)
p(3332) = p(73)*p(347) - p(3122)
p(3333) = p(45)*p(582) - p(3089)
p(3334) = p(1)*p(2249) - p(3153) - p(3151)
p(3335) = p(160)*p(195) - p(3089)
p(3336) = p(14)*p(933)
p(3337) = p(187)*p(191) - p(3039)
p(3338) = p(2)*p(2239) - p(3139) - p(3322)
p(3339) = p(102)*p(310) - p(3039)
p(3340) = p(14)*p(954)
p(3341) = p(3)*p(1501) - p(3157)
p(3342) = p(2)*p(2243) - p(3147) - p(3144)
p(3343) = p(1)*p(2253) - p(3157)
p(3344) = p(6)*p(1498) - p(3149)
p(3345) = p(1)*p(2255) - p(3160) - p(3158)
p(3346) = p(32)*p(589) - p(3149)
p(3347) = p(45)*p(585) - p(3133)
p(3348) = p(2)*p(2249) - p(3155) - p(3338)
p(3349) = p(81)*p(347) - p(3133)
p(3350) = p(14)*p(957)
p(3351) = p(2)*p(2252) - p(3159) - p(3158)
p(3352) = p(2)*p(2253) - p(3160)
p(3353) = p(6)*p(1501) - p(3161)
p(3354) = p(2)*p(2255) - p(3162) - p(3348)
p(3355) = p(13)*p(959) - p(3161)
p(3356) = p(14)*p(959)
p(3357) = p(3)*p(1476) - p(3168)
p(3358) = p(7)*p(1476) - p(3165)
p(3359) = p(1)*p(2259) - p(3170) - p(3358)
p(3360) = p(178)*p(194) - p(3166)
p(3361) = p(39)*p(588) - p(3171)
p(3362) = p(3)*p(1504) - p(3174)
p(3363) = p(1)*p(2263) - p(3175)
p(3364) = p(2)*p(2288) - p(3363)
p(3365) = p(1)*p(2265) - p(3192)
p(3366) = p(8)*p(1504) - p(3365)
p(3367) = p(1)*p(2267) - p(3211)
p(3368) = p(22)*p(958) - p(3367)
p(3369) = p(1)*p(2269) - p(3231)
p(3370) = p(50)*p(588) - p(3369)
p(3371) = p(1)*p(2271) - p(3254)
p(3372) = p(102)*p(346) - p(3371)
p(3373) = p(1)*p(2273) - p(3279)
p(3374) = p(194)*p(195) - p(3373)
p(3375) = p(6)*p(1491) - p(3304)
p(3376) = p(1)*p(2276) - p(3305)
p(3377) = p(101)*p(347) - p(3376)
p(3378) = p(8)*p(1491) - p(3278)
p(3379) = p(1)*p(2279) - p(3325)
p(3380) = p(49)*p(589) - p(3379)
p(3381) = p(2)*p(2278) - p(3324) - p(3378)
p(3382) = p(1)*p(2282) - p(3341)
p(3383) = p(21)*p(959) - p(3382)
p(3384) = p(187)*p(195) - p(3296)
p(3385) = p(1)*p(2285) - p(3351)
p(3386) = p(7)*p(1505) - p(3385)
p(3387) = p(45)*p(589) - p(3336)
p(3388) = p(2)*p(2285) - p(3353)
p(3389) = p(1)*p(2289) - p(3388)
p(3390) = p(6)*p(1505) - p(3356)
p(3391) = p(1)*p(2288) - p(3362)
p(3392) = p(2)*p(2289) - p(3390)

do i=1,6
dm(0) = 0.0D0
dm(1) = dmsdr(i,4)
dm(2) = dmsdr(i,2)
dm(3) = dmsdr(i,1)
dm(4) = dmsdr(i,6)
dm(5) = dmsdr(i,5)
dm(6) = dmsdr(i,3)
dm(7) = dm(1)*rm(2) + rm(1)*dm(2)
dm(8) = dm(1)*rm(3) + rm(1)*dm(3)
dm(9) = dm(2)*rm(3) + rm(2)*dm(3)
dm(10) = dm(3)*rm(4) + rm(3)*dm(4)
dm(11) = dm(2)*rm(5) + rm(2)*dm(5)
dm(12) = dm(1)*rm(6) + rm(1)*dm(6)
dm(13) = dm(4)*rm(5) + rm(4)*dm(5)
dm(14) = dm(4)*rm(6) + rm(4)*dm(6)
dm(15) = dm(5)*rm(6) + rm(5)*dm(6)
dm(16) = dm(1)*rm(9) + rm(1)*dm(9)
dm(17) = dm(1)*rm(10) + rm(1)*dm(10)
dm(18) = dm(2)*rm(10) + rm(2)*dm(10)
dm(19) = dm(1)*rm(11) + rm(1)*dm(11)
dm(20) = dm(3)*rm(11) + rm(3)*dm(11)
dm(21) = dm(2)*rm(12) + rm(2)*dm(12)
dm(22) = dm(3)*rm(12) + rm(3)*dm(12)
dm(23) = dm(2)*rm(13) + rm(2)*dm(13)
dm(24) = dm(3)*rm(13) + rm(3)*dm(13)
dm(25) = dm(1)*rm(14) + rm(1)*dm(14)
dm(26) = dm(3)*rm(14) + rm(3)*dm(14)
dm(27) = dm(1)*rm(15) + rm(1)*dm(15)
dm(28) = dm(2)*rm(15) + rm(2)*dm(15)
dm(29) = dm(4)*rm(15) + rm(4)*dm(15)
dm(30) = dm(2)*rm(24) + rm(2)*dm(24)
dm(31) = dm(1)*rm(26) + rm(1)*dm(26)
dm(32) = dm(1)*rm(28) + rm(1)*dm(28)

dp(0) = dm(0)
dp(1) = dm(1) + dm(2) + dm(3)
dp(2) = dm(4) + dm(5) + dm(6)
dp(3) = dm(7) + dm(8) + dm(9)
dp(4) = dm(10) + dm(11) + dm(12)
dp(5) = dp(1)*p(2) + p(1)*dp(2) - dp(4)
dp(6) = dm(13) + dm(14) + dm(15)
dp(7) = dp(1)*p(1) + p(1)*dp(1) - dp(3) - dp(3)
dp(8) = dp(2)*p(2) + p(2)*dp(2) - dp(6) - dp(6)
dp(9) = dm(16)
dp(10) = dm(17) + dm(18) + dm(19) + dm(20) + dm(21) + dm(22)
dp(11) = dp(2)*p(3) + p(2)*dp(3) - dp(10)
dp(12) = dm(23) + dm(24) + dm(25) + dm(26) + dm(27) + dm(28)
dp(13) = dp(1)*p(6) + p(1)*dp(6) - dp(12)
dp(14) = dm(29)
dp(15) = dp(1)*p(3) + p(1)*dp(3) - dp(9) - dp(9) - dp(9)
dp(16) = dp(1)*p(4) + p(1)*dp(4) - dp(10)
dp(17) = dp(2)*p(7) + p(2)*dp(7) - dp(16)
dp(18) = dp(2)*p(4) + p(2)*dp(4) - dp(12)
dp(19) = dp(1)*p(8) + p(1)*dp(8) - dp(18)
dp(20) = dp(2)*p(6) + p(2)*dp(6) - dp(14) - dp(14) - dp(14)
dp(21) = dp(1)*p(7) + p(1)*dp(7) - dp(15)
dp(22) = dp(2)*p(8) + p(2)*dp(8) - dp(20)
dp(23) = dp(9)*p(2) + p(9)*dp(2)
dp(24) = dm(30) + dm(31) + dm(32)
dp(25) = dp(3)*p(6) + p(3)*dp(6) - dp(24)
dp(26) = dp(14)*p(1) + p(14)*dp(1)
dp(27) = dp(9)*p(1) + p(9)*dp(1)
dp(28) = dp(3)*p(4) + p(3)*dp(4) - dp(23)
dp(29) = dp(1)*p(10) + p(1)*dp(10) - dp(23) - dp(28) - dp(23)
dp(30) = dp(1)*p(11) + p(1)*dp(11) - dp(23)
dp(31) = dp(1)*p(12) + p(1)*dp(12) - dp(25) - dp(24) - dp(24)
dp(32) = dp(1)*p(13) + p(1)*dp(13) - dp(25)
dp(33) = dp(4)*p(5) + p(4)*dp(5) - dp(25) - dp(31)
dp(34) = dp(2)*p(11) + p(2)*dp(11) - dp(25)
dp(35) = dp(4)*p(6) + p(4)*dp(6) - dp(26)
dp(36) = dp(2)*p(12) + p(2)*dp(12) - dp(26) - dp(35) - dp(26)
dp(37) = dp(2)*p(13) + p(2)*dp(13) - dp(26)
dp(38) = dp(14)*p(2) + p(14)*dp(2)
dp(39) = dp(3)*p(3) + p(3)*dp(3) - dp(27) - dp(27)
dp(40) = dp(3)*p(7) + p(3)*dp(7) - dp(27)
dp(41) = dp(1)*p(16) + p(1)*dp(16) - dp(28)
dp(42) = dp(2)*p(21) + p(2)*dp(21) - dp(41)
dp(43) = dp(1)*p(18) + p(1)*dp(18) - dp(33)
dp(44) = dp(7)*p(8) + p(7)*dp(8) - dp(43)
dp(45) = dp(6)*p(6) + p(6)*dp(6) - dp(38) - dp(38)
dp(46) = dp(2)*p(18) + p(2)*dp(18) - dp(35)
dp(47) = dp(1)*p(22) + p(1)*dp(22) - dp(46)
dp(48) = dp(6)*p(8) + p(6)*dp(8) - dp(38)
dp(49) = dp(1)*p(21) + p(1)*dp(21) - dp(40)
dp(50) = dp(2)*p(22) + p(2)*dp(22) - dp(48)
dp(51) = dp(9)*p(6) + p(9)*dp(6)
dp(52) = dp(14)*p(3) + p(14)*dp(3)
dp(53) = dp(9)*p(4) + p(9)*dp(4)
dp(54) = dp(9)*p(5) + p(9)*dp(5)
dp(55) = dp(1)*p(24) + p(1)*dp(24) - dp(51)
dp(56) = dp(3)*p(12) + p(3)*dp(12) - dp(51) - dp(55) - dp(51)
dp(57) = dp(3)*p(13) + p(3)*dp(13) - dp(51)
dp(58) = dp(14)*p(7) + p(14)*dp(7)
dp(59) = dp(9)*p(8) + p(9)*dp(8)
dp(60) = dp(2)*p(24) + p(2)*dp(24) - dp(52)
dp(61) = dp(4)*p(13) + p(4)*dp(13) - dp(58)
dp(62) = dp(6)*p(11) + p(6)*dp(11) - dp(52)
dp(63) = dp(14)*p(4) + p(14)*dp(4)
dp(64) = dp(14)*p(5) + p(14)*dp(5)
dp(65) = dp(9)*p(3) + p(9)*dp(3)
dp(66) = dp(9)*p(7) + p(9)*dp(7)
dp(67) = dp(3)*p(16) + p(3)*dp(16) - dp(53)
dp(68) = dp(4)*p(15) + p(4)*dp(15) - dp(54) - dp(67)
dp(69) = dp(2)*p(39) + p(2)*dp(39) - dp(68)
dp(70) = dp(1)*p(29) + p(1)*dp(29) - dp(54) - dp(68)
dp(71) = dp(7)*p(11) + p(7)*dp(11) - dp(53)
dp(72) = dp(1)*p(31) + p(1)*dp(31) - dp(56) - dp(55)
dp(73) = dp(1)*p(32) + p(1)*dp(32) - dp(57)
dp(74) = dp(3)*p(18) + p(3)*dp(18) - dp(59)
dp(75) = dp(2)*p(29) + p(2)*dp(29) - dp(57) - dp(55)
dp(76) = dp(1)*p(34) + p(1)*dp(34) - dp(59)
dp(77) = dp(6)*p(16) + p(6)*dp(16) - dp(58)
dp(78) = dp(1)*p(36) + p(1)*dp(36) - dp(62) - dp(60)
dp(79) = dp(2)*p(32) + p(2)*dp(32) - dp(58)
dp(80) = dp(6)*p(12) + p(6)*dp(12) - dp(64) - dp(63) - dp(63)
dp(81) = dp(1)*p(45) + p(1)*dp(45) - dp(80)
dp(82) = dp(14)*p(6) + p(14)*dp(6)
dp(83) = dp(2)*p(33) + p(2)*dp(33) - dp(61) - dp(60)
dp(84) = dp(2)*p(34) + p(2)*dp(34) - dp(62)
dp(85) = dp(6)*p(18) + p(6)*dp(18) - dp(63)
dp(86) = dp(2)*p(36) + p(2)*dp(36) - dp(64) - dp(80)
dp(87) = dp(8)*p(13) + p(8)*dp(13) - dp(63)
dp(88) = dp(14)*p(8) + p(14)*dp(8)
dp(89) = dp(1)*p(39) + p(1)*dp(39) - dp(65)
dp(90) = dp(3)*p(21) + p(3)*dp(21) - dp(66)
dp(91) = dp(1)*p(41) + p(1)*dp(41) - dp(67)
dp(92) = dp(2)*p(49) + p(2)*dp(49) - dp(91)
dp(93) = dp(1)*p(43) + p(1)*dp(43) - dp(74)
dp(94) = dp(8)*p(21) + p(8)*dp(21) - dp(93)
dp(95) = dp(1)*p(46) + p(1)*dp(46) - dp(83)
dp(96) = dp(7)*p(22) + p(7)*dp(22) - dp(95)
dp(97) = dp(2)*p(45) + p(2)*dp(45) - dp(82)
dp(98) = dp(2)*p(46) + p(2)*dp(46) - dp(85)
dp(99) = dp(1)*p(50) + p(1)*dp(50) - dp(98)
dp(100) = dp(6)*p(22) + p(6)*dp(22) - dp(88)
dp(101) = dp(1)*p(49) + p(1)*dp(49) - dp(90)
dp(102) = dp(2)*p(50) + p(2)*dp(50) - dp(100)
dp(103) = dp(9)*p(14) + p(9)*dp(14)
dp(104) = dp(9)*p(12) + p(9)*dp(12)
dp(105) = dp(9)*p(13) + p(9)*dp(13)
dp(106) = dp(14)*p(15) + p(14)*dp(15)
dp(107) = dp(9)*p(20) + p(9)*dp(20)
dp(108) = dp(14)*p(10) + p(14)*dp(10)
dp(109) = dp(14)*p(11) + p(14)*dp(11)
dp(110) = dp(9)*p(16) + p(9)*dp(16)
dp(111) = dp(9)*p(10) + p(9)*dp(10)
dp(112) = dp(9)*p(11) + p(9)*dp(11)
dp(113) = dp(9)*p(17) + p(9)*dp(17)
dp(114) = dp(3)*p(24) + p(3)*dp(24) - dp(104)
dp(115) = dp(7)*p(24) + p(7)*dp(24) - dp(105)
dp(116) = dp(3)*p(31) + p(3)*dp(31) - dp(104) - dp(115)
dp(117) = dp(6)*p(39) + p(6)*dp(39) - dp(114)
dp(118) = dp(3)*p(32) + p(3)*dp(32) - dp(105)
dp(119) = dp(14)*p(21) + p(14)*dp(21)
dp(120) = dp(9)*p(18) + p(9)*dp(18)
dp(121) = dp(9)*p(19) + p(9)*dp(19)
dp(122) = dp(4)*p(24) + p(4)*dp(24) - dp(103) - dp(103) - dp(103)
dp(123) = dp(1)*p(60) + p(1)*dp(60) - dp(107) - dp(122)
dp(124) = dp(13)*p(16) + p(13)*dp(16) - dp(119)
dp(125) = dp(2)*p(56) + p(2)*dp(56) - dp(106) - dp(124)
dp(126) = dp(4)*p(32) + p(4)*dp(32) - dp(119)
dp(127) = dp(1)*p(62) + p(1)*dp(62) - dp(107) - dp(125)
dp(128) = dp(14)*p(16) + p(14)*dp(16)
dp(129) = dp(14)*p(17) + p(14)*dp(17)
dp(130) = dp(6)*p(24) + p(6)*dp(24) - dp(108)
dp(131) = dp(3)*p(45) + p(3)*dp(45) - dp(130)
dp(132) = dp(14)*p(12) + p(14)*dp(12)
dp(133) = dp(14)*p(13) + p(14)*dp(13)
dp(134) = dp(9)*p(22) + p(9)*dp(22)
dp(135) = dp(8)*p(24) + p(8)*dp(24) - dp(109)
dp(136) = dp(13)*p(18) + p(13)*dp(18) - dp(128)
dp(137) = dp(6)*p(34) + p(6)*dp(34) - dp(109)
dp(138) = dp(14)*p(18) + p(14)*dp(18)
dp(139) = dp(14)*p(19) + p(14)*dp(19)
dp(140) = dp(9)*p(9) + p(9)*dp(9)
dp(141) = dp(9)*p(15) + p(9)*dp(15)
dp(142) = dp(9)*p(21) + p(9)*dp(21)
dp(143) = dp(3)*p(41) + p(3)*dp(41) - dp(110)
dp(144) = dp(4)*p(39) + p(4)*dp(39) - dp(112)
dp(145) = dp(1)*p(68) + p(1)*dp(68) - dp(111) - dp(144)
dp(146) = dp(1)*p(69) + p(1)*dp(69) - dp(112)
dp(147) = dp(1)*p(70) + p(1)*dp(70) - dp(113) - dp(145)
dp(148) = dp(11)*p(21) + p(11)*dp(21) - dp(110)
dp(149) = dp(1)*p(72) + p(1)*dp(72) - dp(116) - dp(115)
dp(150) = dp(1)*p(73) + p(1)*dp(73) - dp(118)
dp(151) = dp(3)*p(43) + p(3)*dp(43) - dp(120)
dp(152) = dp(15)*p(18) + p(15)*dp(18) - dp(121) - dp(151)
dp(153) = dp(2)*p(69) + p(2)*dp(69) - dp(117)
dp(154) = dp(1)*p(75) + p(1)*dp(75) - dp(121) - dp(152)
dp(155) = dp(7)*p(34) + p(7)*dp(34) - dp(120)
dp(156) = dp(6)*p(41) + p(6)*dp(41) - dp(119)
dp(157) = dp(1)*p(78) + p(1)*dp(78) - dp(125) - dp(123)
dp(158) = dp(2)*p(73) + p(2)*dp(73) - dp(119)
dp(159) = dp(1)*p(80) + p(1)*dp(80) - dp(131) - dp(130) - dp(130)
dp(160) = dp(1)*p(81) + p(1)*dp(81) - dp(131)
dp(161) = dp(14)*p(14) + p(14)*dp(14)
dp(162) = dp(3)*p(46) + p(3)*dp(46) - dp(134)
dp(163) = dp(2)*p(75) + p(2)*dp(75) - dp(126) - dp(123)
dp(164) = dp(1)*p(84) + p(1)*dp(84) - dp(134)
dp(165) = dp(6)*p(43) + p(6)*dp(43) - dp(128)
dp(166) = dp(1)*p(86) + p(1)*dp(86) - dp(137) - dp(135)
dp(167) = dp(8)*p(32) + p(8)*dp(32) - dp(128)
dp(168) = dp(4)*p(45) + p(4)*dp(45) - dp(133)
dp(169) = dp(2)*p(80) + p(2)*dp(80) - dp(132) - dp(168)
dp(170) = dp(2)*p(81) + p(2)*dp(81) - dp(133)
dp(171) = dp(14)*p(20) + p(14)*dp(20)
dp(172) = dp(2)*p(83) + p(2)*dp(83) - dp(136) - dp(135)
dp(173) = dp(2)*p(84) + p(2)*dp(84) - dp(137)
dp(174) = dp(6)*p(46) + p(6)*dp(46) - dp(138)
dp(175) = dp(2)*p(86) + p(2)*dp(86) - dp(139) - dp(169)
dp(176) = dp(13)*p(22) + p(13)*dp(22) - dp(138)
dp(177) = dp(14)*p(22) + p(14)*dp(22)
dp(178) = dp(3)*p(39) + p(3)*dp(39) - dp(141)
dp(179) = dp(1)*p(89) + p(1)*dp(89) - dp(141) - dp(178) - dp(178)
dp(180) = dp(3)*p(49) + p(3)*dp(49) - dp(142)
dp(181) = dp(1)*p(91) + p(1)*dp(91) - dp(143)
dp(182) = dp(2)*p(101) + p(2)*dp(101) - dp(181)
dp(183) = dp(1)*p(93) + p(1)*dp(93) - dp(151)
dp(184) = dp(8)*p(49) + p(8)*dp(49) - dp(183)
dp(185) = dp(1)*p(95) + p(1)*dp(95) - dp(162)
dp(186) = dp(21)*p(22) + p(21)*dp(22) - dp(185)
dp(187) = dp(6)*p(45) + p(6)*dp(45) - dp(171)
dp(188) = dp(1)*p(98) + p(1)*dp(98) - dp(172)
dp(189) = dp(7)*p(50) + p(7)*dp(50) - dp(188)
dp(190) = dp(2)*p(97) + p(2)*dp(97) - dp(171) - dp(187) - dp(187)
dp(191) = dp(2)*p(98) + p(2)*dp(98) - dp(174)
dp(192) = dp(1)*p(102) + p(1)*dp(102) - dp(191)
dp(193) = dp(6)*p(50) + p(6)*dp(50) - dp(177)
dp(194) = dp(1)*p(101) + p(1)*dp(101) - dp(180)
dp(195) = dp(2)*p(102) + p(2)*dp(102) - dp(193)
dp(196) = dp(9)*p(26) + p(9)*dp(26)
dp(197) = dp(9)*p(38) + p(9)*dp(38)
dp(198) = dp(9)*p(24) + p(9)*dp(24)
dp(199) = dp(9)*p(31) + p(9)*dp(31)
dp(200) = dp(9)*p(25) + p(9)*dp(25)
dp(201) = dp(9)*p(32) + p(9)*dp(32)
dp(202) = dp(14)*p(39) + p(14)*dp(39)
dp(203) = dp(14)*p(40) + p(14)*dp(40)
dp(204) = dp(9)*p(35) + p(9)*dp(35)
dp(205) = dp(9)*p(36) + p(9)*dp(36)
dp(206) = dp(9)*p(37) + p(9)*dp(37)
dp(207) = dp(14)*p(28) + p(14)*dp(28)
dp(208) = dp(14)*p(29) + p(14)*dp(29)
dp(209) = dp(14)*p(30) + p(14)*dp(30)
dp(210) = dp(9)*p(45) + p(9)*dp(45)
dp(211) = dp(14)*p(24) + p(14)*dp(24)
dp(212) = dp(14)*p(25) + p(14)*dp(25)
dp(213) = dp(9)*p(48) + p(9)*dp(48)
dp(214) = dp(14)*p(33) + p(14)*dp(33)
dp(215) = dp(14)*p(34) + p(14)*dp(34)
dp(216) = dp(9)*p(41) + p(9)*dp(41)
dp(217) = dp(9)*p(28) + p(9)*dp(28)
dp(218) = dp(9)*p(23) + p(9)*dp(23)
dp(219) = dp(9)*p(29) + p(9)*dp(29)
dp(220) = dp(9)*p(30) + p(9)*dp(30)
dp(221) = dp(9)*p(42) + p(9)*dp(42)
dp(222) = dp(1)*p(114) + p(1)*dp(114) - dp(198)
dp(223) = dp(21)*p(24) + p(21)*dp(24) - dp(201)
dp(224) = dp(3)*p(72) + p(3)*dp(72) - dp(199) - dp(223)
dp(225) = dp(1)*p(116) + p(1)*dp(116) - dp(199) - dp(224)
dp(226) = dp(13)*p(39) + p(13)*dp(39) - dp(198)
dp(227) = dp(3)*p(73) + p(3)*dp(73) - dp(201)
dp(228) = dp(14)*p(49) + p(14)*dp(49)
dp(229) = dp(9)*p(43) + p(9)*dp(43)
dp(230) = dp(9)*p(33) + p(9)*dp(33)
dp(231) = dp(9)*p(34) + p(9)*dp(34)
dp(232) = dp(9)*p(44) + p(9)*dp(44)
dp(233) = dp(16)*p(24) + p(16)*dp(24) - dp(196)
dp(234) = dp(2)*p(114) + p(2)*dp(114) - dp(202)
dp(235) = dp(1)*p(123) + p(1)*dp(123) - dp(205) - dp(234)
dp(236) = dp(13)*p(41) + p(13)*dp(41) - dp(228)
dp(237) = dp(2)*p(116) + p(2)*dp(116) - dp(203) - dp(236)
dp(238) = dp(16)*p(32) + p(16)*dp(32) - dp(228)
dp(239) = dp(6)*p(69) + p(6)*dp(69) - dp(202)
dp(240) = dp(4)*p(73) + p(4)*dp(73) - dp(228)
dp(241) = dp(11)*p(32) + p(11)*dp(32) - dp(196)
dp(242) = dp(14)*p(41) + p(14)*dp(41)
dp(243) = dp(14)*p(42) + p(14)*dp(42)
dp(244) = dp(1)*p(130) + p(1)*dp(130) - dp(210)
dp(245) = dp(4)*p(62) + p(4)*dp(62) - dp(213) - dp(209)
dp(246) = dp(3)*p(81) + p(3)*dp(81) - dp(210)
dp(247) = dp(14)*p(31) + p(14)*dp(31)
dp(248) = dp(14)*p(32) + p(14)*dp(32)
dp(249) = dp(14)*p(26) + p(14)*dp(26)
dp(250) = dp(9)*p(46) + p(9)*dp(46)
dp(251) = dp(9)*p(47) + p(9)*dp(47)
dp(252) = dp(18)*p(24) + p(18)*dp(24) - dp(197)
dp(253) = dp(1)*p(135) + p(1)*dp(135) - dp(213) - dp(252)
dp(254) = dp(13)*p(43) + p(13)*dp(43) - dp(242)
dp(255) = dp(2)*p(125) + p(2)*dp(125) - dp(209) - dp(245)
dp(256) = dp(18)*p(32) + p(18)*dp(32) - dp(242)
dp(257) = dp(13)*p(34) + p(13)*dp(34) - dp(197)
dp(258) = dp(14)*p(43) + p(14)*dp(43)
dp(259) = dp(14)*p(44) + p(14)*dp(44)
dp(260) = dp(2)*p(130) + p(2)*dp(130) - dp(211)
dp(261) = dp(4)*p(81) + p(4)*dp(81) - dp(248)
dp(262) = dp(11)*p(45) + p(11)*dp(45) - dp(211)
dp(263) = dp(14)*p(35) + p(14)*dp(35)
dp(264) = dp(14)*p(36) + p(14)*dp(36)
dp(265) = dp(14)*p(37) + p(14)*dp(37)
dp(266) = dp(9)*p(50) + p(9)*dp(50)
dp(267) = dp(22)*p(24) + p(22)*dp(24) - dp(215)
dp(268) = dp(13)*p(46) + p(13)*dp(46) - dp(258)
dp(269) = dp(6)*p(84) + p(6)*dp(84) - dp(215)
dp(270) = dp(14)*p(46) + p(14)*dp(46)
dp(271) = dp(14)*p(47) + p(14)*dp(47)
dp(272) = dp(9)*p(27) + p(9)*dp(27)
dp(273) = dp(9)*p(39) + p(9)*dp(39)
dp(274) = dp(9)*p(40) + p(9)*dp(40)
dp(275) = dp(9)*p(49) + p(9)*dp(49)
dp(276) = dp(3)*p(91) + p(3)*dp(91) - dp(216)
dp(277) = dp(16)*p(39) + p(16)*dp(39) - dp(218)
dp(278) = dp(1)*p(144) + p(1)*dp(144) - dp(217) - dp(277)
dp(279) = dp(2)*p(178) + p(2)*dp(178) - dp(278)
dp(280) = dp(1)*p(145) + p(1)*dp(145) - dp(219) - dp(278)
dp(281) = dp(7)*p(69) + p(7)*dp(69) - dp(218)
dp(282) = dp(1)*p(147) + p(1)*dp(147) - dp(221) - dp(280)
dp(283) = dp(11)*p(49) + p(11)*dp(49) - dp(216)
dp(284) = dp(1)*p(149) + p(1)*dp(149) - dp(224) - dp(223)
dp(285) = dp(1)*p(150) + p(1)*dp(150) - dp(227)
dp(286) = dp(3)*p(93) + p(3)*dp(93) - dp(229)
dp(287) = dp(18)*p(39) + p(18)*dp(39) - dp(231)
dp(288) = dp(1)*p(152) + p(1)*dp(152) - dp(230) - dp(287)
dp(289) = dp(1)*p(153) + p(1)*dp(153) - dp(231)
dp(290) = dp(1)*p(154) + p(1)*dp(154) - dp(232) - dp(288)
dp(291) = dp(21)*p(34) + p(21)*dp(34) - dp(229)
dp(292) = dp(6)*p(91) + p(6)*dp(91) - dp(228)
dp(293) = dp(1)*p(157) + p(1)*dp(157) - dp(237) - dp(235)
dp(294) = dp(2)*p(150) + p(2)*dp(150) - dp(228)
dp(295) = dp(1)*p(159) + p(1)*dp(159) - dp(245) - dp(244)
dp(296) = dp(1)*p(160) + p(1)*dp(160) - dp(246)
dp(297) = dp(3)*p(95) + p(3)*dp(95) - dp(250)
dp(298) = dp(2)*p(152) + p(2)*dp(152) - dp(238) - dp(234)
dp(299) = dp(2)*p(153) + p(2)*dp(153) - dp(239)
dp(300) = dp(1)*p(163) + p(1)*dp(163) - dp(251) - dp(298)
dp(301) = dp(7)*p(84) + p(7)*dp(84) - dp(250)
dp(302) = dp(6)*p(93) + p(6)*dp(93) - dp(242)
dp(303) = dp(1)*p(166) + p(1)*dp(166) - dp(255) - dp(253)
dp(304) = dp(8)*p(73) + p(8)*dp(73) - dp(242)
dp(305) = dp(16)*p(45) + p(16)*dp(45) - dp(248)
dp(306) = dp(1)*p(169) + p(1)*dp(169) - dp(262) - dp(260)
dp(307) = dp(2)*p(160) + p(2)*dp(160) - dp(248)
dp(308) = dp(14)*p(38) + p(14)*dp(38)
dp(309) = dp(6)*p(80) + p(6)*dp(80) - dp(264) - dp(263)
dp(310) = dp(1)*p(187) + p(1)*dp(187) - dp(309)
dp(311) = dp(14)*p(45) + p(14)*dp(45)
dp(312) = dp(3)*p(98) + p(3)*dp(98) - dp(266)
dp(313) = dp(2)*p(163) + p(2)*dp(163) - dp(256) - dp(253)
dp(314) = dp(1)*p(173) + p(1)*dp(173) - dp(266)
dp(315) = dp(6)*p(95) + p(6)*dp(95) - dp(258)
dp(316) = dp(1)*p(175) + p(1)*dp(175) - dp(269) - dp(267)
dp(317) = dp(22)*p(32) + p(22)*dp(32) - dp(258)
dp(318) = dp(18)*p(45) + p(18)*dp(45) - dp(249)
dp(319) = dp(2)*p(169) + p(2)*dp(169) - dp(264) - dp(309)
dp(320) = dp(8)*p(81) + p(8)*dp(81) - dp(249)
dp(321) = dp(14)*p(48) + p(14)*dp(48)
dp(322) = dp(2)*p(172) + p(2)*dp(172) - dp(268) - dp(267)
dp(323) = dp(2)*p(173) + p(2)*dp(173) - dp(269)
dp(324) = dp(6)*p(98) + p(6)*dp(98) - dp(270)
dp(325) = dp(2)*p(175) + p(2)*dp(175) - dp(271) - dp(319)
dp(326) = dp(13)*p(50) + p(13)*dp(50) - dp(270)
dp(327) = dp(14)*p(50) + p(14)*dp(50)
dp(328) = dp(1)*p(178) + p(1)*dp(178) - dp(273)
dp(329) = dp(21)*p(39) + p(21)*dp(39) - dp(272)
dp(330) = dp(3)*p(101) + p(3)*dp(101) - dp(275)
dp(331) = dp(1)*p(181) + p(1)*dp(181) - dp(276)
dp(332) = dp(2)*p(194) + p(2)*dp(194) - dp(331)
dp(333) = dp(1)*p(183) + p(1)*dp(183) - dp(286)
dp(334) = dp(8)*p(101) + p(8)*dp(101) - dp(333)
dp(335) = dp(1)*p(185) + p(1)*dp(185) - dp(297)
dp(336) = dp(22)*p(49) + p(22)*dp(49) - dp(335)
dp(337) = dp(1)*p(188) + p(1)*dp(188) - dp(312)
dp(338) = dp(21)*p(50) + p(21)*dp(50) - dp(337)
dp(339) = dp(2)*p(187) + p(2)*dp(187) - dp(311)
dp(340) = dp(1)*p(191) + p(1)*dp(191) - dp(322)
dp(341) = dp(7)*p(102) + p(7)*dp(102) - dp(340)
dp(342) = dp(22)*p(45) + p(22)*dp(45) - dp(308)
dp(343) = dp(2)*p(191) + p(2)*dp(191) - dp(324)
dp(344) = dp(1)*p(195) + p(1)*dp(195) - dp(343)
dp(345) = dp(6)*p(102) + p(6)*dp(102) - dp(327)
dp(346) = dp(1)*p(194) + p(1)*dp(194) - dp(330)
dp(347) = dp(2)*p(195) + p(2)*dp(195) - dp(345)
dp(348) = dp(9)*p(52) + p(9)*dp(52)
dp(349) = dp(9)*p(58) + p(9)*dp(58)
dp(350) = dp(9)*p(63) + p(9)*dp(63)
dp(351) = dp(9)*p(64) + p(9)*dp(64)
dp(352) = dp(9)*p(82) + p(9)*dp(82)
dp(353) = dp(9)*p(88) + p(9)*dp(88)
dp(354) = dp(9)*p(55) + p(9)*dp(55)
dp(355) = dp(9)*p(72) + p(9)*dp(72)
dp(356) = dp(9)*p(51) + p(9)*dp(51)
dp(357) = dp(9)*p(56) + p(9)*dp(56)
dp(358) = dp(9)*p(57) + p(9)*dp(57)
dp(359) = dp(9)*p(73) + p(9)*dp(73)
dp(360) = dp(14)*p(89) + p(14)*dp(89)
dp(361) = dp(14)*p(90) + p(14)*dp(90)
dp(362) = dp(9)*p(77) + p(9)*dp(77)
dp(363) = dp(9)*p(60) + p(9)*dp(60)
dp(364) = dp(9)*p(78) + p(9)*dp(78)
dp(365) = dp(9)*p(61) + p(9)*dp(61)
dp(366) = dp(9)*p(62) + p(9)*dp(62)
dp(367) = dp(9)*p(79) + p(9)*dp(79)
dp(368) = dp(14)*p(67) + p(14)*dp(67)
dp(369) = dp(14)*p(68) + p(14)*dp(68)
dp(370) = dp(14)*p(69) + p(14)*dp(69)
dp(371) = dp(14)*p(70) + p(14)*dp(70)
dp(372) = dp(14)*p(71) + p(14)*dp(71)
dp(373) = dp(9)*p(80) + p(9)*dp(80)
dp(374) = dp(9)*p(81) + p(9)*dp(81)
dp(375) = dp(14)*p(55) + p(14)*dp(55)
dp(376) = dp(14)*p(56) + p(14)*dp(56)
dp(377) = dp(14)*p(57) + p(14)*dp(57)
dp(378) = dp(14)*p(52) + p(14)*dp(52)
dp(379) = dp(9)*p(85) + p(9)*dp(85)
dp(380) = dp(9)*p(86) + p(9)*dp(86)
dp(381) = dp(9)*p(87) + p(9)*dp(87)
dp(382) = dp(14)*p(74) + p(14)*dp(74)
dp(383) = dp(14)*p(75) + p(14)*dp(75)
dp(384) = dp(14)*p(76) + p(14)*dp(76)
dp(385) = dp(9)*p(97) + p(9)*dp(97)
dp(386) = dp(14)*p(60) + p(14)*dp(60)
dp(387) = dp(14)*p(61) + p(14)*dp(61)
dp(388) = dp(14)*p(62) + p(14)*dp(62)
dp(389) = dp(9)*p(100) + p(9)*dp(100)
dp(390) = dp(14)*p(83) + p(14)*dp(83)
dp(391) = dp(14)*p(84) + p(14)*dp(84)
dp(392) = dp(9)*p(91) + p(9)*dp(91)
dp(393) = dp(9)*p(67) + p(9)*dp(67)
dp(394) = dp(9)*p(53) + p(9)*dp(53)
dp(395) = dp(9)*p(68) + p(9)*dp(68)
dp(396) = dp(9)*p(54) + p(9)*dp(54)
dp(397) = dp(9)*p(69) + p(9)*dp(69)
dp(398) = dp(9)*p(70) + p(9)*dp(70)
dp(399) = dp(9)*p(71) + p(9)*dp(71)
dp(400) = dp(9)*p(92) + p(9)*dp(92)
dp(401) = dp(3)*p(114) + p(3)*dp(114) - dp(354)
dp(402) = dp(7)*p(114) + p(7)*dp(114) - dp(356)
dp(403) = dp(24)*p(49) + p(24)*dp(49) - dp(359)
dp(404) = dp(3)*p(149) + p(3)*dp(149) - dp(355) - dp(403)
dp(405) = dp(1)*p(224) + p(1)*dp(224) - dp(355) - dp(404)
dp(406) = dp(6)*p(178) + p(6)*dp(178) - dp(401)
dp(407) = dp(32)*p(39) + p(32)*dp(39) - dp(356)
dp(408) = dp(3)*p(150) + p(3)*dp(150) - dp(359)
dp(409) = dp(14)*p(101) + p(14)*dp(101)
dp(410) = dp(9)*p(93) + p(9)*dp(93)
dp(411) = dp(9)*p(74) + p(9)*dp(74)
dp(412) = dp(9)*p(59) + p(9)*dp(59)
dp(413) = dp(9)*p(75) + p(9)*dp(75)
dp(414) = dp(9)*p(76) + p(9)*dp(76)
dp(415) = dp(9)*p(94) + p(9)*dp(94)
dp(416) = dp(24)*p(41) + p(24)*dp(41) - dp(349)
dp(417) = dp(4)*p(114) + p(4)*dp(114) - dp(348)
dp(418) = dp(1)*p(234) + p(1)*dp(234) - dp(363) - dp(417)
dp(419) = dp(1)*p(235) + p(1)*dp(235) - dp(364) - dp(418)
dp(420) = dp(13)*p(91) + p(13)*dp(91) - dp(409)
dp(421) = dp(2)*p(224) + p(2)*dp(224) - dp(361) - dp(420)
dp(422) = dp(32)*p(41) + p(32)*dp(41) - dp(409)
dp(423) = dp(1)*p(237) + p(1)*dp(237) - dp(364) - dp(421)
dp(424) = dp(16)*p(73) + p(16)*dp(73) - dp(409)
dp(425) = dp(13)*p(69) + p(13)*dp(69) - dp(348)
dp(426) = dp(4)*p(150) + p(4)*dp(150) - dp(409)
dp(427) = dp(11)*p(73) + p(11)*dp(73) - dp(349)
dp(428) = dp(14)*p(91) + p(14)*dp(91)
dp(429) = dp(14)*p(92) + p(14)*dp(92)
dp(430) = dp(3)*p(130) + p(3)*dp(130) - dp(373)
dp(431) = dp(7)*p(130) + p(7)*dp(130) - dp(374)
dp(432) = dp(3)*p(159) + p(3)*dp(159) - dp(373) - dp(431)
dp(433) = dp(39)*p(45) + p(39)*dp(45) - dp(430)
dp(434) = dp(3)*p(160) + p(3)*dp(160) - dp(374)
dp(435) = dp(14)*p(72) + p(14)*dp(72)
dp(436) = dp(14)*p(73) + p(14)*dp(73)
dp(437) = dp(14)*p(58) + p(14)*dp(58)
dp(438) = dp(9)*p(95) + p(9)*dp(95)
dp(439) = dp(9)*p(83) + p(9)*dp(83)
dp(440) = dp(9)*p(84) + p(9)*dp(84)
dp(441) = dp(9)*p(96) + p(9)*dp(96)
dp(442) = dp(24)*p(43) + p(24)*dp(43) - dp(350)
dp(443) = dp(8)*p(114) + p(8)*dp(114) - dp(370)
dp(444) = dp(1)*p(253) + p(1)*dp(253) - dp(380) - dp(443)
dp(445) = dp(13)*p(93) + p(13)*dp(93) - dp(428)
dp(446) = dp(2)*p(237) + p(2)*dp(237) - dp(372) - dp(432)
dp(447) = dp(32)*p(43) + p(32)*dp(43) - dp(428)
dp(448) = dp(6)*p(153) + p(6)*dp(153) - dp(370)
dp(449) = dp(18)*p(73) + p(18)*dp(73) - dp(428)
dp(450) = dp(32)*p(34) + p(32)*dp(34) - dp(350)
dp(451) = dp(14)*p(93) + p(14)*dp(93)
dp(452) = dp(14)*p(94) + p(14)*dp(94)
dp(453) = dp(4)*p(130) + p(4)*dp(130) - dp(352)
dp(454) = dp(1)*p(260) + p(1)*dp(260) - dp(385) - dp(453)
dp(455) = dp(16)*p(81) + p(16)*dp(81) - dp(436)
dp(456) = dp(2)*p(245) + p(2)*dp(245) - dp(376) - dp(455)
dp(457) = dp(4)*p(160) + p(4)*dp(160) - dp(436)
dp(458) = dp(11)*p(81) + p(11)*dp(81) - dp(352)
dp(459) = dp(14)*p(77) + p(14)*dp(77)
dp(460) = dp(14)*p(78) + p(14)*dp(78)
dp(461) = dp(14)*p(79) + p(14)*dp(79)
dp(462) = dp(14)*p(63) + p(14)*dp(63)
dp(463) = dp(14)*p(64) + p(14)*dp(64)
dp(464) = dp(6)*p(130) + p(6)*dp(130) - dp(386)
dp(465) = dp(3)*p(187) + p(3)*dp(187) - dp(464)
dp(466) = dp(14)*p(80) + p(14)*dp(80)
dp(467) = dp(14)*p(81) + p(14)*dp(81)
dp(468) = dp(9)*p(98) + p(9)*dp(98)
dp(469) = dp(9)*p(99) + p(9)*dp(99)
dp(470) = dp(24)*p(46) + p(24)*dp(46) - dp(353)
dp(471) = dp(1)*p(267) + p(1)*dp(267) - dp(389) - dp(470)
dp(472) = dp(13)*p(95) + p(13)*dp(95) - dp(451)
dp(473) = dp(2)*p(255) + p(2)*dp(255) - dp(384) - dp(456)
dp(474) = dp(32)*p(46) + p(32)*dp(46) - dp(451)
dp(475) = dp(13)*p(84) + p(13)*dp(84) - dp(353)
dp(476) = dp(14)*p(95) + p(14)*dp(95)
dp(477) = dp(14)*p(96) + p(14)*dp(96)
dp(478) = dp(8)*p(130) + p(8)*dp(130) - dp(378)
dp(479) = dp(18)*p(81) + p(18)*dp(81) - dp(437)
dp(480) = dp(34)*p(45) + p(34)*dp(45) - dp(378)
dp(481) = dp(14)*p(85) + p(14)*dp(85)
dp(482) = dp(14)*p(86) + p(14)*dp(86)
dp(483) = dp(14)*p(87) + p(14)*dp(87)
dp(484) = dp(9)*p(102) + p(9)*dp(102)
dp(485) = dp(24)*p(50) + p(24)*dp(50) - dp(391)
dp(486) = dp(13)*p(98) + p(13)*dp(98) - dp(476)
dp(487) = dp(6)*p(173) + p(6)*dp(173) - dp(391)
dp(488) = dp(14)*p(98) + p(14)*dp(98)
dp(489) = dp(14)*p(99) + p(14)*dp(99)
dp(490) = dp(9)*p(65) + p(9)*dp(65)
dp(491) = dp(9)*p(66) + p(9)*dp(66)
dp(492) = dp(9)*p(89) + p(9)*dp(89)
dp(493) = dp(9)*p(90) + p(9)*dp(90)
dp(494) = dp(9)*p(101) + p(9)*dp(101)
dp(495) = dp(3)*p(181) + p(3)*dp(181) - dp(392)
dp(496) = dp(39)*p(41) + p(39)*dp(41) - dp(394)
dp(497) = dp(4)*p(178) + p(4)*dp(178) - dp(397)
dp(498) = dp(1)*p(278) + p(1)*dp(278) - dp(395) - dp(497)
dp(499) = dp(1)*p(279) + p(1)*dp(279) - dp(397)
dp(500) = dp(1)*p(280) + p(1)*dp(280) - dp(398) - dp(498)
dp(501) = dp(21)*p(69) + p(21)*dp(69) - dp(394)
dp(502) = dp(1)*p(282) + p(1)*dp(282) - dp(400) - dp(500)
dp(503) = dp(11)*p(101) + p(11)*dp(101) - dp(392)
dp(504) = dp(1)*p(284) + p(1)*dp(284) - dp(404) - dp(403)
dp(505) = dp(1)*p(285) + p(1)*dp(285) - dp(408)
dp(506) = dp(3)*p(183) + p(3)*dp(183) - dp(410)
dp(507) = dp(39)*p(43) + p(39)*dp(43) - dp(412)
dp(508) = dp(1)*p(287) + p(1)*dp(287) - dp(411) - dp(507)
dp(509) = dp(2)*p(279) + p(2)*dp(279) - dp(406)
dp(510) = dp(1)*p(288) + p(1)*dp(288) - dp(413) - dp(508)
dp(511) = dp(7)*p(153) + p(7)*dp(153) - dp(412)
dp(512) = dp(1)*p(290) + p(1)*dp(290) - dp(415) - dp(510)
dp(513) = dp(34)*p(49) + p(34)*dp(49) - dp(410)
dp(514) = dp(6)*p(181) + p(6)*dp(181) - dp(409)
dp(515) = dp(1)*p(293) + p(1)*dp(293) - dp(421) - dp(419)
dp(516) = dp(2)*p(285) + p(2)*dp(285) - dp(409)
dp(517) = dp(1)*p(295) + p(1)*dp(295) - dp(432) - dp(431)
dp(518) = dp(1)*p(296) + p(1)*dp(296) - dp(434)
dp(519) = dp(3)*p(185) + p(3)*dp(185) - dp(438)
dp(520) = dp(39)*p(46) + p(39)*dp(46) - dp(440)
dp(521) = dp(1)*p(298) + p(1)*dp(298) - dp(439) - dp(520)
dp(522) = dp(1)*p(299) + p(1)*dp(299) - dp(440)
dp(523) = dp(1)*p(300) + p(1)*dp(300) - dp(441) - dp(521)
dp(524) = dp(21)*p(84) + p(21)*dp(84) - dp(438)
dp(525) = dp(6)*p(183) + p(6)*dp(183) - dp(428)
dp(526) = dp(1)*p(303) + p(1)*dp(303) - dp(446) - dp(444)
dp(527) = dp(8)*p(150) + p(8)*dp(150) - dp(428)
dp(528) = dp(41)*p(45) + p(41)*dp(45) - dp(436)
dp(529) = dp(1)*p(306) + p(1)*dp(306) - dp(456) - dp(454)
dp(530) = dp(2)*p(296) + p(2)*dp(296) - dp(436)
dp(531) = dp(4)*p(169) + p(4)*dp(169) - dp(388) - dp(478)
dp(532) = dp(1)*p(310) + p(1)*dp(310) - dp(465)
dp(533) = dp(14)*p(82) + p(14)*dp(82)
dp(534) = dp(3)*p(188) + p(3)*dp(188) - dp(468)
dp(535) = dp(2)*p(298) + p(2)*dp(298) - dp(447) - dp(443)
dp(536) = dp(2)*p(299) + p(2)*dp(299) - dp(448)
dp(537) = dp(1)*p(313) + p(1)*dp(313) - dp(469) - dp(535)
dp(538) = dp(7)*p(173) + p(7)*dp(173) - dp(468)
dp(539) = dp(6)*p(185) + p(6)*dp(185) - dp(451)
dp(540) = dp(1)*p(316) + p(1)*dp(316) - dp(473) - dp(471)
dp(541) = dp(22)*p(73) + p(22)*dp(73) - dp(451)
dp(542) = dp(43)*p(45) + p(43)*dp(45) - dp(437)
dp(543) = dp(1)*p(319) + p(1)*dp(319) - dp(480) - dp(478)
dp(544) = dp(8)*p(160) + p(8)*dp(160) - dp(437)
dp(545) = dp(14)*p(88) + p(14)*dp(88)
dp(546) = dp(4)*p(187) + p(4)*dp(187) - dp(467)
dp(547) = dp(2)*p(309) + p(2)*dp(309) - dp(466) - dp(546)
dp(548) = dp(2)*p(310) + p(2)*dp(310) - dp(467)
dp(549) = dp(14)*p(97) + p(14)*dp(97)
dp(550) = dp(3)*p(191) + p(3)*dp(191) - dp(484)
dp(551) = dp(2)*p(313) + p(2)*dp(313) - dp(474) - dp(471)
dp(552) = dp(1)*p(323) + p(1)*dp(323) - dp(484)
dp(553) = dp(6)*p(188) + p(6)*dp(188) - dp(476)
dp(554) = dp(1)*p(325) + p(1)*dp(325) - dp(487) - dp(485)
dp(555) = dp(32)*p(50) + p(32)*dp(50) - dp(476)
dp(556) = dp(45)*p(46) + p(45)*dp(46) - dp(462)
dp(557) = dp(2)*p(319) + p(2)*dp(319) - dp(482) - dp(547)
dp(558) = dp(22)*p(81) + p(22)*dp(81) - dp(462)
dp(559) = dp(14)*p(100) + p(14)*dp(100)
dp(560) = dp(2)*p(322) + p(2)*dp(322) - dp(486) - dp(485)
dp(561) = dp(2)*p(323) + p(2)*dp(323) - dp(487)
dp(562) = dp(6)*p(191) + p(6)*dp(191) - dp(488)
dp(563) = dp(2)*p(325) + p(2)*dp(325) - dp(489) - dp(557)
dp(564) = dp(13)*p(102) + p(13)*dp(102) - dp(488)
dp(565) = dp(14)*p(102) + p(14)*dp(102)
dp(566) = dp(3)*p(178) + p(3)*dp(178) - dp(492)
dp(567) = dp(7)*p(178) + p(7)*dp(178) - dp(490)
dp(568) = dp(39)*p(49) + p(39)*dp(49) - dp(491)
dp(569) = dp(3)*p(194) + p(3)*dp(194) - dp(494)
dp(570) = dp(1)*p(331) + p(1)*dp(331) - dp(495)
dp(571) = dp(2)*p(346) + p(2)*dp(346) - dp(570)
dp(572) = dp(1)*p(333) + p(1)*dp(333) - dp(506)
dp(573) = dp(8)*p(194) + p(8)*dp(194) - dp(572)
dp(574) = dp(1)*p(335) + p(1)*dp(335) - dp(519)
dp(575) = dp(22)*p(101) + p(22)*dp(101) - dp(574)
dp(576) = dp(1)*p(337) + p(1)*dp(337) - dp(534)
dp(577) = dp(49)*p(50) + p(49)*dp(50) - dp(576)
dp(578) = dp(6)*p(187) + p(6)*dp(187) - dp(549)
dp(579) = dp(1)*p(340) + p(1)*dp(340) - dp(550)
dp(580) = dp(21)*p(102) + p(21)*dp(102) - dp(579)
dp(581) = dp(8)*p(187) + p(8)*dp(187) - dp(533)
dp(582) = dp(1)*p(343) + p(1)*dp(343) - dp(560)
dp(583) = dp(7)*p(195) + p(7)*dp(195) - dp(582)
dp(584) = dp(45)*p(50) + p(45)*dp(50) - dp(545)
dp(585) = dp(2)*p(343) + p(2)*dp(343) - dp(562)
dp(586) = dp(1)*p(347) + p(1)*dp(347) - dp(585)
dp(587) = dp(6)*p(195) + p(6)*dp(195) - dp(565)
dp(588) = dp(1)*p(346) + p(1)*dp(346) - dp(569)
dp(589) = dp(2)*p(347) + p(2)*dp(347) - dp(587)
dp(590) = dp(9)*p(103) + p(9)*dp(103)
dp(591) = dp(9)*p(106) + p(9)*dp(106)
dp(592) = dp(9)*p(119) + p(9)*dp(119)
dp(593) = dp(9)*p(128) + p(9)*dp(128)
dp(594) = dp(9)*p(108) + p(9)*dp(108)
dp(595) = dp(9)*p(109) + p(9)*dp(109)
dp(596) = dp(9)*p(129) + p(9)*dp(129)
dp(597) = dp(9)*p(132) + p(9)*dp(132)
dp(598) = dp(9)*p(133) + p(9)*dp(133)
dp(599) = dp(9)*p(161) + p(9)*dp(161)
dp(600) = dp(9)*p(138) + p(9)*dp(138)
dp(601) = dp(9)*p(139) + p(9)*dp(139)
dp(602) = dp(9)*p(171) + p(9)*dp(171)
dp(603) = dp(9)*p(177) + p(9)*dp(177)
dp(604) = dp(9)*p(114) + p(9)*dp(114)
dp(605) = dp(9)*p(115) + p(9)*dp(115)
dp(606) = dp(9)*p(149) + p(9)*dp(149)
dp(607) = dp(9)*p(104) + p(9)*dp(104)
dp(608) = dp(9)*p(116) + p(9)*dp(116)
dp(609) = dp(9)*p(105) + p(9)*dp(105)
dp(610) = dp(9)*p(117) + p(9)*dp(117)
dp(611) = dp(9)*p(118) + p(9)*dp(118)
dp(612) = dp(9)*p(150) + p(9)*dp(150)
dp(613) = dp(14)*p(178) + p(14)*dp(178)
dp(614) = dp(14)*p(179) + p(14)*dp(179)
dp(615) = dp(14)*p(180) + p(14)*dp(180)
dp(616) = dp(9)*p(156) + p(9)*dp(156)
dp(617) = dp(9)*p(122) + p(9)*dp(122)
dp(618) = dp(9)*p(123) + p(9)*dp(123)
dp(619) = dp(9)*p(157) + p(9)*dp(157)
dp(620) = dp(9)*p(124) + p(9)*dp(124)
dp(621) = dp(9)*p(107) + p(9)*dp(107)
dp(622) = dp(9)*p(125) + p(9)*dp(125)
dp(623) = dp(9)*p(126) + p(9)*dp(126)
dp(624) = dp(9)*p(127) + p(9)*dp(127)
dp(625) = dp(9)*p(158) + p(9)*dp(158)
dp(626) = dp(14)*p(143) + p(14)*dp(143)
dp(627) = dp(14)*p(144) + p(14)*dp(144)
dp(628) = dp(14)*p(145) + p(14)*dp(145)
dp(629) = dp(14)*p(146) + p(14)*dp(146)
dp(630) = dp(14)*p(147) + p(14)*dp(147)
dp(631) = dp(14)*p(148) + p(14)*dp(148)
dp(632) = dp(9)*p(130) + p(9)*dp(130)
dp(633) = dp(9)*p(159) + p(9)*dp(159)
dp(634) = dp(9)*p(131) + p(9)*dp(131)
dp(635) = dp(9)*p(160) + p(9)*dp(160)
dp(636) = dp(14)*p(114) + p(14)*dp(114)
dp(637) = dp(14)*p(115) + p(14)*dp(115)
dp(638) = dp(14)*p(116) + p(14)*dp(116)
dp(639) = dp(14)*p(117) + p(14)*dp(117)
dp(640) = dp(14)*p(118) + p(14)*dp(118)
dp(641) = dp(14)*p(106) + p(14)*dp(106)
dp(642) = dp(9)*p(165) + p(9)*dp(165)
dp(643) = dp(9)*p(135) + p(9)*dp(135)
dp(644) = dp(9)*p(166) + p(9)*dp(166)
dp(645) = dp(9)*p(136) + p(9)*dp(136)
dp(646) = dp(9)*p(137) + p(9)*dp(137)
dp(647) = dp(9)*p(167) + p(9)*dp(167)
dp(648) = dp(14)*p(151) + p(14)*dp(151)
dp(649) = dp(14)*p(152) + p(14)*dp(152)
dp(650) = dp(14)*p(153) + p(14)*dp(153)
dp(651) = dp(14)*p(154) + p(14)*dp(154)
dp(652) = dp(14)*p(155) + p(14)*dp(155)
dp(653) = dp(9)*p(168) + p(9)*dp(168)
dp(654) = dp(9)*p(169) + p(9)*dp(169)
dp(655) = dp(9)*p(170) + p(9)*dp(170)
dp(656) = dp(14)*p(122) + p(14)*dp(122)
dp(657) = dp(14)*p(123) + p(14)*dp(123)
dp(658) = dp(14)*p(124) + p(14)*dp(124)
dp(659) = dp(14)*p(125) + p(14)*dp(125)
dp(660) = dp(14)*p(126) + p(14)*dp(126)
dp(661) = dp(14)*p(127) + p(14)*dp(127)
dp(662) = dp(14)*p(108) + p(14)*dp(108)
dp(663) = dp(14)*p(109) + p(14)*dp(109)
dp(664) = dp(9)*p(187) + p(9)*dp(187)
dp(665) = dp(14)*p(130) + p(14)*dp(130)
dp(666) = dp(14)*p(131) + p(14)*dp(131)
dp(667) = dp(9)*p(174) + p(9)*dp(174)
dp(668) = dp(9)*p(175) + p(9)*dp(175)
dp(669) = dp(9)*p(176) + p(9)*dp(176)
dp(670) = dp(14)*p(162) + p(14)*dp(162)
dp(671) = dp(14)*p(163) + p(14)*dp(163)
dp(672) = dp(14)*p(164) + p(14)*dp(164)
dp(673) = dp(9)*p(190) + p(9)*dp(190)
dp(674) = dp(14)*p(135) + p(14)*dp(135)
dp(675) = dp(14)*p(136) + p(14)*dp(136)
dp(676) = dp(14)*p(137) + p(14)*dp(137)
dp(677) = dp(9)*p(193) + p(9)*dp(193)
dp(678) = dp(14)*p(172) + p(14)*dp(172)
dp(679) = dp(14)*p(173) + p(14)*dp(173)
dp(680) = dp(9)*p(181) + p(9)*dp(181)
dp(681) = dp(9)*p(143) + p(9)*dp(143)
dp(682) = dp(9)*p(110) + p(9)*dp(110)
dp(683) = dp(9)*p(144) + p(9)*dp(144)
dp(684) = dp(9)*p(111) + p(9)*dp(111)
dp(685) = dp(9)*p(112) + p(9)*dp(112)
dp(686) = dp(9)*p(145) + p(9)*dp(145)
dp(687) = dp(9)*p(113) + p(9)*dp(113)
dp(688) = dp(9)*p(146) + p(9)*dp(146)
dp(689) = dp(9)*p(147) + p(9)*dp(147)
dp(690) = dp(9)*p(148) + p(9)*dp(148)
dp(691) = dp(9)*p(182) + p(9)*dp(182)
dp(692) = dp(1)*p(401) + p(1)*dp(401) - dp(604)
dp(693) = dp(21)*p(114) + p(21)*dp(114) - dp(609)
dp(694) = dp(24)*p(101) + p(24)*dp(101) - dp(612)
dp(695) = dp(3)*p(284) + p(3)*dp(284) - dp(606) - dp(694)
dp(696) = dp(1)*p(404) + p(1)*dp(404) - dp(606) - dp(695)
dp(697) = dp(1)*p(405) + p(1)*dp(405) - dp(608) - dp(696)
dp(698) = dp(13)*p(178) + p(13)*dp(178) - dp(604)
dp(699) = dp(39)*p(73) + p(39)*dp(73) - dp(609)
dp(700) = dp(3)*p(285) + p(3)*dp(285) - dp(612)
dp(701) = dp(14)*p(194) + p(14)*dp(194)
dp(702) = dp(9)*p(183) + p(9)*dp(183)
dp(703) = dp(9)*p(151) + p(9)*dp(151)
dp(704) = dp(9)*p(120) + p(9)*dp(120)
dp(705) = dp(9)*p(152) + p(9)*dp(152)
dp(706) = dp(9)*p(121) + p(9)*dp(121)
dp(707) = dp(9)*p(153) + p(9)*dp(153)
dp(708) = dp(9)*p(154) + p(9)*dp(154)
dp(709) = dp(9)*p(155) + p(9)*dp(155)
dp(710) = dp(9)*p(184) + p(9)*dp(184)
dp(711) = dp(24)*p(91) + p(24)*dp(91) - dp(592)
dp(712) = dp(1)*p(416) + p(1)*dp(416) - dp(616) - dp(711)
dp(713) = dp(2)*p(401) + p(2)*dp(401) - dp(613)
dp(714) = dp(1)*p(418) + p(1)*dp(418) - dp(618) - dp(713)
dp(715) = dp(1)*p(419) + p(1)*dp(419) - dp(619) - dp(714)
dp(716) = dp(13)*p(181) + p(13)*dp(181) - dp(701)
dp(717) = dp(2)*p(404) + p(2)*dp(404) - dp(615) - dp(716)
dp(718) = dp(32)*p(91) + p(32)*dp(91) - dp(701)
dp(719) = dp(1)*p(421) + p(1)*dp(421) - dp(619) - dp(717)
dp(720) = dp(41)*p(73) + p(41)*dp(73) - dp(701)
dp(721) = dp(6)*p(279) + p(6)*dp(279) - dp(613)
dp(722) = dp(16)*p(150) + p(16)*dp(150) - dp(701)
dp(723) = dp(1)*p(425) + p(1)*dp(425) - dp(624) - dp(721)
dp(724) = dp(4)*p(285) + p(4)*dp(285) - dp(701)
dp(725) = dp(11)*p(150) + p(11)*dp(150) - dp(592)
dp(726) = dp(14)*p(181) + p(14)*dp(181)
dp(727) = dp(14)*p(182) + p(14)*dp(182)
dp(728) = dp(1)*p(430) + p(1)*dp(430) - dp(632)
dp(729) = dp(21)*p(130) + p(21)*dp(130) - dp(635)
dp(730) = dp(3)*p(295) + p(3)*dp(295) - dp(633) - dp(729)
dp(731) = dp(1)*p(432) + p(1)*dp(432) - dp(633) - dp(730)
dp(732) = dp(39)*p(81) + p(39)*dp(81) - dp(632)
dp(733) = dp(3)*p(296) + p(3)*dp(296) - dp(635)
dp(734) = dp(14)*p(149) + p(14)*dp(149)
dp(735) = dp(14)*p(150) + p(14)*dp(150)
dp(736) = dp(14)*p(119) + p(14)*dp(119)
dp(737) = dp(9)*p(185) + p(9)*dp(185)
dp(738) = dp(9)*p(162) + p(9)*dp(162)
dp(739) = dp(9)*p(134) + p(9)*dp(134)
dp(740) = dp(9)*p(163) + p(9)*dp(163)
dp(741) = dp(9)*p(164) + p(9)*dp(164)
dp(742) = dp(9)*p(186) + p(9)*dp(186)
dp(743) = dp(24)*p(93) + p(24)*dp(93) - dp(593)
dp(744) = dp(18)*p(114) + p(18)*dp(114) - dp(595)
dp(745) = dp(1)*p(443) + p(1)*dp(443) - dp(643) - dp(744)
dp(746) = dp(1)*p(444) + p(1)*dp(444) - dp(644) - dp(745)
dp(747) = dp(13)*p(183) + p(13)*dp(183) - dp(726)
dp(748) = dp(2)*p(421) + p(2)*dp(421) - dp(631) - dp(730)
dp(749) = dp(32)*p(93) + p(32)*dp(93) - dp(726)
dp(750) = dp(1)*p(446) + p(1)*dp(446) - dp(644) - dp(748)
dp(751) = dp(43)*p(73) + p(43)*dp(73) - dp(726)
dp(752) = dp(13)*p(153) + p(13)*dp(153) - dp(595)
dp(753) = dp(18)*p(150) + p(18)*dp(150) - dp(726)
dp(754) = dp(34)*p(73) + p(34)*dp(73) - dp(593)
dp(755) = dp(14)*p(183) + p(14)*dp(183)
dp(756) = dp(14)*p(184) + p(14)*dp(184)
dp(757) = dp(16)*p(130) + p(16)*dp(130) - dp(598)
dp(758) = dp(2)*p(430) + p(2)*dp(430) - dp(636)
dp(759) = dp(1)*p(454) + p(1)*dp(454) - dp(654) - dp(758)
dp(760) = dp(41)*p(81) + p(41)*dp(81) - dp(735)
dp(761) = dp(2)*p(432) + p(2)*dp(432) - dp(638) - dp(760)
dp(762) = dp(16)*p(160) + p(16)*dp(160) - dp(735)
dp(763) = dp(45)*p(69) + p(45)*dp(69) - dp(636)
dp(764) = dp(4)*p(296) + p(4)*dp(296) - dp(735)
dp(765) = dp(11)*p(160) + p(11)*dp(160) - dp(598)
dp(766) = dp(14)*p(156) + p(14)*dp(156)
dp(767) = dp(14)*p(157) + p(14)*dp(157)
dp(768) = dp(14)*p(158) + p(14)*dp(158)
dp(769) = dp(14)*p(128) + p(14)*dp(128)
dp(770) = dp(14)*p(129) + p(14)*dp(129)
dp(771) = dp(1)*p(464) + p(1)*dp(464) - dp(664)
dp(772) = dp(4)*p(262) + p(4)*dp(262) - dp(673) - dp(661)
dp(773) = dp(3)*p(310) + p(3)*dp(310) - dp(664)
dp(774) = dp(14)*p(159) + p(14)*dp(159)
dp(775) = dp(14)*p(160) + p(14)*dp(160)
dp(776) = dp(14)*p(132) + p(14)*dp(132)
dp(777) = dp(14)*p(133) + p(14)*dp(133)
dp(778) = dp(9)*p(188) + p(9)*dp(188)
dp(779) = dp(9)*p(172) + p(9)*dp(172)
dp(780) = dp(9)*p(173) + p(9)*dp(173)
dp(781) = dp(9)*p(189) + p(9)*dp(189)
dp(782) = dp(24)*p(95) + p(24)*dp(95) - dp(600)
dp(783) = dp(22)*p(114) + p(22)*dp(114) - dp(650)
dp(784) = dp(1)*p(471) + p(1)*dp(471) - dp(668) - dp(783)
dp(785) = dp(13)*p(185) + p(13)*dp(185) - dp(755)
dp(786) = dp(2)*p(446) + p(2)*dp(446) - dp(652) - dp(761)
dp(787) = dp(32)*p(95) + p(32)*dp(95) - dp(755)
dp(788) = dp(6)*p(299) + p(6)*dp(299) - dp(650)
dp(789) = dp(46)*p(73) + p(46)*dp(73) - dp(755)
dp(790) = dp(32)*p(84) + p(32)*dp(84) - dp(600)
dp(791) = dp(14)*p(185) + p(14)*dp(185)
dp(792) = dp(14)*p(186) + p(14)*dp(186)
dp(793) = dp(2)*p(453) + p(2)*dp(453) - dp(656) - dp(771)
dp(794) = dp(1)*p(478) + p(1)*dp(478) - dp(673) - dp(793)
dp(795) = dp(43)*p(81) + p(43)*dp(81) - dp(736)
dp(796) = dp(2)*p(456) + p(2)*dp(456) - dp(659) - dp(772)
dp(797) = dp(18)*p(160) + p(18)*dp(160) - dp(736)
dp(798) = dp(1)*p(480) + p(1)*dp(480) - dp(673) - dp(796)
dp(799) = dp(14)*p(165) + p(14)*dp(165)
dp(800) = dp(14)*p(166) + p(14)*dp(166)
dp(801) = dp(14)*p(167) + p(14)*dp(167)
dp(802) = dp(14)*p(138) + p(14)*dp(138)
dp(803) = dp(14)*p(139) + p(14)*dp(139)
dp(804) = dp(2)*p(464) + p(2)*dp(464) - dp(665)
dp(805) = dp(4)*p(310) + p(4)*dp(310) - dp(775)
dp(806) = dp(11)*p(187) + p(11)*dp(187) - dp(665)
dp(807) = dp(14)*p(168) + p(14)*dp(168)
dp(808) = dp(14)*p(169) + p(14)*dp(169)
dp(809) = dp(14)*p(170) + p(14)*dp(170)
dp(810) = dp(9)*p(191) + p(9)*dp(191)
dp(811) = dp(9)*p(192) + p(9)*dp(192)
dp(812) = dp(24)*p(98) + p(24)*dp(98) - dp(603)
dp(813) = dp(1)*p(485) + p(1)*dp(485) - dp(677) - dp(812)
dp(814) = dp(13)*p(188) + p(13)*dp(188) - dp(791)
dp(815) = dp(2)*p(473) + p(2)*dp(473) - dp(672) - dp(796)
dp(816) = dp(32)*p(98) + p(32)*dp(98) - dp(791)
dp(817) = dp(13)*p(173) + p(13)*dp(173) - dp(603)
dp(818) = dp(14)*p(188) + p(14)*dp(188)
dp(819) = dp(14)*p(189) + p(14)*dp(189)
dp(820) = dp(22)*p(130) + p(22)*dp(130) - dp(663)
dp(821) = dp(46)*p(81) + p(46)*dp(81) - dp(769)
dp(822) = dp(45)*p(84) + p(45)*dp(84) - dp(663)
dp(823) = dp(14)*p(174) + p(14)*dp(174)
dp(824) = dp(14)*p(175) + p(14)*dp(175)
dp(825) = dp(14)*p(176) + p(14)*dp(176)
dp(826) = dp(9)*p(195) + p(9)*dp(195)
dp(827) = dp(24)*p(102) + p(24)*dp(102) - dp(679)
dp(828) = dp(13)*p(191) + p(13)*dp(191) - dp(818)
dp(829) = dp(6)*p(323) + p(6)*dp(323) - dp(679)
dp(830) = dp(14)*p(191) + p(14)*dp(191)
dp(831) = dp(14)*p(192) + p(14)*dp(192)
dp(832) = dp(9)*p(140) + p(9)*dp(140)
dp(833) = dp(9)*p(141) + p(9)*dp(141)
dp(834) = dp(9)*p(178) + p(9)*dp(178)
dp(835) = dp(9)*p(142) + p(9)*dp(142)
dp(836) = dp(9)*p(179) + p(9)*dp(179)
dp(837) = dp(9)*p(180) + p(9)*dp(180)
dp(838) = dp(9)*p(194) + p(9)*dp(194)
dp(839) = dp(3)*p(331) + p(3)*dp(331) - dp(680)
dp(840) = dp(39)*p(91) + p(39)*dp(91) - dp(682)
dp(841) = dp(16)*p(178) + p(16)*dp(178) - dp(685)
dp(842) = dp(1)*p(497) + p(1)*dp(497) - dp(683) - dp(841)
dp(843) = dp(2)*p(566) + p(2)*dp(566) - dp(842)
dp(844) = dp(1)*p(498) + p(1)*dp(498) - dp(686) - dp(842)
dp(845) = dp(7)*p(279) + p(7)*dp(279) - dp(685)
dp(846) = dp(1)*p(500) + p(1)*dp(500) - dp(689) - dp(844)
dp(847) = dp(49)*p(69) + p(49)*dp(69) - dp(682)
dp(848) = dp(1)*p(502) + p(1)*dp(502) - dp(691) - dp(846)
dp(849) = dp(11)*p(194) + p(11)*dp(194) - dp(680)
dp(850) = dp(1)*p(504) + p(1)*dp(504) - dp(695) - dp(694)
dp(851) = dp(1)*p(505) + p(1)*dp(505) - dp(700)
dp(852) = dp(3)*p(333) + p(3)*dp(333) - dp(702)
dp(853) = dp(39)*p(93) + p(39)*dp(93) - dp(704)
dp(854) = dp(18)*p(178) + p(18)*dp(178) - dp(707)
dp(855) = dp(1)*p(508) + p(1)*dp(508) - dp(705) - dp(854)
dp(856) = dp(1)*p(509) + p(1)*dp(509) - dp(707)
dp(857) = dp(1)*p(510) + p(1)*dp(510) - dp(708) - dp(855)
dp(858) = dp(21)*p(153) + p(21)*dp(153) - dp(704)
dp(859) = dp(1)*p(512) + p(1)*dp(512) - dp(710) - dp(857)
dp(860) = dp(34)*p(101) + p(34)*dp(101) - dp(702)
dp(861) = dp(6)*p(331) + p(6)*dp(331) - dp(701)
dp(862) = dp(1)*p(515) + p(1)*dp(515) - dp(717) - dp(715)
dp(863) = dp(2)*p(505) + p(2)*dp(505) - dp(701)
dp(864) = dp(1)*p(517) + p(1)*dp(517) - dp(730) - dp(729)
dp(865) = dp(1)*p(518) + p(1)*dp(518) - dp(733)
dp(866) = dp(3)*p(335) + p(3)*dp(335) - dp(737)
dp(867) = dp(39)*p(95) + p(39)*dp(95) - dp(739)
dp(868) = dp(1)*p(520) + p(1)*dp(520) - dp(738) - dp(867)
dp(869) = dp(2)*p(509) + p(2)*dp(509) - dp(721)
dp(870) = dp(1)*p(521) + p(1)*dp(521) - dp(740) - dp(868)
dp(871) = dp(7)*p(299) + p(7)*dp(299) - dp(739)
dp(872) = dp(1)*p(523) + p(1)*dp(523) - dp(742) - dp(870)
dp(873) = dp(49)*p(84) + p(49)*dp(84) - dp(737)
dp(874) = dp(6)*p(333) + p(6)*dp(333) - dp(726)
dp(875) = dp(1)*p(526) + p(1)*dp(526) - dp(748) - dp(746)
dp(876) = dp(8)*p(285) + p(8)*dp(285) - dp(726)
dp(877) = dp(45)*p(91) + p(45)*dp(91) - dp(735)
dp(878) = dp(1)*p(529) + p(1)*dp(529) - dp(761) - dp(759)
dp(879) = dp(2)*p(518) + p(2)*dp(518) - dp(735)
dp(880) = dp(1)*p(531) + p(1)*dp(531) - dp(772) - dp(771)
dp(881) = dp(1)*p(532) + p(1)*dp(532) - dp(773)
dp(882) = dp(14)*p(161) + p(14)*dp(161)
dp(883) = dp(3)*p(337) + p(3)*dp(337) - dp(778)
dp(884) = dp(39)*p(98) + p(39)*dp(98) - dp(780)
dp(885) = dp(1)*p(535) + p(1)*dp(535) - dp(779) - dp(884)
dp(886) = dp(1)*p(536) + p(1)*dp(536) - dp(780)
dp(887) = dp(1)*p(537) + p(1)*dp(537) - dp(781) - dp(885)
dp(888) = dp(21)*p(173) + p(21)*dp(173) - dp(778)
dp(889) = dp(6)*p(335) + p(6)*dp(335) - dp(755)
dp(890) = dp(1)*p(540) + p(1)*dp(540) - dp(786) - dp(784)
dp(891) = dp(22)*p(150) + p(22)*dp(150) - dp(755)
dp(892) = dp(45)*p(93) + p(45)*dp(93) - dp(736)
dp(893) = dp(1)*p(543) + p(1)*dp(543) - dp(796) - dp(794)
dp(894) = dp(8)*p(296) + p(8)*dp(296) - dp(736)
dp(895) = dp(16)*p(187) + p(16)*dp(187) - dp(775)
dp(896) = dp(1)*p(547) + p(1)*dp(547) - dp(806) - dp(804)
dp(897) = dp(2)*p(532) + p(2)*dp(532) - dp(775)
dp(898) = dp(14)*p(171) + p(14)*dp(171)
dp(899) = dp(6)*p(309) + p(6)*dp(309) - dp(808) - dp(807)
dp(900) = dp(1)*p(578) + p(1)*dp(578) - dp(899)
dp(901) = dp(14)*p(187) + p(14)*dp(187)
dp(902) = dp(3)*p(340) + p(3)*dp(340) - dp(810)
dp(903) = dp(2)*p(535) + p(2)*dp(535) - dp(787) - dp(783)
dp(904) = dp(2)*p(536) + p(2)*dp(536) - dp(788)
dp(905) = dp(1)*p(551) + p(1)*dp(551) - dp(811) - dp(903)
dp(906) = dp(7)*p(323) + p(7)*dp(323) - dp(810)
dp(907) = dp(6)*p(337) + p(6)*dp(337) - dp(791)
dp(908) = dp(1)*p(554) + p(1)*dp(554) - dp(815) - dp(813)
dp(909) = dp(50)*p(73) + p(50)*dp(73) - dp(791)
dp(910) = dp(45)*p(95) + p(45)*dp(95) - dp(769)
dp(911) = dp(1)*p(557) + p(1)*dp(557) - dp(822) - dp(820)
dp(912) = dp(22)*p(160) + p(22)*dp(160) - dp(769)
dp(913) = dp(14)*p(177) + p(14)*dp(177)
dp(914) = dp(18)*p(187) + p(18)*dp(187) - dp(777)
dp(915) = dp(2)*p(547) + p(2)*dp(547) - dp(808) - dp(899)
dp(916) = dp(8)*p(310) + p(8)*dp(310) - dp(777)
dp(917) = dp(14)*p(190) + p(14)*dp(190)
dp(918) = dp(3)*p(343) + p(3)*dp(343) - dp(826)
dp(919) = dp(2)*p(551) + p(2)*dp(551) - dp(816) - dp(813)
dp(920) = dp(1)*p(561) + p(1)*dp(561) - dp(826)
dp(921) = dp(6)*p(340) + p(6)*dp(340) - dp(818)
dp(922) = dp(1)*p(563) + p(1)*dp(563) - dp(829) - dp(827)
dp(923) = dp(32)*p(102) + p(32)*dp(102) - dp(818)
dp(924) = dp(45)*p(98) + p(45)*dp(98) - dp(802)
dp(925) = dp(2)*p(557) + p(2)*dp(557) - dp(824) - dp(915)
dp(926) = dp(50)*p(81) + p(50)*dp(81) - dp(802)
dp(927) = dp(14)*p(193) + p(14)*dp(193)
dp(928) = dp(2)*p(560) + p(2)*dp(560) - dp(828) - dp(827)
dp(929) = dp(2)*p(561) + p(2)*dp(561) - dp(829)
dp(930) = dp(6)*p(343) + p(6)*dp(343) - dp(830)
dp(931) = dp(2)*p(563) + p(2)*dp(563) - dp(831) - dp(925)
dp(932) = dp(13)*p(195) + p(13)*dp(195) - dp(830)
dp(933) = dp(14)*p(195) + p(14)*dp(195)
dp(934) = dp(1)*p(566) + p(1)*dp(566) - dp(834)
dp(935) = dp(1)*p(567) + p(1)*dp(567) - dp(836) - dp(934)
dp(936) = dp(39)*p(101) + p(39)*dp(101) - dp(835)
dp(937) = dp(3)*p(346) + p(3)*dp(346) - dp(838)
dp(938) = dp(1)*p(570) + p(1)*dp(570) - dp(839)
dp(939) = dp(2)*p(588) + p(2)*dp(588) - dp(938)
dp(940) = dp(1)*p(572) + p(1)*dp(572) - dp(852)
dp(941) = dp(8)*p(346) + p(8)*dp(346) - dp(940)
dp(942) = dp(1)*p(574) + p(1)*dp(574) - dp(866)
dp(943) = dp(22)*p(194) + p(22)*dp(194) - dp(942)
dp(944) = dp(1)*p(576) + p(1)*dp(576) - dp(883)
dp(945) = dp(50)*p(101) + p(50)*dp(101) - dp(944)
dp(946) = dp(1)*p(579) + p(1)*dp(579) - dp(902)
dp(947) = dp(49)*p(102) + p(49)*dp(102) - dp(946)
dp(948) = dp(2)*p(578) + p(2)*dp(578) - dp(901)
dp(949) = dp(1)*p(582) + p(1)*dp(582) - dp(918)
dp(950) = dp(21)*p(195) + p(21)*dp(195) - dp(949)
dp(951) = dp(2)*p(581) + p(2)*dp(581) - dp(917) - dp(948)
dp(952) = dp(1)*p(585) + p(1)*dp(585) - dp(928)
dp(953) = dp(7)*p(347) + p(7)*dp(347) - dp(952)
dp(954) = dp(45)*p(102) + p(45)*dp(102) - dp(913)
dp(955) = dp(2)*p(585) + p(2)*dp(585) - dp(930)
dp(956) = dp(1)*p(589) + p(1)*dp(589) - dp(955)
dp(957) = dp(6)*p(347) + p(6)*dp(347) - dp(933)
dp(958) = dp(1)*p(588) + p(1)*dp(588) - dp(937)
dp(959) = dp(2)*p(589) + p(2)*dp(589) - dp(957)
dp(960) = dp(9)*p(196) + p(9)*dp(196)
dp(961) = dp(9)*p(202) + p(9)*dp(202)
dp(962) = dp(9)*p(203) + p(9)*dp(203)
dp(963) = dp(9)*p(228) + p(9)*dp(228)
dp(964) = dp(9)*p(242) + p(9)*dp(242)
dp(965) = dp(9)*p(207) + p(9)*dp(207)
dp(966) = dp(9)*p(197) + p(9)*dp(197)
dp(967) = dp(9)*p(208) + p(9)*dp(208)
dp(968) = dp(9)*p(209) + p(9)*dp(209)
dp(969) = dp(9)*p(243) + p(9)*dp(243)
dp(970) = dp(9)*p(211) + p(9)*dp(211)
dp(971) = dp(9)*p(247) + p(9)*dp(247)
dp(972) = dp(9)*p(212) + p(9)*dp(212)
dp(973) = dp(9)*p(248) + p(9)*dp(248)
dp(974) = dp(9)*p(249) + p(9)*dp(249)
dp(975) = dp(9)*p(258) + p(9)*dp(258)
dp(976) = dp(9)*p(214) + p(9)*dp(214)
dp(977) = dp(9)*p(215) + p(9)*dp(215)
dp(978) = dp(9)*p(259) + p(9)*dp(259)
dp(979) = dp(9)*p(263) + p(9)*dp(263)
dp(980) = dp(9)*p(264) + p(9)*dp(264)
dp(981) = dp(9)*p(265) + p(9)*dp(265)
dp(982) = dp(9)*p(308) + p(9)*dp(308)
dp(983) = dp(9)*p(311) + p(9)*dp(311)
dp(984) = dp(9)*p(270) + p(9)*dp(270)
dp(985) = dp(9)*p(271) + p(9)*dp(271)
dp(986) = dp(9)*p(321) + p(9)*dp(321)
dp(987) = dp(9)*p(327) + p(9)*dp(327)
dp(988) = dp(9)*p(222) + p(9)*dp(222)
dp(989) = dp(9)*p(223) + p(9)*dp(223)
dp(990) = dp(9)*p(284) + p(9)*dp(284)
dp(991) = dp(9)*p(198) + p(9)*dp(198)
dp(992) = dp(9)*p(199) + p(9)*dp(199)
dp(993) = dp(9)*p(224) + p(9)*dp(224)
dp(994) = dp(9)*p(200) + p(9)*dp(200)
dp(995) = dp(9)*p(225) + p(9)*dp(225)
dp(996) = dp(9)*p(201) + p(9)*dp(201)
dp(997) = dp(9)*p(226) + p(9)*dp(226)
dp(998) = dp(9)*p(227) + p(9)*dp(227)
dp(999) = dp(9)*p(285) + p(9)*dp(285)
dp(1000) = dp(14)*p(328) + p(14)*dp(328)
dp(1001) = dp(14)*p(329) + p(14)*dp(329)
dp(1002) = dp(14)*p(330) + p(14)*dp(330)
dp(1003) = dp(9)*p(292) + p(9)*dp(292)
dp(1004) = dp(9)*p(233) + p(9)*dp(233)
dp(1005) = dp(9)*p(234) + p(9)*dp(234)
dp(1006) = dp(9)*p(235) + p(9)*dp(235)
dp(1007) = dp(9)*p(293) + p(9)*dp(293)
dp(1008) = dp(9)*p(236) + p(9)*dp(236)
dp(1009) = dp(9)*p(204) + p(9)*dp(204)
dp(1010) = dp(9)*p(205) + p(9)*dp(205)
dp(1011) = dp(9)*p(237) + p(9)*dp(237)
dp(1012) = dp(9)*p(238) + p(9)*dp(238)
dp(1013) = dp(9)*p(206) + p(9)*dp(206)
dp(1014) = dp(9)*p(239) + p(9)*dp(239)
dp(1015) = dp(9)*p(240) + p(9)*dp(240)
dp(1016) = dp(9)*p(241) + p(9)*dp(241)
dp(1017) = dp(9)*p(294) + p(9)*dp(294)
dp(1018) = dp(14)*p(276) + p(14)*dp(276)
dp(1019) = dp(14)*p(277) + p(14)*dp(277)
dp(1020) = dp(14)*p(278) + p(14)*dp(278)
dp(1021) = dp(14)*p(279) + p(14)*dp(279)
dp(1022) = dp(14)*p(280) + p(14)*dp(280)
dp(1023) = dp(14)*p(281) + p(14)*dp(281)
dp(1024) = dp(14)*p(282) + p(14)*dp(282)
dp(1025) = dp(14)*p(283) + p(14)*dp(283)
dp(1026) = dp(9)*p(244) + p(9)*dp(244)
dp(1027) = dp(9)*p(295) + p(9)*dp(295)
dp(1028) = dp(9)*p(210) + p(9)*dp(210)
dp(1029) = dp(9)*p(245) + p(9)*dp(245)
dp(1030) = dp(9)*p(246) + p(9)*dp(246)
dp(1031) = dp(9)*p(296) + p(9)*dp(296)
dp(1032) = dp(14)*p(222) + p(14)*dp(222)
dp(1033) = dp(14)*p(223) + p(14)*dp(223)
dp(1034) = dp(14)*p(224) + p(14)*dp(224)
dp(1035) = dp(14)*p(225) + p(14)*dp(225)
dp(1036) = dp(14)*p(226) + p(14)*dp(226)
dp(1037) = dp(14)*p(227) + p(14)*dp(227)
dp(1038) = dp(14)*p(202) + p(14)*dp(202)
dp(1039) = dp(14)*p(203) + p(14)*dp(203)
dp(1040) = dp(9)*p(302) + p(9)*dp(302)
dp(1041) = dp(9)*p(252) + p(9)*dp(252)
dp(1042) = dp(9)*p(253) + p(9)*dp(253)
dp(1043) = dp(9)*p(303) + p(9)*dp(303)
dp(1044) = dp(9)*p(254) + p(9)*dp(254)
dp(1045) = dp(9)*p(213) + p(9)*dp(213)
dp(1046) = dp(9)*p(255) + p(9)*dp(255)
dp(1047) = dp(9)*p(256) + p(9)*dp(256)
dp(1048) = dp(9)*p(257) + p(9)*dp(257)
dp(1049) = dp(9)*p(304) + p(9)*dp(304)
dp(1050) = dp(14)*p(286) + p(14)*dp(286)
dp(1051) = dp(14)*p(287) + p(14)*dp(287)
dp(1052) = dp(14)*p(288) + p(14)*dp(288)
dp(1053) = dp(14)*p(289) + p(14)*dp(289)
dp(1054) = dp(14)*p(290) + p(14)*dp(290)
dp(1055) = dp(14)*p(291) + p(14)*dp(291)
dp(1056) = dp(9)*p(305) + p(9)*dp(305)
dp(1057) = dp(9)*p(260) + p(9)*dp(260)
dp(1058) = dp(9)*p(306) + p(9)*dp(306)
dp(1059) = dp(9)*p(261) + p(9)*dp(261)
dp(1060) = dp(9)*p(262) + p(9)*dp(262)
dp(1061) = dp(9)*p(307) + p(9)*dp(307)
dp(1062) = dp(14)*p(233) + p(14)*dp(233)
dp(1063) = dp(14)*p(234) + p(14)*dp(234)
dp(1064) = dp(14)*p(235) + p(14)*dp(235)
dp(1065) = dp(14)*p(236) + p(14)*dp(236)
dp(1066) = dp(14)*p(237) + p(14)*dp(237)
dp(1067) = dp(14)*p(238) + p(14)*dp(238)
dp(1068) = dp(14)*p(239) + p(14)*dp(239)
dp(1069) = dp(14)*p(240) + p(14)*dp(240)
dp(1070) = dp(14)*p(241) + p(14)*dp(241)
dp(1071) = dp(14)*p(207) + p(14)*dp(207)
dp(1072) = dp(14)*p(208) + p(14)*dp(208)
dp(1073) = dp(14)*p(209) + p(14)*dp(209)
dp(1074) = dp(9)*p(309) + p(9)*dp(309)
dp(1075) = dp(9)*p(310) + p(9)*dp(310)
dp(1076) = dp(14)*p(244) + p(14)*dp(244)
dp(1077) = dp(14)*p(245) + p(14)*dp(245)
dp(1078) = dp(14)*p(246) + p(14)*dp(246)
dp(1079) = dp(14)*p(211) + p(14)*dp(211)
dp(1080) = dp(14)*p(212) + p(14)*dp(212)
dp(1081) = dp(9)*p(315) + p(9)*dp(315)
dp(1082) = dp(9)*p(267) + p(9)*dp(267)
dp(1083) = dp(9)*p(316) + p(9)*dp(316)
dp(1084) = dp(9)*p(268) + p(9)*dp(268)
dp(1085) = dp(9)*p(269) + p(9)*dp(269)
dp(1086) = dp(9)*p(317) + p(9)*dp(317)
dp(1087) = dp(14)*p(297) + p(14)*dp(297)
dp(1088) = dp(14)*p(298) + p(14)*dp(298)
dp(1089) = dp(14)*p(299) + p(14)*dp(299)
dp(1090) = dp(14)*p(300) + p(14)*dp(300)
dp(1091) = dp(14)*p(301) + p(14)*dp(301)
dp(1092) = dp(9)*p(318) + p(9)*dp(318)
dp(1093) = dp(9)*p(319) + p(9)*dp(319)
dp(1094) = dp(9)*p(320) + p(9)*dp(320)
dp(1095) = dp(14)*p(252) + p(14)*dp(252)
dp(1096) = dp(14)*p(253) + p(14)*dp(253)
dp(1097) = dp(14)*p(254) + p(14)*dp(254)
dp(1098) = dp(14)*p(255) + p(14)*dp(255)
dp(1099) = dp(14)*p(256) + p(14)*dp(256)
dp(1100) = dp(14)*p(257) + p(14)*dp(257)
dp(1101) = dp(14)*p(214) + p(14)*dp(214)
dp(1102) = dp(14)*p(215) + p(14)*dp(215)
dp(1103) = dp(9)*p(339) + p(9)*dp(339)
dp(1104) = dp(14)*p(260) + p(14)*dp(260)
dp(1105) = dp(14)*p(261) + p(14)*dp(261)
dp(1106) = dp(14)*p(262) + p(14)*dp(262)
dp(1107) = dp(9)*p(324) + p(9)*dp(324)
dp(1108) = dp(9)*p(325) + p(9)*dp(325)
dp(1109) = dp(9)*p(326) + p(9)*dp(326)
dp(1110) = dp(14)*p(312) + p(14)*dp(312)
dp(1111) = dp(14)*p(313) + p(14)*dp(313)
dp(1112) = dp(14)*p(314) + p(14)*dp(314)
dp(1113) = dp(9)*p(342) + p(9)*dp(342)
dp(1114) = dp(14)*p(267) + p(14)*dp(267)
dp(1115) = dp(14)*p(268) + p(14)*dp(268)
dp(1116) = dp(14)*p(269) + p(14)*dp(269)
dp(1117) = dp(9)*p(345) + p(9)*dp(345)
dp(1118) = dp(14)*p(322) + p(14)*dp(322)
dp(1119) = dp(14)*p(323) + p(14)*dp(323)
dp(1120) = dp(9)*p(331) + p(9)*dp(331)
dp(1121) = dp(9)*p(276) + p(9)*dp(276)
dp(1122) = dp(9)*p(216) + p(9)*dp(216)
dp(1123) = dp(9)*p(277) + p(9)*dp(277)
dp(1124) = dp(9)*p(217) + p(9)*dp(217)
dp(1125) = dp(9)*p(218) + p(9)*dp(218)
dp(1126) = dp(9)*p(278) + p(9)*dp(278)
dp(1127) = dp(9)*p(219) + p(9)*dp(219)
dp(1128) = dp(9)*p(220) + p(9)*dp(220)
dp(1129) = dp(9)*p(279) + p(9)*dp(279)
dp(1130) = dp(9)*p(280) + p(9)*dp(280)
dp(1131) = dp(9)*p(221) + p(9)*dp(221)
dp(1132) = dp(9)*p(281) + p(9)*dp(281)
dp(1133) = dp(9)*p(282) + p(9)*dp(282)
dp(1134) = dp(9)*p(283) + p(9)*dp(283)
dp(1135) = dp(9)*p(332) + p(9)*dp(332)
dp(1136) = dp(3)*p(401) + p(3)*dp(401) - dp(988)
dp(1137) = dp(7)*p(401) + p(7)*dp(401) - dp(991)
dp(1138) = dp(49)*p(114) + p(49)*dp(114) - dp(996)
dp(1139) = dp(24)*p(194) + p(24)*dp(194) - dp(999)
dp(1140) = dp(3)*p(504) + p(3)*dp(504) - dp(990) - dp(1139)
dp(1141) = dp(1)*p(695) + p(1)*dp(695) - dp(990) - dp(1140)
dp(1142) = dp(1)*p(696) + p(1)*dp(696) - dp(993) - dp(1141)
dp(1143) = dp(6)*p(566) + p(6)*dp(566) - dp(1136)
dp(1144) = dp(32)*p(178) + p(32)*dp(178) - dp(991)
dp(1145) = dp(39)*p(150) + p(39)*dp(150) - dp(996)
dp(1146) = dp(3)*p(505) + p(3)*dp(505) - dp(999)
dp(1147) = dp(14)*p(346) + p(14)*dp(346)
dp(1148) = dp(9)*p(333) + p(9)*dp(333)
dp(1149) = dp(9)*p(286) + p(9)*dp(286)
dp(1150) = dp(9)*p(229) + p(9)*dp(229)
dp(1151) = dp(9)*p(287) + p(9)*dp(287)
dp(1152) = dp(9)*p(230) + p(9)*dp(230)
dp(1153) = dp(9)*p(231) + p(9)*dp(231)
dp(1154) = dp(9)*p(288) + p(9)*dp(288)
dp(1155) = dp(9)*p(232) + p(9)*dp(232)
dp(1156) = dp(9)*p(289) + p(9)*dp(289)
dp(1157) = dp(9)*p(290) + p(9)*dp(290)
dp(1158) = dp(9)*p(291) + p(9)*dp(291)
dp(1159) = dp(9)*p(334) + p(9)*dp(334)
dp(1160) = dp(24)*p(181) + p(24)*dp(181) - dp(963)
dp(1161) = dp(41)*p(114) + p(41)*dp(114) - dp(960)
dp(1162) = dp(4)*p(401) + p(4)*dp(401) - dp(961)
dp(1163) = dp(1)*p(713) + p(1)*dp(713) - dp(1005) - dp(1162)
dp(1164) = dp(1)*p(714) + p(1)*dp(714) - dp(1006) - dp(1163)
dp(1165) = dp(1)*p(715) + p(1)*dp(715) - dp(1007) - dp(1164)
dp(1166) = dp(13)*p(331) + p(13)*dp(331) - dp(1147)
dp(1167) = dp(2)*p(695) + p(2)*dp(695) - dp(1002) - dp(1166)
dp(1168) = dp(32)*p(181) + p(32)*dp(181) - dp(1147)
dp(1169) = dp(1)*p(717) + p(1)*dp(717) - dp(1007) - dp(1167)
dp(1170) = dp(73)*p(91) + p(73)*dp(91) - dp(1147)
dp(1171) = dp(1)*p(719) + p(1)*dp(719) - dp(1011) - dp(1169)
dp(1172) = dp(41)*p(150) + p(41)*dp(150) - dp(1147)
dp(1173) = dp(13)*p(279) + p(13)*dp(279) - dp(961)
dp(1174) = dp(16)*p(285) + p(16)*dp(285) - dp(1147)
dp(1175) = dp(69)*p(73) + p(69)*dp(73) - dp(960)
dp(1176) = dp(4)*p(505) + p(4)*dp(505) - dp(1147)
dp(1177) = dp(11)*p(285) + p(11)*dp(285) - dp(963)
dp(1178) = dp(14)*p(331) + p(14)*dp(331)
dp(1179) = dp(14)*p(332) + p(14)*dp(332)
dp(1180) = dp(3)*p(430) + p(3)*dp(430) - dp(1026)
dp(1181) = dp(7)*p(430) + p(7)*dp(430) - dp(1028)
dp(1182) = dp(49)*p(130) + p(49)*dp(130) - dp(1031)
dp(1183) = dp(3)*p(517) + p(3)*dp(517) - dp(1027) - dp(1182)
dp(1184) = dp(1)*p(730) + p(1)*dp(730) - dp(1027) - dp(1183)
dp(1185) = dp(45)*p(178) + p(45)*dp(178) - dp(1180)
dp(1186) = dp(39)*p(160) + p(39)*dp(160) - dp(1028)
dp(1187) = dp(3)*p(518) + p(3)*dp(518) - dp(1031)
dp(1188) = dp(14)*p(284) + p(14)*dp(284)
dp(1189) = dp(14)*p(285) + p(14)*dp(285)
dp(1190) = dp(14)*p(228) + p(14)*dp(228)
dp(1191) = dp(9)*p(335) + p(9)*dp(335)
dp(1192) = dp(9)*p(297) + p(9)*dp(297)
dp(1193) = dp(9)*p(250) + p(9)*dp(250)
dp(1194) = dp(9)*p(298) + p(9)*dp(298)
dp(1195) = dp(9)*p(251) + p(9)*dp(251)
dp(1196) = dp(9)*p(299) + p(9)*dp(299)
dp(1197) = dp(9)*p(300) + p(9)*dp(300)
dp(1198) = dp(9)*p(301) + p(9)*dp(301)
dp(1199) = dp(9)*p(336) + p(9)*dp(336)
dp(1200) = dp(24)*p(183) + p(24)*dp(183) - dp(964)
dp(1201) = dp(43)*p(114) + p(43)*dp(114) - dp(966)
dp(1202) = dp(8)*p(401) + p(8)*dp(401) - dp(1021)
dp(1203) = dp(1)*p(745) + p(1)*dp(745) - dp(1042) - dp(1202)
dp(1204) = dp(1)*p(746) + p(1)*dp(746) - dp(1043) - dp(1203)
dp(1205) = dp(13)*p(333) + p(13)*dp(333) - dp(1178)
dp(1206) = dp(2)*p(717) + p(2)*dp(717) - dp(1025) - dp(1183)
dp(1207) = dp(32)*p(183) + p(32)*dp(183) - dp(1178)
dp(1208) = dp(1)*p(748) + p(1)*dp(748) - dp(1043) - dp(1206)
dp(1209) = dp(73)*p(93) + p(73)*dp(93) - dp(1178)
dp(1210) = dp(6)*p(509) + p(6)*dp(509) - dp(1021)
dp(1211) = dp(43)*p(150) + p(43)*dp(150) - dp(1178)
dp(1212) = dp(32)*p(153) + p(32)*dp(153) - dp(966)
dp(1213) = dp(18)*p(285) + p(18)*dp(285) - dp(1178)
dp(1214) = dp(34)*p(150) + p(34)*dp(150) - dp(964)
dp(1215) = dp(14)*p(333) + p(14)*dp(333)
dp(1216) = dp(14)*p(334) + p(14)*dp(334)
dp(1217) = dp(41)*p(130) + p(41)*dp(130) - dp(973)
dp(1218) = dp(4)*p(430) + p(4)*dp(430) - dp(970)
dp(1219) = dp(1)*p(758) + p(1)*dp(758) - dp(1057) - dp(1218)
dp(1220) = dp(1)*p(759) + p(1)*dp(759) - dp(1058) - dp(1219)
dp(1221) = dp(81)*p(91) + p(81)*dp(91) - dp(1189)
dp(1222) = dp(2)*p(730) + p(2)*dp(730) - dp(1034) - dp(1221)
dp(1223) = dp(41)*p(160) + p(41)*dp(160) - dp(1189)
dp(1224) = dp(1)*p(761) + p(1)*dp(761) - dp(1058) - dp(1222)
dp(1225) = dp(16)*p(296) + p(16)*dp(296) - dp(1189)
dp(1226) = dp(69)*p(81) + p(69)*dp(81) - dp(970)
dp(1227) = dp(4)*p(518) + p(4)*dp(518) - dp(1189)
dp(1228) = dp(11)*p(296) + p(11)*dp(296) - dp(973)
dp(1229) = dp(14)*p(292) + p(14)*dp(292)
dp(1230) = dp(14)*p(293) + p(14)*dp(293)
dp(1231) = dp(14)*p(294) + p(14)*dp(294)
dp(1232) = dp(14)*p(242) + p(14)*dp(242)
dp(1233) = dp(14)*p(243) + p(14)*dp(243)
dp(1234) = dp(3)*p(464) + p(3)*dp(464) - dp(1074)
dp(1235) = dp(7)*p(464) + p(7)*dp(464) - dp(1075)
dp(1236) = dp(3)*p(531) + p(3)*dp(531) - dp(1074) - dp(1235)
dp(1237) = dp(39)*p(187) + p(39)*dp(187) - dp(1234)
dp(1238) = dp(3)*p(532) + p(3)*dp(532) - dp(1075)
dp(1239) = dp(14)*p(295) + p(14)*dp(295)
dp(1240) = dp(14)*p(296) + p(14)*dp(296)
dp(1241) = dp(14)*p(247) + p(14)*dp(247)
dp(1242) = dp(14)*p(248) + p(14)*dp(248)
dp(1243) = dp(14)*p(249) + p(14)*dp(249)
dp(1244) = dp(9)*p(337) + p(9)*dp(337)
dp(1245) = dp(9)*p(312) + p(9)*dp(312)
dp(1246) = dp(9)*p(266) + p(9)*dp(266)
dp(1247) = dp(9)*p(313) + p(9)*dp(313)
dp(1248) = dp(9)*p(314) + p(9)*dp(314)
dp(1249) = dp(9)*p(338) + p(9)*dp(338)
dp(1250) = dp(24)*p(185) + p(24)*dp(185) - dp(975)
dp(1251) = dp(46)*p(114) + p(46)*dp(114) - dp(977)
dp(1252) = dp(1)*p(783) + p(1)*dp(783) - dp(1082) - dp(1251)
dp(1253) = dp(1)*p(784) + p(1)*dp(784) - dp(1083) - dp(1252)
dp(1254) = dp(13)*p(335) + p(13)*dp(335) - dp(1215)
dp(1255) = dp(2)*p(748) + p(2)*dp(748) - dp(1055) - dp(1222)
dp(1256) = dp(32)*p(185) + p(32)*dp(185) - dp(1215)
dp(1257) = dp(1)*p(786) + p(1)*dp(786) - dp(1083) - dp(1255)
dp(1258) = dp(73)*p(95) + p(73)*dp(95) - dp(1215)
dp(1259) = dp(13)*p(299) + p(13)*dp(299) - dp(977)
dp(1260) = dp(46)*p(150) + p(46)*dp(150) - dp(1215)
dp(1261) = dp(73)*p(84) + p(73)*dp(84) - dp(975)
dp(1262) = dp(14)*p(335) + p(14)*dp(335)
dp(1263) = dp(14)*p(336) + p(14)*dp(336)
dp(1264) = dp(43)*p(130) + p(43)*dp(130) - dp(974)
dp(1265) = dp(8)*p(430) + p(8)*dp(430) - dp(1038)
dp(1266) = dp(1)*p(794) + p(1)*dp(794) - dp(1093) - dp(1265)
dp(1267) = dp(81)*p(93) + p(81)*dp(93) - dp(1190)
dp(1268) = dp(2)*p(761) + p(2)*dp(761) - dp(1066) - dp(1236)
dp(1269) = dp(43)*p(160) + p(43)*dp(160) - dp(1190)
dp(1270) = dp(45)*p(153) + p(45)*dp(153) - dp(1038)
dp(1271) = dp(18)*p(296) + p(18)*dp(296) - dp(1190)
dp(1272) = dp(34)*p(160) + p(34)*dp(160) - dp(974)
dp(1273) = dp(14)*p(302) + p(14)*dp(302)
dp(1274) = dp(14)*p(303) + p(14)*dp(303)
dp(1275) = dp(14)*p(304) + p(14)*dp(304)
dp(1276) = dp(14)*p(258) + p(14)*dp(258)
dp(1277) = dp(14)*p(259) + p(14)*dp(259)
dp(1278) = dp(4)*p(464) + p(4)*dp(464) - dp(983)
dp(1279) = dp(1)*p(804) + p(1)*dp(804) - dp(1103) - dp(1278)
dp(1280) = dp(16)*p(310) + p(16)*dp(310) - dp(1240)
dp(1281) = dp(2)*p(772) + p(2)*dp(772) - dp(1077) - dp(1280)
dp(1282) = dp(4)*p(532) + p(4)*dp(532) - dp(1240)
dp(1283) = dp(11)*p(310) + p(11)*dp(310) - dp(983)
dp(1284) = dp(14)*p(305) + p(14)*dp(305)
dp(1285) = dp(14)*p(306) + p(14)*dp(306)
dp(1286) = dp(14)*p(307) + p(14)*dp(307)
dp(1287) = dp(14)*p(263) + p(14)*dp(263)
dp(1288) = dp(14)*p(264) + p(14)*dp(264)
dp(1289) = dp(14)*p(265) + p(14)*dp(265)
dp(1290) = dp(6)*p(464) + p(6)*dp(464) - dp(1104)
dp(1291) = dp(3)*p(578) + p(3)*dp(578) - dp(1290)
dp(1292) = dp(14)*p(309) + p(14)*dp(309)
dp(1293) = dp(14)*p(310) + p(14)*dp(310)
dp(1294) = dp(9)*p(340) + p(9)*dp(340)
dp(1295) = dp(9)*p(322) + p(9)*dp(322)
dp(1296) = dp(9)*p(323) + p(9)*dp(323)
dp(1297) = dp(9)*p(341) + p(9)*dp(341)
dp(1298) = dp(24)*p(188) + p(24)*dp(188) - dp(984)
dp(1299) = dp(50)*p(114) + p(50)*dp(114) - dp(1089)
dp(1300) = dp(1)*p(813) + p(1)*dp(813) - dp(1108) - dp(1299)
dp(1301) = dp(13)*p(337) + p(13)*dp(337) - dp(1262)
dp(1302) = dp(2)*p(786) + p(2)*dp(786) - dp(1091) - dp(1268)
dp(1303) = dp(32)*p(188) + p(32)*dp(188) - dp(1262)
dp(1304) = dp(6)*p(536) + p(6)*dp(536) - dp(1089)
dp(1305) = dp(73)*p(98) + p(73)*dp(98) - dp(1262)
dp(1306) = dp(32)*p(173) + p(32)*dp(173) - dp(984)
dp(1307) = dp(14)*p(337) + p(14)*dp(337)
dp(1308) = dp(14)*p(338) + p(14)*dp(338)
dp(1309) = dp(46)*p(130) + p(46)*dp(130) - dp(982)
dp(1310) = dp(1)*p(820) + p(1)*dp(820) - dp(1113) - dp(1309)
dp(1311) = dp(81)*p(95) + p(81)*dp(95) - dp(1232)
dp(1312) = dp(2)*p(796) + p(2)*dp(796) - dp(1098) - dp(1281)
dp(1313) = dp(46)*p(160) + p(46)*dp(160) - dp(1232)
dp(1314) = dp(81)*p(84) + p(81)*dp(84) - dp(982)
dp(1315) = dp(14)*p(315) + p(14)*dp(315)
dp(1316) = dp(14)*p(316) + p(14)*dp(316)
dp(1317) = dp(14)*p(317) + p(14)*dp(317)
dp(1318) = dp(14)*p(270) + p(14)*dp(270)
dp(1319) = dp(14)*p(271) + p(14)*dp(271)
dp(1320) = dp(8)*p(464) + p(8)*dp(464) - dp(1079)
dp(1321) = dp(18)*p(310) + p(18)*dp(310) - dp(1242)
dp(1322) = dp(34)*p(187) + p(34)*dp(187) - dp(1079)
dp(1323) = dp(14)*p(318) + p(14)*dp(318)
dp(1324) = dp(14)*p(319) + p(14)*dp(319)
dp(1325) = dp(14)*p(320) + p(14)*dp(320)
dp(1326) = dp(9)*p(343) + p(9)*dp(343)
dp(1327) = dp(9)*p(344) + p(9)*dp(344)
dp(1328) = dp(24)*p(191) + p(24)*dp(191) - dp(987)
dp(1329) = dp(1)*p(827) + p(1)*dp(827) - dp(1117) - dp(1328)
dp(1330) = dp(13)*p(340) + p(13)*dp(340) - dp(1307)
dp(1331) = dp(2)*p(815) + p(2)*dp(815) - dp(1112) - dp(1312)
dp(1332) = dp(32)*p(191) + p(32)*dp(191) - dp(1307)
dp(1333) = dp(13)*p(323) + p(13)*dp(323) - dp(987)
dp(1334) = dp(14)*p(340) + p(14)*dp(340)
dp(1335) = dp(14)*p(341) + p(14)*dp(341)
dp(1336) = dp(50)*p(130) + p(50)*dp(130) - dp(1102)
dp(1337) = dp(81)*p(98) + p(81)*dp(98) - dp(1276)
dp(1338) = dp(45)*p(173) + p(45)*dp(173) - dp(1102)
dp(1339) = dp(14)*p(324) + p(14)*dp(324)
dp(1340) = dp(14)*p(325) + p(14)*dp(325)
dp(1341) = dp(14)*p(326) + p(14)*dp(326)
dp(1342) = dp(9)*p(347) + p(9)*dp(347)
dp(1343) = dp(24)*p(195) + p(24)*dp(195) - dp(1119)
dp(1344) = dp(13)*p(343) + p(13)*dp(343) - dp(1334)
dp(1345) = dp(6)*p(561) + p(6)*dp(561) - dp(1119)
dp(1346) = dp(14)*p(343) + p(14)*dp(343)
dp(1347) = dp(14)*p(344) + p(14)*dp(344)
dp(1348) = dp(9)*p(272) + p(9)*dp(272)
dp(1349) = dp(9)*p(273) + p(9)*dp(273)
dp(1350) = dp(9)*p(274) + p(9)*dp(274)
dp(1351) = dp(9)*p(328) + p(9)*dp(328)
dp(1352) = dp(9)*p(275) + p(9)*dp(275)
dp(1353) = dp(9)*p(329) + p(9)*dp(329)
dp(1354) = dp(9)*p(330) + p(9)*dp(330)
dp(1355) = dp(9)*p(346) + p(9)*dp(346)
dp(1356) = dp(3)*p(570) + p(3)*dp(570) - dp(1120)
dp(1357) = dp(39)*p(181) + p(39)*dp(181) - dp(1122)
dp(1358) = dp(41)*p(178) + p(41)*dp(178) - dp(1125)
dp(1359) = dp(4)*p(566) + p(4)*dp(566) - dp(1129)
dp(1360) = dp(1)*p(842) + p(1)*dp(842) - dp(1126) - dp(1359)
dp(1361) = dp(1)*p(843) + p(1)*dp(843) - dp(1129)
dp(1362) = dp(1)*p(844) + p(1)*dp(844) - dp(1130) - dp(1360)
dp(1363) = dp(21)*p(279) + p(21)*dp(279) - dp(1125)
dp(1364) = dp(1)*p(846) + p(1)*dp(846) - dp(1133) - dp(1362)
dp(1365) = dp(69)*p(101) + p(69)*dp(101) - dp(1122)
dp(1366) = dp(1)*p(848) + p(1)*dp(848) - dp(1135) - dp(1364)
dp(1367) = dp(11)*p(346) + p(11)*dp(346) - dp(1120)
dp(1368) = dp(1)*p(850) + p(1)*dp(850) - dp(1140) - dp(1139)
dp(1369) = dp(1)*p(851) + p(1)*dp(851) - dp(1146)
dp(1370) = dp(3)*p(572) + p(3)*dp(572) - dp(1148)
dp(1371) = dp(39)*p(183) + p(39)*dp(183) - dp(1150)
dp(1372) = dp(43)*p(178) + p(43)*dp(178) - dp(1153)
dp(1373) = dp(1)*p(854) + p(1)*dp(854) - dp(1151) - dp(1372)
dp(1374) = dp(2)*p(843) + p(2)*dp(843) - dp(1143)
dp(1375) = dp(1)*p(855) + p(1)*dp(855) - dp(1154) - dp(1373)
dp(1376) = dp(7)*p(509) + p(7)*dp(509) - dp(1153)
dp(1377) = dp(1)*p(857) + p(1)*dp(857) - dp(1157) - dp(1375)
dp(1378) = dp(49)*p(153) + p(49)*dp(153) - dp(1150)
dp(1379) = dp(1)*p(859) + p(1)*dp(859) - dp(1159) - dp(1377)
dp(1380) = dp(34)*p(194) + p(34)*dp(194) - dp(1148)
dp(1381) = dp(6)*p(570) + p(6)*dp(570) - dp(1147)
dp(1382) = dp(1)*p(862) + p(1)*dp(862) - dp(1167) - dp(1165)
dp(1383) = dp(2)*p(851) + p(2)*dp(851) - dp(1147)
dp(1384) = dp(1)*p(864) + p(1)*dp(864) - dp(1183) - dp(1182)
dp(1385) = dp(1)*p(865) + p(1)*dp(865) - dp(1187)
dp(1386) = dp(3)*p(574) + p(3)*dp(574) - dp(1191)
dp(1387) = dp(39)*p(185) + p(39)*dp(185) - dp(1193)
dp(1388) = dp(46)*p(178) + p(46)*dp(178) - dp(1196)
dp(1389) = dp(1)*p(868) + p(1)*dp(868) - dp(1194) - dp(1388)
dp(1390) = dp(1)*p(869) + p(1)*dp(869) - dp(1196)
dp(1391) = dp(1)*p(870) + p(1)*dp(870) - dp(1197) - dp(1389)
dp(1392) = dp(21)*p(299) + p(21)*dp(299) - dp(1193)
dp(1393) = dp(1)*p(872) + p(1)*dp(872) - dp(1199) - dp(1391)
dp(1394) = dp(84)*p(101) + p(84)*dp(101) - dp(1191)
dp(1395) = dp(6)*p(572) + p(6)*dp(572) - dp(1178)
dp(1396) = dp(1)*p(875) + p(1)*dp(875) - dp(1206) - dp(1204)
dp(1397) = dp(8)*p(505) + p(8)*dp(505) - dp(1178)
dp(1398) = dp(45)*p(181) + p(45)*dp(181) - dp(1189)
dp(1399) = dp(1)*p(878) + p(1)*dp(878) - dp(1222) - dp(1220)
dp(1400) = dp(2)*p(865) + p(2)*dp(865) - dp(1189)
dp(1401) = dp(1)*p(880) + p(1)*dp(880) - dp(1236) - dp(1235)
dp(1402) = dp(1)*p(881) + p(1)*dp(881) - dp(1238)
dp(1403) = dp(3)*p(576) + p(3)*dp(576) - dp(1244)
dp(1404) = dp(39)*p(188) + p(39)*dp(188) - dp(1246)
dp(1405) = dp(1)*p(884) + p(1)*dp(884) - dp(1245) - dp(1404)
dp(1406) = dp(2)*p(869) + p(2)*dp(869) - dp(1210)
dp(1407) = dp(1)*p(885) + p(1)*dp(885) - dp(1247) - dp(1405)
dp(1408) = dp(7)*p(536) + p(7)*dp(536) - dp(1246)
dp(1409) = dp(1)*p(887) + p(1)*dp(887) - dp(1249) - dp(1407)
dp(1410) = dp(49)*p(173) + p(49)*dp(173) - dp(1244)
dp(1411) = dp(6)*p(574) + p(6)*dp(574) - dp(1215)
dp(1412) = dp(1)*p(890) + p(1)*dp(890) - dp(1255) - dp(1253)
dp(1413) = dp(22)*p(285) + p(22)*dp(285) - dp(1215)
dp(1414) = dp(45)*p(183) + p(45)*dp(183) - dp(1190)
dp(1415) = dp(1)*p(893) + p(1)*dp(893) - dp(1268) - dp(1266)
dp(1416) = dp(8)*p(518) + p(8)*dp(518) - dp(1190)
dp(1417) = dp(41)*p(187) + p(41)*dp(187) - dp(1240)
dp(1418) = dp(1)*p(896) + p(1)*dp(896) - dp(1281) - dp(1279)
dp(1419) = dp(2)*p(881) + p(2)*dp(881) - dp(1240)
dp(1420) = dp(14)*p(308) + p(14)*dp(308)
dp(1421) = dp(4)*p(547) + p(4)*dp(547) - dp(1106) - dp(1320)
dp(1422) = dp(1)*p(900) + p(1)*dp(900) - dp(1291)
dp(1423) = dp(14)*p(311) + p(14)*dp(311)
dp(1424) = dp(3)*p(579) + p(3)*dp(579) - dp(1294)
dp(1425) = dp(39)*p(191) + p(39)*dp(191) - dp(1296)
dp(1426) = dp(1)*p(903) + p(1)*dp(903) - dp(1295) - dp(1425)
dp(1427) = dp(1)*p(904) + p(1)*dp(904) - dp(1296)
dp(1428) = dp(1)*p(905) + p(1)*dp(905) - dp(1297) - dp(1426)
dp(1429) = dp(21)*p(323) + p(21)*dp(323) - dp(1294)
dp(1430) = dp(6)*p(576) + p(6)*dp(576) - dp(1262)
dp(1431) = dp(1)*p(908) + p(1)*dp(908) - dp(1302) - dp(1300)
dp(1432) = dp(50)*p(150) + p(50)*dp(150) - dp(1262)
dp(1433) = dp(45)*p(185) + p(45)*dp(185) - dp(1232)
dp(1434) = dp(1)*p(911) + p(1)*dp(911) - dp(1312) - dp(1310)
dp(1435) = dp(22)*p(296) + p(22)*dp(296) - dp(1232)
dp(1436) = dp(43)*p(187) + p(43)*dp(187) - dp(1242)
dp(1437) = dp(1)*p(915) + p(1)*dp(915) - dp(1322) - dp(1320)
dp(1438) = dp(8)*p(532) + p(8)*dp(532) - dp(1242)
dp(1439) = dp(14)*p(321) + p(14)*dp(321)
dp(1440) = dp(4)*p(578) + p(4)*dp(578) - dp(1293)
dp(1441) = dp(2)*p(899) + p(2)*dp(899) - dp(1292) - dp(1440)
dp(1442) = dp(2)*p(900) + p(2)*dp(900) - dp(1293)
dp(1443) = dp(14)*p(339) + p(14)*dp(339)
dp(1444) = dp(3)*p(582) + p(3)*dp(582) - dp(1326)
dp(1445) = dp(2)*p(903) + p(2)*dp(903) - dp(1303) - dp(1299)
dp(1446) = dp(2)*p(904) + p(2)*dp(904) - dp(1304)
dp(1447) = dp(1)*p(919) + p(1)*dp(919) - dp(1327) - dp(1445)
dp(1448) = dp(7)*p(561) + p(7)*dp(561) - dp(1326)
dp(1449) = dp(6)*p(579) + p(6)*dp(579) - dp(1307)
dp(1450) = dp(1)*p(922) + p(1)*dp(922) - dp(1331) - dp(1329)
dp(1451) = dp(73)*p(102) + p(73)*dp(102) - dp(1307)
dp(1452) = dp(45)*p(188) + p(45)*dp(188) - dp(1276)
dp(1453) = dp(1)*p(925) + p(1)*dp(925) - dp(1338) - dp(1336)
dp(1454) = dp(50)*p(160) + p(50)*dp(160) - dp(1276)
dp(1455) = dp(14)*p(327) + p(14)*dp(327)
dp(1456) = dp(46)*p(187) + p(46)*dp(187) - dp(1243)
dp(1457) = dp(2)*p(915) + p(2)*dp(915) - dp(1324) - dp(1441)
dp(1458) = dp(22)*p(310) + p(22)*dp(310) - dp(1243)
dp(1459) = dp(14)*p(342) + p(14)*dp(342)
dp(1460) = dp(3)*p(585) + p(3)*dp(585) - dp(1342)
dp(1461) = dp(2)*p(919) + p(2)*dp(919) - dp(1332) - dp(1329)
dp(1462) = dp(1)*p(929) + p(1)*dp(929) - dp(1342)
dp(1463) = dp(6)*p(582) + p(6)*dp(582) - dp(1334)
dp(1464) = dp(1)*p(931) + p(1)*dp(931) - dp(1345) - dp(1343)
dp(1465) = dp(32)*p(195) + p(32)*dp(195) - dp(1334)
dp(1466) = dp(45)*p(191) + p(45)*dp(191) - dp(1318)
dp(1467) = dp(2)*p(925) + p(2)*dp(925) - dp(1340) - dp(1457)
dp(1468) = dp(81)*p(102) + p(81)*dp(102) - dp(1318)
dp(1469) = dp(14)*p(345) + p(14)*dp(345)
dp(1470) = dp(2)*p(928) + p(2)*dp(928) - dp(1344) - dp(1343)
dp(1471) = dp(2)*p(929) + p(2)*dp(929) - dp(1345)
dp(1472) = dp(6)*p(585) + p(6)*dp(585) - dp(1346)
dp(1473) = dp(2)*p(931) + p(2)*dp(931) - dp(1347) - dp(1467)
dp(1474) = dp(13)*p(347) + p(13)*dp(347) - dp(1346)
dp(1475) = dp(14)*p(347) + p(14)*dp(347)
dp(1476) = dp(3)*p(566) + p(3)*dp(566) - dp(1351)
dp(1477) = dp(7)*p(566) + p(7)*dp(566) - dp(1349)
dp(1478) = dp(49)*p(178) + p(49)*dp(178) - dp(1348)
dp(1479) = dp(39)*p(194) + p(39)*dp(194) - dp(1352)
dp(1480) = dp(3)*p(588) + p(3)*dp(588) - dp(1355)
dp(1481) = dp(1)*p(938) + p(1)*dp(938) - dp(1356)
dp(1482) = dp(2)*p(958) + p(2)*dp(958) - dp(1481)
dp(1483) = dp(1)*p(940) + p(1)*dp(940) - dp(1370)
dp(1484) = dp(8)*p(588) + p(8)*dp(588) - dp(1483)
dp(1485) = dp(1)*p(942) + p(1)*dp(942) - dp(1386)
dp(1486) = dp(22)*p(346) + p(22)*dp(346) - dp(1485)
dp(1487) = dp(1)*p(944) + p(1)*dp(944) - dp(1403)
dp(1488) = dp(50)*p(194) + p(50)*dp(194) - dp(1487)
dp(1489) = dp(1)*p(946) + p(1)*dp(946) - dp(1424)
dp(1490) = dp(101)*p(102) + p(101)*dp(102) - dp(1489)
dp(1491) = dp(6)*p(578) + p(6)*dp(578) - dp(1443)
dp(1492) = dp(1)*p(949) + p(1)*dp(949) - dp(1444)
dp(1493) = dp(49)*p(195) + p(49)*dp(195) - dp(1492)
dp(1494) = dp(8)*p(578) + p(8)*dp(578) - dp(1423)
dp(1495) = dp(1)*p(952) + p(1)*dp(952) - dp(1460)
dp(1496) = dp(21)*p(347) + p(21)*dp(347) - dp(1495)
dp(1497) = dp(50)*p(187) + p(50)*dp(187) - dp(1420)
dp(1498) = dp(1)*p(955) + p(1)*dp(955) - dp(1470)
dp(1499) = dp(7)*p(589) + p(7)*dp(589) - dp(1498)
dp(1500) = dp(45)*p(195) + p(45)*dp(195) - dp(1455)
dp(1501) = dp(2)*p(955) + p(2)*dp(955) - dp(1472)
dp(1502) = dp(1)*p(959) + p(1)*dp(959) - dp(1501)
dp(1503) = dp(6)*p(589) + p(6)*dp(589) - dp(1475)
dp(1504) = dp(1)*p(958) + p(1)*dp(958) - dp(1480)
dp(1505) = dp(2)*p(959) + p(2)*dp(959) - dp(1503)
dp(1506) = dp(9)*p(348) + p(9)*dp(348)
dp(1507) = dp(9)*p(349) + p(9)*dp(349)
dp(1508) = dp(9)*p(360) + p(9)*dp(360)
dp(1509) = dp(9)*p(361) + p(9)*dp(361)
dp(1510) = dp(9)*p(409) + p(9)*dp(409)
dp(1511) = dp(9)*p(428) + p(9)*dp(428)
dp(1512) = dp(9)*p(368) + p(9)*dp(368)
dp(1513) = dp(9)*p(350) + p(9)*dp(350)
dp(1514) = dp(9)*p(369) + p(9)*dp(369)
dp(1515) = dp(9)*p(351) + p(9)*dp(351)
dp(1516) = dp(9)*p(370) + p(9)*dp(370)
dp(1517) = dp(9)*p(371) + p(9)*dp(371)
dp(1518) = dp(9)*p(372) + p(9)*dp(372)
dp(1519) = dp(9)*p(429) + p(9)*dp(429)
dp(1520) = dp(9)*p(375) + p(9)*dp(375)
dp(1521) = dp(9)*p(435) + p(9)*dp(435)
dp(1522) = dp(9)*p(352) + p(9)*dp(352)
dp(1523) = dp(9)*p(376) + p(9)*dp(376)
dp(1524) = dp(9)*p(377) + p(9)*dp(377)
dp(1525) = dp(9)*p(436) + p(9)*dp(436)
dp(1526) = dp(9)*p(378) + p(9)*dp(378)
dp(1527) = dp(9)*p(437) + p(9)*dp(437)
dp(1528) = dp(9)*p(451) + p(9)*dp(451)
dp(1529) = dp(9)*p(382) + p(9)*dp(382)
dp(1530) = dp(9)*p(353) + p(9)*dp(353)
dp(1531) = dp(9)*p(383) + p(9)*dp(383)
dp(1532) = dp(9)*p(384) + p(9)*dp(384)
dp(1533) = dp(9)*p(452) + p(9)*dp(452)
dp(1534) = dp(9)*p(459) + p(9)*dp(459)
dp(1535) = dp(9)*p(386) + p(9)*dp(386)
dp(1536) = dp(9)*p(460) + p(9)*dp(460)
dp(1537) = dp(9)*p(387) + p(9)*dp(387)
dp(1538) = dp(9)*p(388) + p(9)*dp(388)
dp(1539) = dp(9)*p(461) + p(9)*dp(461)
dp(1540) = dp(9)*p(462) + p(9)*dp(462)
dp(1541) = dp(9)*p(463) + p(9)*dp(463)
dp(1542) = dp(9)*p(466) + p(9)*dp(466)
dp(1543) = dp(9)*p(467) + p(9)*dp(467)
dp(1544) = dp(9)*p(533) + p(9)*dp(533)
dp(1545) = dp(9)*p(476) + p(9)*dp(476)
dp(1546) = dp(9)*p(390) + p(9)*dp(390)
dp(1547) = dp(9)*p(391) + p(9)*dp(391)
dp(1548) = dp(9)*p(477) + p(9)*dp(477)
dp(1549) = dp(9)*p(481) + p(9)*dp(481)
dp(1550) = dp(9)*p(482) + p(9)*dp(482)
dp(1551) = dp(9)*p(483) + p(9)*dp(483)
dp(1552) = dp(9)*p(545) + p(9)*dp(545)
dp(1553) = dp(9)*p(549) + p(9)*dp(549)
dp(1554) = dp(9)*p(488) + p(9)*dp(488)
dp(1555) = dp(9)*p(489) + p(9)*dp(489)
dp(1556) = dp(9)*p(559) + p(9)*dp(559)
dp(1557) = dp(9)*p(565) + p(9)*dp(565)
dp(1558) = dp(9)*p(401) + p(9)*dp(401)
dp(1559) = dp(9)*p(402) + p(9)*dp(402)
dp(1560) = dp(9)*p(403) + p(9)*dp(403)
dp(1561) = dp(9)*p(504) + p(9)*dp(504)
dp(1562) = dp(9)*p(354) + p(9)*dp(354)
dp(1563) = dp(9)*p(355) + p(9)*dp(355)
dp(1564) = dp(9)*p(404) + p(9)*dp(404)
dp(1565) = dp(9)*p(356) + p(9)*dp(356)
dp(1566) = dp(9)*p(357) + p(9)*dp(357)
dp(1567) = dp(9)*p(405) + p(9)*dp(405)
dp(1568) = dp(9)*p(358) + p(9)*dp(358)
dp(1569) = dp(9)*p(406) + p(9)*dp(406)
dp(1570) = dp(9)*p(359) + p(9)*dp(359)
dp(1571) = dp(9)*p(407) + p(9)*dp(407)
dp(1572) = dp(9)*p(408) + p(9)*dp(408)
dp(1573) = dp(9)*p(505) + p(9)*dp(505)
dp(1574) = dp(14)*p(566) + p(14)*dp(566)
dp(1575) = dp(14)*p(567) + p(14)*dp(567)
dp(1576) = dp(14)*p(568) + p(14)*dp(568)
dp(1577) = dp(14)*p(569) + p(14)*dp(569)
dp(1578) = dp(9)*p(514) + p(9)*dp(514)
dp(1579) = dp(9)*p(416) + p(9)*dp(416)
dp(1580) = dp(9)*p(417) + p(9)*dp(417)
dp(1581) = dp(9)*p(418) + p(9)*dp(418)
dp(1582) = dp(9)*p(419) + p(9)*dp(419)
dp(1583) = dp(9)*p(515) + p(9)*dp(515)
dp(1584) = dp(9)*p(420) + p(9)*dp(420)
dp(1585) = dp(9)*p(362) + p(9)*dp(362)
dp(1586) = dp(9)*p(363) + p(9)*dp(363)
dp(1587) = dp(9)*p(364) + p(9)*dp(364)
dp(1588) = dp(9)*p(421) + p(9)*dp(421)
dp(1589) = dp(9)*p(422) + p(9)*dp(422)
dp(1590) = dp(9)*p(365) + p(9)*dp(365)
dp(1591) = dp(9)*p(366) + p(9)*dp(366)
dp(1592) = dp(9)*p(423) + p(9)*dp(423)
dp(1593) = dp(9)*p(424) + p(9)*dp(424)
dp(1594) = dp(9)*p(367) + p(9)*dp(367)
dp(1595) = dp(9)*p(425) + p(9)*dp(425)
dp(1596) = dp(9)*p(426) + p(9)*dp(426)
dp(1597) = dp(9)*p(427) + p(9)*dp(427)
dp(1598) = dp(9)*p(516) + p(9)*dp(516)
dp(1599) = dp(14)*p(495) + p(14)*dp(495)
dp(1600) = dp(14)*p(496) + p(14)*dp(496)
dp(1601) = dp(14)*p(497) + p(14)*dp(497)
dp(1602) = dp(14)*p(498) + p(14)*dp(498)
dp(1603) = dp(14)*p(499) + p(14)*dp(499)
dp(1604) = dp(14)*p(500) + p(14)*dp(500)
dp(1605) = dp(14)*p(501) + p(14)*dp(501)
dp(1606) = dp(14)*p(502) + p(14)*dp(502)
dp(1607) = dp(14)*p(503) + p(14)*dp(503)
dp(1608) = dp(9)*p(430) + p(9)*dp(430)
dp(1609) = dp(9)*p(431) + p(9)*dp(431)
dp(1610) = dp(9)*p(517) + p(9)*dp(517)
dp(1611) = dp(9)*p(373) + p(9)*dp(373)
dp(1612) = dp(9)*p(432) + p(9)*dp(432)
dp(1613) = dp(9)*p(374) + p(9)*dp(374)
dp(1614) = dp(9)*p(433) + p(9)*dp(433)
dp(1615) = dp(9)*p(434) + p(9)*dp(434)
dp(1616) = dp(9)*p(518) + p(9)*dp(518)
dp(1617) = dp(14)*p(401) + p(14)*dp(401)
dp(1618) = dp(14)*p(402) + p(14)*dp(402)
dp(1619) = dp(14)*p(403) + p(14)*dp(403)
dp(1620) = dp(14)*p(404) + p(14)*dp(404)
dp(1621) = dp(14)*p(405) + p(14)*dp(405)
dp(1622) = dp(14)*p(406) + p(14)*dp(406)
dp(1623) = dp(14)*p(407) + p(14)*dp(407)
dp(1624) = dp(14)*p(408) + p(14)*dp(408)
dp(1625) = dp(14)*p(360) + p(14)*dp(360)
dp(1626) = dp(14)*p(361) + p(14)*dp(361)
dp(1627) = dp(9)*p(525) + p(9)*dp(525)
dp(1628) = dp(9)*p(442) + p(9)*dp(442)
dp(1629) = dp(9)*p(443) + p(9)*dp(443)
dp(1630) = dp(9)*p(444) + p(9)*dp(444)
dp(1631) = dp(9)*p(526) + p(9)*dp(526)
dp(1632) = dp(9)*p(445) + p(9)*dp(445)
dp(1633) = dp(9)*p(379) + p(9)*dp(379)
dp(1634) = dp(9)*p(380) + p(9)*dp(380)
dp(1635) = dp(9)*p(446) + p(9)*dp(446)
dp(1636) = dp(9)*p(447) + p(9)*dp(447)
dp(1637) = dp(9)*p(381) + p(9)*dp(381)
dp(1638) = dp(9)*p(448) + p(9)*dp(448)
dp(1639) = dp(9)*p(449) + p(9)*dp(449)
dp(1640) = dp(9)*p(450) + p(9)*dp(450)
dp(1641) = dp(9)*p(527) + p(9)*dp(527)
dp(1642) = dp(14)*p(506) + p(14)*dp(506)
dp(1643) = dp(14)*p(507) + p(14)*dp(507)
dp(1644) = dp(14)*p(508) + p(14)*dp(508)
dp(1645) = dp(14)*p(509) + p(14)*dp(509)
dp(1646) = dp(14)*p(510) + p(14)*dp(510)
dp(1647) = dp(14)*p(511) + p(14)*dp(511)
dp(1648) = dp(14)*p(512) + p(14)*dp(512)
dp(1649) = dp(14)*p(513) + p(14)*dp(513)
dp(1650) = dp(9)*p(528) + p(9)*dp(528)
dp(1651) = dp(9)*p(453) + p(9)*dp(453)
dp(1652) = dp(9)*p(454) + p(9)*dp(454)
dp(1653) = dp(9)*p(529) + p(9)*dp(529)
dp(1654) = dp(9)*p(455) + p(9)*dp(455)
dp(1655) = dp(9)*p(385) + p(9)*dp(385)
dp(1656) = dp(9)*p(456) + p(9)*dp(456)
dp(1657) = dp(9)*p(457) + p(9)*dp(457)
dp(1658) = dp(9)*p(458) + p(9)*dp(458)
dp(1659) = dp(9)*p(530) + p(9)*dp(530)
dp(1660) = dp(14)*p(416) + p(14)*dp(416)
dp(1661) = dp(14)*p(417) + p(14)*dp(417)
dp(1662) = dp(14)*p(418) + p(14)*dp(418)
dp(1663) = dp(14)*p(419) + p(14)*dp(419)
dp(1664) = dp(14)*p(420) + p(14)*dp(420)
dp(1665) = dp(14)*p(421) + p(14)*dp(421)
dp(1666) = dp(14)*p(422) + p(14)*dp(422)
dp(1667) = dp(14)*p(423) + p(14)*dp(423)
dp(1668) = dp(14)*p(424) + p(14)*dp(424)
dp(1669) = dp(14)*p(425) + p(14)*dp(425)
dp(1670) = dp(14)*p(426) + p(14)*dp(426)
dp(1671) = dp(14)*p(427) + p(14)*dp(427)
dp(1672) = dp(14)*p(368) + p(14)*dp(368)
dp(1673) = dp(14)*p(369) + p(14)*dp(369)
dp(1674) = dp(14)*p(370) + p(14)*dp(370)
dp(1675) = dp(14)*p(371) + p(14)*dp(371)
dp(1676) = dp(14)*p(372) + p(14)*dp(372)
dp(1677) = dp(9)*p(464) + p(9)*dp(464)
dp(1678) = dp(9)*p(531) + p(9)*dp(531)
dp(1679) = dp(9)*p(465) + p(9)*dp(465)
dp(1680) = dp(9)*p(532) + p(9)*dp(532)
dp(1681) = dp(14)*p(430) + p(14)*dp(430)
dp(1682) = dp(14)*p(431) + p(14)*dp(431)
dp(1683) = dp(14)*p(432) + p(14)*dp(432)
dp(1684) = dp(14)*p(433) + p(14)*dp(433)
dp(1685) = dp(14)*p(434) + p(14)*dp(434)
dp(1686) = dp(14)*p(375) + p(14)*dp(375)
dp(1687) = dp(14)*p(376) + p(14)*dp(376)
dp(1688) = dp(14)*p(377) + p(14)*dp(377)
dp(1689) = dp(14)*p(378) + p(14)*dp(378)
dp(1690) = dp(9)*p(539) + p(9)*dp(539)
dp(1691) = dp(9)*p(470) + p(9)*dp(470)
dp(1692) = dp(9)*p(471) + p(9)*dp(471)
dp(1693) = dp(9)*p(540) + p(9)*dp(540)
dp(1694) = dp(9)*p(472) + p(9)*dp(472)
dp(1695) = dp(9)*p(389) + p(9)*dp(389)
dp(1696) = dp(9)*p(473) + p(9)*dp(473)
dp(1697) = dp(9)*p(474) + p(9)*dp(474)
dp(1698) = dp(9)*p(475) + p(9)*dp(475)
dp(1699) = dp(9)*p(541) + p(9)*dp(541)
dp(1700) = dp(14)*p(519) + p(14)*dp(519)
dp(1701) = dp(14)*p(520) + p(14)*dp(520)
dp(1702) = dp(14)*p(521) + p(14)*dp(521)
dp(1703) = dp(14)*p(522) + p(14)*dp(522)
dp(1704) = dp(14)*p(523) + p(14)*dp(523)
dp(1705) = dp(14)*p(524) + p(14)*dp(524)
dp(1706) = dp(9)*p(542) + p(9)*dp(542)
dp(1707) = dp(9)*p(478) + p(9)*dp(478)
dp(1708) = dp(9)*p(543) + p(9)*dp(543)
dp(1709) = dp(9)*p(479) + p(9)*dp(479)
dp(1710) = dp(9)*p(480) + p(9)*dp(480)
dp(1711) = dp(9)*p(544) + p(9)*dp(544)
dp(1712) = dp(14)*p(442) + p(14)*dp(442)
dp(1713) = dp(14)*p(443) + p(14)*dp(443)
dp(1714) = dp(14)*p(444) + p(14)*dp(444)
dp(1715) = dp(14)*p(445) + p(14)*dp(445)
dp(1716) = dp(14)*p(446) + p(14)*dp(446)
dp(1717) = dp(14)*p(447) + p(14)*dp(447)
dp(1718) = dp(14)*p(448) + p(14)*dp(448)
dp(1719) = dp(14)*p(449) + p(14)*dp(449)
dp(1720) = dp(14)*p(450) + p(14)*dp(450)
dp(1721) = dp(14)*p(382) + p(14)*dp(382)
dp(1722) = dp(14)*p(383) + p(14)*dp(383)
dp(1723) = dp(14)*p(384) + p(14)*dp(384)
dp(1724) = dp(9)*p(546) + p(9)*dp(546)
dp(1725) = dp(9)*p(547) + p(9)*dp(547)
dp(1726) = dp(9)*p(548) + p(9)*dp(548)
dp(1727) = dp(14)*p(453) + p(14)*dp(453)
dp(1728) = dp(14)*p(454) + p(14)*dp(454)
dp(1729) = dp(14)*p(455) + p(14)*dp(455)
dp(1730) = dp(14)*p(456) + p(14)*dp(456)
dp(1731) = dp(14)*p(457) + p(14)*dp(457)
dp(1732) = dp(14)*p(458) + p(14)*dp(458)
dp(1733) = dp(14)*p(386) + p(14)*dp(386)
dp(1734) = dp(14)*p(387) + p(14)*dp(387)
dp(1735) = dp(14)*p(388) + p(14)*dp(388)
dp(1736) = dp(9)*p(578) + p(9)*dp(578)
dp(1737) = dp(14)*p(464) + p(14)*dp(464)
dp(1738) = dp(14)*p(465) + p(14)*dp(465)
dp(1739) = dp(9)*p(553) + p(9)*dp(553)
dp(1740) = dp(9)*p(485) + p(9)*dp(485)
dp(1741) = dp(9)*p(554) + p(9)*dp(554)
dp(1742) = dp(9)*p(486) + p(9)*dp(486)
dp(1743) = dp(9)*p(487) + p(9)*dp(487)
dp(1744) = dp(9)*p(555) + p(9)*dp(555)
dp(1745) = dp(14)*p(534) + p(14)*dp(534)
dp(1746) = dp(14)*p(535) + p(14)*dp(535)
dp(1747) = dp(14)*p(536) + p(14)*dp(536)
dp(1748) = dp(14)*p(537) + p(14)*dp(537)
dp(1749) = dp(14)*p(538) + p(14)*dp(538)
dp(1750) = dp(9)*p(556) + p(9)*dp(556)
dp(1751) = dp(9)*p(557) + p(9)*dp(557)
dp(1752) = dp(9)*p(558) + p(9)*dp(558)
dp(1753) = dp(14)*p(470) + p(14)*dp(470)
dp(1754) = dp(14)*p(471) + p(14)*dp(471)
dp(1755) = dp(14)*p(472) + p(14)*dp(472)
dp(1756) = dp(14)*p(473) + p(14)*dp(473)
dp(1757) = dp(14)*p(474) + p(14)*dp(474)
dp(1758) = dp(14)*p(475) + p(14)*dp(475)
dp(1759) = dp(14)*p(390) + p(14)*dp(390)
dp(1760) = dp(14)*p(391) + p(14)*dp(391)
dp(1761) = dp(9)*p(581) + p(9)*dp(581)
dp(1762) = dp(14)*p(478) + p(14)*dp(478)
dp(1763) = dp(14)*p(479) + p(14)*dp(479)
dp(1764) = dp(14)*p(480) + p(14)*dp(480)
dp(1765) = dp(9)*p(562) + p(9)*dp(562)
dp(1766) = dp(9)*p(563) + p(9)*dp(563)
dp(1767) = dp(9)*p(564) + p(9)*dp(564)
dp(1768) = dp(14)*p(550) + p(14)*dp(550)
dp(1769) = dp(14)*p(551) + p(14)*dp(551)
dp(1770) = dp(14)*p(552) + p(14)*dp(552)
dp(1771) = dp(9)*p(584) + p(9)*dp(584)
dp(1772) = dp(14)*p(485) + p(14)*dp(485)
dp(1773) = dp(14)*p(486) + p(14)*dp(486)
dp(1774) = dp(14)*p(487) + p(14)*dp(487)
dp(1775) = dp(9)*p(587) + p(9)*dp(587)
dp(1776) = dp(14)*p(560) + p(14)*dp(560)
dp(1777) = dp(14)*p(561) + p(14)*dp(561)
dp(1778) = dp(9)*p(570) + p(9)*dp(570)
dp(1779) = dp(9)*p(495) + p(9)*dp(495)
dp(1780) = dp(9)*p(392) + p(9)*dp(392)
dp(1781) = dp(9)*p(496) + p(9)*dp(496)
dp(1782) = dp(9)*p(393) + p(9)*dp(393)
dp(1783) = dp(9)*p(394) + p(9)*dp(394)
dp(1784) = dp(9)*p(497) + p(9)*dp(497)
dp(1785) = dp(9)*p(395) + p(9)*dp(395)
dp(1786) = dp(9)*p(396) + p(9)*dp(396)
dp(1787) = dp(9)*p(397) + p(9)*dp(397)
dp(1788) = dp(9)*p(498) + p(9)*dp(498)
dp(1789) = dp(9)*p(398) + p(9)*dp(398)
dp(1790) = dp(9)*p(399) + p(9)*dp(399)
dp(1791) = dp(9)*p(499) + p(9)*dp(499)
dp(1792) = dp(9)*p(500) + p(9)*dp(500)
dp(1793) = dp(9)*p(400) + p(9)*dp(400)
dp(1794) = dp(9)*p(501) + p(9)*dp(501)
dp(1795) = dp(9)*p(502) + p(9)*dp(502)
dp(1796) = dp(9)*p(503) + p(9)*dp(503)
dp(1797) = dp(9)*p(571) + p(9)*dp(571)
dp(1798) = dp(1)*p(1136) + p(1)*dp(1136) - dp(1558)
dp(1799) = dp(21)*p(401) + p(21)*dp(401) - dp(1565)
dp(1800) = dp(101)*p(114) + p(101)*dp(114) - dp(1570)
dp(1801) = dp(24)*p(346) + p(24)*dp(346) - dp(1573)
dp(1802) = dp(3)*p(850) + p(3)*dp(850) - dp(1561) - dp(1801)
dp(1803) = dp(1)*p(1140) + p(1)*dp(1140) - dp(1561) - dp(1802)
dp(1804) = dp(1)*p(1141) + p(1)*dp(1141) - dp(1564) - dp(1803)
dp(1805) = dp(1)*p(1142) + p(1)*dp(1142) - dp(1567) - dp(1804)
dp(1806) = dp(13)*p(566) + p(13)*dp(566) - dp(1558)
dp(1807) = dp(73)*p(178) + p(73)*dp(178) - dp(1565)
dp(1808) = dp(39)*p(285) + p(39)*dp(285) - dp(1570)
dp(1809) = dp(3)*p(851) + p(3)*dp(851) - dp(1573)
dp(1810) = dp(14)*p(588) + p(14)*dp(588)
dp(1811) = dp(9)*p(572) + p(9)*dp(572)
dp(1812) = dp(9)*p(506) + p(9)*dp(506)
dp(1813) = dp(9)*p(410) + p(9)*dp(410)
dp(1814) = dp(9)*p(507) + p(9)*dp(507)
dp(1815) = dp(9)*p(411) + p(9)*dp(411)
dp(1816) = dp(9)*p(412) + p(9)*dp(412)
dp(1817) = dp(9)*p(508) + p(9)*dp(508)
dp(1818) = dp(9)*p(413) + p(9)*dp(413)
dp(1819) = dp(9)*p(414) + p(9)*dp(414)
dp(1820) = dp(9)*p(509) + p(9)*dp(509)
dp(1821) = dp(9)*p(510) + p(9)*dp(510)
dp(1822) = dp(9)*p(415) + p(9)*dp(415)
dp(1823) = dp(9)*p(511) + p(9)*dp(511)
dp(1824) = dp(9)*p(512) + p(9)*dp(512)
dp(1825) = dp(9)*p(513) + p(9)*dp(513)
dp(1826) = dp(9)*p(573) + p(9)*dp(573)
dp(1827) = dp(24)*p(331) + p(24)*dp(331) - dp(1510)
dp(1828) = dp(91)*p(114) + p(91)*dp(114) - dp(1507)
dp(1829) = dp(16)*p(401) + p(16)*dp(401) - dp(1506)
dp(1830) = dp(2)*p(1136) + p(2)*dp(1136) - dp(1574)
dp(1831) = dp(1)*p(1163) + p(1)*dp(1163) - dp(1581) - dp(1830)
dp(1832) = dp(1)*p(1164) + p(1)*dp(1164) - dp(1582) - dp(1831)
dp(1833) = dp(1)*p(1165) + p(1)*dp(1165) - dp(1583) - dp(1832)
dp(1834) = dp(13)*p(570) + p(13)*dp(570) - dp(1810)
dp(1835) = dp(2)*p(1140) + p(2)*dp(1140) - dp(1577) - dp(1834)
dp(1836) = dp(32)*p(331) + p(32)*dp(331) - dp(1810)
dp(1837) = dp(1)*p(1167) + p(1)*dp(1167) - dp(1583) - dp(1835)
dp(1838) = dp(73)*p(181) + p(73)*dp(181) - dp(1810)
dp(1839) = dp(1)*p(1169) + p(1)*dp(1169) - dp(1588) - dp(1837)
dp(1840) = dp(91)*p(150) + p(91)*dp(150) - dp(1810)
dp(1841) = dp(6)*p(843) + p(6)*dp(843) - dp(1574)
dp(1842) = dp(41)*p(285) + p(41)*dp(285) - dp(1810)
dp(1843) = dp(32)*p(279) + p(32)*dp(279) - dp(1506)
dp(1844) = dp(16)*p(505) + p(16)*dp(505) - dp(1810)
dp(1845) = dp(69)*p(150) + p(69)*dp(150) - dp(1507)
dp(1846) = dp(4)*p(851) + p(4)*dp(851) - dp(1810)
dp(1847) = dp(11)*p(505) + p(11)*dp(505) - dp(1510)
dp(1848) = dp(14)*p(570) + p(14)*dp(570)
dp(1849) = dp(14)*p(571) + p(14)*dp(571)
dp(1850) = dp(1)*p(1180) + p(1)*dp(1180) - dp(1608)
dp(1851) = dp(21)*p(430) + p(21)*dp(430) - dp(1613)
dp(1852) = dp(101)*p(130) + p(101)*dp(130) - dp(1616)
dp(1853) = dp(3)*p(864) + p(3)*dp(864) - dp(1610) - dp(1852)
dp(1854) = dp(1)*p(1183) + p(1)*dp(1183) - dp(1610) - dp(1853)
dp(1855) = dp(1)*p(1184) + p(1)*dp(1184) - dp(1612) - dp(1854)
dp(1856) = dp(81)*p(178) + p(81)*dp(178) - dp(1608)
dp(1857) = dp(39)*p(296) + p(39)*dp(296) - dp(1613)
dp(1858) = dp(3)*p(865) + p(3)*dp(865) - dp(1616)
dp(1859) = dp(14)*p(504) + p(14)*dp(504)
dp(1860) = dp(14)*p(505) + p(14)*dp(505)
dp(1861) = dp(14)*p(409) + p(14)*dp(409)
dp(1862) = dp(9)*p(574) + p(9)*dp(574)
dp(1863) = dp(9)*p(519) + p(9)*dp(519)
dp(1864) = dp(9)*p(438) + p(9)*dp(438)
dp(1865) = dp(9)*p(520) + p(9)*dp(520)
dp(1866) = dp(9)*p(439) + p(9)*dp(439)
dp(1867) = dp(9)*p(440) + p(9)*dp(440)
dp(1868) = dp(9)*p(521) + p(9)*dp(521)
dp(1869) = dp(9)*p(441) + p(9)*dp(441)
dp(1870) = dp(9)*p(522) + p(9)*dp(522)
dp(1871) = dp(9)*p(523) + p(9)*dp(523)
dp(1872) = dp(9)*p(524) + p(9)*dp(524)
dp(1873) = dp(9)*p(575) + p(9)*dp(575)
dp(1874) = dp(24)*p(333) + p(24)*dp(333) - dp(1511)
dp(1875) = dp(93)*p(114) + p(93)*dp(114) - dp(1513)
dp(1876) = dp(18)*p(401) + p(18)*dp(401) - dp(1516)
dp(1877) = dp(1)*p(1202) + p(1)*dp(1202) - dp(1629) - dp(1876)
dp(1878) = dp(1)*p(1203) + p(1)*dp(1203) - dp(1630) - dp(1877)
dp(1879) = dp(1)*p(1204) + p(1)*dp(1204) - dp(1631) - dp(1878)
dp(1880) = dp(13)*p(572) + p(13)*dp(572) - dp(1848)
dp(1881) = dp(2)*p(1167) + p(2)*dp(1167) - dp(1607) - dp(1853)
dp(1882) = dp(32)*p(333) + p(32)*dp(333) - dp(1848)
dp(1883) = dp(1)*p(1206) + p(1)*dp(1206) - dp(1631) - dp(1881)
dp(1884) = dp(73)*p(183) + p(73)*dp(183) - dp(1848)
dp(1885) = dp(1)*p(1208) + p(1)*dp(1208) - dp(1635) - dp(1883)
dp(1886) = dp(93)*p(150) + p(93)*dp(150) - dp(1848)
dp(1887) = dp(13)*p(509) + p(13)*dp(509) - dp(1516)
dp(1888) = dp(43)*p(285) + p(43)*dp(285) - dp(1848)
dp(1889) = dp(73)*p(153) + p(73)*dp(153) - dp(1513)
dp(1890) = dp(18)*p(505) + p(18)*dp(505) - dp(1848)
dp(1891) = dp(34)*p(285) + p(34)*dp(285) - dp(1511)
dp(1892) = dp(14)*p(572) + p(14)*dp(572)
dp(1893) = dp(14)*p(573) + p(14)*dp(573)
dp(1894) = dp(91)*p(130) + p(91)*dp(130) - dp(1525)
dp(1895) = dp(16)*p(430) + p(16)*dp(430) - dp(1522)
dp(1896) = dp(2)*p(1180) + p(2)*dp(1180) - dp(1617)
dp(1897) = dp(1)*p(1219) + p(1)*dp(1219) - dp(1652) - dp(1896)
dp(1898) = dp(1)*p(1220) + p(1)*dp(1220) - dp(1653) - dp(1897)
dp(1899) = dp(81)*p(181) + p(81)*dp(181) - dp(1860)
dp(1900) = dp(2)*p(1183) + p(2)*dp(1183) - dp(1620) - dp(1899)
dp(1901) = dp(91)*p(160) + p(91)*dp(160) - dp(1860)
dp(1902) = dp(1)*p(1222) + p(1)*dp(1222) - dp(1653) - dp(1900)
dp(1903) = dp(41)*p(296) + p(41)*dp(296) - dp(1860)
dp(1904) = dp(45)*p(279) + p(45)*dp(279) - dp(1617)
dp(1905) = dp(16)*p(518) + p(16)*dp(518) - dp(1860)
dp(1906) = dp(69)*p(160) + p(69)*dp(160) - dp(1522)
dp(1907) = dp(4)*p(865) + p(4)*dp(865) - dp(1860)
dp(1908) = dp(11)*p(518) + p(11)*dp(518) - dp(1525)
dp(1909) = dp(14)*p(514) + p(14)*dp(514)
dp(1910) = dp(14)*p(515) + p(14)*dp(515)
dp(1911) = dp(14)*p(516) + p(14)*dp(516)
dp(1912) = dp(14)*p(428) + p(14)*dp(428)
dp(1913) = dp(14)*p(429) + p(14)*dp(429)
dp(1914) = dp(1)*p(1234) + p(1)*dp(1234) - dp(1677)
dp(1915) = dp(21)*p(464) + p(21)*dp(464) - dp(1680)
dp(1916) = dp(3)*p(880) + p(3)*dp(880) - dp(1678) - dp(1915)
dp(1917) = dp(1)*p(1236) + p(1)*dp(1236) - dp(1678) - dp(1916)
dp(1918) = dp(39)*p(310) + p(39)*dp(310) - dp(1677)
dp(1919) = dp(3)*p(881) + p(3)*dp(881) - dp(1680)
dp(1920) = dp(14)*p(517) + p(14)*dp(517)
dp(1921) = dp(14)*p(518) + p(14)*dp(518)
dp(1922) = dp(14)*p(435) + p(14)*dp(435)
dp(1923) = dp(14)*p(436) + p(14)*dp(436)
dp(1924) = dp(14)*p(437) + p(14)*dp(437)
dp(1925) = dp(9)*p(576) + p(9)*dp(576)
dp(1926) = dp(9)*p(534) + p(9)*dp(534)
dp(1927) = dp(9)*p(468) + p(9)*dp(468)
dp(1928) = dp(9)*p(535) + p(9)*dp(535)
dp(1929) = dp(9)*p(469) + p(9)*dp(469)
dp(1930) = dp(9)*p(536) + p(9)*dp(536)
dp(1931) = dp(9)*p(537) + p(9)*dp(537)
dp(1932) = dp(9)*p(538) + p(9)*dp(538)
dp(1933) = dp(9)*p(577) + p(9)*dp(577)
dp(1934) = dp(24)*p(335) + p(24)*dp(335) - dp(1528)
dp(1935) = dp(95)*p(114) + p(95)*dp(114) - dp(1530)
dp(1936) = dp(22)*p(401) + p(22)*dp(401) - dp(1645)
dp(1937) = dp(1)*p(1252) + p(1)*dp(1252) - dp(1692) - dp(1936)
dp(1938) = dp(1)*p(1253) + p(1)*dp(1253) - dp(1693) - dp(1937)
dp(1939) = dp(13)*p(574) + p(13)*dp(574) - dp(1892)
dp(1940) = dp(2)*p(1206) + p(2)*dp(1206) - dp(1649) - dp(1900)
dp(1941) = dp(32)*p(335) + p(32)*dp(335) - dp(1892)
dp(1942) = dp(1)*p(1255) + p(1)*dp(1255) - dp(1693) - dp(1940)
dp(1943) = dp(73)*p(185) + p(73)*dp(185) - dp(1892)
dp(1944) = dp(6)*p(869) + p(6)*dp(869) - dp(1645)
dp(1945) = dp(95)*p(150) + p(95)*dp(150) - dp(1892)
dp(1946) = dp(32)*p(299) + p(32)*dp(299) - dp(1530)
dp(1947) = dp(46)*p(285) + p(46)*dp(285) - dp(1892)
dp(1948) = dp(84)*p(150) + p(84)*dp(150) - dp(1528)
dp(1949) = dp(14)*p(574) + p(14)*dp(574)
dp(1950) = dp(14)*p(575) + p(14)*dp(575)
dp(1951) = dp(93)*p(130) + p(93)*dp(130) - dp(1527)
dp(1952) = dp(18)*p(430) + p(18)*dp(430) - dp(1526)
dp(1953) = dp(1)*p(1265) + p(1)*dp(1265) - dp(1707) - dp(1952)
dp(1954) = dp(1)*p(1266) + p(1)*dp(1266) - dp(1708) - dp(1953)
dp(1955) = dp(81)*p(183) + p(81)*dp(183) - dp(1861)
dp(1956) = dp(2)*p(1222) + p(2)*dp(1222) - dp(1665) - dp(1916)
dp(1957) = dp(93)*p(160) + p(93)*dp(160) - dp(1861)
dp(1958) = dp(1)*p(1268) + p(1)*dp(1268) - dp(1708) - dp(1956)
dp(1959) = dp(43)*p(296) + p(43)*dp(296) - dp(1861)
dp(1960) = dp(81)*p(153) + p(81)*dp(153) - dp(1526)
dp(1961) = dp(18)*p(518) + p(18)*dp(518) - dp(1861)
dp(1962) = dp(34)*p(296) + p(34)*dp(296) - dp(1527)
dp(1963) = dp(14)*p(525) + p(14)*dp(525)
dp(1964) = dp(14)*p(526) + p(14)*dp(526)
dp(1965) = dp(14)*p(527) + p(14)*dp(527)
dp(1966) = dp(14)*p(451) + p(14)*dp(451)
dp(1967) = dp(14)*p(452) + p(14)*dp(452)
dp(1968) = dp(16)*p(464) + p(16)*dp(464) - dp(1543)
dp(1969) = dp(2)*p(1234) + p(2)*dp(1234) - dp(1681)
dp(1970) = dp(1)*p(1279) + p(1)*dp(1279) - dp(1725) - dp(1969)
dp(1971) = dp(41)*p(310) + p(41)*dp(310) - dp(1921)
dp(1972) = dp(2)*p(1236) + p(2)*dp(1236) - dp(1683) - dp(1971)
dp(1973) = dp(16)*p(532) + p(16)*dp(532) - dp(1921)
dp(1974) = dp(69)*p(187) + p(69)*dp(187) - dp(1681)
dp(1975) = dp(4)*p(881) + p(4)*dp(881) - dp(1921)
dp(1976) = dp(11)*p(532) + p(11)*dp(532) - dp(1543)
dp(1977) = dp(14)*p(528) + p(14)*dp(528)
dp(1978) = dp(14)*p(529) + p(14)*dp(529)
dp(1979) = dp(14)*p(530) + p(14)*dp(530)
dp(1980) = dp(14)*p(459) + p(14)*dp(459)
dp(1981) = dp(14)*p(460) + p(14)*dp(460)
dp(1982) = dp(14)*p(461) + p(14)*dp(461)
dp(1983) = dp(14)*p(462) + p(14)*dp(462)
dp(1984) = dp(14)*p(463) + p(14)*dp(463)
dp(1985) = dp(1)*p(1290) + p(1)*dp(1290) - dp(1736)
dp(1986) = dp(4)*p(806) + p(4)*dp(806) - dp(1761) - dp(1732)
dp(1987) = dp(3)*p(900) + p(3)*dp(900) - dp(1736)
dp(1988) = dp(14)*p(531) + p(14)*dp(531)
dp(1989) = dp(14)*p(532) + p(14)*dp(532)
dp(1990) = dp(14)*p(466) + p(14)*dp(466)
dp(1991) = dp(14)*p(467) + p(14)*dp(467)
dp(1992) = dp(9)*p(579) + p(9)*dp(579)
dp(1993) = dp(9)*p(550) + p(9)*dp(550)
dp(1994) = dp(9)*p(484) + p(9)*dp(484)
dp(1995) = dp(9)*p(551) + p(9)*dp(551)
dp(1996) = dp(9)*p(552) + p(9)*dp(552)
dp(1997) = dp(9)*p(580) + p(9)*dp(580)
dp(1998) = dp(24)*p(337) + p(24)*dp(337) - dp(1545)
dp(1999) = dp(98)*p(114) + p(98)*dp(114) - dp(1547)
dp(2000) = dp(1)*p(1299) + p(1)*dp(1299) - dp(1740) - dp(1999)
dp(2001) = dp(1)*p(1300) + p(1)*dp(1300) - dp(1741) - dp(2000)
dp(2002) = dp(13)*p(576) + p(13)*dp(576) - dp(1949)
dp(2003) = dp(2)*p(1255) + p(2)*dp(1255) - dp(1705) - dp(1956)
dp(2004) = dp(32)*p(337) + p(32)*dp(337) - dp(1949)
dp(2005) = dp(1)*p(1302) + p(1)*dp(1302) - dp(1741) - dp(2003)
dp(2006) = dp(73)*p(188) + p(73)*dp(188) - dp(1949)
dp(2007) = dp(13)*p(536) + p(13)*dp(536) - dp(1547)
dp(2008) = dp(98)*p(150) + p(98)*dp(150) - dp(1949)
dp(2009) = dp(73)*p(173) + p(73)*dp(173) - dp(1545)
dp(2010) = dp(14)*p(576) + p(14)*dp(576)
dp(2011) = dp(14)*p(577) + p(14)*dp(577)
dp(2012) = dp(95)*p(130) + p(95)*dp(130) - dp(1540)
dp(2013) = dp(22)*p(430) + p(22)*dp(430) - dp(1674)
dp(2014) = dp(1)*p(1310) + p(1)*dp(1310) - dp(1751) - dp(2013)
dp(2015) = dp(81)*p(185) + p(81)*dp(185) - dp(1912)
dp(2016) = dp(2)*p(1268) + p(2)*dp(1268) - dp(1716) - dp(1972)
dp(2017) = dp(95)*p(160) + p(95)*dp(160) - dp(1912)
dp(2018) = dp(45)*p(299) + p(45)*dp(299) - dp(1674)
dp(2019) = dp(46)*p(296) + p(46)*dp(296) - dp(1912)
dp(2020) = dp(84)*p(160) + p(84)*dp(160) - dp(1540)
dp(2021) = dp(14)*p(539) + p(14)*dp(539)
dp(2022) = dp(14)*p(540) + p(14)*dp(540)
dp(2023) = dp(14)*p(541) + p(14)*dp(541)
dp(2024) = dp(14)*p(476) + p(14)*dp(476)
dp(2025) = dp(14)*p(477) + p(14)*dp(477)
dp(2026) = dp(18)*p(464) + p(18)*dp(464) - dp(1544)
dp(2027) = dp(1)*p(1320) + p(1)*dp(1320) - dp(1761) - dp(2026)
dp(2028) = dp(43)*p(310) + p(43)*dp(310) - dp(1923)
dp(2029) = dp(2)*p(1281) + p(2)*dp(1281) - dp(1730) - dp(1986)
dp(2030) = dp(18)*p(532) + p(18)*dp(532) - dp(1923)
dp(2031) = dp(34)*p(310) + p(34)*dp(310) - dp(1544)
dp(2032) = dp(14)*p(542) + p(14)*dp(542)
dp(2033) = dp(14)*p(543) + p(14)*dp(543)
dp(2034) = dp(14)*p(544) + p(14)*dp(544)
dp(2035) = dp(14)*p(481) + p(14)*dp(481)
dp(2036) = dp(14)*p(482) + p(14)*dp(482)
dp(2037) = dp(14)*p(483) + p(14)*dp(483)
dp(2038) = dp(2)*p(1290) + p(2)*dp(1290) - dp(1737)
dp(2039) = dp(4)*p(900) + p(4)*dp(900) - dp(1989)
dp(2040) = dp(11)*p(578) + p(11)*dp(578) - dp(1737)
dp(2041) = dp(14)*p(546) + p(14)*dp(546)
dp(2042) = dp(14)*p(547) + p(14)*dp(547)
dp(2043) = dp(14)*p(548) + p(14)*dp(548)
dp(2044) = dp(9)*p(582) + p(9)*dp(582)
dp(2045) = dp(9)*p(560) + p(9)*dp(560)
dp(2046) = dp(9)*p(561) + p(9)*dp(561)
dp(2047) = dp(9)*p(583) + p(9)*dp(583)
dp(2048) = dp(24)*p(340) + p(24)*dp(340) - dp(1554)
dp(2049) = dp(102)*p(114) + p(102)*dp(114) - dp(1747)
dp(2050) = dp(1)*p(1329) + p(1)*dp(1329) - dp(1766) - dp(2049)
dp(2051) = dp(13)*p(579) + p(13)*dp(579) - dp(2010)
dp(2052) = dp(2)*p(1302) + p(2)*dp(1302) - dp(1749) - dp(2016)
dp(2053) = dp(32)*p(340) + p(32)*dp(340) - dp(2010)
dp(2054) = dp(6)*p(904) + p(6)*dp(904) - dp(1747)
dp(2055) = dp(73)*p(191) + p(73)*dp(191) - dp(2010)
dp(2056) = dp(32)*p(323) + p(32)*dp(323) - dp(1554)
dp(2057) = dp(14)*p(579) + p(14)*dp(579)
dp(2058) = dp(14)*p(580) + p(14)*dp(580)
dp(2059) = dp(98)*p(130) + p(98)*dp(130) - dp(1552)
dp(2060) = dp(1)*p(1336) + p(1)*dp(1336) - dp(1771) - dp(2059)
dp(2061) = dp(81)*p(188) + p(81)*dp(188) - dp(1966)
dp(2062) = dp(2)*p(1312) + p(2)*dp(1312) - dp(1756) - dp(2029)
dp(2063) = dp(98)*p(160) + p(98)*dp(160) - dp(1966)
dp(2064) = dp(81)*p(173) + p(81)*dp(173) - dp(1552)
dp(2065) = dp(14)*p(553) + p(14)*dp(553)
dp(2066) = dp(14)*p(554) + p(14)*dp(554)
dp(2067) = dp(14)*p(555) + p(14)*dp(555)
dp(2068) = dp(14)*p(488) + p(14)*dp(488)
dp(2069) = dp(14)*p(489) + p(14)*dp(489)
dp(2070) = dp(22)*p(464) + p(22)*dp(464) - dp(1689)
dp(2071) = dp(46)*p(310) + p(46)*dp(310) - dp(1924)
dp(2072) = dp(84)*p(187) + p(84)*dp(187) - dp(1689)
dp(2073) = dp(14)*p(556) + p(14)*dp(556)
dp(2074) = dp(14)*p(557) + p(14)*dp(557)
dp(2075) = dp(14)*p(558) + p(14)*dp(558)
dp(2076) = dp(9)*p(585) + p(9)*dp(585)
dp(2077) = dp(9)*p(586) + p(9)*dp(586)
dp(2078) = dp(24)*p(343) + p(24)*dp(343) - dp(1557)
dp(2079) = dp(1)*p(1343) + p(1)*dp(1343) - dp(1775) - dp(2078)
dp(2080) = dp(13)*p(582) + p(13)*dp(582) - dp(2057)
dp(2081) = dp(2)*p(1331) + p(2)*dp(1331) - dp(1770) - dp(2062)
dp(2082) = dp(32)*p(343) + p(32)*dp(343) - dp(2057)
dp(2083) = dp(13)*p(561) + p(13)*dp(561) - dp(1557)
dp(2084) = dp(14)*p(582) + p(14)*dp(582)
dp(2085) = dp(14)*p(583) + p(14)*dp(583)
dp(2086) = dp(102)*p(130) + p(102)*dp(130) - dp(1760)
dp(2087) = dp(81)*p(191) + p(81)*dp(191) - dp(2024)
dp(2088) = dp(45)*p(323) + p(45)*dp(323) - dp(1760)
dp(2089) = dp(14)*p(562) + p(14)*dp(562)
dp(2090) = dp(14)*p(563) + p(14)*dp(563)
dp(2091) = dp(14)*p(564) + p(14)*dp(564)
dp(2092) = dp(9)*p(589) + p(9)*dp(589)
dp(2093) = dp(24)*p(347) + p(24)*dp(347) - dp(1777)
dp(2094) = dp(13)*p(585) + p(13)*dp(585) - dp(2084)
dp(2095) = dp(6)*p(929) + p(6)*dp(929) - dp(1777)
dp(2096) = dp(14)*p(585) + p(14)*dp(585)
dp(2097) = dp(14)*p(586) + p(14)*dp(586)
dp(2098) = dp(9)*p(490) + p(9)*dp(490)
dp(2099) = dp(9)*p(491) + p(9)*dp(491)
dp(2100) = dp(9)*p(492) + p(9)*dp(492)
dp(2101) = dp(9)*p(566) + p(9)*dp(566)
dp(2102) = dp(9)*p(493) + p(9)*dp(493)
dp(2103) = dp(9)*p(567) + p(9)*dp(567)
dp(2104) = dp(9)*p(494) + p(9)*dp(494)
dp(2105) = dp(9)*p(568) + p(9)*dp(568)
dp(2106) = dp(9)*p(569) + p(9)*dp(569)
dp(2107) = dp(9)*p(588) + p(9)*dp(588)
dp(2108) = dp(3)*p(938) + p(3)*dp(938) - dp(1778)
dp(2109) = dp(39)*p(331) + p(39)*dp(331) - dp(1780)
dp(2110) = dp(91)*p(178) + p(91)*dp(178) - dp(1783)
dp(2111) = dp(16)*p(566) + p(16)*dp(566) - dp(1787)
dp(2112) = dp(1)*p(1359) + p(1)*dp(1359) - dp(1784) - dp(2111)
dp(2113) = dp(2)*p(1476) + p(2)*dp(1476) - dp(2112)
dp(2114) = dp(1)*p(1360) + p(1)*dp(1360) - dp(1788) - dp(2112)
dp(2115) = dp(7)*p(843) + p(7)*dp(843) - dp(1787)
dp(2116) = dp(1)*p(1362) + p(1)*dp(1362) - dp(1792) - dp(2114)
dp(2117) = dp(49)*p(279) + p(49)*dp(279) - dp(1783)
dp(2118) = dp(1)*p(1364) + p(1)*dp(1364) - dp(1795) - dp(2116)
dp(2119) = dp(69)*p(194) + p(69)*dp(194) - dp(1780)
dp(2120) = dp(1)*p(1366) + p(1)*dp(1366) - dp(1797) - dp(2118)
dp(2121) = dp(11)*p(588) + p(11)*dp(588) - dp(1778)
dp(2122) = dp(1)*p(1368) + p(1)*dp(1368) - dp(1802) - dp(1801)
dp(2123) = dp(1)*p(1369) + p(1)*dp(1369) - dp(1809)
dp(2124) = dp(3)*p(940) + p(3)*dp(940) - dp(1811)
dp(2125) = dp(39)*p(333) + p(39)*dp(333) - dp(1813)
dp(2126) = dp(93)*p(178) + p(93)*dp(178) - dp(1816)
dp(2127) = dp(18)*p(566) + p(18)*dp(566) - dp(1820)
dp(2128) = dp(1)*p(1373) + p(1)*dp(1373) - dp(1817) - dp(2127)
dp(2129) = dp(1)*p(1374) + p(1)*dp(1374) - dp(1820)
dp(2130) = dp(1)*p(1375) + p(1)*dp(1375) - dp(1821) - dp(2128)
dp(2131) = dp(21)*p(509) + p(21)*dp(509) - dp(1816)
dp(2132) = dp(1)*p(1377) + p(1)*dp(1377) - dp(1824) - dp(2130)
dp(2133) = dp(101)*p(153) + p(101)*dp(153) - dp(1813)
dp(2134) = dp(1)*p(1379) + p(1)*dp(1379) - dp(1826) - dp(2132)
dp(2135) = dp(34)*p(346) + p(34)*dp(346) - dp(1811)
dp(2136) = dp(6)*p(938) + p(6)*dp(938) - dp(1810)
dp(2137) = dp(1)*p(1382) + p(1)*dp(1382) - dp(1835) - dp(1833)
dp(2138) = dp(2)*p(1369) + p(2)*dp(1369) - dp(1810)
dp(2139) = dp(1)*p(1384) + p(1)*dp(1384) - dp(1853) - dp(1852)
dp(2140) = dp(1)*p(1385) + p(1)*dp(1385) - dp(1858)
dp(2141) = dp(3)*p(942) + p(3)*dp(942) - dp(1862)
dp(2142) = dp(39)*p(335) + p(39)*dp(335) - dp(1864)
dp(2143) = dp(95)*p(178) + p(95)*dp(178) - dp(1867)
dp(2144) = dp(1)*p(1388) + p(1)*dp(1388) - dp(1865) - dp(2143)
dp(2145) = dp(2)*p(1374) + p(2)*dp(1374) - dp(1841)
dp(2146) = dp(1)*p(1389) + p(1)*dp(1389) - dp(1868) - dp(2144)
dp(2147) = dp(7)*p(869) + p(7)*dp(869) - dp(1867)
dp(2148) = dp(1)*p(1391) + p(1)*dp(1391) - dp(1871) - dp(2146)
dp(2149) = dp(49)*p(299) + p(49)*dp(299) - dp(1864)
dp(2150) = dp(1)*p(1393) + p(1)*dp(1393) - dp(1873) - dp(2148)
dp(2151) = dp(84)*p(194) + p(84)*dp(194) - dp(1862)
dp(2152) = dp(6)*p(940) + p(6)*dp(940) - dp(1848)
dp(2153) = dp(1)*p(1396) + p(1)*dp(1396) - dp(1881) - dp(1879)
dp(2154) = dp(8)*p(851) + p(8)*dp(851) - dp(1848)
dp(2155) = dp(45)*p(331) + p(45)*dp(331) - dp(1860)
dp(2156) = dp(1)*p(1399) + p(1)*dp(1399) - dp(1900) - dp(1898)
dp(2157) = dp(2)*p(1385) + p(2)*dp(1385) - dp(1860)
dp(2158) = dp(1)*p(1401) + p(1)*dp(1401) - dp(1916) - dp(1915)
dp(2159) = dp(1)*p(1402) + p(1)*dp(1402) - dp(1919)
dp(2160) = dp(3)*p(944) + p(3)*dp(944) - dp(1925)
dp(2161) = dp(39)*p(337) + p(39)*dp(337) - dp(1927)
dp(2162) = dp(98)*p(178) + p(98)*dp(178) - dp(1930)
dp(2163) = dp(1)*p(1405) + p(1)*dp(1405) - dp(1928) - dp(2162)
dp(2164) = dp(1)*p(1406) + p(1)*dp(1406) - dp(1930)
dp(2165) = dp(1)*p(1407) + p(1)*dp(1407) - dp(1931) - dp(2163)
dp(2166) = dp(21)*p(536) + p(21)*dp(536) - dp(1927)
dp(2167) = dp(1)*p(1409) + p(1)*dp(1409) - dp(1933) - dp(2165)
dp(2168) = dp(101)*p(173) + p(101)*dp(173) - dp(1925)
dp(2169) = dp(6)*p(942) + p(6)*dp(942) - dp(1892)
dp(2170) = dp(1)*p(1412) + p(1)*dp(1412) - dp(1940) - dp(1938)
dp(2171) = dp(22)*p(505) + p(22)*dp(505) - dp(1892)
dp(2172) = dp(45)*p(333) + p(45)*dp(333) - dp(1861)
dp(2173) = dp(1)*p(1415) + p(1)*dp(1415) - dp(1956) - dp(1954)
dp(2174) = dp(8)*p(865) + p(8)*dp(865) - dp(1861)
dp(2175) = dp(91)*p(187) + p(91)*dp(187) - dp(1921)
dp(2176) = dp(1)*p(1418) + p(1)*dp(1418) - dp(1972) - dp(1970)
dp(2177) = dp(2)*p(1402) + p(2)*dp(1402) - dp(1921)
dp(2178) = dp(1)*p(1421) + p(1)*dp(1421) - dp(1986) - dp(1985)
dp(2179) = dp(1)*p(1422) + p(1)*dp(1422) - dp(1987)
dp(2180) = dp(14)*p(533) + p(14)*dp(533)
dp(2181) = dp(3)*p(946) + p(3)*dp(946) - dp(1992)
dp(2182) = dp(39)*p(340) + p(39)*dp(340) - dp(1994)
dp(2183) = dp(1)*p(1425) + p(1)*dp(1425) - dp(1993) - dp(2182)
dp(2184) = dp(2)*p(1406) + p(2)*dp(1406) - dp(1944)
dp(2185) = dp(1)*p(1426) + p(1)*dp(1426) - dp(1995) - dp(2183)
dp(2186) = dp(7)*p(904) + p(7)*dp(904) - dp(1994)
dp(2187) = dp(1)*p(1428) + p(1)*dp(1428) - dp(1997) - dp(2185)
dp(2188) = dp(49)*p(323) + p(49)*dp(323) - dp(1992)
dp(2189) = dp(6)*p(944) + p(6)*dp(944) - dp(1949)
dp(2190) = dp(1)*p(1431) + p(1)*dp(1431) - dp(2003) - dp(2001)
dp(2191) = dp(50)*p(285) + p(50)*dp(285) - dp(1949)
dp(2192) = dp(45)*p(335) + p(45)*dp(335) - dp(1912)
dp(2193) = dp(1)*p(1434) + p(1)*dp(1434) - dp(2016) - dp(2014)
dp(2194) = dp(22)*p(518) + p(22)*dp(518) - dp(1912)
dp(2195) = dp(93)*p(187) + p(93)*dp(187) - dp(1923)
dp(2196) = dp(1)*p(1437) + p(1)*dp(1437) - dp(2029) - dp(2027)
dp(2197) = dp(8)*p(881) + p(8)*dp(881) - dp(1923)
dp(2198) = dp(14)*p(545) + p(14)*dp(545)
dp(2199) = dp(16)*p(578) + p(16)*dp(578) - dp(1989)
dp(2200) = dp(1)*p(1441) + p(1)*dp(1441) - dp(2040) - dp(2038)
dp(2201) = dp(2)*p(1422) + p(2)*dp(1422) - dp(1989)
dp(2202) = dp(14)*p(549) + p(14)*dp(549)
dp(2203) = dp(6)*p(899) + p(6)*dp(899) - dp(2042) - dp(2041)
dp(2204) = dp(1)*p(1491) + p(1)*dp(1491) - dp(2203)
dp(2205) = dp(14)*p(578) + p(14)*dp(578)
dp(2206) = dp(3)*p(949) + p(3)*dp(949) - dp(2044)
dp(2207) = dp(39)*p(343) + p(39)*dp(343) - dp(2046)
dp(2208) = dp(1)*p(1445) + p(1)*dp(1445) - dp(2045) - dp(2207)
dp(2209) = dp(1)*p(1446) + p(1)*dp(1446) - dp(2046)
dp(2210) = dp(1)*p(1447) + p(1)*dp(1447) - dp(2047) - dp(2208)
dp(2211) = dp(21)*p(561) + p(21)*dp(561) - dp(2044)
dp(2212) = dp(6)*p(946) + p(6)*dp(946) - dp(2010)
dp(2213) = dp(1)*p(1450) + p(1)*dp(1450) - dp(2052) - dp(2050)
dp(2214) = dp(102)*p(150) + p(102)*dp(150) - dp(2010)
dp(2215) = dp(45)*p(337) + p(45)*dp(337) - dp(1966)
dp(2216) = dp(1)*p(1453) + p(1)*dp(1453) - dp(2062) - dp(2060)
dp(2217) = dp(50)*p(296) + p(50)*dp(296) - dp(1966)
dp(2218) = dp(95)*p(187) + p(95)*dp(187) - dp(1924)
dp(2219) = dp(1)*p(1457) + p(1)*dp(1457) - dp(2072) - dp(2070)
dp(2220) = dp(22)*p(532) + p(22)*dp(532) - dp(1924)
dp(2221) = dp(14)*p(559) + p(14)*dp(559)
dp(2222) = dp(18)*p(578) + p(18)*dp(578) - dp(1991)
dp(2223) = dp(2)*p(1441) + p(2)*dp(1441) - dp(2042) - dp(2203)
dp(2224) = dp(8)*p(900) + p(8)*dp(900) - dp(1991)
dp(2225) = dp(14)*p(581) + p(14)*dp(581)
dp(2226) = dp(3)*p(952) + p(3)*dp(952) - dp(2076)
dp(2227) = dp(2)*p(1445) + p(2)*dp(1445) - dp(2053) - dp(2049)
dp(2228) = dp(2)*p(1446) + p(2)*dp(1446) - dp(2054)
dp(2229) = dp(1)*p(1461) + p(1)*dp(1461) - dp(2077) - dp(2227)
dp(2230) = dp(7)*p(929) + p(7)*dp(929) - dp(2076)
dp(2231) = dp(6)*p(949) + p(6)*dp(949) - dp(2057)
dp(2232) = dp(1)*p(1464) + p(1)*dp(1464) - dp(2081) - dp(2079)
dp(2233) = dp(73)*p(195) + p(73)*dp(195) - dp(2057)
dp(2234) = dp(45)*p(340) + p(45)*dp(340) - dp(2024)
dp(2235) = dp(1)*p(1467) + p(1)*dp(1467) - dp(2088) - dp(2086)
dp(2236) = dp(102)*p(160) + p(102)*dp(160) - dp(2024)
dp(2237) = dp(14)*p(565) + p(14)*dp(565)
dp(2238) = dp(98)*p(187) + p(98)*dp(187) - dp(1983)
dp(2239) = dp(2)*p(1457) + p(2)*dp(1457) - dp(2074) - dp(2223)
dp(2240) = dp(50)*p(310) + p(50)*dp(310) - dp(1983)
dp(2241) = dp(14)*p(584) + p(14)*dp(584)
dp(2242) = dp(3)*p(955) + p(3)*dp(955) - dp(2092)
dp(2243) = dp(2)*p(1461) + p(2)*dp(1461) - dp(2082) - dp(2079)
dp(2244) = dp(1)*p(1471) + p(1)*dp(1471) - dp(2092)
dp(2245) = dp(6)*p(952) + p(6)*dp(952) - dp(2084)
dp(2246) = dp(1)*p(1473) + p(1)*dp(1473) - dp(2095) - dp(2093)
dp(2247) = dp(32)*p(347) + p(32)*dp(347) - dp(2084)
dp(2248) = dp(45)*p(343) + p(45)*dp(343) - dp(2068)
dp(2249) = dp(2)*p(1467) + p(2)*dp(1467) - dp(2090) - dp(2239)
dp(2250) = dp(81)*p(195) + p(81)*dp(195) - dp(2068)
dp(2251) = dp(14)*p(587) + p(14)*dp(587)
dp(2252) = dp(2)*p(1470) + p(2)*dp(1470) - dp(2094) - dp(2093)
dp(2253) = dp(2)*p(1471) + p(2)*dp(1471) - dp(2095)
dp(2254) = dp(6)*p(955) + p(6)*dp(955) - dp(2096)
dp(2255) = dp(2)*p(1473) + p(2)*dp(1473) - dp(2097) - dp(2249)
dp(2256) = dp(13)*p(589) + p(13)*dp(589) - dp(2096)
dp(2257) = dp(14)*p(589) + p(14)*dp(589)
dp(2258) = dp(1)*p(1476) + p(1)*dp(1476) - dp(2101)
dp(2259) = dp(21)*p(566) + p(21)*dp(566) - dp(2098)
dp(2260) = dp(101)*p(178) + p(101)*dp(178) - dp(2099)
dp(2261) = dp(39)*p(346) + p(39)*dp(346) - dp(2104)
dp(2262) = dp(3)*p(958) + p(3)*dp(958) - dp(2107)
dp(2263) = dp(1)*p(1481) + p(1)*dp(1481) - dp(2108)
dp(2264) = dp(2)*p(1504) + p(2)*dp(1504) - dp(2263)
dp(2265) = dp(1)*p(1483) + p(1)*dp(1483) - dp(2124)
dp(2266) = dp(8)*p(958) + p(8)*dp(958) - dp(2265)
dp(2267) = dp(1)*p(1485) + p(1)*dp(1485) - dp(2141)
dp(2268) = dp(22)*p(588) + p(22)*dp(588) - dp(2267)
dp(2269) = dp(1)*p(1487) + p(1)*dp(1487) - dp(2160)
dp(2270) = dp(50)*p(346) + p(50)*dp(346) - dp(2269)
dp(2271) = dp(1)*p(1489) + p(1)*dp(1489) - dp(2181)
dp(2272) = dp(102)*p(194) + p(102)*dp(194) - dp(2271)
dp(2273) = dp(1)*p(1492) + p(1)*dp(1492) - dp(2206)
dp(2274) = dp(101)*p(195) + p(101)*dp(195) - dp(2273)
dp(2275) = dp(2)*p(1491) + p(2)*dp(1491) - dp(2205)
dp(2276) = dp(1)*p(1495) + p(1)*dp(1495) - dp(2226)
dp(2277) = dp(49)*p(347) + p(49)*dp(347) - dp(2276)
dp(2278) = dp(22)*p(578) + p(22)*dp(578) - dp(2180)
dp(2279) = dp(1)*p(1498) + p(1)*dp(1498) - dp(2242)
dp(2280) = dp(21)*p(589) + p(21)*dp(589) - dp(2279)
dp(2281) = dp(102)*p(187) + p(102)*dp(187) - dp(2198)
dp(2282) = dp(1)*p(1501) + p(1)*dp(1501) - dp(2252)
dp(2283) = dp(7)*p(959) + p(7)*dp(959) - dp(2282)
dp(2284) = dp(45)*p(347) + p(45)*dp(347) - dp(2237)
dp(2285) = dp(2)*p(1501) + p(2)*dp(1501) - dp(2254)
dp(2286) = dp(1)*p(1505) + p(1)*dp(1505) - dp(2285)
dp(2287) = dp(6)*p(959) + p(6)*dp(959) - dp(2257)
dp(2288) = dp(1)*p(1504) + p(1)*dp(1504) - dp(2262)
dp(2289) = dp(2)*p(1505) + p(2)*dp(1505) - dp(2287)
dp(2290) = dp(9)*p(590) + p(9)*dp(590)
dp(2291) = dp(9)*p(591) + p(9)*dp(591)
dp(2292) = dp(9)*p(613) + p(9)*dp(613)
dp(2293) = dp(9)*p(592) + p(9)*dp(592)
dp(2294) = dp(9)*p(614) + p(9)*dp(614)
dp(2295) = dp(9)*p(615) + p(9)*dp(615)
dp(2296) = dp(9)*p(701) + p(9)*dp(701)
dp(2297) = dp(9)*p(726) + p(9)*dp(726)
dp(2298) = dp(9)*p(626) + p(9)*dp(626)
dp(2299) = dp(9)*p(593) + p(9)*dp(593)
dp(2300) = dp(9)*p(627) + p(9)*dp(627)
dp(2301) = dp(9)*p(594) + p(9)*dp(594)
dp(2302) = dp(9)*p(595) + p(9)*dp(595)
dp(2303) = dp(9)*p(628) + p(9)*dp(628)
dp(2304) = dp(9)*p(596) + p(9)*dp(596)
dp(2305) = dp(9)*p(629) + p(9)*dp(629)
dp(2306) = dp(9)*p(630) + p(9)*dp(630)
dp(2307) = dp(9)*p(631) + p(9)*dp(631)
dp(2308) = dp(9)*p(727) + p(9)*dp(727)
dp(2309) = dp(9)*p(636) + p(9)*dp(636)
dp(2310) = dp(9)*p(637) + p(9)*dp(637)
dp(2311) = dp(9)*p(734) + p(9)*dp(734)
dp(2312) = dp(9)*p(597) + p(9)*dp(597)
dp(2313) = dp(9)*p(638) + p(9)*dp(638)
dp(2314) = dp(9)*p(598) + p(9)*dp(598)
dp(2315) = dp(9)*p(639) + p(9)*dp(639)
dp(2316) = dp(9)*p(640) + p(9)*dp(640)
dp(2317) = dp(9)*p(735) + p(9)*dp(735)
dp(2318) = dp(9)*p(599) + p(9)*dp(599)
dp(2319) = dp(9)*p(641) + p(9)*dp(641)
dp(2320) = dp(9)*p(736) + p(9)*dp(736)
dp(2321) = dp(9)*p(755) + p(9)*dp(755)
dp(2322) = dp(9)*p(648) + p(9)*dp(648)
dp(2323) = dp(9)*p(600) + p(9)*dp(600)
dp(2324) = dp(9)*p(649) + p(9)*dp(649)
dp(2325) = dp(9)*p(601) + p(9)*dp(601)
dp(2326) = dp(9)*p(650) + p(9)*dp(650)
dp(2327) = dp(9)*p(651) + p(9)*dp(651)
dp(2328) = dp(9)*p(652) + p(9)*dp(652)
dp(2329) = dp(9)*p(756) + p(9)*dp(756)
dp(2330) = dp(9)*p(766) + p(9)*dp(766)
dp(2331) = dp(9)*p(656) + p(9)*dp(656)
dp(2332) = dp(9)*p(657) + p(9)*dp(657)
dp(2333) = dp(9)*p(767) + p(9)*dp(767)
dp(2334) = dp(9)*p(658) + p(9)*dp(658)
dp(2335) = dp(9)*p(602) + p(9)*dp(602)
dp(2336) = dp(9)*p(659) + p(9)*dp(659)
dp(2337) = dp(9)*p(660) + p(9)*dp(660)
dp(2338) = dp(9)*p(661) + p(9)*dp(661)
dp(2339) = dp(9)*p(768) + p(9)*dp(768)
dp(2340) = dp(9)*p(769) + p(9)*dp(769)
dp(2341) = dp(9)*p(662) + p(9)*dp(662)
dp(2342) = dp(9)*p(663) + p(9)*dp(663)
dp(2343) = dp(9)*p(770) + p(9)*dp(770)
dp(2344) = dp(9)*p(665) + p(9)*dp(665)
dp(2345) = dp(9)*p(774) + p(9)*dp(774)
dp(2346) = dp(9)*p(666) + p(9)*dp(666)
dp(2347) = dp(9)*p(775) + p(9)*dp(775)
dp(2348) = dp(9)*p(776) + p(9)*dp(776)
dp(2349) = dp(9)*p(777) + p(9)*dp(777)
dp(2350) = dp(9)*p(882) + p(9)*dp(882)
dp(2351) = dp(9)*p(791) + p(9)*dp(791)
dp(2352) = dp(9)*p(670) + p(9)*dp(670)
dp(2353) = dp(9)*p(603) + p(9)*dp(603)
dp(2354) = dp(9)*p(671) + p(9)*dp(671)
dp(2355) = dp(9)*p(672) + p(9)*dp(672)
dp(2356) = dp(9)*p(792) + p(9)*dp(792)
dp(2357) = dp(9)*p(799) + p(9)*dp(799)
dp(2358) = dp(9)*p(674) + p(9)*dp(674)
dp(2359) = dp(9)*p(800) + p(9)*dp(800)
dp(2360) = dp(9)*p(675) + p(9)*dp(675)
dp(2361) = dp(9)*p(676) + p(9)*dp(676)
dp(2362) = dp(9)*p(801) + p(9)*dp(801)
dp(2363) = dp(9)*p(802) + p(9)*dp(802)
dp(2364) = dp(9)*p(803) + p(9)*dp(803)
dp(2365) = dp(9)*p(807) + p(9)*dp(807)
dp(2366) = dp(9)*p(808) + p(9)*dp(808)
dp(2367) = dp(9)*p(809) + p(9)*dp(809)
dp(2368) = dp(9)*p(898) + p(9)*dp(898)
dp(2369) = dp(9)*p(901) + p(9)*dp(901)
dp(2370) = dp(9)*p(818) + p(9)*dp(818)
dp(2371) = dp(9)*p(678) + p(9)*dp(678)
dp(2372) = dp(9)*p(679) + p(9)*dp(679)
dp(2373) = dp(9)*p(819) + p(9)*dp(819)
dp(2374) = dp(9)*p(823) + p(9)*dp(823)
dp(2375) = dp(9)*p(824) + p(9)*dp(824)
dp(2376) = dp(9)*p(825) + p(9)*dp(825)
dp(2377) = dp(9)*p(913) + p(9)*dp(913)
dp(2378) = dp(9)*p(917) + p(9)*dp(917)
dp(2379) = dp(9)*p(830) + p(9)*dp(830)
dp(2380) = dp(9)*p(831) + p(9)*dp(831)
dp(2381) = dp(9)*p(927) + p(9)*dp(927)
dp(2382) = dp(9)*p(933) + p(9)*dp(933)
dp(2383) = dp(9)*p(692) + p(9)*dp(692)
dp(2384) = dp(9)*p(693) + p(9)*dp(693)
dp(2385) = dp(9)*p(694) + p(9)*dp(694)
dp(2386) = dp(9)*p(850) + p(9)*dp(850)
dp(2387) = dp(9)*p(604) + p(9)*dp(604)
dp(2388) = dp(9)*p(605) + p(9)*dp(605)
dp(2389) = dp(9)*p(606) + p(9)*dp(606)
dp(2390) = dp(9)*p(695) + p(9)*dp(695)
dp(2391) = dp(9)*p(607) + p(9)*dp(607)
dp(2392) = dp(9)*p(608) + p(9)*dp(608)
dp(2393) = dp(9)*p(696) + p(9)*dp(696)
dp(2394) = dp(9)*p(609) + p(9)*dp(609)
dp(2395) = dp(9)*p(610) + p(9)*dp(610)
dp(2396) = dp(9)*p(697) + p(9)*dp(697)
dp(2397) = dp(9)*p(611) + p(9)*dp(611)
dp(2398) = dp(9)*p(698) + p(9)*dp(698)
dp(2399) = dp(9)*p(612) + p(9)*dp(612)
dp(2400) = dp(9)*p(699) + p(9)*dp(699)
dp(2401) = dp(9)*p(700) + p(9)*dp(700)
dp(2402) = dp(9)*p(851) + p(9)*dp(851)
dp(2403) = dp(14)*p(934) + p(14)*dp(934)
dp(2404) = dp(14)*p(935) + p(14)*dp(935)
dp(2405) = dp(14)*p(936) + p(14)*dp(936)
dp(2406) = dp(14)*p(937) + p(14)*dp(937)
dp(2407) = dp(9)*p(861) + p(9)*dp(861)
dp(2408) = dp(9)*p(711) + p(9)*dp(711)
dp(2409) = dp(9)*p(712) + p(9)*dp(712)
dp(2410) = dp(9)*p(713) + p(9)*dp(713)
dp(2411) = dp(9)*p(714) + p(9)*dp(714)
dp(2412) = dp(9)*p(715) + p(9)*dp(715)
dp(2413) = dp(9)*p(862) + p(9)*dp(862)
dp(2414) = dp(9)*p(716) + p(9)*dp(716)
dp(2415) = dp(9)*p(616) + p(9)*dp(616)
dp(2416) = dp(9)*p(617) + p(9)*dp(617)
dp(2417) = dp(9)*p(618) + p(9)*dp(618)
dp(2418) = dp(9)*p(619) + p(9)*dp(619)
dp(2419) = dp(9)*p(717) + p(9)*dp(717)
dp(2420) = dp(9)*p(718) + p(9)*dp(718)
dp(2421) = dp(9)*p(620) + p(9)*dp(620)
dp(2422) = dp(9)*p(621) + p(9)*dp(621)
dp(2423) = dp(9)*p(622) + p(9)*dp(622)
dp(2424) = dp(9)*p(719) + p(9)*dp(719)
dp(2425) = dp(9)*p(720) + p(9)*dp(720)
dp(2426) = dp(9)*p(623) + p(9)*dp(623)
dp(2427) = dp(9)*p(624) + p(9)*dp(624)
dp(2428) = dp(9)*p(721) + p(9)*dp(721)
dp(2429) = dp(9)*p(722) + p(9)*dp(722)
dp(2430) = dp(9)*p(625) + p(9)*dp(625)
dp(2431) = dp(9)*p(723) + p(9)*dp(723)
dp(2432) = dp(9)*p(724) + p(9)*dp(724)
dp(2433) = dp(9)*p(725) + p(9)*dp(725)
dp(2434) = dp(9)*p(863) + p(9)*dp(863)
dp(2435) = dp(14)*p(839) + p(14)*dp(839)
dp(2436) = dp(14)*p(840) + p(14)*dp(840)
dp(2437) = dp(14)*p(841) + p(14)*dp(841)
dp(2438) = dp(14)*p(842) + p(14)*dp(842)
dp(2439) = dp(14)*p(843) + p(14)*dp(843)
dp(2440) = dp(14)*p(844) + p(14)*dp(844)
dp(2441) = dp(14)*p(845) + p(14)*dp(845)
dp(2442) = dp(14)*p(846) + p(14)*dp(846)
dp(2443) = dp(14)*p(847) + p(14)*dp(847)
dp(2444) = dp(14)*p(848) + p(14)*dp(848)
dp(2445) = dp(14)*p(849) + p(14)*dp(849)
dp(2446) = dp(9)*p(728) + p(9)*dp(728)
dp(2447) = dp(9)*p(729) + p(9)*dp(729)
dp(2448) = dp(9)*p(864) + p(9)*dp(864)
dp(2449) = dp(9)*p(632) + p(9)*dp(632)
dp(2450) = dp(9)*p(633) + p(9)*dp(633)
dp(2451) = dp(9)*p(730) + p(9)*dp(730)
dp(2452) = dp(9)*p(634) + p(9)*dp(634)
dp(2453) = dp(9)*p(731) + p(9)*dp(731)
dp(2454) = dp(9)*p(635) + p(9)*dp(635)
dp(2455) = dp(9)*p(732) + p(9)*dp(732)
dp(2456) = dp(9)*p(733) + p(9)*dp(733)
dp(2457) = dp(9)*p(865) + p(9)*dp(865)
dp(2458) = dp(14)*p(692) + p(14)*dp(692)
dp(2459) = dp(14)*p(693) + p(14)*dp(693)
dp(2460) = dp(14)*p(694) + p(14)*dp(694)
dp(2461) = dp(14)*p(695) + p(14)*dp(695)
dp(2462) = dp(14)*p(696) + p(14)*dp(696)
dp(2463) = dp(14)*p(697) + p(14)*dp(697)
dp(2464) = dp(14)*p(698) + p(14)*dp(698)
dp(2465) = dp(14)*p(699) + p(14)*dp(699)
dp(2466) = dp(14)*p(700) + p(14)*dp(700)
dp(2467) = dp(14)*p(613) + p(14)*dp(613)
dp(2468) = dp(14)*p(614) + p(14)*dp(614)
dp(2469) = dp(14)*p(615) + p(14)*dp(615)
dp(2470) = dp(9)*p(874) + p(9)*dp(874)
dp(2471) = dp(9)*p(743) + p(9)*dp(743)
dp(2472) = dp(9)*p(744) + p(9)*dp(744)
dp(2473) = dp(9)*p(745) + p(9)*dp(745)
dp(2474) = dp(9)*p(746) + p(9)*dp(746)
dp(2475) = dp(9)*p(875) + p(9)*dp(875)
dp(2476) = dp(9)*p(747) + p(9)*dp(747)
dp(2477) = dp(9)*p(642) + p(9)*dp(642)
dp(2478) = dp(9)*p(643) + p(9)*dp(643)
dp(2479) = dp(9)*p(644) + p(9)*dp(644)
dp(2480) = dp(9)*p(748) + p(9)*dp(748)
dp(2481) = dp(9)*p(749) + p(9)*dp(749)
dp(2482) = dp(9)*p(645) + p(9)*dp(645)
dp(2483) = dp(9)*p(646) + p(9)*dp(646)
dp(2484) = dp(9)*p(750) + p(9)*dp(750)
dp(2485) = dp(9)*p(751) + p(9)*dp(751)
dp(2486) = dp(9)*p(647) + p(9)*dp(647)
dp(2487) = dp(9)*p(752) + p(9)*dp(752)
dp(2488) = dp(9)*p(753) + p(9)*dp(753)
dp(2489) = dp(9)*p(754) + p(9)*dp(754)
dp(2490) = dp(9)*p(876) + p(9)*dp(876)
dp(2491) = dp(14)*p(852) + p(14)*dp(852)
dp(2492) = dp(14)*p(853) + p(14)*dp(853)
dp(2493) = dp(14)*p(854) + p(14)*dp(854)
dp(2494) = dp(14)*p(855) + p(14)*dp(855)
dp(2495) = dp(14)*p(856) + p(14)*dp(856)
dp(2496) = dp(14)*p(857) + p(14)*dp(857)
dp(2497) = dp(14)*p(858) + p(14)*dp(858)
dp(2498) = dp(14)*p(859) + p(14)*dp(859)
dp(2499) = dp(14)*p(860) + p(14)*dp(860)
dp(2500) = dp(9)*p(877) + p(9)*dp(877)
dp(2501) = dp(9)*p(757) + p(9)*dp(757)
dp(2502) = dp(9)*p(758) + p(9)*dp(758)
dp(2503) = dp(9)*p(759) + p(9)*dp(759)
dp(2504) = dp(9)*p(878) + p(9)*dp(878)
dp(2505) = dp(9)*p(760) + p(9)*dp(760)
dp(2506) = dp(9)*p(653) + p(9)*dp(653)
dp(2507) = dp(9)*p(654) + p(9)*dp(654)
dp(2508) = dp(9)*p(761) + p(9)*dp(761)
dp(2509) = dp(9)*p(762) + p(9)*dp(762)
dp(2510) = dp(9)*p(655) + p(9)*dp(655)
dp(2511) = dp(9)*p(763) + p(9)*dp(763)
dp(2512) = dp(9)*p(764) + p(9)*dp(764)
dp(2513) = dp(9)*p(765) + p(9)*dp(765)
dp(2514) = dp(9)*p(879) + p(9)*dp(879)
dp(2515) = dp(14)*p(711) + p(14)*dp(711)
dp(2516) = dp(14)*p(712) + p(14)*dp(712)
dp(2517) = dp(14)*p(713) + p(14)*dp(713)
dp(2518) = dp(14)*p(714) + p(14)*dp(714)
dp(2519) = dp(14)*p(715) + p(14)*dp(715)
dp(2520) = dp(14)*p(716) + p(14)*dp(716)
dp(2521) = dp(14)*p(717) + p(14)*dp(717)
dp(2522) = dp(14)*p(718) + p(14)*dp(718)
dp(2523) = dp(14)*p(719) + p(14)*dp(719)
dp(2524) = dp(14)*p(720) + p(14)*dp(720)
dp(2525) = dp(14)*p(721) + p(14)*dp(721)
dp(2526) = dp(14)*p(722) + p(14)*dp(722)
dp(2527) = dp(14)*p(723) + p(14)*dp(723)
dp(2528) = dp(14)*p(724) + p(14)*dp(724)
dp(2529) = dp(14)*p(725) + p(14)*dp(725)
dp(2530) = dp(14)*p(626) + p(14)*dp(626)
dp(2531) = dp(14)*p(627) + p(14)*dp(627)
dp(2532) = dp(14)*p(628) + p(14)*dp(628)
dp(2533) = dp(14)*p(629) + p(14)*dp(629)
dp(2534) = dp(14)*p(630) + p(14)*dp(630)
dp(2535) = dp(14)*p(631) + p(14)*dp(631)
dp(2536) = dp(9)*p(771) + p(9)*dp(771)
dp(2537) = dp(9)*p(880) + p(9)*dp(880)
dp(2538) = dp(9)*p(664) + p(9)*dp(664)
dp(2539) = dp(9)*p(772) + p(9)*dp(772)
dp(2540) = dp(9)*p(773) + p(9)*dp(773)
dp(2541) = dp(9)*p(881) + p(9)*dp(881)
dp(2542) = dp(14)*p(728) + p(14)*dp(728)
dp(2543) = dp(14)*p(729) + p(14)*dp(729)
dp(2544) = dp(14)*p(730) + p(14)*dp(730)
dp(2545) = dp(14)*p(731) + p(14)*dp(731)
dp(2546) = dp(14)*p(732) + p(14)*dp(732)
dp(2547) = dp(14)*p(733) + p(14)*dp(733)
dp(2548) = dp(14)*p(636) + p(14)*dp(636)
dp(2549) = dp(14)*p(637) + p(14)*dp(637)
dp(2550) = dp(14)*p(638) + p(14)*dp(638)
dp(2551) = dp(14)*p(639) + p(14)*dp(639)
dp(2552) = dp(14)*p(640) + p(14)*dp(640)
dp(2553) = dp(14)*p(641) + p(14)*dp(641)
dp(2554) = dp(9)*p(889) + p(9)*dp(889)
dp(2555) = dp(9)*p(782) + p(9)*dp(782)
dp(2556) = dp(9)*p(783) + p(9)*dp(783)
dp(2557) = dp(9)*p(784) + p(9)*dp(784)
dp(2558) = dp(9)*p(890) + p(9)*dp(890)
dp(2559) = dp(9)*p(785) + p(9)*dp(785)
dp(2560) = dp(9)*p(667) + p(9)*dp(667)
dp(2561) = dp(9)*p(668) + p(9)*dp(668)
dp(2562) = dp(9)*p(786) + p(9)*dp(786)
dp(2563) = dp(9)*p(787) + p(9)*dp(787)
dp(2564) = dp(9)*p(669) + p(9)*dp(669)
dp(2565) = dp(9)*p(788) + p(9)*dp(788)
dp(2566) = dp(9)*p(789) + p(9)*dp(789)
dp(2567) = dp(9)*p(790) + p(9)*dp(790)
dp(2568) = dp(9)*p(891) + p(9)*dp(891)
dp(2569) = dp(14)*p(866) + p(14)*dp(866)
dp(2570) = dp(14)*p(867) + p(14)*dp(867)
dp(2571) = dp(14)*p(868) + p(14)*dp(868)
dp(2572) = dp(14)*p(869) + p(14)*dp(869)
dp(2573) = dp(14)*p(870) + p(14)*dp(870)
dp(2574) = dp(14)*p(871) + p(14)*dp(871)
dp(2575) = dp(14)*p(872) + p(14)*dp(872)
dp(2576) = dp(14)*p(873) + p(14)*dp(873)
dp(2577) = dp(9)*p(892) + p(9)*dp(892)
dp(2578) = dp(9)*p(793) + p(9)*dp(793)
dp(2579) = dp(9)*p(794) + p(9)*dp(794)
dp(2580) = dp(9)*p(893) + p(9)*dp(893)
dp(2581) = dp(9)*p(795) + p(9)*dp(795)
dp(2582) = dp(9)*p(673) + p(9)*dp(673)
dp(2583) = dp(9)*p(796) + p(9)*dp(796)
dp(2584) = dp(9)*p(797) + p(9)*dp(797)
dp(2585) = dp(9)*p(798) + p(9)*dp(798)
dp(2586) = dp(9)*p(894) + p(9)*dp(894)
dp(2587) = dp(14)*p(743) + p(14)*dp(743)
dp(2588) = dp(14)*p(744) + p(14)*dp(744)
dp(2589) = dp(14)*p(745) + p(14)*dp(745)
dp(2590) = dp(14)*p(746) + p(14)*dp(746)
dp(2591) = dp(14)*p(747) + p(14)*dp(747)
dp(2592) = dp(14)*p(748) + p(14)*dp(748)
dp(2593) = dp(14)*p(749) + p(14)*dp(749)
dp(2594) = dp(14)*p(750) + p(14)*dp(750)
dp(2595) = dp(14)*p(751) + p(14)*dp(751)
dp(2596) = dp(14)*p(752) + p(14)*dp(752)
dp(2597) = dp(14)*p(753) + p(14)*dp(753)
dp(2598) = dp(14)*p(754) + p(14)*dp(754)
dp(2599) = dp(14)*p(648) + p(14)*dp(648)
dp(2600) = dp(14)*p(649) + p(14)*dp(649)
dp(2601) = dp(14)*p(650) + p(14)*dp(650)
dp(2602) = dp(14)*p(651) + p(14)*dp(651)
dp(2603) = dp(14)*p(652) + p(14)*dp(652)
dp(2604) = dp(9)*p(895) + p(9)*dp(895)
dp(2605) = dp(9)*p(804) + p(9)*dp(804)
dp(2606) = dp(9)*p(896) + p(9)*dp(896)
dp(2607) = dp(9)*p(805) + p(9)*dp(805)
dp(2608) = dp(9)*p(806) + p(9)*dp(806)
dp(2609) = dp(9)*p(897) + p(9)*dp(897)
dp(2610) = dp(14)*p(757) + p(14)*dp(757)
dp(2611) = dp(14)*p(758) + p(14)*dp(758)
dp(2612) = dp(14)*p(759) + p(14)*dp(759)
dp(2613) = dp(14)*p(760) + p(14)*dp(760)
dp(2614) = dp(14)*p(761) + p(14)*dp(761)
dp(2615) = dp(14)*p(762) + p(14)*dp(762)
dp(2616) = dp(14)*p(763) + p(14)*dp(763)
dp(2617) = dp(14)*p(764) + p(14)*dp(764)
dp(2618) = dp(14)*p(765) + p(14)*dp(765)
dp(2619) = dp(14)*p(656) + p(14)*dp(656)
dp(2620) = dp(14)*p(657) + p(14)*dp(657)
dp(2621) = dp(14)*p(658) + p(14)*dp(658)
dp(2622) = dp(14)*p(659) + p(14)*dp(659)
dp(2623) = dp(14)*p(660) + p(14)*dp(660)
dp(2624) = dp(14)*p(661) + p(14)*dp(661)
dp(2625) = dp(14)*p(662) + p(14)*dp(662)
dp(2626) = dp(14)*p(663) + p(14)*dp(663)
dp(2627) = dp(9)*p(899) + p(9)*dp(899)
dp(2628) = dp(9)*p(900) + p(9)*dp(900)
dp(2629) = dp(14)*p(771) + p(14)*dp(771)
dp(2630) = dp(14)*p(772) + p(14)*dp(772)
dp(2631) = dp(14)*p(773) + p(14)*dp(773)
dp(2632) = dp(14)*p(665) + p(14)*dp(665)
dp(2633) = dp(14)*p(666) + p(14)*dp(666)
dp(2634) = dp(9)*p(907) + p(9)*dp(907)
dp(2635) = dp(9)*p(812) + p(9)*dp(812)
dp(2636) = dp(9)*p(813) + p(9)*dp(813)
dp(2637) = dp(9)*p(908) + p(9)*dp(908)
dp(2638) = dp(9)*p(814) + p(9)*dp(814)
dp(2639) = dp(9)*p(677) + p(9)*dp(677)
dp(2640) = dp(9)*p(815) + p(9)*dp(815)
dp(2641) = dp(9)*p(816) + p(9)*dp(816)
dp(2642) = dp(9)*p(817) + p(9)*dp(817)
dp(2643) = dp(9)*p(909) + p(9)*dp(909)
dp(2644) = dp(14)*p(883) + p(14)*dp(883)
dp(2645) = dp(14)*p(884) + p(14)*dp(884)
dp(2646) = dp(14)*p(885) + p(14)*dp(885)
dp(2647) = dp(14)*p(886) + p(14)*dp(886)
dp(2648) = dp(14)*p(887) + p(14)*dp(887)
dp(2649) = dp(14)*p(888) + p(14)*dp(888)
dp(2650) = dp(9)*p(910) + p(9)*dp(910)
dp(2651) = dp(9)*p(820) + p(9)*dp(820)
dp(2652) = dp(9)*p(911) + p(9)*dp(911)
dp(2653) = dp(9)*p(821) + p(9)*dp(821)
dp(2654) = dp(9)*p(822) + p(9)*dp(822)
dp(2655) = dp(9)*p(912) + p(9)*dp(912)
dp(2656) = dp(14)*p(782) + p(14)*dp(782)
dp(2657) = dp(14)*p(783) + p(14)*dp(783)
dp(2658) = dp(14)*p(784) + p(14)*dp(784)
dp(2659) = dp(14)*p(785) + p(14)*dp(785)
dp(2660) = dp(14)*p(786) + p(14)*dp(786)
dp(2661) = dp(14)*p(787) + p(14)*dp(787)
dp(2662) = dp(14)*p(788) + p(14)*dp(788)
dp(2663) = dp(14)*p(789) + p(14)*dp(789)
dp(2664) = dp(14)*p(790) + p(14)*dp(790)
dp(2665) = dp(14)*p(670) + p(14)*dp(670)
dp(2666) = dp(14)*p(671) + p(14)*dp(671)
dp(2667) = dp(14)*p(672) + p(14)*dp(672)
dp(2668) = dp(9)*p(914) + p(9)*dp(914)
dp(2669) = dp(9)*p(915) + p(9)*dp(915)
dp(2670) = dp(9)*p(916) + p(9)*dp(916)
dp(2671) = dp(14)*p(793) + p(14)*dp(793)
dp(2672) = dp(14)*p(794) + p(14)*dp(794)
dp(2673) = dp(14)*p(795) + p(14)*dp(795)
dp(2674) = dp(14)*p(796) + p(14)*dp(796)
dp(2675) = dp(14)*p(797) + p(14)*dp(797)
dp(2676) = dp(14)*p(798) + p(14)*dp(798)
dp(2677) = dp(14)*p(674) + p(14)*dp(674)
dp(2678) = dp(14)*p(675) + p(14)*dp(675)
dp(2679) = dp(14)*p(676) + p(14)*dp(676)
dp(2680) = dp(9)*p(948) + p(9)*dp(948)
dp(2681) = dp(14)*p(804) + p(14)*dp(804)
dp(2682) = dp(14)*p(805) + p(14)*dp(805)
dp(2683) = dp(14)*p(806) + p(14)*dp(806)
dp(2684) = dp(9)*p(921) + p(9)*dp(921)
dp(2685) = dp(9)*p(827) + p(9)*dp(827)
dp(2686) = dp(9)*p(922) + p(9)*dp(922)
dp(2687) = dp(9)*p(828) + p(9)*dp(828)
dp(2688) = dp(9)*p(829) + p(9)*dp(829)
dp(2689) = dp(9)*p(923) + p(9)*dp(923)
dp(2690) = dp(14)*p(902) + p(14)*dp(902)
dp(2691) = dp(14)*p(903) + p(14)*dp(903)
dp(2692) = dp(14)*p(904) + p(14)*dp(904)
dp(2693) = dp(14)*p(905) + p(14)*dp(905)
dp(2694) = dp(14)*p(906) + p(14)*dp(906)
dp(2695) = dp(9)*p(924) + p(9)*dp(924)
dp(2696) = dp(9)*p(925) + p(9)*dp(925)
dp(2697) = dp(9)*p(926) + p(9)*dp(926)
dp(2698) = dp(14)*p(812) + p(14)*dp(812)
dp(2699) = dp(14)*p(813) + p(14)*dp(813)
dp(2700) = dp(14)*p(814) + p(14)*dp(814)
dp(2701) = dp(14)*p(815) + p(14)*dp(815)
dp(2702) = dp(14)*p(816) + p(14)*dp(816)
dp(2703) = dp(14)*p(817) + p(14)*dp(817)
dp(2704) = dp(14)*p(678) + p(14)*dp(678)
dp(2705) = dp(14)*p(679) + p(14)*dp(679)
dp(2706) = dp(9)*p(951) + p(9)*dp(951)
dp(2707) = dp(14)*p(820) + p(14)*dp(820)
dp(2708) = dp(14)*p(821) + p(14)*dp(821)
dp(2709) = dp(14)*p(822) + p(14)*dp(822)
dp(2710) = dp(9)*p(930) + p(9)*dp(930)
dp(2711) = dp(9)*p(931) + p(9)*dp(931)
dp(2712) = dp(9)*p(932) + p(9)*dp(932)
dp(2713) = dp(14)*p(918) + p(14)*dp(918)
dp(2714) = dp(14)*p(919) + p(14)*dp(919)
dp(2715) = dp(14)*p(920) + p(14)*dp(920)
dp(2716) = dp(9)*p(954) + p(9)*dp(954)
dp(2717) = dp(14)*p(827) + p(14)*dp(827)
dp(2718) = dp(14)*p(828) + p(14)*dp(828)
dp(2719) = dp(14)*p(829) + p(14)*dp(829)
dp(2720) = dp(9)*p(957) + p(9)*dp(957)
dp(2721) = dp(14)*p(928) + p(14)*dp(928)
dp(2722) = dp(14)*p(929) + p(14)*dp(929)
dp(2723) = dp(9)*p(938) + p(9)*dp(938)
dp(2724) = dp(9)*p(839) + p(9)*dp(839)
dp(2725) = dp(9)*p(680) + p(9)*dp(680)
dp(2726) = dp(9)*p(840) + p(9)*dp(840)
dp(2727) = dp(9)*p(681) + p(9)*dp(681)
dp(2728) = dp(9)*p(682) + p(9)*dp(682)
dp(2729) = dp(9)*p(841) + p(9)*dp(841)
dp(2730) = dp(9)*p(683) + p(9)*dp(683)
dp(2731) = dp(9)*p(684) + p(9)*dp(684)
dp(2732) = dp(9)*p(685) + p(9)*dp(685)
dp(2733) = dp(9)*p(842) + p(9)*dp(842)
dp(2734) = dp(9)*p(686) + p(9)*dp(686)
dp(2735) = dp(9)*p(687) + p(9)*dp(687)
dp(2736) = dp(9)*p(688) + p(9)*dp(688)
dp(2737) = dp(9)*p(843) + p(9)*dp(843)
dp(2738) = dp(9)*p(844) + p(9)*dp(844)
dp(2739) = dp(9)*p(689) + p(9)*dp(689)
dp(2740) = dp(9)*p(690) + p(9)*dp(690)
dp(2741) = dp(9)*p(845) + p(9)*dp(845)
dp(2742) = dp(9)*p(846) + p(9)*dp(846)
dp(2743) = dp(9)*p(691) + p(9)*dp(691)
dp(2744) = dp(9)*p(847) + p(9)*dp(847)
dp(2745) = dp(9)*p(848) + p(9)*dp(848)
dp(2746) = dp(9)*p(849) + p(9)*dp(849)
dp(2747) = dp(9)*p(939) + p(9)*dp(939)
dp(2748) = dp(3)*p(1136) + p(3)*dp(1136) - dp(2383)
dp(2749) = dp(7)*p(1136) + p(7)*dp(1136) - dp(2387)
dp(2750) = dp(49)*p(401) + p(49)*dp(401) - dp(2394)
dp(2751) = dp(114)*p(194) + p(114)*dp(194) - dp(2399)
dp(2752) = dp(24)*p(588) + p(24)*dp(588) - dp(2402)
dp(2753) = dp(3)*p(1368) + p(3)*dp(1368) - dp(2386) - dp(2752)
dp(2754) = dp(1)*p(1802) + p(1)*dp(1802) - dp(2386) - dp(2753)
dp(2755) = dp(1)*p(1803) + p(1)*dp(1803) - dp(2390) - dp(2754)
dp(2756) = dp(1)*p(1804) + p(1)*dp(1804) - dp(2393) - dp(2755)
dp(2757) = dp(6)*p(1476) + p(6)*dp(1476) - dp(2748)
dp(2758) = dp(32)*p(566) + p(32)*dp(566) - dp(2387)
dp(2759) = dp(150)*p(178) + p(150)*dp(178) - dp(2394)
dp(2760) = dp(39)*p(505) + p(39)*dp(505) - dp(2399)
dp(2761) = dp(3)*p(1369) + p(3)*dp(1369) - dp(2402)
dp(2762) = dp(14)*p(958) + p(14)*dp(958)
dp(2763) = dp(9)*p(940) + p(9)*dp(940)
dp(2764) = dp(9)*p(852) + p(9)*dp(852)
dp(2765) = dp(9)*p(702) + p(9)*dp(702)
dp(2766) = dp(9)*p(853) + p(9)*dp(853)
dp(2767) = dp(9)*p(703) + p(9)*dp(703)
dp(2768) = dp(9)*p(704) + p(9)*dp(704)
dp(2769) = dp(9)*p(854) + p(9)*dp(854)
dp(2770) = dp(9)*p(705) + p(9)*dp(705)
dp(2771) = dp(9)*p(706) + p(9)*dp(706)
dp(2772) = dp(9)*p(707) + p(9)*dp(707)
dp(2773) = dp(9)*p(855) + p(9)*dp(855)
dp(2774) = dp(9)*p(708) + p(9)*dp(708)
dp(2775) = dp(9)*p(709) + p(9)*dp(709)
dp(2776) = dp(9)*p(856) + p(9)*dp(856)
dp(2777) = dp(9)*p(857) + p(9)*dp(857)
dp(2778) = dp(9)*p(710) + p(9)*dp(710)
dp(2779) = dp(9)*p(858) + p(9)*dp(858)
dp(2780) = dp(9)*p(859) + p(9)*dp(859)
dp(2781) = dp(9)*p(860) + p(9)*dp(860)
dp(2782) = dp(9)*p(941) + p(9)*dp(941)
dp(2783) = dp(24)*p(570) + p(24)*dp(570) - dp(2296)
dp(2784) = dp(114)*p(181) + p(114)*dp(181) - dp(2293)
dp(2785) = dp(1)*p(1828) + p(1)*dp(1828) - dp(2408) - dp(2784)
dp(2786) = dp(4)*p(1136) + p(4)*dp(1136) - dp(2292)
dp(2787) = dp(1)*p(1830) + p(1)*dp(1830) - dp(2410) - dp(2786)
dp(2788) = dp(1)*p(1831) + p(1)*dp(1831) - dp(2411) - dp(2787)
dp(2789) = dp(1)*p(1832) + p(1)*dp(1832) - dp(2412) - dp(2788)
dp(2790) = dp(1)*p(1833) + p(1)*dp(1833) - dp(2413) - dp(2789)
dp(2791) = dp(13)*p(938) + p(13)*dp(938) - dp(2762)
dp(2792) = dp(2)*p(1802) + p(2)*dp(1802) - dp(2406) - dp(2791)
dp(2793) = dp(32)*p(570) + p(32)*dp(570) - dp(2762)
dp(2794) = dp(1)*p(1835) + p(1)*dp(1835) - dp(2413) - dp(2792)
dp(2795) = dp(73)*p(331) + p(73)*dp(331) - dp(2762)
dp(2796) = dp(1)*p(1837) + p(1)*dp(1837) - dp(2419) - dp(2794)
dp(2797) = dp(150)*p(181) + p(150)*dp(181) - dp(2762)
dp(2798) = dp(1)*p(1839) + p(1)*dp(1839) - dp(2424) - dp(2796)
dp(2799) = dp(91)*p(285) + p(91)*dp(285) - dp(2762)
dp(2800) = dp(13)*p(843) + p(13)*dp(843) - dp(2292)
dp(2801) = dp(41)*p(505) + p(41)*dp(505) - dp(2762)
dp(2802) = dp(1)*p(1843) + p(1)*dp(1843) - dp(2431) - dp(2800)
dp(2803) = dp(16)*p(851) + p(16)*dp(851) - dp(2762)
dp(2804) = dp(69)*p(285) + p(69)*dp(285) - dp(2293)
dp(2805) = dp(4)*p(1369) + p(4)*dp(1369) - dp(2762)
dp(2806) = dp(11)*p(851) + p(11)*dp(851) - dp(2296)
dp(2807) = dp(14)*p(938) + p(14)*dp(938)
dp(2808) = dp(14)*p(939) + p(14)*dp(939)
dp(2809) = dp(3)*p(1180) + p(3)*dp(1180) - dp(2446)
dp(2810) = dp(7)*p(1180) + p(7)*dp(1180) - dp(2449)
dp(2811) = dp(49)*p(430) + p(49)*dp(430) - dp(2454)
dp(2812) = dp(130)*p(194) + p(130)*dp(194) - dp(2457)
dp(2813) = dp(3)*p(1384) + p(3)*dp(1384) - dp(2448) - dp(2812)
dp(2814) = dp(1)*p(1853) + p(1)*dp(1853) - dp(2448) - dp(2813)
dp(2815) = dp(1)*p(1854) + p(1)*dp(1854) - dp(2451) - dp(2814)
dp(2816) = dp(45)*p(566) + p(45)*dp(566) - dp(2809)
dp(2817) = dp(160)*p(178) + p(160)*dp(178) - dp(2449)
dp(2818) = dp(39)*p(518) + p(39)*dp(518) - dp(2454)
dp(2819) = dp(3)*p(1385) + p(3)*dp(1385) - dp(2457)
dp(2820) = dp(14)*p(850) + p(14)*dp(850)
dp(2821) = dp(14)*p(851) + p(14)*dp(851)
dp(2822) = dp(14)*p(701) + p(14)*dp(701)
dp(2823) = dp(9)*p(942) + p(9)*dp(942)
dp(2824) = dp(9)*p(866) + p(9)*dp(866)
dp(2825) = dp(9)*p(737) + p(9)*dp(737)
dp(2826) = dp(9)*p(867) + p(9)*dp(867)
dp(2827) = dp(9)*p(738) + p(9)*dp(738)
dp(2828) = dp(9)*p(739) + p(9)*dp(739)
dp(2829) = dp(9)*p(868) + p(9)*dp(868)
dp(2830) = dp(9)*p(740) + p(9)*dp(740)
dp(2831) = dp(9)*p(741) + p(9)*dp(741)
dp(2832) = dp(9)*p(869) + p(9)*dp(869)
dp(2833) = dp(9)*p(870) + p(9)*dp(870)
dp(2834) = dp(9)*p(742) + p(9)*dp(742)
dp(2835) = dp(9)*p(871) + p(9)*dp(871)
dp(2836) = dp(9)*p(872) + p(9)*dp(872)
dp(2837) = dp(9)*p(873) + p(9)*dp(873)
dp(2838) = dp(9)*p(943) + p(9)*dp(943)
dp(2839) = dp(24)*p(572) + p(24)*dp(572) - dp(2297)
dp(2840) = dp(114)*p(183) + p(114)*dp(183) - dp(2299)
dp(2841) = dp(43)*p(401) + p(43)*dp(401) - dp(2302)
dp(2842) = dp(8)*p(1136) + p(8)*dp(1136) - dp(2439)
dp(2843) = dp(1)*p(1877) + p(1)*dp(1877) - dp(2473) - dp(2842)
dp(2844) = dp(1)*p(1878) + p(1)*dp(1878) - dp(2474) - dp(2843)
dp(2845) = dp(1)*p(1879) + p(1)*dp(1879) - dp(2475) - dp(2844)
dp(2846) = dp(13)*p(940) + p(13)*dp(940) - dp(2807)
dp(2847) = dp(2)*p(1835) + p(2)*dp(1835) - dp(2445) - dp(2813)
dp(2848) = dp(32)*p(572) + p(32)*dp(572) - dp(2807)
dp(2849) = dp(1)*p(1881) + p(1)*dp(1881) - dp(2475) - dp(2847)
dp(2850) = dp(73)*p(333) + p(73)*dp(333) - dp(2807)
dp(2851) = dp(1)*p(1883) + p(1)*dp(1883) - dp(2480) - dp(2849)
dp(2852) = dp(150)*p(183) + p(150)*dp(183) - dp(2807)
dp(2853) = dp(6)*p(1374) + p(6)*dp(1374) - dp(2439)
dp(2854) = dp(93)*p(285) + p(93)*dp(285) - dp(2807)
dp(2855) = dp(32)*p(509) + p(32)*dp(509) - dp(2302)
dp(2856) = dp(43)*p(505) + p(43)*dp(505) - dp(2807)
dp(2857) = dp(150)*p(153) + p(150)*dp(153) - dp(2299)
dp(2858) = dp(18)*p(851) + p(18)*dp(851) - dp(2807)
dp(2859) = dp(34)*p(505) + p(34)*dp(505) - dp(2297)
dp(2860) = dp(14)*p(940) + p(14)*dp(940)
dp(2861) = dp(14)*p(941) + p(14)*dp(941)
dp(2862) = dp(130)*p(181) + p(130)*dp(181) - dp(2317)
dp(2863) = dp(41)*p(430) + p(41)*dp(430) - dp(2314)
dp(2864) = dp(4)*p(1180) + p(4)*dp(1180) - dp(2309)
dp(2865) = dp(1)*p(1896) + p(1)*dp(1896) - dp(2502) - dp(2864)
dp(2866) = dp(1)*p(1897) + p(1)*dp(1897) - dp(2503) - dp(2865)
dp(2867) = dp(1)*p(1898) + p(1)*dp(1898) - dp(2504) - dp(2866)
dp(2868) = dp(81)*p(331) + p(81)*dp(331) - dp(2821)
dp(2869) = dp(2)*p(1853) + p(2)*dp(1853) - dp(2461) - dp(2868)
dp(2870) = dp(160)*p(181) + p(160)*dp(181) - dp(2821)
dp(2871) = dp(1)*p(1900) + p(1)*dp(1900) - dp(2504) - dp(2869)
dp(2872) = dp(91)*p(296) + p(91)*dp(296) - dp(2821)
dp(2873) = dp(1)*p(1902) + p(1)*dp(1902) - dp(2508) - dp(2871)
dp(2874) = dp(41)*p(518) + p(41)*dp(518) - dp(2821)
dp(2875) = dp(81)*p(279) + p(81)*dp(279) - dp(2309)
dp(2876) = dp(16)*p(865) + p(16)*dp(865) - dp(2821)
dp(2877) = dp(69)*p(296) + p(69)*dp(296) - dp(2314)
dp(2878) = dp(4)*p(1385) + p(4)*dp(1385) - dp(2821)
dp(2879) = dp(11)*p(865) + p(11)*dp(865) - dp(2317)
dp(2880) = dp(14)*p(861) + p(14)*dp(861)
dp(2881) = dp(14)*p(862) + p(14)*dp(862)
dp(2882) = dp(14)*p(863) + p(14)*dp(863)
dp(2883) = dp(14)*p(726) + p(14)*dp(726)
dp(2884) = dp(14)*p(727) + p(14)*dp(727)
dp(2885) = dp(3)*p(1234) + p(3)*dp(1234) - dp(2536)
dp(2886) = dp(7)*p(1234) + p(7)*dp(1234) - dp(2538)
dp(2887) = dp(49)*p(464) + p(49)*dp(464) - dp(2541)
dp(2888) = dp(3)*p(1401) + p(3)*dp(1401) - dp(2537) - dp(2887)
dp(2889) = dp(1)*p(1916) + p(1)*dp(1916) - dp(2537) - dp(2888)
dp(2890) = dp(178)*p(187) + p(178)*dp(187) - dp(2885)
dp(2891) = dp(39)*p(532) + p(39)*dp(532) - dp(2538)
dp(2892) = dp(3)*p(1402) + p(3)*dp(1402) - dp(2541)
dp(2893) = dp(14)*p(864) + p(14)*dp(864)
dp(2894) = dp(14)*p(865) + p(14)*dp(865)
dp(2895) = dp(14)*p(734) + p(14)*dp(734)
dp(2896) = dp(14)*p(735) + p(14)*dp(735)
dp(2897) = dp(14)*p(736) + p(14)*dp(736)
dp(2898) = dp(9)*p(944) + p(9)*dp(944)
dp(2899) = dp(9)*p(883) + p(9)*dp(883)
dp(2900) = dp(9)*p(778) + p(9)*dp(778)
dp(2901) = dp(9)*p(884) + p(9)*dp(884)
dp(2902) = dp(9)*p(779) + p(9)*dp(779)
dp(2903) = dp(9)*p(780) + p(9)*dp(780)
dp(2904) = dp(9)*p(885) + p(9)*dp(885)
dp(2905) = dp(9)*p(781) + p(9)*dp(781)
dp(2906) = dp(9)*p(886) + p(9)*dp(886)
dp(2907) = dp(9)*p(887) + p(9)*dp(887)
dp(2908) = dp(9)*p(888) + p(9)*dp(888)
dp(2909) = dp(9)*p(945) + p(9)*dp(945)
dp(2910) = dp(24)*p(574) + p(24)*dp(574) - dp(2321)
dp(2911) = dp(114)*p(185) + p(114)*dp(185) - dp(2323)
dp(2912) = dp(46)*p(401) + p(46)*dp(401) - dp(2326)
dp(2913) = dp(1)*p(1936) + p(1)*dp(1936) - dp(2556) - dp(2912)
dp(2914) = dp(1)*p(1937) + p(1)*dp(1937) - dp(2557) - dp(2913)
dp(2915) = dp(1)*p(1938) + p(1)*dp(1938) - dp(2558) - dp(2914)
dp(2916) = dp(13)*p(942) + p(13)*dp(942) - dp(2860)
dp(2917) = dp(2)*p(1881) + p(2)*dp(1881) - dp(2499) - dp(2869)
dp(2918) = dp(32)*p(574) + p(32)*dp(574) - dp(2860)
dp(2919) = dp(1)*p(1940) + p(1)*dp(1940) - dp(2558) - dp(2917)
dp(2920) = dp(73)*p(335) + p(73)*dp(335) - dp(2860)
dp(2921) = dp(1)*p(1942) + p(1)*dp(1942) - dp(2562) - dp(2919)
dp(2922) = dp(150)*p(185) + p(150)*dp(185) - dp(2860)
dp(2923) = dp(13)*p(869) + p(13)*dp(869) - dp(2326)
dp(2924) = dp(95)*p(285) + p(95)*dp(285) - dp(2860)
dp(2925) = dp(73)*p(299) + p(73)*dp(299) - dp(2323)
dp(2926) = dp(46)*p(505) + p(46)*dp(505) - dp(2860)
dp(2927) = dp(84)*p(285) + p(84)*dp(285) - dp(2321)
dp(2928) = dp(14)*p(942) + p(14)*dp(942)
dp(2929) = dp(14)*p(943) + p(14)*dp(943)
dp(2930) = dp(130)*p(183) + p(130)*dp(183) - dp(2320)
dp(2931) = dp(1)*p(1951) + p(1)*dp(1951) - dp(2577) - dp(2930)
dp(2932) = dp(8)*p(1180) + p(8)*dp(1180) - dp(2467)
dp(2933) = dp(1)*p(1953) + p(1)*dp(1953) - dp(2579) - dp(2932)
dp(2934) = dp(1)*p(1954) + p(1)*dp(1954) - dp(2580) - dp(2933)
dp(2935) = dp(81)*p(333) + p(81)*dp(333) - dp(2822)
dp(2936) = dp(2)*p(1900) + p(2)*dp(1900) - dp(2521) - dp(2888)
dp(2937) = dp(160)*p(183) + p(160)*dp(183) - dp(2822)
dp(2938) = dp(1)*p(1956) + p(1)*dp(1956) - dp(2580) - dp(2936)
dp(2939) = dp(93)*p(296) + p(93)*dp(296) - dp(2822)
dp(2940) = dp(45)*p(509) + p(45)*dp(509) - dp(2467)
dp(2941) = dp(43)*p(518) + p(43)*dp(518) - dp(2822)
dp(2942) = dp(1)*p(1960) + p(1)*dp(1960) - dp(2585) - dp(2940)
dp(2943) = dp(18)*p(865) + p(18)*dp(865) - dp(2822)
dp(2944) = dp(34)*p(518) + p(34)*dp(518) - dp(2320)
dp(2945) = dp(14)*p(874) + p(14)*dp(874)
dp(2946) = dp(14)*p(875) + p(14)*dp(875)
dp(2947) = dp(14)*p(876) + p(14)*dp(876)
dp(2948) = dp(14)*p(755) + p(14)*dp(755)
dp(2949) = dp(14)*p(756) + p(14)*dp(756)
dp(2950) = dp(41)*p(464) + p(41)*dp(464) - dp(2347)
dp(2951) = dp(4)*p(1234) + p(4)*dp(1234) - dp(2344)
dp(2952) = dp(1)*p(1969) + p(1)*dp(1969) - dp(2605) - dp(2951)
dp(2953) = dp(1)*p(1970) + p(1)*dp(1970) - dp(2606) - dp(2952)
dp(2954) = dp(91)*p(310) + p(91)*dp(310) - dp(2894)
dp(2955) = dp(2)*p(1916) + p(2)*dp(1916) - dp(2544) - dp(2954)
dp(2956) = dp(41)*p(532) + p(41)*dp(532) - dp(2894)
dp(2957) = dp(1)*p(1972) + p(1)*dp(1972) - dp(2606) - dp(2955)
dp(2958) = dp(16)*p(881) + p(16)*dp(881) - dp(2894)
dp(2959) = dp(69)*p(310) + p(69)*dp(310) - dp(2344)
dp(2960) = dp(4)*p(1402) + p(4)*dp(1402) - dp(2894)
dp(2961) = dp(11)*p(881) + p(11)*dp(881) - dp(2347)
dp(2962) = dp(14)*p(877) + p(14)*dp(877)
dp(2963) = dp(14)*p(878) + p(14)*dp(878)
dp(2964) = dp(14)*p(879) + p(14)*dp(879)
dp(2965) = dp(14)*p(766) + p(14)*dp(766)
dp(2966) = dp(14)*p(767) + p(14)*dp(767)
dp(2967) = dp(14)*p(768) + p(14)*dp(768)
dp(2968) = dp(14)*p(769) + p(14)*dp(769)
dp(2969) = dp(14)*p(770) + p(14)*dp(770)
dp(2970) = dp(3)*p(1290) + p(3)*dp(1290) - dp(2627)
dp(2971) = dp(7)*p(1290) + p(7)*dp(1290) - dp(2628)
dp(2972) = dp(3)*p(1421) + p(3)*dp(1421) - dp(2627) - dp(2971)
dp(2973) = dp(39)*p(578) + p(39)*dp(578) - dp(2970)
dp(2974) = dp(3)*p(1422) + p(3)*dp(1422) - dp(2628)
dp(2975) = dp(14)*p(880) + p(14)*dp(880)
dp(2976) = dp(14)*p(881) + p(14)*dp(881)
dp(2977) = dp(14)*p(774) + p(14)*dp(774)
dp(2978) = dp(14)*p(775) + p(14)*dp(775)
dp(2979) = dp(14)*p(776) + p(14)*dp(776)
dp(2980) = dp(14)*p(777) + p(14)*dp(777)
dp(2981) = dp(9)*p(946) + p(9)*dp(946)
dp(2982) = dp(9)*p(902) + p(9)*dp(902)
dp(2983) = dp(9)*p(810) + p(9)*dp(810)
dp(2984) = dp(9)*p(903) + p(9)*dp(903)
dp(2985) = dp(9)*p(811) + p(9)*dp(811)
dp(2986) = dp(9)*p(904) + p(9)*dp(904)
dp(2987) = dp(9)*p(905) + p(9)*dp(905)
dp(2988) = dp(9)*p(906) + p(9)*dp(906)
dp(2989) = dp(9)*p(947) + p(9)*dp(947)
dp(2990) = dp(24)*p(576) + p(24)*dp(576) - dp(2351)
dp(2991) = dp(114)*p(188) + p(114)*dp(188) - dp(2353)
dp(2992) = dp(50)*p(401) + p(50)*dp(401) - dp(2572)
dp(2993) = dp(1)*p(2000) + p(1)*dp(2000) - dp(2636) - dp(2992)
dp(2994) = dp(1)*p(2001) + p(1)*dp(2001) - dp(2637) - dp(2993)
dp(2995) = dp(13)*p(944) + p(13)*dp(944) - dp(2928)
dp(2996) = dp(2)*p(1940) + p(2)*dp(1940) - dp(2576) - dp(2936)
dp(2997) = dp(32)*p(576) + p(32)*dp(576) - dp(2928)
dp(2998) = dp(1)*p(2003) + p(1)*dp(2003) - dp(2637) - dp(2996)
dp(2999) = dp(73)*p(337) + p(73)*dp(337) - dp(2928)
dp(3000) = dp(6)*p(1406) + p(6)*dp(1406) - dp(2572)
dp(3001) = dp(150)*p(188) + p(150)*dp(188) - dp(2928)
dp(3002) = dp(32)*p(536) + p(32)*dp(536) - dp(2353)
dp(3003) = dp(98)*p(285) + p(98)*dp(285) - dp(2928)
dp(3004) = dp(150)*p(173) + p(150)*dp(173) - dp(2351)
dp(3005) = dp(14)*p(944) + p(14)*dp(944)
dp(3006) = dp(14)*p(945) + p(14)*dp(945)
dp(3007) = dp(130)*p(185) + p(130)*dp(185) - dp(2340)
dp(3008) = dp(46)*p(430) + p(46)*dp(430) - dp(2342)
dp(3009) = dp(1)*p(2013) + p(1)*dp(2013) - dp(2651) - dp(3008)
dp(3010) = dp(1)*p(2014) + p(1)*dp(2014) - dp(2652) - dp(3009)
dp(3011) = dp(81)*p(335) + p(81)*dp(335) - dp(2883)
dp(3012) = dp(2)*p(1956) + p(2)*dp(1956) - dp(2592) - dp(2955)
dp(3013) = dp(160)*p(185) + p(160)*dp(185) - dp(2883)
dp(3014) = dp(1)*p(2016) + p(1)*dp(2016) - dp(2652) - dp(3012)
dp(3015) = dp(95)*p(296) + p(95)*dp(296) - dp(2883)
dp(3016) = dp(81)*p(299) + p(81)*dp(299) - dp(2342)
dp(3017) = dp(46)*p(518) + p(46)*dp(518) - dp(2883)
dp(3018) = dp(84)*p(296) + p(84)*dp(296) - dp(2340)
dp(3019) = dp(14)*p(889) + p(14)*dp(889)
dp(3020) = dp(14)*p(890) + p(14)*dp(890)
dp(3021) = dp(14)*p(891) + p(14)*dp(891)
dp(3022) = dp(14)*p(791) + p(14)*dp(791)
dp(3023) = dp(14)*p(792) + p(14)*dp(792)
dp(3024) = dp(43)*p(464) + p(43)*dp(464) - dp(2349)
dp(3025) = dp(8)*p(1234) + p(8)*dp(1234) - dp(2548)
dp(3026) = dp(1)*p(2027) + p(1)*dp(2027) - dp(2669) - dp(3025)
dp(3027) = dp(93)*p(310) + p(93)*dp(310) - dp(2896)
dp(3028) = dp(2)*p(1972) + p(2)*dp(1972) - dp(2614) - dp(2972)
dp(3029) = dp(43)*p(532) + p(43)*dp(532) - dp(2896)
dp(3030) = dp(153)*p(187) + p(153)*dp(187) - dp(2548)
dp(3031) = dp(18)*p(881) + p(18)*dp(881) - dp(2896)
dp(3032) = dp(34)*p(532) + p(34)*dp(532) - dp(2349)
dp(3033) = dp(14)*p(892) + p(14)*dp(892)
dp(3034) = dp(14)*p(893) + p(14)*dp(893)
dp(3035) = dp(14)*p(894) + p(14)*dp(894)
dp(3036) = dp(14)*p(799) + p(14)*dp(799)
dp(3037) = dp(14)*p(800) + p(14)*dp(800)
dp(3038) = dp(14)*p(801) + p(14)*dp(801)
dp(3039) = dp(14)*p(802) + p(14)*dp(802)
dp(3040) = dp(14)*p(803) + p(14)*dp(803)
dp(3041) = dp(4)*p(1290) + p(4)*dp(1290) - dp(2369)
dp(3042) = dp(1)*p(2038) + p(1)*dp(2038) - dp(2680) - dp(3041)
dp(3043) = dp(16)*p(900) + p(16)*dp(900) - dp(2976)
dp(3044) = dp(2)*p(1986) + p(2)*dp(1986) - dp(2630) - dp(3043)
dp(3045) = dp(4)*p(1422) + p(4)*dp(1422) - dp(2976)
dp(3046) = dp(11)*p(900) + p(11)*dp(900) - dp(2369)
dp(3047) = dp(14)*p(895) + p(14)*dp(895)
dp(3048) = dp(14)*p(896) + p(14)*dp(896)
dp(3049) = dp(14)*p(897) + p(14)*dp(897)
dp(3050) = dp(14)*p(807) + p(14)*dp(807)
dp(3051) = dp(14)*p(808) + p(14)*dp(808)
dp(3052) = dp(14)*p(809) + p(14)*dp(809)
dp(3053) = dp(6)*p(1290) + p(6)*dp(1290) - dp(2681)
dp(3054) = dp(3)*p(1491) + p(3)*dp(1491) - dp(3053)
dp(3055) = dp(14)*p(899) + p(14)*dp(899)
dp(3056) = dp(14)*p(900) + p(14)*dp(900)
dp(3057) = dp(9)*p(949) + p(9)*dp(949)
dp(3058) = dp(9)*p(918) + p(9)*dp(918)
dp(3059) = dp(9)*p(826) + p(9)*dp(826)
dp(3060) = dp(9)*p(919) + p(9)*dp(919)
dp(3061) = dp(9)*p(920) + p(9)*dp(920)
dp(3062) = dp(9)*p(950) + p(9)*dp(950)
dp(3063) = dp(24)*p(579) + p(24)*dp(579) - dp(2370)
dp(3064) = dp(114)*p(191) + p(114)*dp(191) - dp(2372)
dp(3065) = dp(1)*p(2049) + p(1)*dp(2049) - dp(2685) - dp(3064)
dp(3066) = dp(1)*p(2050) + p(1)*dp(2050) - dp(2686) - dp(3065)
dp(3067) = dp(13)*p(946) + p(13)*dp(946) - dp(3005)
dp(3068) = dp(2)*p(2003) + p(2)*dp(2003) - dp(2649) - dp(3012)
dp(3069) = dp(32)*p(579) + p(32)*dp(579) - dp(3005)
dp(3070) = dp(1)*p(2052) + p(1)*dp(2052) - dp(2686) - dp(3068)
dp(3071) = dp(73)*p(340) + p(73)*dp(340) - dp(3005)
dp(3072) = dp(13)*p(904) + p(13)*dp(904) - dp(2372)
dp(3073) = dp(150)*p(191) + p(150)*dp(191) - dp(3005)
dp(3074) = dp(73)*p(323) + p(73)*dp(323) - dp(2370)
dp(3075) = dp(14)*p(946) + p(14)*dp(946)
dp(3076) = dp(14)*p(947) + p(14)*dp(947)
dp(3077) = dp(130)*p(188) + p(130)*dp(188) - dp(2363)
dp(3078) = dp(50)*p(430) + p(50)*dp(430) - dp(2601)
dp(3079) = dp(1)*p(2060) + p(1)*dp(2060) - dp(2696) - dp(3078)
dp(3080) = dp(81)*p(337) + p(81)*dp(337) - dp(2948)
dp(3081) = dp(2)*p(2016) + p(2)*dp(2016) - dp(2660) - dp(3028)
dp(3082) = dp(160)*p(188) + p(160)*dp(188) - dp(2948)
dp(3083) = dp(45)*p(536) + p(45)*dp(536) - dp(2601)
dp(3084) = dp(98)*p(296) + p(98)*dp(296) - dp(2948)
dp(3085) = dp(160)*p(173) + p(160)*dp(173) - dp(2363)
dp(3086) = dp(14)*p(907) + p(14)*dp(907)
dp(3087) = dp(14)*p(908) + p(14)*dp(908)
dp(3088) = dp(14)*p(909) + p(14)*dp(909)
dp(3089) = dp(14)*p(818) + p(14)*dp(818)
dp(3090) = dp(14)*p(819) + p(14)*dp(819)
dp(3091) = dp(2)*p(2026) + p(2)*dp(2026) - dp(2671) - dp(3041)
dp(3092) = dp(1)*p(2070) + p(1)*dp(2070) - dp(2706) - dp(3091)
dp(3093) = dp(95)*p(310) + p(95)*dp(310) - dp(2897)
dp(3094) = dp(2)*p(2029) + p(2)*dp(2029) - dp(2674) - dp(3044)
dp(3095) = dp(46)*p(532) + p(46)*dp(532) - dp(2897)
dp(3096) = dp(1)*p(2072) + p(1)*dp(2072) - dp(2706) - dp(3094)
dp(3097) = dp(14)*p(910) + p(14)*dp(910)
dp(3098) = dp(14)*p(911) + p(14)*dp(911)
dp(3099) = dp(14)*p(912) + p(14)*dp(912)
dp(3100) = dp(14)*p(823) + p(14)*dp(823)
dp(3101) = dp(14)*p(824) + p(14)*dp(824)
dp(3102) = dp(14)*p(825) + p(14)*dp(825)
dp(3103) = dp(8)*p(1290) + p(8)*dp(1290) - dp(2632)
dp(3104) = dp(18)*p(900) + p(18)*dp(900) - dp(2978)
dp(3105) = dp(34)*p(578) + p(34)*dp(578) - dp(2632)
dp(3106) = dp(14)*p(914) + p(14)*dp(914)
dp(3107) = dp(14)*p(915) + p(14)*dp(915)
dp(3108) = dp(14)*p(916) + p(14)*dp(916)
dp(3109) = dp(9)*p(952) + p(9)*dp(952)
dp(3110) = dp(9)*p(928) + p(9)*dp(928)
dp(3111) = dp(9)*p(929) + p(9)*dp(929)
dp(3112) = dp(9)*p(953) + p(9)*dp(953)
dp(3113) = dp(24)*p(582) + p(24)*dp(582) - dp(2379)
dp(3114) = dp(114)*p(195) + p(114)*dp(195) - dp(2692)
dp(3115) = dp(1)*p(2079) + p(1)*dp(2079) - dp(2711) - dp(3114)
dp(3116) = dp(13)*p(949) + p(13)*dp(949) - dp(3075)
dp(3117) = dp(2)*p(2052) + p(2)*dp(2052) - dp(2694) - dp(3081)
dp(3118) = dp(32)*p(582) + p(32)*dp(582) - dp(3075)
dp(3119) = dp(6)*p(1446) + p(6)*dp(1446) - dp(2692)
dp(3120) = dp(73)*p(343) + p(73)*dp(343) - dp(3075)
dp(3121) = dp(32)*p(561) + p(32)*dp(561) - dp(2379)
dp(3122) = dp(14)*p(949) + p(14)*dp(949)
dp(3123) = dp(14)*p(950) + p(14)*dp(950)
dp(3124) = dp(130)*p(191) + p(130)*dp(191) - dp(2377)
dp(3125) = dp(1)*p(2086) + p(1)*dp(2086) - dp(2716) - dp(3124)
dp(3126) = dp(81)*p(340) + p(81)*dp(340) - dp(3022)
dp(3127) = dp(2)*p(2062) + p(2)*dp(2062) - dp(2701) - dp(3094)
dp(3128) = dp(160)*p(191) + p(160)*dp(191) - dp(3022)
dp(3129) = dp(81)*p(323) + p(81)*dp(323) - dp(2377)
dp(3130) = dp(14)*p(921) + p(14)*dp(921)
dp(3131) = dp(14)*p(922) + p(14)*dp(922)
dp(3132) = dp(14)*p(923) + p(14)*dp(923)
dp(3133) = dp(14)*p(830) + p(14)*dp(830)
dp(3134) = dp(14)*p(831) + p(14)*dp(831)
dp(3135) = dp(50)*p(464) + p(50)*dp(464) - dp(2626)
dp(3136) = dp(98)*p(310) + p(98)*dp(310) - dp(2968)
dp(3137) = dp(173)*p(187) + p(173)*dp(187) - dp(2626)
dp(3138) = dp(14)*p(924) + p(14)*dp(924)
dp(3139) = dp(14)*p(925) + p(14)*dp(925)
dp(3140) = dp(14)*p(926) + p(14)*dp(926)
dp(3141) = dp(9)*p(955) + p(9)*dp(955)
dp(3142) = dp(9)*p(956) + p(9)*dp(956)
dp(3143) = dp(24)*p(585) + p(24)*dp(585) - dp(2382)
dp(3144) = dp(1)*p(2093) + p(1)*dp(2093) - dp(2720) - dp(3143)
dp(3145) = dp(13)*p(952) + p(13)*dp(952) - dp(3122)
dp(3146) = dp(2)*p(2081) + p(2)*dp(2081) - dp(2715) - dp(3127)
dp(3147) = dp(32)*p(585) + p(32)*dp(585) - dp(3122)
dp(3148) = dp(13)*p(929) + p(13)*dp(929) - dp(2382)
dp(3149) = dp(14)*p(952) + p(14)*dp(952)
dp(3150) = dp(14)*p(953) + p(14)*dp(953)
dp(3151) = dp(130)*p(195) + p(130)*dp(195) - dp(2705)
dp(3152) = dp(81)*p(343) + p(81)*dp(343) - dp(3089)
dp(3153) = dp(45)*p(561) + p(45)*dp(561) - dp(2705)
dp(3154) = dp(14)*p(930) + p(14)*dp(930)
dp(3155) = dp(14)*p(931) + p(14)*dp(931)
dp(3156) = dp(14)*p(932) + p(14)*dp(932)
dp(3157) = dp(9)*p(959) + p(9)*dp(959)
dp(3158) = dp(24)*p(589) + p(24)*dp(589) - dp(2722)
dp(3159) = dp(13)*p(955) + p(13)*dp(955) - dp(3149)
dp(3160) = dp(6)*p(1471) + p(6)*dp(1471) - dp(2722)
dp(3161) = dp(14)*p(955) + p(14)*dp(955)
dp(3162) = dp(14)*p(956) + p(14)*dp(956)
dp(3163) = dp(9)*p(832) + p(9)*dp(832)
dp(3164) = dp(9)*p(833) + p(9)*dp(833)
dp(3165) = dp(9)*p(834) + p(9)*dp(834)
dp(3166) = dp(9)*p(835) + p(9)*dp(835)
dp(3167) = dp(9)*p(836) + p(9)*dp(836)
dp(3168) = dp(9)*p(934) + p(9)*dp(934)
dp(3169) = dp(9)*p(837) + p(9)*dp(837)
dp(3170) = dp(9)*p(935) + p(9)*dp(935)
dp(3171) = dp(9)*p(838) + p(9)*dp(838)
dp(3172) = dp(9)*p(936) + p(9)*dp(936)
dp(3173) = dp(9)*p(937) + p(9)*dp(937)
dp(3174) = dp(9)*p(958) + p(9)*dp(958)
dp(3175) = dp(3)*p(1481) + p(3)*dp(1481) - dp(2723)
dp(3176) = dp(39)*p(570) + p(39)*dp(570) - dp(2725)
dp(3177) = dp(178)*p(181) + p(178)*dp(181) - dp(2728)
dp(3178) = dp(41)*p(566) + p(41)*dp(566) - dp(2732)
dp(3179) = dp(4)*p(1476) + p(4)*dp(1476) - dp(2737)
dp(3180) = dp(1)*p(2112) + p(1)*dp(2112) - dp(2733) - dp(3179)
dp(3181) = dp(1)*p(2113) + p(1)*dp(2113) - dp(2737)
dp(3182) = dp(1)*p(2114) + p(1)*dp(2114) - dp(2738) - dp(3180)
dp(3183) = dp(21)*p(843) + p(21)*dp(843) - dp(2732)
dp(3184) = dp(1)*p(2116) + p(1)*dp(2116) - dp(2742) - dp(3182)
dp(3185) = dp(101)*p(279) + p(101)*dp(279) - dp(2728)
dp(3186) = dp(1)*p(2118) + p(1)*dp(2118) - dp(2745) - dp(3184)
dp(3187) = dp(69)*p(346) + p(69)*dp(346) - dp(2725)
dp(3188) = dp(1)*p(2120) + p(1)*dp(2120) - dp(2747) - dp(3186)
dp(3189) = dp(11)*p(958) + p(11)*dp(958) - dp(2723)
dp(3190) = dp(1)*p(2122) + p(1)*dp(2122) - dp(2753) - dp(2752)
dp(3191) = dp(1)*p(2123) + p(1)*dp(2123) - dp(2761)
dp(3192) = dp(3)*p(1483) + p(3)*dp(1483) - dp(2763)
dp(3193) = dp(39)*p(572) + p(39)*dp(572) - dp(2765)
dp(3194) = dp(178)*p(183) + p(178)*dp(183) - dp(2768)
dp(3195) = dp(43)*p(566) + p(43)*dp(566) - dp(2772)
dp(3196) = dp(1)*p(2127) + p(1)*dp(2127) - dp(2769) - dp(3195)
dp(3197) = dp(2)*p(2113) + p(2)*dp(2113) - dp(2757)
dp(3198) = dp(1)*p(2128) + p(1)*dp(2128) - dp(2773) - dp(3196)
dp(3199) = dp(7)*p(1374) + p(7)*dp(1374) - dp(2772)
dp(3200) = dp(1)*p(2130) + p(1)*dp(2130) - dp(2777) - dp(3198)
dp(3201) = dp(49)*p(509) + p(49)*dp(509) - dp(2768)
dp(3202) = dp(1)*p(2132) + p(1)*dp(2132) - dp(2780) - dp(3200)
dp(3203) = dp(153)*p(194) + p(153)*dp(194) - dp(2765)
dp(3204) = dp(1)*p(2134) + p(1)*dp(2134) - dp(2782) - dp(3202)
dp(3205) = dp(34)*p(588) + p(34)*dp(588) - dp(2763)
dp(3206) = dp(6)*p(1481) + p(6)*dp(1481) - dp(2762)
dp(3207) = dp(1)*p(2137) + p(1)*dp(2137) - dp(2792) - dp(2790)
dp(3208) = dp(2)*p(2123) + p(2)*dp(2123) - dp(2762)
dp(3209) = dp(1)*p(2139) + p(1)*dp(2139) - dp(2813) - dp(2812)
dp(3210) = dp(1)*p(2140) + p(1)*dp(2140) - dp(2819)
dp(3211) = dp(3)*p(1485) + p(3)*dp(1485) - dp(2823)
dp(3212) = dp(39)*p(574) + p(39)*dp(574) - dp(2825)
dp(3213) = dp(178)*p(185) + p(178)*dp(185) - dp(2828)
dp(3214) = dp(46)*p(566) + p(46)*dp(566) - dp(2832)
dp(3215) = dp(1)*p(2144) + p(1)*dp(2144) - dp(2829) - dp(3214)
dp(3216) = dp(1)*p(2145) + p(1)*dp(2145) - dp(2832)
dp(3217) = dp(1)*p(2146) + p(1)*dp(2146) - dp(2833) - dp(3215)
dp(3218) = dp(21)*p(869) + p(21)*dp(869) - dp(2828)
dp(3219) = dp(1)*p(2148) + p(1)*dp(2148) - dp(2836) - dp(3217)
dp(3220) = dp(101)*p(299) + p(101)*dp(299) - dp(2825)
dp(3221) = dp(1)*p(2150) + p(1)*dp(2150) - dp(2838) - dp(3219)
dp(3222) = dp(84)*p(346) + p(84)*dp(346) - dp(2823)
dp(3223) = dp(6)*p(1483) + p(6)*dp(1483) - dp(2807)
dp(3224) = dp(1)*p(2153) + p(1)*dp(2153) - dp(2847) - dp(2845)
dp(3225) = dp(8)*p(1369) + p(8)*dp(1369) - dp(2807)
dp(3226) = dp(45)*p(570) + p(45)*dp(570) - dp(2821)
dp(3227) = dp(1)*p(2156) + p(1)*dp(2156) - dp(2869) - dp(2867)
dp(3228) = dp(2)*p(2140) + p(2)*dp(2140) - dp(2821)
dp(3229) = dp(1)*p(2158) + p(1)*dp(2158) - dp(2888) - dp(2887)
dp(3230) = dp(1)*p(2159) + p(1)*dp(2159) - dp(2892)
dp(3231) = dp(3)*p(1487) + p(3)*dp(1487) - dp(2898)
dp(3232) = dp(39)*p(576) + p(39)*dp(576) - dp(2900)
dp(3233) = dp(178)*p(188) + p(178)*dp(188) - dp(2903)
dp(3234) = dp(1)*p(2162) + p(1)*dp(2162) - dp(2901) - dp(3233)
dp(3235) = dp(2)*p(2145) + p(2)*dp(2145) - dp(2853)
dp(3236) = dp(1)*p(2163) + p(1)*dp(2163) - dp(2904) - dp(3234)
dp(3237) = dp(7)*p(1406) + p(7)*dp(1406) - dp(2903)
dp(3238) = dp(1)*p(2165) + p(1)*dp(2165) - dp(2907) - dp(3236)
dp(3239) = dp(49)*p(536) + p(49)*dp(536) - dp(2900)
dp(3240) = dp(1)*p(2167) + p(1)*dp(2167) - dp(2909) - dp(3238)
dp(3241) = dp(173)*p(194) + p(173)*dp(194) - dp(2898)
dp(3242) = dp(6)*p(1485) + p(6)*dp(1485) - dp(2860)
dp(3243) = dp(1)*p(2170) + p(1)*dp(2170) - dp(2917) - dp(2915)
dp(3244) = dp(22)*p(851) + p(22)*dp(851) - dp(2860)
dp(3245) = dp(45)*p(572) + p(45)*dp(572) - dp(2822)
dp(3246) = dp(1)*p(2173) + p(1)*dp(2173) - dp(2936) - dp(2934)
dp(3247) = dp(8)*p(1385) + p(8)*dp(1385) - dp(2822)
dp(3248) = dp(181)*p(187) + p(181)*dp(187) - dp(2894)
dp(3249) = dp(1)*p(2176) + p(1)*dp(2176) - dp(2955) - dp(2953)
dp(3250) = dp(2)*p(2159) + p(2)*dp(2159) - dp(2894)
dp(3251) = dp(1)*p(2178) + p(1)*dp(2178) - dp(2972) - dp(2971)
dp(3252) = dp(1)*p(2179) + p(1)*dp(2179) - dp(2974)
dp(3253) = dp(14)*p(882) + p(14)*dp(882)
dp(3254) = dp(3)*p(1489) + p(3)*dp(1489) - dp(2981)
dp(3255) = dp(39)*p(579) + p(39)*dp(579) - dp(2983)
dp(3256) = dp(178)*p(191) + p(178)*dp(191) - dp(2986)
dp(3257) = dp(1)*p(2183) + p(1)*dp(2183) - dp(2984) - dp(3256)
dp(3258) = dp(1)*p(2184) + p(1)*dp(2184) - dp(2986)
dp(3259) = dp(1)*p(2185) + p(1)*dp(2185) - dp(2987) - dp(3257)
dp(3260) = dp(21)*p(904) + p(21)*dp(904) - dp(2983)
dp(3261) = dp(1)*p(2187) + p(1)*dp(2187) - dp(2989) - dp(3259)
dp(3262) = dp(101)*p(323) + p(101)*dp(323) - dp(2981)
dp(3263) = dp(6)*p(1487) + p(6)*dp(1487) - dp(2928)
dp(3264) = dp(1)*p(2190) + p(1)*dp(2190) - dp(2996) - dp(2994)
dp(3265) = dp(50)*p(505) + p(50)*dp(505) - dp(2928)
dp(3266) = dp(45)*p(574) + p(45)*dp(574) - dp(2883)
dp(3267) = dp(1)*p(2193) + p(1)*dp(2193) - dp(3012) - dp(3010)
dp(3268) = dp(22)*p(865) + p(22)*dp(865) - dp(2883)
dp(3269) = dp(183)*p(187) + p(183)*dp(187) - dp(2896)
dp(3270) = dp(1)*p(2196) + p(1)*dp(2196) - dp(3028) - dp(3026)
dp(3271) = dp(8)*p(1402) + p(8)*dp(1402) - dp(2896)
dp(3272) = dp(41)*p(578) + p(41)*dp(578) - dp(2976)
dp(3273) = dp(1)*p(2200) + p(1)*dp(2200) - dp(3044) - dp(3042)
dp(3274) = dp(2)*p(2179) + p(2)*dp(2179) - dp(2976)
dp(3275) = dp(14)*p(898) + p(14)*dp(898)
dp(3276) = dp(4)*p(1441) + p(4)*dp(1441) - dp(2683) - dp(3103)
dp(3277) = dp(1)*p(2204) + p(1)*dp(2204) - dp(3054)
dp(3278) = dp(14)*p(901) + p(14)*dp(901)
dp(3279) = dp(3)*p(1492) + p(3)*dp(1492) - dp(3057)
dp(3280) = dp(39)*p(582) + p(39)*dp(582) - dp(3059)
dp(3281) = dp(1)*p(2207) + p(1)*dp(2207) - dp(3058) - dp(3280)
dp(3282) = dp(2)*p(2184) + p(2)*dp(2184) - dp(3000)
dp(3283) = dp(1)*p(2208) + p(1)*dp(2208) - dp(3060) - dp(3281)
dp(3284) = dp(7)*p(1446) + p(7)*dp(1446) - dp(3059)
dp(3285) = dp(1)*p(2210) + p(1)*dp(2210) - dp(3062) - dp(3283)
dp(3286) = dp(49)*p(561) + p(49)*dp(561) - dp(3057)
dp(3287) = dp(6)*p(1489) + p(6)*dp(1489) - dp(3005)
dp(3288) = dp(1)*p(2213) + p(1)*dp(2213) - dp(3068) - dp(3066)
dp(3289) = dp(102)*p(285) + p(102)*dp(285) - dp(3005)
dp(3290) = dp(45)*p(576) + p(45)*dp(576) - dp(2948)
dp(3291) = dp(1)*p(2216) + p(1)*dp(2216) - dp(3081) - dp(3079)
dp(3292) = dp(50)*p(518) + p(50)*dp(518) - dp(2948)
dp(3293) = dp(185)*p(187) + p(185)*dp(187) - dp(2897)
dp(3294) = dp(1)*p(2219) + p(1)*dp(2219) - dp(3094) - dp(3092)
dp(3295) = dp(22)*p(881) + p(22)*dp(881) - dp(2897)
dp(3296) = dp(14)*p(913) + p(14)*dp(913)
dp(3297) = dp(43)*p(578) + p(43)*dp(578) - dp(2978)
dp(3298) = dp(1)*p(2223) + p(1)*dp(2223) - dp(3105) - dp(3103)
dp(3299) = dp(8)*p(1422) + p(8)*dp(1422) - dp(2978)
dp(3300) = dp(14)*p(917) + p(14)*dp(917)
dp(3301) = dp(4)*p(1491) + p(4)*dp(1491) - dp(3056)
dp(3302) = dp(2)*p(2203) + p(2)*dp(2203) - dp(3055) - dp(3301)
dp(3303) = dp(2)*p(2204) + p(2)*dp(2204) - dp(3056)
dp(3304) = dp(14)*p(948) + p(14)*dp(948)
dp(3305) = dp(3)*p(1495) + p(3)*dp(1495) - dp(3109)
dp(3306) = dp(39)*p(585) + p(39)*dp(585) - dp(3111)
dp(3307) = dp(1)*p(2227) + p(1)*dp(2227) - dp(3110) - dp(3306)
dp(3308) = dp(1)*p(2228) + p(1)*dp(2228) - dp(3111)
dp(3309) = dp(1)*p(2229) + p(1)*dp(2229) - dp(3112) - dp(3307)
dp(3310) = dp(21)*p(929) + p(21)*dp(929) - dp(3109)
dp(3311) = dp(6)*p(1492) + p(6)*dp(1492) - dp(3075)
dp(3312) = dp(1)*p(2232) + p(1)*dp(2232) - dp(3117) - dp(3115)
dp(3313) = dp(150)*p(195) + p(150)*dp(195) - dp(3075)
dp(3314) = dp(45)*p(579) + p(45)*dp(579) - dp(3022)
dp(3315) = dp(1)*p(2235) + p(1)*dp(2235) - dp(3127) - dp(3125)
dp(3316) = dp(102)*p(296) + p(102)*dp(296) - dp(3022)
dp(3317) = dp(187)*p(188) + p(187)*dp(188) - dp(2968)
dp(3318) = dp(1)*p(2239) + p(1)*dp(2239) - dp(3137) - dp(3135)
dp(3319) = dp(50)*p(532) + p(50)*dp(532) - dp(2968)
dp(3320) = dp(14)*p(927) + p(14)*dp(927)
dp(3321) = dp(46)*p(578) + p(46)*dp(578) - dp(2980)
dp(3322) = dp(2)*p(2223) + p(2)*dp(2223) - dp(3107) - dp(3302)
dp(3323) = dp(22)*p(900) + p(22)*dp(900) - dp(2980)
dp(3324) = dp(14)*p(951) + p(14)*dp(951)
dp(3325) = dp(3)*p(1498) + p(3)*dp(1498) - dp(3141)
dp(3326) = dp(2)*p(2227) + p(2)*dp(2227) - dp(3118) - dp(3114)
dp(3327) = dp(2)*p(2228) + p(2)*dp(2228) - dp(3119)
dp(3328) = dp(1)*p(2243) + p(1)*dp(2243) - dp(3142) - dp(3326)
dp(3329) = dp(7)*p(1471) + p(7)*dp(1471) - dp(3141)
dp(3330) = dp(6)*p(1495) + p(6)*dp(1495) - dp(3122)
dp(3331) = dp(1)*p(2246) + p(1)*dp(2246) - dp(3146) - dp(3144)
dp(3332) = dp(73)*p(347) + p(73)*dp(347) - dp(3122)
dp(3333) = dp(45)*p(582) + p(45)*dp(582) - dp(3089)
dp(3334) = dp(1)*p(2249) + p(1)*dp(2249) - dp(3153) - dp(3151)
dp(3335) = dp(160)*p(195) + p(160)*dp(195) - dp(3089)
dp(3336) = dp(14)*p(933) + p(14)*dp(933)
dp(3337) = dp(187)*p(191) + p(187)*dp(191) - dp(3039)
dp(3338) = dp(2)*p(2239) + p(2)*dp(2239) - dp(3139) - dp(3322)
dp(3339) = dp(102)*p(310) + p(102)*dp(310) - dp(3039)
dp(3340) = dp(14)*p(954) + p(14)*dp(954)
dp(3341) = dp(3)*p(1501) + p(3)*dp(1501) - dp(3157)
dp(3342) = dp(2)*p(2243) + p(2)*dp(2243) - dp(3147) - dp(3144)
dp(3343) = dp(1)*p(2253) + p(1)*dp(2253) - dp(3157)
dp(3344) = dp(6)*p(1498) + p(6)*dp(1498) - dp(3149)
dp(3345) = dp(1)*p(2255) + p(1)*dp(2255) - dp(3160) - dp(3158)
dp(3346) = dp(32)*p(589) + p(32)*dp(589) - dp(3149)
dp(3347) = dp(45)*p(585) + p(45)*dp(585) - dp(3133)
dp(3348) = dp(2)*p(2249) + p(2)*dp(2249) - dp(3155) - dp(3338)
dp(3349) = dp(81)*p(347) + p(81)*dp(347) - dp(3133)
dp(3350) = dp(14)*p(957) + p(14)*dp(957)
dp(3351) = dp(2)*p(2252) + p(2)*dp(2252) - dp(3159) - dp(3158)
dp(3352) = dp(2)*p(2253) + p(2)*dp(2253) - dp(3160)
dp(3353) = dp(6)*p(1501) + p(6)*dp(1501) - dp(3161)
dp(3354) = dp(2)*p(2255) + p(2)*dp(2255) - dp(3162) - dp(3348)
dp(3355) = dp(13)*p(959) + p(13)*dp(959) - dp(3161)
dp(3356) = dp(14)*p(959) + p(14)*dp(959)
dp(3357) = dp(3)*p(1476) + p(3)*dp(1476) - dp(3168)
dp(3358) = dp(7)*p(1476) + p(7)*dp(1476) - dp(3165)
dp(3359) = dp(1)*p(2259) + p(1)*dp(2259) - dp(3170) - dp(3358)
dp(3360) = dp(178)*p(194) + p(178)*dp(194) - dp(3166)
dp(3361) = dp(39)*p(588) + p(39)*dp(588) - dp(3171)
dp(3362) = dp(3)*p(1504) + p(3)*dp(1504) - dp(3174)
dp(3363) = dp(1)*p(2263) + p(1)*dp(2263) - dp(3175)
dp(3364) = dp(2)*p(2288) + p(2)*dp(2288) - dp(3363)
dp(3365) = dp(1)*p(2265) + p(1)*dp(2265) - dp(3192)
dp(3366) = dp(8)*p(1504) + p(8)*dp(1504) - dp(3365)
dp(3367) = dp(1)*p(2267) + p(1)*dp(2267) - dp(3211)
dp(3368) = dp(22)*p(958) + p(22)*dp(958) - dp(3367)
dp(3369) = dp(1)*p(2269) + p(1)*dp(2269) - dp(3231)
dp(3370) = dp(50)*p(588) + p(50)*dp(588) - dp(3369)
dp(3371) = dp(1)*p(2271) + p(1)*dp(2271) - dp(3254)
dp(3372) = dp(102)*p(346) + p(102)*dp(346) - dp(3371)
dp(3373) = dp(1)*p(2273) + p(1)*dp(2273) - dp(3279)
dp(3374) = dp(194)*p(195) + p(194)*dp(195) - dp(3373)
dp(3375) = dp(6)*p(1491) + p(6)*dp(1491) - dp(3304)
dp(3376) = dp(1)*p(2276) + p(1)*dp(2276) - dp(3305)
dp(3377) = dp(101)*p(347) + p(101)*dp(347) - dp(3376)
dp(3378) = dp(8)*p(1491) + p(8)*dp(1491) - dp(3278)
dp(3379) = dp(1)*p(2279) + p(1)*dp(2279) - dp(3325)
dp(3380) = dp(49)*p(589) + p(49)*dp(589) - dp(3379)
dp(3381) = dp(2)*p(2278) + p(2)*dp(2278) - dp(3324) - dp(3378)
dp(3382) = dp(1)*p(2282) + p(1)*dp(2282) - dp(3341)
dp(3383) = dp(21)*p(959) + p(21)*dp(959) - dp(3382)
dp(3384) = dp(187)*p(195) + p(187)*dp(195) - dp(3296)
dp(3385) = dp(1)*p(2285) + p(1)*dp(2285) - dp(3351)
dp(3386) = dp(7)*p(1505) + p(7)*dp(1505) - dp(3385)
dp(3387) = dp(45)*p(589) + p(45)*dp(589) - dp(3336)
dp(3388) = dp(2)*p(2285) + p(2)*dp(2285) - dp(3353)
dp(3389) = dp(1)*p(2289) + p(1)*dp(2289) - dp(3388)
dp(3390) = dp(6)*p(1505) + p(6)*dp(1505) - dp(3356)
dp(3391) = dp(1)*p(2288) + p(1)*dp(2288) - dp(3362)
dp(3392) = dp(2)*p(2289) + p(2)*dp(2289) - dp(3390)

do j=1,3393
  dbm1(j)=dp(j-1)
enddo

dbm2(1)=dbm1(4)
dbm2(2)=dbm1(10)
dbm2(3)=dbm1(11)
dbm2(4)=dbm1(12)
dbm2(5)=dbm1(13)
dbm2(6)=dbm1(15)
dbm2(7)=dbm1(16)
dbm2(8)=dbm1(24)
dbm2(9)=dbm1(25)
dbm2(10)=dbm1(26)
dbm2(11)=dbm1(27)
dbm2(12)=dbm1(28)
dbm2(13)=dbm1(29)
dbm2(14)=dbm1(30)
dbm2(15)=dbm1(31)
dbm2(16)=dbm1(32)
dbm2(17)=dbm1(34)
dbm2(18)=dbm1(35)
dbm2(19)=dbm1(36)
dbm2(20)=dbm1(37)
dbm2(21)=dbm1(39)
dbm2(22)=dbm1(40)
dbm2(23)=dbm1(41)
dbm2(24)=dbm1(52)
dbm2(25)=dbm1(53)
dbm2(26)=dbm1(54)
dbm2(27)=dbm1(55)
dbm2(28)=dbm1(56)
dbm2(29)=dbm1(57)
dbm2(30)=dbm1(58)
dbm2(31)=dbm1(59)
dbm2(32)=dbm1(60)
dbm2(33)=dbm1(61)
dbm2(34)=dbm1(62)
dbm2(35)=dbm1(63)
dbm2(36)=dbm1(64)
dbm2(37)=dbm1(65)
dbm2(38)=dbm1(66)
dbm2(39)=dbm1(67)
dbm2(40)=dbm1(68)
dbm2(41)=dbm1(69)
dbm2(42)=dbm1(70)
dbm2(43)=dbm1(71)
dbm2(44)=dbm1(72)
dbm2(45)=dbm1(73)
dbm2(46)=dbm1(75)
dbm2(47)=dbm1(76)
dbm2(48)=dbm1(77)
dbm2(49)=dbm1(78)
dbm2(50)=dbm1(79)
dbm2(51)=dbm1(81)
dbm2(52)=dbm1(83)
dbm2(53)=dbm1(84)
dbm2(54)=dbm1(85)
dbm2(55)=dbm1(86)
dbm2(56)=dbm1(87)
dbm2(57)=dbm1(89)
dbm2(58)=dbm1(90)
dbm2(59)=dbm1(91)
dbm2(60)=dbm1(104)
dbm2(61)=dbm1(105)
dbm2(62)=dbm1(106)
dbm2(63)=dbm1(107)
dbm2(64)=dbm1(108)
dbm2(65)=dbm1(109)
dbm2(66)=dbm1(110)
dbm2(67)=dbm1(111)
dbm2(68)=dbm1(112)
dbm2(69)=dbm1(113)
dbm2(70)=dbm1(114)
dbm2(71)=dbm1(115)
dbm2(72)=dbm1(116)
dbm2(73)=dbm1(117)
dbm2(74)=dbm1(118)
dbm2(75)=dbm1(119)
dbm2(76)=dbm1(120)
dbm2(77)=dbm1(121)
dbm2(78)=dbm1(122)
dbm2(79)=dbm1(123)
dbm2(80)=dbm1(124)
dbm2(81)=dbm1(125)
dbm2(82)=dbm1(126)
dbm2(83)=dbm1(127)
dbm2(84)=dbm1(128)
dbm2(85)=dbm1(129)
dbm2(86)=dbm1(130)
dbm2(87)=dbm1(131)
dbm2(88)=dbm1(132)
dbm2(89)=dbm1(133)
dbm2(90)=dbm1(134)
dbm2(91)=dbm1(135)
dbm2(92)=dbm1(136)
dbm2(93)=dbm1(137)
dbm2(94)=dbm1(138)
dbm2(95)=dbm1(139)
dbm2(96)=dbm1(140)
dbm2(97)=dbm1(141)
dbm2(98)=dbm1(142)
dbm2(99)=dbm1(143)
dbm2(100)=dbm1(144)
dbm2(101)=dbm1(145)
dbm2(102)=dbm1(146)
dbm2(103)=dbm1(147)
dbm2(104)=dbm1(148)
dbm2(105)=dbm1(149)
dbm2(106)=dbm1(150)
dbm2(107)=dbm1(152)
dbm2(108)=dbm1(153)
dbm2(109)=dbm1(154)
dbm2(110)=dbm1(155)
dbm2(111)=dbm1(156)
dbm2(112)=dbm1(157)
dbm2(113)=dbm1(158)
dbm2(114)=dbm1(160)
dbm2(115)=dbm1(162)
dbm2(116)=dbm1(163)
dbm2(117)=dbm1(164)
dbm2(118)=dbm1(165)
dbm2(119)=dbm1(166)
dbm2(120)=dbm1(167)
dbm2(121)=dbm1(169)
dbm2(122)=dbm1(170)
dbm2(123)=dbm1(172)
dbm2(124)=dbm1(173)
dbm2(125)=dbm1(174)
dbm2(126)=dbm1(175)
dbm2(127)=dbm1(176)
dbm2(128)=dbm1(178)
dbm2(129)=dbm1(179)
dbm2(130)=dbm1(180)
dbm2(131)=dbm1(181)
dbm2(132)=dbm1(197)
dbm2(133)=dbm1(198)
dbm2(134)=dbm1(199)
dbm2(135)=dbm1(200)
dbm2(136)=dbm1(201)
dbm2(137)=dbm1(202)
dbm2(138)=dbm1(203)
dbm2(139)=dbm1(204)
dbm2(140)=dbm1(205)
dbm2(141)=dbm1(206)
dbm2(142)=dbm1(207)
dbm2(143)=dbm1(208)
dbm2(144)=dbm1(209)
dbm2(145)=dbm1(210)
dbm2(146)=dbm1(211)
dbm2(147)=dbm1(212)
dbm2(148)=dbm1(213)
dbm2(149)=dbm1(214)
dbm2(150)=dbm1(215)
dbm2(151)=dbm1(216)
dbm2(152)=dbm1(217)
dbm2(153)=dbm1(218)
dbm2(154)=dbm1(219)
dbm2(155)=dbm1(220)
dbm2(156)=dbm1(221)
dbm2(157)=dbm1(222)
dbm2(158)=dbm1(223)
dbm2(159)=dbm1(224)
dbm2(160)=dbm1(225)
dbm2(161)=dbm1(226)
dbm2(162)=dbm1(227)
dbm2(163)=dbm1(228)
dbm2(164)=dbm1(229)
dbm2(165)=dbm1(230)
dbm2(166)=dbm1(231)
dbm2(167)=dbm1(232)
dbm2(168)=dbm1(233)
dbm2(169)=dbm1(234)
dbm2(170)=dbm1(235)
dbm2(171)=dbm1(236)
dbm2(172)=dbm1(237)
dbm2(173)=dbm1(238)
dbm2(174)=dbm1(239)
dbm2(175)=dbm1(240)
dbm2(176)=dbm1(241)
dbm2(177)=dbm1(242)
dbm2(178)=dbm1(243)
dbm2(179)=dbm1(244)
dbm2(180)=dbm1(245)
dbm2(181)=dbm1(246)
dbm2(182)=dbm1(247)
dbm2(183)=dbm1(248)
dbm2(184)=dbm1(249)
dbm2(185)=dbm1(250)
dbm2(186)=dbm1(251)
dbm2(187)=dbm1(252)
dbm2(188)=dbm1(253)
dbm2(189)=dbm1(254)
dbm2(190)=dbm1(255)
dbm2(191)=dbm1(256)
dbm2(192)=dbm1(257)
dbm2(193)=dbm1(258)
dbm2(194)=dbm1(259)
dbm2(195)=dbm1(260)
dbm2(196)=dbm1(261)
dbm2(197)=dbm1(262)
dbm2(198)=dbm1(263)
dbm2(199)=dbm1(264)
dbm2(200)=dbm1(265)
dbm2(201)=dbm1(266)
dbm2(202)=dbm1(267)
dbm2(203)=dbm1(268)
dbm2(204)=dbm1(269)
dbm2(205)=dbm1(270)
dbm2(206)=dbm1(271)
dbm2(207)=dbm1(272)
dbm2(208)=dbm1(273)
dbm2(209)=dbm1(274)
dbm2(210)=dbm1(275)
dbm2(211)=dbm1(276)
dbm2(212)=dbm1(277)
dbm2(213)=dbm1(278)
dbm2(214)=dbm1(279)
dbm2(215)=dbm1(280)
dbm2(216)=dbm1(281)
dbm2(217)=dbm1(282)
dbm2(218)=dbm1(283)
dbm2(219)=dbm1(284)
dbm2(220)=dbm1(285)
dbm2(221)=dbm1(287)
dbm2(222)=dbm1(288)
dbm2(223)=dbm1(289)
dbm2(224)=dbm1(290)
dbm2(225)=dbm1(291)
dbm2(226)=dbm1(292)
dbm2(227)=dbm1(293)
dbm2(228)=dbm1(294)
dbm2(229)=dbm1(296)
dbm2(230)=dbm1(298)
dbm2(231)=dbm1(299)
dbm2(232)=dbm1(300)
dbm2(233)=dbm1(301)
dbm2(234)=dbm1(302)
dbm2(235)=dbm1(303)
dbm2(236)=dbm1(304)
dbm2(237)=dbm1(306)
dbm2(238)=dbm1(307)
dbm2(239)=dbm1(309)
dbm2(240)=dbm1(310)
dbm2(241)=dbm1(312)
dbm2(242)=dbm1(313)
dbm2(243)=dbm1(314)
dbm2(244)=dbm1(315)
dbm2(245)=dbm1(316)
dbm2(246)=dbm1(317)
dbm2(247)=dbm1(319)
dbm2(248)=dbm1(320)
dbm2(249)=dbm1(322)
dbm2(250)=dbm1(323)
dbm2(251)=dbm1(324)
dbm2(252)=dbm1(325)
dbm2(253)=dbm1(326)
dbm2(254)=dbm1(328)
dbm2(255)=dbm1(329)
dbm2(256)=dbm1(330)
dbm2(257)=dbm1(331)
dbm2(258)=dbm1(349)
dbm2(259)=dbm1(350)
dbm2(260)=dbm1(351)
dbm2(261)=dbm1(352)
dbm2(262)=dbm1(353)
dbm2(263)=dbm1(354)
dbm2(264)=dbm1(355)
dbm2(265)=dbm1(356)
dbm2(266)=dbm1(357)
dbm2(267)=dbm1(358)
dbm2(268)=dbm1(359)
dbm2(269)=dbm1(360)
dbm2(270)=dbm1(361)
dbm2(271)=dbm1(362)
dbm2(272)=dbm1(363)
dbm2(273)=dbm1(364)
dbm2(274)=dbm1(365)
dbm2(275)=dbm1(366)
dbm2(276)=dbm1(367)
dbm2(277)=dbm1(368)
dbm2(278)=dbm1(369)
dbm2(279)=dbm1(370)
dbm2(280)=dbm1(371)
dbm2(281)=dbm1(372)
dbm2(282)=dbm1(373)
dbm2(283)=dbm1(374)
dbm2(284)=dbm1(375)
dbm2(285)=dbm1(376)
dbm2(286)=dbm1(377)
dbm2(287)=dbm1(378)
dbm2(288)=dbm1(379)
dbm2(289)=dbm1(380)
dbm2(290)=dbm1(381)
dbm2(291)=dbm1(382)
dbm2(292)=dbm1(383)
dbm2(293)=dbm1(384)
dbm2(294)=dbm1(385)
dbm2(295)=dbm1(386)
dbm2(296)=dbm1(387)
dbm2(297)=dbm1(388)
dbm2(298)=dbm1(389)
dbm2(299)=dbm1(390)
dbm2(300)=dbm1(391)
dbm2(301)=dbm1(392)
dbm2(302)=dbm1(393)
dbm2(303)=dbm1(394)
dbm2(304)=dbm1(395)
dbm2(305)=dbm1(396)
dbm2(306)=dbm1(397)
dbm2(307)=dbm1(398)
dbm2(308)=dbm1(399)
dbm2(309)=dbm1(400)
dbm2(310)=dbm1(401)
dbm2(311)=dbm1(402)
dbm2(312)=dbm1(403)
dbm2(313)=dbm1(404)
dbm2(314)=dbm1(405)
dbm2(315)=dbm1(406)
dbm2(316)=dbm1(407)
dbm2(317)=dbm1(408)
dbm2(318)=dbm1(409)
dbm2(319)=dbm1(410)
dbm2(320)=dbm1(411)
dbm2(321)=dbm1(412)
dbm2(322)=dbm1(413)
dbm2(323)=dbm1(414)
dbm2(324)=dbm1(415)
dbm2(325)=dbm1(416)
dbm2(326)=dbm1(417)
dbm2(327)=dbm1(418)
dbm2(328)=dbm1(419)
dbm2(329)=dbm1(420)
dbm2(330)=dbm1(421)
dbm2(331)=dbm1(422)
dbm2(332)=dbm1(423)
dbm2(333)=dbm1(424)
dbm2(334)=dbm1(425)
dbm2(335)=dbm1(426)
dbm2(336)=dbm1(427)
dbm2(337)=dbm1(428)
dbm2(338)=dbm1(429)
dbm2(339)=dbm1(430)
dbm2(340)=dbm1(431)
dbm2(341)=dbm1(432)
dbm2(342)=dbm1(433)
dbm2(343)=dbm1(434)
dbm2(344)=dbm1(435)
dbm2(345)=dbm1(436)
dbm2(346)=dbm1(437)
dbm2(347)=dbm1(438)
dbm2(348)=dbm1(439)
dbm2(349)=dbm1(440)
dbm2(350)=dbm1(441)
dbm2(351)=dbm1(442)
dbm2(352)=dbm1(443)
dbm2(353)=dbm1(444)
dbm2(354)=dbm1(445)
dbm2(355)=dbm1(446)
dbm2(356)=dbm1(447)
dbm2(357)=dbm1(448)
dbm2(358)=dbm1(449)
dbm2(359)=dbm1(450)
dbm2(360)=dbm1(451)
dbm2(361)=dbm1(452)
dbm2(362)=dbm1(453)
dbm2(363)=dbm1(454)
dbm2(364)=dbm1(455)
dbm2(365)=dbm1(456)
dbm2(366)=dbm1(457)
dbm2(367)=dbm1(458)
dbm2(368)=dbm1(459)
dbm2(369)=dbm1(460)
dbm2(370)=dbm1(461)
dbm2(371)=dbm1(462)
dbm2(372)=dbm1(463)
dbm2(373)=dbm1(464)
dbm2(374)=dbm1(465)
dbm2(375)=dbm1(466)
dbm2(376)=dbm1(467)
dbm2(377)=dbm1(468)
dbm2(378)=dbm1(469)
dbm2(379)=dbm1(470)
dbm2(380)=dbm1(471)
dbm2(381)=dbm1(472)
dbm2(382)=dbm1(473)
dbm2(383)=dbm1(474)
dbm2(384)=dbm1(475)
dbm2(385)=dbm1(476)
dbm2(386)=dbm1(477)
dbm2(387)=dbm1(478)
dbm2(388)=dbm1(479)
dbm2(389)=dbm1(480)
dbm2(390)=dbm1(481)
dbm2(391)=dbm1(482)
dbm2(392)=dbm1(483)
dbm2(393)=dbm1(484)
dbm2(394)=dbm1(485)
dbm2(395)=dbm1(486)
dbm2(396)=dbm1(487)
dbm2(397)=dbm1(488)
dbm2(398)=dbm1(489)
dbm2(399)=dbm1(490)
dbm2(400)=dbm1(491)
dbm2(401)=dbm1(492)
dbm2(402)=dbm1(493)
dbm2(403)=dbm1(494)
dbm2(404)=dbm1(495)
dbm2(405)=dbm1(496)
dbm2(406)=dbm1(497)
dbm2(407)=dbm1(498)
dbm2(408)=dbm1(499)
dbm2(409)=dbm1(500)
dbm2(410)=dbm1(501)
dbm2(411)=dbm1(502)
dbm2(412)=dbm1(503)
dbm2(413)=dbm1(504)
dbm2(414)=dbm1(505)
dbm2(415)=dbm1(507)
dbm2(416)=dbm1(508)
dbm2(417)=dbm1(509)
dbm2(418)=dbm1(510)
dbm2(419)=dbm1(511)
dbm2(420)=dbm1(512)
dbm2(421)=dbm1(513)
dbm2(422)=dbm1(514)
dbm2(423)=dbm1(515)
dbm2(424)=dbm1(516)
dbm2(425)=dbm1(518)
dbm2(426)=dbm1(520)
dbm2(427)=dbm1(521)
dbm2(428)=dbm1(522)
dbm2(429)=dbm1(523)
dbm2(430)=dbm1(524)
dbm2(431)=dbm1(525)
dbm2(432)=dbm1(526)
dbm2(433)=dbm1(527)
dbm2(434)=dbm1(529)
dbm2(435)=dbm1(530)
dbm2(436)=dbm1(532)
dbm2(437)=dbm1(534)
dbm2(438)=dbm1(535)
dbm2(439)=dbm1(536)
dbm2(440)=dbm1(537)
dbm2(441)=dbm1(538)
dbm2(442)=dbm1(539)
dbm2(443)=dbm1(540)
dbm2(444)=dbm1(541)
dbm2(445)=dbm1(543)
dbm2(446)=dbm1(544)
dbm2(447)=dbm1(546)
dbm2(448)=dbm1(547)
dbm2(449)=dbm1(548)
dbm2(450)=dbm1(550)
dbm2(451)=dbm1(551)
dbm2(452)=dbm1(552)
dbm2(453)=dbm1(553)
dbm2(454)=dbm1(554)
dbm2(455)=dbm1(555)
dbm2(456)=dbm1(557)
dbm2(457)=dbm1(558)
dbm2(458)=dbm1(560)
dbm2(459)=dbm1(561)
dbm2(460)=dbm1(562)
dbm2(461)=dbm1(563)
dbm2(462)=dbm1(564)
dbm2(463)=dbm1(566)
dbm2(464)=dbm1(567)
dbm2(465)=dbm1(568)
dbm2(466)=dbm1(569)
dbm2(467)=dbm1(570)
dbm2(468)=dbm1(591)
dbm2(469)=dbm1(592)
dbm2(470)=dbm1(593)
dbm2(471)=dbm1(594)
dbm2(472)=dbm1(595)
dbm2(473)=dbm1(596)
dbm2(474)=dbm1(597)
dbm2(475)=dbm1(598)
dbm2(476)=dbm1(599)
dbm2(477)=dbm1(600)
dbm2(478)=dbm1(601)
dbm2(479)=dbm1(602)
dbm2(480)=dbm1(603)
dbm2(481)=dbm1(604)
dbm2(482)=dbm1(605)
dbm2(483)=dbm1(606)
dbm2(484)=dbm1(607)
dbm2(485)=dbm1(608)
dbm2(486)=dbm1(609)
dbm2(487)=dbm1(610)
dbm2(488)=dbm1(611)
dbm2(489)=dbm1(612)
dbm2(490)=dbm1(613)
dbm2(491)=dbm1(614)
dbm2(492)=dbm1(615)
dbm2(493)=dbm1(616)
dbm2(494)=dbm1(617)
dbm2(495)=dbm1(618)
dbm2(496)=dbm1(619)
dbm2(497)=dbm1(620)
dbm2(498)=dbm1(621)
dbm2(499)=dbm1(622)
dbm2(500)=dbm1(623)
dbm2(501)=dbm1(624)
dbm2(502)=dbm1(625)
dbm2(503)=dbm1(626)
dbm2(504)=dbm1(627)
dbm2(505)=dbm1(628)
dbm2(506)=dbm1(629)
dbm2(507)=dbm1(630)
dbm2(508)=dbm1(631)
dbm2(509)=dbm1(632)
dbm2(510)=dbm1(633)
dbm2(511)=dbm1(634)
dbm2(512)=dbm1(635)
dbm2(513)=dbm1(636)
dbm2(514)=dbm1(637)
dbm2(515)=dbm1(638)
dbm2(516)=dbm1(639)
dbm2(517)=dbm1(640)
dbm2(518)=dbm1(641)
dbm2(519)=dbm1(642)
dbm2(520)=dbm1(643)
dbm2(521)=dbm1(644)
dbm2(522)=dbm1(645)
dbm2(523)=dbm1(646)
dbm2(524)=dbm1(647)
dbm2(525)=dbm1(648)
dbm2(526)=dbm1(649)
dbm2(527)=dbm1(650)
dbm2(528)=dbm1(651)
dbm2(529)=dbm1(652)
dbm2(530)=dbm1(653)
dbm2(531)=dbm1(654)
dbm2(532)=dbm1(655)
dbm2(533)=dbm1(656)
dbm2(534)=dbm1(657)
dbm2(535)=dbm1(658)
dbm2(536)=dbm1(659)
dbm2(537)=dbm1(660)
dbm2(538)=dbm1(661)
dbm2(539)=dbm1(662)
dbm2(540)=dbm1(663)
dbm2(541)=dbm1(664)
dbm2(542)=dbm1(665)
dbm2(543)=dbm1(666)
dbm2(544)=dbm1(667)
dbm2(545)=dbm1(668)
dbm2(546)=dbm1(669)
dbm2(547)=dbm1(670)
dbm2(548)=dbm1(671)
dbm2(549)=dbm1(672)
dbm2(550)=dbm1(673)
dbm2(551)=dbm1(674)
dbm2(552)=dbm1(675)
dbm2(553)=dbm1(676)
dbm2(554)=dbm1(677)
dbm2(555)=dbm1(678)
dbm2(556)=dbm1(679)
dbm2(557)=dbm1(680)
dbm2(558)=dbm1(681)
dbm2(559)=dbm1(682)
dbm2(560)=dbm1(683)
dbm2(561)=dbm1(684)
dbm2(562)=dbm1(685)
dbm2(563)=dbm1(686)
dbm2(564)=dbm1(687)
dbm2(565)=dbm1(688)
dbm2(566)=dbm1(689)
dbm2(567)=dbm1(690)
dbm2(568)=dbm1(691)
dbm2(569)=dbm1(692)
dbm2(570)=dbm1(693)
dbm2(571)=dbm1(694)
dbm2(572)=dbm1(695)
dbm2(573)=dbm1(696)
dbm2(574)=dbm1(697)
dbm2(575)=dbm1(698)
dbm2(576)=dbm1(699)
dbm2(577)=dbm1(700)
dbm2(578)=dbm1(701)
dbm2(579)=dbm1(702)
dbm2(580)=dbm1(703)
dbm2(581)=dbm1(704)
dbm2(582)=dbm1(705)
dbm2(583)=dbm1(706)
dbm2(584)=dbm1(707)
dbm2(585)=dbm1(708)
dbm2(586)=dbm1(709)
dbm2(587)=dbm1(710)
dbm2(588)=dbm1(711)
dbm2(589)=dbm1(712)
dbm2(590)=dbm1(713)
dbm2(591)=dbm1(714)
dbm2(592)=dbm1(715)
dbm2(593)=dbm1(716)
dbm2(594)=dbm1(717)
dbm2(595)=dbm1(718)
dbm2(596)=dbm1(719)
dbm2(597)=dbm1(720)
dbm2(598)=dbm1(721)
dbm2(599)=dbm1(722)
dbm2(600)=dbm1(723)
dbm2(601)=dbm1(724)
dbm2(602)=dbm1(725)
dbm2(603)=dbm1(726)
dbm2(604)=dbm1(727)
dbm2(605)=dbm1(728)
dbm2(606)=dbm1(729)
dbm2(607)=dbm1(730)
dbm2(608)=dbm1(731)
dbm2(609)=dbm1(732)
dbm2(610)=dbm1(733)
dbm2(611)=dbm1(734)
dbm2(612)=dbm1(735)
dbm2(613)=dbm1(736)
dbm2(614)=dbm1(737)
dbm2(615)=dbm1(738)
dbm2(616)=dbm1(739)
dbm2(617)=dbm1(740)
dbm2(618)=dbm1(741)
dbm2(619)=dbm1(742)
dbm2(620)=dbm1(743)
dbm2(621)=dbm1(744)
dbm2(622)=dbm1(745)
dbm2(623)=dbm1(746)
dbm2(624)=dbm1(747)
dbm2(625)=dbm1(748)
dbm2(626)=dbm1(749)
dbm2(627)=dbm1(750)
dbm2(628)=dbm1(751)
dbm2(629)=dbm1(752)
dbm2(630)=dbm1(753)
dbm2(631)=dbm1(754)
dbm2(632)=dbm1(755)
dbm2(633)=dbm1(756)
dbm2(634)=dbm1(757)
dbm2(635)=dbm1(758)
dbm2(636)=dbm1(759)
dbm2(637)=dbm1(760)
dbm2(638)=dbm1(761)
dbm2(639)=dbm1(762)
dbm2(640)=dbm1(763)
dbm2(641)=dbm1(764)
dbm2(642)=dbm1(765)
dbm2(643)=dbm1(766)
dbm2(644)=dbm1(767)
dbm2(645)=dbm1(768)
dbm2(646)=dbm1(769)
dbm2(647)=dbm1(770)
dbm2(648)=dbm1(771)
dbm2(649)=dbm1(772)
dbm2(650)=dbm1(773)
dbm2(651)=dbm1(774)
dbm2(652)=dbm1(775)
dbm2(653)=dbm1(776)
dbm2(654)=dbm1(777)
dbm2(655)=dbm1(778)
dbm2(656)=dbm1(779)
dbm2(657)=dbm1(780)
dbm2(658)=dbm1(781)
dbm2(659)=dbm1(782)
dbm2(660)=dbm1(783)
dbm2(661)=dbm1(784)
dbm2(662)=dbm1(785)
dbm2(663)=dbm1(786)
dbm2(664)=dbm1(787)
dbm2(665)=dbm1(788)
dbm2(666)=dbm1(789)
dbm2(667)=dbm1(790)
dbm2(668)=dbm1(791)
dbm2(669)=dbm1(792)
dbm2(670)=dbm1(793)
dbm2(671)=dbm1(794)
dbm2(672)=dbm1(795)
dbm2(673)=dbm1(796)
dbm2(674)=dbm1(797)
dbm2(675)=dbm1(798)
dbm2(676)=dbm1(799)
dbm2(677)=dbm1(800)
dbm2(678)=dbm1(801)
dbm2(679)=dbm1(802)
dbm2(680)=dbm1(803)
dbm2(681)=dbm1(804)
dbm2(682)=dbm1(805)
dbm2(683)=dbm1(806)
dbm2(684)=dbm1(807)
dbm2(685)=dbm1(808)
dbm2(686)=dbm1(809)
dbm2(687)=dbm1(810)
dbm2(688)=dbm1(811)
dbm2(689)=dbm1(812)
dbm2(690)=dbm1(813)
dbm2(691)=dbm1(814)
dbm2(692)=dbm1(815)
dbm2(693)=dbm1(816)
dbm2(694)=dbm1(817)
dbm2(695)=dbm1(818)
dbm2(696)=dbm1(819)
dbm2(697)=dbm1(820)
dbm2(698)=dbm1(821)
dbm2(699)=dbm1(822)
dbm2(700)=dbm1(823)
dbm2(701)=dbm1(824)
dbm2(702)=dbm1(825)
dbm2(703)=dbm1(826)
dbm2(704)=dbm1(827)
dbm2(705)=dbm1(828)
dbm2(706)=dbm1(829)
dbm2(707)=dbm1(830)
dbm2(708)=dbm1(831)
dbm2(709)=dbm1(832)
dbm2(710)=dbm1(833)
dbm2(711)=dbm1(834)
dbm2(712)=dbm1(835)
dbm2(713)=dbm1(836)
dbm2(714)=dbm1(837)
dbm2(715)=dbm1(838)
dbm2(716)=dbm1(839)
dbm2(717)=dbm1(840)
dbm2(718)=dbm1(841)
dbm2(719)=dbm1(842)
dbm2(720)=dbm1(843)
dbm2(721)=dbm1(844)
dbm2(722)=dbm1(845)
dbm2(723)=dbm1(846)
dbm2(724)=dbm1(847)
dbm2(725)=dbm1(848)
dbm2(726)=dbm1(849)
dbm2(727)=dbm1(850)
dbm2(728)=dbm1(851)
dbm2(729)=dbm1(853)
dbm2(730)=dbm1(854)
dbm2(731)=dbm1(855)
dbm2(732)=dbm1(856)
dbm2(733)=dbm1(857)
dbm2(734)=dbm1(858)
dbm2(735)=dbm1(859)
dbm2(736)=dbm1(860)
dbm2(737)=dbm1(861)
dbm2(738)=dbm1(862)
dbm2(739)=dbm1(863)
dbm2(740)=dbm1(865)
dbm2(741)=dbm1(867)
dbm2(742)=dbm1(868)
dbm2(743)=dbm1(869)
dbm2(744)=dbm1(870)
dbm2(745)=dbm1(871)
dbm2(746)=dbm1(872)
dbm2(747)=dbm1(873)
dbm2(748)=dbm1(874)
dbm2(749)=dbm1(875)
dbm2(750)=dbm1(876)
dbm2(751)=dbm1(878)
dbm2(752)=dbm1(879)
dbm2(753)=dbm1(881)
dbm2(754)=dbm1(883)
dbm2(755)=dbm1(884)
dbm2(756)=dbm1(885)
dbm2(757)=dbm1(886)
dbm2(758)=dbm1(887)
dbm2(759)=dbm1(888)
dbm2(760)=dbm1(889)
dbm2(761)=dbm1(890)
dbm2(762)=dbm1(891)
dbm2(763)=dbm1(893)
dbm2(764)=dbm1(894)
dbm2(765)=dbm1(896)
dbm2(766)=dbm1(897)
dbm2(767)=dbm1(899)
dbm2(768)=dbm1(900)
dbm2(769)=dbm1(902)
dbm2(770)=dbm1(903)
dbm2(771)=dbm1(904)
dbm2(772)=dbm1(905)
dbm2(773)=dbm1(906)
dbm2(774)=dbm1(907)
dbm2(775)=dbm1(908)
dbm2(776)=dbm1(909)
dbm2(777)=dbm1(911)
dbm2(778)=dbm1(912)
dbm2(779)=dbm1(914)
dbm2(780)=dbm1(915)
dbm2(781)=dbm1(916)
dbm2(782)=dbm1(918)
dbm2(783)=dbm1(919)
dbm2(784)=dbm1(920)
dbm2(785)=dbm1(921)
dbm2(786)=dbm1(922)
dbm2(787)=dbm1(923)
dbm2(788)=dbm1(925)
dbm2(789)=dbm1(926)
dbm2(790)=dbm1(928)
dbm2(791)=dbm1(929)
dbm2(792)=dbm1(930)
dbm2(793)=dbm1(931)
dbm2(794)=dbm1(932)
dbm2(795)=dbm1(934)
dbm2(796)=dbm1(935)
dbm2(797)=dbm1(936)
dbm2(798)=dbm1(937)
dbm2(799)=dbm1(938)
dbm2(800)=dbm1(961)
dbm2(801)=dbm1(962)
dbm2(802)=dbm1(963)
dbm2(803)=dbm1(964)
dbm2(804)=dbm1(965)
dbm2(805)=dbm1(966)
dbm2(806)=dbm1(967)
dbm2(807)=dbm1(968)
dbm2(808)=dbm1(969)
dbm2(809)=dbm1(970)
dbm2(810)=dbm1(971)
dbm2(811)=dbm1(972)
dbm2(812)=dbm1(973)
dbm2(813)=dbm1(974)
dbm2(814)=dbm1(975)
dbm2(815)=dbm1(976)
dbm2(816)=dbm1(977)
dbm2(817)=dbm1(978)
dbm2(818)=dbm1(979)
dbm2(819)=dbm1(980)
dbm2(820)=dbm1(981)
dbm2(821)=dbm1(982)
dbm2(822)=dbm1(983)
dbm2(823)=dbm1(984)
dbm2(824)=dbm1(985)
dbm2(825)=dbm1(986)
dbm2(826)=dbm1(987)
dbm2(827)=dbm1(988)
dbm2(828)=dbm1(989)
dbm2(829)=dbm1(990)
dbm2(830)=dbm1(991)
dbm2(831)=dbm1(992)
dbm2(832)=dbm1(993)
dbm2(833)=dbm1(994)
dbm2(834)=dbm1(995)
dbm2(835)=dbm1(996)
dbm2(836)=dbm1(997)
dbm2(837)=dbm1(998)
dbm2(838)=dbm1(999)
dbm2(839)=dbm1(1000)
dbm2(840)=dbm1(1001)
dbm2(841)=dbm1(1002)
dbm2(842)=dbm1(1003)
dbm2(843)=dbm1(1004)
dbm2(844)=dbm1(1005)
dbm2(845)=dbm1(1006)
dbm2(846)=dbm1(1007)
dbm2(847)=dbm1(1008)
dbm2(848)=dbm1(1009)
dbm2(849)=dbm1(1010)
dbm2(850)=dbm1(1011)
dbm2(851)=dbm1(1012)
dbm2(852)=dbm1(1013)
dbm2(853)=dbm1(1014)
dbm2(854)=dbm1(1015)
dbm2(855)=dbm1(1016)
dbm2(856)=dbm1(1017)
dbm2(857)=dbm1(1018)
dbm2(858)=dbm1(1019)
dbm2(859)=dbm1(1020)
dbm2(860)=dbm1(1021)
dbm2(861)=dbm1(1022)
dbm2(862)=dbm1(1023)
dbm2(863)=dbm1(1024)
dbm2(864)=dbm1(1025)
dbm2(865)=dbm1(1026)
dbm2(866)=dbm1(1027)
dbm2(867)=dbm1(1028)
dbm2(868)=dbm1(1029)
dbm2(869)=dbm1(1030)
dbm2(870)=dbm1(1031)
dbm2(871)=dbm1(1032)
dbm2(872)=dbm1(1033)
dbm2(873)=dbm1(1034)
dbm2(874)=dbm1(1035)
dbm2(875)=dbm1(1036)
dbm2(876)=dbm1(1037)
dbm2(877)=dbm1(1038)
dbm2(878)=dbm1(1039)
dbm2(879)=dbm1(1040)
dbm2(880)=dbm1(1041)
dbm2(881)=dbm1(1042)
dbm2(882)=dbm1(1043)
dbm2(883)=dbm1(1044)
dbm2(884)=dbm1(1045)
dbm2(885)=dbm1(1046)
dbm2(886)=dbm1(1047)
dbm2(887)=dbm1(1048)
dbm2(888)=dbm1(1049)
dbm2(889)=dbm1(1050)
dbm2(890)=dbm1(1051)
dbm2(891)=dbm1(1052)
dbm2(892)=dbm1(1053)
dbm2(893)=dbm1(1054)
dbm2(894)=dbm1(1055)
dbm2(895)=dbm1(1056)
dbm2(896)=dbm1(1057)
dbm2(897)=dbm1(1058)
dbm2(898)=dbm1(1059)
dbm2(899)=dbm1(1060)
dbm2(900)=dbm1(1061)
dbm2(901)=dbm1(1062)
dbm2(902)=dbm1(1063)
dbm2(903)=dbm1(1064)
dbm2(904)=dbm1(1065)
dbm2(905)=dbm1(1066)
dbm2(906)=dbm1(1067)
dbm2(907)=dbm1(1068)
dbm2(908)=dbm1(1069)
dbm2(909)=dbm1(1070)
dbm2(910)=dbm1(1071)
dbm2(911)=dbm1(1072)
dbm2(912)=dbm1(1073)
dbm2(913)=dbm1(1074)
dbm2(914)=dbm1(1075)
dbm2(915)=dbm1(1076)
dbm2(916)=dbm1(1077)
dbm2(917)=dbm1(1078)
dbm2(918)=dbm1(1079)
dbm2(919)=dbm1(1080)
dbm2(920)=dbm1(1081)
dbm2(921)=dbm1(1082)
dbm2(922)=dbm1(1083)
dbm2(923)=dbm1(1084)
dbm2(924)=dbm1(1085)
dbm2(925)=dbm1(1086)
dbm2(926)=dbm1(1087)
dbm2(927)=dbm1(1088)
dbm2(928)=dbm1(1089)
dbm2(929)=dbm1(1090)
dbm2(930)=dbm1(1091)
dbm2(931)=dbm1(1092)
dbm2(932)=dbm1(1093)
dbm2(933)=dbm1(1094)
dbm2(934)=dbm1(1095)
dbm2(935)=dbm1(1096)
dbm2(936)=dbm1(1097)
dbm2(937)=dbm1(1098)
dbm2(938)=dbm1(1099)
dbm2(939)=dbm1(1100)
dbm2(940)=dbm1(1101)
dbm2(941)=dbm1(1102)
dbm2(942)=dbm1(1103)
dbm2(943)=dbm1(1104)
dbm2(944)=dbm1(1105)
dbm2(945)=dbm1(1106)
dbm2(946)=dbm1(1107)
dbm2(947)=dbm1(1108)
dbm2(948)=dbm1(1109)
dbm2(949)=dbm1(1110)
dbm2(950)=dbm1(1111)
dbm2(951)=dbm1(1112)
dbm2(952)=dbm1(1113)
dbm2(953)=dbm1(1114)
dbm2(954)=dbm1(1115)
dbm2(955)=dbm1(1116)
dbm2(956)=dbm1(1117)
dbm2(957)=dbm1(1118)
dbm2(958)=dbm1(1119)
dbm2(959)=dbm1(1120)
dbm2(960)=dbm1(1121)
dbm2(961)=dbm1(1122)
dbm2(962)=dbm1(1123)
dbm2(963)=dbm1(1124)
dbm2(964)=dbm1(1125)
dbm2(965)=dbm1(1126)
dbm2(966)=dbm1(1127)
dbm2(967)=dbm1(1128)
dbm2(968)=dbm1(1129)
dbm2(969)=dbm1(1130)
dbm2(970)=dbm1(1131)
dbm2(971)=dbm1(1132)
dbm2(972)=dbm1(1133)
dbm2(973)=dbm1(1134)
dbm2(974)=dbm1(1135)
dbm2(975)=dbm1(1136)
dbm2(976)=dbm1(1137)
dbm2(977)=dbm1(1138)
dbm2(978)=dbm1(1139)
dbm2(979)=dbm1(1140)
dbm2(980)=dbm1(1141)
dbm2(981)=dbm1(1142)
dbm2(982)=dbm1(1143)
dbm2(983)=dbm1(1144)
dbm2(984)=dbm1(1145)
dbm2(985)=dbm1(1146)
dbm2(986)=dbm1(1147)
dbm2(987)=dbm1(1148)
dbm2(988)=dbm1(1149)
dbm2(989)=dbm1(1150)
dbm2(990)=dbm1(1151)
dbm2(991)=dbm1(1152)
dbm2(992)=dbm1(1153)
dbm2(993)=dbm1(1154)
dbm2(994)=dbm1(1155)
dbm2(995)=dbm1(1156)
dbm2(996)=dbm1(1157)
dbm2(997)=dbm1(1158)
dbm2(998)=dbm1(1159)
dbm2(999)=dbm1(1160)
dbm2(1000)=dbm1(1161)
dbm2(1001)=dbm1(1162)
dbm2(1002)=dbm1(1163)
dbm2(1003)=dbm1(1164)
dbm2(1004)=dbm1(1165)
dbm2(1005)=dbm1(1166)
dbm2(1006)=dbm1(1167)
dbm2(1007)=dbm1(1168)
dbm2(1008)=dbm1(1169)
dbm2(1009)=dbm1(1170)
dbm2(1010)=dbm1(1171)
dbm2(1011)=dbm1(1172)
dbm2(1012)=dbm1(1173)
dbm2(1013)=dbm1(1174)
dbm2(1014)=dbm1(1175)
dbm2(1015)=dbm1(1176)
dbm2(1016)=dbm1(1177)
dbm2(1017)=dbm1(1178)
dbm2(1018)=dbm1(1179)
dbm2(1019)=dbm1(1180)
dbm2(1020)=dbm1(1181)
dbm2(1021)=dbm1(1182)
dbm2(1022)=dbm1(1183)
dbm2(1023)=dbm1(1184)
dbm2(1024)=dbm1(1185)
dbm2(1025)=dbm1(1186)
dbm2(1026)=dbm1(1187)
dbm2(1027)=dbm1(1188)
dbm2(1028)=dbm1(1189)
dbm2(1029)=dbm1(1190)
dbm2(1030)=dbm1(1191)
dbm2(1031)=dbm1(1192)
dbm2(1032)=dbm1(1193)
dbm2(1033)=dbm1(1194)
dbm2(1034)=dbm1(1195)
dbm2(1035)=dbm1(1196)
dbm2(1036)=dbm1(1197)
dbm2(1037)=dbm1(1198)
dbm2(1038)=dbm1(1199)
dbm2(1039)=dbm1(1200)
dbm2(1040)=dbm1(1201)
dbm2(1041)=dbm1(1202)
dbm2(1042)=dbm1(1203)
dbm2(1043)=dbm1(1204)
dbm2(1044)=dbm1(1205)
dbm2(1045)=dbm1(1206)
dbm2(1046)=dbm1(1207)
dbm2(1047)=dbm1(1208)
dbm2(1048)=dbm1(1209)
dbm2(1049)=dbm1(1210)
dbm2(1050)=dbm1(1211)
dbm2(1051)=dbm1(1212)
dbm2(1052)=dbm1(1213)
dbm2(1053)=dbm1(1214)
dbm2(1054)=dbm1(1215)
dbm2(1055)=dbm1(1216)
dbm2(1056)=dbm1(1217)
dbm2(1057)=dbm1(1218)
dbm2(1058)=dbm1(1219)
dbm2(1059)=dbm1(1220)
dbm2(1060)=dbm1(1221)
dbm2(1061)=dbm1(1222)
dbm2(1062)=dbm1(1223)
dbm2(1063)=dbm1(1224)
dbm2(1064)=dbm1(1225)
dbm2(1065)=dbm1(1226)
dbm2(1066)=dbm1(1227)
dbm2(1067)=dbm1(1228)
dbm2(1068)=dbm1(1229)
dbm2(1069)=dbm1(1230)
dbm2(1070)=dbm1(1231)
dbm2(1071)=dbm1(1232)
dbm2(1072)=dbm1(1233)
dbm2(1073)=dbm1(1234)
dbm2(1074)=dbm1(1235)
dbm2(1075)=dbm1(1236)
dbm2(1076)=dbm1(1237)
dbm2(1077)=dbm1(1238)
dbm2(1078)=dbm1(1239)
dbm2(1079)=dbm1(1240)
dbm2(1080)=dbm1(1241)
dbm2(1081)=dbm1(1242)
dbm2(1082)=dbm1(1243)
dbm2(1083)=dbm1(1244)
dbm2(1084)=dbm1(1245)
dbm2(1085)=dbm1(1246)
dbm2(1086)=dbm1(1247)
dbm2(1087)=dbm1(1248)
dbm2(1088)=dbm1(1249)
dbm2(1089)=dbm1(1250)
dbm2(1090)=dbm1(1251)
dbm2(1091)=dbm1(1252)
dbm2(1092)=dbm1(1253)
dbm2(1093)=dbm1(1254)
dbm2(1094)=dbm1(1255)
dbm2(1095)=dbm1(1256)
dbm2(1096)=dbm1(1257)
dbm2(1097)=dbm1(1258)
dbm2(1098)=dbm1(1259)
dbm2(1099)=dbm1(1260)
dbm2(1100)=dbm1(1261)
dbm2(1101)=dbm1(1262)
dbm2(1102)=dbm1(1263)
dbm2(1103)=dbm1(1264)
dbm2(1104)=dbm1(1265)
dbm2(1105)=dbm1(1266)
dbm2(1106)=dbm1(1267)
dbm2(1107)=dbm1(1268)
dbm2(1108)=dbm1(1269)
dbm2(1109)=dbm1(1270)
dbm2(1110)=dbm1(1271)
dbm2(1111)=dbm1(1272)
dbm2(1112)=dbm1(1273)
dbm2(1113)=dbm1(1274)
dbm2(1114)=dbm1(1275)
dbm2(1115)=dbm1(1276)
dbm2(1116)=dbm1(1277)
dbm2(1117)=dbm1(1278)
dbm2(1118)=dbm1(1279)
dbm2(1119)=dbm1(1280)
dbm2(1120)=dbm1(1281)
dbm2(1121)=dbm1(1282)
dbm2(1122)=dbm1(1283)
dbm2(1123)=dbm1(1284)
dbm2(1124)=dbm1(1285)
dbm2(1125)=dbm1(1286)
dbm2(1126)=dbm1(1287)
dbm2(1127)=dbm1(1288)
dbm2(1128)=dbm1(1289)
dbm2(1129)=dbm1(1290)
dbm2(1130)=dbm1(1291)
dbm2(1131)=dbm1(1292)
dbm2(1132)=dbm1(1293)
dbm2(1133)=dbm1(1294)
dbm2(1134)=dbm1(1295)
dbm2(1135)=dbm1(1296)
dbm2(1136)=dbm1(1297)
dbm2(1137)=dbm1(1298)
dbm2(1138)=dbm1(1299)
dbm2(1139)=dbm1(1300)
dbm2(1140)=dbm1(1301)
dbm2(1141)=dbm1(1302)
dbm2(1142)=dbm1(1303)
dbm2(1143)=dbm1(1304)
dbm2(1144)=dbm1(1305)
dbm2(1145)=dbm1(1306)
dbm2(1146)=dbm1(1307)
dbm2(1147)=dbm1(1308)
dbm2(1148)=dbm1(1309)
dbm2(1149)=dbm1(1310)
dbm2(1150)=dbm1(1311)
dbm2(1151)=dbm1(1312)
dbm2(1152)=dbm1(1313)
dbm2(1153)=dbm1(1314)
dbm2(1154)=dbm1(1315)
dbm2(1155)=dbm1(1316)
dbm2(1156)=dbm1(1317)
dbm2(1157)=dbm1(1318)
dbm2(1158)=dbm1(1319)
dbm2(1159)=dbm1(1320)
dbm2(1160)=dbm1(1321)
dbm2(1161)=dbm1(1322)
dbm2(1162)=dbm1(1323)
dbm2(1163)=dbm1(1324)
dbm2(1164)=dbm1(1325)
dbm2(1165)=dbm1(1326)
dbm2(1166)=dbm1(1327)
dbm2(1167)=dbm1(1328)
dbm2(1168)=dbm1(1329)
dbm2(1169)=dbm1(1330)
dbm2(1170)=dbm1(1331)
dbm2(1171)=dbm1(1332)
dbm2(1172)=dbm1(1333)
dbm2(1173)=dbm1(1334)
dbm2(1174)=dbm1(1335)
dbm2(1175)=dbm1(1336)
dbm2(1176)=dbm1(1337)
dbm2(1177)=dbm1(1338)
dbm2(1178)=dbm1(1339)
dbm2(1179)=dbm1(1340)
dbm2(1180)=dbm1(1341)
dbm2(1181)=dbm1(1342)
dbm2(1182)=dbm1(1343)
dbm2(1183)=dbm1(1344)
dbm2(1184)=dbm1(1345)
dbm2(1185)=dbm1(1346)
dbm2(1186)=dbm1(1347)
dbm2(1187)=dbm1(1348)
dbm2(1188)=dbm1(1349)
dbm2(1189)=dbm1(1350)
dbm2(1190)=dbm1(1351)
dbm2(1191)=dbm1(1352)
dbm2(1192)=dbm1(1353)
dbm2(1193)=dbm1(1354)
dbm2(1194)=dbm1(1355)
dbm2(1195)=dbm1(1356)
dbm2(1196)=dbm1(1357)
dbm2(1197)=dbm1(1358)
dbm2(1198)=dbm1(1359)
dbm2(1199)=dbm1(1360)
dbm2(1200)=dbm1(1361)
dbm2(1201)=dbm1(1362)
dbm2(1202)=dbm1(1363)
dbm2(1203)=dbm1(1364)
dbm2(1204)=dbm1(1365)
dbm2(1205)=dbm1(1366)
dbm2(1206)=dbm1(1367)
dbm2(1207)=dbm1(1368)
dbm2(1208)=dbm1(1369)
dbm2(1209)=dbm1(1371)
dbm2(1210)=dbm1(1372)
dbm2(1211)=dbm1(1373)
dbm2(1212)=dbm1(1374)
dbm2(1213)=dbm1(1375)
dbm2(1214)=dbm1(1376)
dbm2(1215)=dbm1(1377)
dbm2(1216)=dbm1(1378)
dbm2(1217)=dbm1(1379)
dbm2(1218)=dbm1(1380)
dbm2(1219)=dbm1(1381)
dbm2(1220)=dbm1(1382)
dbm2(1221)=dbm1(1383)
dbm2(1222)=dbm1(1385)
dbm2(1223)=dbm1(1387)
dbm2(1224)=dbm1(1388)
dbm2(1225)=dbm1(1389)
dbm2(1226)=dbm1(1390)
dbm2(1227)=dbm1(1391)
dbm2(1228)=dbm1(1392)
dbm2(1229)=dbm1(1393)
dbm2(1230)=dbm1(1394)
dbm2(1231)=dbm1(1395)
dbm2(1232)=dbm1(1396)
dbm2(1233)=dbm1(1397)
dbm2(1234)=dbm1(1399)
dbm2(1235)=dbm1(1400)
dbm2(1236)=dbm1(1402)
dbm2(1237)=dbm1(1404)
dbm2(1238)=dbm1(1405)
dbm2(1239)=dbm1(1406)
dbm2(1240)=dbm1(1407)
dbm2(1241)=dbm1(1408)
dbm2(1242)=dbm1(1409)
dbm2(1243)=dbm1(1410)
dbm2(1244)=dbm1(1411)
dbm2(1245)=dbm1(1412)
dbm2(1246)=dbm1(1413)
dbm2(1247)=dbm1(1415)
dbm2(1248)=dbm1(1416)
dbm2(1249)=dbm1(1418)
dbm2(1250)=dbm1(1419)
dbm2(1251)=dbm1(1421)
dbm2(1252)=dbm1(1422)
dbm2(1253)=dbm1(1424)
dbm2(1254)=dbm1(1425)
dbm2(1255)=dbm1(1426)
dbm2(1256)=dbm1(1427)
dbm2(1257)=dbm1(1428)
dbm2(1258)=dbm1(1429)
dbm2(1259)=dbm1(1430)
dbm2(1260)=dbm1(1431)
dbm2(1261)=dbm1(1432)
dbm2(1262)=dbm1(1434)
dbm2(1263)=dbm1(1435)
dbm2(1264)=dbm1(1437)
dbm2(1265)=dbm1(1438)
dbm2(1266)=dbm1(1440)
dbm2(1267)=dbm1(1441)
dbm2(1268)=dbm1(1442)
dbm2(1269)=dbm1(1444)
dbm2(1270)=dbm1(1445)
dbm2(1271)=dbm1(1446)
dbm2(1272)=dbm1(1447)
dbm2(1273)=dbm1(1448)
dbm2(1274)=dbm1(1449)
dbm2(1275)=dbm1(1450)
dbm2(1276)=dbm1(1451)
dbm2(1277)=dbm1(1453)
dbm2(1278)=dbm1(1454)
dbm2(1279)=dbm1(1456)
dbm2(1280)=dbm1(1457)
dbm2(1281)=dbm1(1458)
dbm2(1282)=dbm1(1460)
dbm2(1283)=dbm1(1461)
dbm2(1284)=dbm1(1462)
dbm2(1285)=dbm1(1463)
dbm2(1286)=dbm1(1464)
dbm2(1287)=dbm1(1465)
dbm2(1288)=dbm1(1467)
dbm2(1289)=dbm1(1468)
dbm2(1290)=dbm1(1470)
dbm2(1291)=dbm1(1471)
dbm2(1292)=dbm1(1472)
dbm2(1293)=dbm1(1473)
dbm2(1294)=dbm1(1474)
dbm2(1295)=dbm1(1476)
dbm2(1296)=dbm1(1477)
dbm2(1297)=dbm1(1478)
dbm2(1298)=dbm1(1479)
dbm2(1299)=dbm1(1480)
dbm2(1300)=dbm1(1481)
dbm2(1301)=dbm1(1507)
dbm2(1302)=dbm1(1508)
dbm2(1303)=dbm1(1509)
dbm2(1304)=dbm1(1510)
dbm2(1305)=dbm1(1511)
dbm2(1306)=dbm1(1512)
dbm2(1307)=dbm1(1513)
dbm2(1308)=dbm1(1514)
dbm2(1309)=dbm1(1515)
dbm2(1310)=dbm1(1516)
dbm2(1311)=dbm1(1517)
dbm2(1312)=dbm1(1518)
dbm2(1313)=dbm1(1519)
dbm2(1314)=dbm1(1520)
dbm2(1315)=dbm1(1521)
dbm2(1316)=dbm1(1522)
dbm2(1317)=dbm1(1523)
dbm2(1318)=dbm1(1524)
dbm2(1319)=dbm1(1525)
dbm2(1320)=dbm1(1526)
dbm2(1321)=dbm1(1527)
dbm2(1322)=dbm1(1528)
dbm2(1323)=dbm1(1529)
dbm2(1324)=dbm1(1530)
dbm2(1325)=dbm1(1531)
dbm2(1326)=dbm1(1532)
dbm2(1327)=dbm1(1533)
dbm2(1328)=dbm1(1534)
dbm2(1329)=dbm1(1535)
dbm2(1330)=dbm1(1536)
dbm2(1331)=dbm1(1537)
dbm2(1332)=dbm1(1538)
dbm2(1333)=dbm1(1539)
dbm2(1334)=dbm1(1540)
dbm2(1335)=dbm1(1541)
dbm2(1336)=dbm1(1542)
dbm2(1337)=dbm1(1543)
dbm2(1338)=dbm1(1544)
dbm2(1339)=dbm1(1545)
dbm2(1340)=dbm1(1546)
dbm2(1341)=dbm1(1547)
dbm2(1342)=dbm1(1548)
dbm2(1343)=dbm1(1549)
dbm2(1344)=dbm1(1550)
dbm2(1345)=dbm1(1551)
dbm2(1346)=dbm1(1552)
dbm2(1347)=dbm1(1553)
dbm2(1348)=dbm1(1554)
dbm2(1349)=dbm1(1555)
dbm2(1350)=dbm1(1556)
dbm2(1351)=dbm1(1557)
dbm2(1352)=dbm1(1558)
dbm2(1353)=dbm1(1559)
dbm2(1354)=dbm1(1560)
dbm2(1355)=dbm1(1561)
dbm2(1356)=dbm1(1562)
dbm2(1357)=dbm1(1563)
dbm2(1358)=dbm1(1564)
dbm2(1359)=dbm1(1565)
dbm2(1360)=dbm1(1566)
dbm2(1361)=dbm1(1567)
dbm2(1362)=dbm1(1568)
dbm2(1363)=dbm1(1569)
dbm2(1364)=dbm1(1570)
dbm2(1365)=dbm1(1571)
dbm2(1366)=dbm1(1572)
dbm2(1367)=dbm1(1573)
dbm2(1368)=dbm1(1574)
dbm2(1369)=dbm1(1575)
dbm2(1370)=dbm1(1576)
dbm2(1371)=dbm1(1577)
dbm2(1372)=dbm1(1578)
dbm2(1373)=dbm1(1579)
dbm2(1374)=dbm1(1580)
dbm2(1375)=dbm1(1581)
dbm2(1376)=dbm1(1582)
dbm2(1377)=dbm1(1583)
dbm2(1378)=dbm1(1584)
dbm2(1379)=dbm1(1585)
dbm2(1380)=dbm1(1586)
dbm2(1381)=dbm1(1587)
dbm2(1382)=dbm1(1588)
dbm2(1383)=dbm1(1589)
dbm2(1384)=dbm1(1590)
dbm2(1385)=dbm1(1591)
dbm2(1386)=dbm1(1592)
dbm2(1387)=dbm1(1593)
dbm2(1388)=dbm1(1594)
dbm2(1389)=dbm1(1595)
dbm2(1390)=dbm1(1596)
dbm2(1391)=dbm1(1597)
dbm2(1392)=dbm1(1598)
dbm2(1393)=dbm1(1599)
dbm2(1394)=dbm1(1600)
dbm2(1395)=dbm1(1601)
dbm2(1396)=dbm1(1602)
dbm2(1397)=dbm1(1603)
dbm2(1398)=dbm1(1604)
dbm2(1399)=dbm1(1605)
dbm2(1400)=dbm1(1606)
dbm2(1401)=dbm1(1607)
dbm2(1402)=dbm1(1608)
dbm2(1403)=dbm1(1609)
dbm2(1404)=dbm1(1610)
dbm2(1405)=dbm1(1611)
dbm2(1406)=dbm1(1612)
dbm2(1407)=dbm1(1613)
dbm2(1408)=dbm1(1614)
dbm2(1409)=dbm1(1615)
dbm2(1410)=dbm1(1616)
dbm2(1411)=dbm1(1617)
dbm2(1412)=dbm1(1618)
dbm2(1413)=dbm1(1619)
dbm2(1414)=dbm1(1620)
dbm2(1415)=dbm1(1621)
dbm2(1416)=dbm1(1622)
dbm2(1417)=dbm1(1623)
dbm2(1418)=dbm1(1624)
dbm2(1419)=dbm1(1625)
dbm2(1420)=dbm1(1626)
dbm2(1421)=dbm1(1627)
dbm2(1422)=dbm1(1628)
dbm2(1423)=dbm1(1629)
dbm2(1424)=dbm1(1630)
dbm2(1425)=dbm1(1631)
dbm2(1426)=dbm1(1632)
dbm2(1427)=dbm1(1633)
dbm2(1428)=dbm1(1634)
dbm2(1429)=dbm1(1635)
dbm2(1430)=dbm1(1636)
dbm2(1431)=dbm1(1637)
dbm2(1432)=dbm1(1638)
dbm2(1433)=dbm1(1639)
dbm2(1434)=dbm1(1640)
dbm2(1435)=dbm1(1641)
dbm2(1436)=dbm1(1642)
dbm2(1437)=dbm1(1643)
dbm2(1438)=dbm1(1644)
dbm2(1439)=dbm1(1645)
dbm2(1440)=dbm1(1646)
dbm2(1441)=dbm1(1647)
dbm2(1442)=dbm1(1648)
dbm2(1443)=dbm1(1649)
dbm2(1444)=dbm1(1650)
dbm2(1445)=dbm1(1651)
dbm2(1446)=dbm1(1652)
dbm2(1447)=dbm1(1653)
dbm2(1448)=dbm1(1654)
dbm2(1449)=dbm1(1655)
dbm2(1450)=dbm1(1656)
dbm2(1451)=dbm1(1657)
dbm2(1452)=dbm1(1658)
dbm2(1453)=dbm1(1659)
dbm2(1454)=dbm1(1660)
dbm2(1455)=dbm1(1661)
dbm2(1456)=dbm1(1662)
dbm2(1457)=dbm1(1663)
dbm2(1458)=dbm1(1664)
dbm2(1459)=dbm1(1665)
dbm2(1460)=dbm1(1666)
dbm2(1461)=dbm1(1667)
dbm2(1462)=dbm1(1668)
dbm2(1463)=dbm1(1669)
dbm2(1464)=dbm1(1670)
dbm2(1465)=dbm1(1671)
dbm2(1466)=dbm1(1672)
dbm2(1467)=dbm1(1673)
dbm2(1468)=dbm1(1674)
dbm2(1469)=dbm1(1675)
dbm2(1470)=dbm1(1676)
dbm2(1471)=dbm1(1677)
dbm2(1472)=dbm1(1678)
dbm2(1473)=dbm1(1679)
dbm2(1474)=dbm1(1680)
dbm2(1475)=dbm1(1681)
dbm2(1476)=dbm1(1682)
dbm2(1477)=dbm1(1683)
dbm2(1478)=dbm1(1684)
dbm2(1479)=dbm1(1685)
dbm2(1480)=dbm1(1686)
dbm2(1481)=dbm1(1687)
dbm2(1482)=dbm1(1688)
dbm2(1483)=dbm1(1689)
dbm2(1484)=dbm1(1690)
dbm2(1485)=dbm1(1691)
dbm2(1486)=dbm1(1692)
dbm2(1487)=dbm1(1693)
dbm2(1488)=dbm1(1694)
dbm2(1489)=dbm1(1695)
dbm2(1490)=dbm1(1696)
dbm2(1491)=dbm1(1697)
dbm2(1492)=dbm1(1698)
dbm2(1493)=dbm1(1699)
dbm2(1494)=dbm1(1700)
dbm2(1495)=dbm1(1701)
dbm2(1496)=dbm1(1702)
dbm2(1497)=dbm1(1703)
dbm2(1498)=dbm1(1704)
dbm2(1499)=dbm1(1705)
dbm2(1500)=dbm1(1706)
dbm2(1501)=dbm1(1707)
dbm2(1502)=dbm1(1708)
dbm2(1503)=dbm1(1709)
dbm2(1504)=dbm1(1710)
dbm2(1505)=dbm1(1711)
dbm2(1506)=dbm1(1712)
dbm2(1507)=dbm1(1713)
dbm2(1508)=dbm1(1714)
dbm2(1509)=dbm1(1715)
dbm2(1510)=dbm1(1716)
dbm2(1511)=dbm1(1717)
dbm2(1512)=dbm1(1718)
dbm2(1513)=dbm1(1719)
dbm2(1514)=dbm1(1720)
dbm2(1515)=dbm1(1721)
dbm2(1516)=dbm1(1722)
dbm2(1517)=dbm1(1723)
dbm2(1518)=dbm1(1724)
dbm2(1519)=dbm1(1725)
dbm2(1520)=dbm1(1726)
dbm2(1521)=dbm1(1727)
dbm2(1522)=dbm1(1728)
dbm2(1523)=dbm1(1729)
dbm2(1524)=dbm1(1730)
dbm2(1525)=dbm1(1731)
dbm2(1526)=dbm1(1732)
dbm2(1527)=dbm1(1733)
dbm2(1528)=dbm1(1734)
dbm2(1529)=dbm1(1735)
dbm2(1530)=dbm1(1736)
dbm2(1531)=dbm1(1737)
dbm2(1532)=dbm1(1738)
dbm2(1533)=dbm1(1739)
dbm2(1534)=dbm1(1740)
dbm2(1535)=dbm1(1741)
dbm2(1536)=dbm1(1742)
dbm2(1537)=dbm1(1743)
dbm2(1538)=dbm1(1744)
dbm2(1539)=dbm1(1745)
dbm2(1540)=dbm1(1746)
dbm2(1541)=dbm1(1747)
dbm2(1542)=dbm1(1748)
dbm2(1543)=dbm1(1749)
dbm2(1544)=dbm1(1750)
dbm2(1545)=dbm1(1751)
dbm2(1546)=dbm1(1752)
dbm2(1547)=dbm1(1753)
dbm2(1548)=dbm1(1754)
dbm2(1549)=dbm1(1755)
dbm2(1550)=dbm1(1756)
dbm2(1551)=dbm1(1757)
dbm2(1552)=dbm1(1758)
dbm2(1553)=dbm1(1759)
dbm2(1554)=dbm1(1760)
dbm2(1555)=dbm1(1761)
dbm2(1556)=dbm1(1762)
dbm2(1557)=dbm1(1763)
dbm2(1558)=dbm1(1764)
dbm2(1559)=dbm1(1765)
dbm2(1560)=dbm1(1766)
dbm2(1561)=dbm1(1767)
dbm2(1562)=dbm1(1768)
dbm2(1563)=dbm1(1769)
dbm2(1564)=dbm1(1770)
dbm2(1565)=dbm1(1771)
dbm2(1566)=dbm1(1772)
dbm2(1567)=dbm1(1773)
dbm2(1568)=dbm1(1774)
dbm2(1569)=dbm1(1775)
dbm2(1570)=dbm1(1776)
dbm2(1571)=dbm1(1777)
dbm2(1572)=dbm1(1778)
dbm2(1573)=dbm1(1779)
dbm2(1574)=dbm1(1780)
dbm2(1575)=dbm1(1781)
dbm2(1576)=dbm1(1782)
dbm2(1577)=dbm1(1783)
dbm2(1578)=dbm1(1784)
dbm2(1579)=dbm1(1785)
dbm2(1580)=dbm1(1786)
dbm2(1581)=dbm1(1787)
dbm2(1582)=dbm1(1788)
dbm2(1583)=dbm1(1789)
dbm2(1584)=dbm1(1790)
dbm2(1585)=dbm1(1791)
dbm2(1586)=dbm1(1792)
dbm2(1587)=dbm1(1793)
dbm2(1588)=dbm1(1794)
dbm2(1589)=dbm1(1795)
dbm2(1590)=dbm1(1796)
dbm2(1591)=dbm1(1797)
dbm2(1592)=dbm1(1798)
dbm2(1593)=dbm1(1799)
dbm2(1594)=dbm1(1800)
dbm2(1595)=dbm1(1801)
dbm2(1596)=dbm1(1802)
dbm2(1597)=dbm1(1803)
dbm2(1598)=dbm1(1804)
dbm2(1599)=dbm1(1805)
dbm2(1600)=dbm1(1806)
dbm2(1601)=dbm1(1807)
dbm2(1602)=dbm1(1808)
dbm2(1603)=dbm1(1809)
dbm2(1604)=dbm1(1810)
dbm2(1605)=dbm1(1811)
dbm2(1606)=dbm1(1812)
dbm2(1607)=dbm1(1813)
dbm2(1608)=dbm1(1814)
dbm2(1609)=dbm1(1815)
dbm2(1610)=dbm1(1816)
dbm2(1611)=dbm1(1817)
dbm2(1612)=dbm1(1818)
dbm2(1613)=dbm1(1819)
dbm2(1614)=dbm1(1820)
dbm2(1615)=dbm1(1821)
dbm2(1616)=dbm1(1822)
dbm2(1617)=dbm1(1823)
dbm2(1618)=dbm1(1824)
dbm2(1619)=dbm1(1825)
dbm2(1620)=dbm1(1826)
dbm2(1621)=dbm1(1827)
dbm2(1622)=dbm1(1828)
dbm2(1623)=dbm1(1829)
dbm2(1624)=dbm1(1830)
dbm2(1625)=dbm1(1831)
dbm2(1626)=dbm1(1832)
dbm2(1627)=dbm1(1833)
dbm2(1628)=dbm1(1834)
dbm2(1629)=dbm1(1835)
dbm2(1630)=dbm1(1836)
dbm2(1631)=dbm1(1837)
dbm2(1632)=dbm1(1838)
dbm2(1633)=dbm1(1839)
dbm2(1634)=dbm1(1840)
dbm2(1635)=dbm1(1841)
dbm2(1636)=dbm1(1842)
dbm2(1637)=dbm1(1843)
dbm2(1638)=dbm1(1844)
dbm2(1639)=dbm1(1845)
dbm2(1640)=dbm1(1846)
dbm2(1641)=dbm1(1847)
dbm2(1642)=dbm1(1848)
dbm2(1643)=dbm1(1849)
dbm2(1644)=dbm1(1850)
dbm2(1645)=dbm1(1851)
dbm2(1646)=dbm1(1852)
dbm2(1647)=dbm1(1853)
dbm2(1648)=dbm1(1854)
dbm2(1649)=dbm1(1855)
dbm2(1650)=dbm1(1856)
dbm2(1651)=dbm1(1857)
dbm2(1652)=dbm1(1858)
dbm2(1653)=dbm1(1859)
dbm2(1654)=dbm1(1860)
dbm2(1655)=dbm1(1861)
dbm2(1656)=dbm1(1862)
dbm2(1657)=dbm1(1863)
dbm2(1658)=dbm1(1864)
dbm2(1659)=dbm1(1865)
dbm2(1660)=dbm1(1866)
dbm2(1661)=dbm1(1867)
dbm2(1662)=dbm1(1868)
dbm2(1663)=dbm1(1869)
dbm2(1664)=dbm1(1870)
dbm2(1665)=dbm1(1871)
dbm2(1666)=dbm1(1872)
dbm2(1667)=dbm1(1873)
dbm2(1668)=dbm1(1874)
dbm2(1669)=dbm1(1875)
dbm2(1670)=dbm1(1876)
dbm2(1671)=dbm1(1877)
dbm2(1672)=dbm1(1878)
dbm2(1673)=dbm1(1879)
dbm2(1674)=dbm1(1880)
dbm2(1675)=dbm1(1881)
dbm2(1676)=dbm1(1882)
dbm2(1677)=dbm1(1883)
dbm2(1678)=dbm1(1884)
dbm2(1679)=dbm1(1885)
dbm2(1680)=dbm1(1886)
dbm2(1681)=dbm1(1887)
dbm2(1682)=dbm1(1888)
dbm2(1683)=dbm1(1889)
dbm2(1684)=dbm1(1890)
dbm2(1685)=dbm1(1891)
dbm2(1686)=dbm1(1892)
dbm2(1687)=dbm1(1893)
dbm2(1688)=dbm1(1894)
dbm2(1689)=dbm1(1895)
dbm2(1690)=dbm1(1896)
dbm2(1691)=dbm1(1897)
dbm2(1692)=dbm1(1898)
dbm2(1693)=dbm1(1899)
dbm2(1694)=dbm1(1900)
dbm2(1695)=dbm1(1901)
dbm2(1696)=dbm1(1902)
dbm2(1697)=dbm1(1903)
dbm2(1698)=dbm1(1904)
dbm2(1699)=dbm1(1905)
dbm2(1700)=dbm1(1906)
dbm2(1701)=dbm1(1907)
dbm2(1702)=dbm1(1908)
dbm2(1703)=dbm1(1909)
dbm2(1704)=dbm1(1910)
dbm2(1705)=dbm1(1911)
dbm2(1706)=dbm1(1912)
dbm2(1707)=dbm1(1913)
dbm2(1708)=dbm1(1914)
dbm2(1709)=dbm1(1915)
dbm2(1710)=dbm1(1916)
dbm2(1711)=dbm1(1917)
dbm2(1712)=dbm1(1918)
dbm2(1713)=dbm1(1919)
dbm2(1714)=dbm1(1920)
dbm2(1715)=dbm1(1921)
dbm2(1716)=dbm1(1922)
dbm2(1717)=dbm1(1923)
dbm2(1718)=dbm1(1924)
dbm2(1719)=dbm1(1925)
dbm2(1720)=dbm1(1926)
dbm2(1721)=dbm1(1927)
dbm2(1722)=dbm1(1928)
dbm2(1723)=dbm1(1929)
dbm2(1724)=dbm1(1930)
dbm2(1725)=dbm1(1931)
dbm2(1726)=dbm1(1932)
dbm2(1727)=dbm1(1933)
dbm2(1728)=dbm1(1934)
dbm2(1729)=dbm1(1935)
dbm2(1730)=dbm1(1936)
dbm2(1731)=dbm1(1937)
dbm2(1732)=dbm1(1938)
dbm2(1733)=dbm1(1939)
dbm2(1734)=dbm1(1940)
dbm2(1735)=dbm1(1941)
dbm2(1736)=dbm1(1942)
dbm2(1737)=dbm1(1943)
dbm2(1738)=dbm1(1944)
dbm2(1739)=dbm1(1945)
dbm2(1740)=dbm1(1946)
dbm2(1741)=dbm1(1947)
dbm2(1742)=dbm1(1948)
dbm2(1743)=dbm1(1949)
dbm2(1744)=dbm1(1950)
dbm2(1745)=dbm1(1951)
dbm2(1746)=dbm1(1952)
dbm2(1747)=dbm1(1953)
dbm2(1748)=dbm1(1954)
dbm2(1749)=dbm1(1955)
dbm2(1750)=dbm1(1956)
dbm2(1751)=dbm1(1957)
dbm2(1752)=dbm1(1958)
dbm2(1753)=dbm1(1959)
dbm2(1754)=dbm1(1960)
dbm2(1755)=dbm1(1961)
dbm2(1756)=dbm1(1962)
dbm2(1757)=dbm1(1963)
dbm2(1758)=dbm1(1964)
dbm2(1759)=dbm1(1965)
dbm2(1760)=dbm1(1966)
dbm2(1761)=dbm1(1967)
dbm2(1762)=dbm1(1968)
dbm2(1763)=dbm1(1969)
dbm2(1764)=dbm1(1970)
dbm2(1765)=dbm1(1971)
dbm2(1766)=dbm1(1972)
dbm2(1767)=dbm1(1973)
dbm2(1768)=dbm1(1974)
dbm2(1769)=dbm1(1975)
dbm2(1770)=dbm1(1976)
dbm2(1771)=dbm1(1977)
dbm2(1772)=dbm1(1978)
dbm2(1773)=dbm1(1979)
dbm2(1774)=dbm1(1980)
dbm2(1775)=dbm1(1981)
dbm2(1776)=dbm1(1982)
dbm2(1777)=dbm1(1983)
dbm2(1778)=dbm1(1984)
dbm2(1779)=dbm1(1985)
dbm2(1780)=dbm1(1986)
dbm2(1781)=dbm1(1987)
dbm2(1782)=dbm1(1988)
dbm2(1783)=dbm1(1989)
dbm2(1784)=dbm1(1990)
dbm2(1785)=dbm1(1991)
dbm2(1786)=dbm1(1992)
dbm2(1787)=dbm1(1993)
dbm2(1788)=dbm1(1994)
dbm2(1789)=dbm1(1995)
dbm2(1790)=dbm1(1996)
dbm2(1791)=dbm1(1997)
dbm2(1792)=dbm1(1998)
dbm2(1793)=dbm1(1999)
dbm2(1794)=dbm1(2000)
dbm2(1795)=dbm1(2001)
dbm2(1796)=dbm1(2002)
dbm2(1797)=dbm1(2003)
dbm2(1798)=dbm1(2004)
dbm2(1799)=dbm1(2005)
dbm2(1800)=dbm1(2006)
dbm2(1801)=dbm1(2007)
dbm2(1802)=dbm1(2008)
dbm2(1803)=dbm1(2009)
dbm2(1804)=dbm1(2010)
dbm2(1805)=dbm1(2011)
dbm2(1806)=dbm1(2012)
dbm2(1807)=dbm1(2013)
dbm2(1808)=dbm1(2014)
dbm2(1809)=dbm1(2015)
dbm2(1810)=dbm1(2016)
dbm2(1811)=dbm1(2017)
dbm2(1812)=dbm1(2018)
dbm2(1813)=dbm1(2019)
dbm2(1814)=dbm1(2020)
dbm2(1815)=dbm1(2021)
dbm2(1816)=dbm1(2022)
dbm2(1817)=dbm1(2023)
dbm2(1818)=dbm1(2024)
dbm2(1819)=dbm1(2025)
dbm2(1820)=dbm1(2026)
dbm2(1821)=dbm1(2027)
dbm2(1822)=dbm1(2028)
dbm2(1823)=dbm1(2029)
dbm2(1824)=dbm1(2030)
dbm2(1825)=dbm1(2031)
dbm2(1826)=dbm1(2032)
dbm2(1827)=dbm1(2033)
dbm2(1828)=dbm1(2034)
dbm2(1829)=dbm1(2035)
dbm2(1830)=dbm1(2036)
dbm2(1831)=dbm1(2037)
dbm2(1832)=dbm1(2038)
dbm2(1833)=dbm1(2039)
dbm2(1834)=dbm1(2040)
dbm2(1835)=dbm1(2041)
dbm2(1836)=dbm1(2042)
dbm2(1837)=dbm1(2043)
dbm2(1838)=dbm1(2044)
dbm2(1839)=dbm1(2045)
dbm2(1840)=dbm1(2046)
dbm2(1841)=dbm1(2047)
dbm2(1842)=dbm1(2048)
dbm2(1843)=dbm1(2049)
dbm2(1844)=dbm1(2050)
dbm2(1845)=dbm1(2051)
dbm2(1846)=dbm1(2052)
dbm2(1847)=dbm1(2053)
dbm2(1848)=dbm1(2054)
dbm2(1849)=dbm1(2055)
dbm2(1850)=dbm1(2056)
dbm2(1851)=dbm1(2057)
dbm2(1852)=dbm1(2058)
dbm2(1853)=dbm1(2059)
dbm2(1854)=dbm1(2060)
dbm2(1855)=dbm1(2061)
dbm2(1856)=dbm1(2062)
dbm2(1857)=dbm1(2063)
dbm2(1858)=dbm1(2064)
dbm2(1859)=dbm1(2065)
dbm2(1860)=dbm1(2066)
dbm2(1861)=dbm1(2067)
dbm2(1862)=dbm1(2068)
dbm2(1863)=dbm1(2069)
dbm2(1864)=dbm1(2070)
dbm2(1865)=dbm1(2071)
dbm2(1866)=dbm1(2072)
dbm2(1867)=dbm1(2073)
dbm2(1868)=dbm1(2074)
dbm2(1869)=dbm1(2075)
dbm2(1870)=dbm1(2076)
dbm2(1871)=dbm1(2077)
dbm2(1872)=dbm1(2078)
dbm2(1873)=dbm1(2079)
dbm2(1874)=dbm1(2080)
dbm2(1875)=dbm1(2081)
dbm2(1876)=dbm1(2082)
dbm2(1877)=dbm1(2083)
dbm2(1878)=dbm1(2084)
dbm2(1879)=dbm1(2085)
dbm2(1880)=dbm1(2086)
dbm2(1881)=dbm1(2087)
dbm2(1882)=dbm1(2088)
dbm2(1883)=dbm1(2089)
dbm2(1884)=dbm1(2090)
dbm2(1885)=dbm1(2091)
dbm2(1886)=dbm1(2092)
dbm2(1887)=dbm1(2093)
dbm2(1888)=dbm1(2094)
dbm2(1889)=dbm1(2095)
dbm2(1890)=dbm1(2096)
dbm2(1891)=dbm1(2097)
dbm2(1892)=dbm1(2098)
dbm2(1893)=dbm1(2099)
dbm2(1894)=dbm1(2100)
dbm2(1895)=dbm1(2101)
dbm2(1896)=dbm1(2102)
dbm2(1897)=dbm1(2103)
dbm2(1898)=dbm1(2104)
dbm2(1899)=dbm1(2105)
dbm2(1900)=dbm1(2106)
dbm2(1901)=dbm1(2107)
dbm2(1902)=dbm1(2108)
dbm2(1903)=dbm1(2109)
dbm2(1904)=dbm1(2110)
dbm2(1905)=dbm1(2111)
dbm2(1906)=dbm1(2112)
dbm2(1907)=dbm1(2113)
dbm2(1908)=dbm1(2114)
dbm2(1909)=dbm1(2115)
dbm2(1910)=dbm1(2116)
dbm2(1911)=dbm1(2117)
dbm2(1912)=dbm1(2118)
dbm2(1913)=dbm1(2119)
dbm2(1914)=dbm1(2120)
dbm2(1915)=dbm1(2121)
dbm2(1916)=dbm1(2122)
dbm2(1917)=dbm1(2123)
dbm2(1918)=dbm1(2125)
dbm2(1919)=dbm1(2126)
dbm2(1920)=dbm1(2127)
dbm2(1921)=dbm1(2128)
dbm2(1922)=dbm1(2129)
dbm2(1923)=dbm1(2130)
dbm2(1924)=dbm1(2131)
dbm2(1925)=dbm1(2132)
dbm2(1926)=dbm1(2133)
dbm2(1927)=dbm1(2134)
dbm2(1928)=dbm1(2135)
dbm2(1929)=dbm1(2136)
dbm2(1930)=dbm1(2137)
dbm2(1931)=dbm1(2138)
dbm2(1932)=dbm1(2140)
dbm2(1933)=dbm1(2142)
dbm2(1934)=dbm1(2143)
dbm2(1935)=dbm1(2144)
dbm2(1936)=dbm1(2145)
dbm2(1937)=dbm1(2146)
dbm2(1938)=dbm1(2147)
dbm2(1939)=dbm1(2148)
dbm2(1940)=dbm1(2149)
dbm2(1941)=dbm1(2150)
dbm2(1942)=dbm1(2151)
dbm2(1943)=dbm1(2152)
dbm2(1944)=dbm1(2153)
dbm2(1945)=dbm1(2154)
dbm2(1946)=dbm1(2156)
dbm2(1947)=dbm1(2157)
dbm2(1948)=dbm1(2159)
dbm2(1949)=dbm1(2161)
dbm2(1950)=dbm1(2162)
dbm2(1951)=dbm1(2163)
dbm2(1952)=dbm1(2164)
dbm2(1953)=dbm1(2165)
dbm2(1954)=dbm1(2166)
dbm2(1955)=dbm1(2167)
dbm2(1956)=dbm1(2168)
dbm2(1957)=dbm1(2169)
dbm2(1958)=dbm1(2170)
dbm2(1959)=dbm1(2171)
dbm2(1960)=dbm1(2173)
dbm2(1961)=dbm1(2174)
dbm2(1962)=dbm1(2176)
dbm2(1963)=dbm1(2177)
dbm2(1964)=dbm1(2179)
dbm2(1965)=dbm1(2181)
dbm2(1966)=dbm1(2182)
dbm2(1967)=dbm1(2183)
dbm2(1968)=dbm1(2184)
dbm2(1969)=dbm1(2185)
dbm2(1970)=dbm1(2186)
dbm2(1971)=dbm1(2187)
dbm2(1972)=dbm1(2188)
dbm2(1973)=dbm1(2189)
dbm2(1974)=dbm1(2190)
dbm2(1975)=dbm1(2191)
dbm2(1976)=dbm1(2193)
dbm2(1977)=dbm1(2194)
dbm2(1978)=dbm1(2196)
dbm2(1979)=dbm1(2197)
dbm2(1980)=dbm1(2199)
dbm2(1981)=dbm1(2200)
dbm2(1982)=dbm1(2201)
dbm2(1983)=dbm1(2203)
dbm2(1984)=dbm1(2204)
dbm2(1985)=dbm1(2206)
dbm2(1986)=dbm1(2207)
dbm2(1987)=dbm1(2208)
dbm2(1988)=dbm1(2209)
dbm2(1989)=dbm1(2210)
dbm2(1990)=dbm1(2211)
dbm2(1991)=dbm1(2212)
dbm2(1992)=dbm1(2213)
dbm2(1993)=dbm1(2214)
dbm2(1994)=dbm1(2216)
dbm2(1995)=dbm1(2217)
dbm2(1996)=dbm1(2219)
dbm2(1997)=dbm1(2220)
dbm2(1998)=dbm1(2222)
dbm2(1999)=dbm1(2223)
dbm2(2000)=dbm1(2224)
dbm2(2001)=dbm1(2226)
dbm2(2002)=dbm1(2227)
dbm2(2003)=dbm1(2228)
dbm2(2004)=dbm1(2229)
dbm2(2005)=dbm1(2230)
dbm2(2006)=dbm1(2231)
dbm2(2007)=dbm1(2232)
dbm2(2008)=dbm1(2233)
dbm2(2009)=dbm1(2235)
dbm2(2010)=dbm1(2236)
dbm2(2011)=dbm1(2238)
dbm2(2012)=dbm1(2239)
dbm2(2013)=dbm1(2240)
dbm2(2014)=dbm1(2242)
dbm2(2015)=dbm1(2243)
dbm2(2016)=dbm1(2244)
dbm2(2017)=dbm1(2245)
dbm2(2018)=dbm1(2246)
dbm2(2019)=dbm1(2247)
dbm2(2020)=dbm1(2249)
dbm2(2021)=dbm1(2250)
dbm2(2022)=dbm1(2252)
dbm2(2023)=dbm1(2253)
dbm2(2024)=dbm1(2254)
dbm2(2025)=dbm1(2255)
dbm2(2026)=dbm1(2256)
dbm2(2027)=dbm1(2258)
dbm2(2028)=dbm1(2259)
dbm2(2029)=dbm1(2260)
dbm2(2030)=dbm1(2261)
dbm2(2031)=dbm1(2262)
dbm2(2032)=dbm1(2263)
dbm2(2033)=dbm1(2291)
dbm2(2034)=dbm1(2292)
dbm2(2035)=dbm1(2293)
dbm2(2036)=dbm1(2294)
dbm2(2037)=dbm1(2295)
dbm2(2038)=dbm1(2296)
dbm2(2039)=dbm1(2297)
dbm2(2040)=dbm1(2298)
dbm2(2041)=dbm1(2299)
dbm2(2042)=dbm1(2300)
dbm2(2043)=dbm1(2301)
dbm2(2044)=dbm1(2302)
dbm2(2045)=dbm1(2303)
dbm2(2046)=dbm1(2304)
dbm2(2047)=dbm1(2305)
dbm2(2048)=dbm1(2306)
dbm2(2049)=dbm1(2307)
dbm2(2050)=dbm1(2308)
dbm2(2051)=dbm1(2309)
dbm2(2052)=dbm1(2310)
dbm2(2053)=dbm1(2311)
dbm2(2054)=dbm1(2312)
dbm2(2055)=dbm1(2313)
dbm2(2056)=dbm1(2314)
dbm2(2057)=dbm1(2315)
dbm2(2058)=dbm1(2316)
dbm2(2059)=dbm1(2317)
dbm2(2060)=dbm1(2318)
dbm2(2061)=dbm1(2319)
dbm2(2062)=dbm1(2320)
dbm2(2063)=dbm1(2321)
dbm2(2064)=dbm1(2322)
dbm2(2065)=dbm1(2323)
dbm2(2066)=dbm1(2324)
dbm2(2067)=dbm1(2325)
dbm2(2068)=dbm1(2326)
dbm2(2069)=dbm1(2327)
dbm2(2070)=dbm1(2328)
dbm2(2071)=dbm1(2329)
dbm2(2072)=dbm1(2330)
dbm2(2073)=dbm1(2331)
dbm2(2074)=dbm1(2332)
dbm2(2075)=dbm1(2333)
dbm2(2076)=dbm1(2334)
dbm2(2077)=dbm1(2335)
dbm2(2078)=dbm1(2336)
dbm2(2079)=dbm1(2337)
dbm2(2080)=dbm1(2338)
dbm2(2081)=dbm1(2339)
dbm2(2082)=dbm1(2340)
dbm2(2083)=dbm1(2341)
dbm2(2084)=dbm1(2342)
dbm2(2085)=dbm1(2343)
dbm2(2086)=dbm1(2344)
dbm2(2087)=dbm1(2345)
dbm2(2088)=dbm1(2346)
dbm2(2089)=dbm1(2347)
dbm2(2090)=dbm1(2348)
dbm2(2091)=dbm1(2349)
dbm2(2092)=dbm1(2350)
dbm2(2093)=dbm1(2351)
dbm2(2094)=dbm1(2352)
dbm2(2095)=dbm1(2353)
dbm2(2096)=dbm1(2354)
dbm2(2097)=dbm1(2355)
dbm2(2098)=dbm1(2356)
dbm2(2099)=dbm1(2357)
dbm2(2100)=dbm1(2358)
dbm2(2101)=dbm1(2359)
dbm2(2102)=dbm1(2360)
dbm2(2103)=dbm1(2361)
dbm2(2104)=dbm1(2362)
dbm2(2105)=dbm1(2363)
dbm2(2106)=dbm1(2364)
dbm2(2107)=dbm1(2365)
dbm2(2108)=dbm1(2366)
dbm2(2109)=dbm1(2367)
dbm2(2110)=dbm1(2368)
dbm2(2111)=dbm1(2369)
dbm2(2112)=dbm1(2370)
dbm2(2113)=dbm1(2371)
dbm2(2114)=dbm1(2372)
dbm2(2115)=dbm1(2373)
dbm2(2116)=dbm1(2374)
dbm2(2117)=dbm1(2375)
dbm2(2118)=dbm1(2376)
dbm2(2119)=dbm1(2377)
dbm2(2120)=dbm1(2378)
dbm2(2121)=dbm1(2379)
dbm2(2122)=dbm1(2380)
dbm2(2123)=dbm1(2381)
dbm2(2124)=dbm1(2382)
dbm2(2125)=dbm1(2383)
dbm2(2126)=dbm1(2384)
dbm2(2127)=dbm1(2385)
dbm2(2128)=dbm1(2386)
dbm2(2129)=dbm1(2387)
dbm2(2130)=dbm1(2388)
dbm2(2131)=dbm1(2389)
dbm2(2132)=dbm1(2390)
dbm2(2133)=dbm1(2391)
dbm2(2134)=dbm1(2392)
dbm2(2135)=dbm1(2393)
dbm2(2136)=dbm1(2394)
dbm2(2137)=dbm1(2395)
dbm2(2138)=dbm1(2396)
dbm2(2139)=dbm1(2397)
dbm2(2140)=dbm1(2398)
dbm2(2141)=dbm1(2399)
dbm2(2142)=dbm1(2400)
dbm2(2143)=dbm1(2401)
dbm2(2144)=dbm1(2402)
dbm2(2145)=dbm1(2403)
dbm2(2146)=dbm1(2404)
dbm2(2147)=dbm1(2405)
dbm2(2148)=dbm1(2406)
dbm2(2149)=dbm1(2407)
dbm2(2150)=dbm1(2408)
dbm2(2151)=dbm1(2409)
dbm2(2152)=dbm1(2410)
dbm2(2153)=dbm1(2411)
dbm2(2154)=dbm1(2412)
dbm2(2155)=dbm1(2413)
dbm2(2156)=dbm1(2414)
dbm2(2157)=dbm1(2415)
dbm2(2158)=dbm1(2416)
dbm2(2159)=dbm1(2417)
dbm2(2160)=dbm1(2418)
dbm2(2161)=dbm1(2419)
dbm2(2162)=dbm1(2420)
dbm2(2163)=dbm1(2421)
dbm2(2164)=dbm1(2422)
dbm2(2165)=dbm1(2423)
dbm2(2166)=dbm1(2424)
dbm2(2167)=dbm1(2425)
dbm2(2168)=dbm1(2426)
dbm2(2169)=dbm1(2427)
dbm2(2170)=dbm1(2428)
dbm2(2171)=dbm1(2429)
dbm2(2172)=dbm1(2430)
dbm2(2173)=dbm1(2431)
dbm2(2174)=dbm1(2432)
dbm2(2175)=dbm1(2433)
dbm2(2176)=dbm1(2434)
dbm2(2177)=dbm1(2435)
dbm2(2178)=dbm1(2436)
dbm2(2179)=dbm1(2437)
dbm2(2180)=dbm1(2438)
dbm2(2181)=dbm1(2439)
dbm2(2182)=dbm1(2440)
dbm2(2183)=dbm1(2441)
dbm2(2184)=dbm1(2442)
dbm2(2185)=dbm1(2443)
dbm2(2186)=dbm1(2444)
dbm2(2187)=dbm1(2445)
dbm2(2188)=dbm1(2446)
dbm2(2189)=dbm1(2447)
dbm2(2190)=dbm1(2448)
dbm2(2191)=dbm1(2449)
dbm2(2192)=dbm1(2450)
dbm2(2193)=dbm1(2451)
dbm2(2194)=dbm1(2452)
dbm2(2195)=dbm1(2453)
dbm2(2196)=dbm1(2454)
dbm2(2197)=dbm1(2455)
dbm2(2198)=dbm1(2456)
dbm2(2199)=dbm1(2457)
dbm2(2200)=dbm1(2458)
dbm2(2201)=dbm1(2459)
dbm2(2202)=dbm1(2460)
dbm2(2203)=dbm1(2461)
dbm2(2204)=dbm1(2462)
dbm2(2205)=dbm1(2463)
dbm2(2206)=dbm1(2464)
dbm2(2207)=dbm1(2465)
dbm2(2208)=dbm1(2466)
dbm2(2209)=dbm1(2467)
dbm2(2210)=dbm1(2468)
dbm2(2211)=dbm1(2469)
dbm2(2212)=dbm1(2470)
dbm2(2213)=dbm1(2471)
dbm2(2214)=dbm1(2472)
dbm2(2215)=dbm1(2473)
dbm2(2216)=dbm1(2474)
dbm2(2217)=dbm1(2475)
dbm2(2218)=dbm1(2476)
dbm2(2219)=dbm1(2477)
dbm2(2220)=dbm1(2478)
dbm2(2221)=dbm1(2479)
dbm2(2222)=dbm1(2480)
dbm2(2223)=dbm1(2481)
dbm2(2224)=dbm1(2482)
dbm2(2225)=dbm1(2483)
dbm2(2226)=dbm1(2484)
dbm2(2227)=dbm1(2485)
dbm2(2228)=dbm1(2486)
dbm2(2229)=dbm1(2487)
dbm2(2230)=dbm1(2488)
dbm2(2231)=dbm1(2489)
dbm2(2232)=dbm1(2490)
dbm2(2233)=dbm1(2491)
dbm2(2234)=dbm1(2492)
dbm2(2235)=dbm1(2493)
dbm2(2236)=dbm1(2494)
dbm2(2237)=dbm1(2495)
dbm2(2238)=dbm1(2496)
dbm2(2239)=dbm1(2497)
dbm2(2240)=dbm1(2498)
dbm2(2241)=dbm1(2499)
dbm2(2242)=dbm1(2500)
dbm2(2243)=dbm1(2501)
dbm2(2244)=dbm1(2502)
dbm2(2245)=dbm1(2503)
dbm2(2246)=dbm1(2504)
dbm2(2247)=dbm1(2505)
dbm2(2248)=dbm1(2506)
dbm2(2249)=dbm1(2507)
dbm2(2250)=dbm1(2508)
dbm2(2251)=dbm1(2509)
dbm2(2252)=dbm1(2510)
dbm2(2253)=dbm1(2511)
dbm2(2254)=dbm1(2512)
dbm2(2255)=dbm1(2513)
dbm2(2256)=dbm1(2514)
dbm2(2257)=dbm1(2515)
dbm2(2258)=dbm1(2516)
dbm2(2259)=dbm1(2517)
dbm2(2260)=dbm1(2518)
dbm2(2261)=dbm1(2519)
dbm2(2262)=dbm1(2520)
dbm2(2263)=dbm1(2521)
dbm2(2264)=dbm1(2522)
dbm2(2265)=dbm1(2523)
dbm2(2266)=dbm1(2524)
dbm2(2267)=dbm1(2525)
dbm2(2268)=dbm1(2526)
dbm2(2269)=dbm1(2527)
dbm2(2270)=dbm1(2528)
dbm2(2271)=dbm1(2529)
dbm2(2272)=dbm1(2530)
dbm2(2273)=dbm1(2531)
dbm2(2274)=dbm1(2532)
dbm2(2275)=dbm1(2533)
dbm2(2276)=dbm1(2534)
dbm2(2277)=dbm1(2535)
dbm2(2278)=dbm1(2536)
dbm2(2279)=dbm1(2537)
dbm2(2280)=dbm1(2538)
dbm2(2281)=dbm1(2539)
dbm2(2282)=dbm1(2540)
dbm2(2283)=dbm1(2541)
dbm2(2284)=dbm1(2542)
dbm2(2285)=dbm1(2543)
dbm2(2286)=dbm1(2544)
dbm2(2287)=dbm1(2545)
dbm2(2288)=dbm1(2546)
dbm2(2289)=dbm1(2547)
dbm2(2290)=dbm1(2548)
dbm2(2291)=dbm1(2549)
dbm2(2292)=dbm1(2550)
dbm2(2293)=dbm1(2551)
dbm2(2294)=dbm1(2552)
dbm2(2295)=dbm1(2553)
dbm2(2296)=dbm1(2554)
dbm2(2297)=dbm1(2555)
dbm2(2298)=dbm1(2556)
dbm2(2299)=dbm1(2557)
dbm2(2300)=dbm1(2558)
dbm2(2301)=dbm1(2559)
dbm2(2302)=dbm1(2560)
dbm2(2303)=dbm1(2561)
dbm2(2304)=dbm1(2562)
dbm2(2305)=dbm1(2563)
dbm2(2306)=dbm1(2564)
dbm2(2307)=dbm1(2565)
dbm2(2308)=dbm1(2566)
dbm2(2309)=dbm1(2567)
dbm2(2310)=dbm1(2568)
dbm2(2311)=dbm1(2569)
dbm2(2312)=dbm1(2570)
dbm2(2313)=dbm1(2571)
dbm2(2314)=dbm1(2572)
dbm2(2315)=dbm1(2573)
dbm2(2316)=dbm1(2574)
dbm2(2317)=dbm1(2575)
dbm2(2318)=dbm1(2576)
dbm2(2319)=dbm1(2577)
dbm2(2320)=dbm1(2578)
dbm2(2321)=dbm1(2579)
dbm2(2322)=dbm1(2580)
dbm2(2323)=dbm1(2581)
dbm2(2324)=dbm1(2582)
dbm2(2325)=dbm1(2583)
dbm2(2326)=dbm1(2584)
dbm2(2327)=dbm1(2585)
dbm2(2328)=dbm1(2586)
dbm2(2329)=dbm1(2587)
dbm2(2330)=dbm1(2588)
dbm2(2331)=dbm1(2589)
dbm2(2332)=dbm1(2590)
dbm2(2333)=dbm1(2591)
dbm2(2334)=dbm1(2592)
dbm2(2335)=dbm1(2593)
dbm2(2336)=dbm1(2594)
dbm2(2337)=dbm1(2595)
dbm2(2338)=dbm1(2596)
dbm2(2339)=dbm1(2597)
dbm2(2340)=dbm1(2598)
dbm2(2341)=dbm1(2599)
dbm2(2342)=dbm1(2600)
dbm2(2343)=dbm1(2601)
dbm2(2344)=dbm1(2602)
dbm2(2345)=dbm1(2603)
dbm2(2346)=dbm1(2604)
dbm2(2347)=dbm1(2605)
dbm2(2348)=dbm1(2606)
dbm2(2349)=dbm1(2607)
dbm2(2350)=dbm1(2608)
dbm2(2351)=dbm1(2609)
dbm2(2352)=dbm1(2610)
dbm2(2353)=dbm1(2611)
dbm2(2354)=dbm1(2612)
dbm2(2355)=dbm1(2613)
dbm2(2356)=dbm1(2614)
dbm2(2357)=dbm1(2615)
dbm2(2358)=dbm1(2616)
dbm2(2359)=dbm1(2617)
dbm2(2360)=dbm1(2618)
dbm2(2361)=dbm1(2619)
dbm2(2362)=dbm1(2620)
dbm2(2363)=dbm1(2621)
dbm2(2364)=dbm1(2622)
dbm2(2365)=dbm1(2623)
dbm2(2366)=dbm1(2624)
dbm2(2367)=dbm1(2625)
dbm2(2368)=dbm1(2626)
dbm2(2369)=dbm1(2627)
dbm2(2370)=dbm1(2628)
dbm2(2371)=dbm1(2629)
dbm2(2372)=dbm1(2630)
dbm2(2373)=dbm1(2631)
dbm2(2374)=dbm1(2632)
dbm2(2375)=dbm1(2633)
dbm2(2376)=dbm1(2634)
dbm2(2377)=dbm1(2635)
dbm2(2378)=dbm1(2636)
dbm2(2379)=dbm1(2637)
dbm2(2380)=dbm1(2638)
dbm2(2381)=dbm1(2639)
dbm2(2382)=dbm1(2640)
dbm2(2383)=dbm1(2641)
dbm2(2384)=dbm1(2642)
dbm2(2385)=dbm1(2643)
dbm2(2386)=dbm1(2644)
dbm2(2387)=dbm1(2645)
dbm2(2388)=dbm1(2646)
dbm2(2389)=dbm1(2647)
dbm2(2390)=dbm1(2648)
dbm2(2391)=dbm1(2649)
dbm2(2392)=dbm1(2650)
dbm2(2393)=dbm1(2651)
dbm2(2394)=dbm1(2652)
dbm2(2395)=dbm1(2653)
dbm2(2396)=dbm1(2654)
dbm2(2397)=dbm1(2655)
dbm2(2398)=dbm1(2656)
dbm2(2399)=dbm1(2657)
dbm2(2400)=dbm1(2658)
dbm2(2401)=dbm1(2659)
dbm2(2402)=dbm1(2660)
dbm2(2403)=dbm1(2661)
dbm2(2404)=dbm1(2662)
dbm2(2405)=dbm1(2663)
dbm2(2406)=dbm1(2664)
dbm2(2407)=dbm1(2665)
dbm2(2408)=dbm1(2666)
dbm2(2409)=dbm1(2667)
dbm2(2410)=dbm1(2668)
dbm2(2411)=dbm1(2669)
dbm2(2412)=dbm1(2670)
dbm2(2413)=dbm1(2671)
dbm2(2414)=dbm1(2672)
dbm2(2415)=dbm1(2673)
dbm2(2416)=dbm1(2674)
dbm2(2417)=dbm1(2675)
dbm2(2418)=dbm1(2676)
dbm2(2419)=dbm1(2677)
dbm2(2420)=dbm1(2678)
dbm2(2421)=dbm1(2679)
dbm2(2422)=dbm1(2680)
dbm2(2423)=dbm1(2681)
dbm2(2424)=dbm1(2682)
dbm2(2425)=dbm1(2683)
dbm2(2426)=dbm1(2684)
dbm2(2427)=dbm1(2685)
dbm2(2428)=dbm1(2686)
dbm2(2429)=dbm1(2687)
dbm2(2430)=dbm1(2688)
dbm2(2431)=dbm1(2689)
dbm2(2432)=dbm1(2690)
dbm2(2433)=dbm1(2691)
dbm2(2434)=dbm1(2692)
dbm2(2435)=dbm1(2693)
dbm2(2436)=dbm1(2694)
dbm2(2437)=dbm1(2695)
dbm2(2438)=dbm1(2696)
dbm2(2439)=dbm1(2697)
dbm2(2440)=dbm1(2698)
dbm2(2441)=dbm1(2699)
dbm2(2442)=dbm1(2700)
dbm2(2443)=dbm1(2701)
dbm2(2444)=dbm1(2702)
dbm2(2445)=dbm1(2703)
dbm2(2446)=dbm1(2704)
dbm2(2447)=dbm1(2705)
dbm2(2448)=dbm1(2706)
dbm2(2449)=dbm1(2707)
dbm2(2450)=dbm1(2708)
dbm2(2451)=dbm1(2709)
dbm2(2452)=dbm1(2710)
dbm2(2453)=dbm1(2711)
dbm2(2454)=dbm1(2712)
dbm2(2455)=dbm1(2713)
dbm2(2456)=dbm1(2714)
dbm2(2457)=dbm1(2715)
dbm2(2458)=dbm1(2716)
dbm2(2459)=dbm1(2717)
dbm2(2460)=dbm1(2718)
dbm2(2461)=dbm1(2719)
dbm2(2462)=dbm1(2720)
dbm2(2463)=dbm1(2721)
dbm2(2464)=dbm1(2722)
dbm2(2465)=dbm1(2723)
dbm2(2466)=dbm1(2724)
dbm2(2467)=dbm1(2725)
dbm2(2468)=dbm1(2726)
dbm2(2469)=dbm1(2727)
dbm2(2470)=dbm1(2728)
dbm2(2471)=dbm1(2729)
dbm2(2472)=dbm1(2730)
dbm2(2473)=dbm1(2731)
dbm2(2474)=dbm1(2732)
dbm2(2475)=dbm1(2733)
dbm2(2476)=dbm1(2734)
dbm2(2477)=dbm1(2735)
dbm2(2478)=dbm1(2736)
dbm2(2479)=dbm1(2737)
dbm2(2480)=dbm1(2738)
dbm2(2481)=dbm1(2739)
dbm2(2482)=dbm1(2740)
dbm2(2483)=dbm1(2741)
dbm2(2484)=dbm1(2742)
dbm2(2485)=dbm1(2743)
dbm2(2486)=dbm1(2744)
dbm2(2487)=dbm1(2745)
dbm2(2488)=dbm1(2746)
dbm2(2489)=dbm1(2747)
dbm2(2490)=dbm1(2748)
dbm2(2491)=dbm1(2749)
dbm2(2492)=dbm1(2750)
dbm2(2493)=dbm1(2751)
dbm2(2494)=dbm1(2752)
dbm2(2495)=dbm1(2753)
dbm2(2496)=dbm1(2754)
dbm2(2497)=dbm1(2755)
dbm2(2498)=dbm1(2756)
dbm2(2499)=dbm1(2757)
dbm2(2500)=dbm1(2758)
dbm2(2501)=dbm1(2759)
dbm2(2502)=dbm1(2760)
dbm2(2503)=dbm1(2761)
dbm2(2504)=dbm1(2762)
dbm2(2505)=dbm1(2763)
dbm2(2506)=dbm1(2764)
dbm2(2507)=dbm1(2765)
dbm2(2508)=dbm1(2766)
dbm2(2509)=dbm1(2767)
dbm2(2510)=dbm1(2768)
dbm2(2511)=dbm1(2769)
dbm2(2512)=dbm1(2770)
dbm2(2513)=dbm1(2771)
dbm2(2514)=dbm1(2772)
dbm2(2515)=dbm1(2773)
dbm2(2516)=dbm1(2774)
dbm2(2517)=dbm1(2775)
dbm2(2518)=dbm1(2776)
dbm2(2519)=dbm1(2777)
dbm2(2520)=dbm1(2778)
dbm2(2521)=dbm1(2779)
dbm2(2522)=dbm1(2780)
dbm2(2523)=dbm1(2781)
dbm2(2524)=dbm1(2782)
dbm2(2525)=dbm1(2783)
dbm2(2526)=dbm1(2784)
dbm2(2527)=dbm1(2785)
dbm2(2528)=dbm1(2786)
dbm2(2529)=dbm1(2787)
dbm2(2530)=dbm1(2788)
dbm2(2531)=dbm1(2789)
dbm2(2532)=dbm1(2790)
dbm2(2533)=dbm1(2791)
dbm2(2534)=dbm1(2792)
dbm2(2535)=dbm1(2793)
dbm2(2536)=dbm1(2794)
dbm2(2537)=dbm1(2795)
dbm2(2538)=dbm1(2796)
dbm2(2539)=dbm1(2797)
dbm2(2540)=dbm1(2798)
dbm2(2541)=dbm1(2799)
dbm2(2542)=dbm1(2800)
dbm2(2543)=dbm1(2801)
dbm2(2544)=dbm1(2802)
dbm2(2545)=dbm1(2803)
dbm2(2546)=dbm1(2804)
dbm2(2547)=dbm1(2805)
dbm2(2548)=dbm1(2806)
dbm2(2549)=dbm1(2807)
dbm2(2550)=dbm1(2808)
dbm2(2551)=dbm1(2809)
dbm2(2552)=dbm1(2810)
dbm2(2553)=dbm1(2811)
dbm2(2554)=dbm1(2812)
dbm2(2555)=dbm1(2813)
dbm2(2556)=dbm1(2814)
dbm2(2557)=dbm1(2815)
dbm2(2558)=dbm1(2816)
dbm2(2559)=dbm1(2817)
dbm2(2560)=dbm1(2818)
dbm2(2561)=dbm1(2819)
dbm2(2562)=dbm1(2820)
dbm2(2563)=dbm1(2821)
dbm2(2564)=dbm1(2822)
dbm2(2565)=dbm1(2823)
dbm2(2566)=dbm1(2824)
dbm2(2567)=dbm1(2825)
dbm2(2568)=dbm1(2826)
dbm2(2569)=dbm1(2827)
dbm2(2570)=dbm1(2828)
dbm2(2571)=dbm1(2829)
dbm2(2572)=dbm1(2830)
dbm2(2573)=dbm1(2831)
dbm2(2574)=dbm1(2832)
dbm2(2575)=dbm1(2833)
dbm2(2576)=dbm1(2834)
dbm2(2577)=dbm1(2835)
dbm2(2578)=dbm1(2836)
dbm2(2579)=dbm1(2837)
dbm2(2580)=dbm1(2838)
dbm2(2581)=dbm1(2839)
dbm2(2582)=dbm1(2840)
dbm2(2583)=dbm1(2841)
dbm2(2584)=dbm1(2842)
dbm2(2585)=dbm1(2843)
dbm2(2586)=dbm1(2844)
dbm2(2587)=dbm1(2845)
dbm2(2588)=dbm1(2846)
dbm2(2589)=dbm1(2847)
dbm2(2590)=dbm1(2848)
dbm2(2591)=dbm1(2849)
dbm2(2592)=dbm1(2850)
dbm2(2593)=dbm1(2851)
dbm2(2594)=dbm1(2852)
dbm2(2595)=dbm1(2853)
dbm2(2596)=dbm1(2854)
dbm2(2597)=dbm1(2855)
dbm2(2598)=dbm1(2856)
dbm2(2599)=dbm1(2857)
dbm2(2600)=dbm1(2858)
dbm2(2601)=dbm1(2859)
dbm2(2602)=dbm1(2860)
dbm2(2603)=dbm1(2861)
dbm2(2604)=dbm1(2862)
dbm2(2605)=dbm1(2863)
dbm2(2606)=dbm1(2864)
dbm2(2607)=dbm1(2865)
dbm2(2608)=dbm1(2866)
dbm2(2609)=dbm1(2867)
dbm2(2610)=dbm1(2868)
dbm2(2611)=dbm1(2869)
dbm2(2612)=dbm1(2870)
dbm2(2613)=dbm1(2871)
dbm2(2614)=dbm1(2872)
dbm2(2615)=dbm1(2873)
dbm2(2616)=dbm1(2874)
dbm2(2617)=dbm1(2875)
dbm2(2618)=dbm1(2876)
dbm2(2619)=dbm1(2877)
dbm2(2620)=dbm1(2878)
dbm2(2621)=dbm1(2879)
dbm2(2622)=dbm1(2880)
dbm2(2623)=dbm1(2881)
dbm2(2624)=dbm1(2882)
dbm2(2625)=dbm1(2883)
dbm2(2626)=dbm1(2884)
dbm2(2627)=dbm1(2885)
dbm2(2628)=dbm1(2886)
dbm2(2629)=dbm1(2887)
dbm2(2630)=dbm1(2888)
dbm2(2631)=dbm1(2889)
dbm2(2632)=dbm1(2890)
dbm2(2633)=dbm1(2891)
dbm2(2634)=dbm1(2892)
dbm2(2635)=dbm1(2893)
dbm2(2636)=dbm1(2894)
dbm2(2637)=dbm1(2895)
dbm2(2638)=dbm1(2896)
dbm2(2639)=dbm1(2897)
dbm2(2640)=dbm1(2898)
dbm2(2641)=dbm1(2899)
dbm2(2642)=dbm1(2900)
dbm2(2643)=dbm1(2901)
dbm2(2644)=dbm1(2902)
dbm2(2645)=dbm1(2903)
dbm2(2646)=dbm1(2904)
dbm2(2647)=dbm1(2905)
dbm2(2648)=dbm1(2906)
dbm2(2649)=dbm1(2907)
dbm2(2650)=dbm1(2908)
dbm2(2651)=dbm1(2909)
dbm2(2652)=dbm1(2910)
dbm2(2653)=dbm1(2911)
dbm2(2654)=dbm1(2912)
dbm2(2655)=dbm1(2913)
dbm2(2656)=dbm1(2914)
dbm2(2657)=dbm1(2915)
dbm2(2658)=dbm1(2916)
dbm2(2659)=dbm1(2917)
dbm2(2660)=dbm1(2918)
dbm2(2661)=dbm1(2919)
dbm2(2662)=dbm1(2920)
dbm2(2663)=dbm1(2921)
dbm2(2664)=dbm1(2922)
dbm2(2665)=dbm1(2923)
dbm2(2666)=dbm1(2924)
dbm2(2667)=dbm1(2925)
dbm2(2668)=dbm1(2926)
dbm2(2669)=dbm1(2927)
dbm2(2670)=dbm1(2928)
dbm2(2671)=dbm1(2929)
dbm2(2672)=dbm1(2930)
dbm2(2673)=dbm1(2931)
dbm2(2674)=dbm1(2932)
dbm2(2675)=dbm1(2933)
dbm2(2676)=dbm1(2934)
dbm2(2677)=dbm1(2935)
dbm2(2678)=dbm1(2936)
dbm2(2679)=dbm1(2937)
dbm2(2680)=dbm1(2938)
dbm2(2681)=dbm1(2939)
dbm2(2682)=dbm1(2940)
dbm2(2683)=dbm1(2941)
dbm2(2684)=dbm1(2942)
dbm2(2685)=dbm1(2943)
dbm2(2686)=dbm1(2944)
dbm2(2687)=dbm1(2945)
dbm2(2688)=dbm1(2946)
dbm2(2689)=dbm1(2947)
dbm2(2690)=dbm1(2948)
dbm2(2691)=dbm1(2949)
dbm2(2692)=dbm1(2950)
dbm2(2693)=dbm1(2951)
dbm2(2694)=dbm1(2952)
dbm2(2695)=dbm1(2953)
dbm2(2696)=dbm1(2954)
dbm2(2697)=dbm1(2955)
dbm2(2698)=dbm1(2956)
dbm2(2699)=dbm1(2957)
dbm2(2700)=dbm1(2958)
dbm2(2701)=dbm1(2959)
dbm2(2702)=dbm1(2960)
dbm2(2703)=dbm1(2961)
dbm2(2704)=dbm1(2962)
dbm2(2705)=dbm1(2963)
dbm2(2706)=dbm1(2964)
dbm2(2707)=dbm1(2965)
dbm2(2708)=dbm1(2966)
dbm2(2709)=dbm1(2967)
dbm2(2710)=dbm1(2968)
dbm2(2711)=dbm1(2969)
dbm2(2712)=dbm1(2970)
dbm2(2713)=dbm1(2971)
dbm2(2714)=dbm1(2972)
dbm2(2715)=dbm1(2973)
dbm2(2716)=dbm1(2974)
dbm2(2717)=dbm1(2975)
dbm2(2718)=dbm1(2976)
dbm2(2719)=dbm1(2977)
dbm2(2720)=dbm1(2978)
dbm2(2721)=dbm1(2979)
dbm2(2722)=dbm1(2980)
dbm2(2723)=dbm1(2981)
dbm2(2724)=dbm1(2982)
dbm2(2725)=dbm1(2983)
dbm2(2726)=dbm1(2984)
dbm2(2727)=dbm1(2985)
dbm2(2728)=dbm1(2986)
dbm2(2729)=dbm1(2987)
dbm2(2730)=dbm1(2988)
dbm2(2731)=dbm1(2989)
dbm2(2732)=dbm1(2990)
dbm2(2733)=dbm1(2991)
dbm2(2734)=dbm1(2992)
dbm2(2735)=dbm1(2993)
dbm2(2736)=dbm1(2994)
dbm2(2737)=dbm1(2995)
dbm2(2738)=dbm1(2996)
dbm2(2739)=dbm1(2997)
dbm2(2740)=dbm1(2998)
dbm2(2741)=dbm1(2999)
dbm2(2742)=dbm1(3000)
dbm2(2743)=dbm1(3001)
dbm2(2744)=dbm1(3002)
dbm2(2745)=dbm1(3003)
dbm2(2746)=dbm1(3004)
dbm2(2747)=dbm1(3005)
dbm2(2748)=dbm1(3006)
dbm2(2749)=dbm1(3007)
dbm2(2750)=dbm1(3008)
dbm2(2751)=dbm1(3009)
dbm2(2752)=dbm1(3010)
dbm2(2753)=dbm1(3011)
dbm2(2754)=dbm1(3012)
dbm2(2755)=dbm1(3013)
dbm2(2756)=dbm1(3014)
dbm2(2757)=dbm1(3015)
dbm2(2758)=dbm1(3016)
dbm2(2759)=dbm1(3017)
dbm2(2760)=dbm1(3018)
dbm2(2761)=dbm1(3019)
dbm2(2762)=dbm1(3020)
dbm2(2763)=dbm1(3021)
dbm2(2764)=dbm1(3022)
dbm2(2765)=dbm1(3023)
dbm2(2766)=dbm1(3024)
dbm2(2767)=dbm1(3025)
dbm2(2768)=dbm1(3026)
dbm2(2769)=dbm1(3027)
dbm2(2770)=dbm1(3028)
dbm2(2771)=dbm1(3029)
dbm2(2772)=dbm1(3030)
dbm2(2773)=dbm1(3031)
dbm2(2774)=dbm1(3032)
dbm2(2775)=dbm1(3033)
dbm2(2776)=dbm1(3034)
dbm2(2777)=dbm1(3035)
dbm2(2778)=dbm1(3036)
dbm2(2779)=dbm1(3037)
dbm2(2780)=dbm1(3038)
dbm2(2781)=dbm1(3039)
dbm2(2782)=dbm1(3040)
dbm2(2783)=dbm1(3041)
dbm2(2784)=dbm1(3042)
dbm2(2785)=dbm1(3043)
dbm2(2786)=dbm1(3044)
dbm2(2787)=dbm1(3045)
dbm2(2788)=dbm1(3046)
dbm2(2789)=dbm1(3047)
dbm2(2790)=dbm1(3048)
dbm2(2791)=dbm1(3049)
dbm2(2792)=dbm1(3050)
dbm2(2793)=dbm1(3051)
dbm2(2794)=dbm1(3052)
dbm2(2795)=dbm1(3053)
dbm2(2796)=dbm1(3054)
dbm2(2797)=dbm1(3055)
dbm2(2798)=dbm1(3056)
dbm2(2799)=dbm1(3057)
dbm2(2800)=dbm1(3058)
dbm2(2801)=dbm1(3059)
dbm2(2802)=dbm1(3060)
dbm2(2803)=dbm1(3061)
dbm2(2804)=dbm1(3062)
dbm2(2805)=dbm1(3063)
dbm2(2806)=dbm1(3064)
dbm2(2807)=dbm1(3065)
dbm2(2808)=dbm1(3066)
dbm2(2809)=dbm1(3067)
dbm2(2810)=dbm1(3068)
dbm2(2811)=dbm1(3069)
dbm2(2812)=dbm1(3070)
dbm2(2813)=dbm1(3071)
dbm2(2814)=dbm1(3072)
dbm2(2815)=dbm1(3073)
dbm2(2816)=dbm1(3074)
dbm2(2817)=dbm1(3075)
dbm2(2818)=dbm1(3076)
dbm2(2819)=dbm1(3077)
dbm2(2820)=dbm1(3078)
dbm2(2821)=dbm1(3079)
dbm2(2822)=dbm1(3080)
dbm2(2823)=dbm1(3081)
dbm2(2824)=dbm1(3082)
dbm2(2825)=dbm1(3083)
dbm2(2826)=dbm1(3084)
dbm2(2827)=dbm1(3085)
dbm2(2828)=dbm1(3086)
dbm2(2829)=dbm1(3087)
dbm2(2830)=dbm1(3088)
dbm2(2831)=dbm1(3089)
dbm2(2832)=dbm1(3090)
dbm2(2833)=dbm1(3091)
dbm2(2834)=dbm1(3092)
dbm2(2835)=dbm1(3093)
dbm2(2836)=dbm1(3094)
dbm2(2837)=dbm1(3095)
dbm2(2838)=dbm1(3096)
dbm2(2839)=dbm1(3097)
dbm2(2840)=dbm1(3098)
dbm2(2841)=dbm1(3099)
dbm2(2842)=dbm1(3100)
dbm2(2843)=dbm1(3101)
dbm2(2844)=dbm1(3102)
dbm2(2845)=dbm1(3103)
dbm2(2846)=dbm1(3104)
dbm2(2847)=dbm1(3105)
dbm2(2848)=dbm1(3106)
dbm2(2849)=dbm1(3107)
dbm2(2850)=dbm1(3108)
dbm2(2851)=dbm1(3109)
dbm2(2852)=dbm1(3110)
dbm2(2853)=dbm1(3111)
dbm2(2854)=dbm1(3112)
dbm2(2855)=dbm1(3113)
dbm2(2856)=dbm1(3114)
dbm2(2857)=dbm1(3115)
dbm2(2858)=dbm1(3116)
dbm2(2859)=dbm1(3117)
dbm2(2860)=dbm1(3118)
dbm2(2861)=dbm1(3119)
dbm2(2862)=dbm1(3120)
dbm2(2863)=dbm1(3121)
dbm2(2864)=dbm1(3122)
dbm2(2865)=dbm1(3123)
dbm2(2866)=dbm1(3124)
dbm2(2867)=dbm1(3125)
dbm2(2868)=dbm1(3126)
dbm2(2869)=dbm1(3127)
dbm2(2870)=dbm1(3128)
dbm2(2871)=dbm1(3129)
dbm2(2872)=dbm1(3130)
dbm2(2873)=dbm1(3131)
dbm2(2874)=dbm1(3132)
dbm2(2875)=dbm1(3133)
dbm2(2876)=dbm1(3134)
dbm2(2877)=dbm1(3135)
dbm2(2878)=dbm1(3136)
dbm2(2879)=dbm1(3137)
dbm2(2880)=dbm1(3138)
dbm2(2881)=dbm1(3139)
dbm2(2882)=dbm1(3140)
dbm2(2883)=dbm1(3141)
dbm2(2884)=dbm1(3142)
dbm2(2885)=dbm1(3143)
dbm2(2886)=dbm1(3144)
dbm2(2887)=dbm1(3145)
dbm2(2888)=dbm1(3146)
dbm2(2889)=dbm1(3147)
dbm2(2890)=dbm1(3148)
dbm2(2891)=dbm1(3149)
dbm2(2892)=dbm1(3150)
dbm2(2893)=dbm1(3151)
dbm2(2894)=dbm1(3152)
dbm2(2895)=dbm1(3153)
dbm2(2896)=dbm1(3154)
dbm2(2897)=dbm1(3155)
dbm2(2898)=dbm1(3156)
dbm2(2899)=dbm1(3157)
dbm2(2900)=dbm1(3158)
dbm2(2901)=dbm1(3159)
dbm2(2902)=dbm1(3160)
dbm2(2903)=dbm1(3161)
dbm2(2904)=dbm1(3162)
dbm2(2905)=dbm1(3163)
dbm2(2906)=dbm1(3164)
dbm2(2907)=dbm1(3165)
dbm2(2908)=dbm1(3166)
dbm2(2909)=dbm1(3167)
dbm2(2910)=dbm1(3168)
dbm2(2911)=dbm1(3169)
dbm2(2912)=dbm1(3170)
dbm2(2913)=dbm1(3171)
dbm2(2914)=dbm1(3172)
dbm2(2915)=dbm1(3173)
dbm2(2916)=dbm1(3174)
dbm2(2917)=dbm1(3175)
dbm2(2918)=dbm1(3176)
dbm2(2919)=dbm1(3177)
dbm2(2920)=dbm1(3178)
dbm2(2921)=dbm1(3179)
dbm2(2922)=dbm1(3180)
dbm2(2923)=dbm1(3181)
dbm2(2924)=dbm1(3182)
dbm2(2925)=dbm1(3183)
dbm2(2926)=dbm1(3184)
dbm2(2927)=dbm1(3185)
dbm2(2928)=dbm1(3186)
dbm2(2929)=dbm1(3187)
dbm2(2930)=dbm1(3188)
dbm2(2931)=dbm1(3189)
dbm2(2932)=dbm1(3190)
dbm2(2933)=dbm1(3191)
dbm2(2934)=dbm1(3193)
dbm2(2935)=dbm1(3194)
dbm2(2936)=dbm1(3195)
dbm2(2937)=dbm1(3196)
dbm2(2938)=dbm1(3197)
dbm2(2939)=dbm1(3198)
dbm2(2940)=dbm1(3199)
dbm2(2941)=dbm1(3200)
dbm2(2942)=dbm1(3201)
dbm2(2943)=dbm1(3202)
dbm2(2944)=dbm1(3203)
dbm2(2945)=dbm1(3204)
dbm2(2946)=dbm1(3205)
dbm2(2947)=dbm1(3206)
dbm2(2948)=dbm1(3207)
dbm2(2949)=dbm1(3208)
dbm2(2950)=dbm1(3210)
dbm2(2951)=dbm1(3212)
dbm2(2952)=dbm1(3213)
dbm2(2953)=dbm1(3214)
dbm2(2954)=dbm1(3215)
dbm2(2955)=dbm1(3216)
dbm2(2956)=dbm1(3217)
dbm2(2957)=dbm1(3218)
dbm2(2958)=dbm1(3219)
dbm2(2959)=dbm1(3220)
dbm2(2960)=dbm1(3221)
dbm2(2961)=dbm1(3222)
dbm2(2962)=dbm1(3223)
dbm2(2963)=dbm1(3224)
dbm2(2964)=dbm1(3225)
dbm2(2965)=dbm1(3227)
dbm2(2966)=dbm1(3228)
dbm2(2967)=dbm1(3230)
dbm2(2968)=dbm1(3232)
dbm2(2969)=dbm1(3233)
dbm2(2970)=dbm1(3234)
dbm2(2971)=dbm1(3235)
dbm2(2972)=dbm1(3236)
dbm2(2973)=dbm1(3237)
dbm2(2974)=dbm1(3238)
dbm2(2975)=dbm1(3239)
dbm2(2976)=dbm1(3240)
dbm2(2977)=dbm1(3241)
dbm2(2978)=dbm1(3242)
dbm2(2979)=dbm1(3243)
dbm2(2980)=dbm1(3244)
dbm2(2981)=dbm1(3246)
dbm2(2982)=dbm1(3247)
dbm2(2983)=dbm1(3249)
dbm2(2984)=dbm1(3250)
dbm2(2985)=dbm1(3252)
dbm2(2986)=dbm1(3254)
dbm2(2987)=dbm1(3255)
dbm2(2988)=dbm1(3256)
dbm2(2989)=dbm1(3257)
dbm2(2990)=dbm1(3258)
dbm2(2991)=dbm1(3259)
dbm2(2992)=dbm1(3260)
dbm2(2993)=dbm1(3261)
dbm2(2994)=dbm1(3262)
dbm2(2995)=dbm1(3263)
dbm2(2996)=dbm1(3264)
dbm2(2997)=dbm1(3265)
dbm2(2998)=dbm1(3267)
dbm2(2999)=dbm1(3268)
dbm2(3000)=dbm1(3270)
dbm2(3001)=dbm1(3271)
dbm2(3002)=dbm1(3273)
dbm2(3003)=dbm1(3274)
dbm2(3004)=dbm1(3276)
dbm2(3005)=dbm1(3277)
dbm2(3006)=dbm1(3279)
dbm2(3007)=dbm1(3280)
dbm2(3008)=dbm1(3281)
dbm2(3009)=dbm1(3282)
dbm2(3010)=dbm1(3283)
dbm2(3011)=dbm1(3284)
dbm2(3012)=dbm1(3285)
dbm2(3013)=dbm1(3286)
dbm2(3014)=dbm1(3287)
dbm2(3015)=dbm1(3288)
dbm2(3016)=dbm1(3289)
dbm2(3017)=dbm1(3291)
dbm2(3018)=dbm1(3292)
dbm2(3019)=dbm1(3294)
dbm2(3020)=dbm1(3295)
dbm2(3021)=dbm1(3297)
dbm2(3022)=dbm1(3298)
dbm2(3023)=dbm1(3299)
dbm2(3024)=dbm1(3301)
dbm2(3025)=dbm1(3302)
dbm2(3026)=dbm1(3303)
dbm2(3027)=dbm1(3305)
dbm2(3028)=dbm1(3306)
dbm2(3029)=dbm1(3307)
dbm2(3030)=dbm1(3308)
dbm2(3031)=dbm1(3309)
dbm2(3032)=dbm1(3310)
dbm2(3033)=dbm1(3311)
dbm2(3034)=dbm1(3312)
dbm2(3035)=dbm1(3313)
dbm2(3036)=dbm1(3315)
dbm2(3037)=dbm1(3316)
dbm2(3038)=dbm1(3318)
dbm2(3039)=dbm1(3319)
dbm2(3040)=dbm1(3321)
dbm2(3041)=dbm1(3322)
dbm2(3042)=dbm1(3323)
dbm2(3043)=dbm1(3325)
dbm2(3044)=dbm1(3326)
dbm2(3045)=dbm1(3327)
dbm2(3046)=dbm1(3328)
dbm2(3047)=dbm1(3329)
dbm2(3048)=dbm1(3330)
dbm2(3049)=dbm1(3331)
dbm2(3050)=dbm1(3332)
dbm2(3051)=dbm1(3334)
dbm2(3052)=dbm1(3335)
dbm2(3053)=dbm1(3337)
dbm2(3054)=dbm1(3338)
dbm2(3055)=dbm1(3339)
dbm2(3056)=dbm1(3341)
dbm2(3057)=dbm1(3342)
dbm2(3058)=dbm1(3343)
dbm2(3059)=dbm1(3344)
dbm2(3060)=dbm1(3345)
dbm2(3061)=dbm1(3346)
dbm2(3062)=dbm1(3348)
dbm2(3063)=dbm1(3349)
dbm2(3064)=dbm1(3351)
dbm2(3065)=dbm1(3352)
dbm2(3066)=dbm1(3353)
dbm2(3067)=dbm1(3354)
dbm2(3068)=dbm1(3355)
dbm2(3069)=dbm1(3357)
dbm2(3070)=dbm1(3358)
dbm2(3071)=dbm1(3359)
dbm2(3072)=dbm1(3360)
dbm2(3073)=dbm1(3361)
dbm2(3074)=dbm1(3362)
dbm2(3075)=dbm1(3363)

dVdR(i)=dVdR(i) + dot_product(coeff,dbm2)

end do

end subroutine EvU13O10dVdR

!===========================================================================
! *def u23_dvdx(xyz,u23,s23,du23dx)
!  main subtoutine of evaluating derivative of u23 w.r.t. the xyz coordinates
!  xyz is the coordinate
!  u23 is u23
!  s23 is u23 devided by the hyperbolic function
!  du23dx is the gradient of u23
!===========================================================================
subroutine u23_dvdx(xyz,u23,s23,du23dx)
implicit none
integer :: i,j,k
real*8 :: du23dx(4,3),xyz(4,3)
real*8 :: u23,s23
real*8 :: coord(12), r(6)
real*8 :: dudx(12)
real*8 :: dvdx(12), dvdr(6), drdx(6,12)
real*8 :: dtdx(12)
real*8 :: volume

! Initialize dVdX(12)
      do i=1,12
        dVdX(i)=0.0d0
      enddo

! Initialize dRdX(6,12)
      do i=1,6
        do j=1,12
          dRdX(i,j)=0.0d0
        enddo
      enddo

! Initialize dVdR(6)
      do i=1,6
        dVdR(i)=0.0d0
      enddo

! Assign xyz to coord
      do i=1,4
      do j=1,3
        k=3*(i-1)+j
        coord(k)=xyz(i,j)
      enddo
      enddo

! Compute distances r
r(1)=sqrt((xyz(1,1)-xyz(2,1))**2+(xyz(1,2)-xyz(2,2))**2+(xyz(1,3)-xyz(2,3))**2)
r(2)=sqrt((xyz(1,1)-xyz(3,1))**2+(xyz(1,2)-xyz(3,2))**2+(xyz(1,3)-xyz(3,3))**2)
r(3)=sqrt((xyz(1,1)-xyz(4,1))**2+(xyz(1,2)-xyz(4,2))**2+(xyz(1,3)-xyz(4,3))**2)
r(4)=sqrt((xyz(2,1)-xyz(3,1))**2+(xyz(2,2)-xyz(3,2))**2+(xyz(2,3)-xyz(3,3))**2)
r(5)=sqrt((xyz(2,1)-xyz(4,1))**2+(xyz(2,2)-xyz(4,2))**2+(xyz(2,3)-xyz(4,3))**2)
r(6)=sqrt((xyz(3,1)-xyz(4,1))**2+(xyz(3,2)-xyz(4,2))**2+(xyz(3,3)-xyz(4,3))**2)

! Call EvdVdR to evaluate dVdR(6)
      Call EvU23O10dVdR(coord,r,dvdr)

! Call EvdRdX to evaluate dRdX(6,12)
      Call EvdRdX(coord,r,drdx)

! Calculate dVdX by using chain rule: dV/dXi=(dV/dRj)*(dRj/dXi), j=1 to 6
      do i=1,12
        do j=1,6
          dVdX(i)=dVdX(i) + dVdR(j)*dRdX(j,i)
        enddo
      enddo

      do i=1,12
          dVdX(i)=dVdX(i)/627.509474
      enddo

  call P_volume(xyz,volume)

! Now compute numerical dtdx
  call Num_dtdx(xyz,dtdx)

! Chain rule again, because U=tanh*V
! dUdX=dtanhdX*V+tanh*dVdX
do i=1,12
  dudx(i)=0
enddo

do i=1,12
  dudx(i)=dtdx(i)*s23+volume*dvdx(i)
enddo


! Assign dvdx back to du23dvdx
      do i=1,4
      do j=1,3
        k=3*(i-1)+j
        du23dx(i,j)=dudx(k)
      enddo
      enddo

      return

end subroutine u23_dvdx 

!===========================================================================
! *def EvU23O10dVdR(coord,r,dvdr)
!  subtoutine of evaluating derivative of u23 w.r.t. the 6 nuclear distances
!  coord is the xyz coordinate 
!  r is the nuclear distances
!  dvdr is the derivative of u23 w.r.t. the 6 nuclear distances
!===========================================================================
subroutine EvU23O10dVdR(coord,r,dvdr)
integer, parameter:: ddp=kind(0.d0)
integer :: i,j
real(ddp) :: coord(12),r(6),dvdr(6)
real(ddp) :: a1, a2, b1, b2, ra1, ra2, rb1, rb2
integer, parameter :: ncoeff=1300
real(ddp) :: coeff(ncoeff)
real(ddp),dimension(0:32) :: rm
real(ddp),dimension(0:32) :: dm
real(ddp),dimension(0:1506) :: p
real(ddp),dimension(0:1506) :: dp
real(ddp) :: x(6),dmsdr(6,6)
real(ddp) :: dbm1(1506), dbm2(1300)

! Assign coefficients first
a1=1.323343
a2=1.09
b1=1.823343
b2=1.633190
ra1=1.113545
ra2=1.001178
rb1=1.113545
rb2=1.001178
coeff(1)=0.107179913046D+04
coeff(2)=-0.358381394678D+03
coeff(3)=-0.218247544111D+04
coeff(4)=0.955807103042D+03
coeff(5)=-0.508439166000D+03
coeff(6)=-0.968469572647D+03
coeff(7)=-0.288851216283D+04
coeff(8)=-0.568712307727D+04
coeff(9)=0.259531054695D+04
coeff(10)=0.185314071824D+04
coeff(11)=0.221425999196D+04
coeff(12)=0.575360337969D+03
coeff(13)=0.191617586764D+03
coeff(14)=0.566629208364D+04
coeff(15)=-0.175167692908D+04
coeff(16)=0.591768595667D+03
coeff(17)=0.542427270915D+04
coeff(18)=-0.453495255837D+04
coeff(19)=0.262584446168D+04
coeff(20)=-0.296235430960D+04
coeff(21)=0.322544769858D+04
coeff(22)=0.135538130367D+05
coeff(23)=0.190272254228D+04
coeff(24)=0.591333638396D+01
coeff(25)=0.187067251979D+05
coeff(26)=0.704454795792D+04
coeff(27)=0.281445932548D+04
coeff(28)=-0.122297302116D+05
coeff(29)=-0.471258572916D+04
coeff(30)=-0.387352606571D+04
coeff(31)=0.498178750986D+04
coeff(32)=0.123145125779D+05
coeff(33)=-0.487653951946D+04
coeff(34)=0.641163309295D+04
coeff(35)=-0.277152944564D+04
coeff(36)=0.606625793687D+04
coeff(37)=-0.104017508429D+05
coeff(38)=0.194007516868D+04
coeff(39)=-0.435895544772D+04
coeff(40)=0.305466304471D+04
coeff(41)=0.229424471780D+04
coeff(42)=0.756575538609D+04
coeff(43)=0.342123627195D+04
coeff(44)=-0.970984554131D+03
coeff(45)=0.133281155886D+04
coeff(46)=0.113351740537D+04
coeff(47)=-0.155109686737D+05
coeff(48)=0.296008543905D+04
coeff(49)=-0.672319285306D+04
coeff(50)=0.506733748127D+04
coeff(51)=-0.560600912835D+04
coeff(52)=-0.145288465067D+04
coeff(53)=-0.795977940465D+04
coeff(54)=0.676493586144D+04
coeff(55)=-0.247825731303D+04
coeff(56)=0.111492773729D+05
coeff(57)=-0.789410862639D+04
coeff(58)=-0.152268010273D+05
coeff(59)=0.116342700823D+04
coeff(60)=-0.290848498157D+04
coeff(61)=-0.570755424647D+04
coeff(62)=0.106683937552D+05
coeff(63)=-0.159383721698D+05
coeff(64)=-0.781109257532D+03
coeff(65)=-0.112275614291D+05
coeff(66)=0.134754607434D+05
coeff(67)=0.460868717724D+04
coeff(68)=-0.704601964614D+04
coeff(69)=-0.237212369723D+04
coeff(70)=-0.235629142436D+04
coeff(71)=0.113109687523D+05
coeff(72)=0.107467703297D+05
coeff(73)=0.157964649793D+04
coeff(74)=-0.225108998654D+04
coeff(75)=0.492267589146D+04
coeff(76)=0.750547752165D+03
coeff(77)=-0.131885331185D+04
coeff(78)=0.104947201231D+03
coeff(79)=0.279729294979D+04
coeff(80)=0.976600682328D+03
coeff(81)=0.108627295854D+04
coeff(82)=0.140998213290D+04
coeff(83)=0.121362922250D+05
coeff(84)=-0.723191182428D+04
coeff(85)=-0.748846609787D+04
coeff(86)=-0.172335991356D+04
coeff(87)=0.568678575945D+03
coeff(88)=-0.223648941978D+04
coeff(89)=0.940035911871D+04
coeff(90)=0.115116201428D+05
coeff(91)=-0.762971310567D+04
coeff(92)=0.117963494616D+05
coeff(93)=-0.122355449967D+05
coeff(94)=0.130167438293D+04
coeff(95)=-0.192258239691D+05
coeff(96)=0.101515885013D+03
coeff(97)=-0.163655308915D+04
coeff(98)=0.129280765285D+04
coeff(99)=-0.303568056657D+04
coeff(100)=-0.131279033153D+04
coeff(101)=-0.603600325843D+04
coeff(102)=-0.543747625987D+04
coeff(103)=0.283631103070D+03
coeff(104)=-0.101388864769D+05
coeff(105)=-0.417924756109D+04
coeff(106)=0.155406504315D+04
coeff(107)=-0.290160705399D+04
coeff(108)=0.600214668702D+03
coeff(109)=-0.563116530378D+04
coeff(110)=0.113091614981D+05
coeff(111)=0.544953125439D+04
coeff(112)=0.215041573135D+04
coeff(113)=-0.372956941798D+04
coeff(114)=0.142078597799D+05
coeff(115)=-0.192148245585D+05
coeff(116)=-0.602620244366D+04
coeff(117)=0.117436019813D+04
coeff(118)=-0.593319812045D+04
coeff(119)=-0.392472837883D+04
coeff(120)=-0.145276961143D+05
coeff(121)=0.708822881915D+04
coeff(122)=-0.498128675375D+04
coeff(123)=0.640600518627D+04
coeff(124)=0.800728392173D+04
coeff(125)=-0.754861503063D+03
coeff(126)=0.648555488395D+04
coeff(127)=-0.401130003808D+04
coeff(128)=0.408106873243D+04
coeff(129)=0.997375959478D+04
coeff(130)=-0.316522688844D+03
coeff(131)=-0.136686127311D+03
coeff(132)=0.136913450920D+04
coeff(133)=0.213627277497D+04
coeff(134)=-0.712790318790D+03
coeff(135)=0.915142005542D+04
coeff(136)=-0.334814614865D+03
coeff(137)=0.396440313677D+04
coeff(138)=0.138785923993D+04
coeff(139)=0.177351875967D+04
coeff(140)=0.106809533872D+05
coeff(141)=-0.184992157116D+05
coeff(142)=-0.142044657005D+04
coeff(143)=0.287217373545D+03
coeff(144)=-0.158257107202D+04
coeff(145)=-0.487997312999D+04
coeff(146)=-0.112402573430D+05
coeff(147)=-0.137385357284D+05
coeff(148)=0.779952831407D+04
coeff(149)=-0.445040825534D+04
coeff(150)=0.352220810471D+04
coeff(151)=-0.329832041194D+04
coeff(152)=0.494701432690D+04
coeff(153)=0.240257723451D+04
coeff(154)=0.235849035341D+03
coeff(155)=-0.572283449018D+04
coeff(156)=-0.233543419727D+03
coeff(157)=0.343458778379D+04
coeff(158)=-0.874341434517D+04
coeff(159)=0.745140156240D+04
coeff(160)=-0.300935756377D+04
coeff(161)=0.179554887776D+04
coeff(162)=-0.554875350050D+04
coeff(163)=-0.612211596913D+04
coeff(164)=0.454633024096D+04
coeff(165)=-0.172649317364D+05
coeff(166)=0.878495401727D+04
coeff(167)=-0.594936769142D+04
coeff(168)=0.625277339511D+04
coeff(169)=-0.167459375870D+05
coeff(170)=0.112293368433D+05
coeff(171)=0.436159353896D+04
coeff(172)=0.134242078129D+05
coeff(173)=0.584249205112D+04
coeff(174)=0.364020073040D+04
coeff(175)=0.406954791895D+04
coeff(176)=0.134149198629D+05
coeff(177)=0.108062564983D+04
coeff(178)=-0.116821889274D+05
coeff(179)=0.196497484204D+04
coeff(180)=0.961052232504D+04
coeff(181)=-0.294868670003D+04
coeff(182)=0.410162045226D+04
coeff(183)=0.620739519615D+03
coeff(184)=0.110771381290D+05
coeff(185)=-0.105061106389D+04
coeff(186)=0.146620452548D+04
coeff(187)=0.788405122802D+03
coeff(188)=0.819343705806D+04
coeff(189)=-0.247660964333D+03
coeff(190)=0.145296904295D+04
coeff(191)=0.276908366196D+04
coeff(192)=-0.981461414943D+04
coeff(193)=-0.322058896433D+04
coeff(194)=0.617974882230D+04
coeff(195)=0.235556396477D+04
coeff(196)=-0.543558233912D+04
coeff(197)=0.183982822758D+04
coeff(198)=-0.133443178882D+04
coeff(199)=-0.117781371193D+04
coeff(200)=-0.137314799749D+04
coeff(201)=-0.599792274238D+02
coeff(202)=-0.246332708392D+04
coeff(203)=-0.154852388034D+03
coeff(204)=0.906003990242D+03
coeff(205)=0.196763106922D+04
coeff(206)=0.689796458957D+04
coeff(207)=0.300044462455D+04
coeff(208)=0.240312927665D+04
coeff(209)=0.131080333276D+04
coeff(210)=0.373534845487D+04
coeff(211)=0.383506884695D+04
coeff(212)=-0.123166907560D+05
coeff(213)=0.333475294037D+04
coeff(214)=0.591801896570D+04
coeff(215)=0.278355543371D+04
coeff(216)=0.191749124211D+04
coeff(217)=-0.527082055515D+04
coeff(218)=0.643745356136D+04
coeff(219)=0.658756494384D+04
coeff(220)=-0.531749994032D+04
coeff(221)=0.128700335116D+05
coeff(222)=-0.423188827127D+04
coeff(223)=-0.345574711761D+04
coeff(224)=0.655967268404D+03
coeff(225)=-0.730904915769D+04
coeff(226)=-0.837092453546D+03
coeff(227)=0.422308802256D+04
coeff(228)=-0.167114972189D+05
coeff(229)=0.217363898504D+04
coeff(230)=-0.233637554378D+04
coeff(231)=-0.157933820697D+04
coeff(232)=-0.530647133211D+04
coeff(233)=0.126548190980D+04
coeff(234)=-0.118723028688D+04
coeff(235)=0.583651947232D+04
coeff(236)=0.610276591916D+04
coeff(237)=-0.869268780204D+04
coeff(238)=-0.470321364548D+04
coeff(239)=-0.690485478950D+04
coeff(240)=-0.804351213444D+04
coeff(241)=0.369349140293D+04
coeff(242)=0.984814682170D+04
coeff(243)=0.146198350395D+05
coeff(244)=0.745651506832D+04
coeff(245)=0.147715915910D+05
coeff(246)=0.731372060562D+04
coeff(247)=0.864095551838D+04
coeff(248)=0.586445506810D+04
coeff(249)=0.151550442041D+04
coeff(250)=-0.669253071548D+04
coeff(251)=-0.651465782464D+04
coeff(252)=-0.149265376685D+05
coeff(253)=-0.936816305740D+04
coeff(254)=0.639360454986D+00
coeff(255)=0.772777486767D+04
coeff(256)=0.126403108737D+05
coeff(257)=-0.165059993000D+04
coeff(258)=0.311341254172D+04
coeff(259)=0.872900249860D+04
coeff(260)=0.232621289715D+04
coeff(261)=0.961979641919D+04
coeff(262)=0.957261215522D+03
coeff(263)=-0.840536541511D+03
coeff(264)=-0.649438983456D+04
coeff(265)=0.568456455892D+04
coeff(266)=0.116712280327D+05
coeff(267)=0.577246813119D+04
coeff(268)=0.151872190367D+03
coeff(269)=0.283077804918D+04
coeff(270)=0.951859493720D+04
coeff(271)=-0.708311184890D+04
coeff(272)=-0.779748873358D+04
coeff(273)=-0.545705508639D+04
coeff(274)=-0.190049912306D+05
coeff(275)=0.152106610295D+05
coeff(276)=-0.165869526550D+05
coeff(277)=0.153146224630D+04
coeff(278)=0.177559138862D+04
coeff(279)=-0.129959862810D+05
coeff(280)=0.644590592306D+04
coeff(281)=0.791901252982D+04
coeff(282)=0.309112893458D+04
coeff(283)=0.884046588770D+04
coeff(284)=-0.164462271139D+05
coeff(285)=-0.399737204064D+04
coeff(286)=0.456817135590D+04
coeff(287)=0.490407612599D+04
coeff(288)=-0.201334768825D+03
coeff(289)=-0.267095804989D+04
coeff(290)=0.968525177822D+04
coeff(291)=0.649485076430D+04
coeff(292)=0.221621848560D+05
coeff(293)=-0.326821566284D+04
coeff(294)=0.106427711562D+05
coeff(295)=-0.929803288950D+04
coeff(296)=-0.493539749201D+04
coeff(297)=0.526085061839D+03
coeff(298)=-0.641270784024D+04
coeff(299)=0.629240035513D+04
coeff(300)=-0.593189733809D+04
coeff(301)=-0.105774293071D+05
coeff(302)=0.194723348223D+04
coeff(303)=-0.351253161055D+04
coeff(304)=0.639928823605D+04
coeff(305)=0.909618146375D+04
coeff(306)=0.175548951245D+04
coeff(307)=-0.202701769693D+04
coeff(308)=0.368178651284D+04
coeff(309)=-0.142867188277D+04
coeff(310)=-0.119005022832D+05
coeff(311)=0.234913807179D+03
coeff(312)=-0.882086410897D+04
coeff(313)=0.857989691628D+04
coeff(314)=0.112331100610D+03
coeff(315)=-0.677324121347D+04
coeff(316)=-0.579349082325D+04
coeff(317)=0.104963749641D+05
coeff(318)=-0.249250620788D+04
coeff(319)=-0.932417547571D+03
coeff(320)=-0.163377243240D+05
coeff(321)=-0.111235456979D+05
coeff(322)=0.303084817580D+04
coeff(323)=-0.100802740262D+05
coeff(324)=0.756966452338D+04
coeff(325)=-0.306670436262D+03
coeff(326)=-0.117523268573D+05
coeff(327)=0.385648577781D+04
coeff(328)=0.547735920368D+04
coeff(329)=-0.816646678411D+04
coeff(330)=-0.116744927068D+05
coeff(331)=0.163892838970D+03
coeff(332)=-0.173578777733D+04
coeff(333)=0.690262763952D+04
coeff(334)=-0.216590128924D+05
coeff(335)=0.180419203179D+04
coeff(336)=-0.222170321033D+05
coeff(337)=0.665925521248D+04
coeff(338)=0.511625878302D+04
coeff(339)=-0.560676624078D+04
coeff(340)=-0.965505271539D+04
coeff(341)=0.840554830125D+04
coeff(342)=-0.638244008568D+04
coeff(343)=0.695946099301D+04
coeff(344)=0.203892610693D+05
coeff(345)=0.549092143388D+04
coeff(346)=0.270458985503D+04
coeff(347)=-0.132332236198D+05
coeff(348)=0.459183762517D+04
coeff(349)=0.539594973362D+04
coeff(350)=-0.270280248662D+03
coeff(351)=0.113014702634D+03
coeff(352)=0.511816961124D+04
coeff(353)=-0.314196210560D+04
coeff(354)=-0.686009614593D+04
coeff(355)=-0.565014787035D+04
coeff(356)=-0.121862689664D+05
coeff(357)=0.809919833964D+04
coeff(358)=0.150978207269D+05
coeff(359)=-0.126567951578D+04
coeff(360)=-0.204194778168D+05
coeff(361)=-0.170663044095D+05
coeff(362)=-0.113523160304D+05
coeff(363)=0.727739072164D+03
coeff(364)=0.192528988425D+04
coeff(365)=0.404845725576D+04
coeff(366)=-0.113273041257D+05
coeff(367)=-0.849686432795D+04
coeff(368)=-0.204286010623D+05
coeff(369)=0.300657864194D+04
coeff(370)=0.122265252022D+05
coeff(371)=-0.402298888961D+04
coeff(372)=0.182385465813D+05
coeff(373)=-0.980305550847D+04
coeff(374)=-0.192016556044D+05
coeff(375)=0.114619367043D+05
coeff(376)=-0.103375999399D+05
coeff(377)=0.159351298700D+05
coeff(378)=-0.339056289566D+04
coeff(379)=-0.316601918347D+04
coeff(380)=-0.919451964134D+04
coeff(381)=-0.122955211149D+04
coeff(382)=-0.133317554034D+05
coeff(383)=-0.102241831166D+05
coeff(384)=-0.720877466524D+04
coeff(385)=0.146807697655D+05
coeff(386)=0.166462210137D+05
coeff(387)=-0.760522317808D+04
coeff(388)=0.172355304068D+05
coeff(389)=0.880868429241D+04
coeff(390)=0.113759529389D+05
coeff(391)=0.603847207474D+04
coeff(392)=-0.334416721526D+04
coeff(393)=-0.211121431813D+05
coeff(394)=-0.596627060815D+04
coeff(395)=-0.112664955740D+05
coeff(396)=0.848817485561D+04
coeff(397)=0.737372815795D+03
coeff(398)=0.228608473656D+04
coeff(399)=0.112775597377D+05
coeff(400)=0.180967910187D+04
coeff(401)=0.336670771045D+03
coeff(402)=-0.123328904777D+05
coeff(403)=-0.584886986411D+04
coeff(404)=0.563644245379D+04
coeff(405)=0.974835720784D+04
coeff(406)=-0.211126657026D+04
coeff(407)=0.325818014221D+04
coeff(408)=0.234148122814D+04
coeff(409)=0.423425758225D+04
coeff(410)=0.127647357354D+05
coeff(411)=0.886094593658D+03
coeff(412)=-0.965170448630D+04
coeff(413)=0.608332426039D+04
coeff(414)=0.389684771528D+04
coeff(415)=-0.169372180128D+04
coeff(416)=0.774843460532D+04
coeff(417)=-0.122147438418D+05
coeff(418)=0.578015993024D+04
coeff(419)=-0.412954771637D+04
coeff(420)=-0.153923326804D+05
coeff(421)=0.115535869516D+04
coeff(422)=-0.184534981275D+04
coeff(423)=-0.788237049683D+04
coeff(424)=0.232935782138D+05
coeff(425)=-0.145497838639D+05
coeff(426)=0.107589103636D+05
coeff(427)=0.174444682428D+05
coeff(428)=0.241914681311D+04
coeff(429)=0.101896507764D+05
coeff(430)=0.165185755412D+05
coeff(431)=0.460442308771D+04
coeff(432)=0.726178254270D+04
coeff(433)=0.173150448486D+05
coeff(434)=0.136500906855D+05
coeff(435)=0.414151337982D+04
coeff(436)=-0.679269195274D+04
coeff(437)=0.188424317736D+05
coeff(438)=-0.256877036522D+05
coeff(439)=0.116946136703D+05
coeff(440)=0.138969826160D+04
coeff(441)=-0.212994213557D+05
coeff(442)=-0.111556841759D+05
coeff(443)=-0.311104548723D+05
coeff(444)=-0.931372350723D+04
coeff(445)=-0.837828822897D+04
coeff(446)=0.137263719970D+03
coeff(447)=0.908128494590D+03
coeff(448)=0.103693243806D+05
coeff(449)=-0.192663897658D+04
coeff(450)=-0.104305130017D+05
coeff(451)=0.469991869964D+04
coeff(452)=-0.111342330740D+05
coeff(453)=0.239084230402D+04
coeff(454)=0.371579489097D+04
coeff(455)=0.142376378506D+05
coeff(456)=-0.200388501712D+05
coeff(457)=0.394405435096D+02
coeff(458)=-0.329138262674D+04
coeff(459)=0.504445662615D+04
coeff(460)=-0.638377105906D+03
coeff(461)=0.130860988334D+05
coeff(462)=0.672824982921D+04
coeff(463)=-0.107373345423D+03
coeff(464)=-0.255863258113D+04
coeff(465)=-0.157711550515D+05
coeff(466)=-0.990504355604D+04
coeff(467)=0.140776508151D+04
coeff(468)=0.783364228305D+03
coeff(469)=-0.166096928345D+05
coeff(470)=-0.123225277631D+04
coeff(471)=0.391867512361D+04
coeff(472)=0.312232196207D+04
coeff(473)=-0.120171192571D+04
coeff(474)=0.577219069933D+04
coeff(475)=-0.367270582981D+04
coeff(476)=-0.944163313423D+04
coeff(477)=-0.620028141169D+04
coeff(478)=-0.187319199953D+05
coeff(479)=0.998102532404D+04
coeff(480)=-0.113374659300D+05
coeff(481)=-0.268055073226D+04
coeff(482)=-0.188018961268D+04
coeff(483)=0.577519886064D+04
coeff(484)=-0.677831535327D+04
coeff(485)=-0.922207749305D+04
coeff(486)=-0.565561877388D+04
coeff(487)=-0.369621885625D+03
coeff(488)=-0.267813493050D+04
coeff(489)=0.219302513150D+04
coeff(490)=-0.931998854927D+04
coeff(491)=0.237506784996D+04
coeff(492)=0.157006136811D+05
coeff(493)=0.145409957693D+04
coeff(494)=0.174739934635D+05
coeff(495)=0.877348861308D+04
coeff(496)=0.521402789906D+04
coeff(497)=0.707670626627D+04
coeff(498)=0.110387581200D+05
coeff(499)=0.595800601835D+04
coeff(500)=-0.134849834375D+05
coeff(501)=-0.691783353356D+04
coeff(502)=0.412869413146D+02
coeff(503)=0.132325113416D+05
coeff(504)=-0.896245473079D+04
coeff(505)=0.106016561747D+05
coeff(506)=-0.102427731879D+05
coeff(507)=-0.174522874212D+05
coeff(508)=-0.136122804750D+05
coeff(509)=0.127846737120D+04
coeff(510)=0.250555309695D+04
coeff(511)=-0.898761637856D+04
coeff(512)=0.238284111837D+05
coeff(513)=-0.655130561285D+04
coeff(514)=-0.254807282312D+05
coeff(515)=0.341515146520D+05
coeff(516)=-0.115474215382D+05
coeff(517)=-0.108672044978D+05
coeff(518)=0.160479411020D+05
coeff(519)=-0.114353749820D+05
coeff(520)=-0.170084716614D+05
coeff(521)=-0.101255704393D+04
coeff(522)=0.671621655314D+04
coeff(523)=-0.834955950796D+04
coeff(524)=-0.153413447998D+05
coeff(525)=-0.534368831147D+04
coeff(526)=-0.965760491976D+04
coeff(527)=0.492528690710D+04
coeff(528)=0.220469747214D+05
coeff(529)=-0.569770306964D+04
coeff(530)=-0.265310899973D+05
coeff(531)=0.175463820273D+05
coeff(532)=0.156187225422D+05
coeff(533)=-0.176978920492D+05
coeff(534)=-0.650370142789D+04
coeff(535)=0.271171398550D+05
coeff(536)=-0.550798885516D+03
coeff(537)=0.122216539258D+05
coeff(538)=-0.272220811005D+05
coeff(539)=0.209271977998D+05
coeff(540)=0.174512452094D+04
coeff(541)=-0.256490499012D+05
coeff(542)=0.311215069309D+05
coeff(543)=0.739874899282D+04
coeff(544)=0.145593071405D+05
coeff(545)=-0.110909733127D+05
coeff(546)=0.171213888260D+05
coeff(547)=-0.691032541818D+04
coeff(548)=0.308151687956D+04
coeff(549)=0.241330063989D+04
coeff(550)=-0.138413921838D+05
coeff(551)=0.481460073335D+04
coeff(552)=0.567490738584D+04
coeff(553)=-0.723857178811D+04
coeff(554)=0.196476253282D+05
coeff(555)=0.100888858885D+05
coeff(556)=0.283039764009D+04
coeff(557)=-0.876611429470D+04
coeff(558)=-0.750997465243D+04
coeff(559)=-0.175797177264D+04
coeff(560)=-0.320147642348D+04
coeff(561)=0.775631781419D+03
coeff(562)=-0.111447731387D+04
coeff(563)=-0.543635255137D+04
coeff(564)=0.572494875214D+03
coeff(565)=-0.146656944298D+04
coeff(566)=0.497090875426D+04
coeff(567)=-0.791313910132D+04
coeff(568)=-0.271276482925D+03
coeff(569)=0.122694178344D+05
coeff(570)=-0.218276538222D+04
coeff(571)=-0.380448028936D+04
coeff(572)=0.109515527104D+05
coeff(573)=-0.920489988787D+04
coeff(574)=0.644532907618D+04
coeff(575)=-0.134136222703D+04
coeff(576)=0.405975429687D+04
coeff(577)=-0.241365000281D+04
coeff(578)=0.807986591192D+04
coeff(579)=-0.118808969356D+05
coeff(580)=0.158387114728D+05
coeff(581)=0.175298621452D+04
coeff(582)=-0.998327396109D+03
coeff(583)=0.135212501759D+05
coeff(584)=-0.425591042145D+04
coeff(585)=0.839792383385D+04
coeff(586)=0.200743980982D+04
coeff(587)=-0.159052702675D+04
coeff(588)=-0.123312578236D+04
coeff(589)=-0.191834050419D+05
coeff(590)=0.627513386689D+04
coeff(591)=-0.108545396651D+05
coeff(592)=0.318342990025D+05
coeff(593)=-0.326018891449D+05
coeff(594)=0.986279634090D+04
coeff(595)=0.174471819346D+05
coeff(596)=0.127402316157D+05
coeff(597)=-0.120210667780D+05
coeff(598)=-0.495542146525D+04
coeff(599)=0.178230557908D+04
coeff(600)=-0.829618206647D+03
coeff(601)=-0.295742314249D+03
coeff(602)=-0.848287843117D+04
coeff(603)=0.505495763183D+04
coeff(604)=0.713895294622D+04
coeff(605)=0.120977963585D+05
coeff(606)=-0.130451250098D+05
coeff(607)=0.244012576132D+05
coeff(608)=-0.738427206500D+04
coeff(609)=-0.671761061360D+04
coeff(610)=0.499248348729D+04
coeff(611)=-0.225140562799D+05
coeff(612)=-0.720784342823D+04
coeff(613)=-0.173580454302D+05
coeff(614)=0.620091474593D+03
coeff(615)=0.111687759514D+03
coeff(616)=0.119473731860D+04
coeff(617)=-0.536207593460D+04
coeff(618)=0.813567545212D+03
coeff(619)=-0.995175483368D+03
coeff(620)=-0.134068938212D+04
coeff(621)=0.125831420516D+05
coeff(622)=0.906577770841D+04
coeff(623)=-0.774688288813D+04
coeff(624)=0.110884767907D+05
coeff(625)=0.945830281267D+03
coeff(626)=0.208191755241D+05
coeff(627)=-0.911453485805D+04
coeff(628)=0.990853961066D+04
coeff(629)=0.162232578015D+05
coeff(630)=-0.107877571542D+05
coeff(631)=0.114105263745D+05
coeff(632)=0.203851237116D+05
coeff(633)=0.215720813628D+05
coeff(634)=0.189250448513D+05
coeff(635)=-0.115241456406D+05
coeff(636)=-0.713404963518D+04
coeff(637)=-0.118592837241D+05
coeff(638)=-0.768076310722D+04
coeff(639)=-0.834492565513D+04
coeff(640)=0.321565497122D+04
coeff(641)=-0.171630072012D+05
coeff(642)=0.213761512067D+05
coeff(643)=0.122410987436D+05
coeff(644)=-0.301476519063D+04
coeff(645)=0.101750026327D+05
coeff(646)=-0.981257346146D+04
coeff(647)=0.135399390880D+05
coeff(648)=-0.643392129549D+04
coeff(649)=-0.134735452335D+04
coeff(650)=0.114631873374D+05
coeff(651)=0.214145850812D+04
coeff(652)=0.130332007164D+05
coeff(653)=0.962025152261D+04
coeff(654)=-0.956786525143D+04
coeff(655)=0.198301496867D+05
coeff(656)=0.321348699518D+05
coeff(657)=-0.801504527903D+04
coeff(658)=-0.115435828407D+04
coeff(659)=0.251317484057D+03
coeff(660)=0.874078655330D+04
coeff(661)=-0.130137807578D+05
coeff(662)=0.246947528629D+04
coeff(663)=0.118188071740D+05
coeff(664)=-0.613502367359D+04
coeff(665)=-0.282562843547D+04
coeff(666)=0.751644919719D+04
coeff(667)=0.104051967437D+05
coeff(668)=-0.858690782982D+04
coeff(669)=-0.762994366725D+04
coeff(670)=-0.728594719108D+04
coeff(671)=-0.811347155593D+04
coeff(672)=-0.633047913852D+04
coeff(673)=0.689483536729D+04
coeff(674)=-0.233922185641D+03
coeff(675)=-0.297913123215D+04
coeff(676)=-0.143992134181D+05
coeff(677)=-0.317128773666D+05
coeff(678)=-0.186869895992D+05
coeff(679)=0.174310456014D+05
coeff(680)=0.490094144315D+04
coeff(681)=0.427574761413D+04
coeff(682)=0.398086655152D+04
coeff(683)=-0.816082204924D+04
coeff(684)=-0.378733842493D+04
coeff(685)=0.176162424208D+05
coeff(686)=-0.111728760857D+05
coeff(687)=-0.130244662724D+05
coeff(688)=-0.796448396320D+04
coeff(689)=0.165599661000D+03
coeff(690)=-0.753638916771D+04
coeff(691)=0.655140243382D+04
coeff(692)=0.706430759531D+04
coeff(693)=0.669478156630D+04
coeff(694)=-0.753046912980D+04
coeff(695)=-0.814553150325D+03
coeff(696)=0.131462229999D+05
coeff(697)=0.146832890890D+05
coeff(698)=-0.776937930464D+04
coeff(699)=-0.900975983699D+04
coeff(700)=-0.393167380571D+04
coeff(701)=-0.972549782362D+04
coeff(702)=0.566902618597D+04
coeff(703)=0.143796156363D+05
coeff(704)=0.127272484674D+05
coeff(705)=0.852181540555D+04
coeff(706)=0.292996274117D+03
coeff(707)=-0.287980512354D+04
coeff(708)=-0.860168866482D+04
coeff(709)=-0.153095726821D+05
coeff(710)=0.324077648767D+04
coeff(711)=-0.674265672967D+03
coeff(712)=0.100563522437D+05
coeff(713)=-0.691048381501D+04
coeff(714)=0.872151037817D+04
coeff(715)=0.513715928143D+04
coeff(716)=-0.704546794998D+04
coeff(717)=-0.483787169814D+03
coeff(718)=-0.332978066228D+04
coeff(719)=0.368500273712D+04
coeff(720)=-0.962008364627D+04
coeff(721)=0.234963253835D+05
coeff(722)=0.792359784839D+03
coeff(723)=-0.181335587993D+05
coeff(724)=-0.104454194309D+05
coeff(725)=0.752597064228D+03
coeff(726)=0.342606454711D+04
coeff(727)=-0.596357106871D+04
coeff(728)=-0.298185595822D+04
coeff(729)=-0.814792154065D+04
coeff(730)=0.170536201735D+05
coeff(731)=-0.418547023668D+04
coeff(732)=0.821464693744D+04
coeff(733)=-0.661264439548D+04
coeff(734)=0.524620441827D+03
coeff(735)=0.231038077553D+05
coeff(736)=0.126129799785D+05
coeff(737)=-0.112965675869D+05
coeff(738)=0.108116126665D+05
coeff(739)=-0.174881850973D+04
coeff(740)=-0.636649005686D+04
coeff(741)=0.231451244052D+04
coeff(742)=-0.473752084614D+05
coeff(743)=0.271386504280D+04
coeff(744)=-0.165490348796D+04
coeff(745)=-0.941704420051D+04
coeff(746)=-0.156860416490D+05
coeff(747)=-0.765250132371D+04
coeff(748)=0.351991591651D+04
coeff(749)=-0.114668113379D+05
coeff(750)=-0.324324507242D+05
coeff(751)=0.313264944122D+04
coeff(752)=0.134860640084D+05
coeff(753)=-0.715530206038D+04
coeff(754)=0.187976643216D+04
coeff(755)=-0.636620686424D+04
coeff(756)=0.218304669635D+05
coeff(757)=0.168161324027D+05
coeff(758)=0.936581784399D+04
coeff(759)=-0.904766663817D+04
coeff(760)=-0.683119231123D+04
coeff(761)=0.935372322498D+04
coeff(762)=0.101246229127D+05
coeff(763)=0.175048548705D+04
coeff(764)=-0.760501659385D+04
coeff(765)=0.175634175040D+05
coeff(766)=-0.241032547204D+04
coeff(767)=-0.126501764835D+05
coeff(768)=-0.138515004773D+05
coeff(769)=0.190556502054D+05
coeff(770)=0.195831902641D+05
coeff(771)=-0.354162760331D+05
coeff(772)=-0.166706431119D+05
coeff(773)=0.136029646014D+05
coeff(774)=0.173858499647D+05
coeff(775)=0.162044004143D+05
coeff(776)=-0.314216713983D+04
coeff(777)=-0.355371996464D+04
coeff(778)=0.598948467658D+04
coeff(779)=0.197885897321D+05
coeff(780)=0.175028598282D+04
coeff(781)=0.115200126218D+05
coeff(782)=-0.203291654469D+04
coeff(783)=-0.582920765676D+04
coeff(784)=0.833169252706D+04
coeff(785)=-0.679656252280D+04
coeff(786)=-0.988232372958D+04
coeff(787)=-0.151976043318D+05
coeff(788)=0.140205525208D+05
coeff(789)=-0.371871050516D+04
coeff(790)=-0.550718989202D+03
coeff(791)=-0.411273533774D+04
coeff(792)=0.725736617853D+04
coeff(793)=-0.533013835726D+04
coeff(794)=-0.190644244235D+03
coeff(795)=0.174424886350D+04
coeff(796)=-0.535323880456D+03
coeff(797)=0.994736951621D+04
coeff(798)=0.245770096037D+04
coeff(799)=-0.365282779503D+03
coeff(800)=-0.127104760917D+05
coeff(801)=0.172140895404D+05
coeff(802)=0.650892915012D+04
coeff(803)=-0.111873647190D+05
coeff(804)=0.182542401094D+05
coeff(805)=0.747912594788D+04
coeff(806)=-0.562775250458D+04
coeff(807)=-0.142546113593D+05
coeff(808)=0.954252006846D+04
coeff(809)=0.519439180017D+04
coeff(810)=0.121831830679D+05
coeff(811)=-0.180200173088D+05
coeff(812)=0.170657748017D+05
coeff(813)=-0.193680041595D+05
coeff(814)=-0.134510140239D+05
coeff(815)=0.632990092514D+03
coeff(816)=-0.216171099655D+05
coeff(817)=-0.154770247325D+05
coeff(818)=0.222529541368D+05
coeff(819)=0.497392399377D+04
coeff(820)=0.139805860081D+05
coeff(821)=-0.106538834881D+05
coeff(822)=0.158183424475D+05
coeff(823)=-0.254256780335D+04
coeff(824)=0.751608347760D+04
coeff(825)=-0.106659872013D+05
coeff(826)=0.126376608480D+05
coeff(827)=-0.824224086451D+04
coeff(828)=-0.777403455593D+04
coeff(829)=-0.493851062816D+04
coeff(830)=-0.383588501582D+04
coeff(831)=0.658957309529D+04
coeff(832)=0.663117361917D+04
coeff(833)=0.261090820835D+05
coeff(834)=-0.934453102698D+04
coeff(835)=-0.156670592383D+05
coeff(836)=0.142069719272D+05
coeff(837)=0.697425004759D+04
coeff(838)=0.929847286312D+04
coeff(839)=-0.127773936882D+05
coeff(840)=-0.568748853103D+04
coeff(841)=-0.318875655110D+04
coeff(842)=0.402192923815D+04
coeff(843)=0.356152114910D+04
coeff(844)=-0.362450602295D+04
coeff(845)=0.704806154574D+04
coeff(846)=-0.855737434564D+04
coeff(847)=0.111783359793D+04
coeff(848)=-0.238386697924D+05
coeff(849)=0.940864233598D+04
coeff(850)=0.155377547637D+05
coeff(851)=-0.802729818136D+04
coeff(852)=-0.395530140901D+04
coeff(853)=-0.657938108327D+04
coeff(854)=0.621723358420D+04
coeff(855)=-0.440529313458D+04
coeff(856)=-0.311169493733D+05
coeff(857)=0.171596580698D+05
coeff(858)=-0.100015600730D+05
coeff(859)=-0.163605366247D+05
coeff(860)=0.358003952842D+04
coeff(861)=0.153105907797D+05
coeff(862)=0.327758513917D+02
coeff(863)=-0.304111529399D+04
coeff(864)=0.457875879045D+04
coeff(865)=0.840099393335D+02
coeff(866)=-0.259495807092D+04
coeff(867)=-0.196653258416D+05
coeff(868)=-0.157642719076D+04
coeff(869)=0.857618686236D+04
coeff(870)=-0.103019677303D+05
coeff(871)=-0.478721972714D+04
coeff(872)=-0.672122610148D+04
coeff(873)=0.361356276187D+04
coeff(874)=0.826009031158D+04
coeff(875)=0.294662504366D+04
coeff(876)=0.485420205671D+04
coeff(877)=-0.142161191650D+05
coeff(878)=0.307414099067D+04
coeff(879)=0.213461456271D+05
coeff(880)=-0.224622698490D+04
coeff(881)=-0.993530500761D+04
coeff(882)=0.480890285145D+04
coeff(883)=0.837130139690D+04
coeff(884)=0.399448105377D+04
coeff(885)=-0.225495073665D+04
coeff(886)=0.148677216129D+05
coeff(887)=0.249351238906D+05
coeff(888)=0.648150938070D+04
coeff(889)=-0.106181101518D+05
coeff(890)=0.240881084498D+05
coeff(891)=0.120633516787D+05
coeff(892)=0.154302756164D+05
coeff(893)=-0.493321551426D+04
coeff(894)=-0.151279969171D+03
coeff(895)=0.110103345054D+05
coeff(896)=0.259546376130D+05
coeff(897)=-0.180124946627D+05
coeff(898)=-0.750004429506D+04
coeff(899)=0.309649791116D+05
coeff(900)=-0.952956475173D+04
coeff(901)=0.152310684208D+05
coeff(902)=-0.366404608737D+05
coeff(903)=0.155654040759D+05
coeff(904)=-0.286231593895D+04
coeff(905)=-0.633248232756D+04
coeff(906)=-0.146525441536D+05
coeff(907)=0.589395941155D+04
coeff(908)=-0.203774740322D+05
coeff(909)=-0.143492471961D+05
coeff(910)=0.102808893566D+05
coeff(911)=0.202133395012D+05
coeff(912)=-0.404633680045D+05
coeff(913)=0.284315664499D+05
coeff(914)=-0.325219203795D+05
coeff(915)=0.102383341874D+05
coeff(916)=-0.353239437471D+04
coeff(917)=-0.109918787439D+05
coeff(918)=0.169579730825D+05
coeff(919)=-0.306518286977D+04
coeff(920)=-0.247285728575D+05
coeff(921)=0.515183507117D+04
coeff(922)=0.192503634766D+05
coeff(923)=-0.193594227723D+05
coeff(924)=-0.268098865745D+05
coeff(925)=-0.143040829062D+02
coeff(926)=-0.177601019118D+05
coeff(927)=-0.101303753728D+05
coeff(928)=-0.215544039895D+05
coeff(929)=0.329482476895D+04
coeff(930)=0.679004271655D+04
coeff(931)=0.411157021608D+03
coeff(932)=-0.919532250127D+04
coeff(933)=0.332358433882D+04
coeff(934)=-0.364370502858D+04
coeff(935)=0.626153866772D+04
coeff(936)=-0.129804089146D+05
coeff(937)=0.190064232865D+05
coeff(938)=0.844631778834D+04
coeff(939)=0.256727158819D+04
coeff(940)=-0.215037820436D+04
coeff(941)=0.202628226484D+05
coeff(942)=0.983252572409D+04
coeff(943)=-0.724103126547D+04
coeff(944)=-0.391827142589D+04
coeff(945)=0.475346041756D+04
coeff(946)=-0.939217455955D+04
coeff(947)=0.763284253255D+04
coeff(948)=0.172142629701D+03
coeff(949)=0.203631983263D+05
coeff(950)=-0.176585383630D+04
coeff(951)=0.176735489804D+05
coeff(952)=0.942691522127D+03
coeff(953)=0.662643718639D+04
coeff(954)=0.149695421592D+04
coeff(955)=-0.190480426497D+05
coeff(956)=-0.165152345392D+05
coeff(957)=-0.143158214784D+05
coeff(958)=0.129841149764D+03
coeff(959)=0.148630028021D+05
coeff(960)=0.241618165860D+04
coeff(961)=0.353836582966D+04
coeff(962)=0.532254229654D+03
coeff(963)=-0.172556190600D+05
coeff(964)=0.198110132231D+04
coeff(965)=0.398073864513D+04
coeff(966)=0.310764622885D+05
coeff(967)=-0.124445819615D+05
coeff(968)=0.256569147463D+04
coeff(969)=0.469568353477D+04
coeff(970)=-0.120627508419D+05
coeff(971)=0.146199413942D+04
coeff(972)=-0.270487720911D+04
coeff(973)=0.757959908132D+04
coeff(974)=-0.456057344377D+04
coeff(975)=-0.227975307884D+04
coeff(976)=-0.138916227839D+05
coeff(977)=0.183722869337D+05
coeff(978)=-0.559051687667D+04
coeff(979)=-0.492605075483D+04
coeff(980)=0.247739255244D+04
coeff(981)=-0.410984865539D+04
coeff(982)=0.409920145342D+04
coeff(983)=-0.339027229100D+03
coeff(984)=-0.836767160317D+04
coeff(985)=-0.354210473949D+04
coeff(986)=0.114685018635D+04
coeff(987)=0.561258072363D+04
coeff(988)=0.178264249281D+02
coeff(989)=0.380337403117D+04
coeff(990)=0.527026312991D+04
coeff(991)=-0.529591155237D+04
coeff(992)=0.303103038476D+04
coeff(993)=-0.616750599205D+04
coeff(994)=-0.278936688872D+05
coeff(995)=0.204992619314D+05
coeff(996)=-0.104239824060D+05
coeff(997)=0.195452006328D+04
coeff(998)=0.365740902080D+04
coeff(999)=-0.377271533415D+02
coeff(1000)=0.351956472302D+04
coeff(1001)=0.657421381625D+04
coeff(1002)=-0.223454917040D+04
coeff(1003)=-0.154462985282D+04
coeff(1004)=-0.429519742390D+04
coeff(1005)=0.839822245651D+04
coeff(1006)=0.404567179339D+04
coeff(1007)=-0.357765334625D+04
coeff(1008)=-0.144721525805D+05
coeff(1009)=0.933389566768D+04
coeff(1010)=0.129595758300D+04
coeff(1011)=-0.151391230770D+05
coeff(1012)=0.227580781708D+05
coeff(1013)=0.559482415808D+04
coeff(1014)=0.318677884365D+04
coeff(1015)=0.112458116675D+05
coeff(1016)=0.313333616749D+04
coeff(1017)=-0.116209282214D+05
coeff(1018)=-0.395279119060D+04
coeff(1019)=-0.450211678197D+04
coeff(1020)=-0.915168990640D+03
coeff(1021)=-0.667466653569D+03
coeff(1022)=0.547392367781D+03
coeff(1023)=0.276634489015D+03
coeff(1024)=0.254421886100D+05
coeff(1025)=-0.105769918044D+05
coeff(1026)=0.125318541577D+05
coeff(1027)=-0.370138867125D+03
coeff(1028)=0.103064242574D+05
coeff(1029)=0.193252321309D+05
coeff(1030)=-0.572610455267D+04
coeff(1031)=-0.135673082409D+05
coeff(1032)=-0.789231059869D+04
coeff(1033)=0.355473149924D+04
coeff(1034)=0.260833786046D+05
coeff(1035)=-0.219267633440D+05
coeff(1036)=0.599920445726D+04
coeff(1037)=0.909624606757D+04
coeff(1038)=0.656974804940D+04
coeff(1039)=-0.123387502123D+05
coeff(1040)=0.299173107075D+04
coeff(1041)=-0.560274764474D+04
coeff(1042)=0.262457893733D+04
coeff(1043)=-0.162525802102D+05
coeff(1044)=0.932926662578D+04
coeff(1045)=0.149270463130D+03
coeff(1046)=-0.168931587121D+05
coeff(1047)=0.966587312730D+04
coeff(1048)=-0.736032516408D+04
coeff(1049)=-0.192169303134D+05
coeff(1050)=0.212789964317D+05
coeff(1051)=-0.230672488821D+05
coeff(1052)=-0.282866691360D+05
coeff(1053)=-0.561162350393D+04
coeff(1054)=0.154762067840D+05
coeff(1055)=-0.567373109329D+04
coeff(1056)=-0.219870541355D+05
coeff(1057)=0.187888770508D+04
coeff(1058)=0.487020599363D+04
coeff(1059)=0.148554361587D+05
coeff(1060)=-0.621038473109D+04
coeff(1061)=-0.119148060983D+05
coeff(1062)=0.543621305109D+04
coeff(1063)=-0.238966365945D+05
coeff(1064)=0.518979185145D+04
coeff(1065)=-0.162548431032D+05
coeff(1066)=-0.504685279081D+04
coeff(1067)=-0.579866748373D+04
coeff(1068)=0.936216780341D+04
coeff(1069)=-0.157481834145D+05
coeff(1070)=-0.125913001607D+05
coeff(1071)=0.543898071294D+04
coeff(1072)=0.543422605006D+04
coeff(1073)=0.556421465805D+04
coeff(1074)=-0.776977472987D+04
coeff(1075)=0.394971825911D+04
coeff(1076)=0.917031475953D+04
coeff(1077)=0.124276533779D+05
coeff(1078)=-0.644625821339D+04
coeff(1079)=0.154019204426D+05
coeff(1080)=0.338162174440D+04
coeff(1081)=-0.796758105960D+04
coeff(1082)=0.271373401378D+05
coeff(1083)=-0.121408397680D+05
coeff(1084)=0.215987310514D+05
coeff(1085)=-0.448013227350D+04
coeff(1086)=0.650739390901D+04
coeff(1087)=-0.205377065361D+05
coeff(1088)=0.543594268813D+04
coeff(1089)=0.160160914405D+05
coeff(1090)=-0.108213850684D+05
coeff(1091)=-0.587952679296D+04
coeff(1092)=0.755490718702D+04
coeff(1093)=-0.568467046267D+04
coeff(1094)=-0.740372604163D+04
coeff(1095)=0.987565623583D+03
coeff(1096)=0.162406295074D+05
coeff(1097)=-0.150180799031D+05
coeff(1098)=0.303406558392D+05
coeff(1099)=0.265831136076D+05
coeff(1100)=0.223028012244D+05
coeff(1101)=-0.279967388107D+05
coeff(1102)=0.371479060770D+04
coeff(1103)=0.225387997102D+05
coeff(1104)=-0.348807245193D+04
coeff(1105)=0.674845839925D+04
coeff(1106)=0.194018287237D+04
coeff(1107)=0.148275438106D+05
coeff(1108)=0.508010570745D+04
coeff(1109)=0.331645069523D+04
coeff(1110)=0.653181543577D+04
coeff(1111)=-0.952093572195D+04
coeff(1112)=-0.182372466906D+04
coeff(1113)=0.158408344596D+05
coeff(1114)=0.487674967301D+03
coeff(1115)=-0.171963381421D+05
coeff(1116)=-0.386464026463D+04
coeff(1117)=-0.222027925845D+05
coeff(1118)=0.314339151948D+04
coeff(1119)=0.173457091488D+05
coeff(1120)=-0.182399534400D+03
coeff(1121)=-0.213988283135D+05
coeff(1122)=0.197174217268D+05
coeff(1123)=-0.118613521853D+05
coeff(1124)=-0.173537052059D+05
coeff(1125)=-0.129569028924D+04
coeff(1126)=-0.449547161859D+03
coeff(1127)=0.969681770878D+04
coeff(1128)=0.132575936477D+05
coeff(1129)=0.380814559191D+04
coeff(1130)=0.865849303664D+03
coeff(1131)=-0.435197772454D+04
coeff(1132)=0.751853815918D+04
coeff(1133)=-0.509588080270D+04
coeff(1134)=-0.351976825006D+05
coeff(1135)=0.167302811661D+05
coeff(1136)=-0.536781407015D+04
coeff(1137)=-0.401150185877D+04
coeff(1138)=0.804201394654D+04
coeff(1139)=0.484165427790D+04
coeff(1140)=-0.820458350548D+03
coeff(1141)=-0.587324017945D+04
coeff(1142)=0.933604436321D+04
coeff(1143)=-0.201023518497D+05
coeff(1144)=-0.164937561883D+05
coeff(1145)=-0.295724333678D+05
coeff(1146)=0.202206646598D+05
coeff(1147)=-0.818526193114D+04
coeff(1148)=-0.990845415469D+04
coeff(1149)=0.740343121890D+03
coeff(1150)=-0.974072460723D+04
coeff(1151)=-0.676623842359D+04
coeff(1152)=0.127641058830D+05
coeff(1153)=-0.238465781619D+03
coeff(1154)=0.538952800609D+04
coeff(1155)=0.113625829594D+05
coeff(1156)=0.142133410299D+05
coeff(1157)=0.581559515750D+04
coeff(1158)=-0.169567790805D+05
coeff(1159)=-0.479755455331D+04
coeff(1160)=-0.676891953154D+04
coeff(1161)=-0.253370992543D+03
coeff(1162)=0.134576466415D+05
coeff(1163)=-0.534871759191D+04
coeff(1164)=-0.219481868296D+04
coeff(1165)=0.465390962092D+04
coeff(1166)=0.875274384968D+04
coeff(1167)=-0.263280403509D+04
coeff(1168)=0.164527355023D+04
coeff(1169)=-0.350126234992D+04
coeff(1170)=0.400768121110D+04
coeff(1171)=-0.571026187020D+04
coeff(1172)=0.180654010125D+05
coeff(1173)=-0.291256864003D+04
coeff(1174)=-0.686122376732D+04
coeff(1175)=-0.612733954265D+04
coeff(1176)=0.553639466780D+04
coeff(1177)=0.219673595330D+04
coeff(1178)=-0.740925215858D+04
coeff(1179)=0.765554262291D+04
coeff(1180)=-0.421202578604D+03
coeff(1181)=-0.194443903560D+04
coeff(1182)=-0.338662046470D+04
coeff(1183)=-0.188314730111D+04
coeff(1184)=-0.446051437922D+04
coeff(1185)=0.286879017030D+04
coeff(1186)=0.288746285290D+04
coeff(1187)=0.413321381271D+04
coeff(1188)=-0.215167560311D+04
coeff(1189)=0.610337496378D+04
coeff(1190)=-0.785856631163D+03
coeff(1191)=-0.556121577893D+04
coeff(1192)=0.178415810866D+04
coeff(1193)=0.118100036243D+03
coeff(1194)=-0.110649770068D+04
coeff(1195)=0.135602805228D+04
coeff(1196)=-0.133680152060D+04
coeff(1197)=0.368319036697D+04
coeff(1198)=-0.322759121324D+03
coeff(1199)=-0.402849095245D+04
coeff(1200)=0.498986287156D+04
coeff(1201)=-0.433670369319D+04
coeff(1202)=-0.476749679405D+04
coeff(1203)=0.563465723463D+04
coeff(1204)=0.388869358637D+04
coeff(1205)=0.216027524732D+04
coeff(1206)=-0.555787395685D+02
coeff(1207)=0.438014092349D+03
coeff(1208)=0.124429038529D+04
coeff(1209)=0.388426084038D+04
coeff(1210)=-0.122755640078D+05
coeff(1211)=0.225621981354D+04
coeff(1212)=0.569370903027D+03
coeff(1213)=0.621471259842D+04
coeff(1214)=0.344428961543D+04
coeff(1215)=0.318455285731D+04
coeff(1216)=0.963466143007D+03
coeff(1217)=-0.107491736552D+05
coeff(1218)=-0.692147745421D+04
coeff(1219)=0.558401317640D+04
coeff(1220)=-0.415368089399D+04
coeff(1221)=-0.191370752380D+04
coeff(1222)=0.165094265232D+04
coeff(1223)=-0.434725498924D+03
coeff(1224)=0.154253476813D+05
coeff(1225)=0.677697750549D+04
coeff(1226)=-0.454674870283D+04
coeff(1227)=-0.366360268737D+04
coeff(1228)=-0.177403101702D+04
coeff(1229)=0.777426981184D+04
coeff(1230)=0.648474411876D+04
coeff(1231)=-0.285222493307D+03
coeff(1232)=0.325355765054D+04
coeff(1233)=0.889462945917D+04
coeff(1234)=-0.159258634660D+04
coeff(1235)=0.318385845046D+04
coeff(1236)=-0.528134714897D+03
coeff(1237)=-0.305763892855D+04
coeff(1238)=-0.227616239135D+04
coeff(1239)=-0.100025670647D+05
coeff(1240)=-0.247223921547D+04
coeff(1241)=0.635119462230D+04
coeff(1242)=0.387472403851D+02
coeff(1243)=-0.791922284140D+04
coeff(1244)=0.120641020379D+04
coeff(1245)=0.381103947043D+03
coeff(1246)=0.202938498545D+04
coeff(1247)=0.442158777819D+04
coeff(1248)=-0.553901314593D+04
coeff(1249)=-0.722490667625D+04
coeff(1250)=0.452974832952D+04
coeff(1251)=0.256347790830D+04
coeff(1252)=0.300220449892D+04
coeff(1253)=-0.746112180625D+04
coeff(1254)=0.649762151172D+04
coeff(1255)=0.166131280013D+04
coeff(1256)=-0.530223382123D+04
coeff(1257)=-0.475681004122D+04
coeff(1258)=0.750332351590D+04
coeff(1259)=0.533636101478D+03
coeff(1260)=-0.517035950112D+04
coeff(1261)=-0.306928251906D+04
coeff(1262)=-0.449408160166D+04
coeff(1263)=0.977633093689D+03
coeff(1264)=-0.338499850243D+04
coeff(1265)=-0.161358823389D+04
coeff(1266)=0.229099183249D+04
coeff(1267)=0.757561699551D+04
coeff(1268)=-0.519552498041D+04
coeff(1269)=-0.236324487486D+04
coeff(1270)=-0.879554014800D+04
coeff(1271)=0.108021217076D+05
coeff(1272)=0.136141176532D+05
coeff(1273)=-0.375900857591D+04
coeff(1274)=-0.682336540555D+04
coeff(1275)=0.757900550092D+03
coeff(1276)=0.226188716411D+04
coeff(1277)=0.486410546488D+04
coeff(1278)=-0.318659378970D+04
coeff(1279)=-0.598661183525D+04
coeff(1280)=-0.350155129083D+04
coeff(1281)=0.761649874377D+02
coeff(1282)=0.232416953515D+04
coeff(1283)=0.284759882506D+04
coeff(1284)=-0.277046735188D+04
coeff(1285)=0.167778860056D+04
coeff(1286)=0.867139360017D+03
coeff(1287)=0.538083409193D+04
coeff(1288)=-0.354360767683D+04
coeff(1289)=0.850671765735D+03
coeff(1290)=-0.571835653170D+03
coeff(1291)=0.118812569607D+04
coeff(1292)=-0.312140741334D+04
coeff(1293)=0.144237371026D+04
coeff(1294)=-0.712539043512D+03
coeff(1295)=-0.515077608061D+03
coeff(1296)=-0.103996661614D+05
coeff(1297)=0.937670998066D+04
coeff(1298)=-0.583808079765D+04
coeff(1299)=0.349552683224D+03
coeff(1300)=0.750317701718D+02

! Calculate MEG
x(1)=exp(-((r(1)-ra1))/a1 -((r(1)-rb1)**2)/b1)
x(2)=exp(-((r(2)-ra1))/a1 -((r(2)-rb1)**2)/b1)
x(3)=exp(-((r(3)-ra2))/a2 -((r(3)-rb2)**2)/b2)
x(4)=exp(-((r(4)-ra1))/a1 -((r(4)-rb1)**2)/b1)
x(5)=exp(-((r(5)-ra2))/a2 -((r(5)-rb2)**2)/b2)
x(6)=exp(-((r(6)-ra2))/a2 -((r(6)-rb2)**2)/b2)

! Calculate dMEG/dr(6,6) for giving R(6)
! dmsdr(i,j)=0  i!=j
! Initialize dmsdr
      do i=1,6
        do j=1,6
          dmsdr(i,j)=0.0d0
        enddo
      enddo
dmsdr(1,1)=(-2.0d0*(r(1)-rb1)/b1-1/a1)*Exp(-(r(1)-ra1)/a1-((r(1)-rb1)**2.0d0)/b1)
dmsdr(2,2)=(-2.0d0*(r(2)-rb1)/b1-1/a1)*Exp(-(r(2)-ra1)/a1-((r(2)-rb1)**2.0d0)/b1)
dmsdr(3,3)=(-2.0d0*(r(3)-rb2)/b2-1/a2)*Exp(-(r(3)-ra2)/a2-((r(3)-rb2)**2.0d0)/b2)
dmsdr(4,4)=(-2.0d0*(r(4)-rb1)/b1-1/a1)*Exp(-(r(4)-ra1)/a1-((r(4)-rb1)**2.0d0)/b1)
dmsdr(5,5)=(-2.0d0*(r(5)-rb2)/b2-1/a2)*Exp(-(r(5)-ra2)/a2-((r(5)-rb2)**2.0d0)/b2)
dmsdr(6,6)=(-2.0d0*(r(6)-rb2)/b2-1/a2)*Exp(-(r(6)-ra2)/a2-((r(6)-rb2)**2.0d0)/b2)

rm(0) = 1.0D0
rm(1) = x(4)
rm(2) = x(2)
rm(3) = x(1)
rm(4) = x(6)
rm(5) = x(5)
rm(6) = x(3)
rm(7) = rm(1)*rm(2)
rm(8) = rm(1)*rm(3)
rm(9) = rm(2)*rm(3)
rm(10) = rm(3)*rm(4)
rm(11) = rm(2)*rm(5)
rm(12) = rm(1)*rm(6)
rm(13) = rm(4)*rm(5)
rm(14) = rm(4)*rm(6)
rm(15) = rm(5)*rm(6)
rm(16) = rm(1)*rm(9)
rm(17) = rm(1)*rm(10)
rm(18) = rm(2)*rm(10)
rm(19) = rm(1)*rm(11)
rm(20) = rm(3)*rm(11)
rm(21) = rm(2)*rm(12)
rm(22) = rm(3)*rm(12)
rm(23) = rm(2)*rm(13)
rm(24) = rm(3)*rm(13)
rm(25) = rm(1)*rm(14)
rm(26) = rm(3)*rm(14)
rm(27) = rm(1)*rm(15)
rm(28) = rm(2)*rm(15)
rm(29) = rm(4)*rm(15)
rm(30) = rm(2)*rm(24)
rm(31) = rm(1)*rm(26)
rm(32) = rm(1)*rm(28)

p(0) = rm(0)
p(1) = rm(1) + rm(2) + rm(3)
p(2) = rm(4) + rm(5) + rm(6)
p(3) = rm(7) + rm(8) + rm(9)
p(4) = rm(10) + rm(11) + rm(12)
p(5) = p(1)*p(2) - p(4)
p(6) = rm(13) + rm(14) + rm(15)
p(7) = p(1)*p(1) - p(3) - p(3)
p(8) = p(2)*p(2) - p(6) - p(6)
p(9) = rm(16)
p(10) = rm(17) + rm(18) + rm(19) + rm(20) + rm(21) + rm(22)
p(11) = p(2)*p(3) - p(10)
p(12) = rm(23) + rm(24) + rm(25) + rm(26) + rm(27) + rm(28)
p(13) = p(1)*p(6) - p(12)
p(14) = rm(29)
p(15) = p(1)*p(3) - p(9) - p(9) - p(9)
p(16) = p(1)*p(4) - p(10)
p(17) = p(2)*p(7) - p(16)
p(18) = p(2)*p(4) - p(12)
p(19) = p(1)*p(8) - p(18)
p(20) = p(2)*p(6) - p(14) - p(14) - p(14)
p(21) = p(1)*p(7) - p(15)
p(22) = p(2)*p(8) - p(20)
p(23) = p(9)*p(2)
p(24) = rm(30) + rm(31) + rm(32)
p(25) = p(3)*p(6) - p(24)
p(26) = p(14)*p(1)
p(27) = p(9)*p(1)
p(28) = p(3)*p(4) - p(23)
p(29) = p(1)*p(10) - p(23) - p(28) - p(23)
p(30) = p(1)*p(11) - p(23)
p(31) = p(1)*p(12) - p(25) - p(24) - p(24)
p(32) = p(1)*p(13) - p(25)
p(33) = p(4)*p(5) - p(25) - p(31)
p(34) = p(2)*p(11) - p(25)
p(35) = p(4)*p(6) - p(26)
p(36) = p(2)*p(12) - p(26) - p(35) - p(26)
p(37) = p(2)*p(13) - p(26)
p(38) = p(14)*p(2)
p(39) = p(3)*p(3) - p(27) - p(27)
p(40) = p(3)*p(7) - p(27)
p(41) = p(1)*p(16) - p(28)
p(42) = p(2)*p(21) - p(41)
p(43) = p(1)*p(18) - p(33)
p(44) = p(7)*p(8) - p(43)
p(45) = p(6)*p(6) - p(38) - p(38)
p(46) = p(2)*p(18) - p(35)
p(47) = p(1)*p(22) - p(46)
p(48) = p(6)*p(8) - p(38)
p(49) = p(1)*p(21) - p(40)
p(50) = p(2)*p(22) - p(48)
p(51) = p(9)*p(6)
p(52) = p(14)*p(3)
p(53) = p(9)*p(4)
p(54) = p(9)*p(5)
p(55) = p(1)*p(24) - p(51)
p(56) = p(3)*p(12) - p(51) - p(55) - p(51)
p(57) = p(3)*p(13) - p(51)
p(58) = p(14)*p(7)
p(59) = p(9)*p(8)
p(60) = p(2)*p(24) - p(52)
p(61) = p(4)*p(13) - p(58)
p(62) = p(6)*p(11) - p(52)
p(63) = p(14)*p(4)
p(64) = p(14)*p(5)
p(65) = p(9)*p(3)
p(66) = p(9)*p(7)
p(67) = p(3)*p(16) - p(53)
p(68) = p(4)*p(15) - p(54) - p(67)
p(69) = p(2)*p(39) - p(68)
p(70) = p(1)*p(29) - p(54) - p(68)
p(71) = p(7)*p(11) - p(53)
p(72) = p(1)*p(31) - p(56) - p(55)
p(73) = p(1)*p(32) - p(57)
p(74) = p(3)*p(18) - p(59)
p(75) = p(2)*p(29) - p(57) - p(55)
p(76) = p(1)*p(34) - p(59)
p(77) = p(6)*p(16) - p(58)
p(78) = p(1)*p(36) - p(62) - p(60)
p(79) = p(2)*p(32) - p(58)
p(80) = p(6)*p(12) - p(64) - p(63) - p(63)
p(81) = p(1)*p(45) - p(80)
p(82) = p(14)*p(6)
p(83) = p(2)*p(33) - p(61) - p(60)
p(84) = p(2)*p(34) - p(62)
p(85) = p(6)*p(18) - p(63)
p(86) = p(2)*p(36) - p(64) - p(80)
p(87) = p(8)*p(13) - p(63)
p(88) = p(14)*p(8)
p(89) = p(1)*p(39) - p(65)
p(90) = p(3)*p(21) - p(66)
p(91) = p(1)*p(41) - p(67)
p(92) = p(2)*p(49) - p(91)
p(93) = p(1)*p(43) - p(74)
p(94) = p(8)*p(21) - p(93)
p(95) = p(1)*p(46) - p(83)
p(96) = p(7)*p(22) - p(95)
p(97) = p(2)*p(45) - p(82)
p(98) = p(2)*p(46) - p(85)
p(99) = p(1)*p(50) - p(98)
p(100) = p(6)*p(22) - p(88)
p(101) = p(1)*p(49) - p(90)
p(102) = p(2)*p(50) - p(100)
p(103) = p(9)*p(14)
p(104) = p(9)*p(12)
p(105) = p(9)*p(13)
p(106) = p(14)*p(15)
p(107) = p(9)*p(20)
p(108) = p(14)*p(10)
p(109) = p(14)*p(11)
p(110) = p(9)*p(16)
p(111) = p(9)*p(10)
p(112) = p(9)*p(11)
p(113) = p(9)*p(17)
p(114) = p(3)*p(24) - p(104)
p(115) = p(7)*p(24) - p(105)
p(116) = p(3)*p(31) - p(104) - p(115)
p(117) = p(6)*p(39) - p(114)
p(118) = p(3)*p(32) - p(105)
p(119) = p(14)*p(21)
p(120) = p(9)*p(18)
p(121) = p(9)*p(19)
p(122) = p(4)*p(24) - p(103) - p(103) - p(103)
p(123) = p(1)*p(60) - p(107) - p(122)
p(124) = p(13)*p(16) - p(119)
p(125) = p(2)*p(56) - p(106) - p(124)
p(126) = p(4)*p(32) - p(119)
p(127) = p(1)*p(62) - p(107) - p(125)
p(128) = p(14)*p(16)
p(129) = p(14)*p(17)
p(130) = p(6)*p(24) - p(108)
p(131) = p(3)*p(45) - p(130)
p(132) = p(14)*p(12)
p(133) = p(14)*p(13)
p(134) = p(9)*p(22)
p(135) = p(8)*p(24) - p(109)
p(136) = p(13)*p(18) - p(128)
p(137) = p(6)*p(34) - p(109)
p(138) = p(14)*p(18)
p(139) = p(14)*p(19)
p(140) = p(9)*p(9)
p(141) = p(9)*p(15)
p(142) = p(9)*p(21)
p(143) = p(3)*p(41) - p(110)
p(144) = p(4)*p(39) - p(112)
p(145) = p(1)*p(68) - p(111) - p(144)
p(146) = p(1)*p(69) - p(112)
p(147) = p(1)*p(70) - p(113) - p(145)
p(148) = p(11)*p(21) - p(110)
p(149) = p(1)*p(72) - p(116) - p(115)
p(150) = p(1)*p(73) - p(118)
p(151) = p(3)*p(43) - p(120)
p(152) = p(15)*p(18) - p(121) - p(151)
p(153) = p(2)*p(69) - p(117)
p(154) = p(1)*p(75) - p(121) - p(152)
p(155) = p(7)*p(34) - p(120)
p(156) = p(6)*p(41) - p(119)
p(157) = p(1)*p(78) - p(125) - p(123)
p(158) = p(2)*p(73) - p(119)
p(159) = p(1)*p(80) - p(131) - p(130) - p(130)
p(160) = p(1)*p(81) - p(131)
p(161) = p(14)*p(14)
p(162) = p(3)*p(46) - p(134)
p(163) = p(2)*p(75) - p(126) - p(123)
p(164) = p(1)*p(84) - p(134)
p(165) = p(6)*p(43) - p(128)
p(166) = p(1)*p(86) - p(137) - p(135)
p(167) = p(8)*p(32) - p(128)
p(168) = p(4)*p(45) - p(133)
p(169) = p(2)*p(80) - p(132) - p(168)
p(170) = p(2)*p(81) - p(133)
p(171) = p(14)*p(20)
p(172) = p(2)*p(83) - p(136) - p(135)
p(173) = p(2)*p(84) - p(137)
p(174) = p(6)*p(46) - p(138)
p(175) = p(2)*p(86) - p(139) - p(169)
p(176) = p(13)*p(22) - p(138)
p(177) = p(14)*p(22)
p(178) = p(3)*p(39) - p(141)
p(179) = p(1)*p(89) - p(141) - p(178) - p(178)
p(180) = p(3)*p(49) - p(142)
p(181) = p(1)*p(91) - p(143)
p(182) = p(2)*p(101) - p(181)
p(183) = p(1)*p(93) - p(151)
p(184) = p(8)*p(49) - p(183)
p(185) = p(1)*p(95) - p(162)
p(186) = p(21)*p(22) - p(185)
p(187) = p(6)*p(45) - p(171)
p(188) = p(1)*p(98) - p(172)
p(189) = p(7)*p(50) - p(188)
p(190) = p(2)*p(97) - p(171) - p(187) - p(187)
p(191) = p(2)*p(98) - p(174)
p(192) = p(1)*p(102) - p(191)
p(193) = p(6)*p(50) - p(177)
p(194) = p(1)*p(101) - p(180)
p(195) = p(2)*p(102) - p(193)
p(196) = p(9)*p(26)
p(197) = p(9)*p(38)
p(198) = p(9)*p(24)
p(199) = p(9)*p(31)
p(200) = p(9)*p(25)
p(201) = p(9)*p(32)
p(202) = p(14)*p(39)
p(203) = p(14)*p(40)
p(204) = p(9)*p(35)
p(205) = p(9)*p(36)
p(206) = p(9)*p(37)
p(207) = p(14)*p(28)
p(208) = p(14)*p(29)
p(209) = p(14)*p(30)
p(210) = p(9)*p(45)
p(211) = p(14)*p(24)
p(212) = p(14)*p(25)
p(213) = p(9)*p(48)
p(214) = p(14)*p(33)
p(215) = p(14)*p(34)
p(216) = p(9)*p(41)
p(217) = p(9)*p(28)
p(218) = p(9)*p(23)
p(219) = p(9)*p(29)
p(220) = p(9)*p(30)
p(221) = p(9)*p(42)
p(222) = p(1)*p(114) - p(198)
p(223) = p(21)*p(24) - p(201)
p(224) = p(3)*p(72) - p(199) - p(223)
p(225) = p(1)*p(116) - p(199) - p(224)
p(226) = p(13)*p(39) - p(198)
p(227) = p(3)*p(73) - p(201)
p(228) = p(14)*p(49)
p(229) = p(9)*p(43)
p(230) = p(9)*p(33)
p(231) = p(9)*p(34)
p(232) = p(9)*p(44)
p(233) = p(16)*p(24) - p(196)
p(234) = p(2)*p(114) - p(202)
p(235) = p(1)*p(123) - p(205) - p(234)
p(236) = p(13)*p(41) - p(228)
p(237) = p(2)*p(116) - p(203) - p(236)
p(238) = p(16)*p(32) - p(228)
p(239) = p(6)*p(69) - p(202)
p(240) = p(4)*p(73) - p(228)
p(241) = p(11)*p(32) - p(196)
p(242) = p(14)*p(41)
p(243) = p(14)*p(42)
p(244) = p(1)*p(130) - p(210)
p(245) = p(4)*p(62) - p(213) - p(209)
p(246) = p(3)*p(81) - p(210)
p(247) = p(14)*p(31)
p(248) = p(14)*p(32)
p(249) = p(14)*p(26)
p(250) = p(9)*p(46)
p(251) = p(9)*p(47)
p(252) = p(18)*p(24) - p(197)
p(253) = p(1)*p(135) - p(213) - p(252)
p(254) = p(13)*p(43) - p(242)
p(255) = p(2)*p(125) - p(209) - p(245)
p(256) = p(18)*p(32) - p(242)
p(257) = p(13)*p(34) - p(197)
p(258) = p(14)*p(43)
p(259) = p(14)*p(44)
p(260) = p(2)*p(130) - p(211)
p(261) = p(4)*p(81) - p(248)
p(262) = p(11)*p(45) - p(211)
p(263) = p(14)*p(35)
p(264) = p(14)*p(36)
p(265) = p(14)*p(37)
p(266) = p(9)*p(50)
p(267) = p(22)*p(24) - p(215)
p(268) = p(13)*p(46) - p(258)
p(269) = p(6)*p(84) - p(215)
p(270) = p(14)*p(46)
p(271) = p(14)*p(47)
p(272) = p(9)*p(27)
p(273) = p(9)*p(39)
p(274) = p(9)*p(40)
p(275) = p(9)*p(49)
p(276) = p(3)*p(91) - p(216)
p(277) = p(16)*p(39) - p(218)
p(278) = p(1)*p(144) - p(217) - p(277)
p(279) = p(2)*p(178) - p(278)
p(280) = p(1)*p(145) - p(219) - p(278)
p(281) = p(7)*p(69) - p(218)
p(282) = p(1)*p(147) - p(221) - p(280)
p(283) = p(11)*p(49) - p(216)
p(284) = p(1)*p(149) - p(224) - p(223)
p(285) = p(1)*p(150) - p(227)
p(286) = p(3)*p(93) - p(229)
p(287) = p(18)*p(39) - p(231)
p(288) = p(1)*p(152) - p(230) - p(287)
p(289) = p(1)*p(153) - p(231)
p(290) = p(1)*p(154) - p(232) - p(288)
p(291) = p(21)*p(34) - p(229)
p(292) = p(6)*p(91) - p(228)
p(293) = p(1)*p(157) - p(237) - p(235)
p(294) = p(2)*p(150) - p(228)
p(295) = p(1)*p(159) - p(245) - p(244)
p(296) = p(1)*p(160) - p(246)
p(297) = p(3)*p(95) - p(250)
p(298) = p(2)*p(152) - p(238) - p(234)
p(299) = p(2)*p(153) - p(239)
p(300) = p(1)*p(163) - p(251) - p(298)
p(301) = p(7)*p(84) - p(250)
p(302) = p(6)*p(93) - p(242)
p(303) = p(1)*p(166) - p(255) - p(253)
p(304) = p(8)*p(73) - p(242)
p(305) = p(16)*p(45) - p(248)
p(306) = p(1)*p(169) - p(262) - p(260)
p(307) = p(2)*p(160) - p(248)
p(308) = p(14)*p(38)
p(309) = p(6)*p(80) - p(264) - p(263)
p(310) = p(1)*p(187) - p(309)
p(311) = p(14)*p(45)
p(312) = p(3)*p(98) - p(266)
p(313) = p(2)*p(163) - p(256) - p(253)
p(314) = p(1)*p(173) - p(266)
p(315) = p(6)*p(95) - p(258)
p(316) = p(1)*p(175) - p(269) - p(267)
p(317) = p(22)*p(32) - p(258)
p(318) = p(18)*p(45) - p(249)
p(319) = p(2)*p(169) - p(264) - p(309)
p(320) = p(8)*p(81) - p(249)
p(321) = p(14)*p(48)
p(322) = p(2)*p(172) - p(268) - p(267)
p(323) = p(2)*p(173) - p(269)
p(324) = p(6)*p(98) - p(270)
p(325) = p(2)*p(175) - p(271) - p(319)
p(326) = p(13)*p(50) - p(270)
p(327) = p(14)*p(50)
p(328) = p(1)*p(178) - p(273)
p(329) = p(21)*p(39) - p(272)
p(330) = p(3)*p(101) - p(275)
p(331) = p(1)*p(181) - p(276)
p(332) = p(2)*p(194) - p(331)
p(333) = p(1)*p(183) - p(286)
p(334) = p(8)*p(101) - p(333)
p(335) = p(1)*p(185) - p(297)
p(336) = p(22)*p(49) - p(335)
p(337) = p(1)*p(188) - p(312)
p(338) = p(21)*p(50) - p(337)
p(339) = p(2)*p(187) - p(311)
p(340) = p(1)*p(191) - p(322)
p(341) = p(7)*p(102) - p(340)
p(342) = p(22)*p(45) - p(308)
p(343) = p(2)*p(191) - p(324)
p(344) = p(1)*p(195) - p(343)
p(345) = p(6)*p(102) - p(327)
p(346) = p(1)*p(194) - p(330)
p(347) = p(2)*p(195) - p(345)
p(348) = p(9)*p(52)
p(349) = p(9)*p(58)
p(350) = p(9)*p(63)
p(351) = p(9)*p(64)
p(352) = p(9)*p(82)
p(353) = p(9)*p(88)
p(354) = p(9)*p(55)
p(355) = p(9)*p(72)
p(356) = p(9)*p(51)
p(357) = p(9)*p(56)
p(358) = p(9)*p(57)
p(359) = p(9)*p(73)
p(360) = p(14)*p(89)
p(361) = p(14)*p(90)
p(362) = p(9)*p(77)
p(363) = p(9)*p(60)
p(364) = p(9)*p(78)
p(365) = p(9)*p(61)
p(366) = p(9)*p(62)
p(367) = p(9)*p(79)
p(368) = p(14)*p(67)
p(369) = p(14)*p(68)
p(370) = p(14)*p(69)
p(371) = p(14)*p(70)
p(372) = p(14)*p(71)
p(373) = p(9)*p(80)
p(374) = p(9)*p(81)
p(375) = p(14)*p(55)
p(376) = p(14)*p(56)
p(377) = p(14)*p(57)
p(378) = p(14)*p(52)
p(379) = p(9)*p(85)
p(380) = p(9)*p(86)
p(381) = p(9)*p(87)
p(382) = p(14)*p(74)
p(383) = p(14)*p(75)
p(384) = p(14)*p(76)
p(385) = p(9)*p(97)
p(386) = p(14)*p(60)
p(387) = p(14)*p(61)
p(388) = p(14)*p(62)
p(389) = p(9)*p(100)
p(390) = p(14)*p(83)
p(391) = p(14)*p(84)
p(392) = p(9)*p(91)
p(393) = p(9)*p(67)
p(394) = p(9)*p(53)
p(395) = p(9)*p(68)
p(396) = p(9)*p(54)
p(397) = p(9)*p(69)
p(398) = p(9)*p(70)
p(399) = p(9)*p(71)
p(400) = p(9)*p(92)
p(401) = p(3)*p(114) - p(354)
p(402) = p(7)*p(114) - p(356)
p(403) = p(24)*p(49) - p(359)
p(404) = p(3)*p(149) - p(355) - p(403)
p(405) = p(1)*p(224) - p(355) - p(404)
p(406) = p(6)*p(178) - p(401)
p(407) = p(32)*p(39) - p(356)
p(408) = p(3)*p(150) - p(359)
p(409) = p(14)*p(101)
p(410) = p(9)*p(93)
p(411) = p(9)*p(74)
p(412) = p(9)*p(59)
p(413) = p(9)*p(75)
p(414) = p(9)*p(76)
p(415) = p(9)*p(94)
p(416) = p(24)*p(41) - p(349)
p(417) = p(4)*p(114) - p(348)
p(418) = p(1)*p(234) - p(363) - p(417)
p(419) = p(1)*p(235) - p(364) - p(418)
p(420) = p(13)*p(91) - p(409)
p(421) = p(2)*p(224) - p(361) - p(420)
p(422) = p(32)*p(41) - p(409)
p(423) = p(1)*p(237) - p(364) - p(421)
p(424) = p(16)*p(73) - p(409)
p(425) = p(13)*p(69) - p(348)
p(426) = p(4)*p(150) - p(409)
p(427) = p(11)*p(73) - p(349)
p(428) = p(14)*p(91)
p(429) = p(14)*p(92)
p(430) = p(3)*p(130) - p(373)
p(431) = p(7)*p(130) - p(374)
p(432) = p(3)*p(159) - p(373) - p(431)
p(433) = p(39)*p(45) - p(430)
p(434) = p(3)*p(160) - p(374)
p(435) = p(14)*p(72)
p(436) = p(14)*p(73)
p(437) = p(14)*p(58)
p(438) = p(9)*p(95)
p(439) = p(9)*p(83)
p(440) = p(9)*p(84)
p(441) = p(9)*p(96)
p(442) = p(24)*p(43) - p(350)
p(443) = p(8)*p(114) - p(370)
p(444) = p(1)*p(253) - p(380) - p(443)
p(445) = p(13)*p(93) - p(428)
p(446) = p(2)*p(237) - p(372) - p(432)
p(447) = p(32)*p(43) - p(428)
p(448) = p(6)*p(153) - p(370)
p(449) = p(18)*p(73) - p(428)
p(450) = p(32)*p(34) - p(350)
p(451) = p(14)*p(93)
p(452) = p(14)*p(94)
p(453) = p(4)*p(130) - p(352)
p(454) = p(1)*p(260) - p(385) - p(453)
p(455) = p(16)*p(81) - p(436)
p(456) = p(2)*p(245) - p(376) - p(455)
p(457) = p(4)*p(160) - p(436)
p(458) = p(11)*p(81) - p(352)
p(459) = p(14)*p(77)
p(460) = p(14)*p(78)
p(461) = p(14)*p(79)
p(462) = p(14)*p(63)
p(463) = p(14)*p(64)
p(464) = p(6)*p(130) - p(386)
p(465) = p(3)*p(187) - p(464)
p(466) = p(14)*p(80)
p(467) = p(14)*p(81)
p(468) = p(9)*p(98)
p(469) = p(9)*p(99)
p(470) = p(24)*p(46) - p(353)
p(471) = p(1)*p(267) - p(389) - p(470)
p(472) = p(13)*p(95) - p(451)
p(473) = p(2)*p(255) - p(384) - p(456)
p(474) = p(32)*p(46) - p(451)
p(475) = p(13)*p(84) - p(353)
p(476) = p(14)*p(95)
p(477) = p(14)*p(96)
p(478) = p(8)*p(130) - p(378)
p(479) = p(18)*p(81) - p(437)
p(480) = p(34)*p(45) - p(378)
p(481) = p(14)*p(85)
p(482) = p(14)*p(86)
p(483) = p(14)*p(87)
p(484) = p(9)*p(102)
p(485) = p(24)*p(50) - p(391)
p(486) = p(13)*p(98) - p(476)
p(487) = p(6)*p(173) - p(391)
p(488) = p(14)*p(98)
p(489) = p(14)*p(99)
p(490) = p(9)*p(65)
p(491) = p(9)*p(66)
p(492) = p(9)*p(89)
p(493) = p(9)*p(90)
p(494) = p(9)*p(101)
p(495) = p(3)*p(181) - p(392)
p(496) = p(39)*p(41) - p(394)
p(497) = p(4)*p(178) - p(397)
p(498) = p(1)*p(278) - p(395) - p(497)
p(499) = p(1)*p(279) - p(397)
p(500) = p(1)*p(280) - p(398) - p(498)
p(501) = p(21)*p(69) - p(394)
p(502) = p(1)*p(282) - p(400) - p(500)
p(503) = p(11)*p(101) - p(392)
p(504) = p(1)*p(284) - p(404) - p(403)
p(505) = p(1)*p(285) - p(408)
p(506) = p(3)*p(183) - p(410)
p(507) = p(39)*p(43) - p(412)
p(508) = p(1)*p(287) - p(411) - p(507)
p(509) = p(2)*p(279) - p(406)
p(510) = p(1)*p(288) - p(413) - p(508)
p(511) = p(7)*p(153) - p(412)
p(512) = p(1)*p(290) - p(415) - p(510)
p(513) = p(34)*p(49) - p(410)
p(514) = p(6)*p(181) - p(409)
p(515) = p(1)*p(293) - p(421) - p(419)
p(516) = p(2)*p(285) - p(409)
p(517) = p(1)*p(295) - p(432) - p(431)
p(518) = p(1)*p(296) - p(434)
p(519) = p(3)*p(185) - p(438)
p(520) = p(39)*p(46) - p(440)
p(521) = p(1)*p(298) - p(439) - p(520)
p(522) = p(1)*p(299) - p(440)
p(523) = p(1)*p(300) - p(441) - p(521)
p(524) = p(21)*p(84) - p(438)
p(525) = p(6)*p(183) - p(428)
p(526) = p(1)*p(303) - p(446) - p(444)
p(527) = p(8)*p(150) - p(428)
p(528) = p(41)*p(45) - p(436)
p(529) = p(1)*p(306) - p(456) - p(454)
p(530) = p(2)*p(296) - p(436)
p(531) = p(4)*p(169) - p(388) - p(478)
p(532) = p(1)*p(310) - p(465)
p(533) = p(14)*p(82)
p(534) = p(3)*p(188) - p(468)
p(535) = p(2)*p(298) - p(447) - p(443)
p(536) = p(2)*p(299) - p(448)
p(537) = p(1)*p(313) - p(469) - p(535)
p(538) = p(7)*p(173) - p(468)
p(539) = p(6)*p(185) - p(451)
p(540) = p(1)*p(316) - p(473) - p(471)
p(541) = p(22)*p(73) - p(451)
p(542) = p(43)*p(45) - p(437)
p(543) = p(1)*p(319) - p(480) - p(478)
p(544) = p(8)*p(160) - p(437)
p(545) = p(14)*p(88)
p(546) = p(4)*p(187) - p(467)
p(547) = p(2)*p(309) - p(466) - p(546)
p(548) = p(2)*p(310) - p(467)
p(549) = p(14)*p(97)
p(550) = p(3)*p(191) - p(484)
p(551) = p(2)*p(313) - p(474) - p(471)
p(552) = p(1)*p(323) - p(484)
p(553) = p(6)*p(188) - p(476)
p(554) = p(1)*p(325) - p(487) - p(485)
p(555) = p(32)*p(50) - p(476)
p(556) = p(45)*p(46) - p(462)
p(557) = p(2)*p(319) - p(482) - p(547)
p(558) = p(22)*p(81) - p(462)
p(559) = p(14)*p(100)
p(560) = p(2)*p(322) - p(486) - p(485)
p(561) = p(2)*p(323) - p(487)
p(562) = p(6)*p(191) - p(488)
p(563) = p(2)*p(325) - p(489) - p(557)
p(564) = p(13)*p(102) - p(488)
p(565) = p(14)*p(102)
p(566) = p(3)*p(178) - p(492)
p(567) = p(7)*p(178) - p(490)
p(568) = p(39)*p(49) - p(491)
p(569) = p(3)*p(194) - p(494)
p(570) = p(1)*p(331) - p(495)
p(571) = p(2)*p(346) - p(570)
p(572) = p(1)*p(333) - p(506)
p(573) = p(8)*p(194) - p(572)
p(574) = p(1)*p(335) - p(519)
p(575) = p(22)*p(101) - p(574)
p(576) = p(1)*p(337) - p(534)
p(577) = p(49)*p(50) - p(576)
p(578) = p(6)*p(187) - p(549)
p(579) = p(1)*p(340) - p(550)
p(580) = p(21)*p(102) - p(579)
p(581) = p(8)*p(187) - p(533)
p(582) = p(1)*p(343) - p(560)
p(583) = p(7)*p(195) - p(582)
p(584) = p(45)*p(50) - p(545)
p(585) = p(2)*p(343) - p(562)
p(586) = p(1)*p(347) - p(585)
p(587) = p(6)*p(195) - p(565)
p(588) = p(1)*p(346) - p(569)
p(589) = p(2)*p(347) - p(587)
p(590) = p(9)*p(103)
p(591) = p(9)*p(106)
p(592) = p(9)*p(119)
p(593) = p(9)*p(128)
p(594) = p(9)*p(108)
p(595) = p(9)*p(109)
p(596) = p(9)*p(129)
p(597) = p(9)*p(132)
p(598) = p(9)*p(133)
p(599) = p(9)*p(161)
p(600) = p(9)*p(138)
p(601) = p(9)*p(139)
p(602) = p(9)*p(171)
p(603) = p(9)*p(177)
p(604) = p(9)*p(114)
p(605) = p(9)*p(115)
p(606) = p(9)*p(149)
p(607) = p(9)*p(104)
p(608) = p(9)*p(116)
p(609) = p(9)*p(105)
p(610) = p(9)*p(117)
p(611) = p(9)*p(118)
p(612) = p(9)*p(150)
p(613) = p(14)*p(178)
p(614) = p(14)*p(179)
p(615) = p(14)*p(180)
p(616) = p(9)*p(156)
p(617) = p(9)*p(122)
p(618) = p(9)*p(123)
p(619) = p(9)*p(157)
p(620) = p(9)*p(124)
p(621) = p(9)*p(107)
p(622) = p(9)*p(125)
p(623) = p(9)*p(126)
p(624) = p(9)*p(127)
p(625) = p(9)*p(158)
p(626) = p(14)*p(143)
p(627) = p(14)*p(144)
p(628) = p(14)*p(145)
p(629) = p(14)*p(146)
p(630) = p(14)*p(147)
p(631) = p(14)*p(148)
p(632) = p(9)*p(130)
p(633) = p(9)*p(159)
p(634) = p(9)*p(131)
p(635) = p(9)*p(160)
p(636) = p(14)*p(114)
p(637) = p(14)*p(115)
p(638) = p(14)*p(116)
p(639) = p(14)*p(117)
p(640) = p(14)*p(118)
p(641) = p(14)*p(106)
p(642) = p(9)*p(165)
p(643) = p(9)*p(135)
p(644) = p(9)*p(166)
p(645) = p(9)*p(136)
p(646) = p(9)*p(137)
p(647) = p(9)*p(167)
p(648) = p(14)*p(151)
p(649) = p(14)*p(152)
p(650) = p(14)*p(153)
p(651) = p(14)*p(154)
p(652) = p(14)*p(155)
p(653) = p(9)*p(168)
p(654) = p(9)*p(169)
p(655) = p(9)*p(170)
p(656) = p(14)*p(122)
p(657) = p(14)*p(123)
p(658) = p(14)*p(124)
p(659) = p(14)*p(125)
p(660) = p(14)*p(126)
p(661) = p(14)*p(127)
p(662) = p(14)*p(108)
p(663) = p(14)*p(109)
p(664) = p(9)*p(187)
p(665) = p(14)*p(130)
p(666) = p(14)*p(131)
p(667) = p(9)*p(174)
p(668) = p(9)*p(175)
p(669) = p(9)*p(176)
p(670) = p(14)*p(162)
p(671) = p(14)*p(163)
p(672) = p(14)*p(164)
p(673) = p(9)*p(190)
p(674) = p(14)*p(135)
p(675) = p(14)*p(136)
p(676) = p(14)*p(137)
p(677) = p(9)*p(193)
p(678) = p(14)*p(172)
p(679) = p(14)*p(173)
p(680) = p(9)*p(181)
p(681) = p(9)*p(143)
p(682) = p(9)*p(110)
p(683) = p(9)*p(144)
p(684) = p(9)*p(111)
p(685) = p(9)*p(112)
p(686) = p(9)*p(145)
p(687) = p(9)*p(113)
p(688) = p(9)*p(146)
p(689) = p(9)*p(147)
p(690) = p(9)*p(148)
p(691) = p(9)*p(182)
p(692) = p(1)*p(401) - p(604)
p(693) = p(21)*p(114) - p(609)
p(694) = p(24)*p(101) - p(612)
p(695) = p(3)*p(284) - p(606) - p(694)
p(696) = p(1)*p(404) - p(606) - p(695)
p(697) = p(1)*p(405) - p(608) - p(696)
p(698) = p(13)*p(178) - p(604)
p(699) = p(39)*p(73) - p(609)
p(700) = p(3)*p(285) - p(612)
p(701) = p(14)*p(194)
p(702) = p(9)*p(183)
p(703) = p(9)*p(151)
p(704) = p(9)*p(120)
p(705) = p(9)*p(152)
p(706) = p(9)*p(121)
p(707) = p(9)*p(153)
p(708) = p(9)*p(154)
p(709) = p(9)*p(155)
p(710) = p(9)*p(184)
p(711) = p(24)*p(91) - p(592)
p(712) = p(1)*p(416) - p(616) - p(711)
p(713) = p(2)*p(401) - p(613)
p(714) = p(1)*p(418) - p(618) - p(713)
p(715) = p(1)*p(419) - p(619) - p(714)
p(716) = p(13)*p(181) - p(701)
p(717) = p(2)*p(404) - p(615) - p(716)
p(718) = p(32)*p(91) - p(701)
p(719) = p(1)*p(421) - p(619) - p(717)
p(720) = p(41)*p(73) - p(701)
p(721) = p(6)*p(279) - p(613)
p(722) = p(16)*p(150) - p(701)
p(723) = p(1)*p(425) - p(624) - p(721)
p(724) = p(4)*p(285) - p(701)
p(725) = p(11)*p(150) - p(592)
p(726) = p(14)*p(181)
p(727) = p(14)*p(182)
p(728) = p(1)*p(430) - p(632)
p(729) = p(21)*p(130) - p(635)
p(730) = p(3)*p(295) - p(633) - p(729)
p(731) = p(1)*p(432) - p(633) - p(730)
p(732) = p(39)*p(81) - p(632)
p(733) = p(3)*p(296) - p(635)
p(734) = p(14)*p(149)
p(735) = p(14)*p(150)
p(736) = p(14)*p(119)
p(737) = p(9)*p(185)
p(738) = p(9)*p(162)
p(739) = p(9)*p(134)
p(740) = p(9)*p(163)
p(741) = p(9)*p(164)
p(742) = p(9)*p(186)
p(743) = p(24)*p(93) - p(593)
p(744) = p(18)*p(114) - p(595)
p(745) = p(1)*p(443) - p(643) - p(744)
p(746) = p(1)*p(444) - p(644) - p(745)
p(747) = p(13)*p(183) - p(726)
p(748) = p(2)*p(421) - p(631) - p(730)
p(749) = p(32)*p(93) - p(726)
p(750) = p(1)*p(446) - p(644) - p(748)
p(751) = p(43)*p(73) - p(726)
p(752) = p(13)*p(153) - p(595)
p(753) = p(18)*p(150) - p(726)
p(754) = p(34)*p(73) - p(593)
p(755) = p(14)*p(183)
p(756) = p(14)*p(184)
p(757) = p(16)*p(130) - p(598)
p(758) = p(2)*p(430) - p(636)
p(759) = p(1)*p(454) - p(654) - p(758)
p(760) = p(41)*p(81) - p(735)
p(761) = p(2)*p(432) - p(638) - p(760)
p(762) = p(16)*p(160) - p(735)
p(763) = p(45)*p(69) - p(636)
p(764) = p(4)*p(296) - p(735)
p(765) = p(11)*p(160) - p(598)
p(766) = p(14)*p(156)
p(767) = p(14)*p(157)
p(768) = p(14)*p(158)
p(769) = p(14)*p(128)
p(770) = p(14)*p(129)
p(771) = p(1)*p(464) - p(664)
p(772) = p(4)*p(262) - p(673) - p(661)
p(773) = p(3)*p(310) - p(664)
p(774) = p(14)*p(159)
p(775) = p(14)*p(160)
p(776) = p(14)*p(132)
p(777) = p(14)*p(133)
p(778) = p(9)*p(188)
p(779) = p(9)*p(172)
p(780) = p(9)*p(173)
p(781) = p(9)*p(189)
p(782) = p(24)*p(95) - p(600)
p(783) = p(22)*p(114) - p(650)
p(784) = p(1)*p(471) - p(668) - p(783)
p(785) = p(13)*p(185) - p(755)
p(786) = p(2)*p(446) - p(652) - p(761)
p(787) = p(32)*p(95) - p(755)
p(788) = p(6)*p(299) - p(650)
p(789) = p(46)*p(73) - p(755)
p(790) = p(32)*p(84) - p(600)
p(791) = p(14)*p(185)
p(792) = p(14)*p(186)
p(793) = p(2)*p(453) - p(656) - p(771)
p(794) = p(1)*p(478) - p(673) - p(793)
p(795) = p(43)*p(81) - p(736)
p(796) = p(2)*p(456) - p(659) - p(772)
p(797) = p(18)*p(160) - p(736)
p(798) = p(1)*p(480) - p(673) - p(796)
p(799) = p(14)*p(165)
p(800) = p(14)*p(166)
p(801) = p(14)*p(167)
p(802) = p(14)*p(138)
p(803) = p(14)*p(139)
p(804) = p(2)*p(464) - p(665)
p(805) = p(4)*p(310) - p(775)
p(806) = p(11)*p(187) - p(665)
p(807) = p(14)*p(168)
p(808) = p(14)*p(169)
p(809) = p(14)*p(170)
p(810) = p(9)*p(191)
p(811) = p(9)*p(192)
p(812) = p(24)*p(98) - p(603)
p(813) = p(1)*p(485) - p(677) - p(812)
p(814) = p(13)*p(188) - p(791)
p(815) = p(2)*p(473) - p(672) - p(796)
p(816) = p(32)*p(98) - p(791)
p(817) = p(13)*p(173) - p(603)
p(818) = p(14)*p(188)
p(819) = p(14)*p(189)
p(820) = p(22)*p(130) - p(663)
p(821) = p(46)*p(81) - p(769)
p(822) = p(45)*p(84) - p(663)
p(823) = p(14)*p(174)
p(824) = p(14)*p(175)
p(825) = p(14)*p(176)
p(826) = p(9)*p(195)
p(827) = p(24)*p(102) - p(679)
p(828) = p(13)*p(191) - p(818)
p(829) = p(6)*p(323) - p(679)
p(830) = p(14)*p(191)
p(831) = p(14)*p(192)
p(832) = p(9)*p(140)
p(833) = p(9)*p(141)
p(834) = p(9)*p(178)
p(835) = p(9)*p(142)
p(836) = p(9)*p(179)
p(837) = p(9)*p(180)
p(838) = p(9)*p(194)
p(839) = p(3)*p(331) - p(680)
p(840) = p(39)*p(91) - p(682)
p(841) = p(16)*p(178) - p(685)
p(842) = p(1)*p(497) - p(683) - p(841)
p(843) = p(2)*p(566) - p(842)
p(844) = p(1)*p(498) - p(686) - p(842)
p(845) = p(7)*p(279) - p(685)
p(846) = p(1)*p(500) - p(689) - p(844)
p(847) = p(49)*p(69) - p(682)
p(848) = p(1)*p(502) - p(691) - p(846)
p(849) = p(11)*p(194) - p(680)
p(850) = p(1)*p(504) - p(695) - p(694)
p(851) = p(1)*p(505) - p(700)
p(852) = p(3)*p(333) - p(702)
p(853) = p(39)*p(93) - p(704)
p(854) = p(18)*p(178) - p(707)
p(855) = p(1)*p(508) - p(705) - p(854)
p(856) = p(1)*p(509) - p(707)
p(857) = p(1)*p(510) - p(708) - p(855)
p(858) = p(21)*p(153) - p(704)
p(859) = p(1)*p(512) - p(710) - p(857)
p(860) = p(34)*p(101) - p(702)
p(861) = p(6)*p(331) - p(701)
p(862) = p(1)*p(515) - p(717) - p(715)
p(863) = p(2)*p(505) - p(701)
p(864) = p(1)*p(517) - p(730) - p(729)
p(865) = p(1)*p(518) - p(733)
p(866) = p(3)*p(335) - p(737)
p(867) = p(39)*p(95) - p(739)
p(868) = p(1)*p(520) - p(738) - p(867)
p(869) = p(2)*p(509) - p(721)
p(870) = p(1)*p(521) - p(740) - p(868)
p(871) = p(7)*p(299) - p(739)
p(872) = p(1)*p(523) - p(742) - p(870)
p(873) = p(49)*p(84) - p(737)
p(874) = p(6)*p(333) - p(726)
p(875) = p(1)*p(526) - p(748) - p(746)
p(876) = p(8)*p(285) - p(726)
p(877) = p(45)*p(91) - p(735)
p(878) = p(1)*p(529) - p(761) - p(759)
p(879) = p(2)*p(518) - p(735)
p(880) = p(1)*p(531) - p(772) - p(771)
p(881) = p(1)*p(532) - p(773)
p(882) = p(14)*p(161)
p(883) = p(3)*p(337) - p(778)
p(884) = p(39)*p(98) - p(780)
p(885) = p(1)*p(535) - p(779) - p(884)
p(886) = p(1)*p(536) - p(780)
p(887) = p(1)*p(537) - p(781) - p(885)
p(888) = p(21)*p(173) - p(778)
p(889) = p(6)*p(335) - p(755)
p(890) = p(1)*p(540) - p(786) - p(784)
p(891) = p(22)*p(150) - p(755)
p(892) = p(45)*p(93) - p(736)
p(893) = p(1)*p(543) - p(796) - p(794)
p(894) = p(8)*p(296) - p(736)
p(895) = p(16)*p(187) - p(775)
p(896) = p(1)*p(547) - p(806) - p(804)
p(897) = p(2)*p(532) - p(775)
p(898) = p(14)*p(171)
p(899) = p(6)*p(309) - p(808) - p(807)
p(900) = p(1)*p(578) - p(899)
p(901) = p(14)*p(187)
p(902) = p(3)*p(340) - p(810)
p(903) = p(2)*p(535) - p(787) - p(783)
p(904) = p(2)*p(536) - p(788)
p(905) = p(1)*p(551) - p(811) - p(903)
p(906) = p(7)*p(323) - p(810)
p(907) = p(6)*p(337) - p(791)
p(908) = p(1)*p(554) - p(815) - p(813)
p(909) = p(50)*p(73) - p(791)
p(910) = p(45)*p(95) - p(769)
p(911) = p(1)*p(557) - p(822) - p(820)
p(912) = p(22)*p(160) - p(769)
p(913) = p(14)*p(177)
p(914) = p(18)*p(187) - p(777)
p(915) = p(2)*p(547) - p(808) - p(899)
p(916) = p(8)*p(310) - p(777)
p(917) = p(14)*p(190)
p(918) = p(3)*p(343) - p(826)
p(919) = p(2)*p(551) - p(816) - p(813)
p(920) = p(1)*p(561) - p(826)
p(921) = p(6)*p(340) - p(818)
p(922) = p(1)*p(563) - p(829) - p(827)
p(923) = p(32)*p(102) - p(818)
p(924) = p(45)*p(98) - p(802)
p(925) = p(2)*p(557) - p(824) - p(915)
p(926) = p(50)*p(81) - p(802)
p(927) = p(14)*p(193)
p(928) = p(2)*p(560) - p(828) - p(827)
p(929) = p(2)*p(561) - p(829)
p(930) = p(6)*p(343) - p(830)
p(931) = p(2)*p(563) - p(831) - p(925)
p(932) = p(13)*p(195) - p(830)
p(933) = p(14)*p(195)
p(934) = p(1)*p(566) - p(834)
p(935) = p(1)*p(567) - p(836) - p(934)
p(936) = p(39)*p(101) - p(835)
p(937) = p(3)*p(346) - p(838)
p(938) = p(1)*p(570) - p(839)
p(939) = p(2)*p(588) - p(938)
p(940) = p(1)*p(572) - p(852)
p(941) = p(8)*p(346) - p(940)
p(942) = p(1)*p(574) - p(866)
p(943) = p(22)*p(194) - p(942)
p(944) = p(1)*p(576) - p(883)
p(945) = p(50)*p(101) - p(944)
p(946) = p(1)*p(579) - p(902)
p(947) = p(49)*p(102) - p(946)
p(948) = p(2)*p(578) - p(901)
p(949) = p(1)*p(582) - p(918)
p(950) = p(21)*p(195) - p(949)
p(951) = p(2)*p(581) - p(917) - p(948)
p(952) = p(1)*p(585) - p(928)
p(953) = p(7)*p(347) - p(952)
p(954) = p(45)*p(102) - p(913)
p(955) = p(2)*p(585) - p(930)
p(956) = p(1)*p(589) - p(955)
p(957) = p(6)*p(347) - p(933)
p(958) = p(1)*p(588) - p(937)
p(959) = p(2)*p(589) - p(957)
p(960) = p(9)*p(196)
p(961) = p(9)*p(202)
p(962) = p(9)*p(203)
p(963) = p(9)*p(228)
p(964) = p(9)*p(242)
p(965) = p(9)*p(207)
p(966) = p(9)*p(197)
p(967) = p(9)*p(208)
p(968) = p(9)*p(209)
p(969) = p(9)*p(243)
p(970) = p(9)*p(211)
p(971) = p(9)*p(247)
p(972) = p(9)*p(212)
p(973) = p(9)*p(248)
p(974) = p(9)*p(249)
p(975) = p(9)*p(258)
p(976) = p(9)*p(214)
p(977) = p(9)*p(215)
p(978) = p(9)*p(259)
p(979) = p(9)*p(263)
p(980) = p(9)*p(264)
p(981) = p(9)*p(265)
p(982) = p(9)*p(308)
p(983) = p(9)*p(311)
p(984) = p(9)*p(270)
p(985) = p(9)*p(271)
p(986) = p(9)*p(321)
p(987) = p(9)*p(327)
p(988) = p(9)*p(222)
p(989) = p(9)*p(223)
p(990) = p(9)*p(284)
p(991) = p(9)*p(198)
p(992) = p(9)*p(199)
p(993) = p(9)*p(224)
p(994) = p(9)*p(200)
p(995) = p(9)*p(225)
p(996) = p(9)*p(201)
p(997) = p(9)*p(226)
p(998) = p(9)*p(227)
p(999) = p(9)*p(285)
p(1000) = p(14)*p(328)
p(1001) = p(14)*p(329)
p(1002) = p(14)*p(330)
p(1003) = p(9)*p(292)
p(1004) = p(9)*p(233)
p(1005) = p(9)*p(234)
p(1006) = p(9)*p(235)
p(1007) = p(9)*p(293)
p(1008) = p(9)*p(236)
p(1009) = p(9)*p(204)
p(1010) = p(9)*p(205)
p(1011) = p(9)*p(237)
p(1012) = p(9)*p(238)
p(1013) = p(9)*p(206)
p(1014) = p(9)*p(239)
p(1015) = p(9)*p(240)
p(1016) = p(9)*p(241)
p(1017) = p(9)*p(294)
p(1018) = p(14)*p(276)
p(1019) = p(14)*p(277)
p(1020) = p(14)*p(278)
p(1021) = p(14)*p(279)
p(1022) = p(14)*p(280)
p(1023) = p(14)*p(281)
p(1024) = p(14)*p(282)
p(1025) = p(14)*p(283)
p(1026) = p(9)*p(244)
p(1027) = p(9)*p(295)
p(1028) = p(9)*p(210)
p(1029) = p(9)*p(245)
p(1030) = p(9)*p(246)
p(1031) = p(9)*p(296)
p(1032) = p(14)*p(222)
p(1033) = p(14)*p(223)
p(1034) = p(14)*p(224)
p(1035) = p(14)*p(225)
p(1036) = p(14)*p(226)
p(1037) = p(14)*p(227)
p(1038) = p(14)*p(202)
p(1039) = p(14)*p(203)
p(1040) = p(9)*p(302)
p(1041) = p(9)*p(252)
p(1042) = p(9)*p(253)
p(1043) = p(9)*p(303)
p(1044) = p(9)*p(254)
p(1045) = p(9)*p(213)
p(1046) = p(9)*p(255)
p(1047) = p(9)*p(256)
p(1048) = p(9)*p(257)
p(1049) = p(9)*p(304)
p(1050) = p(14)*p(286)
p(1051) = p(14)*p(287)
p(1052) = p(14)*p(288)
p(1053) = p(14)*p(289)
p(1054) = p(14)*p(290)
p(1055) = p(14)*p(291)
p(1056) = p(9)*p(305)
p(1057) = p(9)*p(260)
p(1058) = p(9)*p(306)
p(1059) = p(9)*p(261)
p(1060) = p(9)*p(262)
p(1061) = p(9)*p(307)
p(1062) = p(14)*p(233)
p(1063) = p(14)*p(234)
p(1064) = p(14)*p(235)
p(1065) = p(14)*p(236)
p(1066) = p(14)*p(237)
p(1067) = p(14)*p(238)
p(1068) = p(14)*p(239)
p(1069) = p(14)*p(240)
p(1070) = p(14)*p(241)
p(1071) = p(14)*p(207)
p(1072) = p(14)*p(208)
p(1073) = p(14)*p(209)
p(1074) = p(9)*p(309)
p(1075) = p(9)*p(310)
p(1076) = p(14)*p(244)
p(1077) = p(14)*p(245)
p(1078) = p(14)*p(246)
p(1079) = p(14)*p(211)
p(1080) = p(14)*p(212)
p(1081) = p(9)*p(315)
p(1082) = p(9)*p(267)
p(1083) = p(9)*p(316)
p(1084) = p(9)*p(268)
p(1085) = p(9)*p(269)
p(1086) = p(9)*p(317)
p(1087) = p(14)*p(297)
p(1088) = p(14)*p(298)
p(1089) = p(14)*p(299)
p(1090) = p(14)*p(300)
p(1091) = p(14)*p(301)
p(1092) = p(9)*p(318)
p(1093) = p(9)*p(319)
p(1094) = p(9)*p(320)
p(1095) = p(14)*p(252)
p(1096) = p(14)*p(253)
p(1097) = p(14)*p(254)
p(1098) = p(14)*p(255)
p(1099) = p(14)*p(256)
p(1100) = p(14)*p(257)
p(1101) = p(14)*p(214)
p(1102) = p(14)*p(215)
p(1103) = p(9)*p(339)
p(1104) = p(14)*p(260)
p(1105) = p(14)*p(261)
p(1106) = p(14)*p(262)
p(1107) = p(9)*p(324)
p(1108) = p(9)*p(325)
p(1109) = p(9)*p(326)
p(1110) = p(14)*p(312)
p(1111) = p(14)*p(313)
p(1112) = p(14)*p(314)
p(1113) = p(9)*p(342)
p(1114) = p(14)*p(267)
p(1115) = p(14)*p(268)
p(1116) = p(14)*p(269)
p(1117) = p(9)*p(345)
p(1118) = p(14)*p(322)
p(1119) = p(14)*p(323)
p(1120) = p(9)*p(331)
p(1121) = p(9)*p(276)
p(1122) = p(9)*p(216)
p(1123) = p(9)*p(277)
p(1124) = p(9)*p(217)
p(1125) = p(9)*p(218)
p(1126) = p(9)*p(278)
p(1127) = p(9)*p(219)
p(1128) = p(9)*p(220)
p(1129) = p(9)*p(279)
p(1130) = p(9)*p(280)
p(1131) = p(9)*p(221)
p(1132) = p(9)*p(281)
p(1133) = p(9)*p(282)
p(1134) = p(9)*p(283)
p(1135) = p(9)*p(332)
p(1136) = p(3)*p(401) - p(988)
p(1137) = p(7)*p(401) - p(991)
p(1138) = p(49)*p(114) - p(996)
p(1139) = p(24)*p(194) - p(999)
p(1140) = p(3)*p(504) - p(990) - p(1139)
p(1141) = p(1)*p(695) - p(990) - p(1140)
p(1142) = p(1)*p(696) - p(993) - p(1141)
p(1143) = p(6)*p(566) - p(1136)
p(1144) = p(32)*p(178) - p(991)
p(1145) = p(39)*p(150) - p(996)
p(1146) = p(3)*p(505) - p(999)
p(1147) = p(14)*p(346)
p(1148) = p(9)*p(333)
p(1149) = p(9)*p(286)
p(1150) = p(9)*p(229)
p(1151) = p(9)*p(287)
p(1152) = p(9)*p(230)
p(1153) = p(9)*p(231)
p(1154) = p(9)*p(288)
p(1155) = p(9)*p(232)
p(1156) = p(9)*p(289)
p(1157) = p(9)*p(290)
p(1158) = p(9)*p(291)
p(1159) = p(9)*p(334)
p(1160) = p(24)*p(181) - p(963)
p(1161) = p(41)*p(114) - p(960)
p(1162) = p(4)*p(401) - p(961)
p(1163) = p(1)*p(713) - p(1005) - p(1162)
p(1164) = p(1)*p(714) - p(1006) - p(1163)
p(1165) = p(1)*p(715) - p(1007) - p(1164)
p(1166) = p(13)*p(331) - p(1147)
p(1167) = p(2)*p(695) - p(1002) - p(1166)
p(1168) = p(32)*p(181) - p(1147)
p(1169) = p(1)*p(717) - p(1007) - p(1167)
p(1170) = p(73)*p(91) - p(1147)
p(1171) = p(1)*p(719) - p(1011) - p(1169)
p(1172) = p(41)*p(150) - p(1147)
p(1173) = p(13)*p(279) - p(961)
p(1174) = p(16)*p(285) - p(1147)
p(1175) = p(69)*p(73) - p(960)
p(1176) = p(4)*p(505) - p(1147)
p(1177) = p(11)*p(285) - p(963)
p(1178) = p(14)*p(331)
p(1179) = p(14)*p(332)
p(1180) = p(3)*p(430) - p(1026)
p(1181) = p(7)*p(430) - p(1028)
p(1182) = p(49)*p(130) - p(1031)
p(1183) = p(3)*p(517) - p(1027) - p(1182)
p(1184) = p(1)*p(730) - p(1027) - p(1183)
p(1185) = p(45)*p(178) - p(1180)
p(1186) = p(39)*p(160) - p(1028)
p(1187) = p(3)*p(518) - p(1031)
p(1188) = p(14)*p(284)
p(1189) = p(14)*p(285)
p(1190) = p(14)*p(228)
p(1191) = p(9)*p(335)
p(1192) = p(9)*p(297)
p(1193) = p(9)*p(250)
p(1194) = p(9)*p(298)
p(1195) = p(9)*p(251)
p(1196) = p(9)*p(299)
p(1197) = p(9)*p(300)
p(1198) = p(9)*p(301)
p(1199) = p(9)*p(336)
p(1200) = p(24)*p(183) - p(964)
p(1201) = p(43)*p(114) - p(966)
p(1202) = p(8)*p(401) - p(1021)
p(1203) = p(1)*p(745) - p(1042) - p(1202)
p(1204) = p(1)*p(746) - p(1043) - p(1203)
p(1205) = p(13)*p(333) - p(1178)
p(1206) = p(2)*p(717) - p(1025) - p(1183)
p(1207) = p(32)*p(183) - p(1178)
p(1208) = p(1)*p(748) - p(1043) - p(1206)
p(1209) = p(73)*p(93) - p(1178)
p(1210) = p(6)*p(509) - p(1021)
p(1211) = p(43)*p(150) - p(1178)
p(1212) = p(32)*p(153) - p(966)
p(1213) = p(18)*p(285) - p(1178)
p(1214) = p(34)*p(150) - p(964)
p(1215) = p(14)*p(333)
p(1216) = p(14)*p(334)
p(1217) = p(41)*p(130) - p(973)
p(1218) = p(4)*p(430) - p(970)
p(1219) = p(1)*p(758) - p(1057) - p(1218)
p(1220) = p(1)*p(759) - p(1058) - p(1219)
p(1221) = p(81)*p(91) - p(1189)
p(1222) = p(2)*p(730) - p(1034) - p(1221)
p(1223) = p(41)*p(160) - p(1189)
p(1224) = p(1)*p(761) - p(1058) - p(1222)
p(1225) = p(16)*p(296) - p(1189)
p(1226) = p(69)*p(81) - p(970)
p(1227) = p(4)*p(518) - p(1189)
p(1228) = p(11)*p(296) - p(973)
p(1229) = p(14)*p(292)
p(1230) = p(14)*p(293)
p(1231) = p(14)*p(294)
p(1232) = p(14)*p(242)
p(1233) = p(14)*p(243)
p(1234) = p(3)*p(464) - p(1074)
p(1235) = p(7)*p(464) - p(1075)
p(1236) = p(3)*p(531) - p(1074) - p(1235)
p(1237) = p(39)*p(187) - p(1234)
p(1238) = p(3)*p(532) - p(1075)
p(1239) = p(14)*p(295)
p(1240) = p(14)*p(296)
p(1241) = p(14)*p(247)
p(1242) = p(14)*p(248)
p(1243) = p(14)*p(249)
p(1244) = p(9)*p(337)
p(1245) = p(9)*p(312)
p(1246) = p(9)*p(266)
p(1247) = p(9)*p(313)
p(1248) = p(9)*p(314)
p(1249) = p(9)*p(338)
p(1250) = p(24)*p(185) - p(975)
p(1251) = p(46)*p(114) - p(977)
p(1252) = p(1)*p(783) - p(1082) - p(1251)
p(1253) = p(1)*p(784) - p(1083) - p(1252)
p(1254) = p(13)*p(335) - p(1215)
p(1255) = p(2)*p(748) - p(1055) - p(1222)
p(1256) = p(32)*p(185) - p(1215)
p(1257) = p(1)*p(786) - p(1083) - p(1255)
p(1258) = p(73)*p(95) - p(1215)
p(1259) = p(13)*p(299) - p(977)
p(1260) = p(46)*p(150) - p(1215)
p(1261) = p(73)*p(84) - p(975)
p(1262) = p(14)*p(335)
p(1263) = p(14)*p(336)
p(1264) = p(43)*p(130) - p(974)
p(1265) = p(8)*p(430) - p(1038)
p(1266) = p(1)*p(794) - p(1093) - p(1265)
p(1267) = p(81)*p(93) - p(1190)
p(1268) = p(2)*p(761) - p(1066) - p(1236)
p(1269) = p(43)*p(160) - p(1190)
p(1270) = p(45)*p(153) - p(1038)
p(1271) = p(18)*p(296) - p(1190)
p(1272) = p(34)*p(160) - p(974)
p(1273) = p(14)*p(302)
p(1274) = p(14)*p(303)
p(1275) = p(14)*p(304)
p(1276) = p(14)*p(258)
p(1277) = p(14)*p(259)
p(1278) = p(4)*p(464) - p(983)
p(1279) = p(1)*p(804) - p(1103) - p(1278)
p(1280) = p(16)*p(310) - p(1240)
p(1281) = p(2)*p(772) - p(1077) - p(1280)
p(1282) = p(4)*p(532) - p(1240)
p(1283) = p(11)*p(310) - p(983)
p(1284) = p(14)*p(305)
p(1285) = p(14)*p(306)
p(1286) = p(14)*p(307)
p(1287) = p(14)*p(263)
p(1288) = p(14)*p(264)
p(1289) = p(14)*p(265)
p(1290) = p(6)*p(464) - p(1104)
p(1291) = p(3)*p(578) - p(1290)
p(1292) = p(14)*p(309)
p(1293) = p(14)*p(310)
p(1294) = p(9)*p(340)
p(1295) = p(9)*p(322)
p(1296) = p(9)*p(323)
p(1297) = p(9)*p(341)
p(1298) = p(24)*p(188) - p(984)
p(1299) = p(50)*p(114) - p(1089)
p(1300) = p(1)*p(813) - p(1108) - p(1299)
p(1301) = p(13)*p(337) - p(1262)
p(1302) = p(2)*p(786) - p(1091) - p(1268)
p(1303) = p(32)*p(188) - p(1262)
p(1304) = p(6)*p(536) - p(1089)
p(1305) = p(73)*p(98) - p(1262)
p(1306) = p(32)*p(173) - p(984)
p(1307) = p(14)*p(337)
p(1308) = p(14)*p(338)
p(1309) = p(46)*p(130) - p(982)
p(1310) = p(1)*p(820) - p(1113) - p(1309)
p(1311) = p(81)*p(95) - p(1232)
p(1312) = p(2)*p(796) - p(1098) - p(1281)
p(1313) = p(46)*p(160) - p(1232)
p(1314) = p(81)*p(84) - p(982)
p(1315) = p(14)*p(315)
p(1316) = p(14)*p(316)
p(1317) = p(14)*p(317)
p(1318) = p(14)*p(270)
p(1319) = p(14)*p(271)
p(1320) = p(8)*p(464) - p(1079)
p(1321) = p(18)*p(310) - p(1242)
p(1322) = p(34)*p(187) - p(1079)
p(1323) = p(14)*p(318)
p(1324) = p(14)*p(319)
p(1325) = p(14)*p(320)
p(1326) = p(9)*p(343)
p(1327) = p(9)*p(344)
p(1328) = p(24)*p(191) - p(987)
p(1329) = p(1)*p(827) - p(1117) - p(1328)
p(1330) = p(13)*p(340) - p(1307)
p(1331) = p(2)*p(815) - p(1112) - p(1312)
p(1332) = p(32)*p(191) - p(1307)
p(1333) = p(13)*p(323) - p(987)
p(1334) = p(14)*p(340)
p(1335) = p(14)*p(341)
p(1336) = p(50)*p(130) - p(1102)
p(1337) = p(81)*p(98) - p(1276)
p(1338) = p(45)*p(173) - p(1102)
p(1339) = p(14)*p(324)
p(1340) = p(14)*p(325)
p(1341) = p(14)*p(326)
p(1342) = p(9)*p(347)
p(1343) = p(24)*p(195) - p(1119)
p(1344) = p(13)*p(343) - p(1334)
p(1345) = p(6)*p(561) - p(1119)
p(1346) = p(14)*p(343)
p(1347) = p(14)*p(344)
p(1348) = p(9)*p(272)
p(1349) = p(9)*p(273)
p(1350) = p(9)*p(274)
p(1351) = p(9)*p(328)
p(1352) = p(9)*p(275)
p(1353) = p(9)*p(329)
p(1354) = p(9)*p(330)
p(1355) = p(9)*p(346)
p(1356) = p(3)*p(570) - p(1120)
p(1357) = p(39)*p(181) - p(1122)
p(1358) = p(41)*p(178) - p(1125)
p(1359) = p(4)*p(566) - p(1129)
p(1360) = p(1)*p(842) - p(1126) - p(1359)
p(1361) = p(1)*p(843) - p(1129)
p(1362) = p(1)*p(844) - p(1130) - p(1360)
p(1363) = p(21)*p(279) - p(1125)
p(1364) = p(1)*p(846) - p(1133) - p(1362)
p(1365) = p(69)*p(101) - p(1122)
p(1366) = p(1)*p(848) - p(1135) - p(1364)
p(1367) = p(11)*p(346) - p(1120)
p(1368) = p(1)*p(850) - p(1140) - p(1139)
p(1369) = p(1)*p(851) - p(1146)
p(1370) = p(3)*p(572) - p(1148)
p(1371) = p(39)*p(183) - p(1150)
p(1372) = p(43)*p(178) - p(1153)
p(1373) = p(1)*p(854) - p(1151) - p(1372)
p(1374) = p(2)*p(843) - p(1143)
p(1375) = p(1)*p(855) - p(1154) - p(1373)
p(1376) = p(7)*p(509) - p(1153)
p(1377) = p(1)*p(857) - p(1157) - p(1375)
p(1378) = p(49)*p(153) - p(1150)
p(1379) = p(1)*p(859) - p(1159) - p(1377)
p(1380) = p(34)*p(194) - p(1148)
p(1381) = p(6)*p(570) - p(1147)
p(1382) = p(1)*p(862) - p(1167) - p(1165)
p(1383) = p(2)*p(851) - p(1147)
p(1384) = p(1)*p(864) - p(1183) - p(1182)
p(1385) = p(1)*p(865) - p(1187)
p(1386) = p(3)*p(574) - p(1191)
p(1387) = p(39)*p(185) - p(1193)
p(1388) = p(46)*p(178) - p(1196)
p(1389) = p(1)*p(868) - p(1194) - p(1388)
p(1390) = p(1)*p(869) - p(1196)
p(1391) = p(1)*p(870) - p(1197) - p(1389)
p(1392) = p(21)*p(299) - p(1193)
p(1393) = p(1)*p(872) - p(1199) - p(1391)
p(1394) = p(84)*p(101) - p(1191)
p(1395) = p(6)*p(572) - p(1178)
p(1396) = p(1)*p(875) - p(1206) - p(1204)
p(1397) = p(8)*p(505) - p(1178)
p(1398) = p(45)*p(181) - p(1189)
p(1399) = p(1)*p(878) - p(1222) - p(1220)
p(1400) = p(2)*p(865) - p(1189)
p(1401) = p(1)*p(880) - p(1236) - p(1235)
p(1402) = p(1)*p(881) - p(1238)
p(1403) = p(3)*p(576) - p(1244)
p(1404) = p(39)*p(188) - p(1246)
p(1405) = p(1)*p(884) - p(1245) - p(1404)
p(1406) = p(2)*p(869) - p(1210)
p(1407) = p(1)*p(885) - p(1247) - p(1405)
p(1408) = p(7)*p(536) - p(1246)
p(1409) = p(1)*p(887) - p(1249) - p(1407)
p(1410) = p(49)*p(173) - p(1244)
p(1411) = p(6)*p(574) - p(1215)
p(1412) = p(1)*p(890) - p(1255) - p(1253)
p(1413) = p(22)*p(285) - p(1215)
p(1414) = p(45)*p(183) - p(1190)
p(1415) = p(1)*p(893) - p(1268) - p(1266)
p(1416) = p(8)*p(518) - p(1190)
p(1417) = p(41)*p(187) - p(1240)
p(1418) = p(1)*p(896) - p(1281) - p(1279)
p(1419) = p(2)*p(881) - p(1240)
p(1420) = p(14)*p(308)
p(1421) = p(4)*p(547) - p(1106) - p(1320)
p(1422) = p(1)*p(900) - p(1291)
p(1423) = p(14)*p(311)
p(1424) = p(3)*p(579) - p(1294)
p(1425) = p(39)*p(191) - p(1296)
p(1426) = p(1)*p(903) - p(1295) - p(1425)
p(1427) = p(1)*p(904) - p(1296)
p(1428) = p(1)*p(905) - p(1297) - p(1426)
p(1429) = p(21)*p(323) - p(1294)
p(1430) = p(6)*p(576) - p(1262)
p(1431) = p(1)*p(908) - p(1302) - p(1300)
p(1432) = p(50)*p(150) - p(1262)
p(1433) = p(45)*p(185) - p(1232)
p(1434) = p(1)*p(911) - p(1312) - p(1310)
p(1435) = p(22)*p(296) - p(1232)
p(1436) = p(43)*p(187) - p(1242)
p(1437) = p(1)*p(915) - p(1322) - p(1320)
p(1438) = p(8)*p(532) - p(1242)
p(1439) = p(14)*p(321)
p(1440) = p(4)*p(578) - p(1293)
p(1441) = p(2)*p(899) - p(1292) - p(1440)
p(1442) = p(2)*p(900) - p(1293)
p(1443) = p(14)*p(339)
p(1444) = p(3)*p(582) - p(1326)
p(1445) = p(2)*p(903) - p(1303) - p(1299)
p(1446) = p(2)*p(904) - p(1304)
p(1447) = p(1)*p(919) - p(1327) - p(1445)
p(1448) = p(7)*p(561) - p(1326)
p(1449) = p(6)*p(579) - p(1307)
p(1450) = p(1)*p(922) - p(1331) - p(1329)
p(1451) = p(73)*p(102) - p(1307)
p(1452) = p(45)*p(188) - p(1276)
p(1453) = p(1)*p(925) - p(1338) - p(1336)
p(1454) = p(50)*p(160) - p(1276)
p(1455) = p(14)*p(327)
p(1456) = p(46)*p(187) - p(1243)
p(1457) = p(2)*p(915) - p(1324) - p(1441)
p(1458) = p(22)*p(310) - p(1243)
p(1459) = p(14)*p(342)
p(1460) = p(3)*p(585) - p(1342)
p(1461) = p(2)*p(919) - p(1332) - p(1329)
p(1462) = p(1)*p(929) - p(1342)
p(1463) = p(6)*p(582) - p(1334)
p(1464) = p(1)*p(931) - p(1345) - p(1343)
p(1465) = p(32)*p(195) - p(1334)
p(1466) = p(45)*p(191) - p(1318)
p(1467) = p(2)*p(925) - p(1340) - p(1457)
p(1468) = p(81)*p(102) - p(1318)
p(1469) = p(14)*p(345)
p(1470) = p(2)*p(928) - p(1344) - p(1343)
p(1471) = p(2)*p(929) - p(1345)
p(1472) = p(6)*p(585) - p(1346)
p(1473) = p(2)*p(931) - p(1347) - p(1467)
p(1474) = p(13)*p(347) - p(1346)
p(1475) = p(14)*p(347)
p(1476) = p(3)*p(566) - p(1351)
p(1477) = p(7)*p(566) - p(1349)
p(1478) = p(49)*p(178) - p(1348)
p(1479) = p(39)*p(194) - p(1352)
p(1480) = p(3)*p(588) - p(1355)
p(1481) = p(1)*p(938) - p(1356)
p(1482) = p(2)*p(958) - p(1481)
p(1483) = p(1)*p(940) - p(1370)
p(1484) = p(8)*p(588) - p(1483)
p(1485) = p(1)*p(942) - p(1386)
p(1486) = p(22)*p(346) - p(1485)
p(1487) = p(1)*p(944) - p(1403)
p(1488) = p(50)*p(194) - p(1487)
p(1489) = p(1)*p(946) - p(1424)
p(1490) = p(101)*p(102) - p(1489)
p(1491) = p(6)*p(578) - p(1443)
p(1492) = p(1)*p(949) - p(1444)
p(1493) = p(49)*p(195) - p(1492)
p(1494) = p(8)*p(578) - p(1423)
p(1495) = p(1)*p(952) - p(1460)
p(1496) = p(21)*p(347) - p(1495)
p(1497) = p(50)*p(187) - p(1420)
p(1498) = p(1)*p(955) - p(1470)
p(1499) = p(7)*p(589) - p(1498)
p(1500) = p(45)*p(195) - p(1455)
p(1501) = p(2)*p(955) - p(1472)
p(1502) = p(1)*p(959) - p(1501)
p(1503) = p(6)*p(589) - p(1475)
p(1504) = p(1)*p(958) - p(1480)
p(1505) = p(2)*p(959) - p(1503)


do i=1,6
dm(0) = 0.0D0
dm(1) = dmsdr(i,4)
dm(2) = dmsdr(i,2)
dm(3) = dmsdr(i,1)
dm(4) = dmsdr(i,6)
dm(5) = dmsdr(i,5)
dm(6) = dmsdr(i,3)
dm(7) = dm(1)*rm(2) + rm(1)*dm(2)
dm(8) = dm(1)*rm(3) + rm(1)*dm(3)
dm(9) = dm(2)*rm(3) + rm(2)*dm(3)
dm(10) = dm(3)*rm(4) + rm(3)*dm(4)
dm(11) = dm(2)*rm(5) + rm(2)*dm(5)
dm(12) = dm(1)*rm(6) + rm(1)*dm(6)
dm(13) = dm(4)*rm(5) + rm(4)*dm(5)
dm(14) = dm(4)*rm(6) + rm(4)*dm(6)
dm(15) = dm(5)*rm(6) + rm(5)*dm(6)
dm(16) = dm(1)*rm(9) + rm(1)*dm(9)
dm(17) = dm(1)*rm(10) + rm(1)*dm(10)
dm(18) = dm(2)*rm(10) + rm(2)*dm(10)
dm(19) = dm(1)*rm(11) + rm(1)*dm(11)
dm(20) = dm(3)*rm(11) + rm(3)*dm(11)
dm(21) = dm(2)*rm(12) + rm(2)*dm(12)
dm(22) = dm(3)*rm(12) + rm(3)*dm(12)
dm(23) = dm(2)*rm(13) + rm(2)*dm(13)
dm(24) = dm(3)*rm(13) + rm(3)*dm(13)
dm(25) = dm(1)*rm(14) + rm(1)*dm(14)
dm(26) = dm(3)*rm(14) + rm(3)*dm(14)
dm(27) = dm(1)*rm(15) + rm(1)*dm(15)
dm(28) = dm(2)*rm(15) + rm(2)*dm(15)
dm(29) = dm(4)*rm(15) + rm(4)*dm(15)
dm(30) = dm(2)*rm(24) + rm(2)*dm(24)
dm(31) = dm(1)*rm(26) + rm(1)*dm(26)
dm(32) = dm(1)*rm(28) + rm(1)*dm(28)

dp(0) = dm(0)
dp(1) = dm(1) + dm(2) + dm(3)
dp(2) = dm(4) + dm(5) + dm(6)
dp(3) = dm(7) + dm(8) + dm(9)
dp(4) = dm(10) + dm(11) + dm(12)
dp(5) = dp(1)*p(2) + p(1)*dp(2) - dp(4)
dp(6) = dm(13) + dm(14) + dm(15)
dp(7) = dp(1)*p(1) + p(1)*dp(1) - dp(3) - dp(3)
dp(8) = dp(2)*p(2) + p(2)*dp(2) - dp(6) - dp(6)
dp(9) = dm(16)
dp(10) = dm(17) + dm(18) + dm(19) + dm(20) + dm(21) + dm(22)
dp(11) = dp(2)*p(3) + p(2)*dp(3) - dp(10)
dp(12) = dm(23) + dm(24) + dm(25) + dm(26) + dm(27) + dm(28)
dp(13) = dp(1)*p(6) + p(1)*dp(6) - dp(12)
dp(14) = dm(29)
dp(15) = dp(1)*p(3) + p(1)*dp(3) - dp(9) - dp(9) - dp(9)
dp(16) = dp(1)*p(4) + p(1)*dp(4) - dp(10)
dp(17) = dp(2)*p(7) + p(2)*dp(7) - dp(16)
dp(18) = dp(2)*p(4) + p(2)*dp(4) - dp(12)
dp(19) = dp(1)*p(8) + p(1)*dp(8) - dp(18)
dp(20) = dp(2)*p(6) + p(2)*dp(6) - dp(14) - dp(14) - dp(14)
dp(21) = dp(1)*p(7) + p(1)*dp(7) - dp(15)
dp(22) = dp(2)*p(8) + p(2)*dp(8) - dp(20)
dp(23) = dp(9)*p(2) + p(9)*dp(2)
dp(24) = dm(30) + dm(31) + dm(32)
dp(25) = dp(3)*p(6) + p(3)*dp(6) - dp(24)
dp(26) = dp(14)*p(1) + p(14)*dp(1)
dp(27) = dp(9)*p(1) + p(9)*dp(1)
dp(28) = dp(3)*p(4) + p(3)*dp(4) - dp(23)
dp(29) = dp(1)*p(10) + p(1)*dp(10) - dp(23) - dp(28) - dp(23)
dp(30) = dp(1)*p(11) + p(1)*dp(11) - dp(23)
dp(31) = dp(1)*p(12) + p(1)*dp(12) - dp(25) - dp(24) - dp(24)
dp(32) = dp(1)*p(13) + p(1)*dp(13) - dp(25)
dp(33) = dp(4)*p(5) + p(4)*dp(5) - dp(25) - dp(31)
dp(34) = dp(2)*p(11) + p(2)*dp(11) - dp(25)
dp(35) = dp(4)*p(6) + p(4)*dp(6) - dp(26)
dp(36) = dp(2)*p(12) + p(2)*dp(12) - dp(26) - dp(35) - dp(26)
dp(37) = dp(2)*p(13) + p(2)*dp(13) - dp(26)
dp(38) = dp(14)*p(2) + p(14)*dp(2)
dp(39) = dp(3)*p(3) + p(3)*dp(3) - dp(27) - dp(27)
dp(40) = dp(3)*p(7) + p(3)*dp(7) - dp(27)
dp(41) = dp(1)*p(16) + p(1)*dp(16) - dp(28)
dp(42) = dp(2)*p(21) + p(2)*dp(21) - dp(41)
dp(43) = dp(1)*p(18) + p(1)*dp(18) - dp(33)
dp(44) = dp(7)*p(8) + p(7)*dp(8) - dp(43)
dp(45) = dp(6)*p(6) + p(6)*dp(6) - dp(38) - dp(38)
dp(46) = dp(2)*p(18) + p(2)*dp(18) - dp(35)
dp(47) = dp(1)*p(22) + p(1)*dp(22) - dp(46)
dp(48) = dp(6)*p(8) + p(6)*dp(8) - dp(38)
dp(49) = dp(1)*p(21) + p(1)*dp(21) - dp(40)
dp(50) = dp(2)*p(22) + p(2)*dp(22) - dp(48)
dp(51) = dp(9)*p(6) + p(9)*dp(6)
dp(52) = dp(14)*p(3) + p(14)*dp(3)
dp(53) = dp(9)*p(4) + p(9)*dp(4)
dp(54) = dp(9)*p(5) + p(9)*dp(5)
dp(55) = dp(1)*p(24) + p(1)*dp(24) - dp(51)
dp(56) = dp(3)*p(12) + p(3)*dp(12) - dp(51) - dp(55) - dp(51)
dp(57) = dp(3)*p(13) + p(3)*dp(13) - dp(51)
dp(58) = dp(14)*p(7) + p(14)*dp(7)
dp(59) = dp(9)*p(8) + p(9)*dp(8)
dp(60) = dp(2)*p(24) + p(2)*dp(24) - dp(52)
dp(61) = dp(4)*p(13) + p(4)*dp(13) - dp(58)
dp(62) = dp(6)*p(11) + p(6)*dp(11) - dp(52)
dp(63) = dp(14)*p(4) + p(14)*dp(4)
dp(64) = dp(14)*p(5) + p(14)*dp(5)
dp(65) = dp(9)*p(3) + p(9)*dp(3)
dp(66) = dp(9)*p(7) + p(9)*dp(7)
dp(67) = dp(3)*p(16) + p(3)*dp(16) - dp(53)
dp(68) = dp(4)*p(15) + p(4)*dp(15) - dp(54) - dp(67)
dp(69) = dp(2)*p(39) + p(2)*dp(39) - dp(68)
dp(70) = dp(1)*p(29) + p(1)*dp(29) - dp(54) - dp(68)
dp(71) = dp(7)*p(11) + p(7)*dp(11) - dp(53)
dp(72) = dp(1)*p(31) + p(1)*dp(31) - dp(56) - dp(55)
dp(73) = dp(1)*p(32) + p(1)*dp(32) - dp(57)
dp(74) = dp(3)*p(18) + p(3)*dp(18) - dp(59)
dp(75) = dp(2)*p(29) + p(2)*dp(29) - dp(57) - dp(55)
dp(76) = dp(1)*p(34) + p(1)*dp(34) - dp(59)
dp(77) = dp(6)*p(16) + p(6)*dp(16) - dp(58)
dp(78) = dp(1)*p(36) + p(1)*dp(36) - dp(62) - dp(60)
dp(79) = dp(2)*p(32) + p(2)*dp(32) - dp(58)
dp(80) = dp(6)*p(12) + p(6)*dp(12) - dp(64) - dp(63) - dp(63)
dp(81) = dp(1)*p(45) + p(1)*dp(45) - dp(80)
dp(82) = dp(14)*p(6) + p(14)*dp(6)
dp(83) = dp(2)*p(33) + p(2)*dp(33) - dp(61) - dp(60)
dp(84) = dp(2)*p(34) + p(2)*dp(34) - dp(62)
dp(85) = dp(6)*p(18) + p(6)*dp(18) - dp(63)
dp(86) = dp(2)*p(36) + p(2)*dp(36) - dp(64) - dp(80)
dp(87) = dp(8)*p(13) + p(8)*dp(13) - dp(63)
dp(88) = dp(14)*p(8) + p(14)*dp(8)
dp(89) = dp(1)*p(39) + p(1)*dp(39) - dp(65)
dp(90) = dp(3)*p(21) + p(3)*dp(21) - dp(66)
dp(91) = dp(1)*p(41) + p(1)*dp(41) - dp(67)
dp(92) = dp(2)*p(49) + p(2)*dp(49) - dp(91)
dp(93) = dp(1)*p(43) + p(1)*dp(43) - dp(74)
dp(94) = dp(8)*p(21) + p(8)*dp(21) - dp(93)
dp(95) = dp(1)*p(46) + p(1)*dp(46) - dp(83)
dp(96) = dp(7)*p(22) + p(7)*dp(22) - dp(95)
dp(97) = dp(2)*p(45) + p(2)*dp(45) - dp(82)
dp(98) = dp(2)*p(46) + p(2)*dp(46) - dp(85)
dp(99) = dp(1)*p(50) + p(1)*dp(50) - dp(98)
dp(100) = dp(6)*p(22) + p(6)*dp(22) - dp(88)
dp(101) = dp(1)*p(49) + p(1)*dp(49) - dp(90)
dp(102) = dp(2)*p(50) + p(2)*dp(50) - dp(100)
dp(103) = dp(9)*p(14) + p(9)*dp(14)
dp(104) = dp(9)*p(12) + p(9)*dp(12)
dp(105) = dp(9)*p(13) + p(9)*dp(13)
dp(106) = dp(14)*p(15) + p(14)*dp(15)
dp(107) = dp(9)*p(20) + p(9)*dp(20)
dp(108) = dp(14)*p(10) + p(14)*dp(10)
dp(109) = dp(14)*p(11) + p(14)*dp(11)
dp(110) = dp(9)*p(16) + p(9)*dp(16)
dp(111) = dp(9)*p(10) + p(9)*dp(10)
dp(112) = dp(9)*p(11) + p(9)*dp(11)
dp(113) = dp(9)*p(17) + p(9)*dp(17)
dp(114) = dp(3)*p(24) + p(3)*dp(24) - dp(104)
dp(115) = dp(7)*p(24) + p(7)*dp(24) - dp(105)
dp(116) = dp(3)*p(31) + p(3)*dp(31) - dp(104) - dp(115)
dp(117) = dp(6)*p(39) + p(6)*dp(39) - dp(114)
dp(118) = dp(3)*p(32) + p(3)*dp(32) - dp(105)
dp(119) = dp(14)*p(21) + p(14)*dp(21)
dp(120) = dp(9)*p(18) + p(9)*dp(18)
dp(121) = dp(9)*p(19) + p(9)*dp(19)
dp(122) = dp(4)*p(24) + p(4)*dp(24) - dp(103) - dp(103) - dp(103)
dp(123) = dp(1)*p(60) + p(1)*dp(60) - dp(107) - dp(122)
dp(124) = dp(13)*p(16) + p(13)*dp(16) - dp(119)
dp(125) = dp(2)*p(56) + p(2)*dp(56) - dp(106) - dp(124)
dp(126) = dp(4)*p(32) + p(4)*dp(32) - dp(119)
dp(127) = dp(1)*p(62) + p(1)*dp(62) - dp(107) - dp(125)
dp(128) = dp(14)*p(16) + p(14)*dp(16)
dp(129) = dp(14)*p(17) + p(14)*dp(17)
dp(130) = dp(6)*p(24) + p(6)*dp(24) - dp(108)
dp(131) = dp(3)*p(45) + p(3)*dp(45) - dp(130)
dp(132) = dp(14)*p(12) + p(14)*dp(12)
dp(133) = dp(14)*p(13) + p(14)*dp(13)
dp(134) = dp(9)*p(22) + p(9)*dp(22)
dp(135) = dp(8)*p(24) + p(8)*dp(24) - dp(109)
dp(136) = dp(13)*p(18) + p(13)*dp(18) - dp(128)
dp(137) = dp(6)*p(34) + p(6)*dp(34) - dp(109)
dp(138) = dp(14)*p(18) + p(14)*dp(18)
dp(139) = dp(14)*p(19) + p(14)*dp(19)
dp(140) = dp(9)*p(9) + p(9)*dp(9)
dp(141) = dp(9)*p(15) + p(9)*dp(15)
dp(142) = dp(9)*p(21) + p(9)*dp(21)
dp(143) = dp(3)*p(41) + p(3)*dp(41) - dp(110)
dp(144) = dp(4)*p(39) + p(4)*dp(39) - dp(112)
dp(145) = dp(1)*p(68) + p(1)*dp(68) - dp(111) - dp(144)
dp(146) = dp(1)*p(69) + p(1)*dp(69) - dp(112)
dp(147) = dp(1)*p(70) + p(1)*dp(70) - dp(113) - dp(145)
dp(148) = dp(11)*p(21) + p(11)*dp(21) - dp(110)
dp(149) = dp(1)*p(72) + p(1)*dp(72) - dp(116) - dp(115)
dp(150) = dp(1)*p(73) + p(1)*dp(73) - dp(118)
dp(151) = dp(3)*p(43) + p(3)*dp(43) - dp(120)
dp(152) = dp(15)*p(18) + p(15)*dp(18) - dp(121) - dp(151)
dp(153) = dp(2)*p(69) + p(2)*dp(69) - dp(117)
dp(154) = dp(1)*p(75) + p(1)*dp(75) - dp(121) - dp(152)
dp(155) = dp(7)*p(34) + p(7)*dp(34) - dp(120)
dp(156) = dp(6)*p(41) + p(6)*dp(41) - dp(119)
dp(157) = dp(1)*p(78) + p(1)*dp(78) - dp(125) - dp(123)
dp(158) = dp(2)*p(73) + p(2)*dp(73) - dp(119)
dp(159) = dp(1)*p(80) + p(1)*dp(80) - dp(131) - dp(130) - dp(130)
dp(160) = dp(1)*p(81) + p(1)*dp(81) - dp(131)
dp(161) = dp(14)*p(14) + p(14)*dp(14)
dp(162) = dp(3)*p(46) + p(3)*dp(46) - dp(134)
dp(163) = dp(2)*p(75) + p(2)*dp(75) - dp(126) - dp(123)
dp(164) = dp(1)*p(84) + p(1)*dp(84) - dp(134)
dp(165) = dp(6)*p(43) + p(6)*dp(43) - dp(128)
dp(166) = dp(1)*p(86) + p(1)*dp(86) - dp(137) - dp(135)
dp(167) = dp(8)*p(32) + p(8)*dp(32) - dp(128)
dp(168) = dp(4)*p(45) + p(4)*dp(45) - dp(133)
dp(169) = dp(2)*p(80) + p(2)*dp(80) - dp(132) - dp(168)
dp(170) = dp(2)*p(81) + p(2)*dp(81) - dp(133)
dp(171) = dp(14)*p(20) + p(14)*dp(20)
dp(172) = dp(2)*p(83) + p(2)*dp(83) - dp(136) - dp(135)
dp(173) = dp(2)*p(84) + p(2)*dp(84) - dp(137)
dp(174) = dp(6)*p(46) + p(6)*dp(46) - dp(138)
dp(175) = dp(2)*p(86) + p(2)*dp(86) - dp(139) - dp(169)
dp(176) = dp(13)*p(22) + p(13)*dp(22) - dp(138)
dp(177) = dp(14)*p(22) + p(14)*dp(22)
dp(178) = dp(3)*p(39) + p(3)*dp(39) - dp(141)
dp(179) = dp(1)*p(89) + p(1)*dp(89) - dp(141) - dp(178) - dp(178)
dp(180) = dp(3)*p(49) + p(3)*dp(49) - dp(142)
dp(181) = dp(1)*p(91) + p(1)*dp(91) - dp(143)
dp(182) = dp(2)*p(101) + p(2)*dp(101) - dp(181)
dp(183) = dp(1)*p(93) + p(1)*dp(93) - dp(151)
dp(184) = dp(8)*p(49) + p(8)*dp(49) - dp(183)
dp(185) = dp(1)*p(95) + p(1)*dp(95) - dp(162)
dp(186) = dp(21)*p(22) + p(21)*dp(22) - dp(185)
dp(187) = dp(6)*p(45) + p(6)*dp(45) - dp(171)
dp(188) = dp(1)*p(98) + p(1)*dp(98) - dp(172)
dp(189) = dp(7)*p(50) + p(7)*dp(50) - dp(188)
dp(190) = dp(2)*p(97) + p(2)*dp(97) - dp(171) - dp(187) - dp(187)
dp(191) = dp(2)*p(98) + p(2)*dp(98) - dp(174)
dp(192) = dp(1)*p(102) + p(1)*dp(102) - dp(191)
dp(193) = dp(6)*p(50) + p(6)*dp(50) - dp(177)
dp(194) = dp(1)*p(101) + p(1)*dp(101) - dp(180)
dp(195) = dp(2)*p(102) + p(2)*dp(102) - dp(193)
dp(196) = dp(9)*p(26) + p(9)*dp(26)
dp(197) = dp(9)*p(38) + p(9)*dp(38)
dp(198) = dp(9)*p(24) + p(9)*dp(24)
dp(199) = dp(9)*p(31) + p(9)*dp(31)
dp(200) = dp(9)*p(25) + p(9)*dp(25)
dp(201) = dp(9)*p(32) + p(9)*dp(32)
dp(202) = dp(14)*p(39) + p(14)*dp(39)
dp(203) = dp(14)*p(40) + p(14)*dp(40)
dp(204) = dp(9)*p(35) + p(9)*dp(35)
dp(205) = dp(9)*p(36) + p(9)*dp(36)
dp(206) = dp(9)*p(37) + p(9)*dp(37)
dp(207) = dp(14)*p(28) + p(14)*dp(28)
dp(208) = dp(14)*p(29) + p(14)*dp(29)
dp(209) = dp(14)*p(30) + p(14)*dp(30)
dp(210) = dp(9)*p(45) + p(9)*dp(45)
dp(211) = dp(14)*p(24) + p(14)*dp(24)
dp(212) = dp(14)*p(25) + p(14)*dp(25)
dp(213) = dp(9)*p(48) + p(9)*dp(48)
dp(214) = dp(14)*p(33) + p(14)*dp(33)
dp(215) = dp(14)*p(34) + p(14)*dp(34)
dp(216) = dp(9)*p(41) + p(9)*dp(41)
dp(217) = dp(9)*p(28) + p(9)*dp(28)
dp(218) = dp(9)*p(23) + p(9)*dp(23)
dp(219) = dp(9)*p(29) + p(9)*dp(29)
dp(220) = dp(9)*p(30) + p(9)*dp(30)
dp(221) = dp(9)*p(42) + p(9)*dp(42)
dp(222) = dp(1)*p(114) + p(1)*dp(114) - dp(198)
dp(223) = dp(21)*p(24) + p(21)*dp(24) - dp(201)
dp(224) = dp(3)*p(72) + p(3)*dp(72) - dp(199) - dp(223)
dp(225) = dp(1)*p(116) + p(1)*dp(116) - dp(199) - dp(224)
dp(226) = dp(13)*p(39) + p(13)*dp(39) - dp(198)
dp(227) = dp(3)*p(73) + p(3)*dp(73) - dp(201)
dp(228) = dp(14)*p(49) + p(14)*dp(49)
dp(229) = dp(9)*p(43) + p(9)*dp(43)
dp(230) = dp(9)*p(33) + p(9)*dp(33)
dp(231) = dp(9)*p(34) + p(9)*dp(34)
dp(232) = dp(9)*p(44) + p(9)*dp(44)
dp(233) = dp(16)*p(24) + p(16)*dp(24) - dp(196)
dp(234) = dp(2)*p(114) + p(2)*dp(114) - dp(202)
dp(235) = dp(1)*p(123) + p(1)*dp(123) - dp(205) - dp(234)
dp(236) = dp(13)*p(41) + p(13)*dp(41) - dp(228)
dp(237) = dp(2)*p(116) + p(2)*dp(116) - dp(203) - dp(236)
dp(238) = dp(16)*p(32) + p(16)*dp(32) - dp(228)
dp(239) = dp(6)*p(69) + p(6)*dp(69) - dp(202)
dp(240) = dp(4)*p(73) + p(4)*dp(73) - dp(228)
dp(241) = dp(11)*p(32) + p(11)*dp(32) - dp(196)
dp(242) = dp(14)*p(41) + p(14)*dp(41)
dp(243) = dp(14)*p(42) + p(14)*dp(42)
dp(244) = dp(1)*p(130) + p(1)*dp(130) - dp(210)
dp(245) = dp(4)*p(62) + p(4)*dp(62) - dp(213) - dp(209)
dp(246) = dp(3)*p(81) + p(3)*dp(81) - dp(210)
dp(247) = dp(14)*p(31) + p(14)*dp(31)
dp(248) = dp(14)*p(32) + p(14)*dp(32)
dp(249) = dp(14)*p(26) + p(14)*dp(26)
dp(250) = dp(9)*p(46) + p(9)*dp(46)
dp(251) = dp(9)*p(47) + p(9)*dp(47)
dp(252) = dp(18)*p(24) + p(18)*dp(24) - dp(197)
dp(253) = dp(1)*p(135) + p(1)*dp(135) - dp(213) - dp(252)
dp(254) = dp(13)*p(43) + p(13)*dp(43) - dp(242)
dp(255) = dp(2)*p(125) + p(2)*dp(125) - dp(209) - dp(245)
dp(256) = dp(18)*p(32) + p(18)*dp(32) - dp(242)
dp(257) = dp(13)*p(34) + p(13)*dp(34) - dp(197)
dp(258) = dp(14)*p(43) + p(14)*dp(43)
dp(259) = dp(14)*p(44) + p(14)*dp(44)
dp(260) = dp(2)*p(130) + p(2)*dp(130) - dp(211)
dp(261) = dp(4)*p(81) + p(4)*dp(81) - dp(248)
dp(262) = dp(11)*p(45) + p(11)*dp(45) - dp(211)
dp(263) = dp(14)*p(35) + p(14)*dp(35)
dp(264) = dp(14)*p(36) + p(14)*dp(36)
dp(265) = dp(14)*p(37) + p(14)*dp(37)
dp(266) = dp(9)*p(50) + p(9)*dp(50)
dp(267) = dp(22)*p(24) + p(22)*dp(24) - dp(215)
dp(268) = dp(13)*p(46) + p(13)*dp(46) - dp(258)
dp(269) = dp(6)*p(84) + p(6)*dp(84) - dp(215)
dp(270) = dp(14)*p(46) + p(14)*dp(46)
dp(271) = dp(14)*p(47) + p(14)*dp(47)
dp(272) = dp(9)*p(27) + p(9)*dp(27)
dp(273) = dp(9)*p(39) + p(9)*dp(39)
dp(274) = dp(9)*p(40) + p(9)*dp(40)
dp(275) = dp(9)*p(49) + p(9)*dp(49)
dp(276) = dp(3)*p(91) + p(3)*dp(91) - dp(216)
dp(277) = dp(16)*p(39) + p(16)*dp(39) - dp(218)
dp(278) = dp(1)*p(144) + p(1)*dp(144) - dp(217) - dp(277)
dp(279) = dp(2)*p(178) + p(2)*dp(178) - dp(278)
dp(280) = dp(1)*p(145) + p(1)*dp(145) - dp(219) - dp(278)
dp(281) = dp(7)*p(69) + p(7)*dp(69) - dp(218)
dp(282) = dp(1)*p(147) + p(1)*dp(147) - dp(221) - dp(280)
dp(283) = dp(11)*p(49) + p(11)*dp(49) - dp(216)
dp(284) = dp(1)*p(149) + p(1)*dp(149) - dp(224) - dp(223)
dp(285) = dp(1)*p(150) + p(1)*dp(150) - dp(227)
dp(286) = dp(3)*p(93) + p(3)*dp(93) - dp(229)
dp(287) = dp(18)*p(39) + p(18)*dp(39) - dp(231)
dp(288) = dp(1)*p(152) + p(1)*dp(152) - dp(230) - dp(287)
dp(289) = dp(1)*p(153) + p(1)*dp(153) - dp(231)
dp(290) = dp(1)*p(154) + p(1)*dp(154) - dp(232) - dp(288)
dp(291) = dp(21)*p(34) + p(21)*dp(34) - dp(229)
dp(292) = dp(6)*p(91) + p(6)*dp(91) - dp(228)
dp(293) = dp(1)*p(157) + p(1)*dp(157) - dp(237) - dp(235)
dp(294) = dp(2)*p(150) + p(2)*dp(150) - dp(228)
dp(295) = dp(1)*p(159) + p(1)*dp(159) - dp(245) - dp(244)
dp(296) = dp(1)*p(160) + p(1)*dp(160) - dp(246)
dp(297) = dp(3)*p(95) + p(3)*dp(95) - dp(250)
dp(298) = dp(2)*p(152) + p(2)*dp(152) - dp(238) - dp(234)
dp(299) = dp(2)*p(153) + p(2)*dp(153) - dp(239)
dp(300) = dp(1)*p(163) + p(1)*dp(163) - dp(251) - dp(298)
dp(301) = dp(7)*p(84) + p(7)*dp(84) - dp(250)
dp(302) = dp(6)*p(93) + p(6)*dp(93) - dp(242)
dp(303) = dp(1)*p(166) + p(1)*dp(166) - dp(255) - dp(253)
dp(304) = dp(8)*p(73) + p(8)*dp(73) - dp(242)
dp(305) = dp(16)*p(45) + p(16)*dp(45) - dp(248)
dp(306) = dp(1)*p(169) + p(1)*dp(169) - dp(262) - dp(260)
dp(307) = dp(2)*p(160) + p(2)*dp(160) - dp(248)
dp(308) = dp(14)*p(38) + p(14)*dp(38)
dp(309) = dp(6)*p(80) + p(6)*dp(80) - dp(264) - dp(263)
dp(310) = dp(1)*p(187) + p(1)*dp(187) - dp(309)
dp(311) = dp(14)*p(45) + p(14)*dp(45)
dp(312) = dp(3)*p(98) + p(3)*dp(98) - dp(266)
dp(313) = dp(2)*p(163) + p(2)*dp(163) - dp(256) - dp(253)
dp(314) = dp(1)*p(173) + p(1)*dp(173) - dp(266)
dp(315) = dp(6)*p(95) + p(6)*dp(95) - dp(258)
dp(316) = dp(1)*p(175) + p(1)*dp(175) - dp(269) - dp(267)
dp(317) = dp(22)*p(32) + p(22)*dp(32) - dp(258)
dp(318) = dp(18)*p(45) + p(18)*dp(45) - dp(249)
dp(319) = dp(2)*p(169) + p(2)*dp(169) - dp(264) - dp(309)
dp(320) = dp(8)*p(81) + p(8)*dp(81) - dp(249)
dp(321) = dp(14)*p(48) + p(14)*dp(48)
dp(322) = dp(2)*p(172) + p(2)*dp(172) - dp(268) - dp(267)
dp(323) = dp(2)*p(173) + p(2)*dp(173) - dp(269)
dp(324) = dp(6)*p(98) + p(6)*dp(98) - dp(270)
dp(325) = dp(2)*p(175) + p(2)*dp(175) - dp(271) - dp(319)
dp(326) = dp(13)*p(50) + p(13)*dp(50) - dp(270)
dp(327) = dp(14)*p(50) + p(14)*dp(50)
dp(328) = dp(1)*p(178) + p(1)*dp(178) - dp(273)
dp(329) = dp(21)*p(39) + p(21)*dp(39) - dp(272)
dp(330) = dp(3)*p(101) + p(3)*dp(101) - dp(275)
dp(331) = dp(1)*p(181) + p(1)*dp(181) - dp(276)
dp(332) = dp(2)*p(194) + p(2)*dp(194) - dp(331)
dp(333) = dp(1)*p(183) + p(1)*dp(183) - dp(286)
dp(334) = dp(8)*p(101) + p(8)*dp(101) - dp(333)
dp(335) = dp(1)*p(185) + p(1)*dp(185) - dp(297)
dp(336) = dp(22)*p(49) + p(22)*dp(49) - dp(335)
dp(337) = dp(1)*p(188) + p(1)*dp(188) - dp(312)
dp(338) = dp(21)*p(50) + p(21)*dp(50) - dp(337)
dp(339) = dp(2)*p(187) + p(2)*dp(187) - dp(311)
dp(340) = dp(1)*p(191) + p(1)*dp(191) - dp(322)
dp(341) = dp(7)*p(102) + p(7)*dp(102) - dp(340)
dp(342) = dp(22)*p(45) + p(22)*dp(45) - dp(308)
dp(343) = dp(2)*p(191) + p(2)*dp(191) - dp(324)
dp(344) = dp(1)*p(195) + p(1)*dp(195) - dp(343)
dp(345) = dp(6)*p(102) + p(6)*dp(102) - dp(327)
dp(346) = dp(1)*p(194) + p(1)*dp(194) - dp(330)
dp(347) = dp(2)*p(195) + p(2)*dp(195) - dp(345)
dp(348) = dp(9)*p(52) + p(9)*dp(52)
dp(349) = dp(9)*p(58) + p(9)*dp(58)
dp(350) = dp(9)*p(63) + p(9)*dp(63)
dp(351) = dp(9)*p(64) + p(9)*dp(64)
dp(352) = dp(9)*p(82) + p(9)*dp(82)
dp(353) = dp(9)*p(88) + p(9)*dp(88)
dp(354) = dp(9)*p(55) + p(9)*dp(55)
dp(355) = dp(9)*p(72) + p(9)*dp(72)
dp(356) = dp(9)*p(51) + p(9)*dp(51)
dp(357) = dp(9)*p(56) + p(9)*dp(56)
dp(358) = dp(9)*p(57) + p(9)*dp(57)
dp(359) = dp(9)*p(73) + p(9)*dp(73)
dp(360) = dp(14)*p(89) + p(14)*dp(89)
dp(361) = dp(14)*p(90) + p(14)*dp(90)
dp(362) = dp(9)*p(77) + p(9)*dp(77)
dp(363) = dp(9)*p(60) + p(9)*dp(60)
dp(364) = dp(9)*p(78) + p(9)*dp(78)
dp(365) = dp(9)*p(61) + p(9)*dp(61)
dp(366) = dp(9)*p(62) + p(9)*dp(62)
dp(367) = dp(9)*p(79) + p(9)*dp(79)
dp(368) = dp(14)*p(67) + p(14)*dp(67)
dp(369) = dp(14)*p(68) + p(14)*dp(68)
dp(370) = dp(14)*p(69) + p(14)*dp(69)
dp(371) = dp(14)*p(70) + p(14)*dp(70)
dp(372) = dp(14)*p(71) + p(14)*dp(71)
dp(373) = dp(9)*p(80) + p(9)*dp(80)
dp(374) = dp(9)*p(81) + p(9)*dp(81)
dp(375) = dp(14)*p(55) + p(14)*dp(55)
dp(376) = dp(14)*p(56) + p(14)*dp(56)
dp(377) = dp(14)*p(57) + p(14)*dp(57)
dp(378) = dp(14)*p(52) + p(14)*dp(52)
dp(379) = dp(9)*p(85) + p(9)*dp(85)
dp(380) = dp(9)*p(86) + p(9)*dp(86)
dp(381) = dp(9)*p(87) + p(9)*dp(87)
dp(382) = dp(14)*p(74) + p(14)*dp(74)
dp(383) = dp(14)*p(75) + p(14)*dp(75)
dp(384) = dp(14)*p(76) + p(14)*dp(76)
dp(385) = dp(9)*p(97) + p(9)*dp(97)
dp(386) = dp(14)*p(60) + p(14)*dp(60)
dp(387) = dp(14)*p(61) + p(14)*dp(61)
dp(388) = dp(14)*p(62) + p(14)*dp(62)
dp(389) = dp(9)*p(100) + p(9)*dp(100)
dp(390) = dp(14)*p(83) + p(14)*dp(83)
dp(391) = dp(14)*p(84) + p(14)*dp(84)
dp(392) = dp(9)*p(91) + p(9)*dp(91)
dp(393) = dp(9)*p(67) + p(9)*dp(67)
dp(394) = dp(9)*p(53) + p(9)*dp(53)
dp(395) = dp(9)*p(68) + p(9)*dp(68)
dp(396) = dp(9)*p(54) + p(9)*dp(54)
dp(397) = dp(9)*p(69) + p(9)*dp(69)
dp(398) = dp(9)*p(70) + p(9)*dp(70)
dp(399) = dp(9)*p(71) + p(9)*dp(71)
dp(400) = dp(9)*p(92) + p(9)*dp(92)
dp(401) = dp(3)*p(114) + p(3)*dp(114) - dp(354)
dp(402) = dp(7)*p(114) + p(7)*dp(114) - dp(356)
dp(403) = dp(24)*p(49) + p(24)*dp(49) - dp(359)
dp(404) = dp(3)*p(149) + p(3)*dp(149) - dp(355) - dp(403)
dp(405) = dp(1)*p(224) + p(1)*dp(224) - dp(355) - dp(404)
dp(406) = dp(6)*p(178) + p(6)*dp(178) - dp(401)
dp(407) = dp(32)*p(39) + p(32)*dp(39) - dp(356)
dp(408) = dp(3)*p(150) + p(3)*dp(150) - dp(359)
dp(409) = dp(14)*p(101) + p(14)*dp(101)
dp(410) = dp(9)*p(93) + p(9)*dp(93)
dp(411) = dp(9)*p(74) + p(9)*dp(74)
dp(412) = dp(9)*p(59) + p(9)*dp(59)
dp(413) = dp(9)*p(75) + p(9)*dp(75)
dp(414) = dp(9)*p(76) + p(9)*dp(76)
dp(415) = dp(9)*p(94) + p(9)*dp(94)
dp(416) = dp(24)*p(41) + p(24)*dp(41) - dp(349)
dp(417) = dp(4)*p(114) + p(4)*dp(114) - dp(348)
dp(418) = dp(1)*p(234) + p(1)*dp(234) - dp(363) - dp(417)
dp(419) = dp(1)*p(235) + p(1)*dp(235) - dp(364) - dp(418)
dp(420) = dp(13)*p(91) + p(13)*dp(91) - dp(409)
dp(421) = dp(2)*p(224) + p(2)*dp(224) - dp(361) - dp(420)
dp(422) = dp(32)*p(41) + p(32)*dp(41) - dp(409)
dp(423) = dp(1)*p(237) + p(1)*dp(237) - dp(364) - dp(421)
dp(424) = dp(16)*p(73) + p(16)*dp(73) - dp(409)
dp(425) = dp(13)*p(69) + p(13)*dp(69) - dp(348)
dp(426) = dp(4)*p(150) + p(4)*dp(150) - dp(409)
dp(427) = dp(11)*p(73) + p(11)*dp(73) - dp(349)
dp(428) = dp(14)*p(91) + p(14)*dp(91)
dp(429) = dp(14)*p(92) + p(14)*dp(92)
dp(430) = dp(3)*p(130) + p(3)*dp(130) - dp(373)
dp(431) = dp(7)*p(130) + p(7)*dp(130) - dp(374)
dp(432) = dp(3)*p(159) + p(3)*dp(159) - dp(373) - dp(431)
dp(433) = dp(39)*p(45) + p(39)*dp(45) - dp(430)
dp(434) = dp(3)*p(160) + p(3)*dp(160) - dp(374)
dp(435) = dp(14)*p(72) + p(14)*dp(72)
dp(436) = dp(14)*p(73) + p(14)*dp(73)
dp(437) = dp(14)*p(58) + p(14)*dp(58)
dp(438) = dp(9)*p(95) + p(9)*dp(95)
dp(439) = dp(9)*p(83) + p(9)*dp(83)
dp(440) = dp(9)*p(84) + p(9)*dp(84)
dp(441) = dp(9)*p(96) + p(9)*dp(96)
dp(442) = dp(24)*p(43) + p(24)*dp(43) - dp(350)
dp(443) = dp(8)*p(114) + p(8)*dp(114) - dp(370)
dp(444) = dp(1)*p(253) + p(1)*dp(253) - dp(380) - dp(443)
dp(445) = dp(13)*p(93) + p(13)*dp(93) - dp(428)
dp(446) = dp(2)*p(237) + p(2)*dp(237) - dp(372) - dp(432)
dp(447) = dp(32)*p(43) + p(32)*dp(43) - dp(428)
dp(448) = dp(6)*p(153) + p(6)*dp(153) - dp(370)
dp(449) = dp(18)*p(73) + p(18)*dp(73) - dp(428)
dp(450) = dp(32)*p(34) + p(32)*dp(34) - dp(350)
dp(451) = dp(14)*p(93) + p(14)*dp(93)
dp(452) = dp(14)*p(94) + p(14)*dp(94)
dp(453) = dp(4)*p(130) + p(4)*dp(130) - dp(352)
dp(454) = dp(1)*p(260) + p(1)*dp(260) - dp(385) - dp(453)
dp(455) = dp(16)*p(81) + p(16)*dp(81) - dp(436)
dp(456) = dp(2)*p(245) + p(2)*dp(245) - dp(376) - dp(455)
dp(457) = dp(4)*p(160) + p(4)*dp(160) - dp(436)
dp(458) = dp(11)*p(81) + p(11)*dp(81) - dp(352)
dp(459) = dp(14)*p(77) + p(14)*dp(77)
dp(460) = dp(14)*p(78) + p(14)*dp(78)
dp(461) = dp(14)*p(79) + p(14)*dp(79)
dp(462) = dp(14)*p(63) + p(14)*dp(63)
dp(463) = dp(14)*p(64) + p(14)*dp(64)
dp(464) = dp(6)*p(130) + p(6)*dp(130) - dp(386)
dp(465) = dp(3)*p(187) + p(3)*dp(187) - dp(464)
dp(466) = dp(14)*p(80) + p(14)*dp(80)
dp(467) = dp(14)*p(81) + p(14)*dp(81)
dp(468) = dp(9)*p(98) + p(9)*dp(98)
dp(469) = dp(9)*p(99) + p(9)*dp(99)
dp(470) = dp(24)*p(46) + p(24)*dp(46) - dp(353)
dp(471) = dp(1)*p(267) + p(1)*dp(267) - dp(389) - dp(470)
dp(472) = dp(13)*p(95) + p(13)*dp(95) - dp(451)
dp(473) = dp(2)*p(255) + p(2)*dp(255) - dp(384) - dp(456)
dp(474) = dp(32)*p(46) + p(32)*dp(46) - dp(451)
dp(475) = dp(13)*p(84) + p(13)*dp(84) - dp(353)
dp(476) = dp(14)*p(95) + p(14)*dp(95)
dp(477) = dp(14)*p(96) + p(14)*dp(96)
dp(478) = dp(8)*p(130) + p(8)*dp(130) - dp(378)
dp(479) = dp(18)*p(81) + p(18)*dp(81) - dp(437)
dp(480) = dp(34)*p(45) + p(34)*dp(45) - dp(378)
dp(481) = dp(14)*p(85) + p(14)*dp(85)
dp(482) = dp(14)*p(86) + p(14)*dp(86)
dp(483) = dp(14)*p(87) + p(14)*dp(87)
dp(484) = dp(9)*p(102) + p(9)*dp(102)
dp(485) = dp(24)*p(50) + p(24)*dp(50) - dp(391)
dp(486) = dp(13)*p(98) + p(13)*dp(98) - dp(476)
dp(487) = dp(6)*p(173) + p(6)*dp(173) - dp(391)
dp(488) = dp(14)*p(98) + p(14)*dp(98)
dp(489) = dp(14)*p(99) + p(14)*dp(99)
dp(490) = dp(9)*p(65) + p(9)*dp(65)
dp(491) = dp(9)*p(66) + p(9)*dp(66)
dp(492) = dp(9)*p(89) + p(9)*dp(89)
dp(493) = dp(9)*p(90) + p(9)*dp(90)
dp(494) = dp(9)*p(101) + p(9)*dp(101)
dp(495) = dp(3)*p(181) + p(3)*dp(181) - dp(392)
dp(496) = dp(39)*p(41) + p(39)*dp(41) - dp(394)
dp(497) = dp(4)*p(178) + p(4)*dp(178) - dp(397)
dp(498) = dp(1)*p(278) + p(1)*dp(278) - dp(395) - dp(497)
dp(499) = dp(1)*p(279) + p(1)*dp(279) - dp(397)
dp(500) = dp(1)*p(280) + p(1)*dp(280) - dp(398) - dp(498)
dp(501) = dp(21)*p(69) + p(21)*dp(69) - dp(394)
dp(502) = dp(1)*p(282) + p(1)*dp(282) - dp(400) - dp(500)
dp(503) = dp(11)*p(101) + p(11)*dp(101) - dp(392)
dp(504) = dp(1)*p(284) + p(1)*dp(284) - dp(404) - dp(403)
dp(505) = dp(1)*p(285) + p(1)*dp(285) - dp(408)
dp(506) = dp(3)*p(183) + p(3)*dp(183) - dp(410)
dp(507) = dp(39)*p(43) + p(39)*dp(43) - dp(412)
dp(508) = dp(1)*p(287) + p(1)*dp(287) - dp(411) - dp(507)
dp(509) = dp(2)*p(279) + p(2)*dp(279) - dp(406)
dp(510) = dp(1)*p(288) + p(1)*dp(288) - dp(413) - dp(508)
dp(511) = dp(7)*p(153) + p(7)*dp(153) - dp(412)
dp(512) = dp(1)*p(290) + p(1)*dp(290) - dp(415) - dp(510)
dp(513) = dp(34)*p(49) + p(34)*dp(49) - dp(410)
dp(514) = dp(6)*p(181) + p(6)*dp(181) - dp(409)
dp(515) = dp(1)*p(293) + p(1)*dp(293) - dp(421) - dp(419)
dp(516) = dp(2)*p(285) + p(2)*dp(285) - dp(409)
dp(517) = dp(1)*p(295) + p(1)*dp(295) - dp(432) - dp(431)
dp(518) = dp(1)*p(296) + p(1)*dp(296) - dp(434)
dp(519) = dp(3)*p(185) + p(3)*dp(185) - dp(438)
dp(520) = dp(39)*p(46) + p(39)*dp(46) - dp(440)
dp(521) = dp(1)*p(298) + p(1)*dp(298) - dp(439) - dp(520)
dp(522) = dp(1)*p(299) + p(1)*dp(299) - dp(440)
dp(523) = dp(1)*p(300) + p(1)*dp(300) - dp(441) - dp(521)
dp(524) = dp(21)*p(84) + p(21)*dp(84) - dp(438)
dp(525) = dp(6)*p(183) + p(6)*dp(183) - dp(428)
dp(526) = dp(1)*p(303) + p(1)*dp(303) - dp(446) - dp(444)
dp(527) = dp(8)*p(150) + p(8)*dp(150) - dp(428)
dp(528) = dp(41)*p(45) + p(41)*dp(45) - dp(436)
dp(529) = dp(1)*p(306) + p(1)*dp(306) - dp(456) - dp(454)
dp(530) = dp(2)*p(296) + p(2)*dp(296) - dp(436)
dp(531) = dp(4)*p(169) + p(4)*dp(169) - dp(388) - dp(478)
dp(532) = dp(1)*p(310) + p(1)*dp(310) - dp(465)
dp(533) = dp(14)*p(82) + p(14)*dp(82)
dp(534) = dp(3)*p(188) + p(3)*dp(188) - dp(468)
dp(535) = dp(2)*p(298) + p(2)*dp(298) - dp(447) - dp(443)
dp(536) = dp(2)*p(299) + p(2)*dp(299) - dp(448)
dp(537) = dp(1)*p(313) + p(1)*dp(313) - dp(469) - dp(535)
dp(538) = dp(7)*p(173) + p(7)*dp(173) - dp(468)
dp(539) = dp(6)*p(185) + p(6)*dp(185) - dp(451)
dp(540) = dp(1)*p(316) + p(1)*dp(316) - dp(473) - dp(471)
dp(541) = dp(22)*p(73) + p(22)*dp(73) - dp(451)
dp(542) = dp(43)*p(45) + p(43)*dp(45) - dp(437)
dp(543) = dp(1)*p(319) + p(1)*dp(319) - dp(480) - dp(478)
dp(544) = dp(8)*p(160) + p(8)*dp(160) - dp(437)
dp(545) = dp(14)*p(88) + p(14)*dp(88)
dp(546) = dp(4)*p(187) + p(4)*dp(187) - dp(467)
dp(547) = dp(2)*p(309) + p(2)*dp(309) - dp(466) - dp(546)
dp(548) = dp(2)*p(310) + p(2)*dp(310) - dp(467)
dp(549) = dp(14)*p(97) + p(14)*dp(97)
dp(550) = dp(3)*p(191) + p(3)*dp(191) - dp(484)
dp(551) = dp(2)*p(313) + p(2)*dp(313) - dp(474) - dp(471)
dp(552) = dp(1)*p(323) + p(1)*dp(323) - dp(484)
dp(553) = dp(6)*p(188) + p(6)*dp(188) - dp(476)
dp(554) = dp(1)*p(325) + p(1)*dp(325) - dp(487) - dp(485)
dp(555) = dp(32)*p(50) + p(32)*dp(50) - dp(476)
dp(556) = dp(45)*p(46) + p(45)*dp(46) - dp(462)
dp(557) = dp(2)*p(319) + p(2)*dp(319) - dp(482) - dp(547)
dp(558) = dp(22)*p(81) + p(22)*dp(81) - dp(462)
dp(559) = dp(14)*p(100) + p(14)*dp(100)
dp(560) = dp(2)*p(322) + p(2)*dp(322) - dp(486) - dp(485)
dp(561) = dp(2)*p(323) + p(2)*dp(323) - dp(487)
dp(562) = dp(6)*p(191) + p(6)*dp(191) - dp(488)
dp(563) = dp(2)*p(325) + p(2)*dp(325) - dp(489) - dp(557)
dp(564) = dp(13)*p(102) + p(13)*dp(102) - dp(488)
dp(565) = dp(14)*p(102) + p(14)*dp(102)
dp(566) = dp(3)*p(178) + p(3)*dp(178) - dp(492)
dp(567) = dp(7)*p(178) + p(7)*dp(178) - dp(490)
dp(568) = dp(39)*p(49) + p(39)*dp(49) - dp(491)
dp(569) = dp(3)*p(194) + p(3)*dp(194) - dp(494)
dp(570) = dp(1)*p(331) + p(1)*dp(331) - dp(495)
dp(571) = dp(2)*p(346) + p(2)*dp(346) - dp(570)
dp(572) = dp(1)*p(333) + p(1)*dp(333) - dp(506)
dp(573) = dp(8)*p(194) + p(8)*dp(194) - dp(572)
dp(574) = dp(1)*p(335) + p(1)*dp(335) - dp(519)
dp(575) = dp(22)*p(101) + p(22)*dp(101) - dp(574)
dp(576) = dp(1)*p(337) + p(1)*dp(337) - dp(534)
dp(577) = dp(49)*p(50) + p(49)*dp(50) - dp(576)
dp(578) = dp(6)*p(187) + p(6)*dp(187) - dp(549)
dp(579) = dp(1)*p(340) + p(1)*dp(340) - dp(550)
dp(580) = dp(21)*p(102) + p(21)*dp(102) - dp(579)
dp(581) = dp(8)*p(187) + p(8)*dp(187) - dp(533)
dp(582) = dp(1)*p(343) + p(1)*dp(343) - dp(560)
dp(583) = dp(7)*p(195) + p(7)*dp(195) - dp(582)
dp(584) = dp(45)*p(50) + p(45)*dp(50) - dp(545)
dp(585) = dp(2)*p(343) + p(2)*dp(343) - dp(562)
dp(586) = dp(1)*p(347) + p(1)*dp(347) - dp(585)
dp(587) = dp(6)*p(195) + p(6)*dp(195) - dp(565)
dp(588) = dp(1)*p(346) + p(1)*dp(346) - dp(569)
dp(589) = dp(2)*p(347) + p(2)*dp(347) - dp(587)
dp(590) = dp(9)*p(103) + p(9)*dp(103)
dp(591) = dp(9)*p(106) + p(9)*dp(106)
dp(592) = dp(9)*p(119) + p(9)*dp(119)
dp(593) = dp(9)*p(128) + p(9)*dp(128)
dp(594) = dp(9)*p(108) + p(9)*dp(108)
dp(595) = dp(9)*p(109) + p(9)*dp(109)
dp(596) = dp(9)*p(129) + p(9)*dp(129)
dp(597) = dp(9)*p(132) + p(9)*dp(132)
dp(598) = dp(9)*p(133) + p(9)*dp(133)
dp(599) = dp(9)*p(161) + p(9)*dp(161)
dp(600) = dp(9)*p(138) + p(9)*dp(138)
dp(601) = dp(9)*p(139) + p(9)*dp(139)
dp(602) = dp(9)*p(171) + p(9)*dp(171)
dp(603) = dp(9)*p(177) + p(9)*dp(177)
dp(604) = dp(9)*p(114) + p(9)*dp(114)
dp(605) = dp(9)*p(115) + p(9)*dp(115)
dp(606) = dp(9)*p(149) + p(9)*dp(149)
dp(607) = dp(9)*p(104) + p(9)*dp(104)
dp(608) = dp(9)*p(116) + p(9)*dp(116)
dp(609) = dp(9)*p(105) + p(9)*dp(105)
dp(610) = dp(9)*p(117) + p(9)*dp(117)
dp(611) = dp(9)*p(118) + p(9)*dp(118)
dp(612) = dp(9)*p(150) + p(9)*dp(150)
dp(613) = dp(14)*p(178) + p(14)*dp(178)
dp(614) = dp(14)*p(179) + p(14)*dp(179)
dp(615) = dp(14)*p(180) + p(14)*dp(180)
dp(616) = dp(9)*p(156) + p(9)*dp(156)
dp(617) = dp(9)*p(122) + p(9)*dp(122)
dp(618) = dp(9)*p(123) + p(9)*dp(123)
dp(619) = dp(9)*p(157) + p(9)*dp(157)
dp(620) = dp(9)*p(124) + p(9)*dp(124)
dp(621) = dp(9)*p(107) + p(9)*dp(107)
dp(622) = dp(9)*p(125) + p(9)*dp(125)
dp(623) = dp(9)*p(126) + p(9)*dp(126)
dp(624) = dp(9)*p(127) + p(9)*dp(127)
dp(625) = dp(9)*p(158) + p(9)*dp(158)
dp(626) = dp(14)*p(143) + p(14)*dp(143)
dp(627) = dp(14)*p(144) + p(14)*dp(144)
dp(628) = dp(14)*p(145) + p(14)*dp(145)
dp(629) = dp(14)*p(146) + p(14)*dp(146)
dp(630) = dp(14)*p(147) + p(14)*dp(147)
dp(631) = dp(14)*p(148) + p(14)*dp(148)
dp(632) = dp(9)*p(130) + p(9)*dp(130)
dp(633) = dp(9)*p(159) + p(9)*dp(159)
dp(634) = dp(9)*p(131) + p(9)*dp(131)
dp(635) = dp(9)*p(160) + p(9)*dp(160)
dp(636) = dp(14)*p(114) + p(14)*dp(114)
dp(637) = dp(14)*p(115) + p(14)*dp(115)
dp(638) = dp(14)*p(116) + p(14)*dp(116)
dp(639) = dp(14)*p(117) + p(14)*dp(117)
dp(640) = dp(14)*p(118) + p(14)*dp(118)
dp(641) = dp(14)*p(106) + p(14)*dp(106)
dp(642) = dp(9)*p(165) + p(9)*dp(165)
dp(643) = dp(9)*p(135) + p(9)*dp(135)
dp(644) = dp(9)*p(166) + p(9)*dp(166)
dp(645) = dp(9)*p(136) + p(9)*dp(136)
dp(646) = dp(9)*p(137) + p(9)*dp(137)
dp(647) = dp(9)*p(167) + p(9)*dp(167)
dp(648) = dp(14)*p(151) + p(14)*dp(151)
dp(649) = dp(14)*p(152) + p(14)*dp(152)
dp(650) = dp(14)*p(153) + p(14)*dp(153)
dp(651) = dp(14)*p(154) + p(14)*dp(154)
dp(652) = dp(14)*p(155) + p(14)*dp(155)
dp(653) = dp(9)*p(168) + p(9)*dp(168)
dp(654) = dp(9)*p(169) + p(9)*dp(169)
dp(655) = dp(9)*p(170) + p(9)*dp(170)
dp(656) = dp(14)*p(122) + p(14)*dp(122)
dp(657) = dp(14)*p(123) + p(14)*dp(123)
dp(658) = dp(14)*p(124) + p(14)*dp(124)
dp(659) = dp(14)*p(125) + p(14)*dp(125)
dp(660) = dp(14)*p(126) + p(14)*dp(126)
dp(661) = dp(14)*p(127) + p(14)*dp(127)
dp(662) = dp(14)*p(108) + p(14)*dp(108)
dp(663) = dp(14)*p(109) + p(14)*dp(109)
dp(664) = dp(9)*p(187) + p(9)*dp(187)
dp(665) = dp(14)*p(130) + p(14)*dp(130)
dp(666) = dp(14)*p(131) + p(14)*dp(131)
dp(667) = dp(9)*p(174) + p(9)*dp(174)
dp(668) = dp(9)*p(175) + p(9)*dp(175)
dp(669) = dp(9)*p(176) + p(9)*dp(176)
dp(670) = dp(14)*p(162) + p(14)*dp(162)
dp(671) = dp(14)*p(163) + p(14)*dp(163)
dp(672) = dp(14)*p(164) + p(14)*dp(164)
dp(673) = dp(9)*p(190) + p(9)*dp(190)
dp(674) = dp(14)*p(135) + p(14)*dp(135)
dp(675) = dp(14)*p(136) + p(14)*dp(136)
dp(676) = dp(14)*p(137) + p(14)*dp(137)
dp(677) = dp(9)*p(193) + p(9)*dp(193)
dp(678) = dp(14)*p(172) + p(14)*dp(172)
dp(679) = dp(14)*p(173) + p(14)*dp(173)
dp(680) = dp(9)*p(181) + p(9)*dp(181)
dp(681) = dp(9)*p(143) + p(9)*dp(143)
dp(682) = dp(9)*p(110) + p(9)*dp(110)
dp(683) = dp(9)*p(144) + p(9)*dp(144)
dp(684) = dp(9)*p(111) + p(9)*dp(111)
dp(685) = dp(9)*p(112) + p(9)*dp(112)
dp(686) = dp(9)*p(145) + p(9)*dp(145)
dp(687) = dp(9)*p(113) + p(9)*dp(113)
dp(688) = dp(9)*p(146) + p(9)*dp(146)
dp(689) = dp(9)*p(147) + p(9)*dp(147)
dp(690) = dp(9)*p(148) + p(9)*dp(148)
dp(691) = dp(9)*p(182) + p(9)*dp(182)
dp(692) = dp(1)*p(401) + p(1)*dp(401) - dp(604)
dp(693) = dp(21)*p(114) + p(21)*dp(114) - dp(609)
dp(694) = dp(24)*p(101) + p(24)*dp(101) - dp(612)
dp(695) = dp(3)*p(284) + p(3)*dp(284) - dp(606) - dp(694)
dp(696) = dp(1)*p(404) + p(1)*dp(404) - dp(606) - dp(695)
dp(697) = dp(1)*p(405) + p(1)*dp(405) - dp(608) - dp(696)
dp(698) = dp(13)*p(178) + p(13)*dp(178) - dp(604)
dp(699) = dp(39)*p(73) + p(39)*dp(73) - dp(609)
dp(700) = dp(3)*p(285) + p(3)*dp(285) - dp(612)
dp(701) = dp(14)*p(194) + p(14)*dp(194)
dp(702) = dp(9)*p(183) + p(9)*dp(183)
dp(703) = dp(9)*p(151) + p(9)*dp(151)
dp(704) = dp(9)*p(120) + p(9)*dp(120)
dp(705) = dp(9)*p(152) + p(9)*dp(152)
dp(706) = dp(9)*p(121) + p(9)*dp(121)
dp(707) = dp(9)*p(153) + p(9)*dp(153)
dp(708) = dp(9)*p(154) + p(9)*dp(154)
dp(709) = dp(9)*p(155) + p(9)*dp(155)
dp(710) = dp(9)*p(184) + p(9)*dp(184)
dp(711) = dp(24)*p(91) + p(24)*dp(91) - dp(592)
dp(712) = dp(1)*p(416) + p(1)*dp(416) - dp(616) - dp(711)
dp(713) = dp(2)*p(401) + p(2)*dp(401) - dp(613)
dp(714) = dp(1)*p(418) + p(1)*dp(418) - dp(618) - dp(713)
dp(715) = dp(1)*p(419) + p(1)*dp(419) - dp(619) - dp(714)
dp(716) = dp(13)*p(181) + p(13)*dp(181) - dp(701)
dp(717) = dp(2)*p(404) + p(2)*dp(404) - dp(615) - dp(716)
dp(718) = dp(32)*p(91) + p(32)*dp(91) - dp(701)
dp(719) = dp(1)*p(421) + p(1)*dp(421) - dp(619) - dp(717)
dp(720) = dp(41)*p(73) + p(41)*dp(73) - dp(701)
dp(721) = dp(6)*p(279) + p(6)*dp(279) - dp(613)
dp(722) = dp(16)*p(150) + p(16)*dp(150) - dp(701)
dp(723) = dp(1)*p(425) + p(1)*dp(425) - dp(624) - dp(721)
dp(724) = dp(4)*p(285) + p(4)*dp(285) - dp(701)
dp(725) = dp(11)*p(150) + p(11)*dp(150) - dp(592)
dp(726) = dp(14)*p(181) + p(14)*dp(181)
dp(727) = dp(14)*p(182) + p(14)*dp(182)
dp(728) = dp(1)*p(430) + p(1)*dp(430) - dp(632)
dp(729) = dp(21)*p(130) + p(21)*dp(130) - dp(635)
dp(730) = dp(3)*p(295) + p(3)*dp(295) - dp(633) - dp(729)
dp(731) = dp(1)*p(432) + p(1)*dp(432) - dp(633) - dp(730)
dp(732) = dp(39)*p(81) + p(39)*dp(81) - dp(632)
dp(733) = dp(3)*p(296) + p(3)*dp(296) - dp(635)
dp(734) = dp(14)*p(149) + p(14)*dp(149)
dp(735) = dp(14)*p(150) + p(14)*dp(150)
dp(736) = dp(14)*p(119) + p(14)*dp(119)
dp(737) = dp(9)*p(185) + p(9)*dp(185)
dp(738) = dp(9)*p(162) + p(9)*dp(162)
dp(739) = dp(9)*p(134) + p(9)*dp(134)
dp(740) = dp(9)*p(163) + p(9)*dp(163)
dp(741) = dp(9)*p(164) + p(9)*dp(164)
dp(742) = dp(9)*p(186) + p(9)*dp(186)
dp(743) = dp(24)*p(93) + p(24)*dp(93) - dp(593)
dp(744) = dp(18)*p(114) + p(18)*dp(114) - dp(595)
dp(745) = dp(1)*p(443) + p(1)*dp(443) - dp(643) - dp(744)
dp(746) = dp(1)*p(444) + p(1)*dp(444) - dp(644) - dp(745)
dp(747) = dp(13)*p(183) + p(13)*dp(183) - dp(726)
dp(748) = dp(2)*p(421) + p(2)*dp(421) - dp(631) - dp(730)
dp(749) = dp(32)*p(93) + p(32)*dp(93) - dp(726)
dp(750) = dp(1)*p(446) + p(1)*dp(446) - dp(644) - dp(748)
dp(751) = dp(43)*p(73) + p(43)*dp(73) - dp(726)
dp(752) = dp(13)*p(153) + p(13)*dp(153) - dp(595)
dp(753) = dp(18)*p(150) + p(18)*dp(150) - dp(726)
dp(754) = dp(34)*p(73) + p(34)*dp(73) - dp(593)
dp(755) = dp(14)*p(183) + p(14)*dp(183)
dp(756) = dp(14)*p(184) + p(14)*dp(184)
dp(757) = dp(16)*p(130) + p(16)*dp(130) - dp(598)
dp(758) = dp(2)*p(430) + p(2)*dp(430) - dp(636)
dp(759) = dp(1)*p(454) + p(1)*dp(454) - dp(654) - dp(758)
dp(760) = dp(41)*p(81) + p(41)*dp(81) - dp(735)
dp(761) = dp(2)*p(432) + p(2)*dp(432) - dp(638) - dp(760)
dp(762) = dp(16)*p(160) + p(16)*dp(160) - dp(735)
dp(763) = dp(45)*p(69) + p(45)*dp(69) - dp(636)
dp(764) = dp(4)*p(296) + p(4)*dp(296) - dp(735)
dp(765) = dp(11)*p(160) + p(11)*dp(160) - dp(598)
dp(766) = dp(14)*p(156) + p(14)*dp(156)
dp(767) = dp(14)*p(157) + p(14)*dp(157)
dp(768) = dp(14)*p(158) + p(14)*dp(158)
dp(769) = dp(14)*p(128) + p(14)*dp(128)
dp(770) = dp(14)*p(129) + p(14)*dp(129)
dp(771) = dp(1)*p(464) + p(1)*dp(464) - dp(664)
dp(772) = dp(4)*p(262) + p(4)*dp(262) - dp(673) - dp(661)
dp(773) = dp(3)*p(310) + p(3)*dp(310) - dp(664)
dp(774) = dp(14)*p(159) + p(14)*dp(159)
dp(775) = dp(14)*p(160) + p(14)*dp(160)
dp(776) = dp(14)*p(132) + p(14)*dp(132)
dp(777) = dp(14)*p(133) + p(14)*dp(133)
dp(778) = dp(9)*p(188) + p(9)*dp(188)
dp(779) = dp(9)*p(172) + p(9)*dp(172)
dp(780) = dp(9)*p(173) + p(9)*dp(173)
dp(781) = dp(9)*p(189) + p(9)*dp(189)
dp(782) = dp(24)*p(95) + p(24)*dp(95) - dp(600)
dp(783) = dp(22)*p(114) + p(22)*dp(114) - dp(650)
dp(784) = dp(1)*p(471) + p(1)*dp(471) - dp(668) - dp(783)
dp(785) = dp(13)*p(185) + p(13)*dp(185) - dp(755)
dp(786) = dp(2)*p(446) + p(2)*dp(446) - dp(652) - dp(761)
dp(787) = dp(32)*p(95) + p(32)*dp(95) - dp(755)
dp(788) = dp(6)*p(299) + p(6)*dp(299) - dp(650)
dp(789) = dp(46)*p(73) + p(46)*dp(73) - dp(755)
dp(790) = dp(32)*p(84) + p(32)*dp(84) - dp(600)
dp(791) = dp(14)*p(185) + p(14)*dp(185)
dp(792) = dp(14)*p(186) + p(14)*dp(186)
dp(793) = dp(2)*p(453) + p(2)*dp(453) - dp(656) - dp(771)
dp(794) = dp(1)*p(478) + p(1)*dp(478) - dp(673) - dp(793)
dp(795) = dp(43)*p(81) + p(43)*dp(81) - dp(736)
dp(796) = dp(2)*p(456) + p(2)*dp(456) - dp(659) - dp(772)
dp(797) = dp(18)*p(160) + p(18)*dp(160) - dp(736)
dp(798) = dp(1)*p(480) + p(1)*dp(480) - dp(673) - dp(796)
dp(799) = dp(14)*p(165) + p(14)*dp(165)
dp(800) = dp(14)*p(166) + p(14)*dp(166)
dp(801) = dp(14)*p(167) + p(14)*dp(167)
dp(802) = dp(14)*p(138) + p(14)*dp(138)
dp(803) = dp(14)*p(139) + p(14)*dp(139)
dp(804) = dp(2)*p(464) + p(2)*dp(464) - dp(665)
dp(805) = dp(4)*p(310) + p(4)*dp(310) - dp(775)
dp(806) = dp(11)*p(187) + p(11)*dp(187) - dp(665)
dp(807) = dp(14)*p(168) + p(14)*dp(168)
dp(808) = dp(14)*p(169) + p(14)*dp(169)
dp(809) = dp(14)*p(170) + p(14)*dp(170)
dp(810) = dp(9)*p(191) + p(9)*dp(191)
dp(811) = dp(9)*p(192) + p(9)*dp(192)
dp(812) = dp(24)*p(98) + p(24)*dp(98) - dp(603)
dp(813) = dp(1)*p(485) + p(1)*dp(485) - dp(677) - dp(812)
dp(814) = dp(13)*p(188) + p(13)*dp(188) - dp(791)
dp(815) = dp(2)*p(473) + p(2)*dp(473) - dp(672) - dp(796)
dp(816) = dp(32)*p(98) + p(32)*dp(98) - dp(791)
dp(817) = dp(13)*p(173) + p(13)*dp(173) - dp(603)
dp(818) = dp(14)*p(188) + p(14)*dp(188)
dp(819) = dp(14)*p(189) + p(14)*dp(189)
dp(820) = dp(22)*p(130) + p(22)*dp(130) - dp(663)
dp(821) = dp(46)*p(81) + p(46)*dp(81) - dp(769)
dp(822) = dp(45)*p(84) + p(45)*dp(84) - dp(663)
dp(823) = dp(14)*p(174) + p(14)*dp(174)
dp(824) = dp(14)*p(175) + p(14)*dp(175)
dp(825) = dp(14)*p(176) + p(14)*dp(176)
dp(826) = dp(9)*p(195) + p(9)*dp(195)
dp(827) = dp(24)*p(102) + p(24)*dp(102) - dp(679)
dp(828) = dp(13)*p(191) + p(13)*dp(191) - dp(818)
dp(829) = dp(6)*p(323) + p(6)*dp(323) - dp(679)
dp(830) = dp(14)*p(191) + p(14)*dp(191)
dp(831) = dp(14)*p(192) + p(14)*dp(192)
dp(832) = dp(9)*p(140) + p(9)*dp(140)
dp(833) = dp(9)*p(141) + p(9)*dp(141)
dp(834) = dp(9)*p(178) + p(9)*dp(178)
dp(835) = dp(9)*p(142) + p(9)*dp(142)
dp(836) = dp(9)*p(179) + p(9)*dp(179)
dp(837) = dp(9)*p(180) + p(9)*dp(180)
dp(838) = dp(9)*p(194) + p(9)*dp(194)
dp(839) = dp(3)*p(331) + p(3)*dp(331) - dp(680)
dp(840) = dp(39)*p(91) + p(39)*dp(91) - dp(682)
dp(841) = dp(16)*p(178) + p(16)*dp(178) - dp(685)
dp(842) = dp(1)*p(497) + p(1)*dp(497) - dp(683) - dp(841)
dp(843) = dp(2)*p(566) + p(2)*dp(566) - dp(842)
dp(844) = dp(1)*p(498) + p(1)*dp(498) - dp(686) - dp(842)
dp(845) = dp(7)*p(279) + p(7)*dp(279) - dp(685)
dp(846) = dp(1)*p(500) + p(1)*dp(500) - dp(689) - dp(844)
dp(847) = dp(49)*p(69) + p(49)*dp(69) - dp(682)
dp(848) = dp(1)*p(502) + p(1)*dp(502) - dp(691) - dp(846)
dp(849) = dp(11)*p(194) + p(11)*dp(194) - dp(680)
dp(850) = dp(1)*p(504) + p(1)*dp(504) - dp(695) - dp(694)
dp(851) = dp(1)*p(505) + p(1)*dp(505) - dp(700)
dp(852) = dp(3)*p(333) + p(3)*dp(333) - dp(702)
dp(853) = dp(39)*p(93) + p(39)*dp(93) - dp(704)
dp(854) = dp(18)*p(178) + p(18)*dp(178) - dp(707)
dp(855) = dp(1)*p(508) + p(1)*dp(508) - dp(705) - dp(854)
dp(856) = dp(1)*p(509) + p(1)*dp(509) - dp(707)
dp(857) = dp(1)*p(510) + p(1)*dp(510) - dp(708) - dp(855)
dp(858) = dp(21)*p(153) + p(21)*dp(153) - dp(704)
dp(859) = dp(1)*p(512) + p(1)*dp(512) - dp(710) - dp(857)
dp(860) = dp(34)*p(101) + p(34)*dp(101) - dp(702)
dp(861) = dp(6)*p(331) + p(6)*dp(331) - dp(701)
dp(862) = dp(1)*p(515) + p(1)*dp(515) - dp(717) - dp(715)
dp(863) = dp(2)*p(505) + p(2)*dp(505) - dp(701)
dp(864) = dp(1)*p(517) + p(1)*dp(517) - dp(730) - dp(729)
dp(865) = dp(1)*p(518) + p(1)*dp(518) - dp(733)
dp(866) = dp(3)*p(335) + p(3)*dp(335) - dp(737)
dp(867) = dp(39)*p(95) + p(39)*dp(95) - dp(739)
dp(868) = dp(1)*p(520) + p(1)*dp(520) - dp(738) - dp(867)
dp(869) = dp(2)*p(509) + p(2)*dp(509) - dp(721)
dp(870) = dp(1)*p(521) + p(1)*dp(521) - dp(740) - dp(868)
dp(871) = dp(7)*p(299) + p(7)*dp(299) - dp(739)
dp(872) = dp(1)*p(523) + p(1)*dp(523) - dp(742) - dp(870)
dp(873) = dp(49)*p(84) + p(49)*dp(84) - dp(737)
dp(874) = dp(6)*p(333) + p(6)*dp(333) - dp(726)
dp(875) = dp(1)*p(526) + p(1)*dp(526) - dp(748) - dp(746)
dp(876) = dp(8)*p(285) + p(8)*dp(285) - dp(726)
dp(877) = dp(45)*p(91) + p(45)*dp(91) - dp(735)
dp(878) = dp(1)*p(529) + p(1)*dp(529) - dp(761) - dp(759)
dp(879) = dp(2)*p(518) + p(2)*dp(518) - dp(735)
dp(880) = dp(1)*p(531) + p(1)*dp(531) - dp(772) - dp(771)
dp(881) = dp(1)*p(532) + p(1)*dp(532) - dp(773)
dp(882) = dp(14)*p(161) + p(14)*dp(161)
dp(883) = dp(3)*p(337) + p(3)*dp(337) - dp(778)
dp(884) = dp(39)*p(98) + p(39)*dp(98) - dp(780)
dp(885) = dp(1)*p(535) + p(1)*dp(535) - dp(779) - dp(884)
dp(886) = dp(1)*p(536) + p(1)*dp(536) - dp(780)
dp(887) = dp(1)*p(537) + p(1)*dp(537) - dp(781) - dp(885)
dp(888) = dp(21)*p(173) + p(21)*dp(173) - dp(778)
dp(889) = dp(6)*p(335) + p(6)*dp(335) - dp(755)
dp(890) = dp(1)*p(540) + p(1)*dp(540) - dp(786) - dp(784)
dp(891) = dp(22)*p(150) + p(22)*dp(150) - dp(755)
dp(892) = dp(45)*p(93) + p(45)*dp(93) - dp(736)
dp(893) = dp(1)*p(543) + p(1)*dp(543) - dp(796) - dp(794)
dp(894) = dp(8)*p(296) + p(8)*dp(296) - dp(736)
dp(895) = dp(16)*p(187) + p(16)*dp(187) - dp(775)
dp(896) = dp(1)*p(547) + p(1)*dp(547) - dp(806) - dp(804)
dp(897) = dp(2)*p(532) + p(2)*dp(532) - dp(775)
dp(898) = dp(14)*p(171) + p(14)*dp(171)
dp(899) = dp(6)*p(309) + p(6)*dp(309) - dp(808) - dp(807)
dp(900) = dp(1)*p(578) + p(1)*dp(578) - dp(899)
dp(901) = dp(14)*p(187) + p(14)*dp(187)
dp(902) = dp(3)*p(340) + p(3)*dp(340) - dp(810)
dp(903) = dp(2)*p(535) + p(2)*dp(535) - dp(787) - dp(783)
dp(904) = dp(2)*p(536) + p(2)*dp(536) - dp(788)
dp(905) = dp(1)*p(551) + p(1)*dp(551) - dp(811) - dp(903)
dp(906) = dp(7)*p(323) + p(7)*dp(323) - dp(810)
dp(907) = dp(6)*p(337) + p(6)*dp(337) - dp(791)
dp(908) = dp(1)*p(554) + p(1)*dp(554) - dp(815) - dp(813)
dp(909) = dp(50)*p(73) + p(50)*dp(73) - dp(791)
dp(910) = dp(45)*p(95) + p(45)*dp(95) - dp(769)
dp(911) = dp(1)*p(557) + p(1)*dp(557) - dp(822) - dp(820)
dp(912) = dp(22)*p(160) + p(22)*dp(160) - dp(769)
dp(913) = dp(14)*p(177) + p(14)*dp(177)
dp(914) = dp(18)*p(187) + p(18)*dp(187) - dp(777)
dp(915) = dp(2)*p(547) + p(2)*dp(547) - dp(808) - dp(899)
dp(916) = dp(8)*p(310) + p(8)*dp(310) - dp(777)
dp(917) = dp(14)*p(190) + p(14)*dp(190)
dp(918) = dp(3)*p(343) + p(3)*dp(343) - dp(826)
dp(919) = dp(2)*p(551) + p(2)*dp(551) - dp(816) - dp(813)
dp(920) = dp(1)*p(561) + p(1)*dp(561) - dp(826)
dp(921) = dp(6)*p(340) + p(6)*dp(340) - dp(818)
dp(922) = dp(1)*p(563) + p(1)*dp(563) - dp(829) - dp(827)
dp(923) = dp(32)*p(102) + p(32)*dp(102) - dp(818)
dp(924) = dp(45)*p(98) + p(45)*dp(98) - dp(802)
dp(925) = dp(2)*p(557) + p(2)*dp(557) - dp(824) - dp(915)
dp(926) = dp(50)*p(81) + p(50)*dp(81) - dp(802)
dp(927) = dp(14)*p(193) + p(14)*dp(193)
dp(928) = dp(2)*p(560) + p(2)*dp(560) - dp(828) - dp(827)
dp(929) = dp(2)*p(561) + p(2)*dp(561) - dp(829)
dp(930) = dp(6)*p(343) + p(6)*dp(343) - dp(830)
dp(931) = dp(2)*p(563) + p(2)*dp(563) - dp(831) - dp(925)
dp(932) = dp(13)*p(195) + p(13)*dp(195) - dp(830)
dp(933) = dp(14)*p(195) + p(14)*dp(195)
dp(934) = dp(1)*p(566) + p(1)*dp(566) - dp(834)
dp(935) = dp(1)*p(567) + p(1)*dp(567) - dp(836) - dp(934)
dp(936) = dp(39)*p(101) + p(39)*dp(101) - dp(835)
dp(937) = dp(3)*p(346) + p(3)*dp(346) - dp(838)
dp(938) = dp(1)*p(570) + p(1)*dp(570) - dp(839)
dp(939) = dp(2)*p(588) + p(2)*dp(588) - dp(938)
dp(940) = dp(1)*p(572) + p(1)*dp(572) - dp(852)
dp(941) = dp(8)*p(346) + p(8)*dp(346) - dp(940)
dp(942) = dp(1)*p(574) + p(1)*dp(574) - dp(866)
dp(943) = dp(22)*p(194) + p(22)*dp(194) - dp(942)
dp(944) = dp(1)*p(576) + p(1)*dp(576) - dp(883)
dp(945) = dp(50)*p(101) + p(50)*dp(101) - dp(944)
dp(946) = dp(1)*p(579) + p(1)*dp(579) - dp(902)
dp(947) = dp(49)*p(102) + p(49)*dp(102) - dp(946)
dp(948) = dp(2)*p(578) + p(2)*dp(578) - dp(901)
dp(949) = dp(1)*p(582) + p(1)*dp(582) - dp(918)
dp(950) = dp(21)*p(195) + p(21)*dp(195) - dp(949)
dp(951) = dp(2)*p(581) + p(2)*dp(581) - dp(917) - dp(948)
dp(952) = dp(1)*p(585) + p(1)*dp(585) - dp(928)
dp(953) = dp(7)*p(347) + p(7)*dp(347) - dp(952)
dp(954) = dp(45)*p(102) + p(45)*dp(102) - dp(913)
dp(955) = dp(2)*p(585) + p(2)*dp(585) - dp(930)
dp(956) = dp(1)*p(589) + p(1)*dp(589) - dp(955)
dp(957) = dp(6)*p(347) + p(6)*dp(347) - dp(933)
dp(958) = dp(1)*p(588) + p(1)*dp(588) - dp(937)
dp(959) = dp(2)*p(589) + p(2)*dp(589) - dp(957)
dp(960) = dp(9)*p(196) + p(9)*dp(196)
dp(961) = dp(9)*p(202) + p(9)*dp(202)
dp(962) = dp(9)*p(203) + p(9)*dp(203)
dp(963) = dp(9)*p(228) + p(9)*dp(228)
dp(964) = dp(9)*p(242) + p(9)*dp(242)
dp(965) = dp(9)*p(207) + p(9)*dp(207)
dp(966) = dp(9)*p(197) + p(9)*dp(197)
dp(967) = dp(9)*p(208) + p(9)*dp(208)
dp(968) = dp(9)*p(209) + p(9)*dp(209)
dp(969) = dp(9)*p(243) + p(9)*dp(243)
dp(970) = dp(9)*p(211) + p(9)*dp(211)
dp(971) = dp(9)*p(247) + p(9)*dp(247)
dp(972) = dp(9)*p(212) + p(9)*dp(212)
dp(973) = dp(9)*p(248) + p(9)*dp(248)
dp(974) = dp(9)*p(249) + p(9)*dp(249)
dp(975) = dp(9)*p(258) + p(9)*dp(258)
dp(976) = dp(9)*p(214) + p(9)*dp(214)
dp(977) = dp(9)*p(215) + p(9)*dp(215)
dp(978) = dp(9)*p(259) + p(9)*dp(259)
dp(979) = dp(9)*p(263) + p(9)*dp(263)
dp(980) = dp(9)*p(264) + p(9)*dp(264)
dp(981) = dp(9)*p(265) + p(9)*dp(265)
dp(982) = dp(9)*p(308) + p(9)*dp(308)
dp(983) = dp(9)*p(311) + p(9)*dp(311)
dp(984) = dp(9)*p(270) + p(9)*dp(270)
dp(985) = dp(9)*p(271) + p(9)*dp(271)
dp(986) = dp(9)*p(321) + p(9)*dp(321)
dp(987) = dp(9)*p(327) + p(9)*dp(327)
dp(988) = dp(9)*p(222) + p(9)*dp(222)
dp(989) = dp(9)*p(223) + p(9)*dp(223)
dp(990) = dp(9)*p(284) + p(9)*dp(284)
dp(991) = dp(9)*p(198) + p(9)*dp(198)
dp(992) = dp(9)*p(199) + p(9)*dp(199)
dp(993) = dp(9)*p(224) + p(9)*dp(224)
dp(994) = dp(9)*p(200) + p(9)*dp(200)
dp(995) = dp(9)*p(225) + p(9)*dp(225)
dp(996) = dp(9)*p(201) + p(9)*dp(201)
dp(997) = dp(9)*p(226) + p(9)*dp(226)
dp(998) = dp(9)*p(227) + p(9)*dp(227)
dp(999) = dp(9)*p(285) + p(9)*dp(285)
dp(1000) = dp(14)*p(328) + p(14)*dp(328)
dp(1001) = dp(14)*p(329) + p(14)*dp(329)
dp(1002) = dp(14)*p(330) + p(14)*dp(330)
dp(1003) = dp(9)*p(292) + p(9)*dp(292)
dp(1004) = dp(9)*p(233) + p(9)*dp(233)
dp(1005) = dp(9)*p(234) + p(9)*dp(234)
dp(1006) = dp(9)*p(235) + p(9)*dp(235)
dp(1007) = dp(9)*p(293) + p(9)*dp(293)
dp(1008) = dp(9)*p(236) + p(9)*dp(236)
dp(1009) = dp(9)*p(204) + p(9)*dp(204)
dp(1010) = dp(9)*p(205) + p(9)*dp(205)
dp(1011) = dp(9)*p(237) + p(9)*dp(237)
dp(1012) = dp(9)*p(238) + p(9)*dp(238)
dp(1013) = dp(9)*p(206) + p(9)*dp(206)
dp(1014) = dp(9)*p(239) + p(9)*dp(239)
dp(1015) = dp(9)*p(240) + p(9)*dp(240)
dp(1016) = dp(9)*p(241) + p(9)*dp(241)
dp(1017) = dp(9)*p(294) + p(9)*dp(294)
dp(1018) = dp(14)*p(276) + p(14)*dp(276)
dp(1019) = dp(14)*p(277) + p(14)*dp(277)
dp(1020) = dp(14)*p(278) + p(14)*dp(278)
dp(1021) = dp(14)*p(279) + p(14)*dp(279)
dp(1022) = dp(14)*p(280) + p(14)*dp(280)
dp(1023) = dp(14)*p(281) + p(14)*dp(281)
dp(1024) = dp(14)*p(282) + p(14)*dp(282)
dp(1025) = dp(14)*p(283) + p(14)*dp(283)
dp(1026) = dp(9)*p(244) + p(9)*dp(244)
dp(1027) = dp(9)*p(295) + p(9)*dp(295)
dp(1028) = dp(9)*p(210) + p(9)*dp(210)
dp(1029) = dp(9)*p(245) + p(9)*dp(245)
dp(1030) = dp(9)*p(246) + p(9)*dp(246)
dp(1031) = dp(9)*p(296) + p(9)*dp(296)
dp(1032) = dp(14)*p(222) + p(14)*dp(222)
dp(1033) = dp(14)*p(223) + p(14)*dp(223)
dp(1034) = dp(14)*p(224) + p(14)*dp(224)
dp(1035) = dp(14)*p(225) + p(14)*dp(225)
dp(1036) = dp(14)*p(226) + p(14)*dp(226)
dp(1037) = dp(14)*p(227) + p(14)*dp(227)
dp(1038) = dp(14)*p(202) + p(14)*dp(202)
dp(1039) = dp(14)*p(203) + p(14)*dp(203)
dp(1040) = dp(9)*p(302) + p(9)*dp(302)
dp(1041) = dp(9)*p(252) + p(9)*dp(252)
dp(1042) = dp(9)*p(253) + p(9)*dp(253)
dp(1043) = dp(9)*p(303) + p(9)*dp(303)
dp(1044) = dp(9)*p(254) + p(9)*dp(254)
dp(1045) = dp(9)*p(213) + p(9)*dp(213)
dp(1046) = dp(9)*p(255) + p(9)*dp(255)
dp(1047) = dp(9)*p(256) + p(9)*dp(256)
dp(1048) = dp(9)*p(257) + p(9)*dp(257)
dp(1049) = dp(9)*p(304) + p(9)*dp(304)
dp(1050) = dp(14)*p(286) + p(14)*dp(286)
dp(1051) = dp(14)*p(287) + p(14)*dp(287)
dp(1052) = dp(14)*p(288) + p(14)*dp(288)
dp(1053) = dp(14)*p(289) + p(14)*dp(289)
dp(1054) = dp(14)*p(290) + p(14)*dp(290)
dp(1055) = dp(14)*p(291) + p(14)*dp(291)
dp(1056) = dp(9)*p(305) + p(9)*dp(305)
dp(1057) = dp(9)*p(260) + p(9)*dp(260)
dp(1058) = dp(9)*p(306) + p(9)*dp(306)
dp(1059) = dp(9)*p(261) + p(9)*dp(261)
dp(1060) = dp(9)*p(262) + p(9)*dp(262)
dp(1061) = dp(9)*p(307) + p(9)*dp(307)
dp(1062) = dp(14)*p(233) + p(14)*dp(233)
dp(1063) = dp(14)*p(234) + p(14)*dp(234)
dp(1064) = dp(14)*p(235) + p(14)*dp(235)
dp(1065) = dp(14)*p(236) + p(14)*dp(236)
dp(1066) = dp(14)*p(237) + p(14)*dp(237)
dp(1067) = dp(14)*p(238) + p(14)*dp(238)
dp(1068) = dp(14)*p(239) + p(14)*dp(239)
dp(1069) = dp(14)*p(240) + p(14)*dp(240)
dp(1070) = dp(14)*p(241) + p(14)*dp(241)
dp(1071) = dp(14)*p(207) + p(14)*dp(207)
dp(1072) = dp(14)*p(208) + p(14)*dp(208)
dp(1073) = dp(14)*p(209) + p(14)*dp(209)
dp(1074) = dp(9)*p(309) + p(9)*dp(309)
dp(1075) = dp(9)*p(310) + p(9)*dp(310)
dp(1076) = dp(14)*p(244) + p(14)*dp(244)
dp(1077) = dp(14)*p(245) + p(14)*dp(245)
dp(1078) = dp(14)*p(246) + p(14)*dp(246)
dp(1079) = dp(14)*p(211) + p(14)*dp(211)
dp(1080) = dp(14)*p(212) + p(14)*dp(212)
dp(1081) = dp(9)*p(315) + p(9)*dp(315)
dp(1082) = dp(9)*p(267) + p(9)*dp(267)
dp(1083) = dp(9)*p(316) + p(9)*dp(316)
dp(1084) = dp(9)*p(268) + p(9)*dp(268)
dp(1085) = dp(9)*p(269) + p(9)*dp(269)
dp(1086) = dp(9)*p(317) + p(9)*dp(317)
dp(1087) = dp(14)*p(297) + p(14)*dp(297)
dp(1088) = dp(14)*p(298) + p(14)*dp(298)
dp(1089) = dp(14)*p(299) + p(14)*dp(299)
dp(1090) = dp(14)*p(300) + p(14)*dp(300)
dp(1091) = dp(14)*p(301) + p(14)*dp(301)
dp(1092) = dp(9)*p(318) + p(9)*dp(318)
dp(1093) = dp(9)*p(319) + p(9)*dp(319)
dp(1094) = dp(9)*p(320) + p(9)*dp(320)
dp(1095) = dp(14)*p(252) + p(14)*dp(252)
dp(1096) = dp(14)*p(253) + p(14)*dp(253)
dp(1097) = dp(14)*p(254) + p(14)*dp(254)
dp(1098) = dp(14)*p(255) + p(14)*dp(255)
dp(1099) = dp(14)*p(256) + p(14)*dp(256)
dp(1100) = dp(14)*p(257) + p(14)*dp(257)
dp(1101) = dp(14)*p(214) + p(14)*dp(214)
dp(1102) = dp(14)*p(215) + p(14)*dp(215)
dp(1103) = dp(9)*p(339) + p(9)*dp(339)
dp(1104) = dp(14)*p(260) + p(14)*dp(260)
dp(1105) = dp(14)*p(261) + p(14)*dp(261)
dp(1106) = dp(14)*p(262) + p(14)*dp(262)
dp(1107) = dp(9)*p(324) + p(9)*dp(324)
dp(1108) = dp(9)*p(325) + p(9)*dp(325)
dp(1109) = dp(9)*p(326) + p(9)*dp(326)
dp(1110) = dp(14)*p(312) + p(14)*dp(312)
dp(1111) = dp(14)*p(313) + p(14)*dp(313)
dp(1112) = dp(14)*p(314) + p(14)*dp(314)
dp(1113) = dp(9)*p(342) + p(9)*dp(342)
dp(1114) = dp(14)*p(267) + p(14)*dp(267)
dp(1115) = dp(14)*p(268) + p(14)*dp(268)
dp(1116) = dp(14)*p(269) + p(14)*dp(269)
dp(1117) = dp(9)*p(345) + p(9)*dp(345)
dp(1118) = dp(14)*p(322) + p(14)*dp(322)
dp(1119) = dp(14)*p(323) + p(14)*dp(323)
dp(1120) = dp(9)*p(331) + p(9)*dp(331)
dp(1121) = dp(9)*p(276) + p(9)*dp(276)
dp(1122) = dp(9)*p(216) + p(9)*dp(216)
dp(1123) = dp(9)*p(277) + p(9)*dp(277)
dp(1124) = dp(9)*p(217) + p(9)*dp(217)
dp(1125) = dp(9)*p(218) + p(9)*dp(218)
dp(1126) = dp(9)*p(278) + p(9)*dp(278)
dp(1127) = dp(9)*p(219) + p(9)*dp(219)
dp(1128) = dp(9)*p(220) + p(9)*dp(220)
dp(1129) = dp(9)*p(279) + p(9)*dp(279)
dp(1130) = dp(9)*p(280) + p(9)*dp(280)
dp(1131) = dp(9)*p(221) + p(9)*dp(221)
dp(1132) = dp(9)*p(281) + p(9)*dp(281)
dp(1133) = dp(9)*p(282) + p(9)*dp(282)
dp(1134) = dp(9)*p(283) + p(9)*dp(283)
dp(1135) = dp(9)*p(332) + p(9)*dp(332)
dp(1136) = dp(3)*p(401) + p(3)*dp(401) - dp(988)
dp(1137) = dp(7)*p(401) + p(7)*dp(401) - dp(991)
dp(1138) = dp(49)*p(114) + p(49)*dp(114) - dp(996)
dp(1139) = dp(24)*p(194) + p(24)*dp(194) - dp(999)
dp(1140) = dp(3)*p(504) + p(3)*dp(504) - dp(990) - dp(1139)
dp(1141) = dp(1)*p(695) + p(1)*dp(695) - dp(990) - dp(1140)
dp(1142) = dp(1)*p(696) + p(1)*dp(696) - dp(993) - dp(1141)
dp(1143) = dp(6)*p(566) + p(6)*dp(566) - dp(1136)
dp(1144) = dp(32)*p(178) + p(32)*dp(178) - dp(991)
dp(1145) = dp(39)*p(150) + p(39)*dp(150) - dp(996)
dp(1146) = dp(3)*p(505) + p(3)*dp(505) - dp(999)
dp(1147) = dp(14)*p(346) + p(14)*dp(346)
dp(1148) = dp(9)*p(333) + p(9)*dp(333)
dp(1149) = dp(9)*p(286) + p(9)*dp(286)
dp(1150) = dp(9)*p(229) + p(9)*dp(229)
dp(1151) = dp(9)*p(287) + p(9)*dp(287)
dp(1152) = dp(9)*p(230) + p(9)*dp(230)
dp(1153) = dp(9)*p(231) + p(9)*dp(231)
dp(1154) = dp(9)*p(288) + p(9)*dp(288)
dp(1155) = dp(9)*p(232) + p(9)*dp(232)
dp(1156) = dp(9)*p(289) + p(9)*dp(289)
dp(1157) = dp(9)*p(290) + p(9)*dp(290)
dp(1158) = dp(9)*p(291) + p(9)*dp(291)
dp(1159) = dp(9)*p(334) + p(9)*dp(334)
dp(1160) = dp(24)*p(181) + p(24)*dp(181) - dp(963)
dp(1161) = dp(41)*p(114) + p(41)*dp(114) - dp(960)
dp(1162) = dp(4)*p(401) + p(4)*dp(401) - dp(961)
dp(1163) = dp(1)*p(713) + p(1)*dp(713) - dp(1005) - dp(1162)
dp(1164) = dp(1)*p(714) + p(1)*dp(714) - dp(1006) - dp(1163)
dp(1165) = dp(1)*p(715) + p(1)*dp(715) - dp(1007) - dp(1164)
dp(1166) = dp(13)*p(331) + p(13)*dp(331) - dp(1147)
dp(1167) = dp(2)*p(695) + p(2)*dp(695) - dp(1002) - dp(1166)
dp(1168) = dp(32)*p(181) + p(32)*dp(181) - dp(1147)
dp(1169) = dp(1)*p(717) + p(1)*dp(717) - dp(1007) - dp(1167)
dp(1170) = dp(73)*p(91) + p(73)*dp(91) - dp(1147)
dp(1171) = dp(1)*p(719) + p(1)*dp(719) - dp(1011) - dp(1169)
dp(1172) = dp(41)*p(150) + p(41)*dp(150) - dp(1147)
dp(1173) = dp(13)*p(279) + p(13)*dp(279) - dp(961)
dp(1174) = dp(16)*p(285) + p(16)*dp(285) - dp(1147)
dp(1175) = dp(69)*p(73) + p(69)*dp(73) - dp(960)
dp(1176) = dp(4)*p(505) + p(4)*dp(505) - dp(1147)
dp(1177) = dp(11)*p(285) + p(11)*dp(285) - dp(963)
dp(1178) = dp(14)*p(331) + p(14)*dp(331)
dp(1179) = dp(14)*p(332) + p(14)*dp(332)
dp(1180) = dp(3)*p(430) + p(3)*dp(430) - dp(1026)
dp(1181) = dp(7)*p(430) + p(7)*dp(430) - dp(1028)
dp(1182) = dp(49)*p(130) + p(49)*dp(130) - dp(1031)
dp(1183) = dp(3)*p(517) + p(3)*dp(517) - dp(1027) - dp(1182)
dp(1184) = dp(1)*p(730) + p(1)*dp(730) - dp(1027) - dp(1183)
dp(1185) = dp(45)*p(178) + p(45)*dp(178) - dp(1180)
dp(1186) = dp(39)*p(160) + p(39)*dp(160) - dp(1028)
dp(1187) = dp(3)*p(518) + p(3)*dp(518) - dp(1031)
dp(1188) = dp(14)*p(284) + p(14)*dp(284)
dp(1189) = dp(14)*p(285) + p(14)*dp(285)
dp(1190) = dp(14)*p(228) + p(14)*dp(228)
dp(1191) = dp(9)*p(335) + p(9)*dp(335)
dp(1192) = dp(9)*p(297) + p(9)*dp(297)
dp(1193) = dp(9)*p(250) + p(9)*dp(250)
dp(1194) = dp(9)*p(298) + p(9)*dp(298)
dp(1195) = dp(9)*p(251) + p(9)*dp(251)
dp(1196) = dp(9)*p(299) + p(9)*dp(299)
dp(1197) = dp(9)*p(300) + p(9)*dp(300)
dp(1198) = dp(9)*p(301) + p(9)*dp(301)
dp(1199) = dp(9)*p(336) + p(9)*dp(336)
dp(1200) = dp(24)*p(183) + p(24)*dp(183) - dp(964)
dp(1201) = dp(43)*p(114) + p(43)*dp(114) - dp(966)
dp(1202) = dp(8)*p(401) + p(8)*dp(401) - dp(1021)
dp(1203) = dp(1)*p(745) + p(1)*dp(745) - dp(1042) - dp(1202)
dp(1204) = dp(1)*p(746) + p(1)*dp(746) - dp(1043) - dp(1203)
dp(1205) = dp(13)*p(333) + p(13)*dp(333) - dp(1178)
dp(1206) = dp(2)*p(717) + p(2)*dp(717) - dp(1025) - dp(1183)
dp(1207) = dp(32)*p(183) + p(32)*dp(183) - dp(1178)
dp(1208) = dp(1)*p(748) + p(1)*dp(748) - dp(1043) - dp(1206)
dp(1209) = dp(73)*p(93) + p(73)*dp(93) - dp(1178)
dp(1210) = dp(6)*p(509) + p(6)*dp(509) - dp(1021)
dp(1211) = dp(43)*p(150) + p(43)*dp(150) - dp(1178)
dp(1212) = dp(32)*p(153) + p(32)*dp(153) - dp(966)
dp(1213) = dp(18)*p(285) + p(18)*dp(285) - dp(1178)
dp(1214) = dp(34)*p(150) + p(34)*dp(150) - dp(964)
dp(1215) = dp(14)*p(333) + p(14)*dp(333)
dp(1216) = dp(14)*p(334) + p(14)*dp(334)
dp(1217) = dp(41)*p(130) + p(41)*dp(130) - dp(973)
dp(1218) = dp(4)*p(430) + p(4)*dp(430) - dp(970)
dp(1219) = dp(1)*p(758) + p(1)*dp(758) - dp(1057) - dp(1218)
dp(1220) = dp(1)*p(759) + p(1)*dp(759) - dp(1058) - dp(1219)
dp(1221) = dp(81)*p(91) + p(81)*dp(91) - dp(1189)
dp(1222) = dp(2)*p(730) + p(2)*dp(730) - dp(1034) - dp(1221)
dp(1223) = dp(41)*p(160) + p(41)*dp(160) - dp(1189)
dp(1224) = dp(1)*p(761) + p(1)*dp(761) - dp(1058) - dp(1222)
dp(1225) = dp(16)*p(296) + p(16)*dp(296) - dp(1189)
dp(1226) = dp(69)*p(81) + p(69)*dp(81) - dp(970)
dp(1227) = dp(4)*p(518) + p(4)*dp(518) - dp(1189)
dp(1228) = dp(11)*p(296) + p(11)*dp(296) - dp(973)
dp(1229) = dp(14)*p(292) + p(14)*dp(292)
dp(1230) = dp(14)*p(293) + p(14)*dp(293)
dp(1231) = dp(14)*p(294) + p(14)*dp(294)
dp(1232) = dp(14)*p(242) + p(14)*dp(242)
dp(1233) = dp(14)*p(243) + p(14)*dp(243)
dp(1234) = dp(3)*p(464) + p(3)*dp(464) - dp(1074)
dp(1235) = dp(7)*p(464) + p(7)*dp(464) - dp(1075)
dp(1236) = dp(3)*p(531) + p(3)*dp(531) - dp(1074) - dp(1235)
dp(1237) = dp(39)*p(187) + p(39)*dp(187) - dp(1234)
dp(1238) = dp(3)*p(532) + p(3)*dp(532) - dp(1075)
dp(1239) = dp(14)*p(295) + p(14)*dp(295)
dp(1240) = dp(14)*p(296) + p(14)*dp(296)
dp(1241) = dp(14)*p(247) + p(14)*dp(247)
dp(1242) = dp(14)*p(248) + p(14)*dp(248)
dp(1243) = dp(14)*p(249) + p(14)*dp(249)
dp(1244) = dp(9)*p(337) + p(9)*dp(337)
dp(1245) = dp(9)*p(312) + p(9)*dp(312)
dp(1246) = dp(9)*p(266) + p(9)*dp(266)
dp(1247) = dp(9)*p(313) + p(9)*dp(313)
dp(1248) = dp(9)*p(314) + p(9)*dp(314)
dp(1249) = dp(9)*p(338) + p(9)*dp(338)
dp(1250) = dp(24)*p(185) + p(24)*dp(185) - dp(975)
dp(1251) = dp(46)*p(114) + p(46)*dp(114) - dp(977)
dp(1252) = dp(1)*p(783) + p(1)*dp(783) - dp(1082) - dp(1251)
dp(1253) = dp(1)*p(784) + p(1)*dp(784) - dp(1083) - dp(1252)
dp(1254) = dp(13)*p(335) + p(13)*dp(335) - dp(1215)
dp(1255) = dp(2)*p(748) + p(2)*dp(748) - dp(1055) - dp(1222)
dp(1256) = dp(32)*p(185) + p(32)*dp(185) - dp(1215)
dp(1257) = dp(1)*p(786) + p(1)*dp(786) - dp(1083) - dp(1255)
dp(1258) = dp(73)*p(95) + p(73)*dp(95) - dp(1215)
dp(1259) = dp(13)*p(299) + p(13)*dp(299) - dp(977)
dp(1260) = dp(46)*p(150) + p(46)*dp(150) - dp(1215)
dp(1261) = dp(73)*p(84) + p(73)*dp(84) - dp(975)
dp(1262) = dp(14)*p(335) + p(14)*dp(335)
dp(1263) = dp(14)*p(336) + p(14)*dp(336)
dp(1264) = dp(43)*p(130) + p(43)*dp(130) - dp(974)
dp(1265) = dp(8)*p(430) + p(8)*dp(430) - dp(1038)
dp(1266) = dp(1)*p(794) + p(1)*dp(794) - dp(1093) - dp(1265)
dp(1267) = dp(81)*p(93) + p(81)*dp(93) - dp(1190)
dp(1268) = dp(2)*p(761) + p(2)*dp(761) - dp(1066) - dp(1236)
dp(1269) = dp(43)*p(160) + p(43)*dp(160) - dp(1190)
dp(1270) = dp(45)*p(153) + p(45)*dp(153) - dp(1038)
dp(1271) = dp(18)*p(296) + p(18)*dp(296) - dp(1190)
dp(1272) = dp(34)*p(160) + p(34)*dp(160) - dp(974)
dp(1273) = dp(14)*p(302) + p(14)*dp(302)
dp(1274) = dp(14)*p(303) + p(14)*dp(303)
dp(1275) = dp(14)*p(304) + p(14)*dp(304)
dp(1276) = dp(14)*p(258) + p(14)*dp(258)
dp(1277) = dp(14)*p(259) + p(14)*dp(259)
dp(1278) = dp(4)*p(464) + p(4)*dp(464) - dp(983)
dp(1279) = dp(1)*p(804) + p(1)*dp(804) - dp(1103) - dp(1278)
dp(1280) = dp(16)*p(310) + p(16)*dp(310) - dp(1240)
dp(1281) = dp(2)*p(772) + p(2)*dp(772) - dp(1077) - dp(1280)
dp(1282) = dp(4)*p(532) + p(4)*dp(532) - dp(1240)
dp(1283) = dp(11)*p(310) + p(11)*dp(310) - dp(983)
dp(1284) = dp(14)*p(305) + p(14)*dp(305)
dp(1285) = dp(14)*p(306) + p(14)*dp(306)
dp(1286) = dp(14)*p(307) + p(14)*dp(307)
dp(1287) = dp(14)*p(263) + p(14)*dp(263)
dp(1288) = dp(14)*p(264) + p(14)*dp(264)
dp(1289) = dp(14)*p(265) + p(14)*dp(265)
dp(1290) = dp(6)*p(464) + p(6)*dp(464) - dp(1104)
dp(1291) = dp(3)*p(578) + p(3)*dp(578) - dp(1290)
dp(1292) = dp(14)*p(309) + p(14)*dp(309)
dp(1293) = dp(14)*p(310) + p(14)*dp(310)
dp(1294) = dp(9)*p(340) + p(9)*dp(340)
dp(1295) = dp(9)*p(322) + p(9)*dp(322)
dp(1296) = dp(9)*p(323) + p(9)*dp(323)
dp(1297) = dp(9)*p(341) + p(9)*dp(341)
dp(1298) = dp(24)*p(188) + p(24)*dp(188) - dp(984)
dp(1299) = dp(50)*p(114) + p(50)*dp(114) - dp(1089)
dp(1300) = dp(1)*p(813) + p(1)*dp(813) - dp(1108) - dp(1299)
dp(1301) = dp(13)*p(337) + p(13)*dp(337) - dp(1262)
dp(1302) = dp(2)*p(786) + p(2)*dp(786) - dp(1091) - dp(1268)
dp(1303) = dp(32)*p(188) + p(32)*dp(188) - dp(1262)
dp(1304) = dp(6)*p(536) + p(6)*dp(536) - dp(1089)
dp(1305) = dp(73)*p(98) + p(73)*dp(98) - dp(1262)
dp(1306) = dp(32)*p(173) + p(32)*dp(173) - dp(984)
dp(1307) = dp(14)*p(337) + p(14)*dp(337)
dp(1308) = dp(14)*p(338) + p(14)*dp(338)
dp(1309) = dp(46)*p(130) + p(46)*dp(130) - dp(982)
dp(1310) = dp(1)*p(820) + p(1)*dp(820) - dp(1113) - dp(1309)
dp(1311) = dp(81)*p(95) + p(81)*dp(95) - dp(1232)
dp(1312) = dp(2)*p(796) + p(2)*dp(796) - dp(1098) - dp(1281)
dp(1313) = dp(46)*p(160) + p(46)*dp(160) - dp(1232)
dp(1314) = dp(81)*p(84) + p(81)*dp(84) - dp(982)
dp(1315) = dp(14)*p(315) + p(14)*dp(315)
dp(1316) = dp(14)*p(316) + p(14)*dp(316)
dp(1317) = dp(14)*p(317) + p(14)*dp(317)
dp(1318) = dp(14)*p(270) + p(14)*dp(270)
dp(1319) = dp(14)*p(271) + p(14)*dp(271)
dp(1320) = dp(8)*p(464) + p(8)*dp(464) - dp(1079)
dp(1321) = dp(18)*p(310) + p(18)*dp(310) - dp(1242)
dp(1322) = dp(34)*p(187) + p(34)*dp(187) - dp(1079)
dp(1323) = dp(14)*p(318) + p(14)*dp(318)
dp(1324) = dp(14)*p(319) + p(14)*dp(319)
dp(1325) = dp(14)*p(320) + p(14)*dp(320)
dp(1326) = dp(9)*p(343) + p(9)*dp(343)
dp(1327) = dp(9)*p(344) + p(9)*dp(344)
dp(1328) = dp(24)*p(191) + p(24)*dp(191) - dp(987)
dp(1329) = dp(1)*p(827) + p(1)*dp(827) - dp(1117) - dp(1328)
dp(1330) = dp(13)*p(340) + p(13)*dp(340) - dp(1307)
dp(1331) = dp(2)*p(815) + p(2)*dp(815) - dp(1112) - dp(1312)
dp(1332) = dp(32)*p(191) + p(32)*dp(191) - dp(1307)
dp(1333) = dp(13)*p(323) + p(13)*dp(323) - dp(987)
dp(1334) = dp(14)*p(340) + p(14)*dp(340)
dp(1335) = dp(14)*p(341) + p(14)*dp(341)
dp(1336) = dp(50)*p(130) + p(50)*dp(130) - dp(1102)
dp(1337) = dp(81)*p(98) + p(81)*dp(98) - dp(1276)
dp(1338) = dp(45)*p(173) + p(45)*dp(173) - dp(1102)
dp(1339) = dp(14)*p(324) + p(14)*dp(324)
dp(1340) = dp(14)*p(325) + p(14)*dp(325)
dp(1341) = dp(14)*p(326) + p(14)*dp(326)
dp(1342) = dp(9)*p(347) + p(9)*dp(347)
dp(1343) = dp(24)*p(195) + p(24)*dp(195) - dp(1119)
dp(1344) = dp(13)*p(343) + p(13)*dp(343) - dp(1334)
dp(1345) = dp(6)*p(561) + p(6)*dp(561) - dp(1119)
dp(1346) = dp(14)*p(343) + p(14)*dp(343)
dp(1347) = dp(14)*p(344) + p(14)*dp(344)
dp(1348) = dp(9)*p(272) + p(9)*dp(272)
dp(1349) = dp(9)*p(273) + p(9)*dp(273)
dp(1350) = dp(9)*p(274) + p(9)*dp(274)
dp(1351) = dp(9)*p(328) + p(9)*dp(328)
dp(1352) = dp(9)*p(275) + p(9)*dp(275)
dp(1353) = dp(9)*p(329) + p(9)*dp(329)
dp(1354) = dp(9)*p(330) + p(9)*dp(330)
dp(1355) = dp(9)*p(346) + p(9)*dp(346)
dp(1356) = dp(3)*p(570) + p(3)*dp(570) - dp(1120)
dp(1357) = dp(39)*p(181) + p(39)*dp(181) - dp(1122)
dp(1358) = dp(41)*p(178) + p(41)*dp(178) - dp(1125)
dp(1359) = dp(4)*p(566) + p(4)*dp(566) - dp(1129)
dp(1360) = dp(1)*p(842) + p(1)*dp(842) - dp(1126) - dp(1359)
dp(1361) = dp(1)*p(843) + p(1)*dp(843) - dp(1129)
dp(1362) = dp(1)*p(844) + p(1)*dp(844) - dp(1130) - dp(1360)
dp(1363) = dp(21)*p(279) + p(21)*dp(279) - dp(1125)
dp(1364) = dp(1)*p(846) + p(1)*dp(846) - dp(1133) - dp(1362)
dp(1365) = dp(69)*p(101) + p(69)*dp(101) - dp(1122)
dp(1366) = dp(1)*p(848) + p(1)*dp(848) - dp(1135) - dp(1364)
dp(1367) = dp(11)*p(346) + p(11)*dp(346) - dp(1120)
dp(1368) = dp(1)*p(850) + p(1)*dp(850) - dp(1140) - dp(1139)
dp(1369) = dp(1)*p(851) + p(1)*dp(851) - dp(1146)
dp(1370) = dp(3)*p(572) + p(3)*dp(572) - dp(1148)
dp(1371) = dp(39)*p(183) + p(39)*dp(183) - dp(1150)
dp(1372) = dp(43)*p(178) + p(43)*dp(178) - dp(1153)
dp(1373) = dp(1)*p(854) + p(1)*dp(854) - dp(1151) - dp(1372)
dp(1374) = dp(2)*p(843) + p(2)*dp(843) - dp(1143)
dp(1375) = dp(1)*p(855) + p(1)*dp(855) - dp(1154) - dp(1373)
dp(1376) = dp(7)*p(509) + p(7)*dp(509) - dp(1153)
dp(1377) = dp(1)*p(857) + p(1)*dp(857) - dp(1157) - dp(1375)
dp(1378) = dp(49)*p(153) + p(49)*dp(153) - dp(1150)
dp(1379) = dp(1)*p(859) + p(1)*dp(859) - dp(1159) - dp(1377)
dp(1380) = dp(34)*p(194) + p(34)*dp(194) - dp(1148)
dp(1381) = dp(6)*p(570) + p(6)*dp(570) - dp(1147)
dp(1382) = dp(1)*p(862) + p(1)*dp(862) - dp(1167) - dp(1165)
dp(1383) = dp(2)*p(851) + p(2)*dp(851) - dp(1147)
dp(1384) = dp(1)*p(864) + p(1)*dp(864) - dp(1183) - dp(1182)
dp(1385) = dp(1)*p(865) + p(1)*dp(865) - dp(1187)
dp(1386) = dp(3)*p(574) + p(3)*dp(574) - dp(1191)
dp(1387) = dp(39)*p(185) + p(39)*dp(185) - dp(1193)
dp(1388) = dp(46)*p(178) + p(46)*dp(178) - dp(1196)
dp(1389) = dp(1)*p(868) + p(1)*dp(868) - dp(1194) - dp(1388)
dp(1390) = dp(1)*p(869) + p(1)*dp(869) - dp(1196)
dp(1391) = dp(1)*p(870) + p(1)*dp(870) - dp(1197) - dp(1389)
dp(1392) = dp(21)*p(299) + p(21)*dp(299) - dp(1193)
dp(1393) = dp(1)*p(872) + p(1)*dp(872) - dp(1199) - dp(1391)
dp(1394) = dp(84)*p(101) + p(84)*dp(101) - dp(1191)
dp(1395) = dp(6)*p(572) + p(6)*dp(572) - dp(1178)
dp(1396) = dp(1)*p(875) + p(1)*dp(875) - dp(1206) - dp(1204)
dp(1397) = dp(8)*p(505) + p(8)*dp(505) - dp(1178)
dp(1398) = dp(45)*p(181) + p(45)*dp(181) - dp(1189)
dp(1399) = dp(1)*p(878) + p(1)*dp(878) - dp(1222) - dp(1220)
dp(1400) = dp(2)*p(865) + p(2)*dp(865) - dp(1189)
dp(1401) = dp(1)*p(880) + p(1)*dp(880) - dp(1236) - dp(1235)
dp(1402) = dp(1)*p(881) + p(1)*dp(881) - dp(1238)
dp(1403) = dp(3)*p(576) + p(3)*dp(576) - dp(1244)
dp(1404) = dp(39)*p(188) + p(39)*dp(188) - dp(1246)
dp(1405) = dp(1)*p(884) + p(1)*dp(884) - dp(1245) - dp(1404)
dp(1406) = dp(2)*p(869) + p(2)*dp(869) - dp(1210)
dp(1407) = dp(1)*p(885) + p(1)*dp(885) - dp(1247) - dp(1405)
dp(1408) = dp(7)*p(536) + p(7)*dp(536) - dp(1246)
dp(1409) = dp(1)*p(887) + p(1)*dp(887) - dp(1249) - dp(1407)
dp(1410) = dp(49)*p(173) + p(49)*dp(173) - dp(1244)
dp(1411) = dp(6)*p(574) + p(6)*dp(574) - dp(1215)
dp(1412) = dp(1)*p(890) + p(1)*dp(890) - dp(1255) - dp(1253)
dp(1413) = dp(22)*p(285) + p(22)*dp(285) - dp(1215)
dp(1414) = dp(45)*p(183) + p(45)*dp(183) - dp(1190)
dp(1415) = dp(1)*p(893) + p(1)*dp(893) - dp(1268) - dp(1266)
dp(1416) = dp(8)*p(518) + p(8)*dp(518) - dp(1190)
dp(1417) = dp(41)*p(187) + p(41)*dp(187) - dp(1240)
dp(1418) = dp(1)*p(896) + p(1)*dp(896) - dp(1281) - dp(1279)
dp(1419) = dp(2)*p(881) + p(2)*dp(881) - dp(1240)
dp(1420) = dp(14)*p(308) + p(14)*dp(308)
dp(1421) = dp(4)*p(547) + p(4)*dp(547) - dp(1106) - dp(1320)
dp(1422) = dp(1)*p(900) + p(1)*dp(900) - dp(1291)
dp(1423) = dp(14)*p(311) + p(14)*dp(311)
dp(1424) = dp(3)*p(579) + p(3)*dp(579) - dp(1294)
dp(1425) = dp(39)*p(191) + p(39)*dp(191) - dp(1296)
dp(1426) = dp(1)*p(903) + p(1)*dp(903) - dp(1295) - dp(1425)
dp(1427) = dp(1)*p(904) + p(1)*dp(904) - dp(1296)
dp(1428) = dp(1)*p(905) + p(1)*dp(905) - dp(1297) - dp(1426)
dp(1429) = dp(21)*p(323) + p(21)*dp(323) - dp(1294)
dp(1430) = dp(6)*p(576) + p(6)*dp(576) - dp(1262)
dp(1431) = dp(1)*p(908) + p(1)*dp(908) - dp(1302) - dp(1300)
dp(1432) = dp(50)*p(150) + p(50)*dp(150) - dp(1262)
dp(1433) = dp(45)*p(185) + p(45)*dp(185) - dp(1232)
dp(1434) = dp(1)*p(911) + p(1)*dp(911) - dp(1312) - dp(1310)
dp(1435) = dp(22)*p(296) + p(22)*dp(296) - dp(1232)
dp(1436) = dp(43)*p(187) + p(43)*dp(187) - dp(1242)
dp(1437) = dp(1)*p(915) + p(1)*dp(915) - dp(1322) - dp(1320)
dp(1438) = dp(8)*p(532) + p(8)*dp(532) - dp(1242)
dp(1439) = dp(14)*p(321) + p(14)*dp(321)
dp(1440) = dp(4)*p(578) + p(4)*dp(578) - dp(1293)
dp(1441) = dp(2)*p(899) + p(2)*dp(899) - dp(1292) - dp(1440)
dp(1442) = dp(2)*p(900) + p(2)*dp(900) - dp(1293)
dp(1443) = dp(14)*p(339) + p(14)*dp(339)
dp(1444) = dp(3)*p(582) + p(3)*dp(582) - dp(1326)
dp(1445) = dp(2)*p(903) + p(2)*dp(903) - dp(1303) - dp(1299)
dp(1446) = dp(2)*p(904) + p(2)*dp(904) - dp(1304)
dp(1447) = dp(1)*p(919) + p(1)*dp(919) - dp(1327) - dp(1445)
dp(1448) = dp(7)*p(561) + p(7)*dp(561) - dp(1326)
dp(1449) = dp(6)*p(579) + p(6)*dp(579) - dp(1307)
dp(1450) = dp(1)*p(922) + p(1)*dp(922) - dp(1331) - dp(1329)
dp(1451) = dp(73)*p(102) + p(73)*dp(102) - dp(1307)
dp(1452) = dp(45)*p(188) + p(45)*dp(188) - dp(1276)
dp(1453) = dp(1)*p(925) + p(1)*dp(925) - dp(1338) - dp(1336)
dp(1454) = dp(50)*p(160) + p(50)*dp(160) - dp(1276)
dp(1455) = dp(14)*p(327) + p(14)*dp(327)
dp(1456) = dp(46)*p(187) + p(46)*dp(187) - dp(1243)
dp(1457) = dp(2)*p(915) + p(2)*dp(915) - dp(1324) - dp(1441)
dp(1458) = dp(22)*p(310) + p(22)*dp(310) - dp(1243)
dp(1459) = dp(14)*p(342) + p(14)*dp(342)
dp(1460) = dp(3)*p(585) + p(3)*dp(585) - dp(1342)
dp(1461) = dp(2)*p(919) + p(2)*dp(919) - dp(1332) - dp(1329)
dp(1462) = dp(1)*p(929) + p(1)*dp(929) - dp(1342)
dp(1463) = dp(6)*p(582) + p(6)*dp(582) - dp(1334)
dp(1464) = dp(1)*p(931) + p(1)*dp(931) - dp(1345) - dp(1343)
dp(1465) = dp(32)*p(195) + p(32)*dp(195) - dp(1334)
dp(1466) = dp(45)*p(191) + p(45)*dp(191) - dp(1318)
dp(1467) = dp(2)*p(925) + p(2)*dp(925) - dp(1340) - dp(1457)
dp(1468) = dp(81)*p(102) + p(81)*dp(102) - dp(1318)
dp(1469) = dp(14)*p(345) + p(14)*dp(345)
dp(1470) = dp(2)*p(928) + p(2)*dp(928) - dp(1344) - dp(1343)
dp(1471) = dp(2)*p(929) + p(2)*dp(929) - dp(1345)
dp(1472) = dp(6)*p(585) + p(6)*dp(585) - dp(1346)
dp(1473) = dp(2)*p(931) + p(2)*dp(931) - dp(1347) - dp(1467)
dp(1474) = dp(13)*p(347) + p(13)*dp(347) - dp(1346)
dp(1475) = dp(14)*p(347) + p(14)*dp(347)
dp(1476) = dp(3)*p(566) + p(3)*dp(566) - dp(1351)
dp(1477) = dp(7)*p(566) + p(7)*dp(566) - dp(1349)
dp(1478) = dp(49)*p(178) + p(49)*dp(178) - dp(1348)
dp(1479) = dp(39)*p(194) + p(39)*dp(194) - dp(1352)
dp(1480) = dp(3)*p(588) + p(3)*dp(588) - dp(1355)
dp(1481) = dp(1)*p(938) + p(1)*dp(938) - dp(1356)
dp(1482) = dp(2)*p(958) + p(2)*dp(958) - dp(1481)
dp(1483) = dp(1)*p(940) + p(1)*dp(940) - dp(1370)
dp(1484) = dp(8)*p(588) + p(8)*dp(588) - dp(1483)
dp(1485) = dp(1)*p(942) + p(1)*dp(942) - dp(1386)
dp(1486) = dp(22)*p(346) + p(22)*dp(346) - dp(1485)
dp(1487) = dp(1)*p(944) + p(1)*dp(944) - dp(1403)
dp(1488) = dp(50)*p(194) + p(50)*dp(194) - dp(1487)
dp(1489) = dp(1)*p(946) + p(1)*dp(946) - dp(1424)
dp(1490) = dp(101)*p(102) + p(101)*dp(102) - dp(1489)
dp(1491) = dp(6)*p(578) + p(6)*dp(578) - dp(1443)
dp(1492) = dp(1)*p(949) + p(1)*dp(949) - dp(1444)
dp(1493) = dp(49)*p(195) + p(49)*dp(195) - dp(1492)
dp(1494) = dp(8)*p(578) + p(8)*dp(578) - dp(1423)
dp(1495) = dp(1)*p(952) + p(1)*dp(952) - dp(1460)
dp(1496) = dp(21)*p(347) + p(21)*dp(347) - dp(1495)
dp(1497) = dp(50)*p(187) + p(50)*dp(187) - dp(1420)
dp(1498) = dp(1)*p(955) + p(1)*dp(955) - dp(1470)
dp(1499) = dp(7)*p(589) + p(7)*dp(589) - dp(1498)
dp(1500) = dp(45)*p(195) + p(45)*dp(195) - dp(1455)
dp(1501) = dp(2)*p(955) + p(2)*dp(955) - dp(1472)
dp(1502) = dp(1)*p(959) + p(1)*dp(959) - dp(1501)
dp(1503) = dp(6)*p(589) + p(6)*dp(589) - dp(1475)
dp(1504) = dp(1)*p(958) + p(1)*dp(958) - dp(1480)
dp(1505) = dp(2)*p(959) + p(2)*dp(959) - dp(1503)

do j=1,1506
  dbm1(j)=dp(j-1)
enddo

dbm2(1)=dbm1(4)
dbm2(2)=dbm1(10)
dbm2(3)=dbm1(11)
dbm2(4)=dbm1(12)
dbm2(5)=dbm1(13)
dbm2(6)=dbm1(15)
dbm2(7)=dbm1(16)
dbm2(8)=dbm1(24)
dbm2(9)=dbm1(25)
dbm2(10)=dbm1(26)
dbm2(11)=dbm1(27)
dbm2(12)=dbm1(28)
dbm2(13)=dbm1(29)
dbm2(14)=dbm1(30)
dbm2(15)=dbm1(31)
dbm2(16)=dbm1(32)
dbm2(17)=dbm1(34)
dbm2(18)=dbm1(35)
dbm2(19)=dbm1(36)
dbm2(20)=dbm1(37)
dbm2(21)=dbm1(39)
dbm2(22)=dbm1(40)
dbm2(23)=dbm1(41)
dbm2(24)=dbm1(52)
dbm2(25)=dbm1(53)
dbm2(26)=dbm1(54)
dbm2(27)=dbm1(55)
dbm2(28)=dbm1(56)
dbm2(29)=dbm1(57)
dbm2(30)=dbm1(58)
dbm2(31)=dbm1(59)
dbm2(32)=dbm1(60)
dbm2(33)=dbm1(61)
dbm2(34)=dbm1(62)
dbm2(35)=dbm1(63)
dbm2(36)=dbm1(64)
dbm2(37)=dbm1(65)
dbm2(38)=dbm1(66)
dbm2(39)=dbm1(67)
dbm2(40)=dbm1(68)
dbm2(41)=dbm1(69)
dbm2(42)=dbm1(70)
dbm2(43)=dbm1(71)
dbm2(44)=dbm1(72)
dbm2(45)=dbm1(73)
dbm2(46)=dbm1(75)
dbm2(47)=dbm1(76)
dbm2(48)=dbm1(77)
dbm2(49)=dbm1(78)
dbm2(50)=dbm1(79)
dbm2(51)=dbm1(81)
dbm2(52)=dbm1(83)
dbm2(53)=dbm1(84)
dbm2(54)=dbm1(85)
dbm2(55)=dbm1(86)
dbm2(56)=dbm1(87)
dbm2(57)=dbm1(89)
dbm2(58)=dbm1(90)
dbm2(59)=dbm1(91)
dbm2(60)=dbm1(104)
dbm2(61)=dbm1(105)
dbm2(62)=dbm1(106)
dbm2(63)=dbm1(107)
dbm2(64)=dbm1(108)
dbm2(65)=dbm1(109)
dbm2(66)=dbm1(110)
dbm2(67)=dbm1(111)
dbm2(68)=dbm1(112)
dbm2(69)=dbm1(113)
dbm2(70)=dbm1(114)
dbm2(71)=dbm1(115)
dbm2(72)=dbm1(116)
dbm2(73)=dbm1(117)
dbm2(74)=dbm1(118)
dbm2(75)=dbm1(119)
dbm2(76)=dbm1(120)
dbm2(77)=dbm1(121)
dbm2(78)=dbm1(122)
dbm2(79)=dbm1(123)
dbm2(80)=dbm1(124)
dbm2(81)=dbm1(125)
dbm2(82)=dbm1(126)
dbm2(83)=dbm1(127)
dbm2(84)=dbm1(128)
dbm2(85)=dbm1(129)
dbm2(86)=dbm1(130)
dbm2(87)=dbm1(131)
dbm2(88)=dbm1(132)
dbm2(89)=dbm1(133)
dbm2(90)=dbm1(134)
dbm2(91)=dbm1(135)
dbm2(92)=dbm1(136)
dbm2(93)=dbm1(137)
dbm2(94)=dbm1(138)
dbm2(95)=dbm1(139)
dbm2(96)=dbm1(140)
dbm2(97)=dbm1(141)
dbm2(98)=dbm1(142)
dbm2(99)=dbm1(143)
dbm2(100)=dbm1(144)
dbm2(101)=dbm1(145)
dbm2(102)=dbm1(146)
dbm2(103)=dbm1(147)
dbm2(104)=dbm1(148)
dbm2(105)=dbm1(149)
dbm2(106)=dbm1(150)
dbm2(107)=dbm1(152)
dbm2(108)=dbm1(153)
dbm2(109)=dbm1(154)
dbm2(110)=dbm1(155)
dbm2(111)=dbm1(156)
dbm2(112)=dbm1(157)
dbm2(113)=dbm1(158)
dbm2(114)=dbm1(160)
dbm2(115)=dbm1(162)
dbm2(116)=dbm1(163)
dbm2(117)=dbm1(164)
dbm2(118)=dbm1(165)
dbm2(119)=dbm1(166)
dbm2(120)=dbm1(167)
dbm2(121)=dbm1(169)
dbm2(122)=dbm1(170)
dbm2(123)=dbm1(172)
dbm2(124)=dbm1(173)
dbm2(125)=dbm1(174)
dbm2(126)=dbm1(175)
dbm2(127)=dbm1(176)
dbm2(128)=dbm1(178)
dbm2(129)=dbm1(179)
dbm2(130)=dbm1(180)
dbm2(131)=dbm1(181)
dbm2(132)=dbm1(197)
dbm2(133)=dbm1(198)
dbm2(134)=dbm1(199)
dbm2(135)=dbm1(200)
dbm2(136)=dbm1(201)
dbm2(137)=dbm1(202)
dbm2(138)=dbm1(203)
dbm2(139)=dbm1(204)
dbm2(140)=dbm1(205)
dbm2(141)=dbm1(206)
dbm2(142)=dbm1(207)
dbm2(143)=dbm1(208)
dbm2(144)=dbm1(209)
dbm2(145)=dbm1(210)
dbm2(146)=dbm1(211)
dbm2(147)=dbm1(212)
dbm2(148)=dbm1(213)
dbm2(149)=dbm1(214)
dbm2(150)=dbm1(215)
dbm2(151)=dbm1(216)
dbm2(152)=dbm1(217)
dbm2(153)=dbm1(218)
dbm2(154)=dbm1(219)
dbm2(155)=dbm1(220)
dbm2(156)=dbm1(221)
dbm2(157)=dbm1(222)
dbm2(158)=dbm1(223)
dbm2(159)=dbm1(224)
dbm2(160)=dbm1(225)
dbm2(161)=dbm1(226)
dbm2(162)=dbm1(227)
dbm2(163)=dbm1(228)
dbm2(164)=dbm1(229)
dbm2(165)=dbm1(230)
dbm2(166)=dbm1(231)
dbm2(167)=dbm1(232)
dbm2(168)=dbm1(233)
dbm2(169)=dbm1(234)
dbm2(170)=dbm1(235)
dbm2(171)=dbm1(236)
dbm2(172)=dbm1(237)
dbm2(173)=dbm1(238)
dbm2(174)=dbm1(239)
dbm2(175)=dbm1(240)
dbm2(176)=dbm1(241)
dbm2(177)=dbm1(242)
dbm2(178)=dbm1(243)
dbm2(179)=dbm1(244)
dbm2(180)=dbm1(245)
dbm2(181)=dbm1(246)
dbm2(182)=dbm1(247)
dbm2(183)=dbm1(248)
dbm2(184)=dbm1(249)
dbm2(185)=dbm1(250)
dbm2(186)=dbm1(251)
dbm2(187)=dbm1(252)
dbm2(188)=dbm1(253)
dbm2(189)=dbm1(254)
dbm2(190)=dbm1(255)
dbm2(191)=dbm1(256)
dbm2(192)=dbm1(257)
dbm2(193)=dbm1(258)
dbm2(194)=dbm1(259)
dbm2(195)=dbm1(260)
dbm2(196)=dbm1(261)
dbm2(197)=dbm1(262)
dbm2(198)=dbm1(263)
dbm2(199)=dbm1(264)
dbm2(200)=dbm1(265)
dbm2(201)=dbm1(266)
dbm2(202)=dbm1(267)
dbm2(203)=dbm1(268)
dbm2(204)=dbm1(269)
dbm2(205)=dbm1(270)
dbm2(206)=dbm1(271)
dbm2(207)=dbm1(272)
dbm2(208)=dbm1(273)
dbm2(209)=dbm1(274)
dbm2(210)=dbm1(275)
dbm2(211)=dbm1(276)
dbm2(212)=dbm1(277)
dbm2(213)=dbm1(278)
dbm2(214)=dbm1(279)
dbm2(215)=dbm1(280)
dbm2(216)=dbm1(281)
dbm2(217)=dbm1(282)
dbm2(218)=dbm1(283)
dbm2(219)=dbm1(284)
dbm2(220)=dbm1(285)
dbm2(221)=dbm1(287)
dbm2(222)=dbm1(288)
dbm2(223)=dbm1(289)
dbm2(224)=dbm1(290)
dbm2(225)=dbm1(291)
dbm2(226)=dbm1(292)
dbm2(227)=dbm1(293)
dbm2(228)=dbm1(294)
dbm2(229)=dbm1(296)
dbm2(230)=dbm1(298)
dbm2(231)=dbm1(299)
dbm2(232)=dbm1(300)
dbm2(233)=dbm1(301)
dbm2(234)=dbm1(302)
dbm2(235)=dbm1(303)
dbm2(236)=dbm1(304)
dbm2(237)=dbm1(306)
dbm2(238)=dbm1(307)
dbm2(239)=dbm1(309)
dbm2(240)=dbm1(310)
dbm2(241)=dbm1(312)
dbm2(242)=dbm1(313)
dbm2(243)=dbm1(314)
dbm2(244)=dbm1(315)
dbm2(245)=dbm1(316)
dbm2(246)=dbm1(317)
dbm2(247)=dbm1(319)
dbm2(248)=dbm1(320)
dbm2(249)=dbm1(322)
dbm2(250)=dbm1(323)
dbm2(251)=dbm1(324)
dbm2(252)=dbm1(325)
dbm2(253)=dbm1(326)
dbm2(254)=dbm1(328)
dbm2(255)=dbm1(329)
dbm2(256)=dbm1(330)
dbm2(257)=dbm1(331)
dbm2(258)=dbm1(349)
dbm2(259)=dbm1(350)
dbm2(260)=dbm1(351)
dbm2(261)=dbm1(352)
dbm2(262)=dbm1(353)
dbm2(263)=dbm1(354)
dbm2(264)=dbm1(355)
dbm2(265)=dbm1(356)
dbm2(266)=dbm1(357)
dbm2(267)=dbm1(358)
dbm2(268)=dbm1(359)
dbm2(269)=dbm1(360)
dbm2(270)=dbm1(361)
dbm2(271)=dbm1(362)
dbm2(272)=dbm1(363)
dbm2(273)=dbm1(364)
dbm2(274)=dbm1(365)
dbm2(275)=dbm1(366)
dbm2(276)=dbm1(367)
dbm2(277)=dbm1(368)
dbm2(278)=dbm1(369)
dbm2(279)=dbm1(370)
dbm2(280)=dbm1(371)
dbm2(281)=dbm1(372)
dbm2(282)=dbm1(373)
dbm2(283)=dbm1(374)
dbm2(284)=dbm1(375)
dbm2(285)=dbm1(376)
dbm2(286)=dbm1(377)
dbm2(287)=dbm1(378)
dbm2(288)=dbm1(379)
dbm2(289)=dbm1(380)
dbm2(290)=dbm1(381)
dbm2(291)=dbm1(382)
dbm2(292)=dbm1(383)
dbm2(293)=dbm1(384)
dbm2(294)=dbm1(385)
dbm2(295)=dbm1(386)
dbm2(296)=dbm1(387)
dbm2(297)=dbm1(388)
dbm2(298)=dbm1(389)
dbm2(299)=dbm1(390)
dbm2(300)=dbm1(391)
dbm2(301)=dbm1(392)
dbm2(302)=dbm1(393)
dbm2(303)=dbm1(394)
dbm2(304)=dbm1(395)
dbm2(305)=dbm1(396)
dbm2(306)=dbm1(397)
dbm2(307)=dbm1(398)
dbm2(308)=dbm1(399)
dbm2(309)=dbm1(400)
dbm2(310)=dbm1(401)
dbm2(311)=dbm1(402)
dbm2(312)=dbm1(403)
dbm2(313)=dbm1(404)
dbm2(314)=dbm1(405)
dbm2(315)=dbm1(406)
dbm2(316)=dbm1(407)
dbm2(317)=dbm1(408)
dbm2(318)=dbm1(409)
dbm2(319)=dbm1(410)
dbm2(320)=dbm1(411)
dbm2(321)=dbm1(412)
dbm2(322)=dbm1(413)
dbm2(323)=dbm1(414)
dbm2(324)=dbm1(415)
dbm2(325)=dbm1(416)
dbm2(326)=dbm1(417)
dbm2(327)=dbm1(418)
dbm2(328)=dbm1(419)
dbm2(329)=dbm1(420)
dbm2(330)=dbm1(421)
dbm2(331)=dbm1(422)
dbm2(332)=dbm1(423)
dbm2(333)=dbm1(424)
dbm2(334)=dbm1(425)
dbm2(335)=dbm1(426)
dbm2(336)=dbm1(427)
dbm2(337)=dbm1(428)
dbm2(338)=dbm1(429)
dbm2(339)=dbm1(430)
dbm2(340)=dbm1(431)
dbm2(341)=dbm1(432)
dbm2(342)=dbm1(433)
dbm2(343)=dbm1(434)
dbm2(344)=dbm1(435)
dbm2(345)=dbm1(436)
dbm2(346)=dbm1(437)
dbm2(347)=dbm1(438)
dbm2(348)=dbm1(439)
dbm2(349)=dbm1(440)
dbm2(350)=dbm1(441)
dbm2(351)=dbm1(442)
dbm2(352)=dbm1(443)
dbm2(353)=dbm1(444)
dbm2(354)=dbm1(445)
dbm2(355)=dbm1(446)
dbm2(356)=dbm1(447)
dbm2(357)=dbm1(448)
dbm2(358)=dbm1(449)
dbm2(359)=dbm1(450)
dbm2(360)=dbm1(451)
dbm2(361)=dbm1(452)
dbm2(362)=dbm1(453)
dbm2(363)=dbm1(454)
dbm2(364)=dbm1(455)
dbm2(365)=dbm1(456)
dbm2(366)=dbm1(457)
dbm2(367)=dbm1(458)
dbm2(368)=dbm1(459)
dbm2(369)=dbm1(460)
dbm2(370)=dbm1(461)
dbm2(371)=dbm1(462)
dbm2(372)=dbm1(463)
dbm2(373)=dbm1(464)
dbm2(374)=dbm1(465)
dbm2(375)=dbm1(466)
dbm2(376)=dbm1(467)
dbm2(377)=dbm1(468)
dbm2(378)=dbm1(469)
dbm2(379)=dbm1(470)
dbm2(380)=dbm1(471)
dbm2(381)=dbm1(472)
dbm2(382)=dbm1(473)
dbm2(383)=dbm1(474)
dbm2(384)=dbm1(475)
dbm2(385)=dbm1(476)
dbm2(386)=dbm1(477)
dbm2(387)=dbm1(478)
dbm2(388)=dbm1(479)
dbm2(389)=dbm1(480)
dbm2(390)=dbm1(481)
dbm2(391)=dbm1(482)
dbm2(392)=dbm1(483)
dbm2(393)=dbm1(484)
dbm2(394)=dbm1(485)
dbm2(395)=dbm1(486)
dbm2(396)=dbm1(487)
dbm2(397)=dbm1(488)
dbm2(398)=dbm1(489)
dbm2(399)=dbm1(490)
dbm2(400)=dbm1(491)
dbm2(401)=dbm1(492)
dbm2(402)=dbm1(493)
dbm2(403)=dbm1(494)
dbm2(404)=dbm1(495)
dbm2(405)=dbm1(496)
dbm2(406)=dbm1(497)
dbm2(407)=dbm1(498)
dbm2(408)=dbm1(499)
dbm2(409)=dbm1(500)
dbm2(410)=dbm1(501)
dbm2(411)=dbm1(502)
dbm2(412)=dbm1(503)
dbm2(413)=dbm1(504)
dbm2(414)=dbm1(505)
dbm2(415)=dbm1(507)
dbm2(416)=dbm1(508)
dbm2(417)=dbm1(509)
dbm2(418)=dbm1(510)
dbm2(419)=dbm1(511)
dbm2(420)=dbm1(512)
dbm2(421)=dbm1(513)
dbm2(422)=dbm1(514)
dbm2(423)=dbm1(515)
dbm2(424)=dbm1(516)
dbm2(425)=dbm1(518)
dbm2(426)=dbm1(520)
dbm2(427)=dbm1(521)
dbm2(428)=dbm1(522)
dbm2(429)=dbm1(523)
dbm2(430)=dbm1(524)
dbm2(431)=dbm1(525)
dbm2(432)=dbm1(526)
dbm2(433)=dbm1(527)
dbm2(434)=dbm1(529)
dbm2(435)=dbm1(530)
dbm2(436)=dbm1(532)
dbm2(437)=dbm1(534)
dbm2(438)=dbm1(535)
dbm2(439)=dbm1(536)
dbm2(440)=dbm1(537)
dbm2(441)=dbm1(538)
dbm2(442)=dbm1(539)
dbm2(443)=dbm1(540)
dbm2(444)=dbm1(541)
dbm2(445)=dbm1(543)
dbm2(446)=dbm1(544)
dbm2(447)=dbm1(546)
dbm2(448)=dbm1(547)
dbm2(449)=dbm1(548)
dbm2(450)=dbm1(550)
dbm2(451)=dbm1(551)
dbm2(452)=dbm1(552)
dbm2(453)=dbm1(553)
dbm2(454)=dbm1(554)
dbm2(455)=dbm1(555)
dbm2(456)=dbm1(557)
dbm2(457)=dbm1(558)
dbm2(458)=dbm1(560)
dbm2(459)=dbm1(561)
dbm2(460)=dbm1(562)
dbm2(461)=dbm1(563)
dbm2(462)=dbm1(564)
dbm2(463)=dbm1(566)
dbm2(464)=dbm1(567)
dbm2(465)=dbm1(568)
dbm2(466)=dbm1(569)
dbm2(467)=dbm1(570)
dbm2(468)=dbm1(591)
dbm2(469)=dbm1(592)
dbm2(470)=dbm1(593)
dbm2(471)=dbm1(594)
dbm2(472)=dbm1(595)
dbm2(473)=dbm1(596)
dbm2(474)=dbm1(597)
dbm2(475)=dbm1(598)
dbm2(476)=dbm1(599)
dbm2(477)=dbm1(600)
dbm2(478)=dbm1(601)
dbm2(479)=dbm1(602)
dbm2(480)=dbm1(603)
dbm2(481)=dbm1(604)
dbm2(482)=dbm1(605)
dbm2(483)=dbm1(606)
dbm2(484)=dbm1(607)
dbm2(485)=dbm1(608)
dbm2(486)=dbm1(609)
dbm2(487)=dbm1(610)
dbm2(488)=dbm1(611)
dbm2(489)=dbm1(612)
dbm2(490)=dbm1(613)
dbm2(491)=dbm1(614)
dbm2(492)=dbm1(615)
dbm2(493)=dbm1(616)
dbm2(494)=dbm1(617)
dbm2(495)=dbm1(618)
dbm2(496)=dbm1(619)
dbm2(497)=dbm1(620)
dbm2(498)=dbm1(621)
dbm2(499)=dbm1(622)
dbm2(500)=dbm1(623)
dbm2(501)=dbm1(624)
dbm2(502)=dbm1(625)
dbm2(503)=dbm1(626)
dbm2(504)=dbm1(627)
dbm2(505)=dbm1(628)
dbm2(506)=dbm1(629)
dbm2(507)=dbm1(630)
dbm2(508)=dbm1(631)
dbm2(509)=dbm1(632)
dbm2(510)=dbm1(633)
dbm2(511)=dbm1(634)
dbm2(512)=dbm1(635)
dbm2(513)=dbm1(636)
dbm2(514)=dbm1(637)
dbm2(515)=dbm1(638)
dbm2(516)=dbm1(639)
dbm2(517)=dbm1(640)
dbm2(518)=dbm1(641)
dbm2(519)=dbm1(642)
dbm2(520)=dbm1(643)
dbm2(521)=dbm1(644)
dbm2(522)=dbm1(645)
dbm2(523)=dbm1(646)
dbm2(524)=dbm1(647)
dbm2(525)=dbm1(648)
dbm2(526)=dbm1(649)
dbm2(527)=dbm1(650)
dbm2(528)=dbm1(651)
dbm2(529)=dbm1(652)
dbm2(530)=dbm1(653)
dbm2(531)=dbm1(654)
dbm2(532)=dbm1(655)
dbm2(533)=dbm1(656)
dbm2(534)=dbm1(657)
dbm2(535)=dbm1(658)
dbm2(536)=dbm1(659)
dbm2(537)=dbm1(660)
dbm2(538)=dbm1(661)
dbm2(539)=dbm1(662)
dbm2(540)=dbm1(663)
dbm2(541)=dbm1(664)
dbm2(542)=dbm1(665)
dbm2(543)=dbm1(666)
dbm2(544)=dbm1(667)
dbm2(545)=dbm1(668)
dbm2(546)=dbm1(669)
dbm2(547)=dbm1(670)
dbm2(548)=dbm1(671)
dbm2(549)=dbm1(672)
dbm2(550)=dbm1(673)
dbm2(551)=dbm1(674)
dbm2(552)=dbm1(675)
dbm2(553)=dbm1(676)
dbm2(554)=dbm1(677)
dbm2(555)=dbm1(678)
dbm2(556)=dbm1(679)
dbm2(557)=dbm1(680)
dbm2(558)=dbm1(681)
dbm2(559)=dbm1(682)
dbm2(560)=dbm1(683)
dbm2(561)=dbm1(684)
dbm2(562)=dbm1(685)
dbm2(563)=dbm1(686)
dbm2(564)=dbm1(687)
dbm2(565)=dbm1(688)
dbm2(566)=dbm1(689)
dbm2(567)=dbm1(690)
dbm2(568)=dbm1(691)
dbm2(569)=dbm1(692)
dbm2(570)=dbm1(693)
dbm2(571)=dbm1(694)
dbm2(572)=dbm1(695)
dbm2(573)=dbm1(696)
dbm2(574)=dbm1(697)
dbm2(575)=dbm1(698)
dbm2(576)=dbm1(699)
dbm2(577)=dbm1(700)
dbm2(578)=dbm1(701)
dbm2(579)=dbm1(702)
dbm2(580)=dbm1(703)
dbm2(581)=dbm1(704)
dbm2(582)=dbm1(705)
dbm2(583)=dbm1(706)
dbm2(584)=dbm1(707)
dbm2(585)=dbm1(708)
dbm2(586)=dbm1(709)
dbm2(587)=dbm1(710)
dbm2(588)=dbm1(711)
dbm2(589)=dbm1(712)
dbm2(590)=dbm1(713)
dbm2(591)=dbm1(714)
dbm2(592)=dbm1(715)
dbm2(593)=dbm1(716)
dbm2(594)=dbm1(717)
dbm2(595)=dbm1(718)
dbm2(596)=dbm1(719)
dbm2(597)=dbm1(720)
dbm2(598)=dbm1(721)
dbm2(599)=dbm1(722)
dbm2(600)=dbm1(723)
dbm2(601)=dbm1(724)
dbm2(602)=dbm1(725)
dbm2(603)=dbm1(726)
dbm2(604)=dbm1(727)
dbm2(605)=dbm1(728)
dbm2(606)=dbm1(729)
dbm2(607)=dbm1(730)
dbm2(608)=dbm1(731)
dbm2(609)=dbm1(732)
dbm2(610)=dbm1(733)
dbm2(611)=dbm1(734)
dbm2(612)=dbm1(735)
dbm2(613)=dbm1(736)
dbm2(614)=dbm1(737)
dbm2(615)=dbm1(738)
dbm2(616)=dbm1(739)
dbm2(617)=dbm1(740)
dbm2(618)=dbm1(741)
dbm2(619)=dbm1(742)
dbm2(620)=dbm1(743)
dbm2(621)=dbm1(744)
dbm2(622)=dbm1(745)
dbm2(623)=dbm1(746)
dbm2(624)=dbm1(747)
dbm2(625)=dbm1(748)
dbm2(626)=dbm1(749)
dbm2(627)=dbm1(750)
dbm2(628)=dbm1(751)
dbm2(629)=dbm1(752)
dbm2(630)=dbm1(753)
dbm2(631)=dbm1(754)
dbm2(632)=dbm1(755)
dbm2(633)=dbm1(756)
dbm2(634)=dbm1(757)
dbm2(635)=dbm1(758)
dbm2(636)=dbm1(759)
dbm2(637)=dbm1(760)
dbm2(638)=dbm1(761)
dbm2(639)=dbm1(762)
dbm2(640)=dbm1(763)
dbm2(641)=dbm1(764)
dbm2(642)=dbm1(765)
dbm2(643)=dbm1(766)
dbm2(644)=dbm1(767)
dbm2(645)=dbm1(768)
dbm2(646)=dbm1(769)
dbm2(647)=dbm1(770)
dbm2(648)=dbm1(771)
dbm2(649)=dbm1(772)
dbm2(650)=dbm1(773)
dbm2(651)=dbm1(774)
dbm2(652)=dbm1(775)
dbm2(653)=dbm1(776)
dbm2(654)=dbm1(777)
dbm2(655)=dbm1(778)
dbm2(656)=dbm1(779)
dbm2(657)=dbm1(780)
dbm2(658)=dbm1(781)
dbm2(659)=dbm1(782)
dbm2(660)=dbm1(783)
dbm2(661)=dbm1(784)
dbm2(662)=dbm1(785)
dbm2(663)=dbm1(786)
dbm2(664)=dbm1(787)
dbm2(665)=dbm1(788)
dbm2(666)=dbm1(789)
dbm2(667)=dbm1(790)
dbm2(668)=dbm1(791)
dbm2(669)=dbm1(792)
dbm2(670)=dbm1(793)
dbm2(671)=dbm1(794)
dbm2(672)=dbm1(795)
dbm2(673)=dbm1(796)
dbm2(674)=dbm1(797)
dbm2(675)=dbm1(798)
dbm2(676)=dbm1(799)
dbm2(677)=dbm1(800)
dbm2(678)=dbm1(801)
dbm2(679)=dbm1(802)
dbm2(680)=dbm1(803)
dbm2(681)=dbm1(804)
dbm2(682)=dbm1(805)
dbm2(683)=dbm1(806)
dbm2(684)=dbm1(807)
dbm2(685)=dbm1(808)
dbm2(686)=dbm1(809)
dbm2(687)=dbm1(810)
dbm2(688)=dbm1(811)
dbm2(689)=dbm1(812)
dbm2(690)=dbm1(813)
dbm2(691)=dbm1(814)
dbm2(692)=dbm1(815)
dbm2(693)=dbm1(816)
dbm2(694)=dbm1(817)
dbm2(695)=dbm1(818)
dbm2(696)=dbm1(819)
dbm2(697)=dbm1(820)
dbm2(698)=dbm1(821)
dbm2(699)=dbm1(822)
dbm2(700)=dbm1(823)
dbm2(701)=dbm1(824)
dbm2(702)=dbm1(825)
dbm2(703)=dbm1(826)
dbm2(704)=dbm1(827)
dbm2(705)=dbm1(828)
dbm2(706)=dbm1(829)
dbm2(707)=dbm1(830)
dbm2(708)=dbm1(831)
dbm2(709)=dbm1(832)
dbm2(710)=dbm1(833)
dbm2(711)=dbm1(834)
dbm2(712)=dbm1(835)
dbm2(713)=dbm1(836)
dbm2(714)=dbm1(837)
dbm2(715)=dbm1(838)
dbm2(716)=dbm1(839)
dbm2(717)=dbm1(840)
dbm2(718)=dbm1(841)
dbm2(719)=dbm1(842)
dbm2(720)=dbm1(843)
dbm2(721)=dbm1(844)
dbm2(722)=dbm1(845)
dbm2(723)=dbm1(846)
dbm2(724)=dbm1(847)
dbm2(725)=dbm1(848)
dbm2(726)=dbm1(849)
dbm2(727)=dbm1(850)
dbm2(728)=dbm1(851)
dbm2(729)=dbm1(853)
dbm2(730)=dbm1(854)
dbm2(731)=dbm1(855)
dbm2(732)=dbm1(856)
dbm2(733)=dbm1(857)
dbm2(734)=dbm1(858)
dbm2(735)=dbm1(859)
dbm2(736)=dbm1(860)
dbm2(737)=dbm1(861)
dbm2(738)=dbm1(862)
dbm2(739)=dbm1(863)
dbm2(740)=dbm1(865)
dbm2(741)=dbm1(867)
dbm2(742)=dbm1(868)
dbm2(743)=dbm1(869)
dbm2(744)=dbm1(870)
dbm2(745)=dbm1(871)
dbm2(746)=dbm1(872)
dbm2(747)=dbm1(873)
dbm2(748)=dbm1(874)
dbm2(749)=dbm1(875)
dbm2(750)=dbm1(876)
dbm2(751)=dbm1(878)
dbm2(752)=dbm1(879)
dbm2(753)=dbm1(881)
dbm2(754)=dbm1(883)
dbm2(755)=dbm1(884)
dbm2(756)=dbm1(885)
dbm2(757)=dbm1(886)
dbm2(758)=dbm1(887)
dbm2(759)=dbm1(888)
dbm2(760)=dbm1(889)
dbm2(761)=dbm1(890)
dbm2(762)=dbm1(891)
dbm2(763)=dbm1(893)
dbm2(764)=dbm1(894)
dbm2(765)=dbm1(896)
dbm2(766)=dbm1(897)
dbm2(767)=dbm1(899)
dbm2(768)=dbm1(900)
dbm2(769)=dbm1(902)
dbm2(770)=dbm1(903)
dbm2(771)=dbm1(904)
dbm2(772)=dbm1(905)
dbm2(773)=dbm1(906)
dbm2(774)=dbm1(907)
dbm2(775)=dbm1(908)
dbm2(776)=dbm1(909)
dbm2(777)=dbm1(911)
dbm2(778)=dbm1(912)
dbm2(779)=dbm1(914)
dbm2(780)=dbm1(915)
dbm2(781)=dbm1(916)
dbm2(782)=dbm1(918)
dbm2(783)=dbm1(919)
dbm2(784)=dbm1(920)
dbm2(785)=dbm1(921)
dbm2(786)=dbm1(922)
dbm2(787)=dbm1(923)
dbm2(788)=dbm1(925)
dbm2(789)=dbm1(926)
dbm2(790)=dbm1(928)
dbm2(791)=dbm1(929)
dbm2(792)=dbm1(930)
dbm2(793)=dbm1(931)
dbm2(794)=dbm1(932)
dbm2(795)=dbm1(934)
dbm2(796)=dbm1(935)
dbm2(797)=dbm1(936)
dbm2(798)=dbm1(937)
dbm2(799)=dbm1(938)
dbm2(800)=dbm1(961)
dbm2(801)=dbm1(962)
dbm2(802)=dbm1(963)
dbm2(803)=dbm1(964)
dbm2(804)=dbm1(965)
dbm2(805)=dbm1(966)
dbm2(806)=dbm1(967)
dbm2(807)=dbm1(968)
dbm2(808)=dbm1(969)
dbm2(809)=dbm1(970)
dbm2(810)=dbm1(971)
dbm2(811)=dbm1(972)
dbm2(812)=dbm1(973)
dbm2(813)=dbm1(974)
dbm2(814)=dbm1(975)
dbm2(815)=dbm1(976)
dbm2(816)=dbm1(977)
dbm2(817)=dbm1(978)
dbm2(818)=dbm1(979)
dbm2(819)=dbm1(980)
dbm2(820)=dbm1(981)
dbm2(821)=dbm1(982)
dbm2(822)=dbm1(983)
dbm2(823)=dbm1(984)
dbm2(824)=dbm1(985)
dbm2(825)=dbm1(986)
dbm2(826)=dbm1(987)
dbm2(827)=dbm1(988)
dbm2(828)=dbm1(989)
dbm2(829)=dbm1(990)
dbm2(830)=dbm1(991)
dbm2(831)=dbm1(992)
dbm2(832)=dbm1(993)
dbm2(833)=dbm1(994)
dbm2(834)=dbm1(995)
dbm2(835)=dbm1(996)
dbm2(836)=dbm1(997)
dbm2(837)=dbm1(998)
dbm2(838)=dbm1(999)
dbm2(839)=dbm1(1000)
dbm2(840)=dbm1(1001)
dbm2(841)=dbm1(1002)
dbm2(842)=dbm1(1003)
dbm2(843)=dbm1(1004)
dbm2(844)=dbm1(1005)
dbm2(845)=dbm1(1006)
dbm2(846)=dbm1(1007)
dbm2(847)=dbm1(1008)
dbm2(848)=dbm1(1009)
dbm2(849)=dbm1(1010)
dbm2(850)=dbm1(1011)
dbm2(851)=dbm1(1012)
dbm2(852)=dbm1(1013)
dbm2(853)=dbm1(1014)
dbm2(854)=dbm1(1015)
dbm2(855)=dbm1(1016)
dbm2(856)=dbm1(1017)
dbm2(857)=dbm1(1018)
dbm2(858)=dbm1(1019)
dbm2(859)=dbm1(1020)
dbm2(860)=dbm1(1021)
dbm2(861)=dbm1(1022)
dbm2(862)=dbm1(1023)
dbm2(863)=dbm1(1024)
dbm2(864)=dbm1(1025)
dbm2(865)=dbm1(1026)
dbm2(866)=dbm1(1027)
dbm2(867)=dbm1(1028)
dbm2(868)=dbm1(1029)
dbm2(869)=dbm1(1030)
dbm2(870)=dbm1(1031)
dbm2(871)=dbm1(1032)
dbm2(872)=dbm1(1033)
dbm2(873)=dbm1(1034)
dbm2(874)=dbm1(1035)
dbm2(875)=dbm1(1036)
dbm2(876)=dbm1(1037)
dbm2(877)=dbm1(1038)
dbm2(878)=dbm1(1039)
dbm2(879)=dbm1(1040)
dbm2(880)=dbm1(1041)
dbm2(881)=dbm1(1042)
dbm2(882)=dbm1(1043)
dbm2(883)=dbm1(1044)
dbm2(884)=dbm1(1045)
dbm2(885)=dbm1(1046)
dbm2(886)=dbm1(1047)
dbm2(887)=dbm1(1048)
dbm2(888)=dbm1(1049)
dbm2(889)=dbm1(1050)
dbm2(890)=dbm1(1051)
dbm2(891)=dbm1(1052)
dbm2(892)=dbm1(1053)
dbm2(893)=dbm1(1054)
dbm2(894)=dbm1(1055)
dbm2(895)=dbm1(1056)
dbm2(896)=dbm1(1057)
dbm2(897)=dbm1(1058)
dbm2(898)=dbm1(1059)
dbm2(899)=dbm1(1060)
dbm2(900)=dbm1(1061)
dbm2(901)=dbm1(1062)
dbm2(902)=dbm1(1063)
dbm2(903)=dbm1(1064)
dbm2(904)=dbm1(1065)
dbm2(905)=dbm1(1066)
dbm2(906)=dbm1(1067)
dbm2(907)=dbm1(1068)
dbm2(908)=dbm1(1069)
dbm2(909)=dbm1(1070)
dbm2(910)=dbm1(1071)
dbm2(911)=dbm1(1072)
dbm2(912)=dbm1(1073)
dbm2(913)=dbm1(1074)
dbm2(914)=dbm1(1075)
dbm2(915)=dbm1(1076)
dbm2(916)=dbm1(1077)
dbm2(917)=dbm1(1078)
dbm2(918)=dbm1(1079)
dbm2(919)=dbm1(1080)
dbm2(920)=dbm1(1081)
dbm2(921)=dbm1(1082)
dbm2(922)=dbm1(1083)
dbm2(923)=dbm1(1084)
dbm2(924)=dbm1(1085)
dbm2(925)=dbm1(1086)
dbm2(926)=dbm1(1087)
dbm2(927)=dbm1(1088)
dbm2(928)=dbm1(1089)
dbm2(929)=dbm1(1090)
dbm2(930)=dbm1(1091)
dbm2(931)=dbm1(1092)
dbm2(932)=dbm1(1093)
dbm2(933)=dbm1(1094)
dbm2(934)=dbm1(1095)
dbm2(935)=dbm1(1096)
dbm2(936)=dbm1(1097)
dbm2(937)=dbm1(1098)
dbm2(938)=dbm1(1099)
dbm2(939)=dbm1(1100)
dbm2(940)=dbm1(1101)
dbm2(941)=dbm1(1102)
dbm2(942)=dbm1(1103)
dbm2(943)=dbm1(1104)
dbm2(944)=dbm1(1105)
dbm2(945)=dbm1(1106)
dbm2(946)=dbm1(1107)
dbm2(947)=dbm1(1108)
dbm2(948)=dbm1(1109)
dbm2(949)=dbm1(1110)
dbm2(950)=dbm1(1111)
dbm2(951)=dbm1(1112)
dbm2(952)=dbm1(1113)
dbm2(953)=dbm1(1114)
dbm2(954)=dbm1(1115)
dbm2(955)=dbm1(1116)
dbm2(956)=dbm1(1117)
dbm2(957)=dbm1(1118)
dbm2(958)=dbm1(1119)
dbm2(959)=dbm1(1120)
dbm2(960)=dbm1(1121)
dbm2(961)=dbm1(1122)
dbm2(962)=dbm1(1123)
dbm2(963)=dbm1(1124)
dbm2(964)=dbm1(1125)
dbm2(965)=dbm1(1126)
dbm2(966)=dbm1(1127)
dbm2(967)=dbm1(1128)
dbm2(968)=dbm1(1129)
dbm2(969)=dbm1(1130)
dbm2(970)=dbm1(1131)
dbm2(971)=dbm1(1132)
dbm2(972)=dbm1(1133)
dbm2(973)=dbm1(1134)
dbm2(974)=dbm1(1135)
dbm2(975)=dbm1(1136)
dbm2(976)=dbm1(1137)
dbm2(977)=dbm1(1138)
dbm2(978)=dbm1(1139)
dbm2(979)=dbm1(1140)
dbm2(980)=dbm1(1141)
dbm2(981)=dbm1(1142)
dbm2(982)=dbm1(1143)
dbm2(983)=dbm1(1144)
dbm2(984)=dbm1(1145)
dbm2(985)=dbm1(1146)
dbm2(986)=dbm1(1147)
dbm2(987)=dbm1(1148)
dbm2(988)=dbm1(1149)
dbm2(989)=dbm1(1150)
dbm2(990)=dbm1(1151)
dbm2(991)=dbm1(1152)
dbm2(992)=dbm1(1153)
dbm2(993)=dbm1(1154)
dbm2(994)=dbm1(1155)
dbm2(995)=dbm1(1156)
dbm2(996)=dbm1(1157)
dbm2(997)=dbm1(1158)
dbm2(998)=dbm1(1159)
dbm2(999)=dbm1(1160)
dbm2(1000)=dbm1(1161)
dbm2(1001)=dbm1(1162)
dbm2(1002)=dbm1(1163)
dbm2(1003)=dbm1(1164)
dbm2(1004)=dbm1(1165)
dbm2(1005)=dbm1(1166)
dbm2(1006)=dbm1(1167)
dbm2(1007)=dbm1(1168)
dbm2(1008)=dbm1(1169)
dbm2(1009)=dbm1(1170)
dbm2(1010)=dbm1(1171)
dbm2(1011)=dbm1(1172)
dbm2(1012)=dbm1(1173)
dbm2(1013)=dbm1(1174)
dbm2(1014)=dbm1(1175)
dbm2(1015)=dbm1(1176)
dbm2(1016)=dbm1(1177)
dbm2(1017)=dbm1(1178)
dbm2(1018)=dbm1(1179)
dbm2(1019)=dbm1(1180)
dbm2(1020)=dbm1(1181)
dbm2(1021)=dbm1(1182)
dbm2(1022)=dbm1(1183)
dbm2(1023)=dbm1(1184)
dbm2(1024)=dbm1(1185)
dbm2(1025)=dbm1(1186)
dbm2(1026)=dbm1(1187)
dbm2(1027)=dbm1(1188)
dbm2(1028)=dbm1(1189)
dbm2(1029)=dbm1(1190)
dbm2(1030)=dbm1(1191)
dbm2(1031)=dbm1(1192)
dbm2(1032)=dbm1(1193)
dbm2(1033)=dbm1(1194)
dbm2(1034)=dbm1(1195)
dbm2(1035)=dbm1(1196)
dbm2(1036)=dbm1(1197)
dbm2(1037)=dbm1(1198)
dbm2(1038)=dbm1(1199)
dbm2(1039)=dbm1(1200)
dbm2(1040)=dbm1(1201)
dbm2(1041)=dbm1(1202)
dbm2(1042)=dbm1(1203)
dbm2(1043)=dbm1(1204)
dbm2(1044)=dbm1(1205)
dbm2(1045)=dbm1(1206)
dbm2(1046)=dbm1(1207)
dbm2(1047)=dbm1(1208)
dbm2(1048)=dbm1(1209)
dbm2(1049)=dbm1(1210)
dbm2(1050)=dbm1(1211)
dbm2(1051)=dbm1(1212)
dbm2(1052)=dbm1(1213)
dbm2(1053)=dbm1(1214)
dbm2(1054)=dbm1(1215)
dbm2(1055)=dbm1(1216)
dbm2(1056)=dbm1(1217)
dbm2(1057)=dbm1(1218)
dbm2(1058)=dbm1(1219)
dbm2(1059)=dbm1(1220)
dbm2(1060)=dbm1(1221)
dbm2(1061)=dbm1(1222)
dbm2(1062)=dbm1(1223)
dbm2(1063)=dbm1(1224)
dbm2(1064)=dbm1(1225)
dbm2(1065)=dbm1(1226)
dbm2(1066)=dbm1(1227)
dbm2(1067)=dbm1(1228)
dbm2(1068)=dbm1(1229)
dbm2(1069)=dbm1(1230)
dbm2(1070)=dbm1(1231)
dbm2(1071)=dbm1(1232)
dbm2(1072)=dbm1(1233)
dbm2(1073)=dbm1(1234)
dbm2(1074)=dbm1(1235)
dbm2(1075)=dbm1(1236)
dbm2(1076)=dbm1(1237)
dbm2(1077)=dbm1(1238)
dbm2(1078)=dbm1(1239)
dbm2(1079)=dbm1(1240)
dbm2(1080)=dbm1(1241)
dbm2(1081)=dbm1(1242)
dbm2(1082)=dbm1(1243)
dbm2(1083)=dbm1(1244)
dbm2(1084)=dbm1(1245)
dbm2(1085)=dbm1(1246)
dbm2(1086)=dbm1(1247)
dbm2(1087)=dbm1(1248)
dbm2(1088)=dbm1(1249)
dbm2(1089)=dbm1(1250)
dbm2(1090)=dbm1(1251)
dbm2(1091)=dbm1(1252)
dbm2(1092)=dbm1(1253)
dbm2(1093)=dbm1(1254)
dbm2(1094)=dbm1(1255)
dbm2(1095)=dbm1(1256)
dbm2(1096)=dbm1(1257)
dbm2(1097)=dbm1(1258)
dbm2(1098)=dbm1(1259)
dbm2(1099)=dbm1(1260)
dbm2(1100)=dbm1(1261)
dbm2(1101)=dbm1(1262)
dbm2(1102)=dbm1(1263)
dbm2(1103)=dbm1(1264)
dbm2(1104)=dbm1(1265)
dbm2(1105)=dbm1(1266)
dbm2(1106)=dbm1(1267)
dbm2(1107)=dbm1(1268)
dbm2(1108)=dbm1(1269)
dbm2(1109)=dbm1(1270)
dbm2(1110)=dbm1(1271)
dbm2(1111)=dbm1(1272)
dbm2(1112)=dbm1(1273)
dbm2(1113)=dbm1(1274)
dbm2(1114)=dbm1(1275)
dbm2(1115)=dbm1(1276)
dbm2(1116)=dbm1(1277)
dbm2(1117)=dbm1(1278)
dbm2(1118)=dbm1(1279)
dbm2(1119)=dbm1(1280)
dbm2(1120)=dbm1(1281)
dbm2(1121)=dbm1(1282)
dbm2(1122)=dbm1(1283)
dbm2(1123)=dbm1(1284)
dbm2(1124)=dbm1(1285)
dbm2(1125)=dbm1(1286)
dbm2(1126)=dbm1(1287)
dbm2(1127)=dbm1(1288)
dbm2(1128)=dbm1(1289)
dbm2(1129)=dbm1(1290)
dbm2(1130)=dbm1(1291)
dbm2(1131)=dbm1(1292)
dbm2(1132)=dbm1(1293)
dbm2(1133)=dbm1(1294)
dbm2(1134)=dbm1(1295)
dbm2(1135)=dbm1(1296)
dbm2(1136)=dbm1(1297)
dbm2(1137)=dbm1(1298)
dbm2(1138)=dbm1(1299)
dbm2(1139)=dbm1(1300)
dbm2(1140)=dbm1(1301)
dbm2(1141)=dbm1(1302)
dbm2(1142)=dbm1(1303)
dbm2(1143)=dbm1(1304)
dbm2(1144)=dbm1(1305)
dbm2(1145)=dbm1(1306)
dbm2(1146)=dbm1(1307)
dbm2(1147)=dbm1(1308)
dbm2(1148)=dbm1(1309)
dbm2(1149)=dbm1(1310)
dbm2(1150)=dbm1(1311)
dbm2(1151)=dbm1(1312)
dbm2(1152)=dbm1(1313)
dbm2(1153)=dbm1(1314)
dbm2(1154)=dbm1(1315)
dbm2(1155)=dbm1(1316)
dbm2(1156)=dbm1(1317)
dbm2(1157)=dbm1(1318)
dbm2(1158)=dbm1(1319)
dbm2(1159)=dbm1(1320)
dbm2(1160)=dbm1(1321)
dbm2(1161)=dbm1(1322)
dbm2(1162)=dbm1(1323)
dbm2(1163)=dbm1(1324)
dbm2(1164)=dbm1(1325)
dbm2(1165)=dbm1(1326)
dbm2(1166)=dbm1(1327)
dbm2(1167)=dbm1(1328)
dbm2(1168)=dbm1(1329)
dbm2(1169)=dbm1(1330)
dbm2(1170)=dbm1(1331)
dbm2(1171)=dbm1(1332)
dbm2(1172)=dbm1(1333)
dbm2(1173)=dbm1(1334)
dbm2(1174)=dbm1(1335)
dbm2(1175)=dbm1(1336)
dbm2(1176)=dbm1(1337)
dbm2(1177)=dbm1(1338)
dbm2(1178)=dbm1(1339)
dbm2(1179)=dbm1(1340)
dbm2(1180)=dbm1(1341)
dbm2(1181)=dbm1(1342)
dbm2(1182)=dbm1(1343)
dbm2(1183)=dbm1(1344)
dbm2(1184)=dbm1(1345)
dbm2(1185)=dbm1(1346)
dbm2(1186)=dbm1(1347)
dbm2(1187)=dbm1(1348)
dbm2(1188)=dbm1(1349)
dbm2(1189)=dbm1(1350)
dbm2(1190)=dbm1(1351)
dbm2(1191)=dbm1(1352)
dbm2(1192)=dbm1(1353)
dbm2(1193)=dbm1(1354)
dbm2(1194)=dbm1(1355)
dbm2(1195)=dbm1(1356)
dbm2(1196)=dbm1(1357)
dbm2(1197)=dbm1(1358)
dbm2(1198)=dbm1(1359)
dbm2(1199)=dbm1(1360)
dbm2(1200)=dbm1(1361)
dbm2(1201)=dbm1(1362)
dbm2(1202)=dbm1(1363)
dbm2(1203)=dbm1(1364)
dbm2(1204)=dbm1(1365)
dbm2(1205)=dbm1(1366)
dbm2(1206)=dbm1(1367)
dbm2(1207)=dbm1(1368)
dbm2(1208)=dbm1(1369)
dbm2(1209)=dbm1(1371)
dbm2(1210)=dbm1(1372)
dbm2(1211)=dbm1(1373)
dbm2(1212)=dbm1(1374)
dbm2(1213)=dbm1(1375)
dbm2(1214)=dbm1(1376)
dbm2(1215)=dbm1(1377)
dbm2(1216)=dbm1(1378)
dbm2(1217)=dbm1(1379)
dbm2(1218)=dbm1(1380)
dbm2(1219)=dbm1(1381)
dbm2(1220)=dbm1(1382)
dbm2(1221)=dbm1(1383)
dbm2(1222)=dbm1(1385)
dbm2(1223)=dbm1(1387)
dbm2(1224)=dbm1(1388)
dbm2(1225)=dbm1(1389)
dbm2(1226)=dbm1(1390)
dbm2(1227)=dbm1(1391)
dbm2(1228)=dbm1(1392)
dbm2(1229)=dbm1(1393)
dbm2(1230)=dbm1(1394)
dbm2(1231)=dbm1(1395)
dbm2(1232)=dbm1(1396)
dbm2(1233)=dbm1(1397)
dbm2(1234)=dbm1(1399)
dbm2(1235)=dbm1(1400)
dbm2(1236)=dbm1(1402)
dbm2(1237)=dbm1(1404)
dbm2(1238)=dbm1(1405)
dbm2(1239)=dbm1(1406)
dbm2(1240)=dbm1(1407)
dbm2(1241)=dbm1(1408)
dbm2(1242)=dbm1(1409)
dbm2(1243)=dbm1(1410)
dbm2(1244)=dbm1(1411)
dbm2(1245)=dbm1(1412)
dbm2(1246)=dbm1(1413)
dbm2(1247)=dbm1(1415)
dbm2(1248)=dbm1(1416)
dbm2(1249)=dbm1(1418)
dbm2(1250)=dbm1(1419)
dbm2(1251)=dbm1(1421)
dbm2(1252)=dbm1(1422)
dbm2(1253)=dbm1(1424)
dbm2(1254)=dbm1(1425)
dbm2(1255)=dbm1(1426)
dbm2(1256)=dbm1(1427)
dbm2(1257)=dbm1(1428)
dbm2(1258)=dbm1(1429)
dbm2(1259)=dbm1(1430)
dbm2(1260)=dbm1(1431)
dbm2(1261)=dbm1(1432)
dbm2(1262)=dbm1(1434)
dbm2(1263)=dbm1(1435)
dbm2(1264)=dbm1(1437)
dbm2(1265)=dbm1(1438)
dbm2(1266)=dbm1(1440)
dbm2(1267)=dbm1(1441)
dbm2(1268)=dbm1(1442)
dbm2(1269)=dbm1(1444)
dbm2(1270)=dbm1(1445)
dbm2(1271)=dbm1(1446)
dbm2(1272)=dbm1(1447)
dbm2(1273)=dbm1(1448)
dbm2(1274)=dbm1(1449)
dbm2(1275)=dbm1(1450)
dbm2(1276)=dbm1(1451)
dbm2(1277)=dbm1(1453)
dbm2(1278)=dbm1(1454)
dbm2(1279)=dbm1(1456)
dbm2(1280)=dbm1(1457)
dbm2(1281)=dbm1(1458)
dbm2(1282)=dbm1(1460)
dbm2(1283)=dbm1(1461)
dbm2(1284)=dbm1(1462)
dbm2(1285)=dbm1(1463)
dbm2(1286)=dbm1(1464)
dbm2(1287)=dbm1(1465)
dbm2(1288)=dbm1(1467)
dbm2(1289)=dbm1(1468)
dbm2(1290)=dbm1(1470)
dbm2(1291)=dbm1(1471)
dbm2(1292)=dbm1(1472)
dbm2(1293)=dbm1(1473)
dbm2(1294)=dbm1(1474)
dbm2(1295)=dbm1(1476)
dbm2(1296)=dbm1(1477)
dbm2(1297)=dbm1(1478)
dbm2(1298)=dbm1(1479)
dbm2(1299)=dbm1(1480)
dbm2(1300)=dbm1(1481)

dVdR(i)=dVdR(i) + dot_product(coeff,dbm2)

end do

end subroutine EvU23O10dVdR
